/* This is a generated file! */
/* Generated by `utils/build_parsers.py` via `utils/rustify.py` */
/* This process can be run via tox: `tox -e generate-rs` */
#![cfg_attr(rustfmt, rustfmt_skip)]

use sqlfluffrs_types::{
    GrammarInst, GrammarVariant, GrammarFlags,
    ParseMode, GrammarId, GrammarTables,
    SimpleHintData, RootGrammar
};
// Flattening grammar tree...
// Validating tables...
// Validation passed!

// Table Statistics:
//   Instructions:      5592 × 20 bytes =   111840 bytes
//   Child IDs:         4106 ×  4 bytes =    16424 bytes
//   Terminators:         55 ×  4 bytes =      220 bytes
//   Strings:           2633 strings      =    36658 bytes
//   Aux Data:         15867 ×  4 bytes =    63468 bytes
//   Aux Offsets:       5592 ×  4 bytes =    22368 bytes
//   Regex Patterns:       8 patterns     =     1450 bytes
//   Simple Hints:      1639 hints        =    26224 bytes
//   Hint Indices:      2351 ×  4 bytes =     9404 bytes
//   TOTAL:                                    ≈   288056 bytes

pub static INSTRUCTIONS: &[GrammarInst] = &[
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [0] StringParser("ABORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [1] StringParser("ABS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [2] StringParser("ABSOLUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [3] StringParser("ACCESS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 0, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [4] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [5] Ref(AccountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [6] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5, child_count: 17, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [7] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 22, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [8] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [9] Ref(ResourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [10] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [11] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [12] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [13] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [14] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [15] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [16] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [17] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [18] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [19] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [20] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [21] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [22] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [23] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [24] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [25] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [26] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [27] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [28] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [29] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [30] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [31] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [32] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [33] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [34] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [35] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [36] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [37] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [38] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [39] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [40] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [41] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [42] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [43] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [44] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [45] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [46] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [47] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [48] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [49] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [50] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 2, _padding: 0 }, // [51] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 50, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [52] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [53] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [54] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [55] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [56] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [57] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [58] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [59] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [60] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [61] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [62] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [63] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [64] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 6, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [65] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 64, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [66] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [67] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 5, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [68] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [69] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [70] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [71] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [72] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [73] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [74] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [75] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [76] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [77] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [78] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [79] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [80] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [81] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [82] Ref(ManageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [83] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [84] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [85] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [86] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [87] Ref(ExecutionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [88] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [89] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 26, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [90] OneOf(26 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 110, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [91] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [92] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 4, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [93] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [94] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [95] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [96] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [97] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [98] Ref(PipeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [99] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [100] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [101] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [102] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [103] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [104] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [105] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [106] Ref(ImportedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [107] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [108] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [109] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [110] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [111] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [112] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [113] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [114] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [115] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [116] Ref(OperateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [117] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [118] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [119] Ref(Reference_usageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [120] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [121] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [122] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [123] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [124] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [125] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [126] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [127] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [128] Ref(Use_any_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [129] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [130] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [131] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [132] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [133] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 1, _padding: 0 }, // [134] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [135] Ref(AccessPermissionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [136] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [137] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [138] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [139] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [140] Ref(StageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [141] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [142] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [143] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [144] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [145] Ref(StreamKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [146] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [147] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [148] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [149] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [150] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [151] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [152] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [153] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [154] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [155] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [156] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [157] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [158] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [159] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [160] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [161] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [162] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [163] Ref(StagesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [164] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [165] Ref(ProceduresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [166] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [167] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [168] Ref(StreamsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [169] Ref(TasksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [170] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [171] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [172] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [173] Ref(PoliciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [174] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [175] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [176] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [177] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [178] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [179] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [180] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [181] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [182] Ref(FormatsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [183] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [184] Ref(GrantStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [185] Ref(RevokeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [186] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 181, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [187] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [188] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [189] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [190] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [191] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [192] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [193] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [194] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [195] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [196] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [197] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [198] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [199] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [200] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [201] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [202] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [203] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [204] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [205] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [206] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [207] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 201, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [208] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [209] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [210] StringParser("ACCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [211] StringParser("ACCOUNT_LOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [212] StringParser("ACCOUNT_UNLOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [213] StringParser("ACCOUNTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [214] StringParser("ACTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [215] StringParser("ACTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [216] StringParser("ACTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [217] StringParser("ADA")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [218] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [219] StringParser("ADD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [220] StringParser("ADMIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [221] StringParser("AFTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [222] StringParser("AGAINST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [223] StringParser("AGG_STATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [224] StringParser("AGGREGATE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [225] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [226] StringParser("ALGORITHM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [227] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [228] Meta("indent")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [229] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [230] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 209, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [231] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 211, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [232] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 211, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [233] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [234] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [235] StringParser("(")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [236] StringParser(")")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [237] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [238] Ref(DoubleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [239] Meta("dedent")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [240] StringParser("ALIAS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [241] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 216, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [242] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 216, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [243] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 216, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [244] StringParser("ALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 216, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [245] StringParser("ALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 216, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [246] StringParser("ALSO")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 216, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [247] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 220, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [248] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 220, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [249] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 222, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [250] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 222, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [251] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 222, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [252] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 222, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [253] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 223, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [254] Ref(AlterOptionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 223, child_count: 10, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [255] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [256] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [257] Ref(DefinerSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [258] Ref(EventKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [259] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 233, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [260] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [261] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [262] Ref(ScheduleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [263] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 241, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [264] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 241, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [265] Ref(EveryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 241, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [266] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 241, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [267] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 242, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [268] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 242, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [269] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 243, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [270] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 245, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [271] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 247, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [272] Ref(StartsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 247, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [273] Ref(EndsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 247, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [274] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 247, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [275] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 251, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [276] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 251, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [277] Ref(CompletionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 251, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [278] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 251, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [279] Ref(PreserveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 251, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [280] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 254, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [281] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 254, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [282] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 254, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [283] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 254, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [284] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [285] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [286] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [287] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 260, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [288] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 260, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [289] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 260, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [290] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 262, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [291] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 262, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [292] Ref(SlaveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 262, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [293] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 262, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [294] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 264, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [295] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 264, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [296] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 264, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [297] StringParser("ALTER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 264, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [298] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 265, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [299] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 269, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [300] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 273, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [301] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 273, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [302] Ref(CharsetGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 273, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [303] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 273, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [304] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 277, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [305] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 277, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [306] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 277, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [307] Ref(DoubleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 277, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [308] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 277, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [309] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [310] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [311] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [312] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [313] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [314] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 285, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [315] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 285, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [316] Ref(EncryptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 285, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [317] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 285, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [318] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 285, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [319] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 289, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [320] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 289, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [321] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 289, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [322] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 289, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [323] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 291, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [324] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 291, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [325] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 291, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [326] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 297, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [327] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [328] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [329] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [330] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [331] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [332] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 300, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [333] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [334] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [335] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [336] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [337] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [338] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 306, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [339] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 306, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [340] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 306, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [341] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 306, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [342] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 310, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [343] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 310, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [344] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 310, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [345] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 310, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [346] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 311, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [347] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 311, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [348] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 315, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [349] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 315, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [350] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 315, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [351] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 315, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [352] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 315, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [353] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 317, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [354] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 320, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [355] Ref(AlgorithmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 320, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [356] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 320, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [357] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 324, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [358] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 324, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [359] Ref(InplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 324, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [360] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 324, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [361] Ref(InstantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 324, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [362] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 327, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [363] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 327, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [364] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 327, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [365] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 331, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [366] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 331, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [367] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 331, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [368] Ref(SharedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 331, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [369] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 331, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [370] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 335, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [371] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 338, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [372] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 338, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [373] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 338, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [374] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 340, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [375] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 340, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [376] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 340, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [377] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 344, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [378] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 346, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [379] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 346, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [380] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 346, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [381] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 346, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [382] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 346, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [383] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [384] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [385] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 352, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [386] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 352, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [387] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 352, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [388] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 352, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [389] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 352, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [390] Ref(AlterTableDropColumnGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 352, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [391] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 355, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [392] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 355, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [393] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 357, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [394] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 357, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [395] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 357, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [396] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 357, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [397] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 362, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [398] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 362, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [399] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 362, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [400] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 362, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [401] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 364, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [402] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [403] Ref(TableOptionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [404] Ref(AlterTableOnlineDDLOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [405] Ref(AddDropSystemVersioningGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [406] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 383, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [407] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 383, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [408] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 383, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [409] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 383, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [410] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 383, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [411] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 386, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [412] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 386, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [413] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 388, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [414] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 388, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [415] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 388, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [416] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 388, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [417] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 392, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [418] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 392, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [419] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 392, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [420] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 392, child_count: 2, min_times: 1, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [421] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 394, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [422] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 396, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [423] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [424] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [425] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [426] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [427] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [428] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [429] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [430] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [431] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 403, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [432] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 405, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [433] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 405, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [434] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [435] Ref(InvisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [436] Ref(VisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 407, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [437] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 411, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [438] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 411, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [439] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 411, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [440] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 411, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [441] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 414, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [442] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 414, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [443] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [444] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [445] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [446] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [447] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [448] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [449] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [450] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 423, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [451] Ref(ChangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 423, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [452] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 423, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [453] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 423, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [454] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 423, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [455] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 424, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [456] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 425, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [457] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 427, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [458] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 427, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [459] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 429, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [460] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 429, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [461] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 429, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [462] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [463] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [464] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [465] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 438, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [466] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 438, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [467] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 438, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [468] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 440, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [469] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [470] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [471] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [472] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [473] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [474] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 444, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [475] Ref(ForeignKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 444, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [476] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 444, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [477] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 446, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [478] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [479] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [480] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [481] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [482] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [483] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 452, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [484] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [485] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [486] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [487] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 454, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [488] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [489] Ref(EnforcedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 456, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [490] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 458, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [491] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 458, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [492] Ref(EnforcedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 458, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [493] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 462, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [494] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 462, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [495] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 462, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [496] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 462, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [497] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [498] Ref(VisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [499] Ref(InvisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [500] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 466, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [501] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 466, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [502] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 469, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [503] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 471, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [504] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 473, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [505] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 473, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [506] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 473, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [507] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 473, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [508] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 477, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [509] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [510] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [511] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [512] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [513] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [514] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [515] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 483, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [516] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 483, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [517] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 483, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [518] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 483, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [519] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 483, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [520] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 485, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [521] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 487, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [522] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 487, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [523] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 487, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [524] Ref(KeysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 487, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [525] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [526] Ref(ConvertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [527] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 490, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [528] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 491, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [529] Ref(AlterOptionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 491, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [530] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [531] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [532] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [533] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [534] Ref(DiscardKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [535] Ref(ImportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [536] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [537] Ref(CoalesceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [538] Ref(ReorganizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [539] Ref(ExchangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [540] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [541] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [542] Ref(OptimizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [543] Ref(RebuildKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [544] Ref(RepairKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [545] Ref(RemoveKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [546] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 513, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [547] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 513, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [548] Ref(PartitioningKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 513, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [549] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 517, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [550] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 517, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [551] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 517, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [552] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 517, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [553] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 520, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [554] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [555] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [556] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 522, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [557] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 527, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [558] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 527, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [559] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 527, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [560] Ref(TableReference)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 527, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [561] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 529, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [562] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 529, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [563] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 529, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [564] Ref(ValidationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 529, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [565] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 531, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [566] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 531, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [567] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 534, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [568] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 536, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [569] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 536, child_count: 10, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [570] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 546, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [571] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 546, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [572] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 549, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [573] Ref(AlgorithmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 549, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [574] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 549, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [575] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 552, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [576] Ref(UndefinedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 552, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [577] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 552, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [578] Ref(TemptableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 552, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [579] Ref(DefinerSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 552, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [580] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [581] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [582] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [583] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [584] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [585] Ref(InvokerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [586] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [587] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 557, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [588] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [589] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [590] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 559, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [591] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 562, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [592] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 562, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [593] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 562, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [594] StringParser("ALWAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 562, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [595] StringParser("&")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 562, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [596] StringParser("ANALYSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 562, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [597] StringParser("ANALYZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 562, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [598] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 566, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [599] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 566, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [600] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [601] Ref(No_write_to_binlogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [602] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [603] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 568, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [604] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 571, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [605] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 572, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [606] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 574, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [607] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 574, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [608] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 580, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [609] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 580, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [610] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 580, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [611] Ref(HistogramKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 580, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [612] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 580, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [613] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 582, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [614] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 582, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [615] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 585, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [616] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 585, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [617] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 585, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [618] Ref(BucketsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 585, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [619] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 590, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [620] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 590, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [621] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 590, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [622] Ref(HistogramKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 590, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [623] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 590, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [624] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [625] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [626] StringParser("ANALYZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [627] StringParser("ANALYZER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [628] StringParser("AND")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 592, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [629] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [630] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [631] StringParser("&&")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [632] StringParser("ANTI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [633] StringParser("ANY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [634] StringParser("APPEND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [635] StringParser("APPLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [636] StringParser("ARE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 594, child_count: 12, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [637] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [638] Ref(PlusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [639] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [640] Ref(DivideSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [641] Ref(MultiplySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [642] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [643] Ref(BitwiseAndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [644] Ref(BitwiseOrSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [645] Ref(BitwiseXorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [646] Ref(BitwiseLShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [647] Ref(BitwiseRShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [648] Ref(DivOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [649] Ref(ModOperatorSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [650] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 609, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [651] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 611, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [652] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 613, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [653] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 613, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [654] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 613, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [655] Ref(SliceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 613, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [656] StringParser("[")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 613, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [657] StringParser("]")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 613, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [658] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 613, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [659] StringParser("ARRAY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 613, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [660] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 616, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [661] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 618, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [662] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 618, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [663] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 618, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [664] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [665] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [666] StringParser("AS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [667] StringParser("ASC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [668] StringParser("ASCII")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [669] StringParser("ASENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [670] StringParser("ASSERTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [671] StringParser("ASSIGNMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [672] StringParser("ASYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [673] StringParser("AT")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [674] TypedParser("at_sign_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [675] StringParser("ATOMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [676] StringParser("ATTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [677] StringParser("ATTRIBUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [678] StringParser("AUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [679] StringParser("AUTHENTICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [680] StringParser("AUTHORIZATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [681] StringParser("AUTHORS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 619, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [682] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 620, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [683] Ref(Auto_incrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 620, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [684] StringParser("AUTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 620, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [685] StringParser("AUTO_INCREMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 620, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [686] StringParser("AUTOEXTEND_SIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 620, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [687] StringParser("AVG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 620, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [688] StringParser("AVG_ROW_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 620, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [689] StringParser("BACKEND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 620, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [690] StringParser("BACKENDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 620, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [691] StringParser("BACKUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 620, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [692] StringParser("BACKWARD")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 620, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [693] MultiStringParser(6 templates)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 620, child_count: 10, min_times: 0, first_terminator_idx: 3, terminator_count: 2, _padding: 0 }, // [694] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [695] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [696] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [697] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [698] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [699] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [700] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [701] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [702] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [703] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [704] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [705] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [706] Ref(VariableAssignmentSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [707] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [708] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [709] Token("file")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [710] Token("base")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [711] StringParser("BEFORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [712] StringParser("BEGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [713] StringParser("BELONG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [714] StringParser("BERNOULLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [715] StringParser("BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [716] StringParser("BIGINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [717] StringParser("BIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [718] StringParser("BINARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [719] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [720] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [721] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [722] Ref(BooleanBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [723] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [724] Ref(ColumnPathOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [725] Ref(InlinePathOperatorSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [726] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [727] StringParser("BINDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [728] StringParser("BINLOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [729] StringParser("BIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [730] StringParser("BIT_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [731] StringParser("BITAND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [732] StringParser("BITMAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [733] StringParser("BITMAP_EMPTY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [734] StringParser("BITMAP_UNION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [735] StringParser("BITOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [736] StringParser("BITVAR")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [737] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 638, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [738] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [739] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [740] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [741] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 640, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [742] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 642, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [743] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 642, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [744] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 642, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [745] StringParser("^")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 642, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [746] StringParser("BITXOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 642, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [747] StringParser("BLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 642, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [748] StringParser("BLOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 642, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [749] StringParser("BLOOM_FILTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 642, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [750] StringParser("BOOL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 642, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [751] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [752] Ref(AndOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [753] Ref(OrOperatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [754] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 646, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [755] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 648, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [756] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 648, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [757] Ref(OffKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 648, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [758] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 650, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [759] Ref(TrueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 650, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [760] Ref(FalseKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 650, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [761] StringParser("BOOLEAN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 650, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [762] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [763] Ref(TrueSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [764] Ref(FalseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [765] StringParser("BOTH")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [766] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 655, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [767] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [768] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [769] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 660, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [770] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 662, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [771] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 662, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [772] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 665, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [773] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 667, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [774] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 669, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [775] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [776] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [777] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 674, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [778] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 674, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [779] Ref(IndexColumnPrefixLengthSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 674, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [780] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 677, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [781] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 677, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [782] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 679, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [783] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 679, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [784] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 679, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [785] Token("bracketed")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 679, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [786] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [787] Ref(UnorderedSetExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [788] StringParser("BRANCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [789] StringParser("BREADTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [790] StringParser("BREAK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [791] StringParser("BRIEF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [792] StringParser("BROKER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [793] StringParser("BROWSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [794] StringParser("BTREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [795] StringParser("BUCKETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [796] StringParser("BUILD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [797] StringParser("BUILTIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [798] StringParser("BULK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [799] StringParser("BY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [800] StringParser("BYTE")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 682, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [801] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 685, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [802] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 685, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [803] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 689, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [804] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 689, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [805] Ref(CTEColumnList)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 689, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [806] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 689, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [807] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 692, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [808] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 692, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [809] StringParser("CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 692, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [810] StringParser("CACHED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 692, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [811] StringParser("CALL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 692, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [812] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 694, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [813] Ref(CallKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 694, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [814] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 694, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [815] StringParser("CALLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 694, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [816] StringParser("CANCEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 694, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [817] StringParser("CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 694, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [818] StringParser("CASCADE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 694, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [819] StringParser("CASCADED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 694, child_count: 2, min_times: 0, first_terminator_idx: 11, terminator_count: 3, _padding: 0 }, // [820] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 696, child_count: 6, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [821] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [822] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [823] Meta("implicit_indent")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 702, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 2, _padding: 0 }, // [824] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 703, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [825] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 703, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [826] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 703, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [827] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 703, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 1, _padding: 0 }, // [828] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 703, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [829] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 703, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [830] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 703, child_count: 7, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [831] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [832] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [833] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 710, child_count: 1, min_times: 0, first_terminator_idx: 8, terminator_count: 2, _padding: 0 }, // [834] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [835] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [836] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [837] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 1, _padding: 0 }, // [838] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [839] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [840] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [841] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [842] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [843] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [844] StringParser("CASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [845] StringParser("CAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [846] StringParser("::")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [847] StringParser("CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [848] StringParser("CATALOG_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [849] StringParser("CATALOGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [850] StringParser("CEIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [851] StringParser("CEILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [852] StringParser("CHAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [853] StringParser("CHALLENGE_RESPONSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [854] StringParser("CHANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [855] StringParser("CHANGED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [856] StringParser("CHANNEL")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [857] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [858] StringParser("CHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [859] StringParser("CHAR_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [860] StringParser("CHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [861] StringParser("CHARACTER_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [862] StringParser("CHARACTER_SET_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [863] StringParser("CHARACTER_SET_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [864] StringParser("CHARACTER_SET_SCHEMA")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 4, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [865] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 715, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [866] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [867] Ref(DeterministicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [868] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [869] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [870] Ref(DeterministicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 719, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [871] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 721, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [872] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 721, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [873] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 721, child_count: 4, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [874] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 725, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [875] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [876] Ref(ContainsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [877] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 727, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [878] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 729, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [879] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 729, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [880] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 729, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [881] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [882] Ref(ReadsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [883] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [884] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 732, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [885] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [886] Ref(ModifiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [887] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [888] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 735, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [889] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 738, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [890] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 738, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [891] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 738, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [892] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 740, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [893] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 740, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [894] Ref(InvokerKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 740, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [895] StringParser("CHARACTERISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 740, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [896] StringParser("CHARACTERS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 740, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [897] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [898] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 744, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [899] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 744, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [900] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 744, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [901] Ref(CharsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 744, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [902] StringParser("CHARSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 744, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [903] StringParser("CHECK")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 744, child_count: 4, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [904] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 748, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [905] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 748, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [906] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 748, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [907] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 750, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [908] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 750, child_count: 6, min_times: 1, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [909] AnyNumberOf(6 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [910] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 758, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [911] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 758, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [912] Ref(UpgradeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 758, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [913] Ref(QuickKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 758, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [914] Ref(FastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 758, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [915] Ref(MediumKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 758, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [916] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 758, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [917] Ref(ChangedKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 758, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [918] StringParser("CHECKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 758, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [919] StringParser("CHECKPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 758, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [920] StringParser("CHECKSUM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 758, child_count: 4, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [921] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [922] Ref(ChecksumKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [923] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [924] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 764, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [925] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 764, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [926] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [927] Ref(QuickKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [928] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [929] StringParser("CIPHER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [930] StringParser("CLASS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [931] StringParser("CLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [932] StringParser("CLEAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [933] StringParser("CLIENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [934] StringParser("CLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [935] StringParser("CLONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [936] StringParser("CLOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [937] StringParser("CLUSTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [938] StringParser("CLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [939] StringParser("CLUSTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [940] StringParser("COALESCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [941] StringParser("COBOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [942] StringParser("CODE")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [943] Token("raw")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 766, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [944] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [945] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [946] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [947] StringParser("COLLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [948] StringParser("COLLATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [949] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [950] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 770, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 1, _padding: 0 }, // [951] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 772, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [952] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 772, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [953] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 772, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [954] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 772, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [955] StringParser("COLLATION_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 772, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [956] StringParser("COLLATION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 772, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [957] StringParser("COLLATION_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 772, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [958] StringParser("COLLECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 772, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [959] StringParser("COLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 772, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [960] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 772, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [961] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 772, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [962] StringParser(":")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 772, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [963] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 774, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [964] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 777, child_count: 4, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [965] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [966] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [967] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [968] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [969] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [970] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 6, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [971] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 790, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [972] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 792, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [973] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [974] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [975] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 10, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [976] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 804, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [977] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [978] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [979] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 806, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [980] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [981] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [982] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [983] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [984] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 813, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [985] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 813, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [986] Ref(ColumnConstraintDefaultGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 813, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [987] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [988] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [989] Ref(NotEnforcedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [990] Ref(UniqueKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [991] Ref(AutoIncrementGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [992] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 817, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [993] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 817, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [994] Ref(NotEnforcedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 817, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [995] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 817, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [996] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [997] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [998] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [999] Ref(ColumnGeneratedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1000] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 821, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1001] Ref(CharsetGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 821, child_count: 4, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1002] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1003] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1004] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1005] Ref(DoubleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1006] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1007] Ref(CollateGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 4, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1008] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 829, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1009] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1010] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1011] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1012] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1013] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 834, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1014] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 834, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1015] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1016] Ref(StoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1017] Ref(VirtualKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1018] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 838, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1019] Ref(SridKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 838, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1020] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 838, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1021] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 840, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1022] Ref(InvisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 840, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1023] Ref(VisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 840, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1024] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1025] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1026] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 4, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1027] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 846, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1028] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 846, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1029] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 846, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1030] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 846, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1031] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 849, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1032] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 849, child_count: 5, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1033] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 854, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1034] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 854, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1035] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 856, child_count: 1, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1036] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 857, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1037] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 859, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1038] Ref(DatetimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 859, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1039] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 859, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1040] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 862, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1041] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 864, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1042] Ref(DatetimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 864, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1043] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 864, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1044] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 864, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1045] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 867, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1046] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 869, child_count: 1, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1047] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1048] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1049] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 870, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1050] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 872, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1051] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 872, child_count: 5, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1052] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 877, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1053] Ref(CurrentTimestampLikeFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 877, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1054] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 877, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1055] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 877, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1056] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 877, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1057] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 877, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1058] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 880, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1059] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 880, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1060] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 880, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1061] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 882, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1062] Ref(CurrentTimestampLikeFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 882, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1063] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 882, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1064] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 885, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1065] Anything
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 885, child_count: 1, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1066] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 886, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1067] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 886, child_count: 7, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1068] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 893, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1069] Ref(MaxKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 893, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1070] Ref(MinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 893, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1071] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 893, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1072] Ref(SumKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 893, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1073] Ref(Bitmap_unionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 893, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1074] Ref(Hll_unionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 893, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1075] Ref(Quantile_unionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 893, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1076] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 893, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1077] StringParser("COLUMN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 893, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1078] StringParser("->")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 893, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 1, _padding: 0 }, // [1079] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1080] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1081] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1082] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1083] StringParser("COLUMN_FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1084] StringParser("COLUMN_NAME")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1085] Nothing
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1086] Ref(ColumnsExpressionNameGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1087] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1088] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1089] StringParser("COLUMNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1090] StringParser(",")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1091] StringParser("COMMAND_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1092] StringParser("COMMAND_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 895, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1093] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1094] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1095] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1096] StringParser("COMMENT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1097] Token("comment")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1098] StringParser("COMMENT_START")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1099] StringParser("COMMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1100] StringParser("COMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1101] StringParser("COMPACT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 897, child_count: 9, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1102] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1103] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1104] Ref(GreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1105] Ref(LessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1106] Ref(GreaterThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1107] Ref(LessThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1108] Ref(NotEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1109] Ref(LikeOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1110] Ref(IsDistinctFromGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1111] Ref(NullSafeEqualsSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1112] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1113] StringParser("COMPLETE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1114] StringParser("COMPLETION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1115] StringParser("COMPONENT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1116] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1117] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1118] StringParser("COMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1119] StringParser("COMPRESS_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1120] StringParser("COMPRESSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1121] StringParser("COMPRESSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1122] StringParser("COMPUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 906, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1123] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 908, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1124] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 908, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1125] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 908, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1126] StringParser("CONCURRENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 908, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1127] StringParser("CONDITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 908, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1128] StringParser("CONDITION_NUMBER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 908, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1129] Ref(CrossKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 908, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1130] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1131] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1132] Ref(ConditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1133] Ref(NonStandardJoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1134] StringParser("CONDITIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1135] StringParser("CONFIG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1136] StringParser("CONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1137] StringParser("CONNECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1138] StringParser("CONNECTION_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1139] StringParser("CONNECTION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1140] StringParser("CONSISTENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1141] StringParser("CONSTRAINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1142] StringParser("CONSTRAINT_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1143] StringParser("CONSTRAINT_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1144] StringParser("CONSTRAINT_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1145] StringParser("CONSTRAINTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1146] StringParser("CONSTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1147] StringParser("CONTAINS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1148] StringParser("CONTAINSTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1149] StringParser("CONTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1150] StringParser("CONTINUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1151] StringParser("CONVERSION")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 911, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1152] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1153] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1154] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 917, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1155] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 919, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1156] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 919, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1157] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 919, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1158] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1159] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1160] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1161] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1162] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1163] StringParser("CONVERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1164] StringParser("CONVERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1165] StringParser("CONVERT_LSC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1166] StringParser("COPY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1167] StringParser("CORR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1168] StringParser("CORRESPONDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1169] StringParser("COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1170] StringParser("COVAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1171] StringParser("COVAR_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1172] StringParser("CPU")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 923, child_count: 10, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1173] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1174] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1175] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1176] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 938, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1177] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 938, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1178] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 938, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1179] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 938, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1180] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 938, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1181] Ref(SpecificKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 938, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1182] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 942, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1183] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 942, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1184] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 942, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1185] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 942, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1186] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 944, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1187] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1188] Ref(InstanceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1189] Ref(StaticKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1190] Ref(ConstructorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1191] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1192] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1193] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 947, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1194] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 949, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1195] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 949, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1196] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 949, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1197] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 951, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1198] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 951, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1199] Ref(AssignmentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 951, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1200] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1201] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1202] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 958, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1203] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 958, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1204] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 958, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1205] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 958, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1206] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 958, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1207] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1208] Ref(CreateOptionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 16, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1209] Sequence(16 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1210] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1211] Ref(DefinerSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1212] Ref(EventKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1213] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1214] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1215] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1216] Ref(ScheduleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 975, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1217] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1218] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1219] Ref(EveryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1220] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 977, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1221] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 978, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1222] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 978, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1223] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 979, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1224] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 981, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1225] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 983, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1226] Ref(StartsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 983, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1227] Ref(EndsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 983, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1228] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 983, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1229] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1230] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1231] Ref(CompletionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1232] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1233] Ref(PreserveKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 987, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1234] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1235] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1236] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 990, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1237] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1238] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1239] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 993, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1240] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 995, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1241] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 995, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1242] Ref(SlaveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 995, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1243] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 995, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1244] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 995, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1245] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 995, child_count: 9, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1246] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1247] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1248] Ref(DefinerSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1249] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1250] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1251] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1004, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1252] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1253] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1254] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1255] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1256] Ref(CharacteristicStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1257] Ref(FunctionDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 10, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1258] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1016, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1259] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1016, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1260] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1261] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1262] Ref(FulltextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1263] Ref(SpatialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1264] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1265] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1266] Ref(IndexTypeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1267] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1268] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1269] Ref(BracketedKeyPartListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1270] Ref(IndexOptionsSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1019, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1271] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1021, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1272] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1273] Ref(AlgorithmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1024, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1274] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1275] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1276] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1277] Ref(InplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1278] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1279] Ref(NocopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1280] Ref(InstantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1281] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1032, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1282] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1032, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1283] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1032, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1284] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1285] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1286] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1287] Ref(SharedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1288] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1289] StringParser("CREATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 8, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1290] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1291] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1044, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1292] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1293] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1044, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1294] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1295] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1044, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1296] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1046, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1297] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1046, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1298] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1049, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1299] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1051, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1300] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1054, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1301] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1054, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1302] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1054, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1303] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1056, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1304] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1056, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1305] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1059, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1306] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1061, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1307] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1061, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1308] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1061, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1309] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1061, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1310] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1063, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1311] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1063, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1312] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1066, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1313] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1070, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1314] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1070, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1315] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1070, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1316] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1070, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1317] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1072, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1318] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1072, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1319] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1072, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1320] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1321] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1322] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1323] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1324] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1078, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1325] Ref(EncryptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1078, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1326] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1078, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1327] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1078, child_count: 9, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1328] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1087, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1329] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1087, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1330] Ref(DefinerSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1087, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1331] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1087, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1332] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1087, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1333] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1087, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1334] Ref(ProcedureParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1087, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1335] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1087, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1336] Ref(CharacteristicStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1087, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1337] Ref(FunctionDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1087, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1338] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1091, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1339] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1091, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1340] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1091, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1341] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1091, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1342] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1091, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1343] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1344] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1345] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1346] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1347] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 7, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1348] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1349] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1105, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1350] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1105, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1351] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1105, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1352] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1105, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1353] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1354] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1355] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1356] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1357] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1358] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1359] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1110, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1360] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1112, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1361] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1112, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1362] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1112, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1363] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1112, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1364] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1114, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1365] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1114, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1366] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1114, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1367] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1114, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1368] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1118, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1369] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1118, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1370] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1118, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1371] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1118, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1372] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1119, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1373] Ref(CreateSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1119, child_count: 8, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1374] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1375] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1376] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1377] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1378] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1379] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1380] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1381] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1382] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1130, child_count: 8, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1383] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1138, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1384] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1385] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1143, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1386] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1146, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1387] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1146, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1388] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1146, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1389] Ref(IndexDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1146, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1390] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1391] Ref(EngineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1392] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1393] Ref(EngineTypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1149, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1394] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1152, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1395] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1396] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1397] Ref(DuplicateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1398] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1157, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1399] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1400] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1401] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1402] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1161, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1403] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1161, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1404] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1161, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1405] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1164, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1406] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1166, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1407] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1166, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1408] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1169, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1409] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1169, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1410] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1169, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1411] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1172, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1412] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1413] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1174, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1414] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1174, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1415] Ref(PartitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1174, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1416] Ref(DistributionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1174, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1417] Ref(RollupSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1174, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1418] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1419] Ref(PropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1420] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1421] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1181, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1422] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1184, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1423] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1184, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1424] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1184, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1425] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1184, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1426] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1186, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1427] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1186, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1428] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1186, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1429] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1191, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1430] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1194, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1431] Ref(EngineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1194, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1432] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1194, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1433] Ref(EngineTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1194, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1434] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1194, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1435] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1436] Ref(PropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1437] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1199, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1438] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1201, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1439] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1204, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1440] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1204, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1441] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1204, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1442] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1204, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1443] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1204, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1444] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1206, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1445] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1209, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1446] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1209, child_count: 13, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1447] Sequence(13 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1222, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1448] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1222, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1449] Ref(TriggerOrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1222, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1450] Ref(DefinerSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1222, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1451] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1222, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1452] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1222, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1453] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1222, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1454] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1455] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1456] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1457] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1458] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1459] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1460] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1461] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1462] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1463] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1230, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1464] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1230, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1465] Ref(EachKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1230, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1466] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1230, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1467] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1468] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1469] Ref(FollowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1470] Ref(PrecedesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1471] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1234, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1472] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1236, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1473] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1236, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1474] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1239, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1475] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1239, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1476] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1239, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1477] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1239, child_count: 10, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1478] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1249, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1479] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1249, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1480] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1249, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1481] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1249, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1482] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1251, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1483] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1484] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1253, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1485] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1254, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1486] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1256, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1487] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1258, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1488] Ref(IdentifiedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1258, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1489] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1490] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1491] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1492] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1264, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1493] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1266, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1494] Ref(RandomKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1266, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1495] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1266, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1496] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1266, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1497] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1269, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1498] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1269, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1499] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1269, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1500] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1270, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1501] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1273, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1502] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1275, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1503] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1275, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1504] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1277, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1505] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1506] Ref(RandomKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1507] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1508] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1509] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1281, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1510] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1281, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1511] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1281, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1512] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1285, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1513] Ref(InitialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1285, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1514] Ref(AuthenticationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1285, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1515] Ref(IdentifiedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1285, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1516] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1287, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1517] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1289, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1518] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1289, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1519] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1291, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1520] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1521] Ref(RandomKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1522] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1523] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1524] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1297, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1525] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1297, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1526] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1297, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1527] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1297, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1528] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1297, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1529] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1297, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1530] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1531] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1532] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1300, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1533] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1302, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1534] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1302, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1535] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1536] Ref(RequireKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1537] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1306, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1538] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1306, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1539] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1308, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1540] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1541] Ref(SslKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1542] Ref(X509KeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1313, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1543] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1544] Ref(CipherKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1545] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1315, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1546] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1317, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1547] Ref(IssuerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1317, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1548] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1317, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1549] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1319, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1550] Ref(SubjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1319, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1551] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1319, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1552] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1319, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1553] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1321, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1554] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1321, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1555] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1322, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1556] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1324, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1557] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1558] Ref(Max_queries_per_hourKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1559] Ref(Max_updates_per_hourKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1560] Ref(Max_connections_per_hourKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1561] Ref(Max_user_connectionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1562] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1328, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1563] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1329, child_count: 6, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1564] AnyNumberOf(6 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1335, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1565] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1338, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1566] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1338, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1567] Ref(ExpireKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1338, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1568] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1339, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1569] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1342, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1570] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1342, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1571] Ref(NeverKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1342, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1572] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1345, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1573] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1345, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1574] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1345, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1575] Ref(DayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1345, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1576] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1348, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1577] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1348, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1578] Ref(HistoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1348, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1579] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1350, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1580] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1350, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1581] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1350, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1582] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1354, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1583] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1354, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1584] Ref(ReuseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1354, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1585] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1354, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1586] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1356, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1587] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1356, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1588] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1589] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1590] Ref(DayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1591] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1362, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1592] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1362, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1593] Ref(RequireKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1362, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1594] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1362, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1595] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1363, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1596] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1597] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1598] Ref(OptionalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1599] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1367, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1600] Ref(Failed_login_attemptsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1367, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1601] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1367, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1602] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1369, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1603] Ref(Password_lock_timeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1369, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1604] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1371, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1605] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1371, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1606] Ref(UnboundedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1371, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1607] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1373, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1608] Ref(AccountKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1373, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1609] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1375, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1610] Ref(UnlockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1375, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1611] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1375, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1612] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1613] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1379, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1614] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1379, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1615] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1379, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1616] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1379, child_count: 11, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1617] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1390, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1618] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1390, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1619] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1390, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1620] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1393, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1621] Ref(AlgorithmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1393, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1622] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1393, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1623] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1624] Ref(UndefinedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1625] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1626] Ref(TemptableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1627] Ref(DefinerSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1396, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1628] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1399, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1629] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1399, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1630] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1399, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1631] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1632] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1633] Ref(InvokerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1634] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1635] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1636] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1637] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1638] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1403, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1639] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1640] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1641] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1642] StringParser("CREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1643] StringParser("CREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1644] StringParser("CREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1645] StringParser("CREATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1646] StringParser("CRON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1647] StringParser("CROSS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1648] StringParser("CSV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1649] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1650] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1651] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1652] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1653] Ref(CubeFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1654] Ref(RollupFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1655] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1656] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1657] StringParser("CUME_DIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1658] StringParser("CURRENT")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1413, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1659] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1416, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1660] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1416, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1661] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1662] Ref(Current_timestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1663] Ref(NowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1664] Ref(LocaltimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1665] Ref(LocaltimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1666] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1667] Ref(CurrentTimestampLikeFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1668] Ref(CurrentTimestampLikeFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1669] StringParser("CURRENT_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1670] StringParser("CURRENT_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1671] StringParser("CURRENT_DEFAULT_TRANSFORM_GROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1672] StringParser("CURRENT_PATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1673] StringParser("CURRENT_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1674] StringParser("CURRENT_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1675] StringParser("CURRENT_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1676] StringParser("CURRENT_TRANSFORM_GROUP_FOR_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1677] StringParser("CURRENT_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1678] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1427, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1679] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1427, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1680] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1429, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1681] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1429, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1682] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1429, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1683] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1429, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1684] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1429, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1685] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1431, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1686] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1433, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1687] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1433, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1688] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1433, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1689] StringParser("CURSOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1433, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1690] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1435, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1691] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1692] Ref(CloseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1693] Ref(OpenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1694] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1695] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1696] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1697] StringParser("CURSOR_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1698] StringParser("CYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1699] StringParser("DATA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1700] StringParser("DATABASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1701] StringParser("DATABASES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1702] StringParser("DATAFILE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1703] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1441, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1704] RegexParser
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1441, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1705] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1706] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1707] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1447, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1708] Ref(TimeWithTZGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1447, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1709] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1449, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1710] Ref(DoubleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1449, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1711] Ref(PrecisionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1449, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1712] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1451, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1713] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1453, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1714] Ref(SignedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1453, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1715] Ref(UnsignedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1453, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1716] Ref(IntegerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1453, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1717] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1456, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1718] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1458, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1719] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1720] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1721] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1722] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1723] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1464, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1724] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1464, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1725] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1466, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1726] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1466, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1727] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1466, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1728] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 1468, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1729] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1730] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1731] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1732] Ref(DatatypeIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1470, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1733] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1470, child_count: 6, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1734] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1476, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1735] Ref(CharCharacterSetGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1476, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1736] Ref(SignedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1476, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1737] Ref(UnsignedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1476, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1738] Ref(ZerofillKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1476, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1739] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1478, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1740] Ref(ZerofillKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1478, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1741] Ref(UnsignedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1478, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1742] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1743] Ref(UnsignedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1744] Ref(ZerofillKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1745] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1746] StringParser("DATE")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1747] MultiStringParser(3 templates)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1748] Ref(DatePartFunctionName)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1749] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1750] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1484, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1751] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1486, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1752] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1488, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1753] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1488, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1754] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1488, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1755] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1490, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1756] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1493, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1757] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1493, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1758] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1493, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1759] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1493, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1760] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1761] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1762] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1763] StringParser("DATETIME")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1764] MultiStringParser(20 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1765] StringParser("DATETIME_INTERVAL_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1766] StringParser("DATETIME_INTERVAL_PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1767] StringParser("DATETIMEV1")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1768] StringParser("DATETIMEV2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1769] StringParser("DATEV1")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1770] StringParser("DATEV2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1771] StringParser("DAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1772] StringParser("DAY_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1773] StringParser("DAY_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1774] StringParser("DAY_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1775] StringParser("DAY_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1776] StringParser("DAYOFMONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1777] StringParser("DAYOFWEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1778] StringParser("DAYOFYEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1779] StringParser("DAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1780] StringParser("DBCC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1781] StringParser("DEALLOCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1495, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1782] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1497, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1783] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1784] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1501, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1785] Ref(DeallocateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1501, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1786] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1501, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1787] Ref(PrepareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1501, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1788] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1501, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1789] StringParser("DEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1501, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1790] StringParser("DECIMAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1501, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1791] StringParser("DECIMALV2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1501, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1792] StringParser("DECIMALV3")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1501, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1793] StringParser("DECLARE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1501, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1794] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1505, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1795] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1510, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1796] Ref(DeclareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1510, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1797] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1510, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1798] Ref(CursorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1510, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1799] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1510, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1800] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1510, child_count: 6, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1801] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1516, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1802] Ref(DeclareKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1516, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1803] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1804] Ref(ContinueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1805] Ref(ExitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1806] Ref(UndoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1807] Ref(HandlerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1808] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1809] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1810] Ref(SqlexceptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1811] Ref(SqlwarningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1812] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1526, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1813] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1526, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1814] Ref(FoundKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1526, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1815] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1529, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1816] Ref(SqlstateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1529, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1817] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1529, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1818] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1529, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1819] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1820] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1821] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1822] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1532, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1823] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1533, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1824] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1533, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1825] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1538, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1826] Ref(DeclareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1538, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1827] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1538, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1828] Ref(ConditionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1538, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1829] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1538, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1830] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1831] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1832] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1833] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1544, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1834] Ref(DeclareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1544, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1835] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1544, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1836] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1544, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1837] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1546, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1838] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1546, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1839] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1549, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1840] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1549, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1841] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1549, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1842] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1549, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1843] StringParser("DECOMMISSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1549, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1844] StringParser("DEFAULT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1549, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1845] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1846] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1847] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1848] StringParser("DEFAULT_AUTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1849] StringParser("DEFAULTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1850] StringParser("DEFERRABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1851] StringParser("DEFERRED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1852] StringParser("DEFINED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1853] StringParser("DEFINER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1551, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1854] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1855] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1856] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1857] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1858] StringParser("DEFINITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1859] StringParser("DEGREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1860] StringParser("DELAY_KEY_WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1861] StringParser("DELAYED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1862] StringParser("DELETE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 5, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1863] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1559, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1864] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1559, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1865] Ref(Low_priorityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1559, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1866] Ref(QuickKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1559, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1867] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1559, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1868] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1562, child_count: 4, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1869] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1870] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 1, _padding: 0 }, // [1871] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1872] Ref(DeleteTargetTableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1873] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1874] Ref(DeleteUsingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1875] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1568, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1876] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1571, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 1, _padding: 0 }, // [1877] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1573, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1878] Ref(DeleteTargetTableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1573, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1879] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1573, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1880] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1573, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1881] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1573, child_count: 5, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1882] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1883] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1884] Ref(SelectPartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1885] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1886] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1887] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1888] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1580, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1889] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1580, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1890] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1582, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1891] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1582, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1892] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1582, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1893] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1584, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1894] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1584, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1895] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1586, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1896] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1586, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1897] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1898] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1899] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1900] StringParser("DELIMITER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1901] Ref(DelimiterKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1902] StringParser("DELIMITERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1903] StringParser("DEMAND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1904] StringParser("DENSE_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1905] StringParser("DENY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1906] StringParser("DEPTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1907] StringParser("DEREF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1908] StringParser("DERIVED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1909] StringParser("DES_KEY_FILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1910] StringParser("DESC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1911] StringParser("DESCRIBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1588, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1912] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1913] Ref(DescribeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1914] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1915] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1916] StringParser("DESCRIPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1917] StringParser("DESCRIPTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1918] StringParser("DESTROY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1919] StringParser("DESTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1920] StringParser("DETERMINISTIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1921] StringParser("DIAGNOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1922] StringParser("DIAGNOSIS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1923] StringParser("DIAGNOSTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1924] StringParser("DICTIONARIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1925] StringParser("DICTIONARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1926] StringParser("DIRECTORY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1927] StringParser("DISABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1928] StringParser("DISCARD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1929] StringParser("DISCONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1930] StringParser("DISK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1931] StringParser("DISPATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1932] StringParser("DISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1933] StringParser("DISTINCTPC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1934] StringParser("DISTINCTPCSA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1935] StringParser("DISTINCTROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1936] StringParser("DISTRIBUTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1937] StringParser("DISTRIBUTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1591, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1938] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1594, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1939] Ref(DistributedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1594, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1940] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1594, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1941] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1596, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1942] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1943] Ref(HashKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1599, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1944] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1602, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1945] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1946] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1604, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1947] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1606, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1948] Ref(BucketsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1606, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1949] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1608, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1950] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1608, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1951] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1608, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1952] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1610, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1953] Ref(RandomKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1610, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1954] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1612, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1955] Ref(BucketsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1612, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1956] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1957] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1958] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1959] StringParser("DIV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1960] StringParser("DIV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1961] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1962] StringParser("DO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1963] StringParser("DOMAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1964] StringParser("DORIS_INTERNAL_TABLE_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1965] StringParser(".")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1966] StringParser("DOUBLE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1967] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1968] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1969] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1614, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1970] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1971] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1972] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1973] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1620, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1974] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1620, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1975] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1620, child_count: 5, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1976] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1625, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1977] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1625, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1978] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1625, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1979] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1625, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1980] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1625, child_count: 5, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1981] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1630, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1982] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1630, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1983] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1630, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1984] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1630, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1985] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1630, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1986] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1630, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1987] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1988] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1989] Ref(EventKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1990] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1991] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1992] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1638, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1993] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1638, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1994] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1638, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1995] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1638, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1996] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1638, child_count: 6, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1997] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1998] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1999] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2000] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2001] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2002] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1644, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2003] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1646, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2004] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2005] Ref(AlgorithmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1649, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2006] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2007] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2008] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2009] Ref(InplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2010] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2011] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1655, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2012] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1655, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2013] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1655, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2014] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1659, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2015] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1659, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2016] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1659, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2017] Ref(SharedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1659, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2018] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1659, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2019] StringParser("DROP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1659, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2020] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2021] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2022] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2023] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2024] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2025] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2026] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2027] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1669, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2028] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1669, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2029] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1669, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2030] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1669, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2031] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1669, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2032] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2033] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2034] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2035] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2036] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 5, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2037] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1678, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2038] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1678, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2039] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1678, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2040] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1678, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2041] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1678, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2042] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1678, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2043] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1681, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2044] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1681, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2045] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1681, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2046] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1681, child_count: 5, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2047] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2048] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2049] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2050] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2051] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1686, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2052] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2053] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1687, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2054] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1691, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2055] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1691, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2056] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1691, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2057] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1691, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2058] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1691, child_count: 5, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2059] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2060] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2061] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2062] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2063] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2064] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2065] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1700, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2066] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1700, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2067] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1700, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2068] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1700, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2069] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1700, child_count: 5, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2070] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2071] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2072] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2073] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2074] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2075] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2076] StringParser("DROPP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2077] StringParser("DUAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2078] StringParser("DUAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2079] StringParser("DUMMY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2080] StringParser("DUMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2081] StringParser("DUMPFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2082] StringParser("DUPLICATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2083] StringParser("DYNAMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2084] StringParser("DYNAMIC_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2085] StringParser("DYNAMIC_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2086] StringParser("E")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2087] StringParser("EACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2088] StringParser("ELEMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2089] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2090] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2091] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2092] StringParser("ELSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2093] StringParser("ELSEIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2094] StringParser("EMPTY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1709, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2095] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1711, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2096] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2097] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2098] Ref(EmptyStructLiteralBracketsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2099] StringParser("ENABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2100] StringParser("ENCLOSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2101] StringParser("ENCODING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2102] StringParser("ENCRYPTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2103] StringParser("ENCRYPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2104] StringParser("ENCRYPTKEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2105] StringParser("ENCRYPTKEYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2106] StringParser("END-EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2107] StringParser("}")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2108] StringParser("END")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2109] StringParser("ENDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2110] StringParser("ENFORCED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2111] StringParser("ENGINE")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2112] MultiStringParser(8 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2113] StringParser("ENGINE_ATTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2114] StringParser("ENGINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2115] StringParser("ENTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2116] StringParser("ENUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2117] StringParser("EQUALS")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2118] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2119] StringParser("ERRLVL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2120] StringParser("ERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2121] StringParser("ERRORS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2122] StringParser("ESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2123] StringParser("ESCAPED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2124] StringParser("EVENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2125] StringParser("EVENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2126] StringParser("EVERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2127] StringParser("EXCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2128] StringParser("EXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2129] StringParser("EXCHANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2130] StringParser("EXCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2131] StringParser("EXCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2132] StringParser("EXCLUSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2133] StringParser("EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2134] StringParser("EXECUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1713, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2135] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1716, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2136] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1716, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2137] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1716, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2138] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1718, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2139] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1718, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2140] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2141] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2142] StringParser("EXECUTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2143] StringParser("EXISTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2144] StringParser("EXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2145] StringParser("EXIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2146] StringParser("EXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2147] StringParser("EXPANSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2148] StringParser("EXPIRE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2149] StringParser("EXPIRED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2150] StringParser("EXPLAIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1720, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2151] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2152] Ref(ExplainKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2153] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2154] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2155] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2156] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2157] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2158] StringParser("EXPORT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2159] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2160] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2161] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1728, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2162] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1729, child_count: 10, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2163] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2164] Ref(LikeExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2165] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2166] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2167] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2168] Ref(InOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2169] Ref(QuantifiedComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1741, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2170] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2171] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2172] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2173] Ref(IsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2174] Ref(IsNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2175] Ref(NotNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2176] Ref(CollateGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 5, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2177] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1749, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2178] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2179] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2180] Ref(Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2181] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2182] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2183] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1751, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2184] Ref(PatternMatchingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1751, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2185] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1751, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2186] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1755, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2187] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1756, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2188] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1757, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2189] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1757, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2190] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1757, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2191] Ref(NotOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1757, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2192] Ref(PriorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1757, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2193] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2194] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1759, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2195] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1760, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2196] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1761, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2197] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1763, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2198] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1766, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2199] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1766, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2200] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1766, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2201] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1766, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2202] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1766, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2203] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1768, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2204] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1769, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2205] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1770, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2206] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1770, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2207] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1770, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2208] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1772, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2209] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2210] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2211] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 1, _padding: 0 }, // [2212] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1777, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2213] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1779, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2214] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1779, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2215] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2216] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2217] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1784, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2218] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1786, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2219] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1786, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2220] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1786, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2221] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1787, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2222] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1787, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2223] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1787, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2224] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1787, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2225] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 9, min_times: 0, first_terminator_idx: 19, terminator_count: 1, _padding: 0 }, // [2226] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1798, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2227] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1798, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2228] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1798, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2229] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1801, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2230] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1805, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2231] Ref(LateralColumnAliasExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1805, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2232] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1805, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2233] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1805, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2234] Delimited(5 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1807, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2235] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2236] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2237] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2238] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2239] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2240] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2241] Ref(Expression_D_Potential_Select_Statement_Without_Brackets)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2242] Ref(SingleIdentifierWildcardSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2243] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1814, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2244] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2245] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2246] Ref(MapTypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2247] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2248] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1821, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2249] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1821, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2250] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2251] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1823, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2252] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2253] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2254] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2255] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2256] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2257] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2258] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2259] Ref(ListComprehensionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2260] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2261] Ref(AccessorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 9, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2262] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2263] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2264] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2265] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2266] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2267] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2268] Ref(TypedStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2269] Ref(ArrayExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2270] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2271] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2272] StringParser("EXTENDED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2273] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2274] StringParser("EXTENSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2275] StringParser("EXTENT_SIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2276] StringParser("EXTERNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2277] StringParser("EXTRACT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2278] StringParser("FACTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2279] StringParser("FAILED_LOGIN_ATTEMPTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2280] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2281] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2282] StringParser("FAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2283] StringParser("FAULTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2284] StringParser("FEATURE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2285] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1842, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2286] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1842, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2287] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2288] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2289] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1844, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2290] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1846, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2291] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1846, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2292] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1847, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2293] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1847, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2294] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2295] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2296] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1849, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2297] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1851, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2298] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1851, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2299] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2300] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2301] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2302] StringParser("FETCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2303] StringParser("FIELDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2304] StringParser("FILE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1853, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2305] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1856, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2306] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2307] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 1857, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2308] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2309] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1859, child_count: 1, min_times: 1, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2310] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2311] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1860, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2312] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1861, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2313] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1861, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2314] StringParser("FILE_BLOCK_SIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1861, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2315] StringParser("FILLFACTOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1861, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2316] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1863, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2317] Ref(FilterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1863, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2318] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1866, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2319] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2320] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2321] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2322] StringParser("FILTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2323] StringParser("FINAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2324] StringParser("FINISH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2325] StringParser("FIRST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2326] StringParser("FIRST_VALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2327] StringParser("FIXED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2328] StringParser("FLOAT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2329] StringParser("FLOAT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2330] StringParser("FLOAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2331] StringParser("FLOOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2332] StringParser("FLUSH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2333] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2334] Ref(FlushKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1871, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2335] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2336] Ref(No_write_to_binlogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2337] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2338] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1876, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2339] Delimited(12 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1878, child_count: 12, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2340] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2341] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1892, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2342] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1892, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2343] Ref(LogsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1892, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2344] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2345] Ref(EngineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2346] Ref(LogsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2347] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1896, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2348] Ref(ErrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1896, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2349] Ref(LogsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1896, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2350] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1898, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2351] Ref(GeneralKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1898, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2352] Ref(LogsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1898, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2353] Ref(HostsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1898, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2354] Ref(LogsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1898, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2355] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1898, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2356] Ref(Optimizer_costsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1898, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2357] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1901, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2358] Ref(RelayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1901, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2359] Ref(LogsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1901, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2360] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1904, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2361] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1904, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2362] Ref(ChannelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1904, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2363] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1904, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2364] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1906, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2365] Ref(SlowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1906, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2366] Ref(LogsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1906, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2367] Ref(StatusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1906, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2368] Ref(User_resourcesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1906, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2369] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2370] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1909, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2371] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 1, _padding: 0 }, // [2372] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1912, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2373] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1912, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2374] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1912, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2375] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2376] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2377] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2378] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2379] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1918, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2380] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1918, child_count: 1, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2381] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1919, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 1, _padding: 0 }, // [2382] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2383] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2384] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1921, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2385] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2386] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2387] Ref(ExportKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2388] StringParser("FOLLOWER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2389] StringParser("FOLLOWING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2390] StringParser("FOLLOWS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1923, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2391] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1925, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2392] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1928, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2393] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2394] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2395] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1932, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2396] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1932, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2397] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1932, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2398] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2399] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1934, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2400] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1936, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2401] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1936, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2402] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1938, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2403] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1938, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2404] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1940, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2405] Ref(SkipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1940, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2406] Ref(LockedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1940, child_count: 4, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2407] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1944, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2408] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1944, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2409] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1944, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2410] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1944, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2411] Ref(ModeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1944, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2412] StringParser("FOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1944, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2413] StringParser("FORCE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1944, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2414] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1946, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2415] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1946, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2416] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1946, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2417] StringParser("FOREIGN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1946, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2418] StringParser("FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1946, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2419] StringParser("FORTRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1946, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2420] StringParser("FORWARD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1946, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2421] StringParser("FOUND")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1946, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2422] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2423] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2424] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1950, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2425] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1952, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2426] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1954, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2427] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1954, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2428] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1954, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2429] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 4, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2430] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2431] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2432] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2433] Ref(UnboundedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2434] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2435] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2436] Ref(PrecedingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2437] Ref(FollowingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 4, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2438] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1966, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2439] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1966, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2440] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1966, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2441] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1968, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2442] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1968, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2443] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1968, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2444] StringParser("FREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1968, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2445] StringParser("FREETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1968, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2446] StringParser("FREETEXTTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1968, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2447] StringParser("FREEZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1968, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2448] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1970, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2449] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1970, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2450] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1972, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2451] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1972, child_count: 17, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2452] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1989, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2453] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1989, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2454] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1989, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2455] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1991, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2456] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1991, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2457] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1991, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2458] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2459] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2460] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2461] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2462] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2463] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2464] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2465] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2466] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2467] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2468] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2469] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2470] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2471] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2472] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2473] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2474] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1995, child_count: 7, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2475] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2002, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2476] Ref(PreTableFunctionKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2002, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2477] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2478] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2007, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2479] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2007, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2480] Ref(TemporalQuerySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 2007, child_count: 1, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2481] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2008, child_count: 4, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2482] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2012, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2483] Ref(FromClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2012, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2484] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2012, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2485] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2012, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2486] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2012, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2487] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2015, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2488] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2015, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2489] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2015, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2490] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2015, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2491] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2015, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2492] Ref(PostTableExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2015, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2493] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2018, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2494] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2020, child_count: 1, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2495] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2021, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2496] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2021, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2497] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2023, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2498] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 6, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2499] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2032, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 2, _padding: 0 }, // [2500] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2501] Ref(MLTableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2502] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2503] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2038, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2504] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2038, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2505] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2040, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2506] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2040, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2507] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2040, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2508] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2509] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2510] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2511] Meta(conditional: indented_joins=True -> indent)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2042, child_count: 1, min_times: 0, first_terminator_idx: 24, terminator_count: 2, _padding: 0 }, // [2512] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2043, child_count: 1, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2513] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2044, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2514] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2046, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2515] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2046, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2516] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2046, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2517] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2518] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2519] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2520] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2050, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2521] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2050, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2522] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2050, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2523] Meta(conditional: indented_joins=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2050, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2524] StringParser("FROM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2050, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2525] StringParser("FRONTEND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2050, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2526] StringParser("FRONTENDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2050, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2527] StringParser("FULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2050, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2528] StringParser("FULLTEXT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2050, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2529] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2050, child_count: 11, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2530] AnyNumberOf(11 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2061, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2531] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2061, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2532] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2533] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2534] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2535] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 4, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2536] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2068, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2537] Ref(TrimParametersGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 2068, child_count: 1, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2538] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2069, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2539] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2069, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2540] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2069, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2541] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2069, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2542] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2072, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2543] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2074, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2544] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2074, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2545] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2074, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2546] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2074, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2547] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2074, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2548] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2076, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2549] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2076, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2550] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2551] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2552] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2080, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2553] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2080, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2554] Ref(AggregateOrderByClause)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2080, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2555] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2556] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2557] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2082, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2558] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2085, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2559] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2088, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2560] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2088, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2561] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2088, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2562] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2088, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2563] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2088, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2564] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2091, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2565] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2091, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2566] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2091, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2567] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2091, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2568] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2091, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2569] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2091, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2570] Ref(EmptyStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2091, child_count: 1, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2571] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2092, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2572] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2573] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2574] Ref(TransactionStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2575] StringParser("FUNCTION")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2576] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2095, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2577] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2097, child_count: 1, min_times: 0, first_terminator_idx: 26, terminator_count: 1, _padding: 0 }, // [2578] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2098, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2579] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2100, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2580] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2100, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2581] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2100, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2582] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2100, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 1, _padding: 0 }, // [2583] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2102, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2584] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2102, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2585] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2102, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2586] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2102, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2587] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2104, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2588] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2106, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2589] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2106, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2590] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2108, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2591] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2110, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2592] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2110, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2593] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2110, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2594] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2110, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2595] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2112, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2596] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2114, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2597] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2114, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2598] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2114, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2599] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2114, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2600] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2117, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2601] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2119, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2602] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2119, child_count: 6, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2603] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2125, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2604] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2605] Ref(JsonTableFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2606] Ref(JsonTableFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2607] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2129, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2608] Ref(JsonValueFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2129, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2609] Ref(JsonValueFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2129, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2610] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2131, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2611] Ref(ConvertFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2131, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2612] Ref(ConvertFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2131, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2613] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2133, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2614] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2133, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2615] Ref(DateTimeFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2133, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2616] Ref(ColumnsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2133, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2617] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2135, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2618] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 2137, child_count: 1, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2619] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2138, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2620] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2621] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2622] Ref(ColumnsExpressionFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2623] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2624] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2625] Ref(PostFunctionGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2626] StringParser("FUNCTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2627] StringParser("FUSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2628] StringParser("FUTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2629] StringParser("G")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2630] StringParser("GENERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2631] StringParser("GENERATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2632] StringParser("GENERIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2633] StringParser("GEOMCOLLECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2634] StringParser("GEOMETRY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2635] StringParser("GEOMETRYCOLLECTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2141, child_count: 7, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2636] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2148, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2637] Ref(GetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2148, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2638] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2150, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2639] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2150, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2640] Ref(StackedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2150, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2641] Ref(DiagnosticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2150, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2642] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2152, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2643] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2155, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2644] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2157, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2645] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2157, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2646] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2157, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2647] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2157, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2648] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2159, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2649] Ref(NumberKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2159, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2650] Ref(Row_countKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2159, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2651] Ref(ConditionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2159, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2652] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2653] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2654] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2655] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2162, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2656] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2164, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2657] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2658] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2659] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2660] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2661] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 13, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2662] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2663] Ref(Class_originKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2664] Ref(Subclass_originKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2665] Ref(Returned_sqlstateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2666] Ref(Message_textKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2667] Ref(Mysql_errnoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2668] Ref(Constraint_catalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2669] Ref(Constraint_schemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2670] Ref(Constraint_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2671] Ref(Catalog_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2672] Ref(Schema_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2673] Ref(Table_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2674] Ref(Column_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2675] Ref(Cursor_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2676] StringParser("GET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2677] StringParser("GET_FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2678] StringParser("GET_MASTER_PUBLIC_KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2679] StringParser("GET_SOURCE_PUBLIC_KEY")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2680] TypedParser("glob_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2681] StringParser("GLOBAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2682] StringParser("GO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2683] StringParser("GOTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2684] StringParser("GRANT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2685] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2686] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2687] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2189, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2688] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2689] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2690] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2691] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2692] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2693] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2694] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2695] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2696] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2196, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2697] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2198, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2698] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2201, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2699] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2201, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2700] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2201, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2701] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2201, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2702] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2703] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2704] Ref(AdminKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2705] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2706] StringParser("GRANTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2707] StringParser("GRANTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2708] StringParser("GRAPH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2204, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2709] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2206, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2710] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2206, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2711] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2206, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2712] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2206, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2713] StringParser("GREATEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2206, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2714] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2212, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2715] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2212, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2716] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2212, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2717] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2216, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2718] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2216, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2719] Ref(GroupingSetsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2216, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2720] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2216, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2721] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2217, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 1, _padding: 0 }, // [2722] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2219, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2723] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2724] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2725] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2726] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2727] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2222, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2728] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2729] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2730] Ref(RollupKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2731] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2732] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2232, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2733] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2232, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2734] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2232, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2735] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2232, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2736] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2232, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2737] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2232, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2738] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2232, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2739] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2232, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2740] StringParser("GROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2232, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2741] StringParser("GROUP_REPLICATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2232, child_count: 1, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2742] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 1, _padding: 0 }, // [2743] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2235, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2744] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2745] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2746] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2747] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2239, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2748] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2749] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2750] StringParser("GROUPING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2241, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2751] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2752] Ref(GroupingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2753] Ref(SetsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2754] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2247, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2755] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2249, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2756] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2757] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2758] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2759] StringParser("GROUPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2760] StringParser("GTID_ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2761] StringParser("HANDLER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2762] StringParser("HASH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2763] StringParser("HASH_MAP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2764] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2255, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2765] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2255, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2766] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2257, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2767] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2768] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2260, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2769] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2770] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2771] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2772] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2773] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2774] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2775] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2776] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2777] StringParser("HAVING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2778] StringParser("HDFS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2779] StringParser("HEADER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2780] StringParser("HEAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2781] StringParser("HELP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2267, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2782] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2783] Ref(HelpKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2784] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2785] StringParser("HIERARCHY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2786] StringParser("HIGH_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2787] StringParser("HINT_END")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2788] StringParser("HINT_START")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2789] StringParser("HISTOGRAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2790] StringParser("HISTORY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2791] StringParser("HLL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2792] StringParser("HLL_UNION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2793] StringParser("HOLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2794] StringParser("HOLDLOCK")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2795] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2796] StringParser("HOST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2797] StringParser("HOSTNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2798] StringParser("HOSTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2799] StringParser("HOTSPOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2800] StringParser("HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2801] StringParser("HOUR_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2802] StringParser("HOUR_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2803] StringParser("HOUR_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2804] StringParser("HOURS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2805] StringParser("HUB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2806] StringParser("IDENTIFIED")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2807] Token("identifier")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2808] StringParser("IDENTITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2809] StringParser("IDENTITY_INSERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2810] StringParser("IDENTITYCOL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2811] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2271, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2812] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2271, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2813] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2271, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2814] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2275, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2815] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2816] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2817] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2818] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2819] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2820] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2821] Ref(ElseifKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2822] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2823] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2824] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2283, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2825] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2285, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2826] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2285, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2827] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2285, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2828] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2829] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2830] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2831] StringParser("IF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2287, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2832] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2290, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2833] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2290, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2834] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2290, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2835] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2290, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2836] StringParser("IGNORE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2290, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2837] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2292, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2838] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2839] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2840] Ref(RespectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2841] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2842] StringParser("IGNORE_SERVER_IDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2843] StringParser("ILIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2844] StringParser("IMMEDIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2845] StringParser("IMMUTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2846] StringParser("IMPLEMENTATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2847] StringParser("IMPLICIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2848] StringParser("IMPORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2849] StringParser("IMPORTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2850] StringParser("IN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2294, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2851] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2297, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2852] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2297, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2853] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2297, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2854] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2299, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2855] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2302, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2856] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2304, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2857] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2858] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2859] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2860] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2861] StringParser("INACTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2862] StringParser("INCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2863] StringParser("INCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2864] StringParser("INCREMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2865] StringParser("INCREMENTAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2306, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2866] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2867] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2308, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2868] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2869] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2870] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2310, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2871] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2313, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2872] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2313, child_count: 7, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2873] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2320, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2874] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2320, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2875] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2320, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2876] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2320, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2877] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2323, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2878] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2325, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2879] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2325, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2880] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2327, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2881] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2327, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2882] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2330, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2883] Ref(InvertedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2330, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2884] Ref(BitmapKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2330, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2885] Ref(Bloom_filterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2330, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2886] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2887] Ref(PropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2888] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2335, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2889] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2337, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2890] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2340, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2891] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2340, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2892] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2340, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2893] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2340, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2894] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2342, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2895] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2342, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2896] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2342, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2897] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2898] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2350, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2899] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2350, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2900] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2350, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2901] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2350, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2902] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2352, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2903] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2352, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2904] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2352, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2905] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2354, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2906] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2354, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2907] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2908] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2909] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2359, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2910] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2359, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2911] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2359, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2912] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2361, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2913] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2361, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2914] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2361, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2915] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2364, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2916] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2364, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2917] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2364, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2918] StringParser("INDEX")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2364, child_count: 7, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2919] AnySetOf(7 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2371, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2920] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2374, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2921] Ref(Key_block_sizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2374, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2922] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2374, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2923] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2374, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2924] Ref(IndexTypeGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2374, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2925] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2377, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2926] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2377, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2927] Ref(ParserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2377, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2928] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2377, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2929] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2377, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2930] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2379, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2931] Ref(VisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2379, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2932] Ref(InvisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2379, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2933] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2934] Ref(Engine_attributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2935] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2936] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2937] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2385, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2938] Ref(Secondary_engine_attributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2385, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2939] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2385, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2940] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2385, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2941] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2387, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2942] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2387, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2943] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2944] Ref(BtreeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2945] Ref(HashKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2946] StringParser("INDEXES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2947] StringParser("INDICATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2948] StringParser("INFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2949] StringParser("INFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2950] StringParser("INHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2951] StringParser("INHERITS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2952] StringParser("INITIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2953] StringParser("INITIAL_SIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2954] StringParser("INITIALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2955] StringParser("INITIALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2956] StringParser("INITIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2957] StringParser("->>")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2958] StringParser("INNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2959] StringParser("INOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2960] StringParser("INPLACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2961] StringParser("INPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2962] StringParser("INOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2963] StringParser("IN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2964] StringParser("INSENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2965] StringParser("INSERT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2389, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2966] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2967] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2968] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2392, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2969] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2970] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2395, child_count: 7, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2971] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2402, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2972] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2402, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2973] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2402, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2974] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2402, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2975] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2404, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2976] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2404, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2977] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2407, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2978] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2409, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2979] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2409, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2980] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2412, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2981] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2412, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2982] Ref(LabelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2412, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2983] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2412, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2984] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2412, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2985] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2986] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2987] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2988] StringParser("INSERT_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2989] StringParser("INSERT_METHOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2990] StringParser("INSTALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2991] StringParser("INSTANCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2992] StringParser("INSTANT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2993] StringParser("INSTANTIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2994] StringParser("INSTEAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2995] StringParser("INT1")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2996] StringParser("INT2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2997] StringParser("INT3")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2998] StringParser("INT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2999] StringParser("INT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3000] StringParser("INT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3001] StringParser("INTEGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3002] StringParser("INTEGRATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3003] StringParser("INTEGRATIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3004] StringParser("INTERMEDIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3005] StringParser("INTERSECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3006] StringParser("INTERSECTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2414, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3007] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2416, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3008] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2416, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3009] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3010] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2418, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3011] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2420, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3012] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2420, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3013] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2420, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3014] StringParser("INTERVAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 2420, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 1, _padding: 0 }, // [3015] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3016] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2422, child_count: 1, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3017] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2423, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3018] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2425, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3019] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2428, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3020] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2430, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3021] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2430, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3022] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2430, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3023] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2432, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3024] Ref(DumpfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2432, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3025] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2432, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3026] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2437, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3027] Ref(OutfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2437, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3028] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2437, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3029] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2440, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3030] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2440, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3031] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2440, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3032] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2440, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3033] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2444, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3034] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2446, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3035] Ref(FieldsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2446, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3036] Ref(ColumnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2446, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3037] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2449, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3038] Ref(TerminatedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2449, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3039] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2449, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3040] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2449, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3041] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2453, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3042] Ref(OptionallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2453, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3043] Ref(EnclosedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2453, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3044] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2453, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3045] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2453, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3046] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2456, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3047] Ref(EscapedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2456, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3048] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2456, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3049] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2456, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3050] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2459, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3051] Ref(LinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2459, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3052] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2462, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3053] Ref(StartingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2462, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3054] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2462, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3055] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2462, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3056] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2465, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3057] Ref(TerminatedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2465, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3058] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2465, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3059] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2465, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3060] Ref(SelectClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2465, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3061] StringParser("INTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2465, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3062] StringParser("INVERTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2465, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3063] StringParser("INVISIBLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2465, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3064] StringParser("INVOKER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2465, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3065] StringParser("IO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2465, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3066] StringParser("IO_THREAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2465, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3067] StringParser("IP_TRIE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2465, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3068] StringParser("IPC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2465, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3069] StringParser("IPV4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2465, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3070] StringParser("IPV6")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2465, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3071] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3072] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3073] Ref(NanLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3074] Ref(UnknownLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3075] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3076] Ref(NormalizedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3077] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2474, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3078] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2474, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3079] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2474, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3080] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2474, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3081] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2474, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3082] StringParser("IS")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2474, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3083] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2474, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3084] StringParser("IS_NOT_NULL_PRED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2474, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3085] StringParser("IS_NULL_PRED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2474, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3086] StringParser("ISAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2474, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3087] StringParser("ISNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2474, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3088] StringParser("ISOLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2474, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3089] StringParser("ISSUER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2474, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3090] StringParser("ITERATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2474, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3091] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3092] Ref(IterateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3093] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3094] StringParser("JOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3095] StringParser("JOBS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3096] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2479, child_count: 7, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3097] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2486, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3098] Ref(ConditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2486, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3099] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2486, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3100] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2486, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3101] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2487, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3102] Ref(NestedJoinGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2487, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3103] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2491, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3104] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2491, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3105] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2491, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3106] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2493, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3107] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2493, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3108] Ref(JoinUsingConditionGrammar)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2493, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3109] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2493, child_count: 6, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3110] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3111] Ref(UnconditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3112] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3113] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2499, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3114] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3115] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2503, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3116] Ref(ExtendedNaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2503, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3117] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2503, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3118] StringParser("JOIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2503, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3119] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3120] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3121] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3122] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2508, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3123] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2508, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3124] Meta(conditional: indented_on_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2508, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3125] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2510, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3126] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3127] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3128] Meta(conditional: indented_on_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2513, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3129] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3130] Ref(InnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2515, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3131] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2517, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3132] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2520, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3133] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2520, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3134] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2520, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3135] Ref(RightKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2520, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3136] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2520, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3137] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3138] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3139] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2527, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3140] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2529, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3141] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2529, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3142] StringParser("JSON")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2529, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3143] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3144] Ref(ColumnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3145] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2534, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3146] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2536, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3147] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3148] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3149] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3150] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3151] Ref(OrdinalityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2542, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3152] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3153] Ref(NestedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3154] Ref(PathKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3155] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3156] Ref(JsonTableColumnsClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2546, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3157] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2549, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3158] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2549, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3159] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2549, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3160] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2551, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3161] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2554, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3162] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2554, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3163] Ref(PathKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2554, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3164] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2554, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3165] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2557, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3166] Ref(PathKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2557, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3167] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2557, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3168] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2558, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3169] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2561, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3170] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2564, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3171] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2564, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3172] Ref(ErrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2564, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3173] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2566, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3174] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2566, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3175] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2566, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3176] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2566, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3177] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2568, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3178] Ref(EmptyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2568, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3179] Ref(ErrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2568, child_count: 6, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3180] Bracketed(4 elements -> 4 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2574, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3181] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2574, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3182] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2574, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3183] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2574, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3184] Ref(JsonTableColumnsClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2574, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3185] StringParser("JSON_TABLE")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2574, child_count: 7, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3186] Bracketed(5 elements -> 5 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2581, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3187] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2581, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3188] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2581, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3189] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2581, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3190] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2584, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3191] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2584, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3192] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2584, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3193] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2587, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3194] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2587, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3195] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2587, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3196] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2587, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3197] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2588, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3198] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2591, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3199] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2594, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3200] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2594, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3201] Ref(ErrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2594, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3202] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2596, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3203] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2596, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3204] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2596, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3205] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2596, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3206] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3207] Ref(EmptyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3208] Ref(ErrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3209] StringParser("JSON_VALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3210] StringParser("JSON_TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3211] StringParser("JSON_VALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3212] StringParser("JSONB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3213] StringParser("K")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3214] StringParser("KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3215] StringParser("KEY_BLOCK_SIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3216] StringParser("KEY_MEMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3217] StringParser("KEY_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3218] StringParser("KEYRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3219] StringParser("KEYS")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3220] Token("keyword")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3221] StringParser("KILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3222] StringParser("LABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3223] StringParser("LAG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3224] StringParser("LANCOMPILER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3225] StringParser("LANGUAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3226] StringParser("LARGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3227] StringParser("LARGEINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3228] StringParser("LAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3229] StringParser("LAST_INSERT_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3230] StringParser("LAST_VALUE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3231] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3232] StringParser("LATERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3233] StringParser("LAYOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3234] StringParser("LDAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3235] StringParser("LDAP_ADMIN_PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3236] StringParser("LEAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3237] StringParser("LEADING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3238] StringParser("LEAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3239] StringParser("LEAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3240] StringParser("LEAVES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3241] StringParser("LEFT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3242] StringParser("LEFT_BRACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3243] StringParser("LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3244] StringParser("LESS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3245] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3246] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3247] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3248] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3249] StringParser("LEVEL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2600, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3250] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2603, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3251] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2605, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3252] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2605, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3253] Ref(LikeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2605, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3254] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2605, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3255] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2607, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3256] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2607, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3257] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2607, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3258] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2610, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3259] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2610, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3260] Ref(RlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2610, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3261] Ref(RegexpKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2610, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3262] StringParser("LIKE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2610, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3263] TypedParser("like_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2610, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3264] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2615, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3265] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2615, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3266] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2617, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3267] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3268] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3269] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3270] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3271] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2623, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3272] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2625, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3273] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2627, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3274] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2627, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3275] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2629, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3276] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2629, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3277] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2629, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3278] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3279] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3280] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3281] StringParser("LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3282] StringParser("LINENO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3283] StringParser("LINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3284] StringParser("LINESTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3285] StringParser("LINK")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3286] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3287] StringParser("LIST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2631, child_count: 6, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3288] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2637, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3289] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2637, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3290] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2637, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3291] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2637, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3292] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2637, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3293] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2637, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3294] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2640, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3295] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2642, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3296] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2644, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3297] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2647, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3298] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2649, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3299] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2649, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3300] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2649, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3301] StringParser("LISTEN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2649, child_count: 11, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3302] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3303] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3304] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3305] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3306] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3307] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3308] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3309] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3310] Ref(TypedArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3311] Ref(ObjectLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3312] Ref(DoubleQuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3313] Ref(SystemVariableSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3314] Token("literal")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3315] Token("literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3316] StringParser("LN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 17, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3317] Sequence(17 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2677, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3318] Ref(LoadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2677, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3319] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2677, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3320] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2679, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3321] Ref(Low_priorityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2679, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3322] Ref(ConcurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2679, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3323] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2680, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3324] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2680, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3325] Ref(InfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2680, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3326] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2680, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3327] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3328] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3329] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3330] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3331] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3332] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3333] Ref(SelectPartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2682, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3334] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2685, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3335] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2685, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3336] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2685, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3337] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2685, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3338] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2689, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3339] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2691, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3340] Ref(FieldsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2691, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3341] Ref(ColumnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2691, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3342] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2694, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3343] Ref(TerminatedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2694, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3344] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2694, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3345] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2694, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3346] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2698, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3347] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2699, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3348] Ref(OptionallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2699, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3349] Ref(EnclosedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2699, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3350] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2699, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3351] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2699, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3352] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2702, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3353] Ref(EscapedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2702, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3354] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2702, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3355] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2702, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3356] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3357] Ref(LinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2705, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3358] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2708, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3359] Ref(StartingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2708, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3360] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2708, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3361] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2708, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3362] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2711, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3363] Ref(TerminatedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2711, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3364] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2711, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3365] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2711, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3366] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2714, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3367] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2714, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3368] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2714, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3369] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3370] Ref(LinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3371] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2716, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3372] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2717, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3373] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2720, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3374] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2722, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3375] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2722, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3376] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3377] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3378] Ref(Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3379] StringParser("LOAD")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3380] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3381] StringParser("LOCAL")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3382] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3383] StringParser("LOCALTIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3384] StringParser("LOCALTIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3385] StringParser("LOCATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3386] StringParser("LOCATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3387] StringParser("LOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3388] StringParser("LOCKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3389] StringParser("LOCKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3390] StringParser("LOGFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3391] StringParser("LOGICAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3392] StringParser("LOGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3393] StringParser("LOGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3394] StringParser("LONG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3395] StringParser("LONGBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3396] StringParser("LONGTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3397] StringParser("LOOP")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2724, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3398] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2726, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3399] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2729, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3400] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2731, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3401] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2731, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3402] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2731, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3403] Ref(LoopKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2731, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3404] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2733, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3405] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2733, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3406] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2736, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3407] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2736, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3408] Ref(LoopKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2736, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3409] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2736, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3410] StringParser("LOW_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2736, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3411] StringParser("LOWER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2736, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3412] StringParser("M")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2736, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3413] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2740, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3414] Ref(MlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2740, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3415] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2740, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3416] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2740, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3417] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2744, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3418] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2746, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3419] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2746, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3420] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2746, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3421] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2748, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3422] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2748, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3423] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3424] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3425] StringParser("MANAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3426] StringParser("MANUAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3427] StringParser("MAP")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3428] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3429] StringParser("MASKING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3430] StringParser("MASTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3431] StringParser("MASTER_AUTO_POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3432] StringParser("MASTER_COMPRESSION_ALGORITHMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3433] StringParser("MASTER_CONNECT_RETRY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3434] StringParser("MASTER_DELAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3435] StringParser("MASTER_HEARTBEAT_PERIOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3436] StringParser("MASTER_HOST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3437] StringParser("MASTER_LOG_FILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3438] StringParser("MASTER_LOG_POS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3439] StringParser("MASTER_PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3440] StringParser("MASTER_PORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3441] StringParser("MASTER_PUBLIC_KEY_PATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3442] StringParser("MASTER_RETRY_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3443] StringParser("MASTER_SERVER_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3444] StringParser("MASTER_SSL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3445] StringParser("MASTER_SSL_CA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3446] StringParser("MASTER_SSL_CAPATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3447] StringParser("MASTER_SSL_CERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3448] StringParser("MASTER_SSL_CIPHER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3449] StringParser("MASTER_SSL_CRL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3450] StringParser("MASTER_SSL_CRLPATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3451] StringParser("MASTER_SSL_KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3452] StringParser("MASTER_TLS_CIPHERSUITES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3453] StringParser("MASTER_TLS_VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3454] StringParser("MASTER_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3455] StringParser("MASTER_ZSTD_COMPRESSION_LEVEL")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3456] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3457] StringParser("MATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3458] StringParser("MATCH_ALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3459] StringParser("MATCH_ANY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3460] StringParser("MATCH_PHRASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3461] StringParser("MATCH_PHRASE_EDGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3462] StringParser("MATCH_PHRASE_PREFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3463] StringParser("MATCH_REGEXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3464] StringParser("MATCHED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3465] StringParser("MATERIALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3466] StringParser("MAX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3467] StringParser("MAX_CONNECTIONS_PER_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3468] StringParser("MAX_QUERIES_PER_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3469] StringParser("MAX_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3470] StringParser("MAX_SIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3471] StringParser("MAX_UPDATES_PER_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3472] StringParser("MAX_USER_CONNECTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3473] StringParser("MAXEXTENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3474] StringParser("MAXVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3475] StringParser("MEDIUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3476] StringParser("MEDIUMBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3477] StringParser("MEDIUMINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3478] StringParser("MEDIUMTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3479] StringParser("MEMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3480] StringParser("MEMO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3481] StringParser("MEMORY")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3482] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2751, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3483] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2756, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3484] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2756, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3485] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2756, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3486] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2756, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3487] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3488] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3489] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3490] StringParser("MERGE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2758, child_count: 2, min_times: 1, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3491] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3492] Ref(MergeMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3493] Ref(MergeNotMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2760, child_count: 7, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3494] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2767, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3495] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2767, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3496] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2767, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3497] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2769, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3498] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2769, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3499] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2769, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3500] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2769, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3501] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2771, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3502] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2771, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3503] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2771, child_count: 8, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3504] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2779, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3505] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2779, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3506] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2779, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3507] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2779, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3508] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2781, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3509] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2781, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3510] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2781, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3511] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2781, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3512] Ref(MergeInsertClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2781, child_count: 12, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3513] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2793, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3514] Ref(MergeIntoLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2793, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3515] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3516] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3517] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3518] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2795, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3519] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2799, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3520] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2799, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3521] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2799, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3522] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2801, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3523] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2804, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3524] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2804, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3525] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2804, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3526] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2806, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3527] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2806, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3528] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2806, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3529] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2806, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3530] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2806, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3531] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2806, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3532] Ref(MergeMatchSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2806, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3533] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3534] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3535] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3536] StringParser("MESSAGE_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3537] StringParser("MESSAGE_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3538] StringParser("MESSAGE_TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3539] StringParser("METHOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3540] StringParser("MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3541] StringParser("MIDDLEINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3542] StringParser("MIGRATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3543] StringParser("MIGRATIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3544] StringParser("MILLISECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3545] StringParser("MIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3546] StringParser("MIN_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3547] StringParser("MINUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3548] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3549] StringParser("MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3550] StringParser("MINUTE_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3551] StringParser("MINUTE_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3552] StringParser("MINUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3553] StringParser("MINVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3554] StringParser("ML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3555] StringParser("MLSLABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3556] StringParser("MOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3557] StringParser("MOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3558] StringParser("MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3559] StringParser("MODEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3560] StringParser("MODIFIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3561] StringParser("MODIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3562] StringParser("MODULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3563] StringParser("%")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3564] StringParser("MONITOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3565] StringParser("MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3566] StringParser("MONTHNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3567] StringParser("MORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3568] StringParser("MOVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3569] StringParser("MTMV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3570] StringParser("MULTILINESTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3571] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3572] StringParser("MULTIPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3573] StringParser("MULTIPOLYGON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3574] StringParser("MULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3575] StringParser("MUMPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3576] StringParser("MUTEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3577] StringParser("MYISAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3578] StringParser("MYSQL_ERRNO")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3579] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3580] StringParser("NAME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2810, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3581] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2813, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3582] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2813, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3583] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2813, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3584] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2815, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3585] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2815, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3586] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2818, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3587] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2818, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3588] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2822, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3589] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2822, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3590] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2824, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3591] Ref(NamedWindowExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2824, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3592] StringParser("NAMES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2824, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3593] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2824, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3594] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2824, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3595] StringParser("NATIONAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2824, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3596] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3597] Ref(NaturalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3598] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3599] StringParser("NATURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3600] StringParser("NCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3601] StringParser("NCLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3602] StringParser("NDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3603] StringParser("NDBCLUSTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3604] StringParser("NEGATIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3605] StringParser("-")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3606] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3607] StringParser("NESTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3608] StringParser("NESTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3609] StringParser("NETWORK_NAMESPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3610] StringParser("NEVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3611] StringParser("NEW")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3612] Token("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3613] StringParser("NEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3614] StringParser("NGRAM_BF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3615] StringParser("NO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3616] StringParser("NO_USE_MV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3617] StringParser("NO_WAIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3618] StringParser("NO_WRITE_TO_BINLOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3619] StringParser("NOAUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3620] StringParser("NOCACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3621] StringParser("NOCHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3622] StringParser("NOCOMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3623] StringParser("NOCOPY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3624] StringParser("NOCREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3625] StringParser("NOCREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3626] StringParser("NOCREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3627] StringParser("NOCYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3628] StringParser("NODEGROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3629] StringParser("NOINHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3630] StringParser("NOLOGIN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2826, child_count: 6, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3631] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2832, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3632] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2832, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3633] Ref(UnorderedSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2832, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3634] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3635] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2835, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3636] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2838, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3637] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2838, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3638] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2841, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3639] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2841, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3640] Ref(BracketedSetExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2841, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3641] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2841, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3642] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2845, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3643] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2845, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3644] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2845, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3645] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2845, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3646] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2845, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3647] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2848, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3648] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2848, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3649] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2850, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3650] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2853, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3651] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2853, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3652] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2853, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3653] StringParser("NON_NULLABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2853, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3654] StringParser("NONCLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2853, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3655] StringParser("NONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2853, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3656] StringParser("NOORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2853, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3657] StringParser("NORMALIZE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2853, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3658] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2853, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3659] StringParser("NORMALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2853, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3660] StringParser("NOSUPERUSER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2853, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3661] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2853, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3662] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2855, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3663] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2857, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3664] Ref(RawNotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2857, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3665] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2857, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3666] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2859, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3667] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2859, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3668] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2859, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3669] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2859, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3670] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2859, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3671] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2861, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3672] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2861, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3673] StringParser("!")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2861, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3674] StringParser("NOTHING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2861, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3675] StringParser("NOTIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2861, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3676] StringParser("NOTNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2861, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3677] StringParser("NOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2861, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3678] StringParser("NOWAIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2861, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3679] StringParser("NTH_VALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2861, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3680] StringParser("NTILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2861, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3681] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2861, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3682] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2861, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3683] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3684] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3685] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3686] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3687] StringParser("NULLABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3688] StringParser("NULLIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3689] StringParser("NULLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3690] StringParser("NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3691] StringParser("NUMERIC")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3692] TypedParser("numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3693] StringParser("NVARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3694] StringParser("OBJECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2864, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3695] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2867, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3696] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2867, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3697] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2867, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3698] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2867, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3699] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2870, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3700] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2872, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3701] Ref(ObjectLiteralElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2872, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3702] StringParser("{")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2872, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3703] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2874, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3704] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2874, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3705] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2876, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3706] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2876, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3707] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2876, child_count: 12, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3708] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3709] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3710] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3711] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3712] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3713] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3714] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3715] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3716] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3717] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3718] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3719] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3720] StringParser("OBJECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3721] StringParser("OBSERVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3722] StringParser("OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3723] StringParser("OCTETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3724] StringParser("OF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3725] StringParser("OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3726] StringParser("OFFLINE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2888, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3727] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2891, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3728] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2891, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3729] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2893, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3730] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 2893, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3731] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3732] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2894, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3733] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3734] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3735] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3736] StringParser("OFFSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3737] StringParser("OFFSETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3738] StringParser("OIDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3739] StringParser("OJ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3740] StringParser("OLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3741] StringParser("ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3742] StringParser("ONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3743] StringParser("ONLINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3744] StringParser("ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3745] StringParser("OPEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3746] StringParser("OPENDATASOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3747] StringParser("OPENQUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3748] StringParser("OPENROWSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3749] StringParser("OPENXML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3750] StringParser("OPERATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3751] StringParser("OPERATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3752] StringParser("OPERATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3753] StringParser("OPTIMIZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2896, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3754] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2900, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3755] Ref(OptimizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2900, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3756] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2902, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3757] Ref(No_write_to_binlogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2902, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3758] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2902, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3759] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2902, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3760] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2904, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3761] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2904, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3762] StringParser("OPTIMIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2904, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3763] StringParser("OPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2904, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3764] StringParser("OPTIONAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2904, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3765] StringParser("OPTIONALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2904, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3766] StringParser("OPTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2904, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3767] StringParser("OR")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2904, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3768] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2907, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3769] StringParser("OR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2907, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3770] StringParser("||")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2907, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3771] StringParser("XOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2907, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3772] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2909, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3773] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2909, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3774] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2909, child_count: 5, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3775] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2914, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3776] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2914, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3777] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2914, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 2, _padding: 0 }, // [3778] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2916, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3779] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2920, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3780] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2923, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3781] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2923, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3782] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2923, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3783] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2923, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3784] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3785] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3786] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2925, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3787] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2927, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3788] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2927, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3789] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2929, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3790] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2929, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3791] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2929, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3792] Ref(WithFillSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2929, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3793] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2929, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3794] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2929, child_count: 7, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3795] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3796] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3797] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3798] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3799] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3800] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3801] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3802] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3803] StringParser("ORDER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3804] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2938, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3805] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2938, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3806] Ref(NoorderKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2938, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3807] StringParser("ORDERING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2938, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3808] StringParser("ORDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2938, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3809] StringParser("ORGANIZATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2938, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3810] StringParser("OTHERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2938, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3811] StringParser("OUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2938, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3812] StringParser("OUTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2938, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3813] StringParser("OUTFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2938, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3814] StringParser("OUTPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2938, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3815] StringParser("OUT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2938, child_count: 5, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3816] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3817] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3818] Ref(OverKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2943, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3819] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2945, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3820] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2945, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3821] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3822] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3823] StringParser("OVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2948, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3824] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3825] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2950, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3826] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2952, child_count: 1, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3827] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2953, child_count: 5, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3828] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3829] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3830] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3831] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3832] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3833] StringParser("OVERLAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3834] StringParser("OVERLAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3835] StringParser("OVERRIDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3836] StringParser("OVERWRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3837] StringParser("OWNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3838] StringParser("OWNERSHIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3839] StringParser("PACK_KEYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3840] StringParser("PAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3841] StringParser("PAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3842] StringParser("PARAMETER")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3843] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3844] StringParser("?")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3845] StringParser("PARAMETER_MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3846] StringParser("PARAMETER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3847] StringParser("PARAMETER_ORDINAL_POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3848] StringParser("PARAMETER_SPECIFIC_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3849] StringParser("PARAMETER_SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3850] StringParser("PARAMETER_SPECIFIC_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3851] StringParser("PARAMETERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3852] StringParser("PARSE_GCOL_EXPR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3853] StringParser("PARSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3854] StringParser("PARSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2958, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3855] StringParser("PARTIAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2958, child_count: 5, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3856] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2963, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3857] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2963, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3858] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2963, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3859] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2965, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3860] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2968, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3861] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2970, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3862] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2970, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3863] StringParser("PARTITION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2970, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3864] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2972, child_count: 6, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3865] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2978, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3866] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2978, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3867] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2978, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3868] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2978, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3869] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2978, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3870] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2981, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3871] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2981, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3872] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2983, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3873] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2986, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3874] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2986, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3875] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2986, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3876] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2988, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3877] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2991, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3878] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2991, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3879] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2994, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3880] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2996, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3881] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2996, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3882] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2999, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3883] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3001, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3884] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3003, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3885] Ref(RangePartitionDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3003, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3886] Ref(RangePartitionIntervalSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3003, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3887] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3006, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3888] Ref(ListKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3006, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3889] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3009, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3890] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3011, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3891] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3011, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3892] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3014, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3893] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3016, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3894] Ref(ListPartitionDefinitionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3016, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3895] StringParser("PARTITIONING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3016, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3896] StringParser("PARTITIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3016, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3897] StringParser("PASCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3016, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3898] StringParser("PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3016, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3899] StringParser("PASSWORD_EXPIRE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3016, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3900] StringParser("PASSWORD_HISTORY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3016, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3901] StringParser("PASSWORD_LOCK_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3016, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3902] StringParser("PASSWORD_REUSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3016, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3903] StringParser("PATH")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3016, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3904] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3018, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3905] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3020, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3906] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3020, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3907] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3908] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3909] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3910] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3911] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3912] StringParser("PAUSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3913] StringParser("PCTFREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3914] StringParser("PERCENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3915] StringParser("PERCENT_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3916] StringParser("PERCENTILE_CONT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3917] StringParser("PERCENTILE_DISC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3918] StringParser("PERIOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3919] StringParser("PERMISSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3920] StringParser("PERSIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3921] StringParser("PERSIST_ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3922] StringParser("PHASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3923] StringParser("PHYSICAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3924] StringParser("PI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3925] StringParser("PIPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3926] StringParser("|")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3927] StringParser("PLACEHOLDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3928] StringParser("PLACING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3929] StringParser("PLAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3930] StringParser("PLAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3931] StringParser("PLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3932] StringParser("PLUGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3933] StringParser("PLUGIN_DIR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3934] StringParser("PLUGINS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3935] StringParser("+")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3936] StringParser("POINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3937] StringParser("POLICY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3938] StringParser("POLYGON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3939] StringParser("PORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3940] StringParser("POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3941] StringParser("+")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3022, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3942] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3024, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3943] Ref(OverClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3024, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3944] Ref(FilterClauseGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3024, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3945] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3026, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3946] Ref(IndexHintClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3026, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3947] Ref(SelectPartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3026, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3948] StringParser("POSTFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3026, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3949] StringParser("POWER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3026, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3950] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3026, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3951] StringParser("PRECEDES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3026, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3952] StringParser("PRECEDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3026, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3953] StringParser("PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3026, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3954] StringParser("PREFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3026, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3955] StringParser("PREORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3026, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3956] StringParser("PREPARE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3026, child_count: 4, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3957] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3030, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3958] Ref(PrepareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3030, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3959] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3030, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3960] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3030, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3961] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3033, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3962] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3033, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3963] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3033, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3964] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3033, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3965] StringParser("PREPARED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3033, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3966] StringParser("PRESERVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3033, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3967] StringParser("PREV")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3033, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3968] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3035, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3969] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3035, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3970] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3035, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3971] StringParser("PRIMARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3035, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3972] StringParser("PRINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3035, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3973] StringParser("PRIOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3035, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3974] StringParser("PRIVILEGE_CHECKS_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3035, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3975] StringParser("PRIVILEGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3035, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3976] StringParser("PROC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3035, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3977] StringParser("PROCEDURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3035, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3978] StringParser("PROCEDURE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3035, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3979] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3037, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3980] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3040, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3981] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3043, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3982] Ref(OutputParameterSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3043, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3983] Ref(InputParameterSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3043, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3984] Ref(InputOutputParameterSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3043, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3985] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3043, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3986] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3043, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3987] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3043, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3988] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3046, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3989] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3990] Ref(ProcedureParameterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3991] StringParser("PROCEDURES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3992] StringParser("PROCESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3993] StringParser("PROCESSLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3994] StringParser("PROFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3995] StringParser("PROFILES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3996] StringParser("PROPERTIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3997] StringParser("PROPERTY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3998] StringParser("PROXY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3999] StringParser("PUBLIC")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3048, child_count: 4, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4000] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4001] Ref(PurgeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3052, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4002] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3054, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4003] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3054, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4004] Ref(MasterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3054, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4005] Ref(LogsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3054, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4006] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3056, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4007] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3058, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4008] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3058, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4009] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3058, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4010] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3060, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4011] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3060, child_count: 1, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4012] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3061, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4013] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3061, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4014] StringParser("PURGE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3061, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4015] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4016] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4017] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4018] StringParser("QUALIFY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3063, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4019] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4020] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3066, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4021] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3069, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4022] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3069, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4023] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3069, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4024] Ref(SomeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 3069, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4025] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4026] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4027] StringParser("QUANTILE_STATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4028] StringParser("QUANTILE_UNION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4029] StringParser("QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4030] StringParser("QUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4031] StringParser("QUEUED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4032] StringParser("QUICK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4033] StringParser("QUOTA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4034] StringParser("QUOTE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4035] TypedParser("back_quote")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3072, child_count: 2, min_times: 1, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4036] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3074, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4037] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3074, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4038] Ref(DoubleQuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3074, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4039] StringParser("RAID0")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3074, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4040] StringParser("RAISERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3074, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4041] StringParser("RANDOM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3074, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4042] StringParser("RANGE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3074, child_count: 6, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4043] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3080, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4044] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3080, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4045] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3080, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4046] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3080, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4047] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3080, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4048] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3082, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4049] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3085, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4050] Ref(LessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3085, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4051] Ref(ThanKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3085, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4052] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3087, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4053] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3087, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4054] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3090, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4055] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3092, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4056] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3092, child_count: 1, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4057] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3093, child_count: 5, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4058] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3098, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4059] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3101, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4060] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3103, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4061] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3103, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4062] Ref(,KeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3103, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4063] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3106, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4064] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3108, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4065] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3108, child_count: 1, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4066] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3109, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4067] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3112, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4068] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4069] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3117, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4070] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3117, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4071] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3117, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4072] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3117, child_count: 7, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4073] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3124, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4074] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3124, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4075] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3127, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4076] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3127, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4077] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3127, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4078] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3130, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4079] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3130, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4080] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3130, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4081] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3130, child_count: 6, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4082] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4083] Ref(YearKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4084] Ref(MonthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4085] Ref(DayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4086] Ref(HourKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4087] Ref(MinuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4088] Ref(SecondKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4089] StringParser("RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4090] StringParser("=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4091] StringParser(">")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4092] StringParser("RAW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4093] StringParser("<")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4094] StringParser("!")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4095] StringParser("READ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4096] StringParser("READ_ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4097] StringParser("READS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4098] StringParser("READTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4099] StringParser("REAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4100] StringParser("REBALANCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4101] StringParser("REBUILD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4102] StringParser("RECENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4103] StringParser("RECHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4104] StringParser("RECONFIGURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4105] StringParser("RECOVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4106] StringParser("RECURSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4107] StringParser("RECYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4108] StringParser("REDO_BUFFER_SIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4109] StringParser("REDOFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4110] StringParser("REDUNDANT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3136, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4111] StringParser("REF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3136, child_count: 5, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4112] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3141, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4113] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3141, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4114] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3141, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4115] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3141, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4116] Ref(ReferenceMatchGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3141, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4117] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3143, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4118] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3146, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4119] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3146, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4120] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3146, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4121] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3146, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4122] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3149, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4123] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3149, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4124] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3149, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4125] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3149, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4126] StringParser("REFERENCE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3149, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4127] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3151, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4128] Ref(MatchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3151, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4129] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3154, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4130] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3154, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4131] Ref(PartialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3154, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4132] Ref(SimpleKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3154, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4133] StringParser("REFERENCE_USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3154, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4134] StringParser("REFERENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3154, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4135] StringParser("REFERENCING")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3154, child_count: 5, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4136] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3159, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4137] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3159, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4138] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3159, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4139] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3161, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4140] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3161, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4141] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3161, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4142] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3163, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4143] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3163, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4144] Ref(ActionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3163, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4145] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4146] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4147] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4148] StringParser("REFRESH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4149] StringParser("REGEXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4150] StringParser("REGISTRATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4151] StringParser("REGR_AVGX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4152] StringParser("REGR_AVGY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4153] StringParser("REGR_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4154] StringParser("REGR_INTERCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4155] StringParser("REGR_R2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4156] StringParser("REGR_SLOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4157] StringParser("REGR_SXX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4158] StringParser("REGR_SXY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4159] StringParser("REGR_SYY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4160] StringParser("REINDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4161] StringParser("RELATIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4162] StringParser("RELAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4163] StringParser("RELAY_LOG_FILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4164] StringParser("RELAY_LOG_POS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4165] StringParser("RELAY_THREAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4166] StringParser("RELAYLOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4167] StringParser("RELEASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4168] StringParser("RELOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4169] StringParser("REMOTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4170] StringParser("REMOVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3165, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4171] StringParser("RENAME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3165, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4172] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3168, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4173] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3168, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4174] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3168, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4175] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3170, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4176] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3173, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4177] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3173, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4178] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3173, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4179] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3173, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4180] StringParser("REORGANIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3173, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4181] StringParser("REPAIR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3173, child_count: 5, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4182] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3178, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4183] Ref(RepairKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3178, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4184] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3180, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4185] Ref(No_write_to_binlogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3180, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4186] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3180, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4187] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3180, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4188] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3182, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4189] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3182, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4190] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3185, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4191] Ref(QuickKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3185, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4192] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3185, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4193] Ref(Use_frmKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3185, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4194] StringParser("REPEAT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3185, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4195] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3187, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4196] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3190, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4197] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3192, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4198] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3192, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4199] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3192, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4200] Ref(RepeatKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3192, child_count: 1, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4201] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3193, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4202] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3193, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4203] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3196, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4204] Ref(UntilKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3196, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4205] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3196, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4206] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3199, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4207] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3199, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4208] Ref(RepeatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3199, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4209] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3199, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4210] StringParser("REPEATABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3199, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4211] StringParser("REPLACE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3199, child_count: 6, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4212] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3205, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4213] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3205, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4214] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3207, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4215] Ref(Low_priorityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3207, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4216] Ref(DelayedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3207, child_count: 1, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4217] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3208, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4218] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3208, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4219] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3208, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4220] Ref(SelectPartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3208, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4221] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3211, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4222] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3213, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4223] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3213, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4224] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3213, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4225] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3213, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4226] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3215, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4227] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3215, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4228] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3217, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4229] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3217, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4230] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4231] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4232] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4233] StringParser("REPLACE_IF_NOT_NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4234] StringParser("REPLAYER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4235] StringParser("REPLICA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4236] StringParser("REPLICAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4237] StringParser("REPLICATE_DO_DB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4238] StringParser("REPLICATE_DO_TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4239] StringParser("REPLICATE_IGNORE_DB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4240] StringParser("REPLICATE_IGNORE_TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4241] StringParser("REPLICATE_REWRITE_DB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4242] StringParser("REPLICATE_WILD_DO_TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4243] StringParser("REPLICATE_WILD_IGNORE_TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4244] StringParser("REPLICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4245] StringParser("REPOSITORIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4246] StringParser("REPOSITORY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4247] StringParser("REQUIRE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4248] StringParser("REQUIRE_ROW_FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4249] StringParser("RESET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3219, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4250] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3222, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4251] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3222, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4252] Ref(MasterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3222, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4253] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3224, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4254] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3224, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4255] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3224, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4256] StringParser("RESIGNAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3224, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4257] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3227, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4258] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3229, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4259] Ref(SignalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3229, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4260] Ref(ResignalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3229, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4261] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3231, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4262] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3234, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4263] Ref(SqlstateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3234, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4264] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3234, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4265] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3234, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4266] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3234, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4267] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3236, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4268] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3236, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4269] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3238, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4270] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3241, child_count: 13, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4271] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4272] Ref(Class_originKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4273] Ref(Subclass_originKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4274] Ref(Returned_sqlstateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4275] Ref(Message_textKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4276] Ref(Mysql_errnoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4277] Ref(Constraint_catalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4278] Ref(Constraint_schemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4279] Ref(Constraint_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4280] Ref(Catalog_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4281] Ref(Schema_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4282] Ref(Table_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4283] Ref(Column_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4284] Ref(Cursor_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4285] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4286] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4287] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4288] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4289] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4290] StringParser("RESOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4291] StringParser("RESOURCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4292] StringParser("RESPECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4293] StringParser("RESTART")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4294] StringParser("RESTORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4295] StringParser("RESTRICT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4296] StringParser("RESTRICTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4297] StringParser("RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4298] StringParser("RESUME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4299] StringParser("RETAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4300] StringParser("RETENTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4301] StringParser("RETURN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3257, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4302] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4303] Ref(ReturnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4304] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4305] StringParser("RETURNED_CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4306] StringParser("RETURNED_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4307] StringParser("RETURNED_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4308] StringParser("RETURNED_SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4309] StringParser("RETURNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4310] StringParser("RETURNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4311] StringParser("REUSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4312] StringParser("REVERSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4313] StringParser("REVOKE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3259, child_count: 6, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4314] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3265, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4315] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3265, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4316] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3268, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4317] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3268, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4318] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3268, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4319] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3268, child_count: 4, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4320] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3272, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4321] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3275, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4322] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3275, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4323] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3275, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4324] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3275, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4325] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3277, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4326] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3277, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4327] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3277, child_count: 4, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4328] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3281, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4329] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3281, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4330] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3281, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4331] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3281, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4332] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3281, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4333] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3281, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4334] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3281, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4335] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3281, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4336] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3281, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4337] StringParser("REWRITTEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3281, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4338] StringParser("RIGHT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3281, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4339] StringParser("RIGHT_BRACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3281, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4340] StringParser("RLIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3281, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4341] StringParser("ROLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3281, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4342] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3283, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4343] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3285, child_count: 4, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4344] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3289, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4345] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3289, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4346] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3289, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4347] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3289, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4348] Ref(DoubleQuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 3289, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4349] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3291, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4350] Ref(AtSignLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3291, child_count: 4, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4351] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3295, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4352] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3295, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4353] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3295, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4354] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3295, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4355] Ref(DoubleQuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3295, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4356] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3295, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4357] StringParser("ROLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3295, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4358] StringParser("ROLLBACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3295, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4359] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3295, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4360] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3295, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4361] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3297, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4362] Ref(RollupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3297, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4363] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3300, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4364] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3302, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4365] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3305, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4366] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3305, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4367] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3308, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4368] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3310, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4369] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3310, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4370] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3313, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4371] Ref(DuplicateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3313, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4372] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3313, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4373] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3316, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4374] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4375] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4376] StringParser("ROTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4377] StringParser("ROUTINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4378] StringParser("ROUTINE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4379] StringParser("ROUTINE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4380] StringParser("ROUTINE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4381] StringParser("ROUTINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4382] StringParser("ROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4383] StringParser("ROW_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4384] StringParser("ROW_FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4385] StringParser("ROW_NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4386] StringParser("ROWCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4387] StringParser("ROWGUIDCOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4388] StringParser("ROWID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4389] StringParser("ROWNUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4390] StringParser("ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4391] StringParser("RTREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4392] StringParser("RULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4393] StringParser("S3")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4394] StringParser("SAMPLE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3318, child_count: 4, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4395] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3322, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4396] Ref(TablesampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3322, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4397] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3324, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4398] Ref(BernoulliKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3324, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4399] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3324, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4400] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3327, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4401] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3327, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4402] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3329, child_count: 1, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4403] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3330, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4404] Ref(RepeatableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3330, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4405] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4406] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4407] StringParser("SAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4408] StringParser("SAVEPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4409] StringParser("SCALE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4410] StringParser("SCHEDULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4411] StringParser("SCHEDULER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4412] StringParser("SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4413] StringParser("SCHEMA_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4414] StringParser("SCHEMAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4415] StringParser("SCOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4416] StringParser("SCOPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4417] StringParser("SCOPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4418] StringParser("SCOPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4419] StringParser("SCROLL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4420] StringParser("SEARCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4421] StringParser("SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4422] StringParser("SECOND_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4423] StringParser("SECONDARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4424] StringParser("SECONDARY_ENGINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4425] StringParser("SECONDARY_ENGINE_ATTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4426] StringParser("SECONDARY_LOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4427] StringParser("SECONDARY_UNLOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4428] StringParser("SECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3333, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4429] StringParser("SECURITY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3333, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4430] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3335, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4431] Ref(WildcardExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3335, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4432] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3337, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4433] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3337, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4434] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3337, child_count: 9, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4435] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3346, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4436] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4437] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4438] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4439] Ref(DistinctrowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4440] Ref(High_priorityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4441] Ref(Straight_joinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4442] Ref(Sql_small_resultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4443] Ref(Sql_big_resultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4444] Ref(Sql_buffer_resultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4445] Ref(Sql_cacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4446] Ref(Sql_no_cacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4447] Ref(Sql_calc_found_rowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 3349, child_count: 5, min_times: 0, first_terminator_idx: 33, terminator_count: 2, _padding: 0 }, // [4448] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3354, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4449] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3354, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4450] Ref(SelectClauseModifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 3354, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4451] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3356, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4452] Ref(SelectClauseElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3356, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4453] Ref(SelectClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3356, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4454] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3356, child_count: 7, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4455] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 3363, child_count: 1, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4456] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::PrecededBy, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3364, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4457] PrecededBy(preceding_sequences=(('IS', 'DISTINCT'), ('IS', 'NOT', 'DISTINCT')))
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3364, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4458] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3364, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4459] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3366, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4460] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3366, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4461] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3366, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4462] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3366, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4463] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3366, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4464] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3366, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4465] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3366, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4466] StringParser("SELECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3366, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4467] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3368, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4468] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3368, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4469] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3371, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4470] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3373, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4471] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 3373, child_count: 15, min_times: 0, first_terminator_idx: 35, terminator_count: 3, _padding: 0 }, // [4472] Sequence(15 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3388, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4473] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3388, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4474] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3388, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4475] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3388, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4476] Ref(IndexHintClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3388, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4477] Ref(SelectPartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3388, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4478] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3388, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4479] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3388, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4480] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3388, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4481] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3388, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4482] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3388, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4483] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3388, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4484] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3388, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4485] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3388, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4486] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3388, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4487] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3388, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4488] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3388, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4489] Ref(UpsertClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3388, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4490] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3388, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4491] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3392, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4492] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3394, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4493] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3397, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4494] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3397, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4495] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3399, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4496] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3402, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4497] Ref(WithCompoundNonSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3402, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4498] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3402, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4499] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3405, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4500] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3405, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4501] StringParser("SELF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3405, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4502] StringParser("SEMI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3405, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4503] StringParser(";")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3405, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4504] StringParser("SENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3405, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4505] StringParser("SEPARATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3405, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4506] StringParser("SEQUENCE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3405, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4507] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3407, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4508] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3409, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4509] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3409, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4510] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3409, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4511] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3411, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4512] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3411, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4513] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3411, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4514] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3413, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4515] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3415, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4516] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3415, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4517] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3415, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4518] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3417, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4519] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3417, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4520] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3417, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4521] StringParser("SEQUENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3417, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4522] StringParser("SERIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3417, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4523] StringParser("SERIALIZABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3417, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4524] StringParser("SERVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3417, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4525] StringParser("SERVER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3417, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4526] StringParser("SESSION")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3417, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4527] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3417, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4528] StringParser("SESSION_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3417, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4529] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3419, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4530] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3419, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4531] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3421, child_count: 5, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4532] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3426, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4533] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3428, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4534] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3430, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4535] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3430, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4536] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3430, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4537] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3430, child_count: 5, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4538] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3435, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4539] Ref(GlobalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3435, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4540] Ref(PersistKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3435, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4541] Ref(Persist_onlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3435, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4542] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3435, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4543] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3435, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4544] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3438, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4545] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3438, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4546] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3438, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4547] Ref(SystemVariableSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3438, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4548] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3440, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4549] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3440, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4550] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3440, child_count: 10, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4551] AnyNumberOf(10 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3450, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4552] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3450, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4553] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3450, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4554] Ref(DoubleQuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3450, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4555] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3450, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4556] Ref(SystemVariableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3450, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4557] Ref(BooleanDynamicSystemVariablesGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3450, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4558] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3450, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4559] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3450, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4560] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3450, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4561] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3450, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4562] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3454, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4563] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3454, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4564] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3456, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4565] Ref(SetClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3456, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4566] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3459, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4567] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3459, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4568] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3459, child_count: 7, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4569] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4570] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4571] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4572] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4573] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4574] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4575] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4576] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 6, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4577] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3472, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4578] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3472, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4579] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3472, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4580] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3472, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4581] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3475, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4582] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3475, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4583] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3475, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4584] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4585] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4586] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4587] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3479, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4588] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3479, child_count: 5, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4589] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3484, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4590] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3484, child_count: 1, min_times: 1, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4591] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3485, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4592] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3487, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4593] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3487, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4594] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3487, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4595] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3487, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4596] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3487, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4597] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3487, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4598] StringParser("SET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3487, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4599] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3491, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4600] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3491, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4601] Ref(NamesKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3491, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4602] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3494, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4603] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3494, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4604] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3494, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4605] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3494, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4606] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3496, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4607] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3496, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4608] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3496, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4609] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3499, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4610] Ref(UnionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3499, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4611] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3501, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4612] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3503, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4613] Ref(IntersectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3503, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4614] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3503, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4615] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3503, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4616] Ref(MinusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3503, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4617] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3507, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4618] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3507, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4619] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3507, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4620] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3507, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4621] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3507, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4622] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3511, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4623] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3511, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4624] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4625] Ref(GlobalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4626] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4627] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3513, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4628] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3515, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4629] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3517, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4630] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3520, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4631] Ref(IsolationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3520, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4632] Ref(LevelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3520, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4633] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3523, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4634] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4635] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3525, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4636] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3527, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4637] Ref(CommittedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3527, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4638] Ref(UncommittedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3527, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4639] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3529, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4640] Ref(RepeatableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3529, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4641] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3529, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4642] Ref(SerializableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3529, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4643] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3531, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4644] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3531, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4645] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3533, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4646] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3533, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4647] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3533, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4648] StringParser("SET_SESSION_VARIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3533, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4649] StringParser("SETOF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3533, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4650] StringParser("SETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3533, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4651] StringParser("SETUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3533, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4652] StringParser("SHAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3533, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4653] StringParser("SHARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3533, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4654] StringParser("SHARED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3533, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4655] StringParser("SHARES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3533, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4656] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3535, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4657] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3537, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4658] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3537, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4659] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3537, child_count: 1, min_times: 1, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4660] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3538, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4661] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3541, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4662] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3541, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4663] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3541, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4664] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3541, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4665] StringParser("SHOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3541, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4666] StringParser("SHUTDOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3541, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4667] StringParser("SIGNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3541, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4668] StringParser("SIGNED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3541, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4669] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3543, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4670] Ref(PositiveSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3543, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4671] Ref(NegativeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3543, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4672] StringParser("SIMILAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3543, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4673] StringParser("SIMPLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3543, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 1, _padding: 0 }, // [4674] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3546, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4675] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3546, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4676] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3546, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4677] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3546, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4678] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3546, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4679] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3548, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4680] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3548, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4681] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3551, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4682] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3551, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4683] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3551, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4684] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3551, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4685] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3551, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4686] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3551, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4687] StringParser("SIZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3551, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4688] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4689] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4690] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4691] StringParser("SKEW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4692] StringParser("SKIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4693] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4694] StringParser("SLAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4695] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4696] StringParser("SLOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4697] StringParser("SMALLINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4698] StringParser("SNAPSHOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4699] StringParser("SNAPSHOTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4700] StringParser("SOCKET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4701] StringParser("SOME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4702] StringParser("SONAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4703] StringParser("SOUNDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4704] StringParser("SOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4705] StringParser("SOURCE_AUTO_POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4706] StringParser("SOURCE_BIND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4707] StringParser("SOURCE_COMPRESSION_ALGORITHMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4708] StringParser("SOURCE_CONNECT_RETRY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4709] StringParser("SOURCE_DELAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4710] StringParser("SOURCE_HEARTBEAT_PERIOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4711] StringParser("SOURCE_HOST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4712] StringParser("SOURCE_LOG_FILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4713] StringParser("SOURCE_LOG_POS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4714] StringParser("SOURCE_PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4715] StringParser("SOURCE_PORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4716] StringParser("SOURCE_PUBLIC_KEY_PATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4717] StringParser("SOURCE_RETRY_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4718] StringParser("SOURCE_SSL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4719] StringParser("SOURCE_SSL_CA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4720] StringParser("SOURCE_SSL_CAPATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4721] StringParser("SOURCE_SSL_CERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4722] StringParser("SOURCE_SSL_CIPHER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4723] StringParser("SOURCE_SSL_CRL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4724] StringParser("SOURCE_SSL_CRLPATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4725] StringParser("SOURCE_SSL_KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4726] StringParser("SOURCE_SSL_VERIFY_SERVER_CERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4727] StringParser("SOURCE_TLS_CIPHERSUITES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4728] StringParser("SOURCE_TLS_VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4729] StringParser("SOURCE_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4730] StringParser("SOURCE_ZSTD_COMPRESSION_LEVEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4731] StringParser("SPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4732] StringParser("SPATIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4733] StringParser("SPECIFIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4734] StringParser("SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4735] StringParser("SPECIFICTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4736] StringParser("SPLIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4737] StringParser("SQL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4738] StringParser("SQL_AFTER_GTIDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4739] StringParser("SQL_AFTER_MTS_GAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4740] StringParser("SQL_BEFORE_GTIDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4741] StringParser("SQL_BIG_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4742] StringParser("SQL_BIG_SELECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4743] StringParser("SQL_BIG_TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4744] StringParser("SQL_BLOCK_RULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4745] StringParser("SQL_BUFFER_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4746] StringParser("SQL_CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4747] StringParser("SQL_CALC_FOUND_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4748] StringParser("SQL_LOG_OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4749] StringParser("SQL_LOG_UPDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4750] StringParser("SQL_LOW_PRIORITY_UPDATES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4751] StringParser("SQL_NO_CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4752] StringParser("SQL_SELECT_LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4753] StringParser("SQL_SMALL_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4754] StringParser("SQL_THREAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4755] StringParser("SQL_TSI_DAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4756] StringParser("SQL_TSI_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4757] StringParser("SQL_TSI_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4758] StringParser("SQL_TSI_MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4759] StringParser("SQL_TSI_QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4760] StringParser("SQL_TSI_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4761] StringParser("SQL_TSI_WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4762] StringParser("SQL_TSI_YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4763] StringParser("SQL_WARNINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4764] StringParser("SQLCA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4765] StringParser("SQLCODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4766] StringParser("SQLERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4767] StringParser("SQLEXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4768] StringParser("SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4769] StringParser("SQLWARNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4770] StringParser("SQRT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4771] StringParser("SRID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4772] StringParser("SSL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4773] StringParser("STABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4774] StringParser("STACKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4775] StringParser("STAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4776] StringParser("STAGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4777] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4778] StringParser("START")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4779] StringParser("STARTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4780] StringParser("STARTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4781] StringParser("STATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4782] StringParser("STATEMENT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3553, child_count: 81, min_times: 0, first_terminator_idx: 39, terminator_count: 1, _padding: 0 }, // [4783] OneOf(81 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4784] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4785] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4786] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4787] Ref(TransactionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4788] Ref(DropTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4789] Ref(DropViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4790] Ref(CreateUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4791] Ref(DropUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4792] Ref(TruncateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4793] Ref(AccessStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4794] Ref(CreateTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4795] Ref(CreateRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4796] Ref(DropRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4797] Ref(AlterTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4798] Ref(SetSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4799] Ref(DropSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4800] Ref(DropTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4801] Ref(CreateDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4802] Ref(DropDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4803] Ref(CreateIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4804] Ref(DropIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4805] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4806] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4807] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4808] Ref(CreateCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4809] Ref(DropCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4810] Ref(CreateFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4811] Ref(DropFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4812] Ref(CreateModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4813] Ref(DropModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4814] Ref(DescribeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4815] Ref(UseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4816] Ref(ExplainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4817] Ref(CreateSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4818] Ref(AlterSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4819] Ref(DropSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4820] Ref(CreateTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4821] Ref(DropTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4822] Ref(DelimiterStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4823] Ref(CreateProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4824] Ref(DeclareStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4825] Ref(SetTransactionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4826] Ref(SetAssignmentStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4827] Ref(IfExpressionStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4828] Ref(WhileStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4829] Ref(IterateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4830] Ref(RepeatStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4831] Ref(LoopStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4832] Ref(CallStoredProcedureSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4833] Ref(PrepareSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4834] Ref(ExecuteSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4835] Ref(DeallocateSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4836] Ref(GetDiagnosticsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4837] Ref(ResignalSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4838] Ref(CursorOpenCloseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4839] Ref(CursorFetchSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4840] Ref(DropProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4841] Ref(AlterTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4842] Ref(AlterViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4843] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4844] Ref(RenameTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4845] Ref(ResetMasterStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4846] Ref(PurgeBinaryLogsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4847] Ref(HelpStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4848] Ref(CheckTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4849] Ref(ChecksumTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4850] Ref(AnalyzeTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4851] Ref(RepairTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4852] Ref(OptimizeTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4853] Ref(UpsertClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4854] Ref(InsertRowAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4855] Ref(FlushStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4856] Ref(LoadDataSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4857] Ref(ReplaceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4858] Ref(AlterDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4859] Ref(ReturnStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4860] Ref(SetNamesStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4861] Ref(CreateEventStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4862] Ref(AlterEventStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4863] Ref(DropEventStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4864] Ref(SetDefaultRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4865] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4866] StringParser("STATIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4867] StringParser("STATISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4868] StringParser("STATS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4869] StringParser("STATS_AUTO_RECALC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4870] StringParser("STATS_PERSISTENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4871] StringParser("STATS_SAMPLE_PAGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4872] StringParser("STATUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4873] StringParser("STDDEV_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4874] StringParser("STDDEV_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4875] StringParser("STDIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4876] StringParser("STDOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4877] StringParser("STOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4878] StringParser("STORAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4879] StringParser("STORED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4880] StringParser("STRAIGHT_JOIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4881] StringParser("STREAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4882] StringParser("STREAMING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4883] StringParser("STREAMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4884] StringParser("STRICT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 1, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4885] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3635, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4886] Ref(ConcatSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3635, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4887] StringParser("STRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3635, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4888] StringParser("STRUCT")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3635, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4889] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3638, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4890] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3640, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4891] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4892] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4893] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4894] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4895] StringParser("STRUCTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4896] StringParser("STYLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4897] StringParser("SUBCLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4898] StringParser("SUBJECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4899] StringParser("SUBLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4900] StringParser("SUBMULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4901] StringParser("SUBPARTITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4902] StringParser("SUBPARTITIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4903] StringParser("SUBSTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4904] StringParser("SUCCESSFUL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4905] StringParser("SUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4906] StringParser("SUPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4907] StringParser("SUPERUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4908] StringParser("SUSPEND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4909] StringParser("SWAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4910] StringParser("SWITCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4911] StringParser("SWITCHES")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4912] Token("symbol")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4913] StringParser("SYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4914] StringParser("SYNC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4915] StringParser("SYNONYM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4916] StringParser("SYSDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4917] StringParser("SYSID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4918] StringParser("SYSTEM")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4919] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4920] StringParser("SYSTEM_USER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3642, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4921] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3645, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4922] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3647, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4923] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3649, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4924] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3649, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4925] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3649, child_count: 4, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4926] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3653, child_count: 6, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4927] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3659, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4928] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3659, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4929] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3661, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4930] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3661, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4931] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3661, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4932] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3661, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4933] Ref(IndexTypeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3661, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4934] Ref(BracketedKeyPartListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3661, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4935] Ref(IndexOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3661, child_count: 4, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4936] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3665, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4937] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3665, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4938] Ref(IndexTypeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3665, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4939] Ref(BracketedKeyPartListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3665, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4940] Ref(IndexOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3665, child_count: 7, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4941] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3672, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4942] Ref(ForeignKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3672, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4943] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3672, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4944] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3672, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4945] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3672, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4946] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3672, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4947] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3672, child_count: 1, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4948] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3673, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4949] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3676, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4950] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3676, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4951] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3678, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4952] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3678, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4953] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3678, child_count: 5, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4954] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3683, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4955] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3683, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4956] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3683, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4957] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3685, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4958] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3685, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4959] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3685, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4960] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3687, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4961] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3687, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4962] Ref(ActionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3687, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4963] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3689, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4964] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3689, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4965] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3689, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4966] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3692, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4967] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3692, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4968] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3695, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4969] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3695, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4970] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3697, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4971] Ref(EnforcedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3697, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4972] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3699, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4973] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3699, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4974] Ref(EnforcedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3699, child_count: 5, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4975] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3704, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4976] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3706, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4977] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3706, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4978] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3706, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4979] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3706, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4980] Ref(IndexTypeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3706, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4981] Ref(BracketedKeyPartListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3706, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4982] Ref(IndexOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3706, child_count: 5, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4983] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3711, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4984] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3713, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4985] Ref(FulltextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3713, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4986] Ref(SpatialKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3713, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4987] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3715, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4988] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3715, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4989] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3715, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4990] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3715, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4991] Ref(BracketedKeyPartListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3715, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4992] Ref(IndexOptionsSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3715, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4993] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3715, child_count: 6, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4994] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3721, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4995] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3721, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4996] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3721, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4997] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3721, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4998] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3721, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4999] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5000] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3724, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5001] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3727, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5002] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3727, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5003] StringParser("TABLE")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(20), first_child_idx: 3727, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5004] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3729, child_count: 27, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5005] OneOf(27 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3756, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5006] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3759, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5007] Ref(Autoextend_sizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3759, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5008] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3759, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5009] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3759, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5010] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3762, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5011] Ref(Auto_incrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3762, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5012] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3762, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5013] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3762, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5014] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3765, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5015] Ref(Avg_row_lengthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3765, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5016] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3765, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5017] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3765, child_count: 4, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5018] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3769, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5019] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3769, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5020] Ref(CharsetGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3769, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5021] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3769, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5022] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3772, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5023] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3772, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5024] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3772, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5025] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3772, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5026] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3775, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5027] Ref(ChecksumKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3775, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5028] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3775, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5029] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3775, child_count: 4, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5030] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3779, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5031] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3779, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5032] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3779, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5033] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3779, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5034] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3782, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5035] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3782, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5036] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3782, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5037] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3782, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5038] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3785, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5039] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3785, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5040] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3785, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5041] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3785, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5042] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3788, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5043] Ref(CompressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3788, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5044] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3788, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5045] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3788, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5046] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3791, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5047] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3791, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5048] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3791, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5049] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3791, child_count: 4, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5050] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3795, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5051] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3797, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5052] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3797, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5053] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3797, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5054] Ref(DirectoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3797, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5055] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3797, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5056] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3797, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5057] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3800, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5058] Ref(Delay_key_writeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3800, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5059] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3800, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5060] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3800, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5061] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3803, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5062] Ref(EncryptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3803, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5063] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3803, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5064] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3803, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5065] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5066] Ref(EngineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5067] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3806, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5068] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3808, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5069] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3808, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5070] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3808, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5071] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3811, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5072] Ref(Engine_attributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3811, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5073] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3811, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5074] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3811, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5075] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3814, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5076] Ref(Insert_methodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3814, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5077] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3814, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5078] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3817, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5079] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3817, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5080] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3817, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5081] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3817, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5082] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3820, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5083] Ref(Key_block_sizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3820, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5084] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3820, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5085] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3820, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5086] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3823, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5087] Ref(Max_rowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3823, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5088] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3823, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5089] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3823, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5090] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3826, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5091] Ref(Min_rowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3826, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5092] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3826, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5093] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3826, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5094] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3829, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5095] Ref(Pack_keysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3829, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5096] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3829, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5097] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3831, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5098] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3831, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5099] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3831, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5100] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5101] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5102] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5103] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3834, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5104] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3837, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5105] Ref(Row_formatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3837, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5106] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3837, child_count: 6, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5107] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3843, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5108] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3843, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5109] Ref(DynamicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3843, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5110] Ref(FixedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3843, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5111] Ref(CompressedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3843, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5112] Ref(RedundantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3843, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5113] Ref(CompactKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3843, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5114] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3846, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5115] Ref(Secondary_engine_attributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3846, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5116] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3846, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5117] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3846, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5118] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3849, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5119] Ref(Stats_auto_recalcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3849, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5120] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3849, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5121] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3851, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5122] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3851, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5123] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3851, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5124] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3854, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5125] Ref(Stats_persistentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3854, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5126] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3854, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5127] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3856, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5128] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3856, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5129] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3856, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5130] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3859, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5131] Ref(Stats_sample_pagesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3859, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5132] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3859, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5133] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3859, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5134] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3862, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5135] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3862, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5136] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3862, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5137] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3864, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5138] Ref(StorageKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3864, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5139] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3866, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5140] Ref(DiskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3866, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5141] Ref(MemoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3866, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5142] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3869, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5143] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3869, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5144] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3869, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5145] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3872, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5146] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3874, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5147] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3874, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5148] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3876, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5149] Ref(DualIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3876, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 1, _padding: 0 }, // [5150] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5151] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5152] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5153] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5154] StringParser("TABLE_CHECKSUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5155] StringParser("TABLE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5156] StringParser("TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5157] StringParser("TABLESAMPLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5158] StringParser("TABLESPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5159] StringParser("TABLET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5160] StringParser("TABLETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5161] StringParser("TAG")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3878, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5162] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3880, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 1, _padding: 0 }, // [5163] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3881, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5164] Ref(Expression_A_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3881, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5165] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3881, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5166] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3881, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5167] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3883, child_count: 1, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5168] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3884, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5169] Ref(Expression_B_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3884, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5170] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3884, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5171] StringParser("TASK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3884, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5172] StringParser("TASKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3884, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5173] StringParser("TEMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3884, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5174] StringParser("TEMPLATE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3884, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5175] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3884, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5176] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3886, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5177] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3886, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5178] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3886, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5179] StringParser("TEMPORARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3886, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5180] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3888, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5181] Ref(TransientKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3888, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5182] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3888, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5183] StringParser("TEMPTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3888, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5184] StringParser("TERMINATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3888, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5185] StringParser("TERMINATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3888, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5186] StringParser("TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3888, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5187] StringParser("TEXTSIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3888, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5188] StringParser("THAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3888, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5189] StringParser("THEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3888, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5190] StringParser("THREAD_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3888, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5191] StringParser("TIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3888, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5192] StringParser("~")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3888, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5193] StringParser("TIME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3888, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5194] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3891, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5195] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3893, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5196] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3893, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5197] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3893, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5198] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3893, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5199] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3896, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5200] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3898, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5201] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3898, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5202] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3898, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5203] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3898, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5204] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3898, child_count: 1, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5205] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3899, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5206] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5207] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5208] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5209] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5210] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5211] StringParser("TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5212] StringParser("TIMESTAMPADD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5213] StringParser("TIMESTAMPDIFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5214] StringParser("TIMEZONE_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5215] StringParser("TIMEZONE_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5216] StringParser("TINYBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5217] StringParser("TINYINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5218] StringParser("TINYTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5219] StringParser("TLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5220] StringParser("TO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5221] StringParser("TOAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5222] StringParser("TOKEN_FILTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5223] StringParser("TOKENIZER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5224] StringParser("TOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5225] StringParser("TOP_LEVEL_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5226] StringParser("TRAILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5227] StringParser("TRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5228] StringParser("TRANSACTION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3903, child_count: 6, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5229] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3909, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5230] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3911, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5231] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3911, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5232] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3911, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5233] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3913, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5234] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3915, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5235] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3915, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5236] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3915, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5237] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3918, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5238] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3918, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5239] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3918, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5240] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3918, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5241] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3920, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5242] Ref(LeaveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3920, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5243] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3920, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5244] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3923, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5245] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3923, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5246] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3923, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5247] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3926, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5248] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3926, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5249] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3926, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5250] Ref(ChainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3926, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5251] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3928, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5252] Ref(RollbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3928, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5253] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3928, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5254] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5255] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5256] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5257] StringParser("TRANSACTION_ACTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5258] StringParser("TRANSACTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5259] StringParser("TRANSACTIONS_COMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5260] StringParser("TRANSACTIONS_ROLLED_BACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5261] StringParser("TRANSFORM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5262] StringParser("TRANSFORMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5263] StringParser("TRANSIENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5264] StringParser("TRANSLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5265] StringParser("TRANSLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5266] StringParser("TRASH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5267] StringParser("TREAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5268] StringParser("TREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5269] StringParser("TRIGGER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5270] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5271] StringParser("TRIGGER_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5272] StringParser("TRIGGER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5273] StringParser("TRIGGER_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5274] StringParser("TRIGGERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5275] StringParser("TRIM")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3930, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5276] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3933, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5277] Ref(BothKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3933, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5278] Ref(LeadingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3933, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5279] Ref(TrailingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3933, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5280] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3933, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5281] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3933, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5282] StringParser("TRUNCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3933, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5283] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3936, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5284] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3936, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5285] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3936, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5286] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3936, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5287] StringParser("TRUSTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3936, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5288] StringParser("TSEQUAL")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3936, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5289] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3939, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5290] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5291] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5292] StringParser("TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5293] StringParser("TYPECAST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3941, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5294] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3943, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5295] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3943, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5296] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3943, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5297] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3945, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5298] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3945, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5299] Ref(StructLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3945, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5300] StringParser("TYPES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3945, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5301] StringParser("UESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3945, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5302] StringParser("UID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3945, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5303] StringParser("UNBOUNDED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3945, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5304] StringParser("UNCOMMITTED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3945, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5305] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3945, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5306] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5307] Ref(NaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5308] Ref(UnconditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5309] Ref(HorizontalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5310] StringParser("UNDEFINED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5311] StringParser("UNDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5312] StringParser("UNDO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5313] StringParser("UNDO_BUFFER_SIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5314] StringParser("UNDOFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5315] StringParser("UNENCRYPTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5316] StringParser("UNICODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5317] StringParser("UNINSTALL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3948, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5318] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3950, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5319] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3950, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5320] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5321] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5322] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5323] StringParser("UNION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3952, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5324] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3954, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5325] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3954, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5326] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3954, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5327] StringParser("UNIQUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3954, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5328] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3954, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5329] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3954, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5330] StringParser("UNLISTEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3954, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5331] StringParser("UNLOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3954, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5332] StringParser("UNNAMED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3954, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5333] StringParser("UNNEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 3954, child_count: 11, min_times: 0, first_terminator_idx: 42, terminator_count: 11, _padding: 0 }, // [5334] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5335] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5336] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5337] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5338] Ref(IndexHintClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5339] Ref(SelectPartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5340] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5341] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5342] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5343] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5344] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5345] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5346] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5347] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5348] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5349] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5350] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5351] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5352] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5353] Ref(IndexHintClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5354] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5355] Ref(SelectPartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [5356] Ref(UpsertClauseListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3965, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5357] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3967, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5358] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3967, child_count: 1, min_times: 1, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5359] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3968, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5360] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3970, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5361] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3970, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5362] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3970, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5363] StringParser("UNREGISTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3970, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5364] StringParser("UNSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3970, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5365] StringParser("UNSIGNED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3970, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5366] StringParser("UNTIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3970, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5367] StringParser("UP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3970, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5368] StringParser("UPDATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3970, child_count: 10, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5369] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3980, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5370] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3980, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5371] Ref(Low_priorityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3980, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5372] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3980, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5373] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3982, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5374] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3984, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5375] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3984, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5376] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3984, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5377] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3984, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5378] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3984, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5379] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3984, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5380] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3984, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5381] StringParser("UPDATETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3984, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5382] StringParser("UPGRADE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3984, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5383] StringParser("UPPER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3984, child_count: 5, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5384] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3989, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5385] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3989, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5386] Ref(DuplicateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3989, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5387] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3989, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5388] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3989, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5389] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3991, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5390] Ref(SetClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3991, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5391] StringParser("USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3991, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5392] StringParser("USE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3991, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5393] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5394] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5395] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5396] StringParser("USE_ANY_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5397] StringParser("USE_FRM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5398] StringParser("USE_MV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5399] StringParser("USER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5400] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5401] StringParser("USER_DEFINED_TYPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5402] StringParser("USER_DEFINED_TYPE_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5403] StringParser("USER_DEFINED_TYPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5404] StringParser("USER_DEFINED_TYPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5405] StringParser("USER_RESOURCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5406] StringParser("USERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5407] StringParser("USING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5408] StringParser("UTC_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5409] StringParser("UTC_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5410] StringParser("UTC_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5411] StringParser("VACUUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5412] StringParser("VALID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5413] StringParser("VALIDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5414] StringParser("VALIDATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5415] StringParser("VALIDATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3993, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5416] StringParser("VALUE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3993, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5417] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3995, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5418] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3997, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5419] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3997, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5420] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3997, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5421] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3999, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5422] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4001, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5423] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 4001, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5424] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4004, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5425] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4006, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5426] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4009, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5427] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4009, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5428] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4009, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5429] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4009, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5430] StringParser("VALUES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4009, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5431] StringParser("VAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4009, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5432] StringParser("VAR_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4009, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5433] StringParser("VARBINARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4009, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5434] StringParser("VARCHAR2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4009, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5435] StringParser("VARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4009, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5436] StringParser("VARCHARACTER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4009, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5437] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5438] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5439] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5440] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5441] StringParser("VARIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5442] StringParser("VARIABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5443] StringParser("VARIANT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5444] StringParser("VARYING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5445] StringParser("VAULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5446] StringParser("VAULTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5447] StringParser("VCPU")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5448] StringParser("VERBOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5449] StringParser("VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5450] StringParser("VIEW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5451] StringParser("VIEWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5452] StringParser("VIRTUAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5453] StringParser("VISIBLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5454] StringParser("VOLATILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5455] StringParser("WAIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5456] StringParser("WAITFOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5457] StringParser(":=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5458] StringParser("WAREHOUSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5459] StringParser("WAREHOUSES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5460] StringParser("WARM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5461] StringParser("WARNINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5462] StringParser("WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5463] StringParser("WEEKDAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4012, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5464] StringParser("WEIGHT_STRING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4012, child_count: 8, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5465] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4020, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5466] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4020, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5467] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5468] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5469] Meta(conditional: indented_then=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5470] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5471] Meta(conditional: indented_then_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5472] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5473] Meta(conditional: indented_then_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5474] Meta(conditional: indented_then=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5475] StringParser("WHEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5476] StringParser("WHENEVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4023, child_count: 4, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5477] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4027, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5478] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4027, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5479] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4029, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5480] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4032, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5481] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4032, child_count: 9, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5482] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4041, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5483] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4041, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5484] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4043, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5485] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4043, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5486] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4043, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5487] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4045, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5488] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4045, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5489] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4045, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5490] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4045, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5491] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4045, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5492] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4045, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5493] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4045, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5494] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4045, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5495] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4045, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5496] StringParser("WHERE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4045, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5497] StringParser("WHILE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4045, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5498] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4047, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5499] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4049, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5500] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4051, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5501] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4051, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5502] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4051, child_count: 4, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5503] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4055, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5504] Ref(WhileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4055, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5505] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4055, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5506] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4055, child_count: 1, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5507] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4056, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5508] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4056, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5509] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4059, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5510] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4059, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5511] Ref(WhileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4059, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5512] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4059, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5513] StringParser("WHITELIST")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4059, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5514] Token("whitespace")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4059, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5515] StringParser("WIDTH_BUCKET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4059, child_count: 1, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5516] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4060, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5517] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4060, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5518] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4062, child_count: 1, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5519] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4063, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5520] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4065, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5521] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4067, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5522] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4067, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5523] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4067, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5524] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5525] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5526] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5527] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5528] StringParser("WINDOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4069, child_count: 4, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5529] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 4073, child_count: 1, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5530] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4074, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5531] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5532] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5533] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5534] Ref(PartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5535] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5536] Ref(FrameClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4076, child_count: 4, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5537] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4080, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5538] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4080, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5539] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4082, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5540] Ref(CascadedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4082, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5541] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4082, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5542] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4082, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5543] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4082, child_count: 6, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5544] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4088, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5545] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4088, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5546] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4088, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5547] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4088, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 1, _padding: 0 }, // [5548] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4090, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5549] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4090, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5550] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4090, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5551] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4090, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5552] Ref(NonWithNonSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4090, child_count: 6, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5553] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4096, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5554] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4096, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5555] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4096, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5556] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4096, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 1, _padding: 0 }, // [5557] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4098, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5558] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4098, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5559] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4098, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5560] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4098, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5561] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4098, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5562] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4101, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5563] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4101, child_count: 1, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5564] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4102, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5565] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4102, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5566] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4102, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5567] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4102, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5568] StringParser("WITH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4102, child_count: 4, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5569] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5570] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5571] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5572] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5573] Ref(BindingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5574] StringParser("WITHIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5575] StringParser("WITHOUT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5576] Token("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5577] StringParser("WORK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5578] StringParser("WORKLOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5579] StringParser("WRAPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5580] StringParser("WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5581] StringParser("WRITETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5582] StringParser("X509")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5583] StringParser("XA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5584] StringParser("XID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5585] StringParser("XML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5586] StringParser("XOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5587] StringParser("YAML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5588] StringParser("YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5589] StringParser("YEAR_MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5590] StringParser("ZEROFILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [5591] StringParser("ZONE")
];

pub static CHILD_IDS: &[u32] = &[
    5, 6, 61, 7, 51, 8, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
    21, 28, 33, 38, 39, 44, 9, 10, 22, 23, 24, 25, 26, 27, 29, 30,
    31, 32, 34, 35, 36, 37, 40, 41, 42, 43, 45, 46, 47, 48, 49, 50,
    52, 58, 53, 54, 57, 55, 56, 62, 63, 64, 66, 74, 78, 81, 84, 89,
    67, 68, 69, 70, 71, 72, 73, 75, 76, 77, 79, 80, 82, 83, 85, 86,
    87, 88, 90, 133, 91, 100, 105, 108, 109, 110, 111, 112, 113, 114, 115, 116,
    117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 92, 93,
    94, 95, 98, 99, 96, 97, 101, 102, 103, 104, 106, 107, 131, 132, 135, 58,
    138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 151, 154, 157, 149, 150,
    152, 153, 155, 156, 158, 159, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170,
    171, 174, 177, 180, 172, 173, 175, 176, 178, 179, 181, 182, 184, 185, 187, 191,
    195, 199, 203, 205, 207, 188, 189, 190, 58, 192, 193, 194, 58, 196, 197, 198,
    58, 200, 201, 202, 58, 204, 58, 206, 58, 209, 228, 229, 230, 239, 231, 237,
    238, 232, 233, 234, 235, 236, 242, 243, 248, 249, 252, 253, 250, 251, 254, 256,
    257, 258, 259, 260, 275, 280, 284, 293, 294, 261, 262, 263, 266, 267, 269, 264,
    265, 268, 270, 271, 274, 272, 273, 276, 277, 278, 279, 281, 282, 283, 285, 286,
    287, 288, 289, 290, 291, 292, 295, 296, 299, 300, 309, 314, 319, 301, 302, 303,
    304, 305, 306, 307, 308, 310, 311, 312, 313, 315, 316, 317, 318, 320, 321, 322,
    323, 324, 325, 327, 331, 332, 333, 338, 341, 328, 329, 330, 334, 337, 335, 336,
    339, 340, 343, 344, 345, 346, 347, 349, 350, 351, 352, 354, 362, 355, 356, 357,
    358, 359, 360, 361, 363, 364, 365, 366, 367, 368, 369, 371, 377, 390, 391, 372,
    373, 374, 375, 376, 378, 381, 382, 383, 379, 380, 384, 389, 385, 388, 386, 387,
    392, 393, 396, 394, 395, 398, 399, 400, 401, 530, 402, 58, 403, 404, 405, 406,
    417, 437, 447, 450, 462, 482, 493, 500, 520, 525, 407, 408, 409, 410, 411, 412,
    413, 416, 414, 415, 418, 419, 420, 421, 422, 432, 423, 429, 424, 425, 426, 427,
    428, 430, 431, 433, 434, 435, 436, 438, 439, 440, 441, 442, 443, 446, 444, 445,
    448, 449, 451, 452, 453, 454, 455, 456, 457, 458, 459, 460, 461, 463, 464, 465,
    468, 473, 474, 477, 466, 467, 469, 472, 470, 471, 475, 476, 478, 481, 479, 480,
    483, 484, 487, 488, 485, 486, 489, 490, 491, 492, 494, 495, 496, 497, 498, 499,
    501, 502, 503, 508, 515, 504, 507, 505, 506, 509, 512, 513, 514, 510, 511, 516,
    517, 518, 519, 521, 524, 522, 523, 526, 527, 528, 529, 531, 546, 549, 556, 557,
    565, 532, 533, 534, 535, 536, 537, 538, 539, 540, 541, 542, 543, 544, 545, 547,
    548, 550, 551, 552, 553, 554, 235, 236, 555, 58, 558, 559, 560, 561, 564, 562,
    563, 566, 567, 568, 235, 236, 569, 58, 571, 572, 579, 580, 586, 587, 588, 589,
    590, 593, 573, 574, 575, 576, 577, 578, 581, 582, 583, 584, 585, 591, 592, 592,
    235, 236, 599, 600, 603, 604, 601, 602, 605, 608, 619, 606, 607, 58, 609, 610,
    611, 612, 613, 615, 614, 58, 616, 617, 618, 620, 621, 622, 623, 624, 625, 58,
    630, 631, 638, 639, 640, 641, 642, 643, 644, 645, 646, 647, 648, 649, 651, 656,
    657, 652, 655, 653, 654, 661, 656, 657, 662, 58, 665, 683, 695, 696, 697, 698,
    699, 700, 701, 704, 705, 706, 702, 703, 720, 721, 722, 723, 724, 725, 739, 740,
    743, 744, 752, 753, 755, 758, 756, 757, 759, 760, 763, 764, 767, 235, 236, 768,
    58, 770, 235, 236, 771, 58, 773, 235, 236, 774, 58, 775, 782, 776, 777, 780,
    778, 779, 781, 235, 236, 783, 784, 787, 235, 236, 802, 235, 236, 804, 805, 806,
    807, 808, 235, 236, 813, 814, 821, 831, 822, 823, 824, 828, 239, 830, 825, 832,
    833, 823, 834, 838, 239, 840, 835, 866, 871, 874, 889, 867, 868, 869, 870, 872,
    873, 875, 878, 881, 885, 876, 877, 879, 880, 882, 883, 884, 886, 887, 888, 890,
    891, 892, 893, 894, 898, 901, 899, 900, 905, 906, 907, 909, 908, 58, 910, 913,
    914, 915, 916, 917, 911, 912, 922, 923, 924, 926, 925, 58, 927, 928, 945, 946,
    950, 951, 952, 953, 964, 965, 965, 235, 236, 966, 967, 968, 969, 971, 1024, 1027,
    972, 1000, 1007, 1008, 1018, 1021, 973, 976, 974, 975, 977, 980, 984, 987, 990, 991,
    992, 995, 996, 999, 978, 979, 981, 982, 983, 235, 236, 985, 986, 988, 989, 993,
    994, 997, 998, 1001, 1002, 1003, 1004, 1005, 1006, 1009, 1012, 1013, 1015, 1010, 1011, 1014,
    235, 236, 1016, 1017, 1019, 1020, 1022, 1023, 1025, 1026, 1028, 1029, 1030, 1031, 1032, 235,
    236, 1034, 1035, 1064, 1066, 1068, 1036, 1040, 1037, 1038, 1039, 1041, 1044, 1045, 1042, 1043,
    1046, 1050, 1058, 1047, 1049, 1048, 1051, 1052, 1053, 1054, 1055, 1056, 1057, 1059, 1060, 1061,
    1062, 1063, 1065, 235, 236, 1067, 1069, 1070, 1071, 1072, 1073, 1074, 1075, 1080, 1081, 1094,
    1095, 1103, 1104, 1105, 1106, 1107, 1108, 1109, 1110, 1111, 1124, 1125, 1131, 1132, 1133, 1153,
    1154, 235, 236, 1155, 1158, 1156, 1157, 1159, 1160, 1161, 1162, 1174, 1175, 1176, 1180, 1181,
    1182, 1192, 1193, 1194, 1197, 1177, 1178, 1179, 235, 236, 1183, 1184, 1185, 1186, 1187, 1191,
    1188, 1189, 1190, 1195, 1196, 1198, 1199, 1201, 1202, 1205, 1206, 1207, 1203, 1204, 1208, 1210,
    1211, 1212, 1213, 1214, 1215, 1216, 1217, 1220, 1221, 1223, 1229, 1234, 1243, 1244, 1245, 1218,
    1219, 1222, 1224, 1225, 1228, 1226, 1227, 1230, 1231, 1232, 1233, 1235, 1236, 1237, 1238, 1239,
    1240, 1241, 1242, 1247, 1248, 1249, 1250, 1251, 1252, 1255, 1256, 1257, 1253, 1254, 1259, 1260,
    1264, 1265, 1266, 1267, 1268, 1269, 1270, 1271, 1261, 1262, 1263, 1272, 1281, 1273, 1274, 1275,
    1276, 1277, 1278, 1279, 1280, 1282, 1283, 1284, 1285, 1286, 1287, 1288, 1291, 1292, 1293, 1294,
    1295, 1296, 1308, 1309, 1297, 1298, 1299, 235, 236, 1300, 58, 1301, 1302, 1303, 1304, 1305,
    1306, 656, 657, 1307, 58, 1311, 1312, 1313, 1320, 1324, 1314, 1315, 1316, 1317, 1318, 1319,
    1321, 1322, 1323, 1325, 1326, 1327, 1329, 1330, 1331, 1332, 1333, 1334, 1335, 1336, 1337, 1339,
    1340, 1341, 1342, 1344, 1345, 1346, 1347, 1349, 1353, 1357, 1358, 1359, 1364, 1367, 1350, 1351,
    1352, 1354, 1355, 1356, 1360, 1363, 1361, 1362, 1365, 1366, 1369, 1370, 1371, 1372, 1373, 1375,
    1376, 1377, 1378, 1379, 1380, 1381, 1382, 1383, 1426, 1429, 1384, 1390, 1394, 1414, 1415, 1416,
    1417, 1418, 1385, 235, 236, 1386, 58, 1387, 1388, 1389, 1391, 1392, 1393, 1395, 1405, 1408,
    1396, 1399, 1402, 1397, 1398, 1400, 1401, 1403, 1404, 1406, 235, 236, 1407, 58, 1409, 1410,
    1411, 1412, 235, 236, 1413, 58, 1419, 1420, 1421, 235, 236, 1422, 58, 1423, 1424, 1425,
    1427, 1428, 1430, 1434, 1435, 1443, 1444, 1431, 1432, 1433, 1436, 1437, 1438, 235, 236, 1439,
    58, 1440, 1441, 1442, 1445, 1446, 1446, 235, 236, 1448, 1449, 1450, 1451, 1452, 1453, 1454,
    1457, 1461, 1462, 1463, 1467, 1472, 1455, 1456, 1458, 1459, 1460, 1464, 1465, 1466, 1468, 1471,
    1469, 1470, 1473, 1474, 1475, 1476, 1477, 1479, 1480, 1481, 1482, 1530, 1535, 1553, 1563, 1607,
    1612, 1483, 58, 1484, 1485, 1486, 1487, 1529, 1488, 1489, 1490, 1497, 1491, 1492, 1493, 1496,
    1494, 1495, 1498, 1499, 1500, 1501, 1502, 1509, 1512, 1503, 1504, 1505, 1508, 1506, 1507, 1510,
    1511, 1513, 1514, 1515, 1516, 1517, 1524, 1518, 1519, 1520, 1523, 1521, 1522, 1525, 1526, 1527,
    1528, 1531, 1532, 1533, 1534, 58, 1536, 1537, 1538, 1539, 1540, 1552, 1541, 1542, 1543, 1546,
    1549, 1544, 1545, 1547, 1548, 1550, 1551, 1554, 1555, 1556, 1557, 1562, 1558, 1559, 1560, 1561,
    1564, 1565, 1576, 1582, 1591, 1599, 1602, 1566, 1567, 1568, 1569, 1570, 1571, 1572, 1573, 1574,
    1575, 1577, 1578, 1579, 1580, 1581, 1583, 1584, 1585, 1586, 1587, 1588, 1589, 1590, 1592, 1593,
    1594, 1595, 1596, 1597, 1598, 1600, 1601, 1603, 1604, 1605, 1606, 1608, 1609, 1610, 1611, 1613,
    1616, 1614, 1615, 1618, 1619, 1620, 1627, 1628, 1634, 1635, 1636, 1637, 1638, 1641, 1621, 1622,
    1623, 1624, 1625, 1626, 1629, 1630, 1631, 1632, 1633, 1639, 1640, 1640, 235, 236, 1652, 1655,
    1653, 1654, 1656, 235, 236, 1660, 235, 236, 1662, 1663, 1664, 1665, 1667, 1668, 1679, 1680,
    1683, 1684, 1685, 1681, 1682, 1686, 58, 1687, 1688, 1691, 1694, 1692, 1693, 1695, 1696, 1704,
    1705, 1706, 1708, 1709, 1712, 1717, 1745, 1710, 1711, 1713, 1716, 1714, 1715, 1718, 1733, 1734,
    1719, 1728, 1720, 1723, 1721, 1722, 1724, 1725, 1726, 1727, 1729, 1732, 1730, 1731, 1735, 1736,
    1737, 1738, 1739, 1742, 1740, 1741, 1743, 1744, 1750, 1751, 235, 236, 1752, 58, 1753, 1754,
    1756, 1760, 1757, 1758, 1759, 1761, 1762, 1783, 1788, 1784, 1787, 1785, 1786, 1795, 1801, 1825,
    1833, 1796, 1797, 1798, 1799, 1800, 1802, 1803, 1807, 1808, 1809, 1823, 1804, 1805, 1806, 1810,
    1811, 1812, 1815, 1819, 1813, 1814, 1816, 1817, 1818, 1820, 1821, 1822, 1824, 1826, 1827, 1828,
    1829, 1830, 1831, 1832, 1834, 1835, 1836, 1837, 1838, 1839, 1840, 1841, 1842, 1846, 1847, 1855,
    1856, 1857, 1864, 1865, 1866, 1867, 1868, 1869, 1876, 1882, 1870, 1871, 1874, 1875, 1872, 58,
    1877, 1880, 1881, 1878, 58, 1883, 1884, 1885, 1886, 1887, 1889, 1890, 1891, 1892, 1894, 1895,
    1896, 58, 1898, 1899, 1913, 1914, 1915, 1939, 1940, 1941, 1942, 1952, 1943, 1944, 1947, 1945,
    235, 236, 1946, 58, 1948, 1949, 1950, 1951, 1953, 1954, 1955, 1956, 1957, 1958, 1971, 1972,
    1974, 1975, 1976, 1980, 1977, 1978, 1979, 235, 236, 1982, 1983, 1984, 1985, 1986, 1988, 1989,
    1990, 1991, 1993, 1994, 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2011, 2005, 2006,
    2007, 2008, 2009, 2010, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2021, 2022, 2023, 2024, 2026,
    2027, 2030, 2031, 2028, 2029, 2033, 2034, 2035, 2036, 2038, 2039, 2040, 2041, 2042, 2044, 2045,
    2046, 2048, 2049, 2050, 2051, 2052, 2053, 2055, 2056, 2057, 2058, 2060, 2061, 2062, 2063, 2064,
    2066, 2067, 2068, 2069, 2071, 2072, 2073, 2074, 2075, 2090, 823, 2091, 239, 235, 236, 2097,
    2098, 2136, 2137, 2138, 2139, 2140, 2141, 58, 2152, 2153, 2154, 2155, 2156, 2157, 2161, 2162,
    2163, 2164, 2165, 2168, 2169, 2170, 2174, 2175, 2176, 2177, 2183, 2166, 2167, 2171, 2172, 2173,
    2178, 2179, 2180, 2181, 2182, 2184, 2185, 2187, 2190, 2191, 2192, 2188, 2189, 2194, 2195, 2196,
    2197, 2198, 2202, 2199, 2200, 2201, 2204, 2207, 2205, 2206, 2209, 2212, 2210, 2211, 2213, 2217,
    2223, 2214, 2215, 2216, 235, 236, 2218, 2221, 2219, 2220, 2222, 2226, 2261, 2227, 2228, 2229,
    2241, 2242, 2243, 2250, 2258, 2259, 2230, 235, 236, 2231, 2232, 2233, 2234, 2235, 58, 2236,
    2237, 2238, 2239, 2240, 2244, 2247, 2245, 2246, 2248, 235, 236, 2249, 58, 2251, 2252, 2253,
    2254, 2255, 2256, 2257, 2263, 2264, 2265, 2266, 2267, 2268, 2269, 2270, 2271, 2286, 2287, 2290,
    2294, 2297, 2288, 2289, 2291, 2292, 2293, 2295, 2296, 2298, 2299, 2300, 2301, 2306, 2308, 2312,
    2307, 2309, 2310, 2311, 2313, 2317, 2318, 2319, 235, 236, 2320, 2321, 2334, 2335, 2338, 2336,
    2337, 2339, 2369, 2379, 2340, 58, 2341, 2344, 2347, 2350, 2353, 2354, 2355, 2356, 2357, 2364,
    2367, 2368, 2342, 2343, 2345, 2346, 2348, 2349, 2351, 2352, 2358, 2359, 2360, 2361, 2362, 2363,
    2365, 2366, 2370, 2371, 2375, 2372, 2373, 58, 2376, 2377, 2378, 2380, 2381, 2385, 2382, 2383,
    58, 2386, 2387, 2392, 2407, 2393, 2398, 2402, 2394, 2395, 2396, 2397, 2399, 2400, 2401, 58,
    2403, 2404, 2405, 2406, 2408, 2409, 2410, 2411, 2415, 2416, 2423, 2424, 2425, 2438, 2426, 2429,
    2427, 2428, 2430, 2435, 2431, 2432, 2433, 2434, 2436, 2437, 2439, 2425, 2440, 2425, 2442, 2443,
    2449, 2450, 2451, 58, 2453, 2454, 2455, 2458, 2461, 2462, 2463, 2464, 2465, 2466, 2467, 2468,
    2469, 2470, 2471, 2472, 2473, 2456, 2457, 2459, 2460, 2475, 2493, 2476, 2477, 2480, 2481, 2487,
    2491, 2492, 2478, 2479, 2479, 235, 236, 2482, 2483, 2484, 2485, 2486, 2488, 2489, 2490, 2494,
    235, 236, 2475, 2495, 2496, 2498, 2499, 2499, 235, 236, 823, 2500, 239, 2511, 2512, 2523,
    2501, 2502, 2503, 2504, 235, 236, 2506, 2507, 2509, 2510, 2513, 2514, 2515, 2516, 2518, 2519,
    2521, 2522, 2531, 2532, 2536, 2542, 2548, 2554, 2555, 2558, 2568, 2569, 2570, 2533, 2534, 2535,
    2537, 2538, 2540, 2541, 2539, 2543, 2546, 2547, 2544, 2545, 2549, 2550, 2551, 2552, 2553, 58,
    2556, 2557, 2559, 2563, 2564, 2560, 2561, 2562, 2565, 2566, 2567, 2572, 2573, 235, 236, 2578,
    2583, 2579, 2580, 2581, 2584, 2585, 2588, 2595, 2589, 2590, 2591, 2594, 2592, 2593, 2596, 2599,
    2597, 2598, 2601, 235, 236, 2602, 58, 2604, 2607, 2610, 2613, 2616, 2617, 2605, 2606, 2608,
    2609, 2611, 2612, 2614, 2615, 2618, 2625, 2619, 2624, 2620, 2621, 2622, 2623, 2637, 2638, 2641,
    2642, 2651, 2652, 2656, 2639, 2640, 2643, 58, 2644, 2647, 2648, 2645, 2646, 2649, 2650, 2653,
    2654, 2655, 2657, 58, 2658, 2661, 2662, 2659, 2660, 2663, 2664, 2665, 2666, 2667, 2668, 2669,
    2670, 2671, 2672, 2673, 2674, 2675, 2686, 2687, 2694, 2695, 2697, 2688, 2692, 2689, 2690, 2691,
    2693, 58, 2696, 58, 2698, 2702, 2699, 2700, 2701, 2703, 2704, 2705, 2710, 2711, 2715, 2716,
    228, 2717, 2728, 239, 2718, 2719, 2720, 2721, 2722, 2723, 58, 2724, 2725, 2726, 2729, 2730,
    2732, 2735, 2736, 2737, 2738, 2739, 2733, 2734, 2743, 2744, 58, 2745, 2746, 2747, 2748, 235,
    236, 2752, 2753, 2754, 2755, 235, 236, 2756, 58, 2757, 2758, 2765, 823, 2766, 239, 2767,
    2768, 2768, 235, 236, 2770, 2773, 2774, 2775, 2776, 2771, 2772, 2783, 2784, 2812, 2813, 2815,
    2820, 2825, 2828, 2816, 2817, 2818, 2819, 2821, 2822, 2823, 2824, 2826, 2827, 2829, 2830, 2833,
    2834, 2835, 2838, 2841, 2839, 2840, 2852, 2853, 2854, 2855, 2860, 2856, 235, 236, 2857, 2859,
    2858, 58, 2867, 2868, 2869, 2870, 2872, 235, 236, 2874, 2875, 2876, 2877, 2880, 2886, 2894,
    2878, 235, 236, 2879, 58, 2881, 2882, 2883, 2884, 2885, 2887, 2888, 2889, 235, 236, 2890,
    58, 2891, 2892, 2893, 2895, 2896, 2898, 2902, 2905, 2915, 2917, 2899, 2900, 2901, 2903, 2904,
    2906, 2907, 2908, 2909, 2912, 2910, 2911, 2913, 2914, 2916, 235, 236, 2920, 2924, 2925, 2929,
    2930, 2933, 2937, 2921, 2922, 2923, 2926, 2927, 2928, 2931, 2932, 2934, 2935, 2936, 2938, 2939,
    2940, 2942, 2943, 2944, 2945, 2967, 2968, 2969, 2970, 235, 236, 2972, 2973, 2974, 2975, 2980,
    2984, 2985, 2976, 2977, 2978, 235, 236, 2979, 58, 2981, 2982, 2983, 2986, 2987, 3008, 3009,
    3010, 3011, 3012, 3013, 3016, 3017, 3018, 3019, 58, 3020, 3023, 3026, 3021, 3022, 3024, 3025,
    3027, 3028, 3029, 3033, 3050, 3030, 3031, 3032, 3034, 3037, 3041, 3046, 3035, 3036, 3038, 3039,
    3040, 3042, 3043, 3044, 3045, 3047, 3048, 3049, 3051, 3052, 3056, 3053, 3054, 3055, 3057, 3058,
    3059, 3072, 3073, 3074, 3075, 3076, 3078, 3079, 3080, 3081, 3092, 3093, 3097, 3110, 3115, 3098,
    3099, 228, 3100, 3101, 239, 3103, 3102, 3104, 3105, 3106, 3109, 3107, 3108, 3111, 3112, 228,
    3113, 3114, 239, 3116, 228, 3117, 239, 3120, 3123, 3124, 3125, 3128, 3126, 3127, 3127, 235,
    236, 3130, 3131, 3132, 3136, 3133, 3134, 3135, 3138, 228, 3139, 239, 3140, 235, 236, 3141,
    58, 3144, 3145, 3146, 235, 236, 3147, 58, 3148, 3152, 3157, 3149, 3150, 3151, 3153, 3154,
    3155, 3156, 3158, 3159, 3160, 3161, 3165, 3162, 3163, 3164, 3166, 3167, 3168, 3169, 3170, 3176,
    3177, 3171, 3172, 3173, 3174, 3175, 3178, 3179, 3181, 3182, 3183, 3184, 235, 236, 3187, 3188,
    3189, 3190, 3197, 235, 236, 3191, 3192, 3193, 3194, 3195, 3196, 3198, 3199, 3205, 3206, 3200,
    3201, 3202, 3203, 3204, 3207, 3208, 3246, 3247, 3251, 3254, 3255, 3252, 3253, 3256, 3257, 3259,
    3260, 3261, 3265, 228, 3266, 3272, 239, 3267, 3268, 3268, 235, 236, 3269, 3270, 3271, 3273,
    3278, 3274, 3275, 3276, 3277, 3279, 3280, 3289, 3290, 3291, 3292, 3293, 3294, 3295, 235, 236,
    3296, 58, 3297, 3300, 3298, 235, 236, 3299, 58, 3303, 3304, 3305, 3306, 3307, 3308, 3309,
    3310, 3311, 3312, 3313, 3318, 3319, 3320, 3323, 3325, 3326, 3327, 3330, 3331, 3332, 3333, 3334,
    3338, 3356, 3366, 3372, 3376, 3321, 3322, 3324, 3328, 3329, 3335, 3336, 3337, 3339, 3342, 3346,
    3352, 3340, 3341, 3343, 3344, 3345, 3347, 3349, 3350, 3351, 3348, 3353, 3354, 3355, 3357, 3358,
    3362, 3359, 3360, 3361, 3363, 3364, 3365, 3367, 3368, 3369, 3370, 3371, 3373, 3374, 235, 236,
    3375, 58, 3377, 3378, 3399, 3406, 3400, 3403, 3404, 3401, 3402, 3405, 58, 3407, 3408, 3409,
    3414, 3415, 3416, 3417, 3418, 3421, 235, 236, 3419, 3420, 3422, 3423, 3424, 235, 236, 3484,
    228, 3485, 239, 3486, 3488, 3489, 3492, 3493, 3495, 3496, 3497, 3500, 228, 3501, 239, 3498,
    3499, 3502, 3503, 3505, 3506, 3507, 3508, 3511, 228, 3512, 239, 3509, 3510, 3514, 228, 3515,
    239, 3518, 228, 3519, 239, 3529, 3530, 3531, 3532, 3516, 3517, 3520, 3521, 3522, 3526, 3523,
    3525, 3524, 235, 236, 3527, 3528, 3534, 228, 3535, 239, 3582, 3583, 3584, 3585, 3586, 3587,
    235, 236, 3589, 228, 3590, 239, 3591, 58, 3597, 3598, 3632, 3633, 3634, 3636, 3638, 3640,
    3635, 235, 236, 3637, 235, 236, 3639, 235, 236, 3643, 3644, 3645, 3646, 3648, 3649, 3652,
    3650, 3651, 3651, 235, 236, 3663, 3666, 3664, 3665, 3667, 3668, 3672, 3673, 3684, 3685, 3686,
    3696, 3697, 3698, 3700, 3702, 2107, 3701, 58, 3704, 3705, 3706, 3707, 3709, 3710, 3711, 3712,
    3713, 3714, 3715, 3716, 3717, 3718, 3719, 785, 3728, 3729, 3733, 3730, 3731, 3732, 3734, 3735,
    3755, 3756, 3759, 3760, 3757, 3758, 3761, 58, 3769, 3770, 3771, 3773, 3774, 3776, 3777, 228,
    3778, 239, 3779, 58, 3780, 3784, 3787, 3792, 3781, 3782, 3783, 3785, 3786, 3788, 3789, 3790,
    3791, 3796, 3797, 3798, 3799, 3800, 3801, 3802, 3805, 3806, 228, 3817, 3818, 3819, 239, 3820,
    3821, 3822, 235, 236, 3825, 3826, 3827, 3832, 3828, 3829, 3830, 3831, 235, 236, 3857, 3858,
    228, 3859, 239, 3860, 3861, 3861, 235, 236, 3862, 58, 3865, 3873, 3866, 3867, 3868, 3869,
    3870, 3872, 3871, 235, 236, 235, 236, 3874, 3875, 3876, 3877, 3887, 3878, 3879, 3882, 3880,
    235, 236, 3881, 58, 3883, 235, 236, 3884, 58, 3885, 3886, 3888, 3889, 3892, 3890, 235,
    236, 3891, 58, 3893, 235, 236, 3894, 58, 3905, 3910, 3906, 3907, 3908, 3909, 3943, 3944,
    3946, 3947, 3958, 3959, 3960, 3961, 3962, 3963, 3964, 3969, 3970, 3980, 3987, 3981, 3985, 3986,
    3982, 3983, 3984, 3989, 235, 236, 3990, 58, 4001, 4002, 4005, 4006, 4003, 4004, 4007, 4010,
    4008, 4009, 4011, 4012, 4013, 4016, 4017, 4020, 4021, 4025, 4022, 4023, 4024, 4026, 235, 236,
    4037, 4038, 4044, 4045, 4046, 4047, 4048, 4066, 4049, 4057, 4050, 4051, 4052, 4053, 4054, 4055,
    235, 236, 4056, 58, 4058, 4059, 4062, 4063, 235, 236, 4060, 235, 236, 4061, 58, 4064,
    235, 236, 4065, 58, 4067, 4068, 235, 236, 4069, 58, 4070, 4071, 4072, 4074, 4075, 4077,
    4078, 4080, 4081, 4082, 4076, 235, 236, 4079, 235, 236, 4083, 4084, 4085, 4086, 4087, 4088,
    4113, 4114, 4115, 4116, 4117, 4118, 4122, 4119, 4120, 4121, 4123, 4124, 4125, 4128, 4129, 4130,
    4131, 4132, 4137, 4138, 4139, 4142, 4145, 4140, 4141, 4143, 4144, 4146, 4147, 4173, 4174, 4175,
    4176, 58, 4177, 4178, 4179, 4183, 4184, 4187, 4188, 4190, 4185, 4186, 4189, 58, 4191, 4192,
    4193, 4196, 4203, 4197, 4200, 4201, 4198, 4199, 4202, 4204, 4205, 4206, 4207, 4208, 4209, 4213,
    4214, 4217, 4219, 4220, 4221, 4215, 4216, 4218, 4222, 4225, 4226, 4223, 4224, 4227, 4228, 4229,
    4230, 4231, 4232, 4251, 4252, 4253, 4254, 4255, 4258, 4261, 4267, 4259, 4260, 4262, 4266, 4263,
    4264, 4265, 4268, 4269, 4270, 58, 4271, 4285, 4286, 4272, 4273, 4274, 4275, 4276, 4277, 4278,
    4279, 4280, 4281, 4282, 4283, 4284, 4287, 4288, 4289, 4303, 4304, 4315, 4316, 4320, 4334, 4335,
    4336, 4317, 4318, 4319, 4321, 4325, 4328, 4333, 4322, 4323, 4324, 4326, 4327, 4329, 4330, 4331,
    4332, 4343, 4356, 4344, 4349, 4345, 4346, 4347, 4348, 4350, 4351, 4352, 4353, 4354, 4355, 4362,
    4363, 4364, 235, 236, 4365, 58, 4366, 4367, 4370, 4368, 235, 236, 4369, 58, 4371, 4372,
    4373, 4374, 235, 236, 4375, 58, 4396, 4397, 4400, 4402, 4398, 4399, 4401, 235, 236, 4403,
    4405, 4404, 4406, 235, 236, 4431, 4432, 4433, 4434, 4436, 4440, 4441, 4442, 4443, 4444, 4445,
    4446, 4447, 4437, 4438, 4439, 4449, 4450, 228, 4451, 239, 4452, 58, 4456, 4458, 4459, 4462,
    4463, 4464, 4465, 4457, 4460, 4461, 4468, 4469, 4470, 235, 236, 4471, 58, 4473, 4474, 4475,
    4476, 4477, 4478, 4479, 4480, 4481, 4482, 4483, 4484, 4485, 4486, 4487, 4492, 4495, 4498, 4499,
    4493, 4494, 4494, 235, 236, 4496, 4497, 4497, 235, 236, 4500, 235, 236, 4508, 4511, 4509,
    4510, 4512, 4513, 4515, 4518, 4516, 4517, 4519, 4520, 4530, 4531, 4532, 58, 4533, 4538, 4544,
    4548, 4551, 4534, 4537, 4535, 4536, 4539, 4540, 4541, 4542, 4543, 4545, 4546, 4547, 4549, 4550,
    4552, 4553, 4554, 4555, 4556, 4557, 4558, 4559, 4560, 4561, 4563, 228, 4564, 239, 4565, 58,
    4567, 4568, 4569, 4570, 4571, 4572, 4573, 4574, 4575, 4576, 4578, 4579, 4580, 4581, 4586, 4587,
    4582, 4583, 4584, 4585, 58, 4588, 58, 4590, 4591, 4595, 4596, 4597, 4592, 4593, 4594, 4600,
    4601, 4602, 4606, 4603, 4604, 4605, 4607, 4608, 4610, 4611, 4616, 4612, 4615, 4613, 4614, 4618,
    4619, 4620, 4621, 4623, 4624, 4627, 4628, 4625, 4626, 4629, 58, 4630, 4643, 4631, 4632, 4633,
    4634, 4639, 4642, 4635, 4636, 4637, 4638, 4640, 4641, 4644, 4645, 4646, 4647, 4657, 4660, 4658,
    4659, 4661, 4662, 4663, 4664, 4670, 4671, 4675, 4676, 4677, 4680, 58, 4682, 4683, 4684, 4689,
    4690, 4784, 4785, 4786, 4787, 4788, 4789, 4790, 4791, 4792, 4793, 4794, 4795, 4796, 4797, 4798,
    4799, 4800, 4801, 4802, 4803, 4804, 4805, 4806, 4807, 4808, 4809, 4810, 4811, 4812, 4813, 4814,
    4815, 4816, 4817, 4818, 4819, 4820, 4821, 4822, 4823, 4824, 4825, 4826, 4827, 4828, 4829, 4830,
    4831, 4832, 4833, 4834, 4835, 4836, 4837, 4838, 4839, 4840, 4841, 4842, 4843, 4844, 4845, 4846,
    4847, 4848, 4849, 4850, 4851, 4852, 4853, 4854, 4855, 4856, 4857, 4858, 4859, 4860, 4861, 4862,
    4863, 4864, 4886, 4890, 235, 236, 4891, 58, 4892, 4893, 4922, 4975, 4983, 4923, 4926, 4924,
    4925, 4927, 4936, 4941, 4966, 4928, 4929, 4932, 4933, 4934, 4935, 4930, 4931, 4937, 4938, 4939,
    4940, 4942, 4943, 4944, 4945, 4946, 4947, 4948, 4949, 4950, 4951, 4954, 4952, 4953, 4955, 4956,
    4957, 4960, 4963, 4958, 4959, 4961, 4962, 4964, 4965, 4967, 4968, 4970, 4969, 235, 236, 4971,
    4972, 4973, 4974, 4976, 4979, 4980, 4981, 4982, 4977, 4978, 4984, 4987, 4990, 4991, 4992, 4985,
    4986, 4988, 4989, 4995, 4996, 4997, 4998, 4999, 5001, 5000, 235, 236, 5002, 235, 236, 5005,
    58, 5006, 5010, 5014, 5018, 5026, 5030, 5038, 5042, 5046, 5050, 5057, 5061, 5065, 5071, 5075,
    5082, 5086, 5090, 5094, 5100, 5104, 5114, 5118, 5124, 5130, 5134, 5142, 5007, 5008, 5009, 5011,
    5012, 5013, 5015, 5016, 5017, 5019, 5020, 5021, 5022, 5023, 5024, 5025, 5027, 5028, 5029, 5031,
    5032, 5033, 5034, 5035, 5036, 5037, 5039, 5040, 5041, 5043, 5044, 5045, 5047, 5048, 5049, 5051,
    5054, 5055, 5056, 5052, 5053, 5058, 5059, 5060, 5062, 5063, 5064, 5066, 5067, 5068, 5069, 5070,
    5072, 5073, 5074, 5076, 5077, 5078, 5079, 5080, 5081, 5083, 5084, 5085, 5087, 5088, 5089, 5091,
    5092, 5093, 5095, 5096, 5097, 5098, 5099, 5101, 5102, 5103, 5105, 5106, 5107, 5108, 5109, 5110,
    5111, 5112, 5113, 5115, 5116, 5117, 5119, 5120, 5121, 5122, 5123, 5125, 5126, 5127, 5128, 5129,
    5131, 5132, 5133, 5135, 5136, 5137, 5138, 5139, 5140, 5141, 5143, 5144, 5145, 5146, 235, 236,
    5147, 58, 5149, 5150, 5151, 5152, 5163, 5166, 5164, 5168, 5170, 5169, 5177, 5178, 5181, 5182,
    5195, 5198, 5199, 5196, 5197, 5200, 5203, 5204, 5201, 5202, 5206, 5207, 5208, 5209, 5210, 5230,
    5233, 5241, 5244, 5251, 5254, 5231, 5232, 5234, 5237, 5235, 5236, 5238, 5239, 5240, 5242, 5243,
    5245, 5246, 5247, 5248, 5249, 5250, 5252, 5253, 5255, 5256, 5277, 5278, 5279, 5284, 5285, 5286,
    5290, 235, 236, 5291, 58, 5295, 5296, 5298, 5299, 5307, 5308, 5309, 5319, 5320, 5321, 5322,
    5325, 5326, 5335, 5336, 5337, 5338, 5339, 5340, 5341, 5342, 5343, 5344, 5345, 5358, 5359, 5360,
    5361, 5362, 5370, 5371, 5372, 228, 5373, 239, 5377, 5378, 5379, 5380, 5374, 58, 5375, 5376,
    5385, 5386, 5387, 5388, 5389, 5390, 58, 5394, 5395, 5418, 5421, 5419, 5420, 5422, 58, 5423,
    5424, 5425, 235, 236, 5426, 58, 5427, 5428, 5429, 5438, 5439, 5440, 5466, 5467, 5469, 5470,
    5471, 5472, 5473, 5474, 823, 5468, 239, 5478, 823, 5479, 239, 5480, 5481, 5481, 235, 236,
    5483, 5484, 5487, 5490, 5491, 5492, 5493, 5494, 5495, 5485, 5486, 5488, 5489, 5499, 5509, 5500,
    5503, 5501, 5502, 5504, 5505, 5506, 5507, 5508, 5510, 5511, 5512, 5517, 5519, 5527, 5520, 5521,
    5524, 5522, 5523, 5525, 5526, 5530, 5534, 5535, 5536, 5531, 5532, 5533, 5538, 5539, 5542, 5543,
    5540, 5541, 5545, 5546, 5547, 5548, 5551, 5552, 5549, 58, 5554, 5555, 5556, 5557, 5560, 5561,
    5558, 58, 5563, 5564, 5566, 5565, 5570, 5571, 5572, 5573,
];

pub static TERMINATORS: &[u32] = &[
    59, 60, 136, 707, 708, 826, 827, 829, 836, 837, 839, 841, 842, 843, 954, 1082,
    1873, 1879, 2224, 2260, 2374, 2384, 2505, 2508, 2517, 2520, 2582, 2586, 2727, 2749, 3060, 3793,
    3794, 4453, 4454, 4488, 4489, 4490, 4678, 4865, 5153, 5165, 5346, 5347, 5348, 5349, 5350, 5351,
    5352, 5353, 5354, 5355, 5356, 5550, 5559,
];

pub static STRINGS: &[&str] = &[
    "ABORT", // [0]
    "keyword", // [1]
    "KeywordSegment", // [2]
    "base", // [3]
    "raw", // [4]
    "word", // [5]
    "ABS", // [6]
    "ABSOLUTE", // [7]
    "ACCESS", // [8]
    "AccountKeywordSegment", // [9]
    "ACCOUNT", // [10]
    "ALL", // [11]
    "CATALOG", // [12]
    "DATABASE", // [13]
    "DOMAIN", // [14]
    "EXTERNAL", // [15]
    "FILE", // [16]
    "FOREIGN", // [17]
    "FUNCTION", // [18]
    "FUTURE", // [19]
    "INTEGRATION", // [20]
    "LANGUAGE", // [21]
    "MASKING", // [22]
    "MATERIALIZED", // [23]
    "PROCEDURE", // [24]
    "RESOURCE", // [25]
    "ROLE", // [26]
    "ROUTINE", // [27]
    "SCHEMA", // [28]
    "SEQUENCE", // [29]
    "STAGE", // [30]
    "STREAM", // [31]
    "TABLE", // [32]
    "TABLESPACE", // [33]
    "TASK", // [34]
    "TYPE", // [35]
    "VIEW", // [36]
    "WAREHOUSE", // [37]
    "ResourceKeywordSegment", // [38]
    "MonitorKeywordSegment", // [39]
    "MONITOR", // [40]
    "WarehouseKeywordSegment", // [41]
    "DatabaseKeywordSegment", // [42]
    "DomainKeywordSegment", // [43]
    "IntegrationKeywordSegment", // [44]
    "LanguageKeywordSegment", // [45]
    "SchemaKeywordSegment", // [46]
    "CatalogKeywordSegment", // [47]
    "RoleKeywordSegment", // [48]
    "TablespaceKeywordSegment", // [49]
    "TypeKeywordSegment", // [50]
    "ForeignKeywordSegment", // [51]
    "DATA", // [52]
    "SERVER", // [53]
    "ServerKeywordSegment", // [54]
    "DataKeywordSegment", // [55]
    "WrapperKeywordSegment", // [56]
    "WRAPPER", // [57]
    "AllKeywordSegment", // [58]
    "SchemasKeywordSegment", // [59]
    "SCHEMAS", // [60]
    "InKeywordSegment", // [61]
    "IN", // [62]
    "FutureKeywordSegment", // [63]
    "AccessSchemaObjectSegment", // [64]
    "access_schema_object", // [65]
    "AccessSchemaPluralObjectSegment", // [66]
    "FUNCTIONS", // [67]
    "PROCEDURES", // [68]
    "ROUTINES", // [69]
    "SEQUENCES", // [70]
    "STAGES", // [71]
    "STREAMS", // [72]
    "TABLES", // [73]
    "TASKS", // [74]
    "VIEWS", // [75]
    "access_schema_plural_object", // [76]
    "ObjectReferenceSegment", // [77]
    "object_reference", // [78]
    "FunctionNameSegment", // [79]
    "function_name", // [80]
    "FunctionParameterListGrammar", // [81]
    "(", // [82]
    "function_parameter_list", // [83]
    "WildcardIdentifierSegment", // [84]
    "wildcard_identifier", // [85]
    "CommaSegment", // [86]
    ",", // [87]
    "ToKeywordSegment", // [88]
    "TO", // [89]
    "FromKeywordSegment", // [90]
    "FROM", // [91]
    "LARGE", // [92]
    "LargeKeywordSegment", // [93]
    "ObjectKeywordSegment", // [94]
    "OBJECT", // [95]
    "NumericLiteralSegment", // [96]
    "numeric_literal", // [97]
    "APPLY", // [98]
    "CONNECT", // [99]
    "CREATE", // [100]
    "DELETE", // [101]
    "EXECUTE", // [102]
    "IMPORTED", // [103]
    "INSERT", // [104]
    "MANAGE", // [105]
    "MODIFY", // [106]
    "OPERATE", // [107]
    "OWNERSHIP", // [108]
    "READ", // [109]
    "REFERENCES", // [110]
    "REFERENCE_USAGE", // [111]
    "SELECT", // [112]
    "TEMP", // [113]
    "TEMPORARY", // [114]
    "TRIGGER", // [115]
    "TRUNCATE", // [116]
    "UPDATE", // [117]
    "USAGE", // [118]
    "USE", // [119]
    "USE_ANY_ROLE", // [120]
    "WRITE", // [121]
    "CreateKeywordSegment", // [122]
    "USER", // [123]
    "UserKeywordSegment", // [124]
    "ApplyKeywordSegment", // [125]
    "MaskingKeywordSegment", // [126]
    "PolicyKeywordSegment", // [127]
    "POLICY", // [128]
    "ExecuteKeywordSegment", // [129]
    "TaskKeywordSegment", // [130]
    "ManageKeywordSegment", // [131]
    "GrantsKeywordSegment", // [132]
    "GRANTS", // [133]
    "EXECUTION", // [134]
    "ExecutionKeywordSegment", // [135]
    "UsageKeywordSegment", // [136]
    "PIPE", // [137]
    "PipeKeywordSegment", // [138]
    "UseKeywordSegment", // [139]
    "ImportedKeywordSegment", // [140]
    "PrivilegesKeywordSegment", // [141]
    "PRIVILEGES", // [142]
    "ConnectKeywordSegment", // [143]
    "DeleteKeywordSegment", // [144]
    "InsertKeywordSegment", // [145]
    "ModifyKeywordSegment", // [146]
    "OperateKeywordSegment", // [147]
    "OwnershipKeywordSegment", // [148]
    "ReadKeywordSegment", // [149]
    "Reference_usageKeywordSegment", // [150]
    "ReferencesKeywordSegment", // [151]
    "SelectKeywordSegment", // [152]
    "TempKeywordSegment", // [153]
    "TemporaryKeywordSegment", // [154]
    "TriggerKeywordSegment", // [155]
    "TruncateKeywordSegment", // [156]
    "UpdateKeywordSegment", // [157]
    "Use_any_roleKeywordSegment", // [158]
    "WriteKeywordSegment", // [159]
    "BracketedColumnReferenceListGrammar", // [160]
    "AccessPermissionSegment", // [161]
    "access_permission", // [162]
    "OnKeywordSegment", // [163]
    "ON", // [164]
    "TableKeywordSegment", // [165]
    "ViewKeywordSegment", // [166]
    "StageKeywordSegment", // [167]
    "FunctionKeywordSegment", // [168]
    "ProcedureKeywordSegment", // [169]
    "RoutineKeywordSegment", // [170]
    "SequenceKeywordSegment", // [171]
    "StreamKeywordSegment", // [172]
    "MaterializedKeywordSegment", // [173]
    "ExternalKeywordSegment", // [174]
    "FileKeywordSegment", // [175]
    "FormatKeywordSegment", // [176]
    "FORMAT", // [177]
    "TablesKeywordSegment", // [178]
    "ViewsKeywordSegment", // [179]
    "StagesKeywordSegment", // [180]
    "FunctionsKeywordSegment", // [181]
    "ProceduresKeywordSegment", // [182]
    "RoutinesKeywordSegment", // [183]
    "SequencesKeywordSegment", // [184]
    "StreamsKeywordSegment", // [185]
    "TasksKeywordSegment", // [186]
    "PoliciesKeywordSegment", // [187]
    "FormatsKeywordSegment", // [188]
    "GRANT", // [189]
    "REVOKE", // [190]
    "GrantStatementSegment", // [191]
    "grant_statement", // [192]
    "RevokeStatementSegment", // [193]
    "revoke_statement", // [194]
    "GROUP", // [195]
    "GroupKeywordSegment", // [196]
    "UserReferenceSegment", // [197]
    "user_reference", // [198]
    "RoleReferenceSegment", // [199]
    "role_reference", // [200]
    "SHARE", // [201]
    "ShareKeywordSegment", // [202]
    "FunctionSegment", // [203]
    "function", // [204]
    "PublicKeywordSegment", // [205]
    "PUBLIC", // [206]
    "[", // [207]
    "ArrayAccessorSegment", // [208]
    "array_accessor", // [209]
    "ACCOUNT_LOCK", // [210]
    "ACCOUNT_UNLOCK", // [211]
    "ACCOUNTS", // [212]
    "ACTION", // [213]
    "ACTIONS", // [214]
    "ACTIVE", // [215]
    "ADA", // [216]
    "ADD", // [217]
    "ADMIN", // [218]
    "AFTER", // [219]
    "AGAINST", // [220]
    "AGG_STATE", // [221]
    "AGGREGATE", // [222]
    "OrderByClauseSegment", // [223]
    "ORDER", // [224]
    "orderby_clause", // [225]
    "ALGORITHM", // [226]
    "indent", // [227]
    "Indent", // [228]
    "meta", // [229]
    "AsAliasOperatorSegment", // [230]
    "AS", // [231]
    "alias_operator", // [232]
    "SingleIdentifierGrammar", // [233]
    "SingleIdentifierListSegment", // [234]
    "identifier_list", // [235]
    "start_bracket", // [236]
    "SymbolSegment", // [237]
    "symbol", // [238]
    ")", // [239]
    "end_bracket", // [240]
    "SingleQuotedIdentifierSegment", // [241]
    "single_quote", // [242]
    "DoubleQuotedIdentifierSegment", // [243]
    "double_quote", // [244]
    "dedent", // [245]
    "Dedent", // [246]
    "ALIAS", // [247]
    "TableReferenceSegment", // [248]
    "table_reference", // [249]
    "AliasExpressionSegment", // [250]
    "alias_expression", // [251]
    "ALLOCATE", // [252]
    "ALSO", // [253]
    "ALTER", // [254]
    "AlterKeywordSegment", // [255]
    "DatabaseReferenceSegment", // [256]
    "database_reference", // [257]
    "CHARACTER", // [258]
    "CHARSET", // [259]
    "COLLATE", // [260]
    "DEFAULT", // [261]
    "ENCRYPTION", // [262]
    "AlterOptionSegment", // [263]
    "alter_option_segment", // [264]
    "DefinerSegment", // [265]
    "DEFINER", // [266]
    "definer_segment", // [267]
    "EventKeywordSegment", // [268]
    "EVENT", // [269]
    "ScheduleKeywordSegment", // [270]
    "SCHEDULE", // [271]
    "AT", // [272]
    "EVERY", // [273]
    "AtKeywordSegment", // [274]
    "EveryKeywordSegment", // [275]
    "ExpressionSegment", // [276]
    "expression", // [277]
    "DAY", // [278]
    "DAY_HOUR", // [279]
    "DAY_MICROSECOND", // [280]
    "DAY_MINUTE", // [281]
    "DAY_SECOND", // [282]
    "HOUR", // [283]
    "HOUR_MICROSECOND", // [284]
    "HOUR_MINUTE", // [285]
    "HOUR_SECOND", // [286]
    "MICROSECOND", // [287]
    "MINUTE", // [288]
    "MINUTE_MICROSECOND", // [289]
    "MINUTE_SECOND", // [290]
    "MONTH", // [291]
    "QUARTER", // [292]
    "SECOND", // [293]
    "SECOND_MICROSECOND", // [294]
    "WEEK", // [295]
    "YEAR", // [296]
    "YEAR_MONTH", // [297]
    "DatetimeUnitSegment", // [298]
    "ENDS", // [299]
    "STARTS", // [300]
    "StartsKeywordSegment", // [301]
    "EndsKeywordSegment", // [302]
    "CompletionKeywordSegment", // [303]
    "COMPLETION", // [304]
    "NotKeywordSegment", // [305]
    "NOT", // [306]
    "PreserveKeywordSegment", // [307]
    "PRESERVE", // [308]
    "RENAME", // [309]
    "RenameKeywordSegment", // [310]
    "DISABLE", // [311]
    "ENABLE", // [312]
    "EnableKeywordSegment", // [313]
    "DisableKeywordSegment", // [314]
    "REPLICA", // [315]
    "SLAVE", // [316]
    "ReplicaKeywordSegment", // [317]
    "SlaveKeywordSegment", // [318]
    "CommentClauseSegment", // [319]
    "COMMENT", // [320]
    "comment_clause", // [321]
    "DO", // [322]
    "DoKeywordSegment", // [323]
    "StatementSegment", // [324]
    "statement", // [325]
    "DefaultKeywordSegment", // [326]
    "CharsetGrammar", // [327]
    "EqualsSegment", // [328]
    "=", // [329]
    "comparison_operator", // [330]
    "CollateKeywordSegment", // [331]
    "CollationReferenceSegment", // [332]
    "collation_reference", // [333]
    "EncryptionKeywordSegment", // [334]
    "QuotedLiteralSegment", // [335]
    "OnlyKeywordSegment", // [336]
    "ONLY", // [337]
    "CACHE", // [338]
    "CYCLE", // [339]
    "INCREMENT", // [340]
    "MAXVALUE", // [341]
    "MINVALUE", // [342]
    "NO", // [343]
    "NOCACHE", // [344]
    "NOCYCLE", // [345]
    "NOORDER", // [346]
    "IncrementKeywordSegment", // [347]
    "ByKeywordSegment", // [348]
    "BY", // [349]
    "SequenceMinValueGrammar", // [350]
    "SequenceMaxValueGrammar", // [351]
    "CacheKeywordSegment", // [352]
    "NocacheKeywordSegment", // [353]
    "CycleKeywordSegment", // [354]
    "NocycleKeywordSegment", // [355]
    "OrderNoOrderGrammar", // [356]
    "SequenceReferenceSegment", // [357]
    "sequence_reference", // [358]
    "AlterSequenceOptionsSegment", // [359]
    "alter_sequence_options_segment", // [360]
    "DROP", // [361]
    "DropKeywordSegment", // [362]
    "ColumnKeywordSegment", // [363]
    "COLUMN", // [364]
    "IfExistsGrammar", // [365]
    "IF", // [366]
    "LOCK", // [367]
    "AlgorithmKeywordSegment", // [368]
    "COPY", // [369]
    "INPLACE", // [370]
    "INSTANT", // [371]
    "InplaceKeywordSegment", // [372]
    "CopyKeywordSegment", // [373]
    "InstantKeywordSegment", // [374]
    "LockKeywordSegment", // [375]
    "EXCLUSIVE", // [376]
    "NONE", // [377]
    "SHARED", // [378]
    "NoneKeywordSegment", // [379]
    "SharedKeywordSegment", // [380]
    "ExclusiveKeywordSegment", // [381]
    "ParameterNameSegment", // [382]
    "LiteralGrammar", // [383]
    "NakedIdentifierSegment", // [384]
    "AddKeywordSegment", // [385]
    "ColumnDefinitionSegment", // [386]
    "column_definition", // [387]
    "FIRST", // [388]
    "FirstKeywordSegment", // [389]
    "AfterKeywordSegment", // [390]
    "ColumnReferenceSegment", // [391]
    "column_reference", // [392]
    "AlterTableDropColumnGrammar", // [393]
    "AsKeywordSegment", // [394]
    "TableOptionsSegment", // [395]
    "AUTOEXTEND_SIZE", // [396]
    "AUTO_INCREMENT", // [397]
    "AVG_ROW_LENGTH", // [398]
    "CHECKSUM", // [399]
    "COMPRESSION", // [400]
    "CONNECTION", // [401]
    "DELAY_KEY_WRITE", // [402]
    "ENGINE", // [403]
    "ENGINE_ATTRIBUTE", // [404]
    "INDEX", // [405]
    "INSERT_METHOD", // [406]
    "KEY_BLOCK_SIZE", // [407]
    "MAX_ROWS", // [408]
    "MIN_ROWS", // [409]
    "PACK_KEYS", // [410]
    "PASSWORD", // [411]
    "ROW_FORMAT", // [412]
    "SECONDARY_ENGINE_ATTRIBUTE", // [413]
    "STATS_AUTO_RECALC", // [414]
    "STATS_PERSISTENT", // [415]
    "STATS_SAMPLE_PAGES", // [416]
    "UNION", // [417]
    "table_options", // [418]
    "AlterTableOnlineDDLOptionSegment", // [419]
    "alter_table_online_ddl_option", // [420]
    "AddDropSystemVersioningGrammar", // [421]
    "IfNotExistsGrammar", // [422]
    "SET", // [423]
    "SetKeywordSegment", // [424]
    "INVISIBLE", // [425]
    "VISIBLE", // [426]
    "InvisibleKeywordSegment", // [427]
    "VisibleKeywordSegment", // [428]
    "TableConstraintSegment", // [429]
    "CHECK", // [430]
    "CONSTRAINT", // [431]
    "FULLTEXT", // [432]
    "KEY", // [433]
    "PRIMARY", // [434]
    "SPATIAL", // [435]
    "UNIQUE", // [436]
    "table_constraint", // [437]
    "CHANGE", // [438]
    "ChangeKeywordSegment", // [439]
    "IndexKeywordSegment", // [440]
    "KeyKeywordSegment", // [441]
    "IndexReferenceSegment", // [442]
    "index_reference", // [443]
    "PrimaryKeyGrammar", // [444]
    "ForeignKeyGrammar", // [445]
    "CheckKeywordSegment", // [446]
    "ConstraintKeywordSegment", // [447]
    "ENFORCED", // [448]
    "EnforcedKeywordSegment", // [449]
    "KeysKeywordSegment", // [450]
    "KEYS", // [451]
    "CONVERT", // [452]
    "ConvertKeywordSegment", // [453]
    "ANALYZE", // [454]
    "COALESCE", // [455]
    "DISCARD", // [456]
    "EXCHANGE", // [457]
    "IMPORT", // [458]
    "OPTIMIZE", // [459]
    "REBUILD", // [460]
    "REMOVE", // [461]
    "REORGANIZE", // [462]
    "REPAIR", // [463]
    "DiscardKeywordSegment", // [464]
    "ImportKeywordSegment", // [465]
    "CoalesceKeywordSegment", // [466]
    "ReorganizeKeywordSegment", // [467]
    "ExchangeKeywordSegment", // [468]
    "AnalyzeKeywordSegment", // [469]
    "OptimizeKeywordSegment", // [470]
    "RebuildKeywordSegment", // [471]
    "RepairKeywordSegment", // [472]
    "RemoveKeywordSegment", // [473]
    "PARTITION", // [474]
    "PARTITIONING", // [475]
    "PartitionKeywordSegment", // [476]
    "PartitioningKeywordSegment", // [477]
    "WITH", // [478]
    "WithKeywordSegment", // [479]
    "TableReference", // [480]
    "WITHOUT", // [481]
    "WithoutKeywordSegment", // [482]
    "ValidationKeywordSegment", // [483]
    "VALIDATION", // [484]
    "INTO", // [485]
    "IntoKeywordSegment", // [486]
    "MERGE", // [487]
    "TEMPTABLE", // [488]
    "UNDEFINED", // [489]
    "UndefinedKeywordSegment", // [490]
    "MergeKeywordSegment", // [491]
    "TemptableKeywordSegment", // [492]
    "SQL", // [493]
    "SqlKeywordSegment", // [494]
    "SecurityKeywordSegment", // [495]
    "SECURITY", // [496]
    "INVOKER", // [497]
    "DefinerKeywordSegment", // [498]
    "InvokerKeywordSegment", // [499]
    "VALUE", // [500]
    "VALUES", // [501]
    "SelectableGrammar", // [502]
    "WithCheckOptionSegment", // [503]
    "with_check_options", // [504]
    "ALWAYS", // [505]
    "&", // [506]
    "ampersand", // [507]
    "ANALYSE", // [508]
    "LOCAL", // [509]
    "NO_WRITE_TO_BINLOG", // [510]
    "No_write_to_binlogKeywordSegment", // [511]
    "LocalKeywordSegment", // [512]
    "HistogramKeywordSegment", // [513]
    "HISTOGRAM", // [514]
    "BucketsKeywordSegment", // [515]
    "BUCKETS", // [516]
    "ANALYZED", // [517]
    "ANALYZER", // [518]
    "AND", // [519]
    "&&", // [520]
    "binary_operator", // [521]
    "BinaryOperatorSegment", // [522]
    "ANTI", // [523]
    "ANY", // [524]
    "APPEND", // [525]
    "ARE", // [526]
    "%", // [527]
    "*", // [528]
    "+", // [529]
    "-", // [530]
    "/", // [531]
    "<", // [532]
    ">", // [533]
    "DIV", // [534]
    "MOD", // [535]
    "^", // [536]
    "|", // [537]
    "PlusSegment", // [538]
    "MinusSegment", // [539]
    "DivideSegment", // [540]
    "MultiplySegment", // [541]
    "ModuloSegment", // [542]
    "BitwiseAndSegment", // [543]
    "BitwiseOrSegment", // [544]
    "BitwiseXorSegment", // [545]
    "BitwiseLShiftSegment", // [546]
    "BitwiseRShiftSegment", // [547]
    "DivOperatorSegment", // [548]
    "ModOperatorSegment", // [549]
    "SliceSegment", // [550]
    ":", // [551]
    "start_square_bracket", // [552]
    "]", // [553]
    "end_square_bracket", // [554]
    "ARRAY", // [555]
    "BaseExpressionElementGrammar", // [556]
    "ASC", // [557]
    "ASCII", // [558]
    "ASENSITIVE", // [559]
    "ASSERTION", // [560]
    "ASSIGNMENT", // [561]
    "ASYMMETRIC", // [562]
    "at_sign_literal", // [563]
    "literal", // [564]
    "LiteralSegment", // [565]
    "ATOMIC", // [566]
    "ATTRIBUTE", // [567]
    "ATTRIBUTES", // [568]
    "AUDIT", // [569]
    "AUTHENTICATION", // [570]
    "AUTHORIZATION", // [571]
    "AUTHORS", // [572]
    "Auto_incrementKeywordSegment", // [573]
    "AUTO", // [574]
    "AVG", // [575]
    "BACKEND", // [576]
    "BACKENDS", // [577]
    "BACKUP", // [578]
    "BACKWARD", // [579]
    "CURRENT_DATE", // [580]
    "CURRENT_TIME", // [581]
    "CURRENT_TIMESTAMP", // [582]
    "LOCALTIME", // [583]
    "LOCALTIMESTAMP", // [584]
    "NOW", // [585]
    "bare_function", // [586]
    "CodeSegment", // [587]
    "BareFunctionSegment", // [588]
    "IntervalExpressionSegment", // [589]
    "INTERVAL", // [590]
    "interval_expression", // [591]
    "DatatypeSegment", // [592]
    "data_type", // [593]
    "SessionVariableNameSegment", // [594]
    "LocalVariableNameSegment", // [595]
    "VariableAssignmentSegment", // [596]
    "file", // [597]
    "BaseFileSegment", // [598]
    "BaseSegment", // [599]
    "BEFORE", // [600]
    "BEGIN", // [601]
    "BELONG", // [602]
    "BERNOULLI", // [603]
    "BETWEEN", // [604]
    "BIGINT", // [605]
    "BIN", // [606]
    "BINARY", // [607]
    "!", // [608]
    "->", // [609]
    "->>", // [610]
    "IS", // [611]
    "OR", // [612]
    "XOR", // [613]
    "||", // [614]
    "like_operator", // [615]
    "ArithmeticBinaryOperatorGrammar", // [616]
    "StringBinaryOperatorGrammar", // [617]
    "BooleanBinaryOperatorGrammar", // [618]
    "ComparisonOperatorGrammar", // [619]
    "ColumnPathOperatorSegment", // [620]
    "InlinePathOperatorSegment", // [621]
    "BINDING", // [622]
    "BINLOG", // [623]
    "BIT", // [624]
    "BIT_LENGTH", // [625]
    "BITAND", // [626]
    "BITMAP", // [627]
    "BITMAP_EMPTY", // [628]
    "BITMAP_UNION", // [629]
    "BITOR", // [630]
    "BITVAR", // [631]
    "AmpersandSegment", // [632]
    "RawLessThanSegment", // [633]
    "PipeSegment", // [634]
    "RawGreaterThanSegment", // [635]
    "BITXOR", // [636]
    "BLOB", // [637]
    "BLOCK", // [638]
    "BLOOM_FILTER", // [639]
    "BOOL", // [640]
    "AndOperatorGrammar", // [641]
    "OrOperatorGrammar", // [642]
    "FALSE", // [643]
    "OFF", // [644]
    "TRUE", // [645]
    "OffKeywordSegment", // [646]
    "TrueKeywordSegment", // [647]
    "FalseKeywordSegment", // [648]
    "BOOLEAN", // [649]
    "TrueSegment", // [650]
    "FalseSegment", // [651]
    "BOTH", // [652]
    "IndexColumnPrefixLengthSegment", // [653]
    "bracketed_arguments", // [654]
    "DESC", // [655]
    "AscKeywordSegment", // [656]
    "DescKeywordSegment", // [657]
    "bracketed", // [658]
    "BracketedSegment", // [659]
    "UnorderedSetExpressionSegment", // [660]
    "set_expression", // [661]
    "BRANCH", // [662]
    "BREADTH", // [663]
    "BREAK", // [664]
    "BRIEF", // [665]
    "BROKER", // [666]
    "BROWSE", // [667]
    "BTREE", // [668]
    "BUILD", // [669]
    "BUILTIN", // [670]
    "BULK", // [671]
    "BYTE", // [672]
    "CTEColumnList", // [673]
    "cte_column_list", // [674]
    "CACHED", // [675]
    "CALL", // [676]
    "CallKeywordSegment", // [677]
    "CALLED", // [678]
    "CANCEL", // [679]
    "CARDINALITY", // [680]
    "CASCADE", // [681]
    "CASCADED", // [682]
    "CASE", // [683]
    "CaseKeywordSegment", // [684]
    "implicit_indent", // [685]
    "ImplicitIndent", // [686]
    "WHEN", // [687]
    "WhenClauseSegment", // [688]
    "when_clause", // [689]
    "ElseKeywordSegment", // [690]
    "ELSE", // [691]
    "EndKeywordSegment", // [692]
    "END", // [693]
    "ElseClauseSegment", // [694]
    "else_clause", // [695]
    "BinaryOperatorGrammar", // [696]
    "CAST", // [697]
    "::", // [698]
    "casting_operator", // [699]
    "CATALOG_NAME", // [700]
    "CATALOGS", // [701]
    "CEIL", // [702]
    "CEILING", // [703]
    "CHAIN", // [704]
    "CHALLENGE_RESPONSE", // [705]
    "CHANGED", // [706]
    "CHANNEL", // [707]
    "BinaryKeywordSegment", // [708]
    "CHAR", // [709]
    "CHAR_LENGTH", // [710]
    "CHARACTER_LENGTH", // [711]
    "CHARACTER_SET_CATALOG", // [712]
    "CHARACTER_SET_NAME", // [713]
    "CHARACTER_SET_SCHEMA", // [714]
    "DETERMINISTIC", // [715]
    "DeterministicKeywordSegment", // [716]
    "CONTAINS", // [717]
    "MODIFIES", // [718]
    "READS", // [719]
    "ContainsKeywordSegment", // [720]
    "NoKeywordSegment", // [721]
    "ReadsKeywordSegment", // [722]
    "ModifiesKeywordSegment", // [723]
    "CHARACTERISTICS", // [724]
    "CHARACTERS", // [725]
    "CharacterKeywordSegment", // [726]
    "CharsetKeywordSegment", // [727]
    "EXTENDED", // [728]
    "FAST", // [729]
    "FOR", // [730]
    "MEDIUM", // [731]
    "QUICK", // [732]
    "ForKeywordSegment", // [733]
    "UpgradeKeywordSegment", // [734]
    "UPGRADE", // [735]
    "QuickKeywordSegment", // [736]
    "FastKeywordSegment", // [737]
    "MediumKeywordSegment", // [738]
    "ExtendedKeywordSegment", // [739]
    "ChangedKeywordSegment", // [740]
    "CHECKED", // [741]
    "CHECKPOINT", // [742]
    "ChecksumKeywordSegment", // [743]
    "CIPHER", // [744]
    "CLASS", // [745]
    "CLASS_ORIGIN", // [746]
    "CLEAN", // [747]
    "CLIENT", // [748]
    "CLOB", // [749]
    "CLONE", // [750]
    "CLOSE", // [751]
    "CLUSTER", // [752]
    "CLUSTERED", // [753]
    "CLUSTERS", // [754]
    "COBOL", // [755]
    "CODE", // [756]
    "COLLATION", // [757]
    "ObjectReferenceDelimiterGrammar", // [758]
    ".", // [759]
    "ObjectReferenceTerminatorGrammar", // [760]
    "COLLATION_CATALOG", // [761]
    "COLLATION_NAME", // [762]
    "COLLATION_SCHEMA", // [763]
    "COLLECT", // [764]
    "COLOCATE", // [765]
    "colon_delimiter", // [766]
    "colon_prefix", // [767]
    "colon", // [768]
    "ShorthandCastSegment", // [769]
    "cast_expression", // [770]
    "NULL", // [771]
    "NullKeywordSegment", // [772]
    "ColumnConstraintDefaultGrammar", // [773]
    "NotEnforcedGrammar", // [774]
    "UniqueKeyGrammar", // [775]
    "AutoIncrementGrammar", // [776]
    "ReferenceDefinitionGrammar", // [777]
    "ColumnGeneratedGrammar", // [778]
    "CollateGrammar", // [779]
    "GENERATED", // [780]
    "GeneratedKeywordSegment", // [781]
    "AlwaysKeywordSegment", // [782]
    "STORED", // [783]
    "VIRTUAL", // [784]
    "StoredKeywordSegment", // [785]
    "VirtualKeywordSegment", // [786]
    "SRID", // [787]
    "SridKeywordSegment", // [788]
    "DATETIME", // [789]
    "TIMESTAMP", // [790]
    "DatetimeKeywordSegment", // [791]
    "TimestampKeywordSegment", // [792]
    "BracketedArguments", // [793]
    "CurrentTimestampLikeFunctionSegment", // [794]
    "ColumnConstraintSegment", // [795]
    "column_constraint_segment", // [796]
    "HLL_UNION", // [797]
    "MAX", // [798]
    "MIN", // [799]
    "QUANTILE_UNION", // [800]
    "REPLACE", // [801]
    "SUM", // [802]
    "MaxKeywordSegment", // [803]
    "MinKeywordSegment", // [804]
    "ReplaceKeywordSegment", // [805]
    "SumKeywordSegment", // [806]
    "Bitmap_unionKeywordSegment", // [807]
    "Hll_unionKeywordSegment", // [808]
    "Quantile_unionKeywordSegment", // [809]
    "column_path_operator", // [810]
    "COLUMN_FORMAT", // [811]
    "COLUMN_NAME", // [812]
    "ColumnsExpressionNameGrammar", // [813]
    "COLUMNS", // [814]
    "comma", // [815]
    "COMMAND_FUNCTION", // [816]
    "COMMAND_FUNCTION_CODE", // [817]
    "CommentKeywordSegment", // [818]
    "comment", // [819]
    "CommentSegment", // [820]
    "COMMENT_START", // [821]
    "COMMIT", // [822]
    "COMMITTED", // [823]
    "COMPACT", // [824]
    "GreaterThanSegment", // [825]
    "LessThanSegment", // [826]
    "GreaterThanOrEqualToSegment", // [827]
    "LessThanOrEqualToSegment", // [828]
    "NotEqualToSegment", // [829]
    "LikeOperatorSegment", // [830]
    "IsDistinctFromGrammar", // [831]
    "NullSafeEqualsSegment", // [832]
    "ComparisonOperatorSegment", // [833]
    "COMPLETE", // [834]
    "COMPONENT", // [835]
    "CompositeBinaryOperatorSegment", // [836]
    "CompositeComparisonOperatorSegment", // [837]
    "COMPRESS", // [838]
    "COMPRESS_TYPE", // [839]
    "COMPRESSED", // [840]
    "COMPUTE", // [841]
    "CONCURRENT", // [842]
    "CONDITION", // [843]
    "CONDITION_NUMBER", // [844]
    "CrossKeywordSegment", // [845]
    "CROSS", // [846]
    "JoinTypeKeywordsGrammar", // [847]
    "FULL", // [848]
    "INNER", // [849]
    "LEFT", // [850]
    "RIGHT", // [851]
    "ConditionalCrossJoinKeywordsGrammar", // [852]
    "NonStandardJoinTypeKeywordsGrammar", // [853]
    "CONDITIONS", // [854]
    "CONFIG", // [855]
    "CONNECTION_ID", // [856]
    "CONNECTION_NAME", // [857]
    "CONSISTENT", // [858]
    "CONSTRAINT_CATALOG", // [859]
    "CONSTRAINT_NAME", // [860]
    "CONSTRAINT_SCHEMA", // [861]
    "CONSTRAINTS", // [862]
    "CONSTRUCTOR", // [863]
    "CONTAINSTABLE", // [864]
    "CONTEXT", // [865]
    "CONTINUE", // [866]
    "CONVERSION", // [867]
    "USING", // [868]
    "UsingKeywordSegment", // [869]
    "function_name_identifier", // [870]
    "CONVERT_LSC", // [871]
    "CORR", // [872]
    "CORRESPONDING", // [873]
    "COUNT", // [874]
    "COVAR_POP", // [875]
    "COVAR_SAMP", // [876]
    "CPU", // [877]
    "CastKeywordSegment", // [878]
    "SpecificKeywordSegment", // [879]
    "SPECIFIC", // [880]
    "INSTANCE", // [881]
    "METHOD", // [882]
    "STATIC", // [883]
    "InstanceKeywordSegment", // [884]
    "StaticKeywordSegment", // [885]
    "ConstructorKeywordSegment", // [886]
    "MethodKeywordSegment", // [887]
    "AssignmentKeywordSegment", // [888]
    "CreateOptionSegment", // [889]
    "create_option_segment", // [890]
    "RETURNS", // [891]
    "ReturnsKeywordSegment", // [892]
    "CharacteristicStatement", // [893]
    "characteristic_statement", // [894]
    "FunctionDefinitionGrammar", // [895]
    "function_definition", // [896]
    "UniqueKeywordSegment", // [897]
    "FulltextKeywordSegment", // [898]
    "SpatialKeywordSegment", // [899]
    "IndexTypeGrammar", // [900]
    "index_type", // [901]
    "BracketedKeyPartListGrammar", // [902]
    "IndexOptionsSegment", // [903]
    "index_option", // [904]
    "NOCOPY", // [905]
    "NocopyKeywordSegment", // [906]
    "OrReplaceGrammar", // [907]
    "ModelKeywordSegment", // [908]
    "MODEL", // [909]
    "OPTIONS", // [910]
    "OptionsKeywordSegment", // [911]
    "ProcedureParameterListGrammar", // [912]
    "procedure_parameter_list", // [913]
    "SchemaReferenceSegment", // [914]
    "schema_reference", // [915]
    "START", // [916]
    "StartKeywordSegment", // [917]
    "CreateSequenceOptionsSegment", // [918]
    "create_sequence_options_segment", // [919]
    "LIKE", // [920]
    "PROPERTIES", // [921]
    "IndexDefinitionSegment", // [922]
    "index_definition", // [923]
    "EngineKeywordSegment", // [924]
    "EngineTypeSegment", // [925]
    "ELASTICSEARCH", // [926]
    "HIVE", // [927]
    "HUDI", // [928]
    "ICEBERG", // [929]
    "JDBC", // [930]
    "MYSQL", // [931]
    "OLAP", // [932]
    "DUPLICATE", // [933]
    "DuplicateKeywordSegment", // [934]
    "AggregateKeywordSegment", // [935]
    "ClusterKeywordSegment", // [936]
    "PartitionSegment", // [937]
    "partition_segment", // [938]
    "DistributionSegment", // [939]
    "DISTRIBUTED", // [940]
    "distribution_segment", // [941]
    "RollupSegment", // [942]
    "ROLLUP", // [943]
    "rollup_segment", // [944]
    "PropertiesKeywordSegment", // [945]
    "LikeKeywordSegment", // [946]
    "TriggerOrReplaceGrammar", // [947]
    "TriggerReferenceSegment", // [948]
    "trigger_reference", // [949]
    "BeforeKeywordSegment", // [950]
    "EachKeywordSegment", // [951]
    "EACH", // [952]
    "RowKeywordSegment", // [953]
    "ROW", // [954]
    "FOLLOWS", // [955]
    "PRECEDES", // [956]
    "FollowsKeywordSegment", // [957]
    "PrecedesKeywordSegment", // [958]
    "BeginKeywordSegment", // [959]
    "IDENTIFIED", // [960]
    "IdentifiedKeywordSegment", // [961]
    "RANDOM", // [962]
    "RandomKeywordSegment", // [963]
    "PasswordKeywordSegment", // [964]
    "INITIAL", // [965]
    "InitialKeywordSegment", // [966]
    "AuthenticationKeywordSegment", // [967]
    "AndKeywordSegment", // [968]
    "REQUIRE", // [969]
    "RequireKeywordSegment", // [970]
    "ISSUER", // [971]
    "SSL", // [972]
    "SUBJECT", // [973]
    "X509", // [974]
    "SslKeywordSegment", // [975]
    "X509KeywordSegment", // [976]
    "CipherKeywordSegment", // [977]
    "IssuerKeywordSegment", // [978]
    "SubjectKeywordSegment", // [979]
    "MAX_CONNECTIONS_PER_HOUR", // [980]
    "MAX_QUERIES_PER_HOUR", // [981]
    "MAX_UPDATES_PER_HOUR", // [982]
    "MAX_USER_CONNECTIONS", // [983]
    "Max_queries_per_hourKeywordSegment", // [984]
    "Max_updates_per_hourKeywordSegment", // [985]
    "Max_connections_per_hourKeywordSegment", // [986]
    "Max_user_connectionsKeywordSegment", // [987]
    "FAILED_LOGIN_ATTEMPTS", // [988]
    "PASSWORD_LOCK_TIME", // [989]
    "ExpireKeywordSegment", // [990]
    "EXPIRE", // [991]
    "NEVER", // [992]
    "NeverKeywordSegment", // [993]
    "IntervalKeywordSegment", // [994]
    "DayKeywordSegment", // [995]
    "HistoryKeywordSegment", // [996]
    "HISTORY", // [997]
    "ReuseKeywordSegment", // [998]
    "REUSE", // [999]
    "CurrentKeywordSegment", // [1000]
    "CURRENT", // [1001]
    "OPTIONAL", // [1002]
    "OptionalKeywordSegment", // [1003]
    "Failed_login_attemptsKeywordSegment", // [1004]
    "Password_lock_timeKeywordSegment", // [1005]
    "UNBOUNDED", // [1006]
    "UnboundedKeywordSegment", // [1007]
    "UNLOCK", // [1008]
    "UnlockKeywordSegment", // [1009]
    "AttributeKeywordSegment", // [1010]
    "CREATEDB", // [1011]
    "CREATEROLE", // [1012]
    "CREATEUSER", // [1013]
    "CREATION", // [1014]
    "CRON", // [1015]
    "CSV", // [1016]
    "CUBE", // [1017]
    "CubeFunctionNameSegment", // [1018]
    "RollupFunctionNameSegment", // [1019]
    "GroupingExpressionList", // [1020]
    "grouping_expression_list", // [1021]
    "CUME_DIST", // [1022]
    "Current_timestampKeywordSegment", // [1023]
    "NowKeywordSegment", // [1024]
    "LocaltimeKeywordSegment", // [1025]
    "LocaltimestampKeywordSegment", // [1026]
    "CurrentTimestampLikeFunctionNameSegment", // [1027]
    "CurrentTimestampLikeFunctionContentsSegment", // [1028]
    "function_contents", // [1029]
    "CURRENT_CATALOG", // [1030]
    "CURRENT_DEFAULT_TRANSFORM_GROUP", // [1031]
    "CURRENT_PATH", // [1032]
    "CURRENT_ROLE", // [1033]
    "CURRENT_TRANSFORM_GROUP_FOR_TYPE", // [1034]
    "CURRENT_USER", // [1035]
    "FETCH", // [1036]
    "FetchKeywordSegment", // [1037]
    "NEXT", // [1038]
    "NextKeywordSegment", // [1039]
    "CURSOR", // [1040]
    "OPEN", // [1041]
    "CloseKeywordSegment", // [1042]
    "OpenKeywordSegment", // [1043]
    "QuotedIdentifierSegment", // [1044]
    "back_quote", // [1045]
    "CURSOR_NAME", // [1046]
    "DATABASES", // [1047]
    "DATAFILE", // [1048]
    "data_type_identifier", // [1049]
    "TimeWithTZGrammar", // [1050]
    "TIME", // [1051]
    "DOUBLE", // [1052]
    "DoubleKeywordSegment", // [1053]
    "PrecisionKeywordSegment", // [1054]
    "PRECISION", // [1055]
    "SIGNED", // [1056]
    "UNSIGNED", // [1057]
    "SignedKeywordSegment", // [1058]
    "UnsignedKeywordSegment", // [1059]
    "IntegerKeywordSegment", // [1060]
    "INTEGER", // [1061]
    "VARYING", // [1062]
    "VaryingKeywordSegment", // [1063]
    "DotSegment", // [1064]
    "DatatypeIdentifierSegment", // [1065]
    "ZEROFILL", // [1066]
    "CharCharacterSetGrammar", // [1067]
    "ZerofillKeywordSegment", // [1068]
    "ArrayTypeSegment", // [1069]
    "array_type", // [1070]
    "DATE", // [1071]
    "EXTRACT", // [1072]
    "TIMESTAMPADD", // [1073]
    "TIMESTAMPDIFF", // [1074]
    "DatePartFunctionName", // [1075]
    "FunctionContentsGrammar", // [1076]
    "DateKeywordSegment", // [1077]
    "TimeKeywordSegment", // [1078]
    "date_constructor_literal", // [1079]
    "date_part", // [1080]
    "DATETIME_INTERVAL_CODE", // [1081]
    "DATETIME_INTERVAL_PRECISION", // [1082]
    "DATETIMEV1", // [1083]
    "DATETIMEV2", // [1084]
    "DATEV1", // [1085]
    "DATEV2", // [1086]
    "DAYOFMONTH", // [1087]
    "DAYOFWEEK", // [1088]
    "DAYOFYEAR", // [1089]
    "DAYS", // [1090]
    "DBCC", // [1091]
    "DEALLOCATE", // [1092]
    "DeallocateKeywordSegment", // [1093]
    "PrepareKeywordSegment", // [1094]
    "PREPARE", // [1095]
    "DEC", // [1096]
    "DECIMAL", // [1097]
    "DECIMALV2", // [1098]
    "DECIMALV3", // [1099]
    "DECLARE", // [1100]
    "DeclareKeywordSegment", // [1101]
    "CursorKeywordSegment", // [1102]
    "EXIT", // [1103]
    "UNDO", // [1104]
    "ContinueKeywordSegment", // [1105]
    "ExitKeywordSegment", // [1106]
    "UndoKeywordSegment", // [1107]
    "HandlerKeywordSegment", // [1108]
    "HANDLER", // [1109]
    "SqlexceptionKeywordSegment", // [1110]
    "SQLEXCEPTION", // [1111]
    "SqlwarningKeywordSegment", // [1112]
    "SQLWARNING", // [1113]
    "FoundKeywordSegment", // [1114]
    "FOUND", // [1115]
    "SQLSTATE", // [1116]
    "SqlstateKeywordSegment", // [1117]
    "ValueKeywordSegment", // [1118]
    "ConditionKeywordSegment", // [1119]
    "DECOMMISSION", // [1120]
    "ValuesKeywordSegment", // [1121]
    "DEFAULT_AUTH", // [1122]
    "DEFAULTS", // [1123]
    "DEFERRABLE", // [1124]
    "DEFERRED", // [1125]
    "DEFINED", // [1126]
    "DEFINITION", // [1127]
    "DEGREE", // [1128]
    "DELAYED", // [1129]
    "Low_priorityKeywordSegment", // [1130]
    "LOW_PRIORITY", // [1131]
    "IgnoreKeywordSegment", // [1132]
    "IGNORE", // [1133]
    "DeleteTargetTableSegment", // [1134]
    "delete_target_table", // [1135]
    "DeleteUsingClauseSegment", // [1136]
    "using_clause", // [1137]
    "WhereClauseSegment", // [1138]
    "WHERE", // [1139]
    "where_clause", // [1140]
    "FromClauseSegment", // [1141]
    "from_clause", // [1142]
    "SelectPartitionClauseSegment", // [1143]
    "partition_clause", // [1144]
    "LimitClauseSegment", // [1145]
    "LIMIT", // [1146]
    "limit_clause", // [1147]
    "StarSegment", // [1148]
    "FromExpressionSegment", // [1149]
    "from_expression", // [1150]
    ";", // [1151]
    "~", // [1152]
    "SemicolonSegment", // [1153]
    "TildeSegment", // [1154]
    "DELIMITER", // [1155]
    "DelimiterKeywordSegment", // [1156]
    "DELIMITERS", // [1157]
    "DEMAND", // [1158]
    "DENSE_RANK", // [1159]
    "DENY", // [1160]
    "DEPTH", // [1161]
    "DEREF", // [1162]
    "DERIVED", // [1163]
    "DES_KEY_FILE", // [1164]
    "DESCRIBE", // [1165]
    "DescribeKeywordSegment", // [1166]
    "DESCRIPTION", // [1167]
    "DESCRIPTOR", // [1168]
    "DESTROY", // [1169]
    "DESTRUCTOR", // [1170]
    "DIAGNOSE", // [1171]
    "DIAGNOSIS", // [1172]
    "DIAGNOSTICS", // [1173]
    "DICTIONARIES", // [1174]
    "DICTIONARY", // [1175]
    "DIRECTORY", // [1176]
    "DISCONNECT", // [1177]
    "DISK", // [1178]
    "DISPATCH", // [1179]
    "DISTINCT", // [1180]
    "DISTINCTPC", // [1181]
    "DISTINCTPCSA", // [1182]
    "DISTINCTROW", // [1183]
    "DISTRIBUTION", // [1184]
    "DistributedKeywordSegment", // [1185]
    "HASH", // [1186]
    "HashKeywordSegment", // [1187]
    "AutoKeywordSegment", // [1188]
    "DORIS_INTERNAL_TABLE_ID", // [1189]
    "dot", // [1190]
    "quoted_identifier", // [1191]
    "identifier", // [1192]
    "IdentifierSegment", // [1193]
    "json_path", // [1194]
    "\"", // [1195]
    "quoted_literal", // [1196]
    "RESTRICT", // [1197]
    "RestrictKeywordSegment", // [1198]
    "CascadeKeywordSegment", // [1199]
    "DropBehaviorGrammar", // [1200]
    "FORCE", // [1201]
    "ForceKeywordSegment", // [1202]
    "DROPP", // [1203]
    "DUAL", // [1204]
    "naked_identifier", // [1205]
    "DUMMY", // [1206]
    "DUMP", // [1207]
    "DUMPFILE", // [1208]
    "DYNAMIC", // [1209]
    "DYNAMIC_FUNCTION", // [1210]
    "DYNAMIC_FUNCTION_CODE", // [1211]
    "E", // [1212]
    "ELEMENT", // [1213]
    "ELSEIF", // [1214]
    "EMPTY", // [1215]
    "StructTypeSegment", // [1216]
    "struct_type", // [1217]
    "EmptyStructLiteralBracketsSegment", // [1218]
    "struct_literal", // [1219]
    "ENCLOSED", // [1220]
    "ENCODING", // [1221]
    "ENCRYPTED", // [1222]
    "ENCRYPTKEY", // [1223]
    "ENCRYPTKEYS", // [1224]
    "END-EXEC", // [1225]
    "}", // [1226]
    "end_curly_bracket", // [1227]
    "engine_type", // [1228]
    "ENGINES", // [1229]
    "ENTER", // [1230]
    "ENUM", // [1231]
    "EQUALS", // [1232]
    "RawEqualsSegment", // [1233]
    "ERRLVL", // [1234]
    "ERROR", // [1235]
    "ERRORS", // [1236]
    "ESCAPE", // [1237]
    "ESCAPED", // [1238]
    "EVENTS", // [1239]
    "EXCEPT", // [1240]
    "EXCEPTION", // [1241]
    "EXCLUDE", // [1242]
    "EXCLUDING", // [1243]
    "EXEC", // [1244]
    "EXISTING", // [1245]
    "EXISTS", // [1246]
    "EXP", // [1247]
    "EXPANSION", // [1248]
    "EXPIRED", // [1249]
    "EXPLAIN", // [1250]
    "ExplainKeywordSegment", // [1251]
    "InsertStatementSegment", // [1252]
    "insert_statement", // [1253]
    "UpdateStatementSegment", // [1254]
    "update_statement", // [1255]
    "DeleteStatementSegment", // [1256]
    "delete_statement", // [1257]
    "EXPORT", // [1258]
    "Expression_A_Grammar", // [1259]
    "Tail_Recurse_Expression_A_Grammar", // [1260]
    "LikeExpressionGrammar", // [1261]
    "REGEXP", // [1262]
    "RLIKE", // [1263]
    "InOperatorGrammar", // [1264]
    "QuantifiedComparisonOperatorGrammar", // [1265]
    "IsKeywordSegment", // [1266]
    "IsClauseGrammar", // [1267]
    "IsNullGrammar", // [1268]
    "NotNullGrammar", // [1269]
    "BetweenKeywordSegment", // [1270]
    "Expression_B_Grammar", // [1271]
    "PatternMatchingGrammar", // [1272]
    "PRIOR", // [1273]
    "SignedSegmentGrammar", // [1274]
    "QualifiedNumericLiteralSegment", // [1275]
    "NotOperatorGrammar", // [1276]
    "PriorKeywordSegment", // [1277]
    "Tail_Recurse_Expression_B_Grammar", // [1278]
    "WalrusOperatorSegment", // [1279]
    ":=", // [1280]
    "ExistsKeywordSegment", // [1281]
    "Expression_D_Grammar", // [1282]
    "CaseExpressionSegment", // [1283]
    "case_expression", // [1284]
    "TimeZoneGrammar", // [1285]
    "time_zone_grammar", // [1286]
    "LateralColumnAliasExpressionGrammar", // [1287]
    "LocalAliasSegment", // [1288]
    "local_alias_segment", // [1289]
    "Expression_D_Potential_Select_Statement_Without_Brackets", // [1290]
    "SingleIdentifierWildcardSegment", // [1291]
    "MapTypeSegment", // [1292]
    "map_type", // [1293]
    "BooleanLiteralGrammar", // [1294]
    "NullLiteralSegment", // [1295]
    "DateTimeLiteralGrammar", // [1296]
    "ListComprehensionGrammar", // [1297]
    "AccessorGrammar", // [1298]
    "ValuesClauseSegment", // [1299]
    "values_clause", // [1300]
    "SelectStatementSegment", // [1301]
    "select_statement", // [1302]
    "TypedStructLiteralSegment", // [1303]
    "typed_struct_literal", // [1304]
    "ArrayExpressionSegment", // [1305]
    "array_expression", // [1306]
    "OverlapsClauseSegment", // [1307]
    "OVERLAPS", // [1308]
    "overlaps_clause", // [1309]
    "EXTENSION", // [1310]
    "EXTENT_SIZE", // [1311]
    "FACTOR", // [1312]
    "boolean_literal", // [1313]
    "LiteralKeywordSegment", // [1314]
    "FAULTS", // [1315]
    "FEATURE", // [1316]
    "ROWS", // [1317]
    "RowsKeywordSegment", // [1318]
    "TiesKeywordSegment", // [1319]
    "TIES", // [1320]
    "FIELDS", // [1321]
    "DelimiterGrammar", // [1322]
    "FILE_BLOCK_SIZE", // [1323]
    "FILLFACTOR", // [1324]
    "FILTER", // [1325]
    "FilterKeywordSegment", // [1326]
    "WhereKeywordSegment", // [1327]
    "FINAL", // [1328]
    "FINISH", // [1329]
    "FIRST_VALUE", // [1330]
    "FIXED", // [1331]
    "FLOAT4", // [1332]
    "FLOAT8", // [1333]
    "FLOAT", // [1334]
    "FLOOR", // [1335]
    "FLUSH", // [1336]
    "FlushKeywordSegment", // [1337]
    "LogsKeywordSegment", // [1338]
    "LOGS", // [1339]
    "ErrorKeywordSegment", // [1340]
    "GENERAL", // [1341]
    "GeneralKeywordSegment", // [1342]
    "HostsKeywordSegment", // [1343]
    "HOSTS", // [1344]
    "Optimizer_costsKeywordSegment", // [1345]
    "RELAY", // [1346]
    "RelayKeywordSegment", // [1347]
    "ChannelKeywordSegment", // [1348]
    "SLOW", // [1349]
    "SlowKeywordSegment", // [1350]
    "StatusKeywordSegment", // [1351]
    "STATUS", // [1352]
    "User_resourcesKeywordSegment", // [1353]
    "USER_RESOURCES", // [1354]
    "ExportKeywordSegment", // [1355]
    "FOLLOWER", // [1356]
    "FOLLOWING", // [1357]
    "OF", // [1358]
    "OfKeywordSegment", // [1359]
    "NOWAIT", // [1360]
    "SKIP", // [1361]
    "NowaitKeywordSegment", // [1362]
    "SkipKeywordSegment", // [1363]
    "LockedKeywordSegment", // [1364]
    "LOCKED", // [1365]
    "ModeKeywordSegment", // [1366]
    "MODE", // [1367]
    "FORTRAN", // [1368]
    "FORWARD", // [1369]
    "RANGE", // [1370]
    "FrameClauseUnitGrammar", // [1371]
    "PRECEDING", // [1372]
    "PrecedingKeywordSegment", // [1373]
    "FollowingKeywordSegment", // [1374]
    "RangeKeywordSegment", // [1375]
    "FREE", // [1376]
    "FREETEXT", // [1377]
    "FREETEXTTABLE", // [1378]
    "FREEZE", // [1379]
    "HAVING", // [1380]
    "INTERSECT", // [1381]
    "MINUS", // [1382]
    "OFFSET", // [1383]
    "QUALIFY", // [1384]
    "WINDOW", // [1385]
    "OrderKeywordSegment", // [1386]
    "HavingKeywordSegment", // [1387]
    "QualifyKeywordSegment", // [1388]
    "WindowKeywordSegment", // [1389]
    "SetOperatorSegment", // [1390]
    "set_operator", // [1391]
    "WithNoSchemaBindingClauseSegment", // [1392]
    "with_no_schema_binding_clause", // [1393]
    "WithDataClauseSegment", // [1394]
    "with_data_clause", // [1395]
    "OffsetKeywordSegment", // [1396]
    "ForClauseSegment", // [1397]
    "for_clause", // [1398]
    "IntoClauseSegment", // [1399]
    "into_clause", // [1400]
    "PreTableFunctionKeywordsGrammar", // [1401]
    "TableExpressionSegment", // [1402]
    "table_expression", // [1403]
    "TemporalQuerySegment", // [1404]
    "temporal_query", // [1405]
    "FromClauseTerminatorGrammar", // [1406]
    "SamplingExpressionSegment", // [1407]
    "TABLESAMPLE", // [1408]
    "sample_expression", // [1409]
    "JoinLikeClauseGrammar", // [1410]
    "JoinClauseSegment", // [1411]
    "join_clause", // [1412]
    "PostTableExpressionGrammar", // [1413]
    "MLTableExpressionSegment", // [1414]
    "ML", // [1415]
    "ml_table_expression", // [1416]
    "FromExpressionElementSegment", // [1417]
    "from_expression_element", // [1418]
    "indented_joins", // [1419]
    "FRONTEND", // [1420]
    "FRONTENDS", // [1421]
    "LEADING", // [1422]
    "TRAILING", // [1423]
    "TrimParametersGrammar", // [1424]
    "DistinctKeywordSegment", // [1425]
    "FunctionContentsExpressionGrammar", // [1426]
    "AggregateOrderByClause", // [1427]
    "aggregate_order_by", // [1428]
    "SEPARATOR", // [1429]
    "SeparatorKeywordSegment", // [1430]
    "IgnoreRespectNullsGrammar", // [1431]
    "RESPECT", // [1432]
    "IndexColumnDefinitionSegment", // [1433]
    "index_column_definition", // [1434]
    "EmptyStructLiteralSegment", // [1435]
    "TransactionStatementSegment", // [1436]
    "transaction_statement", // [1437]
    "WordSegment", // [1438]
    "FunctionNameIdentifierSegment", // [1439]
    "AnyKeywordSegment", // [1440]
    "FunctionParameterGrammar", // [1441]
    "JSON_TABLE", // [1442]
    "JsonTableFunctionNameSegment", // [1443]
    "JsonTableFunctionContentsSegment", // [1444]
    "JSON_VALUE", // [1445]
    "JsonValueFunctionNameSegment", // [1446]
    "JsonValueFunctionContentsSegment", // [1447]
    "ConvertFunctionNameSegment", // [1448]
    "ConvertFunctionContentsSegment", // [1449]
    "DatePartFunctionNameSegment", // [1450]
    "DateTimeFunctionContentsSegment", // [1451]
    "ColumnsExpressionGrammar", // [1452]
    "ColumnsExpressionFunctionNameSegment", // [1453]
    "FunctionContentsSegment", // [1454]
    "PostFunctionGrammar", // [1455]
    "FUSION", // [1456]
    "G", // [1457]
    "GENERIC", // [1458]
    "GEOMCOLLECTION", // [1459]
    "GEOMETRY", // [1460]
    "GEOMETRYCOLLECTION", // [1461]
    "GET", // [1462]
    "GetKeywordSegment", // [1463]
    "StackedKeywordSegment", // [1464]
    "STACKED", // [1465]
    "DiagnosticsKeywordSegment", // [1466]
    "NUMBER", // [1467]
    "ROW_COUNT", // [1468]
    "NumberKeywordSegment", // [1469]
    "Row_countKeywordSegment", // [1470]
    "MESSAGE_TEXT", // [1471]
    "MYSQL_ERRNO", // [1472]
    "RETURNED_SQLSTATE", // [1473]
    "SCHEMA_NAME", // [1474]
    "SUBCLASS_ORIGIN", // [1475]
    "TABLE_NAME", // [1476]
    "Class_originKeywordSegment", // [1477]
    "Subclass_originKeywordSegment", // [1478]
    "Returned_sqlstateKeywordSegment", // [1479]
    "Message_textKeywordSegment", // [1480]
    "Mysql_errnoKeywordSegment", // [1481]
    "Constraint_catalogKeywordSegment", // [1482]
    "Constraint_schemaKeywordSegment", // [1483]
    "Constraint_nameKeywordSegment", // [1484]
    "Catalog_nameKeywordSegment", // [1485]
    "Schema_nameKeywordSegment", // [1486]
    "Table_nameKeywordSegment", // [1487]
    "Column_nameKeywordSegment", // [1488]
    "Cursor_nameKeywordSegment", // [1489]
    "GET_FORMAT", // [1490]
    "GET_MASTER_PUBLIC_KEY", // [1491]
    "GET_SOURCE_PUBLIC_KEY", // [1492]
    "glob_operator", // [1493]
    "GLOBAL", // [1494]
    "GO", // [1495]
    "GOTO", // [1496]
    "GrantKeywordSegment", // [1497]
    "AccessPermissionsSegment", // [1498]
    "access_permissions", // [1499]
    "AccessObjectSegment", // [1500]
    "access_object", // [1501]
    "OptionKeywordSegment", // [1502]
    "OPTION", // [1503]
    "AdminKeywordSegment", // [1504]
    "GRANTED", // [1505]
    "GRAPH", // [1506]
    "GREATEST", // [1507]
    "GroupingSetsClauseSegment", // [1508]
    "GROUPING", // [1509]
    "grouping_sets_clause", // [1510]
    "CubeRollupClauseSegment", // [1511]
    "cube_rollup_clause", // [1512]
    "GroupByClauseTerminatorGrammar", // [1513]
    "RollupKeywordSegment", // [1514]
    "GROUP_REPLICATION", // [1515]
    "GroupingKeywordSegment", // [1516]
    "SetsKeywordSegment", // [1517]
    "SETS", // [1518]
    "GROUPS", // [1519]
    "GTID_ONLY", // [1520]
    "HASH_MAP", // [1521]
    "HDFS", // [1522]
    "HEADER", // [1523]
    "HEAP", // [1524]
    "HELP", // [1525]
    "HelpKeywordSegment", // [1526]
    "HIERARCHY", // [1527]
    "HIGH_PRIORITY", // [1528]
    "HINT_END", // [1529]
    "HINT_START", // [1530]
    "HLL", // [1531]
    "HOLD", // [1532]
    "HOLDLOCK", // [1533]
    "HOST", // [1534]
    "HOSTNAME", // [1535]
    "HOTSPOT", // [1536]
    "HOURS", // [1537]
    "HUB", // [1538]
    "IDENTITY", // [1539]
    "IDENTITY_INSERT", // [1540]
    "IDENTITYCOL", // [1541]
    "IfKeywordSegment", // [1542]
    "ThenKeywordSegment", // [1543]
    "THEN", // [1544]
    "ElseifKeywordSegment", // [1545]
    "RespectKeywordSegment", // [1546]
    "NullsKeywordSegment", // [1547]
    "NULLS", // [1548]
    "IGNORE_SERVER_IDS", // [1549]
    "ILIKE", // [1550]
    "IMMEDIATE", // [1551]
    "IMMUTABLE", // [1552]
    "IMPLEMENTATION", // [1553]
    "IMPLICIT", // [1554]
    "INACTIVE", // [1555]
    "INCLUDE", // [1556]
    "INCLUDING", // [1557]
    "INCREMENTAL", // [1558]
    "INVERTED", // [1559]
    "InvertedKeywordSegment", // [1560]
    "BitmapKeywordSegment", // [1561]
    "Bloom_filterKeywordSegment", // [1562]
    "JOIN", // [1563]
    "JoinKeywordSegment", // [1564]
    "JoinOnConditionSegment", // [1565]
    "join_on_condition", // [1566]
    "Key_block_sizeKeywordSegment", // [1567]
    "ParserKeywordSegment", // [1568]
    "PARSER", // [1569]
    "Engine_attributeKeywordSegment", // [1570]
    "Secondary_engine_attributeKeywordSegment", // [1571]
    "BtreeKeywordSegment", // [1572]
    "INDEXES", // [1573]
    "INDICATOR", // [1574]
    "INFILE", // [1575]
    "INFIX", // [1576]
    "INHERIT", // [1577]
    "INHERITS", // [1578]
    "INITIAL_SIZE", // [1579]
    "INITIALIZE", // [1580]
    "INITIALLY", // [1581]
    "INITIATE", // [1582]
    "INOUT", // [1583]
    "INPUT", // [1584]
    "parameter_direction", // [1585]
    "INSENSITIVE", // [1586]
    "LabelKeywordSegment", // [1587]
    "LABEL", // [1588]
    "INSERT_ID", // [1589]
    "INSTALL", // [1590]
    "INSTANTIABLE", // [1591]
    "INSTEAD", // [1592]
    "INT1", // [1593]
    "INT2", // [1594]
    "INT3", // [1595]
    "INT4", // [1596]
    "INT8", // [1597]
    "INT", // [1598]
    "INTEGRATIONS", // [1599]
    "INTERMEDIATE", // [1600]
    "INTERSECTION", // [1601]
    "DumpfileKeywordSegment", // [1602]
    "OUTFILE", // [1603]
    "OutfileKeywordSegment", // [1604]
    "FieldsKeywordSegment", // [1605]
    "ColumnsKeywordSegment", // [1606]
    "TERMINATED", // [1607]
    "TerminatedKeywordSegment", // [1608]
    "OPTIONALLY", // [1609]
    "OptionallyKeywordSegment", // [1610]
    "EnclosedKeywordSegment", // [1611]
    "EscapedKeywordSegment", // [1612]
    "LINES", // [1613]
    "LinesKeywordSegment", // [1614]
    "STARTING", // [1615]
    "StartingKeywordSegment", // [1616]
    "SelectClauseTerminatorGrammar", // [1617]
    "IO", // [1618]
    "IO_THREAD", // [1619]
    "IP_TRIE", // [1620]
    "IPC", // [1621]
    "IPV4", // [1622]
    "IPV6", // [1623]
    "NanLiteralSegment", // [1624]
    "NAN", // [1625]
    "UnknownLiteralSegment", // [1626]
    "NormalizedGrammar", // [1627]
    "IS_NOT_NULL_PRED", // [1628]
    "IS_NULL_PRED", // [1629]
    "ISAM", // [1630]
    "ISNULL", // [1631]
    "ISOLATION", // [1632]
    "ITERATE", // [1633]
    "IterateKeywordSegment", // [1634]
    "JOB", // [1635]
    "JOBS", // [1636]
    "ConditionalJoinKeywordsGrammar", // [1637]
    "JoinKeywordsGrammar", // [1638]
    "NestedJoinGrammar", // [1639]
    "indented_using_on", // [1640]
    "MatchConditionSegment", // [1641]
    "match_condition", // [1642]
    "JoinUsingConditionGrammar", // [1643]
    "UnconditionalJoinKeywordsGrammar", // [1644]
    "ExtendedNaturalJoinKeywordsGrammar", // [1645]
    "indented_on_contents", // [1646]
    "InnerKeywordSegment", // [1647]
    "FullKeywordSegment", // [1648]
    "LeftKeywordSegment", // [1649]
    "RightKeywordSegment", // [1650]
    "OuterKeywordSegment", // [1651]
    "OUTER", // [1652]
    "JSON", // [1653]
    "OrdinalityKeywordSegment", // [1654]
    "ORDINALITY", // [1655]
    "NESTED", // [1656]
    "NestedKeywordSegment", // [1657]
    "PathKeywordSegment", // [1658]
    "PATH", // [1659]
    "JsonTableColumnsClauseSegment", // [1660]
    "json_table_columns_clause", // [1661]
    "EmptyKeywordSegment", // [1662]
    "RETURNING", // [1663]
    "ReturningKeywordSegment", // [1664]
    "JSONB", // [1665]
    "K", // [1666]
    "KEY_MEMBER", // [1667]
    "KEY_TYPE", // [1668]
    "KEYRING", // [1669]
    "KILL", // [1670]
    "LAG", // [1671]
    "LANCOMPILER", // [1672]
    "LARGEINT", // [1673]
    "LAST", // [1674]
    "LAST_INSERT_ID", // [1675]
    "LAST_VALUE", // [1676]
    "LATERAL", // [1677]
    "LAYOUT", // [1678]
    "LDAP", // [1679]
    "LDAP_ADMIN_PASSWORD", // [1680]
    "LEAD", // [1681]
    "LEAST", // [1682]
    "LEAVE", // [1683]
    "LEAVES", // [1684]
    "LEFT_BRACE", // [1685]
    "LENGTH", // [1686]
    "LESS", // [1687]
    "LEVEL", // [1688]
    "LikeGrammar", // [1689]
    "EscapeKeywordSegment", // [1690]
    "RlikeKeywordSegment", // [1691]
    "RegexpKeywordSegment", // [1692]
    "LimitKeywordSegment", // [1693]
    "LINENO", // [1694]
    "LINESTRING", // [1695]
    "LINK", // [1696]
    "LIST", // [1697]
    "LISTEN", // [1698]
    "ArrayLiteralSegment", // [1699]
    "array_literal", // [1700]
    "TypedArrayLiteralSegment", // [1701]
    "typed_array_literal", // [1702]
    "ObjectLiteralSegment", // [1703]
    "{", // [1704]
    "object_literal", // [1705]
    "DoubleQuotedLiteralSegment", // [1706]
    "SystemVariableSegment", // [1707]
    "LN", // [1708]
    "LOAD", // [1709]
    "LoadKeywordSegment", // [1710]
    "ConcurrentKeywordSegment", // [1711]
    "InfileKeywordSegment", // [1712]
    "variable", // [1713]
    "LOCATION", // [1714]
    "LOCATOR", // [1715]
    "LOCKS", // [1716]
    "LOGFILE", // [1717]
    "LOGICAL", // [1718]
    "LOGIN", // [1719]
    "LONG", // [1720]
    "LONGBLOB", // [1721]
    "LONGTEXT", // [1722]
    "LOOP", // [1723]
    "ColonSegment", // [1724]
    "LoopKeywordSegment", // [1725]
    "LOWER", // [1726]
    "M", // [1727]
    "MlKeywordSegment", // [1728]
    "MANUAL", // [1729]
    "MAP", // [1730]
    "MASTER", // [1731]
    "MASTER_AUTO_POSITION", // [1732]
    "MASTER_COMPRESSION_ALGORITHMS", // [1733]
    "MASTER_CONNECT_RETRY", // [1734]
    "MASTER_DELAY", // [1735]
    "MASTER_HEARTBEAT_PERIOD", // [1736]
    "MASTER_HOST", // [1737]
    "MASTER_LOG_FILE", // [1738]
    "MASTER_LOG_POS", // [1739]
    "MASTER_PASSWORD", // [1740]
    "MASTER_PORT", // [1741]
    "MASTER_PUBLIC_KEY_PATH", // [1742]
    "MASTER_RETRY_COUNT", // [1743]
    "MASTER_SERVER_ID", // [1744]
    "MASTER_SSL", // [1745]
    "MASTER_SSL_CA", // [1746]
    "MASTER_SSL_CAPATH", // [1747]
    "MASTER_SSL_CERT", // [1748]
    "MASTER_SSL_CIPHER", // [1749]
    "MASTER_SSL_CRL", // [1750]
    "MASTER_SSL_CRLPATH", // [1751]
    "MASTER_SSL_KEY", // [1752]
    "MASTER_TLS_CIPHERSUITES", // [1753]
    "MASTER_TLS_VERSION", // [1754]
    "MASTER_USER", // [1755]
    "MASTER_ZSTD_COMPRESSION_LEVEL", // [1756]
    "MATCH", // [1757]
    "MATCH_ALL", // [1758]
    "MATCH_ANY", // [1759]
    "MATCH_PHRASE", // [1760]
    "MATCH_PHRASE_EDGE", // [1761]
    "MATCH_PHRASE_PREFIX", // [1762]
    "MATCH_REGEXP", // [1763]
    "MATCHED", // [1764]
    "MAX_SIZE", // [1765]
    "MAXEXTENTS", // [1766]
    "MEDIUMBLOB", // [1767]
    "MEDIUMINT", // [1768]
    "MEDIUMTEXT", // [1769]
    "MEMBER", // [1770]
    "MEMO", // [1771]
    "MEMORY", // [1772]
    "MergeMatchedClauseSegment", // [1773]
    "merge_when_matched_clause", // [1774]
    "MergeNotMatchedClauseSegment", // [1775]
    "merge_when_not_matched_clause", // [1776]
    "WhenKeywordSegment", // [1777]
    "MatchedKeywordSegment", // [1778]
    "MergeUpdateClauseSegment", // [1779]
    "merge_update_clause", // [1780]
    "MergeDeleteClauseSegment", // [1781]
    "merge_delete_clause", // [1782]
    "MergeInsertClauseSegment", // [1783]
    "merge_insert_clause", // [1784]
    "MergeIntoLiteralGrammar", // [1785]
    "AliasedTableReferenceGrammar", // [1786]
    "MergeMatchSegment", // [1787]
    "merge_match", // [1788]
    "SetClauseListSegment", // [1789]
    "set_clause_list", // [1790]
    "MESSAGE_LENGTH", // [1791]
    "MESSAGE_OCTET_LENGTH", // [1792]
    "MIDDLEINT", // [1793]
    "MIGRATE", // [1794]
    "MIGRATIONS", // [1795]
    "MILLISECOND", // [1796]
    "MINUTES", // [1797]
    "MLSLABEL", // [1798]
    "MODULE", // [1799]
    "MONTHNAME", // [1800]
    "MORE", // [1801]
    "MOVE", // [1802]
    "MTMV", // [1803]
    "MULTILINESTRING", // [1804]
    "MULTIPOINT", // [1805]
    "MULTIPOLYGON", // [1806]
    "MULTISET", // [1807]
    "MUMPS", // [1808]
    "MUTEX", // [1809]
    "MYISAM", // [1810]
    "NAME", // [1811]
    "WindowSpecificationSegment", // [1812]
    "window_specification", // [1813]
    "NamedWindowExpressionSegment", // [1814]
    "named_window_expression", // [1815]
    "NAMES", // [1816]
    "null_literal", // [1817]
    "NATIONAL", // [1818]
    "NATURAL", // [1819]
    "NaturalKeywordSegment", // [1820]
    "NCHAR", // [1821]
    "NCLOB", // [1822]
    "NDB", // [1823]
    "NDBCLUSTER", // [1824]
    "NEGATIVE", // [1825]
    "sign_indicator", // [1826]
    "NESTING", // [1827]
    "NETWORK_NAMESPACE", // [1828]
    "NEW", // [1829]
    "newline", // [1830]
    "NewlineSegment", // [1831]
    "NGRAM_BF", // [1832]
    "NO_USE_MV", // [1833]
    "NO_WAIT", // [1834]
    "NOAUDIT", // [1835]
    "NOCHECK", // [1836]
    "NOCOMPRESS", // [1837]
    "NOCREATEDB", // [1838]
    "NOCREATEROLE", // [1839]
    "NOCREATEUSER", // [1840]
    "NODEGROUP", // [1841]
    "NOINHERIT", // [1842]
    "NOLOGIN", // [1843]
    "UnorderedSelectStatementSegment", // [1844]
    "WithCompoundStatementSegment", // [1845]
    "with_compound_statement", // [1846]
    "NonSetSelectableGrammar", // [1847]
    "BracketedSetExpressionGrammar", // [1848]
    "MergeStatementSegment", // [1849]
    "merge_statement", // [1850]
    "SetExpressionSegment", // [1851]
    "NON_NULLABLE", // [1852]
    "NONCLUSTERED", // [1853]
    "NORMALIZE", // [1854]
    "NORMALIZED", // [1855]
    "NOSUPERUSER", // [1856]
    "RawNotSegment", // [1857]
    "not_operator", // [1858]
    "NOTHING", // [1859]
    "NOTIFY", // [1860]
    "NOTNULL", // [1861]
    "NTH_VALUE", // [1862]
    "NTILE", // [1863]
    "NULLABLE", // [1864]
    "NULLIF", // [1865]
    "NUMERIC", // [1866]
    "NVARCHAR", // [1867]
    "ObjectLiteralElementSegment", // [1868]
    "object_literal_element", // [1869]
    "start_curly_bracket", // [1870]
    "CastOperatorSegment", // [1871]
    "StartSquareBracketSegment", // [1872]
    "StartBracketSegment", // [1873]
    "OBJECTS", // [1874]
    "OBSERVER", // [1875]
    "OCTET_LENGTH", // [1876]
    "OCTETS", // [1877]
    "OFFLINE", // [1878]
    "OFFSETS", // [1879]
    "OIDS", // [1880]
    "OJ", // [1881]
    "OLD", // [1882]
    "ONE", // [1883]
    "ONLINE", // [1884]
    "OPENDATASOURCE", // [1885]
    "OPENQUERY", // [1886]
    "OPENROWSET", // [1887]
    "OPENXML", // [1888]
    "OPERATION", // [1889]
    "OPERATOR", // [1890]
    "OPTIMIZED", // [1891]
    "OrKeywordSegment", // [1892]
    "LastKeywordSegment", // [1893]
    "WithFillSegment", // [1894]
    "with_fill", // [1895]
    "NoorderKeywordSegment", // [1896]
    "ORDERING", // [1897]
    "ORGANIZATION", // [1898]
    "OTHERS", // [1899]
    "OUT", // [1900]
    "OUTPUT", // [1901]
    "OverKeywordSegment", // [1902]
    "OVER", // [1903]
    "OverlapsKeywordSegment", // [1904]
    "OVERLAY", // [1905]
    "OVERRIDING", // [1906]
    "OVERWRITE", // [1907]
    "OWNER", // [1908]
    "PAD", // [1909]
    "PAGE", // [1910]
    "PARAMETER", // [1911]
    "parameter", // [1912]
    "?", // [1913]
    "PARAMETER_MODE", // [1914]
    "PARAMETER_NAME", // [1915]
    "PARAMETER_ORDINAL_POSITION", // [1916]
    "PARAMETER_SPECIFIC_CATALOG", // [1917]
    "PARAMETER_SPECIFIC_NAME", // [1918]
    "PARAMETER_SPECIFIC_SCHEMA", // [1919]
    "PARAMETERS", // [1920]
    "PARSE_GCOL_EXPR", // [1921]
    "PARSED", // [1922]
    "PARTIAL", // [1923]
    "RangePartitionDefinitionSegment", // [1924]
    "range_partition_definition", // [1925]
    "RangePartitionIntervalSegment", // [1926]
    "range_partition_interval", // [1927]
    "ListKeywordSegment", // [1928]
    "ListPartitionDefinitionSegment", // [1929]
    "list_partition_definition", // [1930]
    "PARTITIONS", // [1931]
    "PASCAL", // [1932]
    "PASSWORD_EXPIRE", // [1933]
    "PASSWORD_HISTORY", // [1934]
    "PASSWORD_REUSE", // [1935]
    "SlashSegment", // [1936]
    "path_segment", // [1937]
    "PAUSE", // [1938]
    "PCTFREE", // [1939]
    "PERCENT", // [1940]
    "PERCENT_RANK", // [1941]
    "PERCENTILE_CONT", // [1942]
    "PERCENTILE_DISC", // [1943]
    "PERIOD", // [1944]
    "PERMISSIVE", // [1945]
    "PERSIST", // [1946]
    "PERSIST_ONLY", // [1947]
    "PHASE", // [1948]
    "PHYSICAL", // [1949]
    "PI", // [1950]
    "pipe", // [1951]
    "PLACEHOLDER", // [1952]
    "PLACING", // [1953]
    "PLAN", // [1954]
    "PLAY", // [1955]
    "PLI", // [1956]
    "PLUGIN", // [1957]
    "PLUGIN_DIR", // [1958]
    "PLUGINS", // [1959]
    "POINT", // [1960]
    "POLYGON", // [1961]
    "PORT", // [1962]
    "POSITION", // [1963]
    "OverClauseSegment", // [1964]
    "over_clause", // [1965]
    "FilterClauseGrammar", // [1966]
    "IndexHintClauseSegment", // [1967]
    "index_hint_clause", // [1968]
    "POSTFIX", // [1969]
    "POWER", // [1970]
    "PREFIX", // [1971]
    "PREORDER", // [1972]
    "PREPARED", // [1973]
    "PREV", // [1974]
    "PrimaryKeywordSegment", // [1975]
    "PRINT", // [1976]
    "PRIVILEGE_CHECKS_USER", // [1977]
    "PROC", // [1978]
    "PROCEDURAL", // [1979]
    "OutputParameterSegment", // [1980]
    "InputParameterSegment", // [1981]
    "InputOutputParameterSegment", // [1982]
    "ProcedureParameterGrammar", // [1983]
    "PROCESS", // [1984]
    "PROCESSLIST", // [1985]
    "PROFILE", // [1986]
    "PROFILES", // [1987]
    "PROPERTY", // [1988]
    "PROXY", // [1989]
    "PURGE", // [1990]
    "PurgeKeywordSegment", // [1991]
    "MasterKeywordSegment", // [1992]
    "SOME", // [1993]
    "SomeKeywordSegment", // [1994]
    "QUANTILE_STATE", // [1995]
    "QUERY", // [1996]
    "QUEUED", // [1997]
    "QUOTA", // [1998]
    "QUOTE", // [1999]
    "`", // [2000]
    "RAID0", // [2001]
    "RAISERROR", // [2002]
    "LessKeywordSegment", // [2003]
    "ThanKeywordSegment", // [2004]
    "THAN", // [2005]
    "MaxvalueKeywordSegment", // [2006]
    ",KeywordSegment", // [2007]
    "YearKeywordSegment", // [2008]
    "MonthKeywordSegment", // [2009]
    "HourKeywordSegment", // [2010]
    "MinuteKeywordSegment", // [2011]
    "SecondKeywordSegment", // [2012]
    "RANK", // [2013]
    "raw_comparison_operator", // [2014]
    "RAW", // [2015]
    "READ_ONLY", // [2016]
    "READTEXT", // [2017]
    "REAL", // [2018]
    "REBALANCE", // [2019]
    "RECENT", // [2020]
    "RECHECK", // [2021]
    "RECONFIGURE", // [2022]
    "RECOVER", // [2023]
    "RECURSIVE", // [2024]
    "RECYCLE", // [2025]
    "REDO_BUFFER_SIZE", // [2026]
    "REDOFILE", // [2027]
    "REDUNDANT", // [2028]
    "REF", // [2029]
    "ReferenceMatchGrammar", // [2030]
    "ReferentialActionGrammar", // [2031]
    "REFERENCE", // [2032]
    "MatchKeywordSegment", // [2033]
    "SIMPLE", // [2034]
    "PartialKeywordSegment", // [2035]
    "SimpleKeywordSegment", // [2036]
    "REFERENCING", // [2037]
    "ActionKeywordSegment", // [2038]
    "REFRESH", // [2039]
    "REGISTRATION", // [2040]
    "REGR_AVGX", // [2041]
    "REGR_AVGY", // [2042]
    "REGR_COUNT", // [2043]
    "REGR_INTERCEPT", // [2044]
    "REGR_R2", // [2045]
    "REGR_SLOPE", // [2046]
    "REGR_SXX", // [2047]
    "REGR_SXY", // [2048]
    "REGR_SYY", // [2049]
    "REINDEX", // [2050]
    "RELATIVE", // [2051]
    "RELAY_LOG_FILE", // [2052]
    "RELAY_LOG_POS", // [2053]
    "RELAY_THREAD", // [2054]
    "RELAYLOG", // [2055]
    "RELEASE", // [2056]
    "RELOAD", // [2057]
    "REMOTE", // [2058]
    "USE_FRM", // [2059]
    "Use_frmKeywordSegment", // [2060]
    "REPEAT", // [2061]
    "RepeatKeywordSegment", // [2062]
    "UNTIL", // [2063]
    "UntilKeywordSegment", // [2064]
    "REPEATABLE", // [2065]
    "DelayedKeywordSegment", // [2066]
    "REPLACE_IF_NOT_NULL", // [2067]
    "REPLAYER", // [2068]
    "REPLICAS", // [2069]
    "REPLICATE_DO_DB", // [2070]
    "REPLICATE_DO_TABLE", // [2071]
    "REPLICATE_IGNORE_DB", // [2072]
    "REPLICATE_IGNORE_TABLE", // [2073]
    "REPLICATE_REWRITE_DB", // [2074]
    "REPLICATE_WILD_DO_TABLE", // [2075]
    "REPLICATE_WILD_IGNORE_TABLE", // [2076]
    "REPLICATION", // [2077]
    "REPOSITORIES", // [2078]
    "REPOSITORY", // [2079]
    "REQUIRE_ROW_FORMAT", // [2080]
    "RESET", // [2081]
    "ResetKeywordSegment", // [2082]
    "RESIGNAL", // [2083]
    "SIGNAL", // [2084]
    "SignalKeywordSegment", // [2085]
    "ResignalKeywordSegment", // [2086]
    "RESOURCES", // [2087]
    "RESTART", // [2088]
    "RESTORE", // [2089]
    "RESTRICTIVE", // [2090]
    "RESULT", // [2091]
    "RESUME", // [2092]
    "RETAIN", // [2093]
    "RETENTION", // [2094]
    "RETURN", // [2095]
    "ReturnKeywordSegment", // [2096]
    "RETURNED_CARDINALITY", // [2097]
    "RETURNED_LENGTH", // [2098]
    "RETURNED_OCTET_LENGTH", // [2099]
    "REVERSE", // [2100]
    "RevokeKeywordSegment", // [2101]
    "AccessTargetSegment", // [2102]
    "access_target", // [2103]
    "REWRITTEN", // [2104]
    "RIGHT_BRACE", // [2105]
    "AtSignLiteralSegment", // [2106]
    "Current_userKeywordSegment", // [2107]
    "ROLES", // [2108]
    "ROLLBACK", // [2109]
    "ROTATE", // [2110]
    "ROUTINE_CATALOG", // [2111]
    "ROUTINE_NAME", // [2112]
    "ROUTINE_SCHEMA", // [2113]
    "ROW_NUMBER", // [2114]
    "ROWCOUNT", // [2115]
    "ROWGUIDCOL", // [2116]
    "ROWID", // [2117]
    "ROWNUM", // [2118]
    "RTREE", // [2119]
    "RULE", // [2120]
    "S3", // [2121]
    "SAMPLE", // [2122]
    "TablesampleKeywordSegment", // [2123]
    "SYSTEM", // [2124]
    "BernoulliKeywordSegment", // [2125]
    "SystemKeywordSegment", // [2126]
    "RepeatableKeywordSegment", // [2127]
    "SAVE", // [2128]
    "SAVEPOINT", // [2129]
    "SCALE", // [2130]
    "SCHEDULER", // [2131]
    "SCOPE", // [2132]
    "SCOPE_CATALOG", // [2133]
    "SCOPE_NAME", // [2134]
    "SCOPE_SCHEMA", // [2135]
    "SCROLL", // [2136]
    "SEARCH", // [2137]
    "SECONDARY", // [2138]
    "SECONDARY_ENGINE", // [2139]
    "SECONDARY_LOAD", // [2140]
    "SECONDARY_UNLOAD", // [2141]
    "SECTION", // [2142]
    "WildcardExpressionSegment", // [2143]
    "wildcard_expression", // [2144]
    "SQL_BIG_RESULT", // [2145]
    "SQL_BUFFER_RESULT", // [2146]
    "SQL_CACHE", // [2147]
    "SQL_CALC_FOUND_ROWS", // [2148]
    "SQL_NO_CACHE", // [2149]
    "SQL_SMALL_RESULT", // [2150]
    "STRAIGHT_JOIN", // [2151]
    "DistinctrowKeywordSegment", // [2152]
    "High_priorityKeywordSegment", // [2153]
    "Straight_joinKeywordSegment", // [2154]
    "Sql_small_resultKeywordSegment", // [2155]
    "Sql_big_resultKeywordSegment", // [2156]
    "Sql_buffer_resultKeywordSegment", // [2157]
    "Sql_cacheKeywordSegment", // [2158]
    "Sql_no_cacheKeywordSegment", // [2159]
    "Sql_calc_found_rowsKeywordSegment", // [2160]
    "SelectClauseModifierSegment", // [2161]
    "select_clause_modifier", // [2162]
    "SelectClauseElementSegment", // [2163]
    "select_clause_element", // [2164]
    "SelectClauseSegment", // [2165]
    "select_clause", // [2166]
    "GroupByClauseSegment", // [2167]
    "groupby_clause", // [2168]
    "HavingClauseSegment", // [2169]
    "having_clause", // [2170]
    "NamedWindowSegment", // [2171]
    "named_window", // [2172]
    "UpsertClauseListSegment", // [2173]
    "upsert_clause_list", // [2174]
    "WithCompoundNonSelectStatementSegment", // [2175]
    "NonWithSelectableGrammar", // [2176]
    "SELF", // [2177]
    "SEMI", // [2178]
    "statement_terminator", // [2179]
    "SENSITIVE", // [2180]
    "MinvalueKeywordSegment", // [2181]
    "SERIAL", // [2182]
    "SERIALIZABLE", // [2183]
    "SERVER_NAME", // [2184]
    "SESSION", // [2185]
    "SESSION_USER", // [2186]
    "NewKeywordSegment", // [2187]
    "OldKeywordSegment", // [2188]
    "GlobalKeywordSegment", // [2189]
    "PersistKeywordSegment", // [2190]
    "Persist_onlyKeywordSegment", // [2191]
    "SessionKeywordSegment", // [2192]
    "BooleanDynamicSystemVariablesGrammar", // [2193]
    "SetClauseSegment", // [2194]
    "set_clause", // [2195]
    "NamesKeywordSegment", // [2196]
    "UnionGrammar", // [2197]
    "IntersectKeywordSegment", // [2198]
    "ExceptKeywordSegment", // [2199]
    "MinusKeywordSegment", // [2200]
    "TransactionKeywordSegment", // [2201]
    "TRANSACTION", // [2202]
    "IsolationKeywordSegment", // [2203]
    "LevelKeywordSegment", // [2204]
    "UNCOMMITTED", // [2205]
    "CommittedKeywordSegment", // [2206]
    "UncommittedKeywordSegment", // [2207]
    "SerializableKeywordSegment", // [2208]
    "SET_SESSION_VARIABLE", // [2209]
    "SETOF", // [2210]
    "SETUSER", // [2211]
    "SHAPE", // [2212]
    "SHARES", // [2213]
    "SHOW", // [2214]
    "SHUTDOWN", // [2215]
    "PositiveSegment", // [2216]
    "NegativeSegment", // [2217]
    "SIMILAR", // [2218]
    "'", // [2219]
    "SIZE", // [2220]
    "SKEW", // [2221]
    "slash", // [2222]
    "slice", // [2223]
    "SMALLINT", // [2224]
    "SNAPSHOT", // [2225]
    "SNAPSHOTS", // [2226]
    "SOCKET", // [2227]
    "SONAME", // [2228]
    "SOUNDS", // [2229]
    "SOURCE", // [2230]
    "SOURCE_AUTO_POSITION", // [2231]
    "SOURCE_BIND", // [2232]
    "SOURCE_COMPRESSION_ALGORITHMS", // [2233]
    "SOURCE_CONNECT_RETRY", // [2234]
    "SOURCE_DELAY", // [2235]
    "SOURCE_HEARTBEAT_PERIOD", // [2236]
    "SOURCE_HOST", // [2237]
    "SOURCE_LOG_FILE", // [2238]
    "SOURCE_LOG_POS", // [2239]
    "SOURCE_PASSWORD", // [2240]
    "SOURCE_PORT", // [2241]
    "SOURCE_PUBLIC_KEY_PATH", // [2242]
    "SOURCE_RETRY_COUNT", // [2243]
    "SOURCE_SSL", // [2244]
    "SOURCE_SSL_CA", // [2245]
    "SOURCE_SSL_CAPATH", // [2246]
    "SOURCE_SSL_CERT", // [2247]
    "SOURCE_SSL_CIPHER", // [2248]
    "SOURCE_SSL_CRL", // [2249]
    "SOURCE_SSL_CRLPATH", // [2250]
    "SOURCE_SSL_KEY", // [2251]
    "SOURCE_SSL_VERIFY_SERVER_CERT", // [2252]
    "SOURCE_TLS_CIPHERSUITES", // [2253]
    "SOURCE_TLS_VERSION", // [2254]
    "SOURCE_USER", // [2255]
    "SOURCE_ZSTD_COMPRESSION_LEVEL", // [2256]
    "SPACE", // [2257]
    "SPECIFIC_NAME", // [2258]
    "SPECIFICTYPE", // [2259]
    "SPLIT", // [2260]
    "SQL_AFTER_GTIDS", // [2261]
    "SQL_AFTER_MTS_GAPS", // [2262]
    "SQL_BEFORE_GTIDS", // [2263]
    "SQL_BIG_SELECTS", // [2264]
    "SQL_BIG_TABLES", // [2265]
    "SQL_BLOCK_RULE", // [2266]
    "SQL_LOG_OFF", // [2267]
    "SQL_LOG_UPDATE", // [2268]
    "SQL_LOW_PRIORITY_UPDATES", // [2269]
    "SQL_SELECT_LIMIT", // [2270]
    "SQL_THREAD", // [2271]
    "SQL_TSI_DAY", // [2272]
    "SQL_TSI_HOUR", // [2273]
    "SQL_TSI_MINUTE", // [2274]
    "SQL_TSI_MONTH", // [2275]
    "SQL_TSI_QUARTER", // [2276]
    "SQL_TSI_SECOND", // [2277]
    "SQL_TSI_WEEK", // [2278]
    "SQL_TSI_YEAR", // [2279]
    "SQL_WARNINGS", // [2280]
    "SQLCA", // [2281]
    "SQLCODE", // [2282]
    "SQLERROR", // [2283]
    "SQRT", // [2284]
    "STABLE", // [2285]
    "star", // [2286]
    "STATE", // [2287]
    "STATEMENT", // [2288]
    "DropTableStatementSegment", // [2289]
    "drop_table_statement", // [2290]
    "DropViewStatementSegment", // [2291]
    "drop_view_statement", // [2292]
    "CreateUserStatementSegment", // [2293]
    "create_user_statement", // [2294]
    "DropUserStatementSegment", // [2295]
    "drop_user_statement", // [2296]
    "TruncateStatementSegment", // [2297]
    "truncate_table", // [2298]
    "AccessStatementSegment", // [2299]
    "access_statement", // [2300]
    "CreateTableStatementSegment", // [2301]
    "create_table_statement", // [2302]
    "CreateRoleStatementSegment", // [2303]
    "create_role_statement", // [2304]
    "DropRoleStatementSegment", // [2305]
    "drop_role_statement", // [2306]
    "AlterTableStatementSegment", // [2307]
    "alter_table_statement", // [2308]
    "SetSchemaStatementSegment", // [2309]
    "set_schema_statement", // [2310]
    "DropSchemaStatementSegment", // [2311]
    "drop_schema_statement", // [2312]
    "DropTypeStatementSegment", // [2313]
    "drop_type_statement", // [2314]
    "CreateDatabaseStatementSegment", // [2315]
    "create_database_statement", // [2316]
    "DropDatabaseStatementSegment", // [2317]
    "drop_database_statement", // [2318]
    "CreateIndexStatementSegment", // [2319]
    "create_index_statement", // [2320]
    "DropIndexStatementSegment", // [2321]
    "drop_index_statement", // [2322]
    "CreateViewStatementSegment", // [2323]
    "create_view_statement", // [2324]
    "CreateCastStatementSegment", // [2325]
    "create_cast_statement", // [2326]
    "DropCastStatementSegment", // [2327]
    "drop_cast_statement", // [2328]
    "CreateFunctionStatementSegment", // [2329]
    "create_function_statement", // [2330]
    "DropFunctionStatementSegment", // [2331]
    "drop_function_statement", // [2332]
    "CreateModelStatementSegment", // [2333]
    "create_model_statement", // [2334]
    "DropModelStatementSegment", // [2335]
    "drop_MODELstatement", // [2336]
    "DescribeStatementSegment", // [2337]
    "describe_statement", // [2338]
    "UseStatementSegment", // [2339]
    "use_statement", // [2340]
    "ExplainStatementSegment", // [2341]
    "explain_statement", // [2342]
    "CreateSequenceStatementSegment", // [2343]
    "create_sequence_statement", // [2344]
    "AlterSequenceStatementSegment", // [2345]
    "alter_sequence_statement", // [2346]
    "DropSequenceStatementSegment", // [2347]
    "drop_sequence_statement", // [2348]
    "CreateTriggerStatementSegment", // [2349]
    "create_trigger", // [2350]
    "DropTriggerStatementSegment", // [2351]
    "drop_trigger", // [2352]
    "DelimiterStatement", // [2353]
    "delimiter_statement", // [2354]
    "CreateProcedureStatementSegment", // [2355]
    "create_procedure_statement", // [2356]
    "DeclareStatement", // [2357]
    "declare_statement", // [2358]
    "SetTransactionStatementSegment", // [2359]
    "set_transaction_statement", // [2360]
    "SetAssignmentStatementSegment", // [2361]
    "set_statement", // [2362]
    "IfExpressionStatement", // [2363]
    "if_then_statement", // [2364]
    "WhileStatementSegment", // [2365]
    "while_statement", // [2366]
    "IterateStatementSegment", // [2367]
    "iterate_statement", // [2368]
    "RepeatStatementSegment", // [2369]
    "repeat_statement", // [2370]
    "LoopStatementSegment", // [2371]
    "loop_statement", // [2372]
    "CallStoredProcedureSegment", // [2373]
    "call_statement", // [2374]
    "PrepareSegment", // [2375]
    "prepare_segment", // [2376]
    "ExecuteSegment", // [2377]
    "execute_segment", // [2378]
    "DeallocateSegment", // [2379]
    "deallocate_segment", // [2380]
    "GetDiagnosticsSegment", // [2381]
    "get_diagnostics_segment", // [2382]
    "ResignalSegment", // [2383]
    "resignal_segment", // [2384]
    "CursorOpenCloseSegment", // [2385]
    "cursor_open_close_segment", // [2386]
    "CursorFetchSegment", // [2387]
    "cursor_fetch_segment", // [2388]
    "DropProcedureStatementSegment", // [2389]
    "drop_procedure_statement", // [2390]
    "AlterViewStatementSegment", // [2391]
    "alter_view_statement", // [2392]
    "RenameTableStatementSegment", // [2393]
    "rename_table_statement", // [2394]
    "ResetMasterStatementSegment", // [2395]
    "reset_master_statement", // [2396]
    "PurgeBinaryLogsStatementSegment", // [2397]
    "purge_binary_logs_statement", // [2398]
    "HelpStatementSegment", // [2399]
    "help_statement", // [2400]
    "CheckTableStatementSegment", // [2401]
    "check_table_statement", // [2402]
    "ChecksumTableStatementSegment", // [2403]
    "checksum_table_statement", // [2404]
    "AnalyzeTableStatementSegment", // [2405]
    "analyze_table_statement", // [2406]
    "RepairTableStatementSegment", // [2407]
    "repair_table_statement", // [2408]
    "OptimizeTableStatementSegment", // [2409]
    "optimize_table_statement", // [2410]
    "InsertRowAliasSegment", // [2411]
    "insert_row_alias", // [2412]
    "FlushStatementSegment", // [2413]
    "flush_statement", // [2414]
    "LoadDataSegment", // [2415]
    "load_data_statement", // [2416]
    "ReplaceSegment", // [2417]
    "replace_statement", // [2418]
    "AlterDatabaseStatementSegment", // [2419]
    "alter_database_statement", // [2420]
    "ReturnStatementSegment", // [2421]
    "return_statement", // [2422]
    "SetNamesStatementSegment", // [2423]
    "set_names_statement", // [2424]
    "CreateEventStatementSegment", // [2425]
    "create_event_statement", // [2426]
    "AlterEventStatementSegment", // [2427]
    "alter_event_statement", // [2428]
    "DropEventStatementSegment", // [2429]
    "drop_event_statement", // [2430]
    "SetDefaultRoleStatementSegment", // [2431]
    "set_default_role_statement", // [2432]
    "STATISTICS", // [2433]
    "STATS", // [2434]
    "STDDEV_POP", // [2435]
    "STDDEV_SAMP", // [2436]
    "STDIN", // [2437]
    "STDOUT", // [2438]
    "STOP", // [2439]
    "STORAGE", // [2440]
    "STREAMING", // [2441]
    "STRICT", // [2442]
    "ConcatSegment", // [2443]
    "STRING", // [2444]
    "STRUCT", // [2445]
    "STRUCTURE", // [2446]
    "STYLE", // [2447]
    "SUBLIST", // [2448]
    "SUBMULTISET", // [2449]
    "SUBPARTITION", // [2450]
    "SUBPARTITIONS", // [2451]
    "SUBSTRING", // [2452]
    "SUCCESSFUL", // [2453]
    "SUPER", // [2454]
    "SUPERUSER", // [2455]
    "SUSPEND", // [2456]
    "SWAPS", // [2457]
    "SWITCH", // [2458]
    "SWITCHES", // [2459]
    "SYMMETRIC", // [2460]
    "SYNC", // [2461]
    "SYNONYM", // [2462]
    "SYSDATE", // [2463]
    "SYSID", // [2464]
    "system_variable", // [2465]
    "SYSTEM_USER", // [2466]
    "Autoextend_sizeKeywordSegment", // [2467]
    "Avg_row_lengthKeywordSegment", // [2468]
    "CompressionKeywordSegment", // [2469]
    "ConnectionKeywordSegment", // [2470]
    "DirectoryKeywordSegment", // [2471]
    "Delay_key_writeKeywordSegment", // [2472]
    "Insert_methodKeywordSegment", // [2473]
    "Max_rowsKeywordSegment", // [2474]
    "Min_rowsKeywordSegment", // [2475]
    "Pack_keysKeywordSegment", // [2476]
    "Row_formatKeywordSegment", // [2477]
    "DynamicKeywordSegment", // [2478]
    "FixedKeywordSegment", // [2479]
    "CompressedKeywordSegment", // [2480]
    "RedundantKeywordSegment", // [2481]
    "CompactKeywordSegment", // [2482]
    "Stats_auto_recalcKeywordSegment", // [2483]
    "Stats_persistentKeywordSegment", // [2484]
    "Stats_sample_pagesKeywordSegment", // [2485]
    "StorageKeywordSegment", // [2486]
    "DiskKeywordSegment", // [2487]
    "MemoryKeywordSegment", // [2488]
    "UnionKeywordSegment", // [2489]
    "DualIdentifierSegment", // [2490]
    "TABLE_CHECKSUM", // [2491]
    "TABLET", // [2492]
    "TABLETS", // [2493]
    "TAG", // [2494]
    "Expression_A_Unary_Operator_Grammar", // [2495]
    "Expression_C_Grammar", // [2496]
    "Expression_B_Unary_Operator_Grammar", // [2497]
    "TEMPLATE", // [2498]
    "TRANSIENT", // [2499]
    "TransientKeywordSegment", // [2500]
    "TemporaryGrammar", // [2501]
    "TERMINATE", // [2502]
    "TEXT", // [2503]
    "TEXTSIZE", // [2504]
    "THREAD_PRIORITY", // [2505]
    "ZoneKeywordSegment", // [2506]
    "ZONE", // [2507]
    "TIMEZONE_HOUR", // [2508]
    "TIMEZONE_MINUTE", // [2509]
    "TINYBLOB", // [2510]
    "TINYINT", // [2511]
    "TINYTEXT", // [2512]
    "TLS", // [2513]
    "TOAST", // [2514]
    "TOKEN_FILTER", // [2515]
    "TOKENIZER", // [2516]
    "TOP", // [2517]
    "TOP_LEVEL_COUNT", // [2518]
    "TRAN", // [2519]
    "WorkKeywordSegment", // [2520]
    "WORK", // [2521]
    "LeaveKeywordSegment", // [2522]
    "CommitKeywordSegment", // [2523]
    "ChainKeywordSegment", // [2524]
    "RollbackKeywordSegment", // [2525]
    "TRANSACTION_ACTIVE", // [2526]
    "TRANSACTIONS", // [2527]
    "TRANSACTIONS_COMMITTED", // [2528]
    "TRANSACTIONS_ROLLED_BACK", // [2529]
    "TRANSFORM", // [2530]
    "TRANSFORMS", // [2531]
    "TRANSLATE", // [2532]
    "TRANSLATION", // [2533]
    "TRASH", // [2534]
    "TREAT", // [2535]
    "TREE", // [2536]
    "TRIGGER_CATALOG", // [2537]
    "TRIGGER_NAME", // [2538]
    "TRIGGER_SCHEMA", // [2539]
    "TRIGGERS", // [2540]
    "TRIM", // [2541]
    "BothKeywordSegment", // [2542]
    "LeadingKeywordSegment", // [2543]
    "TrailingKeywordSegment", // [2544]
    "TRUSTED", // [2545]
    "TSEQUAL", // [2546]
    "TYPECAST", // [2547]
    "StructLiteralSegment", // [2548]
    "TYPES", // [2549]
    "UESCAPE", // [2550]
    "UID", // [2551]
    "NaturalJoinKeywordsGrammar", // [2552]
    "UnconditionalCrossJoinKeywordsGrammar", // [2553]
    "HorizontalJoinKeywordsGrammar", // [2554]
    "UNDER", // [2555]
    "UNDO_BUFFER_SIZE", // [2556]
    "UNDOFILE", // [2557]
    "UNENCRYPTED", // [2558]
    "UNICODE", // [2559]
    "UNINSTALL", // [2560]
    "UNKNOWN", // [2561]
    "UNLISTEN", // [2562]
    "UNNAMED", // [2563]
    "UNNEST", // [2564]
    "UNREGISTER", // [2565]
    "UNSET", // [2566]
    "UP", // [2567]
    "UPDATETEXT", // [2568]
    "UPPER", // [2569]
    "USE_MV", // [2570]
    "USER_DEFINED_TYPE_CATALOG", // [2571]
    "USER_DEFINED_TYPE_CODE", // [2572]
    "USER_DEFINED_TYPE_NAME", // [2573]
    "USER_DEFINED_TYPE_SCHEMA", // [2574]
    "USERS", // [2575]
    "UTC_DATE", // [2576]
    "UTC_TIME", // [2577]
    "UTC_TIMESTAMP", // [2578]
    "VACUUM", // [2579]
    "VALID", // [2580]
    "VALIDATE", // [2581]
    "VALIDATOR", // [2582]
    "VAR_POP", // [2583]
    "VAR_SAMP", // [2584]
    "VARBINARY", // [2585]
    "VARCHAR2", // [2586]
    "VARCHAR", // [2587]
    "VARCHARACTER", // [2588]
    "VARIABLE", // [2589]
    "VARIABLES", // [2590]
    "VARIANT", // [2591]
    "VAULT", // [2592]
    "VAULTS", // [2593]
    "VCPU", // [2594]
    "VERBOSE", // [2595]
    "VERSION", // [2596]
    "VOLATILE", // [2597]
    "WAIT", // [2598]
    "WAITFOR", // [2599]
    "assignment_operator", // [2600]
    "WAREHOUSES", // [2601]
    "WARM", // [2602]
    "WARNINGS", // [2603]
    "WEEKDAY", // [2604]
    "WEIGHT_STRING", // [2605]
    "indented_then", // [2606]
    "indented_then_contents", // [2607]
    "WHENEVER", // [2608]
    "WHILE", // [2609]
    "WhileKeywordSegment", // [2610]
    "WHITELIST", // [2611]
    "whitespace", // [2612]
    "WhitespaceSegment", // [2613]
    "WIDTH_BUCKET", // [2614]
    "PartitionClauseSegment", // [2615]
    "partitionby_clause", // [2616]
    "FrameClauseSegment", // [2617]
    "frame_clause", // [2618]
    "CascadedKeywordSegment", // [2619]
    "RecursiveKeywordSegment", // [2620]
    "indented_ctes", // [2621]
    "CTEDefinitionSegment", // [2622]
    "common_table_expression", // [2623]
    "NonWithNonSelectableGrammar", // [2624]
    "BindingKeywordSegment", // [2625]
    "WITHIN", // [2626]
    "WORKLOAD", // [2627]
    "WRITETEXT", // [2628]
    "XA", // [2629]
    "XID", // [2630]
    "XML", // [2631]
    "YAML", // [2632]
];

pub static AUX_DATA: &[u32] = &[
    0, 1, 2, 1, 1, 4, 3, 1, 4, 5, 6, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 7, 1, 2, 1, 1, 4, 3, 1, 4, 5, 8, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1, 0, 0, 0,
    0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0,
    1, 0, 1, 0, 1, 0, 0, 0, 4294967295, 4294967295, 0, 10, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 210, 1, 2, 1, 1, 4, 3, 1, 4, 5, 211,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 212, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 213, 1, 2, 1, 1, 4, 3, 1, 4, 5, 214, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 215, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 216, 1, 2, 1, 1, 4, 3, 1, 4, 5, 217, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 218, 1, 2, 1, 1, 4, 3, 1, 4, 5, 219,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 220, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 221, 1, 2, 1, 1, 4, 3, 1, 4, 5, 222, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 226, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 82, 236, 237, 1, 236, 3, 3, 4, 238, 239, 240, 237, 1, 240, 3,
    3, 4, 238, 1, 2, 1, 0, 247, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 11, 1, 2, 1, 1, 4, 3, 1, 4, 5, 252, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 253, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 254, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 4294967295,
    1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    4294967295, 4294967295, 0, 0, 1, 0, 0, 0, 1, 0, 1, 2, 1, 0, 0, 1,
    0, 1, 2, 1, 0, 0, 1, 2, 1, 0, 505, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 506, 507, 237, 1, 507, 3, 3, 4, 238, 508, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 454, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 0, 1, 0, 1, 0, 0, 517, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 518, 1, 2, 1, 1, 4, 3, 1, 4, 5, 519, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 519, 521, 522, 1, 521, 3, 3, 521, 4,
    520, 521, 522, 1, 521, 3, 3, 521, 4, 0, 523, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 524, 1, 2, 1, 1, 4, 3, 1, 4, 5, 525, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 98, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 526, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0,
    207, 552, 237, 1, 552, 3, 3, 4, 238, 553, 554, 237, 1, 554, 3, 3,
    4, 238, 1, 2, 0, 555, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 1, 2, 0, 231, 1, 2, 1, 1, 4, 3, 1, 4, 5, 557, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 558, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 559, 1, 2, 1, 1, 4, 3, 1, 4, 5, 560, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 561, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    562, 1, 2, 1, 1, 4, 3, 1, 4, 5, 272, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 563, 563, 565, 3, 563, 564, 563, 3, 3, 564, 4, 566,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 567, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 568, 1, 2, 1, 1, 4, 3, 1, 4, 5, 569, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 570, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 571, 1, 2, 1, 1, 4, 3, 1, 4, 5, 572, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 574, 1, 2, 1, 1, 4, 3, 1, 4, 5, 397,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 396, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 575, 1, 2, 1, 1, 4, 3, 1, 4, 5, 398, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 576, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 577, 1, 2, 1, 1, 4, 3, 1, 4, 5, 578, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 579, 1, 2, 1, 1, 4, 3, 1, 4, 5, 580,
    581, 582, 583, 584, 585, 815, 6, 586, 587, 1, 586, 2, 3, 4, 0, 597,
    598, 0, 2, 3, 597, 3, 599, 0, 1, 3, 600, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 601, 1, 2, 1, 1, 4, 3, 1, 4, 5, 602, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 603, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 604, 1, 2, 1, 1, 4, 3, 1, 4, 5, 605, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 606, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    607, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 521, 522, 9, 3, 3,
    521, 4, 622, 1, 2, 1, 1, 4, 3, 1, 4, 5, 623, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 624, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    625, 1, 2, 1, 1, 4, 3, 1, 4, 5, 626, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 627, 1, 2, 1, 1, 4, 3, 1, 4, 5, 628, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 629, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 630, 1, 2, 1, 1, 4, 3, 1, 4, 5, 631, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 536, 521, 237, 1, 521, 3, 3, 4, 238, 636,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 637, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 638, 1, 2, 1, 1, 4, 3, 1, 4, 5, 639, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 640, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 0, 0, 649, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    652, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 1,
    0, 1, 2, 1, 1, 2, 1, 0, 0, 1, 0, 1, 2, 1, 658, 659,
    0, 2, 3, 658, 1, 2, 1, 662, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 663, 1, 2, 1, 1, 4, 3, 1, 4, 5, 664, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 665, 1, 2, 1, 1, 4, 3, 1, 4, 5, 666,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 667, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 668, 1, 2, 1, 1, 4, 3, 1, 4, 5, 516, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 669, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 670, 1, 2, 1, 1, 4, 3, 1, 4, 5, 671, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 349, 1, 2, 1, 1, 4, 3, 1, 4, 5, 672,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 2, 1, 338,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 675, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 676, 1, 2, 1, 1, 4, 3, 1, 4, 5, 678, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 679, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 680, 1, 2, 1, 1, 4, 3, 1, 4, 5, 681, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 682, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 0, 683, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 697, 1, 2, 1, 1, 4, 3, 1, 4, 5, 698, 699, 237, 1,
    699, 3, 3, 4, 238, 12, 1, 2, 1, 1, 4, 3, 1, 4, 5, 700,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 701, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 702, 1, 2, 1, 1, 4, 3, 1, 4, 5, 703, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 704, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 705, 1, 2, 1, 1, 4, 3, 1, 4, 5, 438, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 706, 1, 2, 1, 1, 4, 3, 1, 4, 5, 707,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 709, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 710, 1, 2, 1, 1, 4, 3, 1, 4, 5, 258, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 711, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 712, 1, 2, 1, 1, 4, 3, 1, 4, 5, 713, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 714, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 0, 724, 1, 2, 1, 1, 4, 3, 1, 4, 5, 725, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 259, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 430, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 4294967295, 4294967295,
    0, 741, 1, 2, 1, 1, 4, 3, 1, 4, 5, 742, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 399, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 0, 744, 1, 2, 1, 1, 4, 3, 1, 4, 5, 745, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 746, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    747, 1, 2, 1, 1, 4, 3, 1, 4, 5, 748, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 749, 1, 2, 1, 1, 4, 3, 1, 4, 5, 750, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 751, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 752, 1, 2, 1, 1, 4, 3, 1, 4, 5, 753, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 754, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    455, 1, 2, 1, 1, 4, 3, 1, 4, 5, 755, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 756, 1, 2, 1, 1, 4, 3, 1, 4, 5, 4, 587,
    9, 2, 3, 4, 260, 1, 2, 1, 1, 4, 3, 1, 4, 5, 757, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 761, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 762, 1, 2, 1, 1, 4, 3, 1, 4, 5, 763,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 764, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 765, 1, 2, 1, 1, 4, 3, 1, 4, 5, 551, 766, 237,
    1, 766, 3, 3, 4, 238, 551, 767, 237, 1, 767, 3, 3, 4, 238, 551,
    768, 237, 1, 768, 3, 3, 4, 238, 0, 1, 2, 1, 0, 1, 2, 1,
    0, 0, 1, 2, 1, 0, 0, 0, 1, 2, 1, 0, 0, 0, 0, 0,
    0, 4294967295, 4294967295, 0, 0, 1, 2, 1, 0, 4294967295, 4294967295, 0, 0, 364, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 609, 810, 237, 1, 810, 3, 3, 4, 238,
    1, 0, 811, 1, 2, 1, 1, 4, 3, 1, 4, 5, 812, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 814, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    87, 815, 237, 1, 815, 3, 3, 4, 238, 816, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 817, 1, 2, 1, 1, 4, 3, 1, 4, 5, 320, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 819, 820, 10, 3, 3, 819, 4, 821, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 822, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 823, 1, 2, 1, 1, 4, 3, 1, 4, 5, 824, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 330, 833, 9, 3, 3, 330, 4, 834, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 304, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 835, 1, 2, 1, 1, 4, 3, 1, 4, 5, 521, 836, 0, 2,
    3, 521, 330, 837, 0, 2, 3, 330, 838, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 839, 1, 2, 1, 1, 4, 3, 1, 4, 5, 840, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 400, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    841, 1, 2, 1, 1, 4, 3, 1, 4, 5, 842, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 843, 1, 2, 1, 1, 4, 3, 1, 4, 5, 844, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 854, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 855, 1, 2, 1, 1, 4, 3, 1, 4, 5, 99, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 401, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 856, 1, 2, 1, 1, 4, 3, 1, 4, 5, 857, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 858, 1, 2, 1, 1, 4, 3, 1, 4, 5, 431,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 859, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 860, 1, 2, 1, 1, 4, 3, 1, 4, 5, 861, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 862, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 863, 1, 2, 1, 1, 4, 3, 1, 4, 5, 717, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 864, 1, 2, 1, 1, 4, 3, 1, 4, 5, 865,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 866, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 867, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 2,
    3, 1, 452, 870, 2, 1, 870, 4, 3, 1, 4, 5, 452, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 871, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    369, 1, 2, 1, 1, 4, 3, 1, 4, 5, 872, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 873, 1, 2, 1, 1, 4, 3, 1, 4, 5, 874, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 875, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 876, 1, 2, 1, 1, 4, 3, 1, 4, 5, 877, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 3, 4, 1, 0, 0, 0, 0, 4294967295, 4294967295, 0,
    0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0, 0, 4294967295, 1, 0,
    100, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 0, 0,
    1, 0, 1, 2, 1, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1,
    0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 1,
    0, 1, 2, 1, 1, 0, 1, 2, 1, 1, 2, 1, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1,
    0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0,
    0, 0, 0, 0, 1, 2, 1, 0, 1011, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1012, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1013, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1014, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1015, 1, 2, 1, 1, 4, 3, 1, 4, 5, 846, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1016, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1017, 870,
    587, 1, 870, 2, 3, 4, 1017, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1, 2, 1, 1022, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1001, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 1030, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 580, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1031, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1032, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1033, 1, 2, 1, 1, 4, 3, 1, 4, 5, 581, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 582, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1034, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1035, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 0, 1040, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 1046, 1, 2, 1, 1, 4, 3, 1, 4, 5, 339,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 52, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 13, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1047, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1048, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 1049, 587, 1, 1049, 2, 3, 4, 0, 0, 0, 0, 0, 0,
    0, 1071, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1072, 1073, 1074, 2923, 3,
    870, 587, 1, 870, 2, 3, 4, 0, 1, 0, 1, 2, 1, 0, 242, 1079,
    565, 3, 1079, 564, 242, 3, 3, 564, 4, 0, 789, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 278, 279, 280, 281, 282, 283, 284, 285, 286, 287, 288, 289,
    290, 291, 292, 293, 294, 295, 296, 297, 2964, 20, 1080, 587, 1, 1080, 2, 3,
    4, 1081, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1082, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1083, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1084,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1085, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1086, 1, 2, 1, 1, 4, 3, 1, 4, 5, 278, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 279, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 280, 1, 2, 1, 1, 4, 3, 1, 4, 5, 281, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 282, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1087,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1088, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1089, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1090, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1091, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1092, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1096, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1097, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1098, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1099, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1100, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    0, 0, 0, 0, 1120, 1, 2, 1, 1, 4, 3, 1, 4, 5, 261, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1122, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1123, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1124, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1125, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1126, 1, 2, 1, 1, 4, 3, 1, 4, 5, 266, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1127, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1128, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 402, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1129, 1, 2, 1, 1, 4, 3, 1, 4, 5, 101, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 0, 1, 0, 0, 1, 0, 0, 1155, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1157, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1158, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1159, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1160, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1161, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1162, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1163, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1164, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 655, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1165, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1167, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1168, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1169, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1170, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 715, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1171, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1172, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1173, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1174, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1175, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1176, 1, 2, 1, 1, 4, 3, 1, 4, 5, 311, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 456, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1177, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1178, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1179, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1180, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1181, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1182, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1183, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 940, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1184, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 0, 0, 0,
    534, 1, 2, 1, 1, 4, 3, 1, 4, 5, 534, 521, 2, 1, 521, 4,
    3, 1, 4, 5, 531, 521, 237, 1, 521, 3, 3, 4, 238, 322, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 14, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1189, 1, 2, 1, 1, 4, 3, 1, 4, 5, 759, 1190, 237, 1, 1190,
    3, 3, 4, 238, 1052, 1, 2, 1, 1, 4, 3, 1, 4, 5, 244, 1191,
    1193, 3, 1191, 1192, 244, 3, 3, 1192, 4, 244, 1194, 587, 3, 1194, 4, 244,
    2, 3, 4, 244, 1196, 565, 3, 1196, 564, 244, 3, 3, 564, 4, 0, 3,
    4, 1, 0, 0, 0, 361, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1203, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1204, 1205, 1193, 1, 1205, 3,
    3, 1192, 4, 1204, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1206, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1207, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1208, 1, 2, 1, 1, 4, 3, 1, 4, 5, 933, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1209, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1210,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1211, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1212, 1, 2, 1, 1, 4, 3, 1, 4, 5, 952, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1213, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 691, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1214, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1215, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1, 1, 312, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1220, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1221, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1222, 1, 2, 1, 1, 4, 3, 1, 4, 5, 262, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1223, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1224, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1225, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1226, 1227, 237, 1, 1227, 3, 3, 4, 238, 693, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 299, 1, 2, 1, 1, 4, 3, 1, 4, 5, 448,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 403, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 666, 926, 927, 928, 929, 930, 931, 932, 4115, 8, 1228, 587, 1,
    1228, 2, 3, 4, 404, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1229, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1230, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1231, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1232, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1234, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1235, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1236, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1237, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1238, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 269, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1239, 1, 2, 1, 1, 4, 3, 1, 4, 5, 273, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1240, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1241, 1, 2, 1, 1, 4, 3, 1, 4, 5, 457, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1242, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1243, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 376, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1244, 1, 2, 1, 1, 4, 3, 1, 4, 5, 102, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 0, 134, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1245, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1246, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1103, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1247, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1248, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 991, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1249, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1250, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1258, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 4294967295,
    4294967295, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 2, 1, 0, 0, 4294967295,
    4294967295, 0, 0, 0, 1, 0, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1,
    0, 0, 0, 728, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1310, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1311, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 15, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1072, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1312, 1, 2, 1, 1, 4, 3, 1, 4, 5, 988,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 643, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 643, 1313, 1314, 1, 1313, 5, 3, 1, 564, 4, 5, 729, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1315, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1316, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0,
    1036, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1321, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 16, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295,
    4294967295, 0, 1, 4294967295, 4294967295, 0, 1, 0, 0, 4294967295, 4294967295, 0, 1323, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1324, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 2, 1, 1325, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1328, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1329, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 388, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1330, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1331, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1332,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1333, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1334, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1335, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1336, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1356, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1357, 1, 2, 1, 1, 4, 3, 1, 4, 5, 955, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 0, 0, 730, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1201, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 17, 1, 2, 1, 1, 4, 3, 1, 4, 5, 177, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1368, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1369,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1115, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 0, 0, 0, 1376, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1377, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1378, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1379, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 0, 0, 1, 2, 1, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0,
    1, 2, 1, 0, 227, 1419, 1, 0, 0, 4294967295, 4294967295, 0, 245, 1419, 1, 1,
    2, 1, 0, 91, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1420, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1421, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 848, 1, 2, 1, 1, 4, 3, 1, 4, 5, 432, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 1,
    2, 1, 18, 1, 2, 1, 1, 4, 3, 1, 4, 5, 5, 870, 1438, 2,
    870, 5, 3, 3, 4, 5, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 1, 0,
    1, 2, 1, 0, 0, 67, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1456,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 19, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1457, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1341, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 780, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1458, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1459, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1460, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1461,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 0, 0, 0,
    1, 0, 1462, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1490, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1491, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1492, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1493, 1493, 833, 3, 1493, 330,
    1493, 3, 3, 330, 4, 1494, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1495,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1496, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 189, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0,
    1, 0, 0, 1505, 1, 2, 1, 1, 4, 3, 1, 4, 5, 133, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1506, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1507, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 0, 0,
    195, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1515, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1, 1, 0, 1, 0, 1509, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 1519, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1520, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1109, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1186, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1521, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0,
    0, 1380, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1522, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1523, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1524,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1525, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1527, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1528, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1529, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1530, 1, 2, 1, 1, 4, 3, 1, 4, 5, 514, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 997, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1531,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 797, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1532, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1533, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1534, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1535, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1344, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1536, 1, 2, 1, 1, 4, 3, 1, 4, 5, 283,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 284, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 285, 1, 2, 1, 1, 4, 3, 1, 4, 5, 286, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1537, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1538, 1, 2, 1, 1, 4, 3, 1, 4, 5, 960, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1192, 1193, 9, 3, 3, 1192, 4, 1539, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1540, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1541, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 366, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1133, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1549, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1550, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1551, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1552, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1553, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1554, 1, 2, 1, 1, 4, 3, 1, 4, 5, 458,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 103, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 62, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0,
    1, 2, 1, 0, 1555, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1556, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1557, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 340, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1558, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 1, 0, 1, 2, 1, 0,
    1, 0, 1, 2, 1, 0, 0, 0, 1, 2, 1, 405, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 4294967295, 1, 0, 0, 1573, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1574, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1575,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1576, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1577, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1578, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 965, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1579, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1580, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1581, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1582,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 610, 810, 237, 1, 810, 3, 3,
    4, 238, 849, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1583, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 370, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1584, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1583, 1585, 237, 1, 1585, 3,
    3, 4, 238, 62, 1585, 237, 1, 1585, 3, 3, 4, 238, 1586, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 104, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 2, 1, 1, 0, 1, 2, 1, 0, 1589, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 406, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1590, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 881, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 371, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1591, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1592, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1593,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1594, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1595, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1596, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1597, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1598, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1061, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 20, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1599,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1600, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1381, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1601, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 590, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 4294967295, 4294967295, 0, 0, 0, 1, 0, 0, 485, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1559, 1, 2, 1, 1, 4, 3, 1, 4, 5, 425,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 497, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1618, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1619, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1620, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1621, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1622, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1623, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    611, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1628, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1629, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1630, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1631, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1632, 1, 2, 1, 1, 4, 3, 1, 4, 5, 971, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1633, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1635, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1636, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 227, 1640, 1, 0, 245, 1640, 1, 0,
    1563, 1, 2, 1, 1, 4, 3, 1, 4, 5, 685, 1646, 1, 1, 2, 1,
    0, 245, 1646, 1, 0, 0, 1, 0, 1, 2, 1, 1653, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 1, 0, 1,
    2, 1, 4, 5, 1, 1442, 870, 1438, 1, 870, 3, 3, 4, 5, 0, 0,
    0, 2, 4294967295, 0, 5, 6, 1, 1445, 870, 2, 1, 870, 4, 3, 1, 4,
    5, 1442, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1445, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1665, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1666,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 433, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 407, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1667, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1668, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1669, 1, 2, 1, 1, 4, 3, 1, 4, 5, 451, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 2, 9, 4, 3, 1, 4, 5, 1670, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1588, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1671, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1672, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 21, 1, 2, 1, 1, 4, 3, 1, 4, 5, 92,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1673, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1674, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1675, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1676, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1677, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1678, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1679, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1680,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1681, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1422, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1682, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1683, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1684, 1, 2, 1, 1, 4, 3, 1, 4, 5, 850, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1685, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1686,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1687, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1688, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 920, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 615, 615, 833, 3, 615, 330, 615, 3,
    3, 330, 4, 0, 1, 2, 1, 0, 0, 0, 1146, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1694, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1613, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1695, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1696, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1697, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2,
    1, 1698, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 564, 1314, 9, 5,
    3, 1, 564, 4, 5, 564, 565, 9, 3, 3, 564, 4, 1708, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1, 0, 1, 2, 1, 1709,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 509, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2, 4294967295, 1713, 587, 1, 1713, 2, 3, 4, 583, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 584, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1714, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1715, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 367, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1365, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1716, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1717, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1718, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1719, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1339, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1720, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1721, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1722, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1723, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 0, 0, 1131, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1726,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1727, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 2, 1, 2, 3, 1, 105, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1729, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1730, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 22, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1731, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1732, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1733, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1734,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1735, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1736, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1737, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1738, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1739, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1740, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1741, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1742,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1743, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1744, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1745, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1746, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1747, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1748, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1749, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1750,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1751, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1752, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1753, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1754, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1755, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1756, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1757, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1758,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1759, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1760, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1761, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1762, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1763, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1764, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 23, 1, 2, 1, 1, 4, 3, 1, 4, 5, 798,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 980, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 981, 1, 2, 1, 1, 4, 3, 1, 4, 5, 408, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1765, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 982, 1, 2, 1, 1, 4, 3, 1, 4, 5, 983, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1766, 1, 2, 1, 1, 4, 3, 1, 4, 5, 341,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 731, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1767, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1768, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1769, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1770, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1771, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1772, 1, 2, 1, 1, 4, 3, 1, 4, 5, 487,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 0, 0, 1,
    2, 1, 0, 227, 1640, 1, 245, 1640, 1, 1791, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1792, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1471, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 882, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 287, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1793, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1794, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1795,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1796, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 799, 1, 2, 1, 1, 4, 3, 1, 4, 5, 409, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1382, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 530, 521, 237, 1, 521, 3, 3, 4, 238, 288, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 289, 1, 2, 1, 1, 4, 3, 1, 4, 5, 290, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1797, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 342, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1415, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1798, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    535, 1, 2, 1, 1, 4, 3, 1, 4, 5, 535, 521, 2, 1, 521, 4,
    3, 1, 4, 5, 1367, 1, 2, 1, 1, 4, 3, 1, 4, 5, 909, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 718, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 106, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1799, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 527, 521, 237, 1, 521, 3, 3, 4, 238, 40,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 291, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1800, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1801, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1802, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1803, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1804, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 528, 521, 237, 1, 521, 3, 3, 4, 238, 1805, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1806, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1807, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1808, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1809, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1810, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1472, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 3, 4, 1205, 1193, 1, 1205, 3, 3, 1192, 4, 1811, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 1, 0, 1816, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1625, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1625, 1817, 1314, 1, 1817, 5, 3, 1, 564, 4, 5, 1818, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1819, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1821, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1822, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1823, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1824,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1825, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 530, 1826, 237, 1, 1826, 3, 3, 4, 238, 1656, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1827, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1828, 1, 2, 1, 1, 4, 3, 1, 4, 5, 992, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1829, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1830, 1831,
    12, 3, 3, 1830, 4, 1038, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1832,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 343, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1833, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1834, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 510, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1835, 1, 2, 1, 1, 4, 3, 1, 4, 5, 344, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1836, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1837,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 905, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1838, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1839, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1840, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 345, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1841, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1842, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1843,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 2, 1, 1,
    2, 1, 0, 0, 1, 2, 1, 0, 0, 1852, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1853, 1, 2, 1, 1, 4, 3, 1, 4, 5, 377, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 346, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1854, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1855, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1856, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    306, 1, 2, 1, 1, 4, 3, 1, 4, 5, 306, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 608, 1858, 587, 1, 1858, 2, 3, 4, 0, 1859, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1860, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1861, 1, 2, 1, 1, 4, 3, 1, 4, 5, 585, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1360, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1862,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1863, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 771, 1, 2, 1, 1, 4, 3, 1, 4, 5, 771, 1817, 1314,
    1, 1817, 5, 3, 1, 564, 4, 5, 1864, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1865, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1548, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1467, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1866, 1, 2, 1, 1, 4, 3, 1, 4, 5, 97, 97, 565, 3, 97, 564,
    97, 3, 3, 564, 4, 1867, 1, 2, 1, 1, 4, 3, 1, 4, 5, 95,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1704, 1870, 237, 1, 1870,
    3, 3, 4, 238, 1, 2, 0, 0, 0, 1874, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1875, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1876, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1877, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1358, 1, 2, 1, 1, 4, 3, 1, 4, 5, 644, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1878, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 1383, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1879, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1880, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1881,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1882, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 164, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1883, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1884, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 337, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1041, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1885, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1886,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1887, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1888, 1, 2, 1, 1, 4, 3, 1, 4, 5, 107, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1889, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1890, 1, 2, 1, 1, 4, 3, 1, 4, 5, 459, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 0, 1891, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1503, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1002, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1609, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    910, 1, 2, 1, 1, 4, 3, 1, 4, 5, 612, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 612, 521, 522, 1, 521, 3, 3, 521, 4, 614, 521, 522,
    1, 521, 3, 3, 521, 4, 613, 521, 522, 1, 521, 3, 3, 521, 4, 0,
    0, 0, 0, 1, 0, 0, 224, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1897, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1655, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1898, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1899,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1900, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1652, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1603, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1901, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1900, 1585, 237, 1, 1585, 3, 3, 4, 238, 1, 2, 1, 0, 1903, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 3, 4, 1, 0, 1308, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1905, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1906, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1907, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1908, 1, 2, 1, 1, 4, 3, 1, 4, 5, 108, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 410, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1909, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1910, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1911, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    5, 4294967295, 1912, 587, 1, 1912, 2, 3, 4, 1913, 1912, 237, 1, 1912, 3, 3,
    4, 238, 1914, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1915, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1916, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1917, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1918, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1919, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1920, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1921, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1922, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1569, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1923, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 0, 1, 2, 1, 0, 474, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 2, 1, 0, 1, 1, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2,
    1, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 0, 0, 475, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1931, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1932, 1, 2, 1, 1, 4, 3, 1, 4, 5, 411, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1933, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1934,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 989, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1935, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1659, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 5, 1937, 1438, 2, 1937, 5, 3, 3, 4,
    5, 1, 0, 0, 1938, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1939, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1940, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1941, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1942, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1943, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1944, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1945, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1946, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1947, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1948, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1949, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1950, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 137, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    537, 1951, 237, 1, 1951, 3, 3, 4, 238, 1952, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1953, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1954, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1955, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1956, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1957, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1958, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1959,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 529, 521, 237, 1, 521, 3, 3,
    4, 238, 1960, 1, 2, 1, 1, 4, 3, 1, 4, 5, 128, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1961, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1962, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1963, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 529, 1826, 237, 1, 1826, 3, 3, 4, 238, 0, 0, 1969,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1970, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 956, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1372, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1055, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1971, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1972, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1095, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1973, 1, 2, 1, 1, 4, 3, 1, 4, 5, 308, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1974, 1, 2, 1, 1, 4, 3, 1, 4, 5, 434, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1976, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1273, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1977, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 142, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1978, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1979, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 24, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    1, 0, 1, 2, 1, 68, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1984,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1985, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1986, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1987, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 921, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1988, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1989, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 206, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 0, 1990, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1384, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 1995, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 800, 1, 2, 1, 1, 4, 3, 1, 4, 5, 292, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1996, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1997, 1, 2, 1, 1, 4, 3, 1, 4, 5, 732, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1998, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1999, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1045, 1191, 1193, 3, 1191, 1192,
    1045, 3, 3, 1192, 4, 242, 1196, 565, 3, 1196, 564, 242, 3, 3, 564, 4,
    1, 4294967295, 4294967295, 0, 2001, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2002, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 962, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1370, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2,
    1, 0, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 3, 4, 1, 0,
    1, 0, 1, 2, 1, 1, 2, 1, 1, 2, 1, 0, 2013, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 329, 2014, 237, 1, 2014, 3, 3, 4, 238, 533,
    2014, 237, 1, 2014, 3, 3, 4, 238, 2015, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 532, 2014, 237, 1, 2014, 3, 3, 4, 238, 608, 2014, 237, 1, 2014,
    3, 3, 4, 238, 109, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2016, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 719, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2017, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2018, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2019, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    460, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2020, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2021, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2022, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2023, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2024, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2025, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2026, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2027, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2028, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2029, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295,
    1, 0, 2032, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 111, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 110, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2037, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 2039, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1262, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2040, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2041, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2042, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2043, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2044, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2045, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2046, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2047, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2048, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2049, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2050, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2051, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1346, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2052, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2053, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2054, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2055, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2056, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2057, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2058, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 461, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 309, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 462, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 463, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1, 0, 0, 4294967295, 4294967295, 0, 2061, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 2065, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 801, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 2067,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2068, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 315, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2069, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2070, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2071, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2072, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2073, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2074,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2075, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2076, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2077, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2078, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2079, 1, 2, 1, 1, 4, 3, 1, 4, 5, 969, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2080, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2081,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2083, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 0, 0, 1, 0, 25, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2087, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1432, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2088, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2089, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1197, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2090, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2091,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2092, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2093, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2094, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2095, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2097, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2098, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2099, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1473,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1663, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 891, 1, 2, 1, 1, 4, 3, 1, 4, 5, 999, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2100, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 190, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 2104, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 851, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2105, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1263, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 26, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    0, 2108, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2109, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 943, 870, 587, 1, 870, 2, 3, 4, 943, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 1, 0, 1, 2,
    1, 1, 0, 1, 2, 1, 2110, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    27, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2111, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2112, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2113, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 69, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 954, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1468, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 412, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2114, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2115, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2116, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2117, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2118, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1317, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2119, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2120, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2121, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2122, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1, 2, 1, 0, 1, 2, 1, 2128, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2129, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2130, 1, 2, 1, 1, 4, 3, 1, 4, 5, 271, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2131, 1, 2, 1, 1, 4, 3, 1, 4, 5, 28, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1474, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 60, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2132, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2133, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2134, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2135, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2136, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2137, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 293, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 294, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2138, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2139, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    413, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2140, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2141, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2142, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 496, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 0, 1, 0, 2, 12187, 2, 12189, 3, 611, 1180, 611, 306, 1180,
    0, 112, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1,
    1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 2177, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2178, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1151, 2179, 237, 1, 2179, 3, 3, 4, 238, 2180, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1429, 1, 2, 1, 1, 4, 3, 1, 4, 5, 29, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 70, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2182, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2183, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 53, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2184, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2185, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 6, 4294967295, 1713, 587, 1, 1713, 2, 3, 4, 2186, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0,
    1, 0, 1, 0, 0, 1, 0, 0, 1, 0, 1, 4294967295, 4294967295, 0, 423, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 0, 0, 0, 0,
    1, 0, 2209, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2210, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1518, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2211, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2212, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 201, 1, 2, 1, 1, 4, 3, 1, 4, 5, 378, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2213, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1, 4294967295, 4294967295, 0, 2214, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2215, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2084, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1056, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    2218, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2034, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1, 0, 242, 1191, 1193, 3, 1191, 1192, 242, 3, 3,
    1192, 4, 242, 1194, 587, 3, 1194, 4, 242, 2, 3, 4, 2220, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2221, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1361, 1, 2, 1, 1, 4, 3, 1, 4, 5, 531, 2222, 237, 1, 2222, 3,
    3, 4, 238, 316, 1, 2, 1, 1, 4, 3, 1, 4, 5, 551, 2223, 237,
    1, 2223, 3, 3, 4, 238, 1349, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2224, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2225, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2226, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2227, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1993, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2228, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2229, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2230, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2231, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2232, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2233, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2234, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2235, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2236, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2237, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2238, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2239, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2240, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2241, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2242, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2243, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2244, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2245, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2246, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2247, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2248, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2249, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2250, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2251, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2252, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2253, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2254, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2255, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2256, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2257, 1, 2, 1, 1, 4, 3, 1, 4, 5, 435, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 880, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2258, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2259, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2260, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    493, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2261, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2262, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2263, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2145, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2264, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2265, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2266, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2146, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2147, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2148, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2267, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2268, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2269, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2149, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2270, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2150, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2271, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2272, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2273, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2274, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2275, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2276, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2277, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2278, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2279, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2280, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2281, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2282, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2283, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1111, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1116, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1113, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2284, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 787, 1, 2, 1, 1, 4, 3, 1, 4, 5, 972, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2285, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1465, 1, 2, 1, 1, 4, 3, 1, 4, 5, 30, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 71, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    528, 2286, 237, 1, 2286, 3, 3, 4, 238, 916, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1615, 1, 2, 1, 1, 4, 3, 1, 4, 5, 300, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2287, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2288, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 883, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2433, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2434, 1, 2, 1, 1, 4, 3, 1, 4, 5, 414, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 415, 1, 2, 1, 1, 4, 3, 1, 4, 5, 416, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1352, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2435, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2436, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2437, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2438, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2439, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2440, 1, 2, 1, 1, 4, 3, 1, 4, 5, 783, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2151, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 31, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2441, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 72, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2442, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 2444, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2445, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 1, 2, 1, 2446, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2447, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1475, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 973, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2448, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2449, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2450, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2451, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2452, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2453, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 802, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2454, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2455, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2456, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2457, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2458, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2459, 1, 2, 1, 1, 4, 3, 1, 4, 5, 238, 237,
    9, 3, 3, 4, 238, 2460, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2461,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2462, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2463, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2464, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2124, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 7, 4294967295, 2465, 587, 1, 2465, 2, 3, 4, 2466, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 0,
    0, 0, 0, 0, 1, 2, 1, 1, 2, 1, 0, 32, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
    0, 1, 2, 1, 0, 1, 0, 1, 0, 0, 2491, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1476, 1, 2, 1, 1, 4, 3, 1, 4, 5, 73, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1408, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 33, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2492, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2493, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2494, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295,
    4294967295, 0, 34, 1, 2, 1, 1, 4, 3, 1, 4, 5, 74, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 113, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2498, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 114, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 488, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2502, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1607, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2503, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2504, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2005, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1544, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2505, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1320, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1152, 2179, 237, 1, 2179, 3, 3, 4, 238, 1051, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 0, 4294967295, 4294967295, 0, 790, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1073, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1074, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2508, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2509, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2510, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2511, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2512,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2513, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 89, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2514, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2515, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2516, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2517, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2518, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1423,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2519, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2202, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 2526, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2527, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2528, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2529, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2530, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2531, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2499, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2532, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2533, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2534, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2535, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2536, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 115, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2537, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2538, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2539, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2540, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2541, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 645, 1, 2, 1, 1, 4, 3, 1, 4, 5, 645, 1313, 1314,
    1, 1313, 5, 3, 1, 564, 4, 5, 116, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2545, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2546, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 35, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2547, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2549,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2550, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2551, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1006, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2205, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 489, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2555, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1104, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2556, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2557, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2558, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2559, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2560, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 417, 1, 2, 1, 1, 4, 3, 1, 4, 5, 436, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2561, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2562, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1008, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2563, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2564,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 2565, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2566, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1057, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2063, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2567, 1, 2, 1, 1, 4, 3, 1, 4, 5, 117,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 2568, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 735, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2569, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 118, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 119, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    120, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2059, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2570, 1, 2, 1, 1, 4, 3, 1, 4, 5, 123, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2571, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2572, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2573, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2574, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1354, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2575, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 868, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2576, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2577, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2578, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2579, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2580, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2581, 1, 2, 1, 1, 4, 3, 1, 4, 5, 484, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2582, 1, 2, 1, 1, 4, 3, 1, 4, 5, 500, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 1, 2, 1, 1,
    0, 501, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2583, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2584, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2585,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2586, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2587, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2588, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2589, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2590, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2591, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1062, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2592,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2593, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2594, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2595, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2596, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 36, 1, 2, 1, 1, 4, 3, 1, 4, 5, 75, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 784, 1, 2, 1, 1, 4, 3, 1, 4, 5, 426,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2597, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2598, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2599, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1280, 2600, 237, 1, 2600, 3, 3, 4, 238,
    37, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2601, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2602, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2603, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 295, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2604, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2605, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 227, 2606, 1, 685, 2607, 1, 245, 2607, 1, 245,
    2606, 1, 687, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2608, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 0, 1139, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2609, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    4294967295, 4294967295, 0, 0, 2611, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2612, 2613,
    12, 3, 3, 4, 2612, 2614, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 4294967295, 4294967295, 0, 1385, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    227, 2621, 1, 1, 0, 245, 2621, 1, 227, 2621, 1, 1, 0, 245, 2621, 1,
    478, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2626, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 481, 1, 2, 1, 1, 4, 3, 1, 4, 5, 5, 1438,
    9, 3, 3, 4, 5, 2521, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2627,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 57, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 121, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2628, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 974, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2629, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2630, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2631, 1, 2, 1, 1, 4, 3, 1, 4, 5, 613,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2632, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 296, 1, 2, 1, 1, 4, 3, 1, 4, 5, 297, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1066, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2507, 1, 2, 1, 1, 4, 3, 1, 4, 5,
];

pub static AUX_DATA_OFFSETS: &[u32] = &[
    0, 10, 20, 30, 46, 9, 0, 42, 0, 38, 39, 41, 42, 43, 44, 45,
    46, 47, 48, 49, 50, 0, 51, 40, 54, 0, 55, 56, 0, 58, 59, 61,
    42, 0, 63, 59, 61, 42, 64, 0, 58, 66, 61, 46, 0, 63, 66, 61,
    41, 42, 46, 44, 43, 77, 0, 79, 81, 84, 86, 88, 90, 0, 93, 94,
    96, 52, 0, 122, 47, 48, 124, 41, 42, 44, 0, 125, 126, 127, 0, 129,
    130, 0, 131, 132, 0, 39, 48, 135, 136, 0, 51, 0, 122, 49, 46, 0,
    126, 127, 138, 64, 0, 139, 50, 46, 47, 0, 140, 141, 125, 143, 122, 144,
    129, 145, 146, 39, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 136,
    158, 159, 0, 58, 141, 160, 53, 161, 163, 55, 165, 166, 167, 168, 169, 170,
    171, 172, 130, 46, 0, 126, 127, 0, 173, 166, 0, 174, 165, 0, 175, 176,
    56, 178, 179, 180, 181, 182, 183, 184, 185, 186, 59, 0, 126, 187, 0, 173,
    179, 0, 174, 178, 0, 175, 188, 57, 191, 193, 70, 0, 196, 58, 77, 0,
    124, 60, 197, 0, 48, 62, 199, 0, 202, 64, 77, 66, 199, 68, 203, 205,
    71, 208, 75, 85, 95, 105, 115, 125, 135, 145, 0, 155, 165, 175, 185, 195,
    205, 223, 215, 0, 227, 230, 246, 0, 233, 243, 234, 225, 234, 241, 243, 245,
    247, 0, 248, 250, 257, 267, 277, 0, 255, 287, 42, 46, 256, 288, 263, 0,
    255, 265, 268, 77, 0, 163, 270, 292, 274, 275, 276, 293, 298, 295, 0, 294,
    301, 302, 276, 0, 163, 303, 305, 307, 0, 310, 88, 77, 300, 313, 314, 0,
    314, 163, 299, 317, 318, 319, 0, 323, 324, 301, 0, 313, 0, 326, 327, 328,
    311, 233, 241, 243, 326, 0, 326, 331, 328, 332, 0, 326, 334, 328, 335, 0,
    149, 336, 328, 312, 326, 96, 316, 0, 347, 348, 96, 350, 351, 314, 0, 352,
    96, 353, 315, 354, 355, 356, 0, 255, 171, 357, 317, 359, 0, 362, 363, 365,
    233, 323, 0, 368, 328, 321, 326, 372, 373, 374, 0, 375, 328, 322, 326, 379,
    380, 381, 329, 0, 382, 328, 324, 383, 384, 0, 325, 385, 146, 363, 386, 327,
    0, 326, 389, 390, 391, 160, 393, 0, 310, 328, 394, 88, 248, 0, 255, 165,
    248, 356, 355, 395, 419, 421, 0, 385, 363, 422, 386, 330, 389, 0, 390, 391,
    160, 0, 255, 363, 233, 334, 332, 0, 424, 326, 331, 383, 276, 0, 362, 326,
    0, 424, 333, 427, 428, 0, 146, 363, 386, 338, 389, 0, 390, 391, 160, 0,
    385, 429, 0, 439, 363, 391, 386, 340, 0, 339, 389, 0, 390, 391, 0, 362,
    343, 0, 363, 391, 0, 341, 440, 441, 442, 444, 0, 445, 77, 0, 342, 446,
    447, 77, 0, 255, 344, 446, 447, 77, 345, 449, 0, 305, 449, 0, 255, 440,
    442, 346, 428, 427, 0, 310, 349, 0, 347, 394, 88, 248, 0, 348, 440, 441,
    442, 88, 442, 0, 363, 391, 88, 391, 0, 350, 314, 313, 450, 0, 453, 88,
    351, 263, 0, 358, 385, 362, 464, 465, 156, 466, 467, 468, 469, 446, 470, 471,
    472, 473, 359, 476, 477, 365, 233, 96, 58, 362, 360, 77, 49, 0, 479, 165,
    480, 366, 479, 482, 483, 0, 486, 369, 367, 77, 0, 255, 0, 368, 328, 372,
    490, 491, 492, 265, 0, 494, 495, 373, 498, 499, 166, 248, 160, 394, 377, 374,
    502, 503, 378, 388, 397, 407, 0, 469, 417, 511, 512, 165, 424, 0, 418, 248,
    0, 248, 157, 513, 163, 420, 391, 0, 479, 96, 515, 0, 248, 362, 513, 163,
    422, 391, 425, 435, 445, 473, 455, 464, 474, 484, 494, 504, 514, 524, 538, 539,
    540, 541, 542, 543, 544, 545, 546, 547, 548, 549, 546, 526, 525, 96, 276, 550,
    528, 537, 0, 549, 561, 559, 556, 0, 0, 394, 564, 574, 584, 594, 604, 614,
    624, 634, 644, 655, 665, 675, 685, 695, 705, 715, 0, 573, 725, 735, 745, 755,
    765, 775, 785, 795, 805, 821, 830, 383, 588, 589, 203, 391, 276, 0, 592, 383,
    594, 595, 596, 86, 394, 831, 837, 842, 852, 862, 872, 882, 892, 902, 912, 922,
    616, 617, 618, 619, 620, 621, 923, 930, 940, 950, 960, 970, 980, 990, 1000, 1010,
    1020, 632, 0, 633, 633, 634, 0, 635, 635, 1030, 1039, 1049, 1059, 1069, 1079, 1089,
    641, 642, 1092, 1090, 163, 646, 1091, 647, 648, 1093, 1103, 650, 651, 1104, 1116, 1114,
    383, 1121, 1119, 391, 1131, 1129, 0, 1127, 391, 0, 391, 653, 1124, 276, 1128, 656,
    657, 1134, 1140, 660, 1143, 1153, 1163, 1173, 1183, 1193, 1203, 1213, 1223, 1233, 1243, 1253,
    1263, 1273, 234, 0, 233, 673, 394, 1276, 502, 1279, 1289, 1299, 0, 677, 203, 1309,
    1319, 1329, 1339, 1349, 1367, 0, 684, 685, 1359, 688, 690, 692, 694, 692, 692, 0,
    684, 276, 1363, 688, 690, 692, 694, 692, 692, 619, 86, 696, 1368, 1378, 1388, 1397,
    1407, 1417, 1427, 1437, 1447, 1457, 1467, 1477, 1487, 708, 1497, 1507, 1517, 1527, 1537, 1547,
    1557, 0, 1567, 716, 0, 305, 716, 0, 45, 494, 1568, 0, 720, 494, 0, 721,
    494, 0, 722, 494, 55, 0, 723, 494, 55, 0, 494, 495, 1569, 498, 499, 1570,
    1580, 1590, 0, 726, 424, 727, 1591, 1601, 0, 446, 165, 1611, 248, 1613, 0, 733,
    734, 736, 737, 738, 739, 740, 1617, 1627, 1637, 0, 743, 165, 1647, 248, 1649, 736,
    739, 1650, 1660, 1670, 1680, 1690, 1700, 1710, 1720, 1730, 1740, 1750, 1760, 1770, 1780, 1790,
    0, 331, 332, 1796, 1806, 1818, 335, 1816, 233, 758, 760, 1819, 1829, 1839, 1849, 1859,
    1869, 1878, 1887, 1900, 1897, 1896, 769, 383, 203, 588, 1915, 1911, 0, 0, 447, 77,
    1904, 0, 305, 772, 0, 446, 1901, 276, 0, 326, 773, 0, 444, 774, 775, 776,
    0, 777, 774, 319, 0, 331, 332, 778, 0, 327, 1905, 233, 241, 243, 326, 779,
    0, 0, 781, 782, 394, 1906, 276, 1909, 785, 786, 0, 788, 96, 1910, 427, 428,
    0, 394, 276, 0, 781, 782, 394, 1912, 276, 0, 233, 1924, 592, 1916, 791, 792,
    0, 1917, 791, 792, 793, 1920, 0, 0, 305, 772, 0, 326, 1918, 794, 588, 96,
    335, 772, 0, 163, 157, 1919, 794, 588, 1925, 0, 1928, 795, 1932, 803, 804, 805,
    806, 807, 808, 809, 0, 1933, 1943, 1952, 233, 758, 760, 1954, 1964, 0, 813, 0,
    0, 1974, 1984, 1993, 2003, 0, 818, 335, 2013, 2023, 2030, 2040, 2050, 2060, 2070, 328,
    825, 826, 827, 828, 829, 830, 831, 832, 2071, 2078, 2088, 2098, 2108, 2114, 2120, 2130,
    2140, 2150, 2160, 0, 634, 634, 2170, 2180, 2190, 845, 2200, 847, 852, 853, 2201, 2211,
    2221, 2231, 2241, 2251, 2261, 2271, 2281, 2291, 2301, 2311, 2321, 2331, 2341, 2351, 2361, 2371,
    2383, 276, 2382, 0, 86, 592, 0, 869, 2381, 708, 384, 2386, 2396, 2406, 2416, 2426,
    2436, 2446, 2456, 2466, 2476, 0, 122, 878, 2486, 592, 394, 592, 479, 879, 2490, 170,
    168, 169, 0, 2489, 884, 885, 886, 887, 79, 81, 0, 733, 77, 0, 394, 888,
    0, 122, 2491, 42, 46, 422, 256, 2492, 889, 0, 122, 265, 268, 422, 77, 163,
    270, 2496, 274, 275, 276, 2497, 298, 2499, 0, 2498, 301, 302, 276, 0, 163, 303,
    305, 307, 2504, 313, 314, 0, 314, 163, 2503, 317, 318, 319, 323, 324, 0, 122,
    265, 168, 79, 81, 0, 892, 592, 319, 893, 895, 0, 122, 2505, 897, 898, 899,
    440, 442, 900, 163, 248, 902, 903, 2508, 0, 368, 328, 2506, 326, 372, 373, 906,
    374, 0, 375, 328, 2507, 326, 379, 380, 381, 2512, 0, 122, 907, 908, 422, 77,
    0, 911, 2530, 2528, 0, 382, 328, 2527, 383, 2524, 2522, 335, 394, 502, 0, 326,
    2534, 0, 726, 424, 328, 2533, 384, 335, 0, 331, 328, 332, 0, 334, 328, 335,
    0, 122, 265, 169, 422, 79, 912, 319, 893, 895, 0, 122, 48, 422, 199, 0,
    122, 46, 422, 914, 2537, 0, 347, 348, 96, 0, 917, 479, 96, 350, 351, 2535,
    0, 352, 96, 353, 2536, 354, 355, 356, 0, 122, 171, 357, 2538, 918, 0, 122,
    907, 154, 174, 165, 422, 248, 2573, 0, 2545, 2543, 2542, 429, 386, 922, 0, 924,
    328, 925, 0, 2548, 0, 934, 441, 0, 935, 441, 0, 897, 441, 2551, 2549, 391,
    0, 936, 348, 2556, 2554, 391, 319, 937, 939, 942, 0, 945, 2561, 2559, 0, 335,
    328, 335, 0, 946, 248, 0, 0, 924, 328, 925, 319, 0, 945, 2566, 2564, 0,
    335, 328, 335, 394, 2572, 2569, 502, 0, 122, 947, 265, 155, 422, 948, 2574, 950,
    390, 2575, 145, 157, 144, 163, 248, 0, 733, 951, 953, 0, 2576, 957, 958, 233,
    2577, 324, 0, 959, 324, 692, 0, 122, 124, 422, 2586, 0, 199, 0, 2584, 0,
    961, 2583, 0, 348, 2578, 0, 963, 964, 335, 0, 479, 77, 0, 2582, 0, 348,
    2579, 0, 963, 964, 335, 0, 394, 335, 0, 966, 967, 961, 2581, 0, 348, 2580,
    0, 963, 964, 335, 0, 479, 77, 394, 335, 968, 0, 326, 48, 2588, 199, 0,
    970, 2593, 379, 2591, 2590, 975, 976, 0, 977, 335, 0, 978, 335, 0, 979, 335,
    968, 0, 479, 2595, 0, 2594, 984, 985, 986, 987, 96, 0, 2604, 0, 964, 990,
    0, 2599, 326, 993, 0, 994, 96, 995, 0, 964, 996, 2600, 326, 96, 0, 964,
    998, 994, 2601, 326, 0, 96, 995, 0, 964, 970, 1000, 0, 2602, 326, 1003, 0,
    1004, 96, 0, 1005, 2603, 96, 1007, 0, 9, 2608, 1009, 375, 0, 2609, 818, 1010,
    335, 0, 122, 907, 0, 368, 328, 2610, 490, 491, 492, 265, 0, 494, 495, 2611,
    498, 499, 166, 248, 160, 394, 2615, 2612, 502, 503, 2616, 2626, 2636, 2646, 2656, 2666,
    2676, 2686, 2694, 0, 2704, 1018, 1019, 2705, 1020, 2708, 2718, 2728, 96, 2731, 1023, 1024,
    1025, 1026, 0, 1027, 1028, 2732, 2742, 2752, 2762, 2772, 2782, 2792, 2802, 2812, 0, 1037,
    0, 1039, 90, 384, 486, 2823, 2822, 594, 595, 2825, 0, 2835, 1042, 1043, 2836, 233,
    1044, 2837, 2847, 2857, 2867, 2877, 2887, 2906, 2897, 233, 384, 2912, 1050, 0, 1053, 1054,
    0, 2907, 1058, 1059, 1060, 0, 2910, 0, 2908, 726, 708, 2909, 1063, 0, 93, 94,
    0, 0, 233, 1064, 1065, 793, 2911, 1067, 1058, 1059, 1068, 0, 1068, 1059, 0, 1059,
    1068, 1069, 2913, 2926, 1075, 0, 2938, 2936, 2935, 298, 1076, 0, 2941, 1077, 1078, 792,
    2953, 2942, 96, 2954, 2984, 2993, 3003, 3013, 3023, 3033, 3043, 3053, 3063, 3073, 3083, 3093,
    3103, 3113, 3123, 3133, 3143, 3153, 0, 0, 3163, 1093, 362, 1094, 384, 3164, 3174, 3184,
    3194, 3204, 3219, 0, 1101, 384, 1102, 733, 324, 0, 1101, 3214, 1105, 1106, 1107, 1108,
    733, 3216, 1110, 1112, 0, 305, 1114, 0, 1117, 1118, 335, 3215, 335, 96, 384, 0,
    324, 0, 1101, 384, 1119, 733, 3217, 335, 96, 0, 1101, 595, 592, 0, 326, 3218,
    335, 96, 203, 3220, 3230, 0, 326, 1121, 3240, 3250, 3260, 3270, 3280, 3290, 0, 498,
    328, 199, 3300, 3310, 3320, 3330, 3340, 0, 144, 1130, 736, 1132, 3354, 0, 90, 3350,
    1134, 869, 1136, 1138, 0, 3352, 1134, 90, 1141, 1138, 0, 1141, 1143, 1138, 223, 1145,
    0, 248, 0, 1064, 1148, 0, 869, 3355, 1149, 3357, 1153, 1154, 3358, 1156, 3368, 3378,
    3388, 3398, 3408, 3418, 3428, 3438, 3448, 3458, 0, 1166, 384, 77, 3468, 3478, 3488, 3498,
    3508, 3518, 3528, 3538, 3548, 3558, 3568, 3578, 3588, 3598, 3608, 3618, 3628, 3638, 3648, 3658,
    3668, 3678, 0, 1185, 348, 3695, 0, 1187, 3690, 3688, 391, 0, 515, 3693, 96, 1188,
    0, 963, 0, 515, 3694, 96, 1188, 3696, 3706, 3716, 3725, 3735, 3745, 3755, 3764, 3774,
    3785, 3795, 3806, 1198, 1199, 0, 362, 878, 3807, 592, 394, 592, 1200, 0, 362, 42,
    365, 256, 1200, 0, 362, 268, 365, 77, 0, 362, 168, 365, 79, 0, 362, 440,
    442, 163, 248, 3812, 0, 368, 328, 3810, 326, 372, 373, 0, 375, 328, 3811, 326,
    379, 380, 381, 3813, 0, 362, 908, 365, 77, 0, 362, 3823, 169, 168, 365, 77,
    0, 362, 48, 365, 233, 0, 362, 46, 365, 914, 1200, 0, 362, 171, 357, 0,
    362, 165, 365, 248, 0, 1202, 0, 362, 155, 365, 948, 0, 362, 50, 365, 77,
    1200, 0, 362, 124, 365, 199, 0, 362, 166, 365, 248, 1200, 3824, 3834, 3843, 3853,
    3863, 3873, 3883, 3893, 3903, 3913, 3923, 3933, 3943, 0, 690, 276, 3953, 3963, 3973, 3983,
    0, 1216, 1218, 3986, 3996, 4006, 4016, 4026, 4036, 4046, 4056, 4066, 4075, 4085, 4095, 4105,
    4123, 4132, 4142, 4152, 4162, 4172, 1233, 4182, 4192, 4202, 4212, 4222, 4232, 4242, 4252, 4262,
    4272, 4282, 4292, 4302, 4312, 4322, 4332, 0, 129, 384, 0, 869, 4342, 594, 4344, 4354,
    4364, 4374, 4384, 4394, 4404, 4414, 4424, 0, 1251, 4434, 502, 1252, 1254, 1256, 4435, 1259,
    0, 1260, 4446, 4445, 1261, 0, 696, 1260, 1264, 1265, 0, 1266, 305, 1267, 1268, 1269,
    779, 0, 305, 1270, 1271, 968, 1260, 0, 1272, 1259, 4450, 1274, 0, 1275, 1154, 1276,
    1277, 0, 1278, 4453, 4452, 0, 4451, 616, 617, 619, 1278, 4457, 1274, 0, 1275, 1154,
    0, 0, 594, 1279, 4466, 0, 1281, 4458, 502, 0, 4461, 1282, 1283, 4462, 1285, 769,
    86, 0, 4481, 588, 203, 4471, 4470, 1287, 276, 502, 4468, 4467, 383, 391, 203, 1288,
    276, 1290, 1291, 0, 4474, 1216, 1292, 4477, 4475, 276, 0, 592, 4480, 335, 96, 1294,
    1295, 1296, 1288, 1297, 86, 1298, 4482, 594, 1299, 1301, 383, 589, 1303, 1305, 391, 1307,
    4483, 0, 4493, 4503, 4513, 4523, 4533, 4543, 4553, 4563, 4574, 4584, 4594, 0, 1037, 4604,
    389, 1039, 4605, 96, 276, 953, 4606, 953, 1318, 4607, 336, 0, 479, 1319, 4608, 4618,
    4628, 0, 4638, 1322, 4646, 324, 4642, 1322, 4648, 1322, 4652, 4662, 0, 1326, 4672, 0,
    1327, 276, 4675, 4685, 4695, 4705, 4715, 4725, 4735, 4745, 4755, 4765, 4775, 0, 1337, 4785,
    511, 512, 4793, 4787, 4786, 0, 708, 1338, 0, 924, 1338, 0, 1340, 1338, 0, 1342,
    1338, 1343, 1338, 141, 1345, 0, 1347, 1338, 0, 733, 1348, 77, 0, 1350, 1338, 1351,
    1353, 0, 178, 0, 4789, 248, 479, 0, 479, 149, 375, 0, 178, 0, 4791, 248,
    733, 0, 733, 1355, 4794, 4804, 4814, 4828, 0, 0, 733, 4824, 157, 202, 0, 1359,
    4825, 384, 4827, 1362, 0, 1363, 1364, 0, 375, 61, 202, 1366, 4829, 4839, 0, 51,
    441, 4849, 4859, 4869, 4879, 4889, 0, 1371, 4902, 4901, 0, 1000, 953, 0, 4899, 96,
    589, 1007, 391, 4900, 1373, 1374, 0, 1270, 968, 4903, 1318, 1375, 4904, 4914, 4924, 4934,
    0, 90, 4944, 1149, 4946, 1327, 1145, 0, 196, 348, 0, 1386, 348, 1387, 1388, 1389,
    1390, 1392, 1394, 1037, 1396, 1397, 1390, 1392, 503, 1399, 4959, 0, 1401, 4950, 4947, 1402,
    1404, 250, 4951, 1406, 1407, 1410, 1411, 0, 479, 1396, 250, 1407, 1413, 4956, 0, 4952,
    1411, 4978, 4975, 0, 4963, 1414, 1417, 4960, 1149, 0, 1386, 348, 0, 196, 348, 4964,
    4968, 0, 4967, 1411, 1410, 0, 1386, 348, 0, 196, 348, 4972, 4979, 4989, 4999, 5009,
    5019, 276, 5035, 276, 0, 276, 394, 592, 0, 1424, 276, 90, 90, 276, 0, 5029,
    298, 276, 90, 276, 0, 1425, 5032, 1148, 5030, 1426, 1427, 0, 1430, 383, 0, 5033,
    335, 233, 391, 61, 5034, 335, 233, 391, 1431, 1433, 1435, 0, 5039, 1076, 1436, 5042,
    5052, 0, 5062, 0, 233, 1064, 659, 5066, 1439, 1044, 659, 5069, 0, 382, 5067, 0,
    1440, 50, 592, 5068, 0, 1440, 50, 592, 5072, 5070, 1441, 5076, 0, 1443, 1444, 0,
    1446, 1447, 0, 1448, 1449, 0, 1450, 1451, 1452, 0, 0, 79, 5075, 1450, 1453, 1299,
    1454, 1455, 5077, 5087, 5097, 5107, 5117, 5127, 5137, 5147, 5157, 5167, 0, 1463, 0, 1000,
    1464, 1466, 5179, 0, 5177, 594, 595, 328, 5178, 1469, 1470, 1119, 5181, 594, 595, 96,
    5184, 0, 5182, 594, 595, 328, 5183, 1477, 1478, 1479, 1480, 1481, 1482, 1483, 1484, 1485,
    1486, 1487, 1488, 1489, 5186, 5196, 5206, 5216, 5226, 5237, 5247, 5257, 5267, 0, 1497, 5279,
    0, 1498, 163, 1500, 5277, 199, 88, 5280, 199, 5282, 0, 479, 1497, 1502, 0, 479,
    1504, 1502, 5283, 5293, 5303, 0, 635, 1233, 635, 5313, 0, 196, 348, 5326, 58, 1508,
    1511, 0, 5324, 5323, 391, 96, 276, 1513, 0, 479, 1514, 5327, 0, 1386, 348, 1145,
    1387, 1388, 1389, 1037, 5328, 5338, 0, 5352, 5351, 391, 96, 276, 5348, 1513, 5354, 0,
    1516, 1517, 5367, 5365, 5364, 1511, 1020, 5370, 5380, 5390, 5400, 5410, 0, 1387, 5423, 5420,
    276, 5424, 0, 1386, 348, 1145, 1388, 1389, 1037, 5425, 5435, 5445, 5455, 5465, 0, 1526,
    335, 5475, 5485, 5495, 5505, 5515, 5525, 5535, 5545, 5555, 5565, 0, 5575, 5585, 5595, 5605,
    5615, 5625, 5635, 5645, 5655, 5665, 5675, 5685, 5692, 5702, 5712, 0, 1542, 1281, 5722, 0,
    1542, 276, 1543, 324, 0, 1545, 276, 1543, 324, 0, 690, 324, 0, 692, 1542, 5726,
    0, 1542, 305, 1281, 5736, 0, 5746, 1132, 1546, 1547, 5747, 5757, 5767, 5777, 5787, 5797,
    5807, 5817, 5827, 0, 305, 61, 5843, 5840, 5839, 5837, 1259, 502, 203, 5844, 5854, 5864,
    5874, 5884, 0, 233, 5894, 656, 657, 5895, 96, 0, 440, 422, 442, 5900, 5898, 391,
    0, 869, 5903, 1560, 1561, 1562, 0, 945, 5906, 5904, 0, 335, 328, 335, 0, 818,
    335, 0, 5909, 139, 1132, 1202, 5910, 440, 441, 0, 733, 5911, 1564, 0, 1386, 348,
    0, 196, 348, 5912, 77, 1565, 5915, 5926, 0, 1567, 328, 96, 900, 0, 479, 1568,
    77, 319, 5925, 428, 427, 0, 1570, 328, 335, 0, 1571, 328, 335, 0, 869, 5930,
    1572, 1187, 5931, 5941, 5951, 5961, 5971, 5981, 5991, 6001, 6011, 6021, 6031, 6041, 6050, 6060,
    6070, 6080, 6090, 6099, 6108, 6118, 0, 394, 233, 6128, 234, 0, 145, 486, 248, 0,
    476, 6133, 6131, 233, 0, 479, 1587, 233, 160, 6136, 1299, 502, 6137, 6147, 6157, 6167,
    6177, 6187, 6197, 6207, 6217, 6227, 6237, 6247, 6257, 6267, 6277, 6287, 6297, 6307, 6317, 0,
    994, 6327, 298, 0, 276, 298, 6328, 0, 486, 6346, 6344, 6343, 6338, 594, 595, 0,
    1602, 335, 0, 1604, 335, 0, 726, 424, 384, 0, 6342, 1605, 1606, 0, 1608, 348,
    335, 0, 1610, 1611, 348, 335, 0, 1612, 348, 335, 0, 1614, 0, 1616, 348, 335,
    0, 1608, 348, 335, 1617, 6347, 6357, 6367, 6377, 6387, 6397, 6407, 6417, 6427, 6437, 6447,
    1295, 1624, 1626, 1294, 1627, 0, 1266, 305, 1425, 90, 6448, 0, 6458, 6468, 6478, 6488,
    6498, 6508, 6518, 0, 1634, 233, 6528, 6538, 6559, 0, 1637, 1638, 1417, 6548, 1639, 0,
    6552, 1641, 6555, 1565, 1643, 6556, 0, 1644, 1638, 1417, 1641, 0, 1645, 1417, 6560, 0,
    1564, 0, 0, 163, 6570, 6576, 6573, 276, 6577, 6581, 1647, 0, 6580, 1648, 1649, 1650,
    1651, 0, 869, 6584, 6582, 233, 6587, 0, 1606, 6607, 6605, 6604, 0, 233, 733, 1654,
    0, 1657, 1658, 335, 1660, 0, 233, 592, 6603, 0, 1281, 1658, 335, 0, 1658, 335,
    6599, 0, 6597, 772, 1340, 0, 326, 335, 163, 6598, 1662, 1340, 6610, 276, 86, 335,
    1660, 6613, 6628, 276, 86, 276, 0, 1664, 592, 0, 726, 424, 384, 6624, 0, 6622,
    772, 1340, 0, 326, 276, 163, 6623, 1662, 1340, 6631, 6641, 6651, 6661, 6671, 6681, 6691,
    6701, 6711, 6721, 6731, 6741, 6749, 6759, 6769, 6779, 6789, 6799, 6809, 6819, 6829, 6839, 0,
    6849, 6859, 6869, 6879, 6889, 6899, 6909, 6919, 6929, 6939, 6949, 6959, 6969, 0, 633, 1233,
    633, 6979, 0, 0, 305, 1689, 1259, 0, 1690, 1260, 6989, 946, 1691, 1692, 6990, 7000,
    0, 1693, 7015, 7012, 7011, 96, 276, 58, 7017, 0, 1396, 7016, 96, 276, 0, 86,
    96, 7018, 7028, 7038, 7048, 7058, 0, 7068, 0, 476, 422, 77, 1121, 61, 7086, 7084,
    7083, 7080, 7078, 383, 383, 7089, 7099, 335, 96, 1294, 1275, 1295, 1296, 1699, 1701, 1703,
    1706, 1707, 7100, 7109, 7116, 0, 1710, 55, 7126, 1130, 1711, 0, 512, 1712, 335, 7127,
    805, 1132, 486, 165, 248, 1143, 0, 726, 424, 384, 0, 7128, 1605, 1606, 0, 1608,
    348, 335, 0, 0, 1610, 1611, 348, 335, 0, 1612, 348, 335, 0, 1614, 0, 1616,
    348, 335, 0, 1608, 348, 335, 0, 1132, 96, 7129, 1614, 1318, 0, 7132, 7130, 391,
    0, 424, 1271, 7135, 0, 7145, 7155, 7164, 7174, 7184, 7194, 7204, 7214, 7224, 7234, 7244,
    7254, 7264, 7274, 7284, 7294, 7304, 7316, 0, 0, 233, 1724, 1725, 7314, 324, 0, 692,
    1725, 233, 7317, 7327, 7337, 0, 1728, 1064, 233, 7350, 0, 908, 77, 0, 86, 7347,
    502, 7353, 7363, 7373, 0, 7383, 7393, 7403, 7413, 7423, 7433, 7443, 7453, 7463, 7473, 7483,
    7493, 7503, 7513, 7523, 7533, 7543, 7553, 7563, 7573, 7583, 7593, 7603, 7613, 7623, 7633, 7643,
    0, 7653, 7663, 7673, 7683, 7693, 7703, 7713, 7723, 7733, 7743, 7753, 7763, 7773, 7783, 7793,
    7803, 7813, 7823, 7833, 7843, 7853, 7863, 7873, 7883, 7893, 144, 0, 145, 160, 1299, 0,
    491, 486, 7903, 7913, 1773, 1775, 0, 1777, 1778, 0, 968, 276, 1543, 7917, 1779, 1781,
    0, 1777, 305, 1778, 0, 968, 276, 1543, 1783, 0, 1785, 7918, 248, 1786, 869, 7922,
    248, 1786, 0, 7919, 502, 250, 0, 203, 250, 7923, 1565, 7926, 1787, 0, 157, 1789,
    7929, 7939, 7949, 7959, 7969, 7979, 7989, 7999, 8009, 8019, 8029, 8039, 8049, 8058, 8068, 8078,
    8088, 8098, 8108, 8118, 8128, 8138, 8148, 8158, 8168, 8178, 8188, 8198, 8207, 8217, 8227, 8237,
    8247, 8257, 8267, 8277, 8286, 8296, 8306, 8316, 8326, 8336, 8346, 8356, 8366, 0, 233, 394,
    8379, 233, 8376, 1812, 0, 1389, 8380, 1814, 8382, 8392, 8402, 8413, 0, 1820, 847, 8423,
    8433, 8443, 8453, 8463, 8473, 8483, 0, 8492, 8502, 8512, 8522, 8532, 8542, 8549, 8559, 8569,
    8579, 8589, 8599, 8609, 8619, 8629, 8639, 8649, 8659, 8669, 8679, 8689, 8699, 8709, 8719, 8738,
    1299, 1844, 8729, 1301, 8732, 1845, 8735, 1847, 1848, 0, 8739, 1254, 1252, 1256, 1849, 8744,
    1851, 8743, 8740, 1301, 1847, 8745, 8755, 8765, 8775, 8785, 0, 8795, 8805, 0, 8815, 0,
    1857, 1233, 0, 633, 635, 8816, 0, 8844, 8826, 8836, 8845, 8855, 8865, 8875, 8885, 8895,
    8905, 8915, 8925, 0, 633, 1233, 635, 8936, 8946, 8956, 8966, 8976, 8986, 8997, 9007, 0,
    335, 1724, 556, 9028, 9017, 1868, 9019, 9031, 1064, 0, 1064, 1064, 9032, 163, 394, 869,
    86, 1871, 1872, 1873, 696, 1724, 1322, 1410, 9033, 9043, 9053, 9063, 9073, 9083, 9093, 0,
    1396, 9103, 96, 276, 953, 9104, 953, 1318, 9105, 9115, 9125, 9135, 9145, 9155, 9165, 9175,
    9185, 9195, 9205, 9215, 9225, 9235, 9245, 9255, 9265, 9275, 0, 470, 9285, 511, 512, 165,
    9286, 248, 9288, 9298, 9308, 9318, 9328, 9338, 9375, 9348, 9357, 9366, 0, 1892, 805, 0,
    1386, 348, 9379, 0, 9376, 391, 96, 276, 9377, 656, 657, 0, 1547, 9378, 389, 1893,
    1894, 1145, 1371, 9381, 1145, 1387, 1388, 1389, 1371, 1430, 1037, 9382, 9392, 1386, 1896, 9393,
    9403, 9413, 9423, 9433, 9443, 9453, 9463, 9473, 0, 1431, 1902, 9485, 233, 9482, 1812, 9486,
    0, 1904, 9499, 0, 9496, 1296, 86, 1296, 391, 9500, 9510, 9520, 9530, 9540, 9550, 9560,
    9570, 9580, 9590, 9600, 9609, 9618, 9628, 9638, 9648, 9658, 9668, 9678, 9688, 9698, 9708, 9718,
    0, 476, 348, 9733, 9730, 9728, 276, 9734, 9772, 0, 1188, 476, 348, 1375, 9744, 203,
    9747, 0, 476, 348, 9771, 0, 1375, 9752, 9750, 391, 9758, 9756, 9755, 1924, 1926, 0,
    1928, 9763, 9761, 391, 9768, 9766, 1929, 9773, 9783, 9793, 9803, 9813, 9823, 9833, 9843, 9853,
    9875, 0, 1936, 9873, 9863, 1936, 335, 0, 9876, 9886, 9896, 9906, 9916, 9926, 9936, 9946,
    9956, 9966, 9976, 9986, 9996, 10006, 10016, 10025, 10035, 10045, 10055, 10065, 10075, 10085, 10095, 10105,
    10114, 10124, 10134, 10144, 10154, 10164, 10173, 1964, 1966, 10174, 1967, 1143, 10175, 10185, 0, 10195,
    10205, 10215, 10225, 10235, 10245, 0, 1094, 384, 90, 10255, 335, 594, 595, 10256, 10266, 10276,
    0, 1975, 441, 10286, 10296, 10306, 10316, 10326, 10336, 10346, 10356, 10367, 0, 10366, 1980, 1981,
    1982, 382, 592, 592, 10370, 10368, 1983, 10373, 10383, 10393, 10403, 10413, 10423, 10433, 10443, 10453,
    0, 1991, 10463, 708, 1992, 1338, 10465, 0, 88, 335, 0, 950, 10464, 276, 10466, 0,
    1274, 96, 10476, 0, 619, 10486, 58, 1440, 1994, 10487, 502, 10490, 10500, 10510, 10520, 10530,
    10540, 10550, 10560, 10570, 10592, 10581, 1706, 10596, 10606, 10616, 10626, 0, 476, 422, 77, 1121,
    10655, 0, 2003, 2004, 10641, 2006, 10638, 10636, 383, 0, 10652, 10644, 10642, 383, 2007, 10649,
    10647, 383, 0, 10658, 10656, 0, 335, 328, 335, 0, 90, 10661, 335, 88, 10664, 335,
    994, 96, 10667, 2008, 2009, 995, 2010, 2011, 2012, 10668, 10678, 10687, 10696, 10706, 10715, 10724,
    10734, 10744, 10754, 10764, 10774, 10784, 10794, 10804, 10814, 10824, 10834, 10844, 10854, 10864, 10874, 10884,
    0, 151, 248, 160, 2030, 10894, 0, 163, 144, 2031, 0, 163, 157, 2031, 10898, 0,
    2033, 10908, 1648, 2035, 2036, 10909, 10919, 10929, 10939, 1198, 1199, 0, 424, 772, 0, 721,
    2038, 0, 424, 326, 10940, 10950, 10960, 10970, 10980, 10990, 11000, 11010, 11020, 11030, 11040, 11050,
    11060, 11070, 11080, 11090, 11100, 11110, 11120, 11130, 11140, 11150, 11160, 11170, 0, 310, 165, 11180,
    0, 248, 88, 248, 11182, 11192, 0, 472, 11202, 511, 512, 165, 11203, 248, 11205, 736,
    739, 2060, 11209, 11223, 0, 0, 233, 1724, 2062, 11219, 324, 0, 2064, 276, 0, 692,
    2062, 233, 11224, 11234, 0, 805, 11244, 1130, 2066, 0, 486, 248, 1143, 11246, 0, 160,
    1299, 1789, 0, 160, 11245, 502, 0, 165, 248, 11247, 11257, 11267, 11277, 11287, 11297, 11307,
    11317, 11327, 11337, 11347, 11357, 11367, 11377, 11387, 11397, 11407, 0, 2082, 1992, 0, 88, 96,
    11417, 0, 11427, 2085, 2086, 11428, 0, 1117, 1118, 335, 384, 0, 424, 11431, 0, 11429,
    1477, 1478, 1479, 1480, 1481, 1482, 1483, 1484, 1485, 1486, 1487, 1488, 1489, 328, 11430, 594,
    595, 335, 11433, 11443, 11453, 11463, 11473, 11483, 11493, 11503, 11513, 11523, 11533, 11543, 0, 2096,
    276, 11553, 11563, 11573, 11583, 11593, 11603, 11613, 11623, 11633, 0, 2101, 0, 1497, 1502, 733,
    11643, 0, 1498, 163, 1500, 0, 48, 199, 0, 148, 163, 124, 197, 197, 90, 2102,
    1200, 11644, 11654, 11664, 11674, 11684, 11696, 0, 11694, 384, 1044, 241, 1706, 0, 2106, 11695,
    384, 1044, 241, 1706, 2107, 11697, 11707, 11717, 11725, 0, 1514, 11747, 11745, 0, 77, 11737,
    11735, 391, 0, 934, 441, 11742, 11740, 391, 11750, 11760, 11770, 11780, 11790, 11800, 11810, 11820,
    11830, 11840, 11850, 11860, 11870, 11880, 11890, 11900, 11910, 11920, 11930, 0, 2123, 11940, 2125, 2126,
    11941, 96, 0, 11944, 2127, 11945, 96, 11948, 11958, 11968, 11978, 11988, 11998, 12008, 12018, 12028,
    12038, 12048, 12058, 12068, 12078, 12088, 12098, 12108, 12118, 12128, 12138, 12148, 12158, 12168, 12178, 2143,
    0, 556, 250, 0, 12179, 1425, 58, 2152, 2153, 2154, 2155, 2156, 2157, 2158, 2159, 2160,
    0, 152, 2161, 12180, 2163, 1617, 486, 12192, 90, 12182, 1327, 0, 1386, 348, 1145, 1904,
    1390, 1037, 12193, 0, 476, 12205, 12203, 77, 0, 2165, 1399, 1141, 1967, 1143, 1138, 2167,
    2169, 1307, 2171, 1397, 223, 1145, 2171, 1399, 1390, 2173, 503, 12219, 12211, 12208, 1845, 12215,
    12212, 2175, 2176, 12216, 502, 12220, 12230, 12240, 12249, 12259, 12269, 12279, 0, 2006, 96, 0,
    721, 2006, 12280, 0, 2181, 96, 0, 721, 2181, 12281, 12291, 12301, 12311, 12321, 12331, 12341,
    12350, 0, 424, 12368, 0, 0, 12360, 2187, 2188, 1064, 12361, 2189, 2190, 2191, 2192, 512,
    12362, 594, 595, 1707, 12363, 328, 1279, 12364, 96, 335, 1706, 594, 1707, 2193, 595, 203,
    616, 276, 0, 424, 12370, 2194, 0, 391, 328, 12372, 383, 588, 203, 391, 276, 1299,
    326, 0, 424, 326, 48, 12375, 58, 379, 12373, 199, 88, 12376, 199, 0, 1847, 12378,
    0, 1390, 1847, 223, 1145, 2171, 12382, 0, 424, 2196, 12392, 326, 335, 384, 0, 331,
    332, 12394, 2197, 0, 12393, 2198, 2199, 58, 2200, 0, 424, 46, 422, 914, 0, 424,
    12395, 2189, 2192, 2201, 12400, 12399, 0, 2203, 2204, 12397, 0, 149, 12396, 2206, 2207, 0,
    2127, 149, 2208, 0, 149, 12398, 159, 336, 12402, 12412, 12422, 12432, 12442, 12452, 12462, 12472,
    0, 12482, 1282, 1283, 12483, 0, 1871, 592, 1285, 12487, 12497, 12507, 12517, 12527, 2216, 2217,
    12528, 12538, 12548, 384, 1044, 594, 1064, 12549, 233, 0, 233, 758, 1148, 12551, 12562, 12572,
    0, 1069, 208, 12582, 12592, 12602, 12611, 12621, 12630, 12640, 12650, 12660, 12670, 12680, 12690, 12700,
    12710, 12720, 12730, 12740, 12750, 12760, 12770, 12780, 12790, 12800, 12810, 12820, 12830, 12840, 12850, 12860,
    12870, 12880, 12890, 12900, 12910, 12920, 12930, 12940, 12950, 12960, 12970, 12980, 12990, 13000, 13010, 13020,
    13030, 13040, 13050, 13060, 13070, 13080, 13090, 13100, 13110, 13120, 13130, 13140, 13150, 13160, 13170, 13180,
    13190, 13200, 13210, 13220, 13230, 13240, 13250, 13260, 13270, 13280, 13290, 13300, 13310, 13320, 13330, 13340,
    13350, 13360, 13370, 13380, 13390, 13400, 13410, 13420, 13430, 13440, 13449, 13459, 13469, 13479, 13489, 13499,
    502, 1849, 1252, 1436, 2289, 2291, 2293, 2295, 2297, 2299, 2301, 2303, 2305, 2307, 2309, 2311,
    2313, 2315, 2317, 2319, 2321, 2323, 1256, 1254, 2325, 2327, 2329, 2331, 2333, 2335, 2337, 2339,
    2341, 2343, 2345, 2347, 2349, 2351, 2353, 2355, 2357, 2359, 2361, 2363, 2365, 2367, 2369, 2371,
    2373, 2375, 2377, 2379, 2381, 2383, 2385, 2387, 2389, 2307, 2391, 2323, 2393, 2395, 2397, 2399,
    2401, 2403, 2405, 2407, 2409, 2173, 2411, 2413, 2415, 2417, 2419, 2421, 2423, 2425, 2427, 2429,
    2431, 1322, 13500, 13510, 13520, 13530, 13540, 13550, 13560, 13570, 13580, 13590, 13600, 13610, 13620, 13630,
    13640, 13650, 13660, 13670, 13680, 13690, 2443, 13691, 13701, 13713, 13711, 0, 556, 250, 0, 13716,
    13726, 13736, 13746, 13756, 13766, 13776, 13786, 13796, 13806, 13816, 13826, 13836, 13846, 13856, 13866, 13876,
    13886, 13893, 13903, 13913, 13923, 13933, 13943, 13953, 13962, 13987, 0, 0, 447, 77, 13983, 0,
    897, 13972, 440, 441, 442, 900, 902, 903, 0, 444, 900, 902, 903, 0, 445, 442,
    160, 151, 391, 160, 13975, 0, 163, 13973, 144, 157, 13974, 1198, 1199, 0, 424, 772,
    0, 721, 2038, 0, 424, 326, 0, 446, 13979, 276, 13982, 449, 0, 305, 449, 0,
    13984, 440, 441, 442, 900, 902, 903, 0, 13985, 898, 899, 13986, 440, 441, 442, 902,
    903, 0, 13994, 1299, 588, 203, 248, 13988, 502, 13991, 1849, 13995, 14021, 14020, 0, 2467,
    328, 96, 0, 573, 328, 96, 0, 2468, 328, 96, 0, 326, 327, 328, 14005, 335,
    384, 326, 0, 743, 328, 96, 0, 326, 331, 328, 14006, 335, 384, 326, 0, 818,
    328, 335, 0, 2469, 328, 335, 0, 2470, 328, 335, 0, 14007, 55, 440, 2471, 328,
    335, 0, 2472, 328, 96, 0, 334, 328, 335, 0, 924, 328, 14008, 335, 384, 0,
    1570, 328, 335, 0, 2473, 328, 14009, 721, 389, 1893, 0, 1567, 328, 96, 0, 2474,
    328, 96, 0, 2475, 328, 96, 0, 2476, 328, 14010, 96, 326, 0, 964, 328, 335,
    0, 2477, 328, 14011, 326, 2478, 2479, 2480, 2481, 2482, 0, 1571, 328, 335, 0, 2483,
    328, 14012, 326, 96, 0, 2484, 328, 14013, 326, 96, 0, 2485, 328, 96, 0, 49,
    384, 0, 2486, 14014, 2487, 2488, 0, 2489, 328, 14017, 14015, 248, 14025, 2490, 14023, 233,
    758, 760, 14026, 14036, 14046, 14056, 14066, 14076, 14086, 14096, 0, 14106, 2495, 696, 2496, 0,
    14110, 2497, 2496, 14114, 14124, 14134, 14144, 0, 14154, 153, 154, 14155, 14165, 2500, 2501, 14166,
    14176, 14186, 14196, 14206, 14216, 14226, 14236, 14246, 14256, 14265, 0, 14275, 1078, 792, 793, 0,
    14276, 479, 482, 1078, 2506, 14277, 0, 274, 1078, 2506, 276, 14281, 14291, 14301, 14311, 14321,
    14331, 14341, 14351, 14361, 14371, 14381, 14391, 14401, 14411, 14421, 14431, 14441, 14451, 14461, 0, 917,
    2201, 0, 0, 233, 1724, 0, 959, 2520, 324, 0, 2522, 233, 0, 2523, 2520, 0,
    968, 721, 2524, 0, 2525, 2520, 0, 692, 233, 14462, 14472, 14482, 14492, 14502, 14512, 14522,
    14532, 14542, 14552, 14562, 14572, 14582, 0, 14592, 14602, 14612, 14622, 14632, 14642, 2542, 2543, 2544,
    14643, 14653, 14664, 0, 156, 165, 248, 14674, 14684, 14696, 14694, 556, 14699, 14709, 0, 1069,
    1699, 0, 1216, 2548, 14719, 14729, 14739, 14749, 14759, 0, 14769, 2552, 2553, 2554, 14770, 14780,
    14790, 14800, 14810, 14820, 14830, 14840, 0, 2489, 14850, 1425, 58, 14851, 0, 897, 441, 14861,
    14871, 0, 14881, 14891, 14901, 14911, 0, 2165, 1399, 1141, 1967, 1143, 1138, 2167, 2169, 1307,
    2171, 1397, 1390, 1392, 1394, 223, 1145, 1399, 1397, 1967, 503, 1143, 2173, 0, 1847, 14921,
    0, 1390, 1847, 14925, 14935, 14945, 14955, 14965, 14975, 0, 157, 1130, 1132, 14986, 14985, 248,
    1149, 1789, 1138, 223, 1145, 14988, 14998, 15008, 0, 163, 934, 441, 157, 15018, 2194, 15020,
    15030, 0, 139, 256, 15040, 15050, 15060, 15070, 233, 15080, 15090, 15100, 15110, 15120, 15130, 15140,
    15150, 15160, 15170, 15180, 15190, 15200, 15210, 15220, 15230, 0, 15240, 1118, 1121, 15247, 0, 953,
    15244, 15242, 15241, 326, 383, 276, 15249, 15259, 15269, 15279, 15289, 15299, 15309, 0, 594, 1279,
    556, 15319, 15329, 15339, 15349, 15359, 15369, 15379, 15389, 15399, 15409, 15419, 15429, 15439, 15449, 15459,
    15469, 15479, 15488, 15498, 15508, 15518, 15528, 15538, 15548, 0, 1777, 0, 276, 15558, 1543, 15561,
    276, 15564, 15567, 15570, 15580, 0, 1327, 15593, 15590, 276, 15594, 1145, 0, 196, 348, 0,
    1386, 348, 1387, 1388, 1389, 1904, 1037, 1399, 15595, 15605, 15619, 0, 0, 233, 1724, 0,
    2610, 276, 323, 15615, 324, 0, 692, 2610, 233, 15620, 15630, 15637, 0, 84, 0, 15648,
    15647, 0, 233, 758, 0, 1148, 1064, 1148, 15652, 0, 233, 15662, 476, 1386, 2615, 223,
    2617, 0, 479, 15663, 2619, 512, 446, 1502, 0, 479, 2620, 15664, 15667, 2622, 152, 15669,
    2624, 0, 479, 2620, 15672, 15675, 2622, 152, 15677, 2176, 0, 479, 0, 721, 55, 0,
    15680, 0, 479, 721, 46, 2625, 15690, 15700, 15710, 15717, 15727, 15737, 15747, 15757, 15767, 15777,
    15787, 15797, 15807, 15817, 15827, 15837, 15847, 15857,
];

pub static SEGMENT_TYPE_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 65, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 80, 83, 85, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 65, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 162, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 194, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295,
    4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 78, 4294967295, 200, 4294967295, 204, 4294967295,
    4294967295, 209, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 225, 4294967295, 4294967295, 227, 232, 4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295, 4294967295, 4294967295, 4294967295, 245,
    4294967295, 4294967295, 249, 251, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 257, 4294967295, 264, 4294967295,
    4294967295, 267, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 321, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 333, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295,
    4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 358, 4294967295, 360, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 387, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 392, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 249, 4294967295, 4294967295, 4294967295,
    249, 4294967295, 4294967295, 418, 420, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 387, 4294967295, 4294967295, 4294967295, 4294967295, 392,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 387, 4294967295, 4294967295, 4294967295, 4294967295, 392, 4294967295, 4294967295,
    4294967295, 437, 4294967295, 4294967295, 4294967295, 392, 387, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 392, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 392, 4294967295, 4294967295, 4294967295, 4294967295, 443, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295,
    4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    443, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 249, 4294967295, 4294967295, 4294967295, 4294967295,
    443, 4294967295, 443, 4294967295, 4294967295, 392, 4294967295, 392, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 264, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295,
    4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 249, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 504, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 249,
    4294967295, 249, 4294967295, 4294967295, 4294967295, 4294967295, 392, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 249, 4294967295, 4294967295, 4294967295,
    4294967295, 392, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 521, 521, 4294967295, 521, 521, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 591, 204, 392, 277, 4294967295, 593, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 597, 3, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 521, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 392, 4294967295, 4294967295, 4294967295, 4294967295, 392, 4294967295, 392, 654, 4294967295, 277, 4294967295, 4294967295,
    4294967295, 658, 4294967295, 661, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 235, 4294967295, 4294967295, 674, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 227, 4294967295, 689, 4294967295, 4294967295, 695, 4294967295, 4294967295, 4294967295,
    4294967295, 277, 4294967295, 689, 4294967295, 4294967295, 695, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 249, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 249, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4,
    4294967295, 4294967295, 333, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 770, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 321, 4294967295, 4294967295, 333, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 593, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 654, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 796, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 819, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330,
    330, 330, 330, 330, 330, 4294967295, 4294967295, 330, 330, 4294967295, 4294967295, 4294967295, 521, 330, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 277, 4294967295, 4294967295, 4294967295, 593, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 593, 4294967295, 593, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 83, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 257, 4294967295, 890, 4294967295, 4294967295, 267, 4294967295, 4294967295, 78, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 321, 4294967295, 325, 4294967295, 4294967295,
    267, 4294967295, 80, 83, 4294967295, 4294967295, 593, 321, 894, 896, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 443, 901, 4294967295, 249, 4294967295, 904, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 333, 4294967295, 4294967295, 330, 4294967295,
    4294967295, 4294967295, 267, 4294967295, 4294967295, 80, 913, 321, 894, 896, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295,
    4294967295, 4294967295, 4294967295, 915, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 358, 4294967295, 919, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 249, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 437, 387, 923, 4294967295, 4294967295,
    330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 392,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 392, 321, 938, 941, 944, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    330, 4294967295, 4294967295, 4294967295, 249, 4294967295, 4294967295, 4294967295, 330, 4294967295, 321, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 949, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 249, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 325, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 249, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 504, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 80, 4294967295, 1021, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 80, 1029, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 654, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1070, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 593, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    330, 200, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1135, 4294967295, 1137, 1140, 4294967295, 4294967295, 1135, 4294967295, 1142, 1140, 4294967295, 1142, 1144, 1140, 225, 1147,
    4294967295, 249, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1150, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 392, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 593, 4294967295, 593, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 257, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295,
    443, 4294967295, 249, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 915, 4294967295, 4294967295, 4294967295, 4294967295, 358, 4294967295,
    4294967295, 4294967295, 4294967295, 249, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 949, 4294967295, 4294967295, 4294967295, 4294967295, 78,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 249, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1217, 1219, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1253, 1255, 1257, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 97, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 97, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1284, 4294967295, 1286, 770,
    4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 392, 204, 1289,
    277, 4294967295, 85, 4294967295, 4294967295, 1217, 1293, 4294967295, 4294967295, 277, 4294967295, 593, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1300, 1302, 4294967295, 591, 1304, 1306, 392, 1309,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 249, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 249,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    591, 4294967295, 392, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1150, 4294967295, 4294967295, 1147, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1391, 1393, 1395, 4294967295, 4294967295, 1398, 1391, 1393, 504, 1400, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1403,
    1405, 251, 4294967295, 4294967295, 1409, 4294967295, 1412, 4294967295, 4294967295, 4294967295, 251, 1409, 4294967295, 4294967295, 4294967295, 4294967295,
    1412, 4294967295, 4294967295, 4294967295, 4294967295, 1416, 1418, 4294967295, 1150, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 227,
    4294967295, 4294967295, 4294967295, 1412, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 277, 4294967295, 277, 4294967295, 277, 4294967295, 593, 4294967295, 4294967295, 277, 4294967295, 4294967295, 277, 4294967295, 4294967295,
    4294967295, 277, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1428, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 392, 4294967295, 4294967295, 4294967295, 4294967295, 392, 4294967295, 1434, 1304, 4294967295, 4294967295, 4294967295, 1437, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 658, 4294967295, 4294967295, 4294967295, 658, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 593, 4294967295, 4294967295, 4294967295, 4294967295, 593, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 1029, 4294967295,
    80, 1029, 4294967295, 80, 1029, 4294967295, 80, 1029, 4294967295, 4294967295, 4294967295, 80, 4294967295, 80, 80, 1300,
    1029, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1499, 4294967295, 1501, 4294967295, 200, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1510,
    1512, 4294967295, 4294967295, 4294967295, 392, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1147,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 392, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1512, 1021, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    277, 4294967295, 4294967295, 4294967295, 4294967295, 1147, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1192, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 277, 4294967295, 325, 4294967295, 4294967295, 277, 4294967295, 325, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 443, 4294967295, 4294967295, 392,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 78, 1566, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 901, 4294967295, 4294967295, 4294967295,
    78, 321, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295, 4294967295, 4294967295, 249, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1418, 4294967295, 4294967295, 4294967295,
    227, 1642, 4294967295, 1566, 4294967295, 245, 4294967295, 4294967295, 4294967295, 1418, 1642, 4294967295, 4294967295, 1418, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 227, 4294967295, 4294967295, 277, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1661, 4294967295, 4294967295, 593, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295,
    1661, 4294967295, 4294967295, 277, 4294967295, 277, 4294967295, 4294967295, 593, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 97, 4294967295, 4294967295, 1700, 1702, 1705,
    4294967295, 4294967295, 564, 564, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 249, 1144, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 392,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1300, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1774, 1776, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 1780, 1782,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 1784, 4294967295, 4294967295, 4294967295, 249, 4294967295, 4294967295, 4294967295,
    249, 4294967295, 4294967295, 4294967295, 4294967295, 251, 4294967295, 204, 251, 227, 1566, 245, 1788, 4294967295, 4294967295, 1790,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1813, 4294967295, 4294967295, 4294967295, 1815, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1830, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1300, 1302, 4294967295, 1302, 4294967295, 1846, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1255, 1253, 1257, 1850, 4294967295,
    661, 4294967295, 4294967295, 1302, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1869, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 249, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 392, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1895, 1147, 4294967295, 4294967295, 1147, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1813, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 392, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 392, 4294967295, 4294967295, 4294967295, 1925, 1927, 4294967295,
    4294967295, 4294967295, 4294967295, 392, 4294967295, 4294967295, 1930, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1965, 4294967295, 4294967295, 1968, 1144, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 593, 593, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 249, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 249, 4294967295, 249, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 249, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 277, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 249, 1144, 4294967295, 4294967295, 4294967295,
    1300, 1790, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 249, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1499, 4294967295, 1501, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 198, 198, 4294967295, 2103,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295,
    4294967295, 392, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 392, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2144,
    4294967295, 4294967295, 251, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 2162, 4294967295, 2164, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1147, 4294967295,
    1391, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 2166, 1400, 1142, 1968, 1144, 1140, 2168,
    2170, 1309, 2172, 1398, 225, 1147, 2172, 1400, 1391, 2174, 504, 4294967295, 4294967295, 4294967295, 1846, 4294967295,
    4294967295, 1846, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204,
    4294967295, 277, 4294967295, 4294967295, 4294967295, 2195, 4294967295, 392, 330, 4294967295, 4294967295, 4294967295, 204, 392, 277, 1300,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295,
    4294967295, 1391, 4294967295, 225, 1147, 2172, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    333, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 915, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1284, 4294967295, 4294967295, 4294967295, 593, 1286, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1070, 209, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1850, 1253, 1437, 2290, 2292, 2294, 2296, 2298, 2300, 2302, 2304, 2306, 2308, 2310, 2312,
    2314, 2316, 2318, 2320, 2322, 2324, 1257, 1255, 2326, 2328, 2330, 2332, 2334, 2336, 2338, 2340,
    2342, 2344, 2346, 2348, 2350, 2352, 2354, 2356, 2358, 2360, 2362, 2364, 2366, 2368, 2370, 2372,
    2374, 2376, 2378, 2380, 2382, 2384, 2386, 2388, 2390, 2308, 2392, 2324, 2394, 2396, 2398, 2400,
    2402, 2404, 2406, 2408, 2410, 2174, 2412, 2414, 2416, 2418, 2420, 2422, 2424, 2426, 2428, 2430,
    2432, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 521, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 251, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    238, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 443, 901, 4294967295, 904, 4294967295, 4294967295, 901, 4294967295, 904, 4294967295, 4294967295, 443,
    4294967295, 4294967295, 392, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 443, 901, 4294967295, 904, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 443, 4294967295,
    904, 4294967295, 4294967295, 1300, 4294967295, 204, 249, 4294967295, 4294967295, 4294967295, 1850, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    330, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    330, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330,
    4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 330, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295,
    330, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295,
    4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295,
    330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 249, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 654, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 249, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1070,
    1700, 4294967295, 1217, 1219, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2166, 1400, 1142, 1968, 1144, 1140, 2168, 2170, 1309,
    2172, 1398, 1391, 1393, 1395, 225, 1147, 1400, 1398, 1968, 504, 1144, 2174, 4294967295, 4294967295, 4294967295,
    4294967295, 1391, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 249,
    1150, 1790, 1140, 225, 1147, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2195, 4294967295,
    4294967295, 4294967295, 4294967295, 257, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 227, 4294967295, 227,
    277, 245, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 1147, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1400, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 277, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2612, 4294967295, 4294967295, 85, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2616, 225,
    2618, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 227, 4294967295, 2623, 4294967295, 245,
    4294967295, 4294967295, 4294967295, 4294967295, 227, 4294967295, 2623, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 5, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static SEGMENT_CLASS_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 64, 4294967295, 4294967295, 66, 4294967295, 4294967295, 4294967295, 4294967295, 66, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 79, 81, 84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 64, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 161, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 191, 193, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 77, 4294967295, 199, 4294967295, 203, 4294967295,
    4294967295, 208, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 223, 4294967295, 4294967295, 228, 230, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 246,
    4294967295, 4294967295, 248, 250, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 263, 4294967295,
    4294967295, 265, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 319, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 332, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295,
    4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 357, 4294967295, 359, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 386, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 391, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295,
    248, 4294967295, 4294967295, 395, 419, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 386, 4294967295, 4294967295, 4294967295, 4294967295, 391,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 386, 4294967295, 4294967295, 4294967295, 4294967295, 391, 4294967295, 4294967295,
    4294967295, 429, 4294967295, 4294967295, 4294967295, 391, 386, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 391, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 391, 4294967295, 4294967295, 4294967295, 4294967295, 442, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295,
    4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    442, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295,
    442, 4294967295, 442, 4294967295, 4294967295, 391, 4294967295, 391, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 263, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295,
    4294967295, 4294967295, 4294967295, 265, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 503, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248,
    4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295, 391, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295,
    4294967295, 391, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 543, 544, 4294967295, 546, 547, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 589, 203, 391, 276, 4294967295, 592, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 598, 599, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 522, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 391, 4294967295, 4294967295, 4294967295, 4294967295, 391, 4294967295, 391, 653, 4294967295, 276, 4294967295, 4294967295,
    4294967295, 659, 4294967295, 660, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 234, 4294967295, 4294967295, 673, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 686, 4294967295, 688, 4294967295, 4294967295, 694, 4294967295, 4294967295, 4294967295,
    4294967295, 276, 4294967295, 688, 4294967295, 4294967295, 694, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 587,
    4294967295, 4294967295, 332, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 769, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 319, 4294967295, 4294967295, 332, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 592, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 793, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 794, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 794, 4294967295, 4294967295, 4294967295, 4294967295, 795, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 820, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328,
    825, 826, 827, 828, 829, 4294967295, 4294967295, 832, 833, 4294967295, 4294967295, 4294967295, 836, 837, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 276, 4294967295, 4294967295, 4294967295, 592, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 592, 4294967295, 592, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 81, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 889, 4294967295, 4294967295, 265, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 319, 4294967295, 324, 4294967295, 4294967295,
    265, 4294967295, 79, 81, 4294967295, 4294967295, 592, 319, 893, 895, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 442, 900, 4294967295, 248, 4294967295, 903, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 332, 4294967295, 4294967295, 328, 4294967295,
    4294967295, 4294967295, 265, 4294967295, 4294967295, 79, 912, 319, 893, 895, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295,
    4294967295, 4294967295, 4294967295, 914, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 357, 4294967295, 918, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 429, 386, 922, 4294967295, 4294967295,
    328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 391,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 391, 319, 937, 939, 942, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    328, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295, 328, 4294967295, 319, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 265, 4294967295, 4294967295, 948, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 324, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 265, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 503, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1018, 1019, 4294967295, 1020, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1027, 1028, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 793, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1069, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    324, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 592, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    328, 199, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1134, 4294967295, 1136, 1138, 4294967295, 4294967295, 1134, 4294967295, 1141, 1138, 4294967295, 1141, 1143, 1138, 223, 1145,
    4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1149, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 391, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 592, 4294967295, 592, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295,
    442, 4294967295, 248, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 914, 4294967295, 4294967295, 4294967295, 4294967295, 357, 4294967295,
    4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 948, 4294967295, 4294967295, 4294967295, 4294967295, 77,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1216, 1218, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1252, 1254, 1256, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1275, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1275, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1283, 4294967295, 1285, 769,
    4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 391, 203, 1288,
    276, 4294967295, 1291, 4294967295, 4294967295, 1216, 1292, 4294967295, 4294967295, 276, 4294967295, 592, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1288, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1299, 1301, 4294967295, 589, 1303, 1305, 391, 1307,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    589, 4294967295, 391, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1149, 4294967295, 4294967295, 1145, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1390, 1392, 1394, 4294967295, 4294967295, 1397, 1390, 1392, 503, 1399, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1402,
    1404, 250, 4294967295, 4294967295, 1407, 4294967295, 1411, 4294967295, 4294967295, 4294967295, 250, 1407, 4294967295, 4294967295, 4294967295, 4294967295,
    1411, 4294967295, 4294967295, 4294967295, 4294967295, 1414, 1417, 4294967295, 1149, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 228,
    4294967295, 4294967295, 4294967295, 1411, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 276, 4294967295, 276, 4294967295, 276, 4294967295, 592, 4294967295, 4294967295, 276, 4294967295, 4294967295, 276, 4294967295, 4294967295,
    4294967295, 276, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1427, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 391, 4294967295, 4294967295, 4294967295, 4294967295, 391, 4294967295, 1433, 1435, 4294967295, 4294967295, 4294967295, 1436, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 659, 4294967295, 4294967295, 4294967295, 659, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 592, 4294967295, 4294967295, 4294967295, 4294967295, 592, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1443, 1444, 4294967295,
    1446, 1447, 4294967295, 1448, 1449, 4294967295, 1450, 1451, 4294967295, 4294967295, 4294967295, 79, 4294967295, 1450, 1453, 1299,
    1454, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1498, 4294967295, 1500, 4294967295, 199, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1508,
    1511, 4294967295, 4294967295, 4294967295, 391, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1145,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 391, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1511, 1020, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    276, 4294967295, 4294967295, 4294967295, 4294967295, 1145, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1193, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 276, 4294967295, 324, 4294967295, 4294967295, 276, 4294967295, 324, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 442, 4294967295, 4294967295, 391,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 77, 1565, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 900, 4294967295, 4294967295, 4294967295,
    77, 319, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 248, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1417, 4294967295, 4294967295, 4294967295,
    228, 1641, 4294967295, 1565, 4294967295, 246, 4294967295, 4294967295, 4294967295, 1417, 1641, 4294967295, 4294967295, 1417, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 686, 4294967295, 4294967295, 276, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1660, 4294967295, 4294967295, 592, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295,
    1660, 4294967295, 4294967295, 276, 4294967295, 276, 4294967295, 4294967295, 592, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 2, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1275, 4294967295, 4294967295, 1699, 1701, 1703,
    4294967295, 4294967295, 1314, 565, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 248, 1143, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 391,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1299, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1773, 1775, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 1779, 1781,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 1783, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295,
    248, 4294967295, 4294967295, 4294967295, 4294967295, 250, 4294967295, 203, 250, 228, 1565, 246, 1787, 4294967295, 4294967295, 1789,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1812, 4294967295, 4294967295, 4294967295, 1814, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1831, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1299, 1844, 4294967295, 1301, 4294967295, 1845, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1254, 1252, 1256, 1849, 4294967295,
    1851, 4294967295, 4294967295, 1301, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1868, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 391, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1894, 1145, 4294967295, 4294967295, 1145, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1812, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 391, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 391, 4294967295, 4294967295, 4294967295, 1924, 1926, 4294967295,
    4294967295, 4294967295, 4294967295, 391, 4294967295, 4294967295, 1929, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1964, 4294967295, 4294967295, 1967, 1143, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 592, 592, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 248, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295, 276, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248, 1143, 4294967295, 4294967295, 4294967295,
    1299, 1789, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1498, 4294967295, 1500, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 197, 197, 4294967295, 2102,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 391, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 391, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2143,
    4294967295, 4294967295, 250, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 2161, 4294967295, 2163, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1145, 4294967295,
    1390, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 2165, 1399, 1141, 1967, 1143, 1138, 2167,
    2169, 1307, 2171, 1397, 223, 1145, 2171, 1399, 1390, 2173, 503, 4294967295, 4294967295, 4294967295, 1845, 4294967295,
    4294967295, 2175, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203,
    4294967295, 276, 4294967295, 4294967295, 4294967295, 2194, 4294967295, 391, 328, 4294967295, 4294967295, 4294967295, 203, 391, 276, 1299,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295,
    4294967295, 1390, 4294967295, 223, 1145, 2171, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    332, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 914, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1283, 4294967295, 4294967295, 4294967295, 592, 1285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1069, 208, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1849, 1252, 1436, 2289, 2291, 2293, 2295, 2297, 2299, 2301, 2303, 2305, 2307, 2309, 2311,
    2313, 2315, 2317, 2319, 2321, 2323, 1256, 1254, 2325, 2327, 2329, 2331, 2333, 2335, 2337, 2339,
    2341, 2343, 2345, 2347, 2349, 2351, 2353, 2355, 2357, 2359, 2361, 2363, 2365, 2367, 2369, 2371,
    2373, 2375, 2377, 2379, 2381, 2383, 2385, 2387, 2389, 2307, 2391, 2323, 2393, 2395, 2397, 2399,
    2401, 2403, 2405, 2407, 2409, 2173, 2411, 2413, 2415, 2417, 2419, 2421, 2423, 2425, 2427, 2429,
    2431, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2443, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 250, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 442, 900, 4294967295, 903, 4294967295, 4294967295, 900, 4294967295, 903, 4294967295, 4294967295, 442,
    4294967295, 4294967295, 391, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 442, 900, 4294967295, 903, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 442, 4294967295,
    903, 4294967295, 4294967295, 1299, 4294967295, 203, 248, 4294967295, 4294967295, 4294967295, 1849, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    328, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    328, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328,
    4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 328, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295,
    328, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295,
    4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295,
    328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 793, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1069,
    1699, 4294967295, 1216, 2548, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2165, 1399, 1141, 1967, 1143, 1138, 2167, 2169, 1307,
    2171, 1397, 1390, 1392, 1394, 223, 1145, 1399, 1397, 1967, 503, 1143, 2173, 4294967295, 4294967295, 4294967295,
    4294967295, 1390, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248,
    1149, 1789, 1138, 223, 1145, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2194, 4294967295,
    4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 228, 4294967295, 686,
    276, 246, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 1145, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1399, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 276, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2613, 4294967295, 4294967295, 84, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2615, 223,
    2617, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 228, 4294967295, 2622, 4294967295, 246,
    4294967295, 4294967295, 4294967295, 4294967295, 228, 4294967295, 2622, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1438, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static CASEFOLD_SPARSE: &[(u32, u8)] = &[
];

pub static TRIM_CHARS_SPARSE: &[(u32, u32, u8)] = &[
    (1968, 0, 1),
    (1969, 1, 1),
    (4035, 2, 1),
    (4686, 3, 1),
];

pub static TRIM_CHARS_DATA: &[u32] = &[
    1195, 1195, 2000, 2219,
];

pub static SEGMENT_CLASS_TYPES_SPARSE: &[(u32, u32, u8)] = &[
    (38, 0, 2),
    (41, 2, 2),
    (46, 4, 2),
    (53, 6, 2),
    (55, 8, 2),
    (56, 10, 2),
    (57, 12, 3),
    (99, 15, 2),
    (135, 17, 2),
    (184, 19, 2),
    (185, 21, 2),
    (190, 23, 2),
    (194, 25, 3),
    (198, 28, 3),
    (202, 31, 2),
    (204, 33, 3),
    (206, 36, 2),
    (209, 38, 2),
    (225, 40, 2),
    (228, 42, 4),
    (229, 46, 2),
    (234, 48, 2),
    (239, 50, 5),
    (242, 55, 3),
    (243, 58, 2),
    (252, 60, 3),
    (254, 63, 2),
    (257, 65, 2),
    (259, 67, 2),
    (266, 69, 2),
    (274, 71, 2),
    (283, 73, 2),
    (293, 75, 2),
    (296, 77, 2),
    (303, 79, 2),
    (312, 81, 2),
    (313, 83, 3),
    (317, 86, 2),
    (322, 88, 2),
    (345, 90, 3),
    (347, 93, 2),
    (356, 95, 2),
    (364, 97, 2),
    (373, 99, 2),
    (382, 101, 2),
    (388, 103, 3),
    (396, 106, 3),
    (400, 109, 3),
    (403, 112, 2),
    (404, 114, 2),
    (410, 116, 2),
    (415, 118, 3),
    (428, 121, 2),
    (440, 123, 2),
    (445, 125, 3),
    (449, 128, 2),
    (453, 130, 3),
    (454, 133, 2),
    (461, 135, 3),
    (467, 138, 3),
    (472, 141, 3),
    (476, 144, 2),
    (481, 146, 2),
    (487, 148, 2),
    (496, 150, 3),
    (507, 153, 3),
    (512, 156, 3),
    (514, 159, 3),
    (517, 162, 3),
    (519, 165, 3),
    (529, 168, 2),
    (555, 170, 2),
    (569, 172, 2),
    (574, 174, 2),
    (579, 176, 2),
    (587, 178, 3),
    (593, 181, 2),
    (607, 183, 3),
    (609, 186, 3),
    (614, 189, 3),
    (620, 192, 3),
    (625, 195, 3),
    (643, 198, 2),
    (644, 200, 2),
    (646, 202, 2),
    (647, 204, 2),
    (654, 206, 2),
    (697, 208, 2),
    (698, 210, 2),
    (699, 212, 3),
    (700, 215, 2),
    (702, 217, 2),
    (709, 219, 2),
    (710, 221, 1),
    (726, 222, 3),
    (771, 225, 3),
    (776, 228, 3),
    (778, 231, 3),
    (779, 234, 2),
    (781, 236, 2),
    (785, 238, 2),
    (787, 240, 2),
    (802, 242, 2),
    (805, 244, 2),
    (814, 246, 2),
    (823, 248, 4),
    (825, 252, 2),
    (828, 254, 2),
    (833, 256, 2),
    (835, 258, 2),
    (838, 260, 2),
    (908, 262, 3),
    (925, 265, 3),
    (943, 268, 2),
    (946, 270, 3),
    (966, 273, 2),
    (968, 275, 2),
    (975, 277, 2),
    (983, 279, 2),
    (995, 281, 2),
    (998, 283, 3),
    (1014, 286, 2),
    (1026, 288, 2),
    (1032, 290, 2),
    (1036, 292, 2),
    (1044, 294, 2),
    (1053, 296, 2),
    (1062, 298, 2),
    (1067, 300, 2),
    (1097, 302, 3),
    (1103, 305, 2),
    (1104, 307, 2),
    (1105, 309, 2),
    (1106, 311, 2),
    (1107, 313, 2),
    (1108, 315, 2),
    (1111, 317, 2),
    (1112, 319, 3),
    (1116, 322, 2),
    (1117, 324, 2),
    (1153, 326, 2),
    (1157, 328, 2),
    (1177, 330, 2),
    (1179, 332, 2),
    (1192, 334, 2),
    (1193, 336, 2),
    (1196, 338, 2),
    (1206, 340, 3),
    (1208, 343, 2),
    (1211, 345, 2),
    (1214, 347, 2),
    (1220, 349, 2),
    (1228, 351, 2),
    (1243, 353, 2),
    (1245, 355, 2),
    (1248, 357, 2),
    (1250, 359, 2),
    (1251, 361, 2),
    (1254, 363, 2),
    (1255, 365, 2),
    (1256, 367, 2),
    (1257, 369, 2),
    (1265, 371, 3),
    (1266, 374, 2),
    (1268, 376, 3),
    (1270, 379, 2),
    (1274, 381, 2),
    (1283, 383, 2),
    (1295, 385, 2),
    (1302, 387, 2),
    (1316, 389, 2),
    (1322, 391, 2),
    (1323, 393, 3),
    (1326, 396, 2),
    (1330, 398, 2),
    (1333, 400, 2),
    (1334, 402, 2),
    (1335, 404, 2),
    (1336, 406, 2),
    (1337, 408, 2),
    (1342, 410, 3),
    (1347, 413, 3),
    (1371, 416, 3),
    (1373, 419, 2),
    (1381, 421, 3),
    (1387, 424, 2),
    (1388, 426, 2),
    (1389, 428, 2),
    (1392, 430, 2),
    (1407, 432, 3),
    (1413, 435, 3),
    (1414, 438, 2),
    (1415, 440, 2),
    (1416, 442, 2),
    (1417, 444, 2),
    (1424, 446, 2),
    (1428, 448, 3),
    (1432, 451, 2),
    (1434, 453, 2),
    (1441, 455, 2),
    (1450, 457, 2),
    (1453, 459, 3),
    (1462, 462, 3),
    (1473, 465, 2),
    (1476, 467, 2),
    (1484, 469, 3),
    (1499, 472, 2),
    (1526, 474, 2),
    (1534, 476, 3),
    (1622, 479, 2),
    (1627, 481, 2),
    (1635, 483, 3),
    (1641, 486, 2),
    (1653, 488, 2),
    (1654, 490, 2),
    (1656, 492, 2),
    (1667, 494, 2),
    (1668, 496, 2),
    (1733, 498, 2),
    (1745, 500, 2),
    (1800, 502, 2),
    (1824, 504, 2),
    (1836, 506, 2),
    (1842, 508, 2),
    (1856, 510, 2),
    (1857, 512, 3),
    (1872, 515, 2),
    (1874, 517, 2),
    (1875, 519, 2),
    (1878, 521, 2),
    (1880, 523, 2),
    (1881, 525, 2),
    (1883, 527, 2),
    (1884, 529, 2),
    (1885, 531, 2),
    (1886, 533, 2),
    (1887, 535, 2),
    (1889, 537, 3),
    (1896, 540, 2),
    (1915, 542, 2),
    (1946, 544, 3),
    (1977, 547, 2),
    (1979, 549, 2),
    (1985, 551, 3),
    (1991, 554, 2),
    (1996, 556, 2),
    (2000, 558, 3),
    (2002, 561, 3),
    (2006, 564, 2),
    (2013, 566, 2),
    (2024, 568, 2),
    (2031, 570, 2),
    (2041, 572, 3),
    (2046, 575, 3),
    (2051, 578, 3),
    (2058, 581, 3),
    (2063, 584, 2),
    (2069, 586, 3),
    (2074, 589, 3),
    (2091, 592, 2),
    (2097, 594, 2),
    (2098, 596, 2),
    (2155, 598, 2),
    (2156, 600, 2),
    (2157, 602, 2),
    (2189, 604, 2),
    (2206, 606, 2),
    (2220, 608, 2),
    (2222, 610, 2),
    (2223, 612, 2),
    (2228, 614, 2),
    (2232, 616, 2),
    (2237, 618, 3),
    (2238, 621, 2),
    (2239, 623, 2),
    (2240, 625, 2),
    (2242, 627, 3),
    (2245, 630, 2),
    (2246, 632, 2),
    (2249, 634, 2),
    (2251, 636, 2),
    (2258, 638, 2),
    (2264, 640, 2),
    (2265, 642, 2),
    (2267, 644, 2),
    (2268, 646, 2),
    (2269, 648, 2),
    (2270, 650, 3),
    (2271, 653, 2),
    (2292, 655, 2),
    (2309, 657, 2),
    (2321, 659, 2),
    (2363, 661, 2),
    (2373, 663, 3),
    (2383, 666, 3),
    (2432, 669, 2),
    (2434, 671, 3),
    (2451, 674, 2),
    (2454, 676, 2),
    (2464, 678, 2),
    (2465, 680, 2),
    (2466, 682, 2),
    (2469, 684, 2),
    (2470, 686, 2),
    (2471, 688, 2),
    (2472, 690, 2),
    (2473, 692, 2),
    (2479, 694, 2),
    (2480, 696, 2),
    (2481, 702, 2),
    (2484, 698, 2),
    (2486, 700, 2),
    (2490, 704, 2),
    (2491, 706, 2),
    (2496, 708, 2),
    (2501, 710, 2),
    (2502, 712, 2),
    (2504, 714, 2),
    (2515, 716, 2),
    (2529, 718, 2),
    (2531, 720, 2),
    (2533, 722, 2),
    (2535, 724, 2),
    (2538, 726, 2),
    (2541, 728, 2),
    (2545, 730, 2),
    (2547, 732, 2),
    (2554, 734, 2),
    (2562, 736, 3),
    (2567, 739, 3),
    (2569, 742, 2),
    (2570, 744, 2),
    (2574, 746, 2),
    (2582, 748, 2),
    (2586, 750, 2),
    (2594, 752, 2),
    (2599, 754, 2),
    (2605, 756, 2),
    (2606, 758, 2),
    (2608, 760, 2),
    (2609, 762, 2),
    (2611, 764, 2),
    (2612, 766, 2),
    (2614, 768, 2),
    (2615, 770, 2),
    (2619, 778, 2),
    (2621, 772, 2),
    (2622, 774, 2),
    (2623, 776, 2),
    (2624, 780, 2),
    (2647, 782, 2),
    (2661, 784, 2),
    (2689, 786, 2),
    (2691, 788, 2),
    (2693, 790, 3),
    (2696, 793, 3),
    (2719, 796, 2),
    (2720, 798, 2),
    (2724, 800, 3),
    (2726, 803, 2),
    (2735, 805, 2),
    (2745, 807, 3),
    (2747, 810, 2),
    (2757, 812, 2),
    (2758, 814, 2),
    (2768, 816, 2),
    (2773, 818, 2),
    (2807, 820, 3),
    (2817, 823, 2),
    (2819, 825, 2),
    (2822, 827, 2),
    (2824, 829, 2),
    (2827, 831, 2),
    (2860, 833, 2),
    (2876, 835, 3),
    (2879, 838, 3),
    (2892, 841, 2),
    (2916, 843, 2),
    (2917, 845, 2),
    (2922, 847, 2),
    (2924, 849, 2),
    (2928, 851, 2),
    (2929, 853, 2),
    (2935, 855, 2),
    (2939, 857, 2),
    (2970, 859, 2),
    (2974, 861, 3),
    (2986, 864, 2),
    (3012, 866, 2),
    (3100, 868, 2),
    (3105, 870, 2),
    (3107, 872, 2),
    (3113, 874, 2),
    (3114, 876, 2),
    (3117, 878, 2),
    (3127, 880, 2),
    (3156, 882, 2),
    (3159, 884, 2),
    (3181, 886, 2),
    (3184, 888, 2),
    (3187, 890, 2),
    (3189, 892, 2),
    (3192, 894, 2),
    (3204, 896, 2),
    (3220, 898, 4),
    (3270, 902, 2),
    (3277, 904, 2),
    (3291, 906, 2),
    (3306, 908, 2),
    (3309, 910, 2),
    (3310, 912, 2),
    (3311, 914, 2),
    (3314, 916, 5),
    (3315, 921, 3),
    (3332, 924, 3),
    (3333, 927, 2),
    (3375, 929, 3),
    (3405, 932, 2),
    (3420, 934, 2),
    (3486, 936, 2),
    (3492, 938, 2),
    (3493, 940, 2),
    (3499, 942, 2),
    (3502, 944, 2),
    (3503, 946, 2),
    (3510, 948, 2),
    (3512, 950, 2),
    (3516, 952, 3),
    (3520, 955, 3),
    (3525, 958, 2),
    (3527, 960, 2),
    (3528, 962, 2),
    (3530, 964, 2),
    (3532, 966, 2),
    (3535, 968, 2),
    (3587, 970, 2),
    (3591, 972, 2),
    (3612, 974, 3),
    (3632, 977, 2),
    (3633, 979, 2),
    (3635, 981, 2),
    (3637, 983, 2),
    (3643, 985, 2),
    (3644, 987, 2),
    (3645, 989, 2),
    (3646, 991, 2),
    (3648, 993, 2),
    (3651, 995, 2),
    (3701, 997, 2),
    (3731, 999, 2),
    (3761, 1001, 3),
    (3781, 1004, 3),
    (3783, 1007, 2),
    (3792, 1009, 2),
    (3793, 1011, 2),
    (3796, 1013, 2),
    (3822, 1015, 2),
    (3832, 1017, 3),
    (3862, 1020, 2),
    (3871, 1022, 2),
    (3881, 1024, 3),
    (3885, 1027, 2),
    (3886, 1029, 2),
    (3891, 1031, 3),
    (3894, 1034, 2),
    (3943, 1036, 2),
    (3946, 1038, 2),
    (3947, 1040, 2),
    (3986, 1042, 2),
    (3987, 1044, 2),
    (4013, 1046, 2),
    (4046, 1048, 2),
    (4071, 1050, 2),
    (4114, 1052, 3),
    (4177, 1055, 3),
    (4179, 1058, 3),
    (4189, 1061, 3),
    (4202, 1064, 2),
    (4205, 1066, 2),
    (4219, 1068, 3),
    (4220, 1071, 2),
    (4224, 1073, 2),
    (4225, 1075, 2),
    (4232, 1077, 3),
    (4285, 1080, 2),
    (4304, 1082, 2),
    (4322, 1084, 2),
    (4324, 1086, 2),
    (4327, 1088, 3),
    (4332, 1091, 3),
    (4333, 1094, 3),
    (4335, 1097, 2),
    (4366, 1099, 2),
    (4369, 1101, 3),
    (4375, 1104, 3),
    (4431, 1107, 2),
    (4434, 1109, 2),
    (4450, 1111, 2),
    (4452, 1113, 2),
    (4462, 1115, 2),
    (4464, 1117, 2),
    (4471, 1119, 2),
    (4473, 1121, 2),
    (4474, 1123, 2),
    (4475, 1125, 2),
    (4476, 1127, 2),
    (4477, 1129, 2),
    (4478, 1131, 2),
    (4479, 1133, 2),
    (4480, 1135, 2),
    (4481, 1137, 2),
    (4482, 1139, 2),
    (4483, 1141, 2),
    (4484, 1143, 2),
    (4485, 1145, 2),
    (4486, 1147, 2),
    (4487, 1149, 2),
    (4488, 1151, 2),
    (4489, 1153, 2),
    (4490, 1155, 2),
    (4494, 1157, 2),
    (4497, 1159, 2),
    (4549, 1161, 2),
    (4559, 1163, 2),
    (4561, 1165, 2),
    (4565, 1167, 2),
    (4567, 1169, 3),
    (4568, 1172, 2),
    (4572, 1174, 2),
    (4573, 1176, 3),
    (4574, 1179, 2),
    (4575, 1181, 2),
    (4585, 1183, 3),
    (4588, 1186, 3),
    (4593, 1189, 2),
    (4595, 1191, 2),
    (4596, 1193, 2),
    (4597, 1195, 2),
    (4608, 1197, 3),
    (4621, 1200, 3),
    (4659, 1203, 2),
    (4663, 1205, 2),
    (4664, 1207, 2),
    (4689, 1209, 2),
    (4690, 1211, 2),
    (4785, 1213, 2),
    (4786, 1215, 2),
    (4787, 1217, 2),
    (4788, 1219, 2),
    (4789, 1221, 2),
    (4790, 1223, 2),
    (4791, 1225, 2),
    (4792, 1227, 2),
    (4793, 1229, 2),
    (4794, 1231, 2),
    (4795, 1233, 2),
    (4796, 1235, 2),
    (4797, 1237, 2),
    (4798, 1239, 2),
    (4799, 1241, 2),
    (4800, 1243, 2),
    (4801, 1245, 2),
    (4802, 1247, 2),
    (4803, 1249, 2),
    (4804, 1251, 2),
    (4805, 1253, 2),
    (4806, 1255, 2),
    (4807, 1257, 2),
    (4808, 1259, 2),
    (4809, 1261, 2),
    (4810, 1263, 2),
    (4811, 1265, 2),
    (4812, 1267, 2),
    (4813, 1269, 2),
    (4814, 1271, 2),
    (4815, 1273, 2),
    (4816, 1275, 2),
    (4817, 1277, 2),
    (4818, 1279, 2),
    (4819, 1281, 2),
    (4820, 1283, 2),
    (4821, 1285, 2),
    (4822, 1287, 2),
    (4823, 1289, 2),
    (4824, 1291, 2),
    (4825, 1293, 2),
    (4826, 1295, 2),
    (4827, 1297, 2),
    (4828, 1299, 2),
    (4829, 1301, 2),
    (4830, 1303, 2),
    (4831, 1305, 2),
    (4832, 1307, 2),
    (4833, 1309, 2),
    (4834, 1311, 2),
    (4835, 1313, 2),
    (4836, 1315, 2),
    (4837, 1317, 2),
    (4838, 1319, 2),
    (4839, 1321, 2),
    (4840, 1323, 2),
    (4841, 1325, 2),
    (4842, 1327, 2),
    (4843, 1329, 2),
    (4844, 1331, 2),
    (4845, 1333, 2),
    (4846, 1335, 2),
    (4847, 1337, 2),
    (4848, 1339, 2),
    (4849, 1341, 2),
    (4850, 1343, 2),
    (4851, 1345, 2),
    (4852, 1347, 2),
    (4853, 1349, 2),
    (4854, 1351, 2),
    (4855, 1353, 2),
    (4856, 1355, 2),
    (4857, 1357, 2),
    (4858, 1359, 2),
    (4859, 1361, 2),
    (4860, 1363, 2),
    (4861, 1365, 2),
    (4862, 1367, 2),
    (4863, 1369, 2),
    (4864, 1371, 2),
    (4886, 1373, 2),
    (4893, 1375, 2),
    (4912, 1377, 3),
    (4925, 1380, 2),
    (4932, 1382, 3),
    (4933, 1385, 2),
    (4935, 1387, 2),
    (4938, 1389, 2),
    (4940, 1391, 2),
    (4943, 1393, 3),
    (4946, 1396, 3),
    (4969, 1399, 2),
    (4979, 1401, 3),
    (4980, 1404, 2),
    (4982, 1406, 2),
    (4990, 1408, 3),
    (4992, 1411, 2),
    (4995, 1413, 2),
    (4997, 1415, 2),
    (4998, 1417, 3),
    (5002, 1420, 2),
    (5008, 1422, 2),
    (5012, 1424, 2),
    (5016, 1426, 2),
    (5021, 1428, 2),
    (5028, 1430, 2),
    (5033, 1432, 2),
    (5040, 1434, 2),
    (5044, 1436, 2),
    (5048, 1438, 2),
    (5055, 1440, 2),
    (5059, 1442, 2),
    (5063, 1444, 2),
    (5067, 1446, 2),
    (5073, 1448, 2),
    (5077, 1450, 2),
    (5084, 1452, 2),
    (5088, 1454, 2),
    (5092, 1456, 2),
    (5096, 1458, 2),
    (5102, 1460, 2),
    (5106, 1462, 2),
    (5116, 1464, 2),
    (5120, 1466, 2),
    (5126, 1468, 2),
    (5132, 1470, 2),
    (5144, 1472, 2),
    (5147, 1474, 3),
    (5198, 1477, 2),
    (5210, 1479, 2),
    (5240, 1481, 2),
    (5286, 1483, 3),
    (5295, 1486, 2),
    (5296, 1488, 2),
    (5298, 1490, 2),
    (5299, 1492, 2),
    (5335, 1494, 2),
    (5336, 1496, 2),
    (5337, 1498, 2),
    (5338, 1500, 2),
    (5339, 1502, 2),
    (5340, 1504, 2),
    (5341, 1506, 2),
    (5342, 1508, 2),
    (5343, 1510, 2),
    (5344, 1512, 2),
    (5345, 1514, 2),
    (5346, 1516, 2),
    (5347, 1518, 2),
    (5348, 1520, 2),
    (5349, 1522, 2),
    (5350, 1524, 2),
    (5351, 1526, 2),
    (5352, 1528, 2),
    (5353, 1530, 2),
    (5354, 1532, 2),
    (5355, 1534, 2),
    (5356, 1536, 2),
    (5361, 1538, 2),
    (5375, 1540, 3),
    (5376, 1543, 2),
    (5377, 1545, 2),
    (5378, 1547, 2),
    (5379, 1549, 2),
    (5380, 1551, 2),
    (5390, 1553, 2),
    (5395, 1555, 3),
    (5429, 1558, 2),
    (5468, 1560, 2),
    (5472, 1562, 2),
    (5481, 1564, 2),
    (5483, 1566, 2),
    (5495, 1568, 2),
    (5505, 1570, 2),
    (5508, 1572, 2),
    (5514, 1574, 3),
    (5517, 1577, 3),
    (5534, 1580, 2),
    (5535, 1582, 2),
    (5536, 1584, 2),
    (5549, 1586, 2),
    (5558, 1588, 2),
    (5576, 1590, 3),
];

pub static SEGMENT_CLASS_TYPES_DATA: &[u32] = &[
    65, 3, 76, 3, 76, 3, 3, 78, 3, 80, 3, 83, 3, 78, 85, 65,
    3, 162, 3, 3, 192, 3, 194, 3, 78, 3, 78, 198, 3, 78, 200, 3,
    78, 3, 78, 200, 3, 204, 209, 3, 3, 225, 3, 227, 229, 4, 232, 3,
    3, 235, 3, 245, 227, 229, 4, 3, 78, 249, 251, 3, 3, 257, 78, 264,
    3, 3, 267, 3, 78, 3, 277, 3, 277, 3, 78, 3, 321, 3, 325, 3,
    330, 3, 330, 3, 333, 78, 3, 330, 3, 330, 3, 78, 358, 360, 3, 3,
    330, 3, 330, 3, 330, 3, 387, 3, 392, 78, 3, 78, 249, 3, 78, 249,
    3, 418, 420, 3, 3, 387, 3, 392, 78, 3, 277, 3, 387, 3, 392, 78,
    3, 437, 3, 392, 78, 3, 387, 3, 392, 78, 3, 392, 78, 3, 443, 78,
    3, 78, 3, 78, 3, 78, 3, 443, 78, 3, 78, 249, 3, 443, 78, 3,
    443, 78, 3, 392, 78, 3, 392, 78, 264, 3, 3, 78, 3, 78, 3, 330,
    3, 267, 3, 78, 249, 3, 504, 3, 78, 249, 3, 78, 249, 3, 392, 78,
    3, 78, 249, 3, 392, 78, 3, 521, 3, 521, 3, 521, 3, 521, 3, 277,
    3, 591, 3, 204, 3, 392, 78, 3, 277, 3, 593, 3, 597, 3, 3, 521,
    4, 3, 392, 78, 3, 392, 78, 3, 392, 78, 3, 654, 3, 277, 3, 658,
    3, 661, 3, 235, 3, 674, 3, 204, 3, 227, 229, 4, 3, 689, 3, 695,
    3, 277, 3, 689, 3, 695, 3, 78, 249, 3, 78, 249, 3, 4, 3, 333,
    78, 3, 770, 3, 204, 3, 78, 3, 277, 3, 321, 3, 333, 78, 3, 277,
    3, 277, 3, 277, 3, 593, 3, 654, 3, 204, 3, 204, 3, 796, 3, 819,
    4, 3, 330, 3, 330, 3, 330, 3, 330, 3, 330, 3, 330, 3, 330, 3,
    330, 4, 3, 521, 3, 330, 3, 277, 3, 593, 3, 593, 3, 593, 3, 80,
    3, 83, 3, 78, 3, 257, 78, 3, 890, 3, 267, 3, 78, 3, 277, 3,
    277, 3, 321, 3, 325, 3, 267, 3, 80, 3, 83, 3, 593, 3, 321, 3,
    894, 3, 896, 3, 443, 78, 3, 901, 3, 78, 249, 3, 904, 3, 330, 3,
    330, 3, 78, 3, 330, 3, 330, 3, 330, 3, 333, 78, 3, 330, 3, 267,
    3, 80, 3, 913, 3, 321, 3, 894, 3, 896, 3, 78, 200, 3, 78, 915,
    3, 78, 358, 3, 919, 3, 78, 249, 3, 437, 3, 387, 3, 923, 3, 330,
    3, 392, 78, 3, 392, 78, 3, 321, 3, 938, 3, 941, 3, 944, 3, 330,
    3, 78, 249, 3, 330, 3, 321, 3, 330, 3, 267, 3, 78, 949, 3, 78,
    249, 3, 325, 3, 325, 3, 78, 200, 3, 78, 3, 78, 3, 78, 200, 3,
    330, 3, 267, 3, 78, 249, 3, 504, 3, 80, 3, 80, 3, 1021, 3, 80,
    3, 1029, 3, 654, 1070, 3, 3, 325, 3, 325, 3, 593, 3, 204, 3, 330,
    3, 78, 200, 3, 1135, 3, 1137, 3, 1140, 3, 1135, 3, 1142, 3, 1140, 3,
    1142, 3, 1144, 3, 1140, 3, 225, 3, 1147, 3, 78, 249, 3, 1150, 3, 78,
    3, 392, 78, 3, 593, 3, 593, 3, 257, 78, 3, 78, 3, 80, 3, 443,
    78, 3, 78, 249, 3, 330, 3, 330, 3, 78, 3, 78, 3, 78, 915, 3,
    78, 358, 3, 78, 249, 3, 78, 949, 3, 78, 3, 78, 200, 3, 78, 249,
    3, 277, 3, 1217, 3, 1219, 3, 1253, 3, 1255, 3, 1257, 3, 97, 3, 97,
    3, 1284, 3, 1286, 3, 770, 3, 204, 3, 277, 3, 392, 78, 3, 204, 3,
    1289, 3, 277, 3, 78, 85, 3, 1217, 3, 1293, 3, 277, 3, 593, 3, 1289,
    3, 1300, 3, 1302, 3, 591, 3, 1304, 1306, 3, 3, 392, 78, 3, 1309, 3,
    277, 3, 325, 3, 277, 3, 78, 3, 78, 249, 3, 78, 249, 3, 591, 3,
    392, 78, 3, 1150, 3, 1147, 3, 1391, 3, 1393, 3, 1395, 3, 1398, 3, 1391,
    3, 1393, 3, 504, 3, 1400, 3, 1403, 3, 1405, 3, 1409, 3, 1412, 251, 3,
    251, 3, 3, 1409, 3, 1412, 3, 1416, 3, 1418, 3, 1150, 3, 1412, 3, 277,
    3, 277, 3, 277, 3, 593, 3, 277, 3, 277, 3, 277, 3, 277, 1428, 3,
    3, 392, 78, 3, 392, 78, 3, 1434, 3, 1304, 3, 1437, 3, 658, 3, 658,
    3, 593, 3, 593, 3, 80, 3, 1029, 3, 80, 3, 1029, 3, 80, 3, 1029,
    3, 80, 3, 1029, 3, 80, 3, 80, 3, 1300, 3, 80, 3, 1029, 3, 330,
    3, 330, 1499, 3, 1501, 3, 3, 78, 200, 3, 78, 200, 3, 1510, 3, 1512,
    3, 392, 78, 3, 277, 3, 1147, 3, 392, 78, 3, 277, 3, 1512, 3, 1021,
    3, 277, 3, 1147, 3, 1192, 4, 3, 277, 3, 325, 3, 277, 3, 325, 3,
    325, 3, 204, 3, 443, 78, 3, 392, 78, 3, 330, 3, 78, 3, 1566, 3,
    330, 3, 901, 3, 78, 3, 321, 3, 330, 3, 330, 3, 235, 3, 78, 249,
    3, 1300, 3, 277, 3, 1418, 3, 1642, 3, 1566, 3, 1418, 3, 1642, 3, 1418,
    3, 277, 3, 1661, 3, 593, 3, 277, 3, 1661, 3, 277, 3, 277, 3, 593,
    3, 277, 3, 1, 4, 5, 3, 277, 3, 277, 3, 78, 3, 97, 1700, 3,
    3, 1702, 3, 1705, 3, 1, 564, 4, 5, 3, 564, 4, 3, 78, 249, 3,
    1144, 3, 392, 78, 3, 325, 3, 78, 3, 1300, 3, 1774, 3, 1776, 3, 277,
    3, 1780, 3, 1782, 3, 277, 3, 1784, 3, 78, 249, 3, 78, 249, 251, 3,
    3, 204, 251, 3, 3, 1566, 3, 1788, 3, 1790, 3, 1813, 3, 1815, 3, 1830,
    4, 3, 1300, 3, 1302, 3, 1302, 3, 1846, 3, 1255, 3, 1253, 3, 1257, 3,
    1850, 3, 661, 3, 1302, 3, 1869, 3, 277, 3, 78, 249, 3, 392, 78, 3,
    277, 3, 1895, 3, 1147, 3, 1147, 3, 1813, 3, 392, 78, 3, 277, 3, 204,
    3, 392, 78, 3, 1925, 3, 1927, 3, 392, 78, 3, 1930, 3, 1965, 3, 1968,
    3, 1144, 3, 593, 3, 593, 3, 277, 3, 78, 3, 330, 3, 78, 249, 3,
    78, 249, 3, 78, 249, 3, 78, 249, 3, 325, 3, 277, 3, 78, 249, 3,
    1144, 3, 1300, 3, 1790, 3, 78, 249, 3, 330, 3, 277, 1499, 3, 1501, 3,
    3, 78, 200, 3, 78, 198, 3, 78, 198, 2103, 3, 3, 78, 3, 392, 78,
    3, 392, 78, 3, 2144, 251, 3, 3, 2162, 3, 2164, 3, 1147, 3, 1391, 3,
    78, 3, 2166, 3, 1400, 3, 1142, 3, 1968, 3, 1144, 3, 1140, 3, 2168, 3,
    2170, 3, 1309, 3, 2172, 3, 1398, 3, 225, 3, 1147, 3, 2172, 3, 1400, 3,
    1391, 3, 2174, 3, 504, 3, 1846, 3, 1846, 3, 330, 3, 204, 3, 277, 3,
    2195, 3, 392, 78, 3, 330, 3, 204, 3, 392, 78, 3, 277, 3, 1300, 3,
    78, 200, 3, 78, 200, 3, 1391, 3, 225, 3, 1147, 3, 2172, 3, 333, 78,
    3, 78, 915, 3, 1284, 3, 593, 3, 1286, 1070, 3, 209, 3, 3, 1850, 3,
    1253, 3, 1437, 3, 2290, 3, 2292, 3, 2294, 3, 2296, 3, 2298, 2300, 3, 3,
    2302, 3, 2304, 3, 2306, 2308, 3, 3, 2310, 3, 2312, 3, 2314, 3, 2316, 3,
    2318, 3, 2320, 3, 2322, 3, 2324, 3, 1257, 3, 1255, 3, 2326, 3, 2328, 3,
    2330, 3, 2332, 3, 2334, 3, 2336, 3, 2338, 3, 2340, 3, 2342, 3, 2344, 2346,
    3, 3, 2348, 3, 2350, 3, 2352, 3, 2354, 3, 2356, 3, 2358, 3, 2360, 3,
    2362, 3, 2364, 3, 2366, 3, 2368, 3, 2370, 3, 2372, 3, 2374, 3, 2376, 3,
    2378, 3, 2380, 3, 2382, 3, 2384, 3, 2386, 3, 2388, 3, 2390, 2308, 3, 2392,
    3, 3, 2324, 3, 2394, 3, 2396, 3, 2398, 3, 2400, 3, 2402, 3, 2404, 2406,
    3, 3, 2408, 3, 2410, 3, 2174, 3, 2412, 3, 2414, 3, 2416, 3, 2418, 2420,
    3, 3, 2422, 3, 2424, 3, 2426, 2428, 3, 3, 2430, 3, 2432, 3, 521, 251,
    3, 3, 4, 238, 3, 78, 3, 443, 78, 3, 901, 3, 904, 3, 901, 3,
    904, 3, 443, 78, 3, 392, 78, 3, 277, 3, 443, 78, 3, 901, 3, 904,
    3, 443, 78, 3, 904, 3, 1300, 3, 204, 3, 78, 249, 3, 1850, 3, 330,
    3, 330, 3, 330, 3, 330, 3, 330, 3, 330, 3, 330, 3, 330, 3, 330,
    3, 330, 3, 330, 3, 330, 3, 330, 3, 330, 3, 330, 3, 330, 3, 330,
    3, 330, 3, 330, 3, 330, 3, 330, 3, 330, 3, 330, 3, 330, 3, 330,
    3, 330, 3, 78, 249, 3, 654, 3, 277, 3, 325, 3, 78, 249, 1070, 3,
    1700, 3, 3, 1217, 3, 1219, 3, 2166, 3, 1400, 3, 1142, 3, 1968, 3, 1144,
    3, 1140, 3, 2168, 3, 2170, 3, 1309, 3, 2172, 3, 1398, 3, 1391, 3, 1393,
    3, 1395, 3, 225, 3, 1147, 3, 1400, 3, 1398, 3, 1968, 3, 504, 3, 1144,
    3, 2174, 3, 1391, 3, 78, 249, 3, 1150, 3, 1790, 3, 1140, 3, 225, 3,
    1147, 3, 2195, 3, 257, 78, 3, 277, 3, 277, 3, 277, 3, 277, 3, 1147,
    3, 1400, 3, 277, 3, 325, 3, 4, 2612, 3, 78, 85, 3, 2616, 3, 225,
    3, 2618, 3, 2623, 3, 2623, 3, 4, 5,
];

pub static REGEX_PATTERNS: &[&str] = &[
    r#"[A-Z_][A-Z0-9_]*"#, // [0]
    r#"^(NOT)$"#, // [1]
    r#"`?[a-zA-Z0-9_$]*`?"#, // [2]
    r#"([A-Z0-9_]*[A-Z][A-Z0-9_]*)|_"#, // [3]
    r#"^(ACCOUNT_LOCK|ACCOUNT_UNLOCK|ADD|ADMIN|ALL|ALTER|ANALYZE|ANALYZER|AND|ANTI|APPEND|AS|ASC|AUTO|BACKEND|BETWEEN|BIGINT|BINARY|BINLOG|BY|CANCEL|CASE|CAST|CLEAN|COLLATE|COLUMN|CONSTRAINT|CREATE|CROSS|CUBE|CURRENT|DATABASE|DATABASES|DECOMMISSION|DEFAULT|DELETE|DESC|DESCRIBE|DISK|DISTINCT|DISTRIBUTED|DISTRIBUTION|DIV|DOUBLE|DROP|DROPP|DUMP|DUPLICATE|ELSE|ENTER|EXCEPT|EXECUTE|EXISTS|EXPLAIN|EXPORT|EXTENDED|EXTRACT|FALSE|FLOAT|FOLLOWER|FOLLOWING|FOR|FORCE|FOREIGN|FROM|FRONTEND|FULL|FUNCTIONS|GRANT|GRANTS|GROUP|HAVING|HLL|IF|IN|INDEX|INFILE|INNER|INSERT|INSTALL|INT|INTEGER|INTERMEDIATE|INTERSECT|INTERVAL|INTO|IS|JOIN|KEY|KEYS|KILL|LARGEINT|LATERAL|LAYOUT|LEFT|LIKE|LIMIT|LIST|LOAD|LOW_PRIORITY|MATCH|MAXVALUE|MINUS|NATURAL|NOT|NO_USE_MV|NULL|OBSERVER|ON|OR|ORDER|OUTER|OUTFILE|OVER|OVERWRITE|PARTITION|PLACEHOLDER|PLAY|PRECEDING|PREPARE|PRIMARY|PROCEDURE|RANGE|READ|REAL|REBALANCE|REFERENCES|REGEXP|RELEASE|RENAME|REPAIR|REPLICA|REVOKE|RIGHT|ROLE|ROLES|ROW|ROWS|SCHEMAS|SELECT|SEMI|SET|SETS|SHOW|SIGNED|SMALLINT|SQL_BLOCK_RULE|SUPERUSER|SWITCH|SYNC|SYSTEM|TABLE|TABLESAMPLE|TABLET|TABLETS|TERMINATED|THEN|TINYINT|TO|TOKENIZER|TOKEN_FILTER|TRASH|TRIM|TRUE|TYPECAST|UNBOUNDED|UNINSTALL|UNION|UNIQUE|UNSIGNED|UPDATE|USE|USE_MV|USING|VALUES|WHEN|WHERE|WHITELIST|WITH|WORKLOAD|WRITE|XOR)$"#, // [4]
    r#"`?[A-Za-z0-9_]*`?"#, // [5]
    r#"[@][a-zA-Z0-9_$]*"#, // [6]
    r#"@@((session|global|local|persist|persist_only)\.)?[A-Za-z0-9_]+"#, // [7]
];

pub static HINT_STRING_INDICES: &[u32] = &[
    0, 6, 7, 8, 10, 11, 12, 13, 14, 15,
    16, 17, 18, 19, 20, 21, 22, 23, 24, 25,
    26, 27, 28, 29, 30, 31, 32, 33, 34, 35,
    36, 37, 25, 40, 37, 13, 14, 20, 21, 28,
    12, 26, 33, 35, 17, 52, 53, 53, 52, 57,
    11, 60, 62, 19, 15, 16, 18, 22, 23, 24,
    27, 28, 29, 30, 31, 32, 34, 36, 15, 16,
    67, 22, 23, 68, 69, 60, 70, 71, 72, 73,
    74, 75, 13, 28, 82, 87, 89, 91, 92, 95,
    97, 11, 98, 99, 100, 101, 102, 103, 104, 105,
    106, 40, 107, 108, 109, 110, 111, 112, 113, 114,
    115, 116, 117, 118, 119, 120, 121, 100, 13, 20,
    26, 123, 37, 123, 98, 22, 128, 102, 34, 105,
    133, 134, 118, 134, 118, 11, 98, 99, 100, 101,
    102, 103, 104, 106, 40, 107, 108, 109, 110, 111,
    112, 113, 114, 115, 116, 117, 118, 119, 120, 121,
    15, 16, 18, 22, 23, 137, 24, 27, 28, 29,
    30, 31, 32, 34, 36, 137, 119, 12, 28, 103,
    142, 99, 101, 104, 106, 107, 108, 109, 111, 110,
    112, 113, 114, 115, 116, 117, 120, 121, 164, 32,
    36, 30, 18, 24, 27, 29, 31, 23, 15, 16,
    177, 73, 75, 71, 67, 68, 69, 70, 72, 74,
    189, 190, 189, 190, 195, 201, 206, 207, 210, 211,
    212, 213, 214, 215, 216, 217, 218, 219, 220, 221,
    222, 224, 226, 231, 239, 242, 244, 247, 252, 253,
    254, 258, 259, 260, 261, 262, 109, 266, 269, 271,
    272, 273, 272, 273, 278, 279, 280, 281, 282, 283,
    284, 285, 286, 287, 288, 289, 290, 291, 292, 293,
    294, 295, 296, 297, 299, 300, 300, 299, 304, 306,
    308, 309, 311, 312, 312, 311, 315, 316, 315, 316,
    320, 322, 258, 259, 261, 261, 258, 259, 329, 260,
    261, 260, 261, 262, 262, 244, 242, 337, 261, 97,
    338, 339, 340, 341, 342, 343, 344, 345, 346, 224,
    340, 349, 342, 343, 341, 343, 338, 344, 338, 344,
    339, 345, 339, 345, 346, 224, 361, 364, 366, 226,
    367, 369, 261, 370, 371, 370, 369, 371, 367, 261,
    376, 377, 378, 377, 378, 376, 217, 106, 82, 219,
    388, 219, 388, 388, 231, 89, 396, 397, 398, 258,
    259, 399, 260, 320, 400, 401, 52, 261, 402, 262,
    403, 404, 405, 406, 407, 408, 409, 410, 411, 412,
    413, 414, 415, 416, 33, 417, 361, 423, 423, 425,
    426, 425, 426, 430, 431, 17, 432, 405, 433, 434,
    435, 436, 438, 405, 433, 405, 433, 434, 430, 431,
    430, 431, 448, 306, 448, 451, 452, 217, 454, 430,
    455, 456, 361, 457, 458, 459, 460, 461, 462, 463,
    116, 456, 458, 455, 462, 457, 454, 459, 460, 463,
    461, 474, 475, 474, 475, 478, 478, 481, 481, 484,
    485, 487, 488, 489, 489, 487, 488, 493, 496, 266,
    497, 497, 82, 112, 500, 501, 478, 505, 506, 508,
    509, 510, 510, 509, 514, 516, 517, 518, 519, 520,
    519, 520, 523, 524, 525, 526, 527, 506, 528, 529,
    530, 531, 532, 533, 534, 535, 536, 537, 529, 530,
    531, 528, 527, 537, 536, 532, 533, 534, 535, 551,
    553, 555, 557, 558, 559, 560, 561, 562, 563, 566,
    567, 568, 569, 570, 571, 572, 397, 574, 396, 575,
    398, 576, 577, 578, 579, 580, 581, 582, 583, 584,
    585, 590, 600, 601, 602, 603, 604, 605, 606, 607,
    608, 527, 506, 520, 528, 529, 530, 609, 610, 531,
    532, 329, 533, 519, 534, 611, 535, 612, 613, 536,
    537, 614, 615, 520, 519, 612, 613, 614, 608, 532,
    329, 533, 611, 615, 609, 610, 622, 623, 624, 625,
    626, 627, 628, 629, 630, 631, 636, 637, 638, 639,
    640, 612, 613, 614, 643, 644, 164, 645, 644, 164,
    644, 643, 645, 645, 643, 649, 652, 557, 655, 655,
    82, 112, 500, 501, 662, 663, 664, 665, 666, 667,
    668, 669, 670, 671, 672, 675, 676, 678, 679, 680,
    681, 682, 683, 687, 691, 693, 697, 698, 700, 701,
    702, 703, 704, 705, 706, 707, 709, 710, 258, 711,
    712, 713, 714, 715, 306, 715, 717, 718, 343, 719,
    717, 343, 719, 718, 724, 725, 259, 706, 728, 729,
    730, 731, 732, 730, 735, 732, 729, 731, 728, 741,
    742, 399, 728, 732, 744, 745, 746, 747, 748, 749,
    750, 751, 752, 753, 754, 755, 756, 757, 759, 761,
    762, 763, 764, 765, 306, 771, 771, 436, 231, 780,
    780, 783, 784, 783, 784, 787, 789, 790, 789, 790,
    261, 306, 771, 164, 580, 581, 582, 583, 584, 585,
    771, 244, 97, 242, 582, 583, 584, 585, 629, 797,
    798, 799, 800, 801, 802, 798, 799, 801, 802, 797,
    800, 811, 812, 814, 816, 817, 821, 822, 823, 824,
    608, 532, 615, 611, 834, 835, 838, 839, 840, 400,
    841, 842, 843, 844, 846, 848, 849, 850, 851, 854,
    855, 401, 856, 857, 858, 859, 860, 861, 862, 863,
    864, 865, 866, 867, 87, 868, 868, 871, 872, 873,
    874, 875, 876, 877, 880, 863, 18, 881, 882, 24,
    27, 883, 863, 881, 882, 883, 863, 881, 883, 881,
    883, 882, 258, 260, 261, 262, 891, 432, 435, 436,
    432, 435, 320, 404, 425, 407, 413, 868, 426, 478,
    369, 261, 370, 371, 905, 905, 612, 909, 910, 258,
    260, 262, 338, 339, 340, 341, 342, 343, 344, 345,
    346, 224, 916, 916, 82, 231, 320, 403, 920, 921,
    403, 666, 926, 927, 928, 929, 930, 931, 932, 222,
    933, 436, 933, 574, 474, 940, 943, 921, 920, 231,
    320, 403, 921, 219, 600, 101, 104, 117, 952, 954,
    955, 956, 955, 956, 960, 349, 478, 962, 244, 242,
    962, 411, 231, 349, 965, 965, 969, 744, 971, 377,
    972, 973, 974, 744, 971, 972, 973, 974, 972, 974,
    971, 973, 980, 981, 982, 983, 981, 982, 980, 983,
    988, 411, 989, 991, 261, 590, 992, 992, 278, 997,
    999, 1001, 261, 1002, 1002, 988, 989, 1006, 97, 1006,
    367, 1008, 1008, 567, 320, 1011, 1012, 1013, 1014, 1015,
    1016, 1017, 1017, 943, 1022, 582, 585, 583, 584, 1030,
    580, 1031, 1032, 1033, 581, 1034, 1035, 1036, 91, 1038,
    1038, 1040, 751, 1041, 1041, 1045, 1046, 1047, 1048, 1051,
    790, 1052, 1055, 1056, 1057, 1056, 1057, 1061, 607, 258,
    92, 1062, 1062, 607, 1056, 1057, 1066, 1066, 1071, 1072,
    1073, 1074, 1071, 1051, 790, 97, 242, 1071, 1051, 790,
    1051, 97, 242, 1081, 1082, 1083, 1084, 1085, 1086, 279,
    280, 281, 282, 1087, 1088, 1089, 1090, 1091, 1092, 1092,
    361, 1095, 1096, 1097, 1098, 1099, 1100, 866, 1103, 1104,
    1103, 1104, 1109, 1111, 1113, 1115, 1116, 500, 244, 97,
    242, 1120, 501, 1122, 1123, 1124, 1125, 1126, 1127, 1128,
    402, 1129, 1131, 1133, 1139, 1146, 1151, 1152, 1151, 1152,
    1155, 1157, 1158, 1159, 1160, 1161, 1162, 1163, 1164, 1165,
    1167, 1168, 1169, 1170, 1171, 1172, 1173, 1174, 1175, 1176,
    1177, 1178, 1179, 1180, 1181, 1182, 1183, 1184, 1186, 962,
    1186, 574, 97, 1189, 681, 1197, 1197, 369, 261, 370,
    18, 24, 1201, 1203, 1204, 1206, 1207, 1208, 1209, 1210,
    1211, 1212, 1213, 1214, 1215, 1220, 1221, 1222, 1223, 1224,
    1225, 1226, 404, 1229, 1230, 1231, 1232, 1234, 1235, 1236,
    1237, 1238, 1239, 1240, 1241, 1242, 1243, 1244, 1245, 1246,
    1247, 1248, 1249, 1250, 82, 101, 104, 112, 117, 500,
    501, 478, 1258, 920, 306, 1262, 1263, 62, 306, 604,
    306, 608, 529, 530, 306, 1273, 1152, 529, 530, 608,
    306, 1273, 608, 527, 506, 528, 529, 530, 531, 532,
    329, 533, 534, 611, 535, 536, 537, 615, 529, 530,
    1152, 1280, 1071, 643, 771, 1051, 790, 645, 244, 97,
    242, 500, 501, 1308, 1310, 1311, 1072, 1312, 1315, 1316,
    388, 1038, 954, 1317, 1317, 337, 478, 1320, 1321, 1323,
    1324, 1325, 1328, 1329, 1330, 1331, 1332, 1333, 1334, 1335,
    1336, 1339, 1341, 1344, 1346, 1349, 1352, 1354, 1356, 1357,
    730, 367, 201, 117, 1358, 1360, 1361, 1360, 1361, 1365,
    1367, 1368, 1369, 1370, 1317, 1357, 1372, 1372, 1370, 1376,
    1377, 1378, 1379, 1240, 1036, 730, 195, 1380, 1381, 485,
    1146, 367, 1382, 1383, 224, 1384, 417, 1139, 1385, 478,
    1380, 1384, 1385, 1240, 1381, 1382, 417, 1383, 1408, 1201,
    1133, 474, 119, 1415, 1420, 1421, 848, 652, 1422, 1423,
    1429, 1133, 1432, 5, 1045, 5, 1442, 1445, 1456, 1457,
    1458, 1459, 1460, 1461, 1462, 1465, 1467, 1468, 1467, 1468,
    700, 746, 812, 859, 860, 861, 1046, 1471, 1472, 1473,
    1474, 1475, 1476, 1475, 1473, 1471, 1472, 1474, 1476, 1490,
    1491, 1492, 1493, 1494, 1495, 1496, 1503, 1505, 1506, 1507,
    1509, 1036, 1380, 1146, 224, 1384, 1385, 1515, 1518, 1519,
    1520, 1521, 1036, 1146, 224, 1384, 1385, 1522, 1523, 1524,
    1525, 1527, 1528, 1529, 1530, 1531, 1532, 1533, 1534, 1535,
    1536, 283, 284, 285, 286, 1537, 1538, 1539, 1540, 1541,
    691, 1214, 693, 366, 1544, 1432, 1548, 1549, 1550, 1551,
    1552, 1553, 1554, 1555, 1556, 1557, 1558, 627, 639, 1559,
    1559, 1201, 1133, 119, 195, 1563, 224, 1563, 407, 1569,
    413, 668, 1186, 1573, 1574, 1575, 1576, 1577, 1578, 1579,
    1580, 1581, 1582, 849, 1583, 1584, 1586, 1588, 1589, 406,
    1590, 1591, 1592, 1593, 1594, 1595, 1596, 1597, 1598, 1599,
    1600, 1381, 1601, 1603, 814, 1321, 1607, 1220, 1609, 1609,
    1613, 1615, 1240, 1036, 91, 1381, 1146, 1382, 224, 1308,
    417, 1139, 1618, 1619, 1620, 1621, 1622, 1623, 1625, 1628,
    1629, 1630, 1631, 1632, 1633, 1635, 1636, 164, 868, 848,
    850, 851, 850, 851, 1652, 1653, 1655, 1656, 1659, 1246,
    1659, 261, 1235, 771, 1215, 1235, 1663, 1665, 1666, 1667,
    1668, 1669, 1670, 1671, 1672, 1673, 1674, 1675, 1676, 1677,
    1678, 1679, 1680, 1681, 1422, 1682, 1683, 1684, 1685, 1686,
    1687, 1688, 920, 1262, 1263, 1263, 1262, 87, 1383, 1694,
    1695, 1696, 1697, 1698, 1704, 1708, 1709, 842, 1131, 1133,
    801, 1613, 1317, 1714, 1715, 1716, 1717, 1718, 1719, 1720,
    1721, 1722, 1723, 1726, 1727, 1729, 1730, 1731, 1732, 1733,
    1734, 1735, 1736, 1737, 1738, 1739, 1740, 1741, 1742, 1743,
    1744, 1745, 1746, 1747, 1748, 1749, 1750, 1751, 1752, 1753,
    1754, 1755, 1756, 1757, 1758, 1759, 1760, 1761, 1762, 1763,
    1764, 408, 1765, 1766, 341, 1767, 1768, 1769, 1770, 1771,
    1772, 101, 117, 1791, 1792, 287, 1793, 1794, 1795, 1796,
    409, 1382, 288, 289, 290, 1797, 342, 1798, 1799, 291,
    1800, 1801, 1802, 1803, 1804, 1805, 1806, 1807, 1808, 1809,
    1810, 1811, 1816, 1818, 1819, 1821, 1822, 1823, 1824, 1825,
    1827, 1828, 1829, 1832, 1833, 1834, 1835, 1836, 1837, 1838,
    1839, 1840, 1841, 1842, 1843, 101, 104, 487, 117, 82,
    112, 1852, 1853, 346, 1854, 1855, 1856, 608, 1859, 1860,
    1861, 1862, 1863, 1864, 1865, 1866, 1867, 1874, 1875, 1876,
    1877, 1878, 1879, 1880, 1881, 1882, 1883, 1884, 1885, 1886,
    1887, 1888, 1889, 1890, 1891, 614, 613, 388, 1674, 1036,
    1380, 1146, 1384, 1370, 1317, 1429, 1385, 1897, 1898, 1899,
    1900, 1901, 1903, 1905, 1906, 1907, 1908, 410, 1909, 1910,
    1911, 1913, 1914, 1915, 1916, 1917, 1918, 1919, 1920, 1921,
    1922, 1923, 1697, 1370, 91, 474, 1931, 1932, 1933, 1934,
    1935, 531, 244, 242, 1938, 1939, 1940, 1941, 1942, 1943,
    1944, 1945, 1946, 1947, 1948, 1949, 1950, 1952, 1953, 1954,
    1955, 1956, 1957, 1958, 1959, 1960, 1961, 1962, 1963, 1969,
    1970, 1971, 1972, 1973, 1974, 1976, 1977, 1978, 1979, 62,
    1583, 1900, 1984, 1985, 1986, 1987, 1988, 1989, 1990, 607,
    1731, 600, 89, 11, 524, 1993, 1993, 1995, 292, 1996,
    1997, 1998, 1999, 2001, 2002, 82, 1687, 2005, 82, 341,
    278, 283, 288, 291, 293, 296, 296, 293, 2013, 2015,
    2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025,
    2026, 2027, 2028, 2029, 681, 343, 1197, 423, 2032, 848,
    1923, 2034, 2034, 2037, 2039, 2040, 2041, 2042, 2043, 2044,
    2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054,
    2055, 2056, 2057, 2058, 728, 732, 2059, 2059, 2061, 2063,
    2065, 1129, 1131, 82, 112, 423, 32, 500, 501, 478,
    82, 500, 501, 82, 112, 32, 500, 501, 478, 2067,
    2068, 2069, 2070, 2071, 2072, 2073, 2074, 2075, 2076, 2077,
    2078, 2079, 2080, 2081, 2083, 2083, 2084, 2084, 2087, 2088,
    2089, 2090, 2091, 2092, 2093, 2094, 2095, 2097, 2098, 2099,
    2100, 2104, 2105, 2108, 2109, 2110, 2111, 2112, 2113, 412,
    2114, 2115, 2116, 2117, 2118, 2119, 2120, 2121, 2122, 603,
    2124, 2124, 2128, 2129, 2130, 2131, 2132, 2133, 2134, 2135,
    2136, 2137, 294, 2138, 2139, 2140, 2141, 2142, 11, 1180,
    1183, 1528, 2145, 2146, 2147, 2148, 2149, 2150, 2151, 11,
    1180, 1183, 2151, 2150, 2145, 2146, 2147, 2149, 2148, 82,
    478, 2177, 2178, 2180, 2182, 2183, 2184, 2185, 2186, 1829,
    1882, 1494, 509, 1946, 1947, 2185, 1280, 329, 417, 1240,
    1381, 1494, 2185, 2202, 1632, 109, 109, 2065, 2183, 823,
    2205, 2205, 337, 121, 2209, 2210, 2211, 2212, 2213, 2214,
    2215, 2218, 2220, 2221, 2224, 2225, 2226, 2227, 2228, 2229,
    2230, 2231, 2232, 2233, 2234, 2235, 2236, 2237, 2238, 2239,
    2240, 2241, 2242, 2243, 2244, 2245, 2246, 2247, 2248, 2249,
    2250, 2251, 2252, 2253, 2254, 2255, 2256, 2257, 2258, 2259,
    2260, 2261, 2262, 2263, 2264, 2265, 2266, 2267, 2268, 2269,
    2270, 2271, 2272, 2273, 2274, 2275, 2276, 2277, 2278, 2279,
    2280, 2281, 2282, 2283, 2284, 2285, 2287, 2288, 2433, 2434,
    414, 415, 416, 2435, 2436, 2437, 2438, 2439, 2440, 2441,
    2442, 2444, 2445, 2446, 2447, 2448, 2449, 2450, 2451, 2452,
    2453, 2454, 2455, 2456, 2457, 2458, 2459, 2460, 2461, 2462,
    2463, 2464, 2466, 430, 431, 17, 434, 436, 430, 17,
    434, 436, 432, 435, 52, 405, 388, 1674, 343, 824,
    840, 261, 1209, 1331, 2028, 1178, 1772, 2491, 2492, 2493,
    2494, 2498, 113, 114, 113, 114, 2499, 2499, 2502, 2503,
    2504, 2505, 2507, 1073, 1074, 2508, 2509, 2510, 2511, 2512,
    2513, 2514, 2515, 2516, 2517, 2518, 1423, 2519, 2521, 2526,
    2527, 2528, 2529, 2530, 2531, 2532, 2533, 2534, 2535, 2536,
    2537, 2538, 2539, 2540, 2541, 2545, 2546, 2547, 2549, 2550,
    2551, 2555, 2556, 2557, 2558, 2559, 2560, 11, 1180, 2561,
    2562, 2563, 2564, 2565, 2566, 2567, 2568, 2569, 2570, 2571,
    2572, 2573, 2574, 2575, 2576, 2577, 2578, 2579, 2580, 2581,
    2582, 82, 954, 2583, 2584, 2585, 2586, 2587, 2588, 2589,
    2590, 2591, 2592, 2593, 2594, 2595, 2596, 2597, 2598, 2599,
    2601, 2602, 2603, 295, 2604, 2605, 2608, 1036, 195, 1380,
    485, 1146, 224, 1308, 1384, 1385, 2609, 2611, 2614, 224,
    474, 682, 509, 2626, 2627, 2628, 2629, 2630, 2631, 2632,
    297,
];

pub static SIMPLE_HINTS: &[SimpleHintData] = &[
    SimpleHintData::empty(), // [0] reserved for no hint
    SimpleHintData { raw_values_start: 0, raw_values_count: 1, token_types_start: 1, token_types_count: 0 }, // [1]
    SimpleHintData { raw_values_start: 1, raw_values_count: 1, token_types_start: 2, token_types_count: 0 }, // [2]
    SimpleHintData { raw_values_start: 2, raw_values_count: 1, token_types_start: 3, token_types_count: 0 }, // [3]
    SimpleHintData { raw_values_start: 3, raw_values_count: 1, token_types_start: 4, token_types_count: 0 }, // [4]
    SimpleHintData { raw_values_start: 4, raw_values_count: 1, token_types_start: 5, token_types_count: 0 }, // [5]
    SimpleHintData { raw_values_start: 5, raw_values_count: 27, token_types_start: 32, token_types_count: 0 }, // [6]
    SimpleHintData { raw_values_start: 32, raw_values_count: 1, token_types_start: 33, token_types_count: 0 }, // [7]
    SimpleHintData { raw_values_start: 33, raw_values_count: 1, token_types_start: 34, token_types_count: 0 }, // [8]
    SimpleHintData { raw_values_start: 34, raw_values_count: 1, token_types_start: 35, token_types_count: 0 }, // [9]
    SimpleHintData { raw_values_start: 35, raw_values_count: 1, token_types_start: 36, token_types_count: 0 }, // [10]
    SimpleHintData { raw_values_start: 36, raw_values_count: 1, token_types_start: 37, token_types_count: 0 }, // [11]
    SimpleHintData { raw_values_start: 37, raw_values_count: 1, token_types_start: 38, token_types_count: 0 }, // [12]
    SimpleHintData { raw_values_start: 38, raw_values_count: 1, token_types_start: 39, token_types_count: 0 }, // [13]
    SimpleHintData { raw_values_start: 39, raw_values_count: 1, token_types_start: 40, token_types_count: 0 }, // [14]
    SimpleHintData { raw_values_start: 40, raw_values_count: 1, token_types_start: 41, token_types_count: 0 }, // [15]
    SimpleHintData { raw_values_start: 41, raw_values_count: 1, token_types_start: 42, token_types_count: 0 }, // [16]
    SimpleHintData { raw_values_start: 42, raw_values_count: 1, token_types_start: 43, token_types_count: 0 }, // [17]
    SimpleHintData { raw_values_start: 43, raw_values_count: 1, token_types_start: 44, token_types_count: 0 }, // [18]
    SimpleHintData { raw_values_start: 44, raw_values_count: 1, token_types_start: 45, token_types_count: 0 }, // [19]
    SimpleHintData { raw_values_start: 45, raw_values_count: 2, token_types_start: 47, token_types_count: 0 }, // [20]
    SimpleHintData { raw_values_start: 47, raw_values_count: 1, token_types_start: 48, token_types_count: 0 }, // [21]
    SimpleHintData { raw_values_start: 48, raw_values_count: 1, token_types_start: 49, token_types_count: 0 }, // [22]
    SimpleHintData { raw_values_start: 49, raw_values_count: 1, token_types_start: 50, token_types_count: 0 }, // [23]
    SimpleHintData { raw_values_start: 50, raw_values_count: 1, token_types_start: 51, token_types_count: 0 }, // [24]
    SimpleHintData { raw_values_start: 51, raw_values_count: 1, token_types_start: 52, token_types_count: 0 }, // [25]
    SimpleHintData { raw_values_start: 52, raw_values_count: 1, token_types_start: 53, token_types_count: 0 }, // [26]
    SimpleHintData { raw_values_start: 53, raw_values_count: 1, token_types_start: 54, token_types_count: 0 }, // [27]
    SimpleHintData { raw_values_start: 54, raw_values_count: 14, token_types_start: 68, token_types_count: 0 }, // [28]
    SimpleHintData { raw_values_start: 68, raw_values_count: 14, token_types_start: 82, token_types_count: 0 }, // [29]
    SimpleHintData { raw_values_start: 82, raw_values_count: 2, token_types_start: 84, token_types_count: 0 }, // [30]
    SimpleHintData { raw_values_start: 84, raw_values_count: 1, token_types_start: 85, token_types_count: 0 }, // [31]
    SimpleHintData { raw_values_start: 85, raw_values_count: 1, token_types_start: 86, token_types_count: 0 }, // [32]
    SimpleHintData { raw_values_start: 86, raw_values_count: 1, token_types_start: 87, token_types_count: 0 }, // [33]
    SimpleHintData { raw_values_start: 87, raw_values_count: 1, token_types_start: 88, token_types_count: 0 }, // [34]
    SimpleHintData { raw_values_start: 88, raw_values_count: 1, token_types_start: 89, token_types_count: 0 }, // [35]
    SimpleHintData { raw_values_start: 89, raw_values_count: 1, token_types_start: 90, token_types_count: 0 }, // [36]
    SimpleHintData { raw_values_start: 90, raw_values_count: 0, token_types_start: 90, token_types_count: 1 }, // [37]
    SimpleHintData { raw_values_start: 91, raw_values_count: 26, token_types_start: 117, token_types_count: 0 }, // [38]
    SimpleHintData { raw_values_start: 117, raw_values_count: 1, token_types_start: 118, token_types_count: 0 }, // [39]
    SimpleHintData { raw_values_start: 118, raw_values_count: 5, token_types_start: 123, token_types_count: 0 }, // [40]
    SimpleHintData { raw_values_start: 123, raw_values_count: 1, token_types_start: 124, token_types_count: 0 }, // [41]
    SimpleHintData { raw_values_start: 124, raw_values_count: 1, token_types_start: 125, token_types_count: 0 }, // [42]
    SimpleHintData { raw_values_start: 125, raw_values_count: 1, token_types_start: 126, token_types_count: 0 }, // [43]
    SimpleHintData { raw_values_start: 126, raw_values_count: 1, token_types_start: 127, token_types_count: 0 }, // [44]
    SimpleHintData { raw_values_start: 127, raw_values_count: 1, token_types_start: 128, token_types_count: 0 }, // [45]
    SimpleHintData { raw_values_start: 128, raw_values_count: 1, token_types_start: 129, token_types_count: 0 }, // [46]
    SimpleHintData { raw_values_start: 129, raw_values_count: 1, token_types_start: 130, token_types_count: 0 }, // [47]
    SimpleHintData { raw_values_start: 130, raw_values_count: 1, token_types_start: 131, token_types_count: 0 }, // [48]
    SimpleHintData { raw_values_start: 131, raw_values_count: 2, token_types_start: 133, token_types_count: 0 }, // [49]
    SimpleHintData { raw_values_start: 133, raw_values_count: 1, token_types_start: 134, token_types_count: 0 }, // [50]
    SimpleHintData { raw_values_start: 134, raw_values_count: 1, token_types_start: 135, token_types_count: 0 }, // [51]
    SimpleHintData { raw_values_start: 135, raw_values_count: 25, token_types_start: 160, token_types_count: 0 }, // [52]
    SimpleHintData { raw_values_start: 160, raw_values_count: 15, token_types_start: 175, token_types_count: 0 }, // [53]
    SimpleHintData { raw_values_start: 175, raw_values_count: 1, token_types_start: 176, token_types_count: 0 }, // [54]
    SimpleHintData { raw_values_start: 176, raw_values_count: 1, token_types_start: 177, token_types_count: 0 }, // [55]
    SimpleHintData { raw_values_start: 177, raw_values_count: 2, token_types_start: 179, token_types_count: 0 }, // [56]
    SimpleHintData { raw_values_start: 179, raw_values_count: 1, token_types_start: 180, token_types_count: 0 }, // [57]
    SimpleHintData { raw_values_start: 180, raw_values_count: 1, token_types_start: 181, token_types_count: 0 }, // [58]
    SimpleHintData { raw_values_start: 181, raw_values_count: 1, token_types_start: 182, token_types_count: 0 }, // [59]
    SimpleHintData { raw_values_start: 182, raw_values_count: 1, token_types_start: 183, token_types_count: 0 }, // [60]
    SimpleHintData { raw_values_start: 183, raw_values_count: 1, token_types_start: 184, token_types_count: 0 }, // [61]
    SimpleHintData { raw_values_start: 184, raw_values_count: 1, token_types_start: 185, token_types_count: 0 }, // [62]
    SimpleHintData { raw_values_start: 185, raw_values_count: 1, token_types_start: 186, token_types_count: 0 }, // [63]
    SimpleHintData { raw_values_start: 186, raw_values_count: 1, token_types_start: 187, token_types_count: 0 }, // [64]
    SimpleHintData { raw_values_start: 187, raw_values_count: 1, token_types_start: 188, token_types_count: 0 }, // [65]
    SimpleHintData { raw_values_start: 188, raw_values_count: 1, token_types_start: 189, token_types_count: 0 }, // [66]
    SimpleHintData { raw_values_start: 189, raw_values_count: 1, token_types_start: 190, token_types_count: 0 }, // [67]
    SimpleHintData { raw_values_start: 190, raw_values_count: 1, token_types_start: 191, token_types_count: 0 }, // [68]
    SimpleHintData { raw_values_start: 191, raw_values_count: 1, token_types_start: 192, token_types_count: 0 }, // [69]
    SimpleHintData { raw_values_start: 192, raw_values_count: 1, token_types_start: 193, token_types_count: 0 }, // [70]
    SimpleHintData { raw_values_start: 193, raw_values_count: 1, token_types_start: 194, token_types_count: 0 }, // [71]
    SimpleHintData { raw_values_start: 194, raw_values_count: 1, token_types_start: 195, token_types_count: 0 }, // [72]
    SimpleHintData { raw_values_start: 195, raw_values_count: 1, token_types_start: 196, token_types_count: 0 }, // [73]
    SimpleHintData { raw_values_start: 196, raw_values_count: 1, token_types_start: 197, token_types_count: 0 }, // [74]
    SimpleHintData { raw_values_start: 197, raw_values_count: 1, token_types_start: 198, token_types_count: 0 }, // [75]
    SimpleHintData { raw_values_start: 198, raw_values_count: 1, token_types_start: 199, token_types_count: 0 }, // [76]
    SimpleHintData { raw_values_start: 199, raw_values_count: 1, token_types_start: 200, token_types_count: 0 }, // [77]
    SimpleHintData { raw_values_start: 200, raw_values_count: 1, token_types_start: 201, token_types_count: 0 }, // [78]
    SimpleHintData { raw_values_start: 201, raw_values_count: 1, token_types_start: 202, token_types_count: 0 }, // [79]
    SimpleHintData { raw_values_start: 202, raw_values_count: 1, token_types_start: 203, token_types_count: 0 }, // [80]
    SimpleHintData { raw_values_start: 203, raw_values_count: 1, token_types_start: 204, token_types_count: 0 }, // [81]
    SimpleHintData { raw_values_start: 204, raw_values_count: 1, token_types_start: 205, token_types_count: 0 }, // [82]
    SimpleHintData { raw_values_start: 205, raw_values_count: 1, token_types_start: 206, token_types_count: 0 }, // [83]
    SimpleHintData { raw_values_start: 206, raw_values_count: 1, token_types_start: 207, token_types_count: 0 }, // [84]
    SimpleHintData { raw_values_start: 207, raw_values_count: 1, token_types_start: 208, token_types_count: 0 }, // [85]
    SimpleHintData { raw_values_start: 208, raw_values_count: 1, token_types_start: 209, token_types_count: 0 }, // [86]
    SimpleHintData { raw_values_start: 209, raw_values_count: 1, token_types_start: 210, token_types_count: 0 }, // [87]
    SimpleHintData { raw_values_start: 210, raw_values_count: 1, token_types_start: 211, token_types_count: 0 }, // [88]
    SimpleHintData { raw_values_start: 211, raw_values_count: 1, token_types_start: 212, token_types_count: 0 }, // [89]
    SimpleHintData { raw_values_start: 212, raw_values_count: 1, token_types_start: 213, token_types_count: 0 }, // [90]
    SimpleHintData { raw_values_start: 213, raw_values_count: 1, token_types_start: 214, token_types_count: 0 }, // [91]
    SimpleHintData { raw_values_start: 214, raw_values_count: 1, token_types_start: 215, token_types_count: 0 }, // [92]
    SimpleHintData { raw_values_start: 215, raw_values_count: 1, token_types_start: 216, token_types_count: 0 }, // [93]
    SimpleHintData { raw_values_start: 216, raw_values_count: 1, token_types_start: 217, token_types_count: 0 }, // [94]
    SimpleHintData { raw_values_start: 217, raw_values_count: 1, token_types_start: 218, token_types_count: 0 }, // [95]
    SimpleHintData { raw_values_start: 218, raw_values_count: 1, token_types_start: 219, token_types_count: 0 }, // [96]
    SimpleHintData { raw_values_start: 219, raw_values_count: 1, token_types_start: 220, token_types_count: 0 }, // [97]
    SimpleHintData { raw_values_start: 220, raw_values_count: 2, token_types_start: 222, token_types_count: 0 }, // [98]
    SimpleHintData { raw_values_start: 222, raw_values_count: 1, token_types_start: 223, token_types_count: 0 }, // [99]
    SimpleHintData { raw_values_start: 223, raw_values_count: 1, token_types_start: 224, token_types_count: 0 }, // [100]
    SimpleHintData { raw_values_start: 224, raw_values_count: 1, token_types_start: 225, token_types_count: 0 }, // [101]
    SimpleHintData { raw_values_start: 225, raw_values_count: 1, token_types_start: 226, token_types_count: 0 }, // [102]
    SimpleHintData { raw_values_start: 226, raw_values_count: 1, token_types_start: 227, token_types_count: 0 }, // [103]
    SimpleHintData { raw_values_start: 227, raw_values_count: 1, token_types_start: 228, token_types_count: 0 }, // [104]
    SimpleHintData { raw_values_start: 228, raw_values_count: 1, token_types_start: 229, token_types_count: 0 }, // [105]
    SimpleHintData { raw_values_start: 229, raw_values_count: 1, token_types_start: 230, token_types_count: 0 }, // [106]
    SimpleHintData { raw_values_start: 230, raw_values_count: 1, token_types_start: 231, token_types_count: 0 }, // [107]
    SimpleHintData { raw_values_start: 231, raw_values_count: 1, token_types_start: 232, token_types_count: 0 }, // [108]
    SimpleHintData { raw_values_start: 232, raw_values_count: 1, token_types_start: 233, token_types_count: 0 }, // [109]
    SimpleHintData { raw_values_start: 233, raw_values_count: 1, token_types_start: 234, token_types_count: 0 }, // [110]
    SimpleHintData { raw_values_start: 234, raw_values_count: 1, token_types_start: 235, token_types_count: 0 }, // [111]
    SimpleHintData { raw_values_start: 235, raw_values_count: 1, token_types_start: 236, token_types_count: 0 }, // [112]
    SimpleHintData { raw_values_start: 236, raw_values_count: 1, token_types_start: 237, token_types_count: 0 }, // [113]
    SimpleHintData { raw_values_start: 237, raw_values_count: 1, token_types_start: 238, token_types_count: 0 }, // [114]
    SimpleHintData { raw_values_start: 238, raw_values_count: 1, token_types_start: 239, token_types_count: 0 }, // [115]
    SimpleHintData { raw_values_start: 239, raw_values_count: 1, token_types_start: 240, token_types_count: 0 }, // [116]
    SimpleHintData { raw_values_start: 240, raw_values_count: 1, token_types_start: 241, token_types_count: 0 }, // [117]
    SimpleHintData { raw_values_start: 241, raw_values_count: 1, token_types_start: 242, token_types_count: 0 }, // [118]
    SimpleHintData { raw_values_start: 242, raw_values_count: 1, token_types_start: 243, token_types_count: 0 }, // [119]
    SimpleHintData { raw_values_start: 243, raw_values_count: 1, token_types_start: 244, token_types_count: 0 }, // [120]
    SimpleHintData { raw_values_start: 244, raw_values_count: 1, token_types_start: 245, token_types_count: 0 }, // [121]
    SimpleHintData { raw_values_start: 245, raw_values_count: 0, token_types_start: 245, token_types_count: 1 }, // [122]
    SimpleHintData { raw_values_start: 246, raw_values_count: 0, token_types_start: 246, token_types_count: 1 }, // [123]
    SimpleHintData { raw_values_start: 247, raw_values_count: 1, token_types_start: 248, token_types_count: 0 }, // [124]
    SimpleHintData { raw_values_start: 248, raw_values_count: 1, token_types_start: 249, token_types_count: 0 }, // [125]
    SimpleHintData { raw_values_start: 249, raw_values_count: 1, token_types_start: 250, token_types_count: 0 }, // [126]
    SimpleHintData { raw_values_start: 250, raw_values_count: 1, token_types_start: 251, token_types_count: 0 }, // [127]
    SimpleHintData { raw_values_start: 251, raw_values_count: 6, token_types_start: 257, token_types_count: 0 }, // [128]
    SimpleHintData { raw_values_start: 257, raw_values_count: 1, token_types_start: 258, token_types_count: 0 }, // [129]
    SimpleHintData { raw_values_start: 258, raw_values_count: 1, token_types_start: 259, token_types_count: 0 }, // [130]
    SimpleHintData { raw_values_start: 259, raw_values_count: 1, token_types_start: 260, token_types_count: 0 }, // [131]
    SimpleHintData { raw_values_start: 260, raw_values_count: 2, token_types_start: 262, token_types_count: 0 }, // [132]
    SimpleHintData { raw_values_start: 262, raw_values_count: 1, token_types_start: 263, token_types_count: 0 }, // [133]
    SimpleHintData { raw_values_start: 263, raw_values_count: 1, token_types_start: 264, token_types_count: 0 }, // [134]
    SimpleHintData { raw_values_start: 264, raw_values_count: 20, token_types_start: 284, token_types_count: 0 }, // [135]
    SimpleHintData { raw_values_start: 284, raw_values_count: 2, token_types_start: 286, token_types_count: 0 }, // [136]
    SimpleHintData { raw_values_start: 286, raw_values_count: 1, token_types_start: 287, token_types_count: 0 }, // [137]
    SimpleHintData { raw_values_start: 287, raw_values_count: 1, token_types_start: 288, token_types_count: 0 }, // [138]
    SimpleHintData { raw_values_start: 288, raw_values_count: 1, token_types_start: 289, token_types_count: 0 }, // [139]
    SimpleHintData { raw_values_start: 289, raw_values_count: 1, token_types_start: 290, token_types_count: 0 }, // [140]
    SimpleHintData { raw_values_start: 290, raw_values_count: 1, token_types_start: 291, token_types_count: 0 }, // [141]
    SimpleHintData { raw_values_start: 291, raw_values_count: 1, token_types_start: 292, token_types_count: 0 }, // [142]
    SimpleHintData { raw_values_start: 292, raw_values_count: 2, token_types_start: 294, token_types_count: 0 }, // [143]
    SimpleHintData { raw_values_start: 294, raw_values_count: 1, token_types_start: 295, token_types_count: 0 }, // [144]
    SimpleHintData { raw_values_start: 295, raw_values_count: 1, token_types_start: 296, token_types_count: 0 }, // [145]
    SimpleHintData { raw_values_start: 296, raw_values_count: 2, token_types_start: 298, token_types_count: 0 }, // [146]
    SimpleHintData { raw_values_start: 298, raw_values_count: 1, token_types_start: 299, token_types_count: 0 }, // [147]
    SimpleHintData { raw_values_start: 299, raw_values_count: 1, token_types_start: 300, token_types_count: 0 }, // [148]
    SimpleHintData { raw_values_start: 300, raw_values_count: 1, token_types_start: 301, token_types_count: 0 }, // [149]
    SimpleHintData { raw_values_start: 301, raw_values_count: 1, token_types_start: 302, token_types_count: 0 }, // [150]
    SimpleHintData { raw_values_start: 302, raw_values_count: 3, token_types_start: 305, token_types_count: 0 }, // [151]
    SimpleHintData { raw_values_start: 305, raw_values_count: 1, token_types_start: 306, token_types_count: 0 }, // [152]
    SimpleHintData { raw_values_start: 306, raw_values_count: 2, token_types_start: 308, token_types_count: 0 }, // [153]
    SimpleHintData { raw_values_start: 308, raw_values_count: 1, token_types_start: 309, token_types_count: 0 }, // [154]
    SimpleHintData { raw_values_start: 309, raw_values_count: 2, token_types_start: 311, token_types_count: 0 }, // [155]
    SimpleHintData { raw_values_start: 311, raw_values_count: 1, token_types_start: 312, token_types_count: 0 }, // [156]
    SimpleHintData { raw_values_start: 312, raw_values_count: 2, token_types_start: 314, token_types_count: 0 }, // [157]
    SimpleHintData { raw_values_start: 314, raw_values_count: 1, token_types_start: 315, token_types_count: 0 }, // [158]
    SimpleHintData { raw_values_start: 315, raw_values_count: 0, token_types_start: 315, token_types_count: 2 }, // [159]
    SimpleHintData { raw_values_start: 317, raw_values_count: 1, token_types_start: 318, token_types_count: 0 }, // [160]
    SimpleHintData { raw_values_start: 318, raw_values_count: 1, token_types_start: 319, token_types_count: 1 }, // [161]
    SimpleHintData { raw_values_start: 320, raw_values_count: 10, token_types_start: 330, token_types_count: 0 }, // [162]
    SimpleHintData { raw_values_start: 330, raw_values_count: 1, token_types_start: 331, token_types_count: 0 }, // [163]
    SimpleHintData { raw_values_start: 331, raw_values_count: 1, token_types_start: 332, token_types_count: 0 }, // [164]
    SimpleHintData { raw_values_start: 332, raw_values_count: 2, token_types_start: 334, token_types_count: 0 }, // [165]
    SimpleHintData { raw_values_start: 334, raw_values_count: 2, token_types_start: 336, token_types_count: 0 }, // [166]
    SimpleHintData { raw_values_start: 336, raw_values_count: 2, token_types_start: 338, token_types_count: 0 }, // [167]
    SimpleHintData { raw_values_start: 338, raw_values_count: 1, token_types_start: 339, token_types_count: 0 }, // [168]
    SimpleHintData { raw_values_start: 339, raw_values_count: 1, token_types_start: 340, token_types_count: 0 }, // [169]
    SimpleHintData { raw_values_start: 340, raw_values_count: 2, token_types_start: 342, token_types_count: 0 }, // [170]
    SimpleHintData { raw_values_start: 342, raw_values_count: 1, token_types_start: 343, token_types_count: 0 }, // [171]
    SimpleHintData { raw_values_start: 343, raw_values_count: 1, token_types_start: 344, token_types_count: 0 }, // [172]
    SimpleHintData { raw_values_start: 344, raw_values_count: 2, token_types_start: 346, token_types_count: 0 }, // [173]
    SimpleHintData { raw_values_start: 346, raw_values_count: 1, token_types_start: 347, token_types_count: 0 }, // [174]
    SimpleHintData { raw_values_start: 347, raw_values_count: 1, token_types_start: 348, token_types_count: 0 }, // [175]
    SimpleHintData { raw_values_start: 348, raw_values_count: 1, token_types_start: 349, token_types_count: 0 }, // [176]
    SimpleHintData { raw_values_start: 349, raw_values_count: 2, token_types_start: 351, token_types_count: 0 }, // [177]
    SimpleHintData { raw_values_start: 351, raw_values_count: 4, token_types_start: 355, token_types_count: 0 }, // [178]
    SimpleHintData { raw_values_start: 355, raw_values_count: 1, token_types_start: 356, token_types_count: 0 }, // [179]
    SimpleHintData { raw_values_start: 356, raw_values_count: 1, token_types_start: 357, token_types_count: 0 }, // [180]
    SimpleHintData { raw_values_start: 357, raw_values_count: 1, token_types_start: 358, token_types_count: 0 }, // [181]
    SimpleHintData { raw_values_start: 358, raw_values_count: 1, token_types_start: 359, token_types_count: 0 }, // [182]
    SimpleHintData { raw_values_start: 359, raw_values_count: 4, token_types_start: 363, token_types_count: 0 }, // [183]
    SimpleHintData { raw_values_start: 363, raw_values_count: 1, token_types_start: 364, token_types_count: 0 }, // [184]
    SimpleHintData { raw_values_start: 364, raw_values_count: 1, token_types_start: 365, token_types_count: 0 }, // [185]
    SimpleHintData { raw_values_start: 365, raw_values_count: 1, token_types_start: 366, token_types_count: 0 }, // [186]
    SimpleHintData { raw_values_start: 366, raw_values_count: 2, token_types_start: 368, token_types_count: 0 }, // [187]
    SimpleHintData { raw_values_start: 368, raw_values_count: 3, token_types_start: 371, token_types_count: 0 }, // [188]
    SimpleHintData { raw_values_start: 371, raw_values_count: 2, token_types_start: 373, token_types_count: 0 }, // [189]
    SimpleHintData { raw_values_start: 373, raw_values_count: 1, token_types_start: 374, token_types_count: 0 }, // [190]
    SimpleHintData { raw_values_start: 374, raw_values_count: 2, token_types_start: 376, token_types_count: 0 }, // [191]
    SimpleHintData { raw_values_start: 376, raw_values_count: 30, token_types_start: 406, token_types_count: 0 }, // [192]
    SimpleHintData { raw_values_start: 406, raw_values_count: 2, token_types_start: 408, token_types_count: 0 }, // [193]
    SimpleHintData { raw_values_start: 408, raw_values_count: 1, token_types_start: 409, token_types_count: 0 }, // [194]
    SimpleHintData { raw_values_start: 409, raw_values_count: 2, token_types_start: 411, token_types_count: 0 }, // [195]
    SimpleHintData { raw_values_start: 411, raw_values_count: 1, token_types_start: 412, token_types_count: 0 }, // [196]
    SimpleHintData { raw_values_start: 412, raw_values_count: 1, token_types_start: 413, token_types_count: 0 }, // [197]
    SimpleHintData { raw_values_start: 413, raw_values_count: 9, token_types_start: 422, token_types_count: 0 }, // [198]
    SimpleHintData { raw_values_start: 422, raw_values_count: 1, token_types_start: 423, token_types_count: 0 }, // [199]
    SimpleHintData { raw_values_start: 423, raw_values_count: 2, token_types_start: 425, token_types_count: 0 }, // [200]
    SimpleHintData { raw_values_start: 425, raw_values_count: 1, token_types_start: 426, token_types_count: 0 }, // [201]
    SimpleHintData { raw_values_start: 426, raw_values_count: 1, token_types_start: 427, token_types_count: 0 }, // [202]
    SimpleHintData { raw_values_start: 427, raw_values_count: 1, token_types_start: 428, token_types_count: 0 }, // [203]
    SimpleHintData { raw_values_start: 428, raw_values_count: 2, token_types_start: 430, token_types_count: 0 }, // [204]
    SimpleHintData { raw_values_start: 430, raw_values_count: 1, token_types_start: 431, token_types_count: 0 }, // [205]
    SimpleHintData { raw_values_start: 431, raw_values_count: 1, token_types_start: 432, token_types_count: 0 }, // [206]
    SimpleHintData { raw_values_start: 432, raw_values_count: 2, token_types_start: 434, token_types_count: 0 }, // [207]
    SimpleHintData { raw_values_start: 434, raw_values_count: 1, token_types_start: 435, token_types_count: 0 }, // [208]
    SimpleHintData { raw_values_start: 435, raw_values_count: 1, token_types_start: 436, token_types_count: 0 }, // [209]
    SimpleHintData { raw_values_start: 436, raw_values_count: 1, token_types_start: 437, token_types_count: 0 }, // [210]
    SimpleHintData { raw_values_start: 437, raw_values_count: 14, token_types_start: 451, token_types_count: 0 }, // [211]
    SimpleHintData { raw_values_start: 451, raw_values_count: 1, token_types_start: 452, token_types_count: 0 }, // [212]
    SimpleHintData { raw_values_start: 452, raw_values_count: 1, token_types_start: 453, token_types_count: 0 }, // [213]
    SimpleHintData { raw_values_start: 453, raw_values_count: 1, token_types_start: 454, token_types_count: 0 }, // [214]
    SimpleHintData { raw_values_start: 454, raw_values_count: 1, token_types_start: 455, token_types_count: 0 }, // [215]
    SimpleHintData { raw_values_start: 455, raw_values_count: 1, token_types_start: 456, token_types_count: 0 }, // [216]
    SimpleHintData { raw_values_start: 456, raw_values_count: 1, token_types_start: 457, token_types_count: 0 }, // [217]
    SimpleHintData { raw_values_start: 457, raw_values_count: 1, token_types_start: 458, token_types_count: 0 }, // [218]
    SimpleHintData { raw_values_start: 458, raw_values_count: 1, token_types_start: 459, token_types_count: 0 }, // [219]
    SimpleHintData { raw_values_start: 459, raw_values_count: 1, token_types_start: 460, token_types_count: 0 }, // [220]
    SimpleHintData { raw_values_start: 460, raw_values_count: 1, token_types_start: 461, token_types_count: 0 }, // [221]
    SimpleHintData { raw_values_start: 461, raw_values_count: 2, token_types_start: 463, token_types_count: 0 }, // [222]
    SimpleHintData { raw_values_start: 463, raw_values_count: 1, token_types_start: 464, token_types_count: 0 }, // [223]
    SimpleHintData { raw_values_start: 464, raw_values_count: 1, token_types_start: 465, token_types_count: 0 }, // [224]
    SimpleHintData { raw_values_start: 465, raw_values_count: 1, token_types_start: 466, token_types_count: 0 }, // [225]
    SimpleHintData { raw_values_start: 466, raw_values_count: 2, token_types_start: 468, token_types_count: 0 }, // [226]
    SimpleHintData { raw_values_start: 468, raw_values_count: 1, token_types_start: 469, token_types_count: 0 }, // [227]
    SimpleHintData { raw_values_start: 469, raw_values_count: 1, token_types_start: 470, token_types_count: 0 }, // [228]
    SimpleHintData { raw_values_start: 470, raw_values_count: 1, token_types_start: 471, token_types_count: 0 }, // [229]
    SimpleHintData { raw_values_start: 471, raw_values_count: 3, token_types_start: 474, token_types_count: 0 }, // [230]
    SimpleHintData { raw_values_start: 474, raw_values_count: 1, token_types_start: 475, token_types_count: 0 }, // [231]
    SimpleHintData { raw_values_start: 475, raw_values_count: 1, token_types_start: 476, token_types_count: 0 }, // [232]
    SimpleHintData { raw_values_start: 476, raw_values_count: 1, token_types_start: 477, token_types_count: 0 }, // [233]
    SimpleHintData { raw_values_start: 477, raw_values_count: 1, token_types_start: 478, token_types_count: 0 }, // [234]
    SimpleHintData { raw_values_start: 478, raw_values_count: 1, token_types_start: 479, token_types_count: 0 }, // [235]
    SimpleHintData { raw_values_start: 479, raw_values_count: 2, token_types_start: 481, token_types_count: 0 }, // [236]
    SimpleHintData { raw_values_start: 481, raw_values_count: 1, token_types_start: 482, token_types_count: 0 }, // [237]
    SimpleHintData { raw_values_start: 482, raw_values_count: 5, token_types_start: 487, token_types_count: 0 }, // [238]
    SimpleHintData { raw_values_start: 487, raw_values_count: 1, token_types_start: 488, token_types_count: 0 }, // [239]
    SimpleHintData { raw_values_start: 488, raw_values_count: 1, token_types_start: 489, token_types_count: 0 }, // [240]
    SimpleHintData { raw_values_start: 489, raw_values_count: 1, token_types_start: 490, token_types_count: 0 }, // [241]
    SimpleHintData { raw_values_start: 490, raw_values_count: 2, token_types_start: 492, token_types_count: 0 }, // [242]
    SimpleHintData { raw_values_start: 492, raw_values_count: 1, token_types_start: 493, token_types_count: 0 }, // [243]
    SimpleHintData { raw_values_start: 493, raw_values_count: 1, token_types_start: 494, token_types_count: 0 }, // [244]
    SimpleHintData { raw_values_start: 494, raw_values_count: 1, token_types_start: 495, token_types_count: 0 }, // [245]
    SimpleHintData { raw_values_start: 495, raw_values_count: 1, token_types_start: 496, token_types_count: 0 }, // [246]
    SimpleHintData { raw_values_start: 496, raw_values_count: 1, token_types_start: 497, token_types_count: 0 }, // [247]
    SimpleHintData { raw_values_start: 497, raw_values_count: 1, token_types_start: 498, token_types_count: 0 }, // [248]
    SimpleHintData { raw_values_start: 498, raw_values_count: 1, token_types_start: 499, token_types_count: 0 }, // [249]
    SimpleHintData { raw_values_start: 499, raw_values_count: 2, token_types_start: 501, token_types_count: 0 }, // [250]
    SimpleHintData { raw_values_start: 501, raw_values_count: 1, token_types_start: 502, token_types_count: 0 }, // [251]
    SimpleHintData { raw_values_start: 502, raw_values_count: 1, token_types_start: 503, token_types_count: 0 }, // [252]
    SimpleHintData { raw_values_start: 503, raw_values_count: 1, token_types_start: 504, token_types_count: 0 }, // [253]
    SimpleHintData { raw_values_start: 504, raw_values_count: 1, token_types_start: 505, token_types_count: 0 }, // [254]
    SimpleHintData { raw_values_start: 505, raw_values_count: 1, token_types_start: 506, token_types_count: 0 }, // [255]
    SimpleHintData { raw_values_start: 506, raw_values_count: 12, token_types_start: 518, token_types_count: 0 }, // [256]
    SimpleHintData { raw_values_start: 518, raw_values_count: 1, token_types_start: 519, token_types_count: 0 }, // [257]
    SimpleHintData { raw_values_start: 519, raw_values_count: 1, token_types_start: 520, token_types_count: 0 }, // [258]
    SimpleHintData { raw_values_start: 520, raw_values_count: 1, token_types_start: 521, token_types_count: 0 }, // [259]
    SimpleHintData { raw_values_start: 521, raw_values_count: 1, token_types_start: 522, token_types_count: 0 }, // [260]
    SimpleHintData { raw_values_start: 522, raw_values_count: 1, token_types_start: 523, token_types_count: 0 }, // [261]
    SimpleHintData { raw_values_start: 523, raw_values_count: 1, token_types_start: 524, token_types_count: 0 }, // [262]
    SimpleHintData { raw_values_start: 524, raw_values_count: 1, token_types_start: 525, token_types_count: 0 }, // [263]
    SimpleHintData { raw_values_start: 525, raw_values_count: 1, token_types_start: 526, token_types_count: 0 }, // [264]
    SimpleHintData { raw_values_start: 526, raw_values_count: 1, token_types_start: 527, token_types_count: 0 }, // [265]
    SimpleHintData { raw_values_start: 527, raw_values_count: 1, token_types_start: 528, token_types_count: 0 }, // [266]
    SimpleHintData { raw_values_start: 528, raw_values_count: 1, token_types_start: 529, token_types_count: 0 }, // [267]
    SimpleHintData { raw_values_start: 529, raw_values_count: 1, token_types_start: 530, token_types_count: 0 }, // [268]
    SimpleHintData { raw_values_start: 530, raw_values_count: 1, token_types_start: 531, token_types_count: 0 }, // [269]
    SimpleHintData { raw_values_start: 531, raw_values_count: 1, token_types_start: 532, token_types_count: 0 }, // [270]
    SimpleHintData { raw_values_start: 532, raw_values_count: 1, token_types_start: 533, token_types_count: 0 }, // [271]
    SimpleHintData { raw_values_start: 533, raw_values_count: 1, token_types_start: 534, token_types_count: 0 }, // [272]
    SimpleHintData { raw_values_start: 534, raw_values_count: 1, token_types_start: 535, token_types_count: 0 }, // [273]
    SimpleHintData { raw_values_start: 535, raw_values_count: 1, token_types_start: 536, token_types_count: 0 }, // [274]
    SimpleHintData { raw_values_start: 536, raw_values_count: 1, token_types_start: 537, token_types_count: 0 }, // [275]
    SimpleHintData { raw_values_start: 537, raw_values_count: 1, token_types_start: 538, token_types_count: 0 }, // [276]
    SimpleHintData { raw_values_start: 538, raw_values_count: 0, token_types_start: 538, token_types_count: 1 }, // [277]
    SimpleHintData { raw_values_start: 539, raw_values_count: 1, token_types_start: 540, token_types_count: 0 }, // [278]
    SimpleHintData { raw_values_start: 540, raw_values_count: 1, token_types_start: 541, token_types_count: 0 }, // [279]
    SimpleHintData { raw_values_start: 541, raw_values_count: 1, token_types_start: 542, token_types_count: 0 }, // [280]
    SimpleHintData { raw_values_start: 542, raw_values_count: 1, token_types_start: 543, token_types_count: 0 }, // [281]
    SimpleHintData { raw_values_start: 543, raw_values_count: 1, token_types_start: 544, token_types_count: 0 }, // [282]
    SimpleHintData { raw_values_start: 544, raw_values_count: 1, token_types_start: 545, token_types_count: 0 }, // [283]
    SimpleHintData { raw_values_start: 545, raw_values_count: 1, token_types_start: 546, token_types_count: 0 }, // [284]
    SimpleHintData { raw_values_start: 546, raw_values_count: 1, token_types_start: 547, token_types_count: 0 }, // [285]
    SimpleHintData { raw_values_start: 547, raw_values_count: 1, token_types_start: 548, token_types_count: 0 }, // [286]
    SimpleHintData { raw_values_start: 548, raw_values_count: 1, token_types_start: 549, token_types_count: 0 }, // [287]
    SimpleHintData { raw_values_start: 549, raw_values_count: 1, token_types_start: 550, token_types_count: 0 }, // [288]
    SimpleHintData { raw_values_start: 550, raw_values_count: 1, token_types_start: 551, token_types_count: 0 }, // [289]
    SimpleHintData { raw_values_start: 551, raw_values_count: 1, token_types_start: 552, token_types_count: 0 }, // [290]
    SimpleHintData { raw_values_start: 552, raw_values_count: 1, token_types_start: 553, token_types_count: 0 }, // [291]
    SimpleHintData { raw_values_start: 553, raw_values_count: 1, token_types_start: 554, token_types_count: 0 }, // [292]
    SimpleHintData { raw_values_start: 554, raw_values_count: 1, token_types_start: 555, token_types_count: 0 }, // [293]
    SimpleHintData { raw_values_start: 555, raw_values_count: 6, token_types_start: 561, token_types_count: 0 }, // [294]
    SimpleHintData { raw_values_start: 561, raw_values_count: 1, token_types_start: 562, token_types_count: 0 }, // [295]
    SimpleHintData { raw_values_start: 562, raw_values_count: 1, token_types_start: 563, token_types_count: 0 }, // [296]
    SimpleHintData { raw_values_start: 563, raw_values_count: 1, token_types_start: 564, token_types_count: 0 }, // [297]
    SimpleHintData { raw_values_start: 564, raw_values_count: 1, token_types_start: 565, token_types_count: 0 }, // [298]
    SimpleHintData { raw_values_start: 565, raw_values_count: 1, token_types_start: 566, token_types_count: 0 }, // [299]
    SimpleHintData { raw_values_start: 566, raw_values_count: 1, token_types_start: 567, token_types_count: 0 }, // [300]
    SimpleHintData { raw_values_start: 567, raw_values_count: 1, token_types_start: 568, token_types_count: 0 }, // [301]
    SimpleHintData { raw_values_start: 568, raw_values_count: 1, token_types_start: 569, token_types_count: 0 }, // [302]
    SimpleHintData { raw_values_start: 569, raw_values_count: 1, token_types_start: 570, token_types_count: 0 }, // [303]
    SimpleHintData { raw_values_start: 570, raw_values_count: 22, token_types_start: 592, token_types_count: 1 }, // [304]
    SimpleHintData { raw_values_start: 593, raw_values_count: 5, token_types_start: 598, token_types_count: 0 }, // [305]
    SimpleHintData { raw_values_start: 598, raw_values_count: 5, token_types_start: 603, token_types_count: 1 }, // [306]
    SimpleHintData { raw_values_start: 604, raw_values_count: 1, token_types_start: 605, token_types_count: 0 }, // [307]
    SimpleHintData { raw_values_start: 605, raw_values_count: 1, token_types_start: 606, token_types_count: 0 }, // [308]
    SimpleHintData { raw_values_start: 606, raw_values_count: 1, token_types_start: 607, token_types_count: 0 }, // [309]
    SimpleHintData { raw_values_start: 607, raw_values_count: 1, token_types_start: 608, token_types_count: 0 }, // [310]
    SimpleHintData { raw_values_start: 608, raw_values_count: 1, token_types_start: 609, token_types_count: 0 }, // [311]
    SimpleHintData { raw_values_start: 609, raw_values_count: 1, token_types_start: 610, token_types_count: 0 }, // [312]
    SimpleHintData { raw_values_start: 610, raw_values_count: 1, token_types_start: 611, token_types_count: 0 }, // [313]
    SimpleHintData { raw_values_start: 611, raw_values_count: 1, token_types_start: 612, token_types_count: 0 }, // [314]
    SimpleHintData { raw_values_start: 612, raw_values_count: 1, token_types_start: 613, token_types_count: 0 }, // [315]
    SimpleHintData { raw_values_start: 613, raw_values_count: 1, token_types_start: 614, token_types_count: 0 }, // [316]
    SimpleHintData { raw_values_start: 614, raw_values_count: 1, token_types_start: 615, token_types_count: 0 }, // [317]
    SimpleHintData { raw_values_start: 615, raw_values_count: 1, token_types_start: 616, token_types_count: 0 }, // [318]
    SimpleHintData { raw_values_start: 616, raw_values_count: 1, token_types_start: 617, token_types_count: 0 }, // [319]
    SimpleHintData { raw_values_start: 617, raw_values_count: 1, token_types_start: 618, token_types_count: 0 }, // [320]
    SimpleHintData { raw_values_start: 618, raw_values_count: 1, token_types_start: 619, token_types_count: 0 }, // [321]
    SimpleHintData { raw_values_start: 619, raw_values_count: 1, token_types_start: 620, token_types_count: 0 }, // [322]
    SimpleHintData { raw_values_start: 620, raw_values_count: 1, token_types_start: 621, token_types_count: 0 }, // [323]
    SimpleHintData { raw_values_start: 621, raw_values_count: 3, token_types_start: 624, token_types_count: 0 }, // [324]
    SimpleHintData { raw_values_start: 624, raw_values_count: 4, token_types_start: 628, token_types_count: 0 }, // [325]
    SimpleHintData { raw_values_start: 628, raw_values_count: 2, token_types_start: 630, token_types_count: 0 }, // [326]
    SimpleHintData { raw_values_start: 630, raw_values_count: 1, token_types_start: 631, token_types_count: 0 }, // [327]
    SimpleHintData { raw_values_start: 631, raw_values_count: 2, token_types_start: 633, token_types_count: 0 }, // [328]
    SimpleHintData { raw_values_start: 633, raw_values_count: 1, token_types_start: 634, token_types_count: 0 }, // [329]
    SimpleHintData { raw_values_start: 634, raw_values_count: 1, token_types_start: 635, token_types_count: 0 }, // [330]
    SimpleHintData { raw_values_start: 635, raw_values_count: 1, token_types_start: 636, token_types_count: 0 }, // [331]
    SimpleHintData { raw_values_start: 636, raw_values_count: 1, token_types_start: 637, token_types_count: 0 }, // [332]
    SimpleHintData { raw_values_start: 637, raw_values_count: 2, token_types_start: 639, token_types_count: 0 }, // [333]
    SimpleHintData { raw_values_start: 639, raw_values_count: 1, token_types_start: 640, token_types_count: 0 }, // [334]
    SimpleHintData { raw_values_start: 640, raw_values_count: 4, token_types_start: 644, token_types_count: 0 }, // [335]
    SimpleHintData { raw_values_start: 644, raw_values_count: 1, token_types_start: 645, token_types_count: 0 }, // [336]
    SimpleHintData { raw_values_start: 645, raw_values_count: 1, token_types_start: 646, token_types_count: 0 }, // [337]
    SimpleHintData { raw_values_start: 646, raw_values_count: 1, token_types_start: 647, token_types_count: 0 }, // [338]
    SimpleHintData { raw_values_start: 647, raw_values_count: 1, token_types_start: 648, token_types_count: 0 }, // [339]
    SimpleHintData { raw_values_start: 648, raw_values_count: 1, token_types_start: 649, token_types_count: 0 }, // [340]
    SimpleHintData { raw_values_start: 649, raw_values_count: 1, token_types_start: 650, token_types_count: 0 }, // [341]
    SimpleHintData { raw_values_start: 650, raw_values_count: 1, token_types_start: 651, token_types_count: 0 }, // [342]
    SimpleHintData { raw_values_start: 651, raw_values_count: 1, token_types_start: 652, token_types_count: 0 }, // [343]
    SimpleHintData { raw_values_start: 652, raw_values_count: 1, token_types_start: 653, token_types_count: 0 }, // [344]
    SimpleHintData { raw_values_start: 653, raw_values_count: 1, token_types_start: 654, token_types_count: 0 }, // [345]
    SimpleHintData { raw_values_start: 654, raw_values_count: 1, token_types_start: 655, token_types_count: 0 }, // [346]
    SimpleHintData { raw_values_start: 655, raw_values_count: 1, token_types_start: 656, token_types_count: 0 }, // [347]
    SimpleHintData { raw_values_start: 656, raw_values_count: 1, token_types_start: 657, token_types_count: 0 }, // [348]
    SimpleHintData { raw_values_start: 657, raw_values_count: 1, token_types_start: 658, token_types_count: 0 }, // [349]
    SimpleHintData { raw_values_start: 658, raw_values_count: 1, token_types_start: 659, token_types_count: 0 }, // [350]
    SimpleHintData { raw_values_start: 659, raw_values_count: 1, token_types_start: 660, token_types_count: 0 }, // [351]
    SimpleHintData { raw_values_start: 660, raw_values_count: 1, token_types_start: 661, token_types_count: 0 }, // [352]
    SimpleHintData { raw_values_start: 661, raw_values_count: 1, token_types_start: 662, token_types_count: 0 }, // [353]
    SimpleHintData { raw_values_start: 662, raw_values_count: 1, token_types_start: 663, token_types_count: 0 }, // [354]
    SimpleHintData { raw_values_start: 663, raw_values_count: 1, token_types_start: 664, token_types_count: 0 }, // [355]
    SimpleHintData { raw_values_start: 664, raw_values_count: 1, token_types_start: 665, token_types_count: 0 }, // [356]
    SimpleHintData { raw_values_start: 665, raw_values_count: 1, token_types_start: 666, token_types_count: 0 }, // [357]
    SimpleHintData { raw_values_start: 666, raw_values_count: 1, token_types_start: 667, token_types_count: 0 }, // [358]
    SimpleHintData { raw_values_start: 667, raw_values_count: 1, token_types_start: 668, token_types_count: 0 }, // [359]
    SimpleHintData { raw_values_start: 668, raw_values_count: 1, token_types_start: 669, token_types_count: 0 }, // [360]
    SimpleHintData { raw_values_start: 669, raw_values_count: 1, token_types_start: 670, token_types_count: 0 }, // [361]
    SimpleHintData { raw_values_start: 670, raw_values_count: 1, token_types_start: 671, token_types_count: 0 }, // [362]
    SimpleHintData { raw_values_start: 671, raw_values_count: 1, token_types_start: 672, token_types_count: 0 }, // [363]
    SimpleHintData { raw_values_start: 672, raw_values_count: 1, token_types_start: 673, token_types_count: 0 }, // [364]
    SimpleHintData { raw_values_start: 673, raw_values_count: 1, token_types_start: 674, token_types_count: 0 }, // [365]
    SimpleHintData { raw_values_start: 674, raw_values_count: 1, token_types_start: 675, token_types_count: 0 }, // [366]
    SimpleHintData { raw_values_start: 675, raw_values_count: 1, token_types_start: 676, token_types_count: 0 }, // [367]
    SimpleHintData { raw_values_start: 676, raw_values_count: 1, token_types_start: 677, token_types_count: 0 }, // [368]
    SimpleHintData { raw_values_start: 677, raw_values_count: 1, token_types_start: 678, token_types_count: 0 }, // [369]
    SimpleHintData { raw_values_start: 678, raw_values_count: 1, token_types_start: 679, token_types_count: 0 }, // [370]
    SimpleHintData { raw_values_start: 679, raw_values_count: 1, token_types_start: 680, token_types_count: 0 }, // [371]
    SimpleHintData { raw_values_start: 680, raw_values_count: 1, token_types_start: 681, token_types_count: 0 }, // [372]
    SimpleHintData { raw_values_start: 681, raw_values_count: 1, token_types_start: 682, token_types_count: 0 }, // [373]
    SimpleHintData { raw_values_start: 682, raw_values_count: 1, token_types_start: 683, token_types_count: 0 }, // [374]
    SimpleHintData { raw_values_start: 683, raw_values_count: 2, token_types_start: 685, token_types_count: 0 }, // [375]
    SimpleHintData { raw_values_start: 685, raw_values_count: 1, token_types_start: 686, token_types_count: 0 }, // [376]
    SimpleHintData { raw_values_start: 686, raw_values_count: 4, token_types_start: 690, token_types_count: 0 }, // [377]
    SimpleHintData { raw_values_start: 690, raw_values_count: 1, token_types_start: 691, token_types_count: 0 }, // [378]
    SimpleHintData { raw_values_start: 691, raw_values_count: 1, token_types_start: 692, token_types_count: 0 }, // [379]
    SimpleHintData { raw_values_start: 692, raw_values_count: 1, token_types_start: 693, token_types_count: 0 }, // [380]
    SimpleHintData { raw_values_start: 693, raw_values_count: 1, token_types_start: 694, token_types_count: 0 }, // [381]
    SimpleHintData { raw_values_start: 694, raw_values_count: 1, token_types_start: 695, token_types_count: 0 }, // [382]
    SimpleHintData { raw_values_start: 695, raw_values_count: 1, token_types_start: 696, token_types_count: 0 }, // [383]
    SimpleHintData { raw_values_start: 696, raw_values_count: 1, token_types_start: 697, token_types_count: 0 }, // [384]
    SimpleHintData { raw_values_start: 697, raw_values_count: 6, token_types_start: 703, token_types_count: 0 }, // [385]
    SimpleHintData { raw_values_start: 703, raw_values_count: 1, token_types_start: 704, token_types_count: 0 }, // [386]
    SimpleHintData { raw_values_start: 704, raw_values_count: 1, token_types_start: 705, token_types_count: 0 }, // [387]
    SimpleHintData { raw_values_start: 705, raw_values_count: 1, token_types_start: 706, token_types_count: 0 }, // [388]
    SimpleHintData { raw_values_start: 706, raw_values_count: 1, token_types_start: 707, token_types_count: 0 }, // [389]
    SimpleHintData { raw_values_start: 707, raw_values_count: 1, token_types_start: 708, token_types_count: 0 }, // [390]
    SimpleHintData { raw_values_start: 708, raw_values_count: 1, token_types_start: 709, token_types_count: 0 }, // [391]
    SimpleHintData { raw_values_start: 709, raw_values_count: 1, token_types_start: 710, token_types_count: 0 }, // [392]
    SimpleHintData { raw_values_start: 710, raw_values_count: 1, token_types_start: 711, token_types_count: 0 }, // [393]
    SimpleHintData { raw_values_start: 711, raw_values_count: 1, token_types_start: 712, token_types_count: 0 }, // [394]
    SimpleHintData { raw_values_start: 712, raw_values_count: 2, token_types_start: 714, token_types_count: 0 }, // [395]
    SimpleHintData { raw_values_start: 714, raw_values_count: 1, token_types_start: 715, token_types_count: 0 }, // [396]
    SimpleHintData { raw_values_start: 715, raw_values_count: 1, token_types_start: 716, token_types_count: 0 }, // [397]
    SimpleHintData { raw_values_start: 716, raw_values_count: 1, token_types_start: 717, token_types_count: 0 }, // [398]
    SimpleHintData { raw_values_start: 717, raw_values_count: 1, token_types_start: 718, token_types_count: 0 }, // [399]
    SimpleHintData { raw_values_start: 718, raw_values_count: 1, token_types_start: 719, token_types_count: 0 }, // [400]
    SimpleHintData { raw_values_start: 719, raw_values_count: 1, token_types_start: 720, token_types_count: 0 }, // [401]
    SimpleHintData { raw_values_start: 720, raw_values_count: 1, token_types_start: 721, token_types_count: 0 }, // [402]
    SimpleHintData { raw_values_start: 721, raw_values_count: 1, token_types_start: 722, token_types_count: 0 }, // [403]
    SimpleHintData { raw_values_start: 722, raw_values_count: 1, token_types_start: 723, token_types_count: 0 }, // [404]
    SimpleHintData { raw_values_start: 723, raw_values_count: 1, token_types_start: 724, token_types_count: 0 }, // [405]
    SimpleHintData { raw_values_start: 724, raw_values_count: 1, token_types_start: 725, token_types_count: 0 }, // [406]
    SimpleHintData { raw_values_start: 725, raw_values_count: 1, token_types_start: 726, token_types_count: 0 }, // [407]
    SimpleHintData { raw_values_start: 726, raw_values_count: 1, token_types_start: 727, token_types_count: 0 }, // [408]
    SimpleHintData { raw_values_start: 727, raw_values_count: 1, token_types_start: 728, token_types_count: 0 }, // [409]
    SimpleHintData { raw_values_start: 728, raw_values_count: 1, token_types_start: 729, token_types_count: 0 }, // [410]
    SimpleHintData { raw_values_start: 729, raw_values_count: 1, token_types_start: 730, token_types_count: 0 }, // [411]
    SimpleHintData { raw_values_start: 730, raw_values_count: 1, token_types_start: 731, token_types_count: 0 }, // [412]
    SimpleHintData { raw_values_start: 731, raw_values_count: 1, token_types_start: 732, token_types_count: 0 }, // [413]
    SimpleHintData { raw_values_start: 732, raw_values_count: 1, token_types_start: 733, token_types_count: 0 }, // [414]
    SimpleHintData { raw_values_start: 733, raw_values_count: 1, token_types_start: 734, token_types_count: 0 }, // [415]
    SimpleHintData { raw_values_start: 734, raw_values_count: 2, token_types_start: 736, token_types_count: 0 }, // [416]
    SimpleHintData { raw_values_start: 736, raw_values_count: 1, token_types_start: 737, token_types_count: 0 }, // [417]
    SimpleHintData { raw_values_start: 737, raw_values_count: 1, token_types_start: 738, token_types_count: 0 }, // [418]
    SimpleHintData { raw_values_start: 738, raw_values_count: 2, token_types_start: 740, token_types_count: 0 }, // [419]
    SimpleHintData { raw_values_start: 740, raw_values_count: 1, token_types_start: 741, token_types_count: 0 }, // [420]
    SimpleHintData { raw_values_start: 741, raw_values_count: 2, token_types_start: 743, token_types_count: 0 }, // [421]
    SimpleHintData { raw_values_start: 743, raw_values_count: 1, token_types_start: 744, token_types_count: 0 }, // [422]
    SimpleHintData { raw_values_start: 744, raw_values_count: 1, token_types_start: 745, token_types_count: 0 }, // [423]
    SimpleHintData { raw_values_start: 745, raw_values_count: 1, token_types_start: 746, token_types_count: 0 }, // [424]
    SimpleHintData { raw_values_start: 746, raw_values_count: 2, token_types_start: 748, token_types_count: 0 }, // [425]
    SimpleHintData { raw_values_start: 748, raw_values_count: 1, token_types_start: 749, token_types_count: 0 }, // [426]
    SimpleHintData { raw_values_start: 749, raw_values_count: 1, token_types_start: 750, token_types_count: 0 }, // [427]
    SimpleHintData { raw_values_start: 750, raw_values_count: 4, token_types_start: 754, token_types_count: 0 }, // [428]
    SimpleHintData { raw_values_start: 754, raw_values_count: 7, token_types_start: 761, token_types_count: 3 }, // [429]
    SimpleHintData { raw_values_start: 764, raw_values_count: 4, token_types_start: 768, token_types_count: 0 }, // [430]
    SimpleHintData { raw_values_start: 768, raw_values_count: 7, token_types_start: 775, token_types_count: 0 }, // [431]
    SimpleHintData { raw_values_start: 775, raw_values_count: 1, token_types_start: 776, token_types_count: 0 }, // [432]
    SimpleHintData { raw_values_start: 776, raw_values_count: 1, token_types_start: 777, token_types_count: 0 }, // [433]
    SimpleHintData { raw_values_start: 777, raw_values_count: 1, token_types_start: 778, token_types_count: 0 }, // [434]
    SimpleHintData { raw_values_start: 778, raw_values_count: 1, token_types_start: 779, token_types_count: 0 }, // [435]
    SimpleHintData { raw_values_start: 779, raw_values_count: 1, token_types_start: 780, token_types_count: 0 }, // [436]
    SimpleHintData { raw_values_start: 780, raw_values_count: 1, token_types_start: 781, token_types_count: 0 }, // [437]
    SimpleHintData { raw_values_start: 781, raw_values_count: 1, token_types_start: 782, token_types_count: 0 }, // [438]
    SimpleHintData { raw_values_start: 782, raw_values_count: 1, token_types_start: 783, token_types_count: 0 }, // [439]
    SimpleHintData { raw_values_start: 783, raw_values_count: 1, token_types_start: 784, token_types_count: 0 }, // [440]
    SimpleHintData { raw_values_start: 784, raw_values_count: 1, token_types_start: 785, token_types_count: 0 }, // [441]
    SimpleHintData { raw_values_start: 785, raw_values_count: 1, token_types_start: 786, token_types_count: 0 }, // [442]
    SimpleHintData { raw_values_start: 786, raw_values_count: 1, token_types_start: 787, token_types_count: 0 }, // [443]
    SimpleHintData { raw_values_start: 787, raw_values_count: 1, token_types_start: 788, token_types_count: 0 }, // [444]
    SimpleHintData { raw_values_start: 788, raw_values_count: 1, token_types_start: 789, token_types_count: 0 }, // [445]
    SimpleHintData { raw_values_start: 789, raw_values_count: 1, token_types_start: 790, token_types_count: 0 }, // [446]
    SimpleHintData { raw_values_start: 790, raw_values_count: 2, token_types_start: 792, token_types_count: 0 }, // [447]
    SimpleHintData { raw_values_start: 792, raw_values_count: 0, token_types_start: 792, token_types_count: 1 }, // [448]
    SimpleHintData { raw_values_start: 793, raw_values_count: 1, token_types_start: 794, token_types_count: 0 }, // [449]
    SimpleHintData { raw_values_start: 794, raw_values_count: 1, token_types_start: 795, token_types_count: 0 }, // [450]
    SimpleHintData { raw_values_start: 795, raw_values_count: 1, token_types_start: 796, token_types_count: 0 }, // [451]
    SimpleHintData { raw_values_start: 796, raw_values_count: 1, token_types_start: 797, token_types_count: 0 }, // [452]
    SimpleHintData { raw_values_start: 797, raw_values_count: 1, token_types_start: 798, token_types_count: 0 }, // [453]
    SimpleHintData { raw_values_start: 798, raw_values_count: 1, token_types_start: 799, token_types_count: 0 }, // [454]
    SimpleHintData { raw_values_start: 799, raw_values_count: 1, token_types_start: 800, token_types_count: 0 }, // [455]
    SimpleHintData { raw_values_start: 800, raw_values_count: 1, token_types_start: 801, token_types_count: 0 }, // [456]
    SimpleHintData { raw_values_start: 801, raw_values_count: 1, token_types_start: 802, token_types_count: 0 }, // [457]
    SimpleHintData { raw_values_start: 802, raw_values_count: 1, token_types_start: 803, token_types_count: 0 }, // [458]
    SimpleHintData { raw_values_start: 803, raw_values_count: 1, token_types_start: 804, token_types_count: 0 }, // [459]
    SimpleHintData { raw_values_start: 804, raw_values_count: 1, token_types_start: 805, token_types_count: 0 }, // [460]
    SimpleHintData { raw_values_start: 805, raw_values_count: 4, token_types_start: 809, token_types_count: 0 }, // [461]
    SimpleHintData { raw_values_start: 809, raw_values_count: 1, token_types_start: 810, token_types_count: 0 }, // [462]
    SimpleHintData { raw_values_start: 810, raw_values_count: 1, token_types_start: 811, token_types_count: 0 }, // [463]
    SimpleHintData { raw_values_start: 811, raw_values_count: 1, token_types_start: 812, token_types_count: 0 }, // [464]
    SimpleHintData { raw_values_start: 812, raw_values_count: 1, token_types_start: 813, token_types_count: 0 }, // [465]
    SimpleHintData { raw_values_start: 813, raw_values_count: 1, token_types_start: 814, token_types_count: 0 }, // [466]
    SimpleHintData { raw_values_start: 814, raw_values_count: 1, token_types_start: 815, token_types_count: 0 }, // [467]
    SimpleHintData { raw_values_start: 815, raw_values_count: 1, token_types_start: 816, token_types_count: 0 }, // [468]
    SimpleHintData { raw_values_start: 816, raw_values_count: 1, token_types_start: 817, token_types_count: 0 }, // [469]
    SimpleHintData { raw_values_start: 817, raw_values_count: 1, token_types_start: 818, token_types_count: 0 }, // [470]
    SimpleHintData { raw_values_start: 818, raw_values_count: 1, token_types_start: 819, token_types_count: 0 }, // [471]
    SimpleHintData { raw_values_start: 819, raw_values_count: 1, token_types_start: 820, token_types_count: 0 }, // [472]
    SimpleHintData { raw_values_start: 820, raw_values_count: 1, token_types_start: 821, token_types_count: 0 }, // [473]
    SimpleHintData { raw_values_start: 821, raw_values_count: 1, token_types_start: 822, token_types_count: 0 }, // [474]
    SimpleHintData { raw_values_start: 822, raw_values_count: 1, token_types_start: 823, token_types_count: 0 }, // [475]
    SimpleHintData { raw_values_start: 823, raw_values_count: 1, token_types_start: 824, token_types_count: 0 }, // [476]
    SimpleHintData { raw_values_start: 824, raw_values_count: 2, token_types_start: 826, token_types_count: 0 }, // [477]
    SimpleHintData { raw_values_start: 826, raw_values_count: 1, token_types_start: 827, token_types_count: 0 }, // [478]
    SimpleHintData { raw_values_start: 827, raw_values_count: 1, token_types_start: 828, token_types_count: 0 }, // [479]
    SimpleHintData { raw_values_start: 828, raw_values_count: 1, token_types_start: 829, token_types_count: 0 }, // [480]
    SimpleHintData { raw_values_start: 829, raw_values_count: 1, token_types_start: 830, token_types_count: 0 }, // [481]
    SimpleHintData { raw_values_start: 830, raw_values_count: 1, token_types_start: 831, token_types_count: 0 }, // [482]
    SimpleHintData { raw_values_start: 831, raw_values_count: 1, token_types_start: 832, token_types_count: 0 }, // [483]
    SimpleHintData { raw_values_start: 832, raw_values_count: 1, token_types_start: 833, token_types_count: 0 }, // [484]
    SimpleHintData { raw_values_start: 833, raw_values_count: 1, token_types_start: 834, token_types_count: 0 }, // [485]
    SimpleHintData { raw_values_start: 834, raw_values_count: 1, token_types_start: 835, token_types_count: 0 }, // [486]
    SimpleHintData { raw_values_start: 835, raw_values_count: 7, token_types_start: 842, token_types_count: 0 }, // [487]
    SimpleHintData { raw_values_start: 842, raw_values_count: 4, token_types_start: 846, token_types_count: 0 }, // [488]
    SimpleHintData { raw_values_start: 846, raw_values_count: 3, token_types_start: 849, token_types_count: 0 }, // [489]
    SimpleHintData { raw_values_start: 849, raw_values_count: 1, token_types_start: 850, token_types_count: 0 }, // [490]
    SimpleHintData { raw_values_start: 850, raw_values_count: 1, token_types_start: 851, token_types_count: 0 }, // [491]
    SimpleHintData { raw_values_start: 851, raw_values_count: 1, token_types_start: 852, token_types_count: 0 }, // [492]
    SimpleHintData { raw_values_start: 852, raw_values_count: 4, token_types_start: 856, token_types_count: 0 }, // [493]
    SimpleHintData { raw_values_start: 856, raw_values_count: 1, token_types_start: 857, token_types_count: 0 }, // [494]
    SimpleHintData { raw_values_start: 857, raw_values_count: 3, token_types_start: 860, token_types_count: 0 }, // [495]
    SimpleHintData { raw_values_start: 860, raw_values_count: 1, token_types_start: 861, token_types_count: 0 }, // [496]
    SimpleHintData { raw_values_start: 861, raw_values_count: 1, token_types_start: 862, token_types_count: 0 }, // [497]
    SimpleHintData { raw_values_start: 862, raw_values_count: 8, token_types_start: 870, token_types_count: 0 }, // [498]
    SimpleHintData { raw_values_start: 870, raw_values_count: 5, token_types_start: 875, token_types_count: 0 }, // [499]
    SimpleHintData { raw_values_start: 875, raw_values_count: 1, token_types_start: 876, token_types_count: 0 }, // [500]
    SimpleHintData { raw_values_start: 876, raw_values_count: 1, token_types_start: 877, token_types_count: 0 }, // [501]
    SimpleHintData { raw_values_start: 877, raw_values_count: 1, token_types_start: 878, token_types_count: 0 }, // [502]
    SimpleHintData { raw_values_start: 878, raw_values_count: 1, token_types_start: 879, token_types_count: 0 }, // [503]
    SimpleHintData { raw_values_start: 879, raw_values_count: 3, token_types_start: 882, token_types_count: 0 }, // [504]
    SimpleHintData { raw_values_start: 882, raw_values_count: 11, token_types_start: 893, token_types_count: 0 }, // [505]
    SimpleHintData { raw_values_start: 893, raw_values_count: 1, token_types_start: 894, token_types_count: 0 }, // [506]
    SimpleHintData { raw_values_start: 894, raw_values_count: 6, token_types_start: 900, token_types_count: 0 }, // [507]
    SimpleHintData { raw_values_start: 900, raw_values_count: 1, token_types_start: 901, token_types_count: 0 }, // [508]
    SimpleHintData { raw_values_start: 901, raw_values_count: 8, token_types_start: 909, token_types_count: 0 }, // [509]
    SimpleHintData { raw_values_start: 909, raw_values_count: 3, token_types_start: 912, token_types_count: 0 }, // [510]
    SimpleHintData { raw_values_start: 912, raw_values_count: 1, token_types_start: 913, token_types_count: 0 }, // [511]
    SimpleHintData { raw_values_start: 913, raw_values_count: 2, token_types_start: 915, token_types_count: 0 }, // [512]
    SimpleHintData { raw_values_start: 915, raw_values_count: 1, token_types_start: 916, token_types_count: 0 }, // [513]
    SimpleHintData { raw_values_start: 916, raw_values_count: 1, token_types_start: 917, token_types_count: 0 }, // [514]
    SimpleHintData { raw_values_start: 917, raw_values_count: 1, token_types_start: 918, token_types_count: 0 }, // [515]
    SimpleHintData { raw_values_start: 918, raw_values_count: 1, token_types_start: 919, token_types_count: 0 }, // [516]
    SimpleHintData { raw_values_start: 919, raw_values_count: 4, token_types_start: 923, token_types_count: 0 }, // [517]
    SimpleHintData { raw_values_start: 923, raw_values_count: 2, token_types_start: 925, token_types_count: 0 }, // [518]
    SimpleHintData { raw_values_start: 925, raw_values_count: 3, token_types_start: 928, token_types_count: 0 }, // [519]
    SimpleHintData { raw_values_start: 928, raw_values_count: 1, token_types_start: 929, token_types_count: 0 }, // [520]
    SimpleHintData { raw_values_start: 929, raw_values_count: 1, token_types_start: 930, token_types_count: 0 }, // [521]
    SimpleHintData { raw_values_start: 930, raw_values_count: 2, token_types_start: 932, token_types_count: 0 }, // [522]
    SimpleHintData { raw_values_start: 932, raw_values_count: 1, token_types_start: 933, token_types_count: 0 }, // [523]
    SimpleHintData { raw_values_start: 933, raw_values_count: 1, token_types_start: 934, token_types_count: 0 }, // [524]
    SimpleHintData { raw_values_start: 934, raw_values_count: 1, token_types_start: 935, token_types_count: 0 }, // [525]
    SimpleHintData { raw_values_start: 935, raw_values_count: 2, token_types_start: 937, token_types_count: 0 }, // [526]
    SimpleHintData { raw_values_start: 937, raw_values_count: 1, token_types_start: 938, token_types_count: 2 }, // [527]
    SimpleHintData { raw_values_start: 940, raw_values_count: 1, token_types_start: 941, token_types_count: 0 }, // [528]
    SimpleHintData { raw_values_start: 941, raw_values_count: 1, token_types_start: 942, token_types_count: 0 }, // [529]
    SimpleHintData { raw_values_start: 942, raw_values_count: 3, token_types_start: 945, token_types_count: 0 }, // [530]
    SimpleHintData { raw_values_start: 945, raw_values_count: 1, token_types_start: 946, token_types_count: 0 }, // [531]
    SimpleHintData { raw_values_start: 946, raw_values_count: 1, token_types_start: 947, token_types_count: 0 }, // [532]
    SimpleHintData { raw_values_start: 947, raw_values_count: 6, token_types_start: 953, token_types_count: 0 }, // [533]
    SimpleHintData { raw_values_start: 953, raw_values_count: 5, token_types_start: 958, token_types_count: 0 }, // [534]
    SimpleHintData { raw_values_start: 958, raw_values_count: 1, token_types_start: 959, token_types_count: 0 }, // [535]
    SimpleHintData { raw_values_start: 959, raw_values_count: 1, token_types_start: 960, token_types_count: 0 }, // [536]
    SimpleHintData { raw_values_start: 960, raw_values_count: 1, token_types_start: 961, token_types_count: 0 }, // [537]
    SimpleHintData { raw_values_start: 961, raw_values_count: 1, token_types_start: 962, token_types_count: 0 }, // [538]
    SimpleHintData { raw_values_start: 962, raw_values_count: 4, token_types_start: 966, token_types_count: 0 }, // [539]
    SimpleHintData { raw_values_start: 966, raw_values_count: 1, token_types_start: 967, token_types_count: 0 }, // [540]
    SimpleHintData { raw_values_start: 967, raw_values_count: 1, token_types_start: 968, token_types_count: 0 }, // [541]
    SimpleHintData { raw_values_start: 968, raw_values_count: 1, token_types_start: 969, token_types_count: 0 }, // [542]
    SimpleHintData { raw_values_start: 969, raw_values_count: 1, token_types_start: 970, token_types_count: 0 }, // [543]
    SimpleHintData { raw_values_start: 970, raw_values_count: 3, token_types_start: 973, token_types_count: 0 }, // [544]
    SimpleHintData { raw_values_start: 973, raw_values_count: 1, token_types_start: 974, token_types_count: 0 }, // [545]
    SimpleHintData { raw_values_start: 974, raw_values_count: 3, token_types_start: 977, token_types_count: 0 }, // [546]
    SimpleHintData { raw_values_start: 977, raw_values_count: 1, token_types_start: 978, token_types_count: 0 }, // [547]
    SimpleHintData { raw_values_start: 978, raw_values_count: 1, token_types_start: 979, token_types_count: 0 }, // [548]
    SimpleHintData { raw_values_start: 979, raw_values_count: 1, token_types_start: 980, token_types_count: 0 }, // [549]
    SimpleHintData { raw_values_start: 980, raw_values_count: 1, token_types_start: 981, token_types_count: 0 }, // [550]
    SimpleHintData { raw_values_start: 981, raw_values_count: 1, token_types_start: 982, token_types_count: 0 }, // [551]
    SimpleHintData { raw_values_start: 982, raw_values_count: 2, token_types_start: 984, token_types_count: 0 }, // [552]
    SimpleHintData { raw_values_start: 984, raw_values_count: 1, token_types_start: 985, token_types_count: 0 }, // [553]
    SimpleHintData { raw_values_start: 985, raw_values_count: 1, token_types_start: 986, token_types_count: 0 }, // [554]
    SimpleHintData { raw_values_start: 986, raw_values_count: 1, token_types_start: 987, token_types_count: 0 }, // [555]
    SimpleHintData { raw_values_start: 987, raw_values_count: 1, token_types_start: 988, token_types_count: 1 }, // [556]
    SimpleHintData { raw_values_start: 989, raw_values_count: 1, token_types_start: 990, token_types_count: 0 }, // [557]
    SimpleHintData { raw_values_start: 990, raw_values_count: 2, token_types_start: 992, token_types_count: 0 }, // [558]
    SimpleHintData { raw_values_start: 992, raw_values_count: 1, token_types_start: 993, token_types_count: 0 }, // [559]
    SimpleHintData { raw_values_start: 993, raw_values_count: 2, token_types_start: 995, token_types_count: 0 }, // [560]
    SimpleHintData { raw_values_start: 995, raw_values_count: 1, token_types_start: 996, token_types_count: 0 }, // [561]
    SimpleHintData { raw_values_start: 996, raw_values_count: 1, token_types_start: 997, token_types_count: 0 }, // [562]
    SimpleHintData { raw_values_start: 997, raw_values_count: 1, token_types_start: 998, token_types_count: 0 }, // [563]
    SimpleHintData { raw_values_start: 998, raw_values_count: 1, token_types_start: 999, token_types_count: 0 }, // [564]
    SimpleHintData { raw_values_start: 999, raw_values_count: 1, token_types_start: 1000, token_types_count: 0 }, // [565]
    SimpleHintData { raw_values_start: 1000, raw_values_count: 1, token_types_start: 1001, token_types_count: 0 }, // [566]
    SimpleHintData { raw_values_start: 1001, raw_values_count: 1, token_types_start: 1002, token_types_count: 0 }, // [567]
    SimpleHintData { raw_values_start: 1002, raw_values_count: 2, token_types_start: 1004, token_types_count: 0 }, // [568]
    SimpleHintData { raw_values_start: 1004, raw_values_count: 1, token_types_start: 1005, token_types_count: 0 }, // [569]
    SimpleHintData { raw_values_start: 1005, raw_values_count: 1, token_types_start: 1006, token_types_count: 0 }, // [570]
    SimpleHintData { raw_values_start: 1006, raw_values_count: 1, token_types_start: 1007, token_types_count: 0 }, // [571]
    SimpleHintData { raw_values_start: 1007, raw_values_count: 1, token_types_start: 1008, token_types_count: 0 }, // [572]
    SimpleHintData { raw_values_start: 1008, raw_values_count: 1, token_types_start: 1009, token_types_count: 0 }, // [573]
    SimpleHintData { raw_values_start: 1009, raw_values_count: 1, token_types_start: 1010, token_types_count: 0 }, // [574]
    SimpleHintData { raw_values_start: 1010, raw_values_count: 1, token_types_start: 1011, token_types_count: 0 }, // [575]
    SimpleHintData { raw_values_start: 1011, raw_values_count: 1, token_types_start: 1012, token_types_count: 0 }, // [576]
    SimpleHintData { raw_values_start: 1012, raw_values_count: 1, token_types_start: 1013, token_types_count: 0 }, // [577]
    SimpleHintData { raw_values_start: 1013, raw_values_count: 1, token_types_start: 1014, token_types_count: 0 }, // [578]
    SimpleHintData { raw_values_start: 1014, raw_values_count: 1, token_types_start: 1015, token_types_count: 0 }, // [579]
    SimpleHintData { raw_values_start: 1015, raw_values_count: 1, token_types_start: 1016, token_types_count: 0 }, // [580]
    SimpleHintData { raw_values_start: 1016, raw_values_count: 1, token_types_start: 1017, token_types_count: 0 }, // [581]
    SimpleHintData { raw_values_start: 1017, raw_values_count: 1, token_types_start: 1018, token_types_count: 0 }, // [582]
    SimpleHintData { raw_values_start: 1018, raw_values_count: 2, token_types_start: 1020, token_types_count: 0 }, // [583]
    SimpleHintData { raw_values_start: 1020, raw_values_count: 1, token_types_start: 1021, token_types_count: 0 }, // [584]
    SimpleHintData { raw_values_start: 1021, raw_values_count: 1, token_types_start: 1022, token_types_count: 0 }, // [585]
    SimpleHintData { raw_values_start: 1022, raw_values_count: 2, token_types_start: 1024, token_types_count: 0 }, // [586]
    SimpleHintData { raw_values_start: 1024, raw_values_count: 1, token_types_start: 1025, token_types_count: 0 }, // [587]
    SimpleHintData { raw_values_start: 1025, raw_values_count: 0, token_types_start: 1025, token_types_count: 1 }, // [588]
    SimpleHintData { raw_values_start: 1026, raw_values_count: 1, token_types_start: 1027, token_types_count: 0 }, // [589]
    SimpleHintData { raw_values_start: 1027, raw_values_count: 1, token_types_start: 1028, token_types_count: 0 }, // [590]
    SimpleHintData { raw_values_start: 1028, raw_values_count: 1, token_types_start: 1029, token_types_count: 0 }, // [591]
    SimpleHintData { raw_values_start: 1029, raw_values_count: 2, token_types_start: 1031, token_types_count: 0 }, // [592]
    SimpleHintData { raw_values_start: 1031, raw_values_count: 1, token_types_start: 1032, token_types_count: 0 }, // [593]
    SimpleHintData { raw_values_start: 1032, raw_values_count: 1, token_types_start: 1033, token_types_count: 0 }, // [594]
    SimpleHintData { raw_values_start: 1033, raw_values_count: 2, token_types_start: 1035, token_types_count: 0 }, // [595]
    SimpleHintData { raw_values_start: 1035, raw_values_count: 1, token_types_start: 1036, token_types_count: 0 }, // [596]
    SimpleHintData { raw_values_start: 1036, raw_values_count: 1, token_types_start: 1037, token_types_count: 0 }, // [597]
    SimpleHintData { raw_values_start: 1037, raw_values_count: 1, token_types_start: 1038, token_types_count: 0 }, // [598]
    SimpleHintData { raw_values_start: 1038, raw_values_count: 2, token_types_start: 1040, token_types_count: 0 }, // [599]
    SimpleHintData { raw_values_start: 1040, raw_values_count: 2, token_types_start: 1042, token_types_count: 0 }, // [600]
    SimpleHintData { raw_values_start: 1042, raw_values_count: 1, token_types_start: 1043, token_types_count: 0 }, // [601]
    SimpleHintData { raw_values_start: 1043, raw_values_count: 4, token_types_start: 1047, token_types_count: 0 }, // [602]
    SimpleHintData { raw_values_start: 1047, raw_values_count: 1, token_types_start: 1048, token_types_count: 0 }, // [603]
    SimpleHintData { raw_values_start: 1048, raw_values_count: 1, token_types_start: 1049, token_types_count: 0 }, // [604]
    SimpleHintData { raw_values_start: 1049, raw_values_count: 3, token_types_start: 1052, token_types_count: 0 }, // [605]
    SimpleHintData { raw_values_start: 1052, raw_values_count: 3, token_types_start: 1055, token_types_count: 2 }, // [606]
    SimpleHintData { raw_values_start: 1057, raw_values_count: 3, token_types_start: 1060, token_types_count: 0 }, // [607]
    SimpleHintData { raw_values_start: 1060, raw_values_count: 1, token_types_start: 1061, token_types_count: 0 }, // [608]
    SimpleHintData { raw_values_start: 1061, raw_values_count: 0, token_types_start: 1061, token_types_count: 2 }, // [609]
    SimpleHintData { raw_values_start: 1063, raw_values_count: 1, token_types_start: 1064, token_types_count: 0 }, // [610]
    SimpleHintData { raw_values_start: 1064, raw_values_count: 1, token_types_start: 1065, token_types_count: 0 }, // [611]
    SimpleHintData { raw_values_start: 1065, raw_values_count: 1, token_types_start: 1066, token_types_count: 0 }, // [612]
    SimpleHintData { raw_values_start: 1066, raw_values_count: 1, token_types_start: 1067, token_types_count: 0 }, // [613]
    SimpleHintData { raw_values_start: 1067, raw_values_count: 1, token_types_start: 1068, token_types_count: 0 }, // [614]
    SimpleHintData { raw_values_start: 1068, raw_values_count: 1, token_types_start: 1069, token_types_count: 0 }, // [615]
    SimpleHintData { raw_values_start: 1069, raw_values_count: 1, token_types_start: 1070, token_types_count: 0 }, // [616]
    SimpleHintData { raw_values_start: 1070, raw_values_count: 1, token_types_start: 1071, token_types_count: 0 }, // [617]
    SimpleHintData { raw_values_start: 1071, raw_values_count: 1, token_types_start: 1072, token_types_count: 0 }, // [618]
    SimpleHintData { raw_values_start: 1072, raw_values_count: 1, token_types_start: 1073, token_types_count: 0 }, // [619]
    SimpleHintData { raw_values_start: 1073, raw_values_count: 1, token_types_start: 1074, token_types_count: 0 }, // [620]
    SimpleHintData { raw_values_start: 1074, raw_values_count: 1, token_types_start: 1075, token_types_count: 0 }, // [621]
    SimpleHintData { raw_values_start: 1075, raw_values_count: 1, token_types_start: 1076, token_types_count: 0 }, // [622]
    SimpleHintData { raw_values_start: 1076, raw_values_count: 1, token_types_start: 1077, token_types_count: 0 }, // [623]
    SimpleHintData { raw_values_start: 1077, raw_values_count: 1, token_types_start: 1078, token_types_count: 0 }, // [624]
    SimpleHintData { raw_values_start: 1078, raw_values_count: 1, token_types_start: 1079, token_types_count: 0 }, // [625]
    SimpleHintData { raw_values_start: 1079, raw_values_count: 2, token_types_start: 1081, token_types_count: 0 }, // [626]
    SimpleHintData { raw_values_start: 1081, raw_values_count: 1, token_types_start: 1082, token_types_count: 0 }, // [627]
    SimpleHintData { raw_values_start: 1082, raw_values_count: 1, token_types_start: 1083, token_types_count: 0 }, // [628]
    SimpleHintData { raw_values_start: 1083, raw_values_count: 1, token_types_start: 1084, token_types_count: 0 }, // [629]
    SimpleHintData { raw_values_start: 1084, raw_values_count: 1, token_types_start: 1085, token_types_count: 0 }, // [630]
    SimpleHintData { raw_values_start: 1085, raw_values_count: 1, token_types_start: 1086, token_types_count: 0 }, // [631]
    SimpleHintData { raw_values_start: 1086, raw_values_count: 1, token_types_start: 1087, token_types_count: 0 }, // [632]
    SimpleHintData { raw_values_start: 1087, raw_values_count: 3, token_types_start: 1090, token_types_count: 0 }, // [633]
    SimpleHintData { raw_values_start: 1090, raw_values_count: 1, token_types_start: 1091, token_types_count: 0 }, // [634]
    SimpleHintData { raw_values_start: 1091, raw_values_count: 1, token_types_start: 1092, token_types_count: 0 }, // [635]
    SimpleHintData { raw_values_start: 1092, raw_values_count: 1, token_types_start: 1093, token_types_count: 0 }, // [636]
    SimpleHintData { raw_values_start: 1093, raw_values_count: 1, token_types_start: 1094, token_types_count: 0 }, // [637]
    SimpleHintData { raw_values_start: 1094, raw_values_count: 1, token_types_start: 1095, token_types_count: 0 }, // [638]
    SimpleHintData { raw_values_start: 1095, raw_values_count: 1, token_types_start: 1096, token_types_count: 0 }, // [639]
    SimpleHintData { raw_values_start: 1096, raw_values_count: 1, token_types_start: 1097, token_types_count: 0 }, // [640]
    SimpleHintData { raw_values_start: 1097, raw_values_count: 1, token_types_start: 1098, token_types_count: 0 }, // [641]
    SimpleHintData { raw_values_start: 1098, raw_values_count: 0, token_types_start: 1098, token_types_count: 3 }, // [642]
    SimpleHintData { raw_values_start: 1101, raw_values_count: 1, token_types_start: 1102, token_types_count: 0 }, // [643]
    SimpleHintData { raw_values_start: 1102, raw_values_count: 1, token_types_start: 1103, token_types_count: 0 }, // [644]
    SimpleHintData { raw_values_start: 1103, raw_values_count: 1, token_types_start: 1104, token_types_count: 0 }, // [645]
    SimpleHintData { raw_values_start: 1104, raw_values_count: 1, token_types_start: 1105, token_types_count: 0 }, // [646]
    SimpleHintData { raw_values_start: 1105, raw_values_count: 1, token_types_start: 1106, token_types_count: 0 }, // [647]
    SimpleHintData { raw_values_start: 1106, raw_values_count: 1, token_types_start: 1107, token_types_count: 0 }, // [648]
    SimpleHintData { raw_values_start: 1107, raw_values_count: 1, token_types_start: 1108, token_types_count: 0 }, // [649]
    SimpleHintData { raw_values_start: 1108, raw_values_count: 1, token_types_start: 1109, token_types_count: 0 }, // [650]
    SimpleHintData { raw_values_start: 1109, raw_values_count: 1, token_types_start: 1110, token_types_count: 0 }, // [651]
    SimpleHintData { raw_values_start: 1110, raw_values_count: 1, token_types_start: 1111, token_types_count: 0 }, // [652]
    SimpleHintData { raw_values_start: 1111, raw_values_count: 1, token_types_start: 1112, token_types_count: 0 }, // [653]
    SimpleHintData { raw_values_start: 1112, raw_values_count: 1, token_types_start: 1113, token_types_count: 0 }, // [654]
    SimpleHintData { raw_values_start: 1113, raw_values_count: 1, token_types_start: 1114, token_types_count: 0 }, // [655]
    SimpleHintData { raw_values_start: 1114, raw_values_count: 1, token_types_start: 1115, token_types_count: 0 }, // [656]
    SimpleHintData { raw_values_start: 1115, raw_values_count: 1, token_types_start: 1116, token_types_count: 0 }, // [657]
    SimpleHintData { raw_values_start: 1116, raw_values_count: 2, token_types_start: 1118, token_types_count: 0 }, // [658]
    SimpleHintData { raw_values_start: 1118, raw_values_count: 1, token_types_start: 1119, token_types_count: 0 }, // [659]
    SimpleHintData { raw_values_start: 1119, raw_values_count: 1, token_types_start: 1120, token_types_count: 0 }, // [660]
    SimpleHintData { raw_values_start: 1120, raw_values_count: 1, token_types_start: 1121, token_types_count: 0 }, // [661]
    SimpleHintData { raw_values_start: 1121, raw_values_count: 1, token_types_start: 1122, token_types_count: 0 }, // [662]
    SimpleHintData { raw_values_start: 1122, raw_values_count: 1, token_types_start: 1123, token_types_count: 0 }, // [663]
    SimpleHintData { raw_values_start: 1123, raw_values_count: 1, token_types_start: 1124, token_types_count: 0 }, // [664]
    SimpleHintData { raw_values_start: 1124, raw_values_count: 1, token_types_start: 1125, token_types_count: 0 }, // [665]
    SimpleHintData { raw_values_start: 1125, raw_values_count: 1, token_types_start: 1126, token_types_count: 0 }, // [666]
    SimpleHintData { raw_values_start: 1126, raw_values_count: 1, token_types_start: 1127, token_types_count: 0 }, // [667]
    SimpleHintData { raw_values_start: 1127, raw_values_count: 1, token_types_start: 1128, token_types_count: 0 }, // [668]
    SimpleHintData { raw_values_start: 1128, raw_values_count: 1, token_types_start: 1129, token_types_count: 0 }, // [669]
    SimpleHintData { raw_values_start: 1129, raw_values_count: 1, token_types_start: 1130, token_types_count: 0 }, // [670]
    SimpleHintData { raw_values_start: 1130, raw_values_count: 1, token_types_start: 1131, token_types_count: 0 }, // [671]
    SimpleHintData { raw_values_start: 1131, raw_values_count: 1, token_types_start: 1132, token_types_count: 0 }, // [672]
    SimpleHintData { raw_values_start: 1132, raw_values_count: 1, token_types_start: 1133, token_types_count: 0 }, // [673]
    SimpleHintData { raw_values_start: 1133, raw_values_count: 1, token_types_start: 1134, token_types_count: 0 }, // [674]
    SimpleHintData { raw_values_start: 1134, raw_values_count: 1, token_types_start: 1135, token_types_count: 0 }, // [675]
    SimpleHintData { raw_values_start: 1135, raw_values_count: 1, token_types_start: 1136, token_types_count: 0 }, // [676]
    SimpleHintData { raw_values_start: 1136, raw_values_count: 1, token_types_start: 1137, token_types_count: 0 }, // [677]
    SimpleHintData { raw_values_start: 1137, raw_values_count: 1, token_types_start: 1138, token_types_count: 0 }, // [678]
    SimpleHintData { raw_values_start: 1138, raw_values_count: 1, token_types_start: 1139, token_types_count: 0 }, // [679]
    SimpleHintData { raw_values_start: 1139, raw_values_count: 1, token_types_start: 1140, token_types_count: 0 }, // [680]
    SimpleHintData { raw_values_start: 1140, raw_values_count: 1, token_types_start: 1141, token_types_count: 0 }, // [681]
    SimpleHintData { raw_values_start: 1141, raw_values_count: 1, token_types_start: 1142, token_types_count: 0 }, // [682]
    SimpleHintData { raw_values_start: 1142, raw_values_count: 1, token_types_start: 1143, token_types_count: 0 }, // [683]
    SimpleHintData { raw_values_start: 1143, raw_values_count: 1, token_types_start: 1144, token_types_count: 0 }, // [684]
    SimpleHintData { raw_values_start: 1144, raw_values_count: 1, token_types_start: 1145, token_types_count: 0 }, // [685]
    SimpleHintData { raw_values_start: 1145, raw_values_count: 1, token_types_start: 1146, token_types_count: 0 }, // [686]
    SimpleHintData { raw_values_start: 1146, raw_values_count: 1, token_types_start: 1147, token_types_count: 0 }, // [687]
    SimpleHintData { raw_values_start: 1147, raw_values_count: 1, token_types_start: 1148, token_types_count: 0 }, // [688]
    SimpleHintData { raw_values_start: 1148, raw_values_count: 2, token_types_start: 1150, token_types_count: 0 }, // [689]
    SimpleHintData { raw_values_start: 1150, raw_values_count: 1, token_types_start: 1151, token_types_count: 0 }, // [690]
    SimpleHintData { raw_values_start: 1151, raw_values_count: 1, token_types_start: 1152, token_types_count: 1 }, // [691]
    SimpleHintData { raw_values_start: 1153, raw_values_count: 1, token_types_start: 1154, token_types_count: 0 }, // [692]
    SimpleHintData { raw_values_start: 1154, raw_values_count: 2, token_types_start: 1156, token_types_count: 0 }, // [693]
    SimpleHintData { raw_values_start: 1156, raw_values_count: 1, token_types_start: 1157, token_types_count: 0 }, // [694]
    SimpleHintData { raw_values_start: 1157, raw_values_count: 3, token_types_start: 1160, token_types_count: 0 }, // [695]
    SimpleHintData { raw_values_start: 1160, raw_values_count: 2, token_types_start: 1162, token_types_count: 0 }, // [696]
    SimpleHintData { raw_values_start: 1162, raw_values_count: 1, token_types_start: 1163, token_types_count: 0 }, // [697]
    SimpleHintData { raw_values_start: 1163, raw_values_count: 1, token_types_start: 1164, token_types_count: 0 }, // [698]
    SimpleHintData { raw_values_start: 1164, raw_values_count: 1, token_types_start: 1165, token_types_count: 0 }, // [699]
    SimpleHintData { raw_values_start: 1165, raw_values_count: 1, token_types_start: 1166, token_types_count: 0 }, // [700]
    SimpleHintData { raw_values_start: 1166, raw_values_count: 1, token_types_start: 1167, token_types_count: 0 }, // [701]
    SimpleHintData { raw_values_start: 1167, raw_values_count: 1, token_types_start: 1168, token_types_count: 0 }, // [702]
    SimpleHintData { raw_values_start: 1168, raw_values_count: 1, token_types_start: 1169, token_types_count: 0 }, // [703]
    SimpleHintData { raw_values_start: 1169, raw_values_count: 1, token_types_start: 1170, token_types_count: 0 }, // [704]
    SimpleHintData { raw_values_start: 1170, raw_values_count: 1, token_types_start: 1171, token_types_count: 0 }, // [705]
    SimpleHintData { raw_values_start: 1171, raw_values_count: 1, token_types_start: 1172, token_types_count: 0 }, // [706]
    SimpleHintData { raw_values_start: 1172, raw_values_count: 1, token_types_start: 1173, token_types_count: 0 }, // [707]
    SimpleHintData { raw_values_start: 1173, raw_values_count: 1, token_types_start: 1174, token_types_count: 0 }, // [708]
    SimpleHintData { raw_values_start: 1174, raw_values_count: 1, token_types_start: 1175, token_types_count: 0 }, // [709]
    SimpleHintData { raw_values_start: 1175, raw_values_count: 1, token_types_start: 1176, token_types_count: 0 }, // [710]
    SimpleHintData { raw_values_start: 1176, raw_values_count: 1, token_types_start: 1177, token_types_count: 0 }, // [711]
    SimpleHintData { raw_values_start: 1177, raw_values_count: 1, token_types_start: 1178, token_types_count: 0 }, // [712]
    SimpleHintData { raw_values_start: 1178, raw_values_count: 1, token_types_start: 1179, token_types_count: 0 }, // [713]
    SimpleHintData { raw_values_start: 1179, raw_values_count: 1, token_types_start: 1180, token_types_count: 0 }, // [714]
    SimpleHintData { raw_values_start: 1180, raw_values_count: 1, token_types_start: 1181, token_types_count: 0 }, // [715]
    SimpleHintData { raw_values_start: 1181, raw_values_count: 1, token_types_start: 1182, token_types_count: 0 }, // [716]
    SimpleHintData { raw_values_start: 1182, raw_values_count: 1, token_types_start: 1183, token_types_count: 0 }, // [717]
    SimpleHintData { raw_values_start: 1183, raw_values_count: 1, token_types_start: 1184, token_types_count: 0 }, // [718]
    SimpleHintData { raw_values_start: 1184, raw_values_count: 1, token_types_start: 1185, token_types_count: 0 }, // [719]
    SimpleHintData { raw_values_start: 1185, raw_values_count: 1, token_types_start: 1186, token_types_count: 0 }, // [720]
    SimpleHintData { raw_values_start: 1186, raw_values_count: 1, token_types_start: 1187, token_types_count: 0 }, // [721]
    SimpleHintData { raw_values_start: 1187, raw_values_count: 1, token_types_start: 1188, token_types_count: 0 }, // [722]
    SimpleHintData { raw_values_start: 1188, raw_values_count: 1, token_types_start: 1189, token_types_count: 0 }, // [723]
    SimpleHintData { raw_values_start: 1189, raw_values_count: 1, token_types_start: 1190, token_types_count: 0 }, // [724]
    SimpleHintData { raw_values_start: 1190, raw_values_count: 1, token_types_start: 1191, token_types_count: 0 }, // [725]
    SimpleHintData { raw_values_start: 1191, raw_values_count: 1, token_types_start: 1192, token_types_count: 0 }, // [726]
    SimpleHintData { raw_values_start: 1192, raw_values_count: 1, token_types_start: 1193, token_types_count: 0 }, // [727]
    SimpleHintData { raw_values_start: 1193, raw_values_count: 1, token_types_start: 1194, token_types_count: 0 }, // [728]
    SimpleHintData { raw_values_start: 1194, raw_values_count: 1, token_types_start: 1195, token_types_count: 0 }, // [729]
    SimpleHintData { raw_values_start: 1195, raw_values_count: 1, token_types_start: 1196, token_types_count: 0 }, // [730]
    SimpleHintData { raw_values_start: 1196, raw_values_count: 1, token_types_start: 1197, token_types_count: 0 }, // [731]
    SimpleHintData { raw_values_start: 1197, raw_values_count: 1, token_types_start: 1198, token_types_count: 0 }, // [732]
    SimpleHintData { raw_values_start: 1198, raw_values_count: 1, token_types_start: 1199, token_types_count: 0 }, // [733]
    SimpleHintData { raw_values_start: 1199, raw_values_count: 1, token_types_start: 1200, token_types_count: 0 }, // [734]
    SimpleHintData { raw_values_start: 1200, raw_values_count: 1, token_types_start: 1201, token_types_count: 0 }, // [735]
    SimpleHintData { raw_values_start: 1201, raw_values_count: 1, token_types_start: 1202, token_types_count: 0 }, // [736]
    SimpleHintData { raw_values_start: 1202, raw_values_count: 1, token_types_start: 1203, token_types_count: 0 }, // [737]
    SimpleHintData { raw_values_start: 1203, raw_values_count: 1, token_types_start: 1204, token_types_count: 0 }, // [738]
    SimpleHintData { raw_values_start: 1204, raw_values_count: 8, token_types_start: 1212, token_types_count: 0 }, // [739]
    SimpleHintData { raw_values_start: 1212, raw_values_count: 1, token_types_start: 1213, token_types_count: 0 }, // [740]
    SimpleHintData { raw_values_start: 1213, raw_values_count: 4, token_types_start: 1217, token_types_count: 0 }, // [741]
    SimpleHintData { raw_values_start: 1217, raw_values_count: 2, token_types_start: 1219, token_types_count: 0 }, // [742]
    SimpleHintData { raw_values_start: 1219, raw_values_count: 2, token_types_start: 1221, token_types_count: 0 }, // [743]
    SimpleHintData { raw_values_start: 1221, raw_values_count: 6, token_types_start: 1227, token_types_count: 0 }, // [744]
    SimpleHintData { raw_values_start: 1227, raw_values_count: 2, token_types_start: 1229, token_types_count: 0 }, // [745]
    SimpleHintData { raw_values_start: 1229, raw_values_count: 2, token_types_start: 1231, token_types_count: 0 }, // [746]
    SimpleHintData { raw_values_start: 1231, raw_values_count: 1, token_types_start: 1232, token_types_count: 0 }, // [747]
    SimpleHintData { raw_values_start: 1232, raw_values_count: 15, token_types_start: 1247, token_types_count: 1 }, // [748]
    SimpleHintData { raw_values_start: 1248, raw_values_count: 3, token_types_start: 1251, token_types_count: 0 }, // [749]
    SimpleHintData { raw_values_start: 1251, raw_values_count: 1, token_types_start: 1252, token_types_count: 0 }, // [750]
    SimpleHintData { raw_values_start: 1252, raw_values_count: 6, token_types_start: 1258, token_types_count: 3 }, // [751]
    SimpleHintData { raw_values_start: 1261, raw_values_count: 2, token_types_start: 1263, token_types_count: 0 }, // [752]
    SimpleHintData { raw_values_start: 1263, raw_values_count: 1, token_types_start: 1264, token_types_count: 0 }, // [753]
    SimpleHintData { raw_values_start: 1264, raw_values_count: 1, token_types_start: 1265, token_types_count: 0 }, // [754]
    SimpleHintData { raw_values_start: 1265, raw_values_count: 1, token_types_start: 1266, token_types_count: 0 }, // [755]
    SimpleHintData { raw_values_start: 1266, raw_values_count: 1, token_types_start: 1267, token_types_count: 0 }, // [756]
    SimpleHintData { raw_values_start: 1267, raw_values_count: 1, token_types_start: 1268, token_types_count: 0 }, // [757]
    SimpleHintData { raw_values_start: 1268, raw_values_count: 1, token_types_start: 1269, token_types_count: 0 }, // [758]
    SimpleHintData { raw_values_start: 1269, raw_values_count: 1, token_types_start: 1270, token_types_count: 0 }, // [759]
    SimpleHintData { raw_values_start: 1270, raw_values_count: 2, token_types_start: 1272, token_types_count: 0 }, // [760]
    SimpleHintData { raw_values_start: 1272, raw_values_count: 2, token_types_start: 1274, token_types_count: 0 }, // [761]
    SimpleHintData { raw_values_start: 1274, raw_values_count: 1, token_types_start: 1275, token_types_count: 0 }, // [762]
    SimpleHintData { raw_values_start: 1275, raw_values_count: 2, token_types_start: 1277, token_types_count: 0 }, // [763]
    SimpleHintData { raw_values_start: 1277, raw_values_count: 1, token_types_start: 1278, token_types_count: 0 }, // [764]
    SimpleHintData { raw_values_start: 1278, raw_values_count: 1, token_types_start: 1279, token_types_count: 0 }, // [765]
    SimpleHintData { raw_values_start: 1279, raw_values_count: 1, token_types_start: 1280, token_types_count: 0 }, // [766]
    SimpleHintData { raw_values_start: 1280, raw_values_count: 1, token_types_start: 1281, token_types_count: 0 }, // [767]
    SimpleHintData { raw_values_start: 1281, raw_values_count: 1, token_types_start: 1282, token_types_count: 0 }, // [768]
    SimpleHintData { raw_values_start: 1282, raw_values_count: 1, token_types_start: 1283, token_types_count: 0 }, // [769]
    SimpleHintData { raw_values_start: 1283, raw_values_count: 1, token_types_start: 1284, token_types_count: 0 }, // [770]
    SimpleHintData { raw_values_start: 1284, raw_values_count: 1, token_types_start: 1285, token_types_count: 0 }, // [771]
    SimpleHintData { raw_values_start: 1285, raw_values_count: 1, token_types_start: 1286, token_types_count: 0 }, // [772]
    SimpleHintData { raw_values_start: 1286, raw_values_count: 1, token_types_start: 1287, token_types_count: 0 }, // [773]
    SimpleHintData { raw_values_start: 1287, raw_values_count: 1, token_types_start: 1288, token_types_count: 0 }, // [774]
    SimpleHintData { raw_values_start: 1288, raw_values_count: 1, token_types_start: 1289, token_types_count: 0 }, // [775]
    SimpleHintData { raw_values_start: 1289, raw_values_count: 1, token_types_start: 1290, token_types_count: 0 }, // [776]
    SimpleHintData { raw_values_start: 1290, raw_values_count: 1, token_types_start: 1291, token_types_count: 0 }, // [777]
    SimpleHintData { raw_values_start: 1291, raw_values_count: 1, token_types_start: 1292, token_types_count: 0 }, // [778]
    SimpleHintData { raw_values_start: 1292, raw_values_count: 1, token_types_start: 1293, token_types_count: 0 }, // [779]
    SimpleHintData { raw_values_start: 1293, raw_values_count: 1, token_types_start: 1294, token_types_count: 0 }, // [780]
    SimpleHintData { raw_values_start: 1294, raw_values_count: 1, token_types_start: 1295, token_types_count: 0 }, // [781]
    SimpleHintData { raw_values_start: 1295, raw_values_count: 1, token_types_start: 1296, token_types_count: 0 }, // [782]
    SimpleHintData { raw_values_start: 1296, raw_values_count: 1, token_types_start: 1297, token_types_count: 0 }, // [783]
    SimpleHintData { raw_values_start: 1297, raw_values_count: 1, token_types_start: 1298, token_types_count: 0 }, // [784]
    SimpleHintData { raw_values_start: 1298, raw_values_count: 1, token_types_start: 1299, token_types_count: 0 }, // [785]
    SimpleHintData { raw_values_start: 1299, raw_values_count: 1, token_types_start: 1300, token_types_count: 0 }, // [786]
    SimpleHintData { raw_values_start: 1300, raw_values_count: 2, token_types_start: 1302, token_types_count: 0 }, // [787]
    SimpleHintData { raw_values_start: 1302, raw_values_count: 2, token_types_start: 1304, token_types_count: 0 }, // [788]
    SimpleHintData { raw_values_start: 1304, raw_values_count: 1, token_types_start: 1305, token_types_count: 0 }, // [789]
    SimpleHintData { raw_values_start: 1305, raw_values_count: 2, token_types_start: 1307, token_types_count: 0 }, // [790]
    SimpleHintData { raw_values_start: 1307, raw_values_count: 1, token_types_start: 1308, token_types_count: 0 }, // [791]
    SimpleHintData { raw_values_start: 1308, raw_values_count: 1, token_types_start: 1309, token_types_count: 0 }, // [792]
    SimpleHintData { raw_values_start: 1309, raw_values_count: 1, token_types_start: 1310, token_types_count: 0 }, // [793]
    SimpleHintData { raw_values_start: 1310, raw_values_count: 1, token_types_start: 1311, token_types_count: 0 }, // [794]
    SimpleHintData { raw_values_start: 1311, raw_values_count: 1, token_types_start: 1312, token_types_count: 0 }, // [795]
    SimpleHintData { raw_values_start: 1312, raw_values_count: 1, token_types_start: 1313, token_types_count: 0 }, // [796]
    SimpleHintData { raw_values_start: 1313, raw_values_count: 2, token_types_start: 1315, token_types_count: 0 }, // [797]
    SimpleHintData { raw_values_start: 1315, raw_values_count: 2, token_types_start: 1317, token_types_count: 0 }, // [798]
    SimpleHintData { raw_values_start: 1317, raw_values_count: 1, token_types_start: 1318, token_types_count: 0 }, // [799]
    SimpleHintData { raw_values_start: 1318, raw_values_count: 1, token_types_start: 1319, token_types_count: 0 }, // [800]
    SimpleHintData { raw_values_start: 1319, raw_values_count: 1, token_types_start: 1320, token_types_count: 0 }, // [801]
    SimpleHintData { raw_values_start: 1320, raw_values_count: 1, token_types_start: 1321, token_types_count: 0 }, // [802]
    SimpleHintData { raw_values_start: 1321, raw_values_count: 1, token_types_start: 1322, token_types_count: 0 }, // [803]
    SimpleHintData { raw_values_start: 1322, raw_values_count: 1, token_types_start: 1323, token_types_count: 0 }, // [804]
    SimpleHintData { raw_values_start: 1323, raw_values_count: 17, token_types_start: 1340, token_types_count: 0 }, // [805]
    SimpleHintData { raw_values_start: 1340, raw_values_count: 1, token_types_start: 1341, token_types_count: 0 }, // [806]
    SimpleHintData { raw_values_start: 1341, raw_values_count: 1, token_types_start: 1342, token_types_count: 0 }, // [807]
    SimpleHintData { raw_values_start: 1342, raw_values_count: 1, token_types_start: 1343, token_types_count: 0 }, // [808]
    SimpleHintData { raw_values_start: 1343, raw_values_count: 4, token_types_start: 1347, token_types_count: 0 }, // [809]
    SimpleHintData { raw_values_start: 1347, raw_values_count: 1, token_types_start: 1348, token_types_count: 0 }, // [810]
    SimpleHintData { raw_values_start: 1348, raw_values_count: 1, token_types_start: 1349, token_types_count: 0 }, // [811]
    SimpleHintData { raw_values_start: 1349, raw_values_count: 4, token_types_start: 1353, token_types_count: 0 }, // [812]
    SimpleHintData { raw_values_start: 1353, raw_values_count: 1, token_types_start: 1354, token_types_count: 0 }, // [813]
    SimpleHintData { raw_values_start: 1354, raw_values_count: 1, token_types_start: 1355, token_types_count: 0 }, // [814]
    SimpleHintData { raw_values_start: 1355, raw_values_count: 1, token_types_start: 1356, token_types_count: 0 }, // [815]
    SimpleHintData { raw_values_start: 1356, raw_values_count: 1, token_types_start: 1357, token_types_count: 0 }, // [816]
    SimpleHintData { raw_values_start: 1357, raw_values_count: 3, token_types_start: 1360, token_types_count: 0 }, // [817]
    SimpleHintData { raw_values_start: 1360, raw_values_count: 1, token_types_start: 1361, token_types_count: 0 }, // [818]
    SimpleHintData { raw_values_start: 1361, raw_values_count: 2, token_types_start: 1363, token_types_count: 0 }, // [819]
    SimpleHintData { raw_values_start: 1363, raw_values_count: 0, token_types_start: 1363, token_types_count: 1 }, // [820]
    SimpleHintData { raw_values_start: 1364, raw_values_count: 0, token_types_start: 1364, token_types_count: 2 }, // [821]
    SimpleHintData { raw_values_start: 1366, raw_values_count: 1, token_types_start: 1367, token_types_count: 0 }, // [822]
    SimpleHintData { raw_values_start: 1367, raw_values_count: 1, token_types_start: 1368, token_types_count: 0 }, // [823]
    SimpleHintData { raw_values_start: 1368, raw_values_count: 1, token_types_start: 1369, token_types_count: 0 }, // [824]
    SimpleHintData { raw_values_start: 1369, raw_values_count: 1, token_types_start: 1370, token_types_count: 0 }, // [825]
    SimpleHintData { raw_values_start: 1370, raw_values_count: 1, token_types_start: 1371, token_types_count: 0 }, // [826]
    SimpleHintData { raw_values_start: 1371, raw_values_count: 1, token_types_start: 1372, token_types_count: 0 }, // [827]
    SimpleHintData { raw_values_start: 1372, raw_values_count: 1, token_types_start: 1373, token_types_count: 0 }, // [828]
    SimpleHintData { raw_values_start: 1373, raw_values_count: 1, token_types_start: 1374, token_types_count: 0 }, // [829]
    SimpleHintData { raw_values_start: 1374, raw_values_count: 1, token_types_start: 1375, token_types_count: 0 }, // [830]
    SimpleHintData { raw_values_start: 1375, raw_values_count: 1, token_types_start: 1376, token_types_count: 0 }, // [831]
    SimpleHintData { raw_values_start: 1376, raw_values_count: 2, token_types_start: 1378, token_types_count: 0 }, // [832]
    SimpleHintData { raw_values_start: 1378, raw_values_count: 1, token_types_start: 1379, token_types_count: 0 }, // [833]
    SimpleHintData { raw_values_start: 1379, raw_values_count: 1, token_types_start: 1380, token_types_count: 0 }, // [834]
    SimpleHintData { raw_values_start: 1380, raw_values_count: 13, token_types_start: 1393, token_types_count: 0 }, // [835]
    SimpleHintData { raw_values_start: 1393, raw_values_count: 1, token_types_start: 1394, token_types_count: 0 }, // [836]
    SimpleHintData { raw_values_start: 1394, raw_values_count: 1, token_types_start: 1395, token_types_count: 0 }, // [837]
    SimpleHintData { raw_values_start: 1395, raw_values_count: 1, token_types_start: 1396, token_types_count: 0 }, // [838]
    SimpleHintData { raw_values_start: 1396, raw_values_count: 1, token_types_start: 1397, token_types_count: 0 }, // [839]
    SimpleHintData { raw_values_start: 1397, raw_values_count: 1, token_types_start: 1398, token_types_count: 0 }, // [840]
    SimpleHintData { raw_values_start: 1398, raw_values_count: 1, token_types_start: 1399, token_types_count: 0 }, // [841]
    SimpleHintData { raw_values_start: 1399, raw_values_count: 1, token_types_start: 1400, token_types_count: 0 }, // [842]
    SimpleHintData { raw_values_start: 1400, raw_values_count: 1, token_types_start: 1401, token_types_count: 0 }, // [843]
    SimpleHintData { raw_values_start: 1401, raw_values_count: 1, token_types_start: 1402, token_types_count: 0 }, // [844]
    SimpleHintData { raw_values_start: 1402, raw_values_count: 0, token_types_start: 1402, token_types_count: 1 }, // [845]
    SimpleHintData { raw_values_start: 1403, raw_values_count: 1, token_types_start: 1404, token_types_count: 0 }, // [846]
    SimpleHintData { raw_values_start: 1404, raw_values_count: 1, token_types_start: 1405, token_types_count: 0 }, // [847]
    SimpleHintData { raw_values_start: 1405, raw_values_count: 1, token_types_start: 1406, token_types_count: 0 }, // [848]
    SimpleHintData { raw_values_start: 1406, raw_values_count: 1, token_types_start: 1407, token_types_count: 0 }, // [849]
    SimpleHintData { raw_values_start: 1407, raw_values_count: 1, token_types_start: 1408, token_types_count: 0 }, // [850]
    SimpleHintData { raw_values_start: 1408, raw_values_count: 1, token_types_start: 1409, token_types_count: 0 }, // [851]
    SimpleHintData { raw_values_start: 1409, raw_values_count: 1, token_types_start: 1410, token_types_count: 0 }, // [852]
    SimpleHintData { raw_values_start: 1410, raw_values_count: 1, token_types_start: 1411, token_types_count: 0 }, // [853]
    SimpleHintData { raw_values_start: 1411, raw_values_count: 6, token_types_start: 1417, token_types_count: 0 }, // [854]
    SimpleHintData { raw_values_start: 1417, raw_values_count: 1, token_types_start: 1418, token_types_count: 0 }, // [855]
    SimpleHintData { raw_values_start: 1418, raw_values_count: 1, token_types_start: 1419, token_types_count: 0 }, // [856]
    SimpleHintData { raw_values_start: 1419, raw_values_count: 1, token_types_start: 1420, token_types_count: 0 }, // [857]
    SimpleHintData { raw_values_start: 1420, raw_values_count: 1, token_types_start: 1421, token_types_count: 0 }, // [858]
    SimpleHintData { raw_values_start: 1421, raw_values_count: 1, token_types_start: 1422, token_types_count: 0 }, // [859]
    SimpleHintData { raw_values_start: 1422, raw_values_count: 5, token_types_start: 1427, token_types_count: 0 }, // [860]
    SimpleHintData { raw_values_start: 1427, raw_values_count: 1, token_types_start: 1428, token_types_count: 0 }, // [861]
    SimpleHintData { raw_values_start: 1428, raw_values_count: 1, token_types_start: 1429, token_types_count: 0 }, // [862]
    SimpleHintData { raw_values_start: 1429, raw_values_count: 1, token_types_start: 1430, token_types_count: 0 }, // [863]
    SimpleHintData { raw_values_start: 1430, raw_values_count: 1, token_types_start: 1431, token_types_count: 0 }, // [864]
    SimpleHintData { raw_values_start: 1431, raw_values_count: 1, token_types_start: 1432, token_types_count: 0 }, // [865]
    SimpleHintData { raw_values_start: 1432, raw_values_count: 1, token_types_start: 1433, token_types_count: 0 }, // [866]
    SimpleHintData { raw_values_start: 1433, raw_values_count: 1, token_types_start: 1434, token_types_count: 0 }, // [867]
    SimpleHintData { raw_values_start: 1434, raw_values_count: 1, token_types_start: 1435, token_types_count: 0 }, // [868]
    SimpleHintData { raw_values_start: 1435, raw_values_count: 1, token_types_start: 1436, token_types_count: 0 }, // [869]
    SimpleHintData { raw_values_start: 1436, raw_values_count: 1, token_types_start: 1437, token_types_count: 0 }, // [870]
    SimpleHintData { raw_values_start: 1437, raw_values_count: 1, token_types_start: 1438, token_types_count: 0 }, // [871]
    SimpleHintData { raw_values_start: 1438, raw_values_count: 1, token_types_start: 1439, token_types_count: 0 }, // [872]
    SimpleHintData { raw_values_start: 1439, raw_values_count: 1, token_types_start: 1440, token_types_count: 0 }, // [873]
    SimpleHintData { raw_values_start: 1440, raw_values_count: 1, token_types_start: 1441, token_types_count: 0 }, // [874]
    SimpleHintData { raw_values_start: 1441, raw_values_count: 1, token_types_start: 1442, token_types_count: 0 }, // [875]
    SimpleHintData { raw_values_start: 1442, raw_values_count: 1, token_types_start: 1443, token_types_count: 0 }, // [876]
    SimpleHintData { raw_values_start: 1443, raw_values_count: 1, token_types_start: 1444, token_types_count: 0 }, // [877]
    SimpleHintData { raw_values_start: 1444, raw_values_count: 1, token_types_start: 1445, token_types_count: 0 }, // [878]
    SimpleHintData { raw_values_start: 1445, raw_values_count: 1, token_types_start: 1446, token_types_count: 0 }, // [879]
    SimpleHintData { raw_values_start: 1446, raw_values_count: 1, token_types_start: 1447, token_types_count: 0 }, // [880]
    SimpleHintData { raw_values_start: 1447, raw_values_count: 1, token_types_start: 1448, token_types_count: 0 }, // [881]
    SimpleHintData { raw_values_start: 1448, raw_values_count: 1, token_types_start: 1449, token_types_count: 0 }, // [882]
    SimpleHintData { raw_values_start: 1449, raw_values_count: 1, token_types_start: 1450, token_types_count: 0 }, // [883]
    SimpleHintData { raw_values_start: 1450, raw_values_count: 4, token_types_start: 1454, token_types_count: 0 }, // [884]
    SimpleHintData { raw_values_start: 1454, raw_values_count: 1, token_types_start: 1455, token_types_count: 0 }, // [885]
    SimpleHintData { raw_values_start: 1455, raw_values_count: 1, token_types_start: 1456, token_types_count: 0 }, // [886]
    SimpleHintData { raw_values_start: 1456, raw_values_count: 1, token_types_start: 1457, token_types_count: 0 }, // [887]
    SimpleHintData { raw_values_start: 1457, raw_values_count: 1, token_types_start: 1458, token_types_count: 0 }, // [888]
    SimpleHintData { raw_values_start: 1458, raw_values_count: 1, token_types_start: 1459, token_types_count: 0 }, // [889]
    SimpleHintData { raw_values_start: 1459, raw_values_count: 1, token_types_start: 1460, token_types_count: 0 }, // [890]
    SimpleHintData { raw_values_start: 1460, raw_values_count: 1, token_types_start: 1461, token_types_count: 0 }, // [891]
    SimpleHintData { raw_values_start: 1461, raw_values_count: 1, token_types_start: 1462, token_types_count: 0 }, // [892]
    SimpleHintData { raw_values_start: 1462, raw_values_count: 1, token_types_start: 1463, token_types_count: 0 }, // [893]
    SimpleHintData { raw_values_start: 1463, raw_values_count: 1, token_types_start: 1464, token_types_count: 0 }, // [894]
    SimpleHintData { raw_values_start: 1464, raw_values_count: 1, token_types_start: 1465, token_types_count: 0 }, // [895]
    SimpleHintData { raw_values_start: 1465, raw_values_count: 1, token_types_start: 1466, token_types_count: 0 }, // [896]
    SimpleHintData { raw_values_start: 1466, raw_values_count: 1, token_types_start: 1467, token_types_count: 0 }, // [897]
    SimpleHintData { raw_values_start: 1467, raw_values_count: 3, token_types_start: 1470, token_types_count: 0 }, // [898]
    SimpleHintData { raw_values_start: 1470, raw_values_count: 1, token_types_start: 1471, token_types_count: 0 }, // [899]
    SimpleHintData { raw_values_start: 1471, raw_values_count: 3, token_types_start: 1474, token_types_count: 0 }, // [900]
    SimpleHintData { raw_values_start: 1474, raw_values_count: 3, token_types_start: 1477, token_types_count: 0 }, // [901]
    SimpleHintData { raw_values_start: 1477, raw_values_count: 1, token_types_start: 1478, token_types_count: 0 }, // [902]
    SimpleHintData { raw_values_start: 1478, raw_values_count: 1, token_types_start: 1479, token_types_count: 0 }, // [903]
    SimpleHintData { raw_values_start: 1479, raw_values_count: 1, token_types_start: 1480, token_types_count: 0 }, // [904]
    SimpleHintData { raw_values_start: 1480, raw_values_count: 1, token_types_start: 1481, token_types_count: 0 }, // [905]
    SimpleHintData { raw_values_start: 1481, raw_values_count: 2, token_types_start: 1483, token_types_count: 0 }, // [906]
    SimpleHintData { raw_values_start: 1483, raw_values_count: 1, token_types_start: 1484, token_types_count: 0 }, // [907]
    SimpleHintData { raw_values_start: 1484, raw_values_count: 1, token_types_start: 1485, token_types_count: 0 }, // [908]
    SimpleHintData { raw_values_start: 1485, raw_values_count: 1, token_types_start: 1486, token_types_count: 0 }, // [909]
    SimpleHintData { raw_values_start: 1486, raw_values_count: 1, token_types_start: 1487, token_types_count: 0 }, // [910]
    SimpleHintData { raw_values_start: 1487, raw_values_count: 1, token_types_start: 1488, token_types_count: 0 }, // [911]
    SimpleHintData { raw_values_start: 1488, raw_values_count: 1, token_types_start: 1489, token_types_count: 0 }, // [912]
    SimpleHintData { raw_values_start: 1489, raw_values_count: 1, token_types_start: 1490, token_types_count: 0 }, // [913]
    SimpleHintData { raw_values_start: 1490, raw_values_count: 1, token_types_start: 1491, token_types_count: 0 }, // [914]
    SimpleHintData { raw_values_start: 1491, raw_values_count: 1, token_types_start: 1492, token_types_count: 0 }, // [915]
    SimpleHintData { raw_values_start: 1492, raw_values_count: 1, token_types_start: 1493, token_types_count: 0 }, // [916]
    SimpleHintData { raw_values_start: 1493, raw_values_count: 1, token_types_start: 1494, token_types_count: 0 }, // [917]
    SimpleHintData { raw_values_start: 1494, raw_values_count: 1, token_types_start: 1495, token_types_count: 0 }, // [918]
    SimpleHintData { raw_values_start: 1495, raw_values_count: 1, token_types_start: 1496, token_types_count: 0 }, // [919]
    SimpleHintData { raw_values_start: 1496, raw_values_count: 1, token_types_start: 1497, token_types_count: 0 }, // [920]
    SimpleHintData { raw_values_start: 1497, raw_values_count: 1, token_types_start: 1498, token_types_count: 0 }, // [921]
    SimpleHintData { raw_values_start: 1498, raw_values_count: 1, token_types_start: 1499, token_types_count: 0 }, // [922]
    SimpleHintData { raw_values_start: 1499, raw_values_count: 1, token_types_start: 1500, token_types_count: 0 }, // [923]
    SimpleHintData { raw_values_start: 1500, raw_values_count: 1, token_types_start: 1501, token_types_count: 0 }, // [924]
    SimpleHintData { raw_values_start: 1501, raw_values_count: 1, token_types_start: 1502, token_types_count: 0 }, // [925]
    SimpleHintData { raw_values_start: 1502, raw_values_count: 1, token_types_start: 1503, token_types_count: 0 }, // [926]
    SimpleHintData { raw_values_start: 1503, raw_values_count: 1, token_types_start: 1504, token_types_count: 0 }, // [927]
    SimpleHintData { raw_values_start: 1504, raw_values_count: 1, token_types_start: 1505, token_types_count: 0 }, // [928]
    SimpleHintData { raw_values_start: 1505, raw_values_count: 1, token_types_start: 1506, token_types_count: 0 }, // [929]
    SimpleHintData { raw_values_start: 1506, raw_values_count: 1, token_types_start: 1507, token_types_count: 0 }, // [930]
    SimpleHintData { raw_values_start: 1507, raw_values_count: 1, token_types_start: 1508, token_types_count: 0 }, // [931]
    SimpleHintData { raw_values_start: 1508, raw_values_count: 1, token_types_start: 1509, token_types_count: 0 }, // [932]
    SimpleHintData { raw_values_start: 1509, raw_values_count: 1, token_types_start: 1510, token_types_count: 0 }, // [933]
    SimpleHintData { raw_values_start: 1510, raw_values_count: 1, token_types_start: 1511, token_types_count: 0 }, // [934]
    SimpleHintData { raw_values_start: 1511, raw_values_count: 1, token_types_start: 1512, token_types_count: 0 }, // [935]
    SimpleHintData { raw_values_start: 1512, raw_values_count: 1, token_types_start: 1513, token_types_count: 0 }, // [936]
    SimpleHintData { raw_values_start: 1513, raw_values_count: 1, token_types_start: 1514, token_types_count: 0 }, // [937]
    SimpleHintData { raw_values_start: 1514, raw_values_count: 2, token_types_start: 1516, token_types_count: 0 }, // [938]
    SimpleHintData { raw_values_start: 1516, raw_values_count: 1, token_types_start: 1517, token_types_count: 0 }, // [939]
    SimpleHintData { raw_values_start: 1517, raw_values_count: 2, token_types_start: 1519, token_types_count: 0 }, // [940]
    SimpleHintData { raw_values_start: 1519, raw_values_count: 1, token_types_start: 1520, token_types_count: 0 }, // [941]
    SimpleHintData { raw_values_start: 1520, raw_values_count: 1, token_types_start: 1521, token_types_count: 0 }, // [942]
    SimpleHintData { raw_values_start: 1521, raw_values_count: 1, token_types_start: 1522, token_types_count: 0 }, // [943]
    SimpleHintData { raw_values_start: 1522, raw_values_count: 10, token_types_start: 1532, token_types_count: 0 }, // [944]
    SimpleHintData { raw_values_start: 1532, raw_values_count: 1, token_types_start: 1533, token_types_count: 0 }, // [945]
    SimpleHintData { raw_values_start: 1533, raw_values_count: 1, token_types_start: 1534, token_types_count: 0 }, // [946]
    SimpleHintData { raw_values_start: 1534, raw_values_count: 1, token_types_start: 1535, token_types_count: 0 }, // [947]
    SimpleHintData { raw_values_start: 1535, raw_values_count: 1, token_types_start: 1536, token_types_count: 0 }, // [948]
    SimpleHintData { raw_values_start: 1536, raw_values_count: 1, token_types_start: 1537, token_types_count: 0 }, // [949]
    SimpleHintData { raw_values_start: 1537, raw_values_count: 1, token_types_start: 1538, token_types_count: 0 }, // [950]
    SimpleHintData { raw_values_start: 1538, raw_values_count: 1, token_types_start: 1539, token_types_count: 0 }, // [951]
    SimpleHintData { raw_values_start: 1539, raw_values_count: 1, token_types_start: 1540, token_types_count: 0 }, // [952]
    SimpleHintData { raw_values_start: 1540, raw_values_count: 1, token_types_start: 1541, token_types_count: 0 }, // [953]
    SimpleHintData { raw_values_start: 1541, raw_values_count: 1, token_types_start: 1542, token_types_count: 0 }, // [954]
    SimpleHintData { raw_values_start: 1542, raw_values_count: 1, token_types_start: 1543, token_types_count: 0 }, // [955]
    SimpleHintData { raw_values_start: 1543, raw_values_count: 1, token_types_start: 1544, token_types_count: 0 }, // [956]
    SimpleHintData { raw_values_start: 1544, raw_values_count: 1, token_types_start: 1545, token_types_count: 0 }, // [957]
    SimpleHintData { raw_values_start: 1545, raw_values_count: 1, token_types_start: 1546, token_types_count: 0 }, // [958]
    SimpleHintData { raw_values_start: 1546, raw_values_count: 1, token_types_start: 1547, token_types_count: 0 }, // [959]
    SimpleHintData { raw_values_start: 1547, raw_values_count: 2, token_types_start: 1549, token_types_count: 0 }, // [960]
    SimpleHintData { raw_values_start: 1549, raw_values_count: 3, token_types_start: 1552, token_types_count: 0 }, // [961]
    SimpleHintData { raw_values_start: 1552, raw_values_count: 1, token_types_start: 1553, token_types_count: 0 }, // [962]
    SimpleHintData { raw_values_start: 1553, raw_values_count: 1, token_types_start: 1554, token_types_count: 0 }, // [963]
    SimpleHintData { raw_values_start: 1554, raw_values_count: 1, token_types_start: 1555, token_types_count: 0 }, // [964]
    SimpleHintData { raw_values_start: 1555, raw_values_count: 1, token_types_start: 1556, token_types_count: 0 }, // [965]
    SimpleHintData { raw_values_start: 1556, raw_values_count: 1, token_types_start: 1557, token_types_count: 0 }, // [966]
    SimpleHintData { raw_values_start: 1557, raw_values_count: 1, token_types_start: 1558, token_types_count: 0 }, // [967]
    SimpleHintData { raw_values_start: 1558, raw_values_count: 1, token_types_start: 1559, token_types_count: 0 }, // [968]
    SimpleHintData { raw_values_start: 1559, raw_values_count: 2, token_types_start: 1561, token_types_count: 0 }, // [969]
    SimpleHintData { raw_values_start: 1561, raw_values_count: 3, token_types_start: 1564, token_types_count: 0 }, // [970]
    SimpleHintData { raw_values_start: 1564, raw_values_count: 2, token_types_start: 1566, token_types_count: 0 }, // [971]
    SimpleHintData { raw_values_start: 1566, raw_values_count: 1, token_types_start: 1567, token_types_count: 0 }, // [972]
    SimpleHintData { raw_values_start: 1567, raw_values_count: 1, token_types_start: 1568, token_types_count: 0 }, // [973]
    SimpleHintData { raw_values_start: 1568, raw_values_count: 1, token_types_start: 1569, token_types_count: 0 }, // [974]
    SimpleHintData { raw_values_start: 1569, raw_values_count: 1, token_types_start: 1570, token_types_count: 0 }, // [975]
    SimpleHintData { raw_values_start: 1570, raw_values_count: 1, token_types_start: 1571, token_types_count: 0 }, // [976]
    SimpleHintData { raw_values_start: 1571, raw_values_count: 1, token_types_start: 1572, token_types_count: 0 }, // [977]
    SimpleHintData { raw_values_start: 1572, raw_values_count: 1, token_types_start: 1573, token_types_count: 0 }, // [978]
    SimpleHintData { raw_values_start: 1573, raw_values_count: 1, token_types_start: 1574, token_types_count: 0 }, // [979]
    SimpleHintData { raw_values_start: 1574, raw_values_count: 1, token_types_start: 1575, token_types_count: 0 }, // [980]
    SimpleHintData { raw_values_start: 1575, raw_values_count: 1, token_types_start: 1576, token_types_count: 0 }, // [981]
    SimpleHintData { raw_values_start: 1576, raw_values_count: 1, token_types_start: 1577, token_types_count: 0 }, // [982]
    SimpleHintData { raw_values_start: 1577, raw_values_count: 1, token_types_start: 1578, token_types_count: 0 }, // [983]
    SimpleHintData { raw_values_start: 1578, raw_values_count: 1, token_types_start: 1579, token_types_count: 0 }, // [984]
    SimpleHintData { raw_values_start: 1579, raw_values_count: 1, token_types_start: 1580, token_types_count: 0 }, // [985]
    SimpleHintData { raw_values_start: 1580, raw_values_count: 1, token_types_start: 1581, token_types_count: 0 }, // [986]
    SimpleHintData { raw_values_start: 1581, raw_values_count: 1, token_types_start: 1582, token_types_count: 0 }, // [987]
    SimpleHintData { raw_values_start: 1582, raw_values_count: 1, token_types_start: 1583, token_types_count: 0 }, // [988]
    SimpleHintData { raw_values_start: 1583, raw_values_count: 1, token_types_start: 1584, token_types_count: 0 }, // [989]
    SimpleHintData { raw_values_start: 1584, raw_values_count: 1, token_types_start: 1585, token_types_count: 0 }, // [990]
    SimpleHintData { raw_values_start: 1585, raw_values_count: 1, token_types_start: 1586, token_types_count: 0 }, // [991]
    SimpleHintData { raw_values_start: 1586, raw_values_count: 1, token_types_start: 1587, token_types_count: 0 }, // [992]
    SimpleHintData { raw_values_start: 1587, raw_values_count: 1, token_types_start: 1588, token_types_count: 0 }, // [993]
    SimpleHintData { raw_values_start: 1588, raw_values_count: 1, token_types_start: 1589, token_types_count: 0 }, // [994]
    SimpleHintData { raw_values_start: 1589, raw_values_count: 1, token_types_start: 1590, token_types_count: 0 }, // [995]
    SimpleHintData { raw_values_start: 1590, raw_values_count: 1, token_types_start: 1591, token_types_count: 0 }, // [996]
    SimpleHintData { raw_values_start: 1591, raw_values_count: 1, token_types_start: 1592, token_types_count: 0 }, // [997]
    SimpleHintData { raw_values_start: 1592, raw_values_count: 3, token_types_start: 1595, token_types_count: 0 }, // [998]
    SimpleHintData { raw_values_start: 1595, raw_values_count: 1, token_types_start: 1596, token_types_count: 0 }, // [999]
    SimpleHintData { raw_values_start: 1596, raw_values_count: 1, token_types_start: 1597, token_types_count: 0 }, // [1000]
    SimpleHintData { raw_values_start: 1597, raw_values_count: 2, token_types_start: 1599, token_types_count: 0 }, // [1001]
    SimpleHintData { raw_values_start: 1599, raw_values_count: 1, token_types_start: 1600, token_types_count: 0 }, // [1002]
    SimpleHintData { raw_values_start: 1600, raw_values_count: 1, token_types_start: 1601, token_types_count: 0 }, // [1003]
    SimpleHintData { raw_values_start: 1601, raw_values_count: 1, token_types_start: 1602, token_types_count: 0 }, // [1004]
    SimpleHintData { raw_values_start: 1602, raw_values_count: 1, token_types_start: 1603, token_types_count: 0 }, // [1005]
    SimpleHintData { raw_values_start: 1603, raw_values_count: 1, token_types_start: 1604, token_types_count: 0 }, // [1006]
    SimpleHintData { raw_values_start: 1604, raw_values_count: 1, token_types_start: 1605, token_types_count: 0 }, // [1007]
    SimpleHintData { raw_values_start: 1605, raw_values_count: 1, token_types_start: 1606, token_types_count: 0 }, // [1008]
    SimpleHintData { raw_values_start: 1606, raw_values_count: 1, token_types_start: 1607, token_types_count: 0 }, // [1009]
    SimpleHintData { raw_values_start: 1607, raw_values_count: 2, token_types_start: 1609, token_types_count: 0 }, // [1010]
    SimpleHintData { raw_values_start: 1609, raw_values_count: 2, token_types_start: 1611, token_types_count: 0 }, // [1011]
    SimpleHintData { raw_values_start: 1611, raw_values_count: 2, token_types_start: 1613, token_types_count: 0 }, // [1012]
    SimpleHintData { raw_values_start: 1613, raw_values_count: 1, token_types_start: 1614, token_types_count: 0 }, // [1013]
    SimpleHintData { raw_values_start: 1614, raw_values_count: 1, token_types_start: 1615, token_types_count: 0 }, // [1014]
    SimpleHintData { raw_values_start: 1615, raw_values_count: 1, token_types_start: 1616, token_types_count: 0 }, // [1015]
    SimpleHintData { raw_values_start: 1616, raw_values_count: 1, token_types_start: 1617, token_types_count: 0 }, // [1016]
    SimpleHintData { raw_values_start: 1617, raw_values_count: 1, token_types_start: 1618, token_types_count: 0 }, // [1017]
    SimpleHintData { raw_values_start: 1618, raw_values_count: 1, token_types_start: 1619, token_types_count: 0 }, // [1018]
    SimpleHintData { raw_values_start: 1619, raw_values_count: 1, token_types_start: 1620, token_types_count: 0 }, // [1019]
    SimpleHintData { raw_values_start: 1620, raw_values_count: 1, token_types_start: 1621, token_types_count: 0 }, // [1020]
    SimpleHintData { raw_values_start: 1621, raw_values_count: 1, token_types_start: 1622, token_types_count: 0 }, // [1021]
    SimpleHintData { raw_values_start: 1622, raw_values_count: 1, token_types_start: 1623, token_types_count: 0 }, // [1022]
    SimpleHintData { raw_values_start: 1623, raw_values_count: 1, token_types_start: 1624, token_types_count: 0 }, // [1023]
    SimpleHintData { raw_values_start: 1624, raw_values_count: 1, token_types_start: 1625, token_types_count: 0 }, // [1024]
    SimpleHintData { raw_values_start: 1625, raw_values_count: 1, token_types_start: 1626, token_types_count: 0 }, // [1025]
    SimpleHintData { raw_values_start: 1626, raw_values_count: 1, token_types_start: 1627, token_types_count: 0 }, // [1026]
    SimpleHintData { raw_values_start: 1627, raw_values_count: 1, token_types_start: 1628, token_types_count: 0 }, // [1027]
    SimpleHintData { raw_values_start: 1628, raw_values_count: 1, token_types_start: 1629, token_types_count: 0 }, // [1028]
    SimpleHintData { raw_values_start: 1629, raw_values_count: 1, token_types_start: 1630, token_types_count: 0 }, // [1029]
    SimpleHintData { raw_values_start: 1630, raw_values_count: 1, token_types_start: 1631, token_types_count: 0 }, // [1030]
    SimpleHintData { raw_values_start: 1631, raw_values_count: 1, token_types_start: 1632, token_types_count: 0 }, // [1031]
    SimpleHintData { raw_values_start: 1632, raw_values_count: 1, token_types_start: 1633, token_types_count: 0 }, // [1032]
    SimpleHintData { raw_values_start: 1633, raw_values_count: 1, token_types_start: 1634, token_types_count: 0 }, // [1033]
    SimpleHintData { raw_values_start: 1634, raw_values_count: 1, token_types_start: 1635, token_types_count: 0 }, // [1034]
    SimpleHintData { raw_values_start: 1635, raw_values_count: 1, token_types_start: 1636, token_types_count: 0 }, // [1035]
    SimpleHintData { raw_values_start: 1636, raw_values_count: 1, token_types_start: 1637, token_types_count: 0 }, // [1036]
    SimpleHintData { raw_values_start: 1637, raw_values_count: 1, token_types_start: 1638, token_types_count: 0 }, // [1037]
    SimpleHintData { raw_values_start: 1638, raw_values_count: 1, token_types_start: 1639, token_types_count: 0 }, // [1038]
    SimpleHintData { raw_values_start: 1639, raw_values_count: 1, token_types_start: 1640, token_types_count: 0 }, // [1039]
    SimpleHintData { raw_values_start: 1640, raw_values_count: 1, token_types_start: 1641, token_types_count: 0 }, // [1040]
    SimpleHintData { raw_values_start: 1641, raw_values_count: 1, token_types_start: 1642, token_types_count: 0 }, // [1041]
    SimpleHintData { raw_values_start: 1642, raw_values_count: 1, token_types_start: 1643, token_types_count: 0 }, // [1042]
    SimpleHintData { raw_values_start: 1643, raw_values_count: 1, token_types_start: 1644, token_types_count: 0 }, // [1043]
    SimpleHintData { raw_values_start: 1644, raw_values_count: 1, token_types_start: 1645, token_types_count: 0 }, // [1044]
    SimpleHintData { raw_values_start: 1645, raw_values_count: 1, token_types_start: 1646, token_types_count: 0 }, // [1045]
    SimpleHintData { raw_values_start: 1646, raw_values_count: 1, token_types_start: 1647, token_types_count: 0 }, // [1046]
    SimpleHintData { raw_values_start: 1647, raw_values_count: 1, token_types_start: 1648, token_types_count: 0 }, // [1047]
    SimpleHintData { raw_values_start: 1648, raw_values_count: 1, token_types_start: 1649, token_types_count: 0 }, // [1048]
    SimpleHintData { raw_values_start: 1649, raw_values_count: 1, token_types_start: 1650, token_types_count: 0 }, // [1049]
    SimpleHintData { raw_values_start: 1650, raw_values_count: 1, token_types_start: 1651, token_types_count: 0 }, // [1050]
    SimpleHintData { raw_values_start: 1651, raw_values_count: 1, token_types_start: 1652, token_types_count: 0 }, // [1051]
    SimpleHintData { raw_values_start: 1652, raw_values_count: 1, token_types_start: 1653, token_types_count: 0 }, // [1052]
    SimpleHintData { raw_values_start: 1653, raw_values_count: 1, token_types_start: 1654, token_types_count: 0 }, // [1053]
    SimpleHintData { raw_values_start: 1654, raw_values_count: 1, token_types_start: 1655, token_types_count: 0 }, // [1054]
    SimpleHintData { raw_values_start: 1655, raw_values_count: 1, token_types_start: 1656, token_types_count: 0 }, // [1055]
    SimpleHintData { raw_values_start: 1656, raw_values_count: 1, token_types_start: 1657, token_types_count: 0 }, // [1056]
    SimpleHintData { raw_values_start: 1657, raw_values_count: 1, token_types_start: 1658, token_types_count: 0 }, // [1057]
    SimpleHintData { raw_values_start: 1658, raw_values_count: 1, token_types_start: 1659, token_types_count: 0 }, // [1058]
    SimpleHintData { raw_values_start: 1659, raw_values_count: 1, token_types_start: 1660, token_types_count: 0 }, // [1059]
    SimpleHintData { raw_values_start: 1660, raw_values_count: 1, token_types_start: 1661, token_types_count: 0 }, // [1060]
    SimpleHintData { raw_values_start: 1661, raw_values_count: 1, token_types_start: 1662, token_types_count: 0 }, // [1061]
    SimpleHintData { raw_values_start: 1662, raw_values_count: 1, token_types_start: 1663, token_types_count: 0 }, // [1062]
    SimpleHintData { raw_values_start: 1663, raw_values_count: 1, token_types_start: 1664, token_types_count: 0 }, // [1063]
    SimpleHintData { raw_values_start: 1664, raw_values_count: 1, token_types_start: 1665, token_types_count: 0 }, // [1064]
    SimpleHintData { raw_values_start: 1665, raw_values_count: 1, token_types_start: 1666, token_types_count: 0 }, // [1065]
    SimpleHintData { raw_values_start: 1666, raw_values_count: 1, token_types_start: 1667, token_types_count: 0 }, // [1066]
    SimpleHintData { raw_values_start: 1667, raw_values_count: 1, token_types_start: 1668, token_types_count: 0 }, // [1067]
    SimpleHintData { raw_values_start: 1668, raw_values_count: 1, token_types_start: 1669, token_types_count: 0 }, // [1068]
    SimpleHintData { raw_values_start: 1669, raw_values_count: 1, token_types_start: 1670, token_types_count: 0 }, // [1069]
    SimpleHintData { raw_values_start: 1670, raw_values_count: 1, token_types_start: 1671, token_types_count: 0 }, // [1070]
    SimpleHintData { raw_values_start: 1671, raw_values_count: 2, token_types_start: 1673, token_types_count: 0 }, // [1071]
    SimpleHintData { raw_values_start: 1673, raw_values_count: 1, token_types_start: 1674, token_types_count: 0 }, // [1072]
    SimpleHintData { raw_values_start: 1674, raw_values_count: 1, token_types_start: 1675, token_types_count: 0 }, // [1073]
    SimpleHintData { raw_values_start: 1675, raw_values_count: 1, token_types_start: 1676, token_types_count: 0 }, // [1074]
    SimpleHintData { raw_values_start: 1676, raw_values_count: 1, token_types_start: 1677, token_types_count: 0 }, // [1075]
    SimpleHintData { raw_values_start: 1677, raw_values_count: 1, token_types_start: 1678, token_types_count: 0 }, // [1076]
    SimpleHintData { raw_values_start: 1678, raw_values_count: 1, token_types_start: 1679, token_types_count: 0 }, // [1077]
    SimpleHintData { raw_values_start: 1679, raw_values_count: 1, token_types_start: 1680, token_types_count: 0 }, // [1078]
    SimpleHintData { raw_values_start: 1680, raw_values_count: 1, token_types_start: 1681, token_types_count: 0 }, // [1079]
    SimpleHintData { raw_values_start: 1681, raw_values_count: 1, token_types_start: 1682, token_types_count: 0 }, // [1080]
    SimpleHintData { raw_values_start: 1682, raw_values_count: 1, token_types_start: 1683, token_types_count: 0 }, // [1081]
    SimpleHintData { raw_values_start: 1683, raw_values_count: 1, token_types_start: 1684, token_types_count: 0 }, // [1082]
    SimpleHintData { raw_values_start: 1684, raw_values_count: 1, token_types_start: 1685, token_types_count: 0 }, // [1083]
    SimpleHintData { raw_values_start: 1685, raw_values_count: 1, token_types_start: 1686, token_types_count: 0 }, // [1084]
    SimpleHintData { raw_values_start: 1686, raw_values_count: 1, token_types_start: 1687, token_types_count: 0 }, // [1085]
    SimpleHintData { raw_values_start: 1687, raw_values_count: 1, token_types_start: 1688, token_types_count: 0 }, // [1086]
    SimpleHintData { raw_values_start: 1688, raw_values_count: 1, token_types_start: 1689, token_types_count: 0 }, // [1087]
    SimpleHintData { raw_values_start: 1689, raw_values_count: 1, token_types_start: 1690, token_types_count: 0 }, // [1088]
    SimpleHintData { raw_values_start: 1690, raw_values_count: 1, token_types_start: 1691, token_types_count: 0 }, // [1089]
    SimpleHintData { raw_values_start: 1691, raw_values_count: 1, token_types_start: 1692, token_types_count: 0 }, // [1090]
    SimpleHintData { raw_values_start: 1692, raw_values_count: 1, token_types_start: 1693, token_types_count: 0 }, // [1091]
    SimpleHintData { raw_values_start: 1693, raw_values_count: 1, token_types_start: 1694, token_types_count: 0 }, // [1092]
    SimpleHintData { raw_values_start: 1694, raw_values_count: 1, token_types_start: 1695, token_types_count: 0 }, // [1093]
    SimpleHintData { raw_values_start: 1695, raw_values_count: 1, token_types_start: 1696, token_types_count: 0 }, // [1094]
    SimpleHintData { raw_values_start: 1696, raw_values_count: 1, token_types_start: 1697, token_types_count: 0 }, // [1095]
    SimpleHintData { raw_values_start: 1697, raw_values_count: 1, token_types_start: 1698, token_types_count: 0 }, // [1096]
    SimpleHintData { raw_values_start: 1698, raw_values_count: 1, token_types_start: 1699, token_types_count: 0 }, // [1097]
    SimpleHintData { raw_values_start: 1699, raw_values_count: 1, token_types_start: 1700, token_types_count: 0 }, // [1098]
    SimpleHintData { raw_values_start: 1700, raw_values_count: 1, token_types_start: 1701, token_types_count: 0 }, // [1099]
    SimpleHintData { raw_values_start: 1701, raw_values_count: 1, token_types_start: 1702, token_types_count: 0 }, // [1100]
    SimpleHintData { raw_values_start: 1702, raw_values_count: 1, token_types_start: 1703, token_types_count: 0 }, // [1101]
    SimpleHintData { raw_values_start: 1703, raw_values_count: 1, token_types_start: 1704, token_types_count: 0 }, // [1102]
    SimpleHintData { raw_values_start: 1704, raw_values_count: 1, token_types_start: 1705, token_types_count: 0 }, // [1103]
    SimpleHintData { raw_values_start: 1705, raw_values_count: 1, token_types_start: 1706, token_types_count: 0 }, // [1104]
    SimpleHintData { raw_values_start: 1706, raw_values_count: 1, token_types_start: 1707, token_types_count: 0 }, // [1105]
    SimpleHintData { raw_values_start: 1707, raw_values_count: 1, token_types_start: 1708, token_types_count: 0 }, // [1106]
    SimpleHintData { raw_values_start: 1708, raw_values_count: 1, token_types_start: 1709, token_types_count: 0 }, // [1107]
    SimpleHintData { raw_values_start: 1709, raw_values_count: 1, token_types_start: 1710, token_types_count: 0 }, // [1108]
    SimpleHintData { raw_values_start: 1710, raw_values_count: 1, token_types_start: 1711, token_types_count: 0 }, // [1109]
    SimpleHintData { raw_values_start: 1711, raw_values_count: 1, token_types_start: 1712, token_types_count: 0 }, // [1110]
    SimpleHintData { raw_values_start: 1712, raw_values_count: 1, token_types_start: 1713, token_types_count: 0 }, // [1111]
    SimpleHintData { raw_values_start: 1713, raw_values_count: 1, token_types_start: 1714, token_types_count: 0 }, // [1112]
    SimpleHintData { raw_values_start: 1714, raw_values_count: 1, token_types_start: 1715, token_types_count: 0 }, // [1113]
    SimpleHintData { raw_values_start: 1715, raw_values_count: 1, token_types_start: 1716, token_types_count: 0 }, // [1114]
    SimpleHintData { raw_values_start: 1716, raw_values_count: 1, token_types_start: 1717, token_types_count: 0 }, // [1115]
    SimpleHintData { raw_values_start: 1717, raw_values_count: 1, token_types_start: 1718, token_types_count: 0 }, // [1116]
    SimpleHintData { raw_values_start: 1718, raw_values_count: 1, token_types_start: 1719, token_types_count: 0 }, // [1117]
    SimpleHintData { raw_values_start: 1719, raw_values_count: 1, token_types_start: 1720, token_types_count: 0 }, // [1118]
    SimpleHintData { raw_values_start: 1720, raw_values_count: 1, token_types_start: 1721, token_types_count: 0 }, // [1119]
    SimpleHintData { raw_values_start: 1721, raw_values_count: 1, token_types_start: 1722, token_types_count: 0 }, // [1120]
    SimpleHintData { raw_values_start: 1722, raw_values_count: 1, token_types_start: 1723, token_types_count: 0 }, // [1121]
    SimpleHintData { raw_values_start: 1723, raw_values_count: 1, token_types_start: 1724, token_types_count: 0 }, // [1122]
    SimpleHintData { raw_values_start: 1724, raw_values_count: 1, token_types_start: 1725, token_types_count: 0 }, // [1123]
    SimpleHintData { raw_values_start: 1725, raw_values_count: 4, token_types_start: 1729, token_types_count: 0 }, // [1124]
    SimpleHintData { raw_values_start: 1729, raw_values_count: 2, token_types_start: 1731, token_types_count: 0 }, // [1125]
    SimpleHintData { raw_values_start: 1731, raw_values_count: 1, token_types_start: 1732, token_types_count: 0 }, // [1126]
    SimpleHintData { raw_values_start: 1732, raw_values_count: 1, token_types_start: 1733, token_types_count: 0 }, // [1127]
    SimpleHintData { raw_values_start: 1733, raw_values_count: 1, token_types_start: 1734, token_types_count: 0 }, // [1128]
    SimpleHintData { raw_values_start: 1734, raw_values_count: 1, token_types_start: 1735, token_types_count: 0 }, // [1129]
    SimpleHintData { raw_values_start: 1735, raw_values_count: 1, token_types_start: 1736, token_types_count: 0 }, // [1130]
    SimpleHintData { raw_values_start: 1736, raw_values_count: 1, token_types_start: 1737, token_types_count: 0 }, // [1131]
    SimpleHintData { raw_values_start: 1737, raw_values_count: 1, token_types_start: 1738, token_types_count: 0 }, // [1132]
    SimpleHintData { raw_values_start: 1738, raw_values_count: 1, token_types_start: 1739, token_types_count: 0 }, // [1133]
    SimpleHintData { raw_values_start: 1739, raw_values_count: 1, token_types_start: 1740, token_types_count: 0 }, // [1134]
    SimpleHintData { raw_values_start: 1740, raw_values_count: 1, token_types_start: 1741, token_types_count: 0 }, // [1135]
    SimpleHintData { raw_values_start: 1741, raw_values_count: 1, token_types_start: 1742, token_types_count: 0 }, // [1136]
    SimpleHintData { raw_values_start: 1742, raw_values_count: 1, token_types_start: 1743, token_types_count: 0 }, // [1137]
    SimpleHintData { raw_values_start: 1743, raw_values_count: 1, token_types_start: 1744, token_types_count: 0 }, // [1138]
    SimpleHintData { raw_values_start: 1744, raw_values_count: 1, token_types_start: 1745, token_types_count: 0 }, // [1139]
    SimpleHintData { raw_values_start: 1745, raw_values_count: 1, token_types_start: 1746, token_types_count: 0 }, // [1140]
    SimpleHintData { raw_values_start: 1746, raw_values_count: 1, token_types_start: 1747, token_types_count: 0 }, // [1141]
    SimpleHintData { raw_values_start: 1747, raw_values_count: 1, token_types_start: 1748, token_types_count: 0 }, // [1142]
    SimpleHintData { raw_values_start: 1748, raw_values_count: 1, token_types_start: 1749, token_types_count: 0 }, // [1143]
    SimpleHintData { raw_values_start: 1749, raw_values_count: 1, token_types_start: 1750, token_types_count: 0 }, // [1144]
    SimpleHintData { raw_values_start: 1750, raw_values_count: 1, token_types_start: 1751, token_types_count: 0 }, // [1145]
    SimpleHintData { raw_values_start: 1751, raw_values_count: 1, token_types_start: 1752, token_types_count: 0 }, // [1146]
    SimpleHintData { raw_values_start: 1752, raw_values_count: 1, token_types_start: 1753, token_types_count: 0 }, // [1147]
    SimpleHintData { raw_values_start: 1753, raw_values_count: 1, token_types_start: 1754, token_types_count: 0 }, // [1148]
    SimpleHintData { raw_values_start: 1754, raw_values_count: 1, token_types_start: 1755, token_types_count: 0 }, // [1149]
    SimpleHintData { raw_values_start: 1755, raw_values_count: 1, token_types_start: 1756, token_types_count: 0 }, // [1150]
    SimpleHintData { raw_values_start: 1756, raw_values_count: 1, token_types_start: 1757, token_types_count: 0 }, // [1151]
    SimpleHintData { raw_values_start: 1757, raw_values_count: 1, token_types_start: 1758, token_types_count: 0 }, // [1152]
    SimpleHintData { raw_values_start: 1758, raw_values_count: 1, token_types_start: 1759, token_types_count: 0 }, // [1153]
    SimpleHintData { raw_values_start: 1759, raw_values_count: 1, token_types_start: 1760, token_types_count: 0 }, // [1154]
    SimpleHintData { raw_values_start: 1760, raw_values_count: 1, token_types_start: 1761, token_types_count: 0 }, // [1155]
    SimpleHintData { raw_values_start: 1761, raw_values_count: 1, token_types_start: 1762, token_types_count: 0 }, // [1156]
    SimpleHintData { raw_values_start: 1762, raw_values_count: 1, token_types_start: 1763, token_types_count: 0 }, // [1157]
    SimpleHintData { raw_values_start: 1763, raw_values_count: 1, token_types_start: 1764, token_types_count: 0 }, // [1158]
    SimpleHintData { raw_values_start: 1764, raw_values_count: 1, token_types_start: 1765, token_types_count: 0 }, // [1159]
    SimpleHintData { raw_values_start: 1765, raw_values_count: 1, token_types_start: 1766, token_types_count: 0 }, // [1160]
    SimpleHintData { raw_values_start: 1766, raw_values_count: 1, token_types_start: 1767, token_types_count: 0 }, // [1161]
    SimpleHintData { raw_values_start: 1767, raw_values_count: 2, token_types_start: 1769, token_types_count: 0 }, // [1162]
    SimpleHintData { raw_values_start: 1769, raw_values_count: 8, token_types_start: 1777, token_types_count: 0 }, // [1163]
    SimpleHintData { raw_values_start: 1777, raw_values_count: 1, token_types_start: 1778, token_types_count: 0 }, // [1164]
    SimpleHintData { raw_values_start: 1778, raw_values_count: 1, token_types_start: 1779, token_types_count: 0 }, // [1165]
    SimpleHintData { raw_values_start: 1779, raw_values_count: 1, token_types_start: 1780, token_types_count: 0 }, // [1166]
    SimpleHintData { raw_values_start: 1780, raw_values_count: 1, token_types_start: 1781, token_types_count: 0 }, // [1167]
    SimpleHintData { raw_values_start: 1781, raw_values_count: 1, token_types_start: 1782, token_types_count: 0 }, // [1168]
    SimpleHintData { raw_values_start: 1782, raw_values_count: 1, token_types_start: 1783, token_types_count: 0 }, // [1169]
    SimpleHintData { raw_values_start: 1783, raw_values_count: 1, token_types_start: 1784, token_types_count: 0 }, // [1170]
    SimpleHintData { raw_values_start: 1784, raw_values_count: 1, token_types_start: 1785, token_types_count: 0 }, // [1171]
    SimpleHintData { raw_values_start: 1785, raw_values_count: 1, token_types_start: 1786, token_types_count: 0 }, // [1172]
    SimpleHintData { raw_values_start: 1786, raw_values_count: 1, token_types_start: 1787, token_types_count: 0 }, // [1173]
    SimpleHintData { raw_values_start: 1787, raw_values_count: 1, token_types_start: 1788, token_types_count: 0 }, // [1174]
    SimpleHintData { raw_values_start: 1788, raw_values_count: 1, token_types_start: 1789, token_types_count: 0 }, // [1175]
    SimpleHintData { raw_values_start: 1789, raw_values_count: 1, token_types_start: 1790, token_types_count: 0 }, // [1176]
    SimpleHintData { raw_values_start: 1790, raw_values_count: 1, token_types_start: 1791, token_types_count: 0 }, // [1177]
    SimpleHintData { raw_values_start: 1791, raw_values_count: 1, token_types_start: 1792, token_types_count: 0 }, // [1178]
    SimpleHintData { raw_values_start: 1792, raw_values_count: 1, token_types_start: 1793, token_types_count: 0 }, // [1179]
    SimpleHintData { raw_values_start: 1793, raw_values_count: 1, token_types_start: 1794, token_types_count: 0 }, // [1180]
    SimpleHintData { raw_values_start: 1794, raw_values_count: 1, token_types_start: 1795, token_types_count: 0 }, // [1181]
    SimpleHintData { raw_values_start: 1795, raw_values_count: 1, token_types_start: 1796, token_types_count: 0 }, // [1182]
    SimpleHintData { raw_values_start: 1796, raw_values_count: 1, token_types_start: 1797, token_types_count: 0 }, // [1183]
    SimpleHintData { raw_values_start: 1797, raw_values_count: 1, token_types_start: 1798, token_types_count: 0 }, // [1184]
    SimpleHintData { raw_values_start: 1798, raw_values_count: 1, token_types_start: 1799, token_types_count: 0 }, // [1185]
    SimpleHintData { raw_values_start: 1799, raw_values_count: 1, token_types_start: 1800, token_types_count: 0 }, // [1186]
    SimpleHintData { raw_values_start: 1800, raw_values_count: 1, token_types_start: 1801, token_types_count: 0 }, // [1187]
    SimpleHintData { raw_values_start: 1801, raw_values_count: 1, token_types_start: 1802, token_types_count: 0 }, // [1188]
    SimpleHintData { raw_values_start: 1802, raw_values_count: 2, token_types_start: 1804, token_types_count: 0 }, // [1189]
    SimpleHintData { raw_values_start: 1804, raw_values_count: 2, token_types_start: 1806, token_types_count: 0 }, // [1190]
    SimpleHintData { raw_values_start: 1806, raw_values_count: 1, token_types_start: 1807, token_types_count: 0 }, // [1191]
    SimpleHintData { raw_values_start: 1807, raw_values_count: 1, token_types_start: 1808, token_types_count: 0 }, // [1192]
    SimpleHintData { raw_values_start: 1808, raw_values_count: 1, token_types_start: 1809, token_types_count: 0 }, // [1193]
    SimpleHintData { raw_values_start: 1809, raw_values_count: 1, token_types_start: 1810, token_types_count: 0 }, // [1194]
    SimpleHintData { raw_values_start: 1810, raw_values_count: 1, token_types_start: 1811, token_types_count: 0 }, // [1195]
    SimpleHintData { raw_values_start: 1811, raw_values_count: 1, token_types_start: 1812, token_types_count: 2 }, // [1196]
    SimpleHintData { raw_values_start: 1814, raw_values_count: 1, token_types_start: 1815, token_types_count: 0 }, // [1197]
    SimpleHintData { raw_values_start: 1815, raw_values_count: 1, token_types_start: 1816, token_types_count: 0 }, // [1198]
    SimpleHintData { raw_values_start: 1816, raw_values_count: 1, token_types_start: 1817, token_types_count: 0 }, // [1199]
    SimpleHintData { raw_values_start: 1817, raw_values_count: 1, token_types_start: 1818, token_types_count: 0 }, // [1200]
    SimpleHintData { raw_values_start: 1818, raw_values_count: 1, token_types_start: 1819, token_types_count: 0 }, // [1201]
    SimpleHintData { raw_values_start: 1819, raw_values_count: 1, token_types_start: 1820, token_types_count: 0 }, // [1202]
    SimpleHintData { raw_values_start: 1820, raw_values_count: 1, token_types_start: 1821, token_types_count: 0 }, // [1203]
    SimpleHintData { raw_values_start: 1821, raw_values_count: 1, token_types_start: 1822, token_types_count: 0 }, // [1204]
    SimpleHintData { raw_values_start: 1822, raw_values_count: 1, token_types_start: 1823, token_types_count: 0 }, // [1205]
    SimpleHintData { raw_values_start: 1823, raw_values_count: 1, token_types_start: 1824, token_types_count: 0 }, // [1206]
    SimpleHintData { raw_values_start: 1824, raw_values_count: 1, token_types_start: 1825, token_types_count: 0 }, // [1207]
    SimpleHintData { raw_values_start: 1825, raw_values_count: 1, token_types_start: 1826, token_types_count: 0 }, // [1208]
    SimpleHintData { raw_values_start: 1826, raw_values_count: 1, token_types_start: 1827, token_types_count: 0 }, // [1209]
    SimpleHintData { raw_values_start: 1827, raw_values_count: 1, token_types_start: 1828, token_types_count: 0 }, // [1210]
    SimpleHintData { raw_values_start: 1828, raw_values_count: 1, token_types_start: 1829, token_types_count: 0 }, // [1211]
    SimpleHintData { raw_values_start: 1829, raw_values_count: 1, token_types_start: 1830, token_types_count: 0 }, // [1212]
    SimpleHintData { raw_values_start: 1830, raw_values_count: 1, token_types_start: 1831, token_types_count: 0 }, // [1213]
    SimpleHintData { raw_values_start: 1831, raw_values_count: 1, token_types_start: 1832, token_types_count: 0 }, // [1214]
    SimpleHintData { raw_values_start: 1832, raw_values_count: 1, token_types_start: 1833, token_types_count: 0 }, // [1215]
    SimpleHintData { raw_values_start: 1833, raw_values_count: 1, token_types_start: 1834, token_types_count: 0 }, // [1216]
    SimpleHintData { raw_values_start: 1834, raw_values_count: 1, token_types_start: 1835, token_types_count: 0 }, // [1217]
    SimpleHintData { raw_values_start: 1835, raw_values_count: 1, token_types_start: 1836, token_types_count: 0 }, // [1218]
    SimpleHintData { raw_values_start: 1836, raw_values_count: 1, token_types_start: 1837, token_types_count: 0 }, // [1219]
    SimpleHintData { raw_values_start: 1837, raw_values_count: 1, token_types_start: 1838, token_types_count: 0 }, // [1220]
    SimpleHintData { raw_values_start: 1838, raw_values_count: 1, token_types_start: 1839, token_types_count: 0 }, // [1221]
    SimpleHintData { raw_values_start: 1839, raw_values_count: 1, token_types_start: 1840, token_types_count: 0 }, // [1222]
    SimpleHintData { raw_values_start: 1840, raw_values_count: 1, token_types_start: 1841, token_types_count: 0 }, // [1223]
    SimpleHintData { raw_values_start: 1841, raw_values_count: 1, token_types_start: 1842, token_types_count: 0 }, // [1224]
    SimpleHintData { raw_values_start: 1842, raw_values_count: 1, token_types_start: 1843, token_types_count: 0 }, // [1225]
    SimpleHintData { raw_values_start: 1843, raw_values_count: 1, token_types_start: 1844, token_types_count: 0 }, // [1226]
    SimpleHintData { raw_values_start: 1844, raw_values_count: 1, token_types_start: 1845, token_types_count: 0 }, // [1227]
    SimpleHintData { raw_values_start: 1845, raw_values_count: 1, token_types_start: 1846, token_types_count: 0 }, // [1228]
    SimpleHintData { raw_values_start: 1846, raw_values_count: 1, token_types_start: 1847, token_types_count: 0 }, // [1229]
    SimpleHintData { raw_values_start: 1847, raw_values_count: 1, token_types_start: 1848, token_types_count: 0 }, // [1230]
    SimpleHintData { raw_values_start: 1848, raw_values_count: 1, token_types_start: 1849, token_types_count: 0 }, // [1231]
    SimpleHintData { raw_values_start: 1849, raw_values_count: 3, token_types_start: 1852, token_types_count: 0 }, // [1232]
    SimpleHintData { raw_values_start: 1852, raw_values_count: 1, token_types_start: 1853, token_types_count: 0 }, // [1233]
    SimpleHintData { raw_values_start: 1853, raw_values_count: 1, token_types_start: 1854, token_types_count: 0 }, // [1234]
    SimpleHintData { raw_values_start: 1854, raw_values_count: 1, token_types_start: 1855, token_types_count: 0 }, // [1235]
    SimpleHintData { raw_values_start: 1855, raw_values_count: 1, token_types_start: 1856, token_types_count: 0 }, // [1236]
    SimpleHintData { raw_values_start: 1856, raw_values_count: 1, token_types_start: 1857, token_types_count: 0 }, // [1237]
    SimpleHintData { raw_values_start: 1857, raw_values_count: 1, token_types_start: 1858, token_types_count: 0 }, // [1238]
    SimpleHintData { raw_values_start: 1858, raw_values_count: 1, token_types_start: 1859, token_types_count: 0 }, // [1239]
    SimpleHintData { raw_values_start: 1859, raw_values_count: 2, token_types_start: 1861, token_types_count: 0 }, // [1240]
    SimpleHintData { raw_values_start: 1861, raw_values_count: 2, token_types_start: 1863, token_types_count: 0 }, // [1241]
    SimpleHintData { raw_values_start: 1863, raw_values_count: 3, token_types_start: 1866, token_types_count: 0 }, // [1242]
    SimpleHintData { raw_values_start: 1866, raw_values_count: 1, token_types_start: 1867, token_types_count: 0 }, // [1243]
    SimpleHintData { raw_values_start: 1867, raw_values_count: 1, token_types_start: 1868, token_types_count: 0 }, // [1244]
    SimpleHintData { raw_values_start: 1868, raw_values_count: 1, token_types_start: 1869, token_types_count: 0 }, // [1245]
    SimpleHintData { raw_values_start: 1869, raw_values_count: 1, token_types_start: 1870, token_types_count: 0 }, // [1246]
    SimpleHintData { raw_values_start: 1870, raw_values_count: 1, token_types_start: 1871, token_types_count: 0 }, // [1247]
    SimpleHintData { raw_values_start: 1871, raw_values_count: 1, token_types_start: 1872, token_types_count: 0 }, // [1248]
    SimpleHintData { raw_values_start: 1872, raw_values_count: 1, token_types_start: 1873, token_types_count: 0 }, // [1249]
    SimpleHintData { raw_values_start: 1873, raw_values_count: 1, token_types_start: 1874, token_types_count: 0 }, // [1250]
    SimpleHintData { raw_values_start: 1874, raw_values_count: 1, token_types_start: 1875, token_types_count: 0 }, // [1251]
    SimpleHintData { raw_values_start: 1875, raw_values_count: 2, token_types_start: 1877, token_types_count: 0 }, // [1252]
    SimpleHintData { raw_values_start: 1877, raw_values_count: 1, token_types_start: 1878, token_types_count: 0 }, // [1253]
    SimpleHintData { raw_values_start: 1878, raw_values_count: 2, token_types_start: 1880, token_types_count: 0 }, // [1254]
    SimpleHintData { raw_values_start: 1880, raw_values_count: 6, token_types_start: 1886, token_types_count: 0 }, // [1255]
    SimpleHintData { raw_values_start: 1886, raw_values_count: 1, token_types_start: 1887, token_types_count: 0 }, // [1256]
    SimpleHintData { raw_values_start: 1887, raw_values_count: 1, token_types_start: 1888, token_types_count: 0 }, // [1257]
    SimpleHintData { raw_values_start: 1888, raw_values_count: 1, token_types_start: 1889, token_types_count: 0 }, // [1258]
    SimpleHintData { raw_values_start: 1889, raw_values_count: 1, token_types_start: 1890, token_types_count: 0 }, // [1259]
    SimpleHintData { raw_values_start: 1890, raw_values_count: 1, token_types_start: 1891, token_types_count: 0 }, // [1260]
    SimpleHintData { raw_values_start: 1891, raw_values_count: 1, token_types_start: 1892, token_types_count: 0 }, // [1261]
    SimpleHintData { raw_values_start: 1892, raw_values_count: 1, token_types_start: 1893, token_types_count: 0 }, // [1262]
    SimpleHintData { raw_values_start: 1893, raw_values_count: 1, token_types_start: 1894, token_types_count: 0 }, // [1263]
    SimpleHintData { raw_values_start: 1894, raw_values_count: 1, token_types_start: 1895, token_types_count: 0 }, // [1264]
    SimpleHintData { raw_values_start: 1895, raw_values_count: 1, token_types_start: 1896, token_types_count: 0 }, // [1265]
    SimpleHintData { raw_values_start: 1896, raw_values_count: 1, token_types_start: 1897, token_types_count: 0 }, // [1266]
    SimpleHintData { raw_values_start: 1897, raw_values_count: 1, token_types_start: 1898, token_types_count: 0 }, // [1267]
    SimpleHintData { raw_values_start: 1898, raw_values_count: 1, token_types_start: 1899, token_types_count: 0 }, // [1268]
    SimpleHintData { raw_values_start: 1899, raw_values_count: 1, token_types_start: 1900, token_types_count: 0 }, // [1269]
    SimpleHintData { raw_values_start: 1900, raw_values_count: 1, token_types_start: 1901, token_types_count: 0 }, // [1270]
    SimpleHintData { raw_values_start: 1901, raw_values_count: 1, token_types_start: 1902, token_types_count: 0 }, // [1271]
    SimpleHintData { raw_values_start: 1902, raw_values_count: 1, token_types_start: 1903, token_types_count: 0 }, // [1272]
    SimpleHintData { raw_values_start: 1903, raw_values_count: 1, token_types_start: 1904, token_types_count: 0 }, // [1273]
    SimpleHintData { raw_values_start: 1904, raw_values_count: 4, token_types_start: 1908, token_types_count: 0 }, // [1274]
    SimpleHintData { raw_values_start: 1908, raw_values_count: 1, token_types_start: 1909, token_types_count: 0 }, // [1275]
    SimpleHintData { raw_values_start: 1909, raw_values_count: 3, token_types_start: 1912, token_types_count: 0 }, // [1276]
    SimpleHintData { raw_values_start: 1912, raw_values_count: 1, token_types_start: 1913, token_types_count: 0 }, // [1277]
    SimpleHintData { raw_values_start: 1913, raw_values_count: 1, token_types_start: 1914, token_types_count: 0 }, // [1278]
    SimpleHintData { raw_values_start: 1914, raw_values_count: 1, token_types_start: 1915, token_types_count: 0 }, // [1279]
    SimpleHintData { raw_values_start: 1915, raw_values_count: 1, token_types_start: 1916, token_types_count: 0 }, // [1280]
    SimpleHintData { raw_values_start: 1916, raw_values_count: 1, token_types_start: 1917, token_types_count: 0 }, // [1281]
    SimpleHintData { raw_values_start: 1917, raw_values_count: 1, token_types_start: 1918, token_types_count: 0 }, // [1282]
    SimpleHintData { raw_values_start: 1918, raw_values_count: 1, token_types_start: 1919, token_types_count: 0 }, // [1283]
    SimpleHintData { raw_values_start: 1919, raw_values_count: 1, token_types_start: 1920, token_types_count: 0 }, // [1284]
    SimpleHintData { raw_values_start: 1920, raw_values_count: 1, token_types_start: 1921, token_types_count: 0 }, // [1285]
    SimpleHintData { raw_values_start: 1921, raw_values_count: 1, token_types_start: 1922, token_types_count: 0 }, // [1286]
    SimpleHintData { raw_values_start: 1922, raw_values_count: 1, token_types_start: 1923, token_types_count: 0 }, // [1287]
    SimpleHintData { raw_values_start: 1923, raw_values_count: 1, token_types_start: 1924, token_types_count: 0 }, // [1288]
    SimpleHintData { raw_values_start: 1924, raw_values_count: 1, token_types_start: 1925, token_types_count: 0 }, // [1289]
    SimpleHintData { raw_values_start: 1925, raw_values_count: 1, token_types_start: 1926, token_types_count: 0 }, // [1290]
    SimpleHintData { raw_values_start: 1926, raw_values_count: 1, token_types_start: 1927, token_types_count: 0 }, // [1291]
    SimpleHintData { raw_values_start: 1927, raw_values_count: 1, token_types_start: 1928, token_types_count: 0 }, // [1292]
    SimpleHintData { raw_values_start: 1928, raw_values_count: 1, token_types_start: 1929, token_types_count: 0 }, // [1293]
    SimpleHintData { raw_values_start: 1929, raw_values_count: 1, token_types_start: 1930, token_types_count: 0 }, // [1294]
    SimpleHintData { raw_values_start: 1930, raw_values_count: 1, token_types_start: 1931, token_types_count: 0 }, // [1295]
    SimpleHintData { raw_values_start: 1931, raw_values_count: 1, token_types_start: 1932, token_types_count: 0 }, // [1296]
    SimpleHintData { raw_values_start: 1932, raw_values_count: 1, token_types_start: 1933, token_types_count: 0 }, // [1297]
    SimpleHintData { raw_values_start: 1933, raw_values_count: 1, token_types_start: 1934, token_types_count: 0 }, // [1298]
    SimpleHintData { raw_values_start: 1934, raw_values_count: 3, token_types_start: 1937, token_types_count: 0 }, // [1299]
    SimpleHintData { raw_values_start: 1937, raw_values_count: 1, token_types_start: 1938, token_types_count: 0 }, // [1300]
    SimpleHintData { raw_values_start: 1938, raw_values_count: 1, token_types_start: 1939, token_types_count: 0 }, // [1301]
    SimpleHintData { raw_values_start: 1939, raw_values_count: 1, token_types_start: 1940, token_types_count: 0 }, // [1302]
    SimpleHintData { raw_values_start: 1940, raw_values_count: 1, token_types_start: 1941, token_types_count: 0 }, // [1303]
    SimpleHintData { raw_values_start: 1941, raw_values_count: 2, token_types_start: 1943, token_types_count: 0 }, // [1304]
    SimpleHintData { raw_values_start: 1943, raw_values_count: 7, token_types_start: 1950, token_types_count: 0 }, // [1305]
    SimpleHintData { raw_values_start: 1950, raw_values_count: 3, token_types_start: 1953, token_types_count: 0 }, // [1306]
    SimpleHintData { raw_values_start: 1953, raw_values_count: 6, token_types_start: 1959, token_types_count: 0 }, // [1307]
    SimpleHintData { raw_values_start: 1959, raw_values_count: 1, token_types_start: 1960, token_types_count: 0 }, // [1308]
    SimpleHintData { raw_values_start: 1960, raw_values_count: 1, token_types_start: 1961, token_types_count: 0 }, // [1309]
    SimpleHintData { raw_values_start: 1961, raw_values_count: 1, token_types_start: 1962, token_types_count: 0 }, // [1310]
    SimpleHintData { raw_values_start: 1962, raw_values_count: 1, token_types_start: 1963, token_types_count: 0 }, // [1311]
    SimpleHintData { raw_values_start: 1963, raw_values_count: 1, token_types_start: 1964, token_types_count: 0 }, // [1312]
    SimpleHintData { raw_values_start: 1964, raw_values_count: 1, token_types_start: 1965, token_types_count: 0 }, // [1313]
    SimpleHintData { raw_values_start: 1965, raw_values_count: 1, token_types_start: 1966, token_types_count: 0 }, // [1314]
    SimpleHintData { raw_values_start: 1966, raw_values_count: 1, token_types_start: 1967, token_types_count: 0 }, // [1315]
    SimpleHintData { raw_values_start: 1967, raw_values_count: 1, token_types_start: 1968, token_types_count: 0 }, // [1316]
    SimpleHintData { raw_values_start: 1968, raw_values_count: 1, token_types_start: 1969, token_types_count: 0 }, // [1317]
    SimpleHintData { raw_values_start: 1969, raw_values_count: 1, token_types_start: 1970, token_types_count: 0 }, // [1318]
    SimpleHintData { raw_values_start: 1970, raw_values_count: 1, token_types_start: 1971, token_types_count: 0 }, // [1319]
    SimpleHintData { raw_values_start: 1971, raw_values_count: 1, token_types_start: 1972, token_types_count: 0 }, // [1320]
    SimpleHintData { raw_values_start: 1972, raw_values_count: 1, token_types_start: 1973, token_types_count: 0 }, // [1321]
    SimpleHintData { raw_values_start: 1973, raw_values_count: 1, token_types_start: 1974, token_types_count: 0 }, // [1322]
    SimpleHintData { raw_values_start: 1974, raw_values_count: 1, token_types_start: 1975, token_types_count: 0 }, // [1323]
    SimpleHintData { raw_values_start: 1975, raw_values_count: 2, token_types_start: 1977, token_types_count: 0 }, // [1324]
    SimpleHintData { raw_values_start: 1977, raw_values_count: 1, token_types_start: 1978, token_types_count: 0 }, // [1325]
    SimpleHintData { raw_values_start: 1978, raw_values_count: 1, token_types_start: 1979, token_types_count: 0 }, // [1326]
    SimpleHintData { raw_values_start: 1979, raw_values_count: 1, token_types_start: 1980, token_types_count: 0 }, // [1327]
    SimpleHintData { raw_values_start: 1980, raw_values_count: 1, token_types_start: 1981, token_types_count: 0 }, // [1328]
    SimpleHintData { raw_values_start: 1981, raw_values_count: 1, token_types_start: 1982, token_types_count: 0 }, // [1329]
    SimpleHintData { raw_values_start: 1982, raw_values_count: 1, token_types_start: 1983, token_types_count: 0 }, // [1330]
    SimpleHintData { raw_values_start: 1983, raw_values_count: 1, token_types_start: 1984, token_types_count: 0 }, // [1331]
    SimpleHintData { raw_values_start: 1984, raw_values_count: 1, token_types_start: 1985, token_types_count: 0 }, // [1332]
    SimpleHintData { raw_values_start: 1985, raw_values_count: 1, token_types_start: 1986, token_types_count: 0 }, // [1333]
    SimpleHintData { raw_values_start: 1986, raw_values_count: 1, token_types_start: 1987, token_types_count: 0 }, // [1334]
    SimpleHintData { raw_values_start: 1987, raw_values_count: 1, token_types_start: 1988, token_types_count: 0 }, // [1335]
    SimpleHintData { raw_values_start: 1988, raw_values_count: 1, token_types_start: 1989, token_types_count: 0 }, // [1336]
    SimpleHintData { raw_values_start: 1989, raw_values_count: 1, token_types_start: 1990, token_types_count: 0 }, // [1337]
    SimpleHintData { raw_values_start: 1990, raw_values_count: 1, token_types_start: 1991, token_types_count: 0 }, // [1338]
    SimpleHintData { raw_values_start: 1991, raw_values_count: 1, token_types_start: 1992, token_types_count: 0 }, // [1339]
    SimpleHintData { raw_values_start: 1992, raw_values_count: 1, token_types_start: 1993, token_types_count: 0 }, // [1340]
    SimpleHintData { raw_values_start: 1993, raw_values_count: 1, token_types_start: 1994, token_types_count: 0 }, // [1341]
    SimpleHintData { raw_values_start: 1994, raw_values_count: 1, token_types_start: 1995, token_types_count: 0 }, // [1342]
    SimpleHintData { raw_values_start: 1995, raw_values_count: 1, token_types_start: 1996, token_types_count: 0 }, // [1343]
    SimpleHintData { raw_values_start: 1996, raw_values_count: 1, token_types_start: 1997, token_types_count: 0 }, // [1344]
    SimpleHintData { raw_values_start: 1997, raw_values_count: 1, token_types_start: 1998, token_types_count: 0 }, // [1345]
    SimpleHintData { raw_values_start: 1998, raw_values_count: 1, token_types_start: 1999, token_types_count: 0 }, // [1346]
    SimpleHintData { raw_values_start: 1999, raw_values_count: 1, token_types_start: 2000, token_types_count: 0 }, // [1347]
    SimpleHintData { raw_values_start: 2000, raw_values_count: 1, token_types_start: 2001, token_types_count: 0 }, // [1348]
    SimpleHintData { raw_values_start: 2001, raw_values_count: 1, token_types_start: 2002, token_types_count: 0 }, // [1349]
    SimpleHintData { raw_values_start: 2002, raw_values_count: 1, token_types_start: 2003, token_types_count: 0 }, // [1350]
    SimpleHintData { raw_values_start: 2003, raw_values_count: 1, token_types_start: 2004, token_types_count: 0 }, // [1351]
    SimpleHintData { raw_values_start: 2004, raw_values_count: 1, token_types_start: 2005, token_types_count: 0 }, // [1352]
    SimpleHintData { raw_values_start: 2005, raw_values_count: 1, token_types_start: 2006, token_types_count: 0 }, // [1353]
    SimpleHintData { raw_values_start: 2006, raw_values_count: 1, token_types_start: 2007, token_types_count: 0 }, // [1354]
    SimpleHintData { raw_values_start: 2007, raw_values_count: 1, token_types_start: 2008, token_types_count: 0 }, // [1355]
    SimpleHintData { raw_values_start: 2008, raw_values_count: 1, token_types_start: 2009, token_types_count: 0 }, // [1356]
    SimpleHintData { raw_values_start: 2009, raw_values_count: 2, token_types_start: 2011, token_types_count: 0 }, // [1357]
    SimpleHintData { raw_values_start: 2011, raw_values_count: 1, token_types_start: 2012, token_types_count: 0 }, // [1358]
    SimpleHintData { raw_values_start: 2012, raw_values_count: 1, token_types_start: 2013, token_types_count: 0 }, // [1359]
    SimpleHintData { raw_values_start: 2013, raw_values_count: 1, token_types_start: 2014, token_types_count: 0 }, // [1360]
    SimpleHintData { raw_values_start: 2014, raw_values_count: 1, token_types_start: 2015, token_types_count: 0 }, // [1361]
    SimpleHintData { raw_values_start: 2015, raw_values_count: 1, token_types_start: 2016, token_types_count: 0 }, // [1362]
    SimpleHintData { raw_values_start: 2016, raw_values_count: 1, token_types_start: 2017, token_types_count: 0 }, // [1363]
    SimpleHintData { raw_values_start: 2017, raw_values_count: 1, token_types_start: 2018, token_types_count: 0 }, // [1364]
    SimpleHintData { raw_values_start: 2018, raw_values_count: 1, token_types_start: 2019, token_types_count: 0 }, // [1365]
    SimpleHintData { raw_values_start: 2019, raw_values_count: 1, token_types_start: 2020, token_types_count: 0 }, // [1366]
    SimpleHintData { raw_values_start: 2020, raw_values_count: 1, token_types_start: 2021, token_types_count: 0 }, // [1367]
    SimpleHintData { raw_values_start: 2021, raw_values_count: 1, token_types_start: 2022, token_types_count: 0 }, // [1368]
    SimpleHintData { raw_values_start: 2022, raw_values_count: 1, token_types_start: 2023, token_types_count: 0 }, // [1369]
    SimpleHintData { raw_values_start: 2023, raw_values_count: 1, token_types_start: 2024, token_types_count: 0 }, // [1370]
    SimpleHintData { raw_values_start: 2024, raw_values_count: 1, token_types_start: 2025, token_types_count: 0 }, // [1371]
    SimpleHintData { raw_values_start: 2025, raw_values_count: 1, token_types_start: 2026, token_types_count: 0 }, // [1372]
    SimpleHintData { raw_values_start: 2026, raw_values_count: 1, token_types_start: 2027, token_types_count: 0 }, // [1373]
    SimpleHintData { raw_values_start: 2027, raw_values_count: 1, token_types_start: 2028, token_types_count: 0 }, // [1374]
    SimpleHintData { raw_values_start: 2028, raw_values_count: 11, token_types_start: 2039, token_types_count: 0 }, // [1375]
    SimpleHintData { raw_values_start: 2039, raw_values_count: 3, token_types_start: 2042, token_types_count: 0 }, // [1376]
    SimpleHintData { raw_values_start: 2042, raw_values_count: 1, token_types_start: 2043, token_types_count: 0 }, // [1377]
    SimpleHintData { raw_values_start: 2043, raw_values_count: 1, token_types_start: 2044, token_types_count: 0 }, // [1378]
    SimpleHintData { raw_values_start: 2044, raw_values_count: 1, token_types_start: 2045, token_types_count: 0 }, // [1379]
    SimpleHintData { raw_values_start: 2045, raw_values_count: 1, token_types_start: 2046, token_types_count: 0 }, // [1380]
    SimpleHintData { raw_values_start: 2046, raw_values_count: 1, token_types_start: 2047, token_types_count: 0 }, // [1381]
    SimpleHintData { raw_values_start: 2047, raw_values_count: 1, token_types_start: 2048, token_types_count: 0 }, // [1382]
    SimpleHintData { raw_values_start: 2048, raw_values_count: 1, token_types_start: 2049, token_types_count: 0 }, // [1383]
    SimpleHintData { raw_values_start: 2049, raw_values_count: 2, token_types_start: 2051, token_types_count: 0 }, // [1384]
    SimpleHintData { raw_values_start: 2051, raw_values_count: 1, token_types_start: 2052, token_types_count: 0 }, // [1385]
    SimpleHintData { raw_values_start: 2052, raw_values_count: 1, token_types_start: 2053, token_types_count: 0 }, // [1386]
    SimpleHintData { raw_values_start: 2053, raw_values_count: 1, token_types_start: 2054, token_types_count: 0 }, // [1387]
    SimpleHintData { raw_values_start: 2054, raw_values_count: 1, token_types_start: 2055, token_types_count: 0 }, // [1388]
    SimpleHintData { raw_values_start: 2055, raw_values_count: 1, token_types_start: 2056, token_types_count: 0 }, // [1389]
    SimpleHintData { raw_values_start: 2056, raw_values_count: 1, token_types_start: 2057, token_types_count: 0 }, // [1390]
    SimpleHintData { raw_values_start: 2057, raw_values_count: 1, token_types_start: 2058, token_types_count: 0 }, // [1391]
    SimpleHintData { raw_values_start: 2058, raw_values_count: 1, token_types_start: 2059, token_types_count: 0 }, // [1392]
    SimpleHintData { raw_values_start: 2059, raw_values_count: 2, token_types_start: 2061, token_types_count: 0 }, // [1393]
    SimpleHintData { raw_values_start: 2061, raw_values_count: 5, token_types_start: 2066, token_types_count: 0 }, // [1394]
    SimpleHintData { raw_values_start: 2066, raw_values_count: 2, token_types_start: 2068, token_types_count: 0 }, // [1395]
    SimpleHintData { raw_values_start: 2068, raw_values_count: 1, token_types_start: 2069, token_types_count: 0 }, // [1396]
    SimpleHintData { raw_values_start: 2069, raw_values_count: 2, token_types_start: 2071, token_types_count: 0 }, // [1397]
    SimpleHintData { raw_values_start: 2071, raw_values_count: 2, token_types_start: 2073, token_types_count: 0 }, // [1398]
    SimpleHintData { raw_values_start: 2073, raw_values_count: 1, token_types_start: 2074, token_types_count: 0 }, // [1399]
    SimpleHintData { raw_values_start: 2074, raw_values_count: 2, token_types_start: 2076, token_types_count: 0 }, // [1400]
    SimpleHintData { raw_values_start: 2076, raw_values_count: 3, token_types_start: 2079, token_types_count: 0 }, // [1401]
    SimpleHintData { raw_values_start: 2079, raw_values_count: 2, token_types_start: 2081, token_types_count: 0 }, // [1402]
    SimpleHintData { raw_values_start: 2081, raw_values_count: 1, token_types_start: 2082, token_types_count: 0 }, // [1403]
    SimpleHintData { raw_values_start: 2082, raw_values_count: 2, token_types_start: 2084, token_types_count: 0 }, // [1404]
    SimpleHintData { raw_values_start: 2084, raw_values_count: 1, token_types_start: 2085, token_types_count: 0 }, // [1405]
    SimpleHintData { raw_values_start: 2085, raw_values_count: 1, token_types_start: 2086, token_types_count: 0 }, // [1406]
    SimpleHintData { raw_values_start: 2086, raw_values_count: 1, token_types_start: 2087, token_types_count: 0 }, // [1407]
    SimpleHintData { raw_values_start: 2087, raw_values_count: 1, token_types_start: 2088, token_types_count: 0 }, // [1408]
    SimpleHintData { raw_values_start: 2088, raw_values_count: 1, token_types_start: 2089, token_types_count: 0 }, // [1409]
    SimpleHintData { raw_values_start: 2089, raw_values_count: 1, token_types_start: 2090, token_types_count: 0 }, // [1410]
    SimpleHintData { raw_values_start: 2090, raw_values_count: 1, token_types_start: 2091, token_types_count: 0 }, // [1411]
    SimpleHintData { raw_values_start: 2091, raw_values_count: 1, token_types_start: 2092, token_types_count: 0 }, // [1412]
    SimpleHintData { raw_values_start: 2092, raw_values_count: 1, token_types_start: 2093, token_types_count: 0 }, // [1413]
    SimpleHintData { raw_values_start: 2093, raw_values_count: 1, token_types_start: 2094, token_types_count: 0 }, // [1414]
    SimpleHintData { raw_values_start: 2094, raw_values_count: 1, token_types_start: 2095, token_types_count: 0 }, // [1415]
    SimpleHintData { raw_values_start: 2095, raw_values_count: 1, token_types_start: 2096, token_types_count: 0 }, // [1416]
    SimpleHintData { raw_values_start: 2096, raw_values_count: 1, token_types_start: 2097, token_types_count: 0 }, // [1417]
    SimpleHintData { raw_values_start: 2097, raw_values_count: 1, token_types_start: 2098, token_types_count: 0 }, // [1418]
    SimpleHintData { raw_values_start: 2098, raw_values_count: 1, token_types_start: 2099, token_types_count: 0 }, // [1419]
    SimpleHintData { raw_values_start: 2099, raw_values_count: 1, token_types_start: 2100, token_types_count: 0 }, // [1420]
    SimpleHintData { raw_values_start: 2100, raw_values_count: 1, token_types_start: 2101, token_types_count: 0 }, // [1421]
    SimpleHintData { raw_values_start: 2101, raw_values_count: 1, token_types_start: 2102, token_types_count: 0 }, // [1422]
    SimpleHintData { raw_values_start: 2102, raw_values_count: 1, token_types_start: 2103, token_types_count: 0 }, // [1423]
    SimpleHintData { raw_values_start: 2103, raw_values_count: 1, token_types_start: 2104, token_types_count: 0 }, // [1424]
    SimpleHintData { raw_values_start: 2104, raw_values_count: 1, token_types_start: 2105, token_types_count: 0 }, // [1425]
    SimpleHintData { raw_values_start: 2105, raw_values_count: 1, token_types_start: 2106, token_types_count: 0 }, // [1426]
    SimpleHintData { raw_values_start: 2106, raw_values_count: 1, token_types_start: 2107, token_types_count: 0 }, // [1427]
    SimpleHintData { raw_values_start: 2107, raw_values_count: 1, token_types_start: 2108, token_types_count: 0 }, // [1428]
    SimpleHintData { raw_values_start: 2108, raw_values_count: 1, token_types_start: 2109, token_types_count: 0 }, // [1429]
    SimpleHintData { raw_values_start: 2109, raw_values_count: 1, token_types_start: 2110, token_types_count: 0 }, // [1430]
    SimpleHintData { raw_values_start: 2110, raw_values_count: 1, token_types_start: 2111, token_types_count: 0 }, // [1431]
    SimpleHintData { raw_values_start: 2111, raw_values_count: 1, token_types_start: 2112, token_types_count: 0 }, // [1432]
    SimpleHintData { raw_values_start: 2112, raw_values_count: 1, token_types_start: 2113, token_types_count: 0 }, // [1433]
    SimpleHintData { raw_values_start: 2113, raw_values_count: 1, token_types_start: 2114, token_types_count: 0 }, // [1434]
    SimpleHintData { raw_values_start: 2114, raw_values_count: 1, token_types_start: 2115, token_types_count: 0 }, // [1435]
    SimpleHintData { raw_values_start: 2115, raw_values_count: 1, token_types_start: 2116, token_types_count: 0 }, // [1436]
    SimpleHintData { raw_values_start: 2116, raw_values_count: 1, token_types_start: 2117, token_types_count: 0 }, // [1437]
    SimpleHintData { raw_values_start: 2117, raw_values_count: 1, token_types_start: 2118, token_types_count: 0 }, // [1438]
    SimpleHintData { raw_values_start: 2118, raw_values_count: 1, token_types_start: 2119, token_types_count: 0 }, // [1439]
    SimpleHintData { raw_values_start: 2119, raw_values_count: 1, token_types_start: 2120, token_types_count: 0 }, // [1440]
    SimpleHintData { raw_values_start: 2120, raw_values_count: 1, token_types_start: 2121, token_types_count: 0 }, // [1441]
    SimpleHintData { raw_values_start: 2121, raw_values_count: 1, token_types_start: 2122, token_types_count: 0 }, // [1442]
    SimpleHintData { raw_values_start: 2122, raw_values_count: 1, token_types_start: 2123, token_types_count: 0 }, // [1443]
    SimpleHintData { raw_values_start: 2123, raw_values_count: 1, token_types_start: 2124, token_types_count: 0 }, // [1444]
    SimpleHintData { raw_values_start: 2124, raw_values_count: 1, token_types_start: 2125, token_types_count: 0 }, // [1445]
    SimpleHintData { raw_values_start: 2125, raw_values_count: 1, token_types_start: 2126, token_types_count: 0 }, // [1446]
    SimpleHintData { raw_values_start: 2126, raw_values_count: 1, token_types_start: 2127, token_types_count: 0 }, // [1447]
    SimpleHintData { raw_values_start: 2127, raw_values_count: 1, token_types_start: 2128, token_types_count: 0 }, // [1448]
    SimpleHintData { raw_values_start: 2128, raw_values_count: 1, token_types_start: 2129, token_types_count: 0 }, // [1449]
    SimpleHintData { raw_values_start: 2129, raw_values_count: 1, token_types_start: 2130, token_types_count: 0 }, // [1450]
    SimpleHintData { raw_values_start: 2130, raw_values_count: 1, token_types_start: 2131, token_types_count: 0 }, // [1451]
    SimpleHintData { raw_values_start: 2131, raw_values_count: 1, token_types_start: 2132, token_types_count: 0 }, // [1452]
    SimpleHintData { raw_values_start: 2132, raw_values_count: 1, token_types_start: 2133, token_types_count: 0 }, // [1453]
    SimpleHintData { raw_values_start: 2133, raw_values_count: 1, token_types_start: 2134, token_types_count: 0 }, // [1454]
    SimpleHintData { raw_values_start: 2134, raw_values_count: 1, token_types_start: 2135, token_types_count: 0 }, // [1455]
    SimpleHintData { raw_values_start: 2135, raw_values_count: 1, token_types_start: 2136, token_types_count: 0 }, // [1456]
    SimpleHintData { raw_values_start: 2136, raw_values_count: 1, token_types_start: 2137, token_types_count: 0 }, // [1457]
    SimpleHintData { raw_values_start: 2137, raw_values_count: 1, token_types_start: 2138, token_types_count: 0 }, // [1458]
    SimpleHintData { raw_values_start: 2138, raw_values_count: 1, token_types_start: 2139, token_types_count: 0 }, // [1459]
    SimpleHintData { raw_values_start: 2139, raw_values_count: 1, token_types_start: 2140, token_types_count: 0 }, // [1460]
    SimpleHintData { raw_values_start: 2140, raw_values_count: 1, token_types_start: 2141, token_types_count: 0 }, // [1461]
    SimpleHintData { raw_values_start: 2141, raw_values_count: 1, token_types_start: 2142, token_types_count: 0 }, // [1462]
    SimpleHintData { raw_values_start: 2142, raw_values_count: 1, token_types_start: 2143, token_types_count: 0 }, // [1463]
    SimpleHintData { raw_values_start: 2143, raw_values_count: 1, token_types_start: 2144, token_types_count: 0 }, // [1464]
    SimpleHintData { raw_values_start: 2144, raw_values_count: 1, token_types_start: 2145, token_types_count: 0 }, // [1465]
    SimpleHintData { raw_values_start: 2145, raw_values_count: 1, token_types_start: 2146, token_types_count: 0 }, // [1466]
    SimpleHintData { raw_values_start: 2146, raw_values_count: 1, token_types_start: 2147, token_types_count: 0 }, // [1467]
    SimpleHintData { raw_values_start: 2147, raw_values_count: 1, token_types_start: 2148, token_types_count: 0 }, // [1468]
    SimpleHintData { raw_values_start: 2148, raw_values_count: 1, token_types_start: 2149, token_types_count: 0 }, // [1469]
    SimpleHintData { raw_values_start: 2149, raw_values_count: 1, token_types_start: 2150, token_types_count: 0 }, // [1470]
    SimpleHintData { raw_values_start: 2150, raw_values_count: 1, token_types_start: 2151, token_types_count: 0 }, // [1471]
    SimpleHintData { raw_values_start: 2151, raw_values_count: 1, token_types_start: 2152, token_types_count: 0 }, // [1472]
    SimpleHintData { raw_values_start: 2152, raw_values_count: 1, token_types_start: 2153, token_types_count: 0 }, // [1473]
    SimpleHintData { raw_values_start: 2153, raw_values_count: 1, token_types_start: 2154, token_types_count: 0 }, // [1474]
    SimpleHintData { raw_values_start: 2154, raw_values_count: 1, token_types_start: 2155, token_types_count: 0 }, // [1475]
    SimpleHintData { raw_values_start: 2155, raw_values_count: 1, token_types_start: 2156, token_types_count: 0 }, // [1476]
    SimpleHintData { raw_values_start: 2156, raw_values_count: 1, token_types_start: 2157, token_types_count: 0 }, // [1477]
    SimpleHintData { raw_values_start: 2157, raw_values_count: 1, token_types_start: 2158, token_types_count: 0 }, // [1478]
    SimpleHintData { raw_values_start: 2158, raw_values_count: 1, token_types_start: 2159, token_types_count: 0 }, // [1479]
    SimpleHintData { raw_values_start: 2159, raw_values_count: 1, token_types_start: 2160, token_types_count: 0 }, // [1480]
    SimpleHintData { raw_values_start: 2160, raw_values_count: 1, token_types_start: 2161, token_types_count: 0 }, // [1481]
    SimpleHintData { raw_values_start: 2161, raw_values_count: 1, token_types_start: 2162, token_types_count: 0 }, // [1482]
    SimpleHintData { raw_values_start: 2162, raw_values_count: 1, token_types_start: 2163, token_types_count: 0 }, // [1483]
    SimpleHintData { raw_values_start: 2163, raw_values_count: 1, token_types_start: 2164, token_types_count: 0 }, // [1484]
    SimpleHintData { raw_values_start: 2164, raw_values_count: 1, token_types_start: 2165, token_types_count: 0 }, // [1485]
    SimpleHintData { raw_values_start: 2165, raw_values_count: 1, token_types_start: 2166, token_types_count: 0 }, // [1486]
    SimpleHintData { raw_values_start: 2166, raw_values_count: 1, token_types_start: 2167, token_types_count: 0 }, // [1487]
    SimpleHintData { raw_values_start: 2167, raw_values_count: 1, token_types_start: 2168, token_types_count: 0 }, // [1488]
    SimpleHintData { raw_values_start: 2168, raw_values_count: 1, token_types_start: 2169, token_types_count: 0 }, // [1489]
    SimpleHintData { raw_values_start: 2169, raw_values_count: 1, token_types_start: 2170, token_types_count: 0 }, // [1490]
    SimpleHintData { raw_values_start: 2170, raw_values_count: 1, token_types_start: 2171, token_types_count: 0 }, // [1491]
    SimpleHintData { raw_values_start: 2171, raw_values_count: 1, token_types_start: 2172, token_types_count: 0 }, // [1492]
    SimpleHintData { raw_values_start: 2172, raw_values_count: 1, token_types_start: 2173, token_types_count: 0 }, // [1493]
    SimpleHintData { raw_values_start: 2173, raw_values_count: 1, token_types_start: 2174, token_types_count: 0 }, // [1494]
    SimpleHintData { raw_values_start: 2174, raw_values_count: 1, token_types_start: 2175, token_types_count: 0 }, // [1495]
    SimpleHintData { raw_values_start: 2175, raw_values_count: 1, token_types_start: 2176, token_types_count: 0 }, // [1496]
    SimpleHintData { raw_values_start: 2176, raw_values_count: 1, token_types_start: 2177, token_types_count: 0 }, // [1497]
    SimpleHintData { raw_values_start: 2177, raw_values_count: 1, token_types_start: 2178, token_types_count: 0 }, // [1498]
    SimpleHintData { raw_values_start: 2178, raw_values_count: 1, token_types_start: 2179, token_types_count: 0 }, // [1499]
    SimpleHintData { raw_values_start: 2179, raw_values_count: 1, token_types_start: 2180, token_types_count: 0 }, // [1500]
    SimpleHintData { raw_values_start: 2180, raw_values_count: 1, token_types_start: 2181, token_types_count: 0 }, // [1501]
    SimpleHintData { raw_values_start: 2181, raw_values_count: 1, token_types_start: 2182, token_types_count: 0 }, // [1502]
    SimpleHintData { raw_values_start: 2182, raw_values_count: 1, token_types_start: 2183, token_types_count: 0 }, // [1503]
    SimpleHintData { raw_values_start: 2183, raw_values_count: 1, token_types_start: 2184, token_types_count: 0 }, // [1504]
    SimpleHintData { raw_values_start: 2184, raw_values_count: 1, token_types_start: 2185, token_types_count: 0 }, // [1505]
    SimpleHintData { raw_values_start: 2185, raw_values_count: 1, token_types_start: 2186, token_types_count: 0 }, // [1506]
    SimpleHintData { raw_values_start: 2186, raw_values_count: 1, token_types_start: 2187, token_types_count: 0 }, // [1507]
    SimpleHintData { raw_values_start: 2187, raw_values_count: 1, token_types_start: 2188, token_types_count: 0 }, // [1508]
    SimpleHintData { raw_values_start: 2188, raw_values_count: 1, token_types_start: 2189, token_types_count: 0 }, // [1509]
    SimpleHintData { raw_values_start: 2189, raw_values_count: 1, token_types_start: 2190, token_types_count: 0 }, // [1510]
    SimpleHintData { raw_values_start: 2190, raw_values_count: 1, token_types_start: 2191, token_types_count: 0 }, // [1511]
    SimpleHintData { raw_values_start: 2191, raw_values_count: 1, token_types_start: 2192, token_types_count: 0 }, // [1512]
    SimpleHintData { raw_values_start: 2192, raw_values_count: 1, token_types_start: 2193, token_types_count: 0 }, // [1513]
    SimpleHintData { raw_values_start: 2193, raw_values_count: 5, token_types_start: 2198, token_types_count: 0 }, // [1514]
    SimpleHintData { raw_values_start: 2198, raw_values_count: 4, token_types_start: 2202, token_types_count: 0 }, // [1515]
    SimpleHintData { raw_values_start: 2202, raw_values_count: 2, token_types_start: 2204, token_types_count: 0 }, // [1516]
    SimpleHintData { raw_values_start: 2204, raw_values_count: 2, token_types_start: 2206, token_types_count: 0 }, // [1517]
    SimpleHintData { raw_values_start: 2206, raw_values_count: 3, token_types_start: 2209, token_types_count: 0 }, // [1518]
    SimpleHintData { raw_values_start: 2209, raw_values_count: 6, token_types_start: 2215, token_types_count: 0 }, // [1519]
    SimpleHintData { raw_values_start: 2215, raw_values_count: 2, token_types_start: 2217, token_types_count: 0 }, // [1520]
    SimpleHintData { raw_values_start: 2217, raw_values_count: 1, token_types_start: 2218, token_types_count: 0 }, // [1521]
    SimpleHintData { raw_values_start: 2218, raw_values_count: 1, token_types_start: 2219, token_types_count: 0 }, // [1522]
    SimpleHintData { raw_values_start: 2219, raw_values_count: 1, token_types_start: 2220, token_types_count: 0 }, // [1523]
    SimpleHintData { raw_values_start: 2220, raw_values_count: 1, token_types_start: 2221, token_types_count: 0 }, // [1524]
    SimpleHintData { raw_values_start: 2221, raw_values_count: 1, token_types_start: 2222, token_types_count: 0 }, // [1525]
    SimpleHintData { raw_values_start: 2222, raw_values_count: 2, token_types_start: 2224, token_types_count: 0 }, // [1526]
    SimpleHintData { raw_values_start: 2224, raw_values_count: 3, token_types_start: 2227, token_types_count: 0 }, // [1527]
    SimpleHintData { raw_values_start: 2227, raw_values_count: 1, token_types_start: 2228, token_types_count: 0 }, // [1528]
    SimpleHintData { raw_values_start: 2228, raw_values_count: 1, token_types_start: 2229, token_types_count: 0 }, // [1529]
    SimpleHintData { raw_values_start: 2229, raw_values_count: 1, token_types_start: 2230, token_types_count: 0 }, // [1530]
    SimpleHintData { raw_values_start: 2230, raw_values_count: 1, token_types_start: 2231, token_types_count: 0 }, // [1531]
    SimpleHintData { raw_values_start: 2231, raw_values_count: 1, token_types_start: 2232, token_types_count: 0 }, // [1532]
    SimpleHintData { raw_values_start: 2232, raw_values_count: 1, token_types_start: 2233, token_types_count: 0 }, // [1533]
    SimpleHintData { raw_values_start: 2233, raw_values_count: 1, token_types_start: 2234, token_types_count: 0 }, // [1534]
    SimpleHintData { raw_values_start: 2234, raw_values_count: 1, token_types_start: 2235, token_types_count: 0 }, // [1535]
    SimpleHintData { raw_values_start: 2235, raw_values_count: 1, token_types_start: 2236, token_types_count: 0 }, // [1536]
    SimpleHintData { raw_values_start: 2236, raw_values_count: 1, token_types_start: 2237, token_types_count: 0 }, // [1537]
    SimpleHintData { raw_values_start: 2237, raw_values_count: 1, token_types_start: 2238, token_types_count: 0 }, // [1538]
    SimpleHintData { raw_values_start: 2238, raw_values_count: 1, token_types_start: 2239, token_types_count: 0 }, // [1539]
    SimpleHintData { raw_values_start: 2239, raw_values_count: 1, token_types_start: 2240, token_types_count: 0 }, // [1540]
    SimpleHintData { raw_values_start: 2240, raw_values_count: 1, token_types_start: 2241, token_types_count: 0 }, // [1541]
    SimpleHintData { raw_values_start: 2241, raw_values_count: 1, token_types_start: 2242, token_types_count: 0 }, // [1542]
    SimpleHintData { raw_values_start: 2242, raw_values_count: 1, token_types_start: 2243, token_types_count: 0 }, // [1543]
    SimpleHintData { raw_values_start: 2243, raw_values_count: 1, token_types_start: 2244, token_types_count: 0 }, // [1544]
    SimpleHintData { raw_values_start: 2244, raw_values_count: 1, token_types_start: 2245, token_types_count: 0 }, // [1545]
    SimpleHintData { raw_values_start: 2245, raw_values_count: 1, token_types_start: 2246, token_types_count: 0 }, // [1546]
    SimpleHintData { raw_values_start: 2246, raw_values_count: 1, token_types_start: 2247, token_types_count: 0 }, // [1547]
    SimpleHintData { raw_values_start: 2247, raw_values_count: 1, token_types_start: 2248, token_types_count: 0 }, // [1548]
    SimpleHintData { raw_values_start: 2248, raw_values_count: 1, token_types_start: 2249, token_types_count: 0 }, // [1549]
    SimpleHintData { raw_values_start: 2249, raw_values_count: 1, token_types_start: 2250, token_types_count: 0 }, // [1550]
    SimpleHintData { raw_values_start: 2250, raw_values_count: 1, token_types_start: 2251, token_types_count: 0 }, // [1551]
    SimpleHintData { raw_values_start: 2251, raw_values_count: 1, token_types_start: 2252, token_types_count: 0 }, // [1552]
    SimpleHintData { raw_values_start: 2252, raw_values_count: 1, token_types_start: 2253, token_types_count: 0 }, // [1553]
    SimpleHintData { raw_values_start: 2253, raw_values_count: 1, token_types_start: 2254, token_types_count: 0 }, // [1554]
    SimpleHintData { raw_values_start: 2254, raw_values_count: 1, token_types_start: 2255, token_types_count: 0 }, // [1555]
    SimpleHintData { raw_values_start: 2255, raw_values_count: 1, token_types_start: 2256, token_types_count: 0 }, // [1556]
    SimpleHintData { raw_values_start: 2256, raw_values_count: 1, token_types_start: 2257, token_types_count: 0 }, // [1557]
    SimpleHintData { raw_values_start: 2257, raw_values_count: 1, token_types_start: 2258, token_types_count: 0 }, // [1558]
    SimpleHintData { raw_values_start: 2258, raw_values_count: 1, token_types_start: 2259, token_types_count: 0 }, // [1559]
    SimpleHintData { raw_values_start: 2259, raw_values_count: 1, token_types_start: 2260, token_types_count: 0 }, // [1560]
    SimpleHintData { raw_values_start: 2260, raw_values_count: 1, token_types_start: 2261, token_types_count: 0 }, // [1561]
    SimpleHintData { raw_values_start: 2261, raw_values_count: 1, token_types_start: 2262, token_types_count: 0 }, // [1562]
    SimpleHintData { raw_values_start: 2262, raw_values_count: 1, token_types_start: 2263, token_types_count: 0 }, // [1563]
    SimpleHintData { raw_values_start: 2263, raw_values_count: 1, token_types_start: 2264, token_types_count: 0 }, // [1564]
    SimpleHintData { raw_values_start: 2264, raw_values_count: 1, token_types_start: 2265, token_types_count: 0 }, // [1565]
    SimpleHintData { raw_values_start: 2265, raw_values_count: 1, token_types_start: 2266, token_types_count: 0 }, // [1566]
    SimpleHintData { raw_values_start: 2266, raw_values_count: 1, token_types_start: 2267, token_types_count: 0 }, // [1567]
    SimpleHintData { raw_values_start: 2267, raw_values_count: 1, token_types_start: 2268, token_types_count: 0 }, // [1568]
    SimpleHintData { raw_values_start: 2268, raw_values_count: 1, token_types_start: 2269, token_types_count: 0 }, // [1569]
    SimpleHintData { raw_values_start: 2269, raw_values_count: 1, token_types_start: 2270, token_types_count: 0 }, // [1570]
    SimpleHintData { raw_values_start: 2270, raw_values_count: 1, token_types_start: 2271, token_types_count: 0 }, // [1571]
    SimpleHintData { raw_values_start: 2271, raw_values_count: 1, token_types_start: 2272, token_types_count: 0 }, // [1572]
    SimpleHintData { raw_values_start: 2272, raw_values_count: 1, token_types_start: 2273, token_types_count: 0 }, // [1573]
    SimpleHintData { raw_values_start: 2273, raw_values_count: 1, token_types_start: 2274, token_types_count: 0 }, // [1574]
    SimpleHintData { raw_values_start: 2274, raw_values_count: 1, token_types_start: 2275, token_types_count: 0 }, // [1575]
    SimpleHintData { raw_values_start: 2275, raw_values_count: 1, token_types_start: 2276, token_types_count: 0 }, // [1576]
    SimpleHintData { raw_values_start: 2276, raw_values_count: 1, token_types_start: 2277, token_types_count: 0 }, // [1577]
    SimpleHintData { raw_values_start: 2277, raw_values_count: 2, token_types_start: 2279, token_types_count: 0 }, // [1578]
    SimpleHintData { raw_values_start: 2279, raw_values_count: 1, token_types_start: 2280, token_types_count: 0 }, // [1579]
    SimpleHintData { raw_values_start: 2280, raw_values_count: 1, token_types_start: 2281, token_types_count: 0 }, // [1580]
    SimpleHintData { raw_values_start: 2281, raw_values_count: 1, token_types_start: 2282, token_types_count: 0 }, // [1581]
    SimpleHintData { raw_values_start: 2282, raw_values_count: 1, token_types_start: 2283, token_types_count: 0 }, // [1582]
    SimpleHintData { raw_values_start: 2283, raw_values_count: 1, token_types_start: 2284, token_types_count: 0 }, // [1583]
    SimpleHintData { raw_values_start: 2284, raw_values_count: 1, token_types_start: 2285, token_types_count: 0 }, // [1584]
    SimpleHintData { raw_values_start: 2285, raw_values_count: 1, token_types_start: 2286, token_types_count: 0 }, // [1585]
    SimpleHintData { raw_values_start: 2286, raw_values_count: 1, token_types_start: 2287, token_types_count: 0 }, // [1586]
    SimpleHintData { raw_values_start: 2287, raw_values_count: 1, token_types_start: 2288, token_types_count: 0 }, // [1587]
    SimpleHintData { raw_values_start: 2288, raw_values_count: 1, token_types_start: 2289, token_types_count: 0 }, // [1588]
    SimpleHintData { raw_values_start: 2289, raw_values_count: 1, token_types_start: 2290, token_types_count: 0 }, // [1589]
    SimpleHintData { raw_values_start: 2290, raw_values_count: 1, token_types_start: 2291, token_types_count: 0 }, // [1590]
    SimpleHintData { raw_values_start: 2291, raw_values_count: 1, token_types_start: 2292, token_types_count: 0 }, // [1591]
    SimpleHintData { raw_values_start: 2292, raw_values_count: 1, token_types_start: 2293, token_types_count: 0 }, // [1592]
    SimpleHintData { raw_values_start: 2293, raw_values_count: 1, token_types_start: 2294, token_types_count: 0 }, // [1593]
    SimpleHintData { raw_values_start: 2294, raw_values_count: 1, token_types_start: 2295, token_types_count: 0 }, // [1594]
    SimpleHintData { raw_values_start: 2295, raw_values_count: 1, token_types_start: 2296, token_types_count: 0 }, // [1595]
    SimpleHintData { raw_values_start: 2296, raw_values_count: 1, token_types_start: 2297, token_types_count: 0 }, // [1596]
    SimpleHintData { raw_values_start: 2297, raw_values_count: 1, token_types_start: 2298, token_types_count: 0 }, // [1597]
    SimpleHintData { raw_values_start: 2298, raw_values_count: 1, token_types_start: 2299, token_types_count: 0 }, // [1598]
    SimpleHintData { raw_values_start: 2299, raw_values_count: 1, token_types_start: 2300, token_types_count: 0 }, // [1599]
    SimpleHintData { raw_values_start: 2300, raw_values_count: 1, token_types_start: 2301, token_types_count: 0 }, // [1600]
    SimpleHintData { raw_values_start: 2301, raw_values_count: 2, token_types_start: 2303, token_types_count: 0 }, // [1601]
    SimpleHintData { raw_values_start: 2303, raw_values_count: 1, token_types_start: 2304, token_types_count: 0 }, // [1602]
    SimpleHintData { raw_values_start: 2304, raw_values_count: 1, token_types_start: 2305, token_types_count: 0 }, // [1603]
    SimpleHintData { raw_values_start: 2305, raw_values_count: 1, token_types_start: 2306, token_types_count: 0 }, // [1604]
    SimpleHintData { raw_values_start: 2306, raw_values_count: 1, token_types_start: 2307, token_types_count: 0 }, // [1605]
    SimpleHintData { raw_values_start: 2307, raw_values_count: 1, token_types_start: 2308, token_types_count: 0 }, // [1606]
    SimpleHintData { raw_values_start: 2308, raw_values_count: 1, token_types_start: 2309, token_types_count: 0 }, // [1607]
    SimpleHintData { raw_values_start: 2309, raw_values_count: 1, token_types_start: 2310, token_types_count: 0 }, // [1608]
    SimpleHintData { raw_values_start: 2310, raw_values_count: 1, token_types_start: 2311, token_types_count: 0 }, // [1609]
    SimpleHintData { raw_values_start: 2311, raw_values_count: 1, token_types_start: 2312, token_types_count: 0 }, // [1610]
    SimpleHintData { raw_values_start: 2312, raw_values_count: 1, token_types_start: 2313, token_types_count: 0 }, // [1611]
    SimpleHintData { raw_values_start: 2313, raw_values_count: 1, token_types_start: 2314, token_types_count: 0 }, // [1612]
    SimpleHintData { raw_values_start: 2314, raw_values_count: 1, token_types_start: 2315, token_types_count: 0 }, // [1613]
    SimpleHintData { raw_values_start: 2315, raw_values_count: 1, token_types_start: 2316, token_types_count: 0 }, // [1614]
    SimpleHintData { raw_values_start: 2316, raw_values_count: 1, token_types_start: 2317, token_types_count: 0 }, // [1615]
    SimpleHintData { raw_values_start: 2317, raw_values_count: 1, token_types_start: 2318, token_types_count: 0 }, // [1616]
    SimpleHintData { raw_values_start: 2318, raw_values_count: 1, token_types_start: 2319, token_types_count: 0 }, // [1617]
    SimpleHintData { raw_values_start: 2319, raw_values_count: 1, token_types_start: 2320, token_types_count: 0 }, // [1618]
    SimpleHintData { raw_values_start: 2320, raw_values_count: 1, token_types_start: 2321, token_types_count: 0 }, // [1619]
    SimpleHintData { raw_values_start: 2321, raw_values_count: 1, token_types_start: 2322, token_types_count: 0 }, // [1620]
    SimpleHintData { raw_values_start: 2322, raw_values_count: 1, token_types_start: 2323, token_types_count: 0 }, // [1621]
    SimpleHintData { raw_values_start: 2323, raw_values_count: 1, token_types_start: 2324, token_types_count: 0 }, // [1622]
    SimpleHintData { raw_values_start: 2324, raw_values_count: 1, token_types_start: 2325, token_types_count: 0 }, // [1623]
    SimpleHintData { raw_values_start: 2325, raw_values_count: 1, token_types_start: 2326, token_types_count: 0 }, // [1624]
    SimpleHintData { raw_values_start: 2326, raw_values_count: 1, token_types_start: 2327, token_types_count: 0 }, // [1625]
    SimpleHintData { raw_values_start: 2327, raw_values_count: 9, token_types_start: 2336, token_types_count: 0 }, // [1626]
    SimpleHintData { raw_values_start: 2336, raw_values_count: 1, token_types_start: 2337, token_types_count: 0 }, // [1627]
    SimpleHintData { raw_values_start: 2337, raw_values_count: 1, token_types_start: 2338, token_types_count: 0 }, // [1628]
    SimpleHintData { raw_values_start: 2338, raw_values_count: 1, token_types_start: 2339, token_types_count: 0 }, // [1629]
    SimpleHintData { raw_values_start: 2339, raw_values_count: 2, token_types_start: 2341, token_types_count: 0 }, // [1630]
    SimpleHintData { raw_values_start: 2341, raw_values_count: 2, token_types_start: 2343, token_types_count: 0 }, // [1631]
    SimpleHintData { raw_values_start: 2343, raw_values_count: 1, token_types_start: 2344, token_types_count: 0 }, // [1632]
    SimpleHintData { raw_values_start: 2344, raw_values_count: 1, token_types_start: 2345, token_types_count: 0 }, // [1633]
    SimpleHintData { raw_values_start: 2345, raw_values_count: 1, token_types_start: 2346, token_types_count: 0 }, // [1634]
    SimpleHintData { raw_values_start: 2346, raw_values_count: 1, token_types_start: 2347, token_types_count: 0 }, // [1635]
    SimpleHintData { raw_values_start: 2347, raw_values_count: 1, token_types_start: 2348, token_types_count: 0 }, // [1636]
    SimpleHintData { raw_values_start: 2348, raw_values_count: 1, token_types_start: 2349, token_types_count: 0 }, // [1637]
    SimpleHintData { raw_values_start: 2349, raw_values_count: 1, token_types_start: 2350, token_types_count: 0 }, // [1638]
    SimpleHintData { raw_values_start: 2350, raw_values_count: 1, token_types_start: 2351, token_types_count: 0 }, // [1639]
];

pub static SIMPLE_HINT_INDICES: &[u32] = &[
    1, 2, 3, 4, 0, 5, 0, 6, 7, 7, 8, 9, 10, 11, 12, 13,
    14, 15, 16, 17, 18, 19, 19, 20, 21, 22, 22, 23, 24, 24, 25, 26,
    10, 27, 27, 25, 26, 10, 28, 24, 24, 29, 26, 14, 27, 27, 29, 26,
    30, 10, 14, 0, 0, 0, 0, 0, 31, 0, 32, 33, 34, 35, 35, 36,
    37, 38, 39, 39, 40, 16, 41, 9, 10, 12, 42, 42, 43, 44, 45, 45,
    46, 47, 47, 48, 8, 8, 49, 50, 51, 52, 52, 39, 39, 53, 14, 43,
    43, 44, 54, 28, 55, 55, 56, 14, 15, 57, 57, 58, 42, 59, 39, 60,
    45, 61, 62, 8, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 51,
    74, 75, 24, 24, 58, 31, 38, 38, 76, 28, 77, 78, 79, 80, 81, 82,
    83, 84, 46, 14, 43, 43, 44, 85, 85, 78, 86, 86, 77, 87, 87, 88,
    29, 89, 90, 91, 92, 93, 94, 95, 96, 97, 25, 43, 43, 0, 85, 85,
    90, 86, 86, 89, 87, 87, 0, 98, 99, 100, 0, 101, 101, 0, 0, 41,
    41, 0, 0, 16, 16, 0, 0, 102, 102, 0, 0, 0, 0, 0, 0, 103,
    104, 104, 5, 105, 106, 107, 108, 109, 110, 111, 0, 112, 113, 114, 115, 116,
    117, 118, 119, 0, 0, 120, 0, 0, 0, 31, 0, 31, 121, 122, 123, 0,
    124, 0, 0, 0, 24, 125, 126, 127, 127, 30, 10, 14, 0, 128, 128, 127,
    127, 129, 130, 0, 76, 76, 131, 132, 133, 134, 0, 135, 135, 136, 136, 136,
    137, 138, 0, 76, 76, 139, 140, 141, 142, 142, 33, 0, 143, 144, 145, 145,
    145, 76, 146, 147, 148, 149, 150, 150, 0, 127, 128, 128, 151, 152, 153, 154,
    0, 0, 122, 123, 152, 155, 152, 156, 154, 0, 157, 152, 158, 154, 159, 65,
    65, 160, 154, 161, 152, 37, 162, 163, 163, 164, 37, 165, 166, 167, 168, 168,
    37, 169, 170, 171, 172, 173, 127, 127, 83, 0, 162, 162, 174, 174, 175, 176,
    0, 177, 119, 119, 154, 178, 152, 179, 180, 181, 182, 182, 154, 183, 152, 184,
    185, 186, 0, 0, 0, 154, 0, 0, 0, 187, 187, 112, 62, 175, 0, 188,
    189, 189, 190, 114, 0, 31, 174, 142, 142, 191, 120, 33, 0, 127, 127, 77,
    0, 0, 0, 192, 177, 0, 112, 112, 175, 176, 0, 188, 190, 114, 114, 0,
    31, 127, 127, 175, 0, 193, 193, 194, 194, 152, 0, 0, 0, 174, 174, 152,
    194, 194, 195, 196, 197, 62, 62, 175, 0, 188, 190, 114, 114, 0, 31, 112,
    112, 198, 199, 199, 175, 0, 0, 189, 189, 189, 190, 114, 114, 0, 174, 174,
    0, 0, 175, 0, 0, 200, 201, 202, 0, 203, 19, 19, 0, 204, 204, 205,
    206, 0, 127, 127, 204, 205, 206, 0, 207, 208, 140, 140, 208, 127, 127, 201,
    0, 195, 197, 196, 142, 142, 0, 0, 191, 120, 33, 0, 200, 200, 201, 202,
    0, 33, 0, 175, 175, 0, 33, 0, 143, 143, 145, 144, 209, 210, 210, 33,
    128, 128, 211, 211, 112, 174, 212, 213, 72, 214, 215, 216, 217, 205, 218, 219,
    220, 221, 222, 223, 224, 0, 0, 37, 24, 31, 0, 0, 17, 225, 225, 77,
    0, 226, 225, 227, 228, 229, 229, 31, 0, 0, 127, 127, 119, 119, 154, 230,
    231, 232, 233, 129, 234, 234, 235, 236, 129, 237, 78, 0, 31, 120, 238, 31,
    238, 225, 239, 240, 241, 217, 217, 217, 242, 243, 244, 77, 0, 0, 0, 0,
    0, 0, 73, 245, 76, 0, 0, 225, 225, 37, 246, 0, 0, 174, 245, 76,
    0, 0, 247, 248, 249, 250, 249, 251, 252, 253, 254, 42, 255, 256, 257, 258,
    259, 260, 261, 240, 262, 263, 264, 265, 266, 267, 104, 0, 0, 37, 0, 268,
    104, 269, 0, 270, 104, 0, 0, 0, 120, 120, 120, 271, 272, 273, 274, 275,
    276, 133, 277, 278, 279, 280, 281, 282, 283, 284, 285, 285, 286, 285, 287, 288,
    289, 290, 291, 292, 293, 294, 0, 0, 294, 295, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 32, 120, 0, 0, 296, 297, 298, 299, 300, 301, 302, 303, 304,
    256, 262, 305, 306, 307, 308, 0, 309, 310, 311, 312, 313, 314, 315, 316, 317,
    318, 240, 264, 264, 264, 262, 265, 265, 265, 263, 319, 320, 321, 322, 323, 305,
    250, 324, 325, 326, 76, 327, 328, 329, 330, 331, 328, 329, 330, 332, 31, 0,
    0, 31, 0, 0, 31, 0, 0, 0, 0, 0, 0, 31, 31, 0, 333, 271,
    334, 0, 31, 335, 336, 337, 338, 339, 340, 341, 342, 246, 343, 344, 345, 164,
    346, 31, 0, 0, 0, 31, 120, 31, 238, 168, 347, 348, 348, 348, 0, 349,
    350, 351, 352, 353, 354, 354, 354, 0, 355, 355, 356, 357, 356, 357, 357, 354,
    354, 0, 355, 355, 356, 357, 356, 357, 357, 306, 32, 304, 354, 358, 359, 15,
    360, 361, 362, 363, 364, 365, 199, 366, 367, 303, 368, 369, 370, 371, 372, 373,
    374, 375, 375, 376, 140, 140, 376, 13, 13, 234, 377, 378, 378, 234, 379, 379,
    234, 380, 380, 234, 22, 381, 381, 234, 22, 234, 234, 235, 236, 129, 237, 382,
    383, 153, 370, 370, 194, 384, 384, 205, 205, 205, 77, 0, 0, 385, 386, 386,
    387, 388, 389, 390, 391, 366, 392, 393, 394, 394, 394, 77, 0, 0, 395, 388,
    391, 396, 397, 398, 399, 400, 401, 402, 403, 404, 405, 406, 214, 407, 408, 0,
    156, 156, 0, 156, 409, 0, 159, 0, 0, 410, 0, 411, 412, 413, 414, 415,
    268, 268, 268, 0, 31, 0, 0, 0, 0, 294, 0, 0, 0, 206, 206, 0,
    0, 416, 140, 417, 205, 205, 31, 0, 152, 152, 0, 203, 203, 0, 418, 285,
    67, 67, 0, 149, 156, 156, 0, 0, 153, 153, 0, 0, 122, 123, 152, 156,
    419, 420, 420, 239, 120, 31, 0, 421, 422, 423, 424, 424, 37, 195, 196, 197,
    120, 120, 0, 420, 420, 239, 120, 31, 0, 0, 0, 0, 0, 425, 426, 427,
    425, 425, 426, 427, 31, 428, 416, 140, 140, 417, 152, 152, 429, 430, 294, 37,
    159, 417, 76, 76, 73, 294, 430, 294, 31, 0, 0, 0, 431, 432, 433, 434,
    435, 316, 436, 437, 0, 175, 307, 0, 0, 410, 0, 438, 439, 0, 0, 0,
    0, 440, 32, 441, 442, 149, 149, 159, 149, 0, 443, 444, 445, 446, 306, 154,
    265, 264, 265, 264, 447, 448, 449, 264, 0, 450, 139, 451, 0, 0, 452, 453,
    454, 455, 456, 262, 262, 262, 457, 458, 459, 460, 0, 461, 460, 0, 462, 463,
    59, 464, 465, 466, 467, 206, 468, 469, 470, 471, 472, 378, 473, 474, 475, 476,
    31, 0, 477, 32, 32, 0, 478, 478, 0, 303, 0, 210, 210, 479, 180, 480,
    481, 482, 483, 484, 485, 39, 39, 358, 31, 0, 120, 0, 225, 486, 487, 82,
    80, 81, 488, 489, 490, 491, 472, 492, 0, 31, 386, 386, 0, 120, 120, 275,
    39, 39, 30, 10, 14, 176, 0, 493, 493, 39, 39, 129, 130, 176, 0, 76,
    131, 132, 133, 134, 0, 135, 135, 136, 136, 136, 137, 138, 0, 76, 76, 139,
    140, 141, 143, 144, 145, 145, 145, 76, 146, 147, 148, 149, 150, 0, 39, 39,
    129, 80, 0, 31, 494, 494, 0, 149, 375, 0, 39, 39, 495, 418, 496, 497,
    201, 0, 478, 76, 0, 31, 498, 177, 119, 119, 154, 499, 152, 179, 180, 500,
    181, 182, 182, 154, 183, 152, 184, 185, 186, 39, 39, 39, 501, 502, 176, 0,
    503, 503, 31, 0, 0, 0, 154, 0, 0, 104, 159, 159, 120, 238, 493, 152,
    504, 370, 370, 194, 154, 0, 0, 159, 156, 156, 154, 0, 158, 158, 154, 159,
    39, 39, 129, 81, 176, 0, 31, 149, 375, 0, 39, 39, 16, 176, 0, 39,
    39, 14, 176, 0, 505, 163, 163, 164, 37, 506, 506, 225, 37, 165, 166, 167,
    168, 168, 37, 169, 170, 171, 172, 173, 39, 39, 83, 0, 505, 505, 39, 39,
    501, 70, 86, 77, 176, 0, 507, 31, 31, 0, 0, 198, 0, 201, 508, 508,
    154, 509, 510, 510, 511, 511, 202, 117, 117, 202, 418, 418, 202, 31, 0, 0,
    404, 404, 164, 31, 0, 0, 149, 512, 513, 514, 515, 515, 31, 159, 159, 159,
    154, 159, 516, 516, 0, 517, 508, 508, 154, 509, 149, 515, 515, 31, 159, 159,
    159, 154, 159, 120, 238, 31, 238, 39, 39, 0, 129, 71, 176, 0, 518, 296,
    114, 519, 61, 73, 60, 76, 0, 386, 386, 520, 521, 522, 522, 523, 524, 0,
    0, 0, 297, 297, 0, 357, 39, 39, 41, 176, 0, 0, 0, 525, 525, 525,
    525, 526, 164, 164, 527, 528, 528, 529, 159, 225, 225, 0, 530, 530, 164, 164,
    527, 528, 528, 529, 159, 120, 120, 159, 531, 531, 282, 525, 526, 164, 164, 527,
    528, 528, 529, 159, 225, 225, 0, 120, 159, 249, 152, 152, 16, 0, 0, 532,
    532, 533, 184, 534, 534, 535, 536, 396, 396, 159, 537, 537, 159, 538, 538, 159,
    249, 225, 225, 539, 539, 539, 540, 541, 542, 543, 37, 544, 544, 529, 529, 545,
    546, 546, 152, 547, 295, 295, 37, 548, 529, 529, 549, 161, 152, 37, 529, 529,
    550, 295, 161, 152, 37, 37, 548, 529, 529, 532, 551, 552, 552, 152, 553, 554,
    554, 37, 555, 555, 556, 37, 557, 5, 5, 558, 559, 182, 560, 560, 149, 279,
    159, 39, 39, 501, 119, 119, 154, 230, 231, 232, 233, 129, 234, 234, 235, 236,
    129, 237, 78, 0, 31, 120, 238, 31, 238, 225, 561, 562, 563, 564, 565, 460,
    566, 567, 567, 568, 568, 567, 514, 31, 0, 569, 551, 31, 37, 430, 570, 571,
    572, 573, 430, 430, 31, 574, 575, 576, 577, 578, 579, 570, 580, 581, 582, 582,
    583, 584, 34, 0, 229, 0, 0, 0, 0, 585, 586, 586, 403, 587, 0, 0,
    588, 589, 171, 22, 10, 590, 591, 0, 0, 0, 0, 0, 592, 593, 593, 594,
    595, 595, 596, 597, 598, 0, 0, 599, 599, 370, 303, 600, 601, 35, 35, 36,
    0, 0, 0, 410, 0, 31, 602, 303, 596, 597, 603, 603, 603, 597, 597, 597,
    603, 0, 604, 605, 605, 31, 31, 0, 0, 135, 0, 606, 607, 604, 608, 427,
    609, 122, 37, 426, 135, 610, 611, 612, 613, 614, 615, 548, 616, 617, 618, 619,
    620, 621, 622, 623, 624, 625, 626, 626, 626, 625, 174, 627, 0, 628, 629, 630,
    631, 632, 632, 632, 632, 0, 585, 386, 0, 632, 632, 633, 475, 634, 635, 636,
    386, 0, 637, 638, 140, 140, 639, 640, 640, 641, 159, 0, 159, 37, 0, 0,
    0, 632, 632, 0, 458, 386, 642, 159, 37, 632, 632, 0, 0, 152, 152, 0,
    159, 37, 0, 643, 152, 152, 152, 644, 645, 646, 647, 648, 649, 129, 129, 129,
    154, 0, 650, 651, 652, 653, 60, 60, 60, 654, 388, 655, 0, 34, 34, 0,
    0, 478, 478, 656, 0, 0, 0, 34, 34, 656, 34, 34, 223, 656, 118, 657,
    0, 0, 410, 410, 260, 478, 478, 0, 0, 658, 659, 660, 661, 661, 662, 663,
    664, 665, 666, 667, 668, 669, 334, 670, 670, 670, 0, 0, 671, 672, 673, 674,
    376, 675, 676, 677, 678, 679, 680, 145, 212, 681, 682, 683, 684, 685, 686, 687,
    513, 688, 513, 513, 164, 689, 690, 690, 31, 0, 0, 246, 246, 691, 37, 286,
    528, 528, 246, 246, 691, 37, 286, 266, 266, 259, 150, 11, 692, 410, 593, 123,
    123, 123, 693, 694, 352, 174, 174, 358, 31, 0, 120, 0, 693, 174, 174, 10,
    176, 0, 693, 174, 174, 130, 176, 0, 174, 174, 80, 176, 0, 174, 174, 201,
    0, 76, 0, 177, 119, 119, 154, 695, 152, 179, 180, 182, 182, 154, 183, 152,
    184, 185, 186, 174, 174, 174, 502, 176, 0, 174, 174, 696, 81, 80, 176, 0,
    174, 174, 16, 176, 0, 174, 174, 14, 176, 0, 693, 174, 174, 83, 0, 174,
    174, 77, 176, 0, 697, 697, 174, 174, 71, 176, 0, 174, 174, 18, 176, 0,
    693, 174, 174, 41, 176, 0, 174, 174, 78, 176, 0, 693, 698, 699, 699, 700,
    701, 702, 511, 703, 704, 705, 706, 520, 707, 356, 356, 0, 356, 708, 709, 31,
    0, 0, 31, 144, 710, 711, 712, 158, 713, 714, 715, 716, 357, 138, 208, 508,
    509, 717, 718, 719, 720, 721, 154, 722, 723, 724, 725, 726, 130, 727, 134, 728,
    729, 216, 730, 731, 186, 732, 45, 45, 45, 0, 478, 478, 0, 0, 50, 733,
    734, 634, 735, 736, 545, 737, 738, 738, 738, 739, 238, 61, 73, 60, 740, 0,
    0, 0, 0, 0, 741, 304, 304, 0, 742, 306, 449, 449, 140, 0, 0, 0,
    156, 743, 140, 300, 0, 249, 0, 0, 0, 0, 744, 745, 745, 745, 660, 746,
    747, 0, 0, 748, 748, 748, 748, 256, 262, 306, 0, 749, 745, 745, 745, 660,
    0, 0, 0, 750, 0, 734, 734, 31, 238, 0, 0, 0, 354, 133, 133, 0,
    32, 0, 0, 294, 0, 31, 0, 0, 0, 238, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 31, 0, 0, 0, 0, 751, 159, 37, 328,
    417, 606, 0, 0, 32, 104, 0, 0, 752, 68, 0, 295, 0, 0, 0, 753,
    391, 0, 754, 755, 86, 756, 757, 554, 330, 330, 389, 758, 759, 582, 582, 760,
    190, 584, 0, 37, 0, 521, 761, 521, 762, 763, 160, 225, 225, 764, 582, 765,
    87, 0, 658, 658, 0, 0, 658, 658, 658, 658, 766, 767, 768, 768, 31, 656,
    656, 0, 768, 769, 770, 190, 771, 772, 773, 774, 775, 776, 777, 777, 777, 242,
    243, 244, 0, 0, 0, 303, 303, 778, 508, 508, 778, 723, 723, 778, 779, 779,
    778, 780, 778, 58, 0, 781, 781, 778, 386, 386, 367, 0, 782, 782, 778, 783,
    784, 89, 89, 0, 0, 0, 225, 225, 225, 65, 182, 89, 89, 0, 0, 0,
    386, 386, 386, 740, 785, 786, 523, 787, 386, 386, 386, 788, 73, 102, 789, 789,
    0, 0, 790, 791, 792, 792, 793, 182, 182, 26, 102, 794, 386, 697, 19, 19,
    202, 19, 88, 795, 796, 639, 797, 797, 0, 0, 551, 551, 521, 0, 0, 37,
    295, 557, 0, 798, 799, 786, 300, 300, 249, 797, 762, 800, 801, 802, 803, 804,
    34, 34, 0, 0, 805, 656, 657, 101, 101, 164, 118, 118, 164, 806, 807, 808,
    809, 225, 225, 582, 810, 787, 809, 225, 225, 229, 0, 0, 0, 0, 31, 0,
    0, 0, 0, 805, 811, 0, 0, 225, 225, 810, 0, 811, 812, 31, 0, 0,
    0, 0, 31, 0, 0, 813, 0, 31, 0, 118, 118, 164, 101, 101, 164, 0,
    0, 0, 0, 0, 0, 118, 118, 164, 101, 101, 164, 0, 34, 814, 815, 816,
    496, 0, 0, 0, 0, 0, 120, 0, 817, 817, 0, 34, 34, 0, 0, 0,
    135, 0, 34, 0, 0, 684, 0, 260, 0, 0, 118, 818, 818, 0, 0, 0,
    159, 0, 0, 26, 0, 159, 0, 0, 819, 0, 0, 31, 31, 0, 0, 80,
    820, 0, 0, 0, 0, 410, 31, 821, 820, 588, 31, 0, 0, 0, 0, 253,
    253, 18, 0, 0, 253, 253, 18, 0, 31, 0, 0, 0, 822, 822, 31, 823,
    823, 31, 210, 210, 31, 605, 605, 31, 0, 0, 0, 0, 0, 605, 0, 752,
    31, 0, 92, 824, 27, 825, 779, 420, 826, 827, 828, 829, 830, 830, 551, 551,
    831, 677, 0, 0, 0, 0, 0, 154, 832, 833, 834, 458, 0, 0, 0, 37,
    0, 0, 0, 0, 0, 154, 835, 398, 836, 837, 838, 839, 468, 470, 469, 360,
    840, 841, 439, 589, 830, 842, 843, 844, 845, 846, 847, 848, 99, 99, 99, 0,
    38, 38, 76, 0, 0, 0, 33, 0, 0, 225, 225, 225, 99, 849, 225, 225,
    113, 849, 850, 48, 851, 265, 265, 154, 265, 852, 101, 101, 164, 0, 24, 853,
    568, 0, 0, 0, 0, 37, 0, 854, 225, 225, 514, 854, 118, 118, 164, 657,
    806, 807, 808, 582, 101, 855, 0, 0, 0, 0, 37, 0, 31, 854, 853, 853,
    853, 856, 31, 0, 0, 568, 0, 857, 858, 636, 690, 859, 806, 806, 0, 31,
    0, 860, 118, 118, 164, 657, 807, 808, 582, 806, 861, 862, 863, 864, 864, 864,
    159, 865, 866, 867, 868, 245, 549, 869, 436, 870, 871, 0, 872, 873, 780, 874,
    875, 876, 877, 878, 879, 880, 525, 0, 881, 882, 883, 176, 176, 734, 884, 176,
    176, 0, 885, 0, 708, 708, 0, 885, 0, 356, 356, 0, 357, 357, 176, 176,
    176, 176, 140, 734, 655, 819, 819, 655, 886, 887, 888, 889, 890, 891, 892, 893,
    213, 57, 26, 742, 140, 26, 0, 31, 0, 0, 0, 238, 0, 894, 895, 896,
    163, 897, 0, 0, 333, 271, 334, 31, 37, 201, 201, 176, 0, 31, 0, 0,
    478, 478, 898, 899, 314, 322, 515, 515, 31, 159, 159, 159, 154, 159, 149, 149,
    159, 900, 900, 55, 655, 697, 200, 201, 202, 386, 386, 901, 902, 118, 118, 164,
    101, 101, 164, 31, 0, 76, 201, 498, 903, 903, 154, 37, 478, 225, 225, 904,
    0, 149, 195, 197, 196, 717, 717, 154, 159, 905, 905, 154, 159, 478, 478, 906,
    342, 690, 907, 908, 909, 910, 911, 912, 531, 913, 914, 915, 916, 308, 917, 918,
    179, 919, 918, 26, 920, 61, 120, 120, 0, 31, 0, 61, 61, 229, 0, 223,
    223, 31, 0, 0, 225, 225, 921, 0, 31, 238, 752, 238, 922, 923, 924, 490,
    181, 925, 926, 927, 928, 929, 930, 931, 932, 598, 12, 933, 934, 935, 936, 295,
    295, 0, 135, 0, 0, 135, 295, 229, 229, 0, 0, 0, 0, 0, 0, 702,
    702, 159, 937, 937, 159, 370, 370, 194, 0, 938, 938, 765, 440, 939, 939, 164,
    159, 940, 941, 710, 164, 159, 726, 726, 164, 159, 942, 942, 943, 943, 164, 159,
    939, 939, 164, 159, 944, 229, 899, 196, 237, 945, 946, 947, 948, 949, 950, 0,
    417, 951, 0, 328, 0, 449, 449, 140, 684, 34, 449, 0, 952, 953, 954, 955,
    956, 537, 957, 957, 957, 0, 958, 959, 0, 0, 0, 902, 0, 0, 0, 0,
    0, 0, 960, 76, 478, 0, 0, 0, 902, 0, 0, 0, 0, 0, 902, 902,
    902, 0, 76, 76, 0, 0, 31, 0, 0, 461, 917, 961, 961, 816, 962, 963,
    964, 478, 478, 31, 0, 0, 965, 440, 440, 31, 0, 0, 0, 0, 386, 966,
    967, 967, 968, 159, 440, 0, 0, 0, 969, 734, 734, 968, 159, 968, 968, 159,
    970, 970, 970, 417, 723, 152, 152, 159, 76, 971, 709, 723, 31, 0, 32, 159,
    440, 822, 31, 0, 32, 0, 972, 972, 0, 370, 370, 194, 0, 970, 970, 970,
    417, 723, 152, 152, 0, 76, 971, 709, 723, 823, 822, 823, 973, 974, 202, 903,
    975, 976, 977, 209, 0, 978, 921, 979, 980, 13, 35, 981, 982, 983, 984, 0,
    985, 986, 987, 988, 989, 990, 991, 992, 993, 962, 994, 995, 996, 264, 264, 154,
    264, 997, 741, 741, 140, 998, 0, 725, 725, 0, 998, 516, 999, 1000, 516, 448,
    657, 657, 0, 31, 0, 37, 0, 24, 1001, 810, 810, 0, 37, 0, 32, 32,
    37, 657, 1002, 942, 1003, 1004, 0, 1005, 223, 223, 176, 0, 644, 26, 31, 0,
    0, 31, 0, 0, 0, 1006, 0, 159, 37, 328, 745, 417, 606, 104, 0, 1007,
    123, 0, 0, 0, 1008, 1009, 1009, 22, 1010, 654, 457, 244, 244, 909, 159, 1011,
    434, 655, 229, 77, 0, 223, 370, 370, 194, 0, 938, 938, 765, 440, 939, 939,
    164, 159, 940, 941, 941, 710, 164, 159, 726, 726, 164, 159, 942, 942, 943, 943,
    164, 159, 939, 939, 164, 159, 655, 655, 37, 1012, 942, 762, 31, 31, 0, 0,
    194, 194, 0, 1009, 0, 244, 0, 572, 573, 1013, 1014, 182, 793, 1015, 1016, 1017,
    1018, 778, 1019, 1020, 1021, 1022, 0, 0, 0, 0, 268, 1022, 0, 0, 357, 357,
    1022, 0, 654, 1023, 1024, 813, 813, 410, 0, 31, 502, 502, 0, 32, 32, 31,
    238, 47, 1025, 1026, 0, 43, 1027, 1028, 1029, 1030, 1031, 1032, 1033, 1034, 1035, 1036,
    1037, 1038, 1039, 1040, 1041, 1042, 1043, 1044, 1045, 1046, 1047, 1048, 1049, 1050, 1051, 1052,
    0, 1053, 1054, 1055, 1056, 1057, 1058, 1059, 1060, 85, 432, 542, 540, 1061, 1062, 541,
    543, 1063, 1064, 390, 1065, 1066, 1067, 1068, 1069, 1070, 60, 61, 61, 31, 752, 232,
    232, 229, 232, 355, 355, 355, 355, 355, 1060, 249, 249, 0, 885, 1071, 73, 60,
    355, 355, 140, 1060, 249, 249, 0, 885, 61, 232, 232, 0, 0, 0, 478, 0,
    0, 0, 31, 31, 238, 0, 0, 0, 0, 0, 76, 0, 355, 73, 73, 194,
    1072, 1073, 838, 492, 1074, 1075, 1076, 1077, 1078, 433, 1079, 1080, 258, 1081, 1082, 1083,
    1084, 1085, 813, 1086, 267, 267, 794, 502, 381, 62, 1087, 261, 8, 1088, 1089, 1090,
    1091, 1092, 1093, 260, 1094, 1095, 1096, 1097, 1098, 1099, 839, 0, 1100, 0, 0, 120,
    0, 0, 31, 0, 808, 808, 0, 0, 1101, 951, 951, 1102, 1103, 1103, 461, 1103,
    1104, 1105, 1106, 1107, 1108, 258, 0, 967, 1109, 1110, 547, 1111, 0, 584, 1112, 379,
    1113, 1114, 243, 1115, 169, 1116, 1117, 500, 1118, 1119, 1120, 172, 1121, 1122, 1123, 335,
    752, 68, 31, 68, 31, 225, 31, 335, 31, 0, 1124, 73, 61, 60, 232, 335,
    335, 1125, 31, 68, 335, 1126, 1127, 184, 1128, 1129, 0, 1130, 1131, 0, 447, 1132,
    1132, 154, 264, 264, 265, 140, 0, 746, 140, 1132, 1133, 1134, 1135, 571, 791, 1136,
    1137, 417, 417, 264, 264, 154, 265, 1138, 1139, 887, 833, 1140, 37, 1141, 36, 159,
    159, 268, 0, 1007, 159, 159, 1007, 410, 410, 410, 410, 410, 0, 76, 120, 478,
    32, 359, 104, 31, 304, 268, 658, 0, 1142, 1143, 1144, 1145, 789, 327, 1146, 810,
    810, 0, 37, 0, 521, 761, 521, 762, 810, 1147, 1148, 1149, 1150, 76, 1151, 1152,
    160, 587, 1153, 1154, 1155, 1156, 63, 1157, 1158, 218, 218, 218, 242, 243, 244, 77,
    0, 0, 1159, 849, 553, 941, 503, 501, 324, 501, 1160, 1161, 501, 501, 434, 118,
    118, 164, 0, 0, 0, 0, 37, 0, 333, 271, 334, 887, 887, 1162, 190, 982,
    0, 657, 797, 1163, 657, 806, 807, 808, 797, 818, 582, 118, 173, 118, 1128, 1164,
    966, 1165, 1166, 1167, 964, 937, 1168, 1167, 0, 819, 1169, 0, 0, 31, 0, 1169,
    753, 753, 0, 31, 31, 606, 32, 606, 0, 753, 1170, 1171, 1172, 1173, 64, 1174,
    1175, 1176, 1177, 0, 1178, 1179, 1180, 1181, 1182, 1183, 1184, 1185, 1186, 1187, 904, 1188,
    223, 223, 164, 0, 31, 0, 0, 223, 512, 286, 286, 223, 164, 800, 31, 0,
    31, 223, 223, 164, 1189, 800, 800, 31, 0, 0, 31, 1190, 1190, 223, 34, 1005,
    1005, 31, 0, 0, 31, 223, 223, 224, 1191, 1192, 529, 1193, 1194, 555, 1195, 968,
    1196, 259, 259, 820, 820, 259, 159, 0, 1197, 1198, 1199, 1200, 1201, 1202, 1203, 1204,
    1205, 1206, 1207, 1208, 1209, 54, 262, 1210, 1211, 1212, 1213, 1214, 1215, 1216, 1217, 257,
    1218, 44, 1219, 1220, 1221, 257, 0, 0, 768, 812, 900, 223, 1222, 1223, 0, 524,
    799, 594, 1224, 1225, 627, 627, 627, 0, 34, 0, 159, 0, 0, 1226, 141, 1227,
    203, 203, 202, 203, 1228, 747, 1229, 58, 1230, 1231, 81, 0, 0, 1232, 1167, 26,
    918, 0, 0, 0, 31, 0, 0, 93, 1233, 1234, 1235, 1236, 515, 1237, 1238, 103,
    1239, 1239, 1240, 303, 1027, 778, 1241, 33, 33, 159, 296, 296, 0, 0, 1239, 745,
    745, 37, 807, 306, 306, 1242, 24, 253, 1243, 31, 238, 1244, 437, 1245, 1246, 1247,
    388, 1248, 1249, 588, 159, 122, 123, 1250, 1251, 528, 800, 223, 223, 176, 0, 644,
    1252, 996, 996, 1253, 1254, 1064, 31, 0, 0, 31, 31, 31, 0, 0, 0, 31,
    0, 0, 31, 31, 159, 159, 159, 154, 159, 34, 34, 31, 159, 33, 31, 159,
    295, 37, 1255, 1256, 1088, 548, 875, 1081, 1257, 1258, 154, 265, 1259, 264, 1132, 65,
    1260, 380, 1261, 1262, 1263, 219, 1264, 1265, 1266, 1267, 1268, 1269, 1270, 1271, 1272, 1273,
    67, 67, 0, 31, 1053, 76, 76, 76, 60, 1274, 76, 76, 73, 1274, 1275, 1053,
    1053, 1276, 816, 1188, 1277, 66, 67, 1278, 1274, 694, 352, 194, 194, 417, 379, 379,
    108, 194, 194, 152, 1279, 1000, 1280, 1281, 1282, 1283, 1284, 1285, 1286, 1287, 1288, 1289,
    1290, 1291, 781, 1292, 1293, 1294, 1295, 1296, 1297, 1298, 221, 142, 142, 142, 77, 0,
    0, 0, 33, 0, 215, 220, 220, 220, 242, 243, 244, 77, 0, 0, 1299, 388,
    391, 1300, 1301, 0, 0, 0, 0, 268, 1301, 0, 0, 1302, 1302, 0, 357, 357,
    1301, 0, 1303, 434, 434, 434, 1304, 654, 653, 229, 229, 0, 223, 1305, 1306, 31,
    752, 194, 1307, 31, 1307, 238, 77, 77, 0, 1308, 1309, 147, 1310, 1311, 1312, 1313,
    1314, 1315, 1316, 1317, 1318, 1319, 1320, 532, 1321, 1322, 1322, 1322, 1027, 33, 33, 37,
    1323, 1324, 1324, 1325, 1323, 0, 640, 640, 641, 159, 0, 194, 194, 835, 835, 835,
    398, 836, 837, 838, 839, 468, 470, 469, 360, 840, 841, 439, 589, 154, 0, 0,
    0, 159, 7, 1326, 886, 1327, 1328, 694, 1329, 1330, 1331, 1332, 1333, 1334, 1334, 1334,
    0, 1335, 1336, 1337, 837, 972, 494, 550, 1338, 100, 100, 100, 99, 99, 849, 386,
    0, 38, 38, 76, 0, 16, 16, 0, 64, 64, 76, 41, 0, 0, 34, 0,
    693, 1339, 963, 1340, 999, 16, 0, 0, 0, 0, 588, 122, 123, 277, 277, 0,
    0, 588, 122, 123, 581, 1341, 1342, 514, 514, 514, 514, 31, 0, 0, 0, 31,
    0, 0, 511, 511, 202, 31, 0, 0, 1343, 82, 1344, 1345, 1346, 94, 521, 834,
    1347, 1348, 1349, 1350, 1351, 1352, 762, 1353, 1354, 1355, 1356, 811, 811, 1357, 299, 1358,
    31, 37, 1303, 1303, 1303, 31, 37, 1359, 1360, 1361, 131, 1362, 14, 840, 25, 1363,
    1364, 1365, 1366, 1367, 1368, 1257, 1369, 1370, 1371, 905, 1372, 1373, 1374, 235, 0, 0,
    0, 0, 0, 1375, 1376, 684, 24, 687, 866, 1377, 1378, 1379, 1380, 1381, 1382, 1383,
    68, 68, 1375, 0, 0, 944, 229, 944, 34, 0, 656, 118, 118, 164, 657, 753,
    809, 582, 68, 223, 223, 31, 0, 0, 68, 68, 229, 34, 900, 223, 656, 101,
    806, 753, 808, 787, 118, 657, 808, 229, 809, 76, 225, 238, 1384, 31, 225, 1384,
    31, 225, 335, 31, 238, 1385, 1386, 659, 1387, 818, 83, 166, 1064, 1064, 37, 379,
    379, 1064, 165, 1085, 1085, 37, 379, 379, 1085, 95, 1388, 1389, 21, 1390, 1391, 0,
    1392, 194, 194, 0, 0, 1393, 1393, 1111, 1150, 410, 1394, 846, 1205, 1206, 1391, 244,
    0, 0, 0, 0, 1395, 154, 750, 0, 37, 159, 123, 0, 0, 325, 0, 0,
    256, 0, 194, 194, 0, 0, 0, 0, 154, 0, 0, 294, 0, 0, 0, 752,
    152, 194, 194, 152, 16, 0, 24, 184, 0, 0, 33, 0, 0, 335, 335, 809,
    809, 809, 335, 118, 657, 808, 194, 194, 194, 1101, 0, 152, 159, 0, 156, 156,
    0, 809, 1396, 1397, 1397, 935, 728, 24, 1080, 194, 194, 14, 176, 0, 194, 194,
    1398, 846, 1391, 1399, 1400, 1400, 956, 956, 997, 1401, 65, 65, 1402, 445, 1403, 1303,
    1303, 65, 1389, 65, 65, 1404, 75, 160, 1405, 1406, 856, 1407, 1408, 102, 185, 1409,
    0, 0, 0, 354, 359, 359, 359, 0, 133, 1410, 1411, 1325, 596, 745, 257, 258,
    1412, 1277, 0, 0, 588, 0, 410, 0, 0, 0, 0, 410, 260, 122, 122, 1413,
    0, 0, 104, 1414, 792, 259, 148, 268, 782, 1415, 1416, 1417, 1418, 1243, 1419, 1420,
    1421, 1422, 1423, 1424, 1425, 1426, 1427, 1428, 1429, 1430, 1431, 1432, 1433, 1434, 1435, 1436,
    1437, 1438, 1439, 1440, 1441, 1442, 1443, 1444, 1445, 1446, 1447, 1448, 497, 486, 1449, 1450,
    1451, 234, 1452, 1453, 1454, 1379, 1455, 1456, 1457, 1380, 1381, 1383, 1458, 1459, 1460, 1382,
    1461, 1378, 1462, 1463, 1464, 1465, 1466, 1467, 1468, 1469, 1470, 1471, 1472, 1473, 1474, 637,
    640, 638, 1475, 424, 535, 1476, 831, 79, 91, 260, 506, 943, 137, 1477, 1478, 0,
    238, 232, 61, 0, 174, 174, 39, 174, 72, 98, 39, 39, 174, 127, 194, 174,
    174, 39, 174, 39, 174, 39, 60, 73, 39, 174, 39, 174, 39, 174, 670, 55,
    738, 39, 127, 174, 39, 174, 661, 39, 632, 194, 194, 884, 0, 957, 0, 0,
    348, 627, 45, 626, 830, 1324, 586, 582, 174, 127, 127, 39, 142, 1322, 1239, 864,
    205, 394, 217, 220, 218, 76, 120, 777, 1009, 434, 127, 1334, 194, 39, 127, 174,
    194, 658, 491, 1479, 1480, 1481, 1482, 1483, 783, 1484, 1485, 1486, 1487, 1488, 1489, 422,
    1377, 84, 1490, 96, 1491, 262, 262, 1492, 1493, 31, 0, 0, 0, 0, 0, 1494,
    1495, 836, 538, 1496, 1497, 1498, 1499, 1500, 1501, 435, 1502, 1503, 1504, 1505, 1506, 1507,
    0, 1508, 1509, 1510, 1511, 1512, 1358, 0, 1513, 198, 1514, 206, 206, 0, 1515, 418,
    418, 200, 201, 202, 0, 478, 31, 498, 203, 203, 478, 31, 498, 19, 19, 0,
    31, 67, 0, 31, 76, 76, 76, 1071, 60, 73, 1274, 694, 352, 194, 194, 417,
    379, 379, 108, 194, 194, 152, 205, 205, 31, 0, 207, 208, 140, 140, 208, 200,
    200, 201, 202, 0, 478, 31, 498, 1516, 1516, 496, 497, 200, 201, 202, 0, 31,
    498, 0, 0, 752, 294, 0, 0, 31, 238, 31, 232, 77, 192, 192, 287, 287,
    154, 37, 285, 285, 154, 37, 289, 289, 154, 37, 151, 152, 153, 154, 0, 159,
    0, 152, 394, 394, 154, 37, 155, 152, 156, 154, 0, 159, 0, 152, 149, 149,
    154, 159, 455, 455, 154, 159, 464, 464, 154, 159, 1517, 1517, 22, 201, 680, 154,
    159, 652, 652, 154, 37, 158, 158, 154, 159, 508, 508, 154, 0, 159, 0, 717,
    717, 154, 159, 923, 923, 154, 1518, 379, 190, 982, 903, 903, 154, 37, 1061, 1061,
    154, 37, 1079, 1079, 154, 37, 1174, 1174, 154, 161, 37, 152, 529, 529, 154, 159,
    1347, 1347, 154, 1519, 152, 703, 772, 454, 1272, 446, 905, 905, 154, 159, 1481, 1481,
    154, 161, 152, 37, 1482, 1482, 154, 161, 152, 37, 1483, 1483, 154, 37, 17, 17,
    0, 1489, 1489, 1520, 682, 1070, 1396, 1396, 154, 31, 0, 0, 0, 699, 0, 0,
    410, 0, 1521, 841, 89, 811, 17, 1522, 1523, 1524, 0, 744, 744, 304, 0, 0,
    749, 749, 0, 46, 97, 69, 1525, 0, 1526, 69, 70, 70, 1527, 1528, 1526, 233,
    1529, 939, 1530, 1531, 1253, 885, 1532, 764, 660, 608, 592, 592, 608, 427, 31, 226,
    226, 225, 227, 608, 1533, 133, 133, 133, 608, 1533, 0, 427, 1534, 1535, 1536, 1537,
    1538, 1539, 1540, 1541, 33, 1542, 1543, 1544, 1545, 1546, 1547, 1548, 1399, 0, 506, 506,
    1399, 0, 0, 0, 268, 297, 297, 1549, 0, 992, 992, 0, 444, 444, 1549, 249,
    249, 379, 364, 1342, 1342, 1549, 357, 357, 0, 1550, 1551, 1552, 1553, 1554, 1555, 1528,
    1556, 1557, 1558, 1559, 1560, 71, 0, 1561, 1562, 1563, 1564, 1565, 817, 332, 990, 1547,
    329, 329, 72, 72, 72, 77, 0, 1566, 1567, 31, 0, 0, 18, 1568, 0, 0,
    104, 0, 0, 31, 1569, 1570, 1571, 557, 1403, 0, 0, 1103, 0, 0, 231, 1572,
    635, 1573, 1574, 1575, 1576, 1577, 1396, 1396, 1578, 684, 24, 1396, 418, 418, 202, 418,
    1579, 0, 1580, 559, 1581, 1582, 68, 68, 229, 34, 900, 223, 656, 101, 806, 753,
    808, 787, 809, 225, 225, 118, 657, 229, 787, 900, 225, 223, 76, 335, 335, 809,
    809, 809, 335, 1583, 1584, 597, 1302, 1585, 73, 73, 73, 654, 655, 0, 0, 0,
    0, 194, 656, 118, 657, 1586, 387, 1587, 76, 76, 511, 202, 73, 0, 0, 51,
    55, 55, 55, 0, 74, 1300, 1588, 41, 0, 1589, 1590, 1591, 1592, 784, 1593, 478,
    1594, 1595, 1596, 1597, 1598, 1599, 228, 1600, 641, 752, 752, 641, 644, 1601, 1601, 521,
    31, 0, 0, 152, 0, 0, 644, 1602, 1603, 1604, 1605, 1606, 1607, 0, 0, 750,
    0, 1608, 1609, 1610, 601, 1611, 1612, 1613, 1614, 1615, 78, 90, 423, 197, 1616, 1617,
    1618, 750, 9, 1619, 1620, 1621, 1622, 1623, 1624, 355, 355, 0, 0, 0, 885, 0,
    0, 0, 0, 355, 1625, 656, 656, 0, 31, 0, 1626, 657, 101, 101, 164, 118,
    118, 164, 806, 807, 808, 753, 582, 229, 656, 1627, 0, 0, 0, 0, 268, 1627,
    1627, 0, 150, 0, 0, 357, 357, 1627, 0, 1628, 0, 1629, 0, 0, 0, 0,
    0, 0, 0, 410, 260, 260, 410, 260, 808, 0, 0, 1630, 223, 118, 223, 118,
    797, 225, 225, 1631, 353, 244, 205, 849, 225, 225, 1268, 0, 0, 0, 68, 0,
    1124, 225, 225, 1268, 0, 0, 0, 68, 0, 335, 225, 225, 379, 379, 22, 0,
    225, 225, 225, 379, 14, 309, 1632, 227, 0, 1549, 1633, 23, 75, 1634, 536, 1635,
    1636, 1637, 1161, 1638, 1256, 1639, 603, 1533,
];

pub fn get_doris_segment_grammar(name: &str) -> Option<RootGrammar> {
    match name {
        "AbortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(0), tables: &DORIS_TABLES }),
        "AbsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1), tables: &DORIS_TABLES }),
        "AbsoluteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2), tables: &DORIS_TABLES }),
        "AccessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3), tables: &DORIS_TABLES }),
        "AccessObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(4), tables: &DORIS_TABLES }),
        "AccessPermissionSegment" => Some(RootGrammar { grammar_id: GrammarId(65), tables: &DORIS_TABLES }),
        "AccessPermissionsSegment" => Some(RootGrammar { grammar_id: GrammarId(134), tables: &DORIS_TABLES }),
        "AccessSchemaObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(137), tables: &DORIS_TABLES }),
        "AccessSchemaPluralObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(160), tables: &DORIS_TABLES }),
        "AccessStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(183), tables: &DORIS_TABLES }),
        "AccessTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(186), tables: &DORIS_TABLES }),
        "AccessorGrammar" => Some(RootGrammar { grammar_id: GrammarId(208), tables: &DORIS_TABLES }),
        "AccountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(210), tables: &DORIS_TABLES }),
        "Account_lockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(211), tables: &DORIS_TABLES }),
        "Account_unlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(212), tables: &DORIS_TABLES }),
        "AccountsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(213), tables: &DORIS_TABLES }),
        "ActionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(214), tables: &DORIS_TABLES }),
        "ActionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(215), tables: &DORIS_TABLES }),
        "ActiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(216), tables: &DORIS_TABLES }),
        "AdaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(217), tables: &DORIS_TABLES }),
        "AddDropSystemVersioningGrammar" => Some(RootGrammar { grammar_id: GrammarId(218), tables: &DORIS_TABLES }),
        "AddKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(219), tables: &DORIS_TABLES }),
        "AdminKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(220), tables: &DORIS_TABLES }),
        "AfterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(221), tables: &DORIS_TABLES }),
        "AgainstKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(222), tables: &DORIS_TABLES }),
        "Agg_stateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(223), tables: &DORIS_TABLES }),
        "AggregateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(224), tables: &DORIS_TABLES }),
        "AggregateOrderByClause" => Some(RootGrammar { grammar_id: GrammarId(225), tables: &DORIS_TABLES }),
        "AlgorithmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(226), tables: &DORIS_TABLES }),
        "AliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(227), tables: &DORIS_TABLES }),
        "AliasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(240), tables: &DORIS_TABLES }),
        "AliasedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(241), tables: &DORIS_TABLES }),
        "AllKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(244), tables: &DORIS_TABLES }),
        "AllocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(245), tables: &DORIS_TABLES }),
        "AlsoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(246), tables: &DORIS_TABLES }),
        "AlterDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(247), tables: &DORIS_TABLES }),
        "AlterEventStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(255), tables: &DORIS_TABLES }),
        "AlterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(297), tables: &DORIS_TABLES }),
        "AlterOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(298), tables: &DORIS_TABLES }),
        "AlterSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(326), tables: &DORIS_TABLES }),
        "AlterSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(342), tables: &DORIS_TABLES }),
        "AlterTableDropColumnGrammar" => Some(RootGrammar { grammar_id: GrammarId(348), tables: &DORIS_TABLES }),
        "AlterTableOnlineDDLOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(353), tables: &DORIS_TABLES }),
        "AlterTableOptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(370), tables: &DORIS_TABLES }),
        "AlterTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(397), tables: &DORIS_TABLES }),
        "AlterViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(570), tables: &DORIS_TABLES }),
        "AlwaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(594), tables: &DORIS_TABLES }),
        "AmpersandSegment" => Some(RootGrammar { grammar_id: GrammarId(595), tables: &DORIS_TABLES }),
        "AnalyseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(596), tables: &DORIS_TABLES }),
        "AnalyzeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(597), tables: &DORIS_TABLES }),
        "AnalyzeTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(598), tables: &DORIS_TABLES }),
        "AnalyzedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(626), tables: &DORIS_TABLES }),
        "AnalyzerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(627), tables: &DORIS_TABLES }),
        "AndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(628), tables: &DORIS_TABLES }),
        "AndOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(629), tables: &DORIS_TABLES }),
        "AntiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(632), tables: &DORIS_TABLES }),
        "AnyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(633), tables: &DORIS_TABLES }),
        "AppendKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(634), tables: &DORIS_TABLES }),
        "ApplyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(635), tables: &DORIS_TABLES }),
        "AreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(636), tables: &DORIS_TABLES }),
        "ArithmeticBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(637), tables: &DORIS_TABLES }),
        "ArrayAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(650), tables: &DORIS_TABLES }),
        "ArrayExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(658), tables: &DORIS_TABLES }),
        "ArrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(659), tables: &DORIS_TABLES }),
        "ArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(660), tables: &DORIS_TABLES }),
        "ArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(663), tables: &DORIS_TABLES }),
        "AsAliasOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(664), tables: &DORIS_TABLES }),
        "AsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(666), tables: &DORIS_TABLES }),
        "AscKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(667), tables: &DORIS_TABLES }),
        "AsciiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(668), tables: &DORIS_TABLES }),
        "AsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(669), tables: &DORIS_TABLES }),
        "AssertionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(670), tables: &DORIS_TABLES }),
        "AssignmentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(671), tables: &DORIS_TABLES }),
        "AsymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(672), tables: &DORIS_TABLES }),
        "AtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(673), tables: &DORIS_TABLES }),
        "AtSignLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(674), tables: &DORIS_TABLES }),
        "AtomicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(675), tables: &DORIS_TABLES }),
        "AttributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(676), tables: &DORIS_TABLES }),
        "AttributesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(677), tables: &DORIS_TABLES }),
        "AuditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(678), tables: &DORIS_TABLES }),
        "AuthenticationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(679), tables: &DORIS_TABLES }),
        "AuthorizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(680), tables: &DORIS_TABLES }),
        "AuthorsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(681), tables: &DORIS_TABLES }),
        "AutoIncrementGrammar" => Some(RootGrammar { grammar_id: GrammarId(682), tables: &DORIS_TABLES }),
        "AutoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(684), tables: &DORIS_TABLES }),
        "Auto_incrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(685), tables: &DORIS_TABLES }),
        "Autoextend_sizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(686), tables: &DORIS_TABLES }),
        "AvgKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(687), tables: &DORIS_TABLES }),
        "Avg_row_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(688), tables: &DORIS_TABLES }),
        "BackendKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(689), tables: &DORIS_TABLES }),
        "BackendsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(690), tables: &DORIS_TABLES }),
        "BackupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(691), tables: &DORIS_TABLES }),
        "BackwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(692), tables: &DORIS_TABLES }),
        "BareFunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(693), tables: &DORIS_TABLES }),
        "BaseExpressionElementGrammar" => Some(RootGrammar { grammar_id: GrammarId(694), tables: &DORIS_TABLES }),
        "BaseFileSegment" => Some(RootGrammar { grammar_id: GrammarId(709), tables: &DORIS_TABLES }),
        "BaseSegment" => Some(RootGrammar { grammar_id: GrammarId(710), tables: &DORIS_TABLES }),
        "BeforeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(711), tables: &DORIS_TABLES }),
        "BeginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(712), tables: &DORIS_TABLES }),
        "BelongKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(713), tables: &DORIS_TABLES }),
        "BernoulliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(714), tables: &DORIS_TABLES }),
        "BetweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(715), tables: &DORIS_TABLES }),
        "BigintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(716), tables: &DORIS_TABLES }),
        "BinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(717), tables: &DORIS_TABLES }),
        "BinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(718), tables: &DORIS_TABLES }),
        "BinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(719), tables: &DORIS_TABLES }),
        "BinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(726), tables: &DORIS_TABLES }),
        "BindingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(727), tables: &DORIS_TABLES }),
        "BinlogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(728), tables: &DORIS_TABLES }),
        "BitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(729), tables: &DORIS_TABLES }),
        "Bit_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(730), tables: &DORIS_TABLES }),
        "BitandKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(731), tables: &DORIS_TABLES }),
        "BitmapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(732), tables: &DORIS_TABLES }),
        "Bitmap_emptyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(733), tables: &DORIS_TABLES }),
        "Bitmap_unionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(734), tables: &DORIS_TABLES }),
        "BitorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(735), tables: &DORIS_TABLES }),
        "BitvarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(736), tables: &DORIS_TABLES }),
        "BitwiseAndSegment" => Some(RootGrammar { grammar_id: GrammarId(737), tables: &DORIS_TABLES }),
        "BitwiseLShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(738), tables: &DORIS_TABLES }),
        "BitwiseOrSegment" => Some(RootGrammar { grammar_id: GrammarId(741), tables: &DORIS_TABLES }),
        "BitwiseRShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(742), tables: &DORIS_TABLES }),
        "BitwiseXorSegment" => Some(RootGrammar { grammar_id: GrammarId(745), tables: &DORIS_TABLES }),
        "BitxorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(746), tables: &DORIS_TABLES }),
        "BlobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(747), tables: &DORIS_TABLES }),
        "BlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(748), tables: &DORIS_TABLES }),
        "Bloom_filterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(749), tables: &DORIS_TABLES }),
        "BoolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(750), tables: &DORIS_TABLES }),
        "BooleanBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(751), tables: &DORIS_TABLES }),
        "BooleanDynamicSystemVariablesGrammar" => Some(RootGrammar { grammar_id: GrammarId(754), tables: &DORIS_TABLES }),
        "BooleanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(761), tables: &DORIS_TABLES }),
        "BooleanLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(762), tables: &DORIS_TABLES }),
        "BothKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(765), tables: &DORIS_TABLES }),
        "BracketedArguments" => Some(RootGrammar { grammar_id: GrammarId(766), tables: &DORIS_TABLES }),
        "BracketedColumnReferenceListGrammar" => Some(RootGrammar { grammar_id: GrammarId(769), tables: &DORIS_TABLES }),
        "BracketedKeyPartListGrammar" => Some(RootGrammar { grammar_id: GrammarId(772), tables: &DORIS_TABLES }),
        "BracketedSegment" => Some(RootGrammar { grammar_id: GrammarId(785), tables: &DORIS_TABLES }),
        "BracketedSetExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(786), tables: &DORIS_TABLES }),
        "BranchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(788), tables: &DORIS_TABLES }),
        "BreadthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(789), tables: &DORIS_TABLES }),
        "BreakKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(790), tables: &DORIS_TABLES }),
        "BriefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(791), tables: &DORIS_TABLES }),
        "BrokerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(792), tables: &DORIS_TABLES }),
        "BrowseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(793), tables: &DORIS_TABLES }),
        "BtreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(794), tables: &DORIS_TABLES }),
        "BucketsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(795), tables: &DORIS_TABLES }),
        "BuildKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(796), tables: &DORIS_TABLES }),
        "BuiltinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(797), tables: &DORIS_TABLES }),
        "BulkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(798), tables: &DORIS_TABLES }),
        "ByKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(799), tables: &DORIS_TABLES }),
        "ByteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(800), tables: &DORIS_TABLES }),
        "CTEColumnList" => Some(RootGrammar { grammar_id: GrammarId(801), tables: &DORIS_TABLES }),
        "CTEDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(803), tables: &DORIS_TABLES }),
        "CacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(809), tables: &DORIS_TABLES }),
        "CachedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(810), tables: &DORIS_TABLES }),
        "CallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(811), tables: &DORIS_TABLES }),
        "CallStoredProcedureSegment" => Some(RootGrammar { grammar_id: GrammarId(812), tables: &DORIS_TABLES }),
        "CalledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(815), tables: &DORIS_TABLES }),
        "CancelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(816), tables: &DORIS_TABLES }),
        "CardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(817), tables: &DORIS_TABLES }),
        "CascadeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(818), tables: &DORIS_TABLES }),
        "CascadedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(819), tables: &DORIS_TABLES }),
        "CaseExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(820), tables: &DORIS_TABLES }),
        "CaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(844), tables: &DORIS_TABLES }),
        "CastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(845), tables: &DORIS_TABLES }),
        "CastOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(846), tables: &DORIS_TABLES }),
        "CatalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(847), tables: &DORIS_TABLES }),
        "Catalog_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(848), tables: &DORIS_TABLES }),
        "CatalogsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(849), tables: &DORIS_TABLES }),
        "CeilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(850), tables: &DORIS_TABLES }),
        "CeilingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(851), tables: &DORIS_TABLES }),
        "ChainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(852), tables: &DORIS_TABLES }),
        "Challenge_responseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(853), tables: &DORIS_TABLES }),
        "ChangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(854), tables: &DORIS_TABLES }),
        "ChangedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(855), tables: &DORIS_TABLES }),
        "ChannelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(856), tables: &DORIS_TABLES }),
        "CharCharacterSetGrammar" => Some(RootGrammar { grammar_id: GrammarId(857), tables: &DORIS_TABLES }),
        "CharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(858), tables: &DORIS_TABLES }),
        "Char_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(859), tables: &DORIS_TABLES }),
        "CharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(860), tables: &DORIS_TABLES }),
        "Character_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(861), tables: &DORIS_TABLES }),
        "Character_set_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(862), tables: &DORIS_TABLES }),
        "Character_set_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(863), tables: &DORIS_TABLES }),
        "Character_set_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(864), tables: &DORIS_TABLES }),
        "CharacteristicStatement" => Some(RootGrammar { grammar_id: GrammarId(865), tables: &DORIS_TABLES }),
        "CharacteristicsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(895), tables: &DORIS_TABLES }),
        "CharactersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(896), tables: &DORIS_TABLES }),
        "CharsetGrammar" => Some(RootGrammar { grammar_id: GrammarId(897), tables: &DORIS_TABLES }),
        "CharsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(902), tables: &DORIS_TABLES }),
        "CheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(903), tables: &DORIS_TABLES }),
        "CheckTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(904), tables: &DORIS_TABLES }),
        "CheckedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(918), tables: &DORIS_TABLES }),
        "CheckpointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(919), tables: &DORIS_TABLES }),
        "ChecksumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(920), tables: &DORIS_TABLES }),
        "ChecksumTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(921), tables: &DORIS_TABLES }),
        "CipherKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(929), tables: &DORIS_TABLES }),
        "ClassKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(930), tables: &DORIS_TABLES }),
        "Class_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(931), tables: &DORIS_TABLES }),
        "CleanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(932), tables: &DORIS_TABLES }),
        "ClientKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(933), tables: &DORIS_TABLES }),
        "ClobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(934), tables: &DORIS_TABLES }),
        "CloneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(935), tables: &DORIS_TABLES }),
        "CloseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(936), tables: &DORIS_TABLES }),
        "ClusterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(937), tables: &DORIS_TABLES }),
        "ClusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(938), tables: &DORIS_TABLES }),
        "ClustersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(939), tables: &DORIS_TABLES }),
        "CoalesceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(940), tables: &DORIS_TABLES }),
        "CobolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(941), tables: &DORIS_TABLES }),
        "CodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(942), tables: &DORIS_TABLES }),
        "CodeSegment" => Some(RootGrammar { grammar_id: GrammarId(943), tables: &DORIS_TABLES }),
        "CollateGrammar" => Some(RootGrammar { grammar_id: GrammarId(944), tables: &DORIS_TABLES }),
        "CollateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(947), tables: &DORIS_TABLES }),
        "CollationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(948), tables: &DORIS_TABLES }),
        "CollationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(949), tables: &DORIS_TABLES }),
        "Collation_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(955), tables: &DORIS_TABLES }),
        "Collation_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(956), tables: &DORIS_TABLES }),
        "Collation_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(957), tables: &DORIS_TABLES }),
        "CollectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(958), tables: &DORIS_TABLES }),
        "ColocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(959), tables: &DORIS_TABLES }),
        "ColonDelimiterSegment" => Some(RootGrammar { grammar_id: GrammarId(960), tables: &DORIS_TABLES }),
        "ColonPrefixSegment" => Some(RootGrammar { grammar_id: GrammarId(961), tables: &DORIS_TABLES }),
        "ColonSegment" => Some(RootGrammar { grammar_id: GrammarId(962), tables: &DORIS_TABLES }),
        "ColumnConstraintDefaultGrammar" => Some(RootGrammar { grammar_id: GrammarId(963), tables: &DORIS_TABLES }),
        "ColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(970), tables: &DORIS_TABLES }),
        "ColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(1033), tables: &DORIS_TABLES }),
        "ColumnGeneratedGrammar" => Some(RootGrammar { grammar_id: GrammarId(1076), tables: &DORIS_TABLES }),
        "ColumnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1077), tables: &DORIS_TABLES }),
        "ColumnPathOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1078), tables: &DORIS_TABLES }),
        "ColumnReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1079), tables: &DORIS_TABLES }),
        "Column_formatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1083), tables: &DORIS_TABLES }),
        "Column_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1084), tables: &DORIS_TABLES }),
        "ColumnsExpressionFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1085), tables: &DORIS_TABLES }),
        "ColumnsExpressionFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1086), tables: &DORIS_TABLES }),
        "ColumnsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1087), tables: &DORIS_TABLES }),
        "ColumnsExpressionNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(1088), tables: &DORIS_TABLES }),
        "ColumnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1089), tables: &DORIS_TABLES }),
        "CommaSegment" => Some(RootGrammar { grammar_id: GrammarId(1090), tables: &DORIS_TABLES }),
        "Command_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1091), tables: &DORIS_TABLES }),
        "Command_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1092), tables: &DORIS_TABLES }),
        "CommentClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1093), tables: &DORIS_TABLES }),
        "CommentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1096), tables: &DORIS_TABLES }),
        "CommentSegment" => Some(RootGrammar { grammar_id: GrammarId(1097), tables: &DORIS_TABLES }),
        "Comment_startKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1098), tables: &DORIS_TABLES }),
        "CommitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1099), tables: &DORIS_TABLES }),
        "CommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1100), tables: &DORIS_TABLES }),
        "CompactKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1101), tables: &DORIS_TABLES }),
        "ComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1102), tables: &DORIS_TABLES }),
        "ComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1112), tables: &DORIS_TABLES }),
        "CompleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1113), tables: &DORIS_TABLES }),
        "CompletionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1114), tables: &DORIS_TABLES }),
        "ComponentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1115), tables: &DORIS_TABLES }),
        "CompositeBinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1116), tables: &DORIS_TABLES }),
        "CompositeComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1117), tables: &DORIS_TABLES }),
        "CompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1118), tables: &DORIS_TABLES }),
        "Compress_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1119), tables: &DORIS_TABLES }),
        "CompressedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1120), tables: &DORIS_TABLES }),
        "CompressionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1121), tables: &DORIS_TABLES }),
        "ComputeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1122), tables: &DORIS_TABLES }),
        "ConcatSegment" => Some(RootGrammar { grammar_id: GrammarId(1123), tables: &DORIS_TABLES }),
        "ConcurrentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1126), tables: &DORIS_TABLES }),
        "ConditionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1127), tables: &DORIS_TABLES }),
        "Condition_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1128), tables: &DORIS_TABLES }),
        "ConditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1129), tables: &DORIS_TABLES }),
        "ConditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1130), tables: &DORIS_TABLES }),
        "ConditionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1134), tables: &DORIS_TABLES }),
        "ConfigKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1135), tables: &DORIS_TABLES }),
        "ConnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1136), tables: &DORIS_TABLES }),
        "ConnectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1137), tables: &DORIS_TABLES }),
        "Connection_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1138), tables: &DORIS_TABLES }),
        "Connection_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1139), tables: &DORIS_TABLES }),
        "ConsistentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1140), tables: &DORIS_TABLES }),
        "ConstraintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1141), tables: &DORIS_TABLES }),
        "Constraint_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1142), tables: &DORIS_TABLES }),
        "Constraint_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1143), tables: &DORIS_TABLES }),
        "Constraint_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1144), tables: &DORIS_TABLES }),
        "ConstraintsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1145), tables: &DORIS_TABLES }),
        "ConstructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1146), tables: &DORIS_TABLES }),
        "ContainsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1147), tables: &DORIS_TABLES }),
        "ContainstableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1148), tables: &DORIS_TABLES }),
        "ContextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1149), tables: &DORIS_TABLES }),
        "ContinueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1150), tables: &DORIS_TABLES }),
        "ConversionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1151), tables: &DORIS_TABLES }),
        "ConvertFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1152), tables: &DORIS_TABLES }),
        "ConvertFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1163), tables: &DORIS_TABLES }),
        "ConvertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1164), tables: &DORIS_TABLES }),
        "Convert_lscKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1165), tables: &DORIS_TABLES }),
        "CopyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1166), tables: &DORIS_TABLES }),
        "CorrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1167), tables: &DORIS_TABLES }),
        "CorrespondingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1168), tables: &DORIS_TABLES }),
        "CountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1169), tables: &DORIS_TABLES }),
        "Covar_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1170), tables: &DORIS_TABLES }),
        "Covar_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1171), tables: &DORIS_TABLES }),
        "CpuKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1172), tables: &DORIS_TABLES }),
        "CreateCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1173), tables: &DORIS_TABLES }),
        "CreateDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1200), tables: &DORIS_TABLES }),
        "CreateEventStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1209), tables: &DORIS_TABLES }),
        "CreateFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1246), tables: &DORIS_TABLES }),
        "CreateIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1258), tables: &DORIS_TABLES }),
        "CreateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1289), tables: &DORIS_TABLES }),
        "CreateModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1290), tables: &DORIS_TABLES }),
        "CreateOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(1310), tables: &DORIS_TABLES }),
        "CreateProcedureStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1328), tables: &DORIS_TABLES }),
        "CreateRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1338), tables: &DORIS_TABLES }),
        "CreateSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1343), tables: &DORIS_TABLES }),
        "CreateSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(1348), tables: &DORIS_TABLES }),
        "CreateSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1368), tables: &DORIS_TABLES }),
        "CreateTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1374), tables: &DORIS_TABLES }),
        "CreateTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1447), tables: &DORIS_TABLES }),
        "CreateUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1478), tables: &DORIS_TABLES }),
        "CreateViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1617), tables: &DORIS_TABLES }),
        "CreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1642), tables: &DORIS_TABLES }),
        "CreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1643), tables: &DORIS_TABLES }),
        "CreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1644), tables: &DORIS_TABLES }),
        "CreationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1645), tables: &DORIS_TABLES }),
        "CronKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1646), tables: &DORIS_TABLES }),
        "CrossKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1647), tables: &DORIS_TABLES }),
        "CsvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1648), tables: &DORIS_TABLES }),
        "CubeFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1649), tables: &DORIS_TABLES }),
        "CubeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1650), tables: &DORIS_TABLES }),
        "CubeRollupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1651), tables: &DORIS_TABLES }),
        "Cume_distKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1657), tables: &DORIS_TABLES }),
        "CurrentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1658), tables: &DORIS_TABLES }),
        "CurrentTimestampLikeFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1659), tables: &DORIS_TABLES }),
        "CurrentTimestampLikeFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1661), tables: &DORIS_TABLES }),
        "CurrentTimestampLikeFunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(1666), tables: &DORIS_TABLES }),
        "Current_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1669), tables: &DORIS_TABLES }),
        "Current_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1670), tables: &DORIS_TABLES }),
        "Current_default_transform_groupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1671), tables: &DORIS_TABLES }),
        "Current_pathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1672), tables: &DORIS_TABLES }),
        "Current_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1673), tables: &DORIS_TABLES }),
        "Current_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1674), tables: &DORIS_TABLES }),
        "Current_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1675), tables: &DORIS_TABLES }),
        "Current_transform_group_for_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1676), tables: &DORIS_TABLES }),
        "Current_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1677), tables: &DORIS_TABLES }),
        "CursorFetchSegment" => Some(RootGrammar { grammar_id: GrammarId(1678), tables: &DORIS_TABLES }),
        "CursorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1689), tables: &DORIS_TABLES }),
        "CursorOpenCloseSegment" => Some(RootGrammar { grammar_id: GrammarId(1690), tables: &DORIS_TABLES }),
        "Cursor_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1697), tables: &DORIS_TABLES }),
        "CycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1698), tables: &DORIS_TABLES }),
        "DataKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1699), tables: &DORIS_TABLES }),
        "DatabaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1700), tables: &DORIS_TABLES }),
        "DatabaseReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1079), tables: &DORIS_TABLES }),
        "DatabasesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1701), tables: &DORIS_TABLES }),
        "DatafileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1702), tables: &DORIS_TABLES }),
        "DatatypeIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1703), tables: &DORIS_TABLES }),
        "DatatypeSegment" => Some(RootGrammar { grammar_id: GrammarId(1707), tables: &DORIS_TABLES }),
        "DateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1746), tables: &DORIS_TABLES }),
        "DatePartFunctionName" => Some(RootGrammar { grammar_id: GrammarId(1747), tables: &DORIS_TABLES }),
        "DatePartFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1748), tables: &DORIS_TABLES }),
        "DateTimeFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1749), tables: &DORIS_TABLES }),
        "DateTimeLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(1755), tables: &DORIS_TABLES }),
        "DatetimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1763), tables: &DORIS_TABLES }),
        "DatetimeUnitSegment" => Some(RootGrammar { grammar_id: GrammarId(1764), tables: &DORIS_TABLES }),
        "Datetime_interval_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1765), tables: &DORIS_TABLES }),
        "Datetime_interval_precisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1766), tables: &DORIS_TABLES }),
        "Datetimev1KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1767), tables: &DORIS_TABLES }),
        "Datetimev2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1768), tables: &DORIS_TABLES }),
        "Datev1KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1769), tables: &DORIS_TABLES }),
        "Datev2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1770), tables: &DORIS_TABLES }),
        "DayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1771), tables: &DORIS_TABLES }),
        "Day_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1772), tables: &DORIS_TABLES }),
        "Day_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1773), tables: &DORIS_TABLES }),
        "Day_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1774), tables: &DORIS_TABLES }),
        "Day_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1775), tables: &DORIS_TABLES }),
        "DayofmonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1776), tables: &DORIS_TABLES }),
        "DayofweekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1777), tables: &DORIS_TABLES }),
        "DayofyearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1778), tables: &DORIS_TABLES }),
        "DaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1779), tables: &DORIS_TABLES }),
        "DbccKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1780), tables: &DORIS_TABLES }),
        "DeallocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1781), tables: &DORIS_TABLES }),
        "DeallocateSegment" => Some(RootGrammar { grammar_id: GrammarId(1782), tables: &DORIS_TABLES }),
        "DecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1789), tables: &DORIS_TABLES }),
        "DecimalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1790), tables: &DORIS_TABLES }),
        "Decimalv2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1791), tables: &DORIS_TABLES }),
        "Decimalv3KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1792), tables: &DORIS_TABLES }),
        "DeclareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1793), tables: &DORIS_TABLES }),
        "DeclareStatement" => Some(RootGrammar { grammar_id: GrammarId(1794), tables: &DORIS_TABLES }),
        "DecommissionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1843), tables: &DORIS_TABLES }),
        "Dedent" => Some(RootGrammar { grammar_id: GrammarId(239), tables: &DORIS_TABLES }),
        "DefaultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1844), tables: &DORIS_TABLES }),
        "DefaultValuesGrammar" => Some(RootGrammar { grammar_id: GrammarId(1845), tables: &DORIS_TABLES }),
        "Default_authKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1848), tables: &DORIS_TABLES }),
        "DefaultsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1849), tables: &DORIS_TABLES }),
        "DeferrableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1850), tables: &DORIS_TABLES }),
        "DeferredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1851), tables: &DORIS_TABLES }),
        "DefinedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1852), tables: &DORIS_TABLES }),
        "DefinerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1853), tables: &DORIS_TABLES }),
        "DefinerSegment" => Some(RootGrammar { grammar_id: GrammarId(1854), tables: &DORIS_TABLES }),
        "DefinitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1858), tables: &DORIS_TABLES }),
        "DegreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1859), tables: &DORIS_TABLES }),
        "Delay_key_writeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1860), tables: &DORIS_TABLES }),
        "DelayedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1861), tables: &DORIS_TABLES }),
        "DeleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1862), tables: &DORIS_TABLES }),
        "DeleteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1863), tables: &DORIS_TABLES }),
        "DeleteTargetTableSegment" => Some(RootGrammar { grammar_id: GrammarId(1888), tables: &DORIS_TABLES }),
        "DeleteUsingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1893), tables: &DORIS_TABLES }),
        "DelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(1897), tables: &DORIS_TABLES }),
        "DelimiterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1900), tables: &DORIS_TABLES }),
        "DelimiterStatement" => Some(RootGrammar { grammar_id: GrammarId(1901), tables: &DORIS_TABLES }),
        "DelimitersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1902), tables: &DORIS_TABLES }),
        "DemandKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1903), tables: &DORIS_TABLES }),
        "Dense_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1904), tables: &DORIS_TABLES }),
        "DenyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1905), tables: &DORIS_TABLES }),
        "DepthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1906), tables: &DORIS_TABLES }),
        "DerefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1907), tables: &DORIS_TABLES }),
        "DerivedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1908), tables: &DORIS_TABLES }),
        "Des_key_fileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1909), tables: &DORIS_TABLES }),
        "DescKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1910), tables: &DORIS_TABLES }),
        "DescribeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1911), tables: &DORIS_TABLES }),
        "DescribeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1912), tables: &DORIS_TABLES }),
        "DescriptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1916), tables: &DORIS_TABLES }),
        "DescriptorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1917), tables: &DORIS_TABLES }),
        "DestroyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1918), tables: &DORIS_TABLES }),
        "DestructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1919), tables: &DORIS_TABLES }),
        "DeterministicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1920), tables: &DORIS_TABLES }),
        "DiagnoseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1921), tables: &DORIS_TABLES }),
        "DiagnosisKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1922), tables: &DORIS_TABLES }),
        "DiagnosticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1923), tables: &DORIS_TABLES }),
        "DictionariesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1924), tables: &DORIS_TABLES }),
        "DictionaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1925), tables: &DORIS_TABLES }),
        "DirectoryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1926), tables: &DORIS_TABLES }),
        "DisableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1927), tables: &DORIS_TABLES }),
        "DiscardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1928), tables: &DORIS_TABLES }),
        "DisconnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1929), tables: &DORIS_TABLES }),
        "DiskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1930), tables: &DORIS_TABLES }),
        "DispatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1931), tables: &DORIS_TABLES }),
        "DistinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1932), tables: &DORIS_TABLES }),
        "DistinctpcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1933), tables: &DORIS_TABLES }),
        "DistinctpcsaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1934), tables: &DORIS_TABLES }),
        "DistinctrowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1935), tables: &DORIS_TABLES }),
        "DistributedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1936), tables: &DORIS_TABLES }),
        "DistributionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1937), tables: &DORIS_TABLES }),
        "DistributionSegment" => Some(RootGrammar { grammar_id: GrammarId(1938), tables: &DORIS_TABLES }),
        "DivKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1959), tables: &DORIS_TABLES }),
        "DivOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1960), tables: &DORIS_TABLES }),
        "DivideSegment" => Some(RootGrammar { grammar_id: GrammarId(1961), tables: &DORIS_TABLES }),
        "DoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1962), tables: &DORIS_TABLES }),
        "DomainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1963), tables: &DORIS_TABLES }),
        "Doris_internal_table_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1964), tables: &DORIS_TABLES }),
        "DotSegment" => Some(RootGrammar { grammar_id: GrammarId(1965), tables: &DORIS_TABLES }),
        "DoubleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1966), tables: &DORIS_TABLES }),
        "DoubleQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1967), tables: &DORIS_TABLES }),
        "DoubleQuotedJSONPath" => Some(RootGrammar { grammar_id: GrammarId(1968), tables: &DORIS_TABLES }),
        "DoubleQuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(1969), tables: &DORIS_TABLES }),
        "DropBehaviorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1970), tables: &DORIS_TABLES }),
        "DropCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1973), tables: &DORIS_TABLES }),
        "DropDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1981), tables: &DORIS_TABLES }),
        "DropEventStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1987), tables: &DORIS_TABLES }),
        "DropFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1992), tables: &DORIS_TABLES }),
        "DropIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1997), tables: &DORIS_TABLES }),
        "DropKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2019), tables: &DORIS_TABLES }),
        "DropModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2020), tables: &DORIS_TABLES }),
        "DropProcedureStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2025), tables: &DORIS_TABLES }),
        "DropRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2032), tables: &DORIS_TABLES }),
        "DropSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2037), tables: &DORIS_TABLES }),
        "DropSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2043), tables: &DORIS_TABLES }),
        "DropTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2047), tables: &DORIS_TABLES }),
        "DropTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2054), tables: &DORIS_TABLES }),
        "DropTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2059), tables: &DORIS_TABLES }),
        "DropUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2065), tables: &DORIS_TABLES }),
        "DropViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2070), tables: &DORIS_TABLES }),
        "DroppKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2076), tables: &DORIS_TABLES }),
        "DualIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2077), tables: &DORIS_TABLES }),
        "DualKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2078), tables: &DORIS_TABLES }),
        "DummyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2079), tables: &DORIS_TABLES }),
        "DumpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2080), tables: &DORIS_TABLES }),
        "DumpfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2081), tables: &DORIS_TABLES }),
        "DuplicateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2082), tables: &DORIS_TABLES }),
        "DynamicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2083), tables: &DORIS_TABLES }),
        "Dynamic_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2084), tables: &DORIS_TABLES }),
        "Dynamic_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2085), tables: &DORIS_TABLES }),
        "EKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2086), tables: &DORIS_TABLES }),
        "EachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2087), tables: &DORIS_TABLES }),
        "ElementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2088), tables: &DORIS_TABLES }),
        "ElseClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2089), tables: &DORIS_TABLES }),
        "ElseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2092), tables: &DORIS_TABLES }),
        "ElseifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2093), tables: &DORIS_TABLES }),
        "EmptyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2094), tables: &DORIS_TABLES }),
        "EmptyStructLiteralBracketsSegment" => Some(RootGrammar { grammar_id: GrammarId(2095), tables: &DORIS_TABLES }),
        "EmptyStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2096), tables: &DORIS_TABLES }),
        "EnableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2099), tables: &DORIS_TABLES }),
        "EnclosedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2100), tables: &DORIS_TABLES }),
        "EncodingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2101), tables: &DORIS_TABLES }),
        "EncryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2102), tables: &DORIS_TABLES }),
        "EncryptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2103), tables: &DORIS_TABLES }),
        "EncryptkeyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2104), tables: &DORIS_TABLES }),
        "EncryptkeysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2105), tables: &DORIS_TABLES }),
        "End-execKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2106), tables: &DORIS_TABLES }),
        "EndBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(236), tables: &DORIS_TABLES }),
        "EndCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(2107), tables: &DORIS_TABLES }),
        "EndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2108), tables: &DORIS_TABLES }),
        "EndSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(657), tables: &DORIS_TABLES }),
        "EndsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2109), tables: &DORIS_TABLES }),
        "EnforcedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2110), tables: &DORIS_TABLES }),
        "EngineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2111), tables: &DORIS_TABLES }),
        "EngineTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2112), tables: &DORIS_TABLES }),
        "Engine_attributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2113), tables: &DORIS_TABLES }),
        "EnginesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2114), tables: &DORIS_TABLES }),
        "EnterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2115), tables: &DORIS_TABLES }),
        "EnumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2116), tables: &DORIS_TABLES }),
        "EqualsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2117), tables: &DORIS_TABLES }),
        "EqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(2118), tables: &DORIS_TABLES }),
        "ErrlvlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2119), tables: &DORIS_TABLES }),
        "ErrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2120), tables: &DORIS_TABLES }),
        "ErrorsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2121), tables: &DORIS_TABLES }),
        "EscapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2122), tables: &DORIS_TABLES }),
        "EscapedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2123), tables: &DORIS_TABLES }),
        "EventKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2124), tables: &DORIS_TABLES }),
        "EventsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2125), tables: &DORIS_TABLES }),
        "EveryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2126), tables: &DORIS_TABLES }),
        "ExceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2127), tables: &DORIS_TABLES }),
        "ExceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2128), tables: &DORIS_TABLES }),
        "ExchangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2129), tables: &DORIS_TABLES }),
        "ExcludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2130), tables: &DORIS_TABLES }),
        "ExcludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2131), tables: &DORIS_TABLES }),
        "ExclusiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2132), tables: &DORIS_TABLES }),
        "ExecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2133), tables: &DORIS_TABLES }),
        "ExecuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2134), tables: &DORIS_TABLES }),
        "ExecuteSegment" => Some(RootGrammar { grammar_id: GrammarId(2135), tables: &DORIS_TABLES }),
        "ExecutionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2142), tables: &DORIS_TABLES }),
        "ExistingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2143), tables: &DORIS_TABLES }),
        "ExistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2144), tables: &DORIS_TABLES }),
        "ExitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2145), tables: &DORIS_TABLES }),
        "ExpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2146), tables: &DORIS_TABLES }),
        "ExpansionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2147), tables: &DORIS_TABLES }),
        "ExpireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2148), tables: &DORIS_TABLES }),
        "ExpiredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2149), tables: &DORIS_TABLES }),
        "ExplainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2150), tables: &DORIS_TABLES }),
        "ExplainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2151), tables: &DORIS_TABLES }),
        "ExportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2158), tables: &DORIS_TABLES }),
        "ExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2159), tables: &DORIS_TABLES }),
        "Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2160), tables: &DORIS_TABLES }),
        "Expression_A_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2186), tables: &DORIS_TABLES }),
        "Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2193), tables: &DORIS_TABLES }),
        "Expression_B_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2203), tables: &DORIS_TABLES }),
        "Expression_C_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2208), tables: &DORIS_TABLES }),
        "Expression_D_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2225), tables: &DORIS_TABLES }),
        "Expression_D_Potential_Select_Statement_Without_Brackets" => Some(RootGrammar { grammar_id: GrammarId(2262), tables: &DORIS_TABLES }),
        "ExtendedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2272), tables: &DORIS_TABLES }),
        "ExtendedNaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2273), tables: &DORIS_TABLES }),
        "ExtensionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2274), tables: &DORIS_TABLES }),
        "ExtensionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1079), tables: &DORIS_TABLES }),
        "Extent_sizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2275), tables: &DORIS_TABLES }),
        "ExternalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2276), tables: &DORIS_TABLES }),
        "ExtractKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2277), tables: &DORIS_TABLES }),
        "FactorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2278), tables: &DORIS_TABLES }),
        "Failed_login_attemptsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2279), tables: &DORIS_TABLES }),
        "FalseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2280), tables: &DORIS_TABLES }),
        "FalseSegment" => Some(RootGrammar { grammar_id: GrammarId(2281), tables: &DORIS_TABLES }),
        "FastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2282), tables: &DORIS_TABLES }),
        "FaultsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2283), tables: &DORIS_TABLES }),
        "FeatureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2284), tables: &DORIS_TABLES }),
        "FetchClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2285), tables: &DORIS_TABLES }),
        "FetchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2302), tables: &DORIS_TABLES }),
        "FieldsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2303), tables: &DORIS_TABLES }),
        "FileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2304), tables: &DORIS_TABLES }),
        "FileSegment" => Some(RootGrammar { grammar_id: GrammarId(2305), tables: &DORIS_TABLES }),
        "File_block_sizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2314), tables: &DORIS_TABLES }),
        "FillfactorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2315), tables: &DORIS_TABLES }),
        "FilterClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(2316), tables: &DORIS_TABLES }),
        "FilterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2322), tables: &DORIS_TABLES }),
        "FinalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2323), tables: &DORIS_TABLES }),
        "FinishKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2324), tables: &DORIS_TABLES }),
        "FirstKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2325), tables: &DORIS_TABLES }),
        "First_valueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2326), tables: &DORIS_TABLES }),
        "FixedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2327), tables: &DORIS_TABLES }),
        "Float4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2328), tables: &DORIS_TABLES }),
        "Float8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2329), tables: &DORIS_TABLES }),
        "FloatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2330), tables: &DORIS_TABLES }),
        "FloorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2331), tables: &DORIS_TABLES }),
        "FlushKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2332), tables: &DORIS_TABLES }),
        "FlushStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2333), tables: &DORIS_TABLES }),
        "FollowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2388), tables: &DORIS_TABLES }),
        "FollowingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2389), tables: &DORIS_TABLES }),
        "FollowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2390), tables: &DORIS_TABLES }),
        "ForClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2391), tables: &DORIS_TABLES }),
        "ForKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2412), tables: &DORIS_TABLES }),
        "ForceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2413), tables: &DORIS_TABLES }),
        "ForeignKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(2414), tables: &DORIS_TABLES }),
        "ForeignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2417), tables: &DORIS_TABLES }),
        "FormatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2418), tables: &DORIS_TABLES }),
        "FortranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2419), tables: &DORIS_TABLES }),
        "ForwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2420), tables: &DORIS_TABLES }),
        "FoundKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2421), tables: &DORIS_TABLES }),
        "FrameClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2422), tables: &DORIS_TABLES }),
        "FrameClauseUnitGrammar" => Some(RootGrammar { grammar_id: GrammarId(2441), tables: &DORIS_TABLES }),
        "FreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2444), tables: &DORIS_TABLES }),
        "FreetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2445), tables: &DORIS_TABLES }),
        "FreetexttableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2446), tables: &DORIS_TABLES }),
        "FreezeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2447), tables: &DORIS_TABLES }),
        "FromClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2448), tables: &DORIS_TABLES }),
        "FromClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2452), tables: &DORIS_TABLES }),
        "FromExpressionElementSegment" => Some(RootGrammar { grammar_id: GrammarId(2474), tables: &DORIS_TABLES }),
        "FromExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2497), tables: &DORIS_TABLES }),
        "FromKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2524), tables: &DORIS_TABLES }),
        "FrontendKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2525), tables: &DORIS_TABLES }),
        "FrontendsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2526), tables: &DORIS_TABLES }),
        "FullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2527), tables: &DORIS_TABLES }),
        "FulltextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2528), tables: &DORIS_TABLES }),
        "FunctionContentsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2529), tables: &DORIS_TABLES }),
        "FunctionContentsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2530), tables: &DORIS_TABLES }),
        "FunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(2571), tables: &DORIS_TABLES }),
        "FunctionDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2574), tables: &DORIS_TABLES }),
        "FunctionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2575), tables: &DORIS_TABLES }),
        "FunctionNameIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2576), tables: &DORIS_TABLES }),
        "FunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2577), tables: &DORIS_TABLES }),
        "FunctionParameterGrammar" => Some(RootGrammar { grammar_id: GrammarId(2587), tables: &DORIS_TABLES }),
        "FunctionParameterListGrammar" => Some(RootGrammar { grammar_id: GrammarId(2600), tables: &DORIS_TABLES }),
        "FunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(2603), tables: &DORIS_TABLES }),
        "FunctionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2626), tables: &DORIS_TABLES }),
        "FusionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2627), tables: &DORIS_TABLES }),
        "FutureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2628), tables: &DORIS_TABLES }),
        "GKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2629), tables: &DORIS_TABLES }),
        "GeneralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2630), tables: &DORIS_TABLES }),
        "GeneratedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2631), tables: &DORIS_TABLES }),
        "GenericKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2632), tables: &DORIS_TABLES }),
        "GeomcollectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2633), tables: &DORIS_TABLES }),
        "GeometryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2634), tables: &DORIS_TABLES }),
        "GeometrycollectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2635), tables: &DORIS_TABLES }),
        "GetDiagnosticsSegment" => Some(RootGrammar { grammar_id: GrammarId(2636), tables: &DORIS_TABLES }),
        "GetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2676), tables: &DORIS_TABLES }),
        "Get_formatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2677), tables: &DORIS_TABLES }),
        "Get_master_public_keyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2678), tables: &DORIS_TABLES }),
        "Get_source_public_keyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2679), tables: &DORIS_TABLES }),
        "GlobOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2680), tables: &DORIS_TABLES }),
        "GlobalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2681), tables: &DORIS_TABLES }),
        "GoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2682), tables: &DORIS_TABLES }),
        "GotoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2683), tables: &DORIS_TABLES }),
        "GrantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2684), tables: &DORIS_TABLES }),
        "GrantStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2685), tables: &DORIS_TABLES }),
        "GrantedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2706), tables: &DORIS_TABLES }),
        "GrantsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2707), tables: &DORIS_TABLES }),
        "GraphKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2708), tables: &DORIS_TABLES }),
        "GreaterThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(2709), tables: &DORIS_TABLES }),
        "GreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2712), tables: &DORIS_TABLES }),
        "GreatestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2713), tables: &DORIS_TABLES }),
        "GroupByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2714), tables: &DORIS_TABLES }),
        "GroupByClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2731), tables: &DORIS_TABLES }),
        "GroupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2740), tables: &DORIS_TABLES }),
        "Group_replicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2741), tables: &DORIS_TABLES }),
        "GroupingExpressionList" => Some(RootGrammar { grammar_id: GrammarId(2742), tables: &DORIS_TABLES }),
        "GroupingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2750), tables: &DORIS_TABLES }),
        "GroupingSetsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2751), tables: &DORIS_TABLES }),
        "GroupsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2759), tables: &DORIS_TABLES }),
        "Gtid_onlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2760), tables: &DORIS_TABLES }),
        "HandlerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2761), tables: &DORIS_TABLES }),
        "HashKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2762), tables: &DORIS_TABLES }),
        "Hash_mapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2763), tables: &DORIS_TABLES }),
        "HavingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2764), tables: &DORIS_TABLES }),
        "HavingClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2769), tables: &DORIS_TABLES }),
        "HavingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2777), tables: &DORIS_TABLES }),
        "HdfsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2778), tables: &DORIS_TABLES }),
        "HeaderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2779), tables: &DORIS_TABLES }),
        "HeapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2780), tables: &DORIS_TABLES }),
        "HelpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2781), tables: &DORIS_TABLES }),
        "HelpStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2782), tables: &DORIS_TABLES }),
        "HierarchyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2785), tables: &DORIS_TABLES }),
        "High_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2786), tables: &DORIS_TABLES }),
        "Hint_endKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2787), tables: &DORIS_TABLES }),
        "Hint_startKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2788), tables: &DORIS_TABLES }),
        "HistogramKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2789), tables: &DORIS_TABLES }),
        "HistoryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2790), tables: &DORIS_TABLES }),
        "HllKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2791), tables: &DORIS_TABLES }),
        "Hll_unionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2792), tables: &DORIS_TABLES }),
        "HoldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2793), tables: &DORIS_TABLES }),
        "HoldlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2794), tables: &DORIS_TABLES }),
        "HorizontalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2795), tables: &DORIS_TABLES }),
        "HostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2796), tables: &DORIS_TABLES }),
        "HostnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2797), tables: &DORIS_TABLES }),
        "HostsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2798), tables: &DORIS_TABLES }),
        "HotspotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2799), tables: &DORIS_TABLES }),
        "HourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2800), tables: &DORIS_TABLES }),
        "Hour_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2801), tables: &DORIS_TABLES }),
        "Hour_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2802), tables: &DORIS_TABLES }),
        "Hour_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2803), tables: &DORIS_TABLES }),
        "HoursKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2804), tables: &DORIS_TABLES }),
        "HubKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2805), tables: &DORIS_TABLES }),
        "IdentifiedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2806), tables: &DORIS_TABLES }),
        "IdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2807), tables: &DORIS_TABLES }),
        "IdentityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2808), tables: &DORIS_TABLES }),
        "Identity_insertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2809), tables: &DORIS_TABLES }),
        "IdentitycolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2810), tables: &DORIS_TABLES }),
        "IfExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2811), tables: &DORIS_TABLES }),
        "IfExpressionStatement" => Some(RootGrammar { grammar_id: GrammarId(2814), tables: &DORIS_TABLES }),
        "IfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2831), tables: &DORIS_TABLES }),
        "IfNotExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2832), tables: &DORIS_TABLES }),
        "IgnoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2836), tables: &DORIS_TABLES }),
        "IgnoreRespectNullsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2837), tables: &DORIS_TABLES }),
        "Ignore_server_idsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2842), tables: &DORIS_TABLES }),
        "IlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2843), tables: &DORIS_TABLES }),
        "ImmediateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2844), tables: &DORIS_TABLES }),
        "ImmutableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2845), tables: &DORIS_TABLES }),
        "ImplementationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2846), tables: &DORIS_TABLES }),
        "ImplicitIndent" => Some(RootGrammar { grammar_id: GrammarId(823), tables: &DORIS_TABLES }),
        "ImplicitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2847), tables: &DORIS_TABLES }),
        "ImportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2848), tables: &DORIS_TABLES }),
        "ImportedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2849), tables: &DORIS_TABLES }),
        "InKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2850), tables: &DORIS_TABLES }),
        "InOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2851), tables: &DORIS_TABLES }),
        "InactiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2861), tables: &DORIS_TABLES }),
        "IncludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2862), tables: &DORIS_TABLES }),
        "IncludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2863), tables: &DORIS_TABLES }),
        "IncrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2864), tables: &DORIS_TABLES }),
        "IncrementalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2865), tables: &DORIS_TABLES }),
        "Indent" => Some(RootGrammar { grammar_id: GrammarId(228), tables: &DORIS_TABLES }),
        "IndexColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(2866), tables: &DORIS_TABLES }),
        "IndexColumnPrefixLengthSegment" => Some(RootGrammar { grammar_id: GrammarId(2871), tables: &DORIS_TABLES }),
        "IndexDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(2873), tables: &DORIS_TABLES }),
        "IndexHintClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2897), tables: &DORIS_TABLES }),
        "IndexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2918), tables: &DORIS_TABLES }),
        "IndexOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(2919), tables: &DORIS_TABLES }),
        "IndexReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1079), tables: &DORIS_TABLES }),
        "IndexTypeGrammar" => Some(RootGrammar { grammar_id: GrammarId(2941), tables: &DORIS_TABLES }),
        "IndexesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2946), tables: &DORIS_TABLES }),
        "IndicatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2947), tables: &DORIS_TABLES }),
        "InfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2948), tables: &DORIS_TABLES }),
        "InfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2949), tables: &DORIS_TABLES }),
        "InheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2950), tables: &DORIS_TABLES }),
        "InheritsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2951), tables: &DORIS_TABLES }),
        "InitialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2952), tables: &DORIS_TABLES }),
        "Initial_sizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2953), tables: &DORIS_TABLES }),
        "InitializeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2954), tables: &DORIS_TABLES }),
        "InitiallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2955), tables: &DORIS_TABLES }),
        "InitiateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2956), tables: &DORIS_TABLES }),
        "InlinePathOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2957), tables: &DORIS_TABLES }),
        "InnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2958), tables: &DORIS_TABLES }),
        "InoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2959), tables: &DORIS_TABLES }),
        "InplaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2960), tables: &DORIS_TABLES }),
        "InputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2961), tables: &DORIS_TABLES }),
        "InputOutputParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(2962), tables: &DORIS_TABLES }),
        "InputParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(2963), tables: &DORIS_TABLES }),
        "InsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2964), tables: &DORIS_TABLES }),
        "InsertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2965), tables: &DORIS_TABLES }),
        "InsertRowAliasSegment" => Some(RootGrammar { grammar_id: GrammarId(2966), tables: &DORIS_TABLES }),
        "InsertStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2971), tables: &DORIS_TABLES }),
        "Insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2988), tables: &DORIS_TABLES }),
        "Insert_methodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2989), tables: &DORIS_TABLES }),
        "InstallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2990), tables: &DORIS_TABLES }),
        "InstanceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2991), tables: &DORIS_TABLES }),
        "InstantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2992), tables: &DORIS_TABLES }),
        "InstantiableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2993), tables: &DORIS_TABLES }),
        "InsteadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2994), tables: &DORIS_TABLES }),
        "Int1KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2995), tables: &DORIS_TABLES }),
        "Int2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2996), tables: &DORIS_TABLES }),
        "Int3KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2997), tables: &DORIS_TABLES }),
        "Int4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2998), tables: &DORIS_TABLES }),
        "Int8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2999), tables: &DORIS_TABLES }),
        "IntKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3000), tables: &DORIS_TABLES }),
        "IntegerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3001), tables: &DORIS_TABLES }),
        "IntegrationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3002), tables: &DORIS_TABLES }),
        "IntegrationsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3003), tables: &DORIS_TABLES }),
        "IntermediateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3004), tables: &DORIS_TABLES }),
        "IntersectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3005), tables: &DORIS_TABLES }),
        "IntersectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3006), tables: &DORIS_TABLES }),
        "IntervalExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3007), tables: &DORIS_TABLES }),
        "IntervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3014), tables: &DORIS_TABLES }),
        "IntoClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3015), tables: &DORIS_TABLES }),
        "IntoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3061), tables: &DORIS_TABLES }),
        "InvertedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3062), tables: &DORIS_TABLES }),
        "InvisibleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3063), tables: &DORIS_TABLES }),
        "InvokerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3064), tables: &DORIS_TABLES }),
        "IoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3065), tables: &DORIS_TABLES }),
        "Io_threadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3066), tables: &DORIS_TABLES }),
        "Ip_trieKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3067), tables: &DORIS_TABLES }),
        "IpcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3068), tables: &DORIS_TABLES }),
        "Ipv4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3069), tables: &DORIS_TABLES }),
        "Ipv6KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3070), tables: &DORIS_TABLES }),
        "IsClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(3071), tables: &DORIS_TABLES }),
        "IsDistinctFromGrammar" => Some(RootGrammar { grammar_id: GrammarId(3077), tables: &DORIS_TABLES }),
        "IsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3082), tables: &DORIS_TABLES }),
        "IsNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(3083), tables: &DORIS_TABLES }),
        "Is_not_null_predKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3084), tables: &DORIS_TABLES }),
        "Is_null_predKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3085), tables: &DORIS_TABLES }),
        "IsamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3086), tables: &DORIS_TABLES }),
        "IsnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3087), tables: &DORIS_TABLES }),
        "IsolationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3088), tables: &DORIS_TABLES }),
        "IssuerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3089), tables: &DORIS_TABLES }),
        "IterateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3090), tables: &DORIS_TABLES }),
        "IterateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3091), tables: &DORIS_TABLES }),
        "JobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3094), tables: &DORIS_TABLES }),
        "JobsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3095), tables: &DORIS_TABLES }),
        "JoinClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3096), tables: &DORIS_TABLES }),
        "JoinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3118), tables: &DORIS_TABLES }),
        "JoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3119), tables: &DORIS_TABLES }),
        "JoinLikeClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(3121), tables: &DORIS_TABLES }),
        "JoinOnConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(3122), tables: &DORIS_TABLES }),
        "JoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3129), tables: &DORIS_TABLES }),
        "JoinUsingConditionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3137), tables: &DORIS_TABLES }),
        "JsonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3142), tables: &DORIS_TABLES }),
        "JsonTableColumnsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3143), tables: &DORIS_TABLES }),
        "JsonTableFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(3180), tables: &DORIS_TABLES }),
        "JsonTableFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3185), tables: &DORIS_TABLES }),
        "JsonValueFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(3186), tables: &DORIS_TABLES }),
        "JsonValueFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3209), tables: &DORIS_TABLES }),
        "Json_tableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3210), tables: &DORIS_TABLES }),
        "Json_valueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3211), tables: &DORIS_TABLES }),
        "JsonbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3212), tables: &DORIS_TABLES }),
        "KKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3213), tables: &DORIS_TABLES }),
        "KeyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3214), tables: &DORIS_TABLES }),
        "Key_block_sizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3215), tables: &DORIS_TABLES }),
        "Key_memberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3216), tables: &DORIS_TABLES }),
        "Key_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3217), tables: &DORIS_TABLES }),
        "KeyringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3218), tables: &DORIS_TABLES }),
        "KeysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3219), tables: &DORIS_TABLES }),
        "KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3220), tables: &DORIS_TABLES }),
        "KillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3221), tables: &DORIS_TABLES }),
        "LabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3222), tables: &DORIS_TABLES }),
        "LagKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3223), tables: &DORIS_TABLES }),
        "LancompilerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3224), tables: &DORIS_TABLES }),
        "LanguageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3225), tables: &DORIS_TABLES }),
        "LargeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3226), tables: &DORIS_TABLES }),
        "LargeintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3227), tables: &DORIS_TABLES }),
        "LastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3228), tables: &DORIS_TABLES }),
        "Last_insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3229), tables: &DORIS_TABLES }),
        "Last_valueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3230), tables: &DORIS_TABLES }),
        "LateralColumnAliasExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3231), tables: &DORIS_TABLES }),
        "LateralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3232), tables: &DORIS_TABLES }),
        "LayoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3233), tables: &DORIS_TABLES }),
        "LdapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3234), tables: &DORIS_TABLES }),
        "Ldap_admin_passwordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3235), tables: &DORIS_TABLES }),
        "LeadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3236), tables: &DORIS_TABLES }),
        "LeadingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3237), tables: &DORIS_TABLES }),
        "LeastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3238), tables: &DORIS_TABLES }),
        "LeaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3239), tables: &DORIS_TABLES }),
        "LeavesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3240), tables: &DORIS_TABLES }),
        "LeftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3241), tables: &DORIS_TABLES }),
        "Left_braceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3242), tables: &DORIS_TABLES }),
        "LengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3243), tables: &DORIS_TABLES }),
        "LessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3244), tables: &DORIS_TABLES }),
        "LessThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(3245), tables: &DORIS_TABLES }),
        "LessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(3248), tables: &DORIS_TABLES }),
        "LevelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3249), tables: &DORIS_TABLES }),
        "LikeExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3250), tables: &DORIS_TABLES }),
        "LikeGrammar" => Some(RootGrammar { grammar_id: GrammarId(3258), tables: &DORIS_TABLES }),
        "LikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3262), tables: &DORIS_TABLES }),
        "LikeOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(3263), tables: &DORIS_TABLES }),
        "LimitClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3264), tables: &DORIS_TABLES }),
        "LimitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3281), tables: &DORIS_TABLES }),
        "LinenoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3282), tables: &DORIS_TABLES }),
        "LinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3283), tables: &DORIS_TABLES }),
        "LinestringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3284), tables: &DORIS_TABLES }),
        "LinkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3285), tables: &DORIS_TABLES }),
        "ListComprehensionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3286), tables: &DORIS_TABLES }),
        "ListKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3287), tables: &DORIS_TABLES }),
        "ListPartitionDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(3288), tables: &DORIS_TABLES }),
        "ListenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3301), tables: &DORIS_TABLES }),
        "LiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(3302), tables: &DORIS_TABLES }),
        "LiteralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3314), tables: &DORIS_TABLES }),
        "LiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3315), tables: &DORIS_TABLES }),
        "LnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3316), tables: &DORIS_TABLES }),
        "LoadDataSegment" => Some(RootGrammar { grammar_id: GrammarId(3317), tables: &DORIS_TABLES }),
        "LoadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3379), tables: &DORIS_TABLES }),
        "LocalAliasSegment" => Some(RootGrammar { grammar_id: GrammarId(3380), tables: &DORIS_TABLES }),
        "LocalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3381), tables: &DORIS_TABLES }),
        "LocalVariableNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3382), tables: &DORIS_TABLES }),
        "LocaltimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3383), tables: &DORIS_TABLES }),
        "LocaltimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3384), tables: &DORIS_TABLES }),
        "LocationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3385), tables: &DORIS_TABLES }),
        "LocatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3386), tables: &DORIS_TABLES }),
        "LockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3387), tables: &DORIS_TABLES }),
        "LockedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3388), tables: &DORIS_TABLES }),
        "LocksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3389), tables: &DORIS_TABLES }),
        "LogfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3390), tables: &DORIS_TABLES }),
        "LogicalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3391), tables: &DORIS_TABLES }),
        "LoginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3392), tables: &DORIS_TABLES }),
        "LogsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3393), tables: &DORIS_TABLES }),
        "LongKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3394), tables: &DORIS_TABLES }),
        "LongblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3395), tables: &DORIS_TABLES }),
        "LongtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3396), tables: &DORIS_TABLES }),
        "LoopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3397), tables: &DORIS_TABLES }),
        "LoopStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3398), tables: &DORIS_TABLES }),
        "Low_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3410), tables: &DORIS_TABLES }),
        "LowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3411), tables: &DORIS_TABLES }),
        "MKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3412), tables: &DORIS_TABLES }),
        "MLTableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3413), tables: &DORIS_TABLES }),
        "ManageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3425), tables: &DORIS_TABLES }),
        "ManualKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3426), tables: &DORIS_TABLES }),
        "MapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3427), tables: &DORIS_TABLES }),
        "MapTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(3428), tables: &DORIS_TABLES }),
        "MaskingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3429), tables: &DORIS_TABLES }),
        "MasterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3430), tables: &DORIS_TABLES }),
        "Master_auto_positionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3431), tables: &DORIS_TABLES }),
        "Master_compression_algorithmsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3432), tables: &DORIS_TABLES }),
        "Master_connect_retryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3433), tables: &DORIS_TABLES }),
        "Master_delayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3434), tables: &DORIS_TABLES }),
        "Master_heartbeat_periodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3435), tables: &DORIS_TABLES }),
        "Master_hostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3436), tables: &DORIS_TABLES }),
        "Master_log_fileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3437), tables: &DORIS_TABLES }),
        "Master_log_posKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3438), tables: &DORIS_TABLES }),
        "Master_passwordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3439), tables: &DORIS_TABLES }),
        "Master_portKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3440), tables: &DORIS_TABLES }),
        "Master_public_key_pathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3441), tables: &DORIS_TABLES }),
        "Master_retry_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3442), tables: &DORIS_TABLES }),
        "Master_server_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3443), tables: &DORIS_TABLES }),
        "Master_sslKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3444), tables: &DORIS_TABLES }),
        "Master_ssl_caKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3445), tables: &DORIS_TABLES }),
        "Master_ssl_capathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3446), tables: &DORIS_TABLES }),
        "Master_ssl_certKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3447), tables: &DORIS_TABLES }),
        "Master_ssl_cipherKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3448), tables: &DORIS_TABLES }),
        "Master_ssl_crlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3449), tables: &DORIS_TABLES }),
        "Master_ssl_crlpathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3450), tables: &DORIS_TABLES }),
        "Master_ssl_keyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3451), tables: &DORIS_TABLES }),
        "Master_tls_ciphersuitesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3452), tables: &DORIS_TABLES }),
        "Master_tls_versionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3453), tables: &DORIS_TABLES }),
        "Master_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3454), tables: &DORIS_TABLES }),
        "Master_zstd_compression_levelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3455), tables: &DORIS_TABLES }),
        "MatchConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(3456), tables: &DORIS_TABLES }),
        "MatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3457), tables: &DORIS_TABLES }),
        "Match_allKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3458), tables: &DORIS_TABLES }),
        "Match_anyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3459), tables: &DORIS_TABLES }),
        "Match_phraseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3460), tables: &DORIS_TABLES }),
        "Match_phrase_edgeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3461), tables: &DORIS_TABLES }),
        "Match_phrase_prefixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3462), tables: &DORIS_TABLES }),
        "Match_regexpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3463), tables: &DORIS_TABLES }),
        "MatchedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3464), tables: &DORIS_TABLES }),
        "MaterializedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3465), tables: &DORIS_TABLES }),
        "MaxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3466), tables: &DORIS_TABLES }),
        "Max_connections_per_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3467), tables: &DORIS_TABLES }),
        "Max_queries_per_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3468), tables: &DORIS_TABLES }),
        "Max_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3469), tables: &DORIS_TABLES }),
        "Max_sizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3470), tables: &DORIS_TABLES }),
        "Max_updates_per_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3471), tables: &DORIS_TABLES }),
        "Max_user_connectionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3472), tables: &DORIS_TABLES }),
        "MaxextentsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3473), tables: &DORIS_TABLES }),
        "MaxvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3474), tables: &DORIS_TABLES }),
        "MediumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3475), tables: &DORIS_TABLES }),
        "MediumblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3476), tables: &DORIS_TABLES }),
        "MediumintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3477), tables: &DORIS_TABLES }),
        "MediumtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3478), tables: &DORIS_TABLES }),
        "MemberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3479), tables: &DORIS_TABLES }),
        "MemoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3480), tables: &DORIS_TABLES }),
        "MemoryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3481), tables: &DORIS_TABLES }),
        "MergeDeleteClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3482), tables: &DORIS_TABLES }),
        "MergeInsertClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3483), tables: &DORIS_TABLES }),
        "MergeIntoLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(3487), tables: &DORIS_TABLES }),
        "MergeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3490), tables: &DORIS_TABLES }),
        "MergeMatchSegment" => Some(RootGrammar { grammar_id: GrammarId(3491), tables: &DORIS_TABLES }),
        "MergeMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3494), tables: &DORIS_TABLES }),
        "MergeNotMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3504), tables: &DORIS_TABLES }),
        "MergeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3513), tables: &DORIS_TABLES }),
        "MergeUpdateClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3533), tables: &DORIS_TABLES }),
        "Message_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3536), tables: &DORIS_TABLES }),
        "Message_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3537), tables: &DORIS_TABLES }),
        "Message_textKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3538), tables: &DORIS_TABLES }),
        "MethodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3539), tables: &DORIS_TABLES }),
        "MicrosecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3540), tables: &DORIS_TABLES }),
        "MiddleintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3541), tables: &DORIS_TABLES }),
        "MigrateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3542), tables: &DORIS_TABLES }),
        "MigrationsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3543), tables: &DORIS_TABLES }),
        "MillisecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3544), tables: &DORIS_TABLES }),
        "MinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3545), tables: &DORIS_TABLES }),
        "Min_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3546), tables: &DORIS_TABLES }),
        "MinusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3547), tables: &DORIS_TABLES }),
        "MinusSegment" => Some(RootGrammar { grammar_id: GrammarId(3548), tables: &DORIS_TABLES }),
        "MinuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3549), tables: &DORIS_TABLES }),
        "Minute_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3550), tables: &DORIS_TABLES }),
        "Minute_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3551), tables: &DORIS_TABLES }),
        "MinutesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3552), tables: &DORIS_TABLES }),
        "MinvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3553), tables: &DORIS_TABLES }),
        "MlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3554), tables: &DORIS_TABLES }),
        "MlslabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3555), tables: &DORIS_TABLES }),
        "ModKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3556), tables: &DORIS_TABLES }),
        "ModOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(3557), tables: &DORIS_TABLES }),
        "ModeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3558), tables: &DORIS_TABLES }),
        "ModelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3559), tables: &DORIS_TABLES }),
        "ModifiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3560), tables: &DORIS_TABLES }),
        "ModifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3561), tables: &DORIS_TABLES }),
        "ModuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3562), tables: &DORIS_TABLES }),
        "ModuloSegment" => Some(RootGrammar { grammar_id: GrammarId(3563), tables: &DORIS_TABLES }),
        "MonitorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3564), tables: &DORIS_TABLES }),
        "MonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3565), tables: &DORIS_TABLES }),
        "MonthnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3566), tables: &DORIS_TABLES }),
        "MoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3567), tables: &DORIS_TABLES }),
        "MoveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3568), tables: &DORIS_TABLES }),
        "MtmvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3569), tables: &DORIS_TABLES }),
        "MultilinestringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3570), tables: &DORIS_TABLES }),
        "MultiplySegment" => Some(RootGrammar { grammar_id: GrammarId(3571), tables: &DORIS_TABLES }),
        "MultipointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3572), tables: &DORIS_TABLES }),
        "MultipolygonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3573), tables: &DORIS_TABLES }),
        "MultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3574), tables: &DORIS_TABLES }),
        "MumpsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3575), tables: &DORIS_TABLES }),
        "MutexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3576), tables: &DORIS_TABLES }),
        "MyisamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3577), tables: &DORIS_TABLES }),
        "Mysql_errnoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3578), tables: &DORIS_TABLES }),
        "NakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3579), tables: &DORIS_TABLES }),
        "NameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3580), tables: &DORIS_TABLES }),
        "NamedWindowExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3581), tables: &DORIS_TABLES }),
        "NamedWindowSegment" => Some(RootGrammar { grammar_id: GrammarId(3588), tables: &DORIS_TABLES }),
        "NamesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3592), tables: &DORIS_TABLES }),
        "NanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3593), tables: &DORIS_TABLES }),
        "NanLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3594), tables: &DORIS_TABLES }),
        "NationalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3595), tables: &DORIS_TABLES }),
        "NaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3596), tables: &DORIS_TABLES }),
        "NaturalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3599), tables: &DORIS_TABLES }),
        "NcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3600), tables: &DORIS_TABLES }),
        "NclobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3601), tables: &DORIS_TABLES }),
        "NdbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3602), tables: &DORIS_TABLES }),
        "NdbclusterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3603), tables: &DORIS_TABLES }),
        "NegativeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3604), tables: &DORIS_TABLES }),
        "NegativeSegment" => Some(RootGrammar { grammar_id: GrammarId(3605), tables: &DORIS_TABLES }),
        "NestedJoinGrammar" => Some(RootGrammar { grammar_id: GrammarId(3606), tables: &DORIS_TABLES }),
        "NestedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3607), tables: &DORIS_TABLES }),
        "NestingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3608), tables: &DORIS_TABLES }),
        "Network_namespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3609), tables: &DORIS_TABLES }),
        "NeverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3610), tables: &DORIS_TABLES }),
        "NewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3611), tables: &DORIS_TABLES }),
        "NewlineSegment" => Some(RootGrammar { grammar_id: GrammarId(3612), tables: &DORIS_TABLES }),
        "NextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3613), tables: &DORIS_TABLES }),
        "Ngram_bfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3614), tables: &DORIS_TABLES }),
        "NoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3615), tables: &DORIS_TABLES }),
        "No_use_mvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3616), tables: &DORIS_TABLES }),
        "No_waitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3617), tables: &DORIS_TABLES }),
        "No_write_to_binlogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3618), tables: &DORIS_TABLES }),
        "NoauditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3619), tables: &DORIS_TABLES }),
        "NocacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3620), tables: &DORIS_TABLES }),
        "NocheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3621), tables: &DORIS_TABLES }),
        "NocompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3622), tables: &DORIS_TABLES }),
        "NocopyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3623), tables: &DORIS_TABLES }),
        "NocreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3624), tables: &DORIS_TABLES }),
        "NocreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3625), tables: &DORIS_TABLES }),
        "NocreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3626), tables: &DORIS_TABLES }),
        "NocycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3627), tables: &DORIS_TABLES }),
        "NodegroupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3628), tables: &DORIS_TABLES }),
        "NoinheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3629), tables: &DORIS_TABLES }),
        "NologinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3630), tables: &DORIS_TABLES }),
        "NonSetSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(3631), tables: &DORIS_TABLES }),
        "NonStandardJoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3641), tables: &DORIS_TABLES }),
        "NonWithNonSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(3642), tables: &DORIS_TABLES }),
        "NonWithSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(3647), tables: &DORIS_TABLES }),
        "Non_nullableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3653), tables: &DORIS_TABLES }),
        "NonclusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3654), tables: &DORIS_TABLES }),
        "NoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3655), tables: &DORIS_TABLES }),
        "NoorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3656), tables: &DORIS_TABLES }),
        "NormalizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3657), tables: &DORIS_TABLES }),
        "NormalizedGrammar" => Some(RootGrammar { grammar_id: GrammarId(3658), tables: &DORIS_TABLES }),
        "NormalizedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3659), tables: &DORIS_TABLES }),
        "NosuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3660), tables: &DORIS_TABLES }),
        "NotEnforcedGrammar" => Some(RootGrammar { grammar_id: GrammarId(3661), tables: &DORIS_TABLES }),
        "NotEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(3662), tables: &DORIS_TABLES }),
        "NotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3669), tables: &DORIS_TABLES }),
        "NotNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(3670), tables: &DORIS_TABLES }),
        "NotOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3671), tables: &DORIS_TABLES }),
        "NothingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3674), tables: &DORIS_TABLES }),
        "NotifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3675), tables: &DORIS_TABLES }),
        "NotnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3676), tables: &DORIS_TABLES }),
        "NowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3677), tables: &DORIS_TABLES }),
        "NowaitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3678), tables: &DORIS_TABLES }),
        "Nth_valueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3679), tables: &DORIS_TABLES }),
        "NtileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3680), tables: &DORIS_TABLES }),
        "NullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3681), tables: &DORIS_TABLES }),
        "NullLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3682), tables: &DORIS_TABLES }),
        "NullSafeEqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(3683), tables: &DORIS_TABLES }),
        "NullableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3687), tables: &DORIS_TABLES }),
        "NullifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3688), tables: &DORIS_TABLES }),
        "NullsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3689), tables: &DORIS_TABLES }),
        "NumberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3690), tables: &DORIS_TABLES }),
        "NumericKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3691), tables: &DORIS_TABLES }),
        "NumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3692), tables: &DORIS_TABLES }),
        "NvarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3693), tables: &DORIS_TABLES }),
        "ObjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3694), tables: &DORIS_TABLES }),
        "ObjectLiteralElementSegment" => Some(RootGrammar { grammar_id: GrammarId(3695), tables: &DORIS_TABLES }),
        "ObjectLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3699), tables: &DORIS_TABLES }),
        "ObjectReferenceDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(3703), tables: &DORIS_TABLES }),
        "ObjectReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1079), tables: &DORIS_TABLES }),
        "ObjectReferenceTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3708), tables: &DORIS_TABLES }),
        "ObjectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3720), tables: &DORIS_TABLES }),
        "ObserverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3721), tables: &DORIS_TABLES }),
        "Octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3722), tables: &DORIS_TABLES }),
        "OctetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3723), tables: &DORIS_TABLES }),
        "OfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3724), tables: &DORIS_TABLES }),
        "OffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3725), tables: &DORIS_TABLES }),
        "OfflineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3726), tables: &DORIS_TABLES }),
        "OffsetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3727), tables: &DORIS_TABLES }),
        "OffsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3736), tables: &DORIS_TABLES }),
        "OffsetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3737), tables: &DORIS_TABLES }),
        "OidsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3738), tables: &DORIS_TABLES }),
        "OjKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3739), tables: &DORIS_TABLES }),
        "OldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3740), tables: &DORIS_TABLES }),
        "OnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3741), tables: &DORIS_TABLES }),
        "OneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3742), tables: &DORIS_TABLES }),
        "OnlineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3743), tables: &DORIS_TABLES }),
        "OnlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3744), tables: &DORIS_TABLES }),
        "OpenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3745), tables: &DORIS_TABLES }),
        "OpendatasourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3746), tables: &DORIS_TABLES }),
        "OpenqueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3747), tables: &DORIS_TABLES }),
        "OpenrowsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3748), tables: &DORIS_TABLES }),
        "OpenxmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3749), tables: &DORIS_TABLES }),
        "OperateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3750), tables: &DORIS_TABLES }),
        "OperationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3751), tables: &DORIS_TABLES }),
        "OperatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3752), tables: &DORIS_TABLES }),
        "OptimizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3753), tables: &DORIS_TABLES }),
        "OptimizeTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3754), tables: &DORIS_TABLES }),
        "OptimizedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3762), tables: &DORIS_TABLES }),
        "OptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3763), tables: &DORIS_TABLES }),
        "OptionalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3764), tables: &DORIS_TABLES }),
        "OptionallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3765), tables: &DORIS_TABLES }),
        "OptionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3766), tables: &DORIS_TABLES }),
        "OrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3767), tables: &DORIS_TABLES }),
        "OrOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3768), tables: &DORIS_TABLES }),
        "OrReplaceGrammar" => Some(RootGrammar { grammar_id: GrammarId(3772), tables: &DORIS_TABLES }),
        "OrderByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3775), tables: &DORIS_TABLES }),
        "OrderByClauseTerminators" => Some(RootGrammar { grammar_id: GrammarId(3795), tables: &DORIS_TABLES }),
        "OrderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3803), tables: &DORIS_TABLES }),
        "OrderNoOrderGrammar" => Some(RootGrammar { grammar_id: GrammarId(3804), tables: &DORIS_TABLES }),
        "OrderingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3807), tables: &DORIS_TABLES }),
        "OrdinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3808), tables: &DORIS_TABLES }),
        "OrganizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3809), tables: &DORIS_TABLES }),
        "OthersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3810), tables: &DORIS_TABLES }),
        "OutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3811), tables: &DORIS_TABLES }),
        "OuterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3812), tables: &DORIS_TABLES }),
        "OutfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3813), tables: &DORIS_TABLES }),
        "OutputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3814), tables: &DORIS_TABLES }),
        "OutputParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(3815), tables: &DORIS_TABLES }),
        "OverClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3816), tables: &DORIS_TABLES }),
        "OverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3823), tables: &DORIS_TABLES }),
        "OverlapsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3824), tables: &DORIS_TABLES }),
        "OverlapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3833), tables: &DORIS_TABLES }),
        "OverlayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3834), tables: &DORIS_TABLES }),
        "OverridingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3835), tables: &DORIS_TABLES }),
        "OverwriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3836), tables: &DORIS_TABLES }),
        "OwnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3837), tables: &DORIS_TABLES }),
        "OwnershipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3838), tables: &DORIS_TABLES }),
        "Pack_keysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3839), tables: &DORIS_TABLES }),
        "PadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3840), tables: &DORIS_TABLES }),
        "PageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3841), tables: &DORIS_TABLES }),
        "ParameterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3842), tables: &DORIS_TABLES }),
        "ParameterNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3843), tables: &DORIS_TABLES }),
        "ParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(3844), tables: &DORIS_TABLES }),
        "Parameter_modeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3845), tables: &DORIS_TABLES }),
        "Parameter_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3846), tables: &DORIS_TABLES }),
        "Parameter_ordinal_positionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3847), tables: &DORIS_TABLES }),
        "Parameter_specific_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3848), tables: &DORIS_TABLES }),
        "Parameter_specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3849), tables: &DORIS_TABLES }),
        "Parameter_specific_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3850), tables: &DORIS_TABLES }),
        "ParametersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3851), tables: &DORIS_TABLES }),
        "Parse_gcol_exprKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3852), tables: &DORIS_TABLES }),
        "ParsedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3853), tables: &DORIS_TABLES }),
        "ParserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3854), tables: &DORIS_TABLES }),
        "PartialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3855), tables: &DORIS_TABLES }),
        "PartitionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3856), tables: &DORIS_TABLES }),
        "PartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3863), tables: &DORIS_TABLES }),
        "PartitionSegment" => Some(RootGrammar { grammar_id: GrammarId(3864), tables: &DORIS_TABLES }),
        "PartitioningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3895), tables: &DORIS_TABLES }),
        "PartitionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3896), tables: &DORIS_TABLES }),
        "PascalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3897), tables: &DORIS_TABLES }),
        "PasswordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3898), tables: &DORIS_TABLES }),
        "Password_expireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3899), tables: &DORIS_TABLES }),
        "Password_historyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3900), tables: &DORIS_TABLES }),
        "Password_lock_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3901), tables: &DORIS_TABLES }),
        "Password_reuseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3902), tables: &DORIS_TABLES }),
        "PathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3903), tables: &DORIS_TABLES }),
        "PathSegment" => Some(RootGrammar { grammar_id: GrammarId(3904), tables: &DORIS_TABLES }),
        "PatternMatchingGrammar" => Some(RootGrammar { grammar_id: GrammarId(3911), tables: &DORIS_TABLES }),
        "PauseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3912), tables: &DORIS_TABLES }),
        "PctfreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3913), tables: &DORIS_TABLES }),
        "PercentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3914), tables: &DORIS_TABLES }),
        "Percent_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3915), tables: &DORIS_TABLES }),
        "Percentile_contKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3916), tables: &DORIS_TABLES }),
        "Percentile_discKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3917), tables: &DORIS_TABLES }),
        "PeriodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3918), tables: &DORIS_TABLES }),
        "PermissiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3919), tables: &DORIS_TABLES }),
        "PersistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3920), tables: &DORIS_TABLES }),
        "Persist_onlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3921), tables: &DORIS_TABLES }),
        "PhaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3922), tables: &DORIS_TABLES }),
        "PhysicalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3923), tables: &DORIS_TABLES }),
        "PiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3924), tables: &DORIS_TABLES }),
        "PipeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3925), tables: &DORIS_TABLES }),
        "PipeSegment" => Some(RootGrammar { grammar_id: GrammarId(3926), tables: &DORIS_TABLES }),
        "PlaceholderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3927), tables: &DORIS_TABLES }),
        "PlacingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3928), tables: &DORIS_TABLES }),
        "PlanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3929), tables: &DORIS_TABLES }),
        "PlayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3930), tables: &DORIS_TABLES }),
        "PliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3931), tables: &DORIS_TABLES }),
        "PluginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3932), tables: &DORIS_TABLES }),
        "Plugin_dirKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3933), tables: &DORIS_TABLES }),
        "PluginsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3934), tables: &DORIS_TABLES }),
        "PlusSegment" => Some(RootGrammar { grammar_id: GrammarId(3935), tables: &DORIS_TABLES }),
        "PointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3936), tables: &DORIS_TABLES }),
        "PolicyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3937), tables: &DORIS_TABLES }),
        "PolygonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3938), tables: &DORIS_TABLES }),
        "PortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3939), tables: &DORIS_TABLES }),
        "PositionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3940), tables: &DORIS_TABLES }),
        "PositiveSegment" => Some(RootGrammar { grammar_id: GrammarId(3941), tables: &DORIS_TABLES }),
        "PostFunctionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3942), tables: &DORIS_TABLES }),
        "PostTableExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3945), tables: &DORIS_TABLES }),
        "PostfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3948), tables: &DORIS_TABLES }),
        "PowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3949), tables: &DORIS_TABLES }),
        "PreTableFunctionKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3950), tables: &DORIS_TABLES }),
        "PrecedesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3951), tables: &DORIS_TABLES }),
        "PrecedingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3952), tables: &DORIS_TABLES }),
        "PrecisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3953), tables: &DORIS_TABLES }),
        "PrefixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3954), tables: &DORIS_TABLES }),
        "PreorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3955), tables: &DORIS_TABLES }),
        "PrepareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3956), tables: &DORIS_TABLES }),
        "PrepareSegment" => Some(RootGrammar { grammar_id: GrammarId(3957), tables: &DORIS_TABLES }),
        "PreparedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3965), tables: &DORIS_TABLES }),
        "PreserveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3966), tables: &DORIS_TABLES }),
        "PrevKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3967), tables: &DORIS_TABLES }),
        "PrimaryKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(3968), tables: &DORIS_TABLES }),
        "PrimaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3971), tables: &DORIS_TABLES }),
        "PrintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3972), tables: &DORIS_TABLES }),
        "PriorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3973), tables: &DORIS_TABLES }),
        "Privilege_checks_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3974), tables: &DORIS_TABLES }),
        "PrivilegesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3975), tables: &DORIS_TABLES }),
        "ProcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3976), tables: &DORIS_TABLES }),
        "ProceduralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3977), tables: &DORIS_TABLES }),
        "ProcedureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3978), tables: &DORIS_TABLES }),
        "ProcedureParameterGrammar" => Some(RootGrammar { grammar_id: GrammarId(3979), tables: &DORIS_TABLES }),
        "ProcedureParameterListGrammar" => Some(RootGrammar { grammar_id: GrammarId(3988), tables: &DORIS_TABLES }),
        "ProceduresKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3991), tables: &DORIS_TABLES }),
        "ProcessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3992), tables: &DORIS_TABLES }),
        "ProcesslistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3993), tables: &DORIS_TABLES }),
        "ProfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3994), tables: &DORIS_TABLES }),
        "ProfilesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3995), tables: &DORIS_TABLES }),
        "PropertiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3996), tables: &DORIS_TABLES }),
        "PropertyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3997), tables: &DORIS_TABLES }),
        "ProxyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3998), tables: &DORIS_TABLES }),
        "PublicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3999), tables: &DORIS_TABLES }),
        "PurgeBinaryLogsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4000), tables: &DORIS_TABLES }),
        "PurgeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4014), tables: &DORIS_TABLES }),
        "QualifiedNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4015), tables: &DORIS_TABLES }),
        "QualifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4018), tables: &DORIS_TABLES }),
        "QuantifiedComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4019), tables: &DORIS_TABLES }),
        "Quantile_stateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4027), tables: &DORIS_TABLES }),
        "Quantile_unionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4028), tables: &DORIS_TABLES }),
        "QuarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4029), tables: &DORIS_TABLES }),
        "QueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4030), tables: &DORIS_TABLES }),
        "QueuedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4031), tables: &DORIS_TABLES }),
        "QuickKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4032), tables: &DORIS_TABLES }),
        "QuotaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4033), tables: &DORIS_TABLES }),
        "QuoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4034), tables: &DORIS_TABLES }),
        "QuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(4035), tables: &DORIS_TABLES }),
        "QuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4036), tables: &DORIS_TABLES }),
        "Raid0KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4039), tables: &DORIS_TABLES }),
        "RaiserrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4040), tables: &DORIS_TABLES }),
        "RandomKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4041), tables: &DORIS_TABLES }),
        "RangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4042), tables: &DORIS_TABLES }),
        "RangePartitionDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(4043), tables: &DORIS_TABLES }),
        "RangePartitionIntervalSegment" => Some(RootGrammar { grammar_id: GrammarId(4073), tables: &DORIS_TABLES }),
        "RankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4089), tables: &DORIS_TABLES }),
        "RawEqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(4090), tables: &DORIS_TABLES }),
        "RawGreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(4091), tables: &DORIS_TABLES }),
        "RawKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &DORIS_TABLES }),
        "RawLessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(4093), tables: &DORIS_TABLES }),
        "RawNotSegment" => Some(RootGrammar { grammar_id: GrammarId(4094), tables: &DORIS_TABLES }),
        "ReadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4095), tables: &DORIS_TABLES }),
        "Read_onlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4096), tables: &DORIS_TABLES }),
        "ReadsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4097), tables: &DORIS_TABLES }),
        "ReadtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4098), tables: &DORIS_TABLES }),
        "RealKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4099), tables: &DORIS_TABLES }),
        "RebalanceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4100), tables: &DORIS_TABLES }),
        "RebuildKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4101), tables: &DORIS_TABLES }),
        "RecentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4102), tables: &DORIS_TABLES }),
        "RecheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4103), tables: &DORIS_TABLES }),
        "ReconfigureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4104), tables: &DORIS_TABLES }),
        "RecoverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4105), tables: &DORIS_TABLES }),
        "RecursiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4106), tables: &DORIS_TABLES }),
        "RecycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4107), tables: &DORIS_TABLES }),
        "Redo_buffer_sizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4108), tables: &DORIS_TABLES }),
        "RedofileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4109), tables: &DORIS_TABLES }),
        "RedundantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4110), tables: &DORIS_TABLES }),
        "RefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4111), tables: &DORIS_TABLES }),
        "ReferenceDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(4112), tables: &DORIS_TABLES }),
        "ReferenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4126), tables: &DORIS_TABLES }),
        "ReferenceMatchGrammar" => Some(RootGrammar { grammar_id: GrammarId(4127), tables: &DORIS_TABLES }),
        "Reference_usageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4133), tables: &DORIS_TABLES }),
        "ReferencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4134), tables: &DORIS_TABLES }),
        "ReferencingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4135), tables: &DORIS_TABLES }),
        "ReferentialActionGrammar" => Some(RootGrammar { grammar_id: GrammarId(4136), tables: &DORIS_TABLES }),
        "RefreshKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4148), tables: &DORIS_TABLES }),
        "RegexpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4149), tables: &DORIS_TABLES }),
        "RegistrationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4150), tables: &DORIS_TABLES }),
        "Regr_avgxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4151), tables: &DORIS_TABLES }),
        "Regr_avgyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4152), tables: &DORIS_TABLES }),
        "Regr_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4153), tables: &DORIS_TABLES }),
        "Regr_interceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4154), tables: &DORIS_TABLES }),
        "Regr_r2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4155), tables: &DORIS_TABLES }),
        "Regr_slopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4156), tables: &DORIS_TABLES }),
        "Regr_sxxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4157), tables: &DORIS_TABLES }),
        "Regr_sxyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4158), tables: &DORIS_TABLES }),
        "Regr_syyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4159), tables: &DORIS_TABLES }),
        "ReindexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4160), tables: &DORIS_TABLES }),
        "RelativeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4161), tables: &DORIS_TABLES }),
        "RelayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4162), tables: &DORIS_TABLES }),
        "Relay_log_fileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4163), tables: &DORIS_TABLES }),
        "Relay_log_posKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4164), tables: &DORIS_TABLES }),
        "Relay_threadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4165), tables: &DORIS_TABLES }),
        "RelaylogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4166), tables: &DORIS_TABLES }),
        "ReleaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4167), tables: &DORIS_TABLES }),
        "ReloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4168), tables: &DORIS_TABLES }),
        "RemoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4169), tables: &DORIS_TABLES }),
        "RemoveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4170), tables: &DORIS_TABLES }),
        "RenameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4171), tables: &DORIS_TABLES }),
        "RenameTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4172), tables: &DORIS_TABLES }),
        "ReorganizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4180), tables: &DORIS_TABLES }),
        "RepairKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4181), tables: &DORIS_TABLES }),
        "RepairTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4182), tables: &DORIS_TABLES }),
        "RepeatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4194), tables: &DORIS_TABLES }),
        "RepeatStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4195), tables: &DORIS_TABLES }),
        "RepeatableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4210), tables: &DORIS_TABLES }),
        "ReplaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4211), tables: &DORIS_TABLES }),
        "ReplaceSegment" => Some(RootGrammar { grammar_id: GrammarId(4212), tables: &DORIS_TABLES }),
        "Replace_if_not_nullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4233), tables: &DORIS_TABLES }),
        "ReplayerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4234), tables: &DORIS_TABLES }),
        "ReplicaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4235), tables: &DORIS_TABLES }),
        "ReplicasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4236), tables: &DORIS_TABLES }),
        "Replicate_do_dbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4237), tables: &DORIS_TABLES }),
        "Replicate_do_tableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4238), tables: &DORIS_TABLES }),
        "Replicate_ignore_dbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4239), tables: &DORIS_TABLES }),
        "Replicate_ignore_tableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4240), tables: &DORIS_TABLES }),
        "Replicate_rewrite_dbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4241), tables: &DORIS_TABLES }),
        "Replicate_wild_do_tableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4242), tables: &DORIS_TABLES }),
        "Replicate_wild_ignore_tableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4243), tables: &DORIS_TABLES }),
        "ReplicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4244), tables: &DORIS_TABLES }),
        "RepositoriesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4245), tables: &DORIS_TABLES }),
        "RepositoryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4246), tables: &DORIS_TABLES }),
        "RequireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4247), tables: &DORIS_TABLES }),
        "Require_row_formatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4248), tables: &DORIS_TABLES }),
        "ResetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4249), tables: &DORIS_TABLES }),
        "ResetMasterStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4250), tables: &DORIS_TABLES }),
        "ResignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4256), tables: &DORIS_TABLES }),
        "ResignalSegment" => Some(RootGrammar { grammar_id: GrammarId(4257), tables: &DORIS_TABLES }),
        "ResourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4290), tables: &DORIS_TABLES }),
        "ResourcesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4291), tables: &DORIS_TABLES }),
        "RespectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4292), tables: &DORIS_TABLES }),
        "RestartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4293), tables: &DORIS_TABLES }),
        "RestoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4294), tables: &DORIS_TABLES }),
        "RestrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4295), tables: &DORIS_TABLES }),
        "RestrictiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4296), tables: &DORIS_TABLES }),
        "ResultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4297), tables: &DORIS_TABLES }),
        "ResumeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4298), tables: &DORIS_TABLES }),
        "RetainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4299), tables: &DORIS_TABLES }),
        "RetentionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4300), tables: &DORIS_TABLES }),
        "ReturnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4301), tables: &DORIS_TABLES }),
        "ReturnStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4302), tables: &DORIS_TABLES }),
        "Returned_cardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4305), tables: &DORIS_TABLES }),
        "Returned_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4306), tables: &DORIS_TABLES }),
        "Returned_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4307), tables: &DORIS_TABLES }),
        "Returned_sqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4308), tables: &DORIS_TABLES }),
        "ReturningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4309), tables: &DORIS_TABLES }),
        "ReturnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4310), tables: &DORIS_TABLES }),
        "ReuseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4311), tables: &DORIS_TABLES }),
        "ReverseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4312), tables: &DORIS_TABLES }),
        "RevokeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4313), tables: &DORIS_TABLES }),
        "RevokeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4314), tables: &DORIS_TABLES }),
        "RewrittenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4337), tables: &DORIS_TABLES }),
        "RightKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4338), tables: &DORIS_TABLES }),
        "Right_braceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4339), tables: &DORIS_TABLES }),
        "RlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4340), tables: &DORIS_TABLES }),
        "RoleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4341), tables: &DORIS_TABLES }),
        "RoleReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4342), tables: &DORIS_TABLES }),
        "RolesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4357), tables: &DORIS_TABLES }),
        "RollbackKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4358), tables: &DORIS_TABLES }),
        "RollupFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(4359), tables: &DORIS_TABLES }),
        "RollupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4360), tables: &DORIS_TABLES }),
        "RollupSegment" => Some(RootGrammar { grammar_id: GrammarId(4361), tables: &DORIS_TABLES }),
        "RotateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4376), tables: &DORIS_TABLES }),
        "RoutineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4377), tables: &DORIS_TABLES }),
        "Routine_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4378), tables: &DORIS_TABLES }),
        "Routine_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4379), tables: &DORIS_TABLES }),
        "Routine_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4380), tables: &DORIS_TABLES }),
        "RoutinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4381), tables: &DORIS_TABLES }),
        "RowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4382), tables: &DORIS_TABLES }),
        "Row_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4383), tables: &DORIS_TABLES }),
        "Row_formatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4384), tables: &DORIS_TABLES }),
        "Row_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4385), tables: &DORIS_TABLES }),
        "RowcountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4386), tables: &DORIS_TABLES }),
        "RowguidcolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4387), tables: &DORIS_TABLES }),
        "RowidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4388), tables: &DORIS_TABLES }),
        "RownumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4389), tables: &DORIS_TABLES }),
        "RowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4390), tables: &DORIS_TABLES }),
        "RtreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4391), tables: &DORIS_TABLES }),
        "RuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4392), tables: &DORIS_TABLES }),
        "S3KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4393), tables: &DORIS_TABLES }),
        "SampleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4394), tables: &DORIS_TABLES }),
        "SamplingExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4395), tables: &DORIS_TABLES }),
        "SaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4407), tables: &DORIS_TABLES }),
        "SavepointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4408), tables: &DORIS_TABLES }),
        "ScaleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4409), tables: &DORIS_TABLES }),
        "ScheduleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4410), tables: &DORIS_TABLES }),
        "SchedulerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4411), tables: &DORIS_TABLES }),
        "SchemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4412), tables: &DORIS_TABLES }),
        "SchemaReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1079), tables: &DORIS_TABLES }),
        "Schema_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4413), tables: &DORIS_TABLES }),
        "SchemasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4414), tables: &DORIS_TABLES }),
        "ScopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4415), tables: &DORIS_TABLES }),
        "Scope_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4416), tables: &DORIS_TABLES }),
        "Scope_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4417), tables: &DORIS_TABLES }),
        "Scope_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4418), tables: &DORIS_TABLES }),
        "ScrollKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4419), tables: &DORIS_TABLES }),
        "SearchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4420), tables: &DORIS_TABLES }),
        "SecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4421), tables: &DORIS_TABLES }),
        "Second_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4422), tables: &DORIS_TABLES }),
        "SecondaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4423), tables: &DORIS_TABLES }),
        "Secondary_engineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4424), tables: &DORIS_TABLES }),
        "Secondary_engine_attributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4425), tables: &DORIS_TABLES }),
        "Secondary_loadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4426), tables: &DORIS_TABLES }),
        "Secondary_unloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4427), tables: &DORIS_TABLES }),
        "SectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4428), tables: &DORIS_TABLES }),
        "SecurityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4429), tables: &DORIS_TABLES }),
        "SelectClauseElementSegment" => Some(RootGrammar { grammar_id: GrammarId(4430), tables: &DORIS_TABLES }),
        "SelectClauseModifierSegment" => Some(RootGrammar { grammar_id: GrammarId(4435), tables: &DORIS_TABLES }),
        "SelectClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4448), tables: &DORIS_TABLES }),
        "SelectClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4455), tables: &DORIS_TABLES }),
        "SelectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4466), tables: &DORIS_TABLES }),
        "SelectPartitionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4467), tables: &DORIS_TABLES }),
        "SelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4472), tables: &DORIS_TABLES }),
        "SelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(4491), tables: &DORIS_TABLES }),
        "SelfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4501), tables: &DORIS_TABLES }),
        "SemiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4502), tables: &DORIS_TABLES }),
        "SemicolonSegment" => Some(RootGrammar { grammar_id: GrammarId(4503), tables: &DORIS_TABLES }),
        "SensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4504), tables: &DORIS_TABLES }),
        "SeparatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4505), tables: &DORIS_TABLES }),
        "SequenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4506), tables: &DORIS_TABLES }),
        "SequenceMaxValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(4507), tables: &DORIS_TABLES }),
        "SequenceMinValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(4514), tables: &DORIS_TABLES }),
        "SequenceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1079), tables: &DORIS_TABLES }),
        "SequencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4521), tables: &DORIS_TABLES }),
        "SerialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4522), tables: &DORIS_TABLES }),
        "SerializableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4523), tables: &DORIS_TABLES }),
        "ServerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4524), tables: &DORIS_TABLES }),
        "Server_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4525), tables: &DORIS_TABLES }),
        "SessionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4526), tables: &DORIS_TABLES }),
        "SessionVariableNameSegment" => Some(RootGrammar { grammar_id: GrammarId(4527), tables: &DORIS_TABLES }),
        "Session_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4528), tables: &DORIS_TABLES }),
        "SetAssignmentStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4529), tables: &DORIS_TABLES }),
        "SetClauseListSegment" => Some(RootGrammar { grammar_id: GrammarId(4562), tables: &DORIS_TABLES }),
        "SetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4566), tables: &DORIS_TABLES }),
        "SetDefaultRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4577), tables: &DORIS_TABLES }),
        "SetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4589), tables: &DORIS_TABLES }),
        "SetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4598), tables: &DORIS_TABLES }),
        "SetNamesStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4599), tables: &DORIS_TABLES }),
        "SetOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(4609), tables: &DORIS_TABLES }),
        "SetSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4617), tables: &DORIS_TABLES }),
        "SetTransactionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4622), tables: &DORIS_TABLES }),
        "Set_session_variableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4648), tables: &DORIS_TABLES }),
        "SetofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4649), tables: &DORIS_TABLES }),
        "SetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4650), tables: &DORIS_TABLES }),
        "SetuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4651), tables: &DORIS_TABLES }),
        "ShapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4652), tables: &DORIS_TABLES }),
        "ShareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4653), tables: &DORIS_TABLES }),
        "SharedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4654), tables: &DORIS_TABLES }),
        "SharesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4655), tables: &DORIS_TABLES }),
        "ShorthandCastSegment" => Some(RootGrammar { grammar_id: GrammarId(4656), tables: &DORIS_TABLES }),
        "ShowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4665), tables: &DORIS_TABLES }),
        "ShutdownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4666), tables: &DORIS_TABLES }),
        "SignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4667), tables: &DORIS_TABLES }),
        "SignedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4668), tables: &DORIS_TABLES }),
        "SignedSegmentGrammar" => Some(RootGrammar { grammar_id: GrammarId(4669), tables: &DORIS_TABLES }),
        "SimilarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4672), tables: &DORIS_TABLES }),
        "SimpleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4673), tables: &DORIS_TABLES }),
        "SingleIdentifierGrammar" => Some(RootGrammar { grammar_id: GrammarId(4674), tables: &DORIS_TABLES }),
        "SingleIdentifierListSegment" => Some(RootGrammar { grammar_id: GrammarId(4679), tables: &DORIS_TABLES }),
        "SingleIdentifierWildcardSegment" => Some(RootGrammar { grammar_id: GrammarId(4681), tables: &DORIS_TABLES }),
        "SingleQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(4685), tables: &DORIS_TABLES }),
        "SingleQuotedJSONPath" => Some(RootGrammar { grammar_id: GrammarId(4686), tables: &DORIS_TABLES }),
        "SizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4687), tables: &DORIS_TABLES }),
        "SizedArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(4688), tables: &DORIS_TABLES }),
        "SkewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4691), tables: &DORIS_TABLES }),
        "SkipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4692), tables: &DORIS_TABLES }),
        "SlashSegment" => Some(RootGrammar { grammar_id: GrammarId(4693), tables: &DORIS_TABLES }),
        "SlaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4694), tables: &DORIS_TABLES }),
        "SliceSegment" => Some(RootGrammar { grammar_id: GrammarId(4695), tables: &DORIS_TABLES }),
        "SlowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4696), tables: &DORIS_TABLES }),
        "SmallintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4697), tables: &DORIS_TABLES }),
        "SnapshotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4698), tables: &DORIS_TABLES }),
        "SnapshotsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4699), tables: &DORIS_TABLES }),
        "SocketKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4700), tables: &DORIS_TABLES }),
        "SomeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4701), tables: &DORIS_TABLES }),
        "SonameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4702), tables: &DORIS_TABLES }),
        "SoundsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4703), tables: &DORIS_TABLES }),
        "SourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4704), tables: &DORIS_TABLES }),
        "Source_auto_positionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4705), tables: &DORIS_TABLES }),
        "Source_bindKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4706), tables: &DORIS_TABLES }),
        "Source_compression_algorithmsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4707), tables: &DORIS_TABLES }),
        "Source_connect_retryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4708), tables: &DORIS_TABLES }),
        "Source_delayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4709), tables: &DORIS_TABLES }),
        "Source_heartbeat_periodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4710), tables: &DORIS_TABLES }),
        "Source_hostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4711), tables: &DORIS_TABLES }),
        "Source_log_fileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4712), tables: &DORIS_TABLES }),
        "Source_log_posKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4713), tables: &DORIS_TABLES }),
        "Source_passwordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4714), tables: &DORIS_TABLES }),
        "Source_portKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4715), tables: &DORIS_TABLES }),
        "Source_public_key_pathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4716), tables: &DORIS_TABLES }),
        "Source_retry_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4717), tables: &DORIS_TABLES }),
        "Source_sslKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4718), tables: &DORIS_TABLES }),
        "Source_ssl_caKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4719), tables: &DORIS_TABLES }),
        "Source_ssl_capathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4720), tables: &DORIS_TABLES }),
        "Source_ssl_certKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4721), tables: &DORIS_TABLES }),
        "Source_ssl_cipherKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4722), tables: &DORIS_TABLES }),
        "Source_ssl_crlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4723), tables: &DORIS_TABLES }),
        "Source_ssl_crlpathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4724), tables: &DORIS_TABLES }),
        "Source_ssl_keyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4725), tables: &DORIS_TABLES }),
        "Source_ssl_verify_server_certKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4726), tables: &DORIS_TABLES }),
        "Source_tls_ciphersuitesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4727), tables: &DORIS_TABLES }),
        "Source_tls_versionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4728), tables: &DORIS_TABLES }),
        "Source_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4729), tables: &DORIS_TABLES }),
        "Source_zstd_compression_levelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4730), tables: &DORIS_TABLES }),
        "SpaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4731), tables: &DORIS_TABLES }),
        "SpatialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4732), tables: &DORIS_TABLES }),
        "SpecificKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4733), tables: &DORIS_TABLES }),
        "Specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4734), tables: &DORIS_TABLES }),
        "SpecifictypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4735), tables: &DORIS_TABLES }),
        "SplitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4736), tables: &DORIS_TABLES }),
        "SqlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4737), tables: &DORIS_TABLES }),
        "Sql_after_gtidsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4738), tables: &DORIS_TABLES }),
        "Sql_after_mts_gapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4739), tables: &DORIS_TABLES }),
        "Sql_before_gtidsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4740), tables: &DORIS_TABLES }),
        "Sql_big_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4741), tables: &DORIS_TABLES }),
        "Sql_big_selectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4742), tables: &DORIS_TABLES }),
        "Sql_big_tablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4743), tables: &DORIS_TABLES }),
        "Sql_block_ruleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4744), tables: &DORIS_TABLES }),
        "Sql_buffer_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4745), tables: &DORIS_TABLES }),
        "Sql_cacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4746), tables: &DORIS_TABLES }),
        "Sql_calc_found_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4747), tables: &DORIS_TABLES }),
        "Sql_log_offKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4748), tables: &DORIS_TABLES }),
        "Sql_log_updateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4749), tables: &DORIS_TABLES }),
        "Sql_low_priority_updatesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4750), tables: &DORIS_TABLES }),
        "Sql_no_cacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4751), tables: &DORIS_TABLES }),
        "Sql_select_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4752), tables: &DORIS_TABLES }),
        "Sql_small_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4753), tables: &DORIS_TABLES }),
        "Sql_threadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4754), tables: &DORIS_TABLES }),
        "Sql_tsi_dayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4755), tables: &DORIS_TABLES }),
        "Sql_tsi_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4756), tables: &DORIS_TABLES }),
        "Sql_tsi_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4757), tables: &DORIS_TABLES }),
        "Sql_tsi_monthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4758), tables: &DORIS_TABLES }),
        "Sql_tsi_quarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4759), tables: &DORIS_TABLES }),
        "Sql_tsi_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4760), tables: &DORIS_TABLES }),
        "Sql_tsi_weekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4761), tables: &DORIS_TABLES }),
        "Sql_tsi_yearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4762), tables: &DORIS_TABLES }),
        "Sql_warningsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4763), tables: &DORIS_TABLES }),
        "SqlcaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4764), tables: &DORIS_TABLES }),
        "SqlcodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4765), tables: &DORIS_TABLES }),
        "SqlerrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4766), tables: &DORIS_TABLES }),
        "SqlexceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4767), tables: &DORIS_TABLES }),
        "SqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4768), tables: &DORIS_TABLES }),
        "SqlwarningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4769), tables: &DORIS_TABLES }),
        "SqrtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4770), tables: &DORIS_TABLES }),
        "SridKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4771), tables: &DORIS_TABLES }),
        "SslKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4772), tables: &DORIS_TABLES }),
        "StableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4773), tables: &DORIS_TABLES }),
        "StackedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4774), tables: &DORIS_TABLES }),
        "StageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4775), tables: &DORIS_TABLES }),
        "StagesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4776), tables: &DORIS_TABLES }),
        "StarSegment" => Some(RootGrammar { grammar_id: GrammarId(4777), tables: &DORIS_TABLES }),
        "StartBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(235), tables: &DORIS_TABLES }),
        "StartCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(3702), tables: &DORIS_TABLES }),
        "StartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4778), tables: &DORIS_TABLES }),
        "StartSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(656), tables: &DORIS_TABLES }),
        "StartingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4779), tables: &DORIS_TABLES }),
        "StartsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4780), tables: &DORIS_TABLES }),
        "StateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4781), tables: &DORIS_TABLES }),
        "StatementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4782), tables: &DORIS_TABLES }),
        "StatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4783), tables: &DORIS_TABLES }),
        "StaticKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4866), tables: &DORIS_TABLES }),
        "StatisticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4867), tables: &DORIS_TABLES }),
        "StatsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4868), tables: &DORIS_TABLES }),
        "Stats_auto_recalcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4869), tables: &DORIS_TABLES }),
        "Stats_persistentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4870), tables: &DORIS_TABLES }),
        "Stats_sample_pagesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4871), tables: &DORIS_TABLES }),
        "StatusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4872), tables: &DORIS_TABLES }),
        "Stddev_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4873), tables: &DORIS_TABLES }),
        "Stddev_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4874), tables: &DORIS_TABLES }),
        "StdinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4875), tables: &DORIS_TABLES }),
        "StdoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4876), tables: &DORIS_TABLES }),
        "StopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4877), tables: &DORIS_TABLES }),
        "StorageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4878), tables: &DORIS_TABLES }),
        "StoredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4879), tables: &DORIS_TABLES }),
        "Straight_joinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4880), tables: &DORIS_TABLES }),
        "StreamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4881), tables: &DORIS_TABLES }),
        "StreamingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4882), tables: &DORIS_TABLES }),
        "StreamsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4883), tables: &DORIS_TABLES }),
        "StrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4884), tables: &DORIS_TABLES }),
        "StringBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4885), tables: &DORIS_TABLES }),
        "StringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4887), tables: &DORIS_TABLES }),
        "StructKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4888), tables: &DORIS_TABLES }),
        "StructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4889), tables: &DORIS_TABLES }),
        "StructTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(4894), tables: &DORIS_TABLES }),
        "StructureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4895), tables: &DORIS_TABLES }),
        "StyleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4896), tables: &DORIS_TABLES }),
        "Subclass_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4897), tables: &DORIS_TABLES }),
        "SubjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4898), tables: &DORIS_TABLES }),
        "SublistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4899), tables: &DORIS_TABLES }),
        "SubmultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4900), tables: &DORIS_TABLES }),
        "SubpartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4901), tables: &DORIS_TABLES }),
        "SubpartitionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4902), tables: &DORIS_TABLES }),
        "SubstringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4903), tables: &DORIS_TABLES }),
        "SuccessfulKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4904), tables: &DORIS_TABLES }),
        "SumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4905), tables: &DORIS_TABLES }),
        "SuperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4906), tables: &DORIS_TABLES }),
        "SuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4907), tables: &DORIS_TABLES }),
        "SuspendKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4908), tables: &DORIS_TABLES }),
        "SwapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4909), tables: &DORIS_TABLES }),
        "SwitchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4910), tables: &DORIS_TABLES }),
        "SwitchesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4911), tables: &DORIS_TABLES }),
        "SymbolSegment" => Some(RootGrammar { grammar_id: GrammarId(4912), tables: &DORIS_TABLES }),
        "SymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4913), tables: &DORIS_TABLES }),
        "SyncKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4914), tables: &DORIS_TABLES }),
        "SynonymKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4915), tables: &DORIS_TABLES }),
        "SysdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4916), tables: &DORIS_TABLES }),
        "SysidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4917), tables: &DORIS_TABLES }),
        "SystemKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4918), tables: &DORIS_TABLES }),
        "SystemVariableSegment" => Some(RootGrammar { grammar_id: GrammarId(4919), tables: &DORIS_TABLES }),
        "System_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4920), tables: &DORIS_TABLES }),
        "TableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(4921), tables: &DORIS_TABLES }),
        "TableEndClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4993), tables: &DORIS_TABLES }),
        "TableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4994), tables: &DORIS_TABLES }),
        "TableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5003), tables: &DORIS_TABLES }),
        "TableOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(5004), tables: &DORIS_TABLES }),
        "TableReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(5148), tables: &DORIS_TABLES }),
        "Table_checksumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5154), tables: &DORIS_TABLES }),
        "Table_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5155), tables: &DORIS_TABLES }),
        "TablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5156), tables: &DORIS_TABLES }),
        "TablesampleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5157), tables: &DORIS_TABLES }),
        "TablespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5158), tables: &DORIS_TABLES }),
        "TablespaceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1079), tables: &DORIS_TABLES }),
        "TabletKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5159), tables: &DORIS_TABLES }),
        "TabletsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5160), tables: &DORIS_TABLES }),
        "TagKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5161), tables: &DORIS_TABLES }),
        "TagReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1079), tables: &DORIS_TABLES }),
        "Tail_Recurse_Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(5162), tables: &DORIS_TABLES }),
        "Tail_Recurse_Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(5167), tables: &DORIS_TABLES }),
        "TaskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5171), tables: &DORIS_TABLES }),
        "TasksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5172), tables: &DORIS_TABLES }),
        "TempKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5173), tables: &DORIS_TABLES }),
        "TemplateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5174), tables: &DORIS_TABLES }),
        "TemporalQuerySegment" => Some(RootGrammar { grammar_id: GrammarId(5175), tables: &DORIS_TABLES }),
        "TemporaryGrammar" => Some(RootGrammar { grammar_id: GrammarId(5176), tables: &DORIS_TABLES }),
        "TemporaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5179), tables: &DORIS_TABLES }),
        "TemporaryTransientGrammar" => Some(RootGrammar { grammar_id: GrammarId(5180), tables: &DORIS_TABLES }),
        "TemptableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5183), tables: &DORIS_TABLES }),
        "TerminateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5184), tables: &DORIS_TABLES }),
        "TerminatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5185), tables: &DORIS_TABLES }),
        "TextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5186), tables: &DORIS_TABLES }),
        "TextsizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5187), tables: &DORIS_TABLES }),
        "ThanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5188), tables: &DORIS_TABLES }),
        "ThenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5189), tables: &DORIS_TABLES }),
        "Thread_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5190), tables: &DORIS_TABLES }),
        "TiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5191), tables: &DORIS_TABLES }),
        "TildeSegment" => Some(RootGrammar { grammar_id: GrammarId(5192), tables: &DORIS_TABLES }),
        "TimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5193), tables: &DORIS_TABLES }),
        "TimeWithTZGrammar" => Some(RootGrammar { grammar_id: GrammarId(5194), tables: &DORIS_TABLES }),
        "TimeZoneGrammar" => Some(RootGrammar { grammar_id: GrammarId(5205), tables: &DORIS_TABLES }),
        "TimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5211), tables: &DORIS_TABLES }),
        "TimestampaddKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5212), tables: &DORIS_TABLES }),
        "TimestampdiffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5213), tables: &DORIS_TABLES }),
        "Timezone_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5214), tables: &DORIS_TABLES }),
        "Timezone_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5215), tables: &DORIS_TABLES }),
        "TinyblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5216), tables: &DORIS_TABLES }),
        "TinyintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5217), tables: &DORIS_TABLES }),
        "TinytextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5218), tables: &DORIS_TABLES }),
        "TlsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5219), tables: &DORIS_TABLES }),
        "ToKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5220), tables: &DORIS_TABLES }),
        "ToastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5221), tables: &DORIS_TABLES }),
        "Token_filterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5222), tables: &DORIS_TABLES }),
        "TokenizerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5223), tables: &DORIS_TABLES }),
        "TopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5224), tables: &DORIS_TABLES }),
        "Top_level_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5225), tables: &DORIS_TABLES }),
        "TrailingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5226), tables: &DORIS_TABLES }),
        "TranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5227), tables: &DORIS_TABLES }),
        "TransactionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5228), tables: &DORIS_TABLES }),
        "TransactionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5229), tables: &DORIS_TABLES }),
        "Transaction_activeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5257), tables: &DORIS_TABLES }),
        "TransactionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5258), tables: &DORIS_TABLES }),
        "Transactions_committedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5259), tables: &DORIS_TABLES }),
        "Transactions_rolled_backKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5260), tables: &DORIS_TABLES }),
        "TransformKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5261), tables: &DORIS_TABLES }),
        "TransformsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5262), tables: &DORIS_TABLES }),
        "TransientKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5263), tables: &DORIS_TABLES }),
        "TranslateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5264), tables: &DORIS_TABLES }),
        "TranslationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5265), tables: &DORIS_TABLES }),
        "TrashKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5266), tables: &DORIS_TABLES }),
        "TreatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5267), tables: &DORIS_TABLES }),
        "TreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5268), tables: &DORIS_TABLES }),
        "TriggerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5269), tables: &DORIS_TABLES }),
        "TriggerOrReplaceGrammar" => Some(RootGrammar { grammar_id: GrammarId(5270), tables: &DORIS_TABLES }),
        "TriggerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1079), tables: &DORIS_TABLES }),
        "Trigger_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5271), tables: &DORIS_TABLES }),
        "Trigger_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5272), tables: &DORIS_TABLES }),
        "Trigger_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5273), tables: &DORIS_TABLES }),
        "TriggersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5274), tables: &DORIS_TABLES }),
        "TrimKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5275), tables: &DORIS_TABLES }),
        "TrimParametersGrammar" => Some(RootGrammar { grammar_id: GrammarId(5276), tables: &DORIS_TABLES }),
        "TrueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5280), tables: &DORIS_TABLES }),
        "TrueSegment" => Some(RootGrammar { grammar_id: GrammarId(5281), tables: &DORIS_TABLES }),
        "TruncateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5282), tables: &DORIS_TABLES }),
        "TruncateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5283), tables: &DORIS_TABLES }),
        "TrustedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5287), tables: &DORIS_TABLES }),
        "TsequalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5288), tables: &DORIS_TABLES }),
        "TupleSegment" => Some(RootGrammar { grammar_id: GrammarId(5289), tables: &DORIS_TABLES }),
        "TypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5292), tables: &DORIS_TABLES }),
        "TypecastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5293), tables: &DORIS_TABLES }),
        "TypedArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5294), tables: &DORIS_TABLES }),
        "TypedStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5297), tables: &DORIS_TABLES }),
        "TypesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5300), tables: &DORIS_TABLES }),
        "UescapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5301), tables: &DORIS_TABLES }),
        "UidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5302), tables: &DORIS_TABLES }),
        "UnboundedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5303), tables: &DORIS_TABLES }),
        "UncommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5304), tables: &DORIS_TABLES }),
        "UnconditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5305), tables: &DORIS_TABLES }),
        "UnconditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5306), tables: &DORIS_TABLES }),
        "UndefinedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5310), tables: &DORIS_TABLES }),
        "UnderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5311), tables: &DORIS_TABLES }),
        "UndoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5312), tables: &DORIS_TABLES }),
        "Undo_buffer_sizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5313), tables: &DORIS_TABLES }),
        "UndofileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5314), tables: &DORIS_TABLES }),
        "UnencryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5315), tables: &DORIS_TABLES }),
        "UnicodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5316), tables: &DORIS_TABLES }),
        "UninstallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5317), tables: &DORIS_TABLES }),
        "UnionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5318), tables: &DORIS_TABLES }),
        "UnionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5323), tables: &DORIS_TABLES }),
        "UniqueKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(5324), tables: &DORIS_TABLES }),
        "UniqueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5327), tables: &DORIS_TABLES }),
        "UnknownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5328), tables: &DORIS_TABLES }),
        "UnknownLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5329), tables: &DORIS_TABLES }),
        "UnlistenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5330), tables: &DORIS_TABLES }),
        "UnlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5331), tables: &DORIS_TABLES }),
        "UnnamedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5332), tables: &DORIS_TABLES }),
        "UnnestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5333), tables: &DORIS_TABLES }),
        "UnorderedSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5334), tables: &DORIS_TABLES }),
        "UnorderedSetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5357), tables: &DORIS_TABLES }),
        "UnregisterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5363), tables: &DORIS_TABLES }),
        "UnsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5364), tables: &DORIS_TABLES }),
        "UnsignedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5365), tables: &DORIS_TABLES }),
        "UntilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5366), tables: &DORIS_TABLES }),
        "UpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5367), tables: &DORIS_TABLES }),
        "UpdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5368), tables: &DORIS_TABLES }),
        "UpdateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5369), tables: &DORIS_TABLES }),
        "UpdatetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5381), tables: &DORIS_TABLES }),
        "UpgradeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5382), tables: &DORIS_TABLES }),
        "UpperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5383), tables: &DORIS_TABLES }),
        "UpsertClauseListSegment" => Some(RootGrammar { grammar_id: GrammarId(5384), tables: &DORIS_TABLES }),
        "UsageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5391), tables: &DORIS_TABLES }),
        "UseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5392), tables: &DORIS_TABLES }),
        "UseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5393), tables: &DORIS_TABLES }),
        "Use_any_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5396), tables: &DORIS_TABLES }),
        "Use_frmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5397), tables: &DORIS_TABLES }),
        "Use_mvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5398), tables: &DORIS_TABLES }),
        "UserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5399), tables: &DORIS_TABLES }),
        "UserReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(5400), tables: &DORIS_TABLES }),
        "User_defined_type_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5401), tables: &DORIS_TABLES }),
        "User_defined_type_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5402), tables: &DORIS_TABLES }),
        "User_defined_type_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5403), tables: &DORIS_TABLES }),
        "User_defined_type_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5404), tables: &DORIS_TABLES }),
        "User_resourcesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5405), tables: &DORIS_TABLES }),
        "UsersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5406), tables: &DORIS_TABLES }),
        "UsingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5407), tables: &DORIS_TABLES }),
        "Utc_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5408), tables: &DORIS_TABLES }),
        "Utc_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5409), tables: &DORIS_TABLES }),
        "Utc_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5410), tables: &DORIS_TABLES }),
        "VacuumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5411), tables: &DORIS_TABLES }),
        "ValidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5412), tables: &DORIS_TABLES }),
        "ValidateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5413), tables: &DORIS_TABLES }),
        "ValidationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5414), tables: &DORIS_TABLES }),
        "ValidatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5415), tables: &DORIS_TABLES }),
        "ValueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5416), tables: &DORIS_TABLES }),
        "ValuesClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5417), tables: &DORIS_TABLES }),
        "ValuesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5430), tables: &DORIS_TABLES }),
        "Var_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5431), tables: &DORIS_TABLES }),
        "Var_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5432), tables: &DORIS_TABLES }),
        "VarbinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5433), tables: &DORIS_TABLES }),
        "Varchar2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5434), tables: &DORIS_TABLES }),
        "VarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5435), tables: &DORIS_TABLES }),
        "VarcharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5436), tables: &DORIS_TABLES }),
        "VariableAssignmentSegment" => Some(RootGrammar { grammar_id: GrammarId(5437), tables: &DORIS_TABLES }),
        "VariableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5441), tables: &DORIS_TABLES }),
        "VariablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5442), tables: &DORIS_TABLES }),
        "VariantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5443), tables: &DORIS_TABLES }),
        "VaryingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5444), tables: &DORIS_TABLES }),
        "VaultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5445), tables: &DORIS_TABLES }),
        "VaultsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5446), tables: &DORIS_TABLES }),
        "VcpuKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5447), tables: &DORIS_TABLES }),
        "VerboseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5448), tables: &DORIS_TABLES }),
        "VersionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5449), tables: &DORIS_TABLES }),
        "ViewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5450), tables: &DORIS_TABLES }),
        "ViewsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5451), tables: &DORIS_TABLES }),
        "VirtualKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5452), tables: &DORIS_TABLES }),
        "VisibleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5453), tables: &DORIS_TABLES }),
        "VolatileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5454), tables: &DORIS_TABLES }),
        "WaitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5455), tables: &DORIS_TABLES }),
        "WaitforKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5456), tables: &DORIS_TABLES }),
        "WalrusOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(5457), tables: &DORIS_TABLES }),
        "WarehouseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5458), tables: &DORIS_TABLES }),
        "WarehousesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5459), tables: &DORIS_TABLES }),
        "WarmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5460), tables: &DORIS_TABLES }),
        "WarningsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5461), tables: &DORIS_TABLES }),
        "WeekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5462), tables: &DORIS_TABLES }),
        "WeekdayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5463), tables: &DORIS_TABLES }),
        "Weight_stringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5464), tables: &DORIS_TABLES }),
        "WhenClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5465), tables: &DORIS_TABLES }),
        "WhenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5475), tables: &DORIS_TABLES }),
        "WheneverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5476), tables: &DORIS_TABLES }),
        "WhereClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5477), tables: &DORIS_TABLES }),
        "WhereClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5482), tables: &DORIS_TABLES }),
        "WhereKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5496), tables: &DORIS_TABLES }),
        "WhileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5497), tables: &DORIS_TABLES }),
        "WhileStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5498), tables: &DORIS_TABLES }),
        "WhitelistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5513), tables: &DORIS_TABLES }),
        "WhitespaceSegment" => Some(RootGrammar { grammar_id: GrammarId(5514), tables: &DORIS_TABLES }),
        "Width_bucketKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5515), tables: &DORIS_TABLES }),
        "WildcardExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5516), tables: &DORIS_TABLES }),
        "WildcardIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(5518), tables: &DORIS_TABLES }),
        "WindowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5528), tables: &DORIS_TABLES }),
        "WindowSpecificationSegment" => Some(RootGrammar { grammar_id: GrammarId(5529), tables: &DORIS_TABLES }),
        "WithCheckOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(5537), tables: &DORIS_TABLES }),
        "WithCompoundNonSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5544), tables: &DORIS_TABLES }),
        "WithCompoundStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5553), tables: &DORIS_TABLES }),
        "WithDataClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5562), tables: &DORIS_TABLES }),
        "WithFillSegment" => Some(RootGrammar { grammar_id: GrammarId(5567), tables: &DORIS_TABLES }),
        "WithKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5568), tables: &DORIS_TABLES }),
        "WithNoSchemaBindingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5569), tables: &DORIS_TABLES }),
        "WithinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5574), tables: &DORIS_TABLES }),
        "WithoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5575), tables: &DORIS_TABLES }),
        "WordSegment" => Some(RootGrammar { grammar_id: GrammarId(5576), tables: &DORIS_TABLES }),
        "WorkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5577), tables: &DORIS_TABLES }),
        "WorkloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5578), tables: &DORIS_TABLES }),
        "WrapperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5579), tables: &DORIS_TABLES }),
        "WriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5580), tables: &DORIS_TABLES }),
        "WritetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5581), tables: &DORIS_TABLES }),
        "X509KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5582), tables: &DORIS_TABLES }),
        "XaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5583), tables: &DORIS_TABLES }),
        "XidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5584), tables: &DORIS_TABLES }),
        "XmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5585), tables: &DORIS_TABLES }),
        "XorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5586), tables: &DORIS_TABLES }),
        "YamlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5587), tables: &DORIS_TABLES }),
        "YearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5588), tables: &DORIS_TABLES }),
        "Year_monthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5589), tables: &DORIS_TABLES }),
        "ZerofillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5590), tables: &DORIS_TABLES }),
        "ZoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5591), tables: &DORIS_TABLES }),
        _ => None,
    }
}

pub fn get_doris_segment_type(name: &str) -> Option<&'static str> {
    match name {
        "AccessObjectSegment" => Some("access_object"),
        "AccessPermissionSegment" => Some("access_permission"),
        "AccessPermissionsSegment" => Some("access_permissions"),
        "AccessSchemaObjectSegment" => Some("access_schema_object"),
        "AccessSchemaPluralObjectSegment" => Some("access_schema_plural_object"),
        "AccessStatementSegment" => Some("access_statement"),
        "AccessTargetSegment" => Some("access_target"),
        "AggregateOrderByClause" => Some("aggregate_order_by"),
        "AliasExpressionSegment" => Some("alias_expression"),
        "AlterDatabaseStatementSegment" => Some("alter_database_statement"),
        "AlterEventStatementSegment" => Some("alter_event_statement"),
        "AlterOptionSegment" => Some("alter_option_segment"),
        "AlterSequenceOptionsSegment" => Some("alter_sequence_options_segment"),
        "AlterSequenceStatementSegment" => Some("alter_sequence_statement"),
        "AlterTableOnlineDDLOptionSegment" => Some("alter_table_online_ddl_option"),
        "AlterTableStatementSegment" => Some("alter_table_statement"),
        "AlterViewStatementSegment" => Some("alter_view_statement"),
        "AnalyzeTableStatementSegment" => Some("analyze_table_statement"),
        "ArrayAccessorSegment" => Some("array_accessor"),
        "ArrayExpressionSegment" => Some("array_expression"),
        "ArrayLiteralSegment" => Some("array_literal"),
        "ArrayTypeSegment" => Some("array_type"),
        "AsAliasOperatorSegment" => Some("alias_operator"),
        "BaseFileSegment" => Some("file"),
        "BaseSegment" => Some("base"),
        "BinaryOperatorSegment" => Some("binary_operator"),
        "BitwiseAndSegment" => Some("binary_operator"),
        "BitwiseLShiftSegment" => Some("binary_operator"),
        "BitwiseOrSegment" => Some("binary_operator"),
        "BitwiseRShiftSegment" => Some("binary_operator"),
        "BracketedArguments" => Some("bracketed_arguments"),
        "BracketedSegment" => Some("bracketed"),
        "CTEColumnList" => Some("cte_column_list"),
        "CTEDefinitionSegment" => Some("common_table_expression"),
        "CallStoredProcedureSegment" => Some("call_statement"),
        "CaseExpressionSegment" => Some("case_expression"),
        "CharacteristicStatement" => Some("characteristic_statement"),
        "CheckTableStatementSegment" => Some("check_table_statement"),
        "ChecksumTableStatementSegment" => Some("checksum_table_statement"),
        "CodeSegment" => Some("raw"),
        "CollationReferenceSegment" => Some("collation_reference"),
        "ColumnConstraintSegment" => Some("column_constraint_segment"),
        "ColumnDefinitionSegment" => Some("column_definition"),
        "ColumnReferenceSegment" => Some("column_reference"),
        "ColumnsExpressionFunctionContentsSegment" => Some("columns_expression"),
        "ColumnsExpressionFunctionNameSegment" => Some("function_name"),
        "CommentClauseSegment" => Some("comment_clause"),
        "CommentSegment" => Some("comment"),
        "ComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeBinaryOperatorSegment" => Some("binary_operator"),
        "CompositeComparisonOperatorSegment" => Some("comparison_operator"),
        "ConcatSegment" => Some("binary_operator"),
        "ConvertFunctionContentsSegment" => Some("function_contents"),
        "ConvertFunctionNameSegment" => Some("function_name"),
        "CreateCastStatementSegment" => Some("create_cast_statement"),
        "CreateDatabaseStatementSegment" => Some("create_database_statement"),
        "CreateEventStatementSegment" => Some("create_event_statement"),
        "CreateFunctionStatementSegment" => Some("create_function_statement"),
        "CreateIndexStatementSegment" => Some("create_index_statement"),
        "CreateModelStatementSegment" => Some("create_model_statement"),
        "CreateOptionSegment" => Some("create_option_segment"),
        "CreateProcedureStatementSegment" => Some("create_procedure_statement"),
        "CreateRoleStatementSegment" => Some("create_role_statement"),
        "CreateSchemaStatementSegment" => Some("create_schema_statement"),
        "CreateSequenceOptionsSegment" => Some("create_sequence_options_segment"),
        "CreateSequenceStatementSegment" => Some("create_sequence_statement"),
        "CreateTableStatementSegment" => Some("create_table_statement"),
        "CreateTriggerStatementSegment" => Some("create_trigger"),
        "CreateUserStatementSegment" => Some("create_user_statement"),
        "CreateViewStatementSegment" => Some("create_view_statement"),
        "CubeFunctionNameSegment" => Some("function_name"),
        "CubeRollupClauseSegment" => Some("cube_rollup_clause"),
        "CurrentTimestampLikeFunctionContentsSegment" => Some("function_contents"),
        "CurrentTimestampLikeFunctionNameSegment" => Some("function_name"),
        "CurrentTimestampLikeFunctionSegment" => Some("function"),
        "CursorFetchSegment" => Some("cursor_fetch_segment"),
        "CursorOpenCloseSegment" => Some("cursor_open_close_segment"),
        "DatabaseReferenceSegment" => Some("database_reference"),
        "DatatypeSegment" => Some("data_type"),
        "DatePartFunctionNameSegment" => Some("function_name"),
        "DateTimeFunctionContentsSegment" => Some("function_contents"),
        "DeallocateSegment" => Some("deallocate_segment"),
        "DeclareStatement" => Some("declare_statement"),
        "Dedent" => Some("dedent"),
        "DefinerSegment" => Some("definer_segment"),
        "DeleteStatementSegment" => Some("delete_statement"),
        "DeleteTargetTableSegment" => Some("delete_target_table"),
        "DeleteUsingClauseSegment" => Some("using_clause"),
        "DelimiterStatement" => Some("delimiter_statement"),
        "DescribeStatementSegment" => Some("describe_statement"),
        "DistributionSegment" => Some("distribution_segment"),
        "DropCastStatementSegment" => Some("drop_cast_statement"),
        "DropDatabaseStatementSegment" => Some("drop_database_statement"),
        "DropEventStatementSegment" => Some("drop_event_statement"),
        "DropFunctionStatementSegment" => Some("drop_function_statement"),
        "DropIndexStatementSegment" => Some("drop_index_statement"),
        "DropModelStatementSegment" => Some("drop_MODELstatement"),
        "DropProcedureStatementSegment" => Some("drop_procedure_statement"),
        "DropRoleStatementSegment" => Some("drop_role_statement"),
        "DropSchemaStatementSegment" => Some("drop_schema_statement"),
        "DropSequenceStatementSegment" => Some("drop_sequence_statement"),
        "DropTableStatementSegment" => Some("drop_table_statement"),
        "DropTriggerStatementSegment" => Some("drop_trigger"),
        "DropTypeStatementSegment" => Some("drop_type_statement"),
        "DropUserStatementSegment" => Some("drop_user_statement"),
        "DropViewStatementSegment" => Some("drop_view_statement"),
        "ElseClauseSegment" => Some("else_clause"),
        "EmptyStructLiteralBracketsSegment" => Some("struct_literal"),
        "EmptyStructLiteralSegment" => Some("typed_struct_literal"),
        "EqualsSegment" => Some("comparison_operator"),
        "ExecuteSegment" => Some("execute_segment"),
        "ExplainStatementSegment" => Some("explain_statement"),
        "ExpressionSegment" => Some("expression"),
        "ExtensionReferenceSegment" => Some("extension_reference"),
        "FetchClauseSegment" => Some("fetch_clause"),
        "FileSegment" => Some("file"),
        "FlushStatementSegment" => Some("flush_statement"),
        "ForClauseSegment" => Some("for_clause"),
        "FrameClauseSegment" => Some("frame_clause"),
        "FromClauseSegment" => Some("from_clause"),
        "FromExpressionElementSegment" => Some("from_expression_element"),
        "FromExpressionSegment" => Some("from_expression"),
        "FunctionContentsSegment" => Some("function_contents"),
        "FunctionDefinitionGrammar" => Some("function_definition"),
        "FunctionNameSegment" => Some("function_name"),
        "FunctionParameterListGrammar" => Some("function_parameter_list"),
        "FunctionSegment" => Some("function"),
        "GetDiagnosticsSegment" => Some("get_diagnostics_segment"),
        "GrantStatementSegment" => Some("grant_statement"),
        "GreaterThanOrEqualToSegment" => Some("comparison_operator"),
        "GreaterThanSegment" => Some("comparison_operator"),
        "GroupByClauseSegment" => Some("groupby_clause"),
        "GroupingExpressionList" => Some("grouping_expression_list"),
        "GroupingSetsClauseSegment" => Some("grouping_sets_clause"),
        "HavingClauseSegment" => Some("having_clause"),
        "HelpStatementSegment" => Some("help_statement"),
        "IdentifierSegment" => Some("identifier"),
        "IfExpressionStatement" => Some("if_then_statement"),
        "ImplicitIndent" => Some("indent"),
        "Indent" => Some("indent"),
        "IndexColumnDefinitionSegment" => Some("index_column_definition"),
        "IndexColumnPrefixLengthSegment" => Some("bracketed_arguments"),
        "IndexDefinitionSegment" => Some("index_definition"),
        "IndexHintClauseSegment" => Some("index_hint_clause"),
        "IndexOptionsSegment" => Some("index_option"),
        "IndexReferenceSegment" => Some("index_reference"),
        "IndexTypeGrammar" => Some("index_type"),
        "InsertRowAliasSegment" => Some("insert_row_alias"),
        "InsertStatementSegment" => Some("insert_statement"),
        "IntervalExpressionSegment" => Some("interval_expression"),
        "IntoClauseSegment" => Some("into_clause"),
        "IterateStatementSegment" => Some("iterate_statement"),
        "JoinClauseSegment" => Some("join_clause"),
        "JoinOnConditionSegment" => Some("join_on_condition"),
        "JsonTableColumnsClauseSegment" => Some("json_table_columns_clause"),
        "JsonTableFunctionContentsSegment" => Some("function_contents"),
        "JsonTableFunctionNameSegment" => Some("function_name"),
        "JsonValueFunctionContentsSegment" => Some("function_contents"),
        "JsonValueFunctionNameSegment" => Some("function_name"),
        "KeywordSegment" => Some("keyword"),
        "LessThanOrEqualToSegment" => Some("comparison_operator"),
        "LessThanSegment" => Some("comparison_operator"),
        "LimitClauseSegment" => Some("limit_clause"),
        "ListPartitionDefinitionSegment" => Some("list_partition_definition"),
        "LiteralKeywordSegment" => Some("literal"),
        "LiteralSegment" => Some("literal"),
        "LoadDataSegment" => Some("load_data_statement"),
        "LocalAliasSegment" => Some("local_alias_segment"),
        "LoopStatementSegment" => Some("loop_statement"),
        "MLTableExpressionSegment" => Some("ml_table_expression"),
        "MapTypeSegment" => Some("map_type"),
        "MatchConditionSegment" => Some("match_condition"),
        "MergeDeleteClauseSegment" => Some("merge_delete_clause"),
        "MergeInsertClauseSegment" => Some("merge_insert_clause"),
        "MergeMatchSegment" => Some("merge_match"),
        "MergeMatchedClauseSegment" => Some("merge_when_matched_clause"),
        "MergeNotMatchedClauseSegment" => Some("merge_when_not_matched_clause"),
        "MergeStatementSegment" => Some("merge_statement"),
        "MergeUpdateClauseSegment" => Some("merge_update_clause"),
        "NamedWindowExpressionSegment" => Some("named_window_expression"),
        "NamedWindowSegment" => Some("named_window"),
        "NewlineSegment" => Some("newline"),
        "NotEqualToSegment" => Some("comparison_operator"),
        "NullSafeEqualsSegment" => Some("comparison_operator"),
        "ObjectLiteralElementSegment" => Some("object_literal_element"),
        "ObjectLiteralSegment" => Some("object_literal"),
        "ObjectReferenceSegment" => Some("object_reference"),
        "OffsetClauseSegment" => Some("offset_clause"),
        "OptimizeTableStatementSegment" => Some("optimize_table_statement"),
        "OrderByClauseSegment" => Some("orderby_clause"),
        "OverClauseSegment" => Some("over_clause"),
        "OverlapsClauseSegment" => Some("overlaps_clause"),
        "PartitionClauseSegment" => Some("partitionby_clause"),
        "PartitionSegment" => Some("partition_segment"),
        "PathSegment" => Some("path_segment"),
        "PrepareSegment" => Some("prepare_segment"),
        "ProcedureParameterListGrammar" => Some("procedure_parameter_list"),
        "PurgeBinaryLogsStatementSegment" => Some("purge_binary_logs_statement"),
        "QualifiedNumericLiteralSegment" => Some("numeric_literal"),
        "RangePartitionDefinitionSegment" => Some("range_partition_definition"),
        "RangePartitionIntervalSegment" => Some("range_partition_interval"),
        "RenameTableStatementSegment" => Some("rename_table_statement"),
        "RepairTableStatementSegment" => Some("repair_table_statement"),
        "RepeatStatementSegment" => Some("repeat_statement"),
        "ReplaceSegment" => Some("replace_statement"),
        "ResetMasterStatementSegment" => Some("reset_master_statement"),
        "ResignalSegment" => Some("resignal_segment"),
        "ReturnStatementSegment" => Some("return_statement"),
        "RevokeStatementSegment" => Some("revoke_statement"),
        "RoleReferenceSegment" => Some("role_reference"),
        "RollupFunctionNameSegment" => Some("function_name"),
        "RollupSegment" => Some("rollup_segment"),
        "SamplingExpressionSegment" => Some("sample_expression"),
        "SchemaReferenceSegment" => Some("schema_reference"),
        "SelectClauseElementSegment" => Some("select_clause_element"),
        "SelectClauseModifierSegment" => Some("select_clause_modifier"),
        "SelectClauseSegment" => Some("select_clause"),
        "SelectPartitionClauseSegment" => Some("partition_clause"),
        "SelectStatementSegment" => Some("select_statement"),
        "SequenceReferenceSegment" => Some("sequence_reference"),
        "SetAssignmentStatementSegment" => Some("set_statement"),
        "SetClauseListSegment" => Some("set_clause_list"),
        "SetClauseSegment" => Some("set_clause"),
        "SetDefaultRoleStatementSegment" => Some("set_default_role_statement"),
        "SetExpressionSegment" => Some("set_expression"),
        "SetNamesStatementSegment" => Some("set_names_statement"),
        "SetOperatorSegment" => Some("set_operator"),
        "SetSchemaStatementSegment" => Some("set_schema_statement"),
        "SetTransactionStatementSegment" => Some("set_transaction_statement"),
        "ShorthandCastSegment" => Some("cast_expression"),
        "SingleIdentifierListSegment" => Some("identifier_list"),
        "SingleIdentifierWildcardSegment" => Some("wildcard_identifier"),
        "SizedArrayTypeSegment" => Some("sized_array_type"),
        "StatementSegment" => Some("statement"),
        "StructLiteralSegment" => Some("struct_literal"),
        "StructTypeSegment" => Some("struct_type"),
        "SymbolSegment" => Some("symbol"),
        "TableConstraintSegment" => Some("table_constraint"),
        "TableEndClauseSegment" => Some("table_end_clause_segment"),
        "TableExpressionSegment" => Some("table_expression"),
        "TableOptionsSegment" => Some("table_options"),
        "TableReferenceSegment" => Some("table_reference"),
        "TablespaceReferenceSegment" => Some("tablespace_reference"),
        "TagReferenceSegment" => Some("tag_reference"),
        "TemporalQuerySegment" => Some("temporal_query"),
        "TimeZoneGrammar" => Some("time_zone_grammar"),
        "TransactionStatementSegment" => Some("transaction_statement"),
        "TriggerReferenceSegment" => Some("trigger_reference"),
        "TruncateStatementSegment" => Some("truncate_table"),
        "TupleSegment" => Some("tuple"),
        "TypedArrayLiteralSegment" => Some("typed_array_literal"),
        "TypedStructLiteralSegment" => Some("typed_struct_literal"),
        "UnorderedSelectStatementSegment" => Some("select_statement"),
        "UnorderedSetExpressionSegment" => Some("set_expression"),
        "UpdateStatementSegment" => Some("update_statement"),
        "UpsertClauseListSegment" => Some("upsert_clause_list"),
        "UseStatementSegment" => Some("use_statement"),
        "UserReferenceSegment" => Some("user_reference"),
        "ValuesClauseSegment" => Some("values_clause"),
        "WhenClauseSegment" => Some("when_clause"),
        "WhereClauseSegment" => Some("where_clause"),
        "WhileStatementSegment" => Some("while_statement"),
        "WhitespaceSegment" => Some("whitespace"),
        "WildcardExpressionSegment" => Some("wildcard_expression"),
        "WildcardIdentifierSegment" => Some("wildcard_identifier"),
        "WindowSpecificationSegment" => Some("window_specification"),
        "WithCheckOptionSegment" => Some("with_check_options"),
        "WithCompoundNonSelectStatementSegment" => Some("with_compound_statement"),
        "WithCompoundStatementSegment" => Some("with_compound_statement"),
        "WithDataClauseSegment" => Some("with_data_clause"),
        "WithFillSegment" => Some("with_fill"),
        "WithNoSchemaBindingClauseSegment" => Some("with_no_schema_binding_clause"),
        "WordSegment" => Some("word"),
        _ => None,
    }
}

pub static DORIS_TABLES: GrammarTables = GrammarTables {
    instructions: INSTRUCTIONS,
    child_ids: CHILD_IDS,
    terminators: TERMINATORS,
    strings: STRINGS,
    aux_data: AUX_DATA,
    aux_data_offsets: AUX_DATA_OFFSETS,
    regex_patterns: REGEX_PATTERNS,
    simple_hints: SIMPLE_HINTS,
    hint_string_indices: HINT_STRING_INDICES,
    simple_hint_indices: SIMPLE_HINT_INDICES,
    segment_type_offsets: SEGMENT_TYPE_OFFSETS,
    segment_class_offsets: SEGMENT_CLASS_OFFSETS,
    casefold_sparse: CASEFOLD_SPARSE,
    trim_chars_sparse: TRIM_CHARS_SPARSE,
    trim_chars_data: TRIM_CHARS_DATA,
    segment_class_types_sparse: SEGMENT_CLASS_TYPES_SPARSE,
    segment_class_types_data: SEGMENT_CLASS_TYPES_DATA,
};

pub fn get_doris_root_grammar_id() -> GrammarId {
    GrammarId(2305)
}
pub fn get_doris_root_grammar_table() -> RootGrammar {
    RootGrammar {
        grammar_id: get_doris_root_grammar_id(),
        tables: &DORIS_TABLES,
    }
}
pub fn get_doris_root_grammar() -> RootGrammar {
    get_doris_root_grammar_table()
}
