/* This is a generated file! */
/* Generated by `utils/build_parsers.py` via `utils/rustify.py` */
/* This process can be run via tox: `tox -e generate-rs` */
#![cfg_attr(rustfmt, rustfmt_skip)]

use sqlfluffrs_types::{
    GrammarInst, GrammarVariant, GrammarFlags,
    ParseMode, GrammarId, GrammarTables,
    SimpleHintData, RootGrammar
};
// Flattening grammar tree...
// Validating tables...
// Validation passed!

// Table Statistics:
//   Instructions:      5770 × 20 bytes =   115400 bytes
//   Child IDs:         4562 ×  4 bytes =    18248 bytes
//   Terminators:         66 ×  4 bytes =      264 bytes
//   Strings:           2418 strings      =    35227 bytes
//   Aux Data:         12815 ×  4 bytes =    51260 bytes
//   Aux Offsets:       5770 ×  4 bytes =    23080 bytes
//   Regex Patterns:       6 patterns     =     1192 bytes
//   Simple Hints:      1370 hints        =    21920 bytes
//   Hint Indices:      2225 ×  4 bytes =     8900 bytes
//   TOTAL:                                    ≈   275491 bytes

pub static INSTRUCTIONS: &[GrammarInst] = &[
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [0] StringParser("ABORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [1] StringParser("ABS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [2] StringParser("ABSENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [3] StringParser("ABSOLUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [4] StringParser("ACCESS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 0, child_count: 60, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [5] OneOf(60 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 60, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [6] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 62, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [7] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 62, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [8] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 62, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [9] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 64, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [10] Ref(AnalyticKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 64, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [11] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 64, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [12] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [13] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [14] Ref(DimensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [15] Ref(ClassKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [16] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [17] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 68, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [18] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 68, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [19] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 68, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [20] Ref(ContainerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 68, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [21] Ref(ContextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 68, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [22] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 70, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [23] Ref(CubeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 70, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [24] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 72, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [25] Ref(DimensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 72, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [26] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [27] Ref(BuildKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [28] Ref(ProcessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [29] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [30] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 76, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [31] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [32] Ref(LinkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [33] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [34] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [35] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [36] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [37] Ref(Dba_recyclebinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [38] Ref(DictionaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [39] Ref(DimensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [40] Ref(DirectiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [41] Ref(DirectoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [42] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [43] Ref(EditionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [44] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [45] Ref(FineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [46] Ref(GrainedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [47] Ref(AuditKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [48] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [49] Ref(HierarchyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [50] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [51] Ref(IndextypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [52] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 87, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [53] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 87, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [54] Ref(JobKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 87, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [55] Ref(ResourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 87, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [56] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 89, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [57] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 89, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [58] Ref(ManagementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 89, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [59] Ref(LibraryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 89, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [60] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 91, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [61] Ref(LockdownKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 91, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [62] Ref(ProfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 91, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [63] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 93, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [64] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 93, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [65] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 93, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [66] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 95, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [67] Ref(MeasureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 95, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [68] Ref(FolderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 95, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [69] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 97, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [70] Ref(MiningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 97, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [71] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 97, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [72] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 99, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [73] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 99, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [74] Ref(PrivilegeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 99, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [75] Ref(OperatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 99, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [76] Ref(OutlineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 99, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [77] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 101, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [78] Ref(LockdownKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 101, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [79] Ref(ProfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 101, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [80] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 103, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [81] Ref(PluggableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 103, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [82] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 103, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [83] Ref(PrivilegeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 103, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [84] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 103, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [85] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 103, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [86] Ref(ProfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 103, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [87] Ref(ProgramKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 103, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [88] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 105, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [89] Ref(PropertyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 105, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [90] Ref(GraphKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 105, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [91] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 107, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [92] Ref(RedactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 107, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [93] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 107, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [94] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 109, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [95] Ref(RemoteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 109, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [96] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 109, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [97] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 111, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [98] Ref(ResourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 111, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [99] Ref(CostKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 111, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [100] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 111, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [101] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 113, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [102] Ref(RollbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 113, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [103] Ref(SegmentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 113, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [104] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 117, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [105] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 117, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [106] Ref(LevelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 117, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [107] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 117, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [108] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 117, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [109] Ref(SchedulerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 117, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [110] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 117, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [111] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 117, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [112] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 119, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [113] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 119, child_count: 5, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [114] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [115] Ref(FirewallKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [116] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [117] Ref(ManagementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [118] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [119] Ref(ProfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [120] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [121] Ref(TranslationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [122] Ref(ProfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [123] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 130, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [124] Ref(TuningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 130, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [125] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 130, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [126] Ref(SynonymKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 130, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [127] Ref(SysguidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 130, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [128] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 130, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [129] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 130, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [130] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 130, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [131] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 130, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [132] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 130, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [133] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 130, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [134] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 130, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [135] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 130, child_count: 49, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [136] OneOf(49 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 179, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [137] Ref(AdministerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 179, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [138] Ref(AdvisorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 179, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [139] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 179, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [140] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 179, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [141] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 179, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [142] Ref(AuditKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 179, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [143] Ref(BackupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 179, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [144] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 181, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [145] Ref(BecomeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 181, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [146] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 181, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [147] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [148] Ref(ChangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [149] Ref(NotificationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [150] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [151] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [152] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [153] Ref(DebugKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [154] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [155] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [156] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [157] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [158] Ref(DiagnosticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [159] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [160] Ref(ExemptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [161] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 188, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [162] Ref(FlashbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 188, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [163] Ref(ArchiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 188, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [164] Ref(AdministerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 188, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [165] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 188, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [166] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 188, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [167] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 188, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [168] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 188, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [169] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 188, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [170] Ref(KeepKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 188, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [171] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 188, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [172] Ref(LogminingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 188, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [173] Ref(ManageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 188, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [174] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 188, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [175] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [176] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [177] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [178] Ref(RefreshKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [179] Ref(PurgeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [180] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 194, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [181] Ref(GlobalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 194, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [182] Ref(QueryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 194, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [183] Ref(RewriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 194, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [184] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 194, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [185] Ref(RedefineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 194, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [186] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 194, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [187] Ref(ResourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 194, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [188] Ref(RestrictedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 194, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [189] Ref(ResumableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 194, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [190] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 194, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [191] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 194, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [192] Ref(SignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 194, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [193] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 196, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [194] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 196, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [195] Ref(RetentionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 196, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [196] Ref(TranslateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 196, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [197] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 196, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [198] Ref(UnderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 196, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [199] Ref(UnlimitedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 196, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [200] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 196, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [201] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 196, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [202] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 196, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [203] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 198, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [204] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [205] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 204, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [206] Ref(AccessPermissionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 204, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [207] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [208] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [209] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [210] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [211] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [212] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 210, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [213] Ref(AccessPermissionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 210, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [214] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 212, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [215] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 212, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [216] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 212, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [217] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 212, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [218] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [219] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 217, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [220] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 217, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [221] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 217, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [222] StringParser("(")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 217, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [223] StringParser(")")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 217, child_count: 14, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [224] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [225] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [226] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [227] Ref(StageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [228] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [229] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [230] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [231] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [232] Ref(StreamKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [233] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [234] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [235] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [236] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [237] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [238] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 235, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [239] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 235, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [240] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 235, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [241] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 237, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [242] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 237, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [243] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 237, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [244] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [245] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [246] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 14, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [247] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 253, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [248] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 253, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [249] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 253, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [250] Ref(StagesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 253, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [251] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 253, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [252] Ref(ProceduresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 253, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [253] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 253, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [254] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 253, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [255] Ref(StreamsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 253, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [256] Ref(TasksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 253, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [257] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 253, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [258] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [259] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [260] Ref(PoliciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [261] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [262] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [263] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [264] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [265] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [266] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [267] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [268] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [269] Ref(FormatsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [270] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 263, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [271] Ref(GrantStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 263, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [272] Ref(RevokeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 263, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [273] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 266, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [274] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 269, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [275] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 272, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [276] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 272, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [277] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 272, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [278] Ref(PackageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 272, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [279] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 274, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [280] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 274, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [281] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 274, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [282] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 274, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [283] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 276, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [284] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 278, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [285] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 280, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [286] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 280, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [287] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 280, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [288] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 280, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [289] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 282, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [290] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [291] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [292] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 284, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [293] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [294] Ref(AccessibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [295] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [296] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 289, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [297] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [298] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 294, child_count: 5, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [299] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [300] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [301] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [302] Ref(PackageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [303] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [304] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [305] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [306] StringParser("ACCESSIBLE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 299, child_count: 1, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [307] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [308] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [309] StringParser("ACCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [310] StringParser("ACCOUNTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [311] StringParser("ACTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [312] StringParser("ACTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [313] StringParser("ADA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [314] StringParser("ADD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [315] StringParser("ADMIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [316] StringParser("ADMINISTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [317] StringParser("ADVANCED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [318] StringParser("ADVISE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [319] StringParser("ADVISOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [320] StringParser("AFTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [321] StringParser("AGGREGATE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [322] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 300, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [323] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [324] Meta("indent")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [325] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [326] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 306, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [327] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [328] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 308, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [329] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 311, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [330] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 311, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [331] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 311, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [332] Meta("dedent")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 311, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [333] StringParser("ALIAS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 311, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [334] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 313, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [335] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 313, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [336] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 313, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [337] StringParser("ALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 313, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [338] StringParser("ALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 313, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [339] StringParser("ALSO")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 313, child_count: 8, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [340] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [341] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [342] Ref(SharedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [343] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [344] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [345] Ref(LinkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [346] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [347] Ref(DatabaseLinkReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [348] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 323, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [349] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 325, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [350] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 325, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [351] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 327, child_count: 6, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [352] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [353] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [354] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [355] Ref(IdentifiedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [356] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [357] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [358] Ref(DBLinkAuthenticationGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 333, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [359] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 335, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [360] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 335, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [361] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 335, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [362] Ref(DBLinkAuthenticationGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 335, child_count: 5, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [363] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 340, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [364] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 340, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [365] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 342, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [366] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 342, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [367] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 342, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [368] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 342, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [369] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 342, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [370] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 345, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [371] Ref(CompileClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 345, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [372] Ref(EditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 345, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [373] Ref(NoneditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 345, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [374] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [375] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [376] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [377] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 13, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [378] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 362, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [379] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 364, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [380] Ref(RebuildKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 364, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [381] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 366, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [382] Ref(ReverseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 366, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [383] Ref(NoreverseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 366, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [384] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 368, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [385] Ref(MonitoringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 368, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [386] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 368, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [387] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [388] Ref(NomonitoringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [389] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [390] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 372, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [391] Ref(ParametersKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 372, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [392] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 375, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [393] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 375, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [394] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [395] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [396] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [397] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [398] Ref(CompileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [399] Ref(LoggingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [400] Ref(NologgingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [401] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [402] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [403] Ref(UnusableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [404] Ref(InvisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [405] Ref(VisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 378, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [406] StringParser("ALTER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 5, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [407] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 383, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [408] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 383, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [409] Ref(PackageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 383, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [410] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 383, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [411] Ref(PackageReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 383, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [412] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 386, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [413] Ref(CompileClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 386, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [414] Ref(EditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 386, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [415] Ref(NoneditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 386, child_count: 6, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [416] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 392, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [417] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 395, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [418] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 395, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [419] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 395, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [420] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 395, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [421] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 395, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [422] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 395, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [423] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 397, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [424] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [425] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [426] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [427] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [428] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [429] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [430] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [431] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [432] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 405, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [433] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 405, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [434] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 405, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [435] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 405, child_count: 1, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [436] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [437] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [438] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 409, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [439] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 409, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [440] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 409, child_count: 12, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [441] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 421, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [442] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 423, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [443] Ref(AdviseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 423, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [444] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 426, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [445] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 426, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [446] Ref(RollbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 426, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [447] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 426, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [448] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [449] Ref(CloseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [450] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [451] Ref(LinkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [452] Ref(DatabaseLinkReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [453] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 434, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [454] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [455] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [456] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [457] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [458] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [459] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [460] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 438, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [461] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 440, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [462] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 440, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [463] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 440, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [464] Ref(GuardKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 440, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [465] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 444, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [466] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 444, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [467] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 444, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [468] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 447, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [469] Ref(DmlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 447, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [470] Ref(DdlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 447, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [471] Ref(QueryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 447, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [472] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [473] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [474] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 449, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [475] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [476] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [477] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 452, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [478] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 455, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [479] Ref(DmlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 455, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [480] Ref(DdlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 455, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [481] Ref(QueryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 455, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [482] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [483] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [484] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [485] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 462, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [486] Ref(DmlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 462, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [487] Ref(DdlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 462, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [488] Ref(QueryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 462, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [489] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [490] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [491] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [492] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 468, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [493] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 468, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [494] Ref(ResumableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 468, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [495] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 470, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [496] Ref(TimeoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 470, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [497] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 470, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [498] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [499] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [500] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [501] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 474, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [502] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 474, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [503] Ref(ResumableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 474, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [504] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 477, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [505] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [506] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [507] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [508] Ref(ShardKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [509] Ref(DdlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [510] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [511] Ref(SyncKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [512] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [513] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 482, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [514] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [515] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 484, child_count: 6, min_times: 1, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [516] AnyNumberOf(6 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [517] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 493, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [518] Ref(Isolation_levelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 493, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [519] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 493, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [520] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 495, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [521] Ref(SerializableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 495, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [522] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [523] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [524] Ref(CommittedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [525] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 500, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [526] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 502, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [527] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 502, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [528] Ref(ConstraintsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 502, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [529] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 502, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [530] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 505, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [531] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 505, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [532] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 505, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [533] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 505, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [534] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [535] Ref(Time_zoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [536] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [537] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [538] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [539] Ref(DbtimezoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [540] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 5, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [541] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 516, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [542] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 516, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [543] Ref(ArchivalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 516, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [544] Ref(VisibilityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 516, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [545] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 516, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [546] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 518, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [547] Ref(ActiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 518, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [548] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 518, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [549] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [550] Ref(ContainerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [551] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [552] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 522, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [553] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [554] Ref(ServiceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [555] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [556] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [557] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 528, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [558] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 528, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [559] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 528, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [560] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 530, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [561] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 530, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [562] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 530, child_count: 6, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [563] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 536, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [564] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 536, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [565] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 536, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [566] Ref(SynonymKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 536, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [567] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 536, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [568] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 536, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [569] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [570] Ref(EditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [571] Ref(NoneditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [572] Ref(CompileKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 539, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [573] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 542, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [574] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 544, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [575] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 546, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [576] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 546, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [577] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 546, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [578] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 548, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [579] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 548, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [580] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 551, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [581] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 553, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [582] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 553, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [583] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [584] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [585] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [586] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 559, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [587] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 559, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [588] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 559, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [589] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 562, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [590] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 564, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [591] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 564, child_count: 5, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [592] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [593] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [594] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [595] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [596] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [597] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [598] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 572, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [599] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 574, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [600] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 574, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [601] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [602] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [603] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 579, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [604] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 581, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [605] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 581, child_count: 5, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [606] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 586, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [607] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 586, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [608] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 589, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [609] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [610] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [611] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [612] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 593, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [613] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 593, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [614] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 596, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [615] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 596, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [616] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 598, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [617] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 598, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [618] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 598, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [619] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 598, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [620] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [621] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [622] Ref(KeepKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [623] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [624] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [625] Ref(OnlineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 602, child_count: 5, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [626] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 607, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [627] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 607, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [628] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 607, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [629] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 607, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [630] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 607, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [631] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 607, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [632] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 611, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [633] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 611, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [634] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 611, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [635] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 611, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [636] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 611, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [637] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [638] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 618, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [639] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 618, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [640] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 618, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [641] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 620, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [642] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 620, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [643] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 620, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [644] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 624, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [645] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 626, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [646] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 626, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [647] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 626, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [648] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 626, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [649] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 626, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [650] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [651] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [652] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [653] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [654] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [655] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [656] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [657] Ref(AlterTableDropColumnGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [658] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 635, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [659] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 635, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [660] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 637, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [661] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 637, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [662] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 637, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [663] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 637, child_count: 1, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [664] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 638, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [665] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 641, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [666] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 641, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [667] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 641, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [668] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 641, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [669] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 645, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [670] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 645, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [671] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 645, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [672] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 645, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [673] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [674] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 651, child_count: 1, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [675] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [676] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [677] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 655, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [678] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [679] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [680] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [681] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [682] Ref(AlterTablePropertiesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [683] Ref(AlterTableColumnClausesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [684] Ref(AlterTableConstraintClauses)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 5, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [685] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 662, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [686] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 662, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [687] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 662, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [688] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 662, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [689] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 662, child_count: 6, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [690] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 668, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [691] Ref(CompileClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 668, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [692] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 668, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [693] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 668, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [694] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [695] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [696] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [697] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [698] Ref(EditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [699] Ref(NoneditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [700] StringParser("ALWAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [701] StringParser("&")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [702] StringParser("ANALYSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [703] StringParser("ANALYTIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [704] StringParser("ANALYZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [705] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [706] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [707] StringParser("ANY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [708] StringParser("APPLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [709] StringParser("ARCHIVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [710] StringParser("ARCHIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [711] StringParser("ARE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 12, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [712] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [713] Ref(PlusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [714] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [715] Ref(DivideSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [716] Ref(MultiplySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [717] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [718] Ref(BitwiseAndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [719] Ref(BitwiseOrSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [720] Ref(BitwiseXorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [721] Ref(BitwiseLShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [722] Ref(BitwiseRShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [723] Ref(ModOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [724] Ref(PowerOperatorSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [725] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [726] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 688, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [727] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 690, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [728] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 690, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [729] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 690, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [730] Ref(SliceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 690, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [731] StringParser("[")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 690, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [732] StringParser("]")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 690, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [733] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 690, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [734] StringParser("ARRAY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 690, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [735] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 693, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [736] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 695, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [737] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 695, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [738] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 695, child_count: 1, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [739] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 696, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [740] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 696, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [741] StringParser("AS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 696, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [742] StringParser("ASC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 696, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [743] StringParser("ASENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 696, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [744] StringParser("ASSERTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 696, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [745] StringParser("ASSIGNMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 696, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [746] StringParser(":=")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 696, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [747] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 699, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [748] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 703, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [749] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 703, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [750] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 706, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [751] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [752] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [753] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [754] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 709, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [755] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [756] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [757] Ref(TriggerCorrelationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [758] Ref(BindVariableSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [759] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 713, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [760] Ref(AssignmentOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 713, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [761] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 713, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [762] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 713, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [763] StringParser("ASYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 713, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [764] StringParser("AT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 713, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [765] StringParser("@")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 713, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [766] StringParser("ATOMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 713, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [767] StringParser("%")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 713, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [768] StringParser("ATTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 713, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [769] StringParser("ATTRIBUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 713, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [770] StringParser("AUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 713, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [771] StringParser("AUTHENTICATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 713, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [772] StringParser("AUTHID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 713, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [773] StringParser("AUTHORIZATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 713, child_count: 1, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [774] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [775] Ref(Auto_incrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [776] StringParser("AUTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [777] StringParser("AUTO_INCREMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [778] StringParser("AVG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [779] StringParser("AVG_ROW_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [780] StringParser("BACKUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [781] StringParser("BACKWARD")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [782] MultiStringParser(12 templates)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 714, child_count: 9, min_times: 0, first_terminator_idx: 0, terminator_count: 2, _padding: 0 }, // [783] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [784] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [785] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [786] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [787] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [788] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [789] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 723, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [790] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [791] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [792] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [793] Ref(ConnectByRootGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [794] Ref(SubstitutionVariableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [795] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [796] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [797] Token("file")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [798] Token("base")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [799] StringParser("BASIC")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [800] Ref(SlashBufferExecutorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [801] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 727, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [802] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 729, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [803] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 731, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [804] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 733, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [805] Ref(SqlplusSetStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 733, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [806] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 733, child_count: 1, min_times: 1, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [807] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 734, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [808] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 734, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [809] Ref(BatchDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 734, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [810] Ref(BatchDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 734, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [811] StringParser("BECOME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 734, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [812] StringParser("BEFORE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 734, child_count: 8, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [813] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [814] Ref(DeclareSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [815] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [816] Ref(OneOrMoreStatementsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [817] Ref(ExceptionBlockGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [818] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [819] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [820] StringParser("BEGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [821] StringParser("BERNOULLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [822] StringParser("BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [823] StringParser("BIGINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [824] StringParser("BINARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 5, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [825] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [826] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [827] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [828] Ref(BooleanBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [829] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [830] Ref(MultisetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [831] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [832] StringParser(":")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 1, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [833] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 748, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [834] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 750, child_count: 5, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [835] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 755, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [836] Ref(BindColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 755, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [837] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 755, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [838] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 757, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [839] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 757, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [840] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 757, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [841] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [842] StringParser("BINDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [843] StringParser("BIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [844] StringParser("BIT_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [845] StringParser("BITMAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [846] StringParser("BITVAR")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [847] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 760, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [848] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [849] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [850] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [851] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 762, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [852] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 764, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [853] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 764, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [854] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 764, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [855] StringParser("^")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 764, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [856] StringParser("BLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 764, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [857] StringParser("BODY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 764, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [858] StringParser("BOOL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 764, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [859] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [860] Ref(AndOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [861] Ref(OrOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [862] StringParser("BOOLEAN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 766, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [863] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [864] Ref(TrueSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [865] Ref(FalseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [866] StringParser("BOTH")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [867] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 771, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [868] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [869] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [870] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 776, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [871] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [872] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [873] Token("bracketed")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 778, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [874] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [875] Ref(UnorderedSetExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [876] StringParser("BREADTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [877] StringParser("BREAK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [878] StringParser("BROWSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [879] StringParser("BUFFER_POOL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 6, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [880] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 787, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [881] Ref(BulkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 787, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [882] Ref(CollectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 787, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [883] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 787, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [884] Meta("implicit_indent")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 787, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [885] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [886] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 791, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [887] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 791, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [888] Ref(BindVariableSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 791, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [889] StringParser("BULK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 791, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [890] StringParser("BULK_EXCEPTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 791, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [891] StringParser("BULK_ROWCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 791, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [892] StringParser("BY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 791, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [893] StringParser("BYTE")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 791, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [894] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [895] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 4, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [896] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [897] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [898] Ref(CTEColumnList)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [899] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 798, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [900] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 801, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [901] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 801, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [902] StringParser("CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 801, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [903] StringParser("CALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 801, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [904] StringParser("CALLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 801, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [905] StringParser("CAPACITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 801, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [906] StringParser("CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 801, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [907] StringParser("CASCADE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 801, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [908] StringParser("CASCADED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 801, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 3, _padding: 0 }, // [909] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 804, child_count: 6, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [910] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 810, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [911] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 810, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 2, _padding: 0 }, // [912] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 1, _padding: 0 }, // [913] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [914] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [915] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [916] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 1, _padding: 0 }, // [917] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [918] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [919] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 8, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [920] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [921] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 819, child_count: 1, min_times: 0, first_terminator_idx: 7, terminator_count: 2, _padding: 0 }, // [922] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 820, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 1, _padding: 0 }, // [923] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 820, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [924] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 820, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [925] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 820, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [926] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 820, child_count: 0, min_times: 0, first_terminator_idx: 9, terminator_count: 1, _padding: 0 }, // [927] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 820, child_count: 0, min_times: 0, first_terminator_idx: 9, terminator_count: 0, _padding: 0 }, // [928] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 820, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [929] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 820, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [930] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 820, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [931] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 820, child_count: 9, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [932] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 829, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [933] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 829, child_count: 2, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [934] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [935] Ref(TriggerPredicatesExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [936] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 831, child_count: 1, min_times: 0, first_terminator_idx: 11, terminator_count: 2, _padding: 0 }, // [937] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 1, _padding: 0 }, // [938] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [939] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [940] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [941] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 13, terminator_count: 1, _padding: 0 }, // [942] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 13, terminator_count: 0, _padding: 0 }, // [943] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [944] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [945] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [946] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [947] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [948] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [949] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [950] StringParser("CASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [951] StringParser("CAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [952] StringParser("::")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [953] StringParser("CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [954] StringParser("CATALOG_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [955] StringParser("CEIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [956] StringParser("CEILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [957] StringParser("CELL_FLASH_CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [958] StringParser("CHAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [959] StringParser("CHANGE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [960] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [961] StringParser("CHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [962] StringParser("CHAR_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [963] StringParser("CHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [964] StringParser("CHARACTER_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [965] StringParser("CHARACTER_SET_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [966] StringParser("CHARACTER_SET_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [967] StringParser("CHARACTER_SET_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [968] StringParser("CHARACTERISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [969] StringParser("CHARACTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [970] StringParser("CHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [971] StringParser("CHECKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [972] StringParser("CHECKPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [973] StringParser("CHECKSUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [974] StringParser("CLASS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [975] StringParser("CLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [976] StringParser("CLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [977] StringParser("CLOSE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [978] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 834, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [979] Ref(CloseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 834, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [980] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [981] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [982] Ref(BindVariableSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [983] StringParser("CLUSTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [984] StringParser("CLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [985] StringParser("COALESCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [986] StringParser("COBOL")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [987] Token("raw")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [988] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [989] StringParser("COLLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [990] StringParser("COLLATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [991] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 838, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [992] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 838, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 1, _padding: 0 }, // [993] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 840, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [994] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 840, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [995] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 840, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [996] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 840, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [997] StringParser("COLLATION_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 840, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [998] StringParser("COLLATION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 840, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [999] StringParser("COLLATION_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 840, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1000] StringParser("COLLECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 840, child_count: 8, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1001] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 848, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1002] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 848, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1003] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 848, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1004] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 848, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1005] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 850, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1006] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 850, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1007] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 850, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1008] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1009] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1010] Ref(ColumnTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1011] Ref(RowTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 853, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1012] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 855, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1013] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 855, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1014] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 855, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1015] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 857, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1016] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 857, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1017] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 857, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1018] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 860, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1019] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 860, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1020] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 860, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1021] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 860, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1022] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 860, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1023] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 860, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1024] StringParser(":")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 860, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1025] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 862, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1026] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1027] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1028] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1029] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1030] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1031] Ref(SequencePseudocolumnGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1032] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 868, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1033] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1034] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1035] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 870, child_count: 10, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1036] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 880, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1037] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 882, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1038] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 882, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1039] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 882, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1040] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 884, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1041] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 884, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1042] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1043] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1044] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 889, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1045] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 889, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1046] Ref(ColumnConstraintDefaultGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 889, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1047] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 891, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1048] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 891, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1049] Ref(NotEnforcedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 891, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1050] Ref(UniqueKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 891, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1051] Ref(AutoIncrementGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 891, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1052] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 893, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1053] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 893, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1054] Ref(NotEnforcedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 893, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1055] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 893, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1056] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1057] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1058] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1059] Ref(ColumnGeneratedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 895, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1060] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1061] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 897, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1062] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 899, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1063] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 900, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1064] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1065] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 902, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1066] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 904, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1067] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 904, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1068] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 904, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1069] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 908, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1070] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 908, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1071] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 911, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1072] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 913, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1073] Anything
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 913, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1074] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1075] Ref(ByteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1076] Ref(CharKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 915, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1077] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 916, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1078] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1079] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 918, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1080] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1081] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1082] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 920, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1083] Ref(IdentityClauseGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 920, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1084] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 920, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1085] StringParser("COLUMN")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 1, _padding: 0 }, // [1086] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 922, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1087] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 922, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1088] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 922, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [1089] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 922, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1090] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1091] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1092] Ref(AttributeIndicatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1093] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1094] StringParser("COLUMN_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1095] StringParser("COLUMN_VALUE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1096] Nothing
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1097] Ref(ColumnsExpressionNameGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1098] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1099] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1100] StringParser("COLUMNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1101] StringParser(",")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1102] StringParser("COMMAND_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1103] StringParser("COMMAND_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 925, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1104] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 927, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1105] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 927, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1106] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 927, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1107] StringParser("COMMENT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 927, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1108] Token("comment")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 927, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1109] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 930, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1110] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 930, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1111] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 930, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1112] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 932, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1113] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 937, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1114] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 939, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1115] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 939, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1116] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 939, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1117] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 941, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1118] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 941, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1119] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 941, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1120] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1121] Ref(OperatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1122] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1123] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1124] Ref(IndextypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1125] Ref(IndexTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1126] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 948, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1127] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 948, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1128] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 948, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1129] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 948, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1130] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 950, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1131] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 950, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1132] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 952, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1133] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 952, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1134] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 952, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1135] StringParser("COMMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 952, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1136] StringParser("COMMITTED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 952, child_count: 8, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1137] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 960, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1138] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 960, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1139] Ref(GreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 960, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1140] Ref(LessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 960, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1141] Ref(GreaterThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 960, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1142] Ref(LessThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 960, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1143] Ref(NotEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 960, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1144] Ref(LikeOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 960, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1145] Ref(IsDistinctFromGrammar)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 960, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1146] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 960, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1147] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 965, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1148] Ref(CompileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 965, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1149] Ref(DebugKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 965, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1150] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 968, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1151] Ref(PackageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 968, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1152] Ref(SpecificationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 968, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1153] Ref(BodyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 968, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1154] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 970, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1155] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1156] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1157] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1158] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 973, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1159] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1160] Ref(ReuseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1161] Ref(SettingsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1162] StringParser("COMPILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1163] StringParser("COMPLETION")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1164] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1165] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1166] StringParser("COMPOUND")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 975, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1167] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 979, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1168] Ref(CompoundKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 979, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1169] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 979, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1170] Ref(DeclareSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 979, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1171] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1172] Ref(TimingPointSectionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 980, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1173] StringParser("COMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 980, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1174] StringParser("COMPUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 980, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1175] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1176] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1177] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1178] StringParser("CONDITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1179] StringParser("CONDITION_NUMBER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1180] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1181] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 985, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1182] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 985, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1183] Ref(ConditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 985, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1184] Ref(NonStandardJoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 985, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1185] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 989, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1186] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 989, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1187] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 989, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1188] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 989, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1189] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 989, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1190] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1191] Ref(Connect_by_rootKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1192] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1193] StringParser("CONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1194] StringParser("CONNECT_BY_ROOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1195] StringParser("CONNECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1196] StringParser("CONNECTION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1197] StringParser("CONSTANT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1198] StringParser("CONSTRAINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1199] StringParser("CONSTRAINT_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1200] StringParser("CONSTRAINT_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1201] StringParser("CONSTRAINT_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1202] StringParser("CONSTRAINTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1203] StringParser("CONSTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1204] StringParser("CONTAINER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1205] StringParser("CONTAINS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1206] StringParser("CONTAINSTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1207] StringParser("CONTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1208] StringParser("CONTINUE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1209] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 994, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1210] Ref(ContinueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 994, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1211] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 994, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1212] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1213] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1214] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1215] StringParser("CONVERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1216] StringParser("CONVERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1217] StringParser("COPY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1218] StringParser("CORR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1219] StringParser("CORRESPONDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1220] StringParser("COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1221] StringParser("COVAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1222] StringParser("COVAR_SAMP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 10, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1223] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1224] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1225] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1226] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1227] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1228] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1229] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1230] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1011, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1231] Ref(SpecificKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1232] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1015, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1233] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1015, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1234] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1015, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1235] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1015, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1236] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1017, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1237] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1020, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1238] Ref(InstanceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1020, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1239] Ref(StaticKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1020, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1240] Ref(ConstructorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1020, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1241] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1020, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1242] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1020, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1243] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1020, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1244] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1022, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1245] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1022, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1246] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1022, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1247] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1248] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1249] Ref(AssignmentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 9, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1250] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1033, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1251] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1033, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1252] Ref(SharedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1033, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1253] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1033, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1254] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1033, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1255] Ref(LinkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1033, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1256] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1033, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1257] Ref(DatabaseLinkReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1033, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1258] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1034, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1259] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1260] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1038, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1261] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1038, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1262] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1263] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1042, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1264] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1042, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1265] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1266] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1267] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1049, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1268] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1049, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1269] Ref(IdentifiedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1049, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1270] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1049, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1271] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1049, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1272] Ref(DBLinkAuthenticationGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1049, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1273] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1051, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1274] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1051, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1275] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1051, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1276] Ref(DBLinkAuthenticationGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1051, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1277] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1053, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1278] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1053, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1279] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1053, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1280] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1281] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1282] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1283] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1284] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 14, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1285] Sequence(14 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1286] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1071, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1287] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1073, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1288] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1073, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1289] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1073, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1290] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1291] Ref(EditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1292] Ref(NoneditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1293] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1294] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1295] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1296] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1297] Ref(ReturnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1298] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1078, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1299] Ref(ColumnTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1078, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1300] Ref(RowTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1078, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1301] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1078, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1302] Ref(SharingClauseGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1078, child_count: 10, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1303] AnyNumberOf(10 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1304] Ref(DefaultCollationClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1305] Ref(InvokerRightsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1306] Ref(AccessibleByClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1307] Ref(DeterministicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1308] Ref(Shard_enableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1309] Ref(ParallelEnableClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1310] Ref(ResultCacheClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1311] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1091, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1312] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1091, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1313] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1091, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1314] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1091, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1315] Ref(PipelinedClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1091, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1316] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1317] Ref(Sql_macroKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1093, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1318] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1097, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1319] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1099, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1320] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1099, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1321] Ref(RightArrowSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1099, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1322] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1101, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1323] Ref(ScalarKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1101, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1324] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1101, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1325] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1103, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1326] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1103, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1327] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1103, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1328] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1105, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1329] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1109, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1330] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1109, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1331] Ref(JavaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1109, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1332] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1109, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1333] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1109, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1334] Ref(BeginEndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1109, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1335] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1109, child_count: 8, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1336] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1117, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1337] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1117, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1338] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1119, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1339] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1119, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1340] Ref(BitmapKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1119, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1341] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1119, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1342] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1119, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1343] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1119, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1344] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1119, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1345] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1122, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1346] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1347] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1348] Ref(OracleIndexPhysicalAttributesSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1349] StringParser("CREATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 8, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1350] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1132, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1351] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1132, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1352] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1132, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1353] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1132, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1354] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1132, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1355] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1132, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1356] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1134, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1357] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1134, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1358] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1137, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1359] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1139, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1360] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1142, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1361] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1142, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1362] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1142, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1363] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1364] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1144, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1365] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1147, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1366] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1367] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1368] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1369] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 14, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1370] Sequence(14 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1163, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1371] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1163, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1372] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1165, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1373] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1165, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1374] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1165, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1375] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1167, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1376] Ref(EditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1167, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1377] Ref(NoneditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1167, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1378] Ref(PackageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1167, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1379] Ref(BodyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1167, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1380] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1167, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1381] Ref(PackageReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1167, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1382] Ref(SharingClauseGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1167, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1383] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1170, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1384] Ref(DefaultCollationClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1170, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1385] Ref(InvokerRightsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1170, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1386] Ref(AccessibleByClauseGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1170, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1387] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1172, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1388] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1172, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1389] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1172, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1390] Ref(DeclareSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1172, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1391] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1177, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1392] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1177, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1393] Ref(OneOrMoreStatementsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1177, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1394] Ref(ExceptionBlockGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1177, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1395] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1177, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1396] Ref(PackageReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1177, child_count: 12, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1397] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1189, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1398] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1189, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1399] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1191, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1400] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1191, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1401] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1191, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1402] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1403] Ref(EditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1404] Ref(NoneditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1405] Ref(PackageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1406] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1407] Ref(PackageReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1408] Ref(SharingClauseGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1193, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1409] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1410] Ref(DefaultCollationClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1411] Ref(InvokerRightsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1412] Ref(AccessibleByClauseGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1413] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1198, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1414] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1198, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1415] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1198, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1416] Ref(DeclareSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1198, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1417] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1198, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1418] Ref(PackageReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1198, child_count: 13, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1419] Sequence(13 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1211, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1420] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1211, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1421] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1422] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1423] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1213, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1424] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1425] Ref(EditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1426] Ref(NoneditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1427] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1428] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1429] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1430] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1431] Ref(SharingClauseGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1215, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1432] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1433] Ref(DefaultCollationClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1434] Ref(InvokerRightsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1435] Ref(AccessibleByClauseGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1218, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1436] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1220, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1437] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1220, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1438] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1220, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1439] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1221, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1440] Ref(DeclareSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1221, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1441] Ref(BeginEndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1221, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1442] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1221, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1443] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1444] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1445] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1446] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1447] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1448] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1229, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1449] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1229, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1450] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1229, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1451] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1229, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1452] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1229, child_count: 7, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1453] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1236, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1454] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1239, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1455] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1239, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1456] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1239, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1457] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1239, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1458] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1242, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1459] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1242, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1460] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1242, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1461] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1242, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1462] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1242, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1463] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1242, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1464] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1244, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1465] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1466] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1467] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1468] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1246, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1469] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1470] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1471] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1472] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1473] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1252, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1474] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1252, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1475] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1252, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1476] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1252, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1477] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1478] Ref(CreateSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1253, child_count: 11, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1479] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1264, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1480] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1264, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1481] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1264, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1482] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1266, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1483] Ref(EditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1266, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1484] Ref(NoneditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1266, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1485] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1266, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1486] Ref(SynonymKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1266, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1487] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1266, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1488] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1266, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1489] Ref(SharingClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1266, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1490] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1266, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1491] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1266, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1492] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1493] Ref(AtSignSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1494] Ref(DatabaseLinkReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 8, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1495] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1276, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1496] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1276, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1497] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1276, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1498] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1276, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1499] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1276, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1500] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1276, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1501] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1276, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1502] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1503] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1283, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1504] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1286, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1505] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1288, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1506] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1507] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1508] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1509] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1510] Ref(OnCommitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1511] Ref(OraclePhysicalAttributesSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1512] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1294, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1513] Ref(OnCommitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1294, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1514] Ref(OraclePhysicalAttributesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1294, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1515] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1294, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1516] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1296, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1517] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1518] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1519] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1301, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1520] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1301, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1521] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1301, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1522] Ref(TableEndClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1301, child_count: 18, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1523] Sequence(18 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1319, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1524] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1319, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1525] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1321, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1526] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1321, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1527] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1321, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1528] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1323, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1529] Ref(EditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1323, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1530] Ref(NoneditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1323, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1531] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1323, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1532] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1323, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1533] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1323, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1534] Ref(SharingClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1323, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1535] Ref(DefaultCollationClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1323, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1536] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1537] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1538] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1330, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1539] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1330, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1540] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1330, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1541] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1332, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1542] Ref(InsteadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1332, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1543] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1332, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1544] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1332, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1545] Ref(DmlEventClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1332, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1546] Ref(ReferencingClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1332, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1547] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1548] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1549] Ref(EachKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1550] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1335, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1551] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1337, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1552] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1339, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1553] Ref(ForwardKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1339, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1554] Ref(ReverseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1339, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1555] Ref(CrosseditionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1339, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1556] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1557] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1558] Ref(FollowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1559] Ref(PrecedesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1560] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1345, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1561] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1345, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1562] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1563] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1564] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1347, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1565] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1349, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1566] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1349, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1567] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1568] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1569] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1354, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1570] Ref(CompoundTriggerBlock)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1354, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1571] Ref(OneOrMoreStatementsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1354, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1572] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1354, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1573] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1354, child_count: 13, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1574] Sequence(13 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1367, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1575] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1367, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1576] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1369, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1577] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1369, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1578] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1369, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1579] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1371, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1580] Ref(EditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1371, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1581] Ref(NoneditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1371, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1582] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1371, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1583] Ref(BodyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1371, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1584] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1371, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1585] Ref(TypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1371, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1586] Ref(SharingClauseGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1371, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1587] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1373, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1588] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1373, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1589] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1373, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1590] Ref(ElementSpecificationGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1373, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1591] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1373, child_count: 13, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1592] Sequence(13 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1386, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1593] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1386, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1594] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1388, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1595] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1388, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1596] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1388, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1597] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1390, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1598] Ref(EditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1390, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1599] Ref(NoneditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1390, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1600] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1390, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1601] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1390, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1602] Ref(TypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1390, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1603] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1390, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1604] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1394, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1605] Ref(OidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1394, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1606] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1394, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1607] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1394, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1608] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1394, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1609] Ref(SharingClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1394, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1610] Ref(DefaultCollationClauseGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1394, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1611] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1612] Ref(InvokerRightsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1613] Ref(AccessibleByClauseGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1396, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1614] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1615] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1616] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1617] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1618] Ref(ObjectTypeAndSubtypeDefGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1619] Ref(VarrayAndNestedTypeSpecGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1400, child_count: 6, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1620] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1621] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1622] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1623] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1624] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1625] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1626] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1627] Ref(IdentifiedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1628] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1412, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1629] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1415, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1630] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1415, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1631] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1415, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1632] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1418, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1633] Ref(HttpKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1418, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1634] Ref(DigestKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1418, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1635] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1636] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1637] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1638] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1639] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1424, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1640] Ref(ExternallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1424, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1641] Ref(GloballyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1424, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1642] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1426, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1643] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1426, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1644] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1428, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1645] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1428, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1646] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1428, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1647] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1430, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1648] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1430, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1649] Ref(AuthenticationKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1430, child_count: 8, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1650] AnyNumberOf(8 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1438, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1651] Ref(DefaultCollationClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1438, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1652] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1653] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1443, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1654] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1445, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1655] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1445, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1656] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1445, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1657] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1448, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1658] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1448, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1659] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1448, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1660] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1448, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1661] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1662] Ref(QuotaKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1451, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1663] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1453, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1664] Ref(SizeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1453, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1665] Ref(UnlimitedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1453, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1666] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1453, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1667] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1453, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1668] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1669] Ref(ProfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1670] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1457, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1671] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1457, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1672] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1457, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1673] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1459, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1674] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1459, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1675] Ref(ExpireKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1459, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1676] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1461, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1677] Ref(AccountKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1461, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1678] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1679] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1680] Ref(UnlockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1463, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1681] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1465, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1682] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1465, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1683] Ref(EditionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1465, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1684] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1468, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1685] Ref(ContainerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1468, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1686] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1468, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1687] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1688] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1689] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1690] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1691] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1692] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1474, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1693] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1474, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1694] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1474, child_count: 12, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1695] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1486, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1696] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1486, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1697] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1486, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1698] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1488, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1699] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1488, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1700] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1488, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1701] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1491, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1702] Ref(EditioningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1491, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1703] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1493, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1704] Ref(EditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1493, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1705] Ref(EditioningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1493, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1706] Ref(NoneditionableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1493, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1707] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1493, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1708] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1493, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1709] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1493, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1710] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1493, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1711] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1493, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1712] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1493, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 1, _padding: 0 }, // [1713] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1495, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1714] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1715] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1716] Ref(BatchDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1717] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1718] StringParser("CREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1719] StringParser("CREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1720] StringParser("CREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1721] StringParser("CREATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1722] StringParser("CRITICAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1723] StringParser("CROSS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1724] StringParser("CROSSEDITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1725] StringParser("CSV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1726] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1727] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1498, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1728] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1500, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1729] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1502, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1730] Ref(CubeFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1502, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1731] Ref(RollupFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1502, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1732] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1505, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1733] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1505, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1734] StringParser("CUME_DIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1505, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1735] StringParser("CURRENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1505, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1736] StringParser("CURRENT_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1505, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1737] StringParser("CURRENT_DEFAULT_TRANSFORM_GROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1505, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1738] StringParser("CURRENT_PATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1505, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1739] StringParser("CURRENT_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1505, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1740] StringParser("CURRENT_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1505, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1741] StringParser("CURRENT_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1505, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1742] StringParser("CURRENT_TRANSFORM_GROUP_FOR_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1505, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1743] StringParser("CURRENT_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1505, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1744] StringParser("CURRVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1505, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1745] StringParser("CURSOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1505, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1746] StringParser("CURSOR_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1505, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1747] StringParser("CYCLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1505, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1748] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1507, child_count: 6, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1749] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1513, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1750] Ref(AuthenticatedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1513, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1751] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1513, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1752] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1513, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1753] Ref(IdentifiedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1513, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1754] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1513, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1755] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1513, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1756] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1757] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1758] Ref(CredentialKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1759] StringParser("DATA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1760] StringParser("DATABASE")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1761] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1517, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1762] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1517, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1763] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1517, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 1, _padding: 0 }, // [1764] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1765] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1766] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1767] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1768] StringParser("DATABASES")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1769] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1521, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1770] RegexParser
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1521, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1771] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1522, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1772] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1522, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1773] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1774] Ref(IntervalDataTypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1775] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1528, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1776] Ref(TimeWithTZGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1528, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1777] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1530, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1778] Ref(DoubleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1530, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1779] Ref(PrecisionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1530, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1780] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1533, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1781] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1535, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1782] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1537, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1783] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1539, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1784] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1539, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1785] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1539, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1786] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1541, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1787] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1541, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1788] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1543, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1789] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1543, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1790] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1543, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1791] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 1545, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1792] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1547, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1793] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1547, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1794] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1547, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1795] Ref(DatatypeIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1547, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1796] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1547, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1797] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1549, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1798] Ref(UnsignedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1549, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1799] Ref(CharCharacterSetGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1549, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1800] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1549, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1801] StringParser("DATE")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1549, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1802] MultiStringParser(1 templates)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1549, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1803] Ref(DatePartFunctionName)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1549, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1804] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1550, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1805] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1553, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1806] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1555, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1807] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1808] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1809] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1557, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1810] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1560, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1811] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1564, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1812] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1564, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1813] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1564, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1814] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1564, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1815] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1564, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1816] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1564, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1817] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1818] Ref(IntervalUnitsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1566, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1819] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1820] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1821] Ref(IntervalUnitsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1822] StringParser("DATETIME")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1823] MultiStringParser(11 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1824] StringParser("DATETIME_INTERVAL_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1825] StringParser("DATETIME_INTERVAL_PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1826] StringParser("DAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1827] StringParser("DAY_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1828] StringParser("DAY_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1829] StringParser("DAY_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1830] StringParser("DAY_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1831] StringParser("DAYOFMONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1832] StringParser("DAYOFWEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1833] StringParser("DAYOFYEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1834] StringParser("DAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1835] StringParser("DBA_RECYCLEBIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1836] StringParser("DBCC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1837] StringParser("DBTIMEZONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1838] StringParser("DDL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1839] StringParser("DEALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1840] StringParser("DEBUG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1841] StringParser("DEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1842] StringParser("DECIMAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1568, child_count: 6, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1843] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1574, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1844] Ref(CursorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1574, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1845] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1574, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1846] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1574, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1847] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1848] Ref(ReturnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1849] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1850] Ref(ColumnTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1851] Ref(RowTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1852] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1579, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1853] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1854] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1855] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1856] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1857] StringParser("DECLARE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1583, child_count: 4, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1858] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1587, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1859] Ref(DeclareKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1587, child_count: 1, min_times: 1, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1860] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1588, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 2, _padding: 0 }, // [1861] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1590, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1862] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1594, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1863] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1598, child_count: 5, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1864] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1865] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1606, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1866] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1606, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1867] Ref(ConstantKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1606, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1868] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1609, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1869] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1609, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1870] Ref(ColumnTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1609, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1871] Ref(RowTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1609, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1872] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1873] Ref(PragmaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1874] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1875] Ref(CollectionTypeDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1876] Ref(RecordTypeDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1877] Ref(RefCursorTypeDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1611, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1878] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1613, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1879] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1613, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1880] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1613, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1881] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1615, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1882] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1883] Ref(AssignmentOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1884] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1885] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1886] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1887] Ref(CreateProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1888] Ref(CreateFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1889] Ref(DeclareCursorVariableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1890] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1891] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1892] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1617, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1893] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1620, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1894] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1620, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1895] Ref(CollationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1620, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1896] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1620, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1897] StringParser("DEFAULT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1620, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1898] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1899] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1900] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1901] StringParser("DEFAULTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1902] StringParser("DEFERRABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1903] StringParser("DEFERRED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1904] StringParser("DEFINED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1905] StringParser("DEFINER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1906] StringParser("DEFINITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1907] StringParser("DEGREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1908] StringParser("DELAY_KEY_WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1909] StringParser("DELAYED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1910] StringParser("DELEGATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1911] StringParser("DELETE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1622, child_count: 4, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1912] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1913] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1914] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1915] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1916] Ref(ReturningClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1917] StringParser("DELETING")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1918] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1919] StringParser("DELIMITER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1920] StringParser("DELIMITERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1921] StringParser("DENSE_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1922] StringParser("DENY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1923] StringParser("DEPTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1924] StringParser("DEREF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1925] StringParser("DERIVED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1926] StringParser("DESC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1927] StringParser("DESCRIBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1626, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1928] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1929] Ref(DescribeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1930] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1931] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1932] StringParser("DESCRIPTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1933] StringParser("DESTROY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1934] StringParser("DESTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1935] StringParser("DETERMINISTIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1936] StringParser("DIAGNOSTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1937] StringParser("DICTIONARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1938] StringParser("DIGEST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1939] StringParser("DIMENSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1940] StringParser("DIRECTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1941] StringParser("DIRECTORIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1942] StringParser("DIRECTORY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1943] StringParser("DISABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1944] StringParser("DISCONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1945] StringParser("DISK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1946] StringParser("DISPATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1947] StringParser("DISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1948] StringParser("DISTINCTROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1949] StringParser("DISTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1950] StringParser("DISTRIBUTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1951] StringParser("DIV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1952] StringParser("/")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1629, child_count: 5, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1953] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1954] Ref(DmlGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1634, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1955] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1635, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1956] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1957] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1958] Ref(DmlGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1959] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1637, child_count: 4, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1960] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1961] Ref(NestedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1962] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1963] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1964] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1965] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1966] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1967] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1968] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1969] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1646, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1970] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1646, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1971] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1648, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1972] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1648, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1973] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1650, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1974] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1650, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1975] StringParser("DML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1650, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1976] StringParser("DO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1650, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1977] StringParser("DOMAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1650, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1978] StringParser(".")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1650, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1979] StringParser("DOUBLE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1650, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1980] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1652, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1981] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1654, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1982] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1654, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1983] Ref(ConstraintsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1654, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1984] Ref(PurgeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1654, child_count: 4, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1985] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1986] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1987] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 5, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1988] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1989] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1990] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1991] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1992] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 5, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1993] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1668, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1994] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1668, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1995] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1668, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1996] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1668, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1997] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1668, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1998] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1671, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1999] Ref(IncludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1671, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2000] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1671, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2001] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2002] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2003] Ref(ConstraintsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 6, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2004] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2005] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1679, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2006] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2007] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2008] Ref(LinkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1679, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2009] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2010] Ref(DatabaseLinkReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 5, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2011] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1684, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2012] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1684, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2013] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1684, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2014] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1684, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2015] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1684, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2016] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1684, child_count: 4, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2017] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2018] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2019] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2020] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2021] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 5, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2022] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1693, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2023] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1693, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2024] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1693, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2025] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1693, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2026] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1693, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2027] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1693, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2028] StringParser("DROP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1693, child_count: 4, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2029] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1697, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2030] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1697, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2031] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1697, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2032] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1697, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2033] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1697, child_count: 5, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2034] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1702, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2035] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1702, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2036] Ref(PackageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1702, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2037] Ref(BodyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1702, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2038] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1702, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2039] Ref(PackageReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1702, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2040] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2041] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2042] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2043] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 5, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2044] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1710, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2045] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1710, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2046] Ref(ProfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1710, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2047] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1710, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2048] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1710, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2049] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1710, child_count: 4, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2050] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2051] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2052] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2053] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2054] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 5, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2055] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1719, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2056] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1719, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2057] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1719, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2058] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1719, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2059] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1719, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2060] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1719, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2061] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2062] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2063] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2064] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 6, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2065] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2066] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2067] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2068] Ref(SynonymKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2069] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2070] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2071] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1728, child_count: 6, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2072] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1734, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2073] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1734, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2074] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1734, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2075] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1734, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2076] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1734, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2077] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1736, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2078] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1736, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2079] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1736, child_count: 4, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2080] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1740, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2081] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1740, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2082] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1740, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2083] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1740, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2084] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1740, child_count: 7, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2085] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2086] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2087] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2088] Ref(BodyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2089] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2090] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1747, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2091] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2092] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2093] Ref(ValidateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1749, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2094] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 5, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2095] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1754, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2096] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1754, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2097] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1754, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2098] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1754, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2099] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1754, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2100] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1754, child_count: 5, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2101] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2102] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2103] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2104] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2105] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2106] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2107] StringParser("DUMMY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2108] StringParser("DUMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2109] StringParser("DUPLICATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2110] StringParser("DYNAMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2111] StringParser("DYNAMIC_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2112] StringParser("DYNAMIC_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2113] StringParser("EACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2114] StringParser("EDITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2115] StringParser("EDITIONABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2116] StringParser("EDITIONING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2117] StringParser("EDITIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2118] StringParser("ELEMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1759, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2119] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1761, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2120] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1762, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2121] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1764, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2122] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1764, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2123] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1767, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2124] Ref(OverridingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1767, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2125] Ref(FinalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1767, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2126] Ref(InstantiableKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1767, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2127] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1768, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2128] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1770, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2129] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1772, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2130] Ref(MemberKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1772, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2131] Ref(StaticKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1772, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2132] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2133] Ref(CreateFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2134] Ref(CreateProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 4, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2135] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1778, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2136] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1778, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2137] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2138] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2139] Ref(OneOrMoreStatementsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2140] StringParser("ELSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2141] StringParser("ELSEIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2142] StringParser("ELSIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2143] StringParser("EMPTY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2144] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2145] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2146] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2147] Ref(EmptyStructLiteralBracketsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2148] StringParser("ENABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2149] StringParser("ENCLOSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2150] StringParser("ENCODING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2151] StringParser("ENCRYPTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2152] StringParser("END-EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2153] StringParser("}")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2154] StringParser("END")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2155] StringParser("ENUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2156] StringParser("EQUALS")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2157] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2158] StringParser("ERRLVL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2159] StringParser("ERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2160] StringParser("ERRORS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2161] StringParser("ESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2162] StringParser("ESCAPED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2163] StringParser("EVALNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2164] StringParser("EVERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2165] StringParser("EXCEPT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1784, child_count: 5, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2166] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2167] Ref(ExceptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2168] Ref(ExceptionWhenHandlerGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1789, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2169] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1790, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2170] Ref(ExceptionWhenHandlerGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1790, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2171] StringParser("EXCEPTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1790, child_count: 6, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2172] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1796, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2173] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1796, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2174] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1798, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2175] Ref(OthersKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1798, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2176] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2177] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1800, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2178] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1801, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2179] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1803, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2180] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1803, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2181] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1803, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2182] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1803, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2183] Ref(OneOrMoreStatementsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1803, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2184] StringParser("EXCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1803, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2185] StringParser("EXCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1803, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2186] StringParser("EXCLUSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1803, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2187] StringParser("EXEC")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1803, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2188] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1805, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2189] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1807, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2190] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1809, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2191] Ref(AtSignSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1809, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2192] Ref(AtSignSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1809, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2193] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1809, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2194] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2195] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2196] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2197] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 8, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2198] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1820, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2199] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1820, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2200] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1820, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2201] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1820, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2202] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2203] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2204] Ref(BulkCollectIntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1822, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2205] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2206] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1824, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2207] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2208] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1828, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2209] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1831, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2210] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1831, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2211] Ref(OutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1831, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2212] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2213] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2214] Ref(OutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2215] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1833, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2216] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1835, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2217] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2218] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2219] Ref(ReturnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2220] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1839, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2221] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1839, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2222] Ref(BulkCollectIntoClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1839, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2223] StringParser("EXECUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1839, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2224] StringParser("EXECUTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1839, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2225] StringParser("EXEMPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1839, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2226] StringParser("EXISTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1839, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2227] StringParser("EXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1839, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2228] StringParser("EXIT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1839, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2229] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1842, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2230] Ref(ExitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1842, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2231] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1842, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2232] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2233] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2234] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2235] StringParser("EXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2236] StringParser("EXPIRE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2237] StringParser("EXPLAIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2238] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1846, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2239] Ref(ExplainKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1846, child_count: 4, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2240] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1850, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2241] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1850, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2242] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1850, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2243] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1850, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2244] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1850, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2245] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1850, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2246] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1852, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2247] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1852, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2248] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1853, child_count: 10, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2249] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1863, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2250] Ref(LikeExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1863, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2251] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2252] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2253] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2254] Ref(InOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2255] Ref(QuantifiedComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1865, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2256] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2257] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2258] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2259] Ref(IsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2260] Ref(IsNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2261] Ref(NotNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2262] Ref(CollateGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 5, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2263] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1873, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2264] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2265] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2266] Ref(Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2267] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2268] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2269] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2270] Ref(PatternMatchingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2271] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 4, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2272] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1879, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2273] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1880, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2274] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1881, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2275] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1881, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2276] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1881, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2277] Ref(NotOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1881, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2278] Ref(PriorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1881, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2279] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1883, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2280] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1883, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2281] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1884, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2282] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1885, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2283] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1887, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2284] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2285] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2286] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2287] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2288] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2289] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1892, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2290] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2291] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2292] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2293] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 1, _padding: 0 }, // [2294] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1897, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2295] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1899, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2296] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1899, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2297] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1902, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2298] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1902, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2299] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1904, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2300] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1906, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2301] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1906, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2302] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1906, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2303] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1907, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2304] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1907, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2305] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1907, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2306] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1907, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2307] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 18, min_times: 0, first_terminator_idx: 24, terminator_count: 1, _padding: 0 }, // [2308] OneOf(18 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2309] Ref(PlusJoinGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2310] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2311] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2312] Ref(TriggerCorrelationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2313] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2314] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1933, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2315] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1933, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2316] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1933, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2317] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1935, child_count: 4, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2318] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2319] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2320] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2321] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2322] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2323] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2324] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2325] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2326] Ref(TypedStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2327] Ref(ArrayExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2328] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2329] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2330] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2331] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2332] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2333] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1944, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2334] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1944, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2335] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1947, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2336] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1949, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2337] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1949, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2338] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1951, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2339] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1951, child_count: 5, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2340] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2341] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2342] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2343] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2344] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2345] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2346] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2347] Ref(SubstitutionVariableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2348] Ref(ImplicitCursorAttributesGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2349] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1959, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2350] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1959, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2351] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2352] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2353] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2354] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2355] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1965, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2356] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2357] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1965, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2358] Ref(AccessorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 7, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2359] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1972, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2360] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1972, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2361] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1972, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2362] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1972, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2363] Ref(TypedStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1972, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2364] Ref(ArrayExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1972, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2365] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1972, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2366] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1972, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2367] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1972, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2368] StringParser("EXTENSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1972, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2369] StringParser("EXTERNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1972, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2370] StringParser("EXTERNALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1972, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2371] StringParser("EXTRACT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1972, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2372] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1972, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2373] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1972, child_count: 5, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2374] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1977, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2375] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1977, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2376] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1979, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2377] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1979, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2378] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1979, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2379] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1981, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2380] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1981, child_count: 1, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2381] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1982, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2382] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1982, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2383] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2384] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2385] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2386] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1986, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2387] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1986, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2388] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1988, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2389] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1988, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2390] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1988, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2391] StringParser("FETCH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1988, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2392] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1991, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2393] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1991, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2394] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2395] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2396] Ref(BindVariableSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2397] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1995, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2398] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1995, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2399] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1997, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2400] Ref(BulkCollectIntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1997, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2401] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1999, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2402] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1999, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2403] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2404] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2405] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2406] StringParser("FIELDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2407] StringParser("FILE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 1, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2408] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2002, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2409] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2410] Ref(BatchSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2411] Ref(ExecuteFileSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2004, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2412] StringParser("FILLFACTOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2413] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2006, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2414] Ref(FilterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2006, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2415] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2009, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2416] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2417] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2418] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2419] StringParser("FILTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2420] StringParser("FINAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2421] StringParser("FINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2422] StringParser("FIRST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2423] StringParser("FLASH_CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2424] StringParser("FLASHBACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2425] StringParser("FLOAT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2426] StringParser("FLOAT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2427] StringParser("FLOAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2428] StringParser("FLOOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2429] StringParser("FLUSH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2430] StringParser("FOLLOWING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2431] StringParser("FOLLOWS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 6, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2432] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2017, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2433] Ref(ForallKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2017, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2434] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2017, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2435] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2017, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2436] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2437] Ref(IterationSteppedControlGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2019, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2438] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2022, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2439] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2022, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2440] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2022, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2441] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2022, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2442] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2443] Ref(SaveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2444] Ref(ExceptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 4, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2445] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2446] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2447] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2448] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2449] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2028, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2450] StringParser("FOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 5, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2451] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2033, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2452] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2033, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2453] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2454] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2037, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2455] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2037, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2456] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2457] Ref(MutableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2458] Ref(ImmutableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2459] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2460] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2041, child_count: 4, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2461] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2045, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2462] Ref(ReverseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2045, child_count: 4, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2463] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2049, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2464] Ref(IterationSteppedControlGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2049, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2465] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2466] Ref(RepeatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2467] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2051, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2468] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2054, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2469] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2057, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2470] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2057, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2471] Ref(IndicesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2057, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2472] Ref(PairsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2057, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2473] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2057, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2474] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2057, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2475] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2060, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2476] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2060, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2477] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2062, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2478] Ref(WhileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2062, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2479] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2062, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2480] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2481] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2482] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2483] Ref(LoopStatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 4, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2484] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2068, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2485] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2068, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2486] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2068, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2487] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2070, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2488] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2070, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2489] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2070, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2490] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2491] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2492] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2493] Ref(WaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2494] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2075, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2495] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2496] Ref(SkipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2497] Ref(LockedKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2498] StringParser("FORALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2499] StringParser("FORCE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2500] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2079, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2501] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2079, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2502] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2079, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2503] StringParser("FOREIGN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2079, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2504] StringParser("FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2079, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2505] StringParser("FORTRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2079, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2506] StringParser("FORWARD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2079, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2507] StringParser("FOUND")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2079, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2508] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2081, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2509] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2081, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2510] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2083, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2511] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2085, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2512] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2513] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2514] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2515] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2089, child_count: 4, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2516] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2517] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2518] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2519] Ref(UnboundedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2520] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2521] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2522] Ref(PrecedingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2523] Ref(FollowingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2095, child_count: 4, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2524] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2099, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2525] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2099, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2526] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2099, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2527] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2528] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2529] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2530] StringParser("FREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2531] StringParser("FREELIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2532] StringParser("FREELISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2533] StringParser("FREETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2534] StringParser("FREETEXTTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2535] StringParser("FREEZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2101, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2536] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2103, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2537] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2103, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2538] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2105, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2539] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2105, child_count: 12, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2540] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2117, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2541] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2117, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2542] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2117, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2543] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2119, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2544] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2119, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2545] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2119, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2546] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2121, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2547] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2121, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2548] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2121, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2549] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2121, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2550] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2121, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2551] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2121, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2552] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2121, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2553] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2121, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2554] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2121, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2555] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2121, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2556] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2121, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2557] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2123, child_count: 7, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2558] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2130, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2559] Ref(PreTableFunctionKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2130, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2560] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2132, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2561] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2135, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2562] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2135, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2563] Ref(TemporalQuerySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 2135, child_count: 1, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2564] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2136, child_count: 4, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2565] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2140, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2566] Ref(FromClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2140, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2567] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2140, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2568] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2140, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2569] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2140, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2570] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2143, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2571] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2143, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2572] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2143, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2573] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2143, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2574] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2143, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2575] Ref(PostTableExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2143, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2576] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2146, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2577] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2148, child_count: 1, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2578] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2149, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2579] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2149, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2580] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2151, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2581] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2154, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2582] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2160, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 2, _padding: 0 }, // [2583] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2584] Ref(MLTableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2585] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2163, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2586] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2166, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2587] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2166, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2588] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2168, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2589] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2168, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2590] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2168, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2591] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2170, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2592] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2170, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [2593] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2170, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2594] Meta(conditional: indented_joins=True -> indent)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2170, child_count: 1, min_times: 0, first_terminator_idx: 27, terminator_count: 2, _padding: 0 }, // [2595] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2171, child_count: 1, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2596] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2172, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2597] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2174, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2598] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2174, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2599] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2174, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2600] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2176, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2601] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2176, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2602] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2176, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2603] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2604] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2605] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2606] Meta(conditional: indented_joins=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2607] StringParser("FROM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2608] StringParser("FULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2609] StringParser("FULLTEXT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2178, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2610] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2181, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2611] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2181, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2612] Ref(NamedArgumentSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2181, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2613] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2181, child_count: 13, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2614] AnyNumberOf(13 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2615] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2616] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2197, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2617] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2197, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2618] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2197, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2619] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2197, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2620] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2201, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2621] Ref(TrimParametersGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 2201, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2622] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2202, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2623] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2202, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2624] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2202, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2625] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2202, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2626] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2205, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2627] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2207, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2628] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2207, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2629] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2207, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2630] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2207, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2631] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2207, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2632] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2209, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2633] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2209, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2634] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2635] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2211, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2636] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2637] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2638] Ref(AggregateOrderByClause)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2213, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2639] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2215, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2640] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2215, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2641] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2215, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2642] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2218, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2643] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2221, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2644] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2221, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2645] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2221, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2646] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2221, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2647] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2221, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2648] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2649] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2650] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2651] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2652] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2653] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2654] Ref(EmptyStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2655] Ref(ListaggOverflowClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2656] Ref(JSONObjectContentSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2657] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2225, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2658] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2659] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2228, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2660] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2231, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2661] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2231, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2662] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2231, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2663] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2664] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2665] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2666] StringParser("FUNCTION")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2667] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2233, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2668] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2235, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 1, _padding: 0 }, // [2669] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2236, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2670] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2238, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2671] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2238, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2672] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2238, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2673] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2238, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2674] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2240, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 1, _padding: 0 }, // [2675] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2242, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2676] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2242, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2677] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2242, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2678] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2242, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2679] Ref(AtSignSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2242, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2680] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2681] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2682] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2246, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2683] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2249, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2684] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2249, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2685] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2252, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2686] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2252, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2687] Ref(ColumnTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2252, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2688] Ref(RowTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2252, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2689] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2254, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2690] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2256, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2691] Ref(AssignmentOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2256, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2692] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2256, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2693] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2256, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2694] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2695] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2696] Ref(OutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2697] Ref(NocopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2260, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2698] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2699] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2700] Ref(ColumnTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2701] Ref(RowTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2702] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2266, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2703] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2268, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2704] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2268, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2705] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2273, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2706] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2275, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2707] Ref(JsonTableFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2275, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2708] Ref(JsonTableFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2275, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2709] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2710] Ref(XmlAttributesFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2711] Ref(XmlAttributesFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2277, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2712] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2713] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2714] Ref(DateTimeFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2715] Ref(ColumnsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2716] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2281, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2717] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 2283, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2718] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2284, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2719] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2720] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2721] Ref(ColumnsExpressionFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2722] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2723] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2724] Ref(PostFunctionGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2725] StringParser("FUNCTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2726] StringParser("FUSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2727] StringParser("FUTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2728] StringParser("G")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2729] StringParser("GENERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2730] StringParser("GENERATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2731] StringParser("GET")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2732] TypedParser("glob_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2733] StringParser("GLOBAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2734] StringParser("GLOBALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2735] StringParser("GO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2736] StringParser("GOTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2737] StringParser("GRANT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2287, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2738] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2739] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2289, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2740] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2291, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2741] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2293, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2742] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2295, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2743] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2744] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2300, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2745] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2303, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2746] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2303, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2747] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2303, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2748] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2303, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2749] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2303, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2750] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2305, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2751] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2305, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2752] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2309, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2753] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2309, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2754] Ref(IdentifiedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2309, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2755] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2309, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2756] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2757] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2311, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2758] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2314, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2759] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2314, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2760] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2316, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2761] Ref(AdminKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2316, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2762] Ref(DelegateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2316, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2763] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2316, child_count: 8, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2764] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2765] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2766] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2767] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2326, child_count: 6, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2768] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2769] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2770] Ref(DirectoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2771] Ref(EditionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2772] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2773] Ref(MiningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2774] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2775] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2336, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2776] Ref(JavaKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2336, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2777] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2338, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2778] Ref(SourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2338, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2779] Ref(ResourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2338, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2780] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2781] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2782] Ref(TranslationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2783] Ref(ProfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2784] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2343, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2785] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2345, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2786] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2787] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2788] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2789] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2790] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2791] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2347, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2792] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2350, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2793] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2350, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2794] Ref(HierarchyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2350, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2795] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2350, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2796] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2353, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2797] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2353, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2798] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2353, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2799] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2353, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2800] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2355, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2801] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2802] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2803] Ref(ConstraintsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2804] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2357, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2805] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2360, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2806] Ref(ContainerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2360, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2807] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2360, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2808] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2809] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2810] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2362, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2811] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2365, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2812] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2365, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2813] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2365, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2814] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2365, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2815] StringParser("GRANTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2365, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2816] StringParser("GRANTS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2365, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2817] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2367, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2818] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2369, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2819] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2369, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2820] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2369, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2821] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2371, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2822] Ref(RawNotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2371, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2823] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2371, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2824] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2371, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2825] StringParser("GREATEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2371, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2826] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2827] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2828] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2376, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2829] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2380, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2830] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2380, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2831] Ref(GroupingSetsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2380, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2832] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2380, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2833] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2381, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 1, _padding: 0 }, // [2834] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2383, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2835] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2386, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2836] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2386, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2837] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2386, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2838] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2386, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2839] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2386, child_count: 6, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2840] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2392, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2841] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2842] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2843] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2844] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2845] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2846] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2847] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2848] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2849] StringParser("GROUP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 1, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2850] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2395, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 1, _padding: 0 }, // [2851] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2397, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2852] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2853] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2854] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2855] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2856] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2403, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2857] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2403, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2858] StringParser("GROUPING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2403, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2859] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2860] Ref(GroupingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2861] Ref(SetsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2406, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2862] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2409, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2863] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2411, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2864] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2413, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2865] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2413, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2866] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2413, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2867] StringParser("GROUPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2413, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2868] StringParser("GUARD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2413, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2869] StringParser("HANDLER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2413, child_count: 4, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2870] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2417, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2871] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2417, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2872] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2873] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2874] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2422, child_count: 5, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2875] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2427, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2876] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2877] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2878] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2879] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2880] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2881] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2882] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2883] StringParser("HAVING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2884] StringParser("HEADER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2885] StringParser("HEAP")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2886] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2431, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2887] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2433, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2888] Ref(ConnectByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2433, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2889] Ref(StartWithClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2433, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2890] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2891] Ref(StartWithClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2892] Ref(ConnectByClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2893] StringParser("HIERARCHY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2894] StringParser("HIGH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2895] StringParser("HIGH_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2896] StringParser("HOLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2897] StringParser("HOLDLOCK")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2898] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2899] StringParser("HOST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2900] StringParser("HOSTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2901] StringParser("HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2902] StringParser("HOUR_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2903] StringParser("HOUR_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2904] StringParser("HOUR_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2905] StringParser("HTTP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2906] StringParser("IDENTIFIED")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2907] Token("identifier")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2435, child_count: 5, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2908] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2440, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2909] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2440, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2910] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2442, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2911] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2442, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2912] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2445, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2913] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2445, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2914] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2445, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2915] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2448, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2916] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2448, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2917] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2448, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2918] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2451, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2919] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2451, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2920] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2451, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2921] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2453, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2922] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2453, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2923] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2455, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2924] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2455, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2925] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2455, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2926] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2455, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2927] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2455, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2928] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2929] Ref(IdentityOptionsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2930] StringParser("IDENTITY")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2458, child_count: 8, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2931] AnyNumberOf(8 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2466, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2932] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2469, child_count: 5, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2933] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2474, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2934] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2935] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2936] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2937] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2478, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2938] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2478, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2939] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2478, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2940] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2478, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2941] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2478, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2942] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2478, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2943] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2478, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2944] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2480, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2945] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2480, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2946] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2480, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2947] Ref(NomaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2480, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2948] Ref(NominvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2480, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2949] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2480, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2950] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2480, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2951] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2480, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2952] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2480, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2953] Ref(NoorderKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2480, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2954] StringParser("IDENTITY_INSERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2480, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2955] StringParser("IDENTITYCOL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2480, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2956] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2483, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2957] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2483, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2958] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2485, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2959] Ref(TriggerPredicatesExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2485, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2960] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2485, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2961] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2485, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2962] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2487, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2963] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2487, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2964] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2487, child_count: 8, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2965] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2495, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2966] Ref(IfClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2495, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2967] Ref(OneOrMoreStatementsGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2495, child_count: 1, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2968] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2496, child_count: 6, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2969] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2970] Ref(ElsifKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2502, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2971] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2972] Ref(TriggerPredicatesExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2973] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2974] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2975] Ref(OneOrMoreStatementsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2504, child_count: 4, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2976] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2508, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2977] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2508, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2978] Ref(OneOrMoreStatementsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2508, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2979] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2508, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2980] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2508, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2981] StringParser("IF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2508, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2982] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2511, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2983] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2511, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2984] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2511, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2985] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2511, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2986] StringParser("IGNORE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2511, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2987] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2513, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2988] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2989] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2990] Ref(RespectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2991] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2992] StringParser("ILIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2993] StringParser("IMMEDIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2994] StringParser("IMMUTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2995] StringParser("IMPLEMENTATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2515, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2996] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2997] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2998] Ref(AttributeIndicatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2518, child_count: 6, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2999] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3000] Ref(IsopenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3001] Ref(FoundKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3002] Ref(NotfoundKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3003] Ref(RowcountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3004] Ref(Bulk_rowcountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3005] Ref(Bulk_exceptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3006] StringParser("IMPLICIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3007] StringParser("IMPORTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3008] StringParser("IN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3009] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2527, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3010] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2527, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3011] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2527, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3012] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2529, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3013] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2532, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3014] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2534, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3015] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2536, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3016] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2536, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3017] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2536, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3018] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2536, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3019] StringParser("INCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2536, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3020] StringParser("INCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2536, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3021] StringParser("INCREMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2536, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3022] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2538, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3023] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2538, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3024] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2540, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3025] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2540, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3026] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2540, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3027] StringParser("INDEX")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2540, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 1, _padding: 0 }, // [3028] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [3029] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3030] StringParser("INDEXTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3031] StringParser("INDICATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3032] StringParser("INDICES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3033] StringParser("INFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3034] StringParser("INFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3035] StringParser("INHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3036] StringParser("INHERITANY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3037] StringParser("INHERITS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3038] StringParser("INITIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3039] StringParser("INITIALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3040] StringParser("INITIALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3041] StringParser("INITRANS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3042] StringParser("INMEMORY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3043] StringParser("INNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3044] StringParser("INOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3045] StringParser("INPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3046] StringParser("INSENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3047] StringParser("INSERT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2542, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3048] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2544, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3049] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2544, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3050] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2546, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3051] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2549, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3052] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2553, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3053] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2553, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3054] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2555, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3055] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2555, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3056] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2558, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3057] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2558, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3058] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2558, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3059] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2561, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3060] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2563, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3061] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2563, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3062] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2565, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3063] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2567, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3064] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2569, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3065] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2569, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3066] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3067] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2571, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3068] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3069] Ref(SetClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3070] Ref(ReturningClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2573, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3071] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2575, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3072] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2577, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3073] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2577, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3074] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2579, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3075] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2579, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3076] Ref(PositionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2579, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3077] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2579, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3078] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2584, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3079] Ref(LogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2584, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3080] Ref(ErrorsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2584, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3081] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3082] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3083] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2586, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3084] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2589, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3085] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2589, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3086] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2592, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3087] Ref(RejectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2592, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3088] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2592, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3089] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2594, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3090] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2594, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3091] Ref(UnlimitedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2594, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3092] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2597, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3093] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2597, child_count: 1, min_times: 1, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3094] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2598, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3095] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3096] StringParser("INSERT_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3097] StringParser("INSERTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3098] StringParser("INSTANCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3099] StringParser("INSTANTIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3100] StringParser("INSTEAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3101] StringParser("INT1")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3102] StringParser("INT2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3103] StringParser("INT3")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3104] StringParser("INT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3105] StringParser("INT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3106] StringParser("INT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3107] StringParser("INTEGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3108] StringParser("INTEGRATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3109] StringParser("INTEGRATIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3110] StringParser("INTERSECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3111] StringParser("INTERSECTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2601, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3112] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2603, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3113] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2603, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3114] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2605, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3115] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3116] Ref(YearKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2609, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3117] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2612, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3118] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2612, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3119] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2612, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3120] Ref(MonthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2612, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3121] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2617, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3122] Ref(DayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2617, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3123] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3124] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3125] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3126] Ref(SecondKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2620, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3127] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3128] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2623, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3129] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2625, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3130] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2625, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3131] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2628, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3132] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2630, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3133] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2630, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3134] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2632, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3135] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2632, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3136] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2632, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3137] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2632, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3138] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2634, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3139] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2634, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3140] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2636, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3141] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2636, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3142] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2636, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3143] StringParser("INTERVAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2636, child_count: 6, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3144] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2642, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3145] Ref(YearKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2642, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3146] Ref(MonthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2642, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3147] Ref(DayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2642, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3148] Ref(HourKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2642, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3149] Ref(MinuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2642, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3150] Ref(SecondKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2642, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3151] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2644, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3152] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2644, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3153] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2646, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3154] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2649, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3155] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2649, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3156] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2649, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3157] Ref(BindVariableSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2649, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3158] StringParser("INTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2649, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3159] StringParser("INVISIBLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2649, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3160] StringParser("INVOKER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2649, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3161] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3162] Ref(AuthidKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2651, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3163] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2653, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3164] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2653, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3165] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2653, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3166] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2656, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3167] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2661, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3168] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2661, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3169] Ref(NanLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2661, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3170] Ref(UnknownLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2661, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3171] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2661, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3172] Ref(NormalizedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2661, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3173] Ref(EmptyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2661, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3174] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2664, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3175] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2664, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3176] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2664, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3177] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2667, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3178] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2669, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3179] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3180] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3181] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2671, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3182] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2675, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3183] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2675, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3184] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2675, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3185] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2675, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3186] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2675, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3187] StringParser("IS")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2675, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3188] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2675, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3189] StringParser("ISAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2675, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3190] StringParser("ISNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2675, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3191] StringParser("ISOLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2675, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3192] StringParser("ISOLATION_LEVEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2675, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3193] StringParser("ISOPEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2675, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3194] StringParser("ITERATE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2675, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3195] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2679, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3196] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2679, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3197] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2679, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3198] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2679, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3199] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2682, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3200] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2682, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3201] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2682, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3202] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2682, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3203] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2687, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3204] Ref(IterationBoundsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2687, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3205] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2687, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3206] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2687, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3207] Ref(IterationBoundsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2687, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3208] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2689, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3209] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2689, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3210] Ref(StepKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2689, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3211] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2691, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3212] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2693, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3213] Ref(JSONRegularEntrySegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2693, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3214] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2695, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3215] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2695, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3216] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2695, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3217] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2695, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3218] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2700, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3219] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2702, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3220] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2702, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3221] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2704, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3222] Ref(JSONEntrySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2704, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3223] Ref(JSONOnNullClause)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2704, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3224] Ref(JSONReturningClause)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2704, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3225] Ref(StrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2704, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3226] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2707, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3227] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2707, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3228] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2707, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3229] Ref(KeysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2707, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3230] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2710, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3231] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2712, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3232] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2712, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3233] Ref(AbsentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2712, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3234] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2712, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3235] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2712, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3236] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2713, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3237] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3238] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2720, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3239] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2720, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3240] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2720, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3241] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2720, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3242] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2720, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3243] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2722, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3244] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2722, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3245] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3246] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3247] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3248] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2724, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3249] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2726, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3250] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2726, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3251] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2729, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3252] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2732, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3253] Ref(VarcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2732, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3254] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2735, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3255] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2737, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3256] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2737, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3257] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3258] Ref(ByteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3259] Ref(CharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2739, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3260] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2741, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3261] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2741, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3262] Ref(TypenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2741, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3263] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2743, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3264] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3265] Ref(ClobKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3266] Ref(BlobKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3267] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3268] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3269] StringParser("JAVA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3270] StringParser("JOB")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3271] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2748, child_count: 7, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3272] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3273] Ref(ConditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3274] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3275] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2755, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3276] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2756, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3277] Ref(NestedJoinGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2756, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3278] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3279] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3280] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2760, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3281] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2762, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3282] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2762, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3283] Ref(JoinUsingConditionGrammar)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2762, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3284] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2762, child_count: 6, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3285] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2768, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3286] Ref(UnconditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2768, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3287] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2768, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3288] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2768, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3289] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2768, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3290] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2772, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3291] Ref(ExtendedNaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2772, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3292] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2772, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3293] StringParser("JOIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2772, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3294] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2773, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3295] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2773, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3296] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2775, child_count: 2, min_times: 1, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3297] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2777, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3298] Ref(PivotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2777, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3299] Ref(UnpivotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2777, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3300] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2777, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3301] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2781, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3302] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2781, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3303] Meta(conditional: indented_on_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2781, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3304] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2783, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3305] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2786, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3306] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2786, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3307] Meta(conditional: indented_on_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2786, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3308] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2788, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3309] Ref(InnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2788, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3310] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2790, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3311] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2793, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3312] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2793, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3313] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2793, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3314] Ref(RightKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2793, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3315] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2793, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3316] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2797, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3317] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2797, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3318] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2800, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3319] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2802, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3320] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2802, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3321] StringParser("JSON")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2802, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3322] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2805, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3323] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2808, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3324] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2808, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3325] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2808, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3326] Ref(OrdinalityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2808, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3327] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3328] Ref(NestedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3329] Ref(PathKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3330] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3331] Ref(JsonTableColumnsClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3332] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2817, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3333] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2817, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3334] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2817, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3335] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2819, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3336] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2819, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3337] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2819, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3338] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2821, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3339] Ref(PathKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2821, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3340] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2821, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3341] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2822, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3342] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2825, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3343] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2828, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3344] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2828, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3345] Ref(ErrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2828, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3346] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2830, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3347] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2830, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3348] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2830, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3349] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2830, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3350] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2832, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3351] Ref(EmptyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2832, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3352] Ref(ErrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2832, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3353] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2834, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3354] Ref(ColumnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2834, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3355] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2837, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3356] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2839, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3357] Ref(JsonTableColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2839, child_count: 7, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3358] Bracketed(5 elements -> 5 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2846, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3359] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2846, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3360] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2846, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3361] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2846, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3362] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2847, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3363] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2850, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3364] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2852, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3365] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2852, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3366] Ref(ErrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2852, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3367] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2852, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3368] Ref(ErrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2852, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3369] Ref(JsonTableColumnsClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2852, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3370] StringParser("JSON_TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2852, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3371] StringParser("K")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2852, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3372] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2854, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3373] Ref(KeepKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2854, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3374] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2857, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3375] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2860, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3376] Ref(Dense_rankKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2860, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3377] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3378] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3379] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3380] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3381] StringParser("KEEP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3382] StringParser("KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3383] StringParser("KEY_MEMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3384] StringParser("KEY_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3385] StringParser("KEYS")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3386] Token("keyword")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3387] StringParser("KILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3388] StringParser("LANCOMPILER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3389] StringParser("LANGUAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3390] StringParser("LARGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3391] StringParser("LAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3392] StringParser("LAST_INSERT_ID")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3393] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3394] StringParser("LATERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3395] StringParser("LEADING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3396] StringParser("LEAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3397] StringParser("LEAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3398] StringParser("LEFT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3399] StringParser("LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3400] StringParser("LESS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2862, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3401] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2864, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3402] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2866, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3403] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2866, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3404] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2866, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3405] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3406] Ref(RawNotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3407] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3408] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3409] StringParser("LEVEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3410] StringParser("LIBRARY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2868, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3411] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2871, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3412] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2873, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3413] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2873, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3414] Ref(LikeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2873, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3415] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2873, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3416] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2875, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3417] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2875, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3418] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2875, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3419] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2878, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3420] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2878, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3421] Ref(RlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2878, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3422] Ref(IlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2878, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3423] StringParser("LIKE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2878, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3424] TypedParser("like_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2878, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3425] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2883, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3426] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2883, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3427] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2885, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3428] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2888, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3429] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2891, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3430] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2891, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3431] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2891, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3432] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2891, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3433] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2893, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3434] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2895, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3435] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2895, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3436] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2897, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3437] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2897, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3438] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2897, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3439] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2899, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3440] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2899, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3441] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2899, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3442] StringParser("LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2899, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3443] StringParser("LINENO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2899, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3444] StringParser("LINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2899, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3445] StringParser("LINK")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2899, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3446] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2899, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3447] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2902, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3448] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2902, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3449] Ref(OverflowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2902, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3450] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2904, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3451] Ref(ErrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2904, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3452] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2908, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3453] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2908, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3454] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2908, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3455] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2910, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3456] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2910, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3457] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2910, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3458] Ref(CountKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2910, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3459] StringParser("LISTEN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2910, child_count: 14, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3460] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3461] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3462] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3463] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3464] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3465] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3466] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3467] Ref(TriggerCorrelationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3468] Ref(BindVariableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3469] Ref(LevelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3470] Ref(RownumKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3471] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3472] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3473] Ref(TypedArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3474] Ref(ObjectLiteralSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3475] Token("literal")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3476] Token("literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3477] StringParser("LN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3478] StringParser("LOAD")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3479] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3480] StringParser("LOCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3481] StringParser("LOCALTIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3482] StringParser("LOCALTIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3483] StringParser("LOCATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3484] StringParser("LOCATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3485] StringParser("LOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3486] StringParser("LOCKDOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3487] StringParser("LOCKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3488] StringParser("LOCKING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3489] StringParser("LOCKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3490] StringParser("LOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3491] StringParser("LOGGING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3492] StringParser("LOGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3493] StringParser("LOGMINING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3494] StringParser("LOGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3495] StringParser("LONG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3496] StringParser("LONGBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3497] StringParser("LONGTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3498] StringParser("LOOP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 8, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3499] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 2932, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3500] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2933, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3501] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2933, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3502] Ref(LoopKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2933, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3503] Ref(OneOrMoreStatementsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2933, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3504] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2933, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3505] Ref(LoopKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2933, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3506] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2933, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3507] StringParser("LOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2933, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3508] StringParser("LOW_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2933, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3509] StringParser("LOWER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2933, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3510] StringParser("M")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2933, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3511] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2937, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3512] Ref(MlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2937, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3513] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2937, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3514] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2937, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3515] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2941, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3516] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3517] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3518] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2943, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3519] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2945, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3520] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2945, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3521] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3522] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3523] StringParser("MANAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3524] StringParser("MAP")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3525] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3526] StringParser("MASKING")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3527] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3528] StringParser("MATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3529] StringParser("MATCHED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3530] StringParser("MATERIALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3531] StringParser("MAX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3532] StringParser("MAX_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3533] StringParser("MAXEXTENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3534] StringParser("MAXSIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3535] StringParser("MAXTRANS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3536] StringParser("MAXVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3537] StringParser("MEASURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3538] StringParser("MEDIUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3539] StringParser("MEDIUMBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3540] StringParser("MEDIUMINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3541] StringParser("MEDIUMTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3542] StringParser("MEMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3543] StringParser("MEMCOMPRESS")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3544] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2948, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3545] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2953, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3546] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2953, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3547] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2953, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3548] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2953, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3549] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2955, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3550] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2955, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3551] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2955, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3552] StringParser("MERGE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2955, child_count: 2, min_times: 1, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3553] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3554] Ref(MergeMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3555] Ref(MergeNotMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2957, child_count: 7, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3556] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2964, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3557] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2964, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3558] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2964, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3559] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2966, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3560] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2966, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3561] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2966, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3562] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2966, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3563] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2968, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3564] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2968, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3565] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2968, child_count: 8, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3566] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2976, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3567] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2976, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3568] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2976, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3569] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2976, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3570] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2978, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3571] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2978, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3572] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2978, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3573] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2978, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3574] Ref(MergeInsertClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2978, child_count: 12, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3575] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2990, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3576] Ref(MergeIntoLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2990, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3577] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2992, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3578] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2992, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3579] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2992, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3580] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2992, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3581] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2996, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3582] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2996, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3583] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2996, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3584] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2998, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3585] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3001, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3586] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3001, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3587] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3001, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3588] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3003, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3589] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3003, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3590] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3003, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3591] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3003, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3592] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3003, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3593] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3003, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3594] Ref(MergeMatchSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3003, child_count: 6, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3595] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3596] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3597] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3598] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3599] Ref(ReturningClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3600] StringParser("MESSAGE_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3601] StringParser("MESSAGE_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3602] StringParser("MESSAGE_TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3603] StringParser("METHOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3604] StringParser("MIDDLEINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3605] StringParser("MILLISECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3606] StringParser("MIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3607] StringParser("MIN_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3608] StringParser("MINEXTENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3609] StringParser("MINING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3610] StringParser("MINUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3611] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3612] StringParser("MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3613] StringParser("MINUTE_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3614] StringParser("MINUTE_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3615] StringParser("MINVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3616] StringParser("ML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3617] StringParser("MLSLABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3618] StringParser("MOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3619] StringParser("MOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3620] StringParser("MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3621] StringParser("MODEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3622] StringParser("MODIFIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3623] StringParser("MODIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3624] StringParser("MODULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3625] StringParser("%")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3626] StringParser("MONITOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3627] StringParser("MONITORING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3628] StringParser("MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3629] StringParser("MONTHNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3630] StringParser("MORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3631] StringParser("MOVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3632] StringParser("MOVEMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3633] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3634] StringParser("MULTISET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3009, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3635] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3012, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3636] Ref(MultisetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3012, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3637] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3015, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3638] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3015, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3639] Ref(IntersectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3015, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3640] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3015, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3641] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3017, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3642] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3017, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3643] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3017, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3644] StringParser("MUMPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3017, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3645] StringParser("MUTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3017, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3646] StringParser("MYISAM")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3017, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3647] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3017, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3648] StringParser("NAME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3017, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3649] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3020, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3650] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3020, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3651] Ref(RightArrowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3020, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3652] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3020, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3653] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3023, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3654] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3023, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3655] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3023, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3656] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3025, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3657] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 3025, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3658] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3028, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3659] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3028, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3660] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3032, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3661] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3032, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3662] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3034, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3663] Ref(NamedWindowExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3034, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3664] StringParser("NAMES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3034, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3665] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3034, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3666] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3034, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3667] StringParser("NATIONAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3034, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3668] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3669] Ref(NaturalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3670] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3671] StringParser("NATURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3672] StringParser("NCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3673] StringParser("NCLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3674] StringParser("-")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3675] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3676] StringParser("NESTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3677] StringParser("NESTED_TABLE_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3678] StringParser("NESTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3679] StringParser("NEW")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3680] Token("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3681] StringParser("NEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3682] StringParser("NEXTVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3683] StringParser("NO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3684] StringParser("NO_WRITE_TO_BINLOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3685] StringParser("NOAUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3686] StringParser("NOCACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3687] StringParser("NOCHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3688] StringParser("NOCOMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3689] StringParser("NOCOPY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3690] StringParser("NOCREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3691] StringParser("NOCREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3692] StringParser("NOCREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3693] StringParser("NOCYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3694] StringParser("NOINHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3695] StringParser("NOLOGGING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3696] StringParser("NOLOGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3697] StringParser("NOMAXVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3698] StringParser("NOMINVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3699] StringParser("NOMONITORING")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3036, child_count: 6, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3700] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3042, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3701] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3042, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3702] Ref(UnorderedSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3042, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3703] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3045, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3704] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3045, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3705] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3706] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3048, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3707] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3051, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3708] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3051, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3709] Ref(BracketedSetExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3051, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3710] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3051, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3711] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3055, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3712] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3055, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3713] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3055, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3714] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3055, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3715] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3055, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3716] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3058, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3717] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3058, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3718] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3060, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3719] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3720] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3721] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3722] StringParser("NONCLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3723] StringParser("NONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3724] StringParser("NONEDITIONABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3725] StringParser("NOORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3726] StringParser("NOPARALLEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3727] StringParser("NOREVERSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3728] StringParser("NORMALIZE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3729] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3730] StringParser("NORMALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3731] StringParser("NOROWDEPENDENCIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3732] StringParser("NOSORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3733] StringParser("NOSUPERUSER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3734] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3063, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3735] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3065, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3736] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3067, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3737] Ref(RawNotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3067, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3738] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3067, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3739] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3069, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3740] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3069, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3741] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3069, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3742] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3069, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3743] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3069, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3744] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3069, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3745] StringParser("NOTFOUND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3069, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3746] StringParser("NOTHING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3069, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3747] StringParser("NOTIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3069, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3748] StringParser("NOTNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3069, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3749] StringParser("NOWAIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3069, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3750] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3069, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3751] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3069, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3752] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3753] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3754] StringParser("NULLABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3755] StringParser("NULLIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3756] StringParser("NULLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3757] StringParser("NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3758] StringParser("NUMERIC")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3759] TypedParser("numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3760] StringParser("OBJECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3070, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3761] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3073, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3762] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3073, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3763] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3073, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3764] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3073, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3765] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3076, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3766] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3078, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3767] Ref(ObjectLiteralElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3078, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3768] StringParser("{")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3078, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3769] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3080, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3770] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3080, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3771] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3082, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3772] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3082, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3773] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3082, child_count: 12, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3774] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3094, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3775] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3094, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3776] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3094, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3777] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3094, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3778] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3094, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3779] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3094, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3780] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3094, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3781] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3094, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3782] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3094, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3783] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3094, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3784] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3094, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3785] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3094, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3786] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3097, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3787] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3099, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3788] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3099, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3789] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3101, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3790] Ref(UnderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3101, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3791] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3101, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3792] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3104, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3793] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3106, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3794] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3108, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3795] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3110, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3796] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3110, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3797] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3110, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3798] Ref(ElementSpecificationGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3110, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3799] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3111, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3800] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3113, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3801] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3113, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3802] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3803] Ref(FinalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3804] Ref(InstantiableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3805] Ref(PersistableKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3806] StringParser("OBJECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3807] StringParser("OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3808] StringParser("OCTETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3809] StringParser("OF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3810] StringParser("OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3811] StringParser("OFFLINE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3116, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3812] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3119, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3813] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3119, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3814] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3121, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3815] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 3121, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3816] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3817] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3818] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3819] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3820] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3821] StringParser("OFFSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3822] StringParser("OFFSETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3823] StringParser("OID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3824] StringParser("OIDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3825] StringParser("OLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3826] StringParser("OLTP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3124, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3827] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3127, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3828] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3127, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3829] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3127, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3830] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3129, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3831] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3131, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3832] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3833] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3834] Ref(PreserveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3835] Ref(DefinitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3836] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3135, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3837] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3137, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3838] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3137, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3839] Ref(PreserveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3137, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3840] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3137, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3841] StringParser("ON")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3137, child_count: 1, min_times: 1, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3842] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3138, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3843] Ref(StatementAndDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3138, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3844] StringParser("ONLINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3138, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3845] StringParser("ONLY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3138, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3846] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3143, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3847] Ref(OpenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3143, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3848] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3145, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3849] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3145, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3850] Ref(BindVariableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3145, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3851] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3145, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3852] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3148, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3853] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3148, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3854] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3148, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3855] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3148, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3856] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3150, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3857] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3150, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3858] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3152, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3859] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3154, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3860] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3157, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3861] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3157, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3862] Ref(OutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3157, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3863] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3159, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3864] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3159, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3865] Ref(OutKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3159, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3866] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3161, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3867] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3161, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3868] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3161, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3869] StringParser("OPEN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3161, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3870] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3871] Ref(OpenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3872] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3873] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3874] StringParser("OPENDATASOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3875] StringParser("OPENQUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3876] StringParser("OPENROWSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3877] StringParser("OPENXML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3878] StringParser("OPERATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3879] StringParser("OPERATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3880] StringParser("OPERATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3881] StringParser("OPTIMAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3882] StringParser("OPTIMIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3883] StringParser("OPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3884] StringParser("OPTIONALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3885] StringParser("OPTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3886] StringParser("OR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3887] StringParser("OR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3164, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3888] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3166, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3889] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3166, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3890] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3166, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3891] StringParser("ORA_ROWSCN")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3166, child_count: 11, min_times: 1, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3892] AnySetOf(11 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3177, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3893] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3179, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3894] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3179, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3895] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3179, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3896] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3181, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3897] Ref(PctfreeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3181, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3898] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3181, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3899] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3183, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3900] Ref(InitransKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3183, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3901] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3183, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3902] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3185, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3903] Ref(MaxtransKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3185, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3904] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3185, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3905] Ref(StorageClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3185, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3906] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3187, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3907] Ref(LoggingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3187, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3908] Ref(NologgingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3187, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3909] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3189, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3910] Ref(NoparallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3189, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3911] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3191, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3912] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3191, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3913] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3191, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3914] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3193, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3915] Ref(NocompressKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3193, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3916] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3195, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3917] Ref(CompressKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3195, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3918] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3195, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3919] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3197, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3920] Ref(NosortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3197, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3921] Ref(ReverseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3197, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3922] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3199, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3923] Ref(VisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3199, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3924] Ref(InvisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3199, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3925] Ref(OnlineKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3199, child_count: 17, min_times: 1, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3926] AnySetOf(17 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3216, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3927] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3928] Ref(SegmentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3929] Ref(CreationKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3219, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3930] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3221, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3931] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3221, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3932] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3221, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3933] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3223, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3934] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3223, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3935] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3223, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3936] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3225, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3937] Ref(PctfreeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3225, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3938] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3225, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3939] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3227, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3940] Ref(PctusedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3227, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3941] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3227, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3942] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3229, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3943] Ref(InitransKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3229, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3944] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3229, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3945] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3231, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3946] Ref(MaxtransKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3231, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3947] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3231, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3948] Ref(StorageClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3231, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3949] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3233, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3950] Ref(LoggingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3233, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3951] Ref(NologgingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3233, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3952] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3235, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3953] Ref(NoparallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3235, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3954] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3237, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3955] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3237, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3956] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3237, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3957] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3239, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3958] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3239, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3959] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3239, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3960] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3243, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3961] Ref(NocompressKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3243, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3962] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3245, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3963] Ref(CompressKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3245, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3964] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3247, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3965] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3247, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3966] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3250, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3967] Ref(OltpKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3250, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3968] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3252, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3969] Ref(QueryKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3252, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3970] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3971] Ref(LowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3972] Ref(HighKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3973] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3256, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3974] Ref(ArchiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3256, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3975] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3258, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3976] Ref(LowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3258, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3977] Ref(HighKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3258, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3978] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3262, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3979] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3262, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3980] Ref(StoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3262, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3981] Ref(CompressKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3262, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3982] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3264, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3983] Ref(BasicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3264, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3984] Ref(AdvancedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3264, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3985] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3269, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3986] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3269, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3987] Ref(StoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3269, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3988] Ref(CompressKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3269, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3989] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3271, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3990] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3271, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3991] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3273, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3992] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3275, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3993] Ref(QueryKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3275, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3994] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3277, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3995] Ref(LowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3277, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3996] Ref(HighKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3277, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3997] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3279, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3998] Ref(ArchiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3279, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3999] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3281, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4000] Ref(LowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3281, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4001] Ref(HighKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3281, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4002] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3285, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4003] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3285, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4004] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3285, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4005] Ref(LevelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3285, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4006] Ref(LockingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3285, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4007] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3287, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4008] Ref(MonitoringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3287, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4009] Ref(NomonitoringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3287, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4010] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3290, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4011] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3292, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4012] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3292, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4013] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3292, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4014] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3292, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4015] Ref(MovementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3292, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4016] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3294, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4017] Ref(Result_cacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3294, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4018] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3297, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4019] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4020] Ref(ModeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3299, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4021] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3301, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4022] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3301, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4023] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3301, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4024] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3303, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4025] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3305, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4026] Ref(InmemoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3305, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4027] AnySetOf(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3309, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4028] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3311, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4029] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3313, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4030] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3313, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4031] Ref(MemcompressKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3313, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4032] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3316, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4033] Ref(MemcompressKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3316, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4034] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3316, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4035] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4036] Ref(DmlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3320, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4037] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3322, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4038] Ref(QueryKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3322, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4039] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3324, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4040] Ref(LowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3324, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4041] Ref(HighKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3324, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4042] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3326, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4043] Ref(CapacityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3326, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4044] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3328, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4045] Ref(LowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3328, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4046] Ref(HighKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3328, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4047] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3330, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4048] Ref(ArchiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3330, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4049] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4050] Ref(LowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4051] Ref(HighKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3332, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4052] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4053] Ref(PriorityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3334, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4054] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3339, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4055] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3339, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4056] Ref(LowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3339, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4057] Ref(MediumKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3339, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4058] Ref(HighKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3339, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4059] Ref(CriticalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3339, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4060] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3342, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4061] Ref(DistributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3342, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4062] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3344, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4063] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3344, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4064] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3346, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4065] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3346, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4066] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3349, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4067] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3351, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4068] Ref(RowidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3351, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4069] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3351, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4070] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3351, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4071] Ref(SubpartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3351, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4072] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3354, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4073] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3354, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4074] Ref(ServiceKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3354, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4075] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4076] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4077] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4078] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4079] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3358, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4080] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3360, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4081] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3362, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4082] Ref(DuplicateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3362, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4083] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3362, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4084] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3364, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4085] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3364, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4086] Ref(DuplicateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3364, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4087] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3366, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4088] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3366, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4089] Ref(InmemoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3366, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4090] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3368, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4091] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3371, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4092] Ref(FlashbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3371, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4093] Ref(ArchiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3371, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4094] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3371, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4095] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3374, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4096] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3374, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4097] Ref(FlashbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3374, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4098] Ref(ArchiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3374, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4099] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3376, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4100] Ref(RowdependenciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3376, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4101] Ref(NorowdependenciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3376, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4102] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3382, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4103] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3382, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4104] Ref(SiblingsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3382, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4105] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3382, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 2, _padding: 0 }, // [4106] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3384, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4107] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3388, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4108] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3391, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4109] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3391, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4110] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3391, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4111] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3391, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4112] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3393, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4113] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3393, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4114] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3393, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4115] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3395, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4116] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3395, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4117] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3397, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4118] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3397, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4119] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3397, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4120] Ref(WithFillSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3397, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4121] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3397, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4122] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3397, child_count: 7, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4123] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3404, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4124] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3404, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4125] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3404, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4126] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3404, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4127] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3404, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4128] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3404, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4129] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3404, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4130] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3404, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4131] StringParser("ORDER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3404, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4132] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3406, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4133] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3406, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4134] Ref(NoorderKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3406, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4135] StringParser("ORDERING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3406, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4136] StringParser("ORDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3406, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4137] StringParser("OTHERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3406, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4138] StringParser("OUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3406, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4139] StringParser("OUTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3406, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4140] StringParser("OUTFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3406, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4141] StringParser("OUTLINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3406, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4142] StringParser("OUTPUT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3406, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4143] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3411, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4144] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3411, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4145] Ref(OverKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3411, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4146] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3413, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4147] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 3413, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4148] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4149] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4150] StringParser("OVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4151] StringParser("OVERFLOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3416, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4152] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4153] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3418, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4154] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3420, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4155] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3421, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4156] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4157] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4158] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4159] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4160] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4161] StringParser("OVERLAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4162] StringParser("OVERLAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4163] StringParser("OVERRIDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4164] StringParser("OVERWRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4165] StringParser("OWNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4166] StringParser("OWNERSHIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4167] StringParser("PACK_KEYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4168] StringParser("PACKAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4169] StringParser("PAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4170] StringParser("PAIRS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3426, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4171] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3428, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4172] Ref(Parallel_enableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3428, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4173] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3429, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4174] Bracketed(4 elements -> 4 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3435, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4175] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3435, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4176] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3435, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4177] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3435, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4178] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3438, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4179] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3438, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4180] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3441, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4181] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3443, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4182] Ref(HashKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3443, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4183] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3443, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4184] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3446, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4185] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3448, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4186] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3448, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4187] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3452, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4188] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3454, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4189] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3454, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4190] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3454, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4191] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3454, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4192] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3454, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4193] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3457, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4194] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3459, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4195] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3459, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4196] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3461, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4197] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3461, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4198] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4199] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4200] StringParser("PARALLEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4201] StringParser("PARALLEL_ENABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4202] StringParser("PARAMETER")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4203] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4204] StringParser("?")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4205] StringParser("PARAMETER_MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4206] StringParser("PARAMETER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4207] StringParser("PARAMETER_ORDINAL_POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4208] StringParser("PARAMETER_SPECIFIC_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4209] StringParser("PARAMETER_SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4210] StringParser("PARAMETER_SPECIFIC_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4211] StringParser("PARAMETERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4212] StringParser("PARENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4213] StringParser("PARTIAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3464, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4214] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3469, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4215] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3469, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4216] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3469, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4217] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3471, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4218] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3474, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4219] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4220] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4221] StringParser("PARTITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4222] StringParser("PASCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4223] StringParser("PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4224] StringParser("PATH")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3476, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4225] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3478, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4226] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3480, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4227] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3480, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4228] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4229] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4230] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4231] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4232] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4233] StringParser("PCTFREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4234] StringParser("PCTINCREASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4235] StringParser("PCTUSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4236] StringParser("PERCENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4237] StringParser("PERCENT_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4238] StringParser("PERCENTILE_CONT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4239] StringParser("PERCENTILE_DISC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4240] StringParser("PERSISTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4241] StringParser("PIPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4242] StringParser("|")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3482, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4243] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3484, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4244] Ref(PipelinedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3484, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4245] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3486, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4246] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3488, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4247] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3488, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4248] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3488, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4249] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3491, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4250] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3493, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4251] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3493, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4252] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3493, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4253] Ref(PolymorphicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3493, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4254] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3495, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4255] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3495, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4256] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3495, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4257] StringParser("PIPELINED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3495, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4258] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3497, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4259] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3497, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4260] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3499, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4261] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3502, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4262] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3504, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4263] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3504, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4264] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3506, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4265] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3506, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4266] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3509, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4267] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3511, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4268] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4269] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4270] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4271] StringParser("PIVOT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3513, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4272] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3516, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4273] Ref(PivotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3516, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4274] Ref(XmlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3516, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4275] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3521, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4276] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3523, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4277] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4278] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4279] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4280] Ref(PivotForGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4281] Ref(PivotInGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4282] StringParser("PLACING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4283] StringParser("PLAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4284] StringParser("PLI")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3525, child_count: 82, min_times: 0, first_terminator_idx: 36, terminator_count: 1, _padding: 0 }, // [4285] OneOf(82 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4286] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4287] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4288] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4289] Ref(TransactionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4290] Ref(DropTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4291] Ref(DropViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4292] Ref(CreateUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4293] Ref(DropUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4294] Ref(TruncateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4295] Ref(AccessStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4296] Ref(CreateTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4297] Ref(CreateRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4298] Ref(DropRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4299] Ref(AlterTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4300] Ref(CreateSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4301] Ref(SetSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4302] Ref(DropSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4303] Ref(DropTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4304] Ref(CreateDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4305] Ref(DropDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4306] Ref(CreateIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4307] Ref(DropIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4308] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4309] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4310] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4311] Ref(CreateCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4312] Ref(DropCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4313] Ref(CreateFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4314] Ref(DropFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4315] Ref(CreateModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4316] Ref(DropModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4317] Ref(DescribeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4318] Ref(UseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4319] Ref(ExplainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4320] Ref(CreateSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4321] Ref(AlterSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4322] Ref(DropSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4323] Ref(CreateTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4324] Ref(DropTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4325] Ref(CommentStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4326] Ref(CreateProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4327] Ref(DropProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4328] Ref(AlterFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4329] Ref(CreateTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4330] Ref(CreateTypeBodyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4331] Ref(CreatePackageBodyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4332] Ref(CreatePackageStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4333] Ref(AlterSessionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4334] Ref(DropPackageStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4335] Ref(AlterPackageStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4336] Ref(AlterTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4337] Ref(BeginEndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4338] Ref(AssignmentStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4339] Ref(RecordTypeDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4340] Ref(DeclareCursorVariableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4341] Ref(ExecuteImmediateSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4342] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4343] Ref(IfExpressionStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4344] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4345] Ref(NullStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4346] Ref(ForLoopStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4347] Ref(WhileLoopStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4348] Ref(LoopStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4349] Ref(ForAllStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4350] Ref(OpenStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4351] Ref(CloseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4352] Ref(OpenForStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4353] Ref(FetchStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4354] Ref(ExitStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4355] Ref(ContinueStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4356] Ref(RaiseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4357] Ref(ReturnStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4358] Ref(AlterIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4359] Ref(CreateDatabaseLinkStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4360] Ref(DropDatabaseLinkStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4361] Ref(AlterDatabaseLinkStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4362] Ref(CreateSynonymStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4363] Ref(DropSynonymStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4364] Ref(AlterSynonymStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4365] Ref(DropProfileStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4366] Ref(DropClusterStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4367] Ref(ProcedureCallStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [4368] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4369] StringParser("PLUGGABLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4370] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3609, child_count: 4, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4371] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3613, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4372] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3616, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4373] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3616, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4374] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3616, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4375] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3616, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4376] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3616, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4377] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3616, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4378] Ref(PlusJoinSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3616, child_count: 4, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4379] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3620, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4380] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3620, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4381] Ref(PlusJoinSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3620, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4382] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3620, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4383] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3623, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4384] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3623, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4385] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3623, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4386] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3623, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4387] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3626, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4388] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3626, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4389] StringParser("+")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3626, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4390] Ref(EndBracketSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3626, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4391] StringParser("+")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3626, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4392] StringParser("POLICY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3626, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4393] StringParser("POLYMORPHIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3626, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4394] StringParser("POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3626, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4395] StringParser("+")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3626, child_count: 4, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4396] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3630, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4397] Ref(WithinGroupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3630, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4398] Ref(KeepClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3630, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4399] Ref(FilterClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3630, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4400] Ref(OverClauseSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3630, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4401] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3630, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4402] StringParser("POSTFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3630, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4403] StringParser("POWER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3630, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4404] StringParser("**")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3630, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4405] StringParser("PRAGMA")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3630, child_count: 1, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4406] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4407] Ref(LateralKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4408] StringParser("PRECEDES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4409] StringParser("PRECEDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4410] StringParser("PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4411] StringParser("PREFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4412] StringParser("PREORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4413] StringParser("PREPARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4414] StringParser("PREPARED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4415] StringParser("PRESERVE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3631, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4416] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3633, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4417] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3633, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4418] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3633, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4419] StringParser("PRIMARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3633, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4420] StringParser("PRINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3633, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4421] StringParser("PRIOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3633, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4422] StringParser("PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3633, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4423] StringParser("PRIVATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3633, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4424] StringParser("PRIVILEGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3633, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4425] StringParser("PRIVILEGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3633, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4426] StringParser("PROC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3633, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4427] StringParser("PROCEDURAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3633, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4428] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 3635, child_count: 1, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4429] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3636, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4430] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3639, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4431] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3639, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4432] Ref(ExceptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3639, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4433] Ref(ElsifKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3639, child_count: 1, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4434] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3640, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4435] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4436] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 3642, child_count: 1, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4437] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3643, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4438] StringParser("PROCEDURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3643, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4439] StringParser("PROCEDURES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3643, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4440] StringParser("PROCESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3643, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4441] StringParser("PROCESSLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3643, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4442] StringParser("PROFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3643, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4443] StringParser("PROGRAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3643, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4444] StringParser("PROMPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3643, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4445] StringParser("PROPERTY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3643, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4446] StringParser("PUBLIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3643, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4447] StringParser("PURGE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3643, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4448] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3645, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4449] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3645, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4450] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3645, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4451] StringParser("QUALIFY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3645, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4452] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3648, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4453] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3648, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4454] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4455] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4456] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4457] Ref(SomeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 3651, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4458] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3654, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4459] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3654, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4460] StringParser("QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3654, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4461] StringParser("QUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3654, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4462] StringParser("QUOTA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3654, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4463] StringParser("QUOTE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3654, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4464] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3654, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4465] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3654, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4466] StringParser("RAID0")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3654, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4467] StringParser("RAISE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3654, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4468] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4469] Ref(RaiseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4470] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4471] StringParser("RAISERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4472] StringParser("RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4473] StringParser("RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4474] StringParser("=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4475] StringParser(">")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4476] StringParser("RAW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4477] StringParser("<")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4478] StringParser("!")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4479] StringParser("READ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4480] StringParser("READS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4481] StringParser("READTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4482] StringParser("REAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4483] StringParser("REBUILD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4484] StringParser("RECHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4485] StringParser("RECONFIGURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4486] StringParser("RECORD")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3656, child_count: 5, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4487] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3661, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4488] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3661, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4489] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3661, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4490] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3661, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4491] Ref(RecordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3661, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4492] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3664, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4493] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3666, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4494] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3669, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4495] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3669, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4496] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3671, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4497] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3671, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4498] Ref(ColumnTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3671, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4499] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3674, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4500] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3676, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4501] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3676, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4502] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3676, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4503] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3678, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4504] Ref(AssignmentOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3678, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4505] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3678, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4506] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3678, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4507] StringParser("RECURSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3678, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4508] StringParser("RECYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3678, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4509] StringParser("REDACTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3678, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4510] StringParser("REDEFINE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3678, child_count: 6, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4511] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3684, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4512] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3684, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4513] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3684, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4514] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3684, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4515] Ref(RefKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3684, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4516] Ref(CursorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3684, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4517] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3686, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4518] Ref(ReturnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3686, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4519] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3689, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4520] Ref(RowTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3689, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4521] Ref(ColumnTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3689, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4522] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3689, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4523] StringParser("REF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3689, child_count: 5, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4524] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3694, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4525] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3694, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4526] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3694, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4527] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3694, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4528] Ref(ReferenceMatchGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3694, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4529] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3696, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4530] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3699, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4531] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3699, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4532] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3699, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4533] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3699, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4534] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3702, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4535] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3702, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4536] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3702, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4537] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3702, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4538] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3704, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4539] Ref(MatchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3704, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4540] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3707, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4541] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3707, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4542] Ref(PartialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3707, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4543] Ref(SimpleKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3707, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4544] StringParser("REFERENCE_USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3707, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4545] StringParser("REFERENCES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3707, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4546] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3709, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4547] Ref(ReferencingKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3709, child_count: 1, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4548] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3710, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4549] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3713, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4550] Ref(TriggerCorrelationNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3713, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4551] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3713, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4552] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3713, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4553] StringParser("REFERENCING")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3713, child_count: 5, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4554] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3718, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4555] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3718, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4556] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3718, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4557] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3720, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4558] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3720, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4559] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3720, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4560] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3722, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4561] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3722, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4562] Ref(ActionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3722, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4563] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4564] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4565] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4566] StringParser("REFRESH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4567] StringParser("REGEXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4568] StringParser("REGR_AVGX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4569] StringParser("REGR_AVGY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4570] StringParser("REGR_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4571] StringParser("REGR_INTERCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4572] StringParser("REGR_R2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4573] StringParser("REGR_SLOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4574] StringParser("REGR_SXX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4575] StringParser("REGR_SXY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4576] StringParser("REGR_SYY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4577] StringParser("REINDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4578] StringParser("REJECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4579] StringParser("RELATIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4580] StringParser("RELEASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4581] StringParser("RELIES_ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4582] StringParser("RELOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4583] StringParser("REMOTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4584] StringParser("RENAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4585] StringParser("REPEAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4586] StringParser("REPEATABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4587] StringParser("REPLACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4588] StringParser("REPLICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4589] StringParser("REQUIRE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4590] StringParser("RESET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4591] StringParser("RESIGNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4592] StringParser("RESOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4593] StringParser("RESPECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4594] StringParser("RESTART")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4595] StringParser("RESTORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4596] StringParser("RESTRICT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3724, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4597] StringParser("RESTRICTED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3724, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4598] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3726, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4599] Ref(Result_cacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3726, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4600] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3728, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4601] Ref(Relies_onKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3728, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4602] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3731, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4603] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3733, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4604] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3733, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4605] StringParser("RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3733, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4606] StringParser("RESULT_CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3733, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4607] StringParser("RESUMABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3733, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4608] StringParser("RETURN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3733, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4609] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3735, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4610] Ref(ReturnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3735, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4611] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3735, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4612] StringParser("RETURNED_CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3735, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4613] StringParser("RETURNED_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3735, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4614] StringParser("RETURNED_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3735, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4615] StringParser("RETURNED_SQLSTATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3735, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4616] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3738, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4617] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4618] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4619] Ref(ReturnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3740, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4620] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3742, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4621] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3744, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4622] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3746, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4623] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3746, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4624] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3746, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4625] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3748, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4626] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3748, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4627] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3748, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4628] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3750, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4629] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3750, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4630] Ref(BulkCollectIntoClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3750, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4631] StringParser("RETURNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3750, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4632] StringParser("RETURNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3750, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4633] StringParser("REUSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3750, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4634] StringParser("REVERSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3750, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4635] StringParser("REVOKE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3750, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4636] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3752, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4637] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3752, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4638] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3754, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4639] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3756, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4640] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3758, child_count: 4, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4641] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3762, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4642] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3762, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4643] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3765, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4644] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3765, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4645] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3765, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4646] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3765, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4647] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3765, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4648] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3767, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4649] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3767, child_count: 4, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4650] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3771, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4651] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3771, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4652] Ref(IdentifiedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3771, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4653] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3771, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4654] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3773, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4655] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3773, child_count: 6, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4656] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3779, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4657] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3779, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4658] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3779, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4659] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3781, child_count: 6, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4660] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3787, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4661] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3787, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4662] Ref(DirectoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3787, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4663] Ref(EditionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3787, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4664] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3789, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4665] Ref(MiningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3789, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4666] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3789, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4667] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3791, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4668] Ref(JavaKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3791, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4669] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3793, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4670] Ref(SourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3793, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4671] Ref(ResourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3793, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4672] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3796, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4673] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3796, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4674] Ref(TranslationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3796, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4675] Ref(ProfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3796, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4676] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3798, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4677] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3800, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4678] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3802, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4679] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3802, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4680] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3802, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4681] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3802, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4682] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3802, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4683] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3802, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4684] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3804, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4685] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4686] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4687] Ref(ConstraintsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4688] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3806, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4689] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3809, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4690] Ref(ContainerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3809, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4691] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3809, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4692] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3811, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4693] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3811, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4694] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3811, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4695] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3814, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4696] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4697] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4698] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4699] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4700] StringParser("REWRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4701] StringParser("=>")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4702] StringParser("RIGHT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4703] StringParser("RLIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4704] StringParser("ROLE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4705] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4706] StringParser("ROLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4707] StringParser("ROLLBACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4708] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4709] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4710] StringParser("ROUTINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4711] StringParser("ROUTINE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4712] StringParser("ROUTINE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4713] StringParser("ROUTINE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4714] StringParser("ROUTINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4715] StringParser("ROW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3816, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4716] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3819, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4717] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3819, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4718] Ref(AttributeIndicatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3819, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4719] Ref(RowtypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3819, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4720] StringParser("ROW_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3819, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4721] StringParser("ROW_NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3819, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4722] StringParser("ROWCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3819, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4723] StringParser("ROWDEPENDENCIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3819, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4724] StringParser("ROWGUIDCOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3819, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4725] StringParser("ROWID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3819, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4726] StringParser("ROWNUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3819, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4727] StringParser("ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3819, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4728] StringParser("ROWTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3819, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4729] StringParser("RULE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3819, child_count: 4, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4730] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3823, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4731] Ref(TablesampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3823, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4732] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3825, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4733] Ref(BernoulliKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3825, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4734] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3825, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4735] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3828, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4736] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3828, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4737] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3830, child_count: 1, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4738] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3831, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4739] Ref(RepeatableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3831, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4740] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4741] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4742] StringParser("SAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4743] StringParser("SAVEPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4744] StringParser("SCALE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4745] StringParser("SCHEDULER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4746] StringParser("SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4747] StringParser("SCHEMA_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4748] StringParser("SCHEMAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4749] StringParser("SCOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4750] StringParser("SCOPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4751] StringParser("SCOPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4752] StringParser("SCOPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4753] StringParser("SCROLL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4754] StringParser("SEARCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4755] StringParser("SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4756] StringParser("SECOND_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4757] StringParser("SECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4758] StringParser("SECURITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4759] StringParser("SEGMENT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3834, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4760] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3836, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4761] Ref(WildcardExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3836, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4762] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3838, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4763] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3838, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4764] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3838, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4765] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3840, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4766] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3840, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4767] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 3840, child_count: 5, min_times: 0, first_terminator_idx: 37, terminator_count: 1, _padding: 0 }, // [4768] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3845, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4769] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3845, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4770] Ref(SelectClauseModifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 3845, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4771] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3847, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4772] Ref(SelectClauseElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3847, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4773] Ref(SelectClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3847, child_count: 9, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4774] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3856, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4775] Ref(BulkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3856, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4776] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 3856, child_count: 1, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4777] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::PrecededBy, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3857, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4778] PrecededBy(preceding_sequences=(('IS', 'DISTINCT'), ('IS', 'NOT', 'DISTINCT')))
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3857, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4779] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3857, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4780] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3859, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4781] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3859, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4782] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3859, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4783] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3859, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4784] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 3859, child_count: 1, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4785] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::PrecededBy, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3860, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4786] PrecededBy(preceding_sequences=(('MULTISET',),))
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3860, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4787] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3860, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4788] StringParser("SELECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 3860, child_count: 16, min_times: 0, first_terminator_idx: 38, terminator_count: 4, _padding: 0 }, // [4789] Sequence(16 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3876, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4790] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3876, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4791] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4792] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4793] Ref(BulkCollectIntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4794] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4795] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4796] Ref(HierarchicalQueryClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4797] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4798] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4799] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4800] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4801] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4802] Ref(ForUpdateGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4803] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4804] Ref(FetchClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4805] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4806] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4807] Ref(ForUpdateGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4808] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4809] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4810] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4811] Ref(LogKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3878, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4812] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3882, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4813] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3884, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4814] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3887, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4815] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3887, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4816] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3889, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4817] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3892, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4818] Ref(WithCompoundNonSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3892, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4819] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3892, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4820] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3895, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4821] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3895, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4822] StringParser("SELF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3895, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4823] StringParser(";")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3895, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4824] StringParser("SENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3895, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4825] StringParser("SEPARATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3895, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4826] StringParser("SEQUENCE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3895, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4827] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3897, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4828] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3899, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4829] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3899, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4830] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3899, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4831] Ref(NomaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3899, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4832] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3901, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4833] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4834] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4835] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4836] Ref(NominvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3903, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4837] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3905, child_count: 5, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4838] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3910, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4839] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3910, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4840] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3910, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4841] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3910, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4842] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3910, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4843] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3912, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4844] Ref(NextvalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3912, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4845] Ref(CurrvalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3912, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4846] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3915, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4847] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3915, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4848] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3915, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4849] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3917, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4850] Ref(NextvalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3917, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4851] Ref(CurrvalKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3917, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4852] StringParser("SEQUENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3917, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4853] StringParser("SERIALIZABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3917, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4854] StringParser("SERVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3917, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4855] StringParser("SERVER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3917, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4856] StringParser("SERVICE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3917, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4857] StringParser("SESSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3917, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4858] StringParser("SESSION_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3917, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4859] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4860] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3921, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4861] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3923, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4862] Ref(SetClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3923, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4863] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3926, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4864] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3926, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4865] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3926, child_count: 7, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4866] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3933, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4867] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3933, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4868] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3933, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4869] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3933, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4870] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3933, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4871] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3933, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4872] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3933, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4873] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3933, child_count: 5, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4874] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3938, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4875] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3938, child_count: 1, min_times: 1, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4876] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3939, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4877] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4878] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4879] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4880] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4881] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4882] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4883] StringParser("SET")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3941, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4884] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3944, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4885] Ref(UnionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3944, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4886] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3946, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4887] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4888] Ref(IntersectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4889] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4890] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4891] Ref(MinusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3948, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4892] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4893] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4894] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4895] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4896] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4897] StringParser("SETOF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4898] StringParser("SETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4899] StringParser("SETUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4900] StringParser("SHARD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4901] StringParser("SHARD_ENABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4902] StringParser("SHARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4903] StringParser("SHARED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4904] StringParser("SHARES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3952, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4905] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3954, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4906] Ref(SharingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3954, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4907] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3956, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4908] Ref(MetadataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3956, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4909] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3956, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4910] StringParser("SHARING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3956, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4911] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3958, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4912] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3960, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4913] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3960, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4914] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3960, child_count: 1, min_times: 1, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4915] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3961, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4916] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4917] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4918] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4919] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4920] StringParser("SHOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4921] StringParser("SHUTDOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4922] StringParser("SIBLINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4923] StringParser("SIGN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4924] StringParser("SIGNAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4925] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3966, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4926] Ref(PositiveSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3966, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4927] Ref(NegativeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3966, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4928] StringParser("SIMILAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3966, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4929] StringParser("SIMPLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3966, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 1, _padding: 0 }, // [4930] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3969, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4931] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3969, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4932] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3969, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4933] Ref(SubstitutionVariableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3969, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4934] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3969, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4935] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3971, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4936] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3971, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4937] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3974, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4938] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3974, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4939] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3974, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4940] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3974, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4941] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3974, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4942] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3976, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4943] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3976, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4944] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3976, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4945] StringParser("SIZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3976, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4946] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4947] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4948] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4949] StringParser("SKIP")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4950] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4951] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4952] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4953] StringParser("SMALLINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4954] StringParser("SOME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4955] StringParser("SONAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4956] StringParser("SOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4957] StringParser("SPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4958] StringParser("SPATIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4959] StringParser("SPECIFIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4960] StringParser("SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4961] StringParser("SPECIFICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4962] StringParser("SPECIFICTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4963] StringParser("SQL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4964] StringParser("SQL_BIG_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4965] StringParser("SQL_BIG_SELECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4966] StringParser("SQL_BIG_TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4967] StringParser("SQL_CALC_FOUND_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4968] StringParser("SQL_LOG_OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4969] StringParser("SQL_LOG_UPDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4970] StringParser("SQL_LOW_PRIORITY_UPDATES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4971] StringParser("SQL_MACRO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4972] StringParser("SQL_SELECT_LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4973] StringParser("SQL_SMALL_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4974] StringParser("SQL_WARNINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4975] StringParser("SQLCA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4976] StringParser("SQLCODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4977] StringParser("SQLERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4978] StringParser("SQLEXCEPTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3978, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4979] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3981, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4980] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3981, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4981] StringParser("SCAN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3981, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4982] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4983] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4984] Ref(OffKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4985] StringParser("SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4986] StringParser("SQLWARNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4987] StringParser("SQRT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4988] StringParser("SSL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4989] StringParser("STABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4990] StringParser("STAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4991] StringParser("STAGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4992] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4993] StringParser("START")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3983, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4994] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4995] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4996] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4997] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4998] StringParser("STARTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4999] StringParser("STARTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5000] StringParser("STATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3986, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5001] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3988, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5002] Ref(PlsqlStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3988, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5003] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3988, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5004] StringParser("STATEMENT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3988, child_count: 81, min_times: 0, first_terminator_idx: 43, terminator_count: 1, _padding: 0 }, // [5005] OneOf(81 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5006] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5007] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5008] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5009] Ref(TransactionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5010] Ref(DropTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5011] Ref(DropViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5012] Ref(CreateUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5013] Ref(DropUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5014] Ref(TruncateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5015] Ref(AccessStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5016] Ref(CreateTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5017] Ref(CreateRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5018] Ref(DropRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5019] Ref(AlterTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5020] Ref(CreateSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5021] Ref(SetSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5022] Ref(DropSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5023] Ref(DropTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5024] Ref(CreateDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5025] Ref(DropDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5026] Ref(CreateIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5027] Ref(DropIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5028] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5029] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5030] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5031] Ref(CreateCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5032] Ref(DropCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5033] Ref(CreateFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5034] Ref(DropFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5035] Ref(CreateModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5036] Ref(DropModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5037] Ref(DescribeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5038] Ref(UseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5039] Ref(ExplainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5040] Ref(CreateSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5041] Ref(AlterSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5042] Ref(DropSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5043] Ref(CreateTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5044] Ref(DropTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5045] Ref(CommentStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5046] Ref(CreateProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5047] Ref(DropProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5048] Ref(AlterFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5049] Ref(CreateTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5050] Ref(CreateTypeBodyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5051] Ref(CreatePackageBodyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5052] Ref(CreatePackageStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5053] Ref(AlterSessionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5054] Ref(DropPackageStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5055] Ref(AlterPackageStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5056] Ref(AlterTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5057] Ref(BeginEndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5058] Ref(AssignmentStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5059] Ref(RecordTypeDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5060] Ref(DeclareCursorVariableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5061] Ref(ExecuteImmediateSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5062] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5063] Ref(IfExpressionStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5064] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5065] Ref(NullStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5066] Ref(ForLoopStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5067] Ref(WhileLoopStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5068] Ref(LoopStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5069] Ref(ForAllStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5070] Ref(OpenStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5071] Ref(CloseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5072] Ref(OpenForStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5073] Ref(FetchStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5074] Ref(ExitStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5075] Ref(ContinueStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5076] Ref(RaiseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5077] Ref(ReturnStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5078] Ref(AlterIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5079] Ref(CreateDatabaseLinkStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5080] Ref(DropDatabaseLinkStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5081] Ref(AlterDatabaseLinkStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5082] Ref(CreateSynonymStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5083] Ref(DropSynonymStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5084] Ref(AlterSynonymStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5085] Ref(DropProfileStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5086] Ref(DropClusterStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5087] StringParser("STATIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5088] StringParser("STATISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5089] StringParser("STDDEV_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5090] StringParser("STDDEV_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5091] StringParser("STDIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5092] StringParser("STDOUT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5093] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4071, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5094] Ref(StorageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 4071, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5095] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4074, child_count: 12, min_times: 1, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5096] AnySetOf(12 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4086, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5097] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4088, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5098] Ref(InitialKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4088, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5099] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4090, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5100] Ref(SizeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4090, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5101] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4090, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5102] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5103] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5104] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4094, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5105] Ref(SizeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4094, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5106] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4094, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5107] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4096, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5108] Ref(MinextentsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4096, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5109] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4096, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5110] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4098, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5111] Ref(MaxextentsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4098, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5112] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4100, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5113] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4100, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5114] Ref(UnlimitedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4100, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5115] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4102, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5116] Ref(PctincreaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4102, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5117] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4102, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5118] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4104, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5119] Ref(FreelistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4104, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5120] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4104, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5121] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4107, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5122] Ref(FreelistKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4107, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5123] Ref(GroupsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4107, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5124] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4107, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5125] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4109, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5126] Ref(Buffer_poolKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4109, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5127] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4112, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5128] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4112, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5129] Ref(KeepKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4112, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5130] Ref(RecycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4112, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5131] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4114, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5132] Ref(Flash_cacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4114, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5133] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4117, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5134] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4117, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5135] Ref(KeepKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4117, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5136] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4117, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5137] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4119, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5138] Ref(Cell_flash_cacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4119, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5139] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4122, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5140] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4122, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5141] Ref(KeepKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4122, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5142] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4122, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5143] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4124, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5144] Ref(MaxsizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4124, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5145] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4127, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5146] Ref(UnlimitedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4127, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5147] Ref(SizeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4127, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5148] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4127, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5149] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4129, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5150] Ref(OptimalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4129, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5151] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4132, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5152] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4132, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5153] Ref(SizeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4132, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5154] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4132, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5155] StringParser("STORAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4132, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5156] StringParser("STORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4132, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5157] StringParser("STRAIGHT_JOIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4132, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5158] StringParser("STREAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4132, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5159] StringParser("STREAMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4132, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5160] StringParser("STRICT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4132, child_count: 1, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5161] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5162] Ref(ConcatSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5163] StringParser("STRING")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4133, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5164] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4136, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5165] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4138, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5166] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4140, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5167] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4140, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5168] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4140, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5169] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4140, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5170] StringParser("STRUCTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4140, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5171] StringParser("STYLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4140, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5172] StringParser("SUBCLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4140, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5173] StringParser("SUBLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4140, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5174] StringParser("SUBMULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4140, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5175] StringParser("SUBPARTITION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4140, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5176] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4142, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5177] TypedParser("substitution_variable")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4142, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5178] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5179] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5180] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5181] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5182] StringParser("SUBSTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5183] StringParser("SUCCESSFUL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5184] StringParser("SUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5185] StringParser("SUPERUSER")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5186] Token("symbol")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5187] StringParser("SYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5188] StringParser("SYNC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5189] StringParser("SYNONYM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5190] StringParser("SYSDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5191] StringParser("SYSGUID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5192] StringParser("SYSID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5193] StringParser("SYSTEM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5194] StringParser("SYSTEM_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4145, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5195] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4147, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5196] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4149, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5197] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4149, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5198] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4149, child_count: 4, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5199] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4153, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5200] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4156, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5201] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4156, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5202] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4159, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5203] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4159, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5204] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4161, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5205] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4161, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5206] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4161, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5207] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4164, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5208] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4164, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5209] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4164, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5210] Ref(UsingIndexClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4164, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5211] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4167, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5212] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4167, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5213] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4167, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5214] Ref(UsingIndexClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4167, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5215] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4170, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5216] Ref(ForeignKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4170, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5217] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4170, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5218] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4170, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5219] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4170, child_count: 7, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5220] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4177, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5221] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4177, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5222] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4177, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5223] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4177, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5224] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4177, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5225] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4180, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5226] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4180, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5227] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4183, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5228] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4183, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5229] Ref(SubstitutionVariableSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4183, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5230] StringParser("TABLE")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4183, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 12, _padding: 0 }, // [5231] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4185, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5232] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4185, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5233] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4188, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5234] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4188, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5235] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5236] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5237] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5238] Ref(AtSignSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5239] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5240] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5241] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5242] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5243] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5244] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5245] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5246] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5247] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5248] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5249] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5250] StringParser("TABLE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5251] StringParser("TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5252] StringParser("TABLESAMPLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5253] StringParser("TABLESPACE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5254] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4192, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 1, _padding: 0 }, // [5255] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4193, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5256] Ref(Expression_A_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4193, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5257] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4193, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5258] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4193, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5259] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4195, child_count: 1, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5260] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4196, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5261] Ref(Expression_B_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4196, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5262] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4196, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5263] StringParser("TASK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4196, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5264] StringParser("TASKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4196, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5265] StringParser("TEMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4196, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5266] StringParser("TEMPLATE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4196, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5267] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4196, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5268] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4198, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5269] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4200, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5270] Ref(GlobalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4200, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5271] Ref(PrivateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4200, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5272] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4200, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5273] StringParser("TEMPORARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4200, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5274] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4202, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5275] Ref(TransientKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4202, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5276] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4202, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5277] StringParser("TERMINATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4202, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5278] StringParser("TERMINATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4202, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5279] StringParser("TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4202, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5280] StringParser("TEXTSIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4202, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5281] StringParser("THAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4202, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5282] StringParser("THEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4202, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5283] StringParser("TIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4202, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5284] StringParser("~")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4202, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5285] StringParser("TIME")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4202, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5286] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4204, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5287] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4207, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5288] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4207, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5289] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4207, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5290] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4209, child_count: 4, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5291] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4213, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5292] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4213, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5293] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4213, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5294] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4213, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5295] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4213, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5296] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4216, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5297] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4218, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5298] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4218, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5299] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4218, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5300] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4218, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5301] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4218, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5302] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4221, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5303] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4221, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5304] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4221, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5305] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4224, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5306] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4226, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5307] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4226, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5308] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4226, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5309] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4226, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5310] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4226, child_count: 1, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5311] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4227, child_count: 4, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5312] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4231, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5313] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4231, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5314] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4231, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5315] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4231, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5316] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4231, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5317] StringParser("TIME_ZONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4231, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5318] StringParser("TIMEOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4231, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5319] StringParser("TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4231, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5320] StringParser("TIMEZONE_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4231, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5321] StringParser("TIMEZONE_MINUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4231, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5322] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4233, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5323] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4236, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5324] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4236, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5325] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4236, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5326] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4238, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5327] Ref(InsteadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4238, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5328] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4238, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5329] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4240, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5330] Ref(StatementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4240, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5331] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4242, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5332] Ref(EachKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4242, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5333] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4242, child_count: 6, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5334] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4248, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5335] Ref(TimingPointGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4248, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5336] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4248, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5337] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4248, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5338] Ref(OneOrMoreStatementsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4248, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5339] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4250, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5340] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4250, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5341] Ref(TimingPointGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4250, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5342] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4250, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5343] StringParser("TINYBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4250, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5344] StringParser("TINYINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4250, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5345] StringParser("TINYTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4250, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5346] StringParser("TO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4250, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5347] StringParser("TOAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4250, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5348] StringParser("TOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4250, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5349] StringParser("TOP_LEVEL_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4250, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5350] StringParser("TRAILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4250, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5351] StringParser("TRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4250, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5352] StringParser("TRANSACTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4250, child_count: 4, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5353] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4254, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5354] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4257, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5355] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4257, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5356] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4257, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5357] Ref(RollbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4257, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5358] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4259, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5359] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4259, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5360] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4259, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5361] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4261, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5362] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4261, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5363] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4261, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5364] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4264, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5365] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4264, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5366] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4264, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5367] Ref(ChainKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4264, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5368] StringParser("TRANSACTION_ACTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4264, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5369] StringParser("TRANSACTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4264, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5370] StringParser("TRANSACTIONS_COMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4264, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5371] StringParser("TRANSACTIONS_ROLLED_BACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4264, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5372] StringParser("TRANSFORM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4264, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5373] StringParser("TRANSFORMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4264, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5374] StringParser("TRANSIENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4264, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5375] StringParser("TRANSLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4264, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5376] StringParser("TRANSLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4264, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5377] StringParser("TREAT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4264, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5378] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4267, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5379] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4267, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5380] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4267, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5381] Ref(ParentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4267, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5382] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4270, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5383] Ref(BindColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4270, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5384] Ref(TriggerCorrelationNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4270, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5385] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4272, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5386] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4272, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5387] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4272, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5388] StringParser("TRIGGER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4272, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5389] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4275, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5390] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4275, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5391] Ref(TriggerPredicatesGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4275, child_count: 1, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5392] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4276, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5393] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4279, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5394] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4279, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5395] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4279, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5396] Ref(TriggerPredicatesGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4279, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5397] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4281, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5398] Ref(TriggerPredicatesConjunctionGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4281, child_count: 1, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5399] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4282, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5400] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4284, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5401] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4284, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5402] Ref(TriggerPredicatesConjunctionGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4284, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5403] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4287, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5404] Ref(InsertingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4287, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5405] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4289, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5406] Ref(UpdatingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4289, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5407] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4292, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5408] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4292, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5409] Ref(DeletingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4292, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5410] StringParser("TRIGGER_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4292, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5411] StringParser("TRIGGER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4292, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5412] StringParser("TRIGGER_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4292, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5413] StringParser("TRIM")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4292, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5414] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4295, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5415] Ref(BothKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4295, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5416] Ref(LeadingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4295, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5417] Ref(TrailingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4295, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5418] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4295, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5419] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4295, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5420] StringParser("TRUNCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4295, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5421] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4298, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5422] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4298, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5423] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4298, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5424] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4298, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5425] StringParser("TRUSTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4298, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5426] StringParser("TSEQUAL")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4298, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5427] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4301, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5428] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4303, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5429] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4303, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5430] StringParser("TYPE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4303, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5431] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4305, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5432] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4305, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5433] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4305, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5434] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4307, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5435] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4307, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5436] Ref(StructLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4307, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5437] StringParser("UESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4307, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5438] StringParser("UID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4307, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5439] StringParser("UNBOUNDED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4307, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5440] StringParser("UNCOMMITTED")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4307, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5441] Ref(CrossKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4307, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5442] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4310, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5443] Ref(NaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4310, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5444] Ref(UnconditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4310, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5445] Ref(HorizontalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4310, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5446] StringParser("UNDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4310, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5447] StringParser("UNDO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4310, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5448] StringParser("UNENCRYPTED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4310, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5449] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4312, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5450] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4312, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5451] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4314, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5452] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4314, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5453] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4314, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5454] StringParser("UNION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4314, child_count: 1, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5455] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4315, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5456] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4315, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5457] StringParser("UNIQUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4315, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5458] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4315, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5459] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4315, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5460] StringParser("UNLIMITED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4315, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5461] StringParser("UNLISTEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4315, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5462] StringParser("UNLOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4315, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5463] StringParser("UNNAMED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4315, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5464] StringParser("UNNEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 4315, child_count: 9, min_times: 0, first_terminator_idx: 57, terminator_count: 7, _padding: 0 }, // [5465] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4324, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5466] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4324, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5467] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5468] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5469] Ref(BulkCollectIntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5470] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5471] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5472] Ref(HierarchicalQueryClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5473] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5474] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5475] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5476] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5477] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5478] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5479] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5480] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5481] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5482] Ref(HierarchicalQueryClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5483] Ref(LogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4326, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5484] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4328, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5485] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4328, child_count: 1, min_times: 1, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5486] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4329, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5487] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4331, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5488] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4331, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5489] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4331, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5490] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4333, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5491] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4335, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5492] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4337, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5493] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4340, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5494] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4342, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5495] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4342, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5496] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4344, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5497] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4344, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5498] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4346, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5499] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4349, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5500] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4351, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5501] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4351, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5502] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4353, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5503] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4355, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5504] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4357, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5505] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4360, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5506] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4362, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5507] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4362, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5508] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4364, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5509] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4364, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5510] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4366, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5511] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4369, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5512] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4371, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5513] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4371, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5514] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4373, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5515] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4373, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5516] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4376, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5517] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4378, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5518] Ref(UnpivotAsCharacterLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4378, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5519] Ref(UnpivotAsNumericLiteralGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4378, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5520] StringParser("UNPIVOT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4378, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5521] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4380, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5522] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4382, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5523] Ref(IncludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4382, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5524] Ref(ExcludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4382, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5525] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4382, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5526] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4385, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5527] Ref(UnpivotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4385, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5528] Ref(UnpivotNullsGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4385, child_count: 5, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5529] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4390, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5530] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4392, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5531] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4395, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5532] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4397, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5533] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4397, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5534] Ref(PivotForGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4397, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5535] Ref(UnpivotInGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4397, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5536] StringParser("UNSIGNED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4397, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5537] StringParser("UNTIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4397, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5538] StringParser("UNUSABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4397, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5539] StringParser("UPDATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4397, child_count: 9, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5540] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4406, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5541] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4406, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5542] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 4406, child_count: 1, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5543] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4407, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5544] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4407, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5545] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4407, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5546] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4407, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5547] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4407, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5548] Ref(ReturningClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4407, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5549] StringParser("UPDATETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4407, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5550] StringParser("UPDATING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4407, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5551] StringParser("UPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4407, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5552] StringParser("USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4407, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5553] StringParser("USE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4407, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5554] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5555] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5556] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5557] StringParser("USE_ANY_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5558] StringParser("USER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5559] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5560] StringParser("USER_DEFINED_TYPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5561] StringParser("USER_DEFINED_TYPE_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5562] StringParser("USER_DEFINED_TYPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5563] StringParser("USER_DEFINED_TYPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5564] StringParser("USERS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4409, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5565] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4412, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5566] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4412, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5567] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4412, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5568] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4415, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5569] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4415, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5570] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4418, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5571] Ref(CreateIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4418, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5572] Ref(OracleIndexPhysicalAttributesSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4418, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5573] StringParser("USING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4418, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5574] StringParser("UTC_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4418, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5575] StringParser("UTC_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4418, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5576] StringParser("UTC_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4418, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5577] StringParser("VACUUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4418, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5578] StringParser("VALID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4418, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5579] StringParser("VALIDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4418, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5580] StringParser("VALIDATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4418, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5581] StringParser("VALUE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4418, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5582] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4420, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5583] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4422, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5584] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4422, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5585] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4422, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5586] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4424, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5587] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4427, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5588] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4429, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5589] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4432, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5590] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4432, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5591] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4432, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5592] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4432, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5593] StringParser("VALUES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4432, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5594] StringParser("VAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4432, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5595] StringParser("VAR_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4432, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5596] StringParser("VARBINARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4432, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5597] StringParser("VARCHAR2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4432, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5598] StringParser("VARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4432, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5599] StringParser("VARCHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4432, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5600] StringParser("VARIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4432, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5601] StringParser("VARIABLES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4432, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5602] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4435, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5603] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4437, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5604] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4439, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5605] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4441, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5606] Ref(VarrayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4441, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5607] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4443, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5608] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4443, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5609] Ref(ArrayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4443, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5610] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4446, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5611] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4446, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5612] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4446, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5613] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4446, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5614] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4448, child_count: 4, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5615] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4452, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5616] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4452, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5617] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4452, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5618] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4454, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5619] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4454, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5620] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4454, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5621] Ref(EndBracketSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4454, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5622] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4457, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5623] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4460, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5624] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4462, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5625] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4462, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5626] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5627] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5628] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5629] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5630] Ref(PersistableKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5631] StringParser("VARRAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5632] StringParser("VARYING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5633] StringParser("VERBOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5634] StringParser("VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5635] StringParser("VIEW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5636] StringParser("VIEWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5637] StringParser("VISIBILITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5638] StringParser("VISIBLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5639] StringParser("VOLATILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5640] StringParser("WAIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5641] StringParser("WAITFOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5642] StringParser("WAREHOUSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5643] StringParser("WAREHOUSES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5644] StringParser("WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5645] StringParser("WEEKDAY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4464, child_count: 8, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5646] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4472, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5647] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4472, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5648] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4475, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5649] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5650] Ref(TriggerPredicatesExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5651] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5652] Meta(conditional: indented_then=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5653] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5654] Meta(conditional: indented_then_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5655] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4479, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5656] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4479, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5657] Ref(OneOrMoreStatementsGrammar)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4479, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5658] Meta(conditional: indented_then_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4479, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5659] Meta(conditional: indented_then=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4479, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5660] StringParser("WHEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4479, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5661] StringParser("WHENEVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4479, child_count: 4, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5662] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4483, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5663] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4483, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5664] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4485, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5665] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4488, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5666] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4488, child_count: 8, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5667] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4496, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5668] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4496, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5669] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4498, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5670] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4498, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5671] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4498, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5672] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4500, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5673] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4500, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5674] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4500, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5675] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4500, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5676] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4500, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5677] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4500, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5678] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4500, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5679] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4500, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5680] StringParser("WHERE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4500, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5681] StringParser("WHILE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4500, child_count: 3, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5682] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4503, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5683] Ref(WhileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4503, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5684] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4503, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5685] Ref(LoopStatementSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4503, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5686] Token("whitespace")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4503, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5687] StringParser("WIDTH_BUCKET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4503, child_count: 1, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5688] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4504, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5689] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4504, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5690] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4506, child_count: 1, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5691] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4507, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5692] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4509, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5693] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4511, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5694] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4511, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5695] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4511, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5696] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4513, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5697] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4513, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5698] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4513, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5699] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4513, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5700] StringParser("WINDOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4513, child_count: 4, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5701] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 4517, child_count: 1, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5702] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4518, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5703] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4520, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5704] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4520, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5705] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4520, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5706] Ref(PartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4520, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5707] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4520, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5708] Ref(FrameClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4520, child_count: 6, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [5709] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4526, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5710] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4526, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5711] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4526, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5712] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4526, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 1, _padding: 0 }, // [5713] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4528, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5714] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4528, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [5715] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4528, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [5716] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4528, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [5717] Ref(NonWithNonSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4528, child_count: 6, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5718] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4534, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [5719] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4534, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [5720] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4534, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [5721] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4534, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 1, _padding: 0 }, // [5722] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4536, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [5723] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4536, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [5724] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4536, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5725] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4536, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5726] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4536, child_count: 3, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5727] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4539, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5728] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4539, child_count: 1, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5729] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4540, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5730] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4540, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5731] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4540, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5732] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4540, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5733] StringParser("WITH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4540, child_count: 4, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5734] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4544, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5735] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4544, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5736] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4544, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5737] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4544, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5738] Ref(BindingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4544, child_count: 3, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5739] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4547, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5740] Ref(WithinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4547, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5741] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4547, child_count: 3, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5742] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4550, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5743] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4550, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5744] StringParser("WITHIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4550, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5745] StringParser("WITHOUT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4550, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5746] Token("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4550, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5747] StringParser("WORK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4550, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5748] StringParser("WRAPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4550, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5749] StringParser("WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4550, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5750] StringParser("WRITETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4550, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5751] StringParser("X509")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4550, child_count: 3, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5752] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4553, child_count: 2, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5753] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4555, child_count: 2, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5754] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4557, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5755] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4557, child_count: 2, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5756] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4559, child_count: 3, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5757] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4562, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5758] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4562, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5759] Ref(EvalnameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4562, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5760] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4562, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5761] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4562, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5762] StringParser("XMLATTRIBUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4562, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5763] StringParser("XML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4562, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5764] StringParser("XOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4562, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5765] StringParser("YAML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4562, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5766] StringParser("YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4562, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5767] StringParser("YEAR_MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4562, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5768] StringParser("ZEROFILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4562, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [5769] StringParser("ZONE")
];

pub static CHILD_IDS: &[u32] = &[
    6, 9, 12, 15, 16, 17, 20, 21, 22, 29, 34, 37, 38, 39, 40, 41,
    42, 43, 44, 49, 50, 51, 52, 56, 59, 60, 63, 66, 69, 72, 75, 76,
    77, 80, 83, 84, 85, 86, 87, 88, 91, 94, 97, 100, 101, 104, 109, 110,
    111, 112, 126, 127, 128, 129, 130, 131, 132, 133, 134, 135, 7, 8, 10, 11,
    13, 14, 18, 19, 23, 24, 25, 26, 27, 28, 30, 31, 32, 33, 35, 36,
    45, 46, 47, 48, 53, 54, 55, 57, 58, 61, 62, 64, 65, 67, 68, 70,
    71, 73, 74, 78, 79, 81, 82, 89, 90, 92, 93, 95, 96, 98, 99, 102,
    103, 105, 106, 107, 108, 113, 114, 115, 116, 119, 120, 123, 117, 118, 121, 122,
    124, 125, 137, 138, 139, 140, 141, 142, 143, 144, 147, 150, 151, 152, 153, 154,
    155, 156, 159, 160, 161, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175,
    179, 180, 184, 185, 186, 187, 188, 189, 190, 191, 192, 193, 196, 197, 198, 199,
    200, 201, 202, 145, 146, 148, 149, 157, 158, 162, 163, 164, 176, 177, 178, 181,
    182, 183, 194, 195, 204, 221, 205, 211, 212, 206, 207, 210, 208, 209, 213, 214,
    217, 218, 215, 216, 219, 222, 223, 220, 221, 225, 226, 227, 228, 229, 230, 231,
    232, 233, 234, 235, 238, 241, 244, 236, 237, 239, 240, 242, 243, 245, 246, 248,
    249, 250, 251, 252, 253, 254, 255, 256, 257, 258, 261, 264, 267, 259, 260, 262,
    263, 265, 266, 268, 269, 271, 272, 274, 283, 289, 275, 279, 282, 276, 277, 278,
    280, 281, 284, 221, 285, 288, 286, 287, 290, 221, 291, 292, 294, 295, 296, 297,
    221, 298, 222, 223, 299, 305, 300, 301, 302, 303, 304, 308, 324, 325, 326, 332,
    327, 331, 328, 329, 330, 222, 223, 335, 336, 341, 342, 343, 344, 345, 346, 347,
    348, 349, 362, 350, 351, 352, 359, 353, 354, 355, 356, 357, 358, 360, 361, 364,
    365, 368, 369, 370, 366, 367, 371, 372, 373, 375, 376, 377, 378, 379, 384, 387,
    390, 394, 398, 399, 400, 401, 402, 403, 404, 405, 380, 381, 382, 383, 385, 386,
    388, 389, 391, 392, 393, 222, 223, 395, 396, 397, 408, 409, 410, 411, 412, 413,
    414, 415, 417, 421, 422, 423, 428, 431, 418, 419, 420, 424, 427, 425, 426, 429,
    430, 433, 434, 435, 436, 437, 439, 440, 441, 442, 448, 453, 460, 465, 475, 482,
    492, 501, 504, 510, 514, 443, 444, 445, 446, 447, 449, 450, 451, 452, 454, 457,
    458, 459, 455, 456, 461, 464, 462, 463, 466, 467, 468, 472, 469, 470, 471, 473,
    474, 476, 477, 478, 479, 480, 481, 483, 484, 485, 489, 486, 487, 488, 490, 491,
    493, 494, 495, 498, 496, 497, 499, 500, 502, 503, 505, 508, 509, 506, 507, 511,
    512, 513, 515, 516, 517, 525, 534, 541, 549, 557, 518, 519, 520, 521, 522, 523,
    524, 526, 529, 530, 527, 528, 531, 532, 533, 535, 536, 537, 538, 539, 540, 542,
    543, 544, 545, 546, 547, 548, 550, 551, 552, 553, 554, 555, 556, 558, 559, 560,
    561, 562, 564, 565, 566, 567, 568, 569, 570, 571, 572, 574, 583, 592, 575, 578,
    576, 577, 579, 580, 581, 222, 223, 582, 221, 584, 585, 586, 589, 587, 588, 590,
    222, 223, 591, 221, 593, 594, 595, 596, 597, 599, 606, 626, 600, 601, 602, 603,
    604, 222, 223, 605, 221, 607, 608, 619, 620, 625, 609, 612, 616, 610, 611, 613,
    614, 615, 222, 223, 617, 618, 621, 624, 622, 623, 627, 628, 629, 630, 631, 633,
    634, 635, 636, 638, 644, 657, 658, 639, 640, 641, 642, 643, 645, 648, 649, 650,
    646, 647, 651, 656, 652, 655, 653, 654, 659, 660, 663, 661, 662, 665, 666, 667,
    668, 670, 671, 672, 673, 674, 682, 683, 684, 675, 221, 676, 677, 678, 679, 680,
    681, 686, 687, 688, 689, 690, 691, 692, 693, 694, 698, 699, 695, 696, 697, 713,
    714, 715, 716, 717, 718, 719, 720, 721, 722, 723, 724, 726, 731, 732, 727, 730,
    728, 729, 736, 731, 732, 737, 221, 740, 748, 759, 762, 749, 750, 755, 756, 751,
    222, 223, 752, 753, 754, 757, 758, 760, 761, 775, 784, 785, 786, 787, 788, 789,
    790, 793, 794, 791, 792, 802, 810, 803, 809, 804, 807, 805, 806, 808, 814, 815,
    324, 816, 817, 332, 818, 819, 826, 827, 828, 829, 830, 834, 835, 841, 836, 837,
    838, 222, 223, 839, 840, 836, 837, 838, 849, 850, 853, 854, 860, 861, 864, 865,
    868, 222, 223, 869, 221, 871, 222, 223, 872, 221, 875, 222, 223, 881, 882, 883,
    884, 885, 332, 886, 221, 887, 888, 895, 222, 223, 897, 898, 899, 900, 901, 222,
    223, 910, 920, 932, 911, 884, 912, 917, 332, 919, 913, 921, 884, 922, 927, 332,
    929, 930, 931, 923, 933, 934, 884, 937, 942, 332, 944, 945, 946, 935, 936, 938,
    979, 980, 981, 982, 992, 993, 994, 995, 1002, 1003, 1004, 1005, 1008, 1012, 1015, 1018,
    1006, 1007, 1009, 1010, 1011, 1013, 1014, 1016, 1017, 1019, 1020, 1021, 1026, 1031, 1027, 1028,
    1029, 1030, 1033, 1036, 1034, 1035, 1037, 1040, 1044, 1047, 1050, 1051, 1052, 1055, 1056, 1059,
    1038, 1039, 1041, 1042, 1043, 222, 223, 1045, 1046, 1048, 1049, 1053, 1054, 1057, 1058, 1061,
    1062, 1063, 1069, 1064, 1065, 1066, 1067, 1068, 1070, 1071, 1077, 1083, 1072, 222, 223, 1073,
    1074, 1075, 1076, 1078, 1079, 1080, 1081, 1082, 1087, 1088, 1091, 1092, 1093, 1105, 1106, 1110,
    1111, 1112, 1113, 1130, 1114, 1117, 1120, 1123, 1126, 1115, 1116, 1118, 1119, 1121, 1122, 1124,
    1125, 1127, 1128, 1129, 1131, 1132, 1133, 1134, 1138, 1139, 1140, 1141, 1142, 1143, 1144, 1145,
    1148, 1149, 1150, 1154, 1159, 1151, 1152, 1153, 1155, 221, 1156, 1157, 1158, 1160, 1161, 1168,
    1169, 1170, 1171, 1172, 1176, 1177, 1182, 1183, 1184, 1186, 1187, 1188, 1189, 1191, 1192, 1210,
    1211, 1212, 1213, 1214, 1224, 1225, 1226, 1230, 1231, 1232, 1242, 1243, 1244, 1247, 1227, 1228,
    1229, 222, 223, 1233, 1234, 1235, 1236, 1237, 1241, 1238, 1239, 1240, 1245, 1246, 1248, 1249,
    1251, 1252, 1253, 1254, 1255, 1256, 1257, 1258, 1277, 1259, 1260, 1276, 1261, 1262, 1263, 1273,
    1264, 1265, 1266, 1267, 1268, 1269, 1270, 1271, 1272, 1274, 1275, 1278, 1279, 1281, 1282, 1283,
    1284, 1286, 1287, 1290, 1293, 1294, 1295, 1296, 1297, 1298, 1302, 1303, 1325, 1328, 1335, 1288,
    1289, 1291, 1292, 1299, 1300, 1301, 1304, 1305, 1306, 1307, 1308, 1309, 1310, 1311, 1315, 1316,
    1312, 1313, 1314, 1317, 1318, 1319, 1322, 222, 223, 1320, 1321, 1323, 1324, 1326, 1327, 1329,
    1334, 1330, 1331, 1332, 1333, 1337, 1338, 1341, 1342, 1343, 1344, 1345, 1348, 1339, 1340, 1346,
    222, 223, 1347, 221, 1351, 1352, 1353, 1354, 1355, 1356, 1368, 1369, 1357, 1358, 1359, 222,
    223, 1360, 221, 1361, 1362, 1363, 1364, 1365, 1366, 731, 732, 1367, 221, 1371, 1372, 1375,
    1378, 1379, 1380, 1381, 1382, 1383, 1387, 1390, 1391, 1395, 1396, 1373, 1374, 1376, 1377, 1384,
    1385, 1386, 1388, 1389, 1392, 324, 1393, 1394, 332, 1398, 1399, 1402, 1405, 1406, 1407, 1408,
    1409, 1413, 1416, 1417, 1418, 1400, 1401, 1403, 1404, 1410, 1411, 1412, 1414, 1415, 1420, 1421,
    1424, 1427, 1428, 1429, 1430, 1431, 1432, 1436, 1439, 1441, 1442, 1422, 1423, 1425, 1426, 1433,
    1434, 1435, 1437, 1438, 1440, 1444, 1445, 1446, 1447, 1449, 1450, 1451, 1452, 1454, 1458, 1462,
    1463, 1464, 1469, 1472, 1455, 1456, 1457, 1459, 1460, 1461, 1465, 1468, 1466, 1467, 1470, 1471,
    1474, 1475, 1476, 1477, 1478, 1480, 1481, 1482, 1485, 1486, 1487, 1488, 1489, 1490, 1491, 1492,
    1483, 1484, 1493, 1494, 1496, 1497, 1498, 1499, 1500, 1501, 1502, 1522, 1503, 1512, 1519, 1504,
    1509, 1510, 1511, 1505, 222, 223, 1506, 221, 1507, 1508, 1513, 1514, 1515, 1516, 1517, 1518,
    1518, 222, 223, 1520, 1521, 1524, 1525, 1528, 1531, 1532, 1533, 1534, 1535, 1536, 1546, 1547,
    1551, 1556, 1562, 1565, 1569, 1572, 1573, 1526, 1527, 1529, 1530, 1537, 1545, 1538, 1541, 1544,
    1539, 1540, 1542, 1543, 1548, 1549, 1550, 1552, 1555, 1553, 1554, 1557, 1560, 1558, 1559, 1561,
    221, 1563, 1564, 1566, 1567, 1568, 222, 223, 1570, 1571, 1575, 1576, 1579, 1582, 1583, 1584,
    1585, 1586, 1587, 324, 1590, 332, 1591, 1577, 1578, 1580, 1581, 1588, 1589, 1593, 1594, 1597,
    1600, 1601, 1602, 1603, 1604, 1609, 1610, 1611, 1614, 1617, 1595, 1596, 1598, 1599, 1605, 1606,
    1607, 1608, 1612, 1613, 1615, 1616, 1618, 1619, 1621, 1622, 1623, 1624, 1625, 1650, 1626, 1647,
    1627, 1628, 1629, 1638, 1630, 1631, 1632, 1633, 1634, 1635, 1636, 1637, 1639, 1642, 1640, 1641,
    1643, 1644, 1645, 1646, 1648, 1649, 1651, 1652, 1668, 1673, 1676, 1681, 1684, 1690, 1653, 1667,
    1654, 1657, 1661, 1655, 1656, 1658, 1659, 1660, 1662, 1663, 1666, 1664, 1665, 1669, 1670, 1671,
    1672, 1674, 1675, 1677, 1678, 1679, 1680, 1682, 1683, 1685, 1686, 1687, 1688, 1689, 1691, 1692,
    1693, 1694, 1696, 1697, 1698, 1701, 1707, 1708, 1709, 1710, 1711, 1712, 1713, 1717, 1699, 1700,
    1702, 1703, 1706, 1704, 1705, 1714, 1715, 1715, 222, 223, 1729, 1732, 1730, 1731, 1733, 222,
    223, 1749, 1756, 1750, 1751, 1752, 1753, 1754, 1755, 1757, 1758, 1762, 1763, 1765, 1766, 1770,
    1771, 1772, 1774, 1775, 1776, 1777, 1780, 1800, 1778, 1779, 1781, 1796, 1797, 1782, 1791, 1783,
    1786, 1784, 1785, 1787, 1788, 1789, 1790, 1792, 1795, 1793, 1794, 1798, 1799, 1805, 1806, 222,
    223, 1807, 221, 1808, 1809, 1811, 1816, 1817, 1812, 1813, 1814, 1815, 1818, 1819, 1820, 1821,
    1844, 1845, 1846, 1847, 1853, 1856, 1848, 1849, 1850, 1851, 1852, 1854, 324, 1855, 332, 1859,
    324, 1860, 332, 1861, 1862, 1890, 1863, 1887, 1888, 1889, 1864, 1878, 1881, 1886, 1865, 1872,
    1875, 1876, 1877, 1866, 1867, 1868, 1869, 1870, 1871, 1873, 1874, 1879, 1880, 1882, 1885, 1883,
    1884, 1894, 1895, 1896, 1899, 1900, 1913, 1914, 1915, 1916, 1929, 1930, 1931, 1954, 1955, 1959,
    1960, 1965, 1956, 1957, 1958, 1961, 1962, 1963, 1964, 1967, 1968, 1969, 1970, 1971, 1972, 1973,
    1974, 221, 1981, 1984, 1982, 1983, 1986, 1987, 1988, 1992, 1989, 1990, 1991, 222, 223, 1994,
    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2007, 2008, 2009, 2010, 2012,
    2013, 2014, 2015, 2016, 2018, 2019, 2020, 2021, 2023, 2024, 2025, 2026, 2027, 2030, 2031, 2032,
    2033, 2035, 2036, 2037, 2038, 2039, 2041, 2042, 2043, 2045, 2046, 2047, 2048, 2049, 2051, 2052,
    2053, 2054, 2056, 2057, 2058, 2059, 2060, 2062, 2063, 2064, 2066, 2067, 2068, 2069, 2070, 2071,
    2073, 2074, 2075, 2076, 2077, 2079, 2078, 221, 2081, 2082, 2083, 2084, 2086, 2087, 2088, 2089,
    2090, 2091, 2094, 2092, 2093, 2096, 2097, 2098, 2099, 2100, 2102, 2103, 2104, 2105, 2106, 2120,
    2127, 2121, 2122, 2123, 2124, 2125, 2126, 2128, 2129, 2132, 2130, 2131, 2133, 2134, 2136, 884,
    2137, 332, 2138, 2139, 222, 223, 2146, 2147, 2167, 324, 2168, 2169, 332, 2170, 2173, 2174,
    2182, 324, 2183, 332, 2175, 2176, 2177, 2178, 2179, 2180, 2181, 2189, 2194, 2190, 2193, 2191,
    2192, 2195, 2196, 2197, 2199, 2200, 324, 2201, 2202, 2205, 2216, 332, 2203, 2204, 2206, 2207,
    2208, 221, 2209, 2215, 2210, 2211, 2212, 2213, 2214, 2217, 2220, 2218, 2219, 2221, 2222, 2230,
    2231, 2232, 2233, 2234, 2239, 2240, 2241, 2242, 2243, 2244, 2247, 2248, 2249, 2250, 2251, 2254,
    2255, 2256, 2260, 2261, 2262, 2263, 2269, 2252, 2253, 2257, 2258, 2259, 2264, 2265, 2266, 2267,
    2268, 2270, 2271, 2273, 2276, 2277, 2278, 2274, 2275, 2280, 2281, 2282, 2283, 2284, 2288, 2285,
    2286, 2287, 2290, 2293, 2291, 2292, 2295, 2299, 2305, 2296, 2297, 2298, 222, 223, 2300, 2303,
    2301, 2302, 2304, 2308, 2358, 2309, 2310, 2311, 2312, 2313, 2323, 2324, 2325, 2326, 2327, 2328,
    2329, 2333, 2338, 2346, 2347, 2348, 2349, 2314, 222, 223, 2315, 2316, 2317, 2318, 221, 2319,
    2320, 2321, 2322, 2330, 2331, 2332, 2334, 2335, 2336, 222, 223, 2337, 221, 2339, 2340, 2341,
    2342, 2343, 2344, 2345, 2350, 2351, 2356, 2352, 222, 223, 2353, 2354, 2355, 2360, 2361, 2362,
    2363, 2364, 2365, 2366, 2375, 2376, 2379, 2383, 2386, 2377, 2378, 2380, 2381, 2382, 2384, 2385,
    2387, 2388, 2389, 2390, 2393, 2394, 2397, 2395, 2396, 2398, 2399, 2400, 2401, 2402, 2403, 2404,
    2405, 2409, 2410, 2411, 2414, 2415, 2416, 222, 223, 2417, 2418, 2433, 2434, 2435, 2436, 2442,
    2445, 2437, 2438, 2439, 2440, 2441, 2443, 2444, 2446, 2447, 2448, 2449, 2452, 2453, 2459, 2460,
    2483, 2454, 221, 2455, 2456, 2457, 2458, 2461, 221, 2462, 2463, 2477, 2480, 2464, 2465, 2468,
    2475, 2466, 2467, 2469, 2473, 2474, 2470, 2471, 2472, 2476, 222, 223, 2478, 2479, 2481, 2482,
    2485, 2486, 2487, 2490, 2488, 2489, 2491, 2492, 2495, 2493, 2494, 2496, 2497, 2501, 2502, 2509,
    2510, 2511, 2524, 2512, 2515, 2513, 2514, 2516, 2521, 2517, 2518, 2519, 2520, 2522, 2523, 2525,
    2511, 2526, 2511, 2528, 2529, 2537, 2538, 2539, 221, 2541, 2542, 2543, 2546, 2549, 2550, 2551,
    2552, 2553, 2554, 2555, 2556, 2544, 2545, 2547, 2548, 2558, 2576, 2559, 2560, 2563, 2564, 2570,
    2574, 2575, 2561, 2562, 2562, 222, 223, 2565, 2566, 2567, 2568, 2569, 2571, 2572, 2573, 2577,
    222, 223, 2558, 2578, 2579, 2581, 2582, 2582, 222, 223, 884, 2583, 332, 2594, 2595, 2606,
    2584, 2585, 2586, 2587, 222, 223, 2589, 2590, 2592, 2593, 2596, 2597, 2598, 2599, 2601, 2602,
    2604, 2605, 2611, 2612, 2613, 2615, 2616, 2620, 2626, 2632, 2638, 2639, 2642, 2652, 2653, 2654,
    2655, 2656, 2617, 2618, 2619, 2621, 2622, 2624, 2625, 2623, 2627, 2630, 2631, 2628, 2629, 2633,
    2634, 2635, 2636, 2637, 221, 2640, 2641, 2643, 2647, 2648, 2644, 2645, 2646, 2649, 2650, 2651,
    2658, 2659, 222, 223, 2661, 2662, 2663, 2664, 2665, 2669, 2674, 2670, 2671, 2672, 2675, 2679,
    2676, 2677, 2681, 2682, 2683, 2694, 2684, 2685, 2689, 2686, 2687, 2688, 2690, 2693, 2691, 2692,
    2695, 2696, 2697, 2698, 2699, 2700, 2701, 2703, 222, 223, 2704, 221, 2706, 2709, 2712, 2715,
    2716, 2707, 2708, 2710, 2711, 2713, 2714, 2717, 2724, 2718, 2723, 2719, 2720, 2721, 2722, 2739,
    2740, 2741, 2811, 2742, 2805, 2743, 2764, 2744, 2745, 2749, 2750, 2758, 2746, 2747, 2748, 2751,
    2752, 2753, 2754, 2755, 2756, 2757, 221, 2759, 2760, 2763, 2761, 2762, 2765, 2766, 2767, 2790,
    2791, 2792, 2796, 2800, 2768, 2784, 2769, 2770, 2771, 2772, 2775, 2780, 2773, 2774, 2776, 2777,
    2778, 2779, 2781, 2782, 2783, 2785, 221, 2786, 2789, 2787, 2788, 2793, 2794, 2795, 2797, 2798,
    2799, 2801, 2804, 2802, 2803, 2806, 2807, 2808, 2809, 2810, 2812, 2813, 2814, 2818, 2821, 2819,
    2820, 2822, 2823, 2827, 2828, 324, 2829, 332, 2830, 2831, 2832, 2833, 2834, 2835, 221, 2836,
    2837, 2838, 2841, 2844, 2845, 2846, 2847, 2848, 2842, 2843, 2851, 2852, 221, 2853, 2854, 2855,
    2856, 222, 223, 2860, 2861, 2862, 2863, 222, 223, 2864, 221, 2865, 2866, 2871, 884, 2872,
    332, 2873, 2874, 2874, 222, 223, 2876, 2879, 2880, 2881, 2882, 2877, 2878, 2887, 2890, 2888,
    2889, 2891, 2892, 2909, 2910, 2926, 2927, 2928, 2911, 2912, 2913, 2914, 2915, 2916, 2917, 2918,
    2919, 2920, 2921, 2922, 2923, 2924, 2925, 2929, 222, 223, 2932, 2947, 2948, 2949, 2950, 2951,
    2952, 2953, 2933, 2943, 2944, 2934, 2937, 2940, 2941, 2942, 2935, 2936, 2938, 2939, 2945, 2946,
    2957, 2958, 2961, 2959, 2960, 2963, 2964, 2966, 324, 2967, 332, 2968, 2976, 2979, 2980, 2969,
    2970, 2971, 2974, 324, 2975, 332, 2972, 2973, 2977, 324, 2978, 332, 2983, 2984, 2985, 2988,
    2991, 2989, 2990, 2997, 2998, 2999, 3000, 3001, 3002, 3003, 3004, 3005, 3010, 3011, 3012, 3013,
    3018, 3014, 222, 223, 3015, 3017, 3016, 221, 3023, 3024, 3025, 3026, 3029, 1766, 3049, 3050,
    3051, 3092, 3052, 3062, 3078, 3053, 3054, 3058, 3059, 3055, 3056, 3057, 222, 223, 3060, 222,
    223, 3061, 221, 3063, 3071, 3064, 3070, 3065, 3066, 3067, 3068, 3069, 221, 3072, 3077, 3073,
    3074, 3075, 3076, 3079, 3080, 3081, 3084, 3086, 3082, 3083, 3085, 222, 223, 3087, 3088, 3089,
    3090, 3091, 3093, 3094, 3071, 3095, 3052, 3063, 3078, 3113, 3114, 3115, 3121, 3116, 3117, 3119,
    3120, 3118, 222, 223, 3122, 3123, 3125, 3126, 3127, 3124, 222, 223, 3128, 222, 223, 3130,
    3131, 3132, 3137, 3138, 3133, 3134, 3135, 3136, 3139, 3140, 3141, 3142, 3145, 3146, 3147, 3148,
    3149, 3150, 3152, 3153, 3154, 221, 3155, 3156, 3157, 3162, 3163, 3164, 3165, 3167, 3173, 3174,
    3168, 3169, 3170, 3171, 3172, 3175, 3176, 3177, 3178, 222, 223, 3179, 221, 3180, 3181, 3183,
    3184, 3185, 3186, 3196, 3197, 3198, 3199, 3200, 3201, 3202, 3204, 3205, 3206, 3207, 3208, 3209,
    3210, 3212, 3217, 3213, 3214, 3215, 3216, 3219, 3223, 3224, 3225, 3226, 3220, 3221, 3222, 221,
    3227, 3228, 3229, 3231, 3234, 3235, 3232, 3233, 3237, 3238, 3243, 3248, 3239, 3240, 3241, 3242,
    3244, 3245, 3246, 3247, 3250, 3251, 3252, 3263, 3268, 3253, 3254, 3260, 3255, 222, 223, 3256,
    3257, 3258, 3259, 3261, 3262, 3264, 3267, 3265, 3266, 3272, 3285, 3290, 3273, 3274, 324, 3275,
    3276, 332, 3278, 3277, 3279, 3280, 3281, 3284, 3282, 3283, 3286, 3287, 324, 3288, 3289, 332,
    3291, 324, 3292, 332, 3295, 3297, 3300, 3298, 3299, 3302, 3303, 3304, 3307, 3305, 3306, 3306,
    222, 223, 3309, 3310, 3311, 3315, 3312, 3313, 3314, 3317, 324, 3318, 332, 3319, 222, 223,
    3320, 221, 3323, 3327, 3332, 3324, 3325, 3326, 3328, 3329, 3330, 3331, 3333, 3334, 3335, 3338,
    3341, 3336, 3337, 3339, 3340, 3342, 3343, 3349, 3350, 3344, 3345, 3346, 3347, 3348, 3351, 3352,
    3354, 3355, 3356, 222, 223, 3357, 221, 3359, 3360, 3361, 3362, 3369, 222, 223, 3363, 3364,
    3367, 3368, 3365, 3366, 3373, 3374, 3375, 222, 223, 3376, 3377, 3380, 3378, 3379, 3402, 3405,
    3403, 3404, 3406, 3407, 3412, 3415, 3416, 3413, 3414, 3417, 3418, 3420, 3421, 3422, 3426, 324,
    3427, 3433, 332, 3428, 3429, 3429, 222, 223, 3430, 3431, 3432, 3434, 3439, 3435, 3436, 3437,
    3438, 3440, 3441, 3448, 3449, 3450, 3451, 3452, 3453, 3454, 3455, 3458, 3456, 3457, 3461, 3462,
    3463, 3464, 3465, 3466, 3467, 3468, 3469, 3470, 3471, 3472, 3473, 3474, 3500, 3502, 324, 3503,
    332, 3504, 3505, 3506, 3501, 3512, 3513, 3514, 3515, 3516, 3519, 222, 223, 3517, 3518, 3520,
    3521, 3522, 222, 223, 3546, 324, 3547, 332, 3548, 3550, 3551, 3554, 3555, 3557, 3558, 3559,
    3562, 324, 3563, 332, 3560, 3561, 3564, 3565, 3567, 3568, 3569, 3570, 3573, 324, 3574, 332,
    3571, 3572, 3576, 324, 3577, 332, 3580, 324, 3581, 332, 3591, 3592, 3593, 3594, 3578, 3579,
    3582, 3583, 3584, 3588, 3585, 3587, 3586, 222, 223, 3589, 3590, 3596, 324, 3597, 332, 3598,
    3599, 3636, 3637, 3641, 3638, 3639, 3640, 3642, 3643, 3650, 3651, 3652, 3654, 3655, 3656, 3657,
    3658, 3659, 222, 223, 3661, 324, 3662, 332, 3663, 221, 3669, 3670, 3701, 3702, 3703, 3705,
    3707, 3709, 3704, 222, 223, 3706, 222, 223, 3708, 222, 223, 3712, 3713, 3714, 3715, 3717,
    3718, 3721, 3719, 3720, 3720, 222, 223, 3736, 3739, 3737, 3738, 3740, 3741, 3753, 3762, 3763,
    3764, 3766, 3768, 2153, 3767, 221, 3770, 3771, 3772, 3773, 3775, 3776, 3777, 3778, 3779, 3780,
    3781, 3782, 3783, 3784, 3785, 873, 3787, 3792, 3799, 3788, 3789, 3790, 3791, 3793, 222, 223,
    3794, 221, 3795, 3798, 3796, 3797, 3800, 3801, 3802, 3803, 3804, 3805, 3813, 3814, 3818, 3815,
    3816, 3817, 3819, 3820, 3828, 3829, 3830, 3831, 3836, 3832, 3835, 3833, 3834, 3837, 3840, 3838,
    3839, 3843, 3847, 3848, 3851, 3852, 3856, 3849, 3850, 3853, 3854, 3855, 3857, 3858, 3859, 221,
    3860, 3866, 3861, 3862, 3863, 3864, 3865, 3867, 3868, 3871, 3872, 3873, 3889, 3890, 3893, 3896,
    3899, 3902, 3905, 3906, 3909, 3914, 3919, 3922, 3925, 3894, 3895, 3897, 3898, 3900, 3901, 3903,
    3904, 3907, 3908, 3910, 3911, 3912, 3913, 3915, 3916, 3917, 3918, 3920, 3921, 3923, 3924, 3927,
    3933, 3936, 3939, 3942, 3945, 3948, 3949, 3952, 3957, 3960, 4007, 4010, 4016, 4024, 4090, 4099,
    3928, 3929, 3930, 3931, 3932, 3934, 3935, 3937, 3938, 3940, 3941, 3943, 3944, 3946, 3947, 3950,
    3951, 3953, 3954, 3955, 3956, 3958, 3959, 3961, 3962, 3978, 3985, 3963, 3964, 3965, 3966, 3967,
    3968, 3973, 3969, 3970, 3971, 3972, 3974, 3975, 3976, 3977, 3979, 3980, 3981, 3982, 3983, 3984,
    3986, 3987, 3988, 3989, 4002, 3990, 3991, 3992, 3997, 3993, 3994, 3995, 3996, 3998, 3999, 4000,
    4001, 4003, 4004, 4005, 4006, 4008, 4009, 4011, 4014, 4015, 4012, 4013, 4017, 4018, 4019, 222,
    223, 4020, 4021, 4022, 4023, 4025, 4087, 4026, 4027, 4028, 4052, 4060, 4080, 4029, 4032, 4030,
    4031, 4033, 4034, 4035, 4036, 4037, 4042, 4047, 4038, 4039, 4040, 4041, 4043, 4044, 4045, 4046,
    4048, 4049, 4050, 4051, 4053, 4054, 4055, 4056, 4057, 4058, 4059, 4061, 4062, 4072, 4063, 4064,
    4065, 4066, 4067, 4070, 4071, 4068, 4069, 4073, 4074, 4075, 4076, 4077, 4078, 4079, 4081, 4084,
    4082, 4083, 4085, 4086, 4088, 4089, 4091, 4095, 4092, 4093, 4094, 4096, 4097, 4098, 4100, 4101,
    4103, 4104, 4105, 324, 4106, 332, 4107, 221, 4108, 4112, 4115, 4120, 4109, 4110, 4111, 4113,
    4114, 4116, 4117, 4118, 4119, 4124, 4125, 4126, 4127, 4128, 4129, 4130, 4133, 4134, 324, 4144,
    4145, 4146, 332, 4147, 4148, 4149, 222, 223, 4153, 4154, 4155, 4160, 4156, 4157, 4158, 4159,
    222, 223, 4172, 4173, 4174, 4175, 4176, 4177, 4178, 222, 223, 4179, 4180, 4196, 4181, 4184,
    4187, 4182, 4183, 4185, 222, 223, 4186, 221, 4188, 4191, 4192, 4193, 4189, 4190, 4194, 222,
    223, 4195, 221, 4197, 4198, 4199, 222, 223, 4215, 4216, 324, 4217, 332, 4218, 4219, 4219,
    222, 223, 4220, 221, 4226, 4231, 4227, 4228, 4229, 4230, 4244, 4245, 4246, 4249, 4247, 4248,
    4250, 4253, 4254, 4251, 4252, 4255, 4256, 4259, 4260, 4261, 4262, 4262, 222, 223, 4263, 221,
    4265, 4266, 4267, 222, 223, 4268, 221, 4269, 4270, 4273, 4274, 4275, 4276, 4280, 4281, 222,
    223, 4277, 221, 4278, 4279, 4286, 4287, 4288, 4289, 4290, 4291, 4292, 4293, 4294, 4295, 4296,
    4297, 4298, 4299, 4300, 4301, 4302, 4303, 4304, 4305, 4306, 4307, 4308, 4309, 4310, 4311, 4312,
    4313, 4314, 4315, 4316, 4317, 4318, 4319, 4320, 4321, 4322, 4323, 4324, 4325, 4326, 4327, 4328,
    4329, 4330, 4331, 4332, 4333, 4334, 4335, 4336, 4337, 4338, 4339, 4340, 4341, 4342, 4343, 4344,
    4345, 4346, 4347, 4348, 4349, 4350, 4351, 4352, 4353, 4354, 4355, 4356, 4357, 4358, 4359, 4360,
    4361, 4362, 4363, 4364, 4365, 4366, 4367, 4371, 4379, 4372, 4376, 4377, 4378, 4373, 4374, 4375,
    4380, 4381, 4382, 4383, 4384, 4385, 4386, 4388, 4389, 4390, 4397, 4398, 4399, 4400, 4407, 4417,
    4418, 4429, 4434, 4430, 4431, 4432, 4433, 4435, 4436, 4437, 4430, 4449, 4450, 4453, 4454, 4458,
    4455, 4456, 4457, 4459, 222, 223, 4469, 4470, 4488, 4489, 4490, 4491, 4492, 4493, 222, 223,
    4494, 221, 4495, 4496, 4499, 4497, 4498, 4500, 4503, 4506, 4501, 4502, 4504, 4505, 4512, 4513,
    4514, 4515, 4516, 4517, 4518, 4519, 4520, 4521, 4522, 4525, 4526, 4527, 4528, 4529, 4530, 4534,
    4531, 4532, 4533, 4535, 4536, 4537, 4539, 4540, 4541, 4542, 4543, 4547, 4548, 4549, 4550, 4551,
    4552, 4555, 4556, 4557, 4560, 4563, 4558, 4559, 4561, 4562, 4564, 4565, 4599, 4600, 4601, 4602,
    4603, 222, 223, 4604, 221, 4610, 4611, 4617, 4620, 4628, 4618, 4619, 4621, 221, 4622, 4625,
    4623, 4624, 4626, 4627, 4629, 4630, 4637, 4638, 4639, 4695, 4640, 4689, 4641, 4656, 4642, 4643,
    4647, 4648, 4644, 4645, 4646, 4649, 4650, 4651, 4652, 4653, 4654, 4655, 221, 4657, 4658, 4659,
    4682, 4683, 4684, 4660, 4676, 4661, 4662, 4663, 4664, 4667, 4672, 4665, 4666, 4668, 4669, 4670,
    4671, 4673, 4674, 4675, 4677, 221, 4678, 4681, 4679, 4680, 4685, 4688, 4686, 4687, 4690, 4691,
    4692, 4693, 4694, 4696, 4698, 4699, 4697, 221, 4717, 4718, 4719, 4731, 4732, 4735, 4737, 4733,
    4734, 4736, 222, 223, 4738, 4740, 4739, 4741, 222, 223, 4761, 4762, 4763, 4764, 4766, 4767,
    4769, 4770, 324, 4771, 332, 4772, 221, 4775, 4776, 4777, 4779, 4780, 4783, 4784, 4785, 4787,
    4778, 4781, 4782, 4786, 4790, 4791, 4794, 4795, 4796, 4797, 4798, 4799, 4800, 4801, 4802, 4803,
    4804, 4805, 4806, 4807, 4792, 4793, 4813, 4816, 4819, 4820, 4814, 4815, 4815, 222, 223, 4817,
    4818, 4818, 222, 223, 4821, 222, 223, 4828, 4831, 4829, 4830, 4833, 4836, 4834, 4835, 4838,
    4846, 4839, 4840, 4841, 4842, 4843, 4844, 4845, 4847, 4848, 4849, 4850, 4851, 4860, 324, 4861,
    332, 4862, 221, 4864, 4865, 4866, 4867, 4868, 4869, 4870, 4871, 4872, 4873, 4875, 4876, 4880,
    4881, 4882, 4877, 4878, 4879, 4885, 4886, 4891, 4887, 4890, 4888, 4889, 4893, 4894, 4895, 4896,
    4906, 4907, 4908, 4909, 4912, 4915, 4913, 4914, 4916, 4917, 4918, 4919, 4926, 4927, 4931, 4932,
    4933, 4936, 221, 4938, 4939, 4940, 4943, 4944, 4947, 4948, 4980, 4981, 4982, 4983, 4984, 4995,
    4996, 4997, 5002, 5003, 5006, 5007, 5008, 5009, 5010, 5011, 5012, 5013, 5014, 5015, 5016, 5017,
    5018, 5019, 5020, 5021, 5022, 5023, 5024, 5025, 5026, 5027, 5028, 5029, 5030, 5031, 5032, 5033,
    5034, 5035, 5036, 5037, 5038, 5039, 5040, 5041, 5042, 5043, 5044, 5045, 5046, 5047, 5048, 5049,
    5050, 5051, 5052, 5053, 5054, 5055, 5056, 5057, 5058, 5059, 5060, 5061, 5062, 5063, 5064, 5065,
    5066, 5067, 5068, 5069, 5070, 5071, 5072, 5073, 5074, 5075, 5076, 5077, 5078, 5079, 5080, 5081,
    5082, 5083, 5084, 5085, 5086, 5094, 5095, 5096, 222, 223, 5097, 5102, 5107, 5110, 5115, 5118,
    5121, 5125, 5131, 5137, 5143, 5149, 5098, 5099, 5100, 5101, 5103, 5104, 5105, 5106, 5108, 5109,
    5111, 5112, 5113, 5114, 5116, 5117, 5119, 5120, 5122, 5123, 5124, 5126, 5127, 5128, 5129, 5130,
    5132, 5133, 5134, 5135, 5136, 5138, 5139, 5140, 5141, 5142, 5144, 5145, 5146, 5147, 5148, 5150,
    5151, 5152, 5153, 5154, 5162, 5165, 222, 223, 5166, 221, 5167, 5168, 5177, 5178, 5179, 5180,
    5181, 5196, 5199, 5197, 5198, 5200, 5207, 5211, 5215, 5201, 5202, 5204, 5203, 222, 223, 5205,
    5206, 5208, 5209, 5210, 5212, 5213, 5214, 5216, 5217, 5218, 5221, 5222, 5223, 5224, 5225, 5227,
    5229, 5226, 222, 223, 5228, 222, 223, 5232, 5233, 5234, 5235, 5238, 5236, 5237, 5255, 5258,
    5256, 5260, 5262, 5261, 5269, 5272, 5270, 5271, 5275, 5276, 5287, 5302, 5288, 5289, 5290, 5291,
    5296, 5292, 5293, 5294, 5295, 5297, 5300, 5301, 5298, 5299, 5303, 5304, 5305, 5306, 5309, 5310,
    5307, 5308, 5312, 5313, 5314, 5315, 5316, 5323, 5329, 5324, 5325, 5326, 5327, 5328, 5330, 5331,
    5332, 5333, 5335, 5336, 5337, 5338, 5339, 5342, 5340, 5341, 5354, 5358, 5361, 5364, 5355, 5356,
    5357, 5359, 5360, 5362, 5363, 5365, 5366, 5367, 5379, 5380, 5381, 5383, 5384, 5385, 5386, 5387,
    5390, 5391, 5392, 5393, 5394, 5395, 5396, 5398, 5399, 5400, 5401, 5402, 5404, 5405, 5409, 5406,
    5407, 5408, 222, 223, 5415, 5416, 5417, 5422, 5423, 5424, 5428, 222, 223, 5429, 221, 5432,
    5433, 5435, 5436, 5443, 5444, 5445, 5450, 5451, 5452, 5453, 5456, 5466, 5467, 5470, 5471, 5472,
    5473, 5474, 5475, 5476, 5468, 5469, 5485, 5486, 5487, 5488, 5489, 5491, 221, 5492, 5496, 5493,
    5494, 5494, 222, 223, 5495, 221, 5497, 5498, 5499, 5500, 5500, 222, 223, 5501, 221, 5503,
    221, 5504, 5508, 5505, 5506, 5506, 222, 223, 5507, 221, 5509, 5510, 5511, 5512, 5512, 222,
    223, 5513, 221, 5515, 5516, 5517, 222, 223, 5518, 5519, 5522, 5525, 5523, 5524, 5527, 5528,
    5529, 5530, 5534, 5535, 222, 223, 5531, 5532, 5532, 222, 223, 5533, 221, 5541, 324, 5542,
    5543, 332, 5545, 5546, 5547, 5548, 5544, 5555, 5556, 5566, 5567, 5568, 5569, 5570, 5572, 5571,
    222, 223, 5583, 5586, 5584, 5585, 5587, 5588, 5588, 222, 223, 5589, 221, 5590, 5591, 5592,
    5603, 5613, 5614, 5604, 5612, 5605, 5610, 5606, 5607, 5608, 5609, 5611, 222, 223, 5615, 5622,
    5616, 5617, 5618, 5621, 5619, 5620, 5623, 5629, 5630, 5624, 222, 223, 5625, 5626, 5627, 5628,
    5647, 5648, 5652, 5653, 5654, 5655, 5658, 5659, 884, 5649, 332, 5650, 5651, 5656, 5657, 5663,
    884, 5664, 332, 5665, 5666, 5666, 222, 223, 5668, 5669, 5672, 5675, 5676, 5677, 5678, 5679,
    5670, 5671, 5673, 5674, 5683, 5684, 5685, 5689, 5691, 5699, 5692, 5693, 5696, 5694, 5695, 5697,
    5698, 5702, 5706, 5707, 5708, 5703, 5704, 5705, 5710, 5711, 5712, 5713, 5716, 5717, 5714, 221,
    5719, 5720, 5721, 5722, 5725, 5726, 5723, 221, 5728, 5729, 5731, 5730, 5735, 5736, 5737, 5738,
    5740, 5741, 5742, 5743, 222, 223, 5753, 222, 223, 5754, 221, 5755, 5756, 5757, 5761, 5758,
    5759, 5760,
];

pub static TERMINATORS: &[u32] = &[
    795, 796, 914, 915, 916, 918, 924, 925, 926, 928, 939, 940, 941, 943, 947, 948,
    949, 996, 1089, 1716, 1767, 1891, 1892, 2306, 2357, 2588, 2591, 2600, 2603, 2673, 2678, 2839,
    2857, 1767, 4121, 4122, 4368, 4773, 4808, 4809, 4810, 4811, 4934, 4368, 5239, 5240, 5241, 5242,
    5243, 5244, 5245, 5246, 5247, 5248, 5249, 873, 5257, 5477, 5478, 5479, 5480, 5481, 5482, 5483,
    5715, 5724,
];

pub static STRINGS: &[&str] = &[
    "ABORT", // [0]
    "keyword", // [1]
    "KeywordSegment", // [2]
    "base", // [3]
    "raw", // [4]
    "word", // [5]
    "ABS", // [6]
    "ABSENT", // [7]
    "ABSOLUTE", // [8]
    "ACCESS", // [9]
    "ANALYTIC", // [10]
    "ATTRIBUTE", // [11]
    "CLASS", // [12]
    "CLUSTER", // [13]
    "CONNECT", // [14]
    "CONTAINER", // [15]
    "CONTEXT", // [16]
    "CUBE", // [17]
    "DATABASE", // [18]
    "DATE", // [19]
    "DBA_RECYCLEBIN", // [20]
    "DICTIONARY", // [21]
    "DIMENSION", // [22]
    "DIRECTIVE", // [23]
    "DIRECTORY", // [24]
    "DOMAIN", // [25]
    "EDITION", // [26]
    "EXTERNAL", // [27]
    "FINE", // [28]
    "HIERARCHY", // [29]
    "INDEX", // [30]
    "INDEXTYPE", // [31]
    "JOB", // [32]
    "KEY", // [33]
    "LIBRARY", // [34]
    "LOCKDOWN", // [35]
    "MATERIALIZED", // [36]
    "MEASURE", // [37]
    "MINING", // [38]
    "OBJECT", // [39]
    "OPERATOR", // [40]
    "OUTLINE", // [41]
    "PLUGGABLE", // [42]
    "PRIVILEGE", // [43]
    "PRIVILEGES", // [44]
    "PROCEDURE", // [45]
    "PROFILE", // [46]
    "PROGRAM", // [47]
    "PROPERTY", // [48]
    "REDACTION", // [49]
    "REMOTE", // [50]
    "RESOURCE", // [51]
    "ROLE", // [52]
    "ROLLBACK", // [53]
    "ROW", // [54]
    "SCHEDULER", // [55]
    "SEQUENCE", // [56]
    "SESSION", // [57]
    "SQL", // [58]
    "SYNONYM", // [59]
    "SYSGUID", // [60]
    "SYSTEM", // [61]
    "TABLE", // [62]
    "TABLESPACE", // [63]
    "TRANSACTION", // [64]
    "TRIGGER", // [65]
    "TYPE", // [66]
    "USER", // [67]
    "VIEW", // [68]
    "AccessKeywordSegment", // [69]
    "PolicyKeywordSegment", // [70]
    "POLICY", // [71]
    "AnalyticKeywordSegment", // [72]
    "ViewKeywordSegment", // [73]
    "AttributeKeywordSegment", // [74]
    "DimensionKeywordSegment", // [75]
    "ClassKeywordSegment", // [76]
    "ClusterKeywordSegment", // [77]
    "ConnectKeywordSegment", // [78]
    "SessionKeywordSegment", // [79]
    "ContainerKeywordSegment", // [80]
    "ContextKeywordSegment", // [81]
    "CubeKeywordSegment", // [82]
    "BuildKeywordSegment", // [83]
    "ProcessKeywordSegment", // [84]
    "PROCESS", // [85]
    "DatabaseKeywordSegment", // [86]
    "LINK", // [87]
    "LinkKeywordSegment", // [88]
    "TriggerKeywordSegment", // [89]
    "DateKeywordSegment", // [90]
    "TimeKeywordSegment", // [91]
    "TIME", // [92]
    "Dba_recyclebinKeywordSegment", // [93]
    "DictionaryKeywordSegment", // [94]
    "DirectiveKeywordSegment", // [95]
    "DirectoryKeywordSegment", // [96]
    "DomainKeywordSegment", // [97]
    "EditionKeywordSegment", // [98]
    "FineKeywordSegment", // [99]
    "GrainedKeywordSegment", // [100]
    "AuditKeywordSegment", // [101]
    "AUDIT", // [102]
    "HierarchyKeywordSegment", // [103]
    "IndexKeywordSegment", // [104]
    "IndextypeKeywordSegment", // [105]
    "ExternalKeywordSegment", // [106]
    "JobKeywordSegment", // [107]
    "ResourceKeywordSegment", // [108]
    "KeyKeywordSegment", // [109]
    "ManagementKeywordSegment", // [110]
    "LibraryKeywordSegment", // [111]
    "LockdownKeywordSegment", // [112]
    "ProfileKeywordSegment", // [113]
    "MaterializedKeywordSegment", // [114]
    "MeasureKeywordSegment", // [115]
    "FolderKeywordSegment", // [116]
    "MiningKeywordSegment", // [117]
    "ModelKeywordSegment", // [118]
    "MODEL", // [119]
    "ObjectKeywordSegment", // [120]
    "PrivilegeKeywordSegment", // [121]
    "OperatorKeywordSegment", // [122]
    "OutlineKeywordSegment", // [123]
    "PluggableKeywordSegment", // [124]
    "PrivilegesKeywordSegment", // [125]
    "ProcedureKeywordSegment", // [126]
    "ProgramKeywordSegment", // [127]
    "PropertyKeywordSegment", // [128]
    "GraphKeywordSegment", // [129]
    "RedactionKeywordSegment", // [130]
    "RemoteKeywordSegment", // [131]
    "CostKeywordSegment", // [132]
    "RoleKeywordSegment", // [133]
    "RollbackKeywordSegment", // [134]
    "SegmentKeywordSegment", // [135]
    "SEGMENT", // [136]
    "RowKeywordSegment", // [137]
    "LevelKeywordSegment", // [138]
    "LEVEL", // [139]
    "SecurityKeywordSegment", // [140]
    "SECURITY", // [141]
    "SchedulerKeywordSegment", // [142]
    "SequenceKeywordSegment", // [143]
    "SqlKeywordSegment", // [144]
    "FirewallKeywordSegment", // [145]
    "TRANSLATION", // [146]
    "TranslationKeywordSegment", // [147]
    "TuningKeywordSegment", // [148]
    "SetKeywordSegment", // [149]
    "SET", // [150]
    "SynonymKeywordSegment", // [151]
    "SysguidKeywordSegment", // [152]
    "SystemKeywordSegment", // [153]
    "TableKeywordSegment", // [154]
    "TablespaceKeywordSegment", // [155]
    "TransactionKeywordSegment", // [156]
    "TypeKeywordSegment", // [157]
    "UserKeywordSegment", // [158]
    "ADMINISTER", // [159]
    "ADVISOR", // [160]
    "ALL", // [161]
    "ALTER", // [162]
    "ANALYZE", // [163]
    "BACKUP", // [164]
    "BECOME", // [165]
    "CHANGE", // [166]
    "COMMENT", // [167]
    "CREATE", // [168]
    "DEBUG", // [169]
    "DELETE", // [170]
    "DROP", // [171]
    "ENABLE", // [172]
    "EXECUTE", // [173]
    "EXEMPT", // [174]
    "FLASHBACK", // [175]
    "FORCE", // [176]
    "GLOBAL", // [177]
    "GRANT", // [178]
    "INHERIT", // [179]
    "INSERT", // [180]
    "KEEP", // [181]
    "LOCK", // [182]
    "LOGMINING", // [183]
    "MANAGE", // [184]
    "MERGE", // [185]
    "ON", // [186]
    "PURGE", // [187]
    "QUERY", // [188]
    "READ", // [189]
    "REDEFINE", // [190]
    "REFERENCES", // [191]
    "RESTRICTED", // [192]
    "RESUMABLE", // [193]
    "SELECT", // [194]
    "SIGN", // [195]
    "TRANSLATE", // [196]
    "UNDER", // [197]
    "UNLIMITED", // [198]
    "UPDATE", // [199]
    "USE", // [200]
    "WRITE", // [201]
    "AdministerKeywordSegment", // [202]
    "AdvisorKeywordSegment", // [203]
    "AllKeywordSegment", // [204]
    "AlterKeywordSegment", // [205]
    "AnalyzeKeywordSegment", // [206]
    "BackupKeywordSegment", // [207]
    "BecomeKeywordSegment", // [208]
    "ChangeKeywordSegment", // [209]
    "NotificationKeywordSegment", // [210]
    "CommentKeywordSegment", // [211]
    "CreateKeywordSegment", // [212]
    "DebugKeywordSegment", // [213]
    "DeleteKeywordSegment", // [214]
    "DropKeywordSegment", // [215]
    "EnableKeywordSegment", // [216]
    "DiagnosticsKeywordSegment", // [217]
    "DIAGNOSTICS", // [218]
    "ExecuteKeywordSegment", // [219]
    "ExemptKeywordSegment", // [220]
    "FlashbackKeywordSegment", // [221]
    "ArchiveKeywordSegment", // [222]
    "ARCHIVE", // [223]
    "ForceKeywordSegment", // [224]
    "GrantKeywordSegment", // [225]
    "InheritKeywordSegment", // [226]
    "InsertKeywordSegment", // [227]
    "KeepKeywordSegment", // [228]
    "LockKeywordSegment", // [229]
    "LogminingKeywordSegment", // [230]
    "ManageKeywordSegment", // [231]
    "MergeKeywordSegment", // [232]
    "OnKeywordSegment", // [233]
    "CommitKeywordSegment", // [234]
    "COMMIT", // [235]
    "RefreshKeywordSegment", // [236]
    "REFRESH", // [237]
    "PurgeKeywordSegment", // [238]
    "GlobalKeywordSegment", // [239]
    "QueryKeywordSegment", // [240]
    "RewriteKeywordSegment", // [241]
    "REWRITE", // [242]
    "ReadKeywordSegment", // [243]
    "RedefineKeywordSegment", // [244]
    "ReferencesKeywordSegment", // [245]
    "RestrictedKeywordSegment", // [246]
    "ResumableKeywordSegment", // [247]
    "SelectKeywordSegment", // [248]
    "SignKeywordSegment", // [249]
    "RetentionKeywordSegment", // [250]
    "TranslateKeywordSegment", // [251]
    "UnderKeywordSegment", // [252]
    "UnlimitedKeywordSegment", // [253]
    "UpdateKeywordSegment", // [254]
    "UseKeywordSegment", // [255]
    "WriteKeywordSegment", // [256]
    "AccessPermissionSegment", // [257]
    "access_permission", // [258]
    "ANY", // [259]
    "PUBLIC", // [260]
    "AnyKeywordSegment", // [261]
    "PublicKeywordSegment", // [262]
    "AccessObjectSegment", // [263]
    "access_object", // [264]
    "RoleReferenceSegment", // [265]
    "role_reference", // [266]
    "object_reference", // [267]
    "(", // [268]
    "ColumnReferenceSegment", // [269]
    "column_reference", // [270]
    "CommaSegment", // [271]
    ",", // [272]
    "start_bracket", // [273]
    "SymbolSegment", // [274]
    "symbol", // [275]
    ")", // [276]
    "end_bracket", // [277]
    "FILE", // [278]
    "FUNCTION", // [279]
    "MASKING", // [280]
    "ROUTINE", // [281]
    "SCHEMA", // [282]
    "STAGE", // [283]
    "STREAM", // [284]
    "TASK", // [285]
    "StageKeywordSegment", // [286]
    "FunctionKeywordSegment", // [287]
    "RoutineKeywordSegment", // [288]
    "StreamKeywordSegment", // [289]
    "TaskKeywordSegment", // [290]
    "SchemaKeywordSegment", // [291]
    "MaskingKeywordSegment", // [292]
    "FileKeywordSegment", // [293]
    "FormatKeywordSegment", // [294]
    "FORMAT", // [295]
    "FUNCTIONS", // [296]
    "PROCEDURES", // [297]
    "ROUTINES", // [298]
    "SCHEMAS", // [299]
    "SEQUENCES", // [300]
    "STAGES", // [301]
    "STREAMS", // [302]
    "TABLES", // [303]
    "TASKS", // [304]
    "VIEWS", // [305]
    "TablesKeywordSegment", // [306]
    "ViewsKeywordSegment", // [307]
    "StagesKeywordSegment", // [308]
    "FunctionsKeywordSegment", // [309]
    "ProceduresKeywordSegment", // [310]
    "RoutinesKeywordSegment", // [311]
    "SequencesKeywordSegment", // [312]
    "StreamsKeywordSegment", // [313]
    "TasksKeywordSegment", // [314]
    "SchemasKeywordSegment", // [315]
    "PoliciesKeywordSegment", // [316]
    "FormatsKeywordSegment", // [317]
    "REVOKE", // [318]
    "GrantStatementSegment", // [319]
    "grant_statement", // [320]
    "RevokeStatementSegment", // [321]
    "revoke_statement", // [322]
    "PACKAGE", // [323]
    "PackageKeywordSegment", // [324]
    "SchemaReferenceSegment", // [325]
    "schema_reference", // [326]
    "DotSegment", // [327]
    ".", // [328]
    "FunctionNameSegment", // [329]
    "function_name", // [330]
    "ObjectReferenceSegment", // [331]
    "ACCESSIBLE", // [332]
    "AccessibleKeywordSegment", // [333]
    "ByKeywordSegment", // [334]
    "BY", // [335]
    "[", // [336]
    "ArrayAccessorSegment", // [337]
    "array_accessor", // [338]
    "ACCOUNT", // [339]
    "ACCOUNTS", // [340]
    "ACTION", // [341]
    "ACTIVE", // [342]
    "ADA", // [343]
    "ADD", // [344]
    "ADMIN", // [345]
    "ADVANCED", // [346]
    "ADVISE", // [347]
    "AFTER", // [348]
    "AGGREGATE", // [349]
    "OrderByClauseSegment", // [350]
    "ORDER", // [351]
    "orderby_clause", // [352]
    "indent", // [353]
    "Indent", // [354]
    "meta", // [355]
    "AsAliasOperatorSegment", // [356]
    "AS", // [357]
    "alias_operator", // [358]
    "SingleIdentifierGrammar", // [359]
    "SingleIdentifierListSegment", // [360]
    "identifier_list", // [361]
    "SingleQuotedIdentifierSegment", // [362]
    "single_quote", // [363]
    "dedent", // [364]
    "Dedent", // [365]
    "ALIAS", // [366]
    "TableReferenceSegment", // [367]
    "table_reference", // [368]
    "AliasExpressionSegment", // [369]
    "alias_expression", // [370]
    "ALLOCATE", // [371]
    "ALSO", // [372]
    "SharedKeywordSegment", // [373]
    "SHARED", // [374]
    "IfExistsGrammar", // [375]
    "IF", // [376]
    "DatabaseLinkReferenceSegment", // [377]
    "database_link_reference", // [378]
    "AUTHENTICATED", // [379]
    "WITH", // [380]
    "TO", // [381]
    "ToKeywordSegment", // [382]
    "IdentifiedKeywordSegment", // [383]
    "IDENTIFIED", // [384]
    "DBLinkAuthenticationGrammar", // [385]
    "WithKeywordSegment", // [386]
    "COMPILE", // [387]
    "EDITIONABLE", // [388]
    "NONEDITIONABLE", // [389]
    "CompileClauseGrammar", // [390]
    "EditionableKeywordSegment", // [391]
    "NoneditionableKeywordSegment", // [392]
    "IndexReferenceSegment", // [393]
    "index_reference", // [394]
    "DISABLE", // [395]
    "INVISIBLE", // [396]
    "LOGGING", // [397]
    "MONITORING", // [398]
    "NOLOGGING", // [399]
    "NOMONITORING", // [400]
    "PARAMETERS", // [401]
    "REBUILD", // [402]
    "RENAME", // [403]
    "UNUSABLE", // [404]
    "VISIBLE", // [405]
    "RebuildKeywordSegment", // [406]
    "NOREVERSE", // [407]
    "REVERSE", // [408]
    "ReverseKeywordSegment", // [409]
    "NoreverseKeywordSegment", // [410]
    "MonitoringKeywordSegment", // [411]
    "UsageKeywordSegment", // [412]
    "USAGE", // [413]
    "NomonitoringKeywordSegment", // [414]
    "ParametersKeywordSegment", // [415]
    "QuotedLiteralSegment", // [416]
    "RenameKeywordSegment", // [417]
    "CompileKeywordSegment", // [418]
    "LoggingKeywordSegment", // [419]
    "NologgingKeywordSegment", // [420]
    "DisableKeywordSegment", // [421]
    "UnusableKeywordSegment", // [422]
    "InvisibleKeywordSegment", // [423]
    "VisibleKeywordSegment", // [424]
    "PackageReferenceSegment", // [425]
    "package_reference", // [426]
    "CACHE", // [427]
    "CYCLE", // [428]
    "INCREMENT", // [429]
    "MAXVALUE", // [430]
    "MINVALUE", // [431]
    "NOCACHE", // [432]
    "NOCYCLE", // [433]
    "NOMAXVALUE", // [434]
    "NOMINVALUE", // [435]
    "NOORDER", // [436]
    "IncrementKeywordSegment", // [437]
    "NumericLiteralSegment", // [438]
    "numeric_literal", // [439]
    "SequenceMinValueGrammar", // [440]
    "SequenceMaxValueGrammar", // [441]
    "CacheKeywordSegment", // [442]
    "NocacheKeywordSegment", // [443]
    "CycleKeywordSegment", // [444]
    "NocycleKeywordSegment", // [445]
    "OrderNoOrderGrammar", // [446]
    "SequenceReferenceSegment", // [447]
    "sequence_reference", // [448]
    "AlterSequenceOptionsSegment", // [449]
    "alter_sequence_options_segment", // [450]
    "CLOSE", // [451]
    "SYNC", // [452]
    "AdviseKeywordSegment", // [453]
    "NOTHING", // [454]
    "NothingKeywordSegment", // [455]
    "CloseKeywordSegment", // [456]
    "InKeywordSegment", // [457]
    "IN", // [458]
    "GuardKeywordSegment", // [459]
    "GUARD", // [460]
    "ParallelKeywordSegment", // [461]
    "PARALLEL", // [462]
    "DDL", // [463]
    "DML", // [464]
    "DmlKeywordSegment", // [465]
    "DdlKeywordSegment", // [466]
    "TIMEOUT", // [467]
    "TimeoutKeywordSegment", // [468]
    "NAME", // [469]
    "NameKeywordSegment", // [470]
    "ShardKeywordSegment", // [471]
    "SHARD", // [472]
    "SyncKeywordSegment", // [473]
    "PrimaryKeywordSegment", // [474]
    "PRIMARY", // [475]
    "ISOLATION_LEVEL", // [476]
    "Isolation_levelKeywordSegment", // [477]
    "EqualsSegment", // [478]
    "=", // [479]
    "comparison_operator", // [480]
    "SERIALIZABLE", // [481]
    "SerializableKeywordSegment", // [482]
    "CommittedKeywordSegment", // [483]
    "COMMITTED", // [484]
    "CONSTRAINT", // [485]
    "CONSTRAINTS", // [486]
    "ConstraintKeywordSegment", // [487]
    "ConstraintsKeywordSegment", // [488]
    "DEFAULT", // [489]
    "DEFERRED", // [490]
    "IMMEDIATE", // [491]
    "ImmediateKeywordSegment", // [492]
    "DeferredKeywordSegment", // [493]
    "DefaultKeywordSegment", // [494]
    "TIME_ZONE", // [495]
    "Time_zoneKeywordSegment", // [496]
    "DBTIMEZONE", // [497]
    "LOCAL", // [498]
    "LocalKeywordSegment", // [499]
    "DbtimezoneKeywordSegment", // [500]
    "ArchivalKeywordSegment", // [501]
    "ARCHIVAL", // [502]
    "VisibilityKeywordSegment", // [503]
    "VISIBILITY", // [504]
    "ActiveKeywordSegment", // [505]
    "SERVICE", // [506]
    "ServiceKeywordSegment", // [507]
    "ParameterNameSegment", // [508]
    "ExpressionSegment", // [509]
    "expression", // [510]
    "MODIFY", // [511]
    "AddKeywordSegment", // [512]
    "ModifyKeywordSegment", // [513]
    "ColumnDefinitionSegment", // [514]
    "column_definition", // [515]
    "COLUMN", // [516]
    "ColumnKeywordSegment", // [517]
    "CHECK", // [518]
    "FOREIGN", // [519]
    "UNIQUE", // [520]
    "TableConstraintSegment", // [521]
    "table_constraint", // [522]
    "UniqueKeywordSegment", // [523]
    "CascadeKeywordSegment", // [524]
    "CASCADE", // [525]
    "OnlineKeywordSegment", // [526]
    "ONLINE", // [527]
    "LiteralGrammar", // [528]
    "NakedIdentifierSegment", // [529]
    "FIRST", // [530]
    "FirstKeywordSegment", // [531]
    "AfterKeywordSegment", // [532]
    "BracketedColumnReferenceListGrammar", // [533]
    "AlterTableDropColumnGrammar", // [534]
    "AsKeywordSegment", // [535]
    "AlterTablePropertiesSegment", // [536]
    "alter_table_properties", // [537]
    "AlterTableColumnClausesSegment", // [538]
    "alter_table_column_clauses", // [539]
    "AlterTableConstraintClauses", // [540]
    "alter_table_constraint_clauses", // [541]
    "ALWAYS", // [542]
    "&", // [543]
    "ampersand", // [544]
    "ANALYSE", // [545]
    "AND", // [546]
    "binary_operator", // [547]
    "BinaryOperatorSegment", // [548]
    "APPLY", // [549]
    "ARE", // [550]
    "%", // [551]
    "*", // [552]
    "**", // [553]
    "+", // [554]
    "-", // [555]
    "/", // [556]
    "<", // [557]
    ">", // [558]
    "MOD", // [559]
    "^", // [560]
    "|", // [561]
    "PlusSegment", // [562]
    "MinusSegment", // [563]
    "DivideSegment", // [564]
    "MultiplySegment", // [565]
    "ModuloSegment", // [566]
    "BitwiseAndSegment", // [567]
    "BitwiseOrSegment", // [568]
    "BitwiseXorSegment", // [569]
    "BitwiseLShiftSegment", // [570]
    "BitwiseRShiftSegment", // [571]
    "ModOperatorSegment", // [572]
    "PowerOperatorSegment", // [573]
    "SliceSegment", // [574]
    ":", // [575]
    "start_square_bracket", // [576]
    "]", // [577]
    "end_square_bracket", // [578]
    "ARRAY", // [579]
    "BaseExpressionElementGrammar", // [580]
    "ASC", // [581]
    "ASENSITIVE", // [582]
    "ASSERTION", // [583]
    "ASSIGNMENT", // [584]
    ":=", // [585]
    "assignment_operator", // [586]
    "TriggerCorrelationReferenceSegment", // [587]
    "bind_variable", // [588]
    "BindVariableSegment", // [589]
    "AssignmentOperatorSegment", // [590]
    "ASYMMETRIC", // [591]
    "AT", // [592]
    "@", // [593]
    "at_sign", // [594]
    "ATOMIC", // [595]
    "attribute_indicator", // [596]
    "ATTRIBUTES", // [597]
    "AUTHID", // [598]
    "AUTHORIZATION", // [599]
    "AUTO_INCREMENT", // [600]
    "Auto_incrementKeywordSegment", // [601]
    "AUTO", // [602]
    "AVG", // [603]
    "AVG_ROW_LENGTH", // [604]
    "BACKWARD", // [605]
    "COLUMN_VALUE", // [606]
    "CURRENT_DATE", // [607]
    "CURRENT_TIMESTAMP", // [608]
    "LOCALTIMESTAMP", // [609]
    "ORA_ROWSCN", // [610]
    "SESSIONTIMESTAMP", // [611]
    "SESSION_USER", // [612]
    "SYSDATE", // [613]
    "SYSTIMESTAMP", // [614]
    "UID", // [615]
    "bare_function", // [616]
    "CodeSegment", // [617]
    "BareFunctionSegment", // [618]
    "IntervalExpressionSegment", // [619]
    "INTERVAL", // [620]
    "interval_expression", // [621]
    "FunctionSegment", // [622]
    "function", // [623]
    "DatatypeSegment", // [624]
    "data_type", // [625]
    "ConnectByRootGrammar", // [626]
    "CONNECT_BY_ROOT", // [627]
    "SubstitutionVariableSegment", // [628]
    "substitution_variable", // [629]
    "file", // [630]
    "BaseFileSegment", // [631]
    "BaseSegment", // [632]
    "BASIC", // [633]
    "SlashBufferExecutorSegment", // [634]
    "slash_buffer_executor", // [635]
    "SqlplusSetStatementSegment", // [636]
    "sqlplus_set_statement", // [637]
    "StatementSegment", // [638]
    "statement", // [639]
    ";", // [640]
    "DelimiterGrammar", // [641]
    "BatchDelimiterGrammar", // [642]
    "BEFORE", // [643]
    "DeclareSegment", // [644]
    "declare_segment", // [645]
    "BeginKeywordSegment", // [646]
    "BEGIN", // [647]
    "OneOrMoreStatementsGrammar", // [648]
    "ExceptionBlockGrammar", // [649]
    "EXCEPTION", // [650]
    "EndKeywordSegment", // [651]
    "END", // [652]
    "BERNOULLI", // [653]
    "BETWEEN", // [654]
    "BIGINT", // [655]
    "BINARY", // [656]
    "!", // [657]
    "IS", // [658]
    "MULTISET", // [659]
    "OR", // [660]
    "like_operator", // [661]
    "ArithmeticBinaryOperatorGrammar", // [662]
    "StringBinaryOperatorGrammar", // [663]
    "BooleanBinaryOperatorGrammar", // [664]
    "ComparisonOperatorGrammar", // [665]
    "MultisetOperatorSegment", // [666]
    "bind_colon", // [667]
    "BindColonSegment", // [668]
    "BINDING", // [669]
    "BIT", // [670]
    "BIT_LENGTH", // [671]
    "BITMAP", // [672]
    "BITVAR", // [673]
    "AmpersandSegment", // [674]
    "RawLessThanSegment", // [675]
    "PipeSegment", // [676]
    "RawGreaterThanSegment", // [677]
    "BLOB", // [678]
    "BODY", // [679]
    "BOOL", // [680]
    "AndOperatorGrammar", // [681]
    "OrOperatorGrammar", // [682]
    "BOOLEAN", // [683]
    "FALSE", // [684]
    "TRUE", // [685]
    "TrueSegment", // [686]
    "FalseSegment", // [687]
    "BOTH", // [688]
    "bracketed", // [689]
    "BracketedSegment", // [690]
    "UnorderedSetExpressionSegment", // [691]
    "VALUE", // [692]
    "VALUES", // [693]
    "set_expression", // [694]
    "BREADTH", // [695]
    "BREAK", // [696]
    "BROWSE", // [697]
    "BUFFER_POOL", // [698]
    "BULK", // [699]
    "BulkKeywordSegment", // [700]
    "CollectKeywordSegment", // [701]
    "COLLECT", // [702]
    "IntoKeywordSegment", // [703]
    "INTO", // [704]
    "implicit_indent", // [705]
    "ImplicitIndent", // [706]
    "BULK_EXCEPTIONS", // [707]
    "BULK_ROWCOUNT", // [708]
    "BYTE", // [709]
    "CTEColumnList", // [710]
    "cte_column_list", // [711]
    "SelectableGrammar", // [712]
    "CALL", // [713]
    "CALLED", // [714]
    "CAPACITY", // [715]
    "CARDINALITY", // [716]
    "CASCADED", // [717]
    "CASE", // [718]
    "CaseKeywordSegment", // [719]
    "WHEN", // [720]
    "WhenClauseSegment", // [721]
    "WhenKeywordSegment", // [722]
    "when_clause", // [723]
    "ElseKeywordSegment", // [724]
    "ELSE", // [725]
    "ElseClauseSegment", // [726]
    "else_clause", // [727]
    "TriggerPredicatesExpressionGrammar", // [728]
    "DELETING", // [729]
    "INSERTING", // [730]
    "NOT", // [731]
    "UPDATING", // [732]
    "BinaryOperatorGrammar", // [733]
    "CAST", // [734]
    "::", // [735]
    "casting_operator", // [736]
    "CATALOG", // [737]
    "CATALOG_NAME", // [738]
    "CEIL", // [739]
    "CEILING", // [740]
    "CELL_FLASH_CACHE", // [741]
    "CHAIN", // [742]
    "CHAR", // [743]
    "CHAR_LENGTH", // [744]
    "CHARACTER", // [745]
    "CHARACTER_LENGTH", // [746]
    "CHARACTER_SET_CATALOG", // [747]
    "CHARACTER_SET_NAME", // [748]
    "CHARACTER_SET_SCHEMA", // [749]
    "CHARACTERISTICS", // [750]
    "CHARACTERS", // [751]
    "CHECKED", // [752]
    "CHECKPOINT", // [753]
    "CHECKSUM", // [754]
    "CLASS_ORIGIN", // [755]
    "CLOB", // [756]
    "CLUSTERED", // [757]
    "COALESCE", // [758]
    "COBOL", // [759]
    "COLLATE", // [760]
    "COLLATION", // [761]
    "ObjectReferenceDelimiterGrammar", // [762]
    "ObjectReferenceTerminatorGrammar", // [763]
    "PIVOT", // [764]
    "UNPIVOT", // [765]
    "USING", // [766]
    "COLLATION_CATALOG", // [767]
    "COLLATION_NAME", // [768]
    "COLLATION_SCHEMA", // [769]
    "IsKeywordSegment", // [770]
    "OfKeywordSegment", // [771]
    "OF", // [772]
    "ColumnTypeReferenceSegment", // [773]
    "column_type_reference", // [774]
    "RowTypeReferenceSegment", // [775]
    "row_type_reference", // [776]
    "NotKeywordSegment", // [777]
    "NullKeywordSegment", // [778]
    "NULL", // [779]
    "colon_delimiter", // [780]
    "colon_prefix", // [781]
    "colon", // [782]
    "ShorthandCastSegment", // [783]
    "cast_expression", // [784]
    "SequencePseudocolumnGrammar", // [785]
    "CheckKeywordSegment", // [786]
    "ColumnConstraintDefaultGrammar", // [787]
    "PrimaryKeyGrammar", // [788]
    "NotEnforcedGrammar", // [789]
    "UniqueKeyGrammar", // [790]
    "AutoIncrementGrammar", // [791]
    "ReferenceDefinitionGrammar", // [792]
    "CommentClauseSegment", // [793]
    "comment_clause", // [794]
    "CollateKeywordSegment", // [795]
    "CollationReferenceSegment", // [796]
    "collation_reference", // [797]
    "ColumnGeneratedGrammar", // [798]
    "ColumnConstraintSegment", // [799]
    "column_constraint_segment", // [800]
    "ByteKeywordSegment", // [801]
    "CharKeywordSegment", // [802]
    "IdentityClauseGrammar", // [803]
    "GENERATED", // [804]
    "AttributeIndicatorSegment", // [805]
    "COLUMN_NAME", // [806]
    "ColumnsExpressionNameGrammar", // [807]
    "COLUMNS", // [808]
    "comma", // [809]
    "COMMAND_FUNCTION", // [810]
    "COMMAND_FUNCTION_CODE", // [811]
    "comment", // [812]
    "CommentSegment", // [813]
    "IndexTypeReferenceSegment", // [814]
    "indextype_reference", // [815]
    "GreaterThanSegment", // [816]
    "LessThanSegment", // [817]
    "GreaterThanOrEqualToSegment", // [818]
    "LessThanOrEqualToSegment", // [819]
    "NotEqualToSegment", // [820]
    "LikeOperatorSegment", // [821]
    "IsDistinctFromGrammar", // [822]
    "ComparisonOperatorSegment", // [823]
    "SPECIFICATION", // [824]
    "SpecificationKeywordSegment", // [825]
    "BodyKeywordSegment", // [826]
    "REUSE", // [827]
    "ReuseKeywordSegment", // [828]
    "SettingsKeywordSegment", // [829]
    "COMPLETION", // [830]
    "CompositeBinaryOperatorSegment", // [831]
    "CompositeComparisonOperatorSegment", // [832]
    "COMPOUND", // [833]
    "CompoundKeywordSegment", // [834]
    "INSTEAD", // [835]
    "TimingPointSectionSegment", // [836]
    "timing_point_section", // [837]
    "COMPRESS", // [838]
    "COMPUTE", // [839]
    "CONDITION", // [840]
    "CONDITION_NUMBER", // [841]
    "JoinTypeKeywordsGrammar", // [842]
    "FULL", // [843]
    "INNER", // [844]
    "LEFT", // [845]
    "RIGHT", // [846]
    "ConditionalCrossJoinKeywordsGrammar", // [847]
    "NonStandardJoinTypeKeywordsGrammar", // [848]
    "Connect_by_rootKeywordSegment", // [849]
    "CONNECTION", // [850]
    "CONNECTION_NAME", // [851]
    "CONSTANT", // [852]
    "CONSTRAINT_CATALOG", // [853]
    "CONSTRAINT_NAME", // [854]
    "CONSTRAINT_SCHEMA", // [855]
    "CONSTRUCTOR", // [856]
    "CONTAINS", // [857]
    "CONTAINSTABLE", // [858]
    "CONTINUE", // [859]
    "ContinueKeywordSegment", // [860]
    "CONVERSION", // [861]
    "CONVERT", // [862]
    "COPY", // [863]
    "CORR", // [864]
    "CORRESPONDING", // [865]
    "COUNT", // [866]
    "COVAR_POP", // [867]
    "COVAR_SAMP", // [868]
    "CastKeywordSegment", // [869]
    "SpecificKeywordSegment", // [870]
    "SPECIFIC", // [871]
    "INSTANCE", // [872]
    "METHOD", // [873]
    "STATIC", // [874]
    "InstanceKeywordSegment", // [875]
    "StaticKeywordSegment", // [876]
    "ConstructorKeywordSegment", // [877]
    "MethodKeywordSegment", // [878]
    "FunctionParameterListGrammar", // [879]
    "function_parameter_list", // [880]
    "FOR", // [881]
    "ForKeywordSegment", // [882]
    "AssignmentKeywordSegment", // [883]
    "IfNotExistsGrammar", // [884]
    "Current_userKeywordSegment", // [885]
    "CURRENT_USER", // [886]
    "UsingKeywordSegment", // [887]
    "DatabaseReferenceSegment", // [888]
    "database_reference", // [889]
    "OrKeywordSegment", // [890]
    "ReplaceKeywordSegment", // [891]
    "REPLACE", // [892]
    "ReturnKeywordSegment", // [893]
    "RETURN", // [894]
    "SharingClauseGrammar", // [895]
    "SHARING", // [896]
    "DETERMINISTIC", // [897]
    "PARALLEL_ENABLE", // [898]
    "PIPELINED", // [899]
    "RESULT_CACHE", // [900]
    "SHARD_ENABLE", // [901]
    "SQL_MACRO", // [902]
    "DefaultCollationClauseGrammar", // [903]
    "InvokerRightsClauseGrammar", // [904]
    "AccessibleByClauseGrammar", // [905]
    "DeterministicKeywordSegment", // [906]
    "Shard_enableKeywordSegment", // [907]
    "ParallelEnableClauseGrammar", // [908]
    "ResultCacheClauseGrammar", // [909]
    "AggregateKeywordSegment", // [910]
    "PipelinedClauseGrammar", // [911]
    "Sql_macroKeywordSegment", // [912]
    "RightArrowSegment", // [913]
    "=>", // [914]
    "ScalarKeywordSegment", // [915]
    "LANGUAGE", // [916]
    "LanguageKeywordSegment", // [917]
    "JavaKeywordSegment", // [918]
    "JAVA", // [919]
    "BeginEndSegment", // [920]
    "begin_end_block", // [921]
    "BitmapKeywordSegment", // [922]
    "IndexColumnDefinitionSegment", // [923]
    "index_column_definition", // [924]
    "OracleIndexPhysicalAttributesSegment", // [925]
    "INITRANS", // [926]
    "MAXTRANS", // [927]
    "NOCOMPRESS", // [928]
    "NOPARALLEL", // [929]
    "NOSORT", // [930]
    "PCTFREE", // [931]
    "STORAGE", // [932]
    "oracle_index_physical_attributes", // [933]
    "OrReplaceGrammar", // [934]
    "OPTIONS", // [935]
    "OptionsKeywordSegment", // [936]
    "START", // [937]
    "StartKeywordSegment", // [938]
    "CreateSequenceOptionsSegment", // [939]
    "create_sequence_options_segment", // [940]
    "AtSignSegment", // [941]
    "TemporaryGrammar", // [942]
    "PRIVATE", // [943]
    "INMEMORY", // [944]
    "LIKE", // [945]
    "NO", // [946]
    "NOROWDEPENDENCIES", // [947]
    "PCTUSED", // [948]
    "ROWDEPENDENCIES", // [949]
    "OnCommitGrammar", // [950]
    "OraclePhysicalAttributesSegment", // [951]
    "oracle_physical_attributes", // [952]
    "LikeKeywordSegment", // [953]
    "TableEndClauseSegment", // [954]
    "table_end_clause_segment", // [955]
    "TriggerReferenceSegment", // [956]
    "trigger_reference", // [957]
    "BeforeKeywordSegment", // [958]
    "InsteadKeywordSegment", // [959]
    "DmlEventClauseSegment", // [960]
    "dml_event_clause", // [961]
    "ReferencingClauseSegment", // [962]
    "REFERENCING", // [963]
    "referencing_clause", // [964]
    "EachKeywordSegment", // [965]
    "EACH", // [966]
    "CROSSEDITION", // [967]
    "FORWARD", // [968]
    "ForwardKeywordSegment", // [969]
    "CrosseditionKeywordSegment", // [970]
    "FOLLOWS", // [971]
    "PRECEDES", // [972]
    "FollowsKeywordSegment", // [973]
    "PrecedesKeywordSegment", // [974]
    "CompoundTriggerBlock", // [975]
    "compound_trigger_statement", // [976]
    "TypeReferenceSegment", // [977]
    "type_reference", // [978]
    "ElementSpecificationGrammar", // [979]
    "MEMBER", // [980]
    "OID", // [981]
    "OidKeywordSegment", // [982]
    "VARRAY", // [983]
    "VARYING", // [984]
    "ObjectTypeAndSubtypeDefGrammar", // [985]
    "VarrayAndNestedTypeSpecGrammar", // [986]
    "EXTERNALLY", // [987]
    "GLOBALLY", // [988]
    "DIGEST", // [989]
    "HTTP", // [990]
    "HttpKeywordSegment", // [991]
    "DigestKeywordSegment", // [992]
    "ExternallyKeywordSegment", // [993]
    "GloballyKeywordSegment", // [994]
    "double_quote", // [995]
    "QuotedIdentifierSegment", // [996]
    "NoKeywordSegment", // [997]
    "AuthenticationKeywordSegment", // [998]
    "PASSWORD", // [999]
    "QUOTA", // [1000]
    "TEMPORARY", // [1001]
    "TemporaryKeywordSegment", // [1002]
    "QuotaKeywordSegment", // [1003]
    "SizeClauseGrammar", // [1004]
    "PasswordKeywordSegment", // [1005]
    "ExpireKeywordSegment", // [1006]
    "EXPIRE", // [1007]
    "AccountKeywordSegment", // [1008]
    "UNLOCK", // [1009]
    "UnlockKeywordSegment", // [1010]
    "EditionsKeywordSegment", // [1011]
    "EDITIONS", // [1012]
    "CURRENT", // [1013]
    "CurrentKeywordSegment", // [1014]
    "ONLY", // [1015]
    "OnlyKeywordSegment", // [1016]
    "EDITIONING", // [1017]
    "EditioningKeywordSegment", // [1018]
    "WithNoSchemaBindingClauseSegment", // [1019]
    "with_no_schema_binding_clause", // [1020]
    "CREATEDB", // [1021]
    "CREATEROLE", // [1022]
    "CREATEUSER", // [1023]
    "CREATION", // [1024]
    "CRITICAL", // [1025]
    "CROSS", // [1026]
    "CSV", // [1027]
    "function_name_identifier", // [1028]
    "ROLLUP", // [1029]
    "CubeFunctionNameSegment", // [1030]
    "RollupFunctionNameSegment", // [1031]
    "GroupingExpressionList", // [1032]
    "grouping_expression_list", // [1033]
    "CUME_DIST", // [1034]
    "CURRENT_DEFAULT_TRANSFORM_GROUP", // [1035]
    "CURRENT_PATH", // [1036]
    "CURRENT_ROLE", // [1037]
    "CURRENT_TIME", // [1038]
    "CURRENT_TRANSFORM_GROUP_FOR_TYPE", // [1039]
    "CURRVAL", // [1040]
    "CURSOR", // [1041]
    "CURSOR_NAME", // [1042]
    "AuthenticatedKeywordSegment", // [1043]
    "CredentialKeywordSegment", // [1044]
    "DATA", // [1045]
    "DATABASES", // [1046]
    "data_type_identifier", // [1047]
    "IntervalDataTypeSegment", // [1048]
    "interval_data_type", // [1049]
    "TimeWithTZGrammar", // [1050]
    "TIMESTAMP", // [1051]
    "DOUBLE", // [1052]
    "DoubleKeywordSegment", // [1053]
    "PrecisionKeywordSegment", // [1054]
    "PRECISION", // [1055]
    "CharacterKeywordSegment", // [1056]
    "BinaryKeywordSegment", // [1057]
    "LARGE", // [1058]
    "VaryingKeywordSegment", // [1059]
    "LargeKeywordSegment", // [1060]
    "DatatypeIdentifierSegment", // [1061]
    "BracketedArguments", // [1062]
    "bracketed_arguments", // [1063]
    "UnsignedKeywordSegment", // [1064]
    "UNSIGNED", // [1065]
    "CharCharacterSetGrammar", // [1066]
    "ArrayTypeSegment", // [1067]
    "array_type", // [1068]
    "DATEADD", // [1069]
    "DatePartFunctionName", // [1070]
    "DatetimeUnitSegment", // [1071]
    "DAY", // [1072]
    "DAYOFYEAR", // [1073]
    "HOUR", // [1074]
    "MILLISECOND", // [1075]
    "MINUTE", // [1076]
    "MONTH", // [1077]
    "QUARTER", // [1078]
    "SECOND", // [1079]
    "WEEK", // [1080]
    "WEEKDAY", // [1081]
    "YEAR", // [1082]
    "FunctionContentsGrammar", // [1083]
    "TimestampKeywordSegment", // [1084]
    "IntervalKeywordSegment", // [1085]
    "date_constructor_literal", // [1086]
    "literal", // [1087]
    "LiteralSegment", // [1088]
    "IntervalUnitsGrammar", // [1089]
    "DATETIME", // [1090]
    "date_part", // [1091]
    "DATETIME_INTERVAL_CODE", // [1092]
    "DATETIME_INTERVAL_PRECISION", // [1093]
    "DAY_HOUR", // [1094]
    "DAY_MICROSECOND", // [1095]
    "DAY_MINUTE", // [1096]
    "DAY_SECOND", // [1097]
    "DAYOFMONTH", // [1098]
    "DAYOFWEEK", // [1099]
    "DAYS", // [1100]
    "DBCC", // [1101]
    "DEALLOCATE", // [1102]
    "DEC", // [1103]
    "DECIMAL", // [1104]
    "CursorKeywordSegment", // [1105]
    "SelectStatementSegment", // [1106]
    "select_statement", // [1107]
    "DECLARE", // [1108]
    "DeclareKeywordSegment", // [1109]
    "ConstantKeywordSegment", // [1110]
    "PRAGMA", // [1111]
    "PragmaKeywordSegment", // [1112]
    "CollectionTypeDefinitionSegment", // [1113]
    "collection_type", // [1114]
    "RecordTypeDefinitionSegment", // [1115]
    "record_type", // [1116]
    "RefCursorTypeDefinitionSegment", // [1117]
    "ref_cursor_type", // [1118]
    "CreateProcedureStatementSegment", // [1119]
    "create_procedure_statement", // [1120]
    "CreateFunctionStatementSegment", // [1121]
    "create_function_statement", // [1122]
    "DeclareCursorVariableSegment", // [1123]
    "cursor_variable", // [1124]
    "CollationKeywordSegment", // [1125]
    "ValuesKeywordSegment", // [1126]
    "DEFAULTS", // [1127]
    "DEFERRABLE", // [1128]
    "DEFINED", // [1129]
    "DEFINER", // [1130]
    "DEFINITION", // [1131]
    "DEGREE", // [1132]
    "DELAY_KEY_WRITE", // [1133]
    "DELAYED", // [1134]
    "DELEGATE", // [1135]
    "FromClauseSegment", // [1136]
    "FROM", // [1137]
    "from_clause", // [1138]
    "WhereClauseSegment", // [1139]
    "WHERE", // [1140]
    "where_clause", // [1141]
    "ReturningClauseSegment", // [1142]
    "RETURNING", // [1143]
    "returning_clause", // [1144]
    "SemicolonSegment", // [1145]
    "DELIMITER", // [1146]
    "DELIMITERS", // [1147]
    "DENSE_RANK", // [1148]
    "DENY", // [1149]
    "DEPTH", // [1150]
    "DEREF", // [1151]
    "DERIVED", // [1152]
    "DESC", // [1153]
    "DESCRIBE", // [1154]
    "DescribeKeywordSegment", // [1155]
    "DESCRIPTOR", // [1156]
    "DESTROY", // [1157]
    "DESTRUCTOR", // [1158]
    "DIRECTORIES", // [1159]
    "DISCONNECT", // [1160]
    "DISK", // [1161]
    "DISPATCH", // [1162]
    "DISTINCT", // [1163]
    "DISTINCTROW", // [1164]
    "DISTRIBUTE", // [1165]
    "DISTRIBUTED", // [1166]
    "DIV", // [1167]
    "DmlGrammar", // [1168]
    "NESTED", // [1169]
    "NestedKeywordSegment", // [1170]
    "DO", // [1171]
    "dot", // [1172]
    "DropBehaviorGrammar", // [1173]
    "INCLUDING", // [1174]
    "IncludingKeywordSegment", // [1175]
    "VALIDATE", // [1176]
    "ValidateKeywordSegment", // [1177]
    "DUMMY", // [1178]
    "DUMP", // [1179]
    "DUPLICATE", // [1180]
    "DYNAMIC", // [1181]
    "DYNAMIC_FUNCTION", // [1182]
    "DYNAMIC_FUNCTION_CODE", // [1183]
    "ELEMENT", // [1184]
    "FINAL", // [1185]
    "INSTANTIABLE", // [1186]
    "OVERRIDING", // [1187]
    "OverridingKeywordSegment", // [1188]
    "FinalKeywordSegment", // [1189]
    "InstantiableKeywordSegment", // [1190]
    "MemberKeywordSegment", // [1191]
    "ELSEIF", // [1192]
    "ELSIF", // [1193]
    "EMPTY", // [1194]
    "StructTypeSegment", // [1195]
    "struct_type", // [1196]
    "EmptyStructLiteralBracketsSegment", // [1197]
    "struct_literal", // [1198]
    "ENCLOSED", // [1199]
    "ENCODING", // [1200]
    "ENCRYPTED", // [1201]
    "END-EXEC", // [1202]
    "}", // [1203]
    "end_curly_bracket", // [1204]
    "ENUM", // [1205]
    "EQUALS", // [1206]
    "RawEqualsSegment", // [1207]
    "ERRLVL", // [1208]
    "ERROR", // [1209]
    "ERRORS", // [1210]
    "ESCAPE", // [1211]
    "ESCAPED", // [1212]
    "EVALNAME", // [1213]
    "EVERY", // [1214]
    "EXCEPT", // [1215]
    "ExceptionKeywordSegment", // [1216]
    "ExceptionWhenHandlerGrammar", // [1217]
    "OthersKeywordSegment", // [1218]
    "OTHERS", // [1219]
    "ThenKeywordSegment", // [1220]
    "THEN", // [1221]
    "EXCLUDE", // [1222]
    "EXCLUDING", // [1223]
    "EXCLUSIVE", // [1224]
    "EXEC", // [1225]
    "SlashSegment", // [1226]
    "IntoClauseSegment", // [1227]
    "into_clause", // [1228]
    "BulkCollectIntoClauseSegment", // [1229]
    "bulk_collect_into_clause", // [1230]
    "OUT", // [1231]
    "OutKeywordSegment", // [1232]
    "ReturningKeywordSegment", // [1233]
    "EXECUTION", // [1234]
    "EXISTING", // [1235]
    "EXISTS", // [1236]
    "EXIT", // [1237]
    "ExitKeywordSegment", // [1238]
    "EXP", // [1239]
    "EXPLAIN", // [1240]
    "ExplainKeywordSegment", // [1241]
    "InsertStatementSegment", // [1242]
    "insert_statement", // [1243]
    "UpdateStatementSegment", // [1244]
    "update_statement", // [1245]
    "DeleteStatementSegment", // [1246]
    "delete_statement", // [1247]
    "Expression_A_Grammar", // [1248]
    "Tail_Recurse_Expression_A_Grammar", // [1249]
    "LikeExpressionGrammar", // [1250]
    "ILIKE", // [1251]
    "RLIKE", // [1252]
    "InOperatorGrammar", // [1253]
    "QuantifiedComparisonOperatorGrammar", // [1254]
    "IsClauseGrammar", // [1255]
    "IsNullGrammar", // [1256]
    "NotNullGrammar", // [1257]
    "CollateGrammar", // [1258]
    "BetweenKeywordSegment", // [1259]
    "Expression_B_Grammar", // [1260]
    "AndKeywordSegment", // [1261]
    "PatternMatchingGrammar", // [1262]
    "PRIOR", // [1263]
    "~", // [1264]
    "SignedSegmentGrammar", // [1265]
    "QualifiedNumericLiteralSegment", // [1266]
    "TildeSegment", // [1267]
    "NotOperatorGrammar", // [1268]
    "PriorKeywordSegment", // [1269]
    "Tail_Recurse_Expression_B_Grammar", // [1270]
    "ExistsKeywordSegment", // [1271]
    "Expression_D_Grammar", // [1272]
    "CaseExpressionSegment", // [1273]
    "case_expression", // [1274]
    "TimeZoneGrammar", // [1275]
    "time_zone_grammar", // [1276]
    "PlusJoinGrammar", // [1277]
    "LocalAliasSegment", // [1278]
    "local_alias_segment", // [1279]
    "TypedStructLiteralSegment", // [1280]
    "typed_struct_literal", // [1281]
    "ArrayExpressionSegment", // [1282]
    "array_expression", // [1283]
    "StarSegment", // [1284]
    "BooleanLiteralGrammar", // [1285]
    "NullLiteralSegment", // [1286]
    "DateTimeLiteralGrammar", // [1287]
    "ImplicitCursorAttributesGrammar", // [1288]
    "AccessorGrammar", // [1289]
    "OverlapsClauseSegment", // [1290]
    "OVERLAPS", // [1291]
    "overlaps_clause", // [1292]
    "EXTENSION", // [1293]
    "EXTRACT", // [1294]
    "boolean_literal", // [1295]
    "LiteralKeywordSegment", // [1296]
    "FETCH", // [1297]
    "FetchKeywordSegment", // [1298]
    "NEXT", // [1299]
    "NextKeywordSegment", // [1300]
    "ROWS", // [1301]
    "RowsKeywordSegment", // [1302]
    "TiesKeywordSegment", // [1303]
    "TIES", // [1304]
    "LIMIT", // [1305]
    "LimitKeywordSegment", // [1306]
    "FIELDS", // [1307]
    "BatchSegment", // [1308]
    "batch", // [1309]
    "ExecuteFileSegment", // [1310]
    "execute_file_statement", // [1311]
    "FILLFACTOR", // [1312]
    "FILTER", // [1313]
    "FilterKeywordSegment", // [1314]
    "WhereKeywordSegment", // [1315]
    "FLASH_CACHE", // [1316]
    "FLOAT4", // [1317]
    "FLOAT8", // [1318]
    "FLOAT", // [1319]
    "FLOOR", // [1320]
    "FLUSH", // [1321]
    "FOLLOWING", // [1322]
    "FORALL", // [1323]
    "ForallKeywordSegment", // [1324]
    "IterationSteppedControlGrammar", // [1325]
    "SAVE", // [1326]
    "SaveKeywordSegment", // [1327]
    "ExceptionsKeywordSegment", // [1328]
    "IMMUTABLE", // [1329]
    "MUTABLE", // [1330]
    "MutableKeywordSegment", // [1331]
    "ImmutableKeywordSegment", // [1332]
    "RepeatKeywordSegment", // [1333]
    "REPEAT", // [1334]
    "INDICES", // [1335]
    "PAIRS", // [1336]
    "IndicesKeywordSegment", // [1337]
    "PairsKeywordSegment", // [1338]
    "WHILE", // [1339]
    "WhileKeywordSegment", // [1340]
    "LoopStatementSegment", // [1341]
    "loop_statement", // [1342]
    "NOWAIT", // [1343]
    "SKIP", // [1344]
    "WAIT", // [1345]
    "NowaitKeywordSegment", // [1346]
    "WaitKeywordSegment", // [1347]
    "SkipKeywordSegment", // [1348]
    "LockedKeywordSegment", // [1349]
    "LOCKED", // [1350]
    "ForeignKeywordSegment", // [1351]
    "FORTRAN", // [1352]
    "FOUND", // [1353]
    "RANGE", // [1354]
    "FrameClauseUnitGrammar", // [1355]
    "UnboundedKeywordSegment", // [1356]
    "UNBOUNDED", // [1357]
    "PRECEDING", // [1358]
    "PrecedingKeywordSegment", // [1359]
    "FollowingKeywordSegment", // [1360]
    "RangeKeywordSegment", // [1361]
    "FREE", // [1362]
    "FREELIST", // [1363]
    "FREELISTS", // [1364]
    "FREETEXT", // [1365]
    "FREETEXTTABLE", // [1366]
    "FREEZE", // [1367]
    "FromKeywordSegment", // [1368]
    "FromExpressionSegment", // [1369]
    "from_expression", // [1370]
    "GROUP", // [1371]
    "HAVING", // [1372]
    "INTERSECT", // [1373]
    "MINUS", // [1374]
    "OFFSET", // [1375]
    "QUALIFY", // [1376]
    "UNION", // [1377]
    "WINDOW", // [1378]
    "LimitClauseSegment", // [1379]
    "limit_clause", // [1380]
    "GroupKeywordSegment", // [1381]
    "OrderKeywordSegment", // [1382]
    "HavingKeywordSegment", // [1383]
    "QualifyKeywordSegment", // [1384]
    "WindowKeywordSegment", // [1385]
    "SetOperatorSegment", // [1386]
    "set_operator", // [1387]
    "WithDataClauseSegment", // [1388]
    "with_data_clause", // [1389]
    "OffsetKeywordSegment", // [1390]
    "PreTableFunctionKeywordsGrammar", // [1391]
    "LATERAL", // [1392]
    "TableExpressionSegment", // [1393]
    "table_expression", // [1394]
    "TemporalQuerySegment", // [1395]
    "temporal_query", // [1396]
    "FromClauseTerminatorGrammar", // [1397]
    "SamplingExpressionSegment", // [1398]
    "TABLESAMPLE", // [1399]
    "sample_expression", // [1400]
    "JoinLikeClauseGrammar", // [1401]
    "JoinClauseSegment", // [1402]
    "join_clause", // [1403]
    "PostTableExpressionGrammar", // [1404]
    "MLTableExpressionSegment", // [1405]
    "ML", // [1406]
    "ml_table_expression", // [1407]
    "FromExpressionElementSegment", // [1408]
    "from_expression_element", // [1409]
    "indented_joins", // [1410]
    "FULLTEXT", // [1411]
    "NamedArgumentSegment", // [1412]
    "named_argument", // [1413]
    "LEADING", // [1414]
    "TRAILING", // [1415]
    "TrimParametersGrammar", // [1416]
    "DistinctKeywordSegment", // [1417]
    "FunctionContentsExpressionGrammar", // [1418]
    "AggregateOrderByClause", // [1419]
    "aggregate_order_by", // [1420]
    "SEPARATOR", // [1421]
    "SeparatorKeywordSegment", // [1422]
    "IgnoreRespectNullsGrammar", // [1423]
    "IGNORE", // [1424]
    "RESPECT", // [1425]
    "EmptyStructLiteralSegment", // [1426]
    "ListaggOverflowClauseSegment", // [1427]
    "listagg_overflow_clause", // [1428]
    "JSONObjectContentSegment", // [1429]
    "WordSegment", // [1430]
    "FunctionNameIdentifierSegment", // [1431]
    "NocopyKeywordSegment", // [1432]
    "NOCOPY", // [1433]
    "FunctionParameterGrammar", // [1434]
    "JSON_TABLE", // [1435]
    "JsonTableFunctionNameSegment", // [1436]
    "JsonTableFunctionContentsSegment", // [1437]
    "function_contents", // [1438]
    "XMLATTRIBUTES", // [1439]
    "XmlAttributesFunctionNameSegment", // [1440]
    "XmlAttributesFunctionContentsSegment", // [1441]
    "DatePartFunctionNameSegment", // [1442]
    "DateTimeFunctionContentsSegment", // [1443]
    "ColumnsExpressionGrammar", // [1444]
    "ColumnsExpressionFunctionNameSegment", // [1445]
    "ValuesClauseSegment", // [1446]
    "values_clause", // [1447]
    "FunctionContentsSegment", // [1448]
    "PostFunctionGrammar", // [1449]
    "FUSION", // [1450]
    "FUTURE", // [1451]
    "G", // [1452]
    "GENERAL", // [1453]
    "GET", // [1454]
    "glob_operator", // [1455]
    "GO", // [1456]
    "GOTO", // [1457]
    "AccessPermissionsSegment", // [1458]
    "access_permissions", // [1459]
    "AccessTargetSegment", // [1460]
    "access_target", // [1461]
    "AdminKeywordSegment", // [1462]
    "DelegateKeywordSegment", // [1463]
    "OptionKeywordSegment", // [1464]
    "OPTION", // [1465]
    "SOURCE", // [1466]
    "SourceKeywordSegment", // [1467]
    "GRANTED", // [1468]
    "GRANTS", // [1469]
    "RawNotSegment", // [1470]
    "GREATEST", // [1471]
    "GroupingSetsClauseSegment", // [1472]
    "GROUPING", // [1473]
    "grouping_sets_clause", // [1474]
    "CubeRollupClauseSegment", // [1475]
    "cube_rollup_clause", // [1476]
    "GroupByClauseTerminatorGrammar", // [1477]
    "GroupingKeywordSegment", // [1478]
    "SetsKeywordSegment", // [1479]
    "SETS", // [1480]
    "GROUPS", // [1481]
    "HANDLER", // [1482]
    "HEADER", // [1483]
    "HEAP", // [1484]
    "ConnectByClauseSegment", // [1485]
    "connectby_clause", // [1486]
    "StartWithClauseSegment", // [1487]
    "startwith_clause", // [1488]
    "HIGH", // [1489]
    "HIGH_PRIORITY", // [1490]
    "HOLD", // [1491]
    "HOLDLOCK", // [1492]
    "HOST", // [1493]
    "HOSTS", // [1494]
    "HOUR_MICROSECOND", // [1495]
    "HOUR_MINUTE", // [1496]
    "HOUR_SECOND", // [1497]
    "identifier", // [1498]
    "IdentifierSegment", // [1499]
    "GeneratedKeywordSegment", // [1500]
    "AlwaysKeywordSegment", // [1501]
    "IdentityKeywordSegment", // [1502]
    "IDENTITY", // [1503]
    "IdentityOptionsGrammar", // [1504]
    "MaxvalueKeywordSegment", // [1505]
    "MinvalueKeywordSegment", // [1506]
    "ValueKeywordSegment", // [1507]
    "NomaxvalueKeywordSegment", // [1508]
    "NominvalueKeywordSegment", // [1509]
    "NoorderKeywordSegment", // [1510]
    "IDENTITY_INSERT", // [1511]
    "IDENTITYCOL", // [1512]
    "IfKeywordSegment", // [1513]
    "IfClauseSegment", // [1514]
    "if_clause", // [1515]
    "ElsifKeywordSegment", // [1516]
    "IgnoreKeywordSegment", // [1517]
    "RespectKeywordSegment", // [1518]
    "NullsKeywordSegment", // [1519]
    "NULLS", // [1520]
    "IMPLEMENTATION", // [1521]
    "ISOPEN", // [1522]
    "NOTFOUND", // [1523]
    "ROWCOUNT", // [1524]
    "IsopenKeywordSegment", // [1525]
    "FoundKeywordSegment", // [1526]
    "NotfoundKeywordSegment", // [1527]
    "RowcountKeywordSegment", // [1528]
    "Bulk_rowcountKeywordSegment", // [1529]
    "Bulk_exceptionsKeywordSegment", // [1530]
    "IMPLICIT", // [1531]
    "IMPORTED", // [1532]
    "INCLUDE", // [1533]
    "AscKeywordSegment", // [1534]
    "DescKeywordSegment", // [1535]
    "INDICATOR", // [1536]
    "INFILE", // [1537]
    "INFIX", // [1538]
    "INHERITANY", // [1539]
    "INHERITS", // [1540]
    "INITIAL", // [1541]
    "INITIALIZE", // [1542]
    "INITIALLY", // [1543]
    "INOUT", // [1544]
    "INPUT", // [1545]
    "INSENSITIVE", // [1546]
    "SetClauseSegment", // [1547]
    "set_clause", // [1548]
    "POSITION", // [1549]
    "PositionKeywordSegment", // [1550]
    "LOG", // [1551]
    "LogKeywordSegment", // [1552]
    "ErrorsKeywordSegment", // [1553]
    "REJECT", // [1554]
    "RejectKeywordSegment", // [1555]
    "INSERT_ID", // [1556]
    "INT1", // [1557]
    "INT2", // [1558]
    "INT3", // [1559]
    "INT4", // [1560]
    "INT8", // [1561]
    "INT", // [1562]
    "INTEGER", // [1563]
    "INTEGRATION", // [1564]
    "INTEGRATIONS", // [1565]
    "INTERSECTION", // [1566]
    "YearKeywordSegment", // [1567]
    "MonthKeywordSegment", // [1568]
    "DayKeywordSegment", // [1569]
    "SecondKeywordSegment", // [1570]
    "HourKeywordSegment", // [1571]
    "MinuteKeywordSegment", // [1572]
    "INVOKER", // [1573]
    "AuthidKeywordSegment", // [1574]
    "DefinerKeywordSegment", // [1575]
    "NanLiteralSegment", // [1576]
    "NAN", // [1577]
    "UnknownLiteralSegment", // [1578]
    "NormalizedGrammar", // [1579]
    "EmptyKeywordSegment", // [1580]
    "ISAM", // [1581]
    "ISNULL", // [1582]
    "ISOLATION", // [1583]
    "ITERATE", // [1584]
    "IterationBoundsGrammar", // [1585]
    "StepKeywordSegment", // [1586]
    "JSONRegularEntrySegment", // [1587]
    "JsonKeywordSegment", // [1588]
    "JSON", // [1589]
    "WildcardIdentifierSegment", // [1590]
    "wildcard_identifier", // [1591]
    "JSONEntrySegment", // [1592]
    "JSONOnNullClause", // [1593]
    "JSONReturningClause", // [1594]
    "StrictKeywordSegment", // [1595]
    "STRICT", // [1596]
    "KeysKeywordSegment", // [1597]
    "KEYS", // [1598]
    "AbsentKeywordSegment", // [1599]
    "ColonSegment", // [1600]
    "VARCHAR", // [1601]
    "VarcharKeywordSegment", // [1602]
    "TypenameKeywordSegment", // [1603]
    "ClobKeywordSegment", // [1604]
    "BlobKeywordSegment", // [1605]
    "ConditionalJoinKeywordsGrammar", // [1606]
    "JoinKeywordsGrammar", // [1607]
    "JOIN", // [1608]
    "NestedJoinGrammar", // [1609]
    "indented_using_on", // [1610]
    "MatchConditionSegment", // [1611]
    "match_condition", // [1612]
    "JoinOnConditionSegment", // [1613]
    "join_on_condition", // [1614]
    "JoinUsingConditionGrammar", // [1615]
    "UnconditionalJoinKeywordsGrammar", // [1616]
    "ExtendedNaturalJoinKeywordsGrammar", // [1617]
    "JoinKeywordSegment", // [1618]
    "PivotSegment", // [1619]
    "pivot_clause", // [1620]
    "UnpivotSegment", // [1621]
    "unpivot_clause", // [1622]
    "indented_on_contents", // [1623]
    "InnerKeywordSegment", // [1624]
    "FullKeywordSegment", // [1625]
    "LeftKeywordSegment", // [1626]
    "RightKeywordSegment", // [1627]
    "OuterKeywordSegment", // [1628]
    "OUTER", // [1629]
    "OrdinalityKeywordSegment", // [1630]
    "ORDINALITY", // [1631]
    "PathKeywordSegment", // [1632]
    "PATH", // [1633]
    "JsonTableColumnsClauseSegment", // [1634]
    "json_table_columns_clause", // [1635]
    "ErrorKeywordSegment", // [1636]
    "ColumnsKeywordSegment", // [1637]
    "JsonTableColumnDefinitionSegment", // [1638]
    "json_table_column_definition", // [1639]
    "K", // [1640]
    "Dense_rankKeywordSegment", // [1641]
    "LAST", // [1642]
    "LastKeywordSegment", // [1643]
    "KEY_MEMBER", // [1644]
    "KEY_TYPE", // [1645]
    "KILL", // [1646]
    "LANCOMPILER", // [1647]
    "LAST_INSERT_ID", // [1648]
    "LEAST", // [1649]
    "LEAVE", // [1650]
    "LENGTH", // [1651]
    "LESS", // [1652]
    "LikeGrammar", // [1653]
    "EscapeKeywordSegment", // [1654]
    "RlikeKeywordSegment", // [1655]
    "IlikeKeywordSegment", // [1656]
    "LINENO", // [1657]
    "LINES", // [1658]
    "OverflowKeywordSegment", // [1659]
    "OVERFLOW", // [1660]
    "TRUNCATE", // [1661]
    "TruncateKeywordSegment", // [1662]
    "WITHOUT", // [1663]
    "WithoutKeywordSegment", // [1664]
    "CountKeywordSegment", // [1665]
    "LISTEN", // [1666]
    "RownumKeywordSegment", // [1667]
    "ROWNUM", // [1668]
    "ArrayLiteralSegment", // [1669]
    "array_literal", // [1670]
    "TypedArrayLiteralSegment", // [1671]
    "typed_array_literal", // [1672]
    "ObjectLiteralSegment", // [1673]
    "{", // [1674]
    "object_literal", // [1675]
    "LN", // [1676]
    "LOAD", // [1677]
    "LOCALTIME", // [1678]
    "LOCATION", // [1679]
    "LOCATOR", // [1680]
    "LOCKING", // [1681]
    "LOCKS", // [1682]
    "LOGIN", // [1683]
    "LOGS", // [1684]
    "LONG", // [1685]
    "LONGBLOB", // [1686]
    "LONGTEXT", // [1687]
    "LOOP", // [1688]
    "LoopKeywordSegment", // [1689]
    "LOW", // [1690]
    "LOW_PRIORITY", // [1691]
    "LOWER", // [1692]
    "M", // [1693]
    "MlKeywordSegment", // [1694]
    "MAP", // [1695]
    "MATCH", // [1696]
    "MATCHED", // [1697]
    "MAX", // [1698]
    "MAX_ROWS", // [1699]
    "MAXEXTENTS", // [1700]
    "MAXSIZE", // [1701]
    "MEDIUM", // [1702]
    "MEDIUMBLOB", // [1703]
    "MEDIUMINT", // [1704]
    "MEDIUMTEXT", // [1705]
    "MEMCOMPRESS", // [1706]
    "MergeMatchedClauseSegment", // [1707]
    "merge_when_matched_clause", // [1708]
    "MergeNotMatchedClauseSegment", // [1709]
    "merge_when_not_matched_clause", // [1710]
    "MatchedKeywordSegment", // [1711]
    "MergeUpdateClauseSegment", // [1712]
    "merge_update_clause", // [1713]
    "MergeDeleteClauseSegment", // [1714]
    "merge_delete_clause", // [1715]
    "MergeInsertClauseSegment", // [1716]
    "merge_insert_clause", // [1717]
    "MergeIntoLiteralGrammar", // [1718]
    "AliasedTableReferenceGrammar", // [1719]
    "MergeMatchSegment", // [1720]
    "merge_match", // [1721]
    "SetClauseListSegment", // [1722]
    "set_clause_list", // [1723]
    "MESSAGE_LENGTH", // [1724]
    "MESSAGE_OCTET_LENGTH", // [1725]
    "MESSAGE_TEXT", // [1726]
    "MIDDLEINT", // [1727]
    "MIN", // [1728]
    "MIN_ROWS", // [1729]
    "MINEXTENTS", // [1730]
    "MINUTE_MICROSECOND", // [1731]
    "MINUTE_SECOND", // [1732]
    "MLSLABEL", // [1733]
    "MODE", // [1734]
    "MODIFIES", // [1735]
    "MODULE", // [1736]
    "MONITOR", // [1737]
    "MONTHNAME", // [1738]
    "MORE", // [1739]
    "MOVE", // [1740]
    "MOVEMENT", // [1741]
    "MultisetKeywordSegment", // [1742]
    "ExceptKeywordSegment", // [1743]
    "IntersectKeywordSegment", // [1744]
    "UnionKeywordSegment", // [1745]
    "MUMPS", // [1746]
    "MYISAM", // [1747]
    "naked_identifier", // [1748]
    "WindowSpecificationSegment", // [1749]
    "window_specification", // [1750]
    "NamedWindowExpressionSegment", // [1751]
    "named_window_expression", // [1752]
    "NAMES", // [1753]
    "null_literal", // [1754]
    "NATIONAL", // [1755]
    "NATURAL", // [1756]
    "NaturalKeywordSegment", // [1757]
    "NCHAR", // [1758]
    "NCLOB", // [1759]
    "sign_indicator", // [1760]
    "NESTED_TABLE_ID", // [1761]
    "NESTING", // [1762]
    "NEW", // [1763]
    "newline", // [1764]
    "NewlineSegment", // [1765]
    "NEXTVAL", // [1766]
    "NO_WRITE_TO_BINLOG", // [1767]
    "NOAUDIT", // [1768]
    "NOCHECK", // [1769]
    "NOCREATEDB", // [1770]
    "NOCREATEROLE", // [1771]
    "NOCREATEUSER", // [1772]
    "NOINHERIT", // [1773]
    "NOLOGIN", // [1774]
    "UnorderedSelectStatementSegment", // [1775]
    "WithCompoundStatementSegment", // [1776]
    "with_compound_statement", // [1777]
    "NonSetSelectableGrammar", // [1778]
    "BracketedSetExpressionGrammar", // [1779]
    "MergeStatementSegment", // [1780]
    "merge_statement", // [1781]
    "SetExpressionSegment", // [1782]
    "NONCLUSTERED", // [1783]
    "NONE", // [1784]
    "NORMALIZE", // [1785]
    "NORMALIZED", // [1786]
    "NOSUPERUSER", // [1787]
    "NOTIFY", // [1788]
    "NOTNULL", // [1789]
    "NULLABLE", // [1790]
    "NULLIF", // [1791]
    "NUMBER", // [1792]
    "NUMERIC", // [1793]
    "ObjectLiteralElementSegment", // [1794]
    "object_literal_element", // [1795]
    "start_curly_bracket", // [1796]
    "CastOperatorSegment", // [1797]
    "StartSquareBracketSegment", // [1798]
    "StartBracketSegment", // [1799]
    "PERSISTABLE", // [1800]
    "PersistableKeywordSegment", // [1801]
    "OBJECTS", // [1802]
    "OCTET_LENGTH", // [1803]
    "OCTETS", // [1804]
    "OFF", // [1805]
    "OFFLINE", // [1806]
    "OFFSETS", // [1807]
    "OIDS", // [1808]
    "OLD", // [1809]
    "OLTP", // [1810]
    "PRESERVE", // [1811]
    "PreserveKeywordSegment", // [1812]
    "DefinitionKeywordSegment", // [1813]
    "StatementAndDelimiterGrammar", // [1814]
    "OPEN", // [1815]
    "OpenKeywordSegment", // [1816]
    "OPENDATASOURCE", // [1817]
    "OPENQUERY", // [1818]
    "OPENROWSET", // [1819]
    "OPENXML", // [1820]
    "OPERATE", // [1821]
    "OPERATION", // [1822]
    "OPTIMAL", // [1823]
    "OPTIMIZE", // [1824]
    "OPTIONALLY", // [1825]
    "PctfreeKeywordSegment", // [1826]
    "InitransKeywordSegment", // [1827]
    "MaxtransKeywordSegment", // [1828]
    "StorageClauseSegment", // [1829]
    "storage_clause", // [1830]
    "NoparallelKeywordSegment", // [1831]
    "NocompressKeywordSegment", // [1832]
    "CompressKeywordSegment", // [1833]
    "NosortKeywordSegment", // [1834]
    "CreationKeywordSegment", // [1835]
    "PctusedKeywordSegment", // [1836]
    "OltpKeywordSegment", // [1837]
    "LowKeywordSegment", // [1838]
    "HighKeywordSegment", // [1839]
    "StoreKeywordSegment", // [1840]
    "STORE", // [1841]
    "BasicKeywordSegment", // [1842]
    "AdvancedKeywordSegment", // [1843]
    "LockingKeywordSegment", // [1844]
    "MovementKeywordSegment", // [1845]
    "Result_cacheKeywordSegment", // [1846]
    "ModeKeywordSegment", // [1847]
    "InmemoryKeywordSegment", // [1848]
    "PRIORITY", // [1849]
    "MemcompressKeywordSegment", // [1850]
    "CapacityKeywordSegment", // [1851]
    "PriorityKeywordSegment", // [1852]
    "NoneKeywordSegment", // [1853]
    "MediumKeywordSegment", // [1854]
    "CriticalKeywordSegment", // [1855]
    "DistributeKeywordSegment", // [1856]
    "AutoKeywordSegment", // [1857]
    "PARTITION", // [1858]
    "ROWID", // [1859]
    "SUBPARTITION", // [1860]
    "RowidKeywordSegment", // [1861]
    "PartitionKeywordSegment", // [1862]
    "SubpartitionKeywordSegment", // [1863]
    "DuplicateKeywordSegment", // [1864]
    "RowdependenciesKeywordSegment", // [1865]
    "NorowdependenciesKeywordSegment", // [1866]
    "SiblingsKeywordSegment", // [1867]
    "SIBLINGS", // [1868]
    "WithFillSegment", // [1869]
    "with_fill", // [1870]
    "ORDERING", // [1871]
    "OUTFILE", // [1872]
    "OUTPUT", // [1873]
    "OverKeywordSegment", // [1874]
    "OVER", // [1875]
    "OverlapsKeywordSegment", // [1876]
    "OVERLAY", // [1877]
    "OVERWRITE", // [1878]
    "OWNER", // [1879]
    "OWNERSHIP", // [1880]
    "PACK_KEYS", // [1881]
    "PAD", // [1882]
    "Parallel_enableKeywordSegment", // [1883]
    "HashKeywordSegment", // [1884]
    "PARAMETER", // [1885]
    "parameter", // [1886]
    "?", // [1887]
    "PARAMETER_MODE", // [1888]
    "PARAMETER_NAME", // [1889]
    "PARAMETER_ORDINAL_POSITION", // [1890]
    "PARAMETER_SPECIFIC_CATALOG", // [1891]
    "PARAMETER_SPECIFIC_NAME", // [1892]
    "PARAMETER_SPECIFIC_SCHEMA", // [1893]
    "PARENT", // [1894]
    "PARTIAL", // [1895]
    "PASCAL", // [1896]
    "path_segment", // [1897]
    "PCTINCREASE", // [1898]
    "PERCENT", // [1899]
    "PERCENT_RANK", // [1900]
    "PERCENTILE_CONT", // [1901]
    "PERCENTILE_DISC", // [1902]
    "PIPE", // [1903]
    "pipe", // [1904]
    "PipelinedKeywordSegment", // [1905]
    "PolymorphicKeywordSegment", // [1906]
    "POLYMORPHIC", // [1907]
    "PivotKeywordSegment", // [1908]
    "XmlKeywordSegment", // [1909]
    "XML", // [1910]
    "PivotForGrammar", // [1911]
    "PivotInGrammar", // [1912]
    "PLACING", // [1913]
    "PLAN", // [1914]
    "PLI", // [1915]
    "TransactionStatementSegment", // [1916]
    "transaction_statement", // [1917]
    "DropTableStatementSegment", // [1918]
    "drop_table_statement", // [1919]
    "DropViewStatementSegment", // [1920]
    "drop_view_statement", // [1921]
    "CreateUserStatementSegment", // [1922]
    "create_user_statement", // [1923]
    "DropUserStatementSegment", // [1924]
    "drop_user_statement", // [1925]
    "TruncateStatementSegment", // [1926]
    "truncate_table", // [1927]
    "AccessStatementSegment", // [1928]
    "access_statement", // [1929]
    "CreateTableStatementSegment", // [1930]
    "create_table_statement", // [1931]
    "CreateRoleStatementSegment", // [1932]
    "create_role_statement", // [1933]
    "DropRoleStatementSegment", // [1934]
    "drop_role_statement", // [1935]
    "AlterTableStatementSegment", // [1936]
    "alter_table_statement", // [1937]
    "CreateSchemaStatementSegment", // [1938]
    "create_schema_statement", // [1939]
    "SetSchemaStatementSegment", // [1940]
    "set_schema_statement", // [1941]
    "DropSchemaStatementSegment", // [1942]
    "drop_schema_statement", // [1943]
    "DropTypeStatementSegment", // [1944]
    "drop_type_statement", // [1945]
    "CreateDatabaseStatementSegment", // [1946]
    "create_database_statement", // [1947]
    "DropDatabaseStatementSegment", // [1948]
    "drop_database_statement", // [1949]
    "CreateIndexStatementSegment", // [1950]
    "create_index_statement", // [1951]
    "DropIndexStatementSegment", // [1952]
    "drop_index_statement", // [1953]
    "CreateViewStatementSegment", // [1954]
    "create_view_statement", // [1955]
    "CreateCastStatementSegment", // [1956]
    "create_cast_statement", // [1957]
    "DropCastStatementSegment", // [1958]
    "drop_cast_statement", // [1959]
    "DropFunctionStatementSegment", // [1960]
    "drop_function_statement", // [1961]
    "CreateModelStatementSegment", // [1962]
    "create_model_statement", // [1963]
    "DropModelStatementSegment", // [1964]
    "drop_MODELstatement", // [1965]
    "DescribeStatementSegment", // [1966]
    "describe_statement", // [1967]
    "UseStatementSegment", // [1968]
    "use_statement", // [1969]
    "ExplainStatementSegment", // [1970]
    "explain_statement", // [1971]
    "CreateSequenceStatementSegment", // [1972]
    "create_sequence_statement", // [1973]
    "AlterSequenceStatementSegment", // [1974]
    "alter_sequence_statement", // [1975]
    "DropSequenceStatementSegment", // [1976]
    "drop_sequence_statement", // [1977]
    "CreateTriggerStatementSegment", // [1978]
    "create_trigger_statement", // [1979]
    "create_trigger", // [1980]
    "DropTriggerStatementSegment", // [1981]
    "drop_trigger", // [1982]
    "CommentStatementSegment", // [1983]
    "comment_statement", // [1984]
    "DropProcedureStatementSegment", // [1985]
    "drop_procedure_statement", // [1986]
    "AlterFunctionStatementSegment", // [1987]
    "alter_function_statement", // [1988]
    "CreateTypeStatementSegment", // [1989]
    "create_type_statement", // [1990]
    "CreateTypeBodyStatementSegment", // [1991]
    "create_type_body_statement", // [1992]
    "CreatePackageBodyStatementSegment", // [1993]
    "create_package_body_statement", // [1994]
    "CreatePackageStatementSegment", // [1995]
    "create_package_statement", // [1996]
    "AlterSessionStatementSegment", // [1997]
    "alter_session_statement", // [1998]
    "DropPackageStatementSegment", // [1999]
    "drop_package_statement", // [2000]
    "AlterPackageStatementSegment", // [2001]
    "alter_package_statement", // [2002]
    "AlterTriggerStatementSegment", // [2003]
    "alter_trigger_statement", // [2004]
    "AssignmentStatementSegment", // [2005]
    "assignment_segment_statement", // [2006]
    "ExecuteImmediateSegment", // [2007]
    "execute_immediate_statement", // [2008]
    "IfExpressionStatement", // [2009]
    "if_then_statement", // [2010]
    "NullStatementSegment", // [2011]
    "null_statement", // [2012]
    "ForLoopStatementSegment", // [2013]
    "for_loop_statement", // [2014]
    "WhileLoopStatementSegment", // [2015]
    "while_loop_statement", // [2016]
    "ForAllStatementSegment", // [2017]
    "forall_statement", // [2018]
    "OpenStatementSegment", // [2019]
    "open_statement", // [2020]
    "CloseStatementSegment", // [2021]
    "close_statement", // [2022]
    "OpenForStatementSegment", // [2023]
    "open_for_statement", // [2024]
    "FetchStatementSegment", // [2025]
    "fetch_statement", // [2026]
    "ExitStatementSegment", // [2027]
    "exit_statement", // [2028]
    "ContinueStatementSegment", // [2029]
    "continue_statement", // [2030]
    "RaiseStatementSegment", // [2031]
    "RAISE", // [2032]
    "raise_statement", // [2033]
    "ReturnStatementSegment", // [2034]
    "return_statement", // [2035]
    "AlterIndexStatementSegment", // [2036]
    "alter_index_statement", // [2037]
    "CreateDatabaseLinkStatementSegment", // [2038]
    "create_database_link_statement", // [2039]
    "DropDatabaseLinkStatementSegment", // [2040]
    "drop_database_link_statement", // [2041]
    "AlterDatabaseLinkStatementSegment", // [2042]
    "alter_database_link_statement", // [2043]
    "CreateSynonymStatementSegment", // [2044]
    "create_synonym_statement", // [2045]
    "DropSynonymStatementSegment", // [2046]
    "drop_synonym_statement", // [2047]
    "AlterSynonymStatementSegment", // [2048]
    "alter_synonym_statement", // [2049]
    "DropProfileStatementSegment", // [2050]
    "drop_profile_statement", // [2051]
    "DropClusterStatementSegment", // [2052]
    "drop_cluster_statement", // [2053]
    "ProcedureCallStatementSegment", // [2054]
    "procedure_call_statement", // [2055]
    "PlusJoinSegment", // [2056]
    "plus_join_symbol", // [2057]
    "EndBracketSegment", // [2058]
    "WithinGroupClauseSegment", // [2059]
    "WITHIN", // [2060]
    "withingroup_clause", // [2061]
    "KeepClauseSegment", // [2062]
    "keep_clause", // [2063]
    "FilterClauseGrammar", // [2064]
    "OverClauseSegment", // [2065]
    "over_clause", // [2066]
    "POSTFIX", // [2067]
    "POWER", // [2068]
    "LateralKeywordSegment", // [2069]
    "PREFIX", // [2070]
    "PREORDER", // [2071]
    "PREPARE", // [2072]
    "PREPARED", // [2073]
    "PRINT", // [2074]
    "PROC", // [2075]
    "PROCEDURAL", // [2076]
    "PROCESSLIST", // [2077]
    "PROMPT", // [2078]
    "SOME", // [2079]
    "SomeKeywordSegment", // [2080]
    "QUOTE", // [2081]
    "quoted_identifier", // [2082]
    "quoted_literal", // [2083]
    "RAID0", // [2084]
    "RaiseKeywordSegment", // [2085]
    "RAISERROR", // [2086]
    "RANK", // [2087]
    "raw_comparison_operator", // [2088]
    "RAW", // [2089]
    "READS", // [2090]
    "READTEXT", // [2091]
    "REAL", // [2092]
    "RECHECK", // [2093]
    "RECONFIGURE", // [2094]
    "RECORD", // [2095]
    "RecordKeywordSegment", // [2096]
    "RECURSIVE", // [2097]
    "RECYCLE", // [2098]
    "RefKeywordSegment", // [2099]
    "REF", // [2100]
    "ReferenceMatchGrammar", // [2101]
    "ReferentialActionGrammar", // [2102]
    "RESTRICT", // [2103]
    "MatchKeywordSegment", // [2104]
    "SIMPLE", // [2105]
    "PartialKeywordSegment", // [2106]
    "SimpleKeywordSegment", // [2107]
    "REFERENCE_USAGE", // [2108]
    "ReferencingKeywordSegment", // [2109]
    "TriggerCorrelationNameSegment", // [2110]
    "trigger_correlation_name", // [2111]
    "RestrictKeywordSegment", // [2112]
    "ActionKeywordSegment", // [2113]
    "REGEXP", // [2114]
    "REGR_AVGX", // [2115]
    "REGR_AVGY", // [2116]
    "REGR_COUNT", // [2117]
    "REGR_INTERCEPT", // [2118]
    "REGR_R2", // [2119]
    "REGR_SLOPE", // [2120]
    "REGR_SXX", // [2121]
    "REGR_SXY", // [2122]
    "REGR_SYY", // [2123]
    "REINDEX", // [2124]
    "RELATIVE", // [2125]
    "RELEASE", // [2126]
    "RELIES_ON", // [2127]
    "RELOAD", // [2128]
    "REPEATABLE", // [2129]
    "REPLICATION", // [2130]
    "REQUIRE", // [2131]
    "RESET", // [2132]
    "RESIGNAL", // [2133]
    "RESTART", // [2134]
    "RESTORE", // [2135]
    "Relies_onKeywordSegment", // [2136]
    "RESULT", // [2137]
    "RETURNED_CARDINALITY", // [2138]
    "RETURNED_LENGTH", // [2139]
    "RETURNED_OCTET_LENGTH", // [2140]
    "RETURNED_SQLSTATE", // [2141]
    "OldKeywordSegment", // [2142]
    "NewKeywordSegment", // [2143]
    "RETURNS", // [2144]
    "RevokeKeywordSegment", // [2145]
    "right_arrow", // [2146]
    "ROLES", // [2147]
    "ROUTINE_CATALOG", // [2148]
    "ROUTINE_NAME", // [2149]
    "ROUTINE_SCHEMA", // [2150]
    "RowtypeKeywordSegment", // [2151]
    "ROWTYPE", // [2152]
    "ROW_COUNT", // [2153]
    "ROW_NUMBER", // [2154]
    "ROWGUIDCOL", // [2155]
    "RULE", // [2156]
    "TablesampleKeywordSegment", // [2157]
    "BernoulliKeywordSegment", // [2158]
    "RepeatableKeywordSegment", // [2159]
    "SAVEPOINT", // [2160]
    "SCALE", // [2161]
    "SCHEMA_NAME", // [2162]
    "SCOPE", // [2163]
    "SCOPE_CATALOG", // [2164]
    "SCOPE_NAME", // [2165]
    "SCOPE_SCHEMA", // [2166]
    "SCROLL", // [2167]
    "SEARCH", // [2168]
    "SECOND_MICROSECOND", // [2169]
    "SECTION", // [2170]
    "WildcardExpressionSegment", // [2171]
    "wildcard_expression", // [2172]
    "SelectClauseModifierSegment", // [2173]
    "select_clause_modifier", // [2174]
    "SelectClauseElementSegment", // [2175]
    "select_clause_element", // [2176]
    "SelectClauseTerminatorGrammar", // [2177]
    "SelectClauseSegment", // [2178]
    "select_clause", // [2179]
    "HierarchicalQueryClauseSegment", // [2180]
    "hierarchical_query_clause", // [2181]
    "GroupByClauseSegment", // [2182]
    "groupby_clause", // [2183]
    "HavingClauseSegment", // [2184]
    "having_clause", // [2185]
    "NamedWindowSegment", // [2186]
    "named_window", // [2187]
    "ForUpdateGrammar", // [2188]
    "FetchClauseSegment", // [2189]
    "fetch_clause", // [2190]
    "WithCompoundNonSelectStatementSegment", // [2191]
    "NonWithSelectableGrammar", // [2192]
    "SELF", // [2193]
    "statement_terminator", // [2194]
    "SENSITIVE", // [2195]
    "NextvalKeywordSegment", // [2196]
    "CurrvalKeywordSegment", // [2197]
    "SERVER", // [2198]
    "SERVER_NAME", // [2199]
    "UnionGrammar", // [2200]
    "MinusKeywordSegment", // [2201]
    "SETOF", // [2202]
    "SETUSER", // [2203]
    "SHARE", // [2204]
    "SHARES", // [2205]
    "SharingKeywordSegment", // [2206]
    "MetadataKeywordSegment", // [2207]
    "SHOW", // [2208]
    "SHUTDOWN", // [2209]
    "SIGNAL", // [2210]
    "PositiveSegment", // [2211]
    "NegativeSegment", // [2212]
    "SIMILAR", // [2213]
    "size_prefix", // [2214]
    "SIZE", // [2215]
    "slash", // [2216]
    "slice", // [2217]
    "SMALLINT", // [2218]
    "SONAME", // [2219]
    "SPACE", // [2220]
    "SPATIAL", // [2221]
    "SPECIFIC_NAME", // [2222]
    "SPECIFICTYPE", // [2223]
    "SQL_BIG_RESULT", // [2224]
    "SQL_BIG_SELECTS", // [2225]
    "SQL_BIG_TABLES", // [2226]
    "SQL_CALC_FOUND_ROWS", // [2227]
    "SQL_LOG_OFF", // [2228]
    "SQL_LOG_UPDATE", // [2229]
    "SQL_LOW_PRIORITY_UPDATES", // [2230]
    "SQL_SELECT_LIMIT", // [2231]
    "SQL_SMALL_RESULT", // [2232]
    "SQL_WARNINGS", // [2233]
    "SQLCA", // [2234]
    "SQLCODE", // [2235]
    "SQLERROR", // [2236]
    "SQLEXCEPTION", // [2237]
    "SCAN", // [2238]
    "OffKeywordSegment", // [2239]
    "SQLSTATE", // [2240]
    "SQLWARNING", // [2241]
    "SQRT", // [2242]
    "SSL", // [2243]
    "STABLE", // [2244]
    "star", // [2245]
    "STARTING", // [2246]
    "STARTS", // [2247]
    "STATE", // [2248]
    "PlsqlStatementSegment", // [2249]
    "STATEMENT", // [2250]
    "STATISTICS", // [2251]
    "STDDEV_POP", // [2252]
    "STDDEV_SAMP", // [2253]
    "STDIN", // [2254]
    "STDOUT", // [2255]
    "StorageKeywordSegment", // [2256]
    "InitialKeywordSegment", // [2257]
    "MinextentsKeywordSegment", // [2258]
    "MaxextentsKeywordSegment", // [2259]
    "PctincreaseKeywordSegment", // [2260]
    "FreelistsKeywordSegment", // [2261]
    "FreelistKeywordSegment", // [2262]
    "GroupsKeywordSegment", // [2263]
    "Buffer_poolKeywordSegment", // [2264]
    "RecycleKeywordSegment", // [2265]
    "Flash_cacheKeywordSegment", // [2266]
    "Cell_flash_cacheKeywordSegment", // [2267]
    "MaxsizeKeywordSegment", // [2268]
    "OptimalKeywordSegment", // [2269]
    "STRAIGHT_JOIN", // [2270]
    "ConcatSegment", // [2271]
    "STRING", // [2272]
    "STRUCTURE", // [2273]
    "STYLE", // [2274]
    "SUBCLASS_ORIGIN", // [2275]
    "SUBLIST", // [2276]
    "SUBMULTISET", // [2277]
    "SUBSTRING", // [2278]
    "SUCCESSFUL", // [2279]
    "SUM", // [2280]
    "SUPERUSER", // [2281]
    "SYMMETRIC", // [2282]
    "SYSID", // [2283]
    "SYSTEM_USER", // [2284]
    "UsingIndexClauseSegment", // [2285]
    "using_index_clause", // [2286]
    "ForeignKeyGrammar", // [2287]
    "TABLE_NAME", // [2288]
    "Expression_A_Unary_Operator_Grammar", // [2289]
    "Expression_C_Grammar", // [2290]
    "Expression_B_Unary_Operator_Grammar", // [2291]
    "TEMP", // [2292]
    "TEMPLATE", // [2293]
    "PrivateKeywordSegment", // [2294]
    "TRANSIENT", // [2295]
    "TransientKeywordSegment", // [2296]
    "TERMINATE", // [2297]
    "TERMINATED", // [2298]
    "TEXT", // [2299]
    "TEXTSIZE", // [2300]
    "THAN", // [2301]
    "tilde", // [2302]
    "ZoneKeywordSegment", // [2303]
    "ZONE", // [2304]
    "AtKeywordSegment", // [2305]
    "TIMEZONE_HOUR", // [2306]
    "TIMEZONE_MINUTE", // [2307]
    "StatementKeywordSegment", // [2308]
    "TimingPointGrammar", // [2309]
    "TINYBLOB", // [2310]
    "TINYINT", // [2311]
    "TINYTEXT", // [2312]
    "TOAST", // [2313]
    "TOP", // [2314]
    "TOP_LEVEL_COUNT", // [2315]
    "TRAN", // [2316]
    "WORK", // [2317]
    "WorkKeywordSegment", // [2318]
    "ChainKeywordSegment", // [2319]
    "TRANSACTION_ACTIVE", // [2320]
    "TRANSACTIONS", // [2321]
    "TRANSACTIONS_COMMITTED", // [2322]
    "TRANSACTIONS_ROLLED_BACK", // [2323]
    "TRANSFORM", // [2324]
    "TRANSFORMS", // [2325]
    "TREAT", // [2326]
    "ParentKeywordSegment", // [2327]
    "TriggerPredicatesGrammar", // [2328]
    "TriggerPredicatesConjunctionGrammar", // [2329]
    "InsertingKeywordSegment", // [2330]
    "UpdatingKeywordSegment", // [2331]
    "DeletingKeywordSegment", // [2332]
    "TRIGGER_CATALOG", // [2333]
    "TRIGGER_NAME", // [2334]
    "TRIGGER_SCHEMA", // [2335]
    "TRIM", // [2336]
    "BothKeywordSegment", // [2337]
    "LeadingKeywordSegment", // [2338]
    "TrailingKeywordSegment", // [2339]
    "TRUSTED", // [2340]
    "TSEQUAL", // [2341]
    "StructLiteralSegment", // [2342]
    "UESCAPE", // [2343]
    "UNCOMMITTED", // [2344]
    "CrossKeywordSegment", // [2345]
    "NaturalJoinKeywordsGrammar", // [2346]
    "UnconditionalCrossJoinKeywordsGrammar", // [2347]
    "HorizontalJoinKeywordsGrammar", // [2348]
    "UNDO", // [2349]
    "UNENCRYPTED", // [2350]
    "UNKNOWN", // [2351]
    "UNLISTEN", // [2352]
    "UNNAMED", // [2353]
    "UNNEST", // [2354]
    "UnpivotAsCharacterLiteralGrammar", // [2355]
    "UnpivotAsNumericLiteralGrammar", // [2356]
    "IncludeKeywordSegment", // [2357]
    "ExcludeKeywordSegment", // [2358]
    "UnpivotKeywordSegment", // [2359]
    "UnpivotNullsGrammar", // [2360]
    "UnpivotInGrammar", // [2361]
    "UNTIL", // [2362]
    "UPDATETEXT", // [2363]
    "UPPER", // [2364]
    "USE_ANY_ROLE", // [2365]
    "USER_DEFINED_TYPE_CATALOG", // [2366]
    "USER_DEFINED_TYPE_CODE", // [2367]
    "USER_DEFINED_TYPE_NAME", // [2368]
    "USER_DEFINED_TYPE_SCHEMA", // [2369]
    "USERS", // [2370]
    "UTC_DATE", // [2371]
    "UTC_TIME", // [2372]
    "UTC_TIMESTAMP", // [2373]
    "VACUUM", // [2374]
    "VALID", // [2375]
    "VALIDATOR", // [2376]
    "VAR_POP", // [2377]
    "VAR_SAMP", // [2378]
    "VARBINARY", // [2379]
    "VARCHAR2", // [2380]
    "VARCHARACTER", // [2381]
    "VARIABLE", // [2382]
    "VARIABLES", // [2383]
    "VarrayKeywordSegment", // [2384]
    "ArrayKeywordSegment", // [2385]
    "VERBOSE", // [2386]
    "VERSION", // [2387]
    "VOLATILE", // [2388]
    "WAITFOR", // [2389]
    "WAREHOUSE", // [2390]
    "WAREHOUSES", // [2391]
    "indented_then", // [2392]
    "indented_then_contents", // [2393]
    "WHENEVER", // [2394]
    "whitespace", // [2395]
    "WhitespaceSegment", // [2396]
    "WIDTH_BUCKET", // [2397]
    "PartitionClauseSegment", // [2398]
    "partitionby_clause", // [2399]
    "FrameClauseSegment", // [2400]
    "frame_clause", // [2401]
    "RecursiveKeywordSegment", // [2402]
    "indented_ctes", // [2403]
    "CTEDefinitionSegment", // [2404]
    "common_table_expression", // [2405]
    "NonWithNonSelectableGrammar", // [2406]
    "DataKeywordSegment", // [2407]
    "BindingKeywordSegment", // [2408]
    "WithinKeywordSegment", // [2409]
    "WRAPPER", // [2410]
    "WRITETEXT", // [2411]
    "X509", // [2412]
    "EvalnameKeywordSegment", // [2413]
    "XOR", // [2414]
    "YAML", // [2415]
    "YEAR_MONTH", // [2416]
    "ZEROFILL", // [2417]
];

pub static AUX_DATA: &[u32] = &[
    0, 1, 2, 1, 1, 4, 3, 1, 4, 5, 6, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 7, 1, 2, 1, 1, 4, 3, 1, 4, 5, 8, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 9, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 0, 0, 0, 0, 0, 0, 1, 0, 268, 273, 274, 1, 273,
    3, 3, 4, 275, 276, 277, 274, 1, 277, 3, 3, 4, 275, 1, 2, 1,
    0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1, 2,
    1, 1, 0, 332, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295,
    0, 339, 1, 2, 1, 1, 4, 3, 1, 4, 5, 340, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 341, 1, 2, 1, 1, 4, 3, 1, 4, 5, 342,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 343, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 344, 1, 2, 1, 1, 4, 3, 1, 4, 5, 345, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 159, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 346, 1, 2, 1, 1, 4, 3, 1, 4, 5, 347, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 160, 1, 2, 1, 1, 4, 3, 1, 4, 5, 348,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 349, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 2, 1, 0, 366, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 161, 1, 2, 1, 1, 4, 3, 1, 4, 5, 371, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 372, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 0, 0, 0, 1, 2, 1, 0, 162, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 1, 4294967295, 4294967295, 0, 0, 0, 0, 1, 0,
    1, 2, 1, 0, 1, 0, 1, 2, 1, 0, 0, 1, 0, 1, 2, 1,
    0, 1, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    1, 0, 0, 0, 542, 1, 2, 1, 1, 4, 3, 1, 4, 5, 543, 544,
    274, 1, 544, 3, 3, 4, 275, 545, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 10, 1, 2, 1, 1, 4, 3, 1, 4, 5, 163, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 546, 1, 2, 1, 1, 4, 3, 1, 4, 5, 546,
    547, 548, 1, 547, 3, 3, 547, 4, 259, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 549, 1, 2, 1, 1, 4, 3, 1, 4, 5, 502, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 223, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    550, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 336, 576,
    274, 1, 576, 3, 3, 4, 275, 577, 578, 274, 1, 578, 3, 3, 4, 275,
    1, 2, 0, 579, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1,
    2, 0, 357, 1, 2, 1, 1, 4, 3, 1, 4, 5, 581, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 582, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    583, 1, 2, 1, 1, 4, 3, 1, 4, 5, 584, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 585, 586, 274, 1, 586, 3, 3, 4, 275, 0, 1, 2,
    1, 0, 0, 4294967295, 4294967295, 0, 0, 591, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 592, 1, 2, 1, 1, 4, 3, 1, 4, 5, 593, 594, 274, 1, 594,
    3, 3, 4, 275, 595, 1, 2, 1, 1, 4, 3, 1, 4, 5, 551, 596,
    274, 1, 596, 3, 3, 4, 275, 11, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 597, 1, 2, 1, 1, 4, 3, 1, 4, 5, 102, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 379, 1, 2, 1, 1, 4, 3, 1, 4, 5, 598,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 599, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 602, 1, 2, 1, 1, 4, 3, 1, 4, 5, 600, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 603, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 604, 1, 2, 1, 1, 4, 3, 1, 4, 5, 164, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 605, 1, 2, 1, 1, 4, 3, 1, 4, 5, 606,
    607, 608, 497, 609, 610, 611, 612, 613, 614, 615, 67, 767, 12, 616, 617, 1,
    616, 2, 3, 4, 0, 630, 631, 0, 2, 3, 630, 3, 632, 0, 1, 3,
    633, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 4294967295, 4294967295, 0, 1,
    0, 0, 165, 1, 2, 1, 1, 4, 3, 1, 4, 5, 643, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 647, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    653, 1, 2, 1, 1, 4, 3, 1, 4, 5, 654, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 655, 1, 2, 1, 1, 4, 3, 1, 4, 5, 656, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 547, 548, 9, 3, 3, 547, 4,
    575, 667, 274, 1, 667, 3, 3, 4, 275, 3, 4, 1, 0, 669, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 670, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 671, 1, 2, 1, 1, 4, 3, 1, 4, 5, 672, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 673, 1, 2, 1, 1, 4, 3, 1, 4, 5, 560,
    547, 274, 1, 547, 3, 3, 4, 275, 678, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 679, 1, 2, 1, 1, 4, 3, 1, 4, 5, 680, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 683, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 688, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2,
    1, 1, 0, 1, 2, 1, 689, 690, 0, 2, 3, 689, 1, 2, 1, 695,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 696, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 697, 1, 2, 1, 1, 4, 3, 1, 4, 5, 698, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 699, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 707, 1, 2, 1, 1, 4, 3, 1, 4, 5, 708, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 335, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 709, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1,
    2, 1, 427, 1, 2, 1, 1, 4, 3, 1, 4, 5, 713, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 714, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    715, 1, 2, 1, 1, 4, 3, 1, 4, 5, 716, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 525, 1, 2, 1, 1, 4, 3, 1, 4, 5, 717, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0,
    0, 0, 4294967295, 4294967295, 0, 0, 718, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    734, 1, 2, 1, 1, 4, 3, 1, 4, 5, 735, 736, 274, 1, 736, 3,
    3, 4, 275, 737, 1, 2, 1, 1, 4, 3, 1, 4, 5, 738, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 739, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 740, 1, 2, 1, 1, 4, 3, 1, 4, 5, 741, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 742, 1, 2, 1, 1, 4, 3, 1, 4, 5, 166,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 743, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 744, 1, 2, 1, 1, 4, 3, 1, 4, 5, 745, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 746, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 747, 1, 2, 1, 1, 4, 3, 1, 4, 5, 748, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 749, 1, 2, 1, 1, 4, 3, 1, 4, 5, 750,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 751, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 518, 1, 2, 1, 1, 4, 3, 1, 4, 5, 752, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 753, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 754, 1, 2, 1, 1, 4, 3, 1, 4, 5, 12, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 755, 1, 2, 1, 1, 4, 3, 1, 4, 5, 756,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 451, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 13, 1, 2, 1, 1, 4, 3, 1, 4, 5, 757, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 758, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 759, 1, 2, 1, 1, 4, 3, 1, 4, 5, 4, 617, 9, 2,
    3, 4, 760, 1, 2, 1, 1, 4, 3, 1, 4, 5, 761, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 0, 0, 767, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 768, 1, 2, 1, 1, 4, 3, 1, 4, 5, 769, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 702, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 575, 780, 274, 1, 780, 3, 3, 4, 275, 575, 781, 274, 1, 781,
    3, 3, 4, 275, 575, 782, 274, 1, 782, 3, 3, 4, 275, 0, 0, 1,
    2, 1, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 2, 1, 0, 0, 4294967295, 4294967295,
    0, 0, 516, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 806, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 606, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 808, 1, 2, 1, 1, 4, 3, 1, 4, 5, 272, 809, 274, 1,
    809, 3, 3, 4, 275, 810, 1, 2, 1, 1, 4, 3, 1, 4, 5, 811,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 167, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 812, 813, 10, 3, 3, 812, 4, 0, 0, 235, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 484, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 480, 823, 9, 3, 3, 480, 4, 0, 0, 1, 0, 387, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 830, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    547, 831, 0, 2, 3, 547, 480, 832, 0, 2, 3, 480, 833, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 838, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 839, 1, 2, 1, 1, 4, 3, 1, 4, 5, 840, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 841, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 14, 1, 2, 1, 1, 4, 3, 1, 4, 5, 627, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 850, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 851, 1, 2, 1, 1, 4, 3, 1, 4, 5, 852, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 485, 1, 2, 1, 1, 4, 3, 1, 4, 5, 853,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 854, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 855, 1, 2, 1, 1, 4, 3, 1, 4, 5, 486, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 856, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 15, 1, 2, 1, 1, 4, 3, 1, 4, 5, 857, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 858, 1, 2, 1, 1, 4, 3, 1, 4, 5, 16,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 859, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 861, 1, 2, 1, 1, 4, 3, 1, 4, 5, 862, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 863, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 864, 1, 2, 1, 1, 4, 3, 1, 4, 5, 865, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 866, 1, 2, 1, 1, 4, 3, 1, 4, 5, 867,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 868, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 3, 4, 1, 0, 0, 0, 0, 0, 0, 0, 0, 2, 3,
    1, 0, 4294967295, 4294967295, 0, 0, 0, 0, 1, 0, 1, 2, 1, 168, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 0, 0, 1, 0, 1,
    2, 1, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0,
    4294967295, 4294967295, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0,
    0, 1, 0, 1, 2, 1, 1, 2, 1, 0, 0, 0, 0, 0, 0, 0,
    1, 0, 0, 1, 2, 1, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0,
    1, 2, 1, 0, 1021, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1022, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1023, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1024, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1025, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1026, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    967, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1027, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 17, 1028, 617, 1, 1028, 2, 3, 4, 17, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 1034, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1013, 1, 2, 1, 1, 4, 3, 1, 4, 5, 607, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1035, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1036, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1037, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1038, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    608, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1039, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 886, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1040, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1041, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1042, 1, 2, 1, 1, 4, 3, 1, 4, 5, 428, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1045, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 18, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 0, 1046,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 1047, 617, 1, 1047, 2,
    3, 4, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 19, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1069, 2534, 1, 1028, 617, 1, 1028, 2, 3, 4,
    0, 1, 0, 1, 2, 1, 0, 363, 1086, 1088, 3, 1086, 1087, 363, 3, 3,
    1087, 4, 1090, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1072, 1073, 1074, 1075,
    1076, 1077, 1078, 1079, 1080, 1081, 1082, 2572, 11, 1091, 617, 1, 1091, 2, 3, 4,
    1092, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1093, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1072, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1094, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1095, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1096, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1097, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1098, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1099, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1073, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1100, 1, 2, 1, 1, 4, 3, 1, 4, 5, 20, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1101, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 497, 1, 2, 1, 1, 4, 3, 1, 4, 5, 463, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1102, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    169, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1103, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1104, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1108,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1, 0, 1,
    4294967295, 4294967295, 0, 489, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1127, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1128, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 490, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1129, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1130, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1131,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1132, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1133, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1134, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1135, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 170, 1, 2, 1, 1, 4, 3, 1, 4, 5, 729, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1146, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1147,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1148, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1149, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1150, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1151, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1152, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1153, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1154, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1156,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1157, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1158, 1, 2, 1, 1, 4, 3, 1, 4, 5, 897, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 218, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 21, 1, 2, 1, 1, 4, 3, 1, 4, 5, 989, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 22, 1, 2, 1, 1, 4, 3, 1, 4, 5, 23,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1159, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 24, 1, 2, 1, 1, 4, 3, 1, 4, 5, 395, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1160, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1161, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1162, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1163, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1164,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1165, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1166, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1167, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 556, 547, 274, 1, 547, 3, 3, 4, 275,
    0, 4294967295, 4294967295, 0, 1, 0, 0, 464, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1171, 1, 2, 1, 1, 4, 3, 1, 4, 5, 25, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 328, 1172, 274, 1, 1172, 3, 3, 4, 275, 1052, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 3, 4, 1, 171, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 0, 0, 1178, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1179, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1180, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1181, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1182, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1183, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 966, 1, 2, 1, 1, 4, 3, 1, 4, 5, 26, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 388, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1017, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1012, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1184, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 725, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1192, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1193, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1194, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1, 1, 172, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1199, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1200, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1201, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1202,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1203, 1204, 274, 1, 1204, 3, 3,
    4, 275, 652, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1205, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1206, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1208, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1209, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1210, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1211, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1212, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1213, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1214, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1215, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 4294967295, 4294967295, 0, 650, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295,
    4294967295, 0, 0, 1222, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1223, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1224, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1225, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 4294967295, 4294967295, 0,
    0, 0, 1, 0, 0, 0, 173, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1234, 1, 2, 1, 1, 4, 3, 1, 4, 5, 174, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1235, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1236, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1237, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1239, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1007, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1240, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 2,
    1, 0, 0, 4294967295, 4294967295, 0, 0, 0, 1, 0, 0, 1, 2, 1, 1, 0,
    1, 2, 1, 0, 0, 1, 2, 1, 0, 0, 1293, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 27, 1, 2, 1, 1, 4, 3, 1, 4, 5, 987, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1294, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 684, 1, 2, 1, 1, 4, 3, 1, 4, 5, 684, 1295, 1296, 1,
    1295, 5, 3, 1, 1087, 4, 5, 0, 0, 0, 0, 1297, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 0, 1307, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 278, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0,
    1312, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1313, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1185, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 28, 1, 2, 1, 1, 4, 3, 1, 4, 5, 530, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1316, 1, 2, 1, 1, 4, 3, 1, 4, 5, 175,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1317, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1318, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1319, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1320, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1321, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1322, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 971, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 881, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 0, 1,
    2, 1, 0, 1, 0, 0, 1323, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    176, 1, 2, 1, 1, 4, 3, 1, 4, 5, 519, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 295, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1352, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 968, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1353, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0,
    0, 1362, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1363, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1364, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1365,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1366, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1367, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0,
    1, 2, 1, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 1, 2, 1,
    0, 353, 1410, 1, 0, 0, 4294967295, 4294967295, 0, 364, 1410, 1, 1, 2, 1, 0,
    1137, 1, 2, 1, 1, 4, 3, 1, 4, 5, 843, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1411, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    1, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 279, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 5, 1028, 1430, 2, 1028, 5, 3, 3, 4, 5,
    0, 4294967295, 4294967295, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 2, 1,
    0, 0, 296, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1450, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1451, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1452, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1453, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 804, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1454, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1455, 1455, 823, 3, 1455, 480, 1455, 3,
    3, 480, 4, 177, 1, 2, 1, 1, 4, 3, 1, 4, 5, 988, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1456, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1457, 1, 2, 1, 1, 4, 3, 1, 4, 5, 178, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0,
    0, 1468, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1469, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1471, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1, 0, 0, 0, 1371, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1, 1, 0, 1, 0, 1473, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1, 0, 1, 2, 1, 1481, 1, 2, 1, 1, 4, 3, 1, 4, 5, 460,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1482, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 2, 1, 0, 0, 1372, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1483, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1484, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 29, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1489, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1490, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1491, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1492,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1493, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1494, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1074, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1495, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1496, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1497, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 990, 1, 2, 1, 1, 4, 3, 1, 4, 5, 384,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1498, 1499, 9, 3, 3, 1498, 4,
    0, 0, 1, 2, 1, 1503, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 4294967295, 4294967295, 0, 1511, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1512, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 4294967295, 4294967295, 0, 376, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1424, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1251, 1, 2, 1, 1, 4, 3, 1, 4, 5, 491, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1329, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1521, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1531, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1532, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    458, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 2, 1,
    0, 1533, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1174, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 429, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    30, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 31, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1536, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1335, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1537, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1538, 1, 2, 1, 1, 4, 3, 1, 4, 5, 179, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1539, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1540, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1541, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1542, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1543, 1, 2, 1, 1, 4, 3, 1, 4, 5, 926, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 944, 1, 2, 1, 1, 4, 3, 1, 4, 5, 844, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1544, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1545, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1546, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 180, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 2, 1, 0, 1, 0, 1, 2, 1, 1, 0, 0, 0, 0, 1, 2,
    1, 0, 1, 4294967295, 4294967295, 0, 0, 1556, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 730, 1, 2, 1, 1, 4, 3, 1, 4, 5, 872, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1186, 1, 2, 1, 1, 4, 3, 1, 4, 5, 835,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1557, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1558, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1559, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1560, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1561, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1562, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1563, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1564,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1565, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1373, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1566, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 2, 1, 1, 2, 1,
    0, 0, 0, 0, 620, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    1, 0, 704, 1, 2, 1, 1, 4, 3, 1, 4, 5, 396, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1573, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 1, 0, 1, 2, 1, 0, 658, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1581, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1582, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1583, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    476, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1522, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1584, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    1, 0, 0, 0, 0, 0, 1, 2, 1, 0, 0, 919, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 32, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    4294967295, 4294967295, 0, 353, 1610, 1, 0, 364, 1610, 1, 0, 1608, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 705, 1623, 1, 1, 2, 1, 0,
    364, 1623, 1, 0, 0, 1, 0, 1, 2, 1, 1589, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 0, 1, 2, 1,
    0, 0, 4294967295, 4294967295, 0, 5, 6, 1, 1435, 1028, 1430, 1, 1028, 3, 3, 4,
    5, 1640, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 181,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 33, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1644, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1645, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1598, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 2, 9, 4, 3, 1, 4, 5, 1646, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1647, 1, 2, 1, 1, 4, 3, 1, 4, 5, 916, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1058, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1642, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1648, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1392, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1414,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1649, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1650, 1, 2, 1, 1, 4, 3, 1, 4, 5, 845, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1651, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1652, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 139, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 34, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 945, 1, 2, 1, 1, 4, 3, 1, 4, 5, 661, 661, 823, 3, 661,
    480, 661, 3, 3, 480, 4, 0, 1, 2, 1, 0, 0, 0, 1305, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1657, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1658, 1, 2, 1, 1, 4, 3, 1, 4, 5, 87, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 1666, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1087, 1296, 9, 5, 3, 1, 1087, 4, 5, 1087, 1088, 9, 3, 3,
    1087, 4, 1676, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1677, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 498, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1678, 1, 2, 1, 1, 4, 3, 1, 4, 5, 609, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1679, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1680, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 182, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 35, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1350, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1681, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1682, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1551, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 397, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1683, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 183, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1684, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1685, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1686, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1687, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1688, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1690, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1691, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1692, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1693, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 2,
    3, 1, 184, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1695, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 280, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1696, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1697, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 36, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1698, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1699, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1700, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1701, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 927, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    430, 1, 2, 1, 1, 4, 3, 1, 4, 5, 37, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1702, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1703, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1704, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1705, 1, 2, 1, 1, 4, 3, 1, 4, 5, 980, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1706, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    185, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 0, 0,
    1, 2, 1, 0, 353, 1610, 1, 364, 1610, 1, 1724, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1725, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1726, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 873, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1727, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1075, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1728, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1729, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1730, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 38, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1374, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 555, 547, 274, 1, 547, 3, 3, 4,
    275, 1076, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1731, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1732, 1, 2, 1, 1, 4, 3, 1, 4, 5, 431,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1406, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1733, 1, 2, 1, 1, 4, 3, 1, 4, 5, 559, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 559, 547, 1430, 1, 547, 3, 3, 4, 5,
    1734, 1, 2, 1, 1, 4, 3, 1, 4, 5, 119, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1735, 1, 2, 1, 1, 4, 3, 1, 4, 5, 511, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1736, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 551, 547, 274, 1, 547, 3, 3, 4, 275, 1737, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 398, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1077,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1738, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1739, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1740, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1741, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 552, 547, 274, 1, 547, 3, 3, 4, 275, 659, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 0, 1746, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1330, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1747, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2, 3, 1748, 1499, 1, 1748, 3, 3, 1498, 4, 469, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 1, 0, 1753, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1577, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1577, 1754, 1296, 1, 1754, 5, 3, 1, 1087, 4, 5, 1755, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1756, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1758, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1759, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 555, 1760, 274, 1, 1760, 3, 3, 4, 275, 1169, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1761, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1762, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1763, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1764, 1765, 12, 3, 3, 1764, 4, 1299, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1766, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 946, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1767, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1768, 1, 2, 1, 1, 4, 3, 1, 4, 5, 432,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1769, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 928, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1433, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1770, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1771, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1772, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 433, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1773,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 399, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1774, 1, 2, 1, 1, 4, 3, 1, 4, 5, 434, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 435, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 400, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 2,
    1, 1, 2, 1, 0, 0, 1, 2, 1, 0, 0, 1783, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1784, 1, 2, 1, 1, 4, 3, 1, 4, 5, 389,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 436, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 929, 1, 2, 1, 1, 4, 3, 1, 4, 5, 407, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1785, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1786, 1, 2, 1, 1, 4, 3, 1, 4, 5, 947, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 930, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1787,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 731, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 731, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1523, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 454, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1788, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1789, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1343, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    779, 1, 2, 1, 1, 4, 3, 1, 4, 5, 779, 1754, 1296, 1, 1754, 5,
    3, 1, 1087, 4, 5, 1790, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1791,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1520, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1792, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1793, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 439, 439, 1088, 3, 439, 1087, 439, 3, 3,
    1087, 4, 39, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1674, 1796,
    274, 1, 1796, 3, 3, 4, 275, 1, 2, 0, 0, 0, 0, 0, 1, 0,
    1, 2, 1, 0, 0, 4294967295, 4294967295, 0, 1802, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1803, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1804, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 772, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1805, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1806, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 0, 1375, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1807, 1, 2, 1, 1, 4, 3, 1, 4, 5, 981, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1808, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1809, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1810, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 0, 0, 186, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    4294967295, 4294967295, 0, 527, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1015, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1, 0, 1815, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1817, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1818, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1819, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1820, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1821,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1822, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 40, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1823, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1824, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1465, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1825, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 935, 1, 2, 1, 1, 4, 3, 1, 4, 5, 660,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 660, 547, 548, 1, 547, 3, 3,
    547, 4, 610, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0,
    0, 1, 4294967295, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 1, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 4294967295, 1, 0, 0, 0, 0, 1, 4294967295, 1, 0, 0, 0, 0, 1,
    0, 0, 351, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1871, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1631, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1219, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1231, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1629, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1872,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 41, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1873, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1,
    0, 1875, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1660, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 3, 4, 1, 0, 1291, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1877, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1187, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1878, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1879, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1880, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1881, 1, 2, 1, 1, 4, 3, 1, 4, 5, 323,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1882, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1336, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0,
    1, 2, 1, 0, 1, 0, 1, 2, 1, 1, 2, 1, 0, 4, 5, 1,
    462, 1, 2, 1, 1, 4, 3, 1, 4, 5, 898, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1885, 1, 2, 1, 1, 4, 3, 1, 4, 5, 4, 4294967295,
    1886, 617, 1, 1886, 2, 3, 4, 1887, 1886, 274, 1, 1886, 3, 3, 4, 275,
    1888, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1889, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1890, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1891, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1892, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1893, 1, 2, 1, 1, 4, 3, 1, 4, 5, 401, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1894, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1895, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 0,
    1858, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1896, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 999, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1633, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 5, 1897, 1430, 2, 1897, 5, 3, 3,
    4, 5, 1, 0, 0, 931, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1898,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 948, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1899, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1900, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1901, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1902, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1800, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1903, 1, 2, 1, 1, 4, 3, 1, 4, 5, 561,
    1904, 274, 1, 1904, 3, 3, 4, 275, 0, 0, 899, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 764,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 3, 4, 1, 1913, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1914, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1915, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 42, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 554, 2057, 274, 1, 2057, 3,
    3, 4, 275, 554, 547, 274, 1, 547, 3, 3, 4, 275, 71, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1907, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1549, 1, 2, 1, 1, 4, 3, 1, 4, 5, 554, 1760, 274, 1, 1760, 3,
    3, 4, 275, 0, 4294967295, 4294967295, 0, 2067, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2068, 1, 2, 1, 1, 4, 3, 1, 4, 5, 553, 547, 274, 1, 547,
    3, 3, 4, 275, 1111, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 972,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1358, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1055, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2070, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2071, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2072, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2073, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1811, 1, 2, 1, 1, 4, 3, 1, 4, 5, 475,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2074, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1263, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1849, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 943, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 43, 1, 2, 1, 1, 4, 3, 1, 4, 5, 44, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2075, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2076,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 2, 4294967295, 0, 45, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 297, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 85, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2077, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 46, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    47, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2078, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 48, 1, 2, 1, 1, 4, 3, 1, 4, 5, 260, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 187, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1376, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1,
    1078, 1, 2, 1, 1, 4, 3, 1, 4, 5, 188, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1000, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2081, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 995, 2082, 1499, 3, 2082, 1498, 995, 3,
    3, 1498, 4, 363, 2083, 1088, 3, 2083, 1087, 363, 3, 3, 1087, 4, 2084, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2032, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2086, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1354, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2087, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    479, 2088, 274, 1, 2088, 3, 3, 4, 275, 558, 2088, 274, 1, 2088, 3, 3,
    4, 275, 2089, 1, 2, 1, 1, 4, 3, 1, 4, 5, 557, 2088, 274, 1,
    2088, 3, 3, 4, 275, 657, 2088, 274, 1, 2088, 3, 3, 4, 275, 189, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2090, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2091, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2092, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 402, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2093, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2094, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2095, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    1, 0, 1, 2, 1, 2097, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2098,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 49, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 190, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 2100, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 1, 0, 0, 2108, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 191, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 4294967295, 4294967295, 0, 963, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    237, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2114, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2115, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2116, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2117, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2118, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2119, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2120, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2121, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2122, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2123, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2124, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1554, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2125, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2126, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2127, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2128, 1, 2, 1, 1, 4, 3, 1, 4, 5, 50, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 403, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1334, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2129, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 892, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2130, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2131, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2132, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2133, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 51, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1425, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2134, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2135, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2103, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 192, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 0, 1, 2, 1, 2137, 1, 2, 1, 1, 4, 3, 1, 4, 5, 900,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 193, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 894, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2138, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2139, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2140, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2141, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 0, 1, 0, 0, 1143, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2144, 1, 2, 1, 1, 4, 3, 1, 4, 5, 827,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 408, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 318, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0,
    0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 242, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 914, 2146, 274, 1, 2146, 3, 3, 4, 275, 846, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1252, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 52, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2147, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 53, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1029,
    1028, 617, 1, 1028, 2, 3, 4, 1029, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 281, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2148, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2149, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2150,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 298, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 54, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2153, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2154, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1524, 1, 2, 1, 1, 4, 3, 1, 4, 5, 949, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2155, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1859,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1668, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1301, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2152, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2156, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 2, 1, 0, 1, 2, 1, 1326, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2160, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2161, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 55, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 282, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2162, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 299, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2163,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2164, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2165, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2166, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2167, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2168, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1079, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2169, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2170,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 141, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 136, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1,
    0, 2, 10054, 2, 10056, 3, 658, 1163, 658, 731, 1163, 1, 10062, 1, 659, 0,
    194, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 1,
    2, 1, 0, 1, 2, 1, 0, 2193, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 640, 2194, 274, 1, 2194, 3, 3, 4, 275, 2195, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1421, 1, 2, 1, 1, 4, 3, 1, 4, 5, 56, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 0, 300, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 481, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2198, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2199, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 506, 1, 2, 1, 1, 4, 3, 1, 4, 5, 57,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 612, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 0, 0, 1, 4294967295, 4294967295, 0, 150, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 0, 2202, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1480, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2203, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 472, 1, 2, 1, 1, 4, 3, 1, 4, 5, 901, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2204, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 374, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2205, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 896, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 4294967295, 4294967295, 0, 2208, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2209, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1868, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 195, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2210, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 2213, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2105, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0,
    363, 2082, 1499, 3, 2082, 1498, 363, 3, 3, 1498, 4, 5, 4294967295, 2214, 1088, 1,
    2214, 3, 3, 1087, 4, 2215, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1344,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 556, 2216, 274, 1, 2216, 3, 3,
    4, 275, 575, 2217, 274, 1, 2217, 3, 3, 4, 275, 2218, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2079, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2219,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1466, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2220, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2221, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 871, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2222, 1, 2, 1, 1, 4, 3, 1, 4, 5, 824, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2223, 1, 2, 1, 1, 4, 3, 1, 4, 5, 58,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2224, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2225, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2226, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2227, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2228, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2229, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2230, 1, 2, 1, 1, 4, 3, 1, 4, 5, 902,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2231, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2232, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2233, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2234, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2235, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2236, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2237, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2238,
    1, 1430, 1, 1, 3, 3, 4, 5, 0, 2240, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2241, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2242, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2243, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2244, 1, 2, 1, 1, 4, 3, 1, 4, 5, 283, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 301, 1, 2, 1, 1, 4, 3, 1, 4, 5, 552,
    2245, 274, 1, 2245, 3, 3, 4, 275, 937, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2246, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2247, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2248, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2250, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 874, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2251, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2252,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2253, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2254, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2255, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 0, 0, 0, 0, 1,
    4294967295, 1, 0, 1, 2, 1, 932, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1841, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2270, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 284, 1, 2, 1, 1, 4, 3, 1, 4, 5, 302, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1596, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 2272, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1,
    2, 1, 2273, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2274, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2275, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2276, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2277, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1860, 1, 2, 1, 1, 4, 3, 1, 4, 5, 629, 4,
    617, 2, 4, 629, 2, 3, 4, 0, 2278, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2279, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2280, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2281, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    275, 274, 9, 3, 3, 4, 275, 2282, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 452, 1, 2, 1, 1, 4, 3, 1, 4, 5, 59, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 613, 1, 2, 1, 1, 4, 3, 1, 4, 5, 60,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2283, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 61, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2284, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 1, 2, 1, 1, 2,
    1, 0, 62, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 2288,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 303, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1399, 1, 2, 1, 1, 4, 3, 1, 4, 5, 63, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 285,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 304, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2292, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2293, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 1001, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 2297, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2298, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2299, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2300, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2301, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1221, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1304,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1264, 2302, 274, 1, 2302, 3, 3,
    4, 275, 92, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0,
    0, 4294967295, 4294967295, 0, 495, 1, 2, 1, 1, 4, 3, 1, 4, 5, 467, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1051, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2306, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2307, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 2310, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2311, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2312, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 381, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2313, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2314, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2315, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1415, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2316, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 64, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 2320, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2321, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2322, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2323, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2324, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2325, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2295, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 196, 1, 2, 1, 1, 4, 3, 1, 4, 5, 146, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2326, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 65, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295,
    0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 2333, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2334, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2335, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2336, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 685, 1, 2, 1, 1, 4, 3, 1, 4, 5, 685, 1295, 1296, 1,
    1295, 5, 3, 1, 1087, 4, 5, 1661, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2340, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2341, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 66, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2343, 1, 2, 1, 1, 4, 3, 1, 4, 5, 615, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1357, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2344, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 197, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2349, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2350, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1377, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 520, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2351, 1, 2, 1, 1, 4, 3, 1, 4, 5, 198, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2352, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1009, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2353, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2354, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 4294967295, 4294967295,
    0, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 0, 1, 0, 1,
    0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 0, 1, 0, 0, 1, 2,
    1, 765, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2,
    1, 0, 3, 4, 1, 1065, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2362,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 404, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 199, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2363, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 732, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2364, 1, 2, 1, 1, 4, 3, 1, 4, 5, 413, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 200, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2365,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 67, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2366, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2367, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2368, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2369, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2370, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 2, 1, 0, 766, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2371, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2372, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2373, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2374, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2375, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1176, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2376,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 692, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 1, 0, 1, 2, 1, 0, 693, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2377, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2378,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2379, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2380, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1601, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2381, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2382, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2383, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 1, 2, 1, 0, 983, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 984, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2386, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2387, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 68, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    305, 1, 2, 1, 1, 4, 3, 1, 4, 5, 504, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 405, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2388, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1345, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2389, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2390, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2391, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1080, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1081, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 353, 2392, 1, 705, 2393, 1, 0, 364, 2393, 1, 364,
    2392, 1, 720, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2394, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 0, 1140, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1339, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2395,
    2396, 12, 3, 3, 4, 2395, 2397, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 4294967295, 4294967295, 0, 1378, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    353, 2403, 1, 1, 0, 364, 2403, 1, 353, 2403, 1, 1, 0, 364, 2403, 1,
    380, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 2060, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1663, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 5, 1430, 9, 3, 3, 4, 5, 2317, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2410, 1, 2, 1, 1, 4, 3, 1, 4, 5, 201, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2411, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2412, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1,
    1439, 1028, 1430, 1, 1028, 3, 3, 4, 5, 1910, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2414, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2415, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1082, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2416, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2417, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2304, 1, 2, 1, 1, 4, 3, 1, 4, 5,
];

pub static AUX_DATA_OFFSETS: &[u32] = &[
    0, 10, 20, 30, 40, 53, 0, 69, 70, 0, 72, 73, 0, 74, 75, 76,
    77, 0, 78, 79, 80, 81, 0, 82, 50, 75, 0, 83, 84, 0, 86, 51,
    88, 89, 0, 90, 91, 93, 94, 75, 95, 96, 97, 98, 0, 99, 100, 101,
    70, 103, 104, 105, 0, 106, 107, 108, 0, 109, 110, 111, 0, 112, 113, 0,
    114, 73, 0, 115, 116, 0, 117, 118, 0, 120, 121, 122, 123, 0, 112, 113,
    0, 124, 86, 121, 125, 126, 113, 127, 0, 128, 129, 0, 130, 70, 0, 131,
    125, 0, 108, 132, 133, 0, 134, 135, 0, 137, 138, 140, 70, 142, 143, 79,
    0, 144, 52, 145, 0, 110, 120, 113, 0, 147, 113, 0, 148, 149, 151, 152,
    153, 154, 155, 156, 89, 157, 158, 73, 54, 202, 203, 204, 205, 206, 101, 207,
    0, 208, 158, 0, 209, 210, 211, 78, 212, 213, 214, 215, 0, 216, 217, 219,
    220, 0, 221, 222, 202, 224, 225, 104, 226, 227, 228, 229, 230, 231, 232, 0,
    233, 234, 236, 238, 0, 239, 240, 241, 243, 244, 245, 108, 246, 247, 248, 149,
    249, 0, 154, 250, 251, 89, 252, 253, 254, 255, 256, 81, 80, 0, 257, 55,
    261, 262, 263, 265, 0, 257, 56, 261, 262, 263, 77, 57, 269, 271, 59, 68,
    83, 154, 73, 286, 287, 126, 288, 143, 289, 290, 291, 0, 292, 70, 0, 114,
    73, 0, 106, 154, 0, 293, 294, 84, 306, 307, 308, 309, 310, 311, 312, 313,
    314, 315, 0, 292, 316, 0, 114, 307, 0, 106, 306, 0, 293, 317, 85, 319,
    321, 92, 0, 86, 287, 126, 324, 0, 325, 327, 329, 87, 0, 0, 325, 327,
    331, 90, 89, 265, 262, 0, 333, 334, 97, 94, 0, 93, 287, 126, 324, 89,
    157, 329, 99, 109, 337, 113, 123, 133, 143, 153, 163, 173, 183, 193, 203, 213,
    223, 233, 350, 0, 353, 356, 246, 0, 359, 243, 360, 362, 364, 247, 0, 367,
    369, 257, 267, 277, 0, 205, 373, 262, 86, 88, 375, 377, 288, 0, 78, 287,
    0, 382, 265, 383, 334, 359, 385, 0, 386, 359, 385, 0, 205, 289, 287, 126,
    375, 329, 290, 390, 391, 392, 0, 205, 104, 393, 295, 0, 406, 291, 409, 410,
    0, 411, 412, 0, 414, 412, 0, 415, 292, 416, 0, 417, 382, 393, 418, 419,
    420, 216, 421, 422, 423, 424, 296, 0, 205, 324, 375, 425, 306, 390, 391, 392,
    309, 0, 437, 334, 438, 440, 441, 307, 0, 442, 438, 443, 308, 444, 445, 446,
    0, 205, 143, 447, 310, 449, 0, 205, 79, 331, 0, 453, 314, 234, 134, 455,
    0, 456, 86, 88, 377, 0, 315, 216, 421, 234, 457, 126, 0, 316, 216, 421,
    459, 0, 216, 461, 317, 465, 466, 240, 0, 461, 438, 0, 421, 461, 318, 465,
    466, 240, 0, 224, 461, 319, 465, 466, 240, 0, 461, 438, 0, 216, 247, 0,
    468, 438, 0, 470, 416, 0, 421, 247, 0, 320, 216, 421, 471, 466, 0, 473,
    386, 474, 0, 149, 327, 0, 477, 478, 321, 482, 0, 243, 483, 0, 322, 487,
    488, 478, 323, 492, 493, 494, 0, 496, 478, 324, 499, 500, 416, 0, 137, 501,
    503, 478, 325, 505, 204, 0, 80, 478, 331, 0, 507, 478, 331, 0, 508, 478,
    326, 494, 509, 0, 205, 262, 151, 375, 331, 332, 391, 392, 418, 346, 0, 333,
    512, 513, 339, 514, 336, 334, 514, 0, 215, 345, 0, 517, 269, 342, 340, 269,
    0, 417, 517, 269, 382, 269, 358, 0, 512, 352, 521, 349, 347, 521, 0, 215,
    356, 0, 474, 109, 0, 523, 353, 269, 0, 487, 331, 524, 0, 357, 228, 215,
    104, 526, 0, 417, 487, 331, 382, 331, 0, 215, 517, 375, 359, 364, 0, 508,
    478, 359, 528, 529, 0, 360, 512, 513, 517, 514, 362, 0, 361, 531, 532, 269,
    533, 534, 0, 417, 363, 535, 382, 367, 365, 0, 417, 382, 367, 0, 205, 154,
    367, 370, 368, 367, 0, 508, 478, 366, 528, 529, 536, 538, 540, 0, 205, 89,
    375, 329, 371, 390, 216, 421, 0, 417, 382, 329, 391, 392, 372, 382, 391, 401,
    411, 421, 431, 440, 450, 460, 470, 480, 490, 562, 563, 564, 565, 566, 567, 568,
    569, 570, 571, 572, 573, 512, 492, 491, 438, 509, 574, 494, 503, 0, 515, 527,
    525, 580, 0, 0, 535, 530, 540, 550, 560, 570, 580, 0, 594, 331, 590, 589,
    331, 362, 438, 327, 593, 587, 589, 598, 590, 494, 509, 599, 609, 619, 628, 638,
    647, 657, 667, 677, 687, 697, 0, 601, 707, 717, 727, 737, 747, 757, 779, 788,
    528, 618, 619, 622, 269, 509, 0, 624, 528, 626, 628, 271, 535, 789, 795, 800,
    634, 817, 0, 815, 810, 636, 638, 811, 641, 642, 642, 818, 828, 0, 644, 646,
    648, 649, 651, 331, 838, 848, 858, 868, 878, 888, 662, 663, 664, 665, 666, 889,
    896, 0, 908, 905, 668, 508, 0, 327, 508, 0, 909, 919, 929, 939, 949, 674,
    0, 675, 675, 676, 0, 677, 677, 959, 968, 978, 988, 998, 681, 682, 999, 1009,
    686, 687, 1010, 1022, 1020, 528, 1027, 1025, 269, 1030, 1036, 691, 1039, 1049, 1059, 1069,
    0, 700, 701, 703, 705, 1080, 1079, 359, 589, 1082, 1092, 1102, 1112, 1122, 1132, 360,
    0, 359, 710, 535, 1135, 712, 1138, 1148, 1158, 1168, 1178, 1188, 1198, 1221, 0, 719,
    1208, 721, 722, 724, 651, 726, 651, 651, 0, 719, 1212, 721, 722, 724, 651, 726,
    651, 651, 719, 359, 0, 719, 1216, 728, 509, 1217, 721, 722, 724, 651, 726, 651,
    651, 719, 359, 665, 271, 733, 1222, 1232, 1242, 1251, 1261, 1271, 1281, 1291, 1301, 1311,
    0, 1321, 1331, 1341, 1351, 1361, 1371, 1381, 1391, 1401, 1411, 1421, 1431, 1441, 1451, 1461,
    1471, 1481, 0, 456, 1491, 359, 589, 1492, 1502, 1512, 1522, 1532, 0, 1538, 1548, 1560,
    416, 1558, 359, 762, 763, 1561, 1571, 1581, 1591, 0, 157, 359, 770, 0, 154, 771,
    1601, 624, 773, 775, 0, 771, 624, 0, 777, 778, 0, 104, 334, 624, 1602, 1611,
    1620, 1630, 1629, 783, 528, 622, 618, 785, 0, 0, 487, 331, 1634, 0, 777, 778,
    0, 786, 1631, 509, 0, 494, 787, 0, 788, 789, 790, 791, 0, 792, 789, 793,
    0, 795, 796, 798, 0, 359, 1649, 1636, 0, 799, 1635, 216, 421, 0, 624, 1641,
    0, 0, 1640, 801, 802, 1645, 0, 799, 1644, 216, 421, 803, 0, 1650, 1660, 359,
    762, 763, 0, 269, 805, 157, 1662, 1672, 0, 807, 0, 0, 1682, 1692, 1701, 1711,
    0, 211, 416, 1721, 1731, 0, 211, 233, 0, 1738, 0, 154, 367, 0, 517, 269,
    0, 122, 331, 0, 105, 814, 0, 114, 73, 367, 0, 770, 1739, 416, 778, 1740,
    1750, 1760, 478, 816, 817, 818, 819, 820, 821, 822, 1761, 0, 418, 213, 1768, 324,
    825, 826, 1770, 1769, 508, 478, 529, 0, 828, 829, 1772, 1782, 1792, 1798, 1804, 0,
    834, 89, 644, 1814, 836, 1818, 1828, 0, 676, 676, 1838, 1848, 0, 1858, 842, 847,
    848, 0, 78, 334, 445, 509, 0, 849, 529, 1859, 1869, 1879, 1889, 1899, 1909, 1919,
    1929, 1939, 1949, 1959, 1969, 1979, 1989, 1999, 2009, 0, 860, 359, 0, 722, 509, 2019,
    2029, 2039, 2049, 2059, 2069, 2079, 2089, 0, 212, 869, 2099, 624, 535, 624, 386, 870,
    2103, 288, 287, 126, 0, 2102, 875, 876, 877, 878, 329, 879, 0, 882, 331, 0,
    535, 883, 0, 212, 373, 262, 86, 88, 884, 377, 0, 2106, 0, 78, 2105, 0,
    382, 2104, 885, 0, 265, 383, 334, 359, 385, 0, 386, 359, 385, 0, 887, 362,
    0, 212, 86, 884, 888, 0, 212, 0, 890, 891, 2107, 391, 392, 287, 884, 329,
    879, 893, 2108, 773, 775, 624, 895, 2113, 903, 904, 905, 906, 907, 908, 909, 0,
    910, 887, 331, 911, 0, 912, 2110, 0, 157, 913, 2109, 915, 154, 2117, 770, 535,
    2118, 0, 917, 918, 470, 416, 920, 641, 0, 212, 2119, 523, 922, 104, 393, 233,
    367, 2122, 2120, 923, 925, 2125, 0, 212, 934, 118, 884, 331, 0, 936, 2143, 2141,
    0, 508, 478, 2140, 528, 2137, 2135, 416, 535, 712, 0, 212, 0, 890, 891, 2146,
    391, 392, 324, 826, 884, 425, 895, 2147, 903, 904, 905, 2151, 770, 535, 644, 0,
    646, 648, 649, 651, 425, 0, 212, 0, 890, 891, 2152, 391, 392, 324, 884, 425,
    895, 2153, 903, 904, 905, 2157, 770, 535, 644, 651, 425, 0, 212, 0, 890, 891,
    2158, 391, 392, 126, 884, 329, 879, 895, 2159, 903, 904, 905, 2163, 770, 535, 2164,
    644, 920, 641, 0, 212, 133, 884, 265, 0, 212, 291, 884, 325, 2170, 0, 437,
    334, 438, 0, 938, 386, 438, 440, 441, 2168, 0, 442, 438, 443, 2169, 444, 445,
    446, 0, 212, 143, 447, 2171, 939, 0, 212, 934, 2175, 391, 392, 262, 151, 884,
    331, 895, 882, 331, 0, 941, 377, 0, 212, 934, 942, 154, 884, 367, 2186, 0,
    2179, 2177, 2176, 521, 514, 793, 950, 951, 0, 950, 951, 535, 2185, 2182, 712, 0,
    953, 367, 954, 0, 212, 0, 890, 891, 2187, 391, 392, 89, 884, 956, 895, 903,
    0, 2189, 2188, 958, 532, 0, 959, 771, 882, 960, 962, 0, 882, 965, 137, 0,
    2190, 969, 409, 970, 0, 2191, 973, 974, 2192, 956, 2194, 216, 421, 0, 722, 2195,
    509, 2198, 975, 648, 651, 956, 0, 212, 0, 890, 891, 2199, 391, 392, 157, 826,
    884, 977, 895, 2200, 770, 535, 979, 651, 0, 212, 0, 890, 891, 2201, 391, 392,
    157, 884, 977, 224, 0, 982, 362, 331, 362, 895, 903, 2202, 904, 905, 2206, 770,
    535, 2207, 985, 986, 0, 212, 158, 884, 265, 2212, 0, 383, 2211, 0, 334, 359,
    0, 991, 992, 2208, 216, 421, 0, 2209, 993, 994, 0, 535, 2210, 996, 362, 0,
    997, 998, 2219, 903, 0, 2214, 0, 494, 155, 0, 499, 1002, 155, 0, 1003, 2213,
    1004, 253, 233, 331, 0, 113, 2215, 494, 331, 0, 1005, 1006, 0, 1008, 2216, 229,
    1010, 0, 216, 1011, 0, 80, 478, 2217, 1014, 204, 0, 243, 2218, 1016, 256, 0,
    212, 934, 0, 997, 224, 2223, 1018, 0, 391, 1018, 392, 114, 73, 884, 367, 533,
    535, 2227, 2224, 712, 642, 1019, 2228, 2238, 2248, 2258, 2268, 2278, 2288, 2298, 2308, 2316,
    0, 2326, 1030, 1031, 2327, 1032, 2330, 2340, 2350, 2360, 2370, 2380, 2390, 2400, 2410, 2420,
    2430, 2440, 2450, 2460, 2470, 0, 1043, 334, 265, 383, 334, 359, 0, 386, 1044, 2471,
    2481, 2491, 359, 327, 2493, 359, 762, 763, 2495, 2514, 2505, 359, 529, 2523, 1048, 2522,
    1050, 0, 1053, 1054, 0, 2517, 0, 2515, 1056, 1057, 2516, 1059, 0, 1060, 120, 0,
    0, 359, 327, 1061, 1062, 2518, 1064, 1066, 1067, 2524, 2535, 1070, 0, 2547, 2545, 2544,
    1071, 1083, 0, 2550, 90, 91, 1084, 1085, 2551, 0, 1089, 0, 382, 1089, 2562, 2583,
    2592, 2602, 2612, 2622, 2632, 2642, 2652, 2662, 2672, 2682, 2692, 2702, 2712, 2722, 2732, 2742,
    2752, 2762, 2772, 0, 1105, 359, 879, 0, 893, 2782, 773, 775, 624, 0, 770, 1106,
    641, 2783, 0, 1109, 2799, 2797, 2796, 0, 2794, 0, 359, 1110, 2793, 624, 773, 775,
    0, 1112, 622, 1113, 1115, 1117, 0, 777, 778, 0, 2795, 590, 494, 509, 641, 1119,
    1121, 1123, 641, 646, 651, 0, 494, 1125, 529, 2803, 0, 494, 1126, 2813, 2823, 2833,
    2843, 2853, 2863, 2873, 2883, 2893, 2903, 2913, 0, 214, 1136, 1139, 1142, 2923, 1145, 2933,
    2943, 2953, 2963, 2973, 2983, 2993, 3003, 3013, 0, 1155, 529, 331, 3023, 3033, 3043, 3053,
    3063, 3073, 3083, 3093, 3103, 3113, 3123, 3133, 3143, 3153, 3163, 3173, 3183, 3193, 3203, 3213,
    3223, 0, 1168, 3232, 0, 890, 1168, 233, 0, 1170, 154, 269, 771, 367, 3238, 214,
    227, 0, 254, 0, 771, 3236, 269, 3239, 3249, 3259, 3269, 3278, 0, 0, 524, 488,
    238, 0, 215, 869, 3288, 624, 535, 624, 1173, 0, 215, 77, 375, 331, 0, 1175,
    306, 0, 524, 488, 0, 215, 262, 86, 88, 375, 377, 0, 215, 86, 375, 888,
    1173, 0, 215, 287, 375, 329, 0, 215, 104, 375, 393, 1173, 3291, 0, 215, 118,
    375, 331, 0, 215, 324, 826, 375, 425, 0, 215, 126, 329, 0, 215, 113, 375,
    359, 524, 0, 215, 133, 375, 359, 0, 215, 291, 375, 325, 1173, 0, 215, 143,
    447, 0, 215, 262, 151, 375, 331, 224, 0, 215, 942, 154, 375, 3301, 367, 1173,
    0, 215, 89, 375, 956, 0, 215, 157, 826, 375, 331, 3303, 224, 1177, 1173, 0,
    215, 158, 375, 265, 524, 0, 215, 73, 375, 367, 1173, 3304, 3314, 3324, 3334, 3344,
    3354, 3364, 3374, 3384, 3394, 3404, 3414, 0, 3425, 0, 777, 3424, 1188, 1189, 1190, 3431,
    0, 3429, 1191, 876, 3430, 1121, 1119, 0, 724, 3435, 509, 648, 3436, 3446, 3456, 3466,
    3476, 0, 1195, 1197, 3479, 3489, 3499, 3509, 3519, 3529, 3538, 3548, 3558, 1207, 3568, 3578,
    3588, 3598, 3608, 3618, 3628, 3638, 0, 1216, 1217, 3648, 1217, 3652, 0, 722, 3666, 1218,
    0, 359, 3662, 0, 890, 359, 1220, 648, 3667, 3677, 3687, 3697, 0, 3707, 0, 941,
    941, 938, 3708, 359, 327, 1226, 0, 219, 492, 509, 3712, 1227, 1229, 0, 887, 3714,
    0, 3713, 457, 1232, 0, 457, 1232, 509, 0, 3716, 1233, 893, 3717, 1227, 1229, 3718,
    3728, 3738, 3748, 3758, 3768, 0, 1238, 359, 0, 722, 509, 3778, 3788, 3798, 0, 1241,
    3808, 712, 1242, 1244, 1246, 1248, 0, 1249, 3810, 3809, 1250, 0, 733, 1249, 1253, 1254,
    0, 770, 777, 1255, 1256, 1257, 1258, 0, 777, 1259, 1260, 1261, 1249, 0, 1262, 1248,
    3814, 1265, 0, 1266, 1267, 1268, 1269, 0, 1270, 3817, 3816, 0, 3815, 662, 663, 665,
    1270, 3821, 1265, 0, 1266, 1267, 3830, 0, 1271, 3822, 712, 0, 3825, 1272, 1273, 3826,
    1275, 783, 271, 0, 3848, 1277, 618, 622, 587, 3835, 3834, 509, 712, 3832, 3831, 269,
    622, 528, 1278, 1106, 528, 619, 1280, 1282, 269, 0, 359, 762, 1284, 0, 1195, 3840,
    3838, 509, 0, 624, 3843, 416, 438, 1285, 1286, 1287, 1278, 628, 1288, 0, 331, 3845,
    3844, 331, 362, 438, 327, 271, 1289, 3849, 1106, 528, 619, 1280, 1282, 269, 1290, 0,
    3850, 3860, 3870, 3880, 3890, 3900, 0, 1298, 3911, 531, 1300, 3912, 438, 509, 137, 3913,
    137, 1302, 3914, 1016, 0, 386, 1303, 3915, 0, 1298, 3925, 359, 589, 3927, 1227, 0,
    1229, 0, 1306, 3926, 438, 359, 3928, 3938, 0, 3948, 1308, 1310, 3952, 0, 1314, 3962,
    0, 1315, 509, 3965, 3975, 3985, 3995, 4005, 4015, 4025, 4035, 4045, 4055, 4065, 4075, 4085,
    0, 1324, 529, 457, 4095, 1325, 0, 1126, 771, 359, 0, 1327, 1328, 4096, 1246, 1242,
    1106, 1244, 4097, 0, 882, 4108, 0, 359, 4107, 1331, 1332, 457, 4115, 0, 409, 4114,
    1325, 0, 1333, 509, 0, 4110, 1126, 1337, 1338, 771, 359, 4111, 1106, 0, 1340, 509,
    0, 722, 509, 1341, 0, 882, 254, 0, 771, 367, 4117, 1346, 0, 1347, 438, 0,
    1348, 1349, 4118, 4128, 0, 1351, 109, 4138, 4148, 4158, 4168, 4178, 0, 1355, 4191, 4190,
    0, 1014, 137, 0, 4188, 438, 619, 1356, 269, 4189, 1359, 1360, 0, 1259, 1261, 4192,
    1302, 1361, 4193, 4203, 4213, 4223, 4233, 4243, 0, 1368, 4253, 1369, 4255, 1315, 1379, 0,
    1381, 334, 0, 1382, 334, 1383, 1384, 1385, 1386, 1019, 1388, 1298, 1390, 4268, 0, 1391,
    4259, 4256, 1393, 1395, 369, 4260, 1397, 1398, 1401, 1402, 0, 386, 1390, 369, 1398, 1404,
    4265, 0, 4261, 1402, 4287, 4284, 0, 4272, 1405, 1408, 4269, 1369, 0, 1382, 334, 0,
    1381, 334, 4273, 4277, 0, 4276, 1402, 1401, 0, 1382, 334, 0, 1381, 334, 4281, 4288,
    4298, 4308, 4318, 509, 1412, 494, 4325, 509, 0, 509, 535, 624, 0, 1416, 509, 1368,
    1368, 509, 0, 4319, 1071, 509, 1368, 509, 0, 1417, 4322, 1284, 4320, 1418, 1419, 0,
    1422, 528, 0, 4323, 416, 359, 269, 457, 4324, 416, 359, 269, 1423, 923, 1426, 1427,
    1429, 0, 4329, 1083, 0, 535, 416, 0, 917, 529, 4332, 4342, 0, 4352, 0, 359,
    327, 690, 4357, 4356, 1431, 996, 690, 941, 0, 508, 4362, 0, 457, 4359, 624, 773,
    775, 0, 4360, 590, 494, 509, 0, 457, 1232, 1432, 4361, 624, 773, 775, 4365, 4363,
    1434, 4369, 0, 1436, 1437, 0, 1440, 1441, 0, 1442, 1443, 1444, 0, 0, 329, 4368,
    1442, 1445, 1446, 1448, 1449, 4370, 4380, 4390, 4400, 4410, 4420, 4430, 4440, 4451, 4461, 4471,
    4481, 4491, 0, 225, 4512, 0, 4510, 0, 1458, 0, 233, 291, 325, 382, 4503, 1460,
    0, 1460, 383, 334, 4501, 359, 0, 386, 4504, 1462, 1463, 1464, 0, 1458, 233, 0,
    4506, 158, 96, 98, 0, 117, 118, 0, 918, 4505, 1467, 108, 0, 144, 147, 113,
    4507, 0, 0, 325, 327, 331, 382, 1460, 0, 386, 103, 1464, 0, 386, 225, 1464,
    4509, 0, 524, 488, 224, 0, 80, 478, 4511, 1014, 204, 0, 1458, 382, 1460, 4513,
    4523, 4533, 0, 677, 1207, 0, 1470, 675, 677, 4534, 0, 1381, 334, 4547, 204, 1472,
    1475, 0, 4545, 4544, 269, 438, 509, 1477, 4548, 0, 1382, 334, 1379, 1383, 1384, 1385,
    1298, 4549, 0, 4563, 4562, 269, 438, 509, 4559, 1477, 4565, 0, 1478, 1479, 4578, 4576,
    4575, 1475, 1032, 4581, 4591, 4601, 0, 1383, 4614, 4611, 509, 4615, 0, 1382, 334, 1379,
    1384, 1385, 1298, 4616, 4626, 4636, 4646, 0, 1485, 1487, 0, 1487, 1485, 4647, 4657, 4667,
    4677, 4687, 0, 4697, 4707, 4717, 4727, 4737, 4747, 4757, 4767, 4777, 0, 1500, 4785, 1501,
    0, 334, 494, 0, 233, 778, 0, 882, 227, 4784, 1016, 0, 1261, 254, 535, 1502,
    4786, 1504, 4789, 4800, 0, 4799, 0, 938, 386, 0, 437, 334, 1505, 1506, 442, 438,
    0, 1306, 1507, 1508, 1509, 444, 445, 443, 1382, 1510, 4804, 4814, 0, 1513, 4824, 728,
    509, 1220, 0, 1513, 1271, 0, 1514, 648, 4826, 0, 1516, 4825, 728, 509, 1220, 648,
    0, 724, 648, 651, 1513, 4830, 0, 1513, 777, 1271, 4840, 0, 4850, 1517, 1518, 1519,
    4851, 4861, 4871, 4881, 0, 359, 805, 4891, 1525, 1526, 1527, 1528, 1529, 1530, 4892, 4902,
    4912, 0, 777, 457, 4928, 4925, 4924, 4922, 1248, 712, 622, 4929, 4939, 4949, 0, 359,
    4959, 1534, 1535, 4960, 4970, 359, 4972, 4982, 4992, 5002, 5012, 5022, 5032, 5042, 5052, 5062,
    5072, 5082, 5092, 5102, 5112, 5122, 5132, 5142, 0, 227, 5174, 0, 0, 703, 5155, 367,
    5152, 1106, 369, 5158, 5156, 269, 5165, 0, 5163, 1446, 0, 149, 5161, 1547, 1142, 0,
    0, 334, 5164, 470, 1550, 712, 0, 1552, 1553, 0, 703, 367, 5166, 509, 0, 1555,
    1306, 5169, 438, 253, 0, 204, 5170, 0, 5175, 5185, 5195, 5205, 5215, 5225, 5235, 5245,
    5255, 5265, 5275, 5285, 5295, 5305, 5315, 5325, 0, 1085, 5344, 0, 1567, 5335, 438, 382,
    1568, 0, 1569, 5338, 438, 382, 1570, 5341, 438, 0, 1085, 5347, 0, 438, 5345, 416,
    1071, 416, 0, 416, 5346, 416, 1071, 5348, 5358, 1567, 1568, 1569, 1571, 1572, 1570, 0,
    703, 5360, 5359, 359, 331, 589, 5362, 5372, 5382, 0, 1574, 5392, 885, 1575, 5399, 5393,
    1286, 1576, 1578, 1285, 1579, 1580, 0, 771, 157, 5396, 5394, 0, 1016, 331, 0, 770,
    777, 1417, 1368, 5400, 0, 5410, 5420, 5430, 5440, 5450, 5460, 5470, 438, 622, 359, 0,
    359, 327, 359, 0, 1585, 327, 327, 1585, 0, 334, 1586, 5471, 0, 1587, 0, 294,
    1588, 1590, 0, 5474, 1284, 5472, 1592, 1593, 1594, 1595, 0, 386, 523, 1597, 0, 5475,
    778, 1599, 233, 778, 0, 5476, 0, 109, 416, 1507, 509, 0, 509, 0, 1600, 509,
    269, 0, 1233, 5482, 0, 1602, 5478, 0, 438, 5477, 801, 802, 0, 386, 1603, 0,
    5481, 1604, 1605, 359, 1588, 5483, 5493, 5514, 0, 1606, 1607, 1408, 5503, 1609, 0, 5507,
    1611, 5510, 1613, 1615, 5511, 0, 1616, 1607, 1408, 1611, 0, 1617, 1408, 5515, 0, 1618,
    0, 5525, 1619, 1621, 369, 0, 233, 5529, 5535, 5532, 509, 5536, 5540, 1624, 0, 5539,
    1625, 1626, 1627, 1628, 0, 887, 5543, 5541, 359, 5546, 5562, 0, 359, 882, 1630, 0,
    1170, 1632, 416, 1634, 0, 359, 624, 0, 294, 1588, 0, 1632, 416, 5558, 0, 5556,
    778, 1636, 0, 494, 509, 233, 5557, 1580, 1636, 0, 1637, 5565, 5563, 1638, 5573, 509,
    271, 416, 5569, 0, 5568, 778, 1636, 233, 1636, 1634, 5576, 5585, 0, 228, 5596, 0,
    1641, 5595, 531, 1643, 350, 5599, 5609, 5619, 5629, 5639, 5649, 5657, 5667, 5677, 5687, 5697,
    5707, 0, 5717, 5727, 5737, 5747, 5757, 5767, 5777, 5787, 0, 675, 1207, 0, 1470, 677,
    675, 5788, 5798, 0, 0, 777, 1653, 1248, 0, 1654, 1249, 5808, 953, 1655, 1656, 5809,
    5819, 0, 1306, 5834, 5831, 5830, 438, 509, 204, 5836, 0, 1390, 5835, 438, 509, 0,
    271, 438, 5837, 5847, 5857, 5867, 0, 0, 233, 1659, 5878, 1636, 0, 1662, 362, 5877,
    386, 1664, 1665, 5879, 5889, 416, 438, 1285, 1266, 1286, 1287, 587, 589, 138, 1667, 261,
    1669, 1671, 1673, 5890, 5899, 5906, 5916, 0, 5926, 5936, 5946, 5956, 5966, 5976, 5986, 5996,
    6006, 6016, 6026, 6036, 6046, 6056, 6066, 6076, 6086, 6096, 6106, 0, 359, 651, 1689, 648,
    651, 1689, 359, 6116, 6126, 6136, 6146, 0, 1694, 327, 359, 6159, 0, 118, 331, 0,
    271, 6156, 712, 6162, 6172, 0, 6182, 0, 6192, 6202, 6212, 6222, 6232, 6242, 6252, 6262,
    6272, 6282, 6292, 6302, 6312, 6322, 6332, 6342, 214, 0, 227, 533, 1446, 0, 232, 703,
    6352, 6362, 1707, 1709, 0, 722, 1711, 0, 1261, 509, 1220, 6366, 1712, 1714, 0, 722,
    777, 1711, 0, 1261, 509, 1220, 1716, 0, 1718, 6367, 367, 1719, 887, 6371, 367, 1719,
    0, 6368, 712, 369, 0, 622, 369, 6372, 1613, 6375, 1720, 0, 254, 1722, 1139, 1142,
    6378, 6388, 6398, 6408, 6418, 6428, 6438, 6448, 6458, 6468, 6478, 6488, 6497, 6507, 6517, 6527,
    6537, 6547, 6557, 6567, 6576, 6586, 6596, 6606, 6616, 6626, 6635, 6645, 6655, 6665, 6675, 6685,
    6695, 6705, 6714, 0, 1742, 6724, 1743, 1744, 1745, 6725, 204, 1417, 6726, 6736, 6746, 6756,
    6766, 0, 529, 913, 509, 0, 359, 535, 6779, 359, 6776, 1749, 0, 1385, 6780, 1751,
    6782, 6792, 6802, 6813, 0, 1757, 842, 6823, 6833, 6843, 6853, 0, 6862, 6872, 6882, 6892,
    6902, 6909, 6919, 6929, 6939, 6949, 6959, 6969, 6979, 6989, 6999, 7009, 7019, 7029, 7039, 7049,
    7059, 7069, 7079, 7089, 7108, 1446, 1775, 7099, 1106, 7102, 1776, 7105, 1778, 1779, 0, 7109,
    1244, 1242, 1246, 1780, 7114, 1782, 7113, 7110, 1106, 1778, 7115, 7125, 7135, 7145, 7155, 7165,
    7175, 0, 7185, 7195, 7205, 7215, 0, 7225, 0, 1470, 1207, 0, 675, 677, 7226, 0,
    7236, 7246, 7256, 7266, 7276, 7286, 7296, 7306, 0, 778, 7317, 7327, 7337, 7347, 7357, 7367,
    7378, 0, 416, 1600, 580, 7399, 7388, 1794, 7390, 7402, 327, 0, 327, 327, 7403, 233,
    535, 887, 271, 1797, 1798, 1799, 733, 1600, 641, 1401, 0, 7404, 120, 0, 252, 331,
    7408, 7406, 7405, 0, 359, 624, 979, 7412, 0, 777, 7411, 1189, 1190, 1801, 7416, 7426,
    7436, 7446, 7456, 7466, 0, 1390, 7476, 438, 509, 137, 7477, 137, 1302, 7478, 7488, 7498,
    7508, 7518, 7528, 0, 233, 234, 7540, 0, 7538, 215, 1812, 1813, 0, 7539, 214, 1812,
    1302, 7541, 7551, 1814, 7555, 7565, 0, 1816, 7575, 359, 589, 882, 7576, 362, 1106, 359,
    0, 887, 7579, 0, 7577, 457, 1232, 0, 457, 1232, 7578, 359, 362, 7581, 0, 1816,
    359, 1448, 7591, 7601, 7611, 7621, 7631, 7641, 7651, 7661, 7671, 7681, 7691, 7701, 7711, 7721,
    0, 890, 891, 7730, 7745, 0, 155, 331, 0, 1826, 438, 0, 1827, 438, 0, 1828,
    438, 1829, 7740, 419, 420, 7741, 1831, 0, 461, 438, 7742, 1832, 0, 1833, 438, 7743,
    1834, 409, 7744, 424, 423, 526, 7784, 0, 135, 1835, 7749, 492, 493, 0, 155, 331,
    0, 1826, 438, 0, 1836, 438, 0, 1827, 438, 0, 1828, 438, 1829, 7750, 419, 420,
    7751, 1831, 0, 461, 438, 7752, 443, 442, 7760, 1832, 0, 1833, 0, 882, 7755, 1837,
    0, 240, 7753, 1838, 1839, 0, 222, 7754, 1838, 1839, 0, 137, 1840, 1833, 7756, 1842,
    1843, 0, 517, 1840, 1833, 0, 882, 7759, 0, 240, 7757, 1838, 1839, 0, 222, 7758,
    1838, 1839, 0, 997, 137, 138, 1844, 7761, 411, 414, 0, 7762, 216, 421, 137, 1845,
    0, 1846, 7764, 0, 1847, 7763, 494, 224, 7781, 0, 1848, 7777, 7771, 0, 997, 1850,
    0, 1850, 882, 7770, 465, 0, 240, 7767, 1838, 1839, 0, 1851, 7768, 1838, 1839, 0,
    222, 7769, 1838, 1839, 0, 1852, 7772, 1853, 1838, 1854, 1839, 1855, 0, 1856, 7774, 1857,
    0, 334, 7773, 0, 1861, 1361, 1862, 1863, 0, 882, 507, 7775, 494, 204, 1853, 331,
    7776, 0, 1864, 204, 0, 997, 1864, 0, 997, 1848, 7782, 0, 221, 222, 331, 0,
    997, 221, 222, 7783, 1865, 1866, 0, 1382, 1867, 334, 7791, 0, 7788, 269, 438, 509,
    7789, 1534, 1535, 0, 1519, 7790, 531, 1643, 1869, 1379, 1355, 7793, 1379, 1383, 1384, 1385,
    1355, 1422, 1298, 7794, 7804, 1382, 1510, 7805, 7815, 7825, 7835, 7845, 7855, 7865, 7875, 0,
    1423, 1874, 7888, 359, 7885, 1749, 7889, 7899, 0, 1876, 7912, 0, 7909, 1287, 271, 1287,
    269, 7913, 7923, 7933, 7943, 7953, 7963, 7973, 7983, 7993, 8003, 0, 1883, 0, 8029, 1862,
    359, 334, 8028, 261, 0, 8013, 1884, 1361, 8016, 8014, 269, 0, 8019, 1382, 77, 509,
    334, 8022, 8020, 269, 0, 1507, 8025, 269, 8032, 8042, 8052, 8062, 8071, 8080, 8090, 8100,
    8110, 8120, 8130, 8140, 8150, 8160, 0, 1862, 334, 8175, 8172, 8170, 509, 8176, 8186, 8196,
    8206, 8228, 0, 1226, 8226, 8216, 1226, 416, 0, 8229, 8239, 8249, 8259, 8269, 8279, 8289,
    8299, 8309, 8319, 0, 1905, 8329, 0, 887, 331, 0, 8328, 137, 154, 1906, 0, 887,
    331, 8330, 0, 882, 8345, 8342, 8340, 269, 0, 457, 8348, 8346, 0, 1272, 369, 8351,
    0, 1908, 1909, 8363, 8361, 0, 622, 369, 1911, 1912, 8366, 8376, 8386, 8396, 712, 1780,
    1242, 1916, 1918, 1920, 1922, 1924, 1926, 1928, 1930, 1932, 1934, 1936, 1938, 1940, 1942, 1944,
    1946, 1948, 1950, 1952, 1954, 1246, 1244, 1956, 1958, 1121, 1960, 1962, 1964, 1966, 1968, 1970,
    1972, 1974, 1976, 1978, 1981, 1983, 1119, 1985, 1987, 1989, 1991, 1993, 1995, 1997, 1999, 2001,
    2003, 920, 2005, 1115, 1123, 2007, 622, 2009, 1273, 2011, 2013, 2015, 1341, 2017, 2019, 2021,
    2023, 2025, 2027, 2029, 2031, 2034, 2036, 2038, 2040, 2042, 2044, 2046, 2048, 2050, 2052, 2054,
    641, 8397, 8409, 0, 8407, 269, 622, 528, 478, 269, 2056, 0, 269, 2056, 478, 8408,
    269, 622, 528, 0, 1799, 8410, 2058, 8419, 8428, 8438, 8448, 8458, 8467, 2059, 2062, 2064,
    2065, 0, 8471, 8481, 8491, 8500, 8510, 2069, 8511, 8521, 8531, 8541, 8551, 8561, 8571, 8581,
    0, 474, 109, 8591, 8601, 8611, 8621, 8631, 8641, 8651, 8661, 8671, 0, 359, 8681, 651,
    1216, 1516, 8682, 0, 327, 359, 8686, 8696, 8706, 8716, 8726, 8736, 8746, 8756, 8766, 8776,
    0, 1265, 438, 8786, 0, 665, 8796, 204, 261, 2080, 8797, 712, 8800, 8810, 8820, 8830,
    8840, 8851, 8862, 8872, 0, 2085, 359, 8882, 8892, 8902, 8912, 8921, 8930, 8940, 8949, 8958,
    8968, 8978, 8988, 8998, 9008, 9018, 9028, 0, 157, 359, 770, 2096, 9042, 9040, 0, 359,
    9038, 624, 773, 0, 0, 777, 778, 9039, 590, 494, 509, 9045, 9055, 9065, 9075, 0,
    157, 359, 770, 2099, 1105, 0, 893, 9085, 775, 773, 331, 9086, 0, 245, 367, 533,
    2101, 9096, 0, 233, 214, 2102, 0, 233, 254, 2102, 0, 2104, 9100, 1625, 2106, 2107,
    9101, 9111, 0, 2109, 9121, 0, 2110, 535, 529, 9125, 9135, 2112, 524, 0, 149, 778,
    0, 997, 2113, 0, 149, 494, 9136, 9146, 9156, 9166, 9176, 9186, 9196, 9206, 9216, 9226,
    9236, 9246, 9256, 9266, 9276, 9286, 9296, 9306, 9316, 9326, 9336, 9346, 9356, 9366, 9376, 9386,
    9396, 9406, 9416, 9426, 9436, 9446, 0, 1846, 0, 2136, 9458, 9456, 359, 9461, 9471, 9481,
    9491, 0, 893, 509, 9501, 9511, 9521, 9531, 0, 9541, 1233, 893, 9544, 0, 9542, 2142,
    2143, 9543, 359, 509, 9546, 1227, 1229, 9547, 9557, 9567, 9577, 9587, 0, 2145, 9609, 0,
    9605, 0, 1458, 0, 233, 291, 325, 1368, 9599, 1460, 0, 1460, 383, 334, 9597, 359,
    0, 1458, 233, 0, 9601, 158, 96, 98, 0, 117, 118, 0, 918, 9600, 1467, 108,
    0, 144, 147, 113, 9602, 0, 0, 325, 327, 331, 1368, 1460, 9604, 0, 524, 488,
    224, 0, 80, 478, 9606, 1014, 204, 0, 9607, 265, 1368, 1460, 9610, 9620, 9629, 9639,
    9649, 359, 9659, 9669, 9679, 9687, 9697, 9707, 9717, 9727, 9737, 9747, 0, 367, 805, 2151,
    9757, 9767, 9777, 9787, 9797, 9807, 9817, 9827, 9837, 9847, 0, 2157, 9857, 2158, 153, 9858,
    438, 0, 9861, 2159, 9862, 438, 9865, 9875, 9885, 9895, 9905, 9915, 9925, 9935, 9945, 9955,
    9965, 9975, 9985, 9995, 10005, 10015, 10025, 10035, 10045, 2171, 0, 580, 369, 10046, 1417, 204,
    0, 248, 2173, 10047, 2175, 2177, 10063, 700, 703, 1368, 10049, 1315, 0, 1382, 334, 1306,
    1876, 1386, 10059, 1298, 10064, 0, 2178, 10074, 1227, 1229, 1136, 1139, 2180, 2182, 2184, 1290,
    2186, 1227, 2188, 350, 2189, 1379, 2186, 2188, 1386, 1019, 1388, 1552, 10086, 10078, 10075, 1776,
    10082, 10079, 2191, 2192, 10083, 712, 10087, 10097, 10106, 10116, 10126, 10136, 0, 1505, 438, 1508,
    10137, 0, 1506, 438, 1509, 10140, 0, 529, 327, 529, 327, 10138, 2196, 2197, 0, 529,
    327, 10139, 2196, 2197, 10141, 10151, 10161, 10171, 10181, 10191, 10201, 0, 149, 10211, 1547, 0,
    269, 478, 10213, 528, 618, 622, 269, 509, 1446, 494, 0, 1778, 10214, 0, 1386, 1778,
    350, 1379, 2186, 10218, 10229, 2200, 0, 10228, 1744, 1743, 204, 2201, 0, 149, 291, 884,
    325, 10230, 10240, 10250, 10260, 10270, 10280, 10290, 10300, 0, 2206, 10310, 2207, 1853, 10311, 0,
    10321, 1272, 1273, 10322, 0, 1797, 624, 1275, 10326, 10336, 10346, 10356, 10366, 10376, 2211, 2212,
    10377, 10387, 10397, 529, 996, 628, 327, 10398, 359, 0, 359, 762, 1284, 10400, 0, 438,
    10411, 10421, 0, 1067, 337, 10431, 1226, 10441, 10450, 10459, 10469, 10479, 10489, 10499, 10509, 10519,
    10529, 10539, 10549, 10559, 10569, 10579, 10589, 10599, 10609, 10619, 10629, 10639, 10649, 10659, 10669, 10679,
    10689, 10699, 10709, 0, 149, 10719, 10728, 233, 2239, 10729, 10739, 10749, 10759, 10769, 10779, 10789,
    10799, 10808, 0, 938, 386, 509, 10818, 10828, 10838, 0, 2249, 641, 10848, 10858, 712, 1780,
    1242, 1916, 1918, 1920, 1922, 1924, 1926, 1928, 1930, 1932, 1934, 1936, 1938, 1940, 1942, 1944,
    1946, 1948, 1950, 1952, 1954, 1246, 1244, 1956, 1958, 1121, 1960, 1962, 1964, 1966, 1968, 1970,
    1972, 1974, 1976, 1978, 1981, 1983, 1119, 1985, 1987, 1989, 1991, 1993, 1995, 1997, 1999, 2001,
    2003, 920, 2005, 1115, 1123, 2007, 622, 2009, 1273, 2011, 2013, 2015, 1341, 2017, 2019, 2021,
    2023, 2025, 2027, 2029, 2031, 2034, 2036, 2038, 2040, 2042, 2044, 2046, 2048, 2050, 2052, 10859,
    10869, 10879, 10889, 10899, 10909, 0, 2256, 10931, 10927, 0, 2257, 10919, 1004, 438, 0, 1300,
    10920, 1004, 438, 0, 2258, 438, 0, 2259, 10921, 438, 253, 0, 2260, 438, 0, 2261,
    438, 0, 2262, 2263, 438, 0, 2264, 10922, 494, 228, 2265, 0, 2266, 10923, 494, 228,
    1853, 0, 2267, 10924, 494, 228, 1853, 0, 2268, 10925, 253, 1004, 438, 0, 2269, 10926,
    778, 1004, 438, 10934, 10944, 10954, 10964, 10974, 10984, 10994, 2271, 10995, 11007, 11005, 0, 580,
    369, 0, 11010, 11020, 11030, 11040, 11050, 11060, 11079, 11070, 0, 674, 674, 359, 11080, 11090,
    11100, 11110, 11120, 11127, 11137, 11147, 11157, 11167, 11177, 11187, 11197, 0, 0, 487, 331, 11210,
    0, 786, 11207, 509, 0, 997, 226, 0, 523, 533, 2285, 0, 788, 533, 2285, 0,
    2287, 533, 792, 0, 11217, 1446, 618, 622, 367, 11211, 712, 11214, 1780, 628, 11218, 11229,
    359, 11228, 327, 0, 327, 327, 941, 233, 535, 887, 271, 1797, 1798, 1799, 733, 1600,
    641, 1401, 11231, 11241, 11251, 11261, 0, 11271, 2289, 733, 2290, 0, 11275, 2291, 2290, 11279,
    11289, 11299, 11309, 0, 0, 11319, 239, 2294, 1002, 11320, 11330, 2296, 942, 11331, 11341, 11351,
    11361, 11371, 11381, 11391, 11401, 11410, 11423, 0, 1084, 1062, 11421, 0, 386, 499, 91, 2303,
    0, 11420, 386, 1664, 91, 2303, 0, 91, 1062, 0, 11422, 386, 1664, 91, 2303, 11424,
    0, 2305, 91, 2303, 509, 11428, 11438, 11448, 11458, 11468, 0, 11478, 958, 532, 0, 959,
    771, 11479, 2308, 0, 965, 137, 0, 2309, 770, 646, 648, 0, 651, 2309, 641, 11480,
    11490, 11500, 11510, 11520, 11530, 11540, 11550, 11560, 11570, 0, 11580, 938, 234, 134, 11581, 156,
    2318, 0, 470, 359, 0, 1261, 997, 2319, 11582, 11592, 11602, 11612, 11622, 11632, 11642, 11652,
    11662, 11672, 11682, 2142, 2143, 2327, 0, 668, 2110, 0, 327, 359, 11683, 0, 777, 2328,
    11693, 0, 1261, 777, 2328, 0, 2329, 11697, 0, 890, 2329, 11704, 2330, 0, 2331, 11701,
    416, 2332, 11705, 11715, 11725, 11735, 11745, 2337, 2338, 2339, 11746, 11756, 11767, 0, 1662, 154,
    367, 11777, 11787, 11799, 11797, 580, 11802, 0, 1067, 1669, 0, 1195, 2342, 11812, 11822, 11832,
    11842, 2345, 11852, 2346, 2347, 2348, 11853, 11863, 11873, 0, 1745, 11883, 1417, 204, 11884, 0,
    523, 11894, 11904, 0, 11914, 11924, 11934, 11944, 11954, 0, 2178, 11964, 1227, 1229, 1136, 1139,
    2180, 2182, 2184, 1290, 2186, 1386, 1019, 1388, 350, 1379, 2180, 1552, 0, 1778, 11965, 0,
    1386, 1778, 11981, 0, 11974, 11971, 11969, 269, 0, 535, 11980, 11977, 11975, 416, 11995, 0,
    11988, 11985, 11983, 269, 0, 535, 11994, 11991, 11989, 438, 0, 457, 11998, 11997, 2355, 2356,
    12001, 0, 12011, 2357, 2358, 1519, 0, 2359, 2360, 12018, 12017, 12014, 12012, 269, 1911, 2361,
    12021, 12031, 12041, 12051, 0, 254, 367, 369, 149, 1722, 1136, 1139, 1142, 12061, 12071, 12081,
    12091, 12101, 0, 255, 888, 12111, 12121, 359, 12131, 12141, 12151, 12161, 12171, 0, 887, 104,
    12184, 393, 12181, 1950, 925, 12185, 12195, 12205, 12215, 12225, 12235, 12245, 12255, 12265, 0, 12275,
    1507, 1126, 12282, 12279, 12277, 12276, 494, 528, 509, 12283, 12293, 12303, 12313, 12323, 12333, 12343,
    12353, 12363, 0, 12377, 0, 12373, 2384, 0, 1059, 2385, 12374, 438, 154, 771, 12381, 0,
    1799, 624, 0, 777, 778, 2058, 0, 12378, 0, 624, 0, 777, 778, 777, 1801, 12382,
    12392, 12402, 12412, 12422, 12432, 12442, 12452, 12462, 12472, 12482, 12492, 12502, 12512, 12522, 0, 722,
    0, 12532, 728, 509, 12533, 1220, 12536, 12539, 509, 648, 12540, 12543, 12546, 12556, 0, 1315,
    12569, 12566, 509, 12570, 1379, 0, 1381, 334, 0, 1382, 334, 1383, 1384, 1385, 1876, 1298,
    12571, 12581, 0, 1340, 509, 1341, 12591, 12598, 0, 1590, 0, 12609, 12608, 0, 359, 762,
    0, 1284, 327, 1284, 12613, 0, 359, 12623, 1862, 1382, 2398, 350, 2400, 0, 386, 2402,
    12624, 12627, 2404, 248, 12629, 2406, 0, 386, 2402, 12632, 12635, 2404, 248, 12637, 2192, 0,
    386, 0, 997, 2407, 0, 12640, 0, 386, 997, 291, 2408, 0, 2409, 1381, 12650, 350,
    12653, 12663, 12673, 12680, 12690, 12700, 12710, 12720, 12733, 12731, 0, 509, 12730, 0, 535, 2413,
    509, 369, 12736, 12745, 12755, 12765, 12775, 12785, 12795, 12805,
];

pub static SEGMENT_TYPE_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 258, 4294967295,
    4294967295, 4294967295, 264, 266, 4294967295, 258, 4294967295, 4294967295, 4294967295, 264, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 320,
    322, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 330, 4294967295, 4294967295, 4294967295, 326, 4294967295,
    267, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 330, 4294967295, 4294967295, 338, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 352, 4294967295, 353, 358, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 364, 4294967295, 4294967295, 368,
    370, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 394, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 394, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 448, 4294967295, 450, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 480, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 480, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 480, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 480, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 480, 267, 4294967295, 4294967295, 480, 267, 4294967295, 4294967295, 480,
    4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 515, 4294967295, 4294967295, 515, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 270,
    4294967295, 4294967295, 4294967295, 270, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 522, 4294967295, 4294967295, 522, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    480, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 515, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 368, 4294967295, 4294967295, 4294967295, 4294967295, 368, 4294967295, 4294967295, 4294967295,
    368, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 480, 4294967295, 4294967295, 4294967295, 537, 539, 541, 4294967295, 4294967295, 4294967295,
    4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 547, 547,
    4294967295, 547, 547, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295,
    267, 4294967295, 4294967295, 4294967295, 4294967295, 588, 588, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 621, 623, 270, 510, 4294967295, 625, 4294967295, 4294967295, 629, 4294967295, 4294967295, 630, 3, 4294967295,
    635, 4294967295, 4294967295, 4294967295, 4294967295, 637, 639, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 645, 4294967295,
    4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 547, 547,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 689, 4294967295, 694, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 353, 4294967295, 4294967295, 4294967295, 588, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361,
    4294967295, 4294967295, 711, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 723, 4294967295, 4294967295, 4294967295, 727, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 723, 4294967295, 4294967295, 4294967295, 727,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 723, 4294967295, 4294967295, 4294967295, 727, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 588, 4294967295, 4294967295, 4294967295, 4294967295, 4, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 625, 774, 776, 4294967295, 4294967295, 625, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 625, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 784, 4294967295, 623, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 794,
    4294967295, 4294967295, 797, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 800, 4294967295, 4294967295, 4294967295, 4294967295, 625, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 800, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 812, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 368, 4294967295, 4294967295, 270,
    4294967295, 4294967295, 267, 4294967295, 4294967295, 815, 4294967295, 4294967295, 4294967295, 368, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 480, 480, 480, 480, 480, 480, 4294967295, 4294967295, 480, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 480, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 547, 480, 4294967295, 4294967295,
    4294967295, 4294967295, 645, 4294967295, 837, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 625, 4294967295, 625, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 880, 4294967295, 4294967295, 267, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 889, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330,
    880, 4294967295, 4294967295, 774, 776, 625, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 921, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 394, 4294967295,
    368, 4294967295, 4294967295, 924, 933, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 480, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 645, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 645, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 880, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    645, 921, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 448, 4294967295, 940, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    267, 4294967295, 4294967295, 267, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 368, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 522, 515, 794, 4294967295, 952, 4294967295, 4294967295, 952, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 368, 955, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 957, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 961, 964, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 957, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    510, 4294967295, 976, 4294967295, 4294967295, 957, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 978, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 978, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 480, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 368, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1020, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 330, 330, 4294967295, 1033, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1049, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1063, 4294967295, 4294967295, 4294967295, 1068, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 880, 4294967295, 4294967295, 4294967295, 774, 776, 625, 4294967295, 4294967295, 1107,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 625, 774, 776,
    4294967295, 4294967295, 623, 1114, 1116, 1118, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 1120,
    1122, 1124, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1138, 1141, 1144, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 368, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 625, 4294967295, 625, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 889,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 394, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295,
    448, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 368, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 957, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 368, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1122, 1120, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1196, 1198, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 1228, 1230, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1228, 1230, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1243, 1245, 1247, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 439, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 439, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1274, 4294967295,
    1276, 784, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 623, 588, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 270,
    623, 4294967295, 1279, 1107, 4294967295, 621, 1281, 1283, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1196, 4294967295,
    4294967295, 510, 4294967295, 625, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1279, 629, 4294967295, 4294967295, 267, 4294967295,
    4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1107, 4294967295, 621, 1281, 1283, 270, 1292, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 588, 4294967295, 1228, 4294967295,
    1230, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1309, 1311, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1247, 1243,
    1107, 1245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1107, 4294967295, 4294967295, 510,
    4294967295, 4294967295, 510, 1342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 368, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 621, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1370, 4294967295, 4294967295, 1380, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1387, 1020, 1389, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1394, 1396, 370, 4294967295, 4294967295, 1400, 4294967295, 1403, 4294967295, 4294967295, 4294967295, 370, 1400, 4294967295,
    4294967295, 4294967295, 4294967295, 1403, 4294967295, 4294967295, 4294967295, 4294967295, 1407, 1409, 4294967295, 1370, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 353, 4294967295, 4294967295, 4294967295, 1403, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 364, 4294967295,
    4294967295, 4294967295, 4294967295, 510, 1413, 4294967295, 4294967295, 510, 4294967295, 510, 4294967295, 625, 4294967295, 4294967295, 510, 4294967295,
    4294967295, 510, 4294967295, 4294967295, 4294967295, 510, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1420, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 924, 1281, 1428,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 689, 4294967295, 4294967295, 4294967295, 4294967295, 689, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 625, 774,
    776, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 625, 774, 776, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 330, 1438, 4294967295, 330, 1438, 4294967295, 330, 1438, 4294967295, 4294967295, 4294967295, 330, 4294967295,
    330, 330, 1447, 1438, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1459, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 1461,
    4294967295, 1461, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1459, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 326, 4294967295, 267, 4294967295, 1461, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 480, 4294967295, 4294967295, 4294967295, 4294967295, 1459, 4294967295, 1461, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1474,
    1476, 4294967295, 4294967295, 4294967295, 270, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1380, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1476, 1033, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 1380,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1486, 1488, 4294967295, 1488, 1486, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1498, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1515, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 623, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 368,
    4294967295, 1107, 370, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 1447, 4294967295, 4294967295, 4294967295, 1548, 1144, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 368, 4294967295, 510, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 267, 588, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 623, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1591, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 510, 4294967295, 4294967295, 510,
    270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1409, 4294967295, 4294967295, 4294967295, 353,
    1612, 4294967295, 1614, 4294967295, 364, 4294967295, 4294967295, 4294967295, 1409, 1612, 4294967295, 4294967295, 1409, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1620, 1622, 370, 4294967295, 4294967295, 353, 4294967295, 4294967295, 510, 364, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1635, 4294967295, 4294967295, 625, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1639, 4294967295, 510,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1635, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 352, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 439, 4294967295, 4294967295, 588, 588, 4294967295, 4294967295, 4294967295,
    1670, 1672, 1675, 1087, 1087, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1447, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1708, 1710, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 1713, 1715, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 1717, 4294967295, 4294967295, 4294967295, 368, 4294967295, 4294967295, 4294967295, 368, 4294967295,
    4294967295, 4294967295, 4294967295, 370, 4294967295, 623, 370, 353, 1614, 364, 1721, 4294967295, 4294967295, 1723, 1141, 1144,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1750, 4294967295, 4294967295, 4294967295, 1752,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1764, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1447, 1107, 4294967295, 1107, 4294967295, 1777, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1245, 1243, 1247, 1781, 4294967295, 694, 4294967295, 4294967295, 1107, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1795, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 625, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 588, 4294967295, 4294967295, 4294967295, 1107, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1438, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1830, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1830, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 510,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1870, 1380, 4294967295, 4294967295, 1380, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1750, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 510,
    4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 370, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 623, 370, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1781,
    1243, 1917, 1919, 1921, 1923, 1925, 1927, 1929, 1931, 1933, 1935, 1937, 1939, 1941, 1943, 1945,
    1947, 1949, 1951, 1953, 1955, 1247, 1245, 1957, 1959, 1122, 1961, 1963, 1965, 1967, 1969, 1971,
    1973, 1975, 1977, 1979, 1982, 1984, 1120, 1986, 1988, 1990, 1992, 1994, 1996, 1998, 2000, 2002,
    2004, 921, 2006, 1116, 1124, 2008, 623, 2010, 1274, 2012, 2014, 2016, 1342, 2018, 2020, 2022,
    2024, 2026, 2028, 2030, 2033, 2035, 2037, 2039, 2041, 2043, 2045, 2047, 2049, 2051, 2053, 2055,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 623, 4294967295, 480, 270, 4294967295, 4294967295, 270, 4294967295, 480, 4294967295,
    270, 623, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2061, 2063, 4294967295,
    2066, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 625, 774, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 776, 774, 267, 4294967295, 4294967295, 4294967295, 368, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2111, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 510, 4294967295, 1228, 1230, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1459, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 1461, 4294967295, 1461, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1459, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 267, 4294967295, 1461, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 480, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 1461, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 368, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2172, 4294967295, 4294967295, 370, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 2174, 4294967295, 2176, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1387, 4294967295, 4294967295, 4294967295, 4294967295, 2179, 4294967295, 1228, 1230, 1138, 1141, 2181, 2183, 2185, 1292,
    2187, 1228, 4294967295, 352, 2190, 1380, 2187, 4294967295, 1387, 1020, 1389, 4294967295, 4294967295, 4294967295, 4294967295, 1777,
    4294967295, 4294967295, 1777, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1548, 4294967295,
    270, 480, 4294967295, 4294967295, 4294967295, 623, 270, 510, 1447, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1387, 4294967295,
    352, 1380, 2187, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1274, 4294967295, 4294967295, 4294967295, 625, 1276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 629, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1068, 338, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 639, 4294967295, 4294967295, 4294967295, 4294967295, 1781,
    1243, 1917, 1919, 1921, 1923, 1925, 1927, 1929, 1931, 1933, 1935, 1937, 1939, 1941, 1943, 1945,
    1947, 1949, 1951, 1953, 1955, 1247, 1245, 1957, 1959, 1122, 1961, 1963, 1965, 1967, 1969, 1971,
    1973, 1975, 1977, 1979, 1982, 1984, 1120, 1986, 1988, 1990, 1992, 1994, 1996, 1998, 2000, 2002,
    2004, 921, 2006, 1116, 1124, 2008, 623, 2010, 1274, 2012, 2014, 2016, 1342, 2018, 2020, 2022,
    2024, 2026, 2028, 2030, 2033, 2035, 2037, 2039, 2041, 2043, 2045, 2047, 2049, 2051, 2053, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 547, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    370, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 275, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295,
    4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2286, 4294967295, 4294967295, 4294967295, 2286, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1447, 4294967295, 623, 368, 4294967295, 4294967295, 4294967295, 1781, 629, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1063, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1063, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2111, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    368, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1068, 1670, 4294967295, 1196, 1198, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2179, 4294967295, 1228, 1230, 1138, 1141,
    2181, 2183, 2185, 1292, 2187, 1387, 1020, 1389, 352, 1380, 2181, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1387, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 368, 370, 4294967295, 1723, 1138, 1141, 1144, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 889, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 394, 4294967295, 1951, 933, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 625, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 625, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 510, 353, 4294967295, 353, 4294967295, 510, 4294967295, 364, 364, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 510, 4294967295, 1380, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 510, 1342, 2395, 4294967295, 4294967295, 1591, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2399, 352, 2401, 4294967295, 4294967295, 4294967295,
    353, 4294967295, 2405, 4294967295, 364, 4294967295, 4294967295, 4294967295, 4294967295, 353, 4294967295, 2405, 4294967295, 364, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 352,
    4294967295, 4294967295, 5, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295,
    510, 370, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static SEGMENT_CLASS_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 257, 4294967295,
    4294967295, 4294967295, 263, 265, 4294967295, 257, 4294967295, 4294967295, 4294967295, 263, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 319,
    321, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 329, 4294967295, 4294967295, 4294967295, 325, 4294967295,
    331, 4294967295, 4294967295, 265, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 329, 4294967295, 4294967295, 337, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 350, 4294967295, 354, 356, 4294967295, 4294967295, 4294967295, 4294967295, 360, 4294967295, 365, 4294967295, 4294967295, 367,
    369, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 265, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 329, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 393, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 393, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 425, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 447, 4294967295, 449, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 478, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 478, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 478, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 478, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 478, 331, 4294967295, 4294967295, 478, 331, 4294967295, 4294967295, 478,
    4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 514, 4294967295, 4294967295, 514, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 269,
    4294967295, 4294967295, 4294967295, 269, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 521, 4294967295, 4294967295, 521, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 331, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295, 331, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    478, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 514, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 367, 4294967295, 4294967295, 4294967295, 4294967295, 367, 4294967295, 4294967295, 4294967295,
    367, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 478, 4294967295, 4294967295, 4294967295, 536, 538, 540, 4294967295, 4294967295, 4294967295,
    4294967295, 329, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 329, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 567, 568,
    4294967295, 570, 571, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295, 4294967295,
    331, 4294967295, 4294967295, 4294967295, 4294967295, 587, 589, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 619, 622, 269, 509, 4294967295, 624, 4294967295, 4294967295, 628, 4294967295, 4294967295, 631, 632, 4294967295,
    634, 4294967295, 4294967295, 4294967295, 4294967295, 636, 638, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 644, 4294967295,
    4294967295, 4294967295, 4294967295, 331, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 666, 548,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 690, 4294967295, 691, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 706, 4294967295, 4294967295, 4294967295, 589, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 360,
    4294967295, 4294967295, 710, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 721, 4294967295, 4294967295, 4294967295, 726, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 721, 4294967295, 4294967295, 4294967295, 726,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 721, 4294967295, 4294967295, 4294967295, 726, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 589, 4294967295, 4294967295, 4294967295, 4294967295, 617, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 624, 773, 775, 4294967295, 4294967295, 624, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 624, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 783, 4294967295, 622, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 793,
    4294967295, 4294967295, 796, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 799, 4294967295, 4294967295, 4294967295, 4294967295, 624, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 799, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 813, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 367, 4294967295, 4294967295, 269,
    4294967295, 4294967295, 331, 4294967295, 4294967295, 814, 4294967295, 4294967295, 4294967295, 367, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 478, 816, 817, 818, 819, 820, 4294967295, 4294967295, 823, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 478, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 831, 832, 4294967295, 4294967295,
    4294967295, 4294967295, 644, 4294967295, 836, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 624, 4294967295, 624, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 329, 879, 4294967295, 4294967295, 331, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 265, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 888, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 329,
    879, 4294967295, 4294967295, 773, 775, 624, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 331, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 920, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 393, 4294967295,
    367, 4294967295, 4294967295, 923, 925, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 478, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 425, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 644, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 425, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 425,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 644, 4294967295, 425, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 329, 879, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    644, 920, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 265, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 447, 4294967295, 939, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    331, 4294967295, 4294967295, 331, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 367, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 521, 514, 793, 4294967295, 951, 4294967295, 4294967295, 951, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 367, 954, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 956, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 960, 962, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 956, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    509, 4294967295, 975, 4294967295, 4294967295, 956, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 977, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 977, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 265, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 331, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 478, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 367, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1019, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1030, 1031, 4294967295, 1032, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 265, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1048, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1062, 4294967295, 4294967295, 4294967295, 1067, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 879, 4294967295, 4294967295, 4294967295, 773, 775, 624, 4294967295, 4294967295, 1106,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 624, 773, 775,
    4294967295, 4294967295, 622, 1113, 1115, 1117, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 1119,
    1121, 1123, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1136, 1139, 1142, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 367, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 624, 4294967295, 624, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 888,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 329, 4294967295, 4294967295, 4294967295, 4294967295, 393, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 331, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 425, 4294967295, 4294967295, 4294967295, 329, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295,
    447, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 367, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 956, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 265, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 367, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1121, 1119, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1195, 1197, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 1227, 1229, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1227, 1229, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1242, 1244, 1246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1273, 4294967295,
    1275, 783, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 622, 587, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 269,
    622, 4294967295, 1278, 1106, 4294967295, 619, 1280, 1282, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1195, 4294967295,
    4294967295, 509, 4294967295, 624, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1278, 628, 4294967295, 4294967295, 331, 4294967295,
    4294967295, 331, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1106, 4294967295, 619, 1280, 1282, 269, 1290, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 589, 4294967295, 1227, 4294967295,
    1229, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1308, 1310, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1246, 1242,
    1106, 1244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1106, 4294967295, 4294967295, 509,
    4294967295, 4294967295, 509, 1341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 367, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 619, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1369, 4294967295, 4294967295, 1379, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1386, 1019, 1388, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1393, 1395, 369, 4294967295, 4294967295, 1398, 4294967295, 1402, 4294967295, 4294967295, 4294967295, 369, 1398, 4294967295,
    4294967295, 4294967295, 4294967295, 1402, 4294967295, 4294967295, 4294967295, 4294967295, 1405, 1408, 4294967295, 1369, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 1402, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 365, 4294967295,
    4294967295, 4294967295, 4294967295, 509, 1412, 4294967295, 4294967295, 509, 4294967295, 509, 4294967295, 624, 4294967295, 4294967295, 509, 4294967295,
    4294967295, 509, 4294967295, 4294967295, 4294967295, 509, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1419, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 923, 1426, 1427,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 690, 4294967295, 4294967295, 4294967295, 4294967295, 690, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 624, 773,
    775, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 624, 773, 775, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1436, 1437, 4294967295, 1440, 1441, 4294967295, 1442, 1443, 4294967295, 4294967295, 4294967295, 329, 4294967295,
    1442, 1445, 1446, 1448, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1458, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 1460,
    4294967295, 1460, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1458, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 325, 4294967295, 331, 4294967295, 1460, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 478, 4294967295, 4294967295, 4294967295, 4294967295, 1458, 4294967295, 1460, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1472,
    1475, 4294967295, 4294967295, 4294967295, 269, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1379, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1475, 1032, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 1379,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1485, 1487, 4294967295, 1487, 1485, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1499, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1514, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 622, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 367,
    4294967295, 1106, 369, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 1446, 4294967295, 4294967295, 4294967295, 1547, 1142, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 367, 4294967295, 509, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 331, 589, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 622, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1590, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 509, 4294967295, 4294967295, 509,
    269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1408, 4294967295, 4294967295, 4294967295, 354,
    1611, 4294967295, 1613, 4294967295, 365, 4294967295, 4294967295, 4294967295, 1408, 1611, 4294967295, 4294967295, 1408, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1619, 1621, 369, 4294967295, 4294967295, 706, 4294967295, 4294967295, 509, 365, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1634, 4294967295, 4294967295, 624, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1638, 4294967295, 509,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1634, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 350, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1266, 4294967295, 4294967295, 587, 589, 4294967295, 4294967295, 4294967295,
    1669, 1671, 1673, 1296, 1088, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1446, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1707, 1709, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 1712, 1714, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 1716, 4294967295, 4294967295, 4294967295, 367, 4294967295, 4294967295, 4294967295, 367, 4294967295,
    4294967295, 4294967295, 4294967295, 369, 4294967295, 622, 369, 354, 1613, 365, 1720, 4294967295, 4294967295, 1722, 1139, 1142,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1749, 4294967295, 4294967295, 4294967295, 1751,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1765, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1446, 1775, 4294967295, 1106, 4294967295, 1776, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1244, 1242, 1246, 1780, 4294967295, 1782, 4294967295, 4294967295, 1106, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1794, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 624, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 589, 4294967295, 4294967295, 4294967295, 1106, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1448, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1829, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1829, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 509,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1869, 1379, 4294967295, 4294967295, 1379, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1749, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 509,
    4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    331, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 369, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 622, 369, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1780,
    1242, 1916, 1918, 1920, 1922, 1924, 1926, 1928, 1930, 1932, 1934, 1936, 1938, 1940, 1942, 1944,
    1946, 1948, 1950, 1952, 1954, 1246, 1244, 1956, 1958, 1121, 1960, 1962, 1964, 1966, 1968, 1970,
    1972, 1974, 1976, 1978, 1981, 1983, 1119, 1985, 1987, 1989, 1991, 1993, 1995, 1997, 1999, 2001,
    2003, 920, 2005, 1115, 1123, 2007, 622, 2009, 1273, 2011, 2013, 2015, 1341, 2017, 2019, 2021,
    2023, 2025, 2027, 2029, 2031, 2034, 2036, 2038, 2040, 2042, 2044, 2046, 2048, 2050, 2052, 2054,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 622, 4294967295, 478, 269, 4294967295, 4294967295, 269, 4294967295, 478, 4294967295,
    269, 622, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2059, 2062, 4294967295,
    2065, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 624, 773, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 775, 773, 331, 4294967295, 4294967295, 4294967295, 367, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2110, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 509, 4294967295, 1227, 1229, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1458, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 1460, 4294967295, 1460, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1458, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 331, 4294967295, 1460, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 478, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 265, 4294967295, 1460, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 367, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2171, 4294967295, 4294967295, 369, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 2173, 4294967295, 2175, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1386, 4294967295, 4294967295, 4294967295, 4294967295, 2178, 4294967295, 1227, 1229, 1136, 1139, 2180, 2182, 2184, 1290,
    2186, 1227, 4294967295, 350, 2189, 1379, 2186, 4294967295, 1386, 1019, 1388, 4294967295, 4294967295, 4294967295, 4294967295, 1776,
    4294967295, 4294967295, 2191, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1547, 4294967295,
    269, 478, 4294967295, 4294967295, 4294967295, 622, 269, 509, 1446, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1386, 4294967295,
    350, 1379, 2186, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1273, 4294967295, 4294967295, 4294967295, 624, 1275, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 628, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1067, 337, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 2249, 4294967295, 4294967295, 4294967295, 4294967295, 1780,
    1242, 1916, 1918, 1920, 1922, 1924, 1926, 1928, 1930, 1932, 1934, 1936, 1938, 1940, 1942, 1944,
    1946, 1948, 1950, 1952, 1954, 1246, 1244, 1956, 1958, 1121, 1960, 1962, 1964, 1966, 1968, 1970,
    1972, 1974, 1976, 1978, 1981, 1983, 1119, 1985, 1987, 1989, 1991, 1993, 1995, 1997, 1999, 2001,
    2003, 920, 2005, 1115, 1123, 2007, 622, 2009, 1273, 2011, 2013, 2015, 1341, 2017, 2019, 2021,
    2023, 2025, 2027, 2029, 2031, 2034, 2036, 2038, 2040, 2042, 2044, 2046, 2048, 2050, 2052, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2271, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    369, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 274, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295,
    4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2285, 4294967295, 4294967295, 4294967295, 2285, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1446, 4294967295, 622, 367, 4294967295, 4294967295, 4294967295, 1780, 628, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1062, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1062, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2110, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    367, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1067, 1669, 4294967295, 1195, 2342, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2178, 4294967295, 1227, 1229, 1136, 1139,
    2180, 2182, 2184, 1290, 2186, 1386, 1019, 1388, 350, 1379, 2180, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1386, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 367, 369, 4294967295, 1722, 1136, 1139, 1142, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 888, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 393, 4294967295, 1950, 925, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 624, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 624, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 509, 354, 4294967295, 706, 4294967295, 509, 4294967295, 365, 365, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 509, 4294967295, 1379, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 509, 1341, 2396, 4294967295, 4294967295, 1590, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2398, 350, 2400, 4294967295, 4294967295, 4294967295,
    354, 4294967295, 2404, 4294967295, 365, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 2404, 4294967295, 365, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 350,
    4294967295, 4294967295, 1430, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295,
    509, 369, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static CASEFOLD_SPARSE: &[(u32, u8)] = &[
    (3647, 1),
];

pub static TRIM_CHARS_SPARSE: &[(u32, u32, u8)] = &[
];

pub static TRIM_CHARS_DATA: &[u32] = &[
];

pub static SEGMENT_CLASS_TYPES_SPARSE: &[(u32, u32, u8)] = &[
    (206, 0, 2),
    (210, 2, 2),
    (211, 4, 3),
    (213, 7, 2),
    (217, 9, 2),
    (220, 11, 3),
    (271, 14, 2),
    (272, 16, 2),
    (280, 18, 3),
    (282, 21, 2),
    (286, 23, 3),
    (288, 26, 2),
    (291, 28, 3),
    (305, 31, 2),
    (308, 33, 2),
    (322, 35, 2),
    (324, 37, 4),
    (325, 41, 2),
    (330, 43, 2),
    (332, 45, 5),
    (335, 50, 3),
    (336, 53, 2),
    (347, 55, 3),
    (354, 58, 3),
    (369, 61, 2),
    (377, 63, 3),
    (397, 66, 3),
    (411, 69, 3),
    (435, 72, 3),
    (437, 75, 2),
    (452, 77, 3),
    (519, 80, 2),
    (529, 82, 2),
    (536, 84, 2),
    (545, 86, 2),
    (551, 88, 2),
    (552, 90, 2),
    (555, 92, 2),
    (556, 94, 2),
    (559, 96, 2),
    (562, 98, 2),
    (568, 100, 2),
    (579, 102, 2),
    (582, 104, 2),
    (588, 106, 3),
    (591, 109, 3),
    (595, 112, 3),
    (597, 115, 3),
    (602, 118, 2),
    (605, 120, 2),
    (615, 122, 3),
    (618, 125, 2),
    (629, 127, 2),
    (631, 129, 2),
    (640, 131, 2),
    (649, 133, 2),
    (655, 135, 3),
    (663, 138, 3),
    (668, 141, 3),
    (672, 144, 3),
    (678, 147, 2),
    (682, 149, 2),
    (683, 151, 2),
    (684, 153, 2),
    (689, 155, 2),
    (697, 157, 2),
    (718, 159, 2),
    (719, 161, 2),
    (721, 163, 2),
    (722, 165, 2),
    (729, 167, 2),
    (749, 169, 2),
    (752, 171, 2),
    (757, 173, 2),
    (758, 175, 2),
    (762, 177, 2),
    (786, 179, 2),
    (787, 181, 2),
    (788, 183, 3),
    (789, 186, 2),
    (791, 188, 2),
    (794, 190, 2),
    (797, 192, 2),
    (798, 194, 1),
    (800, 195, 2),
    (805, 197, 2),
    (806, 199, 2),
    (814, 201, 2),
    (819, 203, 2),
    (830, 205, 2),
    (831, 207, 3),
    (872, 210, 3),
    (873, 213, 2),
    (875, 215, 2),
    (884, 217, 4),
    (888, 221, 2),
    (895, 223, 2),
    (898, 225, 2),
    (913, 227, 2),
    (917, 229, 2),
    (923, 231, 2),
    (927, 233, 2),
    (936, 235, 2),
    (938, 237, 2),
    (942, 239, 2),
    (982, 241, 2),
    (987, 243, 2),
    (1009, 245, 2),
    (1010, 247, 2),
    (1011, 249, 2),
    (1014, 251, 2),
    (1021, 253, 2),
    (1027, 255, 2),
    (1029, 257, 2),
    (1035, 259, 2),
    (1043, 261, 2),
    (1055, 263, 2),
    (1058, 265, 3),
    (1065, 268, 2),
    (1070, 270, 2),
    (1079, 272, 2),
    (1091, 274, 3),
    (1108, 277, 3),
    (1116, 280, 3),
    (1119, 283, 3),
    (1122, 286, 2),
    (1125, 288, 2),
    (1129, 290, 3),
    (1138, 293, 2),
    (1139, 295, 2),
    (1140, 297, 2),
    (1141, 299, 2),
    (1142, 301, 2),
    (1143, 303, 2),
    (1146, 305, 3),
    (1157, 308, 2),
    (1164, 310, 2),
    (1165, 312, 2),
    (1170, 314, 2),
    (1172, 316, 2),
    (1189, 318, 2),
    (1214, 320, 2),
    (1227, 322, 2),
    (1229, 324, 2),
    (1242, 326, 2),
    (1243, 328, 2),
    (1246, 330, 2),
    (1257, 332, 3),
    (1268, 335, 3),
    (1284, 338, 3),
    (1295, 341, 2),
    (1296, 343, 2),
    (1299, 345, 2),
    (1300, 347, 2),
    (1301, 349, 2),
    (1314, 351, 2),
    (1334, 353, 2),
    (1342, 355, 3),
    (1344, 358, 3),
    (1347, 361, 2),
    (1348, 363, 2),
    (1355, 365, 2),
    (1362, 367, 2),
    (1381, 369, 3),
    (1390, 372, 2),
    (1396, 374, 3),
    (1407, 377, 3),
    (1416, 380, 2),
    (1418, 382, 3),
    (1429, 385, 2),
    (1430, 387, 2),
    (1440, 389, 2),
    (1441, 391, 2),
    (1447, 393, 3),
    (1452, 396, 3),
    (1476, 399, 3),
    (1478, 402, 2),
    (1488, 404, 2),
    (1491, 406, 2),
    (1494, 408, 3),
    (1501, 411, 3),
    (1507, 414, 2),
    (1508, 416, 2),
    (1509, 418, 2),
    (1511, 420, 2),
    (1514, 422, 2),
    (1521, 424, 3),
    (1522, 427, 2),
    (1533, 429, 3),
    (1545, 432, 2),
    (1546, 434, 2),
    (1561, 436, 3),
    (1568, 439, 2),
    (1570, 441, 2),
    (1573, 443, 3),
    (1585, 446, 3),
    (1602, 449, 3),
    (1607, 452, 2),
    (1624, 454, 3),
    (1667, 457, 2),
    (1672, 459, 2),
    (1686, 461, 2),
    (1710, 463, 3),
    (1717, 466, 2),
    (1730, 468, 2),
    (1731, 470, 2),
    (1733, 472, 2),
    (1752, 474, 3),
    (1774, 477, 2),
    (1796, 479, 2),
    (1800, 481, 2),
    (1846, 483, 2),
    (1850, 485, 2),
    (1851, 487, 2),
    (1852, 489, 2),
    (1855, 491, 2),
    (1869, 493, 2),
    (1870, 495, 2),
    (1871, 497, 2),
    (1874, 499, 2),
    (1875, 501, 2),
    (1876, 503, 2),
    (1877, 505, 2),
    (1885, 507, 2),
    (1887, 509, 2),
    (1888, 511, 2),
    (1889, 513, 2),
    (1914, 515, 2),
    (1915, 517, 2),
    (1916, 519, 2),
    (1931, 521, 2),
    (1963, 523, 3),
    (1965, 526, 3),
    (1974, 529, 3),
    (1989, 532, 2),
    (1991, 534, 2),
    (1997, 536, 2),
    (2010, 538, 3),
    (2015, 541, 3),
    (2021, 544, 2),
    (2026, 546, 3),
    (2033, 549, 2),
    (2039, 551, 3),
    (2043, 554, 2),
    (2059, 556, 3),
    (2064, 559, 3),
    (2070, 562, 2),
    (2078, 564, 3),
    (2084, 567, 3),
    (2090, 570, 2),
    (2099, 572, 3),
    (2105, 575, 3),
    (2133, 578, 2),
    (2134, 580, 2),
    (2138, 582, 2),
    (2146, 584, 2),
    (2147, 586, 2),
    (2201, 588, 2),
    (2203, 590, 2),
    (2204, 592, 2),
    (2215, 594, 2),
    (2221, 596, 2),
    (2222, 598, 2),
    (2234, 600, 2),
    (2242, 602, 2),
    (2243, 604, 2),
    (2244, 606, 2),
    (2275, 608, 2),
    (2292, 610, 2),
    (2302, 612, 2),
    (2304, 614, 2),
    (2305, 616, 2),
    (2311, 618, 2),
    (2312, 620, 2),
    (2315, 622, 2),
    (2319, 624, 3),
    (2320, 627, 2),
    (2322, 629, 2),
    (2323, 631, 2),
    (2325, 633, 2),
    (2326, 635, 2),
    (2327, 637, 2),
    (2328, 639, 3),
    (2334, 642, 2),
    (2337, 644, 2),
    (2339, 646, 2),
    (2346, 648, 2),
    (2347, 650, 2),
    (2350, 652, 2),
    (2353, 654, 2),
    (2360, 656, 2),
    (2362, 658, 2),
    (2363, 660, 2),
    (2364, 662, 2),
    (2365, 664, 3),
    (2366, 667, 2),
    (2381, 669, 2),
    (2396, 671, 2),
    (2398, 673, 2),
    (2400, 675, 2),
    (2410, 677, 2),
    (2411, 679, 2),
    (2418, 681, 2),
    (2446, 683, 2),
    (2447, 685, 2),
    (2448, 687, 2),
    (2449, 689, 2),
    (2467, 691, 2),
    (2476, 693, 2),
    (2479, 695, 2),
    (2482, 697, 2),
    (2483, 699, 2),
    (2489, 701, 3),
    (2518, 704, 2),
    (2520, 706, 3),
    (2539, 709, 2),
    (2542, 711, 2),
    (2552, 713, 2),
    (2553, 715, 2),
    (2554, 717, 2),
    (2562, 719, 2),
    (2563, 721, 2),
    (2564, 727, 2),
    (2567, 723, 2),
    (2569, 725, 2),
    (2573, 729, 2),
    (2574, 731, 2),
    (2579, 733, 2),
    (2584, 735, 2),
    (2585, 737, 2),
    (2587, 739, 2),
    (2598, 741, 2),
    (2611, 743, 2),
    (2612, 745, 2),
    (2615, 747, 2),
    (2617, 749, 2),
    (2619, 751, 2),
    (2622, 753, 2),
    (2625, 755, 2),
    (2629, 757, 2),
    (2631, 759, 2),
    (2638, 761, 2),
    (2646, 763, 3),
    (2651, 766, 3),
    (2653, 769, 2),
    (2654, 771, 2),
    (2655, 773, 2),
    (2673, 775, 2),
    (2678, 777, 2),
    (2686, 779, 2),
    (2687, 781, 2),
    (2688, 783, 2),
    (2693, 785, 2),
    (2699, 787, 2),
    (2700, 789, 2),
    (2701, 791, 2),
    (2707, 793, 2),
    (2708, 795, 2),
    (2710, 797, 2),
    (2711, 799, 2),
    (2713, 801, 2),
    (2714, 803, 2),
    (2718, 811, 2),
    (2720, 805, 2),
    (2721, 807, 2),
    (2722, 809, 2),
    (2723, 813, 2),
    (2744, 815, 2),
    (2748, 817, 3),
    (2751, 820, 2),
    (2753, 822, 2),
    (2765, 824, 2),
    (2787, 826, 3),
    (2789, 829, 2),
    (2791, 831, 2),
    (2807, 833, 2),
    (2812, 835, 2),
    (2814, 837, 2),
    (2831, 839, 2),
    (2832, 841, 2),
    (2836, 843, 3),
    (2838, 846, 2),
    (2844, 848, 2),
    (2853, 850, 3),
    (2855, 853, 2),
    (2865, 855, 2),
    (2866, 857, 2),
    (2874, 859, 2),
    (2879, 861, 2),
    (2888, 863, 2),
    (2889, 865, 2),
    (2891, 867, 2),
    (2892, 869, 2),
    (2907, 871, 3),
    (2960, 874, 2),
    (2966, 876, 2),
    (2973, 878, 2),
    (3018, 880, 2),
    (3055, 882, 3),
    (3057, 885, 2),
    (3058, 887, 2),
    (3061, 889, 3),
    (3065, 892, 2),
    (3069, 894, 2),
    (3070, 896, 2),
    (3083, 898, 3),
    (3085, 901, 2),
    (3156, 903, 2),
    (3157, 905, 2),
    (3181, 907, 2),
    (3197, 909, 2),
    (3217, 911, 3),
    (3242, 914, 2),
    (3244, 916, 2),
    (3247, 918, 2),
    (3248, 920, 3),
    (3275, 923, 2),
    (3280, 925, 2),
    (3282, 927, 2),
    (3288, 929, 2),
    (3289, 931, 2),
    (3292, 933, 2),
    (3298, 935, 2),
    (3299, 937, 2),
    (3300, 939, 2),
    (3306, 941, 2),
    (3331, 943, 2),
    (3334, 945, 2),
    (3348, 947, 2),
    (3357, 949, 2),
    (3359, 951, 2),
    (3369, 953, 2),
    (3380, 955, 2),
    (3386, 957, 4),
    (3431, 961, 2),
    (3438, 963, 2),
    (3464, 965, 2),
    (3467, 967, 2),
    (3468, 969, 2),
    (3472, 971, 2),
    (3473, 973, 2),
    (3474, 975, 2),
    (3475, 977, 5),
    (3476, 982, 3),
    (3518, 985, 2),
    (3548, 987, 2),
    (3554, 989, 2),
    (3555, 991, 2),
    (3561, 993, 2),
    (3564, 995, 2),
    (3565, 997, 2),
    (3572, 999, 2),
    (3574, 1001, 2),
    (3578, 1003, 3),
    (3582, 1006, 3),
    (3587, 1009, 2),
    (3589, 1011, 2),
    (3590, 1013, 2),
    (3592, 1015, 2),
    (3594, 1017, 2),
    (3597, 1019, 2),
    (3598, 1021, 2),
    (3599, 1023, 2),
    (3652, 1025, 2),
    (3659, 1027, 2),
    (3663, 1029, 2),
    (3680, 1031, 3),
    (3701, 1034, 2),
    (3702, 1036, 2),
    (3704, 1038, 2),
    (3706, 1040, 2),
    (3712, 1042, 2),
    (3713, 1044, 2),
    (3714, 1046, 2),
    (3715, 1048, 2),
    (3717, 1050, 2),
    (3720, 1052, 2),
    (3767, 1054, 2),
    (3791, 1056, 2),
    (3797, 1058, 2),
    (3816, 1060, 2),
    (3850, 1062, 2),
    (3854, 1064, 2),
    (3873, 1066, 2),
    (3895, 1068, 2),
    (3905, 1070, 2),
    (3935, 1072, 2),
    (3948, 1074, 2),
    (4079, 1076, 2),
    (4094, 1078, 2),
    (4109, 1080, 3),
    (4111, 1083, 2),
    (4120, 1085, 2),
    (4121, 1087, 2),
    (4124, 1089, 2),
    (4149, 1091, 2),
    (4160, 1093, 3),
    (4186, 1096, 3),
    (4191, 1099, 2),
    (4195, 1101, 3),
    (4199, 1104, 3),
    (4220, 1107, 2),
    (4248, 1109, 2),
    (4256, 1111, 2),
    (4263, 1113, 3),
    (4270, 1116, 2),
    (4278, 1118, 2),
    (4279, 1120, 2),
    (4287, 1122, 2),
    (4288, 1124, 2),
    (4289, 1126, 2),
    (4290, 1128, 2),
    (4291, 1130, 2),
    (4292, 1132, 2),
    (4293, 1134, 2),
    (4294, 1136, 2),
    (4295, 1138, 2),
    (4296, 1140, 2),
    (4297, 1142, 2),
    (4298, 1144, 2),
    (4299, 1146, 2),
    (4300, 1148, 2),
    (4301, 1150, 2),
    (4302, 1152, 2),
    (4303, 1154, 2),
    (4304, 1156, 2),
    (4305, 1158, 2),
    (4306, 1160, 2),
    (4307, 1162, 2),
    (4308, 1164, 2),
    (4309, 1166, 2),
    (4310, 1168, 2),
    (4311, 1170, 2),
    (4312, 1172, 2),
    (4313, 1174, 2),
    (4314, 1176, 2),
    (4315, 1178, 2),
    (4316, 1180, 2),
    (4317, 1182, 2),
    (4318, 1184, 2),
    (4319, 1186, 2),
    (4320, 1188, 2),
    (4321, 1190, 2),
    (4322, 1192, 2),
    (4323, 1194, 3),
    (4324, 1197, 2),
    (4325, 1199, 2),
    (4326, 1201, 2),
    (4327, 1203, 2),
    (4328, 1205, 2),
    (4329, 1207, 2),
    (4330, 1209, 2),
    (4331, 1211, 2),
    (4332, 1213, 2),
    (4333, 1215, 2),
    (4334, 1217, 2),
    (4335, 1219, 2),
    (4336, 1221, 2),
    (4337, 1223, 2),
    (4338, 1225, 2),
    (4339, 1227, 2),
    (4340, 1229, 2),
    (4341, 1231, 2),
    (4342, 1233, 2),
    (4343, 1235, 2),
    (4344, 1237, 2),
    (4345, 1239, 2),
    (4346, 1241, 2),
    (4347, 1243, 2),
    (4348, 1245, 2),
    (4349, 1247, 2),
    (4350, 1249, 2),
    (4351, 1251, 2),
    (4352, 1253, 2),
    (4353, 1255, 2),
    (4354, 1257, 2),
    (4355, 1259, 2),
    (4356, 1261, 2),
    (4357, 1263, 2),
    (4358, 1265, 2),
    (4359, 1267, 2),
    (4360, 1269, 2),
    (4361, 1271, 2),
    (4362, 1273, 2),
    (4363, 1275, 2),
    (4364, 1277, 2),
    (4365, 1279, 2),
    (4366, 1281, 2),
    (4367, 1283, 2),
    (4373, 1285, 3),
    (4374, 1288, 2),
    (4376, 1290, 2),
    (4377, 1292, 3),
    (4380, 1295, 3),
    (4382, 1298, 2),
    (4384, 1300, 3),
    (4385, 1303, 2),
    (4397, 1305, 2),
    (4398, 1307, 2),
    (4400, 1309, 2),
    (4497, 1311, 2),
    (4498, 1313, 2),
    (4506, 1315, 2),
    (4520, 1317, 2),
    (4521, 1319, 2),
    (4522, 1321, 2),
    (4526, 1323, 3),
    (4550, 1326, 2),
    (4611, 1328, 2),
    (4627, 1330, 2),
    (4629, 1332, 2),
    (4630, 1334, 2),
    (4642, 1336, 2),
    (4646, 1338, 3),
    (4649, 1341, 2),
    (4651, 1343, 2),
    (4657, 1345, 2),
    (4679, 1347, 3),
    (4681, 1350, 2),
    (4683, 1352, 2),
    (4691, 1354, 2),
    (4697, 1356, 3),
    (4699, 1359, 2),
    (4717, 1361, 3),
    (4761, 1364, 2),
    (4764, 1366, 2),
    (4770, 1368, 2),
    (4772, 1370, 2),
    (4785, 1372, 2),
    (4790, 1374, 2),
    (4792, 1376, 2),
    (4793, 1378, 2),
    (4794, 1380, 2),
    (4795, 1382, 2),
    (4796, 1384, 2),
    (4797, 1386, 2),
    (4798, 1388, 2),
    (4799, 1390, 2),
    (4800, 1392, 2),
    (4801, 1394, 2),
    (4803, 1396, 2),
    (4804, 1398, 2),
    (4805, 1400, 2),
    (4806, 1402, 2),
    (4808, 1404, 2),
    (4809, 1406, 2),
    (4810, 1408, 2),
    (4815, 1410, 2),
    (4818, 1412, 2),
    (4862, 1414, 2),
    (4864, 1416, 3),
    (4865, 1419, 2),
    (4869, 1421, 2),
    (4870, 1423, 3),
    (4871, 1426, 2),
    (4872, 1428, 2),
    (4878, 1430, 2),
    (4880, 1432, 2),
    (4881, 1434, 2),
    (4882, 1436, 2),
    (4896, 1438, 3),
    (4914, 1441, 2),
    (4918, 1443, 2),
    (4919, 1445, 2),
    (4933, 1447, 2),
    (4947, 1449, 2),
    (4948, 1451, 2),
    (4997, 1453, 2),
    (5002, 1455, 2),
    (5007, 1457, 2),
    (5008, 1459, 2),
    (5009, 1461, 2),
    (5010, 1463, 2),
    (5011, 1465, 2),
    (5012, 1467, 2),
    (5013, 1469, 2),
    (5014, 1471, 2),
    (5015, 1473, 2),
    (5016, 1475, 2),
    (5017, 1477, 2),
    (5018, 1479, 2),
    (5019, 1481, 2),
    (5020, 1483, 2),
    (5021, 1485, 2),
    (5022, 1487, 2),
    (5023, 1489, 2),
    (5024, 1491, 2),
    (5025, 1493, 2),
    (5026, 1495, 2),
    (5027, 1497, 2),
    (5028, 1499, 2),
    (5029, 1501, 2),
    (5030, 1503, 2),
    (5031, 1505, 2),
    (5032, 1507, 2),
    (5033, 1509, 2),
    (5034, 1511, 2),
    (5035, 1513, 2),
    (5036, 1515, 2),
    (5037, 1517, 2),
    (5038, 1519, 2),
    (5039, 1521, 2),
    (5040, 1523, 2),
    (5041, 1525, 2),
    (5042, 1527, 2),
    (5043, 1529, 3),
    (5044, 1532, 2),
    (5045, 1534, 2),
    (5046, 1536, 2),
    (5047, 1538, 2),
    (5048, 1540, 2),
    (5049, 1542, 2),
    (5050, 1544, 2),
    (5051, 1546, 2),
    (5052, 1548, 2),
    (5053, 1550, 2),
    (5054, 1552, 2),
    (5055, 1554, 2),
    (5056, 1556, 2),
    (5057, 1558, 2),
    (5058, 1560, 2),
    (5059, 1562, 2),
    (5060, 1564, 2),
    (5061, 1566, 2),
    (5062, 1568, 2),
    (5063, 1570, 2),
    (5064, 1572, 2),
    (5065, 1574, 2),
    (5066, 1576, 2),
    (5067, 1578, 2),
    (5068, 1580, 2),
    (5069, 1582, 2),
    (5070, 1584, 2),
    (5071, 1586, 2),
    (5072, 1588, 2),
    (5073, 1590, 2),
    (5074, 1592, 2),
    (5075, 1594, 2),
    (5076, 1596, 2),
    (5077, 1598, 2),
    (5078, 1600, 2),
    (5079, 1602, 2),
    (5080, 1604, 2),
    (5081, 1606, 2),
    (5082, 1608, 2),
    (5083, 1610, 2),
    (5084, 1612, 2),
    (5085, 1614, 2),
    (5086, 1616, 2),
    (5162, 1618, 2),
    (5168, 1620, 2),
    (5186, 1622, 3),
    (5198, 1625, 2),
    (5203, 1627, 2),
    (5210, 1629, 2),
    (5214, 1631, 2),
    (5221, 1633, 2),
    (5223, 1635, 2),
    (5224, 1637, 3),
    (5228, 1640, 2),
    (5229, 1642, 2),
    (5289, 1644, 2),
    (5304, 1646, 2),
    (5316, 1648, 2),
    (5384, 1650, 2),
    (5424, 1652, 3),
    (5432, 1655, 2),
    (5433, 1657, 2),
    (5435, 1659, 2),
    (5436, 1661, 2),
    (5466, 1663, 2),
    (5468, 1665, 2),
    (5469, 1667, 2),
    (5470, 1669, 2),
    (5471, 1671, 2),
    (5472, 1673, 2),
    (5473, 1675, 2),
    (5474, 1677, 2),
    (5475, 1679, 2),
    (5476, 1681, 2),
    (5477, 1683, 2),
    (5478, 1685, 2),
    (5479, 1687, 2),
    (5480, 1689, 2),
    (5481, 1691, 2),
    (5482, 1693, 2),
    (5488, 1695, 2),
    (5495, 1697, 3),
    (5507, 1700, 3),
    (5533, 1703, 3),
    (5542, 1706, 3),
    (5543, 1709, 2),
    (5545, 1711, 2),
    (5546, 1713, 2),
    (5547, 1715, 2),
    (5548, 1717, 2),
    (5556, 1719, 3),
    (5569, 1722, 3),
    (5571, 1725, 2),
    (5572, 1727, 2),
    (5592, 1729, 2),
    (5617, 1731, 2),
    (5625, 1733, 2),
    (5651, 1735, 2),
    (5656, 1737, 2),
    (5666, 1739, 2),
    (5668, 1741, 2),
    (5684, 1743, 2),
    (5685, 1745, 2),
    (5686, 1747, 3),
    (5689, 1750, 3),
    (5706, 1753, 2),
    (5707, 1755, 2),
    (5708, 1757, 2),
    (5714, 1759, 2),
    (5723, 1761, 2),
    (5743, 1763, 2),
    (5746, 1765, 3),
    (5755, 1768, 2),
    (5760, 1770, 2),
    (5761, 1772, 2),
];

pub static SEGMENT_CLASS_TYPES_DATA: &[u32] = &[
    258, 3, 264, 3, 3, 267, 266, 258, 3, 264, 3, 3, 270, 267, 3, 320,
    3, 322, 3, 267, 326, 3, 330, 3, 267, 326, 3, 267, 3, 267, 266, 3,
    330, 338, 3, 3, 352, 3, 353, 355, 4, 358, 3, 3, 361, 3, 364, 353,
    355, 4, 3, 267, 368, 370, 3, 3, 378, 267, 3, 267, 266, 3, 330, 3,
    394, 267, 3, 394, 267, 3, 267, 426, 3, 267, 448, 450, 3, 3, 378, 267,
    3, 480, 3, 480, 3, 480, 3, 480, 3, 480, 3, 267, 3, 480, 3, 267,
    3, 480, 3, 510, 3, 267, 3, 515, 3, 515, 3, 270, 267, 3, 270, 267,
    3, 270, 267, 3, 270, 267, 3, 522, 3, 522, 3, 270, 267, 3, 267, 3,
    267, 3, 267, 3, 480, 3, 515, 3, 270, 267, 3, 267, 368, 3, 267, 368,
    3, 267, 368, 3, 480, 537, 3, 539, 3, 541, 3, 3, 330, 3, 330, 3,
    547, 3, 547, 3, 547, 3, 547, 3, 510, 3, 267, 3, 267, 3, 588, 3,
    588, 3, 510, 3, 621, 3, 623, 3, 270, 267, 3, 510, 3, 625, 3, 629,
    3, 630, 3, 3, 635, 3, 637, 3, 639, 3, 645, 3, 267, 3, 547, 3,
    547, 4, 3, 270, 267, 3, 689, 3, 694, 3, 353, 355, 4, 3, 588, 3,
    361, 3, 711, 3, 723, 3, 727, 3, 723, 3, 727, 3, 510, 3, 723, 3,
    727, 3, 588, 3, 4, 3, 625, 3, 774, 3, 776, 3, 625, 3, 625, 3,
    784, 3, 623, 3, 267, 3, 510, 3, 794, 3, 797, 267, 3, 800, 3, 625,
    3, 800, 3, 270, 267, 3, 812, 4, 3, 267, 368, 3, 270, 267, 3, 267,
    3, 815, 3, 267, 368, 3, 480, 3, 480, 3, 480, 3, 480, 3, 480, 3,
    480, 3, 480, 4, 3, 480, 3, 547, 3, 480, 3, 645, 3, 837, 3, 510,
    3, 510, 3, 625, 3, 625, 3, 330, 3, 880, 3, 267, 3, 378, 267, 3,
    267, 266, 3, 889, 267, 3, 330, 3, 880, 3, 774, 3, 776, 3, 625, 3,
    267, 3, 921, 3, 394, 267, 3, 267, 368, 3, 924, 3, 933, 3, 267, 3,
    480, 3, 267, 426, 3, 645, 3, 267, 426, 3, 267, 426, 3, 645, 3, 267,
    426, 3, 330, 3, 880, 3, 645, 3, 921, 3, 267, 266, 3, 267, 326, 3,
    267, 448, 3, 940, 3, 267, 3, 267, 3, 378, 267, 3, 267, 368, 3, 522,
    3, 515, 3, 794, 3, 952, 3, 952, 3, 267, 368, 3, 955, 3, 267, 957,
    3, 961, 3, 964, 3, 267, 957, 3, 510, 3, 976, 3, 267, 957, 3, 267,
    978, 3, 267, 978, 3, 267, 3, 267, 266, 3, 267, 3, 267, 3, 480, 3,
    267, 368, 3, 1020, 3, 330, 3, 330, 3, 1033, 3, 267, 266, 3, 1049, 3,
    1063, 1068, 3, 3, 880, 3, 774, 3, 776, 3, 625, 3, 1107, 3, 625, 3,
    774, 3, 776, 3, 623, 3, 1114, 3, 1116, 3, 1118, 3, 510, 3, 1120, 3,
    1122, 3, 1124, 3, 1138, 3, 1141, 3, 1144, 3, 267, 3, 270, 267, 3, 267,
    368, 3, 270, 267, 3, 625, 3, 625, 3, 267, 3, 378, 267, 3, 889, 267,
    3, 330, 3, 394, 267, 3, 267, 3, 267, 426, 3, 330, 3, 267, 326, 3,
    267, 448, 3, 267, 3, 267, 368, 3, 267, 957, 3, 267, 3, 267, 266, 3,
    267, 368, 3, 1122, 3, 1120, 3, 510, 3, 1196, 3, 1198, 3, 510, 3, 1228,
    3, 1230, 3, 510, 3, 1228, 3, 1230, 3, 510, 3, 1243, 3, 1245, 3, 1247,
    3, 439, 3, 439, 3, 1274, 3, 1276, 3, 784, 3, 623, 3, 588, 3, 510,
    3, 270, 267, 3, 623, 3, 1279, 3, 1107, 3, 621, 3, 1281, 1283, 3, 3,
    270, 267, 3, 1196, 3, 510, 3, 625, 3, 1279, 3, 629, 3, 267, 3, 267,
    3, 1107, 3, 621, 3, 1281, 1283, 3, 3, 270, 267, 3, 1292, 3, 510, 3,
    588, 3, 1228, 3, 1230, 3, 1309, 3, 1311, 3, 510, 3, 1247, 3, 1243, 3,
    1107, 3, 1245, 3, 510, 3, 1107, 3, 510, 3, 510, 3, 1342, 3, 267, 368,
    3, 621, 3, 270, 267, 3, 1370, 3, 1380, 3, 1387, 3, 1020, 3, 1389, 3,
    1394, 3, 1396, 3, 1400, 3, 1403, 370, 3, 370, 3, 3, 1400, 3, 1403, 3,
    1407, 3, 1409, 3, 1370, 3, 1403, 3, 510, 3, 1413, 3, 510, 3, 510, 3,
    625, 3, 510, 3, 510, 3, 510, 3, 510, 1420, 3, 3, 270, 267, 3, 270,
    267, 3, 924, 3, 1281, 3, 1428, 3, 689, 3, 689, 3, 625, 3, 774, 3,
    776, 3, 510, 3, 625, 3, 774, 3, 776, 3, 330, 3, 1438, 3, 330, 3,
    1438, 3, 330, 3, 1438, 3, 330, 3, 330, 3, 1447, 3, 330, 3, 1438, 1459,
    3, 3, 267, 326, 1461, 3, 1461, 3, 1459, 3, 3, 267, 326, 3, 267, 1461,
    3, 3, 480, 1459, 3, 1461, 3, 3, 1474, 3, 1476, 3, 270, 267, 3, 510,
    3, 1380, 3, 270, 267, 3, 510, 3, 1476, 3, 1033, 3, 510, 3, 1380, 3,
    1486, 3, 1488, 3, 1488, 3, 1486, 3, 1498, 4, 3, 510, 3, 1515, 3, 510,
    3, 623, 3, 267, 368, 3, 1107, 370, 3, 3, 270, 267, 3, 1447, 3, 1548,
    3, 1144, 3, 267, 368, 3, 510, 3, 267, 3, 588, 3, 267, 3, 623, 3,
    267, 1591, 3, 510, 3, 510, 3, 510, 3, 270, 267, 3, 1409, 3, 1612, 3,
    1614, 3, 1409, 3, 1612, 3, 1409, 3, 1620, 3, 1622, 370, 3, 3, 510, 3,
    1635, 3, 625, 3, 510, 3, 1639, 3, 510, 3, 1635, 3, 352, 3, 1, 4,
    5, 3, 510, 3, 510, 3, 439, 3, 588, 3, 588, 1670, 3, 3, 1672, 3,
    1675, 3, 1, 1087, 4, 5, 3, 1087, 4, 3, 267, 3, 1447, 3, 1708, 3,
    1710, 3, 510, 3, 1713, 3, 1715, 3, 510, 3, 1717, 3, 267, 368, 3, 267,
    368, 370, 3, 3, 623, 370, 3, 3, 1614, 3, 1721, 3, 1723, 3, 1141, 3,
    1144, 3, 510, 3, 1750, 3, 1752, 3, 1764, 4, 3, 1447, 3, 1107, 3, 1107,
    3, 1777, 3, 1245, 3, 1243, 3, 1247, 3, 1781, 3, 694, 3, 1107, 3, 1795,
    3, 267, 3, 625, 3, 510, 3, 588, 3, 1107, 3, 1438, 3, 267, 3, 1830,
    3, 267, 3, 1830, 3, 267, 3, 267, 3, 270, 267, 3, 510, 3, 1870, 3,
    1380, 3, 1380, 3, 1750, 3, 270, 267, 3, 270, 267, 3, 510, 3, 270, 267,
    3, 270, 267, 3, 510, 3, 267, 3, 267, 3, 270, 267, 370, 3, 3, 623,
    370, 3, 3, 1781, 3, 1243, 3, 1917, 3, 1919, 3, 1921, 3, 1923, 3, 1925,
    3, 1927, 1929, 3, 3, 1931, 3, 1933, 3, 1935, 1937, 3, 3, 1939, 3, 1941,
    3, 1943, 3, 1945, 3, 1947, 3, 1949, 3, 1951, 3, 1953, 3, 1955, 3, 1247,
    3, 1245, 3, 1957, 3, 1959, 3, 1122, 3, 1961, 3, 1963, 3, 1965, 3, 1967,
    3, 1969, 3, 1971, 3, 1973, 1975, 3, 3, 1977, 3, 1980, 1979, 3, 1982, 3,
    1984, 3, 1120, 3, 1986, 1988, 3, 3, 1990, 3, 1992, 3, 1994, 3, 1996, 1998,
    3, 3, 2000, 2002, 3, 2004, 3, 3, 921, 2006, 3, 3, 1116, 3, 1124, 3,
    2008, 3, 623, 3, 2010, 3, 1274, 3, 2012, 3, 2014, 3, 2016, 3, 1342, 3,
    2018, 3, 2020, 3, 2022, 3, 2024, 3, 2026, 3, 2028, 3, 2030, 3, 2033, 3,
    2035, 2037, 3, 3, 2039, 3, 2041, 2043, 3, 3, 2045, 3, 2047, 2049, 3, 3,
    2051, 3, 2053, 3, 2055, 3, 270, 267, 3, 623, 3, 480, 3, 270, 267, 3,
    270, 267, 3, 480, 3, 270, 267, 3, 623, 3, 2061, 3, 2063, 3, 2066, 3,
    625, 3, 774, 3, 510, 3, 776, 3, 774, 3, 267, 3, 267, 368, 3, 2111,
    3, 510, 3, 510, 3, 1228, 3, 1230, 1459, 3, 3, 267, 326, 1461, 3, 1461,
    3, 1459, 3, 3, 267, 326, 3, 267, 1461, 3, 3, 480, 3, 267, 266, 1461,
    3, 3, 267, 368, 3, 2172, 370, 3, 3, 2174, 3, 2176, 3, 1387, 3, 2179,
    3, 1228, 3, 1230, 3, 1138, 3, 1141, 3, 2181, 3, 2183, 3, 2185, 3, 1292,
    3, 2187, 3, 1228, 3, 352, 3, 2190, 3, 1380, 3, 2187, 3, 1387, 3, 1020,
    3, 1389, 3, 1777, 3, 1777, 3, 1548, 3, 270, 267, 3, 480, 3, 623, 3,
    270, 267, 3, 510, 3, 1447, 3, 1387, 3, 352, 3, 1380, 3, 2187, 3, 267,
    326, 3, 1274, 3, 625, 3, 1276, 3, 629, 1068, 3, 338, 3, 3, 510, 3,
    639, 3, 1781, 3, 1243, 3, 1917, 3, 1919, 3, 1921, 3, 1923, 3, 1925, 3,
    1927, 1929, 3, 3, 1931, 3, 1933, 3, 1935, 1937, 3, 3, 1939, 3, 1941, 3,
    1943, 3, 1945, 3, 1947, 3, 1949, 3, 1951, 3, 1953, 3, 1955, 3, 1247, 3,
    1245, 3, 1957, 3, 1959, 3, 1122, 3, 1961, 3, 1963, 3, 1965, 3, 1967, 3,
    1969, 3, 1971, 3, 1973, 1975, 3, 3, 1977, 3, 1980, 1979, 3, 1982, 3, 1984,
    3, 1120, 3, 1986, 1988, 3, 3, 1990, 3, 1992, 3, 1994, 3, 1996, 1998, 3,
    3, 2000, 2002, 3, 2004, 3, 3, 921, 2006, 3, 3, 1116, 3, 1124, 3, 2008,
    3, 623, 3, 2010, 3, 1274, 3, 2012, 3, 2014, 3, 2016, 3, 1342, 3, 2018,
    3, 2020, 3, 2022, 3, 2024, 3, 2026, 3, 2028, 3, 2030, 3, 2033, 3, 2035,
    2037, 3, 3, 2039, 3, 2041, 2043, 3, 3, 2045, 3, 2047, 2049, 3, 3, 2051,
    3, 2053, 3, 547, 370, 3, 3, 4, 275, 3, 267, 3, 510, 3, 2286, 3,
    2286, 3, 1447, 3, 623, 3, 267, 368, 3, 1781, 3, 629, 3, 1063, 3, 1063,
    3, 510, 3, 2111, 3, 267, 368, 1068, 3, 1670, 3, 3, 1196, 3, 1198, 3,
    2179, 3, 1228, 3, 1230, 3, 1138, 3, 1141, 3, 2181, 3, 2183, 3, 2185, 3,
    1292, 3, 2187, 3, 1387, 3, 1020, 3, 1389, 3, 352, 3, 1380, 3, 2181, 3,
    1387, 3, 270, 267, 3, 270, 267, 3, 270, 267, 3, 267, 368, 370, 3, 3,
    1723, 3, 1138, 3, 1141, 3, 1144, 3, 889, 267, 3, 394, 267, 3, 1951, 3,
    933, 3, 510, 3, 625, 3, 625, 3, 510, 3, 510, 3, 510, 3, 1380, 3,
    510, 3, 1342, 3, 4, 2395, 3, 267, 1591, 3, 2399, 3, 352, 3, 2401, 3,
    2405, 3, 2405, 3, 352, 3, 4, 5, 3, 510, 3, 510, 370, 3,
];

pub static REGEX_PATTERNS: &[&str] = &[
    r#"[A-Z_][A-Z0-9_]*"#, // [0]
    r#"^(NOT)$"#, // [1]
    r#"[\p{L}\p{N}_]*[\p{L}][\p{L}\p{N}_#$]*"#, // [2]
    r#"^(ACCESS|ADD|ALL|ALTER|AND|ANY|AS|ASC|AUDIT|BETWEEN|BY|CASE|CHAR|CHECK|CLUSTER|COLUMN|COLUMN_VALUE|COMMENT|COMPRESS|CONNECT|CONNECT_BY_ROOT|CONSTRAINT|CREATE|CROSS|CURRENT|DATE|DECIMAL|DEFAULT|DEFINITION|DELETE|DELETING|DESC|DISABLE|DISTINCT|DROP|ELSE|ENABLE|EXCLUSIVE|EXECUTE|EXISTS|FILE|FLOAT|FOR|FORCE|FROM|FULL|GRANT|GROUP|HAVING|IDENTIFIED|IGNORE|IMMEDIATE|IN|INCREMENT|INDEX|INDEXTYPE|INITIAL|INNER|INSERT|INSERTING|INTEGER|INTERSECT|INTERVAL|INTO|INVISIBLE|IS|JOIN|LEFT|LEVEL|LIKE|LOCK|LOGGING|LONG|LOOP|MAXEXTENTS|MINUS|MLSLABEL|MODE|MODIFY|MONITORING|MULTISET|NATURAL|NESTED_TABLE_ID|NOAUDIT|NOCOMPRESS|NOLOGGING|NOMONITORING|NOREVERSE|NOT|NOWAIT|NULL|NUMBER|OF|OFFLINE|ON|ONLINE|OPTION|OR|ORDER|OUTER|OVERFLOW|PARAMETERS|PARTITION|PCTFREE|PIVOT|PRIOR|PRIVATE|PROMPT|PUBLIC|RAW|REBUILD|RENAME|RESOURCE|RESPECT|REVERSE|REVOKE|RIGHT|ROW|ROWID|ROWNUM|ROWS|SELECT|SESSION|SET|SHARE|SIBLINGS|SIZE|SMALLINT|START|SUCCESSFUL|SYNONYM|SYSDATE|TABLE|THEN|TO|TRIGGER|UID|UNION|UNIQUE|UNPIVOT|UNUSABLE|UPDATE|UPDATING|USER|USING|VALIDATE|VALUES|VARCHAR|VARCHAR2|VIEW|VISIBLE|WHEN|WHENEVER|WHERE|WITH)$"#, // [3]
    r#"[A-Z_][A-Z0-9_$]*|"[^"]*""#, // [4]
    r#"[KMGTPE]?"#, // [5]
];

pub static HINT_STRING_INDICES: &[u32] = &[
    0, 6, 7, 8, 9, 9, 10, 11, 12, 13,
    14, 15, 16, 17, 18, 19, 20, 21, 22, 23,
    24, 25, 26, 27, 28, 29, 30, 31, 32, 33,
    34, 35, 36, 37, 38, 39, 40, 41, 42, 43,
    44, 45, 46, 47, 48, 49, 50, 51, 52, 53,
    54, 55, 56, 57, 58, 59, 60, 61, 62, 63,
    64, 65, 66, 67, 68, 71, 10, 68, 11, 22,
    12, 13, 14, 57, 15, 16, 17, 85, 18, 87,
    65, 87, 65, 19, 92, 20, 21, 23, 24, 25,
    26, 28, 102, 29, 30, 31, 27, 32, 27, 32,
    51, 33, 34, 35, 46, 36, 37, 38, 119, 39,
    43, 40, 41, 42, 44, 45, 47, 48, 49, 44,
    50, 50, 52, 53, 136, 54, 139, 141, 55, 56,
    58, 146, 150, 59, 60, 61, 62, 63, 64, 66,
    67, 159, 160, 161, 162, 163, 102, 164, 165, 166,
    167, 14, 168, 169, 170, 171, 172, 173, 174, 175,
    176, 177, 178, 30, 179, 180, 181, 182, 183, 184,
    185, 186, 187, 188, 189, 190, 191, 51, 192, 193,
    194, 150, 195, 62, 196, 65, 197, 198, 199, 200,
    201, 159, 160, 161, 162, 163, 164, 165, 166, 167,
    168, 169, 170, 171, 172, 218, 173, 174, 175, 223,
    176, 178, 179, 180, 181, 182, 183, 184, 185, 186,
    235, 237, 187, 177, 188, 177, 188, 242, 189, 190,
    191, 192, 193, 194, 195, 196, 197, 198, 199, 200,
    201, 259, 260, 259, 260, 268, 272, 276, 27, 278,
    279, 280, 36, 45, 281, 282, 56, 283, 284, 62,
    285, 68, 283, 279, 281, 284, 285, 282, 280, 278,
    295, 27, 278, 296, 280, 36, 297, 298, 299, 300,
    301, 302, 303, 304, 305, 303, 305, 301, 296, 297,
    298, 300, 302, 304, 299, 178, 318, 318, 279, 323,
    45, 323, 328, 332, 335, 279, 323, 45, 65, 66,
    336, 339, 340, 341, 342, 343, 344, 345, 346, 347,
    348, 349, 351, 357, 363, 366, 371, 372, 374, 376,
    379, 14, 380, 381, 380, 381, 384, 379, 380, 380,
    279, 45, 387, 388, 389, 387, 388, 389, 387, 395,
    172, 396, 397, 398, 399, 400, 401, 402, 403, 404,
    405, 402, 407, 408, 408, 407, 398, 413, 400, 401,
    403, 397, 399, 395, 404, 396, 405, 427, 428, 429,
    430, 431, 432, 433, 434, 435, 436, 351, 429, 439,
    431, 435, 430, 434, 427, 432, 427, 432, 428, 433,
    428, 433, 436, 351, 347, 451, 395, 172, 176, 150,
    452, 235, 454, 53, 454, 451, 395, 172, 458, 460,
    462, 463, 464, 188, 464, 463, 467, 469, 472, 452,
    475, 476, 479, 189, 481, 481, 484, 485, 486, 485,
    486, 489, 490, 491, 491, 490, 489, 495, 497, 498,
    363, 498, 497, 502, 504, 342, 161, 506, 344, 171,
    511, 403, 344, 511, 511, 268, 516, 516, 344, 171,
    403, 268, 518, 485, 519, 475, 520, 518, 485, 519,
    475, 520, 485, 475, 520, 520, 525, 171, 181, 527,
    268, 348, 530, 348, 530, 530, 357, 381, 387, 395,
    388, 172, 389, 403, 542, 543, 545, 546, 549, 550,
    551, 543, 552, 553, 554, 555, 556, 557, 558, 559,
    560, 561, 554, 555, 556, 552, 551, 561, 560, 557,
    558, 559, 553, 575, 577, 579, 581, 582, 583, 584,
    585, 268, 575, 585, 489, 591, 592, 593, 595, 597,
    379, 598, 599, 600, 602, 603, 604, 605, 606, 607,
    608, 497, 609, 610, 611, 612, 613, 614, 615, 67,
    620, 627, 543, 629, 633, 640, 643, 647, 650, 652,
    653, 654, 655, 656, 657, 551, 543, 552, 553, 554,
    555, 556, 557, 479, 558, 546, 658, 559, 659, 660,
    560, 561, 661, 546, 660, 657, 557, 479, 558, 658,
    661, 659, 669, 670, 671, 672, 673, 678, 679, 680,
    660, 683, 684, 685, 685, 684, 688, 268, 194, 692,
    693, 695, 696, 697, 698, 699, 702, 704, 707, 708,
    709, 268, 194, 692, 693, 380, 713, 714, 715, 716,
    717, 718, 720, 725, 729, 730, 731, 732, 734, 735,
    737, 738, 739, 740, 741, 742, 743, 744, 745, 746,
    747, 748, 749, 750, 751, 518, 752, 753, 754, 755,
    756, 757, 758, 759, 760, 761, 657, 551, 543, 268,
    552, 553, 554, 272, 555, 556, 575, 735, 640, 557,
    479, 558, 546, 357, 658, 559, 659, 186, 660, 764,
    765, 766, 336, 560, 561, 661, 767, 768, 769, 658,
    772, 731, 779, 731, 779, 709, 743, 804, 806, 606,
    808, 810, 811, 516, 31, 36, 40, 62, 779, 363,
    657, 558, 657, 557, 661, 679, 323, 824, 824, 827,
    830, 833, 348, 643, 835, 838, 839, 840, 841, 843,
    844, 845, 846, 850, 851, 852, 853, 854, 855, 856,
    857, 858, 859, 861, 862, 863, 864, 865, 866, 867,
    868, 871, 856, 279, 872, 873, 45, 281, 874, 856,
    872, 873, 874, 856, 872, 874, 872, 874, 873, 881,
    886, 766, 168, 388, 279, 389, 660, 892, 388, 389,
    894, 896, 332, 349, 598, 489, 897, 898, 899, 900,
    901, 902, 897, 901, 898, 900, 899, 902, 914, 357,
    658, 916, 919, 672, 520, 838, 926, 396, 397, 927,
    928, 399, 929, 930, 527, 462, 931, 408, 932, 63,
    405, 935, 332, 598, 489, 168, 388, 389, 660, 45,
    427, 428, 429, 430, 431, 432, 433, 434, 435, 436,
    351, 937, 937, 177, 943, 268, 357, 427, 516, 838,
    395, 172, 175, 926, 944, 945, 397, 927, 398, 946,
    432, 928, 399, 400, 929, 947, 186, 462, 931, 948,
    900, 54, 949, 136, 932, 63, 427, 516, 838, 395,
    172, 175, 926, 944, 397, 927, 398, 946, 432, 928,
    399, 400, 929, 947, 462, 931, 948, 900, 54, 949,
    136, 932, 63, 357, 427, 516, 838, 395, 172, 175,
    926, 944, 397, 927, 398, 946, 432, 928, 399, 400,
    929, 947, 186, 462, 931, 948, 900, 54, 949, 136,
    932, 63, 945, 348, 643, 881, 835, 348, 643, 835,
    170, 180, 199, 963, 966, 967, 968, 408, 968, 408,
    968, 967, 971, 972, 971, 972, 168, 388, 389, 660,
    66, 980, 731, 874, 981, 332, 598, 579, 39, 62,
    197, 983, 984, 39, 197, 579, 62, 983, 984, 384,
    946, 335, 987, 988, 989, 990, 990, 989, 987, 988,
    987, 988, 995, 363, 995, 946, 339, 15, 489, 172,
    498, 999, 46, 1000, 189, 1001, 489, 498, 1000, 1001,
    498, 1001, 1001, 1000, 198, 439, 999, 1007, 182, 1009,
    1009, 1012, 161, 1013, 1013, 1015, 201, 1015, 176, 946,
    388, 1017, 389, 1017, 1021, 1022, 1023, 1024, 1025, 1026,
    1027, 17, 1029, 1029, 1034, 607, 1035, 1036, 1037, 1038,
    608, 1039, 1040, 1041, 1042, 1045, 1046, 92, 1051, 1052,
    1055, 656, 745, 1058, 984, 984, 1058, 1065, 1069, 1072,
    1073, 1074, 1075, 1076, 1077, 1078, 1079, 1080, 1081, 1082,
    19, 620, 92, 1051, 1051, 1072, 1074, 1076, 1077, 1079,
    1082, 1090, 1092, 1093, 1072, 1094, 1095, 1096, 1097, 1098,
    1099, 1073, 1100, 1101, 1102, 1103, 1104, 1108, 1111, 693,
    1127, 1128, 1129, 1130, 1131, 1132, 1133, 1134, 1135, 1137,
    1140, 894, 1143, 729, 1146, 1147, 1148, 1149, 1150, 1151,
    1152, 1153, 1154, 1156, 1157, 1158, 1159, 1160, 1161, 1162,
    1163, 1164, 1165, 1166, 1167, 1169, 1171, 525, 187, 1174,
    176, 1176, 1176, 1178, 1179, 1180, 1181, 1182, 1183, 1184,
    1185, 1186, 1187, 1187, 1185, 1186, 980, 874, 980, 168,
    388, 279, 389, 660, 45, 1192, 1193, 1194, 1199, 1200,
    1201, 1202, 1203, 1205, 1206, 1208, 1209, 1210, 1211, 1212,
    1213, 1214, 1215, 1219, 1221, 1222, 1223, 1224, 1225, 593,
    937, 699, 704, 458, 1231, 1231, 1143, 1234, 1235, 1236,
    1237, 1239, 1240, 268, 170, 180, 194, 199, 692, 693,
    380, 1251, 945, 731, 1252, 458, 731, 654, 731, 554,
    555, 731, 1263, 1264, 554, 555, 1264, 1263, 657, 551,
    543, 552, 553, 554, 555, 556, 557, 479, 558, 658,
    559, 560, 561, 661, 554, 555, 1264, 19, 684, 620,
    779, 92, 1051, 685, 439, 363, 1291, 1293, 1294, 1297,
    530, 1299, 1299, 54, 1301, 1301, 1015, 380, 1304, 1305,
    1307, 1312, 1313, 1316, 1317, 1318, 1319, 1320, 1321, 1322,
    1323, 1326, 170, 180, 194, 199, 1329, 1330, 1330, 1329,
    1334, 1335, 1336, 693, 1335, 1336, 1339, 1343, 1344, 1345,
    1343, 1345, 1344, 1350, 519, 1352, 1353, 1354, 1301, 1357,
    1322, 1358, 1358, 1354, 1362, 1363, 1364, 1365, 1366, 1367,
    1215, 1297, 1371, 1372, 1373, 1305, 1374, 1375, 351, 1376,
    1377, 1140, 1378, 380, 1371, 1372, 1376, 1378, 1215, 1373,
    1374, 1377, 1375, 1392, 1399, 764, 765, 1406, 843, 1411,
    688, 1414, 1415, 1421, 1424, 1425, 5, 995, 5, 1433,
    1435, 1439, 692, 693, 1450, 1451, 1452, 1453, 1454, 1455,
    1456, 1457, 345, 1135, 1465, 24, 26, 919, 38, 58,
    67, 51, 1466, 1466, 525, 176, 1468, 1469, 657, 1471,
    1473, 1297, 1372, 1305, 351, 1376, 1378, 1480, 1481, 1482,
    1297, 1305, 351, 1376, 1378, 1483, 1484, 14, 937, 1489,
    1490, 1491, 1492, 1493, 1494, 1074, 1495, 1496, 1497, 542,
    335, 546, 1015, 1503, 427, 429, 430, 431, 937, 430,
    431, 692, 434, 435, 436, 1511, 1512, 1424, 1425, 1520,
    1251, 1521, 707, 708, 1353, 1522, 1523, 1524, 1522, 1523,
    1524, 1531, 1532, 1533, 581, 1153, 1536, 1537, 1538, 1539,
    1540, 1541, 1542, 1543, 926, 944, 844, 1544, 1545, 1546,
    161, 704, 268, 335, 194, 150, 692, 693, 380, 150,
    692, 693, 268, 335, 194, 692, 693, 380, 469, 1549,
    1549, 1551, 1554, 1556, 730, 1557, 1558, 1559, 1560, 1561,
    1562, 1563, 1564, 1565, 1373, 1566, 1072, 1082, 1082, 1077,
    1079, 439, 363, 1072, 1073, 1074, 1075, 1076, 1077, 1078,
    1079, 1080, 1081, 1082, 363, 1076, 1573, 886, 1130, 1577,
    1581, 1582, 1583, 1584, 1589, 7, 779, 1596, 1598, 33,
    363, 678, 756, 1589, 1601, 1601, 678, 756, 1608, 186,
    766, 764, 765, 843, 845, 846, 845, 846, 1629, 1631,
    1633, 489, 1209, 779, 1194, 1209, 1209, 779, 1640, 530,
    1642, 1642, 1644, 1645, 1646, 1647, 1648, 1414, 1649, 1650,
    1651, 1652, 1251, 945, 1252, 1252, 272, 1375, 1657, 1658,
    1660, 1209, 1661, 1661, 380, 1663, 1663, 1666, 1668, 1674,
    1676, 1677, 1678, 609, 1679, 1680, 1681, 1682, 1683, 1684,
    1685, 1686, 1687, 1688, 1690, 1691, 1692, 1693, 1695, 1696,
    1697, 1698, 1699, 1700, 1701, 927, 1702, 1703, 1704, 1705,
    1706, 170, 199, 1724, 1725, 1726, 1727, 1075, 1728, 1729,
    1730, 1374, 1731, 1732, 1733, 1734, 1735, 1736, 1737, 1738,
    1739, 1740, 1741, 1215, 1373, 1377, 1377, 161, 1163, 1746,
    1747, 1753, 1755, 1756, 1758, 1759, 1761, 1762, 1763, 1766,
    1767, 1768, 1769, 928, 1770, 1771, 1772, 1773, 1774, 170,
    180, 185, 199, 268, 194, 1783, 1784, 929, 1785, 1786,
    947, 930, 1787, 1788, 1789, 1790, 1791, 1792, 1793, 1185,
    1186, 731, 1800, 1185, 1186, 1800, 1800, 1802, 1803, 1804,
    1805, 1806, 1807, 1808, 1809, 1810, 170, 171, 1811, 171,
    1811, 1811, 170, 1811, 1815, 1817, 1818, 1819, 1820, 1821,
    1822, 1823, 1824, 1825, 610, 931, 932, 397, 399, 929,
    462, 838, 928, 930, 408, 396, 405, 490, 491, 948,
    516, 838, 928, 54, 223, 1810, 188, 1489, 1690, 1841,
    346, 633, 223, 188, 398, 400, 489, 176, 944, 946,
    1165, 1180, 1706, 946, 1849, 1706, 946, 223, 715, 464,
    188, 1849, 1025, 1489, 1690, 1702, 1784, 602, 335, 1858,
    1859, 1860, 1859, 1858, 1860, 1180, 946, 175, 946, 947,
    949, 949, 1868, 1297, 1372, 1305, 1376, 1354, 1301, 1421,
    1378, 1871, 1872, 1873, 1875, 1877, 1878, 1879, 1880, 1881,
    1882, 13, 351, 1885, 1887, 1888, 1889, 1890, 1891, 1892,
    1893, 1894, 1895, 1896, 556, 363, 1898, 1899, 1900, 1901,
    1902, 1903, 54, 62, 766, 54, 62, 1907, 1910, 1913,
    1914, 1915, 235, 53, 937, 2032, 2060, 2067, 2068, 2070,
    2071, 2072, 2073, 2074, 943, 2075, 2076, 1193, 652, 650,
    2077, 2078, 161, 259, 2079, 2079, 1078, 2081, 2084, 2086,
    2087, 2089, 2090, 2091, 2092, 2093, 2094, 2095, 585, 489,
    731, 2097, 2098, 2100, 525, 946, 2103, 150, 843, 1895,
    2105, 2105, 2108, 1763, 1809, 1894, 2103, 2114, 2115, 2116,
    2117, 2118, 2119, 2120, 2121, 2122, 2123, 2124, 2125, 2126,
    2127, 2128, 2129, 2130, 2131, 2132, 2133, 2134, 2135, 2137,
    2138, 2139, 2140, 2141, 1763, 1809, 2144, 2147, 2148, 2149,
    2150, 2152, 2153, 2154, 2155, 2156, 653, 61, 2160, 2161,
    2162, 2163, 2164, 2165, 2166, 2167, 2168, 2169, 2170, 699,
    1215, 1297, 1137, 1373, 704, 1305, 1374, 351, 1291, 1377,
    1140, 268, 380, 2193, 2195, 1040, 1766, 2198, 2199, 612,
    1215, 1373, 2202, 2203, 2204, 2205, 2208, 2209, 2210, 2213,
    2215, 2218, 2219, 2220, 2221, 2222, 2223, 2224, 2225, 2226,
    2227, 2228, 2229, 2230, 2231, 2232, 2233, 2234, 2235, 2236,
    2237, 2238, 1805, 186, 2240, 2241, 2242, 2243, 2244, 2246,
    2247, 2248, 2250, 2251, 2252, 2253, 2254, 2255, 698, 741,
    1316, 1363, 1364, 1541, 1700, 1701, 1730, 1299, 1823, 1898,
    489, 181, 2098, 489, 181, 1784, 779, 439, 2270, 2272,
    2273, 2274, 2275, 2276, 2277, 629, 2278, 2279, 2280, 2281,
    2282, 613, 2283, 2284, 518, 519, 475, 520, 328, 593,
    2288, 2292, 2293, 177, 943, 2295, 2295, 2297, 2298, 2299,
    2300, 2301, 2304, 2306, 2307, 966, 2250, 2310, 2311, 2312,
    2313, 2314, 2315, 1415, 2316, 64, 2317, 2317, 2320, 2321,
    2322, 2323, 2324, 2325, 2326, 729, 730, 732, 732, 2333,
    2334, 2335, 2336, 2340, 2341, 2343, 615, 2344, 2349, 2350,
    2351, 2352, 2353, 2354, 268, 363, 268, 439, 1222, 1533,
    2362, 2363, 2364, 2365, 2366, 2367, 2368, 2369, 2370, 2371,
    2372, 2373, 2374, 2375, 2376, 2377, 2378, 2379, 2380, 2381,
    2382, 2383, 579, 983, 984, 983, 579, 984, 2386, 2387,
    2388, 2389, 2390, 2391, 1080, 1081, 2394, 1297, 1371, 1372,
    1305, 351, 1291, 1376, 1378, 2397, 351, 1858, 2410, 2411,
    2412, 2414, 2415, 2416, 2417,
];

pub static SIMPLE_HINTS: &[SimpleHintData] = &[
    SimpleHintData::empty(), // [0] reserved for no hint
    SimpleHintData { raw_values_start: 0, raw_values_count: 1, token_types_start: 1, token_types_count: 0 }, // [1]
    SimpleHintData { raw_values_start: 1, raw_values_count: 1, token_types_start: 2, token_types_count: 0 }, // [2]
    SimpleHintData { raw_values_start: 2, raw_values_count: 1, token_types_start: 3, token_types_count: 0 }, // [3]
    SimpleHintData { raw_values_start: 3, raw_values_count: 1, token_types_start: 4, token_types_count: 0 }, // [4]
    SimpleHintData { raw_values_start: 4, raw_values_count: 1, token_types_start: 5, token_types_count: 0 }, // [5]
    SimpleHintData { raw_values_start: 5, raw_values_count: 60, token_types_start: 65, token_types_count: 0 }, // [6]
    SimpleHintData { raw_values_start: 65, raw_values_count: 1, token_types_start: 66, token_types_count: 0 }, // [7]
    SimpleHintData { raw_values_start: 66, raw_values_count: 1, token_types_start: 67, token_types_count: 0 }, // [8]
    SimpleHintData { raw_values_start: 67, raw_values_count: 1, token_types_start: 68, token_types_count: 0 }, // [9]
    SimpleHintData { raw_values_start: 68, raw_values_count: 1, token_types_start: 69, token_types_count: 0 }, // [10]
    SimpleHintData { raw_values_start: 69, raw_values_count: 1, token_types_start: 70, token_types_count: 0 }, // [11]
    SimpleHintData { raw_values_start: 70, raw_values_count: 1, token_types_start: 71, token_types_count: 0 }, // [12]
    SimpleHintData { raw_values_start: 71, raw_values_count: 1, token_types_start: 72, token_types_count: 0 }, // [13]
    SimpleHintData { raw_values_start: 72, raw_values_count: 1, token_types_start: 73, token_types_count: 0 }, // [14]
    SimpleHintData { raw_values_start: 73, raw_values_count: 1, token_types_start: 74, token_types_count: 0 }, // [15]
    SimpleHintData { raw_values_start: 74, raw_values_count: 1, token_types_start: 75, token_types_count: 0 }, // [16]
    SimpleHintData { raw_values_start: 75, raw_values_count: 1, token_types_start: 76, token_types_count: 0 }, // [17]
    SimpleHintData { raw_values_start: 76, raw_values_count: 1, token_types_start: 77, token_types_count: 0 }, // [18]
    SimpleHintData { raw_values_start: 77, raw_values_count: 1, token_types_start: 78, token_types_count: 0 }, // [19]
    SimpleHintData { raw_values_start: 78, raw_values_count: 1, token_types_start: 79, token_types_count: 0 }, // [20]
    SimpleHintData { raw_values_start: 79, raw_values_count: 2, token_types_start: 81, token_types_count: 0 }, // [21]
    SimpleHintData { raw_values_start: 81, raw_values_count: 1, token_types_start: 82, token_types_count: 0 }, // [22]
    SimpleHintData { raw_values_start: 82, raw_values_count: 1, token_types_start: 83, token_types_count: 0 }, // [23]
    SimpleHintData { raw_values_start: 83, raw_values_count: 1, token_types_start: 84, token_types_count: 0 }, // [24]
    SimpleHintData { raw_values_start: 84, raw_values_count: 1, token_types_start: 85, token_types_count: 0 }, // [25]
    SimpleHintData { raw_values_start: 85, raw_values_count: 1, token_types_start: 86, token_types_count: 0 }, // [26]
    SimpleHintData { raw_values_start: 86, raw_values_count: 1, token_types_start: 87, token_types_count: 0 }, // [27]
    SimpleHintData { raw_values_start: 87, raw_values_count: 1, token_types_start: 88, token_types_count: 0 }, // [28]
    SimpleHintData { raw_values_start: 88, raw_values_count: 1, token_types_start: 89, token_types_count: 0 }, // [29]
    SimpleHintData { raw_values_start: 89, raw_values_count: 1, token_types_start: 90, token_types_count: 0 }, // [30]
    SimpleHintData { raw_values_start: 90, raw_values_count: 1, token_types_start: 91, token_types_count: 0 }, // [31]
    SimpleHintData { raw_values_start: 91, raw_values_count: 1, token_types_start: 92, token_types_count: 0 }, // [32]
    SimpleHintData { raw_values_start: 92, raw_values_count: 1, token_types_start: 93, token_types_count: 0 }, // [33]
    SimpleHintData { raw_values_start: 93, raw_values_count: 1, token_types_start: 94, token_types_count: 0 }, // [34]
    SimpleHintData { raw_values_start: 94, raw_values_count: 1, token_types_start: 95, token_types_count: 0 }, // [35]
    SimpleHintData { raw_values_start: 95, raw_values_count: 1, token_types_start: 96, token_types_count: 0 }, // [36]
    SimpleHintData { raw_values_start: 96, raw_values_count: 2, token_types_start: 98, token_types_count: 0 }, // [37]
    SimpleHintData { raw_values_start: 98, raw_values_count: 1, token_types_start: 99, token_types_count: 0 }, // [38]
    SimpleHintData { raw_values_start: 99, raw_values_count: 1, token_types_start: 100, token_types_count: 0 }, // [39]
    SimpleHintData { raw_values_start: 100, raw_values_count: 1, token_types_start: 101, token_types_count: 0 }, // [40]
    SimpleHintData { raw_values_start: 101, raw_values_count: 1, token_types_start: 102, token_types_count: 0 }, // [41]
    SimpleHintData { raw_values_start: 102, raw_values_count: 1, token_types_start: 103, token_types_count: 0 }, // [42]
    SimpleHintData { raw_values_start: 103, raw_values_count: 1, token_types_start: 104, token_types_count: 0 }, // [43]
    SimpleHintData { raw_values_start: 104, raw_values_count: 1, token_types_start: 105, token_types_count: 0 }, // [44]
    SimpleHintData { raw_values_start: 105, raw_values_count: 1, token_types_start: 106, token_types_count: 0 }, // [45]
    SimpleHintData { raw_values_start: 106, raw_values_count: 1, token_types_start: 107, token_types_count: 0 }, // [46]
    SimpleHintData { raw_values_start: 107, raw_values_count: 1, token_types_start: 108, token_types_count: 0 }, // [47]
    SimpleHintData { raw_values_start: 108, raw_values_count: 1, token_types_start: 109, token_types_count: 0 }, // [48]
    SimpleHintData { raw_values_start: 109, raw_values_count: 1, token_types_start: 110, token_types_count: 0 }, // [49]
    SimpleHintData { raw_values_start: 110, raw_values_count: 1, token_types_start: 111, token_types_count: 0 }, // [50]
    SimpleHintData { raw_values_start: 111, raw_values_count: 1, token_types_start: 112, token_types_count: 0 }, // [51]
    SimpleHintData { raw_values_start: 112, raw_values_count: 1, token_types_start: 113, token_types_count: 0 }, // [52]
    SimpleHintData { raw_values_start: 113, raw_values_count: 1, token_types_start: 114, token_types_count: 0 }, // [53]
    SimpleHintData { raw_values_start: 114, raw_values_count: 1, token_types_start: 115, token_types_count: 0 }, // [54]
    SimpleHintData { raw_values_start: 115, raw_values_count: 1, token_types_start: 116, token_types_count: 0 }, // [55]
    SimpleHintData { raw_values_start: 116, raw_values_count: 1, token_types_start: 117, token_types_count: 0 }, // [56]
    SimpleHintData { raw_values_start: 117, raw_values_count: 1, token_types_start: 118, token_types_count: 0 }, // [57]
    SimpleHintData { raw_values_start: 118, raw_values_count: 1, token_types_start: 119, token_types_count: 0 }, // [58]
    SimpleHintData { raw_values_start: 119, raw_values_count: 2, token_types_start: 121, token_types_count: 0 }, // [59]
    SimpleHintData { raw_values_start: 121, raw_values_count: 1, token_types_start: 122, token_types_count: 0 }, // [60]
    SimpleHintData { raw_values_start: 122, raw_values_count: 1, token_types_start: 123, token_types_count: 0 }, // [61]
    SimpleHintData { raw_values_start: 123, raw_values_count: 1, token_types_start: 124, token_types_count: 0 }, // [62]
    SimpleHintData { raw_values_start: 124, raw_values_count: 1, token_types_start: 125, token_types_count: 0 }, // [63]
    SimpleHintData { raw_values_start: 125, raw_values_count: 1, token_types_start: 126, token_types_count: 0 }, // [64]
    SimpleHintData { raw_values_start: 126, raw_values_count: 1, token_types_start: 127, token_types_count: 0 }, // [65]
    SimpleHintData { raw_values_start: 127, raw_values_count: 1, token_types_start: 128, token_types_count: 0 }, // [66]
    SimpleHintData { raw_values_start: 128, raw_values_count: 1, token_types_start: 129, token_types_count: 0 }, // [67]
    SimpleHintData { raw_values_start: 129, raw_values_count: 1, token_types_start: 130, token_types_count: 0 }, // [68]
    SimpleHintData { raw_values_start: 130, raw_values_count: 1, token_types_start: 131, token_types_count: 0 }, // [69]
    SimpleHintData { raw_values_start: 131, raw_values_count: 1, token_types_start: 132, token_types_count: 0 }, // [70]
    SimpleHintData { raw_values_start: 132, raw_values_count: 1, token_types_start: 133, token_types_count: 0 }, // [71]
    SimpleHintData { raw_values_start: 133, raw_values_count: 1, token_types_start: 134, token_types_count: 0 }, // [72]
    SimpleHintData { raw_values_start: 134, raw_values_count: 1, token_types_start: 135, token_types_count: 0 }, // [73]
    SimpleHintData { raw_values_start: 135, raw_values_count: 1, token_types_start: 136, token_types_count: 0 }, // [74]
    SimpleHintData { raw_values_start: 136, raw_values_count: 1, token_types_start: 137, token_types_count: 0 }, // [75]
    SimpleHintData { raw_values_start: 137, raw_values_count: 1, token_types_start: 138, token_types_count: 0 }, // [76]
    SimpleHintData { raw_values_start: 138, raw_values_count: 1, token_types_start: 139, token_types_count: 0 }, // [77]
    SimpleHintData { raw_values_start: 139, raw_values_count: 1, token_types_start: 140, token_types_count: 0 }, // [78]
    SimpleHintData { raw_values_start: 140, raw_values_count: 1, token_types_start: 141, token_types_count: 0 }, // [79]
    SimpleHintData { raw_values_start: 141, raw_values_count: 50, token_types_start: 191, token_types_count: 0 }, // [80]
    SimpleHintData { raw_values_start: 191, raw_values_count: 1, token_types_start: 192, token_types_count: 0 }, // [81]
    SimpleHintData { raw_values_start: 192, raw_values_count: 1, token_types_start: 193, token_types_count: 0 }, // [82]
    SimpleHintData { raw_values_start: 193, raw_values_count: 1, token_types_start: 194, token_types_count: 0 }, // [83]
    SimpleHintData { raw_values_start: 194, raw_values_count: 1, token_types_start: 195, token_types_count: 0 }, // [84]
    SimpleHintData { raw_values_start: 195, raw_values_count: 1, token_types_start: 196, token_types_count: 0 }, // [85]
    SimpleHintData { raw_values_start: 196, raw_values_count: 1, token_types_start: 197, token_types_count: 0 }, // [86]
    SimpleHintData { raw_values_start: 197, raw_values_count: 1, token_types_start: 198, token_types_count: 0 }, // [87]
    SimpleHintData { raw_values_start: 198, raw_values_count: 1, token_types_start: 199, token_types_count: 0 }, // [88]
    SimpleHintData { raw_values_start: 199, raw_values_count: 1, token_types_start: 200, token_types_count: 0 }, // [89]
    SimpleHintData { raw_values_start: 200, raw_values_count: 1, token_types_start: 201, token_types_count: 0 }, // [90]
    SimpleHintData { raw_values_start: 201, raw_values_count: 1, token_types_start: 202, token_types_count: 0 }, // [91]
    SimpleHintData { raw_values_start: 202, raw_values_count: 1, token_types_start: 203, token_types_count: 0 }, // [92]
    SimpleHintData { raw_values_start: 203, raw_values_count: 1, token_types_start: 204, token_types_count: 0 }, // [93]
    SimpleHintData { raw_values_start: 204, raw_values_count: 1, token_types_start: 205, token_types_count: 0 }, // [94]
    SimpleHintData { raw_values_start: 205, raw_values_count: 1, token_types_start: 206, token_types_count: 0 }, // [95]
    SimpleHintData { raw_values_start: 206, raw_values_count: 1, token_types_start: 207, token_types_count: 0 }, // [96]
    SimpleHintData { raw_values_start: 207, raw_values_count: 1, token_types_start: 208, token_types_count: 0 }, // [97]
    SimpleHintData { raw_values_start: 208, raw_values_count: 1, token_types_start: 209, token_types_count: 0 }, // [98]
    SimpleHintData { raw_values_start: 209, raw_values_count: 1, token_types_start: 210, token_types_count: 0 }, // [99]
    SimpleHintData { raw_values_start: 210, raw_values_count: 1, token_types_start: 211, token_types_count: 0 }, // [100]
    SimpleHintData { raw_values_start: 211, raw_values_count: 1, token_types_start: 212, token_types_count: 0 }, // [101]
    SimpleHintData { raw_values_start: 212, raw_values_count: 1, token_types_start: 213, token_types_count: 0 }, // [102]
    SimpleHintData { raw_values_start: 213, raw_values_count: 1, token_types_start: 214, token_types_count: 0 }, // [103]
    SimpleHintData { raw_values_start: 214, raw_values_count: 1, token_types_start: 215, token_types_count: 0 }, // [104]
    SimpleHintData { raw_values_start: 215, raw_values_count: 1, token_types_start: 216, token_types_count: 0 }, // [105]
    SimpleHintData { raw_values_start: 216, raw_values_count: 1, token_types_start: 217, token_types_count: 0 }, // [106]
    SimpleHintData { raw_values_start: 217, raw_values_count: 1, token_types_start: 218, token_types_count: 0 }, // [107]
    SimpleHintData { raw_values_start: 218, raw_values_count: 1, token_types_start: 219, token_types_count: 0 }, // [108]
    SimpleHintData { raw_values_start: 219, raw_values_count: 1, token_types_start: 220, token_types_count: 0 }, // [109]
    SimpleHintData { raw_values_start: 220, raw_values_count: 1, token_types_start: 221, token_types_count: 0 }, // [110]
    SimpleHintData { raw_values_start: 221, raw_values_count: 1, token_types_start: 222, token_types_count: 0 }, // [111]
    SimpleHintData { raw_values_start: 222, raw_values_count: 1, token_types_start: 223, token_types_count: 0 }, // [112]
    SimpleHintData { raw_values_start: 223, raw_values_count: 2, token_types_start: 225, token_types_count: 0 }, // [113]
    SimpleHintData { raw_values_start: 225, raw_values_count: 1, token_types_start: 226, token_types_count: 0 }, // [114]
    SimpleHintData { raw_values_start: 226, raw_values_count: 1, token_types_start: 227, token_types_count: 0 }, // [115]
    SimpleHintData { raw_values_start: 227, raw_values_count: 1, token_types_start: 228, token_types_count: 0 }, // [116]
    SimpleHintData { raw_values_start: 228, raw_values_count: 1, token_types_start: 229, token_types_count: 0 }, // [117]
    SimpleHintData { raw_values_start: 229, raw_values_count: 1, token_types_start: 230, token_types_count: 0 }, // [118]
    SimpleHintData { raw_values_start: 230, raw_values_count: 1, token_types_start: 231, token_types_count: 0 }, // [119]
    SimpleHintData { raw_values_start: 231, raw_values_count: 1, token_types_start: 232, token_types_count: 0 }, // [120]
    SimpleHintData { raw_values_start: 232, raw_values_count: 1, token_types_start: 233, token_types_count: 0 }, // [121]
    SimpleHintData { raw_values_start: 233, raw_values_count: 1, token_types_start: 234, token_types_count: 0 }, // [122]
    SimpleHintData { raw_values_start: 234, raw_values_count: 1, token_types_start: 235, token_types_count: 0 }, // [123]
    SimpleHintData { raw_values_start: 235, raw_values_count: 1, token_types_start: 236, token_types_count: 0 }, // [124]
    SimpleHintData { raw_values_start: 236, raw_values_count: 1, token_types_start: 237, token_types_count: 0 }, // [125]
    SimpleHintData { raw_values_start: 237, raw_values_count: 1, token_types_start: 238, token_types_count: 0 }, // [126]
    SimpleHintData { raw_values_start: 238, raw_values_count: 1, token_types_start: 239, token_types_count: 0 }, // [127]
    SimpleHintData { raw_values_start: 239, raw_values_count: 1, token_types_start: 240, token_types_count: 0 }, // [128]
    SimpleHintData { raw_values_start: 240, raw_values_count: 1, token_types_start: 241, token_types_count: 0 }, // [129]
    SimpleHintData { raw_values_start: 241, raw_values_count: 2, token_types_start: 243, token_types_count: 0 }, // [130]
    SimpleHintData { raw_values_start: 243, raw_values_count: 1, token_types_start: 244, token_types_count: 0 }, // [131]
    SimpleHintData { raw_values_start: 244, raw_values_count: 1, token_types_start: 245, token_types_count: 0 }, // [132]
    SimpleHintData { raw_values_start: 245, raw_values_count: 1, token_types_start: 246, token_types_count: 0 }, // [133]
    SimpleHintData { raw_values_start: 246, raw_values_count: 1, token_types_start: 247, token_types_count: 0 }, // [134]
    SimpleHintData { raw_values_start: 247, raw_values_count: 1, token_types_start: 248, token_types_count: 0 }, // [135]
    SimpleHintData { raw_values_start: 248, raw_values_count: 14, token_types_start: 262, token_types_count: 0 }, // [136]
    SimpleHintData { raw_values_start: 262, raw_values_count: 1, token_types_start: 263, token_types_count: 0 }, // [137]
    SimpleHintData { raw_values_start: 263, raw_values_count: 1, token_types_start: 264, token_types_count: 0 }, // [138]
    SimpleHintData { raw_values_start: 264, raw_values_count: 1, token_types_start: 265, token_types_count: 0 }, // [139]
    SimpleHintData { raw_values_start: 265, raw_values_count: 1, token_types_start: 266, token_types_count: 0 }, // [140]
    SimpleHintData { raw_values_start: 266, raw_values_count: 1, token_types_start: 267, token_types_count: 0 }, // [141]
    SimpleHintData { raw_values_start: 267, raw_values_count: 1, token_types_start: 268, token_types_count: 0 }, // [142]
    SimpleHintData { raw_values_start: 268, raw_values_count: 1, token_types_start: 269, token_types_count: 0 }, // [143]
    SimpleHintData { raw_values_start: 269, raw_values_count: 1, token_types_start: 270, token_types_count: 0 }, // [144]
    SimpleHintData { raw_values_start: 270, raw_values_count: 1, token_types_start: 271, token_types_count: 0 }, // [145]
    SimpleHintData { raw_values_start: 271, raw_values_count: 14, token_types_start: 285, token_types_count: 0 }, // [146]
    SimpleHintData { raw_values_start: 285, raw_values_count: 1, token_types_start: 286, token_types_count: 0 }, // [147]
    SimpleHintData { raw_values_start: 286, raw_values_count: 1, token_types_start: 287, token_types_count: 0 }, // [148]
    SimpleHintData { raw_values_start: 287, raw_values_count: 1, token_types_start: 288, token_types_count: 0 }, // [149]
    SimpleHintData { raw_values_start: 288, raw_values_count: 1, token_types_start: 289, token_types_count: 0 }, // [150]
    SimpleHintData { raw_values_start: 289, raw_values_count: 1, token_types_start: 290, token_types_count: 0 }, // [151]
    SimpleHintData { raw_values_start: 290, raw_values_count: 1, token_types_start: 291, token_types_count: 0 }, // [152]
    SimpleHintData { raw_values_start: 291, raw_values_count: 1, token_types_start: 292, token_types_count: 0 }, // [153]
    SimpleHintData { raw_values_start: 292, raw_values_count: 1, token_types_start: 293, token_types_count: 0 }, // [154]
    SimpleHintData { raw_values_start: 293, raw_values_count: 1, token_types_start: 294, token_types_count: 0 }, // [155]
    SimpleHintData { raw_values_start: 294, raw_values_count: 1, token_types_start: 295, token_types_count: 0 }, // [156]
    SimpleHintData { raw_values_start: 295, raw_values_count: 2, token_types_start: 297, token_types_count: 0 }, // [157]
    SimpleHintData { raw_values_start: 297, raw_values_count: 1, token_types_start: 298, token_types_count: 0 }, // [158]
    SimpleHintData { raw_values_start: 298, raw_values_count: 3, token_types_start: 301, token_types_count: 0 }, // [159]
    SimpleHintData { raw_values_start: 301, raw_values_count: 1, token_types_start: 302, token_types_count: 0 }, // [160]
    SimpleHintData { raw_values_start: 302, raw_values_count: 1, token_types_start: 303, token_types_count: 0 }, // [161]
    SimpleHintData { raw_values_start: 303, raw_values_count: 1, token_types_start: 304, token_types_count: 0 }, // [162]
    SimpleHintData { raw_values_start: 304, raw_values_count: 1, token_types_start: 305, token_types_count: 0 }, // [163]
    SimpleHintData { raw_values_start: 305, raw_values_count: 5, token_types_start: 310, token_types_count: 0 }, // [164]
    SimpleHintData { raw_values_start: 310, raw_values_count: 1, token_types_start: 311, token_types_count: 0 }, // [165]
    SimpleHintData { raw_values_start: 311, raw_values_count: 1, token_types_start: 312, token_types_count: 0 }, // [166]
    SimpleHintData { raw_values_start: 312, raw_values_count: 1, token_types_start: 313, token_types_count: 0 }, // [167]
    SimpleHintData { raw_values_start: 313, raw_values_count: 1, token_types_start: 314, token_types_count: 0 }, // [168]
    SimpleHintData { raw_values_start: 314, raw_values_count: 1, token_types_start: 315, token_types_count: 0 }, // [169]
    SimpleHintData { raw_values_start: 315, raw_values_count: 1, token_types_start: 316, token_types_count: 0 }, // [170]
    SimpleHintData { raw_values_start: 316, raw_values_count: 1, token_types_start: 317, token_types_count: 0 }, // [171]
    SimpleHintData { raw_values_start: 317, raw_values_count: 1, token_types_start: 318, token_types_count: 0 }, // [172]
    SimpleHintData { raw_values_start: 318, raw_values_count: 1, token_types_start: 319, token_types_count: 0 }, // [173]
    SimpleHintData { raw_values_start: 319, raw_values_count: 1, token_types_start: 320, token_types_count: 0 }, // [174]
    SimpleHintData { raw_values_start: 320, raw_values_count: 1, token_types_start: 321, token_types_count: 0 }, // [175]
    SimpleHintData { raw_values_start: 321, raw_values_count: 1, token_types_start: 322, token_types_count: 0 }, // [176]
    SimpleHintData { raw_values_start: 322, raw_values_count: 1, token_types_start: 323, token_types_count: 0 }, // [177]
    SimpleHintData { raw_values_start: 323, raw_values_count: 1, token_types_start: 324, token_types_count: 0 }, // [178]
    SimpleHintData { raw_values_start: 324, raw_values_count: 0, token_types_start: 324, token_types_count: 1 }, // [179]
    SimpleHintData { raw_values_start: 325, raw_values_count: 1, token_types_start: 326, token_types_count: 0 }, // [180]
    SimpleHintData { raw_values_start: 326, raw_values_count: 1, token_types_start: 327, token_types_count: 0 }, // [181]
    SimpleHintData { raw_values_start: 327, raw_values_count: 1, token_types_start: 328, token_types_count: 0 }, // [182]
    SimpleHintData { raw_values_start: 328, raw_values_count: 1, token_types_start: 329, token_types_count: 0 }, // [183]
    SimpleHintData { raw_values_start: 329, raw_values_count: 1, token_types_start: 330, token_types_count: 0 }, // [184]
    SimpleHintData { raw_values_start: 330, raw_values_count: 3, token_types_start: 333, token_types_count: 0 }, // [185]
    SimpleHintData { raw_values_start: 333, raw_values_count: 2, token_types_start: 335, token_types_count: 0 }, // [186]
    SimpleHintData { raw_values_start: 335, raw_values_count: 1, token_types_start: 336, token_types_count: 0 }, // [187]
    SimpleHintData { raw_values_start: 336, raw_values_count: 1, token_types_start: 337, token_types_count: 0 }, // [188]
    SimpleHintData { raw_values_start: 337, raw_values_count: 2, token_types_start: 339, token_types_count: 0 }, // [189]
    SimpleHintData { raw_values_start: 339, raw_values_count: 1, token_types_start: 340, token_types_count: 0 }, // [190]
    SimpleHintData { raw_values_start: 340, raw_values_count: 2, token_types_start: 342, token_types_count: 0 }, // [191]
    SimpleHintData { raw_values_start: 342, raw_values_count: 3, token_types_start: 345, token_types_count: 0 }, // [192]
    SimpleHintData { raw_values_start: 345, raw_values_count: 1, token_types_start: 346, token_types_count: 0 }, // [193]
    SimpleHintData { raw_values_start: 346, raw_values_count: 1, token_types_start: 347, token_types_count: 0 }, // [194]
    SimpleHintData { raw_values_start: 347, raw_values_count: 1, token_types_start: 348, token_types_count: 0 }, // [195]
    SimpleHintData { raw_values_start: 348, raw_values_count: 13, token_types_start: 361, token_types_count: 0 }, // [196]
    SimpleHintData { raw_values_start: 361, raw_values_count: 1, token_types_start: 362, token_types_count: 0 }, // [197]
    SimpleHintData { raw_values_start: 362, raw_values_count: 2, token_types_start: 364, token_types_count: 0 }, // [198]
    SimpleHintData { raw_values_start: 364, raw_values_count: 1, token_types_start: 365, token_types_count: 0 }, // [199]
    SimpleHintData { raw_values_start: 365, raw_values_count: 1, token_types_start: 366, token_types_count: 0 }, // [200]
    SimpleHintData { raw_values_start: 366, raw_values_count: 1, token_types_start: 367, token_types_count: 0 }, // [201]
    SimpleHintData { raw_values_start: 367, raw_values_count: 1, token_types_start: 368, token_types_count: 0 }, // [202]
    SimpleHintData { raw_values_start: 368, raw_values_count: 1, token_types_start: 369, token_types_count: 0 }, // [203]
    SimpleHintData { raw_values_start: 369, raw_values_count: 1, token_types_start: 370, token_types_count: 0 }, // [204]
    SimpleHintData { raw_values_start: 370, raw_values_count: 1, token_types_start: 371, token_types_count: 0 }, // [205]
    SimpleHintData { raw_values_start: 371, raw_values_count: 1, token_types_start: 372, token_types_count: 0 }, // [206]
    SimpleHintData { raw_values_start: 372, raw_values_count: 1, token_types_start: 373, token_types_count: 0 }, // [207]
    SimpleHintData { raw_values_start: 373, raw_values_count: 1, token_types_start: 374, token_types_count: 0 }, // [208]
    SimpleHintData { raw_values_start: 374, raw_values_count: 1, token_types_start: 375, token_types_count: 0 }, // [209]
    SimpleHintData { raw_values_start: 375, raw_values_count: 1, token_types_start: 376, token_types_count: 0 }, // [210]
    SimpleHintData { raw_values_start: 376, raw_values_count: 1, token_types_start: 377, token_types_count: 0 }, // [211]
    SimpleHintData { raw_values_start: 377, raw_values_count: 11, token_types_start: 388, token_types_count: 0 }, // [212]
    SimpleHintData { raw_values_start: 388, raw_values_count: 1, token_types_start: 389, token_types_count: 0 }, // [213]
    SimpleHintData { raw_values_start: 389, raw_values_count: 0, token_types_start: 389, token_types_count: 1 }, // [214]
    SimpleHintData { raw_values_start: 390, raw_values_count: 2, token_types_start: 392, token_types_count: 0 }, // [215]
    SimpleHintData { raw_values_start: 392, raw_values_count: 2, token_types_start: 394, token_types_count: 0 }, // [216]
    SimpleHintData { raw_values_start: 394, raw_values_count: 2, token_types_start: 396, token_types_count: 0 }, // [217]
    SimpleHintData { raw_values_start: 396, raw_values_count: 1, token_types_start: 397, token_types_count: 0 }, // [218]
    SimpleHintData { raw_values_start: 397, raw_values_count: 1, token_types_start: 398, token_types_count: 0 }, // [219]
    SimpleHintData { raw_values_start: 398, raw_values_count: 2, token_types_start: 400, token_types_count: 0 }, // [220]
    SimpleHintData { raw_values_start: 400, raw_values_count: 1, token_types_start: 401, token_types_count: 0 }, // [221]
    SimpleHintData { raw_values_start: 401, raw_values_count: 1, token_types_start: 402, token_types_count: 0 }, // [222]
    SimpleHintData { raw_values_start: 402, raw_values_count: 2, token_types_start: 404, token_types_count: 0 }, // [223]
    SimpleHintData { raw_values_start: 404, raw_values_count: 7, token_types_start: 411, token_types_count: 0 }, // [224]
    SimpleHintData { raw_values_start: 411, raw_values_count: 3, token_types_start: 414, token_types_count: 0 }, // [225]
    SimpleHintData { raw_values_start: 414, raw_values_count: 1, token_types_start: 415, token_types_count: 0 }, // [226]
    SimpleHintData { raw_values_start: 415, raw_values_count: 1, token_types_start: 416, token_types_count: 0 }, // [227]
    SimpleHintData { raw_values_start: 416, raw_values_count: 2, token_types_start: 418, token_types_count: 0 }, // [228]
    SimpleHintData { raw_values_start: 418, raw_values_count: 1, token_types_start: 419, token_types_count: 0 }, // [229]
    SimpleHintData { raw_values_start: 419, raw_values_count: 1, token_types_start: 420, token_types_count: 0 }, // [230]
    SimpleHintData { raw_values_start: 420, raw_values_count: 1, token_types_start: 421, token_types_count: 0 }, // [231]
    SimpleHintData { raw_values_start: 421, raw_values_count: 3, token_types_start: 424, token_types_count: 0 }, // [232]
    SimpleHintData { raw_values_start: 424, raw_values_count: 1, token_types_start: 425, token_types_count: 0 }, // [233]
    SimpleHintData { raw_values_start: 425, raw_values_count: 1, token_types_start: 426, token_types_count: 0 }, // [234]
    SimpleHintData { raw_values_start: 426, raw_values_count: 1, token_types_start: 427, token_types_count: 0 }, // [235]
    SimpleHintData { raw_values_start: 427, raw_values_count: 1, token_types_start: 428, token_types_count: 0 }, // [236]
    SimpleHintData { raw_values_start: 428, raw_values_count: 1, token_types_start: 429, token_types_count: 0 }, // [237]
    SimpleHintData { raw_values_start: 429, raw_values_count: 1, token_types_start: 430, token_types_count: 0 }, // [238]
    SimpleHintData { raw_values_start: 430, raw_values_count: 1, token_types_start: 431, token_types_count: 0 }, // [239]
    SimpleHintData { raw_values_start: 431, raw_values_count: 1, token_types_start: 432, token_types_count: 0 }, // [240]
    SimpleHintData { raw_values_start: 432, raw_values_count: 1, token_types_start: 433, token_types_count: 0 }, // [241]
    SimpleHintData { raw_values_start: 433, raw_values_count: 2, token_types_start: 435, token_types_count: 0 }, // [242]
    SimpleHintData { raw_values_start: 435, raw_values_count: 1, token_types_start: 436, token_types_count: 0 }, // [243]
    SimpleHintData { raw_values_start: 436, raw_values_count: 1, token_types_start: 437, token_types_count: 0 }, // [244]
    SimpleHintData { raw_values_start: 437, raw_values_count: 2, token_types_start: 439, token_types_count: 0 }, // [245]
    SimpleHintData { raw_values_start: 439, raw_values_count: 1, token_types_start: 440, token_types_count: 0 }, // [246]
    SimpleHintData { raw_values_start: 440, raw_values_count: 1, token_types_start: 441, token_types_count: 0 }, // [247]
    SimpleHintData { raw_values_start: 441, raw_values_count: 3, token_types_start: 444, token_types_count: 0 }, // [248]
    SimpleHintData { raw_values_start: 444, raw_values_count: 1, token_types_start: 445, token_types_count: 0 }, // [249]
    SimpleHintData { raw_values_start: 445, raw_values_count: 1, token_types_start: 446, token_types_count: 0 }, // [250]
    SimpleHintData { raw_values_start: 446, raw_values_count: 1, token_types_start: 447, token_types_count: 0 }, // [251]
    SimpleHintData { raw_values_start: 447, raw_values_count: 1, token_types_start: 448, token_types_count: 0 }, // [252]
    SimpleHintData { raw_values_start: 448, raw_values_count: 2, token_types_start: 450, token_types_count: 1 }, // [253]
    SimpleHintData { raw_values_start: 451, raw_values_count: 1, token_types_start: 452, token_types_count: 0 }, // [254]
    SimpleHintData { raw_values_start: 452, raw_values_count: 1, token_types_start: 453, token_types_count: 0 }, // [255]
    SimpleHintData { raw_values_start: 453, raw_values_count: 1, token_types_start: 454, token_types_count: 0 }, // [256]
    SimpleHintData { raw_values_start: 454, raw_values_count: 1, token_types_start: 455, token_types_count: 0 }, // [257]
    SimpleHintData { raw_values_start: 455, raw_values_count: 2, token_types_start: 457, token_types_count: 0 }, // [258]
    SimpleHintData { raw_values_start: 457, raw_values_count: 1, token_types_start: 458, token_types_count: 0 }, // [259]
    SimpleHintData { raw_values_start: 458, raw_values_count: 4, token_types_start: 462, token_types_count: 0 }, // [260]
    SimpleHintData { raw_values_start: 462, raw_values_count: 2, token_types_start: 464, token_types_count: 0 }, // [261]
    SimpleHintData { raw_values_start: 464, raw_values_count: 1, token_types_start: 465, token_types_count: 0 }, // [262]
    SimpleHintData { raw_values_start: 465, raw_values_count: 2, token_types_start: 467, token_types_count: 0 }, // [263]
    SimpleHintData { raw_values_start: 467, raw_values_count: 1, token_types_start: 468, token_types_count: 0 }, // [264]
    SimpleHintData { raw_values_start: 468, raw_values_count: 3, token_types_start: 471, token_types_count: 0 }, // [265]
    SimpleHintData { raw_values_start: 471, raw_values_count: 6, token_types_start: 477, token_types_count: 0 }, // [266]
    SimpleHintData { raw_values_start: 477, raw_values_count: 5, token_types_start: 482, token_types_count: 0 }, // [267]
    SimpleHintData { raw_values_start: 482, raw_values_count: 3, token_types_start: 485, token_types_count: 0 }, // [268]
    SimpleHintData { raw_values_start: 485, raw_values_count: 1, token_types_start: 486, token_types_count: 0 }, // [269]
    SimpleHintData { raw_values_start: 486, raw_values_count: 1, token_types_start: 487, token_types_count: 0 }, // [270]
    SimpleHintData { raw_values_start: 487, raw_values_count: 2, token_types_start: 489, token_types_count: 0 }, // [271]
    SimpleHintData { raw_values_start: 489, raw_values_count: 1, token_types_start: 490, token_types_count: 0 }, // [272]
    SimpleHintData { raw_values_start: 490, raw_values_count: 3, token_types_start: 493, token_types_count: 0 }, // [273]
    SimpleHintData { raw_values_start: 493, raw_values_count: 2, token_types_start: 495, token_types_count: 0 }, // [274]
    SimpleHintData { raw_values_start: 495, raw_values_count: 1, token_types_start: 496, token_types_count: 0 }, // [275]
    SimpleHintData { raw_values_start: 496, raw_values_count: 2, token_types_start: 498, token_types_count: 0 }, // [276]
    SimpleHintData { raw_values_start: 498, raw_values_count: 6, token_types_start: 504, token_types_count: 0 }, // [277]
    SimpleHintData { raw_values_start: 504, raw_values_count: 1, token_types_start: 505, token_types_count: 0 }, // [278]
    SimpleHintData { raw_values_start: 505, raw_values_count: 1, token_types_start: 506, token_types_count: 0 }, // [279]
    SimpleHintData { raw_values_start: 506, raw_values_count: 1, token_types_start: 507, token_types_count: 0 }, // [280]
    SimpleHintData { raw_values_start: 507, raw_values_count: 1, token_types_start: 508, token_types_count: 0 }, // [281]
    SimpleHintData { raw_values_start: 508, raw_values_count: 1, token_types_start: 509, token_types_count: 0 }, // [282]
    SimpleHintData { raw_values_start: 509, raw_values_count: 1, token_types_start: 510, token_types_count: 0 }, // [283]
    SimpleHintData { raw_values_start: 510, raw_values_count: 12, token_types_start: 522, token_types_count: 0 }, // [284]
    SimpleHintData { raw_values_start: 522, raw_values_count: 1, token_types_start: 523, token_types_count: 0 }, // [285]
    SimpleHintData { raw_values_start: 523, raw_values_count: 1, token_types_start: 524, token_types_count: 0 }, // [286]
    SimpleHintData { raw_values_start: 524, raw_values_count: 1, token_types_start: 525, token_types_count: 0 }, // [287]
    SimpleHintData { raw_values_start: 525, raw_values_count: 1, token_types_start: 526, token_types_count: 0 }, // [288]
    SimpleHintData { raw_values_start: 526, raw_values_count: 1, token_types_start: 527, token_types_count: 0 }, // [289]
    SimpleHintData { raw_values_start: 527, raw_values_count: 1, token_types_start: 528, token_types_count: 0 }, // [290]
    SimpleHintData { raw_values_start: 528, raw_values_count: 1, token_types_start: 529, token_types_count: 0 }, // [291]
    SimpleHintData { raw_values_start: 529, raw_values_count: 1, token_types_start: 530, token_types_count: 0 }, // [292]
    SimpleHintData { raw_values_start: 530, raw_values_count: 1, token_types_start: 531, token_types_count: 0 }, // [293]
    SimpleHintData { raw_values_start: 531, raw_values_count: 1, token_types_start: 532, token_types_count: 0 }, // [294]
    SimpleHintData { raw_values_start: 532, raw_values_count: 1, token_types_start: 533, token_types_count: 0 }, // [295]
    SimpleHintData { raw_values_start: 533, raw_values_count: 1, token_types_start: 534, token_types_count: 0 }, // [296]
    SimpleHintData { raw_values_start: 534, raw_values_count: 1, token_types_start: 535, token_types_count: 0 }, // [297]
    SimpleHintData { raw_values_start: 535, raw_values_count: 1, token_types_start: 536, token_types_count: 0 }, // [298]
    SimpleHintData { raw_values_start: 536, raw_values_count: 1, token_types_start: 537, token_types_count: 0 }, // [299]
    SimpleHintData { raw_values_start: 537, raw_values_count: 1, token_types_start: 538, token_types_count: 0 }, // [300]
    SimpleHintData { raw_values_start: 538, raw_values_count: 1, token_types_start: 539, token_types_count: 0 }, // [301]
    SimpleHintData { raw_values_start: 539, raw_values_count: 1, token_types_start: 540, token_types_count: 0 }, // [302]
    SimpleHintData { raw_values_start: 540, raw_values_count: 1, token_types_start: 541, token_types_count: 0 }, // [303]
    SimpleHintData { raw_values_start: 541, raw_values_count: 2, token_types_start: 543, token_types_count: 0 }, // [304]
    SimpleHintData { raw_values_start: 543, raw_values_count: 2, token_types_start: 545, token_types_count: 0 }, // [305]
    SimpleHintData { raw_values_start: 545, raw_values_count: 1, token_types_start: 546, token_types_count: 0 }, // [306]
    SimpleHintData { raw_values_start: 546, raw_values_count: 1, token_types_start: 547, token_types_count: 0 }, // [307]
    SimpleHintData { raw_values_start: 547, raw_values_count: 1, token_types_start: 548, token_types_count: 0 }, // [308]
    SimpleHintData { raw_values_start: 548, raw_values_count: 1, token_types_start: 549, token_types_count: 0 }, // [309]
    SimpleHintData { raw_values_start: 549, raw_values_count: 1, token_types_start: 550, token_types_count: 0 }, // [310]
    SimpleHintData { raw_values_start: 550, raw_values_count: 1, token_types_start: 551, token_types_count: 0 }, // [311]
    SimpleHintData { raw_values_start: 551, raw_values_count: 1, token_types_start: 552, token_types_count: 0 }, // [312]
    SimpleHintData { raw_values_start: 552, raw_values_count: 1, token_types_start: 553, token_types_count: 0 }, // [313]
    SimpleHintData { raw_values_start: 553, raw_values_count: 1, token_types_start: 554, token_types_count: 0 }, // [314]
    SimpleHintData { raw_values_start: 554, raw_values_count: 1, token_types_start: 555, token_types_count: 0 }, // [315]
    SimpleHintData { raw_values_start: 555, raw_values_count: 1, token_types_start: 556, token_types_count: 0 }, // [316]
    SimpleHintData { raw_values_start: 556, raw_values_count: 1, token_types_start: 557, token_types_count: 0 }, // [317]
    SimpleHintData { raw_values_start: 557, raw_values_count: 1, token_types_start: 558, token_types_count: 0 }, // [318]
    SimpleHintData { raw_values_start: 558, raw_values_count: 12, token_types_start: 570, token_types_count: 0 }, // [319]
    SimpleHintData { raw_values_start: 570, raw_values_count: 1, token_types_start: 571, token_types_count: 0 }, // [320]
    SimpleHintData { raw_values_start: 571, raw_values_count: 1, token_types_start: 572, token_types_count: 0 }, // [321]
    SimpleHintData { raw_values_start: 572, raw_values_count: 1, token_types_start: 573, token_types_count: 1 }, // [322]
    SimpleHintData { raw_values_start: 574, raw_values_count: 1, token_types_start: 575, token_types_count: 0 }, // [323]
    SimpleHintData { raw_values_start: 575, raw_values_count: 1, token_types_start: 576, token_types_count: 0 }, // [324]
    SimpleHintData { raw_values_start: 576, raw_values_count: 1, token_types_start: 577, token_types_count: 0 }, // [325]
    SimpleHintData { raw_values_start: 577, raw_values_count: 1, token_types_start: 578, token_types_count: 0 }, // [326]
    SimpleHintData { raw_values_start: 578, raw_values_count: 1, token_types_start: 579, token_types_count: 0 }, // [327]
    SimpleHintData { raw_values_start: 579, raw_values_count: 1, token_types_start: 580, token_types_count: 0 }, // [328]
    SimpleHintData { raw_values_start: 580, raw_values_count: 1, token_types_start: 581, token_types_count: 0 }, // [329]
    SimpleHintData { raw_values_start: 581, raw_values_count: 1, token_types_start: 582, token_types_count: 0 }, // [330]
    SimpleHintData { raw_values_start: 582, raw_values_count: 1, token_types_start: 583, token_types_count: 0 }, // [331]
    SimpleHintData { raw_values_start: 583, raw_values_count: 1, token_types_start: 584, token_types_count: 0 }, // [332]
    SimpleHintData { raw_values_start: 584, raw_values_count: 18, token_types_start: 602, token_types_count: 1 }, // [333]
    SimpleHintData { raw_values_start: 603, raw_values_count: 2, token_types_start: 605, token_types_count: 0 }, // [334]
    SimpleHintData { raw_values_start: 605, raw_values_count: 5, token_types_start: 610, token_types_count: 1 }, // [335]
    SimpleHintData { raw_values_start: 611, raw_values_count: 1, token_types_start: 612, token_types_count: 0 }, // [336]
    SimpleHintData { raw_values_start: 612, raw_values_count: 1, token_types_start: 613, token_types_count: 0 }, // [337]
    SimpleHintData { raw_values_start: 613, raw_values_count: 1, token_types_start: 614, token_types_count: 0 }, // [338]
    SimpleHintData { raw_values_start: 614, raw_values_count: 1, token_types_start: 615, token_types_count: 0 }, // [339]
    SimpleHintData { raw_values_start: 615, raw_values_count: 1, token_types_start: 616, token_types_count: 0 }, // [340]
    SimpleHintData { raw_values_start: 616, raw_values_count: 1, token_types_start: 617, token_types_count: 0 }, // [341]
    SimpleHintData { raw_values_start: 617, raw_values_count: 1, token_types_start: 618, token_types_count: 0 }, // [342]
    SimpleHintData { raw_values_start: 618, raw_values_count: 1, token_types_start: 619, token_types_count: 0 }, // [343]
    SimpleHintData { raw_values_start: 619, raw_values_count: 1, token_types_start: 620, token_types_count: 0 }, // [344]
    SimpleHintData { raw_values_start: 620, raw_values_count: 1, token_types_start: 621, token_types_count: 0 }, // [345]
    SimpleHintData { raw_values_start: 621, raw_values_count: 1, token_types_start: 622, token_types_count: 0 }, // [346]
    SimpleHintData { raw_values_start: 622, raw_values_count: 2, token_types_start: 624, token_types_count: 0 }, // [347]
    SimpleHintData { raw_values_start: 624, raw_values_count: 1, token_types_start: 625, token_types_count: 0 }, // [348]
    SimpleHintData { raw_values_start: 625, raw_values_count: 1, token_types_start: 626, token_types_count: 0 }, // [349]
    SimpleHintData { raw_values_start: 626, raw_values_count: 1, token_types_start: 627, token_types_count: 0 }, // [350]
    SimpleHintData { raw_values_start: 627, raw_values_count: 4, token_types_start: 631, token_types_count: 0 }, // [351]
    SimpleHintData { raw_values_start: 631, raw_values_count: 1, token_types_start: 632, token_types_count: 0 }, // [352]
    SimpleHintData { raw_values_start: 632, raw_values_count: 1, token_types_start: 633, token_types_count: 0 }, // [353]
    SimpleHintData { raw_values_start: 633, raw_values_count: 1, token_types_start: 634, token_types_count: 0 }, // [354]
    SimpleHintData { raw_values_start: 634, raw_values_count: 1, token_types_start: 635, token_types_count: 0 }, // [355]
    SimpleHintData { raw_values_start: 635, raw_values_count: 1, token_types_start: 636, token_types_count: 0 }, // [356]
    SimpleHintData { raw_values_start: 636, raw_values_count: 1, token_types_start: 637, token_types_count: 0 }, // [357]
    SimpleHintData { raw_values_start: 637, raw_values_count: 1, token_types_start: 638, token_types_count: 0 }, // [358]
    SimpleHintData { raw_values_start: 638, raw_values_count: 1, token_types_start: 639, token_types_count: 0 }, // [359]
    SimpleHintData { raw_values_start: 639, raw_values_count: 1, token_types_start: 640, token_types_count: 0 }, // [360]
    SimpleHintData { raw_values_start: 640, raw_values_count: 1, token_types_start: 641, token_types_count: 0 }, // [361]
    SimpleHintData { raw_values_start: 641, raw_values_count: 5, token_types_start: 646, token_types_count: 0 }, // [362]
    SimpleHintData { raw_values_start: 646, raw_values_count: 1, token_types_start: 647, token_types_count: 0 }, // [363]
    SimpleHintData { raw_values_start: 647, raw_values_count: 1, token_types_start: 648, token_types_count: 0 }, // [364]
    SimpleHintData { raw_values_start: 648, raw_values_count: 1, token_types_start: 649, token_types_count: 0 }, // [365]
    SimpleHintData { raw_values_start: 649, raw_values_count: 1, token_types_start: 650, token_types_count: 0 }, // [366]
    SimpleHintData { raw_values_start: 650, raw_values_count: 1, token_types_start: 651, token_types_count: 0 }, // [367]
    SimpleHintData { raw_values_start: 651, raw_values_count: 1, token_types_start: 652, token_types_count: 0 }, // [368]
    SimpleHintData { raw_values_start: 652, raw_values_count: 1, token_types_start: 653, token_types_count: 0 }, // [369]
    SimpleHintData { raw_values_start: 653, raw_values_count: 1, token_types_start: 654, token_types_count: 0 }, // [370]
    SimpleHintData { raw_values_start: 654, raw_values_count: 4, token_types_start: 658, token_types_count: 0 }, // [371]
    SimpleHintData { raw_values_start: 658, raw_values_count: 1, token_types_start: 659, token_types_count: 0 }, // [372]
    SimpleHintData { raw_values_start: 659, raw_values_count: 1, token_types_start: 660, token_types_count: 0 }, // [373]
    SimpleHintData { raw_values_start: 660, raw_values_count: 1, token_types_start: 661, token_types_count: 0 }, // [374]
    SimpleHintData { raw_values_start: 661, raw_values_count: 1, token_types_start: 662, token_types_count: 0 }, // [375]
    SimpleHintData { raw_values_start: 662, raw_values_count: 1, token_types_start: 663, token_types_count: 0 }, // [376]
    SimpleHintData { raw_values_start: 663, raw_values_count: 1, token_types_start: 664, token_types_count: 0 }, // [377]
    SimpleHintData { raw_values_start: 664, raw_values_count: 1, token_types_start: 665, token_types_count: 0 }, // [378]
    SimpleHintData { raw_values_start: 665, raw_values_count: 1, token_types_start: 666, token_types_count: 0 }, // [379]
    SimpleHintData { raw_values_start: 666, raw_values_count: 1, token_types_start: 667, token_types_count: 0 }, // [380]
    SimpleHintData { raw_values_start: 667, raw_values_count: 1, token_types_start: 668, token_types_count: 0 }, // [381]
    SimpleHintData { raw_values_start: 668, raw_values_count: 1, token_types_start: 669, token_types_count: 0 }, // [382]
    SimpleHintData { raw_values_start: 669, raw_values_count: 1, token_types_start: 670, token_types_count: 0 }, // [383]
    SimpleHintData { raw_values_start: 670, raw_values_count: 1, token_types_start: 671, token_types_count: 0 }, // [384]
    SimpleHintData { raw_values_start: 671, raw_values_count: 1, token_types_start: 672, token_types_count: 0 }, // [385]
    SimpleHintData { raw_values_start: 672, raw_values_count: 1, token_types_start: 673, token_types_count: 0 }, // [386]
    SimpleHintData { raw_values_start: 673, raw_values_count: 1, token_types_start: 674, token_types_count: 0 }, // [387]
    SimpleHintData { raw_values_start: 674, raw_values_count: 1, token_types_start: 675, token_types_count: 0 }, // [388]
    SimpleHintData { raw_values_start: 675, raw_values_count: 1, token_types_start: 676, token_types_count: 0 }, // [389]
    SimpleHintData { raw_values_start: 676, raw_values_count: 1, token_types_start: 677, token_types_count: 0 }, // [390]
    SimpleHintData { raw_values_start: 677, raw_values_count: 1, token_types_start: 678, token_types_count: 0 }, // [391]
    SimpleHintData { raw_values_start: 678, raw_values_count: 1, token_types_start: 679, token_types_count: 0 }, // [392]
    SimpleHintData { raw_values_start: 679, raw_values_count: 1, token_types_start: 680, token_types_count: 0 }, // [393]
    SimpleHintData { raw_values_start: 680, raw_values_count: 1, token_types_start: 681, token_types_count: 0 }, // [394]
    SimpleHintData { raw_values_start: 681, raw_values_count: 1, token_types_start: 682, token_types_count: 0 }, // [395]
    SimpleHintData { raw_values_start: 682, raw_values_count: 1, token_types_start: 683, token_types_count: 0 }, // [396]
    SimpleHintData { raw_values_start: 683, raw_values_count: 1, token_types_start: 684, token_types_count: 0 }, // [397]
    SimpleHintData { raw_values_start: 684, raw_values_count: 1, token_types_start: 685, token_types_count: 0 }, // [398]
    SimpleHintData { raw_values_start: 685, raw_values_count: 1, token_types_start: 686, token_types_count: 0 }, // [399]
    SimpleHintData { raw_values_start: 686, raw_values_count: 29, token_types_start: 715, token_types_count: 1 }, // [400]
    SimpleHintData { raw_values_start: 716, raw_values_count: 1, token_types_start: 717, token_types_count: 0 }, // [401]
    SimpleHintData { raw_values_start: 717, raw_values_count: 1, token_types_start: 718, token_types_count: 0 }, // [402]
    SimpleHintData { raw_values_start: 718, raw_values_count: 1, token_types_start: 719, token_types_count: 0 }, // [403]
    SimpleHintData { raw_values_start: 719, raw_values_count: 1, token_types_start: 720, token_types_count: 0 }, // [404]
    SimpleHintData { raw_values_start: 720, raw_values_count: 1, token_types_start: 721, token_types_count: 0 }, // [405]
    SimpleHintData { raw_values_start: 721, raw_values_count: 1, token_types_start: 722, token_types_count: 0 }, // [406]
    SimpleHintData { raw_values_start: 722, raw_values_count: 1, token_types_start: 723, token_types_count: 0 }, // [407]
    SimpleHintData { raw_values_start: 723, raw_values_count: 2, token_types_start: 725, token_types_count: 0 }, // [408]
    SimpleHintData { raw_values_start: 725, raw_values_count: 2, token_types_start: 727, token_types_count: 0 }, // [409]
    SimpleHintData { raw_values_start: 727, raw_values_count: 1, token_types_start: 728, token_types_count: 0 }, // [410]
    SimpleHintData { raw_values_start: 728, raw_values_count: 1, token_types_start: 729, token_types_count: 0 }, // [411]
    SimpleHintData { raw_values_start: 729, raw_values_count: 1, token_types_start: 730, token_types_count: 0 }, // [412]
    SimpleHintData { raw_values_start: 730, raw_values_count: 1, token_types_start: 731, token_types_count: 0 }, // [413]
    SimpleHintData { raw_values_start: 731, raw_values_count: 1, token_types_start: 732, token_types_count: 0 }, // [414]
    SimpleHintData { raw_values_start: 732, raw_values_count: 1, token_types_start: 733, token_types_count: 0 }, // [415]
    SimpleHintData { raw_values_start: 733, raw_values_count: 5, token_types_start: 738, token_types_count: 0 }, // [416]
    SimpleHintData { raw_values_start: 738, raw_values_count: 1, token_types_start: 739, token_types_count: 1 }, // [417]
    SimpleHintData { raw_values_start: 740, raw_values_count: 2, token_types_start: 742, token_types_count: 0 }, // [418]
    SimpleHintData { raw_values_start: 742, raw_values_count: 2, token_types_start: 744, token_types_count: 0 }, // [419]
    SimpleHintData { raw_values_start: 744, raw_values_count: 0, token_types_start: 744, token_types_count: 1 }, // [420]
    SimpleHintData { raw_values_start: 745, raw_values_count: 3, token_types_start: 748, token_types_count: 0 }, // [421]
    SimpleHintData { raw_values_start: 748, raw_values_count: 1, token_types_start: 749, token_types_count: 0 }, // [422]
    SimpleHintData { raw_values_start: 749, raw_values_count: 1, token_types_start: 750, token_types_count: 0 }, // [423]
    SimpleHintData { raw_values_start: 750, raw_values_count: 1, token_types_start: 751, token_types_count: 0 }, // [424]
    SimpleHintData { raw_values_start: 751, raw_values_count: 1, token_types_start: 752, token_types_count: 0 }, // [425]
    SimpleHintData { raw_values_start: 752, raw_values_count: 3, token_types_start: 755, token_types_count: 0 }, // [426]
    SimpleHintData { raw_values_start: 755, raw_values_count: 1, token_types_start: 756, token_types_count: 0 }, // [427]
    SimpleHintData { raw_values_start: 756, raw_values_count: 1, token_types_start: 757, token_types_count: 0 }, // [428]
    SimpleHintData { raw_values_start: 757, raw_values_count: 1, token_types_start: 758, token_types_count: 0 }, // [429]
    SimpleHintData { raw_values_start: 758, raw_values_count: 1, token_types_start: 759, token_types_count: 0 }, // [430]
    SimpleHintData { raw_values_start: 759, raw_values_count: 4, token_types_start: 763, token_types_count: 0 }, // [431]
    SimpleHintData { raw_values_start: 763, raw_values_count: 1, token_types_start: 764, token_types_count: 0 }, // [432]
    SimpleHintData { raw_values_start: 764, raw_values_count: 1, token_types_start: 765, token_types_count: 0 }, // [433]
    SimpleHintData { raw_values_start: 765, raw_values_count: 1, token_types_start: 766, token_types_count: 0 }, // [434]
    SimpleHintData { raw_values_start: 766, raw_values_count: 1, token_types_start: 767, token_types_count: 0 }, // [435]
    SimpleHintData { raw_values_start: 767, raw_values_count: 1, token_types_start: 768, token_types_count: 0 }, // [436]
    SimpleHintData { raw_values_start: 768, raw_values_count: 1, token_types_start: 769, token_types_count: 0 }, // [437]
    SimpleHintData { raw_values_start: 769, raw_values_count: 1, token_types_start: 770, token_types_count: 0 }, // [438]
    SimpleHintData { raw_values_start: 770, raw_values_count: 1, token_types_start: 771, token_types_count: 0 }, // [439]
    SimpleHintData { raw_values_start: 771, raw_values_count: 1, token_types_start: 772, token_types_count: 0 }, // [440]
    SimpleHintData { raw_values_start: 772, raw_values_count: 1, token_types_start: 773, token_types_count: 0 }, // [441]
    SimpleHintData { raw_values_start: 773, raw_values_count: 1, token_types_start: 774, token_types_count: 0 }, // [442]
    SimpleHintData { raw_values_start: 774, raw_values_count: 1, token_types_start: 775, token_types_count: 0 }, // [443]
    SimpleHintData { raw_values_start: 775, raw_values_count: 1, token_types_start: 776, token_types_count: 0 }, // [444]
    SimpleHintData { raw_values_start: 776, raw_values_count: 1, token_types_start: 777, token_types_count: 0 }, // [445]
    SimpleHintData { raw_values_start: 777, raw_values_count: 1, token_types_start: 778, token_types_count: 0 }, // [446]
    SimpleHintData { raw_values_start: 778, raw_values_count: 1, token_types_start: 779, token_types_count: 0 }, // [447]
    SimpleHintData { raw_values_start: 779, raw_values_count: 1, token_types_start: 780, token_types_count: 0 }, // [448]
    SimpleHintData { raw_values_start: 780, raw_values_count: 1, token_types_start: 781, token_types_count: 0 }, // [449]
    SimpleHintData { raw_values_start: 781, raw_values_count: 1, token_types_start: 782, token_types_count: 0 }, // [450]
    SimpleHintData { raw_values_start: 782, raw_values_count: 7, token_types_start: 789, token_types_count: 0 }, // [451]
    SimpleHintData { raw_values_start: 789, raw_values_count: 4, token_types_start: 793, token_types_count: 0 }, // [452]
    SimpleHintData { raw_values_start: 793, raw_values_count: 3, token_types_start: 796, token_types_count: 0 }, // [453]
    SimpleHintData { raw_values_start: 796, raw_values_count: 1, token_types_start: 797, token_types_count: 0 }, // [454]
    SimpleHintData { raw_values_start: 797, raw_values_count: 1, token_types_start: 798, token_types_count: 0 }, // [455]
    SimpleHintData { raw_values_start: 798, raw_values_count: 1, token_types_start: 799, token_types_count: 0 }, // [456]
    SimpleHintData { raw_values_start: 799, raw_values_count: 1, token_types_start: 800, token_types_count: 0 }, // [457]
    SimpleHintData { raw_values_start: 800, raw_values_count: 1, token_types_start: 801, token_types_count: 0 }, // [458]
    SimpleHintData { raw_values_start: 801, raw_values_count: 1, token_types_start: 802, token_types_count: 0 }, // [459]
    SimpleHintData { raw_values_start: 802, raw_values_count: 5, token_types_start: 807, token_types_count: 0 }, // [460]
    SimpleHintData { raw_values_start: 807, raw_values_count: 1, token_types_start: 808, token_types_count: 0 }, // [461]
    SimpleHintData { raw_values_start: 808, raw_values_count: 2, token_types_start: 810, token_types_count: 0 }, // [462]
    SimpleHintData { raw_values_start: 810, raw_values_count: 1, token_types_start: 811, token_types_count: 0 }, // [463]
    SimpleHintData { raw_values_start: 811, raw_values_count: 1, token_types_start: 812, token_types_count: 0 }, // [464]
    SimpleHintData { raw_values_start: 812, raw_values_count: 10, token_types_start: 822, token_types_count: 0 }, // [465]
    SimpleHintData { raw_values_start: 822, raw_values_count: 1, token_types_start: 823, token_types_count: 0 }, // [466]
    SimpleHintData { raw_values_start: 823, raw_values_count: 1, token_types_start: 824, token_types_count: 0 }, // [467]
    SimpleHintData { raw_values_start: 824, raw_values_count: 1, token_types_start: 825, token_types_count: 0 }, // [468]
    SimpleHintData { raw_values_start: 825, raw_values_count: 1, token_types_start: 826, token_types_count: 0 }, // [469]
    SimpleHintData { raw_values_start: 826, raw_values_count: 1, token_types_start: 827, token_types_count: 0 }, // [470]
    SimpleHintData { raw_values_start: 827, raw_values_count: 1, token_types_start: 828, token_types_count: 0 }, // [471]
    SimpleHintData { raw_values_start: 828, raw_values_count: 1, token_types_start: 829, token_types_count: 0 }, // [472]
    SimpleHintData { raw_values_start: 829, raw_values_count: 2, token_types_start: 831, token_types_count: 0 }, // [473]
    SimpleHintData { raw_values_start: 831, raw_values_count: 1, token_types_start: 832, token_types_count: 0 }, // [474]
    SimpleHintData { raw_values_start: 832, raw_values_count: 1, token_types_start: 833, token_types_count: 0 }, // [475]
    SimpleHintData { raw_values_start: 833, raw_values_count: 2, token_types_start: 835, token_types_count: 0 }, // [476]
    SimpleHintData { raw_values_start: 835, raw_values_count: 16, token_types_start: 851, token_types_count: 0 }, // [477]
    SimpleHintData { raw_values_start: 851, raw_values_count: 1, token_types_start: 852, token_types_count: 0 }, // [478]
    SimpleHintData { raw_values_start: 852, raw_values_count: 3, token_types_start: 855, token_types_count: 0 }, // [479]
    SimpleHintData { raw_values_start: 855, raw_values_count: 5, token_types_start: 860, token_types_count: 0 }, // [480]
    SimpleHintData { raw_values_start: 860, raw_values_count: 12, token_types_start: 872, token_types_count: 0 }, // [481]
    SimpleHintData { raw_values_start: 872, raw_values_count: 1, token_types_start: 873, token_types_count: 0 }, // [482]
    SimpleHintData { raw_values_start: 873, raw_values_count: 2, token_types_start: 875, token_types_count: 0 }, // [483]
    SimpleHintData { raw_values_start: 875, raw_values_count: 31, token_types_start: 906, token_types_count: 0 }, // [484]
    SimpleHintData { raw_values_start: 906, raw_values_count: 27, token_types_start: 933, token_types_count: 0 }, // [485]
    SimpleHintData { raw_values_start: 933, raw_values_count: 29, token_types_start: 962, token_types_count: 0 }, // [486]
    SimpleHintData { raw_values_start: 962, raw_values_count: 1, token_types_start: 963, token_types_count: 0 }, // [487]
    SimpleHintData { raw_values_start: 963, raw_values_count: 4, token_types_start: 967, token_types_count: 0 }, // [488]
    SimpleHintData { raw_values_start: 967, raw_values_count: 2, token_types_start: 969, token_types_count: 0 }, // [489]
    SimpleHintData { raw_values_start: 969, raw_values_count: 1, token_types_start: 970, token_types_count: 0 }, // [490]
    SimpleHintData { raw_values_start: 970, raw_values_count: 3, token_types_start: 973, token_types_count: 0 }, // [491]
    SimpleHintData { raw_values_start: 973, raw_values_count: 1, token_types_start: 974, token_types_count: 0 }, // [492]
    SimpleHintData { raw_values_start: 974, raw_values_count: 1, token_types_start: 975, token_types_count: 0 }, // [493]
    SimpleHintData { raw_values_start: 975, raw_values_count: 3, token_types_start: 978, token_types_count: 0 }, // [494]
    SimpleHintData { raw_values_start: 978, raw_values_count: 2, token_types_start: 980, token_types_count: 0 }, // [495]
    SimpleHintData { raw_values_start: 980, raw_values_count: 1, token_types_start: 981, token_types_count: 0 }, // [496]
    SimpleHintData { raw_values_start: 981, raw_values_count: 1, token_types_start: 982, token_types_count: 0 }, // [497]
    SimpleHintData { raw_values_start: 982, raw_values_count: 2, token_types_start: 984, token_types_count: 0 }, // [498]
    SimpleHintData { raw_values_start: 984, raw_values_count: 1, token_types_start: 985, token_types_count: 0 }, // [499]
    SimpleHintData { raw_values_start: 985, raw_values_count: 1, token_types_start: 986, token_types_count: 0 }, // [500]
    SimpleHintData { raw_values_start: 986, raw_values_count: 5, token_types_start: 991, token_types_count: 0 }, // [501]
    SimpleHintData { raw_values_start: 991, raw_values_count: 3, token_types_start: 994, token_types_count: 0 }, // [502]
    SimpleHintData { raw_values_start: 994, raw_values_count: 1, token_types_start: 995, token_types_count: 0 }, // [503]
    SimpleHintData { raw_values_start: 995, raw_values_count: 2, token_types_start: 997, token_types_count: 0 }, // [504]
    SimpleHintData { raw_values_start: 997, raw_values_count: 6, token_types_start: 1003, token_types_count: 0 }, // [505]
    SimpleHintData { raw_values_start: 1003, raw_values_count: 2, token_types_start: 1005, token_types_count: 0 }, // [506]
    SimpleHintData { raw_values_start: 1005, raw_values_count: 4, token_types_start: 1009, token_types_count: 0 }, // [507]
    SimpleHintData { raw_values_start: 1009, raw_values_count: 2, token_types_start: 1011, token_types_count: 0 }, // [508]
    SimpleHintData { raw_values_start: 1011, raw_values_count: 3, token_types_start: 1014, token_types_count: 0 }, // [509]
    SimpleHintData { raw_values_start: 1014, raw_values_count: 2, token_types_start: 1016, token_types_count: 0 }, // [510]
    SimpleHintData { raw_values_start: 1016, raw_values_count: 1, token_types_start: 1017, token_types_count: 0 }, // [511]
    SimpleHintData { raw_values_start: 1017, raw_values_count: 1, token_types_start: 1018, token_types_count: 0 }, // [512]
    SimpleHintData { raw_values_start: 1018, raw_values_count: 2, token_types_start: 1020, token_types_count: 0 }, // [513]
    SimpleHintData { raw_values_start: 1020, raw_values_count: 1, token_types_start: 1021, token_types_count: 0 }, // [514]
    SimpleHintData { raw_values_start: 1021, raw_values_count: 1, token_types_start: 1022, token_types_count: 0 }, // [515]
    SimpleHintData { raw_values_start: 1022, raw_values_count: 0, token_types_start: 1022, token_types_count: 2 }, // [516]
    SimpleHintData { raw_values_start: 1024, raw_values_count: 0, token_types_start: 1024, token_types_count: 1 }, // [517]
    SimpleHintData { raw_values_start: 1025, raw_values_count: 1, token_types_start: 1026, token_types_count: 0 }, // [518]
    SimpleHintData { raw_values_start: 1026, raw_values_count: 10, token_types_start: 1036, token_types_count: 0 }, // [519]
    SimpleHintData { raw_values_start: 1036, raw_values_count: 4, token_types_start: 1040, token_types_count: 0 }, // [520]
    SimpleHintData { raw_values_start: 1040, raw_values_count: 2, token_types_start: 1042, token_types_count: 0 }, // [521]
    SimpleHintData { raw_values_start: 1042, raw_values_count: 1, token_types_start: 1043, token_types_count: 0 }, // [522]
    SimpleHintData { raw_values_start: 1043, raw_values_count: 1, token_types_start: 1044, token_types_count: 0 }, // [523]
    SimpleHintData { raw_values_start: 1044, raw_values_count: 1, token_types_start: 1045, token_types_count: 1 }, // [524]
    SimpleHintData { raw_values_start: 1046, raw_values_count: 1, token_types_start: 1047, token_types_count: 0 }, // [525]
    SimpleHintData { raw_values_start: 1047, raw_values_count: 1, token_types_start: 1048, token_types_count: 0 }, // [526]
    SimpleHintData { raw_values_start: 1048, raw_values_count: 2, token_types_start: 1050, token_types_count: 0 }, // [527]
    SimpleHintData { raw_values_start: 1050, raw_values_count: 1, token_types_start: 1051, token_types_count: 0 }, // [528]
    SimpleHintData { raw_values_start: 1051, raw_values_count: 1, token_types_start: 1052, token_types_count: 0 }, // [529]
    SimpleHintData { raw_values_start: 1052, raw_values_count: 2, token_types_start: 1054, token_types_count: 0 }, // [530]
    SimpleHintData { raw_values_start: 1054, raw_values_count: 1, token_types_start: 1055, token_types_count: 0 }, // [531]
    SimpleHintData { raw_values_start: 1055, raw_values_count: 2, token_types_start: 1057, token_types_count: 0 }, // [532]
    SimpleHintData { raw_values_start: 1057, raw_values_count: 1, token_types_start: 1058, token_types_count: 0 }, // [533]
    SimpleHintData { raw_values_start: 1058, raw_values_count: 2, token_types_start: 1060, token_types_count: 0 }, // [534]
    SimpleHintData { raw_values_start: 1060, raw_values_count: 3, token_types_start: 1063, token_types_count: 0 }, // [535]
    SimpleHintData { raw_values_start: 1063, raw_values_count: 1, token_types_start: 1064, token_types_count: 0 }, // [536]
    SimpleHintData { raw_values_start: 1064, raw_values_count: 1, token_types_start: 1065, token_types_count: 0 }, // [537]
    SimpleHintData { raw_values_start: 1065, raw_values_count: 1, token_types_start: 1066, token_types_count: 0 }, // [538]
    SimpleHintData { raw_values_start: 1066, raw_values_count: 1, token_types_start: 1067, token_types_count: 0 }, // [539]
    SimpleHintData { raw_values_start: 1067, raw_values_count: 1, token_types_start: 1068, token_types_count: 0 }, // [540]
    SimpleHintData { raw_values_start: 1068, raw_values_count: 1, token_types_start: 1069, token_types_count: 0 }, // [541]
    SimpleHintData { raw_values_start: 1069, raw_values_count: 1, token_types_start: 1070, token_types_count: 0 }, // [542]
    SimpleHintData { raw_values_start: 1070, raw_values_count: 1, token_types_start: 1071, token_types_count: 0 }, // [543]
    SimpleHintData { raw_values_start: 1071, raw_values_count: 2, token_types_start: 1073, token_types_count: 0 }, // [544]
    SimpleHintData { raw_values_start: 1073, raw_values_count: 1, token_types_start: 1074, token_types_count: 0 }, // [545]
    SimpleHintData { raw_values_start: 1074, raw_values_count: 1, token_types_start: 1075, token_types_count: 0 }, // [546]
    SimpleHintData { raw_values_start: 1075, raw_values_count: 1, token_types_start: 1076, token_types_count: 0 }, // [547]
    SimpleHintData { raw_values_start: 1076, raw_values_count: 1, token_types_start: 1077, token_types_count: 0 }, // [548]
    SimpleHintData { raw_values_start: 1077, raw_values_count: 1, token_types_start: 1078, token_types_count: 0 }, // [549]
    SimpleHintData { raw_values_start: 1078, raw_values_count: 1, token_types_start: 1079, token_types_count: 0 }, // [550]
    SimpleHintData { raw_values_start: 1079, raw_values_count: 1, token_types_start: 1080, token_types_count: 0 }, // [551]
    SimpleHintData { raw_values_start: 1080, raw_values_count: 1, token_types_start: 1081, token_types_count: 0 }, // [552]
    SimpleHintData { raw_values_start: 1081, raw_values_count: 1, token_types_start: 1082, token_types_count: 0 }, // [553]
    SimpleHintData { raw_values_start: 1082, raw_values_count: 1, token_types_start: 1083, token_types_count: 0 }, // [554]
    SimpleHintData { raw_values_start: 1083, raw_values_count: 1, token_types_start: 1084, token_types_count: 0 }, // [555]
    SimpleHintData { raw_values_start: 1084, raw_values_count: 1, token_types_start: 1085, token_types_count: 0 }, // [556]
    SimpleHintData { raw_values_start: 1085, raw_values_count: 1, token_types_start: 1086, token_types_count: 0 }, // [557]
    SimpleHintData { raw_values_start: 1086, raw_values_count: 1, token_types_start: 1087, token_types_count: 0 }, // [558]
    SimpleHintData { raw_values_start: 1087, raw_values_count: 2, token_types_start: 1089, token_types_count: 0 }, // [559]
    SimpleHintData { raw_values_start: 1089, raw_values_count: 1, token_types_start: 1090, token_types_count: 0 }, // [560]
    SimpleHintData { raw_values_start: 1090, raw_values_count: 1, token_types_start: 1091, token_types_count: 0 }, // [561]
    SimpleHintData { raw_values_start: 1091, raw_values_count: 2, token_types_start: 1093, token_types_count: 0 }, // [562]
    SimpleHintData { raw_values_start: 1093, raw_values_count: 2, token_types_start: 1095, token_types_count: 0 }, // [563]
    SimpleHintData { raw_values_start: 1095, raw_values_count: 1, token_types_start: 1096, token_types_count: 0 }, // [564]
    SimpleHintData { raw_values_start: 1096, raw_values_count: 1, token_types_start: 1097, token_types_count: 0 }, // [565]
    SimpleHintData { raw_values_start: 1097, raw_values_count: 1, token_types_start: 1098, token_types_count: 0 }, // [566]
    SimpleHintData { raw_values_start: 1098, raw_values_count: 1, token_types_start: 1099, token_types_count: 0 }, // [567]
    SimpleHintData { raw_values_start: 1099, raw_values_count: 11, token_types_start: 1110, token_types_count: 0 }, // [568]
    SimpleHintData { raw_values_start: 1110, raw_values_count: 4, token_types_start: 1114, token_types_count: 0 }, // [569]
    SimpleHintData { raw_values_start: 1114, raw_values_count: 1, token_types_start: 1115, token_types_count: 0 }, // [570]
    SimpleHintData { raw_values_start: 1115, raw_values_count: 6, token_types_start: 1121, token_types_count: 0 }, // [571]
    SimpleHintData { raw_values_start: 1121, raw_values_count: 1, token_types_start: 1122, token_types_count: 0 }, // [572]
    SimpleHintData { raw_values_start: 1122, raw_values_count: 1, token_types_start: 1123, token_types_count: 0 }, // [573]
    SimpleHintData { raw_values_start: 1123, raw_values_count: 1, token_types_start: 1124, token_types_count: 0 }, // [574]
    SimpleHintData { raw_values_start: 1124, raw_values_count: 1, token_types_start: 1125, token_types_count: 0 }, // [575]
    SimpleHintData { raw_values_start: 1125, raw_values_count: 1, token_types_start: 1126, token_types_count: 0 }, // [576]
    SimpleHintData { raw_values_start: 1126, raw_values_count: 1, token_types_start: 1127, token_types_count: 0 }, // [577]
    SimpleHintData { raw_values_start: 1127, raw_values_count: 1, token_types_start: 1128, token_types_count: 0 }, // [578]
    SimpleHintData { raw_values_start: 1128, raw_values_count: 1, token_types_start: 1129, token_types_count: 0 }, // [579]
    SimpleHintData { raw_values_start: 1129, raw_values_count: 1, token_types_start: 1130, token_types_count: 0 }, // [580]
    SimpleHintData { raw_values_start: 1130, raw_values_count: 1, token_types_start: 1131, token_types_count: 0 }, // [581]
    SimpleHintData { raw_values_start: 1131, raw_values_count: 1, token_types_start: 1132, token_types_count: 0 }, // [582]
    SimpleHintData { raw_values_start: 1132, raw_values_count: 1, token_types_start: 1133, token_types_count: 0 }, // [583]
    SimpleHintData { raw_values_start: 1133, raw_values_count: 1, token_types_start: 1134, token_types_count: 0 }, // [584]
    SimpleHintData { raw_values_start: 1134, raw_values_count: 1, token_types_start: 1135, token_types_count: 0 }, // [585]
    SimpleHintData { raw_values_start: 1135, raw_values_count: 1, token_types_start: 1136, token_types_count: 0 }, // [586]
    SimpleHintData { raw_values_start: 1136, raw_values_count: 1, token_types_start: 1137, token_types_count: 0 }, // [587]
    SimpleHintData { raw_values_start: 1137, raw_values_count: 1, token_types_start: 1138, token_types_count: 0 }, // [588]
    SimpleHintData { raw_values_start: 1138, raw_values_count: 1, token_types_start: 1139, token_types_count: 0 }, // [589]
    SimpleHintData { raw_values_start: 1139, raw_values_count: 1, token_types_start: 1140, token_types_count: 0 }, // [590]
    SimpleHintData { raw_values_start: 1140, raw_values_count: 1, token_types_start: 1141, token_types_count: 0 }, // [591]
    SimpleHintData { raw_values_start: 1141, raw_values_count: 1, token_types_start: 1142, token_types_count: 0 }, // [592]
    SimpleHintData { raw_values_start: 1142, raw_values_count: 1, token_types_start: 1143, token_types_count: 0 }, // [593]
    SimpleHintData { raw_values_start: 1143, raw_values_count: 1, token_types_start: 1144, token_types_count: 0 }, // [594]
    SimpleHintData { raw_values_start: 1144, raw_values_count: 1, token_types_start: 1145, token_types_count: 0 }, // [595]
    SimpleHintData { raw_values_start: 1145, raw_values_count: 1, token_types_start: 1146, token_types_count: 0 }, // [596]
    SimpleHintData { raw_values_start: 1146, raw_values_count: 1, token_types_start: 1147, token_types_count: 0 }, // [597]
    SimpleHintData { raw_values_start: 1147, raw_values_count: 1, token_types_start: 1148, token_types_count: 0 }, // [598]
    SimpleHintData { raw_values_start: 1148, raw_values_count: 1, token_types_start: 1149, token_types_count: 0 }, // [599]
    SimpleHintData { raw_values_start: 1149, raw_values_count: 1, token_types_start: 1150, token_types_count: 0 }, // [600]
    SimpleHintData { raw_values_start: 1150, raw_values_count: 1, token_types_start: 1151, token_types_count: 0 }, // [601]
    SimpleHintData { raw_values_start: 1151, raw_values_count: 2, token_types_start: 1153, token_types_count: 0 }, // [602]
    SimpleHintData { raw_values_start: 1153, raw_values_count: 1, token_types_start: 1154, token_types_count: 0 }, // [603]
    SimpleHintData { raw_values_start: 1154, raw_values_count: 1, token_types_start: 1155, token_types_count: 0 }, // [604]
    SimpleHintData { raw_values_start: 1155, raw_values_count: 1, token_types_start: 1156, token_types_count: 0 }, // [605]
    SimpleHintData { raw_values_start: 1156, raw_values_count: 1, token_types_start: 1157, token_types_count: 0 }, // [606]
    SimpleHintData { raw_values_start: 1157, raw_values_count: 1, token_types_start: 1158, token_types_count: 0 }, // [607]
    SimpleHintData { raw_values_start: 1158, raw_values_count: 1, token_types_start: 1159, token_types_count: 0 }, // [608]
    SimpleHintData { raw_values_start: 1159, raw_values_count: 1, token_types_start: 1160, token_types_count: 0 }, // [609]
    SimpleHintData { raw_values_start: 1160, raw_values_count: 1, token_types_start: 1161, token_types_count: 0 }, // [610]
    SimpleHintData { raw_values_start: 1161, raw_values_count: 1, token_types_start: 1162, token_types_count: 0 }, // [611]
    SimpleHintData { raw_values_start: 1162, raw_values_count: 1, token_types_start: 1163, token_types_count: 0 }, // [612]
    SimpleHintData { raw_values_start: 1163, raw_values_count: 1, token_types_start: 1164, token_types_count: 0 }, // [613]
    SimpleHintData { raw_values_start: 1164, raw_values_count: 1, token_types_start: 1165, token_types_count: 0 }, // [614]
    SimpleHintData { raw_values_start: 1165, raw_values_count: 1, token_types_start: 1166, token_types_count: 0 }, // [615]
    SimpleHintData { raw_values_start: 1166, raw_values_count: 1, token_types_start: 1167, token_types_count: 0 }, // [616]
    SimpleHintData { raw_values_start: 1167, raw_values_count: 1, token_types_start: 1168, token_types_count: 0 }, // [617]
    SimpleHintData { raw_values_start: 1168, raw_values_count: 1, token_types_start: 1169, token_types_count: 0 }, // [618]
    SimpleHintData { raw_values_start: 1169, raw_values_count: 1, token_types_start: 1170, token_types_count: 0 }, // [619]
    SimpleHintData { raw_values_start: 1170, raw_values_count: 1, token_types_start: 1171, token_types_count: 0 }, // [620]
    SimpleHintData { raw_values_start: 1171, raw_values_count: 1, token_types_start: 1172, token_types_count: 0 }, // [621]
    SimpleHintData { raw_values_start: 1172, raw_values_count: 1, token_types_start: 1173, token_types_count: 0 }, // [622]
    SimpleHintData { raw_values_start: 1173, raw_values_count: 1, token_types_start: 1174, token_types_count: 0 }, // [623]
    SimpleHintData { raw_values_start: 1174, raw_values_count: 1, token_types_start: 1175, token_types_count: 0 }, // [624]
    SimpleHintData { raw_values_start: 1175, raw_values_count: 1, token_types_start: 1176, token_types_count: 0 }, // [625]
    SimpleHintData { raw_values_start: 1176, raw_values_count: 1, token_types_start: 1177, token_types_count: 0 }, // [626]
    SimpleHintData { raw_values_start: 1177, raw_values_count: 2, token_types_start: 1179, token_types_count: 0 }, // [627]
    SimpleHintData { raw_values_start: 1179, raw_values_count: 1, token_types_start: 1180, token_types_count: 0 }, // [628]
    SimpleHintData { raw_values_start: 1180, raw_values_count: 2, token_types_start: 1182, token_types_count: 0 }, // [629]
    SimpleHintData { raw_values_start: 1182, raw_values_count: 1, token_types_start: 1183, token_types_count: 0 }, // [630]
    SimpleHintData { raw_values_start: 1183, raw_values_count: 1, token_types_start: 1184, token_types_count: 0 }, // [631]
    SimpleHintData { raw_values_start: 1184, raw_values_count: 1, token_types_start: 1185, token_types_count: 0 }, // [632]
    SimpleHintData { raw_values_start: 1185, raw_values_count: 1, token_types_start: 1186, token_types_count: 0 }, // [633]
    SimpleHintData { raw_values_start: 1186, raw_values_count: 1, token_types_start: 1187, token_types_count: 0 }, // [634]
    SimpleHintData { raw_values_start: 1187, raw_values_count: 1, token_types_start: 1188, token_types_count: 0 }, // [635]
    SimpleHintData { raw_values_start: 1188, raw_values_count: 1, token_types_start: 1189, token_types_count: 0 }, // [636]
    SimpleHintData { raw_values_start: 1189, raw_values_count: 1, token_types_start: 1190, token_types_count: 0 }, // [637]
    SimpleHintData { raw_values_start: 1190, raw_values_count: 3, token_types_start: 1193, token_types_count: 0 }, // [638]
    SimpleHintData { raw_values_start: 1193, raw_values_count: 1, token_types_start: 1194, token_types_count: 0 }, // [639]
    SimpleHintData { raw_values_start: 1194, raw_values_count: 1, token_types_start: 1195, token_types_count: 0 }, // [640]
    SimpleHintData { raw_values_start: 1195, raw_values_count: 1, token_types_start: 1196, token_types_count: 0 }, // [641]
    SimpleHintData { raw_values_start: 1196, raw_values_count: 2, token_types_start: 1198, token_types_count: 0 }, // [642]
    SimpleHintData { raw_values_start: 1198, raw_values_count: 1, token_types_start: 1199, token_types_count: 0 }, // [643]
    SimpleHintData { raw_values_start: 1199, raw_values_count: 6, token_types_start: 1205, token_types_count: 0 }, // [644]
    SimpleHintData { raw_values_start: 1205, raw_values_count: 1, token_types_start: 1206, token_types_count: 0 }, // [645]
    SimpleHintData { raw_values_start: 1206, raw_values_count: 1, token_types_start: 1207, token_types_count: 0 }, // [646]
    SimpleHintData { raw_values_start: 1207, raw_values_count: 1, token_types_start: 1208, token_types_count: 0 }, // [647]
    SimpleHintData { raw_values_start: 1208, raw_values_count: 1, token_types_start: 1209, token_types_count: 0 }, // [648]
    SimpleHintData { raw_values_start: 1209, raw_values_count: 1, token_types_start: 1210, token_types_count: 0 }, // [649]
    SimpleHintData { raw_values_start: 1210, raw_values_count: 1, token_types_start: 1211, token_types_count: 0 }, // [650]
    SimpleHintData { raw_values_start: 1211, raw_values_count: 1, token_types_start: 1212, token_types_count: 0 }, // [651]
    SimpleHintData { raw_values_start: 1212, raw_values_count: 1, token_types_start: 1213, token_types_count: 0 }, // [652]
    SimpleHintData { raw_values_start: 1213, raw_values_count: 1, token_types_start: 1214, token_types_count: 0 }, // [653]
    SimpleHintData { raw_values_start: 1214, raw_values_count: 1, token_types_start: 1215, token_types_count: 0 }, // [654]
    SimpleHintData { raw_values_start: 1215, raw_values_count: 1, token_types_start: 1216, token_types_count: 0 }, // [655]
    SimpleHintData { raw_values_start: 1216, raw_values_count: 1, token_types_start: 1217, token_types_count: 0 }, // [656]
    SimpleHintData { raw_values_start: 1217, raw_values_count: 1, token_types_start: 1218, token_types_count: 0 }, // [657]
    SimpleHintData { raw_values_start: 1218, raw_values_count: 1, token_types_start: 1219, token_types_count: 0 }, // [658]
    SimpleHintData { raw_values_start: 1219, raw_values_count: 1, token_types_start: 1220, token_types_count: 0 }, // [659]
    SimpleHintData { raw_values_start: 1220, raw_values_count: 1, token_types_start: 1221, token_types_count: 0 }, // [660]
    SimpleHintData { raw_values_start: 1221, raw_values_count: 1, token_types_start: 1222, token_types_count: 0 }, // [661]
    SimpleHintData { raw_values_start: 1222, raw_values_count: 1, token_types_start: 1223, token_types_count: 0 }, // [662]
    SimpleHintData { raw_values_start: 1223, raw_values_count: 1, token_types_start: 1224, token_types_count: 0 }, // [663]
    SimpleHintData { raw_values_start: 1224, raw_values_count: 1, token_types_start: 1225, token_types_count: 0 }, // [664]
    SimpleHintData { raw_values_start: 1225, raw_values_count: 1, token_types_start: 1226, token_types_count: 0 }, // [665]
    SimpleHintData { raw_values_start: 1226, raw_values_count: 1, token_types_start: 1227, token_types_count: 0 }, // [666]
    SimpleHintData { raw_values_start: 1227, raw_values_count: 1, token_types_start: 1228, token_types_count: 0 }, // [667]
    SimpleHintData { raw_values_start: 1228, raw_values_count: 1, token_types_start: 1229, token_types_count: 0 }, // [668]
    SimpleHintData { raw_values_start: 1229, raw_values_count: 2, token_types_start: 1231, token_types_count: 0 }, // [669]
    SimpleHintData { raw_values_start: 1231, raw_values_count: 2, token_types_start: 1233, token_types_count: 0 }, // [670]
    SimpleHintData { raw_values_start: 1233, raw_values_count: 2, token_types_start: 1235, token_types_count: 0 }, // [671]
    SimpleHintData { raw_values_start: 1235, raw_values_count: 1, token_types_start: 1236, token_types_count: 0 }, // [672]
    SimpleHintData { raw_values_start: 1236, raw_values_count: 1, token_types_start: 1237, token_types_count: 0 }, // [673]
    SimpleHintData { raw_values_start: 1237, raw_values_count: 1, token_types_start: 1238, token_types_count: 0 }, // [674]
    SimpleHintData { raw_values_start: 1238, raw_values_count: 1, token_types_start: 1239, token_types_count: 0 }, // [675]
    SimpleHintData { raw_values_start: 1239, raw_values_count: 1, token_types_start: 1240, token_types_count: 0 }, // [676]
    SimpleHintData { raw_values_start: 1240, raw_values_count: 1, token_types_start: 1241, token_types_count: 0 }, // [677]
    SimpleHintData { raw_values_start: 1241, raw_values_count: 1, token_types_start: 1242, token_types_count: 0 }, // [678]
    SimpleHintData { raw_values_start: 1242, raw_values_count: 1, token_types_start: 1243, token_types_count: 0 }, // [679]
    SimpleHintData { raw_values_start: 1243, raw_values_count: 8, token_types_start: 1251, token_types_count: 0 }, // [680]
    SimpleHintData { raw_values_start: 1251, raw_values_count: 4, token_types_start: 1255, token_types_count: 0 }, // [681]
    SimpleHintData { raw_values_start: 1255, raw_values_count: 2, token_types_start: 1257, token_types_count: 0 }, // [682]
    SimpleHintData { raw_values_start: 1257, raw_values_count: 2, token_types_start: 1259, token_types_count: 0 }, // [683]
    SimpleHintData { raw_values_start: 1259, raw_values_count: 5, token_types_start: 1264, token_types_count: 0 }, // [684]
    SimpleHintData { raw_values_start: 1264, raw_values_count: 2, token_types_start: 1266, token_types_count: 0 }, // [685]
    SimpleHintData { raw_values_start: 1266, raw_values_count: 1, token_types_start: 1267, token_types_count: 0 }, // [686]
    SimpleHintData { raw_values_start: 1267, raw_values_count: 1, token_types_start: 1268, token_types_count: 0 }, // [687]
    SimpleHintData { raw_values_start: 1268, raw_values_count: 15, token_types_start: 1283, token_types_count: 1 }, // [688]
    SimpleHintData { raw_values_start: 1284, raw_values_count: 3, token_types_start: 1287, token_types_count: 0 }, // [689]
    SimpleHintData { raw_values_start: 1287, raw_values_count: 7, token_types_start: 1294, token_types_count: 2 }, // [690]
    SimpleHintData { raw_values_start: 1296, raw_values_count: 1, token_types_start: 1297, token_types_count: 0 }, // [691]
    SimpleHintData { raw_values_start: 1297, raw_values_count: 1, token_types_start: 1298, token_types_count: 0 }, // [692]
    SimpleHintData { raw_values_start: 1298, raw_values_count: 1, token_types_start: 1299, token_types_count: 0 }, // [693]
    SimpleHintData { raw_values_start: 1299, raw_values_count: 1, token_types_start: 1300, token_types_count: 0 }, // [694]
    SimpleHintData { raw_values_start: 1300, raw_values_count: 2, token_types_start: 1302, token_types_count: 0 }, // [695]
    SimpleHintData { raw_values_start: 1302, raw_values_count: 1, token_types_start: 1303, token_types_count: 0 }, // [696]
    SimpleHintData { raw_values_start: 1303, raw_values_count: 2, token_types_start: 1305, token_types_count: 0 }, // [697]
    SimpleHintData { raw_values_start: 1305, raw_values_count: 1, token_types_start: 1306, token_types_count: 0 }, // [698]
    SimpleHintData { raw_values_start: 1306, raw_values_count: 2, token_types_start: 1308, token_types_count: 0 }, // [699]
    SimpleHintData { raw_values_start: 1308, raw_values_count: 1, token_types_start: 1309, token_types_count: 0 }, // [700]
    SimpleHintData { raw_values_start: 1309, raw_values_count: 1, token_types_start: 1310, token_types_count: 0 }, // [701]
    SimpleHintData { raw_values_start: 1310, raw_values_count: 1, token_types_start: 1311, token_types_count: 0 }, // [702]
    SimpleHintData { raw_values_start: 1311, raw_values_count: 1, token_types_start: 1312, token_types_count: 0 }, // [703]
    SimpleHintData { raw_values_start: 1312, raw_values_count: 1, token_types_start: 1313, token_types_count: 0 }, // [704]
    SimpleHintData { raw_values_start: 1313, raw_values_count: 1, token_types_start: 1314, token_types_count: 0 }, // [705]
    SimpleHintData { raw_values_start: 1314, raw_values_count: 1, token_types_start: 1315, token_types_count: 0 }, // [706]
    SimpleHintData { raw_values_start: 1315, raw_values_count: 1, token_types_start: 1316, token_types_count: 0 }, // [707]
    SimpleHintData { raw_values_start: 1316, raw_values_count: 1, token_types_start: 1317, token_types_count: 0 }, // [708]
    SimpleHintData { raw_values_start: 1317, raw_values_count: 1, token_types_start: 1318, token_types_count: 0 }, // [709]
    SimpleHintData { raw_values_start: 1318, raw_values_count: 1, token_types_start: 1319, token_types_count: 0 }, // [710]
    SimpleHintData { raw_values_start: 1319, raw_values_count: 1, token_types_start: 1320, token_types_count: 0 }, // [711]
    SimpleHintData { raw_values_start: 1320, raw_values_count: 1, token_types_start: 1321, token_types_count: 0 }, // [712]
    SimpleHintData { raw_values_start: 1321, raw_values_count: 1, token_types_start: 1322, token_types_count: 0 }, // [713]
    SimpleHintData { raw_values_start: 1322, raw_values_count: 4, token_types_start: 1326, token_types_count: 0 }, // [714]
    SimpleHintData { raw_values_start: 1326, raw_values_count: 2, token_types_start: 1328, token_types_count: 0 }, // [715]
    SimpleHintData { raw_values_start: 1328, raw_values_count: 1, token_types_start: 1329, token_types_count: 0 }, // [716]
    SimpleHintData { raw_values_start: 1329, raw_values_count: 1, token_types_start: 1330, token_types_count: 0 }, // [717]
    SimpleHintData { raw_values_start: 1330, raw_values_count: 1, token_types_start: 1331, token_types_count: 0 }, // [718]
    SimpleHintData { raw_values_start: 1331, raw_values_count: 3, token_types_start: 1334, token_types_count: 0 }, // [719]
    SimpleHintData { raw_values_start: 1334, raw_values_count: 1, token_types_start: 1335, token_types_count: 0 }, // [720]
    SimpleHintData { raw_values_start: 1335, raw_values_count: 1, token_types_start: 1336, token_types_count: 0 }, // [721]
    SimpleHintData { raw_values_start: 1336, raw_values_count: 1, token_types_start: 1337, token_types_count: 0 }, // [722]
    SimpleHintData { raw_values_start: 1337, raw_values_count: 3, token_types_start: 1340, token_types_count: 0 }, // [723]
    SimpleHintData { raw_values_start: 1340, raw_values_count: 1, token_types_start: 1341, token_types_count: 0 }, // [724]
    SimpleHintData { raw_values_start: 1341, raw_values_count: 1, token_types_start: 1342, token_types_count: 0 }, // [725]
    SimpleHintData { raw_values_start: 1342, raw_values_count: 1, token_types_start: 1343, token_types_count: 0 }, // [726]
    SimpleHintData { raw_values_start: 1343, raw_values_count: 1, token_types_start: 1344, token_types_count: 0 }, // [727]
    SimpleHintData { raw_values_start: 1344, raw_values_count: 1, token_types_start: 1345, token_types_count: 0 }, // [728]
    SimpleHintData { raw_values_start: 1345, raw_values_count: 1, token_types_start: 1346, token_types_count: 0 }, // [729]
    SimpleHintData { raw_values_start: 1346, raw_values_count: 1, token_types_start: 1347, token_types_count: 0 }, // [730]
    SimpleHintData { raw_values_start: 1347, raw_values_count: 2, token_types_start: 1349, token_types_count: 0 }, // [731]
    SimpleHintData { raw_values_start: 1349, raw_values_count: 1, token_types_start: 1350, token_types_count: 0 }, // [732]
    SimpleHintData { raw_values_start: 1350, raw_values_count: 2, token_types_start: 1352, token_types_count: 0 }, // [733]
    SimpleHintData { raw_values_start: 1352, raw_values_count: 1, token_types_start: 1353, token_types_count: 0 }, // [734]
    SimpleHintData { raw_values_start: 1353, raw_values_count: 1, token_types_start: 1354, token_types_count: 0 }, // [735]
    SimpleHintData { raw_values_start: 1354, raw_values_count: 1, token_types_start: 1355, token_types_count: 0 }, // [736]
    SimpleHintData { raw_values_start: 1355, raw_values_count: 1, token_types_start: 1356, token_types_count: 0 }, // [737]
    SimpleHintData { raw_values_start: 1356, raw_values_count: 1, token_types_start: 1357, token_types_count: 0 }, // [738]
    SimpleHintData { raw_values_start: 1357, raw_values_count: 1, token_types_start: 1358, token_types_count: 0 }, // [739]
    SimpleHintData { raw_values_start: 1358, raw_values_count: 1, token_types_start: 1359, token_types_count: 0 }, // [740]
    SimpleHintData { raw_values_start: 1359, raw_values_count: 1, token_types_start: 1360, token_types_count: 0 }, // [741]
    SimpleHintData { raw_values_start: 1360, raw_values_count: 14, token_types_start: 1374, token_types_count: 0 }, // [742]
    SimpleHintData { raw_values_start: 1374, raw_values_count: 1, token_types_start: 1375, token_types_count: 0 }, // [743]
    SimpleHintData { raw_values_start: 1375, raw_values_count: 1, token_types_start: 1376, token_types_count: 0 }, // [744]
    SimpleHintData { raw_values_start: 1376, raw_values_count: 1, token_types_start: 1377, token_types_count: 0 }, // [745]
    SimpleHintData { raw_values_start: 1377, raw_values_count: 1, token_types_start: 1378, token_types_count: 0 }, // [746]
    SimpleHintData { raw_values_start: 1378, raw_values_count: 4, token_types_start: 1382, token_types_count: 0 }, // [747]
    SimpleHintData { raw_values_start: 1382, raw_values_count: 1, token_types_start: 1383, token_types_count: 0 }, // [748]
    SimpleHintData { raw_values_start: 1383, raw_values_count: 1, token_types_start: 1384, token_types_count: 0 }, // [749]
    SimpleHintData { raw_values_start: 1384, raw_values_count: 1, token_types_start: 1385, token_types_count: 0 }, // [750]
    SimpleHintData { raw_values_start: 1385, raw_values_count: 2, token_types_start: 1387, token_types_count: 0 }, // [751]
    SimpleHintData { raw_values_start: 1387, raw_values_count: 1, token_types_start: 1388, token_types_count: 0 }, // [752]
    SimpleHintData { raw_values_start: 1388, raw_values_count: 1, token_types_start: 1389, token_types_count: 0 }, // [753]
    SimpleHintData { raw_values_start: 1389, raw_values_count: 1, token_types_start: 1390, token_types_count: 0 }, // [754]
    SimpleHintData { raw_values_start: 1390, raw_values_count: 3, token_types_start: 1393, token_types_count: 0 }, // [755]
    SimpleHintData { raw_values_start: 1393, raw_values_count: 1, token_types_start: 1394, token_types_count: 0 }, // [756]
    SimpleHintData { raw_values_start: 1394, raw_values_count: 2, token_types_start: 1396, token_types_count: 0 }, // [757]
    SimpleHintData { raw_values_start: 1396, raw_values_count: 0, token_types_start: 1396, token_types_count: 1 }, // [758]
    SimpleHintData { raw_values_start: 1397, raw_values_count: 0, token_types_start: 1397, token_types_count: 2 }, // [759]
    SimpleHintData { raw_values_start: 1399, raw_values_count: 1, token_types_start: 1400, token_types_count: 0 }, // [760]
    SimpleHintData { raw_values_start: 1400, raw_values_count: 1, token_types_start: 1401, token_types_count: 0 }, // [761]
    SimpleHintData { raw_values_start: 1401, raw_values_count: 1, token_types_start: 1402, token_types_count: 0 }, // [762]
    SimpleHintData { raw_values_start: 1402, raw_values_count: 2, token_types_start: 1404, token_types_count: 0 }, // [763]
    SimpleHintData { raw_values_start: 1404, raw_values_count: 1, token_types_start: 1405, token_types_count: 0 }, // [764]
    SimpleHintData { raw_values_start: 1405, raw_values_count: 1, token_types_start: 1406, token_types_count: 0 }, // [765]
    SimpleHintData { raw_values_start: 1406, raw_values_count: 1, token_types_start: 1407, token_types_count: 0 }, // [766]
    SimpleHintData { raw_values_start: 1407, raw_values_count: 1, token_types_start: 1408, token_types_count: 0 }, // [767]
    SimpleHintData { raw_values_start: 1408, raw_values_count: 1, token_types_start: 1409, token_types_count: 0 }, // [768]
    SimpleHintData { raw_values_start: 1409, raw_values_count: 0, token_types_start: 1409, token_types_count: 1 }, // [769]
    SimpleHintData { raw_values_start: 1410, raw_values_count: 1, token_types_start: 1411, token_types_count: 0 }, // [770]
    SimpleHintData { raw_values_start: 1411, raw_values_count: 1, token_types_start: 1412, token_types_count: 0 }, // [771]
    SimpleHintData { raw_values_start: 1412, raw_values_count: 2, token_types_start: 1414, token_types_count: 0 }, // [772]
    SimpleHintData { raw_values_start: 1414, raw_values_count: 1, token_types_start: 1415, token_types_count: 0 }, // [773]
    SimpleHintData { raw_values_start: 1415, raw_values_count: 6, token_types_start: 1421, token_types_count: 0 }, // [774]
    SimpleHintData { raw_values_start: 1421, raw_values_count: 2, token_types_start: 1423, token_types_count: 0 }, // [775]
    SimpleHintData { raw_values_start: 1423, raw_values_count: 1, token_types_start: 1424, token_types_count: 0 }, // [776]
    SimpleHintData { raw_values_start: 1424, raw_values_count: 2, token_types_start: 1426, token_types_count: 0 }, // [777]
    SimpleHintData { raw_values_start: 1426, raw_values_count: 1, token_types_start: 1427, token_types_count: 0 }, // [778]
    SimpleHintData { raw_values_start: 1427, raw_values_count: 1, token_types_start: 1428, token_types_count: 0 }, // [779]
    SimpleHintData { raw_values_start: 1428, raw_values_count: 1, token_types_start: 1429, token_types_count: 0 }, // [780]
    SimpleHintData { raw_values_start: 1429, raw_values_count: 1, token_types_start: 1430, token_types_count: 0 }, // [781]
    SimpleHintData { raw_values_start: 1430, raw_values_count: 1, token_types_start: 1431, token_types_count: 0 }, // [782]
    SimpleHintData { raw_values_start: 1431, raw_values_count: 6, token_types_start: 1437, token_types_count: 0 }, // [783]
    SimpleHintData { raw_values_start: 1437, raw_values_count: 1, token_types_start: 1438, token_types_count: 0 }, // [784]
    SimpleHintData { raw_values_start: 1438, raw_values_count: 1, token_types_start: 1439, token_types_count: 0 }, // [785]
    SimpleHintData { raw_values_start: 1439, raw_values_count: 1, token_types_start: 1440, token_types_count: 0 }, // [786]
    SimpleHintData { raw_values_start: 1440, raw_values_count: 5, token_types_start: 1445, token_types_count: 0 }, // [787]
    SimpleHintData { raw_values_start: 1445, raw_values_count: 1, token_types_start: 1446, token_types_count: 0 }, // [788]
    SimpleHintData { raw_values_start: 1446, raw_values_count: 1, token_types_start: 1447, token_types_count: 0 }, // [789]
    SimpleHintData { raw_values_start: 1447, raw_values_count: 2, token_types_start: 1449, token_types_count: 0 }, // [790]
    SimpleHintData { raw_values_start: 1449, raw_values_count: 1, token_types_start: 1450, token_types_count: 0 }, // [791]
    SimpleHintData { raw_values_start: 1450, raw_values_count: 1, token_types_start: 1451, token_types_count: 0 }, // [792]
    SimpleHintData { raw_values_start: 1451, raw_values_count: 1, token_types_start: 1452, token_types_count: 0 }, // [793]
    SimpleHintData { raw_values_start: 1452, raw_values_count: 1, token_types_start: 1453, token_types_count: 0 }, // [794]
    SimpleHintData { raw_values_start: 1453, raw_values_count: 1, token_types_start: 1454, token_types_count: 0 }, // [795]
    SimpleHintData { raw_values_start: 1454, raw_values_count: 1, token_types_start: 1455, token_types_count: 0 }, // [796]
    SimpleHintData { raw_values_start: 1455, raw_values_count: 1, token_types_start: 1456, token_types_count: 0 }, // [797]
    SimpleHintData { raw_values_start: 1456, raw_values_count: 1, token_types_start: 1457, token_types_count: 0 }, // [798]
    SimpleHintData { raw_values_start: 1457, raw_values_count: 1, token_types_start: 1458, token_types_count: 0 }, // [799]
    SimpleHintData { raw_values_start: 1458, raw_values_count: 1, token_types_start: 1459, token_types_count: 0 }, // [800]
    SimpleHintData { raw_values_start: 1459, raw_values_count: 2, token_types_start: 1461, token_types_count: 0 }, // [801]
    SimpleHintData { raw_values_start: 1461, raw_values_count: 2, token_types_start: 1463, token_types_count: 0 }, // [802]
    SimpleHintData { raw_values_start: 1463, raw_values_count: 1, token_types_start: 1464, token_types_count: 0 }, // [803]
    SimpleHintData { raw_values_start: 1464, raw_values_count: 5, token_types_start: 1469, token_types_count: 0 }, // [804]
    SimpleHintData { raw_values_start: 1469, raw_values_count: 1, token_types_start: 1470, token_types_count: 0 }, // [805]
    SimpleHintData { raw_values_start: 1470, raw_values_count: 1, token_types_start: 1471, token_types_count: 0 }, // [806]
    SimpleHintData { raw_values_start: 1471, raw_values_count: 1, token_types_start: 1472, token_types_count: 0 }, // [807]
    SimpleHintData { raw_values_start: 1472, raw_values_count: 1, token_types_start: 1473, token_types_count: 0 }, // [808]
    SimpleHintData { raw_values_start: 1473, raw_values_count: 1, token_types_start: 1474, token_types_count: 0 }, // [809]
    SimpleHintData { raw_values_start: 1474, raw_values_count: 1, token_types_start: 1475, token_types_count: 0 }, // [810]
    SimpleHintData { raw_values_start: 1475, raw_values_count: 1, token_types_start: 1476, token_types_count: 0 }, // [811]
    SimpleHintData { raw_values_start: 1476, raw_values_count: 1, token_types_start: 1477, token_types_count: 0 }, // [812]
    SimpleHintData { raw_values_start: 1477, raw_values_count: 1, token_types_start: 1478, token_types_count: 0 }, // [813]
    SimpleHintData { raw_values_start: 1478, raw_values_count: 1, token_types_start: 1479, token_types_count: 0 }, // [814]
    SimpleHintData { raw_values_start: 1479, raw_values_count: 1, token_types_start: 1480, token_types_count: 0 }, // [815]
    SimpleHintData { raw_values_start: 1480, raw_values_count: 1, token_types_start: 1481, token_types_count: 0 }, // [816]
    SimpleHintData { raw_values_start: 1481, raw_values_count: 1, token_types_start: 1482, token_types_count: 0 }, // [817]
    SimpleHintData { raw_values_start: 1482, raw_values_count: 6, token_types_start: 1488, token_types_count: 0 }, // [818]
    SimpleHintData { raw_values_start: 1488, raw_values_count: 1, token_types_start: 1489, token_types_count: 0 }, // [819]
    SimpleHintData { raw_values_start: 1489, raw_values_count: 1, token_types_start: 1490, token_types_count: 0 }, // [820]
    SimpleHintData { raw_values_start: 1490, raw_values_count: 1, token_types_start: 1491, token_types_count: 0 }, // [821]
    SimpleHintData { raw_values_start: 1491, raw_values_count: 1, token_types_start: 1492, token_types_count: 0 }, // [822]
    SimpleHintData { raw_values_start: 1492, raw_values_count: 1, token_types_start: 1493, token_types_count: 0 }, // [823]
    SimpleHintData { raw_values_start: 1493, raw_values_count: 1, token_types_start: 1494, token_types_count: 0 }, // [824]
    SimpleHintData { raw_values_start: 1494, raw_values_count: 2, token_types_start: 1496, token_types_count: 0 }, // [825]
    SimpleHintData { raw_values_start: 1496, raw_values_count: 1, token_types_start: 1497, token_types_count: 0 }, // [826]
    SimpleHintData { raw_values_start: 1497, raw_values_count: 1, token_types_start: 1498, token_types_count: 0 }, // [827]
    SimpleHintData { raw_values_start: 1498, raw_values_count: 1, token_types_start: 1499, token_types_count: 0 }, // [828]
    SimpleHintData { raw_values_start: 1499, raw_values_count: 1, token_types_start: 1500, token_types_count: 0 }, // [829]
    SimpleHintData { raw_values_start: 1500, raw_values_count: 1, token_types_start: 1501, token_types_count: 0 }, // [830]
    SimpleHintData { raw_values_start: 1501, raw_values_count: 1, token_types_start: 1502, token_types_count: 0 }, // [831]
    SimpleHintData { raw_values_start: 1502, raw_values_count: 1, token_types_start: 1503, token_types_count: 0 }, // [832]
    SimpleHintData { raw_values_start: 1503, raw_values_count: 1, token_types_start: 1504, token_types_count: 0 }, // [833]
    SimpleHintData { raw_values_start: 1504, raw_values_count: 1, token_types_start: 1505, token_types_count: 0 }, // [834]
    SimpleHintData { raw_values_start: 1505, raw_values_count: 1, token_types_start: 1506, token_types_count: 0 }, // [835]
    SimpleHintData { raw_values_start: 1506, raw_values_count: 1, token_types_start: 1507, token_types_count: 0 }, // [836]
    SimpleHintData { raw_values_start: 1507, raw_values_count: 1, token_types_start: 1508, token_types_count: 0 }, // [837]
    SimpleHintData { raw_values_start: 1508, raw_values_count: 1, token_types_start: 1509, token_types_count: 0 }, // [838]
    SimpleHintData { raw_values_start: 1509, raw_values_count: 1, token_types_start: 1510, token_types_count: 0 }, // [839]
    SimpleHintData { raw_values_start: 1510, raw_values_count: 2, token_types_start: 1512, token_types_count: 0 }, // [840]
    SimpleHintData { raw_values_start: 1512, raw_values_count: 7, token_types_start: 1519, token_types_count: 0 }, // [841]
    SimpleHintData { raw_values_start: 1519, raw_values_count: 3, token_types_start: 1522, token_types_count: 0 }, // [842]
    SimpleHintData { raw_values_start: 1522, raw_values_count: 6, token_types_start: 1528, token_types_count: 0 }, // [843]
    SimpleHintData { raw_values_start: 1528, raw_values_count: 2, token_types_start: 1530, token_types_count: 0 }, // [844]
    SimpleHintData { raw_values_start: 1530, raw_values_count: 1, token_types_start: 1531, token_types_count: 0 }, // [845]
    SimpleHintData { raw_values_start: 1531, raw_values_count: 1, token_types_start: 1532, token_types_count: 0 }, // [846]
    SimpleHintData { raw_values_start: 1532, raw_values_count: 1, token_types_start: 1533, token_types_count: 0 }, // [847]
    SimpleHintData { raw_values_start: 1533, raw_values_count: 1, token_types_start: 1534, token_types_count: 0 }, // [848]
    SimpleHintData { raw_values_start: 1534, raw_values_count: 1, token_types_start: 1535, token_types_count: 0 }, // [849]
    SimpleHintData { raw_values_start: 1535, raw_values_count: 1, token_types_start: 1536, token_types_count: 0 }, // [850]
    SimpleHintData { raw_values_start: 1536, raw_values_count: 1, token_types_start: 1537, token_types_count: 0 }, // [851]
    SimpleHintData { raw_values_start: 1537, raw_values_count: 1, token_types_start: 1538, token_types_count: 0 }, // [852]
    SimpleHintData { raw_values_start: 1538, raw_values_count: 1, token_types_start: 1539, token_types_count: 0 }, // [853]
    SimpleHintData { raw_values_start: 1539, raw_values_count: 1, token_types_start: 1540, token_types_count: 0 }, // [854]
    SimpleHintData { raw_values_start: 1540, raw_values_count: 1, token_types_start: 1541, token_types_count: 0 }, // [855]
    SimpleHintData { raw_values_start: 1541, raw_values_count: 1, token_types_start: 1542, token_types_count: 0 }, // [856]
    SimpleHintData { raw_values_start: 1542, raw_values_count: 1, token_types_start: 1543, token_types_count: 0 }, // [857]
    SimpleHintData { raw_values_start: 1543, raw_values_count: 1, token_types_start: 1544, token_types_count: 0 }, // [858]
    SimpleHintData { raw_values_start: 1544, raw_values_count: 1, token_types_start: 1545, token_types_count: 0 }, // [859]
    SimpleHintData { raw_values_start: 1545, raw_values_count: 1, token_types_start: 1546, token_types_count: 0 }, // [860]
    SimpleHintData { raw_values_start: 1546, raw_values_count: 2, token_types_start: 1548, token_types_count: 0 }, // [861]
    SimpleHintData { raw_values_start: 1548, raw_values_count: 1, token_types_start: 1549, token_types_count: 0 }, // [862]
    SimpleHintData { raw_values_start: 1549, raw_values_count: 1, token_types_start: 1550, token_types_count: 0 }, // [863]
    SimpleHintData { raw_values_start: 1550, raw_values_count: 1, token_types_start: 1551, token_types_count: 0 }, // [864]
    SimpleHintData { raw_values_start: 1551, raw_values_count: 0, token_types_start: 1551, token_types_count: 2 }, // [865]
    SimpleHintData { raw_values_start: 1553, raw_values_count: 11, token_types_start: 1564, token_types_count: 1 }, // [866]
    SimpleHintData { raw_values_start: 1565, raw_values_count: 1, token_types_start: 1566, token_types_count: 0 }, // [867]
    SimpleHintData { raw_values_start: 1566, raw_values_count: 1, token_types_start: 1567, token_types_count: 0 }, // [868]
    SimpleHintData { raw_values_start: 1567, raw_values_count: 2, token_types_start: 1569, token_types_count: 0 }, // [869]
    SimpleHintData { raw_values_start: 1569, raw_values_count: 1, token_types_start: 1570, token_types_count: 0 }, // [870]
    SimpleHintData { raw_values_start: 1570, raw_values_count: 1, token_types_start: 1571, token_types_count: 0 }, // [871]
    SimpleHintData { raw_values_start: 1571, raw_values_count: 1, token_types_start: 1572, token_types_count: 0 }, // [872]
    SimpleHintData { raw_values_start: 1572, raw_values_count: 1, token_types_start: 1573, token_types_count: 0 }, // [873]
    SimpleHintData { raw_values_start: 1573, raw_values_count: 1, token_types_start: 1574, token_types_count: 0 }, // [874]
    SimpleHintData { raw_values_start: 1574, raw_values_count: 1, token_types_start: 1575, token_types_count: 0 }, // [875]
    SimpleHintData { raw_values_start: 1575, raw_values_count: 2, token_types_start: 1577, token_types_count: 0 }, // [876]
    SimpleHintData { raw_values_start: 1577, raw_values_count: 1, token_types_start: 1578, token_types_count: 0 }, // [877]
    SimpleHintData { raw_values_start: 1578, raw_values_count: 1, token_types_start: 1579, token_types_count: 0 }, // [878]
    SimpleHintData { raw_values_start: 1579, raw_values_count: 1, token_types_start: 1580, token_types_count: 1 }, // [879]
    SimpleHintData { raw_values_start: 1581, raw_values_count: 4, token_types_start: 1585, token_types_count: 0 }, // [880]
    SimpleHintData { raw_values_start: 1585, raw_values_count: 1, token_types_start: 1586, token_types_count: 0 }, // [881]
    SimpleHintData { raw_values_start: 1586, raw_values_count: 2, token_types_start: 1588, token_types_count: 0 }, // [882]
    SimpleHintData { raw_values_start: 1588, raw_values_count: 1, token_types_start: 1589, token_types_count: 0 }, // [883]
    SimpleHintData { raw_values_start: 1589, raw_values_count: 2, token_types_start: 1591, token_types_count: 0 }, // [884]
    SimpleHintData { raw_values_start: 1591, raw_values_count: 1, token_types_start: 1592, token_types_count: 0 }, // [885]
    SimpleHintData { raw_values_start: 1592, raw_values_count: 1, token_types_start: 1593, token_types_count: 0 }, // [886]
    SimpleHintData { raw_values_start: 1593, raw_values_count: 3, token_types_start: 1596, token_types_count: 0 }, // [887]
    SimpleHintData { raw_values_start: 1596, raw_values_count: 1, token_types_start: 1597, token_types_count: 0 }, // [888]
    SimpleHintData { raw_values_start: 1597, raw_values_count: 1, token_types_start: 1598, token_types_count: 0 }, // [889]
    SimpleHintData { raw_values_start: 1598, raw_values_count: 1, token_types_start: 1599, token_types_count: 0 }, // [890]
    SimpleHintData { raw_values_start: 1599, raw_values_count: 1, token_types_start: 1600, token_types_count: 0 }, // [891]
    SimpleHintData { raw_values_start: 1600, raw_values_count: 1, token_types_start: 1601, token_types_count: 0 }, // [892]
    SimpleHintData { raw_values_start: 1601, raw_values_count: 3, token_types_start: 1604, token_types_count: 0 }, // [893]
    SimpleHintData { raw_values_start: 1604, raw_values_count: 2, token_types_start: 1606, token_types_count: 0 }, // [894]
    SimpleHintData { raw_values_start: 1606, raw_values_count: 2, token_types_start: 1608, token_types_count: 0 }, // [895]
    SimpleHintData { raw_values_start: 1608, raw_values_count: 1, token_types_start: 1609, token_types_count: 0 }, // [896]
    SimpleHintData { raw_values_start: 1609, raw_values_count: 2, token_types_start: 1611, token_types_count: 0 }, // [897]
    SimpleHintData { raw_values_start: 1611, raw_values_count: 1, token_types_start: 1612, token_types_count: 0 }, // [898]
    SimpleHintData { raw_values_start: 1612, raw_values_count: 1, token_types_start: 1613, token_types_count: 0 }, // [899]
    SimpleHintData { raw_values_start: 1613, raw_values_count: 1, token_types_start: 1614, token_types_count: 0 }, // [900]
    SimpleHintData { raw_values_start: 1614, raw_values_count: 1, token_types_start: 1615, token_types_count: 0 }, // [901]
    SimpleHintData { raw_values_start: 1615, raw_values_count: 1, token_types_start: 1616, token_types_count: 0 }, // [902]
    SimpleHintData { raw_values_start: 1616, raw_values_count: 1, token_types_start: 1617, token_types_count: 0 }, // [903]
    SimpleHintData { raw_values_start: 1617, raw_values_count: 1, token_types_start: 1618, token_types_count: 0 }, // [904]
    SimpleHintData { raw_values_start: 1618, raw_values_count: 1, token_types_start: 1619, token_types_count: 0 }, // [905]
    SimpleHintData { raw_values_start: 1619, raw_values_count: 1, token_types_start: 1620, token_types_count: 0 }, // [906]
    SimpleHintData { raw_values_start: 1620, raw_values_count: 1, token_types_start: 1621, token_types_count: 0 }, // [907]
    SimpleHintData { raw_values_start: 1621, raw_values_count: 1, token_types_start: 1622, token_types_count: 0 }, // [908]
    SimpleHintData { raw_values_start: 1622, raw_values_count: 3, token_types_start: 1625, token_types_count: 0 }, // [909]
    SimpleHintData { raw_values_start: 1625, raw_values_count: 1, token_types_start: 1626, token_types_count: 0 }, // [910]
    SimpleHintData { raw_values_start: 1626, raw_values_count: 2, token_types_start: 1628, token_types_count: 0 }, // [911]
    SimpleHintData { raw_values_start: 1628, raw_values_count: 1, token_types_start: 1629, token_types_count: 0 }, // [912]
    SimpleHintData { raw_values_start: 1629, raw_values_count: 1, token_types_start: 1630, token_types_count: 0 }, // [913]
    SimpleHintData { raw_values_start: 1630, raw_values_count: 1, token_types_start: 1631, token_types_count: 0 }, // [914]
    SimpleHintData { raw_values_start: 1631, raw_values_count: 2, token_types_start: 1633, token_types_count: 0 }, // [915]
    SimpleHintData { raw_values_start: 1633, raw_values_count: 1, token_types_start: 1634, token_types_count: 0 }, // [916]
    SimpleHintData { raw_values_start: 1634, raw_values_count: 2, token_types_start: 1636, token_types_count: 0 }, // [917]
    SimpleHintData { raw_values_start: 1636, raw_values_count: 1, token_types_start: 1637, token_types_count: 0 }, // [918]
    SimpleHintData { raw_values_start: 1637, raw_values_count: 1, token_types_start: 1638, token_types_count: 0 }, // [919]
    SimpleHintData { raw_values_start: 1638, raw_values_count: 1, token_types_start: 1639, token_types_count: 0 }, // [920]
    SimpleHintData { raw_values_start: 1639, raw_values_count: 1, token_types_start: 1640, token_types_count: 0 }, // [921]
    SimpleHintData { raw_values_start: 1640, raw_values_count: 1, token_types_start: 1641, token_types_count: 0 }, // [922]
    SimpleHintData { raw_values_start: 1641, raw_values_count: 1, token_types_start: 1642, token_types_count: 0 }, // [923]
    SimpleHintData { raw_values_start: 1642, raw_values_count: 1, token_types_start: 1643, token_types_count: 0 }, // [924]
    SimpleHintData { raw_values_start: 1643, raw_values_count: 1, token_types_start: 1644, token_types_count: 0 }, // [925]
    SimpleHintData { raw_values_start: 1644, raw_values_count: 1, token_types_start: 1645, token_types_count: 0 }, // [926]
    SimpleHintData { raw_values_start: 1645, raw_values_count: 1, token_types_start: 1646, token_types_count: 0 }, // [927]
    SimpleHintData { raw_values_start: 1646, raw_values_count: 1, token_types_start: 1647, token_types_count: 0 }, // [928]
    SimpleHintData { raw_values_start: 1647, raw_values_count: 1, token_types_start: 1648, token_types_count: 0 }, // [929]
    SimpleHintData { raw_values_start: 1648, raw_values_count: 1, token_types_start: 1649, token_types_count: 0 }, // [930]
    SimpleHintData { raw_values_start: 1649, raw_values_count: 1, token_types_start: 1650, token_types_count: 0 }, // [931]
    SimpleHintData { raw_values_start: 1650, raw_values_count: 1, token_types_start: 1651, token_types_count: 0 }, // [932]
    SimpleHintData { raw_values_start: 1651, raw_values_count: 1, token_types_start: 1652, token_types_count: 0 }, // [933]
    SimpleHintData { raw_values_start: 1652, raw_values_count: 1, token_types_start: 1653, token_types_count: 0 }, // [934]
    SimpleHintData { raw_values_start: 1653, raw_values_count: 1, token_types_start: 1654, token_types_count: 0 }, // [935]
    SimpleHintData { raw_values_start: 1654, raw_values_count: 1, token_types_start: 1655, token_types_count: 0 }, // [936]
    SimpleHintData { raw_values_start: 1655, raw_values_count: 1, token_types_start: 1656, token_types_count: 0 }, // [937]
    SimpleHintData { raw_values_start: 1656, raw_values_count: 1, token_types_start: 1657, token_types_count: 0 }, // [938]
    SimpleHintData { raw_values_start: 1657, raw_values_count: 1, token_types_start: 1658, token_types_count: 0 }, // [939]
    SimpleHintData { raw_values_start: 1658, raw_values_count: 1, token_types_start: 1659, token_types_count: 0 }, // [940]
    SimpleHintData { raw_values_start: 1659, raw_values_count: 1, token_types_start: 1660, token_types_count: 0 }, // [941]
    SimpleHintData { raw_values_start: 1660, raw_values_count: 1, token_types_start: 1661, token_types_count: 0 }, // [942]
    SimpleHintData { raw_values_start: 1661, raw_values_count: 1, token_types_start: 1662, token_types_count: 0 }, // [943]
    SimpleHintData { raw_values_start: 1662, raw_values_count: 1, token_types_start: 1663, token_types_count: 0 }, // [944]
    SimpleHintData { raw_values_start: 1663, raw_values_count: 1, token_types_start: 1664, token_types_count: 0 }, // [945]
    SimpleHintData { raw_values_start: 1664, raw_values_count: 1, token_types_start: 1665, token_types_count: 0 }, // [946]
    SimpleHintData { raw_values_start: 1665, raw_values_count: 1, token_types_start: 1666, token_types_count: 0 }, // [947]
    SimpleHintData { raw_values_start: 1666, raw_values_count: 1, token_types_start: 1667, token_types_count: 0 }, // [948]
    SimpleHintData { raw_values_start: 1667, raw_values_count: 1, token_types_start: 1668, token_types_count: 0 }, // [949]
    SimpleHintData { raw_values_start: 1668, raw_values_count: 1, token_types_start: 1669, token_types_count: 0 }, // [950]
    SimpleHintData { raw_values_start: 1669, raw_values_count: 1, token_types_start: 1670, token_types_count: 0 }, // [951]
    SimpleHintData { raw_values_start: 1670, raw_values_count: 1, token_types_start: 1671, token_types_count: 0 }, // [952]
    SimpleHintData { raw_values_start: 1671, raw_values_count: 2, token_types_start: 1673, token_types_count: 0 }, // [953]
    SimpleHintData { raw_values_start: 1673, raw_values_count: 1, token_types_start: 1674, token_types_count: 0 }, // [954]
    SimpleHintData { raw_values_start: 1674, raw_values_count: 1, token_types_start: 1675, token_types_count: 0 }, // [955]
    SimpleHintData { raw_values_start: 1675, raw_values_count: 1, token_types_start: 1676, token_types_count: 0 }, // [956]
    SimpleHintData { raw_values_start: 1676, raw_values_count: 1, token_types_start: 1677, token_types_count: 0 }, // [957]
    SimpleHintData { raw_values_start: 1677, raw_values_count: 1, token_types_start: 1678, token_types_count: 0 }, // [958]
    SimpleHintData { raw_values_start: 1678, raw_values_count: 1, token_types_start: 1679, token_types_count: 0 }, // [959]
    SimpleHintData { raw_values_start: 1679, raw_values_count: 1, token_types_start: 1680, token_types_count: 0 }, // [960]
    SimpleHintData { raw_values_start: 1680, raw_values_count: 1, token_types_start: 1681, token_types_count: 0 }, // [961]
    SimpleHintData { raw_values_start: 1681, raw_values_count: 1, token_types_start: 1682, token_types_count: 0 }, // [962]
    SimpleHintData { raw_values_start: 1682, raw_values_count: 1, token_types_start: 1683, token_types_count: 0 }, // [963]
    SimpleHintData { raw_values_start: 1683, raw_values_count: 1, token_types_start: 1684, token_types_count: 0 }, // [964]
    SimpleHintData { raw_values_start: 1684, raw_values_count: 1, token_types_start: 1685, token_types_count: 0 }, // [965]
    SimpleHintData { raw_values_start: 1685, raw_values_count: 1, token_types_start: 1686, token_types_count: 0 }, // [966]
    SimpleHintData { raw_values_start: 1686, raw_values_count: 1, token_types_start: 1687, token_types_count: 0 }, // [967]
    SimpleHintData { raw_values_start: 1687, raw_values_count: 1, token_types_start: 1688, token_types_count: 0 }, // [968]
    SimpleHintData { raw_values_start: 1688, raw_values_count: 1, token_types_start: 1689, token_types_count: 0 }, // [969]
    SimpleHintData { raw_values_start: 1689, raw_values_count: 1, token_types_start: 1690, token_types_count: 0 }, // [970]
    SimpleHintData { raw_values_start: 1690, raw_values_count: 1, token_types_start: 1691, token_types_count: 0 }, // [971]
    SimpleHintData { raw_values_start: 1691, raw_values_count: 1, token_types_start: 1692, token_types_count: 0 }, // [972]
    SimpleHintData { raw_values_start: 1692, raw_values_count: 1, token_types_start: 1693, token_types_count: 0 }, // [973]
    SimpleHintData { raw_values_start: 1693, raw_values_count: 3, token_types_start: 1696, token_types_count: 0 }, // [974]
    SimpleHintData { raw_values_start: 1696, raw_values_count: 1, token_types_start: 1697, token_types_count: 0 }, // [975]
    SimpleHintData { raw_values_start: 1697, raw_values_count: 2, token_types_start: 1699, token_types_count: 0 }, // [976]
    SimpleHintData { raw_values_start: 1699, raw_values_count: 1, token_types_start: 1700, token_types_count: 0 }, // [977]
    SimpleHintData { raw_values_start: 1700, raw_values_count: 1, token_types_start: 1701, token_types_count: 0 }, // [978]
    SimpleHintData { raw_values_start: 1701, raw_values_count: 1, token_types_start: 1702, token_types_count: 0 }, // [979]
    SimpleHintData { raw_values_start: 1702, raw_values_count: 1, token_types_start: 1703, token_types_count: 0 }, // [980]
    SimpleHintData { raw_values_start: 1703, raw_values_count: 1, token_types_start: 1704, token_types_count: 0 }, // [981]
    SimpleHintData { raw_values_start: 1704, raw_values_count: 1, token_types_start: 1705, token_types_count: 0 }, // [982]
    SimpleHintData { raw_values_start: 1705, raw_values_count: 1, token_types_start: 1706, token_types_count: 0 }, // [983]
    SimpleHintData { raw_values_start: 1706, raw_values_count: 1, token_types_start: 1707, token_types_count: 0 }, // [984]
    SimpleHintData { raw_values_start: 1707, raw_values_count: 1, token_types_start: 1708, token_types_count: 0 }, // [985]
    SimpleHintData { raw_values_start: 1708, raw_values_count: 1, token_types_start: 1709, token_types_count: 0 }, // [986]
    SimpleHintData { raw_values_start: 1709, raw_values_count: 1, token_types_start: 1710, token_types_count: 0 }, // [987]
    SimpleHintData { raw_values_start: 1710, raw_values_count: 1, token_types_start: 1711, token_types_count: 0 }, // [988]
    SimpleHintData { raw_values_start: 1711, raw_values_count: 1, token_types_start: 1712, token_types_count: 0 }, // [989]
    SimpleHintData { raw_values_start: 1712, raw_values_count: 1, token_types_start: 1713, token_types_count: 0 }, // [990]
    SimpleHintData { raw_values_start: 1713, raw_values_count: 1, token_types_start: 1714, token_types_count: 0 }, // [991]
    SimpleHintData { raw_values_start: 1714, raw_values_count: 1, token_types_start: 1715, token_types_count: 0 }, // [992]
    SimpleHintData { raw_values_start: 1715, raw_values_count: 1, token_types_start: 1716, token_types_count: 0 }, // [993]
    SimpleHintData { raw_values_start: 1716, raw_values_count: 1, token_types_start: 1717, token_types_count: 0 }, // [994]
    SimpleHintData { raw_values_start: 1717, raw_values_count: 1, token_types_start: 1718, token_types_count: 0 }, // [995]
    SimpleHintData { raw_values_start: 1718, raw_values_count: 1, token_types_start: 1719, token_types_count: 0 }, // [996]
    SimpleHintData { raw_values_start: 1719, raw_values_count: 4, token_types_start: 1723, token_types_count: 0 }, // [997]
    SimpleHintData { raw_values_start: 1723, raw_values_count: 2, token_types_start: 1725, token_types_count: 0 }, // [998]
    SimpleHintData { raw_values_start: 1725, raw_values_count: 1, token_types_start: 1726, token_types_count: 0 }, // [999]
    SimpleHintData { raw_values_start: 1726, raw_values_count: 1, token_types_start: 1727, token_types_count: 0 }, // [1000]
    SimpleHintData { raw_values_start: 1727, raw_values_count: 1, token_types_start: 1728, token_types_count: 0 }, // [1001]
    SimpleHintData { raw_values_start: 1728, raw_values_count: 1, token_types_start: 1729, token_types_count: 0 }, // [1002]
    SimpleHintData { raw_values_start: 1729, raw_values_count: 1, token_types_start: 1730, token_types_count: 0 }, // [1003]
    SimpleHintData { raw_values_start: 1730, raw_values_count: 1, token_types_start: 1731, token_types_count: 0 }, // [1004]
    SimpleHintData { raw_values_start: 1731, raw_values_count: 1, token_types_start: 1732, token_types_count: 0 }, // [1005]
    SimpleHintData { raw_values_start: 1732, raw_values_count: 1, token_types_start: 1733, token_types_count: 0 }, // [1006]
    SimpleHintData { raw_values_start: 1733, raw_values_count: 1, token_types_start: 1734, token_types_count: 0 }, // [1007]
    SimpleHintData { raw_values_start: 1734, raw_values_count: 1, token_types_start: 1735, token_types_count: 0 }, // [1008]
    SimpleHintData { raw_values_start: 1735, raw_values_count: 1, token_types_start: 1736, token_types_count: 0 }, // [1009]
    SimpleHintData { raw_values_start: 1736, raw_values_count: 1, token_types_start: 1737, token_types_count: 0 }, // [1010]
    SimpleHintData { raw_values_start: 1737, raw_values_count: 1, token_types_start: 1738, token_types_count: 0 }, // [1011]
    SimpleHintData { raw_values_start: 1738, raw_values_count: 1, token_types_start: 1739, token_types_count: 0 }, // [1012]
    SimpleHintData { raw_values_start: 1739, raw_values_count: 4, token_types_start: 1743, token_types_count: 0 }, // [1013]
    SimpleHintData { raw_values_start: 1743, raw_values_count: 3, token_types_start: 1746, token_types_count: 0 }, // [1014]
    SimpleHintData { raw_values_start: 1746, raw_values_count: 1, token_types_start: 1747, token_types_count: 0 }, // [1015]
    SimpleHintData { raw_values_start: 1747, raw_values_count: 1, token_types_start: 1748, token_types_count: 0 }, // [1016]
    SimpleHintData { raw_values_start: 1748, raw_values_count: 1, token_types_start: 1749, token_types_count: 0 }, // [1017]
    SimpleHintData { raw_values_start: 1749, raw_values_count: 1, token_types_start: 1750, token_types_count: 0 }, // [1018]
    SimpleHintData { raw_values_start: 1750, raw_values_count: 1, token_types_start: 1751, token_types_count: 0 }, // [1019]
    SimpleHintData { raw_values_start: 1751, raw_values_count: 1, token_types_start: 1752, token_types_count: 0 }, // [1020]
    SimpleHintData { raw_values_start: 1752, raw_values_count: 1, token_types_start: 1753, token_types_count: 0 }, // [1021]
    SimpleHintData { raw_values_start: 1753, raw_values_count: 1, token_types_start: 1754, token_types_count: 0 }, // [1022]
    SimpleHintData { raw_values_start: 1754, raw_values_count: 1, token_types_start: 1755, token_types_count: 0 }, // [1023]
    SimpleHintData { raw_values_start: 1755, raw_values_count: 1, token_types_start: 1756, token_types_count: 0 }, // [1024]
    SimpleHintData { raw_values_start: 1756, raw_values_count: 3, token_types_start: 1759, token_types_count: 0 }, // [1025]
    SimpleHintData { raw_values_start: 1759, raw_values_count: 2, token_types_start: 1761, token_types_count: 0 }, // [1026]
    SimpleHintData { raw_values_start: 1761, raw_values_count: 1, token_types_start: 1762, token_types_count: 0 }, // [1027]
    SimpleHintData { raw_values_start: 1762, raw_values_count: 2, token_types_start: 1764, token_types_count: 0 }, // [1028]
    SimpleHintData { raw_values_start: 1764, raw_values_count: 1, token_types_start: 1765, token_types_count: 0 }, // [1029]
    SimpleHintData { raw_values_start: 1765, raw_values_count: 1, token_types_start: 1766, token_types_count: 0 }, // [1030]
    SimpleHintData { raw_values_start: 1766, raw_values_count: 1, token_types_start: 1767, token_types_count: 0 }, // [1031]
    SimpleHintData { raw_values_start: 1767, raw_values_count: 1, token_types_start: 1768, token_types_count: 0 }, // [1032]
    SimpleHintData { raw_values_start: 1768, raw_values_count: 1, token_types_start: 1769, token_types_count: 0 }, // [1033]
    SimpleHintData { raw_values_start: 1769, raw_values_count: 1, token_types_start: 1770, token_types_count: 0 }, // [1034]
    SimpleHintData { raw_values_start: 1770, raw_values_count: 1, token_types_start: 1771, token_types_count: 0 }, // [1035]
    SimpleHintData { raw_values_start: 1771, raw_values_count: 1, token_types_start: 1772, token_types_count: 0 }, // [1036]
    SimpleHintData { raw_values_start: 1772, raw_values_count: 1, token_types_start: 1773, token_types_count: 0 }, // [1037]
    SimpleHintData { raw_values_start: 1773, raw_values_count: 1, token_types_start: 1774, token_types_count: 0 }, // [1038]
    SimpleHintData { raw_values_start: 1774, raw_values_count: 1, token_types_start: 1775, token_types_count: 0 }, // [1039]
    SimpleHintData { raw_values_start: 1775, raw_values_count: 1, token_types_start: 1776, token_types_count: 0 }, // [1040]
    SimpleHintData { raw_values_start: 1776, raw_values_count: 1, token_types_start: 1777, token_types_count: 0 }, // [1041]
    SimpleHintData { raw_values_start: 1777, raw_values_count: 2, token_types_start: 1779, token_types_count: 0 }, // [1042]
    SimpleHintData { raw_values_start: 1779, raw_values_count: 2, token_types_start: 1781, token_types_count: 0 }, // [1043]
    SimpleHintData { raw_values_start: 1781, raw_values_count: 2, token_types_start: 1783, token_types_count: 0 }, // [1044]
    SimpleHintData { raw_values_start: 1783, raw_values_count: 2, token_types_start: 1785, token_types_count: 0 }, // [1045]
    SimpleHintData { raw_values_start: 1785, raw_values_count: 2, token_types_start: 1787, token_types_count: 0 }, // [1046]
    SimpleHintData { raw_values_start: 1787, raw_values_count: 2, token_types_start: 1789, token_types_count: 0 }, // [1047]
    SimpleHintData { raw_values_start: 1789, raw_values_count: 1, token_types_start: 1790, token_types_count: 0 }, // [1048]
    SimpleHintData { raw_values_start: 1790, raw_values_count: 4, token_types_start: 1794, token_types_count: 0 }, // [1049]
    SimpleHintData { raw_values_start: 1794, raw_values_count: 3, token_types_start: 1797, token_types_count: 0 }, // [1050]
    SimpleHintData { raw_values_start: 1797, raw_values_count: 2, token_types_start: 1799, token_types_count: 0 }, // [1051]
    SimpleHintData { raw_values_start: 1799, raw_values_count: 1, token_types_start: 1800, token_types_count: 0 }, // [1052]
    SimpleHintData { raw_values_start: 1800, raw_values_count: 2, token_types_start: 1802, token_types_count: 0 }, // [1053]
    SimpleHintData { raw_values_start: 1802, raw_values_count: 2, token_types_start: 1804, token_types_count: 0 }, // [1054]
    SimpleHintData { raw_values_start: 1804, raw_values_count: 2, token_types_start: 1806, token_types_count: 0 }, // [1055]
    SimpleHintData { raw_values_start: 1806, raw_values_count: 2, token_types_start: 1808, token_types_count: 0 }, // [1056]
    SimpleHintData { raw_values_start: 1808, raw_values_count: 2, token_types_start: 1810, token_types_count: 0 }, // [1057]
    SimpleHintData { raw_values_start: 1810, raw_values_count: 5, token_types_start: 1815, token_types_count: 0 }, // [1058]
    SimpleHintData { raw_values_start: 1815, raw_values_count: 2, token_types_start: 1817, token_types_count: 0 }, // [1059]
    SimpleHintData { raw_values_start: 1817, raw_values_count: 4, token_types_start: 1821, token_types_count: 0 }, // [1060]
    SimpleHintData { raw_values_start: 1821, raw_values_count: 1, token_types_start: 1822, token_types_count: 0 }, // [1061]
    SimpleHintData { raw_values_start: 1822, raw_values_count: 5, token_types_start: 1827, token_types_count: 0 }, // [1062]
    SimpleHintData { raw_values_start: 1827, raw_values_count: 2, token_types_start: 1829, token_types_count: 0 }, // [1063]
    SimpleHintData { raw_values_start: 1829, raw_values_count: 3, token_types_start: 1832, token_types_count: 0 }, // [1064]
    SimpleHintData { raw_values_start: 1832, raw_values_count: 1, token_types_start: 1833, token_types_count: 0 }, // [1065]
    SimpleHintData { raw_values_start: 1833, raw_values_count: 1, token_types_start: 1834, token_types_count: 0 }, // [1066]
    SimpleHintData { raw_values_start: 1834, raw_values_count: 1, token_types_start: 1835, token_types_count: 0 }, // [1067]
    SimpleHintData { raw_values_start: 1835, raw_values_count: 2, token_types_start: 1837, token_types_count: 0 }, // [1068]
    SimpleHintData { raw_values_start: 1837, raw_values_count: 2, token_types_start: 1839, token_types_count: 0 }, // [1069]
    SimpleHintData { raw_values_start: 1839, raw_values_count: 2, token_types_start: 1841, token_types_count: 0 }, // [1070]
    SimpleHintData { raw_values_start: 1841, raw_values_count: 1, token_types_start: 1842, token_types_count: 0 }, // [1071]
    SimpleHintData { raw_values_start: 1842, raw_values_count: 1, token_types_start: 1843, token_types_count: 0 }, // [1072]
    SimpleHintData { raw_values_start: 1843, raw_values_count: 8, token_types_start: 1851, token_types_count: 0 }, // [1073]
    SimpleHintData { raw_values_start: 1851, raw_values_count: 1, token_types_start: 1852, token_types_count: 0 }, // [1074]
    SimpleHintData { raw_values_start: 1852, raw_values_count: 1, token_types_start: 1853, token_types_count: 0 }, // [1075]
    SimpleHintData { raw_values_start: 1853, raw_values_count: 1, token_types_start: 1854, token_types_count: 0 }, // [1076]
    SimpleHintData { raw_values_start: 1854, raw_values_count: 1, token_types_start: 1855, token_types_count: 0 }, // [1077]
    SimpleHintData { raw_values_start: 1855, raw_values_count: 1, token_types_start: 1856, token_types_count: 0 }, // [1078]
    SimpleHintData { raw_values_start: 1856, raw_values_count: 1, token_types_start: 1857, token_types_count: 0 }, // [1079]
    SimpleHintData { raw_values_start: 1857, raw_values_count: 1, token_types_start: 1858, token_types_count: 0 }, // [1080]
    SimpleHintData { raw_values_start: 1858, raw_values_count: 1, token_types_start: 1859, token_types_count: 0 }, // [1081]
    SimpleHintData { raw_values_start: 1859, raw_values_count: 1, token_types_start: 1860, token_types_count: 0 }, // [1082]
    SimpleHintData { raw_values_start: 1860, raw_values_count: 1, token_types_start: 1861, token_types_count: 0 }, // [1083]
    SimpleHintData { raw_values_start: 1861, raw_values_count: 2, token_types_start: 1863, token_types_count: 0 }, // [1084]
    SimpleHintData { raw_values_start: 1863, raw_values_count: 1, token_types_start: 1864, token_types_count: 0 }, // [1085]
    SimpleHintData { raw_values_start: 1864, raw_values_count: 1, token_types_start: 1865, token_types_count: 0 }, // [1086]
    SimpleHintData { raw_values_start: 1865, raw_values_count: 1, token_types_start: 1866, token_types_count: 0 }, // [1087]
    SimpleHintData { raw_values_start: 1866, raw_values_count: 1, token_types_start: 1867, token_types_count: 0 }, // [1088]
    SimpleHintData { raw_values_start: 1867, raw_values_count: 1, token_types_start: 1868, token_types_count: 0 }, // [1089]
    SimpleHintData { raw_values_start: 1868, raw_values_count: 1, token_types_start: 1869, token_types_count: 0 }, // [1090]
    SimpleHintData { raw_values_start: 1869, raw_values_count: 1, token_types_start: 1870, token_types_count: 0 }, // [1091]
    SimpleHintData { raw_values_start: 1870, raw_values_count: 1, token_types_start: 1871, token_types_count: 0 }, // [1092]
    SimpleHintData { raw_values_start: 1871, raw_values_count: 1, token_types_start: 1872, token_types_count: 0 }, // [1093]
    SimpleHintData { raw_values_start: 1872, raw_values_count: 1, token_types_start: 1873, token_types_count: 0 }, // [1094]
    SimpleHintData { raw_values_start: 1873, raw_values_count: 1, token_types_start: 1874, token_types_count: 0 }, // [1095]
    SimpleHintData { raw_values_start: 1874, raw_values_count: 1, token_types_start: 1875, token_types_count: 1 }, // [1096]
    SimpleHintData { raw_values_start: 1876, raw_values_count: 1, token_types_start: 1877, token_types_count: 0 }, // [1097]
    SimpleHintData { raw_values_start: 1877, raw_values_count: 1, token_types_start: 1878, token_types_count: 0 }, // [1098]
    SimpleHintData { raw_values_start: 1878, raw_values_count: 1, token_types_start: 1879, token_types_count: 0 }, // [1099]
    SimpleHintData { raw_values_start: 1879, raw_values_count: 1, token_types_start: 1880, token_types_count: 0 }, // [1100]
    SimpleHintData { raw_values_start: 1880, raw_values_count: 1, token_types_start: 1881, token_types_count: 0 }, // [1101]
    SimpleHintData { raw_values_start: 1881, raw_values_count: 1, token_types_start: 1882, token_types_count: 0 }, // [1102]
    SimpleHintData { raw_values_start: 1882, raw_values_count: 3, token_types_start: 1885, token_types_count: 0 }, // [1103]
    SimpleHintData { raw_values_start: 1885, raw_values_count: 2, token_types_start: 1887, token_types_count: 0 }, // [1104]
    SimpleHintData { raw_values_start: 1887, raw_values_count: 1, token_types_start: 1888, token_types_count: 0 }, // [1105]
    SimpleHintData { raw_values_start: 1888, raw_values_count: 1, token_types_start: 1889, token_types_count: 0 }, // [1106]
    SimpleHintData { raw_values_start: 1889, raw_values_count: 1, token_types_start: 1890, token_types_count: 0 }, // [1107]
    SimpleHintData { raw_values_start: 1890, raw_values_count: 1, token_types_start: 1891, token_types_count: 0 }, // [1108]
    SimpleHintData { raw_values_start: 1891, raw_values_count: 1, token_types_start: 1892, token_types_count: 0 }, // [1109]
    SimpleHintData { raw_values_start: 1892, raw_values_count: 3, token_types_start: 1895, token_types_count: 0 }, // [1110]
    SimpleHintData { raw_values_start: 1895, raw_values_count: 1, token_types_start: 1896, token_types_count: 0 }, // [1111]
    SimpleHintData { raw_values_start: 1896, raw_values_count: 1, token_types_start: 1897, token_types_count: 0 }, // [1112]
    SimpleHintData { raw_values_start: 1897, raw_values_count: 1, token_types_start: 1898, token_types_count: 0 }, // [1113]
    SimpleHintData { raw_values_start: 1898, raw_values_count: 1, token_types_start: 1899, token_types_count: 0 }, // [1114]
    SimpleHintData { raw_values_start: 1899, raw_values_count: 1, token_types_start: 1900, token_types_count: 0 }, // [1115]
    SimpleHintData { raw_values_start: 1900, raw_values_count: 1, token_types_start: 1901, token_types_count: 0 }, // [1116]
    SimpleHintData { raw_values_start: 1901, raw_values_count: 1, token_types_start: 1902, token_types_count: 0 }, // [1117]
    SimpleHintData { raw_values_start: 1902, raw_values_count: 1, token_types_start: 1903, token_types_count: 0 }, // [1118]
    SimpleHintData { raw_values_start: 1903, raw_values_count: 1, token_types_start: 1904, token_types_count: 0 }, // [1119]
    SimpleHintData { raw_values_start: 1904, raw_values_count: 1, token_types_start: 1905, token_types_count: 0 }, // [1120]
    SimpleHintData { raw_values_start: 1905, raw_values_count: 1, token_types_start: 1906, token_types_count: 0 }, // [1121]
    SimpleHintData { raw_values_start: 1906, raw_values_count: 1, token_types_start: 1907, token_types_count: 0 }, // [1122]
    SimpleHintData { raw_values_start: 1907, raw_values_count: 3, token_types_start: 1910, token_types_count: 0 }, // [1123]
    SimpleHintData { raw_values_start: 1910, raw_values_count: 1, token_types_start: 1911, token_types_count: 0 }, // [1124]
    SimpleHintData { raw_values_start: 1911, raw_values_count: 1, token_types_start: 1912, token_types_count: 0 }, // [1125]
    SimpleHintData { raw_values_start: 1912, raw_values_count: 3, token_types_start: 1915, token_types_count: 0 }, // [1126]
    SimpleHintData { raw_values_start: 1915, raw_values_count: 1, token_types_start: 1916, token_types_count: 0 }, // [1127]
    SimpleHintData { raw_values_start: 1916, raw_values_count: 1, token_types_start: 1917, token_types_count: 0 }, // [1128]
    SimpleHintData { raw_values_start: 1917, raw_values_count: 1, token_types_start: 1918, token_types_count: 0 }, // [1129]
    SimpleHintData { raw_values_start: 1918, raw_values_count: 1, token_types_start: 1919, token_types_count: 0 }, // [1130]
    SimpleHintData { raw_values_start: 1919, raw_values_count: 1, token_types_start: 1920, token_types_count: 0 }, // [1131]
    SimpleHintData { raw_values_start: 1920, raw_values_count: 1, token_types_start: 1921, token_types_count: 0 }, // [1132]
    SimpleHintData { raw_values_start: 1921, raw_values_count: 1, token_types_start: 1922, token_types_count: 0 }, // [1133]
    SimpleHintData { raw_values_start: 1922, raw_values_count: 1, token_types_start: 1923, token_types_count: 0 }, // [1134]
    SimpleHintData { raw_values_start: 1923, raw_values_count: 1, token_types_start: 1924, token_types_count: 0 }, // [1135]
    SimpleHintData { raw_values_start: 1924, raw_values_count: 1, token_types_start: 1925, token_types_count: 0 }, // [1136]
    SimpleHintData { raw_values_start: 1925, raw_values_count: 1, token_types_start: 1926, token_types_count: 0 }, // [1137]
    SimpleHintData { raw_values_start: 1926, raw_values_count: 1, token_types_start: 1927, token_types_count: 0 }, // [1138]
    SimpleHintData { raw_values_start: 1927, raw_values_count: 1, token_types_start: 1928, token_types_count: 0 }, // [1139]
    SimpleHintData { raw_values_start: 1928, raw_values_count: 3, token_types_start: 1931, token_types_count: 0 }, // [1140]
    SimpleHintData { raw_values_start: 1931, raw_values_count: 1, token_types_start: 1932, token_types_count: 0 }, // [1141]
    SimpleHintData { raw_values_start: 1932, raw_values_count: 1, token_types_start: 1933, token_types_count: 0 }, // [1142]
    SimpleHintData { raw_values_start: 1933, raw_values_count: 1, token_types_start: 1934, token_types_count: 0 }, // [1143]
    SimpleHintData { raw_values_start: 1934, raw_values_count: 4, token_types_start: 1938, token_types_count: 0 }, // [1144]
    SimpleHintData { raw_values_start: 1938, raw_values_count: 3, token_types_start: 1941, token_types_count: 0 }, // [1145]
    SimpleHintData { raw_values_start: 1941, raw_values_count: 1, token_types_start: 1942, token_types_count: 0 }, // [1146]
    SimpleHintData { raw_values_start: 1942, raw_values_count: 1, token_types_start: 1943, token_types_count: 0 }, // [1147]
    SimpleHintData { raw_values_start: 1943, raw_values_count: 3, token_types_start: 1946, token_types_count: 0 }, // [1148]
    SimpleHintData { raw_values_start: 1946, raw_values_count: 1, token_types_start: 1947, token_types_count: 0 }, // [1149]
    SimpleHintData { raw_values_start: 1947, raw_values_count: 1, token_types_start: 1948, token_types_count: 0 }, // [1150]
    SimpleHintData { raw_values_start: 1948, raw_values_count: 1, token_types_start: 1949, token_types_count: 0 }, // [1151]
    SimpleHintData { raw_values_start: 1949, raw_values_count: 1, token_types_start: 1950, token_types_count: 0 }, // [1152]
    SimpleHintData { raw_values_start: 1950, raw_values_count: 1, token_types_start: 1951, token_types_count: 0 }, // [1153]
    SimpleHintData { raw_values_start: 1951, raw_values_count: 1, token_types_start: 1952, token_types_count: 0 }, // [1154]
    SimpleHintData { raw_values_start: 1952, raw_values_count: 1, token_types_start: 1953, token_types_count: 0 }, // [1155]
    SimpleHintData { raw_values_start: 1953, raw_values_count: 1, token_types_start: 1954, token_types_count: 0 }, // [1156]
    SimpleHintData { raw_values_start: 1954, raw_values_count: 1, token_types_start: 1955, token_types_count: 0 }, // [1157]
    SimpleHintData { raw_values_start: 1955, raw_values_count: 1, token_types_start: 1956, token_types_count: 0 }, // [1158]
    SimpleHintData { raw_values_start: 1956, raw_values_count: 1, token_types_start: 1957, token_types_count: 0 }, // [1159]
    SimpleHintData { raw_values_start: 1957, raw_values_count: 1, token_types_start: 1958, token_types_count: 0 }, // [1160]
    SimpleHintData { raw_values_start: 1958, raw_values_count: 1, token_types_start: 1959, token_types_count: 0 }, // [1161]
    SimpleHintData { raw_values_start: 1959, raw_values_count: 1, token_types_start: 1960, token_types_count: 0 }, // [1162]
    SimpleHintData { raw_values_start: 1960, raw_values_count: 1, token_types_start: 1961, token_types_count: 0 }, // [1163]
    SimpleHintData { raw_values_start: 1961, raw_values_count: 1, token_types_start: 1962, token_types_count: 0 }, // [1164]
    SimpleHintData { raw_values_start: 1962, raw_values_count: 1, token_types_start: 1963, token_types_count: 0 }, // [1165]
    SimpleHintData { raw_values_start: 1963, raw_values_count: 1, token_types_start: 1964, token_types_count: 0 }, // [1166]
    SimpleHintData { raw_values_start: 1964, raw_values_count: 1, token_types_start: 1965, token_types_count: 0 }, // [1167]
    SimpleHintData { raw_values_start: 1965, raw_values_count: 1, token_types_start: 1966, token_types_count: 0 }, // [1168]
    SimpleHintData { raw_values_start: 1966, raw_values_count: 1, token_types_start: 1967, token_types_count: 0 }, // [1169]
    SimpleHintData { raw_values_start: 1967, raw_values_count: 1, token_types_start: 1968, token_types_count: 0 }, // [1170]
    SimpleHintData { raw_values_start: 1968, raw_values_count: 1, token_types_start: 1969, token_types_count: 0 }, // [1171]
    SimpleHintData { raw_values_start: 1969, raw_values_count: 1, token_types_start: 1970, token_types_count: 0 }, // [1172]
    SimpleHintData { raw_values_start: 1970, raw_values_count: 1, token_types_start: 1971, token_types_count: 0 }, // [1173]
    SimpleHintData { raw_values_start: 1971, raw_values_count: 1, token_types_start: 1972, token_types_count: 0 }, // [1174]
    SimpleHintData { raw_values_start: 1972, raw_values_count: 1, token_types_start: 1973, token_types_count: 0 }, // [1175]
    SimpleHintData { raw_values_start: 1973, raw_values_count: 1, token_types_start: 1974, token_types_count: 0 }, // [1176]
    SimpleHintData { raw_values_start: 1974, raw_values_count: 2, token_types_start: 1976, token_types_count: 0 }, // [1177]
    SimpleHintData { raw_values_start: 1976, raw_values_count: 1, token_types_start: 1977, token_types_count: 0 }, // [1178]
    SimpleHintData { raw_values_start: 1977, raw_values_count: 1, token_types_start: 1978, token_types_count: 0 }, // [1179]
    SimpleHintData { raw_values_start: 1978, raw_values_count: 1, token_types_start: 1979, token_types_count: 0 }, // [1180]
    SimpleHintData { raw_values_start: 1979, raw_values_count: 1, token_types_start: 1980, token_types_count: 0 }, // [1181]
    SimpleHintData { raw_values_start: 1980, raw_values_count: 1, token_types_start: 1981, token_types_count: 0 }, // [1182]
    SimpleHintData { raw_values_start: 1981, raw_values_count: 1, token_types_start: 1982, token_types_count: 0 }, // [1183]
    SimpleHintData { raw_values_start: 1982, raw_values_count: 1, token_types_start: 1983, token_types_count: 0 }, // [1184]
    SimpleHintData { raw_values_start: 1983, raw_values_count: 1, token_types_start: 1984, token_types_count: 0 }, // [1185]
    SimpleHintData { raw_values_start: 1984, raw_values_count: 1, token_types_start: 1985, token_types_count: 0 }, // [1186]
    SimpleHintData { raw_values_start: 1985, raw_values_count: 1, token_types_start: 1986, token_types_count: 0 }, // [1187]
    SimpleHintData { raw_values_start: 1986, raw_values_count: 2, token_types_start: 1988, token_types_count: 0 }, // [1188]
    SimpleHintData { raw_values_start: 1988, raw_values_count: 1, token_types_start: 1989, token_types_count: 0 }, // [1189]
    SimpleHintData { raw_values_start: 1989, raw_values_count: 1, token_types_start: 1990, token_types_count: 0 }, // [1190]
    SimpleHintData { raw_values_start: 1990, raw_values_count: 1, token_types_start: 1991, token_types_count: 0 }, // [1191]
    SimpleHintData { raw_values_start: 1991, raw_values_count: 1, token_types_start: 1992, token_types_count: 0 }, // [1192]
    SimpleHintData { raw_values_start: 1992, raw_values_count: 1, token_types_start: 1993, token_types_count: 0 }, // [1193]
    SimpleHintData { raw_values_start: 1993, raw_values_count: 1, token_types_start: 1994, token_types_count: 0 }, // [1194]
    SimpleHintData { raw_values_start: 1994, raw_values_count: 1, token_types_start: 1995, token_types_count: 0 }, // [1195]
    SimpleHintData { raw_values_start: 1995, raw_values_count: 1, token_types_start: 1996, token_types_count: 0 }, // [1196]
    SimpleHintData { raw_values_start: 1996, raw_values_count: 1, token_types_start: 1997, token_types_count: 0 }, // [1197]
    SimpleHintData { raw_values_start: 1997, raw_values_count: 1, token_types_start: 1998, token_types_count: 0 }, // [1198]
    SimpleHintData { raw_values_start: 1998, raw_values_count: 1, token_types_start: 1999, token_types_count: 0 }, // [1199]
    SimpleHintData { raw_values_start: 1999, raw_values_count: 12, token_types_start: 2011, token_types_count: 0 }, // [1200]
    SimpleHintData { raw_values_start: 2011, raw_values_count: 2, token_types_start: 2013, token_types_count: 0 }, // [1201]
    SimpleHintData { raw_values_start: 2013, raw_values_count: 1, token_types_start: 2014, token_types_count: 0 }, // [1202]
    SimpleHintData { raw_values_start: 2014, raw_values_count: 1, token_types_start: 2015, token_types_count: 0 }, // [1203]
    SimpleHintData { raw_values_start: 2015, raw_values_count: 2, token_types_start: 2017, token_types_count: 0 }, // [1204]
    SimpleHintData { raw_values_start: 2017, raw_values_count: 1, token_types_start: 2018, token_types_count: 0 }, // [1205]
    SimpleHintData { raw_values_start: 2018, raw_values_count: 1, token_types_start: 2019, token_types_count: 0 }, // [1206]
    SimpleHintData { raw_values_start: 2019, raw_values_count: 1, token_types_start: 2020, token_types_count: 0 }, // [1207]
    SimpleHintData { raw_values_start: 2020, raw_values_count: 2, token_types_start: 2022, token_types_count: 0 }, // [1208]
    SimpleHintData { raw_values_start: 2022, raw_values_count: 1, token_types_start: 2023, token_types_count: 0 }, // [1209]
    SimpleHintData { raw_values_start: 2023, raw_values_count: 1, token_types_start: 2024, token_types_count: 0 }, // [1210]
    SimpleHintData { raw_values_start: 2024, raw_values_count: 1, token_types_start: 2025, token_types_count: 0 }, // [1211]
    SimpleHintData { raw_values_start: 2025, raw_values_count: 1, token_types_start: 2026, token_types_count: 0 }, // [1212]
    SimpleHintData { raw_values_start: 2026, raw_values_count: 1, token_types_start: 2027, token_types_count: 0 }, // [1213]
    SimpleHintData { raw_values_start: 2027, raw_values_count: 1, token_types_start: 2028, token_types_count: 0 }, // [1214]
    SimpleHintData { raw_values_start: 2028, raw_values_count: 1, token_types_start: 2029, token_types_count: 0 }, // [1215]
    SimpleHintData { raw_values_start: 2029, raw_values_count: 1, token_types_start: 2030, token_types_count: 0 }, // [1216]
    SimpleHintData { raw_values_start: 2030, raw_values_count: 1, token_types_start: 2031, token_types_count: 0 }, // [1217]
    SimpleHintData { raw_values_start: 2031, raw_values_count: 1, token_types_start: 2032, token_types_count: 0 }, // [1218]
    SimpleHintData { raw_values_start: 2032, raw_values_count: 1, token_types_start: 2033, token_types_count: 0 }, // [1219]
    SimpleHintData { raw_values_start: 2033, raw_values_count: 1, token_types_start: 2034, token_types_count: 0 }, // [1220]
    SimpleHintData { raw_values_start: 2034, raw_values_count: 1, token_types_start: 2035, token_types_count: 0 }, // [1221]
    SimpleHintData { raw_values_start: 2035, raw_values_count: 1, token_types_start: 2036, token_types_count: 0 }, // [1222]
    SimpleHintData { raw_values_start: 2036, raw_values_count: 1, token_types_start: 2037, token_types_count: 0 }, // [1223]
    SimpleHintData { raw_values_start: 2037, raw_values_count: 1, token_types_start: 2038, token_types_count: 0 }, // [1224]
    SimpleHintData { raw_values_start: 2038, raw_values_count: 1, token_types_start: 2039, token_types_count: 0 }, // [1225]
    SimpleHintData { raw_values_start: 2039, raw_values_count: 1, token_types_start: 2040, token_types_count: 0 }, // [1226]
    SimpleHintData { raw_values_start: 2040, raw_values_count: 1, token_types_start: 2041, token_types_count: 0 }, // [1227]
    SimpleHintData { raw_values_start: 2041, raw_values_count: 1, token_types_start: 2042, token_types_count: 0 }, // [1228]
    SimpleHintData { raw_values_start: 2042, raw_values_count: 1, token_types_start: 2043, token_types_count: 0 }, // [1229]
    SimpleHintData { raw_values_start: 2043, raw_values_count: 1, token_types_start: 2044, token_types_count: 0 }, // [1230]
    SimpleHintData { raw_values_start: 2044, raw_values_count: 1, token_types_start: 2045, token_types_count: 0 }, // [1231]
    SimpleHintData { raw_values_start: 2045, raw_values_count: 1, token_types_start: 2046, token_types_count: 0 }, // [1232]
    SimpleHintData { raw_values_start: 2046, raw_values_count: 1, token_types_start: 2047, token_types_count: 0 }, // [1233]
    SimpleHintData { raw_values_start: 2047, raw_values_count: 1, token_types_start: 2048, token_types_count: 0 }, // [1234]
    SimpleHintData { raw_values_start: 2048, raw_values_count: 1, token_types_start: 2049, token_types_count: 0 }, // [1235]
    SimpleHintData { raw_values_start: 2049, raw_values_count: 1, token_types_start: 2050, token_types_count: 0 }, // [1236]
    SimpleHintData { raw_values_start: 2050, raw_values_count: 1, token_types_start: 2051, token_types_count: 0 }, // [1237]
    SimpleHintData { raw_values_start: 2051, raw_values_count: 1, token_types_start: 2052, token_types_count: 0 }, // [1238]
    SimpleHintData { raw_values_start: 2052, raw_values_count: 2, token_types_start: 2054, token_types_count: 0 }, // [1239]
    SimpleHintData { raw_values_start: 2054, raw_values_count: 1, token_types_start: 2055, token_types_count: 0 }, // [1240]
    SimpleHintData { raw_values_start: 2055, raw_values_count: 1, token_types_start: 2056, token_types_count: 0 }, // [1241]
    SimpleHintData { raw_values_start: 2056, raw_values_count: 1, token_types_start: 2057, token_types_count: 0 }, // [1242]
    SimpleHintData { raw_values_start: 2057, raw_values_count: 1, token_types_start: 2058, token_types_count: 0 }, // [1243]
    SimpleHintData { raw_values_start: 2058, raw_values_count: 1, token_types_start: 2059, token_types_count: 0 }, // [1244]
    SimpleHintData { raw_values_start: 2059, raw_values_count: 1, token_types_start: 2060, token_types_count: 0 }, // [1245]
    SimpleHintData { raw_values_start: 2060, raw_values_count: 1, token_types_start: 2061, token_types_count: 0 }, // [1246]
    SimpleHintData { raw_values_start: 2061, raw_values_count: 1, token_types_start: 2062, token_types_count: 0 }, // [1247]
    SimpleHintData { raw_values_start: 2062, raw_values_count: 1, token_types_start: 2063, token_types_count: 0 }, // [1248]
    SimpleHintData { raw_values_start: 2063, raw_values_count: 1, token_types_start: 2064, token_types_count: 0 }, // [1249]
    SimpleHintData { raw_values_start: 2064, raw_values_count: 1, token_types_start: 2065, token_types_count: 0 }, // [1250]
    SimpleHintData { raw_values_start: 2065, raw_values_count: 1, token_types_start: 2066, token_types_count: 0 }, // [1251]
    SimpleHintData { raw_values_start: 2066, raw_values_count: 1, token_types_start: 2067, token_types_count: 0 }, // [1252]
    SimpleHintData { raw_values_start: 2067, raw_values_count: 1, token_types_start: 2068, token_types_count: 0 }, // [1253]
    SimpleHintData { raw_values_start: 2068, raw_values_count: 12, token_types_start: 2080, token_types_count: 0 }, // [1254]
    SimpleHintData { raw_values_start: 2080, raw_values_count: 3, token_types_start: 2083, token_types_count: 0 }, // [1255]
    SimpleHintData { raw_values_start: 2083, raw_values_count: 3, token_types_start: 2086, token_types_count: 0 }, // [1256]
    SimpleHintData { raw_values_start: 2086, raw_values_count: 1, token_types_start: 2087, token_types_count: 1 }, // [1257]
    SimpleHintData { raw_values_start: 2088, raw_values_count: 1, token_types_start: 2089, token_types_count: 0 }, // [1258]
    SimpleHintData { raw_values_start: 2089, raw_values_count: 1, token_types_start: 2090, token_types_count: 0 }, // [1259]
    SimpleHintData { raw_values_start: 2090, raw_values_count: 1, token_types_start: 2091, token_types_count: 0 }, // [1260]
    SimpleHintData { raw_values_start: 2091, raw_values_count: 1, token_types_start: 2092, token_types_count: 0 }, // [1261]
    SimpleHintData { raw_values_start: 2092, raw_values_count: 1, token_types_start: 2093, token_types_count: 0 }, // [1262]
    SimpleHintData { raw_values_start: 2093, raw_values_count: 1, token_types_start: 2094, token_types_count: 0 }, // [1263]
    SimpleHintData { raw_values_start: 2094, raw_values_count: 1, token_types_start: 2095, token_types_count: 0 }, // [1264]
    SimpleHintData { raw_values_start: 2095, raw_values_count: 0, token_types_start: 2095, token_types_count: 1 }, // [1265]
    SimpleHintData { raw_values_start: 2096, raw_values_count: 1, token_types_start: 2097, token_types_count: 0 }, // [1266]
    SimpleHintData { raw_values_start: 2097, raw_values_count: 1, token_types_start: 2098, token_types_count: 0 }, // [1267]
    SimpleHintData { raw_values_start: 2098, raw_values_count: 1, token_types_start: 2099, token_types_count: 0 }, // [1268]
    SimpleHintData { raw_values_start: 2099, raw_values_count: 1, token_types_start: 2100, token_types_count: 0 }, // [1269]
    SimpleHintData { raw_values_start: 2100, raw_values_count: 1, token_types_start: 2101, token_types_count: 0 }, // [1270]
    SimpleHintData { raw_values_start: 2101, raw_values_count: 1, token_types_start: 2102, token_types_count: 0 }, // [1271]
    SimpleHintData { raw_values_start: 2102, raw_values_count: 1, token_types_start: 2103, token_types_count: 0 }, // [1272]
    SimpleHintData { raw_values_start: 2103, raw_values_count: 1, token_types_start: 2104, token_types_count: 0 }, // [1273]
    SimpleHintData { raw_values_start: 2104, raw_values_count: 4, token_types_start: 2108, token_types_count: 0 }, // [1274]
    SimpleHintData { raw_values_start: 2108, raw_values_count: 2, token_types_start: 2110, token_types_count: 0 }, // [1275]
    SimpleHintData { raw_values_start: 2110, raw_values_count: 1, token_types_start: 2111, token_types_count: 0 }, // [1276]
    SimpleHintData { raw_values_start: 2111, raw_values_count: 1, token_types_start: 2112, token_types_count: 0 }, // [1277]
    SimpleHintData { raw_values_start: 2112, raw_values_count: 1, token_types_start: 2113, token_types_count: 0 }, // [1278]
    SimpleHintData { raw_values_start: 2113, raw_values_count: 3, token_types_start: 2116, token_types_count: 0 }, // [1279]
    SimpleHintData { raw_values_start: 2116, raw_values_count: 1, token_types_start: 2117, token_types_count: 0 }, // [1280]
    SimpleHintData { raw_values_start: 2117, raw_values_count: 1, token_types_start: 2118, token_types_count: 0 }, // [1281]
    SimpleHintData { raw_values_start: 2118, raw_values_count: 1, token_types_start: 2119, token_types_count: 0 }, // [1282]
    SimpleHintData { raw_values_start: 2119, raw_values_count: 1, token_types_start: 2120, token_types_count: 0 }, // [1283]
    SimpleHintData { raw_values_start: 2120, raw_values_count: 1, token_types_start: 2121, token_types_count: 0 }, // [1284]
    SimpleHintData { raw_values_start: 2121, raw_values_count: 1, token_types_start: 2122, token_types_count: 0 }, // [1285]
    SimpleHintData { raw_values_start: 2122, raw_values_count: 1, token_types_start: 2123, token_types_count: 0 }, // [1286]
    SimpleHintData { raw_values_start: 2123, raw_values_count: 1, token_types_start: 2124, token_types_count: 0 }, // [1287]
    SimpleHintData { raw_values_start: 2124, raw_values_count: 1, token_types_start: 2125, token_types_count: 0 }, // [1288]
    SimpleHintData { raw_values_start: 2125, raw_values_count: 2, token_types_start: 2127, token_types_count: 0 }, // [1289]
    SimpleHintData { raw_values_start: 2127, raw_values_count: 1, token_types_start: 2128, token_types_count: 0 }, // [1290]
    SimpleHintData { raw_values_start: 2128, raw_values_count: 1, token_types_start: 2129, token_types_count: 0 }, // [1291]
    SimpleHintData { raw_values_start: 2129, raw_values_count: 1, token_types_start: 2130, token_types_count: 0 }, // [1292]
    SimpleHintData { raw_values_start: 2130, raw_values_count: 1, token_types_start: 2131, token_types_count: 0 }, // [1293]
    SimpleHintData { raw_values_start: 2131, raw_values_count: 1, token_types_start: 2132, token_types_count: 0 }, // [1294]
    SimpleHintData { raw_values_start: 2132, raw_values_count: 1, token_types_start: 2133, token_types_count: 0 }, // [1295]
    SimpleHintData { raw_values_start: 2133, raw_values_count: 1, token_types_start: 2134, token_types_count: 0 }, // [1296]
    SimpleHintData { raw_values_start: 2134, raw_values_count: 1, token_types_start: 2135, token_types_count: 0 }, // [1297]
    SimpleHintData { raw_values_start: 2135, raw_values_count: 2, token_types_start: 2137, token_types_count: 0 }, // [1298]
    SimpleHintData { raw_values_start: 2137, raw_values_count: 1, token_types_start: 2138, token_types_count: 0 }, // [1299]
    SimpleHintData { raw_values_start: 2138, raw_values_count: 1, token_types_start: 2139, token_types_count: 0 }, // [1300]
    SimpleHintData { raw_values_start: 2139, raw_values_count: 1, token_types_start: 2140, token_types_count: 0 }, // [1301]
    SimpleHintData { raw_values_start: 2140, raw_values_count: 1, token_types_start: 2141, token_types_count: 0 }, // [1302]
    SimpleHintData { raw_values_start: 2141, raw_values_count: 1, token_types_start: 2142, token_types_count: 0 }, // [1303]
    SimpleHintData { raw_values_start: 2142, raw_values_count: 1, token_types_start: 2143, token_types_count: 0 }, // [1304]
    SimpleHintData { raw_values_start: 2143, raw_values_count: 1, token_types_start: 2144, token_types_count: 0 }, // [1305]
    SimpleHintData { raw_values_start: 2144, raw_values_count: 1, token_types_start: 2145, token_types_count: 0 }, // [1306]
    SimpleHintData { raw_values_start: 2145, raw_values_count: 3, token_types_start: 2148, token_types_count: 0 }, // [1307]
    SimpleHintData { raw_values_start: 2148, raw_values_count: 1, token_types_start: 2149, token_types_count: 0 }, // [1308]
    SimpleHintData { raw_values_start: 2149, raw_values_count: 1, token_types_start: 2150, token_types_count: 0 }, // [1309]
    SimpleHintData { raw_values_start: 2150, raw_values_count: 1, token_types_start: 2151, token_types_count: 0 }, // [1310]
    SimpleHintData { raw_values_start: 2151, raw_values_count: 1, token_types_start: 2152, token_types_count: 0 }, // [1311]
    SimpleHintData { raw_values_start: 2152, raw_values_count: 1, token_types_start: 2153, token_types_count: 0 }, // [1312]
    SimpleHintData { raw_values_start: 2153, raw_values_count: 1, token_types_start: 2154, token_types_count: 0 }, // [1313]
    SimpleHintData { raw_values_start: 2154, raw_values_count: 1, token_types_start: 2155, token_types_count: 0 }, // [1314]
    SimpleHintData { raw_values_start: 2155, raw_values_count: 1, token_types_start: 2156, token_types_count: 0 }, // [1315]
    SimpleHintData { raw_values_start: 2156, raw_values_count: 1, token_types_start: 2157, token_types_count: 0 }, // [1316]
    SimpleHintData { raw_values_start: 2157, raw_values_count: 1, token_types_start: 2158, token_types_count: 0 }, // [1317]
    SimpleHintData { raw_values_start: 2158, raw_values_count: 1, token_types_start: 2159, token_types_count: 0 }, // [1318]
    SimpleHintData { raw_values_start: 2159, raw_values_count: 1, token_types_start: 2160, token_types_count: 0 }, // [1319]
    SimpleHintData { raw_values_start: 2160, raw_values_count: 1, token_types_start: 2161, token_types_count: 0 }, // [1320]
    SimpleHintData { raw_values_start: 2161, raw_values_count: 1, token_types_start: 2162, token_types_count: 0 }, // [1321]
    SimpleHintData { raw_values_start: 2162, raw_values_count: 1, token_types_start: 2163, token_types_count: 0 }, // [1322]
    SimpleHintData { raw_values_start: 2163, raw_values_count: 1, token_types_start: 2164, token_types_count: 0 }, // [1323]
    SimpleHintData { raw_values_start: 2164, raw_values_count: 1, token_types_start: 2165, token_types_count: 1 }, // [1324]
    SimpleHintData { raw_values_start: 2166, raw_values_count: 1, token_types_start: 2167, token_types_count: 1 }, // [1325]
    SimpleHintData { raw_values_start: 2168, raw_values_count: 2, token_types_start: 2170, token_types_count: 0 }, // [1326]
    SimpleHintData { raw_values_start: 2170, raw_values_count: 1, token_types_start: 2171, token_types_count: 0 }, // [1327]
    SimpleHintData { raw_values_start: 2171, raw_values_count: 1, token_types_start: 2172, token_types_count: 0 }, // [1328]
    SimpleHintData { raw_values_start: 2172, raw_values_count: 1, token_types_start: 2173, token_types_count: 0 }, // [1329]
    SimpleHintData { raw_values_start: 2173, raw_values_count: 1, token_types_start: 2174, token_types_count: 0 }, // [1330]
    SimpleHintData { raw_values_start: 2174, raw_values_count: 1, token_types_start: 2175, token_types_count: 0 }, // [1331]
    SimpleHintData { raw_values_start: 2175, raw_values_count: 1, token_types_start: 2176, token_types_count: 0 }, // [1332]
    SimpleHintData { raw_values_start: 2176, raw_values_count: 1, token_types_start: 2177, token_types_count: 0 }, // [1333]
    SimpleHintData { raw_values_start: 2177, raw_values_count: 1, token_types_start: 2178, token_types_count: 0 }, // [1334]
    SimpleHintData { raw_values_start: 2178, raw_values_count: 1, token_types_start: 2179, token_types_count: 0 }, // [1335]
    SimpleHintData { raw_values_start: 2179, raw_values_count: 1, token_types_start: 2180, token_types_count: 0 }, // [1336]
    SimpleHintData { raw_values_start: 2180, raw_values_count: 1, token_types_start: 2181, token_types_count: 0 }, // [1337]
    SimpleHintData { raw_values_start: 2181, raw_values_count: 1, token_types_start: 2182, token_types_count: 0 }, // [1338]
    SimpleHintData { raw_values_start: 2182, raw_values_count: 1, token_types_start: 2183, token_types_count: 0 }, // [1339]
    SimpleHintData { raw_values_start: 2183, raw_values_count: 1, token_types_start: 2184, token_types_count: 0 }, // [1340]
    SimpleHintData { raw_values_start: 2184, raw_values_count: 1, token_types_start: 2185, token_types_count: 0 }, // [1341]
    SimpleHintData { raw_values_start: 2185, raw_values_count: 1, token_types_start: 2186, token_types_count: 0 }, // [1342]
    SimpleHintData { raw_values_start: 2186, raw_values_count: 1, token_types_start: 2187, token_types_count: 0 }, // [1343]
    SimpleHintData { raw_values_start: 2187, raw_values_count: 1, token_types_start: 2188, token_types_count: 0 }, // [1344]
    SimpleHintData { raw_values_start: 2188, raw_values_count: 1, token_types_start: 2189, token_types_count: 0 }, // [1345]
    SimpleHintData { raw_values_start: 2189, raw_values_count: 1, token_types_start: 2190, token_types_count: 0 }, // [1346]
    SimpleHintData { raw_values_start: 2190, raw_values_count: 1, token_types_start: 2191, token_types_count: 0 }, // [1347]
    SimpleHintData { raw_values_start: 2191, raw_values_count: 1, token_types_start: 2192, token_types_count: 0 }, // [1348]
    SimpleHintData { raw_values_start: 2192, raw_values_count: 3, token_types_start: 2195, token_types_count: 0 }, // [1349]
    SimpleHintData { raw_values_start: 2195, raw_values_count: 1, token_types_start: 2196, token_types_count: 0 }, // [1350]
    SimpleHintData { raw_values_start: 2196, raw_values_count: 2, token_types_start: 2198, token_types_count: 0 }, // [1351]
    SimpleHintData { raw_values_start: 2198, raw_values_count: 1, token_types_start: 2199, token_types_count: 0 }, // [1352]
    SimpleHintData { raw_values_start: 2199, raw_values_count: 1, token_types_start: 2200, token_types_count: 0 }, // [1353]
    SimpleHintData { raw_values_start: 2200, raw_values_count: 1, token_types_start: 2201, token_types_count: 0 }, // [1354]
    SimpleHintData { raw_values_start: 2201, raw_values_count: 1, token_types_start: 2202, token_types_count: 0 }, // [1355]
    SimpleHintData { raw_values_start: 2202, raw_values_count: 1, token_types_start: 2203, token_types_count: 0 }, // [1356]
    SimpleHintData { raw_values_start: 2203, raw_values_count: 1, token_types_start: 2204, token_types_count: 0 }, // [1357]
    SimpleHintData { raw_values_start: 2204, raw_values_count: 1, token_types_start: 2205, token_types_count: 0 }, // [1358]
    SimpleHintData { raw_values_start: 2205, raw_values_count: 1, token_types_start: 2206, token_types_count: 0 }, // [1359]
    SimpleHintData { raw_values_start: 2206, raw_values_count: 1, token_types_start: 2207, token_types_count: 0 }, // [1360]
    SimpleHintData { raw_values_start: 2207, raw_values_count: 8, token_types_start: 2215, token_types_count: 0 }, // [1361]
    SimpleHintData { raw_values_start: 2215, raw_values_count: 1, token_types_start: 2216, token_types_count: 0 }, // [1362]
    SimpleHintData { raw_values_start: 2216, raw_values_count: 2, token_types_start: 2218, token_types_count: 0 }, // [1363]
    SimpleHintData { raw_values_start: 2218, raw_values_count: 1, token_types_start: 2219, token_types_count: 0 }, // [1364]
    SimpleHintData { raw_values_start: 2219, raw_values_count: 1, token_types_start: 2220, token_types_count: 0 }, // [1365]
    SimpleHintData { raw_values_start: 2220, raw_values_count: 1, token_types_start: 2221, token_types_count: 0 }, // [1366]
    SimpleHintData { raw_values_start: 2221, raw_values_count: 1, token_types_start: 2222, token_types_count: 0 }, // [1367]
    SimpleHintData { raw_values_start: 2222, raw_values_count: 1, token_types_start: 2223, token_types_count: 0 }, // [1368]
    SimpleHintData { raw_values_start: 2223, raw_values_count: 1, token_types_start: 2224, token_types_count: 0 }, // [1369]
    SimpleHintData { raw_values_start: 2224, raw_values_count: 1, token_types_start: 2225, token_types_count: 0 }, // [1370]
];

pub static SIMPLE_HINT_INDICES: &[u32] = &[
    1, 2, 3, 4, 5, 6, 5, 5, 7, 8, 8, 9, 10, 10, 11, 12,
    13, 14, 14, 15, 16, 17, 18, 18, 0, 11, 0, 0, 19, 20, 20, 21,
    22, 23, 24, 24, 25, 26, 27, 11, 28, 29, 30, 31, 32, 32, 0, 33,
    7, 34, 35, 36, 37, 38, 39, 40, 41, 41, 0, 42, 43, 43, 44, 45,
    45, 9, 46, 46, 0, 47, 47, 48, 49, 49, 50, 51, 52, 43, 43, 44,
    53, 53, 20, 50, 54, 55, 44, 56, 57, 57, 0, 58, 58, 7, 59, 60,
    54, 40, 40, 0, 61, 62, 62, 63, 64, 64, 65, 66, 7, 67, 68, 15,
    69, 69, 0, 0, 0, 0, 49, 44, 70, 70, 44, 0, 0, 71, 72, 73,
    74, 75, 76, 77, 23, 78, 79, 9, 80, 81, 82, 83, 84, 85, 33, 86,
    87, 87, 79, 88, 88, 0, 89, 14, 90, 91, 92, 93, 94, 94, 95, 96,
    97, 98, 98, 99, 81, 100, 101, 35, 102, 103, 104, 105, 106, 107, 108, 109,
    109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 40, 120, 121, 122, 71,
    123, 75, 75, 0, 124, 23, 125, 126, 127, 128, 129, 0, 0, 80, 80, 130,
    131, 132, 6, 0, 80, 80, 130, 131, 132, 6, 133, 0, 0, 134, 133, 135,
    136, 75, 9, 137, 138, 55, 139, 68, 140, 141, 142, 143, 143, 7, 45, 45,
    9, 38, 38, 75, 144, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154,
    155, 156, 143, 143, 0, 45, 45, 148, 38, 38, 147, 144, 144, 0, 157, 101,
    158, 0, 159, 159, 138, 55, 160, 0, 0, 161, 0, 0, 0, 0, 0, 161,
    0, 0, 0, 0, 132, 162, 162, 163, 133, 133, 0, 164, 138, 55, 160, 23,
    78, 0, 162, 165, 165, 166, 167, 168, 169, 170, 171, 172, 81, 173, 174, 82,
    175, 176, 177, 0, 0, 178, 0, 0, 0, 133, 0, 179, 0, 180, 0, 0,
    0, 83, 181, 182, 84, 84, 183, 132, 20, 22, 184, 0, 185, 14, 14, 186,
    187, 187, 0, 188, 163, 0, 189, 190, 190, 0, 189, 84, 84, 191, 138, 55,
    184, 0, 192, 193, 194, 195, 84, 84, 35, 0, 196, 197, 197, 198, 199, 200,
    201, 201, 202, 203, 203, 202, 204, 204, 133, 179, 205, 205, 187, 0, 193, 206,
    207, 94, 208, 209, 210, 211, 84, 84, 84, 160, 184, 0, 192, 193, 194, 195,
    212, 213, 213, 163, 214, 215, 216, 217, 218, 218, 214, 219, 220, 221, 222, 223,
    84, 84, 68, 0, 212, 212, 84, 84, 15, 224, 174, 174, 225, 110, 62, 226,
    227, 227, 20, 22, 0, 228, 228, 94, 208, 110, 229, 55, 228, 228, 94, 208,
    230, 94, 94, 231, 232, 233, 234, 115, 231, 231, 214, 208, 208, 231, 232, 233,
    234, 115, 100, 100, 231, 232, 233, 234, 115, 231, 231, 214, 94, 94, 121, 235,
    235, 214, 236, 236, 179, 208, 208, 121, 228, 228, 94, 208, 237, 234, 238, 238,
    190, 239, 71, 71, 0, 240, 240, 241, 242, 243, 117, 117, 244, 245, 245, 246,
    247, 241, 248, 249, 250, 251, 252, 252, 241, 253, 254, 255, 179, 64, 64, 256,
    257, 241, 258, 169, 83, 16, 16, 241, 0, 259, 259, 241, 0, 0, 0, 241,
    0, 251, 0, 84, 84, 132, 72, 184, 0, 192, 194, 195, 193, 260, 261, 261,
    171, 262, 0, 0, 133, 0, 0, 93, 93, 263, 264, 264, 0, 133, 0, 0,
    205, 205, 264, 0, 187, 0, 265, 171, 171, 266, 267, 133, 267, 267, 93, 93,
    268, 239, 239, 41, 269, 269, 133, 0, 246, 246, 0, 270, 271, 271, 104, 93,
    35, 272, 205, 205, 246, 0, 187, 0, 93, 93, 264, 184, 0, 0, 0, 0,
    241, 0, 0, 0, 261, 261, 171, 262, 264, 0, 273, 274, 274, 275, 175, 0,
    133, 93, 205, 205, 276, 178, 187, 0, 205, 205, 205, 187, 0, 84, 84, 75,
    0, 0, 0, 0, 0, 0, 241, 0, 0, 0, 205, 260, 265, 84, 84, 23,
    184, 0, 277, 193, 94, 208, 205, 205, 187, 0, 194, 195, 278, 279, 280, 8,
    85, 281, 281, 131, 282, 256, 99, 283, 284, 285, 286, 287, 288, 289, 279, 290,
    291, 292, 293, 294, 295, 165, 0, 0, 214, 0, 296, 165, 297, 0, 298, 165,
    0, 0, 0, 178, 178, 178, 299, 300, 301, 302, 303, 0, 0, 0, 133, 0,
    0, 179, 214, 161, 304, 296, 304, 305, 303, 251, 0, 306, 307, 308, 309, 289,
    10, 310, 33, 311, 312, 313, 314, 314, 315, 314, 316, 317, 86, 318, 319, 0,
    0, 319, 320, 0, 0, 0, 0, 0, 0, 321, 322, 134, 178, 0, 0, 323,
    287, 0, 0, 0, 0, 71, 0, 324, 324, 287, 287, 87, 325, 0, 0, 326,
    0, 327, 328, 0, 326, 329, 330, 331, 332, 333, 284, 290, 334, 335, 336, 0,
    296, 304, 304, 133, 296, 0, 161, 161, 0, 296, 337, 338, 339, 340, 341, 279,
    292, 292, 292, 290, 293, 293, 293, 291, 342, 343, 344, 334, 281, 345, 346, 347,
    348, 349, 350, 133, 0, 0, 133, 0, 0, 0, 133, 351, 352, 353, 354, 355,
    356, 356, 357, 358, 0, 0, 0, 0, 304, 356, 359, 360, 163, 361, 133, 0,
    0, 0, 133, 178, 133, 362, 218, 363, 364, 365, 366, 270, 367, 368, 368, 368,
    369, 369, 369, 370, 328, 370, 328, 328, 368, 368, 369, 369, 369, 370, 328, 370,
    328, 328, 368, 0, 368, 368, 0, 371, 0, 369, 369, 369, 370, 328, 370, 328,
    328, 368, 0, 335, 134, 333, 368, 372, 373, 374, 375, 376, 377, 378, 379, 88,
    0, 380, 381, 382, 383, 384, 385, 386, 387, 388, 389, 390, 391, 392, 12, 393,
    394, 227, 227, 227, 0, 0, 304, 13, 395, 396, 397, 0, 0, 398, 399, 0,
    179, 0, 0, 161, 400, 401, 402, 403, 357, 78, 78, 0, 404, 75, 75, 405,
    0, 0, 0, 0, 405, 405, 0, 406, 406, 407, 35, 35, 163, 0, 296, 296,
    296, 0, 0, 0, 0, 0, 319, 0, 0, 246, 246, 0, 0, 408, 406, 407,
    389, 389, 133, 0, 251, 251, 0, 239, 239, 0, 269, 314, 119, 119, 0, 89,
    398, 398, 0, 0, 0, 0, 0, 0, 0, 0, 228, 94, 208, 0, 0, 133,
    0, 0, 409, 361, 380, 0, 0, 0, 228, 94, 208, 410, 0, 264, 0, 0,
    161, 400, 0, 0, 289, 78, 411, 412, 0, 0, 0, 0, 413, 134, 414, 415,
    89, 89, 179, 89, 0, 89, 89, 109, 416, 416, 75, 75, 0, 264, 264, 0,
    51, 51, 0, 36, 36, 0, 45, 45, 9, 0, 404, 404, 417, 179, 407, 110,
    244, 335, 241, 293, 292, 418, 419, 419, 420, 404, 0, 193, 193, 91, 421, 160,
    422, 343, 0, 0, 0, 241, 0, 423, 423, 0, 193, 424, 0, 0, 425, 425,
    425, 23, 0, 426, 426, 427, 428, 290, 290, 290, 429, 430, 0, 0, 431, 0,
    0, 14, 14, 163, 222, 0, 321, 321, 0, 14, 321, 432, 433, 434, 246, 435,
    436, 437, 247, 438, 16, 439, 440, 17, 441, 441, 441, 0, 369, 369, 0, 442,
    443, 444, 445, 446, 447, 448, 449, 90, 90, 372, 133, 0, 178, 0, 190, 450,
    451, 139, 138, 55, 452, 453, 454, 455, 438, 456, 0, 133, 457, 457, 0, 178,
    178, 302, 90, 90, 183, 132, 20, 22, 184, 0, 185, 185, 14, 14, 186, 187,
    187, 0, 458, 0, 0, 188, 163, 0, 189, 190, 190, 0, 189, 459, 459, 179,
    90, 90, 20, 184, 0, 460, 90, 345, 345, 461, 462, 194, 195, 138, 184, 0,
    133, 463, 0, 0, 0, 0, 464, 465, 251, 312, 162, 466, 467, 468, 469, 176,
    176, 459, 0, 470, 471, 471, 133, 78, 78, 472, 0, 0, 75, 473, 404, 178,
    0, 474, 474, 475, 236, 179, 0, 324, 90, 90, 476, 269, 340, 35, 0, 109,
    0, 133, 0, 0, 477, 90, 90, 90, 345, 48, 184, 0, 478, 478, 133, 0,
    0, 0, 241, 0, 0, 165, 179, 179, 178, 362, 90, 90, 345, 345, 461, 462,
    194, 195, 160, 343, 184, 0, 464, 479, 251, 312, 162, 473, 404, 178, 0, 326,
    326, 0, 327, 328, 0, 90, 90, 345, 345, 461, 462, 194, 195, 160, 184, 0,
    464, 479, 251, 312, 162, 473, 404, 178, 0, 328, 0, 480, 90, 345, 345, 461,
    462, 194, 195, 55, 184, 0, 133, 464, 479, 251, 312, 162, 473, 404, 178, 0,
    0, 0, 324, 90, 90, 61, 184, 0, 90, 90, 142, 184, 0, 481, 213, 213,
    163, 214, 482, 482, 190, 214, 215, 216, 217, 218, 218, 214, 219, 220, 221, 222,
    223, 90, 90, 68, 0, 481, 481, 90, 90, 345, 462, 194, 195, 132, 72, 184,
    0, 464, 457, 0, 308, 308, 0, 90, 90, 345, 483, 75, 184, 0, 484, 133,
    133, 0, 0, 267, 0, 89, 109, 485, 486, 109, 485, 178, 362, 133, 362, 487,
    487, 0, 0, 90, 90, 345, 345, 461, 462, 194, 195, 23, 184, 0, 464, 251,
    488, 488, 489, 325, 175, 490, 490, 405, 457, 491, 492, 457, 457, 493, 64, 494,
    495, 496, 199, 497, 498, 498, 499, 500, 0, 0, 228, 94, 208, 369, 369, 133,
    0, 0, 425, 0, 328, 0, 501, 90, 345, 345, 461, 462, 194, 195, 78, 343,
    184, 0, 464, 473, 404, 178, 502, 328, 501, 90, 345, 345, 461, 462, 194, 195,
    78, 184, 0, 100, 503, 503, 179, 0, 179, 464, 251, 504, 312, 162, 473, 404,
    178, 505, 506, 507, 90, 90, 79, 184, 0, 508, 188, 188, 509, 163, 163, 0,
    510, 511, 512, 228, 94, 208, 513, 513, 514, 515, 178, 178, 516, 517, 179, 518,
    518, 0, 519, 251, 520, 520, 251, 251, 76, 521, 254, 522, 76, 523, 523, 524,
    214, 126, 109, 0, 44, 44, 0, 251, 0, 525, 525, 526, 166, 166, 527, 105,
    528, 94, 94, 529, 16, 16, 241, 530, 531, 83, 117, 117, 532, 533, 129, 90,
    90, 345, 534, 518, 100, 535, 536, 194, 194, 536, 195, 45, 9, 184, 0, 133,
    178, 362, 133, 362, 287, 190, 537, 538, 539, 540, 541, 542, 497, 543, 18, 18,
    544, 544, 18, 545, 133, 0, 546, 531, 547, 548, 549, 550, 551, 552, 553, 458,
    554, 555, 556, 221, 189, 311, 311, 163, 0, 188, 163, 0, 190, 190, 0, 557,
    20, 0, 0, 161, 0, 0, 161, 400, 558, 0, 0, 0, 0, 0, 320, 0,
    559, 560, 560, 561, 0, 0, 562, 562, 382, 332, 563, 564, 565, 565, 49, 0,
    0, 0, 161, 0, 133, 0, 566, 0, 0, 24, 567, 567, 133, 133, 0, 0,
    568, 0, 569, 569, 24, 25, 570, 320, 179, 571, 571, 187, 187, 571, 572, 568,
    573, 574, 575, 576, 577, 578, 579, 580, 581, 582, 583, 26, 584, 255, 234, 585,
    91, 586, 587, 555, 555, 0, 133, 463, 463, 0, 0, 0, 0, 404, 404, 122,
    324, 588, 0, 588, 0, 0, 0, 0, 0, 0, 0, 434, 0, 0, 0, 0,
    589, 589, 0, 78, 78, 78, 406, 406, 407, 305, 305, 303, 251, 0, 324, 480,
    460, 555, 324, 326, 328, 251, 251, 399, 0, 251, 251, 251, 590, 591, 592, 250,
    593, 594, 595, 596, 597, 598, 599, 92, 92, 92, 600, 601, 602, 603, 324, 604,
    605, 606, 607, 608, 609, 610, 611, 612, 612, 612, 0, 0, 613, 614, 615, 466,
    95, 27, 512, 11, 28, 616, 29, 208, 617, 618, 619, 620, 621, 622, 623, 624,
    287, 491, 491, 345, 345, 345, 491, 109, 625, 625, 75, 0, 405, 0, 491, 92,
    103, 127, 127, 405, 405, 0, 0, 233, 626, 30, 161, 560, 627, 270, 270, 247,
    112, 93, 93, 372, 133, 0, 178, 0, 627, 93, 93, 13, 184, 0, 628, 628,
    147, 270, 270, 247, 93, 93, 132, 20, 22, 184, 0, 93, 93, 20, 184, 0,
    627, 93, 93, 138, 184, 0, 93, 93, 35, 184, 0, 627, 93, 93, 93, 48,
    184, 0, 93, 93, 160, 343, 184, 0, 93, 93, 55, 0, 93, 93, 44, 184,
    0, 270, 93, 93, 61, 184, 0, 93, 93, 142, 184, 0, 627, 93, 93, 68,
    0, 93, 93, 132, 72, 184, 0, 100, 93, 93, 483, 75, 184, 0, 0, 627,
    93, 93, 23, 184, 0, 93, 93, 78, 343, 184, 0, 629, 100, 630, 627, 93,
    93, 79, 184, 0, 270, 93, 93, 9, 184, 0, 627, 631, 632, 633, 634, 635,
    636, 493, 31, 194, 536, 529, 637, 502, 406, 406, 406, 638, 639, 640, 641, 642,
    642, 642, 643, 455, 644, 460, 480, 370, 370, 0, 0, 0, 370, 645, 646, 647,
    133, 0, 0, 133, 94, 648, 649, 650, 651, 652, 328, 653, 654, 241, 655, 656,
    657, 658, 659, 660, 661, 662, 327, 327, 369, 369, 369, 327, 369, 369, 0, 663,
    0, 0, 345, 345, 345, 0, 664, 0, 665, 666, 667, 668, 669, 669, 308, 308,
    308, 482, 0, 0, 161, 287, 96, 96, 249, 0, 670, 358, 356, 459, 459, 0,
    0, 671, 229, 672, 229, 229, 672, 0, 602, 602, 673, 463, 670, 358, 356, 96,
    674, 97, 675, 676, 677, 677, 677, 0, 369, 369, 0, 678, 526, 679, 679, 679,
    680, 362, 103, 127, 92, 0, 0, 0, 0, 0, 681, 333, 333, 0, 682, 335,
    404, 404, 406, 0, 0, 0, 0, 683, 406, 330, 0, 281, 0, 0, 0, 0,
    684, 685, 685, 685, 686, 406, 687, 0, 0, 688, 688, 688, 688, 284, 290, 335,
    0, 689, 685, 685, 685, 686, 0, 676, 676, 133, 362, 0, 0, 0, 368, 307,
    307, 0, 134, 0, 0, 0, 319, 0, 296, 133, 0, 0, 362, 0, 0, 0,
    0, 0, 0, 122, 0, 320, 0, 0, 0, 0, 0, 161, 288, 0, 0, 133,
    0, 0, 0, 0, 690, 179, 214, 347, 407, 569, 0, 322, 0, 0, 0, 133,
    0, 0, 179, 214, 161, 134, 165, 0, 122, 0, 320, 0, 0, 0, 691, 0,
    692, 38, 514, 693, 349, 349, 694, 694, 695, 275, 696, 0, 214, 0, 64, 697,
    64, 698, 699, 533, 190, 190, 700, 694, 694, 694, 0, 0, 304, 670, 358, 356,
    356, 701, 701, 0, 214, 0, 702, 144, 0, 0, 0, 669, 703, 704, 704, 133,
    601, 601, 0, 704, 640, 32, 275, 705, 98, 706, 707, 708, 709, 710, 711, 499,
    712, 712, 0, 229, 0, 0, 590, 590, 405, 0, 713, 713, 0, 714, 92, 103,
    122, 127, 457, 457, 457, 0, 0, 0, 715, 716, 717, 229, 0, 0, 199, 0,
    0, 0, 718, 0, 719, 719, 590, 720, 721, 405, 0, 133, 122, 722, 722, 0,
    369, 369, 0, 0, 457, 457, 127, 405, 405, 0, 723, 724, 725, 725, 214, 726,
    726, 727, 712, 100, 728, 728, 41, 728, 145, 729, 496, 730, 731, 731, 0, 0,
    531, 531, 64, 0, 0, 214, 320, 732, 0, 733, 734, 711, 330, 330, 281, 731,
    698, 735, 736, 737, 738, 739, 740, 741, 600, 600, 0, 0, 742, 601, 701, 743,
    743, 163, 177, 177, 163, 744, 745, 746, 747, 190, 190, 694, 748, 0, 0, 749,
    0, 133, 0, 0, 0, 0, 742, 750, 751, 0, 190, 190, 748, 0, 750, 0,
    133, 0, 0, 0, 0, 133, 0, 0, 752, 0, 133, 0, 177, 177, 163, 743,
    743, 163, 0, 0, 0, 0, 0, 751, 177, 177, 163, 743, 743, 163, 0, 600,
    753, 754, 0, 0, 0, 251, 0, 0, 0, 0, 178, 0, 755, 755, 0, 600,
    600, 0, 0, 0, 568, 0, 600, 0, 0, 620, 0, 288, 0, 0, 177, 756,
    756, 0, 0, 0, 179, 0, 0, 229, 0, 179, 0, 0, 757, 0, 0, 109,
    0, 133, 133, 0, 178, 178, 179, 474, 474, 0, 138, 758, 0, 0, 0, 0,
    161, 133, 759, 759, 758, 517, 133, 308, 0, 0, 0, 0, 229, 0, 0, 0,
    0, 305, 305, 303, 251, 0, 671, 229, 672, 760, 0, 0, 0, 0, 133, 0,
    0, 0, 761, 761, 133, 762, 762, 133, 567, 567, 133, 0, 0, 0, 0, 0,
    567, 0, 763, 133, 0, 150, 764, 765, 766, 767, 410, 768, 769, 114, 515, 770,
    771, 101, 101, 101, 0, 0, 0, 0, 0, 109, 109, 142, 0, 187, 0, 0,
    0, 0, 188, 163, 0, 0, 190, 190, 772, 172, 599, 773, 0, 0, 109, 0,
    774, 79, 29, 31, 47, 47, 48, 475, 475, 775, 776, 40, 69, 69, 70, 44,
    0, 0, 0, 0, 161, 0, 187, 0, 190, 190, 34, 773, 190, 190, 101, 773,
    777, 270, 270, 247, 100, 16, 16, 241, 530, 531, 83, 0, 0, 187, 0, 778,
    779, 418, 293, 293, 241, 780, 780, 292, 293, 781, 743, 743, 163, 0, 83, 782,
    544, 0, 0, 0, 0, 214, 0, 783, 783, 177, 177, 163, 701, 744, 745, 746,
    694, 743, 0, 0, 0, 0, 214, 0, 133, 783, 782, 782, 782, 784, 133, 0,
    0, 544, 0, 785, 230, 786, 744, 744, 0, 133, 0, 787, 177, 177, 163, 701,
    745, 746, 694, 744, 788, 789, 790, 14, 14, 482, 482, 482, 14, 34, 791, 792,
    793, 794, 0, 795, 796, 797, 798, 799, 800, 511, 188, 0, 410, 410, 801, 278,
    163, 163, 251, 109, 109, 407, 457, 457, 103, 802, 533, 281, 281, 127, 178, 803,
    133, 481, 803, 481, 804, 804, 482, 482, 190, 213, 213, 163, 805, 806, 218, 214,
    701, 701, 807, 808, 809, 221, 222, 219, 177, 810, 811, 812, 184, 184, 0, 371,
    0, 664, 184, 184, 676, 184, 184, 0, 646, 646, 646, 0, 371, 0, 664, 0,
    370, 370, 0, 328, 184, 184, 184, 184, 406, 676, 813, 757, 757, 813, 814, 815,
    816, 249, 717, 817, 0, 0, 289, 818, 819, 730, 820, 821, 360, 359, 822, 823,
    229, 682, 406, 229, 0, 133, 0, 0, 0, 362, 0, 824, 628, 213, 0, 0,
    825, 299, 611, 35, 0, 0, 36, 826, 720, 827, 828, 102, 829, 830, 831, 832,
    833, 834, 835, 836, 837, 838, 839, 103, 103, 103, 840, 358, 358, 358, 0, 0,
    133, 122, 0, 133, 0, 0, 841, 842, 842, 763, 71, 71, 0, 0, 602, 843,
    163, 163, 844, 236, 845, 362, 846, 846, 657, 358, 358, 0, 133, 0, 847, 847,
    701, 524, 214, 126, 83, 83, 358, 358, 848, 849, 454, 641, 490, 850, 851, 852,
    853, 854, 855, 856, 857, 858, 859, 860, 320, 320, 861, 862, 862, 133, 214, 187,
    863, 575, 575, 133, 214, 187, 864, 133, 214, 320, 320, 865, 214, 214, 866, 179,
    568, 179, 179, 179, 866, 179, 568, 320, 571, 862, 863, 575, 797, 867, 864, 358,
    358, 0, 0, 0, 0, 304, 358, 210, 868, 312, 312, 869, 458, 594, 0, 0,
    407, 870, 0, 347, 0, 647, 405, 405, 78, 133, 0, 0, 533, 0, 404, 404,
    406, 620, 600, 404, 0, 871, 872, 873, 240, 819, 874, 0, 214, 0, 0, 0,
    0, 161, 0, 0, 0, 161, 161, 0, 163, 163, 0, 0, 0, 0, 145, 145,
    875, 0, 0, 0, 288, 0, 0, 876, 673, 877, 190, 190, 269, 878, 876, 876,
    407, 3, 109, 407, 0, 0, 879, 41, 179, 807, 0, 0, 0, 296, 296, 0,
    0, 673, 673, 880, 881, 881, 133, 214, 214, 409, 361, 380, 190, 190, 0, 882,
    882, 394, 342, 0, 875, 475, 39, 0, 0, 0, 883, 0, 0, 0, 0, 0,
    0, 884, 109, 459, 0, 0, 0, 883, 0, 0, 0, 0, 0, 883, 883, 883,
    751, 751, 885, 886, 0, 109, 109, 0, 0, 133, 0, 0, 431, 836, 887, 887,
    753, 888, 889, 890, 459, 459, 133, 0, 0, 875, 0, 0, 0, 457, 891, 625,
    625, 892, 179, 413, 0, 0, 0, 145, 145, 875, 892, 892, 179, 893, 893, 893,
    407, 656, 251, 251, 0, 109, 894, 647, 656, 413, 413, 133, 0, 0, 133, 0,
    134, 179, 895, 895, 895, 407, 656, 109, 656, 413, 761, 896, 104, 104, 133, 606,
    606, 897, 275, 898, 177, 104, 41, 899, 900, 878, 0, 901, 902, 474, 565, 898,
    903, 0, 749, 904, 905, 906, 888, 907, 908, 419, 292, 292, 241, 780, 780, 293,
    292, 65, 42, 681, 681, 406, 909, 0, 658, 658, 0, 909, 487, 910, 816, 487,
    420, 701, 701, 0, 133, 0, 214, 0, 83, 911, 748, 748, 0, 214, 0, 134,
    134, 214, 701, 912, 913, 22, 0, 109, 109, 914, 915, 656, 916, 916, 179, 917,
    190, 918, 447, 919, 0, 179, 214, 347, 685, 407, 569, 296, 304, 65, 920, 131,
    165, 0, 921, 0, 0, 922, 923, 0, 254, 924, 925, 926, 927, 105, 43, 727,
    928, 929, 846, 206, 930, 106, 931, 932, 933, 934, 935, 0, 0, 328, 935, 0,
    328, 935, 0, 936, 937, 938, 939, 752, 752, 161, 0, 133, 48, 48, 0, 134,
    134, 133, 362, 107, 940, 0, 143, 0, 941, 942, 45, 943, 944, 945, 946, 947,
    805, 46, 948, 949, 950, 951, 643, 952, 92, 103, 103, 133, 763, 108, 108, 358,
    108, 369, 369, 369, 369, 369, 942, 281, 281, 0, 664, 953, 127, 92, 369, 369,
    406, 942, 281, 281, 0, 664, 103, 108, 108, 0, 0, 0, 459, 0, 0, 0,
    133, 133, 362, 0, 0, 0, 0, 0, 109, 0, 369, 127, 127, 71, 601, 602,
    954, 955, 956, 456, 957, 958, 959, 960, 961, 47, 962, 286, 867, 963, 964, 806,
    752, 965, 294, 294, 966, 48, 967, 262, 968, 289, 969, 201, 863, 970, 971, 972,
    973, 288, 336, 336, 336, 974, 662, 859, 975, 976, 83, 620, 977, 716, 978, 0,
    236, 0, 0, 472, 0, 0, 0, 178, 0, 0, 133, 0, 746, 746, 0, 0,
    979, 870, 870, 980, 981, 981, 431, 981, 982, 983, 286, 0, 625, 984, 985, 986,
    0, 696, 987, 518, 988, 989, 219, 990, 991, 760, 992, 993, 994, 222, 995, 207,
    996, 808, 809, 203, 351, 763, 122, 133, 122, 133, 190, 133, 351, 133, 0, 997,
    127, 103, 92, 108, 351, 351, 998, 133, 122, 351, 999, 1000, 195, 810, 1001, 200,
    1002, 0, 1003, 1004, 1005, 1006, 0, 419, 780, 780, 241, 292, 292, 293, 406, 0,
    406, 820, 226, 1007, 1008, 724, 407, 407, 407, 407, 1009, 1010, 815, 1011, 1012, 214,
    49, 179, 179, 296, 0, 921, 179, 179, 921, 161, 161, 161, 161, 161, 400, 109,
    178, 459, 134, 373, 165, 133, 333, 296, 324, 751, 506, 506, 49, 125, 125, 0,
    133, 0, 0, 0, 0, 0, 502, 1013, 1013, 406, 1014, 640, 641, 1015, 1016, 1017,
    1018, 405, 1019, 1020, 748, 748, 0, 214, 0, 64, 697, 64, 698, 748, 1021, 503,
    1022, 1023, 1024, 109, 109, 110, 1025, 1026, 1026, 93, 1027, 595, 1028, 1028, 92, 1027,
    698, 109, 0, 0, 272, 533, 1029, 1029, 0, 0, 304, 457, 0, 179, 122, 0,
    459, 459, 0, 0, 671, 229, 672, 229, 229, 672, 0, 0, 179, 1029, 1029, 1029,
    0, 133, 1030, 1031, 1032, 1033, 1034, 1035, 51, 1036, 1037, 773, 1038, 478, 345, 345,
    345, 345, 461, 1039, 477, 76, 76, 0, 1040, 1040, 214, 834, 834, 214, 947, 947,
    214, 1041, 1042, 206, 207, 1043, 1001, 231, 231, 214, 1044, 991, 427, 427, 214, 1045,
    1005, 199, 1046, 211, 210, 272, 485, 63, 63, 540, 1047, 249, 250, 76, 76, 0,
    1040, 1040, 214, 1048, 1048, 214, 834, 834, 214, 947, 947, 214, 1041, 1042, 206, 207,
    1043, 1001, 231, 231, 214, 217, 219, 218, 1049, 991, 427, 427, 457, 457, 1050, 1024,
    115, 115, 1051, 936, 791, 99, 99, 1051, 936, 791, 64, 64, 1052, 427, 1053, 323,
    173, 264, 264, 1052, 427, 457, 457, 1054, 115, 115, 1051, 936, 791, 99, 99, 1051,
    936, 791, 518, 518, 64, 65, 928, 1055, 201, 203, 228, 228, 94, 208, 64, 973,
    469, 469, 133, 966, 966, 1056, 251, 100, 1057, 835, 835, 1058, 1059, 518, 518, 952,
    952, 952, 457, 1060, 233, 115, 115, 1051, 936, 791, 365, 365, 1051, 936, 791, 99,
    99, 1051, 936, 791, 1061, 1061, 1062, 1000, 936, 948, 791, 541, 622, 622, 1063, 315,
    163, 163, 1064, 1065, 1065, 735, 1066, 1067, 457, 457, 259, 0, 251, 83, 1000, 0,
    1068, 633, 633, 83, 518, 518, 633, 518, 518, 835, 1069, 98, 98, 99, 0, 518,
    518, 98, 99, 1070, 1071, 1004, 177, 177, 1072, 163, 0, 0, 0, 0, 214, 0,
    825, 299, 611, 815, 815, 897, 275, 898, 0, 701, 731, 1073, 701, 744, 745, 746,
    731, 756, 694, 177, 223, 177, 810, 1074, 891, 663, 672, 890, 1075, 52, 1076, 0,
    757, 1077, 0, 0, 133, 0, 1077, 914, 691, 691, 0, 133, 133, 569, 134, 569,
    0, 691, 1078, 639, 1079, 1080, 1081, 1082, 160, 1083, 721, 468, 468, 133, 133, 1066,
    0, 163, 0, 131, 0, 0, 0, 735, 133, 0, 0, 1084, 1084, 177, 13, 0,
    163, 133, 0, 0, 807, 807, 133, 0, 231, 468, 1085, 0, 1086, 1087, 1088, 1089,
    1090, 1091, 1092, 204, 1093, 1094, 1066, 1066, 163, 0, 133, 0, 0, 1066, 1095, 525,
    892, 1096, 287, 287, 758, 758, 287, 179, 0, 1040, 1097, 1048, 1098, 1099, 1100, 1101,
    1015, 1102, 290, 470, 470, 1103, 459, 459, 0, 1104, 1104, 64, 75, 1105, 459, 459,
    0, 470, 457, 457, 0, 133, 0, 0, 229, 229, 133, 0, 0, 0, 0, 885,
    885, 885, 1106, 133, 0, 0, 0, 0, 457, 229, 1107, 1108, 1109, 0, 362, 108,
    103, 1110, 93, 93, 90, 93, 916, 157, 90, 90, 93, 84, 90, 71, 93, 93,
    90, 93, 90, 93, 90, 92, 127, 90, 93, 460, 93, 90, 93, 612, 128, 679,
    90, 84, 93, 90, 93, 89, 480, 93, 84, 501, 501, 90, 90, 84, 93, 84,
    84, 0, 0, 78, 555, 96, 0, 184, 368, 407, 457, 722, 0, 712, 1029, 227,
    1029, 694, 677, 441, 1111, 463, 84, 90, 93, 84, 90, 93, 84, 93, 93, 0,
    324, 53, 0, 0, 0, 0, 0, 0, 241, 0, 133, 0, 0, 133, 241, 0,
    0, 0, 0, 133, 133, 285, 135, 285, 7, 1105, 845, 285, 0, 1112, 104, 704,
    0, 0, 1113, 1114, 295, 589, 749, 749, 500, 734, 561, 1115, 1116, 1117, 1118, 1027,
    239, 239, 41, 239, 1119, 687, 1061, 1120, 50, 54, 1121, 1122, 0, 0, 1123, 328,
    327, 646, 161, 161, 161, 0, 55, 151, 19, 1124, 44, 56, 1125, 57, 132, 112,
    685, 685, 214, 745, 335, 335, 1126, 83, 131, 1127, 133, 362, 1128, 115, 523, 1129,
    517, 179, 1130, 1111, 1111, 1111, 0, 1131, 735, 1132, 241, 293, 1133, 292, 780, 117,
    1134, 1135, 1136, 197, 1137, 1138, 1139, 78, 78, 0, 404, 1139, 133, 0, 0, 0,
    0, 0, 0, 1140, 406, 406, 407, 305, 303, 251, 0, 1141, 1142, 58, 118, 78,
    78, 0, 404, 1143, 555, 463, 463, 0, 0, 0, 0, 1143, 119, 119, 0, 133,
    941, 109, 109, 109, 92, 1144, 109, 109, 127, 1144, 941, 941, 1145, 753, 1094, 1146,
    1147, 119, 492, 492, 1148, 1148, 1148, 178, 0, 492, 1144, 1149, 270, 71, 71, 407,
    518, 518, 168, 71, 71, 251, 111, 1150, 1151, 1152, 1153, 1154, 1155, 1156, 1157, 1158,
    1159, 1160, 847, 1161, 1162, 1163, 1164, 60, 205, 718, 1165, 461, 1166, 1167, 1168, 1169,
    40, 814, 1170, 1171, 1149, 120, 469, 469, 1163, 1163, 133, 0, 0, 1172, 469, 121,
    463, 463, 463, 0, 1173, 1174, 1175, 1176, 602, 602, 673, 463, 0, 0, 1177, 1023,
    986, 0, 0, 0, 670, 358, 356, 673, 1178, 423, 199, 158, 158, 158, 0, 0,
    0, 0, 0, 109, 109, 142, 0, 600, 0, 0, 0, 0, 188, 163, 0, 0,
    0, 0, 109, 0, 774, 79, 29, 31, 47, 47, 48, 475, 475, 775, 776, 40,
    69, 69, 70, 44, 0, 0, 0, 0, 161, 0, 600, 0, 777, 270, 270, 247,
    100, 16, 16, 241, 530, 531, 83, 0, 0, 0, 600, 0, 116, 472, 889, 910,
    61, 0, 1179, 62, 545, 545, 139, 1180, 1181, 1182, 152, 64, 0, 0, 289, 1183,
    1184, 1185, 821, 1071, 1186, 1065, 920, 698, 1183, 1187, 750, 750, 1188, 329, 74, 133,
    214, 1165, 1165, 1165, 133, 214, 713, 1189, 1190, 67, 142, 1191, 156, 1192, 1193, 1194,
    1195, 1196, 1197, 864, 1198, 1199, 66, 63, 0, 0, 0, 0, 0, 976, 620, 83,
    122, 122, 976, 0, 0, 1200, 1200, 356, 358, 600, 0, 601, 177, 177, 163, 701,
    691, 747, 0, 694, 122, 122, 122, 670, 358, 356, 600, 601, 790, 743, 744, 691,
    746, 358, 457, 177, 694, 701, 746, 457, 747, 190, 190, 846, 362, 1201, 133, 190,
    1201, 133, 190, 351, 133, 362, 1202, 324, 1203, 756, 68, 216, 805, 805, 214, 808,
    215, 806, 806, 214, 809, 0, 0, 0, 161, 0, 161, 1204, 987, 554, 0, 0,
    161, 1204, 987, 554, 153, 243, 1205, 1206, 259, 15, 1207, 71, 71, 0, 0, 0,
    0, 241, 0, 0, 319, 0, 0, 0, 763, 251, 351, 351, 747, 747, 747, 351,
    177, 701, 746, 71, 747, 975, 1208, 1208, 859, 662, 83, 962, 71, 71, 142, 184,
    0, 1209, 784, 1210, 237, 467, 1211, 183, 1212, 464, 464, 0, 0, 1000, 464, 0,
    0, 0, 368, 373, 373, 373, 0, 307, 1213, 1214, 1072, 123, 1215, 685, 285, 286,
    1216, 1146, 0, 0, 517, 322, 161, 0, 0, 0, 0, 161, 288, 179, 214, 214,
    0, 1217, 0, 0, 165, 726, 287, 287, 296, 1218, 1127, 1219, 776, 1220, 1221, 450,
    1222, 422, 1223, 69, 1224, 1225, 1226, 1227, 1228, 1229, 1230, 471, 1231, 1232, 1233, 1234,
    1235, 1236, 1237, 71, 71, 1238, 1239, 109, 1019, 1240, 1241, 1242, 1243, 1244, 137, 149,
    288, 482, 482, 482, 190, 0, 1245, 1246, 1247, 0, 0, 324, 1248, 0, 362, 108,
    103, 1110, 93, 93, 90, 93, 916, 157, 90, 90, 93, 84, 90, 71, 93, 93,
    90, 93, 90, 93, 90, 92, 127, 90, 93, 460, 93, 90, 93, 612, 128, 679,
    90, 84, 93, 90, 93, 89, 480, 93, 84, 501, 501, 90, 90, 84, 93, 84,
    84, 0, 0, 78, 555, 96, 0, 184, 368, 407, 457, 722, 0, 712, 1029, 227,
    1029, 694, 677, 441, 1111, 463, 84, 90, 93, 84, 90, 93, 84, 93, 93, 455,
    1249, 1250, 1251, 1252, 1253, 1041, 1041, 133, 1254, 831, 831, 214, 214, 214, 696, 696,
    214, 214, 214, 961, 961, 214, 945, 945, 524, 214, 126, 1097, 1097, 214, 738, 738,
    214, 737, 737, 785, 214, 355, 355, 1255, 251, 104, 1142, 705, 705, 1256, 251, 104,
    1000, 378, 378, 1256, 251, 104, 1000, 946, 946, 524, 126, 214, 214, 1036, 1036, 1257,
    407, 214, 214, 1041, 1052, 1258, 140, 154, 877, 290, 290, 1259, 133, 0, 0, 0,
    0, 0, 1260, 1261, 1262, 1263, 1264, 1067, 322, 1265, 279, 279, 279, 0, 1266, 1267,
    1268, 1269, 0, 1270, 238, 72, 1271, 73, 1272, 74, 1273, 267, 246, 246, 0, 1274,
    389, 389, 133, 0, 518, 518, 102, 269, 269, 133, 459, 239, 239, 133, 459, 728,
    728, 133, 119, 0, 0, 763, 319, 0, 0, 133, 362, 133, 108, 322, 75, 0,
    0, 1275, 161, 161, 161, 161, 308, 109, 178, 459, 134, 373, 165, 133, 333, 296,
    324, 751, 1276, 147, 750, 76, 0, 684, 684, 333, 0, 0, 689, 689, 0, 141,
    155, 1277, 1278, 0, 483, 483, 114, 1120, 522, 522, 1279, 1280, 483, 1281, 1282, 1283,
    1284, 1285, 664, 700, 686, 25, 559, 570, 570, 133, 917, 190, 190, 254, 25, 1286,
    917, 917, 190, 918, 25, 1286, 25, 25, 133, 917, 917, 190, 918, 25, 1286, 307,
    307, 307, 25, 1286, 0, 252, 235, 570, 1287, 1288, 426, 426, 325, 175, 490, 490,
    405, 1289, 1248, 493, 493, 64, 426, 426, 404, 326, 0, 328, 328, 426, 324, 1290,
    1291, 1292, 187, 1293, 1294, 1295, 1296, 1297, 77, 1110, 1110, 482, 110, 62, 1298, 77,
    1299, 236, 236, 0, 281, 281, 518, 379, 1300, 1301, 1302, 1303, 1304, 1305, 1280, 124,
    70, 1306, 1148, 1023, 986, 1093, 296, 296, 1148, 161, 161, 0, 23, 371, 406, 1307,
    281, 281, 281, 406, 1307, 371, 371, 345, 345, 345, 371, 1307, 849, 1308, 1308, 133,
    179, 603, 1309, 1310, 1311, 1312, 755, 350, 904, 1296, 348, 348, 916, 916, 916, 75,
    0, 1313, 1314, 133, 0, 0, 78, 0, 0, 165, 0, 0, 133, 1315, 1316, 732,
    1317, 542, 0, 981, 542, 0, 125, 1318, 1319, 975, 975, 976, 620, 83, 975, 269,
    269, 269, 1320, 0, 126, 1321, 528, 1322, 1323, 122, 122, 670, 358, 356, 600, 601,
    790, 743, 744, 691, 746, 747, 190, 190, 177, 701, 790, 846, 351, 351, 747, 747,
    747, 351, 0, 0, 0, 133, 0, 0, 178, 178, 1324, 133, 179, 179, 0, 0,
    0, 133, 0, 0, 178, 178, 1325, 133, 214, 214, 229, 229, 133, 0, 0, 0,
    886, 1326, 1326, 824, 665, 815, 886, 886, 1326, 133, 0, 133, 0, 0, 457, 229,
    566, 1327, 209, 127, 127, 127, 0, 0, 71, 71, 600, 601, 602, 1328, 1308, 1329,
    202, 128, 128, 128, 0, 1330, 79, 0, 1331, 1332, 1333, 1334, 1335, 459, 459, 35,
    0, 0, 133, 90, 477, 459, 1336, 1337, 1338, 1339, 1340, 630, 1341, 807, 763, 763,
    807, 590, 0, 133, 0, 0, 251, 0, 0, 590, 1342, 1343, 1344, 1345, 881, 1346,
    1347, 1348, 507, 507, 1349, 1349, 1350, 1351, 564, 298, 133, 214, 75, 405, 0, 0,
    133, 0, 406, 406, 407, 135, 133, 133, 0, 0, 406, 406, 407, 406, 1015, 1350,
    564, 1352, 1353, 9, 148, 257, 211, 1354, 725, 1355, 1356, 1357, 1358, 1359, 369, 369,
    0, 0, 371, 0, 0, 664, 0, 0, 0, 0, 0, 0, 369, 1360, 601, 601,
    0, 133, 0, 1361, 701, 743, 743, 163, 177, 177, 163, 744, 745, 746, 691, 694,
    601, 722, 722, 722, 0, 0, 0, 1362, 0, 0, 0, 0, 0, 0, 0, 161,
    288, 288, 161, 288, 746, 0, 0, 1363, 1066, 177, 1066, 177, 731, 190, 190, 1141,
    0, 0, 0, 122, 0, 997, 190, 190, 1141, 0, 0, 0, 122, 0, 351, 190,
    190, 518, 518, 557, 0, 190, 190, 190, 518, 142, 337, 1112, 1112, 743, 133, 177,
    1112, 918, 0, 1299, 1364, 129, 1365, 1366, 133, 0, 0, 0, 0, 178, 178, 660,
    0, 0, 762, 1106, 1367, 1368, 862, 1369, 1370, 1286,
];

pub fn get_oracle_segment_grammar(name: &str) -> Option<RootGrammar> {
    match name {
        "AbortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(0), tables: &ORACLE_TABLES }),
        "AbsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1), tables: &ORACLE_TABLES }),
        "AbsentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2), tables: &ORACLE_TABLES }),
        "AbsoluteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3), tables: &ORACLE_TABLES }),
        "AccessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4), tables: &ORACLE_TABLES }),
        "AccessObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(5), tables: &ORACLE_TABLES }),
        "AccessPermissionSegment" => Some(RootGrammar { grammar_id: GrammarId(136), tables: &ORACLE_TABLES }),
        "AccessPermissionsSegment" => Some(RootGrammar { grammar_id: GrammarId(203), tables: &ORACLE_TABLES }),
        "AccessSchemaObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(224), tables: &ORACLE_TABLES }),
        "AccessSchemaPluralObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(247), tables: &ORACLE_TABLES }),
        "AccessStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(270), tables: &ORACLE_TABLES }),
        "AccessTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(273), tables: &ORACLE_TABLES }),
        "AccessibleByClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(293), tables: &ORACLE_TABLES }),
        "AccessibleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(306), tables: &ORACLE_TABLES }),
        "AccessorGrammar" => Some(RootGrammar { grammar_id: GrammarId(307), tables: &ORACLE_TABLES }),
        "AccountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(309), tables: &ORACLE_TABLES }),
        "AccountsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(310), tables: &ORACLE_TABLES }),
        "ActionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(311), tables: &ORACLE_TABLES }),
        "ActiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(312), tables: &ORACLE_TABLES }),
        "AdaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(313), tables: &ORACLE_TABLES }),
        "AddKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(314), tables: &ORACLE_TABLES }),
        "AdminKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(315), tables: &ORACLE_TABLES }),
        "AdministerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(316), tables: &ORACLE_TABLES }),
        "AdvancedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(317), tables: &ORACLE_TABLES }),
        "AdviseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(318), tables: &ORACLE_TABLES }),
        "AdvisorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(319), tables: &ORACLE_TABLES }),
        "AfterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(320), tables: &ORACLE_TABLES }),
        "AggregateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(321), tables: &ORACLE_TABLES }),
        "AggregateOrderByClause" => Some(RootGrammar { grammar_id: GrammarId(322), tables: &ORACLE_TABLES }),
        "AliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(323), tables: &ORACLE_TABLES }),
        "AliasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(333), tables: &ORACLE_TABLES }),
        "AliasedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(334), tables: &ORACLE_TABLES }),
        "AllKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(337), tables: &ORACLE_TABLES }),
        "AllocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(338), tables: &ORACLE_TABLES }),
        "AlsoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(339), tables: &ORACLE_TABLES }),
        "AlterDatabaseLinkStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(340), tables: &ORACLE_TABLES }),
        "AlterFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(363), tables: &ORACLE_TABLES }),
        "AlterIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(374), tables: &ORACLE_TABLES }),
        "AlterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(406), tables: &ORACLE_TABLES }),
        "AlterPackageStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(407), tables: &ORACLE_TABLES }),
        "AlterSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(416), tables: &ORACLE_TABLES }),
        "AlterSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(432), tables: &ORACLE_TABLES }),
        "AlterSessionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(438), tables: &ORACLE_TABLES }),
        "AlterSynonymStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(563), tables: &ORACLE_TABLES }),
        "AlterTableColumnClausesSegment" => Some(RootGrammar { grammar_id: GrammarId(573), tables: &ORACLE_TABLES }),
        "AlterTableConstraintClauses" => Some(RootGrammar { grammar_id: GrammarId(598), tables: &ORACLE_TABLES }),
        "AlterTableDropColumnGrammar" => Some(RootGrammar { grammar_id: GrammarId(632), tables: &ORACLE_TABLES }),
        "AlterTableOptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(637), tables: &ORACLE_TABLES }),
        "AlterTablePropertiesSegment" => Some(RootGrammar { grammar_id: GrammarId(664), tables: &ORACLE_TABLES }),
        "AlterTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(669), tables: &ORACLE_TABLES }),
        "AlterTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(685), tables: &ORACLE_TABLES }),
        "AlwaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(700), tables: &ORACLE_TABLES }),
        "AmpersandSegment" => Some(RootGrammar { grammar_id: GrammarId(701), tables: &ORACLE_TABLES }),
        "AnalyseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(702), tables: &ORACLE_TABLES }),
        "AnalyticKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(703), tables: &ORACLE_TABLES }),
        "AnalyzeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(704), tables: &ORACLE_TABLES }),
        "AndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(705), tables: &ORACLE_TABLES }),
        "AndOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(706), tables: &ORACLE_TABLES }),
        "AnyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(707), tables: &ORACLE_TABLES }),
        "ApplyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(708), tables: &ORACLE_TABLES }),
        "ArchivalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(709), tables: &ORACLE_TABLES }),
        "ArchiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(710), tables: &ORACLE_TABLES }),
        "AreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(711), tables: &ORACLE_TABLES }),
        "ArithmeticBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(712), tables: &ORACLE_TABLES }),
        "ArrayAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(725), tables: &ORACLE_TABLES }),
        "ArrayExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(733), tables: &ORACLE_TABLES }),
        "ArrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(734), tables: &ORACLE_TABLES }),
        "ArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(735), tables: &ORACLE_TABLES }),
        "ArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(738), tables: &ORACLE_TABLES }),
        "AsAliasOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(739), tables: &ORACLE_TABLES }),
        "AsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(741), tables: &ORACLE_TABLES }),
        "AscKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(742), tables: &ORACLE_TABLES }),
        "AsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(743), tables: &ORACLE_TABLES }),
        "AssertionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(744), tables: &ORACLE_TABLES }),
        "AssignmentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(745), tables: &ORACLE_TABLES }),
        "AssignmentOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(746), tables: &ORACLE_TABLES }),
        "AssignmentStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(747), tables: &ORACLE_TABLES }),
        "AsymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(763), tables: &ORACLE_TABLES }),
        "AtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(764), tables: &ORACLE_TABLES }),
        "AtSignSegment" => Some(RootGrammar { grammar_id: GrammarId(765), tables: &ORACLE_TABLES }),
        "AtomicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(766), tables: &ORACLE_TABLES }),
        "AttributeIndicatorSegment" => Some(RootGrammar { grammar_id: GrammarId(767), tables: &ORACLE_TABLES }),
        "AttributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(768), tables: &ORACLE_TABLES }),
        "AttributesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(769), tables: &ORACLE_TABLES }),
        "AuditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(770), tables: &ORACLE_TABLES }),
        "AuthenticatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(771), tables: &ORACLE_TABLES }),
        "AuthidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(772), tables: &ORACLE_TABLES }),
        "AuthorizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(773), tables: &ORACLE_TABLES }),
        "AutoIncrementGrammar" => Some(RootGrammar { grammar_id: GrammarId(774), tables: &ORACLE_TABLES }),
        "AutoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(776), tables: &ORACLE_TABLES }),
        "Auto_incrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(777), tables: &ORACLE_TABLES }),
        "AvgKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(778), tables: &ORACLE_TABLES }),
        "Avg_row_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(779), tables: &ORACLE_TABLES }),
        "BackupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(780), tables: &ORACLE_TABLES }),
        "BackwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(781), tables: &ORACLE_TABLES }),
        "BareFunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(782), tables: &ORACLE_TABLES }),
        "BaseExpressionElementGrammar" => Some(RootGrammar { grammar_id: GrammarId(783), tables: &ORACLE_TABLES }),
        "BaseFileSegment" => Some(RootGrammar { grammar_id: GrammarId(797), tables: &ORACLE_TABLES }),
        "BaseSegment" => Some(RootGrammar { grammar_id: GrammarId(798), tables: &ORACLE_TABLES }),
        "BasicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(799), tables: &ORACLE_TABLES }),
        "BatchDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(800), tables: &ORACLE_TABLES }),
        "BatchSegment" => Some(RootGrammar { grammar_id: GrammarId(801), tables: &ORACLE_TABLES }),
        "BecomeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(811), tables: &ORACLE_TABLES }),
        "BeforeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(812), tables: &ORACLE_TABLES }),
        "BeginEndSegment" => Some(RootGrammar { grammar_id: GrammarId(813), tables: &ORACLE_TABLES }),
        "BeginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(820), tables: &ORACLE_TABLES }),
        "BernoulliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(821), tables: &ORACLE_TABLES }),
        "BetweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(822), tables: &ORACLE_TABLES }),
        "BigintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(823), tables: &ORACLE_TABLES }),
        "BinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(824), tables: &ORACLE_TABLES }),
        "BinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(825), tables: &ORACLE_TABLES }),
        "BinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(831), tables: &ORACLE_TABLES }),
        "BindColonSegment" => Some(RootGrammar { grammar_id: GrammarId(832), tables: &ORACLE_TABLES }),
        "BindVariableSegment" => Some(RootGrammar { grammar_id: GrammarId(833), tables: &ORACLE_TABLES }),
        "BindingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(842), tables: &ORACLE_TABLES }),
        "BitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(843), tables: &ORACLE_TABLES }),
        "Bit_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(844), tables: &ORACLE_TABLES }),
        "BitmapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(845), tables: &ORACLE_TABLES }),
        "BitvarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(846), tables: &ORACLE_TABLES }),
        "BitwiseAndSegment" => Some(RootGrammar { grammar_id: GrammarId(847), tables: &ORACLE_TABLES }),
        "BitwiseLShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(848), tables: &ORACLE_TABLES }),
        "BitwiseOrSegment" => Some(RootGrammar { grammar_id: GrammarId(851), tables: &ORACLE_TABLES }),
        "BitwiseRShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(852), tables: &ORACLE_TABLES }),
        "BitwiseXorSegment" => Some(RootGrammar { grammar_id: GrammarId(855), tables: &ORACLE_TABLES }),
        "BlobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(856), tables: &ORACLE_TABLES }),
        "BodyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(857), tables: &ORACLE_TABLES }),
        "BoolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(858), tables: &ORACLE_TABLES }),
        "BooleanBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(859), tables: &ORACLE_TABLES }),
        "BooleanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(862), tables: &ORACLE_TABLES }),
        "BooleanLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(863), tables: &ORACLE_TABLES }),
        "BothKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(866), tables: &ORACLE_TABLES }),
        "BracketedArguments" => Some(RootGrammar { grammar_id: GrammarId(867), tables: &ORACLE_TABLES }),
        "BracketedColumnReferenceListGrammar" => Some(RootGrammar { grammar_id: GrammarId(870), tables: &ORACLE_TABLES }),
        "BracketedSegment" => Some(RootGrammar { grammar_id: GrammarId(873), tables: &ORACLE_TABLES }),
        "BracketedSetExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(874), tables: &ORACLE_TABLES }),
        "BreadthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(876), tables: &ORACLE_TABLES }),
        "BreakKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(877), tables: &ORACLE_TABLES }),
        "BrowseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(878), tables: &ORACLE_TABLES }),
        "Buffer_poolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(879), tables: &ORACLE_TABLES }),
        "BulkCollectIntoClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(880), tables: &ORACLE_TABLES }),
        "BulkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(889), tables: &ORACLE_TABLES }),
        "Bulk_exceptionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(890), tables: &ORACLE_TABLES }),
        "Bulk_rowcountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(891), tables: &ORACLE_TABLES }),
        "ByKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(892), tables: &ORACLE_TABLES }),
        "ByteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(893), tables: &ORACLE_TABLES }),
        "CTEColumnList" => Some(RootGrammar { grammar_id: GrammarId(894), tables: &ORACLE_TABLES }),
        "CTEDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(896), tables: &ORACLE_TABLES }),
        "CacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(902), tables: &ORACLE_TABLES }),
        "CallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(903), tables: &ORACLE_TABLES }),
        "CalledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(904), tables: &ORACLE_TABLES }),
        "CapacityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(905), tables: &ORACLE_TABLES }),
        "CardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(906), tables: &ORACLE_TABLES }),
        "CascadeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(907), tables: &ORACLE_TABLES }),
        "CascadedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(908), tables: &ORACLE_TABLES }),
        "CaseExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(909), tables: &ORACLE_TABLES }),
        "CaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(950), tables: &ORACLE_TABLES }),
        "CastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(951), tables: &ORACLE_TABLES }),
        "CastOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(952), tables: &ORACLE_TABLES }),
        "CatalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(953), tables: &ORACLE_TABLES }),
        "Catalog_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(954), tables: &ORACLE_TABLES }),
        "CeilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(955), tables: &ORACLE_TABLES }),
        "CeilingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(956), tables: &ORACLE_TABLES }),
        "Cell_flash_cacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(957), tables: &ORACLE_TABLES }),
        "ChainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(958), tables: &ORACLE_TABLES }),
        "ChangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(959), tables: &ORACLE_TABLES }),
        "CharCharacterSetGrammar" => Some(RootGrammar { grammar_id: GrammarId(960), tables: &ORACLE_TABLES }),
        "CharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(961), tables: &ORACLE_TABLES }),
        "Char_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(962), tables: &ORACLE_TABLES }),
        "CharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(963), tables: &ORACLE_TABLES }),
        "Character_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(964), tables: &ORACLE_TABLES }),
        "Character_set_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(965), tables: &ORACLE_TABLES }),
        "Character_set_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(966), tables: &ORACLE_TABLES }),
        "Character_set_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(967), tables: &ORACLE_TABLES }),
        "CharacteristicsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(968), tables: &ORACLE_TABLES }),
        "CharactersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(969), tables: &ORACLE_TABLES }),
        "CheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(970), tables: &ORACLE_TABLES }),
        "CheckedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(971), tables: &ORACLE_TABLES }),
        "CheckpointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(972), tables: &ORACLE_TABLES }),
        "ChecksumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(973), tables: &ORACLE_TABLES }),
        "ClassKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(974), tables: &ORACLE_TABLES }),
        "Class_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(975), tables: &ORACLE_TABLES }),
        "ClobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(976), tables: &ORACLE_TABLES }),
        "CloseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(977), tables: &ORACLE_TABLES }),
        "CloseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(978), tables: &ORACLE_TABLES }),
        "ClusterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(983), tables: &ORACLE_TABLES }),
        "ClusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(984), tables: &ORACLE_TABLES }),
        "CoalesceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(985), tables: &ORACLE_TABLES }),
        "CobolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(986), tables: &ORACLE_TABLES }),
        "CodeSegment" => Some(RootGrammar { grammar_id: GrammarId(987), tables: &ORACLE_TABLES }),
        "CollateGrammar" => Some(RootGrammar { grammar_id: GrammarId(988), tables: &ORACLE_TABLES }),
        "CollateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(989), tables: &ORACLE_TABLES }),
        "CollationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(990), tables: &ORACLE_TABLES }),
        "CollationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(991), tables: &ORACLE_TABLES }),
        "Collation_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(997), tables: &ORACLE_TABLES }),
        "Collation_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(998), tables: &ORACLE_TABLES }),
        "Collation_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(999), tables: &ORACLE_TABLES }),
        "CollectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1000), tables: &ORACLE_TABLES }),
        "CollectionTypeDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(1001), tables: &ORACLE_TABLES }),
        "ColonDelimiterSegment" => Some(RootGrammar { grammar_id: GrammarId(1022), tables: &ORACLE_TABLES }),
        "ColonPrefixSegment" => Some(RootGrammar { grammar_id: GrammarId(1023), tables: &ORACLE_TABLES }),
        "ColonSegment" => Some(RootGrammar { grammar_id: GrammarId(1024), tables: &ORACLE_TABLES }),
        "ColumnConstraintDefaultGrammar" => Some(RootGrammar { grammar_id: GrammarId(1025), tables: &ORACLE_TABLES }),
        "ColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(1032), tables: &ORACLE_TABLES }),
        "ColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(1060), tables: &ORACLE_TABLES }),
        "ColumnGeneratedGrammar" => Some(RootGrammar { grammar_id: GrammarId(1084), tables: &ORACLE_TABLES }),
        "ColumnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1085), tables: &ORACLE_TABLES }),
        "ColumnReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1086), tables: &ORACLE_TABLES }),
        "ColumnTypeReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1090), tables: &ORACLE_TABLES }),
        "Column_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1094), tables: &ORACLE_TABLES }),
        "Column_valueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1095), tables: &ORACLE_TABLES }),
        "ColumnsExpressionFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1096), tables: &ORACLE_TABLES }),
        "ColumnsExpressionFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1097), tables: &ORACLE_TABLES }),
        "ColumnsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1098), tables: &ORACLE_TABLES }),
        "ColumnsExpressionNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(1099), tables: &ORACLE_TABLES }),
        "ColumnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1100), tables: &ORACLE_TABLES }),
        "CommaSegment" => Some(RootGrammar { grammar_id: GrammarId(1101), tables: &ORACLE_TABLES }),
        "Command_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1102), tables: &ORACLE_TABLES }),
        "Command_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1103), tables: &ORACLE_TABLES }),
        "CommentClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1104), tables: &ORACLE_TABLES }),
        "CommentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1107), tables: &ORACLE_TABLES }),
        "CommentSegment" => Some(RootGrammar { grammar_id: GrammarId(1108), tables: &ORACLE_TABLES }),
        "CommentStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1109), tables: &ORACLE_TABLES }),
        "CommitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1135), tables: &ORACLE_TABLES }),
        "CommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1136), tables: &ORACLE_TABLES }),
        "ComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1137), tables: &ORACLE_TABLES }),
        "ComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1146), tables: &ORACLE_TABLES }),
        "CompileClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1147), tables: &ORACLE_TABLES }),
        "CompileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1162), tables: &ORACLE_TABLES }),
        "CompletionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1163), tables: &ORACLE_TABLES }),
        "CompositeBinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1164), tables: &ORACLE_TABLES }),
        "CompositeComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1165), tables: &ORACLE_TABLES }),
        "CompoundKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1166), tables: &ORACLE_TABLES }),
        "CompoundTriggerBlock" => Some(RootGrammar { grammar_id: GrammarId(1167), tables: &ORACLE_TABLES }),
        "CompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1173), tables: &ORACLE_TABLES }),
        "ComputeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1174), tables: &ORACLE_TABLES }),
        "ConcatSegment" => Some(RootGrammar { grammar_id: GrammarId(1175), tables: &ORACLE_TABLES }),
        "ConditionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1178), tables: &ORACLE_TABLES }),
        "Condition_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1179), tables: &ORACLE_TABLES }),
        "ConditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1180), tables: &ORACLE_TABLES }),
        "ConditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1181), tables: &ORACLE_TABLES }),
        "ConnectByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1185), tables: &ORACLE_TABLES }),
        "ConnectByRootGrammar" => Some(RootGrammar { grammar_id: GrammarId(1190), tables: &ORACLE_TABLES }),
        "ConnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1193), tables: &ORACLE_TABLES }),
        "Connect_by_rootKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1194), tables: &ORACLE_TABLES }),
        "ConnectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1195), tables: &ORACLE_TABLES }),
        "Connection_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1196), tables: &ORACLE_TABLES }),
        "ConstantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1197), tables: &ORACLE_TABLES }),
        "ConstraintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1198), tables: &ORACLE_TABLES }),
        "Constraint_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1199), tables: &ORACLE_TABLES }),
        "Constraint_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1200), tables: &ORACLE_TABLES }),
        "Constraint_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1201), tables: &ORACLE_TABLES }),
        "ConstraintsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1202), tables: &ORACLE_TABLES }),
        "ConstructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1203), tables: &ORACLE_TABLES }),
        "ContainerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1204), tables: &ORACLE_TABLES }),
        "ContainsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1205), tables: &ORACLE_TABLES }),
        "ContainstableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1206), tables: &ORACLE_TABLES }),
        "ContextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1207), tables: &ORACLE_TABLES }),
        "ContinueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1208), tables: &ORACLE_TABLES }),
        "ContinueStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1209), tables: &ORACLE_TABLES }),
        "ConversionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1215), tables: &ORACLE_TABLES }),
        "ConvertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1216), tables: &ORACLE_TABLES }),
        "CopyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1217), tables: &ORACLE_TABLES }),
        "CorrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1218), tables: &ORACLE_TABLES }),
        "CorrespondingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1219), tables: &ORACLE_TABLES }),
        "CountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1220), tables: &ORACLE_TABLES }),
        "Covar_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1221), tables: &ORACLE_TABLES }),
        "Covar_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1222), tables: &ORACLE_TABLES }),
        "CreateCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1223), tables: &ORACLE_TABLES }),
        "CreateDatabaseLinkStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1250), tables: &ORACLE_TABLES }),
        "CreateDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1280), tables: &ORACLE_TABLES }),
        "CreateFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1285), tables: &ORACLE_TABLES }),
        "CreateIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1336), tables: &ORACLE_TABLES }),
        "CreateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1349), tables: &ORACLE_TABLES }),
        "CreateModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1350), tables: &ORACLE_TABLES }),
        "CreatePackageBodyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1370), tables: &ORACLE_TABLES }),
        "CreatePackageStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1397), tables: &ORACLE_TABLES }),
        "CreateProcedureStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1419), tables: &ORACLE_TABLES }),
        "CreateRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1443), tables: &ORACLE_TABLES }),
        "CreateSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1448), tables: &ORACLE_TABLES }),
        "CreateSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(1453), tables: &ORACLE_TABLES }),
        "CreateSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1473), tables: &ORACLE_TABLES }),
        "CreateSynonymStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1479), tables: &ORACLE_TABLES }),
        "CreateTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1495), tables: &ORACLE_TABLES }),
        "CreateTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1523), tables: &ORACLE_TABLES }),
        "CreateTypeBodyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1574), tables: &ORACLE_TABLES }),
        "CreateTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1592), tables: &ORACLE_TABLES }),
        "CreateUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1620), tables: &ORACLE_TABLES }),
        "CreateViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1695), tables: &ORACLE_TABLES }),
        "CreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1718), tables: &ORACLE_TABLES }),
        "CreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1719), tables: &ORACLE_TABLES }),
        "CreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1720), tables: &ORACLE_TABLES }),
        "CreationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1721), tables: &ORACLE_TABLES }),
        "CriticalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1722), tables: &ORACLE_TABLES }),
        "CrossKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1723), tables: &ORACLE_TABLES }),
        "CrosseditionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1724), tables: &ORACLE_TABLES }),
        "CsvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1725), tables: &ORACLE_TABLES }),
        "CubeFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1726), tables: &ORACLE_TABLES }),
        "CubeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1727), tables: &ORACLE_TABLES }),
        "CubeRollupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1728), tables: &ORACLE_TABLES }),
        "Cume_distKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1734), tables: &ORACLE_TABLES }),
        "CurrentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1735), tables: &ORACLE_TABLES }),
        "Current_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1736), tables: &ORACLE_TABLES }),
        "Current_default_transform_groupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1737), tables: &ORACLE_TABLES }),
        "Current_pathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1738), tables: &ORACLE_TABLES }),
        "Current_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1739), tables: &ORACLE_TABLES }),
        "Current_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1740), tables: &ORACLE_TABLES }),
        "Current_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1741), tables: &ORACLE_TABLES }),
        "Current_transform_group_for_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1742), tables: &ORACLE_TABLES }),
        "Current_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1743), tables: &ORACLE_TABLES }),
        "CurrvalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1744), tables: &ORACLE_TABLES }),
        "CursorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1745), tables: &ORACLE_TABLES }),
        "Cursor_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1746), tables: &ORACLE_TABLES }),
        "CycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1747), tables: &ORACLE_TABLES }),
        "DBLinkAuthenticationGrammar" => Some(RootGrammar { grammar_id: GrammarId(1748), tables: &ORACLE_TABLES }),
        "DataKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1759), tables: &ORACLE_TABLES }),
        "DatabaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1760), tables: &ORACLE_TABLES }),
        "DatabaseLinkReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1761), tables: &ORACLE_TABLES }),
        "DatabaseReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1764), tables: &ORACLE_TABLES }),
        "DatabasesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1768), tables: &ORACLE_TABLES }),
        "DatatypeIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1769), tables: &ORACLE_TABLES }),
        "DatatypeSegment" => Some(RootGrammar { grammar_id: GrammarId(1773), tables: &ORACLE_TABLES }),
        "DateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1801), tables: &ORACLE_TABLES }),
        "DatePartFunctionName" => Some(RootGrammar { grammar_id: GrammarId(1802), tables: &ORACLE_TABLES }),
        "DatePartFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1803), tables: &ORACLE_TABLES }),
        "DateTimeFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1804), tables: &ORACLE_TABLES }),
        "DateTimeLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(1810), tables: &ORACLE_TABLES }),
        "DatetimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1822), tables: &ORACLE_TABLES }),
        "DatetimeUnitSegment" => Some(RootGrammar { grammar_id: GrammarId(1823), tables: &ORACLE_TABLES }),
        "Datetime_interval_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1824), tables: &ORACLE_TABLES }),
        "Datetime_interval_precisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1825), tables: &ORACLE_TABLES }),
        "DayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1826), tables: &ORACLE_TABLES }),
        "Day_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1827), tables: &ORACLE_TABLES }),
        "Day_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1828), tables: &ORACLE_TABLES }),
        "Day_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1829), tables: &ORACLE_TABLES }),
        "Day_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1830), tables: &ORACLE_TABLES }),
        "DayofmonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1831), tables: &ORACLE_TABLES }),
        "DayofweekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1832), tables: &ORACLE_TABLES }),
        "DayofyearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1833), tables: &ORACLE_TABLES }),
        "DaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1834), tables: &ORACLE_TABLES }),
        "Dba_recyclebinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1835), tables: &ORACLE_TABLES }),
        "DbccKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1836), tables: &ORACLE_TABLES }),
        "DbtimezoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1837), tables: &ORACLE_TABLES }),
        "DdlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1838), tables: &ORACLE_TABLES }),
        "DeallocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1839), tables: &ORACLE_TABLES }),
        "DebugKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1840), tables: &ORACLE_TABLES }),
        "DecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1841), tables: &ORACLE_TABLES }),
        "DecimalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1842), tables: &ORACLE_TABLES }),
        "DeclareCursorVariableSegment" => Some(RootGrammar { grammar_id: GrammarId(1843), tables: &ORACLE_TABLES }),
        "DeclareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1857), tables: &ORACLE_TABLES }),
        "DeclareSegment" => Some(RootGrammar { grammar_id: GrammarId(1858), tables: &ORACLE_TABLES }),
        "Dedent" => Some(RootGrammar { grammar_id: GrammarId(332), tables: &ORACLE_TABLES }),
        "DefaultCollationClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1893), tables: &ORACLE_TABLES }),
        "DefaultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1897), tables: &ORACLE_TABLES }),
        "DefaultValuesGrammar" => Some(RootGrammar { grammar_id: GrammarId(1898), tables: &ORACLE_TABLES }),
        "DefaultsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1901), tables: &ORACLE_TABLES }),
        "DeferrableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1902), tables: &ORACLE_TABLES }),
        "DeferredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1903), tables: &ORACLE_TABLES }),
        "DefinedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1904), tables: &ORACLE_TABLES }),
        "DefinerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1905), tables: &ORACLE_TABLES }),
        "DefinitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1906), tables: &ORACLE_TABLES }),
        "DegreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1907), tables: &ORACLE_TABLES }),
        "Delay_key_writeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1908), tables: &ORACLE_TABLES }),
        "DelayedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1909), tables: &ORACLE_TABLES }),
        "DelegateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1910), tables: &ORACLE_TABLES }),
        "DeleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1911), tables: &ORACLE_TABLES }),
        "DeleteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1912), tables: &ORACLE_TABLES }),
        "DeletingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1917), tables: &ORACLE_TABLES }),
        "DelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(1918), tables: &ORACLE_TABLES }),
        "DelimiterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1919), tables: &ORACLE_TABLES }),
        "DelimitersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1920), tables: &ORACLE_TABLES }),
        "Dense_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1921), tables: &ORACLE_TABLES }),
        "DenyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1922), tables: &ORACLE_TABLES }),
        "DepthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1923), tables: &ORACLE_TABLES }),
        "DerefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1924), tables: &ORACLE_TABLES }),
        "DerivedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1925), tables: &ORACLE_TABLES }),
        "DescKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1926), tables: &ORACLE_TABLES }),
        "DescribeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1927), tables: &ORACLE_TABLES }),
        "DescribeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1928), tables: &ORACLE_TABLES }),
        "DescriptorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1932), tables: &ORACLE_TABLES }),
        "DestroyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1933), tables: &ORACLE_TABLES }),
        "DestructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1934), tables: &ORACLE_TABLES }),
        "DeterministicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1935), tables: &ORACLE_TABLES }),
        "DiagnosticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1936), tables: &ORACLE_TABLES }),
        "DictionaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1937), tables: &ORACLE_TABLES }),
        "DigestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1938), tables: &ORACLE_TABLES }),
        "DimensionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1939), tables: &ORACLE_TABLES }),
        "DirectiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1940), tables: &ORACLE_TABLES }),
        "DirectoriesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1941), tables: &ORACLE_TABLES }),
        "DirectoryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1942), tables: &ORACLE_TABLES }),
        "DisableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1943), tables: &ORACLE_TABLES }),
        "DisconnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1944), tables: &ORACLE_TABLES }),
        "DiskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1945), tables: &ORACLE_TABLES }),
        "DispatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1946), tables: &ORACLE_TABLES }),
        "DistinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1947), tables: &ORACLE_TABLES }),
        "DistinctrowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1948), tables: &ORACLE_TABLES }),
        "DistributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1949), tables: &ORACLE_TABLES }),
        "DistributedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1950), tables: &ORACLE_TABLES }),
        "DivKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1951), tables: &ORACLE_TABLES }),
        "DivideSegment" => Some(RootGrammar { grammar_id: GrammarId(1952), tables: &ORACLE_TABLES }),
        "DmlEventClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1953), tables: &ORACLE_TABLES }),
        "DmlGrammar" => Some(RootGrammar { grammar_id: GrammarId(1966), tables: &ORACLE_TABLES }),
        "DmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1975), tables: &ORACLE_TABLES }),
        "DoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1976), tables: &ORACLE_TABLES }),
        "DomainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1977), tables: &ORACLE_TABLES }),
        "DotSegment" => Some(RootGrammar { grammar_id: GrammarId(1978), tables: &ORACLE_TABLES }),
        "DoubleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1979), tables: &ORACLE_TABLES }),
        "DropBehaviorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1980), tables: &ORACLE_TABLES }),
        "DropCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1985), tables: &ORACLE_TABLES }),
        "DropClusterStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1993), tables: &ORACLE_TABLES }),
        "DropDatabaseLinkStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2004), tables: &ORACLE_TABLES }),
        "DropDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2011), tables: &ORACLE_TABLES }),
        "DropFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2017), tables: &ORACLE_TABLES }),
        "DropIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2022), tables: &ORACLE_TABLES }),
        "DropKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2028), tables: &ORACLE_TABLES }),
        "DropModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2029), tables: &ORACLE_TABLES }),
        "DropPackageStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2034), tables: &ORACLE_TABLES }),
        "DropProcedureStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2040), tables: &ORACLE_TABLES }),
        "DropProfileStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2044), tables: &ORACLE_TABLES }),
        "DropRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2050), tables: &ORACLE_TABLES }),
        "DropSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2055), tables: &ORACLE_TABLES }),
        "DropSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2061), tables: &ORACLE_TABLES }),
        "DropSynonymStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2065), tables: &ORACLE_TABLES }),
        "DropTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2072), tables: &ORACLE_TABLES }),
        "DropTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2080), tables: &ORACLE_TABLES }),
        "DropTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2085), tables: &ORACLE_TABLES }),
        "DropUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2095), tables: &ORACLE_TABLES }),
        "DropViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2101), tables: &ORACLE_TABLES }),
        "DummyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2107), tables: &ORACLE_TABLES }),
        "DumpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2108), tables: &ORACLE_TABLES }),
        "DuplicateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2109), tables: &ORACLE_TABLES }),
        "DynamicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2110), tables: &ORACLE_TABLES }),
        "Dynamic_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2111), tables: &ORACLE_TABLES }),
        "Dynamic_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2112), tables: &ORACLE_TABLES }),
        "EachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2113), tables: &ORACLE_TABLES }),
        "EditionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2114), tables: &ORACLE_TABLES }),
        "EditionableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2115), tables: &ORACLE_TABLES }),
        "EditioningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2116), tables: &ORACLE_TABLES }),
        "EditionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2117), tables: &ORACLE_TABLES }),
        "ElementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2118), tables: &ORACLE_TABLES }),
        "ElementSpecificationGrammar" => Some(RootGrammar { grammar_id: GrammarId(2119), tables: &ORACLE_TABLES }),
        "ElseClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2135), tables: &ORACLE_TABLES }),
        "ElseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2140), tables: &ORACLE_TABLES }),
        "ElseifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2141), tables: &ORACLE_TABLES }),
        "ElsifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2142), tables: &ORACLE_TABLES }),
        "EmptyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2143), tables: &ORACLE_TABLES }),
        "EmptyStructLiteralBracketsSegment" => Some(RootGrammar { grammar_id: GrammarId(2144), tables: &ORACLE_TABLES }),
        "EmptyStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2145), tables: &ORACLE_TABLES }),
        "EnableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2148), tables: &ORACLE_TABLES }),
        "EnclosedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2149), tables: &ORACLE_TABLES }),
        "EncodingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2150), tables: &ORACLE_TABLES }),
        "EncryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2151), tables: &ORACLE_TABLES }),
        "End-execKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2152), tables: &ORACLE_TABLES }),
        "EndBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(223), tables: &ORACLE_TABLES }),
        "EndCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(2153), tables: &ORACLE_TABLES }),
        "EndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2154), tables: &ORACLE_TABLES }),
        "EndSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(732), tables: &ORACLE_TABLES }),
        "EnumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2155), tables: &ORACLE_TABLES }),
        "EqualsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2156), tables: &ORACLE_TABLES }),
        "EqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(2157), tables: &ORACLE_TABLES }),
        "ErrlvlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2158), tables: &ORACLE_TABLES }),
        "ErrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2159), tables: &ORACLE_TABLES }),
        "ErrorsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2160), tables: &ORACLE_TABLES }),
        "EscapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2161), tables: &ORACLE_TABLES }),
        "EscapedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2162), tables: &ORACLE_TABLES }),
        "EvalnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2163), tables: &ORACLE_TABLES }),
        "EveryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2164), tables: &ORACLE_TABLES }),
        "ExceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2165), tables: &ORACLE_TABLES }),
        "ExceptionBlockGrammar" => Some(RootGrammar { grammar_id: GrammarId(2166), tables: &ORACLE_TABLES }),
        "ExceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2171), tables: &ORACLE_TABLES }),
        "ExceptionWhenHandlerGrammar" => Some(RootGrammar { grammar_id: GrammarId(2172), tables: &ORACLE_TABLES }),
        "ExcludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2184), tables: &ORACLE_TABLES }),
        "ExcludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2185), tables: &ORACLE_TABLES }),
        "ExclusiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2186), tables: &ORACLE_TABLES }),
        "ExecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2187), tables: &ORACLE_TABLES }),
        "ExecuteFileSegment" => Some(RootGrammar { grammar_id: GrammarId(2188), tables: &ORACLE_TABLES }),
        "ExecuteImmediateSegment" => Some(RootGrammar { grammar_id: GrammarId(2198), tables: &ORACLE_TABLES }),
        "ExecuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2223), tables: &ORACLE_TABLES }),
        "ExecutionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2224), tables: &ORACLE_TABLES }),
        "ExemptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2225), tables: &ORACLE_TABLES }),
        "ExistingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2226), tables: &ORACLE_TABLES }),
        "ExistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2227), tables: &ORACLE_TABLES }),
        "ExitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2228), tables: &ORACLE_TABLES }),
        "ExitStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2229), tables: &ORACLE_TABLES }),
        "ExpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2235), tables: &ORACLE_TABLES }),
        "ExpireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2236), tables: &ORACLE_TABLES }),
        "ExplainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2237), tables: &ORACLE_TABLES }),
        "ExplainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2238), tables: &ORACLE_TABLES }),
        "ExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2245), tables: &ORACLE_TABLES }),
        "Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2246), tables: &ORACLE_TABLES }),
        "Expression_A_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2272), tables: &ORACLE_TABLES }),
        "Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2279), tables: &ORACLE_TABLES }),
        "Expression_B_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2289), tables: &ORACLE_TABLES }),
        "Expression_C_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2294), tables: &ORACLE_TABLES }),
        "Expression_D_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2307), tables: &ORACLE_TABLES }),
        "Expression_D_Potential_Select_Statement_Without_Brackets" => Some(RootGrammar { grammar_id: GrammarId(2359), tables: &ORACLE_TABLES }),
        "ExtendedNaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2367), tables: &ORACLE_TABLES }),
        "ExtensionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2368), tables: &ORACLE_TABLES }),
        "ExtensionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1764), tables: &ORACLE_TABLES }),
        "ExternalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2369), tables: &ORACLE_TABLES }),
        "ExternallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2370), tables: &ORACLE_TABLES }),
        "ExtractKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2371), tables: &ORACLE_TABLES }),
        "FalseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2372), tables: &ORACLE_TABLES }),
        "FalseSegment" => Some(RootGrammar { grammar_id: GrammarId(2373), tables: &ORACLE_TABLES }),
        "FetchClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2374), tables: &ORACLE_TABLES }),
        "FetchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2391), tables: &ORACLE_TABLES }),
        "FetchStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2392), tables: &ORACLE_TABLES }),
        "FieldsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2406), tables: &ORACLE_TABLES }),
        "FileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2407), tables: &ORACLE_TABLES }),
        "FileSegment" => Some(RootGrammar { grammar_id: GrammarId(2408), tables: &ORACLE_TABLES }),
        "FillfactorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2412), tables: &ORACLE_TABLES }),
        "FilterClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(2413), tables: &ORACLE_TABLES }),
        "FilterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2419), tables: &ORACLE_TABLES }),
        "FinalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2420), tables: &ORACLE_TABLES }),
        "FineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2421), tables: &ORACLE_TABLES }),
        "FirstKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2422), tables: &ORACLE_TABLES }),
        "Flash_cacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2423), tables: &ORACLE_TABLES }),
        "FlashbackKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2424), tables: &ORACLE_TABLES }),
        "Float4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2425), tables: &ORACLE_TABLES }),
        "Float8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2426), tables: &ORACLE_TABLES }),
        "FloatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2427), tables: &ORACLE_TABLES }),
        "FloorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2428), tables: &ORACLE_TABLES }),
        "FlushKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2429), tables: &ORACLE_TABLES }),
        "FollowingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2430), tables: &ORACLE_TABLES }),
        "FollowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2431), tables: &ORACLE_TABLES }),
        "ForAllStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2432), tables: &ORACLE_TABLES }),
        "ForKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2450), tables: &ORACLE_TABLES }),
        "ForLoopStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2451), tables: &ORACLE_TABLES }),
        "ForUpdateGrammar" => Some(RootGrammar { grammar_id: GrammarId(2484), tables: &ORACLE_TABLES }),
        "ForallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2498), tables: &ORACLE_TABLES }),
        "ForceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2499), tables: &ORACLE_TABLES }),
        "ForeignKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(2500), tables: &ORACLE_TABLES }),
        "ForeignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2503), tables: &ORACLE_TABLES }),
        "FormatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2504), tables: &ORACLE_TABLES }),
        "FortranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2505), tables: &ORACLE_TABLES }),
        "ForwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2506), tables: &ORACLE_TABLES }),
        "FoundKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2507), tables: &ORACLE_TABLES }),
        "FrameClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2508), tables: &ORACLE_TABLES }),
        "FrameClauseUnitGrammar" => Some(RootGrammar { grammar_id: GrammarId(2527), tables: &ORACLE_TABLES }),
        "FreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2530), tables: &ORACLE_TABLES }),
        "FreelistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2531), tables: &ORACLE_TABLES }),
        "FreelistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2532), tables: &ORACLE_TABLES }),
        "FreetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2533), tables: &ORACLE_TABLES }),
        "FreetexttableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2534), tables: &ORACLE_TABLES }),
        "FreezeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2535), tables: &ORACLE_TABLES }),
        "FromClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2536), tables: &ORACLE_TABLES }),
        "FromClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2540), tables: &ORACLE_TABLES }),
        "FromExpressionElementSegment" => Some(RootGrammar { grammar_id: GrammarId(2557), tables: &ORACLE_TABLES }),
        "FromExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2580), tables: &ORACLE_TABLES }),
        "FromKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2607), tables: &ORACLE_TABLES }),
        "FullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2608), tables: &ORACLE_TABLES }),
        "FulltextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2609), tables: &ORACLE_TABLES }),
        "FunctionContentsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2610), tables: &ORACLE_TABLES }),
        "FunctionContentsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2614), tables: &ORACLE_TABLES }),
        "FunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(2657), tables: &ORACLE_TABLES }),
        "FunctionDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2660), tables: &ORACLE_TABLES }),
        "FunctionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2666), tables: &ORACLE_TABLES }),
        "FunctionNameIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2667), tables: &ORACLE_TABLES }),
        "FunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2668), tables: &ORACLE_TABLES }),
        "FunctionParameterGrammar" => Some(RootGrammar { grammar_id: GrammarId(2680), tables: &ORACLE_TABLES }),
        "FunctionParameterListGrammar" => Some(RootGrammar { grammar_id: GrammarId(2702), tables: &ORACLE_TABLES }),
        "FunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(2705), tables: &ORACLE_TABLES }),
        "FunctionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2725), tables: &ORACLE_TABLES }),
        "FusionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2726), tables: &ORACLE_TABLES }),
        "FutureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2727), tables: &ORACLE_TABLES }),
        "GKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2728), tables: &ORACLE_TABLES }),
        "GeneralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2729), tables: &ORACLE_TABLES }),
        "GeneratedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2730), tables: &ORACLE_TABLES }),
        "GetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2731), tables: &ORACLE_TABLES }),
        "GlobOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2732), tables: &ORACLE_TABLES }),
        "GlobalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2733), tables: &ORACLE_TABLES }),
        "GloballyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2734), tables: &ORACLE_TABLES }),
        "GoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2735), tables: &ORACLE_TABLES }),
        "GotoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2736), tables: &ORACLE_TABLES }),
        "GrantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2737), tables: &ORACLE_TABLES }),
        "GrantStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2738), tables: &ORACLE_TABLES }),
        "GrantedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2815), tables: &ORACLE_TABLES }),
        "GrantsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2816), tables: &ORACLE_TABLES }),
        "GreaterThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(2817), tables: &ORACLE_TABLES }),
        "GreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2824), tables: &ORACLE_TABLES }),
        "GreatestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2825), tables: &ORACLE_TABLES }),
        "GroupByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2826), tables: &ORACLE_TABLES }),
        "GroupByClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2840), tables: &ORACLE_TABLES }),
        "GroupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2849), tables: &ORACLE_TABLES }),
        "GroupingExpressionList" => Some(RootGrammar { grammar_id: GrammarId(2850), tables: &ORACLE_TABLES }),
        "GroupingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2858), tables: &ORACLE_TABLES }),
        "GroupingSetsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2859), tables: &ORACLE_TABLES }),
        "GroupsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2867), tables: &ORACLE_TABLES }),
        "GuardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2868), tables: &ORACLE_TABLES }),
        "HandlerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2869), tables: &ORACLE_TABLES }),
        "HavingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2870), tables: &ORACLE_TABLES }),
        "HavingClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2875), tables: &ORACLE_TABLES }),
        "HavingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2883), tables: &ORACLE_TABLES }),
        "HeaderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2884), tables: &ORACLE_TABLES }),
        "HeapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2885), tables: &ORACLE_TABLES }),
        "HierarchicalQueryClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2886), tables: &ORACLE_TABLES }),
        "HierarchyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2893), tables: &ORACLE_TABLES }),
        "HighKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2894), tables: &ORACLE_TABLES }),
        "High_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2895), tables: &ORACLE_TABLES }),
        "HoldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2896), tables: &ORACLE_TABLES }),
        "HoldlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2897), tables: &ORACLE_TABLES }),
        "HorizontalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2898), tables: &ORACLE_TABLES }),
        "HostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2899), tables: &ORACLE_TABLES }),
        "HostsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2900), tables: &ORACLE_TABLES }),
        "HourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2901), tables: &ORACLE_TABLES }),
        "Hour_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2902), tables: &ORACLE_TABLES }),
        "Hour_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2903), tables: &ORACLE_TABLES }),
        "Hour_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2904), tables: &ORACLE_TABLES }),
        "HttpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2905), tables: &ORACLE_TABLES }),
        "IdentifiedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2906), tables: &ORACLE_TABLES }),
        "IdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2907), tables: &ORACLE_TABLES }),
        "IdentityClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(2908), tables: &ORACLE_TABLES }),
        "IdentityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2930), tables: &ORACLE_TABLES }),
        "IdentityOptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2931), tables: &ORACLE_TABLES }),
        "Identity_insertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2954), tables: &ORACLE_TABLES }),
        "IdentitycolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2955), tables: &ORACLE_TABLES }),
        "IfClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2956), tables: &ORACLE_TABLES }),
        "IfExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2962), tables: &ORACLE_TABLES }),
        "IfExpressionStatement" => Some(RootGrammar { grammar_id: GrammarId(2965), tables: &ORACLE_TABLES }),
        "IfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2981), tables: &ORACLE_TABLES }),
        "IfNotExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2982), tables: &ORACLE_TABLES }),
        "IgnoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2986), tables: &ORACLE_TABLES }),
        "IgnoreRespectNullsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2987), tables: &ORACLE_TABLES }),
        "IlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2992), tables: &ORACLE_TABLES }),
        "ImmediateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2993), tables: &ORACLE_TABLES }),
        "ImmutableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2994), tables: &ORACLE_TABLES }),
        "ImplementationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2995), tables: &ORACLE_TABLES }),
        "ImplicitCursorAttributesGrammar" => Some(RootGrammar { grammar_id: GrammarId(2996), tables: &ORACLE_TABLES }),
        "ImplicitIndent" => Some(RootGrammar { grammar_id: GrammarId(884), tables: &ORACLE_TABLES }),
        "ImplicitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3006), tables: &ORACLE_TABLES }),
        "ImportedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3007), tables: &ORACLE_TABLES }),
        "InKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3008), tables: &ORACLE_TABLES }),
        "InOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3009), tables: &ORACLE_TABLES }),
        "IncludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3019), tables: &ORACLE_TABLES }),
        "IncludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3020), tables: &ORACLE_TABLES }),
        "IncrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3021), tables: &ORACLE_TABLES }),
        "Indent" => Some(RootGrammar { grammar_id: GrammarId(324), tables: &ORACLE_TABLES }),
        "IndexColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(3022), tables: &ORACLE_TABLES }),
        "IndexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3027), tables: &ORACLE_TABLES }),
        "IndexReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1764), tables: &ORACLE_TABLES }),
        "IndexTypeReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3028), tables: &ORACLE_TABLES }),
        "IndextypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3030), tables: &ORACLE_TABLES }),
        "IndicatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3031), tables: &ORACLE_TABLES }),
        "IndicesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3032), tables: &ORACLE_TABLES }),
        "InfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3033), tables: &ORACLE_TABLES }),
        "InfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3034), tables: &ORACLE_TABLES }),
        "InheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3035), tables: &ORACLE_TABLES }),
        "InheritanyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3036), tables: &ORACLE_TABLES }),
        "InheritsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3037), tables: &ORACLE_TABLES }),
        "InitialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3038), tables: &ORACLE_TABLES }),
        "InitializeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3039), tables: &ORACLE_TABLES }),
        "InitiallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3040), tables: &ORACLE_TABLES }),
        "InitransKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3041), tables: &ORACLE_TABLES }),
        "InmemoryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3042), tables: &ORACLE_TABLES }),
        "InnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3043), tables: &ORACLE_TABLES }),
        "InoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3044), tables: &ORACLE_TABLES }),
        "InputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3045), tables: &ORACLE_TABLES }),
        "InsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3046), tables: &ORACLE_TABLES }),
        "InsertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3047), tables: &ORACLE_TABLES }),
        "InsertStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3048), tables: &ORACLE_TABLES }),
        "Insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3096), tables: &ORACLE_TABLES }),
        "InsertingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3097), tables: &ORACLE_TABLES }),
        "InstanceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3098), tables: &ORACLE_TABLES }),
        "InstantiableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3099), tables: &ORACLE_TABLES }),
        "InsteadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3100), tables: &ORACLE_TABLES }),
        "Int1KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3101), tables: &ORACLE_TABLES }),
        "Int2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3102), tables: &ORACLE_TABLES }),
        "Int3KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3103), tables: &ORACLE_TABLES }),
        "Int4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3104), tables: &ORACLE_TABLES }),
        "Int8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3105), tables: &ORACLE_TABLES }),
        "IntKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3106), tables: &ORACLE_TABLES }),
        "IntegerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3107), tables: &ORACLE_TABLES }),
        "IntegrationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3108), tables: &ORACLE_TABLES }),
        "IntegrationsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3109), tables: &ORACLE_TABLES }),
        "IntersectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3110), tables: &ORACLE_TABLES }),
        "IntersectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3111), tables: &ORACLE_TABLES }),
        "IntervalDataTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(3112), tables: &ORACLE_TABLES }),
        "IntervalExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3129), tables: &ORACLE_TABLES }),
        "IntervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3143), tables: &ORACLE_TABLES }),
        "IntervalUnitsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3144), tables: &ORACLE_TABLES }),
        "IntoClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3151), tables: &ORACLE_TABLES }),
        "IntoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3158), tables: &ORACLE_TABLES }),
        "InvisibleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3159), tables: &ORACLE_TABLES }),
        "InvokerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3160), tables: &ORACLE_TABLES }),
        "InvokerRightsClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(3161), tables: &ORACLE_TABLES }),
        "IsClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(3166), tables: &ORACLE_TABLES }),
        "IsDistinctFromGrammar" => Some(RootGrammar { grammar_id: GrammarId(3182), tables: &ORACLE_TABLES }),
        "IsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3187), tables: &ORACLE_TABLES }),
        "IsNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(3188), tables: &ORACLE_TABLES }),
        "IsamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3189), tables: &ORACLE_TABLES }),
        "IsnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3190), tables: &ORACLE_TABLES }),
        "IsolationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3191), tables: &ORACLE_TABLES }),
        "Isolation_levelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3192), tables: &ORACLE_TABLES }),
        "IsopenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3193), tables: &ORACLE_TABLES }),
        "IterateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3194), tables: &ORACLE_TABLES }),
        "IterationBoundsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3195), tables: &ORACLE_TABLES }),
        "IterationSteppedControlGrammar" => Some(RootGrammar { grammar_id: GrammarId(3203), tables: &ORACLE_TABLES }),
        "JSONEntrySegment" => Some(RootGrammar { grammar_id: GrammarId(3211), tables: &ORACLE_TABLES }),
        "JSONObjectContentSegment" => Some(RootGrammar { grammar_id: GrammarId(3218), tables: &ORACLE_TABLES }),
        "JSONOnNullClause" => Some(RootGrammar { grammar_id: GrammarId(3230), tables: &ORACLE_TABLES }),
        "JSONRegularEntrySegment" => Some(RootGrammar { grammar_id: GrammarId(3236), tables: &ORACLE_TABLES }),
        "JSONReturningClause" => Some(RootGrammar { grammar_id: GrammarId(3249), tables: &ORACLE_TABLES }),
        "JavaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3269), tables: &ORACLE_TABLES }),
        "JobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3270), tables: &ORACLE_TABLES }),
        "JoinClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3271), tables: &ORACLE_TABLES }),
        "JoinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3293), tables: &ORACLE_TABLES }),
        "JoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3294), tables: &ORACLE_TABLES }),
        "JoinLikeClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(3296), tables: &ORACLE_TABLES }),
        "JoinOnConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(3301), tables: &ORACLE_TABLES }),
        "JoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3308), tables: &ORACLE_TABLES }),
        "JoinUsingConditionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3316), tables: &ORACLE_TABLES }),
        "JsonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3321), tables: &ORACLE_TABLES }),
        "JsonTableColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(3322), tables: &ORACLE_TABLES }),
        "JsonTableColumnsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3353), tables: &ORACLE_TABLES }),
        "JsonTableFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(3358), tables: &ORACLE_TABLES }),
        "JsonTableFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3370), tables: &ORACLE_TABLES }),
        "KKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3371), tables: &ORACLE_TABLES }),
        "KeepClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3372), tables: &ORACLE_TABLES }),
        "KeepKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3381), tables: &ORACLE_TABLES }),
        "KeyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3382), tables: &ORACLE_TABLES }),
        "Key_memberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3383), tables: &ORACLE_TABLES }),
        "Key_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3384), tables: &ORACLE_TABLES }),
        "KeysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3385), tables: &ORACLE_TABLES }),
        "KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3386), tables: &ORACLE_TABLES }),
        "KillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3387), tables: &ORACLE_TABLES }),
        "LancompilerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3388), tables: &ORACLE_TABLES }),
        "LanguageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3389), tables: &ORACLE_TABLES }),
        "LargeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3390), tables: &ORACLE_TABLES }),
        "LastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3391), tables: &ORACLE_TABLES }),
        "Last_insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3392), tables: &ORACLE_TABLES }),
        "LateralColumnAliasExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3393), tables: &ORACLE_TABLES }),
        "LateralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3394), tables: &ORACLE_TABLES }),
        "LeadingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3395), tables: &ORACLE_TABLES }),
        "LeastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3396), tables: &ORACLE_TABLES }),
        "LeaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3397), tables: &ORACLE_TABLES }),
        "LeftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3398), tables: &ORACLE_TABLES }),
        "LengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3399), tables: &ORACLE_TABLES }),
        "LessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3400), tables: &ORACLE_TABLES }),
        "LessThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(3401), tables: &ORACLE_TABLES }),
        "LessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(3408), tables: &ORACLE_TABLES }),
        "LevelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3409), tables: &ORACLE_TABLES }),
        "LibraryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3410), tables: &ORACLE_TABLES }),
        "LikeExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3411), tables: &ORACLE_TABLES }),
        "LikeGrammar" => Some(RootGrammar { grammar_id: GrammarId(3419), tables: &ORACLE_TABLES }),
        "LikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3423), tables: &ORACLE_TABLES }),
        "LikeOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(3424), tables: &ORACLE_TABLES }),
        "LimitClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3425), tables: &ORACLE_TABLES }),
        "LimitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3442), tables: &ORACLE_TABLES }),
        "LinenoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3443), tables: &ORACLE_TABLES }),
        "LinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3444), tables: &ORACLE_TABLES }),
        "LinkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3445), tables: &ORACLE_TABLES }),
        "ListComprehensionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3446), tables: &ORACLE_TABLES }),
        "ListaggOverflowClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3447), tables: &ORACLE_TABLES }),
        "ListenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3459), tables: &ORACLE_TABLES }),
        "LiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(3460), tables: &ORACLE_TABLES }),
        "LiteralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3475), tables: &ORACLE_TABLES }),
        "LiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3476), tables: &ORACLE_TABLES }),
        "LnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3477), tables: &ORACLE_TABLES }),
        "LoadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3478), tables: &ORACLE_TABLES }),
        "LocalAliasSegment" => Some(RootGrammar { grammar_id: GrammarId(3479), tables: &ORACLE_TABLES }),
        "LocalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3480), tables: &ORACLE_TABLES }),
        "LocaltimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3481), tables: &ORACLE_TABLES }),
        "LocaltimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3482), tables: &ORACLE_TABLES }),
        "LocationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3483), tables: &ORACLE_TABLES }),
        "LocatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3484), tables: &ORACLE_TABLES }),
        "LockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3485), tables: &ORACLE_TABLES }),
        "LockdownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3486), tables: &ORACLE_TABLES }),
        "LockedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3487), tables: &ORACLE_TABLES }),
        "LockingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3488), tables: &ORACLE_TABLES }),
        "LocksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3489), tables: &ORACLE_TABLES }),
        "LogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3490), tables: &ORACLE_TABLES }),
        "LoggingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3491), tables: &ORACLE_TABLES }),
        "LoginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3492), tables: &ORACLE_TABLES }),
        "LogminingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3493), tables: &ORACLE_TABLES }),
        "LogsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3494), tables: &ORACLE_TABLES }),
        "LongKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3495), tables: &ORACLE_TABLES }),
        "LongblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3496), tables: &ORACLE_TABLES }),
        "LongtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3497), tables: &ORACLE_TABLES }),
        "LoopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3498), tables: &ORACLE_TABLES }),
        "LoopStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3499), tables: &ORACLE_TABLES }),
        "LowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3507), tables: &ORACLE_TABLES }),
        "Low_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3508), tables: &ORACLE_TABLES }),
        "LowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3509), tables: &ORACLE_TABLES }),
        "MKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3510), tables: &ORACLE_TABLES }),
        "MLTableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3511), tables: &ORACLE_TABLES }),
        "ManageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3523), tables: &ORACLE_TABLES }),
        "MapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3524), tables: &ORACLE_TABLES }),
        "MapTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(3525), tables: &ORACLE_TABLES }),
        "MaskingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3526), tables: &ORACLE_TABLES }),
        "MatchConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(3527), tables: &ORACLE_TABLES }),
        "MatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3528), tables: &ORACLE_TABLES }),
        "MatchedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3529), tables: &ORACLE_TABLES }),
        "MaterializedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3530), tables: &ORACLE_TABLES }),
        "MaxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3531), tables: &ORACLE_TABLES }),
        "Max_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3532), tables: &ORACLE_TABLES }),
        "MaxextentsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3533), tables: &ORACLE_TABLES }),
        "MaxsizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3534), tables: &ORACLE_TABLES }),
        "MaxtransKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3535), tables: &ORACLE_TABLES }),
        "MaxvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3536), tables: &ORACLE_TABLES }),
        "MeasureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3537), tables: &ORACLE_TABLES }),
        "MediumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3538), tables: &ORACLE_TABLES }),
        "MediumblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3539), tables: &ORACLE_TABLES }),
        "MediumintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3540), tables: &ORACLE_TABLES }),
        "MediumtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3541), tables: &ORACLE_TABLES }),
        "MemberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3542), tables: &ORACLE_TABLES }),
        "MemcompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3543), tables: &ORACLE_TABLES }),
        "MergeDeleteClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3544), tables: &ORACLE_TABLES }),
        "MergeInsertClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3545), tables: &ORACLE_TABLES }),
        "MergeIntoLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(3549), tables: &ORACLE_TABLES }),
        "MergeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3552), tables: &ORACLE_TABLES }),
        "MergeMatchSegment" => Some(RootGrammar { grammar_id: GrammarId(3553), tables: &ORACLE_TABLES }),
        "MergeMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3556), tables: &ORACLE_TABLES }),
        "MergeNotMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3566), tables: &ORACLE_TABLES }),
        "MergeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3575), tables: &ORACLE_TABLES }),
        "MergeUpdateClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3595), tables: &ORACLE_TABLES }),
        "Message_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3600), tables: &ORACLE_TABLES }),
        "Message_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3601), tables: &ORACLE_TABLES }),
        "Message_textKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3602), tables: &ORACLE_TABLES }),
        "MethodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3603), tables: &ORACLE_TABLES }),
        "MiddleintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3604), tables: &ORACLE_TABLES }),
        "MillisecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3605), tables: &ORACLE_TABLES }),
        "MinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3606), tables: &ORACLE_TABLES }),
        "Min_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3607), tables: &ORACLE_TABLES }),
        "MinextentsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3608), tables: &ORACLE_TABLES }),
        "MiningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3609), tables: &ORACLE_TABLES }),
        "MinusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3610), tables: &ORACLE_TABLES }),
        "MinusSegment" => Some(RootGrammar { grammar_id: GrammarId(3611), tables: &ORACLE_TABLES }),
        "MinuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3612), tables: &ORACLE_TABLES }),
        "Minute_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3613), tables: &ORACLE_TABLES }),
        "Minute_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3614), tables: &ORACLE_TABLES }),
        "MinvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3615), tables: &ORACLE_TABLES }),
        "MlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3616), tables: &ORACLE_TABLES }),
        "MlslabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3617), tables: &ORACLE_TABLES }),
        "ModKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3618), tables: &ORACLE_TABLES }),
        "ModOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(3619), tables: &ORACLE_TABLES }),
        "ModeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3620), tables: &ORACLE_TABLES }),
        "ModelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3621), tables: &ORACLE_TABLES }),
        "ModifiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3622), tables: &ORACLE_TABLES }),
        "ModifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3623), tables: &ORACLE_TABLES }),
        "ModuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3624), tables: &ORACLE_TABLES }),
        "ModuloSegment" => Some(RootGrammar { grammar_id: GrammarId(3625), tables: &ORACLE_TABLES }),
        "MonitorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3626), tables: &ORACLE_TABLES }),
        "MonitoringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3627), tables: &ORACLE_TABLES }),
        "MonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3628), tables: &ORACLE_TABLES }),
        "MonthnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3629), tables: &ORACLE_TABLES }),
        "MoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3630), tables: &ORACLE_TABLES }),
        "MoveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3631), tables: &ORACLE_TABLES }),
        "MovementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3632), tables: &ORACLE_TABLES }),
        "MultiplySegment" => Some(RootGrammar { grammar_id: GrammarId(3633), tables: &ORACLE_TABLES }),
        "MultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3634), tables: &ORACLE_TABLES }),
        "MultisetOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(3635), tables: &ORACLE_TABLES }),
        "MumpsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3644), tables: &ORACLE_TABLES }),
        "MutableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3645), tables: &ORACLE_TABLES }),
        "MyisamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3646), tables: &ORACLE_TABLES }),
        "NakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3647), tables: &ORACLE_TABLES }),
        "NameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3648), tables: &ORACLE_TABLES }),
        "NamedArgumentSegment" => Some(RootGrammar { grammar_id: GrammarId(3649), tables: &ORACLE_TABLES }),
        "NamedWindowExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3653), tables: &ORACLE_TABLES }),
        "NamedWindowSegment" => Some(RootGrammar { grammar_id: GrammarId(3660), tables: &ORACLE_TABLES }),
        "NamesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3664), tables: &ORACLE_TABLES }),
        "NanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3665), tables: &ORACLE_TABLES }),
        "NanLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3666), tables: &ORACLE_TABLES }),
        "NationalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3667), tables: &ORACLE_TABLES }),
        "NaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3668), tables: &ORACLE_TABLES }),
        "NaturalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3671), tables: &ORACLE_TABLES }),
        "NcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3672), tables: &ORACLE_TABLES }),
        "NclobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3673), tables: &ORACLE_TABLES }),
        "NegativeSegment" => Some(RootGrammar { grammar_id: GrammarId(3674), tables: &ORACLE_TABLES }),
        "NestedJoinGrammar" => Some(RootGrammar { grammar_id: GrammarId(3675), tables: &ORACLE_TABLES }),
        "NestedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3676), tables: &ORACLE_TABLES }),
        "Nested_table_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3677), tables: &ORACLE_TABLES }),
        "NestingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3678), tables: &ORACLE_TABLES }),
        "NewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3679), tables: &ORACLE_TABLES }),
        "NewlineSegment" => Some(RootGrammar { grammar_id: GrammarId(3680), tables: &ORACLE_TABLES }),
        "NextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3681), tables: &ORACLE_TABLES }),
        "NextvalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3682), tables: &ORACLE_TABLES }),
        "NoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3683), tables: &ORACLE_TABLES }),
        "No_write_to_binlogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3684), tables: &ORACLE_TABLES }),
        "NoauditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3685), tables: &ORACLE_TABLES }),
        "NocacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3686), tables: &ORACLE_TABLES }),
        "NocheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3687), tables: &ORACLE_TABLES }),
        "NocompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3688), tables: &ORACLE_TABLES }),
        "NocopyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3689), tables: &ORACLE_TABLES }),
        "NocreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3690), tables: &ORACLE_TABLES }),
        "NocreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3691), tables: &ORACLE_TABLES }),
        "NocreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3692), tables: &ORACLE_TABLES }),
        "NocycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3693), tables: &ORACLE_TABLES }),
        "NoinheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3694), tables: &ORACLE_TABLES }),
        "NologgingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3695), tables: &ORACLE_TABLES }),
        "NologinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3696), tables: &ORACLE_TABLES }),
        "NomaxvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3697), tables: &ORACLE_TABLES }),
        "NominvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3698), tables: &ORACLE_TABLES }),
        "NomonitoringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3699), tables: &ORACLE_TABLES }),
        "NonSetSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(3700), tables: &ORACLE_TABLES }),
        "NonStandardJoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3710), tables: &ORACLE_TABLES }),
        "NonWithNonSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(3711), tables: &ORACLE_TABLES }),
        "NonWithSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(3716), tables: &ORACLE_TABLES }),
        "NonclusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3722), tables: &ORACLE_TABLES }),
        "NoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3723), tables: &ORACLE_TABLES }),
        "NoneditionableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3724), tables: &ORACLE_TABLES }),
        "NoorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3725), tables: &ORACLE_TABLES }),
        "NoparallelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3726), tables: &ORACLE_TABLES }),
        "NoreverseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3727), tables: &ORACLE_TABLES }),
        "NormalizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3728), tables: &ORACLE_TABLES }),
        "NormalizedGrammar" => Some(RootGrammar { grammar_id: GrammarId(3729), tables: &ORACLE_TABLES }),
        "NormalizedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3730), tables: &ORACLE_TABLES }),
        "NorowdependenciesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3731), tables: &ORACLE_TABLES }),
        "NosortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3732), tables: &ORACLE_TABLES }),
        "NosuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3733), tables: &ORACLE_TABLES }),
        "NotEnforcedGrammar" => Some(RootGrammar { grammar_id: GrammarId(3734), tables: &ORACLE_TABLES }),
        "NotEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(3735), tables: &ORACLE_TABLES }),
        "NotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3742), tables: &ORACLE_TABLES }),
        "NotNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(3743), tables: &ORACLE_TABLES }),
        "NotOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3744), tables: &ORACLE_TABLES }),
        "NotfoundKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3745), tables: &ORACLE_TABLES }),
        "NothingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3746), tables: &ORACLE_TABLES }),
        "NotifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3747), tables: &ORACLE_TABLES }),
        "NotnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3748), tables: &ORACLE_TABLES }),
        "NowaitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3749), tables: &ORACLE_TABLES }),
        "NullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3750), tables: &ORACLE_TABLES }),
        "NullLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3751), tables: &ORACLE_TABLES }),
        "NullStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3752), tables: &ORACLE_TABLES }),
        "NullableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3754), tables: &ORACLE_TABLES }),
        "NullifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3755), tables: &ORACLE_TABLES }),
        "NullsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3756), tables: &ORACLE_TABLES }),
        "NumberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3757), tables: &ORACLE_TABLES }),
        "NumericKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3758), tables: &ORACLE_TABLES }),
        "NumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3759), tables: &ORACLE_TABLES }),
        "ObjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3760), tables: &ORACLE_TABLES }),
        "ObjectLiteralElementSegment" => Some(RootGrammar { grammar_id: GrammarId(3761), tables: &ORACLE_TABLES }),
        "ObjectLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3765), tables: &ORACLE_TABLES }),
        "ObjectReferenceDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(3769), tables: &ORACLE_TABLES }),
        "ObjectReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1086), tables: &ORACLE_TABLES }),
        "ObjectReferenceTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3774), tables: &ORACLE_TABLES }),
        "ObjectTypeAndSubtypeDefGrammar" => Some(RootGrammar { grammar_id: GrammarId(3786), tables: &ORACLE_TABLES }),
        "ObjectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3806), tables: &ORACLE_TABLES }),
        "Octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3807), tables: &ORACLE_TABLES }),
        "OctetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3808), tables: &ORACLE_TABLES }),
        "OfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3809), tables: &ORACLE_TABLES }),
        "OffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3810), tables: &ORACLE_TABLES }),
        "OfflineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3811), tables: &ORACLE_TABLES }),
        "OffsetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3812), tables: &ORACLE_TABLES }),
        "OffsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3821), tables: &ORACLE_TABLES }),
        "OffsetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3822), tables: &ORACLE_TABLES }),
        "OidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3823), tables: &ORACLE_TABLES }),
        "OidsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3824), tables: &ORACLE_TABLES }),
        "OldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3825), tables: &ORACLE_TABLES }),
        "OltpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3826), tables: &ORACLE_TABLES }),
        "OnCommitGrammar" => Some(RootGrammar { grammar_id: GrammarId(3827), tables: &ORACLE_TABLES }),
        "OnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3841), tables: &ORACLE_TABLES }),
        "OneOrMoreStatementsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3842), tables: &ORACLE_TABLES }),
        "OnlineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3844), tables: &ORACLE_TABLES }),
        "OnlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3845), tables: &ORACLE_TABLES }),
        "OpenForStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3846), tables: &ORACLE_TABLES }),
        "OpenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3869), tables: &ORACLE_TABLES }),
        "OpenStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3870), tables: &ORACLE_TABLES }),
        "OpendatasourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3874), tables: &ORACLE_TABLES }),
        "OpenqueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3875), tables: &ORACLE_TABLES }),
        "OpenrowsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3876), tables: &ORACLE_TABLES }),
        "OpenxmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3877), tables: &ORACLE_TABLES }),
        "OperateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3878), tables: &ORACLE_TABLES }),
        "OperationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3879), tables: &ORACLE_TABLES }),
        "OperatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3880), tables: &ORACLE_TABLES }),
        "OptimalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3881), tables: &ORACLE_TABLES }),
        "OptimizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3882), tables: &ORACLE_TABLES }),
        "OptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3883), tables: &ORACLE_TABLES }),
        "OptionallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3884), tables: &ORACLE_TABLES }),
        "OptionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3885), tables: &ORACLE_TABLES }),
        "OrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3886), tables: &ORACLE_TABLES }),
        "OrOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3887), tables: &ORACLE_TABLES }),
        "OrReplaceGrammar" => Some(RootGrammar { grammar_id: GrammarId(3888), tables: &ORACLE_TABLES }),
        "Ora_rowscnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3891), tables: &ORACLE_TABLES }),
        "OracleIndexPhysicalAttributesSegment" => Some(RootGrammar { grammar_id: GrammarId(3892), tables: &ORACLE_TABLES }),
        "OraclePhysicalAttributesSegment" => Some(RootGrammar { grammar_id: GrammarId(3926), tables: &ORACLE_TABLES }),
        "OrderByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4102), tables: &ORACLE_TABLES }),
        "OrderByClauseTerminators" => Some(RootGrammar { grammar_id: GrammarId(4123), tables: &ORACLE_TABLES }),
        "OrderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4131), tables: &ORACLE_TABLES }),
        "OrderNoOrderGrammar" => Some(RootGrammar { grammar_id: GrammarId(4132), tables: &ORACLE_TABLES }),
        "OrderingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4135), tables: &ORACLE_TABLES }),
        "OrdinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4136), tables: &ORACLE_TABLES }),
        "OthersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4137), tables: &ORACLE_TABLES }),
        "OutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4138), tables: &ORACLE_TABLES }),
        "OuterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4139), tables: &ORACLE_TABLES }),
        "OutfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4140), tables: &ORACLE_TABLES }),
        "OutlineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4141), tables: &ORACLE_TABLES }),
        "OutputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4142), tables: &ORACLE_TABLES }),
        "OverClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4143), tables: &ORACLE_TABLES }),
        "OverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4150), tables: &ORACLE_TABLES }),
        "OverflowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4151), tables: &ORACLE_TABLES }),
        "OverlapsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4152), tables: &ORACLE_TABLES }),
        "OverlapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4161), tables: &ORACLE_TABLES }),
        "OverlayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4162), tables: &ORACLE_TABLES }),
        "OverridingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4163), tables: &ORACLE_TABLES }),
        "OverwriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4164), tables: &ORACLE_TABLES }),
        "OwnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4165), tables: &ORACLE_TABLES }),
        "OwnershipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4166), tables: &ORACLE_TABLES }),
        "Pack_keysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4167), tables: &ORACLE_TABLES }),
        "PackageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4168), tables: &ORACLE_TABLES }),
        "PackageReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1086), tables: &ORACLE_TABLES }),
        "PadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4169), tables: &ORACLE_TABLES }),
        "PairsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4170), tables: &ORACLE_TABLES }),
        "ParallelEnableClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(4171), tables: &ORACLE_TABLES }),
        "ParallelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4200), tables: &ORACLE_TABLES }),
        "Parallel_enableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4201), tables: &ORACLE_TABLES }),
        "ParameterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4202), tables: &ORACLE_TABLES }),
        "ParameterNameSegment" => Some(RootGrammar { grammar_id: GrammarId(4203), tables: &ORACLE_TABLES }),
        "ParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(4204), tables: &ORACLE_TABLES }),
        "Parameter_modeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4205), tables: &ORACLE_TABLES }),
        "Parameter_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4206), tables: &ORACLE_TABLES }),
        "Parameter_ordinal_positionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4207), tables: &ORACLE_TABLES }),
        "Parameter_specific_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4208), tables: &ORACLE_TABLES }),
        "Parameter_specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4209), tables: &ORACLE_TABLES }),
        "Parameter_specific_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4210), tables: &ORACLE_TABLES }),
        "ParametersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4211), tables: &ORACLE_TABLES }),
        "ParentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4212), tables: &ORACLE_TABLES }),
        "PartialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4213), tables: &ORACLE_TABLES }),
        "PartitionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4214), tables: &ORACLE_TABLES }),
        "PartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4221), tables: &ORACLE_TABLES }),
        "PascalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4222), tables: &ORACLE_TABLES }),
        "PasswordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4223), tables: &ORACLE_TABLES }),
        "PathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4224), tables: &ORACLE_TABLES }),
        "PathSegment" => Some(RootGrammar { grammar_id: GrammarId(4225), tables: &ORACLE_TABLES }),
        "PatternMatchingGrammar" => Some(RootGrammar { grammar_id: GrammarId(4232), tables: &ORACLE_TABLES }),
        "PctfreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4233), tables: &ORACLE_TABLES }),
        "PctincreaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4234), tables: &ORACLE_TABLES }),
        "PctusedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4235), tables: &ORACLE_TABLES }),
        "PercentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4236), tables: &ORACLE_TABLES }),
        "Percent_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4237), tables: &ORACLE_TABLES }),
        "Percentile_contKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4238), tables: &ORACLE_TABLES }),
        "Percentile_discKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4239), tables: &ORACLE_TABLES }),
        "PersistableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4240), tables: &ORACLE_TABLES }),
        "PipeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4241), tables: &ORACLE_TABLES }),
        "PipeSegment" => Some(RootGrammar { grammar_id: GrammarId(4242), tables: &ORACLE_TABLES }),
        "PipelinedClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(4243), tables: &ORACLE_TABLES }),
        "PipelinedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4257), tables: &ORACLE_TABLES }),
        "PivotForGrammar" => Some(RootGrammar { grammar_id: GrammarId(4258), tables: &ORACLE_TABLES }),
        "PivotInGrammar" => Some(RootGrammar { grammar_id: GrammarId(4264), tables: &ORACLE_TABLES }),
        "PivotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4271), tables: &ORACLE_TABLES }),
        "PivotSegment" => Some(RootGrammar { grammar_id: GrammarId(4272), tables: &ORACLE_TABLES }),
        "PlacingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4282), tables: &ORACLE_TABLES }),
        "PlanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4283), tables: &ORACLE_TABLES }),
        "PliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4284), tables: &ORACLE_TABLES }),
        "PlsqlStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4285), tables: &ORACLE_TABLES }),
        "PluggableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4369), tables: &ORACLE_TABLES }),
        "PlusJoinGrammar" => Some(RootGrammar { grammar_id: GrammarId(4370), tables: &ORACLE_TABLES }),
        "PlusJoinSegment" => Some(RootGrammar { grammar_id: GrammarId(4387), tables: &ORACLE_TABLES }),
        "PlusSegment" => Some(RootGrammar { grammar_id: GrammarId(4391), tables: &ORACLE_TABLES }),
        "PolicyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4392), tables: &ORACLE_TABLES }),
        "PolymorphicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4393), tables: &ORACLE_TABLES }),
        "PositionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4394), tables: &ORACLE_TABLES }),
        "PositiveSegment" => Some(RootGrammar { grammar_id: GrammarId(4395), tables: &ORACLE_TABLES }),
        "PostFunctionGrammar" => Some(RootGrammar { grammar_id: GrammarId(4396), tables: &ORACLE_TABLES }),
        "PostTableExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(4401), tables: &ORACLE_TABLES }),
        "PostfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4402), tables: &ORACLE_TABLES }),
        "PowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4403), tables: &ORACLE_TABLES }),
        "PowerOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(4404), tables: &ORACLE_TABLES }),
        "PragmaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4405), tables: &ORACLE_TABLES }),
        "PreTableFunctionKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(4406), tables: &ORACLE_TABLES }),
        "PrecedesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4408), tables: &ORACLE_TABLES }),
        "PrecedingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4409), tables: &ORACLE_TABLES }),
        "PrecisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4410), tables: &ORACLE_TABLES }),
        "PrefixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4411), tables: &ORACLE_TABLES }),
        "PreorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4412), tables: &ORACLE_TABLES }),
        "PrepareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4413), tables: &ORACLE_TABLES }),
        "PreparedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4414), tables: &ORACLE_TABLES }),
        "PreserveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4415), tables: &ORACLE_TABLES }),
        "PrimaryKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(4416), tables: &ORACLE_TABLES }),
        "PrimaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4419), tables: &ORACLE_TABLES }),
        "PrintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4420), tables: &ORACLE_TABLES }),
        "PriorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4421), tables: &ORACLE_TABLES }),
        "PriorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4422), tables: &ORACLE_TABLES }),
        "PrivateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4423), tables: &ORACLE_TABLES }),
        "PrivilegeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4424), tables: &ORACLE_TABLES }),
        "PrivilegesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4425), tables: &ORACLE_TABLES }),
        "ProcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4426), tables: &ORACLE_TABLES }),
        "ProceduralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4427), tables: &ORACLE_TABLES }),
        "ProcedureCallStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4428), tables: &ORACLE_TABLES }),
        "ProcedureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4438), tables: &ORACLE_TABLES }),
        "ProceduresKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4439), tables: &ORACLE_TABLES }),
        "ProcessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4440), tables: &ORACLE_TABLES }),
        "ProcesslistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4441), tables: &ORACLE_TABLES }),
        "ProfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4442), tables: &ORACLE_TABLES }),
        "ProgramKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4443), tables: &ORACLE_TABLES }),
        "PromptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4444), tables: &ORACLE_TABLES }),
        "PropertyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4445), tables: &ORACLE_TABLES }),
        "PublicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4446), tables: &ORACLE_TABLES }),
        "PurgeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4447), tables: &ORACLE_TABLES }),
        "QualifiedNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4448), tables: &ORACLE_TABLES }),
        "QualifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4451), tables: &ORACLE_TABLES }),
        "QuantifiedComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4452), tables: &ORACLE_TABLES }),
        "QuarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4460), tables: &ORACLE_TABLES }),
        "QueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4461), tables: &ORACLE_TABLES }),
        "QuotaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4462), tables: &ORACLE_TABLES }),
        "QuoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4463), tables: &ORACLE_TABLES }),
        "QuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(4464), tables: &ORACLE_TABLES }),
        "QuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4465), tables: &ORACLE_TABLES }),
        "Raid0KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4466), tables: &ORACLE_TABLES }),
        "RaiseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4467), tables: &ORACLE_TABLES }),
        "RaiseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4468), tables: &ORACLE_TABLES }),
        "RaiserrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4471), tables: &ORACLE_TABLES }),
        "RangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4472), tables: &ORACLE_TABLES }),
        "RankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4473), tables: &ORACLE_TABLES }),
        "RawEqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(4474), tables: &ORACLE_TABLES }),
        "RawGreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(4475), tables: &ORACLE_TABLES }),
        "RawKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4476), tables: &ORACLE_TABLES }),
        "RawLessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(4477), tables: &ORACLE_TABLES }),
        "RawNotSegment" => Some(RootGrammar { grammar_id: GrammarId(4478), tables: &ORACLE_TABLES }),
        "ReadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4479), tables: &ORACLE_TABLES }),
        "ReadsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4480), tables: &ORACLE_TABLES }),
        "ReadtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4481), tables: &ORACLE_TABLES }),
        "RealKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4482), tables: &ORACLE_TABLES }),
        "RebuildKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4483), tables: &ORACLE_TABLES }),
        "RecheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4484), tables: &ORACLE_TABLES }),
        "ReconfigureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4485), tables: &ORACLE_TABLES }),
        "RecordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4486), tables: &ORACLE_TABLES }),
        "RecordTypeDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(4487), tables: &ORACLE_TABLES }),
        "RecursiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4507), tables: &ORACLE_TABLES }),
        "RecycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4508), tables: &ORACLE_TABLES }),
        "RedactionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4509), tables: &ORACLE_TABLES }),
        "RedefineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4510), tables: &ORACLE_TABLES }),
        "RefCursorTypeDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(4511), tables: &ORACLE_TABLES }),
        "RefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4523), tables: &ORACLE_TABLES }),
        "ReferenceDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(4524), tables: &ORACLE_TABLES }),
        "ReferenceMatchGrammar" => Some(RootGrammar { grammar_id: GrammarId(4538), tables: &ORACLE_TABLES }),
        "Reference_usageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4544), tables: &ORACLE_TABLES }),
        "ReferencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4545), tables: &ORACLE_TABLES }),
        "ReferencingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4546), tables: &ORACLE_TABLES }),
        "ReferencingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4553), tables: &ORACLE_TABLES }),
        "ReferentialActionGrammar" => Some(RootGrammar { grammar_id: GrammarId(4554), tables: &ORACLE_TABLES }),
        "RefreshKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4566), tables: &ORACLE_TABLES }),
        "RegexpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4567), tables: &ORACLE_TABLES }),
        "Regr_avgxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4568), tables: &ORACLE_TABLES }),
        "Regr_avgyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4569), tables: &ORACLE_TABLES }),
        "Regr_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4570), tables: &ORACLE_TABLES }),
        "Regr_interceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4571), tables: &ORACLE_TABLES }),
        "Regr_r2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4572), tables: &ORACLE_TABLES }),
        "Regr_slopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4573), tables: &ORACLE_TABLES }),
        "Regr_sxxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4574), tables: &ORACLE_TABLES }),
        "Regr_sxyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4575), tables: &ORACLE_TABLES }),
        "Regr_syyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4576), tables: &ORACLE_TABLES }),
        "ReindexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4577), tables: &ORACLE_TABLES }),
        "RejectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4578), tables: &ORACLE_TABLES }),
        "RelativeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4579), tables: &ORACLE_TABLES }),
        "ReleaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4580), tables: &ORACLE_TABLES }),
        "Relies_onKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4581), tables: &ORACLE_TABLES }),
        "ReloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4582), tables: &ORACLE_TABLES }),
        "RemoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4583), tables: &ORACLE_TABLES }),
        "RenameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4584), tables: &ORACLE_TABLES }),
        "RepeatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4585), tables: &ORACLE_TABLES }),
        "RepeatableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4586), tables: &ORACLE_TABLES }),
        "ReplaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4587), tables: &ORACLE_TABLES }),
        "ReplicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4588), tables: &ORACLE_TABLES }),
        "RequireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4589), tables: &ORACLE_TABLES }),
        "ResetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4590), tables: &ORACLE_TABLES }),
        "ResignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4591), tables: &ORACLE_TABLES }),
        "ResourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4592), tables: &ORACLE_TABLES }),
        "RespectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4593), tables: &ORACLE_TABLES }),
        "RestartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4594), tables: &ORACLE_TABLES }),
        "RestoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4595), tables: &ORACLE_TABLES }),
        "RestrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4596), tables: &ORACLE_TABLES }),
        "RestrictedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4597), tables: &ORACLE_TABLES }),
        "ResultCacheClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(4598), tables: &ORACLE_TABLES }),
        "ResultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4605), tables: &ORACLE_TABLES }),
        "Result_cacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4606), tables: &ORACLE_TABLES }),
        "ResumableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4607), tables: &ORACLE_TABLES }),
        "ReturnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4608), tables: &ORACLE_TABLES }),
        "ReturnStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4609), tables: &ORACLE_TABLES }),
        "Returned_cardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4612), tables: &ORACLE_TABLES }),
        "Returned_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4613), tables: &ORACLE_TABLES }),
        "Returned_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4614), tables: &ORACLE_TABLES }),
        "Returned_sqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4615), tables: &ORACLE_TABLES }),
        "ReturningClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4616), tables: &ORACLE_TABLES }),
        "ReturningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4631), tables: &ORACLE_TABLES }),
        "ReturnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4632), tables: &ORACLE_TABLES }),
        "ReuseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4633), tables: &ORACLE_TABLES }),
        "ReverseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4634), tables: &ORACLE_TABLES }),
        "RevokeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4635), tables: &ORACLE_TABLES }),
        "RevokeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4636), tables: &ORACLE_TABLES }),
        "RewriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4700), tables: &ORACLE_TABLES }),
        "RightArrowSegment" => Some(RootGrammar { grammar_id: GrammarId(4701), tables: &ORACLE_TABLES }),
        "RightKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4702), tables: &ORACLE_TABLES }),
        "RlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4703), tables: &ORACLE_TABLES }),
        "RoleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4704), tables: &ORACLE_TABLES }),
        "RoleReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4705), tables: &ORACLE_TABLES }),
        "RolesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4706), tables: &ORACLE_TABLES }),
        "RollbackKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4707), tables: &ORACLE_TABLES }),
        "RollupFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(4708), tables: &ORACLE_TABLES }),
        "RollupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4709), tables: &ORACLE_TABLES }),
        "RoutineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4710), tables: &ORACLE_TABLES }),
        "Routine_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4711), tables: &ORACLE_TABLES }),
        "Routine_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4712), tables: &ORACLE_TABLES }),
        "Routine_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4713), tables: &ORACLE_TABLES }),
        "RoutinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4714), tables: &ORACLE_TABLES }),
        "RowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4715), tables: &ORACLE_TABLES }),
        "RowTypeReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4716), tables: &ORACLE_TABLES }),
        "Row_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4720), tables: &ORACLE_TABLES }),
        "Row_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4721), tables: &ORACLE_TABLES }),
        "RowcountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4722), tables: &ORACLE_TABLES }),
        "RowdependenciesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4723), tables: &ORACLE_TABLES }),
        "RowguidcolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4724), tables: &ORACLE_TABLES }),
        "RowidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4725), tables: &ORACLE_TABLES }),
        "RownumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4726), tables: &ORACLE_TABLES }),
        "RowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4727), tables: &ORACLE_TABLES }),
        "RowtypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4728), tables: &ORACLE_TABLES }),
        "RuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4729), tables: &ORACLE_TABLES }),
        "SamplingExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4730), tables: &ORACLE_TABLES }),
        "SaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4742), tables: &ORACLE_TABLES }),
        "SavepointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4743), tables: &ORACLE_TABLES }),
        "ScaleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4744), tables: &ORACLE_TABLES }),
        "SchedulerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4745), tables: &ORACLE_TABLES }),
        "SchemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4746), tables: &ORACLE_TABLES }),
        "SchemaReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1764), tables: &ORACLE_TABLES }),
        "Schema_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4747), tables: &ORACLE_TABLES }),
        "SchemasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4748), tables: &ORACLE_TABLES }),
        "ScopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4749), tables: &ORACLE_TABLES }),
        "Scope_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4750), tables: &ORACLE_TABLES }),
        "Scope_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4751), tables: &ORACLE_TABLES }),
        "Scope_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4752), tables: &ORACLE_TABLES }),
        "ScrollKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4753), tables: &ORACLE_TABLES }),
        "SearchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4754), tables: &ORACLE_TABLES }),
        "SecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4755), tables: &ORACLE_TABLES }),
        "Second_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4756), tables: &ORACLE_TABLES }),
        "SectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4757), tables: &ORACLE_TABLES }),
        "SecurityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4758), tables: &ORACLE_TABLES }),
        "SegmentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4759), tables: &ORACLE_TABLES }),
        "SelectClauseElementSegment" => Some(RootGrammar { grammar_id: GrammarId(4760), tables: &ORACLE_TABLES }),
        "SelectClauseModifierSegment" => Some(RootGrammar { grammar_id: GrammarId(4765), tables: &ORACLE_TABLES }),
        "SelectClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4768), tables: &ORACLE_TABLES }),
        "SelectClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4774), tables: &ORACLE_TABLES }),
        "SelectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4788), tables: &ORACLE_TABLES }),
        "SelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4789), tables: &ORACLE_TABLES }),
        "SelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(4812), tables: &ORACLE_TABLES }),
        "SelfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4822), tables: &ORACLE_TABLES }),
        "SemicolonSegment" => Some(RootGrammar { grammar_id: GrammarId(4823), tables: &ORACLE_TABLES }),
        "SensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4824), tables: &ORACLE_TABLES }),
        "SeparatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4825), tables: &ORACLE_TABLES }),
        "SequenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4826), tables: &ORACLE_TABLES }),
        "SequenceMaxValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(4827), tables: &ORACLE_TABLES }),
        "SequenceMinValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(4832), tables: &ORACLE_TABLES }),
        "SequencePseudocolumnGrammar" => Some(RootGrammar { grammar_id: GrammarId(4837), tables: &ORACLE_TABLES }),
        "SequenceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1764), tables: &ORACLE_TABLES }),
        "SequencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4852), tables: &ORACLE_TABLES }),
        "SerializableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4853), tables: &ORACLE_TABLES }),
        "ServerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4854), tables: &ORACLE_TABLES }),
        "Server_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4855), tables: &ORACLE_TABLES }),
        "ServiceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4856), tables: &ORACLE_TABLES }),
        "SessionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4857), tables: &ORACLE_TABLES }),
        "Session_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4858), tables: &ORACLE_TABLES }),
        "SetClauseListSegment" => Some(RootGrammar { grammar_id: GrammarId(4859), tables: &ORACLE_TABLES }),
        "SetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4863), tables: &ORACLE_TABLES }),
        "SetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4874), tables: &ORACLE_TABLES }),
        "SetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4883), tables: &ORACLE_TABLES }),
        "SetOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(4884), tables: &ORACLE_TABLES }),
        "SetSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4892), tables: &ORACLE_TABLES }),
        "SetofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4897), tables: &ORACLE_TABLES }),
        "SetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4898), tables: &ORACLE_TABLES }),
        "SetuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4899), tables: &ORACLE_TABLES }),
        "ShardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4900), tables: &ORACLE_TABLES }),
        "Shard_enableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4901), tables: &ORACLE_TABLES }),
        "ShareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4902), tables: &ORACLE_TABLES }),
        "SharedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4903), tables: &ORACLE_TABLES }),
        "SharesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4904), tables: &ORACLE_TABLES }),
        "SharingClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(4905), tables: &ORACLE_TABLES }),
        "SharingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4910), tables: &ORACLE_TABLES }),
        "ShorthandCastSegment" => Some(RootGrammar { grammar_id: GrammarId(4911), tables: &ORACLE_TABLES }),
        "ShowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4920), tables: &ORACLE_TABLES }),
        "ShutdownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4921), tables: &ORACLE_TABLES }),
        "SiblingsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4922), tables: &ORACLE_TABLES }),
        "SignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4923), tables: &ORACLE_TABLES }),
        "SignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4924), tables: &ORACLE_TABLES }),
        "SignedSegmentGrammar" => Some(RootGrammar { grammar_id: GrammarId(4925), tables: &ORACLE_TABLES }),
        "SimilarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4928), tables: &ORACLE_TABLES }),
        "SimpleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4929), tables: &ORACLE_TABLES }),
        "SingleIdentifierGrammar" => Some(RootGrammar { grammar_id: GrammarId(4930), tables: &ORACLE_TABLES }),
        "SingleIdentifierListSegment" => Some(RootGrammar { grammar_id: GrammarId(4935), tables: &ORACLE_TABLES }),
        "SingleIdentifierWildcardSegment" => Some(RootGrammar { grammar_id: GrammarId(4937), tables: &ORACLE_TABLES }),
        "SingleQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(4941), tables: &ORACLE_TABLES }),
        "SizeClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(4942), tables: &ORACLE_TABLES }),
        "SizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4945), tables: &ORACLE_TABLES }),
        "SizedArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(4946), tables: &ORACLE_TABLES }),
        "SkipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4949), tables: &ORACLE_TABLES }),
        "SlashBufferExecutorSegment" => Some(RootGrammar { grammar_id: GrammarId(4950), tables: &ORACLE_TABLES }),
        "SlashSegment" => Some(RootGrammar { grammar_id: GrammarId(4951), tables: &ORACLE_TABLES }),
        "SliceSegment" => Some(RootGrammar { grammar_id: GrammarId(4952), tables: &ORACLE_TABLES }),
        "SmallintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4953), tables: &ORACLE_TABLES }),
        "SomeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4954), tables: &ORACLE_TABLES }),
        "SonameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4955), tables: &ORACLE_TABLES }),
        "SourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4956), tables: &ORACLE_TABLES }),
        "SpaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4957), tables: &ORACLE_TABLES }),
        "SpatialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4958), tables: &ORACLE_TABLES }),
        "SpecificKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4959), tables: &ORACLE_TABLES }),
        "Specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4960), tables: &ORACLE_TABLES }),
        "SpecificationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4961), tables: &ORACLE_TABLES }),
        "SpecifictypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4962), tables: &ORACLE_TABLES }),
        "SqlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4963), tables: &ORACLE_TABLES }),
        "Sql_big_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4964), tables: &ORACLE_TABLES }),
        "Sql_big_selectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4965), tables: &ORACLE_TABLES }),
        "Sql_big_tablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4966), tables: &ORACLE_TABLES }),
        "Sql_calc_found_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4967), tables: &ORACLE_TABLES }),
        "Sql_log_offKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4968), tables: &ORACLE_TABLES }),
        "Sql_log_updateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4969), tables: &ORACLE_TABLES }),
        "Sql_low_priority_updatesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4970), tables: &ORACLE_TABLES }),
        "Sql_macroKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4971), tables: &ORACLE_TABLES }),
        "Sql_select_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4972), tables: &ORACLE_TABLES }),
        "Sql_small_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4973), tables: &ORACLE_TABLES }),
        "Sql_warningsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4974), tables: &ORACLE_TABLES }),
        "SqlcaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4975), tables: &ORACLE_TABLES }),
        "SqlcodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4976), tables: &ORACLE_TABLES }),
        "SqlerrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4977), tables: &ORACLE_TABLES }),
        "SqlexceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4978), tables: &ORACLE_TABLES }),
        "SqlplusSetStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4979), tables: &ORACLE_TABLES }),
        "SqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4985), tables: &ORACLE_TABLES }),
        "SqlwarningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4986), tables: &ORACLE_TABLES }),
        "SqrtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4987), tables: &ORACLE_TABLES }),
        "SslKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4988), tables: &ORACLE_TABLES }),
        "StableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4989), tables: &ORACLE_TABLES }),
        "StageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4990), tables: &ORACLE_TABLES }),
        "StagesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4991), tables: &ORACLE_TABLES }),
        "StarSegment" => Some(RootGrammar { grammar_id: GrammarId(4992), tables: &ORACLE_TABLES }),
        "StartBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(222), tables: &ORACLE_TABLES }),
        "StartCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(3768), tables: &ORACLE_TABLES }),
        "StartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4993), tables: &ORACLE_TABLES }),
        "StartSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(731), tables: &ORACLE_TABLES }),
        "StartWithClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4994), tables: &ORACLE_TABLES }),
        "StartingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4998), tables: &ORACLE_TABLES }),
        "StartsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4999), tables: &ORACLE_TABLES }),
        "StateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5000), tables: &ORACLE_TABLES }),
        "StatementAndDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(5001), tables: &ORACLE_TABLES }),
        "StatementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5004), tables: &ORACLE_TABLES }),
        "StatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5005), tables: &ORACLE_TABLES }),
        "StaticKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5087), tables: &ORACLE_TABLES }),
        "StatisticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5088), tables: &ORACLE_TABLES }),
        "Stddev_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5089), tables: &ORACLE_TABLES }),
        "Stddev_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5090), tables: &ORACLE_TABLES }),
        "StdinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5091), tables: &ORACLE_TABLES }),
        "StdoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5092), tables: &ORACLE_TABLES }),
        "StorageClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5093), tables: &ORACLE_TABLES }),
        "StorageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5155), tables: &ORACLE_TABLES }),
        "StoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5156), tables: &ORACLE_TABLES }),
        "Straight_joinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5157), tables: &ORACLE_TABLES }),
        "StreamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5158), tables: &ORACLE_TABLES }),
        "StreamsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5159), tables: &ORACLE_TABLES }),
        "StrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5160), tables: &ORACLE_TABLES }),
        "StringBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5161), tables: &ORACLE_TABLES }),
        "StringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5163), tables: &ORACLE_TABLES }),
        "StructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5164), tables: &ORACLE_TABLES }),
        "StructTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(5169), tables: &ORACLE_TABLES }),
        "StructureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5170), tables: &ORACLE_TABLES }),
        "StyleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5171), tables: &ORACLE_TABLES }),
        "Subclass_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5172), tables: &ORACLE_TABLES }),
        "SublistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5173), tables: &ORACLE_TABLES }),
        "SubmultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5174), tables: &ORACLE_TABLES }),
        "SubpartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5175), tables: &ORACLE_TABLES }),
        "SubstitutionVariableSegment" => Some(RootGrammar { grammar_id: GrammarId(5176), tables: &ORACLE_TABLES }),
        "SubstringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5182), tables: &ORACLE_TABLES }),
        "SuccessfulKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5183), tables: &ORACLE_TABLES }),
        "SumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5184), tables: &ORACLE_TABLES }),
        "SuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5185), tables: &ORACLE_TABLES }),
        "SymbolSegment" => Some(RootGrammar { grammar_id: GrammarId(5186), tables: &ORACLE_TABLES }),
        "SymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5187), tables: &ORACLE_TABLES }),
        "SyncKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5188), tables: &ORACLE_TABLES }),
        "SynonymKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5189), tables: &ORACLE_TABLES }),
        "SysdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5190), tables: &ORACLE_TABLES }),
        "SysguidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5191), tables: &ORACLE_TABLES }),
        "SysidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5192), tables: &ORACLE_TABLES }),
        "SystemKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5193), tables: &ORACLE_TABLES }),
        "System_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5194), tables: &ORACLE_TABLES }),
        "TableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(5195), tables: &ORACLE_TABLES }),
        "TableEndClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5219), tables: &ORACLE_TABLES }),
        "TableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5220), tables: &ORACLE_TABLES }),
        "TableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5230), tables: &ORACLE_TABLES }),
        "TableReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(5231), tables: &ORACLE_TABLES }),
        "Table_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5250), tables: &ORACLE_TABLES }),
        "TablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5251), tables: &ORACLE_TABLES }),
        "TablesampleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5252), tables: &ORACLE_TABLES }),
        "TablespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5253), tables: &ORACLE_TABLES }),
        "TablespaceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1764), tables: &ORACLE_TABLES }),
        "TagReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1764), tables: &ORACLE_TABLES }),
        "Tail_Recurse_Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(5254), tables: &ORACLE_TABLES }),
        "Tail_Recurse_Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(5259), tables: &ORACLE_TABLES }),
        "TaskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5263), tables: &ORACLE_TABLES }),
        "TasksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5264), tables: &ORACLE_TABLES }),
        "TempKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5265), tables: &ORACLE_TABLES }),
        "TemplateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5266), tables: &ORACLE_TABLES }),
        "TemporalQuerySegment" => Some(RootGrammar { grammar_id: GrammarId(5267), tables: &ORACLE_TABLES }),
        "TemporaryGrammar" => Some(RootGrammar { grammar_id: GrammarId(5268), tables: &ORACLE_TABLES }),
        "TemporaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5273), tables: &ORACLE_TABLES }),
        "TemporaryTransientGrammar" => Some(RootGrammar { grammar_id: GrammarId(5274), tables: &ORACLE_TABLES }),
        "TerminateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5277), tables: &ORACLE_TABLES }),
        "TerminatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5278), tables: &ORACLE_TABLES }),
        "TextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5279), tables: &ORACLE_TABLES }),
        "TextsizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5280), tables: &ORACLE_TABLES }),
        "ThanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5281), tables: &ORACLE_TABLES }),
        "ThenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5282), tables: &ORACLE_TABLES }),
        "TiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5283), tables: &ORACLE_TABLES }),
        "TildeSegment" => Some(RootGrammar { grammar_id: GrammarId(5284), tables: &ORACLE_TABLES }),
        "TimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5285), tables: &ORACLE_TABLES }),
        "TimeWithTZGrammar" => Some(RootGrammar { grammar_id: GrammarId(5286), tables: &ORACLE_TABLES }),
        "TimeZoneGrammar" => Some(RootGrammar { grammar_id: GrammarId(5311), tables: &ORACLE_TABLES }),
        "Time_zoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5317), tables: &ORACLE_TABLES }),
        "TimeoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5318), tables: &ORACLE_TABLES }),
        "TimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5319), tables: &ORACLE_TABLES }),
        "Timezone_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5320), tables: &ORACLE_TABLES }),
        "Timezone_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5321), tables: &ORACLE_TABLES }),
        "TimingPointGrammar" => Some(RootGrammar { grammar_id: GrammarId(5322), tables: &ORACLE_TABLES }),
        "TimingPointSectionSegment" => Some(RootGrammar { grammar_id: GrammarId(5334), tables: &ORACLE_TABLES }),
        "TinyblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5343), tables: &ORACLE_TABLES }),
        "TinyintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5344), tables: &ORACLE_TABLES }),
        "TinytextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5345), tables: &ORACLE_TABLES }),
        "ToKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5346), tables: &ORACLE_TABLES }),
        "ToastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5347), tables: &ORACLE_TABLES }),
        "TopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5348), tables: &ORACLE_TABLES }),
        "Top_level_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5349), tables: &ORACLE_TABLES }),
        "TrailingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5350), tables: &ORACLE_TABLES }),
        "TranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5351), tables: &ORACLE_TABLES }),
        "TransactionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5352), tables: &ORACLE_TABLES }),
        "TransactionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5353), tables: &ORACLE_TABLES }),
        "Transaction_activeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5368), tables: &ORACLE_TABLES }),
        "TransactionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5369), tables: &ORACLE_TABLES }),
        "Transactions_committedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5370), tables: &ORACLE_TABLES }),
        "Transactions_rolled_backKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5371), tables: &ORACLE_TABLES }),
        "TransformKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5372), tables: &ORACLE_TABLES }),
        "TransformsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5373), tables: &ORACLE_TABLES }),
        "TransientKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5374), tables: &ORACLE_TABLES }),
        "TranslateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5375), tables: &ORACLE_TABLES }),
        "TranslationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5376), tables: &ORACLE_TABLES }),
        "TreatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5377), tables: &ORACLE_TABLES }),
        "TriggerCorrelationNameSegment" => Some(RootGrammar { grammar_id: GrammarId(5378), tables: &ORACLE_TABLES }),
        "TriggerCorrelationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(5382), tables: &ORACLE_TABLES }),
        "TriggerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5388), tables: &ORACLE_TABLES }),
        "TriggerPredicatesConjunctionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5389), tables: &ORACLE_TABLES }),
        "TriggerPredicatesExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5397), tables: &ORACLE_TABLES }),
        "TriggerPredicatesGrammar" => Some(RootGrammar { grammar_id: GrammarId(5403), tables: &ORACLE_TABLES }),
        "TriggerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1764), tables: &ORACLE_TABLES }),
        "Trigger_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5410), tables: &ORACLE_TABLES }),
        "Trigger_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5411), tables: &ORACLE_TABLES }),
        "Trigger_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5412), tables: &ORACLE_TABLES }),
        "TrimKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5413), tables: &ORACLE_TABLES }),
        "TrimParametersGrammar" => Some(RootGrammar { grammar_id: GrammarId(5414), tables: &ORACLE_TABLES }),
        "TrueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5418), tables: &ORACLE_TABLES }),
        "TrueSegment" => Some(RootGrammar { grammar_id: GrammarId(5419), tables: &ORACLE_TABLES }),
        "TruncateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5420), tables: &ORACLE_TABLES }),
        "TruncateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5421), tables: &ORACLE_TABLES }),
        "TrustedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5425), tables: &ORACLE_TABLES }),
        "TsequalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5426), tables: &ORACLE_TABLES }),
        "TupleSegment" => Some(RootGrammar { grammar_id: GrammarId(5427), tables: &ORACLE_TABLES }),
        "TypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5430), tables: &ORACLE_TABLES }),
        "TypeReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1086), tables: &ORACLE_TABLES }),
        "TypedArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5431), tables: &ORACLE_TABLES }),
        "TypedStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5434), tables: &ORACLE_TABLES }),
        "UescapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5437), tables: &ORACLE_TABLES }),
        "UidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5438), tables: &ORACLE_TABLES }),
        "UnboundedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5439), tables: &ORACLE_TABLES }),
        "UncommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5440), tables: &ORACLE_TABLES }),
        "UnconditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5441), tables: &ORACLE_TABLES }),
        "UnconditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5442), tables: &ORACLE_TABLES }),
        "UnderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5446), tables: &ORACLE_TABLES }),
        "UndoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5447), tables: &ORACLE_TABLES }),
        "UnencryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5448), tables: &ORACLE_TABLES }),
        "UnionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5449), tables: &ORACLE_TABLES }),
        "UnionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5454), tables: &ORACLE_TABLES }),
        "UniqueKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(5455), tables: &ORACLE_TABLES }),
        "UniqueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5457), tables: &ORACLE_TABLES }),
        "UnknownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5458), tables: &ORACLE_TABLES }),
        "UnknownLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5459), tables: &ORACLE_TABLES }),
        "UnlimitedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5460), tables: &ORACLE_TABLES }),
        "UnlistenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5461), tables: &ORACLE_TABLES }),
        "UnlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5462), tables: &ORACLE_TABLES }),
        "UnnamedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5463), tables: &ORACLE_TABLES }),
        "UnnestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5464), tables: &ORACLE_TABLES }),
        "UnorderedSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5465), tables: &ORACLE_TABLES }),
        "UnorderedSetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5484), tables: &ORACLE_TABLES }),
        "UnpivotAsCharacterLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(5490), tables: &ORACLE_TABLES }),
        "UnpivotAsNumericLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(5502), tables: &ORACLE_TABLES }),
        "UnpivotInGrammar" => Some(RootGrammar { grammar_id: GrammarId(5514), tables: &ORACLE_TABLES }),
        "UnpivotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5520), tables: &ORACLE_TABLES }),
        "UnpivotNullsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5521), tables: &ORACLE_TABLES }),
        "UnpivotSegment" => Some(RootGrammar { grammar_id: GrammarId(5526), tables: &ORACLE_TABLES }),
        "UnsignedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5536), tables: &ORACLE_TABLES }),
        "UntilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5537), tables: &ORACLE_TABLES }),
        "UnusableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5538), tables: &ORACLE_TABLES }),
        "UpdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5539), tables: &ORACLE_TABLES }),
        "UpdateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5540), tables: &ORACLE_TABLES }),
        "UpdatetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5549), tables: &ORACLE_TABLES }),
        "UpdatingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5550), tables: &ORACLE_TABLES }),
        "UpperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5551), tables: &ORACLE_TABLES }),
        "UsageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5552), tables: &ORACLE_TABLES }),
        "UseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5553), tables: &ORACLE_TABLES }),
        "UseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5554), tables: &ORACLE_TABLES }),
        "Use_any_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5557), tables: &ORACLE_TABLES }),
        "UserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5558), tables: &ORACLE_TABLES }),
        "UserReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(5559), tables: &ORACLE_TABLES }),
        "User_defined_type_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5560), tables: &ORACLE_TABLES }),
        "User_defined_type_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5561), tables: &ORACLE_TABLES }),
        "User_defined_type_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5562), tables: &ORACLE_TABLES }),
        "User_defined_type_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5563), tables: &ORACLE_TABLES }),
        "UsersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5564), tables: &ORACLE_TABLES }),
        "UsingIndexClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5565), tables: &ORACLE_TABLES }),
        "UsingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5573), tables: &ORACLE_TABLES }),
        "Utc_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5574), tables: &ORACLE_TABLES }),
        "Utc_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5575), tables: &ORACLE_TABLES }),
        "Utc_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5576), tables: &ORACLE_TABLES }),
        "VacuumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5577), tables: &ORACLE_TABLES }),
        "ValidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5578), tables: &ORACLE_TABLES }),
        "ValidateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5579), tables: &ORACLE_TABLES }),
        "ValidatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5580), tables: &ORACLE_TABLES }),
        "ValueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5581), tables: &ORACLE_TABLES }),
        "ValuesClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5582), tables: &ORACLE_TABLES }),
        "ValuesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5593), tables: &ORACLE_TABLES }),
        "Var_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5594), tables: &ORACLE_TABLES }),
        "Var_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5595), tables: &ORACLE_TABLES }),
        "VarbinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5596), tables: &ORACLE_TABLES }),
        "Varchar2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5597), tables: &ORACLE_TABLES }),
        "VarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5598), tables: &ORACLE_TABLES }),
        "VarcharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5599), tables: &ORACLE_TABLES }),
        "VariableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5600), tables: &ORACLE_TABLES }),
        "VariablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5601), tables: &ORACLE_TABLES }),
        "VarrayAndNestedTypeSpecGrammar" => Some(RootGrammar { grammar_id: GrammarId(5602), tables: &ORACLE_TABLES }),
        "VarrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5631), tables: &ORACLE_TABLES }),
        "VaryingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5632), tables: &ORACLE_TABLES }),
        "VerboseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5633), tables: &ORACLE_TABLES }),
        "VersionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5634), tables: &ORACLE_TABLES }),
        "ViewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5635), tables: &ORACLE_TABLES }),
        "ViewsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5636), tables: &ORACLE_TABLES }),
        "VisibilityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5637), tables: &ORACLE_TABLES }),
        "VisibleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5638), tables: &ORACLE_TABLES }),
        "VolatileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5639), tables: &ORACLE_TABLES }),
        "WaitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5640), tables: &ORACLE_TABLES }),
        "WaitforKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5641), tables: &ORACLE_TABLES }),
        "WarehouseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5642), tables: &ORACLE_TABLES }),
        "WarehousesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5643), tables: &ORACLE_TABLES }),
        "WeekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5644), tables: &ORACLE_TABLES }),
        "WeekdayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5645), tables: &ORACLE_TABLES }),
        "WhenClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5646), tables: &ORACLE_TABLES }),
        "WhenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5660), tables: &ORACLE_TABLES }),
        "WheneverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5661), tables: &ORACLE_TABLES }),
        "WhereClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5662), tables: &ORACLE_TABLES }),
        "WhereClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5667), tables: &ORACLE_TABLES }),
        "WhereKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5680), tables: &ORACLE_TABLES }),
        "WhileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5681), tables: &ORACLE_TABLES }),
        "WhileLoopStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5682), tables: &ORACLE_TABLES }),
        "WhitespaceSegment" => Some(RootGrammar { grammar_id: GrammarId(5686), tables: &ORACLE_TABLES }),
        "Width_bucketKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5687), tables: &ORACLE_TABLES }),
        "WildcardExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5688), tables: &ORACLE_TABLES }),
        "WildcardIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(5690), tables: &ORACLE_TABLES }),
        "WindowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5700), tables: &ORACLE_TABLES }),
        "WindowSpecificationSegment" => Some(RootGrammar { grammar_id: GrammarId(5701), tables: &ORACLE_TABLES }),
        "WithCompoundNonSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5709), tables: &ORACLE_TABLES }),
        "WithCompoundStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5718), tables: &ORACLE_TABLES }),
        "WithDataClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5727), tables: &ORACLE_TABLES }),
        "WithFillSegment" => Some(RootGrammar { grammar_id: GrammarId(5732), tables: &ORACLE_TABLES }),
        "WithKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5733), tables: &ORACLE_TABLES }),
        "WithNoSchemaBindingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5734), tables: &ORACLE_TABLES }),
        "WithinGroupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5739), tables: &ORACLE_TABLES }),
        "WithinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5744), tables: &ORACLE_TABLES }),
        "WithoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5745), tables: &ORACLE_TABLES }),
        "WordSegment" => Some(RootGrammar { grammar_id: GrammarId(5746), tables: &ORACLE_TABLES }),
        "WorkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5747), tables: &ORACLE_TABLES }),
        "WrapperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5748), tables: &ORACLE_TABLES }),
        "WriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5749), tables: &ORACLE_TABLES }),
        "WritetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5750), tables: &ORACLE_TABLES }),
        "X509KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5751), tables: &ORACLE_TABLES }),
        "XmlAttributesFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(5752), tables: &ORACLE_TABLES }),
        "XmlAttributesFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(5762), tables: &ORACLE_TABLES }),
        "XmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5763), tables: &ORACLE_TABLES }),
        "XorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5764), tables: &ORACLE_TABLES }),
        "YamlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5765), tables: &ORACLE_TABLES }),
        "YearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5766), tables: &ORACLE_TABLES }),
        "Year_monthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5767), tables: &ORACLE_TABLES }),
        "ZerofillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5768), tables: &ORACLE_TABLES }),
        "ZoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5769), tables: &ORACLE_TABLES }),
        _ => None,
    }
}

pub fn get_oracle_segment_type(name: &str) -> Option<&'static str> {
    match name {
        "AccessObjectSegment" => Some("access_object"),
        "AccessPermissionSegment" => Some("access_permission"),
        "AccessPermissionsSegment" => Some("access_permissions"),
        "AccessSchemaObjectSegment" => Some("access_schema_object"),
        "AccessSchemaPluralObjectSegment" => Some("access_schema_plural_object"),
        "AccessStatementSegment" => Some("access_statement"),
        "AccessTargetSegment" => Some("access_target"),
        "AggregateOrderByClause" => Some("aggregate_order_by"),
        "AliasExpressionSegment" => Some("alias_expression"),
        "AlterDatabaseLinkStatementSegment" => Some("alter_database_link_statement"),
        "AlterFunctionStatementSegment" => Some("alter_function_statement"),
        "AlterIndexStatementSegment" => Some("alter_index_statement"),
        "AlterPackageStatementSegment" => Some("alter_package_statement"),
        "AlterSequenceOptionsSegment" => Some("alter_sequence_options_segment"),
        "AlterSequenceStatementSegment" => Some("alter_sequence_statement"),
        "AlterSessionStatementSegment" => Some("alter_session_statement"),
        "AlterSynonymStatementSegment" => Some("alter_synonym_statement"),
        "AlterTableColumnClausesSegment" => Some("alter_table_column_clauses"),
        "AlterTableConstraintClauses" => Some("alter_table_constraint_clauses"),
        "AlterTablePropertiesSegment" => Some("alter_table_properties"),
        "AlterTableStatementSegment" => Some("alter_table_statement"),
        "AlterTriggerStatementSegment" => Some("alter_trigger_statement"),
        "ArrayAccessorSegment" => Some("array_accessor"),
        "ArrayExpressionSegment" => Some("array_expression"),
        "ArrayLiteralSegment" => Some("array_literal"),
        "ArrayTypeSegment" => Some("array_type"),
        "AsAliasOperatorSegment" => Some("alias_operator"),
        "AssignmentStatementSegment" => Some("assignment_segment_statement"),
        "BaseFileSegment" => Some("file"),
        "BaseSegment" => Some("base"),
        "BatchSegment" => Some("batch"),
        "BeginEndSegment" => Some("begin_end_block"),
        "BinaryOperatorSegment" => Some("binary_operator"),
        "BindVariableSegment" => Some("bind_variable"),
        "BitwiseAndSegment" => Some("binary_operator"),
        "BitwiseLShiftSegment" => Some("binary_operator"),
        "BitwiseOrSegment" => Some("binary_operator"),
        "BitwiseRShiftSegment" => Some("binary_operator"),
        "BracketedArguments" => Some("bracketed_arguments"),
        "BracketedSegment" => Some("bracketed"),
        "BulkCollectIntoClauseSegment" => Some("bulk_collect_into_clause"),
        "CTEColumnList" => Some("cte_column_list"),
        "CTEDefinitionSegment" => Some("common_table_expression"),
        "CaseExpressionSegment" => Some("case_expression"),
        "CloseStatementSegment" => Some("close_statement"),
        "CodeSegment" => Some("raw"),
        "CollationReferenceSegment" => Some("collation_reference"),
        "CollectionTypeDefinitionSegment" => Some("collection_type"),
        "ColumnConstraintSegment" => Some("column_constraint_segment"),
        "ColumnDefinitionSegment" => Some("column_definition"),
        "ColumnReferenceSegment" => Some("column_reference"),
        "ColumnTypeReferenceSegment" => Some("column_type_reference"),
        "ColumnsExpressionFunctionContentsSegment" => Some("columns_expression"),
        "ColumnsExpressionFunctionNameSegment" => Some("function_name"),
        "CommentClauseSegment" => Some("comment_clause"),
        "CommentSegment" => Some("comment"),
        "CommentStatementSegment" => Some("comment_statement"),
        "ComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeBinaryOperatorSegment" => Some("binary_operator"),
        "CompositeComparisonOperatorSegment" => Some("comparison_operator"),
        "CompoundTriggerBlock" => Some("compound_trigger_statement"),
        "ConcatSegment" => Some("binary_operator"),
        "ConnectByClauseSegment" => Some("connectby_clause"),
        "ContinueStatementSegment" => Some("continue_statement"),
        "CreateCastStatementSegment" => Some("create_cast_statement"),
        "CreateDatabaseLinkStatementSegment" => Some("create_database_link_statement"),
        "CreateDatabaseStatementSegment" => Some("create_database_statement"),
        "CreateFunctionStatementSegment" => Some("create_function_statement"),
        "CreateIndexStatementSegment" => Some("create_index_statement"),
        "CreateModelStatementSegment" => Some("create_model_statement"),
        "CreatePackageBodyStatementSegment" => Some("create_package_body_statement"),
        "CreatePackageStatementSegment" => Some("create_package_statement"),
        "CreateProcedureStatementSegment" => Some("create_procedure_statement"),
        "CreateRoleStatementSegment" => Some("create_role_statement"),
        "CreateSchemaStatementSegment" => Some("create_schema_statement"),
        "CreateSequenceOptionsSegment" => Some("create_sequence_options_segment"),
        "CreateSequenceStatementSegment" => Some("create_sequence_statement"),
        "CreateSynonymStatementSegment" => Some("create_synonym_statement"),
        "CreateTableStatementSegment" => Some("create_table_statement"),
        "CreateTriggerStatementSegment" => Some("create_trigger_statement"),
        "CreateTypeBodyStatementSegment" => Some("create_type_body_statement"),
        "CreateTypeStatementSegment" => Some("create_type_statement"),
        "CreateUserStatementSegment" => Some("create_user_statement"),
        "CreateViewStatementSegment" => Some("create_view_statement"),
        "CubeFunctionNameSegment" => Some("function_name"),
        "CubeRollupClauseSegment" => Some("cube_rollup_clause"),
        "DatabaseLinkReferenceSegment" => Some("database_link_reference"),
        "DatabaseReferenceSegment" => Some("database_reference"),
        "DatatypeSegment" => Some("data_type"),
        "DatePartFunctionNameSegment" => Some("function_name"),
        "DateTimeFunctionContentsSegment" => Some("function_contents"),
        "DeclareCursorVariableSegment" => Some("cursor_variable"),
        "DeclareSegment" => Some("declare_segment"),
        "Dedent" => Some("dedent"),
        "DeleteStatementSegment" => Some("delete_statement"),
        "DescribeStatementSegment" => Some("describe_statement"),
        "DmlEventClauseSegment" => Some("dml_event_clause"),
        "DropCastStatementSegment" => Some("drop_cast_statement"),
        "DropClusterStatementSegment" => Some("drop_cluster_statement"),
        "DropDatabaseLinkStatementSegment" => Some("drop_database_link_statement"),
        "DropDatabaseStatementSegment" => Some("drop_database_statement"),
        "DropFunctionStatementSegment" => Some("drop_function_statement"),
        "DropIndexStatementSegment" => Some("drop_index_statement"),
        "DropModelStatementSegment" => Some("drop_MODELstatement"),
        "DropPackageStatementSegment" => Some("drop_package_statement"),
        "DropProcedureStatementSegment" => Some("drop_procedure_statement"),
        "DropProfileStatementSegment" => Some("drop_profile_statement"),
        "DropRoleStatementSegment" => Some("drop_role_statement"),
        "DropSchemaStatementSegment" => Some("drop_schema_statement"),
        "DropSequenceStatementSegment" => Some("drop_sequence_statement"),
        "DropSynonymStatementSegment" => Some("drop_synonym_statement"),
        "DropTableStatementSegment" => Some("drop_table_statement"),
        "DropTriggerStatementSegment" => Some("drop_trigger"),
        "DropTypeStatementSegment" => Some("drop_type_statement"),
        "DropUserStatementSegment" => Some("drop_user_statement"),
        "DropViewStatementSegment" => Some("drop_view_statement"),
        "ElseClauseSegment" => Some("else_clause"),
        "EmptyStructLiteralBracketsSegment" => Some("struct_literal"),
        "EmptyStructLiteralSegment" => Some("typed_struct_literal"),
        "EqualsSegment" => Some("comparison_operator"),
        "ExecuteFileSegment" => Some("execute_file_statement"),
        "ExecuteImmediateSegment" => Some("execute_immediate_statement"),
        "ExitStatementSegment" => Some("exit_statement"),
        "ExplainStatementSegment" => Some("explain_statement"),
        "ExpressionSegment" => Some("expression"),
        "ExtensionReferenceSegment" => Some("extension_reference"),
        "FetchClauseSegment" => Some("fetch_clause"),
        "FetchStatementSegment" => Some("fetch_statement"),
        "FileSegment" => Some("file"),
        "ForAllStatementSegment" => Some("forall_statement"),
        "ForLoopStatementSegment" => Some("for_loop_statement"),
        "FrameClauseSegment" => Some("frame_clause"),
        "FromClauseSegment" => Some("from_clause"),
        "FromExpressionElementSegment" => Some("from_expression_element"),
        "FromExpressionSegment" => Some("from_expression"),
        "FunctionContentsSegment" => Some("function_contents"),
        "FunctionDefinitionGrammar" => Some("function_definition"),
        "FunctionNameSegment" => Some("function_name"),
        "FunctionParameterListGrammar" => Some("function_parameter_list"),
        "FunctionSegment" => Some("function"),
        "GrantStatementSegment" => Some("grant_statement"),
        "GreaterThanOrEqualToSegment" => Some("comparison_operator"),
        "GreaterThanSegment" => Some("comparison_operator"),
        "GroupByClauseSegment" => Some("groupby_clause"),
        "GroupingExpressionList" => Some("grouping_expression_list"),
        "GroupingSetsClauseSegment" => Some("grouping_sets_clause"),
        "HavingClauseSegment" => Some("having_clause"),
        "HierarchicalQueryClauseSegment" => Some("hierarchical_query_clause"),
        "IdentifierSegment" => Some("identifier"),
        "IfClauseSegment" => Some("if_clause"),
        "IfExpressionStatement" => Some("if_then_statement"),
        "ImplicitIndent" => Some("indent"),
        "Indent" => Some("indent"),
        "IndexColumnDefinitionSegment" => Some("index_column_definition"),
        "IndexReferenceSegment" => Some("index_reference"),
        "IndexTypeReferenceSegment" => Some("indextype_reference"),
        "InsertStatementSegment" => Some("insert_statement"),
        "IntervalDataTypeSegment" => Some("interval_data_type"),
        "IntervalExpressionSegment" => Some("interval_expression"),
        "IntoClauseSegment" => Some("into_clause"),
        "JoinClauseSegment" => Some("join_clause"),
        "JoinOnConditionSegment" => Some("join_on_condition"),
        "JsonTableColumnDefinitionSegment" => Some("json_table_column_definition"),
        "JsonTableColumnsClauseSegment" => Some("json_table_columns_clause"),
        "JsonTableFunctionContentsSegment" => Some("function_contents"),
        "JsonTableFunctionNameSegment" => Some("function_name"),
        "KeepClauseSegment" => Some("keep_clause"),
        "KeywordSegment" => Some("keyword"),
        "LessThanOrEqualToSegment" => Some("comparison_operator"),
        "LessThanSegment" => Some("comparison_operator"),
        "LimitClauseSegment" => Some("limit_clause"),
        "ListaggOverflowClauseSegment" => Some("listagg_overflow_clause"),
        "LiteralKeywordSegment" => Some("literal"),
        "LiteralSegment" => Some("literal"),
        "LocalAliasSegment" => Some("local_alias_segment"),
        "LoopStatementSegment" => Some("loop_statement"),
        "MLTableExpressionSegment" => Some("ml_table_expression"),
        "MapTypeSegment" => Some("map_type"),
        "MatchConditionSegment" => Some("match_condition"),
        "MergeDeleteClauseSegment" => Some("merge_delete_clause"),
        "MergeInsertClauseSegment" => Some("merge_insert_clause"),
        "MergeMatchSegment" => Some("merge_match"),
        "MergeMatchedClauseSegment" => Some("merge_when_matched_clause"),
        "MergeNotMatchedClauseSegment" => Some("merge_when_not_matched_clause"),
        "MergeStatementSegment" => Some("merge_statement"),
        "MergeUpdateClauseSegment" => Some("merge_update_clause"),
        "MultisetOperatorSegment" => Some("binary_operator"),
        "NamedArgumentSegment" => Some("named_argument"),
        "NamedWindowExpressionSegment" => Some("named_window_expression"),
        "NamedWindowSegment" => Some("named_window"),
        "NewlineSegment" => Some("newline"),
        "NotEqualToSegment" => Some("comparison_operator"),
        "NullStatementSegment" => Some("null_statement"),
        "ObjectLiteralElementSegment" => Some("object_literal_element"),
        "ObjectLiteralSegment" => Some("object_literal"),
        "ObjectReferenceSegment" => Some("object_reference"),
        "OffsetClauseSegment" => Some("offset_clause"),
        "OpenForStatementSegment" => Some("open_for_statement"),
        "OpenStatementSegment" => Some("open_statement"),
        "OracleIndexPhysicalAttributesSegment" => Some("oracle_index_physical_attributes"),
        "OraclePhysicalAttributesSegment" => Some("oracle_physical_attributes"),
        "OrderByClauseSegment" => Some("orderby_clause"),
        "OverClauseSegment" => Some("over_clause"),
        "OverlapsClauseSegment" => Some("overlaps_clause"),
        "PackageReferenceSegment" => Some("package_reference"),
        "PartitionClauseSegment" => Some("partitionby_clause"),
        "PathSegment" => Some("path_segment"),
        "PivotSegment" => Some("pivot_clause"),
        "PlsqlStatementSegment" => Some("statement"),
        "ProcedureCallStatementSegment" => Some("procedure_call_statement"),
        "QualifiedNumericLiteralSegment" => Some("numeric_literal"),
        "RaiseStatementSegment" => Some("raise_statement"),
        "RecordTypeDefinitionSegment" => Some("record_type"),
        "RefCursorTypeDefinitionSegment" => Some("ref_cursor_type"),
        "ReferencingClauseSegment" => Some("referencing_clause"),
        "ReturnStatementSegment" => Some("return_statement"),
        "ReturningClauseSegment" => Some("returning_clause"),
        "RevokeStatementSegment" => Some("revoke_statement"),
        "RoleReferenceSegment" => Some("role_reference"),
        "RollupFunctionNameSegment" => Some("function_name"),
        "RowTypeReferenceSegment" => Some("row_type_reference"),
        "SamplingExpressionSegment" => Some("sample_expression"),
        "SchemaReferenceSegment" => Some("schema_reference"),
        "SelectClauseElementSegment" => Some("select_clause_element"),
        "SelectClauseModifierSegment" => Some("select_clause_modifier"),
        "SelectClauseSegment" => Some("select_clause"),
        "SelectStatementSegment" => Some("select_statement"),
        "SequenceReferenceSegment" => Some("sequence_reference"),
        "SetClauseListSegment" => Some("set_clause_list"),
        "SetClauseSegment" => Some("set_clause"),
        "SetExpressionSegment" => Some("set_expression"),
        "SetOperatorSegment" => Some("set_operator"),
        "SetSchemaStatementSegment" => Some("set_schema_statement"),
        "ShorthandCastSegment" => Some("cast_expression"),
        "SingleIdentifierListSegment" => Some("identifier_list"),
        "SingleIdentifierWildcardSegment" => Some("wildcard_identifier"),
        "SizedArrayTypeSegment" => Some("sized_array_type"),
        "SlashBufferExecutorSegment" => Some("slash_buffer_executor"),
        "SqlplusSetStatementSegment" => Some("sqlplus_set_statement"),
        "StartWithClauseSegment" => Some("startwith_clause"),
        "StatementSegment" => Some("statement"),
        "StorageClauseSegment" => Some("storage_clause"),
        "StructLiteralSegment" => Some("struct_literal"),
        "StructTypeSegment" => Some("struct_type"),
        "SubstitutionVariableSegment" => Some("substitution_variable"),
        "SymbolSegment" => Some("symbol"),
        "TableConstraintSegment" => Some("table_constraint"),
        "TableEndClauseSegment" => Some("table_end_clause_segment"),
        "TableExpressionSegment" => Some("table_expression"),
        "TableReferenceSegment" => Some("table_reference"),
        "TablespaceReferenceSegment" => Some("tablespace_reference"),
        "TagReferenceSegment" => Some("tag_reference"),
        "TemporalQuerySegment" => Some("temporal_query"),
        "TimeZoneGrammar" => Some("time_zone_grammar"),
        "TimingPointSectionSegment" => Some("timing_point_section"),
        "TransactionStatementSegment" => Some("transaction_statement"),
        "TriggerCorrelationNameSegment" => Some("trigger_correlation_name"),
        "TriggerCorrelationReferenceSegment" => Some("bind_variable"),
        "TriggerReferenceSegment" => Some("trigger_reference"),
        "TruncateStatementSegment" => Some("truncate_table"),
        "TupleSegment" => Some("tuple"),
        "TypeReferenceSegment" => Some("type_reference"),
        "TypedArrayLiteralSegment" => Some("typed_array_literal"),
        "TypedStructLiteralSegment" => Some("typed_struct_literal"),
        "UnorderedSelectStatementSegment" => Some("select_statement"),
        "UnorderedSetExpressionSegment" => Some("set_expression"),
        "UnpivotSegment" => Some("unpivot_clause"),
        "UpdateStatementSegment" => Some("update_statement"),
        "UseStatementSegment" => Some("use_statement"),
        "UserReferenceSegment" => Some("user_reference"),
        "UsingIndexClauseSegment" => Some("using_index_clause"),
        "ValuesClauseSegment" => Some("values_clause"),
        "WhenClauseSegment" => Some("when_clause"),
        "WhereClauseSegment" => Some("where_clause"),
        "WhileLoopStatementSegment" => Some("while_loop_statement"),
        "WhitespaceSegment" => Some("whitespace"),
        "WildcardExpressionSegment" => Some("wildcard_expression"),
        "WildcardIdentifierSegment" => Some("wildcard_identifier"),
        "WindowSpecificationSegment" => Some("window_specification"),
        "WithCompoundNonSelectStatementSegment" => Some("with_compound_statement"),
        "WithCompoundStatementSegment" => Some("with_compound_statement"),
        "WithDataClauseSegment" => Some("with_data_clause"),
        "WithFillSegment" => Some("with_fill"),
        "WithNoSchemaBindingClauseSegment" => Some("with_no_schema_binding_clause"),
        "WithinGroupClauseSegment" => Some("withingroup_clause"),
        "WordSegment" => Some("word"),
        "XmlAttributesFunctionContentsSegment" => Some("function_contents"),
        "XmlAttributesFunctionNameSegment" => Some("function_name"),
        _ => None,
    }
}

pub static ORACLE_TABLES: GrammarTables = GrammarTables {
    instructions: INSTRUCTIONS,
    child_ids: CHILD_IDS,
    terminators: TERMINATORS,
    strings: STRINGS,
    aux_data: AUX_DATA,
    aux_data_offsets: AUX_DATA_OFFSETS,
    regex_patterns: REGEX_PATTERNS,
    simple_hints: SIMPLE_HINTS,
    hint_string_indices: HINT_STRING_INDICES,
    simple_hint_indices: SIMPLE_HINT_INDICES,
    segment_type_offsets: SEGMENT_TYPE_OFFSETS,
    segment_class_offsets: SEGMENT_CLASS_OFFSETS,
    casefold_sparse: CASEFOLD_SPARSE,
    trim_chars_sparse: TRIM_CHARS_SPARSE,
    trim_chars_data: TRIM_CHARS_DATA,
    segment_class_types_sparse: SEGMENT_CLASS_TYPES_SPARSE,
    segment_class_types_data: SEGMENT_CLASS_TYPES_DATA,
};

pub fn get_oracle_root_grammar_id() -> GrammarId {
    GrammarId(2408)
}
pub fn get_oracle_root_grammar_table() -> RootGrammar {
    RootGrammar {
        grammar_id: get_oracle_root_grammar_id(),
        tables: &ORACLE_TABLES,
    }
}
pub fn get_oracle_root_grammar() -> RootGrammar {
    get_oracle_root_grammar_table()
}
