/* This is a generated file! */
/* Generated by `utils/build_parsers.py` via `utils/rustify.py` */
/* This process can be run via tox: `tox -e generate-rs` */
#![cfg_attr(rustfmt, rustfmt_skip)]

use sqlfluffrs_types::{
    GrammarInst, GrammarVariant, GrammarFlags,
    ParseMode, GrammarId, GrammarTables,
    SimpleHintData, RootGrammar
};
// Flattening grammar tree...
// Validating tables...
// Validation passed!

// Table Statistics:
//   Instructions:      3648 × 20 bytes =    72960 bytes
//   Child IDs:         2504 ×  4 bytes =    10016 bytes
//   Terminators:         67 ×  4 bytes =      268 bytes
//   Strings:           1914 strings      =    25474 bytes
//   Aux Data:         11367 ×  4 bytes =    45468 bytes
//   Aux Offsets:       3648 ×  4 bytes =    14592 bytes
//   Regex Patterns:       6 patterns     =     1050 bytes
//   Simple Hints:      1147 hints        =    18352 bytes
//   Hint Indices:      1748 ×  4 bytes =     6992 bytes
//   TOTAL:                                    ≈   195172 bytes

pub static INSTRUCTIONS: &[GrammarInst] = &[
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [0] StringParser("ABORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [1] StringParser("ABS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [2] StringParser("ABSOLUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [3] StringParser("ACCESS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 0, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [4] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [5] Ref(AccountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [6] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5, child_count: 17, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [7] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 22, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [8] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [9] Ref(ResourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [10] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [11] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [12] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [13] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [14] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [15] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [16] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [17] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [18] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [19] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [20] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [21] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [22] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [23] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [24] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [25] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [26] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [27] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [28] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [29] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [30] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [31] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [32] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [33] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [34] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [35] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [36] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [37] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [38] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [39] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [40] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [41] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [42] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [43] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [44] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [45] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [46] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [47] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [48] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [49] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [50] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 2, _padding: 0 }, // [51] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 50, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [52] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [53] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [54] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [55] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [56] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [57] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [58] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [59] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [60] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [61] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [62] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [63] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [64] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 6, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [65] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 64, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [66] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [67] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 5, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [68] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [69] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [70] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [71] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [72] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [73] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [74] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [75] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [76] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [77] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [78] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [79] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [80] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [81] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [82] Ref(ManageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [83] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [84] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [85] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [86] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [87] Ref(ExecutionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [88] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [89] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 26, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [90] OneOf(26 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 110, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [91] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [92] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 4, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [93] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [94] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [95] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [96] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [97] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [98] Ref(PipeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [99] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [100] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [101] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [102] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [103] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [104] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [105] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [106] Ref(ImportedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [107] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [108] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [109] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [110] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [111] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [112] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [113] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [114] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [115] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [116] Ref(OperateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [117] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [118] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [119] Ref(Reference_usageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [120] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [121] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [122] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [123] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [124] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [125] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [126] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [127] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [128] Ref(Use_any_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [129] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [130] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [131] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [132] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [133] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 1, _padding: 0 }, // [134] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [135] Ref(AccessPermissionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [136] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [137] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [138] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [139] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [140] Ref(StageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [141] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [142] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [143] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [144] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [145] Ref(StreamKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [146] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [147] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [148] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [149] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [150] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [151] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [152] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [153] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [154] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [155] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [156] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [157] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [158] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [159] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [160] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [161] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [162] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [163] Ref(StagesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [164] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [165] Ref(ProceduresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [166] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [167] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [168] Ref(StreamsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [169] Ref(TasksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [170] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [171] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [172] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [173] Ref(PoliciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [174] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [175] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [176] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [177] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [178] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [179] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [180] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [181] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [182] Ref(FormatsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [183] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [184] Ref(GrantStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [185] Ref(RevokeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [186] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 181, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [187] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [188] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [189] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [190] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [191] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [192] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [193] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [194] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [195] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [196] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [197] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [198] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [199] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [200] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [201] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [202] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [203] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [204] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [205] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [206] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [207] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 201, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [208] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [209] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [210] StringParser("ACCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [211] StringParser("ACCOUNTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [212] StringParser("ACTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [213] StringParser("ADA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [214] StringParser("ADD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [215] StringParser("ADMIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [216] StringParser("AFTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [217] StringParser("AGGREGATE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [218] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [219] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [220] Meta("indent")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [221] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [222] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 208, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [223] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 210, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [224] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 210, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [225] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [226] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [227] StringParser("(")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [228] StringParser(")")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [229] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [230] Meta("dedent")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [231] StringParser("ALIAS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 213, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [232] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [233] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [234] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [235] StringParser("ALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [236] StringParser("ALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [237] StringParser("ALSO")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [238] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 220, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [239] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 220, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [240] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 222, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [241] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 222, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [242] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 222, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [243] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 222, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [244] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 222, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [245] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 226, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [246] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [247] Ref(DbpropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [248] Ref(BracketedPropertyListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [249] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [250] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [251] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [252] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [253] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [254] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [255] Ref(LocationGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [256] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 235, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [257] Ref(ManagedlocationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 235, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [258] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 235, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [259] StringParser("ALTER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 235, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [260] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 241, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [261] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 244, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [262] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 244, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [263] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 244, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [264] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 244, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [265] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 244, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [266] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 244, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [267] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [268] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [269] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [270] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [271] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [272] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 250, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [273] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 250, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [274] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 250, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [275] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 250, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [276] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 254, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [277] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 254, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [278] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 254, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [279] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 254, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [280] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [281] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [282] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [283] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [284] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [285] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [286] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [287] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 264, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [288] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 267, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [289] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 267, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [290] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 267, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [291] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 269, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [292] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 269, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [293] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 269, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [294] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 273, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [295] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [296] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [297] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 275, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [298] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [299] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 275, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [300] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 277, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [301] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 279, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [302] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [303] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [304] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [305] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [306] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [307] Ref(AlterTableDropColumnGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [308] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [309] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 284, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [310] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 286, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [311] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 286, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [312] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 286, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [313] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 286, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [314] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 291, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [315] Ref(ExchangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 291, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [316] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 291, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [317] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 291, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [318] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 291, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [319] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 291, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [320] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 295, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [321] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 295, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [322] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 295, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [323] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 295, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [324] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 297, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [325] Ref(AlterTableOptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 297, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [326] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 301, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [327] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 301, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [328] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 301, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [329] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 301, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [330] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 303, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [331] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 305, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [332] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 305, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [333] Ref(TablePropertiesGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 305, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [334] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [335] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [336] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 309, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [337] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [338] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [339] StringParser("ALWAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [340] StringParser("&")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [341] StringParser("ANALYSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [342] StringParser("ANALYZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [343] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [344] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [345] StringParser("ANY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [346] StringParser("APPLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [347] StringParser("ARCHIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [348] StringParser("ARE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 10, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [349] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [350] Ref(PlusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [351] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [352] Ref(DivideSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [353] Ref(MultiplySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [354] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [355] Ref(BitwiseAndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [356] Ref(BitwiseOrSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [357] Ref(BitwiseXorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [358] Ref(BitwiseLShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [359] Ref(BitwiseRShiftSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [360] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 325, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [361] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 327, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [362] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [363] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [364] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [365] Ref(SliceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [366] StringParser("[")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [367] StringParser("]")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [368] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [369] StringParser("ARRAY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 329, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [370] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 332, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [371] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 334, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [372] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 334, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [373] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 336, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [374] Ref(ArrayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 336, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [375] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [376] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [377] StringParser("<")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [378] StringParser(">")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 339, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [379] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 340, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [380] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 340, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [381] StringParser("AS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 340, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [382] StringParser("ASC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 340, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [383] StringParser("ASENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 340, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [384] StringParser("ASSERTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 340, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [385] StringParser("ASSIGNMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 340, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [386] StringParser("ASYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 340, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [387] StringParser("AT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 340, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [388] StringParser("ATOMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 340, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [389] StringParser("ATTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 340, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [390] StringParser("ATTRIBUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 340, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [391] StringParser("AUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 340, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [392] StringParser("AUTHORIZATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 340, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [393] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [394] Ref(Auto_incrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [395] StringParser("AUTO_INCREMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [396] StringParser("AUTOCOMMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [397] StringParser("AVG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [398] StringParser("AVG_ROW_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [399] StringParser("AVRO")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [400] TypedParser("back_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [401] StringParser("BACKUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [402] StringParser("BACKWARD")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [403] MultiStringParser(3 templates)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 341, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 2, _padding: 0 }, // [404] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [405] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [406] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [407] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [408] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [409] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [410] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [411] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [412] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [413] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [414] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [415] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [416] Token("file")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [417] Token("base")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [418] StringParser("BEFORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [419] StringParser("BEGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [420] StringParser("BERNOULLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [421] StringParser("BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [422] StringParser("BIGINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [423] StringParser("BINARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [424] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [425] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [426] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [427] Ref(BooleanBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [428] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [429] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [430] StringParser("BINDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [431] StringParser("BIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [432] StringParser("BIT_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [433] StringParser("BITVAR")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [434] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 354, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [435] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [436] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [437] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [438] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [439] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [440] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [441] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 358, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [442] StringParser("^")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 358, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [443] StringParser("BLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 358, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [444] StringParser("BOOL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [445] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [446] Ref(AndOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [447] Ref(OrOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [448] StringParser("BOOLEAN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [449] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 362, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [450] Ref(TrueSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 362, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [451] Ref(FalseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 362, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [452] StringParser("BOTH")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 362, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [453] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 365, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [454] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 367, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [455] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 367, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [456] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [457] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 372, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [458] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 372, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [459] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 375, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [460] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 377, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [461] Ref(PropertyGrammar)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 377, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [462] Token("bracketed")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 377, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [463] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 380, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [464] Ref(UnorderedSetExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 380, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [465] StringParser("BREADTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 380, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [466] StringParser("BREAK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 380, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [467] StringParser("BROWSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 380, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [468] StringParser("BUCKET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 380, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [469] StringParser("BUCKETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 380, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [470] StringParser("BULK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 380, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [471] StringParser("BY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 380, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [472] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 383, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [473] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 383, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [474] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 387, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [475] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 387, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [476] Ref(CTEColumnList)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 387, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [477] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 387, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [478] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 390, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [479] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 390, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [480] StringParser("CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 390, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [481] StringParser("CALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 390, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [482] StringParser("CALLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 390, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [483] StringParser("CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 390, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [484] StringParser("CASCADE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 390, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [485] StringParser("CASCADED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 390, child_count: 2, min_times: 0, first_terminator_idx: 11, terminator_count: 3, _padding: 0 }, // [486] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 392, child_count: 6, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [487] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 398, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [488] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 398, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [489] Meta("implicit_indent")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 398, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 2, _padding: 0 }, // [490] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [491] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [492] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [493] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 1, _padding: 0 }, // [494] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [495] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [496] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 7, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [497] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [498] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [499] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 406, child_count: 1, min_times: 0, first_terminator_idx: 8, terminator_count: 2, _padding: 0 }, // [500] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [501] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [502] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [503] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 1, _padding: 0 }, // [504] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [505] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [506] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [507] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [508] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [509] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [510] StringParser("CASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [511] StringParser("CAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [512] StringParser("::")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [513] StringParser("CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [514] StringParser("CATALOG_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [515] StringParser("CEIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [516] StringParser("CEILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [517] StringParser("CHAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [518] StringParser("CHANGE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [519] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [520] StringParser("CHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [521] StringParser("CHAR_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [522] StringParser("CHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [523] StringParser("CHARACTER_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [524] StringParser("CHARACTER_SET_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [525] StringParser("CHARACTER_SET_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [526] StringParser("CHARACTER_SET_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [527] StringParser("CHARACTERISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [528] StringParser("CHARACTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [529] StringParser("CHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [530] StringParser("CHECKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [531] StringParser("CHECKPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [532] StringParser("CHECKSUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [533] StringParser("CLASS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [534] StringParser("CLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [535] StringParser("CLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [536] StringParser("CLOSE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 407, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [537] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [538] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [539] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 2, _padding: 0 }, // [540] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 414, child_count: 1, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [541] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [542] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [543] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [544] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [545] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [546] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [547] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [548] StringParser("CLUSTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [549] StringParser("CLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [550] StringParser("CLUSTERSTATUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [551] StringParser("COALESCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [552] StringParser("COBOL")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [553] Token("raw")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [554] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [555] StringParser("COLLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [556] StringParser("COLLATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [557] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 420, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [558] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 420, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 1, _padding: 0 }, // [559] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [560] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [561] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [562] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [563] StringParser("COLLATION_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [564] StringParser("COLLATION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [565] StringParser("COLLATION_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [566] StringParser("COLLECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [567] StringParser("COLLECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [568] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [569] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [570] StringParser(":")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 422, child_count: 4, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [571] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 426, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [572] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 426, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [573] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 426, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [574] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 426, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [575] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 426, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [576] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 428, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [577] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [578] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [579] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 10, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [580] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 440, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [581] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [582] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [583] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [584] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 444, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [585] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 444, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [586] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 447, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [587] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 447, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [588] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [589] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [590] Ref(ColumnConstraintDefaultGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 449, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [591] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 451, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [592] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 451, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [593] Ref(NotEnforcedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 451, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [594] Ref(UniqueKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 451, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [595] Ref(AutoIncrementGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 451, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [596] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 453, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [597] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 453, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [598] Ref(NotEnforcedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 453, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [599] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 453, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [600] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 455, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [601] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 455, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [602] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 455, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [603] Ref(ColumnGeneratedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 455, child_count: 4, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [604] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [605] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [606] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 459, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [607] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 462, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [608] Anything
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 462, child_count: 1, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [609] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 463, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [610] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 463, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [611] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 463, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [612] StringParser("COLUMN")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 463, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 1, _padding: 0 }, // [613] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 465, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [614] Ref(HiveReferenceIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 465, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [615] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 465, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [616] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 465, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [617] StringParser("COLUMN_NAME")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 465, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [618] Nothing
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 465, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [619] Ref(ColumnsExpressionNameGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 465, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [620] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 465, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [621] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 465, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [622] StringParser("COLUMNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 465, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [623] StringParser(",")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 465, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [624] StringParser("COMMAND_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 465, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [625] StringParser("COMMAND_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 465, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [626] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 467, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [627] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 467, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [628] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 467, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [629] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 469, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [630] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 469, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [631] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 469, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [632] StringParser("COMMENT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 469, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [633] Token("comment")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 469, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [634] StringParser("COMMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 469, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [635] StringParser("COMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 469, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [636] StringParser("COMPACT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 469, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [637] StringParser("COMPACTIONS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 469, child_count: 8, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [638] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 477, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [639] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 477, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [640] Ref(GreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 477, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [641] Ref(LessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 477, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [642] Ref(GreaterThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 477, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [643] Ref(LessThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 477, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [644] Ref(NotEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 477, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [645] Ref(LikeOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 477, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [646] Ref(IsDistinctFromGrammar)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 477, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [647] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 477, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [648] StringParser("COMPLETION")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 477, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [649] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 477, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [650] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 477, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [651] StringParser("COMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 477, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [652] StringParser("COMPUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 477, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [653] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [654] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [655] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [656] StringParser("CONCATENATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [657] StringParser("CONDITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [658] StringParser("CONDITION_NUMBER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [659] Ref(CrossKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [660] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [661] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [662] Ref(ConditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [663] Ref(NonStandardJoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [664] StringParser("CONF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [665] StringParser("CONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [666] StringParser("CONNECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [667] StringParser("CONNECTION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [668] StringParser("CONSTRAINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [669] StringParser("CONSTRAINT_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [670] StringParser("CONSTRAINT_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [671] StringParser("CONSTRAINT_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [672] StringParser("CONSTRAINTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [673] StringParser("CONSTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [674] StringParser("CONTAINS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [675] StringParser("CONTAINSTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [676] StringParser("CONTINUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [677] StringParser("CONVERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [678] StringParser("CONVERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [679] StringParser("COPY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [680] StringParser("CORR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [681] StringParser("CORRESPONDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [682] StringParser("COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [683] StringParser("COVAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [684] StringParser("COVAR_SAMP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 482, child_count: 10, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [685] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [686] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [687] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 5, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [688] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [689] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [690] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [691] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [692] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 497, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [693] Ref(SpecificKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [694] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [695] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [696] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [697] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [698] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 503, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [699] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 506, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [700] Ref(InstanceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 506, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [701] Ref(StaticKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 506, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [702] Ref(ConstructorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 506, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [703] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 506, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [704] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 506, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [705] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 506, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [706] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [707] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [708] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 508, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [709] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 510, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [710] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 510, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [711] Ref(AssignmentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 510, child_count: 8, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [712] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 518, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [713] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 518, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [714] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 520, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [715] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 520, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [716] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 520, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [717] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 520, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [718] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 520, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [719] Ref(CommentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 520, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [720] Ref(LocationGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 520, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [721] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [722] Ref(ManagedlocationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [723] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 522, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [724] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [725] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [726] Ref(DbpropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [727] Ref(BracketedPropertyListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 9, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [728] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 534, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [729] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 534, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [730] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 534, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [731] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 534, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [732] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 534, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [733] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 534, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [734] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 534, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [735] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 534, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [736] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 536, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [737] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 536, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [738] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 536, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [739] Ref(FunctionDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 536, child_count: 9, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [740] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [741] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [742] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [743] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [744] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [745] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [746] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [747] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [748] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 545, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [749] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 546, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [750] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 549, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [751] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 551, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [752] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 551, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [753] StringParser("CREATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 551, child_count: 8, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [754] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 559, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [755] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 559, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [756] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 559, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [757] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 559, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [758] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 559, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [759] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 559, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [760] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 561, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [761] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 561, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [762] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 564, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [763] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 566, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [764] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [765] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [766] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [767] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 571, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [768] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 571, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [769] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 574, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [770] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [771] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [772] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [773] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [774] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 580, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [775] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 580, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [776] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 580, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [777] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 580, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [778] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 580, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [779] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 584, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [780] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 584, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [781] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 584, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [782] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 584, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [783] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 584, child_count: 7, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [784] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [785] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [786] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [787] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [788] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 594, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [789] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [790] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [791] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [792] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [793] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [794] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [795] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 599, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [796] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 601, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [797] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 601, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [798] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 601, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [799] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 601, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [800] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 603, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [801] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 603, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [802] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 603, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [803] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 603, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [804] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 607, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [805] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 607, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [806] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 607, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [807] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 607, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [808] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 608, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [809] Ref(CreateSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 608, child_count: 7, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [810] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [811] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [812] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [813] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [814] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [815] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [816] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [817] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 617, child_count: 12, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [818] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 629, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [819] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [820] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 634, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [821] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 636, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [822] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 636, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [823] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [824] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [825] Ref(CommentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [826] Ref(CommentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [827] Ref(StoredAsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 638, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [828] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 641, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [829] Ref(PartitionedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 641, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [830] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 641, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [831] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [832] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 646, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [833] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 648, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [834] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 648, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [835] Ref(CommentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 648, child_count: 7, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [836] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [837] Ref(ClusteredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [838] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [839] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 655, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [840] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 658, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [841] Ref(SortedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 658, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [842] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 658, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [843] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [844] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 663, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [845] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 665, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [846] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 665, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [847] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 667, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [848] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 667, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [849] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 667, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [850] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 667, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [851] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 667, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [852] Ref(BucketsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 667, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [853] Ref(StoredAsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 667, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [854] Ref(SkewedByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 667, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [855] Ref(StorageFormatGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 667, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [856] Ref(LocationGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 667, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [857] Ref(TablePropertiesGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 667, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [858] Ref(CommentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 667, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [859] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 669, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [860] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 669, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [861] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [862] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 674, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [863] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 674, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [864] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 678, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [865] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 678, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [866] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 678, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [867] Ref(LocationGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 678, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [868] Ref(TablePropertiesGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 678, child_count: 9, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [869] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 687, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [870] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 687, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [871] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 687, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [872] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 687, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [873] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 690, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [874] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 690, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [875] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 690, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [876] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 692, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [877] Ref(InsteadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 692, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [878] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 692, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 1, _padding: 0 }, // [879] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 694, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [880] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [881] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [882] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [883] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [884] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [885] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 2, _padding: 0 }, // [886] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [887] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [888] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [889] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [890] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [891] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [892] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [893] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 702, child_count: 5, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [894] AnyNumberOf(5 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 707, child_count: 9, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [895] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [896] Ref(ReferencingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [897] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [898] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [899] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [900] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [901] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [902] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [903] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [904] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [905] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 718, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [906] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 718, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [907] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 718, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [908] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 720, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [909] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [910] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [911] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 722, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [912] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [913] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 724, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [914] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 726, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [915] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 728, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [916] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 728, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [917] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 728, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [918] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 730, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [919] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 730, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [920] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 730, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [921] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 733, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [922] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 733, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [923] Ref(EachKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 733, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [924] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [925] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [926] Ref(StatementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [927] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 737, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [928] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 737, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [929] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 740, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [930] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 740, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [931] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 744, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [932] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 744, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [933] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 744, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [934] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 744, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [935] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 744, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [936] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [937] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [938] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [939] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 9, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [940] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [941] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [942] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [943] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [944] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [945] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [946] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [947] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [948] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 758, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [949] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 761, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [950] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 761, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [951] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 761, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [952] StringParser("CREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 761, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [953] StringParser("CREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 761, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [954] StringParser("CREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 761, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [955] StringParser("CROSS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 761, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [956] StringParser("CSV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 761, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [957] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 761, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [958] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 761, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [959] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 763, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [960] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [961] Ref(CubeFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [962] Ref(RollupFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [963] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [964] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [965] StringParser("CUME_DIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [966] StringParser("CURRENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [967] StringParser("CURRENT_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [968] StringParser("CURRENT_DEFAULT_TRANSFORM_GROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [969] StringParser("CURRENT_PATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [970] StringParser("CURRENT_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [971] StringParser("CURRENT_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [972] StringParser("CURRENT_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [973] StringParser("CURRENT_TRANSFORM_GROUP_FOR_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [974] StringParser("CURRENT_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [975] StringParser("CURSOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [976] StringParser("CURSOR_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [977] StringParser("CYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [978] StringParser("DATA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [979] StringParser("DATABASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [980] StringParser("DATABASES")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [981] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [982] RegexParser
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 770, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [983] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 771, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [984] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 771, child_count: 6, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [985] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 777, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [986] Ref(PrimitiveTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 777, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [987] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 777, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [988] Ref(SizedArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 777, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [989] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [990] Ref(MapKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 779, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [991] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 782, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [992] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 785, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [993] Ref(PrimitiveTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 785, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [994] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 785, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [995] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 785, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [996] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 785, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [997] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 787, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [998] Ref(UniontypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 787, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [999] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 790, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1000] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 792, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1001] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 792, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1002] StringParser("DATE")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 792, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1003] MultiStringParser(1 templates)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 792, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1004] Ref(DatePartFunctionName)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 792, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1005] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 793, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1006] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 796, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1007] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 798, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1008] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 800, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1009] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 800, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1010] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 800, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1011] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 802, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1012] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1013] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1014] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1015] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1016] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1017] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1018] StringParser("DATETIME")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1019] MultiStringParser(20 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1020] StringParser("DATETIME_INTERVAL_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1021] StringParser("DATETIME_INTERVAL_PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1022] StringParser("DAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1023] StringParser("DAY_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1024] StringParser("DAY_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1025] StringParser("DAY_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1026] StringParser("DAY_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1027] StringParser("DAYOFMONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1028] StringParser("DAYOFWEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1029] StringParser("DAYOFYEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1030] StringParser("DAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1031] StringParser("DBCC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1032] StringParser("DBPROPERTIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1033] StringParser("DEALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1034] StringParser("DEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1035] StringParser("DECIMAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1036] StringParser("DECLARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1037] StringParser("DEFAULT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 806, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1038] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1039] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1040] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1041] StringParser("DEFAULTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1042] StringParser("DEFERRABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1043] StringParser("DEFERRED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1044] StringParser("DEFINED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1045] StringParser("DEFINER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1046] StringParser("DEGREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1047] StringParser("DELAY_KEY_WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1048] StringParser("DELAYED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1049] StringParser("DELETE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1050] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1051] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1052] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1053] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1054] StringParser("DELIMITED")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1055] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1056] StringParser("DELIMITER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1057] StringParser("DELIMITERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1058] StringParser("DENSE_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1059] StringParser("DENY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1060] StringParser("DEPENDENCY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1061] StringParser("DEPTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1062] StringParser("DEREF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1063] StringParser("DERIVED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1064] StringParser("DESC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1065] StringParser("DESCRIBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1066] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1067] Ref(DescribeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1068] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1069] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1070] StringParser("DESCRIPTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1071] StringParser("DESTROY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1072] StringParser("DESTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1073] StringParser("DETAIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1074] StringParser("DETERMINISTIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1075] StringParser("DIAGNOSTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1076] StringParser("DICTIONARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1077] StringParser("DIRECTORIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1078] StringParser("DIRECTORY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1079] StringParser("DISABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1080] StringParser("DISCONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1081] StringParser("DISK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1082] StringParser("DISPATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1083] StringParser("DISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1084] StringParser("DISTINCTROW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 814, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1085] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1086] Ref(DistributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1087] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 7, _padding: 0 }, // [1088] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 821, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1089] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1090] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1091] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1092] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1093] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1094] Ref(SortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1095] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1096] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1097] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1098] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1099] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1100] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1101] StringParser("DISTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1102] StringParser("DISTRIBUTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1103] StringParser("DIV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1104] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1105] StringParser("DO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1106] StringParser("DOMAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1107] StringParser(".")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1108] StringParser("DOUBLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1109] StringParser("DOW")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 825, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1110] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 827, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1111] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 827, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1112] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 827, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1113] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1114] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1115] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1116] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1117] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1118] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1119] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1120] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1121] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 841, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1122] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 841, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1123] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 841, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1124] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 841, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1125] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 841, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1126] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 841, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1127] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1128] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1129] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1130] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1131] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 845, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1132] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 850, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1133] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 850, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1134] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 850, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1135] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 850, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1136] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 850, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1137] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 850, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1138] StringParser("DROP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 850, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1139] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 854, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1140] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 854, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1141] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 854, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1142] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 854, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1143] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 854, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1144] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 858, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1145] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 858, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1146] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 858, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1147] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 858, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1148] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 858, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1149] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1150] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1151] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1152] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1153] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1154] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1155] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1156] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1157] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1158] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1159] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 871, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1160] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 871, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1161] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 871, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1162] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 871, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1163] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 871, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1164] Ref(PurgeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 871, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1165] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 875, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1166] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 875, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1167] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 875, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1168] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 875, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1169] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 875, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1170] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 880, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1171] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 880, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1172] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 880, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1173] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 880, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1174] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 880, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1175] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 880, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1176] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 884, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1177] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 884, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1178] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 884, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1179] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 884, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1180] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 884, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1181] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 889, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1182] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 889, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1183] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 889, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1184] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 889, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1185] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 889, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1186] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 889, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1187] StringParser("DUMMY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 889, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1188] StringParser("DUMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 889, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1189] StringParser("DYNAMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 889, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1190] StringParser("DYNAMIC_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 889, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1191] StringParser("DYNAMIC_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 889, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1192] StringParser("EACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 889, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1193] StringParser("ELEM_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 889, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1194] StringParser("ELEMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 889, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1195] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 893, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1196] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 893, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1197] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 893, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1198] StringParser("ELSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 893, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1199] StringParser("ELSEIF")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 893, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1200] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 895, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1201] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1202] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1203] Ref(EmptyStructLiteralBracketsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1204] StringParser("ENABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1205] StringParser("ENCLOSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1206] StringParser("ENCODING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1207] StringParser("ENCRYPTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1208] StringParser("END-EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1209] StringParser("}")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1210] StringParser("END")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1211] StringParser("ENUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1212] StringParser("EQUALS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 897, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1213] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1214] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1215] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1216] StringParser("ERRLVL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1217] StringParser("ESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1218] StringParser("ESCAPED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1219] StringParser("EVERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1220] StringParser("EXCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1221] StringParser("EXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1222] StringParser("EXCHANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1223] StringParser("EXCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1224] StringParser("EXCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1225] StringParser("EXCLUSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1226] StringParser("EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1227] StringParser("EXECUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1228] StringParser("EXECUTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1229] StringParser("EXISTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1230] StringParser("EXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1231] StringParser("EXIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1232] StringParser("EXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1233] StringParser("EXPLAIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 899, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1234] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1235] Ref(ExplainKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1236] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1237] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1238] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1239] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1240] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1241] StringParser("EXPORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1242] StringParser("EXPRESSION")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1243] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1244] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 907, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1245] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 907, child_count: 1, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1246] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 908, child_count: 10, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1247] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1248] Ref(LikeExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1249] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1250] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1251] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1252] Ref(InOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1253] Ref(QuantifiedComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1254] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1255] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1256] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1257] Ref(IsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1258] Ref(IsNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1259] Ref(NotNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1260] Ref(CollateGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 923, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1261] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 928, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1262] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 928, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1263] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 928, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1264] Ref(Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 928, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1265] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 928, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1266] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 928, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1267] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 930, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1268] Ref(PatternMatchingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 930, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1269] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 930, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1270] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 934, child_count: 1, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1271] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 935, child_count: 1, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1272] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 936, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1273] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 936, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1274] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 936, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1275] Ref(NotOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 936, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1276] Ref(PriorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 936, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1277] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 938, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1278] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 938, child_count: 1, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1279] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 939, child_count: 1, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1280] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 940, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1281] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 942, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1282] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1283] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1284] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1285] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1286] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1287] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 947, child_count: 1, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1288] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 948, child_count: 1, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1289] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 949, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1290] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 949, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1291] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 949, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 1, _padding: 0 }, // [1292] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 952, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1293] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 954, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1294] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 954, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1295] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 957, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1296] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 957, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1297] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1298] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 961, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1299] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 961, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1300] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 961, child_count: 1, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1301] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1302] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1303] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1304] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1305] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 964, child_count: 9, min_times: 0, first_terminator_idx: 29, terminator_count: 1, _padding: 0 }, // [1306] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1307] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1308] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1309] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 976, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1310] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1311] Ref(LateralColumnAliasExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1312] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1313] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1314] Delimited(5 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 982, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1315] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1316] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1317] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1318] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1319] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1320] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1321] Ref(Expression_D_Potential_Select_Statement_Without_Brackets)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1322] Ref(SingleIdentifierWildcardSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1323] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 989, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1324] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1325] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1326] Ref(MapTypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1327] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 994, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1328] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1329] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1330] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 998, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1331] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 998, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1332] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1333] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1334] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1335] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1336] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1337] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1338] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1339] Ref(ListComprehensionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1340] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1341] Ref(AccessorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 7, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1342] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1343] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1344] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1345] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1346] Ref(TypedStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1347] Ref(ArrayExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1348] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1349] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1350] StringParser("EXTENDED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1351] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1352] StringParser("EXTENSION")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1010, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 1, _padding: 0 }, // [1353] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1012, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1354] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1012, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1355] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1012, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1356] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1012, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1357] StringParser("EXTERNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1012, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1358] StringParser("EXTRACT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1012, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1359] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1012, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1360] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1012, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1361] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1017, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1362] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1017, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1363] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1364] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1365] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1019, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1366] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1367] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1021, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1368] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1022, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1369] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1022, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1370] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1371] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1372] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1373] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1026, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1374] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1026, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1375] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1028, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1376] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1028, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1377] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1028, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1378] StringParser("FETCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1028, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1379] StringParser("FIELDS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1028, child_count: 8, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1380] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1381] Ref(SequencefileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1382] Ref(TextfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1383] Ref(RcfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1384] Ref(OrcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1385] Ref(ParquetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1386] Ref(AvroKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1387] Ref(JsonfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1388] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1389] Ref(InputformatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1390] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1391] Ref(OutputformatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1392] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1393] StringParser("FILE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1394] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1043, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1395] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1396] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 1044, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1397] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1046, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1398] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1046, child_count: 1, min_times: 1, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1399] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1400] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1047, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1401] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1048, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1402] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1048, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1403] StringParser("FILEFORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1048, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1404] StringParser("FILLFACTOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1048, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1405] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1050, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1406] Ref(FilterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1050, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1407] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1053, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1408] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1409] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1410] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1411] StringParser("FILTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1412] StringParser("FINAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1413] StringParser("FIRST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1414] StringParser("FLOAT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1415] StringParser("FLOAT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1416] StringParser("FLOAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1417] StringParser("FLOOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1418] StringParser("FLUSH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1419] StringParser("FOLLOWING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1420] StringParser("FOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1421] StringParser("FORCE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1055, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1422] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1423] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1424] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1425] StringParser("FOREIGN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1426] StringParser("FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1427] StringParser("FORMATTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1428] StringParser("FORTRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1429] StringParser("FORWARD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1430] StringParser("FOUND")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1431] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1059, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1432] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1059, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1433] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1061, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1434] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1063, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1435] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1436] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1437] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1438] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1067, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1439] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1440] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1441] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1442] Ref(UnboundedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1443] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1444] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1073, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1445] Ref(PrecedingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1073, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1446] Ref(FollowingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1073, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1447] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1077, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1448] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1077, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1449] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1077, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1450] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1451] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1452] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1453] StringParser("FREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1454] StringParser("FREETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1455] StringParser("FREETEXTTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1456] StringParser("FREEZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1457] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1081, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1458] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1081, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1459] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1083, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1460] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1083, child_count: 15, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1461] OneOf(15 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1462] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1463] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1098, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1464] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1465] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1466] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1467] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1468] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1469] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1470] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1471] Ref(DistributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1472] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1473] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1474] Ref(SortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1475] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1476] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1477] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1478] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1479] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1480] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1481] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1482] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1483] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1484] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1485] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1486] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 8, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1487] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1116, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1488] Ref(PreTableFunctionKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1116, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1489] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1118, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1490] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1491] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1492] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1493] Ref(TemporalQuerySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1121, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1494] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1122, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1495] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1126, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1496] Ref(FromClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1126, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1497] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1126, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1498] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1126, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1499] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1126, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1500] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1129, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1501] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1129, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1502] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1129, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1503] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1129, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1504] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1129, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1505] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1130, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1506] Ref(LateralViewClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1130, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1507] Ref(PostTableExpressionGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1130, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [1508] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1132, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [1509] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1135, child_count: 6, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [1510] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 2, _padding: 0 }, // [1511] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1512] Ref(MLTableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1513] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1514] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1147, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1515] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1147, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1516] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1517] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1518] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1519] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1151, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1520] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1151, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1521] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1151, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [1522] Meta(conditional: indented_joins=True -> indent)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1151, child_count: 1, min_times: 0, first_terminator_idx: 33, terminator_count: 2, _padding: 0 }, // [1523] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1152, child_count: 1, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [1524] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1153, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [1525] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [1526] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [1527] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [1528] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [1529] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [1530] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1157, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [1531] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [1532] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [1533] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1159, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [1534] Meta(conditional: indented_joins=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1535] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1161, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [1536] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1161, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1537] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1163, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [1538] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1165, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [1539] Ref(OverwriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1165, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [1540] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1167, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1541] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1172, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [1542] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1172, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [1543] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1172, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [1544] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1172, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [1545] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1172, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 1, _padding: 0 }, // [1546] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1172, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [1547] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1172, child_count: 6, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [1548] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1178, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1549] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1550] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1551] Ref(DirectoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1552] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1553] Ref(RowFormatClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1554] Ref(StoredAsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 1, _padding: 0 }, // [1555] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1556] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 6, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1557] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1185, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [1558] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1185, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [1559] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1185, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [1560] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1185, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [1561] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1185, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [1562] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1185, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1563] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1187, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 1, _padding: 0 }, // [1564] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1187, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [1565] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1187, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1566] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1187, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1567] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1189, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1568] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1189, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1569] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1191, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1570] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1191, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1571] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1572] Ref(FromInsertSourceClauseSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 1, min_times: 1, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1573] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1194, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1574] Ref(FromInsertClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1194, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1575] StringParser("FROM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1194, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1576] StringParser("FULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1194, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1577] StringParser("FULLTEXT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1194, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1578] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1194, child_count: 11, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1579] AnyNumberOf(11 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1205, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1580] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1205, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1581] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1208, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1582] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1208, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1583] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1208, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1584] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1208, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1585] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1586] Ref(TrimParametersGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1212, child_count: 1, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1587] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1588] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1589] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1590] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1213, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1591] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1216, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1592] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1593] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1594] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1595] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1596] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1597] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1220, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1598] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1220, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1599] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1222, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1600] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1222, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1601] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1602] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1603] Ref(AggregateOrderByClause)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1604] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1226, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1605] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1226, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1606] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1226, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1607] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1229, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1608] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1609] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1610] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1611] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1612] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1613] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1235, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1614] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1235, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1615] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1235, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1616] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1235, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1617] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1235, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1618] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1235, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1619] Ref(EmptyStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1235, child_count: 1, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1620] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1236, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1621] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1239, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1622] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1239, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1623] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1242, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1624] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1242, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1625] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1242, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1626] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1244, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1627] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1244, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1628] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1244, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1629] StringParser("FUNCTION")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1244, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1630] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1244, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1631] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1246, child_count: 1, min_times: 0, first_terminator_idx: 38, terminator_count: 1, _padding: 0 }, // [1632] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1247, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1633] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1249, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1634] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1249, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1635] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1249, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1636] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1249, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 1, _padding: 0 }, // [1637] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1251, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [1638] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1251, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [1639] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1251, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [1640] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1251, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1641] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1253, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1642] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1643] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1644] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1257, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1645] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1646] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1647] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1648] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1649] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1261, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1650] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1263, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1651] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1263, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1652] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1263, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1653] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1263, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1654] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1266, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1655] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1656] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1657] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1271, child_count: 1, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1658] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1272, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1659] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1274, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1660] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1274, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1661] Ref(DateTimeFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1274, child_count: 5, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1662] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1663] StringParser("ROW")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1664] Ref(RowFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1665] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1666] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1667] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1282, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1668] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1284, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1669] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1286, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1670] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1286, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1671] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1286, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1672] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1288, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1673] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1290, child_count: 1, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1674] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1291, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1675] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1676] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1677] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1678] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1679] Ref(PostFunctionGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1680] StringParser("FUNCTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1681] StringParser("FUSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1682] StringParser("FUTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1683] StringParser("G")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1684] StringParser("GENERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1685] StringParser("GENERATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1686] StringParser("GET")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1687] TypedParser("glob_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1688] StringParser("GLOBAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1689] StringParser("GO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1690] StringParser("GOTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1691] StringParser("GRANT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 6, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1692] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1693] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 4, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1694] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1303, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1695] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1306, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1696] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1306, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1697] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1306, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1698] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1306, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1699] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1308, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1700] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1308, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1701] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1308, child_count: 4, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1702] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1703] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1704] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1705] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1706] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1707] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1708] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1709] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1312, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1710] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1315, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1711] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1318, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1712] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1318, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1713] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1318, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1714] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1318, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1715] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1321, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1716] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1321, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1717] Ref(AdminKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1321, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1718] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1321, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1719] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1324, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1720] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1324, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1721] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1324, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1722] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1324, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1723] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1327, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1724] Ref(GrantedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1327, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1725] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1327, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1726] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1330, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1727] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1330, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1728] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1330, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1729] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1330, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1730] StringParser("GRANTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1330, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1731] StringParser("GRANTS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1330, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1732] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1332, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1733] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1332, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1734] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1332, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1735] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1332, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1736] StringParser("GREATEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1332, child_count: 5, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1737] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1337, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1738] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1337, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1739] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1337, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1740] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1741] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1742] Ref(GroupingSetsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1743] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1744] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 1, _padding: 0 }, // [1745] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1345, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1746] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1348, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1747] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1348, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1748] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1348, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1749] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1348, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [1750] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1348, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1751] Ref(GroupingSetsClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1348, child_count: 5, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1752] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1353, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1753] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1355, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1754] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1755] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1756] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1757] Ref(DistributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1758] Ref(SortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1759] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1760] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1761] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1762] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1763] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1359, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1764] StringParser("GROUP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 1, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1765] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1360, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 1, _padding: 0 }, // [1766] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1362, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1767] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1366, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1768] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1366, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1769] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1366, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1770] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1366, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1771] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1368, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1772] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1368, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1773] StringParser("GROUPING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1368, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1774] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1371, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1775] Ref(GroupingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1371, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1776] Ref(SetsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1371, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1777] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1374, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1778] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1376, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1779] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1378, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1780] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1378, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1781] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1378, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1782] StringParser("HANDLER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1378, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1783] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1382, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1784] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1382, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1785] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1384, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1786] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1387, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1787] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1387, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1788] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1789] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1393, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1790] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1791] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1792] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1793] Ref(DistributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1794] Ref(SortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1795] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1796] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1797] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1798] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1799] StringParser("HAVING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1800] StringParser("HEADER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1801] StringParser("HEAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1802] StringParser("HIERARCHY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1803] StringParser("HIGH_PRIORITY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 1, _padding: 0 }, // [1804] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1805] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1806] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1807] Ref(BackQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1808] Ref(HiveVariableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [1809] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1810] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1403, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1811] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1812] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1813] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1814] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1815] TypedParser("hive_variable")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1816] StringParser("HOLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1817] StringParser("HOLD_DDLTIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1818] StringParser("HOLDLOCK")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1819] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1820] StringParser("HOST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1821] StringParser("HOSTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1822] StringParser("HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1823] StringParser("HOUR_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1824] StringParser("HOUR_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1825] StringParser("HOUR_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1826] StringParser("HOURS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1827] StringParser("IDENTIFIED")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1828] Token("identifier")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1829] StringParser("IDENTITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1830] StringParser("IDENTITY_INSERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1831] StringParser("IDENTITYCOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1832] StringParser("IDXPROPERTIES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1833] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1834] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1835] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1410, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1836] StringParser("IF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1837] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1413, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1838] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1413, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1839] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1413, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1840] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1413, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1841] StringParser("IGNORE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1413, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1842] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1415, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1843] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1844] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1845] Ref(RespectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1846] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1847] StringParser("ILIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1848] StringParser("IMMEDIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1849] StringParser("IMMUTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1850] StringParser("IMPLEMENTATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1851] StringParser("IMPLICIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1852] StringParser("IMPORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1853] StringParser("IMPORTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1854] StringParser("IN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1855] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1420, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1856] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1857] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1858] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1423, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1859] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1426, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1860] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1428, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1861] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1430, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1862] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1430, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1863] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1430, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1864] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1430, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1865] Ref(HiveVariableSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1430, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1866] StringParser("INCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1430, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1867] StringParser("INCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1430, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1868] StringParser("INCREMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1430, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1869] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1432, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1870] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1432, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1871] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1872] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1873] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1874] StringParser("INDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1875] StringParser("INDEXES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1876] StringParser("INDICATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1877] StringParser("INFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1878] StringParser("INFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1879] StringParser("INHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1880] StringParser("INHERITS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1881] StringParser("INITIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1882] StringParser("INITIALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1883] StringParser("INITIALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1884] StringParser("INNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1885] StringParser("INOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1886] StringParser("INPATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1887] StringParser("INPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1888] StringParser("INPUTDRIVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1889] StringParser("INPUTFORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1890] StringParser("INSENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1891] StringParser("INSERT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1434, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1892] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1436, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1893] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1436, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1894] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1438, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1895] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1896] Ref(OverwriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1897] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 5, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1898] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1447, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1899] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1447, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1900] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1447, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1901] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1447, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1902] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1447, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1903] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1447, child_count: 6, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1904] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1453, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1905] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1906] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1907] Ref(DirectoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1908] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1909] Ref(RowFormatClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1910] Ref(StoredAsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1911] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 6, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1912] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1913] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1914] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1915] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1916] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1917] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1918] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1919] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1920] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1921] StringParser("INSERT_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1922] StringParser("INSTANCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1923] StringParser("INSTANTIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1924] StringParser("INSTEAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1925] StringParser("INT1")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1926] StringParser("INT2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1927] StringParser("INT3")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1928] StringParser("INT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1929] StringParser("INT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1930] StringParser("INT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1931] StringParser("INTEGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1932] StringParser("INTEGRATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1933] StringParser("INTEGRATIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1934] StringParser("INTERSECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1935] StringParser("INTERSECTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1936] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1464, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1937] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1464, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1938] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1465, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1939] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1468, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1940] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1471, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1941] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1471, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1942] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1471, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1943] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1474, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1944] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1474, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1945] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1474, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1946] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1476, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1947] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1476, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1948] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1476, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1949] StringParser("INTERVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1476, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1950] StringParser("INTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1476, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1951] StringParser("INVOKER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1476, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1952] StringParser("IREGEXP")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1476, child_count: 5, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1953] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1954] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1955] Ref(NanLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1956] Ref(UnknownLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1957] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1958] Ref(NormalizedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1959] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1960] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1961] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1962] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1963] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1964] StringParser("IS")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1965] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1966] StringParser("ISAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1967] StringParser("ISNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1968] StringParser("ISOLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1969] StringParser("ITEMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1970] StringParser("ITERATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1971] StringParser("JAR")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1485, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1972] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1488, child_count: 7, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1973] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1974] Ref(ConditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1975] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1976] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1495, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1977] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1496, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1978] Ref(NestedJoinGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1496, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1979] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1980] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1981] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1500, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1982] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1502, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1983] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1502, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1984] Ref(JoinUsingConditionGrammar)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1502, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1985] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1502, child_count: 6, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1986] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1987] Ref(UnconditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1988] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1989] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1508, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1990] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1991] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1992] Ref(ExtendedNaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1993] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1994] StringParser("JOIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1995] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1514, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1996] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1997] Ref(SemiKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1998] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [1999] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2000] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2001] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2002] Meta(conditional: indented_on_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2003] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1521, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2004] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2005] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2006] Meta(conditional: indented_on_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2007] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1526, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2008] Ref(InnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1526, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2009] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1528, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2010] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2011] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2012] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2013] Ref(RightKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2014] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2015] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1535, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2016] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1535, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2017] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1538, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2018] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2019] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2020] StringParser("JSON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2021] StringParser("JSONFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2022] StringParser("K")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2023] StringParser("KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2024] StringParser("KEY_MEMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2025] StringParser("KEY_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2026] StringParser("KEYS")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2027] Token("keyword")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2028] StringParser("KILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2029] StringParser("LANCOMPILER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2030] StringParser("LANGUAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2031] StringParser("LARGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2032] StringParser("LAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2033] StringParser("LAST_INSERT_ID")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2034] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2035] StringParser("LATERAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 8, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2036] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2037] Ref(LateralKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2038] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1548, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2039] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2040] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1548, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2041] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2042] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1550, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2043] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1550, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2044] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1552, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2045] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1552, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2046] StringParser("LEADING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1552, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2047] StringParser("LEAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1552, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2048] StringParser("LEAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1552, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2049] StringParser("LEFT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1552, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2050] StringParser("LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1552, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2051] StringParser("LESS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1552, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2052] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2053] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2054] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2055] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2056] StringParser("LEVEL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2057] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1557, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2058] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1559, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2059] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1559, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2060] Ref(LikeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1559, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2061] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1559, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2062] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1561, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2063] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1561, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2064] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1561, child_count: 5, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2065] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2066] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2067] Ref(RlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2068] Ref(IlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2069] Ref(RegexpKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2070] Ref(IregexpKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1566, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2071] StringParser("LIKE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1566, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2072] TypedParser("like_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 5, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2073] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1571, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2074] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1571, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2075] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1573, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2076] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2077] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2078] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2079] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2080] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1579, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2081] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1581, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2082] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2083] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1583, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2084] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1585, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2085] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1585, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2086] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1585, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2087] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1587, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2088] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1587, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2089] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1587, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2090] StringParser("LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1587, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2091] StringParser("LINENO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1587, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2092] StringParser("LINES")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1587, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2093] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1587, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2094] StringParser("LISTEN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1587, child_count: 10, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2095] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2096] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2097] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2098] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2099] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2100] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2101] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2102] Ref(HiveVariableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2103] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2104] Ref(TypedArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2105] Ref(ObjectLiteralSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2106] Token("literal")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2107] Token("literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2108] StringParser("LN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2109] StringParser("LOAD")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2110] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2111] StringParser("LOCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2112] StringParser("LOCALTIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2113] StringParser("LOCALTIMESTAMP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1597, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2114] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2115] Ref(LocationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2116] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2117] StringParser("LOCATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2118] StringParser("LOCATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2119] StringParser("LOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2120] StringParser("LOCKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2121] StringParser("LOGICAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2122] StringParser("LOGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2123] StringParser("LOGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2124] StringParser("LONG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2125] StringParser("LONGBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2126] StringParser("LONGTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2127] StringParser("LOOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2128] StringParser("LOW_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2129] StringParser("LOWER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2130] StringParser("M")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1599, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2131] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2132] Ref(MlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2133] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2134] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2135] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2136] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1609, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2137] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1609, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2138] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1609, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2139] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2140] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2141] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2142] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2143] StringParser("MACRO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2144] StringParser("MANAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2145] StringParser("MANAGEDLOCATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2146] StringParser("MAP")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2147] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2148] StringParser("MAPJOIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2149] StringParser("MASKING")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2150] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2151] StringParser("MATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2152] StringParser("MATCHED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2153] StringParser("MATERIALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2154] StringParser("MAX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2155] StringParser("MAX_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2156] StringParser("MAXEXTENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2157] StringParser("MAXVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2158] StringParser("MEDIUMBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2159] StringParser("MEDIUMINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2160] StringParser("MEDIUMTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2161] StringParser("MEMBER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2162] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1614, child_count: 5, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2163] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1619, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2164] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1619, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2165] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1619, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2166] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1619, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2167] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1621, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2168] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1621, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2169] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1621, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2170] StringParser("MERGE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1621, child_count: 2, min_times: 1, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2171] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1623, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2172] Ref(MergeMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1623, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2173] Ref(MergeNotMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1623, child_count: 7, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2174] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1630, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2175] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1630, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2176] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1630, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2177] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1632, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2178] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1632, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2179] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1632, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2180] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1632, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2181] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2182] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2183] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 8, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2184] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1642, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2185] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1642, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2186] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1642, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2187] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1642, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2188] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2189] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2190] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2191] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2192] Ref(MergeInsertClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 12, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2193] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2194] Ref(MergeIntoLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2195] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2196] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2197] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2198] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2199] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1662, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2200] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1662, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2201] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1662, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2202] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1664, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2203] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2204] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2205] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2206] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1669, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2207] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1669, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2208] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1669, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2209] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1669, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2210] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1669, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2211] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1669, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2212] Ref(MergeMatchSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1669, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2213] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2214] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2215] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2216] StringParser("MESSAGE_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2217] StringParser("MESSAGE_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2218] StringParser("MESSAGE_TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2219] StringParser("METADATA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2220] StringParser("METHOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2221] StringParser("MIDDLEINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2222] StringParser("MILLISECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2223] StringParser("MIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2224] StringParser("MIN_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2225] StringParser("MINUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2226] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2227] StringParser("MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2228] StringParser("MINUTE_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2229] StringParser("MINUTE_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2230] StringParser("MINUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2231] StringParser("MINVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2232] StringParser("ML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2233] StringParser("MLSLABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2234] StringParser("MOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2235] StringParser("MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2236] StringParser("MODEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2237] StringParser("MODIFIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2238] StringParser("MODIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2239] StringParser("MODULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2240] StringParser("%")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2241] StringParser("MONITOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2242] StringParser("MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2243] StringParser("MONTHNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2244] StringParser("MONTHS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2245] StringParser("MORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2246] StringParser("MOVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2247] StringParser("MSCK")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 5, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2248] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1678, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2249] Ref(MsckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1678, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2250] Ref(RepairKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1678, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2251] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1678, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2252] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1678, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2253] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1680, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2254] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1683, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2255] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1683, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2256] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1683, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2257] Ref(SyncKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1683, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2258] Ref(PartitionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1683, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2259] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2260] Ref(MsckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2261] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2262] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1687, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2263] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1689, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2264] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1692, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2265] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1692, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2266] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1692, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2267] Ref(SyncKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1692, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2268] Ref(PartitionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1692, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2269] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1692, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2270] StringParser("MULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1692, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2271] StringParser("MUMPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1692, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2272] StringParser("MYISAM")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1692, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2273] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1692, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2274] StringParser("NAME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1692, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2275] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1695, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2276] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1695, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2277] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1695, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2278] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1697, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2279] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1697, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2280] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1700, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2281] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1700, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2282] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1704, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2283] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1704, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2284] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1706, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2285] Ref(NamedWindowExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1706, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2286] StringParser("NAMES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1706, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2287] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1706, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2288] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1706, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2289] StringParser("NATIONAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1706, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2290] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2291] Ref(NaturalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2292] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2293] StringParser("NATURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2294] StringParser("NCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2295] StringParser("NCLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2296] StringParser("-")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2297] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2298] StringParser("NESTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2299] StringParser("NEW")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2300] Token("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2301] StringParser("NEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2302] StringParser("NO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2303] StringParser("NO_DROP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2304] StringParser("NO_WRITE_TO_BINLOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2305] StringParser("NOAUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2306] StringParser("NOCACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2307] StringParser("NOCHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2308] StringParser("NOCOMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2309] StringParser("NOCREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2310] StringParser("NOCREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2311] StringParser("NOCREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2312] StringParser("NOCYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2313] StringParser("NOINHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2314] StringParser("NOLOGIN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 6, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2315] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2316] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2317] Ref(UnorderedSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2318] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2319] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1717, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2320] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2321] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1720, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2322] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1723, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2323] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1723, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2324] Ref(BracketedSetExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1723, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2325] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1723, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2326] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1727, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2327] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1727, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2328] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1727, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2329] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1727, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2330] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1727, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2331] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1730, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2332] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1730, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2333] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1732, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2334] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2335] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2336] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2337] StringParser("NONCLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2338] StringParser("NONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2339] StringParser("NOORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2340] StringParser("NORELY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2341] StringParser("NORMALIZE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2342] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2343] StringParser("NORMALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2344] StringParser("NOSCAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2345] StringParser("NOSUPERUSER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2346] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2347] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1737, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2348] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2349] Ref(RawNotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2350] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1739, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2351] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2352] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2353] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2354] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2355] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2356] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2357] StringParser("NOTHING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2358] StringParser("NOTIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2359] StringParser("NOTNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2360] StringParser("NOVALIDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2361] StringParser("NOWAIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2362] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2363] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2364] StringParser("NULLABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2365] StringParser("NULLIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2366] StringParser("NULLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2367] StringParser("NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2368] StringParser("NUMERIC")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2369] TypedParser("numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2370] StringParser("OBJECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1741, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2371] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2372] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2373] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2374] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2375] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1747, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2376] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2377] Ref(ObjectLiteralElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1749, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2378] StringParser("{")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2379] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1751, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2380] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1751, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2381] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1753, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2382] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1753, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2383] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1753, child_count: 12, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2384] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2385] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2386] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2387] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2388] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2389] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2390] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2391] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2392] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2393] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2394] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2395] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2396] StringParser("OBJECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2397] StringParser("OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2398] StringParser("OCTETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2399] StringParser("OF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2400] StringParser("OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2401] StringParser("OFFLINE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2402] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1768, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2403] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1768, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2404] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1770, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2405] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1770, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2406] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1771, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2407] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1771, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2408] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2409] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2410] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2411] StringParser("OFFSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2412] StringParser("OFFSETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2413] StringParser("OIDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2414] StringParser("OLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2415] StringParser("ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2416] StringParser("ONLINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2417] StringParser("ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2418] StringParser("OPEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2419] StringParser("OPENDATASOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2420] StringParser("OPENQUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2421] StringParser("OPENROWSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2422] StringParser("OPENXML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2423] StringParser("OPERATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2424] StringParser("OPERATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2425] StringParser("OPERATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2426] StringParser("OPTIMIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2427] StringParser("OPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2428] StringParser("OPTIONALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2429] StringParser("OPTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2430] StringParser("OR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2431] StringParser("OR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2432] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1775, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2433] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1775, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2434] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1775, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2435] StringParser("ORC")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1775, child_count: 5, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2436] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2437] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2438] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 2, _padding: 0 }, // [2439] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2440] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1786, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2441] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2442] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2443] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2444] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1789, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2445] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1791, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2446] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1791, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2447] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1791, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2448] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2449] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1793, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2450] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2451] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2452] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2453] Ref(WithFillSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2454] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [2455] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 7, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2456] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2457] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2458] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2459] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2460] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2461] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2462] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2463] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2464] StringParser("ORDER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2465] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1804, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2466] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1804, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2467] Ref(NoorderKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1804, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2468] StringParser("ORDERING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1804, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2469] StringParser("ORDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1804, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2470] StringParser("OTHERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1804, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2471] StringParser("OUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1804, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2472] StringParser("OUTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1804, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2473] StringParser("OUTFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1804, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2474] StringParser("OUTPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1804, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2475] StringParser("OUTPUTDRIVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1804, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2476] StringParser("OUTPUTFORMAT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1804, child_count: 5, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2477] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1809, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2478] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1809, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2479] Ref(OverKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1809, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2480] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1811, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2481] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1811, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2482] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1814, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2483] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1814, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2484] StringParser("OVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1814, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2485] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2486] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2487] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1818, child_count: 1, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2488] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 5, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2489] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2490] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2491] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2492] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2493] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2494] StringParser("OVERLAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2495] StringParser("OVERLAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2496] StringParser("OVERRIDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2497] StringParser("OVERWRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2498] StringParser("OWNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2499] StringParser("OWNERSHIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2500] StringParser("PACK_KEYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2501] StringParser("PAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2502] StringParser("PARAMETER")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2503] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2504] StringParser("?")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2505] StringParser("PARAMETER_MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2506] StringParser("PARAMETER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2507] StringParser("PARAMETER_ORDINAL_POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2508] StringParser("PARAMETER_SPECIFIC_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2509] StringParser("PARAMETER_SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2510] StringParser("PARAMETER_SPECIFIC_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2511] StringParser("PARAMETERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2512] StringParser("PARQUET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2513] StringParser("PARTIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2514] StringParser("PARTIALSCAN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1824, child_count: 5, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2515] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1829, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2516] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1829, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2517] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1829, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2518] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1831, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2519] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1834, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2520] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1836, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2521] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1836, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2522] StringParser("PARTITION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1836, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2523] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1838, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2524] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1838, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2525] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1841, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2526] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1843, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2527] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1845, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2528] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1845, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2529] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1847, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2530] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1847, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2531] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1847, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2532] StringParser("PARTITIONED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1847, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2533] StringParser("PARTITIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1847, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2534] StringParser("PASCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1847, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2535] StringParser("PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1847, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2536] StringParser("PATH")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1847, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2537] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1849, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2538] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1851, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2539] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1851, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2540] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2541] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2542] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2543] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2544] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2545] StringParser("PCTFREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2546] StringParser("PERCENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2547] StringParser("PERCENT_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2548] StringParser("PERCENTILE_CONT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2549] StringParser("PERCENTILE_DISC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2550] StringParser("PIPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2551] StringParser("|")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2552] StringParser("PLACING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2553] StringParser("PLAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2554] StringParser("PLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2555] StringParser("PLUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2556] StringParser("+")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2557] StringParser("POLICY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2558] StringParser("POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2559] StringParser("+")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1853, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2560] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1855, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2561] Ref(OverClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1855, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2562] Ref(FilterClauseGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1855, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2563] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1855, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2564] StringParser("POSTFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1855, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2565] StringParser("POWER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1855, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2566] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1855, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2567] StringParser("PRECEDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1855, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2568] StringParser("PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1855, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2569] StringParser("PREFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1855, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2570] StringParser("PREORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1855, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2571] StringParser("PREPARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1855, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2572] StringParser("PREPARED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1855, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2573] StringParser("PRESERVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1855, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2574] StringParser("PRETTY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1855, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2575] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2576] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2577] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2578] StringParser("PRIMARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 16, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2579] OneOf(16 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2580] Ref(TinyintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2581] Ref(SmallintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2582] Ref(IntKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2583] Ref(IntegerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2584] Ref(BigintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2585] Ref(BooleanKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2586] Ref(FloatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2587] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2588] Ref(DoubleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2589] Ref(PrecisionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2590] Ref(StringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2591] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2592] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2593] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1877, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2594] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2595] Ref(DecimalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2596] Ref(DecKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2597] Ref(NumericKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2598] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2599] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2600] Ref(VarcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2601] Ref(CharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2602] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2603] StringParser("PRINCIPALS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2604] StringParser("PRINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2605] StringParser("PRIOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2606] StringParser("PRIVILEGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2607] StringParser("PROC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2608] StringParser("PROCEDURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2609] StringParser("PROCEDURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2610] StringParser("PROCEDURES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2611] StringParser("PROCESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2612] StringParser("PROCESSLIST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1880, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2613] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1883, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2614] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1883, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2615] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1883, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2616] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1883, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2617] StringParser("PROTECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1883, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2618] StringParser("PUBLIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1883, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2619] StringParser("PURGE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1883, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2620] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1885, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2621] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1885, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2622] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1885, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2623] StringParser("QUALIFY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1885, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2624] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1888, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2625] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1888, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2626] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1891, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2627] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1891, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2628] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1891, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2629] Ref(SomeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1891, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2630] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2631] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2632] StringParser("QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2633] StringParser("QUOTE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2634] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2635] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2636] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2637] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2638] TypedParser("back_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2639] StringParser("RAID0")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2640] StringParser("RAISERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2641] StringParser("RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2642] StringParser("RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2643] StringParser("=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2644] StringParser(">")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2645] StringParser("RAW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2646] StringParser("<")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2647] StringParser("!")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2648] StringParser("RCFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2649] StringParser("READ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2650] StringParser("READONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2651] StringParser("READS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2652] StringParser("READTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2653] StringParser("REAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2654] StringParser("REBUILD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2655] StringParser("RECHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2656] StringParser("RECONFIGURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2657] StringParser("RECORDREADER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2658] StringParser("RECORDWRITER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2659] StringParser("RECURSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2660] StringParser("REDUCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2661] StringParser("REF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1897, child_count: 5, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2662] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1902, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2663] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1902, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2664] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1902, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2665] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1902, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2666] Ref(ReferenceMatchGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1902, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2667] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1904, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2668] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1907, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2669] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1907, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2670] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1907, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2671] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1907, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2672] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2673] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2674] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2675] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2676] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1912, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2677] Ref(MatchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1912, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2678] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2679] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2680] Ref(PartialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2681] Ref(SimpleKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1915, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2682] StringParser("REFERENCE_USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1915, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2683] StringParser("REFERENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1915, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2684] StringParser("REFERENCING")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 5, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2685] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1920, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2686] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1920, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2687] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1920, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2688] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1922, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2689] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1922, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2690] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1922, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2691] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1924, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2692] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1924, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2693] Ref(ActionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1924, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2694] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2695] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2696] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2697] StringParser("REGEXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2698] StringParser("REGR_AVGX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2699] StringParser("REGR_AVGY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2700] StringParser("REGR_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2701] StringParser("REGR_INTERCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2702] StringParser("REGR_R2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2703] StringParser("REGR_SLOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2704] StringParser("REGR_SXX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2705] StringParser("REGR_SXY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2706] StringParser("REGR_SYY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2707] StringParser("REINDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2708] StringParser("RELATIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2709] StringParser("RELEASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2710] StringParser("RELOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2711] StringParser("RELY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2712] StringParser("RENAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2713] StringParser("REPAIR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2714] StringParser("REPEAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2715] StringParser("REPEATABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2716] StringParser("REPLACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2717] StringParser("REPLICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2718] StringParser("REQUIRE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2719] StringParser("RESET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2720] StringParser("RESIGNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2721] StringParser("RESOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2722] StringParser("RESPECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2723] StringParser("RESTART")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2724] StringParser("RESTORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2725] StringParser("RESTRICT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2726] StringParser("RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2727] StringParser("RETURN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2728] StringParser("RETURNED_CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2729] StringParser("RETURNED_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2730] StringParser("RETURNED_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2731] StringParser("RETURNED_SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2732] StringParser("RETURNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2733] StringParser("REVOKE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1926, child_count: 6, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2734] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1932, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2735] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1932, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2736] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1935, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2737] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1935, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2738] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1935, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2739] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1935, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2740] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2741] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2742] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2743] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2744] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2745] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1944, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2746] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1944, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2747] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1944, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2748] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2749] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2750] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2751] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2752] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2753] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2754] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2755] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2756] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2757] StringParser("REWRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2758] StringParser("RIGHT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2759] StringParser("RLIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2760] StringParser("ROLE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2761] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2762] StringParser("ROLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2763] StringParser("ROLLBACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2764] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2765] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2766] StringParser("ROUTINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2767] StringParser("ROUTINE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2768] StringParser("ROUTINE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2769] StringParser("ROUTINE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2770] StringParser("ROUTINES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2771] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1951, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2772] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1951, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2773] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1951, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2774] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1953, child_count: 6, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2775] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1959, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2776] Ref(DelimitedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1959, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2777] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2778] Ref(FieldsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2779] Ref(TerminatedByGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1962, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2780] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2781] Ref(EscapedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2782] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2783] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1965, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2784] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1968, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2785] Ref(CollectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1968, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2786] Ref(ItemsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1968, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2787] Ref(TerminatedByGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1968, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2788] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1971, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2789] Ref(MapKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1971, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2790] Ref(KeysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1971, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2791] Ref(TerminatedByGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1971, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2792] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1973, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2793] Ref(LinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1973, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2794] Ref(TerminatedByGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1973, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2795] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1977, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2796] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1977, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2797] Ref(DefinedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1977, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2798] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1977, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2799] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1977, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2800] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1980, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2801] Ref(SerdeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1980, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2802] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1980, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2803] Ref(SerdePropertiesGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1980, child_count: 1, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2804] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1981, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2805] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2806] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1986, child_count: 1, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2807] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2808] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2809] StringParser("ROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2810] StringParser("ROW_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2811] StringParser("ROW_NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2812] StringParser("ROWCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2813] StringParser("ROWGUIDCOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2814] StringParser("ROWID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2815] StringParser("ROWNUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2816] StringParser("ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2817] StringParser("RULE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2818] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1990, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2819] Ref(TablesampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1990, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2820] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2821] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 6, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2822] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2002, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2823] Ref(BucketKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2002, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2824] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2002, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2825] Ref(OutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2002, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2826] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2002, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2827] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2002, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2828] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2829] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2830] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2006, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2831] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2006, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2832] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2006, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2833] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2008, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2834] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2008, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2835] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2836] Ref(PercentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2837] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2838] RegexParser
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2839] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2840] StringParser("SAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2841] StringParser("SAVEPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2842] StringParser("SCALE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2843] StringParser("SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2844] StringParser("SCHEMA_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2845] StringParser("SCHEMAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2846] StringParser("SCOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2847] StringParser("SCOPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2848] StringParser("SCOPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2849] StringParser("SCOPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2850] StringParser("SCROLL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2851] StringParser("SEARCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2852] StringParser("SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2853] StringParser("SECOND_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2854] StringParser("SECONDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2855] StringParser("SECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2856] StringParser("SECURITY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2010, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2857] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2012, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2858] Ref(WildcardExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2012, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2859] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2014, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2860] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2014, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2861] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2014, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2862] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2016, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2863] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2016, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2864] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 2016, child_count: 5, min_times: 0, first_terminator_idx: 45, terminator_count: 4, _padding: 0 }, // [2865] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2021, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2866] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2021, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2867] Ref(SelectClauseModifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 2021, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2868] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2869] Ref(SelectClauseElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2870] Ref(SelectClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2023, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2871] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2025, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2872] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2025, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2873] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2025, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2874] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2027, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2875] Ref(DistributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2027, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2876] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2027, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2877] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2029, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2878] Ref(SortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2029, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2879] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2029, child_count: 10, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2880] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 2039, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2881] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::PrecededBy, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2040, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2882] PrecededBy(preceding_sequences=(('IS', 'DISTINCT'), ('IS', 'NOT', 'DISTINCT')))
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2040, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2883] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2040, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2884] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2885] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2886] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2887] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2044, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2888] Ref(DistributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2044, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2889] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2044, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2890] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2046, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2891] Ref(SortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2046, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2892] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2046, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2893] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2894] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2895] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2896] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2897] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2898] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2899] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2048, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2900] StringParser("SELECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 15, min_times: 0, first_terminator_idx: 49, terminator_count: 3, _padding: 0 }, // [2901] Sequence(15 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2902] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2903] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2904] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2905] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2906] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2907] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2908] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2909] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2910] Ref(OffsetClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2911] Ref(FetchClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2912] Ref(ClusterByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2913] Ref(DistributeByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2914] Ref(SortByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2915] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2916] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2917] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2918] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2919] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2063, child_count: 4, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2920] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2921] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2069, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2922] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2072, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2923] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2072, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2924] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2074, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2925] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2926] Ref(WithCompoundNonSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2927] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2928] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2080, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2929] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2080, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2930] StringParser("SELF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2080, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2931] StringParser("SEMI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2080, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2932] StringParser(";")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2080, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2933] StringParser("SENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2080, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2934] StringParser("SEPARATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2080, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2935] StringParser("SEQUENCE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2080, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2936] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2082, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2937] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2938] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2939] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2940] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2086, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2941] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2086, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2942] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2086, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2943] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2088, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2944] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2945] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2946] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2947] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2092, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2948] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2092, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2949] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2092, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2950] StringParser("SEQUENCEFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2092, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2951] StringParser("SEQUENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2092, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2952] StringParser("SERDE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2092, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2953] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2954] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2955] Ref(SerdepropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2956] Ref(BracketedPropertyListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2957] StringParser("SERDEPROPERTIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2958] StringParser("SERIALIZABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2959] StringParser("SERVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2960] StringParser("SERVER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2961] StringParser("SESSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2962] StringParser("SESSION_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2095, child_count: 4, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2963] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2099, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2964] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2099, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2965] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2966] Ref(SetClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2101, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2967] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2968] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2969] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2104, child_count: 7, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2970] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2111, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2971] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2111, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2972] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2111, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2973] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2111, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2974] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2111, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2975] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2111, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2976] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2111, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2977] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2111, child_count: 8, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2978] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2119, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2979] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2119, child_count: 1, min_times: 1, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2980] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2120, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2981] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2122, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2982] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2122, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2983] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2122, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2984] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2122, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2985] Ref(ClusterByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2122, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2986] Ref(DistributeByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2122, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2987] Ref(SortByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2122, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2988] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2122, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2989] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2122, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2990] StringParser("SET")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2122, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2991] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2125, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2992] Ref(UnionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2125, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2993] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2994] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2129, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2995] Ref(IntersectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2129, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2996] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2129, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2997] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2129, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2998] Ref(MinusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2129, child_count: 4, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [2999] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2133, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3000] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2133, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3001] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2133, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3002] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2133, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3003] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2133, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3004] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2135, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3005] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2135, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3006] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2137, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3007] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3008] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3009] StringParser("V")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3010] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2142, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3011] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2144, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3012] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2146, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3013] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2146, child_count: 1, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3014] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2147, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3015] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2149, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3016] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2149, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3017] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2149, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3018] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3019] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3020] Ref(ColonDelimiterSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3021] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3022] Ref(HiveSetValueGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3023] StringParser("SETOF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3024] StringParser("SETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3025] StringParser("SETUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3026] StringParser("SHARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3027] StringParser("SHARED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3028] StringParser("SHARES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2151, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3029] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2153, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3030] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2155, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3031] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2155, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3032] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2155, child_count: 1, min_times: 1, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3033] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2156, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3034] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2159, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3035] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2159, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3036] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2159, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3037] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2159, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3038] StringParser("SHOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2159, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3039] StringParser("SHOW_DATABASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2159, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3040] StringParser("SHUTDOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2159, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3041] StringParser("SIGNAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2159, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3042] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2161, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3043] Ref(PositiveSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2161, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3044] Ref(NegativeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2161, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3045] StringParser("SIMILAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2161, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3046] StringParser("SIMPLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2161, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 1, _padding: 0 }, // [3047] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2164, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3048] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2164, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3049] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2164, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3050] Ref(BackQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2164, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3051] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2164, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3052] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2166, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3053] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2166, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3054] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3055] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3056] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3057] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3058] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3059] StringParser("SIZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3060] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2171, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3061] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2171, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3062] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2171, child_count: 6, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3063] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3064] Ref(SkewedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3065] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3066] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3067] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2177, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3068] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2180, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3069] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3070] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2184, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3071] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2184, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3072] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3073] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3074] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2189, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3075] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3076] Ref(StoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3077] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3078] Ref(DirectoriesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3079] StringParser("SKEWED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3080] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3081] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3082] StringParser("SMALLINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3083] StringParser("SNAPSHOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3084] StringParser("SOME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3085] StringParser("SONAME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 5, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3086] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2197, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3087] Ref(SortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2197, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3088] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2197, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 2, _padding: 0 }, // [3089] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2199, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3090] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2202, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3091] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2205, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3092] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2205, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3093] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2205, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3094] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2205, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3095] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2207, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3096] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2207, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3097] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2207, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3098] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2209, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3099] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2209, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3100] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3101] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3102] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3103] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3104] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3105] StringParser("SORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3106] StringParser("SORTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3107] StringParser("SOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3108] StringParser("SPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3109] StringParser("SPATIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3110] StringParser("SPECIFIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3111] StringParser("SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3112] StringParser("SPECIFICTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3113] StringParser("SQL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3114] StringParser("SQL_BIG_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3115] StringParser("SQL_BIG_SELECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3116] StringParser("SQL_BIG_TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3117] StringParser("SQL_CALC_FOUND_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3118] StringParser("SQL_LOG_OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3119] StringParser("SQL_LOG_UPDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3120] StringParser("SQL_LOW_PRIORITY_UPDATES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3121] StringParser("SQL_SELECT_LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3122] StringParser("SQL_SMALL_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3123] StringParser("SQL_WARNINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3124] StringParser("SQLCA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3125] StringParser("SQLCODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3126] StringParser("SQLERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3127] StringParser("SQLEXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3128] StringParser("SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3129] StringParser("SQLWARNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3130] StringParser("SQRT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3131] StringParser("SSL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3132] StringParser("STABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3133] StringParser("STAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3134] StringParser("STAGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3135] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3136] StringParser("START")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3137] StringParser("STARTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3138] StringParser("STARTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3139] StringParser("STATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3140] StringParser("STATEMENT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2211, child_count: 40, min_times: 0, first_terminator_idx: 55, terminator_count: 1, _padding: 0 }, // [3141] OneOf(40 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3142] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3143] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3144] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3145] Ref(DropTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3146] Ref(DropViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3147] Ref(CreateUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3148] Ref(DropUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3149] Ref(TruncateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3150] Ref(AccessStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3151] Ref(CreateTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3152] Ref(CreateRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3153] Ref(DropRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3154] Ref(AlterTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3155] Ref(DropSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3156] Ref(DropTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3157] Ref(CreateDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3158] Ref(DropDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3159] Ref(CreateIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3160] Ref(DropIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3161] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3162] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3163] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3164] Ref(CreateCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3165] Ref(DropCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3166] Ref(CreateFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3167] Ref(DropFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3168] Ref(DescribeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3169] Ref(UseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3170] Ref(ExplainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3171] Ref(CreateSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3172] Ref(AlterSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3173] Ref(DropSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3174] Ref(CreateTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3175] Ref(DropTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3176] Ref(FromInsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3177] Ref(AlterDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3178] Ref(MsckRepairTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3179] Ref(MsckTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3180] Ref(SetStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3181] Ref(AlterViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3182] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3183] StringParser("STATIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3184] StringParser("STATISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3185] StringParser("STDDEV_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3186] StringParser("STDDEV_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3187] StringParser("STDIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3188] StringParser("STDOUT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3189] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2253, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3190] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2255, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3191] Ref(RowFormatClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2255, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3192] Ref(StoredAsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2255, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3193] Ref(StoredByGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2255, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3194] StringParser("STORAGE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2255, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3195] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2258, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3196] Ref(StoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2258, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3197] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2258, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3198] Ref(FileFormatGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2258, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3199] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2262, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3200] Ref(StoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2262, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3201] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2262, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3202] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2262, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3203] Ref(SerdePropertiesGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2262, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3204] StringParser("STORED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2262, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3205] StringParser("STRAIGHT_JOIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2262, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3206] StringParser("STREAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2262, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3207] StringParser("STREAMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2262, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3208] StringParser("STREAMTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2262, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3209] StringParser("STRICT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2262, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3210] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3211] Ref(ConcatSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3212] StringParser("STRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3213] StringParser("STRUCT")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3214] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2266, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3215] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2268, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3216] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2270, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3217] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2270, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3218] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2270, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3219] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2273, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3220] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2275, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3221] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3222] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3223] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3224] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2279, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3225] Ref(CommentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3226] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3227] Ref(StructKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3228] Ref(StructTypeSchemaSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3229] StringParser("STRUCTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3230] StringParser("STYLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3231] StringParser("SUBCLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3232] StringParser("SUBLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3233] StringParser("SUBMULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3234] StringParser("SUBSTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3235] StringParser("SUCCESSFUL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3236] StringParser("SUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3237] StringParser("SUMMARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3238] StringParser("SUPERUSER")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3239] Token("symbol")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3240] StringParser("SYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3241] StringParser("SYNC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3242] StringParser("SYNONYM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3243] StringParser("SYSDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3244] StringParser("SYSID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3245] StringParser("SYSTEM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3246] StringParser("SYSTEM_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2281, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3247] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2283, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3248] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2285, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3249] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2285, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3250] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2285, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3251] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2288, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3252] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2290, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3253] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2290, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3254] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2290, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3255] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2293, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3256] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2293, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3257] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2293, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3258] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2296, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3259] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2296, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3260] Ref(NovalidateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2296, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3261] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2298, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3262] Ref(RelyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2298, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3263] Ref(NorelyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2298, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3264] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2302, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3265] Ref(ForeignKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2302, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3266] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2302, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3267] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2302, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3268] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2304, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3269] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2304, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3270] Ref(NovalidateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2304, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3271] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2304, child_count: 6, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3272] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3273] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3274] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3275] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3276] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2310, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3277] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2313, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3278] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2313, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3279] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2316, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3280] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2316, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3281] StringParser("TABLE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2316, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3282] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2318, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3283] Ref(TblpropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2318, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3284] Ref(BracketedPropertyListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2318, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3285] StringParser("TABLE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2318, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3286] StringParser("TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2318, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3287] StringParser("TABLESAMPLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2318, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3288] StringParser("TABLESPACE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2318, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3289] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2320, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 1, _padding: 0 }, // [3290] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3291] Ref(Expression_A_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [3292] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3293] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3294] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2323, child_count: 1, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3295] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3296] Ref(Expression_B_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3297] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3298] StringParser("TASK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3299] StringParser("TASKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3300] StringParser("TBLPROPERTIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3301] StringParser("TEMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3302] StringParser("TEMPLATE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3303] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3304] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3305] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3306] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2326, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3307] StringParser("TEMPORARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2326, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3308] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3309] Ref(TransientKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3310] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2328, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3311] StringParser("TERMINATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3312] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2331, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3313] Ref(TerminatedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2331, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3314] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2331, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3315] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2331, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3316] StringParser("TERMINATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2331, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3317] StringParser("TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2331, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3318] StringParser("TEXTFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2331, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3319] StringParser("TEXTSIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2331, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3320] StringParser("THAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2331, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3321] StringParser("THEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2331, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3322] StringParser("TIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2331, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3323] StringParser("~")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2331, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3324] StringParser("TIME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2331, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3325] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3326] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2336, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3327] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2336, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3328] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2336, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3329] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2336, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3330] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2339, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3331] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3332] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3333] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3334] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3335] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2341, child_count: 1, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3336] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2342, child_count: 4, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3337] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3338] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3339] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3340] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3341] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3342] StringParser("TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3343] StringParser("TIMESTAMPTZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3344] StringParser("TIMEZONE_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3345] StringParser("TIMEZONE_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3346] StringParser("TINYBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3347] StringParser("TINYINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3348] StringParser("TINYTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3349] StringParser("TO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3350] StringParser("TOAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3351] StringParser("TOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3352] StringParser("TOP_LEVEL_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3353] StringParser("TOUCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3354] StringParser("TRAILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3355] StringParser("TRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3356] StringParser("TRANSACTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2346, child_count: 4, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3357] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2350, child_count: 5, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3358] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2355, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3359] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2355, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3360] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2355, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3361] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2355, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3362] Ref(RollbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2355, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3363] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2355, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3364] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3365] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3366] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2357, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3367] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2359, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3368] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2359, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3369] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2359, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3370] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3371] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3372] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3373] Ref(ChainKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3374] StringParser("TRANSACTION_ACTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3375] StringParser("TRANSACTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3376] StringParser("TRANSACTIONS_COMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3377] StringParser("TRANSACTIONS_ROLLED_BACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3378] StringParser("TRANSFORM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3379] StringParser("TRANSFORMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3380] StringParser("TRANSIENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3381] StringParser("TRANSLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3382] StringParser("TRANSLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3383] StringParser("TREAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3384] StringParser("TRIGGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3385] StringParser("TRIGGER_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3386] StringParser("TRIGGER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3387] StringParser("TRIGGER_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3388] StringParser("TRIM")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3389] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3390] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3391] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3392] StringParser("TRUNCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2362, child_count: 4, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3393] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2366, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3394] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2366, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3395] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2366, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3396] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2366, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3397] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2366, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3398] StringParser("TRUSTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2366, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3399] StringParser("TSEQUAL")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2366, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3400] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2369, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3401] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2371, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3402] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2371, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3403] StringParser("TYPE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2371, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3404] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2373, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3405] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2373, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3406] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2373, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3407] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2375, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3408] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2375, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3409] Ref(StructLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2375, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3410] StringParser("UESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2375, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3411] StringParser("UID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2375, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3412] StringParser("UNARCHIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2375, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3413] StringParser("UNBOUNDED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2375, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3414] StringParser("UNCOMMITTED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2375, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3415] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2375, child_count: 3, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3416] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3417] Ref(NaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3418] Ref(UnconditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3419] Ref(HorizontalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3420] StringParser("UNDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3421] StringParser("UNDO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3422] StringParser("UNENCRYPTED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2378, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3423] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2380, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3424] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2380, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3425] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3426] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3427] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3428] StringParser("UNION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3429] StringParser("UNIONTYPE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 1, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3430] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2383, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3431] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2383, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3432] StringParser("UNIQUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2383, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3433] StringParser("UNIQUEJOIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2383, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3434] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2383, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3435] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2383, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3436] StringParser("UNLISTEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2383, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3437] StringParser("UNLOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2383, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3438] StringParser("UNNAMED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2383, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3439] StringParser("UNNEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 2383, child_count: 7, min_times: 0, first_terminator_idx: 57, terminator_count: 8, _padding: 0 }, // [3440] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2390, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3441] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2390, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3442] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2390, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3443] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2390, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3444] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2390, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3445] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2390, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3446] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2390, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3447] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2390, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3448] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2390, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3449] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2390, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3450] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2390, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3451] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2390, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3452] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2390, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3453] Ref(ClusterByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2390, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3454] Ref(DistributeByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2390, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [3455] Ref(SortByClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2390, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3456] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3457] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2392, child_count: 1, min_times: 1, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3458] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2393, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3459] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3460] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3461] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3462] StringParser("UNSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3463] StringParser("UNSIGNED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3464] StringParser("UNTIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3465] StringParser("UPDATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2395, child_count: 8, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3466] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2403, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3467] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2403, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3468] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 2403, child_count: 1, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3469] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2404, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3470] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2404, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3471] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2404, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3472] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2404, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3473] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2404, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3474] StringParser("UPDATETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2404, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3475] StringParser("UPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2404, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3476] StringParser("URI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2404, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3477] StringParser("USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2404, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3478] StringParser("USE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2404, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3479] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3480] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3481] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3482] StringParser("USE_ANY_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3483] StringParser("USER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3484] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3485] StringParser("USER_DEFINED_TYPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3486] StringParser("USER_DEFINED_TYPE_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3487] StringParser("USER_DEFINED_TYPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3488] StringParser("USER_DEFINED_TYPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3489] StringParser("USERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3490] StringParser("USING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3491] StringParser("UTC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3492] StringParser("UTC_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3493] StringParser("UTC_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3494] StringParser("UTC_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3495] StringParser("UTC_TMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3496] StringParser("UTCTIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3497] StringParser("VACUUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3498] StringParser("VALID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3499] StringParser("VALIDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3500] StringParser("VALIDATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3501] StringParser("VALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3502] StringParser("VALUE_TYPE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2406, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3503] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2408, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3504] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2410, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3505] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2410, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3506] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2410, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3507] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2412, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3508] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3509] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2414, child_count: 3, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3510] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2417, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3511] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2419, child_count: 3, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3512] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3513] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3514] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3515] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3516] StringParser("VALUES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3517] StringParser("VAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3518] StringParser("VAR_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3519] StringParser("VARBINARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3520] StringParser("VARCHAR2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3521] StringParser("VARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3522] StringParser("VARCHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3523] StringParser("VARIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3524] StringParser("VARIABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3525] StringParser("VARYING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3526] StringParser("VECTORIZATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3527] StringParser("VERBOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3528] StringParser("VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3529] StringParser("VIEW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3530] StringParser("VIEWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3531] StringParser("VOLATILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3532] StringParser("WAITFOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3533] StringParser("WAREHOUSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3534] StringParser("WAREHOUSES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3535] StringParser("WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3536] StringParser("WEEKDAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3537] StringParser("WEEKS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2422, child_count: 8, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3538] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2430, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3539] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2430, child_count: 3, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3540] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2433, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3541] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2433, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3542] Meta(conditional: indented_then=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2433, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3543] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2433, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3544] Meta(conditional: indented_then_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2433, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3545] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2433, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3546] Meta(conditional: indented_then_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2433, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3547] Meta(conditional: indented_then=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2433, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3548] StringParser("WHEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2433, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3549] StringParser("WHENEVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2433, child_count: 4, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3550] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2437, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3551] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2437, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3552] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2439, child_count: 3, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3553] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2442, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3554] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2442, child_count: 11, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3555] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2453, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3556] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2453, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3557] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2455, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3558] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2455, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3559] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2455, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3560] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3561] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3562] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3563] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2459, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3564] Ref(DistributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2459, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3565] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2459, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3566] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2461, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3567] Ref(SortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2461, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3568] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2461, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3569] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2463, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3570] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2463, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3571] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2463, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3572] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2463, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3573] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2463, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3574] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2463, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3575] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2463, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3576] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2463, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3577] StringParser("WHERE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2463, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3578] StringParser("WHILE")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2463, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3579] Token("whitespace")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2463, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3580] StringParser("WIDTH_BUCKET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2463, child_count: 1, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3581] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3582] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2464, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3583] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2466, child_count: 1, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3584] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2467, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3585] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2469, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3586] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2471, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3587] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2471, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3588] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2471, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3589] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2473, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3590] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2473, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3591] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2473, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3592] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2473, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3593] StringParser("WINDOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2473, child_count: 4, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3594] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 2477, child_count: 1, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3595] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2478, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3596] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2480, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3597] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2480, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3598] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2480, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3599] Ref(PartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2480, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3600] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2480, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3601] Ref(FrameClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2480, child_count: 6, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [3602] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2486, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3603] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2486, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3604] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2486, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3605] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2486, child_count: 2, min_times: 0, first_terminator_idx: 65, terminator_count: 1, _padding: 0 }, // [3606] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2488, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3607] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2488, child_count: 0, min_times: 0, first_terminator_idx: 65, terminator_count: 0, _padding: 0 }, // [3608] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2488, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [3609] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2488, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [3610] Ref(NonWithNonSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2488, child_count: 6, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3611] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2494, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [3612] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2494, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [3613] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2494, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [3614] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2494, child_count: 2, min_times: 0, first_terminator_idx: 66, terminator_count: 1, _padding: 0 }, // [3615] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2496, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [3616] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2496, child_count: 0, min_times: 0, first_terminator_idx: 66, terminator_count: 0, _padding: 0 }, // [3617] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2496, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3618] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2496, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3619] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2496, child_count: 3, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3620] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3621] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2499, child_count: 1, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3622] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2500, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3623] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2500, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3624] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2500, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3625] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2500, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3626] StringParser("WITH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2500, child_count: 4, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3627] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3628] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3629] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3630] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3631] Ref(BindingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3632] StringParser("WITHIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3633] StringParser("WITHOUT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3634] Token("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3635] StringParser("WORK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3636] StringParser("WRAPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3637] StringParser("WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3638] StringParser("WRITETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3639] StringParser("X509")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3640] StringParser("XML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3641] StringParser("XOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3642] StringParser("YAML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3643] StringParser("YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3644] StringParser("YEAR_MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3645] StringParser("YEARS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3646] StringParser("ZEROFILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 67, terminator_count: 0, _padding: 0 }, // [3647] StringParser("ZONE")
];

pub static CHILD_IDS: &[u32] = &[
    5, 6, 61, 7, 51, 8, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
    21, 28, 33, 38, 39, 44, 9, 10, 22, 23, 24, 25, 26, 27, 29, 30,
    31, 32, 34, 35, 36, 37, 40, 41, 42, 43, 45, 46, 47, 48, 49, 50,
    52, 58, 53, 54, 57, 55, 56, 62, 63, 64, 66, 74, 78, 81, 84, 89,
    67, 68, 69, 70, 71, 72, 73, 75, 76, 77, 79, 80, 82, 83, 85, 86,
    87, 88, 90, 133, 91, 100, 105, 108, 109, 110, 111, 112, 113, 114, 115, 116,
    117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 92, 93,
    94, 95, 98, 99, 96, 97, 101, 102, 103, 104, 106, 107, 131, 132, 135, 58,
    138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 151, 154, 157, 149, 150,
    152, 153, 155, 156, 158, 159, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170,
    171, 174, 177, 180, 172, 173, 175, 176, 178, 179, 181, 182, 184, 185, 187, 191,
    195, 199, 203, 205, 207, 188, 189, 190, 58, 192, 193, 194, 58, 196, 197, 198,
    58, 200, 201, 202, 58, 204, 58, 206, 58, 209, 220, 221, 222, 230, 223, 229,
    224, 225, 226, 227, 228, 233, 234, 239, 240, 243, 244, 245, 241, 242, 246, 249,
    255, 256, 247, 248, 250, 251, 254, 252, 253, 257, 258, 261, 265, 266, 267, 272,
    275, 262, 263, 264, 268, 271, 269, 270, 273, 274, 277, 278, 279, 280, 281, 283,
    284, 285, 286, 288, 294, 307, 308, 314, 289, 290, 291, 292, 293, 295, 298, 299,
    300, 296, 297, 301, 306, 302, 305, 303, 304, 309, 310, 313, 311, 312, 315, 316,
    317, 318, 319, 321, 322, 323, 324, 325, 58, 327, 328, 329, 330, 331, 334, 332,
    333, 335, 336, 337, 338, 338, 227, 228, 350, 351, 352, 353, 354, 355, 356, 357,
    358, 359, 361, 366, 367, 362, 365, 363, 364, 371, 366, 367, 372, 58, 374, 375,
    376, 377, 378, 380, 394, 405, 406, 407, 408, 409, 410, 411, 412, 413, 425, 426,
    427, 428, 436, 437, 440, 441, 446, 447, 450, 451, 454, 227, 228, 455, 58, 457,
    227, 228, 458, 58, 460, 227, 228, 461, 58, 464, 227, 228, 473, 227, 228, 475,
    476, 477, 478, 479, 227, 228, 487, 497, 488, 489, 490, 494, 230, 496, 491, 498,
    499, 489, 500, 504, 230, 506, 501, 538, 539, 220, 540, 230, 541, 58, 542, 543,
    544, 545, 558, 559, 560, 561, 572, 573, 574, 575, 577, 580, 578, 579, 581, 584,
    588, 591, 594, 595, 596, 599, 600, 603, 582, 583, 585, 586, 587, 227, 228, 589,
    590, 592, 593, 597, 598, 601, 602, 605, 606, 607, 609, 608, 227, 228, 610, 614,
    615, 627, 628, 630, 631, 639, 640, 641, 642, 643, 644, 645, 646, 654, 655, 661,
    662, 663, 686, 687, 688, 692, 693, 694, 704, 705, 706, 709, 689, 690, 691, 227,
    228, 695, 696, 697, 698, 699, 703, 700, 701, 702, 707, 708, 710, 711, 713, 714,
    717, 718, 719, 720, 721, 724, 715, 716, 722, 723, 725, 726, 727, 729, 730, 731,
    732, 733, 734, 735, 736, 739, 737, 738, 741, 742, 743, 744, 745, 746, 747, 748,
    749, 750, 751, 227, 228, 752, 58, 755, 756, 757, 758, 759, 760, 772, 773, 761,
    762, 763, 227, 228, 764, 58, 765, 766, 767, 768, 769, 770, 366, 367, 771, 58,
    775, 776, 777, 778, 780, 781, 782, 783, 785, 789, 793, 794, 795, 800, 803, 786,
    787, 788, 790, 791, 792, 796, 799, 797, 798, 801, 802, 805, 806, 807, 808, 809,
    811, 812, 813, 814, 815, 816, 817, 818, 864, 819, 826, 827, 828, 836, 853, 854,
    855, 856, 857, 858, 859, 820, 227, 228, 821, 58, 822, 823, 824, 825, 829, 830,
    831, 832, 227, 228, 833, 58, 834, 835, 837, 838, 839, 840, 850, 851, 852, 841,
    842, 843, 844, 227, 228, 845, 58, 846, 847, 848, 849, 860, 861, 862, 863, 863,
    227, 228, 865, 866, 867, 868, 870, 871, 872, 873, 879, 892, 893, 894, 931, 874,
    875, 876, 877, 878, 880, 890, 881, 882, 883, 884, 885, 886, 887, 58, 895, 905,
    908, 921, 927, 896, 897, 898, 899, 900, 901, 902, 903, 904, 906, 907, 909, 912,
    910, 911, 913, 914, 915, 918, 916, 917, 919, 920, 922, 923, 924, 925, 926, 928,
    929, 930, 227, 228, 932, 933, 934, 935, 937, 938, 939, 941, 942, 943, 944, 945,
    946, 947, 948, 951, 949, 950, 950, 227, 228, 960, 963, 961, 962, 964, 227, 228,
    982, 983, 984, 986, 987, 988, 989, 996, 997, 990, 991, 992, 377, 378, 993, 994,
    995, 998, 999, 1000, 377, 378, 1001, 58, 1006, 1007, 227, 228, 1008, 58, 1009, 1010,
    1012, 1017, 1013, 1014, 1015, 1016, 1039, 1040, 1051, 1052, 1053, 1067, 1068, 1069, 1086, 1087,
    220, 1088, 230, 1089, 58, 1090, 1091, 1092, 1093, 1111, 1112, 1114, 1115, 1116, 1120, 1117,
    1118, 1119, 227, 228, 1122, 1123, 1124, 1125, 1126, 1128, 1129, 1130, 1131, 1133, 1134, 1135,
    1136, 1137, 1140, 1141, 1142, 1143, 1145, 1146, 1147, 1148, 1150, 1151, 1152, 1153, 1154, 1156,
    1157, 1158, 1160, 1161, 1162, 1163, 1164, 1166, 1167, 1168, 1169, 1171, 1172, 1173, 1174, 1175,
    1177, 1178, 1179, 1180, 1182, 1183, 1184, 1185, 1186, 1196, 489, 1197, 230, 227, 228, 1202,
    1203, 1214, 1215, 1235, 1236, 1237, 1238, 1239, 1240, 1245, 1246, 1247, 1248, 1249, 1252, 1253,
    1254, 1258, 1259, 1260, 1261, 1267, 1250, 1251, 1255, 1256, 1257, 1262, 1263, 1264, 1265, 1266,
    1268, 1269, 1271, 1274, 1275, 1276, 1272, 1273, 1278, 1279, 1280, 1281, 1282, 1286, 1283, 1284,
    1285, 1288, 1291, 1289, 1290, 1293, 1297, 1303, 1294, 1295, 1296, 227, 228, 1298, 1301, 1299,
    1300, 1302, 1306, 1341, 1307, 1308, 1309, 1321, 1322, 1323, 1330, 1338, 1339, 1310, 227, 228,
    1311, 1312, 1313, 1314, 1315, 58, 1316, 1317, 1318, 1319, 1320, 1324, 1327, 1325, 1326, 1328,
    227, 228, 1329, 58, 1331, 1332, 1333, 1334, 1335, 1336, 1337, 1343, 1344, 1345, 1346, 1347,
    1348, 1349, 1354, 1355, 1362, 1363, 1366, 1370, 1373, 1364, 1365, 1367, 1368, 1369, 1371, 1372,
    1374, 1375, 1376, 1377, 1381, 1382, 1383, 1384, 1385, 1386, 1387, 1388, 1389, 1390, 1391, 1392,
    1395, 1397, 1401, 1396, 1398, 1399, 1400, 1402, 1406, 1407, 1408, 227, 228, 1409, 1410, 1423,
    1424, 1432, 1433, 1434, 1447, 1435, 1438, 1436, 1437, 1439, 1444, 1440, 1441, 1442, 1443, 1445,
    1446, 1448, 1434, 1449, 1434, 1451, 1452, 1458, 1459, 1460, 58, 1462, 1463, 1464, 1467, 1470,
    1473, 1476, 1479, 1480, 1481, 1482, 1483, 1484, 1485, 1486, 1465, 1466, 1468, 1469, 1471, 1472,
    1474, 1475, 1477, 1478, 1488, 1489, 1493, 1494, 1500, 1504, 1505, 1507, 1490, 1492, 1491, 227,
    228, 1495, 1496, 1497, 1498, 1499, 1501, 1502, 1503, 1506, 1509, 1510, 1510, 227, 228, 489,
    1511, 230, 1522, 1523, 1534, 1512, 1513, 1514, 1515, 227, 228, 1517, 1518, 1520, 1521, 1524,
    1525, 1526, 1527, 1529, 1530, 1532, 1533, 1536, 1537, 1538, 1557, 1539, 1540, 1541, 1548, 1542,
    1543, 1544, 1545, 1546, 1549, 1551, 1552, 1553, 1554, 1555, 1550, 1558, 1559, 1560, 1561, 1562,
    1563, 1564, 1566, 1568, 1569, 1570, 58, 1572, 1573, 1574, 1580, 1581, 1585, 1591, 1597, 1603,
    1604, 1607, 1617, 1618, 1619, 1582, 1583, 1584, 1586, 1587, 1589, 1590, 1588, 1592, 1595, 1596,
    1593, 1594, 1598, 1599, 1600, 1601, 1602, 58, 1605, 1606, 1608, 1612, 1613, 1609, 1610, 1611,
    1614, 1615, 1616, 1621, 1622, 227, 228, 1624, 1625, 1626, 1627, 1628, 1632, 1637, 1633, 1634,
    1635, 1638, 1639, 1642, 1649, 1643, 1644, 1645, 1648, 1646, 1647, 1650, 1653, 1651, 1652, 1655,
    227, 228, 1656, 58, 1658, 1662, 1672, 1659, 1660, 1661, 1663, 1664, 1665, 1666, 1667, 1668,
    227, 228, 1669, 58, 1670, 1671, 1673, 1679, 1674, 1678, 1675, 1676, 1677, 1693, 1694, 1708,
    1709, 1710, 1723, 1695, 1699, 1702, 1707, 1696, 1697, 1698, 1700, 1701, 1703, 1704, 1705, 1706,
    1711, 1715, 1719, 1712, 1713, 1714, 1716, 1717, 1718, 1720, 1721, 1722, 1724, 1725, 1726, 1727,
    1728, 1729, 1733, 1734, 1738, 1739, 220, 1740, 230, 1741, 1742, 1743, 1744, 1745, 1751, 1746,
    58, 1747, 1748, 1749, 1753, 1760, 1761, 1762, 1763, 1754, 1759, 1755, 1756, 1757, 1758, 1766,
    1767, 58, 1768, 1769, 1770, 1771, 227, 228, 1775, 1776, 1777, 1778, 227, 228, 1779, 58,
    1780, 1781, 1784, 489, 1785, 230, 1786, 1787, 1787, 227, 228, 1789, 1796, 1797, 1798, 1790,
    1795, 1791, 1792, 1793, 1794, 1805, 1806, 1807, 1808, 1811, 1814, 1812, 227, 228, 1813, 58,
    1834, 1835, 1838, 1839, 1840, 1843, 1846, 1844, 1845, 1856, 1857, 1858, 1859, 1864, 1865, 1860,
    227, 228, 1861, 1863, 1862, 58, 1870, 1871, 1872, 1873, 1893, 1894, 1895, 1912, 1896, 1897,
    1898, 1904, 1899, 1900, 1901, 1902, 1903, 1905, 1907, 1908, 1909, 1910, 1911, 1906, 1913, 1914,
    1915, 1916, 1917, 1918, 1919, 1920, 1937, 1938, 1939, 1940, 1945, 1946, 1941, 1942, 1943, 1944,
    227, 228, 1947, 1948, 1954, 1955, 1956, 1957, 1958, 1960, 1961, 1962, 1963, 1973, 1986, 1991,
    1974, 1975, 220, 1976, 1977, 230, 1979, 1978, 1980, 1981, 1982, 1985, 1983, 1984, 1987, 1988,
    220, 1989, 1990, 230, 1992, 220, 1993, 230, 1996, 1998, 1997, 2001, 2002, 2003, 2006, 2004,
    2005, 2005, 227, 228, 2008, 2009, 2010, 2014, 2011, 2012, 2013, 2016, 220, 2017, 230, 2018,
    227, 228, 2019, 58, 220, 2037, 2038, 2039, 2040, 2041, 2042, 230, 2043, 2044, 2045, 58,
    2053, 2054, 2058, 2061, 2062, 2059, 2060, 2063, 2064, 2066, 2067, 2068, 2069, 2070, 2074, 220,
    2075, 2081, 230, 2076, 2077, 2077, 227, 228, 2078, 2079, 2080, 2082, 2087, 2083, 2084, 2085,
    2086, 2088, 2089, 2096, 2097, 2098, 2099, 2100, 2101, 2102, 2103, 2104, 2105, 2115, 2116, 2132,
    2133, 2134, 2135, 2136, 2139, 227, 228, 2137, 2138, 2140, 2141, 2142, 227, 228, 2164, 220,
    2165, 230, 2166, 2168, 2169, 2172, 2173, 2175, 2176, 2177, 2180, 220, 2181, 230, 2178, 2179,
    2182, 2183, 2185, 2186, 2187, 2188, 2191, 220, 2192, 230, 2189, 2190, 2194, 220, 2195, 230,
    2198, 220, 2199, 230, 2209, 2210, 2211, 2212, 2196, 2197, 2200, 2201, 2202, 2206, 2203, 2205,
    2204, 227, 228, 2207, 2208, 2214, 220, 2215, 230, 2249, 2250, 2251, 2252, 2253, 2254, 2258,
    2255, 2256, 2257, 2260, 2261, 2262, 2263, 2264, 2268, 2265, 2266, 2267, 2276, 2277, 2278, 2279,
    2280, 2281, 227, 228, 2283, 220, 2284, 230, 2285, 58, 2291, 2292, 2316, 2317, 2318, 2320,
    2322, 2324, 2319, 227, 228, 2321, 227, 228, 2323, 227, 228, 2327, 2328, 2329, 2330, 2332,
    2333, 2336, 2334, 2335, 2335, 227, 228, 2348, 2351, 2349, 2350, 2352, 2353, 2372, 2373, 2374,
    2376, 2378, 1209, 2377, 58, 2380, 2381, 2382, 2383, 2385, 2386, 2387, 2388, 2389, 2390, 2391,
    2392, 2393, 2394, 2395, 462, 2403, 2404, 2408, 2405, 2406, 2407, 2409, 2410, 2433, 2434, 2437,
    2438, 220, 2439, 230, 2440, 58, 2441, 2445, 2448, 2453, 2442, 2443, 2444, 2446, 2447, 2449,
    2450, 2451, 2452, 2457, 2458, 2459, 2460, 2461, 2462, 2463, 2466, 2467, 220, 2478, 2479, 2480,
    230, 2481, 2482, 2483, 227, 228, 2486, 2487, 2488, 2493, 2489, 2490, 2491, 2492, 227, 228,
    2516, 2517, 220, 2518, 230, 2519, 2520, 2520, 227, 228, 2521, 58, 2524, 2525, 2526, 227,
    228, 2527, 58, 2528, 2529, 2530, 2531, 2538, 2543, 2539, 2540, 2541, 2542, 2561, 2562, 2576,
    2577, 2580, 2581, 2582, 2583, 2584, 2585, 2586, 2587, 2590, 2591, 2592, 2593, 2599, 2600, 2601,
    2602, 2588, 2589, 2594, 2598, 2595, 2596, 2597, 2614, 2615, 2616, 2621, 2622, 2625, 2626, 2630,
    2627, 2628, 2629, 2631, 227, 228, 2636, 2637, 2638, 2663, 2664, 2665, 2666, 2667, 2668, 2672,
    2669, 2670, 2671, 2673, 2674, 2675, 2677, 2678, 2679, 2680, 2681, 2686, 2687, 2688, 2691, 2694,
    2689, 2690, 2692, 2693, 2695, 2696, 2735, 2736, 2740, 2754, 2755, 2756, 2737, 2738, 2739, 2741,
    2745, 2748, 2753, 2742, 2743, 2744, 2746, 2747, 2749, 2750, 2751, 2752, 2772, 2773, 2774, 2775,
    2800, 2776, 2777, 2784, 2788, 2792, 2795, 2778, 2779, 2780, 2781, 2782, 2783, 2785, 2786, 2787,
    2789, 2790, 2791, 2793, 2794, 2796, 2797, 2798, 2799, 2801, 2802, 2803, 2805, 2806, 227, 228,
    2807, 58, 2808, 2819, 2820, 2839, 2821, 227, 228, 2822, 2833, 2838, 2823, 2824, 2825, 2826,
    2827, 2828, 2829, 2830, 2831, 2832, 2834, 2835, 2836, 2837, 2858, 2859, 2860, 2861, 2863, 2864,
    2866, 2867, 220, 2868, 230, 2869, 58, 2872, 2873, 2875, 2876, 2878, 2879, 2881, 2883, 2884,
    2887, 2890, 2893, 2896, 2897, 2898, 2899, 2882, 2885, 2886, 2888, 2889, 2891, 2892, 2894, 2895,
    2902, 2903, 2904, 2905, 2906, 2907, 2908, 2909, 2910, 2911, 2912, 2913, 2914, 2915, 2916, 2921,
    2924, 2927, 2928, 2922, 2923, 2923, 227, 228, 2925, 2926, 2926, 227, 228, 2929, 227, 228,
    2937, 2940, 2938, 2939, 2941, 2942, 2944, 2947, 2945, 2946, 2948, 2949, 2954, 2955, 2956, 2964,
    220, 2965, 230, 2966, 58, 2968, 2969, 2970, 2971, 2972, 2973, 2974, 2975, 2976, 2977, 2979,
    2980, 2984, 2985, 2986, 2987, 2988, 2989, 2981, 2982, 2983, 2992, 2993, 2998, 2994, 2997, 2995,
    2996, 3000, 3001, 3002, 3003, 3005, 3006, 3007, 3010, 3008, 3009, 3011, 3021, 3022, 3012, 3018,
    3013, 3014, 3015, 3016, 3017, 3019, 3020, 3030, 3033, 3031, 3032, 3034, 3035, 3036, 3037, 3043,
    3044, 3048, 3049, 3050, 3053, 58, 3055, 3056, 3057, 3061, 3062, 3064, 3065, 3066, 3067, 3068,
    3075, 3069, 227, 228, 3070, 58, 3071, 3072, 3073, 227, 228, 3074, 58, 3076, 3077, 3078,
    3087, 3088, 220, 3089, 230, 3090, 58, 3091, 3095, 3098, 3092, 3093, 3094, 3096, 3097, 3099,
    3100, 3101, 3102, 3142, 3143, 3144, 3145, 3146, 3147, 3148, 3149, 3150, 3151, 3152, 3153, 3154,
    3155, 3156, 3157, 3158, 3159, 3160, 3161, 3162, 3163, 3164, 3165, 3166, 3167, 3168, 3169, 3170,
    3171, 3172, 3173, 3174, 3175, 3176, 3177, 3178, 3179, 3180, 3181, 3190, 3193, 3191, 3192, 3196,
    3197, 3198, 3200, 3201, 3202, 3203, 3211, 3215, 227, 228, 3216, 58, 3217, 3218, 3220, 377,
    378, 3221, 58, 3222, 3223, 3224, 3225, 3227, 3228, 3248, 3251, 3249, 3250, 3252, 3255, 3264,
    3253, 3254, 3256, 3257, 3258, 3259, 3260, 3261, 3262, 3263, 3265, 3266, 3267, 3268, 3269, 3270,
    3273, 3274, 3275, 3276, 3277, 3279, 3278, 227, 228, 3280, 227, 228, 3283, 3284, 3290, 3293,
    3291, 3295, 3297, 3296, 3305, 3306, 3309, 3310, 3313, 3314, 3315, 3326, 3329, 3330, 3327, 3328,
    3331, 3334, 3335, 3332, 3333, 3337, 3338, 3339, 3340, 3341, 3358, 3364, 3367, 3370, 3359, 3360,
    3361, 3362, 3363, 3365, 3366, 3368, 3369, 3371, 3372, 3373, 3394, 3395, 3396, 3397, 3401, 227,
    228, 3402, 58, 3405, 3406, 3408, 3409, 3417, 3418, 3419, 3424, 3425, 3426, 3427, 3431, 3441,
    3442, 3443, 3444, 3445, 3446, 3447, 3457, 3458, 3459, 3460, 3461, 3467, 220, 3468, 3469, 230,
    3471, 3472, 3473, 3470, 3480, 3481, 3504, 3507, 3505, 3506, 3508, 58, 3509, 3510, 3511, 227,
    228, 3512, 58, 3513, 3514, 3515, 3539, 3540, 3542, 3543, 3544, 3545, 3546, 3547, 489, 3541,
    230, 3551, 489, 3552, 230, 3553, 3554, 3554, 227, 228, 3556, 3557, 3560, 3563, 3566, 3569,
    3572, 3573, 3574, 3575, 3576, 3558, 3559, 3561, 3562, 3564, 3565, 3567, 3568, 3570, 3571, 3582,
    3584, 3592, 3585, 3586, 3589, 3587, 3588, 3590, 3591, 3595, 3599, 3600, 3601, 3596, 3597, 3598,
    3603, 3604, 3605, 3606, 3609, 3610, 3607, 58, 3612, 3613, 3614, 3615, 3618, 3619, 3616, 58,
    3621, 3622, 3624, 3623, 3628, 3629, 3630, 3631,
];

pub static TERMINATORS: &[u32] = &[
    59, 60, 136, 414, 415, 492, 493, 495, 502, 503, 505, 507, 508, 509, 546, 547,
    562, 616, 888, 889, 891, 1094, 1095, 1096, 1097, 1098, 1099, 1100, 1304, 1340, 1356, 1516,
    1519, 1528, 1531, 1547, 1556, 1565, 1636, 1640, 1750, 1772, 1809, 2454, 2455, 2870, 2871, 2874,
    2877, 2917, 2918, 2919, 3051, 3103, 3104, 3182, 3292, 3448, 3449, 3450, 3451, 3452, 3453, 3454,
    3455, 3608, 3617,
];

pub static STRINGS: &[&str] = &[
    "ABORT", // [0]
    "keyword", // [1]
    "KeywordSegment", // [2]
    "base", // [3]
    "raw", // [4]
    "word", // [5]
    "ABS", // [6]
    "ABSOLUTE", // [7]
    "ACCESS", // [8]
    "AccountKeywordSegment", // [9]
    "ACCOUNT", // [10]
    "ALL", // [11]
    "CATALOG", // [12]
    "DATABASE", // [13]
    "DOMAIN", // [14]
    "EXTERNAL", // [15]
    "FILE", // [16]
    "FOREIGN", // [17]
    "FUNCTION", // [18]
    "FUTURE", // [19]
    "INTEGRATION", // [20]
    "LANGUAGE", // [21]
    "MASKING", // [22]
    "MATERIALIZED", // [23]
    "PROCEDURE", // [24]
    "RESOURCE", // [25]
    "ROLE", // [26]
    "ROUTINE", // [27]
    "SCHEMA", // [28]
    "SEQUENCE", // [29]
    "STAGE", // [30]
    "STREAM", // [31]
    "TABLE", // [32]
    "TABLESPACE", // [33]
    "TASK", // [34]
    "TYPE", // [35]
    "VIEW", // [36]
    "WAREHOUSE", // [37]
    "ResourceKeywordSegment", // [38]
    "MonitorKeywordSegment", // [39]
    "MONITOR", // [40]
    "WarehouseKeywordSegment", // [41]
    "DatabaseKeywordSegment", // [42]
    "DomainKeywordSegment", // [43]
    "IntegrationKeywordSegment", // [44]
    "LanguageKeywordSegment", // [45]
    "SchemaKeywordSegment", // [46]
    "CatalogKeywordSegment", // [47]
    "RoleKeywordSegment", // [48]
    "TablespaceKeywordSegment", // [49]
    "TypeKeywordSegment", // [50]
    "ForeignKeywordSegment", // [51]
    "DATA", // [52]
    "SERVER", // [53]
    "ServerKeywordSegment", // [54]
    "DataKeywordSegment", // [55]
    "WrapperKeywordSegment", // [56]
    "WRAPPER", // [57]
    "AllKeywordSegment", // [58]
    "SchemasKeywordSegment", // [59]
    "SCHEMAS", // [60]
    "InKeywordSegment", // [61]
    "IN", // [62]
    "FutureKeywordSegment", // [63]
    "AccessSchemaObjectSegment", // [64]
    "access_schema_object", // [65]
    "AccessSchemaPluralObjectSegment", // [66]
    "FUNCTIONS", // [67]
    "PROCEDURES", // [68]
    "ROUTINES", // [69]
    "SEQUENCES", // [70]
    "STAGES", // [71]
    "STREAMS", // [72]
    "TABLES", // [73]
    "TASKS", // [74]
    "VIEWS", // [75]
    "access_schema_plural_object", // [76]
    "ObjectReferenceSegment", // [77]
    "object_reference", // [78]
    "FunctionNameSegment", // [79]
    "function_name", // [80]
    "FunctionParameterListGrammar", // [81]
    "(", // [82]
    "function_parameter_list", // [83]
    "WildcardIdentifierSegment", // [84]
    "wildcard_identifier", // [85]
    "CommaSegment", // [86]
    ",", // [87]
    "ToKeywordSegment", // [88]
    "TO", // [89]
    "FromKeywordSegment", // [90]
    "FROM", // [91]
    "LARGE", // [92]
    "LargeKeywordSegment", // [93]
    "ObjectKeywordSegment", // [94]
    "OBJECT", // [95]
    "NumericLiteralSegment", // [96]
    "numeric_literal", // [97]
    "APPLY", // [98]
    "CONNECT", // [99]
    "CREATE", // [100]
    "DELETE", // [101]
    "EXECUTE", // [102]
    "IMPORTED", // [103]
    "INSERT", // [104]
    "MANAGE", // [105]
    "MODIFY", // [106]
    "OPERATE", // [107]
    "OWNERSHIP", // [108]
    "READ", // [109]
    "REFERENCES", // [110]
    "REFERENCE_USAGE", // [111]
    "SELECT", // [112]
    "TEMP", // [113]
    "TEMPORARY", // [114]
    "TRIGGER", // [115]
    "TRUNCATE", // [116]
    "UPDATE", // [117]
    "USAGE", // [118]
    "USE", // [119]
    "USE_ANY_ROLE", // [120]
    "WRITE", // [121]
    "CreateKeywordSegment", // [122]
    "USER", // [123]
    "UserKeywordSegment", // [124]
    "ApplyKeywordSegment", // [125]
    "MaskingKeywordSegment", // [126]
    "PolicyKeywordSegment", // [127]
    "POLICY", // [128]
    "ExecuteKeywordSegment", // [129]
    "TaskKeywordSegment", // [130]
    "ManageKeywordSegment", // [131]
    "GrantsKeywordSegment", // [132]
    "GRANTS", // [133]
    "EXECUTION", // [134]
    "ExecutionKeywordSegment", // [135]
    "UsageKeywordSegment", // [136]
    "PIPE", // [137]
    "PipeKeywordSegment", // [138]
    "UseKeywordSegment", // [139]
    "ImportedKeywordSegment", // [140]
    "PrivilegesKeywordSegment", // [141]
    "PRIVILEGES", // [142]
    "ConnectKeywordSegment", // [143]
    "DeleteKeywordSegment", // [144]
    "InsertKeywordSegment", // [145]
    "ModifyKeywordSegment", // [146]
    "OperateKeywordSegment", // [147]
    "OwnershipKeywordSegment", // [148]
    "ReadKeywordSegment", // [149]
    "Reference_usageKeywordSegment", // [150]
    "ReferencesKeywordSegment", // [151]
    "SelectKeywordSegment", // [152]
    "TempKeywordSegment", // [153]
    "TemporaryKeywordSegment", // [154]
    "TriggerKeywordSegment", // [155]
    "TruncateKeywordSegment", // [156]
    "UpdateKeywordSegment", // [157]
    "Use_any_roleKeywordSegment", // [158]
    "WriteKeywordSegment", // [159]
    "BracketedColumnReferenceListGrammar", // [160]
    "AccessPermissionSegment", // [161]
    "access_permission", // [162]
    "OnKeywordSegment", // [163]
    "ON", // [164]
    "TableKeywordSegment", // [165]
    "ViewKeywordSegment", // [166]
    "StageKeywordSegment", // [167]
    "FunctionKeywordSegment", // [168]
    "ProcedureKeywordSegment", // [169]
    "RoutineKeywordSegment", // [170]
    "SequenceKeywordSegment", // [171]
    "StreamKeywordSegment", // [172]
    "MaterializedKeywordSegment", // [173]
    "ExternalKeywordSegment", // [174]
    "FileKeywordSegment", // [175]
    "FormatKeywordSegment", // [176]
    "FORMAT", // [177]
    "TablesKeywordSegment", // [178]
    "ViewsKeywordSegment", // [179]
    "StagesKeywordSegment", // [180]
    "FunctionsKeywordSegment", // [181]
    "ProceduresKeywordSegment", // [182]
    "RoutinesKeywordSegment", // [183]
    "SequencesKeywordSegment", // [184]
    "StreamsKeywordSegment", // [185]
    "TasksKeywordSegment", // [186]
    "PoliciesKeywordSegment", // [187]
    "FormatsKeywordSegment", // [188]
    "GRANT", // [189]
    "REVOKE", // [190]
    "GrantStatementSegment", // [191]
    "grant_statement", // [192]
    "RevokeStatementSegment", // [193]
    "revoke_statement", // [194]
    "GROUP", // [195]
    "GroupKeywordSegment", // [196]
    "UserReferenceSegment", // [197]
    "user_reference", // [198]
    "RoleReferenceSegment", // [199]
    "role_reference", // [200]
    "SHARE", // [201]
    "ShareKeywordSegment", // [202]
    "FunctionSegment", // [203]
    "function", // [204]
    "PublicKeywordSegment", // [205]
    "PUBLIC", // [206]
    "[", // [207]
    "ArrayAccessorSegment", // [208]
    "array_accessor", // [209]
    "ACCOUNTS", // [210]
    "ACTION", // [211]
    "ADA", // [212]
    "ADD", // [213]
    "ADMIN", // [214]
    "AFTER", // [215]
    "AGGREGATE", // [216]
    "OrderByClauseSegment", // [217]
    "ORDER", // [218]
    "orderby_clause", // [219]
    "indent", // [220]
    "Indent", // [221]
    "meta", // [222]
    "AsAliasOperatorSegment", // [223]
    "AS", // [224]
    "alias_operator", // [225]
    "SingleIdentifierGrammar", // [226]
    "SingleIdentifierListSegment", // [227]
    "identifier_list", // [228]
    "start_bracket", // [229]
    "SymbolSegment", // [230]
    "symbol", // [231]
    ")", // [232]
    "end_bracket", // [233]
    "dedent", // [234]
    "Dedent", // [235]
    "ALIAS", // [236]
    "TableReferenceSegment", // [237]
    "table_reference", // [238]
    "AliasExpressionSegment", // [239]
    "alias_expression", // [240]
    "ALLOCATE", // [241]
    "ALSO", // [242]
    "ALTER", // [243]
    "AlterKeywordSegment", // [244]
    "DatabaseReferenceSegment", // [245]
    "database_reference", // [246]
    "SetKeywordSegment", // [247]
    "SET", // [248]
    "DBPROPERTIES", // [249]
    "LOCATION", // [250]
    "MANAGEDLOCATION", // [251]
    "OWNER", // [252]
    "DbpropertiesKeywordSegment", // [253]
    "BracketedPropertyListGrammar", // [254]
    "OwnerKeywordSegment", // [255]
    "QuotedLiteralSegment", // [256]
    "back_quote", // [257]
    "double_quote", // [258]
    "single_quote", // [259]
    "LocationGrammar", // [260]
    "ManagedlocationKeywordSegment", // [261]
    "CACHE", // [262]
    "CYCLE", // [263]
    "INCREMENT", // [264]
    "MAXVALUE", // [265]
    "MINVALUE", // [266]
    "NO", // [267]
    "NOCACHE", // [268]
    "NOCYCLE", // [269]
    "NOORDER", // [270]
    "IncrementKeywordSegment", // [271]
    "ByKeywordSegment", // [272]
    "BY", // [273]
    "SequenceMinValueGrammar", // [274]
    "SequenceMaxValueGrammar", // [275]
    "CacheKeywordSegment", // [276]
    "NocacheKeywordSegment", // [277]
    "CycleKeywordSegment", // [278]
    "NocycleKeywordSegment", // [279]
    "OrderNoOrderGrammar", // [280]
    "SequenceReferenceSegment", // [281]
    "sequence_reference", // [282]
    "AlterSequenceOptionsSegment", // [283]
    "alter_sequence_options_segment", // [284]
    "DROP", // [285]
    "DropKeywordSegment", // [286]
    "ColumnKeywordSegment", // [287]
    "COLUMN", // [288]
    "IfExistsGrammar", // [289]
    "IF", // [290]
    "ParameterNameSegment", // [291]
    "EqualsSegment", // [292]
    "=", // [293]
    "comparison_operator", // [294]
    "LiteralGrammar", // [295]
    "+", // [296]
    "-", // [297]
    "ARRAY", // [298]
    "DATE", // [299]
    "FALSE", // [300]
    "INTERVAL", // [301]
    "NULL", // [302]
    "TIME", // [303]
    "TIMESTAMP", // [304]
    "TRUE", // [305]
    "{", // [306]
    "hive_variable", // [307]
    "NakedIdentifierSegment", // [308]
    "AddKeywordSegment", // [309]
    "ColumnDefinitionSegment", // [310]
    "column_definition", // [311]
    "FIRST", // [312]
    "FirstKeywordSegment", // [313]
    "AfterKeywordSegment", // [314]
    "ColumnReferenceSegment", // [315]
    "column_reference", // [316]
    "AlterTableDropColumnGrammar", // [317]
    "RENAME", // [318]
    "RenameKeywordSegment", // [319]
    "AsKeywordSegment", // [320]
    "EXCHANGE", // [321]
    "ExchangeKeywordSegment", // [322]
    "PartitionSpecGrammar", // [323]
    "PARTITION", // [324]
    "WithKeywordSegment", // [325]
    "WITH", // [326]
    "AlterTableOptionsGrammar", // [327]
    "TablePropertiesGrammar", // [328]
    "TBLPROPERTIES", // [329]
    "SelectStatementSegment", // [330]
    "select_statement", // [331]
    "ALWAYS", // [332]
    "&", // [333]
    "ampersand", // [334]
    "ANALYSE", // [335]
    "ANALYZE", // [336]
    "AND", // [337]
    "binary_operator", // [338]
    "BinaryOperatorSegment", // [339]
    "ANY", // [340]
    "ARCHIVE", // [341]
    "ARE", // [342]
    "%", // [343]
    "*", // [344]
    "/", // [345]
    "<", // [346]
    ">", // [347]
    "^", // [348]
    "|", // [349]
    "PlusSegment", // [350]
    "MinusSegment", // [351]
    "DivideSegment", // [352]
    "MultiplySegment", // [353]
    "ModuloSegment", // [354]
    "BitwiseAndSegment", // [355]
    "BitwiseOrSegment", // [356]
    "BitwiseXorSegment", // [357]
    "BitwiseLShiftSegment", // [358]
    "BitwiseRShiftSegment", // [359]
    "ExpressionSegment", // [360]
    "expression", // [361]
    "SliceSegment", // [362]
    ":", // [363]
    "start_square_bracket", // [364]
    "]", // [365]
    "end_square_bracket", // [366]
    "BaseExpressionElementGrammar", // [367]
    "ArrayKeywordSegment", // [368]
    "DatatypeSegment", // [369]
    "BIGINT", // [370]
    "BINARY", // [371]
    "BOOLEAN", // [372]
    "CHAR", // [373]
    "DEC", // [374]
    "DECIMAL", // [375]
    "DOUBLE", // [376]
    "FLOAT", // [377]
    "INT", // [378]
    "INTEGER", // [379]
    "JSON", // [380]
    "MAP", // [381]
    "NUMERIC", // [382]
    "SMALLINT", // [383]
    "STRING", // [384]
    "STRUCT", // [385]
    "TINYINT", // [386]
    "UNIONTYPE", // [387]
    "VARCHAR", // [388]
    "data_type", // [389]
    "start_angle_bracket", // [390]
    "end_angle_bracket", // [391]
    "ASC", // [392]
    "ASENSITIVE", // [393]
    "ASSERTION", // [394]
    "ASSIGNMENT", // [395]
    "ASYMMETRIC", // [396]
    "AT", // [397]
    "ATOMIC", // [398]
    "ATTRIBUTE", // [399]
    "ATTRIBUTES", // [400]
    "AUDIT", // [401]
    "AUTHORIZATION", // [402]
    "AUTO_INCREMENT", // [403]
    "Auto_incrementKeywordSegment", // [404]
    "AUTOCOMMIT", // [405]
    "AVG", // [406]
    "AVG_ROW_LENGTH", // [407]
    "AVRO", // [408]
    "quoted_identifier", // [409]
    "identifier", // [410]
    "IdentifierSegment", // [411]
    "BACKUP", // [412]
    "BACKWARD", // [413]
    "CURRENT_DATE", // [414]
    "CURRENT_TIME", // [415]
    "CURRENT_TIMESTAMP", // [416]
    "bare_function", // [417]
    "CodeSegment", // [418]
    "BareFunctionSegment", // [419]
    "IntervalExpressionSegment", // [420]
    "interval_expression", // [421]
    "file", // [422]
    "BaseFileSegment", // [423]
    "BaseSegment", // [424]
    "BEFORE", // [425]
    "BEGIN", // [426]
    "BERNOULLI", // [427]
    "BETWEEN", // [428]
    "!", // [429]
    "IS", // [430]
    "OR", // [431]
    "like_operator", // [432]
    "ArithmeticBinaryOperatorGrammar", // [433]
    "StringBinaryOperatorGrammar", // [434]
    "BooleanBinaryOperatorGrammar", // [435]
    "ComparisonOperatorGrammar", // [436]
    "BINDING", // [437]
    "BIT", // [438]
    "BIT_LENGTH", // [439]
    "BITVAR", // [440]
    "AmpersandSegment", // [441]
    "RawLessThanSegment", // [442]
    "PipeSegment", // [443]
    "RawGreaterThanSegment", // [444]
    "BLOB", // [445]
    "BOOL", // [446]
    "AndOperatorGrammar", // [447]
    "OrOperatorGrammar", // [448]
    "TrueSegment", // [449]
    "FalseSegment", // [450]
    "BOTH", // [451]
    "PropertyGrammar", // [452]
    "bracketed", // [453]
    "BracketedSegment", // [454]
    "UnorderedSetExpressionSegment", // [455]
    "VALUE", // [456]
    "VALUES", // [457]
    "set_expression", // [458]
    "BREADTH", // [459]
    "BREAK", // [460]
    "BROWSE", // [461]
    "BUCKET", // [462]
    "BUCKETS", // [463]
    "BULK", // [464]
    "CTEColumnList", // [465]
    "cte_column_list", // [466]
    "SelectableGrammar", // [467]
    "CALL", // [468]
    "CALLED", // [469]
    "CARDINALITY", // [470]
    "CASCADE", // [471]
    "CASCADED", // [472]
    "CASE", // [473]
    "CaseKeywordSegment", // [474]
    "implicit_indent", // [475]
    "ImplicitIndent", // [476]
    "WHEN", // [477]
    "WhenClauseSegment", // [478]
    "when_clause", // [479]
    "ElseKeywordSegment", // [480]
    "ELSE", // [481]
    "EndKeywordSegment", // [482]
    "END", // [483]
    "ElseClauseSegment", // [484]
    "else_clause", // [485]
    "BinaryOperatorGrammar", // [486]
    "CAST", // [487]
    "::", // [488]
    "casting_operator", // [489]
    "CATALOG_NAME", // [490]
    "CEIL", // [491]
    "CEILING", // [492]
    "CHAIN", // [493]
    "CHANGE", // [494]
    "CHAR_LENGTH", // [495]
    "CHARACTER", // [496]
    "CHARACTER_LENGTH", // [497]
    "CHARACTER_SET_CATALOG", // [498]
    "CHARACTER_SET_NAME", // [499]
    "CHARACTER_SET_SCHEMA", // [500]
    "CHARACTERISTICS", // [501]
    "CHARACTERS", // [502]
    "CHECK", // [503]
    "CHECKED", // [504]
    "CHECKPOINT", // [505]
    "CHECKSUM", // [506]
    "CLASS", // [507]
    "CLASS_ORIGIN", // [508]
    "CLOB", // [509]
    "CLOSE", // [510]
    "CLUSTER", // [511]
    "ClusterKeywordSegment", // [512]
    "LimitKeywordSegment", // [513]
    "LIMIT", // [514]
    "FrameClauseUnitGrammar", // [515]
    "RANGE", // [516]
    "ROWS", // [517]
    "CLUSTERED", // [518]
    "CLUSTERSTATUS", // [519]
    "COALESCE", // [520]
    "COBOL", // [521]
    "COLLATE", // [522]
    "COLLATION", // [523]
    "ObjectReferenceDelimiterGrammar", // [524]
    ".", // [525]
    "ObjectReferenceTerminatorGrammar", // [526]
    "COLLATION_CATALOG", // [527]
    "COLLATION_NAME", // [528]
    "COLLATION_SCHEMA", // [529]
    "COLLECT", // [530]
    "COLLECTION", // [531]
    "colon_delimiter", // [532]
    "colon_prefix", // [533]
    "colon", // [534]
    "ShorthandCastSegment", // [535]
    "cast_expression", // [536]
    "CONSTRAINT", // [537]
    "ConstraintKeywordSegment", // [538]
    "NOT", // [539]
    "NotKeywordSegment", // [540]
    "NullKeywordSegment", // [541]
    "CheckKeywordSegment", // [542]
    "DEFAULT", // [543]
    "DefaultKeywordSegment", // [544]
    "ColumnConstraintDefaultGrammar", // [545]
    "PRIMARY", // [546]
    "PrimaryKeyGrammar", // [547]
    "NotEnforcedGrammar", // [548]
    "UniqueKeyGrammar", // [549]
    "UNIQUE", // [550]
    "AutoIncrementGrammar", // [551]
    "ReferenceDefinitionGrammar", // [552]
    "CommentClauseSegment", // [553]
    "COMMENT", // [554]
    "comment_clause", // [555]
    "CollateKeywordSegment", // [556]
    "CollationReferenceSegment", // [557]
    "collation_reference", // [558]
    "ColumnGeneratedGrammar", // [559]
    "ColumnConstraintSegment", // [560]
    "column_constraint_segment", // [561]
    "HiveReferenceIdentifierGrammar", // [562]
    "COLUMN_NAME", // [563]
    "ColumnsExpressionNameGrammar", // [564]
    "COLUMNS", // [565]
    "comma", // [566]
    "COMMAND_FUNCTION", // [567]
    "COMMAND_FUNCTION_CODE", // [568]
    "CommentKeywordSegment", // [569]
    "comment", // [570]
    "CommentSegment", // [571]
    "COMMIT", // [572]
    "COMMITTED", // [573]
    "COMPACT", // [574]
    "COMPACTIONS", // [575]
    "GreaterThanSegment", // [576]
    "LessThanSegment", // [577]
    "GreaterThanOrEqualToSegment", // [578]
    "LessThanOrEqualToSegment", // [579]
    "NotEqualToSegment", // [580]
    "LikeOperatorSegment", // [581]
    "IsDistinctFromGrammar", // [582]
    "ComparisonOperatorSegment", // [583]
    "COMPLETION", // [584]
    "CompositeBinaryOperatorSegment", // [585]
    "CompositeComparisonOperatorSegment", // [586]
    "COMPRESS", // [587]
    "COMPUTE", // [588]
    "CONCATENATE", // [589]
    "CONDITION", // [590]
    "CONDITION_NUMBER", // [591]
    "CrossKeywordSegment", // [592]
    "CROSS", // [593]
    "JoinTypeKeywordsGrammar", // [594]
    "FULL", // [595]
    "INNER", // [596]
    "LEFT", // [597]
    "RIGHT", // [598]
    "ConditionalCrossJoinKeywordsGrammar", // [599]
    "NonStandardJoinTypeKeywordsGrammar", // [600]
    "CONF", // [601]
    "CONNECTION", // [602]
    "CONNECTION_NAME", // [603]
    "CONSTRAINT_CATALOG", // [604]
    "CONSTRAINT_NAME", // [605]
    "CONSTRAINT_SCHEMA", // [606]
    "CONSTRAINTS", // [607]
    "CONSTRUCTOR", // [608]
    "CONTAINS", // [609]
    "CONTAINSTABLE", // [610]
    "CONTINUE", // [611]
    "CONVERSION", // [612]
    "CONVERT", // [613]
    "COPY", // [614]
    "CORR", // [615]
    "CORRESPONDING", // [616]
    "COUNT", // [617]
    "COVAR_POP", // [618]
    "COVAR_SAMP", // [619]
    "CastKeywordSegment", // [620]
    "SpecificKeywordSegment", // [621]
    "SPECIFIC", // [622]
    "INSTANCE", // [623]
    "METHOD", // [624]
    "STATIC", // [625]
    "InstanceKeywordSegment", // [626]
    "StaticKeywordSegment", // [627]
    "ConstructorKeywordSegment", // [628]
    "MethodKeywordSegment", // [629]
    "FOR", // [630]
    "ForKeywordSegment", // [631]
    "AssignmentKeywordSegment", // [632]
    "IfNotExistsGrammar", // [633]
    "CommentGrammar", // [634]
    "OrReplaceGrammar", // [635]
    "TemporaryGrammar", // [636]
    "RETURNS", // [637]
    "ReturnsKeywordSegment", // [638]
    "FunctionDefinitionGrammar", // [639]
    "function_definition", // [640]
    "UniqueKeywordSegment", // [641]
    "IndexKeywordSegment", // [642]
    "INDEX", // [643]
    "IndexReferenceSegment", // [644]
    "index_reference", // [645]
    "IndexColumnDefinitionSegment", // [646]
    "index_column_definition", // [647]
    "ModelKeywordSegment", // [648]
    "MODEL", // [649]
    "OPTIONS", // [650]
    "OptionsKeywordSegment", // [651]
    "SchemaReferenceSegment", // [652]
    "schema_reference", // [653]
    "START", // [654]
    "StartKeywordSegment", // [655]
    "CreateSequenceOptionsSegment", // [656]
    "create_sequence_options_segment", // [657]
    "LIKE", // [658]
    "PARTITIONED", // [659]
    "ROW", // [660]
    "SKEWED", // [661]
    "STORED", // [662]
    "TableConstraintSegment", // [663]
    "table_constraint", // [664]
    "StoredAsGrammar", // [665]
    "PartitionedKeywordSegment", // [666]
    "ClusteredKeywordSegment", // [667]
    "SORTED", // [668]
    "SortedKeywordSegment", // [669]
    "DESC", // [670]
    "AscKeywordSegment", // [671]
    "DescKeywordSegment", // [672]
    "IntoKeywordSegment", // [673]
    "INTO", // [674]
    "BucketsKeywordSegment", // [675]
    "SkewedByClauseSegment", // [676]
    "skewed_by_clause", // [677]
    "StorageFormatGrammar", // [678]
    "LikeKeywordSegment", // [679]
    "TriggerReferenceSegment", // [680]
    "trigger_reference", // [681]
    "INSTEAD", // [682]
    "BeforeKeywordSegment", // [683]
    "InsteadKeywordSegment", // [684]
    "OfKeywordSegment", // [685]
    "OF", // [686]
    "OrKeywordSegment", // [687]
    "DEFERRABLE", // [688]
    "INITIALLY", // [689]
    "REFERENCING", // [690]
    "ReferencingKeywordSegment", // [691]
    "OldKeywordSegment", // [692]
    "OLD", // [693]
    "RowKeywordSegment", // [694]
    "NewKeywordSegment", // [695]
    "NEW", // [696]
    "DeferrableKeywordSegment", // [697]
    "InitiallyKeywordSegment", // [698]
    "ImmediateKeywordSegment", // [699]
    "IMMEDIATE", // [700]
    "DeferredKeywordSegment", // [701]
    "DEFERRED", // [702]
    "EachKeywordSegment", // [703]
    "EACH", // [704]
    "STATEMENT", // [705]
    "StatementKeywordSegment", // [706]
    "WhenKeywordSegment", // [707]
    "FunctionNameIdentifierSegment", // [708]
    "FunctionContentsSegment", // [709]
    "function_contents", // [710]
    "WithNoSchemaBindingClauseSegment", // [711]
    "with_no_schema_binding_clause", // [712]
    "CREATEDB", // [713]
    "CREATEROLE", // [714]
    "CREATEUSER", // [715]
    "CSV", // [716]
    "CUBE", // [717]
    "function_name_identifier", // [718]
    "ROLLUP", // [719]
    "CubeFunctionNameSegment", // [720]
    "RollupFunctionNameSegment", // [721]
    "GroupingExpressionList", // [722]
    "grouping_expression_list", // [723]
    "CUME_DIST", // [724]
    "CURRENT", // [725]
    "CURRENT_DEFAULT_TRANSFORM_GROUP", // [726]
    "CURRENT_PATH", // [727]
    "CURRENT_ROLE", // [728]
    "CURRENT_TRANSFORM_GROUP_FOR_TYPE", // [729]
    "CURRENT_USER", // [730]
    "CURSOR", // [731]
    "CURSOR_NAME", // [732]
    "DATABASES", // [733]
    "data_type_identifier", // [734]
    "PrimitiveTypeSegment", // [735]
    "primitive_type", // [736]
    "ArrayTypeSegment", // [737]
    "array_type", // [738]
    "SizedArrayTypeSegment", // [739]
    "sized_array_type", // [740]
    "MapKeywordSegment", // [741]
    "StructTypeSegment", // [742]
    "struct_type", // [743]
    "UniontypeKeywordSegment", // [744]
    "DATEADD", // [745]
    "DatePartFunctionName", // [746]
    "DatetimeUnitSegment", // [747]
    "DAY", // [748]
    "DAYOFYEAR", // [749]
    "DAYS", // [750]
    "HOUR", // [751]
    "HOURS", // [752]
    "MILLISECOND", // [753]
    "MINUTE", // [754]
    "MINUTES", // [755]
    "MONTH", // [756]
    "MONTHS", // [757]
    "NANO", // [758]
    "NANOS", // [759]
    "QUARTER", // [760]
    "SECOND", // [761]
    "SECONDS", // [762]
    "WEEK", // [763]
    "WEEKDAY", // [764]
    "WEEKS", // [765]
    "YEAR", // [766]
    "YEARS", // [767]
    "FunctionContentsGrammar", // [768]
    "DateKeywordSegment", // [769]
    "TimeKeywordSegment", // [770]
    "TimestampKeywordSegment", // [771]
    "IntervalKeywordSegment", // [772]
    "date_constructor_literal", // [773]
    "literal", // [774]
    "LiteralSegment", // [775]
    "DATETIME", // [776]
    "date_part", // [777]
    "DATETIME_INTERVAL_CODE", // [778]
    "DATETIME_INTERVAL_PRECISION", // [779]
    "DAY_HOUR", // [780]
    "DAY_MICROSECOND", // [781]
    "DAY_MINUTE", // [782]
    "DAY_SECOND", // [783]
    "DAYOFMONTH", // [784]
    "DAYOFWEEK", // [785]
    "DBCC", // [786]
    "DEALLOCATE", // [787]
    "DECLARE", // [788]
    "ValuesKeywordSegment", // [789]
    "DEFAULTS", // [790]
    "DEFINED", // [791]
    "DEFINER", // [792]
    "DEGREE", // [793]
    "DELAY_KEY_WRITE", // [794]
    "DELAYED", // [795]
    "FromClauseSegment", // [796]
    "from_clause", // [797]
    "WhereClauseSegment", // [798]
    "WHERE", // [799]
    "where_clause", // [800]
    "DELIMITED", // [801]
    "SemicolonSegment", // [802]
    ";", // [803]
    "DELIMITER", // [804]
    "DELIMITERS", // [805]
    "DENSE_RANK", // [806]
    "DENY", // [807]
    "DEPENDENCY", // [808]
    "DEPTH", // [809]
    "DEREF", // [810]
    "DERIVED", // [811]
    "DESCRIBE", // [812]
    "DescribeKeywordSegment", // [813]
    "DESCRIPTOR", // [814]
    "DESTROY", // [815]
    "DESTRUCTOR", // [816]
    "DETAIL", // [817]
    "DETERMINISTIC", // [818]
    "DIAGNOSTICS", // [819]
    "DICTIONARY", // [820]
    "DIRECTORIES", // [821]
    "DIRECTORY", // [822]
    "DISABLE", // [823]
    "DISCONNECT", // [824]
    "DISK", // [825]
    "DISPATCH", // [826]
    "DISTINCT", // [827]
    "DISTINCTROW", // [828]
    "DISTRIBUTE", // [829]
    "DistributeKeywordSegment", // [830]
    "SortKeywordSegment", // [831]
    "SORT", // [832]
    "HavingKeywordSegment", // [833]
    "HAVING", // [834]
    "QualifyKeywordSegment", // [835]
    "QUALIFY", // [836]
    "WindowKeywordSegment", // [837]
    "WINDOW", // [838]
    "SeparatorKeywordSegment", // [839]
    "SEPARATOR", // [840]
    "DISTRIBUTED", // [841]
    "DIV", // [842]
    "DO", // [843]
    "dot", // [844]
    "DOW", // [845]
    "RESTRICT", // [846]
    "RestrictKeywordSegment", // [847]
    "CascadeKeywordSegment", // [848]
    "DropBehaviorGrammar", // [849]
    "PurgeKeywordSegment", // [850]
    "PURGE", // [851]
    "DUMMY", // [852]
    "DUMP", // [853]
    "DYNAMIC", // [854]
    "DYNAMIC_FUNCTION", // [855]
    "DYNAMIC_FUNCTION_CODE", // [856]
    "ELEM_TYPE", // [857]
    "ELEMENT", // [858]
    "ELSEIF", // [859]
    "EmptyStructLiteralBracketsSegment", // [860]
    "struct_literal", // [861]
    "ENABLE", // [862]
    "ENCLOSED", // [863]
    "ENCODING", // [864]
    "ENCRYPTED", // [865]
    "END-EXEC", // [866]
    "}", // [867]
    "end_curly_bracket", // [868]
    "ENUM", // [869]
    "EQUALS", // [870]
    "RawEqualsSegment", // [871]
    "ERRLVL", // [872]
    "ESCAPE", // [873]
    "ESCAPED", // [874]
    "EVERY", // [875]
    "EXCEPT", // [876]
    "EXCEPTION", // [877]
    "EXCLUDE", // [878]
    "EXCLUDING", // [879]
    "EXCLUSIVE", // [880]
    "EXEC", // [881]
    "EXISTING", // [882]
    "EXISTS", // [883]
    "EXIT", // [884]
    "EXP", // [885]
    "EXPLAIN", // [886]
    "ExplainKeywordSegment", // [887]
    "InsertStatementSegment", // [888]
    "insert_statement", // [889]
    "UpdateStatementSegment", // [890]
    "update_statement", // [891]
    "DeleteStatementSegment", // [892]
    "delete_statement", // [893]
    "EXPORT", // [894]
    "EXPRESSION", // [895]
    "Expression_A_Grammar", // [896]
    "Tail_Recurse_Expression_A_Grammar", // [897]
    "LikeExpressionGrammar", // [898]
    "ILIKE", // [899]
    "IREGEXP", // [900]
    "REGEXP", // [901]
    "RLIKE", // [902]
    "InOperatorGrammar", // [903]
    "QuantifiedComparisonOperatorGrammar", // [904]
    "IsKeywordSegment", // [905]
    "IsClauseGrammar", // [906]
    "IsNullGrammar", // [907]
    "NotNullGrammar", // [908]
    "CollateGrammar", // [909]
    "BetweenKeywordSegment", // [910]
    "Expression_B_Grammar", // [911]
    "AndKeywordSegment", // [912]
    "PatternMatchingGrammar", // [913]
    "PRIOR", // [914]
    "~", // [915]
    "SignedSegmentGrammar", // [916]
    "QualifiedNumericLiteralSegment", // [917]
    "TildeSegment", // [918]
    "NotOperatorGrammar", // [919]
    "PriorKeywordSegment", // [920]
    "Tail_Recurse_Expression_B_Grammar", // [921]
    "ExistsKeywordSegment", // [922]
    "Expression_D_Grammar", // [923]
    "CaseExpressionSegment", // [924]
    "case_expression", // [925]
    "TimeZoneGrammar", // [926]
    "time_zone_grammar", // [927]
    "LateralColumnAliasExpressionGrammar", // [928]
    "LocalAliasSegment", // [929]
    "local_alias_segment", // [930]
    "Expression_D_Potential_Select_Statement_Without_Brackets", // [931]
    "SingleIdentifierWildcardSegment", // [932]
    "MapTypeSegment", // [933]
    "map_type", // [934]
    "BooleanLiteralGrammar", // [935]
    "NullLiteralSegment", // [936]
    "DateTimeLiteralGrammar", // [937]
    "ListComprehensionGrammar", // [938]
    "AccessorGrammar", // [939]
    "TypedStructLiteralSegment", // [940]
    "typed_struct_literal", // [941]
    "ArrayExpressionSegment", // [942]
    "array_expression", // [943]
    "OverlapsClauseSegment", // [944]
    "OVERLAPS", // [945]
    "overlaps_clause", // [946]
    "EXTENDED", // [947]
    "EXTENSION", // [948]
    "EXTRACT", // [949]
    "boolean_literal", // [950]
    "LiteralKeywordSegment", // [951]
    "FETCH", // [952]
    "FetchKeywordSegment", // [953]
    "NEXT", // [954]
    "NextKeywordSegment", // [955]
    "RowsKeywordSegment", // [956]
    "ONLY", // [957]
    "OnlyKeywordSegment", // [958]
    "TiesKeywordSegment", // [959]
    "TIES", // [960]
    "FIELDS", // [961]
    "INPUTFORMAT", // [962]
    "JSONFILE", // [963]
    "ORC", // [964]
    "PARQUET", // [965]
    "RCFILE", // [966]
    "SEQUENCEFILE", // [967]
    "TEXTFILE", // [968]
    "SequencefileKeywordSegment", // [969]
    "TextfileKeywordSegment", // [970]
    "RcfileKeywordSegment", // [971]
    "OrcKeywordSegment", // [972]
    "ParquetKeywordSegment", // [973]
    "AvroKeywordSegment", // [974]
    "JsonfileKeywordSegment", // [975]
    "InputformatKeywordSegment", // [976]
    "OutputformatKeywordSegment", // [977]
    "OUTPUTFORMAT", // [978]
    "MERGE", // [979]
    "MSCK", // [980]
    "DelimiterGrammar", // [981]
    "StatementSegment", // [982]
    "statement", // [983]
    "FILEFORMAT", // [984]
    "FILLFACTOR", // [985]
    "FILTER", // [986]
    "FilterKeywordSegment", // [987]
    "WhereKeywordSegment", // [988]
    "FINAL", // [989]
    "FLOAT4", // [990]
    "FLOAT8", // [991]
    "FLOOR", // [992]
    "FLUSH", // [993]
    "FOLLOWING", // [994]
    "FORCE", // [995]
    "KeyKeywordSegment", // [996]
    "KEY", // [997]
    "FORMATTED", // [998]
    "FORTRAN", // [999]
    "FORWARD", // [1000]
    "FOUND", // [1001]
    "CurrentKeywordSegment", // [1002]
    "UnboundedKeywordSegment", // [1003]
    "UNBOUNDED", // [1004]
    "PRECEDING", // [1005]
    "PrecedingKeywordSegment", // [1006]
    "FollowingKeywordSegment", // [1007]
    "RangeKeywordSegment", // [1008]
    "FREE", // [1009]
    "FREETEXT", // [1010]
    "FREETEXTTABLE", // [1011]
    "FREEZE", // [1012]
    "FromExpressionSegment", // [1013]
    "from_expression", // [1014]
    "INTERSECT", // [1015]
    "MINUS", // [1016]
    "OFFSET", // [1017]
    "UNION", // [1018]
    "LimitClauseSegment", // [1019]
    "limit_clause", // [1020]
    "OrderKeywordSegment", // [1021]
    "SetOperatorSegment", // [1022]
    "set_operator", // [1023]
    "WithDataClauseSegment", // [1024]
    "with_data_clause", // [1025]
    "OffsetKeywordSegment", // [1026]
    "PreTableFunctionKeywordsGrammar", // [1027]
    "TableExpressionSegment", // [1028]
    "table_expression", // [1029]
    "TemporalQuerySegment", // [1030]
    "temporal_query", // [1031]
    "FromClauseTerminatorGrammar", // [1032]
    "SamplingExpressionSegment", // [1033]
    "TABLESAMPLE", // [1034]
    "sample_expression", // [1035]
    "JoinLikeClauseGrammar", // [1036]
    "JoinClauseSegment", // [1037]
    "join_clause", // [1038]
    "LateralViewClauseSegment", // [1039]
    "lateral_view_clause", // [1040]
    "PostTableExpressionGrammar", // [1041]
    "MLTableExpressionSegment", // [1042]
    "ML", // [1043]
    "ml_table_expression", // [1044]
    "FromExpressionElementSegment", // [1045]
    "from_expression_element", // [1046]
    "indented_joins", // [1047]
    "OVERWRITE", // [1048]
    "OverwriteKeywordSegment", // [1049]
    "LOCAL", // [1050]
    "LocalKeywordSegment", // [1051]
    "DirectoryKeywordSegment", // [1052]
    "RowFormatClauseSegment", // [1053]
    "row_format_clause", // [1054]
    "ValuesClauseSegment", // [1055]
    "values_clause", // [1056]
    "FromInsertSourceClauseSegment", // [1057]
    "from_insert_source_clause", // [1058]
    "FromInsertClauseSegment", // [1059]
    "from_insert_clause", // [1060]
    "FULLTEXT", // [1061]
    "TrimParametersGrammar", // [1062]
    "DistinctKeywordSegment", // [1063]
    "StarSegment", // [1064]
    "FunctionContentsExpressionGrammar", // [1065]
    "AggregateOrderByClause", // [1066]
    "aggregate_order_by", // [1067]
    "IgnoreRespectNullsGrammar", // [1068]
    "IGNORE", // [1069]
    "RESPECT", // [1070]
    "EmptyStructLiteralSegment", // [1071]
    "WordSegment", // [1072]
    "DotSegment", // [1073]
    "QuotedIdentifierSegment", // [1074]
    "AnyKeywordSegment", // [1075]
    "FunctionParameterGrammar", // [1076]
    "DatePartFunctionNameSegment", // [1077]
    "DateTimeFunctionContentsSegment", // [1078]
    "RowFunctionContentsSegment", // [1079]
    "PostFunctionGrammar", // [1080]
    "FUSION", // [1081]
    "G", // [1082]
    "GENERAL", // [1083]
    "GENERATED", // [1084]
    "GET", // [1085]
    "glob_operator", // [1086]
    "GLOBAL", // [1087]
    "GO", // [1088]
    "GOTO", // [1089]
    "GrantKeywordSegment", // [1090]
    "AccessPermissionsSegment", // [1091]
    "access_permissions", // [1092]
    "AccessObjectSegment", // [1093]
    "access_object", // [1094]
    "AccessTargetSegment", // [1095]
    "access_target", // [1096]
    "OptionKeywordSegment", // [1097]
    "OPTION", // [1098]
    "AdminKeywordSegment", // [1099]
    "CopyKeywordSegment", // [1100]
    "GRANTED", // [1101]
    "GrantedKeywordSegment", // [1102]
    "Current_userKeywordSegment", // [1103]
    "Session_userKeywordSegment", // [1104]
    "SESSION_USER", // [1105]
    "GREATEST", // [1106]
    "GroupingSetsClauseSegment", // [1107]
    "GROUPING", // [1108]
    "grouping_sets_clause", // [1109]
    "CubeRollupClauseSegment", // [1110]
    "cube_rollup_clause", // [1111]
    "GroupByClauseTerminatorGrammar", // [1112]
    "GroupingKeywordSegment", // [1113]
    "SetsKeywordSegment", // [1114]
    "SETS", // [1115]
    "HANDLER", // [1116]
    "HEADER", // [1117]
    "HEAP", // [1118]
    "HIERARCHY", // [1119]
    "HIGH_PRIORITY", // [1120]
    "BackQuotedIdentifierSegment", // [1121]
    "HiveVariableSegment", // [1122]
    "HOLD", // [1123]
    "HOLD_DDLTIME", // [1124]
    "HOLDLOCK", // [1125]
    "HOST", // [1126]
    "HOSTS", // [1127]
    "HOUR_MICROSECOND", // [1128]
    "HOUR_MINUTE", // [1129]
    "HOUR_SECOND", // [1130]
    "IDENTIFIED", // [1131]
    "IDENTITY", // [1132]
    "IDENTITY_INSERT", // [1133]
    "IDENTITYCOL", // [1134]
    "IDXPROPERTIES", // [1135]
    "IfKeywordSegment", // [1136]
    "IgnoreKeywordSegment", // [1137]
    "RespectKeywordSegment", // [1138]
    "NullsKeywordSegment", // [1139]
    "NULLS", // [1140]
    "IMMUTABLE", // [1141]
    "IMPLEMENTATION", // [1142]
    "IMPLICIT", // [1143]
    "IMPORT", // [1144]
    "INCLUDE", // [1145]
    "INCLUDING", // [1146]
    "INDEXES", // [1147]
    "INDICATOR", // [1148]
    "INFILE", // [1149]
    "INFIX", // [1150]
    "INHERIT", // [1151]
    "INHERITS", // [1152]
    "INITIAL", // [1153]
    "INITIALIZE", // [1154]
    "INOUT", // [1155]
    "INPATH", // [1156]
    "INPUT", // [1157]
    "INPUTDRIVER", // [1158]
    "INSENSITIVE", // [1159]
    "INSERT_ID", // [1160]
    "INSTANTIABLE", // [1161]
    "INT1", // [1162]
    "INT2", // [1163]
    "INT3", // [1164]
    "INT4", // [1165]
    "INT8", // [1166]
    "INTEGRATIONS", // [1167]
    "INTERSECTION", // [1168]
    "INVOKER", // [1169]
    "NanLiteralSegment", // [1170]
    "NAN", // [1171]
    "UnknownLiteralSegment", // [1172]
    "NormalizedGrammar", // [1173]
    "ISAM", // [1174]
    "ISNULL", // [1175]
    "ISOLATION", // [1176]
    "ITEMS", // [1177]
    "ITERATE", // [1178]
    "JAR", // [1179]
    "ConditionalJoinKeywordsGrammar", // [1180]
    "JoinKeywordsGrammar", // [1181]
    "JOIN", // [1182]
    "SEMI", // [1183]
    "NestedJoinGrammar", // [1184]
    "indented_using_on", // [1185]
    "MatchConditionSegment", // [1186]
    "match_condition", // [1187]
    "USING", // [1188]
    "JoinOnConditionSegment", // [1189]
    "join_on_condition", // [1190]
    "JoinUsingConditionGrammar", // [1191]
    "UnconditionalJoinKeywordsGrammar", // [1192]
    "ExtendedNaturalJoinKeywordsGrammar", // [1193]
    "SemiKeywordSegment", // [1194]
    "JoinKeywordSegment", // [1195]
    "indented_on_contents", // [1196]
    "InnerKeywordSegment", // [1197]
    "FullKeywordSegment", // [1198]
    "LeftKeywordSegment", // [1199]
    "RightKeywordSegment", // [1200]
    "OuterKeywordSegment", // [1201]
    "OUTER", // [1202]
    "UsingKeywordSegment", // [1203]
    "file_format", // [1204]
    "K", // [1205]
    "KEY_MEMBER", // [1206]
    "KEY_TYPE", // [1207]
    "KEYS", // [1208]
    "KILL", // [1209]
    "LANCOMPILER", // [1210]
    "LAST", // [1211]
    "LAST_INSERT_ID", // [1212]
    "LATERAL", // [1213]
    "LateralKeywordSegment", // [1214]
    "LEADING", // [1215]
    "LEAST", // [1216]
    "LEAVE", // [1217]
    "LENGTH", // [1218]
    "LESS", // [1219]
    "LEVEL", // [1220]
    "LikeGrammar", // [1221]
    "EscapeKeywordSegment", // [1222]
    "RlikeKeywordSegment", // [1223]
    "IlikeKeywordSegment", // [1224]
    "RegexpKeywordSegment", // [1225]
    "IregexpKeywordSegment", // [1226]
    "LINENO", // [1227]
    "LINES", // [1228]
    "LISTEN", // [1229]
    "ArrayLiteralSegment", // [1230]
    "array_literal", // [1231]
    "TypedArrayLiteralSegment", // [1232]
    "typed_array_literal", // [1233]
    "ObjectLiteralSegment", // [1234]
    "object_literal", // [1235]
    "LN", // [1236]
    "LOAD", // [1237]
    "LOCALTIME", // [1238]
    "LOCALTIMESTAMP", // [1239]
    "LocationKeywordSegment", // [1240]
    "LOCATOR", // [1241]
    "LOCK", // [1242]
    "LOCKS", // [1243]
    "LOGICAL", // [1244]
    "LOGIN", // [1245]
    "LOGS", // [1246]
    "LONG", // [1247]
    "LONGBLOB", // [1248]
    "LONGTEXT", // [1249]
    "LOOP", // [1250]
    "LOW_PRIORITY", // [1251]
    "LOWER", // [1252]
    "M", // [1253]
    "MlKeywordSegment", // [1254]
    "MACRO", // [1255]
    "MAPJOIN", // [1256]
    "MATCH", // [1257]
    "MATCHED", // [1258]
    "MAX", // [1259]
    "MAX_ROWS", // [1260]
    "MAXEXTENTS", // [1261]
    "MEDIUMBLOB", // [1262]
    "MEDIUMINT", // [1263]
    "MEDIUMTEXT", // [1264]
    "MEMBER", // [1265]
    "MergeKeywordSegment", // [1266]
    "MergeMatchedClauseSegment", // [1267]
    "merge_when_matched_clause", // [1268]
    "MergeNotMatchedClauseSegment", // [1269]
    "merge_when_not_matched_clause", // [1270]
    "MatchedKeywordSegment", // [1271]
    "ThenKeywordSegment", // [1272]
    "THEN", // [1273]
    "MergeUpdateClauseSegment", // [1274]
    "merge_update_clause", // [1275]
    "MergeDeleteClauseSegment", // [1276]
    "merge_delete_clause", // [1277]
    "MergeInsertClauseSegment", // [1278]
    "merge_insert_clause", // [1279]
    "MergeIntoLiteralGrammar", // [1280]
    "AliasedTableReferenceGrammar", // [1281]
    "MergeMatchSegment", // [1282]
    "merge_match", // [1283]
    "SetClauseListSegment", // [1284]
    "set_clause_list", // [1285]
    "MESSAGE_LENGTH", // [1286]
    "MESSAGE_OCTET_LENGTH", // [1287]
    "MESSAGE_TEXT", // [1288]
    "METADATA", // [1289]
    "MIDDLEINT", // [1290]
    "MIN", // [1291]
    "MIN_ROWS", // [1292]
    "MINUTE_MICROSECOND", // [1293]
    "MINUTE_SECOND", // [1294]
    "MLSLABEL", // [1295]
    "MOD", // [1296]
    "MODE", // [1297]
    "MODIFIES", // [1298]
    "MODULE", // [1299]
    "MONTHNAME", // [1300]
    "MORE", // [1301]
    "MOVE", // [1302]
    "MsckKeywordSegment", // [1303]
    "RepairKeywordSegment", // [1304]
    "REPAIR", // [1305]
    "SYNC", // [1306]
    "SyncKeywordSegment", // [1307]
    "PartitionsKeywordSegment", // [1308]
    "PARTITIONS", // [1309]
    "MULTISET", // [1310]
    "MUMPS", // [1311]
    "MYISAM", // [1312]
    "naked_identifier", // [1313]
    "NAME", // [1314]
    "WindowSpecificationSegment", // [1315]
    "window_specification", // [1316]
    "NamedWindowExpressionSegment", // [1317]
    "named_window_expression", // [1318]
    "NAMES", // [1319]
    "null_literal", // [1320]
    "NATIONAL", // [1321]
    "NATURAL", // [1322]
    "NaturalKeywordSegment", // [1323]
    "NCHAR", // [1324]
    "NCLOB", // [1325]
    "sign_indicator", // [1326]
    "NESTING", // [1327]
    "newline", // [1328]
    "NewlineSegment", // [1329]
    "NO_DROP", // [1330]
    "NO_WRITE_TO_BINLOG", // [1331]
    "NOAUDIT", // [1332]
    "NOCHECK", // [1333]
    "NOCOMPRESS", // [1334]
    "NOCREATEDB", // [1335]
    "NOCREATEROLE", // [1336]
    "NOCREATEUSER", // [1337]
    "NOINHERIT", // [1338]
    "NOLOGIN", // [1339]
    "UnorderedSelectStatementSegment", // [1340]
    "WithCompoundStatementSegment", // [1341]
    "with_compound_statement", // [1342]
    "NonSetSelectableGrammar", // [1343]
    "BracketedSetExpressionGrammar", // [1344]
    "MergeStatementSegment", // [1345]
    "merge_statement", // [1346]
    "SetExpressionSegment", // [1347]
    "NONCLUSTERED", // [1348]
    "NONE", // [1349]
    "NORELY", // [1350]
    "NORMALIZE", // [1351]
    "NORMALIZED", // [1352]
    "NOSCAN", // [1353]
    "NOSUPERUSER", // [1354]
    "RawNotSegment", // [1355]
    "NOTHING", // [1356]
    "NOTIFY", // [1357]
    "NOTNULL", // [1358]
    "NOVALIDATE", // [1359]
    "NOWAIT", // [1360]
    "NULLABLE", // [1361]
    "NULLIF", // [1362]
    "NUMBER", // [1363]
    "ColonSegment", // [1364]
    "ObjectLiteralElementSegment", // [1365]
    "object_literal_element", // [1366]
    "start_curly_bracket", // [1367]
    "CastOperatorSegment", // [1368]
    "StartSquareBracketSegment", // [1369]
    "StartBracketSegment", // [1370]
    "OBJECTS", // [1371]
    "OCTET_LENGTH", // [1372]
    "OCTETS", // [1373]
    "OFF", // [1374]
    "OFFLINE", // [1375]
    "OFFSETS", // [1376]
    "OIDS", // [1377]
    "ONLINE", // [1378]
    "OPEN", // [1379]
    "OPENDATASOURCE", // [1380]
    "OPENQUERY", // [1381]
    "OPENROWSET", // [1382]
    "OPENXML", // [1383]
    "OPERATION", // [1384]
    "OPERATOR", // [1385]
    "OPTIMIZE", // [1386]
    "OPTIONALLY", // [1387]
    "ReplaceKeywordSegment", // [1388]
    "REPLACE", // [1389]
    "LastKeywordSegment", // [1390]
    "WithFillSegment", // [1391]
    "with_fill", // [1392]
    "NoorderKeywordSegment", // [1393]
    "ORDERING", // [1394]
    "ORDINALITY", // [1395]
    "OTHERS", // [1396]
    "OUT", // [1397]
    "OUTFILE", // [1398]
    "OUTPUT", // [1399]
    "OUTPUTDRIVER", // [1400]
    "OverKeywordSegment", // [1401]
    "OVER", // [1402]
    "OverlapsKeywordSegment", // [1403]
    "OVERLAY", // [1404]
    "OVERRIDING", // [1405]
    "PACK_KEYS", // [1406]
    "PAD", // [1407]
    "PARAMETER", // [1408]
    "parameter", // [1409]
    "?", // [1410]
    "PARAMETER_MODE", // [1411]
    "PARAMETER_NAME", // [1412]
    "PARAMETER_ORDINAL_POSITION", // [1413]
    "PARAMETER_SPECIFIC_CATALOG", // [1414]
    "PARAMETER_SPECIFIC_NAME", // [1415]
    "PARAMETER_SPECIFIC_SCHEMA", // [1416]
    "PARAMETERS", // [1417]
    "PARTIAL", // [1418]
    "PARTIALSCAN", // [1419]
    "PartitionKeywordSegment", // [1420]
    "PASCAL", // [1421]
    "PASSWORD", // [1422]
    "PATH", // [1423]
    "SlashSegment", // [1424]
    "path_segment", // [1425]
    "PCTFREE", // [1426]
    "PERCENT", // [1427]
    "PERCENT_RANK", // [1428]
    "PERCENTILE_CONT", // [1429]
    "PERCENTILE_DISC", // [1430]
    "pipe", // [1431]
    "PLACING", // [1432]
    "PLAN", // [1433]
    "PLI", // [1434]
    "PLUS", // [1435]
    "POSITION", // [1436]
    "OverClauseSegment", // [1437]
    "over_clause", // [1438]
    "FilterClauseGrammar", // [1439]
    "POSTFIX", // [1440]
    "POWER", // [1441]
    "PRECISION", // [1442]
    "PREFIX", // [1443]
    "PREORDER", // [1444]
    "PREPARE", // [1445]
    "PREPARED", // [1446]
    "PRESERVE", // [1447]
    "PRETTY", // [1448]
    "PrimaryKeywordSegment", // [1449]
    "TinyintKeywordSegment", // [1450]
    "SmallintKeywordSegment", // [1451]
    "IntKeywordSegment", // [1452]
    "IntegerKeywordSegment", // [1453]
    "BigintKeywordSegment", // [1454]
    "BooleanKeywordSegment", // [1455]
    "FloatKeywordSegment", // [1456]
    "DoubleKeywordSegment", // [1457]
    "PrecisionKeywordSegment", // [1458]
    "StringKeywordSegment", // [1459]
    "BinaryKeywordSegment", // [1460]
    "DecimalKeywordSegment", // [1461]
    "DecKeywordSegment", // [1462]
    "NumericKeywordSegment", // [1463]
    "BracketedArguments", // [1464]
    "bracketed_arguments", // [1465]
    "VarcharKeywordSegment", // [1466]
    "CharKeywordSegment", // [1467]
    "JsonKeywordSegment", // [1468]
    "PRINCIPALS", // [1469]
    "PRINT", // [1470]
    "PROC", // [1471]
    "PROCEDURAL", // [1472]
    "PROCESS", // [1473]
    "PROCESSLIST", // [1474]
    "PROTECTION", // [1475]
    "SOME", // [1476]
    "SomeKeywordSegment", // [1477]
    "QUOTE", // [1478]
    "quoted_literal", // [1479]
    "RAID0", // [1480]
    "RAISERROR", // [1481]
    "RANK", // [1482]
    "raw_comparison_operator", // [1483]
    "RAW", // [1484]
    "READONLY", // [1485]
    "READS", // [1486]
    "READTEXT", // [1487]
    "REAL", // [1488]
    "REBUILD", // [1489]
    "RECHECK", // [1490]
    "RECONFIGURE", // [1491]
    "RECORDREADER", // [1492]
    "RECORDWRITER", // [1493]
    "RECURSIVE", // [1494]
    "REDUCE", // [1495]
    "REF", // [1496]
    "ReferenceMatchGrammar", // [1497]
    "ReferentialActionGrammar", // [1498]
    "MatchKeywordSegment", // [1499]
    "SIMPLE", // [1500]
    "PartialKeywordSegment", // [1501]
    "SimpleKeywordSegment", // [1502]
    "NoKeywordSegment", // [1503]
    "ActionKeywordSegment", // [1504]
    "REGR_AVGX", // [1505]
    "REGR_AVGY", // [1506]
    "REGR_COUNT", // [1507]
    "REGR_INTERCEPT", // [1508]
    "REGR_R2", // [1509]
    "REGR_SLOPE", // [1510]
    "REGR_SXX", // [1511]
    "REGR_SXY", // [1512]
    "REGR_SYY", // [1513]
    "REINDEX", // [1514]
    "RELATIVE", // [1515]
    "RELEASE", // [1516]
    "RELOAD", // [1517]
    "RELY", // [1518]
    "REPEAT", // [1519]
    "REPEATABLE", // [1520]
    "REPLICATION", // [1521]
    "REQUIRE", // [1522]
    "RESET", // [1523]
    "RESIGNAL", // [1524]
    "RESTART", // [1525]
    "RESTORE", // [1526]
    "RESULT", // [1527]
    "RETURN", // [1528]
    "RETURNED_CARDINALITY", // [1529]
    "RETURNED_LENGTH", // [1530]
    "RETURNED_OCTET_LENGTH", // [1531]
    "RETURNED_SQLSTATE", // [1532]
    "RevokeKeywordSegment", // [1533]
    "REWRITE", // [1534]
    "ROLES", // [1535]
    "ROLLBACK", // [1536]
    "ROUTINE_CATALOG", // [1537]
    "ROUTINE_NAME", // [1538]
    "ROUTINE_SCHEMA", // [1539]
    "SERDE", // [1540]
    "DelimitedKeywordSegment", // [1541]
    "FieldsKeywordSegment", // [1542]
    "TerminatedByGrammar", // [1543]
    "TERMINATED", // [1544]
    "EscapedKeywordSegment", // [1545]
    "CollectionKeywordSegment", // [1546]
    "ItemsKeywordSegment", // [1547]
    "KeysKeywordSegment", // [1548]
    "LinesKeywordSegment", // [1549]
    "DefinedKeywordSegment", // [1550]
    "SerdeKeywordSegment", // [1551]
    "SerdePropertiesGrammar", // [1552]
    "ROW_COUNT", // [1553]
    "ROW_NUMBER", // [1554]
    "ROWCOUNT", // [1555]
    "ROWGUIDCOL", // [1556]
    "ROWID", // [1557]
    "ROWNUM", // [1558]
    "RULE", // [1559]
    "TablesampleKeywordSegment", // [1560]
    "BucketKeywordSegment", // [1561]
    "OutKeywordSegment", // [1562]
    "PercentKeywordSegment", // [1563]
    "byte_length_literal", // [1564]
    "SAVE", // [1565]
    "SAVEPOINT", // [1566]
    "SCALE", // [1567]
    "SCHEMA_NAME", // [1568]
    "SCOPE", // [1569]
    "SCOPE_CATALOG", // [1570]
    "SCOPE_NAME", // [1571]
    "SCOPE_SCHEMA", // [1572]
    "SCROLL", // [1573]
    "SEARCH", // [1574]
    "SECOND_MICROSECOND", // [1575]
    "SECTION", // [1576]
    "SECURITY", // [1577]
    "WildcardExpressionSegment", // [1578]
    "wildcard_expression", // [1579]
    "SelectClauseModifierSegment", // [1580]
    "select_clause_modifier", // [1581]
    "SelectClauseElementSegment", // [1582]
    "select_clause_element", // [1583]
    "SelectClauseTerminatorGrammar", // [1584]
    "SelectClauseSegment", // [1585]
    "select_clause", // [1586]
    "GroupByClauseSegment", // [1587]
    "groupby_clause", // [1588]
    "HavingClauseSegment", // [1589]
    "having_clause", // [1590]
    "NamedWindowSegment", // [1591]
    "named_window", // [1592]
    "OffsetClauseSegment", // [1593]
    "offset_clause", // [1594]
    "FetchClauseSegment", // [1595]
    "fetch_clause", // [1596]
    "ClusterByClauseSegment", // [1597]
    "clusterby_clause", // [1598]
    "DistributeByClauseSegment", // [1599]
    "distributeby_clause", // [1600]
    "SortByClauseSegment", // [1601]
    "sortby_clause", // [1602]
    "WithCompoundNonSelectStatementSegment", // [1603]
    "NonWithSelectableGrammar", // [1604]
    "SELF", // [1605]
    "statement_terminator", // [1606]
    "SENSITIVE", // [1607]
    "MaxvalueKeywordSegment", // [1608]
    "MinvalueKeywordSegment", // [1609]
    "SerdepropertiesKeywordSegment", // [1610]
    "SERDEPROPERTIES", // [1611]
    "SERIALIZABLE", // [1612]
    "SERVER_NAME", // [1613]
    "SESSION", // [1614]
    "SetClauseSegment", // [1615]
    "set_clause", // [1616]
    "UnionGrammar", // [1617]
    "IntersectKeywordSegment", // [1618]
    "ExceptKeywordSegment", // [1619]
    "MinusKeywordSegment", // [1620]
    "option_indicator", // [1621]
    "V", // [1622]
    "option", // [1623]
    "ColonDelimiterSegment", // [1624]
    "HiveSetValueGrammar", // [1625]
    "SETOF", // [1626]
    "SETUSER", // [1627]
    "SHARED", // [1628]
    "SHARES", // [1629]
    "SHOW", // [1630]
    "SHOW_DATABASE", // [1631]
    "SHUTDOWN", // [1632]
    "SIGNAL", // [1633]
    "PositiveSegment", // [1634]
    "NegativeSegment", // [1635]
    "SIMILAR", // [1636]
    "SIZE", // [1637]
    "SkewedKeywordSegment", // [1638]
    "StoredKeywordSegment", // [1639]
    "DirectoriesKeywordSegment", // [1640]
    "slash", // [1641]
    "slice", // [1642]
    "SNAPSHOT", // [1643]
    "SONAME", // [1644]
    "SOURCE", // [1645]
    "SPACE", // [1646]
    "SPATIAL", // [1647]
    "SPECIFIC_NAME", // [1648]
    "SPECIFICTYPE", // [1649]
    "SQL", // [1650]
    "SQL_BIG_RESULT", // [1651]
    "SQL_BIG_SELECTS", // [1652]
    "SQL_BIG_TABLES", // [1653]
    "SQL_CALC_FOUND_ROWS", // [1654]
    "SQL_LOG_OFF", // [1655]
    "SQL_LOG_UPDATE", // [1656]
    "SQL_LOW_PRIORITY_UPDATES", // [1657]
    "SQL_SELECT_LIMIT", // [1658]
    "SQL_SMALL_RESULT", // [1659]
    "SQL_WARNINGS", // [1660]
    "SQLCA", // [1661]
    "SQLCODE", // [1662]
    "SQLERROR", // [1663]
    "SQLEXCEPTION", // [1664]
    "SQLSTATE", // [1665]
    "SQLWARNING", // [1666]
    "SQRT", // [1667]
    "SSL", // [1668]
    "STABLE", // [1669]
    "star", // [1670]
    "STARTING", // [1671]
    "STARTS", // [1672]
    "STATE", // [1673]
    "DropTableStatementSegment", // [1674]
    "drop_table_statement", // [1675]
    "DropViewStatementSegment", // [1676]
    "drop_view_statement", // [1677]
    "CreateUserStatementSegment", // [1678]
    "create_user_statement", // [1679]
    "DropUserStatementSegment", // [1680]
    "drop_user_statement", // [1681]
    "TruncateStatementSegment", // [1682]
    "truncate_table", // [1683]
    "AccessStatementSegment", // [1684]
    "access_statement", // [1685]
    "CreateTableStatementSegment", // [1686]
    "create_table_statement", // [1687]
    "CreateRoleStatementSegment", // [1688]
    "create_role_statement", // [1689]
    "DropRoleStatementSegment", // [1690]
    "drop_role_statement", // [1691]
    "AlterTableStatementSegment", // [1692]
    "alter_table_statement", // [1693]
    "DropSchemaStatementSegment", // [1694]
    "drop_schema_statement", // [1695]
    "DropTypeStatementSegment", // [1696]
    "drop_type_statement", // [1697]
    "CreateDatabaseStatementSegment", // [1698]
    "create_database_statement", // [1699]
    "DropDatabaseStatementSegment", // [1700]
    "drop_database_statement", // [1701]
    "CreateIndexStatementSegment", // [1702]
    "create_index_statement", // [1703]
    "DropIndexStatementSegment", // [1704]
    "drop_index_statement", // [1705]
    "CreateViewStatementSegment", // [1706]
    "create_view_statement", // [1707]
    "CreateCastStatementSegment", // [1708]
    "create_cast_statement", // [1709]
    "DropCastStatementSegment", // [1710]
    "drop_cast_statement", // [1711]
    "CreateFunctionStatementSegment", // [1712]
    "create_function_statement", // [1713]
    "DropFunctionStatementSegment", // [1714]
    "drop_function_statement", // [1715]
    "DescribeStatementSegment", // [1716]
    "describe_statement", // [1717]
    "UseStatementSegment", // [1718]
    "use_statement", // [1719]
    "ExplainStatementSegment", // [1720]
    "explain_statement", // [1721]
    "CreateSequenceStatementSegment", // [1722]
    "create_sequence_statement", // [1723]
    "AlterSequenceStatementSegment", // [1724]
    "alter_sequence_statement", // [1725]
    "DropSequenceStatementSegment", // [1726]
    "drop_sequence_statement", // [1727]
    "CreateTriggerStatementSegment", // [1728]
    "create_trigger", // [1729]
    "DropTriggerStatementSegment", // [1730]
    "drop_trigger", // [1731]
    "FromInsertStatementSegment", // [1732]
    "from_insert_statement", // [1733]
    "AlterDatabaseStatementSegment", // [1734]
    "alter_database_statement", // [1735]
    "MsckRepairTableStatementSegment", // [1736]
    "msck_repair_table_statement", // [1737]
    "MsckTableStatementSegment", // [1738]
    "msck_table_statement", // [1739]
    "SetStatementSegment", // [1740]
    "set_statement", // [1741]
    "AlterViewStatementSegment", // [1742]
    "alter_view_statement", // [1743]
    "STATISTICS", // [1744]
    "STDDEV_POP", // [1745]
    "STDDEV_SAMP", // [1746]
    "STDIN", // [1747]
    "STDOUT", // [1748]
    "StoredByGrammar", // [1749]
    "STORAGE", // [1750]
    "FileFormatGrammar", // [1751]
    "STRAIGHT_JOIN", // [1752]
    "STREAMTABLE", // [1753]
    "STRICT", // [1754]
    "ConcatSegment", // [1755]
    "StructKeywordSegment", // [1756]
    "StructTypeSchemaSegment", // [1757]
    "struct_type_schema", // [1758]
    "STRUCTURE", // [1759]
    "STYLE", // [1760]
    "SUBCLASS_ORIGIN", // [1761]
    "SUBLIST", // [1762]
    "SUBMULTISET", // [1763]
    "SUBSTRING", // [1764]
    "SUCCESSFUL", // [1765]
    "SUM", // [1766]
    "SUMMARY", // [1767]
    "SUPERUSER", // [1768]
    "SYMMETRIC", // [1769]
    "SYNONYM", // [1770]
    "SYSDATE", // [1771]
    "SYSID", // [1772]
    "SYSTEM", // [1773]
    "SYSTEM_USER", // [1774]
    "DisableKeywordSegment", // [1775]
    "NovalidateKeywordSegment", // [1776]
    "RelyKeywordSegment", // [1777]
    "NorelyKeywordSegment", // [1778]
    "ForeignKeyGrammar", // [1779]
    "TblpropertiesKeywordSegment", // [1780]
    "TABLE_NAME", // [1781]
    "Expression_A_Unary_Operator_Grammar", // [1782]
    "Expression_C_Grammar", // [1783]
    "Expression_B_Unary_Operator_Grammar", // [1784]
    "TEMPLATE", // [1785]
    "TRANSIENT", // [1786]
    "TransientKeywordSegment", // [1787]
    "TERMINATE", // [1788]
    "TerminatedKeywordSegment", // [1789]
    "TEXT", // [1790]
    "TEXTSIZE", // [1791]
    "THAN", // [1792]
    "tilde", // [1793]
    "WITHOUT", // [1794]
    "WithoutKeywordSegment", // [1795]
    "ZoneKeywordSegment", // [1796]
    "ZONE", // [1797]
    "AtKeywordSegment", // [1798]
    "TIMESTAMPTZ", // [1799]
    "TIMEZONE_HOUR", // [1800]
    "TIMEZONE_MINUTE", // [1801]
    "TINYBLOB", // [1802]
    "TINYTEXT", // [1803]
    "TOAST", // [1804]
    "TOP", // [1805]
    "TOP_LEVEL_COUNT", // [1806]
    "TOUCH", // [1807]
    "TRAILING", // [1808]
    "TRAN", // [1809]
    "TRANSACTION", // [1810]
    "BeginKeywordSegment", // [1811]
    "CommitKeywordSegment", // [1812]
    "RollbackKeywordSegment", // [1813]
    "WORK", // [1814]
    "TransactionKeywordSegment", // [1815]
    "WorkKeywordSegment", // [1816]
    "NameKeywordSegment", // [1817]
    "ChainKeywordSegment", // [1818]
    "TRANSACTION_ACTIVE", // [1819]
    "TRANSACTIONS", // [1820]
    "TRANSACTIONS_COMMITTED", // [1821]
    "TRANSACTIONS_ROLLED_BACK", // [1822]
    "TRANSFORM", // [1823]
    "TRANSFORMS", // [1824]
    "TRANSLATE", // [1825]
    "TRANSLATION", // [1826]
    "TREAT", // [1827]
    "TRIGGER_CATALOG", // [1828]
    "TRIGGER_NAME", // [1829]
    "TRIGGER_SCHEMA", // [1830]
    "TRIM", // [1831]
    "TRUSTED", // [1832]
    "TSEQUAL", // [1833]
    "StructLiteralSegment", // [1834]
    "UESCAPE", // [1835]
    "UID", // [1836]
    "UNARCHIVE", // [1837]
    "UNCOMMITTED", // [1838]
    "NaturalJoinKeywordsGrammar", // [1839]
    "UnconditionalCrossJoinKeywordsGrammar", // [1840]
    "HorizontalJoinKeywordsGrammar", // [1841]
    "UNDER", // [1842]
    "UNDO", // [1843]
    "UNENCRYPTED", // [1844]
    "UnionKeywordSegment", // [1845]
    "UNIQUEJOIN", // [1846]
    "UNKNOWN", // [1847]
    "UNLISTEN", // [1848]
    "UNLOCK", // [1849]
    "UNNAMED", // [1850]
    "UNNEST", // [1851]
    "UNSET", // [1852]
    "UNSIGNED", // [1853]
    "UNTIL", // [1854]
    "UPDATETEXT", // [1855]
    "UPPER", // [1856]
    "URI", // [1857]
    "USER_DEFINED_TYPE_CATALOG", // [1858]
    "USER_DEFINED_TYPE_CODE", // [1859]
    "USER_DEFINED_TYPE_NAME", // [1860]
    "USER_DEFINED_TYPE_SCHEMA", // [1861]
    "USERS", // [1862]
    "UTC", // [1863]
    "UTC_DATE", // [1864]
    "UTC_TIME", // [1865]
    "UTC_TIMESTAMP", // [1866]
    "UTC_TMESTAMP", // [1867]
    "UTCTIMESTAMP", // [1868]
    "VACUUM", // [1869]
    "VALID", // [1870]
    "VALIDATE", // [1871]
    "VALIDATOR", // [1872]
    "VALUE_TYPE", // [1873]
    "ValueKeywordSegment", // [1874]
    "VAR_POP", // [1875]
    "VAR_SAMP", // [1876]
    "VARBINARY", // [1877]
    "VARCHAR2", // [1878]
    "VARCHARACTER", // [1879]
    "VARIABLE", // [1880]
    "VARIABLES", // [1881]
    "VARYING", // [1882]
    "VECTORIZATION", // [1883]
    "VERBOSE", // [1884]
    "VERSION", // [1885]
    "VOLATILE", // [1886]
    "WAITFOR", // [1887]
    "WAREHOUSES", // [1888]
    "indented_then", // [1889]
    "indented_then_contents", // [1890]
    "WHENEVER", // [1891]
    "WHILE", // [1892]
    "whitespace", // [1893]
    "WhitespaceSegment", // [1894]
    "WIDTH_BUCKET", // [1895]
    "PartitionClauseSegment", // [1896]
    "partitionby_clause", // [1897]
    "FrameClauseSegment", // [1898]
    "frame_clause", // [1899]
    "RecursiveKeywordSegment", // [1900]
    "indented_ctes", // [1901]
    "CTEDefinitionSegment", // [1902]
    "common_table_expression", // [1903]
    "NonWithNonSelectableGrammar", // [1904]
    "BindingKeywordSegment", // [1905]
    "WITHIN", // [1906]
    "WRITETEXT", // [1907]
    "X509", // [1908]
    "XML", // [1909]
    "XOR", // [1910]
    "YAML", // [1911]
    "YEAR_MONTH", // [1912]
    "ZEROFILL", // [1913]
];

pub static AUX_DATA: &[u32] = &[
    0, 1, 2, 1, 1, 4, 3, 1, 4, 5, 6, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 7, 1, 2, 1, 1, 4, 3, 1, 4, 5, 8, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1, 0, 0, 0,
    0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0,
    1, 0, 1, 0, 1, 0, 0, 0, 4294967295, 4294967295, 0, 10, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 210, 1, 2, 1, 1, 4, 3, 1, 4, 5, 211,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 212, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 213, 1, 2, 1, 1, 4, 3, 1, 4, 5, 214, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 215, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 216, 1, 2, 1, 1, 4, 3, 1, 4, 5, 82, 229, 230, 1, 229,
    3, 3, 4, 231, 232, 233, 230, 1, 233, 3, 3, 4, 231, 1, 2, 1,
    0, 236, 1, 2, 1, 1, 4, 3, 1, 4, 5, 11, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 241, 1, 2, 1, 1, 4, 3, 1, 4, 5, 242,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 243, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0,
    0, 0, 0, 1, 0, 1, 2, 1, 0, 0, 332, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 333, 334, 230, 1, 334, 3, 3, 4, 231, 335, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 336, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 337, 1, 2, 1, 1, 4, 3, 1, 4, 5, 337, 338, 339, 1, 338,
    3, 3, 338, 4, 340, 1, 2, 1, 1, 4, 3, 1, 4, 5, 98, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 341, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 342, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0,
    207, 364, 230, 1, 364, 3, 3, 4, 231, 365, 366, 230, 1, 366, 3, 3,
    4, 231, 1, 2, 0, 298, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 1, 2, 0, 346, 390, 230, 1, 390, 3, 3, 4, 231, 347, 391, 230,
    1, 391, 3, 3, 4, 231, 1, 2, 0, 224, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 392, 1, 2, 1, 1, 4, 3, 1, 4, 5, 393, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 394, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 395, 1, 2, 1, 1, 4, 3, 1, 4, 5, 396, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 397, 1, 2, 1, 1, 4, 3, 1, 4, 5, 398,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 399, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 400, 1, 2, 1, 1, 4, 3, 1, 4, 5, 401, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 402, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 403, 1, 2, 1, 1, 4, 3, 1, 4, 5, 405, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 406, 1, 2, 1, 1, 4, 3, 1, 4, 5, 407,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 408, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 257, 409, 411, 3, 409, 410, 257, 3, 3, 410, 4, 412, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 413, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 414, 415, 416, 610, 3, 417, 418, 1, 417, 2, 3, 4, 0, 422,
    423, 0, 2, 3, 422, 3, 424, 0, 1, 3, 425, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 426, 1, 2, 1, 1, 4, 3, 1, 4, 5, 427, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 428, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 370, 1, 2, 1, 1, 4, 3, 1, 4, 5, 371, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 338, 339, 9, 3, 3, 338, 4, 437, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 438, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 439, 1, 2, 1, 1, 4, 3, 1, 4, 5, 440, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 348, 338, 230, 1, 338, 3, 3, 4, 231, 445,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 446, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 372, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 451,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 1, 0,
    1, 2, 1, 1, 0, 1, 2, 1, 453, 454, 0, 2, 3, 453, 1, 2,
    1, 459, 1, 2, 1, 1, 4, 3, 1, 4, 5, 460, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 461, 1, 2, 1, 1, 4, 3, 1, 4, 5, 462,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 463, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 464, 1, 2, 1, 1, 4, 3, 1, 4, 5, 273, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 2, 1, 262, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 468, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 469, 1, 2, 1, 1, 4, 3, 1, 4, 5, 470, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 471, 1, 2, 1, 1, 4, 3, 1, 4, 5, 472,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295,
    0, 0, 473, 1, 2, 1, 1, 4, 3, 1, 4, 5, 487, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 488, 489, 230, 1, 489, 3, 3, 4, 231, 12,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 490, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 491, 1, 2, 1, 1, 4, 3, 1, 4, 5, 492, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 493, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 494, 1, 2, 1, 1, 4, 3, 1, 4, 5, 373, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 495, 1, 2, 1, 1, 4, 3, 1, 4, 5, 496,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 497, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 498, 1, 2, 1, 1, 4, 3, 1, 4, 5, 499, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 500, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 501, 1, 2, 1, 1, 4, 3, 1, 4, 5, 502, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 503, 1, 2, 1, 1, 4, 3, 1, 4, 5, 504,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 505, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 506, 1, 2, 1, 1, 4, 3, 1, 4, 5, 507, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 508, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 509, 1, 2, 1, 1, 4, 3, 1, 4, 5, 510, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 0, 511, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 518, 1, 2, 1, 1, 4, 3, 1, 4, 5, 519, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 520, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    521, 1, 2, 1, 1, 4, 3, 1, 4, 5, 4, 418, 9, 2, 3, 4,
    522, 1, 2, 1, 1, 4, 3, 1, 4, 5, 523, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 0, 0, 527, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 528, 1, 2, 1, 1, 4, 3, 1, 4, 5, 529, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 530, 1, 2, 1, 1, 4, 3, 1, 4, 5, 531,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 363, 532, 230, 1, 532, 3, 3,
    4, 231, 363, 533, 230, 1, 533, 3, 3, 4, 231, 363, 534, 230, 1, 534,
    3, 3, 4, 231, 0, 1, 2, 1, 0, 1, 2, 1, 0, 4294967295, 4294967295, 0,
    288, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 563, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 565, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    87, 566, 230, 1, 566, 3, 3, 4, 231, 567, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 568, 1, 2, 1, 1, 4, 3, 1, 4, 5, 554, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 570, 571, 10, 3, 3, 570, 4, 572, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 573, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 574, 1, 2, 1, 1, 4, 3, 1, 4, 5, 575, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 294, 583, 9, 3, 3, 294, 4, 584, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 338, 585, 0, 2, 3, 338, 294, 586,
    0, 2, 3, 294, 587, 1, 2, 1, 1, 4, 3, 1, 4, 5, 588, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 589, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 590, 1, 2, 1, 1, 4, 3, 1, 4, 5, 591, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 601, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 99, 1, 2, 1, 1, 4, 3, 1, 4, 5, 602, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 603, 1, 2, 1, 1, 4, 3, 1, 4, 5, 537,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 604, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 605, 1, 2, 1, 1, 4, 3, 1, 4, 5, 606, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 607, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 608, 1, 2, 1, 1, 4, 3, 1, 4, 5, 609, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 610, 1, 2, 1, 1, 4, 3, 1, 4, 5, 611,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 612, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 613, 1, 2, 1, 1, 4, 3, 1, 4, 5, 614, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 615, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 616, 1, 2, 1, 1, 4, 3, 1, 4, 5, 617, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 618, 1, 2, 1, 1, 4, 3, 1, 4, 5, 619,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 3, 4, 1, 0, 0, 0, 1,
    0, 1, 2, 1, 100, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0,
    1, 2, 0, 0, 1, 0, 1, 2, 1, 0, 0, 0, 0, 4294967295, 4294967295, 0,
    0, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2,
    1, 1, 2, 1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1,
    2, 1, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 713, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 714, 1, 2, 1, 1, 4, 3, 1, 4, 5, 715, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 593, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 716, 1, 2, 1, 1, 4, 3, 1, 4, 5, 717, 718, 418, 1,
    718, 2, 3, 4, 717, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1,
    2, 1, 724, 1, 2, 1, 1, 4, 3, 1, 4, 5, 725, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 414, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    726, 1, 2, 1, 1, 4, 3, 1, 4, 5, 727, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 728, 1, 2, 1, 1, 4, 3, 1, 4, 5, 415, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 416, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 729, 1, 2, 1, 1, 4, 3, 1, 4, 5, 730, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 731, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    732, 1, 2, 1, 1, 4, 3, 1, 4, 5, 263, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 52, 1, 2, 1, 1, 4, 3, 1, 4, 5, 13, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 733, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1, 734, 418, 1, 734, 2, 3, 4, 0, 1, 2, 0, 1,
    0, 1, 2, 0, 0, 299, 1, 2, 1, 1, 4, 3, 1, 4, 5, 745,
    2143, 1, 718, 418, 1, 718, 2, 3, 4, 0, 1, 0, 1, 2, 1, 0,
    259, 773, 775, 3, 773, 774, 259, 3, 3, 774, 4, 776, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 748, 749, 750, 751, 752, 753, 754, 755, 756, 757, 758,
    759, 760, 761, 762, 763, 764, 765, 766, 767, 2181, 20, 777, 418, 1, 777, 2,
    3, 4, 778, 1, 2, 1, 1, 4, 3, 1, 4, 5, 779, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 748, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    780, 1, 2, 1, 1, 4, 3, 1, 4, 5, 781, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 782, 1, 2, 1, 1, 4, 3, 1, 4, 5, 783, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 784, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 785, 1, 2, 1, 1, 4, 3, 1, 4, 5, 749, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 750, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    786, 1, 2, 1, 1, 4, 3, 1, 4, 5, 249, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 787, 1, 2, 1, 1, 4, 3, 1, 4, 5, 374, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 375, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 788, 1, 2, 1, 1, 4, 3, 1, 4, 5, 543, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 790, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    688, 1, 2, 1, 1, 4, 3, 1, 4, 5, 702, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 791, 1, 2, 1, 1, 4, 3, 1, 4, 5, 792, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 793, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 794, 1, 2, 1, 1, 4, 3, 1, 4, 5, 795, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 101, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    801, 1, 2, 1, 1, 4, 3, 1, 4, 5, 804, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 805, 1, 2, 1, 1, 4, 3, 1, 4, 5, 806, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 807, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 808, 1, 2, 1, 1, 4, 3, 1, 4, 5, 809, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 810, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    811, 1, 2, 1, 1, 4, 3, 1, 4, 5, 670, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 812, 1, 2, 1, 1, 4, 3, 1, 4, 5, 814, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 815, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 816, 1, 2, 1, 1, 4, 3, 1, 4, 5, 817, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 818, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    819, 1, 2, 1, 1, 4, 3, 1, 4, 5, 820, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 821, 1, 2, 1, 1, 4, 3, 1, 4, 5, 822, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 823, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 824, 1, 2, 1, 1, 4, 3, 1, 4, 5, 825, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 826, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    827, 1, 2, 1, 1, 4, 3, 1, 4, 5, 828, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1, 0, 829, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 841, 1, 2, 1, 1, 4, 3, 1, 4, 5, 842, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 345, 338, 230, 1, 338, 3, 3, 4, 231, 843, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 14, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 525, 844, 230, 1, 844, 3, 3, 4, 231, 376, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 845, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    3, 4, 1, 285, 1, 2, 1, 1, 4, 3, 1, 4, 5, 852, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 853, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 854, 1, 2, 1, 1, 4, 3, 1, 4, 5, 855, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 856, 1, 2, 1, 1, 4, 3, 1, 4, 5, 704,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 857, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 858, 1, 2, 1, 1, 4, 3, 1, 4, 5, 481, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 859, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 1, 862, 1, 2, 1, 1, 4, 3, 1, 4, 5, 863, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 864, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 865, 1, 2, 1, 1, 4, 3, 1, 4, 5, 866, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 867, 868, 230, 1, 868, 3, 3, 4, 231, 483,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 869, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 870, 1, 2, 1, 1, 4, 3, 1, 4, 5, 872, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 873, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 874, 1, 2, 1, 1, 4, 3, 1, 4, 5, 875, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 876, 1, 2, 1, 1, 4, 3, 1, 4, 5, 877,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 321, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 878, 1, 2, 1, 1, 4, 3, 1, 4, 5, 879, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 880, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 881, 1, 2, 1, 1, 4, 3, 1, 4, 5, 102, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 134, 1, 2, 1, 1, 4, 3, 1, 4, 5, 882,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 883, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 884, 1, 2, 1, 1, 4, 3, 1, 4, 5, 885, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 886, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 894, 1, 2, 1, 1, 4, 3, 1, 4, 5, 895, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 4294967295,
    4294967295, 0, 0, 1, 2, 1, 0, 0, 4294967295, 4294967295, 0, 0, 0, 1, 0, 0,
    1, 2, 1, 0, 1, 0, 1, 2, 1, 0, 0, 0, 947, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 948, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 0, 15, 1, 2, 1, 1, 4, 3, 1, 4, 5, 949, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 300, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    300, 950, 951, 1, 950, 5, 3, 1, 774, 4, 5, 0, 0, 0, 0, 952,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 961, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 16, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295,
    4294967295, 0, 1, 4294967295, 4294967295, 0, 1, 0, 0, 4294967295, 4294967295, 0, 984, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 985, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 2, 1, 986, 1, 2, 1, 1, 4, 3, 1, 4, 5, 989, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 312, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 990, 1, 2, 1, 1, 4, 3, 1, 4, 5, 991, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 377, 1, 2, 1, 1, 4, 3, 1, 4, 5, 992,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 993, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 994, 1, 2, 1, 1, 4, 3, 1, 4, 5, 630, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 995, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 17, 1, 2, 1, 1, 4, 3, 1, 4, 5, 177, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 998, 1, 2, 1, 1, 4, 3, 1, 4, 5, 999,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1000, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1001, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0,
    0, 0, 1009, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1010, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1011, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1012, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 2, 1,
    0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 220, 1047, 1, 0, 0, 4294967295,
    4294967295, 0, 234, 1047, 1, 1, 2, 1, 0, 0, 0, 0, 1, 0, 1, 4294967295,
    4294967295, 0, 91, 1, 2, 1, 1, 4, 3, 1, 4, 5, 595, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1061, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 18, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 5, 718, 1072, 2, 718, 5, 3, 3, 4,
    5, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 1, 0, 1, 2, 1, 660, 80,
    2, 1, 80, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 0, 0, 67,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1081, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 19, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1082, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1083, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1084, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1085, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1086, 1086, 583, 3, 1086, 294, 1086, 3, 3, 294, 4,
    1087, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1088, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1089, 1, 2, 1, 1, 4, 3, 1, 4, 5, 189, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 1101, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 133, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1106,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 0, 0, 0, 195,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 1, 0, 1, 0, 1108,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 1116,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 0, 0, 834,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1117, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1118, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1119, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1120, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 0, 1, 2, 1, 0, 307, 307, 418, 3, 307, 4, 307, 2,
    3, 4, 1123, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1124, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1125, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1126, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1127, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 751, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1128, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1129, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1130, 1, 2, 1, 1, 4, 3, 1, 4, 5, 752, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1131, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    410, 411, 9, 3, 3, 410, 4, 1132, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1133, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1134, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1135, 1, 2, 1, 1, 4, 3, 1, 4, 5, 290,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1069, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 899, 1, 2, 1, 1, 4, 3, 1, 4, 5, 700, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1141, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1142, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1143, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1144, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    103, 1, 2, 1, 1, 4, 3, 1, 4, 5, 62, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 0, 0, 1, 2, 1, 0, 1145, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1146, 1, 2, 1, 1, 4, 3, 1, 4, 5, 264,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 643, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1147, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1148, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1149, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1150, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1151, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1152, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1153, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1154, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 689, 1, 2, 1, 1, 4, 3, 1, 4, 5, 596, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1155, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1156, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1157, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1158, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    962, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1159, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 104, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    0, 1160, 1, 2, 1, 1, 4, 3, 1, 4, 5, 623, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1161, 1, 2, 1, 1, 4, 3, 1, 4, 5, 682,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1162, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1163, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1164, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1165, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1166, 1, 2, 1, 1, 4, 3, 1, 4, 5, 378, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 379, 1, 2, 1, 1, 4, 3, 1, 4, 5, 20,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1167, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1015, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1168, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 0, 301, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 674, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1169, 1, 2, 1, 1, 4, 3, 1, 4, 5, 900, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 430, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1174,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1175, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1176, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1177, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1178, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1179, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 220,
    1185, 1, 0, 234, 1185, 1, 0, 1182, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 475, 1196, 1, 1, 2, 1, 0, 234, 1196, 1, 0, 0, 1, 0, 1,
    2, 1, 380, 1, 2, 1, 1, 4, 3, 1, 4, 5, 963, 1204, 2, 1,
    1204, 4, 3, 1, 4, 5, 1205, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    997, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1206, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1207, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1208, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 9, 4, 3, 1, 4, 5,
    1209, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1210, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 21, 1, 2, 1, 1, 4, 3, 1, 4, 5, 92, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1211, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1212, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1213, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 0, 1215, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1216, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1217, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 597, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1218, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1219, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1220, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 658,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 432, 432, 583, 3, 432, 294, 432,
    3, 3, 294, 4, 0, 1, 2, 1, 0, 0, 0, 514, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1227, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1228,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1229, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 774, 951, 9, 5, 3, 1, 774, 4, 5, 774, 775, 9,
    3, 3, 774, 4, 1236, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1237, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1050, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1238, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1239, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 250, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1241, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1242, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1243, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1244, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1245, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1246, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1247, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1248, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1249, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1250, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1251, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1252, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1253, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 2, 1, 2, 3, 1, 1255, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 105, 1, 2, 1, 1, 4, 3, 1, 4, 5, 251, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 381, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1256, 1, 2, 1, 1, 4, 3, 1, 4, 5, 22, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1257, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1258, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 23, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1259, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1260, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1261, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    265, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1262, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1263, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1264, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1265, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 979, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0,
    0, 0, 1, 2, 1, 0, 220, 1185, 1, 234, 1185, 1, 1286, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1287, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1288, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1289, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 624, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1290, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 753, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1291, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1292, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1016, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    297, 338, 230, 1, 338, 3, 3, 4, 231, 754, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1293, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1294, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 755, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 266, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1043, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1295, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1296,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1297, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 649, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1298, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 106, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1299, 1, 2, 1, 1, 4, 3, 1, 4, 5, 343, 338, 230, 1, 338,
    3, 3, 4, 231, 40, 1, 2, 1, 1, 4, 3, 1, 4, 5, 756, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1300, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 757, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1301, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1302, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    980, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 344, 338, 230, 1,
    338, 3, 3, 4, 231, 1310, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1311,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1312, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2, 3, 1313, 411, 1, 1313, 3, 3, 410, 4, 1314, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 1, 0, 1319, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1171, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1171, 1320, 951, 1, 1320, 5, 3, 1, 774, 4, 5, 1321, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1322, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1324, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1325, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 297, 1326, 230, 1, 1326, 3, 3, 4, 231, 1327, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 696, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1328, 1329, 12, 3, 3, 1328, 4, 954, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 267, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1330, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1331, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1332, 1, 2, 1, 1, 4, 3, 1, 4, 5, 268, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1333, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1334, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1335, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1336, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1337, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 269, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1338, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1339, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 2, 1, 1, 2, 1, 1, 2, 1, 0, 0, 1,
    2, 1, 0, 0, 1348, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1349, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 270, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1350, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1351, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1352, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1353, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1354, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 539, 1, 2, 1, 1, 4, 3, 1, 4, 5, 539,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1356, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1357, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1358, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1359, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1360, 1, 2, 1, 1, 4, 3, 1, 4, 5, 302, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 302, 1320, 951, 1, 1320, 5, 3, 1, 774, 4, 5,
    1361, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1362, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1140, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1363, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 382, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 97, 97, 775, 3, 97, 774, 97, 3, 3, 774, 4, 95, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 0, 306, 1367, 230, 1, 1367, 3, 3,
    4, 231, 1, 2, 0, 0, 0, 1371, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1372, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1373, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 686, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1374,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1375, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 1017, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1376,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1377, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 693, 1, 2, 1, 1, 4, 3, 1, 4, 5, 164, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1378, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 957, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1379, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1380, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1381,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1382, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1383, 1, 2, 1, 1, 4, 3, 1, 4, 5, 107, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1384, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1385, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1386, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1098, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1387,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 650, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 431, 1, 2, 1, 1, 4, 3, 1, 4, 5, 431, 338, 339,
    1, 338, 3, 3, 338, 4, 964, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 0, 1, 0, 0, 218, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1394, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1395, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1396, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1397,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1202, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1398, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1399, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1400, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 978, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 1402,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 3, 4, 1, 0, 945, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1404, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1405, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1048, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 252, 1, 2, 1, 1, 4, 3, 1, 4, 5, 108,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1406, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1407, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1408, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 4, 4294967295, 1409, 418, 1, 1409, 2, 3, 4,
    1410, 1409, 230, 1, 1409, 3, 3, 4, 231, 1411, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1412, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1413, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1414, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1415, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1416, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1417, 1, 2, 1, 1, 4, 3, 1, 4, 5, 965,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1418, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1419, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1,
    2, 1, 0, 324, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1,
    2, 1, 659, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1309, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1421, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1422, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1423, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 5, 1425, 1072, 2, 1425, 5, 3, 3, 4, 5, 1, 0,
    0, 1426, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1427, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1428, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1429,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1430, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 137, 1, 2, 1, 1, 4, 3, 1, 4, 5, 349, 1431, 230,
    1, 1431, 3, 3, 4, 231, 1432, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1433, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1434, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1435, 1, 2, 1, 1, 4, 3, 1, 4, 5, 296, 338,
    230, 1, 338, 3, 3, 4, 231, 128, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1436, 1, 2, 1, 1, 4, 3, 1, 4, 5, 296, 1326, 230, 1, 1326,
    3, 3, 4, 231, 0, 1440, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1441,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1005, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1442, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1443, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1444, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1445, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1446, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1447, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1448,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 546, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 1469, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1470,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 914, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 142, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1471, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1472, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 24, 1, 2, 1, 1, 4, 3, 1, 4, 5, 68, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1473, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1474,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1475, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 206, 1, 2, 1, 1, 4, 3, 1, 4, 5, 851, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 836, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 2, 1, 760, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1478,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 258, 409, 411, 3, 409, 410, 258,
    3, 3, 410, 4, 259, 1479, 775, 3, 1479, 774, 259, 3, 3, 774, 4, 258,
    1479, 775, 3, 1479, 774, 258, 3, 3, 774, 4, 257, 1479, 775, 3, 1479, 774,
    257, 3, 3, 774, 4, 0, 1480, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1481, 1, 2, 1, 1, 4, 3, 1, 4, 5, 516, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1482, 1, 2, 1, 1, 4, 3, 1, 4, 5, 293, 1483,
    230, 1, 1483, 3, 3, 4, 231, 347, 1483, 230, 1, 1483, 3, 3, 4, 231,
    1484, 1, 2, 1, 1, 4, 3, 1, 4, 5, 346, 1483, 230, 1, 1483, 3,
    3, 4, 231, 429, 1483, 230, 1, 1483, 3, 3, 4, 231, 966, 1204, 2, 1,
    1204, 4, 3, 1, 4, 5, 109, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1485, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1486, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1487, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1488, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1489, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1490, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1491, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1492, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1493, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1494, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1495, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1496, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 1, 0, 0, 111, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 110, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 690, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 901, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1505, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1506, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1507, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1508, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1509, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1510, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1511, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1512, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1513, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1514, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1515, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1516, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1517, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1518, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 318, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1305, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1519, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1520, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1389, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1521, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1522, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1523, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1524, 1, 2, 1, 1, 4, 3, 1, 4, 5, 25, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1070, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1525, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1526, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 846, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1527, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1528, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1529, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1530, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1531, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1532, 1, 2, 1, 1, 4, 3, 1, 4, 5, 637, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 190, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1534,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 598, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 902, 1, 2, 1, 1, 4, 3, 1, 4, 5, 26, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1535, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1536, 1, 2, 1, 1, 4, 3, 1, 4, 5, 719, 718, 418, 1, 718,
    2, 3, 4, 719, 1, 2, 1, 1, 4, 3, 1, 4, 5, 27, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1537, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1538, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1539, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 69, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1, 0, 1, 2, 1, 660, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1553,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1554, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1555, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1556, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1557, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1558, 1, 2, 1, 1, 4, 3, 1, 4, 5, 517, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1559, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 5, 4294967295, 1564, 418, 1, 1564, 2, 3, 4, 0, 1, 2, 1, 1565, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1566, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1567, 1, 2, 1, 1, 4, 3, 1, 4, 5, 28, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1568, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    60, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1569, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1570, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1571, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1572, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1573, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1574, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 761, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1575, 1, 2, 1, 1, 4, 3, 1, 4, 5, 762, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1576, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1577, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 2, 8641, 2, 8643,
    3, 430, 827, 430, 539, 827, 0, 112, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1605, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1183, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 803, 1606, 230, 1, 1606, 3, 3, 4, 231, 1607, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 840, 1, 2, 1, 1, 4, 3, 1, 4, 5, 29, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 967, 1204, 2, 1, 1204, 4,
    3, 1, 4, 5, 70, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1540, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1611, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1612, 1, 2, 1, 1, 4, 3, 1, 4, 5, 53, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1613, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1614, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1105, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 0, 0, 1, 4294967295, 4294967295, 0, 248, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 297, 1621, 230, 1, 1621, 3, 3, 4, 231,
    1622, 1623, 418, 1, 1623, 2, 3, 4, 0, 4294967295, 4294967295, 0, 0, 1, 0, 0,
    1626, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1115, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1627, 1, 2, 1, 1, 4, 3, 1, 4, 5, 201, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1628, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1629, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 4294967295, 4294967295,
    0, 1630, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1631, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1632, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1633,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1636, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1500, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1,
    0, 259, 409, 411, 3, 409, 410, 259, 3, 3, 410, 4, 1637, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2,
    1, 661, 1, 2, 1, 1, 4, 3, 1, 4, 5, 345, 1641, 230, 1, 1641,
    3, 3, 4, 231, 363, 1642, 230, 1, 1642, 3, 3, 4, 231, 383, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1643, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1476, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1644, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 0, 1, 0, 832, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 668, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1645, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1646, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1647, 1, 2, 1, 1, 4, 3, 1, 4, 5, 622, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1648, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1649, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1650, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1651, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1652, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1653, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1654, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1655, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1656, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1657, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1658, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1659, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1660, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1661, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1662, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1663, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1664, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1665, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1666, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1667, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1668, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1669, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 30, 1, 2, 1, 1, 4, 3, 1, 4, 5, 71, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 344, 1670, 230, 1, 1670, 3, 3, 4, 231, 654,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1671, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1672, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1673, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 705, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 625, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1744, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1745, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1746, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1747, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1748, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1750,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 662, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1752, 1, 2, 1, 1, 4, 3, 1, 4, 5, 31, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 72, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1753, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1754, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 384, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    385, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 1,
    0, 1, 2, 0, 1759, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1760, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1761, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1762, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1763, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1764, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1765, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1766, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1767, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1768, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 231, 230, 9, 3, 3, 4, 231, 1769,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1306, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1770, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1771, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1772, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1773, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1774, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 1, 2, 1, 1, 2, 1, 0, 32, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1781, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 73, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1034, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 33, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 34, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 74, 1, 2, 1, 1, 4, 3, 1, 4, 5, 329, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 113, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1785, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 114, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1788, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1544, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1790, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 968, 1204, 2, 1, 1204, 4, 3, 1, 4, 5, 1791, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1792, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1273, 1, 2, 1, 1, 4, 3, 1, 4, 5, 960, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 915, 1793, 230, 1, 1793, 3, 3, 4, 231, 303,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 4294967295, 4294967295, 0, 304,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1799, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1800, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1801, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1802, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 386, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1803, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 89, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1804,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1805, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1806, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1807, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1808, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1809, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1810, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 1819, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1820, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1821, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1822, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1823,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1824, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1786, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1825, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1826, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1827, 1, 2, 1, 1, 4, 3, 1, 4, 5, 115, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1828, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1829,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1830, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1831, 1, 2, 1, 1, 4, 3, 1, 4, 5, 305, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 305, 950, 951, 1, 950, 5, 3, 1, 774,
    4, 5, 116, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1832, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1833, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 0, 1, 2, 1, 35, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1835,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1836, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1837, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1004, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1838, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1842, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1843, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1844, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1018, 1, 2, 1, 1, 4, 3, 1, 4, 5, 387, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 550, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1846,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1847, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1848, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1849, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1850, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1851, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 1852,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1853, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1854, 1, 2, 1, 1, 4, 3, 1, 4, 5, 117, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1855, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1856, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1857, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 118, 1, 2, 1, 1, 4, 3, 1, 4, 5, 119,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 120, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 123, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1858, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1859, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1860, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1861, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1862, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1188,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1863, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1864, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1865, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1866, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1867, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1868, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1869, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1870,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1871, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1872, 1, 2, 1, 1, 4, 3, 1, 4, 5, 456, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1873, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 1, 0, 1, 2, 1, 1, 0, 457, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1875, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1876, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1877, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1878, 1, 2, 1, 1, 4, 3, 1, 4, 5, 388, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1879, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1880, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1881, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1882, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1883, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1884, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1885, 1, 2, 1, 1, 4, 3, 1, 4, 5, 36, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 75, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1886, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1887, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 37, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1888, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 763, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 764, 1, 2, 1, 1, 4, 3, 1, 4, 5, 765, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 220, 1889, 1, 475, 1890, 1, 234, 1890, 1, 234,
    1889, 1, 477, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1891, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 0, 799, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1892, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1893,
    1894, 12, 3, 3, 4, 1893, 1895, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 4294967295, 4294967295, 0, 838, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    220, 1901, 1, 1, 0, 234, 1901, 1, 220, 1901, 1, 1, 0, 234, 1901, 1,
    326, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1906, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1794, 1, 2, 1, 1, 4, 3, 1, 4, 5, 5, 1072,
    9, 3, 3, 4, 5, 1814, 1, 2, 1, 1, 4, 3, 1, 4, 5, 57,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 121, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1907, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1908, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1909, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1910, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1911, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 766, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1912,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 767, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1913, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1797, 1, 2,
    1, 1, 4, 3, 1, 4, 5,
];

pub static AUX_DATA_OFFSETS: &[u32] = &[
    0, 10, 20, 30, 46, 9, 0, 42, 0, 38, 39, 41, 42, 43, 44, 45,
    46, 47, 48, 49, 50, 0, 51, 40, 54, 0, 55, 56, 0, 58, 59, 61,
    42, 0, 63, 59, 61, 42, 64, 0, 58, 66, 61, 46, 0, 63, 66, 61,
    41, 42, 46, 44, 43, 77, 0, 79, 81, 84, 86, 88, 90, 0, 93, 94,
    96, 52, 0, 122, 47, 48, 124, 41, 42, 44, 0, 125, 126, 127, 0, 129,
    130, 0, 131, 132, 0, 39, 48, 135, 136, 0, 51, 0, 122, 49, 46, 0,
    126, 127, 138, 64, 0, 139, 50, 46, 47, 0, 140, 141, 125, 143, 122, 144,
    129, 145, 146, 39, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 136,
    158, 159, 0, 58, 141, 160, 53, 161, 163, 55, 165, 166, 167, 168, 169, 170,
    171, 172, 130, 46, 0, 126, 127, 0, 173, 166, 0, 174, 165, 0, 175, 176,
    56, 178, 179, 180, 181, 182, 183, 184, 185, 186, 59, 0, 126, 187, 0, 173,
    179, 0, 174, 178, 0, 175, 188, 57, 191, 193, 70, 0, 196, 58, 77, 0,
    124, 60, 197, 0, 48, 62, 199, 0, 202, 64, 77, 66, 199, 68, 203, 205,
    71, 208, 75, 85, 95, 105, 115, 125, 135, 145, 217, 0, 220, 223, 176, 0,
    226, 173, 227, 155, 164, 226, 234, 177, 0, 237, 239, 187, 197, 207, 0, 244,
    217, 42, 46, 245, 247, 219, 0, 253, 254, 0, 255, 218, 124, 48, 256, 260,
    0, 261, 256, 220, 232, 0, 271, 272, 96, 274, 275, 230, 0, 276, 96, 277,
    231, 278, 279, 280, 0, 244, 171, 281, 233, 283, 0, 286, 287, 289, 226, 242,
    0, 291, 292, 237, 295, 308, 0, 238, 309, 146, 287, 310, 240, 0, 239, 313,
    314, 315, 160, 317, 0, 319, 241, 320, 88, 237, 0, 322, 323, 325, 165, 237,
    0, 244, 165, 237, 243, 327, 0, 244, 166, 237, 249, 0, 247, 328, 0, 320,
    248, 245, 330, 250, 260, 269, 279, 289, 299, 308, 318, 328, 338, 348, 350, 351,
    352, 353, 354, 355, 356, 357, 358, 359, 370, 350, 349, 96, 360, 362, 352, 361,
    0, 373, 385, 383, 367, 0, 368, 406, 369, 388, 397, 0, 320, 409, 419, 429,
    439, 449, 459, 469, 479, 489, 499, 509, 519, 0, 404, 529, 539, 549, 559, 569,
    579, 590, 600, 613, 622, 295, 419, 420, 203, 315, 360, 0, 369, 295, 86, 320,
    623, 629, 634, 644, 654, 664, 674, 684, 694, 433, 434, 435, 436, 695, 702, 712,
    722, 732, 441, 0, 442, 442, 443, 0, 444, 444, 742, 751, 761, 771, 447, 448,
    772, 782, 449, 450, 783, 795, 793, 295, 800, 798, 315, 805, 803, 452, 808, 814,
    455, 817, 827, 837, 847, 857, 867, 877, 887, 227, 0, 226, 465, 320, 890, 467,
    893, 903, 913, 923, 933, 943, 961, 0, 474, 475, 953, 478, 480, 482, 484, 482,
    482, 0, 474, 360, 957, 478, 480, 482, 484, 482, 482, 436, 86, 486, 962, 972,
    982, 991, 1001, 1011, 1021, 1031, 1041, 0, 1051, 1061, 1071, 1081, 1091, 1101, 1111, 1121,
    1131, 1141, 1151, 1161, 1171, 1181, 1191, 1201, 1211, 0, 512, 272, 1222, 0, 1221, 315,
    96, 360, 513, 515, 1224, 1234, 1244, 1254, 1264, 1274, 0, 1280, 1290, 1302, 256, 1300,
    226, 524, 526, 1303, 1313, 1323, 1333, 1343, 1353, 1362, 1371, 1380, 535, 295, 203, 419,
    0, 0, 538, 77, 1384, 0, 540, 541, 0, 542, 1381, 360, 0, 544, 545, 0,
    547, 548, 549, 551, 0, 552, 548, 553, 0, 556, 557, 559, 0, 226, 369, 1385,
    0, 1388, 560, 0, 1392, 1402, 562, 524, 526, 1404, 0, 564, 0, 0, 1414, 1424,
    1433, 1443, 0, 569, 256, 0, 569, 256, 1453, 1463, 1470, 1480, 1490, 1500, 1510, 292,
    576, 577, 578, 579, 580, 581, 582, 1511, 1518, 1528, 1534, 1540, 1550, 0, 443, 443,
    1560, 1570, 1580, 592, 1590, 594, 599, 600, 1591, 1601, 1611, 1621, 1631, 1641, 1651, 1661,
    1671, 1681, 1691, 1701, 1711, 1721, 1731, 1741, 1751, 1761, 1771, 1781, 1791, 0, 122, 620,
    1801, 369, 320, 369, 325, 621, 1805, 170, 168, 169, 0, 1804, 626, 627, 628, 629,
    79, 81, 0, 631, 77, 0, 320, 632, 0, 122, 1806, 42, 46, 633, 245, 634,
    260, 0, 261, 256, 0, 325, 253, 254, 0, 122, 635, 636, 168, 633, 79, 81,
    0, 638, 369, 639, 0, 122, 635, 641, 642, 633, 644, 163, 237, 0, 1809, 1807,
    646, 1812, 0, 122, 635, 648, 633, 77, 0, 651, 1830, 1828, 0, 291, 292, 1827,
    295, 1824, 1822, 256, 320, 467, 0, 122, 48, 633, 199, 0, 122, 46, 633, 652,
    1835, 0, 271, 272, 96, 0, 655, 325, 96, 274, 275, 1833, 0, 276, 96, 277,
    1834, 278, 279, 280, 0, 122, 171, 281, 1836, 656, 0, 122, 154, 174, 165, 633,
    237, 1861, 0, 1843, 1841, 1840, 663, 0, 310, 634, 634, 665, 0, 666, 272, 1848,
    1846, 0, 310, 634, 0, 667, 272, 160, 0, 669, 272, 1854, 1852, 0, 315, 1851,
    671, 672, 673, 96, 675, 665, 676, 678, 260, 328, 634, 0, 320, 1860, 1857, 467,
    0, 679, 237, 260, 328, 0, 122, 155, 680, 1862, 683, 314, 0, 684, 685, 1866,
    1865, 145, 144, 0, 157, 685, 1863, 315, 687, 163, 687, 163, 163, 237, 1874, 0,
    691, 692, 694, 320, 291, 695, 694, 320, 291, 0, 90, 237, 1869, 0, 540, 697,
    0, 697, 1868, 0, 698, 699, 0, 698, 701, 0, 631, 703, 1870, 694, 706, 0,
    707, 1871, 360, 0, 129, 169, 708, 709, 0, 122, 124, 199, 0, 122, 635, 166,
    633, 237, 160, 320, 1881, 1878, 467, 711, 1882, 1892, 1902, 1912, 1922, 1932, 1940, 0,
    1950, 720, 721, 1951, 722, 1954, 1964, 1974, 1984, 1994, 2004, 2014, 2024, 2034, 2044, 2054,
    2064, 2074, 2084, 2094, 2104, 2123, 2114, 226, 308, 2132, 735, 737, 739, 0, 741, 2124,
    0, 735, 86, 369, 742, 0, 744, 2129, 2127, 369, 2133, 2144, 746, 0, 2156, 2154,
    2153, 747, 768, 0, 2159, 769, 770, 771, 772, 2160, 2171, 2201, 2210, 2220, 2230, 2240,
    2250, 2260, 2270, 2280, 2290, 2300, 2310, 2320, 2330, 2340, 2350, 2360, 2370, 2380, 0, 544,
    789, 2390, 2400, 2410, 2420, 2430, 2440, 2450, 2460, 2470, 0, 144, 796, 798, 2480, 802,
    2490, 2500, 2510, 2520, 2530, 2540, 2550, 2560, 2570, 2580, 0, 813, 308, 77, 2590, 2600,
    2610, 2620, 2630, 2640, 2650, 2660, 2670, 2680, 2690, 2700, 2710, 2720, 2730, 0, 830, 272,
    2741, 0, 2740, 315, 96, 360, 831, 513, 833, 835, 837, 515, 839, 2743, 2753, 2763,
    2773, 2782, 2792, 2802, 2811, 2821, 2831, 847, 848, 0, 286, 620, 2832, 369, 320, 369,
    849, 0, 286, 42, 289, 245, 849, 0, 286, 168, 289, 79, 0, 286, 642, 289,
    644, 849, 2835, 0, 286, 648, 289, 77, 0, 286, 48, 289, 226, 0, 286, 46,
    289, 652, 849, 0, 286, 171, 281, 0, 286, 165, 289, 237, 850, 0, 286, 155,
    289, 680, 0, 286, 50, 289, 77, 849, 0, 286, 124, 289, 199, 0, 286, 166,
    289, 237, 849, 2845, 2855, 2865, 2875, 2885, 2895, 2905, 2915, 0, 480, 360, 2925, 2935,
    2945, 0, 742, 860, 2948, 2958, 2968, 2978, 2988, 2998, 3007, 3017, 3027, 0, 871, 871,
    3037, 3047, 3057, 3067, 3077, 3087, 3097, 3107, 3117, 3127, 3137, 3147, 3157, 3167, 3177, 3187,
    3197, 3207, 0, 887, 3217, 467, 888, 890, 892, 3218, 3228, 896, 0, 897, 3239, 3238,
    898, 0, 486, 897, 903, 904, 0, 905, 540, 906, 907, 908, 909, 0, 540, 910,
    911, 912, 897, 0, 913, 896, 3243, 916, 0, 917, 918, 919, 920, 0, 921, 3246,
    3245, 0, 3244, 433, 434, 436, 921, 3250, 916, 0, 917, 918, 3259, 0, 922, 3251,
    467, 0, 3254, 923, 924, 3255, 926, 535, 86, 0, 3274, 419, 203, 3264, 3263, 928,
    360, 467, 3261, 3260, 295, 315, 203, 929, 360, 931, 932, 0, 3267, 742, 933, 3270,
    3268, 360, 0, 369, 3273, 256, 96, 935, 936, 937, 929, 938, 86, 939, 3275, 330,
    295, 420, 940, 942, 315, 944, 3276, 0, 3286, 3296, 226, 524, 526, 3298, 3308, 3318,
    3328, 0, 953, 3339, 313, 955, 3340, 96, 360, 694, 3341, 694, 956, 3342, 958, 0,
    325, 959, 3343, 3353, 3363, 969, 970, 971, 972, 973, 974, 975, 0, 976, 256, 977,
    256, 3364, 0, 3374, 981, 3382, 982, 3378, 981, 3384, 981, 3388, 3398, 0, 987, 3408,
    0, 988, 360, 3411, 3421, 3431, 3441, 3451, 3461, 3471, 3481, 3491, 3501, 3511, 0, 51,
    996, 3521, 3531, 3541, 3551, 3561, 3571, 0, 515, 3584, 3583, 0, 1002, 694, 0, 3581,
    96, 420, 1003, 315, 3582, 1006, 1007, 0, 910, 912, 3585, 956, 1008, 3586, 3596, 3606,
    3616, 0, 90, 3626, 1013, 3628, 988, 1019, 0, 196, 272, 0, 512, 272, 0, 830,
    272, 0, 831, 272, 0, 1021, 272, 833, 835, 837, 1022, 711, 1024, 953, 1026, 0,
    1027, 3632, 3629, 1028, 1028, 1030, 239, 3633, 1032, 1033, 1036, 1037, 0, 325, 1026, 239,
    1033, 3634, 1039, 1041, 3656, 3653, 0, 3641, 1042, 1045, 3638, 1013, 0, 1021, 272, 0,
    196, 272, 3642, 3646, 0, 3645, 1037, 1036, 0, 1021, 272, 0, 196, 272, 3650, 0,
    145, 3659, 0, 1049, 3657, 0, 165, 237, 323, 633, 467, 145, 0, 0, 1051, 1052,
    256, 1053, 665, 467, 145, 0, 673, 165, 237, 323, 160, 3658, 467, 145, 1055, 0,
    90, 3660, 1013, 0, 1057, 3662, 1059, 3666, 3676, 3686, 360, 3702, 360, 0, 360, 320,
    369, 0, 1062, 360, 90, 90, 360, 0, 3696, 747, 360, 90, 360, 0, 1063, 3699,
    1064, 3697, 1065, 1066, 0, 839, 295, 0, 3700, 256, 226, 315, 61, 3701, 256, 226,
    315, 1068, 646, 1071, 0, 3706, 768, 0, 320, 256, 0, 45, 308, 3709, 3719, 0,
    3729, 0, 226, 1073, 454, 3733, 708, 1074, 454, 3736, 0, 291, 3734, 0, 1075, 50,
    369, 3735, 0, 1075, 50, 369, 3739, 3737, 1076, 3758, 0, 0, 1077, 1078, 0, 3742,
    1079, 320, 694, 3754, 3752, 0, 226, 369, 0, 0, 79, 3757, 1077, 1055, 709, 1080,
    3759, 3769, 3779, 3789, 3799, 3809, 3819, 3829, 3840, 3850, 3860, 3870, 0, 1090, 3880, 0,
    1091, 163, 1093, 0, 48, 199, 0, 148, 163, 124, 197, 77, 88, 1095, 3881, 0,
    325, 1090, 1097, 0, 325, 1099, 1097, 0, 1100, 1002, 132, 0, 1102, 272, 3882, 1103,
    1104, 197, 3883, 3893, 0, 444, 871, 444, 3903, 0, 196, 272, 3916, 58, 1107, 1110,
    0, 3914, 3913, 315, 96, 360, 1112, 1107, 3918, 0, 3917, 1021, 512, 830, 831, 272,
    513, 833, 835, 837, 3919, 0, 3933, 3932, 315, 96, 360, 3929, 1112, 3935, 0, 1113,
    1114, 3948, 3946, 3945, 1110, 722, 3951, 0, 833, 3964, 3961, 360, 3966, 0, 3965, 1021,
    512, 830, 831, 272, 513, 835, 837, 3967, 3977, 3987, 3997, 4007, 4017, 308, 1074, 1121,
    1122, 1073, 4023, 4020, 4018, 360, 360, 4024, 4034, 4044, 4054, 0, 4064, 4074, 4084, 4094,
    4104, 4114, 4124, 4134, 4144, 4151, 4161, 4171, 4181, 0, 1136, 922, 4191, 0, 1136, 540,
    922, 4201, 0, 4211, 1137, 1138, 1139, 4212, 4222, 4232, 4242, 4252, 4262, 4272, 4282, 0,
    540, 61, 4298, 4295, 4294, 4292, 896, 467, 203, 1122, 4299, 4309, 4319, 0, 226, 4329,
    671, 672, 4330, 4340, 4350, 4360, 4370, 4380, 4390, 4400, 4410, 4420, 4430, 4440, 4450, 4460,
    4470, 4480, 4490, 4500, 0, 145, 4512, 0, 1049, 4510, 0, 165, 237, 323, 633, 467,
    0, 0, 1051, 1052, 256, 1053, 665, 467, 0, 673, 165, 237, 323, 160, 4511, 467,
    1055, 4513, 4523, 4533, 4543, 4553, 4563, 4573, 4583, 4593, 4603, 4613, 4623, 4633, 4643, 4653,
    0, 772, 4667, 0, 4666, 256, 96, 4663, 360, 747, 0, 88, 747, 4668, 4678, 4688,
    4698, 4708, 936, 1170, 1172, 935, 1173, 0, 905, 540, 1063, 90, 4709, 0, 4719, 4729,
    4739, 4749, 4759, 4769, 4790, 0, 1180, 1181, 1045, 4779, 1184, 0, 4783, 1186, 4786, 1189,
    1191, 4787, 0, 1192, 1181, 1045, 1186, 0, 1193, 1045, 4791, 0, 0, 1194, 1195, 0,
    0, 163, 4801, 4807, 4804, 360, 4808, 4812, 1197, 0, 4811, 1198, 1199, 1200, 1201, 0,
    1203, 4815, 4813, 226, 4818, 4828, 4838, 4848, 4858, 4868, 4878, 4888, 4896, 4906, 4916, 4926,
    4936, 4946, 0, 4956, 0, 1214, 166, 1201, 203, 226, 0, 320, 4966, 226, 4968, 4978,
    4988, 4998, 5008, 5018, 0, 442, 871, 442, 5028, 0, 0, 540, 1221, 896, 0, 1222,
    897, 5038, 679, 1223, 1224, 1225, 1226, 5039, 5049, 0, 513, 5064, 5061, 5060, 96, 360,
    58, 5066, 0, 1026, 5065, 96, 360, 0, 86, 96, 5067, 5077, 5087, 0, 5097, 5107,
    256, 96, 935, 917, 936, 937, 1122, 1230, 1232, 1234, 5108, 5117, 5124, 5134, 0, 5144,
    5154, 5164, 0, 1240, 256, 5174, 5184, 5194, 5204, 5214, 5224, 5234, 5244, 5254, 5264, 5274,
    5284, 5294, 5304, 0, 1254, 1073, 226, 5317, 0, 648, 77, 0, 86, 5314, 467, 5320,
    5330, 5340, 5350, 0, 5360, 5370, 0, 5380, 5390, 5400, 5410, 5420, 5430, 5440, 5450, 5460,
    5470, 5480, 144, 0, 145, 160, 1055, 0, 1266, 673, 5490, 5500, 1267, 1269, 0, 707,
    1271, 0, 912, 360, 1272, 5504, 1274, 1276, 0, 707, 540, 1271, 0, 912, 360, 1272,
    1278, 0, 1280, 5505, 237, 1281, 1203, 5509, 237, 1281, 0, 5506, 467, 239, 0, 203,
    239, 5510, 1189, 5513, 1282, 0, 157, 1284, 5516, 5526, 5536, 5546, 5556, 5566, 5576, 5586,
    5596, 5606, 5616, 5625, 5635, 5645, 5655, 5665, 5675, 5685, 5695, 5705, 5715, 5725, 5735, 5745,
    5755, 5764, 5774, 5784, 5794, 5804, 5814, 5824, 0, 1303, 1304, 165, 237, 0, 5834, 309,
    286, 1307, 1308, 0, 1303, 165, 237, 0, 5835, 309, 286, 1307, 1308, 5836, 5845, 5855,
    5865, 5875, 5885, 0, 226, 320, 5898, 226, 5895, 1315, 0, 837, 5899, 1317, 5901, 5911,
    5921, 5932, 0, 1323, 594, 5942, 5952, 5962, 5972, 0, 5981, 5991, 6001, 6008, 6018, 6028,
    6038, 6048, 6058, 6068, 6078, 6088, 6098, 6108, 6118, 6128, 6138, 6157, 1055, 1340, 6148, 330,
    6151, 1341, 6154, 1343, 1344, 0, 6158, 890, 888, 892, 1345, 6163, 1347, 6162, 6159, 330,
    1343, 6164, 6174, 6184, 6194, 6204, 0, 6214, 6224, 6234, 0, 6244, 0, 1355, 871, 0,
    442, 444, 6245, 0, 6255, 6265, 6275, 6285, 6295, 6305, 6315, 6325, 6336, 6346, 6356, 6366,
    6376, 6386, 6397, 0, 256, 1364, 367, 6418, 6407, 1365, 6409, 6421, 1073, 0, 1073, 1073,
    6422, 163, 320, 1203, 86, 1368, 1369, 1370, 486, 1364, 981, 1036, 6423, 6433, 6443, 6453,
    6463, 6473, 0, 1026, 6483, 96, 360, 694, 6484, 694, 956, 6485, 6495, 6505, 6515, 6525,
    6535, 6545, 6555, 6565, 6575, 6585, 6595, 6605, 6615, 6625, 6635, 6645, 6655, 6665, 6675, 6685,
    0, 687, 1388, 6694, 0, 1021, 272, 6707, 0, 6704, 315, 96, 360, 6705, 671, 672,
    0, 1139, 6706, 313, 1390, 1391, 1019, 515, 6709, 1019, 833, 835, 837, 515, 839, 953,
    6710, 6720, 1021, 1393, 6721, 6731, 6741, 6751, 6761, 6771, 6781, 6791, 6801, 0, 1068, 1401,
    6814, 226, 6811, 1315, 6815, 0, 1403, 6828, 0, 6825, 937, 86, 937, 315, 6829, 6839,
    6849, 6859, 6869, 6879, 6889, 6899, 6909, 6919, 6928, 6937, 6947, 6957, 6967, 6977, 6987, 6997,
    7007, 7017, 7027, 0, 1420, 272, 7042, 7039, 7037, 360, 7043, 0, 1420, 7055, 7053, 0,
    315, 0, 292, 295, 7058, 7068, 7078, 7088, 7098, 7120, 0, 1424, 7118, 7108, 1424, 256,
    0, 7121, 7131, 7141, 7151, 7161, 7171, 7181, 7190, 7200, 7210, 7220, 7230, 7239, 7249, 7259,
    7268, 1437, 1439, 0, 7269, 7279, 0, 7289, 7299, 7309, 7319, 7329, 7339, 7349, 7359, 0,
    1449, 996, 7369, 7380, 1450, 1451, 1452, 1453, 1454, 1455, 1456, 0, 1457, 1458, 1459, 1460,
    771, 0, 7379, 1461, 1462, 1463, 1464, 769, 1466, 1467, 1468, 7381, 7391, 7401, 7411, 7421,
    7431, 7441, 7451, 7461, 7471, 0, 256, 292, 256, 7481, 7491, 7501, 0, 916, 96, 7511,
    0, 436, 7521, 58, 1075, 1477, 7522, 467, 7525, 7535, 7545, 7589, 7556, 7567, 7578, 7590,
    7600, 7610, 7620, 7630, 7639, 7648, 7658, 7667, 7676, 7686, 7696, 7706, 7716, 7726, 7736, 7746,
    7756, 7766, 7776, 7786, 7796, 7806, 0, 151, 237, 160, 1497, 7816, 0, 163, 144, 1498,
    0, 163, 157, 1498, 0, 1499, 7820, 1198, 1501, 1502, 7821, 7831, 7841, 7851, 847, 848,
    0, 247, 541, 0, 1503, 1504, 0, 247, 544, 7852, 7862, 7872, 7882, 7892, 7902, 7912,
    7922, 7932, 7942, 7952, 7962, 7972, 7982, 7992, 8002, 8012, 8022, 8032, 8042, 8052, 8062, 8072,
    8082, 8092, 8102, 8112, 8122, 8132, 8142, 8152, 8162, 8172, 8182, 8192, 8202, 8212, 0, 1533,
    0, 1090, 1097, 631, 8222, 0, 1091, 163, 1093, 0, 48, 199, 0, 148, 163, 124,
    197, 197, 90, 1095, 849, 8223, 8233, 8243, 8253, 226, 8263, 8273, 8283, 8291, 8301, 8311,
    8321, 8331, 8341, 0, 694, 176, 8351, 0, 1541, 0, 1542, 1543, 0, 1545, 272, 256,
    0, 1546, 1547, 1543, 0, 741, 1548, 1543, 0, 1549, 1543, 0, 541, 1550, 320, 256,
    0, 1551, 256, 1552, 0, 8354, 8352, 0, 367, 8357, 8367, 8377, 8387, 8397, 8407, 8417,
    8427, 8437, 0, 1560, 8459, 8458, 0, 1561, 96, 1562, 685, 96, 0, 163, 8447, 226,
    203, 0, 96, 8448, 1563, 956, 8449, 239, 8462, 8472, 8482, 8492, 8502, 8512, 8522, 8532,
    8542, 8552, 8562, 8572, 8582, 8592, 8602, 8612, 8622, 8632, 1578, 0, 367, 239, 8633, 1063,
    58, 0, 152, 1580, 8634, 1582, 1584, 0, 512, 272, 0, 830, 272, 0, 831, 272,
    8646, 90, 8636, 988, 0, 512, 272, 0, 830, 272, 0, 831, 272, 0, 1021, 272,
    1019, 1403, 1022, 953, 8647, 0, 1585, 796, 798, 1587, 1589, 944, 1591, 217, 1593, 1595,
    1597, 1599, 1601, 1019, 1591, 1022, 711, 1024, 8668, 8660, 8657, 1341, 8664, 8661, 1603, 1604,
    8665, 467, 8669, 8679, 8689, 8698, 8708, 8718, 8728, 0, 1608, 96, 0, 1503, 1608, 8729,
    0, 1609, 96, 0, 1503, 1609, 8730, 8740, 8750, 0, 325, 1610, 254, 8760, 8770, 8780,
    8790, 8800, 8810, 0, 247, 8820, 1615, 0, 315, 292, 8822, 295, 419, 203, 315, 360,
    1055, 544, 0, 1343, 8823, 0, 1022, 1343, 217, 1597, 1599, 1601, 1019, 1591, 8827, 8838,
    1617, 0, 8837, 1618, 1619, 58, 1620, 0, 247, 46, 633, 652, 0, 247, 8863, 0,
    8839, 8848, 0, 8861, 0, 291, 8856, 0, 351, 291, 8860, 1073, 1624, 871, 1625, 8864,
    8874, 8884, 8894, 8904, 8914, 0, 8924, 923, 924, 8925, 0, 1368, 369, 926, 8929, 8939,
    8949, 8959, 8969, 1634, 1635, 8970, 8980, 8990, 308, 1074, 1121, 1073, 8991, 226, 0, 226,
    524, 1064, 8993, 9004, 0, 737, 208, 0, 1638, 272, 160, 163, 9022, 9020, 9019, 295,
    9016, 9014, 295, 0, 1639, 320, 1640, 9025, 9035, 9044, 9053, 9063, 9073, 9083, 0, 831,
    272, 9096, 0, 9093, 315, 96, 360, 9094, 671, 672, 0, 1139, 9095, 313, 1390, 513,
    515, 9098, 9108, 9118, 9128, 9138, 9148, 9158, 9168, 9178, 9188, 9198, 9208, 9218, 9228, 9238,
    9248, 9258, 9268, 9278, 9288, 9298, 9308, 9318, 9328, 9338, 9348, 9358, 9368, 9378, 9388, 9398,
    9407, 9417, 9427, 9437, 9447, 9457, 467, 1345, 888, 1674, 1676, 1678, 1680, 1682, 1684, 1686,
    1688, 1690, 1692, 1694, 1696, 1698, 1700, 1702, 1704, 1706, 892, 890, 1708, 1710, 1712, 1714,
    1716, 1718, 1720, 1722, 1724, 1726, 1728, 1730, 1732, 1734, 1736, 1738, 1740, 1742, 981, 9458,
    9468, 9478, 9488, 9498, 9508, 9518, 0, 1053, 665, 1749, 9519, 0, 1639, 320, 1751, 0,
    1639, 272, 256, 1552, 9529, 9539, 9549, 9559, 9569, 9579, 9589, 1755, 9590, 9600, 9612, 9610,
    0, 367, 239, 9617, 9615, 0, 226, 1364, 369, 634, 0, 1756, 1757, 9620, 9630, 9640,
    9650, 9660, 9670, 9680, 9690, 9700, 9710, 9720, 9727, 9737, 9747, 9757, 9767, 9777, 9787, 0,
    0, 538, 77, 9798, 0, 641, 160, 0, 547, 160, 0, 1775, 1776, 9797, 1777, 1778,
    0, 1779, 160, 552, 0, 1775, 1776, 0, 9805, 1055, 419, 203, 237, 9799, 467, 9802,
    1345, 9806, 0, 1780, 254, 9816, 9826, 9836, 9846, 0, 9856, 1782, 486, 1783, 0, 9860,
    1784, 1783, 9864, 9874, 9884, 9894, 9904, 0, 9914, 153, 154, 9915, 9925, 1787, 636, 9926,
    0, 1789, 272, 256, 9936, 9946, 9956, 9966, 9976, 9986, 9996, 10006, 10015, 0, 10025, 770,
    771, 1464, 0, 10026, 325, 1795, 770, 1796, 10027, 0, 1798, 770, 1796, 360, 10031, 10041,
    10051, 10061, 10071, 10081, 10091, 10101, 10111, 10121, 10131, 10141, 10151, 10161, 10171, 0, 10181, 655,
    1811, 1812, 1813, 482, 10182, 1815, 1816, 0, 1817, 226, 0, 912, 1503, 1818, 10183, 10193,
    10203, 10213, 10223, 10233, 10243, 10253, 10263, 10273, 10283, 10293, 10303, 10313, 10323, 0, 10333, 10343,
    10354, 0, 156, 165, 237, 323, 10364, 10374, 10386, 10384, 367, 10389, 0, 737, 1230, 0,
    742, 1834, 10399, 10409, 10419, 10429, 10439, 0, 10449, 1839, 1840, 1841, 10450, 10460, 10470, 0,
    1845, 10480, 1063, 58, 10481, 10491, 0, 641, 10501, 10511, 10521, 0, 10531, 10541, 10551, 10561,
    0, 1585, 796, 798, 1587, 1589, 944, 1591, 1022, 711, 1024, 217, 1019, 1597, 1599, 1601,
    0, 1343, 10571, 0, 1022, 1343, 10575, 10585, 10595, 10605, 0, 157, 237, 239, 247, 1284,
    796, 798, 10615, 10625, 10635, 10645, 10655, 0, 139, 245, 10665, 10675, 226, 10685, 10695, 10705,
    10715, 10725, 10735, 10745, 10755, 10765, 10775, 10785, 10795, 10805, 10815, 10825, 10835, 10845, 10855, 0,
    10865, 1874, 789, 10872, 0, 694, 10869, 10867, 10866, 544, 295, 360, 10874, 10884, 10894, 10904,
    10914, 10924, 10934, 10944, 10954, 10964, 10974, 10984, 10994, 11004, 11014, 11024, 11034, 11044, 11054, 11064,
    11074, 11084, 0, 707, 0, 360, 11094, 1272, 11097, 360, 11100, 11103, 11106, 11116, 0, 988,
    11129, 11126, 360, 11130, 1019, 0, 196, 272, 0, 512, 272, 0, 830, 272, 0, 831,
    272, 0, 1021, 272, 833, 835, 837, 1403, 953, 11131, 11141, 11151, 11158, 0, 84, 0,
    11169, 11168, 0, 226, 524, 0, 1064, 1073, 1064, 11173, 0, 226, 11183, 1420, 1021, 1896,
    217, 1898, 0, 325, 1900, 11184, 11187, 1902, 152, 11189, 1904, 0, 325, 1900, 11192, 11195,
    1902, 152, 11197, 1604, 0, 325, 0, 1503, 55, 0, 11200, 0, 325, 1503, 46, 1905,
    11210, 11220, 11230, 11237, 11247, 11257, 11267, 11277, 11287, 11297, 11307, 11317, 11327, 11337, 11347, 11357,
];

pub static SEGMENT_TYPE_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 65, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 80, 83, 85, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 65, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 162, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 194, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295,
    4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 78, 4294967295, 200, 4294967295, 204, 4294967295,
    4294967295, 209, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 219, 4294967295, 220, 225, 4294967295, 4294967295,
    4294967295, 4294967295, 228, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 238, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 282, 4294967295, 284, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 311, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 316, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 238, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 238,
    4294967295, 4294967295, 4294967295, 238, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 238, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 331, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 338, 338, 4294967295, 338, 338, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 389, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 421, 204, 316, 361, 4294967295, 389, 4294967295, 4294967295, 4294967295,
    422, 3, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 338, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 316, 4294967295, 4294967295, 4294967295, 453, 4294967295,
    458, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 228, 4294967295, 4294967295, 466, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 220, 4294967295, 479, 4294967295, 4294967295, 485, 4294967295,
    4294967295, 4294967295, 4294967295, 361, 4294967295, 479, 4294967295, 4294967295, 485, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 316,
    4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 536, 4294967295, 204, 4294967295,
    4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 555, 4294967295, 4294967295, 558, 4294967295, 4294967295, 4294967295, 389, 4294967295,
    4294967295, 4294967295, 561, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 570, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294,
    294, 294, 294, 294, 294, 4294967295, 4294967295, 294, 4294967295, 338, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 389, 4294967295, 389, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    80, 83, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 83,
    4294967295, 4294967295, 389, 640, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 645, 4294967295, 238, 4294967295, 4294967295, 4294967295,
    647, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 653,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 282, 4294967295, 657, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    238, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 664, 4294967295, 311, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 311, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 316, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 677, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 238, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 681, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 316, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 238, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 238, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295, 710, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 238, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 712, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 80, 80, 4294967295, 723, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 736, 738, 740, 4294967295, 4294967295, 4294967295,
    4294967295, 736, 4294967295, 389, 743, 4294967295, 4294967295, 4294967295, 4294967295, 389, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 797, 800, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 316, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 389, 4294967295, 389,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295,
    645, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 653, 4294967295, 4294967295, 4294967295, 4294967295, 282, 4294967295, 4294967295, 4294967295, 4294967295, 238, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 681, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295,
    4294967295, 238, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295,
    4294967295, 4294967295, 743, 861, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 889, 891, 893, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 97, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 97, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 925, 4294967295, 927, 536, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295,
    361, 4294967295, 4294967295, 4294967295, 4294967295, 316, 204, 930, 361, 4294967295, 85, 4294967295, 4294967295, 743, 934, 4294967295,
    4294967295, 361, 4294967295, 389, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 930, 4294967295, 4294967295, 4294967295, 4294967295, 331,
    4294967295, 421, 941, 943, 316, 946, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 983, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 421, 4294967295, 316, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1014, 4294967295, 4294967295, 1020, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1023, 712, 1025, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1029, 1029, 1031, 240, 4294967295, 4294967295, 1035, 4294967295, 1038, 4294967295, 4294967295, 4294967295, 240,
    1035, 4294967295, 1040, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1044, 1046, 4294967295, 1014, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 220, 4294967295, 4294967295, 4294967295, 1038, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 238, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1054, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 238, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1056, 4294967295,
    4294967295, 4294967295, 1014, 4294967295, 1058, 4294967295, 1060, 4294967295, 4294967295, 4294967295, 361, 4294967295, 361, 4294967295, 361, 4294967295,
    389, 4294967295, 4294967295, 361, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 361, 4294967295, 361, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1067, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 316, 4294967295, 4294967295, 4294967295, 4294967295,
    316, 4294967295, 647, 941, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 453, 4294967295, 4294967295, 4294967295, 453, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    389, 4294967295, 4294967295, 4294967295, 4294967295, 389, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 710, 4294967295, 4294967295,
    710, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 389, 4294967295, 4294967295, 80, 4294967295, 80, 1056, 710, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1092, 4294967295, 1094, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 198, 78, 4294967295, 1096, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 198, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1109, 1111,
    4294967295, 4294967295, 4294967295, 316, 4294967295, 361, 4294967295, 1109, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 316, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1111, 723, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 361, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 410, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 238, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1054, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 238, 4294967295, 4294967295, 4294967295, 4294967295,
    1056, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1046, 4294967295, 4294967295, 4294967295, 220, 1187, 4294967295, 1190,
    4294967295, 234, 4294967295, 4294967295, 4294967295, 1046, 1187, 4294967295, 4294967295, 1046, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 220, 4294967295, 4294967295, 361, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 97, 4294967295, 4294967295, 4294967295, 1231, 1233, 1235, 774, 774, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1056, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1268, 1270, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 1275, 1277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295,
    1279, 4294967295, 4294967295, 4294967295, 238, 4294967295, 4294967295, 4294967295, 238, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 204,
    240, 220, 1190, 234, 1283, 4294967295, 4294967295, 1285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 238, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 238, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1316, 4294967295, 4294967295, 4294967295, 1318, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1328, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1056, 331, 4294967295, 331,
    4294967295, 1342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 891, 889, 893, 1346, 4294967295, 458, 4294967295, 4294967295, 331,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1366, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 316, 4294967295, 361, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1392, 1020, 4294967295, 4294967295, 1020, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1316, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 316, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    316, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1438, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1465, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 238, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1092, 4294967295, 1094, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295,
    198, 198, 4294967295, 1096, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1579, 4294967295, 4294967295, 240, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1581, 4294967295, 1583, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1020, 4294967295, 1023, 4294967295, 4294967295, 4294967295, 1586, 797, 800, 1588, 1590, 946, 1592, 219, 1594, 1596,
    1598, 1600, 1602, 1020, 1592, 1023, 712, 1025, 4294967295, 4294967295, 4294967295, 1342, 4294967295, 4294967295, 1342, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1616, 4294967295, 316, 294, 4294967295, 4294967295, 4294967295, 204, 316, 361,
    1056, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1023, 4294967295, 219, 1598, 1600, 1602, 1020, 1592, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 653, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 925, 4294967295, 4294967295, 4294967295, 389, 927, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 738, 209, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 316, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1346, 889, 1675, 1677, 1679, 1681, 1683, 1685, 1687,
    1689, 1691, 1693, 1695, 1697, 1699, 1701, 1703, 1705, 1707, 893, 891, 1709, 1711, 1713, 1715,
    1717, 1719, 1721, 1723, 1725, 1727, 1729, 1731, 1733, 1735, 1737, 1739, 1741, 1743, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1054, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 338, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 389, 4294967295, 4294967295, 4294967295, 1758, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 231, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1056, 4294967295, 204, 238, 4294967295, 4294967295, 4294967295,
    1346, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1465, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 238, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 738, 1231, 4294967295,
    743, 861, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1586, 797, 800, 1588, 1590, 946, 1592, 1023, 712, 1025, 219, 1020, 1598, 1600, 1602,
    4294967295, 4294967295, 4294967295, 4294967295, 1023, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 238, 240, 4294967295, 1285,
    797, 800, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 220, 4294967295, 220, 361, 234, 234, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 361, 4294967295, 1020, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1893, 4294967295, 4294967295, 85, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1897,
    219, 1899, 4294967295, 4294967295, 4294967295, 220, 4294967295, 1903, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 220, 4294967295,
    1903, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 5, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static SEGMENT_CLASS_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 64, 4294967295, 4294967295, 66, 4294967295, 4294967295, 4294967295, 4294967295, 66, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 79, 81, 84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 64, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 161, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 191, 193, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 77, 4294967295, 199, 4294967295, 203, 4294967295,
    4294967295, 208, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 217, 4294967295, 221, 223, 4294967295, 4294967295,
    4294967295, 4294967295, 227, 4294967295, 4294967295, 4294967295, 235, 4294967295, 4294967295, 237, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 281, 4294967295, 283, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 292, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 310, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 315, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 237,
    4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 355, 356, 4294967295, 358, 359, 4294967295, 4294967295, 4294967295, 4294967295, 360, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 369, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 420, 203, 315, 360, 4294967295, 369, 4294967295, 4294967295, 4294967295,
    423, 424, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 339, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295, 4294967295, 4294967295, 454, 4294967295,
    455, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 227, 4294967295, 4294967295, 465, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 476, 4294967295, 478, 4294967295, 4294967295, 484, 4294967295,
    4294967295, 4294967295, 4294967295, 360, 4294967295, 478, 4294967295, 4294967295, 484, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 315,
    4294967295, 360, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 535, 4294967295, 203, 4294967295,
    4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 360, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 553, 4294967295, 4294967295, 557, 4294967295, 4294967295, 4294967295, 369, 4294967295,
    4294967295, 4294967295, 560, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 571, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 292,
    576, 577, 578, 579, 580, 4294967295, 4294967295, 583, 4294967295, 585, 586, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 369, 4294967295, 369, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    79, 81, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 81,
    4294967295, 4294967295, 369, 639, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 644, 4294967295, 237, 4294967295, 4294967295, 4294967295,
    646, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 292, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 652,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 281, 4294967295, 656, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 663, 4294967295, 310, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 310, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 676, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 680, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 360, 4294967295, 4294967295, 4294967295, 4294967295, 709, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 711, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 720, 721, 4294967295, 722, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 735, 737, 739, 4294967295, 4294967295, 4294967295,
    4294967295, 735, 4294967295, 369, 742, 4294967295, 4294967295, 4294967295, 4294967295, 369, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 796, 798, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 315, 4294967295, 360, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 369, 4294967295, 369,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 4294967295,
    644, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 652, 4294967295, 4294967295, 4294967295, 4294967295, 281, 4294967295, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 680, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295,
    4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 360, 4294967295, 4294967295,
    4294967295, 4294967295, 742, 860, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 888, 890, 892, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 917, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 917, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 924, 4294967295, 926, 535, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295,
    360, 4294967295, 4294967295, 4294967295, 4294967295, 315, 203, 929, 360, 4294967295, 932, 4294967295, 4294967295, 742, 933, 4294967295,
    4294967295, 360, 4294967295, 369, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 929, 4294967295, 4294967295, 4294967295, 4294967295, 330,
    4294967295, 420, 940, 942, 315, 944, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 360, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 982, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 360, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 420, 4294967295, 315, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1013, 4294967295, 4294967295, 1019, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1022, 711, 1024, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1028, 1028, 1030, 239, 4294967295, 4294967295, 1033, 4294967295, 1037, 4294967295, 4294967295, 4294967295, 239,
    1033, 4294967295, 1039, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1042, 1045, 4294967295, 1013, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 221, 4294967295, 4294967295, 4294967295, 1037, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1053, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1055, 4294967295,
    4294967295, 4294967295, 1013, 4294967295, 1057, 4294967295, 1059, 4294967295, 4294967295, 4294967295, 360, 4294967295, 360, 4294967295, 360, 4294967295,
    369, 4294967295, 4294967295, 360, 4294967295, 4294967295, 360, 4294967295, 4294967295, 4294967295, 360, 4294967295, 360, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1066, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295, 4294967295, 4294967295, 4294967295,
    315, 4294967295, 646, 1071, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 454, 4294967295, 4294967295, 4294967295, 454, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    369, 4294967295, 4294967295, 4294967295, 4294967295, 369, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1077, 1078, 4294967295, 4294967295,
    1079, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 369, 4294967295, 4294967295, 79, 4294967295, 1077, 1055, 709, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1091, 4294967295, 1093, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 197, 77, 4294967295, 1095, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 197, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1107, 1110,
    4294967295, 4294967295, 4294967295, 315, 4294967295, 360, 4294967295, 1107, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295, 360, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1110, 722, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 360, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 360, 360, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 411, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1053, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295,
    1055, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 360, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1045, 4294967295, 4294967295, 4294967295, 221, 1186, 4294967295, 1189,
    4294967295, 235, 4294967295, 4294967295, 4294967295, 1045, 1186, 4294967295, 4294967295, 1045, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 476, 4294967295, 4294967295, 360, 235, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 360,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 360, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 917, 4294967295, 4294967295, 4294967295, 1230, 1232, 1234, 951, 775, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1055, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1267, 1269, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 360, 4294967295, 4294967295, 1274, 1276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 360, 4294967295,
    1278, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 203,
    239, 221, 1189, 235, 1282, 4294967295, 4294967295, 1284, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1315, 4294967295, 4294967295, 4294967295, 1317, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1329, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1055, 1340, 4294967295, 330,
    4294967295, 1341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 890, 888, 892, 1345, 4294967295, 1347, 4294967295, 4294967295, 330,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1365, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 360, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295, 360, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1391, 1019, 4294967295, 4294967295, 1019, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1315, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 360, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    315, 4294967295, 292, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1437, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1464, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 292, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1091, 4294967295, 1093, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295,
    197, 197, 4294967295, 1095, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1578, 4294967295, 4294967295, 239, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1580, 4294967295, 1582, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1019, 4294967295, 1022, 4294967295, 4294967295, 4294967295, 1585, 796, 798, 1587, 1589, 944, 1591, 217, 1593, 1595,
    1597, 1599, 1601, 1019, 1591, 1022, 711, 1024, 4294967295, 4294967295, 4294967295, 1341, 4294967295, 4294967295, 1603, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1615, 4294967295, 315, 292, 4294967295, 4294967295, 4294967295, 203, 315, 360,
    1055, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1022, 4294967295, 217, 1597, 1599, 1601, 1019, 1591, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 652, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 924, 4294967295, 4294967295, 4294967295, 369, 926, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 737, 208, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295, 360, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1345, 888, 1674, 1676, 1678, 1680, 1682, 1684, 1686,
    1688, 1690, 1692, 1694, 1696, 1698, 1700, 1702, 1704, 1706, 892, 890, 1708, 1710, 1712, 1714,
    1716, 1718, 1720, 1722, 1724, 1726, 1728, 1730, 1732, 1734, 1736, 1738, 1740, 1742, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1053, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1755, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 369, 4294967295, 4294967295, 4294967295, 1757, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 230, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1055, 4294967295, 203, 237, 4294967295, 4294967295, 4294967295,
    1345, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1464, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 360, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 737, 1230, 4294967295,
    742, 1834, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1585, 796, 798, 1587, 1589, 944, 1591, 1022, 711, 1024, 217, 1019, 1597, 1599, 1601,
    4294967295, 4294967295, 4294967295, 4294967295, 1022, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 237, 239, 4294967295, 1284,
    796, 798, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 360, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 360, 221, 4294967295, 476, 360, 235, 235, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 360, 4294967295, 1019, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1894, 4294967295, 4294967295, 84, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1896,
    217, 1898, 4294967295, 4294967295, 4294967295, 221, 4294967295, 1902, 4294967295, 235, 4294967295, 4294967295, 4294967295, 4294967295, 221, 4294967295,
    1902, 4294967295, 235, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1072, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static CASEFOLD_SPARSE: &[(u32, u8)] = &[
    (400, 2),
    (2273, 2),
    (2636, 2),
    (2637, 2),
    (2638, 2),
];

pub static TRIM_CHARS_SPARSE: &[(u32, u32, u8)] = &[
];

pub static TRIM_CHARS_DATA: &[u32] = &[
];

pub static SEGMENT_CLASS_TYPES_SPARSE: &[(u32, u32, u8)] = &[
    (38, 0, 2),
    (41, 2, 2),
    (46, 4, 2),
    (53, 6, 2),
    (55, 8, 2),
    (56, 10, 2),
    (57, 12, 3),
    (99, 15, 2),
    (135, 17, 2),
    (184, 19, 2),
    (185, 21, 2),
    (190, 23, 2),
    (194, 25, 3),
    (198, 28, 3),
    (202, 31, 2),
    (204, 33, 3),
    (206, 36, 2),
    (209, 38, 2),
    (218, 40, 2),
    (220, 42, 4),
    (221, 46, 2),
    (226, 48, 2),
    (230, 50, 5),
    (233, 55, 3),
    (234, 58, 2),
    (243, 60, 3),
    (279, 63, 3),
    (281, 66, 2),
    (290, 68, 2),
    (299, 70, 2),
    (305, 72, 3),
    (313, 75, 3),
    (319, 78, 3),
    (323, 81, 3),
    (329, 84, 3),
    (338, 87, 2),
    (355, 89, 2),
    (356, 91, 2),
    (358, 93, 2),
    (359, 95, 2),
    (364, 97, 2),
    (376, 99, 2),
    (407, 101, 2),
    (408, 103, 2),
    (409, 105, 3),
    (410, 108, 2),
    (412, 110, 2),
    (416, 112, 2),
    (417, 114, 1),
    (429, 115, 3),
    (458, 118, 3),
    (462, 121, 2),
    (464, 123, 2),
    (473, 125, 2),
    (476, 127, 2),
    (489, 129, 4),
    (491, 133, 2),
    (494, 135, 2),
    (499, 137, 2),
    (501, 139, 2),
    (504, 141, 2),
    (543, 143, 3),
    (545, 146, 2),
    (553, 148, 2),
    (572, 150, 2),
    (574, 152, 2),
    (579, 154, 2),
    (587, 156, 2),
    (599, 158, 2),
    (602, 160, 3),
    (606, 163, 2),
    (610, 165, 2),
    (633, 167, 3),
    (639, 170, 2),
    (640, 172, 2),
    (641, 174, 2),
    (642, 176, 2),
    (643, 178, 2),
    (644, 180, 2),
    (647, 182, 3),
    (649, 185, 2),
    (650, 187, 2),
    (689, 189, 2),
    (691, 191, 2),
    (704, 193, 2),
    (705, 195, 2),
    (708, 197, 2),
    (718, 199, 3),
    (734, 202, 2),
    (735, 204, 2),
    (738, 206, 2),
    (739, 208, 2),
    (746, 210, 3),
    (748, 213, 3),
    (752, 216, 2),
    (759, 218, 2),
    (766, 220, 2),
    (778, 222, 3),
    (783, 225, 3),
    (807, 228, 3),
    (809, 231, 2),
    (816, 233, 3),
    (822, 236, 2),
    (824, 238, 2),
    (834, 240, 2),
    (846, 242, 3),
    (854, 245, 2),
    (866, 247, 3),
    (872, 250, 3),
    (887, 253, 3),
    (893, 256, 3),
    (907, 259, 3),
    (930, 262, 2),
    (935, 264, 2),
    (939, 266, 3),
    (945, 269, 3),
    (951, 272, 2),
    (961, 274, 2),
    (962, 276, 2),
    (964, 278, 2),
    (986, 280, 2),
    (987, 282, 2),
    (988, 284, 2),
    (993, 286, 2),
    (995, 288, 2),
    (996, 290, 2),
    (1001, 292, 2),
    (1052, 294, 2),
    (1053, 296, 2),
    (1069, 298, 2),
    (1091, 300, 3),
    (1093, 303, 2),
    (1117, 305, 2),
    (1119, 307, 2),
    (1125, 309, 3),
    (1131, 312, 2),
    (1136, 314, 3),
    (1143, 317, 2),
    (1153, 319, 3),
    (1158, 322, 3),
    (1163, 325, 3),
    (1169, 328, 3),
    (1174, 331, 2),
    (1180, 333, 3),
    (1185, 336, 3),
    (1197, 339, 2),
    (1202, 341, 2),
    (1203, 343, 2),
    (1238, 345, 2),
    (1239, 347, 2),
    (1240, 349, 2),
    (1273, 351, 2),
    (1290, 353, 2),
    (1300, 355, 2),
    (1302, 357, 2),
    (1303, 359, 2),
    (1308, 361, 2),
    (1312, 363, 2),
    (1317, 365, 3),
    (1318, 368, 2),
    (1319, 370, 2),
    (1320, 372, 2),
    (1322, 374, 3),
    (1325, 377, 2),
    (1326, 379, 2),
    (1329, 381, 2),
    (1331, 383, 2),
    (1338, 385, 2),
    (1343, 387, 2),
    (1345, 389, 2),
    (1346, 391, 2),
    (1347, 393, 2),
    (1348, 395, 3),
    (1349, 398, 2),
    (1368, 400, 2),
    (1398, 402, 2),
    (1410, 404, 2),
    (1441, 406, 2),
    (1443, 408, 3),
    (1460, 411, 2),
    (1463, 413, 2),
    (1482, 415, 2),
    (1483, 417, 2),
    (1484, 419, 2),
    (1491, 421, 2),
    (1492, 423, 2),
    (1493, 425, 2),
    (1494, 431, 2),
    (1497, 427, 2),
    (1499, 429, 2),
    (1503, 433, 2),
    (1504, 435, 2),
    (1506, 437, 2),
    (1512, 439, 2),
    (1513, 441, 2),
    (1515, 443, 2),
    (1526, 445, 2),
    (1543, 447, 3),
    (1553, 450, 2),
    (1560, 452, 3),
    (1566, 455, 2),
    (1570, 457, 2),
    (1572, 459, 2),
    (1574, 461, 2),
    (1578, 463, 2),
    (1580, 465, 2),
    (1582, 467, 2),
    (1584, 469, 2),
    (1587, 471, 2),
    (1590, 473, 2),
    (1594, 475, 2),
    (1596, 477, 2),
    (1603, 479, 2),
    (1611, 481, 3),
    (1616, 484, 3),
    (1618, 487, 2),
    (1619, 489, 2),
    (1636, 491, 2),
    (1640, 493, 2),
    (1648, 495, 2),
    (1653, 497, 2),
    (1660, 499, 2),
    (1661, 501, 2),
    (1664, 503, 2),
    (1671, 505, 2),
    (1674, 511, 2),
    (1676, 507, 2),
    (1677, 509, 2),
    (1678, 513, 2),
    (1696, 515, 2),
    (1698, 517, 2),
    (1701, 519, 3),
    (1706, 522, 3),
    (1707, 525, 2),
    (1709, 527, 2),
    (1729, 529, 3),
    (1742, 532, 2),
    (1743, 534, 2),
    (1747, 536, 3),
    (1749, 539, 2),
    (1751, 541, 2),
    (1768, 543, 3),
    (1770, 546, 2),
    (1780, 548, 2),
    (1781, 550, 2),
    (1787, 552, 2),
    (1813, 554, 2),
    (1814, 556, 2),
    (1828, 558, 3),
    (1864, 561, 2),
    (1900, 563, 3),
    (1909, 566, 2),
    (1915, 568, 3),
    (1920, 571, 2),
    (1944, 573, 2),
    (1976, 575, 2),
    (1981, 577, 2),
    (1983, 579, 2),
    (1989, 581, 2),
    (1990, 583, 2),
    (1993, 585, 2),
    (2005, 587, 2),
    (2027, 589, 4),
    (2040, 593, 2),
    (2079, 595, 2),
    (2086, 597, 2),
    (2099, 599, 2),
    (2103, 601, 2),
    (2104, 603, 2),
    (2105, 605, 2),
    (2106, 607, 5),
    (2107, 612, 3),
    (2138, 615, 2),
    (2166, 617, 2),
    (2172, 619, 2),
    (2173, 621, 2),
    (2179, 623, 2),
    (2182, 625, 2),
    (2183, 627, 2),
    (2190, 629, 2),
    (2192, 631, 2),
    (2196, 633, 3),
    (2200, 636, 3),
    (2205, 639, 2),
    (2207, 641, 2),
    (2208, 643, 2),
    (2210, 645, 2),
    (2212, 647, 2),
    (2215, 649, 2),
    (2252, 651, 3),
    (2262, 654, 3),
    (2281, 657, 2),
    (2285, 659, 2),
    (2300, 661, 3),
    (2316, 664, 2),
    (2317, 666, 2),
    (2319, 668, 2),
    (2321, 670, 2),
    (2327, 672, 2),
    (2328, 674, 2),
    (2329, 676, 2),
    (2330, 678, 2),
    (2332, 680, 2),
    (2335, 682, 2),
    (2377, 684, 2),
    (2406, 686, 2),
    (2442, 688, 3),
    (2444, 691, 2),
    (2453, 693, 2),
    (2454, 695, 2),
    (2457, 697, 2),
    (2483, 699, 2),
    (2493, 701, 3),
    (2521, 704, 2),
    (2528, 706, 3),
    (2530, 709, 2),
    (2561, 711, 2),
    (2598, 713, 2),
    (2615, 715, 2),
    (2664, 717, 3),
    (2742, 720, 2),
    (2744, 722, 2),
    (2747, 724, 3),
    (2752, 727, 3),
    (2753, 730, 3),
    (2755, 733, 2),
    (2832, 735, 2),
    (2839, 737, 2),
    (2858, 739, 2),
    (2861, 741, 2),
    (2867, 743, 2),
    (2869, 745, 2),
    (2896, 747, 2),
    (2898, 749, 2),
    (2902, 751, 2),
    (2903, 753, 2),
    (2904, 755, 2),
    (2905, 757, 2),
    (2906, 759, 2),
    (2907, 761, 2),
    (2908, 763, 2),
    (2909, 765, 2),
    (2910, 767, 2),
    (2911, 769, 2),
    (2912, 771, 3),
    (2913, 774, 3),
    (2914, 777, 3),
    (2915, 780, 2),
    (2916, 782, 2),
    (2917, 784, 2),
    (2918, 786, 2),
    (2919, 788, 2),
    (2923, 790, 2),
    (2926, 792, 2),
    (2966, 794, 2),
    (2968, 796, 3),
    (2969, 799, 2),
    (2973, 801, 2),
    (2974, 803, 3),
    (2975, 806, 2),
    (2976, 808, 2),
    (2982, 810, 2),
    (2984, 812, 2),
    (2985, 814, 3),
    (2986, 817, 3),
    (2987, 820, 3),
    (2988, 823, 2),
    (2989, 825, 2),
    (3003, 827, 3),
    (3032, 830, 2),
    (3036, 832, 2),
    (3037, 834, 2),
    (3061, 836, 2),
    (3062, 838, 2),
    (3092, 840, 3),
    (3094, 843, 2),
    (3143, 845, 2),
    (3144, 847, 2),
    (3145, 849, 2),
    (3146, 851, 2),
    (3147, 853, 2),
    (3148, 855, 2),
    (3149, 857, 2),
    (3150, 859, 2),
    (3151, 861, 2),
    (3152, 863, 2),
    (3153, 865, 2),
    (3154, 867, 2),
    (3155, 869, 2),
    (3156, 871, 2),
    (3157, 873, 2),
    (3158, 875, 2),
    (3159, 877, 2),
    (3160, 879, 2),
    (3161, 881, 2),
    (3162, 883, 2),
    (3163, 885, 2),
    (3164, 887, 2),
    (3165, 889, 2),
    (3166, 891, 2),
    (3167, 893, 2),
    (3168, 895, 2),
    (3169, 897, 2),
    (3170, 899, 2),
    (3171, 901, 2),
    (3172, 903, 2),
    (3173, 905, 2),
    (3174, 907, 2),
    (3175, 909, 2),
    (3176, 911, 2),
    (3177, 913, 2),
    (3178, 915, 2),
    (3179, 917, 2),
    (3180, 919, 2),
    (3181, 921, 2),
    (3191, 923, 2),
    (3211, 925, 2),
    (3218, 927, 2),
    (3224, 929, 2),
    (3228, 931, 2),
    (3239, 933, 3),
    (3250, 936, 2),
    (3273, 938, 2),
    (3275, 940, 2),
    (3276, 942, 3),
    (3280, 945, 2),
    (3329, 947, 2),
    (3341, 949, 2),
    (3396, 951, 3),
    (3405, 954, 2),
    (3406, 956, 2),
    (3408, 958, 2),
    (3409, 960, 2),
    (3441, 962, 2),
    (3442, 964, 2),
    (3443, 966, 2),
    (3444, 968, 2),
    (3445, 970, 2),
    (3446, 972, 2),
    (3447, 974, 2),
    (3448, 976, 2),
    (3449, 978, 2),
    (3450, 980, 2),
    (3451, 982, 2),
    (3452, 984, 2),
    (3453, 986, 3),
    (3454, 989, 3),
    (3455, 992, 3),
    (3460, 995, 2),
    (3468, 997, 3),
    (3469, 1000, 2),
    (3471, 1002, 2),
    (3472, 1004, 2),
    (3473, 1006, 2),
    (3481, 1008, 3),
    (3515, 1011, 2),
    (3541, 1013, 2),
    (3545, 1015, 2),
    (3554, 1017, 2),
    (3556, 1019, 2),
    (3579, 1021, 3),
    (3582, 1024, 3),
    (3599, 1027, 2),
    (3600, 1029, 2),
    (3601, 1031, 2),
    (3607, 1033, 2),
    (3616, 1035, 2),
    (3634, 1037, 3),
];

pub static SEGMENT_CLASS_TYPES_DATA: &[u32] = &[
    65, 3, 76, 3, 76, 3, 3, 78, 3, 80, 3, 83, 3, 78, 85, 65,
    3, 162, 3, 3, 192, 3, 194, 3, 78, 3, 78, 198, 3, 78, 200, 3,
    78, 3, 78, 200, 3, 204, 209, 3, 3, 219, 3, 220, 222, 4, 225, 3,
    3, 228, 3, 234, 220, 222, 4, 3, 78, 238, 240, 3, 3, 246, 78, 3,
    78, 282, 284, 3, 3, 294, 3, 311, 3, 316, 78, 3, 78, 238, 3, 78,
    238, 3, 78, 238, 3, 78, 238, 3, 331, 3, 338, 3, 338, 3, 338, 3,
    338, 3, 361, 3, 389, 3, 421, 3, 204, 3, 316, 78, 3, 361, 3, 389,
    3, 422, 3, 3, 338, 4, 3, 316, 78, 3, 453, 3, 458, 3, 228, 3,
    466, 3, 220, 222, 4, 3, 479, 3, 485, 3, 361, 3, 479, 3, 485, 3,
    316, 78, 3, 361, 3, 4, 3, 536, 3, 204, 3, 78, 3, 361, 3, 555,
    3, 558, 78, 3, 389, 3, 561, 3, 570, 4, 3, 294, 3, 294, 3, 294,
    3, 294, 3, 294, 3, 294, 3, 294, 4, 3, 338, 3, 294, 3, 389, 3,
    389, 3, 80, 3, 83, 3, 78, 3, 246, 78, 3, 80, 3, 83, 3, 389,
    3, 640, 3, 645, 78, 3, 78, 238, 3, 647, 3, 78, 3, 294, 3, 78,
    200, 3, 78, 653, 3, 78, 282, 3, 657, 3, 78, 238, 3, 664, 3, 311,
    3, 311, 3, 316, 78, 3, 677, 3, 78, 238, 3, 78, 681, 3, 316, 78,
    3, 78, 238, 3, 78, 238, 3, 361, 3, 710, 3, 78, 200, 3, 78, 238,
    3, 712, 3, 80, 3, 80, 3, 723, 3, 736, 738, 3, 3, 740, 3, 736,
    3, 389, 3, 743, 3, 389, 3, 797, 3, 800, 3, 78, 3, 316, 78, 3,
    361, 3, 389, 3, 389, 3, 246, 78, 3, 80, 3, 645, 78, 3, 78, 3,
    78, 653, 3, 78, 282, 3, 78, 238, 3, 78, 681, 3, 78, 3, 78, 200,
    3, 78, 238, 3, 361, 3, 743, 3, 861, 3, 889, 3, 891, 3, 893, 3,
    97, 3, 97, 3, 925, 3, 927, 3, 536, 3, 204, 3, 361, 3, 316, 78,
    3, 204, 3, 930, 3, 361, 3, 78, 85, 3, 743, 3, 934, 3, 361, 3,
    389, 3, 930, 3, 331, 3, 421, 3, 941, 943, 3, 3, 316, 78, 3, 946,
    3, 361, 3, 983, 3, 361, 3, 421, 3, 316, 78, 3, 1014, 3, 1020, 3,
    1023, 3, 712, 3, 1025, 3, 1029, 3, 1029, 3, 1031, 3, 1035, 3, 1038, 240,
    3, 240, 3, 3, 1035, 3, 1040, 3, 1044, 3, 1046, 3, 1014, 3, 1038, 3,
    78, 238, 3, 1054, 3, 78, 238, 3, 1056, 3, 1014, 3, 1058, 3, 1060, 3,
    361, 3, 361, 3, 361, 3, 389, 3, 361, 3, 361, 3, 361, 3, 361, 1067,
    3, 3, 316, 78, 3, 316, 78, 3, 647, 3, 941, 3, 453, 3, 453, 3,
    389, 3, 389, 3, 80, 3, 710, 3, 710, 3, 389, 3, 80, 3, 1056, 3,
    80, 3, 710, 1092, 3, 1094, 3, 3, 78, 200, 3, 78, 198, 3, 78, 1096,
    3, 3, 78, 198, 3, 1109, 3, 1111, 3, 316, 78, 3, 361, 3, 1109, 3,
    316, 78, 3, 361, 3, 1111, 3, 723, 3, 361, 3, 361, 3, 361, 3, 410,
    4, 3, 204, 3, 78, 238, 3, 1054, 3, 78, 238, 3, 1056, 3, 361, 3,
    1046, 3, 1187, 3, 1190, 3, 1046, 3, 1187, 3, 1046, 3, 361, 3, 1, 4,
    5, 3, 204, 3, 361, 3, 361, 3, 97, 1231, 3, 3, 1233, 3, 1235, 3,
    1, 774, 4, 5, 3, 774, 4, 3, 78, 3, 1056, 3, 1268, 3, 1270, 3,
    361, 3, 1275, 3, 1277, 3, 361, 3, 1279, 3, 78, 238, 3, 78, 238, 240,
    3, 3, 204, 240, 3, 3, 1190, 3, 1283, 3, 1285, 3, 78, 238, 3, 78,
    238, 3, 1316, 3, 1318, 3, 1328, 4, 3, 1056, 3, 331, 3, 331, 3, 1342,
    3, 891, 3, 889, 3, 893, 3, 1346, 3, 458, 3, 331, 3, 1366, 3, 361,
    3, 316, 78, 3, 361, 3, 1392, 3, 1020, 3, 1020, 3, 1316, 3, 316, 78,
    3, 361, 3, 316, 78, 3, 294, 3, 1438, 3, 1465, 3, 294, 3, 78, 238,
    1092, 3, 1094, 3, 3, 78, 200, 3, 78, 198, 3, 78, 198, 1096, 3, 3,
    204, 240, 3, 3, 1579, 240, 3, 3, 1581, 3, 1583, 3, 1020, 3, 1023, 3,
    1586, 3, 797, 3, 800, 3, 1588, 3, 1590, 3, 946, 3, 1592, 3, 219, 3,
    1594, 3, 1596, 3, 1598, 219, 3, 1600, 219, 3, 219, 1602, 3, 1020, 3, 1592,
    3, 1023, 3, 712, 3, 1025, 3, 1342, 3, 1342, 3, 1616, 3, 316, 78, 3,
    294, 3, 204, 3, 316, 78, 3, 361, 3, 1056, 3, 1023, 3, 219, 3, 1598,
    219, 3, 1600, 219, 3, 219, 1602, 3, 1020, 3, 1592, 3, 78, 653, 3, 925,
    3, 389, 3, 927, 738, 3, 209, 3, 3, 316, 78, 3, 361, 3, 1346, 3,
    889, 3, 1675, 3, 1677, 3, 1679, 3, 1681, 3, 1683, 1685, 3, 3, 1687, 3,
    1689, 3, 1691, 1693, 3, 3, 1695, 3, 1697, 3, 1699, 3, 1701, 3, 1703, 3,
    1705, 3, 1707, 3, 893, 3, 891, 3, 1709, 3, 1711, 3, 1713, 3, 1715, 3,
    1717, 3, 1719, 3, 1721, 3, 1723, 1725, 3, 3, 1727, 3, 1729, 3, 1731, 3,
    1733, 1735, 3, 3, 1737, 3, 1739, 3, 1741, 1743, 3, 3, 1054, 3, 338, 240,
    3, 3, 389, 3, 1758, 3, 4, 231, 3, 78, 3, 1056, 3, 204, 3, 78,
    238, 3, 1346, 3, 1465, 3, 361, 3, 78, 238, 738, 3, 1231, 3, 3, 743,
    3, 861, 3, 1586, 3, 797, 3, 800, 3, 1588, 3, 1590, 3, 946, 3, 1592,
    3, 1023, 3, 712, 3, 1025, 3, 219, 3, 1020, 3, 1598, 219, 3, 1600, 219,
    3, 219, 1602, 3, 1023, 3, 78, 238, 240, 3, 3, 1285, 3, 797, 3, 800,
    3, 246, 78, 3, 361, 3, 361, 3, 361, 3, 361, 3, 1020, 3, 4, 1893,
    3, 78, 85, 3, 1897, 3, 219, 3, 1899, 3, 1903, 3, 1903, 3, 4, 5,
];

pub static REGEX_PATTERNS: &[&str] = &[
    r#"[A-Z_][A-Z0-9_]*"#, // [0]
    r#"^(NOT)$"#, // [1]
    r#"[A-Z0-9_]*[A-Z][A-Z0-9_]*"#, // [2]
    r#"^(ALL|ALTER|AND|ARRAY|AS|AUTHORIZATION|BETWEEN|BIGINT|BINARY|BOOLEAN|BOTH|BY|CACHE|CASE|CAST|CHAR|COLUMN|COMMIT|CONF|CONSTRAINT|CREATE|CROSS|CUBE|CURRENT|CURRENT_DATE|CURRENT_TIMESTAMP|CURSOR|DATABASE|DATE|DAYOFWEEK|DEC|DECIMAL|DELETE|DESCRIBE|DISTINCT|DOUBLE|DROP|ELSE|END|EXCHANGE|EXISTS|EXTENDED|EXTERNAL|EXTRACT|FALSE|FETCH|FLOAT|FLOOR|FOLLOWING|FOR|FOREIGN|FROM|FULL|FUNCTION|GRANT|GROUP|GROUPING|HAVING|IF|IGNORE|IMPORT|IN|INNER|INSERT|INT|INTEGER|INTERSECT|INTERVAL|INTO|IS|JOIN|LATERAL|LEFT|LESS|LIKE|LOCAL|MACRO|MAP|MORE|NATURAL|NONE|NOT|NULL|NUMERIC|OF|ON|ONLY|OR|ORDER|OUT|OUTER|OVER|PARTIALSCAN|PARTITION|PERCENT|PRECEDING|PRECISION|PRESERVE|PRIMARY|PROCEDURE|RANGE|READS|REDUCE|REFERENCES|REGEXP|RESPECT|REVOKE|RIGHT|RLIKE|ROLLBACK|ROLLUP|ROW|ROWS|SELECT|SET|SMALLINT|START|SYNC|TABLE|TABLESAMPLE|THEN|TIME|TIMESTAMP|TO|TRANSFORM|TRIGGER|TRUE|TRUNCATE|UNBOUNDED|UNION|UNIQUEJOIN|UPDATE|USER|USING|UTC_TMESTAMP|VALUES|VARCHAR|VIEWS|WHEN|WHERE|WINDOW|WITH)$"#, // [3]
    r#"\"?[A-Z][A-Z0-9_]*\"?"#, // [4]
    r#"\d+[bBkKmMgG]"#, // [5]
];

pub static HINT_STRING_INDICES: &[u32] = &[
    0, 6, 7, 8, 10, 11, 12, 13, 14, 15,
    16, 17, 18, 19, 20, 21, 22, 23, 24, 25,
    26, 27, 28, 29, 30, 31, 32, 33, 34, 35,
    36, 37, 25, 40, 37, 13, 14, 20, 21, 28,
    12, 26, 33, 35, 17, 52, 53, 53, 52, 57,
    11, 60, 62, 19, 15, 16, 18, 22, 23, 24,
    27, 28, 29, 30, 31, 32, 34, 36, 15, 16,
    67, 22, 23, 68, 69, 60, 70, 71, 72, 73,
    74, 75, 13, 28, 82, 87, 89, 91, 92, 95,
    97, 11, 98, 99, 100, 101, 102, 103, 104, 105,
    106, 40, 107, 108, 109, 110, 111, 112, 113, 114,
    115, 116, 117, 118, 119, 120, 121, 100, 13, 20,
    26, 123, 37, 123, 98, 22, 128, 102, 34, 105,
    133, 134, 118, 134, 118, 11, 98, 99, 100, 101,
    102, 103, 104, 106, 40, 107, 108, 109, 110, 111,
    112, 113, 114, 115, 116, 117, 118, 119, 120, 121,
    15, 16, 18, 22, 23, 137, 24, 27, 28, 29,
    30, 31, 32, 34, 36, 137, 119, 12, 28, 103,
    142, 99, 101, 104, 106, 107, 108, 109, 111, 110,
    112, 113, 114, 115, 116, 117, 120, 121, 164, 32,
    36, 30, 18, 24, 27, 29, 31, 23, 15, 16,
    177, 73, 75, 71, 67, 68, 69, 70, 72, 74,
    189, 190, 189, 190, 195, 201, 206, 207, 210, 211,
    212, 213, 214, 215, 216, 218, 224, 232, 236, 241,
    242, 243, 248, 249, 250, 251, 252, 249, 252, 26,
    123, 257, 258, 259, 250, 251, 262, 263, 264, 265,
    266, 267, 268, 269, 270, 218, 264, 273, 266, 267,
    265, 267, 262, 268, 262, 268, 263, 269, 263, 269,
    270, 218, 285, 288, 290, 293, 296, 297, 298, 299,
    300, 301, 302, 303, 304, 305, 207, 306, 257, 258,
    307, 97, 259, 213, 106, 82, 215, 312, 215, 312,
    312, 318, 224, 89, 321, 324, 326, 224, 248, 329,
    82, 112, 332, 333, 335, 336, 337, 340, 341, 342,
    343, 333, 344, 296, 297, 345, 346, 347, 348, 349,
    296, 297, 345, 344, 343, 349, 348, 346, 347, 363,
    365, 298, 298, 370, 371, 372, 373, 299, 374, 375,
    376, 377, 378, 379, 380, 381, 382, 383, 384, 385,
    304, 386, 387, 388, 392, 393, 394, 395, 396, 397,
    398, 399, 400, 401, 402, 403, 405, 406, 407, 408,
    257, 412, 413, 414, 415, 416, 82, 301, 257, 258,
    97, 259, 425, 426, 427, 428, 370, 371, 429, 343,
    333, 344, 296, 297, 345, 346, 293, 347, 337, 430,
    431, 348, 349, 432, 337, 431, 429, 346, 293, 347,
    430, 432, 437, 438, 439, 440, 445, 446, 431, 372,
    300, 305, 305, 300, 451, 82, 112, 456, 457, 459,
    460, 461, 462, 463, 464, 82, 112, 456, 457, 326,
    468, 469, 470, 471, 472, 473, 477, 481, 483, 487,
    488, 490, 491, 492, 493, 494, 373, 495, 496, 497,
    498, 499, 500, 501, 502, 503, 504, 505, 506, 507,
    508, 509, 510, 511, 514, 516, 517, 518, 519, 520,
    521, 522, 523, 525, 527, 528, 529, 530, 531, 537,
    539, 302, 539, 302, 543, 546, 550, 554, 563, 565,
    567, 568, 572, 573, 574, 575, 429, 346, 432, 430,
    584, 587, 588, 589, 590, 591, 593, 595, 596, 597,
    598, 601, 602, 603, 604, 605, 606, 607, 608, 609,
    610, 611, 612, 613, 614, 615, 616, 617, 618, 619,
    622, 608, 18, 623, 624, 24, 27, 625, 608, 623,
    624, 625, 608, 623, 625, 623, 625, 624, 630, 113,
    114, 637, 643, 649, 650, 262, 263, 264, 265, 266,
    267, 268, 269, 270, 218, 654, 654, 82, 224, 518,
    554, 658, 250, 659, 660, 661, 662, 329, 82, 224,
    518, 554, 250, 659, 660, 661, 662, 329, 537, 17,
    546, 550, 662, 659, 668, 392, 670, 670, 674, 661,
    660, 662, 658, 215, 425, 682, 682, 686, 101, 104,
    117, 688, 630, 91, 689, 539, 690, 477, 690, 693,
    660, 696, 688, 689, 539, 688, 688, 689, 689, 700,
    702, 704, 660, 705, 705, 5, 713, 714, 715, 716,
    717, 717, 719, 719, 724, 725, 414, 726, 727, 728,
    415, 416, 729, 730, 731, 732, 733, 370, 371, 372,
    373, 299, 374, 375, 376, 377, 378, 379, 380, 382,
    383, 384, 304, 386, 388, 381, 385, 387, 299, 745,
    748, 749, 750, 751, 752, 753, 754, 755, 756, 757,
    758, 759, 760, 761, 762, 763, 764, 765, 766, 767,
    299, 301, 303, 304, 303, 304, 301, 259, 776, 778,
    779, 748, 780, 781, 782, 783, 784, 785, 749, 750,
    786, 787, 374, 375, 788, 457, 790, 791, 792, 793,
    794, 795, 799, 801, 803, 804, 805, 806, 807, 808,
    809, 810, 811, 812, 814, 815, 816, 817, 818, 819,
    820, 821, 822, 823, 824, 825, 826, 827, 828, 829,
    832, 834, 836, 838, 840, 841, 842, 843, 376, 845,
    471, 846, 846, 851, 852, 853, 854, 855, 856, 857,
    858, 859, 862, 863, 864, 865, 866, 867, 869, 870,
    872, 873, 874, 875, 876, 877, 878, 879, 880, 881,
    882, 883, 884, 885, 886, 82, 101, 104, 112, 117,
    456, 457, 326, 894, 895, 899, 900, 658, 539, 901,
    902, 62, 539, 428, 539, 296, 297, 539, 914, 915,
    296, 297, 915, 914, 429, 343, 333, 344, 296, 297,
    345, 346, 293, 347, 430, 348, 349, 432, 296, 297,
    915, 299, 300, 301, 302, 303, 304, 305, 257, 258,
    97, 259, 945, 947, 948, 949, 952, 312, 954, 954,
    660, 517, 517, 957, 326, 957, 960, 961, 408, 962,
    963, 964, 965, 966, 967, 968, 967, 968, 966, 964,
    965, 963, 962, 978, 82, 803, 243, 100, 101, 812,
    285, 886, 91, 189, 104, 979, 980, 190, 112, 248,
    116, 117, 119, 456, 457, 326, 82, 243, 100, 101,
    812, 285, 886, 91, 189, 104, 979, 980, 190, 112,
    248, 116, 117, 119, 456, 457, 326, 984, 985, 986,
    989, 990, 991, 377, 992, 993, 994, 995, 997, 998,
    999, 1000, 1001, 1004, 994, 1005, 1005, 516, 1009, 1010,
    1011, 1012, 511, 829, 876, 952, 195, 834, 1015, 514,
    1016, 1017, 218, 836, 832, 1018, 799, 838, 326, 876,
    1015, 1016, 1018, 1017, 1034, 1043, 674, 1048, 1048, 822,
    1050, 32, 822, 1050, 1050, 456, 457, 595, 1061, 1069,
    1070, 258, 5, 258, 340, 298, 370, 371, 372, 373,
    299, 374, 375, 376, 377, 378, 379, 380, 381, 382,
    383, 384, 385, 304, 386, 387, 388, 745, 456, 457,
    1081, 1082, 1083, 1084, 1085, 1086, 1087, 1088, 1089, 614,
    326, 1098, 1101, 1105, 1106, 1108, 511, 829, 834, 514,
    218, 836, 832, 838, 511, 829, 218, 832, 1115, 1116,
    511, 829, 514, 218, 836, 832, 838, 1117, 1118, 1119,
    1120, 307, 1123, 1124, 1125, 1126, 1127, 751, 1128, 1129,
    1130, 752, 1131, 1132, 1133, 1134, 1135, 1069, 1070, 1140,
    899, 1141, 1142, 1143, 1144, 1145, 1146, 1147, 1148, 1149,
    1150, 1151, 1152, 1153, 1154, 596, 1155, 1156, 1157, 1158,
    1159, 1160, 1161, 1162, 1163, 1164, 1165, 1166, 378, 379,
    1167, 1015, 1168, 82, 257, 258, 97, 259, 1169, 900,
    1171, 1174, 1175, 1176, 1177, 1178, 1179, 1182, 1183, 164,
    1188, 1188, 1182, 1183, 595, 597, 598, 597, 598, 1202,
    380, 1205, 1206, 1207, 1208, 1209, 1210, 1211, 1212, 1213,
    1215, 1216, 1217, 1218, 1219, 1220, 899, 900, 658, 901,
    902, 902, 901, 87, 1017, 1227, 1228, 1229, 306, 1236,
    1237, 1238, 1239, 1241, 1242, 1243, 1244, 1245, 1246, 1247,
    1248, 1249, 1250, 1251, 1252, 1253, 1255, 1256, 1257, 1258,
    1259, 1260, 1261, 265, 1262, 1263, 1264, 1265, 979, 1273,
    101, 117, 1286, 1287, 1288, 1289, 1290, 753, 1291, 1292,
    1016, 754, 1293, 1294, 755, 266, 1295, 1296, 1297, 1298,
    1299, 756, 1300, 757, 1301, 1302, 980, 1305, 213, 285,
    1306, 1306, 1309, 1310, 1311, 1312, 1314, 1319, 1321, 1322,
    1324, 1325, 1327, 267, 1330, 1331, 1332, 1333, 1334, 1335,
    1336, 1337, 1338, 1339, 101, 104, 979, 117, 1348, 1349,
    270, 1350, 1351, 1352, 1353, 1354, 429, 1356, 1357, 1358,
    1359, 1360, 1361, 1362, 1363, 382, 1371, 1372, 1373, 1374,
    1375, 1376, 1377, 1378, 1379, 1380, 1381, 1382, 1383, 1384,
    1385, 1386, 1387, 1389, 312, 1211, 952, 834, 514, 836,
    516, 517, 840, 838, 1394, 1395, 1396, 1397, 1398, 1399,
    1400, 1402, 1404, 1405, 1406, 1407, 1408, 1410, 1411, 1412,
    1413, 1414, 1415, 1416, 1417, 1418, 1419, 1421, 1422, 1423,
    345, 257, 258, 259, 1426, 1427, 1428, 1429, 1430, 1432,
    1433, 1434, 1435, 1436, 1440, 1441, 1442, 1443, 1444, 1445,
    1446, 1447, 1448, 386, 383, 384, 374, 375, 382, 388,
    1469, 1470, 1471, 1472, 1473, 1474, 1475, 11, 340, 1476,
    1476, 760, 1478, 1480, 1481, 1482, 1484, 1485, 1486, 1487,
    1488, 1489, 1490, 1491, 1492, 1493, 1494, 1495, 1496, 471,
    267, 846, 248, 595, 1418, 1500, 1500, 1505, 1506, 1507,
    1508, 1509, 1510, 1511, 1512, 1513, 1514, 1515, 1516, 1517,
    1518, 1519, 1520, 1521, 1522, 1523, 1524, 1525, 1526, 1527,
    1528, 1529, 1530, 1531, 1532, 1534, 1535, 1536, 1537, 1538,
    1539, 801, 1540, 1544, 1540, 1553, 1554, 1555, 1556, 1557,
    1558, 1559, 1427, 517, 1565, 1566, 1567, 1568, 1569, 1570,
    1571, 1572, 1573, 1574, 761, 1575, 762, 1576, 1577, 11,
    827, 511, 829, 876, 952, 91, 1015, 514, 1016, 218,
    945, 832, 1018, 799, 82, 326, 1605, 1607, 1611, 1612,
    1613, 1614, 1018, 876, 1015, 1622, 525, 363, 1626, 1627,
    1628, 1629, 1630, 1631, 1632, 1633, 1636, 1637, 82, 296,
    297, 298, 299, 300, 301, 302, 303, 304, 305, 207,
    306, 257, 258, 307, 97, 259, 1643, 1644, 1645, 1646,
    1647, 1648, 1649, 1650, 1651, 1652, 1653, 1654, 1655, 1656,
    1657, 1658, 1659, 1660, 1661, 1662, 1663, 1664, 1665, 1666,
    1667, 1668, 1669, 1671, 1672, 1673, 1744, 1745, 1746, 1747,
    1748, 1750, 1752, 1753, 1754, 1759, 1760, 1761, 1762, 1763,
    1764, 1765, 1766, 1767, 1768, 1769, 1770, 1771, 1772, 1773,
    1774, 17, 546, 550, 1350, 1518, 1781, 1785, 113, 114,
    1786, 1786, 1788, 1790, 1791, 1792, 303, 304, 326, 1794,
    1794, 1797, 1799, 1800, 1801, 1802, 1803, 1804, 1805, 1806,
    1807, 1808, 1809, 1810, 426, 572, 483, 1536, 654, 1810,
    1814, 1814, 1819, 1820, 1821, 1822, 1823, 1824, 1825, 1826,
    1827, 1828, 1829, 1830, 1831, 1832, 1833, 1835, 1836, 1837,
    1838, 1842, 1843, 1844, 1846, 1847, 1848, 1849, 1850, 1851,
    1852, 1853, 1854, 1855, 1856, 1857, 1858, 1859, 1860, 1861,
    1862, 1863, 1864, 1865, 1866, 1867, 1868, 1869, 1870, 1871,
    1872, 456, 1873, 82, 660, 1875, 1876, 1877, 1878, 1879,
    1880, 1881, 1882, 1883, 1884, 1885, 1886, 1887, 1888, 763,
    764, 765, 1891, 511, 829, 952, 195, 834, 514, 218,
    945, 836, 832, 838, 1892, 1895, 218, 324, 1906, 1907,
    1908, 1909, 1910, 1911, 766, 1912, 767, 1913,
];

pub static SIMPLE_HINTS: &[SimpleHintData] = &[
    SimpleHintData::empty(), // [0] reserved for no hint
    SimpleHintData { raw_values_start: 0, raw_values_count: 1, token_types_start: 1, token_types_count: 0 }, // [1]
    SimpleHintData { raw_values_start: 1, raw_values_count: 1, token_types_start: 2, token_types_count: 0 }, // [2]
    SimpleHintData { raw_values_start: 2, raw_values_count: 1, token_types_start: 3, token_types_count: 0 }, // [3]
    SimpleHintData { raw_values_start: 3, raw_values_count: 1, token_types_start: 4, token_types_count: 0 }, // [4]
    SimpleHintData { raw_values_start: 4, raw_values_count: 1, token_types_start: 5, token_types_count: 0 }, // [5]
    SimpleHintData { raw_values_start: 5, raw_values_count: 27, token_types_start: 32, token_types_count: 0 }, // [6]
    SimpleHintData { raw_values_start: 32, raw_values_count: 1, token_types_start: 33, token_types_count: 0 }, // [7]
    SimpleHintData { raw_values_start: 33, raw_values_count: 1, token_types_start: 34, token_types_count: 0 }, // [8]
    SimpleHintData { raw_values_start: 34, raw_values_count: 1, token_types_start: 35, token_types_count: 0 }, // [9]
    SimpleHintData { raw_values_start: 35, raw_values_count: 1, token_types_start: 36, token_types_count: 0 }, // [10]
    SimpleHintData { raw_values_start: 36, raw_values_count: 1, token_types_start: 37, token_types_count: 0 }, // [11]
    SimpleHintData { raw_values_start: 37, raw_values_count: 1, token_types_start: 38, token_types_count: 0 }, // [12]
    SimpleHintData { raw_values_start: 38, raw_values_count: 1, token_types_start: 39, token_types_count: 0 }, // [13]
    SimpleHintData { raw_values_start: 39, raw_values_count: 1, token_types_start: 40, token_types_count: 0 }, // [14]
    SimpleHintData { raw_values_start: 40, raw_values_count: 1, token_types_start: 41, token_types_count: 0 }, // [15]
    SimpleHintData { raw_values_start: 41, raw_values_count: 1, token_types_start: 42, token_types_count: 0 }, // [16]
    SimpleHintData { raw_values_start: 42, raw_values_count: 1, token_types_start: 43, token_types_count: 0 }, // [17]
    SimpleHintData { raw_values_start: 43, raw_values_count: 1, token_types_start: 44, token_types_count: 0 }, // [18]
    SimpleHintData { raw_values_start: 44, raw_values_count: 1, token_types_start: 45, token_types_count: 0 }, // [19]
    SimpleHintData { raw_values_start: 45, raw_values_count: 2, token_types_start: 47, token_types_count: 0 }, // [20]
    SimpleHintData { raw_values_start: 47, raw_values_count: 1, token_types_start: 48, token_types_count: 0 }, // [21]
    SimpleHintData { raw_values_start: 48, raw_values_count: 1, token_types_start: 49, token_types_count: 0 }, // [22]
    SimpleHintData { raw_values_start: 49, raw_values_count: 1, token_types_start: 50, token_types_count: 0 }, // [23]
    SimpleHintData { raw_values_start: 50, raw_values_count: 1, token_types_start: 51, token_types_count: 0 }, // [24]
    SimpleHintData { raw_values_start: 51, raw_values_count: 1, token_types_start: 52, token_types_count: 0 }, // [25]
    SimpleHintData { raw_values_start: 52, raw_values_count: 1, token_types_start: 53, token_types_count: 0 }, // [26]
    SimpleHintData { raw_values_start: 53, raw_values_count: 1, token_types_start: 54, token_types_count: 0 }, // [27]
    SimpleHintData { raw_values_start: 54, raw_values_count: 14, token_types_start: 68, token_types_count: 0 }, // [28]
    SimpleHintData { raw_values_start: 68, raw_values_count: 14, token_types_start: 82, token_types_count: 0 }, // [29]
    SimpleHintData { raw_values_start: 82, raw_values_count: 2, token_types_start: 84, token_types_count: 0 }, // [30]
    SimpleHintData { raw_values_start: 84, raw_values_count: 1, token_types_start: 85, token_types_count: 0 }, // [31]
    SimpleHintData { raw_values_start: 85, raw_values_count: 1, token_types_start: 86, token_types_count: 0 }, // [32]
    SimpleHintData { raw_values_start: 86, raw_values_count: 1, token_types_start: 87, token_types_count: 0 }, // [33]
    SimpleHintData { raw_values_start: 87, raw_values_count: 1, token_types_start: 88, token_types_count: 0 }, // [34]
    SimpleHintData { raw_values_start: 88, raw_values_count: 1, token_types_start: 89, token_types_count: 0 }, // [35]
    SimpleHintData { raw_values_start: 89, raw_values_count: 1, token_types_start: 90, token_types_count: 0 }, // [36]
    SimpleHintData { raw_values_start: 90, raw_values_count: 0, token_types_start: 90, token_types_count: 1 }, // [37]
    SimpleHintData { raw_values_start: 91, raw_values_count: 26, token_types_start: 117, token_types_count: 0 }, // [38]
    SimpleHintData { raw_values_start: 117, raw_values_count: 1, token_types_start: 118, token_types_count: 0 }, // [39]
    SimpleHintData { raw_values_start: 118, raw_values_count: 5, token_types_start: 123, token_types_count: 0 }, // [40]
    SimpleHintData { raw_values_start: 123, raw_values_count: 1, token_types_start: 124, token_types_count: 0 }, // [41]
    SimpleHintData { raw_values_start: 124, raw_values_count: 1, token_types_start: 125, token_types_count: 0 }, // [42]
    SimpleHintData { raw_values_start: 125, raw_values_count: 1, token_types_start: 126, token_types_count: 0 }, // [43]
    SimpleHintData { raw_values_start: 126, raw_values_count: 1, token_types_start: 127, token_types_count: 0 }, // [44]
    SimpleHintData { raw_values_start: 127, raw_values_count: 1, token_types_start: 128, token_types_count: 0 }, // [45]
    SimpleHintData { raw_values_start: 128, raw_values_count: 1, token_types_start: 129, token_types_count: 0 }, // [46]
    SimpleHintData { raw_values_start: 129, raw_values_count: 1, token_types_start: 130, token_types_count: 0 }, // [47]
    SimpleHintData { raw_values_start: 130, raw_values_count: 1, token_types_start: 131, token_types_count: 0 }, // [48]
    SimpleHintData { raw_values_start: 131, raw_values_count: 2, token_types_start: 133, token_types_count: 0 }, // [49]
    SimpleHintData { raw_values_start: 133, raw_values_count: 1, token_types_start: 134, token_types_count: 0 }, // [50]
    SimpleHintData { raw_values_start: 134, raw_values_count: 1, token_types_start: 135, token_types_count: 0 }, // [51]
    SimpleHintData { raw_values_start: 135, raw_values_count: 25, token_types_start: 160, token_types_count: 0 }, // [52]
    SimpleHintData { raw_values_start: 160, raw_values_count: 15, token_types_start: 175, token_types_count: 0 }, // [53]
    SimpleHintData { raw_values_start: 175, raw_values_count: 1, token_types_start: 176, token_types_count: 0 }, // [54]
    SimpleHintData { raw_values_start: 176, raw_values_count: 1, token_types_start: 177, token_types_count: 0 }, // [55]
    SimpleHintData { raw_values_start: 177, raw_values_count: 2, token_types_start: 179, token_types_count: 0 }, // [56]
    SimpleHintData { raw_values_start: 179, raw_values_count: 1, token_types_start: 180, token_types_count: 0 }, // [57]
    SimpleHintData { raw_values_start: 180, raw_values_count: 1, token_types_start: 181, token_types_count: 0 }, // [58]
    SimpleHintData { raw_values_start: 181, raw_values_count: 1, token_types_start: 182, token_types_count: 0 }, // [59]
    SimpleHintData { raw_values_start: 182, raw_values_count: 1, token_types_start: 183, token_types_count: 0 }, // [60]
    SimpleHintData { raw_values_start: 183, raw_values_count: 1, token_types_start: 184, token_types_count: 0 }, // [61]
    SimpleHintData { raw_values_start: 184, raw_values_count: 1, token_types_start: 185, token_types_count: 0 }, // [62]
    SimpleHintData { raw_values_start: 185, raw_values_count: 1, token_types_start: 186, token_types_count: 0 }, // [63]
    SimpleHintData { raw_values_start: 186, raw_values_count: 1, token_types_start: 187, token_types_count: 0 }, // [64]
    SimpleHintData { raw_values_start: 187, raw_values_count: 1, token_types_start: 188, token_types_count: 0 }, // [65]
    SimpleHintData { raw_values_start: 188, raw_values_count: 1, token_types_start: 189, token_types_count: 0 }, // [66]
    SimpleHintData { raw_values_start: 189, raw_values_count: 1, token_types_start: 190, token_types_count: 0 }, // [67]
    SimpleHintData { raw_values_start: 190, raw_values_count: 1, token_types_start: 191, token_types_count: 0 }, // [68]
    SimpleHintData { raw_values_start: 191, raw_values_count: 1, token_types_start: 192, token_types_count: 0 }, // [69]
    SimpleHintData { raw_values_start: 192, raw_values_count: 1, token_types_start: 193, token_types_count: 0 }, // [70]
    SimpleHintData { raw_values_start: 193, raw_values_count: 1, token_types_start: 194, token_types_count: 0 }, // [71]
    SimpleHintData { raw_values_start: 194, raw_values_count: 1, token_types_start: 195, token_types_count: 0 }, // [72]
    SimpleHintData { raw_values_start: 195, raw_values_count: 1, token_types_start: 196, token_types_count: 0 }, // [73]
    SimpleHintData { raw_values_start: 196, raw_values_count: 1, token_types_start: 197, token_types_count: 0 }, // [74]
    SimpleHintData { raw_values_start: 197, raw_values_count: 1, token_types_start: 198, token_types_count: 0 }, // [75]
    SimpleHintData { raw_values_start: 198, raw_values_count: 1, token_types_start: 199, token_types_count: 0 }, // [76]
    SimpleHintData { raw_values_start: 199, raw_values_count: 1, token_types_start: 200, token_types_count: 0 }, // [77]
    SimpleHintData { raw_values_start: 200, raw_values_count: 1, token_types_start: 201, token_types_count: 0 }, // [78]
    SimpleHintData { raw_values_start: 201, raw_values_count: 1, token_types_start: 202, token_types_count: 0 }, // [79]
    SimpleHintData { raw_values_start: 202, raw_values_count: 1, token_types_start: 203, token_types_count: 0 }, // [80]
    SimpleHintData { raw_values_start: 203, raw_values_count: 1, token_types_start: 204, token_types_count: 0 }, // [81]
    SimpleHintData { raw_values_start: 204, raw_values_count: 1, token_types_start: 205, token_types_count: 0 }, // [82]
    SimpleHintData { raw_values_start: 205, raw_values_count: 1, token_types_start: 206, token_types_count: 0 }, // [83]
    SimpleHintData { raw_values_start: 206, raw_values_count: 1, token_types_start: 207, token_types_count: 0 }, // [84]
    SimpleHintData { raw_values_start: 207, raw_values_count: 1, token_types_start: 208, token_types_count: 0 }, // [85]
    SimpleHintData { raw_values_start: 208, raw_values_count: 1, token_types_start: 209, token_types_count: 0 }, // [86]
    SimpleHintData { raw_values_start: 209, raw_values_count: 1, token_types_start: 210, token_types_count: 0 }, // [87]
    SimpleHintData { raw_values_start: 210, raw_values_count: 1, token_types_start: 211, token_types_count: 0 }, // [88]
    SimpleHintData { raw_values_start: 211, raw_values_count: 1, token_types_start: 212, token_types_count: 0 }, // [89]
    SimpleHintData { raw_values_start: 212, raw_values_count: 1, token_types_start: 213, token_types_count: 0 }, // [90]
    SimpleHintData { raw_values_start: 213, raw_values_count: 1, token_types_start: 214, token_types_count: 0 }, // [91]
    SimpleHintData { raw_values_start: 214, raw_values_count: 1, token_types_start: 215, token_types_count: 0 }, // [92]
    SimpleHintData { raw_values_start: 215, raw_values_count: 1, token_types_start: 216, token_types_count: 0 }, // [93]
    SimpleHintData { raw_values_start: 216, raw_values_count: 1, token_types_start: 217, token_types_count: 0 }, // [94]
    SimpleHintData { raw_values_start: 217, raw_values_count: 1, token_types_start: 218, token_types_count: 0 }, // [95]
    SimpleHintData { raw_values_start: 218, raw_values_count: 1, token_types_start: 219, token_types_count: 0 }, // [96]
    SimpleHintData { raw_values_start: 219, raw_values_count: 1, token_types_start: 220, token_types_count: 0 }, // [97]
    SimpleHintData { raw_values_start: 220, raw_values_count: 2, token_types_start: 222, token_types_count: 0 }, // [98]
    SimpleHintData { raw_values_start: 222, raw_values_count: 1, token_types_start: 223, token_types_count: 0 }, // [99]
    SimpleHintData { raw_values_start: 223, raw_values_count: 1, token_types_start: 224, token_types_count: 0 }, // [100]
    SimpleHintData { raw_values_start: 224, raw_values_count: 1, token_types_start: 225, token_types_count: 0 }, // [101]
    SimpleHintData { raw_values_start: 225, raw_values_count: 1, token_types_start: 226, token_types_count: 0 }, // [102]
    SimpleHintData { raw_values_start: 226, raw_values_count: 1, token_types_start: 227, token_types_count: 0 }, // [103]
    SimpleHintData { raw_values_start: 227, raw_values_count: 1, token_types_start: 228, token_types_count: 0 }, // [104]
    SimpleHintData { raw_values_start: 228, raw_values_count: 1, token_types_start: 229, token_types_count: 0 }, // [105]
    SimpleHintData { raw_values_start: 229, raw_values_count: 1, token_types_start: 230, token_types_count: 0 }, // [106]
    SimpleHintData { raw_values_start: 230, raw_values_count: 1, token_types_start: 231, token_types_count: 0 }, // [107]
    SimpleHintData { raw_values_start: 231, raw_values_count: 1, token_types_start: 232, token_types_count: 0 }, // [108]
    SimpleHintData { raw_values_start: 232, raw_values_count: 1, token_types_start: 233, token_types_count: 0 }, // [109]
    SimpleHintData { raw_values_start: 233, raw_values_count: 1, token_types_start: 234, token_types_count: 0 }, // [110]
    SimpleHintData { raw_values_start: 234, raw_values_count: 1, token_types_start: 235, token_types_count: 0 }, // [111]
    SimpleHintData { raw_values_start: 235, raw_values_count: 1, token_types_start: 236, token_types_count: 0 }, // [112]
    SimpleHintData { raw_values_start: 236, raw_values_count: 1, token_types_start: 237, token_types_count: 0 }, // [113]
    SimpleHintData { raw_values_start: 237, raw_values_count: 1, token_types_start: 238, token_types_count: 0 }, // [114]
    SimpleHintData { raw_values_start: 238, raw_values_count: 1, token_types_start: 239, token_types_count: 0 }, // [115]
    SimpleHintData { raw_values_start: 239, raw_values_count: 1, token_types_start: 240, token_types_count: 0 }, // [116]
    SimpleHintData { raw_values_start: 240, raw_values_count: 1, token_types_start: 241, token_types_count: 0 }, // [117]
    SimpleHintData { raw_values_start: 241, raw_values_count: 1, token_types_start: 242, token_types_count: 0 }, // [118]
    SimpleHintData { raw_values_start: 242, raw_values_count: 1, token_types_start: 243, token_types_count: 0 }, // [119]
    SimpleHintData { raw_values_start: 243, raw_values_count: 4, token_types_start: 247, token_types_count: 0 }, // [120]
    SimpleHintData { raw_values_start: 247, raw_values_count: 1, token_types_start: 248, token_types_count: 0 }, // [121]
    SimpleHintData { raw_values_start: 248, raw_values_count: 1, token_types_start: 249, token_types_count: 0 }, // [122]
    SimpleHintData { raw_values_start: 249, raw_values_count: 2, token_types_start: 251, token_types_count: 0 }, // [123]
    SimpleHintData { raw_values_start: 251, raw_values_count: 0, token_types_start: 251, token_types_count: 3 }, // [124]
    SimpleHintData { raw_values_start: 254, raw_values_count: 1, token_types_start: 255, token_types_count: 0 }, // [125]
    SimpleHintData { raw_values_start: 255, raw_values_count: 1, token_types_start: 256, token_types_count: 0 }, // [126]
    SimpleHintData { raw_values_start: 256, raw_values_count: 10, token_types_start: 266, token_types_count: 0 }, // [127]
    SimpleHintData { raw_values_start: 266, raw_values_count: 1, token_types_start: 267, token_types_count: 0 }, // [128]
    SimpleHintData { raw_values_start: 267, raw_values_count: 1, token_types_start: 268, token_types_count: 0 }, // [129]
    SimpleHintData { raw_values_start: 268, raw_values_count: 2, token_types_start: 270, token_types_count: 0 }, // [130]
    SimpleHintData { raw_values_start: 270, raw_values_count: 2, token_types_start: 272, token_types_count: 0 }, // [131]
    SimpleHintData { raw_values_start: 272, raw_values_count: 2, token_types_start: 274, token_types_count: 0 }, // [132]
    SimpleHintData { raw_values_start: 274, raw_values_count: 1, token_types_start: 275, token_types_count: 0 }, // [133]
    SimpleHintData { raw_values_start: 275, raw_values_count: 1, token_types_start: 276, token_types_count: 0 }, // [134]
    SimpleHintData { raw_values_start: 276, raw_values_count: 2, token_types_start: 278, token_types_count: 0 }, // [135]
    SimpleHintData { raw_values_start: 278, raw_values_count: 1, token_types_start: 279, token_types_count: 0 }, // [136]
    SimpleHintData { raw_values_start: 279, raw_values_count: 1, token_types_start: 280, token_types_count: 0 }, // [137]
    SimpleHintData { raw_values_start: 280, raw_values_count: 2, token_types_start: 282, token_types_count: 0 }, // [138]
    SimpleHintData { raw_values_start: 282, raw_values_count: 1, token_types_start: 283, token_types_count: 0 }, // [139]
    SimpleHintData { raw_values_start: 283, raw_values_count: 1, token_types_start: 284, token_types_count: 0 }, // [140]
    SimpleHintData { raw_values_start: 284, raw_values_count: 1, token_types_start: 285, token_types_count: 0 }, // [141]
    SimpleHintData { raw_values_start: 285, raw_values_count: 1, token_types_start: 286, token_types_count: 0 }, // [142]
    SimpleHintData { raw_values_start: 286, raw_values_count: 12, token_types_start: 298, token_types_count: 5 }, // [143]
    SimpleHintData { raw_values_start: 303, raw_values_count: 2, token_types_start: 305, token_types_count: 0 }, // [144]
    SimpleHintData { raw_values_start: 305, raw_values_count: 3, token_types_start: 308, token_types_count: 0 }, // [145]
    SimpleHintData { raw_values_start: 308, raw_values_count: 2, token_types_start: 310, token_types_count: 0 }, // [146]
    SimpleHintData { raw_values_start: 310, raw_values_count: 1, token_types_start: 311, token_types_count: 0 }, // [147]
    SimpleHintData { raw_values_start: 311, raw_values_count: 1, token_types_start: 312, token_types_count: 0 }, // [148]
    SimpleHintData { raw_values_start: 312, raw_values_count: 2, token_types_start: 314, token_types_count: 0 }, // [149]
    SimpleHintData { raw_values_start: 314, raw_values_count: 1, token_types_start: 315, token_types_count: 0 }, // [150]
    SimpleHintData { raw_values_start: 315, raw_values_count: 1, token_types_start: 316, token_types_count: 0 }, // [151]
    SimpleHintData { raw_values_start: 316, raw_values_count: 1, token_types_start: 317, token_types_count: 0 }, // [152]
    SimpleHintData { raw_values_start: 317, raw_values_count: 2, token_types_start: 319, token_types_count: 0 }, // [153]
    SimpleHintData { raw_values_start: 319, raw_values_count: 1, token_types_start: 320, token_types_count: 0 }, // [154]
    SimpleHintData { raw_values_start: 320, raw_values_count: 2, token_types_start: 322, token_types_count: 0 }, // [155]
    SimpleHintData { raw_values_start: 322, raw_values_count: 1, token_types_start: 323, token_types_count: 0 }, // [156]
    SimpleHintData { raw_values_start: 323, raw_values_count: 1, token_types_start: 324, token_types_count: 0 }, // [157]
    SimpleHintData { raw_values_start: 324, raw_values_count: 1, token_types_start: 325, token_types_count: 0 }, // [158]
    SimpleHintData { raw_values_start: 325, raw_values_count: 1, token_types_start: 326, token_types_count: 0 }, // [159]
    SimpleHintData { raw_values_start: 326, raw_values_count: 1, token_types_start: 327, token_types_count: 0 }, // [160]
    SimpleHintData { raw_values_start: 327, raw_values_count: 1, token_types_start: 328, token_types_count: 0 }, // [161]
    SimpleHintData { raw_values_start: 328, raw_values_count: 1, token_types_start: 329, token_types_count: 0 }, // [162]
    SimpleHintData { raw_values_start: 329, raw_values_count: 1, token_types_start: 330, token_types_count: 0 }, // [163]
    SimpleHintData { raw_values_start: 330, raw_values_count: 10, token_types_start: 340, token_types_count: 0 }, // [164]
    SimpleHintData { raw_values_start: 340, raw_values_count: 1, token_types_start: 341, token_types_count: 0 }, // [165]
    SimpleHintData { raw_values_start: 341, raw_values_count: 1, token_types_start: 342, token_types_count: 0 }, // [166]
    SimpleHintData { raw_values_start: 342, raw_values_count: 1, token_types_start: 343, token_types_count: 0 }, // [167]
    SimpleHintData { raw_values_start: 343, raw_values_count: 1, token_types_start: 344, token_types_count: 0 }, // [168]
    SimpleHintData { raw_values_start: 344, raw_values_count: 1, token_types_start: 345, token_types_count: 0 }, // [169]
    SimpleHintData { raw_values_start: 345, raw_values_count: 1, token_types_start: 346, token_types_count: 0 }, // [170]
    SimpleHintData { raw_values_start: 346, raw_values_count: 1, token_types_start: 347, token_types_count: 0 }, // [171]
    SimpleHintData { raw_values_start: 347, raw_values_count: 1, token_types_start: 348, token_types_count: 0 }, // [172]
    SimpleHintData { raw_values_start: 348, raw_values_count: 1, token_types_start: 349, token_types_count: 0 }, // [173]
    SimpleHintData { raw_values_start: 349, raw_values_count: 1, token_types_start: 350, token_types_count: 0 }, // [174]
    SimpleHintData { raw_values_start: 350, raw_values_count: 1, token_types_start: 351, token_types_count: 0 }, // [175]
    SimpleHintData { raw_values_start: 351, raw_values_count: 1, token_types_start: 352, token_types_count: 0 }, // [176]
    SimpleHintData { raw_values_start: 352, raw_values_count: 22, token_types_start: 374, token_types_count: 0 }, // [177]
    SimpleHintData { raw_values_start: 374, raw_values_count: 1, token_types_start: 375, token_types_count: 0 }, // [178]
    SimpleHintData { raw_values_start: 375, raw_values_count: 1, token_types_start: 376, token_types_count: 0 }, // [179]
    SimpleHintData { raw_values_start: 376, raw_values_count: 1, token_types_start: 377, token_types_count: 0 }, // [180]
    SimpleHintData { raw_values_start: 377, raw_values_count: 1, token_types_start: 378, token_types_count: 0 }, // [181]
    SimpleHintData { raw_values_start: 378, raw_values_count: 1, token_types_start: 379, token_types_count: 0 }, // [182]
    SimpleHintData { raw_values_start: 379, raw_values_count: 1, token_types_start: 380, token_types_count: 0 }, // [183]
    SimpleHintData { raw_values_start: 380, raw_values_count: 1, token_types_start: 381, token_types_count: 0 }, // [184]
    SimpleHintData { raw_values_start: 381, raw_values_count: 1, token_types_start: 382, token_types_count: 0 }, // [185]
    SimpleHintData { raw_values_start: 382, raw_values_count: 1, token_types_start: 383, token_types_count: 0 }, // [186]
    SimpleHintData { raw_values_start: 383, raw_values_count: 1, token_types_start: 384, token_types_count: 0 }, // [187]
    SimpleHintData { raw_values_start: 384, raw_values_count: 1, token_types_start: 385, token_types_count: 0 }, // [188]
    SimpleHintData { raw_values_start: 385, raw_values_count: 1, token_types_start: 386, token_types_count: 0 }, // [189]
    SimpleHintData { raw_values_start: 386, raw_values_count: 1, token_types_start: 387, token_types_count: 0 }, // [190]
    SimpleHintData { raw_values_start: 387, raw_values_count: 1, token_types_start: 388, token_types_count: 0 }, // [191]
    SimpleHintData { raw_values_start: 388, raw_values_count: 1, token_types_start: 389, token_types_count: 0 }, // [192]
    SimpleHintData { raw_values_start: 389, raw_values_count: 1, token_types_start: 390, token_types_count: 0 }, // [193]
    SimpleHintData { raw_values_start: 390, raw_values_count: 0, token_types_start: 390, token_types_count: 1 }, // [194]
    SimpleHintData { raw_values_start: 391, raw_values_count: 1, token_types_start: 392, token_types_count: 0 }, // [195]
    SimpleHintData { raw_values_start: 392, raw_values_count: 1, token_types_start: 393, token_types_count: 0 }, // [196]
    SimpleHintData { raw_values_start: 393, raw_values_count: 3, token_types_start: 396, token_types_count: 0 }, // [197]
    SimpleHintData { raw_values_start: 396, raw_values_count: 2, token_types_start: 398, token_types_count: 4 }, // [198]
    SimpleHintData { raw_values_start: 402, raw_values_count: 1, token_types_start: 403, token_types_count: 0 }, // [199]
    SimpleHintData { raw_values_start: 403, raw_values_count: 1, token_types_start: 404, token_types_count: 0 }, // [200]
    SimpleHintData { raw_values_start: 404, raw_values_count: 1, token_types_start: 405, token_types_count: 0 }, // [201]
    SimpleHintData { raw_values_start: 405, raw_values_count: 1, token_types_start: 406, token_types_count: 0 }, // [202]
    SimpleHintData { raw_values_start: 406, raw_values_count: 1, token_types_start: 407, token_types_count: 0 }, // [203]
    SimpleHintData { raw_values_start: 407, raw_values_count: 1, token_types_start: 408, token_types_count: 0 }, // [204]
    SimpleHintData { raw_values_start: 408, raw_values_count: 15, token_types_start: 423, token_types_count: 1 }, // [205]
    SimpleHintData { raw_values_start: 424, raw_values_count: 2, token_types_start: 426, token_types_count: 0 }, // [206]
    SimpleHintData { raw_values_start: 426, raw_values_count: 5, token_types_start: 431, token_types_count: 1 }, // [207]
    SimpleHintData { raw_values_start: 432, raw_values_count: 1, token_types_start: 433, token_types_count: 0 }, // [208]
    SimpleHintData { raw_values_start: 433, raw_values_count: 1, token_types_start: 434, token_types_count: 0 }, // [209]
    SimpleHintData { raw_values_start: 434, raw_values_count: 1, token_types_start: 435, token_types_count: 0 }, // [210]
    SimpleHintData { raw_values_start: 435, raw_values_count: 1, token_types_start: 436, token_types_count: 0 }, // [211]
    SimpleHintData { raw_values_start: 436, raw_values_count: 1, token_types_start: 437, token_types_count: 0 }, // [212]
    SimpleHintData { raw_values_start: 437, raw_values_count: 1, token_types_start: 438, token_types_count: 0 }, // [213]
    SimpleHintData { raw_values_start: 438, raw_values_count: 1, token_types_start: 439, token_types_count: 0 }, // [214]
    SimpleHintData { raw_values_start: 439, raw_values_count: 1, token_types_start: 440, token_types_count: 0 }, // [215]
    SimpleHintData { raw_values_start: 440, raw_values_count: 2, token_types_start: 442, token_types_count: 0 }, // [216]
    SimpleHintData { raw_values_start: 442, raw_values_count: 1, token_types_start: 443, token_types_count: 0 }, // [217]
    SimpleHintData { raw_values_start: 443, raw_values_count: 1, token_types_start: 444, token_types_count: 0 }, // [218]
    SimpleHintData { raw_values_start: 444, raw_values_count: 1, token_types_start: 445, token_types_count: 0 }, // [219]
    SimpleHintData { raw_values_start: 445, raw_values_count: 4, token_types_start: 449, token_types_count: 0 }, // [220]
    SimpleHintData { raw_values_start: 449, raw_values_count: 1, token_types_start: 450, token_types_count: 0 }, // [221]
    SimpleHintData { raw_values_start: 450, raw_values_count: 1, token_types_start: 451, token_types_count: 0 }, // [222]
    SimpleHintData { raw_values_start: 451, raw_values_count: 1, token_types_start: 452, token_types_count: 0 }, // [223]
    SimpleHintData { raw_values_start: 452, raw_values_count: 1, token_types_start: 453, token_types_count: 0 }, // [224]
    SimpleHintData { raw_values_start: 453, raw_values_count: 1, token_types_start: 454, token_types_count: 0 }, // [225]
    SimpleHintData { raw_values_start: 454, raw_values_count: 1, token_types_start: 455, token_types_count: 0 }, // [226]
    SimpleHintData { raw_values_start: 455, raw_values_count: 5, token_types_start: 460, token_types_count: 0 }, // [227]
    SimpleHintData { raw_values_start: 460, raw_values_count: 1, token_types_start: 461, token_types_count: 0 }, // [228]
    SimpleHintData { raw_values_start: 461, raw_values_count: 1, token_types_start: 462, token_types_count: 0 }, // [229]
    SimpleHintData { raw_values_start: 462, raw_values_count: 1, token_types_start: 463, token_types_count: 0 }, // [230]
    SimpleHintData { raw_values_start: 463, raw_values_count: 1, token_types_start: 464, token_types_count: 0 }, // [231]
    SimpleHintData { raw_values_start: 464, raw_values_count: 1, token_types_start: 465, token_types_count: 0 }, // [232]
    SimpleHintData { raw_values_start: 465, raw_values_count: 1, token_types_start: 466, token_types_count: 0 }, // [233]
    SimpleHintData { raw_values_start: 466, raw_values_count: 1, token_types_start: 467, token_types_count: 0 }, // [234]
    SimpleHintData { raw_values_start: 467, raw_values_count: 1, token_types_start: 468, token_types_count: 0 }, // [235]
    SimpleHintData { raw_values_start: 468, raw_values_count: 1, token_types_start: 469, token_types_count: 0 }, // [236]
    SimpleHintData { raw_values_start: 469, raw_values_count: 1, token_types_start: 470, token_types_count: 0 }, // [237]
    SimpleHintData { raw_values_start: 470, raw_values_count: 1, token_types_start: 471, token_types_count: 0 }, // [238]
    SimpleHintData { raw_values_start: 471, raw_values_count: 1, token_types_start: 472, token_types_count: 0 }, // [239]
    SimpleHintData { raw_values_start: 472, raw_values_count: 1, token_types_start: 473, token_types_count: 0 }, // [240]
    SimpleHintData { raw_values_start: 473, raw_values_count: 1, token_types_start: 474, token_types_count: 0 }, // [241]
    SimpleHintData { raw_values_start: 474, raw_values_count: 1, token_types_start: 475, token_types_count: 0 }, // [242]
    SimpleHintData { raw_values_start: 475, raw_values_count: 1, token_types_start: 476, token_types_count: 0 }, // [243]
    SimpleHintData { raw_values_start: 476, raw_values_count: 1, token_types_start: 477, token_types_count: 0 }, // [244]
    SimpleHintData { raw_values_start: 477, raw_values_count: 1, token_types_start: 478, token_types_count: 0 }, // [245]
    SimpleHintData { raw_values_start: 478, raw_values_count: 1, token_types_start: 479, token_types_count: 0 }, // [246]
    SimpleHintData { raw_values_start: 479, raw_values_count: 1, token_types_start: 480, token_types_count: 0 }, // [247]
    SimpleHintData { raw_values_start: 480, raw_values_count: 1, token_types_start: 481, token_types_count: 0 }, // [248]
    SimpleHintData { raw_values_start: 481, raw_values_count: 1, token_types_start: 482, token_types_count: 0 }, // [249]
    SimpleHintData { raw_values_start: 482, raw_values_count: 1, token_types_start: 483, token_types_count: 0 }, // [250]
    SimpleHintData { raw_values_start: 483, raw_values_count: 1, token_types_start: 484, token_types_count: 0 }, // [251]
    SimpleHintData { raw_values_start: 484, raw_values_count: 1, token_types_start: 485, token_types_count: 0 }, // [252]
    SimpleHintData { raw_values_start: 485, raw_values_count: 1, token_types_start: 486, token_types_count: 0 }, // [253]
    SimpleHintData { raw_values_start: 486, raw_values_count: 1, token_types_start: 487, token_types_count: 0 }, // [254]
    SimpleHintData { raw_values_start: 487, raw_values_count: 1, token_types_start: 488, token_types_count: 0 }, // [255]
    SimpleHintData { raw_values_start: 488, raw_values_count: 1, token_types_start: 489, token_types_count: 0 }, // [256]
    SimpleHintData { raw_values_start: 489, raw_values_count: 1, token_types_start: 490, token_types_count: 0 }, // [257]
    SimpleHintData { raw_values_start: 490, raw_values_count: 1, token_types_start: 491, token_types_count: 0 }, // [258]
    SimpleHintData { raw_values_start: 491, raw_values_count: 1, token_types_start: 492, token_types_count: 0 }, // [259]
    SimpleHintData { raw_values_start: 492, raw_values_count: 1, token_types_start: 493, token_types_count: 0 }, // [260]
    SimpleHintData { raw_values_start: 493, raw_values_count: 1, token_types_start: 494, token_types_count: 0 }, // [261]
    SimpleHintData { raw_values_start: 494, raw_values_count: 1, token_types_start: 495, token_types_count: 0 }, // [262]
    SimpleHintData { raw_values_start: 495, raw_values_count: 2, token_types_start: 497, token_types_count: 0 }, // [263]
    SimpleHintData { raw_values_start: 497, raw_values_count: 1, token_types_start: 498, token_types_count: 0 }, // [264]
    SimpleHintData { raw_values_start: 498, raw_values_count: 1, token_types_start: 499, token_types_count: 0 }, // [265]
    SimpleHintData { raw_values_start: 499, raw_values_count: 1, token_types_start: 500, token_types_count: 0 }, // [266]
    SimpleHintData { raw_values_start: 500, raw_values_count: 1, token_types_start: 501, token_types_count: 0 }, // [267]
    SimpleHintData { raw_values_start: 501, raw_values_count: 1, token_types_start: 502, token_types_count: 0 }, // [268]
    SimpleHintData { raw_values_start: 502, raw_values_count: 1, token_types_start: 503, token_types_count: 0 }, // [269]
    SimpleHintData { raw_values_start: 503, raw_values_count: 1, token_types_start: 504, token_types_count: 0 }, // [270]
    SimpleHintData { raw_values_start: 504, raw_values_count: 1, token_types_start: 505, token_types_count: 0 }, // [271]
    SimpleHintData { raw_values_start: 505, raw_values_count: 1, token_types_start: 506, token_types_count: 0 }, // [272]
    SimpleHintData { raw_values_start: 506, raw_values_count: 1, token_types_start: 507, token_types_count: 0 }, // [273]
    SimpleHintData { raw_values_start: 507, raw_values_count: 1, token_types_start: 508, token_types_count: 0 }, // [274]
    SimpleHintData { raw_values_start: 508, raw_values_count: 1, token_types_start: 509, token_types_count: 0 }, // [275]
    SimpleHintData { raw_values_start: 509, raw_values_count: 1, token_types_start: 510, token_types_count: 0 }, // [276]
    SimpleHintData { raw_values_start: 510, raw_values_count: 2, token_types_start: 512, token_types_count: 0 }, // [277]
    SimpleHintData { raw_values_start: 512, raw_values_count: 1, token_types_start: 513, token_types_count: 0 }, // [278]
    SimpleHintData { raw_values_start: 513, raw_values_count: 1, token_types_start: 514, token_types_count: 0 }, // [279]
    SimpleHintData { raw_values_start: 514, raw_values_count: 1, token_types_start: 515, token_types_count: 0 }, // [280]
    SimpleHintData { raw_values_start: 515, raw_values_count: 1, token_types_start: 516, token_types_count: 0 }, // [281]
    SimpleHintData { raw_values_start: 516, raw_values_count: 1, token_types_start: 517, token_types_count: 0 }, // [282]
    SimpleHintData { raw_values_start: 517, raw_values_count: 1, token_types_start: 518, token_types_count: 0 }, // [283]
    SimpleHintData { raw_values_start: 518, raw_values_count: 1, token_types_start: 519, token_types_count: 0 }, // [284]
    SimpleHintData { raw_values_start: 519, raw_values_count: 1, token_types_start: 520, token_types_count: 0 }, // [285]
    SimpleHintData { raw_values_start: 520, raw_values_count: 1, token_types_start: 521, token_types_count: 0 }, // [286]
    SimpleHintData { raw_values_start: 521, raw_values_count: 1, token_types_start: 522, token_types_count: 0 }, // [287]
    SimpleHintData { raw_values_start: 522, raw_values_count: 1, token_types_start: 523, token_types_count: 0 }, // [288]
    SimpleHintData { raw_values_start: 523, raw_values_count: 1, token_types_start: 524, token_types_count: 0 }, // [289]
    SimpleHintData { raw_values_start: 524, raw_values_count: 1, token_types_start: 525, token_types_count: 0 }, // [290]
    SimpleHintData { raw_values_start: 525, raw_values_count: 1, token_types_start: 526, token_types_count: 0 }, // [291]
    SimpleHintData { raw_values_start: 526, raw_values_count: 2, token_types_start: 528, token_types_count: 0 }, // [292]
    SimpleHintData { raw_values_start: 528, raw_values_count: 0, token_types_start: 528, token_types_count: 1 }, // [293]
    SimpleHintData { raw_values_start: 529, raw_values_count: 1, token_types_start: 530, token_types_count: 0 }, // [294]
    SimpleHintData { raw_values_start: 530, raw_values_count: 1, token_types_start: 531, token_types_count: 0 }, // [295]
    SimpleHintData { raw_values_start: 531, raw_values_count: 1, token_types_start: 532, token_types_count: 0 }, // [296]
    SimpleHintData { raw_values_start: 532, raw_values_count: 1, token_types_start: 533, token_types_count: 0 }, // [297]
    SimpleHintData { raw_values_start: 533, raw_values_count: 1, token_types_start: 534, token_types_count: 0 }, // [298]
    SimpleHintData { raw_values_start: 534, raw_values_count: 1, token_types_start: 535, token_types_count: 0 }, // [299]
    SimpleHintData { raw_values_start: 535, raw_values_count: 1, token_types_start: 536, token_types_count: 0 }, // [300]
    SimpleHintData { raw_values_start: 536, raw_values_count: 1, token_types_start: 537, token_types_count: 0 }, // [301]
    SimpleHintData { raw_values_start: 537, raw_values_count: 4, token_types_start: 541, token_types_count: 0 }, // [302]
    SimpleHintData { raw_values_start: 541, raw_values_count: 1, token_types_start: 542, token_types_count: 0 }, // [303]
    SimpleHintData { raw_values_start: 542, raw_values_count: 1, token_types_start: 543, token_types_count: 0 }, // [304]
    SimpleHintData { raw_values_start: 543, raw_values_count: 1, token_types_start: 544, token_types_count: 0 }, // [305]
    SimpleHintData { raw_values_start: 544, raw_values_count: 1, token_types_start: 545, token_types_count: 0 }, // [306]
    SimpleHintData { raw_values_start: 545, raw_values_count: 1, token_types_start: 546, token_types_count: 0 }, // [307]
    SimpleHintData { raw_values_start: 546, raw_values_count: 1, token_types_start: 547, token_types_count: 0 }, // [308]
    SimpleHintData { raw_values_start: 547, raw_values_count: 1, token_types_start: 548, token_types_count: 0 }, // [309]
    SimpleHintData { raw_values_start: 548, raw_values_count: 1, token_types_start: 549, token_types_count: 0 }, // [310]
    SimpleHintData { raw_values_start: 549, raw_values_count: 1, token_types_start: 550, token_types_count: 0 }, // [311]
    SimpleHintData { raw_values_start: 550, raw_values_count: 1, token_types_start: 551, token_types_count: 0 }, // [312]
    SimpleHintData { raw_values_start: 551, raw_values_count: 1, token_types_start: 552, token_types_count: 0 }, // [313]
    SimpleHintData { raw_values_start: 552, raw_values_count: 1, token_types_start: 553, token_types_count: 0 }, // [314]
    SimpleHintData { raw_values_start: 553, raw_values_count: 1, token_types_start: 554, token_types_count: 0 }, // [315]
    SimpleHintData { raw_values_start: 554, raw_values_count: 1, token_types_start: 555, token_types_count: 0 }, // [316]
    SimpleHintData { raw_values_start: 555, raw_values_count: 1, token_types_start: 556, token_types_count: 0 }, // [317]
    SimpleHintData { raw_values_start: 556, raw_values_count: 1, token_types_start: 557, token_types_count: 0 }, // [318]
    SimpleHintData { raw_values_start: 557, raw_values_count: 1, token_types_start: 558, token_types_count: 0 }, // [319]
    SimpleHintData { raw_values_start: 558, raw_values_count: 1, token_types_start: 559, token_types_count: 0 }, // [320]
    SimpleHintData { raw_values_start: 559, raw_values_count: 1, token_types_start: 560, token_types_count: 0 }, // [321]
    SimpleHintData { raw_values_start: 560, raw_values_count: 1, token_types_start: 561, token_types_count: 0 }, // [322]
    SimpleHintData { raw_values_start: 561, raw_values_count: 7, token_types_start: 568, token_types_count: 0 }, // [323]
    SimpleHintData { raw_values_start: 568, raw_values_count: 4, token_types_start: 572, token_types_count: 0 }, // [324]
    SimpleHintData { raw_values_start: 572, raw_values_count: 3, token_types_start: 575, token_types_count: 0 }, // [325]
    SimpleHintData { raw_values_start: 575, raw_values_count: 1, token_types_start: 576, token_types_count: 0 }, // [326]
    SimpleHintData { raw_values_start: 576, raw_values_count: 1, token_types_start: 577, token_types_count: 0 }, // [327]
    SimpleHintData { raw_values_start: 577, raw_values_count: 1, token_types_start: 578, token_types_count: 0 }, // [328]
    SimpleHintData { raw_values_start: 578, raw_values_count: 1, token_types_start: 579, token_types_count: 0 }, // [329]
    SimpleHintData { raw_values_start: 579, raw_values_count: 2, token_types_start: 581, token_types_count: 0 }, // [330]
    SimpleHintData { raw_values_start: 581, raw_values_count: 1, token_types_start: 582, token_types_count: 0 }, // [331]
    SimpleHintData { raw_values_start: 582, raw_values_count: 1, token_types_start: 583, token_types_count: 0 }, // [332]
    SimpleHintData { raw_values_start: 583, raw_values_count: 1, token_types_start: 584, token_types_count: 0 }, // [333]
    SimpleHintData { raw_values_start: 584, raw_values_count: 1, token_types_start: 585, token_types_count: 0 }, // [334]
    SimpleHintData { raw_values_start: 585, raw_values_count: 11, token_types_start: 596, token_types_count: 0 }, // [335]
    SimpleHintData { raw_values_start: 596, raw_values_count: 1, token_types_start: 597, token_types_count: 0 }, // [336]
    SimpleHintData { raw_values_start: 597, raw_values_count: 11, token_types_start: 608, token_types_count: 0 }, // [337]
    SimpleHintData { raw_values_start: 608, raw_values_count: 10, token_types_start: 618, token_types_count: 0 }, // [338]
    SimpleHintData { raw_values_start: 618, raw_values_count: 4, token_types_start: 622, token_types_count: 0 }, // [339]
    SimpleHintData { raw_values_start: 622, raw_values_count: 1, token_types_start: 623, token_types_count: 0 }, // [340]
    SimpleHintData { raw_values_start: 623, raw_values_count: 1, token_types_start: 624, token_types_count: 0 }, // [341]
    SimpleHintData { raw_values_start: 624, raw_values_count: 1, token_types_start: 625, token_types_count: 0 }, // [342]
    SimpleHintData { raw_values_start: 625, raw_values_count: 2, token_types_start: 627, token_types_count: 0 }, // [343]
    SimpleHintData { raw_values_start: 627, raw_values_count: 1, token_types_start: 628, token_types_count: 0 }, // [344]
    SimpleHintData { raw_values_start: 628, raw_values_count: 1, token_types_start: 629, token_types_count: 0 }, // [345]
    SimpleHintData { raw_values_start: 629, raw_values_count: 1, token_types_start: 630, token_types_count: 0 }, // [346]
    SimpleHintData { raw_values_start: 630, raw_values_count: 2, token_types_start: 632, token_types_count: 0 }, // [347]
    SimpleHintData { raw_values_start: 632, raw_values_count: 1, token_types_start: 633, token_types_count: 0 }, // [348]
    SimpleHintData { raw_values_start: 633, raw_values_count: 3, token_types_start: 636, token_types_count: 0 }, // [349]
    SimpleHintData { raw_values_start: 636, raw_values_count: 1, token_types_start: 637, token_types_count: 0 }, // [350]
    SimpleHintData { raw_values_start: 637, raw_values_count: 1, token_types_start: 638, token_types_count: 0 }, // [351]
    SimpleHintData { raw_values_start: 638, raw_values_count: 3, token_types_start: 641, token_types_count: 0 }, // [352]
    SimpleHintData { raw_values_start: 641, raw_values_count: 7, token_types_start: 648, token_types_count: 0 }, // [353]
    SimpleHintData { raw_values_start: 648, raw_values_count: 1, token_types_start: 649, token_types_count: 0 }, // [354]
    SimpleHintData { raw_values_start: 649, raw_values_count: 1, token_types_start: 650, token_types_count: 0 }, // [355]
    SimpleHintData { raw_values_start: 650, raw_values_count: 1, token_types_start: 651, token_types_count: 0 }, // [356]
    SimpleHintData { raw_values_start: 651, raw_values_count: 1, token_types_start: 652, token_types_count: 0 }, // [357]
    SimpleHintData { raw_values_start: 652, raw_values_count: 3, token_types_start: 655, token_types_count: 0 }, // [358]
    SimpleHintData { raw_values_start: 655, raw_values_count: 1, token_types_start: 656, token_types_count: 0 }, // [359]
    SimpleHintData { raw_values_start: 656, raw_values_count: 2, token_types_start: 658, token_types_count: 0 }, // [360]
    SimpleHintData { raw_values_start: 658, raw_values_count: 1, token_types_start: 659, token_types_count: 0 }, // [361]
    SimpleHintData { raw_values_start: 659, raw_values_count: 1, token_types_start: 660, token_types_count: 0 }, // [362]
    SimpleHintData { raw_values_start: 660, raw_values_count: 1, token_types_start: 661, token_types_count: 0 }, // [363]
    SimpleHintData { raw_values_start: 661, raw_values_count: 1, token_types_start: 662, token_types_count: 0 }, // [364]
    SimpleHintData { raw_values_start: 662, raw_values_count: 2, token_types_start: 664, token_types_count: 0 }, // [365]
    SimpleHintData { raw_values_start: 664, raw_values_count: 1, token_types_start: 665, token_types_count: 0 }, // [366]
    SimpleHintData { raw_values_start: 665, raw_values_count: 0, token_types_start: 665, token_types_count: 1 }, // [367]
    SimpleHintData { raw_values_start: 666, raw_values_count: 1, token_types_start: 667, token_types_count: 0 }, // [368]
    SimpleHintData { raw_values_start: 667, raw_values_count: 1, token_types_start: 668, token_types_count: 0 }, // [369]
    SimpleHintData { raw_values_start: 668, raw_values_count: 1, token_types_start: 669, token_types_count: 0 }, // [370]
    SimpleHintData { raw_values_start: 669, raw_values_count: 1, token_types_start: 670, token_types_count: 0 }, // [371]
    SimpleHintData { raw_values_start: 670, raw_values_count: 1, token_types_start: 671, token_types_count: 0 }, // [372]
    SimpleHintData { raw_values_start: 671, raw_values_count: 2, token_types_start: 673, token_types_count: 0 }, // [373]
    SimpleHintData { raw_values_start: 673, raw_values_count: 1, token_types_start: 674, token_types_count: 0 }, // [374]
    SimpleHintData { raw_values_start: 674, raw_values_count: 1, token_types_start: 675, token_types_count: 0 }, // [375]
    SimpleHintData { raw_values_start: 675, raw_values_count: 1, token_types_start: 676, token_types_count: 0 }, // [376]
    SimpleHintData { raw_values_start: 676, raw_values_count: 1, token_types_start: 677, token_types_count: 0 }, // [377]
    SimpleHintData { raw_values_start: 677, raw_values_count: 1, token_types_start: 678, token_types_count: 0 }, // [378]
    SimpleHintData { raw_values_start: 678, raw_values_count: 1, token_types_start: 679, token_types_count: 0 }, // [379]
    SimpleHintData { raw_values_start: 679, raw_values_count: 1, token_types_start: 680, token_types_count: 0 }, // [380]
    SimpleHintData { raw_values_start: 680, raw_values_count: 1, token_types_start: 681, token_types_count: 0 }, // [381]
    SimpleHintData { raw_values_start: 681, raw_values_count: 1, token_types_start: 682, token_types_count: 0 }, // [382]
    SimpleHintData { raw_values_start: 682, raw_values_count: 1, token_types_start: 683, token_types_count: 0 }, // [383]
    SimpleHintData { raw_values_start: 683, raw_values_count: 1, token_types_start: 684, token_types_count: 0 }, // [384]
    SimpleHintData { raw_values_start: 684, raw_values_count: 1, token_types_start: 685, token_types_count: 0 }, // [385]
    SimpleHintData { raw_values_start: 685, raw_values_count: 1, token_types_start: 686, token_types_count: 0 }, // [386]
    SimpleHintData { raw_values_start: 686, raw_values_count: 1, token_types_start: 687, token_types_count: 0 }, // [387]
    SimpleHintData { raw_values_start: 687, raw_values_count: 18, token_types_start: 705, token_types_count: 0 }, // [388]
    SimpleHintData { raw_values_start: 705, raw_values_count: 1, token_types_start: 706, token_types_count: 0 }, // [389]
    SimpleHintData { raw_values_start: 706, raw_values_count: 1, token_types_start: 707, token_types_count: 0 }, // [390]
    SimpleHintData { raw_values_start: 707, raw_values_count: 1, token_types_start: 708, token_types_count: 0 }, // [391]
    SimpleHintData { raw_values_start: 708, raw_values_count: 1, token_types_start: 709, token_types_count: 0 }, // [392]
    SimpleHintData { raw_values_start: 709, raw_values_count: 1, token_types_start: 710, token_types_count: 0 }, // [393]
    SimpleHintData { raw_values_start: 710, raw_values_count: 20, token_types_start: 730, token_types_count: 0 }, // [394]
    SimpleHintData { raw_values_start: 730, raw_values_count: 4, token_types_start: 734, token_types_count: 0 }, // [395]
    SimpleHintData { raw_values_start: 734, raw_values_count: 1, token_types_start: 735, token_types_count: 0 }, // [396]
    SimpleHintData { raw_values_start: 735, raw_values_count: 1, token_types_start: 736, token_types_count: 0 }, // [397]
    SimpleHintData { raw_values_start: 736, raw_values_count: 1, token_types_start: 737, token_types_count: 0 }, // [398]
    SimpleHintData { raw_values_start: 737, raw_values_count: 0, token_types_start: 737, token_types_count: 1 }, // [399]
    SimpleHintData { raw_values_start: 738, raw_values_count: 1, token_types_start: 739, token_types_count: 0 }, // [400]
    SimpleHintData { raw_values_start: 739, raw_values_count: 1, token_types_start: 740, token_types_count: 0 }, // [401]
    SimpleHintData { raw_values_start: 740, raw_values_count: 1, token_types_start: 741, token_types_count: 0 }, // [402]
    SimpleHintData { raw_values_start: 741, raw_values_count: 1, token_types_start: 742, token_types_count: 0 }, // [403]
    SimpleHintData { raw_values_start: 742, raw_values_count: 1, token_types_start: 743, token_types_count: 0 }, // [404]
    SimpleHintData { raw_values_start: 743, raw_values_count: 1, token_types_start: 744, token_types_count: 0 }, // [405]
    SimpleHintData { raw_values_start: 744, raw_values_count: 1, token_types_start: 745, token_types_count: 0 }, // [406]
    SimpleHintData { raw_values_start: 745, raw_values_count: 1, token_types_start: 746, token_types_count: 0 }, // [407]
    SimpleHintData { raw_values_start: 746, raw_values_count: 1, token_types_start: 747, token_types_count: 0 }, // [408]
    SimpleHintData { raw_values_start: 747, raw_values_count: 1, token_types_start: 748, token_types_count: 0 }, // [409]
    SimpleHintData { raw_values_start: 748, raw_values_count: 1, token_types_start: 749, token_types_count: 0 }, // [410]
    SimpleHintData { raw_values_start: 749, raw_values_count: 1, token_types_start: 750, token_types_count: 0 }, // [411]
    SimpleHintData { raw_values_start: 750, raw_values_count: 1, token_types_start: 751, token_types_count: 0 }, // [412]
    SimpleHintData { raw_values_start: 751, raw_values_count: 1, token_types_start: 752, token_types_count: 0 }, // [413]
    SimpleHintData { raw_values_start: 752, raw_values_count: 1, token_types_start: 753, token_types_count: 0 }, // [414]
    SimpleHintData { raw_values_start: 753, raw_values_count: 1, token_types_start: 754, token_types_count: 0 }, // [415]
    SimpleHintData { raw_values_start: 754, raw_values_count: 1, token_types_start: 755, token_types_count: 0 }, // [416]
    SimpleHintData { raw_values_start: 755, raw_values_count: 1, token_types_start: 756, token_types_count: 0 }, // [417]
    SimpleHintData { raw_values_start: 756, raw_values_count: 1, token_types_start: 757, token_types_count: 0 }, // [418]
    SimpleHintData { raw_values_start: 757, raw_values_count: 1, token_types_start: 758, token_types_count: 0 }, // [419]
    SimpleHintData { raw_values_start: 758, raw_values_count: 1, token_types_start: 759, token_types_count: 0 }, // [420]
    SimpleHintData { raw_values_start: 759, raw_values_count: 1, token_types_start: 760, token_types_count: 0 }, // [421]
    SimpleHintData { raw_values_start: 760, raw_values_count: 1, token_types_start: 761, token_types_count: 0 }, // [422]
    SimpleHintData { raw_values_start: 761, raw_values_count: 1, token_types_start: 762, token_types_count: 0 }, // [423]
    SimpleHintData { raw_values_start: 762, raw_values_count: 1, token_types_start: 763, token_types_count: 0 }, // [424]
    SimpleHintData { raw_values_start: 763, raw_values_count: 1, token_types_start: 764, token_types_count: 0 }, // [425]
    SimpleHintData { raw_values_start: 764, raw_values_count: 1, token_types_start: 765, token_types_count: 0 }, // [426]
    SimpleHintData { raw_values_start: 765, raw_values_count: 1, token_types_start: 766, token_types_count: 0 }, // [427]
    SimpleHintData { raw_values_start: 766, raw_values_count: 1, token_types_start: 767, token_types_count: 0 }, // [428]
    SimpleHintData { raw_values_start: 767, raw_values_count: 1, token_types_start: 768, token_types_count: 0 }, // [429]
    SimpleHintData { raw_values_start: 768, raw_values_count: 1, token_types_start: 769, token_types_count: 0 }, // [430]
    SimpleHintData { raw_values_start: 769, raw_values_count: 1, token_types_start: 770, token_types_count: 0 }, // [431]
    SimpleHintData { raw_values_start: 770, raw_values_count: 1, token_types_start: 771, token_types_count: 0 }, // [432]
    SimpleHintData { raw_values_start: 771, raw_values_count: 1, token_types_start: 772, token_types_count: 0 }, // [433]
    SimpleHintData { raw_values_start: 772, raw_values_count: 1, token_types_start: 773, token_types_count: 0 }, // [434]
    SimpleHintData { raw_values_start: 773, raw_values_count: 1, token_types_start: 774, token_types_count: 0 }, // [435]
    SimpleHintData { raw_values_start: 774, raw_values_count: 1, token_types_start: 775, token_types_count: 0 }, // [436]
    SimpleHintData { raw_values_start: 775, raw_values_count: 1, token_types_start: 776, token_types_count: 0 }, // [437]
    SimpleHintData { raw_values_start: 776, raw_values_count: 1, token_types_start: 777, token_types_count: 0 }, // [438]
    SimpleHintData { raw_values_start: 777, raw_values_count: 1, token_types_start: 778, token_types_count: 0 }, // [439]
    SimpleHintData { raw_values_start: 778, raw_values_count: 1, token_types_start: 779, token_types_count: 0 }, // [440]
    SimpleHintData { raw_values_start: 779, raw_values_count: 1, token_types_start: 780, token_types_count: 0 }, // [441]
    SimpleHintData { raw_values_start: 780, raw_values_count: 1, token_types_start: 781, token_types_count: 0 }, // [442]
    SimpleHintData { raw_values_start: 781, raw_values_count: 1, token_types_start: 782, token_types_count: 0 }, // [443]
    SimpleHintData { raw_values_start: 782, raw_values_count: 1, token_types_start: 783, token_types_count: 0 }, // [444]
    SimpleHintData { raw_values_start: 783, raw_values_count: 1, token_types_start: 784, token_types_count: 0 }, // [445]
    SimpleHintData { raw_values_start: 784, raw_values_count: 1, token_types_start: 785, token_types_count: 0 }, // [446]
    SimpleHintData { raw_values_start: 785, raw_values_count: 1, token_types_start: 786, token_types_count: 0 }, // [447]
    SimpleHintData { raw_values_start: 786, raw_values_count: 1, token_types_start: 787, token_types_count: 0 }, // [448]
    SimpleHintData { raw_values_start: 787, raw_values_count: 1, token_types_start: 788, token_types_count: 0 }, // [449]
    SimpleHintData { raw_values_start: 788, raw_values_count: 1, token_types_start: 789, token_types_count: 0 }, // [450]
    SimpleHintData { raw_values_start: 789, raw_values_count: 1, token_types_start: 790, token_types_count: 0 }, // [451]
    SimpleHintData { raw_values_start: 790, raw_values_count: 1, token_types_start: 791, token_types_count: 0 }, // [452]
    SimpleHintData { raw_values_start: 791, raw_values_count: 1, token_types_start: 792, token_types_count: 0 }, // [453]
    SimpleHintData { raw_values_start: 792, raw_values_count: 1, token_types_start: 793, token_types_count: 0 }, // [454]
    SimpleHintData { raw_values_start: 793, raw_values_count: 1, token_types_start: 794, token_types_count: 0 }, // [455]
    SimpleHintData { raw_values_start: 794, raw_values_count: 1, token_types_start: 795, token_types_count: 0 }, // [456]
    SimpleHintData { raw_values_start: 795, raw_values_count: 1, token_types_start: 796, token_types_count: 0 }, // [457]
    SimpleHintData { raw_values_start: 796, raw_values_count: 1, token_types_start: 797, token_types_count: 0 }, // [458]
    SimpleHintData { raw_values_start: 797, raw_values_count: 1, token_types_start: 798, token_types_count: 0 }, // [459]
    SimpleHintData { raw_values_start: 798, raw_values_count: 1, token_types_start: 799, token_types_count: 0 }, // [460]
    SimpleHintData { raw_values_start: 799, raw_values_count: 1, token_types_start: 800, token_types_count: 0 }, // [461]
    SimpleHintData { raw_values_start: 800, raw_values_count: 2, token_types_start: 802, token_types_count: 0 }, // [462]
    SimpleHintData { raw_values_start: 802, raw_values_count: 1, token_types_start: 803, token_types_count: 0 }, // [463]
    SimpleHintData { raw_values_start: 803, raw_values_count: 1, token_types_start: 804, token_types_count: 0 }, // [464]
    SimpleHintData { raw_values_start: 804, raw_values_count: 1, token_types_start: 805, token_types_count: 0 }, // [465]
    SimpleHintData { raw_values_start: 805, raw_values_count: 1, token_types_start: 806, token_types_count: 0 }, // [466]
    SimpleHintData { raw_values_start: 806, raw_values_count: 1, token_types_start: 807, token_types_count: 0 }, // [467]
    SimpleHintData { raw_values_start: 807, raw_values_count: 1, token_types_start: 808, token_types_count: 0 }, // [468]
    SimpleHintData { raw_values_start: 808, raw_values_count: 1, token_types_start: 809, token_types_count: 0 }, // [469]
    SimpleHintData { raw_values_start: 809, raw_values_count: 1, token_types_start: 810, token_types_count: 0 }, // [470]
    SimpleHintData { raw_values_start: 810, raw_values_count: 1, token_types_start: 811, token_types_count: 0 }, // [471]
    SimpleHintData { raw_values_start: 811, raw_values_count: 1, token_types_start: 812, token_types_count: 0 }, // [472]
    SimpleHintData { raw_values_start: 812, raw_values_count: 1, token_types_start: 813, token_types_count: 0 }, // [473]
    SimpleHintData { raw_values_start: 813, raw_values_count: 1, token_types_start: 814, token_types_count: 0 }, // [474]
    SimpleHintData { raw_values_start: 814, raw_values_count: 1, token_types_start: 815, token_types_count: 0 }, // [475]
    SimpleHintData { raw_values_start: 815, raw_values_count: 1, token_types_start: 816, token_types_count: 0 }, // [476]
    SimpleHintData { raw_values_start: 816, raw_values_count: 1, token_types_start: 817, token_types_count: 0 }, // [477]
    SimpleHintData { raw_values_start: 817, raw_values_count: 1, token_types_start: 818, token_types_count: 0 }, // [478]
    SimpleHintData { raw_values_start: 818, raw_values_count: 1, token_types_start: 819, token_types_count: 0 }, // [479]
    SimpleHintData { raw_values_start: 819, raw_values_count: 1, token_types_start: 820, token_types_count: 0 }, // [480]
    SimpleHintData { raw_values_start: 820, raw_values_count: 1, token_types_start: 821, token_types_count: 0 }, // [481]
    SimpleHintData { raw_values_start: 821, raw_values_count: 1, token_types_start: 822, token_types_count: 0 }, // [482]
    SimpleHintData { raw_values_start: 822, raw_values_count: 1, token_types_start: 823, token_types_count: 0 }, // [483]
    SimpleHintData { raw_values_start: 823, raw_values_count: 1, token_types_start: 824, token_types_count: 0 }, // [484]
    SimpleHintData { raw_values_start: 824, raw_values_count: 1, token_types_start: 825, token_types_count: 0 }, // [485]
    SimpleHintData { raw_values_start: 825, raw_values_count: 1, token_types_start: 826, token_types_count: 0 }, // [486]
    SimpleHintData { raw_values_start: 826, raw_values_count: 1, token_types_start: 827, token_types_count: 0 }, // [487]
    SimpleHintData { raw_values_start: 827, raw_values_count: 1, token_types_start: 828, token_types_count: 0 }, // [488]
    SimpleHintData { raw_values_start: 828, raw_values_count: 1, token_types_start: 829, token_types_count: 0 }, // [489]
    SimpleHintData { raw_values_start: 829, raw_values_count: 1, token_types_start: 830, token_types_count: 0 }, // [490]
    SimpleHintData { raw_values_start: 830, raw_values_count: 1, token_types_start: 831, token_types_count: 0 }, // [491]
    SimpleHintData { raw_values_start: 831, raw_values_count: 1, token_types_start: 832, token_types_count: 0 }, // [492]
    SimpleHintData { raw_values_start: 832, raw_values_count: 1, token_types_start: 833, token_types_count: 0 }, // [493]
    SimpleHintData { raw_values_start: 833, raw_values_count: 1, token_types_start: 834, token_types_count: 0 }, // [494]
    SimpleHintData { raw_values_start: 834, raw_values_count: 1, token_types_start: 835, token_types_count: 0 }, // [495]
    SimpleHintData { raw_values_start: 835, raw_values_count: 8, token_types_start: 843, token_types_count: 0 }, // [496]
    SimpleHintData { raw_values_start: 843, raw_values_count: 1, token_types_start: 844, token_types_count: 0 }, // [497]
    SimpleHintData { raw_values_start: 844, raw_values_count: 1, token_types_start: 845, token_types_count: 0 }, // [498]
    SimpleHintData { raw_values_start: 845, raw_values_count: 6, token_types_start: 851, token_types_count: 0 }, // [499]
    SimpleHintData { raw_values_start: 851, raw_values_count: 2, token_types_start: 853, token_types_count: 0 }, // [500]
    SimpleHintData { raw_values_start: 853, raw_values_count: 2, token_types_start: 855, token_types_count: 0 }, // [501]
    SimpleHintData { raw_values_start: 855, raw_values_count: 5, token_types_start: 860, token_types_count: 0 }, // [502]
    SimpleHintData { raw_values_start: 860, raw_values_count: 2, token_types_start: 862, token_types_count: 0 }, // [503]
    SimpleHintData { raw_values_start: 862, raw_values_count: 1, token_types_start: 863, token_types_count: 0 }, // [504]
    SimpleHintData { raw_values_start: 863, raw_values_count: 1, token_types_start: 864, token_types_count: 0 }, // [505]
    SimpleHintData { raw_values_start: 864, raw_values_count: 13, token_types_start: 877, token_types_count: 1 }, // [506]
    SimpleHintData { raw_values_start: 878, raw_values_count: 3, token_types_start: 881, token_types_count: 0 }, // [507]
    SimpleHintData { raw_values_start: 881, raw_values_count: 7, token_types_start: 888, token_types_count: 4 }, // [508]
    SimpleHintData { raw_values_start: 892, raw_values_count: 1, token_types_start: 893, token_types_count: 0 }, // [509]
    SimpleHintData { raw_values_start: 893, raw_values_count: 1, token_types_start: 894, token_types_count: 0 }, // [510]
    SimpleHintData { raw_values_start: 894, raw_values_count: 1, token_types_start: 895, token_types_count: 0 }, // [511]
    SimpleHintData { raw_values_start: 895, raw_values_count: 1, token_types_start: 896, token_types_count: 0 }, // [512]
    SimpleHintData { raw_values_start: 896, raw_values_count: 1, token_types_start: 897, token_types_count: 0 }, // [513]
    SimpleHintData { raw_values_start: 897, raw_values_count: 2, token_types_start: 899, token_types_count: 0 }, // [514]
    SimpleHintData { raw_values_start: 899, raw_values_count: 1, token_types_start: 900, token_types_count: 0 }, // [515]
    SimpleHintData { raw_values_start: 900, raw_values_count: 2, token_types_start: 902, token_types_count: 0 }, // [516]
    SimpleHintData { raw_values_start: 902, raw_values_count: 1, token_types_start: 903, token_types_count: 0 }, // [517]
    SimpleHintData { raw_values_start: 903, raw_values_count: 2, token_types_start: 905, token_types_count: 0 }, // [518]
    SimpleHintData { raw_values_start: 905, raw_values_count: 1, token_types_start: 906, token_types_count: 0 }, // [519]
    SimpleHintData { raw_values_start: 906, raw_values_count: 1, token_types_start: 907, token_types_count: 0 }, // [520]
    SimpleHintData { raw_values_start: 907, raw_values_count: 1, token_types_start: 908, token_types_count: 0 }, // [521]
    SimpleHintData { raw_values_start: 908, raw_values_count: 8, token_types_start: 916, token_types_count: 0 }, // [522]
    SimpleHintData { raw_values_start: 916, raw_values_count: 1, token_types_start: 917, token_types_count: 0 }, // [523]
    SimpleHintData { raw_values_start: 917, raw_values_count: 1, token_types_start: 918, token_types_count: 0 }, // [524]
    SimpleHintData { raw_values_start: 918, raw_values_count: 1, token_types_start: 919, token_types_count: 0 }, // [525]
    SimpleHintData { raw_values_start: 919, raw_values_count: 1, token_types_start: 920, token_types_count: 0 }, // [526]
    SimpleHintData { raw_values_start: 920, raw_values_count: 1, token_types_start: 921, token_types_count: 0 }, // [527]
    SimpleHintData { raw_values_start: 921, raw_values_count: 1, token_types_start: 922, token_types_count: 0 }, // [528]
    SimpleHintData { raw_values_start: 922, raw_values_count: 1, token_types_start: 923, token_types_count: 0 }, // [529]
    SimpleHintData { raw_values_start: 923, raw_values_count: 1, token_types_start: 924, token_types_count: 0 }, // [530]
    SimpleHintData { raw_values_start: 924, raw_values_count: 22, token_types_start: 946, token_types_count: 0 }, // [531]
    SimpleHintData { raw_values_start: 946, raw_values_count: 21, token_types_start: 967, token_types_count: 0 }, // [532]
    SimpleHintData { raw_values_start: 967, raw_values_count: 1, token_types_start: 968, token_types_count: 0 }, // [533]
    SimpleHintData { raw_values_start: 968, raw_values_count: 1, token_types_start: 969, token_types_count: 0 }, // [534]
    SimpleHintData { raw_values_start: 969, raw_values_count: 1, token_types_start: 970, token_types_count: 0 }, // [535]
    SimpleHintData { raw_values_start: 970, raw_values_count: 1, token_types_start: 971, token_types_count: 0 }, // [536]
    SimpleHintData { raw_values_start: 971, raw_values_count: 1, token_types_start: 972, token_types_count: 0 }, // [537]
    SimpleHintData { raw_values_start: 972, raw_values_count: 1, token_types_start: 973, token_types_count: 0 }, // [538]
    SimpleHintData { raw_values_start: 973, raw_values_count: 1, token_types_start: 974, token_types_count: 0 }, // [539]
    SimpleHintData { raw_values_start: 974, raw_values_count: 1, token_types_start: 975, token_types_count: 0 }, // [540]
    SimpleHintData { raw_values_start: 975, raw_values_count: 1, token_types_start: 976, token_types_count: 0 }, // [541]
    SimpleHintData { raw_values_start: 976, raw_values_count: 1, token_types_start: 977, token_types_count: 0 }, // [542]
    SimpleHintData { raw_values_start: 977, raw_values_count: 1, token_types_start: 978, token_types_count: 0 }, // [543]
    SimpleHintData { raw_values_start: 978, raw_values_count: 1, token_types_start: 979, token_types_count: 0 }, // [544]
    SimpleHintData { raw_values_start: 979, raw_values_count: 1, token_types_start: 980, token_types_count: 0 }, // [545]
    SimpleHintData { raw_values_start: 980, raw_values_count: 1, token_types_start: 981, token_types_count: 0 }, // [546]
    SimpleHintData { raw_values_start: 981, raw_values_count: 1, token_types_start: 982, token_types_count: 0 }, // [547]
    SimpleHintData { raw_values_start: 982, raw_values_count: 1, token_types_start: 983, token_types_count: 0 }, // [548]
    SimpleHintData { raw_values_start: 983, raw_values_count: 1, token_types_start: 984, token_types_count: 0 }, // [549]
    SimpleHintData { raw_values_start: 984, raw_values_count: 2, token_types_start: 986, token_types_count: 0 }, // [550]
    SimpleHintData { raw_values_start: 986, raw_values_count: 1, token_types_start: 987, token_types_count: 0 }, // [551]
    SimpleHintData { raw_values_start: 987, raw_values_count: 1, token_types_start: 988, token_types_count: 0 }, // [552]
    SimpleHintData { raw_values_start: 988, raw_values_count: 1, token_types_start: 989, token_types_count: 0 }, // [553]
    SimpleHintData { raw_values_start: 989, raw_values_count: 1, token_types_start: 990, token_types_count: 0 }, // [554]
    SimpleHintData { raw_values_start: 990, raw_values_count: 1, token_types_start: 991, token_types_count: 0 }, // [555]
    SimpleHintData { raw_values_start: 991, raw_values_count: 1, token_types_start: 992, token_types_count: 0 }, // [556]
    SimpleHintData { raw_values_start: 992, raw_values_count: 17, token_types_start: 1009, token_types_count: 0 }, // [557]
    SimpleHintData { raw_values_start: 1009, raw_values_count: 4, token_types_start: 1013, token_types_count: 0 }, // [558]
    SimpleHintData { raw_values_start: 1013, raw_values_count: 1, token_types_start: 1014, token_types_count: 0 }, // [559]
    SimpleHintData { raw_values_start: 1014, raw_values_count: 1, token_types_start: 1015, token_types_count: 0 }, // [560]
    SimpleHintData { raw_values_start: 1015, raw_values_count: 1, token_types_start: 1016, token_types_count: 0 }, // [561]
    SimpleHintData { raw_values_start: 1016, raw_values_count: 2, token_types_start: 1018, token_types_count: 0 }, // [562]
    SimpleHintData { raw_values_start: 1018, raw_values_count: 1, token_types_start: 1019, token_types_count: 0 }, // [563]
    SimpleHintData { raw_values_start: 1019, raw_values_count: 3, token_types_start: 1022, token_types_count: 0 }, // [564]
    SimpleHintData { raw_values_start: 1022, raw_values_count: 2, token_types_start: 1024, token_types_count: 0 }, // [565]
    SimpleHintData { raw_values_start: 1024, raw_values_count: 1, token_types_start: 1025, token_types_count: 0 }, // [566]
    SimpleHintData { raw_values_start: 1025, raw_values_count: 2, token_types_start: 1027, token_types_count: 0 }, // [567]
    SimpleHintData { raw_values_start: 1027, raw_values_count: 1, token_types_start: 1028, token_types_count: 0 }, // [568]
    SimpleHintData { raw_values_start: 1028, raw_values_count: 1, token_types_start: 1029, token_types_count: 0 }, // [569]
    SimpleHintData { raw_values_start: 1029, raw_values_count: 2, token_types_start: 1031, token_types_count: 0 }, // [570]
    SimpleHintData { raw_values_start: 1031, raw_values_count: 0, token_types_start: 1031, token_types_count: 2 }, // [571]
    SimpleHintData { raw_values_start: 1033, raw_values_count: 0, token_types_start: 1033, token_types_count: 1 }, // [572]
    SimpleHintData { raw_values_start: 1034, raw_values_count: 23, token_types_start: 1057, token_types_count: 0 }, // [573]
    SimpleHintData { raw_values_start: 1057, raw_values_count: 3, token_types_start: 1060, token_types_count: 0 }, // [574]
    SimpleHintData { raw_values_start: 1060, raw_values_count: 1, token_types_start: 1061, token_types_count: 0 }, // [575]
    SimpleHintData { raw_values_start: 1061, raw_values_count: 1, token_types_start: 1062, token_types_count: 0 }, // [576]
    SimpleHintData { raw_values_start: 1062, raw_values_count: 1, token_types_start: 1063, token_types_count: 0 }, // [577]
    SimpleHintData { raw_values_start: 1063, raw_values_count: 1, token_types_start: 1064, token_types_count: 0 }, // [578]
    SimpleHintData { raw_values_start: 1064, raw_values_count: 1, token_types_start: 1065, token_types_count: 0 }, // [579]
    SimpleHintData { raw_values_start: 1065, raw_values_count: 0, token_types_start: 1065, token_types_count: 1 }, // [580]
    SimpleHintData { raw_values_start: 1066, raw_values_count: 1, token_types_start: 1067, token_types_count: 0 }, // [581]
    SimpleHintData { raw_values_start: 1067, raw_values_count: 1, token_types_start: 1068, token_types_count: 0 }, // [582]
    SimpleHintData { raw_values_start: 1068, raw_values_count: 1, token_types_start: 1069, token_types_count: 0 }, // [583]
    SimpleHintData { raw_values_start: 1069, raw_values_count: 2, token_types_start: 1071, token_types_count: 0 }, // [584]
    SimpleHintData { raw_values_start: 1071, raw_values_count: 1, token_types_start: 1072, token_types_count: 0 }, // [585]
    SimpleHintData { raw_values_start: 1072, raw_values_count: 1, token_types_start: 1073, token_types_count: 0 }, // [586]
    SimpleHintData { raw_values_start: 1073, raw_values_count: 1, token_types_start: 1074, token_types_count: 0 }, // [587]
    SimpleHintData { raw_values_start: 1074, raw_values_count: 1, token_types_start: 1075, token_types_count: 0 }, // [588]
    SimpleHintData { raw_values_start: 1075, raw_values_count: 1, token_types_start: 1076, token_types_count: 0 }, // [589]
    SimpleHintData { raw_values_start: 1076, raw_values_count: 8, token_types_start: 1084, token_types_count: 0 }, // [590]
    SimpleHintData { raw_values_start: 1084, raw_values_count: 4, token_types_start: 1088, token_types_count: 0 }, // [591]
    SimpleHintData { raw_values_start: 1088, raw_values_count: 1, token_types_start: 1089, token_types_count: 0 }, // [592]
    SimpleHintData { raw_values_start: 1089, raw_values_count: 1, token_types_start: 1090, token_types_count: 0 }, // [593]
    SimpleHintData { raw_values_start: 1090, raw_values_count: 7, token_types_start: 1097, token_types_count: 0 }, // [594]
    SimpleHintData { raw_values_start: 1097, raw_values_count: 1, token_types_start: 1098, token_types_count: 0 }, // [595]
    SimpleHintData { raw_values_start: 1098, raw_values_count: 1, token_types_start: 1099, token_types_count: 0 }, // [596]
    SimpleHintData { raw_values_start: 1099, raw_values_count: 1, token_types_start: 1100, token_types_count: 0 }, // [597]
    SimpleHintData { raw_values_start: 1100, raw_values_count: 1, token_types_start: 1101, token_types_count: 0 }, // [598]
    SimpleHintData { raw_values_start: 1101, raw_values_count: 0, token_types_start: 1101, token_types_count: 1 }, // [599]
    SimpleHintData { raw_values_start: 1102, raw_values_count: 1, token_types_start: 1103, token_types_count: 0 }, // [600]
    SimpleHintData { raw_values_start: 1103, raw_values_count: 1, token_types_start: 1104, token_types_count: 0 }, // [601]
    SimpleHintData { raw_values_start: 1104, raw_values_count: 1, token_types_start: 1105, token_types_count: 0 }, // [602]
    SimpleHintData { raw_values_start: 1105, raw_values_count: 1, token_types_start: 1106, token_types_count: 0 }, // [603]
    SimpleHintData { raw_values_start: 1106, raw_values_count: 1, token_types_start: 1107, token_types_count: 0 }, // [604]
    SimpleHintData { raw_values_start: 1107, raw_values_count: 1, token_types_start: 1108, token_types_count: 0 }, // [605]
    SimpleHintData { raw_values_start: 1108, raw_values_count: 1, token_types_start: 1109, token_types_count: 0 }, // [606]
    SimpleHintData { raw_values_start: 1109, raw_values_count: 1, token_types_start: 1110, token_types_count: 0 }, // [607]
    SimpleHintData { raw_values_start: 1110, raw_values_count: 1, token_types_start: 1111, token_types_count: 0 }, // [608]
    SimpleHintData { raw_values_start: 1111, raw_values_count: 1, token_types_start: 1112, token_types_count: 0 }, // [609]
    SimpleHintData { raw_values_start: 1112, raw_values_count: 1, token_types_start: 1113, token_types_count: 0 }, // [610]
    SimpleHintData { raw_values_start: 1113, raw_values_count: 1, token_types_start: 1114, token_types_count: 0 }, // [611]
    SimpleHintData { raw_values_start: 1114, raw_values_count: 1, token_types_start: 1115, token_types_count: 0 }, // [612]
    SimpleHintData { raw_values_start: 1115, raw_values_count: 1, token_types_start: 1116, token_types_count: 0 }, // [613]
    SimpleHintData { raw_values_start: 1116, raw_values_count: 1, token_types_start: 1117, token_types_count: 0 }, // [614]
    SimpleHintData { raw_values_start: 1117, raw_values_count: 1, token_types_start: 1118, token_types_count: 0 }, // [615]
    SimpleHintData { raw_values_start: 1118, raw_values_count: 1, token_types_start: 1119, token_types_count: 0 }, // [616]
    SimpleHintData { raw_values_start: 1119, raw_values_count: 1, token_types_start: 1120, token_types_count: 0 }, // [617]
    SimpleHintData { raw_values_start: 1120, raw_values_count: 1, token_types_start: 1121, token_types_count: 0 }, // [618]
    SimpleHintData { raw_values_start: 1121, raw_values_count: 1, token_types_start: 1122, token_types_count: 0 }, // [619]
    SimpleHintData { raw_values_start: 1122, raw_values_count: 1, token_types_start: 1123, token_types_count: 0 }, // [620]
    SimpleHintData { raw_values_start: 1123, raw_values_count: 1, token_types_start: 1124, token_types_count: 0 }, // [621]
    SimpleHintData { raw_values_start: 1124, raw_values_count: 1, token_types_start: 1125, token_types_count: 0 }, // [622]
    SimpleHintData { raw_values_start: 1125, raw_values_count: 1, token_types_start: 1126, token_types_count: 0 }, // [623]
    SimpleHintData { raw_values_start: 1126, raw_values_count: 1, token_types_start: 1127, token_types_count: 0 }, // [624]
    SimpleHintData { raw_values_start: 1127, raw_values_count: 1, token_types_start: 1128, token_types_count: 0 }, // [625]
    SimpleHintData { raw_values_start: 1128, raw_values_count: 1, token_types_start: 1129, token_types_count: 0 }, // [626]
    SimpleHintData { raw_values_start: 1129, raw_values_count: 1, token_types_start: 1130, token_types_count: 0 }, // [627]
    SimpleHintData { raw_values_start: 1130, raw_values_count: 1, token_types_start: 1131, token_types_count: 0 }, // [628]
    SimpleHintData { raw_values_start: 1131, raw_values_count: 1, token_types_start: 1132, token_types_count: 0 }, // [629]
    SimpleHintData { raw_values_start: 1132, raw_values_count: 1, token_types_start: 1133, token_types_count: 0 }, // [630]
    SimpleHintData { raw_values_start: 1133, raw_values_count: 1, token_types_start: 1134, token_types_count: 0 }, // [631]
    SimpleHintData { raw_values_start: 1134, raw_values_count: 1, token_types_start: 1135, token_types_count: 0 }, // [632]
    SimpleHintData { raw_values_start: 1135, raw_values_count: 1, token_types_start: 1136, token_types_count: 0 }, // [633]
    SimpleHintData { raw_values_start: 1136, raw_values_count: 1, token_types_start: 1137, token_types_count: 0 }, // [634]
    SimpleHintData { raw_values_start: 1137, raw_values_count: 1, token_types_start: 1138, token_types_count: 0 }, // [635]
    SimpleHintData { raw_values_start: 1138, raw_values_count: 1, token_types_start: 1139, token_types_count: 0 }, // [636]
    SimpleHintData { raw_values_start: 1139, raw_values_count: 1, token_types_start: 1140, token_types_count: 0 }, // [637]
    SimpleHintData { raw_values_start: 1140, raw_values_count: 1, token_types_start: 1141, token_types_count: 0 }, // [638]
    SimpleHintData { raw_values_start: 1141, raw_values_count: 1, token_types_start: 1142, token_types_count: 0 }, // [639]
    SimpleHintData { raw_values_start: 1142, raw_values_count: 1, token_types_start: 1143, token_types_count: 0 }, // [640]
    SimpleHintData { raw_values_start: 1143, raw_values_count: 1, token_types_start: 1144, token_types_count: 0 }, // [641]
    SimpleHintData { raw_values_start: 1144, raw_values_count: 1, token_types_start: 1145, token_types_count: 0 }, // [642]
    SimpleHintData { raw_values_start: 1145, raw_values_count: 1, token_types_start: 1146, token_types_count: 0 }, // [643]
    SimpleHintData { raw_values_start: 1146, raw_values_count: 1, token_types_start: 1147, token_types_count: 0 }, // [644]
    SimpleHintData { raw_values_start: 1147, raw_values_count: 1, token_types_start: 1148, token_types_count: 0 }, // [645]
    SimpleHintData { raw_values_start: 1148, raw_values_count: 1, token_types_start: 1149, token_types_count: 0 }, // [646]
    SimpleHintData { raw_values_start: 1149, raw_values_count: 1, token_types_start: 1150, token_types_count: 0 }, // [647]
    SimpleHintData { raw_values_start: 1150, raw_values_count: 1, token_types_start: 1151, token_types_count: 0 }, // [648]
    SimpleHintData { raw_values_start: 1151, raw_values_count: 1, token_types_start: 1152, token_types_count: 0 }, // [649]
    SimpleHintData { raw_values_start: 1152, raw_values_count: 1, token_types_start: 1153, token_types_count: 0 }, // [650]
    SimpleHintData { raw_values_start: 1153, raw_values_count: 1, token_types_start: 1154, token_types_count: 4 }, // [651]
    SimpleHintData { raw_values_start: 1158, raw_values_count: 1, token_types_start: 1159, token_types_count: 0 }, // [652]
    SimpleHintData { raw_values_start: 1159, raw_values_count: 1, token_types_start: 1160, token_types_count: 0 }, // [653]
    SimpleHintData { raw_values_start: 1160, raw_values_count: 1, token_types_start: 1161, token_types_count: 0 }, // [654]
    SimpleHintData { raw_values_start: 1161, raw_values_count: 1, token_types_start: 1162, token_types_count: 0 }, // [655]
    SimpleHintData { raw_values_start: 1162, raw_values_count: 1, token_types_start: 1163, token_types_count: 0 }, // [656]
    SimpleHintData { raw_values_start: 1163, raw_values_count: 1, token_types_start: 1164, token_types_count: 0 }, // [657]
    SimpleHintData { raw_values_start: 1164, raw_values_count: 1, token_types_start: 1165, token_types_count: 0 }, // [658]
    SimpleHintData { raw_values_start: 1165, raw_values_count: 1, token_types_start: 1166, token_types_count: 0 }, // [659]
    SimpleHintData { raw_values_start: 1166, raw_values_count: 1, token_types_start: 1167, token_types_count: 0 }, // [660]
    SimpleHintData { raw_values_start: 1167, raw_values_count: 2, token_types_start: 1169, token_types_count: 0 }, // [661]
    SimpleHintData { raw_values_start: 1169, raw_values_count: 2, token_types_start: 1171, token_types_count: 0 }, // [662]
    SimpleHintData { raw_values_start: 1171, raw_values_count: 1, token_types_start: 1172, token_types_count: 0 }, // [663]
    SimpleHintData { raw_values_start: 1172, raw_values_count: 1, token_types_start: 1173, token_types_count: 0 }, // [664]
    SimpleHintData { raw_values_start: 1173, raw_values_count: 1, token_types_start: 1174, token_types_count: 0 }, // [665]
    SimpleHintData { raw_values_start: 1174, raw_values_count: 3, token_types_start: 1177, token_types_count: 0 }, // [666]
    SimpleHintData { raw_values_start: 1177, raw_values_count: 1, token_types_start: 1178, token_types_count: 0 }, // [667]
    SimpleHintData { raw_values_start: 1178, raw_values_count: 1, token_types_start: 1179, token_types_count: 0 }, // [668]
    SimpleHintData { raw_values_start: 1179, raw_values_count: 1, token_types_start: 1180, token_types_count: 0 }, // [669]
    SimpleHintData { raw_values_start: 1180, raw_values_count: 1, token_types_start: 1181, token_types_count: 0 }, // [670]
    SimpleHintData { raw_values_start: 1181, raw_values_count: 1, token_types_start: 1182, token_types_count: 0 }, // [671]
    SimpleHintData { raw_values_start: 1182, raw_values_count: 1, token_types_start: 1183, token_types_count: 0 }, // [672]
    SimpleHintData { raw_values_start: 1183, raw_values_count: 1, token_types_start: 1184, token_types_count: 0 }, // [673]
    SimpleHintData { raw_values_start: 1184, raw_values_count: 1, token_types_start: 1185, token_types_count: 0 }, // [674]
    SimpleHintData { raw_values_start: 1185, raw_values_count: 1, token_types_start: 1186, token_types_count: 0 }, // [675]
    SimpleHintData { raw_values_start: 1186, raw_values_count: 1, token_types_start: 1187, token_types_count: 0 }, // [676]
    SimpleHintData { raw_values_start: 1187, raw_values_count: 1, token_types_start: 1188, token_types_count: 0 }, // [677]
    SimpleHintData { raw_values_start: 1188, raw_values_count: 1, token_types_start: 1189, token_types_count: 0 }, // [678]
    SimpleHintData { raw_values_start: 1189, raw_values_count: 1, token_types_start: 1190, token_types_count: 0 }, // [679]
    SimpleHintData { raw_values_start: 1190, raw_values_count: 1, token_types_start: 1191, token_types_count: 0 }, // [680]
    SimpleHintData { raw_values_start: 1191, raw_values_count: 1, token_types_start: 1192, token_types_count: 0 }, // [681]
    SimpleHintData { raw_values_start: 1192, raw_values_count: 1, token_types_start: 1193, token_types_count: 0 }, // [682]
    SimpleHintData { raw_values_start: 1193, raw_values_count: 1, token_types_start: 1194, token_types_count: 0 }, // [683]
    SimpleHintData { raw_values_start: 1194, raw_values_count: 1, token_types_start: 1195, token_types_count: 0 }, // [684]
    SimpleHintData { raw_values_start: 1195, raw_values_count: 1, token_types_start: 1196, token_types_count: 0 }, // [685]
    SimpleHintData { raw_values_start: 1196, raw_values_count: 5, token_types_start: 1201, token_types_count: 0 }, // [686]
    SimpleHintData { raw_values_start: 1201, raw_values_count: 1, token_types_start: 1202, token_types_count: 0 }, // [687]
    SimpleHintData { raw_values_start: 1202, raw_values_count: 1, token_types_start: 1203, token_types_count: 0 }, // [688]
    SimpleHintData { raw_values_start: 1203, raw_values_count: 2, token_types_start: 1205, token_types_count: 0 }, // [689]
    SimpleHintData { raw_values_start: 1205, raw_values_count: 1, token_types_start: 1206, token_types_count: 0 }, // [690]
    SimpleHintData { raw_values_start: 1206, raw_values_count: 1, token_types_start: 1207, token_types_count: 0 }, // [691]
    SimpleHintData { raw_values_start: 1207, raw_values_count: 1, token_types_start: 1208, token_types_count: 0 }, // [692]
    SimpleHintData { raw_values_start: 1208, raw_values_count: 1, token_types_start: 1209, token_types_count: 0 }, // [693]
    SimpleHintData { raw_values_start: 1209, raw_values_count: 1, token_types_start: 1210, token_types_count: 0 }, // [694]
    SimpleHintData { raw_values_start: 1210, raw_values_count: 1, token_types_start: 1211, token_types_count: 0 }, // [695]
    SimpleHintData { raw_values_start: 1211, raw_values_count: 1, token_types_start: 1212, token_types_count: 0 }, // [696]
    SimpleHintData { raw_values_start: 1212, raw_values_count: 1, token_types_start: 1213, token_types_count: 0 }, // [697]
    SimpleHintData { raw_values_start: 1213, raw_values_count: 1, token_types_start: 1214, token_types_count: 0 }, // [698]
    SimpleHintData { raw_values_start: 1214, raw_values_count: 1, token_types_start: 1215, token_types_count: 0 }, // [699]
    SimpleHintData { raw_values_start: 1215, raw_values_count: 1, token_types_start: 1216, token_types_count: 0 }, // [700]
    SimpleHintData { raw_values_start: 1216, raw_values_count: 1, token_types_start: 1217, token_types_count: 0 }, // [701]
    SimpleHintData { raw_values_start: 1217, raw_values_count: 1, token_types_start: 1218, token_types_count: 0 }, // [702]
    SimpleHintData { raw_values_start: 1218, raw_values_count: 1, token_types_start: 1219, token_types_count: 0 }, // [703]
    SimpleHintData { raw_values_start: 1219, raw_values_count: 1, token_types_start: 1220, token_types_count: 0 }, // [704]
    SimpleHintData { raw_values_start: 1220, raw_values_count: 1, token_types_start: 1221, token_types_count: 0 }, // [705]
    SimpleHintData { raw_values_start: 1221, raw_values_count: 1, token_types_start: 1222, token_types_count: 0 }, // [706]
    SimpleHintData { raw_values_start: 1222, raw_values_count: 1, token_types_start: 1223, token_types_count: 0 }, // [707]
    SimpleHintData { raw_values_start: 1223, raw_values_count: 1, token_types_start: 1224, token_types_count: 0 }, // [708]
    SimpleHintData { raw_values_start: 1224, raw_values_count: 1, token_types_start: 1225, token_types_count: 0 }, // [709]
    SimpleHintData { raw_values_start: 1225, raw_values_count: 1, token_types_start: 1226, token_types_count: 0 }, // [710]
    SimpleHintData { raw_values_start: 1226, raw_values_count: 1, token_types_start: 1227, token_types_count: 0 }, // [711]
    SimpleHintData { raw_values_start: 1227, raw_values_count: 1, token_types_start: 1228, token_types_count: 0 }, // [712]
    SimpleHintData { raw_values_start: 1228, raw_values_count: 1, token_types_start: 1229, token_types_count: 0 }, // [713]
    SimpleHintData { raw_values_start: 1229, raw_values_count: 1, token_types_start: 1230, token_types_count: 0 }, // [714]
    SimpleHintData { raw_values_start: 1230, raw_values_count: 1, token_types_start: 1231, token_types_count: 0 }, // [715]
    SimpleHintData { raw_values_start: 1231, raw_values_count: 1, token_types_start: 1232, token_types_count: 0 }, // [716]
    SimpleHintData { raw_values_start: 1232, raw_values_count: 1, token_types_start: 1233, token_types_count: 0 }, // [717]
    SimpleHintData { raw_values_start: 1233, raw_values_count: 1, token_types_start: 1234, token_types_count: 0 }, // [718]
    SimpleHintData { raw_values_start: 1234, raw_values_count: 1, token_types_start: 1235, token_types_count: 0 }, // [719]
    SimpleHintData { raw_values_start: 1235, raw_values_count: 1, token_types_start: 1236, token_types_count: 0 }, // [720]
    SimpleHintData { raw_values_start: 1236, raw_values_count: 1, token_types_start: 1237, token_types_count: 0 }, // [721]
    SimpleHintData { raw_values_start: 1237, raw_values_count: 1, token_types_start: 1238, token_types_count: 0 }, // [722]
    SimpleHintData { raw_values_start: 1238, raw_values_count: 1, token_types_start: 1239, token_types_count: 0 }, // [723]
    SimpleHintData { raw_values_start: 1239, raw_values_count: 1, token_types_start: 1240, token_types_count: 0 }, // [724]
    SimpleHintData { raw_values_start: 1240, raw_values_count: 2, token_types_start: 1242, token_types_count: 0 }, // [725]
    SimpleHintData { raw_values_start: 1242, raw_values_count: 1, token_types_start: 1243, token_types_count: 0 }, // [726]
    SimpleHintData { raw_values_start: 1243, raw_values_count: 1, token_types_start: 1244, token_types_count: 0 }, // [727]
    SimpleHintData { raw_values_start: 1244, raw_values_count: 1, token_types_start: 1245, token_types_count: 0 }, // [728]
    SimpleHintData { raw_values_start: 1245, raw_values_count: 1, token_types_start: 1246, token_types_count: 0 }, // [729]
    SimpleHintData { raw_values_start: 1246, raw_values_count: 1, token_types_start: 1247, token_types_count: 0 }, // [730]
    SimpleHintData { raw_values_start: 1247, raw_values_count: 1, token_types_start: 1248, token_types_count: 0 }, // [731]
    SimpleHintData { raw_values_start: 1248, raw_values_count: 1, token_types_start: 1249, token_types_count: 0 }, // [732]
    SimpleHintData { raw_values_start: 1249, raw_values_count: 1, token_types_start: 1250, token_types_count: 0 }, // [733]
    SimpleHintData { raw_values_start: 1250, raw_values_count: 1, token_types_start: 1251, token_types_count: 0 }, // [734]
    SimpleHintData { raw_values_start: 1251, raw_values_count: 1, token_types_start: 1252, token_types_count: 0 }, // [735]
    SimpleHintData { raw_values_start: 1252, raw_values_count: 1, token_types_start: 1253, token_types_count: 0 }, // [736]
    SimpleHintData { raw_values_start: 1253, raw_values_count: 1, token_types_start: 1254, token_types_count: 0 }, // [737]
    SimpleHintData { raw_values_start: 1254, raw_values_count: 1, token_types_start: 1255, token_types_count: 0 }, // [738]
    SimpleHintData { raw_values_start: 1255, raw_values_count: 1, token_types_start: 1256, token_types_count: 0 }, // [739]
    SimpleHintData { raw_values_start: 1256, raw_values_count: 1, token_types_start: 1257, token_types_count: 0 }, // [740]
    SimpleHintData { raw_values_start: 1257, raw_values_count: 1, token_types_start: 1258, token_types_count: 0 }, // [741]
    SimpleHintData { raw_values_start: 1258, raw_values_count: 1, token_types_start: 1259, token_types_count: 0 }, // [742]
    SimpleHintData { raw_values_start: 1259, raw_values_count: 1, token_types_start: 1260, token_types_count: 0 }, // [743]
    SimpleHintData { raw_values_start: 1260, raw_values_count: 1, token_types_start: 1261, token_types_count: 0 }, // [744]
    SimpleHintData { raw_values_start: 1261, raw_values_count: 1, token_types_start: 1262, token_types_count: 0 }, // [745]
    SimpleHintData { raw_values_start: 1262, raw_values_count: 1, token_types_start: 1263, token_types_count: 0 }, // [746]
    SimpleHintData { raw_values_start: 1263, raw_values_count: 1, token_types_start: 1264, token_types_count: 0 }, // [747]
    SimpleHintData { raw_values_start: 1264, raw_values_count: 1, token_types_start: 1265, token_types_count: 0 }, // [748]
    SimpleHintData { raw_values_start: 1265, raw_values_count: 1, token_types_start: 1266, token_types_count: 0 }, // [749]
    SimpleHintData { raw_values_start: 1266, raw_values_count: 1, token_types_start: 1267, token_types_count: 0 }, // [750]
    SimpleHintData { raw_values_start: 1267, raw_values_count: 1, token_types_start: 1268, token_types_count: 0 }, // [751]
    SimpleHintData { raw_values_start: 1268, raw_values_count: 3, token_types_start: 1271, token_types_count: 0 }, // [752]
    SimpleHintData { raw_values_start: 1271, raw_values_count: 1, token_types_start: 1272, token_types_count: 0 }, // [753]
    SimpleHintData { raw_values_start: 1272, raw_values_count: 1, token_types_start: 1273, token_types_count: 0 }, // [754]
    SimpleHintData { raw_values_start: 1273, raw_values_count: 1, token_types_start: 1274, token_types_count: 0 }, // [755]
    SimpleHintData { raw_values_start: 1274, raw_values_count: 1, token_types_start: 1275, token_types_count: 0 }, // [756]
    SimpleHintData { raw_values_start: 1275, raw_values_count: 1, token_types_start: 1276, token_types_count: 0 }, // [757]
    SimpleHintData { raw_values_start: 1276, raw_values_count: 1, token_types_start: 1277, token_types_count: 0 }, // [758]
    SimpleHintData { raw_values_start: 1277, raw_values_count: 1, token_types_start: 1278, token_types_count: 0 }, // [759]
    SimpleHintData { raw_values_start: 1278, raw_values_count: 1, token_types_start: 1279, token_types_count: 0 }, // [760]
    SimpleHintData { raw_values_start: 1279, raw_values_count: 1, token_types_start: 1280, token_types_count: 0 }, // [761]
    SimpleHintData { raw_values_start: 1280, raw_values_count: 1, token_types_start: 1281, token_types_count: 0 }, // [762]
    SimpleHintData { raw_values_start: 1281, raw_values_count: 1, token_types_start: 1282, token_types_count: 0 }, // [763]
    SimpleHintData { raw_values_start: 1282, raw_values_count: 1, token_types_start: 1283, token_types_count: 0 }, // [764]
    SimpleHintData { raw_values_start: 1283, raw_values_count: 1, token_types_start: 1284, token_types_count: 0 }, // [765]
    SimpleHintData { raw_values_start: 1284, raw_values_count: 1, token_types_start: 1285, token_types_count: 0 }, // [766]
    SimpleHintData { raw_values_start: 1285, raw_values_count: 1, token_types_start: 1286, token_types_count: 0 }, // [767]
    SimpleHintData { raw_values_start: 1286, raw_values_count: 1, token_types_start: 1287, token_types_count: 0 }, // [768]
    SimpleHintData { raw_values_start: 1287, raw_values_count: 1, token_types_start: 1288, token_types_count: 0 }, // [769]
    SimpleHintData { raw_values_start: 1288, raw_values_count: 1, token_types_start: 1289, token_types_count: 0 }, // [770]
    SimpleHintData { raw_values_start: 1289, raw_values_count: 1, token_types_start: 1290, token_types_count: 0 }, // [771]
    SimpleHintData { raw_values_start: 1290, raw_values_count: 1, token_types_start: 1291, token_types_count: 0 }, // [772]
    SimpleHintData { raw_values_start: 1291, raw_values_count: 1, token_types_start: 1292, token_types_count: 0 }, // [773]
    SimpleHintData { raw_values_start: 1292, raw_values_count: 1, token_types_start: 1293, token_types_count: 0 }, // [774]
    SimpleHintData { raw_values_start: 1293, raw_values_count: 1, token_types_start: 1294, token_types_count: 0 }, // [775]
    SimpleHintData { raw_values_start: 1294, raw_values_count: 4, token_types_start: 1298, token_types_count: 0 }, // [776]
    SimpleHintData { raw_values_start: 1298, raw_values_count: 1, token_types_start: 1299, token_types_count: 0 }, // [777]
    SimpleHintData { raw_values_start: 1299, raw_values_count: 1, token_types_start: 1300, token_types_count: 0 }, // [778]
    SimpleHintData { raw_values_start: 1300, raw_values_count: 1, token_types_start: 1301, token_types_count: 0 }, // [779]
    SimpleHintData { raw_values_start: 1301, raw_values_count: 1, token_types_start: 1302, token_types_count: 0 }, // [780]
    SimpleHintData { raw_values_start: 1302, raw_values_count: 1, token_types_start: 1303, token_types_count: 0 }, // [781]
    SimpleHintData { raw_values_start: 1303, raw_values_count: 1, token_types_start: 1304, token_types_count: 0 }, // [782]
    SimpleHintData { raw_values_start: 1304, raw_values_count: 1, token_types_start: 1305, token_types_count: 0 }, // [783]
    SimpleHintData { raw_values_start: 1305, raw_values_count: 1, token_types_start: 1306, token_types_count: 0 }, // [784]
    SimpleHintData { raw_values_start: 1306, raw_values_count: 1, token_types_start: 1307, token_types_count: 0 }, // [785]
    SimpleHintData { raw_values_start: 1307, raw_values_count: 1, token_types_start: 1308, token_types_count: 0 }, // [786]
    SimpleHintData { raw_values_start: 1308, raw_values_count: 1, token_types_start: 1309, token_types_count: 0 }, // [787]
    SimpleHintData { raw_values_start: 1309, raw_values_count: 1, token_types_start: 1310, token_types_count: 0 }, // [788]
    SimpleHintData { raw_values_start: 1310, raw_values_count: 1, token_types_start: 1311, token_types_count: 0 }, // [789]
    SimpleHintData { raw_values_start: 1311, raw_values_count: 1, token_types_start: 1312, token_types_count: 0 }, // [790]
    SimpleHintData { raw_values_start: 1312, raw_values_count: 1, token_types_start: 1313, token_types_count: 0 }, // [791]
    SimpleHintData { raw_values_start: 1313, raw_values_count: 1, token_types_start: 1314, token_types_count: 0 }, // [792]
    SimpleHintData { raw_values_start: 1314, raw_values_count: 1, token_types_start: 1315, token_types_count: 0 }, // [793]
    SimpleHintData { raw_values_start: 1315, raw_values_count: 1, token_types_start: 1316, token_types_count: 0 }, // [794]
    SimpleHintData { raw_values_start: 1316, raw_values_count: 1, token_types_start: 1317, token_types_count: 0 }, // [795]
    SimpleHintData { raw_values_start: 1317, raw_values_count: 1, token_types_start: 1318, token_types_count: 0 }, // [796]
    SimpleHintData { raw_values_start: 1318, raw_values_count: 1, token_types_start: 1319, token_types_count: 0 }, // [797]
    SimpleHintData { raw_values_start: 1319, raw_values_count: 1, token_types_start: 1320, token_types_count: 0 }, // [798]
    SimpleHintData { raw_values_start: 1320, raw_values_count: 1, token_types_start: 1321, token_types_count: 0 }, // [799]
    SimpleHintData { raw_values_start: 1321, raw_values_count: 1, token_types_start: 1322, token_types_count: 0 }, // [800]
    SimpleHintData { raw_values_start: 1322, raw_values_count: 1, token_types_start: 1323, token_types_count: 0 }, // [801]
    SimpleHintData { raw_values_start: 1323, raw_values_count: 1, token_types_start: 1324, token_types_count: 0 }, // [802]
    SimpleHintData { raw_values_start: 1324, raw_values_count: 1, token_types_start: 1325, token_types_count: 0 }, // [803]
    SimpleHintData { raw_values_start: 1325, raw_values_count: 1, token_types_start: 1326, token_types_count: 0 }, // [804]
    SimpleHintData { raw_values_start: 1326, raw_values_count: 1, token_types_start: 1327, token_types_count: 0 }, // [805]
    SimpleHintData { raw_values_start: 1327, raw_values_count: 1, token_types_start: 1328, token_types_count: 0 }, // [806]
    SimpleHintData { raw_values_start: 1328, raw_values_count: 1, token_types_start: 1329, token_types_count: 0 }, // [807]
    SimpleHintData { raw_values_start: 1329, raw_values_count: 1, token_types_start: 1330, token_types_count: 0 }, // [808]
    SimpleHintData { raw_values_start: 1330, raw_values_count: 1, token_types_start: 1331, token_types_count: 0 }, // [809]
    SimpleHintData { raw_values_start: 1331, raw_values_count: 1, token_types_start: 1332, token_types_count: 0 }, // [810]
    SimpleHintData { raw_values_start: 1332, raw_values_count: 1, token_types_start: 1333, token_types_count: 0 }, // [811]
    SimpleHintData { raw_values_start: 1333, raw_values_count: 1, token_types_start: 1334, token_types_count: 0 }, // [812]
    SimpleHintData { raw_values_start: 1334, raw_values_count: 2, token_types_start: 1336, token_types_count: 0 }, // [813]
    SimpleHintData { raw_values_start: 1336, raw_values_count: 8, token_types_start: 1344, token_types_count: 0 }, // [814]
    SimpleHintData { raw_values_start: 1344, raw_values_count: 1, token_types_start: 1345, token_types_count: 0 }, // [815]
    SimpleHintData { raw_values_start: 1345, raw_values_count: 1, token_types_start: 1346, token_types_count: 0 }, // [816]
    SimpleHintData { raw_values_start: 1346, raw_values_count: 1, token_types_start: 1347, token_types_count: 0 }, // [817]
    SimpleHintData { raw_values_start: 1347, raw_values_count: 1, token_types_start: 1348, token_types_count: 0 }, // [818]
    SimpleHintData { raw_values_start: 1348, raw_values_count: 1, token_types_start: 1349, token_types_count: 0 }, // [819]
    SimpleHintData { raw_values_start: 1349, raw_values_count: 1, token_types_start: 1350, token_types_count: 0 }, // [820]
    SimpleHintData { raw_values_start: 1350, raw_values_count: 1, token_types_start: 1351, token_types_count: 0 }, // [821]
    SimpleHintData { raw_values_start: 1351, raw_values_count: 1, token_types_start: 1352, token_types_count: 0 }, // [822]
    SimpleHintData { raw_values_start: 1352, raw_values_count: 1, token_types_start: 1353, token_types_count: 0 }, // [823]
    SimpleHintData { raw_values_start: 1353, raw_values_count: 1, token_types_start: 1354, token_types_count: 0 }, // [824]
    SimpleHintData { raw_values_start: 1354, raw_values_count: 1, token_types_start: 1355, token_types_count: 0 }, // [825]
    SimpleHintData { raw_values_start: 1355, raw_values_count: 1, token_types_start: 1356, token_types_count: 0 }, // [826]
    SimpleHintData { raw_values_start: 1356, raw_values_count: 1, token_types_start: 1357, token_types_count: 0 }, // [827]
    SimpleHintData { raw_values_start: 1357, raw_values_count: 1, token_types_start: 1358, token_types_count: 0 }, // [828]
    SimpleHintData { raw_values_start: 1358, raw_values_count: 1, token_types_start: 1359, token_types_count: 0 }, // [829]
    SimpleHintData { raw_values_start: 1359, raw_values_count: 1, token_types_start: 1360, token_types_count: 0 }, // [830]
    SimpleHintData { raw_values_start: 1360, raw_values_count: 1, token_types_start: 1361, token_types_count: 0 }, // [831]
    SimpleHintData { raw_values_start: 1361, raw_values_count: 1, token_types_start: 1362, token_types_count: 0 }, // [832]
    SimpleHintData { raw_values_start: 1362, raw_values_count: 1, token_types_start: 1363, token_types_count: 0 }, // [833]
    SimpleHintData { raw_values_start: 1363, raw_values_count: 1, token_types_start: 1364, token_types_count: 0 }, // [834]
    SimpleHintData { raw_values_start: 1364, raw_values_count: 1, token_types_start: 1365, token_types_count: 0 }, // [835]
    SimpleHintData { raw_values_start: 1365, raw_values_count: 1, token_types_start: 1366, token_types_count: 0 }, // [836]
    SimpleHintData { raw_values_start: 1366, raw_values_count: 1, token_types_start: 1367, token_types_count: 0 }, // [837]
    SimpleHintData { raw_values_start: 1367, raw_values_count: 1, token_types_start: 1368, token_types_count: 0 }, // [838]
    SimpleHintData { raw_values_start: 1368, raw_values_count: 1, token_types_start: 1369, token_types_count: 0 }, // [839]
    SimpleHintData { raw_values_start: 1369, raw_values_count: 1, token_types_start: 1370, token_types_count: 0 }, // [840]
    SimpleHintData { raw_values_start: 1370, raw_values_count: 1, token_types_start: 1371, token_types_count: 3 }, // [841]
    SimpleHintData { raw_values_start: 1374, raw_values_count: 1, token_types_start: 1375, token_types_count: 0 }, // [842]
    SimpleHintData { raw_values_start: 1375, raw_values_count: 1, token_types_start: 1376, token_types_count: 0 }, // [843]
    SimpleHintData { raw_values_start: 1376, raw_values_count: 1, token_types_start: 1377, token_types_count: 0 }, // [844]
    SimpleHintData { raw_values_start: 1377, raw_values_count: 1, token_types_start: 1378, token_types_count: 0 }, // [845]
    SimpleHintData { raw_values_start: 1378, raw_values_count: 1, token_types_start: 1379, token_types_count: 0 }, // [846]
    SimpleHintData { raw_values_start: 1379, raw_values_count: 1, token_types_start: 1380, token_types_count: 0 }, // [847]
    SimpleHintData { raw_values_start: 1380, raw_values_count: 1, token_types_start: 1381, token_types_count: 0 }, // [848]
    SimpleHintData { raw_values_start: 1381, raw_values_count: 1, token_types_start: 1382, token_types_count: 0 }, // [849]
    SimpleHintData { raw_values_start: 1382, raw_values_count: 1, token_types_start: 1383, token_types_count: 0 }, // [850]
    SimpleHintData { raw_values_start: 1383, raw_values_count: 1, token_types_start: 1384, token_types_count: 0 }, // [851]
    SimpleHintData { raw_values_start: 1384, raw_values_count: 1, token_types_start: 1385, token_types_count: 0 }, // [852]
    SimpleHintData { raw_values_start: 1385, raw_values_count: 1, token_types_start: 1386, token_types_count: 0 }, // [853]
    SimpleHintData { raw_values_start: 1386, raw_values_count: 1, token_types_start: 1387, token_types_count: 0 }, // [854]
    SimpleHintData { raw_values_start: 1387, raw_values_count: 1, token_types_start: 1388, token_types_count: 0 }, // [855]
    SimpleHintData { raw_values_start: 1388, raw_values_count: 1, token_types_start: 1389, token_types_count: 0 }, // [856]
    SimpleHintData { raw_values_start: 1389, raw_values_count: 1, token_types_start: 1390, token_types_count: 0 }, // [857]
    SimpleHintData { raw_values_start: 1390, raw_values_count: 1, token_types_start: 1391, token_types_count: 0 }, // [858]
    SimpleHintData { raw_values_start: 1391, raw_values_count: 1, token_types_start: 1392, token_types_count: 0 }, // [859]
    SimpleHintData { raw_values_start: 1392, raw_values_count: 1, token_types_start: 1393, token_types_count: 0 }, // [860]
    SimpleHintData { raw_values_start: 1393, raw_values_count: 1, token_types_start: 1394, token_types_count: 0 }, // [861]
    SimpleHintData { raw_values_start: 1394, raw_values_count: 1, token_types_start: 1395, token_types_count: 0 }, // [862]
    SimpleHintData { raw_values_start: 1395, raw_values_count: 1, token_types_start: 1396, token_types_count: 0 }, // [863]
    SimpleHintData { raw_values_start: 1396, raw_values_count: 3, token_types_start: 1399, token_types_count: 0 }, // [864]
    SimpleHintData { raw_values_start: 1399, raw_values_count: 1, token_types_start: 1400, token_types_count: 0 }, // [865]
    SimpleHintData { raw_values_start: 1400, raw_values_count: 1, token_types_start: 1401, token_types_count: 0 }, // [866]
    SimpleHintData { raw_values_start: 1401, raw_values_count: 1, token_types_start: 1402, token_types_count: 0 }, // [867]
    SimpleHintData { raw_values_start: 1402, raw_values_count: 1, token_types_start: 1403, token_types_count: 0 }, // [868]
    SimpleHintData { raw_values_start: 1403, raw_values_count: 1, token_types_start: 1404, token_types_count: 0 }, // [869]
    SimpleHintData { raw_values_start: 1404, raw_values_count: 1, token_types_start: 1405, token_types_count: 0 }, // [870]
    SimpleHintData { raw_values_start: 1405, raw_values_count: 1, token_types_start: 1406, token_types_count: 0 }, // [871]
    SimpleHintData { raw_values_start: 1406, raw_values_count: 1, token_types_start: 1407, token_types_count: 0 }, // [872]
    SimpleHintData { raw_values_start: 1407, raw_values_count: 3, token_types_start: 1410, token_types_count: 0 }, // [873]
    SimpleHintData { raw_values_start: 1410, raw_values_count: 1, token_types_start: 1411, token_types_count: 0 }, // [874]
    SimpleHintData { raw_values_start: 1411, raw_values_count: 1, token_types_start: 1412, token_types_count: 0 }, // [875]
    SimpleHintData { raw_values_start: 1412, raw_values_count: 1, token_types_start: 1413, token_types_count: 0 }, // [876]
    SimpleHintData { raw_values_start: 1413, raw_values_count: 1, token_types_start: 1414, token_types_count: 0 }, // [877]
    SimpleHintData { raw_values_start: 1414, raw_values_count: 1, token_types_start: 1415, token_types_count: 0 }, // [878]
    SimpleHintData { raw_values_start: 1415, raw_values_count: 1, token_types_start: 1416, token_types_count: 0 }, // [879]
    SimpleHintData { raw_values_start: 1416, raw_values_count: 1, token_types_start: 1417, token_types_count: 0 }, // [880]
    SimpleHintData { raw_values_start: 1417, raw_values_count: 1, token_types_start: 1418, token_types_count: 0 }, // [881]
    SimpleHintData { raw_values_start: 1418, raw_values_count: 1, token_types_start: 1419, token_types_count: 0 }, // [882]
    SimpleHintData { raw_values_start: 1419, raw_values_count: 1, token_types_start: 1420, token_types_count: 0 }, // [883]
    SimpleHintData { raw_values_start: 1420, raw_values_count: 1, token_types_start: 1421, token_types_count: 0 }, // [884]
    SimpleHintData { raw_values_start: 1421, raw_values_count: 1, token_types_start: 1422, token_types_count: 0 }, // [885]
    SimpleHintData { raw_values_start: 1422, raw_values_count: 1, token_types_start: 1423, token_types_count: 0 }, // [886]
    SimpleHintData { raw_values_start: 1423, raw_values_count: 1, token_types_start: 1424, token_types_count: 0 }, // [887]
    SimpleHintData { raw_values_start: 1424, raw_values_count: 1, token_types_start: 1425, token_types_count: 0 }, // [888]
    SimpleHintData { raw_values_start: 1425, raw_values_count: 1, token_types_start: 1426, token_types_count: 0 }, // [889]
    SimpleHintData { raw_values_start: 1426, raw_values_count: 1, token_types_start: 1427, token_types_count: 0 }, // [890]
    SimpleHintData { raw_values_start: 1427, raw_values_count: 1, token_types_start: 1428, token_types_count: 0 }, // [891]
    SimpleHintData { raw_values_start: 1428, raw_values_count: 1, token_types_start: 1429, token_types_count: 0 }, // [892]
    SimpleHintData { raw_values_start: 1429, raw_values_count: 4, token_types_start: 1433, token_types_count: 0 }, // [893]
    SimpleHintData { raw_values_start: 1433, raw_values_count: 3, token_types_start: 1436, token_types_count: 0 }, // [894]
    SimpleHintData { raw_values_start: 1436, raw_values_count: 1, token_types_start: 1437, token_types_count: 0 }, // [895]
    SimpleHintData { raw_values_start: 1437, raw_values_count: 1, token_types_start: 1438, token_types_count: 0 }, // [896]
    SimpleHintData { raw_values_start: 1438, raw_values_count: 1, token_types_start: 1439, token_types_count: 0 }, // [897]
    SimpleHintData { raw_values_start: 1439, raw_values_count: 1, token_types_start: 1440, token_types_count: 0 }, // [898]
    SimpleHintData { raw_values_start: 1440, raw_values_count: 1, token_types_start: 1441, token_types_count: 0 }, // [899]
    SimpleHintData { raw_values_start: 1441, raw_values_count: 1, token_types_start: 1442, token_types_count: 0 }, // [900]
    SimpleHintData { raw_values_start: 1442, raw_values_count: 1, token_types_start: 1443, token_types_count: 0 }, // [901]
    SimpleHintData { raw_values_start: 1443, raw_values_count: 1, token_types_start: 1444, token_types_count: 0 }, // [902]
    SimpleHintData { raw_values_start: 1444, raw_values_count: 1, token_types_start: 1445, token_types_count: 0 }, // [903]
    SimpleHintData { raw_values_start: 1445, raw_values_count: 1, token_types_start: 1446, token_types_count: 0 }, // [904]
    SimpleHintData { raw_values_start: 1446, raw_values_count: 1, token_types_start: 1447, token_types_count: 0 }, // [905]
    SimpleHintData { raw_values_start: 1447, raw_values_count: 1, token_types_start: 1448, token_types_count: 0 }, // [906]
    SimpleHintData { raw_values_start: 1448, raw_values_count: 1, token_types_start: 1449, token_types_count: 0 }, // [907]
    SimpleHintData { raw_values_start: 1449, raw_values_count: 1, token_types_start: 1450, token_types_count: 0 }, // [908]
    SimpleHintData { raw_values_start: 1450, raw_values_count: 1, token_types_start: 1451, token_types_count: 0 }, // [909]
    SimpleHintData { raw_values_start: 1451, raw_values_count: 1, token_types_start: 1452, token_types_count: 0 }, // [910]
    SimpleHintData { raw_values_start: 1452, raw_values_count: 1, token_types_start: 1453, token_types_count: 0 }, // [911]
    SimpleHintData { raw_values_start: 1453, raw_values_count: 1, token_types_start: 1454, token_types_count: 0 }, // [912]
    SimpleHintData { raw_values_start: 1454, raw_values_count: 1, token_types_start: 1455, token_types_count: 0 }, // [913]
    SimpleHintData { raw_values_start: 1455, raw_values_count: 1, token_types_start: 1456, token_types_count: 0 }, // [914]
    SimpleHintData { raw_values_start: 1456, raw_values_count: 1, token_types_start: 1457, token_types_count: 0 }, // [915]
    SimpleHintData { raw_values_start: 1457, raw_values_count: 1, token_types_start: 1458, token_types_count: 0 }, // [916]
    SimpleHintData { raw_values_start: 1458, raw_values_count: 1, token_types_start: 1459, token_types_count: 0 }, // [917]
    SimpleHintData { raw_values_start: 1459, raw_values_count: 1, token_types_start: 1460, token_types_count: 0 }, // [918]
    SimpleHintData { raw_values_start: 1460, raw_values_count: 1, token_types_start: 1461, token_types_count: 0 }, // [919]
    SimpleHintData { raw_values_start: 1461, raw_values_count: 1, token_types_start: 1462, token_types_count: 0 }, // [920]
    SimpleHintData { raw_values_start: 1462, raw_values_count: 1, token_types_start: 1463, token_types_count: 0 }, // [921]
    SimpleHintData { raw_values_start: 1463, raw_values_count: 1, token_types_start: 1464, token_types_count: 0 }, // [922]
    SimpleHintData { raw_values_start: 1464, raw_values_count: 1, token_types_start: 1465, token_types_count: 0 }, // [923]
    SimpleHintData { raw_values_start: 1465, raw_values_count: 1, token_types_start: 1466, token_types_count: 0 }, // [924]
    SimpleHintData { raw_values_start: 1466, raw_values_count: 1, token_types_start: 1467, token_types_count: 0 }, // [925]
    SimpleHintData { raw_values_start: 1467, raw_values_count: 1, token_types_start: 1468, token_types_count: 0 }, // [926]
    SimpleHintData { raw_values_start: 1468, raw_values_count: 1, token_types_start: 1469, token_types_count: 0 }, // [927]
    SimpleHintData { raw_values_start: 1469, raw_values_count: 1, token_types_start: 1470, token_types_count: 0 }, // [928]
    SimpleHintData { raw_values_start: 1470, raw_values_count: 1, token_types_start: 1471, token_types_count: 0 }, // [929]
    SimpleHintData { raw_values_start: 1471, raw_values_count: 2, token_types_start: 1473, token_types_count: 0 }, // [930]
    SimpleHintData { raw_values_start: 1473, raw_values_count: 1, token_types_start: 1474, token_types_count: 0 }, // [931]
    SimpleHintData { raw_values_start: 1474, raw_values_count: 1, token_types_start: 1475, token_types_count: 0 }, // [932]
    SimpleHintData { raw_values_start: 1475, raw_values_count: 1, token_types_start: 1476, token_types_count: 0 }, // [933]
    SimpleHintData { raw_values_start: 1476, raw_values_count: 1, token_types_start: 1477, token_types_count: 0 }, // [934]
    SimpleHintData { raw_values_start: 1477, raw_values_count: 1, token_types_start: 1478, token_types_count: 0 }, // [935]
    SimpleHintData { raw_values_start: 1478, raw_values_count: 1, token_types_start: 1479, token_types_count: 0 }, // [936]
    SimpleHintData { raw_values_start: 1479, raw_values_count: 1, token_types_start: 1480, token_types_count: 0 }, // [937]
    SimpleHintData { raw_values_start: 1480, raw_values_count: 1, token_types_start: 1481, token_types_count: 0 }, // [938]
    SimpleHintData { raw_values_start: 1481, raw_values_count: 1, token_types_start: 1482, token_types_count: 0 }, // [939]
    SimpleHintData { raw_values_start: 1482, raw_values_count: 2, token_types_start: 1484, token_types_count: 0 }, // [940]
    SimpleHintData { raw_values_start: 1484, raw_values_count: 1, token_types_start: 1485, token_types_count: 0 }, // [941]
    SimpleHintData { raw_values_start: 1485, raw_values_count: 1, token_types_start: 1486, token_types_count: 0 }, // [942]
    SimpleHintData { raw_values_start: 1486, raw_values_count: 1, token_types_start: 1487, token_types_count: 0 }, // [943]
    SimpleHintData { raw_values_start: 1487, raw_values_count: 1, token_types_start: 1488, token_types_count: 0 }, // [944]
    SimpleHintData { raw_values_start: 1488, raw_values_count: 1, token_types_start: 1489, token_types_count: 0 }, // [945]
    SimpleHintData { raw_values_start: 1489, raw_values_count: 1, token_types_start: 1490, token_types_count: 0 }, // [946]
    SimpleHintData { raw_values_start: 1490, raw_values_count: 1, token_types_start: 1491, token_types_count: 0 }, // [947]
    SimpleHintData { raw_values_start: 1491, raw_values_count: 1, token_types_start: 1492, token_types_count: 0 }, // [948]
    SimpleHintData { raw_values_start: 1492, raw_values_count: 1, token_types_start: 1493, token_types_count: 0 }, // [949]
    SimpleHintData { raw_values_start: 1493, raw_values_count: 1, token_types_start: 1494, token_types_count: 0 }, // [950]
    SimpleHintData { raw_values_start: 1494, raw_values_count: 1, token_types_start: 1495, token_types_count: 0 }, // [951]
    SimpleHintData { raw_values_start: 1495, raw_values_count: 1, token_types_start: 1496, token_types_count: 0 }, // [952]
    SimpleHintData { raw_values_start: 1496, raw_values_count: 1, token_types_start: 1497, token_types_count: 0 }, // [953]
    SimpleHintData { raw_values_start: 1497, raw_values_count: 1, token_types_start: 1498, token_types_count: 0 }, // [954]
    SimpleHintData { raw_values_start: 1498, raw_values_count: 1, token_types_start: 1499, token_types_count: 0 }, // [955]
    SimpleHintData { raw_values_start: 1499, raw_values_count: 2, token_types_start: 1501, token_types_count: 0 }, // [956]
    SimpleHintData { raw_values_start: 1501, raw_values_count: 13, token_types_start: 1514, token_types_count: 0 }, // [957]
    SimpleHintData { raw_values_start: 1514, raw_values_count: 2, token_types_start: 1516, token_types_count: 0 }, // [958]
    SimpleHintData { raw_values_start: 1516, raw_values_count: 1, token_types_start: 1517, token_types_count: 0 }, // [959]
    SimpleHintData { raw_values_start: 1517, raw_values_count: 1, token_types_start: 1518, token_types_count: 0 }, // [960]
    SimpleHintData { raw_values_start: 1518, raw_values_count: 1, token_types_start: 1519, token_types_count: 0 }, // [961]
    SimpleHintData { raw_values_start: 1519, raw_values_count: 1, token_types_start: 1520, token_types_count: 0 }, // [962]
    SimpleHintData { raw_values_start: 1520, raw_values_count: 1, token_types_start: 1521, token_types_count: 0 }, // [963]
    SimpleHintData { raw_values_start: 1521, raw_values_count: 1, token_types_start: 1522, token_types_count: 0 }, // [964]
    SimpleHintData { raw_values_start: 1522, raw_values_count: 1, token_types_start: 1523, token_types_count: 0 }, // [965]
    SimpleHintData { raw_values_start: 1523, raw_values_count: 2, token_types_start: 1525, token_types_count: 0 }, // [966]
    SimpleHintData { raw_values_start: 1525, raw_values_count: 1, token_types_start: 1526, token_types_count: 0 }, // [967]
    SimpleHintData { raw_values_start: 1526, raw_values_count: 2, token_types_start: 1528, token_types_count: 0 }, // [968]
    SimpleHintData { raw_values_start: 1528, raw_values_count: 1, token_types_start: 1529, token_types_count: 0 }, // [969]
    SimpleHintData { raw_values_start: 1529, raw_values_count: 1, token_types_start: 1530, token_types_count: 0 }, // [970]
    SimpleHintData { raw_values_start: 1530, raw_values_count: 1, token_types_start: 1531, token_types_count: 0 }, // [971]
    SimpleHintData { raw_values_start: 1531, raw_values_count: 1, token_types_start: 1532, token_types_count: 0 }, // [972]
    SimpleHintData { raw_values_start: 1532, raw_values_count: 1, token_types_start: 1533, token_types_count: 0 }, // [973]
    SimpleHintData { raw_values_start: 1533, raw_values_count: 1, token_types_start: 1534, token_types_count: 0 }, // [974]
    SimpleHintData { raw_values_start: 1534, raw_values_count: 1, token_types_start: 1535, token_types_count: 0 }, // [975]
    SimpleHintData { raw_values_start: 1535, raw_values_count: 1, token_types_start: 1536, token_types_count: 0 }, // [976]
    SimpleHintData { raw_values_start: 1536, raw_values_count: 1, token_types_start: 1537, token_types_count: 0 }, // [977]
    SimpleHintData { raw_values_start: 1537, raw_values_count: 1, token_types_start: 1538, token_types_count: 0 }, // [978]
    SimpleHintData { raw_values_start: 1538, raw_values_count: 13, token_types_start: 1551, token_types_count: 5 }, // [979]
    SimpleHintData { raw_values_start: 1556, raw_values_count: 1, token_types_start: 1557, token_types_count: 0 }, // [980]
    SimpleHintData { raw_values_start: 1557, raw_values_count: 1, token_types_start: 1558, token_types_count: 0 }, // [981]
    SimpleHintData { raw_values_start: 1558, raw_values_count: 1, token_types_start: 1559, token_types_count: 0 }, // [982]
    SimpleHintData { raw_values_start: 1559, raw_values_count: 1, token_types_start: 1560, token_types_count: 0 }, // [983]
    SimpleHintData { raw_values_start: 1560, raw_values_count: 1, token_types_start: 1561, token_types_count: 0 }, // [984]
    SimpleHintData { raw_values_start: 1561, raw_values_count: 1, token_types_start: 1562, token_types_count: 0 }, // [985]
    SimpleHintData { raw_values_start: 1562, raw_values_count: 1, token_types_start: 1563, token_types_count: 0 }, // [986]
    SimpleHintData { raw_values_start: 1563, raw_values_count: 1, token_types_start: 1564, token_types_count: 0 }, // [987]
    SimpleHintData { raw_values_start: 1564, raw_values_count: 1, token_types_start: 1565, token_types_count: 0 }, // [988]
    SimpleHintData { raw_values_start: 1565, raw_values_count: 1, token_types_start: 1566, token_types_count: 0 }, // [989]
    SimpleHintData { raw_values_start: 1566, raw_values_count: 1, token_types_start: 1567, token_types_count: 0 }, // [990]
    SimpleHintData { raw_values_start: 1567, raw_values_count: 1, token_types_start: 1568, token_types_count: 0 }, // [991]
    SimpleHintData { raw_values_start: 1568, raw_values_count: 1, token_types_start: 1569, token_types_count: 0 }, // [992]
    SimpleHintData { raw_values_start: 1569, raw_values_count: 1, token_types_start: 1570, token_types_count: 0 }, // [993]
    SimpleHintData { raw_values_start: 1570, raw_values_count: 1, token_types_start: 1571, token_types_count: 0 }, // [994]
    SimpleHintData { raw_values_start: 1571, raw_values_count: 1, token_types_start: 1572, token_types_count: 0 }, // [995]
    SimpleHintData { raw_values_start: 1572, raw_values_count: 1, token_types_start: 1573, token_types_count: 0 }, // [996]
    SimpleHintData { raw_values_start: 1573, raw_values_count: 1, token_types_start: 1574, token_types_count: 0 }, // [997]
    SimpleHintData { raw_values_start: 1574, raw_values_count: 1, token_types_start: 1575, token_types_count: 0 }, // [998]
    SimpleHintData { raw_values_start: 1575, raw_values_count: 1, token_types_start: 1576, token_types_count: 0 }, // [999]
    SimpleHintData { raw_values_start: 1576, raw_values_count: 1, token_types_start: 1577, token_types_count: 0 }, // [1000]
    SimpleHintData { raw_values_start: 1577, raw_values_count: 1, token_types_start: 1578, token_types_count: 0 }, // [1001]
    SimpleHintData { raw_values_start: 1578, raw_values_count: 1, token_types_start: 1579, token_types_count: 0 }, // [1002]
    SimpleHintData { raw_values_start: 1579, raw_values_count: 1, token_types_start: 1580, token_types_count: 0 }, // [1003]
    SimpleHintData { raw_values_start: 1580, raw_values_count: 1, token_types_start: 1581, token_types_count: 0 }, // [1004]
    SimpleHintData { raw_values_start: 1581, raw_values_count: 1, token_types_start: 1582, token_types_count: 0 }, // [1005]
    SimpleHintData { raw_values_start: 1582, raw_values_count: 1, token_types_start: 1583, token_types_count: 0 }, // [1006]
    SimpleHintData { raw_values_start: 1583, raw_values_count: 1, token_types_start: 1584, token_types_count: 0 }, // [1007]
    SimpleHintData { raw_values_start: 1584, raw_values_count: 1, token_types_start: 1585, token_types_count: 0 }, // [1008]
    SimpleHintData { raw_values_start: 1585, raw_values_count: 1, token_types_start: 1586, token_types_count: 0 }, // [1009]
    SimpleHintData { raw_values_start: 1586, raw_values_count: 1, token_types_start: 1587, token_types_count: 0 }, // [1010]
    SimpleHintData { raw_values_start: 1587, raw_values_count: 1, token_types_start: 1588, token_types_count: 0 }, // [1011]
    SimpleHintData { raw_values_start: 1588, raw_values_count: 1, token_types_start: 1589, token_types_count: 0 }, // [1012]
    SimpleHintData { raw_values_start: 1589, raw_values_count: 1, token_types_start: 1590, token_types_count: 0 }, // [1013]
    SimpleHintData { raw_values_start: 1590, raw_values_count: 1, token_types_start: 1591, token_types_count: 0 }, // [1014]
    SimpleHintData { raw_values_start: 1591, raw_values_count: 1, token_types_start: 1592, token_types_count: 0 }, // [1015]
    SimpleHintData { raw_values_start: 1592, raw_values_count: 1, token_types_start: 1593, token_types_count: 0 }, // [1016]
    SimpleHintData { raw_values_start: 1593, raw_values_count: 1, token_types_start: 1594, token_types_count: 0 }, // [1017]
    SimpleHintData { raw_values_start: 1594, raw_values_count: 1, token_types_start: 1595, token_types_count: 0 }, // [1018]
    SimpleHintData { raw_values_start: 1595, raw_values_count: 1, token_types_start: 1596, token_types_count: 0 }, // [1019]
    SimpleHintData { raw_values_start: 1596, raw_values_count: 1, token_types_start: 1597, token_types_count: 0 }, // [1020]
    SimpleHintData { raw_values_start: 1597, raw_values_count: 1, token_types_start: 1598, token_types_count: 0 }, // [1021]
    SimpleHintData { raw_values_start: 1598, raw_values_count: 1, token_types_start: 1599, token_types_count: 0 }, // [1022]
    SimpleHintData { raw_values_start: 1599, raw_values_count: 1, token_types_start: 1600, token_types_count: 0 }, // [1023]
    SimpleHintData { raw_values_start: 1600, raw_values_count: 1, token_types_start: 1601, token_types_count: 0 }, // [1024]
    SimpleHintData { raw_values_start: 1601, raw_values_count: 1, token_types_start: 1602, token_types_count: 0 }, // [1025]
    SimpleHintData { raw_values_start: 1602, raw_values_count: 1, token_types_start: 1603, token_types_count: 0 }, // [1026]
    SimpleHintData { raw_values_start: 1603, raw_values_count: 1, token_types_start: 1604, token_types_count: 0 }, // [1027]
    SimpleHintData { raw_values_start: 1604, raw_values_count: 1, token_types_start: 1605, token_types_count: 0 }, // [1028]
    SimpleHintData { raw_values_start: 1605, raw_values_count: 1, token_types_start: 1606, token_types_count: 0 }, // [1029]
    SimpleHintData { raw_values_start: 1606, raw_values_count: 1, token_types_start: 1607, token_types_count: 0 }, // [1030]
    SimpleHintData { raw_values_start: 1607, raw_values_count: 1, token_types_start: 1608, token_types_count: 0 }, // [1031]
    SimpleHintData { raw_values_start: 1608, raw_values_count: 1, token_types_start: 1609, token_types_count: 0 }, // [1032]
    SimpleHintData { raw_values_start: 1609, raw_values_count: 1, token_types_start: 1610, token_types_count: 0 }, // [1033]
    SimpleHintData { raw_values_start: 1610, raw_values_count: 1, token_types_start: 1611, token_types_count: 0 }, // [1034]
    SimpleHintData { raw_values_start: 1611, raw_values_count: 3, token_types_start: 1614, token_types_count: 0 }, // [1035]
    SimpleHintData { raw_values_start: 1614, raw_values_count: 2, token_types_start: 1616, token_types_count: 0 }, // [1036]
    SimpleHintData { raw_values_start: 1616, raw_values_count: 1, token_types_start: 1617, token_types_count: 0 }, // [1037]
    SimpleHintData { raw_values_start: 1617, raw_values_count: 1, token_types_start: 1618, token_types_count: 0 }, // [1038]
    SimpleHintData { raw_values_start: 1618, raw_values_count: 3, token_types_start: 1621, token_types_count: 0 }, // [1039]
    SimpleHintData { raw_values_start: 1621, raw_values_count: 1, token_types_start: 1622, token_types_count: 0 }, // [1040]
    SimpleHintData { raw_values_start: 1622, raw_values_count: 1, token_types_start: 1623, token_types_count: 0 }, // [1041]
    SimpleHintData { raw_values_start: 1623, raw_values_count: 1, token_types_start: 1624, token_types_count: 0 }, // [1042]
    SimpleHintData { raw_values_start: 1624, raw_values_count: 1, token_types_start: 1625, token_types_count: 0 }, // [1043]
    SimpleHintData { raw_values_start: 1625, raw_values_count: 1, token_types_start: 1626, token_types_count: 0 }, // [1044]
    SimpleHintData { raw_values_start: 1626, raw_values_count: 2, token_types_start: 1628, token_types_count: 0 }, // [1045]
    SimpleHintData { raw_values_start: 1628, raw_values_count: 2, token_types_start: 1630, token_types_count: 0 }, // [1046]
    SimpleHintData { raw_values_start: 1630, raw_values_count: 1, token_types_start: 1631, token_types_count: 0 }, // [1047]
    SimpleHintData { raw_values_start: 1631, raw_values_count: 1, token_types_start: 1632, token_types_count: 0 }, // [1048]
    SimpleHintData { raw_values_start: 1632, raw_values_count: 1, token_types_start: 1633, token_types_count: 0 }, // [1049]
    SimpleHintData { raw_values_start: 1633, raw_values_count: 1, token_types_start: 1634, token_types_count: 0 }, // [1050]
    SimpleHintData { raw_values_start: 1634, raw_values_count: 1, token_types_start: 1635, token_types_count: 0 }, // [1051]
    SimpleHintData { raw_values_start: 1635, raw_values_count: 1, token_types_start: 1636, token_types_count: 0 }, // [1052]
    SimpleHintData { raw_values_start: 1636, raw_values_count: 1, token_types_start: 1637, token_types_count: 0 }, // [1053]
    SimpleHintData { raw_values_start: 1637, raw_values_count: 1, token_types_start: 1638, token_types_count: 0 }, // [1054]
    SimpleHintData { raw_values_start: 1638, raw_values_count: 1, token_types_start: 1639, token_types_count: 0 }, // [1055]
    SimpleHintData { raw_values_start: 1639, raw_values_count: 1, token_types_start: 1640, token_types_count: 0 }, // [1056]
    SimpleHintData { raw_values_start: 1640, raw_values_count: 1, token_types_start: 1641, token_types_count: 0 }, // [1057]
    SimpleHintData { raw_values_start: 1641, raw_values_count: 1, token_types_start: 1642, token_types_count: 0 }, // [1058]
    SimpleHintData { raw_values_start: 1642, raw_values_count: 1, token_types_start: 1643, token_types_count: 0 }, // [1059]
    SimpleHintData { raw_values_start: 1643, raw_values_count: 1, token_types_start: 1644, token_types_count: 0 }, // [1060]
    SimpleHintData { raw_values_start: 1644, raw_values_count: 5, token_types_start: 1649, token_types_count: 0 }, // [1061]
    SimpleHintData { raw_values_start: 1649, raw_values_count: 2, token_types_start: 1651, token_types_count: 0 }, // [1062]
    SimpleHintData { raw_values_start: 1651, raw_values_count: 1, token_types_start: 1652, token_types_count: 0 }, // [1063]
    SimpleHintData { raw_values_start: 1652, raw_values_count: 1, token_types_start: 1653, token_types_count: 0 }, // [1064]
    SimpleHintData { raw_values_start: 1653, raw_values_count: 1, token_types_start: 1654, token_types_count: 0 }, // [1065]
    SimpleHintData { raw_values_start: 1654, raw_values_count: 1, token_types_start: 1655, token_types_count: 0 }, // [1066]
    SimpleHintData { raw_values_start: 1655, raw_values_count: 1, token_types_start: 1656, token_types_count: 0 }, // [1067]
    SimpleHintData { raw_values_start: 1656, raw_values_count: 1, token_types_start: 1657, token_types_count: 0 }, // [1068]
    SimpleHintData { raw_values_start: 1657, raw_values_count: 1, token_types_start: 1658, token_types_count: 0 }, // [1069]
    SimpleHintData { raw_values_start: 1658, raw_values_count: 1, token_types_start: 1659, token_types_count: 0 }, // [1070]
    SimpleHintData { raw_values_start: 1659, raw_values_count: 1, token_types_start: 1660, token_types_count: 0 }, // [1071]
    SimpleHintData { raw_values_start: 1660, raw_values_count: 1, token_types_start: 1661, token_types_count: 0 }, // [1072]
    SimpleHintData { raw_values_start: 1661, raw_values_count: 1, token_types_start: 1662, token_types_count: 0 }, // [1073]
    SimpleHintData { raw_values_start: 1662, raw_values_count: 1, token_types_start: 1663, token_types_count: 0 }, // [1074]
    SimpleHintData { raw_values_start: 1663, raw_values_count: 1, token_types_start: 1664, token_types_count: 0 }, // [1075]
    SimpleHintData { raw_values_start: 1664, raw_values_count: 1, token_types_start: 1665, token_types_count: 0 }, // [1076]
    SimpleHintData { raw_values_start: 1665, raw_values_count: 1, token_types_start: 1666, token_types_count: 0 }, // [1077]
    SimpleHintData { raw_values_start: 1666, raw_values_count: 1, token_types_start: 1667, token_types_count: 0 }, // [1078]
    SimpleHintData { raw_values_start: 1667, raw_values_count: 1, token_types_start: 1668, token_types_count: 0 }, // [1079]
    SimpleHintData { raw_values_start: 1668, raw_values_count: 1, token_types_start: 1669, token_types_count: 0 }, // [1080]
    SimpleHintData { raw_values_start: 1669, raw_values_count: 1, token_types_start: 1670, token_types_count: 0 }, // [1081]
    SimpleHintData { raw_values_start: 1670, raw_values_count: 1, token_types_start: 1671, token_types_count: 0 }, // [1082]
    SimpleHintData { raw_values_start: 1671, raw_values_count: 1, token_types_start: 1672, token_types_count: 0 }, // [1083]
    SimpleHintData { raw_values_start: 1672, raw_values_count: 1, token_types_start: 1673, token_types_count: 0 }, // [1084]
    SimpleHintData { raw_values_start: 1673, raw_values_count: 1, token_types_start: 1674, token_types_count: 0 }, // [1085]
    SimpleHintData { raw_values_start: 1674, raw_values_count: 1, token_types_start: 1675, token_types_count: 0 }, // [1086]
    SimpleHintData { raw_values_start: 1675, raw_values_count: 1, token_types_start: 1676, token_types_count: 0 }, // [1087]
    SimpleHintData { raw_values_start: 1676, raw_values_count: 1, token_types_start: 1677, token_types_count: 0 }, // [1088]
    SimpleHintData { raw_values_start: 1677, raw_values_count: 1, token_types_start: 1678, token_types_count: 0 }, // [1089]
    SimpleHintData { raw_values_start: 1678, raw_values_count: 1, token_types_start: 1679, token_types_count: 0 }, // [1090]
    SimpleHintData { raw_values_start: 1679, raw_values_count: 1, token_types_start: 1680, token_types_count: 0 }, // [1091]
    SimpleHintData { raw_values_start: 1680, raw_values_count: 1, token_types_start: 1681, token_types_count: 0 }, // [1092]
    SimpleHintData { raw_values_start: 1681, raw_values_count: 1, token_types_start: 1682, token_types_count: 0 }, // [1093]
    SimpleHintData { raw_values_start: 1682, raw_values_count: 1, token_types_start: 1683, token_types_count: 0 }, // [1094]
    SimpleHintData { raw_values_start: 1683, raw_values_count: 1, token_types_start: 1684, token_types_count: 0 }, // [1095]
    SimpleHintData { raw_values_start: 1684, raw_values_count: 1, token_types_start: 1685, token_types_count: 0 }, // [1096]
    SimpleHintData { raw_values_start: 1685, raw_values_count: 1, token_types_start: 1686, token_types_count: 0 }, // [1097]
    SimpleHintData { raw_values_start: 1686, raw_values_count: 1, token_types_start: 1687, token_types_count: 0 }, // [1098]
    SimpleHintData { raw_values_start: 1687, raw_values_count: 1, token_types_start: 1688, token_types_count: 0 }, // [1099]
    SimpleHintData { raw_values_start: 1688, raw_values_count: 1, token_types_start: 1689, token_types_count: 0 }, // [1100]
    SimpleHintData { raw_values_start: 1689, raw_values_count: 1, token_types_start: 1690, token_types_count: 0 }, // [1101]
    SimpleHintData { raw_values_start: 1690, raw_values_count: 1, token_types_start: 1691, token_types_count: 0 }, // [1102]
    SimpleHintData { raw_values_start: 1691, raw_values_count: 1, token_types_start: 1692, token_types_count: 0 }, // [1103]
    SimpleHintData { raw_values_start: 1692, raw_values_count: 1, token_types_start: 1693, token_types_count: 0 }, // [1104]
    SimpleHintData { raw_values_start: 1693, raw_values_count: 1, token_types_start: 1694, token_types_count: 0 }, // [1105]
    SimpleHintData { raw_values_start: 1694, raw_values_count: 1, token_types_start: 1695, token_types_count: 0 }, // [1106]
    SimpleHintData { raw_values_start: 1695, raw_values_count: 1, token_types_start: 1696, token_types_count: 0 }, // [1107]
    SimpleHintData { raw_values_start: 1696, raw_values_count: 1, token_types_start: 1697, token_types_count: 0 }, // [1108]
    SimpleHintData { raw_values_start: 1697, raw_values_count: 1, token_types_start: 1698, token_types_count: 0 }, // [1109]
    SimpleHintData { raw_values_start: 1698, raw_values_count: 1, token_types_start: 1699, token_types_count: 0 }, // [1110]
    SimpleHintData { raw_values_start: 1699, raw_values_count: 1, token_types_start: 1700, token_types_count: 0 }, // [1111]
    SimpleHintData { raw_values_start: 1700, raw_values_count: 1, token_types_start: 1701, token_types_count: 0 }, // [1112]
    SimpleHintData { raw_values_start: 1701, raw_values_count: 1, token_types_start: 1702, token_types_count: 0 }, // [1113]
    SimpleHintData { raw_values_start: 1702, raw_values_count: 1, token_types_start: 1703, token_types_count: 0 }, // [1114]
    SimpleHintData { raw_values_start: 1703, raw_values_count: 2, token_types_start: 1705, token_types_count: 0 }, // [1115]
    SimpleHintData { raw_values_start: 1705, raw_values_count: 1, token_types_start: 1706, token_types_count: 0 }, // [1116]
    SimpleHintData { raw_values_start: 1706, raw_values_count: 1, token_types_start: 1707, token_types_count: 0 }, // [1117]
    SimpleHintData { raw_values_start: 1707, raw_values_count: 1, token_types_start: 1708, token_types_count: 0 }, // [1118]
    SimpleHintData { raw_values_start: 1708, raw_values_count: 1, token_types_start: 1709, token_types_count: 0 }, // [1119]
    SimpleHintData { raw_values_start: 1709, raw_values_count: 1, token_types_start: 1710, token_types_count: 0 }, // [1120]
    SimpleHintData { raw_values_start: 1710, raw_values_count: 1, token_types_start: 1711, token_types_count: 0 }, // [1121]
    SimpleHintData { raw_values_start: 1711, raw_values_count: 1, token_types_start: 1712, token_types_count: 0 }, // [1122]
    SimpleHintData { raw_values_start: 1712, raw_values_count: 1, token_types_start: 1713, token_types_count: 0 }, // [1123]
    SimpleHintData { raw_values_start: 1713, raw_values_count: 1, token_types_start: 1714, token_types_count: 0 }, // [1124]
    SimpleHintData { raw_values_start: 1714, raw_values_count: 1, token_types_start: 1715, token_types_count: 0 }, // [1125]
    SimpleHintData { raw_values_start: 1715, raw_values_count: 1, token_types_start: 1716, token_types_count: 0 }, // [1126]
    SimpleHintData { raw_values_start: 1716, raw_values_count: 1, token_types_start: 1717, token_types_count: 0 }, // [1127]
    SimpleHintData { raw_values_start: 1717, raw_values_count: 1, token_types_start: 1718, token_types_count: 0 }, // [1128]
    SimpleHintData { raw_values_start: 1718, raw_values_count: 1, token_types_start: 1719, token_types_count: 0 }, // [1129]
    SimpleHintData { raw_values_start: 1719, raw_values_count: 1, token_types_start: 1720, token_types_count: 0 }, // [1130]
    SimpleHintData { raw_values_start: 1720, raw_values_count: 1, token_types_start: 1721, token_types_count: 0 }, // [1131]
    SimpleHintData { raw_values_start: 1721, raw_values_count: 1, token_types_start: 1722, token_types_count: 0 }, // [1132]
    SimpleHintData { raw_values_start: 1722, raw_values_count: 1, token_types_start: 1723, token_types_count: 0 }, // [1133]
    SimpleHintData { raw_values_start: 1723, raw_values_count: 11, token_types_start: 1734, token_types_count: 0 }, // [1134]
    SimpleHintData { raw_values_start: 1734, raw_values_count: 1, token_types_start: 1735, token_types_count: 0 }, // [1135]
    SimpleHintData { raw_values_start: 1735, raw_values_count: 1, token_types_start: 1736, token_types_count: 0 }, // [1136]
    SimpleHintData { raw_values_start: 1736, raw_values_count: 2, token_types_start: 1738, token_types_count: 0 }, // [1137]
    SimpleHintData { raw_values_start: 1738, raw_values_count: 1, token_types_start: 1739, token_types_count: 0 }, // [1138]
    SimpleHintData { raw_values_start: 1739, raw_values_count: 1, token_types_start: 1740, token_types_count: 0 }, // [1139]
    SimpleHintData { raw_values_start: 1740, raw_values_count: 1, token_types_start: 1741, token_types_count: 0 }, // [1140]
    SimpleHintData { raw_values_start: 1741, raw_values_count: 1, token_types_start: 1742, token_types_count: 0 }, // [1141]
    SimpleHintData { raw_values_start: 1742, raw_values_count: 1, token_types_start: 1743, token_types_count: 0 }, // [1142]
    SimpleHintData { raw_values_start: 1743, raw_values_count: 1, token_types_start: 1744, token_types_count: 0 }, // [1143]
    SimpleHintData { raw_values_start: 1744, raw_values_count: 1, token_types_start: 1745, token_types_count: 0 }, // [1144]
    SimpleHintData { raw_values_start: 1745, raw_values_count: 1, token_types_start: 1746, token_types_count: 0 }, // [1145]
    SimpleHintData { raw_values_start: 1746, raw_values_count: 1, token_types_start: 1747, token_types_count: 0 }, // [1146]
    SimpleHintData { raw_values_start: 1747, raw_values_count: 1, token_types_start: 1748, token_types_count: 0 }, // [1147]
];

pub static SIMPLE_HINT_INDICES: &[u32] = &[
    1, 2, 3, 4, 0, 5, 0, 6, 7, 7, 8, 9, 10, 11, 12, 13,
    14, 15, 16, 17, 18, 19, 19, 20, 21, 22, 22, 23, 24, 24, 25, 26,
    10, 27, 27, 25, 26, 10, 28, 24, 24, 29, 26, 14, 27, 27, 29, 26,
    30, 10, 14, 0, 0, 0, 0, 0, 31, 0, 32, 33, 34, 35, 35, 36,
    37, 38, 39, 39, 40, 16, 41, 9, 10, 12, 42, 42, 43, 44, 45, 45,
    46, 47, 47, 48, 8, 8, 49, 50, 51, 52, 52, 39, 39, 53, 14, 43,
    43, 44, 54, 28, 55, 55, 56, 14, 15, 57, 57, 58, 42, 59, 39, 60,
    45, 61, 62, 8, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 51,
    74, 75, 24, 24, 58, 31, 38, 38, 76, 28, 77, 78, 79, 80, 81, 82,
    83, 84, 46, 14, 43, 43, 44, 85, 85, 78, 86, 86, 77, 87, 87, 88,
    29, 89, 90, 91, 92, 93, 94, 95, 96, 97, 25, 43, 43, 0, 85, 85,
    90, 86, 86, 89, 87, 87, 0, 98, 99, 100, 0, 101, 101, 0, 0, 41,
    41, 0, 0, 16, 16, 0, 0, 102, 102, 0, 0, 0, 0, 0, 0, 103,
    104, 104, 5, 105, 106, 107, 108, 109, 110, 111, 112, 0, 0, 113, 0, 0,
    0, 31, 0, 31, 114, 0, 0, 115, 0, 0, 0, 24, 116, 117, 118, 118,
    30, 10, 14, 0, 119, 120, 121, 121, 31, 122, 122, 123, 41, 16, 124, 125,
    126, 126, 124, 118, 127, 128, 128, 129, 37, 130, 131, 132, 133, 133, 37, 134,
    135, 136, 137, 138, 118, 118, 83, 0, 127, 127, 139, 139, 140, 141, 0, 0,
    0, 0, 142, 0, 143, 0, 144, 144, 108, 62, 140, 0, 145, 146, 146, 147,
    110, 0, 31, 139, 148, 148, 149, 113, 33, 0, 150, 150, 151, 152, 77, 0,
    118, 118, 77, 0, 0, 0, 118, 118, 78, 0, 153, 119, 119, 154, 113, 113,
    155, 31, 68, 156, 157, 158, 159, 160, 160, 161, 42, 162, 163, 164, 165, 166,
    167, 168, 169, 157, 170, 171, 172, 173, 104, 0, 0, 37, 0, 174, 104, 175,
    0, 176, 104, 0, 0, 176, 176, 172, 177, 172, 173, 113, 113, 113, 178, 179,
    180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 189, 189, 190, 191, 192, 193,
    194, 195, 196, 197, 0, 143, 197, 198, 0, 0, 0, 177, 177, 143, 32, 113,
    0, 0, 199, 200, 201, 202, 203, 204, 205, 164, 170, 206, 207, 0, 208, 209,
    210, 211, 157, 172, 172, 172, 170, 173, 173, 173, 171, 212, 213, 206, 160, 214,
    215, 216, 217, 218, 219, 31, 143, 143, 31, 0, 0, 31, 124, 124, 0, 31,
    220, 221, 222, 223, 224, 225, 226, 129, 31, 0, 0, 0, 31, 113, 31, 227,
    133, 228, 229, 230, 231, 232, 233, 233, 233, 0, 234, 234, 235, 236, 235, 236,
    236, 233, 233, 0, 234, 234, 235, 236, 235, 236, 236, 207, 32, 205, 233, 237,
    238, 15, 239, 240, 241, 242, 243, 0, 244, 245, 246, 247, 248, 249, 250, 251,
    252, 253, 254, 255, 256, 257, 258, 259, 260, 261, 261, 129, 0, 0, 0, 0,
    37, 0, 262, 263, 261, 264, 265, 266, 267, 0, 0, 268, 269, 0, 124, 0,
    0, 270, 0, 271, 272, 273, 274, 275, 174, 174, 174, 0, 0, 143, 0, 197,
    0, 276, 276, 0, 0, 277, 278, 279, 253, 253, 31, 0, 280, 280, 0, 281,
    281, 0, 282, 189, 67, 67, 0, 283, 268, 268, 0, 0, 0, 0, 177, 31,
    0, 0, 0, 0, 140, 0, 0, 270, 0, 284, 0, 0, 0, 0, 285, 32,
    286, 287, 283, 283, 124, 283, 283, 124, 283, 0, 288, 289, 290, 291, 207, 142,
    173, 172, 173, 172, 292, 293, 294, 0, 295, 0, 0, 296, 297, 170, 170, 170,
    298, 299, 300, 301, 0, 302, 301, 0, 303, 59, 304, 305, 276, 306, 307, 308,
    309, 310, 311, 312, 313, 314, 315, 316, 317, 318, 319, 320, 321, 39, 39, 237,
    31, 177, 113, 177, 152, 322, 323, 82, 80, 81, 324, 325, 326, 327, 310, 328,
    0, 31, 329, 329, 0, 113, 113, 181, 39, 39, 30, 10, 14, 141, 0, 283,
    125, 126, 126, 124, 152, 152, 121, 31, 39, 39, 214, 330, 80, 141, 0, 31,
    331, 331, 177, 113, 39, 39, 214, 282, 332, 141, 0, 76, 0, 31, 31, 0,
    0, 39, 39, 39, 214, 333, 141, 0, 334, 334, 31, 0, 0, 0, 142, 143,
    143, 104, 124, 124, 113, 227, 39, 39, 16, 141, 0, 39, 39, 14, 141, 0,
    335, 128, 128, 129, 37, 336, 336, 152, 37, 130, 131, 132, 133, 133, 37, 134,
    135, 136, 137, 138, 39, 39, 83, 0, 335, 335, 39, 39, 70, 86, 77, 141,
    0, 337, 338, 31, 0, 0, 339, 0, 0, 283, 283, 340, 341, 341, 129, 31,
    0, 0, 0, 283, 264, 264, 129, 31, 342, 342, 129, 31, 0, 0, 0, 343,
    178, 344, 345, 37, 225, 340, 346, 347, 125, 154, 283, 113, 113, 227, 31, 227,
    348, 348, 0, 125, 154, 39, 39, 71, 0, 349, 199, 110, 350, 350, 351, 352,
    352, 61, 60, 73, 73, 351, 0, 0, 214, 76, 214, 76, 76, 0, 353, 354,
    354, 355, 356, 113, 0, 357, 356, 113, 0, 34, 34, 0, 358, 278, 278, 359,
    360, 359, 361, 361, 361, 362, 361, 361, 363, 329, 329, 364, 365, 356, 366, 234,
    234, 31, 0, 45, 45, 81, 367, 31, 39, 39, 41, 0, 39, 39, 214, 78,
    141, 0, 31, 113, 227, 31, 227, 152, 368, 369, 370, 301, 371, 372, 372, 373,
    373, 372, 374, 31, 0, 375, 376, 377, 378, 379, 380, 381, 382, 383, 384, 385,
    386, 136, 22, 10, 387, 0, 0, 0, 0, 177, 388, 176, 176, 389, 389, 172,
    388, 388, 32, 177, 390, 391, 391, 172, 177, 177, 392, 393, 393, 31, 31, 0,
    0, 394, 0, 395, 395, 392, 396, 397, 398, 399, 400, 394, 401, 402, 403, 404,
    405, 406, 407, 408, 409, 410, 411, 412, 121, 413, 414, 415, 416, 280, 280, 280,
    417, 418, 359, 363, 419, 420, 421, 422, 423, 60, 60, 60, 34, 424, 425, 426,
    427, 428, 429, 430, 431, 432, 433, 434, 344, 435, 435, 435, 0, 0, 436, 437,
    438, 439, 440, 441, 442, 443, 444, 445, 446, 447, 448, 449, 450, 451, 451, 129,
    0, 0, 0, 0, 37, 0, 452, 262, 453, 454, 455, 263, 456, 451, 457, 458,
    167, 459, 11, 270, 460, 461, 462, 463, 231, 139, 139, 237, 31, 177, 113, 177,
    462, 139, 139, 10, 141, 0, 462, 139, 139, 80, 141, 0, 139, 139, 332, 141,
    0, 462, 139, 139, 139, 333, 141, 0, 139, 139, 16, 141, 0, 139, 139, 14,
    141, 0, 462, 139, 139, 83, 0, 139, 139, 77, 141, 0, 464, 139, 139, 71,
    141, 0, 139, 139, 18, 141, 0, 462, 139, 139, 41, 141, 0, 139, 139, 78,
    141, 0, 462, 465, 466, 467, 468, 469, 364, 470, 471, 235, 235, 0, 235, 472,
    31, 390, 390, 31, 473, 474, 475, 476, 477, 478, 236, 479, 480, 142, 142, 142,
    481, 482, 483, 484, 485, 486, 150, 487, 488, 489, 490, 45, 50, 491, 492, 493,
    494, 495, 495, 495, 496, 227, 61, 73, 60, 497, 498, 0, 0, 0, 0, 0,
    499, 205, 205, 0, 500, 207, 294, 294, 278, 0, 0, 0, 0, 501, 278, 202,
    0, 160, 0, 0, 0, 0, 502, 503, 503, 503, 504, 278, 505, 0, 0, 506,
    506, 506, 506, 164, 170, 207, 0, 507, 503, 503, 503, 504, 0, 492, 492, 31,
    227, 0, 0, 0, 233, 183, 183, 0, 32, 0, 0, 197, 0, 31, 0, 0,
    0, 227, 0, 0, 143, 0, 0, 0, 0, 0, 0, 0, 0, 390, 0, 31,
    0, 0, 177, 177, 508, 124, 37, 216, 279, 395, 0, 0, 32, 104, 0, 68,
    143, 198, 390, 0, 0, 509, 510, 0, 511, 0, 0, 270, 0, 86, 512, 218,
    218, 513, 513, 514, 147, 515, 0, 37, 0, 356, 516, 356, 517, 518, 519, 152,
    152, 520, 513, 521, 522, 523, 524, 525, 526, 527, 193, 528, 529, 529, 124, 530,
    124, 87, 531, 426, 426, 532, 532, 426, 426, 426, 426, 533, 534, 535, 535, 31,
    424, 424, 0, 535, 536, 147, 537, 538, 539, 540, 541, 542, 329, 543, 19, 19,
    544, 19, 88, 545, 546, 547, 548, 263, 263, 0, 0, 376, 376, 356, 0, 0,
    37, 198, 549, 0, 550, 551, 542, 202, 202, 160, 263, 517, 552, 553, 554, 555,
    556, 34, 34, 0, 0, 557, 424, 262, 101, 101, 129, 261, 261, 129, 451, 451,
    129, 452, 452, 129, 112, 112, 129, 453, 454, 455, 558, 152, 152, 513, 559, 0,
    0, 0, 31, 0, 0, 0, 0, 0, 557, 560, 0, 0, 152, 152, 559, 0,
    560, 0, 0, 0, 0, 31, 0, 0, 561, 0, 31, 0, 112, 112, 129, 101,
    101, 129, 0, 0, 0, 0, 0, 0, 112, 112, 129, 101, 101, 129, 0, 61,
    61, 562, 563, 563, 564, 77, 77, 0, 151, 141, 227, 61, 565, 566, 566, 444,
    124, 356, 340, 227, 61, 345, 345, 77, 0, 151, 31, 227, 227, 61, 567, 34,
    34, 0, 0, 34, 34, 61, 61, 34, 568, 569, 0, 0, 0, 0, 0, 113,
    177, 0, 0, 0, 34, 34, 0, 0, 0, 394, 0, 34, 0, 0, 449, 0,
    168, 0, 0, 112, 456, 456, 143, 0, 0, 124, 0, 0, 26, 0, 124, 0,
    0, 570, 0, 390, 31, 31, 0, 113, 113, 124, 13, 13, 0, 80, 367, 0,
    0, 0, 0, 270, 31, 571, 367, 572, 31, 0, 0, 0, 573, 161, 161, 18,
    177, 573, 161, 161, 18, 177, 31, 0, 0, 0, 393, 393, 393, 31, 356, 356,
    31, 113, 356, 31, 0, 0, 0, 177, 0, 0, 0, 574, 393, 567, 31, 0,
    92, 575, 27, 576, 577, 578, 579, 580, 581, 582, 583, 99, 99, 99, 0, 38,
    38, 76, 0, 16, 16, 0, 64, 64, 76, 41, 0, 0, 33, 0, 584, 152,
    152, 99, 585, 152, 152, 109, 585, 316, 316, 376, 48, 586, 586, 129, 0, 384,
    587, 0, 586, 48, 173, 173, 142, 173, 588, 101, 101, 129, 0, 24, 589, 373,
    0, 0, 0, 0, 37, 0, 590, 589, 590, 591, 591, 112, 261, 451, 452, 129,
    262, 453, 454, 455, 101, 0, 0, 0, 0, 37, 0, 31, 590, 589, 589, 589,
    592, 31, 0, 0, 373, 0, 593, 453, 453, 0, 31, 0, 594, 591, 591, 112,
    261, 451, 452, 129, 262, 454, 455, 453, 595, 596, 597, 598, 0, 0, 572, 194,
    599, 270, 0, 31, 0, 0, 0, 599, 600, 601, 602, 0, 603, 604, 605, 606,
    607, 608, 609, 610, 0, 611, 612, 613, 614, 141, 141, 492, 141, 141, 141, 278,
    492, 615, 570, 570, 615, 616, 617, 618, 362, 619, 620, 621, 622, 57, 26, 500,
    278, 26, 0, 31, 0, 0, 0, 227, 0, 599, 623, 624, 128, 0, 0, 343,
    178, 344, 332, 625, 626, 627, 628, 629, 630, 631, 632, 361, 633, 634, 635, 636,
    637, 529, 638, 61, 61, 61, 562, 563, 563, 564, 77, 77, 0, 151, 141, 227,
    565, 566, 566, 444, 124, 356, 340, 227, 345, 345, 77, 0, 151, 31, 227, 227,
    567, 639, 326, 640, 350, 641, 642, 643, 644, 645, 646, 647, 12, 648, 649, 650,
    198, 398, 651, 651, 651, 124, 37, 31, 0, 394, 33, 33, 394, 398, 345, 652,
    653, 0, 279, 654, 0, 216, 0, 294, 294, 278, 449, 34, 294, 0, 655, 656,
    657, 658, 659, 660, 0, 0, 0, 661, 0, 0, 0, 0, 0, 0, 662, 76,
    663, 0, 0, 0, 661, 0, 0, 0, 0, 0, 664, 661, 665, 665, 664, 0,
    76, 76, 0, 0, 31, 0, 0, 302, 633, 666, 666, 568, 667, 668, 669, 663,
    663, 31, 0, 0, 670, 528, 671, 544, 672, 673, 674, 0, 675, 676, 13, 35,
    677, 678, 0, 679, 0, 679, 78, 669, 0, 0, 113, 113, 0, 0, 680, 681,
    682, 667, 683, 684, 172, 172, 142, 172, 685, 499, 499, 278, 686, 0, 482, 482,
    0, 686, 348, 687, 618, 688, 653, 348, 293, 262, 262, 0, 31, 0, 37, 0,
    24, 689, 559, 559, 0, 37, 0, 32, 32, 37, 262, 690, 691, 0, 692, 143,
    124, 37, 216, 503, 279, 395, 599, 104, 176, 693, 0, 0, 694, 695, 0, 566,
    696, 697, 125, 125, 124, 125, 698, 699, 700, 701, 702, 703, 704, 705, 706, 707,
    708, 709, 710, 561, 561, 270, 0, 31, 333, 333, 0, 32, 32, 31, 227, 711,
    47, 126, 389, 0, 712, 43, 0, 713, 714, 85, 715, 716, 717, 718, 719, 720,
    721, 722, 60, 61, 61, 31, 567, 723, 723, 345, 723, 234, 234, 234, 234, 234,
    714, 160, 160, 0, 724, 725, 73, 60, 234, 234, 278, 714, 160, 160, 0, 724,
    61, 723, 723, 0, 0, 0, 663, 0, 0, 0, 31, 31, 227, 0, 0, 0,
    0, 0, 76, 0, 234, 73, 73, 119, 726, 727, 728, 729, 328, 730, 731, 732,
    733, 734, 166, 735, 736, 737, 738, 739, 561, 740, 741, 742, 333, 743, 62, 744,
    169, 8, 745, 746, 747, 748, 749, 750, 750, 750, 751, 77, 0, 752, 752, 108,
    139, 753, 754, 750, 750, 77, 0, 752, 752, 108, 139, 753, 754, 168, 755, 756,
    757, 0, 758, 0, 0, 113, 0, 0, 31, 0, 455, 455, 0, 0, 759, 654,
    654, 760, 761, 761, 302, 761, 762, 763, 166, 0, 764, 357, 0, 515, 765, 766,
    767, 768, 134, 769, 770, 771, 772, 773, 137, 774, 775, 220, 567, 68, 31, 68,
    31, 152, 31, 220, 31, 0, 776, 73, 61, 60, 723, 220, 220, 155, 31, 68,
    220, 777, 778, 779, 780, 781, 0, 782, 783, 784, 0, 292, 785, 785, 142, 172,
    172, 173, 278, 0, 278, 786, 787, 788, 789, 790, 279, 279, 791, 792, 617, 793,
    794, 37, 36, 124, 124, 174, 0, 693, 124, 124, 693, 270, 270, 270, 270, 270,
    0, 76, 113, 663, 32, 238, 104, 31, 205, 174, 426, 0, 795, 796, 797, 351,
    798, 799, 559, 559, 0, 37, 0, 356, 516, 356, 517, 559, 800, 801, 355, 76,
    802, 519, 803, 804, 805, 806, 807, 63, 808, 809, 810, 585, 811, 334, 214, 214,
    214, 214, 812, 526, 112, 112, 129, 0, 0, 0, 0, 37, 0, 343, 178, 344,
    617, 617, 813, 147, 677, 0, 262, 263, 814, 262, 453, 454, 455, 263, 456, 513,
    112, 138, 112, 779, 815, 816, 817, 818, 669, 819, 820, 821, 530, 0, 570, 822,
    0, 0, 31, 0, 822, 509, 509, 0, 31, 31, 395, 32, 395, 0, 509, 823,
    824, 563, 122, 64, 825, 826, 827, 0, 828, 829, 830, 831, 832, 833, 834, 835,
    527, 836, 837, 151, 151, 129, 0, 31, 0, 0, 151, 151, 151, 31, 0, 0,
    0, 142, 142, 143, 341, 754, 838, 839, 840, 841, 167, 167, 367, 367, 167, 124,
    0, 842, 843, 844, 845, 846, 54, 170, 847, 848, 849, 850, 165, 44, 851, 165,
    0, 0, 535, 0, 852, 853, 0, 551, 854, 855, 856, 857, 858, 859, 860, 281,
    281, 544, 281, 388, 861, 862, 646, 647, 203, 215, 539, 460, 460, 854, 863, 204,
    397, 864, 864, 415, 414, 794, 31, 392, 865, 244, 670, 866, 867, 505, 58, 868,
    869, 81, 93, 870, 871, 124, 124, 142, 124, 872, 103, 464, 503, 503, 37, 454,
    207, 207, 873, 24, 161, 874, 31, 227, 875, 876, 572, 124, 399, 572, 194, 877,
    878, 552, 879, 142, 173, 880, 172, 785, 525, 65, 881, 882, 883, 884, 885, 886,
    887, 888, 889, 890, 891, 892, 67, 67, 0, 31, 713, 76, 76, 76, 60, 893,
    76, 76, 73, 893, 713, 713, 894, 568, 836, 895, 66, 67, 354, 893, 463, 231,
    119, 119, 279, 765, 765, 106, 119, 119, 280, 688, 896, 897, 898, 899, 900, 901,
    902, 903, 904, 905, 906, 907, 908, 909, 148, 751, 910, 911, 812, 912, 913, 914,
    915, 7, 616, 916, 917, 463, 918, 919, 920, 921, 922, 923, 331, 100, 100, 100,
    99, 99, 585, 329, 0, 38, 38, 76, 0, 16, 16, 0, 64, 64, 76, 41,
    0, 0, 34, 0, 462, 924, 668, 687, 16, 0, 925, 926, 374, 374, 82, 927,
    928, 929, 94, 356, 356, 88, 930, 425, 425, 521, 521, 931, 483, 483, 129, 124,
    275, 275, 658, 931, 389, 389, 674, 931, 691, 691, 931, 279, 279, 419, 113, 124,
    932, 932, 124, 152, 31, 31, 0, 0, 0, 356, 933, 934, 935, 936, 937, 938,
    517, 939, 560, 560, 31, 0, 224, 224, 37, 818, 351, 37, 76, 76, 0, 0,
    0, 37, 37, 940, 843, 517, 0, 0, 941, 942, 943, 14, 944, 25, 945, 946,
    947, 948, 949, 950, 951, 952, 953, 954, 955, 0, 0, 0, 0, 0, 956, 449,
    24, 68, 68, 956, 0, 0, 957, 261, 261, 129, 451, 451, 129, 452, 452, 129,
    957, 34, 0, 424, 261, 261, 129, 451, 451, 129, 452, 452, 129, 112, 112, 129,
    262, 509, 558, 513, 68, 68, 68, 34, 424, 101, 453, 509, 455, 112, 559, 513,
    261, 451, 452, 262, 455, 558, 152, 152, 227, 958, 31, 152, 958, 31, 152, 220,
    31, 227, 959, 665, 426, 960, 456, 83, 131, 718, 718, 37, 765, 765, 718, 130,
    739, 739, 37, 765, 765, 739, 523, 95, 932, 152, 152, 961, 31, 961, 962, 21,
    963, 964, 587, 119, 119, 0, 0, 0, 0, 142, 0, 143, 197, 0, 0, 0,
    567, 280, 220, 220, 558, 558, 558, 220, 112, 261, 451, 452, 262, 455, 119, 558,
    965, 966, 966, 649, 485, 24, 734, 119, 119, 14, 141, 0, 119, 119, 0, 166,
    166, 967, 0, 0, 0, 0, 166, 166, 166, 0, 968, 270, 174, 142, 0, 969,
    592, 970, 102, 971, 972, 0, 0, 0, 233, 238, 238, 238, 177, 183, 973, 974,
    975, 976, 503, 165, 166, 977, 895, 0, 0, 572, 194, 270, 0, 0, 0, 0,
    270, 168, 399, 978, 176, 176, 104, 346, 346, 129, 31, 76, 31, 979, 979, 143,
    31, 143, 143, 340, 340, 113, 443, 346, 167, 174, 862, 980, 874, 981, 452, 452,
    129, 0, 0, 0, 0, 37, 0, 343, 178, 344, 617, 617, 813, 147, 677, 262,
    263, 452, 342, 982, 983, 984, 322, 985, 986, 987, 988, 989, 990, 991, 992, 993,
    994, 995, 996, 997, 998, 999, 1000, 1001, 1002, 1003, 1004, 1005, 1006, 79, 91, 168,
    336, 1007, 1008, 1009, 366, 532, 227, 723, 61, 139, 139, 39, 139, 72, 98, 39,
    39, 139, 118, 139, 139, 39, 139, 39, 139, 39, 60, 73, 39, 139, 39, 139,
    435, 55, 495, 39, 118, 139, 39, 139, 34, 118, 750, 750, 119, 118, 426, 327,
    1010, 1011, 1012, 1013, 1014, 347, 347, 356, 340, 340, 1015, 340, 340, 113, 522, 340,
    340, 129, 124, 152, 340, 1016, 84, 96, 1017, 1018, 170, 170, 863, 390, 31, 0,
    0, 0, 0, 172, 0, 0, 0, 174, 177, 283, 390, 390, 172, 1019, 1020, 1021,
    1022, 1023, 1024, 1025, 1026, 1027, 1028, 0, 1029, 753, 1030, 1031, 1032, 1033, 1034, 339,
    276, 276, 0, 1035, 282, 282, 31, 281, 281, 31, 445, 445, 789, 1036, 909, 780,
    19, 19, 31, 67, 445, 445, 789, 0, 0, 567, 197, 0, 0, 31, 227, 31,
    723, 77, 154, 154, 31, 1037, 89, 560, 17, 0, 502, 502, 205, 0, 0, 507,
    507, 0, 46, 97, 154, 69, 1038, 0, 330, 69, 70, 70, 1039, 1040, 330, 1041,
    931, 931, 129, 124, 931, 1042, 524, 1043, 1044, 724, 520, 504, 396, 1045, 1045, 396,
    397, 31, 1046, 1046, 152, 1047, 396, 1048, 183, 183, 183, 396, 1048, 0, 397, 1049,
    1050, 1051, 1052, 861, 1053, 33, 1054, 1055, 1056, 1057, 1058, 1059, 1060, 1061, 1061, 336,
    200, 288, 926, 236, 1062, 1060, 1063, 758, 758, 0, 160, 160, 765, 242, 1064, 1065,
    1066, 1067, 1068, 1069, 1040, 1070, 1071, 1072, 71, 1073, 1074, 1075, 1076, 0, 217, 217,
    72, 72, 72, 77, 0, 151, 1077, 1078, 31, 0, 0, 18, 176, 176, 104, 390,
    390, 31, 1079, 1080, 1081, 549, 1082, 0, 0, 761, 0, 0, 1083, 1084, 1085, 965,
    965, 956, 449, 24, 965, 391, 282, 282, 282, 1086, 1087, 0, 1088, 1089, 1090, 1091,
    68, 68, 34, 424, 101, 453, 509, 455, 558, 152, 152, 112, 262, 261, 451, 452,
    220, 220, 558, 558, 558, 220, 1092, 1093, 1094, 73, 73, 73, 0, 0, 119, 119,
    34, 424, 1095, 1096, 1097, 51, 55, 55, 55, 0, 74, 41, 0, 1098, 1099, 1100,
    1101, 1102, 663, 1103, 1104, 1105, 1106, 1107, 1108, 1109, 1110, 1111, 1112, 1113, 1114, 567,
    567, 1113, 417, 1115, 1115, 356, 31, 0, 0, 280, 143, 0, 417, 1116, 1117, 1118,
    1119, 865, 1120, 1121, 1122, 1123, 1124, 1125, 1126, 78, 90, 1127, 1128, 9, 1129, 1130,
    1131, 1132, 234, 234, 0, 0, 0, 724, 0, 0, 0, 0, 234, 1133, 424, 424,
    0, 31, 0, 1134, 262, 101, 101, 129, 261, 261, 129, 451, 451, 129, 452, 452,
    129, 112, 112, 129, 453, 454, 455, 509, 513, 424, 1135, 0, 1136, 0, 0, 0,
    0, 0, 0, 0, 270, 168, 168, 270, 168, 455, 0, 0, 1137, 151, 112, 151,
    112, 263, 152, 152, 890, 0, 0, 0, 68, 0, 776, 152, 152, 890, 0, 0,
    0, 68, 0, 220, 152, 152, 765, 765, 22, 0, 152, 152, 152, 765, 14, 208,
    1138, 1047, 0, 1063, 23, 75, 1139, 1140, 1141, 1142, 1143, 1144, 1145, 1146, 1147, 1048,
];

pub fn get_hive_segment_grammar(name: &str) -> Option<RootGrammar> {
    match name {
        "AbortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(0), tables: &HIVE_TABLES }),
        "AbsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1), tables: &HIVE_TABLES }),
        "AbsoluteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2), tables: &HIVE_TABLES }),
        "AccessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3), tables: &HIVE_TABLES }),
        "AccessObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(4), tables: &HIVE_TABLES }),
        "AccessPermissionSegment" => Some(RootGrammar { grammar_id: GrammarId(65), tables: &HIVE_TABLES }),
        "AccessPermissionsSegment" => Some(RootGrammar { grammar_id: GrammarId(134), tables: &HIVE_TABLES }),
        "AccessSchemaObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(137), tables: &HIVE_TABLES }),
        "AccessSchemaPluralObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(160), tables: &HIVE_TABLES }),
        "AccessStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(183), tables: &HIVE_TABLES }),
        "AccessTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(186), tables: &HIVE_TABLES }),
        "AccessorGrammar" => Some(RootGrammar { grammar_id: GrammarId(208), tables: &HIVE_TABLES }),
        "AccountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(210), tables: &HIVE_TABLES }),
        "AccountsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(211), tables: &HIVE_TABLES }),
        "ActionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(212), tables: &HIVE_TABLES }),
        "AdaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(213), tables: &HIVE_TABLES }),
        "AddKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(214), tables: &HIVE_TABLES }),
        "AdminKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(215), tables: &HIVE_TABLES }),
        "AfterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(216), tables: &HIVE_TABLES }),
        "AggregateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(217), tables: &HIVE_TABLES }),
        "AggregateOrderByClause" => Some(RootGrammar { grammar_id: GrammarId(218), tables: &HIVE_TABLES }),
        "AliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(219), tables: &HIVE_TABLES }),
        "AliasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(231), tables: &HIVE_TABLES }),
        "AliasedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(232), tables: &HIVE_TABLES }),
        "AllKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(235), tables: &HIVE_TABLES }),
        "AllocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(236), tables: &HIVE_TABLES }),
        "AlsoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(237), tables: &HIVE_TABLES }),
        "AlterDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(238), tables: &HIVE_TABLES }),
        "AlterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(259), tables: &HIVE_TABLES }),
        "AlterSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(260), tables: &HIVE_TABLES }),
        "AlterSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(276), tables: &HIVE_TABLES }),
        "AlterTableDropColumnGrammar" => Some(RootGrammar { grammar_id: GrammarId(282), tables: &HIVE_TABLES }),
        "AlterTableOptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(287), tables: &HIVE_TABLES }),
        "AlterTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(320), tables: &HIVE_TABLES }),
        "AlterViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(326), tables: &HIVE_TABLES }),
        "AlwaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(339), tables: &HIVE_TABLES }),
        "AmpersandSegment" => Some(RootGrammar { grammar_id: GrammarId(340), tables: &HIVE_TABLES }),
        "AnalyseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(341), tables: &HIVE_TABLES }),
        "AnalyzeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(342), tables: &HIVE_TABLES }),
        "AndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(343), tables: &HIVE_TABLES }),
        "AndOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(344), tables: &HIVE_TABLES }),
        "AnyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(345), tables: &HIVE_TABLES }),
        "ApplyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(346), tables: &HIVE_TABLES }),
        "ArchiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(347), tables: &HIVE_TABLES }),
        "AreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(348), tables: &HIVE_TABLES }),
        "ArithmeticBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(349), tables: &HIVE_TABLES }),
        "ArrayAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(360), tables: &HIVE_TABLES }),
        "ArrayExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(368), tables: &HIVE_TABLES }),
        "ArrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(369), tables: &HIVE_TABLES }),
        "ArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(370), tables: &HIVE_TABLES }),
        "ArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(373), tables: &HIVE_TABLES }),
        "AsAliasOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(379), tables: &HIVE_TABLES }),
        "AsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(381), tables: &HIVE_TABLES }),
        "AscKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(382), tables: &HIVE_TABLES }),
        "AsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(383), tables: &HIVE_TABLES }),
        "AssertionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(384), tables: &HIVE_TABLES }),
        "AssignmentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(385), tables: &HIVE_TABLES }),
        "AsymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(386), tables: &HIVE_TABLES }),
        "AtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(387), tables: &HIVE_TABLES }),
        "AtomicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(388), tables: &HIVE_TABLES }),
        "AttributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(389), tables: &HIVE_TABLES }),
        "AttributesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(390), tables: &HIVE_TABLES }),
        "AuditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(391), tables: &HIVE_TABLES }),
        "AuthorizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(392), tables: &HIVE_TABLES }),
        "AutoIncrementGrammar" => Some(RootGrammar { grammar_id: GrammarId(393), tables: &HIVE_TABLES }),
        "Auto_incrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(395), tables: &HIVE_TABLES }),
        "AutocommitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(396), tables: &HIVE_TABLES }),
        "AvgKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(397), tables: &HIVE_TABLES }),
        "Avg_row_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(398), tables: &HIVE_TABLES }),
        "AvroKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(399), tables: &HIVE_TABLES }),
        "BackQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(400), tables: &HIVE_TABLES }),
        "BackupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(401), tables: &HIVE_TABLES }),
        "BackwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(402), tables: &HIVE_TABLES }),
        "BareFunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(403), tables: &HIVE_TABLES }),
        "BaseExpressionElementGrammar" => Some(RootGrammar { grammar_id: GrammarId(404), tables: &HIVE_TABLES }),
        "BaseFileSegment" => Some(RootGrammar { grammar_id: GrammarId(416), tables: &HIVE_TABLES }),
        "BaseSegment" => Some(RootGrammar { grammar_id: GrammarId(417), tables: &HIVE_TABLES }),
        "BeforeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(418), tables: &HIVE_TABLES }),
        "BeginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(419), tables: &HIVE_TABLES }),
        "BernoulliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(420), tables: &HIVE_TABLES }),
        "BetweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(421), tables: &HIVE_TABLES }),
        "BigintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(422), tables: &HIVE_TABLES }),
        "BinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(423), tables: &HIVE_TABLES }),
        "BinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(424), tables: &HIVE_TABLES }),
        "BinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(429), tables: &HIVE_TABLES }),
        "BindingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(430), tables: &HIVE_TABLES }),
        "BitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(431), tables: &HIVE_TABLES }),
        "Bit_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(432), tables: &HIVE_TABLES }),
        "BitvarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(433), tables: &HIVE_TABLES }),
        "BitwiseAndSegment" => Some(RootGrammar { grammar_id: GrammarId(434), tables: &HIVE_TABLES }),
        "BitwiseLShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(435), tables: &HIVE_TABLES }),
        "BitwiseOrSegment" => Some(RootGrammar { grammar_id: GrammarId(438), tables: &HIVE_TABLES }),
        "BitwiseRShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(439), tables: &HIVE_TABLES }),
        "BitwiseXorSegment" => Some(RootGrammar { grammar_id: GrammarId(442), tables: &HIVE_TABLES }),
        "BlobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(443), tables: &HIVE_TABLES }),
        "BoolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(444), tables: &HIVE_TABLES }),
        "BooleanBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(445), tables: &HIVE_TABLES }),
        "BooleanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(448), tables: &HIVE_TABLES }),
        "BooleanLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(449), tables: &HIVE_TABLES }),
        "BothKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(452), tables: &HIVE_TABLES }),
        "BracketedArguments" => Some(RootGrammar { grammar_id: GrammarId(453), tables: &HIVE_TABLES }),
        "BracketedColumnReferenceListGrammar" => Some(RootGrammar { grammar_id: GrammarId(456), tables: &HIVE_TABLES }),
        "BracketedPropertyListGrammar" => Some(RootGrammar { grammar_id: GrammarId(459), tables: &HIVE_TABLES }),
        "BracketedSegment" => Some(RootGrammar { grammar_id: GrammarId(462), tables: &HIVE_TABLES }),
        "BracketedSetExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(463), tables: &HIVE_TABLES }),
        "BreadthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(465), tables: &HIVE_TABLES }),
        "BreakKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(466), tables: &HIVE_TABLES }),
        "BrowseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(467), tables: &HIVE_TABLES }),
        "BucketKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(468), tables: &HIVE_TABLES }),
        "BucketsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(469), tables: &HIVE_TABLES }),
        "BulkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(470), tables: &HIVE_TABLES }),
        "ByKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(471), tables: &HIVE_TABLES }),
        "CTEColumnList" => Some(RootGrammar { grammar_id: GrammarId(472), tables: &HIVE_TABLES }),
        "CTEDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(474), tables: &HIVE_TABLES }),
        "CacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(480), tables: &HIVE_TABLES }),
        "CallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(481), tables: &HIVE_TABLES }),
        "CalledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(482), tables: &HIVE_TABLES }),
        "CardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(483), tables: &HIVE_TABLES }),
        "CascadeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(484), tables: &HIVE_TABLES }),
        "CascadedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(485), tables: &HIVE_TABLES }),
        "CaseExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(486), tables: &HIVE_TABLES }),
        "CaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(510), tables: &HIVE_TABLES }),
        "CastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(511), tables: &HIVE_TABLES }),
        "CastOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(512), tables: &HIVE_TABLES }),
        "CatalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(513), tables: &HIVE_TABLES }),
        "Catalog_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(514), tables: &HIVE_TABLES }),
        "CeilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(515), tables: &HIVE_TABLES }),
        "CeilingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(516), tables: &HIVE_TABLES }),
        "ChainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(517), tables: &HIVE_TABLES }),
        "ChangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(518), tables: &HIVE_TABLES }),
        "CharCharacterSetGrammar" => Some(RootGrammar { grammar_id: GrammarId(519), tables: &HIVE_TABLES }),
        "CharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(520), tables: &HIVE_TABLES }),
        "Char_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(521), tables: &HIVE_TABLES }),
        "CharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(522), tables: &HIVE_TABLES }),
        "Character_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(523), tables: &HIVE_TABLES }),
        "Character_set_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(524), tables: &HIVE_TABLES }),
        "Character_set_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(525), tables: &HIVE_TABLES }),
        "Character_set_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(526), tables: &HIVE_TABLES }),
        "CharacteristicsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(527), tables: &HIVE_TABLES }),
        "CharactersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(528), tables: &HIVE_TABLES }),
        "CheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(529), tables: &HIVE_TABLES }),
        "CheckedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(530), tables: &HIVE_TABLES }),
        "CheckpointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(531), tables: &HIVE_TABLES }),
        "ChecksumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(532), tables: &HIVE_TABLES }),
        "ClassKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(533), tables: &HIVE_TABLES }),
        "Class_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(534), tables: &HIVE_TABLES }),
        "ClobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(535), tables: &HIVE_TABLES }),
        "CloseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(536), tables: &HIVE_TABLES }),
        "ClusterByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(537), tables: &HIVE_TABLES }),
        "ClusterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(548), tables: &HIVE_TABLES }),
        "ClusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(549), tables: &HIVE_TABLES }),
        "ClusterstatusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(550), tables: &HIVE_TABLES }),
        "CoalesceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(551), tables: &HIVE_TABLES }),
        "CobolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(552), tables: &HIVE_TABLES }),
        "CodeSegment" => Some(RootGrammar { grammar_id: GrammarId(553), tables: &HIVE_TABLES }),
        "CollateGrammar" => Some(RootGrammar { grammar_id: GrammarId(554), tables: &HIVE_TABLES }),
        "CollateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(555), tables: &HIVE_TABLES }),
        "CollationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(556), tables: &HIVE_TABLES }),
        "CollationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(557), tables: &HIVE_TABLES }),
        "Collation_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(563), tables: &HIVE_TABLES }),
        "Collation_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(564), tables: &HIVE_TABLES }),
        "Collation_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(565), tables: &HIVE_TABLES }),
        "CollectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(566), tables: &HIVE_TABLES }),
        "CollectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(567), tables: &HIVE_TABLES }),
        "ColonDelimiterSegment" => Some(RootGrammar { grammar_id: GrammarId(568), tables: &HIVE_TABLES }),
        "ColonPrefixSegment" => Some(RootGrammar { grammar_id: GrammarId(569), tables: &HIVE_TABLES }),
        "ColonSegment" => Some(RootGrammar { grammar_id: GrammarId(570), tables: &HIVE_TABLES }),
        "ColumnConstraintDefaultGrammar" => Some(RootGrammar { grammar_id: GrammarId(571), tables: &HIVE_TABLES }),
        "ColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(576), tables: &HIVE_TABLES }),
        "ColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(604), tables: &HIVE_TABLES }),
        "ColumnGeneratedGrammar" => Some(RootGrammar { grammar_id: GrammarId(611), tables: &HIVE_TABLES }),
        "ColumnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(612), tables: &HIVE_TABLES }),
        "ColumnReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(613), tables: &HIVE_TABLES }),
        "Column_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(617), tables: &HIVE_TABLES }),
        "ColumnsExpressionFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(618), tables: &HIVE_TABLES }),
        "ColumnsExpressionFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(619), tables: &HIVE_TABLES }),
        "ColumnsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(620), tables: &HIVE_TABLES }),
        "ColumnsExpressionNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(621), tables: &HIVE_TABLES }),
        "ColumnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(622), tables: &HIVE_TABLES }),
        "CommaSegment" => Some(RootGrammar { grammar_id: GrammarId(623), tables: &HIVE_TABLES }),
        "Command_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(624), tables: &HIVE_TABLES }),
        "Command_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(625), tables: &HIVE_TABLES }),
        "CommentClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(626), tables: &HIVE_TABLES }),
        "CommentGrammar" => Some(RootGrammar { grammar_id: GrammarId(629), tables: &HIVE_TABLES }),
        "CommentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(632), tables: &HIVE_TABLES }),
        "CommentSegment" => Some(RootGrammar { grammar_id: GrammarId(633), tables: &HIVE_TABLES }),
        "CommitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(634), tables: &HIVE_TABLES }),
        "CommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(635), tables: &HIVE_TABLES }),
        "CompactKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(636), tables: &HIVE_TABLES }),
        "CompactionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(637), tables: &HIVE_TABLES }),
        "ComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(638), tables: &HIVE_TABLES }),
        "ComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(647), tables: &HIVE_TABLES }),
        "CompletionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(648), tables: &HIVE_TABLES }),
        "CompositeBinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(649), tables: &HIVE_TABLES }),
        "CompositeComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(650), tables: &HIVE_TABLES }),
        "CompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(651), tables: &HIVE_TABLES }),
        "ComputeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(652), tables: &HIVE_TABLES }),
        "ConcatSegment" => Some(RootGrammar { grammar_id: GrammarId(653), tables: &HIVE_TABLES }),
        "ConcatenateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(656), tables: &HIVE_TABLES }),
        "ConditionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(657), tables: &HIVE_TABLES }),
        "Condition_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(658), tables: &HIVE_TABLES }),
        "ConditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(659), tables: &HIVE_TABLES }),
        "ConditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(660), tables: &HIVE_TABLES }),
        "ConfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(664), tables: &HIVE_TABLES }),
        "ConnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(665), tables: &HIVE_TABLES }),
        "ConnectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(666), tables: &HIVE_TABLES }),
        "Connection_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(667), tables: &HIVE_TABLES }),
        "ConstraintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(668), tables: &HIVE_TABLES }),
        "Constraint_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(669), tables: &HIVE_TABLES }),
        "Constraint_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(670), tables: &HIVE_TABLES }),
        "Constraint_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(671), tables: &HIVE_TABLES }),
        "ConstraintsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(672), tables: &HIVE_TABLES }),
        "ConstructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(673), tables: &HIVE_TABLES }),
        "ContainsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(674), tables: &HIVE_TABLES }),
        "ContainstableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(675), tables: &HIVE_TABLES }),
        "ContinueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(676), tables: &HIVE_TABLES }),
        "ConversionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(677), tables: &HIVE_TABLES }),
        "ConvertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(678), tables: &HIVE_TABLES }),
        "CopyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(679), tables: &HIVE_TABLES }),
        "CorrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(680), tables: &HIVE_TABLES }),
        "CorrespondingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(681), tables: &HIVE_TABLES }),
        "CountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(682), tables: &HIVE_TABLES }),
        "Covar_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(683), tables: &HIVE_TABLES }),
        "Covar_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(684), tables: &HIVE_TABLES }),
        "CreateCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(685), tables: &HIVE_TABLES }),
        "CreateDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(712), tables: &HIVE_TABLES }),
        "CreateFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(728), tables: &HIVE_TABLES }),
        "CreateIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(740), tables: &HIVE_TABLES }),
        "CreateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(753), tables: &HIVE_TABLES }),
        "CreateModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(754), tables: &HIVE_TABLES }),
        "CreateRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(774), tables: &HIVE_TABLES }),
        "CreateSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(779), tables: &HIVE_TABLES }),
        "CreateSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(784), tables: &HIVE_TABLES }),
        "CreateSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(804), tables: &HIVE_TABLES }),
        "CreateTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(810), tables: &HIVE_TABLES }),
        "CreateTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(869), tables: &HIVE_TABLES }),
        "CreateUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(936), tables: &HIVE_TABLES }),
        "CreateViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(940), tables: &HIVE_TABLES }),
        "CreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(952), tables: &HIVE_TABLES }),
        "CreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(953), tables: &HIVE_TABLES }),
        "CreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(954), tables: &HIVE_TABLES }),
        "CrossKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(955), tables: &HIVE_TABLES }),
        "CsvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(956), tables: &HIVE_TABLES }),
        "CubeFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(957), tables: &HIVE_TABLES }),
        "CubeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(958), tables: &HIVE_TABLES }),
        "CubeRollupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(959), tables: &HIVE_TABLES }),
        "Cume_distKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(965), tables: &HIVE_TABLES }),
        "CurrentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(966), tables: &HIVE_TABLES }),
        "Current_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(967), tables: &HIVE_TABLES }),
        "Current_default_transform_groupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(968), tables: &HIVE_TABLES }),
        "Current_pathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(969), tables: &HIVE_TABLES }),
        "Current_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(970), tables: &HIVE_TABLES }),
        "Current_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(971), tables: &HIVE_TABLES }),
        "Current_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(972), tables: &HIVE_TABLES }),
        "Current_transform_group_for_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(973), tables: &HIVE_TABLES }),
        "Current_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(974), tables: &HIVE_TABLES }),
        "CursorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(975), tables: &HIVE_TABLES }),
        "Cursor_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(976), tables: &HIVE_TABLES }),
        "CycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(977), tables: &HIVE_TABLES }),
        "DataKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(978), tables: &HIVE_TABLES }),
        "DatabaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(979), tables: &HIVE_TABLES }),
        "DatabaseReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(613), tables: &HIVE_TABLES }),
        "DatabasesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(980), tables: &HIVE_TABLES }),
        "DatatypeIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(981), tables: &HIVE_TABLES }),
        "DatatypeSegment" => Some(RootGrammar { grammar_id: GrammarId(985), tables: &HIVE_TABLES }),
        "DateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1002), tables: &HIVE_TABLES }),
        "DatePartFunctionName" => Some(RootGrammar { grammar_id: GrammarId(1003), tables: &HIVE_TABLES }),
        "DatePartFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1004), tables: &HIVE_TABLES }),
        "DateTimeFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1005), tables: &HIVE_TABLES }),
        "DateTimeLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(1011), tables: &HIVE_TABLES }),
        "DatetimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1018), tables: &HIVE_TABLES }),
        "DatetimeUnitSegment" => Some(RootGrammar { grammar_id: GrammarId(1019), tables: &HIVE_TABLES }),
        "Datetime_interval_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1020), tables: &HIVE_TABLES }),
        "Datetime_interval_precisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1021), tables: &HIVE_TABLES }),
        "DayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1022), tables: &HIVE_TABLES }),
        "Day_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1023), tables: &HIVE_TABLES }),
        "Day_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1024), tables: &HIVE_TABLES }),
        "Day_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1025), tables: &HIVE_TABLES }),
        "Day_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1026), tables: &HIVE_TABLES }),
        "DayofmonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1027), tables: &HIVE_TABLES }),
        "DayofweekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1028), tables: &HIVE_TABLES }),
        "DayofyearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1029), tables: &HIVE_TABLES }),
        "DaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1030), tables: &HIVE_TABLES }),
        "DbccKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1031), tables: &HIVE_TABLES }),
        "DbpropertiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1032), tables: &HIVE_TABLES }),
        "DeallocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1033), tables: &HIVE_TABLES }),
        "DecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1034), tables: &HIVE_TABLES }),
        "DecimalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1035), tables: &HIVE_TABLES }),
        "DeclareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1036), tables: &HIVE_TABLES }),
        "Dedent" => Some(RootGrammar { grammar_id: GrammarId(230), tables: &HIVE_TABLES }),
        "DefaultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1037), tables: &HIVE_TABLES }),
        "DefaultValuesGrammar" => Some(RootGrammar { grammar_id: GrammarId(1038), tables: &HIVE_TABLES }),
        "DefaultsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1041), tables: &HIVE_TABLES }),
        "DeferrableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1042), tables: &HIVE_TABLES }),
        "DeferredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1043), tables: &HIVE_TABLES }),
        "DefinedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1044), tables: &HIVE_TABLES }),
        "DefinerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1045), tables: &HIVE_TABLES }),
        "DegreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1046), tables: &HIVE_TABLES }),
        "Delay_key_writeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1047), tables: &HIVE_TABLES }),
        "DelayedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1048), tables: &HIVE_TABLES }),
        "DeleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1049), tables: &HIVE_TABLES }),
        "DeleteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1050), tables: &HIVE_TABLES }),
        "DelimitedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1054), tables: &HIVE_TABLES }),
        "DelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(1055), tables: &HIVE_TABLES }),
        "DelimiterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1056), tables: &HIVE_TABLES }),
        "DelimitersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1057), tables: &HIVE_TABLES }),
        "Dense_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1058), tables: &HIVE_TABLES }),
        "DenyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1059), tables: &HIVE_TABLES }),
        "DependencyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1060), tables: &HIVE_TABLES }),
        "DepthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1061), tables: &HIVE_TABLES }),
        "DerefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1062), tables: &HIVE_TABLES }),
        "DerivedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1063), tables: &HIVE_TABLES }),
        "DescKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1064), tables: &HIVE_TABLES }),
        "DescribeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1065), tables: &HIVE_TABLES }),
        "DescribeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1066), tables: &HIVE_TABLES }),
        "DescriptorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1070), tables: &HIVE_TABLES }),
        "DestroyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1071), tables: &HIVE_TABLES }),
        "DestructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1072), tables: &HIVE_TABLES }),
        "DetailKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1073), tables: &HIVE_TABLES }),
        "DeterministicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1074), tables: &HIVE_TABLES }),
        "DiagnosticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1075), tables: &HIVE_TABLES }),
        "DictionaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1076), tables: &HIVE_TABLES }),
        "DirectoriesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1077), tables: &HIVE_TABLES }),
        "DirectoryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1078), tables: &HIVE_TABLES }),
        "DisableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1079), tables: &HIVE_TABLES }),
        "DisconnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1080), tables: &HIVE_TABLES }),
        "DiskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1081), tables: &HIVE_TABLES }),
        "DispatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1082), tables: &HIVE_TABLES }),
        "DistinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1083), tables: &HIVE_TABLES }),
        "DistinctrowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1084), tables: &HIVE_TABLES }),
        "DistributeByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1085), tables: &HIVE_TABLES }),
        "DistributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1101), tables: &HIVE_TABLES }),
        "DistributedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1102), tables: &HIVE_TABLES }),
        "DivKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1103), tables: &HIVE_TABLES }),
        "DivideSegment" => Some(RootGrammar { grammar_id: GrammarId(1104), tables: &HIVE_TABLES }),
        "DoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1105), tables: &HIVE_TABLES }),
        "DomainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1106), tables: &HIVE_TABLES }),
        "DotSegment" => Some(RootGrammar { grammar_id: GrammarId(1107), tables: &HIVE_TABLES }),
        "DoubleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1108), tables: &HIVE_TABLES }),
        "DowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1109), tables: &HIVE_TABLES }),
        "DropBehaviorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1110), tables: &HIVE_TABLES }),
        "DropCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1113), tables: &HIVE_TABLES }),
        "DropDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1121), tables: &HIVE_TABLES }),
        "DropFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1127), tables: &HIVE_TABLES }),
        "DropIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1132), tables: &HIVE_TABLES }),
        "DropKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1138), tables: &HIVE_TABLES }),
        "DropModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1139), tables: &HIVE_TABLES }),
        "DropRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1144), tables: &HIVE_TABLES }),
        "DropSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1149), tables: &HIVE_TABLES }),
        "DropSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1155), tables: &HIVE_TABLES }),
        "DropTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1159), tables: &HIVE_TABLES }),
        "DropTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1165), tables: &HIVE_TABLES }),
        "DropTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1170), tables: &HIVE_TABLES }),
        "DropUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1176), tables: &HIVE_TABLES }),
        "DropViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1181), tables: &HIVE_TABLES }),
        "DummyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1187), tables: &HIVE_TABLES }),
        "DumpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1188), tables: &HIVE_TABLES }),
        "DynamicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1189), tables: &HIVE_TABLES }),
        "Dynamic_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1190), tables: &HIVE_TABLES }),
        "Dynamic_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1191), tables: &HIVE_TABLES }),
        "EachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1192), tables: &HIVE_TABLES }),
        "Elem_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1193), tables: &HIVE_TABLES }),
        "ElementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1194), tables: &HIVE_TABLES }),
        "ElseClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1195), tables: &HIVE_TABLES }),
        "ElseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1198), tables: &HIVE_TABLES }),
        "ElseifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1199), tables: &HIVE_TABLES }),
        "EmptyStructLiteralBracketsSegment" => Some(RootGrammar { grammar_id: GrammarId(1200), tables: &HIVE_TABLES }),
        "EmptyStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(1201), tables: &HIVE_TABLES }),
        "EnableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1204), tables: &HIVE_TABLES }),
        "EnclosedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1205), tables: &HIVE_TABLES }),
        "EncodingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1206), tables: &HIVE_TABLES }),
        "EncryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1207), tables: &HIVE_TABLES }),
        "End-execKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1208), tables: &HIVE_TABLES }),
        "EndAngleBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(378), tables: &HIVE_TABLES }),
        "EndBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(228), tables: &HIVE_TABLES }),
        "EndCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(1209), tables: &HIVE_TABLES }),
        "EndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1210), tables: &HIVE_TABLES }),
        "EndSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(367), tables: &HIVE_TABLES }),
        "EnumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1211), tables: &HIVE_TABLES }),
        "EqualsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1212), tables: &HIVE_TABLES }),
        "EqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(1213), tables: &HIVE_TABLES }),
        "ErrlvlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1216), tables: &HIVE_TABLES }),
        "EscapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1217), tables: &HIVE_TABLES }),
        "EscapedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1218), tables: &HIVE_TABLES }),
        "EveryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1219), tables: &HIVE_TABLES }),
        "ExceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1220), tables: &HIVE_TABLES }),
        "ExceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1221), tables: &HIVE_TABLES }),
        "ExchangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1222), tables: &HIVE_TABLES }),
        "ExcludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1223), tables: &HIVE_TABLES }),
        "ExcludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1224), tables: &HIVE_TABLES }),
        "ExclusiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1225), tables: &HIVE_TABLES }),
        "ExecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1226), tables: &HIVE_TABLES }),
        "ExecuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1227), tables: &HIVE_TABLES }),
        "ExecutionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1228), tables: &HIVE_TABLES }),
        "ExistingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1229), tables: &HIVE_TABLES }),
        "ExistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1230), tables: &HIVE_TABLES }),
        "ExitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1231), tables: &HIVE_TABLES }),
        "ExpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1232), tables: &HIVE_TABLES }),
        "ExplainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1233), tables: &HIVE_TABLES }),
        "ExplainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1234), tables: &HIVE_TABLES }),
        "ExportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1241), tables: &HIVE_TABLES }),
        "ExpressionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1242), tables: &HIVE_TABLES }),
        "ExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1243), tables: &HIVE_TABLES }),
        "Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1244), tables: &HIVE_TABLES }),
        "Expression_A_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1270), tables: &HIVE_TABLES }),
        "Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1277), tables: &HIVE_TABLES }),
        "Expression_B_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1287), tables: &HIVE_TABLES }),
        "Expression_C_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1292), tables: &HIVE_TABLES }),
        "Expression_D_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1305), tables: &HIVE_TABLES }),
        "Expression_D_Potential_Select_Statement_Without_Brackets" => Some(RootGrammar { grammar_id: GrammarId(1342), tables: &HIVE_TABLES }),
        "ExtendedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1350), tables: &HIVE_TABLES }),
        "ExtendedNaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1351), tables: &HIVE_TABLES }),
        "ExtensionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1352), tables: &HIVE_TABLES }),
        "ExtensionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1353), tables: &HIVE_TABLES }),
        "ExternalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1357), tables: &HIVE_TABLES }),
        "ExtractKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1358), tables: &HIVE_TABLES }),
        "FalseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1359), tables: &HIVE_TABLES }),
        "FalseSegment" => Some(RootGrammar { grammar_id: GrammarId(1360), tables: &HIVE_TABLES }),
        "FetchClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1361), tables: &HIVE_TABLES }),
        "FetchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1378), tables: &HIVE_TABLES }),
        "FieldsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1379), tables: &HIVE_TABLES }),
        "FileFormatGrammar" => Some(RootGrammar { grammar_id: GrammarId(1380), tables: &HIVE_TABLES }),
        "FileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1393), tables: &HIVE_TABLES }),
        "FileSegment" => Some(RootGrammar { grammar_id: GrammarId(1394), tables: &HIVE_TABLES }),
        "FileformatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1403), tables: &HIVE_TABLES }),
        "FillfactorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1404), tables: &HIVE_TABLES }),
        "FilterClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1405), tables: &HIVE_TABLES }),
        "FilterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1411), tables: &HIVE_TABLES }),
        "FinalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1412), tables: &HIVE_TABLES }),
        "FirstKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1413), tables: &HIVE_TABLES }),
        "Float4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1414), tables: &HIVE_TABLES }),
        "Float8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1415), tables: &HIVE_TABLES }),
        "FloatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1416), tables: &HIVE_TABLES }),
        "FloorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1417), tables: &HIVE_TABLES }),
        "FlushKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1418), tables: &HIVE_TABLES }),
        "FollowingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1419), tables: &HIVE_TABLES }),
        "ForKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1420), tables: &HIVE_TABLES }),
        "ForceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1421), tables: &HIVE_TABLES }),
        "ForeignKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(1422), tables: &HIVE_TABLES }),
        "ForeignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1425), tables: &HIVE_TABLES }),
        "FormatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1426), tables: &HIVE_TABLES }),
        "FormattedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1427), tables: &HIVE_TABLES }),
        "FortranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1428), tables: &HIVE_TABLES }),
        "ForwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1429), tables: &HIVE_TABLES }),
        "FoundKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1430), tables: &HIVE_TABLES }),
        "FrameClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1431), tables: &HIVE_TABLES }),
        "FrameClauseUnitGrammar" => Some(RootGrammar { grammar_id: GrammarId(1450), tables: &HIVE_TABLES }),
        "FreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1453), tables: &HIVE_TABLES }),
        "FreetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1454), tables: &HIVE_TABLES }),
        "FreetexttableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1455), tables: &HIVE_TABLES }),
        "FreezeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1456), tables: &HIVE_TABLES }),
        "FromClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1457), tables: &HIVE_TABLES }),
        "FromClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1461), tables: &HIVE_TABLES }),
        "FromExpressionElementSegment" => Some(RootGrammar { grammar_id: GrammarId(1487), tables: &HIVE_TABLES }),
        "FromExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1508), tables: &HIVE_TABLES }),
        "FromInsertClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1535), tables: &HIVE_TABLES }),
        "FromInsertSourceClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1567), tables: &HIVE_TABLES }),
        "FromInsertStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1571), tables: &HIVE_TABLES }),
        "FromKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1575), tables: &HIVE_TABLES }),
        "FullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1576), tables: &HIVE_TABLES }),
        "FulltextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1577), tables: &HIVE_TABLES }),
        "FunctionContentsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1578), tables: &HIVE_TABLES }),
        "FunctionContentsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1579), tables: &HIVE_TABLES }),
        "FunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1620), tables: &HIVE_TABLES }),
        "FunctionDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1623), tables: &HIVE_TABLES }),
        "FunctionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1629), tables: &HIVE_TABLES }),
        "FunctionNameIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1630), tables: &HIVE_TABLES }),
        "FunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1631), tables: &HIVE_TABLES }),
        "FunctionParameterGrammar" => Some(RootGrammar { grammar_id: GrammarId(1641), tables: &HIVE_TABLES }),
        "FunctionParameterListGrammar" => Some(RootGrammar { grammar_id: GrammarId(1654), tables: &HIVE_TABLES }),
        "FunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(1657), tables: &HIVE_TABLES }),
        "FunctionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1680), tables: &HIVE_TABLES }),
        "FusionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1681), tables: &HIVE_TABLES }),
        "FutureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1682), tables: &HIVE_TABLES }),
        "GKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1683), tables: &HIVE_TABLES }),
        "GeneralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1684), tables: &HIVE_TABLES }),
        "GeneratedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1685), tables: &HIVE_TABLES }),
        "GetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1686), tables: &HIVE_TABLES }),
        "GlobOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1687), tables: &HIVE_TABLES }),
        "GlobalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1688), tables: &HIVE_TABLES }),
        "GoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1689), tables: &HIVE_TABLES }),
        "GotoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1690), tables: &HIVE_TABLES }),
        "GrantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1691), tables: &HIVE_TABLES }),
        "GrantStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1692), tables: &HIVE_TABLES }),
        "GrantedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1730), tables: &HIVE_TABLES }),
        "GrantsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1731), tables: &HIVE_TABLES }),
        "GreaterThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(1732), tables: &HIVE_TABLES }),
        "GreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(1735), tables: &HIVE_TABLES }),
        "GreatestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1736), tables: &HIVE_TABLES }),
        "GroupByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1737), tables: &HIVE_TABLES }),
        "GroupByClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1752), tables: &HIVE_TABLES }),
        "GroupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1764), tables: &HIVE_TABLES }),
        "GroupingExpressionList" => Some(RootGrammar { grammar_id: GrammarId(1765), tables: &HIVE_TABLES }),
        "GroupingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1773), tables: &HIVE_TABLES }),
        "GroupingSetsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1774), tables: &HIVE_TABLES }),
        "HandlerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1782), tables: &HIVE_TABLES }),
        "HavingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1783), tables: &HIVE_TABLES }),
        "HavingClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1788), tables: &HIVE_TABLES }),
        "HavingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1799), tables: &HIVE_TABLES }),
        "HeaderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1800), tables: &HIVE_TABLES }),
        "HeapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1801), tables: &HIVE_TABLES }),
        "HierarchyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1802), tables: &HIVE_TABLES }),
        "High_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1803), tables: &HIVE_TABLES }),
        "HiveObjectReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(613), tables: &HIVE_TABLES }),
        "HiveReferenceIdentifierGrammar" => Some(RootGrammar { grammar_id: GrammarId(1804), tables: &HIVE_TABLES }),
        "HiveSetValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(1810), tables: &HIVE_TABLES }),
        "HiveVariableSegment" => Some(RootGrammar { grammar_id: GrammarId(1815), tables: &HIVE_TABLES }),
        "HoldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1816), tables: &HIVE_TABLES }),
        "Hold_ddltimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1817), tables: &HIVE_TABLES }),
        "HoldlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1818), tables: &HIVE_TABLES }),
        "HorizontalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1819), tables: &HIVE_TABLES }),
        "HostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1820), tables: &HIVE_TABLES }),
        "HostsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1821), tables: &HIVE_TABLES }),
        "HourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1822), tables: &HIVE_TABLES }),
        "Hour_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1823), tables: &HIVE_TABLES }),
        "Hour_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1824), tables: &HIVE_TABLES }),
        "Hour_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1825), tables: &HIVE_TABLES }),
        "HoursKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1826), tables: &HIVE_TABLES }),
        "IdentifiedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1827), tables: &HIVE_TABLES }),
        "IdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1828), tables: &HIVE_TABLES }),
        "IdentityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1829), tables: &HIVE_TABLES }),
        "Identity_insertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1830), tables: &HIVE_TABLES }),
        "IdentitycolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1831), tables: &HIVE_TABLES }),
        "IdxpropertiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1832), tables: &HIVE_TABLES }),
        "IfExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1833), tables: &HIVE_TABLES }),
        "IfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1836), tables: &HIVE_TABLES }),
        "IfNotExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1837), tables: &HIVE_TABLES }),
        "IgnoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1841), tables: &HIVE_TABLES }),
        "IgnoreRespectNullsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1842), tables: &HIVE_TABLES }),
        "IlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1847), tables: &HIVE_TABLES }),
        "ImmediateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1848), tables: &HIVE_TABLES }),
        "ImmutableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1849), tables: &HIVE_TABLES }),
        "ImplementationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1850), tables: &HIVE_TABLES }),
        "ImplicitIndent" => Some(RootGrammar { grammar_id: GrammarId(489), tables: &HIVE_TABLES }),
        "ImplicitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1851), tables: &HIVE_TABLES }),
        "ImportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1852), tables: &HIVE_TABLES }),
        "ImportedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1853), tables: &HIVE_TABLES }),
        "InKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1854), tables: &HIVE_TABLES }),
        "InOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1855), tables: &HIVE_TABLES }),
        "IncludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1866), tables: &HIVE_TABLES }),
        "IncludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1867), tables: &HIVE_TABLES }),
        "IncrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1868), tables: &HIVE_TABLES }),
        "Indent" => Some(RootGrammar { grammar_id: GrammarId(220), tables: &HIVE_TABLES }),
        "IndexColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(1869), tables: &HIVE_TABLES }),
        "IndexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1874), tables: &HIVE_TABLES }),
        "IndexReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1353), tables: &HIVE_TABLES }),
        "IndexesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1875), tables: &HIVE_TABLES }),
        "IndicatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1876), tables: &HIVE_TABLES }),
        "InfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1877), tables: &HIVE_TABLES }),
        "InfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1878), tables: &HIVE_TABLES }),
        "InheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1879), tables: &HIVE_TABLES }),
        "InheritsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1880), tables: &HIVE_TABLES }),
        "InitialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1881), tables: &HIVE_TABLES }),
        "InitializeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1882), tables: &HIVE_TABLES }),
        "InitiallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1883), tables: &HIVE_TABLES }),
        "InnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1884), tables: &HIVE_TABLES }),
        "InoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1885), tables: &HIVE_TABLES }),
        "InpathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1886), tables: &HIVE_TABLES }),
        "InputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1887), tables: &HIVE_TABLES }),
        "InputdriverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1888), tables: &HIVE_TABLES }),
        "InputformatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1889), tables: &HIVE_TABLES }),
        "InsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1890), tables: &HIVE_TABLES }),
        "InsertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1891), tables: &HIVE_TABLES }),
        "InsertStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1892), tables: &HIVE_TABLES }),
        "Insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1921), tables: &HIVE_TABLES }),
        "InstanceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1922), tables: &HIVE_TABLES }),
        "InstantiableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1923), tables: &HIVE_TABLES }),
        "InsteadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1924), tables: &HIVE_TABLES }),
        "Int1KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1925), tables: &HIVE_TABLES }),
        "Int2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1926), tables: &HIVE_TABLES }),
        "Int3KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1927), tables: &HIVE_TABLES }),
        "Int4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1928), tables: &HIVE_TABLES }),
        "Int8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1929), tables: &HIVE_TABLES }),
        "IntKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1930), tables: &HIVE_TABLES }),
        "IntegerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1931), tables: &HIVE_TABLES }),
        "IntegrationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1932), tables: &HIVE_TABLES }),
        "IntegrationsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1933), tables: &HIVE_TABLES }),
        "IntersectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1934), tables: &HIVE_TABLES }),
        "IntersectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1935), tables: &HIVE_TABLES }),
        "IntervalExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1936), tables: &HIVE_TABLES }),
        "IntervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1949), tables: &HIVE_TABLES }),
        "IntoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1950), tables: &HIVE_TABLES }),
        "InvokerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1951), tables: &HIVE_TABLES }),
        "IregexpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1952), tables: &HIVE_TABLES }),
        "IsClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1953), tables: &HIVE_TABLES }),
        "IsDistinctFromGrammar" => Some(RootGrammar { grammar_id: GrammarId(1959), tables: &HIVE_TABLES }),
        "IsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1964), tables: &HIVE_TABLES }),
        "IsNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(1965), tables: &HIVE_TABLES }),
        "IsamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1966), tables: &HIVE_TABLES }),
        "IsnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1967), tables: &HIVE_TABLES }),
        "IsolationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1968), tables: &HIVE_TABLES }),
        "ItemsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1969), tables: &HIVE_TABLES }),
        "IterateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1970), tables: &HIVE_TABLES }),
        "JarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1971), tables: &HIVE_TABLES }),
        "JoinClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1972), tables: &HIVE_TABLES }),
        "JoinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1994), tables: &HIVE_TABLES }),
        "JoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1995), tables: &HIVE_TABLES }),
        "JoinLikeClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1999), tables: &HIVE_TABLES }),
        "JoinOnConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(2000), tables: &HIVE_TABLES }),
        "JoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2007), tables: &HIVE_TABLES }),
        "JoinUsingConditionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2015), tables: &HIVE_TABLES }),
        "JsonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2020), tables: &HIVE_TABLES }),
        "JsonfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2021), tables: &HIVE_TABLES }),
        "KKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2022), tables: &HIVE_TABLES }),
        "KeyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2023), tables: &HIVE_TABLES }),
        "Key_memberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2024), tables: &HIVE_TABLES }),
        "Key_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2025), tables: &HIVE_TABLES }),
        "KeysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2026), tables: &HIVE_TABLES }),
        "KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2027), tables: &HIVE_TABLES }),
        "KillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2028), tables: &HIVE_TABLES }),
        "LancompilerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2029), tables: &HIVE_TABLES }),
        "LanguageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2030), tables: &HIVE_TABLES }),
        "LargeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2031), tables: &HIVE_TABLES }),
        "LastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2032), tables: &HIVE_TABLES }),
        "Last_insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2033), tables: &HIVE_TABLES }),
        "LateralColumnAliasExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2034), tables: &HIVE_TABLES }),
        "LateralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2035), tables: &HIVE_TABLES }),
        "LateralViewClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2036), tables: &HIVE_TABLES }),
        "LeadingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2046), tables: &HIVE_TABLES }),
        "LeastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2047), tables: &HIVE_TABLES }),
        "LeaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2048), tables: &HIVE_TABLES }),
        "LeftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2049), tables: &HIVE_TABLES }),
        "LengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2050), tables: &HIVE_TABLES }),
        "LessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2051), tables: &HIVE_TABLES }),
        "LessThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(2052), tables: &HIVE_TABLES }),
        "LessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2055), tables: &HIVE_TABLES }),
        "LevelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2056), tables: &HIVE_TABLES }),
        "LikeExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2057), tables: &HIVE_TABLES }),
        "LikeGrammar" => Some(RootGrammar { grammar_id: GrammarId(2065), tables: &HIVE_TABLES }),
        "LikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2071), tables: &HIVE_TABLES }),
        "LikeOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2072), tables: &HIVE_TABLES }),
        "LimitClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2073), tables: &HIVE_TABLES }),
        "LimitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2090), tables: &HIVE_TABLES }),
        "LinenoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2091), tables: &HIVE_TABLES }),
        "LinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2092), tables: &HIVE_TABLES }),
        "ListComprehensionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2093), tables: &HIVE_TABLES }),
        "ListenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2094), tables: &HIVE_TABLES }),
        "LiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(2095), tables: &HIVE_TABLES }),
        "LiteralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2106), tables: &HIVE_TABLES }),
        "LiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2107), tables: &HIVE_TABLES }),
        "LnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2108), tables: &HIVE_TABLES }),
        "LoadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2109), tables: &HIVE_TABLES }),
        "LocalAliasSegment" => Some(RootGrammar { grammar_id: GrammarId(2110), tables: &HIVE_TABLES }),
        "LocalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2111), tables: &HIVE_TABLES }),
        "LocaltimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2112), tables: &HIVE_TABLES }),
        "LocaltimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2113), tables: &HIVE_TABLES }),
        "LocationGrammar" => Some(RootGrammar { grammar_id: GrammarId(2114), tables: &HIVE_TABLES }),
        "LocationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2117), tables: &HIVE_TABLES }),
        "LocatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2118), tables: &HIVE_TABLES }),
        "LockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2119), tables: &HIVE_TABLES }),
        "LocksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2120), tables: &HIVE_TABLES }),
        "LogicalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2121), tables: &HIVE_TABLES }),
        "LoginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2122), tables: &HIVE_TABLES }),
        "LogsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2123), tables: &HIVE_TABLES }),
        "LongKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2124), tables: &HIVE_TABLES }),
        "LongblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2125), tables: &HIVE_TABLES }),
        "LongtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2126), tables: &HIVE_TABLES }),
        "LoopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2127), tables: &HIVE_TABLES }),
        "Low_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2128), tables: &HIVE_TABLES }),
        "LowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2129), tables: &HIVE_TABLES }),
        "MKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2130), tables: &HIVE_TABLES }),
        "MLTableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2131), tables: &HIVE_TABLES }),
        "MacroKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2143), tables: &HIVE_TABLES }),
        "ManageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2144), tables: &HIVE_TABLES }),
        "ManagedlocationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2145), tables: &HIVE_TABLES }),
        "MapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2146), tables: &HIVE_TABLES }),
        "MapTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2147), tables: &HIVE_TABLES }),
        "MapjoinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2148), tables: &HIVE_TABLES }),
        "MaskingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2149), tables: &HIVE_TABLES }),
        "MatchConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(2150), tables: &HIVE_TABLES }),
        "MatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2151), tables: &HIVE_TABLES }),
        "MatchedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2152), tables: &HIVE_TABLES }),
        "MaterializedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2153), tables: &HIVE_TABLES }),
        "MaxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2154), tables: &HIVE_TABLES }),
        "Max_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2155), tables: &HIVE_TABLES }),
        "MaxextentsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2156), tables: &HIVE_TABLES }),
        "MaxvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2157), tables: &HIVE_TABLES }),
        "MediumblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2158), tables: &HIVE_TABLES }),
        "MediumintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2159), tables: &HIVE_TABLES }),
        "MediumtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2160), tables: &HIVE_TABLES }),
        "MemberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2161), tables: &HIVE_TABLES }),
        "MergeDeleteClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2162), tables: &HIVE_TABLES }),
        "MergeInsertClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2163), tables: &HIVE_TABLES }),
        "MergeIntoLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(2167), tables: &HIVE_TABLES }),
        "MergeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2170), tables: &HIVE_TABLES }),
        "MergeMatchSegment" => Some(RootGrammar { grammar_id: GrammarId(2171), tables: &HIVE_TABLES }),
        "MergeMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2174), tables: &HIVE_TABLES }),
        "MergeNotMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2184), tables: &HIVE_TABLES }),
        "MergeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2193), tables: &HIVE_TABLES }),
        "MergeUpdateClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2213), tables: &HIVE_TABLES }),
        "Message_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2216), tables: &HIVE_TABLES }),
        "Message_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2217), tables: &HIVE_TABLES }),
        "Message_textKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2218), tables: &HIVE_TABLES }),
        "MetadataKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2219), tables: &HIVE_TABLES }),
        "MethodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2220), tables: &HIVE_TABLES }),
        "MiddleintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2221), tables: &HIVE_TABLES }),
        "MillisecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2222), tables: &HIVE_TABLES }),
        "MinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2223), tables: &HIVE_TABLES }),
        "Min_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2224), tables: &HIVE_TABLES }),
        "MinusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2225), tables: &HIVE_TABLES }),
        "MinusSegment" => Some(RootGrammar { grammar_id: GrammarId(2226), tables: &HIVE_TABLES }),
        "MinuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2227), tables: &HIVE_TABLES }),
        "Minute_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2228), tables: &HIVE_TABLES }),
        "Minute_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2229), tables: &HIVE_TABLES }),
        "MinutesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2230), tables: &HIVE_TABLES }),
        "MinvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2231), tables: &HIVE_TABLES }),
        "MlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2232), tables: &HIVE_TABLES }),
        "MlslabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2233), tables: &HIVE_TABLES }),
        "ModKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2234), tables: &HIVE_TABLES }),
        "ModeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2235), tables: &HIVE_TABLES }),
        "ModelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2236), tables: &HIVE_TABLES }),
        "ModifiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2237), tables: &HIVE_TABLES }),
        "ModifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2238), tables: &HIVE_TABLES }),
        "ModuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2239), tables: &HIVE_TABLES }),
        "ModuloSegment" => Some(RootGrammar { grammar_id: GrammarId(2240), tables: &HIVE_TABLES }),
        "MonitorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2241), tables: &HIVE_TABLES }),
        "MonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2242), tables: &HIVE_TABLES }),
        "MonthnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2243), tables: &HIVE_TABLES }),
        "MonthsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2244), tables: &HIVE_TABLES }),
        "MoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2245), tables: &HIVE_TABLES }),
        "MoveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2246), tables: &HIVE_TABLES }),
        "MsckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2247), tables: &HIVE_TABLES }),
        "MsckRepairTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2248), tables: &HIVE_TABLES }),
        "MsckTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2259), tables: &HIVE_TABLES }),
        "MultiplySegment" => Some(RootGrammar { grammar_id: GrammarId(2269), tables: &HIVE_TABLES }),
        "MultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2270), tables: &HIVE_TABLES }),
        "MumpsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2271), tables: &HIVE_TABLES }),
        "MyisamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2272), tables: &HIVE_TABLES }),
        "NakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2273), tables: &HIVE_TABLES }),
        "NameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2274), tables: &HIVE_TABLES }),
        "NamedWindowExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2275), tables: &HIVE_TABLES }),
        "NamedWindowSegment" => Some(RootGrammar { grammar_id: GrammarId(2282), tables: &HIVE_TABLES }),
        "NamesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2286), tables: &HIVE_TABLES }),
        "NanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2287), tables: &HIVE_TABLES }),
        "NanLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2288), tables: &HIVE_TABLES }),
        "NationalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2289), tables: &HIVE_TABLES }),
        "NaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2290), tables: &HIVE_TABLES }),
        "NaturalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2293), tables: &HIVE_TABLES }),
        "NcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2294), tables: &HIVE_TABLES }),
        "NclobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2295), tables: &HIVE_TABLES }),
        "NegativeSegment" => Some(RootGrammar { grammar_id: GrammarId(2296), tables: &HIVE_TABLES }),
        "NestedJoinGrammar" => Some(RootGrammar { grammar_id: GrammarId(2297), tables: &HIVE_TABLES }),
        "NestingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2298), tables: &HIVE_TABLES }),
        "NewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2299), tables: &HIVE_TABLES }),
        "NewlineSegment" => Some(RootGrammar { grammar_id: GrammarId(2300), tables: &HIVE_TABLES }),
        "NextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2301), tables: &HIVE_TABLES }),
        "NoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2302), tables: &HIVE_TABLES }),
        "No_dropKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2303), tables: &HIVE_TABLES }),
        "No_write_to_binlogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2304), tables: &HIVE_TABLES }),
        "NoauditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2305), tables: &HIVE_TABLES }),
        "NocacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2306), tables: &HIVE_TABLES }),
        "NocheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2307), tables: &HIVE_TABLES }),
        "NocompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2308), tables: &HIVE_TABLES }),
        "NocreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2309), tables: &HIVE_TABLES }),
        "NocreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2310), tables: &HIVE_TABLES }),
        "NocreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2311), tables: &HIVE_TABLES }),
        "NocycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2312), tables: &HIVE_TABLES }),
        "NoinheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2313), tables: &HIVE_TABLES }),
        "NologinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2314), tables: &HIVE_TABLES }),
        "NonSetSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2315), tables: &HIVE_TABLES }),
        "NonStandardJoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2325), tables: &HIVE_TABLES }),
        "NonWithNonSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2326), tables: &HIVE_TABLES }),
        "NonWithSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2331), tables: &HIVE_TABLES }),
        "NonclusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2337), tables: &HIVE_TABLES }),
        "NoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2338), tables: &HIVE_TABLES }),
        "NoorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2339), tables: &HIVE_TABLES }),
        "NorelyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2340), tables: &HIVE_TABLES }),
        "NormalizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2341), tables: &HIVE_TABLES }),
        "NormalizedGrammar" => Some(RootGrammar { grammar_id: GrammarId(2342), tables: &HIVE_TABLES }),
        "NormalizedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2343), tables: &HIVE_TABLES }),
        "NoscanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2344), tables: &HIVE_TABLES }),
        "NosuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2345), tables: &HIVE_TABLES }),
        "NotEnforcedGrammar" => Some(RootGrammar { grammar_id: GrammarId(2346), tables: &HIVE_TABLES }),
        "NotEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(2347), tables: &HIVE_TABLES }),
        "NotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2354), tables: &HIVE_TABLES }),
        "NotNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(2355), tables: &HIVE_TABLES }),
        "NotOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2356), tables: &HIVE_TABLES }),
        "NothingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2357), tables: &HIVE_TABLES }),
        "NotifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2358), tables: &HIVE_TABLES }),
        "NotnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2359), tables: &HIVE_TABLES }),
        "NovalidateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2360), tables: &HIVE_TABLES }),
        "NowaitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2361), tables: &HIVE_TABLES }),
        "NullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2362), tables: &HIVE_TABLES }),
        "NullLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2363), tables: &HIVE_TABLES }),
        "NullableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2364), tables: &HIVE_TABLES }),
        "NullifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2365), tables: &HIVE_TABLES }),
        "NullsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2366), tables: &HIVE_TABLES }),
        "NumberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2367), tables: &HIVE_TABLES }),
        "NumericKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2368), tables: &HIVE_TABLES }),
        "NumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2369), tables: &HIVE_TABLES }),
        "ObjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2370), tables: &HIVE_TABLES }),
        "ObjectLiteralElementSegment" => Some(RootGrammar { grammar_id: GrammarId(2371), tables: &HIVE_TABLES }),
        "ObjectLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2375), tables: &HIVE_TABLES }),
        "ObjectReferenceDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(2379), tables: &HIVE_TABLES }),
        "ObjectReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1353), tables: &HIVE_TABLES }),
        "ObjectReferenceTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2384), tables: &HIVE_TABLES }),
        "ObjectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2396), tables: &HIVE_TABLES }),
        "Octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2397), tables: &HIVE_TABLES }),
        "OctetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2398), tables: &HIVE_TABLES }),
        "OfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2399), tables: &HIVE_TABLES }),
        "OffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2400), tables: &HIVE_TABLES }),
        "OfflineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2401), tables: &HIVE_TABLES }),
        "OffsetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2402), tables: &HIVE_TABLES }),
        "OffsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2411), tables: &HIVE_TABLES }),
        "OffsetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2412), tables: &HIVE_TABLES }),
        "OidsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2413), tables: &HIVE_TABLES }),
        "OldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2414), tables: &HIVE_TABLES }),
        "OnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2415), tables: &HIVE_TABLES }),
        "OnlineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2416), tables: &HIVE_TABLES }),
        "OnlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2417), tables: &HIVE_TABLES }),
        "OpenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2418), tables: &HIVE_TABLES }),
        "OpendatasourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2419), tables: &HIVE_TABLES }),
        "OpenqueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2420), tables: &HIVE_TABLES }),
        "OpenrowsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2421), tables: &HIVE_TABLES }),
        "OpenxmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2422), tables: &HIVE_TABLES }),
        "OperateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2423), tables: &HIVE_TABLES }),
        "OperationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2424), tables: &HIVE_TABLES }),
        "OperatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2425), tables: &HIVE_TABLES }),
        "OptimizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2426), tables: &HIVE_TABLES }),
        "OptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2427), tables: &HIVE_TABLES }),
        "OptionallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2428), tables: &HIVE_TABLES }),
        "OptionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2429), tables: &HIVE_TABLES }),
        "OrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2430), tables: &HIVE_TABLES }),
        "OrOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2431), tables: &HIVE_TABLES }),
        "OrReplaceGrammar" => Some(RootGrammar { grammar_id: GrammarId(2432), tables: &HIVE_TABLES }),
        "OrcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2435), tables: &HIVE_TABLES }),
        "OrderByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2436), tables: &HIVE_TABLES }),
        "OrderByClauseTerminators" => Some(RootGrammar { grammar_id: GrammarId(2456), tables: &HIVE_TABLES }),
        "OrderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2464), tables: &HIVE_TABLES }),
        "OrderNoOrderGrammar" => Some(RootGrammar { grammar_id: GrammarId(2465), tables: &HIVE_TABLES }),
        "OrderingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2468), tables: &HIVE_TABLES }),
        "OrdinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2469), tables: &HIVE_TABLES }),
        "OthersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2470), tables: &HIVE_TABLES }),
        "OutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2471), tables: &HIVE_TABLES }),
        "OuterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2472), tables: &HIVE_TABLES }),
        "OutfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2473), tables: &HIVE_TABLES }),
        "OutputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2474), tables: &HIVE_TABLES }),
        "OutputdriverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2475), tables: &HIVE_TABLES }),
        "OutputformatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2476), tables: &HIVE_TABLES }),
        "OverClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2477), tables: &HIVE_TABLES }),
        "OverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2484), tables: &HIVE_TABLES }),
        "OverlapsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2485), tables: &HIVE_TABLES }),
        "OverlapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2494), tables: &HIVE_TABLES }),
        "OverlayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2495), tables: &HIVE_TABLES }),
        "OverridingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2496), tables: &HIVE_TABLES }),
        "OverwriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2497), tables: &HIVE_TABLES }),
        "OwnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2498), tables: &HIVE_TABLES }),
        "OwnershipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2499), tables: &HIVE_TABLES }),
        "Pack_keysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2500), tables: &HIVE_TABLES }),
        "PadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2501), tables: &HIVE_TABLES }),
        "ParameterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2502), tables: &HIVE_TABLES }),
        "ParameterNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2503), tables: &HIVE_TABLES }),
        "ParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(2504), tables: &HIVE_TABLES }),
        "Parameter_modeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2505), tables: &HIVE_TABLES }),
        "Parameter_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2506), tables: &HIVE_TABLES }),
        "Parameter_ordinal_positionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2507), tables: &HIVE_TABLES }),
        "Parameter_specific_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2508), tables: &HIVE_TABLES }),
        "Parameter_specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2509), tables: &HIVE_TABLES }),
        "Parameter_specific_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2510), tables: &HIVE_TABLES }),
        "ParametersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2511), tables: &HIVE_TABLES }),
        "ParquetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2512), tables: &HIVE_TABLES }),
        "PartialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2513), tables: &HIVE_TABLES }),
        "PartialscanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2514), tables: &HIVE_TABLES }),
        "PartitionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2515), tables: &HIVE_TABLES }),
        "PartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2522), tables: &HIVE_TABLES }),
        "PartitionSpecGrammar" => Some(RootGrammar { grammar_id: GrammarId(2523), tables: &HIVE_TABLES }),
        "PartitionedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2532), tables: &HIVE_TABLES }),
        "PartitionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2533), tables: &HIVE_TABLES }),
        "PascalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2534), tables: &HIVE_TABLES }),
        "PasswordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2535), tables: &HIVE_TABLES }),
        "PathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2536), tables: &HIVE_TABLES }),
        "PathSegment" => Some(RootGrammar { grammar_id: GrammarId(2537), tables: &HIVE_TABLES }),
        "PatternMatchingGrammar" => Some(RootGrammar { grammar_id: GrammarId(2544), tables: &HIVE_TABLES }),
        "PctfreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2545), tables: &HIVE_TABLES }),
        "PercentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2546), tables: &HIVE_TABLES }),
        "Percent_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2547), tables: &HIVE_TABLES }),
        "Percentile_contKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2548), tables: &HIVE_TABLES }),
        "Percentile_discKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2549), tables: &HIVE_TABLES }),
        "PipeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2550), tables: &HIVE_TABLES }),
        "PipeSegment" => Some(RootGrammar { grammar_id: GrammarId(2551), tables: &HIVE_TABLES }),
        "PlacingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2552), tables: &HIVE_TABLES }),
        "PlanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2553), tables: &HIVE_TABLES }),
        "PliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2554), tables: &HIVE_TABLES }),
        "PlusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2555), tables: &HIVE_TABLES }),
        "PlusSegment" => Some(RootGrammar { grammar_id: GrammarId(2556), tables: &HIVE_TABLES }),
        "PolicyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2557), tables: &HIVE_TABLES }),
        "PositionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2558), tables: &HIVE_TABLES }),
        "PositiveSegment" => Some(RootGrammar { grammar_id: GrammarId(2559), tables: &HIVE_TABLES }),
        "PostFunctionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2560), tables: &HIVE_TABLES }),
        "PostTableExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2563), tables: &HIVE_TABLES }),
        "PostfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2564), tables: &HIVE_TABLES }),
        "PowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2565), tables: &HIVE_TABLES }),
        "PreTableFunctionKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2566), tables: &HIVE_TABLES }),
        "PrecedingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2567), tables: &HIVE_TABLES }),
        "PrecisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2568), tables: &HIVE_TABLES }),
        "PrefixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2569), tables: &HIVE_TABLES }),
        "PreorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2570), tables: &HIVE_TABLES }),
        "PrepareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2571), tables: &HIVE_TABLES }),
        "PreparedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2572), tables: &HIVE_TABLES }),
        "PreserveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2573), tables: &HIVE_TABLES }),
        "PrettyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2574), tables: &HIVE_TABLES }),
        "PrimaryKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(2575), tables: &HIVE_TABLES }),
        "PrimaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2578), tables: &HIVE_TABLES }),
        "PrimitiveTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2579), tables: &HIVE_TABLES }),
        "PrincipalsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2603), tables: &HIVE_TABLES }),
        "PrintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2604), tables: &HIVE_TABLES }),
        "PriorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2605), tables: &HIVE_TABLES }),
        "PrivilegesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2606), tables: &HIVE_TABLES }),
        "ProcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2607), tables: &HIVE_TABLES }),
        "ProceduralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2608), tables: &HIVE_TABLES }),
        "ProcedureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2609), tables: &HIVE_TABLES }),
        "ProceduresKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2610), tables: &HIVE_TABLES }),
        "ProcessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2611), tables: &HIVE_TABLES }),
        "ProcesslistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2612), tables: &HIVE_TABLES }),
        "PropertyGrammar" => Some(RootGrammar { grammar_id: GrammarId(2613), tables: &HIVE_TABLES }),
        "ProtectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2617), tables: &HIVE_TABLES }),
        "PublicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2618), tables: &HIVE_TABLES }),
        "PurgeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2619), tables: &HIVE_TABLES }),
        "QualifiedNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2620), tables: &HIVE_TABLES }),
        "QualifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2623), tables: &HIVE_TABLES }),
        "QuantifiedComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2624), tables: &HIVE_TABLES }),
        "QuarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2632), tables: &HIVE_TABLES }),
        "QuoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2633), tables: &HIVE_TABLES }),
        "QuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2634), tables: &HIVE_TABLES }),
        "QuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2635), tables: &HIVE_TABLES }),
        "Raid0KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2639), tables: &HIVE_TABLES }),
        "RaiserrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2640), tables: &HIVE_TABLES }),
        "RangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2641), tables: &HIVE_TABLES }),
        "RankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2642), tables: &HIVE_TABLES }),
        "RawEqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(2643), tables: &HIVE_TABLES }),
        "RawGreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2644), tables: &HIVE_TABLES }),
        "RawKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2645), tables: &HIVE_TABLES }),
        "RawLessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2646), tables: &HIVE_TABLES }),
        "RawNotSegment" => Some(RootGrammar { grammar_id: GrammarId(2647), tables: &HIVE_TABLES }),
        "RcfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2648), tables: &HIVE_TABLES }),
        "ReadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2649), tables: &HIVE_TABLES }),
        "ReadonlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2650), tables: &HIVE_TABLES }),
        "ReadsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2651), tables: &HIVE_TABLES }),
        "ReadtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2652), tables: &HIVE_TABLES }),
        "RealKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2653), tables: &HIVE_TABLES }),
        "RebuildKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2654), tables: &HIVE_TABLES }),
        "RecheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2655), tables: &HIVE_TABLES }),
        "ReconfigureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2656), tables: &HIVE_TABLES }),
        "RecordreaderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2657), tables: &HIVE_TABLES }),
        "RecordwriterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2658), tables: &HIVE_TABLES }),
        "RecursiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2659), tables: &HIVE_TABLES }),
        "ReduceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2660), tables: &HIVE_TABLES }),
        "RefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2661), tables: &HIVE_TABLES }),
        "ReferenceDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2662), tables: &HIVE_TABLES }),
        "ReferenceMatchGrammar" => Some(RootGrammar { grammar_id: GrammarId(2676), tables: &HIVE_TABLES }),
        "Reference_usageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2682), tables: &HIVE_TABLES }),
        "ReferencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2683), tables: &HIVE_TABLES }),
        "ReferencingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2684), tables: &HIVE_TABLES }),
        "ReferentialActionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2685), tables: &HIVE_TABLES }),
        "RegexpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2697), tables: &HIVE_TABLES }),
        "Regr_avgxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2698), tables: &HIVE_TABLES }),
        "Regr_avgyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2699), tables: &HIVE_TABLES }),
        "Regr_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2700), tables: &HIVE_TABLES }),
        "Regr_interceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2701), tables: &HIVE_TABLES }),
        "Regr_r2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2702), tables: &HIVE_TABLES }),
        "Regr_slopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2703), tables: &HIVE_TABLES }),
        "Regr_sxxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2704), tables: &HIVE_TABLES }),
        "Regr_sxyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2705), tables: &HIVE_TABLES }),
        "Regr_syyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2706), tables: &HIVE_TABLES }),
        "ReindexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2707), tables: &HIVE_TABLES }),
        "RelativeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2708), tables: &HIVE_TABLES }),
        "ReleaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2709), tables: &HIVE_TABLES }),
        "ReloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2710), tables: &HIVE_TABLES }),
        "RelyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2711), tables: &HIVE_TABLES }),
        "RenameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2712), tables: &HIVE_TABLES }),
        "RepairKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2713), tables: &HIVE_TABLES }),
        "RepeatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2714), tables: &HIVE_TABLES }),
        "RepeatableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2715), tables: &HIVE_TABLES }),
        "ReplaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2716), tables: &HIVE_TABLES }),
        "ReplicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2717), tables: &HIVE_TABLES }),
        "RequireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2718), tables: &HIVE_TABLES }),
        "ResetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2719), tables: &HIVE_TABLES }),
        "ResignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2720), tables: &HIVE_TABLES }),
        "ResourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2721), tables: &HIVE_TABLES }),
        "RespectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2722), tables: &HIVE_TABLES }),
        "RestartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2723), tables: &HIVE_TABLES }),
        "RestoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2724), tables: &HIVE_TABLES }),
        "RestrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2725), tables: &HIVE_TABLES }),
        "ResultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2726), tables: &HIVE_TABLES }),
        "ReturnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2727), tables: &HIVE_TABLES }),
        "Returned_cardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2728), tables: &HIVE_TABLES }),
        "Returned_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2729), tables: &HIVE_TABLES }),
        "Returned_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2730), tables: &HIVE_TABLES }),
        "Returned_sqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2731), tables: &HIVE_TABLES }),
        "ReturnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2732), tables: &HIVE_TABLES }),
        "RevokeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2733), tables: &HIVE_TABLES }),
        "RevokeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2734), tables: &HIVE_TABLES }),
        "RewriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2757), tables: &HIVE_TABLES }),
        "RightKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2758), tables: &HIVE_TABLES }),
        "RlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2759), tables: &HIVE_TABLES }),
        "RoleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2760), tables: &HIVE_TABLES }),
        "RoleReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2761), tables: &HIVE_TABLES }),
        "RolesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2762), tables: &HIVE_TABLES }),
        "RollbackKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2763), tables: &HIVE_TABLES }),
        "RollupFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2764), tables: &HIVE_TABLES }),
        "RollupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2765), tables: &HIVE_TABLES }),
        "RoutineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2766), tables: &HIVE_TABLES }),
        "Routine_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2767), tables: &HIVE_TABLES }),
        "Routine_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2768), tables: &HIVE_TABLES }),
        "Routine_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2769), tables: &HIVE_TABLES }),
        "RoutinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2770), tables: &HIVE_TABLES }),
        "RowFormatClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2771), tables: &HIVE_TABLES }),
        "RowFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(2804), tables: &HIVE_TABLES }),
        "RowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2809), tables: &HIVE_TABLES }),
        "Row_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2810), tables: &HIVE_TABLES }),
        "Row_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2811), tables: &HIVE_TABLES }),
        "RowcountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2812), tables: &HIVE_TABLES }),
        "RowguidcolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2813), tables: &HIVE_TABLES }),
        "RowidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2814), tables: &HIVE_TABLES }),
        "RownumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2815), tables: &HIVE_TABLES }),
        "RowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2816), tables: &HIVE_TABLES }),
        "RuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2817), tables: &HIVE_TABLES }),
        "SamplingExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2818), tables: &HIVE_TABLES }),
        "SaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2840), tables: &HIVE_TABLES }),
        "SavepointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2841), tables: &HIVE_TABLES }),
        "ScaleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2842), tables: &HIVE_TABLES }),
        "SchemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2843), tables: &HIVE_TABLES }),
        "SchemaReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(613), tables: &HIVE_TABLES }),
        "Schema_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2844), tables: &HIVE_TABLES }),
        "SchemasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2845), tables: &HIVE_TABLES }),
        "ScopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2846), tables: &HIVE_TABLES }),
        "Scope_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2847), tables: &HIVE_TABLES }),
        "Scope_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2848), tables: &HIVE_TABLES }),
        "Scope_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2849), tables: &HIVE_TABLES }),
        "ScrollKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2850), tables: &HIVE_TABLES }),
        "SearchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2851), tables: &HIVE_TABLES }),
        "SecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2852), tables: &HIVE_TABLES }),
        "Second_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2853), tables: &HIVE_TABLES }),
        "SecondsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2854), tables: &HIVE_TABLES }),
        "SectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2855), tables: &HIVE_TABLES }),
        "SecurityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2856), tables: &HIVE_TABLES }),
        "SelectClauseElementSegment" => Some(RootGrammar { grammar_id: GrammarId(2857), tables: &HIVE_TABLES }),
        "SelectClauseModifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2862), tables: &HIVE_TABLES }),
        "SelectClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2865), tables: &HIVE_TABLES }),
        "SelectClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2880), tables: &HIVE_TABLES }),
        "SelectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2900), tables: &HIVE_TABLES }),
        "SelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2901), tables: &HIVE_TABLES }),
        "SelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2920), tables: &HIVE_TABLES }),
        "SelfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2930), tables: &HIVE_TABLES }),
        "SemiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2931), tables: &HIVE_TABLES }),
        "SemicolonSegment" => Some(RootGrammar { grammar_id: GrammarId(2932), tables: &HIVE_TABLES }),
        "SensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2933), tables: &HIVE_TABLES }),
        "SeparatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2934), tables: &HIVE_TABLES }),
        "SequenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2935), tables: &HIVE_TABLES }),
        "SequenceMaxValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(2936), tables: &HIVE_TABLES }),
        "SequenceMinValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(2943), tables: &HIVE_TABLES }),
        "SequenceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1353), tables: &HIVE_TABLES }),
        "SequencefileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2950), tables: &HIVE_TABLES }),
        "SequencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2951), tables: &HIVE_TABLES }),
        "SerdeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2952), tables: &HIVE_TABLES }),
        "SerdePropertiesGrammar" => Some(RootGrammar { grammar_id: GrammarId(2953), tables: &HIVE_TABLES }),
        "SerdepropertiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2957), tables: &HIVE_TABLES }),
        "SerializableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2958), tables: &HIVE_TABLES }),
        "ServerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2959), tables: &HIVE_TABLES }),
        "Server_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2960), tables: &HIVE_TABLES }),
        "SessionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2961), tables: &HIVE_TABLES }),
        "Session_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2962), tables: &HIVE_TABLES }),
        "SetClauseListSegment" => Some(RootGrammar { grammar_id: GrammarId(2963), tables: &HIVE_TABLES }),
        "SetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2967), tables: &HIVE_TABLES }),
        "SetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2978), tables: &HIVE_TABLES }),
        "SetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2990), tables: &HIVE_TABLES }),
        "SetOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2991), tables: &HIVE_TABLES }),
        "SetSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2999), tables: &HIVE_TABLES }),
        "SetStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3004), tables: &HIVE_TABLES }),
        "SetofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3023), tables: &HIVE_TABLES }),
        "SetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3024), tables: &HIVE_TABLES }),
        "SetuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3025), tables: &HIVE_TABLES }),
        "ShareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3026), tables: &HIVE_TABLES }),
        "SharedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3027), tables: &HIVE_TABLES }),
        "SharesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3028), tables: &HIVE_TABLES }),
        "ShorthandCastSegment" => Some(RootGrammar { grammar_id: GrammarId(3029), tables: &HIVE_TABLES }),
        "ShowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3038), tables: &HIVE_TABLES }),
        "Show_databaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3039), tables: &HIVE_TABLES }),
        "ShutdownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3040), tables: &HIVE_TABLES }),
        "SignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3041), tables: &HIVE_TABLES }),
        "SignedSegmentGrammar" => Some(RootGrammar { grammar_id: GrammarId(3042), tables: &HIVE_TABLES }),
        "SimilarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3045), tables: &HIVE_TABLES }),
        "SimpleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3046), tables: &HIVE_TABLES }),
        "SingleIdentifierGrammar" => Some(RootGrammar { grammar_id: GrammarId(3047), tables: &HIVE_TABLES }),
        "SingleIdentifierListSegment" => Some(RootGrammar { grammar_id: GrammarId(3052), tables: &HIVE_TABLES }),
        "SingleIdentifierWildcardSegment" => Some(RootGrammar { grammar_id: GrammarId(3054), tables: &HIVE_TABLES }),
        "SingleQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3058), tables: &HIVE_TABLES }),
        "SizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3059), tables: &HIVE_TABLES }),
        "SizedArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(3060), tables: &HIVE_TABLES }),
        "SkewedByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3063), tables: &HIVE_TABLES }),
        "SkewedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3079), tables: &HIVE_TABLES }),
        "SlashSegment" => Some(RootGrammar { grammar_id: GrammarId(3080), tables: &HIVE_TABLES }),
        "SliceSegment" => Some(RootGrammar { grammar_id: GrammarId(3081), tables: &HIVE_TABLES }),
        "SmallintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3082), tables: &HIVE_TABLES }),
        "SnapshotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3083), tables: &HIVE_TABLES }),
        "SomeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3084), tables: &HIVE_TABLES }),
        "SonameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3085), tables: &HIVE_TABLES }),
        "SortByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3086), tables: &HIVE_TABLES }),
        "SortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3105), tables: &HIVE_TABLES }),
        "SortedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3106), tables: &HIVE_TABLES }),
        "SourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3107), tables: &HIVE_TABLES }),
        "SpaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3108), tables: &HIVE_TABLES }),
        "SpatialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3109), tables: &HIVE_TABLES }),
        "SpecificKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3110), tables: &HIVE_TABLES }),
        "Specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3111), tables: &HIVE_TABLES }),
        "SpecifictypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3112), tables: &HIVE_TABLES }),
        "SqlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3113), tables: &HIVE_TABLES }),
        "Sql_big_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3114), tables: &HIVE_TABLES }),
        "Sql_big_selectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3115), tables: &HIVE_TABLES }),
        "Sql_big_tablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3116), tables: &HIVE_TABLES }),
        "Sql_calc_found_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3117), tables: &HIVE_TABLES }),
        "Sql_log_offKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3118), tables: &HIVE_TABLES }),
        "Sql_log_updateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3119), tables: &HIVE_TABLES }),
        "Sql_low_priority_updatesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3120), tables: &HIVE_TABLES }),
        "Sql_select_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3121), tables: &HIVE_TABLES }),
        "Sql_small_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3122), tables: &HIVE_TABLES }),
        "Sql_warningsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3123), tables: &HIVE_TABLES }),
        "SqlcaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3124), tables: &HIVE_TABLES }),
        "SqlcodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3125), tables: &HIVE_TABLES }),
        "SqlerrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3126), tables: &HIVE_TABLES }),
        "SqlexceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3127), tables: &HIVE_TABLES }),
        "SqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3128), tables: &HIVE_TABLES }),
        "SqlwarningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3129), tables: &HIVE_TABLES }),
        "SqrtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3130), tables: &HIVE_TABLES }),
        "SslKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3131), tables: &HIVE_TABLES }),
        "StableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3132), tables: &HIVE_TABLES }),
        "StageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3133), tables: &HIVE_TABLES }),
        "StagesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3134), tables: &HIVE_TABLES }),
        "StarSegment" => Some(RootGrammar { grammar_id: GrammarId(3135), tables: &HIVE_TABLES }),
        "StartAngleBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(377), tables: &HIVE_TABLES }),
        "StartBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(227), tables: &HIVE_TABLES }),
        "StartCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(2378), tables: &HIVE_TABLES }),
        "StartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3136), tables: &HIVE_TABLES }),
        "StartSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(366), tables: &HIVE_TABLES }),
        "StartingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3137), tables: &HIVE_TABLES }),
        "StartsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3138), tables: &HIVE_TABLES }),
        "StateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3139), tables: &HIVE_TABLES }),
        "StatementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3140), tables: &HIVE_TABLES }),
        "StatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3141), tables: &HIVE_TABLES }),
        "StaticKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3183), tables: &HIVE_TABLES }),
        "StatisticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3184), tables: &HIVE_TABLES }),
        "Stddev_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3185), tables: &HIVE_TABLES }),
        "Stddev_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3186), tables: &HIVE_TABLES }),
        "StdinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3187), tables: &HIVE_TABLES }),
        "StdoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3188), tables: &HIVE_TABLES }),
        "StorageFormatGrammar" => Some(RootGrammar { grammar_id: GrammarId(3189), tables: &HIVE_TABLES }),
        "StorageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3194), tables: &HIVE_TABLES }),
        "StoredAsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3195), tables: &HIVE_TABLES }),
        "StoredByGrammar" => Some(RootGrammar { grammar_id: GrammarId(3199), tables: &HIVE_TABLES }),
        "StoredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3204), tables: &HIVE_TABLES }),
        "Straight_joinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3205), tables: &HIVE_TABLES }),
        "StreamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3206), tables: &HIVE_TABLES }),
        "StreamsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3207), tables: &HIVE_TABLES }),
        "StreamtableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3208), tables: &HIVE_TABLES }),
        "StrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3209), tables: &HIVE_TABLES }),
        "StringBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3210), tables: &HIVE_TABLES }),
        "StringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3212), tables: &HIVE_TABLES }),
        "StructKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3213), tables: &HIVE_TABLES }),
        "StructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3214), tables: &HIVE_TABLES }),
        "StructTypeSchemaSegment" => Some(RootGrammar { grammar_id: GrammarId(3219), tables: &HIVE_TABLES }),
        "StructTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(3226), tables: &HIVE_TABLES }),
        "StructureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3229), tables: &HIVE_TABLES }),
        "StyleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3230), tables: &HIVE_TABLES }),
        "Subclass_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3231), tables: &HIVE_TABLES }),
        "SublistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3232), tables: &HIVE_TABLES }),
        "SubmultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3233), tables: &HIVE_TABLES }),
        "SubstringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3234), tables: &HIVE_TABLES }),
        "SuccessfulKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3235), tables: &HIVE_TABLES }),
        "SumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3236), tables: &HIVE_TABLES }),
        "SummaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3237), tables: &HIVE_TABLES }),
        "SuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3238), tables: &HIVE_TABLES }),
        "SymbolSegment" => Some(RootGrammar { grammar_id: GrammarId(3239), tables: &HIVE_TABLES }),
        "SymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3240), tables: &HIVE_TABLES }),
        "SyncKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3241), tables: &HIVE_TABLES }),
        "SynonymKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3242), tables: &HIVE_TABLES }),
        "SysdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3243), tables: &HIVE_TABLES }),
        "SysidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3244), tables: &HIVE_TABLES }),
        "SystemKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3245), tables: &HIVE_TABLES }),
        "System_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3246), tables: &HIVE_TABLES }),
        "TableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(3247), tables: &HIVE_TABLES }),
        "TableEndClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3271), tables: &HIVE_TABLES }),
        "TableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3272), tables: &HIVE_TABLES }),
        "TableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3281), tables: &HIVE_TABLES }),
        "TablePropertiesGrammar" => Some(RootGrammar { grammar_id: GrammarId(3282), tables: &HIVE_TABLES }),
        "TableReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(613), tables: &HIVE_TABLES }),
        "Table_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3285), tables: &HIVE_TABLES }),
        "TablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3286), tables: &HIVE_TABLES }),
        "TablesampleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3287), tables: &HIVE_TABLES }),
        "TablespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3288), tables: &HIVE_TABLES }),
        "TablespaceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1353), tables: &HIVE_TABLES }),
        "TagReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1353), tables: &HIVE_TABLES }),
        "Tail_Recurse_Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(3289), tables: &HIVE_TABLES }),
        "Tail_Recurse_Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(3294), tables: &HIVE_TABLES }),
        "TaskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3298), tables: &HIVE_TABLES }),
        "TasksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3299), tables: &HIVE_TABLES }),
        "TblpropertiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3300), tables: &HIVE_TABLES }),
        "TempKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3301), tables: &HIVE_TABLES }),
        "TemplateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3302), tables: &HIVE_TABLES }),
        "TemporalQuerySegment" => Some(RootGrammar { grammar_id: GrammarId(3303), tables: &HIVE_TABLES }),
        "TemporaryGrammar" => Some(RootGrammar { grammar_id: GrammarId(3304), tables: &HIVE_TABLES }),
        "TemporaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3307), tables: &HIVE_TABLES }),
        "TemporaryTransientGrammar" => Some(RootGrammar { grammar_id: GrammarId(3308), tables: &HIVE_TABLES }),
        "TerminateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3311), tables: &HIVE_TABLES }),
        "TerminatedByGrammar" => Some(RootGrammar { grammar_id: GrammarId(3312), tables: &HIVE_TABLES }),
        "TerminatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3316), tables: &HIVE_TABLES }),
        "TextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3317), tables: &HIVE_TABLES }),
        "TextfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3318), tables: &HIVE_TABLES }),
        "TextsizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3319), tables: &HIVE_TABLES }),
        "ThanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3320), tables: &HIVE_TABLES }),
        "ThenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3321), tables: &HIVE_TABLES }),
        "TiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3322), tables: &HIVE_TABLES }),
        "TildeSegment" => Some(RootGrammar { grammar_id: GrammarId(3323), tables: &HIVE_TABLES }),
        "TimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3324), tables: &HIVE_TABLES }),
        "TimeWithTZGrammar" => Some(RootGrammar { grammar_id: GrammarId(3325), tables: &HIVE_TABLES }),
        "TimeZoneGrammar" => Some(RootGrammar { grammar_id: GrammarId(3336), tables: &HIVE_TABLES }),
        "TimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3342), tables: &HIVE_TABLES }),
        "TimestamptzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3343), tables: &HIVE_TABLES }),
        "Timezone_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3344), tables: &HIVE_TABLES }),
        "Timezone_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3345), tables: &HIVE_TABLES }),
        "TinyblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3346), tables: &HIVE_TABLES }),
        "TinyintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3347), tables: &HIVE_TABLES }),
        "TinytextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3348), tables: &HIVE_TABLES }),
        "ToKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3349), tables: &HIVE_TABLES }),
        "ToastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3350), tables: &HIVE_TABLES }),
        "TopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3351), tables: &HIVE_TABLES }),
        "Top_level_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3352), tables: &HIVE_TABLES }),
        "TouchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3353), tables: &HIVE_TABLES }),
        "TrailingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3354), tables: &HIVE_TABLES }),
        "TranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3355), tables: &HIVE_TABLES }),
        "TransactionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3356), tables: &HIVE_TABLES }),
        "TransactionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3357), tables: &HIVE_TABLES }),
        "Transaction_activeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3374), tables: &HIVE_TABLES }),
        "TransactionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3375), tables: &HIVE_TABLES }),
        "Transactions_committedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3376), tables: &HIVE_TABLES }),
        "Transactions_rolled_backKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3377), tables: &HIVE_TABLES }),
        "TransformKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3378), tables: &HIVE_TABLES }),
        "TransformsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3379), tables: &HIVE_TABLES }),
        "TransientKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3380), tables: &HIVE_TABLES }),
        "TranslateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3381), tables: &HIVE_TABLES }),
        "TranslationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3382), tables: &HIVE_TABLES }),
        "TreatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3383), tables: &HIVE_TABLES }),
        "TriggerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3384), tables: &HIVE_TABLES }),
        "TriggerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1353), tables: &HIVE_TABLES }),
        "Trigger_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3385), tables: &HIVE_TABLES }),
        "Trigger_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3386), tables: &HIVE_TABLES }),
        "Trigger_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3387), tables: &HIVE_TABLES }),
        "TrimKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3388), tables: &HIVE_TABLES }),
        "TrimParametersGrammar" => Some(RootGrammar { grammar_id: GrammarId(3389), tables: &HIVE_TABLES }),
        "TrueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3390), tables: &HIVE_TABLES }),
        "TrueSegment" => Some(RootGrammar { grammar_id: GrammarId(3391), tables: &HIVE_TABLES }),
        "TruncateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3392), tables: &HIVE_TABLES }),
        "TruncateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3393), tables: &HIVE_TABLES }),
        "TrustedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3398), tables: &HIVE_TABLES }),
        "TsequalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3399), tables: &HIVE_TABLES }),
        "TupleSegment" => Some(RootGrammar { grammar_id: GrammarId(3400), tables: &HIVE_TABLES }),
        "TypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3403), tables: &HIVE_TABLES }),
        "TypedArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3404), tables: &HIVE_TABLES }),
        "TypedStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3407), tables: &HIVE_TABLES }),
        "UescapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3410), tables: &HIVE_TABLES }),
        "UidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3411), tables: &HIVE_TABLES }),
        "UnarchiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3412), tables: &HIVE_TABLES }),
        "UnboundedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3413), tables: &HIVE_TABLES }),
        "UncommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3414), tables: &HIVE_TABLES }),
        "UnconditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3415), tables: &HIVE_TABLES }),
        "UnconditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3416), tables: &HIVE_TABLES }),
        "UnderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3420), tables: &HIVE_TABLES }),
        "UndoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3421), tables: &HIVE_TABLES }),
        "UnencryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3422), tables: &HIVE_TABLES }),
        "UnionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3423), tables: &HIVE_TABLES }),
        "UnionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3428), tables: &HIVE_TABLES }),
        "UniontypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3429), tables: &HIVE_TABLES }),
        "UniqueKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(3430), tables: &HIVE_TABLES }),
        "UniqueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3432), tables: &HIVE_TABLES }),
        "UniquejoinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3433), tables: &HIVE_TABLES }),
        "UnknownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3434), tables: &HIVE_TABLES }),
        "UnknownLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3435), tables: &HIVE_TABLES }),
        "UnlistenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3436), tables: &HIVE_TABLES }),
        "UnlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3437), tables: &HIVE_TABLES }),
        "UnnamedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3438), tables: &HIVE_TABLES }),
        "UnnestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3439), tables: &HIVE_TABLES }),
        "UnorderedSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3440), tables: &HIVE_TABLES }),
        "UnorderedSetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3456), tables: &HIVE_TABLES }),
        "UnsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3462), tables: &HIVE_TABLES }),
        "UnsignedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3463), tables: &HIVE_TABLES }),
        "UntilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3464), tables: &HIVE_TABLES }),
        "UpdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3465), tables: &HIVE_TABLES }),
        "UpdateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3466), tables: &HIVE_TABLES }),
        "UpdatetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3474), tables: &HIVE_TABLES }),
        "UpperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3475), tables: &HIVE_TABLES }),
        "UriKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3476), tables: &HIVE_TABLES }),
        "UsageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3477), tables: &HIVE_TABLES }),
        "UseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3478), tables: &HIVE_TABLES }),
        "UseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3479), tables: &HIVE_TABLES }),
        "Use_any_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3482), tables: &HIVE_TABLES }),
        "UserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3483), tables: &HIVE_TABLES }),
        "UserReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3484), tables: &HIVE_TABLES }),
        "User_defined_type_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3485), tables: &HIVE_TABLES }),
        "User_defined_type_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3486), tables: &HIVE_TABLES }),
        "User_defined_type_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3487), tables: &HIVE_TABLES }),
        "User_defined_type_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3488), tables: &HIVE_TABLES }),
        "UsersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3489), tables: &HIVE_TABLES }),
        "UsingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3490), tables: &HIVE_TABLES }),
        "UtcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3491), tables: &HIVE_TABLES }),
        "Utc_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3492), tables: &HIVE_TABLES }),
        "Utc_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3493), tables: &HIVE_TABLES }),
        "Utc_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3494), tables: &HIVE_TABLES }),
        "Utc_tmestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3495), tables: &HIVE_TABLES }),
        "UtctimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3496), tables: &HIVE_TABLES }),
        "VacuumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3497), tables: &HIVE_TABLES }),
        "ValidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3498), tables: &HIVE_TABLES }),
        "ValidateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3499), tables: &HIVE_TABLES }),
        "ValidatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3500), tables: &HIVE_TABLES }),
        "ValueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3501), tables: &HIVE_TABLES }),
        "Value_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3502), tables: &HIVE_TABLES }),
        "ValuesClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3503), tables: &HIVE_TABLES }),
        "ValuesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3516), tables: &HIVE_TABLES }),
        "Var_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3517), tables: &HIVE_TABLES }),
        "Var_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3518), tables: &HIVE_TABLES }),
        "VarbinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3519), tables: &HIVE_TABLES }),
        "Varchar2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3520), tables: &HIVE_TABLES }),
        "VarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3521), tables: &HIVE_TABLES }),
        "VarcharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3522), tables: &HIVE_TABLES }),
        "VariableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3523), tables: &HIVE_TABLES }),
        "VariablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3524), tables: &HIVE_TABLES }),
        "VaryingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3525), tables: &HIVE_TABLES }),
        "VectorizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3526), tables: &HIVE_TABLES }),
        "VerboseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3527), tables: &HIVE_TABLES }),
        "VersionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3528), tables: &HIVE_TABLES }),
        "ViewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3529), tables: &HIVE_TABLES }),
        "ViewsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3530), tables: &HIVE_TABLES }),
        "VolatileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3531), tables: &HIVE_TABLES }),
        "WaitforKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3532), tables: &HIVE_TABLES }),
        "WarehouseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3533), tables: &HIVE_TABLES }),
        "WarehousesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3534), tables: &HIVE_TABLES }),
        "WeekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3535), tables: &HIVE_TABLES }),
        "WeekdayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3536), tables: &HIVE_TABLES }),
        "WeeksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3537), tables: &HIVE_TABLES }),
        "WhenClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3538), tables: &HIVE_TABLES }),
        "WhenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3548), tables: &HIVE_TABLES }),
        "WheneverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3549), tables: &HIVE_TABLES }),
        "WhereClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3550), tables: &HIVE_TABLES }),
        "WhereClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3555), tables: &HIVE_TABLES }),
        "WhereKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3577), tables: &HIVE_TABLES }),
        "WhileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3578), tables: &HIVE_TABLES }),
        "WhitespaceSegment" => Some(RootGrammar { grammar_id: GrammarId(3579), tables: &HIVE_TABLES }),
        "Width_bucketKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3580), tables: &HIVE_TABLES }),
        "WildcardExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3581), tables: &HIVE_TABLES }),
        "WildcardIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3583), tables: &HIVE_TABLES }),
        "WindowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3593), tables: &HIVE_TABLES }),
        "WindowSpecificationSegment" => Some(RootGrammar { grammar_id: GrammarId(3594), tables: &HIVE_TABLES }),
        "WithCompoundNonSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3602), tables: &HIVE_TABLES }),
        "WithCompoundStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3611), tables: &HIVE_TABLES }),
        "WithDataClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3620), tables: &HIVE_TABLES }),
        "WithFillSegment" => Some(RootGrammar { grammar_id: GrammarId(3625), tables: &HIVE_TABLES }),
        "WithKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3626), tables: &HIVE_TABLES }),
        "WithNoSchemaBindingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3627), tables: &HIVE_TABLES }),
        "WithinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3632), tables: &HIVE_TABLES }),
        "WithoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3633), tables: &HIVE_TABLES }),
        "WordSegment" => Some(RootGrammar { grammar_id: GrammarId(3634), tables: &HIVE_TABLES }),
        "WorkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3635), tables: &HIVE_TABLES }),
        "WrapperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3636), tables: &HIVE_TABLES }),
        "WriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3637), tables: &HIVE_TABLES }),
        "WritetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3638), tables: &HIVE_TABLES }),
        "X509KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3639), tables: &HIVE_TABLES }),
        "XmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3640), tables: &HIVE_TABLES }),
        "XorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3641), tables: &HIVE_TABLES }),
        "YamlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3642), tables: &HIVE_TABLES }),
        "YearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3643), tables: &HIVE_TABLES }),
        "Year_monthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3644), tables: &HIVE_TABLES }),
        "YearsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3645), tables: &HIVE_TABLES }),
        "ZerofillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3646), tables: &HIVE_TABLES }),
        "ZoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3647), tables: &HIVE_TABLES }),
        _ => None,
    }
}

pub fn get_hive_segment_type(name: &str) -> Option<&'static str> {
    match name {
        "AccessObjectSegment" => Some("access_object"),
        "AccessPermissionSegment" => Some("access_permission"),
        "AccessPermissionsSegment" => Some("access_permissions"),
        "AccessSchemaObjectSegment" => Some("access_schema_object"),
        "AccessSchemaPluralObjectSegment" => Some("access_schema_plural_object"),
        "AccessStatementSegment" => Some("access_statement"),
        "AccessTargetSegment" => Some("access_target"),
        "AggregateOrderByClause" => Some("aggregate_order_by"),
        "AliasExpressionSegment" => Some("alias_expression"),
        "AlterDatabaseStatementSegment" => Some("alter_database_statement"),
        "AlterSequenceOptionsSegment" => Some("alter_sequence_options_segment"),
        "AlterSequenceStatementSegment" => Some("alter_sequence_statement"),
        "AlterTableStatementSegment" => Some("alter_table_statement"),
        "AlterViewStatementSegment" => Some("alter_view_statement"),
        "ArrayAccessorSegment" => Some("array_accessor"),
        "ArrayExpressionSegment" => Some("array_expression"),
        "ArrayLiteralSegment" => Some("array_literal"),
        "ArrayTypeSegment" => Some("array_type"),
        "AsAliasOperatorSegment" => Some("alias_operator"),
        "BaseFileSegment" => Some("file"),
        "BaseSegment" => Some("base"),
        "BinaryOperatorSegment" => Some("binary_operator"),
        "BitwiseAndSegment" => Some("binary_operator"),
        "BitwiseLShiftSegment" => Some("binary_operator"),
        "BitwiseOrSegment" => Some("binary_operator"),
        "BitwiseRShiftSegment" => Some("binary_operator"),
        "BracketedArguments" => Some("bracketed_arguments"),
        "BracketedSegment" => Some("bracketed"),
        "CTEColumnList" => Some("cte_column_list"),
        "CTEDefinitionSegment" => Some("common_table_expression"),
        "CaseExpressionSegment" => Some("case_expression"),
        "ClusterByClauseSegment" => Some("clusterby_clause"),
        "CodeSegment" => Some("raw"),
        "CollationReferenceSegment" => Some("collation_reference"),
        "ColumnConstraintSegment" => Some("column_constraint_segment"),
        "ColumnDefinitionSegment" => Some("column_definition"),
        "ColumnReferenceSegment" => Some("column_reference"),
        "ColumnsExpressionFunctionContentsSegment" => Some("columns_expression"),
        "ColumnsExpressionFunctionNameSegment" => Some("function_name"),
        "CommentClauseSegment" => Some("comment_clause"),
        "CommentSegment" => Some("comment"),
        "ComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeBinaryOperatorSegment" => Some("binary_operator"),
        "CompositeComparisonOperatorSegment" => Some("comparison_operator"),
        "ConcatSegment" => Some("binary_operator"),
        "CreateCastStatementSegment" => Some("create_cast_statement"),
        "CreateDatabaseStatementSegment" => Some("create_database_statement"),
        "CreateFunctionStatementSegment" => Some("create_function_statement"),
        "CreateIndexStatementSegment" => Some("create_index_statement"),
        "CreateModelStatementSegment" => Some("create_model_statement"),
        "CreateRoleStatementSegment" => Some("create_role_statement"),
        "CreateSchemaStatementSegment" => Some("create_schema_statement"),
        "CreateSequenceOptionsSegment" => Some("create_sequence_options_segment"),
        "CreateSequenceStatementSegment" => Some("create_sequence_statement"),
        "CreateTableStatementSegment" => Some("create_table_statement"),
        "CreateTriggerStatementSegment" => Some("create_trigger"),
        "CreateUserStatementSegment" => Some("create_user_statement"),
        "CreateViewStatementSegment" => Some("create_view_statement"),
        "CubeFunctionNameSegment" => Some("function_name"),
        "CubeRollupClauseSegment" => Some("cube_rollup_clause"),
        "DatabaseReferenceSegment" => Some("database_reference"),
        "DatatypeSegment" => Some("data_type"),
        "DatePartFunctionNameSegment" => Some("function_name"),
        "DateTimeFunctionContentsSegment" => Some("function_contents"),
        "Dedent" => Some("dedent"),
        "DeleteStatementSegment" => Some("delete_statement"),
        "DescribeStatementSegment" => Some("describe_statement"),
        "DistributeByClauseSegment" => Some("distributeby_clause"),
        "DropCastStatementSegment" => Some("drop_cast_statement"),
        "DropDatabaseStatementSegment" => Some("drop_database_statement"),
        "DropFunctionStatementSegment" => Some("drop_function_statement"),
        "DropIndexStatementSegment" => Some("drop_index_statement"),
        "DropModelStatementSegment" => Some("drop_MODELstatement"),
        "DropRoleStatementSegment" => Some("drop_role_statement"),
        "DropSchemaStatementSegment" => Some("drop_schema_statement"),
        "DropSequenceStatementSegment" => Some("drop_sequence_statement"),
        "DropTableStatementSegment" => Some("drop_table_statement"),
        "DropTriggerStatementSegment" => Some("drop_trigger"),
        "DropTypeStatementSegment" => Some("drop_type_statement"),
        "DropUserStatementSegment" => Some("drop_user_statement"),
        "DropViewStatementSegment" => Some("drop_view_statement"),
        "ElseClauseSegment" => Some("else_clause"),
        "EmptyStructLiteralBracketsSegment" => Some("struct_literal"),
        "EmptyStructLiteralSegment" => Some("typed_struct_literal"),
        "EqualsSegment" => Some("comparison_operator"),
        "ExplainStatementSegment" => Some("explain_statement"),
        "ExpressionSegment" => Some("expression"),
        "ExtensionReferenceSegment" => Some("extension_reference"),
        "FetchClauseSegment" => Some("fetch_clause"),
        "FileSegment" => Some("file"),
        "FrameClauseSegment" => Some("frame_clause"),
        "FromClauseSegment" => Some("from_clause"),
        "FromExpressionElementSegment" => Some("from_expression_element"),
        "FromExpressionSegment" => Some("from_expression"),
        "FromInsertClauseSegment" => Some("from_insert_clause"),
        "FromInsertSourceClauseSegment" => Some("from_insert_source_clause"),
        "FromInsertStatementSegment" => Some("from_insert_statement"),
        "FunctionContentsSegment" => Some("function_contents"),
        "FunctionDefinitionGrammar" => Some("function_definition"),
        "FunctionNameSegment" => Some("function_name"),
        "FunctionParameterListGrammar" => Some("function_parameter_list"),
        "FunctionSegment" => Some("function"),
        "GrantStatementSegment" => Some("grant_statement"),
        "GreaterThanOrEqualToSegment" => Some("comparison_operator"),
        "GreaterThanSegment" => Some("comparison_operator"),
        "GroupByClauseSegment" => Some("groupby_clause"),
        "GroupingExpressionList" => Some("grouping_expression_list"),
        "GroupingSetsClauseSegment" => Some("grouping_sets_clause"),
        "HavingClauseSegment" => Some("having_clause"),
        "HiveObjectReferenceSegment" => Some("object_reference"),
        "IdentifierSegment" => Some("identifier"),
        "ImplicitIndent" => Some("indent"),
        "Indent" => Some("indent"),
        "IndexColumnDefinitionSegment" => Some("index_column_definition"),
        "IndexReferenceSegment" => Some("index_reference"),
        "InsertStatementSegment" => Some("insert_statement"),
        "IntervalExpressionSegment" => Some("interval_expression"),
        "JoinClauseSegment" => Some("join_clause"),
        "JoinOnConditionSegment" => Some("join_on_condition"),
        "KeywordSegment" => Some("keyword"),
        "LateralViewClauseSegment" => Some("lateral_view_clause"),
        "LessThanOrEqualToSegment" => Some("comparison_operator"),
        "LessThanSegment" => Some("comparison_operator"),
        "LimitClauseSegment" => Some("limit_clause"),
        "LiteralKeywordSegment" => Some("literal"),
        "LiteralSegment" => Some("literal"),
        "LocalAliasSegment" => Some("local_alias_segment"),
        "MLTableExpressionSegment" => Some("ml_table_expression"),
        "MapTypeSegment" => Some("map_type"),
        "MatchConditionSegment" => Some("match_condition"),
        "MergeDeleteClauseSegment" => Some("merge_delete_clause"),
        "MergeInsertClauseSegment" => Some("merge_insert_clause"),
        "MergeMatchSegment" => Some("merge_match"),
        "MergeMatchedClauseSegment" => Some("merge_when_matched_clause"),
        "MergeNotMatchedClauseSegment" => Some("merge_when_not_matched_clause"),
        "MergeStatementSegment" => Some("merge_statement"),
        "MergeUpdateClauseSegment" => Some("merge_update_clause"),
        "MsckRepairTableStatementSegment" => Some("msck_repair_table_statement"),
        "MsckTableStatementSegment" => Some("msck_table_statement"),
        "NamedWindowExpressionSegment" => Some("named_window_expression"),
        "NamedWindowSegment" => Some("named_window"),
        "NewlineSegment" => Some("newline"),
        "NotEqualToSegment" => Some("comparison_operator"),
        "ObjectLiteralElementSegment" => Some("object_literal_element"),
        "ObjectLiteralSegment" => Some("object_literal"),
        "ObjectReferenceSegment" => Some("object_reference"),
        "OffsetClauseSegment" => Some("offset_clause"),
        "OrderByClauseSegment" => Some("orderby_clause"),
        "OverClauseSegment" => Some("over_clause"),
        "OverlapsClauseSegment" => Some("overlaps_clause"),
        "PartitionClauseSegment" => Some("partitionby_clause"),
        "PathSegment" => Some("path_segment"),
        "PrimitiveTypeSegment" => Some("primitive_type"),
        "QualifiedNumericLiteralSegment" => Some("numeric_literal"),
        "RevokeStatementSegment" => Some("revoke_statement"),
        "RoleReferenceSegment" => Some("role_reference"),
        "RollupFunctionNameSegment" => Some("function_name"),
        "RowFormatClauseSegment" => Some("row_format_clause"),
        "RowFunctionContentsSegment" => Some("function_contents"),
        "SamplingExpressionSegment" => Some("sample_expression"),
        "SchemaReferenceSegment" => Some("schema_reference"),
        "SelectClauseElementSegment" => Some("select_clause_element"),
        "SelectClauseModifierSegment" => Some("select_clause_modifier"),
        "SelectClauseSegment" => Some("select_clause"),
        "SelectStatementSegment" => Some("select_statement"),
        "SequenceReferenceSegment" => Some("sequence_reference"),
        "SetClauseListSegment" => Some("set_clause_list"),
        "SetClauseSegment" => Some("set_clause"),
        "SetExpressionSegment" => Some("set_expression"),
        "SetOperatorSegment" => Some("set_operator"),
        "SetSchemaStatementSegment" => Some("set_schema_statement"),
        "SetStatementSegment" => Some("set_statement"),
        "ShorthandCastSegment" => Some("cast_expression"),
        "SingleIdentifierListSegment" => Some("identifier_list"),
        "SingleIdentifierWildcardSegment" => Some("wildcard_identifier"),
        "SizedArrayTypeSegment" => Some("sized_array_type"),
        "SkewedByClauseSegment" => Some("skewed_by_clause"),
        "SortByClauseSegment" => Some("sortby_clause"),
        "StatementSegment" => Some("statement"),
        "StructLiteralSegment" => Some("struct_literal"),
        "StructTypeSchemaSegment" => Some("struct_type_schema"),
        "StructTypeSegment" => Some("struct_type"),
        "SymbolSegment" => Some("symbol"),
        "TableConstraintSegment" => Some("table_constraint"),
        "TableEndClauseSegment" => Some("table_end_clause_segment"),
        "TableExpressionSegment" => Some("table_expression"),
        "TableReferenceSegment" => Some("table_reference"),
        "TablespaceReferenceSegment" => Some("tablespace_reference"),
        "TagReferenceSegment" => Some("tag_reference"),
        "TemporalQuerySegment" => Some("temporal_query"),
        "TimeZoneGrammar" => Some("time_zone_grammar"),
        "TransactionStatementSegment" => Some("transaction_statement"),
        "TriggerReferenceSegment" => Some("trigger_reference"),
        "TruncateStatementSegment" => Some("truncate_table"),
        "TupleSegment" => Some("tuple"),
        "TypedArrayLiteralSegment" => Some("typed_array_literal"),
        "TypedStructLiteralSegment" => Some("typed_struct_literal"),
        "UnorderedSelectStatementSegment" => Some("select_statement"),
        "UnorderedSetExpressionSegment" => Some("set_expression"),
        "UpdateStatementSegment" => Some("update_statement"),
        "UseStatementSegment" => Some("use_statement"),
        "UserReferenceSegment" => Some("user_reference"),
        "ValuesClauseSegment" => Some("values_clause"),
        "WhenClauseSegment" => Some("when_clause"),
        "WhereClauseSegment" => Some("where_clause"),
        "WhitespaceSegment" => Some("whitespace"),
        "WildcardExpressionSegment" => Some("wildcard_expression"),
        "WildcardIdentifierSegment" => Some("wildcard_identifier"),
        "WindowSpecificationSegment" => Some("window_specification"),
        "WithCompoundNonSelectStatementSegment" => Some("with_compound_statement"),
        "WithCompoundStatementSegment" => Some("with_compound_statement"),
        "WithDataClauseSegment" => Some("with_data_clause"),
        "WithFillSegment" => Some("with_fill"),
        "WithNoSchemaBindingClauseSegment" => Some("with_no_schema_binding_clause"),
        "WordSegment" => Some("word"),
        _ => None,
    }
}

pub static HIVE_TABLES: GrammarTables = GrammarTables {
    instructions: INSTRUCTIONS,
    child_ids: CHILD_IDS,
    terminators: TERMINATORS,
    strings: STRINGS,
    aux_data: AUX_DATA,
    aux_data_offsets: AUX_DATA_OFFSETS,
    regex_patterns: REGEX_PATTERNS,
    simple_hints: SIMPLE_HINTS,
    hint_string_indices: HINT_STRING_INDICES,
    simple_hint_indices: SIMPLE_HINT_INDICES,
    segment_type_offsets: SEGMENT_TYPE_OFFSETS,
    segment_class_offsets: SEGMENT_CLASS_OFFSETS,
    casefold_sparse: CASEFOLD_SPARSE,
    trim_chars_sparse: TRIM_CHARS_SPARSE,
    trim_chars_data: TRIM_CHARS_DATA,
    segment_class_types_sparse: SEGMENT_CLASS_TYPES_SPARSE,
    segment_class_types_data: SEGMENT_CLASS_TYPES_DATA,
};

pub fn get_hive_root_grammar_id() -> GrammarId {
    GrammarId(1394)
}
pub fn get_hive_root_grammar_table() -> RootGrammar {
    RootGrammar {
        grammar_id: get_hive_root_grammar_id(),
        tables: &HIVE_TABLES,
    }
}
pub fn get_hive_root_grammar() -> RootGrammar {
    get_hive_root_grammar_table()
}
