/* This is a generated file! */
/* Generated by `utils/build_parsers.py` via `utils/rustify.py` */
/* This process can be run via tox: `tox -e generate-rs` */
#![cfg_attr(rustfmt, rustfmt_skip)]

use sqlfluffrs_types::{
    GrammarInst, GrammarVariant, GrammarFlags,
    ParseMode, GrammarId, GrammarTables,
    SimpleHintData, RootGrammar
};
// Flattening grammar tree...
// Validating tables...
// Validation passed!

// Table Statistics:
//   Instructions:      7707 × 20 bytes =   154140 bytes
//   Child IDs:         6855 ×  4 bytes =    27420 bytes
//   Terminators:         82 ×  4 bytes =      328 bytes
//   Strings:           2408 strings      =    38275 bytes
//   Aux Data:         11714 ×  4 bytes =    46856 bytes
//   Aux Offsets:       7707 ×  4 bytes =    30828 bytes
//   Regex Patterns:      12 patterns     =     1001 bytes
//   Simple Hints:      1255 hints        =    20080 bytes
//   Hint Indices:      2885 ×  4 bytes =    11540 bytes
//   TOTAL:                                    ≈   330468 bytes

pub static INSTRUCTIONS: &[GrammarInst] = &[
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [0] StringParser("ABORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [1] StringParser("ABSOLUTE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [2] TypedParser("at")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [3] StringParser("ACCESS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 0, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [4] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [5] Ref(AccountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [6] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5, child_count: 17, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [7] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 22, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [8] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [9] Ref(ResourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [10] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [11] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [12] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [13] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [14] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [15] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [16] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [17] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [18] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [19] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [20] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [21] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [22] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [23] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [24] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [25] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [26] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [27] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [28] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [29] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [30] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [31] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [32] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [33] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [34] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [35] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [36] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [37] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [38] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [39] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [40] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [41] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [42] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [43] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [44] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [45] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [46] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [47] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [48] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [49] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [50] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 2, _padding: 0 }, // [51] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 50, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [52] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [53] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [54] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [55] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [56] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [57] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [58] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [59] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [60] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [61] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [62] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [63] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [64] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 6, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [65] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 64, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [66] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [67] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 5, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [68] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [69] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [70] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [71] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [72] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [73] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [74] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [75] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [76] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [77] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [78] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [79] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [80] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [81] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [82] Ref(ManageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [83] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [84] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [85] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [86] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [87] Ref(ExecutionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [88] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [89] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 26, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [90] OneOf(26 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 110, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [91] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [92] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 4, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [93] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [94] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [95] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [96] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [97] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [98] Ref(PipeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [99] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [100] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [101] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [102] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [103] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [104] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [105] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [106] Ref(ImportedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [107] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [108] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [109] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [110] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [111] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [112] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [113] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [114] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [115] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [116] Ref(OperateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [117] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [118] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [119] Ref(Reference_usageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [120] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [121] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [122] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [123] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [124] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [125] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [126] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [127] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [128] Ref(Use_any_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [129] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [130] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [131] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [132] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [133] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 1, _padding: 0 }, // [134] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [135] Ref(AccessPermissionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [136] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [137] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [138] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [139] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [140] Ref(StageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [141] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [142] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [143] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [144] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [145] Ref(StreamKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [146] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [147] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [148] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [149] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [150] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [151] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [152] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [153] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [154] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [155] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [156] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [157] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [158] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [159] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [160] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [161] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [162] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [163] Ref(StagesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [164] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [165] Ref(ProceduresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [166] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [167] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [168] Ref(StreamsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [169] Ref(TasksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [170] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [171] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [172] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [173] Ref(PoliciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [174] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [175] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [176] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [177] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [178] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [179] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [180] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [181] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [182] Ref(FormatsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [183] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [184] Ref(GrantStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [185] Ref(RevokeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [186] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 181, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [187] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [188] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [189] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [190] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [191] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [192] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [193] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [194] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [195] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [196] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [197] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [198] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [199] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [200] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [201] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [202] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [203] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [204] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [205] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [206] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [207] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 201, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [208] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [209] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [210] Ref(SemiStructuredAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [211] StringParser("ACCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [212] StringParser("ACCOUNTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [213] StringParser("ACTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [214] StringParser("ADD")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [215] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [216] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [217] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [218] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [219] StringParser("ADMIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [220] StringParser("AFTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [221] StringParser("AGGREGATE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [222] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [223] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 210, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [224] Meta("indent")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 210, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [225] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 210, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [226] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 212, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [227] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [228] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 214, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [229] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 217, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [230] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 217, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [231] StringParser("(")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 217, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [232] StringParser(")")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 217, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [233] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 217, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [234] Meta("dedent")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 217, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [235] StringParser("ALIAS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 217, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [236] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 219, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [237] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 219, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [238] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 219, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [239] StringParser("ALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 219, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [240] StringParser("ALLOW_CONNECTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 219, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [241] StringParser("ALSO")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 219, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [242] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [243] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [244] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [245] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [246] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 227, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [247] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 230, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [248] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 230, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [249] Anything
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 230, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [250] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 230, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [251] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [252] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 236, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [253] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 236, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [254] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 236, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [255] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 236, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [256] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [257] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [258] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [259] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 243, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [260] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 243, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [261] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 243, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [262] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 243, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [263] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 243, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [264] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [265] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [266] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [267] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [268] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 250, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [269] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 250, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [270] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 250, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [271] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 250, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [272] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 256, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [273] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 258, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [274] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 258, child_count: 3, min_times: 1, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [275] AnyNumberOf(3 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [276] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 263, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [277] Ref(Allow_connectionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 263, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [278] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 263, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [279] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 266, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [280] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 266, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [281] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 266, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [282] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 266, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [283] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 268, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [284] Ref(Is_templateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 268, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [285] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 268, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [286] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [287] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [288] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [289] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [290] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 274, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [291] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 274, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [292] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 274, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [293] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 278, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [294] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 278, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [295] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 278, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [296] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 278, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [297] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 278, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [298] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [299] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [300] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [301] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [302] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [303] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [304] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 284, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [305] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 286, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [306] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 288, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [307] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [308] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [309] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [310] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [311] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [312] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [313] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [314] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 294, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [315] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 296, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [316] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 296, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [317] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 296, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [318] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 298, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [319] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 298, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [320] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [321] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [322] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 300, child_count: 7, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [323] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [324] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [325] Ref(AlterDefaultPrivilegesObjectPrivilegesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [326] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [327] Ref(AlterDefaultPrivilegesSchemaObjectsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [328] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 1, _padding: 0 }, // [329] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 309, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [330] Ref(AlterDefaultPrivilegesToFromRolesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 309, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [331] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 309, child_count: 3, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [332] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [333] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [334] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [335] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [336] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 314, child_count: 2, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [337] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [338] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [339] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 2, min_times: 0, first_terminator_idx: 4, terminator_count: 1, _padding: 0 }, // [340] Delimited(10 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 318, child_count: 10, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [341] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 328, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [342] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 328, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [343] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 328, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [344] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 328, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [345] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 328, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [346] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 328, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [347] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 328, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [348] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 328, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [349] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 328, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [350] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 328, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [351] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 328, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [352] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 328, child_count: 8, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [353] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 336, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [354] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 336, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [355] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [356] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [357] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [358] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [359] Ref(AlterDefaultPrivilegesObjectPrivilegesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [360] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [361] Ref(AlterDefaultPrivilegesSchemaObjectsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [362] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 339, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 2, _padding: 0 }, // [363] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [364] Ref(AlterDefaultPrivilegesToFromRolesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [365] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [366] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [367] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 341, child_count: 6, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [368] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 347, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [369] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 347, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [370] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 347, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [371] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 347, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [372] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 347, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [373] Ref(TypesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 347, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [374] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 347, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [375] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 353, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [376] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 353, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [377] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 353, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [378] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 353, child_count: 3, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [379] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [380] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 2, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [381] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [382] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [383] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 2, min_times: 0, first_terminator_idx: 7, terminator_count: 3, _padding: 0 }, // [384] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [385] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [386] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [387] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [388] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 360, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [389] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 363, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [390] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 363, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [391] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 363, child_count: 2, min_times: 0, first_terminator_idx: 10, terminator_count: 2, _padding: 0 }, // [392] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 365, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [393] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 365, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [394] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 365, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [395] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 365, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [396] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 367, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [397] Ref(AlterDefaultPrivilegesGrantSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 367, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [398] Ref(AlterDefaultPrivilegesRevokeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 367, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [399] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 369, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [400] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 371, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [401] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 371, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [402] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 371, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [403] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 371, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [404] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 375, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [405] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 375, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [406] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 375, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [407] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 375, child_count: 10, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [408] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 385, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [409] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 388, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [410] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 388, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [411] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 388, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [412] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 388, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [413] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 390, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [414] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 390, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [415] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 390, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [416] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 393, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [417] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 395, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [418] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 395, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [419] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 395, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [420] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 395, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [421] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 395, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [422] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [423] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 399, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [424] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [425] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [426] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [427] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 403, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [428] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 405, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [429] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 405, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [430] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 405, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [431] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [432] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [433] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 407, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [434] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 409, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [435] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 409, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [436] Ref(ValidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 409, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [437] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 414, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [438] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 414, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [439] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 414, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [440] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 414, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [441] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 414, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [442] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [443] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [444] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [445] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 421, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [446] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 421, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [447] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 421, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [448] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 421, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [449] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 421, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [450] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 421, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [451] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 424, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [452] Ref(ValidateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 424, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [453] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 424, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [454] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 424, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [455] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 427, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [456] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 427, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [457] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 427, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [458] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [459] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [460] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [461] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [462] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [463] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 434, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [464] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 434, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [465] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 434, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [466] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 434, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [467] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 437, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [468] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 437, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [469] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 437, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [470] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 437, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [471] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 441, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [472] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 441, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [473] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 441, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [474] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 441, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [475] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 444, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [476] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 446, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [477] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 446, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [478] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [479] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [480] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [481] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 451, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [482] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 451, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [483] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 451, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [484] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 453, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [485] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 453, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [486] Ref(Current_schemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 453, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [487] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 455, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [488] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 457, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [489] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 457, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [490] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 457, child_count: 7, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [491] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [492] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 466, child_count: 13, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [493] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [494] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 481, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [495] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 481, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [496] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 481, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [497] Ref(CollationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 481, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [498] Ref(ConversionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 481, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [499] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 481, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [500] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 483, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [501] Ref(EventKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 483, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [502] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 483, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [503] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 486, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [504] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 486, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [505] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 486, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [506] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 486, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [507] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 488, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [508] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 488, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [509] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 488, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [510] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 490, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [511] Ref(ProceduralKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [512] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [513] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [514] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [515] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [516] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 493, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [517] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 493, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [518] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 493, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [519] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [520] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [521] Ref(DictionaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [522] Ref(ParserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [523] Ref(TemplateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [524] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [525] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [526] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [527] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 502, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [528] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 504, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [529] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 504, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [530] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 504, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [531] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 504, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [532] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 504, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [533] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 504, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [534] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 507, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [535] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 507, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [536] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 507, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [537] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 510, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [538] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [539] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [540] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 513, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [541] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 513, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [542] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 516, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [543] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [544] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [545] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [546] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [547] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 521, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [548] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 524, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [549] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 524, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [550] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 524, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [551] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 524, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [552] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 526, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [553] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 528, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [554] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 528, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [555] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 528, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [556] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 530, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [557] Ref(OperatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 530, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [558] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 532, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [559] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 534, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [560] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 534, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [561] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 537, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [562] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 539, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [563] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 542, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [564] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 542, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [565] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 542, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [566] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 542, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [567] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 546, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [568] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 548, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [569] Ref(ClassKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 548, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [570] Ref(FamilyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 548, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [571] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 548, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [572] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 548, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [573] Ref(IndexAccessMethodSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 548, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [574] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 552, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [575] Ref(TransformKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 552, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [576] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 552, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [577] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 552, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [578] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 552, child_count: 24, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [579] OneOf(24 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 7, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [580] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [581] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [582] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [583] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [584] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [585] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 583, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [586] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 585, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [587] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 585, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [588] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 585, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [589] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 586, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [590] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 586, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [591] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [592] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 591, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [593] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 591, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [594] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [595] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 591, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [596] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [597] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [598] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 595, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [599] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [600] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 13, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [601] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 608, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [602] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 613, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [603] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [604] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [605] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [606] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [607] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 615, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [608] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 617, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [609] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 617, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [610] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 617, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [611] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [612] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 619, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [613] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 620, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [614] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 620, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [615] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 623, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [616] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 623, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [617] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 623, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [618] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 624, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [619] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [620] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [621] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [622] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [623] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [624] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [625] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [626] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [627] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 633, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [628] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 635, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [629] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 635, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [630] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 635, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [631] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 635, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [632] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 635, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [633] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [634] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [635] Ref(ExpressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [636] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 638, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [637] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [638] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [639] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [640] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 646, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [641] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 646, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [642] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 648, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [643] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 648, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [644] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 648, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [645] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 648, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [646] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 648, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [647] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 651, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [648] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [649] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [650] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [651] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 656, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [652] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [653] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [654] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [655] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [656] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [657] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 663, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [658] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 663, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [659] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 663, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [660] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 665, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [661] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 665, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [662] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 665, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [663] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 667, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [664] Ref(RestartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 667, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [665] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 669, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [666] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 669, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [667] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 669, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [668] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [669] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [670] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [671] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [672] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 675, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [673] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 675, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [674] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 675, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [675] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 675, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [676] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 677, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [677] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 677, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [678] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 677, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [679] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 679, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [680] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 679, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [681] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 679, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [682] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [683] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [684] Ref(StorageKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 682, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [685] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [686] Ref(PlainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [687] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [688] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [689] Ref(MainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [690] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 689, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [691] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 689, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [692] Ref(CompressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 689, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [693] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 689, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [694] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 691, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [695] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 691, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [696] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 691, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [697] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 693, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [698] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 693, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [699] Ref(TableConstraintUsingIndexSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 693, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [700] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 698, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [701] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 698, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [702] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 698, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [703] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 698, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [704] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [705] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [706] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [707] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [708] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 702, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [709] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 704, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [710] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 706, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [711] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 706, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [712] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 706, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [713] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 708, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [714] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 708, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [715] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 708, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [716] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [717] Ref(ValidateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [718] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [719] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [720] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [721] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [722] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [723] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [724] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [725] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [726] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [727] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 721, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [728] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 721, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [729] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 721, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [730] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 721, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [731] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [732] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [733] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [734] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 724, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [735] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 728, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [736] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 728, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [737] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 730, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [738] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 730, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [739] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 730, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [740] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 730, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [741] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 730, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [742] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 733, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [743] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 737, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [744] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 737, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [745] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 737, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [746] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 739, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [747] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 739, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [748] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 739, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [749] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [750] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [751] Ref(RuleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [752] Ref(RuleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [753] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [754] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 745, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [755] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 749, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [756] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 749, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [757] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 749, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [758] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 749, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [759] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [760] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [761] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [762] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [763] Ref(LevelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [764] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [765] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [766] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [767] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [768] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [769] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 757, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [770] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 757, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [771] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 757, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [772] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 759, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [773] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 759, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [774] Ref(OidsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 759, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [775] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [776] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [777] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [778] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [779] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 764, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [780] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 764, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [781] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [782] Ref(LoggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [783] Ref(UnloggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 766, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [784] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [785] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [786] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [787] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [788] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [789] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 770, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [790] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [791] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [792] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [793] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [794] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [795] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [796] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [797] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 777, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [798] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 777, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [799] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 777, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [800] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 780, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [801] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 780, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [802] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 780, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [803] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [804] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [805] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [806] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [807] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [808] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 787, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [809] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 787, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [810] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 787, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [811] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 791, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [812] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 791, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [813] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [814] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [815] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [816] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [817] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [818] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [819] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 797, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [820] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 800, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [821] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 800, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [822] Ref(COLUMN)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 800, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [823] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 800, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [824] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 800, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [825] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [826] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 805, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [827] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 808, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [828] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [829] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [830] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [831] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [832] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [833] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [834] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [835] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [836] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [837] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [838] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 820, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [839] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 820, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [840] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 820, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [841] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 820, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [842] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 820, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [843] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 823, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [844] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [845] Ref(AlterForeignTableActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [846] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 830, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [847] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 830, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [848] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 830, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [849] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 830, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [850] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 830, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [851] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 830, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [852] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 833, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [853] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 833, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [854] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 833, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [855] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 833, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [856] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 835, child_count: 10, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [857] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 845, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [858] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 848, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [859] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 852, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [860] Ref(CalledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 852, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [861] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 852, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [862] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 852, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [863] Ref(InputKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 852, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [864] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 857, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [865] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 857, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [866] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 857, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [867] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 857, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [868] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 857, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [869] Ref(InputKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 857, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [870] Ref(StrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 857, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [871] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 860, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [872] Ref(ImmutableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 860, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [873] Ref(StableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 860, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [874] Ref(VolatileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 860, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [875] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 862, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [876] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 862, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [877] Ref(LeakproofKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 862, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [878] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 865, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [879] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 865, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [880] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 865, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [881] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 867, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [882] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 867, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [883] Ref(InvokerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 867, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [884] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 869, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [885] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 869, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [886] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 872, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [887] Ref(UnsafeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 872, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [888] Ref(RestrictedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 872, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [889] Ref(SafeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 872, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [890] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 874, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [891] Ref(CostKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 874, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [892] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 874, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [893] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [894] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [895] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 876, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [896] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 878, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [897] Ref(SupportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 878, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [898] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 878, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [899] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 881, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [900] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 881, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [901] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 881, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [902] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 883, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [903] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [904] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [905] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [906] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [907] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 890, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [908] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 890, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [909] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 890, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [910] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 890, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [911] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [912] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [913] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [914] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 894, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [915] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 894, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [916] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 896, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [917] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 896, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [918] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 896, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [919] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 896, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [920] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [921] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [922] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 900, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [923] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [924] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 904, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [925] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 904, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [926] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 904, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [927] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [928] Ref(AlterFunctionActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [929] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [930] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [931] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [932] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 912, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [933] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [934] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [935] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [936] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [937] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [938] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [939] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [940] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 922, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [941] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 924, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [942] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 924, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [943] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 924, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [944] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 924, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [945] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 924, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [946] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 924, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [947] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [948] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [949] Ref(DependsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [950] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [951] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [952] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [953] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 932, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [954] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 932, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [955] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 932, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [956] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 934, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [957] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 937, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [958] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 937, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [959] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 937, child_count: 7, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [960] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 944, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [961] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [962] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [963] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [964] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [965] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 950, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [966] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 950, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [967] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 950, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [968] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 950, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [969] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 953, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [970] Ref(AttachKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 953, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [971] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 953, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [972] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 953, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [973] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 958, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [974] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 958, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [975] Ref(DependsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 958, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [976] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 958, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [977] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 958, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [978] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 958, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [979] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 960, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [980] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 960, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [981] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 963, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [982] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 965, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [983] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 967, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [984] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 967, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [985] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 969, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [986] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 969, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [987] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 969, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [988] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 971, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [989] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 971, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [990] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 974, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [991] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 976, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [992] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 976, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [993] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [994] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [995] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [996] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [997] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [998] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [999] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 9, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1000] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1001] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1002] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1003] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1004] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 991, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1005] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 994, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1006] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 994, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1007] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 994, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1008] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1009] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1010] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1011] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1012] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1013] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1014] StringParser("ALTER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1015] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1002, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1016] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1017] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1018] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1019] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1020] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1021] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1014, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1022] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1014, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1023] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1014, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1024] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1014, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1025] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1016, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1026] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1016, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1027] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1028] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1021, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1029] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1030] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1031] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1032] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1033] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1026, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1034] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1026, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1035] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1036] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1031, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1037] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1031, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1038] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1034, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1039] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1034, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1040] Ref(StorageKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1034, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1041] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1038, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1042] Ref(PlainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1038, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1043] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1038, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1044] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1038, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1045] Ref(MainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1038, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1046] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1047] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1048] Ref(CompressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1049] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1050] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1051] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1052] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1053] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1054] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1055] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1056] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1057] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1047, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1058] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1049, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1059] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1049, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1060] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1052, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1061] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1054, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1062] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1056, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1063] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1056, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1064] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1058, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1065] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1058, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1066] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1058, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1067] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1068] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1069] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1063, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1070] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1071] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1072] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1068, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1073] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1068, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1074] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1068, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1075] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1072, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1076] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1072, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1077] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1072, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1078] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1072, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1079] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1072, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1080] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1076, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1081] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1076, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1082] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1076, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1083] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1076, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1084] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1085] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1082, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1086] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1082, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1087] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1082, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1088] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1086, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1089] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1090] Ref(AlterMaterializedViewActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1091] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1092] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1093, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1093] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1094, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1094] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1094, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1095] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1094, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1096] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1094, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1097] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1094, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1098] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1097, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1099] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1097, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1100] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1097, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1101] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1097, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1102] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1103] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1104] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1105] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1106] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1107] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1106, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1108] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1109] Ref(DependsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1110] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1111] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1112] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 9, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1113] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1115, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1114] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1115, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1115] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1115, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1116] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1115, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1117] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1115, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1118] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1118, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1119] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1118, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1120] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1118, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1121] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1120, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1122] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1120, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1123] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1120, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1124] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1120, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1125] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1120, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1126] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1127] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1128] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1129] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1130] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1131] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1132] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1133] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1134] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1129, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1135] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1132, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1136] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1132, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1137] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1132, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1138] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1132, child_count: 3, min_times: 1, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1139] AnySetOf(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1135, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1140] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1137, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1141] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1137, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1142] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1139, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1143] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1144] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1145] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1146] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1147] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1148] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1149] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1146, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1150] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1146, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1151] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1152] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1153] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1152, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1154] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1152, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1155] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1152, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1156] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1157] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1158] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1157, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1159] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1160, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1160] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1163, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1161] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1163, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1162] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1163, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1163] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1165, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1164] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1165, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1165] Ref(InvokerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1165, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1166] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1167] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1168] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1168, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1169] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1170, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1170] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1172, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1171] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1172] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1173] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1174] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1177, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1175] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1177, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1176] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1177, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1177] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1177, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1178] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1179] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1180] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1181] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1181, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1182] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1181, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1183] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1183, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1184] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1183, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1185] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1183, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1186] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1183, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1187] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1187, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1188] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1187, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1189] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1187, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1190] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1189, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1191] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1191, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1192] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1191, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1193] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1191, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1194] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1197, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1195] Ref(AlterProcedureActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1197, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1196] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1197] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1198] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1199] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1200] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1203, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1201] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1203, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1202] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1203, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1203] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1203, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1204] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1206, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1205] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1206, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1206] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1206, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1207] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1208, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1208] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1210, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1209] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1210] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1211] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1212] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1214, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1213] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1216, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1214] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1216, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1215] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1216, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1216] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1217] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1218] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1219] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1221, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1220] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1221, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1221] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1221, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1222] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1223] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1224] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1225] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1226] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1227] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1228] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1229] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1230] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1231] Ref(DependsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1232] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1233] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1234] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1235] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1236, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1236] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1236, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1237] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1236, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1238] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1236, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1239] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1242, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1240] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1244, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1241] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1244, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1242] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1244, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1243] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1244] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1246, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1245] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1246] Ref(PublicationObjectsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1247] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1248] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1250, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1249] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1252, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1250] Ref(PublicationObjectsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1252, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1251] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1254, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1252] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1254, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1253] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1256, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1254] Ref(PublicationObjectsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1256, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1255] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1256] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1257] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1258] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1259] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1260] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1261] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1262] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1266, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1263] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1266, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1264] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1266, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1265] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1266, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1266] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1266, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1267] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1269, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1268] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1269, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1269] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1271, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1270] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1271, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1271] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1271, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1272] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1274, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1273] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1277, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1274] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1281, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1275] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1281, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1276] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1281, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1277] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1281, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1278] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1281, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1279] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1281, child_count: 10, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1280] AnySetOf(10 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1291, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1281] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1282] Ref(SuperuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1283] Ref(NosuperuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1284] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1295, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1285] Ref(CreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1295, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1286] Ref(NocreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1295, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1287] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1297, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1288] Ref(CreateroleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1297, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1289] Ref(NocreateroleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1297, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1290] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1291] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1292] Ref(NoinheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1293] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1301, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1294] Ref(LoginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1301, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1295] Ref(NologinKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1301, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1296] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1303, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1297] Ref(ReplicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1303, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1298] Ref(NoreplicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1303, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1299] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1305, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1300] Ref(BypassrlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1305, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1301] Ref(NobypassrlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1305, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1302] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1308, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1303] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1308, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1304] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1308, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1305] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1308, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1306] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1311, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1307] Ref(EncryptedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1311, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1308] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1311, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1309] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1310] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1311] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1313, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1312] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1316, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1313] Ref(ValidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1316, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1314] Ref(UntilKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1316, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1315] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1316, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1316] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1318, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1317] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1318, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1318] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1321, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1319] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1321, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1320] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1321, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1321] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1321, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1322] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1324, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1323] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1329, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1324] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1329, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1325] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1329, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1326] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1329, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1327] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1329, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1328] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1329, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1329] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1332, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1330] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1332, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1331] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1332, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1332] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1332, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1333] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1334] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1337, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1335] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1337, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1336] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1337, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1337] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1339, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1338] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1339] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1340] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1341] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1342] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1345, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1343] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1345, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1344] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1347, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1345] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1350, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1346] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1350, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1347] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1350, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1348] Ref(OnKeywordAsIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1350, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1349] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1350] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1351] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1352] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1354, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1353] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1354, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1354] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1356, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1355] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1356, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1356] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1356, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1357] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1360, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1358] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1360, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1359] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1360, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1360] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1360, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1361] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1362, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1362] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1363] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1364] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1365] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1366] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1368, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1367] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1368, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1368] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1368, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1369] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1368, child_count: 10, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1370] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1378, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1371] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1380, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1372] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1380, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1373] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1380, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1374] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1384, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1375] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1384, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1376] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1384, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1377] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1384, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1378] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1384, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1379] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1386, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1380] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1389, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1381] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1389, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1382] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1389, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1383] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1389, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1384] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1385] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1386] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1387] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1393, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1388] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1389] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1390] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1391] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1396, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1392] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1393] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1394] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1395] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1396] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1397] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1398] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1399] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1405, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1400] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1405, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1401] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1405, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1402] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1405, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1403] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1405, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1404] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1409, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1405] Ref(RestartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1409, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1406] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1409, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1407] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1409, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1408] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1409, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1409] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1411, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1410] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1411, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1411] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1411, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1412] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1413, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1413] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1413, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1414] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1413, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1415] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1416, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1416] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1416, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1417] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1416, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1418] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1418, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1419] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1418, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1420] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1418, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1421] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1423, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1422] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1423, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1423] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1423, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1424] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1423, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1425] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1423, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1426] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1427, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1427] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1428, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1428] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1428, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1429] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1431, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1430] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1431, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1431] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1431, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1432] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1433] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1434] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1435] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1434, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1436] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1437] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1438] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1439] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1440] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1441] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1442] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1443] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1444] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1445, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1445] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1445, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1446] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1445, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1447] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1445, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1448] Ref(StatisticsReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1445, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1449] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1448, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1450] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1451] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1452] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1451, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1453] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1454] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1457, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1455] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1457, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1456] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1457, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1457] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1457, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1458] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1457, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1459] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1457, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1460] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1461] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1462] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1463] Ref(StatisticsReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1464] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1465] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1466] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1464, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1467] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1466, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1468] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1466, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1469] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1466, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1470] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1468, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1471] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1468, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1472] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1468, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1473] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1474] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1475] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1476] Ref(SubscriptionReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 9, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1477] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1478] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1483, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1479] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1483, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1480] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1483, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1481] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1487, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1482] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1490, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1483] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1490, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1484] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1490, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1485] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1490, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1486] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1490, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1487] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1488] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1492, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1489] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1494, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1490] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1494, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1491] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1494, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1492] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1497, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1493] Ref(RefreshKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1497, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1494] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1497, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1495] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1496] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1497] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1498] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1499] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1500] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1501, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1501] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1501, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1502] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1501, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1503] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1503, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1504] Ref(SkipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1503, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1505] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1506] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1507] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1508] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1509] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1511, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1510] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1511, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1511] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1511, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1512] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1513] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1514] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1515] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1516] Ref(Current_sessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1517] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1518, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1518] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1518, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1519] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1518, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1520] Ref(SubscriptionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1518, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1521] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1521, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1522] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1521, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1523] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1521, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1524] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1523, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1525] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1527, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1526] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1527, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1527] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1527, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1528] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1529, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1529] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1529, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1530] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1529, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1531] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1532] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1533] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1533, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1534] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1537, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1535] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1537, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1536] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1537, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1537] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1537, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1538] Ref(OnKeywordAsIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1537, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1539] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1539, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1540] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1539, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1541] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1541, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1542] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1541, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1543] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1541, child_count: 23, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1544] OneOf(23 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1564, child_count: 7, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1545] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1571, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1546] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1571, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1547] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1571, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1548] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1571, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1549] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1571, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1550] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1571, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1551] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1573, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1552] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1573, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1553] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1573, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1554] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1574, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1555] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1574, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1556] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1557] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1558] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1559] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1560] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1561] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1579, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1562] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1563] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1564] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1565] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1583, child_count: 13, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1566] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1596, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1567] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1601, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1568] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1569] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1570] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1571] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1572] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1603, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1573] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1574] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1575] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1605, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1576] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1577] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1578] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1608, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1579] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1608, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1580] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1581] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1582] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1583] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1612, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1584] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1585] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1586] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1587] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1588] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1589] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1618, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1590] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1618, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1591] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1618, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1592] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1621, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1593] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1623, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1594] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1623, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1595] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1623, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1596] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1623, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1597] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1623, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1598] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1599] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1600] Ref(ExpressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1601] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1626, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1602] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1632, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1603] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1632, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1604] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1632, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1605] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1606] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1607] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1608] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1609] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1610] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1611] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1636, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1612] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1639, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1613] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1640, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1614] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1640, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1615] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1616] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1617] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1647, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1618] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1647, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1619] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1647, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1620] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1621] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1622] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1651, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1623] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1651, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1624] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1651, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1625] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1653, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1626] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1653, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1627] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1653, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1628] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1655, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1629] Ref(RestartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1655, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1630] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1657, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1631] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1657, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1632] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1657, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1633] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1660, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1634] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1660, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1635] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1660, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1636] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1660, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1637] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1638] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1639] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1640] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1641] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1665, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1642] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1665, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1643] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1665, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1644] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1645] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1646] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1647] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1670, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1648] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1670, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1649] Ref(StorageKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1670, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1650] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1674, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1651] Ref(PlainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1674, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1652] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1674, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1653] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1674, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1654] Ref(MainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1674, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1655] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1677, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1656] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1677, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1657] Ref(CompressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1677, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1658] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1677, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1659] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1660] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1661] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1662] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1681, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1663] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1681, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1664] Ref(TableConstraintUsingIndexSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1681, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1665] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1666] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1667] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1668] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1686, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1669] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1670] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1671] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1690, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1672] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1690, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1673] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1690, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1674] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1692, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1675] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1694, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1676] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1694, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1677] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1694, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1678] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1679] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1680] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1681] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1699, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1682] Ref(ValidateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1699, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1683] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1699, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1684] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1699, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1685] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1704, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1686] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1704, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1687] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1704, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1688] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1704, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1689] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1704, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1690] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1704, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1691] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1707, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1692] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1693] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1694] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1695] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1709, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1696] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1712, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1697] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1712, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1698] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1712, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1699] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1712, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1700] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1716, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1701] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1716, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1702] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1718, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1703] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1718, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1704] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1718, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1705] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1718, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1706] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1718, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1707] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1721, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1708] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1725, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1709] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1725, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1710] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1725, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1711] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1727, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1712] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1727, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1713] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1727, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1714] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1715] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1716] Ref(RuleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1717] Ref(RuleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1718] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1729, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1719] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1733, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1720] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1737, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1721] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1737, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1722] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1737, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1723] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1737, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1724] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1725] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1726] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1727] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1728] Ref(LevelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1729] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1730] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1742, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1731] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1742, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1732] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1742, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1733] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1742, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1734] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1745, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1735] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1745, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1736] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1745, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1737] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1738] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1739] Ref(OidsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1740] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1750, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1741] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1750, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1742] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1750, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1743] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1750, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1744] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1752, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1745] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1752, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1746] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1754, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1747] Ref(LoggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1754, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1748] Ref(UnloggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1754, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1749] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1750] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1751] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1756, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1752] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1753] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1754] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1758, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1755] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1756] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1757] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1758] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1761, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1759] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1763, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1760] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1763, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1761] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1763, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1762] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1763] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1764] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1765] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1768, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1766] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1768, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1767] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1768, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1768] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1772, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1769] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1772, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1770] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1772, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1771] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1772, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1772] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1772, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1773] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1775, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1774] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1775, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1775] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1775, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1776] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1779, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1777] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1779, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1778] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1779] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1780] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1781] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1782] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1783] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1784] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1786, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1785] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1786, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1786] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1786, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1787] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1786, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1788] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1786, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1789] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1790, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1790] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1791] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1792] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1793, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1793] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1794] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1795] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1796] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1797] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1798] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1799] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1800] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1801] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1801, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1802] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1803, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1803] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1805, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1804] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1807, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1805] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1807, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1806] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1807, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1807] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1807, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1808] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1807, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1809] Ref(AlterTableDropColumnGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1807, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1810] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1810, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1811] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1810, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1812] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1813] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1814] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1815] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1816] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1815, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1817] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1815, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1818] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1815, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1819] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1818, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1820] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1821] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1822] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1823] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1824] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1823, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1825] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1826] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1827] Ref(AlterTableActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1828] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1829] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1830] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1831] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1832] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1833] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1834] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1838, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1835] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1838, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1836] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1838, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1837] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1838, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1838] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1838, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1839] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1838, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1840] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1841, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1841] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1841, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1842] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1841, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1843] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1845, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1844] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1845] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1846] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1847] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1848, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1848] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1851, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1849] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1851, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1850] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1851, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1851] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1851, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1852] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1855, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1853] Ref(AttachKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1855, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1854] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1855, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1855] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1855, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1856] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1857] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1858] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1859] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1860] Ref(PartitionBoundSpecSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1861] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1862] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1863] Ref(DetachKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1864] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1865] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1866] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1867] Ref(FinalizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1865, child_count: 9, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1868] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1874, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1869] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1874, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1870] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1874, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1871] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1874, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1872] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1874, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1873] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1877, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1874] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1877, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1875] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1877, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1876] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1879, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1877] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1879, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1878] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1879, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1879] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1879, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1880] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1879, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1881] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1879, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1882] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1885, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1883] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1885, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1884] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1885, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1885] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1885, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1886] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1885, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1887] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1885, child_count: 8, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1888] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1889] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1899, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1890] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1899, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1891] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1899, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1892] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1899, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1893] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1901, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1894] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1901, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1895] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1901, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1896] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1897] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1903, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1898] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1899] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1900] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1901] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1902] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1911, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1903] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1911, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1904] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1911, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1905] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1906] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1913, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1907] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1908] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1909] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1910] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1911] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1912] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1913] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1919, child_count: 8, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1914] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1915] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1916] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1917] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1918] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1929, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1919] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1929, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1920] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1929, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1921] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1929, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1922] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1929, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1923] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1929, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1924] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1925] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1926] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1927] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1928] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1934, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1929] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1936, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1930] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1936, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1931] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1932] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1933] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1934] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1935] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1936] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1937] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1938] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1946, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1939] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1946, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1940] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1946, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1941] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1946, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1942] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1946, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1943] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1949, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1944] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1949, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1945] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1949, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1946] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1949, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1947] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1948] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1949] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1950] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1951] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1952] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1953] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1957, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1954] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1955] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1956] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1957] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1958] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1965, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1959] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1960] Ref(DependsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1961] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1962] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1963] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1964] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1965] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1966] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1967] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1968] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1975, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1969] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1978, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1970] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1978, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1971] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1978, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1972] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1982, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1973] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1982, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1974] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1982, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1975] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1982, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1976] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1982, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1977] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1978] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1979] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1980] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1981] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1982] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1983] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1990, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1984] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1990, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1985] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1990, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1986] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1990, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1987] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1988] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1989] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1990] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1991] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1995, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1992] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1999, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1993] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2005, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1994] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2005, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1995] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2005, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1996] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2005, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1997] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2005, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1998] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2007, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1999] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2007, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2000] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2007, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2001] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2007, child_count: 8, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2002] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2015, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2003] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2015, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2004] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2015, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2005] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2015, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2006] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2017, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2007] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2017, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2008] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2017, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2009] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2017, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2010] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2017, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2011] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2012] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2013] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2014] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2019, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2015] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2016] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2017] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2024, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2018] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2019] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2024, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2020] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2021] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2022] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2023] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2024] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2025] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2026] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2027] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2030, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2028] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2029] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2030] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2031] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2032] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2035, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2033] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2037, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2034] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2035] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2036] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2037] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2038] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2044, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2039] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2044, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2040] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2044, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2041] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2044, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2042] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2044, child_count: 7, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2043] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2051, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2044] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2055, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2045] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2055, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2046] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2055, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2047] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2055, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2048] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2057, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2049] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2060, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2050] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2060, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2051] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2060, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2052] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2053] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2054] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2055] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2056] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2057] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2066, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2058] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2066, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2059] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2066, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2060] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2069, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2061] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2069, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2062] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2069, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2063] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2064] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2065] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2066] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2067] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2068] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2069] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2078, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2070] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2071] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2072] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2073] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2074] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2081, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2075] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2081, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2076] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2081, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2077] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2081, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2078] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2079] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2080] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2081] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2082] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2086, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2083] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2086, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2084] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2089, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2085] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2091, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2086] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2087] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2093, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2088] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2089] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2090] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2095, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2091] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2097, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2092] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2097, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2093] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2100, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2094] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2102, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2095] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2102, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2096] StringParser("ALWAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2102, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2097] StringParser("&")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2102, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2098] StringParser("ANALYSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2102, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2099] StringParser("ANALYZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2102, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2100] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2105, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2101] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2107, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2102] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2107, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2103] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2107, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2104] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2109, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2105] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2112, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2106] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2114, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2107] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2116, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2108] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2118, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2109] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2118, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2110] Ref(Skip_lockedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2118, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2111] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2118, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2112] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2118, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2113] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2120, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2114] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2122, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2115] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2122, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2116] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2125, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2117] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2118] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2119] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2120] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2121] StringParser("ANTI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2122] StringParser("ANY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2123] StringParser("APPEND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2124] StringParser("APPLY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 11, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2125] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2138, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2126] Ref(PlusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2138, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2127] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2138, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2128] Ref(DivideSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2138, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2129] Ref(MultiplySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2138, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2130] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2138, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2131] Ref(BitwiseAndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2138, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2132] Ref(BitwiseOrSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2138, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2133] Ref(BitwiseXorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2138, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2134] Ref(BitwiseLShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2138, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2135] Ref(BitwiseRShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2138, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2136] Ref(PowerOperatorSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2138, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2137] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2141, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2138] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2145, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2139] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2145, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2140] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2145, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2141] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2145, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2142] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2148, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2143] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2144] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2145] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2146] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2147] Ref(SliceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2151, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2148] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2149] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2150] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2151] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2152] StringParser("[")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2153] StringParser("]")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2154] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2155] StringParser("ARRAY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2154, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2156] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(13), first_child_idx: 2157, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2157] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2159, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2158] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2159, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2159] Ref(ArrayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2159, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2160] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2161] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2162] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2166, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2163] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2166, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2164] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2166, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2165] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2166] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2167] StringParser("AS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2168] StringParser("ASC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2169] StringParser("ASOF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2170] StringParser("ASSERTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2171] StringParser("ASSIGNMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2172] StringParser("ASYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2173] StringParser("AT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2174] StringParser("ATOMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2175] StringParser("ATTACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2176] StringParser("ATTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2177] StringParser("AUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2178] StringParser("AUTHORIZATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2179] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2168, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2180] Ref(Auto_incrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2168, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2181] StringParser("AUTO_INCREMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2168, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2182] StringParser("AVG_ROW_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2168, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2183] StringParser("BACKUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2168, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2184] StringParser("BACKWARD")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2168, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2185] MultiStringParser(12 templates)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2168, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2186] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2171, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2187] Ref(MapLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2171, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2188] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2173, child_count: 7, min_times: 0, first_terminator_idx: 12, terminator_count: 2, _padding: 0 }, // [2189] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2190] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2191] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2192] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2193] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2194] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2195] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2180, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2196] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2197] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2198] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2199] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2200] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2201] Ref(CompositeValueExpansionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2202] Ref(ColumnIndexSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2203] Token("file")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2204] Token("base")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2205] StringParser("BEFORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2206] StringParser("BEGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2207] StringParser("BERNOULLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2208] StringParser("BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2209] StringParser("BIGINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2210] StringParser("BIGSERIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2211] StringParser("BINARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 5, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2212] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2213] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2214] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2215] Ref(BooleanBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2216] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2217] Ref(JsonOperatorSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2218] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2219] StringParser("BINDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2220] StringParser("BIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2221] StringParser("BITVAR")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2222] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2187, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2223] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2224] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2225] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2226] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2227] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2191, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2228] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2191, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2229] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2191, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2230] StringParser("^")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2191, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2231] StringParser("BOOL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2191, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2232] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2193, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2233] Ref(AndOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2193, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2234] Ref(OrOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2193, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2235] StringParser("BOOLEAN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2193, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2236] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2195, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2237] Ref(TrueSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2195, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2238] Ref(FalseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2195, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2239] StringParser("BOTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2195, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2240] StringParser("BOX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2195, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2241] StringParser("BPCHAR")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2195, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2242] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2198, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2243] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2200, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2244] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2200, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2245] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2203, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2246] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2205, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2247] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2205, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2248] Token("bracketed")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2205, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2249] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2208, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2250] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2208, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2251] StringParser("BREADTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2208, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2252] StringParser("BREAK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2208, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2253] StringParser("BROWSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2208, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2254] StringParser("BUFFERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2208, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2255] StringParser("BULK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2208, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2256] StringParser("BY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2208, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2257] StringParser("BYPASSRLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2208, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2258] StringParser("BYTEA")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2208, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2259] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2260] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2211, child_count: 6, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2261] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2217, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2262] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2217, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2263] Ref(CTEColumnList)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2217, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2264] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2217, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2265] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2219, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2266] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2219, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2267] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2219, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2268] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2269] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2222, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2270] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 7, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2271] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2231, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2272] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2231, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2273] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2274] Ref(BreadthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2275] Ref(DepthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2276] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2277] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2278] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2279] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2280] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 6, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2281] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2282] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2283] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2284] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2285] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2286] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2287] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2288] StringParser("CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2289] StringParser("CALL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2239, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2290] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2291] Ref(CallKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2292] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2293] StringParser("CALLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2294] StringParser("CASCADE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2241, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2295] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2243, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2296] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2243, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2297] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2243, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2298] StringParser("CASCADED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2243, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 3, _padding: 0 }, // [2299] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2245, child_count: 6, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2300] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2301] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2302] Meta("implicit_indent")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 2251, child_count: 1, min_times: 0, first_terminator_idx: 14, terminator_count: 2, _padding: 0 }, // [2303] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2252, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2304] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2252, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2305] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2252, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2306] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 2252, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 1, _padding: 0 }, // [2307] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2252, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2308] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2252, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2309] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2252, child_count: 7, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2310] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2259, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2311] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2259, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2312] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 2259, child_count: 1, min_times: 0, first_terminator_idx: 17, terminator_count: 2, _padding: 0 }, // [2313] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2314] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2315] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2316] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 1, _padding: 0 }, // [2317] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2318] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2319] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2320] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2321] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2322] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2323] StringParser("CASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2324] StringParser("CAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2325] StringParser("::")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2326] StringParser("CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2327] StringParser("CHAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2328] StringParser("CHANGE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2329] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2330] StringParser("CHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2331] StringParser("CHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2332] StringParser("CHARACTERISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2333] StringParser("CHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2334] StringParser("CHECKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2335] StringParser("CHECKPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2336] StringParser("CHECKSUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2337] StringParser("CIDR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2338] StringParser("CIRCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2339] StringParser("CIRCULARSTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2340] StringParser("CLASS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2341] StringParser("CLOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2260, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2342] StringParser("CLUSTER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2260, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2343] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2344] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2345] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2263, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2346] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2347] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2348] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2267, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2349] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2350] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2351] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2352] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2272, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2353] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2272, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2354] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2272, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2355] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2272, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2356] StringParser("CLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2272, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2357] StringParser("COALESCE")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2272, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2358] Token("raw")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2272, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2359] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2272, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2360] StringParser("COLLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2272, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2361] StringParser("COLLATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2272, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2362] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2274, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2363] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2274, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 1, _padding: 0 }, // [2364] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2276, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2365] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2276, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2366] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2276, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2367] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2276, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2368] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2276, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2369] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2276, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2370] StringParser(":")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2276, child_count: 4, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2371] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2372] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2373] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2374] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2375] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 1, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2376] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2281, child_count: 7, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2377] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2288, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2378] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2290, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2379] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2290, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2380] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2290, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2381] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2292, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2382] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2292, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2383] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2295, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2384] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2295, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2385] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2297, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2386] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2297, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2387] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2388] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2389] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2390] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2299, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2391] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2301, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2392] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2301, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2393] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2301, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2394] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2301, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2395] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2303, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2396] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2303, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2397] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2303, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2398] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2399] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2400] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2306, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2401] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2402] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2308, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2403] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2404] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2405] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2406] Nothing
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2407] TypedParser("column_index")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2408] StringParser("COLUMN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2310, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2409] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2312, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2410] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 2312, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2411] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2314, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2412] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2316, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2413] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2316, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2414] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2318, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2415] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2318, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2416] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2318, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 12, _padding: 0 }, // [2417] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2320, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2418] Ref(SingleIdentifierFullGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2320, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2419] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2322, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2420] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2322, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2421] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2422] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2423] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2424] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2425] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2426] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2427] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2428] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2429] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2430] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2431] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2432] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2433] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2434] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2435] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2327, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2436] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2327, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2437] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2327, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2438] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2327, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2439] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2440] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2331, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2441] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2442] Ref(WildcardExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2443] Ref(LambdaExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2444] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2445] Ref(ColumnsExpressionNameGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2446] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2336, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2447] Ref(ColumnsExpressionFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2336, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2448] Ref(ColumnsExpressionFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2336, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2449] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2338, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2450] Ref(UnpackingOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2338, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2451] Ref(ColumnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2338, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2452] StringParser("COLUMNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2338, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2453] StringParser(",")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2338, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2454] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2340, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2455] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2340, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2456] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2340, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2457] StringParser("COMMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2340, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2458] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2343, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2459] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2343, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2460] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2343, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2461] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2345, child_count: 11, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2462] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2463] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2358, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2464] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2360, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2465] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2360, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2466] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2360, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2467] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2360, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2468] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2469] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2362, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2470] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2365, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2471] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2368, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2472] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2368, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2473] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2368, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2474] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2368, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2475] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2476] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2477] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2370, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2478] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2373, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2479] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2373, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2480] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2373, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2481] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2482] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2483] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2484] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2376, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2485] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2486] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2487] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2378, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2488] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2380, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2489] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2380, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2490] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2380, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2491] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2383, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2492] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2383, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2493] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2383, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2494] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2384, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2495] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2384, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2496] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2386, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2497] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2386, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2498] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2386, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2499] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2500] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2501] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2502] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2391, child_count: 24, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2503] OneOf(24 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2504] Ref(CollationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2505] Ref(ConversionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2506] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2507] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2508] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2509] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2510] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2511] Ref(RuleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2512] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2513] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2514] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2515] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2516] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2517] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2518] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2519] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2417, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2520] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2417, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2521] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2417, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2522] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2523] Ref(EventKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2524] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2525] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2526] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2527] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2422, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2528] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2422, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2529] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2530] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2531] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2424, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2532] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2533] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2534] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2426, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2535] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2536] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2537] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2538] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2539] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2432, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2540] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2432, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2541] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2432, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2542] Ref(DictionaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2432, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2543] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2544] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2545] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2546] Ref(ParserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2435, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2547] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2438, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2548] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2438, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2549] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2438, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2550] Ref(TemplateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2438, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2551] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2438, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2552] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2440, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2553] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2440, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2554] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2440, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2555] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2443, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2556] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2446, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2557] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2446, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2558] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2446, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2559] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2446, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2560] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2446, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2561] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2449, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2562] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2450, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2563] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2450, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2564] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2452, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2565] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2452, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2566] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2454, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2567] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2454, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2568] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2454, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2569] Token("comment")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2454, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2570] StringParser("COMMENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2454, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2571] StringParser("COMMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2454, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2572] StringParser("COMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2454, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2573] StringParser("COMMUTATOR")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2454, child_count: 10, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2574] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2575] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2576] Ref(GreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2577] Ref(LessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2578] Ref(GreaterThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2579] Ref(LessThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2580] Ref(NotEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2581] Ref(LikeOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2582] Ref(IsDistinctFromGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2583] Ref(EqualsSegment_a)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2584] Ref(GlobOperatorSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2585] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2586] StringParser("COMPLETION")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2587] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2588] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2589] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2467, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2590] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2591] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2592] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2593] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2594] StringParser("COMPOUNDCURVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2595] StringParser("COMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2596] StringParser("COMPRESSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2597] StringParser("COMPRESSION_LEVEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2598] StringParser("COMPUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2599] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2472, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2600] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2472, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2601] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2472, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2602] StringParser("CONCURRENTLY")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2472, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2603] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2472, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2604] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2475, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2605] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2475, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2606] Ref(ConditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2475, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2607] Ref(NonStandardJoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2475, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2608] StringParser("CONFIGURATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2475, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2609] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2477, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2610] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2477, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2611] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2479, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2612] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2479, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2613] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2487, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2614] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2487, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2615] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2487, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2616] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2489, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2617] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2618] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2495, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2619] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2495, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2620] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2495, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2621] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2497, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2622] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2497, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2623] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2497, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2624] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2501, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2625] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2626] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2506, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2627] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2506, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2628] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2506, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2629] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2506, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2630] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2509, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2631] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2511, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2632] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2633] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2634] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2513, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2635] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2516, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2636] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2519, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2637] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2521, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2638] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2521, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2639] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2521, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2640] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2641] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2524, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2642] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2526, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2643] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2526, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2644] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2526, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2645] StringParser("CONFLICT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2526, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2646] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2528, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2647] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2530, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2648] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2533, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2649] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2535, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2650] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2538, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2651] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2652] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2541, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2653] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2544, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2654] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2544, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2655] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2544, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2656] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2657] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2658] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2659] Ref(OperationClassReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2546, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2660] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2661] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2662] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2548, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2663] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2664] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2665] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2666] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2667] StringParser("CONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2668] StringParser("CONNECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2669] StringParser("CONSTRAINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2670] StringParser("CONSTRAINTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2671] StringParser("CONTAINSTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2672] StringParser("CONTENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2673] StringParser("CONTINUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2674] StringParser("CONVERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2675] StringParser("COPY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2551, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2676] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2553, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2677] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2553, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2678] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2555, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2679] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2559, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2680] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2561, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2681] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2563, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2682] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2563, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2683] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2566, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2684] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2568, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2685] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2568, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2686] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2687] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2688] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2689] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2571, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2690] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2691] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2573, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2692] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2576, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2693] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2578, child_count: 11, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2694] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2589, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2695] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2591, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2696] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2591, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2697] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2591, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2698] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2699] Ref(Overwrite_or_ignoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2700] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2701] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2702] Ref(OverwriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2703] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2595, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2704] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2597, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2705] Ref(AppendKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2597, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2706] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2597, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2707] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2599, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2708] Ref(Partition_byKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2599, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2709] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2601, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2710] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2604, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2711] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2606, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2712] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2606, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2713] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2606, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2714] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2608, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2715] Ref(Write_partition_columnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2608, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2716] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2608, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2717] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2610, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2718] Ref(CompressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2610, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2719] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2612, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2720] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2612, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2721] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2612, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2722] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2614, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2723] Ref(Compression_levelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2614, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2724] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2614, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2725] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2616, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2726] Ref(Row_group_size_bytesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2616, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2727] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2616, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2728] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2729] Ref(Row_group_sizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2730] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2618, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2731] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2732] Ref(Parquet_versionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2733] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2734] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2735] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2736] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2624, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2737] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2626, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2738] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2626, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2739] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2629, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2740] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2631, child_count: 11, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2741] AnySetOf(11 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2642, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2742] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2644, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2743] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2644, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2744] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2644, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2745] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2646, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2746] Ref(FreezeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2646, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2747] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2646, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2748] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2648, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2749] Ref(DelimiterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2648, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2750] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2648, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2751] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2650, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2752] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2650, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2753] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2650, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2754] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2652, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2755] Ref(HeaderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2652, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2756] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2652, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2757] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2654, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2758] Ref(QuoteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2654, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2759] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2654, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2760] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2656, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2761] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2656, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2762] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2656, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2763] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2658, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2764] Ref(Force_quoteKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2658, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2765] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2766] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2663, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2767] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2665, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2768] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2665, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2769] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2665, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2770] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2667, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2771] Ref(Force_not_nullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2667, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2772] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2670, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2773] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2672, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2774] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2672, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2775] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2674, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2776] Ref(Force_nullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2674, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2777] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2677, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2778] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2679, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2779] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2679, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2780] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2681, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2781] Ref(EncodingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2681, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2782] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2681, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2783] StringParser("CORRELATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2681, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2784] StringParser("COST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2681, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2785] StringParser("COSTS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2681, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2786] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2687, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2787] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2687, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2788] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2687, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2789] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2687, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2790] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2687, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2791] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2690, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2792] Anything
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2690, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2793] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2690, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2794] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2695, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2795] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2695, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2796] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2695, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2797] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2700, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2798] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2700, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2799] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2700, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2800] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2700, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2801] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2703, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2802] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2707, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2803] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2707, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2804] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2707, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2805] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2707, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2806] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2707, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2807] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2709, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2808] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2709, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2809] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2709, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2810] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2711, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2811] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2711, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2812] Ref(InoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2711, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2813] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2713, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2814] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2715, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2815] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2715, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2816] Ref(AssignmentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2715, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2817] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2717, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2818] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2717, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2819] Ref(ImplicitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2717, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2820] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2722, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2821] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2722, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2822] Ref(CollationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2722, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2823] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2722, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2824] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2722, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2825] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2724, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2826] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2727, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2827] Delimited(6 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2729, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2828] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2735, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2829] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2738, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2830] Ref(LocaleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2738, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2831] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2738, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2832] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2738, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2833] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2741, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2834] Ref(Lc_collateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2741, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2835] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2741, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2836] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2741, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2837] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2744, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2838] Ref(Lc_ctypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2744, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2839] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2744, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2840] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2744, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2841] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2747, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2842] Ref(ProviderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2747, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2843] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2747, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2844] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2749, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2845] Ref(IcuKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2749, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2846] Ref(LibcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2749, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2847] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2752, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2848] Ref(DeterministicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2752, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2849] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2752, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2850] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2752, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2851] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2852] Ref(VersionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2853] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2854] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2755, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2855] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2757, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2856] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2757, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2857] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2757, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2858] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2762, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2859] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2762, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2860] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2762, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2861] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2762, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2862] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2762, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2863] AnyNumberOf(8 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2770, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2864] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2773, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2865] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2773, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2866] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2773, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2867] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2773, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2868] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2776, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2869] Ref(TemplateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2776, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2870] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2776, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2871] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2776, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2872] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2779, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2873] Ref(EncodingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2779, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2874] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2779, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2875] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2781, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2876] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2781, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2877] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2781, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2878] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2783, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2879] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2786, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2880] Ref(LocaleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2786, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2881] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2786, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2882] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2786, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2883] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2788, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2884] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2791, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2885] Ref(Lc_collateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2791, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2886] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2791, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2887] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2791, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2888] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2794, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2889] Ref(Lc_ctypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2794, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2890] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2794, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2891] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2794, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2892] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2797, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2893] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2797, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2894] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2797, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2895] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2799, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2896] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2799, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2897] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2799, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2898] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2802, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2899] Ref(Allow_connectionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2802, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2900] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2802, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2901] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2802, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2902] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2806, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2903] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2806, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2904] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2806, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2905] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2806, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2906] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2806, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2907] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2809, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2908] Ref(Is_templateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2809, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2909] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2809, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2910] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2809, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2911] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2817, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2912] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2817, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2913] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2817, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2914] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2817, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2915] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2818, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2916] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2818, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2917] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2818, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2918] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2919] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2920] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2820, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2921] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2822, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2922] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2822, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2923] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2822, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2924] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2823, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2925] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2825, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2926] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2827, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2927] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2827, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2928] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2827, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2929] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2829, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2930] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2831, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2931] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2831, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2932] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2831, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2933] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2934] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2935] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 9, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2936] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2842, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2937] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2842, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2938] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2842, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2939] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2842, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2940] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2842, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2941] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2842, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2942] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2844, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2943] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2844, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2944] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2844, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2945] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2846, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2946] Ref(VersionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2846, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2947] Ref(VersionIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2846, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2948] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2848, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2949] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2848, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2950] Ref(VersionIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2848, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2951] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2848, child_count: 11, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2952] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2859, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2953] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2859, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2954] Ref(ForeignDataWrapperGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2859, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2955] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2859, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2956] Ref(HandlerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2859, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2957] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2859, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2958] Ref(ValidatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2859, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2959] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2859, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2960] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2961] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2962] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2963] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2862, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2964] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2864, child_count: 7, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2965] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2871, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2966] Ref(CreateForeignTableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2871, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2967] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2871, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2968] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2871, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2969] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2874, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2970] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2876, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2971] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2878, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2972] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2883, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2973] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2883, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2974] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2883, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2975] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2883, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2976] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2885, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2977] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2885, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2978] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2885, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2979] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2886, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2980] Ref(ForeignTableColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2886, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2981] Ref(ForeignTableTableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2886, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2982] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2888, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2983] Ref(InheritsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2888, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2984] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2891, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2985] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2893, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2986] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2893, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2987] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2895, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2988] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2895, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2989] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2895, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2990] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2895, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2991] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2992] Ref(CreateForeignTableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2993] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2994] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2901, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2995] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2906, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2996] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2906, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2997] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2906, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2998] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2906, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2999] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2909, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3000] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2911, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3001] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2913, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3002] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3003] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2916, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3004] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2918, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3005] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2918, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3006] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2918, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3007] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2919, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3008] Ref(ForeignTableColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2919, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3009] Ref(ForeignTableTableConstraintSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2919, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3010] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2921, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3011] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3012] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3013] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3014] Ref(PartitionBoundSpecSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3015] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3016] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2926, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3017] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2926, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3018] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2926, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3019] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2926, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3020] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2934, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3021] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2934, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3022] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2934, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3023] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2934, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3024] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3025] Ref(MacroKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3026] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3027] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3028] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3029] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3030] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2938, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3031] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2942, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3032] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2942, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3033] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2942, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3034] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2942, child_count: 15, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3035] Sequence(15 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3036] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3037] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3038] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3039] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2957, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3040] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2959, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3041] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2959, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3042] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2959, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3043] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2959, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3044] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2959, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3045] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2959, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3046] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2961, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3047] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2961, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3048] Ref(IndexAccessMethodSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2961, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3049] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2964, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3050] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2966, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3051] Ref(IndexElementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2966, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3052] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2968, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3053] Ref(IncludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2968, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3054] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2971, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3055] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2973, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3056] Ref(IndexElementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2973, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3057] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2976, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3058] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2976, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3059] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2976, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3060] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2976, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3061] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2978, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3062] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2978, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3063] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2978, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3064] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2980, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3065] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2980, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3066] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2980, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3067] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2982, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3068] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2982, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3069] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2982, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3070] StringParser("CREATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2982, child_count: 13, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3071] Sequence(13 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2995, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3072] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2995, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3073] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2995, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3074] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2995, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3075] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2995, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3076] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2995, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3077] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2995, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3078] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2995, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3079] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2997, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3080] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2997, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3081] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2997, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3082] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2999, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3083] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2999, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3084] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2999, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3085] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3001, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3086] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3001, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3087] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3001, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3088] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3001, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3089] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3005, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3090] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3007, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3091] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3010, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3092] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3010, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3093] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3012, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3094] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3015, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3095] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3017, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3096] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3017, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3097] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3017, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3098] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3017, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3099] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3019, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3100] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3022, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3101] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3024, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3102] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3024, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3103] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3024, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3104] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3024, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3105] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3032, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3106] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3032, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3107] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3032, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3108] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3032, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3109] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3032, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3110] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3032, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3111] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3034, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3112] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3034, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3113] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3037, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3114] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3039, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3115] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3042, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3116] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3042, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3117] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3042, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3118] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3044, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3119] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3044, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3120] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3047, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3121] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3049, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3122] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3049, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3123] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3049, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3124] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3049, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3125] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3126] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3127] Ref(OperatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3053, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3128] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3054, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3129] RegexParser
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3054, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3130] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3057, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3131] Delimited(6 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3059, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3132] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3065, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3133] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3068, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3134] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3135] Ref(LeftargKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3136] Ref(RightargKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3137] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3138] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3070, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3139] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3073, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3140] Ref(CommutatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3073, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3141] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3073, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3142] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3074, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3143] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3074, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3144] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3077, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3145] Ref(NegatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3077, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3146] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3077, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3147] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3078, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3148] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3078, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3149] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3081, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3150] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3084, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3151] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3084, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3152] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3084, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3153] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3086, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3154] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3086, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3155] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3086, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3156] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3086, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3157] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3086, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3158] Ref(HashesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3086, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3159] Ref(MergesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3086, child_count: 10, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3160] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3096, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3161] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3096, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3162] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3096, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3163] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3096, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3164] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3096, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3165] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3096, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3166] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3098, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3167] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3098, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3168] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3100, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3169] Ref(PermissiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3100, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3170] Ref(RestrictiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3100, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3171] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3102, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3172] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3102, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3173] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3107, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3174] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3107, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3175] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3107, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3176] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3107, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3177] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3107, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3178] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3107, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3179] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3109, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3180] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3109, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3181] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3111, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3182] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3183] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3184] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3185] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3186] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3187] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3116, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3188] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3118, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3189] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3118, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3190] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3121, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3191] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3121, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3192] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3124, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3193] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3124, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3194] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3124, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3195] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3127, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3196] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3127, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3197] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3198] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3199] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3200] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3201] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3202] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3203] Ref(FunctionDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3204] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3138, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3205] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3138, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3206] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3138, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3207] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3138, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3208] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3140, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3209] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3143, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3210] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3143, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3211] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3143, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3212] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3143, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3213] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3145, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3214] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3145, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3215] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3147, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3216] Ref(PublicationObjectsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3147, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3217] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3149, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3218] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3149, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3219] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3149, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3220] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3153, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3221] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3153, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3222] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3155, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3223] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3155, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3224] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3155, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3225] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3155, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3226] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3157, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3227] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3157, child_count: 16, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3228] AnySetOf(16 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3173, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3229] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3175, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3230] Ref(SuperuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3175, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3231] Ref(NosuperuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3175, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3232] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3177, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3233] Ref(CreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3177, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3234] Ref(NocreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3177, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3235] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3179, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3236] Ref(CreateroleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3179, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3237] Ref(NocreateroleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3179, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3238] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3181, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3239] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3181, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3240] Ref(NoinheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3181, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3241] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3183, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3242] Ref(LoginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3183, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3243] Ref(NologinKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3183, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3244] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3185, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3245] Ref(ReplicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3185, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3246] Ref(NoreplicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3185, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3247] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3187, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3248] Ref(BypassrlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3187, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3249] Ref(NobypassrlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3187, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3250] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3190, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3251] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3190, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3252] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3190, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3253] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3190, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3254] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3193, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3255] Ref(EncryptedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3193, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3256] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3193, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3257] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3195, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3258] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3195, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3259] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3195, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3260] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3198, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3261] Ref(ValidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3198, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3262] Ref(UntilKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3198, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3263] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3198, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3264] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3201, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3265] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3201, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3266] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3201, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3267] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3201, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3268] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3204, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3269] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3204, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3270] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3204, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3271] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3204, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3272] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3273] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3274] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3206, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3275] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3208, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3276] Ref(AdminKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3208, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3277] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3208, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3278] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3210, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3279] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3210, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3280] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3210, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3281] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3212, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3282] Ref(SysidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3212, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3283] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3212, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3284] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3216, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3285] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3216, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3286] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3216, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3287] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3216, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3288] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3218, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3289] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3221, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3290] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3221, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3291] Ref(AuthorizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3221, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3292] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3221, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3293] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3221, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3294] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3229, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3295] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3231, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3296] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3231, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3297] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3231, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3298] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3235, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3299] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3235, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3300] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3235, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3301] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3235, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3302] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3235, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3303] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3237, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3304] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3240, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3305] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3240, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3306] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3240, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3307] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3240, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3308] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3242, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3309] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3242, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3310] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3242, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3311] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3244, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3312] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3247, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3313] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3247, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3314] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3247, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3315] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3247, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3316] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3249, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3317] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3249, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3318] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3249, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3319] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3253, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3320] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3253, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3321] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3253, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3322] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3253, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3323] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3253, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3324] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3255, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3325] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3255, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3326] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3255, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3327] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3328] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3257, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3329] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3330] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3331] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3259, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3332] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3262, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3333] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3262, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3334] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3262, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3335] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3264, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3336] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3264, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3337] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3264, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3338] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3270, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3339] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3270, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3340] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3270, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3341] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3270, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3342] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3270, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3343] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3270, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3344] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3271, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3345] Ref(CreateSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3271, child_count: 9, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3346] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3280, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3347] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3280, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3348] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3280, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3349] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3280, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3350] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3280, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3351] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3352] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3353] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3282, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3354] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3284, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3355] Ref(VersionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3284, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3356] Ref(VersionIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3284, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3357] Ref(ForeignDataWrapperGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3284, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3358] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3284, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3359] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3284, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3360] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3292, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3361] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3292, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3362] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3292, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3363] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3294, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3364] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3294, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3365] Ref(StatisticsReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3294, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3366] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3297, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3367] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3299, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3368] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3303, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3369] Ref(DependenciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3303, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3370] Ref(McvKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3303, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3371] Ref(NdistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3303, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3372] Ref(CorrelationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3303, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3373] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3303, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3374] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3305, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3375] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3307, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3376] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3307, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3377] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3307, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3378] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3307, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3379] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3307, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3380] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3315, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3381] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3315, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3382] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3315, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3383] Ref(SubscriptionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3315, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3384] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3315, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3385] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3315, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3386] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3315, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3387] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3317, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3388] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3317, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3389] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3319, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3390] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3319, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3391] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3319, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3392] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3319, child_count: 7, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3393] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3326, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3394] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3326, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3395] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3326, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3396] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3326, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3397] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3326, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3398] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3326, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3399] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3326, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3400] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3328, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3401] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3330, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3402] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3330, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3403] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3332, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3404] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3335, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3405] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3335, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3406] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3338, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3407] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3340, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3408] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3342, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3409] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3344, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3410] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3344, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3411] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3346, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3412] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3348, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3413] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3348, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3414] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3349, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3415] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3350, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3416] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3350, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3417] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 3355, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3418] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3356, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3419] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3356, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3420] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3421] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3422] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3423] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3358, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3424] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3361, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3425] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3361, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3426] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3363, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3427] Ref(StoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3363, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3428] Ref(VirtualKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3363, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3429] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3363, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3430] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3431] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3432] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3433] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3434] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3435] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3436] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3372, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3437] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3374, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3438] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3377, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3439] Ref(ParserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3377, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3440] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3377, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3441] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3377, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3442] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3380, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3443] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3380, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3444] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3380, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3445] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3380, child_count: 11, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3446] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3391, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3447] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3391, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3448] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3391, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3449] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3391, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3450] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3391, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3451] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3391, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3452] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3394, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3453] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3394, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3454] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3394, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3455] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3456] Ref(InsteadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3457] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3396, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3458] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3398, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3459] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3402, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3460] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3402, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3461] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3402, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3462] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3402, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3463] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3404, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3464] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3404, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3465] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3406, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3466] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3406, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 2, _padding: 0 }, // [3467] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3408, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3468] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3408, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3469] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3408, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3470] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3408, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3471] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3408, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3472] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3408, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3473] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3408, child_count: 5, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3474] AnyNumberOf(5 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3413, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3475] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3415, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3476] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3415, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3477] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3415, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3478] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3417, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3479] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3419, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3480] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3419, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3481] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3419, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3482] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3421, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3483] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3421, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3484] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3423, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3485] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3425, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3486] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3425, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3487] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3425, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3488] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3427, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3489] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3427, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3490] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3427, child_count: 6, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3491] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3492] Ref(ReferencingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3433, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3493] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3435, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3494] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3435, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3495] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3435, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3496] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3435, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3497] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3435, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3498] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3435, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3499] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3439, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3500] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3441, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3501] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3441, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3502] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3441, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3503] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3441, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3504] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3441, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3505] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3441, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3506] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3444, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3507] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3444, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3508] Ref(EachKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3444, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3509] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3446, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3510] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3446, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3511] Ref(StatementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3446, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3512] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3448, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3513] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3448, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3514] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3451, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3515] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3451, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3516] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3454, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3517] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3454, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3518] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3456, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3519] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3456, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3520] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3456, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3521] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3456, child_count: 5, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3522] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3461, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3523] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3461, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3524] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3461, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3525] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3461, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3526] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3461, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3527] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3465, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3528] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3465, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3529] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3467, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3530] Ref(EnumKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3467, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3531] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3470, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3532] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3472, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3533] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3472, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3534] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3472, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3535] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3474, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3536] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3474, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3537] Ref(StructTypeSchemaSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3474, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3538] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3539] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3540] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3541] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 7, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3542] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3484, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3543] Ref(CreateUserMappingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3484, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3544] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3484, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3545] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3484, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3546] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3487, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3547] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3487, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3548] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3487, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3549] Ref(SessionInformationUserFunctionsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3487, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3550] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3487, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3551] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3487, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3552] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3487, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3553] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3490, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3554] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3490, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3555] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3490, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3556] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3490, child_count: 9, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3557] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3499, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3558] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3499, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3559] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3499, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3560] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3499, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3561] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3499, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3562] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3499, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3563] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3499, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3564] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3499, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3565] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3499, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3566] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3501, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3567] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3503, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3568] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3506, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3569] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3506, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3570] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3506, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3571] StringParser("CREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3506, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3572] StringParser("CREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3506, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3573] StringParser("CREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3506, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3574] StringParser("CROSS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3506, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3575] StringParser("CSV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3506, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3576] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3506, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3577] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3506, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3578] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3508, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3579] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3510, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3580] Ref(CubeFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3510, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3581] Ref(RollupFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3510, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3582] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3583] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3584] StringParser("CURRENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3585] StringParser("CURRENT_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3586] StringParser("CURRENT_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3587] StringParser("CURRENT_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3588] StringParser("CURRENT_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3589] StringParser("CURRENT_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3590] StringParser("CURRENT_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3591] StringParser("CURRENT_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3592] StringParser("CURSOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3593] StringParser("CURVEPOLYGON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3594] StringParser("CYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3595] StringParser("DATA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3596] StringParser("DATABASE")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3513, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 1, _padding: 0 }, // [3597] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3515, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3598] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3515, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3599] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3515, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3600] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3515, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3601] StringParser("DATABASES")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3515, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3602] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3517, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3603] RegexParser
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 3517, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3604] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3518, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3605] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3518, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3606] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 3521, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3607] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3523, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3608] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3523, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3609] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3523, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3610] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3529, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3611] Ref(WellKnownTextGeometrySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3529, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3612] Ref(DateTimeTypeIdentifier)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3529, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3613] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3529, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3614] Ref(MapTypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3529, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3615] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3530, child_count: 38, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3616] OneOf(38 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3568, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3617] Ref(SmallintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3568, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3618] Ref(IntegerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3568, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3619] Ref(IntKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3568, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3620] Ref(Int2KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3568, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3621] Ref(Int4KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3568, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3622] Ref(Int8KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3568, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3623] Ref(BigintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3568, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3624] Ref(Float4KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3568, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3625] Ref(Float8KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3568, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3626] Ref(RealKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3568, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3627] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3570, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3628] Ref(DoubleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3570, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3629] Ref(PrecisionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3570, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3630] Ref(SmallserialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3570, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3631] Ref(SerialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3570, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3632] Ref(Serial2KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3570, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3633] Ref(Serial4KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3570, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3634] Ref(Serial8KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3570, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3635] Ref(BigserialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3570, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3636] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3572, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3637] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3573, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3638] Ref(FloatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3573, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3639] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3573, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3640] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3575, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3641] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3577, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3642] Ref(DecimalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3577, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3643] Ref(NumericKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3577, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3644] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3577, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3645] Ref(MoneyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3577, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3646] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3579, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3647] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3581, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3648] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3587, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3649] Ref(BpcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3587, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3650] Ref(CharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3587, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3651] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3589, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3652] Ref(CharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3589, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3653] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3589, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3654] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3589, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3655] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3591, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3656] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3591, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3657] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3591, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3658] Ref(VarcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3591, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3659] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3591, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3660] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3591, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3661] Ref(ByteaKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3591, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3662] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3593, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3663] Ref(BooleanKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3593, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3664] Ref(BoolKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3593, child_count: 7, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3665] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3600, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3666] Ref(PointKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3600, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3667] Ref(LineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3600, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3668] Ref(LsegKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3600, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3669] Ref(BoxKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3600, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3670] Ref(PathKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3600, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3671] Ref(PolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3600, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3672] Ref(CircleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3600, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3673] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3604, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3674] Ref(CidrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3604, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3675] Ref(InetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3604, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3676] Ref(MacaddrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3604, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3677] Ref(Macaddr8KeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3604, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3678] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3606, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3679] Ref(TsvectorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3606, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3680] Ref(TsqueryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3606, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3681] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3609, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3682] Ref(BitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3609, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3683] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3610, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3684] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3610, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3685] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3610, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3686] Ref(UuidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3610, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3687] Ref(XmlKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3610, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3688] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3612, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3689] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3612, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3690] Ref(JsonbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3612, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3691] Ref(Int4rangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3612, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3692] Ref(Int8rangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3612, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3693] Ref(NumrangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3612, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3694] Ref(TsrangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3612, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3695] Ref(TstzrangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3612, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3696] Ref(DaterangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3612, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3697] Ref(Pg_lsnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3612, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3698] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3614, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3699] Ref(VectorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3614, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3700] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3614, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3701] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3616, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3702] Ref(DatatypeIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3616, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3703] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3616, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3704] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3619, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3705] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3620, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3706] Ref(ArrayTypeSuffixSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3620, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3707] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3620, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3708] Ref(SizedArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3620, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3709] StringParser("DATE")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3620, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3710] MultiStringParser(0 templates)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3620, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3711] Ref(DatePartFunctionName)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3620, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3712] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3621, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3713] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3624, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3714] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3626, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3715] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3628, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3716] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3628, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3717] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3628, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3718] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3630, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3719] Ref(DateTimeTypeIdentifier)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3630, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3720] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3630, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3721] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3633, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3722] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3633, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3723] Ref(TimeWithTZGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3633, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3724] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3635, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3725] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3638, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3726] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3638, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3727] Ref(TimetzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3638, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3728] Ref(TimestamptzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3638, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3729] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3730] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3731] StringParser("DATERANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3732] StringParser("DATETIME")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3733] MultiStringParser(33 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3734] StringParser("DAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3735] StringParser("DAY_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3736] StringParser("DAY_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3737] StringParser("DAY_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3738] StringParser("DAY_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3739] StringParser("DAYOFMONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3740] StringParser("DAYOFWEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3741] StringParser("DAYOFYEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3742] StringParser("DAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3743] StringParser("DBCC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3744] StringParser("DEALLOCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3641, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3745] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3644, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3746] Ref(DeallocateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3644, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3747] Ref(PrepareKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3644, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3748] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3646, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3749] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3646, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3750] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3646, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3751] StringParser("DEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3646, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3752] StringParser("DECIMAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3646, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3753] StringParser("DECLARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3646, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3754] StringParser("DEFAULT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3646, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3755] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3648, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3756] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3648, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3757] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3648, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3758] StringParser("DEFAULTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3648, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3759] StringParser("DEFERRABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3648, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3760] StringParser("DEFERRED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3648, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3761] StringParser("DEFINER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3648, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3762] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3650, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3763] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3650, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3764] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3650, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3765] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3652, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3766] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3652, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3767] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3654, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3768] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3654, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3769] Ref(DefinitionArgumentValueGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3654, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3770] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3657, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3771] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3659, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3772] Ref(DefinitionParameterSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3659, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3773] StringParser("DELAY_KEY_WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3659, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3774] StringParser("DELAYED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3659, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3775] StringParser("DELETE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3659, child_count: 10, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3776] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3669, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3777] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3669, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3778] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3669, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3779] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3669, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3780] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3669, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3781] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3669, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3782] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3669, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3783] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3673, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3784] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3673, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3785] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3675, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3786] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3677, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3787] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3677, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3788] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3677, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3789] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3677, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3790] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3679, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3791] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3683, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3792] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3683, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3793] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3683, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3794] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3683, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3795] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3683, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3796] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3683, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3797] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3687, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3798] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3687, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3799] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3689, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3800] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3689, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3801] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3691, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3802] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3693, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3803] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3693, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3804] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3693, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3805] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3693, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3806] StringParser("DELIMITER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3693, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3807] StringParser("DELIMITERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3693, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3808] StringParser("DENY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3693, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3809] StringParser("DEPENDENCIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3693, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3810] StringParser("DEPENDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3693, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3811] StringParser("DEPTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3693, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3812] StringParser("DESC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3693, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3813] StringParser("DESCRIBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3693, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3814] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3696, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3815] Ref(DescribeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3696, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3816] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3696, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3817] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3696, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3818] StringParser("DESTROY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3696, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3819] StringParser("DESTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3696, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3820] StringParser("DETACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3696, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3821] StringParser("DETERMINISTIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3696, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3822] StringParser("DICTIONARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3696, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3823] StringParser("DISABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3696, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3824] StringParser("DISABLE_PAGE_SKIPPING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3696, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3825] StringParser("DISCARD")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3696, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3826] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3698, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3827] Ref(DiscardKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3698, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3828] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3703, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3829] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3703, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3830] Ref(PlansKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3703, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3831] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3703, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3832] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3703, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3833] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3703, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3834] StringParser("DISK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3703, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3835] StringParser("DISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3703, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3836] StringParser("DISTINCTROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3703, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3837] StringParser("DISTRIBUTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3703, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3838] StringParser("DIV")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3703, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3839] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3705, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3840] StringParser("//")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3705, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3841] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3705, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3842] StringParser("DO")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3705, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3843] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3707, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3844] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3707, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3845] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3709, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3846] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3711, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3847] Ref(LanguageClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3711, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3848] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3711, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3849] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3713, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3850] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3713, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3851] Ref(LanguageClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3713, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3852] StringParser("DOCUMENT")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3713, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3853] TypedParser("dollar_numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3713, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3854] StringParser("DOMAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3713, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3855] StringParser(".")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3713, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3856] StringParser("DOUBLE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3713, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3857] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3718, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3858] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3718, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3859] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3718, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3860] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3718, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3861] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3720, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3862] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3722, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3863] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3722, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3864] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3725, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3865] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3725, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3866] Anything
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3725, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3867] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3725, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3868] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3725, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3869] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3727, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3870] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3727, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3871] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3727, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3872] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3873] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3874] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3875] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3876] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3737, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3877] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3737, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3878] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3737, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3879] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3737, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3880] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3737, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3881] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3742, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3882] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3742, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3883] Ref(CollationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3742, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3884] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3742, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3885] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3742, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3886] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3742, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3887] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3747, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3888] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3747, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3889] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3747, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3890] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3747, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3891] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3747, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3892] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3749, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3893] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3749, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3894] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3752, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3895] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3752, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3896] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3757, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3897] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3757, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3898] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3757, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3899] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3757, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3900] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3759, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3901] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3759, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3902] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3759, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3903] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3764, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3904] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3764, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3905] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3764, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3906] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3764, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3907] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3764, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3908] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3764, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3909] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3770, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3910] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3770, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3911] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3770, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3912] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3770, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3913] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3770, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3914] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3772, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3915] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3772, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3916] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3772, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3917] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3778, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3918] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3778, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3919] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3780, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3920] Ref(MacroKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3780, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3921] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3780, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3922] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3780, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3923] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3780, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3924] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3780, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3925] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3780, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3926] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3786, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3927] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3786, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3928] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3786, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3929] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3786, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3930] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3786, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3931] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3788, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3932] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3788, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3933] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3788, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3934] StringParser("DROP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3788, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3935] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3794, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3936] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3794, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3937] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3794, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3938] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3794, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3939] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3794, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3940] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3796, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3941] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3796, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3942] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3796, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3943] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3800, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3944] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3800, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3945] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3800, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3946] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3800, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3947] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3800, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3948] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3805, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3949] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3805, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3950] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3805, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3951] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3805, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3952] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3807, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3953] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3811, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3954] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3811, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3955] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3811, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3956] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3811, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3957] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3811, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3958] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3811, child_count: 7, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3959] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3818, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3960] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3818, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3961] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3818, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3962] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3818, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3963] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3818, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3964] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3818, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3965] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3818, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3966] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3818, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3967] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3823, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3968] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3823, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3969] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3823, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3970] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3823, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3971] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3825, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3972] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3827, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3973] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3827, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3974] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3827, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3975] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3829, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3976] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3829, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3977] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3829, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3978] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3979] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3980] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3834, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3981] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3834, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3982] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3836, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3983] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3836, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3984] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3836, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3985] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3840, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3986] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3840, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3987] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3840, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3988] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3840, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3989] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3840, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3990] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3845, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3991] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3845, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3992] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3845, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3993] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3845, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3994] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3845, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3995] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3845, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3996] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3850, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3997] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3850, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3998] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3850, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3999] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3850, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4000] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3852, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4001] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3852, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4002] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3852, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4003] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3857, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4004] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3857, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4005] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3857, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4006] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3857, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4007] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3859, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4008] Ref(StatisticsReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3859, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4009] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3861, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4010] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3861, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4011] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3861, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4012] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3866, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4013] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3866, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4014] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3866, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4015] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3866, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4016] Ref(SubscriptionReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3866, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4017] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3868, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4018] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3868, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4019] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3868, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4020] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3874, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4021] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3874, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4022] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3874, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4023] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3874, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4024] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3874, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4025] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3876, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4026] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3876, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4027] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3876, child_count: 7, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4028] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3883, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4029] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3883, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4030] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3883, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4031] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3883, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4032] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3883, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4033] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3883, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4034] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3883, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4035] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3883, child_count: 7, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4036] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3890, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4037] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3890, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4038] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3890, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4039] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3890, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4040] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3890, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4041] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3890, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4042] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3890, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4043] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3890, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4044] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3895, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4045] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3895, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4046] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3895, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4047] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3895, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4048] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3897, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4049] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3897, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4050] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3897, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4051] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3901, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4052] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3901, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4053] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3901, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4054] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3901, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4055] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3901, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4056] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3906, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4057] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3906, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4058] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3906, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4059] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3906, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4060] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4061] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4062] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4063] StringParser("DUMMY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4064] StringParser("DUMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4065] StringParser("EACH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3908, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4066] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3912, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4067] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3912, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4068] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3912, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4069] StringParser("ELSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3912, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4070] StringParser("ELSEIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3912, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4071] StringParser("EMPTY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3912, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4072] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3914, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4073] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4074] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4075] Ref(EmptyStructLiteralBracketsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4076] StringParser("ENABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4077] StringParser("ENCLOSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4078] StringParser("ENCODING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4079] StringParser("ENCRYPTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4080] StringParser("}")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4081] StringParser("END")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4082] StringParser("ENUM")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4083] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4084] StringParser("==")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4085] StringParser("ERRLVL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4086] StringParser("ESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4087] StringParser("ESCAPED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4088] StringParser("EVENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4089] StringParser("EXCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4090] StringParser("EXCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4091] StringParser("EXCLUDING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3916, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4092] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3919, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4093] Ref(IndexElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3919, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4094] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3919, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4095] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3919, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4096] StringParser("EXCLUSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3919, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4097] StringParser("EXECUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3919, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4098] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3922, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4099] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3922, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4100] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3922, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4101] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3925, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4102] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3927, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4103] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3927, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4104] StringParser("EXISTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3927, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4105] StringParser("EXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3927, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4106] StringParser("EXIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3927, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4107] StringParser("EXPLAIN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3927, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4108] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3929, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4109] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3931, child_count: 9, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4110] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3940, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4111] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3940, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4112] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3940, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4113] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3940, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4114] Ref(CostsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3940, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4115] Ref(SettingsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3940, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4116] Ref(BuffersKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3940, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4117] Ref(WalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3940, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4118] Ref(TimingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3940, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4119] Ref(SummaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3940, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4120] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3940, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4121] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3942, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4122] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3942, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4123] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3946, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4124] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3946, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4125] Ref(XmlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3946, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4126] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3946, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4127] Ref(YamlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3946, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4128] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3949, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4129] Ref(ExplainKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3949, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4130] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3951, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4131] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3953, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4132] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3955, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4133] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3955, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4134] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3955, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4135] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3955, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4136] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3958, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4137] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3960, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4138] Ref(ExplainOptionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3960, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4139] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4140] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4141] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4142] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4143] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4144] StringParser("EXPRESSION")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4145] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4146] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3966, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4147] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3966, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4148] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3967, child_count: 10, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4149] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3977, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4150] Ref(LikeExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3977, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4151] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3979, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4152] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3979, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4153] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3979, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4154] Ref(InOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3979, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4155] Ref(QuantifiedComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3979, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4156] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3982, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4157] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3982, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4158] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3982, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4159] Ref(IsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3982, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4160] Ref(IsNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3982, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4161] Ref(NotNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3982, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4162] Ref(CollateGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3982, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4163] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3987, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4164] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3987, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4165] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3987, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4166] Ref(Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3987, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4167] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3987, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4168] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3987, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4169] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3989, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4170] Ref(PatternMatchingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3989, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4171] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3989, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4172] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 3994, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4173] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3995, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4174] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4175] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4176] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4177] Ref(NotOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4178] Ref(PriorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4179] Ref(AbsoluteValueOperatorSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3996, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4180] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3998, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4181] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3998, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4182] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3999, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4183] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4000, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4184] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4002, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4185] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4005, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4186] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4005, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4187] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4005, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4188] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4005, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4189] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4005, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4190] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 4007, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4191] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4008, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4192] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4009, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4193] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4009, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4194] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4009, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4195] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4011, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4196] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4013, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4197] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4013, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4198] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4015, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 1, _padding: 0 }, // [4199] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4018, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4200] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4020, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4201] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4020, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4202] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4203] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4023, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4204] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4025, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4205] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4027, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4206] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4027, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4207] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4027, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4208] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4028, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4209] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4028, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4210] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4028, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4211] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4028, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4212] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4030, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4213] Ref(FullTextSearchOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4030, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4214] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 4030, child_count: 1, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4215] Ref(FactorialOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4031, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4216] Ref(NotEqualToSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4031, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4217] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4033, child_count: 9, min_times: 0, first_terminator_idx: 40, terminator_count: 1, _padding: 0 }, // [4218] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4042, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4219] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4042, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4220] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 4042, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4221] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4045, child_count: 4, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4222] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4049, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4223] Ref(LateralColumnAliasExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4049, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4224] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4049, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4225] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4049, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4226] Delimited(5 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4051, child_count: 5, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4227] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4056, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4228] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4056, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4229] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4056, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4230] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4056, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4231] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4056, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4232] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4056, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4233] Ref(Expression_D_Potential_Select_Statement_Without_Brackets)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4056, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4234] Ref(SingleIdentifierWildcardSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4056, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4235] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4058, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4236] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4060, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4237] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4060, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4238] Ref(MapTypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4060, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4239] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4063, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4240] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4065, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4241] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4065, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4242] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4067, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4243] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4067, child_count: 5, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4244] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4072, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4245] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4072, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4246] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4072, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4247] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4072, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4248] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4072, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4249] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4072, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4250] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4072, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4251] Ref(ListComprehensionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4072, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4252] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4072, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4253] Ref(AccessorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4072, child_count: 7, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4254] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4255] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4256] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4257] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4258] Ref(TypedStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4259] Ref(ArrayExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4260] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4261] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4262] StringParser("EXTENDED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4263] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4079, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4264] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4082, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4265] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4082, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4266] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4084, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4267] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4084, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4268] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4086, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4269] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4089, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4270] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4089, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4271] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4091, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4272] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4091, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4273] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4091, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4274] StringParser("EXTENSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4091, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4275] StringParser("EXTERNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4091, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4276] StringParser("EXTRACT")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4091, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4277] TypedParser("not")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4091, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4278] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4091, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4279] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4091, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4280] StringParser("FAMILY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4091, child_count: 5, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4281] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4096, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4282] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4096, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4283] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4098, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4284] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4098, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4285] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4098, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4286] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4100, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4287] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 4100, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4288] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4101, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4289] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4101, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4290] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4103, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4291] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4103, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4292] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4103, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4293] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4105, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4294] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4105, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4295] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4107, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4296] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4107, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4297] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4107, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4298] StringParser("FETCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4107, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4299] StringParser("FIELDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4107, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4300] StringParser("FILE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4107, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4301] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4111, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4302] Ref(PostgresCopyStdinDataStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4111, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4303] Ref(PsqlCopyMetaCommandStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4111, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4304] Ref(PsqlSetMetaCommandStatementSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 4111, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4305] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4113, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4306] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4113, child_count: 1, min_times: 1, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4307] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4114, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4308] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4114, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4309] StringParser("FILLFACTOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4114, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4310] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4116, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4311] Ref(FilterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4116, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4312] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4119, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4313] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4121, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4314] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4121, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4315] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4121, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4316] StringParser("FILTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4121, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4317] StringParser("FINALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4121, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4318] StringParser("FIRST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4121, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4319] StringParser("FLOAT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4121, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4320] StringParser("FLOAT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4121, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4321] StringParser("FLOAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4121, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4322] StringParser("FLUSH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4121, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4323] StringParser("FOLLOWING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4121, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4324] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4125, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4325] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4125, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4326] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4129, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4327] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4129, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4328] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4132, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4329] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4132, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4330] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4132, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4331] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4132, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4332] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4132, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4333] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4334] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4335] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4134, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4336] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4136, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4337] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4136, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4338] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4339] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4138, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4340] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4140, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4341] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4140, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4342] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4142, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4343] Ref(SkipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4142, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4344] Ref(LockedKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4142, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4345] StringParser("FOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4142, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4346] StringParser("FORCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4142, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4347] StringParser("FORCE_NOT_NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4142, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4348] StringParser("FORCE_NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4142, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4349] StringParser("FORCE_QUOTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4142, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4350] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4351] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4352] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4145, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4353] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4145, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4354] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4147, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4355] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4147, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4356] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4147, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4357] StringParser("FOREIGN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4147, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4358] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4149, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4359] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4151, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4360] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4151, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4361] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4151, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4362] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4155, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4363] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4157, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4364] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4157, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4365] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4157, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4366] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4160, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4367] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4160, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4368] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4163, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4369] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4163, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4370] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4165, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4371] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4165, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4372] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4165, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4373] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4167, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4374] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4167, child_count: 5, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4375] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4172, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4376] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4172, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4377] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4172, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4378] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4172, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4379] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4172, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4380] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4172, child_count: 5, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4381] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4177, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4382] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4177, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4383] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4177, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4384] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4177, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4385] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4177, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4386] Ref(StoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4177, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4387] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4179, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4388] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4181, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4389] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4181, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4390] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4181, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4391] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4184, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4392] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4184, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4393] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4187, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4394] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4187, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4395] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4189, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4396] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4189, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4397] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4189, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4398] StringParser("FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4189, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4399] StringParser("FORWARD")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4189, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4400] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4192, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4401] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4192, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4402] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4194, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4403] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4196, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4404] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4198, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4405] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4198, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4406] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4198, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4407] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4200, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4408] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4204, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4409] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4204, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4410] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4204, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4411] Ref(UnboundedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4204, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4412] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4204, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4413] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4206, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4414] Ref(PrecedingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4206, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4415] Ref(FollowingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4206, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4416] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4210, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4417] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4210, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4418] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4210, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4419] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4212, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4420] Ref(ExcludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4212, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4421] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4216, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4422] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4218, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4423] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4218, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4424] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4218, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4425] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4218, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4426] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4218, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4427] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4220, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4428] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4220, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4429] Ref(OthersKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4220, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4430] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4223, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4431] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4223, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4432] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4223, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4433] Ref(GroupsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4223, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4434] StringParser("FREETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4223, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4435] StringParser("FREETEXTTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4223, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4436] StringParser("FREEZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4223, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4437] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4225, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4438] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4225, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4439] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4227, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4440] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4227, child_count: 13, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4441] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4240, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4442] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4240, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4443] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4240, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4444] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4242, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4445] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4242, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4446] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4242, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4447] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4244, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4448] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4244, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4449] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4244, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4450] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4244, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4451] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4244, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4452] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4244, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4453] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4244, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4454] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4244, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4455] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4244, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4456] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4244, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4457] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4244, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4458] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4244, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4459] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4246, child_count: 7, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4460] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4253, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4461] Ref(PreTableFunctionKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4253, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4462] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4255, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4463] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4258, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4464] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4258, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4465] Ref(TemporalQuerySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 4258, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4466] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4259, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4467] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4263, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4468] Ref(FromClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4263, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4469] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4263, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4470] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4263, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4471] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4263, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4472] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4266, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4473] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4266, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4474] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4266, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4475] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4266, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4476] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4266, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4477] Ref(PostTableExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4266, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4478] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4269, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4479] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4271, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4480] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4272, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4481] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4272, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4482] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4274, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4483] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4277, child_count: 6, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4484] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4283, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 2, _padding: 0 }, // [4485] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4286, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4486] Ref(MLTableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4286, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4487] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4286, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4488] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4489] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4289, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4490] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4291, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4491] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4291, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4492] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4291, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4493] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4293, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4494] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4293, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4495] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4293, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4496] Meta(conditional: indented_joins=True -> indent)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4293, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 2, _padding: 0 }, // [4497] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4294, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4498] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4295, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4499] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4297, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4500] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4297, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4501] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4297, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4502] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4299, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4503] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4299, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4504] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4299, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4505] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4301, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4506] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4301, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4507] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4301, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4508] Meta(conditional: indented_joins=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4301, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4509] StringParser("FROM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(6), first_child_idx: 4301, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4510] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4303, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4511] Ref(PivotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4303, child_count: 8, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4512] Bracketed(6 elements -> 6 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 4311, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4513] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4313, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4514] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4315, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4515] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4315, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4516] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4315, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4517] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4315, child_count: 1, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4518] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4316, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4519] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4319, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4520] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4319, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4521] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4319, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4522] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 4322, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4523] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4324, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4524] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4324, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4525] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4324, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4526] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4324, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4527] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4324, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4528] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4327, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4529] Ref(UnpivotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4327, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4530] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4329, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4531] Ref(IncludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4329, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4532] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4329, child_count: 5, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4533] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4334, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4534] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4336, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4535] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4336, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4536] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 4339, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4537] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4341, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4538] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4341, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4539] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4341, child_count: 1, min_times: 1, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4540] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4342, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4541] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4345, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4542] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4345, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4543] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4345, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4544] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 4348, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4545] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4350, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4546] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4352, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4547] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4354, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4548] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4356, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4549] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 4359, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4550] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4361, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4551] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4361, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4552] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4361, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4553] Ref(ColumnsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4361, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4554] StringParser("FULL")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4361, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4555] TypedParser("full_text_search_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4361, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4556] StringParser("FULLTEXT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4361, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4557] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4364, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4558] Ref(LambdaExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4364, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4559] Ref(NamedArgumentSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4364, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4560] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4364, child_count: 14, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4561] AnyNumberOf(14 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4378, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4562] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4378, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4563] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4380, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4564] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4383, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4565] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4383, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4566] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4386, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4567] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4386, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4568] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4386, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4569] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4386, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4570] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4390, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4571] Ref(TrimParametersGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 4390, child_count: 1, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4572] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4391, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4573] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4391, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4574] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4391, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4575] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4391, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4576] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4393, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4577] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4395, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4578] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4395, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4579] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4395, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4580] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4397, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4581] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4399, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4582] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4399, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4583] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4399, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4584] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4401, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4585] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4401, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4586] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4401, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4587] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4403, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4588] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4403, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4589] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4403, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4590] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4405, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4591] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4405, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4592] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4407, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4593] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4407, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4594] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4595] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4596] Ref(AggregateOrderByClause)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4409, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4597] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4411, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4598] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4411, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4599] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4411, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4600] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4414, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4601] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4418, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4602] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4418, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4603] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4418, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4604] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4418, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4605] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4418, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4606] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4418, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4607] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4422, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4608] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4422, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4609] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4422, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4610] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4422, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4611] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4422, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4612] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4422, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4613] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4422, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4614] Ref(EmptyStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4422, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4615] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4424, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4616] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4427, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4617] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4427, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4618] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4429, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4619] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4429, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4620] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4429, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4621] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4429, child_count: 1, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4622] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4430, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4623] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4433, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4624] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4433, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4625] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4435, child_count: 15, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4626] AnyNumberOf(15 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4450, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4627] Ref(LanguageClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4450, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4628] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4454, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4629] Ref(TransformKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4454, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4630] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4454, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4631] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4454, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4632] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4454, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4633] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4454, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4634] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4457, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4635] Ref(ImmutableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4457, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4636] Ref(StableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4457, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4637] Ref(VolatileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4457, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4638] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4459, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4639] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4459, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4640] Ref(LeakproofKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4459, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4641] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4462, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4642] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4466, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4643] Ref(CalledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4466, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4644] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4466, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4645] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4466, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4646] Ref(InputKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4466, child_count: 5, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4647] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4471, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4648] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4471, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4649] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4471, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4650] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4471, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4651] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4471, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4652] Ref(InputKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4471, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4653] Ref(StrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4471, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4654] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4474, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4655] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4474, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4656] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4474, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4657] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4476, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4658] Ref(InvokerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4476, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4659] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4476, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4660] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4478, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4661] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4478, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4662] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4481, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4663] Ref(UnsafeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4481, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4664] Ref(RestrictedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4481, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4665] Ref(SafeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4481, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4666] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4483, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4667] Ref(CostKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4483, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4668] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4483, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4669] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4485, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4670] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4485, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4671] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4485, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4672] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4487, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4673] Ref(SupportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4487, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4674] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4487, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4675] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4490, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4676] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4490, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4677] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4490, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4678] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4492, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4679] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4494, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4680] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4496, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4681] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4496, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4682] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4496, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4683] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4498, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4684] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4500, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4685] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4500, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4686] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4500, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4687] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4502, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4688] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4502, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4689] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4502, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4690] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4504, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4691] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4504, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4692] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4506, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4693] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4506, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4694] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4509, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4695] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4509, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4696] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4509, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4697] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4509, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4698] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4511, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4699] Ref(ReturnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4511, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4700] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4511, child_count: 6, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4701] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4517, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4702] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4517, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4703] Ref(AtomicKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4517, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4704] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4521, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4705] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4523, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4706] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4523, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4707] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4523, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4708] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4525, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4709] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4525, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4710] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4525, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4711] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4527, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4712] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4530, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4713] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4530, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4714] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4530, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4715] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4530, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4716] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4530, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4717] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4533, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4718] Ref(ReturnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4533, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4719] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4533, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4720] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4533, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4721] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4533, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4722] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4535, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4723] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4535, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4724] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4538, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4725] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4540, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4726] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4540, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4727] StringParser("FUNCTION")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4540, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4728] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4540, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4729] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4542, child_count: 1, min_times: 0, first_terminator_idx: 45, terminator_count: 1, _padding: 0 }, // [4730] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4543, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4731] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4545, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4732] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4545, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4733] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4545, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4734] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4545, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 1, _padding: 0 }, // [4735] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4547, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [4736] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4547, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [4737] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4547, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [4738] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4547, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4739] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4550, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4740] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4554, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4741] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4554, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4742] Ref(OutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4554, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4743] Ref(InoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4554, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4744] Ref(VariadicKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4554, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4745] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4556, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4746] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4556, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4747] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4559, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4748] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4559, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4749] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4563, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4750] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4563, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4751] Ref(OutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4563, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4752] Ref(InoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4563, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4753] Ref(VariadicKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4563, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4754] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4565, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4755] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4565, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4756] Ref(ColumnTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4565, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4757] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4567, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4758] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4570, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4759] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4570, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4760] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4570, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4761] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4570, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4762] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4570, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4763] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4573, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4764] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4575, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4765] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4575, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4766] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4578, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4767] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4580, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4768] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4580, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4769] Ref(DateTimeFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4580, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4770] Ref(ColumnsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4580, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4771] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4582, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4772] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 4584, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4773] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4585, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4774] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4775] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4776] Ref(ColumnsExpressionFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4777] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4778] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4779] Ref(PostFunctionGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4780] StringParser("FUNCTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4781] StringParser("FUTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4782] StringParser("GENERATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4783] StringParser("GEOGRAPHY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4784] StringParser("GEOMETRY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4785] StringParser("GEOMETRYCOLLECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4786] StringParser("GEOMETRYCOLLECTIONM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4787] StringParser("GEOMETRYCOLLECTIONZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4788] StringParser("GEOMETRYCOLLECTIONZM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4789] StringParser("GLOB")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4790] TypedParser("glob_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4791] StringParser("GLOBAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4792] StringParser("GRANT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4588, child_count: 6, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4793] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4594, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4794] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4594, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4795] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4598, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4796] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4601, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4797] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4601, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4798] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4601, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4799] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4601, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4800] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4603, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4801] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4603, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4802] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4603, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4803] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4607, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4804] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4607, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4805] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4607, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4806] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4607, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4807] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4607, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4808] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4607, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4809] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4607, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4810] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4607, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4811] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4610, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4812] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4613, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4813] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4613, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4814] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4613, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4815] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4613, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4816] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4616, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4817] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4616, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4818] Ref(AdminKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4616, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4819] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4616, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4820] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4821] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4822] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4823] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4619, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4824] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4622, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4825] Ref(GrantedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4622, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4826] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4622, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4827] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4625, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4828] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4625, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4829] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4625, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4830] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4625, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4831] StringParser("GRANTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4625, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4832] StringParser("GRANTS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4625, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4833] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4627, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4834] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4627, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4835] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4627, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4836] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4627, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4837] StringParser("GREATEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4627, child_count: 5, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [4838] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4632, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4839] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4632, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4840] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 4632, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 1, _padding: 0 }, // [4841] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4634, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4842] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4638, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4843] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4638, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4844] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4638, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4845] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4638, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4846] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4638, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4847] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4638, child_count: 6, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [4848] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4644, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [4849] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4646, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [4850] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4646, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [4851] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4646, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [4852] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4646, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [4853] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4646, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [4854] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4646, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [4855] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4646, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [4856] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4646, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [4857] StringParser("GROUP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4646, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4858] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4647, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 1, _padding: 0 }, // [4859] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4649, child_count: 4, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [4860] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4653, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [4861] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4653, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [4862] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4653, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [4863] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4653, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [4864] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4655, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [4865] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4655, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4866] StringParser("GROUPING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4655, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4867] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4658, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4868] Ref(GroupingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4658, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4869] Ref(SetsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4658, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4870] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4661, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4871] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4663, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4872] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4873] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4874] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4875] StringParser("GROUPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4876] StringParser("HANDLER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4877] StringParser("HASH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4878] StringParser("HASHES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4665, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4879] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4669, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4880] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4669, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4881] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4671, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4882] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4674, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4883] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4674, child_count: 5, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4884] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4679, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4885] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4886] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4887] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4888] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4889] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4890] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4891] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4892] StringParser("HAVING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4893] StringParser("HEADER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4894] StringParser("HEAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4895] StringParser("HIGH_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4896] StringParser("HOLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4897] StringParser("HOLDLOCK")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4898] Ref(PositionalKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4899] StringParser("HOST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4900] StringParser("HOSTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4901] StringParser("HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4902] StringParser("HOUR_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4903] StringParser("HOUR_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4904] StringParser("HOUR_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4905] StringParser("ICU")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4906] StringParser("IDENTIFIED")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4907] Token("identifier")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4908] StringParser("IDENTITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4909] StringParser("IDENTITY_INSERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4910] StringParser("IDENTITYCOL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4681, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4911] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4683, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4912] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4683, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4913] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4683, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4914] StringParser("IF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4683, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4915] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4686, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4916] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4686, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4917] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4686, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4918] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4686, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4919] StringParser("IGNORE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4686, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4920] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4688, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4921] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4690, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4922] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4690, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4923] Ref(RespectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4690, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4924] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4690, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4925] StringParser("ILIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4690, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4926] StringParser("IMMEDIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4690, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4927] StringParser("IMMUTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4690, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4928] StringParser("IMPLICIT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4690, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4929] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4693, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4930] Ref(ImportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4693, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4931] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4693, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4932] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4693, child_count: 9, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4933] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4702, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4934] Ref(ImportForeignSchemaGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4702, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4935] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4702, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4936] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4704, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4937] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4706, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4938] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4708, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4939] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4708, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4940] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4708, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4941] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4708, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4942] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4711, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4943] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4713, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4944] Ref(NakedIdentifierFullSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4713, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4945] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4713, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4946] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4713, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4947] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4713, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4948] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4713, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4949] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4713, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4950] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4713, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4951] StringParser("IMPORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4713, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4952] StringParser("IMPORTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4713, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4953] StringParser("IN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4713, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4954] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4716, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4955] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4716, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4956] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4716, child_count: 6, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4957] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 4722, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4958] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4725, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4959] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 4727, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4960] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4729, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4961] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4729, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4962] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4729, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4963] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4729, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4964] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4729, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4965] Ref(MapLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4729, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4966] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4729, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4967] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4729, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4968] StringParser("INCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4729, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4969] StringParser("INCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4729, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4970] StringParser("INCREMENT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4729, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4971] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4729, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4972] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4731, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4973] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4731, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4974] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4733, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4975] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4733, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4976] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4733, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4977] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4737, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4978] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4739, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4979] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4739, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4980] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4739, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4981] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 4741, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4982] Ref(OperatorClassReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4742, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4983] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4744, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4984] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4744, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4985] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4746, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4986] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4746, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4987] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4746, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4988] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4746, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4989] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4748, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4990] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4748, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4991] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4748, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4992] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4750, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4993] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4750, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4994] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4752, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4995] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4752, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4996] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4752, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4997] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4754, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4998] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4757, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [4999] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4757, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5000] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4757, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5001] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4760, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5002] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4760, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5003] Ref(IndexElementOptionsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4760, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5004] StringParser("INDEX")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4760, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5005] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4763, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5006] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4765, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5007] Ref(IncludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4765, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5008] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4765, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5009] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4767, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5010] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4767, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5011] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4767, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5012] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5013] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5014] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5015] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5016] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5017] StringParser("INDEX_CLEANUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5018] StringParser("INDEXES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5019] StringParser("INET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5020] StringParser("INFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5021] StringParser("INFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5022] StringParser("INHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5023] StringParser("INHERITS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5024] StringParser("INITIALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5025] StringParser("INITIALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5026] StringParser("INLINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5027] StringParser("INNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5028] StringParser("INOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5029] StringParser("INPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5030] StringParser("INSENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5031] StringParser("INSERT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4771, child_count: 9, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5032] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4780, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5033] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4780, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5034] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4782, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5035] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4782, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5036] Ref(OrIgnoreGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4782, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5037] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4782, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5038] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4782, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5039] Ref(AsAliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4782, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5040] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4785, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5041] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4785, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5042] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4787, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5043] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4787, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5044] Ref(PositionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4787, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5045] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4789, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5046] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4789, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5047] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4789, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5048] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4792, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5049] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5050] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5051] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5052] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5053] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4796, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5054] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4796, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5055] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4798, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5056] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4798, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5057] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4800, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5058] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4803, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5059] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4803, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5060] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4807, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5061] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4807, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5062] Ref(ConflictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4807, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5063] Ref(ConflictTargetSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4807, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5064] Ref(ConflictActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4807, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5065] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4809, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5066] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4809, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5067] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4811, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5068] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4811, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5069] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4813, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5070] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4815, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5071] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4815, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5072] Ref(AsAliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4815, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5073] StringParser("INSERT_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4815, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5074] StringParser("INSTEAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4815, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5075] StringParser("INT1")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4815, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5076] StringParser("INT2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4815, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5077] StringParser("INT3")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4815, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5078] StringParser("INT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4815, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5079] StringParser("INT4RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4815, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5080] StringParser("INT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4815, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5081] StringParser("INT8RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4815, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5082] StringParser("INT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4815, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5083] StringParser("INTEGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4815, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5084] StringParser("INTEGRATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4815, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5085] StringParser("INTEGRATIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4815, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5086] StringParser("INTERSECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4815, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5087] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4817, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5088] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4817, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5089] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4821, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5090] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4823, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5091] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4823, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5092] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4825, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5093] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4825, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5094] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4825, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5095] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4825, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5096] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4827, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5097] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4827, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5098] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4829, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5099] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4829, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5100] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4829, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5101] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4831, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5102] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4831, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5103] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4831, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5104] StringParser("INTERVAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4831, child_count: 6, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5105] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4837, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5106] Ref(YearKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4837, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5107] Ref(MonthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4837, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5108] Ref(DayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4837, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5109] Ref(HourKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4837, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5110] Ref(MinuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4837, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5111] Ref(SecondKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4837, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5112] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4841, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5113] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4841, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5114] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4844, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5115] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4844, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5116] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4844, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5117] Ref(UnloggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4844, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5118] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4844, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5119] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4844, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5120] StringParser("INTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4844, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5121] StringParser("INVOKER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4844, child_count: 6, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5122] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4850, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5123] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4850, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5124] Ref(NanLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4850, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5125] Ref(UnknownLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4850, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5126] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4850, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5127] Ref(NormalizedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4850, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5128] Ref(JsonTestGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4850, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5129] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4854, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5130] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4854, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5131] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4854, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5132] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4854, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5133] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4854, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5134] StringParser("IS")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4854, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5135] Ref(IsnullKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4854, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5136] StringParser("IS_SUPERUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4854, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5137] StringParser("IS_TEMPLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4854, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5138] StringParser("ISAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4854, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5139] StringParser("ISNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4854, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5140] StringParser("ISOLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4854, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5141] StringParser("ITERATE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4854, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5142] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4857, child_count: 7, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5143] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4864, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5144] Ref(ConditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4864, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5145] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4864, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5146] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4864, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5147] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4865, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5148] Ref(NestedJoinGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4865, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5149] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4869, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5150] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4869, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5151] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4869, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5152] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5153] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5154] Ref(JoinUsingConditionGrammar)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5155] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4871, child_count: 6, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5156] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4877, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5157] Ref(UnconditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4877, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5158] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4877, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5159] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4877, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5160] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4877, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5161] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4881, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5162] Ref(ExtendedNaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4881, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5163] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4881, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5164] StringParser("JOIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4881, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5165] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4882, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5166] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4882, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5167] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4884, child_count: 2, min_times: 1, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5168] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4886, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5169] Ref(FromPivotExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4886, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5170] Ref(FromUnpivotExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4886, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5171] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4886, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5172] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5173] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5174] Meta(conditional: indented_on_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4890, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5175] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4892, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5176] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4895, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5177] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4895, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5178] Meta(conditional: indented_on_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4895, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5179] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4897, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5180] Ref(InnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4897, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5181] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4899, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5182] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4902, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5183] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4902, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5184] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4902, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5185] Ref(RightKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4902, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5186] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4902, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5187] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4906, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5188] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 4906, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5189] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4909, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5190] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4911, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5191] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4911, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5192] StringParser("JSON")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4911, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5193] TypedParser("json_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4911, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5194] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4914, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5195] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4914, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5196] Ref(JsonTypeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4914, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5197] Ref(JsonUniqueKeysGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4914, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5198] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4918, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5199] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4918, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5200] Ref(ScalarKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4918, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5201] Ref(ArrayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4918, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5202] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4918, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5203] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4921, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5204] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4923, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5205] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4923, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5206] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4923, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5207] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4923, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5208] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4924, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5209] Ref(KeysKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4924, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5210] StringParser("JSONB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4924, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5211] StringParser("KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4924, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5212] StringParser("KEYS")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4924, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5213] Token("keyword")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4924, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5214] StringParser("KILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4924, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5215] StringParser("LABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4924, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5216] StringParser("->")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4924, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5217] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4926, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5218] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4929, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5219] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4931, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5220] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4931, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5221] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4934, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5222] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4936, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5223] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4936, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5224] Ref(LambdaArrowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4936, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5225] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4936, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5226] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4940, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5227] Ref(LambdaKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4940, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5228] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4942, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5229] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4942, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5230] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4945, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5231] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4947, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5232] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4947, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5233] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4947, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5234] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4947, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5235] StringParser("LAMBDA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4947, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5236] StringParser("LANCOMPILER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4947, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5237] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4949, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5238] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4949, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5239] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5240] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5241] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5242] StringParser("LANGUAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5243] StringParser("LARGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5244] StringParser("LAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5245] StringParser("LAST_INSERT_ID")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5246] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5247] StringParser("LATERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5248] StringParser("LC_COLLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5249] StringParser("LC_CTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5250] StringParser("LEADING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5251] StringParser("LEAKPROOF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5252] StringParser("LEAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5253] StringParser("LEAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5254] StringParser("LEFT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5255] StringParser("LEFTARG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5256] StringParser("LESS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5257] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4953, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5258] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4953, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5259] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4953, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5260] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4953, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5261] StringParser("LEVEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4953, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5262] StringParser("LIBC")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4953, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5263] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4956, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5264] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4958, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5265] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4958, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5266] Ref(LikeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4958, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5267] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4958, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5268] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4960, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5269] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4960, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5270] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4960, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5271] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4964, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5272] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4964, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5273] Ref(IlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4964, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5274] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4966, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5275] Ref(SimilarKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4966, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5276] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4966, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5277] Ref(GlobKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4966, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5278] StringParser("LIKE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4966, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5279] TypedParser("like_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4966, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5280] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4968, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5281] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4970, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5282] Ref(IncludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4970, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5283] Ref(ExcludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4970, child_count: 9, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5284] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4979, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5285] Ref(CommentsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4979, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5286] Ref(ConstraintsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4979, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5287] Ref(DefaultsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4979, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5288] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4979, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5289] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4979, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5290] Ref(IndexesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4979, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5291] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4979, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5292] Ref(StorageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4979, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5293] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4979, child_count: 5, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5294] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4984, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5295] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4984, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5296] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4986, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5297] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4989, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5298] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4992, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5299] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4992, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5300] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4992, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5301] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4992, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5302] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4994, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5303] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4996, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5304] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4996, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5305] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4998, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5306] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4998, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5307] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4998, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5308] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5000, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5309] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5000, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5310] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5000, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5311] StringParser("LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5000, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5312] StringParser("LINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5000, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5313] StringParser("LINENO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5000, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5314] StringParser("LINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5000, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5315] StringParser("LINESTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5000, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5316] StringParser("LINESTRINGM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5000, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5317] StringParser("LINESTRINGZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5000, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5318] StringParser("LINESTRINGZM")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5000, child_count: 8, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5319] Bracketed(6 elements -> 6 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5008, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5320] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5008, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5321] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5008, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5322] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5008, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5323] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5008, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5324] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5008, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5325] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5010, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5326] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5010, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5327] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5010, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5328] Ref(ListComprehensionExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5010, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5329] StringParser("LIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5010, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5330] StringParser("LISTEN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5010, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5331] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5012, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5332] Ref(ListenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5012, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5333] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5012, child_count: 11, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5334] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5023, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5335] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5023, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5336] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5023, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5337] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5023, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5338] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5023, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5339] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5023, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5340] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5023, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5341] Ref(DollarNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5023, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5342] Ref(PsqlVariableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5023, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5343] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5023, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5344] Ref(TypedArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5023, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5345] Ref(ObjectLiteralSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5023, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5346] Token("literal")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5023, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5347] Token("literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5023, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5348] StringParser("LOAD")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5023, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5349] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5025, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5350] Ref(LoadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5025, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5351] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5025, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5352] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5025, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5353] StringParser("LOCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5025, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5354] StringParser("LOCALE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5025, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5355] StringParser("LOCALTIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5025, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5356] StringParser("LOCALTIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5025, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5357] StringParser("LOCATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5025, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5358] StringParser("LOCK")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5025, child_count: 6, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5359] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5031, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5360] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5031, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5361] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5031, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5362] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5031, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5363] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5033, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5364] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5035, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5365] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5035, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5366] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5035, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5367] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5038, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5368] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5038, child_count: 8, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5369] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5046, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5370] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5048, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5371] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5048, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5372] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5048, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5373] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5050, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5374] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5050, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5375] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5050, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5376] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5052, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5377] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5052, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5378] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5052, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5379] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5055, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5380] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5055, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5381] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5055, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5382] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5055, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5383] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5055, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5384] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5058, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5385] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5058, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5386] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5058, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5387] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5058, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5388] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5058, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5389] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5060, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5390] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5060, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5391] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5060, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5392] Ref(ModeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5060, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5393] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5060, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5394] StringParser("LOCKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5060, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5395] StringParser("LOCKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5060, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5396] StringParser("LOGGED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5060, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5397] StringParser("LOGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5060, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5398] StringParser("LOGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5060, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5399] StringParser("LONG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5060, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5400] StringParser("LONGBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5060, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5401] StringParser("LONGTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5060, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5402] StringParser("LOOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5060, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5403] StringParser("LOW_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5060, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5404] StringParser("LSEG")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5060, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5405] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5064, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5406] Ref(MlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5064, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5407] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5064, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5408] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5064, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5409] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5068, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5410] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5070, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5411] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5070, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5412] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5070, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5413] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5072, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5414] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5072, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5415] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5075, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5416] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5075, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5417] StringParser("MACADDR8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5075, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5418] StringParser("MACADDR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5075, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5419] StringParser("MACRO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5075, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5420] StringParser("MAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5075, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5421] StringParser("MANAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5075, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5422] StringParser("MAP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5075, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5423] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5078, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5424] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5078, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5425] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5078, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5426] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5078, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5427] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5080, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5428] Ref(MapKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5080, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5429] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(13), first_child_idx: 5083, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5430] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5085, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5431] Ref(MapLiteralElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5085, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5432] StringParser("{")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5085, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5433] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5088, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5434] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5090, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5435] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5090, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5436] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5437] Ref(MapKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5438] Ref(MapTypeSchemaSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5439] StringParser("MAPPING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5440] StringParser("MASKING")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5441] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5442] StringParser("MATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5443] StringParser("MATCHED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5444] StringParser("MATERIALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5445] StringParser("MAX_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5446] StringParser("MAXEXTENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5447] StringParser("MAXVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5448] StringParser("MCV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5449] StringParser("MEDIUMBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5450] StringParser("MEDIUMINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5451] StringParser("MEDIUMTEXT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5452] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5092, child_count: 5, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5453] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5097, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5454] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5097, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5455] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5097, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5456] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5097, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5457] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5099, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5458] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5099, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5459] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5099, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5460] StringParser("MERGE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5099, child_count: 3, min_times: 1, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5461] AnyNumberOf(3 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5102, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5462] Ref(MergeMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5102, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5463] Ref(MergeNotMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5102, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5464] Ref(MergeNotMatchedBySourceClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5102, child_count: 7, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5465] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5109, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5466] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5109, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5467] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5109, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5468] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5111, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5469] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5111, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5470] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5111, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5471] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5111, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5472] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5114, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5473] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5114, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5474] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5114, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5475] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5116, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5476] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5116, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5477] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5116, child_count: 10, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5478] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5126, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5479] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5126, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5480] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5126, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5481] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5126, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5482] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5126, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5483] Ref(SourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5126, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5484] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5128, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5485] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5128, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5486] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5128, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5487] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5128, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5488] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5131, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5489] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5131, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5490] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5131, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5491] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5133, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5492] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5133, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5493] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5133, child_count: 9, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5494] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5142, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5495] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5142, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5496] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5142, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5497] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5142, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5498] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5144, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5499] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5144, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5500] Ref(TargetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5144, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5501] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5146, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5502] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5146, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5503] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5146, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5504] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5146, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5505] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5148, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5506] Ref(MergeInsertClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5148, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5507] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5508] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5509] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5150, child_count: 12, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5510] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5162, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5511] Ref(MergeIntoLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5162, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5512] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5164, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5513] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5164, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5514] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5164, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5515] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5164, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5516] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5168, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5517] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5168, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5518] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5168, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5519] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5170, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5520] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5173, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5521] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5173, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5522] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5173, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5523] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5175, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5524] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5175, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5525] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5175, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5526] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5175, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5527] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5175, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5528] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5175, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5529] Ref(MergeMatchSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5175, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5530] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5179, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5531] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5179, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5532] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5179, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5533] StringParser("MERGES")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5179, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5534] TypedParser("meta_command_query_buffer")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5179, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5535] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5180, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5536] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5181, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5537] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5538] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5539] Ref(MetaCommandQueryBufferSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5540] StringParser("METHOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5541] StringParser("MIDDLEINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5542] StringParser("MILLISECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5543] StringParser("MIN_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5544] StringParser("MINUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5545] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5546] StringParser("MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5547] StringParser("MINUTE_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5548] StringParser("MINUTE_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5549] StringParser("MINVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5550] StringParser("ML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5551] StringParser("MLSLABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5552] StringParser("MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5553] StringParser("MODEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5554] StringParser("MODIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5555] StringParser("%")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5556] StringParser("MODULUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5557] StringParser("MONEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5558] StringParser("MONITOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5559] StringParser("MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5560] StringParser("MONTHNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5561] StringParser("MOVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5562] StringParser("MULTICURVE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 1, min_times: 1, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5563] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5564] Ref(MultilineConcatenateNewline)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5565] TypedParser("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5566] StringParser("MULTILINESTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5567] StringParser("MULTILINESTRINGM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5568] StringParser("MULTILINESTRINGZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5569] StringParser("MULTILINESTRINGZM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5570] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5571] StringParser("MULTIPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5572] StringParser("MULTIPOINTM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5573] StringParser("MULTIPOINTZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5574] StringParser("MULTIPOINTZM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5575] StringParser("MULTIPOLYGON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5576] StringParser("MULTIPOLYGONM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5577] StringParser("MULTIPOLYGONZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5578] StringParser("MULTIPOLYGONZM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5579] StringParser("MULTISURFACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5580] StringParser("MYISAM")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5581] TypedParser("word")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5582] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5184, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5583] StringParser("NAME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5184, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5584] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5187, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5585] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5187, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5586] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5190, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5587] Ref(RightArrowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5190, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5588] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5190, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5589] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5190, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5590] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5190, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5591] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5193, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5592] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5193, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5593] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5193, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5594] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5195, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5595] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 5195, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5596] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5198, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5597] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5198, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5598] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5202, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5599] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5202, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5600] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5601] Ref(NamedWindowExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5602] StringParser("NAMES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5603] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5604] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5605] StringParser("NATIONAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5204, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5606] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5607] Ref(NaturalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5608] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5609] StringParser("NATURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5610] StringParser("NCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5611] StringParser("NDISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5612] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5613] StringParser("NEGATOR")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5614] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5615] StringParser("NEW")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5616] Token("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5617] StringParser("NEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5618] StringParser("NFC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5619] StringParser("NFD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5620] StringParser("NFKC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5621] StringParser("NFKD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5622] StringParser("NO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5623] StringParser("NO_WRITE_TO_BINLOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5624] StringParser("NOAUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5625] StringParser("NOBYPASSRLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5626] StringParser("NOCACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5627] StringParser("NOCHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5628] StringParser("NOCOMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5629] StringParser("NOCREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5630] StringParser("NOCREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5631] StringParser("NOCREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5632] StringParser("NOCYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5633] StringParser("NOINHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5634] StringParser("NOLOGIN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5206, child_count: 8, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5635] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5214, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5636] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5214, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5637] Ref(UnorderedSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5214, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5638] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5217, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5639] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5217, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5640] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5220, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5641] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5220, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5642] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5223, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5643] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5223, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5644] Ref(BracketedSetExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5223, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5645] Ref(SimplifiedPivotExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5223, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5646] Ref(SimplifiedUnpivotExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5223, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5647] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5226, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5648] Ref(AntiKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5226, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5649] Ref(SemiKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5226, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5650] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5228, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5651] Ref(AsofKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5228, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5652] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5231, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5653] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5231, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5654] Ref(AntiKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5231, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5655] Ref(SemiKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5231, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5656] OneOf(0 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5231, child_count: 6, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5657] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5237, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5658] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5237, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5659] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5239, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5660] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5242, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5661] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5242, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5662] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5242, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5663] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5242, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5664] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5242, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5665] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5242, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5666] StringParser("NONCLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5242, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5667] StringParser("NONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5242, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5668] StringParser("NOORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5242, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5669] StringParser("NOREPLICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5242, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5670] StringParser("NORMALIZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5242, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5671] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5244, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5672] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5248, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5673] Ref(NfcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5248, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5674] Ref(NfdKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5248, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5675] Ref(NfkcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5248, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5676] Ref(NfkdKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5248, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5677] Ref(NormalizedKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5248, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5678] StringParser("NORMALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5248, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5679] StringParser("NOSUPERUSER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5248, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5680] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5248, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5681] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5250, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5682] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5252, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5683] Ref(RawNotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5252, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5684] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5252, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5685] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5254, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5686] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5254, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5687] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5254, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5688] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5689] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5690] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5691] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5258, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5692] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5258, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5693] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5258, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5694] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5258, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5695] Ref(NotnullKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5258, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5696] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5258, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5697] StringParser("NOTHING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5258, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5698] StringParser("NOTIFY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5258, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5699] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5261, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5700] Ref(NotifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5261, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5701] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5261, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5702] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5263, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5703] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5263, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5704] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5263, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5705] StringParser("NOTNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5263, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5706] StringParser("NOWAIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5263, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5707] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5263, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5708] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5263, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5709] StringParser("NULLIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5263, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5710] StringParser("NULLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5263, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5711] StringParser("NUMERIC")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5263, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5712] TypedParser("numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5263, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5713] StringParser("NUMRANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5263, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5714] StringParser("OBJECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5263, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5715] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5266, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5716] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5268, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5717] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5268, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5718] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5268, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5719] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5268, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5720] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5268, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5721] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5271, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5722] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5273, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5723] Ref(ObjectLiteralElementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5273, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5724] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5275, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5725] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5275, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5726] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5277, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5727] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5277, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5728] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5277, child_count: 12, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5729] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5289, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5730] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5289, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5731] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5289, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5732] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5289, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5733] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5289, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5734] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5289, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5735] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5289, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5736] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5289, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5737] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5289, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5738] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5289, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5739] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5289, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5740] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5289, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5741] StringParser("OBJECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5289, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5742] StringParser("OF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5289, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5743] StringParser("OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5289, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5744] StringParser("OFFLINE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5289, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5745] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5293, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5746] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5293, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5747] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5748] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5749] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5750] StringParser("OFFSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5751] StringParser("OFFSETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5752] StringParser("OIDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5753] StringParser("OLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5754] StringParser("ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5755] StringParser("ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5756] StringParser("ONLINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5757] StringParser("ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5758] StringParser("OPENDATASOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5759] StringParser("OPENQUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5760] StringParser("OPENROWSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5761] StringParser("OPENXML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5762] StringParser("OPERATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5763] StringParser("OPERATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5764] StringParser("OPERATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5765] StringParser("OPTIMIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5766] StringParser("OPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5767] StringParser("OPTIONALLY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5295, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5768] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5297, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5769] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5297, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5770] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5300, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5771] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5301, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5772] Ref(OptionsListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5301, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5773] StringParser("OPTIONS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5301, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5774] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5302, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5775] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5304, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5776] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5306, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5777] Ref(NakedIdentifierFullSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5306, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5778] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5306, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5779] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5308, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5780] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5308, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5781] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5308, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5782] StringParser("OR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5308, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5783] StringParser("OR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5308, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5784] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5310, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5785] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5310, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5786] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5310, child_count: 5, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5787] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5315, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5788] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5315, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5789] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 5315, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 1, _padding: 0 }, // [5790] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5317, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5791] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5320, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5792] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5324, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5793] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5324, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5794] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5324, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5795] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5324, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5796] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5324, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5797] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5326, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5798] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5326, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5799] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5326, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5800] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5328, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5801] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5328, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5802] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5330, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5803] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5330, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5804] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5330, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [5805] Ref(OrderByClauseTerminators)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5330, child_count: 8, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5806] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5338, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5807] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5338, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5808] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5338, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5809] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5338, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5810] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5338, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5811] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5338, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5812] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5338, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5813] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5340, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5814] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5340, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5815] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5340, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5816] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5340, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5817] StringParser("ORDER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5340, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5818] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5342, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5819] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5342, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5820] Ref(NoorderKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5342, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5821] StringParser("ORDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5342, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5822] StringParser("OTHERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5342, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5823] StringParser("OUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5342, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5824] StringParser("OUTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5342, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5825] StringParser("OUTFILE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5342, child_count: 5, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5826] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5827] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5828] Ref(OverKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5347, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5829] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5349, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5830] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 5349, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5831] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5352, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5832] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5352, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5833] StringParser("OVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5352, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5834] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5354, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5835] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5354, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5836] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5354, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5837] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5357, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5838] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5359, child_count: 1, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5839] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5360, child_count: 5, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5840] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5365, child_count: 4, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5841] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5369, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5842] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5369, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5843] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5369, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5844] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5369, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5845] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5369, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5846] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5369, child_count: 4, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5847] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5373, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5848] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5373, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5849] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5373, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5850] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5373, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5851] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5373, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5852] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5373, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5853] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5373, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5854] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5375, child_count: 1, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5855] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5376, child_count: 5, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5856] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5381, child_count: 4, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5857] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5385, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5858] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5385, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5859] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5385, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5860] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5385, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5861] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5385, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5862] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5385, child_count: 4, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5863] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5864] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5865] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5866] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5867] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5868] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5869] StringParser("OVERLAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5870] StringParser("OVERLAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5871] StringParser("OVERRIDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5872] StringParser("OVERWRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5873] StringParser("OVERWRITE_OR_IGNORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5874] StringParser("OWNED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5875] StringParser("OWNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5876] StringParser("OWNERSHIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5877] StringParser("PACK_KEYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5878] StringParser("PARALLEL")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5879] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5880] StringParser("?")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5881] StringParser("PARAMETERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5882] StringParser("PARQUET_VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5883] StringParser("PARSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5389, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5884] StringParser("PARTIAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5389, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5885] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5392, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5886] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5394, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5887] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5394, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5888] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5397, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5889] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5399, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5890] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5399, child_count: 4, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5891] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5403, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5892] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5403, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5893] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5406, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5894] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5408, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5895] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5411, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5896] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5411, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5897] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5411, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5898] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5411, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5899] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5411, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5900] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5414, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5901] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5416, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5902] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5419, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5903] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5419, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5904] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5419, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5905] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5419, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5906] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5421, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5907] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5421, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5908] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5424, child_count: 5, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5909] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5429, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5910] Ref(ModulusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5429, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5911] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5429, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5912] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5429, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5913] Ref(RemainderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5429, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5914] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5429, child_count: 5, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5915] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5434, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5916] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5434, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5917] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5434, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5918] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5436, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5919] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5439, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5920] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5921] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5922] StringParser("PARTITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5923] StringParser("PARTITION_BY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5924] StringParser("PASSING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5925] StringParser("PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5926] StringParser("PATH")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5441, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5927] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5443, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5928] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5445, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5929] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5445, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5930] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5931] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5932] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5933] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5934] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5935] StringParser("PCTFREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5936] StringParser("PERCENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5937] StringParser("PERMISSIVE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5938] TypedParser("pg_trgm_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5939] StringParser("PG_LSN")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5940] TypedParser("pgvector_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5941] StringParser("PIPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5942] StringParser("|")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5943] StringParser("PIVOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5944] StringParser("PIVOT_LONGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5945] StringParser("PIVOT_WIDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5946] StringParser("PLACING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5947] StringParser("PLAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5948] StringParser("PLANS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5949] StringParser("+")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5950] StringParser("POINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5951] StringParser("POINTM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5952] StringParser("POINTZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5953] StringParser("POINTZM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5954] StringParser("POLICY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5955] StringParser("POLYGON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5956] StringParser("POLYGONM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5957] StringParser("POLYGONZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5958] StringParser("POLYGONZM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5959] StringParser("POLYHEDRALSURFACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5960] StringParser("POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5961] StringParser("POSITIONAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5962] StringParser("+")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5447, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5963] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5450, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5964] Ref(WithinGroupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5450, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5965] Ref(OverClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5450, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5966] Ref(FilterClauseGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5450, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5967] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5450, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5968] StringParser("POSTFIX")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5450, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5969] TypedParser("postgis_operator")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5450, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5970] TypedParser("postgres_copy_stdin_data_block")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5450, child_count: 1, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5971] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5451, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5972] Ref(PostgresCopyStdinDataSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5451, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5973] TypedParser("power_operator")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5451, child_count: 1, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5974] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5452, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5975] Ref(LateralKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5452, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5976] StringParser("PRECEDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5452, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5977] StringParser("PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5452, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5978] StringParser("PREFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5452, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5979] StringParser("PREORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5452, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5980] StringParser("PREPARE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5452, child_count: 5, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5981] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5457, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5982] Ref(PrepareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5457, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5983] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5457, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5984] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5460, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5985] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5462, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5986] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5462, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5987] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5462, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5988] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5464, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5989] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5464, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5990] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5464, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5991] StringParser("PREPARED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5464, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5992] StringParser("PRESERVE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5464, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5993] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5994] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5995] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5996] StringParser("PRIMARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5997] StringParser("PRINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5998] StringParser("PRIOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [5999] StringParser("PRIVILEGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6000] StringParser("PROC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6001] StringParser("PROCEDURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6002] StringParser("PROCEDURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6003] StringParser("PROCEDURES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6004] StringParser("PROCESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6005] StringParser("PROCESS_TOAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6006] StringParser("PROCESSLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6007] StringParser("PROGRAM")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6008] TypedParser("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6009] StringParser("PROVIDER")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6010] TypedParser("psql_copy_command")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5466, child_count: 1, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6011] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5467, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6012] Ref(PsqlCopyMetaCommandSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5467, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6013] TypedParser("psql_set_command")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5467, child_count: 1, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6014] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5468, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6015] Ref(PsqlSetMetaCommandSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5468, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6016] StringParser(":")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5468, child_count: 1, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6017] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5469, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6018] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5471, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6019] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5474, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6020] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5476, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6021] Ref(PsqlVariableColonSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5476, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6022] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5478, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6023] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5478, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6024] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5478, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6025] StringParser("PUBLIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5478, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6026] StringParser("PUBLICATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5478, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6027] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5480, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6028] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5482, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6029] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5482, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 1, _padding: 0 }, // [6030] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5484, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6031] Ref(PublicationTableSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5484, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6032] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5486, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6033] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5486, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6034] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5488, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6035] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5488, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6036] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5488, child_count: 4, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6037] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6038] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6039] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6040] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5492, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 1, _padding: 0 }, // [6041] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5494, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6042] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5496, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6043] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5496, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6044] Ref(Current_schemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5496, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6045] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5498, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6046] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5498, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6047] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5500, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6048] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5500, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6049] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5500, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6050] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5500, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6051] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5503, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6052] Ref(ExtendedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5503, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6053] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5503, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6054] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5505, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6055] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5505, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6056] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5508, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6057] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5508, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6058] StringParser("PURGE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5508, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6059] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5510, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6060] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5510, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6061] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5510, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6062] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5512, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6063] Ref(OperatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5512, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6064] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5515, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6065] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5518, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6066] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5518, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6067] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5518, child_count: 1, min_times: 1, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6068] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5519, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6069] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5519, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6070] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5521, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6071] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 5521, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6072] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5523, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6073] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5523, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6074] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5523, child_count: 4, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6075] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5527, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6076] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5527, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6077] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5529, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6078] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5532, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6079] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5532, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6080] StringParser("QUALIFY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5532, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6081] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5535, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6082] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5535, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6083] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5538, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6084] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5538, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6085] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5538, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6086] Ref(SomeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 5538, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6087] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5541, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6088] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5541, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6089] StringParser("QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5541, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6090] StringParser("QUOTE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5541, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6091] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5541, child_count: 4, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6092] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5545, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6093] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5547, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6094] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5549, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6095] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5549, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6096] TypedParser("escaped_single_quote")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5549, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6097] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5551, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6098] Ref(MultilineConcatenateDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5551, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6099] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5551, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6100] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5553, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6101] TypedParser("bit_string_literal")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5553, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6102] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5555, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6103] Ref(MultilineConcatenateDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5555, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6104] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5555, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6105] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5558, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6106] TypedParser("unicode_single_quote")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5558, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6107] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5560, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6108] Ref(MultilineConcatenateDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5560, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6109] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5560, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6110] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5562, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6111] Ref(UescapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5562, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6112] RegexParser
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5562, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6113] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5564, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6114] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5566, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6115] TypedParser("dollar_quote")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5566, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6116] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5568, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6117] Ref(MultilineConcatenateDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5568, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6118] TypedParser("dollar_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5568, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6119] StringParser("RAID0")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5568, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6120] StringParser("RAISERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5568, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6121] StringParser("RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5568, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6122] StringParser("=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5568, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6123] StringParser(">")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5568, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6124] StringParser("RAW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5568, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6125] StringParser("<")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5568, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6126] StringParser("!")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5568, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6127] StringParser("READ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5568, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6128] StringParser("READTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5568, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6129] StringParser("REAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5568, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6130] StringParser("REASSIGN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5568, child_count: 6, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6131] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5574, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6132] Ref(ReassignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5574, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6133] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5574, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6134] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5574, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6135] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5576, child_count: 4, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6136] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5580, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6137] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5580, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6138] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5580, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6139] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5580, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6140] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5580, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6141] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5580, child_count: 4, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6142] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5584, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6143] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5584, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6144] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5584, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6145] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5584, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6146] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5584, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6147] StringParser("RECHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5584, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6148] StringParser("RECONFIGURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5584, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6149] StringParser("RECURSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5584, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6150] StringParser("REF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5584, child_count: 5, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6151] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5589, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6152] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5589, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6153] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5589, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6154] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5589, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6155] Ref(ReferenceMatchGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5589, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6156] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5591, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6157] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5594, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6158] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5594, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6159] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5594, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6160] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5594, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6161] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5597, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6162] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5597, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6163] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5597, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6164] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5597, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6165] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5599, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6166] Ref(MatchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5599, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6167] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5602, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6168] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5602, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6169] Ref(PartialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5602, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6170] Ref(SimpleKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5602, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6171] StringParser("REFERENCE_USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5602, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6172] StringParser("REFERENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5602, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6173] StringParser("REFERENCING")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5602, child_count: 4, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6174] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5606, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6175] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5606, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6176] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6177] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5609, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6178] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5611, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6179] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5611, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6180] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5611, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6181] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5614, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6182] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5616, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6183] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5616, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6184] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5616, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6185] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6186] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6187] Ref(ActionKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6188] StringParser("REFRESH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5618, child_count: 6, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6189] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5624, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6190] Ref(RefreshKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5624, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6191] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5624, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6192] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5624, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6193] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5624, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6194] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5624, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6195] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5624, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6196] StringParser("REGEXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5624, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6197] StringParser("REINDEX")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5624, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6198] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5627, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6199] Ref(ReindexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5627, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6200] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5630, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6201] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5632, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6202] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5635, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6203] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5637, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6204] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5637, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6205] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5637, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6206] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5639, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6207] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5639, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6208] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5639, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6209] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6210] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6211] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5641, child_count: 4, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6212] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5645, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6213] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5648, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6214] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5648, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6215] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5648, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6216] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5648, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6217] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5651, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6218] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5651, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6219] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5651, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6220] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5651, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6221] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5654, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6222] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5654, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6223] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5654, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6224] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5654, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6225] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5657, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6226] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5659, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6227] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5659, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6228] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5659, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6229] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5659, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6230] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5659, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6231] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5662, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6232] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5662, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6233] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5664, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6234] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5664, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6235] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5664, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6236] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5666, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6237] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5666, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6238] Ref(DefinitionArgumentValueGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5666, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6239] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5669, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6240] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5671, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6241] Ref(RelationOptionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5671, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6242] StringParser("RELATIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5671, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6243] StringParser("RELEASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5671, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6244] StringParser("RELOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5671, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6245] StringParser("REMAINDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5671, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6246] StringParser("RENAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5671, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6247] StringParser("REPEAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5671, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6248] StringParser("REPEATABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5671, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6249] StringParser("REPLACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5671, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6250] StringParser("REPLICA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5671, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6251] StringParser("REPLICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5671, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6252] StringParser("REQUIRE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5671, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6253] StringParser("RESERVOIR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5671, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6254] StringParser("RESET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5671, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6255] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5674, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6256] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5674, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6257] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5674, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6258] Ref(AuthorizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5674, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6259] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5676, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6260] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5676, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6261] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5679, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6262] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5679, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6263] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5679, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6264] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5679, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6265] StringParser("RESIGNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5679, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6266] StringParser("RESOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5679, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6267] StringParser("RESPECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5679, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6268] StringParser("RESTART")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5679, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6269] StringParser("RESTRICT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5679, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6270] StringParser("RESTRICTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5679, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6271] StringParser("RESTRICTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5679, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6272] StringParser("RETRIEVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5679, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6273] StringParser("RETURN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5679, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6274] StringParser("RETURNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5679, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6275] StringParser("RETURNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5679, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6276] StringParser("REVOKE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5679, child_count: 6, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6277] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5685, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6278] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5685, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6279] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5688, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6280] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5688, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6281] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5688, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6282] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5688, child_count: 4, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6283] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5692, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6284] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5695, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6285] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5695, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6286] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5695, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6287] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5695, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6288] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5697, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6289] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5697, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6290] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5697, child_count: 4, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6291] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6292] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6293] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6294] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6295] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6296] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6297] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6298] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6299] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6300] StringParser("=>")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6301] StringParser("RIGHT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6302] StringParser("RIGHTARG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6303] StringParser("RLIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6304] StringParser("ROLE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6305] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6306] StringParser("ROLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6307] StringParser("ROLLBACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6308] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6309] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6310] StringParser("ROUTINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6311] StringParser("ROUTINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6312] StringParser("ROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6313] StringParser("ROW_GROUP_SIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6314] StringParser("ROW_GROUP_SIZE_BYTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6315] StringParser("ROWCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6316] StringParser("ROWGUIDCOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6317] StringParser("ROWID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6318] StringParser("ROWNUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6319] StringParser("ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6320] StringParser("RULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6321] StringParser("SAFE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5701, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6322] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5703, child_count: 4, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6323] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5707, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6324] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5709, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6325] Ref(ReservoirKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5709, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6326] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5712, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6327] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5714, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6328] Ref(SamplePercentageGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5714, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6329] Ref(SampleRowCountGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5714, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6330] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5716, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6331] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6332] Ref(BernoulliKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6333] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5718, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6334] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5721, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6335] Ref(SamplePercentageGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5721, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6336] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5723, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6337] Ref(SamplePercentageGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5723, child_count: 4, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6338] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5727, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6339] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5730, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6340] Ref(ReservoirKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5730, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6341] Ref(BernoulliKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5730, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6342] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5730, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6343] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5732, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6344] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5732, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6345] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5732, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6346] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5734, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6347] Ref(SampleRowCountGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5734, child_count: 4, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6348] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5738, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6349] Ref(ReservoirKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5738, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6350] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5740, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6351] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5740, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6352] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5740, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6353] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5742, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6354] Ref(RepeatableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5742, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6355] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5745, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6356] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5745, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6357] StringParser("SAMPLE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5745, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6358] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5747, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6359] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5747, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6360] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5749, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6361] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5749, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6362] Ref(PercentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5749, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6363] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5751, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6364] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5751, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6365] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5751, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6366] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5753, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6367] Ref(TablesampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5753, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6368] Ref(SampleExpressionGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5753, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6369] StringParser("SAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5753, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6370] StringParser("SAVEPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5753, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6371] StringParser("SCALAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5753, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6372] StringParser("SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5753, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6373] StringParser("SCHEMAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5753, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6374] StringParser("SCROLL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5753, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6375] StringParser("SEARCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5753, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6376] StringParser("SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5753, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6377] StringParser("SECOND_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5753, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6378] StringParser("SECURITY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5753, child_count: 7, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6379] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5760, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6380] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5760, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6381] Ref(LabelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5760, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6382] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5762, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6383] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5762, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6384] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5762, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6385] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5762, child_count: 21, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6386] OneOf(21 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5783, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6387] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5785, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6388] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5785, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6389] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5785, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6390] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6391] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6392] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5787, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6393] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5790, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6394] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5790, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6395] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5790, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6396] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5793, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6397] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5793, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6398] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5795, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6399] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5795, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6400] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5795, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6401] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5797, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6402] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5797, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6403] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5797, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6404] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5800, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6405] Ref(EventKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5800, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6406] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5800, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6407] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5800, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6408] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5803, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6409] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5803, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6410] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5803, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6411] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5803, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6412] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5806, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6413] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5806, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6414] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5806, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6415] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5808, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6416] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5811, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6417] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5811, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6418] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5814, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6419] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5814, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6420] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5814, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6421] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5814, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6422] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6423] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6424] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6425] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5817, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6426] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5820, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6427] Ref(ProceduralKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5820, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6428] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5820, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6429] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5820, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6430] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5823, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6431] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5823, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6432] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5823, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6433] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5825, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6434] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5828, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6435] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5828, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6436] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5830, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6437] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5830, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6438] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5830, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6439] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5832, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6440] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5832, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6441] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5832, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6442] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5835, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6443] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5835, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6444] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5835, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6445] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5837, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6446] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5840, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6447] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5840, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6448] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5842, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6449] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5842, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6450] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5842, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6451] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5844, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6452] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5844, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6453] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5844, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6454] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5846, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6455] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5846, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6456] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5846, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6457] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5848, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6458] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5848, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6459] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5848, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6460] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5850, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6461] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5850, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6462] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5850, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6463] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5852, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6464] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5852, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6465] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5852, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6466] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5852, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6467] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5854, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6468] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5854, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6469] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5854, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6470] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5856, child_count: 1, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6471] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5857, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6472] Ref(WildcardExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5857, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6473] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5859, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6474] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5859, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6475] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5859, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6476] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5861, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6477] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5863, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6478] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5863, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6479] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5865, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6480] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5865, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6481] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5868, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6482] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5870, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6483] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5870, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6484] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 5870, child_count: 5, min_times: 0, first_terminator_idx: 52, terminator_count: 12, _padding: 0 }, // [6485] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5875, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6486] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5875, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6487] Ref(SelectClauseModifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(13), first_child_idx: 5875, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6488] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5877, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6489] Ref(SelectClauseElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5877, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6490] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5877, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6491] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5877, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6492] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5877, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6493] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5879, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6494] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5879, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6495] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5879, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6496] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5881, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6497] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5881, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6498] Ref(ConflictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5881, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6499] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5881, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6500] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5881, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6501] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5881, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6502] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5881, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6503] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5884, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6504] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5884, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6505] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5884, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6506] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5884, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6507] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5884, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [6508] Ref(MetaCommandQueryBufferSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5884, child_count: 9, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6509] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5893, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6510] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 5893, child_count: 1, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6511] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::PrecededBy, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5894, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6512] PrecededBy(preceding_sequences=(('IS', 'DISTINCT'), ('IS', 'NOT', 'DISTINCT')))
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5894, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6513] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5894, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6514] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5896, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6515] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5896, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6516] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5896, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6517] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5896, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6518] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5896, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6519] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5896, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6520] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5896, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6521] Ref(MetaCommandQueryBufferSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5896, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6522] StringParser("SELECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 5896, child_count: 14, min_times: 0, first_terminator_idx: 64, terminator_count: 7, _padding: 0 }, // [6523] Sequence(14 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5910, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6524] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5910, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6525] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5910, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6526] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5910, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6527] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5910, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6528] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5910, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6529] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5910, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6530] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5910, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6531] Ref(QualifyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5910, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6532] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5910, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6533] Ref(OffsetClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5910, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6534] Ref(FetchClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5910, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6535] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5910, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6536] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5910, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6537] Ref(UsingSampleClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5910, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6538] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5910, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6539] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5910, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6540] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5910, child_count: 2, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6541] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5912, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6542] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5912, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6543] Ref(ConflictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5912, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6544] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5912, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6545] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5912, child_count: 0, min_times: 0, first_terminator_idx: 64, terminator_count: 0, _padding: 0 }, // [6546] Ref(MetaCommandQueryBufferSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5912, child_count: 4, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6547] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5916, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6548] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5918, child_count: 3, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6549] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5921, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6550] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5921, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6551] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5923, child_count: 3, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6552] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5926, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6553] Ref(WithCompoundNonSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5926, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6554] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5926, child_count: 3, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6555] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5929, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6556] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5929, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6557] StringParser("SEMI")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5929, child_count: 4, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6558] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5933, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6559] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5933, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6560] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5933, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6561] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5933, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6562] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5935, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6563] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5937, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6564] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5937, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6565] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5937, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6566] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5937, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6567] StringParser(";")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5937, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6568] StringParser("SEPARATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5937, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6569] StringParser("SEQUENCE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5937, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6570] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5939, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6571] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5941, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6572] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5941, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6573] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5941, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6574] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5943, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6575] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5943, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6576] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5943, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6577] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5945, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6578] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5947, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6579] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5947, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6580] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5947, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6581] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5949, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6582] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5949, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6583] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5949, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6584] StringParser("SEQUENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5949, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6585] StringParser("SERIAL2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5949, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6586] StringParser("SERIAL4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5949, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6587] StringParser("SERIAL8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5949, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6588] StringParser("SERIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5949, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6589] StringParser("SERIALIZABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5949, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6590] StringParser("SERVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5949, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6591] StringParser("SERVER_ENCODING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5949, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6592] StringParser("SERVER_VERSION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5949, child_count: 4, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6593] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5953, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6594] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5953, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6595] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5953, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6596] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5953, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6597] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5953, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6598] StringParser("SESSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5953, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6599] StringParser("SESSION_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5953, child_count: 4, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6600] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5957, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6601] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5957, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6602] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5959, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6603] Ref(SetClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5959, child_count: 1, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6604] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5960, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6605] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5962, child_count: 5, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6606] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5967, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6607] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5967, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6608] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5967, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6609] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5967, child_count: 6, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6610] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5973, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6611] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5973, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6612] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5973, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6613] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5973, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6614] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5973, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6615] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5973, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6616] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5973, child_count: 1, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6617] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5974, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6618] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5974, child_count: 3, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6619] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5977, child_count: 3, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6620] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5980, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6621] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5982, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6622] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5982, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6623] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5982, child_count: 3, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6624] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5985, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6625] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5987, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6626] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5987, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6627] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5989, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6628] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5991, child_count: 6, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6629] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5997, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6630] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5997, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6631] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5997, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6632] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5997, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6633] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5997, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6634] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5997, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6635] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5997, child_count: 1, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6636] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5998, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6637] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5998, child_count: 4, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6638] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6002, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6639] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6002, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6640] Ref(ConstraintsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6002, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6641] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6004, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6642] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6004, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6643] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6006, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6644] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6006, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6645] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6008, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6646] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6008, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6647] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6008, child_count: 5, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6648] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6013, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6649] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6013, child_count: 1, min_times: 1, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6650] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6014, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6651] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6016, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6652] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6016, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6653] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6016, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6654] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6016, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6655] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6016, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6656] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6016, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6657] StringParser("SET")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6016, child_count: 3, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6658] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6019, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6659] Ref(UnionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6019, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6660] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6021, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6661] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6023, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6662] Ref(IntersectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6023, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6663] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6023, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6664] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6023, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6665] Ref(MinusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6023, child_count: 4, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6666] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6027, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6667] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6027, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6668] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6027, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6669] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6027, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6670] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6027, child_count: 4, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6671] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6031, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6672] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6031, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6673] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6033, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6674] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6035, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6675] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6035, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6676] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6035, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6677] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6037, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6678] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6037, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6679] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6037, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6680] Ref(AuthorizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6037, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6681] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6039, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6682] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6039, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6683] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6039, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6684] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6041, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6685] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6041, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6686] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6043, child_count: 4, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6687] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6047, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6688] Ref(VariableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6047, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6689] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6047, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6690] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6049, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6691] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6049, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6692] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6049, child_count: 3, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6693] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6052, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6694] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6052, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6695] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6052, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6696] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6052, child_count: 4, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6697] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6056, child_count: 3, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6698] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6059, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6699] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6059, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6700] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6059, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6701] Ref(GlobalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6059, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6702] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6059, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6703] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6061, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6704] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6061, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6705] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6061, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6706] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6063, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6707] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6063, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6708] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6065, child_count: 4, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6709] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6069, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6710] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6069, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6711] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6069, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6712] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6069, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6713] Ref(OnKeywordAsIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6069, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6714] StringParser("SETOF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6069, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6715] StringParser("SETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6069, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6716] StringParser("SETTINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6069, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6717] StringParser("SETUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6069, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6718] StringParser("SHARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6069, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6719] StringParser("SHARES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6069, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6720] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6071, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6721] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6073, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6722] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6073, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6723] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6073, child_count: 1, min_times: 1, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6724] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6074, child_count: 3, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6725] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6077, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6726] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6077, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6727] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6077, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6728] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6077, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6729] StringParser("SHOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6077, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6730] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6079, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6731] Ref(ShowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6079, child_count: 7, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6732] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6086, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6733] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6086, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6734] Ref(Is_superuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6086, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6735] Ref(Lc_collateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6086, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6736] Ref(Lc_ctypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6086, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6737] Ref(Server_encodingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6086, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6738] Ref(Server_versionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6086, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6739] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6086, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6740] StringParser("SHUTDOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6086, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6741] StringParser("SIGNAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6086, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6742] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6088, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6743] Ref(PositiveSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6088, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6744] Ref(NegativeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6088, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6745] StringParser("SIMILAR")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6088, child_count: 1, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6746] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6089, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6747] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6089, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6748] StringParser("SIMPLE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6089, child_count: 7, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6749] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6096, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6750] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6098, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6751] Ref(PivotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6098, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6752] Ref(Pivot_widerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6098, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6753] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6098, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6754] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6100, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6755] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 6100, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6756] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6102, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6757] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6104, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6758] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6106, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6759] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6106, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6760] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6106, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6761] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6108, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6762] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6108, child_count: 3, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6763] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 6111, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6764] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6113, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6765] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6113, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6766] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6767] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 6115, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6768] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6117, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6769] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6119, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6770] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6119, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6771] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6119, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6772] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6119, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6773] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6119, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6774] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6119, child_count: 7, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6775] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6126, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6776] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6128, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6777] Ref(UnpivotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6128, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6778] Ref(Pivot_longerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6128, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6779] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6128, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6780] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 6128, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6781] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6130, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6782] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6132, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6783] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6134, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6784] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6134, child_count: 3, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6785] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 6137, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6786] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6139, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6787] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6139, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6788] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6139, child_count: 5, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6789] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6144, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6790] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6144, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6791] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6144, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6792] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6144, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6793] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 6144, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6794] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6146, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6795] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6146, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6796] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6146, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6797] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6146, child_count: 3, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6798] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6149, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6799] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6149, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6800] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6149, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6801] Ref(NakedIdentifierFullSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6149, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 1, _padding: 0 }, // [6802] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6151, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6803] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6151, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6804] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6151, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [6805] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6151, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6806] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6153, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6807] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6153, child_count: 3, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6808] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6156, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6809] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6156, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6810] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6156, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6811] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6156, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6812] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6156, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6813] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6814] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6815] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6816] StringParser("SKIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6817] StringParser("SKIP_LOCKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6818] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6819] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6820] StringParser("SMALLINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6821] StringParser("SMALLSERIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6822] StringParser("SNAPSHOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6823] StringParser("SOME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6824] StringParser("SONAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6825] StringParser("SOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6826] StringParser("SPATIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6827] StringParser("SQL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6828] StringParser("SQL_BIG_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6829] StringParser("SQL_BIG_SELECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6830] StringParser("SQL_BIG_TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6831] StringParser("SQL_CALC_FOUND_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6832] StringParser("SQL_LOG_OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6833] StringParser("SQL_LOG_UPDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6834] StringParser("SQL_LOW_PRIORITY_UPDATES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6835] StringParser("SQL_SELECT_LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6836] StringParser("SQL_SMALL_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6837] StringParser("SQL_WARNINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6838] StringParser("SQLCA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6839] StringParser("SSL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6840] StringParser("STABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6841] StringParser("STAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6842] StringParser("STAGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6843] StringParser("STANDALONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6844] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6845] StringParser("START")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6846] StringParser("STARTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6847] StringParser("STARTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6158, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6848] StringParser("STATEMENT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6158, child_count: 123, min_times: 0, first_terminator_idx: 72, terminator_count: 1, _padding: 0 }, // [6849] OneOf(123 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6850] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6851] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6852] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6853] Ref(TransactionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6854] Ref(DropTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6855] Ref(DropViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6856] Ref(CreateUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6857] Ref(DropUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6858] Ref(TruncateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6859] Ref(AccessStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6860] Ref(CreateTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6861] Ref(CreateRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6862] Ref(DropRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6863] Ref(AlterTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6864] Ref(CreateSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6865] Ref(SetSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6866] Ref(DropSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6867] Ref(DropTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6868] Ref(CreateDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6869] Ref(DropDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6870] Ref(CreateIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6871] Ref(DropIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6872] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6873] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6874] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6875] Ref(CreateCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6876] Ref(DropCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6877] Ref(CreateFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6878] Ref(DropFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6879] Ref(CreateModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6880] Ref(DropModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6881] Ref(DescribeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6882] Ref(UseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6883] Ref(ExplainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6884] Ref(CreateSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6885] Ref(AlterSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6886] Ref(DropSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6887] Ref(CreateTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6888] Ref(DropTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6889] Ref(AlterDefaultPrivilegesStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6890] Ref(DropOwnedStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6891] Ref(ReassignOwnedStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6892] Ref(CommentOnStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6893] Ref(AnalyzeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6894] Ref(CreateTableAsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6895] Ref(AlterTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6896] Ref(SetStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6897] Ref(AlterSystemStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6898] Ref(AlterPolicyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6899] Ref(CreatePolicyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6900] Ref(DropPolicyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6901] Ref(CreateDomainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6902] Ref(AlterDomainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6903] Ref(DropDomainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6904] Ref(CreateMaterializedViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6905] Ref(AlterMaterializedViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6906] Ref(DropMaterializedViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6907] Ref(RefreshMaterializedViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6908] Ref(AlterDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6909] Ref(DropDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6910] Ref(VacuumStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6911] Ref(AlterFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6912] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6913] Ref(AlterViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6914] Ref(ListenStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6915] Ref(NotifyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6916] Ref(UnlistenStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6917] Ref(LoadStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6918] Ref(ResetStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6919] Ref(DiscardStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6920] Ref(AlterProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6921] Ref(CreateProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6922] Ref(DropProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6923] Ref(CopyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6924] Ref(DoStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6925] Ref(AlterIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6926] Ref(ReindexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6927] Ref(AlterRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6928] Ref(CreateExtensionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6929] Ref(DropExtensionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6930] Ref(AlterExtensionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6931] Ref(CreateSubscriptionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6932] Ref(AlterSubscriptionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6933] Ref(DropSubscriptionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6934] Ref(CreatePublicationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6935] Ref(AlterPublicationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6936] Ref(DropPublicationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6937] Ref(CreateTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6938] Ref(AlterTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6939] Ref(CreateTextSearchConfigurationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6940] Ref(AlterTextSearchConfigurationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6941] Ref(DropTextSearchConfigurationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6942] Ref(AlterSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6943] Ref(LockTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6944] Ref(ClusterStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6945] Ref(CreateCollationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6946] Ref(DropCollationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6947] Ref(CallStoredProcedureSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6948] Ref(CreateServerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6949] Ref(CreateUserMappingStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6950] Ref(ImportForeignSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6951] Ref(CreateForeignTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6952] Ref(DropAggregateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6953] Ref(CreateAggregateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6954] Ref(AlterAggregateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6955] Ref(CreateStatisticsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6956] Ref(AlterStatisticsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6957] Ref(DropStatisticsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6958] Ref(ShowStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6959] Ref(SetConstraintsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6960] Ref(CreateForeignDataWrapperStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6961] Ref(MetaCommandQueryBufferStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6962] Ref(DropForeignTableStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6963] Ref(CreateOperatorStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6964] Ref(AlterForeignTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6965] Ref(SecurityLabelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6966] Ref(PrepareStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6967] Ref(ExecuteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6968] Ref(DeallocateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6969] Ref(SetSessionAuthorizationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6970] Ref(ResetSessionAuthorizationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6971] Ref(SimplifiedPivotExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6972] Ref(SimplifiedUnpivotExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [6973] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6974] StringParser("STATISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6975] StringParser("STDIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6976] StringParser("STDOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6977] StringParser("STORAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6978] StringParser("STORED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6979] StringParser("STRAIGHT_JOIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6980] StringParser("STREAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6981] StringParser("STREAMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6982] StringParser("STRICT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6983] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6283, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6984] Ref(ConcatSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6283, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6985] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6283, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6986] StringParser("STRIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6283, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6987] StringParser("STRUCT")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6283, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6988] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6286, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6989] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6288, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6990] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6290, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6991] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6290, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6992] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6290, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6993] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6293, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6994] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6295, child_count: 1, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6995] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6296, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6996] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6298, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6997] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6298, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6998] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6300, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [6999] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6300, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7000] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6300, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7001] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6302, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7002] Ref(StructKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6302, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7003] Ref(StructTypeSchemaSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6302, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7004] StringParser("SUBLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6302, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7005] StringParser("SUBSCRIPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6302, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7006] StringParser("SUBSTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6302, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7007] StringParser("SUCCESSFUL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6302, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7008] StringParser("SUMMARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6302, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7009] StringParser("SUPERUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6302, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7010] StringParser("SUPPORT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6302, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7011] Token("symbol")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6302, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7012] StringParser("SYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6302, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7013] StringParser("SYNONYM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6302, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7014] StringParser("SYSDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6302, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7015] StringParser("SYSID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6302, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7016] StringParser("SYSTEM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6302, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7017] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6305, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7018] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6307, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7019] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6307, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7020] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6307, child_count: 5, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7021] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6312, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7022] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6315, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7023] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6315, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7024] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6318, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7025] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6318, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7026] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6320, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7027] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6320, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7028] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6320, child_count: 4, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7029] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6324, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7030] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6324, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7031] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6327, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7032] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6327, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7033] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6327, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7034] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6327, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7035] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6327, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7036] Ref(IndexParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6327, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7037] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6330, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7038] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6330, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7039] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6330, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7040] Ref(IndexParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6330, child_count: 5, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7041] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6335, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7042] Ref(ExcludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6335, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7043] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6337, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7044] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6337, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7045] Ref(IndexAccessMethodSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6337, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7046] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6340, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7047] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6342, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7048] Ref(ExclusionConstraintElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6342, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7049] Ref(IndexParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6342, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7050] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6344, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7051] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6344, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7052] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6347, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7053] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6347, child_count: 4, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7054] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6351, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7055] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6351, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7056] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6351, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7057] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6351, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7058] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6351, child_count: 4, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7059] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6355, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7060] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6357, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7061] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6357, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7062] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6359, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7063] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6359, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7064] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6359, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7065] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6361, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7066] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6363, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7067] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6363, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7068] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6363, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7069] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6365, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7070] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6365, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7071] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6365, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7072] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6367, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7073] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6367, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7074] Ref(ValidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6367, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7075] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6369, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7076] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6369, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7077] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6369, child_count: 4, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7078] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6373, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7079] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6375, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7080] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6375, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7081] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6375, child_count: 4, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7082] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6379, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7083] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6381, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7084] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6381, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7085] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6381, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7086] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6381, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7087] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6381, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7088] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6381, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7089] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6383, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7090] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6383, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7091] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6385, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7092] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6385, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7093] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6385, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7094] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6387, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7095] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6389, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7096] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6389, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7097] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6389, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7098] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6391, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7099] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6391, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7100] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6391, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7101] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6391, child_count: 6, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7102] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6397, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7103] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6397, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7104] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6397, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7105] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6399, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7106] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6399, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7107] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6401, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7108] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6401, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7109] Ref(OrdinalityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6401, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7110] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6401, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7111] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6404, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7112] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6404, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7113] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6407, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7114] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6407, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7115] StringParser("TABLE")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6407, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 1, _padding: 0 }, // [7116] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 6409, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7117] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6411, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7118] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6411, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7119] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6411, child_count: 0, min_times: 0, first_terminator_idx: 74, terminator_count: 0, _padding: 0 }, // [7120] StringParser("TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6411, child_count: 0, min_times: 0, first_terminator_idx: 74, terminator_count: 0, _padding: 0 }, // [7121] StringParser("TABLESAMPLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6411, child_count: 0, min_times: 0, first_terminator_idx: 74, terminator_count: 0, _padding: 0 }, // [7122] StringParser("TABLESPACE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6411, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7123] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6413, child_count: 1, min_times: 0, first_terminator_idx: 74, terminator_count: 1, _padding: 0 }, // [7124] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6414, child_count: 0, min_times: 0, first_terminator_idx: 74, terminator_count: 0, _padding: 0 }, // [7125] Ref(Expression_A_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6414, child_count: 0, min_times: 0, first_terminator_idx: 74, terminator_count: 0, _padding: 0 }, // [7126] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6414, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7127] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6414, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7128] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6416, child_count: 1, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7129] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6417, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7130] Ref(Expression_B_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6417, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7131] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6417, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7132] StringParser("TARGET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6417, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7133] StringParser("TASK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6417, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7134] StringParser("TASKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6417, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7135] StringParser("TEMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6417, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7136] StringParser("TEMPLATE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6417, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7137] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6417, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7138] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6419, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7139] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6419, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7140] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6419, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7141] StringParser("TEMPORARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6419, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7142] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6421, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7143] Ref(TransientKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6421, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7144] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6421, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7145] StringParser("TERMINATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6421, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7146] StringParser("TERMINATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6421, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7147] StringParser("TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6421, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7148] StringParser("TEXTSIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6421, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7149] StringParser("THAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6421, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7150] StringParser("THEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6421, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7151] StringParser("TIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6421, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7152] StringParser("~")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6421, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7153] StringParser("TIME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6421, child_count: 3, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7154] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6424, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7155] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6426, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7156] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6426, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7157] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6426, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7158] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6426, child_count: 3, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7159] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6429, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7160] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6431, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7161] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6431, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7162] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6431, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7163] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6431, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7164] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6431, child_count: 1, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7165] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6432, child_count: 4, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7166] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7167] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7168] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7169] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7170] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7171] StringParser("TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7172] StringParser("TIMESTAMPTZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7173] StringParser("TIMETZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7174] StringParser("TIMING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7175] StringParser("TIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7176] StringParser("TINYBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7177] StringParser("TINYINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7178] StringParser("TINYTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7179] StringParser("TO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7180] StringParser("TOAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7181] StringParser("TOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7182] StringParser("TRAILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7183] StringParser("TRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7184] StringParser("TRANSACTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6436, child_count: 4, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7185] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6440, child_count: 5, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7186] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6445, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7187] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6445, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7188] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6445, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7189] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6445, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7190] Ref(RollbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6445, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7191] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6445, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7192] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6447, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7193] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6447, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7194] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6447, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7195] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6449, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7196] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6449, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7197] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6449, child_count: 3, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7198] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6452, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7199] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6452, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7200] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6452, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7201] Ref(ChainKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6452, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7202] StringParser("TRANSACTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6452, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7203] StringParser("TRANSFORM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6452, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7204] StringParser("TRANSIENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6452, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7205] StringParser("TREAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6452, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7206] StringParser("TRIANGLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6452, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7207] StringParser("TRIGGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6452, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7208] StringParser("TRIM")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6452, child_count: 3, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7209] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6455, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7210] Ref(BothKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6455, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7211] Ref(LeadingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6455, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7212] Ref(TrailingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6455, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7213] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6455, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7214] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6455, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7215] StringParser("TRUNCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6455, child_count: 5, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7216] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6460, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7217] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6460, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7218] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6460, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7219] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6462, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7220] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6464, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7221] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6466, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7222] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6466, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7223] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6466, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7224] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6468, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7225] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6468, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7226] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6468, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7227] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6470, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7228] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6472, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7229] Ref(RestartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6472, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7230] Ref(ContinueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6472, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7231] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6472, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7232] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6472, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7233] StringParser("TRUSTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6472, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7234] StringParser("TSEQUAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6472, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7235] StringParser("TSQUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6472, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7236] StringParser("TSRANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6472, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7237] StringParser("TSTZRANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6472, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7238] StringParser("TSVECTOR")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6472, child_count: 3, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7239] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6475, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7240] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6477, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7241] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6477, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7242] StringParser("TYPE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6477, child_count: 3, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7243] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6480, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7244] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6482, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7245] Ref(VariadicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6482, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7246] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6484, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7247] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6484, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7248] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6484, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7249] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6484, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7250] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6484, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7251] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6486, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7252] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6486, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7253] Ref(StructLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6486, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7254] StringParser("TYPES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6486, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7255] StringParser("UESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6486, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7256] StringParser("UID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6486, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7257] StringParser("UNBOUNDED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6486, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7258] StringParser("UNCOMMITTED")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6486, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7259] Ref(CrossKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6486, child_count: 3, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7260] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6489, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7261] Ref(NaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6489, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7262] Ref(UnconditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6489, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7263] Ref(HorizontalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6489, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7264] StringParser("UNDO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6489, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7265] StringParser("UNENCRYPTED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6489, child_count: 3, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7266] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6492, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7267] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6492, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7268] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6494, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7269] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6494, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7270] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6494, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7271] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6496, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7272] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6496, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7273] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6496, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7274] StringParser("UNION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6496, child_count: 1, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7275] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6497, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7276] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6497, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7277] StringParser("UNIQUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6497, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7278] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6497, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7279] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6497, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7280] StringParser("UNLISTEN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6497, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7281] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6499, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7282] Ref(UnlistenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6499, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7283] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6501, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7284] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6501, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7285] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6501, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7286] StringParser("UNLOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6501, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7287] StringParser("UNLOGGED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6501, child_count: 7, min_times: 0, first_terminator_idx: 75, terminator_count: 5, _padding: 0 }, // [7288] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6508, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7289] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6510, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7290] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6512, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7291] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6512, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7292] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6512, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7293] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6514, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7294] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6514, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7295] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6514, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7296] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6514, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7297] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6514, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7298] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6514, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7299] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6514, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7300] Ref(QualifyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6514, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7301] Ref(UsingSampleClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6514, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7302] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6514, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7303] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6514, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7304] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6514, child_count: 2, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7305] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6516, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7306] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6516, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7307] Ref(ConflictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6516, child_count: 0, min_times: 0, first_terminator_idx: 75, terminator_count: 0, _padding: 0 }, // [7308] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6516, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7309] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6518, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7310] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6518, child_count: 1, min_times: 1, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7311] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6519, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7312] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6521, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7313] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6521, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7314] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6521, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7315] TypedParser("star")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6521, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7316] StringParser("UNPIVOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6521, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7317] StringParser("UNSAFE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6521, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7318] StringParser("UNSIGNED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6521, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7319] StringParser("UNTIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6521, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7320] StringParser("UPDATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6521, child_count: 10, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7321] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6531, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7322] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6531, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7323] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6531, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7324] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 6531, child_count: 1, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7325] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6532, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7326] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6532, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7327] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6532, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7328] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6532, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7329] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6534, child_count: 4, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7330] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6538, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7331] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6538, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7332] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6538, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7333] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6538, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7334] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6538, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7335] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6538, child_count: 4, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7336] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6542, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7337] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6542, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7338] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6544, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7339] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6544, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7340] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6546, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7341] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6548, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7342] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6548, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7343] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6548, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7344] StringParser("UPDATETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6548, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7345] StringParser("USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6548, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7346] StringParser("USE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6548, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7347] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6550, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7348] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6550, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7349] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6550, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7350] StringParser("USE_ANY_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6550, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7351] StringParser("USER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6550, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7352] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6550, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7353] StringParser("USERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6550, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7354] StringParser("USING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6550, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7355] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6553, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7356] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6553, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7357] Ref(SampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6553, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7358] Ref(SampleExpressionGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6553, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7359] StringParser("UTC_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6553, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7360] StringParser("UTC_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6553, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7361] StringParser("UTC_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6553, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7362] StringParser("UUID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6553, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7363] StringParser("VACUUM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6553, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7364] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6556, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7365] Ref(VacuumKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6556, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7366] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6558, child_count: 4, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7367] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6562, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7368] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6562, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7369] Ref(FreezeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6562, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7370] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6562, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7371] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6564, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7372] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6564, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7373] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6564, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7374] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6567, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7375] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6569, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7376] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6571, child_count: 11, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7377] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7378] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7379] Ref(FreezeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7380] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7381] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7382] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7383] Ref(Disable_page_skippingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7384] Ref(Skip_lockedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7385] Ref(Index_cleanupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7386] Ref(Process_toastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7387] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7388] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6582, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7389] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6585, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7390] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6585, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7391] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6585, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7392] Ref(OnKeywordAsIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6585, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7393] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6587, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7394] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6589, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7395] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6589, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7396] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6589, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7397] StringParser("VALID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6589, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7398] StringParser("VALIDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6589, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7399] StringParser("VALIDATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6589, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7400] StringParser("VALUE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6589, child_count: 5, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7401] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6594, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7402] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 6594, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7403] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 6596, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7404] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 6599, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7405] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6601, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7406] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6603, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7407] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6603, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7408] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6603, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7409] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6603, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7410] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6603, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7411] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6603, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7412] StringParser("VALUES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6603, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7413] StringParser("VARCHAR2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6603, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7414] StringParser("VARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6603, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7415] StringParser("VARCHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6603, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7416] StringParser("VARIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6603, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7417] StringParser("VARIABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6603, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7418] StringParser("VARIADIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6603, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7419] StringParser("VARYING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6603, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7420] StringParser("VECTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6603, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7421] StringParser("VERBOSE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6603, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7422] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6605, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7423] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6605, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7424] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6605, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7425] StringParser("VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6605, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7426] StringParser("VIEW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6605, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7427] StringParser("VIEWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6605, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7428] StringParser("VIRTUAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6605, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7429] StringParser("VOLATILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6605, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7430] StringParser("WAITFOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6605, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7431] StringParser("WAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6605, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7432] StringParser(":=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6605, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7433] StringParser("WAREHOUSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6605, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7434] StringParser("WAREHOUSES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6605, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7435] StringParser("WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6605, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7436] StringParser("WEEKDAY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6605, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7437] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6607, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7438] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6609, child_count: 36, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7439] OneOf(36 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7440] Ref(PointKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7441] Ref(LinestringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7442] Ref(PolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7443] Ref(MultipointKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7444] Ref(MultilinestringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7445] Ref(MultipolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7446] Ref(GeometrycollectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7447] Ref(PointzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7448] Ref(LinestringzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7449] Ref(PolygonzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7450] Ref(MultipointzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7451] Ref(MultilinestringzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7452] Ref(MultipolygonzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7453] Ref(GeometrycollectionzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7454] Ref(PointmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7455] Ref(LinestringmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7456] Ref(PolygonmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7457] Ref(MultipointmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7458] Ref(MultilinestringmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7459] Ref(MultipolygonmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7460] Ref(GeometrycollectionmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7461] Ref(PointzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7462] Ref(LinestringzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7463] Ref(PolygonzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7464] Ref(MultipointzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7465] Ref(MultilinestringzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7466] Ref(MultipolygonzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7467] Ref(GeometrycollectionzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7468] Ref(CircularstringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7469] Ref(CompoundcurveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7470] Ref(CurvepolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7471] Ref(MulticurveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7472] Ref(MultisurfaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7473] Ref(PolyhedralsurfaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7474] Ref(TriangleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7475] Ref(TinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7476] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6648, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7477] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6650, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7478] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6653, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7479] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6655, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7480] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6658, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7481] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6660, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7482] Ref(SimpleGeometryGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6660, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7483] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6663, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7484] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6665, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7485] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6668, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7486] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6670, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7487] Ref(SimpleGeometryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6670, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7488] Ref(WellKnownTextGeometrySegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6670, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7489] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6672, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7490] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7491] Ref(GeometryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7492] Ref(GeographyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7493] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6677, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7494] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6679, child_count: 38, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7495] OneOf(38 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7496] Ref(PointKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7497] Ref(LinestringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7498] Ref(PolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7499] Ref(MultipointKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7500] Ref(MultilinestringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7501] Ref(MultipolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7502] Ref(GeometrycollectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7503] Ref(PointzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7504] Ref(LinestringzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7505] Ref(PolygonzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7506] Ref(MultipointzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7507] Ref(MultilinestringzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7508] Ref(MultipolygonzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7509] Ref(GeometrycollectionzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7510] Ref(PointmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7511] Ref(LinestringmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7512] Ref(PolygonmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7513] Ref(MultipointmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7514] Ref(MultilinestringmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7515] Ref(MultipolygonmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7516] Ref(GeometrycollectionmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7517] Ref(PointzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7518] Ref(LinestringzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7519] Ref(PolygonzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7520] Ref(MultipointzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7521] Ref(MultilinestringzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7522] Ref(MultipolygonzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7523] Ref(GeometrycollectionzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7524] Ref(CircularstringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7525] Ref(CompoundcurveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7526] Ref(CurvepolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7527] Ref(MulticurveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7528] Ref(MultisurfaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7529] Ref(PolyhedralsurfaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7530] Ref(TriangleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7531] Ref(TinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7532] Ref(GeometryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6717, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7533] Ref(GeographyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6717, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7534] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6719, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7535] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6719, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7536] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6719, child_count: 8, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7537] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6727, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7538] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6727, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7539] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6730, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7540] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 6730, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7541] Meta(conditional: indented_then=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6730, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7542] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 6730, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7543] Meta(conditional: indented_then_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6730, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7544] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 6730, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7545] Meta(conditional: indented_then_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 6730, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7546] Meta(conditional: indented_then=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6730, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7547] StringParser("WHEN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6730, child_count: 4, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7548] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6734, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7549] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6734, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7550] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6736, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7551] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6739, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7552] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6739, child_count: 10, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7553] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6749, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7554] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6749, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7555] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6751, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7556] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6751, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7557] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6751, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7558] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6753, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7559] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6753, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7560] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6753, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7561] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6753, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7562] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6753, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7563] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6753, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7564] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6753, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7565] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6753, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7566] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6755, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7567] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6755, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7568] Ref(ConflictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6755, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7569] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6755, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7570] StringParser("WHERE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6755, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7571] StringParser("WHILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6755, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7572] StringParser("WHITESPACE")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6755, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7573] Token("whitespace")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6755, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7574] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7575] Ref(ExcludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6757, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7576] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6759, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7577] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6759, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7578] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 6762, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7579] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6764, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7580] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6764, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7581] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6767, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7582] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6767, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7583] Ref(WildcardExcludeExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6767, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7584] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6769, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7585] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6771, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7586] Ref(WildcardReplaceExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6771, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7587] Ref(WildcardRenameExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6771, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7588] Ref(WildcardPatternMatchingSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6771, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7589] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6773, child_count: 1, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7590] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6774, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7591] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6776, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7592] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6778, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7593] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6778, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7594] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6778, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7595] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6780, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7596] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6780, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7597] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6780, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7598] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6780, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7599] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6782, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7600] Ref(LikeExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6782, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7601] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6784, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7602] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6786, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7603] Ref(LikeOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6786, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7604] Ref(GlobOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6786, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7605] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6786, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7606] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6788, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7607] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6788, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7608] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6790, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7609] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 6793, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7610] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6795, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7611] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6797, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7612] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6797, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7613] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6797, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7614] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6799, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7615] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6799, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7616] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6799, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7617] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6801, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7618] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6801, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7619] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6803, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7620] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 6806, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7621] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6808, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7622] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7623] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7624] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6810, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7625] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6812, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7626] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6812, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7627] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6812, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7628] StringParser("WINDOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6812, child_count: 4, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7629] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 6816, child_count: 1, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7630] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6817, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7631] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6819, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7632] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6819, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7633] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6819, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7634] Ref(PartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6819, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7635] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6819, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7636] Ref(FrameClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6819, child_count: 4, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7637] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6823, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7638] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6823, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7639] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6825, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7640] Ref(CascadedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6825, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7641] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6825, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7642] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6825, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7643] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6825, child_count: 6, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7644] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6831, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7645] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6831, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7646] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 6831, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7647] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6831, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 1, _padding: 0 }, // [7648] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6833, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7649] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6833, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7650] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 6833, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7651] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6833, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7652] Ref(NonWithNonSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6833, child_count: 6, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7653] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6839, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7654] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6839, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7655] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 6839, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7656] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6839, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 1, _padding: 0 }, // [7657] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7658] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7659] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7660] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7661] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6841, child_count: 3, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7662] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6844, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7663] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6844, child_count: 1, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7664] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6845, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7665] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6845, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7666] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6845, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7667] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6845, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7668] StringParser("WITH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6845, child_count: 4, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7669] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6849, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7670] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6849, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7671] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6849, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7672] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6849, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7673] Ref(BindingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6849, child_count: 3, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7674] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6852, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7675] Ref(WithinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6852, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7676] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6852, child_count: 3, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7677] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7678] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7679] StringParser("WITHIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7680] StringParser("WITHOUT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7681] Token("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7682] StringParser("WORK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7683] StringParser("WRAPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7684] StringParser("WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7685] StringParser("WRITE_PARTITION_COLUMNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7686] StringParser("WRITETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7687] StringParser("X509")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7688] StringParser("XML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7689] StringParser("XMLATTRIBUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7690] StringParser("XMLCONCAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7691] StringParser("XMLELEMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7692] StringParser("XMLEXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7693] StringParser("XMLFOREST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7694] StringParser("XMLNAMESPACES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7695] StringParser("XMLPARSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7696] StringParser("XMLPI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7697] StringParser("XMLROOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7698] StringParser("XMLSERIALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7699] StringParser("XMLTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7700] StringParser("XOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7701] StringParser("YAML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7702] StringParser("YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7703] StringParser("YEAR_MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7704] StringParser("YES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7705] StringParser("ZEROFILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 82, terminator_count: 0, _padding: 0 }, // [7706] StringParser("ZONE")
];

pub static CHILD_IDS: &[u32] = &[
    5, 6, 61, 7, 51, 8, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
    21, 28, 33, 38, 39, 44, 9, 10, 22, 23, 24, 25, 26, 27, 29, 30,
    31, 32, 34, 35, 36, 37, 40, 41, 42, 43, 45, 46, 47, 48, 49, 50,
    52, 58, 53, 54, 57, 55, 56, 62, 63, 64, 66, 74, 78, 81, 84, 89,
    67, 68, 69, 70, 71, 72, 73, 75, 76, 77, 79, 80, 82, 83, 85, 86,
    87, 88, 90, 133, 91, 100, 105, 108, 109, 110, 111, 112, 113, 114, 115, 116,
    117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 92, 93,
    94, 95, 98, 99, 96, 97, 101, 102, 103, 104, 106, 107, 131, 132, 135, 58,
    138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 151, 154, 157, 149, 150,
    152, 153, 155, 156, 158, 159, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170,
    171, 174, 177, 180, 172, 173, 175, 176, 178, 179, 181, 182, 184, 185, 187, 191,
    195, 199, 203, 205, 207, 188, 189, 190, 58, 192, 193, 194, 58, 196, 197, 198,
    58, 200, 201, 202, 58, 204, 58, 206, 58, 209, 210, 216, 217, 218, 224, 225,
    226, 234, 227, 233, 228, 229, 230, 231, 232, 237, 238, 243, 244, 245, 246, 251,
    247, 231, 232, 248, 249, 250, 252, 256, 264, 253, 254, 255, 257, 258, 259, 260,
    261, 262, 263, 265, 266, 267, 269, 270, 271, 272, 273, 286, 290, 298, 302, 318,
    274, 275, 276, 279, 283, 277, 278, 280, 281, 282, 284, 285, 287, 288, 289, 291,
    292, 293, 294, 295, 296, 297, 299, 300, 301, 303, 304, 305, 306, 315, 307, 310,
    308, 309, 311, 312, 313, 314, 316, 317, 319, 320, 321, 322, 324, 325, 326, 327,
    328, 329, 332, 330, 58, 333, 334, 335, 337, 340, 338, 339, 341, 58, 342, 343,
    344, 345, 346, 347, 348, 349, 350, 351, 354, 355, 359, 360, 361, 362, 363, 367,
    356, 357, 358, 364, 58, 369, 370, 371, 372, 373, 374, 376, 377, 378, 379, 389,
    396, 380, 381, 384, 382, 383, 385, 58, 390, 391, 392, 393, 58, 397, 398, 400,
    403, 401, 402, 405, 406, 407, 408, 409, 413, 416, 422, 437, 445, 451, 455, 463,
    467, 410, 411, 412, 414, 415, 417, 420, 421, 418, 419, 423, 424, 427, 434, 425,
    426, 428, 431, 429, 430, 432, 433, 435, 436, 438, 439, 440, 441, 442, 443, 444,
    446, 447, 448, 449, 450, 452, 453, 454, 456, 457, 458, 459, 460, 461, 462, 464,
    465, 466, 468, 469, 470, 472, 473, 474, 475, 476, 481, 487, 477, 478, 479, 480,
    482, 483, 484, 485, 486, 488, 491, 489, 490, 492, 526, 534, 540, 547, 556, 574,
    493, 525, 494, 497, 498, 499, 500, 503, 507, 510, 513, 514, 515, 516, 524, 495,
    496, 501, 502, 504, 505, 506, 508, 509, 511, 512, 517, 518, 519, 520, 521, 522,
    523, 527, 533, 528, 531, 532, 529, 530, 535, 536, 537, 538, 231, 232, 539, 541,
    542, 543, 231, 232, 544, 545, 546, 548, 552, 549, 550, 551, 553, 58, 554, 555,
    557, 558, 559, 567, 560, 561, 562, 231, 232, 563, 58, 564, 565, 566, 568, 571,
    572, 573, 569, 570, 575, 576, 577, 578, 580, 591, 597, 694, 697, 700, 716, 720,
    726, 735, 742, 754, 765, 769, 775, 779, 784, 787, 790, 794, 797, 800, 808, 819,
    581, 582, 583, 584, 585, 586, 589, 587, 588, 590, 592, 593, 594, 595, 596, 598,
    599, 600, 601, 602, 615, 624, 627, 633, 637, 650, 668, 672, 676, 679, 682, 690,
    603, 606, 607, 608, 611, 604, 605, 609, 610, 612, 613, 614, 616, 617, 618, 619,
    620, 621, 622, 623, 625, 626, 628, 631, 632, 629, 630, 634, 635, 636, 638, 639,
    640, 645, 646, 647, 641, 642, 643, 644, 648, 231, 232, 649, 651, 652, 660, 663,
    653, 654, 655, 656, 657, 658, 659, 661, 662, 664, 665, 666, 667, 669, 670, 671,
    673, 674, 675, 677, 678, 680, 681, 683, 684, 685, 686, 687, 688, 689, 691, 692,
    693, 695, 696, 698, 699, 701, 702, 703, 704, 709, 705, 706, 707, 708, 710, 713,
    711, 712, 714, 715, 717, 718, 719, 721, 722, 723, 724, 725, 727, 730, 731, 728,
    729, 732, 733, 734, 736, 737, 740, 741, 738, 739, 743, 752, 753, 744, 745, 746,
    749, 747, 748, 750, 751, 755, 762, 763, 764, 756, 757, 758, 759, 760, 761, 766,
    767, 768, 770, 771, 772, 773, 774, 776, 777, 778, 780, 781, 782, 783, 785, 786,
    788, 789, 791, 792, 793, 795, 796, 798, 799, 801, 802, 803, 804, 805, 806, 807,
    809, 810, 811, 812, 813, 817, 818, 814, 815, 816, 820, 824, 825, 821, 822, 823,
    826, 231, 232, 827, 58, 828, 832, 833, 829, 830, 831, 835, 836, 837, 838, 839,
    840, 841, 842, 843, 844, 846, 852, 845, 58, 847, 848, 849, 850, 851, 853, 854,
    855, 857, 919, 858, 871, 875, 878, 884, 890, 893, 896, 899, 914, 859, 864, 870,
    860, 861, 862, 863, 865, 866, 867, 868, 869, 872, 873, 874, 876, 877, 879, 880,
    881, 882, 883, 885, 886, 887, 888, 889, 891, 892, 894, 895, 897, 898, 900, 901,
    902, 903, 911, 904, 907, 905, 906, 908, 909, 910, 912, 913, 915, 916, 917, 918,
    921, 922, 923, 927, 924, 58, 925, 926, 928, 929, 933, 937, 947, 930, 931, 932,
    934, 935, 936, 938, 939, 940, 941, 944, 945, 946, 942, 943, 948, 949, 950, 951,
    952, 954, 955, 956, 957, 1000, 958, 959, 960, 961, 965, 969, 973, 979, 988, 993,
    962, 963, 964, 966, 967, 968, 970, 971, 972, 974, 975, 976, 977, 978, 980, 981,
    982, 231, 232, 983, 58, 984, 985, 986, 987, 989, 990, 991, 231, 232, 992, 58,
    994, 995, 996, 997, 998, 999, 1001, 1002, 1003, 1004, 1005, 1010, 1011, 1012, 1013, 1006,
    1007, 1008, 1009, 58, 1016, 1050, 1054, 1058, 1067, 1072, 1017, 1018, 1019, 1020, 1021, 1025,
    1033, 1038, 1046, 1022, 1023, 1024, 1026, 1027, 1028, 231, 232, 1029, 58, 1030, 1031, 1032,
    1034, 1035, 1036, 231, 232, 1037, 58, 1039, 1040, 1041, 1042, 1043, 1044, 1045, 1047, 1048,
    1049, 1051, 1052, 1053, 1055, 1056, 1057, 1059, 1060, 1061, 231, 232, 1062, 58, 1063, 1064,
    1065, 1066, 1068, 1069, 1070, 231, 232, 1071, 58, 1073, 1074, 1075, 1076, 1077, 1078, 1079,
    1081, 1082, 1083, 1084, 1085, 1106, 1113, 1086, 1087, 1088, 1089, 1091, 1098, 1102, 1090, 58,
    1092, 1093, 1095, 1096, 1097, 1094, 1099, 1100, 1101, 1103, 1104, 1105, 1107, 1108, 1109, 1110,
    1111, 1112, 1114, 1115, 1116, 1117, 1118, 1123, 1124, 1125, 1126, 1119, 1120, 1121, 1122, 58,
    1127, 1129, 1130, 1131, 1132, 1133, 1134, 1135, 1139, 1136, 1137, 1138, 1140, 1149, 1153, 1141,
    1142, 1143, 58, 1144, 1145, 1146, 1147, 1148, 1150, 1151, 1152, 231, 232, 1154, 1155, 1156,
    1157, 231, 232, 1159, 1186, 1160, 1166, 1181, 1161, 1162, 1163, 1164, 1165, 1167, 1168, 1169,
    1170, 1178, 1171, 1174, 1172, 1173, 1175, 1176, 1177, 1179, 1180, 1182, 1183, 1184, 1185, 1188,
    1189, 1190, 1194, 1191, 58, 1192, 1193, 1195, 1196, 1200, 1204, 1219, 1229, 1197, 1198, 1199,
    1201, 1202, 1203, 1205, 1206, 1207, 1208, 1216, 1209, 1212, 1210, 1211, 1213, 58, 1214, 1215,
    1217, 1218, 1220, 1221, 1222, 1223, 1226, 1227, 1228, 1224, 1225, 1230, 1231, 1232, 1233, 1234,
    1236, 1237, 1238, 1239, 1240, 1243, 1247, 1251, 1255, 1259, 1241, 1242, 1244, 1245, 1246, 58,
    1248, 1249, 1250, 58, 1252, 1253, 1254, 58, 1256, 1257, 1258, 1260, 1261, 1262, 1263, 1264,
    1265, 1266, 1268, 1269, 1272, 1270, 1271, 1273, 1316, 1322, 1274, 1279, 1280, 1275, 1276, 1277,
    1278, 1281, 1284, 1287, 1290, 1293, 1296, 1299, 1302, 1306, 1312, 1282, 1283, 1285, 1286, 1288,
    1289, 1291, 1292, 1294, 1295, 1297, 1298, 1300, 1301, 1303, 1304, 1305, 1307, 1308, 1309, 1310,
    1311, 1313, 1314, 1315, 1317, 1318, 1319, 1320, 1321, 1323, 1329, 1333, 1324, 1325, 1326, 1327,
    1328, 1330, 1331, 1332, 1334, 1352, 1335, 1336, 1337, 1338, 1349, 1339, 1342, 1340, 1341, 1343,
    1344, 1345, 58, 1346, 1347, 1348, 1350, 1351, 1353, 1354, 1355, 1356, 1358, 1359, 1360, 1361,
    1362, 1366, 1363, 1364, 1365, 1367, 1368, 1369, 1371, 1374, 1379, 1387, 1395, 1399, 1404, 1409,
    1412, 1415, 1372, 1373, 1375, 1376, 1377, 1378, 1380, 1384, 1381, 1382, 1383, 1385, 1386, 1388,
    1392, 1389, 1390, 1391, 1393, 1394, 1396, 1397, 1398, 1400, 1401, 1402, 1403, 1405, 1406, 1407,
    1408, 1410, 1411, 1413, 1414, 1416, 1417, 1418, 1419, 1420, 1422, 1423, 1424, 1425, 1426, 1427,
    1429, 1436, 1440, 1428, 1430, 1431, 1432, 1433, 1434, 1435, 1437, 1438, 1439, 1441, 1442, 1443,
    1445, 1446, 1447, 1448, 1449, 1450, 1460, 1464, 1451, 1452, 1453, 1454, 1457, 1458, 1459, 1455,
    1456, 1461, 1462, 1463, 1465, 1466, 1467, 1470, 1468, 1469, 1471, 1472, 1474, 1475, 1476, 1477,
    1478, 1481, 1492, 1498, 1499, 1500, 1503, 1509, 1517, 1479, 1480, 1482, 1486, 1487, 1489, 1483,
    1484, 1485, 1488, 58, 1490, 1491, 1493, 1494, 1495, 1496, 1497, 1501, 1502, 1504, 1505, 1506,
    1507, 1508, 231, 232, 1510, 1511, 1512, 1513, 1514, 1515, 1516, 1518, 1519, 1520, 1522, 1523,
    1524, 1525, 1539, 1526, 1527, 1528, 1531, 1529, 1530, 1532, 1533, 1534, 58, 1535, 1536, 1537,
    1538, 1540, 1541, 1542, 1543, 1545, 1556, 1562, 1659, 1662, 1665, 1681, 1685, 1691, 1700, 1707,
    1719, 1730, 1734, 1740, 1744, 1749, 1752, 1755, 1759, 1762, 1765, 1773, 1546, 1547, 1548, 1549,
    1550, 1551, 1554, 1552, 1553, 1555, 1557, 1558, 1559, 1560, 1561, 1563, 1564, 1565, 1566, 1567,
    1580, 1589, 1592, 1598, 1602, 1615, 1633, 1637, 1641, 1644, 1647, 1655, 1568, 1571, 1572, 1573,
    1576, 1569, 1570, 1574, 1575, 1577, 1578, 1579, 1581, 1582, 1583, 1584, 1585, 1586, 1587, 1588,
    1590, 1591, 1593, 1596, 1597, 1594, 1595, 1599, 1600, 1601, 1603, 1604, 1605, 1610, 1611, 1612,
    1606, 1607, 1608, 1609, 1613, 231, 232, 1614, 1616, 1617, 1625, 1628, 1618, 1619, 1620, 1621,
    1622, 1623, 1624, 1626, 1627, 1629, 1630, 1631, 1632, 1634, 1635, 1636, 1638, 1639, 1640, 1642,
    1643, 1645, 1646, 1648, 1649, 1650, 1651, 1652, 1653, 1654, 1656, 1657, 1658, 1660, 1661, 1663,
    1664, 1666, 1667, 1668, 1669, 1674, 1670, 1671, 1672, 1673, 1675, 1678, 1676, 1677, 1679, 1680,
    1682, 1683, 1684, 1686, 1687, 1688, 1689, 1690, 1692, 1695, 1696, 1693, 1694, 1697, 1698, 1699,
    1701, 1702, 1705, 1706, 1703, 1704, 1708, 1717, 1718, 1709, 1710, 1711, 1714, 1712, 1713, 1715,
    1716, 1720, 1727, 1728, 1729, 1721, 1722, 1723, 1724, 1725, 1726, 1731, 1732, 1733, 1735, 1736,
    1737, 1738, 1739, 1741, 1742, 1743, 1745, 1746, 1747, 1748, 1750, 1751, 1753, 1754, 1756, 1757,
    1758, 1760, 1761, 1763, 1764, 1766, 1767, 1768, 1769, 1770, 1771, 1772, 1774, 1775, 1776, 1777,
    1778, 1782, 1783, 1779, 1780, 1781, 1785, 1786, 1787, 1788, 1790, 1796, 1809, 1810, 1791, 1792,
    1793, 1794, 1795, 1797, 1800, 1801, 1802, 1798, 1799, 1803, 1808, 1804, 1807, 1805, 1806, 1811,
    1812, 1815, 1813, 1814, 1817, 1818, 1819, 1820, 1840, 1868, 1821, 1822, 1823, 1824, 1825, 1826,
    1828, 1834, 1827, 58, 1829, 1830, 1831, 1832, 1833, 1835, 1836, 1837, 1838, 1839, 1841, 1842,
    1843, 1844, 1848, 1852, 1862, 1845, 1846, 1847, 1849, 1850, 1851, 1853, 1854, 1855, 1856, 1857,
    1861, 1858, 1859, 1860, 1863, 1864, 1865, 1866, 1867, 1869, 1870, 1871, 1872, 1873, 1878, 1879,
    1880, 1881, 1874, 1875, 1876, 1877, 58, 1883, 1884, 1885, 1886, 1887, 1888, 1889, 1898, 1907,
    1914, 1924, 1931, 1935, 1943, 1890, 1891, 1892, 1893, 1895, 1896, 1894, 58, 1897, 58, 1899,
    1900, 1901, 1902, 1904, 1905, 1903, 58, 1906, 58, 1908, 1909, 1910, 1911, 1912, 1913, 1915,
    1916, 1917, 1918, 1920, 1921, 1922, 1923, 1919, 58, 1925, 1926, 1927, 1928, 1929, 1930, 58,
    1932, 1933, 1934, 1936, 1937, 1938, 1939, 1940, 1941, 1942, 1944, 1945, 1946, 1948, 1949, 1950,
    1951, 1952, 1953, 1954, 1958, 1955, 1956, 1957, 1959, 1960, 1961, 1962, 1963, 1965, 1966, 1967,
    1968, 1969, 1977, 1983, 1987, 1991, 2028, 1970, 1971, 1972, 1973, 1974, 1975, 1976, 1978, 1979,
    1980, 1981, 1982, 1984, 1985, 1986, 1988, 1989, 1990, 1992, 58, 1993, 2002, 2015, 2021, 1994,
    1995, 1996, 1997, 1998, 2001, 1999, 2000, 2003, 2004, 2005, 2006, 2009, 2010, 2011, 2014, 2007,
    2008, 2012, 2013, 2016, 2017, 2018, 2019, 2020, 2022, 2023, 2024, 2025, 2026, 2027, 2029, 2030,
    2031, 2032, 2033, 2034, 2037, 2035, 2036, 2039, 2040, 2041, 2042, 2043, 2044, 2060, 2068, 2074,
    2078, 2082, 2091, 2045, 2046, 2047, 2048, 2049, 2057, 2050, 2051, 2052, 2053, 2054, 2055, 2056,
    2058, 2059, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2069, 2070, 2071, 2072, 2073, 2075, 2076,
    2077, 2079, 2080, 2081, 2083, 2084, 2085, 231, 232, 2086, 58, 2087, 2088, 2089, 2090, 2092,
    2093, 2094, 231, 232, 2095, 58, 2101, 2104, 2113, 2102, 2103, 2105, 2112, 2106, 231, 232,
    2107, 58, 2108, 2111, 2109, 2110, 2114, 58, 2115, 2116, 2117, 231, 232, 2118, 58, 2126,
    2127, 2128, 2129, 2130, 2131, 2132, 2133, 2134, 2135, 2136, 2138, 2152, 2153, 2139, 2140, 2141,
    2142, 2143, 2147, 2148, 2144, 2145, 2146, 2149, 2150, 2151, 2157, 2152, 2153, 2158, 58, 2161,
    2152, 2153, 224, 2163, 2164, 234, 2166, 2180, 2187, 2188, 2202, 2189, 2201, 2190, 2191, 2192,
    2193, 2194, 2195, 2196, 2197, 2198, 2213, 2214, 2215, 2216, 2217, 2224, 2225, 2228, 2229, 2233,
    2234, 2237, 2238, 2243, 231, 232, 2244, 58, 2246, 231, 232, 2247, 58, 2250, 231, 232,
    2260, 231, 232, 2262, 2263, 2264, 2265, 2268, 2270, 2266, 2267, 2269, 231, 232, 2271, 2281,
    2272, 2273, 2276, 2277, 2278, 2279, 2280, 2274, 2275, 2282, 2283, 2284, 2285, 2286, 2287, 2291,
    2292, 2296, 2297, 2300, 2310, 2301, 2302, 2303, 2307, 234, 2309, 2304, 2311, 2312, 2302, 2313,
    2317, 234, 2319, 2314, 2344, 2345, 2346, 2347, 2352, 2348, 2349, 2350, 2351, 2353, 2354, 2355,
    2363, 2364, 2365, 2366, 2372, 2373, 2374, 2375, 2377, 2378, 2381, 2385, 2390, 2391, 2394, 2395,
    2379, 2380, 2382, 2383, 2384, 231, 232, 2386, 2387, 2388, 2389, 2392, 2393, 2396, 2397, 2399,
    2400, 2401, 2402, 2403, 2404, 2405, 2410, 2411, 2412, 2417, 2413, 2414, 2415, 2416, 2418, 2419,
    2420, 2421, 2422, 2423, 2436, 2437, 2438, 2440, 2441, 231, 232, 2442, 2443, 2444, 2447, 2448,
    2450, 2451, 2455, 2456, 2459, 2460, 2461, 2462, 2564, 2463, 2468, 2475, 2478, 2485, 2488, 2491,
    2496, 2499, 2502, 2555, 2464, 2467, 2465, 2466, 2469, 2470, 2471, 231, 232, 2472, 2473, 2474,
    2476, 2477, 2479, 2480, 2481, 2482, 2483, 2484, 2486, 2487, 2489, 2490, 2492, 2493, 2494, 2495,
    2497, 2498, 2500, 2501, 2503, 2551, 2552, 2504, 2505, 2506, 2507, 2508, 2509, 2510, 2511, 2512,
    2513, 2514, 2515, 2516, 2517, 2518, 2519, 2522, 2525, 2529, 2532, 2535, 2539, 2543, 2547, 2520,
    2521, 2523, 2524, 2526, 2527, 2528, 2530, 2531, 2533, 2534, 2536, 2537, 2538, 2540, 2541, 2542,
    2544, 2545, 2546, 2548, 2549, 2550, 2553, 2554, 2556, 2560, 2561, 2557, 2558, 2559, 2562, 231,
    232, 2563, 2565, 2566, 2567, 2568, 2575, 2576, 2577, 2578, 2579, 2580, 2581, 2582, 2583, 2584,
    2590, 2592, 2593, 2591, 231, 232, 2600, 2601, 2605, 2606, 2607, 2610, 2611, 2612, 2613, 2614,
    224, 2615, 2302, 2616, 234, 2642, 234, 2617, 58, 2618, 2624, 2635, 2619, 2620, 2621, 2622,
    2623, 2625, 2628, 2629, 2630, 2626, 231, 232, 2627, 58, 2631, 231, 232, 2632, 58, 2633,
    2634, 2636, 2639, 2640, 2637, 231, 232, 2638, 58, 2641, 231, 232, 2643, 2644, 2647, 2663,
    2648, 2660, 2649, 231, 232, 2650, 58, 2651, 2656, 2659, 2652, 2653, 2655, 2654, 231, 232,
    2657, 2658, 2661, 2662, 2664, 2665, 2666, 2677, 2678, 2679, 2734, 2680, 2688, 2689, 2690, 2681,
    2686, 2682, 2683, 2684, 231, 232, 2685, 58, 2687, 231, 232, 2691, 2692, 2693, 231, 232,
    2694, 58, 2695, 2698, 2701, 2704, 2707, 2714, 2717, 2722, 2725, 2728, 2731, 2696, 2697, 2699,
    2700, 2702, 2703, 2705, 2706, 2708, 2709, 2710, 2713, 2711, 231, 232, 2712, 58, 2715, 2716,
    2718, 2719, 2720, 2721, 2723, 2724, 2726, 2727, 2729, 2730, 2732, 2733, 2681, 2735, 2736, 2737,
    2738, 2739, 2740, 231, 232, 2741, 58, 2742, 2745, 2748, 2751, 2754, 2757, 2760, 2763, 2770,
    2775, 2780, 2743, 2744, 2746, 2747, 2749, 2750, 2752, 2753, 2755, 2756, 2758, 2759, 2761, 2762,
    2764, 2765, 2766, 2769, 2767, 231, 232, 2768, 58, 2771, 2772, 2773, 231, 232, 2774, 58,
    2776, 2777, 2778, 231, 232, 2779, 58, 2781, 2782, 2787, 2788, 2789, 2790, 2791, 2793, 2792,
    231, 232, 2795, 2796, 2797, 2801, 2813, 2798, 2799, 2800, 231, 232, 2802, 2807, 2810, 2803,
    2804, 2805, 2806, 2808, 2809, 2811, 2812, 2814, 2817, 2815, 2816, 2818, 2819, 2821, 2822, 2823,
    2824, 2825, 2826, 2855, 2827, 231, 232, 2828, 58, 2829, 2833, 2837, 2841, 2847, 2851, 2830,
    2831, 2832, 2834, 2835, 2836, 2838, 2839, 2840, 2842, 2843, 2844, 2845, 2846, 2848, 2849, 2850,
    2852, 2853, 2854, 2856, 2857, 2859, 2860, 2861, 2862, 2863, 2864, 2868, 2872, 2878, 2892, 2898,
    2902, 2907, 2865, 2866, 2867, 2869, 2870, 2871, 2873, 2874, 2875, 2876, 2877, 2879, 2883, 2880,
    2881, 2882, 2884, 2888, 2885, 2886, 2887, 2889, 2890, 2891, 2893, 2894, 2895, 2896, 2897, 2899,
    2900, 2901, 2903, 2904, 2905, 2906, 2908, 2909, 2910, 2912, 2913, 2914, 2915, 2917, 2918, 2921,
    2924, 2916, 2919, 2920, 2922, 2923, 2925, 2926, 2929, 2927, 2928, 2930, 2933, 2931, 2932, 2934,
    2935, 2937, 2938, 2939, 2940, 2941, 2942, 2945, 2948, 2951, 2943, 2944, 2946, 2947, 2949, 2950,
    2953, 2954, 2955, 224, 2956, 2957, 234, 224, 2958, 2959, 234, 2961, 2962, 2963, 2965, 2991,
    2966, 2967, 2968, 2969, 2982, 2987, 2990, 2970, 231, 232, 2971, 58, 2972, 2981, 2973, 2974,
    2975, 2976, 2979, 2977, 2978, 2980, 2983, 2984, 2985, 231, 232, 2986, 58, 2988, 2989, 2992,
    2993, 2994, 2995, 3016, 3019, 2996, 2997, 2998, 2999, 3010, 3000, 231, 232, 3001, 58, 3002,
    3009, 3003, 3004, 3007, 3005, 3006, 3008, 3011, 3015, 3012, 3013, 3014, 3017, 3018, 3021, 3022,
    3023, 3024, 3027, 3028, 3029, 3030, 3025, 3026, 3031, 3034, 3032, 224, 3033, 234, 3036, 3037,
    3038, 3039, 3040, 3043, 3044, 3045, 3046, 3049, 3052, 3057, 3061, 3064, 3067, 3041, 3042, 3047,
    3048, 3050, 231, 232, 3051, 58, 3053, 3054, 3055, 231, 232, 3056, 58, 3058, 3059, 3060,
    3062, 3063, 3065, 3066, 3068, 3069, 3072, 3073, 3074, 3075, 3076, 3077, 3078, 3079, 3082, 3085,
    3088, 3089, 3104, 3080, 3081, 3083, 3084, 3086, 3087, 3090, 3093, 3098, 3099, 3091, 3092, 3092,
    231, 232, 3094, 3095, 3095, 231, 232, 3096, 3097, 3100, 3101, 3101, 231, 232, 3102, 3103,
    3106, 3107, 3108, 3109, 3110, 3111, 3123, 3124, 3112, 3113, 3114, 231, 232, 3115, 58, 3116,
    3117, 3118, 3119, 3120, 3121, 2152, 2153, 3122, 58, 3126, 3127, 3128, 3130, 3129, 3131, 231,
    232, 3132, 58, 3133, 3139, 3144, 3149, 3158, 3159, 3134, 3137, 3138, 3135, 3136, 3140, 3141,
    3142, 3143, 3145, 3146, 3147, 3148, 3150, 3156, 3157, 3151, 3152, 3153, 3154, 3155, 3161, 3162,
    3163, 3164, 3165, 3166, 3171, 3179, 3188, 3192, 3167, 3168, 3169, 3170, 3172, 3173, 3174, 3175,
    3176, 3177, 3178, 3180, 3181, 3182, 58, 3183, 3184, 3185, 3186, 3187, 3189, 3190, 3191, 231,
    232, 3193, 3194, 3195, 3196, 231, 232, 3198, 3199, 3200, 3201, 3202, 3203, 3205, 3206, 3207,
    3208, 3217, 3209, 3213, 3210, 3211, 3212, 3214, 3215, 3216, 58, 3218, 3219, 3221, 3222, 3225,
    3226, 3223, 3224, 3227, 3228, 3229, 3232, 3235, 3238, 3241, 3244, 3247, 3250, 3254, 3260, 3264,
    3268, 3272, 3275, 3278, 3281, 3230, 3231, 3233, 3234, 3236, 3237, 3239, 3240, 3242, 3243, 3245,
    3246, 3248, 3249, 3251, 3252, 3253, 3255, 3256, 3257, 3258, 3259, 3261, 3262, 3263, 3265, 3266,
    3267, 3269, 3270, 3271, 3273, 3274, 3276, 3277, 3279, 3280, 3282, 3283, 3285, 3286, 3287, 3288,
    3289, 3293, 3290, 3291, 3292, 3295, 3298, 3303, 3311, 3319, 3324, 3327, 3332, 3296, 3297, 3299,
    3300, 3301, 3302, 3304, 3308, 3305, 3306, 3307, 3309, 3310, 3312, 3316, 3313, 3314, 3315, 3317,
    3318, 3320, 3321, 3322, 3323, 3325, 3326, 3328, 3329, 3330, 3331, 3333, 3334, 3335, 3336, 3337,
    3339, 3340, 3341, 3342, 3343, 3344, 3345, 3347, 3348, 3349, 3350, 3351, 3354, 3357, 3358, 3359,
    3352, 3353, 3355, 3356, 3361, 3362, 3363, 3366, 3373, 3374, 3378, 3379, 3364, 3365, 3367, 231,
    232, 3368, 58, 3369, 3370, 3371, 3372, 3375, 58, 3376, 3377, 3381, 3382, 3383, 3384, 3385,
    3386, 3387, 3389, 3388, 58, 3390, 3391, 3394, 3395, 3396, 3397, 3398, 3399, 3400, 3401, 3406,
    3402, 3403, 3404, 3405, 3405, 231, 232, 3407, 231, 232, 3408, 58, 3409, 3429, 3410, 3411,
    3412, 3417, 3413, 3414, 3415, 3416, 3418, 3420, 3423, 3424, 3426, 3419, 3421, 3422, 3425, 231,
    232, 3427, 3428, 3431, 3432, 3433, 3434, 3435, 3436, 3437, 231, 232, 3438, 3442, 3439, 3440,
    3441, 3443, 3444, 3445, 3447, 3448, 3449, 3450, 3451, 3452, 3458, 3472, 3473, 3474, 3516, 3453,
    3454, 3455, 3456, 3457, 3459, 3471, 3460, 3461, 3462, 3463, 3464, 3465, 3466, 3467, 3468, 58,
    3475, 3478, 3491, 3506, 3512, 3476, 3477, 3479, 3482, 3480, 3481, 3483, 3484, 3485, 3488, 3486,
    3487, 3489, 3490, 3492, 3493, 3496, 3497, 3498, 3499, 3494, 3495, 3500, 3503, 3504, 3505, 3501,
    3502, 3507, 3508, 3509, 3510, 3511, 3513, 3514, 3515, 231, 232, 3517, 3518, 3521, 3519, 3520,
    3523, 3524, 3525, 3526, 3527, 3528, 3529, 3534, 3535, 3530, 3531, 3532, 231, 232, 3533, 58,
    3536, 3537, 3539, 3540, 3541, 3543, 3544, 3545, 3546, 3550, 3551, 3552, 3547, 3548, 3549, 3554,
    3555, 3556, 3558, 3559, 3560, 3561, 3562, 3563, 3564, 3565, 3566, 3567, 3570, 3568, 3569, 3569,
    231, 232, 3579, 3582, 3580, 3581, 3583, 231, 232, 3598, 3599, 3603, 3604, 3605, 3607, 3610,
    3704, 3608, 3609, 3611, 3612, 3613, 3614, 3615, 3701, 3616, 3617, 3618, 3619, 3620, 3621, 3622,
    3623, 3624, 3625, 3626, 3627, 3630, 3631, 3632, 3633, 3634, 3635, 3636, 3640, 3645, 3646, 3661,
    3662, 3665, 3673, 3678, 3681, 3686, 3687, 3688, 3691, 3692, 3693, 3694, 3695, 3696, 3697, 3698,
    3628, 3629, 3637, 3639, 3638, 3641, 3644, 3642, 3643, 3647, 3660, 3648, 3659, 3649, 3650, 3651,
    3654, 3655, 3658, 3652, 3653, 3656, 3657, 3663, 3664, 3666, 3667, 3668, 3669, 3670, 3671, 3672,
    3674, 3675, 3676, 3677, 3679, 3680, 3682, 3683, 3685, 3684, 3689, 3690, 3699, 3700, 3702, 3703,
    3705, 3707, 3708, 3706, 3713, 3714, 231, 232, 3715, 58, 3716, 3717, 3719, 3720, 3722, 3723,
    3724, 3725, 3729, 3726, 3727, 3728, 3730, 231, 232, 3746, 3747, 3748, 3749, 3750, 3756, 3757,
    3763, 3764, 3766, 3767, 3768, 3769, 3771, 231, 232, 3772, 58, 3777, 3778, 3779, 3780, 3781,
    3782, 3783, 3789, 3790, 3797, 3784, 224, 3785, 234, 3786, 58, 3787, 3788, 3791, 3796, 3792,
    3793, 3794, 3795, 3798, 224, 3799, 234, 3800, 3801, 3802, 58, 3803, 3804, 3815, 3816, 3817,
    3827, 3828, 3829, 3830, 3831, 3832, 3833, 3840, 3841, 3844, 3845, 3846, 3849, 3847, 3848, 3850,
    3851, 3858, 3859, 3860, 3861, 3868, 3862, 58, 3863, 3864, 3865, 3866, 3867, 3870, 3871, 3873,
    3874, 3875, 3876, 3880, 3877, 3878, 3879, 231, 232, 3882, 3883, 3884, 3885, 3886, 3888, 3889,
    3890, 3891, 3892, 3893, 3894, 3895, 231, 232, 3897, 3898, 3899, 3900, 3902, 3901, 58, 3904,
    3905, 3906, 3907, 3908, 3910, 3911, 3912, 3913, 3914, 3916, 3915, 58, 3918, 3919, 3922, 3923,
    3924, 3925, 3920, 3921, 3927, 3928, 3929, 3930, 3931, 3933, 3932, 58, 3936, 3937, 3938, 3939,
    3940, 3942, 3941, 58, 3944, 3945, 3946, 3947, 3949, 3950, 3951, 3952, 3958, 3953, 58, 3954,
    3955, 3956, 3957, 3960, 3961, 3962, 3963, 3964, 3965, 3966, 3968, 3969, 3970, 3971, 3975, 3972,
    58, 3973, 3974, 3976, 3977, 3979, 3980, 3981, 3982, 3984, 3983, 58, 3986, 3987, 3988, 3989,
    3991, 3992, 3993, 3994, 3995, 3997, 3998, 3999, 4000, 4002, 4001, 58, 4004, 4005, 4006, 4007,
    4009, 4008, 58, 4010, 4011, 4013, 4014, 4015, 4016, 4017, 4018, 4019, 4021, 4022, 4023, 4024,
    4025, 4027, 4026, 58, 4029, 4030, 4031, 4032, 4033, 4034, 4035, 4037, 4038, 4039, 4040, 4041,
    4042, 4043, 4045, 4046, 4047, 4048, 4050, 4049, 58, 4052, 4053, 4054, 4055, 4057, 4058, 4059,
    4060, 4062, 4061, 58, 4067, 2302, 4068, 234, 231, 232, 4074, 4075, 4093, 4094, 4095, 4099,
    4100, 4101, 4102, 231, 232, 4103, 58, 4109, 4121, 4110, 4120, 4111, 4112, 4113, 4114, 4115,
    4116, 4117, 4118, 4119, 4122, 4123, 4124, 4125, 4126, 4127, 4129, 4130, 4139, 4131, 4136, 4132,
    4135, 4133, 4134, 4137, 231, 232, 4138, 58, 4140, 4141, 4142, 4143, 4147, 4148, 4149, 4150,
    4151, 4154, 4155, 4156, 4160, 4161, 4162, 4163, 4169, 4152, 4153, 4157, 4158, 4159, 4164, 4165,
    4166, 4167, 4168, 4170, 4171, 4173, 4176, 4177, 4178, 4179, 4174, 4175, 4181, 4182, 4183, 4184,
    4185, 4189, 4186, 4187, 4188, 4191, 4194, 4192, 4193, 4196, 4215, 4197, 4198, 4199, 4212, 4200,
    4204, 4210, 4201, 4202, 4203, 231, 232, 4205, 4208, 4206, 4207, 4209, 4213, 4214, 4216, 4218,
    4253, 4219, 4220, 4221, 4233, 4234, 4235, 4242, 4250, 4251, 4222, 231, 232, 4223, 4224, 4225,
    4226, 4227, 58, 4228, 4229, 4230, 4231, 4232, 4236, 4239, 4237, 4238, 4240, 231, 232, 4241,
    58, 4243, 4244, 4245, 4246, 4247, 4248, 4249, 4255, 4256, 4257, 4258, 4259, 4260, 4261, 4265,
    4266, 4271, 4267, 4268, 4269, 4270, 4270, 231, 232, 4272, 4273, 4282, 4283, 4286, 4290, 4293,
    4284, 4285, 4287, 4288, 4289, 4291, 4292, 4294, 4295, 4296, 4297, 4302, 4303, 4304, 4305, 4306,
    4307, 4308, 4311, 4312, 4313, 231, 232, 4314, 4315, 4325, 4326, 4336, 4340, 4327, 4328, 4332,
    4333, 4329, 4330, 4331, 4334, 4335, 4337, 4338, 4339, 58, 4341, 4342, 4343, 4344, 4351, 4352,
    4353, 4355, 4356, 4359, 4362, 4360, 4361, 4363, 4366, 4373, 4381, 4364, 4365, 4367, 4368, 4370,
    4369, 231, 232, 4371, 4372, 4374, 4375, 4376, 4377, 4378, 4379, 4380, 4382, 4383, 4384, 4385,
    4386, 4388, 4391, 4389, 4390, 4392, 4393, 4395, 4394, 231, 232, 4396, 4397, 4401, 4402, 4419,
    4403, 4416, 4404, 4407, 4405, 4406, 4408, 4413, 4409, 4410, 4411, 4412, 4414, 4415, 4417, 4403,
    4418, 4403, 4420, 4421, 4422, 4425, 4426, 4427, 4423, 4424, 4428, 4429, 4431, 4432, 4433, 4438,
    4439, 4440, 58, 4442, 4443, 4444, 4447, 4450, 4451, 4452, 4453, 4454, 4455, 4456, 4457, 4458,
    4445, 4446, 4448, 4449, 4460, 4478, 4461, 4462, 4465, 4466, 4472, 4476, 4477, 4463, 4464, 4464,
    231, 232, 4467, 4468, 4469, 4470, 4471, 4473, 4474, 4475, 4479, 231, 232, 4460, 4480, 4481,
    4483, 4484, 4484, 231, 232, 2302, 4485, 234, 4496, 4497, 4508, 4486, 4487, 4488, 4489, 231,
    232, 4491, 4492, 4494, 4495, 4498, 4499, 4500, 4501, 4503, 4504, 4506, 4507, 4511, 4512, 4513,
    4517, 4518, 4525, 4526, 4527, 231, 232, 4514, 58, 4515, 4516, 4519, 4520, 4521, 4522, 4523,
    231, 232, 4524, 58, 4529, 4530, 4533, 4531, 4532, 4534, 4539, 4540, 231, 232, 4535, 4536,
    4537, 231, 232, 4538, 58, 4541, 4542, 4543, 4544, 4545, 231, 232, 4546, 58, 4547, 4553,
    4548, 4552, 4549, 4550, 4550, 231, 232, 4551, 58, 4558, 4559, 4560, 4562, 4563, 4566, 4570,
    4576, 4587, 4590, 4596, 4597, 4600, 4612, 4613, 4614, 4615, 4564, 4565, 4565, 231, 232, 4567,
    4568, 4569, 4571, 4572, 4574, 4575, 4573, 4577, 4580, 4578, 4579, 4581, 4584, 4582, 4583, 4585,
    4586, 4588, 4589, 4591, 4592, 4593, 4594, 4595, 58, 4598, 4599, 4601, 4606, 4607, 4602, 4603,
    4604, 4605, 4608, 4609, 4610, 4611, 4616, 58, 4617, 4618, 4621, 4619, 4620, 4623, 4624, 231,
    232, 4626, 4722, 4627, 4628, 4633, 4634, 4638, 4641, 4654, 4660, 4666, 4669, 4672, 4675, 4690,
    4698, 4701, 4629, 4630, 4631, 4632, 4635, 4636, 4637, 4639, 4640, 4642, 4647, 4653, 4643, 4644,
    4645, 4646, 4648, 4649, 4650, 4651, 4652, 4655, 4656, 4657, 4658, 4659, 4661, 4662, 4663, 4664,
    4665, 4667, 4668, 4670, 4671, 4673, 4674, 4676, 4677, 4678, 4679, 4687, 4680, 4683, 4681, 4682,
    4684, 58, 4685, 4686, 4688, 4689, 4691, 4692, 4693, 4694, 4695, 4696, 4697, 4699, 4700, 4702,
    4703, 224, 4704, 234, 4721, 4705, 4708, 4711, 4717, 4706, 4707, 4709, 4710, 4712, 4716, 4713,
    4714, 4715, 4718, 4719, 4720, 4723, 4724, 4725, 231, 232, 4726, 58, 4730, 4735, 4731, 4732,
    4733, 4736, 4737, 4740, 4745, 4757, 4741, 4742, 4743, 4744, 4746, 4747, 4748, 4749, 4754, 4750,
    4751, 4752, 4753, 4755, 4756, 4758, 4762, 4759, 4760, 4761, 4764, 231, 232, 4765, 58, 4767,
    4770, 4771, 4768, 4769, 4772, 4779, 4773, 4778, 4774, 4775, 4776, 4777, 4794, 4795, 4809, 4810,
    4811, 4824, 4796, 4800, 4803, 4808, 4797, 4798, 4799, 4801, 4802, 4804, 4805, 4806, 4807, 4812,
    4816, 4820, 4813, 4814, 4815, 4817, 4818, 4819, 4821, 4822, 4823, 4825, 4826, 4827, 4828, 4829,
    4830, 4834, 4835, 4839, 4840, 224, 4841, 234, 4842, 58, 4843, 4844, 4845, 4846, 4849, 4852,
    4853, 4854, 4855, 4856, 4850, 4851, 4859, 4860, 58, 4861, 4862, 4863, 4864, 231, 232, 4868,
    4869, 4870, 4871, 231, 232, 4872, 58, 4873, 4874, 4880, 2302, 4881, 234, 4882, 4883, 4883,
    231, 232, 4885, 4888, 4889, 4890, 4891, 4886, 4887, 4912, 4913, 4916, 4917, 4918, 4921, 4924,
    4922, 4923, 4930, 4931, 4932, 4934, 4935, 4936, 4945, 4946, 4947, 4948, 4949, 4950, 4937, 4942,
    4938, 4941, 4939, 4940, 4943, 231, 232, 4944, 58, 4955, 4956, 4957, 4958, 4963, 4964, 4965,
    4966, 4967, 4959, 231, 232, 4960, 4962, 4961, 58, 4973, 4974, 4975, 4976, 4978, 4981, 4989,
    4992, 4979, 4980, 4982, 4988, 4983, 4984, 4985, 4986, 4987, 4990, 4991, 4993, 4994, 4995, 4996,
    4998, 5003, 4999, 5000, 5001, 5002, 231, 232, 5006, 5009, 5012, 5007, 5008, 5010, 5011, 5013,
    5014, 5015, 5016, 5033, 5034, 5037, 5038, 5039, 5040, 5048, 5060, 5065, 5035, 5036, 5041, 5042,
    5045, 5043, 5044, 5046, 5047, 5049, 5052, 5053, 5050, 5051, 5054, 5055, 5056, 5057, 5058, 5059,
    5059, 231, 232, 5061, 5062, 5063, 5064, 5066, 5067, 5068, 5069, 5070, 58, 5071, 5072, 5088,
    5089, 5090, 5095, 5096, 5101, 5091, 5092, 5093, 5094, 5097, 5098, 5099, 5100, 5102, 5103, 5106,
    5107, 5108, 5109, 5110, 5111, 5113, 5114, 5118, 5119, 5115, 5116, 5117, 5123, 5124, 5125, 5126,
    5127, 5128, 5130, 5131, 5132, 5133, 5143, 5156, 5161, 5144, 5145, 224, 5146, 5147, 234, 5149,
    5148, 5150, 5151, 5152, 5155, 5153, 5154, 5157, 5158, 224, 5159, 5160, 234, 5162, 224, 5163,
    234, 5166, 5168, 5171, 5169, 5170, 5173, 5174, 5175, 5178, 5176, 5177, 5177, 231, 232, 5180,
    5181, 5182, 5186, 5183, 5184, 5185, 5188, 224, 5189, 234, 5190, 231, 232, 5191, 58, 5195,
    5196, 5197, 5199, 5200, 5201, 5202, 5204, 5207, 5208, 5205, 5206, 5209, 5218, 5226, 5219, 5224,
    5225, 5220, 5221, 5222, 231, 232, 5223, 58, 5227, 5228, 5233, 5234, 5229, 5230, 5231, 231,
    232, 5232, 58, 5238, 5239, 5240, 5241, 5258, 5259, 5264, 5267, 5268, 5265, 5266, 5269, 5270,
    5272, 5273, 5274, 5277, 5275, 5276, 5281, 5284, 5282, 5283, 5285, 5286, 5287, 5288, 5289, 5290,
    5291, 5292, 5293, 5295, 224, 5296, 5302, 234, 5297, 5298, 5298, 231, 232, 5299, 5300, 5301,
    5303, 5308, 5304, 5305, 5306, 5307, 5309, 5310, 5320, 5321, 5322, 5323, 5324, 5325, 2152, 2153,
    5326, 5327, 5332, 5333, 5335, 5336, 5337, 5338, 5339, 5340, 5341, 5342, 5343, 5344, 5345, 5350,
    5351, 5360, 5361, 5362, 5363, 5367, 5393, 5364, 5366, 5365, 58, 5368, 5369, 5392, 5370, 5373,
    5376, 5379, 5383, 5384, 5388, 5389, 5371, 5372, 5374, 5375, 5377, 5378, 5380, 5381, 5382, 5385,
    5386, 5387, 5390, 5391, 5406, 5407, 5408, 5409, 5410, 5413, 231, 232, 5411, 5412, 5414, 5415,
    5416, 231, 232, 5424, 5425, 5426, 5428, 5429, 5430, 5432, 4080, 5431, 58, 5434, 231, 232,
    5435, 58, 5437, 5438, 5454, 224, 5455, 234, 5456, 5458, 5459, 5462, 5463, 5464, 5466, 5467,
    5468, 5471, 224, 5472, 234, 5469, 5470, 5473, 5474, 5475, 5476, 5477, 5479, 5480, 5481, 5482,
    5483, 5484, 5487, 224, 5488, 234, 5485, 5486, 5489, 5490, 5491, 5492, 5493, 5495, 5496, 5497,
    5498, 5501, 5504, 224, 5505, 234, 5499, 5500, 5502, 5503, 5506, 5507, 5508, 5509, 5511, 224,
    5512, 234, 5515, 224, 5516, 234, 5526, 5527, 5528, 5529, 5513, 5514, 5517, 5518, 5519, 5523,
    5520, 5522, 5521, 231, 232, 5524, 5525, 5531, 224, 5532, 234, 5536, 5537, 5538, 5539, 5564,
    5585, 5586, 5590, 5587, 5588, 5589, 5592, 5593, 5594, 5595, 5596, 5597, 231, 232, 5599, 224,
    5600, 234, 5601, 58, 5607, 5608, 5636, 5637, 5638, 5640, 5642, 5644, 5645, 5646, 5639, 231,
    232, 5641, 231, 232, 5643, 231, 232, 5648, 5649, 5650, 5651, 5652, 5653, 5654, 5655, 5658,
    5659, 5662, 5663, 5664, 5665, 5660, 5661, 5661, 231, 232, 5672, 5677, 5673, 5674, 5675, 5676,
    5682, 5685, 5683, 5684, 5686, 5687, 5689, 5690, 5692, 5693, 5700, 5701, 5702, 5703, 5704, 5716,
    5719, 5720, 5717, 5718, 5722, 5432, 4080, 5723, 58, 5725, 5726, 5727, 5728, 5730, 5731, 5732,
    5733, 5734, 5735, 5736, 5737, 5738, 5739, 5740, 2248, 5746, 224, 5747, 234, 5748, 5749, 5769,
    5770, 5771, 231, 232, 5772, 5775, 5776, 58, 5777, 5778, 5780, 5781, 5785, 5786, 5788, 5789,
    224, 5790, 234, 5791, 58, 5792, 5797, 5800, 5793, 5794, 5795, 5796, 5798, 5799, 5801, 5802,
    5803, 5804, 5807, 5808, 5809, 5810, 5811, 5812, 5813, 5816, 5814, 5815, 5819, 5820, 224, 5827,
    5828, 5829, 234, 5830, 5831, 5832, 231, 232, 5835, 5836, 5838, 5853, 5854, 5839, 5852, 5840,
    5841, 5846, 5847, 231, 232, 5842, 5843, 5844, 5845, 5848, 5849, 5850, 5851, 5855, 5868, 5856,
    5857, 5862, 5863, 231, 232, 5858, 5859, 5860, 5861, 5864, 5865, 5866, 5867, 5886, 5891, 5906,
    5887, 5888, 5889, 231, 232, 5890, 58, 5892, 5893, 5899, 5900, 5894, 231, 232, 5895, 58,
    5896, 5897, 5898, 5901, 231, 232, 5902, 58, 5903, 5904, 5905, 5907, 5908, 5909, 231, 232,
    5910, 5911, 5912, 5913, 5914, 5916, 5917, 224, 5918, 234, 5919, 5920, 5920, 231, 232, 5921,
    58, 5928, 5933, 5929, 5930, 5931, 5932, 5964, 5965, 5966, 5972, 5975, 5982, 5983, 5984, 5987,
    5988, 5985, 231, 232, 5986, 58, 5989, 5990, 5994, 5995, 6012, 6015, 6018, 6019, 6020, 6020,
    231, 232, 6021, 6022, 6023, 6024, 6028, 6037, 6029, 6030, 6031, 58, 6033, 6034, 6035, 6036,
    6038, 6039, 6040, 6041, 6042, 58, 6043, 6044, 6046, 6047, 6048, 6049, 6052, 6053, 6054, 6055,
    6056, 6057, 231, 232, 6060, 6061, 6063, 6064, 6065, 231, 232, 6066, 6067, 6068, 6069, 6071,
    6072, 6073, 6074, 6076, 2302, 6077, 234, 6078, 6079, 6079, 231, 232, 6082, 6083, 6087, 6084,
    6085, 6086, 6088, 231, 232, 6093, 6100, 6105, 6113, 6094, 6097, 6095, 6096, 6098, 6099, 6101,
    6102, 6103, 6104, 6106, 6107, 6110, 6108, 6109, 6111, 6112, 6114, 58, 6115, 6116, 6117, 6118,
    6132, 6133, 6134, 6135, 6141, 6142, 6136, 58, 6137, 6138, 6139, 6140, 6143, 6144, 6145, 6146,
    6152, 6153, 6154, 6155, 6156, 6157, 6161, 6158, 6159, 6160, 6162, 6163, 6164, 6166, 6167, 6168,
    6169, 6170, 6175, 6176, 6184, 6185, 6177, 6178, 6181, 6179, 6180, 6182, 231, 232, 6183, 58,
    6186, 6187, 6190, 6191, 6192, 6193, 6194, 6195, 6199, 6200, 6212, 6201, 231, 232, 6202, 58,
    6203, 6206, 6209, 6204, 6205, 6207, 6208, 6210, 6211, 6213, 6217, 6221, 6225, 6214, 6215, 6216,
    6218, 6219, 6220, 6222, 6223, 6224, 6226, 6229, 6230, 6227, 6228, 6232, 6233, 6236, 6234, 6235,
    6237, 6238, 6240, 231, 232, 6241, 58, 6256, 6257, 6258, 6260, 6261, 6262, 6263, 6264, 6278,
    6279, 6283, 6297, 6298, 6299, 6280, 6281, 6282, 6284, 6288, 6291, 6296, 6285, 6286, 6287, 6289,
    6290, 6292, 6293, 6294, 6295, 6323, 6353, 6324, 6330, 6336, 6346, 6325, 6326, 6327, 231, 232,
    6328, 6329, 6331, 6334, 6332, 6333, 6335, 231, 232, 6337, 6338, 6339, 6343, 231, 232, 6340,
    6341, 6342, 6344, 6345, 6347, 6348, 6349, 6350, 231, 232, 6351, 6352, 6354, 6355, 6356, 231,
    232, 6359, 6360, 6361, 6362, 6364, 6365, 6367, 6368, 6380, 6381, 6382, 6385, 6386, 6466, 6467,
    6383, 6384, 6387, 6390, 6393, 6398, 6401, 6404, 6408, 6412, 6418, 6422, 6426, 6430, 6436, 6439,
    6442, 6448, 6451, 6454, 6457, 6460, 6463, 6388, 6389, 6391, 6392, 6394, 6395, 6396, 6397, 231,
    232, 6399, 6400, 6402, 6403, 6405, 6406, 6407, 6409, 6410, 6411, 6413, 6414, 6415, 6416, 6417,
    6417, 231, 232, 6419, 6420, 6421, 6423, 6424, 6425, 6427, 6428, 6429, 6431, 6432, 6433, 6434,
    6435, 6435, 231, 232, 6437, 6438, 6440, 6441, 6443, 6444, 6445, 6446, 6447, 6447, 231, 232,
    6449, 6450, 6452, 6453, 6455, 6456, 6458, 6459, 6461, 6462, 6464, 6465, 6468, 6469, 6471, 6473,
    6472, 6474, 6475, 6477, 6484, 6478, 6479, 6480, 6481, 6482, 231, 232, 6483, 58, 6486, 6487,
    224, 6488, 234, 6489, 58, 6494, 6495, 6497, 6498, 6504, 6505, 6506, 6510, 6511, 6513, 6514,
    6517, 6518, 6519, 6520, 6521, 6512, 6515, 6516, 6524, 6525, 6526, 6527, 6528, 6529, 6530, 6531,
    6532, 6533, 6534, 6535, 6536, 6537, 6542, 6543, 6548, 6551, 6554, 6555, 6549, 6550, 6550, 231,
    232, 6552, 6553, 6553, 231, 232, 6556, 231, 232, 6559, 6560, 6561, 6562, 6563, 6566, 6564,
    6565, 6571, 6574, 6572, 6573, 6575, 6576, 6578, 6581, 6579, 6580, 6582, 6583, 6594, 6595, 6596,
    6597, 6601, 224, 6602, 234, 6603, 58, 6605, 6606, 6619, 6607, 6608, 6609, 6610, 6617, 6611,
    6612, 6613, 6614, 6615, 6616, 6618, 6620, 6623, 6624, 6621, 231, 232, 6622, 58, 6625, 231,
    232, 6626, 6627, 6628, 58, 6629, 6636, 6630, 6631, 6632, 6633, 6634, 6635, 6637, 6639, 6640,
    6641, 6645, 6642, 6643, 6644, 58, 6646, 6647, 6649, 6650, 6654, 6655, 6656, 6651, 6652, 6653,
    6659, 6660, 6665, 6661, 6664, 6662, 6663, 6667, 6668, 6669, 6670, 6672, 6673, 6680, 6681, 6674,
    6677, 6675, 6676, 6678, 6679, 6682, 6683, 6685, 6686, 6687, 6697, 6688, 6689, 6690, 6693, 6691,
    6692, 6694, 6695, 6696, 6698, 6702, 6703, 6706, 6699, 6700, 6701, 6704, 6705, 6707, 6708, 6709,
    58, 6710, 6711, 6712, 6713, 6721, 6724, 6722, 6723, 6725, 6726, 6727, 6728, 6731, 6732, 6733,
    6734, 6735, 6736, 6737, 6738, 6739, 6743, 6744, 6747, 6750, 6753, 6754, 6766, 6772, 6773, 6774,
    6751, 6752, 6755, 6756, 6757, 58, 6758, 6761, 6759, 6760, 6762, 6763, 6764, 231, 232, 6765,
    58, 6767, 6768, 6769, 58, 6770, 6771, 6776, 6779, 6780, 6781, 6789, 6796, 6797, 6777, 6778,
    6782, 58, 6783, 6788, 6784, 6785, 6786, 231, 232, 6787, 58, 6790, 6791, 6792, 6793, 6794,
    6795, 58, 6799, 6800, 6801, 6803, 6804, 6807, 58, 6809, 6810, 6811, 6814, 6815, 6850, 6851,
    6852, 6853, 6854, 6855, 6856, 6857, 6858, 6859, 6860, 6861, 6862, 6863, 6864, 6865, 6866, 6867,
    6868, 6869, 6870, 6871, 6872, 6873, 6874, 6875, 6876, 6877, 6878, 6879, 6880, 6881, 6882, 6883,
    6884, 6885, 6886, 6887, 6888, 6889, 6890, 6891, 6892, 6893, 6894, 6895, 6896, 6897, 6898, 6899,
    6900, 6901, 6902, 6903, 6904, 6905, 6906, 6907, 6908, 6909, 6910, 6911, 6912, 6913, 6914, 6915,
    6916, 6917, 6918, 6919, 6920, 6921, 6922, 6923, 6924, 6925, 6926, 6927, 6928, 6929, 6930, 6931,
    6932, 6933, 6934, 6935, 6936, 6937, 6938, 6939, 6940, 6941, 6942, 6943, 6944, 6945, 6946, 6947,
    6948, 6949, 6950, 6951, 6952, 6953, 6954, 6955, 6956, 6957, 6958, 6959, 6960, 6961, 6962, 6963,
    6964, 6965, 6966, 6967, 6968, 6969, 6970, 6971, 6972, 6984, 6985, 6989, 231, 232, 6990, 58,
    6991, 6992, 6994, 231, 232, 6995, 58, 6996, 6997, 6998, 6999, 7000, 7002, 7003, 7018, 7021,
    7059, 7019, 7020, 7022, 7029, 7037, 7041, 7054, 7023, 7024, 7026, 7025, 231, 232, 7027, 7028,
    7030, 7031, 7035, 7036, 7032, 7033, 7034, 7038, 7039, 7040, 7042, 7043, 7046, 7049, 7050, 7044,
    7045, 7047, 231, 232, 7048, 58, 7051, 7052, 7053, 231, 232, 7055, 7056, 7057, 7058, 7060,
    7065, 7072, 7075, 7061, 7062, 7063, 7064, 7066, 7069, 7067, 7068, 7070, 7071, 7073, 7074, 7076,
    7077, 7079, 7082, 7089, 7094, 7080, 7081, 7083, 7086, 7087, 7088, 7084, 7085, 7090, 7091, 7092,
    7093, 7095, 7098, 7096, 7097, 7099, 7100, 7103, 7104, 7105, 7110, 7111, 7113, 7106, 7107, 7108,
    7109, 7112, 231, 232, 7114, 231, 232, 7117, 3599, 7118, 7119, 7124, 7127, 7125, 7129, 7131,
    7130, 7139, 7140, 7143, 7144, 7155, 7158, 7159, 7156, 7157, 7160, 7163, 7164, 7161, 7162, 7166,
    7167, 7168, 7169, 7170, 7186, 7192, 7195, 7198, 7187, 7188, 7189, 7190, 7191, 7193, 7194, 7196,
    7197, 7199, 7200, 7201, 7210, 7211, 7212, 7217, 7218, 7219, 7227, 7232, 7220, 58, 7221, 7224,
    7222, 7223, 7225, 7226, 7228, 7231, 7229, 7230, 7240, 231, 232, 7241, 58, 7244, 7249, 7250,
    7245, 7246, 7247, 7248, 7252, 7253, 7261, 7262, 7263, 7267, 7268, 7271, 7269, 7270, 7272, 7273,
    7276, 7282, 7283, 7284, 7285, 7289, 7296, 7297, 7298, 7299, 7300, 7301, 7290, 7293, 7291, 7292,
    7294, 7295, 7306, 7307, 7310, 7311, 7312, 7313, 7314, 7322, 7323, 224, 7324, 7325, 234, 7327,
    7328, 7329, 7336, 7326, 7330, 7335, 7331, 7332, 7333, 7334, 7337, 224, 7338, 234, 7339, 7340,
    7341, 58, 7342, 7343, 7348, 7349, 7356, 7357, 7358, 7365, 7366, 7393, 7367, 7374, 7368, 7369,
    7370, 7371, 7372, 7373, 7375, 231, 232, 7376, 58, 7377, 7389, 7378, 7379, 7380, 7381, 7382,
    7383, 7384, 7385, 7386, 7387, 7388, 7390, 7391, 7392, 7394, 58, 7395, 7396, 7402, 7403, 7409,
    7410, 7411, 7404, 58, 7405, 231, 232, 7406, 58, 7407, 7408, 7423, 7424, 7438, 7489, 7439,
    7476, 7440, 7441, 7442, 7443, 7444, 7445, 7446, 7447, 7448, 7449, 7450, 7451, 7452, 7453, 7454,
    7455, 7456, 7457, 7458, 7459, 7460, 7461, 7462, 7463, 7464, 7465, 7466, 7467, 7468, 7469, 7470,
    7471, 7472, 7473, 7474, 7475, 7477, 231, 232, 7478, 58, 7479, 7483, 7488, 7480, 7481, 7481,
    231, 232, 7482, 58, 7484, 231, 232, 7485, 58, 7486, 231, 232, 7487, 58, 7490, 7493,
    7491, 7492, 7494, 231, 232, 7495, 7534, 7496, 7497, 7498, 7499, 7500, 7501, 7502, 7503, 7504,
    7505, 7506, 7507, 7508, 7509, 7510, 7511, 7512, 7513, 7514, 7515, 7516, 7517, 7518, 7519, 7520,
    7521, 7522, 7523, 7524, 7525, 7526, 7527, 7528, 7529, 7530, 7531, 7532, 7533, 7535, 7536, 7538,
    7539, 7541, 7542, 7543, 7544, 7545, 7546, 2302, 7540, 234, 7549, 2302, 7550, 234, 7551, 7552,
    7552, 231, 232, 7554, 7555, 7558, 7561, 7562, 7563, 7564, 7565, 7566, 7569, 7556, 7557, 7559,
    7560, 7567, 7568, 7575, 7576, 7577, 7578, 7579, 231, 232, 7580, 58, 7582, 7583, 7584, 7585,
    7588, 7586, 7587, 7590, 7598, 7591, 7592, 7595, 7593, 7594, 7596, 7597, 7600, 7601, 7602, 7605,
    7603, 7604, 7607, 7608, 7609, 7614, 7610, 231, 232, 7611, 58, 7612, 7613, 7615, 7616, 7618,
    7619, 7620, 7625, 7621, 231, 232, 7622, 58, 7623, 7624, 7626, 7627, 7630, 7634, 7635, 7636,
    7631, 7632, 7633, 7638, 7639, 7642, 7643, 7640, 7641, 7645, 7646, 7647, 7648, 7651, 7652, 7649,
    58, 7654, 7655, 7656, 7657, 7660, 7661, 7658, 58, 7663, 7664, 7666, 7665, 7670, 7671, 7672,
    7673, 7675, 7676, 7677, 7678, 231, 232,
];

pub static TERMINATORS: &[u32] = &[
    59, 60, 136, 331, 352, 365, 366, 386, 387, 388, 394, 395, 2199, 2200, 2305, 2306,
    2308, 2315, 2316, 2318, 2320, 2321, 2322, 2367, 2424, 2425, 2426, 2427, 2428, 2429, 2430, 2431,
    2432, 2433, 2434, 2248, 3469, 3470, 3600, 4211, 4252, 4490, 4493, 4502, 4505, 4734, 4738, 4847,
    4865, 5805, 6032, 6045, 6490, 6491, 6492, 6493, 6496, 6499, 6500, 6501, 6502, 6503, 6507, 6508,
    6538, 6539, 6540, 6541, 6544, 6545, 6546, 6805, 6973, 3600, 7126, 7302, 7303, 7304, 7305, 7308,
    7650, 7659,
];

pub static STRINGS: &[&str] = &[
    "ABORT", // [0]
    "keyword", // [1]
    "KeywordSegment", // [2]
    "base", // [3]
    "raw", // [4]
    "word", // [5]
    "ABSOLUTE", // [6]
    "at", // [7]
    "sign_indicator", // [8]
    "symbol", // [9]
    "SymbolSegment", // [10]
    "ACCESS", // [11]
    "AccountKeywordSegment", // [12]
    "ACCOUNT", // [13]
    "ALL", // [14]
    "CATALOG", // [15]
    "DATABASE", // [16]
    "DOMAIN", // [17]
    "EXTERNAL", // [18]
    "FILE", // [19]
    "FOREIGN", // [20]
    "FUNCTION", // [21]
    "FUTURE", // [22]
    "INTEGRATION", // [23]
    "LANGUAGE", // [24]
    "MASKING", // [25]
    "MATERIALIZED", // [26]
    "PROCEDURE", // [27]
    "RESOURCE", // [28]
    "ROLE", // [29]
    "ROUTINE", // [30]
    "SCHEMA", // [31]
    "SEQUENCE", // [32]
    "STAGE", // [33]
    "STREAM", // [34]
    "TABLE", // [35]
    "TABLESPACE", // [36]
    "TASK", // [37]
    "TYPE", // [38]
    "VIEW", // [39]
    "WAREHOUSE", // [40]
    "ResourceKeywordSegment", // [41]
    "MonitorKeywordSegment", // [42]
    "MONITOR", // [43]
    "WarehouseKeywordSegment", // [44]
    "DatabaseKeywordSegment", // [45]
    "DomainKeywordSegment", // [46]
    "IntegrationKeywordSegment", // [47]
    "LanguageKeywordSegment", // [48]
    "SchemaKeywordSegment", // [49]
    "CatalogKeywordSegment", // [50]
    "RoleKeywordSegment", // [51]
    "TablespaceKeywordSegment", // [52]
    "TypeKeywordSegment", // [53]
    "ForeignKeywordSegment", // [54]
    "DATA", // [55]
    "SERVER", // [56]
    "ServerKeywordSegment", // [57]
    "DataKeywordSegment", // [58]
    "WrapperKeywordSegment", // [59]
    "WRAPPER", // [60]
    "AllKeywordSegment", // [61]
    "SchemasKeywordSegment", // [62]
    "SCHEMAS", // [63]
    "InKeywordSegment", // [64]
    "IN", // [65]
    "FutureKeywordSegment", // [66]
    "AccessSchemaObjectSegment", // [67]
    "access_schema_object", // [68]
    "AccessSchemaPluralObjectSegment", // [69]
    "FUNCTIONS", // [70]
    "PROCEDURES", // [71]
    "ROUTINES", // [72]
    "SEQUENCES", // [73]
    "STAGES", // [74]
    "STREAMS", // [75]
    "TABLES", // [76]
    "TASKS", // [77]
    "VIEWS", // [78]
    "access_schema_plural_object", // [79]
    "ObjectReferenceSegment", // [80]
    "object_reference", // [81]
    "FunctionNameSegment", // [82]
    "function_name", // [83]
    "FunctionParameterListGrammar", // [84]
    "(", // [85]
    "function_parameter_list", // [86]
    "WildcardIdentifierSegment", // [87]
    "wildcard_identifier", // [88]
    "CommaSegment", // [89]
    ",", // [90]
    "ToKeywordSegment", // [91]
    "TO", // [92]
    "FromKeywordSegment", // [93]
    "FROM", // [94]
    "LARGE", // [95]
    "LargeKeywordSegment", // [96]
    "ObjectKeywordSegment", // [97]
    "OBJECT", // [98]
    "NumericLiteralSegment", // [99]
    "numeric_literal", // [100]
    "APPLY", // [101]
    "CONNECT", // [102]
    "CREATE", // [103]
    "DELETE", // [104]
    "EXECUTE", // [105]
    "IMPORTED", // [106]
    "INSERT", // [107]
    "MANAGE", // [108]
    "MODIFY", // [109]
    "OPERATE", // [110]
    "OWNERSHIP", // [111]
    "READ", // [112]
    "REFERENCES", // [113]
    "REFERENCE_USAGE", // [114]
    "SELECT", // [115]
    "TEMP", // [116]
    "TEMPORARY", // [117]
    "TRIGGER", // [118]
    "TRUNCATE", // [119]
    "UPDATE", // [120]
    "USAGE", // [121]
    "USE", // [122]
    "USE_ANY_ROLE", // [123]
    "WRITE", // [124]
    "CreateKeywordSegment", // [125]
    "USER", // [126]
    "UserKeywordSegment", // [127]
    "ApplyKeywordSegment", // [128]
    "MaskingKeywordSegment", // [129]
    "PolicyKeywordSegment", // [130]
    "POLICY", // [131]
    "ExecuteKeywordSegment", // [132]
    "TaskKeywordSegment", // [133]
    "ManageKeywordSegment", // [134]
    "GrantsKeywordSegment", // [135]
    "GRANTS", // [136]
    "ExecutionKeywordSegment", // [137]
    "UsageKeywordSegment", // [138]
    "PIPE", // [139]
    "PipeKeywordSegment", // [140]
    "UseKeywordSegment", // [141]
    "ImportedKeywordSegment", // [142]
    "PrivilegesKeywordSegment", // [143]
    "PRIVILEGES", // [144]
    "ConnectKeywordSegment", // [145]
    "DeleteKeywordSegment", // [146]
    "InsertKeywordSegment", // [147]
    "ModifyKeywordSegment", // [148]
    "OperateKeywordSegment", // [149]
    "OwnershipKeywordSegment", // [150]
    "ReadKeywordSegment", // [151]
    "Reference_usageKeywordSegment", // [152]
    "ReferencesKeywordSegment", // [153]
    "SelectKeywordSegment", // [154]
    "TempKeywordSegment", // [155]
    "TemporaryKeywordSegment", // [156]
    "TriggerKeywordSegment", // [157]
    "TruncateKeywordSegment", // [158]
    "UpdateKeywordSegment", // [159]
    "Use_any_roleKeywordSegment", // [160]
    "WriteKeywordSegment", // [161]
    "BracketedColumnReferenceListGrammar", // [162]
    "AccessPermissionSegment", // [163]
    "access_permission", // [164]
    "OnKeywordSegment", // [165]
    "ON", // [166]
    "TableKeywordSegment", // [167]
    "ViewKeywordSegment", // [168]
    "StageKeywordSegment", // [169]
    "FunctionKeywordSegment", // [170]
    "ProcedureKeywordSegment", // [171]
    "RoutineKeywordSegment", // [172]
    "SequenceKeywordSegment", // [173]
    "StreamKeywordSegment", // [174]
    "MaterializedKeywordSegment", // [175]
    "ExternalKeywordSegment", // [176]
    "FileKeywordSegment", // [177]
    "FormatKeywordSegment", // [178]
    "FORMAT", // [179]
    "TablesKeywordSegment", // [180]
    "ViewsKeywordSegment", // [181]
    "StagesKeywordSegment", // [182]
    "FunctionsKeywordSegment", // [183]
    "ProceduresKeywordSegment", // [184]
    "RoutinesKeywordSegment", // [185]
    "SequencesKeywordSegment", // [186]
    "StreamsKeywordSegment", // [187]
    "TasksKeywordSegment", // [188]
    "PoliciesKeywordSegment", // [189]
    "FormatsKeywordSegment", // [190]
    "GRANT", // [191]
    "REVOKE", // [192]
    "GrantStatementSegment", // [193]
    "grant_statement", // [194]
    "RevokeStatementSegment", // [195]
    "revoke_statement", // [196]
    "GROUP", // [197]
    "GroupKeywordSegment", // [198]
    "UserReferenceSegment", // [199]
    "user_reference", // [200]
    "RoleReferenceSegment", // [201]
    "role_reference", // [202]
    "SHARE", // [203]
    "ShareKeywordSegment", // [204]
    "FunctionSegment", // [205]
    "function", // [206]
    "PublicKeywordSegment", // [207]
    "PUBLIC", // [208]
    ".", // [209]
    "[", // [210]
    "ArrayAccessorSegment", // [211]
    "array_accessor", // [212]
    "SemiStructuredAccessorSegment", // [213]
    "semi_structured_expression", // [214]
    "ACCOUNTS", // [215]
    "ACTION", // [216]
    "ADD", // [217]
    "-", // [218]
    "MinusSegment", // [219]
    "PipeSegment", // [220]
    "|", // [221]
    "ADMIN", // [222]
    "AFTER", // [223]
    "AGGREGATE", // [224]
    "OrderByClauseSegment", // [225]
    "ORDER", // [226]
    "orderby_clause", // [227]
    "indent", // [228]
    "Indent", // [229]
    "meta", // [230]
    "AsAliasOperatorSegment", // [231]
    "AS", // [232]
    "alias_operator", // [233]
    "SingleIdentifierGrammar", // [234]
    "SingleIdentifierListSegment", // [235]
    "identifier_list", // [236]
    "start_bracket", // [237]
    ")", // [238]
    "end_bracket", // [239]
    "SingleQuotedIdentifierSegment", // [240]
    "single_quote", // [241]
    "dedent", // [242]
    "Dedent", // [243]
    "ALIAS", // [244]
    "TableReferenceSegment", // [245]
    "table_reference", // [246]
    "AliasExpressionSegment", // [247]
    "alias_expression", // [248]
    "ALLOW_CONNECTIONS", // [249]
    "ALSO", // [250]
    "ALTER", // [251]
    "AlterKeywordSegment", // [252]
    "AggregateKeywordSegment", // [253]
    "StarSegment", // [254]
    "*", // [255]
    "OWNER", // [256]
    "RENAME", // [257]
    "SET", // [258]
    "RenameKeywordSegment", // [259]
    "OwnerKeywordSegment", // [260]
    "Current_roleKeywordSegment", // [261]
    "CURRENT_ROLE", // [262]
    "Current_userKeywordSegment", // [263]
    "CURRENT_USER", // [264]
    "Session_userKeywordSegment", // [265]
    "SESSION_USER", // [266]
    "SetKeywordSegment", // [267]
    "SchemaReferenceSegment", // [268]
    "schema_reference", // [269]
    "DatabaseReferenceSegment", // [270]
    "database_reference", // [271]
    "CONNECTION", // [272]
    "IS_TEMPLATE", // [273]
    "RESET", // [274]
    "WITH", // [275]
    "WithKeywordSegment", // [276]
    "Allow_connectionsKeywordSegment", // [277]
    "BooleanLiteralGrammar", // [278]
    "FALSE", // [279]
    "TRUE", // [280]
    "ConnectionKeywordSegment", // [281]
    "LimitKeywordSegment", // [282]
    "LIMIT", // [283]
    "Is_templateKeywordSegment", // [284]
    "TablespaceReferenceSegment", // [285]
    "tablespace_reference", // [286]
    "ParameterNameSegment", // [287]
    "=", // [288]
    "EqualsSegment", // [289]
    "comparison_operator", // [290]
    "DefaultKeywordSegment", // [291]
    "DEFAULT", // [292]
    "LiteralGrammar", // [293]
    "+", // [294]
    ":", // [295]
    "ARRAY", // [296]
    "DATE", // [297]
    "INTERVAL", // [298]
    "NULL", // [299]
    "TIME", // [300]
    "TIMESTAMP", // [301]
    "TIMESTAMPTZ", // [302]
    "TIMETZ", // [303]
    "VARIADIC", // [304]
    "{", // [305]
    "bit_string_literal", // [306]
    "dollar_numeric_literal", // [307]
    "dollar_quote", // [308]
    "escaped_single_quote", // [309]
    "newline", // [310]
    "unicode_single_quote", // [311]
    "NakedIdentifierSegment", // [312]
    "QuotedIdentifierSegment", // [313]
    "double_quote", // [314]
    "CurrentKeywordSegment", // [315]
    "CURRENT", // [316]
    "ResetKeywordSegment", // [317]
    "GrantKeywordSegment", // [318]
    "AlterDefaultPrivilegesObjectPrivilegesSegment", // [319]
    "alter_default_privileges_object_privilege", // [320]
    "AlterDefaultPrivilegesSchemaObjectsSegment", // [321]
    "TYPES", // [322]
    "alter_default_privileges_schema_object", // [323]
    "AlterDefaultPrivilegesToFromRolesSegment", // [324]
    "alter_default_privileges_to_from_roles", // [325]
    "OptionKeywordSegment", // [326]
    "OPTION", // [327]
    "RevokeKeywordSegment", // [328]
    "ForKeywordSegment", // [329]
    "FOR", // [330]
    "RestrictKeywordSegment", // [331]
    "RESTRICT", // [332]
    "CascadeKeywordSegment", // [333]
    "CASCADE", // [334]
    "DropBehaviorGrammar", // [335]
    "TypesKeywordSegment", // [336]
    "AlterDefaultPrivilegesGrantSegment", // [337]
    "alter_default_privileges_grant", // [338]
    "AlterDefaultPrivilegesRevokeSegment", // [339]
    "alter_default_privileges_revoke", // [340]
    "DROP", // [341]
    "VALIDATE", // [342]
    "ExpressionSegment", // [343]
    "expression", // [344]
    "DropKeywordSegment", // [345]
    "NotKeywordSegment", // [346]
    "NOT", // [347]
    "NullKeywordSegment", // [348]
    "AddKeywordSegment", // [349]
    "CONSTRAINT", // [350]
    "ConstraintKeywordSegment", // [351]
    "CHECK", // [352]
    "CheckKeywordSegment", // [353]
    "ValidKeywordSegment", // [354]
    "VALID", // [355]
    "IfExistsGrammar", // [356]
    "IF", // [357]
    "ValidateKeywordSegment", // [358]
    "ExtensionKeywordSegment", // [359]
    "EXTENSION", // [360]
    "ExtensionReferenceSegment", // [361]
    "extension_reference", // [362]
    "Current_schemaKeywordSegment", // [363]
    "CURRENT_SCHEMA", // [364]
    "CAST", // [365]
    "COLLATION", // [366]
    "CONVERSION", // [367]
    "EVENT", // [368]
    "OPERATOR", // [369]
    "PROCEDURAL", // [370]
    "TEXT", // [371]
    "TRANSFORM", // [372]
    "AccessKeywordSegment", // [373]
    "MethodKeywordSegment", // [374]
    "METHOD", // [375]
    "CollationKeywordSegment", // [376]
    "ConversionKeywordSegment", // [377]
    "EventKeywordSegment", // [378]
    "ProceduralKeywordSegment", // [379]
    "TextKeywordSegment", // [380]
    "SearchKeywordSegment", // [381]
    "SEARCH", // [382]
    "CONFIGURATION", // [383]
    "DICTIONARY", // [384]
    "PARSER", // [385]
    "TEMPLATE", // [386]
    "ConfigurationKeywordSegment", // [387]
    "DictionaryKeywordSegment", // [388]
    "ParserKeywordSegment", // [389]
    "TemplateKeywordSegment", // [390]
    "CastKeywordSegment", // [391]
    "AsKeywordSegment", // [392]
    "OperatorKeywordSegment", // [393]
    "DatatypeSegment", // [394]
    "data_type", // [395]
    "CLASS", // [396]
    "FAMILY", // [397]
    "ClassKeywordSegment", // [398]
    "FamilyKeywordSegment", // [399]
    "UsingKeywordSegment", // [400]
    "USING", // [401]
    "IndexAccessMethodSegment", // [402]
    "index_access_method", // [403]
    "TransformKeywordSegment", // [404]
    "CLUSTER", // [405]
    "DISABLE", // [406]
    "ENABLE", // [407]
    "FORCE", // [408]
    "INHERIT", // [409]
    "NO", // [410]
    "OF", // [411]
    "OPTIONS", // [412]
    "REPLICA", // [413]
    "ColumnKeywordSegment", // [414]
    "COLUMN", // [415]
    "IfNotExistsGrammar", // [416]
    "ColumnReferenceSegment", // [417]
    "column_reference", // [418]
    "COLLATE", // [419]
    "CollateKeywordSegment", // [420]
    "CollationReferenceSegment", // [421]
    "collation_reference", // [422]
    "PRIMARY", // [423]
    "UNIQUE", // [424]
    "ColumnConstraintSegment", // [425]
    "column_constraint_segment", // [426]
    "RESTART", // [427]
    "BareFunctionSegment", // [428]
    "CURRENT_CATALOG", // [429]
    "CURRENT_DATE", // [430]
    "CURRENT_TIME", // [431]
    "CURRENT_TIMESTAMP", // [432]
    "LOCALTIME", // [433]
    "LOCALTIMESTAMP", // [434]
    "SYSTEM_USER", // [435]
    "ExpressionKeywordSegment", // [436]
    "EXPRESSION", // [437]
    "GeneratedKeywordSegment", // [438]
    "GENERATED", // [439]
    "ALWAYS", // [440]
    "BY", // [441]
    "AlwaysKeywordSegment", // [442]
    "ByKeywordSegment", // [443]
    "IdentityKeywordSegment", // [444]
    "IDENTITY", // [445]
    "CACHE", // [446]
    "CYCLE", // [447]
    "INCREMENT", // [448]
    "MAXVALUE", // [449]
    "MINVALUE", // [450]
    "OWNED", // [451]
    "START", // [452]
    "AlterSequenceOptionsSegment", // [453]
    "alter_sequence_options_segment", // [454]
    "RestartKeywordSegment", // [455]
    "StatisticsKeywordSegment", // [456]
    "STATISTICS", // [457]
    "RelationOptionsSegment", // [458]
    "relation_options", // [459]
    "StorageKeywordSegment", // [460]
    "STORAGE", // [461]
    "EXTENDED", // [462]
    "MAIN", // [463]
    "PLAIN", // [464]
    "PlainKeywordSegment", // [465]
    "ExtendedKeywordSegment", // [466]
    "MainKeywordSegment", // [467]
    "CompressionKeywordSegment", // [468]
    "COMPRESSION", // [469]
    "TableConstraintSegment", // [470]
    "EXCLUDE", // [471]
    "table_constraint", // [472]
    "TableConstraintUsingIndexSegment", // [473]
    "DEFERRABLE", // [474]
    "DeferrableKeywordSegment", // [475]
    "INITIALLY", // [476]
    "InitiallyKeywordSegment", // [477]
    "DeferredKeywordSegment", // [478]
    "DEFERRED", // [479]
    "ImmediateKeywordSegment", // [480]
    "IMMEDIATE", // [481]
    "EnableKeywordSegment", // [482]
    "DisableKeywordSegment", // [483]
    "ReplicaKeywordSegment", // [484]
    "RuleKeywordSegment", // [485]
    "RULE", // [486]
    "ForceKeywordSegment", // [487]
    "NoKeywordSegment", // [488]
    "RowKeywordSegment", // [489]
    "ROW", // [490]
    "LevelKeywordSegment", // [491]
    "LEVEL", // [492]
    "SecurityKeywordSegment", // [493]
    "SECURITY", // [494]
    "ClusterKeywordSegment", // [495]
    "WithoutKeywordSegment", // [496]
    "WITHOUT", // [497]
    "OIDS", // [498]
    "OidsKeywordSegment", // [499]
    "LOGGED", // [500]
    "UNLOGGED", // [501]
    "LoggedKeywordSegment", // [502]
    "UnloggedKeywordSegment", // [503]
    "InheritKeywordSegment", // [504]
    "OfKeywordSegment", // [505]
    "FULL", // [506]
    "NOTHING", // [507]
    "IndexKeywordSegment", // [508]
    "INDEX", // [509]
    "IndexReferenceSegment", // [510]
    "index_reference", // [511]
    "FullKeywordSegment", // [512]
    "NothingKeywordSegment", // [513]
    "OptionsKeywordSegment", // [514]
    "QuotedLiteralSegment", // [515]
    "OnlyKeywordSegment", // [516]
    "ONLY", // [517]
    "AlterForeignTableActionSegment", // [518]
    "alter_foreign_table_action_segment", // [519]
    "alter_table_action_segment", // [520]
    "CALLED", // [521]
    "COST", // [522]
    "IMMUTABLE", // [523]
    "LEAKPROOF", // [524]
    "PARALLEL", // [525]
    "RETURNS", // [526]
    "ROWS", // [527]
    "STABLE", // [528]
    "STRICT", // [529]
    "SUPPORT", // [530]
    "VOLATILE", // [531]
    "CalledKeywordSegment", // [532]
    "InputKeywordSegment", // [533]
    "INPUT", // [534]
    "ReturnsKeywordSegment", // [535]
    "StrictKeywordSegment", // [536]
    "ImmutableKeywordSegment", // [537]
    "StableKeywordSegment", // [538]
    "VolatileKeywordSegment", // [539]
    "LeakproofKeywordSegment", // [540]
    "DEFINER", // [541]
    "INVOKER", // [542]
    "DefinerKeywordSegment", // [543]
    "InvokerKeywordSegment", // [544]
    "ParallelKeywordSegment", // [545]
    "RESTRICTED", // [546]
    "SAFE", // [547]
    "UNSAFE", // [548]
    "UnsafeKeywordSegment", // [549]
    "RestrictedKeywordSegment", // [550]
    "SafeKeywordSegment", // [551]
    "CostKeywordSegment", // [552]
    "RowsKeywordSegment", // [553]
    "SupportKeywordSegment", // [554]
    "DEPENDS", // [555]
    "AlterFunctionActionSegment", // [556]
    "alter_function_action_segment", // [557]
    "DependsKeywordSegment", // [558]
    "ATTACH", // [559]
    "AttachKeywordSegment", // [560]
    "PartitionKeywordSegment", // [561]
    "PARTITION", // [562]
    "OwnedKeywordSegment", // [563]
    "NowaitKeywordSegment", // [564]
    "NOWAIT", // [565]
    "QualifiedParameterNameGrammar", // [566]
    "AlterMaterializedViewActionSegment", // [567]
    "alter_materialized_view_action_segment", // [568]
    "AlterProcedureActionSegment", // [569]
    "alter_procedure_action_segment", // [570]
    "PublicationKeywordSegment", // [571]
    "PUBLICATION", // [572]
    "PublicationReferenceSegment", // [573]
    "publication_reference", // [574]
    "DefinitionParametersSegment", // [575]
    "definition_parameters", // [576]
    "PublicationObjectsSegment", // [577]
    "publication_objects", // [578]
    "BYPASSRLS", // [579]
    "CREATEDB", // [580]
    "CREATEROLE", // [581]
    "ENCRYPTED", // [582]
    "LOGIN", // [583]
    "NOBYPASSRLS", // [584]
    "NOCREATEDB", // [585]
    "NOCREATEROLE", // [586]
    "NOINHERIT", // [587]
    "NOLOGIN", // [588]
    "NOREPLICATION", // [589]
    "NOSUPERUSER", // [590]
    "PASSWORD", // [591]
    "REPLICATION", // [592]
    "SUPERUSER", // [593]
    "SuperuserKeywordSegment", // [594]
    "NosuperuserKeywordSegment", // [595]
    "CreatedbKeywordSegment", // [596]
    "NocreatedbKeywordSegment", // [597]
    "CreateroleKeywordSegment", // [598]
    "NocreateroleKeywordSegment", // [599]
    "NoinheritKeywordSegment", // [600]
    "LoginKeywordSegment", // [601]
    "NologinKeywordSegment", // [602]
    "ReplicationKeywordSegment", // [603]
    "NoreplicationKeywordSegment", // [604]
    "BypassrlsKeywordSegment", // [605]
    "NobypassrlsKeywordSegment", // [606]
    "EncryptedKeywordSegment", // [607]
    "PasswordKeywordSegment", // [608]
    "UntilKeywordSegment", // [609]
    "UNTIL", // [610]
    "OnKeywordAsIdentifierSegment", // [611]
    "IncrementKeywordSegment", // [612]
    "SignedSegmentGrammar", // [613]
    "MinvalueKeywordSegment", // [614]
    "MaxvalueKeywordSegment", // [615]
    "NameKeywordSegment", // [616]
    "NAME", // [617]
    "SequenceReferenceSegment", // [618]
    "sequence_reference", // [619]
    "StartKeywordSegment", // [620]
    "CacheKeywordSegment", // [621]
    "CycleKeywordSegment", // [622]
    "NoneKeywordSegment", // [623]
    "NONE", // [624]
    "StatisticsReferenceSegment", // [625]
    "statistics_reference", // [626]
    "SubscriptionKeywordSegment", // [627]
    "SUBSCRIPTION", // [628]
    "SubscriptionReferenceSegment", // [629]
    "subscription_reference", // [630]
    "REFRESH", // [631]
    "SKIP", // [632]
    "RefreshKeywordSegment", // [633]
    "SkipKeywordSegment", // [634]
    "RawEqualsSegment", // [635]
    "Current_sessionKeywordSegment", // [636]
    "SystemKeywordSegment", // [637]
    "SYSTEM", // [638]
    "ColumnDefinitionSegment", // [639]
    "column_definition", // [640]
    "FIRST", // [641]
    "FirstKeywordSegment", // [642]
    "AfterKeywordSegment", // [643]
    "AlterTableDropColumnGrammar", // [644]
    "AlterTableActionSegment", // [645]
    "DETACH", // [646]
    "ValuesKeywordSegment", // [647]
    "VALUES", // [648]
    "PartitionBoundSpecSegment", // [649]
    "partition_bound_spec", // [650]
    "DetachKeywordSegment", // [651]
    "ConcurrentlyKeywordSegment", // [652]
    "CONCURRENTLY", // [653]
    "FinalizeKeywordSegment", // [654]
    "FINALIZE", // [655]
    "MappingKeywordSegment", // [656]
    "MAPPING", // [657]
    "ReplaceKeywordSegment", // [658]
    "REPLACE", // [659]
    "TriggerReferenceSegment", // [660]
    "trigger_reference", // [661]
    "ValueKeywordSegment", // [662]
    "VALUE", // [663]
    "AttributeKeywordSegment", // [664]
    "ATTRIBUTE", // [665]
    "CascadeRestrictGrammar", // [666]
    "BEFORE", // [667]
    "BeforeKeywordSegment", // [668]
    "&", // [669]
    "ampersand", // [670]
    "ANALYSE", // [671]
    "ANALYZE", // [672]
    "AnalyzeKeywordSegment", // [673]
    "AnalyseKeywordSegment", // [674]
    "VERBOSE", // [675]
    "SKIP_LOCKED", // [676]
    "VerboseKeywordSegment", // [677]
    "Skip_lockedKeywordSegment", // [678]
    "AND", // [679]
    "binary_operator", // [680]
    "BinaryOperatorSegment", // [681]
    "ANTI", // [682]
    "ANY", // [683]
    "APPEND", // [684]
    "%", // [685]
    "/", // [686]
    "//", // [687]
    "<", // [688]
    ">", // [689]
    "^", // [690]
    "power_operator", // [691]
    "PlusSegment", // [692]
    "DivideSegment", // [693]
    "MultiplySegment", // [694]
    "ModuloSegment", // [695]
    "BitwiseAndSegment", // [696]
    "BitwiseOrSegment", // [697]
    "BitwiseXorSegment", // [698]
    "BitwiseLShiftSegment", // [699]
    "BitwiseRShiftSegment", // [700]
    "PowerOperatorSegment", // [701]
    "QualifiedNumericLiteralSegment", // [702]
    "SliceSegment", // [703]
    "start_square_bracket", // [704]
    "]", // [705]
    "end_square_bracket", // [706]
    "BaseExpressionElementGrammar", // [707]
    "ArrayKeywordSegment", // [708]
    "ASC", // [709]
    "ASOF", // [710]
    "ASSERTION", // [711]
    "ASSIGNMENT", // [712]
    "ASYMMETRIC", // [713]
    "AT", // [714]
    "ATOMIC", // [715]
    "AUDIT", // [716]
    "AUTHORIZATION", // [717]
    "AUTO_INCREMENT", // [718]
    "Auto_incrementKeywordSegment", // [719]
    "AVG_ROW_LENGTH", // [720]
    "BACKUP", // [721]
    "BACKWARD", // [722]
    "bare_function", // [723]
    "CodeSegment", // [724]
    "MapLiteralSegment", // [725]
    "MAP", // [726]
    "map_literal", // [727]
    "IntervalExpressionSegment", // [728]
    "interval_expression", // [729]
    "CompositeValueExpansionSegment", // [730]
    "composite_value_expansion", // [731]
    "ColumnIndexSegment", // [732]
    "column_index", // [733]
    "file", // [734]
    "BaseFileSegment", // [735]
    "BaseSegment", // [736]
    "BEGIN", // [737]
    "BERNOULLI", // [738]
    "BETWEEN", // [739]
    "BIGINT", // [740]
    "BIGSERIAL", // [741]
    "BINARY", // [742]
    "!", // [743]
    "==", // [744]
    "IS", // [745]
    "OR", // [746]
    "glob_operator", // [747]
    "json_operator", // [748]
    "like_operator", // [749]
    "ArithmeticBinaryOperatorGrammar", // [750]
    "StringBinaryOperatorGrammar", // [751]
    "BooleanBinaryOperatorGrammar", // [752]
    "ComparisonOperatorGrammar", // [753]
    "JsonOperatorSegment", // [754]
    "BINDING", // [755]
    "BIT", // [756]
    "BITVAR", // [757]
    "AmpersandSegment", // [758]
    "RawLessThanSegment", // [759]
    "RawGreaterThanSegment", // [760]
    "BOOL", // [761]
    "AndOperatorGrammar", // [762]
    "OrOperatorGrammar", // [763]
    "BOOLEAN", // [764]
    "TrueSegment", // [765]
    "FalseSegment", // [766]
    "BOTH", // [767]
    "BOX", // [768]
    "BPCHAR", // [769]
    "bracketed", // [770]
    "BracketedSegment", // [771]
    "SetExpressionSegment", // [772]
    "PIVOT", // [773]
    "PIVOT_LONGER", // [774]
    "PIVOT_WIDER", // [775]
    "UNPIVOT", // [776]
    "set_expression", // [777]
    "BREADTH", // [778]
    "BREAK", // [779]
    "BROWSE", // [780]
    "BUFFERS", // [781]
    "BULK", // [782]
    "BYTEA", // [783]
    "CTEColumnList", // [784]
    "cte_column_list", // [785]
    "SelectableGrammar", // [786]
    "DEPTH", // [787]
    "BreadthKeywordSegment", // [788]
    "DepthKeywordSegment", // [789]
    "CALL", // [790]
    "CallKeywordSegment", // [791]
    "CASCADED", // [792]
    "CASE", // [793]
    "CaseKeywordSegment", // [794]
    "implicit_indent", // [795]
    "ImplicitIndent", // [796]
    "WHEN", // [797]
    "WhenClauseSegment", // [798]
    "when_clause", // [799]
    "ElseKeywordSegment", // [800]
    "ELSE", // [801]
    "EndKeywordSegment", // [802]
    "END", // [803]
    "ElseClauseSegment", // [804]
    "else_clause", // [805]
    "BinaryOperatorGrammar", // [806]
    "::", // [807]
    "casting_operator", // [808]
    "CHAIN", // [809]
    "CHANGE", // [810]
    "CHAR", // [811]
    "CHARACTER", // [812]
    "CHARACTERISTICS", // [813]
    "CHECKED", // [814]
    "CHECKPOINT", // [815]
    "CHECKSUM", // [816]
    "CIDR", // [817]
    "CIRCLE", // [818]
    "CIRCULARSTRING", // [819]
    "CLOSE", // [820]
    "CLUSTERED", // [821]
    "COALESCE", // [822]
    "ObjectReferenceDelimiterGrammar", // [823]
    "ObjectReferenceTerminatorGrammar", // [824]
    ";", // [825]
    "colon_delimiter", // [826]
    "colon_prefix", // [827]
    "colon", // [828]
    "ShorthandCastSegment", // [829]
    "cast_expression", // [830]
    "UniqueKeywordSegment", // [831]
    "PrimaryKeywordSegment", // [832]
    "KeyKeywordSegment", // [833]
    "KEY", // [834]
    "ReferenceDefinitionGrammar", // [835]
    "DotSegment", // [836]
    "SingleIdentifierFullGrammar", // [837]
    "CastOperatorSegment", // [838]
    "StartSquareBracketSegment", // [839]
    "StartBracketSegment", // [840]
    "ColonSegment", // [841]
    "DelimiterGrammar", // [842]
    "JoinLikeClauseGrammar", // [843]
    "WildcardExpressionSegment", // [844]
    "wildcard_expression", // [845]
    "LambdaExpressionSegment", // [846]
    "lambda_function", // [847]
    "ColumnsExpressionNameGrammar", // [848]
    "COLUMNS", // [849]
    "star", // [850]
    "ColumnsExpressionFunctionNameSegment", // [851]
    "ColumnsExpressionFunctionContentsSegment", // [852]
    "function_contents", // [853]
    "columns_expression", // [854]
    "UnpackingOperatorSegment", // [855]
    "ColumnsKeywordSegment", // [856]
    "comma", // [857]
    "COMMENT", // [858]
    "CommentKeywordSegment", // [859]
    "IsKeywordSegment", // [860]
    "comment", // [861]
    "CommentSegment", // [862]
    "COMMENTS", // [863]
    "COMMIT", // [864]
    "COMMITTED", // [865]
    "COMMUTATOR", // [866]
    "GreaterThanSegment", // [867]
    "LessThanSegment", // [868]
    "GreaterThanOrEqualToSegment", // [869]
    "LessThanOrEqualToSegment", // [870]
    "NotEqualToSegment", // [871]
    "LikeOperatorSegment", // [872]
    "IsDistinctFromGrammar", // [873]
    "EqualsSegment_a", // [874]
    "GlobOperatorSegment", // [875]
    "ComparisonOperatorSegment", // [876]
    "COMPLETION", // [877]
    "CompositeBinaryOperatorSegment", // [878]
    "CompositeComparisonOperatorSegment", // [879]
    "COMPOUNDCURVE", // [880]
    "COMPRESS", // [881]
    "COMPRESSION_LEVEL", // [882]
    "COMPUTE", // [883]
    "JoinTypeKeywordsGrammar", // [884]
    "INNER", // [885]
    "LEFT", // [886]
    "RIGHT", // [887]
    "ConditionalCrossJoinKeywordsGrammar", // [888]
    "NonStandardJoinTypeKeywordsGrammar", // [889]
    "SEMI", // [890]
    "DO", // [891]
    "DoKeywordSegment", // [892]
    "WHERE", // [893]
    "WhereKeywordSegment", // [894]
    "CONFLICT", // [895]
    "OperationClassReferenceSegment", // [896]
    "operation_class_reference", // [897]
    "CONSTRAINTS", // [898]
    "CONTAINSTABLE", // [899]
    "CONTENT", // [900]
    "CONTINUE", // [901]
    "COPY", // [902]
    "CopyKeywordSegment", // [903]
    "OVERWRITE", // [904]
    "OVERWRITE_OR_IGNORE", // [905]
    "PARQUET_VERSION", // [906]
    "PARTITION_BY", // [907]
    "ROW_GROUP_SIZE", // [908]
    "ROW_GROUP_SIZE_BYTES", // [909]
    "WRITE_PARTITION_COLUMNS", // [910]
    "Overwrite_or_ignoreKeywordSegment", // [911]
    "OverwriteKeywordSegment", // [912]
    "AppendKeywordSegment", // [913]
    "Partition_byKeywordSegment", // [914]
    "Write_partition_columnsKeywordSegment", // [915]
    "Compression_levelKeywordSegment", // [916]
    "Row_group_size_bytesKeywordSegment", // [917]
    "Row_group_sizeKeywordSegment", // [918]
    "Parquet_versionKeywordSegment", // [919]
    "DELIMITER", // [920]
    "ENCODING", // [921]
    "ESCAPE", // [922]
    "FORCE_NOT_NULL", // [923]
    "FORCE_NULL", // [924]
    "FORCE_QUOTE", // [925]
    "FREEZE", // [926]
    "HEADER", // [927]
    "QUOTE", // [928]
    "FreezeKeywordSegment", // [929]
    "DelimiterKeywordSegment", // [930]
    "HeaderKeywordSegment", // [931]
    "QuoteKeywordSegment", // [932]
    "EscapeKeywordSegment", // [933]
    "Force_quoteKeywordSegment", // [934]
    "Force_not_nullKeywordSegment", // [935]
    "Force_nullKeywordSegment", // [936]
    "EncodingKeywordSegment", // [937]
    "CORRELATION", // [938]
    "COSTS", // [939]
    "OrReplaceGrammar", // [940]
    "InoutKeywordSegment", // [941]
    "INOUT", // [942]
    "AssignmentKeywordSegment", // [943]
    "ImplicitKeywordSegment", // [944]
    "IMPLICIT", // [945]
    "DETERMINISTIC", // [946]
    "LC_COLLATE", // [947]
    "LC_CTYPE", // [948]
    "LOCALE", // [949]
    "PROVIDER", // [950]
    "VERSION", // [951]
    "LocaleKeywordSegment", // [952]
    "Lc_collateKeywordSegment", // [953]
    "Lc_ctypeKeywordSegment", // [954]
    "ProviderKeywordSegment", // [955]
    "ICU", // [956]
    "LIBC", // [957]
    "IcuKeywordSegment", // [958]
    "LibcKeywordSegment", // [959]
    "DeterministicKeywordSegment", // [960]
    "VersionKeywordSegment", // [961]
    "VersionIdentifierSegment", // [962]
    "version_identifier", // [963]
    "ForeignDataWrapperGrammar", // [964]
    "HandlerKeywordSegment", // [965]
    "HANDLER", // [966]
    "ValidatorKeywordSegment", // [967]
    "VALIDATOR", // [968]
    "CreateForeignTableGrammar", // [969]
    "OptionsGrammar", // [970]
    "ForeignTableColumnConstraintSegment", // [971]
    "ForeignTableTableConstraintSegment", // [972]
    "INHERITS", // [973]
    "InheritsKeywordSegment", // [974]
    "ServerReferenceSegment", // [975]
    "server_reference", // [976]
    "TemporaryGrammar", // [977]
    "MACRO", // [978]
    "MacroKeywordSegment", // [979]
    "IndexElementSegment", // [980]
    "index_element", // [981]
    "INCLUDE", // [982]
    "IncludeKeywordSegment", // [983]
    "NULLS", // [984]
    "NullsKeywordSegment", // [985]
    "DistinctKeywordSegment", // [986]
    "DISTINCT", // [987]
    "ValuesClauseSegment", // [988]
    "values_clause", // [989]
    "WithDataClauseSegment", // [990]
    "with_data_clause", // [991]
    "ModelKeywordSegment", // [992]
    "MODEL", // [993]
    "commutator", // [994]
    "HASHES", // [995]
    "JOIN", // [996]
    "LEFTARG", // [997]
    "MERGES", // [998]
    "NEGATOR", // [999]
    "RIGHTARG", // [1000]
    "LeftargKeywordSegment", // [1001]
    "RightargKeywordSegment", // [1002]
    "CommutatorKeywordSegment", // [1003]
    "NegatorKeywordSegment", // [1004]
    "negator", // [1005]
    "JoinKeywordSegment", // [1006]
    "HashesKeywordSegment", // [1007]
    "MergesKeywordSegment", // [1008]
    "PERMISSIVE", // [1009]
    "RESTRICTIVE", // [1010]
    "PermissiveKeywordSegment", // [1011]
    "RestrictiveKeywordSegment", // [1012]
    "FunctionDefinitionGrammar", // [1013]
    "RETURN", // [1014]
    "WINDOW", // [1015]
    "function_definition", // [1016]
    "SYSID", // [1017]
    "AdminKeywordSegment", // [1018]
    "SysidKeywordSegment", // [1019]
    "AuthorizationKeywordSegment", // [1020]
    "CreateSequenceOptionsSegment", // [1021]
    "create_sequence_options_segment", // [1022]
    "DEPENDENCIES", // [1023]
    "MCV", // [1024]
    "NDISTINCT", // [1025]
    "DependenciesKeywordSegment", // [1026]
    "McvKeywordSegment", // [1027]
    "NdistinctKeywordSegment", // [1028]
    "CorrelationKeywordSegment", // [1029]
    "STORED", // [1030]
    "VIRTUAL", // [1031]
    "StoredKeywordSegment", // [1032]
    "VirtualKeywordSegment", // [1033]
    "INSTEAD", // [1034]
    "InsteadKeywordSegment", // [1035]
    "OrKeywordSegment", // [1036]
    "REFERENCING", // [1037]
    "ReferencingKeywordSegment", // [1038]
    "NEW", // [1039]
    "OLD", // [1040]
    "OldKeywordSegment", // [1041]
    "NewKeywordSegment", // [1042]
    "EachKeywordSegment", // [1043]
    "EACH", // [1044]
    "STATEMENT", // [1045]
    "StatementKeywordSegment", // [1046]
    "WhenKeywordSegment", // [1047]
    "ENUM", // [1048]
    "EnumKeywordSegment", // [1049]
    "StructTypeSegment", // [1050]
    "STRUCT", // [1051]
    "struct_type", // [1052]
    "UNION", // [1053]
    "UnionKeywordSegment", // [1054]
    "StructTypeSchemaSegment", // [1055]
    "struct_type_schema", // [1056]
    "CreateUserMappingGrammar", // [1057]
    "SessionInformationUserFunctionsGrammar", // [1058]
    "CREATEUSER", // [1059]
    "CROSS", // [1060]
    "CSV", // [1061]
    "CUBE", // [1062]
    "function_name_identifier", // [1063]
    "ROLLUP", // [1064]
    "CubeFunctionNameSegment", // [1065]
    "RollupFunctionNameSegment", // [1066]
    "GroupingExpressionList", // [1067]
    "grouping_expression_list", // [1068]
    "CURSOR", // [1069]
    "CURVEPOLYGON", // [1070]
    "DATABASES", // [1071]
    "data_type_identifier", // [1072]
    "WellKnownTextGeometrySegment", // [1073]
    "GEOGRAPHY", // [1074]
    "GEOMETRY", // [1075]
    "GEOMETRYCOLLECTION", // [1076]
    "GEOMETRYCOLLECTIONM", // [1077]
    "GEOMETRYCOLLECTIONZ", // [1078]
    "GEOMETRYCOLLECTIONZM", // [1079]
    "LINESTRING", // [1080]
    "LINESTRINGM", // [1081]
    "LINESTRINGZ", // [1082]
    "LINESTRINGZM", // [1083]
    "MULTICURVE", // [1084]
    "MULTILINESTRING", // [1085]
    "MULTILINESTRINGM", // [1086]
    "MULTILINESTRINGZ", // [1087]
    "MULTILINESTRINGZM", // [1088]
    "MULTIPOINT", // [1089]
    "MULTIPOINTM", // [1090]
    "MULTIPOINTZ", // [1091]
    "MULTIPOINTZM", // [1092]
    "MULTIPOLYGON", // [1093]
    "MULTIPOLYGONM", // [1094]
    "MULTIPOLYGONZ", // [1095]
    "MULTIPOLYGONZM", // [1096]
    "MULTISURFACE", // [1097]
    "POINT", // [1098]
    "POINTM", // [1099]
    "POINTZ", // [1100]
    "POINTZM", // [1101]
    "POLYGON", // [1102]
    "POLYGONM", // [1103]
    "POLYGONZ", // [1104]
    "POLYGONZM", // [1105]
    "POLYHEDRALSURFACE", // [1106]
    "TIN", // [1107]
    "TRIANGLE", // [1108]
    "wkt_geometry_type", // [1109]
    "DateTimeTypeIdentifier", // [1110]
    "datetime_type_identifier", // [1111]
    "MapTypeSegment", // [1112]
    "map_type", // [1113]
    "DATERANGE", // [1114]
    "DECIMAL", // [1115]
    "DOUBLE", // [1116]
    "FLOAT", // [1117]
    "FLOAT4", // [1118]
    "FLOAT8", // [1119]
    "INET", // [1120]
    "INT", // [1121]
    "INT2", // [1122]
    "INT4", // [1123]
    "INT4RANGE", // [1124]
    "INT8", // [1125]
    "INT8RANGE", // [1126]
    "INTEGER", // [1127]
    "JSON", // [1128]
    "JSONB", // [1129]
    "LINE", // [1130]
    "LSEG", // [1131]
    "MACADDR", // [1132]
    "MACADDR8", // [1133]
    "MONEY", // [1134]
    "NUMERIC", // [1135]
    "NUMRANGE", // [1136]
    "PATH", // [1137]
    "PG_LSN", // [1138]
    "REAL", // [1139]
    "SERIAL", // [1140]
    "SERIAL2", // [1141]
    "SERIAL4", // [1142]
    "SERIAL8", // [1143]
    "SMALLINT", // [1144]
    "SMALLSERIAL", // [1145]
    "TSQUERY", // [1146]
    "TSRANGE", // [1147]
    "TSTZRANGE", // [1148]
    "TSVECTOR", // [1149]
    "UUID", // [1150]
    "VARCHAR", // [1151]
    "VECTOR", // [1152]
    "XML", // [1153]
    "SmallintKeywordSegment", // [1154]
    "IntegerKeywordSegment", // [1155]
    "IntKeywordSegment", // [1156]
    "Int2KeywordSegment", // [1157]
    "Int4KeywordSegment", // [1158]
    "Int8KeywordSegment", // [1159]
    "BigintKeywordSegment", // [1160]
    "Float4KeywordSegment", // [1161]
    "Float8KeywordSegment", // [1162]
    "RealKeywordSegment", // [1163]
    "DoubleKeywordSegment", // [1164]
    "PrecisionKeywordSegment", // [1165]
    "PRECISION", // [1166]
    "SmallserialKeywordSegment", // [1167]
    "SerialKeywordSegment", // [1168]
    "Serial2KeywordSegment", // [1169]
    "Serial4KeywordSegment", // [1170]
    "Serial8KeywordSegment", // [1171]
    "BigserialKeywordSegment", // [1172]
    "FloatKeywordSegment", // [1173]
    "BracketedArguments", // [1174]
    "bracketed_arguments", // [1175]
    "DecimalKeywordSegment", // [1176]
    "NumericKeywordSegment", // [1177]
    "MoneyKeywordSegment", // [1178]
    "BpcharKeywordSegment", // [1179]
    "CharKeywordSegment", // [1180]
    "VaryingKeywordSegment", // [1181]
    "VARYING", // [1182]
    "CharacterKeywordSegment", // [1183]
    "VarcharKeywordSegment", // [1184]
    "ByteaKeywordSegment", // [1185]
    "BooleanKeywordSegment", // [1186]
    "BoolKeywordSegment", // [1187]
    "PointKeywordSegment", // [1188]
    "LineKeywordSegment", // [1189]
    "LsegKeywordSegment", // [1190]
    "BoxKeywordSegment", // [1191]
    "PathKeywordSegment", // [1192]
    "PolygonKeywordSegment", // [1193]
    "CircleKeywordSegment", // [1194]
    "CidrKeywordSegment", // [1195]
    "InetKeywordSegment", // [1196]
    "MacaddrKeywordSegment", // [1197]
    "Macaddr8KeywordSegment", // [1198]
    "TsvectorKeywordSegment", // [1199]
    "TsqueryKeywordSegment", // [1200]
    "BitKeywordSegment", // [1201]
    "UuidKeywordSegment", // [1202]
    "XmlKeywordSegment", // [1203]
    "JsonKeywordSegment", // [1204]
    "JsonbKeywordSegment", // [1205]
    "Int4rangeKeywordSegment", // [1206]
    "Int8rangeKeywordSegment", // [1207]
    "NumrangeKeywordSegment", // [1208]
    "TsrangeKeywordSegment", // [1209]
    "TstzrangeKeywordSegment", // [1210]
    "DaterangeKeywordSegment", // [1211]
    "Pg_lsnKeywordSegment", // [1212]
    "VectorKeywordSegment", // [1213]
    "DatatypeIdentifierSegment", // [1214]
    "ArrayTypeSuffixSegment", // [1215]
    "array_type_suffix", // [1216]
    "ArrayTypeSegment", // [1217]
    "array_type", // [1218]
    "SizedArrayTypeSegment", // [1219]
    "sized_array_type", // [1220]
    "DatePartFunctionName", // [1221]
    "DatetimeUnitSegment", // [1222]
    "CENTURY", // [1223]
    "DAY", // [1224]
    "DAYOFYEAR", // [1225]
    "DAYS", // [1226]
    "DECADE", // [1227]
    "DOW", // [1228]
    "DOY", // [1229]
    "EPOCH", // [1230]
    "HOUR", // [1231]
    "HOURS", // [1232]
    "ISODOW", // [1233]
    "ISOYEAR", // [1234]
    "MICROSECOND", // [1235]
    "MICROSECONDS", // [1236]
    "MILLENNIUM", // [1237]
    "MILLISECOND", // [1238]
    "MILLISECONDS", // [1239]
    "MINUTE", // [1240]
    "MINUTES", // [1241]
    "MONTH", // [1242]
    "MONTHS", // [1243]
    "QUARTER", // [1244]
    "QUARTERS", // [1245]
    "SECOND", // [1246]
    "SECONDS", // [1247]
    "TIMEZONE", // [1248]
    "TIMEZONE_HOUR", // [1249]
    "TIMEZONE_MINUTE", // [1250]
    "WEEK", // [1251]
    "WEEKDAY", // [1252]
    "WEEKS", // [1253]
    "YEAR", // [1254]
    "YEARS", // [1255]
    "FunctionContentsGrammar", // [1256]
    "DateKeywordSegment", // [1257]
    "TimeWithTZGrammar", // [1258]
    "IntervalKeywordSegment", // [1259]
    "TimetzKeywordSegment", // [1260]
    "TimestamptzKeywordSegment", // [1261]
    "DATETIME", // [1262]
    "date_part", // [1263]
    "DAY_HOUR", // [1264]
    "DAY_MICROSECOND", // [1265]
    "DAY_MINUTE", // [1266]
    "DAY_SECOND", // [1267]
    "DAYOFMONTH", // [1268]
    "DAYOFWEEK", // [1269]
    "DBCC", // [1270]
    "DEALLOCATE", // [1271]
    "DeallocateKeywordSegment", // [1272]
    "PrepareKeywordSegment", // [1273]
    "PREPARE", // [1274]
    "DEC", // [1275]
    "DECLARE", // [1276]
    "DEFAULTS", // [1277]
    "PropertiesNakedIdentifierSegment", // [1278]
    "DefinitionArgumentValueGrammar", // [1279]
    "DefinitionParameterSegment", // [1280]
    "definition_parameter", // [1281]
    "DELAY_KEY_WRITE", // [1282]
    "DELAYED", // [1283]
    "TableExpressionSegment", // [1284]
    "table_expression", // [1285]
    "JoinClauseSegment", // [1286]
    "join_clause", // [1287]
    "WhereClauseSegment", // [1288]
    "where_clause", // [1289]
    "RETURNING", // [1290]
    "ReturningKeywordSegment", // [1291]
    "SemicolonSegment", // [1292]
    "DELIMITERS", // [1293]
    "DENY", // [1294]
    "DESC", // [1295]
    "DESCRIBE", // [1296]
    "DescribeKeywordSegment", // [1297]
    "DESTROY", // [1298]
    "DESTRUCTOR", // [1299]
    "DISABLE_PAGE_SKIPPING", // [1300]
    "DISCARD", // [1301]
    "DiscardKeywordSegment", // [1302]
    "PLANS", // [1303]
    "PlansKeywordSegment", // [1304]
    "DISK", // [1305]
    "DISTINCTROW", // [1306]
    "DISTRIBUTED", // [1307]
    "DIV", // [1308]
    "LanguageClauseSegment", // [1309]
    "language_clause", // [1310]
    "DOCUMENT", // [1311]
    "literal", // [1312]
    "LiteralSegment", // [1313]
    "dot", // [1314]
    "DUMMY", // [1315]
    "DUMP", // [1316]
    "ELSEIF", // [1317]
    "EMPTY", // [1318]
    "EmptyStructLiteralBracketsSegment", // [1319]
    "struct_literal", // [1320]
    "ENCLOSED", // [1321]
    "}", // [1322]
    "end_curly_bracket", // [1323]
    "ERRLVL", // [1324]
    "ESCAPED", // [1325]
    "EXCEPT", // [1326]
    "EXCLUDING", // [1327]
    "EXCLUSIVE", // [1328]
    "EXISTING", // [1329]
    "EXISTS", // [1330]
    "EXIT", // [1331]
    "EXPLAIN", // [1332]
    "SETTINGS", // [1333]
    "SUMMARY", // [1334]
    "TIMING", // [1335]
    "WAL", // [1336]
    "CostsKeywordSegment", // [1337]
    "SettingsKeywordSegment", // [1338]
    "BuffersKeywordSegment", // [1339]
    "WalKeywordSegment", // [1340]
    "TimingKeywordSegment", // [1341]
    "SummaryKeywordSegment", // [1342]
    "YAML", // [1343]
    "YamlKeywordSegment", // [1344]
    "ExplainKeywordSegment", // [1345]
    "ExplainOptionSegment", // [1346]
    "explain_option", // [1347]
    "InsertStatementSegment", // [1348]
    "insert_statement", // [1349]
    "UpdateStatementSegment", // [1350]
    "update_statement", // [1351]
    "DeleteStatementSegment", // [1352]
    "delete_statement", // [1353]
    "Expression_A_Grammar", // [1354]
    "Tail_Recurse_Expression_A_Grammar", // [1355]
    "LikeExpressionGrammar", // [1356]
    "GLOB", // [1357]
    "ILIKE", // [1358]
    "LIKE", // [1359]
    "SIMILAR", // [1360]
    "InOperatorGrammar", // [1361]
    "QuantifiedComparisonOperatorGrammar", // [1362]
    "IsClauseGrammar", // [1363]
    "NAN", // [1364]
    "NFC", // [1365]
    "NFD", // [1366]
    "NFKC", // [1367]
    "NFKD", // [1368]
    "NORMALIZED", // [1369]
    "UNKNOWN", // [1370]
    "IsNullGrammar", // [1371]
    "ISNULL", // [1372]
    "NotNullGrammar", // [1373]
    "NOTNULL", // [1374]
    "CollateGrammar", // [1375]
    "BetweenKeywordSegment", // [1376]
    "Expression_B_Grammar", // [1377]
    "AndKeywordSegment", // [1378]
    "PatternMatchingGrammar", // [1379]
    "PRIOR", // [1380]
    "~", // [1381]
    "TildeSegment", // [1382]
    "NotOperatorGrammar", // [1383]
    "PriorKeywordSegment", // [1384]
    "AbsoluteValueOperatorSegment", // [1385]
    "Tail_Recurse_Expression_B_Grammar", // [1386]
    "WalrusOperatorSegment", // [1387]
    ":=", // [1388]
    "ExistsKeywordSegment", // [1389]
    "Expression_D_Grammar", // [1390]
    "CaseExpressionSegment", // [1391]
    "case_expression", // [1392]
    "TimeZoneGrammar", // [1393]
    "time_zone_grammar", // [1394]
    "FullTextSearchOperatorSegment", // [1395]
    "full_text_search_operator", // [1396]
    "FactorialOperatorSegment", // [1397]
    "not", // [1398]
    "LateralColumnAliasExpressionGrammar", // [1399]
    "LocalAliasSegment", // [1400]
    "local_alias_segment", // [1401]
    "Expression_D_Potential_Select_Statement_Without_Brackets", // [1402]
    "SingleIdentifierWildcardSegment", // [1403]
    "NullLiteralSegment", // [1404]
    "DateTimeLiteralGrammar", // [1405]
    "datetime_literal", // [1406]
    "ListComprehensionGrammar", // [1407]
    "AccessorGrammar", // [1408]
    "SelectStatementSegment", // [1409]
    "select_statement", // [1410]
    "TypedStructLiteralSegment", // [1411]
    "typed_struct_literal", // [1412]
    "ArrayExpressionSegment", // [1413]
    "array_expression", // [1414]
    "OverlapsClauseSegment", // [1415]
    "overlaps_clause", // [1416]
    "EXTRACT", // [1417]
    "factorial_operator", // [1418]
    "boolean_literal", // [1419]
    "LiteralKeywordSegment", // [1420]
    "FETCH", // [1421]
    "FetchKeywordSegment", // [1422]
    "NEXT", // [1423]
    "NextKeywordSegment", // [1424]
    "TiesKeywordSegment", // [1425]
    "TIES", // [1426]
    "FIELDS", // [1427]
    "PostgresCopyStdinDataStatementSegment", // [1428]
    "postgres_copy_stdin_data_block", // [1429]
    "postgres_copy_stdin_data_statement", // [1430]
    "PsqlCopyMetaCommandStatementSegment", // [1431]
    "psql_copy_command", // [1432]
    "psql_copy_meta_command_statement", // [1433]
    "PsqlSetMetaCommandStatementSegment", // [1434]
    "psql_set_command", // [1435]
    "psql_set_meta_command_statement", // [1436]
    "StatementSegment", // [1437]
    "statement", // [1438]
    "FILLFACTOR", // [1439]
    "FILTER", // [1440]
    "FilterKeywordSegment", // [1441]
    "FLUSH", // [1442]
    "FOLLOWING", // [1443]
    "LockedKeywordSegment", // [1444]
    "LOCKED", // [1445]
    "FORWARD", // [1446]
    "GROUPS", // [1447]
    "RANGE", // [1448]
    "FrameClauseUnitGrammar", // [1449]
    "UnboundedKeywordSegment", // [1450]
    "UNBOUNDED", // [1451]
    "PRECEDING", // [1452]
    "PrecedingKeywordSegment", // [1453]
    "FollowingKeywordSegment", // [1454]
    "ExcludeKeywordSegment", // [1455]
    "OthersKeywordSegment", // [1456]
    "OTHERS", // [1457]
    "RangeKeywordSegment", // [1458]
    "GroupsKeywordSegment", // [1459]
    "FREETEXT", // [1460]
    "FREETEXTTABLE", // [1461]
    "FromExpressionSegment", // [1462]
    "from_expression", // [1463]
    "HAVING", // [1464]
    "INTERSECT", // [1465]
    "MINUS", // [1466]
    "OFFSET", // [1467]
    "QUALIFY", // [1468]
    "LimitClauseSegment", // [1469]
    "limit_clause", // [1470]
    "OrderKeywordSegment", // [1471]
    "HavingKeywordSegment", // [1472]
    "QualifyKeywordSegment", // [1473]
    "WindowKeywordSegment", // [1474]
    "SetOperatorSegment", // [1475]
    "set_operator", // [1476]
    "WithNoSchemaBindingClauseSegment", // [1477]
    "with_no_schema_binding_clause", // [1478]
    "OffsetKeywordSegment", // [1479]
    "ForClauseSegment", // [1480]
    "for_clause", // [1481]
    "PreTableFunctionKeywordsGrammar", // [1482]
    "LATERAL", // [1483]
    "TemporalQuerySegment", // [1484]
    "temporal_query", // [1485]
    "FromClauseTerminatorGrammar", // [1486]
    "SamplingExpressionSegment", // [1487]
    "TABLESAMPLE", // [1488]
    "sample_expression", // [1489]
    "PostTableExpressionGrammar", // [1490]
    "MLTableExpressionSegment", // [1491]
    "ML", // [1492]
    "ml_table_expression", // [1493]
    "FromExpressionElementSegment", // [1494]
    "from_expression_element", // [1495]
    "indented_joins", // [1496]
    "PivotKeywordSegment", // [1497]
    "GroupByClauseSegment", // [1498]
    "groupby_clause", // [1499]
    "UnpivotKeywordSegment", // [1500]
    "ColumnsExpressionGrammar", // [1501]
    "FULLTEXT", // [1502]
    "NamedArgumentSegment", // [1503]
    "named_argument", // [1504]
    "LEADING", // [1505]
    "TRAILING", // [1506]
    "TrimParametersGrammar", // [1507]
    "FunctionContentsExpressionGrammar", // [1508]
    "AggregateOrderByClause", // [1509]
    "aggregate_order_by", // [1510]
    "SEPARATOR", // [1511]
    "SeparatorKeywordSegment", // [1512]
    "IgnoreRespectNullsGrammar", // [1513]
    "IGNORE", // [1514]
    "RESPECT", // [1515]
    "IndexColumnDefinitionSegment", // [1516]
    "index_column_definition", // [1517]
    "EmptyStructLiteralSegment", // [1518]
    "ReturnKeywordSegment", // [1519]
    "BeginKeywordSegment", // [1520]
    "AtomicKeywordSegment", // [1521]
    "WithCompoundStatementSegment", // [1522]
    "with_compound_statement", // [1523]
    "FunctionNameIdentifierSegment", // [1524]
    "OUT", // [1525]
    "OutKeywordSegment", // [1526]
    "VariadicKeywordSegment", // [1527]
    "ColumnTypeReferenceSegment", // [1528]
    "column_type_reference", // [1529]
    "FunctionParameterGrammar", // [1530]
    "DatePartFunctionNameSegment", // [1531]
    "DateTimeFunctionContentsSegment", // [1532]
    "FunctionContentsSegment", // [1533]
    "PostFunctionGrammar", // [1534]
    "GLOBAL", // [1535]
    "AccessPermissionsSegment", // [1536]
    "access_permissions", // [1537]
    "AccessObjectSegment", // [1538]
    "access_object", // [1539]
    "AccessTargetSegment", // [1540]
    "access_target", // [1541]
    "GRANTED", // [1542]
    "GrantedKeywordSegment", // [1543]
    "GREATEST", // [1544]
    "GroupByClauseTerminatorGrammar", // [1545]
    "GROUPING", // [1546]
    "GroupingKeywordSegment", // [1547]
    "SetsKeywordSegment", // [1548]
    "SETS", // [1549]
    "CubeRollupClauseSegment", // [1550]
    "cube_rollup_clause", // [1551]
    "HASH", // [1552]
    "HEAP", // [1553]
    "HIGH_PRIORITY", // [1554]
    "HOLD", // [1555]
    "HOLDLOCK", // [1556]
    "PositionalKeywordSegment", // [1557]
    "POSITIONAL", // [1558]
    "HOST", // [1559]
    "HOSTS", // [1560]
    "HOUR_MICROSECOND", // [1561]
    "HOUR_MINUTE", // [1562]
    "HOUR_SECOND", // [1563]
    "IDENTIFIED", // [1564]
    "identifier", // [1565]
    "IdentifierSegment", // [1566]
    "IDENTITY_INSERT", // [1567]
    "IDENTITYCOL", // [1568]
    "IfKeywordSegment", // [1569]
    "IgnoreKeywordSegment", // [1570]
    "RespectKeywordSegment", // [1571]
    "IMPORT", // [1572]
    "ImportKeywordSegment", // [1573]
    "ImportForeignSchemaGrammar", // [1574]
    "ExceptKeywordSegment", // [1575]
    "NakedIdentifierFullSegment", // [1576]
    "IntoKeywordSegment", // [1577]
    "INTO", // [1578]
    "ArrayLiteralSegment", // [1579]
    "array_literal", // [1580]
    "INCLUDING", // [1581]
    "AscKeywordSegment", // [1582]
    "DescKeywordSegment", // [1583]
    "OperatorClassReferenceSegment", // [1584]
    "LAST", // [1585]
    "LastKeywordSegment", // [1586]
    "operator_class_reference", // [1587]
    "IndexElementOptionsSegment", // [1588]
    "index_element_options", // [1589]
    "INDEX_CLEANUP", // [1590]
    "INDEXES", // [1591]
    "INFILE", // [1592]
    "INFIX", // [1593]
    "INITIALIZE", // [1594]
    "INLINE", // [1595]
    "INSENSITIVE", // [1596]
    "OrIgnoreGrammar", // [1597]
    "AsAliasExpressionSegment", // [1598]
    "PositionKeywordSegment", // [1599]
    "POSITION", // [1600]
    "ConflictKeywordSegment", // [1601]
    "ConflictTargetSegment", // [1602]
    "conflict_target", // [1603]
    "ConflictActionSegment", // [1604]
    "conflict_action", // [1605]
    "INSERT_ID", // [1606]
    "INT1", // [1607]
    "INT3", // [1608]
    "INTEGRATIONS", // [1609]
    "YearKeywordSegment", // [1610]
    "MonthKeywordSegment", // [1611]
    "DayKeywordSegment", // [1612]
    "HourKeywordSegment", // [1613]
    "MinuteKeywordSegment", // [1614]
    "SecondKeywordSegment", // [1615]
    "NanLiteralSegment", // [1616]
    "UnknownLiteralSegment", // [1617]
    "NormalizedGrammar", // [1618]
    "JsonTestGrammar", // [1619]
    "IsnullKeywordSegment", // [1620]
    "IS_SUPERUSER", // [1621]
    "ISAM", // [1622]
    "ISOLATION", // [1623]
    "ITERATE", // [1624]
    "ConditionalJoinKeywordsGrammar", // [1625]
    "JoinKeywordsGrammar", // [1626]
    "NestedJoinGrammar", // [1627]
    "indented_using_on", // [1628]
    "MatchConditionSegment", // [1629]
    "match_condition", // [1630]
    "JoinOnConditionSegment", // [1631]
    "join_on_condition", // [1632]
    "JoinUsingConditionGrammar", // [1633]
    "NATURAL", // [1634]
    "UnconditionalJoinKeywordsGrammar", // [1635]
    "ExtendedNaturalJoinKeywordsGrammar", // [1636]
    "FromPivotExpressionSegment", // [1637]
    "from_pivot_expression", // [1638]
    "FromUnpivotExpressionSegment", // [1639]
    "from_unpivot_expression", // [1640]
    "indented_on_contents", // [1641]
    "InnerKeywordSegment", // [1642]
    "LeftKeywordSegment", // [1643]
    "RightKeywordSegment", // [1644]
    "OuterKeywordSegment", // [1645]
    "OUTER", // [1646]
    "JsonTypeGrammar", // [1647]
    "SCALAR", // [1648]
    "JsonUniqueKeysGrammar", // [1649]
    "ScalarKeywordSegment", // [1650]
    "KEYS", // [1651]
    "KeysKeywordSegment", // [1652]
    "KILL", // [1653]
    "LABEL", // [1654]
    "->", // [1655]
    "lambda_arrow", // [1656]
    "LambdaArrowSegment", // [1657]
    "LAMBDA", // [1658]
    "LambdaKeywordSegment", // [1659]
    "LANCOMPILER", // [1660]
    "LAST_INSERT_ID", // [1661]
    "LEAST", // [1662]
    "LEAVE", // [1663]
    "LESS", // [1664]
    "LikeGrammar", // [1665]
    "LikeKeywordSegment", // [1666]
    "IlikeKeywordSegment", // [1667]
    "SimilarKeywordSegment", // [1668]
    "GlobKeywordSegment", // [1669]
    "IncludingKeywordSegment", // [1670]
    "ExcludingKeywordSegment", // [1671]
    "CommentsKeywordSegment", // [1672]
    "ConstraintsKeywordSegment", // [1673]
    "DefaultsKeywordSegment", // [1674]
    "IndexesKeywordSegment", // [1675]
    "LINENO", // [1676]
    "LINES", // [1677]
    "ListComprehensionExpressionSegment", // [1678]
    "list_comprehension", // [1679]
    "LIST", // [1680]
    "LISTEN", // [1681]
    "ListenKeywordSegment", // [1682]
    "DollarNumericLiteralSegment", // [1683]
    "PsqlVariableGrammar", // [1684]
    "psql_variable", // [1685]
    "TypedArrayLiteralSegment", // [1686]
    "typed_array_literal", // [1687]
    "ObjectLiteralSegment", // [1688]
    "object_literal", // [1689]
    "LOAD", // [1690]
    "LoadKeywordSegment", // [1691]
    "LOCAL", // [1692]
    "LOCATION", // [1693]
    "LOCK", // [1694]
    "LockKeywordSegment", // [1695]
    "ExclusiveKeywordSegment", // [1696]
    "ModeKeywordSegment", // [1697]
    "MODE", // [1698]
    "LOCKS", // [1699]
    "LOGS", // [1700]
    "LONG", // [1701]
    "LONGBLOB", // [1702]
    "LONGTEXT", // [1703]
    "LOOP", // [1704]
    "LOW_PRIORITY", // [1705]
    "MlKeywordSegment", // [1706]
    "MapKeywordSegment", // [1707]
    "MapLiteralElementSegment", // [1708]
    "map_literal_element", // [1709]
    "start_curly_bracket", // [1710]
    "MapTypeSchemaSegment", // [1711]
    "map_type_schema", // [1712]
    "MATCH", // [1713]
    "MATCHED", // [1714]
    "MAX_ROWS", // [1715]
    "MAXEXTENTS", // [1716]
    "MEDIUMBLOB", // [1717]
    "MEDIUMINT", // [1718]
    "MEDIUMTEXT", // [1719]
    "MERGE", // [1720]
    "MergeKeywordSegment", // [1721]
    "MergeMatchedClauseSegment", // [1722]
    "merge_when_matched_clause", // [1723]
    "MergeNotMatchedClauseSegment", // [1724]
    "merge_when_not_matched_clause", // [1725]
    "MergeNotMatchedBySourceClauseSegment", // [1726]
    "merge_when_not_matched_by_source_clause", // [1727]
    "MatchedKeywordSegment", // [1728]
    "ThenKeywordSegment", // [1729]
    "THEN", // [1730]
    "MergeUpdateClauseSegment", // [1731]
    "merge_update_clause", // [1732]
    "MergeDeleteClauseSegment", // [1733]
    "merge_delete_clause", // [1734]
    "SourceKeywordSegment", // [1735]
    "SOURCE", // [1736]
    "TargetKeywordSegment", // [1737]
    "TARGET", // [1738]
    "MergeInsertClauseSegment", // [1739]
    "merge_insert_clause", // [1740]
    "MergeIntoLiteralGrammar", // [1741]
    "AliasedTableReferenceGrammar", // [1742]
    "MergeMatchSegment", // [1743]
    "merge_match", // [1744]
    "SetClauseListSegment", // [1745]
    "set_clause_list", // [1746]
    "meta_command_query_buffer", // [1747]
    "meta_command", // [1748]
    "MetaCommandQueryBufferSegment", // [1749]
    "MIDDLEINT", // [1750]
    "MIN_ROWS", // [1751]
    "MINUTE_MICROSECOND", // [1752]
    "MINUTE_SECOND", // [1753]
    "MLSLABEL", // [1754]
    "MODULUS", // [1755]
    "MONTHNAME", // [1756]
    "MOVE", // [1757]
    "MultilineConcatenateNewline", // [1758]
    "NewlineSegment", // [1759]
    "MYISAM", // [1760]
    "naked_identifier_all", // [1761]
    "naked_identifier", // [1762]
    "=>", // [1763]
    "RightArrowSegment", // [1764]
    "WindowSpecificationSegment", // [1765]
    "window_specification", // [1766]
    "NamedWindowExpressionSegment", // [1767]
    "named_window_expression", // [1768]
    "NAMES", // [1769]
    "null_literal", // [1770]
    "NATIONAL", // [1771]
    "NaturalKeywordSegment", // [1772]
    "NCHAR", // [1773]
    "NO_WRITE_TO_BINLOG", // [1774]
    "NOAUDIT", // [1775]
    "NOCACHE", // [1776]
    "NOCHECK", // [1777]
    "NOCOMPRESS", // [1778]
    "NOCREATEUSER", // [1779]
    "NOCYCLE", // [1780]
    "UnorderedSelectStatementSegment", // [1781]
    "NonSetSelectableGrammar", // [1782]
    "BracketedSetExpressionGrammar", // [1783]
    "SimplifiedPivotExpressionSegment", // [1784]
    "simplified_pivot", // [1785]
    "SimplifiedUnpivotExpressionSegment", // [1786]
    "simplified_unpivot", // [1787]
    "AntiKeywordSegment", // [1788]
    "SemiKeywordSegment", // [1789]
    "AsofKeywordSegment", // [1790]
    "NONCLUSTERED", // [1791]
    "NOORDER", // [1792]
    "NORMALIZE", // [1793]
    "NfcKeywordSegment", // [1794]
    "NfdKeywordSegment", // [1795]
    "NfkcKeywordSegment", // [1796]
    "NfkdKeywordSegment", // [1797]
    "NormalizedKeywordSegment", // [1798]
    "RawNotSegment", // [1799]
    "NotnullKeywordSegment", // [1800]
    "NOTIFY", // [1801]
    "NotifyKeywordSegment", // [1802]
    "NULLIF", // [1803]
    "ObjectLiteralElementSegment", // [1804]
    "object_literal_element", // [1805]
    "OBJECTS", // [1806]
    "OFF", // [1807]
    "OFFLINE", // [1808]
    "OFFSETS", // [1809]
    "ONLINE", // [1810]
    "OPENDATASOURCE", // [1811]
    "OPENQUERY", // [1812]
    "OPENROWSET", // [1813]
    "OPENXML", // [1814]
    "OPERATION", // [1815]
    "OPTIMIZE", // [1816]
    "OPTIONALLY", // [1817]
    "OptionsListGrammar", // [1818]
    "OrderByClauseTerminators", // [1819]
    "NoorderKeywordSegment", // [1820]
    "ORDINALITY", // [1821]
    "OUTFILE", // [1822]
    "OverKeywordSegment", // [1823]
    "OVER", // [1824]
    "OverlapsKeywordSegment", // [1825]
    "OVERLAPS", // [1826]
    "OVERLAY", // [1827]
    "OVERRIDING", // [1828]
    "PACK_KEYS", // [1829]
    "parameter", // [1830]
    "?", // [1831]
    "PARAMETERS", // [1832]
    "PARTIAL", // [1833]
    "ModulusKeywordSegment", // [1834]
    "RemainderKeywordSegment", // [1835]
    "REMAINDER", // [1836]
    "PASSING", // [1837]
    "SlashSegment", // [1838]
    "path_segment", // [1839]
    "WordSegment", // [1840]
    "PCTFREE", // [1841]
    "PERCENT", // [1842]
    "pg_trgm_operator", // [1843]
    "pgvector_operator", // [1844]
    "pipe", // [1845]
    "PLACING", // [1846]
    "WithinGroupClauseSegment", // [1847]
    "WITHIN", // [1848]
    "withingroup_clause", // [1849]
    "OverClauseSegment", // [1850]
    "over_clause", // [1851]
    "FilterClauseGrammar", // [1852]
    "POSTFIX", // [1853]
    "postgis_operator", // [1854]
    "PostgresCopyStdinDataSegment", // [1855]
    "LateralKeywordSegment", // [1856]
    "PREFIX", // [1857]
    "PREORDER", // [1858]
    "MergeStatementSegment", // [1859]
    "merge_statement", // [1860]
    "PREPARED", // [1861]
    "PRESERVE", // [1862]
    "PRINT", // [1863]
    "PROC", // [1864]
    "PROCESS", // [1865]
    "PROCESS_TOAST", // [1866]
    "PROCESSLIST", // [1867]
    "PROGRAM", // [1868]
    "properties_naked_identifier", // [1869]
    "PsqlCopyMetaCommandSegment", // [1870]
    "PsqlSetMetaCommandSegment", // [1871]
    "psql_variable_colon", // [1872]
    "PsqlVariableColonSegment", // [1873]
    "PublicationTableSegment", // [1874]
    "publication_table", // [1875]
    "ExtendedTableReferenceGrammar", // [1876]
    "PURGE", // [1877]
    "operator", // [1878]
    "SOME", // [1879]
    "AnyKeywordSegment", // [1880]
    "SomeKeywordSegment", // [1881]
    "quoted_identifier", // [1882]
    "quoted_literal", // [1883]
    "MultilineConcatenateDelimiterGrammar", // [1884]
    "UESCAPE", // [1885]
    "UescapeKeywordSegment", // [1886]
    "unicode_escape_value", // [1887]
    "RAID0", // [1888]
    "RAISERROR", // [1889]
    "raw_comparison_operator", // [1890]
    "RAW", // [1891]
    "READTEXT", // [1892]
    "REASSIGN", // [1893]
    "ReassignKeywordSegment", // [1894]
    "RECHECK", // [1895]
    "RECONFIGURE", // [1896]
    "RECURSIVE", // [1897]
    "REF", // [1898]
    "ReferenceMatchGrammar", // [1899]
    "ReferentialActionGrammar", // [1900]
    "MatchKeywordSegment", // [1901]
    "SIMPLE", // [1902]
    "PartialKeywordSegment", // [1903]
    "SimpleKeywordSegment", // [1904]
    "ActionKeywordSegment", // [1905]
    "REGEXP", // [1906]
    "REINDEX", // [1907]
    "ReindexKeywordSegment", // [1908]
    "RelationOptionSegment", // [1909]
    "relation_option", // [1910]
    "RELATIVE", // [1911]
    "RELEASE", // [1912]
    "RELOAD", // [1913]
    "REPEAT", // [1914]
    "REPEATABLE", // [1915]
    "REQUIRE", // [1916]
    "RESERVOIR", // [1917]
    "SessionKeywordSegment", // [1918]
    "SESSION", // [1919]
    "RESIGNAL", // [1920]
    "RETRIEVE", // [1921]
    "right_arrow", // [1922]
    "RLIKE", // [1923]
    "ROLES", // [1924]
    "ROLLBACK", // [1925]
    "ROWCOUNT", // [1926]
    "ROWGUIDCOL", // [1927]
    "ROWID", // [1928]
    "ROWNUM", // [1929]
    "ReservoirKeywordSegment", // [1930]
    "SamplePercentageGrammar", // [1931]
    "SampleRowCountGrammar", // [1932]
    "BernoulliKeywordSegment", // [1933]
    "RepeatableKeywordSegment", // [1934]
    "SAMPLE", // [1935]
    "PercentKeywordSegment", // [1936]
    "TablesampleKeywordSegment", // [1937]
    "SampleExpressionGrammar", // [1938]
    "SAVE", // [1939]
    "SAVEPOINT", // [1940]
    "SCROLL", // [1941]
    "SECOND_MICROSECOND", // [1942]
    "LabelKeywordSegment", // [1943]
    "SelectClauseModifierSegment", // [1944]
    "select_clause_modifier", // [1945]
    "SelectClauseElementSegment", // [1946]
    "select_clause_element", // [1947]
    "WithCheckOptionSegment", // [1948]
    "with_check_option", // [1949]
    "SelectClauseSegment", // [1950]
    "select_clause", // [1951]
    "FromClauseSegment", // [1952]
    "from_clause", // [1953]
    "HavingClauseSegment", // [1954]
    "having_clause", // [1955]
    "NamedWindowSegment", // [1956]
    "named_window", // [1957]
    "QualifyClauseSegment", // [1958]
    "qualify_clause", // [1959]
    "OffsetClauseSegment", // [1960]
    "offset_clause", // [1961]
    "FetchClauseSegment", // [1962]
    "fetch_clause", // [1963]
    "UsingSampleClauseSegment", // [1964]
    "WithCompoundNonSelectStatementSegment", // [1965]
    "NonWithSelectableGrammar", // [1966]
    "statement_terminator", // [1967]
    "SERIALIZABLE", // [1968]
    "SERVER_ENCODING", // [1969]
    "SERVER_VERSION", // [1970]
    "SetClauseSegment", // [1971]
    "set_clause", // [1972]
    "UnionGrammar", // [1973]
    "IntersectKeywordSegment", // [1974]
    "MinusKeywordSegment", // [1975]
    "LocalKeywordSegment", // [1976]
    "VARIABLE", // [1977]
    "VariableKeywordSegment", // [1978]
    "GlobalKeywordSegment", // [1979]
    "SETOF", // [1980]
    "SETUSER", // [1981]
    "SHARES", // [1982]
    "SHOW", // [1983]
    "ShowKeywordSegment", // [1984]
    "Is_superuserKeywordSegment", // [1985]
    "Server_encodingKeywordSegment", // [1986]
    "Server_versionKeywordSegment", // [1987]
    "SHUTDOWN", // [1988]
    "SIGNAL", // [1989]
    "PositiveSegment", // [1990]
    "NegativeSegment", // [1991]
    "Pivot_widerKeywordSegment", // [1992]
    "Pivot_longerKeywordSegment", // [1993]
    "slash", // [1994]
    "slice", // [1995]
    "SNAPSHOT", // [1996]
    "SONAME", // [1997]
    "SPATIAL", // [1998]
    "SQL", // [1999]
    "SQL_BIG_RESULT", // [2000]
    "SQL_BIG_SELECTS", // [2001]
    "SQL_BIG_TABLES", // [2002]
    "SQL_CALC_FOUND_ROWS", // [2003]
    "SQL_LOG_OFF", // [2004]
    "SQL_LOG_UPDATE", // [2005]
    "SQL_LOW_PRIORITY_UPDATES", // [2006]
    "SQL_SELECT_LIMIT", // [2007]
    "SQL_SMALL_RESULT", // [2008]
    "SQL_WARNINGS", // [2009]
    "SQLCA", // [2010]
    "SSL", // [2011]
    "STANDALONE", // [2012]
    "STARTING", // [2013]
    "STARTS", // [2014]
    "TransactionStatementSegment", // [2015]
    "transaction_statement", // [2016]
    "DropTableStatementSegment", // [2017]
    "drop_table_statement", // [2018]
    "DropViewStatementSegment", // [2019]
    "drop_view_statement", // [2020]
    "CreateUserStatementSegment", // [2021]
    "create_user_statement", // [2022]
    "DropUserStatementSegment", // [2023]
    "drop_user_statement", // [2024]
    "TruncateStatementSegment", // [2025]
    "truncate_table", // [2026]
    "AccessStatementSegment", // [2027]
    "access_statement", // [2028]
    "CreateTableStatementSegment", // [2029]
    "create_table_statement", // [2030]
    "CreateRoleStatementSegment", // [2031]
    "create_role_statement", // [2032]
    "DropRoleStatementSegment", // [2033]
    "drop_role_statement", // [2034]
    "AlterTableStatementSegment", // [2035]
    "alter_table_statement", // [2036]
    "CreateSchemaStatementSegment", // [2037]
    "create_schema_statement", // [2038]
    "SetSchemaStatementSegment", // [2039]
    "set_schema_statement", // [2040]
    "DropSchemaStatementSegment", // [2041]
    "drop_schema_statement", // [2042]
    "DropTypeStatementSegment", // [2043]
    "drop_type_statement", // [2044]
    "CreateDatabaseStatementSegment", // [2045]
    "create_database_statement", // [2046]
    "DropDatabaseStatementSegment", // [2047]
    "drop_database_statement", // [2048]
    "CreateIndexStatementSegment", // [2049]
    "create_index_statement", // [2050]
    "DropIndexStatementSegment", // [2051]
    "drop_index_statement", // [2052]
    "CreateViewStatementSegment", // [2053]
    "create_view_statement", // [2054]
    "CreateCastStatementSegment", // [2055]
    "create_cast_statement", // [2056]
    "DropCastStatementSegment", // [2057]
    "drop_cast_statement", // [2058]
    "CreateFunctionStatementSegment", // [2059]
    "create_function_statement", // [2060]
    "DropFunctionStatementSegment", // [2061]
    "drop_function_statement", // [2062]
    "CreateModelStatementSegment", // [2063]
    "create_model_statement", // [2064]
    "DropModelStatementSegment", // [2065]
    "drop_MODELstatement", // [2066]
    "DescribeStatementSegment", // [2067]
    "describe_statement", // [2068]
    "UseStatementSegment", // [2069]
    "use_statement", // [2070]
    "ExplainStatementSegment", // [2071]
    "explain_statement", // [2072]
    "CreateSequenceStatementSegment", // [2073]
    "create_sequence_statement", // [2074]
    "AlterSequenceStatementSegment", // [2075]
    "alter_sequence_statement", // [2076]
    "DropSequenceStatementSegment", // [2077]
    "drop_sequence_statement", // [2078]
    "CreateTriggerStatementSegment", // [2079]
    "create_trigger", // [2080]
    "DropTriggerStatementSegment", // [2081]
    "drop_trigger", // [2082]
    "AlterDefaultPrivilegesStatementSegment", // [2083]
    "alter_default_privileges_statement", // [2084]
    "DropOwnedStatementSegment", // [2085]
    "drop_owned_statement", // [2086]
    "ReassignOwnedStatementSegment", // [2087]
    "reassign_owned_statement", // [2088]
    "CommentOnStatementSegment", // [2089]
    "comment_clause", // [2090]
    "AnalyzeStatementSegment", // [2091]
    "analyze_statement", // [2092]
    "CreateTableAsStatementSegment", // [2093]
    "AlterTriggerStatementSegment", // [2094]
    "alter_trigger", // [2095]
    "SetStatementSegment", // [2096]
    "set_statement", // [2097]
    "AlterSystemStatementSegment", // [2098]
    "alter_system_statement", // [2099]
    "AlterPolicyStatementSegment", // [2100]
    "alter_policy_statement", // [2101]
    "CreatePolicyStatementSegment", // [2102]
    "create_policy_statement", // [2103]
    "DropPolicyStatementSegment", // [2104]
    "drop_policy_statement", // [2105]
    "CreateDomainStatementSegment", // [2106]
    "create_domain_statement", // [2107]
    "AlterDomainStatementSegment", // [2108]
    "alter_domain_statement", // [2109]
    "DropDomainStatementSegment", // [2110]
    "drop_domain_statement", // [2111]
    "CreateMaterializedViewStatementSegment", // [2112]
    "create_materialized_view_statement", // [2113]
    "AlterMaterializedViewStatementSegment", // [2114]
    "alter_materialized_view_statement", // [2115]
    "DropMaterializedViewStatementSegment", // [2116]
    "drop_materialized_view_statement", // [2117]
    "RefreshMaterializedViewStatementSegment", // [2118]
    "refresh_materialized_view_statement", // [2119]
    "AlterDatabaseStatementSegment", // [2120]
    "alter_database_statement", // [2121]
    "VacuumStatementSegment", // [2122]
    "VACUUM", // [2123]
    "vacuum_statement", // [2124]
    "AlterFunctionStatementSegment", // [2125]
    "alter_function_statement", // [2126]
    "AlterViewStatementSegment", // [2127]
    "alter_view_statement", // [2128]
    "ListenStatementSegment", // [2129]
    "listen_statement", // [2130]
    "NotifyStatementSegment", // [2131]
    "notify_statement", // [2132]
    "UnlistenStatementSegment", // [2133]
    "UNLISTEN", // [2134]
    "unlisten_statement", // [2135]
    "LoadStatementSegment", // [2136]
    "load_statement", // [2137]
    "ResetStatementSegment", // [2138]
    "reset_statement", // [2139]
    "DiscardStatementSegment", // [2140]
    "discard_statement", // [2141]
    "AlterProcedureStatementSegment", // [2142]
    "alter_procedure_statement", // [2143]
    "CreateProcedureStatementSegment", // [2144]
    "create_procedure_statement", // [2145]
    "DropProcedureStatementSegment", // [2146]
    "drop_procedure_statement", // [2147]
    "CopyStatementSegment", // [2148]
    "copy_statement", // [2149]
    "DoStatementSegment", // [2150]
    "do_statement", // [2151]
    "AlterIndexStatementSegment", // [2152]
    "alter_index_statement", // [2153]
    "ReindexStatementSegment", // [2154]
    "reindex_statement_segment", // [2155]
    "AlterRoleStatementSegment", // [2156]
    "alter_role_statement", // [2157]
    "CreateExtensionStatementSegment", // [2158]
    "create_extension_statement", // [2159]
    "DropExtensionStatementSegment", // [2160]
    "drop_extension_statement", // [2161]
    "AlterExtensionStatementSegment", // [2162]
    "alter_extension_statement", // [2163]
    "CreateSubscriptionStatementSegment", // [2164]
    "create_subscription", // [2165]
    "AlterSubscriptionStatementSegment", // [2166]
    "alter_subscription", // [2167]
    "DropSubscriptionStatementSegment", // [2168]
    "drop_subscription", // [2169]
    "CreatePublicationStatementSegment", // [2170]
    "create_publication_statement", // [2171]
    "AlterPublicationStatementSegment", // [2172]
    "alter_publication_statement", // [2173]
    "DropPublicationStatementSegment", // [2174]
    "drop_publication_statement", // [2175]
    "CreateTypeStatementSegment", // [2176]
    "create_type_statement", // [2177]
    "AlterTypeStatementSegment", // [2178]
    "alter_type_statement", // [2179]
    "CreateTextSearchConfigurationStatementSegment", // [2180]
    "create_text_search_configuration_statement", // [2181]
    "AlterTextSearchConfigurationStatementSegment", // [2182]
    "alter_text_search_configuration_statement", // [2183]
    "DropTextSearchConfigurationStatementSegment", // [2184]
    "drop_text_search_configuration_statement", // [2185]
    "AlterSchemaStatementSegment", // [2186]
    "alter_schema_statement", // [2187]
    "LockTableStatementSegment", // [2188]
    "lock_table_statement", // [2189]
    "ClusterStatementSegment", // [2190]
    "cluster_statement", // [2191]
    "CreateCollationStatementSegment", // [2192]
    "create_collation_statement", // [2193]
    "DropCollationStatementSegment", // [2194]
    "drop_collation_statement", // [2195]
    "CallStoredProcedureSegment", // [2196]
    "call_statement", // [2197]
    "CreateServerStatementSegment", // [2198]
    "create_server_statement", // [2199]
    "CreateUserMappingStatementSegment", // [2200]
    "create_user_mapping_statement", // [2201]
    "ImportForeignSchemaStatementSegment", // [2202]
    "import_foreign_schema_statement", // [2203]
    "CreateForeignTableStatementSegment", // [2204]
    "create_foreign_table_statement", // [2205]
    "DropAggregateStatementSegment", // [2206]
    "drop_aggregate_statement", // [2207]
    "CreateAggregateStatementSegment", // [2208]
    "create_aggregate_statement", // [2209]
    "AlterAggregateStatementSegment", // [2210]
    "alter_aggregate_statement", // [2211]
    "CreateStatisticsStatementSegment", // [2212]
    "create_statistics_statement", // [2213]
    "AlterStatisticsStatementSegment", // [2214]
    "alter_statistics_statement", // [2215]
    "DropStatisticsStatementSegment", // [2216]
    "drop_statistics_statement", // [2217]
    "ShowStatementSegment", // [2218]
    "show_statement", // [2219]
    "SetConstraintsStatementSegment", // [2220]
    "set_constraint_statement", // [2221]
    "CreateForeignDataWrapperStatementSegment", // [2222]
    "create_foreign_data_wrapper", // [2223]
    "MetaCommandQueryBufferStatement", // [2224]
    "meta_command_statement", // [2225]
    "DropForeignTableStatement", // [2226]
    "drop_foreign_table_statement", // [2227]
    "CreateOperatorStatementSegment", // [2228]
    "create_operator_statement", // [2229]
    "AlterForeignTableStatementSegment", // [2230]
    "alter_foreign_table_statement", // [2231]
    "SecurityLabelStatementSegment", // [2232]
    "security_label_statement", // [2233]
    "PrepareStatementSegment", // [2234]
    "prepare_statement", // [2235]
    "ExecuteStatementSegment", // [2236]
    "execute_statement", // [2237]
    "DeallocateStatementSegment", // [2238]
    "deallocate_statement", // [2239]
    "SetSessionAuthorizationStatementSegment", // [2240]
    "set_session_authorization_statement", // [2241]
    "ResetSessionAuthorizationStatementSegment", // [2242]
    "reset_session_authorization_statement", // [2243]
    "STDIN", // [2244]
    "STDOUT", // [2245]
    "STRAIGHT_JOIN", // [2246]
    "ConcatSegment", // [2247]
    "STRIP", // [2248]
    "StructKeywordSegment", // [2249]
    "SUBLIST", // [2250]
    "SUBSTRING", // [2251]
    "SUCCESSFUL", // [2252]
    "SYMMETRIC", // [2253]
    "SYNONYM", // [2254]
    "SYSDATE", // [2255]
    "IndexParametersSegment", // [2256]
    "index_parameters", // [2257]
    "PrimaryKeyGrammar", // [2258]
    "ExclusionConstraintElementSegment", // [2259]
    "exclusion_constraint_element", // [2260]
    "OrdinalityKeywordSegment", // [2261]
    "Expression_A_Unary_Operator_Grammar", // [2262]
    "Expression_C_Grammar", // [2263]
    "Expression_B_Unary_Operator_Grammar", // [2264]
    "TRANSIENT", // [2265]
    "TransientKeywordSegment", // [2266]
    "TERMINATE", // [2267]
    "TERMINATED", // [2268]
    "TEXTSIZE", // [2269]
    "THAN", // [2270]
    "tilde", // [2271]
    "TimeKeywordSegment", // [2272]
    "TimestampKeywordSegment", // [2273]
    "ZoneKeywordSegment", // [2274]
    "ZONE", // [2275]
    "AtKeywordSegment", // [2276]
    "TINYBLOB", // [2277]
    "TINYINT", // [2278]
    "TINYTEXT", // [2279]
    "TOAST", // [2280]
    "TOP", // [2281]
    "TRAN", // [2282]
    "TRANSACTION", // [2283]
    "CommitKeywordSegment", // [2284]
    "RollbackKeywordSegment", // [2285]
    "WORK", // [2286]
    "TransactionKeywordSegment", // [2287]
    "WorkKeywordSegment", // [2288]
    "ChainKeywordSegment", // [2289]
    "TRANSACTIONS", // [2290]
    "TREAT", // [2291]
    "TRIM", // [2292]
    "BothKeywordSegment", // [2293]
    "LeadingKeywordSegment", // [2294]
    "TrailingKeywordSegment", // [2295]
    "ContinueKeywordSegment", // [2296]
    "TRUSTED", // [2297]
    "TSEQUAL", // [2298]
    "StructLiteralSegment", // [2299]
    "UID", // [2300]
    "UNCOMMITTED", // [2301]
    "CrossKeywordSegment", // [2302]
    "NaturalJoinKeywordsGrammar", // [2303]
    "UnconditionalCrossJoinKeywordsGrammar", // [2304]
    "HorizontalJoinKeywordsGrammar", // [2305]
    "UNDO", // [2306]
    "UNENCRYPTED", // [2307]
    "UnlistenKeywordSegment", // [2308]
    "UNLOCK", // [2309]
    "unpacking_operator", // [2310]
    "UNSIGNED", // [2311]
    "UPDATETEXT", // [2312]
    "USERS", // [2313]
    "SampleKeywordSegment", // [2314]
    "UTC_DATE", // [2315]
    "UTC_TIME", // [2316]
    "UTC_TIMESTAMP", // [2317]
    "VacuumKeywordSegment", // [2318]
    "Disable_page_skippingKeywordSegment", // [2319]
    "Index_cleanupKeywordSegment", // [2320]
    "Process_toastKeywordSegment", // [2321]
    "VARCHAR2", // [2322]
    "VARCHARACTER", // [2323]
    "VARIABLES", // [2324]
    "WAITFOR", // [2325]
    "assignment_operator", // [2326]
    "WAREHOUSES", // [2327]
    "LinestringKeywordSegment", // [2328]
    "MultipointKeywordSegment", // [2329]
    "MultilinestringKeywordSegment", // [2330]
    "MultipolygonKeywordSegment", // [2331]
    "GeometrycollectionKeywordSegment", // [2332]
    "PointzKeywordSegment", // [2333]
    "LinestringzKeywordSegment", // [2334]
    "PolygonzKeywordSegment", // [2335]
    "MultipointzKeywordSegment", // [2336]
    "MultilinestringzKeywordSegment", // [2337]
    "MultipolygonzKeywordSegment", // [2338]
    "GeometrycollectionzKeywordSegment", // [2339]
    "PointmKeywordSegment", // [2340]
    "LinestringmKeywordSegment", // [2341]
    "PolygonmKeywordSegment", // [2342]
    "MultipointmKeywordSegment", // [2343]
    "MultilinestringmKeywordSegment", // [2344]
    "MultipolygonmKeywordSegment", // [2345]
    "GeometrycollectionmKeywordSegment", // [2346]
    "PointzmKeywordSegment", // [2347]
    "LinestringzmKeywordSegment", // [2348]
    "PolygonzmKeywordSegment", // [2349]
    "MultipointzmKeywordSegment", // [2350]
    "MultilinestringzmKeywordSegment", // [2351]
    "MultipolygonzmKeywordSegment", // [2352]
    "GeometrycollectionzmKeywordSegment", // [2353]
    "CircularstringKeywordSegment", // [2354]
    "CompoundcurveKeywordSegment", // [2355]
    "CurvepolygonKeywordSegment", // [2356]
    "MulticurveKeywordSegment", // [2357]
    "MultisurfaceKeywordSegment", // [2358]
    "PolyhedralsurfaceKeywordSegment", // [2359]
    "TriangleKeywordSegment", // [2360]
    "TinKeywordSegment", // [2361]
    "SimpleGeometryGrammar", // [2362]
    "GeometryKeywordSegment", // [2363]
    "GeographyKeywordSegment", // [2364]
    "indented_then", // [2365]
    "indented_then_contents", // [2366]
    "WHILE", // [2367]
    "WHITESPACE", // [2368]
    "whitespace", // [2369]
    "WhitespaceSegment", // [2370]
    "WildcardExcludeExpressionSegment", // [2371]
    "wildcard_exclude", // [2372]
    "WildcardReplaceExpressionSegment", // [2373]
    "wildcard_replace", // [2374]
    "WildcardRenameExpressionSegment", // [2375]
    "wildcard_rename", // [2376]
    "WildcardPatternMatchingSegment", // [2377]
    "wildcard_pattern_matching", // [2378]
    "PartitionClauseSegment", // [2379]
    "partitionby_clause", // [2380]
    "FrameClauseSegment", // [2381]
    "frame_clause", // [2382]
    "CascadedKeywordSegment", // [2383]
    "RecursiveKeywordSegment", // [2384]
    "indented_ctes", // [2385]
    "CTEDefinitionSegment", // [2386]
    "common_table_expression", // [2387]
    "NonWithNonSelectableGrammar", // [2388]
    "BindingKeywordSegment", // [2389]
    "WithinKeywordSegment", // [2390]
    "WRITETEXT", // [2391]
    "X509", // [2392]
    "XMLATTRIBUTES", // [2393]
    "XMLCONCAT", // [2394]
    "XMLELEMENT", // [2395]
    "XMLEXISTS", // [2396]
    "XMLFOREST", // [2397]
    "XMLNAMESPACES", // [2398]
    "XMLPARSE", // [2399]
    "XMLPI", // [2400]
    "XMLROOT", // [2401]
    "XMLSERIALIZE", // [2402]
    "XMLTABLE", // [2403]
    "XOR", // [2404]
    "YEAR_MONTH", // [2405]
    "YES", // [2406]
    "ZEROFILL", // [2407]
];

pub static AUX_DATA: &[u32] = &[
    0, 1, 2, 1, 1, 4, 3, 1, 4, 5, 6, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 7, 8, 10, 3, 8, 9, 7, 3, 3, 4, 9, 11,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1, 0, 0,
    0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1,
    0, 1, 0, 1, 0, 1, 0, 0, 0, 4294967295, 4294967295, 0, 13, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 215, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    216, 1, 2, 1, 1, 4, 3, 1, 4, 5, 217, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 222, 1, 2, 1, 1, 4, 3, 1, 4, 5, 223, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 224, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 85, 237, 10, 1, 237, 3, 3, 4, 9, 238, 239, 10, 1, 239,
    3, 3, 4, 9, 1, 2, 1, 0, 244, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 14, 1, 2, 1, 1, 4, 3, 1, 4, 5, 249, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 250, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1, 2, 1, 0, 0, 1, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0, 0,
    1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 1, 1, 2, 1,
    0, 1, 0, 0, 1, 0, 1, 2, 1, 0, 0, 0, 0, 0, 4294967295, 4294967295,
    0, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1,
    2, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
    0, 0, 0, 0, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 0, 1,
    0, 0, 251, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2,
    1, 1, 0, 1, 2, 1, 0, 0, 1, 0, 1, 2, 1, 1, 0, 1,
    2, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 2, 1,
    1, 2, 1, 1, 4294967295, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0,
    0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4294967295, 1, 0, 0, 0,
    0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295,
    0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 3, 4, 1, 0, 0, 0,
    0, 1, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0, 0,
    4294967295, 4294967295, 0, 1, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0,
    0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0,
    0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 2,
    1, 1, 0, 1, 2, 1, 0, 440, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 669, 670, 10, 1, 670, 3, 3, 4, 9, 671, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 672, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    1, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 1, 0, 679, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 679, 680, 681, 1, 680, 3, 3, 680, 4,
    682, 1, 2, 1, 1, 4, 3, 1, 4, 5, 683, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 684, 1, 2, 1, 1, 4, 3, 1, 4, 5, 101, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 210, 704, 10, 1,
    704, 3, 3, 4, 9, 705, 706, 10, 1, 706, 3, 3, 4, 9, 1, 2,
    0, 296, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 0,
    1, 2, 0, 232, 1, 2, 1, 1, 4, 3, 1, 4, 5, 709, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 710, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 711, 1, 2, 1, 1, 4, 3, 1, 4, 5, 712, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 713, 1, 2, 1, 1, 4, 3, 1, 4, 5, 714,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 715, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 559, 1, 2, 1, 1, 4, 3, 1, 4, 5, 665, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 716, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 717, 1, 2, 1, 1, 4, 3, 1, 4, 5, 718, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 720, 1, 2, 1, 1, 4, 3, 1, 4, 5, 721,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 722, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 429, 430, 262, 364, 431, 432, 264, 433, 434, 266, 435, 126, 867,
    12, 723, 724, 1, 723, 2, 3, 4, 0, 0, 0, 734, 735, 0, 2, 3,
    734, 3, 736, 0, 1, 3, 667, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    737, 1, 2, 1, 1, 4, 3, 1, 4, 5, 738, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 739, 1, 2, 1, 1, 4, 3, 1, 4, 5, 740, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 741, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 742, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 680, 681, 9,
    3, 3, 680, 4, 755, 1, 2, 1, 1, 4, 3, 1, 4, 5, 756, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 757, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 690, 680, 10, 1, 680, 3, 3, 4, 9, 761, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 764, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 767, 1, 2, 1, 1, 4, 3, 1, 4, 5, 768, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 769, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 1, 2, 1, 1, 0, 1, 2, 1, 770, 771, 0, 2, 3, 770, 1,
    2, 1, 778, 1, 2, 1, 1, 4, 3, 1, 4, 5, 779, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 780, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    781, 1, 2, 1, 1, 4, 3, 1, 4, 5, 782, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 441, 1, 2, 1, 1, 4, 3, 1, 4, 5, 579, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 783, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 2, 1, 1, 2, 1, 0, 0, 446, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 790, 1, 2, 1, 1, 4, 3, 1, 4, 5, 521, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 334, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 792, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295,
    0, 0, 4294967295, 4294967295, 0, 0, 793, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    365, 1, 2, 1, 1, 4, 3, 1, 4, 5, 807, 808, 10, 1, 808, 3,
    3, 4, 9, 15, 1, 2, 1, 1, 4, 3, 1, 4, 5, 809, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 810, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 811, 1, 2, 1, 1, 4, 3, 1, 4, 5, 812, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 813, 1, 2, 1, 1, 4, 3, 1, 4, 5, 352,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 814, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 815, 1, 2, 1, 1, 4, 3, 1, 4, 5, 816, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 817, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 818, 1, 2, 1, 1, 4, 3, 1, 4, 5, 819, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 396, 1, 2, 1, 1, 4, 3, 1, 4, 5, 820,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 405, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 821, 1, 2, 1, 1, 4, 3, 1, 4, 5, 822, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 4, 724, 9, 2, 3, 4, 419, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 366, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 0, 0, 295, 826, 10, 1, 826, 3, 3, 4, 9, 295, 827,
    10, 1, 827, 3, 3, 4, 9, 295, 828, 10, 1, 828, 3, 3, 4, 9,
    0, 1, 2, 1, 0, 0, 0, 4294967295, 4294967295, 0, 733, 733, 724, 3, 733, 4,
    733, 2, 3, 4, 415, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    1, 0, 0, 1, 2, 1, 849, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    90, 857, 10, 1, 857, 3, 3, 4, 9, 858, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 2, 1, 0, 0, 1, 2, 1, 0, 0, 861, 862,
    10, 3, 3, 861, 4, 863, 1, 2, 1, 1, 4, 3, 1, 4, 5, 864,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 865, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 866, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 290, 876,
    9, 3, 3, 290, 4, 877, 1, 2, 1, 1, 4, 3, 1, 4, 5, 680,
    878, 0, 2, 3, 680, 290, 879, 0, 2, 3, 290, 1, 2, 1, 880, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 881, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 469, 1, 2, 1, 1, 4, 3, 1, 4, 5, 882, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 883, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    653, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 383, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2,
    1, 1, 0, 1, 2, 1, 1, 2, 1, 0, 1, 0, 0, 895, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 1, 0, 1, 2, 1,
    0, 102, 1, 2, 1, 1, 4, 3, 1, 4, 5, 272, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 350, 1, 2, 1, 1, 4, 3, 1, 4, 5, 898,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 899, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 900, 1, 2, 1, 1, 4, 3, 1, 4, 5, 901, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 367, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 902, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1,
    1, 2, 1, 0, 1, 0, 1, 2, 1, 0, 0, 0, 1, 0, 1, 2,
    1, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 1, 0, 1, 2,
    1, 0, 4294967295, 1, 0, 1, 0, 1, 2, 1, 0, 938, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 522, 1, 2, 1, 1, 4, 3, 1, 4, 5, 939,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 3, 4, 1, 0,
    0, 0, 0, 1, 0, 1, 2, 1, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0,
    0, 4294967295, 4294967295, 0, 0, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 0, 1, 0,
    1, 2, 1, 1, 0, 1, 2, 1, 0, 4294967295, 4294967295, 0, 0, 1, 0, 1,
    2, 1, 0, 0, 0, 0, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1,
    103, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 1, 2,
    1, 0, 1, 2, 1, 0, 0, 1, 0, 1, 2, 0, 0, 1, 0, 1,
    2, 1, 0, 4294967295, 994, 10, 1, 994, 3, 3, 4, 9, 0, 4294967295, 4294967295, 0,
    0, 0, 4294967295, 994, 10, 1, 994, 3, 3, 4, 9, 0, 4294967295, 4294967295, 0, 0,
    4294967295, 1005, 10, 1, 1005, 3, 3, 4, 9, 0, 4294967295, 4294967295, 0, 0, 0, 0,
    1, 0, 1, 2, 1, 0, 0, 0, 1, 0, 1, 2, 1, 1, 2, 1,
    1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4294967295, 1, 0,
    0, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 0, 1, 2, 1,
    0, 1, 0, 1, 0, 1, 2, 1, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2,
    1, 0, 0, 0, 1, 0, 1, 2, 1, 0, 0, 1, 2, 1, 0, 1,
    0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 2, 1, 0, 4294967295, 4294967295, 0,
    0, 1, 0, 1, 2, 1, 0, 0, 1, 2, 1, 0, 0, 580, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 581, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1059, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1060, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1061, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1062,
    1063, 724, 1, 1063, 2, 3, 4, 1062, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 2, 1, 316, 1, 2, 1, 1, 4, 3, 1, 4, 5, 429,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 430, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 262, 1, 2, 1, 1, 4, 3, 1, 4, 5, 364, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 431, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 432, 1, 2, 1, 1, 4, 3, 1, 4, 5, 264, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1069, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1070,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 447, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 55, 1, 2, 1, 1, 4, 3, 1, 4, 5, 16, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 0, 1071, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 2, 1072, 724, 1, 1072, 2, 3, 4, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 297, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2456, 0, 1063, 724, 1, 1063, 2, 3,
    4, 0, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1114, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1262, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1223, 1224, 1225, 1226, 1227, 1228, 1229, 1230, 1231, 1232, 1233, 1234, 1235, 1236, 1237, 1238,
    1239, 1240, 1241, 1242, 1243, 1244, 1245, 1246, 1247, 1248, 1249, 1250, 1251, 1252, 1253, 1254,
    1255, 2496, 33, 1263, 724, 1, 1263, 2, 3, 4, 1224, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1264, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1265, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1266, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1267, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1268, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1269, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1225, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1226, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1270, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1271, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1275, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1115, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1276, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 292, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1277, 1, 2, 1, 1, 4, 3, 1, 4, 5, 474, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 479, 1, 2, 1, 1, 4, 3, 1, 4, 5, 541,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 1282,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1283, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 104, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0,
    1, 0, 0, 920, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1293, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1294, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1023, 1, 2, 1, 1, 4, 3, 1, 4, 5, 555, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 787, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1295,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1296, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1298, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1299, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 646, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 946, 1, 2, 1, 1, 4, 3, 1, 4, 5, 384, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 406, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1300,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1301, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1305, 1, 2, 1, 1, 4, 3, 1, 4, 5, 987, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1306, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1307, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1308, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 687, 680, 681, 1, 680, 3, 3, 680, 4, 686,
    680, 681, 1, 680, 3, 3, 680, 4, 0, 891, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1311, 1, 2, 1, 1, 4, 3, 1, 4, 5, 307, 307,
    1313, 3, 307, 1312, 307, 3, 3, 1312, 4, 17, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 209, 1314, 10, 1, 1314, 3, 3, 4, 9, 1116, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 0, 0, 3, 4, 1, 1, 2, 1,
    1, 0, 1, 0, 0, 1, 0, 341, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 0,
    0, 1, 0, 1, 0, 1, 0, 1315, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1316, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1044, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 801, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1317,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1318, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 1, 407, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1321, 1, 2, 1, 1, 4, 3, 1, 4, 5, 921, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 582, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1322, 1323,
    10, 1, 1323, 3, 3, 4, 9, 803, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1048, 1, 2, 1, 1, 4, 3, 1, 4, 5, 744, 290, 876, 1, 290,
    3, 3, 290, 4, 1324, 1, 2, 1, 1, 4, 3, 1, 4, 5, 922, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1325, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 368, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1326, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 471, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1327, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1328, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 105, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0,
    1, 2, 1, 1329, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1330, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1331, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1332, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1,
    0, 1, 2, 1, 0, 0, 437, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 2, 1,
    0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 1, 0, 0, 1, 2, 1, 0, 1,
    0, 1, 2, 1, 0, 0, 0, 462, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 2, 1, 0, 0, 360, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    18, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1417, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1398, 1418, 10, 3, 1418, 9, 1398, 3, 3, 4, 9, 279,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 279, 1419, 1420, 1, 1419, 5, 3,
    1, 1312, 4, 5, 397, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    0, 0, 1421, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1427, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 19, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 4294967295, 4294967295, 0, 1, 0, 0, 4294967295, 4294967295, 0, 1439, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 2, 1, 1440, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 655, 1, 2, 1, 1, 4, 3, 1, 4, 5, 641, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1118, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1119,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1117, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1442, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1443, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 0, 330, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 408, 1, 2, 1, 1, 4, 3, 1, 4, 5, 923,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 924, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 925, 1, 2, 1, 1, 4, 3, 1, 4, 5, 20, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 0, 1, 2, 1, 179,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1446, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 0, 0, 0, 0, 1460, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1461, 1, 2, 1, 1, 4, 3, 1, 4, 5, 926, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 2, 1, 0, 0, 0,
    4294967295, 4294967295, 0, 1, 2, 1, 0, 1, 2, 1, 0, 228, 1496, 1, 0, 0,
    4294967295, 4294967295, 0, 242, 1496, 1, 1, 2, 1, 0, 94, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 0, 1, 0, 1, 2, 1, 0, 4294967295, 4294967295, 0, 6,
    7, 1, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 0, 0, 1,
    0, 1, 2, 1, 1, 4294967295, 4294967295, 0, 3, 4, 1, 506, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1396, 1396, 1313, 3, 1396, 1312, 1396, 3, 3, 1312, 4,
    1502, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 0,
    0, 4294967295, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 4294967295, 4294967295, 0,
    1, 2, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 4294967295,
    4294967295, 0, 0, 4294967295, 4294967295, 0, 1, 0, 1, 2, 1, 21, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 3, 4, 1063, 724, 1, 1063, 2, 3, 4, 0, 4294967295,
    4294967295, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 2, 1, 0, 0, 70,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 22, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 439, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1074, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1075, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1076, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1077, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1078, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1079,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1357, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 747, 747, 876, 3, 747, 290, 747, 3, 3, 290, 4, 1535, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 191, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 0, 0, 1542, 1, 2, 1, 1, 4, 3, 1, 4, 5, 136,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1544, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 0, 0, 197, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 1, 0, 1, 0, 1546, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 0, 1, 2, 1, 1447, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 966, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1552, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 995, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    2, 1, 0, 0, 1464, 1, 2, 1, 1, 4, 3, 1, 4, 5, 927, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1553, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1554, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1555, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1556, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1559, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1560, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1231, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1561, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1562, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1563, 1, 2, 1, 1, 4, 3, 1, 4, 5, 956, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1564, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1565, 1566, 9, 3, 3, 1565, 4, 445, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1567, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1568, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 357, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1514,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1358, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 481, 1, 2, 1, 1, 4, 3, 1, 4, 5, 523, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 945, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1, 0, 1, 2, 1, 1572, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 106, 1, 2, 1, 1, 4, 3, 1, 4, 5, 65, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 2, 1, 0, 982, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1581, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 448, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1,
    2, 1, 0, 509, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1590, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1591, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1120, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1592, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1593, 1, 2, 1, 1, 4, 3, 1, 4, 5, 409,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 973, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1594, 1, 2, 1, 1, 4, 3, 1, 4, 5, 476, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1595, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 885, 1, 2, 1, 1, 4, 3, 1, 4, 5, 942, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 534, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1596,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 107, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 1, 2, 1, 0, 0, 0, 1, 0, 0, 1606, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1034, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1607, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1122, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1608, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1123, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1124, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1125, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1126, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1121, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1127, 1, 2, 1, 1, 4, 3, 1, 4, 5, 23, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1609, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1465, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 298, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 1578, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 542, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 745, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1621, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 273, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1622, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1372, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1623, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1624, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 228, 1628, 1, 0, 242, 1628, 1, 0,
    996, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 795, 1641,
    1, 1, 2, 1, 0, 242, 1641, 1, 0, 0, 1, 0, 1, 2, 1, 1128,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 748, 680, 10, 3, 680, 9, 748,
    3, 3, 4, 9, 0, 0, 1129, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    834, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1651, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 2, 9, 4, 3, 1, 4, 5, 1653, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1654, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1655, 1656, 10, 1, 1656, 3, 3, 4, 9, 1, 0, 1, 2, 1, 0, 1,
    0, 1, 2, 1, 0, 0, 1658, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1660, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 24, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 95, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1585,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1661, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1483, 1, 2, 1, 1, 4, 3, 1, 4, 5, 947, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 948, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1505, 1, 2, 1, 1, 4, 3, 1, 4, 5, 524, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1662, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1663,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 886, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 997, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1664, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 492, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 957, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1359, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 749, 749, 876, 3, 749, 290, 749, 3, 3, 290,
    4, 0, 0, 0, 1, 2, 1, 0, 0, 0, 283, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1130, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1676, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1677, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1080, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1081, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1082, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1083, 1, 2, 1, 1, 4, 3, 1, 4, 5, 6, 7, 0, 1680, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1681, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1312, 1420, 9, 5, 3, 1, 1312, 4, 5, 1312, 1313, 9, 3, 3,
    1312, 4, 1690, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1692, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 949, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    433, 1, 2, 1, 1, 4, 3, 1, 4, 5, 434, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1693, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1694, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 0, 1445, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1699, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    500, 1, 2, 1, 1, 4, 3, 1, 4, 5, 583, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1700, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1701, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1702, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1703, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1704, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1705, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1131, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 2, 3, 1,
    1133, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1132, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 978, 1, 2, 1, 1, 4, 3, 1, 4, 5, 463, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 108, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 726, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 305, 1710,
    10, 1, 1710, 3, 3, 4, 9, 1, 2, 0, 1, 0, 1, 2, 1, 657,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 25, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1713, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1714, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 26, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1715, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1716, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 449, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1024,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1717, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1718, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1719, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1720, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 4294967295, 4294967295, 0, 0, 0, 0, 0, 1, 2, 1, 0, 228, 1628, 1,
    242, 1628, 1, 998, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1747, 1748, 10,
    3, 1748, 9, 1747, 3, 3, 4, 9, 0, 4294967295, 4294967295, 0, 375, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1750, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1238, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1751, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1466, 1, 2, 1, 1, 4, 3, 1, 4, 5, 218, 680,
    10, 1, 680, 3, 3, 4, 9, 1240, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1752, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1753, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 450, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1492,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1754, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1698, 1, 2, 1, 1, 4, 3, 1, 4, 5, 993, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 109, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 685, 680, 10, 1, 680, 3, 3, 4, 9, 1755, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1134, 1, 2, 1, 1, 4, 3, 1, 4, 5, 43, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1242, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1756, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1757, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1084, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 4294967295, 4294967295, 0, 310, 310, 1759, 1, 310, 3, 3, 310, 4, 1085, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1086, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1087, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1088, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 255, 680, 10, 1, 680, 3, 3, 4, 9, 1089, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1090, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1091, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1092, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1093, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1094, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1095, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1096, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1097, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1760, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 5, 1761, 1566, 3, 1761, 1565, 5, 3, 3, 1565, 4, 5, 6, 1762,
    1566, 1, 1762, 3, 3, 1565, 4, 617, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 2, 1, 0, 1, 0, 1769, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1364, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1364, 1770, 1420, 1,
    1770, 5, 3, 1, 1312, 4, 5, 1771, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1634, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1773, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1025, 1, 2, 1, 1, 4, 3, 1, 4, 5, 218,
    8, 10, 1, 8, 3, 3, 4, 9, 999, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1039, 1, 2, 1, 1, 4, 3, 1, 4, 5, 310, 1759, 12, 3,
    3, 310, 4, 1423, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1365, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1366, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1367, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1368, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 410, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1774,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1775, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 584, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1776, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1777, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1778, 1, 2, 1, 1, 4, 3, 1, 4, 5, 585, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 586, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1779,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1780, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 587, 1, 2, 1, 1, 4, 3, 1, 4, 5, 588, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 2, 1, 1, 2, 1,
    0, 0, 0, 0, 1, 2, 1, 0, 0, 1791, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 624, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1792, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 589, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1793, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1369, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 590, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 347, 1, 2, 1, 1, 4, 3, 1, 4, 5, 347, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 507, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1801,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1374, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 565, 1, 2, 1, 1, 4, 3, 1, 4, 5, 299, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 299, 1770, 1420, 1, 1770, 5, 3, 1, 1312,
    4, 5, 1803, 1, 2, 1, 1, 4, 3, 1, 4, 5, 984, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1135, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    100, 100, 1313, 3, 100, 1312, 100, 3, 3, 1312, 4, 1136, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 98, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1, 0, 1, 2, 0, 0, 0, 1806, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 411, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1807, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1808, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1467, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1809, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 498, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1040, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 166, 1762, 1566, 1, 1762, 3, 3, 1565,
    4, 166, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1810, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 517, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1811,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1812, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1813, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1814, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 110, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1815, 1, 2, 1, 1, 4, 3, 1, 4, 5, 369, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1816, 1, 2, 1, 1, 4, 3, 1, 4, 5, 327,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1817, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 4294967295, 4294967295, 0, 1, 2, 1, 412, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1, 0, 746, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 746, 680, 681, 1, 680, 3, 3, 680, 4, 0, 0, 0, 1, 0, 0,
    226, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1821, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1457, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1525,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1646, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1822, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1,
    0, 1824, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 3, 4, 1,
    0, 0, 0, 3, 4, 1, 0, 1826, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1827, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1828, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 904, 1, 2, 1, 1, 4, 3, 1, 4, 5, 905,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 451, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 256, 1, 2, 1, 1, 4, 3, 1, 4, 5, 111, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1829, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 525, 1, 2, 1, 1, 4, 3, 1, 4, 5, 7, 4294967295, 1830, 724, 1,
    1830, 2, 3, 4, 1831, 1830, 10, 1, 1830, 3, 3, 4, 9, 1832, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 906, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 385, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1833, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1,
    0, 1, 0, 1, 2, 1, 1, 2, 1, 0, 1, 0, 1, 2, 1, 0,
    562, 1, 2, 1, 1, 4, 3, 1, 4, 5, 907, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1837, 1, 2, 1, 1, 4, 3, 1, 4, 5, 591, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1137, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 5, 1839, 1840, 2, 1839, 5, 3, 3, 4, 5, 1, 0, 0, 1841,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1842, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1009, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1843, 680, 10,
    3, 680, 9, 1843, 3, 3, 4, 9, 1138, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1844, 680, 10, 3, 680, 9, 1844, 3, 3, 4, 9, 139, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 221, 1845, 10, 1, 1845, 3, 3, 4, 9,
    773, 1, 2, 1, 1, 4, 3, 1, 4, 5, 774, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 775, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1846, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 464, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1303, 1, 2, 1, 1, 4, 3, 1, 4, 5, 294, 680, 10, 1,
    680, 3, 3, 4, 9, 1098, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1099,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1100, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1101, 1, 2, 1, 1, 4, 3, 1, 4, 5, 131, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1102, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1103, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1104, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1105, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1106,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1600, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1558, 1, 2, 1, 1, 4, 3, 1, 4, 5, 294, 8, 10,
    1, 8, 3, 3, 4, 9, 0, 4294967295, 4294967295, 0, 1853, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1854, 680, 10, 3, 680, 9, 1854, 3, 3, 4, 9, 1429,
    1430, 724, 3, 1430, 4, 1429, 2, 3, 4, 691, 680, 10, 3, 680, 9, 691,
    3, 3, 4, 9, 0, 1452, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1166,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1857, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1858, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1274, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 0, 1861, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1862, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 423, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1863, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1380, 1, 2, 1, 1, 4, 3, 1, 4, 5, 144,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1864, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 370, 1, 2, 1, 1, 4, 3, 1, 4, 5, 27, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 71, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1865, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1866, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1867, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1868,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 5, 1869, 724, 3, 1869, 4, 5,
    2, 3, 4, 950, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1432, 1432, 724,
    3, 1432, 4, 1432, 2, 3, 4, 1435, 1435, 724, 3, 1435, 4, 1435, 2, 3,
    4, 295, 1872, 10, 1, 1872, 3, 3, 4, 9, 0, 1, 2, 1, 0, 208,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 572, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 0, 0, 0, 1, 0, 0, 0, 1, 2, 1, 1877, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 8, 4294967295, 1878, 10, 1, 1878, 3, 3,
    4, 9, 1, 4294967295, 4294967295, 0, 1, 2, 1, 1, 2, 1, 0, 1468, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 1244, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 928, 1, 2, 1, 1, 4, 3, 1, 4, 5, 314,
    1882, 1566, 3, 1882, 1565, 314, 3, 3, 1565, 4, 241, 1883, 1313, 3, 1883, 1312,
    241, 3, 3, 1312, 4, 309, 1883, 1313, 3, 1883, 1312, 309, 3, 3, 1312, 4,
    0, 241, 1883, 1313, 3, 1883, 1312, 241, 3, 3, 1312, 4, 0, 4294967295, 4294967295, 0,
    306, 1883, 1313, 3, 1883, 1312, 306, 3, 3, 1312, 4, 9, 4294967295, 1883, 1313, 1,
    1883, 3, 3, 1312, 4, 0, 4294967295, 4294967295, 0, 311, 1883, 1313, 3, 1883, 1312, 311,
    3, 3, 1312, 4, 10, 4294967295, 1883, 1313, 1, 1883, 3, 3, 1312, 4, 0, 4294967295,
    4294967295, 0, 11, 4294967295, 1887, 724, 1, 1887, 2, 3, 4, 308, 1883, 1313, 3, 1883,
    1312, 308, 3, 3, 1312, 4, 308, 1883, 1313, 3, 1883, 1312, 308, 3, 3, 1312,
    4, 0, 4294967295, 4294967295, 0, 0, 1, 0, 0, 1888, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1889, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1448, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 288, 1890, 10, 1, 1890, 3, 3, 4, 9,
    689, 1890, 10, 1, 1890, 3, 3, 4, 9, 1891, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 688, 1890, 10, 1, 1890, 3, 3, 4, 9, 743, 1890, 10, 1,
    1890, 3, 3, 4, 9, 112, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1892,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1139, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1893, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0,
    0, 1895, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1896, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1897, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1898,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 1, 0, 0, 114, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 113, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1037, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1,
    2, 1, 0, 631, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1906, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1907, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 0, 1, 2, 1, 0, 0, 1, 0, 1, 2, 1, 1911, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1912, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1913, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1836, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 257, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1914, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1915, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 659, 1, 2, 1, 1, 4, 3, 1, 4, 5, 413, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 592, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1916, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1917, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 274, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1920, 1, 2, 1, 1, 4, 3, 1, 4, 5, 28, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1515, 1, 2, 1, 1, 4, 3, 1, 4, 5, 427,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 332, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 546, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1010, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1921, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1014, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1290, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 526, 1, 2, 1, 1, 4, 3, 1, 4, 5, 192,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1763, 1922, 10, 1, 1922, 3,
    3, 4, 9, 887, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1000, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1923, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 29, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1924, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1925, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1064,
    1063, 724, 1, 1063, 2, 3, 4, 1064, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 30, 1, 2, 1, 1, 4, 3, 1, 4, 5, 72, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 490, 1, 2, 1, 1, 4, 3, 1, 4, 5, 908,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 909, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1926, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1927, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1928, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1929, 1, 2, 1, 1, 4, 3, 1, 4, 5, 527, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 486, 1, 2, 1, 1, 4, 3, 1, 4, 5, 547,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 1, 2,
    1, 0, 2, 3, 1, 2, 3, 1, 0, 1, 2, 1, 1935, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1939, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1940, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1648, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 31, 1, 2, 1, 1, 4, 3, 1, 4, 5, 63,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1941, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 382, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1246, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1942, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 494, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 2,
    1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 0, 0, 0, 1, 0, 1,
    2, 1, 0, 1, 0, 2, 9130, 2, 9132, 3, 745, 987, 745, 347, 987, 0,
    115, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 1, 2,
    1, 0, 1, 2, 1, 0, 890, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 4294967295, 4294967295, 0, 825, 1967, 10, 1, 1967, 3, 3, 4, 9, 1511, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 32, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 73, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1141, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1142, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1143, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1140, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1968, 1, 2, 1, 1, 4, 3, 1, 4, 5, 56,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1969, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1970, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1919, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 266, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 0, 0, 0, 4294967295, 4294967295, 0, 1, 0, 1, 2, 1, 0, 0,
    4294967295, 4294967295, 0, 1, 0, 0, 1, 2, 1, 0, 1, 0, 0, 0, 1, 4294967295,
    4294967295, 0, 258, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 1, 0, 0, 0, 1980, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1549, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1333, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1981, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 203, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1982, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 4294967295, 4294967295, 0, 1983, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1988, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1989,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1360, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 1902, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 0, 1, 0, 1, 2, 1, 0, 1, 0, 1, 0, 0, 1,
    0, 1, 2, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 241, 1882, 1566,
    3, 1882, 1565, 241, 3, 3, 1565, 4, 632, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 676, 1, 2, 1, 1, 4, 3, 1, 4, 5, 686, 1994, 10, 1,
    1994, 3, 3, 4, 9, 295, 1995, 10, 1, 1995, 3, 3, 4, 9, 1144, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1145, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1996, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1879, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1997, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1736, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1998, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1999, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2000, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2001, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2002, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2003, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2004, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2005, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2006, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2007, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2008, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2009, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2010, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2011, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 528, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    33, 1, 2, 1, 1, 4, 3, 1, 4, 5, 74, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2012, 1, 2, 1, 1, 4, 3, 1, 4, 5, 255, 850,
    10, 1, 850, 3, 3, 4, 9, 452, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2013, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2014, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1045, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    457, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2244, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2245, 1, 2, 1, 1, 4, 3, 1, 4, 5, 461, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1030, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2246, 1, 2, 1, 1, 4, 3, 1, 4, 5, 34, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 75, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    529, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 2248, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1051, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 2250, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 628, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2251, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2252, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1334, 1, 2, 1, 1, 4, 3, 1, 4, 5, 593, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 530, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    9, 10, 9, 3, 3, 4, 9, 2253, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2254, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2255, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1017, 1, 2, 1, 1, 4, 3, 1, 4, 5, 638,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 0, 1, 2,
    1, 1, 2, 1, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 1, 2,
    1, 1, 2, 1, 0, 35, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1, 0, 76, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1488, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 36, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 1738, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 37, 1, 2, 1, 1, 4, 3, 1, 4, 5, 77, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 116, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    386, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 117, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 2267, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2268, 1, 2, 1, 1, 4, 3, 1, 4, 5, 371, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2269, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2270, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1730, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1426, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1381, 2271, 10, 1,
    2271, 3, 3, 4, 9, 300, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 0, 4294967295, 4294967295, 0, 301, 1, 2, 1, 1, 4, 3, 1, 4, 5, 302,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 303, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1335, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1107, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2277, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2278, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2279, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 92, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2280,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2281, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1506, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2282, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2283, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 2290, 1, 2, 1, 1, 4, 3, 1, 4, 5, 372, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2265, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2291, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1108, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 118, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2292,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 280, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 280, 1419, 1420, 1, 1419, 5, 3, 1, 1312, 4, 5, 119,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 0, 2297, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2298, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1146, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1147, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1148, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1149,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 38, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 322, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1885, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2300, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1451, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2301, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 2306, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2307, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1053, 1, 2, 1, 1, 4, 3, 1, 4, 5, 424, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1370, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1370, 1770,
    1420, 1, 1770, 5, 3, 1, 1312, 4, 5, 2134, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 2309, 1, 2, 1, 1, 4, 3, 1, 4, 5, 501, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 4294967295, 4294967295, 0, 850, 2310, 10,
    3, 2310, 9, 850, 3, 3, 4, 9, 776, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 548, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2311, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 610, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    120, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 0, 2312, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 121, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 122, 1, 2, 1, 1, 4, 3, 1, 4, 5, 123, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 126, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2313, 1, 2, 1, 1, 4, 3, 1, 4, 5, 401, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2315, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2316, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2317, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1150, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2123, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 0, 1, 0, 1, 2, 1, 0, 1,
    0, 355, 1, 2, 1, 1, 4, 3, 1, 4, 5, 342, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 968, 1, 2, 1, 1, 4, 3, 1, 4, 5, 663,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 1,
    0, 648, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2322, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1151, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2323,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1977, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2324, 1, 2, 1, 1, 4, 3, 1, 4, 5, 304, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1182, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1152, 1, 2, 1, 1, 4, 3, 1, 4, 5, 675, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 951, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    39, 1, 2, 1, 1, 4, 3, 1, 4, 5, 78, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1031, 1, 2, 1, 1, 4, 3, 1, 4, 5, 531, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2325, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1336, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1388, 2326, 10, 1,
    2326, 3, 3, 4, 9, 40, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2327,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1251, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1252, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0,
    1, 2, 1, 0, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 0, 1,
    0, 1, 2, 1, 0, 0, 1, 2, 1, 0, 228, 2365, 1, 795, 2366, 1,
    242, 2366, 1, 242, 2365, 1, 797, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 2, 1, 0, 0, 893, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2367,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2368, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2369, 2370, 12, 3, 3, 4, 2369, 1, 0, 1, 2, 1, 0,
    0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 1, 0, 1, 2, 1, 0, 1, 0,
    1, 2, 1, 0, 1015, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    228, 2385, 1, 1, 0, 242, 2385, 1, 228, 2385, 1, 1, 0, 242, 2385, 1,
    275, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1848, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 497, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 5, 1840, 9, 3, 3, 4, 5, 2286, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 60, 1, 2, 1, 1, 4, 3, 1, 4, 5, 124, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 910, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2391, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2392, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1153, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2393, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2394, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2395, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2396, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2397, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2398, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2399, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2400, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2401, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2402, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2403, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2404, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1343, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1254, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2405, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2406, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2407, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2275, 1, 2, 1, 1, 4, 3, 1,
    4, 5,
];

pub static AUX_DATA_OFFSETS: &[u32] = &[
    0, 10, 20, 31, 47, 12, 0, 43, 0, 41, 42, 44, 45, 46, 47, 48,
    49, 50, 51, 52, 53, 0, 54, 41, 57, 0, 58, 59, 0, 61, 62, 64,
    45, 0, 66, 62, 64, 45, 67, 0, 61, 69, 64, 49, 0, 66, 69, 64,
    42, 45, 49, 45, 44, 80, 0, 82, 84, 87, 89, 91, 93, 0, 96, 97,
    99, 53, 0, 125, 48, 51, 127, 44, 45, 47, 0, 128, 129, 130, 0, 132,
    133, 0, 134, 135, 0, 42, 49, 137, 138, 0, 52, 0, 125, 50, 49, 0,
    129, 130, 140, 67, 0, 141, 51, 49, 50, 0, 142, 143, 128, 145, 125, 146,
    132, 147, 148, 42, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159, 138,
    160, 161, 0, 61, 143, 162, 54, 163, 165, 56, 167, 168, 169, 170, 171, 172,
    173, 174, 133, 49, 0, 129, 130, 0, 175, 168, 0, 176, 167, 0, 177, 178,
    57, 180, 181, 182, 183, 184, 185, 186, 187, 188, 62, 0, 129, 189, 0, 175,
    181, 0, 176, 180, 0, 177, 190, 58, 193, 195, 71, 0, 198, 59, 80, 0,
    127, 61, 199, 0, 51, 63, 201, 0, 204, 65, 80, 67, 201, 69, 205, 207,
    72, 211, 213, 76, 86, 96, 106, 0, 219, 220, 219, 116, 126, 136, 225, 0,
    228, 231, 167, 0, 234, 164, 235, 146, 155, 240, 242, 168, 0, 245, 247, 178,
    188, 198, 0, 252, 253, 80, 209, 208, 84, 0, 254, 213, 0, 259, 91, 82,
    0, 260, 91, 212, 261, 263, 265, 201, 0, 267, 49, 268, 0, 252, 45, 270,
    223, 0, 276, 214, 0, 277, 278, 0, 281, 282, 99, 0, 284, 278, 0, 259,
    91, 270, 0, 260, 91, 218, 80, 261, 263, 265, 0, 267, 52, 285, 0, 267,
    287, 221, 0, 219, 91, 289, 220, 291, 293, 312, 313, 0, 93, 315, 0, 317,
    222, 61, 287, 0, 318, 319, 165, 321, 91, 224, 324, 276, 0, 276, 318, 326,
    229, 0, 61, 143, 227, 226, 125, 146, 132, 147, 153, 154, 157, 158, 159, 138,
    165, 0, 328, 0, 318, 326, 329, 319, 165, 321, 93, 230, 324, 331, 333, 335,
    232, 180, 183, 185, 186, 336, 62, 0, 252, 291, 143, 0, 329, 233, 51, 127,
    234, 80, 64, 318, 328, 0, 64, 49, 236, 268, 318, 328, 238, 337, 339, 239,
    0, 198, 201, 207, 0, 252, 46, 80, 244, 0, 267, 291, 343, 0, 345, 291,
    0, 240, 267, 345, 346, 348, 0, 349, 0, 351, 80, 241, 0, 346, 348, 0,
    353, 343, 0, 346, 354, 0, 345, 351, 356, 80, 242, 331, 333, 0, 259, 351,
    80, 91, 80, 0, 358, 351, 80, 0, 260, 91, 243, 80, 261, 263, 265, 0,
    259, 91, 80, 0, 267, 49, 80, 0, 252, 359, 361, 268, 0, 159, 0, 91,
    293, 0, 267, 49, 245, 268, 363, 0, 246, 349, 345, 267, 0, 248, 0, 373,
    374, 376, 377, 46, 0, 378, 157, 0, 54, 58, 59, 0, 54, 167, 0, 379,
    48, 49, 173, 57, 0, 380, 381, 247, 387, 388, 389, 390, 53, 80, 0, 249,
    0, 175, 168, 167, 168, 245, 0, 253, 80, 250, 0, 0, 0, 391, 253, 0,
    80, 392, 80, 0, 256, 170, 171, 172, 257, 0, 82, 84, 0, 393, 266, 0,
    80, 262, 260, 259, 394, 89, 394, 0, 265, 398, 399, 80, 400, 402, 0, 404,
    329, 53, 287, 306, 0, 349, 414, 416, 417, 394, 0, 420, 421, 269, 425, 0,
    345, 414, 356, 417, 335, 0, 252, 414, 417, 288, 0, 0, 267, 58, 53, 394,
    0, 420, 421, 0, 400, 273, 343, 0, 267, 291, 275, 274, 293, 205, 428, 343,
    0, 345, 291, 0, 276, 267, 345, 346, 348, 0, 345, 436, 356, 0, 349, 438,
    277, 442, 0, 443, 291, 392, 444, 282, 278, 453, 0, 286, 0, 267, 438, 285,
    442, 0, 443, 291, 0, 267, 453, 0, 455, 0, 276, 99, 0, 345, 444, 356,
    0, 267, 456, 99, 0, 267, 458, 0, 317, 458, 0, 267, 460, 287, 465, 176,
    466, 467, 0, 267, 468, 287, 0, 349, 470, 0, 349, 473, 0, 252, 351, 287,
    289, 475, 0, 346, 475, 290, 0, 477, 478, 0, 477, 480, 0, 358, 351, 287,
    0, 345, 351, 356, 287, 335, 0, 291, 482, 483, 157, 292, 287, 61, 127, 0,
    482, 293, 484, 442, 157, 287, 0, 294, 482, 483, 0, 482, 484, 0, 482, 485,
    485, 287, 0, 295, 483, 482, 487, 0, 488, 487, 489, 491, 493, 0, 495, 165,
    287, 0, 267, 496, 296, 495, 499, 0, 267, 52, 285, 0, 267, 297, 502, 503,
    0, 267, 458, 0, 317, 458, 0, 488, 504, 245, 0, 505, 287, 0, 346, 505,
    0, 260, 91, 298, 287, 261, 263, 265, 0, 484, 444, 299, 291, 0, 400, 508,
    510, 512, 513, 0, 0, 252, 415, 417, 514, 303, 301, 0, 300, 349, 267, 345,
    234, 515, 0, 252, 54, 167, 0, 356, 516, 245, 254, 309, 307, 518, 0, 259,
    414, 417, 91, 417, 0, 259, 91, 245, 0, 318, 310, 0, 532, 165, 348, 533,
    0, 535, 348, 165, 348, 533, 536, 311, 537, 538, 539, 0, 346, 540, 0, 176,
    493, 312, 543, 544, 0, 545, 313, 549, 550, 551, 0, 552, 99, 0, 553, 99,
    0, 554, 287, 0, 267, 287, 316, 0, 314, 91, 289, 315, 293, 312, 291, 0,
    93, 315, 0, 317, 317, 61, 287, 331, 0, 252, 170, 319, 0, 82, 84, 323,
    556, 0, 259, 91, 82, 0, 267, 49, 268, 0, 260, 91, 322, 321, 287, 313,
    261, 263, 265, 0, 488, 558, 165, 359, 361, 0, 252, 508, 337, 0, 356, 510,
    334, 0, 259, 91, 510, 0, 267, 52, 285, 0, 560, 561, 510, 0, 488, 558,
    165, 359, 361, 0, 267, 326, 324, 0, 287, 0, 289, 293, 0, 317, 331, 329,
    287, 0, 252, 414, 99, 267, 456, 99, 0, 61, 64, 52, 285, 0, 563, 443,
    335, 201, 267, 52, 285, 564, 338, 371, 0, 252, 414, 417, 359, 0, 267, 456,
    99, 0, 267, 350, 348, 0, 566, 289, 293, 0, 317, 355, 353, 566, 0, 267,
    460, 358, 465, 176, 466, 467, 0, 267, 468, 287, 0, 495, 165, 287, 0, 267,
    496, 495, 0, 267, 362, 360, 0, 566, 0, 289, 293, 0, 317, 367, 365, 566,
    0, 260, 91, 370, 80, 261, 263, 265, 0, 252, 175, 168, 377, 0, 356, 245,
    374, 372, 567, 0, 259, 0, 414, 417, 91, 417, 0, 259, 91, 245, 0, 267,
    49, 268, 0, 245, 488, 558, 165, 359, 361, 0, 61, 64, 52, 285, 0, 563,
    443, 375, 80, 267, 52, 285, 0, 564, 0, 252, 130, 80, 165, 245, 391, 0,
    259, 91, 80, 387, 0, 91, 379, 378, 201, 207, 261, 263, 265, 0, 400, 381,
    343, 0, 276, 353, 384, 343, 0, 397, 0, 176, 493, 392, 543, 544, 0, 267,
    287, 395, 0, 393, 91, 289, 394, 293, 312, 291, 0, 93, 315, 0, 317, 396,
    61, 287, 331, 0, 252, 171, 398, 0, 82, 84, 407, 569, 0, 259, 91, 82,
    0, 267, 49, 268, 0, 267, 287, 404, 0, 400, 91, 289, 402, 401, 287, 293,
    0, 93, 315, 0, 260, 91, 406, 405, 287, 313, 261, 263, 265, 0, 488, 558,
    165, 359, 361, 0, 252, 571, 573, 415, 0, 267, 575, 0, 349, 408, 577, 0,
    267, 410, 577, 0, 345, 412, 577, 0, 259, 91, 573, 0, 260, 91, 414, 261,
    263, 265, 201, 0, 252, 416, 51, 127, 439, 0, 417, 261, 263, 265, 201, 276,
    426, 418, 594, 595, 419, 596, 597, 420, 598, 599, 421, 504, 600, 422, 601, 602,
    423, 603, 604, 424, 605, 606, 0, 281, 282, 99, 0, 607, 608, 425, 515, 348,
    0, 354, 609, 515, 0, 201, 0, 259, 91, 201, 0, 430, 261, 263, 265, 61,
    201, 0, 64, 45, 270, 438, 0, 267, 287, 436, 0, 431, 91, 289, 435, 291,
    433, 432, 293, 312, 611, 0, 93, 315, 0, 317, 437, 287, 61, 0, 252, 49,
    268, 440, 0, 259, 91, 268, 0, 260, 91, 201, 444, 0, 392, 394, 0, 612,
    443, 613, 99, 441, 0, 614, 613, 99, 0, 488, 614, 442, 0, 615, 613, 99,
    0, 488, 615, 0, 173, 616, 618, 0, 620, 276, 613, 99, 0, 455, 276, 613,
    99, 0, 621, 99, 0, 488, 622, 0, 563, 443, 443, 623, 417, 0, 252, 173,
    356, 618, 450, 445, 453, 0, 260, 91, 449, 287, 263, 265, 0, 259, 91, 618,
    0, 267, 49, 268, 0, 252, 456, 356, 625, 454, 0, 260, 91, 452, 451, 287,
    313, 261, 263, 265, 0, 259, 91, 625, 0, 267, 453, 0, 49, 268, 0, 456,
    99, 0, 252, 627, 629, 462, 0, 281, 515, 0, 455, 267, 349, 345, 571, 456,
    573, 0, 276, 575, 0, 633, 571, 0, 276, 575, 482, 483, 0, 267, 575, 0,
    634, 458, 287, 635, 343, 0, 260, 91, 461, 80, 261, 263, 636, 0, 259, 91,
    629, 0, 252, 637, 469, 0, 267, 287, 463, 91, 289, 467, 291, 465, 464, 293,
    312, 313, 611, 0, 317, 468, 61, 287, 501, 0, 349, 414, 416, 417, 394, 0,
    420, 421, 470, 425, 0, 345, 414, 356, 417, 335, 0, 252, 414, 417, 489, 0,
    0, 267, 58, 53, 394, 0, 420, 421, 0, 400, 474, 343, 0, 267, 291, 476,
    475, 293, 205, 428, 343, 0, 345, 291, 0, 477, 267, 345, 346, 348, 0, 345,
    436, 356, 0, 349, 438, 478, 442, 0, 443, 291, 392, 444, 483, 479, 453, 0,
    487, 0, 267, 438, 486, 442, 0, 443, 291, 0, 267, 453, 0, 455, 0, 276,
    99, 0, 345, 444, 356, 0, 267, 456, 99, 0, 267, 458, 0, 317, 458, 0,
    267, 460, 488, 465, 176, 466, 467, 0, 267, 468, 287, 0, 349, 470, 0, 349,
    473, 0, 252, 351, 287, 490, 475, 0, 346, 475, 491, 0, 477, 478, 0, 477,
    480, 0, 358, 351, 287, 0, 345, 351, 356, 287, 335, 0, 492, 482, 483, 157,
    493, 287, 61, 127, 0, 482, 494, 484, 442, 157, 287, 0, 495, 482, 483, 0,
    482, 484, 0, 482, 485, 485, 287, 0, 496, 483, 482, 487, 0, 488, 487, 489,
    491, 493, 0, 495, 165, 287, 0, 267, 496, 497, 495, 499, 0, 267, 52, 285,
    0, 267, 498, 502, 503, 0, 267, 458, 0, 317, 458, 0, 488, 504, 245, 0,
    505, 287, 0, 346, 505, 0, 260, 91, 499, 287, 261, 263, 265, 0, 484, 444,
    500, 291, 0, 400, 508, 510, 512, 513, 0, 345, 414, 356, 234, 507, 0, 287,
    289, 502, 293, 312, 0, 503, 349, 148, 414, 639, 505, 0, 504, 642, 643, 417,
    162, 644, 0, 259, 506, 392, 91, 245, 0, 252, 167, 515, 0, 356, 516, 245,
    254, 510, 508, 645, 0, 259, 414, 417, 91, 417, 0, 259, 351, 287, 91, 287,
    0, 356, 245, 512, 0, 259, 91, 245, 0, 267, 49, 268, 0, 560, 561, 287,
    511, 0, 329, 647, 649, 291, 0, 651, 561, 287, 652, 654, 0, 61, 64, 52,
    285, 0, 563, 443, 513, 80, 267, 52, 285, 564, 0, 252, 380, 381, 387, 80,
    529, 0, 349, 656, 329, 516, 80, 276, 518, 80, 0, 252, 656, 329, 520, 80,
    276, 522, 80, 0, 252, 656, 658, 80, 276, 80, 0, 252, 656, 329, 524, 80,
    658, 80, 276, 80, 0, 345, 656, 356, 329, 526, 80, 0, 259, 91, 80, 0,
    260, 91, 528, 80, 261, 263, 265, 0, 267, 49, 80, 0, 252, 157, 660, 165,
    245, 530, 0, 259, 91, 660, 0, 488, 558, 165, 359, 361, 0, 252, 53, 394,
    536, 0, 260, 91, 531, 263, 265, 261, 80, 0, 259, 662, 515, 91, 515, 0,
    259, 91, 80, 0, 267, 49, 268, 533, 532, 0, 349, 664, 417, 394, 0, 420,
    421, 666, 0, 252, 664, 417, 0, 267, 58, 53, 394, 0, 420, 421, 666, 0,
    345, 664, 356, 417, 666, 0, 259, 664, 417, 91, 417, 666, 0, 349, 662, 416,
    515, 0, 535, 668, 643, 515, 0, 252, 168, 356, 245, 550, 0, 252, 414, 417,
    538, 0, 267, 291, 537, 293, 205, 428, 343, 0, 345, 291, 0, 260, 91, 539,
    80, 261, 263, 265, 0, 259, 414, 417, 91, 417, 0, 259, 91, 245, 0, 267,
    49, 268, 0, 267, 542, 540, 0, 566, 0, 289, 293, 0, 317, 547, 545, 566,
    551, 561, 570, 580, 0, 590, 673, 674, 597, 594, 592, 0, 591, 677, 678, 278,
    677, 603, 0, 245, 600, 598, 417, 605, 615, 624, 634, 644, 654, 664, 692, 219,
    693, 694, 695, 696, 697, 698, 699, 700, 701, 686, 667, 702, 99, 343, 0, 665,
    702, 99, 343, 703, 666, 702, 99, 343, 668, 677, 0, 689, 701, 699, 707, 708,
    704, 343, 0, 231, 234, 0, 392, 707, 717, 727, 737, 747, 757, 767, 777, 787,
    797, 807, 817, 0, 719, 827, 837, 847, 857, 879, 890, 725, 889, 888, 293, 428,
    728, 205, 417, 343, 0, 394, 293, 89, 392, 730, 732, 891, 897, 902, 912, 922,
    932, 942, 952, 962, 972, 750, 751, 752, 753, 754, 973, 980, 990, 1000, 758, 0,
    759, 759, 220, 0, 760, 760, 1010, 1019, 1029, 762, 763, 1030, 1040, 765, 766, 1041,
    1051, 1061, 1073, 1071, 293, 1078, 1076, 417, 1081, 1087, 772, 1090, 1100, 1110, 1120, 1130,
    1140, 1150, 1160, 1170, 235, 0, 234, 784, 392, 0, 346, 175, 1173, 786, 1177, 0,
    381, 1176, 788, 789, 642, 443, 417, 267, 417, 0, 622, 417, 267, 417, 400, 417,
    1178, 1188, 0, 791, 205, 1198, 1208, 1218, 333, 331, 1219, 1237, 0, 794, 795, 1229,
    798, 800, 802, 804, 802, 802, 0, 794, 343, 1233, 798, 800, 802, 804, 802, 802,
    753, 89, 806, 1238, 1248, 1258, 1267, 1277, 1287, 0, 1297, 1307, 1317, 1327, 1337, 1347,
    1357, 1367, 1377, 1387, 1397, 1407, 1417, 0, 495, 677, 1427, 0, 245, 0, 400, 510,
    0, 510, 165, 245, 1428, 1438, 1448, 0, 1454, 1464, 1476, 515, 1474, 234, 823, 824,
    1477, 1486, 1495, 1504, 829, 293, 205, 428, 0, 1509, 0, 346, 348, 0, 353, 1505,
    343, 0, 291, 1508, 293, 343, 831, 0, 832, 833, 835, 0, 420, 421, 0, 234,
    394, 1510, 425, 0, 420, 421, 0, 1514, 1524, 0, 234, 0, 1534, 836, 0, 836,
    836, 1536, 837, 1535, 836, 0, 836, 836, 165, 392, 400, 89, 838, 839, 840, 806,
    841, 842, 843, 0, 417, 695, 53, 0, 1539, 1538, 844, 846, 707, 848, 0, 851,
    852, 0, 855, 856, 1542, 1552, 0, 859, 515, 1561, 0, 859, 165, 0, 1580, 0,
    1571, 167, 168, 245, 0, 391, 1572, 0, 80, 392, 80, 0, 414, 417, 0, 351,
    80, 0, 165, 46, 80, 0, 45, 270, 0, 359, 361, 0, 170, 82, 0, 84,
    0, 508, 510, 0, 49, 268, 0, 1575, 376, 377, 46, 48, 130, 571, 51, 485,
    173, 57, 456, 627, 52, 157, 53, 0, 373, 374, 0, 378, 157, 0, 54, 58,
    59, 0, 54, 167, 0, 175, 168, 0, 380, 381, 387, 0, 380, 381, 388, 0,
    380, 381, 389, 0, 380, 381, 390, 80, 0, 165, 80, 0, 1576, 253, 171, 172,
    80, 1577, 0, 0, 0, 860, 1581, 515, 348, 1582, 1589, 1599, 1609, 1619, 1629, 289,
    867, 868, 869, 870, 871, 872, 873, 874, 875, 1630, 1637, 1647, 1653, 0, 1659, 343,
    836, 254, 1662, 1672, 1682, 1692, 1702, 0, 220, 220, 1712, 0, 1722, 884, 888, 889,
    1723, 0, 892, 1756, 513, 0, 159, 267, 1754, 1753, 0, 417, 289, 1733, 343, 291,
    0, 1736, 1734, 417, 289, 489, 1742, 1740, 1739, 343, 291, 0, 1747, 1745, 417, 289,
    1750, 786, 0, 894, 343, 1757, 1776, 0, 1773, 1771, 0, 1770, 417, 1767, 343, 205,
    0, 420, 421, 896, 0, 894, 343, 0, 165, 351, 287, 1777, 1787, 1797, 1807, 1817,
    1827, 1837, 1847, 1857, 0, 903, 1914, 0, 1875, 0, 245, 1869, 1867, 417, 1872, 786,
    91, 515, 0, 276, 1886, 1884, 1883, 0, 178, 234, 0, 911, 278, 0, 912, 278,
    0, 913, 278, 0, 914, 1881, 1878, 1876, 417, 417, 0, 915, 278, 0, 468, 1882,
    515, 234, 0, 916, 99, 0, 917, 99, 0, 918, 99, 0, 919, 515, 0, 93,
    515, 0, 276, 1911, 1909, 1905, 0, 178, 234, 0, 929, 278, 0, 930, 515, 0,
    348, 515, 0, 931, 278, 0, 932, 515, 0, 933, 515, 0, 934, 1894, 1891, 1889,
    417, 254, 0, 935, 1897, 1895, 417, 0, 936, 1902, 1900, 417, 0, 937, 515, 1915,
    1925, 1935, 0, 125, 940, 253, 80, 1945, 0, 84, 0, 125, 391, 1948, 394, 392,
    394, 1951, 0, 276, 170, 82, 84, 0, 496, 170, 0, 276, 941, 1952, 0, 392,
    943, 0, 392, 944, 0, 125, 376, 416, 80, 1960, 1957, 1955, 1954, 0, 952, 289,
    515, 0, 953, 289, 515, 0, 954, 289, 515, 0, 955, 289, 1953, 958, 959, 0,
    960, 289, 278, 0, 961, 289, 515, 0, 93, 80, 0, 125, 45, 270, 276, 1968,
    0, 260, 289, 80, 0, 390, 289, 80, 0, 937, 289, 1961, 515, 291, 1966, 0,
    952, 289, 515, 1962, 0, 953, 289, 515, 0, 954, 289, 515, 0, 52, 289, 1967,
    285, 291, 0, 277, 289, 278, 0, 281, 282, 289, 99, 0, 284, 289, 278, 0,
    125, 46, 80, 0, 392, 394, 0, 420, 421, 0, 291, 343, 1973, 0, 0, 351,
    80, 1972, 0, 346, 348, 0, 353, 343, 0, 125, 359, 416, 361, 276, 0, 49,
    268, 0, 961, 962, 0, 93, 962, 333, 0, 125, 964, 234, 965, 234, 967, 234,
    0, 125, 54, 167, 2003, 0, 969, 416, 245, 1984, 1982, 1981, 0, 417, 394, 970,
    0, 420, 421, 1977, 971, 972, 0, 974, 1989, 1987, 245, 0, 57, 975, 970, 0,
    969, 416, 245, 0, 561, 505, 245, 1999, 1997, 1996, 0, 417, 0, 276, 514, 1992,
    971, 972, 2002, 0, 329, 647, 649, 291, 0, 57, 975, 970, 0, 125, 940, 977,
    2004, 979, 170, 82, 84, 392, 2005, 0, 167, 786, 343, 0, 125, 831, 508, 652,
    0, 416, 510, 165, 516, 245, 0, 400, 402, 2008, 2006, 980, 0, 983, 2013, 2011,
    980, 0, 985, 346, 986, 0, 276, 458, 0, 52, 285, 0, 894, 343, 2016, 0,
    125, 940, 175, 168, 416, 245, 162, 0, 400, 287, 0, 276, 458, 0, 52, 285,
    392, 2038, 2029, 2026, 786, 2033, 2030, 0, 167, 245, 988, 2037, 2034, 0, 132, 205,
    990, 0, 125, 940, 992, 416, 80, 0, 514, 2047, 2045, 0, 287, 289, 2044, 293,
    2041, 2039, 515, 392, 786, 0, 125, 393, 2060, 2050, 2098, 2096, 2095, 0, 2064, 1001,
    1002, 289, 80, 0, 1003, 289, 2075, 2065, 0, 1004, 289, 2089, 2079, 0, 2094, 331,
    1006, 2093, 171, 170, 289, 82, 1007, 1008, 0, 125, 130, 80, 165, 245, 0, 392,
    2101, 1011, 1012, 0, 329, 2102, 61, 154, 147, 159, 146, 0, 91, 2104, 2103, 80,
    207, 261, 263, 265, 0, 400, 2106, 343, 0, 276, 353, 2109, 343, 0, 125, 940,
    171, 82, 84, 1013, 0, 125, 571, 573, 2114, 0, 329, 61, 180, 0, 329, 2112,
    577, 0, 276, 575, 0, 125, 2115, 51, 127, 201, 0, 276, 2124, 2116, 594, 595,
    2117, 596, 597, 2118, 598, 599, 2119, 504, 600, 2120, 601, 602, 2121, 603, 604, 2122,
    605, 606, 0, 281, 282, 99, 0, 607, 608, 2123, 515, 348, 0, 354, 609, 515,
    0, 64, 51, 201, 0, 64, 198, 201, 0, 51, 201, 0, 1018, 201, 0, 127,
    201, 0, 1019, 99, 0, 125, 49, 416, 2128, 0, 268, 1020, 201, 268, 2133, 0,
    392, 394, 0, 612, 443, 613, 99, 2129, 0, 614, 613, 99, 0, 488, 614, 2130,
    0, 615, 613, 99, 0, 488, 615, 0, 620, 276, 613, 99, 0, 621, 99, 2131,
    622, 0, 488, 622, 0, 563, 443, 2132, 623, 417, 0, 125, 977, 173, 416, 618,
    2134, 1021, 0, 125, 57, 416, 975, 0, 53, 515, 0, 961, 962, 964, 80, 970,
    0, 125, 456, 0, 416, 625, 2141, 2139, 2138, 1026, 1027, 1028, 1029, 165, 2145, 2144,
    417, 343, 93, 245, 0, 125, 627, 629, 281, 515, 571, 2147, 573, 0, 276, 575,
    0, 0, 125, 940, 977, 167, 416, 245, 2169, 0, 392, 2152, 2149, 786, 2166, 2164,
    2163, 0, 417, 2162, 0, 394, 2154, 2153, 425, 0, 394, 392, 0, 438, 442, 392,
    2158, 343, 2161, 1032, 1033, 470, 0, 125, 380, 381, 387, 80, 2171, 2170, 0, 389,
    289, 80, 0, 903, 289, 80, 0, 125, 940, 351, 157, 660, 2174, 668, 643, 0,
    1035, 505, 2178, 2177, 147, 146, 158, 0, 159, 0, 505, 2175, 417, 1036, 165, 1036,
    165, 245, 2188, 0, 93, 245, 2181, 0, 346, 475, 0, 475, 2180, 0, 477, 480,
    0, 477, 478, 0, 1038, 2182, 1041, 1042, 167, 392, 245, 0, 2183, 1041, 1042, 167,
    392, 245, 0, 329, 1043, 2184, 489, 1046, 0, 1047, 2185, 343, 0, 132, 2192, 170,
    171, 205, 0, 125, 53, 394, 392, 2198, 394, 0, 1049, 2195, 2193, 515, 1050, 0,
    1054, 1055, 0, 125, 127, 656, 0, 1057, 416, 329, 2199, 207, 234, 1058, 57, 975,
    970, 0, 125, 127, 201, 0, 125, 940, 977, 168, 416, 245, 162, 392, 2204, 2203,
    2200, 786, 988, 2205, 2215, 2225, 2235, 2245, 2255, 2263, 0, 2273, 1065, 1066, 2274, 1067,
    2277, 2287, 2297, 2307, 2317, 2327, 2337, 2347, 2357, 2367, 2377, 2387, 2397, 2407, 234, 823,
    824, 2409, 2428, 2419, 234, 312, 0, 0, 234, 836, 2440, 1073, 1110, 1050, 1112, 0,
    2439, 1154, 1155, 1156, 1157, 1158, 1159, 1160, 1161, 1162, 1163, 0, 1164, 1165, 1167, 1168,
    1169, 1170, 1171, 1172, 0, 2429, 1173, 1174, 0, 2430, 1176, 1177, 1174, 1178, 2432, 0,
    2431, 1179, 1180, 0, 1180, 1181, 1183, 0, 1183, 1181, 1184, 1174, 380, 1185, 2433, 1186,
    1187, 2434, 1188, 1189, 1190, 1191, 1192, 1193, 1194, 2435, 1195, 1196, 1197, 1198, 2436, 1199,
    1200, 0, 1201, 2437, 1181, 1174, 1202, 1203, 2438, 1204, 1205, 1206, 1207, 1208, 1209, 1210,
    1211, 1212, 0, 1213, 1174, 0, 1214, 1174, 2445, 2441, 1215, 1217, 1219, 2446, 2456, 1221,
    0, 2468, 2466, 2465, 1222, 1256, 0, 1110, 515, 2475, 1257, 1258, 0, 2471, 1259, 1260,
    1261, 2472, 99, 2476, 2486, 2529, 2538, 2548, 2558, 2568, 2578, 2588, 2598, 2608, 2618, 2628,
    2638, 0, 1272, 1273, 2648, 80, 61, 2649, 2659, 2669, 2679, 0, 291, 647, 2689, 2699,
    2709, 2719, 2729, 293, 1278, 0, 1278, 0, 289, 1279, 2732, 2730, 1280, 2735, 2745, 2755,
    0, 146, 93, 516, 245, 254, 247, 0, 400, 2765, 0, 1284, 247, 1286, 2767, 0,
    894, 315, 505, 80, 1288, 0, 1291, 2770, 254, 2768, 0, 343, 247, 1292, 2771, 2781,
    2791, 2801, 2811, 2821, 2831, 2841, 0, 1297, 312, 80, 2851, 2861, 2871, 2881, 2891, 2901,
    2911, 2921, 0, 1302, 2931, 61, 1304, 186, 156, 155, 2932, 2942, 2952, 2962, 2972, 3000,
    2982, 2991, 3001, 0, 892, 3011, 0, 1309, 515, 0, 515, 1309, 3012, 3022, 3033, 3043,
    3052, 0, 345, 253, 356, 3063, 0, 80, 3062, 84, 0, 254, 335, 3065, 331, 333,
    0, 345, 391, 356, 3066, 394, 392, 394, 335, 0, 345, 376, 356, 80, 335, 0,
    345, 45, 356, 270, 0, 276, 3069, 487, 0, 345, 46, 356, 3072, 80, 335, 0,
    345, 359, 356, 361, 335, 0, 345, 54, 167, 356, 3074, 245, 666, 0, 345, 3076,
    979, 170, 167, 356, 82, 335, 0, 345, 508, 652, 356, 3077, 510, 335, 3079, 0,
    345, 175, 168, 356, 3089, 245, 335, 0, 345, 992, 356, 80, 0, 345, 563, 443,
    3092, 3091, 261, 263, 265, 201, 335, 0, 345, 130, 356, 80, 165, 245, 335, 0,
    345, 171, 356, 3094, 0, 82, 84, 3096, 333, 331, 0, 345, 571, 356, 3097, 573,
    335, 0, 345, 51, 356, 234, 0, 345, 49, 356, 268, 335, 0, 345, 173, 356,
    3099, 618, 335, 0, 345, 456, 356, 3101, 625, 3103, 333, 331, 0, 345, 627, 356,
    629, 3104, 333, 331, 0, 345, 977, 167, 356, 3105, 245, 335, 0, 345, 380, 381,
    387, 356, 80, 335, 0, 345, 157, 356, 660, 165, 245, 335, 0, 345, 53, 356,
    3107, 394, 335, 0, 345, 127, 356, 201, 0, 345, 168, 356, 3109, 245, 335, 3111,
    3121, 3131, 0, 800, 343, 3141, 3151, 3161, 3171, 0, 1050, 1319, 3174, 3184, 3194, 3204,
    3214, 3223, 3233, 635, 3243, 3252, 3262, 3272, 3282, 3292, 3302, 3312, 0, 980, 276, 753,
    3322, 3332, 0, 132, 80, 3344, 3342, 343, 3347, 3357, 3367, 3377, 3389, 0, 3387, 673,
    674, 677, 1337, 1338, 1339, 1340, 1341, 1342, 278, 0, 178, 3388, 380, 1203, 1204, 1344,
    0, 1345, 3396, 0, 3390, 673, 674, 677, 3393, 3391, 1346, 3397, 786, 1348, 1350, 1352,
    3398, 1354, 0, 1355, 3409, 3408, 1356, 0, 806, 1355, 1361, 1362, 0, 860, 346, 1363,
    1371, 1373, 1375, 0, 346, 1376, 1377, 1378, 1355, 0, 1379, 1354, 3413, 613, 0, 702,
    1382, 1383, 1384, 1385, 0, 1386, 3416, 3415, 0, 3414, 750, 751, 753, 1386, 3420, 613,
    0, 702, 1382, 0, 0, 1387, 3430, 3429, 0, 1389, 3421, 786, 0, 3424, 1390, 1391,
    3425, 1393, 829, 89, 0, 1395, 829, 1397, 871, 0, 3445, 428, 205, 3435, 3434, 1399,
    343, 786, 3432, 3431, 293, 417, 205, 1400, 343, 1402, 1403, 0, 3438, 1050, 1112, 3441,
    3439, 343, 0, 394, 3444, 515, 99, 278, 1404, 1405, 1400, 1407, 89, 1408, 3446, 1409,
    293, 728, 1411, 1413, 417, 1415, 3447, 0, 3461, 245, 0, 516, 3460, 3457, 245, 0,
    245, 254, 3462, 3472, 3482, 3492, 3503, 3513, 3524, 0, 1422, 3534, 642, 1424, 3535, 99,
    343, 489, 3536, 489, 553, 3537, 516, 0, 276, 1425, 3538, 3548, 3558, 3574, 1428, 1431,
    1434, 3572, 1437, 3568, 842, 3578, 0, 1441, 3588, 0, 894, 343, 3591, 3601, 3611, 3621,
    3631, 3641, 3651, 3661, 0, 329, 3671, 159, 0, 488, 833, 159, 204, 0, 833, 204,
    0, 505, 3672, 245, 3674, 564, 0, 634, 1444, 3675, 3685, 3695, 3705, 3715, 0, 54,
    58, 59, 0, 54, 833, 3725, 0, 0, 351, 80, 3739, 0, 346, 348, 0, 353,
    3735, 343, 0, 488, 504, 0, 291, 3738, 829, 293, 205, 428, 343, 0, 438, 442,
    392, 343, 1032, 0, 0, 351, 80, 0, 353, 3740, 343, 0, 488, 504, 3743, 3753,
    0, 1449, 3766, 3765, 0, 315, 489, 0, 3763, 99, 728, 1450, 417, 3764, 1453, 1454,
    0, 1376, 1378, 0, 1455, 3767, 0, 315, 489, 198, 1425, 0, 488, 1456, 3768, 1458,
    553, 1459, 3769, 3779, 3789, 0, 93, 3799, 1462, 3801, 894, 1469, 0, 198, 443, 0,
    1471, 443, 1472, 1473, 1474, 1475, 1477, 990, 1422, 1479, 1480, 3814, 0, 1482, 3805, 3802,
    1284, 1484, 247, 3806, 1486, 1487, 843, 1286, 0, 276, 1479, 247, 1487, 1490, 3811, 0,
    3807, 1286, 3833, 3830, 0, 3818, 1491, 1494, 3815, 1462, 0, 1471, 443, 0, 198, 443,
    3819, 3823, 0, 3822, 1286, 843, 0, 1471, 443, 0, 198, 443, 3827, 3834, 0, 1497,
    3855, 3844, 0, 205, 247, 329, 3851, 0, 234, 64, 3848, 3846, 293, 1498, 225, 1469,
    0, 1500, 0, 983, 985, 3880, 3863, 234, 3860, 3858, 234, 329, 3876, 0, 234, 64,
    3873, 3871, 3870, 0, 3869, 3866, 3864, 234, 247, 1501, 3883, 3893, 3904, 3914, 846, 1503,
    343, 3932, 343, 3918, 3915, 772, 0, 343, 392, 394, 0, 1507, 343, 93, 93, 343,
    0, 3919, 1222, 343, 3920, 0, 93, 343, 0, 329, 343, 0, 1291, 394, 0, 986,
    3926, 254, 3924, 1508, 1509, 0, 1512, 293, 0, 3927, 515, 234, 417, 343, 64, 3928,
    515, 234, 417, 343, 1513, 1516, 1518, 3930, 0, 343, 3929, 662, 841, 343, 0, 3936,
    1256, 0, 3954, 1309, 0, 404, 329, 53, 287, 1474, 3939, 537, 538, 539, 0, 346,
    540, 3940, 0, 532, 165, 348, 533, 0, 535, 348, 165, 348, 533, 536, 0, 176,
    493, 3941, 544, 543, 0, 545, 3942, 549, 550, 551, 0, 552, 99, 0, 553, 99,
    0, 554, 287, 0, 267, 287, 3947, 0, 3943, 91, 289, 3945, 3944, 287, 293, 0,
    93, 315, 0, 392, 3948, 515, 0, 515, 89, 515, 0, 1519, 343, 0, 1520, 1521,
    3950, 0, 1348, 1292, 0, 1350, 1292, 0, 3949, 1522, 772, 1409, 1292, 0, 1519, 343,
    1292, 802, 0, 276, 3960, 3958, 287, 3963, 3973, 0, 3982, 0, 234, 836, 771, 3986,
    1524, 313, 771, 0, 3987, 64, 1526, 941, 1527, 3990, 394, 0, 287, 3988, 64, 1526,
    941, 1527, 3989, 394, 1528, 0, 3991, 291, 289, 1387, 343, 3994, 3992, 1530, 3998, 0,
    1531, 1532, 1501, 0, 0, 82, 3997, 1531, 851, 988, 1533, 1534, 3999, 4009, 4019, 4029,
    4039, 4049, 4059, 4069, 4079, 4089, 4099, 4110, 4120, 0, 318, 4130, 0, 1536, 165, 1538,
    0, 51, 201, 0, 150, 165, 127, 199, 80, 91, 1540, 4131, 0, 276, 318, 326,
    0, 276, 1018, 326, 0, 903, 315, 135, 0, 1543, 443, 4132, 263, 265, 199, 4133,
    4143, 0, 760, 635, 760, 4153, 0, 198, 443, 4164, 4163, 61, 417, 99, 343, 1545,
    4166, 0, 1471, 443, 1469, 1472, 1473, 1474, 1422, 4167, 0, 4181, 4180, 417, 99, 343,
    4177, 1545, 4183, 0, 1547, 1548, 4196, 4194, 4193, 1550, 1067, 4199, 4209, 4219, 4229, 0,
    1472, 4242, 4239, 343, 4243, 0, 1471, 443, 1469, 1473, 1474, 1422, 4244, 4254, 4264, 4274,
    4284, 4294, 1557, 4304, 4314, 4324, 4334, 4344, 4354, 4364, 4374, 4384, 4391, 4401, 4411, 0,
    1569, 1389, 4421, 0, 1569, 346, 1389, 4431, 0, 4441, 1570, 1571, 985, 4442, 4452, 4462,
    4472, 0, 1573, 54, 49, 0, 1574, 268, 0, 4482, 0, 282, 91, 1575, 4485, 4483,
    1576, 93, 57, 975, 1577, 268, 970, 4488, 4498, 4508, 0, 346, 64, 4524, 4521, 4520,
    4518, 1354, 786, 205, 1579, 725, 515, 417, 4525, 4535, 4545, 234, 0, 234, 4555, 1582,
    1583, 0, 0, 420, 421, 0, 1584, 0, 985, 4556, 642, 1586, 458, 4557, 1582, 1583,
    0, 985, 4558, 642, 1586, 0, 4562, 417, 205, 4559, 343, 1588, 4563, 0, 0, 983,
    162, 0, 276, 575, 0, 400, 508, 52, 285, 4573, 4583, 4593, 4603, 4613, 4623, 4633,
    4643, 4653, 4663, 4673, 4683, 4693, 4703, 4713, 0, 147, 4723, 940, 1597, 1577, 245, 1598,
    4724, 162, 0, 443, 1599, 0, 443, 616, 4730, 0, 291, 647, 786, 0, 162, 4729,
    988, 4728, 4725, 786, 0, 165, 1601, 1602, 1604, 0, 1291, 4733, 254, 4731, 0, 343,
    1598, 4734, 4744, 4754, 4764, 4774, 4784, 4794, 4804, 4814, 4824, 4834, 4844, 4854, 4864, 0,
    1259, 4876, 0, 99, 4874, 515, 1222, 515, 0, 515, 4875, 515, 1222, 0, 343, 1222,
    4877, 4887, 1610, 1611, 1612, 1613, 1614, 1615, 0, 1577, 4888, 156, 155, 503, 167, 245,
    4889, 4899, 4909, 1404, 1616, 1617, 278, 1618, 1619, 0, 860, 346, 986, 93, 4910, 1620,
    4920, 4930, 4940, 4950, 4960, 4970, 4991, 0, 1625, 1626, 1494, 4980, 1627, 0, 4984, 1629,
    4987, 1631, 1633, 4988, 0, 1635, 1626, 1494, 1629, 0, 1636, 1494, 4992, 0, 1006, 0,
    5002, 1637, 1639, 247, 0, 165, 5006, 5012, 5009, 343, 5013, 5017, 1642, 0, 5016, 512,
    1643, 1644, 1645, 0, 400, 5020, 5018, 234, 5023, 5033, 0, 1204, 1647, 1649, 5044, 662,
    1650, 708, 97, 0, 5045, 276, 496, 831, 0, 1652, 5046, 5056, 5066, 5076, 5084, 5094,
    5104, 5125, 0, 5118, 287, 5115, 5113, 287, 1657, 343, 0, 1659, 5124, 287, 5121, 5119,
    287, 841, 343, 5126, 5136, 0, 48, 5146, 312, 240, 5147, 5157, 5167, 5177, 0, 5187,
    5197, 5207, 5217, 5227, 5237, 5247, 5257, 5267, 5277, 0, 759, 635, 759, 5287, 5297, 0,
    0, 346, 1665, 1354, 0, 933, 1355, 5307, 1666, 1667, 0, 1668, 91, 1669, 5308, 5318,
    0, 5329, 1670, 1671, 5330, 1672, 1673, 1674, 438, 444, 1675, 456, 460, 61, 0, 282,
    5335, 5332, 5331, 99, 343, 61, 5337, 0, 1479, 5336, 99, 343, 0, 89, 99, 5338,
    5348, 5358, 5368, 5378, 5388, 5398, 5408, 5418, 343, 329, 287, 64, 343, 0, 1569, 343,
    1678, 5421, 5431, 0, 1682, 234, 5441, 515, 99, 278, 702, 1404, 1405, 1683, 1684, 1579,
    1686, 1688, 5442, 5451, 5458, 0, 1691, 515, 0, 5468, 5478, 5488, 5498, 5508, 5518, 0,
    1695, 167, 516, 5530, 5528, 245, 254, 0, 64, 5531, 0, 373, 204, 0, 489, 204,
    0, 489, 1696, 0, 204, 159, 1696, 204, 0, 204, 489, 1696, 1696, 0, 373, 1696,
    1697, 564, 5532, 5542, 5552, 5562, 5572, 5582, 5592, 5602, 5612, 5622, 5632, 0, 1706, 836,
    234, 5645, 0, 992, 80, 0, 89, 5642, 786, 5648, 5658, 5668, 5678, 5688, 5698, 0,
    707, 841, 707, 0, 1707, 5719, 5708, 1708, 5710, 5724, 5722, 394, 0, 1707, 1711, 5727,
    5737, 0, 5747, 5757, 5767, 5777, 5787, 5797, 5807, 5817, 5827, 5837, 146, 0, 147, 162,
    988, 0, 1721, 1577, 5847, 5857, 1722, 1724, 1726, 0, 1047, 1728, 0, 1378, 343, 1729,
    5861, 1731, 1733, 0, 892, 513, 0, 1047, 346, 1728, 443, 1735, 0, 1378, 343, 1729,
    5862, 1731, 1733, 0, 892, 513, 0, 1047, 346, 1728, 0, 443, 1737, 0, 1378, 343,
    1729, 5863, 1739, 0, 892, 513, 0, 1741, 5864, 245, 1742, 400, 5868, 245, 1742, 0,
    5865, 786, 247, 0, 205, 247, 5869, 1631, 5872, 1743, 0, 159, 1745, 5875, 5885, 0,
    5896, 0, 786, 1749, 5900, 5910, 5920, 5930, 5940, 5950, 5959, 5969, 5979, 5989, 5999, 6009,
    6019, 6029, 6039, 6049, 6058, 6068, 6078, 6088, 6098, 6108, 6118, 6128, 1758, 6132, 6141, 6151,
    6161, 6171, 6181, 6190, 6200, 6210, 6220, 6230, 6240, 6250, 6260, 6270, 6280, 6290, 6301, 6311,
    0, 312, 6321, 1764, 1387, 289, 343, 0, 234, 392, 6325, 234, 6322, 1765, 0, 1474,
    6326, 1767, 6328, 6338, 6348, 6359, 0, 1772, 884, 6369, 6379, 6389, 6399, 6408, 0, 6418,
    6428, 6435, 6445, 6455, 6465, 6475, 6485, 6495, 6505, 6515, 6525, 6535, 6545, 6555, 6565, 6575,
    6585, 6595, 6605, 6624, 988, 1781, 6615, 1409, 6618, 1522, 6621, 1782, 1783, 1784, 1786, 6626,
    1788, 1789, 0, 1790, 6625, 884, 1788, 1789, 6627, 6632, 772, 6631, 6628, 1409, 1782, 1350,
    1348, 1352, 6633, 6643, 6653, 6663, 6673, 0, 6683, 1794, 1795, 1796, 1797, 1798, 6684, 6694,
    0, 6704, 0, 1799, 635, 0, 759, 760, 0, 758, 759, 0, 758, 760, 6705, 1800,
    6715, 6725, 6735, 0, 1802, 234, 0, 89, 515, 6745, 6755, 6765, 6775, 6786, 6796, 6806,
    6816, 6827, 6837, 0, 6847, 234, 515, 841, 707, 6850, 6848, 1804, 6853, 836, 0, 836,
    836, 6854, 165, 392, 400, 89, 838, 839, 840, 806, 841, 842, 843, 6855, 6865, 6875,
    6885, 0, 1479, 6895, 99, 343, 6896, 6906, 6916, 6926, 6936, 6945, 6955, 6965, 6975, 6985,
    6995, 7005, 7015, 7025, 7035, 7045, 7055, 7065, 0, 514, 7079, 7075, 1818, 7082, 0, 7093,
    7092, 1576, 515, 0, 1036, 1570, 7095, 7105, 0, 1036, 658, 0, 1471, 443, 7117, 0,
    7114, 61, 417, 99, 343, 7115, 1582, 1583, 0, 985, 7116, 642, 1586, 1819, 7119, 282,
    1472, 1473, 1474, 1449, 1512, 0, 276, 58, 1480, 7120, 7130, 1471, 1820, 7131, 7141, 7151,
    7161, 7171, 0, 1513, 1823, 7184, 234, 7181, 1765, 7185, 0, 758, 758, 0, 7200, 0,
    7197, 7195, 417, 1405, 829, 1508, 89, 7196, 417, 1405, 829, 1508, 417, 1825, 7206, 0,
    7203, 7201, 417, 1405, 829, 1508, 89, 7202, 417, 1405, 829, 1508, 417, 7207, 7217, 7227,
    7237, 7247, 7257, 7267, 7277, 7287, 7297, 7307, 7316, 7325, 7335, 7345, 7355, 7385, 0, 64,
    7367, 7365, 343, 0, 93, 7373, 7371, 7370, 343, 614, 615, 91, 7379, 7377, 7376, 343,
    614, 615, 0, 276, 7382, 0, 1834, 99, 89, 1835, 99, 0, 561, 443, 7391, 7388,
    7386, 343, 7392, 7402, 7412, 7422, 7432, 7454, 0, 1838, 7452, 7442, 1838, 515, 0, 7455,
    7465, 7475, 7485, 7496, 7506, 7517, 7527, 7536, 7546, 7556, 7566, 7576, 7586, 7596, 7605, 7615,
    7625, 7635, 7645, 7655, 7665, 7675, 7685, 7695, 7705, 7715, 7725, 7734, 1847, 1850, 1852, 0,
    7738, 7748, 7759, 0, 1855, 7769, 7780, 1856, 7781, 7791, 7801, 7811, 7821, 0, 1273, 80,
    7833, 7831, 394, 392, 7836, 786, 1859, 7837, 7847, 0, 832, 833, 7857, 7867, 7877, 7887,
    7897, 7907, 7917, 7927, 7937, 7947, 7957, 7967, 7977, 7987, 7997, 0, 1870, 8007, 0, 1871,
    8017, 0, 8030, 8027, 0, 1873, 8026, 287, 515, 8031, 8041, 8058, 0, 167, 8051, 1874,
    0, 89, 8053, 167, 180, 0, 180, 64, 49, 8055, 8054, 268, 363, 0, 89, 8057,
    167, 180, 234, 0, 1876, 162, 0, 894, 8059, 343, 8062, 0, 613, 99, 0, 393,
    8086, 0, 312, 836, 8082, 8072, 0, 287, 0, 836, 287, 0, 1473, 8092, 8089, 343,
    8093, 0, 753, 8103, 61, 1880, 1881, 8104, 786, 8107, 8117, 8127, 8264, 0, 8160, 8138,
    8149, 8172, 1884, 8161, 0, 8176, 8197, 1884, 8187, 0, 8201, 8222, 1884, 8212, 0, 1886,
    8226, 8262, 8261, 8235, 8257, 1884, 8246, 8265, 8275, 8285, 8295, 8304, 8313, 8323, 8332, 8341,
    8351, 8361, 8371, 0, 1894, 563, 443, 8382, 8381, 261, 263, 265, 201, 91, 8384, 261,
    263, 265, 201, 8385, 8395, 8405, 8415, 0, 153, 245, 162, 1899, 8425, 0, 165, 146,
    1900, 0, 165, 159, 1900, 0, 1901, 8429, 512, 1903, 1904, 8430, 8440, 8450, 8466, 333,
    0, 267, 8460, 291, 348, 8463, 8461, 417, 331, 0, 488, 1905, 8467, 0, 633, 175,
    168, 652, 245, 990, 8477, 8487, 0, 1908, 8500, 8498, 8497, 0, 652, 278, 0, 52,
    285, 0, 677, 278, 8504, 0, 508, 652, 510, 0, 167, 652, 245, 0, 49, 652,
    268, 0, 8503, 45, 637, 652, 270, 0, 1278, 0, 836, 1278, 0, 289, 1279, 8507,
    8505, 1909, 8510, 8520, 8530, 8540, 8550, 8560, 8570, 8580, 8590, 8600, 8610, 8620, 8630, 0,
    317, 1918, 1020, 0, 317, 8640, 61, 51, 287, 8641, 8651, 8661, 8671, 8681, 8691, 8701,
    8711, 8721, 8731, 8741, 8751, 0, 328, 0, 318, 326, 329, 8761, 0, 1536, 165, 1538,
    0, 51, 201, 0, 150, 165, 127, 199, 199, 93, 1540, 335, 8762, 8771, 8781, 8791,
    8801, 234, 8811, 8821, 8831, 8839, 8849, 8859, 8869, 8879, 8889, 8899, 8909, 8919, 8929, 8939,
    8949, 8959, 0, 8984, 0, 1930, 8970, 8969, 1931, 1932, 0, 8973, 1933, 637, 8974, 1931,
    0, 1931, 8978, 8977, 1930, 1933, 637, 0, 89, 99, 0, 1932, 8981, 1930, 0, 89,
    99, 0, 1934, 8985, 99, 8988, 0, 99, 8998, 695, 1936, 0, 99, 553, 0, 1937,
    1938, 8999, 9009, 9019, 9029, 9039, 9049, 9059, 9069, 9079, 9089, 0, 493, 1943, 0, 329,
    80, 165, 9114, 0, 167, 245, 0, 414, 417, 0, 253, 82, 9099, 84, 0, 45,
    270, 0, 46, 80, 0, 378, 157, 80, 0, 54, 167, 245, 0, 170, 82, 9105,
    9102, 1530, 0, 96, 97, 99, 0, 175, 168, 245, 0, 379, 48, 80, 0, 171,
    82, 9109, 9106, 1530, 0, 571, 80, 0, 51, 201, 0, 172, 82, 9113, 9110, 1530,
    0, 49, 268, 0, 173, 80, 0, 627, 80, 0, 52, 80, 0, 53, 80, 0,
    168, 245, 860, 9115, 515, 348, 9116, 0, 844, 0, 707, 247, 9122, 0, 986, 0,
    165, 9119, 9117, 343, 61, 0, 154, 1944, 9123, 1946, 1577, 93, 894, 0, 1471, 443,
    0, 165, 1601, 282, 1291, 1825, 1475, 0, 276, 488, 58, 1948, 1749, 9135, 1577, 93,
    9125, 894, 0, 1471, 443, 282, 1291, 89, 1475, 1749, 9136, 0, 1950, 1952, 1288, 1498,
    1954, 1415, 1956, 1958, 225, 1960, 1962, 1469, 1956, 1964, 1475, 1477, 990, 0, 165, 1601,
    1291, 1948, 1749, 9157, 9149, 9146, 1522, 9153, 9150, 1965, 1966, 9154, 786, 9158, 0, 836,
    234, 211, 9168, 0, 836, 234, 211, 9172, 9181, 9191, 9201, 0, 615, 99, 0, 488,
    615, 9202, 0, 614, 99, 0, 488, 614, 9203, 9213, 9223, 9233, 9243, 9253, 9263, 9273,
    9283, 9293, 127, 261, 263, 265, 9294, 9304, 0, 267, 9314, 1971, 0, 9337, 0, 417,
    211, 289, 9316, 293, 428, 205, 417, 343, 291, 9317, 829, 0, 9323, 9321, 417, 289,
    9334, 9333, 786, 9331, 0, 9326, 293, 428, 205, 417, 343, 291, 9327, 829, 0, 267,
    1673, 9340, 61, 9338, 80, 9341, 478, 480, 0, 1782, 9342, 0, 1475, 1782, 225, 1469,
    1956, 9346, 9357, 1973, 0, 9356, 1974, 1575, 61, 1975, 0, 267, 49, 416, 268, 0,
    267, 9358, 0, 1976, 1918, 0, 1918, 1918, 1020, 9359, 201, 291, 0, 267, 9368, 0,
    1978, 312, 9360, 91, 289, 9361, 293, 312, 313, 0, 9362, 1918, 1976, 1979, 287, 9363,
    91, 289, 9367, 291, 9365, 9364, 293, 312, 313, 611, 9369, 9379, 9389, 9399, 9409, 9419,
    0, 9429, 1390, 1391, 9430, 0, 838, 394, 1393, 9434, 0, 1984, 9444, 61, 1985, 953,
    954, 1986, 1987, 287, 9445, 9455, 9465, 1990, 1991, 9466, 9476, 99, 9480, 0, 9490, 1497,
    1992, 1284, 0, 165, 9498, 9497, 9491, 417, 343, 0, 64, 9494, 9492, 293, 0, 400,
    9500, 0, 205, 247, 1498, 225, 1469, 0, 9502, 1500, 1993, 1284, 165, 9509, 0, 9508,
    343, 9505, 9503, 343, 247, 0, 1577, 616, 234, 662, 9511, 234, 225, 1469, 9513, 312,
    313, 1576, 9514, 312, 313, 836, 9515, 234, 0, 234, 823, 254, 9517, 0, 1217, 211,
    9528, 9538, 9548, 9557, 9566, 9576, 9586, 9596, 9606, 9616, 9626, 9636, 9646, 9656, 9666, 9676,
    9686, 9696, 9706, 9716, 9726, 9736, 9746, 9756, 9766, 9776, 9786, 9796, 9806, 9815, 9825, 9835,
    9845, 9855, 786, 1859, 1348, 2015, 2017, 2019, 2021, 2023, 2025, 2027, 2029, 2031, 2033, 2035,
    2037, 2039, 2041, 2043, 2045, 2047, 2049, 2051, 2053, 1352, 1350, 2055, 2057, 2059, 2061, 2063,
    2065, 2067, 2069, 2071, 2073, 2075, 2077, 2079, 2081, 2083, 2085, 2087, 2089, 2091, 2093, 2094,
    2096, 2098, 2100, 2102, 2104, 2106, 2108, 2110, 2112, 2114, 2116, 2118, 2120, 2047, 2122, 2125,
    2053, 2127, 2129, 2131, 2133, 2136, 2138, 2140, 2142, 2144, 2146, 2148, 2150, 2152, 2154, 2156,
    2158, 2160, 2162, 2164, 2166, 2168, 2170, 2172, 2174, 2176, 2178, 2180, 2182, 2184, 2186, 2188,
    2190, 2192, 2194, 2196, 2198, 2200, 2202, 2204, 2206, 2208, 2210, 2212, 2214, 2216, 2218, 2220,
    2222, 2224, 2226, 2228, 2230, 2232, 2234, 2236, 2238, 2240, 2242, 1784, 1786, 842, 9856, 9866,
    9876, 9886, 9896, 9906, 9916, 9926, 9936, 9946, 2247, 420, 9947, 9957, 9969, 9967, 0, 707,
    247, 9975, 9973, 0, 9972, 394, 0, 287, 394, 0, 2249, 1055, 9978, 9988, 9998, 10008,
    10018, 10028, 10038, 10048, 10055, 10065, 10075, 10085, 10095, 0, 0, 351, 80, 10116, 0, 353,
    10105, 343, 0, 488, 504, 0, 831, 0, 985, 346, 986, 162, 2256, 0, 2258, 162,
    2256, 0, 1455, 0, 400, 402, 10110, 10108, 2259, 2256, 0, 894, 10113, 343, 0, 54,
    833, 162, 835, 10119, 10117, 475, 0, 346, 475, 10118, 0, 477, 478, 0, 477, 480,
    0, 346, 354, 0, 488, 504, 0, 0, 351, 80, 0, 10123, 831, 2258, 400, 508,
    510, 10124, 475, 0, 346, 475, 10125, 0, 477, 478, 0, 477, 480, 0, 10132, 988,
    428, 0, 205, 0, 276, 2261, 245, 10126, 786, 10129, 1859, 10133, 10144, 10143, 234, 240,
    10146, 10156, 10166, 0, 10176, 2262, 806, 2263, 0, 10180, 2264, 2263, 10184, 10194, 10204, 10214,
    10224, 0, 10234, 155, 156, 10235, 10245, 2266, 977, 10246, 10256, 10266, 10276, 10286, 10296, 10306,
    10316, 10325, 0, 10335, 2272, 2273, 1174, 0, 10336, 276, 496, 2272, 2274, 10337, 0, 2276,
    2272, 2274, 343, 10341, 10351, 10361, 10371, 10381, 10391, 10401, 10411, 10421, 10431, 10441, 10451, 10461,
    10471, 0, 10481, 620, 1520, 2284, 2285, 802, 10482, 2287, 2288, 0, 616, 234, 0, 1378,
    488, 2289, 10483, 10493, 10503, 10513, 10523, 10533, 10543, 10553, 2293, 2294, 2295, 10554, 10564, 10575,
    0, 158, 167, 10586, 10585, 0, 516, 245, 0, 245, 254, 0, 10588, 455, 2296, 444,
    335, 10589, 10599, 10609, 10619, 10629, 10639, 10651, 10649, 707, 10654, 0, 0, 1527, 0, 312,
    1387, 1217, 1579, 0, 1050, 2299, 10664, 10674, 10684, 10694, 10704, 2302, 10714, 2303, 2304, 2305,
    10715, 10725, 0, 1054, 10735, 986, 61, 0, 443, 616, 10736, 0, 831, 10746, 10756, 10766,
    10777, 0, 2308, 10787, 234, 254, 10788, 10798, 0, 10808, 0, 1950, 1952, 0, 1952, 1950,
    1288, 1498, 1954, 1956, 1958, 1964, 1475, 225, 1469, 0, 165, 1601, 1291, 0, 1782, 10809,
    0, 1475, 1782, 10813, 10824, 10834, 10844, 10854, 10864, 0, 159, 516, 245, 247, 267, 1745,
    1952, 10874, 0, 894, 315, 505, 80, 1288, 0, 1291, 10877, 254, 10875, 0, 343, 247,
    10878, 10888, 10898, 0, 141, 270, 10908, 10918, 234, 10928, 10938, 0, 400, 2314, 1938, 10948,
    10958, 10968, 10978, 10988, 0, 2318, 11006, 0, 512, 929, 677, 10998, 673, 674, 11003, 11001,
    0, 10999, 512, 929, 677, 673, 674, 2319, 678, 2320, 2321, 158, 545, 11000, 293, 312,
    611, 11007, 0, 245, 162, 11009, 11019, 11029, 11039, 0, 647, 11055, 11052, 11050, 11049, 343,
    291, 247, 225, 1469, 11057, 11067, 11077, 11087, 11097, 11107, 11117, 11127, 11137, 11147, 11157, 515,
    312, 11158, 11168, 11178, 11188, 11198, 11208, 11218, 11228, 11237, 11247, 11257, 11267, 11305, 0, 11277,
    1188, 2328, 1193, 2329, 2330, 2331, 2332, 2333, 2334, 2335, 2336, 2337, 2338, 2339, 2340, 2341,
    2342, 2343, 2344, 2345, 2346, 2347, 2348, 2349, 2350, 2351, 2352, 2353, 2354, 2355, 2356, 2357,
    2358, 2359, 2360, 2361, 11297, 11295, 11294, 11283, 11280, 11278, 2362, 11291, 11289, 11286, 11284, 2362,
    1073, 0, 11300, 2363, 2364, 11302, 0, 11301, 1188, 2328, 1193, 2329, 2330, 2331, 2332, 2333,
    2334, 2335, 2336, 2337, 2338, 2339, 2340, 2341, 2342, 2343, 2344, 2345, 2346, 2347, 2348, 2349,
    2350, 2351, 2352, 2353, 2354, 2355, 2356, 2357, 2358, 2359, 2360, 2361, 2363, 2364, 0, 89,
    99, 0, 1047, 0, 343, 11306, 1729, 11309, 343, 11312, 11315, 11318, 0, 894, 11331, 11328,
    343, 11332, 282, 0, 198, 443, 0, 1471, 443, 1472, 1473, 1474, 1825, 1291, 0, 165,
    1601, 1480, 11333, 11343, 11353, 11363, 0, 1455, 11375, 417, 11372, 11370, 417, 0, 87, 2371,
    11376, 0, 2373, 2375, 2377, 0, 11378, 11377, 0, 234, 823, 0, 254, 836, 254, 11383,
    1356, 0, 11382, 872, 875, 515, 0, 259, 11389, 11386, 11384, 0, 707, 247, 0, 707,
    247, 0, 658, 11395, 11392, 11390, 0, 707, 247, 0, 707, 247, 11396, 0, 234, 11406,
    561, 1471, 2379, 225, 2381, 0, 276, 11407, 2383, 1976, 353, 326, 0, 276, 2384, 11408,
    11411, 2386, 154, 11413, 2388, 0, 276, 2384, 11416, 11419, 2386, 154, 11421, 1966, 0, 276,
    0, 488, 58, 0, 11424, 0, 276, 488, 49, 2389, 0, 2390, 198, 11434, 225, 11437,
    11447, 11457, 11464, 11474, 11484, 11494, 11504, 11514, 11524, 11534, 11544, 11554, 11564, 11574, 11584, 11594,
    11604, 11614, 11624, 11634, 11644, 11654, 11664, 11674, 11684, 11694, 11704,
];

pub static SEGMENT_TYPE_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 68, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 83, 86, 88, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 68, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 194, 196, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295,
    4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 81, 4294967295, 202, 4294967295, 206, 4294967295,
    4294967295, 212, 214, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 227, 4294967295,
    228, 233, 4294967295, 4294967295, 4294967295, 4294967295, 236, 4294967295, 4294967295, 4294967295, 242, 4294967295, 4294967295, 246, 248, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 86, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 83,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 271,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 271, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 286, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 320, 4294967295, 323, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 320, 4294967295, 323, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 338, 340, 4294967295,
    4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    81, 4294967295, 81, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 362, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    81, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 83, 86, 4294967295, 4294967295, 4294967295, 4294967295,
    81, 4294967295, 4294967295, 4294967295, 395, 4294967295, 395, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 403, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 395, 4294967295, 4294967295, 422, 4294967295, 426, 4294967295,
    4294967295, 4294967295, 4294967295, 418, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 395,
    4294967295, 4294967295, 422, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 206, 4294967295, 344,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 454, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 454, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 459, 4294967295, 4294967295, 459, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 472, 4294967295, 4294967295, 472, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 286, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 459, 4294967295, 4294967295, 459, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    511, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 519, 4294967295, 4294967295,
    4294967295, 418, 4294967295, 418, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 83, 86, 4294967295,
    557, 4294967295, 4294967295, 4294967295, 83, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 362, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 511,
    4294967295, 4294967295, 4294967295, 4294967295, 511, 4294967295, 4294967295, 4294967295, 286, 4294967295, 4294967295, 4294967295, 511, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 362, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 286, 4294967295, 4294967295, 4294967295,
    4294967295, 202, 4294967295, 4294967295, 286, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246,
    4294967295, 4294967295, 568, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 418, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295,
    4294967295, 269, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 362, 4294967295, 4294967295, 4294967295, 4294967295, 286, 4294967295, 4294967295,
    4294967295, 4294967295, 81, 4294967295, 4294967295, 286, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 246, 4294967295, 4294967295,
    4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    344, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 83, 86, 4294967295, 570, 4294967295, 4294967295, 4294967295, 83,
    4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 362, 4294967295, 4294967295, 4294967295, 574, 4294967295, 4294967295, 4294967295, 576, 4294967295, 4294967295, 4294967295, 578, 4294967295,
    4294967295, 4294967295, 578, 4294967295, 4294967295, 4294967295, 578, 4294967295, 4294967295, 4294967295, 574, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 202, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    202, 4294967295, 4294967295, 4294967295, 271, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    269, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 395, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 619, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 4294967295, 4294967295,
    4294967295, 619, 4294967295, 4294967295, 454, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 619,
    4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 626, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 626, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 630, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    574, 4294967295, 4294967295, 576, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 576, 4294967295, 4294967295, 4294967295, 4294967295, 576, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    630, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 395, 4294967295,
    4294967295, 422, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 395, 4294967295, 4294967295, 422, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 206, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 454, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 454, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 459, 4294967295, 4294967295, 459, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 472, 4294967295, 4294967295,
    472, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 286,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 459, 4294967295, 4294967295, 459, 4294967295, 4294967295, 4294967295, 246, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 511, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 640, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246,
    4294967295, 4294967295, 4294967295, 520, 4294967295, 4294967295, 4294967295, 418, 4294967295, 418, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 650, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    286, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 286, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81,
    4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81,
    4294967295, 81, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 81, 4294967295,
    4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 661, 4294967295,
    246, 4294967295, 4294967295, 4294967295, 4294967295, 661, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 362, 4294967295, 4294967295, 4294967295, 395,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 395, 4294967295, 4294967295,
    422, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 4294967295, 4294967295, 4294967295, 395, 4294967295, 4294967295, 422, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 418, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 418, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 418,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 206, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 418, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295,
    4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 418, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 680, 680, 4294967295, 680, 680, 4294967295, 4294967295, 4294967295, 100, 4294967295, 344, 4294967295, 4294967295,
    100, 4294967295, 344, 4294967295, 4294967295, 100, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 344, 4294967295, 233, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 727, 4294967295, 4294967295, 4294967295, 4294967295,
    729, 206, 418, 344, 4294967295, 395, 4294967295, 4294967295, 4294967295, 731, 733, 734, 3, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 680, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 770, 4294967295, 777, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 236, 4294967295, 4294967295, 785, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 418, 4294967295, 4294967295, 418, 4294967295, 418, 4294967295, 418,
    4294967295, 4294967295, 4294967295, 4294967295, 206, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 228, 4294967295,
    799, 4294967295, 4294967295, 805, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 799, 4294967295, 4294967295, 805, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 511,
    4294967295, 511, 4294967295, 246, 4294967295, 4294967295, 4, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 830, 4294967295, 206, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    344, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 422, 4294967295, 4294967295,
    395, 4294967295, 426, 4294967295, 4294967295, 422, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 845, 847, 4294967295, 4294967295, 4294967295, 83,
    853, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 81, 4294967295, 4294967295, 418, 4294967295, 4294967295,
    81, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 271, 4294967295, 4294967295, 362, 4294967295, 4294967295, 83, 4294967295, 86,
    4294967295, 4294967295, 511, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 861, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290,
    290, 290, 290, 290, 290, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 680, 290, 4294967295, 4294967295, 344,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 290, 4294967295, 344, 4294967295,
    4294967295, 4294967295, 4294967295, 418, 290, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 418, 290,
    4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 344, 206,
    4294967295, 4294967295, 422, 897, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 418, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 418, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    418, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 86, 4294967295, 4294967295, 4294967295, 4294967295, 395, 4294967295,
    395, 4294967295, 4294967295, 4294967295, 4294967295, 83, 86, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290,
    4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 290, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 271, 4294967295, 4294967295,
    4294967295, 4294967295, 290, 81, 4294967295, 4294967295, 290, 81, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 290, 4294967295,
    286, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295,
    4294967295, 4294967295, 81, 4294967295, 4294967295, 395, 4294967295, 4294967295, 422, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295,
    81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 362, 4294967295, 4294967295, 4294967295,
    269, 4294967295, 4294967295, 963, 4294967295, 4294967295, 963, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 418, 395, 4294967295,
    4294967295, 4294967295, 422, 4294967295, 426, 472, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 976, 4294967295, 4294967295,
    4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 4294967295, 4294967295, 4294967295,
    426, 472, 4294967295, 4294967295, 4294967295, 4294967295, 650, 4294967295, 4294967295, 4294967295, 976, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 83, 86, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 511, 4294967295, 4294967295, 246, 4294967295, 4294967295, 403, 4294967295, 4294967295, 981, 4294967295, 4294967295, 4294967295, 4294967295,
    981, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 459, 4294967295, 4294967295, 286, 4294967295, 4294967295, 344, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 459, 4294967295, 4294967295, 286,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 989, 4294967295, 4294967295, 4294967295, 4294967295, 206,
    991, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 290, 81, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 290, 83, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 246, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295,
    4294967295, 83, 86, 1016, 4294967295, 4294967295, 4294967295, 574, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    578, 4294967295, 4294967295, 576, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 202, 4294967295, 4294967295, 202, 4294967295, 4294967295,
    202, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 202, 269, 4294967295, 4294967295,
    4294967295, 395, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 619,
    4294967295, 1022, 4294967295, 4294967295, 4294967295, 4294967295, 976, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 963, 4294967295, 81, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 626, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    418, 344, 4294967295, 246, 4294967295, 4294967295, 4294967295, 630, 4294967295, 4294967295, 4294967295, 4294967295, 574, 4294967295, 4294967295, 576,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 418, 4294967295, 4294967295, 395, 4294967295, 4294967295, 426, 4294967295, 395, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 344, 4294967295, 4294967295, 4294967295, 472, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295,
    290, 81, 4294967295, 4294967295, 290, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 661, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 4294967295, 4294967295,
    4294967295, 246, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 206, 4294967295, 4294967295, 4294967295, 395, 4294967295, 4294967295, 395, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1052, 4294967295,
    4294967295, 1056, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 976,
    4294967295, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 989, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 83, 83, 4294967295, 1068,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1109, 1111, 1052, 1113, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1175, 4294967295, 4294967295, 4294967295, 4294967295, 1175, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1175, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1175, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1175, 4294967295, 4294967295, 1175, 4294967295, 4294967295, 1216, 1218, 1220, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1111, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 1281, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295, 1285, 248, 1287, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 81, 1289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 248, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1310, 4294967295, 4294967295, 4294967295, 1310, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 86, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 395, 4294967295, 395, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 271, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 362, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 83, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 511, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 246, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 83, 86, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 574,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 619, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 626, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    630, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 661, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 395, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1052, 1320, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 981, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1347, 4294967295, 4294967295, 1349, 1351, 1353,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 100,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 100, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1392,
    4294967295, 1394, 830, 4294967295, 4294967295, 4294967295, 830, 4294967295, 290, 4294967295, 4294967295, 4294967295, 206, 4294967295, 4294967295, 4294967295,
    344, 4294967295, 4294967295, 4294967295, 4294967295, 418, 206, 1401, 344, 4294967295, 88, 4294967295, 4294967295, 1052, 1113, 4294967295,
    4294967295, 344, 4294967295, 395, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1406, 1401, 4294967295, 4294967295, 4294967295, 4294967295, 1410,
    4294967295, 729, 1412, 1414, 418, 1416, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295,
    246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1430, 1433,
    1436, 4294967295, 1438, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 830, 4294967295, 206, 4294967295, 344, 4294967295, 4294967295, 4294967295,
    4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 729, 4294967295, 418, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1463, 4294967295, 4294967295, 1470, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1476, 1478, 991, 4294967295, 4294967295, 1481, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1285, 1485, 248, 4294967295, 4294967295, 1489, 4294967295, 1287, 4294967295, 4294967295, 4294967295, 248, 1489, 4294967295, 4294967295, 4294967295,
    4294967295, 1287, 4294967295, 4294967295, 4294967295, 4294967295, 1493, 1495, 4294967295, 1463, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    228, 4294967295, 4294967295, 4294967295, 1287, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 242, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 206, 248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1499, 227, 1470,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 847, 1504,
    344, 4294967295, 344, 4294967295, 4294967295, 777, 4294967295, 344, 4294967295, 395, 4294967295, 4294967295, 344, 4294967295, 4294967295, 344,
    4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 344, 4294967295, 4294967295, 395, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 344, 4294967295, 4294967295,
    4294967295, 4294967295, 418, 344, 4294967295, 1517, 1412, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1310, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1349, 4294967295, 4294967295, 1351, 4294967295, 4294967295, 4294967295, 1523, 777, 1410, 4294967295, 4294967295, 4294967295, 344,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 770, 4294967295,
    4294967295, 4294967295, 770, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 395, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 395, 1529, 4294967295, 4294967295, 4294967295, 290, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    83, 853, 4294967295, 4294967295, 4294967295, 83, 4294967295, 83, 83, 989, 853, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1537, 4294967295, 1539,
    4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 4294967295, 200, 81, 4294967295, 1541, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 344, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1470, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 344,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1551, 1068, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 1470, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1565, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 976, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 206, 1580, 727, 4294967295, 418, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 422, 4294967295, 1587, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 459, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 206, 4294967295, 344, 1589, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 576, 4294967295, 4294967295, 4294967295, 4294967295, 286, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 248,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    989, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1603, 1605, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344,
    248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1495, 4294967295, 4294967295, 4294967295, 228, 1630,
    4294967295, 1632, 4294967295, 242, 4294967295, 4294967295, 4294967295, 1495, 1630, 4294967295, 4294967295, 1495, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1638, 1640, 248, 4294967295, 4294967295, 228, 4294967295, 4294967295, 344, 242, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 344,
    1679, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 100, 4294967295, 1406, 4294967295, 1685, 1580,
    1687, 1689, 1312, 1312, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1709, 4294967295, 4294967295, 4294967295, 395, 4294967295, 4294967295, 1712, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    989, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1723, 1725, 1727, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295,
    4294967295, 1732, 1734, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295,
    4294967295, 1732, 1734, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344,
    4294967295, 4294967295, 1740, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295,
    4294967295, 4294967295, 248, 4294967295, 206, 248, 228, 1632, 242, 1744, 4294967295, 4294967295, 1746, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1766, 4294967295, 4294967295,
    4294967295, 1768, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    310, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 989, 1410, 4294967295, 1410, 4294967295, 1523, 4294967295, 4294967295, 4294967295, 1785, 1787, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 777, 4294967295, 4294967295, 1410, 4294967295, 1351,
    1349, 1353, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1805, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 418, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1481, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1766, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 418, 1406, 830, 4294967295, 4294967295, 4294967295, 418, 1406, 830, 4294967295, 418, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 418, 1406, 830, 4294967295, 4294967295, 4294967295, 418, 1406, 830, 4294967295, 418, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1849, 1851, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81,
    4294967295, 4294967295, 395, 4294967295, 4294967295, 4294967295, 1860, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1875,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 246, 991, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    286, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 511, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295,
    269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 271, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295,
    4294967295, 1910, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1537, 4294967295, 1539,
    4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 4294967295, 200, 200, 4294967295, 1541, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    81, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 418, 4294967295, 4294967295, 83, 4294967295, 86, 4294967295, 4294967295,
    271, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 83, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295,
    83, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 202, 4294967295, 4294967295, 83, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 269, 4294967295, 4294967295, 81, 4294967295, 4294967295, 81, 4294967295, 4294967295, 81, 4294967295, 4294967295, 81, 4294967295,
    4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 845, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 1945, 4294967295, 1947, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1476, 4294967295, 4294967295, 4294967295, 4294967295, 1949, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1476, 4294967295, 4294967295, 4294967295, 1951, 1953, 1289, 1499,
    1955, 1416, 1957, 1959, 227, 1961, 1963, 1470, 1957, 1489, 1476, 1478, 991, 4294967295, 4294967295, 4294967295,
    4294967295, 1949, 4294967295, 4294967295, 4294967295, 4294967295, 1523, 4294967295, 4294967295, 1523, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 212, 4294967295, 4294967295, 4294967295, 4294967295, 212, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1972, 4294967295, 4294967295, 4294967295, 418,
    212, 290, 4294967295, 4294967295, 4294967295, 206, 418, 344, 4294967295, 4294967295, 830, 4294967295, 4294967295, 4294967295, 418, 290,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 206, 418, 344, 4294967295, 4294967295, 830, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1476, 4294967295, 227, 1470,
    1957, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1392, 4294967295, 4294967295, 4294967295, 395, 1394, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 418, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 206, 248, 1499, 227, 1470, 4294967295, 4294967295, 4294967295, 4294967295, 1285, 4294967295, 4294967295, 4294967295, 4294967295,
    344, 4294967295, 4294967295, 344, 248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 227, 1470, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1218, 212,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1860, 1349, 2016, 2018, 2020, 2022, 2024, 2026, 2028, 2030, 2032, 2034, 2036,
    2038, 2040, 2042, 2044, 2046, 2048, 2050, 2052, 2054, 1353, 1351, 2056, 2058, 2060, 2062, 2064,
    2066, 2068, 2070, 2072, 2074, 2076, 2078, 2080, 2082, 2084, 2086, 2088, 2090, 2092, 4294967295, 2095,
    2097, 2099, 2101, 2103, 2105, 2107, 2109, 2111, 2113, 2115, 2117, 2119, 2121, 2048, 2124, 2126,
    2054, 2128, 2130, 2132, 2135, 2137, 2139, 2141, 2143, 2145, 2147, 2149, 2151, 2153, 2155, 2157,
    2159, 2161, 2163, 2165, 2167, 2169, 2171, 2173, 2175, 2177, 2179, 2181, 2183, 2185, 2187, 2189,
    2191, 2193, 2195, 2197, 2199, 2201, 2203, 2205, 2207, 2209, 2211, 2213, 2215, 2217, 2219, 2221,
    2223, 2225, 2227, 2229, 2231, 2233, 2235, 2237, 2239, 2241, 2243, 1785, 1787, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 680, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    248, 4294967295, 4294967295, 4294967295, 4294967295, 395, 4294967295, 4294967295, 395, 4294967295, 4294967295, 1056, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 9, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295,
    4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2257, 4294967295, 4294967295, 4294967295,
    2257, 4294967295, 4294967295, 4294967295, 4294967295, 403, 4294967295, 4294967295, 2260, 2257, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    511, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 989,
    4294967295, 4294967295, 206, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 1860, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1175, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1218, 1580, 4294967295, 1052, 1320, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1951, 1953, 4294967295, 1953, 1951,
    1289, 1499, 1955, 1957, 1959, 1489, 1476, 227, 1470, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1476, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 248, 4294967295, 1746,
    1953, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 1289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 248,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 271, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344,
    4294967295, 248, 227, 1470, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1109, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 344, 228, 4294967295, 228, 344, 242, 242, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1481, 4294967295, 4294967295, 4294967295, 2369, 4294967295, 4294967295, 4294967295, 418, 4294967295, 4294967295, 418, 4294967295, 88, 2372,
    4294967295, 4294967295, 2374, 2376, 2378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295,
    248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 2380, 227, 2382, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 228,
    4294967295, 2387, 4294967295, 242, 4294967295, 4294967295, 4294967295, 4294967295, 228, 4294967295, 2387, 4294967295, 242, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 227, 4294967295,
    4294967295, 5, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static SEGMENT_CLASS_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 67, 4294967295, 4294967295, 69, 4294967295, 4294967295, 4294967295, 4294967295, 69, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 82, 84, 87, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 67, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 163, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 193, 195, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295,
    4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 80, 4294967295, 201, 4294967295, 205, 4294967295,
    4294967295, 211, 213, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295,
    229, 231, 4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295, 4294967295, 4294967295, 243, 4294967295, 4294967295, 245, 247, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 270,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 285, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 319, 4294967295, 321, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 319, 4294967295, 321, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 337, 339, 4294967295,
    4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    80, 4294967295, 80, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    80, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 84, 4294967295, 4294967295, 4294967295, 4294967295,
    80, 4294967295, 4294967295, 4294967295, 394, 4294967295, 394, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 402, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 394, 4294967295, 4294967295, 421, 4294967295, 425, 4294967295,
    4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 394,
    4294967295, 4294967295, 421, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295, 343,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 453, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 453, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 458, 4294967295, 4294967295, 458, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 470, 4294967295, 4294967295, 473, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 458, 4294967295, 4294967295, 458, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 518, 4294967295, 4294967295,
    4294967295, 417, 4294967295, 417, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 84, 4294967295,
    556, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510,
    4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295,
    4294967295, 201, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245,
    4294967295, 4294967295, 567, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 417, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295,
    4294967295, 268, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295, 285, 4294967295, 4294967295,
    4294967295, 4294967295, 80, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 245, 4294967295, 4294967295,
    4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    343, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 84, 4294967295, 569, 4294967295, 4294967295, 4294967295, 82,
    4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 573, 4294967295, 4294967295, 4294967295, 575, 4294967295, 4294967295, 4294967295, 577, 4294967295,
    4294967295, 4294967295, 577, 4294967295, 4294967295, 4294967295, 577, 4294967295, 4294967295, 4294967295, 573, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 201, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    201, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    268, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 394, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 618, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295,
    4294967295, 618, 4294967295, 4294967295, 453, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 618,
    4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 625, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 625, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 629, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    573, 4294967295, 4294967295, 575, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 575, 4294967295, 4294967295, 4294967295, 4294967295, 575, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    629, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 394, 4294967295,
    4294967295, 421, 4294967295, 425, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 394, 4294967295, 4294967295, 421, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 205, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 453, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 453, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 458, 4294967295, 4294967295, 458, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 470, 4294967295, 4294967295,
    473, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 285,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 458, 4294967295, 4294967295, 458, 4294967295, 4294967295, 4294967295, 245, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 639, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245,
    4294967295, 4294967295, 4294967295, 645, 4294967295, 4294967295, 4294967295, 417, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 649, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    285, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80,
    4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80,
    4294967295, 80, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 80, 4294967295,
    4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 660, 4294967295,
    245, 4294967295, 4294967295, 4294967295, 4294967295, 660, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 394,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 394, 4294967295, 4294967295,
    421, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 394, 4294967295, 4294967295, 421, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 417,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 417, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295,
    4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 696, 697, 4294967295, 699, 700, 4294967295, 4294967295, 4294967295, 702, 4294967295, 343, 4294967295, 4294967295,
    702, 4294967295, 343, 4294967295, 4294967295, 702, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 343, 4294967295, 231, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 725, 4294967295, 4294967295, 4294967295, 4294967295,
    728, 205, 417, 343, 4294967295, 394, 4294967295, 4294967295, 4294967295, 730, 732, 735, 736, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 681, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 771, 4294967295, 772, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295, 4294967295, 784, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 417, 4294967295, 4294967295, 417, 4294967295, 417, 4294967295, 417,
    4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 796, 4294967295,
    798, 4294967295, 4294967295, 804, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 798, 4294967295, 4294967295, 804, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 510,
    4294967295, 510, 4294967295, 245, 4294967295, 4294967295, 724, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 829, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    343, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 421, 4294967295, 4294967295,
    394, 4294967295, 425, 4294967295, 4294967295, 421, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 844, 846, 4294967295, 4294967295, 4294967295, 851,
    852, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 80, 4294967295, 4294967295, 417, 4294967295, 4294967295,
    80, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 270, 4294967295, 4294967295, 361, 4294967295, 4294967295, 82, 4294967295, 84,
    4294967295, 4294967295, 510, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 862, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289,
    867, 868, 869, 870, 871, 4294967295, 4294967295, 4294967295, 4294967295, 876, 4294967295, 878, 879, 4294967295, 4294967295, 343,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 289, 4294967295, 343, 4294967295,
    4294967295, 4294967295, 4294967295, 417, 289, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 417, 289,
    4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 343, 205,
    4294967295, 4294967295, 421, 896, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 417, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 417, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    417, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 84, 4294967295, 4294967295, 4294967295, 4294967295, 394, 4294967295,
    394, 4294967295, 4294967295, 4294967295, 4294967295, 82, 84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289,
    4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 289, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295,
    4294967295, 4294967295, 289, 80, 4294967295, 4294967295, 289, 80, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 289, 4294967295,
    285, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295,
    4294967295, 4294967295, 80, 4294967295, 4294967295, 394, 4294967295, 4294967295, 421, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295,
    80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295,
    268, 4294967295, 4294967295, 962, 4294967295, 4294967295, 962, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 417, 394, 4294967295,
    4294967295, 4294967295, 421, 4294967295, 971, 972, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 975, 4294967295, 4294967295,
    4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295,
    971, 972, 4294967295, 4294967295, 4294967295, 4294967295, 649, 4294967295, 4294967295, 4294967295, 975, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 82, 84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 510, 4294967295, 4294967295, 245, 4294967295, 4294967295, 402, 4294967295, 4294967295, 980, 4294967295, 4294967295, 4294967295, 4294967295,
    980, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 458, 4294967295, 4294967295, 285, 4294967295, 4294967295, 343, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 458, 4294967295, 4294967295, 285,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 988, 4294967295, 4294967295, 4294967295, 4294967295, 205,
    990, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 289, 80, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 289, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 245, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295,
    4294967295, 82, 84, 1013, 4294967295, 4294967295, 4294967295, 573, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    577, 4294967295, 4294967295, 575, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 201, 4294967295, 4294967295, 201, 4294967295, 4294967295,
    201, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 201, 268, 4294967295, 4294967295,
    4294967295, 394, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 618,
    4294967295, 1021, 4294967295, 4294967295, 4294967295, 4294967295, 975, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 962, 4294967295, 80, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 625, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    417, 343, 4294967295, 245, 4294967295, 4294967295, 4294967295, 629, 4294967295, 4294967295, 4294967295, 4294967295, 573, 4294967295, 4294967295, 575,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 417, 4294967295, 4294967295, 394, 4294967295, 4294967295, 425, 4294967295, 394, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 343, 4294967295, 4294967295, 4294967295, 470, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295,
    289, 80, 4294967295, 4294967295, 289, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 660, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295,
    4294967295, 245, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 205, 4294967295, 4294967295, 4294967295, 394, 4294967295, 4294967295, 394, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1050, 4294967295,
    4294967295, 1055, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 975,
    4294967295, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 988, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1065, 1066, 4294967295, 1067,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1073, 1110, 1050, 1112, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1174, 4294967295, 4294967295, 4294967295, 4294967295, 1174, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1174, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1174, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1174, 4294967295, 4294967295, 1174, 4294967295, 4294967295, 1215, 1217, 1219, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1110, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 1280, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 247, 4294967295, 4294967295, 4294967295, 4294967295, 1284, 247, 1286, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 80, 1288, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 247, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1309, 4294967295, 4294967295, 4294967295, 1309, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 394, 4294967295, 394, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 245, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 573,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 618, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 625, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    629, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 660, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 394, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1050, 1319, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 980, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1346, 4294967295, 4294967295, 1348, 1350, 1352,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 702,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 702, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1391,
    4294967295, 1393, 829, 4294967295, 4294967295, 4294967295, 829, 4294967295, 871, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295,
    343, 4294967295, 4294967295, 4294967295, 4294967295, 417, 205, 1400, 343, 4294967295, 1403, 4294967295, 4294967295, 1050, 1112, 4294967295,
    4294967295, 343, 4294967295, 394, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1405, 1400, 4294967295, 4294967295, 4294967295, 4294967295, 1409,
    4294967295, 728, 1411, 1413, 417, 1415, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295,
    245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1428, 1431,
    1434, 4294967295, 1437, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 829, 4294967295, 205, 4294967295, 343, 4294967295, 4294967295, 4294967295,
    4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 728, 4294967295, 417, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1462, 4294967295, 4294967295, 1469, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1475, 1477, 990, 4294967295, 4294967295, 1480, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1284, 1484, 247, 4294967295, 4294967295, 1487, 4294967295, 1286, 4294967295, 4294967295, 4294967295, 247, 1487, 4294967295, 4294967295, 4294967295,
    4294967295, 1286, 4294967295, 4294967295, 4294967295, 4294967295, 1491, 1494, 4294967295, 1462, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    229, 4294967295, 4294967295, 4294967295, 1286, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 243, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 205, 247, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1498, 225, 1469,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 247, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 846, 1503,
    343, 4294967295, 343, 4294967295, 4294967295, 772, 4294967295, 343, 4294967295, 394, 4294967295, 4294967295, 343, 4294967295, 4294967295, 343,
    4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 343, 4294967295, 4294967295, 394, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 343, 4294967295, 4294967295,
    4294967295, 4294967295, 417, 343, 4294967295, 1516, 1518, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1309, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1348, 4294967295, 4294967295, 1350, 4294967295, 4294967295, 4294967295, 1522, 772, 1409, 4294967295, 4294967295, 4294967295, 343,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 771, 4294967295,
    4294967295, 4294967295, 771, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 394, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 394, 1528, 4294967295, 4294967295, 4294967295, 289, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1531, 1532, 4294967295, 4294967295, 4294967295, 82, 4294967295, 1531, 851, 988, 1533, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1536, 4294967295, 1538,
    4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295, 199, 80, 4294967295, 1540, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 343, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1469, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 343,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1550, 1067, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 1469, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1566, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 975, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 205, 1579, 725, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 421, 4294967295, 1584, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 458, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 205, 4294967295, 343, 1588, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 575, 4294967295, 4294967295, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 1598,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    988, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1602, 1604, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343,
    1598, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1494, 4294967295, 4294967295, 4294967295, 229, 1629,
    4294967295, 1631, 4294967295, 243, 4294967295, 4294967295, 4294967295, 1494, 1629, 4294967295, 4294967295, 1494, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1637, 1639, 247, 4294967295, 4294967295, 796, 4294967295, 4294967295, 343, 243, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 343,
    1678, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 702, 4294967295, 1405, 4294967295, 1684, 1579,
    1686, 1688, 1420, 1313, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1708, 4294967295, 4294967295, 4294967295, 394, 4294967295, 4294967295, 1711, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    988, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1722, 1724, 1726, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295,
    4294967295, 1731, 1733, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295,
    4294967295, 1731, 1733, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343,
    4294967295, 4294967295, 1739, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295,
    4294967295, 4294967295, 247, 4294967295, 205, 247, 229, 1631, 243, 1743, 4294967295, 4294967295, 1745, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1765, 4294967295, 4294967295,
    4294967295, 1767, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1759, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 988, 1781, 4294967295, 1409, 4294967295, 1522, 4294967295, 4294967295, 4294967295, 1784, 1786, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 772, 4294967295, 4294967295, 1409, 4294967295, 1350,
    1348, 1352, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1804, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 417, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1480, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1765, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 417, 1405, 829, 4294967295, 4294967295, 4294967295, 417, 1405, 829, 4294967295, 417, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 417, 1405, 829, 4294967295, 4294967295, 4294967295, 417, 1405, 829, 4294967295, 417, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1847, 1850, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80,
    4294967295, 4294967295, 394, 4294967295, 4294967295, 4294967295, 1859, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1874,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 245, 990, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 510, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295,
    268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295,
    4294967295, 1909, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1536, 4294967295, 1538,
    4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295, 199, 199, 4294967295, 1540, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    80, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 417, 4294967295, 4294967295, 82, 4294967295, 84, 4294967295, 4294967295,
    270, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 82, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295,
    82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 201, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 268, 4294967295, 4294967295, 80, 4294967295, 4294967295, 80, 4294967295, 4294967295, 80, 4294967295, 4294967295, 80, 4294967295,
    4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 844, 4294967295, 4294967295, 247, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 1944, 4294967295, 1946, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1475, 4294967295, 4294967295, 4294967295, 4294967295, 1948, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1475, 4294967295, 4294967295, 4294967295, 1950, 1952, 1288, 1498,
    1954, 1415, 1956, 1958, 225, 1960, 1962, 1469, 1956, 1964, 1475, 1477, 990, 4294967295, 4294967295, 4294967295,
    4294967295, 1948, 4294967295, 4294967295, 4294967295, 4294967295, 1522, 4294967295, 4294967295, 1965, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 211, 4294967295, 4294967295, 4294967295, 4294967295, 211, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1971, 4294967295, 4294967295, 4294967295, 417,
    211, 289, 4294967295, 4294967295, 4294967295, 205, 417, 343, 4294967295, 4294967295, 829, 4294967295, 4294967295, 4294967295, 417, 289,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 205, 417, 343, 4294967295, 4294967295, 829, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1475, 4294967295, 225, 1469,
    1956, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1391, 4294967295, 4294967295, 4294967295, 394, 1393, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1284, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 205, 247, 1498, 225, 1469, 4294967295, 4294967295, 4294967295, 4294967295, 1284, 4294967295, 4294967295, 4294967295, 4294967295,
    343, 4294967295, 4294967295, 343, 247, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 1469, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1217, 211,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1859, 1348, 2015, 2017, 2019, 2021, 2023, 2025, 2027, 2029, 2031, 2033, 2035,
    2037, 2039, 2041, 2043, 2045, 2047, 2049, 2051, 2053, 1352, 1350, 2055, 2057, 2059, 2061, 2063,
    2065, 2067, 2069, 2071, 2073, 2075, 2077, 2079, 2081, 2083, 2085, 2087, 2089, 2091, 4294967295, 2094,
    2096, 2098, 2100, 2102, 2104, 2106, 2108, 2110, 2112, 2114, 2116, 2118, 2120, 2047, 2122, 2125,
    2053, 2127, 2129, 2131, 2133, 2136, 2138, 2140, 2142, 2144, 2146, 2148, 2150, 2152, 2154, 2156,
    2158, 2160, 2162, 2164, 2166, 2168, 2170, 2172, 2174, 2176, 2178, 2180, 2182, 2184, 2186, 2188,
    2190, 2192, 2194, 2196, 2198, 2200, 2202, 2204, 2206, 2208, 2210, 2212, 2214, 2216, 2218, 2220,
    2222, 2224, 2226, 2228, 2230, 2232, 2234, 2236, 2238, 2240, 2242, 1784, 1786, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2247, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    247, 4294967295, 4294967295, 4294967295, 4294967295, 394, 4294967295, 4294967295, 394, 4294967295, 4294967295, 1055, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 10, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295,
    4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2256, 4294967295, 4294967295, 4294967295,
    2256, 4294967295, 4294967295, 4294967295, 4294967295, 402, 4294967295, 4294967295, 2259, 2256, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    510, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 988,
    4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 1859, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1174, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1217, 1579, 4294967295, 1050, 2299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1950, 1952, 4294967295, 1952, 1950,
    1288, 1498, 1954, 1956, 1958, 1964, 1475, 225, 1469, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1475, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 247, 4294967295, 1745,
    1952, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 1288, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 247,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343,
    4294967295, 247, 225, 1469, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1073, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 343, 229, 4294967295, 796, 343, 243, 243, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1480, 4294967295, 4294967295, 4294967295, 2370, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 417, 4294967295, 87, 2371,
    4294967295, 4294967295, 2373, 2375, 2377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 247, 4294967295, 4294967295,
    247, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 247, 4294967295, 4294967295, 247, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 2379, 225, 2381, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 229,
    4294967295, 2386, 4294967295, 243, 4294967295, 4294967295, 4294967295, 4294967295, 229, 4294967295, 2386, 4294967295, 243, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295,
    4294967295, 1840, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static CASEFOLD_SPARSE: &[(u32, u8)] = &[
    (5582, 2),
    (6091, 2),
    (6812, 2),
];

pub static TRIM_CHARS_SPARSE: &[(u32, u32, u8)] = &[
];

pub static TRIM_CHARS_DATA: &[u32] = &[
];

pub static SEGMENT_CLASS_TYPES_SPARSE: &[(u32, u32, u8)] = &[
    (38, 0, 2),
    (41, 2, 2),
    (46, 4, 2),
    (53, 6, 2),
    (55, 8, 2),
    (56, 10, 2),
    (57, 12, 3),
    (99, 15, 2),
    (135, 17, 2),
    (184, 19, 2),
    (185, 21, 2),
    (190, 23, 2),
    (194, 25, 3),
    (198, 28, 3),
    (202, 31, 2),
    (204, 33, 3),
    (206, 36, 2),
    (209, 38, 2),
    (210, 40, 2),
    (222, 42, 2),
    (224, 44, 4),
    (225, 48, 2),
    (230, 50, 2),
    (234, 52, 5),
    (237, 57, 3),
    (238, 60, 2),
    (245, 62, 2),
    (248, 64, 2),
    (255, 66, 2),
    (263, 68, 3),
    (267, 71, 3),
    (271, 74, 3),
    (289, 77, 3),
    (294, 80, 2),
    (301, 82, 3),
    (309, 85, 2),
    (325, 87, 2),
    (327, 89, 2),
    (330, 91, 2),
    (359, 93, 2),
    (361, 95, 2),
    (364, 97, 2),
    (385, 99, 2),
    (393, 101, 3),
    (397, 104, 2),
    (398, 106, 2),
    (402, 108, 3),
    (407, 111, 2),
    (412, 113, 2),
    (426, 115, 2),
    (433, 117, 2),
    (441, 119, 2),
    (448, 121, 2),
    (450, 123, 2),
    (454, 125, 2),
    (459, 127, 2),
    (466, 129, 2),
    (470, 131, 2),
    (474, 133, 3),
    (485, 136, 3),
    (525, 139, 2),
    (533, 141, 3),
    (536, 144, 2),
    (544, 146, 2),
    (546, 148, 2),
    (554, 150, 2),
    (555, 152, 2),
    (560, 154, 2),
    (564, 156, 2),
    (566, 158, 2),
    (571, 160, 2),
    (573, 162, 2),
    (584, 164, 3),
    (585, 167, 2),
    (588, 169, 3),
    (590, 172, 2),
    (595, 174, 3),
    (600, 177, 3),
    (607, 180, 2),
    (610, 182, 3),
    (614, 185, 2),
    (621, 187, 2),
    (623, 189, 2),
    (649, 191, 2),
    (662, 193, 2),
    (678, 195, 2),
    (681, 197, 2),
    (696, 199, 2),
    (699, 201, 2),
    (778, 203, 3),
    (786, 206, 2),
    (789, 208, 2),
    (793, 210, 3),
    (816, 213, 3),
    (823, 216, 3),
    (841, 219, 3),
    (845, 222, 3),
    (849, 225, 3),
    (851, 228, 3),
    (855, 231, 3),
    (906, 234, 2),
    (925, 236, 2),
    (926, 238, 2),
    (928, 240, 2),
    (932, 242, 2),
    (936, 244, 3),
    (952, 247, 3),
    (959, 250, 3),
    (964, 253, 3),
    (968, 256, 3),
    (972, 259, 3),
    (978, 262, 3),
    (986, 265, 2),
    (1004, 267, 3),
    (1009, 270, 3),
    (1012, 273, 3),
    (1019, 276, 3),
    (1031, 279, 2),
    (1065, 281, 2),
    (1076, 283, 2),
    (1087, 285, 3),
    (1090, 288, 2),
    (1095, 290, 3),
    (1097, 293, 3),
    (1101, 296, 3),
    (1105, 299, 3),
    (1107, 302, 3),
    (1112, 305, 3),
    (1117, 308, 3),
    (1122, 311, 2),
    (1125, 313, 3),
    (1131, 316, 2),
    (1133, 318, 3),
    (1138, 321, 2),
    (1144, 323, 3),
    (1152, 326, 2),
    (1157, 328, 2),
    (1173, 330, 2),
    (1192, 332, 2),
    (1193, 334, 2),
    (1195, 336, 2),
    (1199, 338, 2),
    (1203, 340, 3),
    (1211, 343, 2),
    (1234, 345, 3),
    (1238, 348, 3),
    (1242, 351, 2),
    (1246, 353, 2),
    (1250, 355, 2),
    (1254, 357, 2),
    (1258, 359, 3),
    (1266, 362, 3),
    (1278, 365, 3),
    (1317, 368, 3),
    (1321, 371, 3),
    (1328, 374, 3),
    (1332, 377, 3),
    (1341, 380, 2),
    (1360, 382, 3),
    (1365, 385, 3),
    (1369, 388, 3),
    (1373, 391, 2),
    (1398, 393, 3),
    (1420, 396, 3),
    (1425, 399, 3),
    (1428, 402, 2),
    (1439, 404, 3),
    (1443, 407, 3),
    (1448, 410, 3),
    (1463, 413, 3),
    (1469, 416, 3),
    (1476, 419, 3),
    (1488, 422, 3),
    (1491, 425, 2),
    (1497, 427, 2),
    (1502, 429, 2),
    (1508, 431, 2),
    (1513, 433, 2),
    (1520, 435, 3),
    (1530, 438, 2),
    (1549, 440, 3),
    (1550, 443, 2),
    (1553, 445, 3),
    (1555, 448, 2),
    (1560, 450, 3),
    (1565, 453, 3),
    (1572, 456, 2),
    (1575, 458, 3),
    (1579, 461, 2),
    (1586, 463, 2),
    (1588, 465, 2),
    (1614, 467, 2),
    (1627, 469, 2),
    (1643, 471, 2),
    (1646, 473, 2),
    (1661, 475, 2),
    (1664, 477, 2),
    (1743, 479, 3),
    (1751, 482, 2),
    (1754, 484, 2),
    (1758, 486, 3),
    (1781, 489, 3),
    (1792, 492, 2),
    (1801, 494, 2),
    (1807, 496, 3),
    (1815, 499, 3),
    (1823, 502, 3),
    (1827, 505, 2),
    (1831, 507, 3),
    (1833, 510, 3),
    (1842, 513, 3),
    (1847, 516, 3),
    (1851, 519, 3),
    (1860, 522, 2),
    (1872, 524, 3),
    (1877, 527, 2),
    (1880, 529, 3),
    (1887, 532, 2),
    (1894, 534, 2),
    (1897, 536, 2),
    (1903, 538, 2),
    (1906, 540, 2),
    (1911, 542, 2),
    (1913, 544, 2),
    (1919, 546, 2),
    (1921, 548, 2),
    (1923, 550, 2),
    (1930, 552, 2),
    (1934, 554, 2),
    (1939, 556, 2),
    (1946, 558, 2),
    (1950, 560, 3),
    (1952, 563, 3),
    (1957, 566, 3),
    (1963, 569, 3),
    (1967, 572, 2),
    (1976, 574, 2),
    (1986, 576, 2),
    (1990, 578, 3),
    (1996, 581, 3),
    (1997, 584, 2),
    (2000, 586, 3),
    (2005, 589, 3),
    (2010, 592, 2),
    (2013, 594, 3),
    (2019, 597, 3),
    (2024, 600, 3),
    (2026, 603, 3),
    (2042, 606, 3),
    (2047, 609, 3),
    (2054, 612, 2),
    (2056, 614, 2),
    (2064, 616, 2),
    (2071, 618, 3),
    (2073, 621, 3),
    (2077, 624, 3),
    (2081, 627, 3),
    (2089, 630, 2),
    (2115, 632, 3),
    (2118, 635, 3),
    (2131, 638, 2),
    (2132, 640, 2),
    (2134, 642, 2),
    (2135, 644, 2),
    (2139, 646, 2),
    (2141, 648, 2),
    (2144, 650, 2),
    (2146, 652, 2),
    (2149, 654, 2),
    (2151, 656, 2),
    (2161, 658, 2),
    (2163, 660, 2),
    (2187, 662, 2),
    (2192, 664, 2),
    (2193, 666, 2),
    (2194, 668, 3),
    (2195, 671, 2),
    (2197, 673, 2),
    (2201, 675, 2),
    (2202, 677, 2),
    (2203, 679, 2),
    (2204, 681, 1),
    (2218, 682, 3),
    (2247, 685, 3),
    (2248, 688, 2),
    (2250, 690, 2),
    (2260, 692, 2),
    (2263, 694, 2),
    (2278, 696, 3),
    (2280, 699, 3),
    (2283, 702, 3),
    (2285, 705, 3),
    (2287, 708, 3),
    (2292, 711, 2),
    (2302, 713, 4),
    (2304, 717, 2),
    (2307, 719, 2),
    (2312, 721, 2),
    (2314, 723, 2),
    (2317, 725, 2),
    (2348, 727, 3),
    (2351, 730, 3),
    (2353, 733, 3),
    (2355, 736, 3),
    (2358, 739, 2),
    (2372, 741, 2),
    (2374, 743, 2),
    (2384, 745, 2),
    (2389, 747, 2),
    (2397, 749, 3),
    (2400, 752, 2),
    (2402, 754, 2),
    (2405, 756, 3),
    (2436, 759, 3),
    (2442, 762, 2),
    (2443, 764, 2),
    (2447, 766, 2),
    (2448, 768, 3),
    (2467, 771, 3),
    (2472, 774, 2),
    (2474, 776, 2),
    (2477, 778, 3),
    (2480, 781, 2),
    (2484, 783, 2),
    (2487, 785, 3),
    (2490, 788, 3),
    (2493, 791, 2),
    (2495, 793, 2),
    (2498, 795, 3),
    (2501, 798, 3),
    (2551, 801, 2),
    (2554, 803, 2),
    (2560, 805, 2),
    (2569, 807, 3),
    (2575, 810, 2),
    (2576, 812, 2),
    (2577, 814, 2),
    (2578, 816, 2),
    (2579, 818, 2),
    (2580, 820, 2),
    (2585, 822, 3),
    (2587, 825, 2),
    (2588, 827, 2),
    (2591, 829, 2),
    (2619, 831, 3),
    (2620, 834, 2),
    (2622, 836, 2),
    (2627, 838, 3),
    (2628, 841, 2),
    (2633, 843, 2),
    (2638, 845, 3),
    (2639, 848, 2),
    (2644, 850, 2),
    (2652, 852, 3),
    (2654, 855, 2),
    (2655, 857, 2),
    (2658, 859, 3),
    (2659, 862, 3),
    (2662, 865, 2),
    (2682, 867, 3),
    (2685, 870, 3),
    (2712, 873, 3),
    (2713, 876, 3),
    (2768, 879, 3),
    (2774, 882, 3),
    (2779, 885, 3),
    (2790, 888, 2),
    (2793, 890, 2),
    (2798, 892, 2),
    (2800, 894, 2),
    (2805, 896, 2),
    (2806, 898, 2),
    (2824, 900, 2),
    (2831, 902, 2),
    (2835, 904, 2),
    (2839, 906, 2),
    (2843, 908, 2),
    (2849, 910, 2),
    (2853, 912, 2),
    (2857, 914, 2),
    (2861, 916, 3),
    (2866, 919, 2),
    (2867, 921, 2),
    (2870, 923, 2),
    (2871, 925, 2),
    (2874, 927, 2),
    (2881, 929, 2),
    (2886, 931, 2),
    (2890, 933, 2),
    (2894, 935, 2),
    (2896, 937, 3),
    (2900, 940, 2),
    (2905, 942, 2),
    (2909, 944, 2),
    (2914, 946, 2),
    (2917, 948, 2),
    (2920, 950, 3),
    (2923, 953, 2),
    (2928, 955, 2),
    (2935, 957, 2),
    (2940, 959, 3),
    (2944, 962, 3),
    (2947, 965, 2),
    (2950, 967, 2),
    (2968, 969, 3),
    (2973, 972, 3),
    (2974, 975, 2),
    (2978, 977, 3),
    (2980, 980, 2),
    (2981, 982, 2),
    (2986, 984, 3),
    (2989, 987, 3),
    (2994, 990, 3),
    (2998, 993, 3),
    (3003, 996, 3),
    (3008, 999, 2),
    (3009, 1001, 2),
    (3014, 1003, 2),
    (3018, 1005, 3),
    (3027, 1008, 2),
    (3028, 1010, 2),
    (3034, 1012, 2),
    (3042, 1014, 3),
    (3045, 1017, 3),
    (3048, 1020, 2),
    (3051, 1022, 2),
    (3056, 1024, 2),
    (3063, 1026, 2),
    (3066, 1028, 3),
    (3069, 1031, 2),
    (3077, 1033, 3),
    (3084, 1036, 2),
    (3087, 1038, 3),
    (3097, 1041, 3),
    (3098, 1044, 2),
    (3103, 1046, 2),
    (3104, 1048, 2),
    (3110, 1050, 2),
    (3117, 1052, 2),
    (3137, 1054, 2),
    (3138, 1056, 2),
    (3141, 1058, 2),
    (3146, 1060, 2),
    (3156, 1062, 2),
    (3157, 1064, 2),
    (3163, 1066, 2),
    (3165, 1068, 3),
    (3183, 1071, 2),
    (3191, 1073, 2),
    (3196, 1075, 2),
    (3201, 1077, 2),
    (3202, 1079, 2),
    (3203, 1081, 2),
    (3207, 1083, 3),
    (3216, 1086, 2),
    (3219, 1088, 2),
    (3225, 1090, 3),
    (3267, 1093, 3),
    (3271, 1096, 3),
    (3274, 1099, 3),
    (3277, 1102, 3),
    (3280, 1105, 3),
    (3290, 1108, 3),
    (3292, 1111, 3),
    (3293, 1114, 3),
    (3297, 1117, 2),
    (3337, 1119, 3),
    (3343, 1122, 3),
    (3345, 1125, 2),
    (3350, 1127, 3),
    (3356, 1130, 2),
    (3358, 1132, 2),
    (3365, 1134, 3),
    (3376, 1137, 3),
    (3377, 1140, 2),
    (3379, 1142, 3),
    (3383, 1145, 3),
    (3388, 1148, 3),
    (3391, 1151, 2),
    (3399, 1153, 3),
    (3410, 1156, 3),
    (3413, 1159, 2),
    (3416, 1161, 2),
    (3418, 1163, 2),
    (3425, 1165, 2),
    (3429, 1167, 2),
    (3435, 1169, 2),
    (3440, 1171, 2),
    (3441, 1173, 2),
    (3444, 1175, 2),
    (3445, 1177, 2),
    (3451, 1179, 3),
    (3468, 1182, 3),
    (3473, 1185, 3),
    (3477, 1188, 3),
    (3498, 1191, 3),
    (3505, 1194, 3),
    (3515, 1197, 2),
    (3521, 1199, 2),
    (3525, 1201, 2),
    (3528, 1203, 2),
    (3534, 1205, 2),
    (3537, 1207, 2),
    (3551, 1209, 3),
    (3556, 1212, 3),
    (3563, 1215, 3),
    (3570, 1218, 2),
    (3580, 1220, 2),
    (3581, 1222, 2),
    (3583, 1224, 2),
    (3611, 1226, 2),
    (3612, 1228, 2),
    (3613, 1230, 2),
    (3614, 1232, 2),
    (3639, 1234, 2),
    (3644, 1236, 2),
    (3659, 1238, 2),
    (3685, 1240, 2),
    (3700, 1242, 2),
    (3703, 1244, 2),
    (3706, 1246, 2),
    (3707, 1248, 2),
    (3708, 1250, 2),
    (3719, 1252, 2),
    (3749, 1254, 2),
    (3768, 1256, 2),
    (3772, 1258, 2),
    (3780, 1260, 3),
    (3782, 1263, 2),
    (3787, 1265, 2),
    (3788, 1267, 2),
    (3789, 1269, 2),
    (3795, 1271, 2),
    (3796, 1273, 2),
    (3803, 1275, 2),
    (3804, 1277, 2),
    (3817, 1279, 2),
    (3847, 1281, 2),
    (3851, 1283, 2),
    (3863, 1285, 2),
    (3865, 1287, 2),
    (3877, 1289, 2),
    (3879, 1291, 2),
    (3885, 1293, 2),
    (3891, 1295, 3),
    (3901, 1298, 2),
    (3907, 1300, 3),
    (3915, 1303, 3),
    (3924, 1306, 2),
    (3932, 1308, 3),
    (3941, 1311, 3),
    (3947, 1314, 2),
    (3957, 1316, 3),
    (3963, 1319, 2),
    (3965, 1321, 3),
    (3973, 1324, 2),
    (3974, 1326, 2),
    (3983, 1328, 3),
    (3994, 1331, 3),
    (4001, 1334, 3),
    (4008, 1337, 3),
    (4016, 1340, 3),
    (4026, 1343, 3),
    (4034, 1346, 2),
    (4040, 1348, 3),
    (4042, 1351, 3),
    (4049, 1354, 2),
    (4055, 1356, 3),
    (4061, 1359, 3),
    (4068, 1362, 2),
    (4074, 1364, 2),
    (4075, 1366, 2),
    (4093, 1368, 2),
    (4100, 1370, 2),
    (4103, 1372, 2),
    (4138, 1374, 2),
    (4141, 1376, 2),
    (4142, 1378, 2),
    (4143, 1380, 2),
    (4175, 1382, 2),
    (4193, 1384, 2),
    (4207, 1386, 2),
    (4209, 1388, 2),
    (4210, 1390, 2),
    (4214, 1392, 2),
    (4216, 1394, 2),
    (4220, 1396, 2),
    (4224, 1398, 2),
    (4229, 1400, 3),
    (4230, 1403, 2),
    (4231, 1405, 2),
    (4232, 1407, 2),
    (4234, 1409, 3),
    (4237, 1412, 2),
    (4238, 1414, 2),
    (4241, 1416, 2),
    (4243, 1418, 2),
    (4249, 1420, 2),
    (4250, 1422, 2),
    (4255, 1424, 2),
    (4257, 1426, 2),
    (4258, 1428, 2),
    (4259, 1430, 2),
    (4260, 1432, 3),
    (4261, 1435, 2),
    (4265, 1437, 3),
    (4270, 1440, 3),
    (4272, 1443, 3),
    (4288, 1446, 2),
    (4302, 1448, 2),
    (4303, 1450, 2),
    (4304, 1452, 2),
    (4306, 1454, 2),
    (4315, 1456, 2),
    (4339, 1458, 3),
    (4361, 1461, 2),
    (4369, 1463, 2),
    (4376, 1465, 2),
    (4378, 1467, 2),
    (4380, 1469, 2),
    (4385, 1471, 2),
    (4390, 1473, 2),
    (4394, 1475, 2),
    (4410, 1477, 2),
    (4412, 1479, 3),
    (4440, 1482, 2),
    (4443, 1484, 2),
    (4453, 1486, 2),
    (4454, 1488, 2),
    (4455, 1490, 2),
    (4458, 1492, 2),
    (4464, 1494, 2),
    (4465, 1496, 2),
    (4466, 1502, 2),
    (4469, 1498, 2),
    (4471, 1500, 2),
    (4475, 1504, 2),
    (4476, 1506, 2),
    (4481, 1508, 2),
    (4486, 1510, 2),
    (4487, 1512, 2),
    (4489, 1514, 2),
    (4500, 1516, 2),
    (4515, 1518, 2),
    (4516, 1520, 2),
    (4525, 1522, 2),
    (4526, 1524, 2),
    (4527, 1526, 2),
    (4552, 1528, 2),
    (4558, 1530, 2),
    (4559, 1532, 2),
    (4560, 1534, 2),
    (4562, 1536, 2),
    (4565, 1538, 2),
    (4567, 1540, 2),
    (4569, 1542, 2),
    (4572, 1544, 2),
    (4575, 1546, 2),
    (4579, 1548, 2),
    (4583, 1550, 2),
    (4586, 1552, 2),
    (4589, 1554, 2),
    (4596, 1556, 2),
    (4604, 1558, 3),
    (4605, 1561, 2),
    (4610, 1563, 3),
    (4611, 1566, 2),
    (4613, 1568, 2),
    (4614, 1570, 2),
    (4617, 1572, 2),
    (4621, 1574, 2),
    (4627, 1576, 2),
    (4682, 1578, 2),
    (4700, 1580, 2),
    (4706, 1582, 2),
    (4709, 1584, 2),
    (4713, 1586, 2),
    (4714, 1588, 2),
    (4715, 1590, 2),
    (4719, 1592, 2),
    (4734, 1594, 2),
    (4738, 1596, 2),
    (4746, 1598, 2),
    (4755, 1600, 2),
    (4756, 1602, 2),
    (4760, 1604, 2),
    (4762, 1606, 2),
    (4768, 1608, 2),
    (4769, 1610, 2),
    (4773, 1618, 2),
    (4775, 1612, 2),
    (4776, 1614, 2),
    (4777, 1616, 2),
    (4778, 1620, 2),
    (4797, 1622, 2),
    (4799, 1624, 2),
    (4802, 1626, 3),
    (4807, 1629, 3),
    (4808, 1632, 2),
    (4810, 1634, 2),
    (4830, 1636, 3),
    (4844, 1639, 3),
    (4846, 1642, 2),
    (4852, 1644, 2),
    (4861, 1646, 3),
    (4863, 1649, 2),
    (4873, 1651, 2),
    (4874, 1653, 2),
    (4883, 1655, 2),
    (4888, 1657, 2),
    (4907, 1659, 3),
    (4935, 1662, 3),
    (4947, 1665, 3),
    (4949, 1668, 3),
    (4963, 1671, 2),
    (4964, 1673, 2),
    (4965, 1675, 2),
    (4967, 1677, 3),
    (4980, 1680, 3),
    (4982, 1683, 3),
    (4988, 1686, 2),
    (4999, 1688, 3),
    (5000, 1691, 2),
    (5002, 1693, 2),
    (5003, 1695, 2),
    (5011, 1697, 2),
    (5016, 1699, 3),
    (5038, 1702, 3),
    (5039, 1705, 2),
    (5056, 1707, 2),
    (5063, 1709, 2),
    (5064, 1711, 2),
    (5071, 1713, 2),
    (5072, 1715, 2),
    (5102, 1717, 2),
    (5119, 1719, 3),
    (5146, 1722, 2),
    (5151, 1724, 2),
    (5153, 1726, 2),
    (5159, 1728, 2),
    (5160, 1730, 2),
    (5163, 1732, 2),
    (5169, 1734, 2),
    (5170, 1736, 2),
    (5171, 1738, 2),
    (5177, 1740, 2),
    (5213, 1742, 4),
    (5225, 1746, 2),
    (5234, 1748, 2),
    (5300, 1750, 2),
    (5307, 1752, 2),
    (5320, 1754, 2),
    (5324, 1756, 2),
    (5327, 1758, 2),
    (5328, 1760, 2),
    (5338, 1762, 2),
    (5340, 1764, 2),
    (5342, 1766, 2),
    (5343, 1768, 2),
    (5344, 1770, 2),
    (5345, 1772, 2),
    (5346, 1774, 5),
    (5347, 1779, 3),
    (5365, 1782, 3),
    (5412, 1785, 2),
    (5431, 1787, 2),
    (5435, 1789, 2),
    (5438, 1791, 2),
    (5456, 1793, 2),
    (5462, 1795, 2),
    (5463, 1797, 2),
    (5464, 1799, 2),
    (5470, 1801, 2),
    (5473, 1803, 2),
    (5474, 1805, 2),
    (5486, 1807, 2),
    (5489, 1809, 2),
    (5490, 1811, 2),
    (5503, 1813, 2),
    (5506, 1815, 2),
    (5513, 1817, 3),
    (5517, 1820, 3),
    (5522, 1823, 2),
    (5524, 1825, 2),
    (5525, 1827, 2),
    (5527, 1829, 2),
    (5529, 1831, 2),
    (5532, 1833, 2),
    (5589, 1835, 2),
    (5590, 1837, 2),
    (5597, 1839, 2),
    (5601, 1841, 2),
    (5616, 1843, 3),
    (5636, 1846, 2),
    (5637, 1848, 2),
    (5639, 1850, 2),
    (5641, 1852, 2),
    (5645, 1854, 2),
    (5646, 1856, 2),
    (5658, 1858, 2),
    (5661, 1860, 2),
    (5663, 1862, 2),
    (5664, 1864, 2),
    (5665, 1866, 2),
    (5723, 1868, 2),
    (5749, 1870, 2),
    (5794, 1872, 3),
    (5796, 1875, 2),
    (5816, 1877, 2),
    (5832, 1879, 2),
    (5842, 1881, 3),
    (5843, 1884, 2),
    (5844, 1886, 2),
    (5848, 1888, 3),
    (5849, 1891, 2),
    (5850, 1893, 2),
    (5852, 1895, 3),
    (5858, 1898, 3),
    (5859, 1901, 2),
    (5860, 1903, 2),
    (5864, 1905, 3),
    (5865, 1908, 2),
    (5866, 1910, 2),
    (5868, 1912, 3),
    (5890, 1915, 2),
    (5896, 1917, 2),
    (5903, 1919, 2),
    (5921, 1921, 2),
    (5964, 1923, 2),
    (5965, 1925, 2),
    (5983, 1927, 2),
    (5986, 1929, 2),
    (5990, 1931, 2),
    (6031, 1933, 2),
    (6043, 1935, 3),
    (6057, 1938, 2),
    (6079, 1940, 2),
    (6140, 1942, 3),
    (6146, 1945, 3),
    (6153, 1948, 3),
    (6183, 1951, 3),
    (6194, 1954, 3),
    (6195, 1957, 2),
    (6208, 1959, 3),
    (6216, 1962, 3),
    (6220, 1965, 3),
    (6224, 1968, 3),
    (6230, 1971, 3),
    (6237, 1974, 2),
    (6241, 1976, 2),
    (6285, 1978, 2),
    (6287, 1980, 2),
    (6290, 1982, 3),
    (6295, 1985, 3),
    (6296, 1988, 3),
    (6298, 1991, 2),
    (6384, 1993, 2),
    (6389, 1995, 3),
    (6392, 1998, 3),
    (6395, 2001, 2),
    (6397, 2003, 2),
    (6400, 2005, 3),
    (6403, 2008, 2),
    (6407, 2010, 2),
    (6411, 2012, 3),
    (6414, 2015, 2),
    (6425, 2017, 3),
    (6429, 2020, 2),
    (6432, 2022, 2),
    (6438, 2024, 2),
    (6441, 2026, 3),
    (6444, 2029, 2),
    (6450, 2031, 3),
    (6453, 2034, 2),
    (6456, 2036, 2),
    (6459, 2038, 2),
    (6462, 2040, 2),
    (6465, 2042, 3),
    (6472, 2045, 2),
    (6475, 2047, 2),
    (6483, 2049, 2),
    (6487, 2051, 2),
    (6489, 2053, 2),
    (6502, 2055, 2),
    (6507, 2057, 2),
    (6520, 2059, 2),
    (6524, 2061, 2),
    (6525, 2063, 2),
    (6526, 2065, 2),
    (6527, 2067, 2),
    (6528, 2069, 2),
    (6529, 2071, 2),
    (6530, 2073, 2),
    (6531, 2075, 2),
    (6532, 2077, 2),
    (6533, 2079, 2),
    (6534, 2081, 2),
    (6535, 2083, 2),
    (6536, 2085, 2),
    (6537, 2087, 2),
    (6538, 2089, 2),
    (6539, 2091, 2),
    (6540, 2093, 2),
    (6545, 2095, 2),
    (6550, 2097, 2),
    (6553, 2099, 2),
    (6561, 2101, 2),
    (6566, 2103, 2),
    (6603, 2105, 2),
    (6607, 2107, 3),
    (6608, 2110, 2),
    (6609, 2112, 2),
    (6613, 2114, 2),
    (6614, 2116, 3),
    (6615, 2119, 2),
    (6618, 2121, 2),
    (6622, 2123, 3),
    (6623, 2126, 2),
    (6632, 2128, 2),
    (6633, 2130, 3),
    (6634, 2133, 2),
    (6637, 2135, 2),
    (6644, 2137, 2),
    (6652, 2139, 2),
    (6654, 2141, 2),
    (6655, 2143, 2),
    (6656, 2145, 2),
    (6670, 2147, 3),
    (6682, 2150, 3),
    (6692, 2153, 2),
    (6705, 2155, 2),
    (6723, 2157, 2),
    (6727, 2159, 2),
    (6728, 2161, 2),
    (6753, 2163, 2),
    (6759, 2165, 3),
    (6760, 2168, 2),
    (6770, 2170, 2),
    (6771, 2172, 2),
    (6772, 2174, 2),
    (6773, 2176, 2),
    (6774, 2178, 2),
    (6779, 2180, 2),
    (6784, 2182, 2),
    (6787, 2184, 2),
    (6788, 2186, 2),
    (6796, 2188, 2),
    (6797, 2190, 2),
    (6814, 2192, 2),
    (6815, 2194, 2),
    (6851, 2196, 2),
    (6852, 2198, 2),
    (6853, 2200, 2),
    (6854, 2202, 2),
    (6855, 2204, 2),
    (6856, 2206, 2),
    (6857, 2208, 2),
    (6858, 2210, 2),
    (6859, 2212, 2),
    (6860, 2214, 2),
    (6861, 2216, 2),
    (6862, 2218, 2),
    (6863, 2220, 2),
    (6864, 2222, 2),
    (6865, 2224, 2),
    (6866, 2226, 2),
    (6867, 2228, 2),
    (6868, 2230, 2),
    (6869, 2232, 2),
    (6870, 2234, 2),
    (6871, 2236, 2),
    (6872, 2238, 2),
    (6873, 2240, 2),
    (6874, 2242, 2),
    (6875, 2244, 2),
    (6876, 2246, 2),
    (6877, 2248, 2),
    (6878, 2250, 2),
    (6879, 2252, 2),
    (6880, 2254, 2),
    (6881, 2256, 2),
    (6882, 2258, 2),
    (6883, 2260, 2),
    (6884, 2262, 2),
    (6885, 2264, 2),
    (6886, 2266, 2),
    (6887, 2268, 2),
    (6888, 2270, 2),
    (6889, 2272, 2),
    (6890, 2274, 2),
    (6891, 2276, 2),
    (6892, 2278, 2),
    (6893, 2280, 2),
    (6895, 2282, 2),
    (6896, 2284, 2),
    (6897, 2286, 2),
    (6898, 2288, 2),
    (6899, 2290, 2),
    (6900, 2292, 2),
    (6901, 2294, 2),
    (6902, 2296, 2),
    (6903, 2298, 2),
    (6904, 2300, 2),
    (6905, 2302, 2),
    (6906, 2304, 2),
    (6907, 2306, 2),
    (6908, 2308, 2),
    (6909, 2310, 2),
    (6910, 2312, 2),
    (6911, 2314, 2),
    (6912, 2316, 2),
    (6913, 2318, 2),
    (6914, 2320, 2),
    (6915, 2322, 2),
    (6916, 2324, 2),
    (6917, 2326, 2),
    (6918, 2328, 2),
    (6919, 2330, 2),
    (6920, 2332, 2),
    (6921, 2334, 2),
    (6922, 2336, 2),
    (6923, 2338, 2),
    (6924, 2340, 2),
    (6925, 2342, 2),
    (6926, 2344, 2),
    (6927, 2346, 2),
    (6928, 2348, 2),
    (6929, 2350, 2),
    (6930, 2352, 2),
    (6931, 2354, 2),
    (6932, 2356, 2),
    (6933, 2358, 2),
    (6934, 2360, 2),
    (6935, 2362, 2),
    (6936, 2364, 2),
    (6937, 2366, 2),
    (6938, 2368, 2),
    (6939, 2370, 2),
    (6940, 2372, 2),
    (6941, 2374, 2),
    (6942, 2376, 2),
    (6943, 2378, 2),
    (6944, 2380, 2),
    (6945, 2382, 2),
    (6946, 2384, 2),
    (6947, 2386, 2),
    (6948, 2388, 2),
    (6949, 2390, 2),
    (6950, 2392, 2),
    (6951, 2394, 2),
    (6952, 2396, 2),
    (6953, 2398, 2),
    (6954, 2400, 2),
    (6955, 2402, 2),
    (6956, 2404, 2),
    (6957, 2406, 2),
    (6958, 2408, 2),
    (6959, 2410, 2),
    (6960, 2412, 2),
    (6961, 2414, 2),
    (6962, 2416, 2),
    (6963, 2418, 2),
    (6964, 2420, 2),
    (6965, 2422, 2),
    (6966, 2424, 2),
    (6967, 2426, 2),
    (6968, 2428, 2),
    (6969, 2430, 2),
    (6970, 2432, 2),
    (6971, 2434, 2),
    (6972, 2436, 2),
    (6984, 2438, 2),
    (6992, 2440, 2),
    (6997, 2442, 2),
    (7000, 2444, 2),
    (7003, 2446, 2),
    (7011, 2448, 3),
    (7020, 2451, 2),
    (7025, 2453, 2),
    (7036, 2455, 2),
    (7040, 2457, 2),
    (7045, 2459, 2),
    (7048, 2461, 2),
    (7049, 2463, 2),
    (7053, 2465, 2),
    (7081, 2467, 2),
    (7088, 2469, 3),
    (7103, 2472, 2),
    (7106, 2474, 2),
    (7110, 2476, 3),
    (7114, 2479, 2),
    (7158, 2481, 2),
    (7170, 2483, 2),
    (7223, 2485, 3),
    (7225, 2488, 3),
    (7249, 2491, 2),
    (7250, 2493, 2),
    (7252, 2495, 2),
    (7253, 2497, 2),
    (7291, 2499, 2),
    (7292, 2501, 2),
    (7294, 2503, 2),
    (7295, 2505, 2),
    (7296, 2507, 2),
    (7297, 2509, 2),
    (7298, 2511, 2),
    (7299, 2513, 2),
    (7300, 2515, 2),
    (7301, 2517, 2),
    (7302, 2519, 2),
    (7303, 2521, 2),
    (7304, 2523, 2),
    (7313, 2525, 2),
    (7324, 2527, 3),
    (7325, 2530, 2),
    (7327, 2532, 2),
    (7328, 2534, 2),
    (7334, 2536, 2),
    (7335, 2538, 2),
    (7342, 2540, 2),
    (7343, 2542, 2),
    (7349, 2544, 3),
    (7395, 2547, 3),
    (7407, 2550, 2),
    (7409, 2552, 2),
    (7410, 2554, 2),
    (7411, 2556, 2),
    (7488, 2558, 2),
    (7540, 2560, 2),
    (7544, 2562, 2),
    (7552, 2564, 2),
    (7569, 2566, 2),
    (7573, 2568, 3),
    (7577, 2571, 3),
    (7580, 2574, 3),
    (7582, 2577, 3),
    (7583, 2580, 2),
    (7586, 2582, 2),
    (7587, 2584, 2),
    (7588, 2586, 2),
    (7613, 2588, 2),
    (7616, 2590, 2),
    (7624, 2592, 2),
    (7627, 2594, 2),
    (7634, 2596, 2),
    (7635, 2598, 2),
    (7636, 2600, 2),
    (7649, 2602, 2),
    (7658, 2604, 2),
    (7678, 2606, 2),
    (7681, 2608, 3),
];

pub static SEGMENT_CLASS_TYPES_DATA: &[u32] = &[
    68, 3, 79, 3, 79, 3, 3, 81, 3, 83, 3, 86, 3, 81, 88, 68,
    3, 164, 3, 3, 194, 3, 196, 3, 81, 3, 81, 200, 3, 81, 202, 3,
    81, 3, 81, 202, 3, 206, 212, 3, 3, 214, 3, 227, 3, 228, 230, 4,
    233, 3, 3, 236, 3, 242, 228, 230, 4, 3, 81, 246, 248, 3, 3, 81,
    3, 86, 3, 83, 3, 81, 202, 3, 81, 269, 3, 271, 81, 3, 271, 81,
    3, 81, 3, 81, 286, 3, 290, 320, 3, 323, 3, 325, 3, 320, 3, 323,
    3, 325, 3, 3, 81, 3, 81, 269, 338, 3, 340, 3, 3, 81, 202, 3,
    81, 3, 344, 3, 81, 3, 344, 3, 81, 3, 81, 3, 81, 3, 81, 3,
    81, 3, 81, 3, 81, 3, 362, 81, 3, 81, 269, 3, 81, 3, 81, 246,
    3, 81, 3, 81, 3, 81, 3, 83, 3, 86, 3, 81, 3, 395, 3, 395,
    3, 81, 3, 403, 3, 418, 81, 3, 395, 3, 422, 81, 3, 426, 3, 418,
    81, 3, 418, 81, 3, 395, 3, 422, 81, 3, 344, 3, 206, 3, 344, 454,
    3, 454, 3, 3, 459, 3, 459, 3, 472, 3, 472, 3, 81, 286, 3, 459,
    3, 459, 3, 81, 246, 3, 511, 81, 3, 418, 81, 3, 81, 246, 519, 520,
    3, 3, 418, 81, 3, 418, 81, 3, 81, 246, 3, 290, 3, 83, 3, 86,
    557, 3, 3, 83, 3, 81, 269, 3, 362, 81, 3, 511, 81, 3, 511, 81,
    3, 81, 286, 3, 511, 81, 3, 362, 81, 3, 290, 3, 81, 286, 3, 81,
    202, 3, 81, 286, 3, 418, 81, 3, 290, 3, 290, 3, 81, 3, 81, 246,
    568, 3, 3, 418, 81, 3, 418, 81, 3, 81, 246, 3, 81, 269, 3, 81,
    246, 3, 362, 81, 3, 81, 286, 3, 81, 3, 81, 286, 3, 81, 3, 81,
    246, 3, 81, 3, 81, 202, 3, 344, 3, 344, 3, 290, 3, 83, 3, 86,
    570, 3, 3, 83, 3, 81, 269, 3, 290, 3, 362, 81, 3, 81, 574, 3,
    576, 3, 578, 3, 578, 3, 578, 3, 81, 574, 3, 81, 202, 3, 81, 202,
    3, 81, 202, 3, 81, 202, 3, 81, 202, 3, 271, 81, 3, 290, 3, 81,
    269, 3, 81, 269, 3, 81, 202, 3, 395, 3, 81, 619, 3, 418, 81, 3,
    81, 619, 454, 3, 3, 81, 619, 3, 81, 269, 3, 81, 626, 3, 81, 626,
    3, 81, 269, 3, 81, 630, 3, 81, 574, 3, 576, 3, 576, 3, 576, 3,
    344, 3, 81, 3, 81, 630, 3, 290, 3, 418, 81, 3, 395, 3, 422, 81,
    3, 426, 3, 418, 81, 3, 418, 81, 3, 395, 3, 422, 81, 3, 344, 3,
    206, 3, 344, 454, 3, 454, 3, 3, 459, 3, 459, 3, 472, 3, 472, 3,
    81, 286, 3, 459, 3, 459, 3, 81, 246, 3, 511, 81, 3, 290, 3, 640,
    3, 418, 81, 3, 81, 246, 3, 81, 246, 520, 3, 3, 418, 81, 3, 418,
    81, 3, 81, 246, 3, 81, 246, 3, 81, 269, 3, 650, 3, 81, 286, 3,
    81, 3, 81, 286, 3, 81, 3, 81, 3, 81, 3, 81, 3, 81, 3, 81,
    3, 81, 3, 81, 3, 81, 3, 81, 3, 81, 3, 81, 3, 81, 3, 81,
    3, 81, 661, 3, 81, 246, 3, 81, 661, 3, 362, 81, 3, 395, 3, 81,
    3, 81, 3, 81, 269, 3, 418, 81, 3, 395, 3, 422, 81, 3, 418, 81,
    3, 395, 3, 422, 81, 3, 418, 81, 3, 418, 81, 3, 418, 81, 3, 81,
    246, 3, 418, 81, 3, 206, 3, 344, 3, 81, 3, 418, 81, 3, 418, 81,
    3, 81, 246, 3, 81, 269, 3, 290, 3, 81, 246, 3, 418, 81, 3, 680,
    3, 680, 3, 680, 3, 680, 3, 100, 3, 344, 3, 100, 3, 344, 3, 100,
    3, 344, 3, 344, 233, 3, 3, 727, 3, 729, 3, 206, 3, 418, 81, 3,
    344, 3, 395, 3, 731, 3, 733, 3, 734, 3, 3, 680, 4, 3, 418, 81,
    3, 770, 3, 777, 3, 236, 3, 785, 3, 418, 81, 3, 418, 81, 3, 418,
    81, 3, 418, 81, 3, 418, 81, 3, 206, 3, 228, 230, 4, 3, 799, 3,
    805, 3, 344, 3, 799, 3, 805, 3, 81, 246, 3, 511, 81, 3, 511, 81,
    3, 81, 246, 3, 4, 3, 830, 3, 206, 3, 344, 3, 344, 3, 422, 81,
    3, 395, 3, 426, 3, 422, 81, 3, 418, 81, 3, 845, 3, 847, 3, 83,
    3, 854, 853, 3, 81, 246, 3, 81, 3, 81, 3, 418, 81, 3, 81, 3,
    81, 3, 271, 81, 3, 362, 81, 3, 83, 3, 86, 3, 511, 81, 3, 81,
    269, 3, 81, 3, 81, 3, 81, 3, 861, 4, 3, 290, 3, 290, 3, 290,
    3, 290, 3, 290, 3, 290, 3, 290, 4, 3, 680, 3, 290, 3, 344, 3,
    418, 81, 3, 290, 3, 344, 3, 418, 81, 3, 290, 3, 344, 3, 418, 81,
    3, 290, 3, 344, 3, 418, 81, 3, 344, 3, 206, 3, 422, 81, 3, 81,
    897, 3, 344, 3, 81, 246, 3, 418, 81, 3, 418, 81, 3, 418, 81, 3,
    418, 81, 3, 418, 81, 3, 418, 81, 3, 81, 3, 86, 3, 395, 3, 395,
    3, 83, 3, 86, 3, 81, 3, 290, 3, 290, 3, 290, 3, 290, 3, 290,
    3, 290, 3, 81, 3, 271, 81, 3, 290, 3, 81, 3, 290, 3, 81, 3,
    290, 3, 290, 3, 290, 3, 290, 3, 290, 3, 81, 286, 3, 290, 3, 290,
    3, 290, 3, 81, 3, 395, 3, 422, 81, 3, 344, 3, 81, 3, 344, 3,
    362, 81, 3, 81, 269, 3, 963, 3, 963, 3, 81, 246, 3, 418, 81, 3,
    395, 3, 422, 81, 3, 426, 3, 472, 3, 81, 246, 3, 81, 976, 3, 81,
    246, 3, 81, 246, 3, 418, 81, 3, 426, 3, 472, 3, 650, 3, 81, 976,
    3, 83, 3, 86, 3, 344, 3, 511, 81, 3, 81, 246, 3, 403, 3, 981,
    3, 981, 3, 459, 3, 81, 286, 3, 344, 3, 81, 246, 3, 459, 3, 81,
    286, 3, 81, 246, 3, 989, 3, 206, 3, 991, 3, 81, 3, 290, 3, 290,
    3, 81, 3, 290, 3, 290, 3, 290, 3, 83, 3, 81, 3, 81, 246, 3,
    81, 3, 344, 3, 344, 3, 83, 3, 86, 3, 1016, 3, 81, 574, 3, 578,
    3, 576, 3, 81, 202, 3, 81, 202, 3, 81, 202, 3, 81, 202, 3, 81,
    202, 3, 81, 202, 3, 81, 269, 3, 81, 202, 3, 81, 269, 3, 395, 3,
    418, 81, 3, 81, 619, 3, 1022, 3, 81, 976, 3, 963, 3, 81, 3, 81,
    626, 3, 418, 81, 3, 344, 3, 81, 246, 3, 81, 630, 3, 81, 574, 3,
    576, 3, 81, 246, 3, 418, 81, 3, 395, 3, 426, 3, 395, 3, 344, 3,
    472, 3, 81, 3, 290, 3, 81, 3, 290, 3, 81, 3, 81, 661, 3, 418,
    81, 3, 81, 246, 3, 81, 246, 3, 81, 246, 3, 81, 246, 3, 344, 3,
    206, 3, 395, 3, 395, 3, 1052, 3, 1056, 3, 81, 976, 3, 81, 202, 3,
    81, 246, 3, 989, 3, 83, 3, 83, 3, 1068, 3, 1109, 3, 1111, 3, 1052,
    3, 1113, 3, 1175, 3, 1175, 3, 1175, 3, 1175, 3, 1175, 3, 1175, 1216, 3,
    1218, 3, 3, 1220, 3, 1111, 3, 81, 3, 290, 3, 1281, 3, 81, 246, 248,
    3, 3, 1285, 248, 3, 3, 1287, 3, 81, 3, 1289, 3, 344, 248, 3, 3,
    81, 3, 1310, 3, 1310, 3, 81, 3, 86, 3, 395, 3, 395, 3, 81, 3,
    271, 81, 3, 81, 3, 362, 81, 3, 81, 246, 3, 83, 3, 511, 81, 3,
    81, 246, 3, 81, 3, 81, 202, 3, 81, 3, 81, 246, 3, 83, 3, 86,
    3, 81, 574, 3, 81, 269, 3, 81, 619, 3, 81, 626, 3, 81, 630, 3,
    81, 246, 3, 81, 3, 81, 661, 3, 81, 246, 3, 395, 3, 81, 202, 3,
    81, 246, 3, 344, 3, 1052, 3, 1320, 3, 981, 3, 81, 3, 344, 3, 1347,
    3, 1349, 3, 1351, 3, 1353, 3, 100, 3, 100, 3, 1392, 3, 1394, 3, 830,
    3, 830, 3, 290, 3, 206, 3, 344, 3, 418, 81, 3, 206, 3, 1401, 3,
    344, 3, 81, 88, 3, 1052, 3, 1113, 3, 344, 3, 395, 3, 1406, 3, 1401,
    3, 1410, 3, 729, 3, 1412, 1414, 3, 3, 418, 81, 3, 1416, 3, 81, 246,
    3, 81, 246, 3, 81, 246, 3, 344, 3, 1430, 3, 1433, 3, 1436, 3, 1438,
    3, 344, 3, 81, 246, 3, 81, 3, 344, 3, 830, 3, 206, 3, 344, 3,
    344, 3, 81, 3, 344, 3, 729, 3, 418, 81, 3, 1463, 3, 1470, 3, 1476,
    3, 1478, 3, 991, 3, 1481, 3, 1285, 3, 1485, 3, 1489, 3, 1287, 248, 3,
    248, 3, 3, 1489, 3, 1287, 3, 1493, 3, 1495, 3, 1463, 3, 1287, 3, 206,
    248, 3, 3, 1499, 3, 227, 3, 1470, 248, 3, 3, 847, 3, 1504, 3, 344,
    3, 344, 3, 777, 3, 344, 3, 395, 3, 344, 3, 344, 3, 344, 3, 344,
    3, 344, 3, 395, 1510, 3, 3, 418, 81, 3, 344, 3, 418, 81, 3, 344,
    3, 1517, 3, 1412, 3, 344, 3, 344, 3, 1310, 3, 290, 3, 344, 3, 1349,
    3, 1351, 3, 1523, 3, 777, 3, 1410, 3, 344, 3, 770, 3, 770, 3, 395,
    3, 395, 3, 1529, 3, 290, 3, 344, 3, 83, 3, 853, 3, 83, 3, 83,
    3, 989, 3, 83, 3, 853, 1537, 3, 1539, 3, 3, 81, 202, 3, 81, 200,
    3, 81, 1541, 3, 3, 81, 200, 3, 418, 81, 3, 344, 3, 1470, 3, 418,
    81, 3, 344, 3, 1551, 3, 1068, 3, 344, 3, 1470, 3, 1565, 4, 3, 81,
    269, 3, 81, 976, 3, 81, 269, 3, 206, 1580, 3, 3, 727, 3, 418, 81,
    3, 422, 81, 3, 81, 1587, 3, 459, 3, 418, 81, 3, 206, 3, 344, 3,
    1589, 3, 576, 3, 81, 286, 3, 81, 246, 248, 3, 3, 989, 3, 1603, 3,
    1605, 3, 344, 248, 3, 3, 344, 3, 81, 246, 3, 1495, 3, 1630, 3, 1632,
    3, 1495, 3, 1630, 3, 1495, 3, 1638, 3, 1640, 248, 3, 3, 344, 3, 1,
    4, 5, 3, 344, 3, 344, 3, 344, 3, 344, 3, 344, 3, 344, 3, 344,
    3, 1679, 3, 100, 3, 1406, 3, 1685, 1580, 3, 3, 1687, 3, 1689, 3, 1,
    1312, 4, 5, 3, 1312, 4, 3, 81, 246, 3, 81, 3, 1709, 3, 395, 3,
    1712, 3, 989, 3, 1723, 3, 1725, 3, 1727, 3, 344, 3, 1732, 3, 1734, 3,
    344, 3, 1732, 3, 1734, 3, 344, 3, 1740, 3, 81, 246, 3, 81, 246, 248,
    3, 3, 206, 248, 3, 3, 1632, 3, 1744, 3, 1746, 3, 290, 3, 344, 3,
    1766, 3, 1768, 3, 310, 4, 3, 989, 3, 1410, 3, 1410, 3, 1523, 3, 1785,
    3, 1787, 3, 777, 3, 1410, 3, 1351, 3, 1349, 3, 1353, 3, 1805, 3, 344,
    3, 418, 81, 3, 344, 3, 1481, 3, 1766, 3, 418, 81, 3, 1406, 3, 830,
    3, 418, 81, 3, 1406, 3, 830, 3, 418, 81, 3, 418, 81, 3, 1406, 3,
    830, 3, 418, 81, 3, 1406, 3, 830, 3, 418, 81, 3, 344, 3, 344, 3,
    344, 3, 344, 3, 1849, 3, 1851, 3, 81, 3, 395, 3, 1860, 3, 1875, 3,
    81, 269, 3, 344, 3, 344, 3, 81, 202, 3, 81, 202, 3, 81, 246, 3,
    418, 81, 3, 81, 246, 3, 991, 3, 81, 286, 3, 511, 81, 3, 81, 246,
    3, 81, 269, 3, 271, 81, 3, 290, 3, 1910, 1537, 3, 1539, 3, 3, 81,
    202, 3, 81, 200, 3, 81, 200, 1541, 3, 3, 81, 3, 81, 246, 3, 418,
    81, 3, 83, 3, 86, 3, 271, 81, 3, 81, 3, 81, 3, 81, 246, 3,
    83, 3, 81, 246, 3, 81, 3, 83, 3, 81, 3, 81, 202, 3, 83, 3,
    81, 269, 3, 81, 3, 81, 3, 81, 3, 81, 3, 81, 246, 3, 845, 248,
    3, 3, 344, 3, 1945, 3, 1947, 3, 1476, 3, 1949, 3, 1476, 3, 1951, 3,
    1953, 3, 1289, 3, 1499, 3, 1955, 3, 1416, 3, 1957, 3, 1959, 3, 227, 3,
    1961, 3, 1963, 3, 1470, 3, 1957, 3, 1489, 3, 1476, 3, 1478, 3, 991, 3,
    1949, 3, 1523, 3, 1523, 212, 3, 212, 3, 3, 1972, 3, 418, 81, 212, 3,
    3, 290, 3, 206, 3, 418, 81, 3, 344, 3, 830, 3, 418, 81, 3, 290,
    3, 206, 3, 418, 81, 3, 344, 3, 830, 3, 81, 3, 1476, 3, 227, 3,
    1470, 3, 1957, 3, 81, 269, 3, 81, 202, 3, 290, 3, 290, 3, 1392, 3,
    395, 3, 1394, 3, 1285, 3, 418, 81, 3, 344, 3, 206, 248, 3, 3, 1499,
    3, 227, 3, 1470, 3, 1285, 3, 344, 3, 344, 248, 3, 3, 227, 3, 1470,
    1218, 3, 212, 3, 3, 1860, 3, 1349, 3, 2016, 3, 2018, 3, 2020, 3, 2022,
    3, 2024, 3, 2026, 2028, 3, 3, 2030, 3, 2032, 3, 2034, 2036, 3, 3, 2038,
    3, 2040, 3, 2042, 3, 2044, 3, 2046, 3, 2048, 3, 2050, 3, 2052, 3, 2054,
    3, 1353, 3, 1351, 3, 2056, 3, 2058, 3, 2060, 3, 2062, 3, 2064, 3, 2066,
    3, 2068, 3, 2070, 3, 2072, 3, 2074, 2076, 3, 3, 2078, 3, 2080, 3, 2082,
    2084, 3, 3, 2086, 3, 2088, 3, 2090, 2092, 3, 2095, 3, 3, 2097, 2099, 3,
    2101, 3, 3, 2103, 3, 2105, 3, 2107, 2109, 3, 3, 2111, 3, 2113, 2115, 3,
    3, 2117, 3, 2119, 2121, 3, 3, 2048, 3, 2124, 2126, 3, 3, 2054, 2128, 3,
    3, 2130, 3, 2132, 3, 2135, 3, 2137, 3, 2139, 3, 2141, 2143, 3, 3, 2145,
    3, 2147, 3, 2149, 3, 2151, 2153, 3, 3, 2155, 2157, 3, 3, 2159, 3, 2161,
    2163, 3, 3, 2165, 2167, 3, 3, 2169, 3, 2171, 2173, 3, 3, 2175, 3, 2177,
    2179, 3, 3, 2181, 2183, 3, 3, 2185, 2187, 3, 3, 2189, 3, 2191, 3, 2193,
    3, 2195, 3, 2197, 3, 2199, 3, 2201, 3, 2203, 3, 2205, 3, 2207, 3, 2209,
    2211, 3, 3, 2213, 2215, 3, 3, 2217, 3, 2219, 3, 2221, 3, 2223, 3, 2225,
    3, 2227, 3, 2229, 2231, 3, 3, 2233, 3, 2235, 3, 2237, 3, 2239, 3, 2241,
    3, 2243, 3, 1785, 3, 1787, 3, 680, 248, 3, 3, 395, 3, 395, 3, 1056,
    3, 4, 9, 3, 81, 3, 344, 3, 2257, 3, 2257, 3, 403, 3, 2260, 3,
    2257, 3, 344, 3, 81, 3, 511, 81, 3, 989, 3, 206, 3, 81, 246, 3,
    1860, 3, 1175, 3, 344, 3, 81, 246, 3, 81, 246, 1218, 3, 1580, 3, 3,
    1052, 3, 1320, 3, 1951, 3, 1953, 3, 1953, 3, 1951, 3, 1289, 3, 1499, 3,
    1955, 3, 1957, 3, 1959, 3, 1489, 3, 1476, 3, 227, 3, 1470, 3, 1476, 3,
    81, 246, 248, 3, 3, 1746, 3, 1953, 3, 81, 3, 1289, 3, 344, 248, 3,
    3, 271, 81, 3, 81, 246, 3, 344, 248, 3, 3, 227, 3, 1470, 3, 1109,
    3, 344, 3, 344, 3, 344, 3, 1481, 3, 4, 2369, 3, 418, 81, 3, 418,
    81, 3, 81, 88, 3, 2372, 3, 2374, 3, 2376, 3, 2378, 248, 3, 248, 3,
    248, 3, 248, 3, 3, 2380, 3, 227, 3, 2382, 3, 2387, 3, 2387, 3, 227,
    3, 4, 5,
];

pub static REGEX_PATTERNS: &[&str] = &[
    r#"^[+\-*/<>=~!@#%^&|`?]+$"#, // [0]
    r#"[A-Z_][A-Z0-9_]*"#, // [1]
    r#"^(NOT)$"#, // [2]
    r#"[A-Z_][A-Z0-9_$]*"#, // [3]
    r#"^(STRUCT|UNION|ENUM)$"#, // [4]
    r#"[\p{L}_][\p{L}\p{N}_$]*"#, // [5]
    r#"^(ALL|ANALYSE|ANALYZE|AND|ANY|ARRAY|AS|ASC|ASYMMETRIC|AUTHORIZATION|BINARY|BOTH|CASE|CAST|CHECK|COLLATE|COLUMN|COMMUTATOR|CONCURRENTLY|CONNECT|CONSTRAINT|CREATE|CROSS|CURRENT_CATALOG|CURRENT_DATE|CURRENT_ROLE|CURRENT_SCHEMA|CURRENT_TIME|CURRENT_TIMESTAMP|DEFAULT|DEFERRABLE|DESC|DISTINCT|DO|ELSE|END|EXCEPT|FALSE|FETCH|FOR|FOREIGN|FREEZE|FROM|FULL|GRANT|GROUP|HASHES|HAVING|ILIKE|IN|INITIALLY|INNER|INTERSECT|INTO|IS|ISNULL|JOIN|LAMBDA|LATERAL|LEADING|LEFT|LIKE|LIMIT|LOCALTIME|LOCALTIMESTAMP|MERGES|NATURAL|NEGATOR|NOT|NOTNULL|NULL|OFFSET|ON|ONLY|OR|ORDER|OUTER|OVERLAPS|PIVOT|PIVOT_LONGER|PIVOT_WIDER|PLACING|PRIMARY|REFERENCES|RETURNING|RIGHT|RIGHTARG|SELECT|SESSION_USER|SIMILAR|SOME|SYMMETRIC|TABLESAMPLE|THEN|TO|TRAILING|TRUE|UNION|UNIQUE|UNPIVOT|USING|VARIADIC|VERBOSE|WHEN|WHERE|WINDOW|WITH)$"#, // [6]
    r#"[A-Z_][A-Z0-9_$]*|"[^"]*""#, // [7]
    r#"[!<>=~@#%^&|`?+\-*/]+"#, // [8]
    r#"(?i)'[0-9a-f]*'"#, // [9]
    r#"'([^']|'')*'"#, // [10]
    r#"'[^0-9A-Fa-f'+\-\s)]'"#, // [11]
];

pub static HINT_STRING_INDICES: &[u32] = &[
    0, 6, 7, 11, 13, 14, 15, 16, 17, 18,
    19, 20, 21, 22, 23, 24, 25, 26, 27, 28,
    29, 30, 31, 32, 33, 34, 35, 36, 37, 38,
    39, 40, 28, 43, 40, 16, 17, 23, 24, 31,
    15, 29, 36, 38, 20, 55, 56, 56, 55, 60,
    14, 63, 65, 22, 18, 19, 21, 25, 26, 27,
    30, 31, 32, 33, 34, 35, 37, 39, 18, 19,
    70, 25, 26, 71, 72, 63, 73, 74, 75, 76,
    77, 78, 16, 31, 85, 90, 92, 94, 95, 98,
    100, 14, 101, 102, 103, 104, 105, 106, 107, 108,
    109, 43, 110, 111, 112, 113, 114, 115, 116, 117,
    118, 119, 120, 121, 122, 123, 124, 103, 16, 23,
    29, 126, 40, 126, 101, 25, 131, 105, 37, 108,
    136, 121, 14, 101, 102, 103, 104, 105, 106, 107,
    109, 43, 110, 111, 112, 113, 114, 115, 116, 117,
    118, 119, 120, 121, 122, 123, 124, 18, 19, 21,
    25, 26, 139, 27, 30, 31, 32, 33, 34, 35,
    37, 39, 139, 122, 15, 31, 106, 144, 102, 104,
    107, 109, 110, 111, 112, 114, 113, 115, 116, 117,
    118, 119, 120, 123, 124, 166, 35, 39, 33, 21,
    27, 30, 32, 34, 26, 18, 19, 179, 76, 78,
    74, 70, 71, 72, 73, 75, 77, 191, 192, 191,
    192, 197, 203, 208, 209, 210, 210, 209, 215, 216,
    217, 218, 221, 222, 223, 224, 226, 232, 238, 241,
    244, 249, 250, 251, 255, 256, 257, 258, 257, 256,
    262, 264, 266, 258, 249, 272, 273, 256, 257, 274,
    258, 275, 249, 272, 273, 275, 275, 249, 272, 273,
    279, 280, 272, 283, 273, 288, 94, 92, 288, 92,
    288, 292, 85, 294, 218, 295, 296, 297, 279, 298,
    299, 300, 301, 302, 303, 280, 304, 210, 305, 306,
    307, 308, 309, 310, 100, 241, 311, 314, 316, 274,
    14, 103, 104, 105, 107, 113, 115, 118, 119, 120,
    121, 70, 72, 63, 73, 76, 322, 327, 103, 104,
    105, 107, 113, 115, 118, 119, 120, 121, 330, 332,
    334, 334, 332, 322, 29, 126, 217, 341, 256, 257,
    258, 342, 341, 341, 258, 347, 299, 350, 352, 347,
    299, 347, 299, 352, 355, 357, 342, 360, 217, 341,
    258, 120, 364, 217, 341, 11, 224, 365, 366, 367,
    17, 368, 20, 21, 24, 26, 369, 370, 27, 30,
    31, 32, 56, 35, 371, 372, 38, 39, 11, 366,
    367, 17, 368, 20, 24, 370, 31, 32, 56, 371,
    38, 375, 366, 367, 368, 24, 370, 370, 371, 382,
    383, 384, 385, 386, 383, 384, 385, 386, 26, 35,
    39, 365, 21, 27, 30, 369, 396, 397, 396, 397,
    401, 372, 217, 251, 405, 406, 341, 407, 408, 409,
    410, 347, 411, 412, 256, 413, 274, 258, 342, 415,
    419, 352, 419, 292, 347, 299, 423, 113, 424, 217,
    341, 347, 274, 427, 258, 38, 258, 38, 429, 430,
    262, 364, 431, 432, 264, 433, 434, 266, 435, 126,
    341, 347, 258, 437, 439, 440, 441, 440, 441, 445,
    232, 446, 447, 448, 449, 450, 410, 451, 427, 32,
    452, 427, 258, 427, 457, 461, 462, 18, 463, 464,
    464, 462, 463, 469, 352, 350, 471, 20, 423, 424,
    350, 423, 424, 474, 347, 474, 476, 479, 481, 406,
    407, 407, 406, 440, 413, 413, 486, 406, 407, 408,
    410, 408, 410, 490, 492, 494, 405, 497, 405, 498,
    498, 500, 501, 500, 501, 409, 410, 409, 411, 292,
    506, 507, 401, 509, 506, 507, 251, 412, 412, 217,
    341, 258, 306, 308, 309, 310, 241, 311, 517, 217,
    251, 405, 406, 341, 407, 408, 409, 410, 347, 411,
    412, 256, 257, 413, 274, 258, 342, 521, 522, 18,
    523, 524, 347, 525, 274, 526, 527, 494, 258, 528,
    529, 530, 531, 521, 526, 529, 521, 534, 526, 529,
    523, 528, 531, 523, 528, 531, 524, 347, 524, 18,
    494, 541, 542, 541, 542, 525, 546, 547, 548, 548,
    546, 547, 522, 527, 530, 521, 522, 555, 18, 523,
    524, 410, 347, 256, 525, 257, 274, 526, 527, 494,
    258, 528, 529, 530, 531, 555, 410, 555, 251, 559,
    555, 410, 257, 274, 258, 559, 562, 451, 565, 251,
    405, 256, 274, 258, 274, 258, 251, 405, 256, 257,
    274, 258, 257, 92, 401, 275, 92, 401, 275, 18,
    274, 494, 258, 555, 18, 410, 256, 257, 274, 494,
    258, 572, 217, 341, 256, 257, 258, 35, 76, 579,
    272, 580, 581, 582, 409, 583, 584, 585, 586, 587,
    588, 589, 590, 591, 592, 593, 355, 590, 593, 593,
    590, 580, 585, 580, 585, 581, 586, 581, 586, 409,
    587, 587, 583, 588, 583, 588, 589, 592, 592, 589,
    579, 584, 579, 584, 582, 591, 582, 591, 299, 306,
    308, 309, 310, 241, 311, 610, 256, 257, 448, 294,
    218, 450, 410, 450, 449, 410, 449, 617, 452, 446,
    447, 410, 447, 624, 232, 446, 447, 448, 449, 450,
    410, 451, 256, 257, 427, 32, 258, 452, 31, 457,
    628, 217, 272, 406, 341, 407, 256, 631, 257, 258,
    632, 631, 632, 638, 217, 251, 405, 406, 341, 407,
    408, 409, 410, 347, 411, 256, 413, 274, 258, 342,
    217, 109, 85, 223, 641, 223, 641, 641, 232, 92,
    217, 251, 405, 406, 341, 407, 408, 409, 410, 347,
    411, 256, 257, 413, 274, 258, 342, 559, 646, 257,
    258, 292, 330, 648, 94, 65, 275, 646, 653, 655,
    217, 251, 341, 256, 257, 258, 657, 659, 555, 410,
    257, 663, 217, 251, 341, 257, 665, 223, 667, 667,
    251, 256, 257, 274, 258, 669, 671, 672, 671, 672,
    85, 675, 676, 675, 675, 676, 679, 682, 683, 684,
    685, 669, 255, 294, 218, 686, 687, 688, 689, 690,
    221, 691, 294, 686, 687, 685, 690, 688, 689, 691,
    295, 705, 296, 709, 710, 711, 712, 713, 714, 715,
    716, 717, 718, 720, 721, 722, 726, 298, 733, 737,
    738, 739, 740, 741, 742, 743, 685, 669, 255, 294,
    218, 686, 687, 688, 288, 744, 689, 679, 419, 745,
    746, 690, 221, 747, 748, 749, 691, 419, 221, 679,
    746, 743, 688, 288, 744, 689, 745, 747, 749, 748,
    755, 756, 757, 761, 746, 764, 280, 279, 767, 768,
    769, 85, 94, 773, 774, 775, 115, 776, 648, 778,
    779, 780, 781, 782, 783, 26, 347, 85, 104, 94,
    107, 773, 774, 775, 115, 776, 120, 648, 275, 447,
    382, 778, 787, 787, 790, 792, 793, 797, 801, 803,
    807, 809, 810, 811, 812, 813, 814, 815, 816, 817,
    818, 819, 820, 821, 822, 743, 685, 669, 85, 255,
    294, 90, 218, 686, 687, 295, 807, 825, 688, 288,
    744, 689, 679, 232, 419, 745, 166, 746, 773, 776,
    401, 210, 690, 221, 747, 748, 749, 691, 424, 423,
    834, 825, 773, 776, 849, 850, 850, 849, 858, 11,
    224, 365, 366, 415, 350, 367, 16, 17, 368, 360,
    20, 21, 509, 24, 26, 131, 27, 572, 29, 30,
    486, 31, 32, 56, 457, 628, 35, 36, 371, 118,
    38, 39, 35, 39, 11, 366, 367, 17, 368, 20,
    24, 26, 131, 572, 29, 486, 32, 56, 457, 628,
    36, 371, 118, 38, 224, 27, 30, 745, 863, 864,
    865, 866, 743, 688, 749, 744, 747, 877, 880, 881,
    882, 883, 506, 885, 886, 887, 682, 710, 890, 891,
    507, 120, 893, 895, 85, 166, 898, 899, 900, 901,
    902, 85, 275, 684, 469, 882, 179, 904, 905, 906,
    907, 908, 909, 910, 905, 904, 907, 910, 909, 908,
    906, 920, 921, 922, 923, 924, 925, 179, 926, 927,
    299, 928, 926, 920, 927, 928, 922, 925, 85, 255,
    923, 924, 921, 938, 939, 275, 497, 942, 945, 85,
    94, 946, 947, 948, 949, 950, 951, 949, 947, 948,
    950, 956, 957, 956, 957, 946, 951, 249, 272, 921,
    273, 947, 948, 949, 256, 36, 386, 292, 306, 308,
    309, 310, 241, 311, 947, 948, 949, 947, 948, 352,
    350, 347, 299, 966, 968, 352, 350, 292, 439, 347,
    299, 352, 350, 973, 116, 117, 21, 978, 978, 982,
    984, 987, 85, 104, 105, 94, 107, 773, 774, 775,
    115, 35, 776, 120, 648, 275, 85, 35, 85, 105,
    993, 866, 21, 995, 996, 997, 998, 999, 27, 332,
    1000, 997, 1000, 997, 1000, 999, 21, 996, 27, 332,
    996, 21, 27, 995, 998, 1009, 1010, 1009, 1010, 14,
    104, 107, 115, 120, 232, 737, 521, 522, 18, 523,
    24, 524, 347, 525, 1014, 526, 527, 494, 258, 528,
    529, 530, 372, 531, 1015, 275, 222, 579, 272, 580,
    581, 582, 65, 409, 583, 584, 585, 586, 587, 588,
    589, 590, 591, 592, 29, 593, 1017, 126, 355, 275,
    222, 579, 272, 580, 581, 582, 65, 409, 583, 584,
    585, 586, 587, 588, 589, 590, 591, 592, 29, 593,
    1017, 126, 355, 1017, 232, 446, 447, 448, 449, 450,
    410, 451, 452, 938, 1023, 1024, 1025, 1023, 1024, 1025,
    85, 232, 1030, 1031, 1030, 1031, 902, 385, 223, 667,
    1034, 1034, 104, 107, 119, 120, 474, 330, 94, 476,
    347, 1037, 797, 474, 476, 347, 474, 476, 1037, 1039,
    1040, 1040, 1039, 1044, 490, 1045, 1045, 1048, 1051, 1053,
    262, 264, 266, 126, 1059, 1060, 1061, 1062, 1062, 1064,
    1064, 429, 430, 431, 432, 1069, 1070, 1071, 819, 880,
    1070, 1074, 1075, 1076, 1077, 1078, 1079, 1080, 1081, 1082,
    1083, 1084, 1085, 1086, 1087, 1088, 1089, 1090, 1091, 1092,
    1093, 1094, 1095, 1096, 1097, 1098, 1099, 1100, 1101, 1102,
    1103, 1104, 1105, 1106, 1107, 1108, 297, 298, 300, 301,
    302, 303, 740, 741, 756, 761, 764, 768, 769, 783,
    811, 812, 817, 818, 1114, 1115, 1116, 1117, 1118, 1119,
    1120, 1121, 1122, 1123, 1124, 1125, 1126, 1127, 1128, 1129,
    1130, 1131, 1132, 1133, 1134, 1135, 1136, 1137, 1138, 1098,
    1102, 1139, 1140, 1141, 1142, 1143, 1144, 1145, 371, 1146,
    1147, 1148, 1149, 1150, 1151, 1152, 1153, 1144, 1127, 1121,
    1122, 1123, 1125, 1118, 1119, 1139, 1116, 1166, 1145, 1140,
    1141, 1142, 1143, 1117, 1115, 1135, 1115, 1135, 1134, 769,
    811, 812, 371, 1151, 769, 811, 812, 1151, 1182, 1151,
    761, 764, 768, 818, 1130, 1131, 1137, 1098, 1102, 1098,
    1130, 1131, 1137, 1102, 817, 1120, 1132, 1133, 1120, 1132,
    1133, 1146, 1149, 1149, 1146, 1150, 1153, 1128, 1129, 1128,
    1129, 1124, 1126, 1136, 1147, 1148, 1114, 1138, 1152, 296,
    210, 297, 1223, 1224, 1225, 1226, 1227, 1228, 1229, 1230,
    1231, 1232, 1233, 1234, 1235, 1236, 1237, 1238, 1239, 1240,
    1241, 1242, 1243, 1244, 1245, 1246, 1247, 1248, 1249, 1250,
    1251, 1252, 1253, 1254, 1255, 297, 298, 300, 301, 302,
    303, 306, 308, 309, 310, 241, 311, 300, 301, 298,
    302, 303, 303, 302, 1262, 1224, 1264, 1265, 1266, 1267,
    1268, 1269, 1225, 1226, 1270, 1271, 1274, 1275, 1276, 1277,
    85, 294, 218, 295, 296, 297, 279, 298, 299, 300,
    301, 302, 303, 280, 304, 210, 305, 306, 307, 308,
    309, 310, 100, 241, 311, 5, 5, 1282, 1283, 1290,
    1293, 1294, 1295, 1296, 1298, 1299, 1300, 1301, 14, 1303,
    73, 116, 117, 1303, 1305, 1306, 1307, 1308, 687, 686,
    24, 306, 308, 309, 310, 241, 311, 1311, 307, 1315,
    1316, 1317, 1318, 1321, 1322, 1324, 1325, 1326, 471, 1327,
    1328, 1329, 1330, 1331, 1332, 671, 672, 781, 939, 179,
    1333, 1334, 1335, 675, 1336, 671, 672, 781, 939, 1333,
    1334, 1335, 675, 1336, 1333, 1336, 1335, 1334, 1128, 371,
    1153, 1343, 1343, 85, 671, 672, 675, 671, 672, 675,
    1357, 1358, 1359, 347, 1360, 65, 347, 279, 1128, 1364,
    1365, 1366, 1367, 1368, 1369, 299, 280, 1370, 1372, 1374,
    739, 347, 294, 218, 347, 1380, 1381, 7, 1381, 1380,
    743, 685, 669, 255, 294, 218, 686, 687, 688, 288,
    744, 689, 419, 745, 690, 221, 747, 749, 691, 294,
    218, 1381, 1388, 1396, 1398, 726, 1051, 297, 279, 298,
    299, 300, 301, 302, 303, 280, 306, 308, 309, 310,
    100, 241, 311, 1417, 1421, 641, 1423, 1423, 490, 527,
    517, 275, 1426, 1427, 1429, 1432, 1435, 1439, 1440, 1442,
    1443, 834, 410, 203, 120, 565, 632, 1445, 352, 292,
    439, 347, 299, 1446, 1447, 1448, 527, 1451, 1443, 1452,
    1452, 316, 197, 410, 1426, 1457, 1448, 1447, 1460, 1461,
    1326, 1421, 330, 197, 1464, 1465, 283, 1466, 1467, 226,
    1468, 1053, 893, 1015, 275, 1464, 1468, 1015, 1326, 1465,
    1466, 1053, 1467, 1483, 1488, 1492, 773, 776, 1502, 767,
    1505, 1506, 330, 94, 1511, 1514, 1515, 295, 663, 232,
    737, 521, 522, 18, 523, 24, 524, 347, 525, 1014,
    526, 527, 494, 258, 528, 529, 530, 372, 531, 1015,
    1014, 85, 94, 107, 773, 774, 775, 1014, 115, 776,
    120, 648, 275, 85, 94, 773, 774, 775, 115, 776,
    648, 275, 65, 942, 1525, 304, 1525, 304, 1388, 288,
    292, 849, 648, 850, 1074, 1075, 1076, 1077, 1078, 1079,
    1357, 1535, 902, 275, 1542, 1544, 1421, 1464, 283, 226,
    1468, 1015, 1546, 1549, 1552, 1421, 283, 226, 1468, 1015,
    1553, 1554, 1555, 1556, 1558, 1559, 1560, 1231, 1561, 1562,
    1563, 1564, 1567, 1568, 1514, 1515, 1358, 1572, 1326, 283,
    1578, 1581, 709, 1295, 641, 1585, 1585, 982, 401, 275,
    1590, 1591, 1592, 1593, 1594, 1595, 885, 1596, 85, 441,
    1600, 85, 292, 104, 94, 107, 773, 774, 775, 115,
    776, 120, 648, 275, 1606, 1607, 1608, 1609, 1465, 1223,
    1224, 1225, 1226, 1227, 1228, 1229, 1230, 1231, 1232, 1233,
    1234, 1235, 1236, 1237, 1238, 1239, 1240, 1241, 1242, 1243,
    1244, 1245, 1246, 1247, 1248, 1249, 1250, 1251, 1252, 1253,
    1254, 1255, 306, 308, 309, 310, 241, 311, 1224, 1231,
    1240, 1242, 1246, 1254, 1254, 1242, 1240, 1246, 116, 117,
    501, 1364, 1370, 1365, 1366, 1367, 1368, 1369, 1621, 1622,
    1623, 1624, 166, 401, 1060, 1634, 1558, 506, 886, 887,
    886, 887, 1646, 296, 98, 1648, 663, 1648, 1651, 1653,
    1654, 1655, 1658, 1660, 1661, 1505, 1662, 1663, 1664, 1357,
    1358, 1359, 1360, 1359, 1360, 1327, 1581, 14, 863, 898,
    1277, 439, 445, 1591, 457, 461, 90, 1467, 1676, 1677,
    1080, 1081, 1082, 1083, 1680, 1681, 85, 295, 296, 304,
    305, 1690, 1692, 433, 434, 1693, 1694, 11, 1328, 490,
    203, 1698, 1699, 1700, 1701, 1702, 1703, 1704, 1705, 1713,
    1714, 1715, 1716, 1717, 1718, 1719, 1720, 1730, 104, 891,
    120, 1736, 1738, 891, 107, 1747, 1750, 1238, 1751, 1466,
    1752, 1753, 1754, 1755, 1756, 1757, 1084, 310, 1085, 1086,
    1087, 1088, 1089, 1090, 1091, 1092, 1093, 1094, 1095, 1096,
    1097, 1760, 1388, 288, 1763, 1763, 1769, 1771, 1634, 1773,
    1365, 1366, 1367, 1368, 1774, 1775, 1776, 1777, 1778, 1779,
    1780, 94, 115, 773, 775, 774, 776, 890, 682, 506,
    885, 886, 887, 890, 85, 104, 94, 107, 773, 774,
    775, 115, 776, 120, 648, 85, 115, 1791, 1792, 1793,
    1365, 1366, 1367, 1368, 1369, 743, 1801, 1803, 1806, 1807,
    1808, 1809, 1810, 1811, 1812, 1813, 1814, 1815, 1816, 1817,
    306, 308, 309, 310, 241, 311, 5, 330, 1447, 1464,
    283, 1468, 1448, 527, 1511, 1015, 275, 1792, 226, 1821,
    1822, 1824, 1826, 1827, 1828, 1829, 1831, 1832, 1833, 1836,
    1837, 686, 306, 308, 309, 310, 241, 311, 1841, 1842,
    1843, 1844, 774, 775, 1846, 1099, 1100, 1101, 1103, 1104,
    1105, 1106, 1848, 1853, 1854, 1857, 1858, 85, 104, 94,
    107, 1720, 773, 774, 775, 115, 776, 120, 648, 275,
    1861, 1862, 1863, 1864, 1865, 1866, 1867, 1868, 1877, 14,
    683, 1879, 1879, 1244, 309, 241, 309, 310, 241, 306,
    311, 1885, 308, 310, 308, 1888, 1889, 1891, 1892, 1893,
    1895, 1896, 1897, 1898, 334, 410, 332, 258, 506, 1833,
    1902, 1902, 292, 299, 1906, 1907, 653, 36, 675, 16,
    509, 31, 638, 35, 16, 638, 1911, 1912, 1913, 1914,
    1915, 1916, 1917, 1919, 1920, 1921, 1923, 1924, 1925, 1926,
    1927, 1928, 1929, 738, 1917, 638, 100, 738, 638, 738,
    1917, 638, 1935, 685, 1842, 1939, 1940, 1941, 1942, 224,
    415, 16, 17, 368, 20, 21, 24, 95, 26, 370,
    27, 572, 29, 30, 31, 32, 628, 35, 36, 38,
    39, 14, 987, 90, 1326, 94, 1465, 1578, 283, 1466,
    226, 1290, 1053, 893, 1747, 1968, 1969, 1970, 479, 481,
    1326, 1465, 1692, 1919, 1977, 1535, 1692, 1919, 1980, 1981,
    1982, 1983, 1988, 1989, 1996, 1997, 1998, 1999, 2000, 2001,
    2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
    2012, 2013, 2014, 737, 864, 803, 1925, 452, 2123, 2134,
    2244, 2245, 2246, 2248, 2250, 2251, 2252, 2253, 2254, 2255,
    352, 471, 20, 423, 424, 474, 476, 410, 347, 423,
    424, 116, 117, 2265, 2265, 2267, 2268, 2269, 2270, 300,
    301, 2275, 1107, 2277, 2278, 2279, 2280, 2281, 1506, 2282,
    2283, 2283, 2286, 2286, 2290, 2291, 1108, 2292, 901, 427,
    2297, 2298, 2300, 2301, 2306, 2307, 2309, 2311, 2312, 2313,
    2315, 2316, 2317, 85, 671, 672, 926, 506, 675, 671,
    672, 926, 506, 675, 671, 672, 1300, 926, 506, 1590,
    525, 1866, 676, 119, 675, 2322, 2323, 2324, 2325, 2327,
    1251, 1252, 819, 880, 1070, 1076, 1077, 1078, 1079, 1080,
    1081, 1082, 1083, 1084, 1085, 1086, 1087, 1088, 1089, 1090,
    1091, 1092, 1093, 1094, 1095, 1096, 1097, 1098, 1099, 1100,
    1101, 1102, 1103, 1104, 1105, 1106, 1107, 1108, 85, 819,
    880, 1070, 1074, 1075, 1076, 1077, 1078, 1079, 1080, 1081,
    1082, 1083, 1084, 1085, 1086, 1087, 1088, 1089, 1090, 1091,
    1092, 1093, 1094, 1095, 1096, 1097, 1098, 1099, 1100, 1101,
    1102, 1103, 1104, 1105, 1106, 1107, 1108, 100, 85, 100,
    1074, 1075, 330, 197, 1464, 283, 166, 226, 1826, 1468,
    1290, 1015, 2367, 2368, 1357, 1358, 1359, 347, 257, 659,
    1360, 747, 749, 257, 659, 1357, 1358, 1359, 347, 1360,
    747, 749, 747, 749, 226, 562, 792, 1692, 2391, 2392,
    2393, 2394, 2395, 2396, 2397, 2398, 2399, 2400, 2401, 2402,
    2403, 2404, 2405, 2406, 2407,
];

pub static SIMPLE_HINTS: &[SimpleHintData] = &[
    SimpleHintData::empty(), // [0] reserved for no hint
    SimpleHintData { raw_values_start: 0, raw_values_count: 1, token_types_start: 1, token_types_count: 0 }, // [1]
    SimpleHintData { raw_values_start: 1, raw_values_count: 1, token_types_start: 2, token_types_count: 0 }, // [2]
    SimpleHintData { raw_values_start: 2, raw_values_count: 0, token_types_start: 2, token_types_count: 1 }, // [3]
    SimpleHintData { raw_values_start: 3, raw_values_count: 1, token_types_start: 4, token_types_count: 0 }, // [4]
    SimpleHintData { raw_values_start: 4, raw_values_count: 1, token_types_start: 5, token_types_count: 0 }, // [5]
    SimpleHintData { raw_values_start: 5, raw_values_count: 27, token_types_start: 32, token_types_count: 0 }, // [6]
    SimpleHintData { raw_values_start: 32, raw_values_count: 1, token_types_start: 33, token_types_count: 0 }, // [7]
    SimpleHintData { raw_values_start: 33, raw_values_count: 1, token_types_start: 34, token_types_count: 0 }, // [8]
    SimpleHintData { raw_values_start: 34, raw_values_count: 1, token_types_start: 35, token_types_count: 0 }, // [9]
    SimpleHintData { raw_values_start: 35, raw_values_count: 1, token_types_start: 36, token_types_count: 0 }, // [10]
    SimpleHintData { raw_values_start: 36, raw_values_count: 1, token_types_start: 37, token_types_count: 0 }, // [11]
    SimpleHintData { raw_values_start: 37, raw_values_count: 1, token_types_start: 38, token_types_count: 0 }, // [12]
    SimpleHintData { raw_values_start: 38, raw_values_count: 1, token_types_start: 39, token_types_count: 0 }, // [13]
    SimpleHintData { raw_values_start: 39, raw_values_count: 1, token_types_start: 40, token_types_count: 0 }, // [14]
    SimpleHintData { raw_values_start: 40, raw_values_count: 1, token_types_start: 41, token_types_count: 0 }, // [15]
    SimpleHintData { raw_values_start: 41, raw_values_count: 1, token_types_start: 42, token_types_count: 0 }, // [16]
    SimpleHintData { raw_values_start: 42, raw_values_count: 1, token_types_start: 43, token_types_count: 0 }, // [17]
    SimpleHintData { raw_values_start: 43, raw_values_count: 1, token_types_start: 44, token_types_count: 0 }, // [18]
    SimpleHintData { raw_values_start: 44, raw_values_count: 1, token_types_start: 45, token_types_count: 0 }, // [19]
    SimpleHintData { raw_values_start: 45, raw_values_count: 2, token_types_start: 47, token_types_count: 0 }, // [20]
    SimpleHintData { raw_values_start: 47, raw_values_count: 1, token_types_start: 48, token_types_count: 0 }, // [21]
    SimpleHintData { raw_values_start: 48, raw_values_count: 1, token_types_start: 49, token_types_count: 0 }, // [22]
    SimpleHintData { raw_values_start: 49, raw_values_count: 1, token_types_start: 50, token_types_count: 0 }, // [23]
    SimpleHintData { raw_values_start: 50, raw_values_count: 1, token_types_start: 51, token_types_count: 0 }, // [24]
    SimpleHintData { raw_values_start: 51, raw_values_count: 1, token_types_start: 52, token_types_count: 0 }, // [25]
    SimpleHintData { raw_values_start: 52, raw_values_count: 1, token_types_start: 53, token_types_count: 0 }, // [26]
    SimpleHintData { raw_values_start: 53, raw_values_count: 1, token_types_start: 54, token_types_count: 0 }, // [27]
    SimpleHintData { raw_values_start: 54, raw_values_count: 14, token_types_start: 68, token_types_count: 0 }, // [28]
    SimpleHintData { raw_values_start: 68, raw_values_count: 14, token_types_start: 82, token_types_count: 0 }, // [29]
    SimpleHintData { raw_values_start: 82, raw_values_count: 2, token_types_start: 84, token_types_count: 0 }, // [30]
    SimpleHintData { raw_values_start: 84, raw_values_count: 1, token_types_start: 85, token_types_count: 0 }, // [31]
    SimpleHintData { raw_values_start: 85, raw_values_count: 1, token_types_start: 86, token_types_count: 0 }, // [32]
    SimpleHintData { raw_values_start: 86, raw_values_count: 1, token_types_start: 87, token_types_count: 0 }, // [33]
    SimpleHintData { raw_values_start: 87, raw_values_count: 1, token_types_start: 88, token_types_count: 0 }, // [34]
    SimpleHintData { raw_values_start: 88, raw_values_count: 1, token_types_start: 89, token_types_count: 0 }, // [35]
    SimpleHintData { raw_values_start: 89, raw_values_count: 1, token_types_start: 90, token_types_count: 0 }, // [36]
    SimpleHintData { raw_values_start: 90, raw_values_count: 0, token_types_start: 90, token_types_count: 1 }, // [37]
    SimpleHintData { raw_values_start: 91, raw_values_count: 26, token_types_start: 117, token_types_count: 0 }, // [38]
    SimpleHintData { raw_values_start: 117, raw_values_count: 1, token_types_start: 118, token_types_count: 0 }, // [39]
    SimpleHintData { raw_values_start: 118, raw_values_count: 5, token_types_start: 123, token_types_count: 0 }, // [40]
    SimpleHintData { raw_values_start: 123, raw_values_count: 1, token_types_start: 124, token_types_count: 0 }, // [41]
    SimpleHintData { raw_values_start: 124, raw_values_count: 1, token_types_start: 125, token_types_count: 0 }, // [42]
    SimpleHintData { raw_values_start: 125, raw_values_count: 1, token_types_start: 126, token_types_count: 0 }, // [43]
    SimpleHintData { raw_values_start: 126, raw_values_count: 1, token_types_start: 127, token_types_count: 0 }, // [44]
    SimpleHintData { raw_values_start: 127, raw_values_count: 1, token_types_start: 128, token_types_count: 0 }, // [45]
    SimpleHintData { raw_values_start: 128, raw_values_count: 1, token_types_start: 129, token_types_count: 0 }, // [46]
    SimpleHintData { raw_values_start: 129, raw_values_count: 1, token_types_start: 130, token_types_count: 0 }, // [47]
    SimpleHintData { raw_values_start: 130, raw_values_count: 1, token_types_start: 131, token_types_count: 0 }, // [48]
    SimpleHintData { raw_values_start: 131, raw_values_count: 1, token_types_start: 132, token_types_count: 0 }, // [49]
    SimpleHintData { raw_values_start: 132, raw_values_count: 25, token_types_start: 157, token_types_count: 0 }, // [50]
    SimpleHintData { raw_values_start: 157, raw_values_count: 15, token_types_start: 172, token_types_count: 0 }, // [51]
    SimpleHintData { raw_values_start: 172, raw_values_count: 1, token_types_start: 173, token_types_count: 0 }, // [52]
    SimpleHintData { raw_values_start: 173, raw_values_count: 1, token_types_start: 174, token_types_count: 0 }, // [53]
    SimpleHintData { raw_values_start: 174, raw_values_count: 2, token_types_start: 176, token_types_count: 0 }, // [54]
    SimpleHintData { raw_values_start: 176, raw_values_count: 1, token_types_start: 177, token_types_count: 0 }, // [55]
    SimpleHintData { raw_values_start: 177, raw_values_count: 1, token_types_start: 178, token_types_count: 0 }, // [56]
    SimpleHintData { raw_values_start: 178, raw_values_count: 1, token_types_start: 179, token_types_count: 0 }, // [57]
    SimpleHintData { raw_values_start: 179, raw_values_count: 1, token_types_start: 180, token_types_count: 0 }, // [58]
    SimpleHintData { raw_values_start: 180, raw_values_count: 1, token_types_start: 181, token_types_count: 0 }, // [59]
    SimpleHintData { raw_values_start: 181, raw_values_count: 1, token_types_start: 182, token_types_count: 0 }, // [60]
    SimpleHintData { raw_values_start: 182, raw_values_count: 1, token_types_start: 183, token_types_count: 0 }, // [61]
    SimpleHintData { raw_values_start: 183, raw_values_count: 1, token_types_start: 184, token_types_count: 0 }, // [62]
    SimpleHintData { raw_values_start: 184, raw_values_count: 1, token_types_start: 185, token_types_count: 0 }, // [63]
    SimpleHintData { raw_values_start: 185, raw_values_count: 1, token_types_start: 186, token_types_count: 0 }, // [64]
    SimpleHintData { raw_values_start: 186, raw_values_count: 1, token_types_start: 187, token_types_count: 0 }, // [65]
    SimpleHintData { raw_values_start: 187, raw_values_count: 1, token_types_start: 188, token_types_count: 0 }, // [66]
    SimpleHintData { raw_values_start: 188, raw_values_count: 1, token_types_start: 189, token_types_count: 0 }, // [67]
    SimpleHintData { raw_values_start: 189, raw_values_count: 1, token_types_start: 190, token_types_count: 0 }, // [68]
    SimpleHintData { raw_values_start: 190, raw_values_count: 1, token_types_start: 191, token_types_count: 0 }, // [69]
    SimpleHintData { raw_values_start: 191, raw_values_count: 1, token_types_start: 192, token_types_count: 0 }, // [70]
    SimpleHintData { raw_values_start: 192, raw_values_count: 1, token_types_start: 193, token_types_count: 0 }, // [71]
    SimpleHintData { raw_values_start: 193, raw_values_count: 1, token_types_start: 194, token_types_count: 0 }, // [72]
    SimpleHintData { raw_values_start: 194, raw_values_count: 1, token_types_start: 195, token_types_count: 0 }, // [73]
    SimpleHintData { raw_values_start: 195, raw_values_count: 1, token_types_start: 196, token_types_count: 0 }, // [74]
    SimpleHintData { raw_values_start: 196, raw_values_count: 1, token_types_start: 197, token_types_count: 0 }, // [75]
    SimpleHintData { raw_values_start: 197, raw_values_count: 1, token_types_start: 198, token_types_count: 0 }, // [76]
    SimpleHintData { raw_values_start: 198, raw_values_count: 1, token_types_start: 199, token_types_count: 0 }, // [77]
    SimpleHintData { raw_values_start: 199, raw_values_count: 1, token_types_start: 200, token_types_count: 0 }, // [78]
    SimpleHintData { raw_values_start: 200, raw_values_count: 1, token_types_start: 201, token_types_count: 0 }, // [79]
    SimpleHintData { raw_values_start: 201, raw_values_count: 1, token_types_start: 202, token_types_count: 0 }, // [80]
    SimpleHintData { raw_values_start: 202, raw_values_count: 1, token_types_start: 203, token_types_count: 0 }, // [81]
    SimpleHintData { raw_values_start: 203, raw_values_count: 1, token_types_start: 204, token_types_count: 0 }, // [82]
    SimpleHintData { raw_values_start: 204, raw_values_count: 1, token_types_start: 205, token_types_count: 0 }, // [83]
    SimpleHintData { raw_values_start: 205, raw_values_count: 1, token_types_start: 206, token_types_count: 0 }, // [84]
    SimpleHintData { raw_values_start: 206, raw_values_count: 1, token_types_start: 207, token_types_count: 0 }, // [85]
    SimpleHintData { raw_values_start: 207, raw_values_count: 1, token_types_start: 208, token_types_count: 0 }, // [86]
    SimpleHintData { raw_values_start: 208, raw_values_count: 1, token_types_start: 209, token_types_count: 0 }, // [87]
    SimpleHintData { raw_values_start: 209, raw_values_count: 1, token_types_start: 210, token_types_count: 0 }, // [88]
    SimpleHintData { raw_values_start: 210, raw_values_count: 1, token_types_start: 211, token_types_count: 0 }, // [89]
    SimpleHintData { raw_values_start: 211, raw_values_count: 1, token_types_start: 212, token_types_count: 0 }, // [90]
    SimpleHintData { raw_values_start: 212, raw_values_count: 1, token_types_start: 213, token_types_count: 0 }, // [91]
    SimpleHintData { raw_values_start: 213, raw_values_count: 1, token_types_start: 214, token_types_count: 0 }, // [92]
    SimpleHintData { raw_values_start: 214, raw_values_count: 1, token_types_start: 215, token_types_count: 0 }, // [93]
    SimpleHintData { raw_values_start: 215, raw_values_count: 1, token_types_start: 216, token_types_count: 0 }, // [94]
    SimpleHintData { raw_values_start: 216, raw_values_count: 1, token_types_start: 217, token_types_count: 0 }, // [95]
    SimpleHintData { raw_values_start: 217, raw_values_count: 2, token_types_start: 219, token_types_count: 0 }, // [96]
    SimpleHintData { raw_values_start: 219, raw_values_count: 1, token_types_start: 220, token_types_count: 0 }, // [97]
    SimpleHintData { raw_values_start: 220, raw_values_count: 1, token_types_start: 221, token_types_count: 0 }, // [98]
    SimpleHintData { raw_values_start: 221, raw_values_count: 1, token_types_start: 222, token_types_count: 0 }, // [99]
    SimpleHintData { raw_values_start: 222, raw_values_count: 1, token_types_start: 223, token_types_count: 0 }, // [100]
    SimpleHintData { raw_values_start: 223, raw_values_count: 1, token_types_start: 224, token_types_count: 0 }, // [101]
    SimpleHintData { raw_values_start: 224, raw_values_count: 2, token_types_start: 226, token_types_count: 0 }, // [102]
    SimpleHintData { raw_values_start: 226, raw_values_count: 1, token_types_start: 227, token_types_count: 0 }, // [103]
    SimpleHintData { raw_values_start: 227, raw_values_count: 1, token_types_start: 228, token_types_count: 0 }, // [104]
    SimpleHintData { raw_values_start: 228, raw_values_count: 1, token_types_start: 229, token_types_count: 0 }, // [105]
    SimpleHintData { raw_values_start: 229, raw_values_count: 1, token_types_start: 230, token_types_count: 0 }, // [106]
    SimpleHintData { raw_values_start: 230, raw_values_count: 1, token_types_start: 231, token_types_count: 0 }, // [107]
    SimpleHintData { raw_values_start: 231, raw_values_count: 1, token_types_start: 232, token_types_count: 0 }, // [108]
    SimpleHintData { raw_values_start: 232, raw_values_count: 1, token_types_start: 233, token_types_count: 0 }, // [109]
    SimpleHintData { raw_values_start: 233, raw_values_count: 1, token_types_start: 234, token_types_count: 0 }, // [110]
    SimpleHintData { raw_values_start: 234, raw_values_count: 1, token_types_start: 235, token_types_count: 0 }, // [111]
    SimpleHintData { raw_values_start: 235, raw_values_count: 1, token_types_start: 236, token_types_count: 0 }, // [112]
    SimpleHintData { raw_values_start: 236, raw_values_count: 1, token_types_start: 237, token_types_count: 0 }, // [113]
    SimpleHintData { raw_values_start: 237, raw_values_count: 1, token_types_start: 238, token_types_count: 0 }, // [114]
    SimpleHintData { raw_values_start: 238, raw_values_count: 1, token_types_start: 239, token_types_count: 0 }, // [115]
    SimpleHintData { raw_values_start: 239, raw_values_count: 0, token_types_start: 239, token_types_count: 1 }, // [116]
    SimpleHintData { raw_values_start: 240, raw_values_count: 1, token_types_start: 241, token_types_count: 0 }, // [117]
    SimpleHintData { raw_values_start: 241, raw_values_count: 1, token_types_start: 242, token_types_count: 0 }, // [118]
    SimpleHintData { raw_values_start: 242, raw_values_count: 1, token_types_start: 243, token_types_count: 0 }, // [119]
    SimpleHintData { raw_values_start: 243, raw_values_count: 1, token_types_start: 244, token_types_count: 0 }, // [120]
    SimpleHintData { raw_values_start: 244, raw_values_count: 1, token_types_start: 245, token_types_count: 0 }, // [121]
    SimpleHintData { raw_values_start: 245, raw_values_count: 3, token_types_start: 248, token_types_count: 0 }, // [122]
    SimpleHintData { raw_values_start: 248, raw_values_count: 1, token_types_start: 249, token_types_count: 0 }, // [123]
    SimpleHintData { raw_values_start: 249, raw_values_count: 1, token_types_start: 250, token_types_count: 0 }, // [124]
    SimpleHintData { raw_values_start: 250, raw_values_count: 1, token_types_start: 251, token_types_count: 0 }, // [125]
    SimpleHintData { raw_values_start: 251, raw_values_count: 1, token_types_start: 252, token_types_count: 0 }, // [126]
    SimpleHintData { raw_values_start: 252, raw_values_count: 1, token_types_start: 253, token_types_count: 0 }, // [127]
    SimpleHintData { raw_values_start: 253, raw_values_count: 1, token_types_start: 254, token_types_count: 0 }, // [128]
    SimpleHintData { raw_values_start: 254, raw_values_count: 8, token_types_start: 262, token_types_count: 0 }, // [129]
    SimpleHintData { raw_values_start: 262, raw_values_count: 4, token_types_start: 266, token_types_count: 0 }, // [130]
    SimpleHintData { raw_values_start: 266, raw_values_count: 1, token_types_start: 267, token_types_count: 0 }, // [131]
    SimpleHintData { raw_values_start: 267, raw_values_count: 3, token_types_start: 270, token_types_count: 0 }, // [132]
    SimpleHintData { raw_values_start: 270, raw_values_count: 2, token_types_start: 272, token_types_count: 0 }, // [133]
    SimpleHintData { raw_values_start: 272, raw_values_count: 1, token_types_start: 273, token_types_count: 0 }, // [134]
    SimpleHintData { raw_values_start: 273, raw_values_count: 1, token_types_start: 274, token_types_count: 0 }, // [135]
    SimpleHintData { raw_values_start: 274, raw_values_count: 1, token_types_start: 275, token_types_count: 0 }, // [136]
    SimpleHintData { raw_values_start: 275, raw_values_count: 3, token_types_start: 278, token_types_count: 0 }, // [137]
    SimpleHintData { raw_values_start: 278, raw_values_count: 2, token_types_start: 280, token_types_count: 0 }, // [138]
    SimpleHintData { raw_values_start: 280, raw_values_count: 1, token_types_start: 281, token_types_count: 0 }, // [139]
    SimpleHintData { raw_values_start: 281, raw_values_count: 1, token_types_start: 282, token_types_count: 0 }, // [140]
    SimpleHintData { raw_values_start: 282, raw_values_count: 17, token_types_start: 299, token_types_count: 8 }, // [141]
    SimpleHintData { raw_values_start: 307, raw_values_count: 0, token_types_start: 307, token_types_count: 1 }, // [142]
    SimpleHintData { raw_values_start: 308, raw_values_count: 1, token_types_start: 309, token_types_count: 0 }, // [143]
    SimpleHintData { raw_values_start: 309, raw_values_count: 1, token_types_start: 310, token_types_count: 0 }, // [144]
    SimpleHintData { raw_values_start: 310, raw_values_count: 11, token_types_start: 321, token_types_count: 0 }, // [145]
    SimpleHintData { raw_values_start: 321, raw_values_count: 6, token_types_start: 327, token_types_count: 0 }, // [146]
    SimpleHintData { raw_values_start: 327, raw_values_count: 1, token_types_start: 328, token_types_count: 0 }, // [147]
    SimpleHintData { raw_values_start: 328, raw_values_count: 10, token_types_start: 338, token_types_count: 0 }, // [148]
    SimpleHintData { raw_values_start: 338, raw_values_count: 1, token_types_start: 339, token_types_count: 0 }, // [149]
    SimpleHintData { raw_values_start: 339, raw_values_count: 1, token_types_start: 340, token_types_count: 0 }, // [150]
    SimpleHintData { raw_values_start: 340, raw_values_count: 1, token_types_start: 341, token_types_count: 0 }, // [151]
    SimpleHintData { raw_values_start: 341, raw_values_count: 2, token_types_start: 343, token_types_count: 0 }, // [152]
    SimpleHintData { raw_values_start: 343, raw_values_count: 1, token_types_start: 344, token_types_count: 0 }, // [153]
    SimpleHintData { raw_values_start: 344, raw_values_count: 2, token_types_start: 346, token_types_count: 0 }, // [154]
    SimpleHintData { raw_values_start: 346, raw_values_count: 6, token_types_start: 352, token_types_count: 0 }, // [155]
    SimpleHintData { raw_values_start: 352, raw_values_count: 1, token_types_start: 353, token_types_count: 0 }, // [156]
    SimpleHintData { raw_values_start: 353, raw_values_count: 2, token_types_start: 355, token_types_count: 0 }, // [157]
    SimpleHintData { raw_values_start: 355, raw_values_count: 1, token_types_start: 356, token_types_count: 0 }, // [158]
    SimpleHintData { raw_values_start: 356, raw_values_count: 1, token_types_start: 357, token_types_count: 0 }, // [159]
    SimpleHintData { raw_values_start: 357, raw_values_count: 1, token_types_start: 358, token_types_count: 0 }, // [160]
    SimpleHintData { raw_values_start: 358, raw_values_count: 3, token_types_start: 361, token_types_count: 0 }, // [161]
    SimpleHintData { raw_values_start: 361, raw_values_count: 2, token_types_start: 363, token_types_count: 0 }, // [162]
    SimpleHintData { raw_values_start: 363, raw_values_count: 1, token_types_start: 364, token_types_count: 0 }, // [163]
    SimpleHintData { raw_values_start: 364, raw_values_count: 1, token_types_start: 365, token_types_count: 0 }, // [164]
    SimpleHintData { raw_values_start: 365, raw_values_count: 1, token_types_start: 366, token_types_count: 0 }, // [165]
    SimpleHintData { raw_values_start: 366, raw_values_count: 1, token_types_start: 367, token_types_count: 0 }, // [166]
    SimpleHintData { raw_values_start: 367, raw_values_count: 1, token_types_start: 368, token_types_count: 0 }, // [167]
    SimpleHintData { raw_values_start: 368, raw_values_count: 4, token_types_start: 372, token_types_count: 0 }, // [168]
    SimpleHintData { raw_values_start: 372, raw_values_count: 1, token_types_start: 373, token_types_count: 0 }, // [169]
    SimpleHintData { raw_values_start: 373, raw_values_count: 2, token_types_start: 375, token_types_count: 0 }, // [170]
    SimpleHintData { raw_values_start: 375, raw_values_count: 23, token_types_start: 398, token_types_count: 0 }, // [171]
    SimpleHintData { raw_values_start: 398, raw_values_count: 13, token_types_start: 411, token_types_count: 0 }, // [172]
    SimpleHintData { raw_values_start: 411, raw_values_count: 1, token_types_start: 412, token_types_count: 0 }, // [173]
    SimpleHintData { raw_values_start: 412, raw_values_count: 1, token_types_start: 413, token_types_count: 0 }, // [174]
    SimpleHintData { raw_values_start: 413, raw_values_count: 1, token_types_start: 414, token_types_count: 0 }, // [175]
    SimpleHintData { raw_values_start: 414, raw_values_count: 1, token_types_start: 415, token_types_count: 0 }, // [176]
    SimpleHintData { raw_values_start: 415, raw_values_count: 2, token_types_start: 417, token_types_count: 0 }, // [177]
    SimpleHintData { raw_values_start: 417, raw_values_count: 1, token_types_start: 418, token_types_count: 0 }, // [178]
    SimpleHintData { raw_values_start: 418, raw_values_count: 1, token_types_start: 419, token_types_count: 0 }, // [179]
    SimpleHintData { raw_values_start: 419, raw_values_count: 1, token_types_start: 420, token_types_count: 0 }, // [180]
    SimpleHintData { raw_values_start: 420, raw_values_count: 4, token_types_start: 424, token_types_count: 0 }, // [181]
    SimpleHintData { raw_values_start: 424, raw_values_count: 1, token_types_start: 425, token_types_count: 0 }, // [182]
    SimpleHintData { raw_values_start: 425, raw_values_count: 1, token_types_start: 426, token_types_count: 0 }, // [183]
    SimpleHintData { raw_values_start: 426, raw_values_count: 1, token_types_start: 427, token_types_count: 0 }, // [184]
    SimpleHintData { raw_values_start: 427, raw_values_count: 1, token_types_start: 428, token_types_count: 0 }, // [185]
    SimpleHintData { raw_values_start: 428, raw_values_count: 3, token_types_start: 431, token_types_count: 0 }, // [186]
    SimpleHintData { raw_values_start: 431, raw_values_count: 1, token_types_start: 432, token_types_count: 0 }, // [187]
    SimpleHintData { raw_values_start: 432, raw_values_count: 3, token_types_start: 435, token_types_count: 0 }, // [188]
    SimpleHintData { raw_values_start: 435, raw_values_count: 1, token_types_start: 436, token_types_count: 0 }, // [189]
    SimpleHintData { raw_values_start: 436, raw_values_count: 2, token_types_start: 438, token_types_count: 0 }, // [190]
    SimpleHintData { raw_values_start: 438, raw_values_count: 1, token_types_start: 439, token_types_count: 0 }, // [191]
    SimpleHintData { raw_values_start: 439, raw_values_count: 1, token_types_start: 440, token_types_count: 0 }, // [192]
    SimpleHintData { raw_values_start: 440, raw_values_count: 1, token_types_start: 441, token_types_count: 0 }, // [193]
    SimpleHintData { raw_values_start: 441, raw_values_count: 1, token_types_start: 442, token_types_count: 0 }, // [194]
    SimpleHintData { raw_values_start: 442, raw_values_count: 17, token_types_start: 459, token_types_count: 0 }, // [195]
    SimpleHintData { raw_values_start: 459, raw_values_count: 1, token_types_start: 460, token_types_count: 0 }, // [196]
    SimpleHintData { raw_values_start: 460, raw_values_count: 1, token_types_start: 461, token_types_count: 0 }, // [197]
    SimpleHintData { raw_values_start: 461, raw_values_count: 8, token_types_start: 469, token_types_count: 0 }, // [198]
    SimpleHintData { raw_values_start: 469, raw_values_count: 7, token_types_start: 476, token_types_count: 0 }, // [199]
    SimpleHintData { raw_values_start: 476, raw_values_count: 2, token_types_start: 478, token_types_count: 0 }, // [200]
    SimpleHintData { raw_values_start: 478, raw_values_count: 12, token_types_start: 490, token_types_count: 0 }, // [201]
    SimpleHintData { raw_values_start: 490, raw_values_count: 3, token_types_start: 493, token_types_count: 0 }, // [202]
    SimpleHintData { raw_values_start: 493, raw_values_count: 1, token_types_start: 494, token_types_count: 0 }, // [203]
    SimpleHintData { raw_values_start: 494, raw_values_count: 1, token_types_start: 495, token_types_count: 0 }, // [204]
    SimpleHintData { raw_values_start: 495, raw_values_count: 2, token_types_start: 497, token_types_count: 0 }, // [205]
    SimpleHintData { raw_values_start: 497, raw_values_count: 1, token_types_start: 498, token_types_count: 0 }, // [206]
    SimpleHintData { raw_values_start: 498, raw_values_count: 1, token_types_start: 499, token_types_count: 0 }, // [207]
    SimpleHintData { raw_values_start: 499, raw_values_count: 1, token_types_start: 500, token_types_count: 0 }, // [208]
    SimpleHintData { raw_values_start: 500, raw_values_count: 11, token_types_start: 511, token_types_count: 0 }, // [209]
    SimpleHintData { raw_values_start: 511, raw_values_count: 2, token_types_start: 513, token_types_count: 0 }, // [210]
    SimpleHintData { raw_values_start: 513, raw_values_count: 1, token_types_start: 514, token_types_count: 0 }, // [211]
    SimpleHintData { raw_values_start: 514, raw_values_count: 1, token_types_start: 515, token_types_count: 0 }, // [212]
    SimpleHintData { raw_values_start: 515, raw_values_count: 1, token_types_start: 516, token_types_count: 0 }, // [213]
    SimpleHintData { raw_values_start: 516, raw_values_count: 4, token_types_start: 520, token_types_count: 0 }, // [214]
    SimpleHintData { raw_values_start: 520, raw_values_count: 1, token_types_start: 521, token_types_count: 0 }, // [215]
    SimpleHintData { raw_values_start: 521, raw_values_count: 1, token_types_start: 522, token_types_count: 0 }, // [216]
    SimpleHintData { raw_values_start: 522, raw_values_count: 1, token_types_start: 523, token_types_count: 0 }, // [217]
    SimpleHintData { raw_values_start: 523, raw_values_count: 1, token_types_start: 524, token_types_count: 0 }, // [218]
    SimpleHintData { raw_values_start: 524, raw_values_count: 6, token_types_start: 530, token_types_count: 0 }, // [219]
    SimpleHintData { raw_values_start: 530, raw_values_count: 3, token_types_start: 533, token_types_count: 0 }, // [220]
    SimpleHintData { raw_values_start: 533, raw_values_count: 2, token_types_start: 535, token_types_count: 0 }, // [221]
    SimpleHintData { raw_values_start: 535, raw_values_count: 1, token_types_start: 536, token_types_count: 0 }, // [222]
    SimpleHintData { raw_values_start: 536, raw_values_count: 1, token_types_start: 537, token_types_count: 0 }, // [223]
    SimpleHintData { raw_values_start: 537, raw_values_count: 1, token_types_start: 538, token_types_count: 0 }, // [224]
    SimpleHintData { raw_values_start: 538, raw_values_count: 1, token_types_start: 539, token_types_count: 0 }, // [225]
    SimpleHintData { raw_values_start: 539, raw_values_count: 2, token_types_start: 541, token_types_count: 0 }, // [226]
    SimpleHintData { raw_values_start: 541, raw_values_count: 1, token_types_start: 542, token_types_count: 0 }, // [227]
    SimpleHintData { raw_values_start: 542, raw_values_count: 1, token_types_start: 543, token_types_count: 0 }, // [228]
    SimpleHintData { raw_values_start: 543, raw_values_count: 2, token_types_start: 545, token_types_count: 0 }, // [229]
    SimpleHintData { raw_values_start: 545, raw_values_count: 1, token_types_start: 546, token_types_count: 0 }, // [230]
    SimpleHintData { raw_values_start: 546, raw_values_count: 1, token_types_start: 547, token_types_count: 0 }, // [231]
    SimpleHintData { raw_values_start: 547, raw_values_count: 4, token_types_start: 551, token_types_count: 0 }, // [232]
    SimpleHintData { raw_values_start: 551, raw_values_count: 1, token_types_start: 552, token_types_count: 0 }, // [233]
    SimpleHintData { raw_values_start: 552, raw_values_count: 1, token_types_start: 553, token_types_count: 0 }, // [234]
    SimpleHintData { raw_values_start: 553, raw_values_count: 1, token_types_start: 554, token_types_count: 0 }, // [235]
    SimpleHintData { raw_values_start: 554, raw_values_count: 1, token_types_start: 555, token_types_count: 0 }, // [236]
    SimpleHintData { raw_values_start: 555, raw_values_count: 1, token_types_start: 556, token_types_count: 0 }, // [237]
    SimpleHintData { raw_values_start: 556, raw_values_count: 1, token_types_start: 557, token_types_count: 0 }, // [238]
    SimpleHintData { raw_values_start: 557, raw_values_count: 1, token_types_start: 558, token_types_count: 0 }, // [239]
    SimpleHintData { raw_values_start: 558, raw_values_count: 2, token_types_start: 560, token_types_count: 0 }, // [240]
    SimpleHintData { raw_values_start: 560, raw_values_count: 1, token_types_start: 561, token_types_count: 0 }, // [241]
    SimpleHintData { raw_values_start: 561, raw_values_count: 2, token_types_start: 563, token_types_count: 0 }, // [242]
    SimpleHintData { raw_values_start: 563, raw_values_count: 1, token_types_start: 564, token_types_count: 0 }, // [243]
    SimpleHintData { raw_values_start: 564, raw_values_count: 1, token_types_start: 565, token_types_count: 0 }, // [244]
    SimpleHintData { raw_values_start: 565, raw_values_count: 2, token_types_start: 567, token_types_count: 0 }, // [245]
    SimpleHintData { raw_values_start: 567, raw_values_count: 1, token_types_start: 568, token_types_count: 0 }, // [246]
    SimpleHintData { raw_values_start: 568, raw_values_count: 1, token_types_start: 569, token_types_count: 0 }, // [247]
    SimpleHintData { raw_values_start: 569, raw_values_count: 4, token_types_start: 573, token_types_count: 0 }, // [248]
    SimpleHintData { raw_values_start: 573, raw_values_count: 1, token_types_start: 574, token_types_count: 0 }, // [249]
    SimpleHintData { raw_values_start: 574, raw_values_count: 1, token_types_start: 575, token_types_count: 0 }, // [250]
    SimpleHintData { raw_values_start: 575, raw_values_count: 1, token_types_start: 576, token_types_count: 0 }, // [251]
    SimpleHintData { raw_values_start: 576, raw_values_count: 2, token_types_start: 578, token_types_count: 0 }, // [252]
    SimpleHintData { raw_values_start: 578, raw_values_count: 1, token_types_start: 579, token_types_count: 0 }, // [253]
    SimpleHintData { raw_values_start: 579, raw_values_count: 3, token_types_start: 582, token_types_count: 0 }, // [254]
    SimpleHintData { raw_values_start: 582, raw_values_count: 0, token_types_start: 582, token_types_count: 6 }, // [255]
    SimpleHintData { raw_values_start: 588, raw_values_count: 1, token_types_start: 589, token_types_count: 0 }, // [256]
    SimpleHintData { raw_values_start: 589, raw_values_count: 18, token_types_start: 607, token_types_count: 0 }, // [257]
    SimpleHintData { raw_values_start: 607, raw_values_count: 16, token_types_start: 623, token_types_count: 0 }, // [258]
    SimpleHintData { raw_values_start: 623, raw_values_count: 3, token_types_start: 626, token_types_count: 0 }, // [259]
    SimpleHintData { raw_values_start: 626, raw_values_count: 1, token_types_start: 627, token_types_count: 0 }, // [260]
    SimpleHintData { raw_values_start: 627, raw_values_count: 1, token_types_start: 628, token_types_count: 0 }, // [261]
    SimpleHintData { raw_values_start: 628, raw_values_count: 1, token_types_start: 629, token_types_count: 0 }, // [262]
    SimpleHintData { raw_values_start: 629, raw_values_count: 1, token_types_start: 630, token_types_count: 0 }, // [263]
    SimpleHintData { raw_values_start: 630, raw_values_count: 3, token_types_start: 633, token_types_count: 0 }, // [264]
    SimpleHintData { raw_values_start: 633, raw_values_count: 1, token_types_start: 634, token_types_count: 0 }, // [265]
    SimpleHintData { raw_values_start: 634, raw_values_count: 1, token_types_start: 635, token_types_count: 0 }, // [266]
    SimpleHintData { raw_values_start: 635, raw_values_count: 1, token_types_start: 636, token_types_count: 0 }, // [267]
    SimpleHintData { raw_values_start: 636, raw_values_count: 2, token_types_start: 638, token_types_count: 0 }, // [268]
    SimpleHintData { raw_values_start: 638, raw_values_count: 1, token_types_start: 639, token_types_count: 0 }, // [269]
    SimpleHintData { raw_values_start: 639, raw_values_count: 2, token_types_start: 641, token_types_count: 0 }, // [270]
    SimpleHintData { raw_values_start: 641, raw_values_count: 2, token_types_start: 643, token_types_count: 0 }, // [271]
    SimpleHintData { raw_values_start: 643, raw_values_count: 1, token_types_start: 644, token_types_count: 0 }, // [272]
    SimpleHintData { raw_values_start: 644, raw_values_count: 1, token_types_start: 645, token_types_count: 0 }, // [273]
    SimpleHintData { raw_values_start: 645, raw_values_count: 1, token_types_start: 646, token_types_count: 0 }, // [274]
    SimpleHintData { raw_values_start: 646, raw_values_count: 3, token_types_start: 649, token_types_count: 0 }, // [275]
    SimpleHintData { raw_values_start: 649, raw_values_count: 1, token_types_start: 650, token_types_count: 0 }, // [276]
    SimpleHintData { raw_values_start: 650, raw_values_count: 1, token_types_start: 651, token_types_count: 0 }, // [277]
    SimpleHintData { raw_values_start: 651, raw_values_count: 1, token_types_start: 652, token_types_count: 0 }, // [278]
    SimpleHintData { raw_values_start: 652, raw_values_count: 1, token_types_start: 653, token_types_count: 0 }, // [279]
    SimpleHintData { raw_values_start: 653, raw_values_count: 1, token_types_start: 654, token_types_count: 0 }, // [280]
    SimpleHintData { raw_values_start: 654, raw_values_count: 1, token_types_start: 655, token_types_count: 0 }, // [281]
    SimpleHintData { raw_values_start: 655, raw_values_count: 20, token_types_start: 675, token_types_count: 0 }, // [282]
    SimpleHintData { raw_values_start: 675, raw_values_count: 2, token_types_start: 677, token_types_count: 0 }, // [283]
    SimpleHintData { raw_values_start: 677, raw_values_count: 1, token_types_start: 678, token_types_count: 0 }, // [284]
    SimpleHintData { raw_values_start: 678, raw_values_count: 7, token_types_start: 685, token_types_count: 0 }, // [285]
    SimpleHintData { raw_values_start: 685, raw_values_count: 1, token_types_start: 686, token_types_count: 0 }, // [286]
    SimpleHintData { raw_values_start: 686, raw_values_count: 1, token_types_start: 687, token_types_count: 0 }, // [287]
    SimpleHintData { raw_values_start: 687, raw_values_count: 1, token_types_start: 688, token_types_count: 0 }, // [288]
    SimpleHintData { raw_values_start: 688, raw_values_count: 1, token_types_start: 689, token_types_count: 0 }, // [289]
    SimpleHintData { raw_values_start: 689, raw_values_count: 5, token_types_start: 694, token_types_count: 0 }, // [290]
    SimpleHintData { raw_values_start: 694, raw_values_count: 2, token_types_start: 696, token_types_count: 0 }, // [291]
    SimpleHintData { raw_values_start: 696, raw_values_count: 6, token_types_start: 702, token_types_count: 0 }, // [292]
    SimpleHintData { raw_values_start: 702, raw_values_count: 4, token_types_start: 706, token_types_count: 0 }, // [293]
    SimpleHintData { raw_values_start: 706, raw_values_count: 3, token_types_start: 709, token_types_count: 0 }, // [294]
    SimpleHintData { raw_values_start: 709, raw_values_count: 4, token_types_start: 713, token_types_count: 0 }, // [295]
    SimpleHintData { raw_values_start: 713, raw_values_count: 8, token_types_start: 721, token_types_count: 0 }, // [296]
    SimpleHintData { raw_values_start: 721, raw_values_count: 1, token_types_start: 722, token_types_count: 0 }, // [297]
    SimpleHintData { raw_values_start: 722, raw_values_count: 5, token_types_start: 727, token_types_count: 0 }, // [298]
    SimpleHintData { raw_values_start: 727, raw_values_count: 2, token_types_start: 729, token_types_count: 0 }, // [299]
    SimpleHintData { raw_values_start: 729, raw_values_count: 18, token_types_start: 747, token_types_count: 0 }, // [300]
    SimpleHintData { raw_values_start: 747, raw_values_count: 2, token_types_start: 749, token_types_count: 0 }, // [301]
    SimpleHintData { raw_values_start: 749, raw_values_count: 1, token_types_start: 750, token_types_count: 0 }, // [302]
    SimpleHintData { raw_values_start: 750, raw_values_count: 1, token_types_start: 751, token_types_count: 0 }, // [303]
    SimpleHintData { raw_values_start: 751, raw_values_count: 2, token_types_start: 753, token_types_count: 0 }, // [304]
    SimpleHintData { raw_values_start: 753, raw_values_count: 1, token_types_start: 754, token_types_count: 0 }, // [305]
    SimpleHintData { raw_values_start: 754, raw_values_count: 1, token_types_start: 755, token_types_count: 0 }, // [306]
    SimpleHintData { raw_values_start: 755, raw_values_count: 2, token_types_start: 757, token_types_count: 0 }, // [307]
    SimpleHintData { raw_values_start: 757, raw_values_count: 1, token_types_start: 758, token_types_count: 0 }, // [308]
    SimpleHintData { raw_values_start: 758, raw_values_count: 1, token_types_start: 759, token_types_count: 0 }, // [309]
    SimpleHintData { raw_values_start: 759, raw_values_count: 2, token_types_start: 761, token_types_count: 0 }, // [310]
    SimpleHintData { raw_values_start: 761, raw_values_count: 1, token_types_start: 762, token_types_count: 0 }, // [311]
    SimpleHintData { raw_values_start: 762, raw_values_count: 2, token_types_start: 764, token_types_count: 0 }, // [312]
    SimpleHintData { raw_values_start: 764, raw_values_count: 1, token_types_start: 765, token_types_count: 0 }, // [313]
    SimpleHintData { raw_values_start: 765, raw_values_count: 1, token_types_start: 766, token_types_count: 0 }, // [314]
    SimpleHintData { raw_values_start: 766, raw_values_count: 2, token_types_start: 768, token_types_count: 0 }, // [315]
    SimpleHintData { raw_values_start: 768, raw_values_count: 1, token_types_start: 769, token_types_count: 0 }, // [316]
    SimpleHintData { raw_values_start: 769, raw_values_count: 1, token_types_start: 770, token_types_count: 0 }, // [317]
    SimpleHintData { raw_values_start: 770, raw_values_count: 2, token_types_start: 772, token_types_count: 0 }, // [318]
    SimpleHintData { raw_values_start: 772, raw_values_count: 1, token_types_start: 773, token_types_count: 0 }, // [319]
    SimpleHintData { raw_values_start: 773, raw_values_count: 1, token_types_start: 774, token_types_count: 0 }, // [320]
    SimpleHintData { raw_values_start: 774, raw_values_count: 2, token_types_start: 776, token_types_count: 0 }, // [321]
    SimpleHintData { raw_values_start: 776, raw_values_count: 1, token_types_start: 777, token_types_count: 0 }, // [322]
    SimpleHintData { raw_values_start: 777, raw_values_count: 1, token_types_start: 778, token_types_count: 0 }, // [323]
    SimpleHintData { raw_values_start: 778, raw_values_count: 1, token_types_start: 779, token_types_count: 6 }, // [324]
    SimpleHintData { raw_values_start: 785, raw_values_count: 1, token_types_start: 786, token_types_count: 0 }, // [325]
    SimpleHintData { raw_values_start: 786, raw_values_count: 2, token_types_start: 788, token_types_count: 0 }, // [326]
    SimpleHintData { raw_values_start: 788, raw_values_count: 1, token_types_start: 789, token_types_count: 0 }, // [327]
    SimpleHintData { raw_values_start: 789, raw_values_count: 2, token_types_start: 791, token_types_count: 0 }, // [328]
    SimpleHintData { raw_values_start: 791, raw_values_count: 2, token_types_start: 793, token_types_count: 0 }, // [329]
    SimpleHintData { raw_values_start: 793, raw_values_count: 1, token_types_start: 794, token_types_count: 0 }, // [330]
    SimpleHintData { raw_values_start: 794, raw_values_count: 2, token_types_start: 796, token_types_count: 0 }, // [331]
    SimpleHintData { raw_values_start: 796, raw_values_count: 1, token_types_start: 797, token_types_count: 0 }, // [332]
    SimpleHintData { raw_values_start: 797, raw_values_count: 1, token_types_start: 798, token_types_count: 0 }, // [333]
    SimpleHintData { raw_values_start: 798, raw_values_count: 1, token_types_start: 799, token_types_count: 0 }, // [334]
    SimpleHintData { raw_values_start: 799, raw_values_count: 1, token_types_start: 800, token_types_count: 0 }, // [335]
    SimpleHintData { raw_values_start: 800, raw_values_count: 2, token_types_start: 802, token_types_count: 0 }, // [336]
    SimpleHintData { raw_values_start: 802, raw_values_count: 1, token_types_start: 803, token_types_count: 0 }, // [337]
    SimpleHintData { raw_values_start: 803, raw_values_count: 1, token_types_start: 804, token_types_count: 0 }, // [338]
    SimpleHintData { raw_values_start: 804, raw_values_count: 14, token_types_start: 818, token_types_count: 0 }, // [339]
    SimpleHintData { raw_values_start: 818, raw_values_count: 2, token_types_start: 820, token_types_count: 0 }, // [340]
    SimpleHintData { raw_values_start: 820, raw_values_count: 1, token_types_start: 821, token_types_count: 0 }, // [341]
    SimpleHintData { raw_values_start: 821, raw_values_count: 10, token_types_start: 831, token_types_count: 0 }, // [342]
    SimpleHintData { raw_values_start: 831, raw_values_count: 1, token_types_start: 832, token_types_count: 0 }, // [343]
    SimpleHintData { raw_values_start: 832, raw_values_count: 1, token_types_start: 833, token_types_count: 0 }, // [344]
    SimpleHintData { raw_values_start: 833, raw_values_count: 1, token_types_start: 834, token_types_count: 0 }, // [345]
    SimpleHintData { raw_values_start: 834, raw_values_count: 16, token_types_start: 850, token_types_count: 0 }, // [346]
    SimpleHintData { raw_values_start: 850, raw_values_count: 2, token_types_start: 852, token_types_count: 0 }, // [347]
    SimpleHintData { raw_values_start: 852, raw_values_count: 3, token_types_start: 855, token_types_count: 0 }, // [348]
    SimpleHintData { raw_values_start: 855, raw_values_count: 2, token_types_start: 857, token_types_count: 0 }, // [349]
    SimpleHintData { raw_values_start: 857, raw_values_count: 1, token_types_start: 858, token_types_count: 0 }, // [350]
    SimpleHintData { raw_values_start: 858, raw_values_count: 2, token_types_start: 860, token_types_count: 0 }, // [351]
    SimpleHintData { raw_values_start: 860, raw_values_count: 17, token_types_start: 877, token_types_count: 0 }, // [352]
    SimpleHintData { raw_values_start: 877, raw_values_count: 4, token_types_start: 881, token_types_count: 0 }, // [353]
    SimpleHintData { raw_values_start: 881, raw_values_count: 2, token_types_start: 883, token_types_count: 0 }, // [354]
    SimpleHintData { raw_values_start: 883, raw_values_count: 1, token_types_start: 884, token_types_count: 0 }, // [355]
    SimpleHintData { raw_values_start: 884, raw_values_count: 3, token_types_start: 887, token_types_count: 0 }, // [356]
    SimpleHintData { raw_values_start: 887, raw_values_count: 1, token_types_start: 888, token_types_count: 0 }, // [357]
    SimpleHintData { raw_values_start: 888, raw_values_count: 1, token_types_start: 889, token_types_count: 0 }, // [358]
    SimpleHintData { raw_values_start: 889, raw_values_count: 1, token_types_start: 890, token_types_count: 0 }, // [359]
    SimpleHintData { raw_values_start: 890, raw_values_count: 6, token_types_start: 896, token_types_count: 0 }, // [360]
    SimpleHintData { raw_values_start: 896, raw_values_count: 1, token_types_start: 897, token_types_count: 0 }, // [361]
    SimpleHintData { raw_values_start: 897, raw_values_count: 1, token_types_start: 898, token_types_count: 0 }, // [362]
    SimpleHintData { raw_values_start: 898, raw_values_count: 3, token_types_start: 901, token_types_count: 0 }, // [363]
    SimpleHintData { raw_values_start: 901, raw_values_count: 1, token_types_start: 902, token_types_count: 0 }, // [364]
    SimpleHintData { raw_values_start: 902, raw_values_count: 4, token_types_start: 906, token_types_count: 0 }, // [365]
    SimpleHintData { raw_values_start: 906, raw_values_count: 1, token_types_start: 907, token_types_count: 0 }, // [366]
    SimpleHintData { raw_values_start: 907, raw_values_count: 2, token_types_start: 909, token_types_count: 0 }, // [367]
    SimpleHintData { raw_values_start: 909, raw_values_count: 1, token_types_start: 910, token_types_count: 0 }, // [368]
    SimpleHintData { raw_values_start: 910, raw_values_count: 5, token_types_start: 915, token_types_count: 0 }, // [369]
    SimpleHintData { raw_values_start: 915, raw_values_count: 1, token_types_start: 916, token_types_count: 0 }, // [370]
    SimpleHintData { raw_values_start: 916, raw_values_count: 1, token_types_start: 917, token_types_count: 0 }, // [371]
    SimpleHintData { raw_values_start: 917, raw_values_count: 1, token_types_start: 918, token_types_count: 0 }, // [372]
    SimpleHintData { raw_values_start: 918, raw_values_count: 2, token_types_start: 920, token_types_count: 0 }, // [373]
    SimpleHintData { raw_values_start: 920, raw_values_count: 2, token_types_start: 922, token_types_count: 0 }, // [374]
    SimpleHintData { raw_values_start: 922, raw_values_count: 2, token_types_start: 924, token_types_count: 0 }, // [375]
    SimpleHintData { raw_values_start: 924, raw_values_count: 1, token_types_start: 925, token_types_count: 0 }, // [376]
    SimpleHintData { raw_values_start: 925, raw_values_count: 1, token_types_start: 926, token_types_count: 0 }, // [377]
    SimpleHintData { raw_values_start: 926, raw_values_count: 1, token_types_start: 927, token_types_count: 0 }, // [378]
    SimpleHintData { raw_values_start: 927, raw_values_count: 1, token_types_start: 928, token_types_count: 0 }, // [379]
    SimpleHintData { raw_values_start: 928, raw_values_count: 1, token_types_start: 929, token_types_count: 0 }, // [380]
    SimpleHintData { raw_values_start: 929, raw_values_count: 1, token_types_start: 930, token_types_count: 0 }, // [381]
    SimpleHintData { raw_values_start: 930, raw_values_count: 11, token_types_start: 941, token_types_count: 1 }, // [382]
    SimpleHintData { raw_values_start: 942, raw_values_count: 1, token_types_start: 943, token_types_count: 0 }, // [383]
    SimpleHintData { raw_values_start: 943, raw_values_count: 2, token_types_start: 945, token_types_count: 0 }, // [384]
    SimpleHintData { raw_values_start: 945, raw_values_count: 1, token_types_start: 946, token_types_count: 0 }, // [385]
    SimpleHintData { raw_values_start: 946, raw_values_count: 1, token_types_start: 947, token_types_count: 0 }, // [386]
    SimpleHintData { raw_values_start: 947, raw_values_count: 1, token_types_start: 948, token_types_count: 0 }, // [387]
    SimpleHintData { raw_values_start: 948, raw_values_count: 1, token_types_start: 949, token_types_count: 0 }, // [388]
    SimpleHintData { raw_values_start: 949, raw_values_count: 0, token_types_start: 949, token_types_count: 1 }, // [389]
    SimpleHintData { raw_values_start: 950, raw_values_count: 1, token_types_start: 951, token_types_count: 0 }, // [390]
    SimpleHintData { raw_values_start: 951, raw_values_count: 1, token_types_start: 952, token_types_count: 0 }, // [391]
    SimpleHintData { raw_values_start: 952, raw_values_count: 1, token_types_start: 953, token_types_count: 0 }, // [392]
    SimpleHintData { raw_values_start: 953, raw_values_count: 1, token_types_start: 954, token_types_count: 0 }, // [393]
    SimpleHintData { raw_values_start: 954, raw_values_count: 1, token_types_start: 955, token_types_count: 0 }, // [394]
    SimpleHintData { raw_values_start: 955, raw_values_count: 1, token_types_start: 956, token_types_count: 0 }, // [395]
    SimpleHintData { raw_values_start: 956, raw_values_count: 1, token_types_start: 957, token_types_count: 0 }, // [396]
    SimpleHintData { raw_values_start: 957, raw_values_count: 1, token_types_start: 958, token_types_count: 0 }, // [397]
    SimpleHintData { raw_values_start: 958, raw_values_count: 1, token_types_start: 959, token_types_count: 0 }, // [398]
    SimpleHintData { raw_values_start: 959, raw_values_count: 1, token_types_start: 960, token_types_count: 0 }, // [399]
    SimpleHintData { raw_values_start: 960, raw_values_count: 1, token_types_start: 961, token_types_count: 0 }, // [400]
    SimpleHintData { raw_values_start: 961, raw_values_count: 1, token_types_start: 962, token_types_count: 0 }, // [401]
    SimpleHintData { raw_values_start: 962, raw_values_count: 1, token_types_start: 963, token_types_count: 0 }, // [402]
    SimpleHintData { raw_values_start: 963, raw_values_count: 1, token_types_start: 964, token_types_count: 0 }, // [403]
    SimpleHintData { raw_values_start: 964, raw_values_count: 1, token_types_start: 965, token_types_count: 0 }, // [404]
    SimpleHintData { raw_values_start: 965, raw_values_count: 1, token_types_start: 966, token_types_count: 0 }, // [405]
    SimpleHintData { raw_values_start: 966, raw_values_count: 1, token_types_start: 967, token_types_count: 0 }, // [406]
    SimpleHintData { raw_values_start: 967, raw_values_count: 1, token_types_start: 968, token_types_count: 0 }, // [407]
    SimpleHintData { raw_values_start: 968, raw_values_count: 0, token_types_start: 968, token_types_count: 1 }, // [408]
    SimpleHintData { raw_values_start: 969, raw_values_count: 1, token_types_start: 970, token_types_count: 0 }, // [409]
    SimpleHintData { raw_values_start: 970, raw_values_count: 1, token_types_start: 971, token_types_count: 0 }, // [410]
    SimpleHintData { raw_values_start: 971, raw_values_count: 1, token_types_start: 972, token_types_count: 0 }, // [411]
    SimpleHintData { raw_values_start: 972, raw_values_count: 1, token_types_start: 973, token_types_count: 0 }, // [412]
    SimpleHintData { raw_values_start: 973, raw_values_count: 1, token_types_start: 974, token_types_count: 0 }, // [413]
    SimpleHintData { raw_values_start: 974, raw_values_count: 1, token_types_start: 975, token_types_count: 0 }, // [414]
    SimpleHintData { raw_values_start: 975, raw_values_count: 18, token_types_start: 993, token_types_count: 4 }, // [415]
    SimpleHintData { raw_values_start: 997, raw_values_count: 2, token_types_start: 999, token_types_count: 0 }, // [416]
    SimpleHintData { raw_values_start: 999, raw_values_count: 2, token_types_start: 1001, token_types_count: 0 }, // [417]
    SimpleHintData { raw_values_start: 1001, raw_values_count: 6, token_types_start: 1007, token_types_count: 2 }, // [418]
    SimpleHintData { raw_values_start: 1009, raw_values_count: 0, token_types_start: 1009, token_types_count: 1 }, // [419]
    SimpleHintData { raw_values_start: 1010, raw_values_count: 1, token_types_start: 1011, token_types_count: 0 }, // [420]
    SimpleHintData { raw_values_start: 1011, raw_values_count: 1, token_types_start: 1012, token_types_count: 0 }, // [421]
    SimpleHintData { raw_values_start: 1012, raw_values_count: 1, token_types_start: 1013, token_types_count: 0 }, // [422]
    SimpleHintData { raw_values_start: 1013, raw_values_count: 1, token_types_start: 1014, token_types_count: 0 }, // [423]
    SimpleHintData { raw_values_start: 1014, raw_values_count: 1, token_types_start: 1015, token_types_count: 0 }, // [424]
    SimpleHintData { raw_values_start: 1015, raw_values_count: 1, token_types_start: 1016, token_types_count: 0 }, // [425]
    SimpleHintData { raw_values_start: 1016, raw_values_count: 1, token_types_start: 1017, token_types_count: 0 }, // [426]
    SimpleHintData { raw_values_start: 1017, raw_values_count: 1, token_types_start: 1018, token_types_count: 0 }, // [427]
    SimpleHintData { raw_values_start: 1018, raw_values_count: 1, token_types_start: 1019, token_types_count: 0 }, // [428]
    SimpleHintData { raw_values_start: 1019, raw_values_count: 1, token_types_start: 1020, token_types_count: 0 }, // [429]
    SimpleHintData { raw_values_start: 1020, raw_values_count: 1, token_types_start: 1021, token_types_count: 0 }, // [430]
    SimpleHintData { raw_values_start: 1021, raw_values_count: 8, token_types_start: 1029, token_types_count: 0 }, // [431]
    SimpleHintData { raw_values_start: 1029, raw_values_count: 1, token_types_start: 1030, token_types_count: 0 }, // [432]
    SimpleHintData { raw_values_start: 1030, raw_values_count: 1, token_types_start: 1031, token_types_count: 0 }, // [433]
    SimpleHintData { raw_values_start: 1031, raw_values_count: 1, token_types_start: 1032, token_types_count: 0 }, // [434]
    SimpleHintData { raw_values_start: 1032, raw_values_count: 1, token_types_start: 1033, token_types_count: 0 }, // [435]
    SimpleHintData { raw_values_start: 1033, raw_values_count: 1, token_types_start: 1034, token_types_count: 0 }, // [436]
    SimpleHintData { raw_values_start: 1034, raw_values_count: 1, token_types_start: 1035, token_types_count: 0 }, // [437]
    SimpleHintData { raw_values_start: 1035, raw_values_count: 2, token_types_start: 1037, token_types_count: 0 }, // [438]
    SimpleHintData { raw_values_start: 1037, raw_values_count: 12, token_types_start: 1049, token_types_count: 0 }, // [439]
    SimpleHintData { raw_values_start: 1049, raw_values_count: 2, token_types_start: 1051, token_types_count: 0 }, // [440]
    SimpleHintData { raw_values_start: 1051, raw_values_count: 2, token_types_start: 1053, token_types_count: 0 }, // [441]
    SimpleHintData { raw_values_start: 1053, raw_values_count: 1, token_types_start: 1054, token_types_count: 0 }, // [442]
    SimpleHintData { raw_values_start: 1054, raw_values_count: 1, token_types_start: 1055, token_types_count: 0 }, // [443]
    SimpleHintData { raw_values_start: 1055, raw_values_count: 1, token_types_start: 1056, token_types_count: 0 }, // [444]
    SimpleHintData { raw_values_start: 1056, raw_values_count: 1, token_types_start: 1057, token_types_count: 0 }, // [445]
    SimpleHintData { raw_values_start: 1057, raw_values_count: 1, token_types_start: 1058, token_types_count: 0 }, // [446]
    SimpleHintData { raw_values_start: 1058, raw_values_count: 1, token_types_start: 1059, token_types_count: 0 }, // [447]
    SimpleHintData { raw_values_start: 1059, raw_values_count: 1, token_types_start: 1060, token_types_count: 0 }, // [448]
    SimpleHintData { raw_values_start: 1060, raw_values_count: 1, token_types_start: 1061, token_types_count: 0 }, // [449]
    SimpleHintData { raw_values_start: 1061, raw_values_count: 1, token_types_start: 1062, token_types_count: 0 }, // [450]
    SimpleHintData { raw_values_start: 1062, raw_values_count: 1, token_types_start: 1063, token_types_count: 0 }, // [451]
    SimpleHintData { raw_values_start: 1063, raw_values_count: 1, token_types_start: 1064, token_types_count: 0 }, // [452]
    SimpleHintData { raw_values_start: 1064, raw_values_count: 1, token_types_start: 1065, token_types_count: 0 }, // [453]
    SimpleHintData { raw_values_start: 1065, raw_values_count: 1, token_types_start: 1066, token_types_count: 0 }, // [454]
    SimpleHintData { raw_values_start: 1066, raw_values_count: 1, token_types_start: 1067, token_types_count: 0 }, // [455]
    SimpleHintData { raw_values_start: 1067, raw_values_count: 1, token_types_start: 1068, token_types_count: 0 }, // [456]
    SimpleHintData { raw_values_start: 1068, raw_values_count: 1, token_types_start: 1069, token_types_count: 0 }, // [457]
    SimpleHintData { raw_values_start: 1069, raw_values_count: 1, token_types_start: 1070, token_types_count: 0 }, // [458]
    SimpleHintData { raw_values_start: 1070, raw_values_count: 1, token_types_start: 1071, token_types_count: 0 }, // [459]
    SimpleHintData { raw_values_start: 1071, raw_values_count: 1, token_types_start: 1072, token_types_count: 0 }, // [460]
    SimpleHintData { raw_values_start: 1072, raw_values_count: 1, token_types_start: 1073, token_types_count: 0 }, // [461]
    SimpleHintData { raw_values_start: 1073, raw_values_count: 1, token_types_start: 1074, token_types_count: 0 }, // [462]
    SimpleHintData { raw_values_start: 1074, raw_values_count: 1, token_types_start: 1075, token_types_count: 0 }, // [463]
    SimpleHintData { raw_values_start: 1075, raw_values_count: 29, token_types_start: 1104, token_types_count: 4 }, // [464]
    SimpleHintData { raw_values_start: 1108, raw_values_count: 1, token_types_start: 1109, token_types_count: 0 }, // [465]
    SimpleHintData { raw_values_start: 1109, raw_values_count: 1, token_types_start: 1110, token_types_count: 0 }, // [466]
    SimpleHintData { raw_values_start: 1110, raw_values_count: 1, token_types_start: 1111, token_types_count: 0 }, // [467]
    SimpleHintData { raw_values_start: 1111, raw_values_count: 1, token_types_start: 1112, token_types_count: 0 }, // [468]
    SimpleHintData { raw_values_start: 1112, raw_values_count: 2, token_types_start: 1114, token_types_count: 0 }, // [469]
    SimpleHintData { raw_values_start: 1114, raw_values_count: 1, token_types_start: 1115, token_types_count: 1 }, // [470]
    SimpleHintData { raw_values_start: 1116, raw_values_count: 0, token_types_start: 1116, token_types_count: 1 }, // [471]
    SimpleHintData { raw_values_start: 1117, raw_values_count: 1, token_types_start: 1118, token_types_count: 0 }, // [472]
    SimpleHintData { raw_values_start: 1118, raw_values_count: 1, token_types_start: 1119, token_types_count: 0 }, // [473]
    SimpleHintData { raw_values_start: 1119, raw_values_count: 33, token_types_start: 1152, token_types_count: 0 }, // [474]
    SimpleHintData { raw_values_start: 1152, raw_values_count: 2, token_types_start: 1154, token_types_count: 0 }, // [475]
    SimpleHintData { raw_values_start: 1154, raw_values_count: 20, token_types_start: 1174, token_types_count: 0 }, // [476]
    SimpleHintData { raw_values_start: 1174, raw_values_count: 3, token_types_start: 1177, token_types_count: 0 }, // [477]
    SimpleHintData { raw_values_start: 1177, raw_values_count: 1, token_types_start: 1178, token_types_count: 0 }, // [478]
    SimpleHintData { raw_values_start: 1178, raw_values_count: 1, token_types_start: 1179, token_types_count: 0 }, // [479]
    SimpleHintData { raw_values_start: 1179, raw_values_count: 1, token_types_start: 1180, token_types_count: 0 }, // [480]
    SimpleHintData { raw_values_start: 1180, raw_values_count: 1, token_types_start: 1181, token_types_count: 0 }, // [481]
    SimpleHintData { raw_values_start: 1181, raw_values_count: 1, token_types_start: 1182, token_types_count: 0 }, // [482]
    SimpleHintData { raw_values_start: 1182, raw_values_count: 2, token_types_start: 1184, token_types_count: 0 }, // [483]
    SimpleHintData { raw_values_start: 1184, raw_values_count: 0, token_types_start: 1184, token_types_count: 1 }, // [484]
    SimpleHintData { raw_values_start: 1185, raw_values_count: 1, token_types_start: 1186, token_types_count: 0 }, // [485]
    SimpleHintData { raw_values_start: 1186, raw_values_count: 0, token_types_start: 1186, token_types_count: 1 }, // [486]
    SimpleHintData { raw_values_start: 1187, raw_values_count: 1, token_types_start: 1188, token_types_count: 0 }, // [487]
    SimpleHintData { raw_values_start: 1188, raw_values_count: 1, token_types_start: 1189, token_types_count: 0 }, // [488]
    SimpleHintData { raw_values_start: 1189, raw_values_count: 1, token_types_start: 1190, token_types_count: 0 }, // [489]
    SimpleHintData { raw_values_start: 1190, raw_values_count: 1, token_types_start: 1191, token_types_count: 0 }, // [490]
    SimpleHintData { raw_values_start: 1191, raw_values_count: 1, token_types_start: 1192, token_types_count: 0 }, // [491]
    SimpleHintData { raw_values_start: 1192, raw_values_count: 4, token_types_start: 1196, token_types_count: 0 }, // [492]
    SimpleHintData { raw_values_start: 1196, raw_values_count: 3, token_types_start: 1199, token_types_count: 0 }, // [493]
    SimpleHintData { raw_values_start: 1199, raw_values_count: 1, token_types_start: 1200, token_types_count: 0 }, // [494]
    SimpleHintData { raw_values_start: 1200, raw_values_count: 2, token_types_start: 1202, token_types_count: 0 }, // [495]
    SimpleHintData { raw_values_start: 1202, raw_values_count: 1, token_types_start: 1203, token_types_count: 0 }, // [496]
    SimpleHintData { raw_values_start: 1203, raw_values_count: 1, token_types_start: 1204, token_types_count: 0 }, // [497]
    SimpleHintData { raw_values_start: 1204, raw_values_count: 2, token_types_start: 1206, token_types_count: 0 }, // [498]
    SimpleHintData { raw_values_start: 1206, raw_values_count: 1, token_types_start: 1207, token_types_count: 0 }, // [499]
    SimpleHintData { raw_values_start: 1207, raw_values_count: 1, token_types_start: 1208, token_types_count: 0 }, // [500]
    SimpleHintData { raw_values_start: 1208, raw_values_count: 1, token_types_start: 1209, token_types_count: 0 }, // [501]
    SimpleHintData { raw_values_start: 1209, raw_values_count: 1, token_types_start: 1210, token_types_count: 0 }, // [502]
    SimpleHintData { raw_values_start: 1210, raw_values_count: 1, token_types_start: 1211, token_types_count: 0 }, // [503]
    SimpleHintData { raw_values_start: 1211, raw_values_count: 2, token_types_start: 1213, token_types_count: 0 }, // [504]
    SimpleHintData { raw_values_start: 1213, raw_values_count: 11, token_types_start: 1224, token_types_count: 0 }, // [505]
    SimpleHintData { raw_values_start: 1224, raw_values_count: 1, token_types_start: 1225, token_types_count: 0 }, // [506]
    SimpleHintData { raw_values_start: 1225, raw_values_count: 1, token_types_start: 1226, token_types_count: 0 }, // [507]
    SimpleHintData { raw_values_start: 1226, raw_values_count: 1, token_types_start: 1227, token_types_count: 0 }, // [508]
    SimpleHintData { raw_values_start: 1227, raw_values_count: 1, token_types_start: 1228, token_types_count: 0 }, // [509]
    SimpleHintData { raw_values_start: 1228, raw_values_count: 1, token_types_start: 1229, token_types_count: 0 }, // [510]
    SimpleHintData { raw_values_start: 1229, raw_values_count: 1, token_types_start: 1230, token_types_count: 0 }, // [511]
    SimpleHintData { raw_values_start: 1230, raw_values_count: 1, token_types_start: 1231, token_types_count: 0 }, // [512]
    SimpleHintData { raw_values_start: 1231, raw_values_count: 11, token_types_start: 1242, token_types_count: 0 }, // [513]
    SimpleHintData { raw_values_start: 1242, raw_values_count: 1, token_types_start: 1243, token_types_count: 0 }, // [514]
    SimpleHintData { raw_values_start: 1243, raw_values_count: 1, token_types_start: 1244, token_types_count: 0 }, // [515]
    SimpleHintData { raw_values_start: 1244, raw_values_count: 1, token_types_start: 1245, token_types_count: 0 }, // [516]
    SimpleHintData { raw_values_start: 1245, raw_values_count: 1, token_types_start: 1246, token_types_count: 0 }, // [517]
    SimpleHintData { raw_values_start: 1246, raw_values_count: 1, token_types_start: 1247, token_types_count: 0 }, // [518]
    SimpleHintData { raw_values_start: 1247, raw_values_count: 1, token_types_start: 1248, token_types_count: 0 }, // [519]
    SimpleHintData { raw_values_start: 1248, raw_values_count: 2, token_types_start: 1250, token_types_count: 0 }, // [520]
    SimpleHintData { raw_values_start: 1250, raw_values_count: 1, token_types_start: 1251, token_types_count: 0 }, // [521]
    SimpleHintData { raw_values_start: 1251, raw_values_count: 1, token_types_start: 1252, token_types_count: 0 }, // [522]
    SimpleHintData { raw_values_start: 1252, raw_values_count: 1, token_types_start: 1253, token_types_count: 0 }, // [523]
    SimpleHintData { raw_values_start: 1253, raw_values_count: 1, token_types_start: 1254, token_types_count: 0 }, // [524]
    SimpleHintData { raw_values_start: 1254, raw_values_count: 1, token_types_start: 1255, token_types_count: 0 }, // [525]
    SimpleHintData { raw_values_start: 1255, raw_values_count: 2, token_types_start: 1257, token_types_count: 0 }, // [526]
    SimpleHintData { raw_values_start: 1257, raw_values_count: 1, token_types_start: 1258, token_types_count: 0 }, // [527]
    SimpleHintData { raw_values_start: 1258, raw_values_count: 1, token_types_start: 1259, token_types_count: 0 }, // [528]
    SimpleHintData { raw_values_start: 1259, raw_values_count: 2, token_types_start: 1261, token_types_count: 0 }, // [529]
    SimpleHintData { raw_values_start: 1261, raw_values_count: 6, token_types_start: 1267, token_types_count: 0 }, // [530]
    SimpleHintData { raw_values_start: 1267, raw_values_count: 1, token_types_start: 1268, token_types_count: 0 }, // [531]
    SimpleHintData { raw_values_start: 1268, raw_values_count: 1, token_types_start: 1269, token_types_count: 0 }, // [532]
    SimpleHintData { raw_values_start: 1269, raw_values_count: 1, token_types_start: 1270, token_types_count: 0 }, // [533]
    SimpleHintData { raw_values_start: 1270, raw_values_count: 1, token_types_start: 1271, token_types_count: 0 }, // [534]
    SimpleHintData { raw_values_start: 1271, raw_values_count: 2, token_types_start: 1273, token_types_count: 0 }, // [535]
    SimpleHintData { raw_values_start: 1273, raw_values_count: 1, token_types_start: 1274, token_types_count: 0 }, // [536]
    SimpleHintData { raw_values_start: 1274, raw_values_count: 1, token_types_start: 1275, token_types_count: 0 }, // [537]
    SimpleHintData { raw_values_start: 1275, raw_values_count: 1, token_types_start: 1276, token_types_count: 0 }, // [538]
    SimpleHintData { raw_values_start: 1276, raw_values_count: 1, token_types_start: 1277, token_types_count: 0 }, // [539]
    SimpleHintData { raw_values_start: 1277, raw_values_count: 10, token_types_start: 1287, token_types_count: 0 }, // [540]
    SimpleHintData { raw_values_start: 1287, raw_values_count: 1, token_types_start: 1288, token_types_count: 6 }, // [541]
    SimpleHintData { raw_values_start: 1294, raw_values_count: 3, token_types_start: 1297, token_types_count: 0 }, // [542]
    SimpleHintData { raw_values_start: 1297, raw_values_count: 2, token_types_start: 1299, token_types_count: 0 }, // [543]
    SimpleHintData { raw_values_start: 1299, raw_values_count: 4, token_types_start: 1303, token_types_count: 0 }, // [544]
    SimpleHintData { raw_values_start: 1303, raw_values_count: 1, token_types_start: 1304, token_types_count: 0 }, // [545]
    SimpleHintData { raw_values_start: 1304, raw_values_count: 1, token_types_start: 1305, token_types_count: 0 }, // [546]
    SimpleHintData { raw_values_start: 1305, raw_values_count: 6, token_types_start: 1311, token_types_count: 0 }, // [547]
    SimpleHintData { raw_values_start: 1311, raw_values_count: 2, token_types_start: 1313, token_types_count: 0 }, // [548]
    SimpleHintData { raw_values_start: 1313, raw_values_count: 1, token_types_start: 1314, token_types_count: 0 }, // [549]
    SimpleHintData { raw_values_start: 1314, raw_values_count: 2, token_types_start: 1316, token_types_count: 0 }, // [550]
    SimpleHintData { raw_values_start: 1316, raw_values_count: 2, token_types_start: 1318, token_types_count: 0 }, // [551]
    SimpleHintData { raw_values_start: 1318, raw_values_count: 1, token_types_start: 1319, token_types_count: 0 }, // [552]
    SimpleHintData { raw_values_start: 1319, raw_values_count: 1, token_types_start: 1320, token_types_count: 0 }, // [553]
    SimpleHintData { raw_values_start: 1320, raw_values_count: 1, token_types_start: 1321, token_types_count: 0 }, // [554]
    SimpleHintData { raw_values_start: 1321, raw_values_count: 1, token_types_start: 1322, token_types_count: 0 }, // [555]
    SimpleHintData { raw_values_start: 1322, raw_values_count: 14, token_types_start: 1336, token_types_count: 0 }, // [556]
    SimpleHintData { raw_values_start: 1336, raw_values_count: 2, token_types_start: 1338, token_types_count: 0 }, // [557]
    SimpleHintData { raw_values_start: 1338, raw_values_count: 2, token_types_start: 1340, token_types_count: 0 }, // [558]
    SimpleHintData { raw_values_start: 1340, raw_values_count: 1, token_types_start: 1341, token_types_count: 0 }, // [559]
    SimpleHintData { raw_values_start: 1341, raw_values_count: 10, token_types_start: 1351, token_types_count: 0 }, // [560]
    SimpleHintData { raw_values_start: 1351, raw_values_count: 2, token_types_start: 1353, token_types_count: 0 }, // [561]
    SimpleHintData { raw_values_start: 1353, raw_values_count: 1, token_types_start: 1354, token_types_count: 0 }, // [562]
    SimpleHintData { raw_values_start: 1354, raw_values_count: 1, token_types_start: 1355, token_types_count: 0 }, // [563]
    SimpleHintData { raw_values_start: 1355, raw_values_count: 1, token_types_start: 1356, token_types_count: 0 }, // [564]
    SimpleHintData { raw_values_start: 1356, raw_values_count: 4, token_types_start: 1360, token_types_count: 0 }, // [565]
    SimpleHintData { raw_values_start: 1360, raw_values_count: 1, token_types_start: 1361, token_types_count: 0 }, // [566]
    SimpleHintData { raw_values_start: 1361, raw_values_count: 2, token_types_start: 1363, token_types_count: 0 }, // [567]
    SimpleHintData { raw_values_start: 1363, raw_values_count: 1, token_types_start: 1364, token_types_count: 0 }, // [568]
    SimpleHintData { raw_values_start: 1364, raw_values_count: 1, token_types_start: 1365, token_types_count: 0 }, // [569]
    SimpleHintData { raw_values_start: 1365, raw_values_count: 2, token_types_start: 1367, token_types_count: 0 }, // [570]
    SimpleHintData { raw_values_start: 1367, raw_values_count: 1, token_types_start: 1368, token_types_count: 0 }, // [571]
    SimpleHintData { raw_values_start: 1368, raw_values_count: 1, token_types_start: 1369, token_types_count: 0 }, // [572]
    SimpleHintData { raw_values_start: 1369, raw_values_count: 5, token_types_start: 1374, token_types_count: 0 }, // [573]
    SimpleHintData { raw_values_start: 1374, raw_values_count: 22, token_types_start: 1396, token_types_count: 0 }, // [574]
    SimpleHintData { raw_values_start: 1396, raw_values_count: 24, token_types_start: 1420, token_types_count: 0 }, // [575]
    SimpleHintData { raw_values_start: 1420, raw_values_count: 23, token_types_start: 1443, token_types_count: 0 }, // [576]
    SimpleHintData { raw_values_start: 1443, raw_values_count: 1, token_types_start: 1444, token_types_count: 0 }, // [577]
    SimpleHintData { raw_values_start: 1444, raw_values_count: 9, token_types_start: 1453, token_types_count: 0 }, // [578]
    SimpleHintData { raw_values_start: 1453, raw_values_count: 4, token_types_start: 1457, token_types_count: 0 }, // [579]
    SimpleHintData { raw_values_start: 1457, raw_values_count: 1, token_types_start: 1458, token_types_count: 0 }, // [580]
    SimpleHintData { raw_values_start: 1458, raw_values_count: 1, token_types_start: 1459, token_types_count: 0 }, // [581]
    SimpleHintData { raw_values_start: 1459, raw_values_count: 1, token_types_start: 1460, token_types_count: 0 }, // [582]
    SimpleHintData { raw_values_start: 1460, raw_values_count: 2, token_types_start: 1462, token_types_count: 0 }, // [583]
    SimpleHintData { raw_values_start: 1462, raw_values_count: 2, token_types_start: 1464, token_types_count: 0 }, // [584]
    SimpleHintData { raw_values_start: 1464, raw_values_count: 1, token_types_start: 1465, token_types_count: 0 }, // [585]
    SimpleHintData { raw_values_start: 1465, raw_values_count: 1, token_types_start: 1466, token_types_count: 0 }, // [586]
    SimpleHintData { raw_values_start: 1466, raw_values_count: 2, token_types_start: 1468, token_types_count: 0 }, // [587]
    SimpleHintData { raw_values_start: 1468, raw_values_count: 3, token_types_start: 1471, token_types_count: 0 }, // [588]
    SimpleHintData { raw_values_start: 1471, raw_values_count: 1, token_types_start: 1472, token_types_count: 0 }, // [589]
    SimpleHintData { raw_values_start: 1472, raw_values_count: 4, token_types_start: 1476, token_types_count: 0 }, // [590]
    SimpleHintData { raw_values_start: 1476, raw_values_count: 7, token_types_start: 1483, token_types_count: 0 }, // [591]
    SimpleHintData { raw_values_start: 1483, raw_values_count: 3, token_types_start: 1486, token_types_count: 0 }, // [592]
    SimpleHintData { raw_values_start: 1486, raw_values_count: 2, token_types_start: 1488, token_types_count: 0 }, // [593]
    SimpleHintData { raw_values_start: 1488, raw_values_count: 1, token_types_start: 1489, token_types_count: 0 }, // [594]
    SimpleHintData { raw_values_start: 1489, raw_values_count: 2, token_types_start: 1491, token_types_count: 0 }, // [595]
    SimpleHintData { raw_values_start: 1491, raw_values_count: 1, token_types_start: 1492, token_types_count: 0 }, // [596]
    SimpleHintData { raw_values_start: 1492, raw_values_count: 1, token_types_start: 1493, token_types_count: 0 }, // [597]
    SimpleHintData { raw_values_start: 1493, raw_values_count: 1, token_types_start: 1494, token_types_count: 0 }, // [598]
    SimpleHintData { raw_values_start: 1494, raw_values_count: 2, token_types_start: 1496, token_types_count: 0 }, // [599]
    SimpleHintData { raw_values_start: 1496, raw_values_count: 1, token_types_start: 1497, token_types_count: 0 }, // [600]
    SimpleHintData { raw_values_start: 1497, raw_values_count: 1, token_types_start: 1498, token_types_count: 0 }, // [601]
    SimpleHintData { raw_values_start: 1498, raw_values_count: 1, token_types_start: 1499, token_types_count: 0 }, // [602]
    SimpleHintData { raw_values_start: 1499, raw_values_count: 1, token_types_start: 1500, token_types_count: 0 }, // [603]
    SimpleHintData { raw_values_start: 1500, raw_values_count: 4, token_types_start: 1504, token_types_count: 0 }, // [604]
    SimpleHintData { raw_values_start: 1504, raw_values_count: 1, token_types_start: 1505, token_types_count: 0 }, // [605]
    SimpleHintData { raw_values_start: 1505, raw_values_count: 1, token_types_start: 1506, token_types_count: 0 }, // [606]
    SimpleHintData { raw_values_start: 1506, raw_values_count: 1, token_types_start: 1507, token_types_count: 0 }, // [607]
    SimpleHintData { raw_values_start: 1507, raw_values_count: 1, token_types_start: 1508, token_types_count: 0 }, // [608]
    SimpleHintData { raw_values_start: 1508, raw_values_count: 2, token_types_start: 1510, token_types_count: 0 }, // [609]
    SimpleHintData { raw_values_start: 1510, raw_values_count: 1, token_types_start: 1511, token_types_count: 0 }, // [610]
    SimpleHintData { raw_values_start: 1511, raw_values_count: 1, token_types_start: 1512, token_types_count: 0 }, // [611]
    SimpleHintData { raw_values_start: 1512, raw_values_count: 1, token_types_start: 1513, token_types_count: 0 }, // [612]
    SimpleHintData { raw_values_start: 1513, raw_values_count: 1, token_types_start: 1514, token_types_count: 0 }, // [613]
    SimpleHintData { raw_values_start: 1514, raw_values_count: 1, token_types_start: 1515, token_types_count: 0 }, // [614]
    SimpleHintData { raw_values_start: 1515, raw_values_count: 1, token_types_start: 1516, token_types_count: 0 }, // [615]
    SimpleHintData { raw_values_start: 1516, raw_values_count: 1, token_types_start: 1517, token_types_count: 0 }, // [616]
    SimpleHintData { raw_values_start: 1517, raw_values_count: 1, token_types_start: 1518, token_types_count: 0 }, // [617]
    SimpleHintData { raw_values_start: 1518, raw_values_count: 38, token_types_start: 1556, token_types_count: 0 }, // [618]
    SimpleHintData { raw_values_start: 1556, raw_values_count: 6, token_types_start: 1562, token_types_count: 0 }, // [619]
    SimpleHintData { raw_values_start: 1562, raw_values_count: 55, token_types_start: 1617, token_types_count: 0 }, // [620]
    SimpleHintData { raw_values_start: 1617, raw_values_count: 1, token_types_start: 1618, token_types_count: 0 }, // [621]
    SimpleHintData { raw_values_start: 1618, raw_values_count: 1, token_types_start: 1619, token_types_count: 0 }, // [622]
    SimpleHintData { raw_values_start: 1619, raw_values_count: 1, token_types_start: 1620, token_types_count: 0 }, // [623]
    SimpleHintData { raw_values_start: 1620, raw_values_count: 1, token_types_start: 1621, token_types_count: 0 }, // [624]
    SimpleHintData { raw_values_start: 1621, raw_values_count: 1, token_types_start: 1622, token_types_count: 0 }, // [625]
    SimpleHintData { raw_values_start: 1622, raw_values_count: 1, token_types_start: 1623, token_types_count: 0 }, // [626]
    SimpleHintData { raw_values_start: 1623, raw_values_count: 1, token_types_start: 1624, token_types_count: 0 }, // [627]
    SimpleHintData { raw_values_start: 1624, raw_values_count: 1, token_types_start: 1625, token_types_count: 0 }, // [628]
    SimpleHintData { raw_values_start: 1625, raw_values_count: 1, token_types_start: 1626, token_types_count: 0 }, // [629]
    SimpleHintData { raw_values_start: 1626, raw_values_count: 1, token_types_start: 1627, token_types_count: 0 }, // [630]
    SimpleHintData { raw_values_start: 1627, raw_values_count: 1, token_types_start: 1628, token_types_count: 0 }, // [631]
    SimpleHintData { raw_values_start: 1628, raw_values_count: 1, token_types_start: 1629, token_types_count: 0 }, // [632]
    SimpleHintData { raw_values_start: 1629, raw_values_count: 1, token_types_start: 1630, token_types_count: 0 }, // [633]
    SimpleHintData { raw_values_start: 1630, raw_values_count: 1, token_types_start: 1631, token_types_count: 0 }, // [634]
    SimpleHintData { raw_values_start: 1631, raw_values_count: 1, token_types_start: 1632, token_types_count: 0 }, // [635]
    SimpleHintData { raw_values_start: 1632, raw_values_count: 1, token_types_start: 1633, token_types_count: 0 }, // [636]
    SimpleHintData { raw_values_start: 1633, raw_values_count: 1, token_types_start: 1634, token_types_count: 0 }, // [637]
    SimpleHintData { raw_values_start: 1634, raw_values_count: 2, token_types_start: 1636, token_types_count: 0 }, // [638]
    SimpleHintData { raw_values_start: 1636, raw_values_count: 1, token_types_start: 1637, token_types_count: 0 }, // [639]
    SimpleHintData { raw_values_start: 1637, raw_values_count: 1, token_types_start: 1638, token_types_count: 0 }, // [640]
    SimpleHintData { raw_values_start: 1638, raw_values_count: 1, token_types_start: 1639, token_types_count: 0 }, // [641]
    SimpleHintData { raw_values_start: 1639, raw_values_count: 5, token_types_start: 1644, token_types_count: 0 }, // [642]
    SimpleHintData { raw_values_start: 1644, raw_values_count: 4, token_types_start: 1648, token_types_count: 0 }, // [643]
    SimpleHintData { raw_values_start: 1648, raw_values_count: 1, token_types_start: 1649, token_types_count: 0 }, // [644]
    SimpleHintData { raw_values_start: 1649, raw_values_count: 1, token_types_start: 1650, token_types_count: 0 }, // [645]
    SimpleHintData { raw_values_start: 1650, raw_values_count: 2, token_types_start: 1652, token_types_count: 0 }, // [646]
    SimpleHintData { raw_values_start: 1652, raw_values_count: 7, token_types_start: 1659, token_types_count: 0 }, // [647]
    SimpleHintData { raw_values_start: 1659, raw_values_count: 1, token_types_start: 1660, token_types_count: 0 }, // [648]
    SimpleHintData { raw_values_start: 1660, raw_values_count: 1, token_types_start: 1661, token_types_count: 0 }, // [649]
    SimpleHintData { raw_values_start: 1661, raw_values_count: 1, token_types_start: 1662, token_types_count: 0 }, // [650]
    SimpleHintData { raw_values_start: 1662, raw_values_count: 1, token_types_start: 1663, token_types_count: 0 }, // [651]
    SimpleHintData { raw_values_start: 1663, raw_values_count: 1, token_types_start: 1664, token_types_count: 0 }, // [652]
    SimpleHintData { raw_values_start: 1664, raw_values_count: 4, token_types_start: 1668, token_types_count: 0 }, // [653]
    SimpleHintData { raw_values_start: 1668, raw_values_count: 1, token_types_start: 1669, token_types_count: 0 }, // [654]
    SimpleHintData { raw_values_start: 1669, raw_values_count: 1, token_types_start: 1670, token_types_count: 0 }, // [655]
    SimpleHintData { raw_values_start: 1670, raw_values_count: 1, token_types_start: 1671, token_types_count: 0 }, // [656]
    SimpleHintData { raw_values_start: 1671, raw_values_count: 2, token_types_start: 1673, token_types_count: 0 }, // [657]
    SimpleHintData { raw_values_start: 1673, raw_values_count: 1, token_types_start: 1674, token_types_count: 0 }, // [658]
    SimpleHintData { raw_values_start: 1674, raw_values_count: 1, token_types_start: 1675, token_types_count: 0 }, // [659]
    SimpleHintData { raw_values_start: 1675, raw_values_count: 1, token_types_start: 1676, token_types_count: 0 }, // [660]
    SimpleHintData { raw_values_start: 1676, raw_values_count: 1, token_types_start: 1677, token_types_count: 0 }, // [661]
    SimpleHintData { raw_values_start: 1677, raw_values_count: 2, token_types_start: 1679, token_types_count: 0 }, // [662]
    SimpleHintData { raw_values_start: 1679, raw_values_count: 1, token_types_start: 1680, token_types_count: 0 }, // [663]
    SimpleHintData { raw_values_start: 1680, raw_values_count: 1, token_types_start: 1681, token_types_count: 0 }, // [664]
    SimpleHintData { raw_values_start: 1681, raw_values_count: 1, token_types_start: 1682, token_types_count: 0 }, // [665]
    SimpleHintData { raw_values_start: 1682, raw_values_count: 1, token_types_start: 1683, token_types_count: 0 }, // [666]
    SimpleHintData { raw_values_start: 1683, raw_values_count: 1, token_types_start: 1684, token_types_count: 0 }, // [667]
    SimpleHintData { raw_values_start: 1684, raw_values_count: 1, token_types_start: 1685, token_types_count: 0 }, // [668]
    SimpleHintData { raw_values_start: 1685, raw_values_count: 1, token_types_start: 1686, token_types_count: 0 }, // [669]
    SimpleHintData { raw_values_start: 1686, raw_values_count: 1, token_types_start: 1687, token_types_count: 0 }, // [670]
    SimpleHintData { raw_values_start: 1687, raw_values_count: 1, token_types_start: 1688, token_types_count: 0 }, // [671]
    SimpleHintData { raw_values_start: 1688, raw_values_count: 1, token_types_start: 1689, token_types_count: 0 }, // [672]
    SimpleHintData { raw_values_start: 1689, raw_values_count: 2, token_types_start: 1691, token_types_count: 0 }, // [673]
    SimpleHintData { raw_values_start: 1691, raw_values_count: 1, token_types_start: 1692, token_types_count: 0 }, // [674]
    SimpleHintData { raw_values_start: 1692, raw_values_count: 0, token_types_start: 1692, token_types_count: 0 }, // [675]
    SimpleHintData { raw_values_start: 1692, raw_values_count: 33, token_types_start: 1725, token_types_count: 0 }, // [676]
    SimpleHintData { raw_values_start: 1725, raw_values_count: 6, token_types_start: 1731, token_types_count: 6 }, // [677]
    SimpleHintData { raw_values_start: 1737, raw_values_count: 2, token_types_start: 1739, token_types_count: 0 }, // [678]
    SimpleHintData { raw_values_start: 1739, raw_values_count: 3, token_types_start: 1742, token_types_count: 0 }, // [679]
    SimpleHintData { raw_values_start: 1742, raw_values_count: 1, token_types_start: 1743, token_types_count: 0 }, // [680]
    SimpleHintData { raw_values_start: 1743, raw_values_count: 1, token_types_start: 1744, token_types_count: 0 }, // [681]
    SimpleHintData { raw_values_start: 1744, raw_values_count: 1, token_types_start: 1745, token_types_count: 0 }, // [682]
    SimpleHintData { raw_values_start: 1745, raw_values_count: 1, token_types_start: 1746, token_types_count: 0 }, // [683]
    SimpleHintData { raw_values_start: 1746, raw_values_count: 1, token_types_start: 1747, token_types_count: 0 }, // [684]
    SimpleHintData { raw_values_start: 1747, raw_values_count: 1, token_types_start: 1748, token_types_count: 0 }, // [685]
    SimpleHintData { raw_values_start: 1748, raw_values_count: 1, token_types_start: 1749, token_types_count: 0 }, // [686]
    SimpleHintData { raw_values_start: 1749, raw_values_count: 1, token_types_start: 1750, token_types_count: 0 }, // [687]
    SimpleHintData { raw_values_start: 1750, raw_values_count: 1, token_types_start: 1751, token_types_count: 0 }, // [688]
    SimpleHintData { raw_values_start: 1751, raw_values_count: 1, token_types_start: 1752, token_types_count: 0 }, // [689]
    SimpleHintData { raw_values_start: 1752, raw_values_count: 1, token_types_start: 1753, token_types_count: 0 }, // [690]
    SimpleHintData { raw_values_start: 1753, raw_values_count: 1, token_types_start: 1754, token_types_count: 0 }, // [691]
    SimpleHintData { raw_values_start: 1754, raw_values_count: 1, token_types_start: 1755, token_types_count: 0 }, // [692]
    SimpleHintData { raw_values_start: 1755, raw_values_count: 1, token_types_start: 1756, token_types_count: 0 }, // [693]
    SimpleHintData { raw_values_start: 1756, raw_values_count: 1, token_types_start: 1757, token_types_count: 0 }, // [694]
    SimpleHintData { raw_values_start: 1757, raw_values_count: 1, token_types_start: 1758, token_types_count: 0 }, // [695]
    SimpleHintData { raw_values_start: 1758, raw_values_count: 1, token_types_start: 1759, token_types_count: 0 }, // [696]
    SimpleHintData { raw_values_start: 1759, raw_values_count: 1, token_types_start: 1760, token_types_count: 0 }, // [697]
    SimpleHintData { raw_values_start: 1760, raw_values_count: 17, token_types_start: 1777, token_types_count: 9 }, // [698]
    SimpleHintData { raw_values_start: 1786, raw_values_count: 0, token_types_start: 1786, token_types_count: 1 }, // [699]
    SimpleHintData { raw_values_start: 1787, raw_values_count: 1, token_types_start: 1788, token_types_count: 0 }, // [700]
    SimpleHintData { raw_values_start: 1788, raw_values_count: 1, token_types_start: 1789, token_types_count: 0 }, // [701]
    SimpleHintData { raw_values_start: 1789, raw_values_count: 1, token_types_start: 1790, token_types_count: 0 }, // [702]
    SimpleHintData { raw_values_start: 1790, raw_values_count: 1, token_types_start: 1791, token_types_count: 0 }, // [703]
    SimpleHintData { raw_values_start: 1791, raw_values_count: 1, token_types_start: 1792, token_types_count: 0 }, // [704]
    SimpleHintData { raw_values_start: 1792, raw_values_count: 1, token_types_start: 1793, token_types_count: 0 }, // [705]
    SimpleHintData { raw_values_start: 1793, raw_values_count: 1, token_types_start: 1794, token_types_count: 0 }, // [706]
    SimpleHintData { raw_values_start: 1794, raw_values_count: 1, token_types_start: 1795, token_types_count: 0 }, // [707]
    SimpleHintData { raw_values_start: 1795, raw_values_count: 1, token_types_start: 1796, token_types_count: 0 }, // [708]
    SimpleHintData { raw_values_start: 1796, raw_values_count: 1, token_types_start: 1797, token_types_count: 0 }, // [709]
    SimpleHintData { raw_values_start: 1797, raw_values_count: 1, token_types_start: 1798, token_types_count: 0 }, // [710]
    SimpleHintData { raw_values_start: 1798, raw_values_count: 5, token_types_start: 1803, token_types_count: 0 }, // [711]
    SimpleHintData { raw_values_start: 1803, raw_values_count: 1, token_types_start: 1804, token_types_count: 0 }, // [712]
    SimpleHintData { raw_values_start: 1804, raw_values_count: 1, token_types_start: 1805, token_types_count: 0 }, // [713]
    SimpleHintData { raw_values_start: 1805, raw_values_count: 1, token_types_start: 1806, token_types_count: 0 }, // [714]
    SimpleHintData { raw_values_start: 1806, raw_values_count: 1, token_types_start: 1807, token_types_count: 0 }, // [715]
    SimpleHintData { raw_values_start: 1807, raw_values_count: 1, token_types_start: 1808, token_types_count: 0 }, // [716]
    SimpleHintData { raw_values_start: 1808, raw_values_count: 1, token_types_start: 1809, token_types_count: 0 }, // [717]
    SimpleHintData { raw_values_start: 1809, raw_values_count: 1, token_types_start: 1810, token_types_count: 0 }, // [718]
    SimpleHintData { raw_values_start: 1810, raw_values_count: 1, token_types_start: 1811, token_types_count: 6 }, // [719]
    SimpleHintData { raw_values_start: 1817, raw_values_count: 1, token_types_start: 1818, token_types_count: 0 }, // [720]
    SimpleHintData { raw_values_start: 1818, raw_values_count: 0, token_types_start: 1818, token_types_count: 1 }, // [721]
    SimpleHintData { raw_values_start: 1819, raw_values_count: 1, token_types_start: 1820, token_types_count: 0 }, // [722]
    SimpleHintData { raw_values_start: 1820, raw_values_count: 1, token_types_start: 1821, token_types_count: 0 }, // [723]
    SimpleHintData { raw_values_start: 1821, raw_values_count: 1, token_types_start: 1822, token_types_count: 0 }, // [724]
    SimpleHintData { raw_values_start: 1822, raw_values_count: 1, token_types_start: 1823, token_types_count: 0 }, // [725]
    SimpleHintData { raw_values_start: 1823, raw_values_count: 1, token_types_start: 1824, token_types_count: 0 }, // [726]
    SimpleHintData { raw_values_start: 1824, raw_values_count: 1, token_types_start: 1825, token_types_count: 0 }, // [727]
    SimpleHintData { raw_values_start: 1825, raw_values_count: 1, token_types_start: 1826, token_types_count: 0 }, // [728]
    SimpleHintData { raw_values_start: 1826, raw_values_count: 1, token_types_start: 1827, token_types_count: 0 }, // [729]
    SimpleHintData { raw_values_start: 1827, raw_values_count: 1, token_types_start: 1828, token_types_count: 0 }, // [730]
    SimpleHintData { raw_values_start: 1828, raw_values_count: 1, token_types_start: 1829, token_types_count: 0 }, // [731]
    SimpleHintData { raw_values_start: 1829, raw_values_count: 1, token_types_start: 1830, token_types_count: 0 }, // [732]
    SimpleHintData { raw_values_start: 1830, raw_values_count: 1, token_types_start: 1831, token_types_count: 0 }, // [733]
    SimpleHintData { raw_values_start: 1831, raw_values_count: 1, token_types_start: 1832, token_types_count: 0 }, // [734]
    SimpleHintData { raw_values_start: 1832, raw_values_count: 1, token_types_start: 1833, token_types_count: 0 }, // [735]
    SimpleHintData { raw_values_start: 1833, raw_values_count: 1, token_types_start: 1834, token_types_count: 0 }, // [736]
    SimpleHintData { raw_values_start: 1834, raw_values_count: 1, token_types_start: 1835, token_types_count: 0 }, // [737]
    SimpleHintData { raw_values_start: 1835, raw_values_count: 10, token_types_start: 1845, token_types_count: 0 }, // [738]
    SimpleHintData { raw_values_start: 1845, raw_values_count: 9, token_types_start: 1854, token_types_count: 0 }, // [739]
    SimpleHintData { raw_values_start: 1854, raw_values_count: 1, token_types_start: 1855, token_types_count: 0 }, // [740]
    SimpleHintData { raw_values_start: 1855, raw_values_count: 1, token_types_start: 1856, token_types_count: 0 }, // [741]
    SimpleHintData { raw_values_start: 1856, raw_values_count: 1, token_types_start: 1857, token_types_count: 0 }, // [742]
    SimpleHintData { raw_values_start: 1857, raw_values_count: 1, token_types_start: 1858, token_types_count: 0 }, // [743]
    SimpleHintData { raw_values_start: 1858, raw_values_count: 4, token_types_start: 1862, token_types_count: 0 }, // [744]
    SimpleHintData { raw_values_start: 1862, raw_values_count: 1, token_types_start: 1863, token_types_count: 0 }, // [745]
    SimpleHintData { raw_values_start: 1863, raw_values_count: 4, token_types_start: 1867, token_types_count: 0 }, // [746]
    SimpleHintData { raw_values_start: 1867, raw_values_count: 3, token_types_start: 1870, token_types_count: 0 }, // [747]
    SimpleHintData { raw_values_start: 1870, raw_values_count: 5, token_types_start: 1875, token_types_count: 0 }, // [748]
    SimpleHintData { raw_values_start: 1875, raw_values_count: 2, token_types_start: 1877, token_types_count: 0 }, // [749]
    SimpleHintData { raw_values_start: 1877, raw_values_count: 11, token_types_start: 1888, token_types_count: 0 }, // [750]
    SimpleHintData { raw_values_start: 1888, raw_values_count: 1, token_types_start: 1889, token_types_count: 0 }, // [751]
    SimpleHintData { raw_values_start: 1889, raw_values_count: 1, token_types_start: 1890, token_types_count: 0 }, // [752]
    SimpleHintData { raw_values_start: 1890, raw_values_count: 2, token_types_start: 1892, token_types_count: 0 }, // [753]
    SimpleHintData { raw_values_start: 1892, raw_values_count: 5, token_types_start: 1897, token_types_count: 1 }, // [754]
    SimpleHintData { raw_values_start: 1898, raw_values_count: 1, token_types_start: 1899, token_types_count: 0 }, // [755]
    SimpleHintData { raw_values_start: 1899, raw_values_count: 1, token_types_start: 1900, token_types_count: 0 }, // [756]
    SimpleHintData { raw_values_start: 1900, raw_values_count: 16, token_types_start: 1916, token_types_count: 3 }, // [757]
    SimpleHintData { raw_values_start: 1919, raw_values_count: 3, token_types_start: 1922, token_types_count: 0 }, // [758]
    SimpleHintData { raw_values_start: 1922, raw_values_count: 1, token_types_start: 1923, token_types_count: 0 }, // [759]
    SimpleHintData { raw_values_start: 1923, raw_values_count: 0, token_types_start: 1923, token_types_count: 1 }, // [760]
    SimpleHintData { raw_values_start: 1924, raw_values_count: 0, token_types_start: 1924, token_types_count: 1 }, // [761]
    SimpleHintData { raw_values_start: 1925, raw_values_count: 2, token_types_start: 1927, token_types_count: 0 }, // [762]
    SimpleHintData { raw_values_start: 1927, raw_values_count: 9, token_types_start: 1936, token_types_count: 7 }, // [763]
    SimpleHintData { raw_values_start: 1943, raw_values_count: 1, token_types_start: 1944, token_types_count: 0 }, // [764]
    SimpleHintData { raw_values_start: 1944, raw_values_count: 1, token_types_start: 1945, token_types_count: 0 }, // [765]
    SimpleHintData { raw_values_start: 1945, raw_values_count: 2, token_types_start: 1947, token_types_count: 0 }, // [766]
    SimpleHintData { raw_values_start: 1947, raw_values_count: 1, token_types_start: 1948, token_types_count: 0 }, // [767]
    SimpleHintData { raw_values_start: 1948, raw_values_count: 2, token_types_start: 1950, token_types_count: 0 }, // [768]
    SimpleHintData { raw_values_start: 1950, raw_values_count: 2, token_types_start: 1952, token_types_count: 0 }, // [769]
    SimpleHintData { raw_values_start: 1952, raw_values_count: 1, token_types_start: 1953, token_types_count: 0 }, // [770]
    SimpleHintData { raw_values_start: 1953, raw_values_count: 1, token_types_start: 1954, token_types_count: 0 }, // [771]
    SimpleHintData { raw_values_start: 1954, raw_values_count: 0, token_types_start: 1954, token_types_count: 1 }, // [772]
    SimpleHintData { raw_values_start: 1955, raw_values_count: 0, token_types_start: 1955, token_types_count: 1 }, // [773]
    SimpleHintData { raw_values_start: 1956, raw_values_count: 0, token_types_start: 1956, token_types_count: 1 }, // [774]
    SimpleHintData { raw_values_start: 1957, raw_values_count: 1, token_types_start: 1958, token_types_count: 0 }, // [775]
    SimpleHintData { raw_values_start: 1958, raw_values_count: 1, token_types_start: 1959, token_types_count: 0 }, // [776]
    SimpleHintData { raw_values_start: 1959, raw_values_count: 1, token_types_start: 1960, token_types_count: 0 }, // [777]
    SimpleHintData { raw_values_start: 1960, raw_values_count: 1, token_types_start: 1961, token_types_count: 0 }, // [778]
    SimpleHintData { raw_values_start: 1961, raw_values_count: 4, token_types_start: 1965, token_types_count: 0 }, // [779]
    SimpleHintData { raw_values_start: 1965, raw_values_count: 2, token_types_start: 1967, token_types_count: 0 }, // [780]
    SimpleHintData { raw_values_start: 1967, raw_values_count: 1, token_types_start: 1968, token_types_count: 0 }, // [781]
    SimpleHintData { raw_values_start: 1968, raw_values_count: 5, token_types_start: 1973, token_types_count: 0 }, // [782]
    SimpleHintData { raw_values_start: 1973, raw_values_count: 1, token_types_start: 1974, token_types_count: 0 }, // [783]
    SimpleHintData { raw_values_start: 1974, raw_values_count: 3, token_types_start: 1977, token_types_count: 0 }, // [784]
    SimpleHintData { raw_values_start: 1977, raw_values_count: 1, token_types_start: 1978, token_types_count: 0 }, // [785]
    SimpleHintData { raw_values_start: 1978, raw_values_count: 2, token_types_start: 1980, token_types_count: 0 }, // [786]
    SimpleHintData { raw_values_start: 1980, raw_values_count: 1, token_types_start: 1981, token_types_count: 0 }, // [787]
    SimpleHintData { raw_values_start: 1981, raw_values_count: 4, token_types_start: 1985, token_types_count: 0 }, // [788]
    SimpleHintData { raw_values_start: 1985, raw_values_count: 1, token_types_start: 1986, token_types_count: 0 }, // [789]
    SimpleHintData { raw_values_start: 1986, raw_values_count: 1, token_types_start: 1987, token_types_count: 0 }, // [790]
    SimpleHintData { raw_values_start: 1987, raw_values_count: 1, token_types_start: 1988, token_types_count: 0 }, // [791]
    SimpleHintData { raw_values_start: 1988, raw_values_count: 1, token_types_start: 1989, token_types_count: 0 }, // [792]
    SimpleHintData { raw_values_start: 1989, raw_values_count: 1, token_types_start: 1990, token_types_count: 0 }, // [793]
    SimpleHintData { raw_values_start: 1990, raw_values_count: 15, token_types_start: 2005, token_types_count: 0 }, // [794]
    SimpleHintData { raw_values_start: 2005, raw_values_count: 1, token_types_start: 2006, token_types_count: 0 }, // [795]
    SimpleHintData { raw_values_start: 2006, raw_values_count: 1, token_types_start: 2007, token_types_count: 0 }, // [796]
    SimpleHintData { raw_values_start: 2007, raw_values_count: 1, token_types_start: 2008, token_types_count: 0 }, // [797]
    SimpleHintData { raw_values_start: 2008, raw_values_count: 4, token_types_start: 2012, token_types_count: 0 }, // [798]
    SimpleHintData { raw_values_start: 2012, raw_values_count: 1, token_types_start: 2013, token_types_count: 0 }, // [799]
    SimpleHintData { raw_values_start: 2013, raw_values_count: 1, token_types_start: 2014, token_types_count: 0 }, // [800]
    SimpleHintData { raw_values_start: 2014, raw_values_count: 1, token_types_start: 2015, token_types_count: 0 }, // [801]
    SimpleHintData { raw_values_start: 2015, raw_values_count: 1, token_types_start: 2016, token_types_count: 0 }, // [802]
    SimpleHintData { raw_values_start: 2016, raw_values_count: 1, token_types_start: 2017, token_types_count: 0 }, // [803]
    SimpleHintData { raw_values_start: 2017, raw_values_count: 1, token_types_start: 2018, token_types_count: 0 }, // [804]
    SimpleHintData { raw_values_start: 2018, raw_values_count: 1, token_types_start: 2019, token_types_count: 0 }, // [805]
    SimpleHintData { raw_values_start: 2019, raw_values_count: 3, token_types_start: 2022, token_types_count: 0 }, // [806]
    SimpleHintData { raw_values_start: 2022, raw_values_count: 2, token_types_start: 2024, token_types_count: 0 }, // [807]
    SimpleHintData { raw_values_start: 2024, raw_values_count: 1, token_types_start: 2025, token_types_count: 0 }, // [808]
    SimpleHintData { raw_values_start: 2025, raw_values_count: 2, token_types_start: 2027, token_types_count: 0 }, // [809]
    SimpleHintData { raw_values_start: 2027, raw_values_count: 2, token_types_start: 2029, token_types_count: 0 }, // [810]
    SimpleHintData { raw_values_start: 2029, raw_values_count: 21, token_types_start: 2050, token_types_count: 0 }, // [811]
    SimpleHintData { raw_values_start: 2050, raw_values_count: 1, token_types_start: 2051, token_types_count: 0 }, // [812]
    SimpleHintData { raw_values_start: 2051, raw_values_count: 12, token_types_start: 2063, token_types_count: 0 }, // [813]
    SimpleHintData { raw_values_start: 2063, raw_values_count: 9, token_types_start: 2072, token_types_count: 0 }, // [814]
    SimpleHintData { raw_values_start: 2072, raw_values_count: 4, token_types_start: 2076, token_types_count: 0 }, // [815]
    SimpleHintData { raw_values_start: 2076, raw_values_count: 1, token_types_start: 2077, token_types_count: 0 }, // [816]
    SimpleHintData { raw_values_start: 2077, raw_values_count: 1, token_types_start: 2078, token_types_count: 0 }, // [817]
    SimpleHintData { raw_values_start: 2078, raw_values_count: 3, token_types_start: 2081, token_types_count: 0 }, // [818]
    SimpleHintData { raw_values_start: 2081, raw_values_count: 2, token_types_start: 2083, token_types_count: 1 }, // [819]
    SimpleHintData { raw_values_start: 2084, raw_values_count: 1, token_types_start: 2085, token_types_count: 0 }, // [820]
    SimpleHintData { raw_values_start: 2085, raw_values_count: 1, token_types_start: 2086, token_types_count: 0 }, // [821]
    SimpleHintData { raw_values_start: 2086, raw_values_count: 1, token_types_start: 2087, token_types_count: 0 }, // [822]
    SimpleHintData { raw_values_start: 2087, raw_values_count: 1, token_types_start: 2088, token_types_count: 0 }, // [823]
    SimpleHintData { raw_values_start: 2088, raw_values_count: 1, token_types_start: 2089, token_types_count: 0 }, // [824]
    SimpleHintData { raw_values_start: 2089, raw_values_count: 1, token_types_start: 2090, token_types_count: 0 }, // [825]
    SimpleHintData { raw_values_start: 2090, raw_values_count: 1, token_types_start: 2091, token_types_count: 0 }, // [826]
    SimpleHintData { raw_values_start: 2091, raw_values_count: 1, token_types_start: 2092, token_types_count: 0 }, // [827]
    SimpleHintData { raw_values_start: 2092, raw_values_count: 2, token_types_start: 2094, token_types_count: 0 }, // [828]
    SimpleHintData { raw_values_start: 2094, raw_values_count: 1, token_types_start: 2095, token_types_count: 0 }, // [829]
    SimpleHintData { raw_values_start: 2095, raw_values_count: 1, token_types_start: 2096, token_types_count: 0 }, // [830]
    SimpleHintData { raw_values_start: 2096, raw_values_count: 6, token_types_start: 2102, token_types_count: 0 }, // [831]
    SimpleHintData { raw_values_start: 2102, raw_values_count: 1, token_types_start: 2103, token_types_count: 0 }, // [832]
    SimpleHintData { raw_values_start: 2103, raw_values_count: 1, token_types_start: 2104, token_types_count: 0 }, // [833]
    SimpleHintData { raw_values_start: 2104, raw_values_count: 1, token_types_start: 2105, token_types_count: 0 }, // [834]
    SimpleHintData { raw_values_start: 2105, raw_values_count: 5, token_types_start: 2110, token_types_count: 0 }, // [835]
    SimpleHintData { raw_values_start: 2110, raw_values_count: 1, token_types_start: 2111, token_types_count: 0 }, // [836]
    SimpleHintData { raw_values_start: 2111, raw_values_count: 1, token_types_start: 2112, token_types_count: 0 }, // [837]
    SimpleHintData { raw_values_start: 2112, raw_values_count: 1, token_types_start: 2113, token_types_count: 0 }, // [838]
    SimpleHintData { raw_values_start: 2113, raw_values_count: 1, token_types_start: 2114, token_types_count: 0 }, // [839]
    SimpleHintData { raw_values_start: 2114, raw_values_count: 1, token_types_start: 2115, token_types_count: 0 }, // [840]
    SimpleHintData { raw_values_start: 2115, raw_values_count: 1, token_types_start: 2116, token_types_count: 0 }, // [841]
    SimpleHintData { raw_values_start: 2116, raw_values_count: 1, token_types_start: 2117, token_types_count: 0 }, // [842]
    SimpleHintData { raw_values_start: 2117, raw_values_count: 1, token_types_start: 2118, token_types_count: 0 }, // [843]
    SimpleHintData { raw_values_start: 2118, raw_values_count: 1, token_types_start: 2119, token_types_count: 0 }, // [844]
    SimpleHintData { raw_values_start: 2119, raw_values_count: 1, token_types_start: 2120, token_types_count: 0 }, // [845]
    SimpleHintData { raw_values_start: 2120, raw_values_count: 1, token_types_start: 2121, token_types_count: 0 }, // [846]
    SimpleHintData { raw_values_start: 2121, raw_values_count: 1, token_types_start: 2122, token_types_count: 0 }, // [847]
    SimpleHintData { raw_values_start: 2122, raw_values_count: 1, token_types_start: 2123, token_types_count: 0 }, // [848]
    SimpleHintData { raw_values_start: 2123, raw_values_count: 1, token_types_start: 2124, token_types_count: 0 }, // [849]
    SimpleHintData { raw_values_start: 2124, raw_values_count: 1, token_types_start: 2125, token_types_count: 0 }, // [850]
    SimpleHintData { raw_values_start: 2125, raw_values_count: 1, token_types_start: 2126, token_types_count: 0 }, // [851]
    SimpleHintData { raw_values_start: 2126, raw_values_count: 1, token_types_start: 2127, token_types_count: 0 }, // [852]
    SimpleHintData { raw_values_start: 2127, raw_values_count: 1, token_types_start: 2128, token_types_count: 0 }, // [853]
    SimpleHintData { raw_values_start: 2128, raw_values_count: 2, token_types_start: 2130, token_types_count: 0 }, // [854]
    SimpleHintData { raw_values_start: 2130, raw_values_count: 1, token_types_start: 2131, token_types_count: 0 }, // [855]
    SimpleHintData { raw_values_start: 2131, raw_values_count: 1, token_types_start: 2132, token_types_count: 0 }, // [856]
    SimpleHintData { raw_values_start: 2132, raw_values_count: 2, token_types_start: 2134, token_types_count: 0 }, // [857]
    SimpleHintData { raw_values_start: 2134, raw_values_count: 2, token_types_start: 2136, token_types_count: 0 }, // [858]
    SimpleHintData { raw_values_start: 2136, raw_values_count: 1, token_types_start: 2137, token_types_count: 0 }, // [859]
    SimpleHintData { raw_values_start: 2137, raw_values_count: 3, token_types_start: 2140, token_types_count: 0 }, // [860]
    SimpleHintData { raw_values_start: 2140, raw_values_count: 1, token_types_start: 2141, token_types_count: 0 }, // [861]
    SimpleHintData { raw_values_start: 2141, raw_values_count: 1, token_types_start: 2142, token_types_count: 0 }, // [862]
    SimpleHintData { raw_values_start: 2142, raw_values_count: 1, token_types_start: 2143, token_types_count: 0 }, // [863]
    SimpleHintData { raw_values_start: 2143, raw_values_count: 1, token_types_start: 2144, token_types_count: 0 }, // [864]
    SimpleHintData { raw_values_start: 2144, raw_values_count: 1, token_types_start: 2145, token_types_count: 0 }, // [865]
    SimpleHintData { raw_values_start: 2145, raw_values_count: 1, token_types_start: 2146, token_types_count: 0 }, // [866]
    SimpleHintData { raw_values_start: 2146, raw_values_count: 1, token_types_start: 2147, token_types_count: 0 }, // [867]
    SimpleHintData { raw_values_start: 2147, raw_values_count: 1, token_types_start: 2148, token_types_count: 0 }, // [868]
    SimpleHintData { raw_values_start: 2148, raw_values_count: 2, token_types_start: 2150, token_types_count: 0 }, // [869]
    SimpleHintData { raw_values_start: 2150, raw_values_count: 1, token_types_start: 2151, token_types_count: 0 }, // [870]
    SimpleHintData { raw_values_start: 2151, raw_values_count: 13, token_types_start: 2164, token_types_count: 0 }, // [871]
    SimpleHintData { raw_values_start: 2164, raw_values_count: 1, token_types_start: 2165, token_types_count: 0 }, // [872]
    SimpleHintData { raw_values_start: 2165, raw_values_count: 1, token_types_start: 2166, token_types_count: 0 }, // [873]
    SimpleHintData { raw_values_start: 2166, raw_values_count: 1, token_types_start: 2167, token_types_count: 0 }, // [874]
    SimpleHintData { raw_values_start: 2167, raw_values_count: 1, token_types_start: 2168, token_types_count: 0 }, // [875]
    SimpleHintData { raw_values_start: 2168, raw_values_count: 1, token_types_start: 2169, token_types_count: 0 }, // [876]
    SimpleHintData { raw_values_start: 2169, raw_values_count: 33, token_types_start: 2202, token_types_count: 6 }, // [877]
    SimpleHintData { raw_values_start: 2208, raw_values_count: 6, token_types_start: 2214, token_types_count: 0 }, // [878]
    SimpleHintData { raw_values_start: 2214, raw_values_count: 1, token_types_start: 2215, token_types_count: 0 }, // [879]
    SimpleHintData { raw_values_start: 2215, raw_values_count: 1, token_types_start: 2216, token_types_count: 0 }, // [880]
    SimpleHintData { raw_values_start: 2216, raw_values_count: 1, token_types_start: 2217, token_types_count: 0 }, // [881]
    SimpleHintData { raw_values_start: 2217, raw_values_count: 1, token_types_start: 2218, token_types_count: 0 }, // [882]
    SimpleHintData { raw_values_start: 2218, raw_values_count: 3, token_types_start: 2221, token_types_count: 0 }, // [883]
    SimpleHintData { raw_values_start: 2221, raw_values_count: 1, token_types_start: 2222, token_types_count: 0 }, // [884]
    SimpleHintData { raw_values_start: 2222, raw_values_count: 1, token_types_start: 2223, token_types_count: 0 }, // [885]
    SimpleHintData { raw_values_start: 2223, raw_values_count: 5, token_types_start: 2228, token_types_count: 0 }, // [886]
    SimpleHintData { raw_values_start: 2228, raw_values_count: 1, token_types_start: 2229, token_types_count: 0 }, // [887]
    SimpleHintData { raw_values_start: 2229, raw_values_count: 1, token_types_start: 2230, token_types_count: 0 }, // [888]
    SimpleHintData { raw_values_start: 2230, raw_values_count: 1, token_types_start: 2231, token_types_count: 0 }, // [889]
    SimpleHintData { raw_values_start: 2231, raw_values_count: 1, token_types_start: 2232, token_types_count: 0 }, // [890]
    SimpleHintData { raw_values_start: 2232, raw_values_count: 2, token_types_start: 2234, token_types_count: 0 }, // [891]
    SimpleHintData { raw_values_start: 2234, raw_values_count: 3, token_types_start: 2237, token_types_count: 0 }, // [892]
    SimpleHintData { raw_values_start: 2237, raw_values_count: 3, token_types_start: 2240, token_types_count: 0 }, // [893]
    SimpleHintData { raw_values_start: 2240, raw_values_count: 1, token_types_start: 2241, token_types_count: 0 }, // [894]
    SimpleHintData { raw_values_start: 2241, raw_values_count: 1, token_types_start: 2242, token_types_count: 0 }, // [895]
    SimpleHintData { raw_values_start: 2242, raw_values_count: 1, token_types_start: 2243, token_types_count: 0 }, // [896]
    SimpleHintData { raw_values_start: 2243, raw_values_count: 4, token_types_start: 2247, token_types_count: 0 }, // [897]
    SimpleHintData { raw_values_start: 2247, raw_values_count: 1, token_types_start: 2248, token_types_count: 0 }, // [898]
    SimpleHintData { raw_values_start: 2248, raw_values_count: 1, token_types_start: 2249, token_types_count: 0 }, // [899]
    SimpleHintData { raw_values_start: 2249, raw_values_count: 1, token_types_start: 2250, token_types_count: 0 }, // [900]
    SimpleHintData { raw_values_start: 2250, raw_values_count: 1, token_types_start: 2251, token_types_count: 0 }, // [901]
    SimpleHintData { raw_values_start: 2251, raw_values_count: 1, token_types_start: 2252, token_types_count: 0 }, // [902]
    SimpleHintData { raw_values_start: 2252, raw_values_count: 1, token_types_start: 2253, token_types_count: 0 }, // [903]
    SimpleHintData { raw_values_start: 2253, raw_values_count: 1, token_types_start: 2254, token_types_count: 0 }, // [904]
    SimpleHintData { raw_values_start: 2254, raw_values_count: 1, token_types_start: 2255, token_types_count: 0 }, // [905]
    SimpleHintData { raw_values_start: 2255, raw_values_count: 1, token_types_start: 2256, token_types_count: 0 }, // [906]
    SimpleHintData { raw_values_start: 2256, raw_values_count: 1, token_types_start: 2257, token_types_count: 0 }, // [907]
    SimpleHintData { raw_values_start: 2257, raw_values_count: 1, token_types_start: 2258, token_types_count: 0 }, // [908]
    SimpleHintData { raw_values_start: 2258, raw_values_count: 1, token_types_start: 2259, token_types_count: 0 }, // [909]
    SimpleHintData { raw_values_start: 2259, raw_values_count: 4, token_types_start: 2263, token_types_count: 0 }, // [910]
    SimpleHintData { raw_values_start: 2263, raw_values_count: 1, token_types_start: 2264, token_types_count: 0 }, // [911]
    SimpleHintData { raw_values_start: 2264, raw_values_count: 1, token_types_start: 2265, token_types_count: 0 }, // [912]
    SimpleHintData { raw_values_start: 2265, raw_values_count: 2, token_types_start: 2267, token_types_count: 0 }, // [913]
    SimpleHintData { raw_values_start: 2267, raw_values_count: 9, token_types_start: 2276, token_types_count: 0 }, // [914]
    SimpleHintData { raw_values_start: 2276, raw_values_count: 2, token_types_start: 2278, token_types_count: 0 }, // [915]
    SimpleHintData { raw_values_start: 2278, raw_values_count: 1, token_types_start: 2279, token_types_count: 0 }, // [916]
    SimpleHintData { raw_values_start: 2279, raw_values_count: 1, token_types_start: 2280, token_types_count: 0 }, // [917]
    SimpleHintData { raw_values_start: 2280, raw_values_count: 1, token_types_start: 2281, token_types_count: 0 }, // [918]
    SimpleHintData { raw_values_start: 2281, raw_values_count: 1, token_types_start: 2282, token_types_count: 0 }, // [919]
    SimpleHintData { raw_values_start: 2282, raw_values_count: 1, token_types_start: 2283, token_types_count: 0 }, // [920]
    SimpleHintData { raw_values_start: 2283, raw_values_count: 1, token_types_start: 2284, token_types_count: 0 }, // [921]
    SimpleHintData { raw_values_start: 2284, raw_values_count: 1, token_types_start: 2285, token_types_count: 0 }, // [922]
    SimpleHintData { raw_values_start: 2285, raw_values_count: 1, token_types_start: 2286, token_types_count: 0 }, // [923]
    SimpleHintData { raw_values_start: 2286, raw_values_count: 2, token_types_start: 2288, token_types_count: 0 }, // [924]
    SimpleHintData { raw_values_start: 2288, raw_values_count: 2, token_types_start: 2290, token_types_count: 0 }, // [925]
    SimpleHintData { raw_values_start: 2290, raw_values_count: 1, token_types_start: 2291, token_types_count: 0 }, // [926]
    SimpleHintData { raw_values_start: 2291, raw_values_count: 1, token_types_start: 2292, token_types_count: 0 }, // [927]
    SimpleHintData { raw_values_start: 2292, raw_values_count: 1, token_types_start: 2293, token_types_count: 0 }, // [928]
    SimpleHintData { raw_values_start: 2293, raw_values_count: 1, token_types_start: 2294, token_types_count: 0 }, // [929]
    SimpleHintData { raw_values_start: 2294, raw_values_count: 1, token_types_start: 2295, token_types_count: 0 }, // [930]
    SimpleHintData { raw_values_start: 2295, raw_values_count: 1, token_types_start: 2296, token_types_count: 0 }, // [931]
    SimpleHintData { raw_values_start: 2296, raw_values_count: 1, token_types_start: 2297, token_types_count: 0 }, // [932]
    SimpleHintData { raw_values_start: 2297, raw_values_count: 4, token_types_start: 2301, token_types_count: 0 }, // [933]
    SimpleHintData { raw_values_start: 2301, raw_values_count: 1, token_types_start: 2302, token_types_count: 0 }, // [934]
    SimpleHintData { raw_values_start: 2302, raw_values_count: 1, token_types_start: 2303, token_types_count: 0 }, // [935]
    SimpleHintData { raw_values_start: 2303, raw_values_count: 1, token_types_start: 2304, token_types_count: 0 }, // [936]
    SimpleHintData { raw_values_start: 2304, raw_values_count: 1, token_types_start: 2305, token_types_count: 0 }, // [937]
    SimpleHintData { raw_values_start: 2305, raw_values_count: 1, token_types_start: 2306, token_types_count: 0 }, // [938]
    SimpleHintData { raw_values_start: 2306, raw_values_count: 1, token_types_start: 2307, token_types_count: 0 }, // [939]
    SimpleHintData { raw_values_start: 2307, raw_values_count: 1, token_types_start: 2308, token_types_count: 0 }, // [940]
    SimpleHintData { raw_values_start: 2308, raw_values_count: 1, token_types_start: 2309, token_types_count: 0 }, // [941]
    SimpleHintData { raw_values_start: 2309, raw_values_count: 1, token_types_start: 2310, token_types_count: 0 }, // [942]
    SimpleHintData { raw_values_start: 2310, raw_values_count: 1, token_types_start: 2311, token_types_count: 0 }, // [943]
    SimpleHintData { raw_values_start: 2311, raw_values_count: 1, token_types_start: 2312, token_types_count: 0 }, // [944]
    SimpleHintData { raw_values_start: 2312, raw_values_count: 1, token_types_start: 2313, token_types_count: 0 }, // [945]
    SimpleHintData { raw_values_start: 2313, raw_values_count: 1, token_types_start: 2314, token_types_count: 0 }, // [946]
    SimpleHintData { raw_values_start: 2314, raw_values_count: 1, token_types_start: 2315, token_types_count: 0 }, // [947]
    SimpleHintData { raw_values_start: 2315, raw_values_count: 1, token_types_start: 2316, token_types_count: 0 }, // [948]
    SimpleHintData { raw_values_start: 2316, raw_values_count: 1, token_types_start: 2317, token_types_count: 0 }, // [949]
    SimpleHintData { raw_values_start: 2317, raw_values_count: 1, token_types_start: 2318, token_types_count: 0 }, // [950]
    SimpleHintData { raw_values_start: 2318, raw_values_count: 3, token_types_start: 2321, token_types_count: 0 }, // [951]
    SimpleHintData { raw_values_start: 2321, raw_values_count: 1, token_types_start: 2322, token_types_count: 0 }, // [952]
    SimpleHintData { raw_values_start: 2322, raw_values_count: 1, token_types_start: 2323, token_types_count: 0 }, // [953]
    SimpleHintData { raw_values_start: 2323, raw_values_count: 2, token_types_start: 2325, token_types_count: 0 }, // [954]
    SimpleHintData { raw_values_start: 2325, raw_values_count: 0, token_types_start: 2325, token_types_count: 1 }, // [955]
    SimpleHintData { raw_values_start: 2326, raw_values_count: 1, token_types_start: 2327, token_types_count: 0 }, // [956]
    SimpleHintData { raw_values_start: 2327, raw_values_count: 1, token_types_start: 2328, token_types_count: 0 }, // [957]
    SimpleHintData { raw_values_start: 2328, raw_values_count: 1, token_types_start: 2329, token_types_count: 0 }, // [958]
    SimpleHintData { raw_values_start: 2329, raw_values_count: 1, token_types_start: 2330, token_types_count: 0 }, // [959]
    SimpleHintData { raw_values_start: 2330, raw_values_count: 1, token_types_start: 2331, token_types_count: 0 }, // [960]
    SimpleHintData { raw_values_start: 2331, raw_values_count: 1, token_types_start: 2332, token_types_count: 0 }, // [961]
    SimpleHintData { raw_values_start: 2332, raw_values_count: 1, token_types_start: 2333, token_types_count: 0 }, // [962]
    SimpleHintData { raw_values_start: 2333, raw_values_count: 1, token_types_start: 2334, token_types_count: 0 }, // [963]
    SimpleHintData { raw_values_start: 2334, raw_values_count: 1, token_types_start: 2335, token_types_count: 0 }, // [964]
    SimpleHintData { raw_values_start: 2335, raw_values_count: 1, token_types_start: 2336, token_types_count: 0 }, // [965]
    SimpleHintData { raw_values_start: 2336, raw_values_count: 1, token_types_start: 2337, token_types_count: 0 }, // [966]
    SimpleHintData { raw_values_start: 2337, raw_values_count: 0, token_types_start: 2337, token_types_count: 1 }, // [967]
    SimpleHintData { raw_values_start: 2338, raw_values_count: 1, token_types_start: 2339, token_types_count: 0 }, // [968]
    SimpleHintData { raw_values_start: 2339, raw_values_count: 1, token_types_start: 2340, token_types_count: 0 }, // [969]
    SimpleHintData { raw_values_start: 2340, raw_values_count: 1, token_types_start: 2341, token_types_count: 0 }, // [970]
    SimpleHintData { raw_values_start: 2341, raw_values_count: 1, token_types_start: 2342, token_types_count: 0 }, // [971]
    SimpleHintData { raw_values_start: 2342, raw_values_count: 1, token_types_start: 2343, token_types_count: 0 }, // [972]
    SimpleHintData { raw_values_start: 2343, raw_values_count: 1, token_types_start: 2344, token_types_count: 0 }, // [973]
    SimpleHintData { raw_values_start: 2344, raw_values_count: 1, token_types_start: 2345, token_types_count: 0 }, // [974]
    SimpleHintData { raw_values_start: 2345, raw_values_count: 1, token_types_start: 2346, token_types_count: 0 }, // [975]
    SimpleHintData { raw_values_start: 2346, raw_values_count: 1, token_types_start: 2347, token_types_count: 0 }, // [976]
    SimpleHintData { raw_values_start: 2347, raw_values_count: 1, token_types_start: 2348, token_types_count: 0 }, // [977]
    SimpleHintData { raw_values_start: 2348, raw_values_count: 1, token_types_start: 2349, token_types_count: 0 }, // [978]
    SimpleHintData { raw_values_start: 2349, raw_values_count: 1, token_types_start: 2350, token_types_count: 0 }, // [979]
    SimpleHintData { raw_values_start: 2350, raw_values_count: 1, token_types_start: 2351, token_types_count: 0 }, // [980]
    SimpleHintData { raw_values_start: 2351, raw_values_count: 1, token_types_start: 2352, token_types_count: 0 }, // [981]
    SimpleHintData { raw_values_start: 2352, raw_values_count: 3, token_types_start: 2355, token_types_count: 0 }, // [982]
    SimpleHintData { raw_values_start: 2355, raw_values_count: 1, token_types_start: 2356, token_types_count: 0 }, // [983]
    SimpleHintData { raw_values_start: 2356, raw_values_count: 1, token_types_start: 2357, token_types_count: 0 }, // [984]
    SimpleHintData { raw_values_start: 2357, raw_values_count: 1, token_types_start: 2358, token_types_count: 0 }, // [985]
    SimpleHintData { raw_values_start: 2358, raw_values_count: 1, token_types_start: 2359, token_types_count: 0 }, // [986]
    SimpleHintData { raw_values_start: 2359, raw_values_count: 1, token_types_start: 2360, token_types_count: 0 }, // [987]
    SimpleHintData { raw_values_start: 2360, raw_values_count: 1, token_types_start: 2361, token_types_count: 0 }, // [988]
    SimpleHintData { raw_values_start: 2361, raw_values_count: 1, token_types_start: 2362, token_types_count: 0 }, // [989]
    SimpleHintData { raw_values_start: 2362, raw_values_count: 1, token_types_start: 2363, token_types_count: 0 }, // [990]
    SimpleHintData { raw_values_start: 2363, raw_values_count: 1, token_types_start: 2364, token_types_count: 0 }, // [991]
    SimpleHintData { raw_values_start: 2364, raw_values_count: 1, token_types_start: 2365, token_types_count: 0 }, // [992]
    SimpleHintData { raw_values_start: 2365, raw_values_count: 1, token_types_start: 2366, token_types_count: 0 }, // [993]
    SimpleHintData { raw_values_start: 2366, raw_values_count: 1, token_types_start: 2367, token_types_count: 0 }, // [994]
    SimpleHintData { raw_values_start: 2367, raw_values_count: 1, token_types_start: 2368, token_types_count: 0 }, // [995]
    SimpleHintData { raw_values_start: 2368, raw_values_count: 1, token_types_start: 2369, token_types_count: 0 }, // [996]
    SimpleHintData { raw_values_start: 2369, raw_values_count: 1, token_types_start: 2370, token_types_count: 0 }, // [997]
    SimpleHintData { raw_values_start: 2370, raw_values_count: 1, token_types_start: 2371, token_types_count: 0 }, // [998]
    SimpleHintData { raw_values_start: 2371, raw_values_count: 2, token_types_start: 2373, token_types_count: 0 }, // [999]
    SimpleHintData { raw_values_start: 2373, raw_values_count: 2, token_types_start: 2375, token_types_count: 0 }, // [1000]
    SimpleHintData { raw_values_start: 2375, raw_values_count: 2, token_types_start: 2377, token_types_count: 0 }, // [1001]
    SimpleHintData { raw_values_start: 2377, raw_values_count: 1, token_types_start: 2378, token_types_count: 0 }, // [1002]
    SimpleHintData { raw_values_start: 2378, raw_values_count: 6, token_types_start: 2384, token_types_count: 0 }, // [1003]
    SimpleHintData { raw_values_start: 2384, raw_values_count: 11, token_types_start: 2395, token_types_count: 0 }, // [1004]
    SimpleHintData { raw_values_start: 2395, raw_values_count: 2, token_types_start: 2397, token_types_count: 0 }, // [1005]
    SimpleHintData { raw_values_start: 2397, raw_values_count: 1, token_types_start: 2398, token_types_count: 0 }, // [1006]
    SimpleHintData { raw_values_start: 2398, raw_values_count: 1, token_types_start: 2399, token_types_count: 0 }, // [1007]
    SimpleHintData { raw_values_start: 2399, raw_values_count: 1, token_types_start: 2400, token_types_count: 0 }, // [1008]
    SimpleHintData { raw_values_start: 2400, raw_values_count: 4, token_types_start: 2404, token_types_count: 0 }, // [1009]
    SimpleHintData { raw_values_start: 2404, raw_values_count: 1, token_types_start: 2405, token_types_count: 0 }, // [1010]
    SimpleHintData { raw_values_start: 2405, raw_values_count: 1, token_types_start: 2406, token_types_count: 0 }, // [1011]
    SimpleHintData { raw_values_start: 2406, raw_values_count: 1, token_types_start: 2407, token_types_count: 0 }, // [1012]
    SimpleHintData { raw_values_start: 2407, raw_values_count: 1, token_types_start: 2408, token_types_count: 0 }, // [1013]
    SimpleHintData { raw_values_start: 2408, raw_values_count: 1, token_types_start: 2409, token_types_count: 0 }, // [1014]
    SimpleHintData { raw_values_start: 2409, raw_values_count: 1, token_types_start: 2410, token_types_count: 0 }, // [1015]
    SimpleHintData { raw_values_start: 2410, raw_values_count: 1, token_types_start: 2411, token_types_count: 0 }, // [1016]
    SimpleHintData { raw_values_start: 2411, raw_values_count: 1, token_types_start: 2412, token_types_count: 0 }, // [1017]
    SimpleHintData { raw_values_start: 2412, raw_values_count: 1, token_types_start: 2413, token_types_count: 0 }, // [1018]
    SimpleHintData { raw_values_start: 2413, raw_values_count: 1, token_types_start: 2414, token_types_count: 0 }, // [1019]
    SimpleHintData { raw_values_start: 2414, raw_values_count: 1, token_types_start: 2415, token_types_count: 0 }, // [1020]
    SimpleHintData { raw_values_start: 2415, raw_values_count: 1, token_types_start: 2416, token_types_count: 0 }, // [1021]
    SimpleHintData { raw_values_start: 2416, raw_values_count: 1, token_types_start: 2417, token_types_count: 0 }, // [1022]
    SimpleHintData { raw_values_start: 2417, raw_values_count: 1, token_types_start: 2418, token_types_count: 0 }, // [1023]
    SimpleHintData { raw_values_start: 2418, raw_values_count: 1, token_types_start: 2419, token_types_count: 0 }, // [1024]
    SimpleHintData { raw_values_start: 2419, raw_values_count: 1, token_types_start: 2420, token_types_count: 0 }, // [1025]
    SimpleHintData { raw_values_start: 2420, raw_values_count: 0, token_types_start: 2420, token_types_count: 7 }, // [1026]
    SimpleHintData { raw_values_start: 2427, raw_values_count: 10, token_types_start: 2437, token_types_count: 0 }, // [1027]
    SimpleHintData { raw_values_start: 2437, raw_values_count: 2, token_types_start: 2439, token_types_count: 0 }, // [1028]
    SimpleHintData { raw_values_start: 2439, raw_values_count: 1, token_types_start: 2440, token_types_count: 0 }, // [1029]
    SimpleHintData { raw_values_start: 2440, raw_values_count: 1, token_types_start: 2441, token_types_count: 0 }, // [1030]
    SimpleHintData { raw_values_start: 2441, raw_values_count: 1, token_types_start: 2442, token_types_count: 0 }, // [1031]
    SimpleHintData { raw_values_start: 2442, raw_values_count: 1, token_types_start: 2443, token_types_count: 0 }, // [1032]
    SimpleHintData { raw_values_start: 2443, raw_values_count: 1, token_types_start: 2444, token_types_count: 0 }, // [1033]
    SimpleHintData { raw_values_start: 2444, raw_values_count: 1, token_types_start: 2445, token_types_count: 0 }, // [1034]
    SimpleHintData { raw_values_start: 2445, raw_values_count: 1, token_types_start: 2446, token_types_count: 0 }, // [1035]
    SimpleHintData { raw_values_start: 2446, raw_values_count: 1, token_types_start: 2447, token_types_count: 0 }, // [1036]
    SimpleHintData { raw_values_start: 2447, raw_values_count: 1, token_types_start: 2448, token_types_count: 0 }, // [1037]
    SimpleHintData { raw_values_start: 2448, raw_values_count: 1, token_types_start: 2449, token_types_count: 0 }, // [1038]
    SimpleHintData { raw_values_start: 2449, raw_values_count: 1, token_types_start: 2450, token_types_count: 0 }, // [1039]
    SimpleHintData { raw_values_start: 2450, raw_values_count: 1, token_types_start: 2451, token_types_count: 0 }, // [1040]
    SimpleHintData { raw_values_start: 2451, raw_values_count: 1, token_types_start: 2452, token_types_count: 6 }, // [1041]
    SimpleHintData { raw_values_start: 2458, raw_values_count: 1, token_types_start: 2459, token_types_count: 0 }, // [1042]
    SimpleHintData { raw_values_start: 2459, raw_values_count: 1, token_types_start: 2460, token_types_count: 0 }, // [1043]
    SimpleHintData { raw_values_start: 2460, raw_values_count: 0, token_types_start: 2460, token_types_count: 1 }, // [1044]
    SimpleHintData { raw_values_start: 2461, raw_values_count: 0, token_types_start: 2461, token_types_count: 1 }, // [1045]
    SimpleHintData { raw_values_start: 2462, raw_values_count: 1, token_types_start: 2463, token_types_count: 0 }, // [1046]
    SimpleHintData { raw_values_start: 2463, raw_values_count: 1, token_types_start: 2464, token_types_count: 0 }, // [1047]
    SimpleHintData { raw_values_start: 2464, raw_values_count: 1, token_types_start: 2465, token_types_count: 0 }, // [1048]
    SimpleHintData { raw_values_start: 2465, raw_values_count: 1, token_types_start: 2466, token_types_count: 0 }, // [1049]
    SimpleHintData { raw_values_start: 2466, raw_values_count: 1, token_types_start: 2467, token_types_count: 0 }, // [1050]
    SimpleHintData { raw_values_start: 2467, raw_values_count: 1, token_types_start: 2468, token_types_count: 0 }, // [1051]
    SimpleHintData { raw_values_start: 2468, raw_values_count: 1, token_types_start: 2469, token_types_count: 0 }, // [1052]
    SimpleHintData { raw_values_start: 2469, raw_values_count: 1, token_types_start: 2470, token_types_count: 0 }, // [1053]
    SimpleHintData { raw_values_start: 2470, raw_values_count: 1, token_types_start: 2471, token_types_count: 0 }, // [1054]
    SimpleHintData { raw_values_start: 2471, raw_values_count: 1, token_types_start: 2472, token_types_count: 0 }, // [1055]
    SimpleHintData { raw_values_start: 2472, raw_values_count: 1, token_types_start: 2473, token_types_count: 0 }, // [1056]
    SimpleHintData { raw_values_start: 2473, raw_values_count: 1, token_types_start: 2474, token_types_count: 0 }, // [1057]
    SimpleHintData { raw_values_start: 2474, raw_values_count: 0, token_types_start: 2474, token_types_count: 1 }, // [1058]
    SimpleHintData { raw_values_start: 2475, raw_values_count: 1, token_types_start: 2476, token_types_count: 0 }, // [1059]
    SimpleHintData { raw_values_start: 2476, raw_values_count: 1, token_types_start: 2477, token_types_count: 0 }, // [1060]
    SimpleHintData { raw_values_start: 2477, raw_values_count: 13, token_types_start: 2490, token_types_count: 0 }, // [1061]
    SimpleHintData { raw_values_start: 2490, raw_values_count: 1, token_types_start: 2491, token_types_count: 0 }, // [1062]
    SimpleHintData { raw_values_start: 2491, raw_values_count: 1, token_types_start: 2492, token_types_count: 0 }, // [1063]
    SimpleHintData { raw_values_start: 2492, raw_values_count: 1, token_types_start: 2493, token_types_count: 0 }, // [1064]
    SimpleHintData { raw_values_start: 2493, raw_values_count: 1, token_types_start: 2494, token_types_count: 0 }, // [1065]
    SimpleHintData { raw_values_start: 2494, raw_values_count: 1, token_types_start: 2495, token_types_count: 0 }, // [1066]
    SimpleHintData { raw_values_start: 2495, raw_values_count: 1, token_types_start: 2496, token_types_count: 0 }, // [1067]
    SimpleHintData { raw_values_start: 2496, raw_values_count: 1, token_types_start: 2497, token_types_count: 0 }, // [1068]
    SimpleHintData { raw_values_start: 2497, raw_values_count: 1, token_types_start: 2498, token_types_count: 0 }, // [1069]
    SimpleHintData { raw_values_start: 2498, raw_values_count: 1, token_types_start: 2499, token_types_count: 0 }, // [1070]
    SimpleHintData { raw_values_start: 2499, raw_values_count: 3, token_types_start: 2502, token_types_count: 0 }, // [1071]
    SimpleHintData { raw_values_start: 2502, raw_values_count: 1, token_types_start: 2503, token_types_count: 0 }, // [1072]
    SimpleHintData { raw_values_start: 2503, raw_values_count: 1, token_types_start: 2504, token_types_count: 0 }, // [1073]
    SimpleHintData { raw_values_start: 2504, raw_values_count: 0, token_types_start: 2504, token_types_count: 2 }, // [1074]
    SimpleHintData { raw_values_start: 2506, raw_values_count: 0, token_types_start: 2506, token_types_count: 1 }, // [1075]
    SimpleHintData { raw_values_start: 2507, raw_values_count: 0, token_types_start: 2507, token_types_count: 2 }, // [1076]
    SimpleHintData { raw_values_start: 2509, raw_values_count: 0, token_types_start: 2509, token_types_count: 1 }, // [1077]
    SimpleHintData { raw_values_start: 2510, raw_values_count: 0, token_types_start: 2510, token_types_count: 1 }, // [1078]
    SimpleHintData { raw_values_start: 2511, raw_values_count: 1, token_types_start: 2512, token_types_count: 0 }, // [1079]
    SimpleHintData { raw_values_start: 2512, raw_values_count: 0, token_types_start: 2512, token_types_count: 2 }, // [1080]
    SimpleHintData { raw_values_start: 2514, raw_values_count: 0, token_types_start: 2514, token_types_count: 1 }, // [1081]
    SimpleHintData { raw_values_start: 2515, raw_values_count: 1, token_types_start: 2516, token_types_count: 0 }, // [1082]
    SimpleHintData { raw_values_start: 2516, raw_values_count: 1, token_types_start: 2517, token_types_count: 0 }, // [1083]
    SimpleHintData { raw_values_start: 2517, raw_values_count: 1, token_types_start: 2518, token_types_count: 0 }, // [1084]
    SimpleHintData { raw_values_start: 2518, raw_values_count: 1, token_types_start: 2519, token_types_count: 0 }, // [1085]
    SimpleHintData { raw_values_start: 2519, raw_values_count: 1, token_types_start: 2520, token_types_count: 0 }, // [1086]
    SimpleHintData { raw_values_start: 2520, raw_values_count: 1, token_types_start: 2521, token_types_count: 0 }, // [1087]
    SimpleHintData { raw_values_start: 2521, raw_values_count: 1, token_types_start: 2522, token_types_count: 0 }, // [1088]
    SimpleHintData { raw_values_start: 2522, raw_values_count: 1, token_types_start: 2523, token_types_count: 0 }, // [1089]
    SimpleHintData { raw_values_start: 2523, raw_values_count: 1, token_types_start: 2524, token_types_count: 0 }, // [1090]
    SimpleHintData { raw_values_start: 2524, raw_values_count: 4, token_types_start: 2528, token_types_count: 0 }, // [1091]
    SimpleHintData { raw_values_start: 2528, raw_values_count: 3, token_types_start: 2531, token_types_count: 0 }, // [1092]
    SimpleHintData { raw_values_start: 2531, raw_values_count: 1, token_types_start: 2532, token_types_count: 0 }, // [1093]
    SimpleHintData { raw_values_start: 2532, raw_values_count: 2, token_types_start: 2534, token_types_count: 0 }, // [1094]
    SimpleHintData { raw_values_start: 2534, raw_values_count: 1, token_types_start: 2535, token_types_count: 0 }, // [1095]
    SimpleHintData { raw_values_start: 2535, raw_values_count: 1, token_types_start: 2536, token_types_count: 0 }, // [1096]
    SimpleHintData { raw_values_start: 2536, raw_values_count: 3, token_types_start: 2539, token_types_count: 0 }, // [1097]
    SimpleHintData { raw_values_start: 2539, raw_values_count: 5, token_types_start: 2544, token_types_count: 0 }, // [1098]
    SimpleHintData { raw_values_start: 2544, raw_values_count: 2, token_types_start: 2546, token_types_count: 0 }, // [1099]
    SimpleHintData { raw_values_start: 2546, raw_values_count: 1, token_types_start: 2547, token_types_count: 0 }, // [1100]
    SimpleHintData { raw_values_start: 2547, raw_values_count: 1, token_types_start: 2548, token_types_count: 0 }, // [1101]
    SimpleHintData { raw_values_start: 2548, raw_values_count: 1, token_types_start: 2549, token_types_count: 0 }, // [1102]
    SimpleHintData { raw_values_start: 2549, raw_values_count: 1, token_types_start: 2550, token_types_count: 0 }, // [1103]
    SimpleHintData { raw_values_start: 2550, raw_values_count: 1, token_types_start: 2551, token_types_count: 0 }, // [1104]
    SimpleHintData { raw_values_start: 2551, raw_values_count: 1, token_types_start: 2552, token_types_count: 0 }, // [1105]
    SimpleHintData { raw_values_start: 2552, raw_values_count: 1, token_types_start: 2553, token_types_count: 0 }, // [1106]
    SimpleHintData { raw_values_start: 2553, raw_values_count: 1, token_types_start: 2554, token_types_count: 0 }, // [1107]
    SimpleHintData { raw_values_start: 2554, raw_values_count: 1, token_types_start: 2555, token_types_count: 0 }, // [1108]
    SimpleHintData { raw_values_start: 2555, raw_values_count: 1, token_types_start: 2556, token_types_count: 0 }, // [1109]
    SimpleHintData { raw_values_start: 2556, raw_values_count: 1, token_types_start: 2557, token_types_count: 0 }, // [1110]
    SimpleHintData { raw_values_start: 2557, raw_values_count: 1, token_types_start: 2558, token_types_count: 0 }, // [1111]
    SimpleHintData { raw_values_start: 2558, raw_values_count: 1, token_types_start: 2559, token_types_count: 0 }, // [1112]
    SimpleHintData { raw_values_start: 2559, raw_values_count: 1, token_types_start: 2560, token_types_count: 0 }, // [1113]
    SimpleHintData { raw_values_start: 2560, raw_values_count: 1, token_types_start: 2561, token_types_count: 0 }, // [1114]
    SimpleHintData { raw_values_start: 2561, raw_values_count: 1, token_types_start: 2562, token_types_count: 0 }, // [1115]
    SimpleHintData { raw_values_start: 2562, raw_values_count: 1, token_types_start: 2563, token_types_count: 0 }, // [1116]
    SimpleHintData { raw_values_start: 2563, raw_values_count: 3, token_types_start: 2566, token_types_count: 1 }, // [1117]
    SimpleHintData { raw_values_start: 2567, raw_values_count: 2, token_types_start: 2569, token_types_count: 0 }, // [1118]
    SimpleHintData { raw_values_start: 2569, raw_values_count: 3, token_types_start: 2572, token_types_count: 0 }, // [1119]
    SimpleHintData { raw_values_start: 2572, raw_values_count: 1, token_types_start: 2573, token_types_count: 0 }, // [1120]
    SimpleHintData { raw_values_start: 2573, raw_values_count: 2, token_types_start: 2575, token_types_count: 0 }, // [1121]
    SimpleHintData { raw_values_start: 2575, raw_values_count: 1, token_types_start: 2576, token_types_count: 0 }, // [1122]
    SimpleHintData { raw_values_start: 2576, raw_values_count: 1, token_types_start: 2577, token_types_count: 0 }, // [1123]
    SimpleHintData { raw_values_start: 2577, raw_values_count: 1, token_types_start: 2578, token_types_count: 0 }, // [1124]
    SimpleHintData { raw_values_start: 2578, raw_values_count: 1, token_types_start: 2579, token_types_count: 0 }, // [1125]
    SimpleHintData { raw_values_start: 2579, raw_values_count: 22, token_types_start: 2601, token_types_count: 0 }, // [1126]
    SimpleHintData { raw_values_start: 2601, raw_values_count: 2, token_types_start: 2603, token_types_count: 0 }, // [1127]
    SimpleHintData { raw_values_start: 2603, raw_values_count: 11, token_types_start: 2614, token_types_count: 1 }, // [1128]
    SimpleHintData { raw_values_start: 2615, raw_values_count: 1, token_types_start: 2616, token_types_count: 0 }, // [1129]
    SimpleHintData { raw_values_start: 2616, raw_values_count: 1, token_types_start: 2617, token_types_count: 0 }, // [1130]
    SimpleHintData { raw_values_start: 2617, raw_values_count: 1, token_types_start: 2618, token_types_count: 0 }, // [1131]
    SimpleHintData { raw_values_start: 2618, raw_values_count: 2, token_types_start: 2620, token_types_count: 0 }, // [1132]
    SimpleHintData { raw_values_start: 2620, raw_values_count: 2, token_types_start: 2622, token_types_count: 0 }, // [1133]
    SimpleHintData { raw_values_start: 2622, raw_values_count: 2, token_types_start: 2624, token_types_count: 0 }, // [1134]
    SimpleHintData { raw_values_start: 2624, raw_values_count: 1, token_types_start: 2625, token_types_count: 0 }, // [1135]
    SimpleHintData { raw_values_start: 2625, raw_values_count: 3, token_types_start: 2628, token_types_count: 0 }, // [1136]
    SimpleHintData { raw_values_start: 2628, raw_values_count: 1, token_types_start: 2629, token_types_count: 0 }, // [1137]
    SimpleHintData { raw_values_start: 2629, raw_values_count: 1, token_types_start: 2630, token_types_count: 0 }, // [1138]
    SimpleHintData { raw_values_start: 2630, raw_values_count: 1, token_types_start: 2631, token_types_count: 0 }, // [1139]
    SimpleHintData { raw_values_start: 2631, raw_values_count: 1, token_types_start: 2632, token_types_count: 0 }, // [1140]
    SimpleHintData { raw_values_start: 2632, raw_values_count: 1, token_types_start: 2633, token_types_count: 0 }, // [1141]
    SimpleHintData { raw_values_start: 2633, raw_values_count: 1, token_types_start: 2634, token_types_count: 0 }, // [1142]
    SimpleHintData { raw_values_start: 2634, raw_values_count: 1, token_types_start: 2635, token_types_count: 0 }, // [1143]
    SimpleHintData { raw_values_start: 2635, raw_values_count: 1, token_types_start: 2636, token_types_count: 0 }, // [1144]
    SimpleHintData { raw_values_start: 2636, raw_values_count: 1, token_types_start: 2637, token_types_count: 0 }, // [1145]
    SimpleHintData { raw_values_start: 2637, raw_values_count: 1, token_types_start: 2638, token_types_count: 0 }, // [1146]
    SimpleHintData { raw_values_start: 2638, raw_values_count: 1, token_types_start: 2639, token_types_count: 0 }, // [1147]
    SimpleHintData { raw_values_start: 2639, raw_values_count: 1, token_types_start: 2640, token_types_count: 0 }, // [1148]
    SimpleHintData { raw_values_start: 2640, raw_values_count: 1, token_types_start: 2641, token_types_count: 0 }, // [1149]
    SimpleHintData { raw_values_start: 2641, raw_values_count: 1, token_types_start: 2642, token_types_count: 0 }, // [1150]
    SimpleHintData { raw_values_start: 2642, raw_values_count: 1, token_types_start: 2643, token_types_count: 0 }, // [1151]
    SimpleHintData { raw_values_start: 2643, raw_values_count: 1, token_types_start: 2644, token_types_count: 0 }, // [1152]
    SimpleHintData { raw_values_start: 2644, raw_values_count: 1, token_types_start: 2645, token_types_count: 0 }, // [1153]
    SimpleHintData { raw_values_start: 2645, raw_values_count: 1, token_types_start: 2646, token_types_count: 0 }, // [1154]
    SimpleHintData { raw_values_start: 2646, raw_values_count: 1, token_types_start: 2647, token_types_count: 0 }, // [1155]
    SimpleHintData { raw_values_start: 2647, raw_values_count: 1, token_types_start: 2648, token_types_count: 0 }, // [1156]
    SimpleHintData { raw_values_start: 2648, raw_values_count: 1, token_types_start: 2649, token_types_count: 0 }, // [1157]
    SimpleHintData { raw_values_start: 2649, raw_values_count: 1, token_types_start: 2650, token_types_count: 0 }, // [1158]
    SimpleHintData { raw_values_start: 2650, raw_values_count: 1, token_types_start: 2651, token_types_count: 0 }, // [1159]
    SimpleHintData { raw_values_start: 2651, raw_values_count: 1, token_types_start: 2652, token_types_count: 0 }, // [1160]
    SimpleHintData { raw_values_start: 2652, raw_values_count: 1, token_types_start: 2653, token_types_count: 0 }, // [1161]
    SimpleHintData { raw_values_start: 2653, raw_values_count: 5, token_types_start: 2658, token_types_count: 0 }, // [1162]
    SimpleHintData { raw_values_start: 2658, raw_values_count: 1, token_types_start: 2659, token_types_count: 0 }, // [1163]
    SimpleHintData { raw_values_start: 2659, raw_values_count: 1, token_types_start: 2660, token_types_count: 0 }, // [1164]
    SimpleHintData { raw_values_start: 2660, raw_values_count: 1, token_types_start: 2661, token_types_count: 0 }, // [1165]
    SimpleHintData { raw_values_start: 2661, raw_values_count: 1, token_types_start: 2662, token_types_count: 0 }, // [1166]
    SimpleHintData { raw_values_start: 2662, raw_values_count: 1, token_types_start: 2663, token_types_count: 0 }, // [1167]
    SimpleHintData { raw_values_start: 2663, raw_values_count: 1, token_types_start: 2664, token_types_count: 0 }, // [1168]
    SimpleHintData { raw_values_start: 2664, raw_values_count: 1, token_types_start: 2665, token_types_count: 0 }, // [1169]
    SimpleHintData { raw_values_start: 2665, raw_values_count: 1, token_types_start: 2666, token_types_count: 0 }, // [1170]
    SimpleHintData { raw_values_start: 2666, raw_values_count: 1, token_types_start: 2667, token_types_count: 0 }, // [1171]
    SimpleHintData { raw_values_start: 2667, raw_values_count: 1, token_types_start: 2668, token_types_count: 0 }, // [1172]
    SimpleHintData { raw_values_start: 2668, raw_values_count: 1, token_types_start: 2669, token_types_count: 0 }, // [1173]
    SimpleHintData { raw_values_start: 2669, raw_values_count: 1, token_types_start: 2670, token_types_count: 0 }, // [1174]
    SimpleHintData { raw_values_start: 2670, raw_values_count: 5, token_types_start: 2675, token_types_count: 0 }, // [1175]
    SimpleHintData { raw_values_start: 2675, raw_values_count: 4, token_types_start: 2679, token_types_count: 0 }, // [1176]
    SimpleHintData { raw_values_start: 2679, raw_values_count: 2, token_types_start: 2681, token_types_count: 0 }, // [1177]
    SimpleHintData { raw_values_start: 2681, raw_values_count: 3, token_types_start: 2684, token_types_count: 0 }, // [1178]
    SimpleHintData { raw_values_start: 2684, raw_values_count: 1, token_types_start: 2685, token_types_count: 0 }, // [1179]
    SimpleHintData { raw_values_start: 2685, raw_values_count: 1, token_types_start: 2686, token_types_count: 0 }, // [1180]
    SimpleHintData { raw_values_start: 2686, raw_values_count: 1, token_types_start: 2687, token_types_count: 0 }, // [1181]
    SimpleHintData { raw_values_start: 2687, raw_values_count: 1, token_types_start: 2688, token_types_count: 0 }, // [1182]
    SimpleHintData { raw_values_start: 2688, raw_values_count: 1, token_types_start: 2689, token_types_count: 0 }, // [1183]
    SimpleHintData { raw_values_start: 2689, raw_values_count: 1, token_types_start: 2690, token_types_count: 0 }, // [1184]
    SimpleHintData { raw_values_start: 2690, raw_values_count: 1, token_types_start: 2691, token_types_count: 0 }, // [1185]
    SimpleHintData { raw_values_start: 2691, raw_values_count: 1, token_types_start: 2692, token_types_count: 0 }, // [1186]
    SimpleHintData { raw_values_start: 2692, raw_values_count: 1, token_types_start: 2693, token_types_count: 0 }, // [1187]
    SimpleHintData { raw_values_start: 2693, raw_values_count: 1, token_types_start: 2694, token_types_count: 0 }, // [1188]
    SimpleHintData { raw_values_start: 2694, raw_values_count: 1, token_types_start: 2695, token_types_count: 0 }, // [1189]
    SimpleHintData { raw_values_start: 2695, raw_values_count: 1, token_types_start: 2696, token_types_count: 0 }, // [1190]
    SimpleHintData { raw_values_start: 2696, raw_values_count: 1, token_types_start: 2697, token_types_count: 0 }, // [1191]
    SimpleHintData { raw_values_start: 2697, raw_values_count: 1, token_types_start: 2698, token_types_count: 0 }, // [1192]
    SimpleHintData { raw_values_start: 2698, raw_values_count: 1, token_types_start: 2699, token_types_count: 0 }, // [1193]
    SimpleHintData { raw_values_start: 2699, raw_values_count: 1, token_types_start: 2700, token_types_count: 0 }, // [1194]
    SimpleHintData { raw_values_start: 2700, raw_values_count: 1, token_types_start: 2701, token_types_count: 0 }, // [1195]
    SimpleHintData { raw_values_start: 2701, raw_values_count: 2, token_types_start: 2703, token_types_count: 0 }, // [1196]
    SimpleHintData { raw_values_start: 2703, raw_values_count: 1, token_types_start: 2704, token_types_count: 0 }, // [1197]
    SimpleHintData { raw_values_start: 2704, raw_values_count: 1, token_types_start: 2705, token_types_count: 0 }, // [1198]
    SimpleHintData { raw_values_start: 2705, raw_values_count: 1, token_types_start: 2706, token_types_count: 0 }, // [1199]
    SimpleHintData { raw_values_start: 2706, raw_values_count: 1, token_types_start: 2707, token_types_count: 0 }, // [1200]
    SimpleHintData { raw_values_start: 2707, raw_values_count: 1, token_types_start: 2708, token_types_count: 0 }, // [1201]
    SimpleHintData { raw_values_start: 2708, raw_values_count: 2, token_types_start: 2710, token_types_count: 0 }, // [1202]
    SimpleHintData { raw_values_start: 2710, raw_values_count: 1, token_types_start: 2711, token_types_count: 0 }, // [1203]
    SimpleHintData { raw_values_start: 2711, raw_values_count: 1, token_types_start: 2712, token_types_count: 0 }, // [1204]
    SimpleHintData { raw_values_start: 2712, raw_values_count: 1, token_types_start: 2713, token_types_count: 0 }, // [1205]
    SimpleHintData { raw_values_start: 2713, raw_values_count: 1, token_types_start: 2714, token_types_count: 0 }, // [1206]
    SimpleHintData { raw_values_start: 2714, raw_values_count: 1, token_types_start: 2715, token_types_count: 0 }, // [1207]
    SimpleHintData { raw_values_start: 2715, raw_values_count: 1, token_types_start: 2716, token_types_count: 0 }, // [1208]
    SimpleHintData { raw_values_start: 2716, raw_values_count: 1, token_types_start: 2717, token_types_count: 0 }, // [1209]
    SimpleHintData { raw_values_start: 2717, raw_values_count: 1, token_types_start: 2718, token_types_count: 0 }, // [1210]
    SimpleHintData { raw_values_start: 2718, raw_values_count: 1, token_types_start: 2719, token_types_count: 0 }, // [1211]
    SimpleHintData { raw_values_start: 2719, raw_values_count: 1, token_types_start: 2720, token_types_count: 0 }, // [1212]
    SimpleHintData { raw_values_start: 2720, raw_values_count: 1, token_types_start: 2721, token_types_count: 0 }, // [1213]
    SimpleHintData { raw_values_start: 2721, raw_values_count: 1, token_types_start: 2722, token_types_count: 0 }, // [1214]
    SimpleHintData { raw_values_start: 2722, raw_values_count: 1, token_types_start: 2723, token_types_count: 0 }, // [1215]
    SimpleHintData { raw_values_start: 2723, raw_values_count: 6, token_types_start: 2729, token_types_count: 0 }, // [1216]
    SimpleHintData { raw_values_start: 2729, raw_values_count: 5, token_types_start: 2734, token_types_count: 0 }, // [1217]
    SimpleHintData { raw_values_start: 2734, raw_values_count: 11, token_types_start: 2745, token_types_count: 0 }, // [1218]
    SimpleHintData { raw_values_start: 2745, raw_values_count: 1, token_types_start: 2746, token_types_count: 0 }, // [1219]
    SimpleHintData { raw_values_start: 2746, raw_values_count: 1, token_types_start: 2747, token_types_count: 0 }, // [1220]
    SimpleHintData { raw_values_start: 2747, raw_values_count: 1, token_types_start: 2748, token_types_count: 0 }, // [1221]
    SimpleHintData { raw_values_start: 2748, raw_values_count: 1, token_types_start: 2749, token_types_count: 0 }, // [1222]
    SimpleHintData { raw_values_start: 2749, raw_values_count: 1, token_types_start: 2750, token_types_count: 0 }, // [1223]
    SimpleHintData { raw_values_start: 2750, raw_values_count: 1, token_types_start: 2751, token_types_count: 0 }, // [1224]
    SimpleHintData { raw_values_start: 2751, raw_values_count: 1, token_types_start: 2752, token_types_count: 0 }, // [1225]
    SimpleHintData { raw_values_start: 2752, raw_values_count: 36, token_types_start: 2788, token_types_count: 0 }, // [1226]
    SimpleHintData { raw_values_start: 2788, raw_values_count: 39, token_types_start: 2827, token_types_count: 1 }, // [1227]
    SimpleHintData { raw_values_start: 2828, raw_values_count: 1, token_types_start: 2829, token_types_count: 1 }, // [1228]
    SimpleHintData { raw_values_start: 2830, raw_values_count: 2, token_types_start: 2832, token_types_count: 0 }, // [1229]
    SimpleHintData { raw_values_start: 2832, raw_values_count: 10, token_types_start: 2842, token_types_count: 0 }, // [1230]
    SimpleHintData { raw_values_start: 2842, raw_values_count: 1, token_types_start: 2843, token_types_count: 0 }, // [1231]
    SimpleHintData { raw_values_start: 2843, raw_values_count: 1, token_types_start: 2844, token_types_count: 0 }, // [1232]
    SimpleHintData { raw_values_start: 2844, raw_values_count: 7, token_types_start: 2851, token_types_count: 2 }, // [1233]
    SimpleHintData { raw_values_start: 2853, raw_values_count: 2, token_types_start: 2855, token_types_count: 0 }, // [1234]
    SimpleHintData { raw_values_start: 2855, raw_values_count: 5, token_types_start: 2860, token_types_count: 2 }, // [1235]
    SimpleHintData { raw_values_start: 2862, raw_values_count: 0, token_types_start: 2862, token_types_count: 2 }, // [1236]
    SimpleHintData { raw_values_start: 2864, raw_values_count: 2, token_types_start: 2866, token_types_count: 0 }, // [1237]
    SimpleHintData { raw_values_start: 2866, raw_values_count: 2, token_types_start: 2868, token_types_count: 0 }, // [1238]
    SimpleHintData { raw_values_start: 2868, raw_values_count: 1, token_types_start: 2869, token_types_count: 0 }, // [1239]
    SimpleHintData { raw_values_start: 2869, raw_values_count: 1, token_types_start: 2870, token_types_count: 0 }, // [1240]
    SimpleHintData { raw_values_start: 2870, raw_values_count: 1, token_types_start: 2871, token_types_count: 0 }, // [1241]
    SimpleHintData { raw_values_start: 2871, raw_values_count: 1, token_types_start: 2872, token_types_count: 0 }, // [1242]
    SimpleHintData { raw_values_start: 2872, raw_values_count: 1, token_types_start: 2873, token_types_count: 0 }, // [1243]
    SimpleHintData { raw_values_start: 2873, raw_values_count: 1, token_types_start: 2874, token_types_count: 0 }, // [1244]
    SimpleHintData { raw_values_start: 2874, raw_values_count: 1, token_types_start: 2875, token_types_count: 0 }, // [1245]
    SimpleHintData { raw_values_start: 2875, raw_values_count: 1, token_types_start: 2876, token_types_count: 0 }, // [1246]
    SimpleHintData { raw_values_start: 2876, raw_values_count: 1, token_types_start: 2877, token_types_count: 0 }, // [1247]
    SimpleHintData { raw_values_start: 2877, raw_values_count: 1, token_types_start: 2878, token_types_count: 0 }, // [1248]
    SimpleHintData { raw_values_start: 2878, raw_values_count: 1, token_types_start: 2879, token_types_count: 0 }, // [1249]
    SimpleHintData { raw_values_start: 2879, raw_values_count: 1, token_types_start: 2880, token_types_count: 0 }, // [1250]
    SimpleHintData { raw_values_start: 2880, raw_values_count: 1, token_types_start: 2881, token_types_count: 0 }, // [1251]
    SimpleHintData { raw_values_start: 2881, raw_values_count: 1, token_types_start: 2882, token_types_count: 0 }, // [1252]
    SimpleHintData { raw_values_start: 2882, raw_values_count: 1, token_types_start: 2883, token_types_count: 0 }, // [1253]
    SimpleHintData { raw_values_start: 2883, raw_values_count: 1, token_types_start: 2884, token_types_count: 0 }, // [1254]
    SimpleHintData { raw_values_start: 2884, raw_values_count: 1, token_types_start: 2885, token_types_count: 0 }, // [1255]
];

pub static SIMPLE_HINT_INDICES: &[u32] = &[
    1, 2, 3, 4, 0, 5, 0, 6, 7, 7, 8, 9, 10, 11, 12, 13,
    14, 15, 16, 17, 18, 19, 19, 20, 21, 22, 22, 23, 24, 24, 25, 26,
    10, 27, 27, 25, 26, 10, 28, 24, 24, 29, 26, 14, 27, 27, 29, 26,
    30, 10, 14, 0, 0, 0, 0, 0, 31, 0, 32, 33, 34, 35, 35, 36,
    37, 38, 39, 39, 40, 16, 41, 9, 10, 12, 42, 42, 43, 44, 45, 45,
    46, 47, 47, 48, 8, 8, 0, 0, 49, 50, 50, 39, 39, 51, 14, 43,
    43, 44, 52, 28, 53, 53, 54, 14, 15, 55, 55, 56, 42, 57, 39, 58,
    45, 59, 60, 8, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 49,
    72, 73, 24, 24, 56, 31, 38, 38, 74, 28, 75, 76, 77, 78, 79, 80,
    81, 82, 46, 14, 43, 43, 44, 83, 83, 76, 84, 84, 75, 85, 85, 86,
    29, 87, 88, 89, 90, 91, 92, 93, 94, 95, 25, 43, 43, 0, 83, 83,
    88, 84, 84, 87, 85, 85, 0, 96, 97, 98, 0, 99, 99, 0, 0, 41,
    41, 0, 0, 16, 16, 0, 0, 100, 100, 0, 0, 0, 0, 0, 0, 101,
    102, 103, 104, 5, 105, 106, 107, 108, 108, 109, 108, 110, 111, 112, 113, 0,
    0, 114, 0, 0, 0, 31, 0, 31, 115, 116, 0, 117, 0, 0, 0, 24,
    118, 119, 120, 120, 112, 0, 31, 0, 31, 0, 121, 122, 123, 123, 33, 0,
    124, 124, 33, 0, 125, 126, 127, 0, 128, 128, 14, 0, 120, 120, 10, 0,
    129, 130, 131, 132, 118, 118, 133, 134, 134, 135, 37, 136, 136, 133, 123, 123,
    33, 0, 124, 124, 33, 0, 0, 125, 126, 127, 128, 128, 17, 0, 128, 128,
    0, 137, 138, 138, 33, 139, 0, 140, 141, 0, 142, 34, 34, 143, 144, 144,
    0, 24, 0, 97, 97, 145, 74, 146, 33, 0, 0, 131, 131, 131, 97, 147,
    145, 24, 24, 56, 148, 148, 39, 58, 45, 59, 65, 66, 69, 70, 71, 49,
    74, 98, 98, 97, 97, 147, 149, 145, 74, 146, 34, 0, 0, 150, 151, 152,
    146, 87, 90, 92, 93, 153, 25, 120, 120, 140, 56, 149, 149, 154, 16, 41,
    0, 0, 26, 97, 98, 26, 26, 14, 0, 0, 97, 98, 96, 97, 98, 0,
    0, 99, 0, 101, 120, 120, 11, 0, 155, 128, 128, 140, 0, 156, 156, 140,
    157, 157, 128, 156, 158, 159, 107, 107, 160, 160, 0, 161, 162, 158, 159, 163,
    163, 0, 158, 158, 164, 156, 156, 160, 165, 0, 152, 150, 151, 123, 123, 160,
    0, 33, 0, 166, 166, 160, 0, 124, 124, 33, 0, 0, 125, 126, 127, 123,
    123, 33, 0, 128, 128, 14, 0, 120, 120, 167, 0, 168, 71, 71, 33, 33,
    141, 128, 128, 14, 0, 0, 169, 170, 170, 107, 156, 171, 172, 172, 4, 4,
    173, 174, 175, 11, 176, 176, 69, 19, 19, 22, 23, 19, 19, 75, 177, 178,
    13, 14, 81, 21, 179, 179, 180, 181, 182, 183, 184, 185, 18, 0, 186, 186,
    83, 83, 76, 75, 76, 0, 112, 112, 0, 31, 0, 0, 187, 187, 31, 0,
    0, 114, 0, 188, 188, 78, 79, 80, 0, 0, 0, 31, 189, 189, 0, 0,
    0, 31, 0, 0, 0, 32, 0, 190, 190, 191, 192, 0, 193, 0, 194, 194,
    149, 18, 0, 195, 107, 107, 196, 165, 0, 0, 197, 197, 0, 198, 198, 156,
    156, 196, 165, 0, 152, 120, 120, 196, 0, 199, 200, 128, 128, 22, 18, 0,
    197, 197, 0, 193, 193, 0, 0, 128, 128, 140, 0, 0, 141, 0, 201, 0,
    156, 156, 140, 202, 157, 128, 156, 158, 159, 156, 156, 203, 165, 107, 107, 204,
    205, 206, 207, 207, 140, 114, 208, 31, 209, 209, 210, 210, 128, 128, 204, 205,
    206, 207, 207, 140, 128, 128, 209, 211, 211, 131, 131, 37, 156, 156, 208, 165,
    128, 128, 212, 37, 128, 128, 31, 144, 144, 31, 128, 128, 213, 214, 215, 84,
    216, 217, 128, 128, 218, 0, 107, 107, 219, 107, 107, 220, 120, 120, 160, 0,
    221, 222, 158, 158, 222, 223, 223, 223, 224, 223, 223, 225, 166, 166, 160, 0,
    156, 156, 160, 165, 0, 152, 226, 226, 227, 228, 69, 0, 0, 24, 41, 227,
    227, 229, 230, 206, 69, 0, 226, 226, 227, 228, 227, 227, 230, 227, 227, 231,
    231, 0, 232, 232, 228, 227, 233, 234, 234, 233, 235, 236, 237, 238, 238, 74,
    0, 128, 128, 239, 240, 238, 241, 128, 128, 17, 0, 128, 128, 242, 243, 244,
    128, 128, 31, 144, 144, 31, 245, 234, 246, 0, 247, 247, 0, 158, 158, 247,
    124, 124, 33, 0, 0, 125, 126, 127, 230, 230, 208, 248, 140, 193, 193, 249,
    0, 250, 251, 252, 120, 120, 0, 0, 253, 31, 0, 0, 254, 107, 128, 156,
    0, 255, 120, 120, 19, 75, 0, 165, 256, 0, 121, 257, 195, 195, 123, 123,
    196, 0, 33, 0, 123, 123, 33, 0, 258, 258, 259, 260, 260, 74, 159, 261,
    262, 262, 159, 74, 159, 261, 263, 264, 265, 266, 267, 268, 158, 269, 270, 84,
    237, 271, 272, 273, 274, 274, 275, 276, 277, 278, 279, 279, 37, 280, 280, 37,
    281, 281, 0, 128, 128, 0, 137, 138, 138, 33, 139, 0, 141, 0, 140, 34,
    34, 143, 144, 144, 0, 24, 0, 150, 120, 120, 78, 0, 0, 0, 31, 282,
    258, 123, 123, 33, 0, 128, 128, 14, 0, 124, 124, 33, 0, 0, 0, 142,
    125, 126, 127, 283, 234, 284, 74, 167, 0, 120, 120, 249, 0, 0, 165, 0,
    285, 123, 123, 33, 0, 128, 128, 17, 0, 286, 286, 287, 0, 283, 234, 284,
    74, 167, 0, 128, 128, 31, 0, 0, 0, 139, 139, 141, 144, 144, 31, 0,
    0, 120, 120, 196, 37, 128, 212, 37, 24, 24, 26, 17, 0, 288, 288, 207,
    0, 0, 128, 17, 0, 289, 120, 290, 120, 120, 196, 0, 291, 128, 128, 212,
    37, 128, 128, 31, 0, 0, 0, 139, 141, 144, 144, 31, 0, 0, 128, 128,
    213, 214, 215, 84, 216, 217, 128, 128, 218, 0, 238, 238, 74, 0, 128, 128,
    239, 238, 128, 128, 31, 0, 0, 0, 139, 139, 141, 144, 144, 31, 0, 0,
    124, 124, 33, 0, 0, 125, 126, 127, 120, 120, 83, 76, 0, 0, 165, 0,
    292, 290, 290, 123, 123, 196, 196, 0, 33, 0, 123, 123, 33, 0, 128, 128,
    14, 0, 0, 0, 234, 284, 74, 167, 0, 24, 24, 26, 17, 0, 288, 288,
    207, 0, 0, 128, 17, 0, 289, 289, 120, 120, 44, 0, 74, 0, 293, 123,
    123, 33, 0, 294, 33, 33, 0, 0, 0, 101, 125, 126, 127, 193, 193, 31,
    0, 131, 131, 163, 31, 0, 295, 295, 270, 84, 237, 271, 272, 273, 128, 128,
    0, 137, 138, 138, 33, 139, 0, 141, 0, 140, 34, 34, 143, 144, 144, 0,
    24, 0, 150, 120, 120, 79, 0, 0, 0, 31, 296, 295, 123, 123, 33, 0,
    128, 128, 14, 0, 128, 128, 0, 137, 138, 138, 33, 139, 0, 0, 0, 141,
    34, 34, 143, 124, 124, 33, 0, 0, 0, 142, 125, 126, 127, 283, 234, 284,
    74, 167, 0, 120, 120, 297, 0, 298, 128, 128, 31, 107, 107, 299, 299, 128,
    128, 299, 299, 156, 156, 299, 299, 123, 123, 33, 0, 124, 124, 33, 0, 125,
    126, 127, 0, 120, 120, 154, 16, 41, 0, 0, 0, 125, 126, 127, 0, 131,
    300, 301, 302, 303, 304, 305, 306, 307, 308, 309, 310, 246, 311, 312, 313, 314,
    315, 316, 317, 318, 319, 320, 134, 134, 135, 37, 321, 322, 323, 324, 255, 159,
    164, 164, 325, 255, 0, 0, 123, 123, 33, 0, 0, 0, 125, 126, 127, 24,
    0, 26, 26, 10, 0, 291, 128, 128, 0, 137, 138, 138, 33, 139, 0, 140,
    0, 0, 141, 0, 74, 34, 34, 143, 144, 144, 0, 0, 24, 120, 120, 14,
    0, 326, 123, 123, 33, 0, 124, 124, 33, 0, 209, 114, 114, 0, 327, 327,
    207, 328, 37, 329, 330, 330, 328, 37, 234, 234, 330, 331, 332, 332, 328, 37,
    234, 234, 332, 81, 81, 333, 0, 334, 334, 131, 328, 37, 211, 211, 131, 328,
    37, 335, 335, 37, 336, 234, 337, 288, 288, 207, 0, 338, 0, 120, 120, 81,
    165, 0, 339, 209, 209, 124, 124, 33, 0, 0, 126, 127, 123, 123, 33, 0,
    128, 128, 14, 0, 120, 120, 212, 165, 0, 122, 124, 124, 33, 0, 0, 0,
    142, 125, 126, 127, 123, 123, 33, 0, 128, 128, 340, 14, 14, 0, 212, 212,
    37, 120, 120, 341, 0, 342, 134, 134, 255, 254, 254, 128, 107, 156, 297, 0,
    0, 131, 131, 31, 343, 343, 297, 131, 131, 31, 227, 228, 128, 128, 31, 344,
    344, 31, 0, 139, 0, 124, 124, 33, 0, 0, 125, 126, 0, 123, 123, 33,
    0, 120, 120, 345, 291, 128, 128, 0, 138, 33, 139, 0, 140, 0, 0, 141,
    0, 142, 74, 144, 144, 0, 24, 0, 346, 107, 107, 196, 165, 0, 0, 197,
    197, 0, 198, 198, 156, 156, 196, 165, 0, 152, 120, 120, 196, 0, 199, 200,
    128, 128, 22, 18, 0, 197, 197, 0, 193, 193, 0, 0, 128, 128, 140, 0,
    0, 141, 0, 201, 0, 156, 156, 140, 202, 157, 128, 156, 158, 159, 156, 156,
    203, 165, 107, 107, 204, 205, 206, 207, 207, 140, 114, 208, 31, 209, 209, 210,
    210, 128, 128, 204, 205, 206, 207, 207, 140, 128, 128, 209, 211, 211, 131, 131,
    37, 156, 156, 208, 165, 128, 128, 212, 37, 128, 128, 31, 144, 144, 31, 128,
    128, 213, 214, 215, 84, 216, 217, 128, 128, 218, 0, 107, 107, 219, 107, 107,
    220, 120, 120, 160, 0, 221, 222, 158, 158, 222, 223, 223, 223, 224, 223, 223,
    225, 166, 166, 160, 0, 156, 156, 160, 165, 0, 152, 226, 226, 227, 228, 69,
    0, 0, 24, 41, 227, 227, 229, 230, 206, 69, 0, 226, 226, 227, 228, 227,
    227, 230, 227, 227, 231, 231, 0, 232, 232, 228, 227, 233, 234, 234, 233, 235,
    236, 237, 238, 238, 74, 0, 128, 128, 239, 240, 238, 241, 128, 128, 17, 0,
    128, 128, 242, 243, 244, 128, 128, 31, 144, 144, 31, 245, 234, 246, 0, 247,
    247, 0, 158, 158, 247, 124, 124, 33, 0, 0, 125, 126, 127, 230, 230, 208,
    248, 140, 193, 193, 249, 0, 250, 251, 156, 156, 196, 165, 0, 0, 0, 0,
    139, 0, 141, 0, 347, 347, 107, 60, 196, 0, 348, 349, 349, 350, 111, 0,
    31, 156, 123, 123, 351, 114, 33, 0, 120, 120, 75, 0, 0, 165, 256, 0,
    121, 352, 346, 346, 123, 123, 196, 0, 33, 0, 123, 123, 160, 0, 33, 0,
    0, 165, 0, 353, 123, 123, 33, 0, 128, 128, 14, 0, 286, 286, 287, 0,
    354, 149, 149, 355, 356, 140, 357, 357, 287, 0, 358, 359, 24, 24, 26, 17,
    0, 288, 288, 207, 0, 0, 128, 17, 0, 289, 120, 120, 179, 180, 182, 0,
    360, 107, 107, 361, 149, 0, 0, 131, 0, 0, 120, 120, 361, 149, 0, 0,
    131, 0, 0, 120, 120, 361, 362, 0, 131, 0, 120, 120, 361, 149, 0, 0,
    362, 0, 131, 0, 156, 156, 361, 165, 149, 0, 0, 123, 123, 33, 0, 124,
    124, 33, 0, 0, 125, 126, 127, 128, 128, 14, 0, 120, 120, 69, 0, 74,
    0, 363, 123, 123, 33, 0, 283, 234, 284, 74, 167, 0, 120, 120, 18, 0,
    360, 124, 124, 33, 0, 126, 127, 125, 0, 123, 123, 364, 255, 33, 255, 123,
    123, 33, 0, 128, 128, 14, 0, 365, 365, 107, 107, 366, 0, 0, 197, 197,
    0, 152, 120, 120, 366, 0, 128, 128, 22, 18, 0, 197, 197, 0, 152, 156,
    156, 366, 165, 0, 152, 123, 123, 366, 0, 33, 0, 152, 107, 107, 364, 165,
    255, 367, 367, 368, 111, 255, 120, 120, 76, 165, 0, 369, 120, 120, 196, 0,
    157, 128, 128, 140, 0, 141, 0, 201, 0, 156, 156, 140, 124, 124, 33, 0,
    0, 125, 126, 127, 123, 123, 196, 0, 33, 0, 123, 123, 33, 0, 128, 128,
    14, 0, 128, 128, 31, 0, 0, 0, 139, 139, 141, 144, 144, 31, 0, 0,
    206, 370, 371, 372, 373, 373, 372, 371, 374, 31, 375, 375, 375, 376, 377, 133,
    376, 0, 0, 0, 31, 0, 0, 378, 378, 379, 380, 381, 42, 382, 383, 108,
    384, 121, 385, 370, 109, 386, 387, 388, 389, 103, 0, 328, 37, 0, 0, 0,
    328, 37, 0, 390, 0, 328, 37, 0, 103, 391, 0, 392, 103, 0, 0, 392,
    103, 0, 0, 114, 0, 114, 114, 114, 393, 394, 395, 396, 397, 398, 399, 286,
    366, 400, 401, 402, 402, 402, 403, 404, 405, 201, 0, 406, 0, 0, 141, 201,
    407, 0, 0, 0, 0, 0, 141, 32, 114, 31, 408, 0, 0, 368, 409, 410,
    411, 412, 413, 414, 415, 382, 416, 417, 418, 419, 0, 420, 421, 422, 370, 387,
    387, 387, 109, 388, 388, 388, 386, 423, 417, 378, 424, 425, 133, 426, 427, 428,
    429, 430, 31, 141, 141, 31, 0, 0, 0, 31, 431, 432, 433, 434, 435, 436,
    207, 319, 437, 31, 0, 0, 0, 31, 114, 438, 158, 83, 31, 439, 440, 180,
    180, 441, 432, 442, 350, 207, 0, 128, 0, 337, 337, 0, 128, 0, 193, 0,
    335, 443, 443, 443, 0, 260, 151, 152, 151, 150, 444, 445, 445, 445, 0, 446,
    446, 447, 448, 447, 448, 448, 445, 445, 0, 446, 446, 447, 448, 447, 448, 448,
    418, 32, 415, 445, 187, 449, 15, 450, 451, 0, 452, 453, 454, 163, 455, 456,
    457, 458, 459, 460, 191, 461, 238, 238, 238, 376, 0, 0, 0, 193, 193, 0,
    0, 0, 74, 0, 462, 463, 0, 0, 197, 174, 0, 255, 0, 0, 104, 464,
    390, 390, 390, 0, 0, 141, 0, 201, 198, 198, 162, 158, 159, 163, 163, 31,
    0, 140, 140, 0, 141, 0, 465, 466, 466, 467, 65, 197, 197, 0, 0, 0,
    0, 198, 198, 197, 197, 0, 0, 408, 196, 0, 0, 104, 104, 104, 104, 104,
    104, 0, 0, 104, 104, 104, 104, 104, 74, 114, 193, 32, 449, 103, 31, 415,
    390, 468, 469, 0, 0, 385, 18, 31, 31, 0, 0, 0, 0, 470, 470, 470,
    31, 470, 471, 472, 472, 32, 473, 473, 255, 473, 473, 473, 74, 474, 474, 475,
    475, 75, 76, 0, 187, 187, 31, 0, 0, 114, 0, 196, 196, 0, 160, 160,
    0, 74, 74, 11, 0, 10, 10, 0, 167, 167, 0, 78, 78, 0, 31, 31,
    249, 249, 0, 14, 14, 0, 476, 476, 174, 175, 11, 13, 44, 297, 16, 231,
    81, 21, 212, 341, 17, 69, 18, 4, 4, 173, 176, 176, 69, 19, 19, 22,
    23, 19, 19, 75, 83, 83, 76, 179, 179, 180, 182, 179, 179, 180, 183, 179,
    179, 180, 184, 179, 179, 180, 185, 0, 74, 74, 0, 477, 477, 112, 79, 80,
    0, 31, 0, 0, 478, 478, 324, 255, 159, 0, 479, 480, 481, 482, 418, 139,
    388, 387, 388, 387, 483, 484, 478, 485, 486, 0, 487, 0, 0, 31, 31, 0,
    104, 121, 488, 489, 218, 490, 491, 109, 109, 109, 358, 0, 0, 492, 0, 493,
    182, 494, 494, 495, 251, 71, 71, 128, 0, 0, 0, 0, 139, 0, 0, 140,
    31, 31, 0, 0, 139, 235, 31, 0, 0, 0, 140, 31, 31, 0, 0, 139,
    31, 439, 496, 496, 0, 497, 498, 31, 31, 0, 0, 0, 0, 31, 0, 0,
    197, 197, 0, 0, 496, 496, 0, 74, 74, 160, 0, 57, 134, 160, 499, 500,
    501, 502, 175, 503, 503, 503, 0, 0, 0, 0, 0, 31, 0, 0, 31, 439,
    33, 255, 504, 131, 31, 505, 505, 86, 86, 0, 506, 506, 133, 507, 507, 133,
    381, 381, 133, 508, 508, 0, 31, 0, 0, 0, 509, 509, 133, 218, 218, 0,
    255, 0, 490, 490, 37, 510, 510, 37, 511, 511, 37, 512, 512, 255, 0, 34,
    255, 504, 131, 31, 513, 513, 86, 86, 0, 514, 514, 133, 515, 515, 255, 159,
    159, 255, 516, 516, 133, 517, 517, 255, 518, 518, 255, 519, 519, 520, 31, 0,
    0, 121, 521, 521, 31, 0, 0, 522, 522, 31, 0, 0, 523, 523, 255, 524,
    279, 525, 39, 39, 424, 112, 0, 31, 0, 31, 39, 39, 187, 31, 0, 114,
    0, 526, 131, 131, 78, 0, 31, 239, 239, 78, 131, 131, 527, 114, 114, 114,
    396, 114, 114, 528, 39, 39, 174, 165, 0, 529, 31, 530, 530, 531, 531, 139,
    255, 532, 532, 139, 255, 533, 533, 139, 255, 534, 534, 139, 535, 536, 537, 538,
    538, 139, 133, 539, 539, 139, 255, 34, 34, 0, 39, 39, 10, 0, 131, 540,
    124, 124, 139, 0, 185, 185, 139, 0, 523, 523, 139, 541, 255, 140, 542, 531,
    531, 139, 255, 543, 532, 532, 139, 255, 533, 533, 139, 255, 17, 17, 139, 0,
    0, 140, 118, 118, 139, 133, 134, 134, 135, 139, 37, 136, 136, 139, 133, 39,
    39, 11, 0, 114, 114, 0, 197, 197, 0, 140, 140, 0, 544, 544, 160, 160,
    0, 161, 162, 158, 159, 163, 163, 0, 39, 39, 167, 165, 0, 131, 14, 14,
    0, 539, 539, 0, 34, 34, 0, 151, 39, 39, 19, 0, 545, 0, 546, 0,
    39, 39, 19, 75, 39, 39, 39, 165, 0, 31, 0, 0, 0, 0, 0, 253,
    197, 197, 0, 547, 547, 548, 549, 549, 31, 0, 0, 21, 21, 0, 253, 39,
    39, 165, 0, 287, 287, 247, 0, 31, 0, 0, 0, 0, 131, 131, 253, 547,
    547, 548, 354, 149, 149, 355, 356, 140, 21, 21, 0, 253, 39, 39, 424, 550,
    551, 552, 78, 0, 31, 114, 0, 75, 75, 439, 0, 39, 39, 465, 249, 358,
    0, 165, 0, 74, 256, 0, 193, 193, 0, 31, 0, 0, 553, 553, 31, 0,
    0, 554, 554, 158, 555, 131, 131, 31, 17, 17, 0, 496, 496, 0, 39, 39,
    39, 424, 83, 76, 165, 0, 31, 193, 193, 0, 131, 131, 31, 17, 17, 0,
    114, 556, 439, 31, 439, 557, 31, 75, 75, 0, 355, 558, 31, 45, 45, 0,
    131, 39, 39, 424, 559, 165, 0, 253, 253, 31, 0, 0, 0, 139, 141, 141,
    103, 255, 255, 114, 439, 39, 39, 189, 0, 0, 31, 560, 560, 561, 561, 562,
    563, 139, 0, 482, 482, 139, 0, 0, 564, 564, 139, 0, 0, 565, 565, 150,
    566, 567, 79, 78, 139, 0, 568, 569, 39, 39, 44, 0, 74, 0, 114, 114,
    570, 571, 572, 149, 149, 573, 24, 66, 59, 71, 58, 33, 33, 0, 0, 0,
    101, 125, 126, 127, 193, 193, 31, 0, 131, 131, 163, 31, 0, 39, 39, 424,
    79, 0, 31, 574, 39, 39, 297, 0, 149, 149, 149, 24, 87, 149, 149, 299,
    299, 131, 131, 31, 39, 39, 154, 16, 41, 0, 575, 131, 576, 301, 302, 303,
    304, 305, 306, 307, 308, 309, 310, 246, 311, 312, 313, 314, 315, 316, 317, 318,
    319, 320, 134, 134, 135, 37, 321, 322, 323, 324, 255, 159, 164, 164, 325, 255,
    26, 26, 16, 0, 26, 26, 99, 0, 16, 16, 0, 110, 110, 0, 41, 41,
    0, 577, 577, 37, 39, 39, 14, 165, 0, 0, 0, 401, 0, 0, 578, 114,
    114, 0, 327, 327, 207, 328, 37, 329, 330, 330, 328, 37, 234, 234, 330, 331,
    332, 332, 328, 37, 234, 234, 332, 334, 334, 131, 328, 37, 335, 335, 37, 336,
    337, 234, 234, 337, 288, 288, 207, 0, 338, 0, 39, 39, 550, 81, 165, 0,
    578, 578, 39, 39, 21, 165, 0, 18, 18, 255, 539, 539, 0, 19, 0, 253,
    39, 39, 212, 0, 165, 0, 31, 579, 579, 580, 581, 582, 524, 74, 0, 0,
    0, 0, 34, 0, 39, 39, 341, 0, 134, 255, 297, 0, 0, 131, 131, 31,
    0, 39, 39, 424, 550, 75, 165, 0, 583, 114, 114, 439, 31, 439, 31, 0,
    0, 0, 0, 0, 0, 0, 198, 198, 198, 0, 0, 114, 204, 204, 206, 114,
    31, 0, 584, 585, 586, 219, 39, 39, 179, 180, 182, 0, 31, 587, 184, 184,
    139, 0, 503, 503, 139, 0, 39, 39, 424, 160, 69, 0, 588, 368, 111, 589,
    589, 247, 590, 590, 59, 58, 70, 71, 71, 247, 247, 0, 0, 424, 74, 424,
    74, 0, 591, 34, 34, 0, 592, 158, 158, 222, 593, 222, 223, 223, 223, 225,
    223, 223, 224, 594, 594, 595, 596, 597, 75, 114, 0, 595, 595, 596, 597, 75,
    114, 0, 149, 149, 598, 599, 235, 600, 446, 446, 31, 0, 45, 45, 567, 78,
    79, 0, 39, 39, 18, 0, 114, 0, 0, 601, 601, 31, 255, 255, 602, 603,
    603, 31, 39, 39, 41, 361, 39, 39, 165, 149, 0, 101, 0, 604, 21, 0,
    253, 39, 39, 41, 0, 39, 39, 424, 550, 76, 165, 0, 31, 114, 439, 439,
    31, 439, 355, 305, 308, 605, 606, 607, 608, 608, 609, 609, 608, 610, 31, 0,
    143, 611, 612, 125, 169, 613, 614, 126, 615, 616, 337, 22, 10, 0, 0, 104,
    464, 617, 0, 0, 0, 0, 0, 0, 0, 104, 0, 618, 619, 602, 406, 620,
    620, 621, 622, 623, 624, 625, 626, 412, 627, 628, 629, 630, 630, 631, 632, 633,
    634, 635, 636, 413, 637, 637, 637, 31, 638, 638, 639, 640, 31, 641, 642, 643,
    643, 430, 452, 452, 452, 644, 453, 453, 453, 644, 645, 31, 179, 437, 646, 425,
    423, 647, 648, 649, 650, 429, 651, 652, 459, 653, 458, 654, 655, 656, 657, 658,
    659, 421, 421, 644, 644, 31, 660, 661, 662, 663, 664, 665, 666, 667, 668, 669,
    670, 671, 672, 672, 31, 0, 0, 31, 673, 103, 103, 392, 392, 674, 675, 675,
    31, 31, 0, 0, 676, 0, 677, 619, 255, 619, 674, 678, 679, 679, 407, 680,
    681, 31, 37, 670, 682, 676, 683, 684, 685, 686, 687, 688, 689, 690, 691, 692,
    693, 693, 693, 694, 0, 0, 24, 695, 639, 696, 140, 140, 140, 355, 697, 222,
    224, 272, 698, 141, 699, 699, 699, 139, 139, 698, 31, 699, 699, 700, 701, 58,
    58, 58, 34, 256, 0, 121, 0, 193, 193, 0, 0, 0, 0, 0, 496, 496,
    496, 143, 247, 0, 496, 702, 702, 0, 121, 0, 0, 0, 0, 468, 515, 703,
    704, 580, 284, 442, 705, 706, 706, 706, 0, 0, 707, 708, 357, 538, 183, 228,
    709, 710, 710, 710, 711, 24, 712, 93, 68, 67, 713, 555, 714, 715, 716, 384,
    717, 718, 494, 494, 494, 719, 719, 13, 255, 255, 255, 13, 720, 721, 11, 104,
    630, 156, 156, 112, 165, 0, 0, 0, 0, 31, 0, 121, 152, 152, 150, 151,
    156, 156, 187, 165, 31, 0, 114, 0, 152, 156, 156, 174, 165, 0, 152, 156,
    156, 10, 165, 0, 504, 131, 31, 233, 156, 156, 11, 165, 0, 0, 152, 156,
    156, 167, 165, 0, 152, 156, 156, 19, 75, 165, 0, 0, 152, 156, 156, 551,
    552, 78, 75, 165, 0, 152, 156, 156, 249, 358, 165, 0, 0, 152, 156, 156,
    156, 83, 76, 165, 0, 0, 152, 156, 156, 559, 165, 0, 156, 156, 288, 207,
    0, 0, 125, 126, 127, 0, 152, 156, 156, 44, 165, 0, 74, 0, 152, 156,
    156, 79, 165, 0, 0, 0, 31, 152, 151, 150, 156, 156, 297, 165, 0, 0,
    152, 156, 156, 16, 165, 0, 156, 156, 14, 165, 0, 152, 156, 156, 81, 165,
    0, 0, 152, 156, 156, 212, 165, 0, 0, 152, 151, 150, 156, 156, 341, 165,
    0, 152, 151, 150, 156, 156, 550, 75, 165, 0, 0, 152, 156, 156, 179, 180,
    182, 165, 0, 152, 156, 156, 69, 165, 0, 74, 0, 152, 156, 156, 18, 165,
    0, 0, 152, 156, 156, 41, 165, 0, 156, 156, 76, 165, 0, 0, 152, 722,
    723, 598, 447, 447, 0, 447, 724, 725, 31, 602, 602, 31, 227, 726, 523, 322,
    727, 448, 601, 139, 485, 728, 518, 729, 176, 730, 731, 732, 0, 0, 131, 418,
    733, 45, 45, 45, 0, 31, 0, 0, 734, 735, 736, 737, 738, 739, 739, 372,
    371, 376, 525, 740, 435, 741, 742, 743, 133, 86, 86, 744, 179, 661, 663, 745,
    737, 737, 746, 747, 373, 372, 371, 376, 31, 738, 738, 439, 439, 59, 71, 58,
    203, 0, 0, 0, 0, 0, 748, 415, 415, 0, 749, 418, 478, 478, 158, 750,
    751, 752, 0, 753, 158, 411, 0, 378, 0, 0, 0, 0, 754, 328, 328, 328,
    755, 158, 756, 3, 0, 0, 757, 757, 757, 757, 382, 416, 418, 0, 758, 328,
    328, 328, 755, 0, 0, 759, 0, 0, 735, 735, 31, 439, 0, 0, 0, 445,
    398, 398, 0, 32, 0, 760, 0, 761, 483, 0, 0, 201, 0, 31, 0, 0,
    0, 439, 0, 0, 141, 0, 0, 0, 0, 0, 0, 762, 762, 602, 406, 31,
    0, 0, 0, 0, 763, 255, 37, 133, 159, 677, 0, 103, 32, 102, 0, 66,
    141, 407, 602, 0, 0, 0, 216, 0, 0, 0, 256, 256, 0, 31, 0, 0,
    0, 121, 167, 84, 764, 761, 427, 427, 192, 765, 765, 766, 350, 767, 0, 37,
    0, 235, 768, 235, 280, 769, 256, 131, 131, 770, 765, 771, 85, 0, 772, 773,
    774, 0, 0, 468, 468, 775, 776, 776, 31, 0, 496, 0, 776, 359, 350, 627,
    628, 637, 777, 778, 149, 149, 779, 71, 234, 234, 467, 71, 100, 467, 467, 100,
    247, 247, 0, 0, 780, 289, 344, 344, 781, 149, 233, 521, 522, 519, 19, 19,
    22, 23, 19, 19, 467, 19, 547, 160, 160, 0, 782, 162, 158, 159, 163, 163,
    31, 0, 234, 234, 246, 140, 140, 0, 0, 141, 0, 201, 0, 204, 204, 206,
    114, 0, 585, 548, 160, 160, 0, 163, 163, 31, 0, 234, 234, 246, 86, 783,
    784, 784, 0, 0, 143, 143, 235, 0, 0, 37, 407, 785, 0, 786, 787, 778,
    411, 411, 378, 731, 731, 788, 143, 143, 235, 99, 770, 234, 234, 789, 784, 790,
    280, 791, 792, 793, 514, 34, 34, 0, 0, 794, 496, 135, 99, 99, 207, 113,
    113, 207, 795, 796, 797, 798, 131, 131, 765, 799, 149, 0, 0, 800, 0, 31,
    0, 0, 0, 0, 794, 801, 469, 0, 131, 131, 799, 0, 801, 0, 31, 0,
    0, 0, 0, 31, 0, 0, 802, 0, 31, 0, 113, 113, 207, 99, 99, 207,
    0, 0, 0, 0, 0, 469, 113, 113, 207, 99, 99, 207, 0, 34, 803, 803,
    31, 0, 0, 0, 0, 149, 0, 0, 0, 26, 31, 141, 141, 99, 113, 135,
    804, 804, 553, 553, 554, 31, 0, 0, 31, 0, 0, 149, 0, 0, 0, 26,
    31, 0, 0, 0, 0, 31, 0, 0, 0, 470, 250, 760, 805, 0, 0, 0,
    0, 0, 0, 431, 31, 431, 0, 0, 114, 0, 806, 806, 0, 34, 34, 0,
    0, 0, 676, 0, 807, 34, 34, 0, 149, 149, 0, 702, 702, 0, 0, 555,
    0, 121, 0, 0, 113, 808, 808, 141, 0, 0, 255, 0, 0, 0, 26, 0,
    255, 0, 0, 0, 809, 0, 602, 0, 0, 0, 810, 364, 390, 0, 31, 31,
    0, 574, 811, 13, 194, 194, 149, 18, 0, 797, 264, 265, 266, 267, 268, 158,
    269, 259, 260, 260, 74, 159, 261, 262, 262, 159, 74, 159, 261, 263, 270, 84,
    237, 271, 273, 272, 274, 274, 275, 276, 277, 278, 279, 279, 37, 280, 280, 37,
    281, 281, 0, 128, 128, 0, 137, 138, 138, 33, 139, 0, 0, 0, 141, 34,
    34, 143, 114, 114, 255, 255, 255, 255, 32, 255, 812, 812, 0, 409, 409, 399,
    813, 59, 59, 468, 71, 71, 468, 814, 814, 131, 431, 66, 468, 812, 812, 0,
    468, 448, 131, 131, 31, 0, 0, 78, 0, 0, 0, 0, 0, 104, 31, 0,
    0, 142, 31, 0, 815, 26, 816, 527, 817, 0, 0, 0, 0, 815, 26, 816,
    527, 817, 0, 0, 0, 818, 818, 140, 139, 759, 0, 31, 0, 0, 0, 675,
    675, 31, 470, 0, 0, 0, 819, 675, 470, 355, 31, 0, 90, 27, 204, 820,
    821, 822, 823, 824, 825, 826, 486, 827, 97, 97, 97, 0, 38, 38, 74, 0,
    16, 16, 0, 62, 62, 74, 41, 0, 0, 33, 0, 828, 131, 131, 97, 147,
    131, 131, 110, 147, 503, 503, 143, 48, 829, 829, 207, 0, 126, 127, 0, 829,
    48, 388, 388, 139, 388, 830, 99, 99, 207, 0, 0, 24, 0, 37, 0, 831,
    831, 113, 113, 207, 135, 795, 796, 797, 765, 99, 0, 0, 0, 0, 37, 0,
    31, 831, 832, 832, 832, 833, 31, 0, 0, 609, 0, 791, 545, 834, 568, 795,
    795, 0, 31, 0, 835, 113, 113, 207, 135, 796, 797, 765, 795, 516, 836, 837,
    838, 839, 840, 841, 842, 843, 844, 845, 846, 536, 847, 0, 208, 848, 849, 165,
    165, 735, 165, 165, 165, 158, 735, 850, 809, 809, 850, 851, 554, 852, 225, 265,
    528, 853, 853, 19, 14, 853, 853, 0, 854, 854, 135, 135, 33, 730, 31, 699,
    699, 34, 21, 0, 855, 0, 253, 853, 55, 26, 749, 158, 26, 0, 31, 0,
    0, 0, 439, 0, 103, 406, 255, 0, 553, 856, 327, 0, 0, 0, 857, 393,
    705, 0, 197, 197, 0, 0, 0, 554, 554, 858, 350, 859, 31, 857, 393, 705,
    554, 554, 858, 350, 859, 0, 0, 0, 0, 31, 0, 0, 249, 860, 553, 553,
    31, 131, 131, 31, 193, 193, 249, 17, 0, 861, 862, 654, 863, 864, 246, 549,
    865, 223, 866, 867, 527, 261, 868, 59, 59, 59, 424, 424, 424, 855, 0, 0,
    869, 31, 207, 207, 870, 207, 207, 333, 871, 140, 140, 355, 439, 439, 31, 439,
    355, 439, 31, 439, 74, 74, 497, 498, 494, 702, 702, 0, 121, 0, 0, 0,
    0, 872, 589, 873, 624, 874, 625, 665, 626, 666, 623, 622, 12, 875, 876, 407,
    407, 0, 37, 37, 877, 255, 676, 255, 255, 255, 877, 255, 676, 0, 0, 676,
    407, 878, 879, 880, 683, 843, 881, 882, 855, 855, 883, 68, 67, 244, 75, 0,
    855, 273, 750, 159, 884, 885, 133, 886, 663, 478, 478, 158, 555, 34, 478, 751,
    887, 136, 888, 751, 889, 890, 0, 0, 0, 566, 0, 0, 0, 0, 0, 0,
    891, 74, 193, 0, 892, 892, 566, 0, 0, 0, 0, 0, 566, 566, 566, 469,
    469, 803, 804, 0, 74, 74, 0, 0, 31, 0, 0, 492, 867, 893, 893, 250,
    894, 895, 896, 193, 193, 31, 0, 0, 663, 419, 663, 663, 897, 526, 897, 364,
    898, 392, 36, 526, 526, 131, 239, 465, 899, 899, 664, 467, 899, 0, 900, 901,
    902, 0, 0, 0, 0, 31, 0, 0, 902, 0, 903, 903, 0, 0, 31, 0,
    0, 390, 0, 903, 904, 13, 13, 0, 0, 116, 13, 35, 859, 905, 0, 800,
    532, 533, 906, 269, 907, 908, 894, 562, 909, 387, 387, 139, 387, 236, 537, 748,
    748, 158, 910, 0, 518, 518, 0, 910, 911, 852, 912, 912, 33, 826, 911, 484,
    913, 913, 856, 732, 914, 479, 499, 697, 204, 208, 862, 212, 213, 24, 135, 135,
    0, 31, 0, 37, 0, 24, 915, 799, 799, 0, 37, 0, 32, 32, 37, 135,
    649, 916, 917, 918, 919, 920, 921, 103, 0, 149, 0, 26, 0, 165, 165, 0,
    103, 922, 923, 923, 923, 0, 141, 255, 37, 133, 328, 159, 677, 721, 924, 103,
    925, 926, 0, 0, 927, 927, 927, 255, 0, 928, 531, 929, 930, 931, 932, 932,
    932, 75, 256, 0, 0, 0, 121, 26, 26, 933, 4, 4, 100, 235, 235, 100,
    235, 235, 733, 100, 100, 71, 733, 100, 100, 100, 235, 733, 733, 4, 4, 733,
    934, 289, 781, 935, 243, 313, 936, 937, 938, 939, 940, 941, 650, 802, 802, 104,
    0, 31, 559, 559, 0, 32, 32, 31, 439, 656, 655, 552, 217, 47, 406, 0,
    0, 390, 0, 406, 406, 926, 0, 0, 926, 31, 0, 0, 406, 406, 31, 361,
    43, 0, 942, 943, 83, 944, 945, 332, 581, 946, 947, 948, 58, 59, 59, 31,
    355, 949, 949, 855, 949, 446, 446, 446, 446, 446, 446, 943, 378, 378, 0, 950,
    951, 71, 58, 494, 494, 251, 446, 446, 158, 943, 207, 952, 378, 378, 0, 950,
    951, 71, 58, 494, 494, 251, 446, 446, 158, 943, 207, 207, 953, 378, 378, 0,
    950, 954, 59, 494, 494, 251, 949, 949, 0, 0, 0, 193, 0, 0, 0, 31,
    31, 439, 0, 0, 0, 0, 0, 74, 0, 446, 71, 71, 128, 569, 955, 439,
    439, 439, 439, 955, 173, 956, 957, 958, 959, 108, 881, 960, 961, 330, 802, 962,
    934, 559, 60, 385, 963, 641, 8, 880, 964, 965, 966, 967, 967, 967, 968, 969,
    970, 971, 121, 972, 973, 974, 975, 976, 977, 978, 979, 980, 981, 699, 0, 333,
    0, 0, 982, 983, 759, 139, 0, 0, 0, 114, 0, 0, 31, 0, 797, 797,
    0, 0, 984, 884, 884, 985, 986, 986, 492, 986, 987, 582, 108, 564, 0, 597,
    0, 767, 988, 989, 990, 991, 234, 992, 993, 320, 994, 995, 996, 306, 309, 997,
    998, 311, 314, 431, 355, 999, 31, 66, 31, 131, 31, 431, 31, 1000, 1001, 493,
    379, 1002, 394, 394, 1003, 492, 379, 1002, 675, 1004, 431, 1005, 31, 66, 431, 71,
    59, 58, 1006, 338, 1007, 317, 1008, 886, 1009, 988, 989, 990, 991, 1010, 1010, 303,
    0, 483, 1011, 1011, 139, 387, 387, 388, 370, 370, 387, 370, 370, 388, 158, 752,
    158, 251, 1012, 1012, 1012, 0, 32, 32, 255, 752, 289, 159, 159, 1013, 554, 640,
    37, 667, 36, 0, 0, 0, 255, 390, 0, 926, 0, 0, 104, 104, 104, 104,
    104, 464, 74, 114, 193, 32, 449, 103, 31, 415, 390, 468, 469, 1014, 247, 1015,
    1016, 799, 799, 0, 37, 0, 799, 1017, 241, 596, 74, 74, 1018, 256, 1019, 1020,
    1021, 1022, 61, 1023, 189, 1024, 147, 1025, 253, 253, 31, 1026, 1026, 253, 1026, 1026,
    1026, 699, 255, 424, 424, 850, 424, 424, 424, 424, 362, 113, 113, 207, 0, 0,
    0, 24, 0, 37, 0, 857, 393, 705, 554, 554, 858, 350, 859, 1027, 1027, 135,
    795, 796, 797, 784, 808, 131, 131, 22, 149, 113, 1028, 113, 1007, 1029, 789, 816,
    896, 1030, 0, 809, 1031, 0, 0, 31, 0, 1031, 370, 370, 370, 0, 0, 31,
    31, 0, 0, 677, 0, 0, 32, 0, 0, 677, 0, 0, 0, 1032, 0, 31,
    31, 0, 0, 677, 0, 0, 32, 0, 0, 677, 0, 0, 0, 1032, 1033, 1034,
    507, 506, 288, 124, 62, 1035, 274, 0, 1036, 1037, 512, 184, 1038, 356, 26, 26,
    31, 0, 0, 34, 34, 31, 0, 0, 0, 330, 332, 33, 31, 0, 0, 0,
    330, 332, 131, 131, 31, 963, 963, 37, 32, 1039, 37, 287, 287, 207, 0, 31,
    0, 0, 287, 508, 1040, 323, 651, 1041, 718, 718, 699, 699, 718, 255, 0, 1042,
    1043, 571, 1044, 671, 1045, 52, 109, 803, 1046, 1047, 1048, 215, 712, 383, 648, 1049,
    1050, 1051, 44, 652, 1052, 1053, 1054, 1055, 870, 840, 383, 0, 1056, 0, 776, 0,
    1057, 1058, 772, 772, 772, 389, 800, 800, 787, 631, 1059, 1060, 694, 694, 694, 0,
    31, 0, 0, 114, 1061, 439, 949, 1062, 1063, 466, 466, 467, 466, 1064, 756, 56,
    1065, 178, 79, 91, 1066, 1067, 1068, 1069, 699, 534, 773, 773, 773, 774, 774, 774,
    390, 924, 924, 31, 390, 390, 0, 0, 255, 101, 297, 299, 75, 75, 0, 0,
    32, 32, 299, 75, 87, 87, 87, 26, 14, 0, 0, 0, 169, 32, 32, 299,
    75, 87, 0, 0, 0, 31, 496, 496, 31, 0, 1070, 328, 328, 37, 189, 189,
    31, 0, 0, 104, 0, 0, 0, 0, 104, 104, 0, 796, 796, 0, 31, 0,
    796, 418, 418, 1071, 24, 380, 1072, 31, 439, 1073, 517, 142, 255, 1074, 1074, 116,
    1075, 1076, 967, 116, 1077, 1077, 0, 967, 0, 1078, 1078, 0, 967, 0, 1079, 1079,
    0, 1080, 1080, 1081, 1080, 967, 1081, 1082, 1083, 790, 139, 388, 1084, 387, 1011, 63,
    1085, 629, 1086, 1086, 1086, 288, 207, 0, 0, 125, 126, 127, 0, 33, 0, 125,
    126, 127, 0, 1087, 1088, 1089, 1090, 65, 65, 0, 31, 942, 74, 74, 74, 58,
    1091, 74, 74, 71, 1091, 942, 942, 1092, 250, 1038, 1093, 64, 65, 594, 1091, 151,
    128, 128, 1094, 140, 159, 31, 0, 0, 150, 234, 234, 106, 343, 343, 343, 83,
    76, 358, 0, 131, 1095, 1096, 1096, 1096, 31, 1097, 1097, 358, 358, 133, 17, 17,
    0, 376, 376, 133, 1098, 249, 249, 358, 0, 75, 75, 358, 0, 14, 14, 358,
    0, 1099, 1099, 10, 345, 358, 0, 699, 699, 104, 104, 699, 139, 139, 698, 31,
    699, 699, 1100, 1101, 1102, 1039, 123, 1103, 1104, 362, 230, 316, 1105, 1106, 144, 144,
    144, 1107, 401, 144, 144, 0, 24, 16, 0, 1108, 7, 851, 211, 150, 277, 572,
    1109, 812, 702, 262, 98, 98, 98, 97, 97, 147, 149, 0, 38, 38, 74, 0,
    16, 16, 0, 62, 62, 74, 41, 0, 0, 34, 0, 152, 983, 895, 563, 1110,
    16, 0, 1111, 1112, 610, 610, 80, 92, 235, 511, 510, 1113, 1114, 1115, 1116, 280,
    231, 278, 1117, 1117, 1106, 1106, 31, 37, 37, 37, 1118, 1118, 410, 345, 31, 37,
    37, 37, 31, 1119, 1106, 410, 345, 32, 32, 37, 37, 37, 31, 1106, 32, 32,
    37, 1104, 1104, 31, 37, 1120, 37, 37, 1121, 385, 1043, 37, 37, 280, 801, 801,
    1117, 1122, 1123, 898, 14, 25, 1124, 180, 882, 1125, 237, 237, 237, 901, 149, 149,
    0, 74, 1126, 75, 75, 0, 196, 196, 0, 112, 112, 0, 31, 31, 10, 10,
    0, 11, 11, 0, 176, 176, 69, 0, 19, 19, 75, 0, 78, 78, 0, 0,
    31, 0, 35, 35, 36, 37, 83, 83, 76, 0, 177, 178, 13, 0, 79, 79,
    0, 0, 31, 0, 297, 297, 0, 16, 16, 0, 80, 80, 0, 0, 31, 0,
    14, 14, 0, 81, 81, 0, 341, 341, 0, 17, 17, 0, 18, 18, 0, 76,
    76, 0, 478, 324, 255, 159, 0, 0, 0, 0, 0, 0, 1127, 555, 555, 74,
    74, 31, 0, 0, 24, 66, 66, 1127, 0, 0, 855, 34, 496, 113, 113, 207,
    74, 74, 497, 135, 702, 1032, 798, 131, 131, 234, 22, 131, 955, 1128, 855, 34,
    0, 496, 113, 113, 207, 135, 702, 32, 798, 955, 66, 66, 66, 34, 496, 99,
    795, 0, 797, 796, 113, 799, 765, 135, 797, 193, 798, 131, 131, 74, 74, 497,
    702, 131, 955, 439, 504, 31, 131, 504, 31, 131, 1004, 31, 439, 1002, 104, 104,
    0, 103, 102, 104, 104, 0, 103, 468, 808, 81, 331, 332, 332, 37, 234, 234,
    332, 329, 330, 330, 37, 234, 234, 330, 93, 634, 635, 636, 633, 1129, 21, 1130,
    1131, 604, 41, 125, 126, 127, 1107, 127, 128, 128, 0, 0, 0, 0, 0, 0,
    103, 139, 0, 141, 201, 0, 0, 0, 140, 0, 0, 31, 31, 0, 0, 139,
    31, 0, 439, 0, 0, 0, 141, 201, 0, 0, 0, 140, 0, 0, 128, 128,
    499, 0, 24, 0, 0, 1132, 224, 225, 431, 431, 798, 798, 798, 431, 113, 135,
    797, 128, 798, 603, 1133, 1133, 876, 730, 24, 959, 128, 128, 14, 165, 0, 128,
    128, 1134, 1134, 928, 1107, 1107, 1107, 1107, 401, 0, 0, 140, 128, 128, 0, 1135,
    1135, 0, 138, 33, 139, 0, 141, 0, 142, 0, 1136, 1107, 928, 827, 0, 138,
    33, 139, 0, 140, 0, 0, 141, 0, 142, 74, 1137, 833, 740, 1138, 100, 1139,
    0, 0, 0, 445, 449, 449, 449, 0, 398, 1140, 1140, 1140, 0, 24, 887, 532,
    533, 1130, 1131, 0, 1141, 1142, 328, 383, 108, 912, 37, 37, 1093, 1000, 1000, 803,
    1047, 0, 74, 74, 0, 0, 0, 0, 0, 26, 26, 31, 141, 141, 193, 193,
    0, 0, 0, 0, 99, 113, 135, 1001, 1001, 804, 1046, 0, 74, 0, 0, 0,
    0, 31, 0, 0, 0, 855, 855, 333, 0, 364, 0, 0, 113, 135, 0, 0,
    142, 699, 0, 0, 142, 104, 0, 0, 0, 0, 104, 121, 116, 392, 392, 103,
    344, 377, 718, 390, 621, 632, 1143, 1072, 1144, 952, 1145, 1146, 1147, 1148, 1149, 1150,
    1151, 1152, 1153, 1154, 1155, 1156, 1157, 1158, 266, 77, 89, 1159, 121, 334, 1160, 1161,
    600, 0, 439, 949, 59, 1162, 156, 156, 39, 156, 70, 96, 39, 39, 156, 120,
    39, 128, 156, 156, 39, 156, 39, 156, 39, 58, 71, 39, 156, 39, 156, 39,
    156, 706, 53, 737, 39, 120, 156, 39, 156, 120, 156, 1086, 473, 373, 0, 120,
    128, 120, 120, 39, 156, 39, 120, 156, 39, 120, 156, 343, 120, 156, 1163, 120,
    39, 120, 923, 1012, 1164, 927, 144, 710, 120, 39, 156, 503, 494, 120, 1096, 120,
    39, 156, 120, 39, 120, 156, 39, 120, 156, 39, 120, 39, 120, 156, 120, 932,
    238, 39, 156, 443, 39, 39, 853, 39, 156, 39, 120, 39, 120, 156, 1140, 128,
    39, 439, 156, 39, 120, 237, 694, 45, 693, 128, 144, 1000, 1001, 468, 212, 1165,
    1166, 213, 585, 1167, 82, 94, 263, 416, 109, 197, 1168, 602, 31, 0, 0, 0,
    0, 31, 0, 0, 0, 0, 0, 0, 0, 602, 602, 31, 1169, 341, 1170, 1171,
    743, 302, 281, 0, 1172, 1173, 1174, 577, 345, 219, 160, 160, 0, 1175, 163, 163,
    31, 0, 234, 234, 246, 465, 465, 554, 554, 158, 555, 31, 860, 466, 466, 31,
    860, 731, 731, 193, 193, 0, 31, 0, 0, 860, 496, 496, 31, 0, 19, 19,
    467, 31, 65, 1176, 221, 222, 158, 158, 222, 223, 223, 223, 224, 223, 223, 225,
    158, 158, 164, 234, 234, 246, 220, 160, 160, 0, 1177, 1177, 465, 466, 193, 249,
    0, 221, 222, 158, 158, 222, 223, 223, 223, 224, 223, 223, 225, 0, 0, 355,
    201, 0, 0, 131, 131, 1029, 0, 31, 439, 31, 949, 75, 0, 0, 0, 116,
    87, 801, 17, 0, 754, 754, 415, 0, 0, 758, 758, 0, 953, 46, 95, 67,
    185, 0, 550, 67, 68, 68, 1178, 1179, 550, 1180, 1181, 179, 1182, 1183, 950, 770,
    755, 1184, 678, 678, 1184, 1185, 31, 526, 526, 131, 239, 1184, 1186, 398, 398, 398,
    1184, 1186, 0, 1185, 681, 680, 742, 1187, 1188, 1189, 1190, 33, 1191, 1192, 1193, 1194,
    1195, 1162, 1162, 334, 409, 480, 1112, 448, 1196, 1195, 1197, 333, 333, 0, 378, 378,
    234, 450, 1198, 194, 1179, 1199, 1200, 69, 1201, 806, 428, 906, 1193, 426, 426, 70,
    70, 70, 75, 0, 0, 0, 256, 0, 0, 0, 121, 1202, 1202, 211, 502, 208,
    152, 1203, 1204, 659, 668, 669, 658, 31, 0, 0, 18, 925, 817, 817, 0, 0,
    759, 392, 103, 602, 602, 31, 153, 1079, 1205, 785, 1206, 606, 892, 986, 606, 840,
    1207, 1208, 603, 603, 1127, 555, 24, 207, 207, 333, 603, 465, 465, 465, 885, 885,
    1164, 1164, 1164, 0, 0, 121, 1209, 244, 999, 999, 66, 66, 34, 34, 34, 66,
    496, 99, 795, 797, 796, 193, 798, 113, 135, 74, 74, 497, 702, 431, 431, 798,
    798, 798, 431, 471, 804, 276, 1210, 325, 71, 71, 71, 256, 0, 0, 128, 128,
    34, 496, 496, 496, 143, 247, 0, 496, 702, 702, 0, 121, 0, 0, 0, 0,
    1211, 49, 53, 53, 53, 0, 72, 41, 0, 1212, 193, 193, 193, 1120, 1117, 1213,
    1214, 1215, 660, 1163, 1163, 1163, 1216, 1217, 250, 514, 376, 373, 372, 371, 31, 1218,
    1218, 1218, 250, 514, 376, 372, 371, 709, 377, 861, 1067, 70, 274, 0, 141, 0,
    74, 0, 0, 0, 31, 164, 166, 546, 364, 355, 355, 31, 31, 0, 0, 0,
    140, 0, 113, 135, 355, 1219, 645, 1220, 1135, 1221, 817, 644, 672, 376, 0, 255,
    0, 539, 76, 88, 586, 267, 1222, 741, 759, 9, 1223, 1224, 1225, 618, 1226, 1226,
    648, 918, 652, 972, 968, 976, 822, 1050, 920, 1053, 974, 970, 978, 824, 1049, 919,
    1052, 973, 969, 977, 823, 1051, 921, 1054, 975, 971, 979, 825, 460, 488, 616, 966,
    980, 1055, 1200, 1187, 31, 1227, 1227, 1228, 31, 37, 37, 31, 31, 31, 37, 37,
    618, 1229, 1229, 821, 820, 31, 618, 618, 648, 918, 652, 972, 968, 976, 822, 1050,
    920, 1053, 974, 970, 978, 824, 1049, 919, 1052, 973, 969, 977, 823, 1051, 921, 1054,
    975, 971, 979, 825, 460, 488, 616, 966, 980, 1055, 1200, 1187, 821, 820, 32, 32,
    37, 446, 446, 0, 0, 0, 950, 0, 0, 0, 0, 446, 496, 496, 0, 31,
    0, 1230, 135, 99, 99, 207, 113, 113, 207, 795, 796, 797, 1032, 702, 74, 74,
    497, 149, 496, 1231, 1232, 0, 731, 731, 0, 0, 31, 0, 0, 0, 0, 731,
    1233, 1234, 362, 123, 1235, 0, 0, 0, 0, 0, 104, 121, 121, 104, 121, 1235,
    748, 1236, 1236, 484, 486, 255, 123, 123, 0, 31, 0, 0, 0, 0, 0, 0,
    0, 362, 362, 0, 31, 0, 0, 0, 0, 0, 0, 0, 797, 0, 0, 1237,
    287, 113, 287, 113, 784, 131, 131, 1238, 444, 928, 163, 147, 131, 131, 1089, 0,
    0, 0, 66, 0, 675, 131, 131, 1089, 0, 0, 0, 66, 0, 1004, 131, 131,
    234, 234, 22, 0, 131, 131, 131, 234, 14, 420, 1056, 1056, 99, 31, 113, 1056,
    239, 0, 1197, 23, 73, 509, 1239, 1240, 661, 1241, 1242, 1243, 1244, 1245, 1246, 1247,
    1248, 1249, 1250, 1251, 1252, 745, 879, 1253, 1254, 1255, 1186,
];

pub fn get_duckdb_segment_grammar(name: &str) -> Option<RootGrammar> {
    match name {
        "AbortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(0), tables: &DUCKDB_TABLES }),
        "AbsoluteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1), tables: &DUCKDB_TABLES }),
        "AbsoluteValueOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2), tables: &DUCKDB_TABLES }),
        "AccessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3), tables: &DUCKDB_TABLES }),
        "AccessObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(4), tables: &DUCKDB_TABLES }),
        "AccessPermissionSegment" => Some(RootGrammar { grammar_id: GrammarId(65), tables: &DUCKDB_TABLES }),
        "AccessPermissionsSegment" => Some(RootGrammar { grammar_id: GrammarId(134), tables: &DUCKDB_TABLES }),
        "AccessSchemaObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(137), tables: &DUCKDB_TABLES }),
        "AccessSchemaPluralObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(160), tables: &DUCKDB_TABLES }),
        "AccessStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(183), tables: &DUCKDB_TABLES }),
        "AccessTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(186), tables: &DUCKDB_TABLES }),
        "AccessorGrammar" => Some(RootGrammar { grammar_id: GrammarId(208), tables: &DUCKDB_TABLES }),
        "AccountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(211), tables: &DUCKDB_TABLES }),
        "AccountsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(212), tables: &DUCKDB_TABLES }),
        "ActionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(213), tables: &DUCKDB_TABLES }),
        "AddKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(214), tables: &DUCKDB_TABLES }),
        "AdjacentSegment" => Some(RootGrammar { grammar_id: GrammarId(215), tables: &DUCKDB_TABLES }),
        "AdminKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(219), tables: &DUCKDB_TABLES }),
        "AfterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(220), tables: &DUCKDB_TABLES }),
        "AggregateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(221), tables: &DUCKDB_TABLES }),
        "AggregateOrderByClause" => Some(RootGrammar { grammar_id: GrammarId(222), tables: &DUCKDB_TABLES }),
        "AliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(223), tables: &DUCKDB_TABLES }),
        "AliasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(235), tables: &DUCKDB_TABLES }),
        "AliasedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(236), tables: &DUCKDB_TABLES }),
        "AllKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(239), tables: &DUCKDB_TABLES }),
        "Allow_connectionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(240), tables: &DUCKDB_TABLES }),
        "AlsoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(241), tables: &DUCKDB_TABLES }),
        "AlterAggregateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(242), tables: &DUCKDB_TABLES }),
        "AlterDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(268), tables: &DUCKDB_TABLES }),
        "AlterDefaultPrivilegesGrantSegment" => Some(RootGrammar { grammar_id: GrammarId(323), tables: &DUCKDB_TABLES }),
        "AlterDefaultPrivilegesObjectPrivilegesSegment" => Some(RootGrammar { grammar_id: GrammarId(336), tables: &DUCKDB_TABLES }),
        "AlterDefaultPrivilegesRevokeSegment" => Some(RootGrammar { grammar_id: GrammarId(353), tables: &DUCKDB_TABLES }),
        "AlterDefaultPrivilegesSchemaObjectsSegment" => Some(RootGrammar { grammar_id: GrammarId(368), tables: &DUCKDB_TABLES }),
        "AlterDefaultPrivilegesStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(375), tables: &DUCKDB_TABLES }),
        "AlterDefaultPrivilegesToFromRolesSegment" => Some(RootGrammar { grammar_id: GrammarId(399), tables: &DUCKDB_TABLES }),
        "AlterDomainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(404), tables: &DUCKDB_TABLES }),
        "AlterExtensionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(471), tables: &DUCKDB_TABLES }),
        "AlterForeignTableActionSegment" => Some(RootGrammar { grammar_id: GrammarId(579), tables: &DUCKDB_TABLES }),
        "AlterForeignTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(834), tables: &DUCKDB_TABLES }),
        "AlterFunctionActionSegment" => Some(RootGrammar { grammar_id: GrammarId(856), tables: &DUCKDB_TABLES }),
        "AlterFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(920), tables: &DUCKDB_TABLES }),
        "AlterIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(953), tables: &DUCKDB_TABLES }),
        "AlterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1014), tables: &DUCKDB_TABLES }),
        "AlterMaterializedViewActionSegment" => Some(RootGrammar { grammar_id: GrammarId(1015), tables: &DUCKDB_TABLES }),
        "AlterMaterializedViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1080), tables: &DUCKDB_TABLES }),
        "AlterPolicyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1128), tables: &DUCKDB_TABLES }),
        "AlterProcedureActionSegment" => Some(RootGrammar { grammar_id: GrammarId(1158), tables: &DUCKDB_TABLES }),
        "AlterProcedureStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1187), tables: &DUCKDB_TABLES }),
        "AlterPublicationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1235), tables: &DUCKDB_TABLES }),
        "AlterRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1267), tables: &DUCKDB_TABLES }),
        "AlterSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1357), tables: &DUCKDB_TABLES }),
        "AlterSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(1370), tables: &DUCKDB_TABLES }),
        "AlterSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1421), tables: &DUCKDB_TABLES }),
        "AlterStatisticsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1444), tables: &DUCKDB_TABLES }),
        "AlterSubscriptionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1473), tables: &DUCKDB_TABLES }),
        "AlterSystemStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1521), tables: &DUCKDB_TABLES }),
        "AlterTableActionSegment" => Some(RootGrammar { grammar_id: GrammarId(1544), tables: &DUCKDB_TABLES }),
        "AlterTableDropColumnGrammar" => Some(RootGrammar { grammar_id: GrammarId(1784), tables: &DUCKDB_TABLES }),
        "AlterTableOptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1789), tables: &DUCKDB_TABLES }),
        "AlterTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1816), tables: &DUCKDB_TABLES }),
        "AlterTextSearchConfigurationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1882), tables: &DUCKDB_TABLES }),
        "AlterTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1947), tables: &DUCKDB_TABLES }),
        "AlterTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1964), tables: &DUCKDB_TABLES }),
        "AlterViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2038), tables: &DUCKDB_TABLES }),
        "AlwaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2096), tables: &DUCKDB_TABLES }),
        "AmpersandSegment" => Some(RootGrammar { grammar_id: GrammarId(2097), tables: &DUCKDB_TABLES }),
        "AnalyseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2098), tables: &DUCKDB_TABLES }),
        "AnalyzeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2099), tables: &DUCKDB_TABLES }),
        "AnalyzeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2100), tables: &DUCKDB_TABLES }),
        "AndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2119), tables: &DUCKDB_TABLES }),
        "AndOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2120), tables: &DUCKDB_TABLES }),
        "AntiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2121), tables: &DUCKDB_TABLES }),
        "AnyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2122), tables: &DUCKDB_TABLES }),
        "AppendKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2123), tables: &DUCKDB_TABLES }),
        "ApplyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2124), tables: &DUCKDB_TABLES }),
        "ArithmeticBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2125), tables: &DUCKDB_TABLES }),
        "ArrayAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(2137), tables: &DUCKDB_TABLES }),
        "ArrayExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2154), tables: &DUCKDB_TABLES }),
        "ArrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2155), tables: &DUCKDB_TABLES }),
        "ArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2156), tables: &DUCKDB_TABLES }),
        "ArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2159), tables: &DUCKDB_TABLES }),
        "ArrayTypeSuffixSegment" => Some(RootGrammar { grammar_id: GrammarId(2160), tables: &DUCKDB_TABLES }),
        "AsAliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2162), tables: &DUCKDB_TABLES }),
        "AsAliasOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2165), tables: &DUCKDB_TABLES }),
        "AsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2167), tables: &DUCKDB_TABLES }),
        "AscKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2168), tables: &DUCKDB_TABLES }),
        "AsofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2169), tables: &DUCKDB_TABLES }),
        "AssertionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2170), tables: &DUCKDB_TABLES }),
        "AssignmentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2171), tables: &DUCKDB_TABLES }),
        "AsymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2172), tables: &DUCKDB_TABLES }),
        "AtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2173), tables: &DUCKDB_TABLES }),
        "AtomicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2174), tables: &DUCKDB_TABLES }),
        "AttachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2175), tables: &DUCKDB_TABLES }),
        "AttributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2176), tables: &DUCKDB_TABLES }),
        "AuditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2177), tables: &DUCKDB_TABLES }),
        "AuthorizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2178), tables: &DUCKDB_TABLES }),
        "AutoIncrementGrammar" => Some(RootGrammar { grammar_id: GrammarId(2179), tables: &DUCKDB_TABLES }),
        "Auto_incrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2181), tables: &DUCKDB_TABLES }),
        "Avg_row_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2182), tables: &DUCKDB_TABLES }),
        "BackupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2183), tables: &DUCKDB_TABLES }),
        "BackwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2184), tables: &DUCKDB_TABLES }),
        "BareFunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(2185), tables: &DUCKDB_TABLES }),
        "BaseExpressionElementGrammar" => Some(RootGrammar { grammar_id: GrammarId(2186), tables: &DUCKDB_TABLES }),
        "BaseFileSegment" => Some(RootGrammar { grammar_id: GrammarId(2203), tables: &DUCKDB_TABLES }),
        "BaseSegment" => Some(RootGrammar { grammar_id: GrammarId(2204), tables: &DUCKDB_TABLES }),
        "BeforeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2205), tables: &DUCKDB_TABLES }),
        "BeginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2206), tables: &DUCKDB_TABLES }),
        "BernoulliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2207), tables: &DUCKDB_TABLES }),
        "BetweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2208), tables: &DUCKDB_TABLES }),
        "BigintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2209), tables: &DUCKDB_TABLES }),
        "BigserialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2210), tables: &DUCKDB_TABLES }),
        "BinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2211), tables: &DUCKDB_TABLES }),
        "BinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2212), tables: &DUCKDB_TABLES }),
        "BinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2218), tables: &DUCKDB_TABLES }),
        "BindingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2219), tables: &DUCKDB_TABLES }),
        "BitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2220), tables: &DUCKDB_TABLES }),
        "BitvarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2221), tables: &DUCKDB_TABLES }),
        "BitwiseAndSegment" => Some(RootGrammar { grammar_id: GrammarId(2222), tables: &DUCKDB_TABLES }),
        "BitwiseLShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(2223), tables: &DUCKDB_TABLES }),
        "BitwiseOrSegment" => Some(RootGrammar { grammar_id: GrammarId(2226), tables: &DUCKDB_TABLES }),
        "BitwiseRShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(2227), tables: &DUCKDB_TABLES }),
        "BitwiseXorSegment" => Some(RootGrammar { grammar_id: GrammarId(2230), tables: &DUCKDB_TABLES }),
        "BoolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2231), tables: &DUCKDB_TABLES }),
        "BooleanBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2232), tables: &DUCKDB_TABLES }),
        "BooleanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2235), tables: &DUCKDB_TABLES }),
        "BooleanLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(2236), tables: &DUCKDB_TABLES }),
        "BothKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2239), tables: &DUCKDB_TABLES }),
        "BoxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2240), tables: &DUCKDB_TABLES }),
        "BpcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2241), tables: &DUCKDB_TABLES }),
        "BracketedArguments" => Some(RootGrammar { grammar_id: GrammarId(2242), tables: &DUCKDB_TABLES }),
        "BracketedColumnReferenceListGrammar" => Some(RootGrammar { grammar_id: GrammarId(2245), tables: &DUCKDB_TABLES }),
        "BracketedSegment" => Some(RootGrammar { grammar_id: GrammarId(2248), tables: &DUCKDB_TABLES }),
        "BracketedSetExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2249), tables: &DUCKDB_TABLES }),
        "BreadthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2251), tables: &DUCKDB_TABLES }),
        "BreakKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2252), tables: &DUCKDB_TABLES }),
        "BrowseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2253), tables: &DUCKDB_TABLES }),
        "BuffersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2254), tables: &DUCKDB_TABLES }),
        "BulkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2255), tables: &DUCKDB_TABLES }),
        "ByKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2256), tables: &DUCKDB_TABLES }),
        "BypassrlsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2257), tables: &DUCKDB_TABLES }),
        "ByteaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2258), tables: &DUCKDB_TABLES }),
        "CTEColumnList" => Some(RootGrammar { grammar_id: GrammarId(2259), tables: &DUCKDB_TABLES }),
        "CTEDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(2261), tables: &DUCKDB_TABLES }),
        "CacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2288), tables: &DUCKDB_TABLES }),
        "CallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2289), tables: &DUCKDB_TABLES }),
        "CallStoredProcedureSegment" => Some(RootGrammar { grammar_id: GrammarId(2290), tables: &DUCKDB_TABLES }),
        "CalledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2293), tables: &DUCKDB_TABLES }),
        "CascadeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2294), tables: &DUCKDB_TABLES }),
        "CascadeRestrictGrammar" => Some(RootGrammar { grammar_id: GrammarId(2295), tables: &DUCKDB_TABLES }),
        "CascadedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2298), tables: &DUCKDB_TABLES }),
        "CaseExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2299), tables: &DUCKDB_TABLES }),
        "CaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2323), tables: &DUCKDB_TABLES }),
        "CastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2324), tables: &DUCKDB_TABLES }),
        "CastOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2325), tables: &DUCKDB_TABLES }),
        "CatalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2326), tables: &DUCKDB_TABLES }),
        "ChainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2327), tables: &DUCKDB_TABLES }),
        "ChangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2328), tables: &DUCKDB_TABLES }),
        "CharCharacterSetGrammar" => Some(RootGrammar { grammar_id: GrammarId(2329), tables: &DUCKDB_TABLES }),
        "CharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2330), tables: &DUCKDB_TABLES }),
        "CharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2331), tables: &DUCKDB_TABLES }),
        "CharacteristicsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2332), tables: &DUCKDB_TABLES }),
        "CheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2333), tables: &DUCKDB_TABLES }),
        "CheckedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2334), tables: &DUCKDB_TABLES }),
        "CheckpointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2335), tables: &DUCKDB_TABLES }),
        "ChecksumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2336), tables: &DUCKDB_TABLES }),
        "CidrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2337), tables: &DUCKDB_TABLES }),
        "CircleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2338), tables: &DUCKDB_TABLES }),
        "CircularstringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2339), tables: &DUCKDB_TABLES }),
        "ClassKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2340), tables: &DUCKDB_TABLES }),
        "CloseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2341), tables: &DUCKDB_TABLES }),
        "ClusterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2342), tables: &DUCKDB_TABLES }),
        "ClusterStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2343), tables: &DUCKDB_TABLES }),
        "ClusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2356), tables: &DUCKDB_TABLES }),
        "CoalesceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2357), tables: &DUCKDB_TABLES }),
        "CodeSegment" => Some(RootGrammar { grammar_id: GrammarId(2358), tables: &DUCKDB_TABLES }),
        "CollateGrammar" => Some(RootGrammar { grammar_id: GrammarId(2359), tables: &DUCKDB_TABLES }),
        "CollateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2360), tables: &DUCKDB_TABLES }),
        "CollationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2361), tables: &DUCKDB_TABLES }),
        "CollationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2362), tables: &DUCKDB_TABLES }),
        "ColonDelimiterSegment" => Some(RootGrammar { grammar_id: GrammarId(2368), tables: &DUCKDB_TABLES }),
        "ColonPrefixSegment" => Some(RootGrammar { grammar_id: GrammarId(2369), tables: &DUCKDB_TABLES }),
        "ColonSegment" => Some(RootGrammar { grammar_id: GrammarId(2370), tables: &DUCKDB_TABLES }),
        "ColumnConstraintDefaultGrammar" => Some(RootGrammar { grammar_id: GrammarId(2371), tables: &DUCKDB_TABLES }),
        "ColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(2376), tables: &DUCKDB_TABLES }),
        "ColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(2398), tables: &DUCKDB_TABLES }),
        "ColumnGeneratedGrammar" => Some(RootGrammar { grammar_id: GrammarId(2406), tables: &DUCKDB_TABLES }),
        "ColumnIndexSegment" => Some(RootGrammar { grammar_id: GrammarId(2407), tables: &DUCKDB_TABLES }),
        "ColumnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2408), tables: &DUCKDB_TABLES }),
        "ColumnReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2409), tables: &DUCKDB_TABLES }),
        "ColumnTypeReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2435), tables: &DUCKDB_TABLES }),
        "ColumnsExpressionFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(2439), tables: &DUCKDB_TABLES }),
        "ColumnsExpressionFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2445), tables: &DUCKDB_TABLES }),
        "ColumnsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2446), tables: &DUCKDB_TABLES }),
        "ColumnsExpressionNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(2449), tables: &DUCKDB_TABLES }),
        "ColumnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2452), tables: &DUCKDB_TABLES }),
        "CommaSegment" => Some(RootGrammar { grammar_id: GrammarId(2453), tables: &DUCKDB_TABLES }),
        "CommentClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2454), tables: &DUCKDB_TABLES }),
        "CommentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2457), tables: &DUCKDB_TABLES }),
        "CommentOnStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2458), tables: &DUCKDB_TABLES }),
        "CommentSegment" => Some(RootGrammar { grammar_id: GrammarId(2569), tables: &DUCKDB_TABLES }),
        "CommentsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2570), tables: &DUCKDB_TABLES }),
        "CommitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2571), tables: &DUCKDB_TABLES }),
        "CommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2572), tables: &DUCKDB_TABLES }),
        "CommutatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2573), tables: &DUCKDB_TABLES }),
        "ComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2574), tables: &DUCKDB_TABLES }),
        "ComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2585), tables: &DUCKDB_TABLES }),
        "CompletionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2586), tables: &DUCKDB_TABLES }),
        "CompositeBinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2587), tables: &DUCKDB_TABLES }),
        "CompositeComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2588), tables: &DUCKDB_TABLES }),
        "CompositeValueExpansionSegment" => Some(RootGrammar { grammar_id: GrammarId(2589), tables: &DUCKDB_TABLES }),
        "CompoundcurveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2594), tables: &DUCKDB_TABLES }),
        "CompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2595), tables: &DUCKDB_TABLES }),
        "CompressionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2596), tables: &DUCKDB_TABLES }),
        "Compression_levelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2597), tables: &DUCKDB_TABLES }),
        "ComputeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2598), tables: &DUCKDB_TABLES }),
        "ConcatSegment" => Some(RootGrammar { grammar_id: GrammarId(2599), tables: &DUCKDB_TABLES }),
        "ConcurrentlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2602), tables: &DUCKDB_TABLES }),
        "ConditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2603), tables: &DUCKDB_TABLES }),
        "ConditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2604), tables: &DUCKDB_TABLES }),
        "ConfigurationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2608), tables: &DUCKDB_TABLES }),
        "ConflictActionSegment" => Some(RootGrammar { grammar_id: GrammarId(2609), tables: &DUCKDB_TABLES }),
        "ConflictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2645), tables: &DUCKDB_TABLES }),
        "ConflictTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(2646), tables: &DUCKDB_TABLES }),
        "ConnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2667), tables: &DUCKDB_TABLES }),
        "ConnectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2668), tables: &DUCKDB_TABLES }),
        "ConstraintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2669), tables: &DUCKDB_TABLES }),
        "ConstraintsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2670), tables: &DUCKDB_TABLES }),
        "ContainstableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2671), tables: &DUCKDB_TABLES }),
        "ContentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2672), tables: &DUCKDB_TABLES }),
        "ContinueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2673), tables: &DUCKDB_TABLES }),
        "ConversionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2674), tables: &DUCKDB_TABLES }),
        "CopyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2675), tables: &DUCKDB_TABLES }),
        "CopyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2676), tables: &DUCKDB_TABLES }),
        "CorrelationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2783), tables: &DUCKDB_TABLES }),
        "CostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2784), tables: &DUCKDB_TABLES }),
        "CostsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2785), tables: &DUCKDB_TABLES }),
        "CreateAggregateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2786), tables: &DUCKDB_TABLES }),
        "CreateCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2794), tables: &DUCKDB_TABLES }),
        "CreateCollationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2820), tables: &DUCKDB_TABLES }),
        "CreateDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2858), tables: &DUCKDB_TABLES }),
        "CreateDomainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2911), tables: &DUCKDB_TABLES }),
        "CreateExtensionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2936), tables: &DUCKDB_TABLES }),
        "CreateForeignDataWrapperStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2952), tables: &DUCKDB_TABLES }),
        "CreateForeignTableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2960), tables: &DUCKDB_TABLES }),
        "CreateForeignTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2964), tables: &DUCKDB_TABLES }),
        "CreateFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3020), tables: &DUCKDB_TABLES }),
        "CreateIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3035), tables: &DUCKDB_TABLES }),
        "CreateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3070), tables: &DUCKDB_TABLES }),
        "CreateMaterializedViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3071), tables: &DUCKDB_TABLES }),
        "CreateModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3105), tables: &DUCKDB_TABLES }),
        "CreateOperatorStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3125), tables: &DUCKDB_TABLES }),
        "CreatePolicyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3160), tables: &DUCKDB_TABLES }),
        "CreateProcedureStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3197), tables: &DUCKDB_TABLES }),
        "CreatePublicationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3204), tables: &DUCKDB_TABLES }),
        "CreateRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3220), tables: &DUCKDB_TABLES }),
        "CreateSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3284), tables: &DUCKDB_TABLES }),
        "CreateSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(3294), tables: &DUCKDB_TABLES }),
        "CreateSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3338), tables: &DUCKDB_TABLES }),
        "CreateServerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3346), tables: &DUCKDB_TABLES }),
        "CreateStatisticsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3360), tables: &DUCKDB_TABLES }),
        "CreateSubscriptionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3380), tables: &DUCKDB_TABLES }),
        "CreateTableAsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3392), tables: &DUCKDB_TABLES }),
        "CreateTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3393), tables: &DUCKDB_TABLES }),
        "CreateTextSearchConfigurationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3430), tables: &DUCKDB_TABLES }),
        "CreateTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3446), tables: &DUCKDB_TABLES }),
        "CreateTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3522), tables: &DUCKDB_TABLES }),
        "CreateUserMappingGrammar" => Some(RootGrammar { grammar_id: GrammarId(3538), tables: &DUCKDB_TABLES }),
        "CreateUserMappingStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3542), tables: &DUCKDB_TABLES }),
        "CreateUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3553), tables: &DUCKDB_TABLES }),
        "CreateViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3557), tables: &DUCKDB_TABLES }),
        "CreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3571), tables: &DUCKDB_TABLES }),
        "CreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3572), tables: &DUCKDB_TABLES }),
        "CreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3573), tables: &DUCKDB_TABLES }),
        "CrossKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3574), tables: &DUCKDB_TABLES }),
        "CsvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3575), tables: &DUCKDB_TABLES }),
        "CubeFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3576), tables: &DUCKDB_TABLES }),
        "CubeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3577), tables: &DUCKDB_TABLES }),
        "CubeRollupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3578), tables: &DUCKDB_TABLES }),
        "CurrentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3584), tables: &DUCKDB_TABLES }),
        "Current_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3585), tables: &DUCKDB_TABLES }),
        "Current_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3586), tables: &DUCKDB_TABLES }),
        "Current_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3587), tables: &DUCKDB_TABLES }),
        "Current_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3588), tables: &DUCKDB_TABLES }),
        "Current_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3589), tables: &DUCKDB_TABLES }),
        "Current_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3590), tables: &DUCKDB_TABLES }),
        "Current_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3591), tables: &DUCKDB_TABLES }),
        "CursorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3592), tables: &DUCKDB_TABLES }),
        "CurvepolygonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3593), tables: &DUCKDB_TABLES }),
        "CycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3594), tables: &DUCKDB_TABLES }),
        "DataKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3595), tables: &DUCKDB_TABLES }),
        "DatabaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3596), tables: &DUCKDB_TABLES }),
        "DatabaseReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3597), tables: &DUCKDB_TABLES }),
        "DatabasesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3601), tables: &DUCKDB_TABLES }),
        "DatatypeIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3602), tables: &DUCKDB_TABLES }),
        "DatatypeSegment" => Some(RootGrammar { grammar_id: GrammarId(3606), tables: &DUCKDB_TABLES }),
        "DateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3709), tables: &DUCKDB_TABLES }),
        "DatePartFunctionName" => Some(RootGrammar { grammar_id: GrammarId(3710), tables: &DUCKDB_TABLES }),
        "DatePartFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3711), tables: &DUCKDB_TABLES }),
        "DateTimeFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(3712), tables: &DUCKDB_TABLES }),
        "DateTimeLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(3718), tables: &DUCKDB_TABLES }),
        "DateTimeTypeIdentifier" => Some(RootGrammar { grammar_id: GrammarId(3721), tables: &DUCKDB_TABLES }),
        "DaterangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3731), tables: &DUCKDB_TABLES }),
        "DatetimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3732), tables: &DUCKDB_TABLES }),
        "DatetimeUnitSegment" => Some(RootGrammar { grammar_id: GrammarId(3733), tables: &DUCKDB_TABLES }),
        "DayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3734), tables: &DUCKDB_TABLES }),
        "Day_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3735), tables: &DUCKDB_TABLES }),
        "Day_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3736), tables: &DUCKDB_TABLES }),
        "Day_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3737), tables: &DUCKDB_TABLES }),
        "Day_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3738), tables: &DUCKDB_TABLES }),
        "DayofmonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3739), tables: &DUCKDB_TABLES }),
        "DayofweekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3740), tables: &DUCKDB_TABLES }),
        "DayofyearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3741), tables: &DUCKDB_TABLES }),
        "DaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3742), tables: &DUCKDB_TABLES }),
        "DbccKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3743), tables: &DUCKDB_TABLES }),
        "DeallocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3744), tables: &DUCKDB_TABLES }),
        "DeallocateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3745), tables: &DUCKDB_TABLES }),
        "DecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3751), tables: &DUCKDB_TABLES }),
        "DecimalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3752), tables: &DUCKDB_TABLES }),
        "DeclareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3753), tables: &DUCKDB_TABLES }),
        "Dedent" => Some(RootGrammar { grammar_id: GrammarId(234), tables: &DUCKDB_TABLES }),
        "DefaultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3754), tables: &DUCKDB_TABLES }),
        "DefaultValuesGrammar" => Some(RootGrammar { grammar_id: GrammarId(3755), tables: &DUCKDB_TABLES }),
        "DefaultsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3758), tables: &DUCKDB_TABLES }),
        "DeferrableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3759), tables: &DUCKDB_TABLES }),
        "DeferredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3760), tables: &DUCKDB_TABLES }),
        "DefinerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3761), tables: &DUCKDB_TABLES }),
        "DefinitionArgumentValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(3762), tables: &DUCKDB_TABLES }),
        "DefinitionParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(3765), tables: &DUCKDB_TABLES }),
        "DefinitionParametersSegment" => Some(RootGrammar { grammar_id: GrammarId(3770), tables: &DUCKDB_TABLES }),
        "Delay_key_writeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3773), tables: &DUCKDB_TABLES }),
        "DelayedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3774), tables: &DUCKDB_TABLES }),
        "DeleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3775), tables: &DUCKDB_TABLES }),
        "DeleteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3776), tables: &DUCKDB_TABLES }),
        "DelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(3805), tables: &DUCKDB_TABLES }),
        "DelimiterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3806), tables: &DUCKDB_TABLES }),
        "DelimitersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3807), tables: &DUCKDB_TABLES }),
        "DenyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3808), tables: &DUCKDB_TABLES }),
        "DependenciesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3809), tables: &DUCKDB_TABLES }),
        "DependsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3810), tables: &DUCKDB_TABLES }),
        "DepthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3811), tables: &DUCKDB_TABLES }),
        "DescKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3812), tables: &DUCKDB_TABLES }),
        "DescribeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3813), tables: &DUCKDB_TABLES }),
        "DescribeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3814), tables: &DUCKDB_TABLES }),
        "DestroyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3818), tables: &DUCKDB_TABLES }),
        "DestructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3819), tables: &DUCKDB_TABLES }),
        "DetachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3820), tables: &DUCKDB_TABLES }),
        "DeterministicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3821), tables: &DUCKDB_TABLES }),
        "DictionaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3822), tables: &DUCKDB_TABLES }),
        "DisableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3823), tables: &DUCKDB_TABLES }),
        "Disable_page_skippingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3824), tables: &DUCKDB_TABLES }),
        "DiscardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3825), tables: &DUCKDB_TABLES }),
        "DiscardStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3826), tables: &DUCKDB_TABLES }),
        "DiskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3834), tables: &DUCKDB_TABLES }),
        "DistinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3835), tables: &DUCKDB_TABLES }),
        "DistinctrowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3836), tables: &DUCKDB_TABLES }),
        "DistributedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3837), tables: &DUCKDB_TABLES }),
        "DivKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3838), tables: &DUCKDB_TABLES }),
        "DivideSegment" => Some(RootGrammar { grammar_id: GrammarId(3839), tables: &DUCKDB_TABLES }),
        "DoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3842), tables: &DUCKDB_TABLES }),
        "DoStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3843), tables: &DUCKDB_TABLES }),
        "DocumentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3852), tables: &DUCKDB_TABLES }),
        "DollarNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3853), tables: &DUCKDB_TABLES }),
        "DomainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3854), tables: &DUCKDB_TABLES }),
        "DotSegment" => Some(RootGrammar { grammar_id: GrammarId(3855), tables: &DUCKDB_TABLES }),
        "DoubleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3856), tables: &DUCKDB_TABLES }),
        "DropAggregateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3857), tables: &DUCKDB_TABLES }),
        "DropBehaviorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3869), tables: &DUCKDB_TABLES }),
        "DropCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3872), tables: &DUCKDB_TABLES }),
        "DropCollationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3881), tables: &DUCKDB_TABLES }),
        "DropDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3887), tables: &DUCKDB_TABLES }),
        "DropDomainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3896), tables: &DUCKDB_TABLES }),
        "DropExtensionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3903), tables: &DUCKDB_TABLES }),
        "DropForeignTableStatement" => Some(RootGrammar { grammar_id: GrammarId(3909), tables: &DUCKDB_TABLES }),
        "DropFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3917), tables: &DUCKDB_TABLES }),
        "DropIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3926), tables: &DUCKDB_TABLES }),
        "DropKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3934), tables: &DUCKDB_TABLES }),
        "DropMaterializedViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3935), tables: &DUCKDB_TABLES }),
        "DropModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3943), tables: &DUCKDB_TABLES }),
        "DropOwnedStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3948), tables: &DUCKDB_TABLES }),
        "DropPolicyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3959), tables: &DUCKDB_TABLES }),
        "DropProcedureStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3967), tables: &DUCKDB_TABLES }),
        "DropPublicationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3978), tables: &DUCKDB_TABLES }),
        "DropRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3985), tables: &DUCKDB_TABLES }),
        "DropSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3990), tables: &DUCKDB_TABLES }),
        "DropSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3996), tables: &DUCKDB_TABLES }),
        "DropStatisticsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4003), tables: &DUCKDB_TABLES }),
        "DropSubscriptionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4012), tables: &DUCKDB_TABLES }),
        "DropTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4020), tables: &DUCKDB_TABLES }),
        "DropTextSearchConfigurationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4028), tables: &DUCKDB_TABLES }),
        "DropTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4036), tables: &DUCKDB_TABLES }),
        "DropTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4044), tables: &DUCKDB_TABLES }),
        "DropUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4051), tables: &DUCKDB_TABLES }),
        "DropViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4056), tables: &DUCKDB_TABLES }),
        "DummyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4063), tables: &DUCKDB_TABLES }),
        "DumpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4064), tables: &DUCKDB_TABLES }),
        "EachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4065), tables: &DUCKDB_TABLES }),
        "ElseClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4066), tables: &DUCKDB_TABLES }),
        "ElseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4069), tables: &DUCKDB_TABLES }),
        "ElseifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4070), tables: &DUCKDB_TABLES }),
        "EmptyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4071), tables: &DUCKDB_TABLES }),
        "EmptyStructLiteralBracketsSegment" => Some(RootGrammar { grammar_id: GrammarId(4072), tables: &DUCKDB_TABLES }),
        "EmptyStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4073), tables: &DUCKDB_TABLES }),
        "EnableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4076), tables: &DUCKDB_TABLES }),
        "EnclosedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4077), tables: &DUCKDB_TABLES }),
        "EncodingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4078), tables: &DUCKDB_TABLES }),
        "EncryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4079), tables: &DUCKDB_TABLES }),
        "EndBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(232), tables: &DUCKDB_TABLES }),
        "EndCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(4080), tables: &DUCKDB_TABLES }),
        "EndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4081), tables: &DUCKDB_TABLES }),
        "EndSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(2153), tables: &DUCKDB_TABLES }),
        "EnumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4082), tables: &DUCKDB_TABLES }),
        "EqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(4083), tables: &DUCKDB_TABLES }),
        "EqualsSegment_a" => Some(RootGrammar { grammar_id: GrammarId(4084), tables: &DUCKDB_TABLES }),
        "ErrlvlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4085), tables: &DUCKDB_TABLES }),
        "EscapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4086), tables: &DUCKDB_TABLES }),
        "EscapedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4087), tables: &DUCKDB_TABLES }),
        "EventKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4088), tables: &DUCKDB_TABLES }),
        "ExceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4089), tables: &DUCKDB_TABLES }),
        "ExcludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4090), tables: &DUCKDB_TABLES }),
        "ExcludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4091), tables: &DUCKDB_TABLES }),
        "ExclusionConstraintElementSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &DUCKDB_TABLES }),
        "ExclusiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4096), tables: &DUCKDB_TABLES }),
        "ExecuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4097), tables: &DUCKDB_TABLES }),
        "ExecuteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4098), tables: &DUCKDB_TABLES }),
        "ExistingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4104), tables: &DUCKDB_TABLES }),
        "ExistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4105), tables: &DUCKDB_TABLES }),
        "ExitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4106), tables: &DUCKDB_TABLES }),
        "ExplainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4107), tables: &DUCKDB_TABLES }),
        "ExplainOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(4108), tables: &DUCKDB_TABLES }),
        "ExplainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4128), tables: &DUCKDB_TABLES }),
        "ExpressionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4144), tables: &DUCKDB_TABLES }),
        "ExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4145), tables: &DUCKDB_TABLES }),
        "Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4146), tables: &DUCKDB_TABLES }),
        "Expression_A_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4172), tables: &DUCKDB_TABLES }),
        "Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4180), tables: &DUCKDB_TABLES }),
        "Expression_B_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4190), tables: &DUCKDB_TABLES }),
        "Expression_C_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4195), tables: &DUCKDB_TABLES }),
        "Expression_D_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4217), tables: &DUCKDB_TABLES }),
        "Expression_D_Potential_Select_Statement_Without_Brackets" => Some(RootGrammar { grammar_id: GrammarId(4254), tables: &DUCKDB_TABLES }),
        "ExtendedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4262), tables: &DUCKDB_TABLES }),
        "ExtendedNaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(4263), tables: &DUCKDB_TABLES }),
        "ExtendedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(4264), tables: &DUCKDB_TABLES }),
        "ExtensionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4274), tables: &DUCKDB_TABLES }),
        "ExtensionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3597), tables: &DUCKDB_TABLES }),
        "ExternalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4275), tables: &DUCKDB_TABLES }),
        "ExtractKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4276), tables: &DUCKDB_TABLES }),
        "FactorialOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(4277), tables: &DUCKDB_TABLES }),
        "FalseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4278), tables: &DUCKDB_TABLES }),
        "FalseSegment" => Some(RootGrammar { grammar_id: GrammarId(4279), tables: &DUCKDB_TABLES }),
        "FamilyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4280), tables: &DUCKDB_TABLES }),
        "FetchClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4281), tables: &DUCKDB_TABLES }),
        "FetchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4298), tables: &DUCKDB_TABLES }),
        "FieldsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4299), tables: &DUCKDB_TABLES }),
        "FileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4300), tables: &DUCKDB_TABLES }),
        "FileSegment" => Some(RootGrammar { grammar_id: GrammarId(4301), tables: &DUCKDB_TABLES }),
        "FillfactorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4309), tables: &DUCKDB_TABLES }),
        "FilterClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(4310), tables: &DUCKDB_TABLES }),
        "FilterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4316), tables: &DUCKDB_TABLES }),
        "FinalizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4317), tables: &DUCKDB_TABLES }),
        "FirstKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4318), tables: &DUCKDB_TABLES }),
        "Float4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4319), tables: &DUCKDB_TABLES }),
        "Float8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4320), tables: &DUCKDB_TABLES }),
        "FloatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4321), tables: &DUCKDB_TABLES }),
        "FlushKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4322), tables: &DUCKDB_TABLES }),
        "FollowingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4323), tables: &DUCKDB_TABLES }),
        "ForClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4324), tables: &DUCKDB_TABLES }),
        "ForKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4345), tables: &DUCKDB_TABLES }),
        "ForceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4346), tables: &DUCKDB_TABLES }),
        "Force_not_nullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4347), tables: &DUCKDB_TABLES }),
        "Force_nullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4348), tables: &DUCKDB_TABLES }),
        "Force_quoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4349), tables: &DUCKDB_TABLES }),
        "ForeignDataWrapperGrammar" => Some(RootGrammar { grammar_id: GrammarId(4350), tables: &DUCKDB_TABLES }),
        "ForeignKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(4354), tables: &DUCKDB_TABLES }),
        "ForeignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4357), tables: &DUCKDB_TABLES }),
        "ForeignTableColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(4358), tables: &DUCKDB_TABLES }),
        "ForeignTableTableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(4387), tables: &DUCKDB_TABLES }),
        "FormatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4398), tables: &DUCKDB_TABLES }),
        "ForwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4399), tables: &DUCKDB_TABLES }),
        "FrameClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4400), tables: &DUCKDB_TABLES }),
        "FrameClauseUnitGrammar" => Some(RootGrammar { grammar_id: GrammarId(4430), tables: &DUCKDB_TABLES }),
        "FreetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4434), tables: &DUCKDB_TABLES }),
        "FreetexttableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4435), tables: &DUCKDB_TABLES }),
        "FreezeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4436), tables: &DUCKDB_TABLES }),
        "FromClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4437), tables: &DUCKDB_TABLES }),
        "FromClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4441), tables: &DUCKDB_TABLES }),
        "FromExpressionElementSegment" => Some(RootGrammar { grammar_id: GrammarId(4459), tables: &DUCKDB_TABLES }),
        "FromExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4482), tables: &DUCKDB_TABLES }),
        "FromKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4509), tables: &DUCKDB_TABLES }),
        "FromPivotExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4510), tables: &DUCKDB_TABLES }),
        "FromUnpivotExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4528), tables: &DUCKDB_TABLES }),
        "FullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4554), tables: &DUCKDB_TABLES }),
        "FullTextSearchOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(4555), tables: &DUCKDB_TABLES }),
        "FulltextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4556), tables: &DUCKDB_TABLES }),
        "FunctionContentsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(4557), tables: &DUCKDB_TABLES }),
        "FunctionContentsGrammar" => Some(RootGrammar { grammar_id: GrammarId(4561), tables: &DUCKDB_TABLES }),
        "FunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(4622), tables: &DUCKDB_TABLES }),
        "FunctionDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(4625), tables: &DUCKDB_TABLES }),
        "FunctionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4727), tables: &DUCKDB_TABLES }),
        "FunctionNameIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(4728), tables: &DUCKDB_TABLES }),
        "FunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(4729), tables: &DUCKDB_TABLES }),
        "FunctionParameterGrammar" => Some(RootGrammar { grammar_id: GrammarId(4739), tables: &DUCKDB_TABLES }),
        "FunctionParameterListGrammar" => Some(RootGrammar { grammar_id: GrammarId(4763), tables: &DUCKDB_TABLES }),
        "FunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(4766), tables: &DUCKDB_TABLES }),
        "FunctionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4780), tables: &DUCKDB_TABLES }),
        "FutureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4781), tables: &DUCKDB_TABLES }),
        "GeneratedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4782), tables: &DUCKDB_TABLES }),
        "GeographyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4783), tables: &DUCKDB_TABLES }),
        "GeometryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4784), tables: &DUCKDB_TABLES }),
        "GeometrycollectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4785), tables: &DUCKDB_TABLES }),
        "GeometrycollectionmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4786), tables: &DUCKDB_TABLES }),
        "GeometrycollectionzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4787), tables: &DUCKDB_TABLES }),
        "GeometrycollectionzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4788), tables: &DUCKDB_TABLES }),
        "GlobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4789), tables: &DUCKDB_TABLES }),
        "GlobOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(4790), tables: &DUCKDB_TABLES }),
        "GlobalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4791), tables: &DUCKDB_TABLES }),
        "GrantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4792), tables: &DUCKDB_TABLES }),
        "GrantStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4793), tables: &DUCKDB_TABLES }),
        "GrantedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4831), tables: &DUCKDB_TABLES }),
        "GrantsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4832), tables: &DUCKDB_TABLES }),
        "GreaterThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(4833), tables: &DUCKDB_TABLES }),
        "GreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(4836), tables: &DUCKDB_TABLES }),
        "GreatestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4837), tables: &DUCKDB_TABLES }),
        "GroupByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4838), tables: &DUCKDB_TABLES }),
        "GroupByClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4848), tables: &DUCKDB_TABLES }),
        "GroupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4857), tables: &DUCKDB_TABLES }),
        "GroupingExpressionList" => Some(RootGrammar { grammar_id: GrammarId(4858), tables: &DUCKDB_TABLES }),
        "GroupingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4866), tables: &DUCKDB_TABLES }),
        "GroupingSetsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4867), tables: &DUCKDB_TABLES }),
        "GroupsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4875), tables: &DUCKDB_TABLES }),
        "HandlerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4876), tables: &DUCKDB_TABLES }),
        "HashKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4877), tables: &DUCKDB_TABLES }),
        "HashesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4878), tables: &DUCKDB_TABLES }),
        "HavingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4879), tables: &DUCKDB_TABLES }),
        "HavingClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4884), tables: &DUCKDB_TABLES }),
        "HavingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4892), tables: &DUCKDB_TABLES }),
        "HeaderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4893), tables: &DUCKDB_TABLES }),
        "HeapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4894), tables: &DUCKDB_TABLES }),
        "High_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4895), tables: &DUCKDB_TABLES }),
        "HoldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4896), tables: &DUCKDB_TABLES }),
        "HoldlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4897), tables: &DUCKDB_TABLES }),
        "HorizontalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(4898), tables: &DUCKDB_TABLES }),
        "HostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4899), tables: &DUCKDB_TABLES }),
        "HostsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4900), tables: &DUCKDB_TABLES }),
        "HourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4901), tables: &DUCKDB_TABLES }),
        "Hour_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4902), tables: &DUCKDB_TABLES }),
        "Hour_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4903), tables: &DUCKDB_TABLES }),
        "Hour_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4904), tables: &DUCKDB_TABLES }),
        "IcuKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4905), tables: &DUCKDB_TABLES }),
        "IdentifiedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4906), tables: &DUCKDB_TABLES }),
        "IdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(4907), tables: &DUCKDB_TABLES }),
        "IdentityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4908), tables: &DUCKDB_TABLES }),
        "Identity_insertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4909), tables: &DUCKDB_TABLES }),
        "IdentitycolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4910), tables: &DUCKDB_TABLES }),
        "IfExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(4911), tables: &DUCKDB_TABLES }),
        "IfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4914), tables: &DUCKDB_TABLES }),
        "IfNotExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(4915), tables: &DUCKDB_TABLES }),
        "IgnoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4919), tables: &DUCKDB_TABLES }),
        "IgnoreRespectNullsGrammar" => Some(RootGrammar { grammar_id: GrammarId(4920), tables: &DUCKDB_TABLES }),
        "IlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4925), tables: &DUCKDB_TABLES }),
        "ImmediateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4926), tables: &DUCKDB_TABLES }),
        "ImmutableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4927), tables: &DUCKDB_TABLES }),
        "ImplicitIndent" => Some(RootGrammar { grammar_id: GrammarId(2302), tables: &DUCKDB_TABLES }),
        "ImplicitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4928), tables: &DUCKDB_TABLES }),
        "ImportForeignSchemaGrammar" => Some(RootGrammar { grammar_id: GrammarId(4929), tables: &DUCKDB_TABLES }),
        "ImportForeignSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4933), tables: &DUCKDB_TABLES }),
        "ImportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4951), tables: &DUCKDB_TABLES }),
        "ImportedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4952), tables: &DUCKDB_TABLES }),
        "InKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4953), tables: &DUCKDB_TABLES }),
        "InOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4954), tables: &DUCKDB_TABLES }),
        "IncludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4968), tables: &DUCKDB_TABLES }),
        "IncludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4969), tables: &DUCKDB_TABLES }),
        "IncrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4970), tables: &DUCKDB_TABLES }),
        "Indent" => Some(RootGrammar { grammar_id: GrammarId(224), tables: &DUCKDB_TABLES }),
        "IndexAccessMethodSegment" => Some(RootGrammar { grammar_id: GrammarId(4971), tables: &DUCKDB_TABLES }),
        "IndexColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(4972), tables: &DUCKDB_TABLES }),
        "IndexElementOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(4977), tables: &DUCKDB_TABLES }),
        "IndexElementSegment" => Some(RootGrammar { grammar_id: GrammarId(4997), tables: &DUCKDB_TABLES }),
        "IndexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5004), tables: &DUCKDB_TABLES }),
        "IndexParametersSegment" => Some(RootGrammar { grammar_id: GrammarId(5005), tables: &DUCKDB_TABLES }),
        "IndexReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3597), tables: &DUCKDB_TABLES }),
        "Index_cleanupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5017), tables: &DUCKDB_TABLES }),
        "IndexesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5018), tables: &DUCKDB_TABLES }),
        "InetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5019), tables: &DUCKDB_TABLES }),
        "InfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5020), tables: &DUCKDB_TABLES }),
        "InfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5021), tables: &DUCKDB_TABLES }),
        "InheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5022), tables: &DUCKDB_TABLES }),
        "InheritsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5023), tables: &DUCKDB_TABLES }),
        "InitializeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5024), tables: &DUCKDB_TABLES }),
        "InitiallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5025), tables: &DUCKDB_TABLES }),
        "InlineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5026), tables: &DUCKDB_TABLES }),
        "InnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5027), tables: &DUCKDB_TABLES }),
        "InoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5028), tables: &DUCKDB_TABLES }),
        "InputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5029), tables: &DUCKDB_TABLES }),
        "InsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5030), tables: &DUCKDB_TABLES }),
        "InsertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5031), tables: &DUCKDB_TABLES }),
        "InsertStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5032), tables: &DUCKDB_TABLES }),
        "Insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5073), tables: &DUCKDB_TABLES }),
        "InsteadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5074), tables: &DUCKDB_TABLES }),
        "Int1KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5075), tables: &DUCKDB_TABLES }),
        "Int2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5076), tables: &DUCKDB_TABLES }),
        "Int3KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5077), tables: &DUCKDB_TABLES }),
        "Int4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5078), tables: &DUCKDB_TABLES }),
        "Int4rangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5079), tables: &DUCKDB_TABLES }),
        "Int8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5080), tables: &DUCKDB_TABLES }),
        "Int8rangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5081), tables: &DUCKDB_TABLES }),
        "IntKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5082), tables: &DUCKDB_TABLES }),
        "IntegerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5083), tables: &DUCKDB_TABLES }),
        "IntegrationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5084), tables: &DUCKDB_TABLES }),
        "IntegrationsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5085), tables: &DUCKDB_TABLES }),
        "IntersectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5086), tables: &DUCKDB_TABLES }),
        "IntervalExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5087), tables: &DUCKDB_TABLES }),
        "IntervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5104), tables: &DUCKDB_TABLES }),
        "IntervalUnitsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5105), tables: &DUCKDB_TABLES }),
        "IntoClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5112), tables: &DUCKDB_TABLES }),
        "IntoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5120), tables: &DUCKDB_TABLES }),
        "InvokerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5121), tables: &DUCKDB_TABLES }),
        "IsClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(5122), tables: &DUCKDB_TABLES }),
        "IsDistinctFromGrammar" => Some(RootGrammar { grammar_id: GrammarId(5129), tables: &DUCKDB_TABLES }),
        "IsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5134), tables: &DUCKDB_TABLES }),
        "IsNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(5135), tables: &DUCKDB_TABLES }),
        "Is_superuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5136), tables: &DUCKDB_TABLES }),
        "Is_templateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5137), tables: &DUCKDB_TABLES }),
        "IsamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5138), tables: &DUCKDB_TABLES }),
        "IsnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5139), tables: &DUCKDB_TABLES }),
        "IsolationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5140), tables: &DUCKDB_TABLES }),
        "IterateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5141), tables: &DUCKDB_TABLES }),
        "JoinClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5142), tables: &DUCKDB_TABLES }),
        "JoinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5164), tables: &DUCKDB_TABLES }),
        "JoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5165), tables: &DUCKDB_TABLES }),
        "JoinLikeClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(5167), tables: &DUCKDB_TABLES }),
        "JoinOnConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(5172), tables: &DUCKDB_TABLES }),
        "JoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5179), tables: &DUCKDB_TABLES }),
        "JoinUsingConditionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5187), tables: &DUCKDB_TABLES }),
        "JsonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5192), tables: &DUCKDB_TABLES }),
        "JsonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(5193), tables: &DUCKDB_TABLES }),
        "JsonTestGrammar" => Some(RootGrammar { grammar_id: GrammarId(5194), tables: &DUCKDB_TABLES }),
        "JsonTypeGrammar" => Some(RootGrammar { grammar_id: GrammarId(5198), tables: &DUCKDB_TABLES }),
        "JsonUniqueKeysGrammar" => Some(RootGrammar { grammar_id: GrammarId(5203), tables: &DUCKDB_TABLES }),
        "JsonbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5210), tables: &DUCKDB_TABLES }),
        "KeyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5211), tables: &DUCKDB_TABLES }),
        "KeysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5212), tables: &DUCKDB_TABLES }),
        "KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5213), tables: &DUCKDB_TABLES }),
        "KillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5214), tables: &DUCKDB_TABLES }),
        "LabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5215), tables: &DUCKDB_TABLES }),
        "LambdaArrowSegment" => Some(RootGrammar { grammar_id: GrammarId(5216), tables: &DUCKDB_TABLES }),
        "LambdaExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5217), tables: &DUCKDB_TABLES }),
        "LambdaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5235), tables: &DUCKDB_TABLES }),
        "LancompilerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5236), tables: &DUCKDB_TABLES }),
        "LanguageClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5237), tables: &DUCKDB_TABLES }),
        "LanguageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5242), tables: &DUCKDB_TABLES }),
        "LargeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5243), tables: &DUCKDB_TABLES }),
        "LastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5244), tables: &DUCKDB_TABLES }),
        "Last_insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5245), tables: &DUCKDB_TABLES }),
        "LateralColumnAliasExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5246), tables: &DUCKDB_TABLES }),
        "LateralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5247), tables: &DUCKDB_TABLES }),
        "Lc_collateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5248), tables: &DUCKDB_TABLES }),
        "Lc_ctypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5249), tables: &DUCKDB_TABLES }),
        "LeadingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5250), tables: &DUCKDB_TABLES }),
        "LeakproofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5251), tables: &DUCKDB_TABLES }),
        "LeastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5252), tables: &DUCKDB_TABLES }),
        "LeaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5253), tables: &DUCKDB_TABLES }),
        "LeftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5254), tables: &DUCKDB_TABLES }),
        "LeftargKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5255), tables: &DUCKDB_TABLES }),
        "LessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5256), tables: &DUCKDB_TABLES }),
        "LessThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(5257), tables: &DUCKDB_TABLES }),
        "LessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(5260), tables: &DUCKDB_TABLES }),
        "LevelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5261), tables: &DUCKDB_TABLES }),
        "LibcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5262), tables: &DUCKDB_TABLES }),
        "LikeExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5263), tables: &DUCKDB_TABLES }),
        "LikeGrammar" => Some(RootGrammar { grammar_id: GrammarId(5271), tables: &DUCKDB_TABLES }),
        "LikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5278), tables: &DUCKDB_TABLES }),
        "LikeOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(5279), tables: &DUCKDB_TABLES }),
        "LikeOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(5280), tables: &DUCKDB_TABLES }),
        "LimitClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5294), tables: &DUCKDB_TABLES }),
        "LimitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5311), tables: &DUCKDB_TABLES }),
        "LineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5312), tables: &DUCKDB_TABLES }),
        "LinenoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5313), tables: &DUCKDB_TABLES }),
        "LinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5314), tables: &DUCKDB_TABLES }),
        "LinestringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5315), tables: &DUCKDB_TABLES }),
        "LinestringmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5316), tables: &DUCKDB_TABLES }),
        "LinestringzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5317), tables: &DUCKDB_TABLES }),
        "LinestringzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5318), tables: &DUCKDB_TABLES }),
        "ListComprehensionExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5319), tables: &DUCKDB_TABLES }),
        "ListComprehensionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5328), tables: &DUCKDB_TABLES }),
        "ListKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5329), tables: &DUCKDB_TABLES }),
        "ListenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5330), tables: &DUCKDB_TABLES }),
        "ListenStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5331), tables: &DUCKDB_TABLES }),
        "LiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(5334), tables: &DUCKDB_TABLES }),
        "LiteralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5346), tables: &DUCKDB_TABLES }),
        "LiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5347), tables: &DUCKDB_TABLES }),
        "LoadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5348), tables: &DUCKDB_TABLES }),
        "LoadStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5349), tables: &DUCKDB_TABLES }),
        "LocalAliasSegment" => Some(RootGrammar { grammar_id: GrammarId(5352), tables: &DUCKDB_TABLES }),
        "LocalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5353), tables: &DUCKDB_TABLES }),
        "LocaleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5354), tables: &DUCKDB_TABLES }),
        "LocaltimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5355), tables: &DUCKDB_TABLES }),
        "LocaltimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5356), tables: &DUCKDB_TABLES }),
        "LocationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5357), tables: &DUCKDB_TABLES }),
        "LockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5358), tables: &DUCKDB_TABLES }),
        "LockTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5359), tables: &DUCKDB_TABLES }),
        "LockedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5394), tables: &DUCKDB_TABLES }),
        "LocksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5395), tables: &DUCKDB_TABLES }),
        "LoggedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5396), tables: &DUCKDB_TABLES }),
        "LoginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5397), tables: &DUCKDB_TABLES }),
        "LogsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5398), tables: &DUCKDB_TABLES }),
        "LongKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5399), tables: &DUCKDB_TABLES }),
        "LongblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5400), tables: &DUCKDB_TABLES }),
        "LongtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5401), tables: &DUCKDB_TABLES }),
        "LoopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5402), tables: &DUCKDB_TABLES }),
        "Low_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5403), tables: &DUCKDB_TABLES }),
        "LsegKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5404), tables: &DUCKDB_TABLES }),
        "MLTableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5405), tables: &DUCKDB_TABLES }),
        "Macaddr8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5417), tables: &DUCKDB_TABLES }),
        "MacaddrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5418), tables: &DUCKDB_TABLES }),
        "MacroKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5419), tables: &DUCKDB_TABLES }),
        "MainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5420), tables: &DUCKDB_TABLES }),
        "ManageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5421), tables: &DUCKDB_TABLES }),
        "MapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5422), tables: &DUCKDB_TABLES }),
        "MapLiteralElementSegment" => Some(RootGrammar { grammar_id: GrammarId(5423), tables: &DUCKDB_TABLES }),
        "MapLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5427), tables: &DUCKDB_TABLES }),
        "MapTypeSchemaSegment" => Some(RootGrammar { grammar_id: GrammarId(5433), tables: &DUCKDB_TABLES }),
        "MapTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(5436), tables: &DUCKDB_TABLES }),
        "MappingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5439), tables: &DUCKDB_TABLES }),
        "MaskingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5440), tables: &DUCKDB_TABLES }),
        "MatchConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(5441), tables: &DUCKDB_TABLES }),
        "MatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5442), tables: &DUCKDB_TABLES }),
        "MatchedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5443), tables: &DUCKDB_TABLES }),
        "MaterializedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5444), tables: &DUCKDB_TABLES }),
        "Max_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5445), tables: &DUCKDB_TABLES }),
        "MaxextentsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5446), tables: &DUCKDB_TABLES }),
        "MaxvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5447), tables: &DUCKDB_TABLES }),
        "McvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5448), tables: &DUCKDB_TABLES }),
        "MediumblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5449), tables: &DUCKDB_TABLES }),
        "MediumintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5450), tables: &DUCKDB_TABLES }),
        "MediumtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5451), tables: &DUCKDB_TABLES }),
        "MergeDeleteClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5452), tables: &DUCKDB_TABLES }),
        "MergeInsertClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5453), tables: &DUCKDB_TABLES }),
        "MergeIntoLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(5457), tables: &DUCKDB_TABLES }),
        "MergeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5460), tables: &DUCKDB_TABLES }),
        "MergeMatchSegment" => Some(RootGrammar { grammar_id: GrammarId(5461), tables: &DUCKDB_TABLES }),
        "MergeMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5465), tables: &DUCKDB_TABLES }),
        "MergeNotMatchedBySourceClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5478), tables: &DUCKDB_TABLES }),
        "MergeNotMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5494), tables: &DUCKDB_TABLES }),
        "MergeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5510), tables: &DUCKDB_TABLES }),
        "MergeUpdateClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5530), tables: &DUCKDB_TABLES }),
        "MergesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5533), tables: &DUCKDB_TABLES }),
        "MetaCommandQueryBufferSegment" => Some(RootGrammar { grammar_id: GrammarId(5534), tables: &DUCKDB_TABLES }),
        "MetaCommandQueryBufferStatement" => Some(RootGrammar { grammar_id: GrammarId(5535), tables: &DUCKDB_TABLES }),
        "MethodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5540), tables: &DUCKDB_TABLES }),
        "MiddleintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5541), tables: &DUCKDB_TABLES }),
        "MillisecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5542), tables: &DUCKDB_TABLES }),
        "Min_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5543), tables: &DUCKDB_TABLES }),
        "MinusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5544), tables: &DUCKDB_TABLES }),
        "MinusSegment" => Some(RootGrammar { grammar_id: GrammarId(5545), tables: &DUCKDB_TABLES }),
        "MinuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5546), tables: &DUCKDB_TABLES }),
        "Minute_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5547), tables: &DUCKDB_TABLES }),
        "Minute_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5548), tables: &DUCKDB_TABLES }),
        "MinvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5549), tables: &DUCKDB_TABLES }),
        "MlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5550), tables: &DUCKDB_TABLES }),
        "MlslabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5551), tables: &DUCKDB_TABLES }),
        "ModeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5552), tables: &DUCKDB_TABLES }),
        "ModelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5553), tables: &DUCKDB_TABLES }),
        "ModifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5554), tables: &DUCKDB_TABLES }),
        "ModuloSegment" => Some(RootGrammar { grammar_id: GrammarId(5555), tables: &DUCKDB_TABLES }),
        "ModulusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5556), tables: &DUCKDB_TABLES }),
        "MoneyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5557), tables: &DUCKDB_TABLES }),
        "MonitorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5558), tables: &DUCKDB_TABLES }),
        "MonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5559), tables: &DUCKDB_TABLES }),
        "MonthnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5560), tables: &DUCKDB_TABLES }),
        "MoveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5561), tables: &DUCKDB_TABLES }),
        "MulticurveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5562), tables: &DUCKDB_TABLES }),
        "MultilineConcatenateDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(5563), tables: &DUCKDB_TABLES }),
        "MultilineConcatenateNewline" => Some(RootGrammar { grammar_id: GrammarId(5565), tables: &DUCKDB_TABLES }),
        "MultilinestringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5566), tables: &DUCKDB_TABLES }),
        "MultilinestringmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5567), tables: &DUCKDB_TABLES }),
        "MultilinestringzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5568), tables: &DUCKDB_TABLES }),
        "MultilinestringzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5569), tables: &DUCKDB_TABLES }),
        "MultiplySegment" => Some(RootGrammar { grammar_id: GrammarId(5570), tables: &DUCKDB_TABLES }),
        "MultipointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5571), tables: &DUCKDB_TABLES }),
        "MultipointmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5572), tables: &DUCKDB_TABLES }),
        "MultipointzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5573), tables: &DUCKDB_TABLES }),
        "MultipointzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5574), tables: &DUCKDB_TABLES }),
        "MultipolygonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5575), tables: &DUCKDB_TABLES }),
        "MultipolygonmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5576), tables: &DUCKDB_TABLES }),
        "MultipolygonzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5577), tables: &DUCKDB_TABLES }),
        "MultipolygonzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5578), tables: &DUCKDB_TABLES }),
        "MultisurfaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5579), tables: &DUCKDB_TABLES }),
        "MyisamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5580), tables: &DUCKDB_TABLES }),
        "NakedIdentifierFullSegment" => Some(RootGrammar { grammar_id: GrammarId(5581), tables: &DUCKDB_TABLES }),
        "NakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(5582), tables: &DUCKDB_TABLES }),
        "NameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5583), tables: &DUCKDB_TABLES }),
        "NamedArgumentSegment" => Some(RootGrammar { grammar_id: GrammarId(5584), tables: &DUCKDB_TABLES }),
        "NamedWindowExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5591), tables: &DUCKDB_TABLES }),
        "NamedWindowSegment" => Some(RootGrammar { grammar_id: GrammarId(5598), tables: &DUCKDB_TABLES }),
        "NamesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5602), tables: &DUCKDB_TABLES }),
        "NanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5603), tables: &DUCKDB_TABLES }),
        "NanLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5604), tables: &DUCKDB_TABLES }),
        "NationalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5605), tables: &DUCKDB_TABLES }),
        "NaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5606), tables: &DUCKDB_TABLES }),
        "NaturalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5609), tables: &DUCKDB_TABLES }),
        "NcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5610), tables: &DUCKDB_TABLES }),
        "NdistinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5611), tables: &DUCKDB_TABLES }),
        "NegativeSegment" => Some(RootGrammar { grammar_id: GrammarId(5612), tables: &DUCKDB_TABLES }),
        "NegatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5613), tables: &DUCKDB_TABLES }),
        "NestedJoinGrammar" => Some(RootGrammar { grammar_id: GrammarId(5614), tables: &DUCKDB_TABLES }),
        "NewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5615), tables: &DUCKDB_TABLES }),
        "NewlineSegment" => Some(RootGrammar { grammar_id: GrammarId(5616), tables: &DUCKDB_TABLES }),
        "NextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5617), tables: &DUCKDB_TABLES }),
        "NfcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5618), tables: &DUCKDB_TABLES }),
        "NfdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5619), tables: &DUCKDB_TABLES }),
        "NfkcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5620), tables: &DUCKDB_TABLES }),
        "NfkdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5621), tables: &DUCKDB_TABLES }),
        "NoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5622), tables: &DUCKDB_TABLES }),
        "No_write_to_binlogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5623), tables: &DUCKDB_TABLES }),
        "NoauditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5624), tables: &DUCKDB_TABLES }),
        "NobypassrlsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5625), tables: &DUCKDB_TABLES }),
        "NocacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5626), tables: &DUCKDB_TABLES }),
        "NocheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5627), tables: &DUCKDB_TABLES }),
        "NocompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5628), tables: &DUCKDB_TABLES }),
        "NocreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5629), tables: &DUCKDB_TABLES }),
        "NocreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5630), tables: &DUCKDB_TABLES }),
        "NocreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5631), tables: &DUCKDB_TABLES }),
        "NocycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5632), tables: &DUCKDB_TABLES }),
        "NoinheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5633), tables: &DUCKDB_TABLES }),
        "NologinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5634), tables: &DUCKDB_TABLES }),
        "NonSetSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(5635), tables: &DUCKDB_TABLES }),
        "NonStandardJoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5647), tables: &DUCKDB_TABLES }),
        "NonWithNonSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(5656), tables: &DUCKDB_TABLES }),
        "NonWithSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(5657), tables: &DUCKDB_TABLES }),
        "NonclusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5666), tables: &DUCKDB_TABLES }),
        "NoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5667), tables: &DUCKDB_TABLES }),
        "NoorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5668), tables: &DUCKDB_TABLES }),
        "NoreplicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5669), tables: &DUCKDB_TABLES }),
        "NormalizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5670), tables: &DUCKDB_TABLES }),
        "NormalizedGrammar" => Some(RootGrammar { grammar_id: GrammarId(5671), tables: &DUCKDB_TABLES }),
        "NormalizedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5678), tables: &DUCKDB_TABLES }),
        "NosuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5679), tables: &DUCKDB_TABLES }),
        "NotEnforcedGrammar" => Some(RootGrammar { grammar_id: GrammarId(5680), tables: &DUCKDB_TABLES }),
        "NotEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(5681), tables: &DUCKDB_TABLES }),
        "NotExtendLeftSegment" => Some(RootGrammar { grammar_id: GrammarId(5688), tables: &DUCKDB_TABLES }),
        "NotExtendRightSegment" => Some(RootGrammar { grammar_id: GrammarId(5691), tables: &DUCKDB_TABLES }),
        "NotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5694), tables: &DUCKDB_TABLES }),
        "NotNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(5695), tables: &DUCKDB_TABLES }),
        "NotOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5696), tables: &DUCKDB_TABLES }),
        "NothingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5697), tables: &DUCKDB_TABLES }),
        "NotifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5698), tables: &DUCKDB_TABLES }),
        "NotifyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5699), tables: &DUCKDB_TABLES }),
        "NotnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5705), tables: &DUCKDB_TABLES }),
        "NowaitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5706), tables: &DUCKDB_TABLES }),
        "NullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5707), tables: &DUCKDB_TABLES }),
        "NullLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5708), tables: &DUCKDB_TABLES }),
        "NullifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5709), tables: &DUCKDB_TABLES }),
        "NullsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5710), tables: &DUCKDB_TABLES }),
        "NumericKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5711), tables: &DUCKDB_TABLES }),
        "NumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5712), tables: &DUCKDB_TABLES }),
        "NumrangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5713), tables: &DUCKDB_TABLES }),
        "ObjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5714), tables: &DUCKDB_TABLES }),
        "ObjectLiteralElementSegment" => Some(RootGrammar { grammar_id: GrammarId(5715), tables: &DUCKDB_TABLES }),
        "ObjectLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5721), tables: &DUCKDB_TABLES }),
        "ObjectReferenceDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(5724), tables: &DUCKDB_TABLES }),
        "ObjectReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3597), tables: &DUCKDB_TABLES }),
        "ObjectReferenceTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5729), tables: &DUCKDB_TABLES }),
        "ObjectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5741), tables: &DUCKDB_TABLES }),
        "OfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5742), tables: &DUCKDB_TABLES }),
        "OffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5743), tables: &DUCKDB_TABLES }),
        "OfflineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5744), tables: &DUCKDB_TABLES }),
        "OffsetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5745), tables: &DUCKDB_TABLES }),
        "OffsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5750), tables: &DUCKDB_TABLES }),
        "OffsetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5751), tables: &DUCKDB_TABLES }),
        "OidsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5752), tables: &DUCKDB_TABLES }),
        "OldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5753), tables: &DUCKDB_TABLES }),
        "OnKeywordAsIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(5754), tables: &DUCKDB_TABLES }),
        "OnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5755), tables: &DUCKDB_TABLES }),
        "OnlineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5756), tables: &DUCKDB_TABLES }),
        "OnlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5757), tables: &DUCKDB_TABLES }),
        "OpendatasourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5758), tables: &DUCKDB_TABLES }),
        "OpenqueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5759), tables: &DUCKDB_TABLES }),
        "OpenrowsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5760), tables: &DUCKDB_TABLES }),
        "OpenxmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5761), tables: &DUCKDB_TABLES }),
        "OperateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5762), tables: &DUCKDB_TABLES }),
        "OperationClassReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3597), tables: &DUCKDB_TABLES }),
        "OperationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5763), tables: &DUCKDB_TABLES }),
        "OperatorClassReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3597), tables: &DUCKDB_TABLES }),
        "OperatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5764), tables: &DUCKDB_TABLES }),
        "OptimizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5765), tables: &DUCKDB_TABLES }),
        "OptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5766), tables: &DUCKDB_TABLES }),
        "OptionallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5767), tables: &DUCKDB_TABLES }),
        "OptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5768), tables: &DUCKDB_TABLES }),
        "OptionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5773), tables: &DUCKDB_TABLES }),
        "OptionsListGrammar" => Some(RootGrammar { grammar_id: GrammarId(5774), tables: &DUCKDB_TABLES }),
        "OrIgnoreGrammar" => Some(RootGrammar { grammar_id: GrammarId(5779), tables: &DUCKDB_TABLES }),
        "OrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5782), tables: &DUCKDB_TABLES }),
        "OrOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5783), tables: &DUCKDB_TABLES }),
        "OrReplaceGrammar" => Some(RootGrammar { grammar_id: GrammarId(5784), tables: &DUCKDB_TABLES }),
        "OrderByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5787), tables: &DUCKDB_TABLES }),
        "OrderByClauseTerminators" => Some(RootGrammar { grammar_id: GrammarId(5806), tables: &DUCKDB_TABLES }),
        "OrderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5817), tables: &DUCKDB_TABLES }),
        "OrderNoOrderGrammar" => Some(RootGrammar { grammar_id: GrammarId(5818), tables: &DUCKDB_TABLES }),
        "OrdinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5821), tables: &DUCKDB_TABLES }),
        "OthersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5822), tables: &DUCKDB_TABLES }),
        "OutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5823), tables: &DUCKDB_TABLES }),
        "OuterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5824), tables: &DUCKDB_TABLES }),
        "OutfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5825), tables: &DUCKDB_TABLES }),
        "OverClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5826), tables: &DUCKDB_TABLES }),
        "OverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5833), tables: &DUCKDB_TABLES }),
        "OverlapSegment" => Some(RootGrammar { grammar_id: GrammarId(5834), tables: &DUCKDB_TABLES }),
        "OverlapsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5837), tables: &DUCKDB_TABLES }),
        "OverlapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5869), tables: &DUCKDB_TABLES }),
        "OverlayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5870), tables: &DUCKDB_TABLES }),
        "OverridingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5871), tables: &DUCKDB_TABLES }),
        "OverwriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5872), tables: &DUCKDB_TABLES }),
        "Overwrite_or_ignoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5873), tables: &DUCKDB_TABLES }),
        "OwnedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5874), tables: &DUCKDB_TABLES }),
        "OwnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5875), tables: &DUCKDB_TABLES }),
        "OwnershipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5876), tables: &DUCKDB_TABLES }),
        "Pack_keysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5877), tables: &DUCKDB_TABLES }),
        "ParallelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5878), tables: &DUCKDB_TABLES }),
        "ParameterNameSegment" => Some(RootGrammar { grammar_id: GrammarId(5879), tables: &DUCKDB_TABLES }),
        "ParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(5880), tables: &DUCKDB_TABLES }),
        "ParametersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5881), tables: &DUCKDB_TABLES }),
        "Parquet_versionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5882), tables: &DUCKDB_TABLES }),
        "ParserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5883), tables: &DUCKDB_TABLES }),
        "PartialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5884), tables: &DUCKDB_TABLES }),
        "PartitionBoundSpecSegment" => Some(RootGrammar { grammar_id: GrammarId(5885), tables: &DUCKDB_TABLES }),
        "PartitionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5915), tables: &DUCKDB_TABLES }),
        "PartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5922), tables: &DUCKDB_TABLES }),
        "Partition_byKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5923), tables: &DUCKDB_TABLES }),
        "PassingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5924), tables: &DUCKDB_TABLES }),
        "PasswordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5925), tables: &DUCKDB_TABLES }),
        "PathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5926), tables: &DUCKDB_TABLES }),
        "PathSegment" => Some(RootGrammar { grammar_id: GrammarId(5927), tables: &DUCKDB_TABLES }),
        "PatternMatchingGrammar" => Some(RootGrammar { grammar_id: GrammarId(5934), tables: &DUCKDB_TABLES }),
        "PctfreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5935), tables: &DUCKDB_TABLES }),
        "PercentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5936), tables: &DUCKDB_TABLES }),
        "PermissiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5937), tables: &DUCKDB_TABLES }),
        "PgTrgmOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(5938), tables: &DUCKDB_TABLES }),
        "Pg_lsnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5939), tables: &DUCKDB_TABLES }),
        "PgvectorOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(5940), tables: &DUCKDB_TABLES }),
        "PipeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5941), tables: &DUCKDB_TABLES }),
        "PipeSegment" => Some(RootGrammar { grammar_id: GrammarId(5942), tables: &DUCKDB_TABLES }),
        "PivotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5943), tables: &DUCKDB_TABLES }),
        "Pivot_longerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5944), tables: &DUCKDB_TABLES }),
        "Pivot_widerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5945), tables: &DUCKDB_TABLES }),
        "PlacingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5946), tables: &DUCKDB_TABLES }),
        "PlainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5947), tables: &DUCKDB_TABLES }),
        "PlansKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5948), tables: &DUCKDB_TABLES }),
        "PlusSegment" => Some(RootGrammar { grammar_id: GrammarId(5949), tables: &DUCKDB_TABLES }),
        "PointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5950), tables: &DUCKDB_TABLES }),
        "PointmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5951), tables: &DUCKDB_TABLES }),
        "PointzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5952), tables: &DUCKDB_TABLES }),
        "PointzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5953), tables: &DUCKDB_TABLES }),
        "PolicyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5954), tables: &DUCKDB_TABLES }),
        "PolygonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5955), tables: &DUCKDB_TABLES }),
        "PolygonmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5956), tables: &DUCKDB_TABLES }),
        "PolygonzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5957), tables: &DUCKDB_TABLES }),
        "PolygonzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5958), tables: &DUCKDB_TABLES }),
        "PolyhedralsurfaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5959), tables: &DUCKDB_TABLES }),
        "PositionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5960), tables: &DUCKDB_TABLES }),
        "PositionalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5961), tables: &DUCKDB_TABLES }),
        "PositiveSegment" => Some(RootGrammar { grammar_id: GrammarId(5962), tables: &DUCKDB_TABLES }),
        "PostFunctionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5963), tables: &DUCKDB_TABLES }),
        "PostTableExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5967), tables: &DUCKDB_TABLES }),
        "PostfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5968), tables: &DUCKDB_TABLES }),
        "PostgisOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(5969), tables: &DUCKDB_TABLES }),
        "PostgresCopyStdinDataSegment" => Some(RootGrammar { grammar_id: GrammarId(5970), tables: &DUCKDB_TABLES }),
        "PostgresCopyStdinDataStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5971), tables: &DUCKDB_TABLES }),
        "PowerOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(5973), tables: &DUCKDB_TABLES }),
        "PreTableFunctionKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5974), tables: &DUCKDB_TABLES }),
        "PrecedingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5976), tables: &DUCKDB_TABLES }),
        "PrecisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5977), tables: &DUCKDB_TABLES }),
        "PrefixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5978), tables: &DUCKDB_TABLES }),
        "PreorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5979), tables: &DUCKDB_TABLES }),
        "PrepareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5980), tables: &DUCKDB_TABLES }),
        "PrepareStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5981), tables: &DUCKDB_TABLES }),
        "PreparedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5991), tables: &DUCKDB_TABLES }),
        "PreserveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5992), tables: &DUCKDB_TABLES }),
        "PrimaryKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(5993), tables: &DUCKDB_TABLES }),
        "PrimaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5996), tables: &DUCKDB_TABLES }),
        "PrintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5997), tables: &DUCKDB_TABLES }),
        "PriorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5998), tables: &DUCKDB_TABLES }),
        "PrivilegesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5999), tables: &DUCKDB_TABLES }),
        "ProcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6000), tables: &DUCKDB_TABLES }),
        "ProceduralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6001), tables: &DUCKDB_TABLES }),
        "ProcedureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6002), tables: &DUCKDB_TABLES }),
        "ProceduresKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6003), tables: &DUCKDB_TABLES }),
        "ProcessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6004), tables: &DUCKDB_TABLES }),
        "Process_toastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6005), tables: &DUCKDB_TABLES }),
        "ProcesslistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6006), tables: &DUCKDB_TABLES }),
        "ProgramKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6007), tables: &DUCKDB_TABLES }),
        "PropertiesNakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(6008), tables: &DUCKDB_TABLES }),
        "ProviderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6009), tables: &DUCKDB_TABLES }),
        "PsqlCopyMetaCommandSegment" => Some(RootGrammar { grammar_id: GrammarId(6010), tables: &DUCKDB_TABLES }),
        "PsqlCopyMetaCommandStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6011), tables: &DUCKDB_TABLES }),
        "PsqlSetMetaCommandSegment" => Some(RootGrammar { grammar_id: GrammarId(6013), tables: &DUCKDB_TABLES }),
        "PsqlSetMetaCommandStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6014), tables: &DUCKDB_TABLES }),
        "PsqlVariableColonSegment" => Some(RootGrammar { grammar_id: GrammarId(6016), tables: &DUCKDB_TABLES }),
        "PsqlVariableGrammar" => Some(RootGrammar { grammar_id: GrammarId(6017), tables: &DUCKDB_TABLES }),
        "PublicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6025), tables: &DUCKDB_TABLES }),
        "PublicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6026), tables: &DUCKDB_TABLES }),
        "PublicationObjectsSegment" => Some(RootGrammar { grammar_id: GrammarId(6027), tables: &DUCKDB_TABLES }),
        "PublicationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(6050), tables: &DUCKDB_TABLES }),
        "PublicationTableSegment" => Some(RootGrammar { grammar_id: GrammarId(6051), tables: &DUCKDB_TABLES }),
        "PurgeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6058), tables: &DUCKDB_TABLES }),
        "QualifiedNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(6059), tables: &DUCKDB_TABLES }),
        "QualifiedOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(6062), tables: &DUCKDB_TABLES }),
        "QualifiedParameterNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(6070), tables: &DUCKDB_TABLES }),
        "QualifyClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6075), tables: &DUCKDB_TABLES }),
        "QualifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6080), tables: &DUCKDB_TABLES }),
        "QuantifiedComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(6081), tables: &DUCKDB_TABLES }),
        "QuarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6089), tables: &DUCKDB_TABLES }),
        "QuoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6090), tables: &DUCKDB_TABLES }),
        "QuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(6091), tables: &DUCKDB_TABLES }),
        "QuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(6092), tables: &DUCKDB_TABLES }),
        "Raid0KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6119), tables: &DUCKDB_TABLES }),
        "RaiserrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6120), tables: &DUCKDB_TABLES }),
        "RangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6121), tables: &DUCKDB_TABLES }),
        "RawEqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(6122), tables: &DUCKDB_TABLES }),
        "RawGreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(6123), tables: &DUCKDB_TABLES }),
        "RawKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6124), tables: &DUCKDB_TABLES }),
        "RawLessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(6125), tables: &DUCKDB_TABLES }),
        "RawNotSegment" => Some(RootGrammar { grammar_id: GrammarId(6126), tables: &DUCKDB_TABLES }),
        "ReadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6127), tables: &DUCKDB_TABLES }),
        "ReadtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6128), tables: &DUCKDB_TABLES }),
        "RealKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6129), tables: &DUCKDB_TABLES }),
        "ReassignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6130), tables: &DUCKDB_TABLES }),
        "ReassignOwnedStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6131), tables: &DUCKDB_TABLES }),
        "RecheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6147), tables: &DUCKDB_TABLES }),
        "ReconfigureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6148), tables: &DUCKDB_TABLES }),
        "RecursiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6149), tables: &DUCKDB_TABLES }),
        "RefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6150), tables: &DUCKDB_TABLES }),
        "ReferenceDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(6151), tables: &DUCKDB_TABLES }),
        "ReferenceMatchGrammar" => Some(RootGrammar { grammar_id: GrammarId(6165), tables: &DUCKDB_TABLES }),
        "Reference_usageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6171), tables: &DUCKDB_TABLES }),
        "ReferencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6172), tables: &DUCKDB_TABLES }),
        "ReferencingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6173), tables: &DUCKDB_TABLES }),
        "ReferentialActionGrammar" => Some(RootGrammar { grammar_id: GrammarId(6174), tables: &DUCKDB_TABLES }),
        "RefreshKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6188), tables: &DUCKDB_TABLES }),
        "RefreshMaterializedViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6189), tables: &DUCKDB_TABLES }),
        "RegexpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6196), tables: &DUCKDB_TABLES }),
        "ReindexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6197), tables: &DUCKDB_TABLES }),
        "ReindexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6198), tables: &DUCKDB_TABLES }),
        "RelationOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(6231), tables: &DUCKDB_TABLES }),
        "RelationOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(6239), tables: &DUCKDB_TABLES }),
        "RelativeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6242), tables: &DUCKDB_TABLES }),
        "ReleaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6243), tables: &DUCKDB_TABLES }),
        "ReloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6244), tables: &DUCKDB_TABLES }),
        "RemainderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6245), tables: &DUCKDB_TABLES }),
        "RenameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6246), tables: &DUCKDB_TABLES }),
        "RepeatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6247), tables: &DUCKDB_TABLES }),
        "RepeatableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6248), tables: &DUCKDB_TABLES }),
        "ReplaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6249), tables: &DUCKDB_TABLES }),
        "ReplicaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6250), tables: &DUCKDB_TABLES }),
        "ReplicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6251), tables: &DUCKDB_TABLES }),
        "RequireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6252), tables: &DUCKDB_TABLES }),
        "ReservoirKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6253), tables: &DUCKDB_TABLES }),
        "ResetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6254), tables: &DUCKDB_TABLES }),
        "ResetSessionAuthorizationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6255), tables: &DUCKDB_TABLES }),
        "ResetStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6259), tables: &DUCKDB_TABLES }),
        "ResignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6265), tables: &DUCKDB_TABLES }),
        "ResourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6266), tables: &DUCKDB_TABLES }),
        "RespectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6267), tables: &DUCKDB_TABLES }),
        "RestartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6268), tables: &DUCKDB_TABLES }),
        "RestrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6269), tables: &DUCKDB_TABLES }),
        "RestrictedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6270), tables: &DUCKDB_TABLES }),
        "RestrictiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6271), tables: &DUCKDB_TABLES }),
        "RetrieveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6272), tables: &DUCKDB_TABLES }),
        "ReturnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6273), tables: &DUCKDB_TABLES }),
        "ReturningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6274), tables: &DUCKDB_TABLES }),
        "ReturnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6275), tables: &DUCKDB_TABLES }),
        "RevokeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6276), tables: &DUCKDB_TABLES }),
        "RevokeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6277), tables: &DUCKDB_TABLES }),
        "RightArrowSegment" => Some(RootGrammar { grammar_id: GrammarId(6300), tables: &DUCKDB_TABLES }),
        "RightKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6301), tables: &DUCKDB_TABLES }),
        "RightargKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6302), tables: &DUCKDB_TABLES }),
        "RlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6303), tables: &DUCKDB_TABLES }),
        "RoleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6304), tables: &DUCKDB_TABLES }),
        "RoleReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(6305), tables: &DUCKDB_TABLES }),
        "RolesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6306), tables: &DUCKDB_TABLES }),
        "RollbackKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6307), tables: &DUCKDB_TABLES }),
        "RollupFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(6308), tables: &DUCKDB_TABLES }),
        "RollupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6309), tables: &DUCKDB_TABLES }),
        "RoutineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6310), tables: &DUCKDB_TABLES }),
        "RoutinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6311), tables: &DUCKDB_TABLES }),
        "RowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6312), tables: &DUCKDB_TABLES }),
        "Row_group_sizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6313), tables: &DUCKDB_TABLES }),
        "Row_group_size_bytesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6314), tables: &DUCKDB_TABLES }),
        "RowcountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6315), tables: &DUCKDB_TABLES }),
        "RowguidcolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6316), tables: &DUCKDB_TABLES }),
        "RowidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6317), tables: &DUCKDB_TABLES }),
        "RownumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6318), tables: &DUCKDB_TABLES }),
        "RowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6319), tables: &DUCKDB_TABLES }),
        "RuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6320), tables: &DUCKDB_TABLES }),
        "SafeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6321), tables: &DUCKDB_TABLES }),
        "SampleExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(6322), tables: &DUCKDB_TABLES }),
        "SampleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6357), tables: &DUCKDB_TABLES }),
        "SamplePercentageGrammar" => Some(RootGrammar { grammar_id: GrammarId(6358), tables: &DUCKDB_TABLES }),
        "SampleRowCountGrammar" => Some(RootGrammar { grammar_id: GrammarId(6363), tables: &DUCKDB_TABLES }),
        "SamplingExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(6366), tables: &DUCKDB_TABLES }),
        "SaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6369), tables: &DUCKDB_TABLES }),
        "SavepointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6370), tables: &DUCKDB_TABLES }),
        "ScalarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6371), tables: &DUCKDB_TABLES }),
        "SchemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6372), tables: &DUCKDB_TABLES }),
        "SchemaReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3597), tables: &DUCKDB_TABLES }),
        "SchemasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6373), tables: &DUCKDB_TABLES }),
        "ScrollKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6374), tables: &DUCKDB_TABLES }),
        "SearchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6375), tables: &DUCKDB_TABLES }),
        "SecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6376), tables: &DUCKDB_TABLES }),
        "Second_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6377), tables: &DUCKDB_TABLES }),
        "SecurityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6378), tables: &DUCKDB_TABLES }),
        "SecurityLabelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6379), tables: &DUCKDB_TABLES }),
        "SelectClauseElementSegment" => Some(RootGrammar { grammar_id: GrammarId(6470), tables: &DUCKDB_TABLES }),
        "SelectClauseModifierSegment" => Some(RootGrammar { grammar_id: GrammarId(6476), tables: &DUCKDB_TABLES }),
        "SelectClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6485), tables: &DUCKDB_TABLES }),
        "SelectClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(6509), tables: &DUCKDB_TABLES }),
        "SelectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6522), tables: &DUCKDB_TABLES }),
        "SelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6523), tables: &DUCKDB_TABLES }),
        "SelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(6547), tables: &DUCKDB_TABLES }),
        "SemiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6557), tables: &DUCKDB_TABLES }),
        "SemiStructuredAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(6558), tables: &DUCKDB_TABLES }),
        "SemicolonSegment" => Some(RootGrammar { grammar_id: GrammarId(6567), tables: &DUCKDB_TABLES }),
        "SeparatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6568), tables: &DUCKDB_TABLES }),
        "SequenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6569), tables: &DUCKDB_TABLES }),
        "SequenceMaxValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(6570), tables: &DUCKDB_TABLES }),
        "SequenceMinValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(6577), tables: &DUCKDB_TABLES }),
        "SequenceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3597), tables: &DUCKDB_TABLES }),
        "SequencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6584), tables: &DUCKDB_TABLES }),
        "Serial2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6585), tables: &DUCKDB_TABLES }),
        "Serial4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6586), tables: &DUCKDB_TABLES }),
        "Serial8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6587), tables: &DUCKDB_TABLES }),
        "SerialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6588), tables: &DUCKDB_TABLES }),
        "SerializableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6589), tables: &DUCKDB_TABLES }),
        "ServerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6590), tables: &DUCKDB_TABLES }),
        "ServerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3597), tables: &DUCKDB_TABLES }),
        "Server_encodingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6591), tables: &DUCKDB_TABLES }),
        "Server_versionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6592), tables: &DUCKDB_TABLES }),
        "SessionInformationUserFunctionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(6593), tables: &DUCKDB_TABLES }),
        "SessionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6598), tables: &DUCKDB_TABLES }),
        "Session_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6599), tables: &DUCKDB_TABLES }),
        "SetClauseListSegment" => Some(RootGrammar { grammar_id: GrammarId(6600), tables: &DUCKDB_TABLES }),
        "SetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6604), tables: &DUCKDB_TABLES }),
        "SetConstraintsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6638), tables: &DUCKDB_TABLES }),
        "SetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(6648), tables: &DUCKDB_TABLES }),
        "SetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6657), tables: &DUCKDB_TABLES }),
        "SetOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(6658), tables: &DUCKDB_TABLES }),
        "SetSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6666), tables: &DUCKDB_TABLES }),
        "SetSessionAuthorizationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6671), tables: &DUCKDB_TABLES }),
        "SetStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6684), tables: &DUCKDB_TABLES }),
        "SetofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6714), tables: &DUCKDB_TABLES }),
        "SetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6715), tables: &DUCKDB_TABLES }),
        "SettingsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6716), tables: &DUCKDB_TABLES }),
        "SetuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6717), tables: &DUCKDB_TABLES }),
        "ShareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6718), tables: &DUCKDB_TABLES }),
        "SharesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6719), tables: &DUCKDB_TABLES }),
        "ShorthandCastSegment" => Some(RootGrammar { grammar_id: GrammarId(6720), tables: &DUCKDB_TABLES }),
        "ShowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6729), tables: &DUCKDB_TABLES }),
        "ShowStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6730), tables: &DUCKDB_TABLES }),
        "ShutdownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6740), tables: &DUCKDB_TABLES }),
        "SignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6741), tables: &DUCKDB_TABLES }),
        "SignedSegmentGrammar" => Some(RootGrammar { grammar_id: GrammarId(6742), tables: &DUCKDB_TABLES }),
        "SimilarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6745), tables: &DUCKDB_TABLES }),
        "SimpleGeometryGrammar" => Some(RootGrammar { grammar_id: GrammarId(6746), tables: &DUCKDB_TABLES }),
        "SimpleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6748), tables: &DUCKDB_TABLES }),
        "SimplifiedPivotExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(6749), tables: &DUCKDB_TABLES }),
        "SimplifiedUnpivotExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(6775), tables: &DUCKDB_TABLES }),
        "SingleIdentifierFullGrammar" => Some(RootGrammar { grammar_id: GrammarId(6798), tables: &DUCKDB_TABLES }),
        "SingleIdentifierGrammar" => Some(RootGrammar { grammar_id: GrammarId(6802), tables: &DUCKDB_TABLES }),
        "SingleIdentifierListSegment" => Some(RootGrammar { grammar_id: GrammarId(6806), tables: &DUCKDB_TABLES }),
        "SingleIdentifierWildcardSegment" => Some(RootGrammar { grammar_id: GrammarId(6808), tables: &DUCKDB_TABLES }),
        "SingleQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(6812), tables: &DUCKDB_TABLES }),
        "SizedArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(6813), tables: &DUCKDB_TABLES }),
        "SkipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6816), tables: &DUCKDB_TABLES }),
        "Skip_lockedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6817), tables: &DUCKDB_TABLES }),
        "SlashSegment" => Some(RootGrammar { grammar_id: GrammarId(6818), tables: &DUCKDB_TABLES }),
        "SliceSegment" => Some(RootGrammar { grammar_id: GrammarId(6819), tables: &DUCKDB_TABLES }),
        "SmallintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6820), tables: &DUCKDB_TABLES }),
        "SmallserialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6821), tables: &DUCKDB_TABLES }),
        "SnapshotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6822), tables: &DUCKDB_TABLES }),
        "SomeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6823), tables: &DUCKDB_TABLES }),
        "SonameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6824), tables: &DUCKDB_TABLES }),
        "SourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6825), tables: &DUCKDB_TABLES }),
        "SpatialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6826), tables: &DUCKDB_TABLES }),
        "SqlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6827), tables: &DUCKDB_TABLES }),
        "Sql_big_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6828), tables: &DUCKDB_TABLES }),
        "Sql_big_selectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6829), tables: &DUCKDB_TABLES }),
        "Sql_big_tablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6830), tables: &DUCKDB_TABLES }),
        "Sql_calc_found_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6831), tables: &DUCKDB_TABLES }),
        "Sql_log_offKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6832), tables: &DUCKDB_TABLES }),
        "Sql_log_updateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6833), tables: &DUCKDB_TABLES }),
        "Sql_low_priority_updatesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6834), tables: &DUCKDB_TABLES }),
        "Sql_select_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6835), tables: &DUCKDB_TABLES }),
        "Sql_small_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6836), tables: &DUCKDB_TABLES }),
        "Sql_warningsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6837), tables: &DUCKDB_TABLES }),
        "SqlcaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6838), tables: &DUCKDB_TABLES }),
        "SslKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6839), tables: &DUCKDB_TABLES }),
        "StableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6840), tables: &DUCKDB_TABLES }),
        "StageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6841), tables: &DUCKDB_TABLES }),
        "StagesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6842), tables: &DUCKDB_TABLES }),
        "StandaloneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6843), tables: &DUCKDB_TABLES }),
        "StarSegment" => Some(RootGrammar { grammar_id: GrammarId(6844), tables: &DUCKDB_TABLES }),
        "StartBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(231), tables: &DUCKDB_TABLES }),
        "StartCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(5432), tables: &DUCKDB_TABLES }),
        "StartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6845), tables: &DUCKDB_TABLES }),
        "StartSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(2152), tables: &DUCKDB_TABLES }),
        "StartingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6846), tables: &DUCKDB_TABLES }),
        "StartsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6847), tables: &DUCKDB_TABLES }),
        "StatementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6848), tables: &DUCKDB_TABLES }),
        "StatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6849), tables: &DUCKDB_TABLES }),
        "StatisticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6974), tables: &DUCKDB_TABLES }),
        "StatisticsReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3597), tables: &DUCKDB_TABLES }),
        "StdinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6975), tables: &DUCKDB_TABLES }),
        "StdoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6976), tables: &DUCKDB_TABLES }),
        "StorageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6977), tables: &DUCKDB_TABLES }),
        "StoredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6978), tables: &DUCKDB_TABLES }),
        "Straight_joinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6979), tables: &DUCKDB_TABLES }),
        "StreamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6980), tables: &DUCKDB_TABLES }),
        "StreamsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6981), tables: &DUCKDB_TABLES }),
        "StrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6982), tables: &DUCKDB_TABLES }),
        "StringBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(6983), tables: &DUCKDB_TABLES }),
        "StripKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6986), tables: &DUCKDB_TABLES }),
        "StructKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6987), tables: &DUCKDB_TABLES }),
        "StructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(6988), tables: &DUCKDB_TABLES }),
        "StructTypeSchemaSegment" => Some(RootGrammar { grammar_id: GrammarId(6993), tables: &DUCKDB_TABLES }),
        "StructTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(7001), tables: &DUCKDB_TABLES }),
        "SublistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7004), tables: &DUCKDB_TABLES }),
        "SubscriptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7005), tables: &DUCKDB_TABLES }),
        "SubscriptionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3597), tables: &DUCKDB_TABLES }),
        "SubstringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7006), tables: &DUCKDB_TABLES }),
        "SuccessfulKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7007), tables: &DUCKDB_TABLES }),
        "SummaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7008), tables: &DUCKDB_TABLES }),
        "SuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7009), tables: &DUCKDB_TABLES }),
        "SupportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7010), tables: &DUCKDB_TABLES }),
        "SymbolSegment" => Some(RootGrammar { grammar_id: GrammarId(7011), tables: &DUCKDB_TABLES }),
        "SymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7012), tables: &DUCKDB_TABLES }),
        "SynonymKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7013), tables: &DUCKDB_TABLES }),
        "SysdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7014), tables: &DUCKDB_TABLES }),
        "SysidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7015), tables: &DUCKDB_TABLES }),
        "SystemKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7016), tables: &DUCKDB_TABLES }),
        "TableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(7017), tables: &DUCKDB_TABLES }),
        "TableConstraintUsingIndexSegment" => Some(RootGrammar { grammar_id: GrammarId(7078), tables: &DUCKDB_TABLES }),
        "TableEndClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7101), tables: &DUCKDB_TABLES }),
        "TableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(7102), tables: &DUCKDB_TABLES }),
        "TableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7115), tables: &DUCKDB_TABLES }),
        "TableReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(7116), tables: &DUCKDB_TABLES }),
        "TablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7120), tables: &DUCKDB_TABLES }),
        "TablesampleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7121), tables: &DUCKDB_TABLES }),
        "TablespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7122), tables: &DUCKDB_TABLES }),
        "TablespaceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3597), tables: &DUCKDB_TABLES }),
        "TagReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3597), tables: &DUCKDB_TABLES }),
        "Tail_Recurse_Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(7123), tables: &DUCKDB_TABLES }),
        "Tail_Recurse_Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(7128), tables: &DUCKDB_TABLES }),
        "TargetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7132), tables: &DUCKDB_TABLES }),
        "TaskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7133), tables: &DUCKDB_TABLES }),
        "TasksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7134), tables: &DUCKDB_TABLES }),
        "TempKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7135), tables: &DUCKDB_TABLES }),
        "TemplateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7136), tables: &DUCKDB_TABLES }),
        "TemporalQuerySegment" => Some(RootGrammar { grammar_id: GrammarId(7137), tables: &DUCKDB_TABLES }),
        "TemporaryGrammar" => Some(RootGrammar { grammar_id: GrammarId(7138), tables: &DUCKDB_TABLES }),
        "TemporaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7141), tables: &DUCKDB_TABLES }),
        "TemporaryTransientGrammar" => Some(RootGrammar { grammar_id: GrammarId(7142), tables: &DUCKDB_TABLES }),
        "TerminateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7145), tables: &DUCKDB_TABLES }),
        "TerminatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7146), tables: &DUCKDB_TABLES }),
        "TextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7147), tables: &DUCKDB_TABLES }),
        "TextsizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7148), tables: &DUCKDB_TABLES }),
        "ThanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7149), tables: &DUCKDB_TABLES }),
        "ThenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7150), tables: &DUCKDB_TABLES }),
        "TiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7151), tables: &DUCKDB_TABLES }),
        "TildeSegment" => Some(RootGrammar { grammar_id: GrammarId(7152), tables: &DUCKDB_TABLES }),
        "TimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7153), tables: &DUCKDB_TABLES }),
        "TimeWithTZGrammar" => Some(RootGrammar { grammar_id: GrammarId(7154), tables: &DUCKDB_TABLES }),
        "TimeZoneGrammar" => Some(RootGrammar { grammar_id: GrammarId(7165), tables: &DUCKDB_TABLES }),
        "TimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7171), tables: &DUCKDB_TABLES }),
        "TimestamptzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7172), tables: &DUCKDB_TABLES }),
        "TimetzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7173), tables: &DUCKDB_TABLES }),
        "TimingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7174), tables: &DUCKDB_TABLES }),
        "TinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7175), tables: &DUCKDB_TABLES }),
        "TinyblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7176), tables: &DUCKDB_TABLES }),
        "TinyintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7177), tables: &DUCKDB_TABLES }),
        "TinytextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7178), tables: &DUCKDB_TABLES }),
        "ToKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7179), tables: &DUCKDB_TABLES }),
        "ToastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7180), tables: &DUCKDB_TABLES }),
        "TopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7181), tables: &DUCKDB_TABLES }),
        "TrailingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7182), tables: &DUCKDB_TABLES }),
        "TranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7183), tables: &DUCKDB_TABLES }),
        "TransactionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7184), tables: &DUCKDB_TABLES }),
        "TransactionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7185), tables: &DUCKDB_TABLES }),
        "TransactionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7202), tables: &DUCKDB_TABLES }),
        "TransformKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7203), tables: &DUCKDB_TABLES }),
        "TransientKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7204), tables: &DUCKDB_TABLES }),
        "TreatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7205), tables: &DUCKDB_TABLES }),
        "TriangleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7206), tables: &DUCKDB_TABLES }),
        "TriggerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7207), tables: &DUCKDB_TABLES }),
        "TriggerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3597), tables: &DUCKDB_TABLES }),
        "TrimKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7208), tables: &DUCKDB_TABLES }),
        "TrimParametersGrammar" => Some(RootGrammar { grammar_id: GrammarId(7209), tables: &DUCKDB_TABLES }),
        "TrueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7213), tables: &DUCKDB_TABLES }),
        "TrueSegment" => Some(RootGrammar { grammar_id: GrammarId(7214), tables: &DUCKDB_TABLES }),
        "TruncateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7215), tables: &DUCKDB_TABLES }),
        "TruncateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7216), tables: &DUCKDB_TABLES }),
        "TrustedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7233), tables: &DUCKDB_TABLES }),
        "TsequalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7234), tables: &DUCKDB_TABLES }),
        "TsqueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7235), tables: &DUCKDB_TABLES }),
        "TsrangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7236), tables: &DUCKDB_TABLES }),
        "TstzrangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7237), tables: &DUCKDB_TABLES }),
        "TsvectorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7238), tables: &DUCKDB_TABLES }),
        "TupleSegment" => Some(RootGrammar { grammar_id: GrammarId(7239), tables: &DUCKDB_TABLES }),
        "TypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7242), tables: &DUCKDB_TABLES }),
        "TypedArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(7243), tables: &DUCKDB_TABLES }),
        "TypedStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(7251), tables: &DUCKDB_TABLES }),
        "TypesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7254), tables: &DUCKDB_TABLES }),
        "UescapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7255), tables: &DUCKDB_TABLES }),
        "UidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7256), tables: &DUCKDB_TABLES }),
        "UnboundedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7257), tables: &DUCKDB_TABLES }),
        "UncommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7258), tables: &DUCKDB_TABLES }),
        "UnconditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(7259), tables: &DUCKDB_TABLES }),
        "UnconditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(7260), tables: &DUCKDB_TABLES }),
        "UndoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7264), tables: &DUCKDB_TABLES }),
        "UnencryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7265), tables: &DUCKDB_TABLES }),
        "UnionGrammar" => Some(RootGrammar { grammar_id: GrammarId(7266), tables: &DUCKDB_TABLES }),
        "UnionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7274), tables: &DUCKDB_TABLES }),
        "UniqueKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(7275), tables: &DUCKDB_TABLES }),
        "UniqueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7277), tables: &DUCKDB_TABLES }),
        "UnknownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7278), tables: &DUCKDB_TABLES }),
        "UnknownLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(7279), tables: &DUCKDB_TABLES }),
        "UnlistenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7280), tables: &DUCKDB_TABLES }),
        "UnlistenStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7281), tables: &DUCKDB_TABLES }),
        "UnlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7286), tables: &DUCKDB_TABLES }),
        "UnloggedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7287), tables: &DUCKDB_TABLES }),
        "UnorderedSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7288), tables: &DUCKDB_TABLES }),
        "UnorderedSetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(7309), tables: &DUCKDB_TABLES }),
        "UnpackingOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(7315), tables: &DUCKDB_TABLES }),
        "UnpivotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7316), tables: &DUCKDB_TABLES }),
        "UnsafeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7317), tables: &DUCKDB_TABLES }),
        "UnsignedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7318), tables: &DUCKDB_TABLES }),
        "UntilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7319), tables: &DUCKDB_TABLES }),
        "UpdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7320), tables: &DUCKDB_TABLES }),
        "UpdateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7321), tables: &DUCKDB_TABLES }),
        "UpdatetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7344), tables: &DUCKDB_TABLES }),
        "UsageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7345), tables: &DUCKDB_TABLES }),
        "UseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7346), tables: &DUCKDB_TABLES }),
        "UseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7347), tables: &DUCKDB_TABLES }),
        "Use_any_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7350), tables: &DUCKDB_TABLES }),
        "UserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7351), tables: &DUCKDB_TABLES }),
        "UserReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(7352), tables: &DUCKDB_TABLES }),
        "UsersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7353), tables: &DUCKDB_TABLES }),
        "UsingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7354), tables: &DUCKDB_TABLES }),
        "UsingSampleClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7355), tables: &DUCKDB_TABLES }),
        "Utc_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7359), tables: &DUCKDB_TABLES }),
        "Utc_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7360), tables: &DUCKDB_TABLES }),
        "Utc_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7361), tables: &DUCKDB_TABLES }),
        "UuidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7362), tables: &DUCKDB_TABLES }),
        "VacuumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7363), tables: &DUCKDB_TABLES }),
        "VacuumStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7364), tables: &DUCKDB_TABLES }),
        "ValidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7397), tables: &DUCKDB_TABLES }),
        "ValidateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7398), tables: &DUCKDB_TABLES }),
        "ValidatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7399), tables: &DUCKDB_TABLES }),
        "ValueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7400), tables: &DUCKDB_TABLES }),
        "ValuesClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7401), tables: &DUCKDB_TABLES }),
        "ValuesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7412), tables: &DUCKDB_TABLES }),
        "Varchar2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7413), tables: &DUCKDB_TABLES }),
        "VarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7414), tables: &DUCKDB_TABLES }),
        "VarcharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7415), tables: &DUCKDB_TABLES }),
        "VariableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7416), tables: &DUCKDB_TABLES }),
        "VariablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7417), tables: &DUCKDB_TABLES }),
        "VariadicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7418), tables: &DUCKDB_TABLES }),
        "VaryingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7419), tables: &DUCKDB_TABLES }),
        "VectorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7420), tables: &DUCKDB_TABLES }),
        "VerboseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7421), tables: &DUCKDB_TABLES }),
        "VersionIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(7422), tables: &DUCKDB_TABLES }),
        "VersionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7425), tables: &DUCKDB_TABLES }),
        "ViewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7426), tables: &DUCKDB_TABLES }),
        "ViewsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7427), tables: &DUCKDB_TABLES }),
        "VirtualKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7428), tables: &DUCKDB_TABLES }),
        "VolatileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7429), tables: &DUCKDB_TABLES }),
        "WaitforKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7430), tables: &DUCKDB_TABLES }),
        "WalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7431), tables: &DUCKDB_TABLES }),
        "WalrusOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(7432), tables: &DUCKDB_TABLES }),
        "WarehouseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7433), tables: &DUCKDB_TABLES }),
        "WarehousesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7434), tables: &DUCKDB_TABLES }),
        "WeekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7435), tables: &DUCKDB_TABLES }),
        "WeekdayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7436), tables: &DUCKDB_TABLES }),
        "WellKnownTextGeometrySegment" => Some(RootGrammar { grammar_id: GrammarId(7437), tables: &DUCKDB_TABLES }),
        "WhenClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7537), tables: &DUCKDB_TABLES }),
        "WhenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7547), tables: &DUCKDB_TABLES }),
        "WhereClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7548), tables: &DUCKDB_TABLES }),
        "WhereClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(7553), tables: &DUCKDB_TABLES }),
        "WhereKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7570), tables: &DUCKDB_TABLES }),
        "WhileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7571), tables: &DUCKDB_TABLES }),
        "WhitespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7572), tables: &DUCKDB_TABLES }),
        "WhitespaceSegment" => Some(RootGrammar { grammar_id: GrammarId(7573), tables: &DUCKDB_TABLES }),
        "WildcardExcludeExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(7574), tables: &DUCKDB_TABLES }),
        "WildcardExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(7581), tables: &DUCKDB_TABLES }),
        "WildcardIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(7589), tables: &DUCKDB_TABLES }),
        "WildcardPatternMatchingSegment" => Some(RootGrammar { grammar_id: GrammarId(7599), tables: &DUCKDB_TABLES }),
        "WildcardRenameExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(7606), tables: &DUCKDB_TABLES }),
        "WildcardReplaceExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(7617), tables: &DUCKDB_TABLES }),
        "WindowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7628), tables: &DUCKDB_TABLES }),
        "WindowSpecificationSegment" => Some(RootGrammar { grammar_id: GrammarId(7629), tables: &DUCKDB_TABLES }),
        "WithCheckOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(7637), tables: &DUCKDB_TABLES }),
        "WithCompoundNonSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7644), tables: &DUCKDB_TABLES }),
        "WithCompoundStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7653), tables: &DUCKDB_TABLES }),
        "WithDataClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7662), tables: &DUCKDB_TABLES }),
        "WithFillSegment" => Some(RootGrammar { grammar_id: GrammarId(7667), tables: &DUCKDB_TABLES }),
        "WithKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7668), tables: &DUCKDB_TABLES }),
        "WithNoSchemaBindingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7669), tables: &DUCKDB_TABLES }),
        "WithinGroupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7674), tables: &DUCKDB_TABLES }),
        "WithinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7679), tables: &DUCKDB_TABLES }),
        "WithoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7680), tables: &DUCKDB_TABLES }),
        "WordSegment" => Some(RootGrammar { grammar_id: GrammarId(7681), tables: &DUCKDB_TABLES }),
        "WorkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7682), tables: &DUCKDB_TABLES }),
        "WrapperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7683), tables: &DUCKDB_TABLES }),
        "WriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7684), tables: &DUCKDB_TABLES }),
        "Write_partition_columnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7685), tables: &DUCKDB_TABLES }),
        "WritetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7686), tables: &DUCKDB_TABLES }),
        "X509KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7687), tables: &DUCKDB_TABLES }),
        "XmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7688), tables: &DUCKDB_TABLES }),
        "XmlattributesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7689), tables: &DUCKDB_TABLES }),
        "XmlconcatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7690), tables: &DUCKDB_TABLES }),
        "XmlelementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7691), tables: &DUCKDB_TABLES }),
        "XmlexistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7692), tables: &DUCKDB_TABLES }),
        "XmlforestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7693), tables: &DUCKDB_TABLES }),
        "XmlnamespacesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7694), tables: &DUCKDB_TABLES }),
        "XmlparseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7695), tables: &DUCKDB_TABLES }),
        "XmlpiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7696), tables: &DUCKDB_TABLES }),
        "XmlrootKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7697), tables: &DUCKDB_TABLES }),
        "XmlserializeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7698), tables: &DUCKDB_TABLES }),
        "XmltableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7699), tables: &DUCKDB_TABLES }),
        "XorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7700), tables: &DUCKDB_TABLES }),
        "YamlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7701), tables: &DUCKDB_TABLES }),
        "YearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7702), tables: &DUCKDB_TABLES }),
        "Year_monthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7703), tables: &DUCKDB_TABLES }),
        "YesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7704), tables: &DUCKDB_TABLES }),
        "ZerofillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7705), tables: &DUCKDB_TABLES }),
        "ZoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7706), tables: &DUCKDB_TABLES }),
        _ => None,
    }
}

pub fn get_duckdb_segment_type(name: &str) -> Option<&'static str> {
    match name {
        "AccessObjectSegment" => Some("access_object"),
        "AccessPermissionSegment" => Some("access_permission"),
        "AccessPermissionsSegment" => Some("access_permissions"),
        "AccessSchemaObjectSegment" => Some("access_schema_object"),
        "AccessSchemaPluralObjectSegment" => Some("access_schema_plural_object"),
        "AccessStatementSegment" => Some("access_statement"),
        "AccessTargetSegment" => Some("access_target"),
        "AdjacentSegment" => Some("comparison_operator"),
        "AggregateOrderByClause" => Some("aggregate_order_by"),
        "AliasExpressionSegment" => Some("alias_expression"),
        "AlterAggregateStatementSegment" => Some("alter_aggregate_statement"),
        "AlterDatabaseStatementSegment" => Some("alter_database_statement"),
        "AlterDefaultPrivilegesGrantSegment" => Some("alter_default_privileges_grant"),
        "AlterDefaultPrivilegesObjectPrivilegesSegment" => Some("alter_default_privileges_object_privilege"),
        "AlterDefaultPrivilegesRevokeSegment" => Some("alter_default_privileges_revoke"),
        "AlterDefaultPrivilegesSchemaObjectsSegment" => Some("alter_default_privileges_schema_object"),
        "AlterDefaultPrivilegesStatementSegment" => Some("alter_default_privileges_statement"),
        "AlterDefaultPrivilegesToFromRolesSegment" => Some("alter_default_privileges_to_from_roles"),
        "AlterDomainStatementSegment" => Some("alter_domain_statement"),
        "AlterExtensionStatementSegment" => Some("alter_extension_statement"),
        "AlterForeignTableActionSegment" => Some("alter_foreign_table_action_segment"),
        "AlterForeignTableStatementSegment" => Some("alter_foreign_table_statement"),
        "AlterFunctionActionSegment" => Some("alter_function_action_segment"),
        "AlterFunctionStatementSegment" => Some("alter_function_statement"),
        "AlterIndexStatementSegment" => Some("alter_index_statement"),
        "AlterMaterializedViewActionSegment" => Some("alter_materialized_view_action_segment"),
        "AlterMaterializedViewStatementSegment" => Some("alter_materialized_view_statement"),
        "AlterPolicyStatementSegment" => Some("alter_policy_statement"),
        "AlterProcedureActionSegment" => Some("alter_procedure_action_segment"),
        "AlterProcedureStatementSegment" => Some("alter_procedure_statement"),
        "AlterPublicationStatementSegment" => Some("alter_publication_statement"),
        "AlterRoleStatementSegment" => Some("alter_role_statement"),
        "AlterSchemaStatementSegment" => Some("alter_schema_statement"),
        "AlterSequenceOptionsSegment" => Some("alter_sequence_options_segment"),
        "AlterSequenceStatementSegment" => Some("alter_sequence_statement"),
        "AlterStatisticsStatementSegment" => Some("alter_statistics_statement"),
        "AlterSubscriptionStatementSegment" => Some("alter_subscription"),
        "AlterSystemStatementSegment" => Some("alter_system_statement"),
        "AlterTableActionSegment" => Some("alter_table_action_segment"),
        "AlterTableStatementSegment" => Some("alter_table_statement"),
        "AlterTextSearchConfigurationStatementSegment" => Some("alter_text_search_configuration_statement"),
        "AlterTriggerStatementSegment" => Some("alter_trigger"),
        "AlterTypeStatementSegment" => Some("alter_type_statement"),
        "AlterViewStatementSegment" => Some("alter_view_statement"),
        "AnalyzeStatementSegment" => Some("analyze_statement"),
        "ArrayAccessorSegment" => Some("array_accessor"),
        "ArrayExpressionSegment" => Some("array_expression"),
        "ArrayLiteralSegment" => Some("array_literal"),
        "ArrayTypeSegment" => Some("array_type"),
        "ArrayTypeSuffixSegment" => Some("array_type_suffix"),
        "AsAliasExpressionSegment" => Some("alias_expression"),
        "AsAliasOperatorSegment" => Some("alias_operator"),
        "BaseFileSegment" => Some("file"),
        "BaseSegment" => Some("base"),
        "BinaryOperatorSegment" => Some("binary_operator"),
        "BitwiseAndSegment" => Some("binary_operator"),
        "BitwiseLShiftSegment" => Some("binary_operator"),
        "BitwiseOrSegment" => Some("binary_operator"),
        "BitwiseRShiftSegment" => Some("binary_operator"),
        "BracketedArguments" => Some("bracketed_arguments"),
        "BracketedSegment" => Some("bracketed"),
        "CTEColumnList" => Some("cte_column_list"),
        "CTEDefinitionSegment" => Some("common_table_expression"),
        "CallStoredProcedureSegment" => Some("call_statement"),
        "CaseExpressionSegment" => Some("case_expression"),
        "ClusterStatementSegment" => Some("cluster_statement"),
        "CodeSegment" => Some("raw"),
        "CollationReferenceSegment" => Some("collation_reference"),
        "ColumnConstraintSegment" => Some("column_constraint_segment"),
        "ColumnDefinitionSegment" => Some("column_definition"),
        "ColumnIndexSegment" => Some("column_index"),
        "ColumnReferenceSegment" => Some("column_reference"),
        "ColumnTypeReferenceSegment" => Some("column_type_reference"),
        "ColumnsExpressionFunctionContentsSegment" => Some("function_contents"),
        "ColumnsExpressionFunctionNameSegment" => Some("function_name"),
        "CommentClauseSegment" => Some("comment_clause"),
        "CommentOnStatementSegment" => Some("comment_clause"),
        "CommentSegment" => Some("comment"),
        "ComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeBinaryOperatorSegment" => Some("binary_operator"),
        "CompositeComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeValueExpansionSegment" => Some("composite_value_expansion"),
        "ConcatSegment" => Some("binary_operator"),
        "ConflictActionSegment" => Some("conflict_action"),
        "ConflictTargetSegment" => Some("conflict_target"),
        "CopyStatementSegment" => Some("copy_statement"),
        "CreateAggregateStatementSegment" => Some("create_aggregate_statement"),
        "CreateCastStatementSegment" => Some("create_cast_statement"),
        "CreateCollationStatementSegment" => Some("create_collation_statement"),
        "CreateDatabaseStatementSegment" => Some("create_database_statement"),
        "CreateDomainStatementSegment" => Some("create_domain_statement"),
        "CreateExtensionStatementSegment" => Some("create_extension_statement"),
        "CreateForeignDataWrapperStatementSegment" => Some("create_foreign_data_wrapper"),
        "CreateForeignTableStatementSegment" => Some("create_foreign_table_statement"),
        "CreateFunctionStatementSegment" => Some("create_function_statement"),
        "CreateIndexStatementSegment" => Some("create_index_statement"),
        "CreateMaterializedViewStatementSegment" => Some("create_materialized_view_statement"),
        "CreateModelStatementSegment" => Some("create_model_statement"),
        "CreateOperatorStatementSegment" => Some("create_operator_statement"),
        "CreatePolicyStatementSegment" => Some("create_policy_statement"),
        "CreateProcedureStatementSegment" => Some("create_procedure_statement"),
        "CreatePublicationStatementSegment" => Some("create_publication_statement"),
        "CreateRoleStatementSegment" => Some("create_role_statement"),
        "CreateSchemaStatementSegment" => Some("create_schema_statement"),
        "CreateSequenceOptionsSegment" => Some("create_sequence_options_segment"),
        "CreateSequenceStatementSegment" => Some("create_sequence_statement"),
        "CreateServerStatementSegment" => Some("create_server_statement"),
        "CreateStatisticsStatementSegment" => Some("create_statistics_statement"),
        "CreateSubscriptionStatementSegment" => Some("create_subscription"),
        "CreateTableStatementSegment" => Some("create_table_statement"),
        "CreateTextSearchConfigurationStatementSegment" => Some("create_text_search_configuration_statement"),
        "CreateTriggerStatementSegment" => Some("create_trigger"),
        "CreateTypeStatementSegment" => Some("create_type_statement"),
        "CreateUserMappingStatementSegment" => Some("create_user_mapping_statement"),
        "CreateUserStatementSegment" => Some("create_user_statement"),
        "CreateViewStatementSegment" => Some("create_view_statement"),
        "CubeFunctionNameSegment" => Some("function_name"),
        "CubeRollupClauseSegment" => Some("cube_rollup_clause"),
        "DatabaseReferenceSegment" => Some("database_reference"),
        "DatatypeSegment" => Some("data_type"),
        "DatePartFunctionNameSegment" => Some("function_name"),
        "DateTimeFunctionContentsSegment" => Some("function_contents"),
        "DateTimeLiteralGrammar" => Some("datetime_literal"),
        "DateTimeTypeIdentifier" => Some("datetime_type_identifier"),
        "DeallocateStatementSegment" => Some("deallocate_statement"),
        "Dedent" => Some("dedent"),
        "DefinitionParameterSegment" => Some("definition_parameter"),
        "DefinitionParametersSegment" => Some("definition_parameters"),
        "DeleteStatementSegment" => Some("delete_statement"),
        "DescribeStatementSegment" => Some("describe_statement"),
        "DiscardStatementSegment" => Some("discard_statement"),
        "DoStatementSegment" => Some("do_statement"),
        "DropAggregateStatementSegment" => Some("drop_aggregate_statement"),
        "DropCastStatementSegment" => Some("drop_cast_statement"),
        "DropCollationStatementSegment" => Some("drop_collation_statement"),
        "DropDatabaseStatementSegment" => Some("drop_database_statement"),
        "DropDomainStatementSegment" => Some("drop_domain_statement"),
        "DropExtensionStatementSegment" => Some("drop_extension_statement"),
        "DropForeignTableStatement" => Some("drop_foreign_table_statement"),
        "DropFunctionStatementSegment" => Some("drop_function_statement"),
        "DropIndexStatementSegment" => Some("drop_index_statement"),
        "DropMaterializedViewStatementSegment" => Some("drop_materialized_view_statement"),
        "DropModelStatementSegment" => Some("drop_MODELstatement"),
        "DropOwnedStatementSegment" => Some("drop_owned_statement"),
        "DropPolicyStatementSegment" => Some("drop_policy_statement"),
        "DropProcedureStatementSegment" => Some("drop_procedure_statement"),
        "DropPublicationStatementSegment" => Some("drop_publication_statement"),
        "DropRoleStatementSegment" => Some("drop_role_statement"),
        "DropSchemaStatementSegment" => Some("drop_schema_statement"),
        "DropSequenceStatementSegment" => Some("drop_sequence_statement"),
        "DropStatisticsStatementSegment" => Some("drop_statistics_statement"),
        "DropSubscriptionStatementSegment" => Some("drop_subscription"),
        "DropTableStatementSegment" => Some("drop_table_statement"),
        "DropTextSearchConfigurationStatementSegment" => Some("drop_text_search_configuration_statement"),
        "DropTriggerStatementSegment" => Some("drop_trigger"),
        "DropTypeStatementSegment" => Some("drop_type_statement"),
        "DropUserStatementSegment" => Some("drop_user_statement"),
        "DropViewStatementSegment" => Some("drop_view_statement"),
        "ElseClauseSegment" => Some("else_clause"),
        "EmptyStructLiteralBracketsSegment" => Some("struct_literal"),
        "EmptyStructLiteralSegment" => Some("typed_struct_literal"),
        "EqualsSegment" => Some("comparison_operator"),
        "ExclusionConstraintElementSegment" => Some("exclusion_constraint_element"),
        "ExecuteStatementSegment" => Some("execute_statement"),
        "ExplainOptionSegment" => Some("explain_option"),
        "ExplainStatementSegment" => Some("explain_statement"),
        "ExpressionSegment" => Some("expression"),
        "ExtensionReferenceSegment" => Some("extension_reference"),
        "FetchClauseSegment" => Some("fetch_clause"),
        "FileSegment" => Some("file"),
        "ForClauseSegment" => Some("for_clause"),
        "ForeignTableColumnConstraintSegment" => Some("column_constraint_segment"),
        "ForeignTableTableConstraintSegment" => Some("table_constraint"),
        "FrameClauseSegment" => Some("frame_clause"),
        "FromClauseSegment" => Some("from_clause"),
        "FromExpressionElementSegment" => Some("from_expression_element"),
        "FromExpressionSegment" => Some("from_expression"),
        "FromPivotExpressionSegment" => Some("from_pivot_expression"),
        "FromUnpivotExpressionSegment" => Some("from_unpivot_expression"),
        "FunctionContentsSegment" => Some("function_contents"),
        "FunctionDefinitionGrammar" => Some("function_definition"),
        "FunctionNameSegment" => Some("function_name"),
        "FunctionParameterListGrammar" => Some("function_parameter_list"),
        "FunctionSegment" => Some("function"),
        "GrantStatementSegment" => Some("grant_statement"),
        "GreaterThanOrEqualToSegment" => Some("comparison_operator"),
        "GreaterThanSegment" => Some("comparison_operator"),
        "GroupByClauseSegment" => Some("groupby_clause"),
        "GroupingExpressionList" => Some("grouping_expression_list"),
        "GroupingSetsClauseSegment" => Some("grouping_sets_clause"),
        "HavingClauseSegment" => Some("having_clause"),
        "IdentifierSegment" => Some("identifier"),
        "ImplicitIndent" => Some("indent"),
        "ImportForeignSchemaStatementSegment" => Some("import_foreign_schema_statement"),
        "Indent" => Some("indent"),
        "IndexAccessMethodSegment" => Some("index_access_method"),
        "IndexColumnDefinitionSegment" => Some("index_column_definition"),
        "IndexElementOptionsSegment" => Some("index_element_options"),
        "IndexElementSegment" => Some("index_element"),
        "IndexParametersSegment" => Some("index_parameters"),
        "IndexReferenceSegment" => Some("index_reference"),
        "InsertStatementSegment" => Some("insert_statement"),
        "IntervalExpressionSegment" => Some("interval_expression"),
        "IntoClauseSegment" => Some("into_clause"),
        "JoinClauseSegment" => Some("join_clause"),
        "JoinOnConditionSegment" => Some("join_on_condition"),
        "KeywordSegment" => Some("keyword"),
        "LambdaExpressionSegment" => Some("lambda_function"),
        "LanguageClauseSegment" => Some("language_clause"),
        "LessThanOrEqualToSegment" => Some("comparison_operator"),
        "LessThanSegment" => Some("comparison_operator"),
        "LikeOptionSegment" => Some("like_option_segment"),
        "LimitClauseSegment" => Some("limit_clause"),
        "ListComprehensionExpressionSegment" => Some("list_comprehension"),
        "ListenStatementSegment" => Some("listen_statement"),
        "LiteralKeywordSegment" => Some("literal"),
        "LiteralSegment" => Some("literal"),
        "LoadStatementSegment" => Some("load_statement"),
        "LocalAliasSegment" => Some("local_alias_segment"),
        "LockTableStatementSegment" => Some("lock_table_statement"),
        "MLTableExpressionSegment" => Some("ml_table_expression"),
        "MapLiteralElementSegment" => Some("map_literal_element"),
        "MapLiteralSegment" => Some("map_literal"),
        "MapTypeSchemaSegment" => Some("map_type_schema"),
        "MapTypeSegment" => Some("map_type"),
        "MatchConditionSegment" => Some("match_condition"),
        "MergeDeleteClauseSegment" => Some("merge_delete_clause"),
        "MergeInsertClauseSegment" => Some("merge_insert_clause"),
        "MergeMatchSegment" => Some("merge_match"),
        "MergeMatchedClauseSegment" => Some("merge_when_matched_clause"),
        "MergeNotMatchedBySourceClauseSegment" => Some("merge_when_not_matched_by_source_clause"),
        "MergeNotMatchedClauseSegment" => Some("merge_when_not_matched_clause"),
        "MergeStatementSegment" => Some("merge_statement"),
        "MergeUpdateClauseSegment" => Some("merge_update_clause"),
        "MetaCommandQueryBufferStatement" => Some("meta_command_statement"),
        "NamedArgumentSegment" => Some("named_argument"),
        "NamedWindowExpressionSegment" => Some("named_window_expression"),
        "NamedWindowSegment" => Some("named_window"),
        "NewlineSegment" => Some("newline"),
        "NotEqualToSegment" => Some("comparison_operator"),
        "NotExtendLeftSegment" => Some("comparison_operator"),
        "NotExtendRightSegment" => Some("comparison_operator"),
        "NotifyStatementSegment" => Some("notify_statement"),
        "ObjectLiteralElementSegment" => Some("object_literal_element"),
        "ObjectLiteralSegment" => Some("object_literal"),
        "ObjectReferenceSegment" => Some("object_reference"),
        "OffsetClauseSegment" => Some("offset_clause"),
        "OperationClassReferenceSegment" => Some("operation_class_reference"),
        "OperatorClassReferenceSegment" => Some("operator_class_reference"),
        "OrderByClauseSegment" => Some("orderby_clause"),
        "OverClauseSegment" => Some("over_clause"),
        "OverlapSegment" => Some("comparison_operator"),
        "OverlapsClauseSegment" => Some("overlaps_clause"),
        "PartitionBoundSpecSegment" => Some("partition_bound_spec"),
        "PartitionClauseSegment" => Some("partitionby_clause"),
        "PathSegment" => Some("path_segment"),
        "PostgresCopyStdinDataStatementSegment" => Some("postgres_copy_stdin_data_statement"),
        "PrepareStatementSegment" => Some("prepare_statement"),
        "PsqlCopyMetaCommandStatementSegment" => Some("psql_copy_meta_command_statement"),
        "PsqlSetMetaCommandStatementSegment" => Some("psql_set_meta_command_statement"),
        "PsqlVariableGrammar" => Some("psql_variable"),
        "PublicationObjectsSegment" => Some("publication_objects"),
        "PublicationReferenceSegment" => Some("publication_reference"),
        "PublicationTableSegment" => Some("publication_table"),
        "QualifiedNumericLiteralSegment" => Some("numeric_literal"),
        "QualifiedOperatorSegment" => Some("qualified_operator"),
        "QualifyClauseSegment" => Some("qualify_clause"),
        "ReassignOwnedStatementSegment" => Some("reassign_owned_statement"),
        "RefreshMaterializedViewStatementSegment" => Some("refresh_materialized_view_statement"),
        "ReindexStatementSegment" => Some("reindex_statement_segment"),
        "RelationOptionSegment" => Some("relation_option"),
        "RelationOptionsSegment" => Some("relation_options"),
        "ResetSessionAuthorizationStatementSegment" => Some("reset_session_authorization_statement"),
        "ResetStatementSegment" => Some("reset_statement"),
        "RevokeStatementSegment" => Some("revoke_statement"),
        "RoleReferenceSegment" => Some("role_reference"),
        "RollupFunctionNameSegment" => Some("function_name"),
        "SamplingExpressionSegment" => Some("sample_expression"),
        "SchemaReferenceSegment" => Some("schema_reference"),
        "SecurityLabelStatementSegment" => Some("security_label_statement"),
        "SelectClauseElementSegment" => Some("select_clause_element"),
        "SelectClauseModifierSegment" => Some("select_clause_modifier"),
        "SelectClauseSegment" => Some("select_clause"),
        "SelectStatementSegment" => Some("select_statement"),
        "SemiStructuredAccessorSegment" => Some("semi_structured_expression"),
        "SequenceReferenceSegment" => Some("sequence_reference"),
        "ServerReferenceSegment" => Some("server_reference"),
        "SetClauseListSegment" => Some("set_clause_list"),
        "SetClauseSegment" => Some("set_clause"),
        "SetConstraintsStatementSegment" => Some("set_constraint_statement"),
        "SetExpressionSegment" => Some("set_expression"),
        "SetOperatorSegment" => Some("set_operator"),
        "SetSchemaStatementSegment" => Some("set_schema_statement"),
        "SetSessionAuthorizationStatementSegment" => Some("set_session_authorization_statement"),
        "SetStatementSegment" => Some("set_statement"),
        "ShorthandCastSegment" => Some("cast_expression"),
        "ShowStatementSegment" => Some("show_statement"),
        "SimplifiedPivotExpressionSegment" => Some("simplified_pivot"),
        "SimplifiedUnpivotExpressionSegment" => Some("simplified_unpivot"),
        "SingleIdentifierListSegment" => Some("identifier_list"),
        "SingleIdentifierWildcardSegment" => Some("wildcard_identifier"),
        "SizedArrayTypeSegment" => Some("sized_array_type"),
        "StatementSegment" => Some("statement"),
        "StatisticsReferenceSegment" => Some("statistics_reference"),
        "StructLiteralSegment" => Some("struct_literal"),
        "StructTypeSchemaSegment" => Some("struct_type_schema"),
        "StructTypeSegment" => Some("struct_type"),
        "SubscriptionReferenceSegment" => Some("subscription_reference"),
        "SymbolSegment" => Some("symbol"),
        "TableConstraintSegment" => Some("table_constraint"),
        "TableConstraintUsingIndexSegment" => Some("table_constraint"),
        "TableEndClauseSegment" => Some("table_end_clause_segment"),
        "TableExpressionSegment" => Some("table_expression"),
        "TableReferenceSegment" => Some("table_reference"),
        "TablespaceReferenceSegment" => Some("tablespace_reference"),
        "TagReferenceSegment" => Some("tag_reference"),
        "TemporalQuerySegment" => Some("temporal_query"),
        "TimeZoneGrammar" => Some("time_zone_grammar"),
        "TransactionStatementSegment" => Some("transaction_statement"),
        "TriggerReferenceSegment" => Some("trigger_reference"),
        "TruncateStatementSegment" => Some("truncate_table"),
        "TupleSegment" => Some("tuple"),
        "TypedArrayLiteralSegment" => Some("typed_array_literal"),
        "TypedStructLiteralSegment" => Some("typed_struct_literal"),
        "UnlistenStatementSegment" => Some("unlisten_statement"),
        "UnorderedSelectStatementSegment" => Some("select_statement"),
        "UnorderedSetExpressionSegment" => Some("set_expression"),
        "UpdateStatementSegment" => Some("update_statement"),
        "UseStatementSegment" => Some("use_statement"),
        "UserReferenceSegment" => Some("user_reference"),
        "UsingSampleClauseSegment" => Some("sample_expression"),
        "VacuumStatementSegment" => Some("vacuum_statement"),
        "ValuesClauseSegment" => Some("values_clause"),
        "VersionIdentifierSegment" => Some("version_identifier"),
        "WellKnownTextGeometrySegment" => Some("wkt_geometry_type"),
        "WhenClauseSegment" => Some("when_clause"),
        "WhereClauseSegment" => Some("where_clause"),
        "WhitespaceSegment" => Some("whitespace"),
        "WildcardExcludeExpressionSegment" => Some("wildcard_exclude"),
        "WildcardExpressionSegment" => Some("wildcard_expression"),
        "WildcardIdentifierSegment" => Some("wildcard_identifier"),
        "WildcardPatternMatchingSegment" => Some("wildcard_pattern_matching"),
        "WildcardRenameExpressionSegment" => Some("wildcard_rename"),
        "WildcardReplaceExpressionSegment" => Some("wildcard_replace"),
        "WindowSpecificationSegment" => Some("window_specification"),
        "WithCheckOptionSegment" => Some("with_check_option"),
        "WithCompoundNonSelectStatementSegment" => Some("with_compound_statement"),
        "WithCompoundStatementSegment" => Some("with_compound_statement"),
        "WithDataClauseSegment" => Some("with_data_clause"),
        "WithFillSegment" => Some("with_fill"),
        "WithNoSchemaBindingClauseSegment" => Some("with_no_schema_binding_clause"),
        "WithinGroupClauseSegment" => Some("withingroup_clause"),
        "WordSegment" => Some("word"),
        _ => None,
    }
}

pub static DUCKDB_TABLES: GrammarTables = GrammarTables {
    instructions: INSTRUCTIONS,
    child_ids: CHILD_IDS,
    terminators: TERMINATORS,
    strings: STRINGS,
    aux_data: AUX_DATA,
    aux_data_offsets: AUX_DATA_OFFSETS,
    regex_patterns: REGEX_PATTERNS,
    simple_hints: SIMPLE_HINTS,
    hint_string_indices: HINT_STRING_INDICES,
    simple_hint_indices: SIMPLE_HINT_INDICES,
    segment_type_offsets: SEGMENT_TYPE_OFFSETS,
    segment_class_offsets: SEGMENT_CLASS_OFFSETS,
    casefold_sparse: CASEFOLD_SPARSE,
    trim_chars_sparse: TRIM_CHARS_SPARSE,
    trim_chars_data: TRIM_CHARS_DATA,
    segment_class_types_sparse: SEGMENT_CLASS_TYPES_SPARSE,
    segment_class_types_data: SEGMENT_CLASS_TYPES_DATA,
};

pub fn get_duckdb_root_grammar_id() -> GrammarId {
    GrammarId(4301)
}
pub fn get_duckdb_root_grammar_table() -> RootGrammar {
    RootGrammar {
        grammar_id: get_duckdb_root_grammar_id(),
        tables: &DUCKDB_TABLES,
    }
}
pub fn get_duckdb_root_grammar() -> RootGrammar {
    get_duckdb_root_grammar_table()
}
