/* This is a generated file! */
/* Generated by `utils/build_parsers.py` via `utils/rustify.py` */
/* This process can be run via tox: `tox -e generate-rs` */
#![cfg_attr(rustfmt, rustfmt_skip)]

use sqlfluffrs_types::{
    GrammarInst, GrammarVariant, GrammarFlags,
    ParseMode, GrammarId, GrammarTables,
    SimpleHintData, RootGrammar
};
// Flattening grammar tree...
// Validating tables...
// Validation passed!

// Table Statistics:
//   Instructions:      2683 × 20 bytes =    53660 bytes
//   Child IDs:         2250 ×  4 bytes =     9000 bytes
//   Terminators:         40 ×  4 bytes =      160 bytes
//   Strings:           1132 strings      =    17715 bytes
//   Aux Data:          4334 ×  4 bytes =    17336 bytes
//   Aux Offsets:       2683 ×  4 bytes =    10732 bytes
//   Regex Patterns:       5 patterns     =      696 bytes
//   Simple Hints:       422 hints        =     6752 bytes
//   Hint Indices:       769 ×  4 bytes =     3076 bytes
//   TOTAL:                                    ≈   119127 bytes

pub static INSTRUCTIONS: &[GrammarInst] = &[
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [0] StringParser("ABSENT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 0, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [1] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [2] Ref(AccountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [3] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5, child_count: 17, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [4] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 22, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [5] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [6] Ref(ResourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [7] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [8] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [9] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [10] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [11] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [12] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [13] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [14] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [15] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [16] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [17] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [18] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [19] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [20] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [21] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [22] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [23] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [24] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [25] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [26] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [27] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [28] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [29] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [30] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [31] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [32] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [33] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [34] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [35] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [36] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [37] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [38] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [39] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [40] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [41] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [42] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [43] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [44] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [45] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [46] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [47] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 2, _padding: 0 }, // [48] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 50, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [49] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [50] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [51] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [52] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [53] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [54] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [55] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [56] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [57] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [58] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [59] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [60] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [61] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 6, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [62] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 64, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [63] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [64] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 5, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [65] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [66] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [67] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [68] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [69] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [70] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [71] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [72] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [73] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [74] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [75] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [76] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [77] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [78] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [79] Ref(ManageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [80] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [81] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [82] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [83] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [84] Ref(ExecutionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [85] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [86] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 26, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [87] OneOf(26 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 110, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [88] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [89] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 4, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [90] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [91] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [92] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [93] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [94] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [95] Ref(PipeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [96] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [97] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [98] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [99] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [100] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [101] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [102] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [103] Ref(ImportedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [104] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [105] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [106] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [107] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [108] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [109] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [110] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [111] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [112] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [113] Ref(OperateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [114] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [115] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [116] Ref(Reference_usageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [117] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [118] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [119] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [120] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [121] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [122] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [123] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [124] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [125] Ref(Use_any_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [126] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [127] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [128] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [129] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [130] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 1, _padding: 0 }, // [131] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [132] Ref(AccessPermissionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [133] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [134] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [135] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [136] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [137] Ref(StageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [138] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [139] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [140] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [141] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [142] Ref(StreamKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [143] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [144] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [145] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [146] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [147] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [148] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [149] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [150] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [151] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [152] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [153] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [154] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [155] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [156] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [157] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [158] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [159] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [160] Ref(StagesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [161] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [162] Ref(ProceduresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [163] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [164] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [165] Ref(StreamsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [166] Ref(TasksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [167] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [168] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [169] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [170] Ref(PoliciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [171] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [172] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [173] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [174] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [175] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [176] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [177] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [178] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [179] Ref(FormatsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [180] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [181] Ref(GrantStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [182] Ref(RevokeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [183] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 181, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [184] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [185] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [186] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [187] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [188] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [189] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [190] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [191] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [192] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [193] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [194] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [195] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [196] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [197] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [198] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [199] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [200] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [201] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [202] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [203] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [204] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 201, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [205] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [206] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [207] Ref(SemiStructuredAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [208] StringParser("ADD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [209] StringParser("ADMIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [210] StringParser("AFTER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [211] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 203, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [212] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 207, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [213] Meta("indent")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 207, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [214] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 207, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [215] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 209, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [216] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 211, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [217] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 211, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [218] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [219] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [220] StringParser("(")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [221] StringParser(")")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [222] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [223] Meta("dedent")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [224] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 216, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [225] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 216, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [226] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 216, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [227] StringParser("ALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 216, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [228] StringParser("ALTER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 216, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [229] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 222, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [230] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 225, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [231] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 225, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [232] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 225, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [233] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 225, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [234] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 225, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [235] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 225, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [236] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 227, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [237] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 229, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [238] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 229, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [239] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 229, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [240] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 229, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [241] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [242] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [243] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [244] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [245] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 235, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [246] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 235, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [247] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 235, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [248] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 235, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [249] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 236, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [250] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 236, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [251] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 240, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [252] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 240, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [253] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 240, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [254] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 240, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [255] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 240, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [256] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 244, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [257] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 247, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [258] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 247, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [259] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 247, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [260] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 249, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [261] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 249, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [262] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 249, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [263] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 253, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [264] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [265] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [266] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [267] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [268] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 255, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [269] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [270] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [271] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [272] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [273] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [274] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [275] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [276] Ref(AlterTableDropColumnGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [277] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 264, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [278] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 264, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [279] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 266, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [280] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 266, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [281] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 266, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [282] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 266, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [283] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [284] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [285] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [286] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [287] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 8, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [288] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 279, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [289] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 282, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [290] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 282, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [291] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 282, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [292] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 282, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [293] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [294] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [295] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [296] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [297] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 287, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [298] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [299] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [300] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [301] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [302] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [303] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [304] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 296, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [305] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 296, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [306] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 296, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [307] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 296, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [308] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 296, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [309] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [310] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [311] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [312] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [313] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [314] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [315] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [316] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 306, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [317] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 306, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [318] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 306, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [319] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 306, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [320] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 308, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [321] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [322] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [323] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [324] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [325] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [326] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 315, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [327] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 315, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [328] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 315, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [329] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 315, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [330] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 318, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [331] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 318, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [332] Ref(AuthorizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 318, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [333] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [334] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [335] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 323, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [336] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 323, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [337] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 323, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [338] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 325, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [339] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 325, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [340] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 325, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [341] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 328, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [342] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 328, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [343] Ref(PropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 328, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [344] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 330, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [345] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [346] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [347] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [348] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 333, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [349] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [350] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [351] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 337, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [352] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 340, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [353] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 342, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [354] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 345, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [355] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 345, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [356] Ref(ExecuteArrowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 345, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [357] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 345, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [358] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 347, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [359] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 347, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [360] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 347, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [361] StringParser("&")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 347, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [362] StringParser("ANALYZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 347, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [363] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [364] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [365] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 350, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [366] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 352, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [367] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 352, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [368] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 355, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [369] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 357, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [370] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [371] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [372] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [373] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [374] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [375] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [376] StringParser("ANY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 10, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [377] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [378] Ref(PlusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [379] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [380] Ref(DivideSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [381] Ref(MultiplySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [382] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [383] Ref(BitwiseAndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [384] Ref(BitwiseOrSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [385] Ref(BitwiseXorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [386] Ref(BitwiseLShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [387] Ref(BitwiseRShiftSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [388] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 373, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [389] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 375, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [390] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 377, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [391] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 377, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [392] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 377, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [393] Ref(SliceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 377, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [394] StringParser("[")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 377, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [395] StringParser("]")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 377, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [396] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 377, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [397] StringParser("ARRAY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 377, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [398] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 380, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [399] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 382, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [400] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 382, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [401] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 384, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [402] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 387, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [403] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 387, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [404] StringParser("<")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 387, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [405] StringParser(">")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 387, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [406] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 390, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [407] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 390, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [408] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 392, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [409] Ref(ArrayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 392, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [410] Ref(ArrayTypeSchemaSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 392, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [411] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 393, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [412] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 393, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [413] StringParser("AS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 393, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [414] StringParser("ASC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 393, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [415] StringParser("AT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 393, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [416] StringParser("AUTHORIZATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 393, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [417] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 394, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [418] Ref(Auto_incrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 394, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [419] MultiStringParser(5 templates)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 394, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 2, _padding: 0 }, // [420] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [421] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [422] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [423] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [424] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [425] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [426] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [427] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [428] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [429] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [430] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [431] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [432] Token("file")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [433] Token("base")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [434] StringParser("BERNOULLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [435] StringParser("BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [436] StringParser("BIGINT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 403, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [437] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 408, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [438] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 408, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [439] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 408, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [440] Ref(BooleanBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 408, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [441] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 408, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [442] Ref(RightArrowOperator)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 408, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [443] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 408, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [444] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 408, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [445] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 410, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [446] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 410, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [447] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 410, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [448] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 410, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [449] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [450] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [451] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 412, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [452] StringParser("^")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [453] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 414, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [454] Ref(AndOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 414, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [455] Ref(OrOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 414, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [456] StringParser("BOOLEAN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 414, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [457] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [458] Ref(TrueSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [459] Ref(FalseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 416, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [460] StringParser("BOTH")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [461] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 419, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [462] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 421, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [463] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 421, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [464] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 424, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [465] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 426, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [466] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 426, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [467] Token("bracketed")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 426, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [468] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 429, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [469] Ref(UnorderedSetExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 429, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [470] StringParser("BY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 429, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [471] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 432, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [472] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 432, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [473] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [474] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 436, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [475] Ref(CTEColumnList)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 436, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [476] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [477] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 439, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [478] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 439, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [479] StringParser("CALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 439, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [480] StringParser("CASCADE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 439, child_count: 2, min_times: 0, first_terminator_idx: 11, terminator_count: 3, _padding: 0 }, // [481] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 441, child_count: 6, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [482] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 447, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [483] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 447, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [484] Meta("implicit_indent")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 447, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 2, _padding: 0 }, // [485] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [486] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [487] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [488] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 1, _padding: 0 }, // [489] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [490] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [491] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 7, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [492] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 455, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [493] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 455, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [494] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 455, child_count: 1, min_times: 0, first_terminator_idx: 8, terminator_count: 2, _padding: 0 }, // [495] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [496] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [497] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [498] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 1, _padding: 0 }, // [499] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [500] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [501] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [502] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [503] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [504] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [505] StringParser("CASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [506] StringParser("CAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [507] StringParser("::")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [508] StringParser("CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [509] StringParser("CATALOGS")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [510] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [511] StringParser("CHAR")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [512] Token("raw")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [513] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 456, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [514] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 458, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [515] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 458, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 1, _padding: 0 }, // [516] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 460, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [517] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 460, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [518] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 460, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [519] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 460, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [520] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 460, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [521] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 460, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [522] StringParser(":")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 460, child_count: 4, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [523] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [524] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [525] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [526] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [527] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [528] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 466, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [529] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 468, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [530] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 468, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [531] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 468, child_count: 10, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [532] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 478, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [533] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 480, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [534] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 480, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [535] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 480, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [536] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 482, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [537] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 482, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [538] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 485, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [539] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 485, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [540] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 487, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [541] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 487, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [542] Ref(ColumnConstraintDefaultGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 487, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [543] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 489, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [544] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 489, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [545] Ref(NotEnforcedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 489, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [546] Ref(UniqueKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 489, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [547] Ref(AutoIncrementGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 489, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [548] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 491, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [549] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 491, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [550] Ref(NotEnforcedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 491, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [551] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 491, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [552] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 493, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [553] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 493, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [554] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 493, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [555] Ref(ColumnGeneratedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 493, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [556] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 496, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [557] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 496, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [558] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 496, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [559] AnySetOf(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [560] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [561] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [562] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [563] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [564] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 503, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [565] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 503, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [566] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 506, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [567] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 508, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [568] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [569] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [570] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [571] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [572] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [573] StringParser("COLUMN")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 1, _padding: 0 }, // [574] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 513, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [575] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 513, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [576] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 513, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [577] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 513, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [578] Nothing
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 513, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [579] Ref(ColumnsExpressionNameGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 513, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [580] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 513, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [581] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 513, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [582] StringParser("COLUMNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 513, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [583] StringParser(",")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 513, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [584] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 515, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [585] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 515, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [586] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 515, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [587] StringParser("COMMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 515, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [588] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 518, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [589] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 518, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [590] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 518, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [591] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 520, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [592] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [593] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 524, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [594] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 526, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [595] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 526, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [596] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 526, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [597] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 526, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [598] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 528, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [599] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 528, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [600] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 528, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [601] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 530, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [602] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 530, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [603] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 532, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [604] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 532, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [605] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 532, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [606] Token("comment")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 532, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [607] StringParser("COMMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 532, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [608] StringParser("COMMITTED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 532, child_count: 8, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [609] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 540, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [610] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 540, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [611] Ref(GreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 540, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [612] Ref(LessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 540, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [613] Ref(GreaterThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 540, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [614] Ref(LessThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 540, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [615] Ref(NotEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 540, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [616] Ref(LikeOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 540, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [617] Ref(IsDistinctFromGrammar)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 540, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [618] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 540, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [619] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 540, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [620] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 540, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [621] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 542, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [622] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 542, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [623] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 542, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [624] Ref(CrossKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 542, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [625] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [626] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [627] Ref(ConditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [628] Ref(NonStandardJoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [629] StringParser("CONDITIONAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [630] StringParser("CONSTRAINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [631] StringParser("COPARTITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [632] StringParser("COUNT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 545, child_count: 10, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [633] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [634] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [635] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [636] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 560, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [637] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 560, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [638] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 560, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [639] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 560, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [640] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 560, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [641] Ref(SpecificKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 560, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [642] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 564, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [643] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 564, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [644] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 564, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [645] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 564, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [646] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 566, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [647] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [648] Ref(InstanceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [649] Ref(StaticKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [650] Ref(ConstructorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [651] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [652] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [653] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 569, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [654] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 571, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [655] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 571, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [656] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 571, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [657] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 573, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [658] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 573, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [659] Ref(AssignmentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 573, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [660] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 577, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [661] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 577, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [662] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 577, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [663] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 577, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [664] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 577, child_count: 9, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [665] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 586, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [666] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 586, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [667] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 586, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [668] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 586, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [669] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 586, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [670] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 586, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [671] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 586, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [672] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 586, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [673] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [674] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [675] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [676] Ref(FunctionDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 588, child_count: 9, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [677] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [678] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [679] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [680] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [681] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [682] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [683] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [684] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [685] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [686] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 598, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [687] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 601, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [688] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 603, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [689] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 603, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [690] StringParser("CREATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 603, child_count: 8, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [691] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 611, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [692] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 611, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [693] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 611, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [694] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 611, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [695] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 611, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [696] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 611, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [697] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 613, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [698] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 613, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [699] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 616, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [700] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 618, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [701] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [702] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [703] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [704] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 623, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [705] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 623, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [706] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 626, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [707] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [708] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [709] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [710] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [711] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [712] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [713] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [714] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [715] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [716] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 636, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [717] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 636, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [718] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 636, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [719] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 636, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [720] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 636, child_count: 7, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [721] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 643, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [722] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 646, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [723] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 646, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [724] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 646, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [725] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 646, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [726] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [727] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [728] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [729] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [730] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [731] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [732] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 651, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [733] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [734] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [735] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [736] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [737] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [738] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [739] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [740] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [741] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [742] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [743] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [744] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 659, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [745] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 660, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [746] Ref(CreateSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 660, child_count: 10, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [747] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [748] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [749] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [750] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [751] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [752] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 670, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [753] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 673, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [754] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 675, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [755] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 678, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [756] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 678, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [757] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 678, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [758] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 681, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [759] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 681, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [760] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 681, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [761] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [762] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 685, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [763] Ref(IncludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 685, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [764] Ref(ExcludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 685, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [765] Ref(PropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 685, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [766] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 685, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [767] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 687, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [768] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 687, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [769] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 690, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [770] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 692, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [771] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 695, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [772] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 695, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [773] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 695, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [774] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 695, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [775] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [776] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [777] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 699, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [778] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [779] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 702, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [780] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 705, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [781] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 705, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [782] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 705, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [783] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 705, child_count: 9, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [784] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [785] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [786] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [787] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 714, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [788] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [789] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [790] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [791] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [792] Ref(InsteadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [793] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 1, _padding: 0 }, // [794] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 721, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [795] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [796] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [797] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 724, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [798] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [799] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [800] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 727, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 2, _padding: 0 }, // [801] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [802] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [803] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [804] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 729, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [805] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 729, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [806] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 729, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [807] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 729, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [808] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 729, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [809] AnyNumberOf(5 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 734, child_count: 9, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [810] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 743, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [811] Ref(ReferencingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 743, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [812] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 743, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [813] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 743, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [814] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 743, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [815] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 743, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [816] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 743, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [817] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 743, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [818] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 743, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [819] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 743, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [820] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 745, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [821] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 745, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [822] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 745, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [823] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [824] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 749, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [825] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 749, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [826] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 749, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [827] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 751, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [828] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [829] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 753, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [830] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 755, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [831] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 755, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [832] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 755, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [833] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 757, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [834] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 757, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [835] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 757, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [836] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [837] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [838] Ref(EachKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [839] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [840] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [841] Ref(StatementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [842] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 764, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [843] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 764, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [844] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 767, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [845] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 767, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [846] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 771, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [847] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 771, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [848] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 771, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [849] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 771, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [850] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 771, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [851] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 774, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [852] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 774, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [853] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 774, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [854] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 774, child_count: 9, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [855] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [856] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [857] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [858] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [859] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [860] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [861] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [862] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 783, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [863] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 785, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [864] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 788, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [865] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 788, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [866] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 788, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [867] StringParser("CROSS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 788, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [868] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 788, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [869] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 788, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [870] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 790, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [871] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 792, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [872] Ref(CubeFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 792, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [873] Ref(RollupFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 792, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [874] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 795, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [875] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 795, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [876] StringParser("CURRENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 795, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [877] StringParser("CURRENT_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 795, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [878] StringParser("CURRENT_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 795, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [879] StringParser("CURRENT_PATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 795, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [880] StringParser("CURRENT_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 795, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [881] StringParser("CURRENT_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 795, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [882] StringParser("CURRENT_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 795, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [883] StringParser("CURRENT_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 795, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [884] StringParser("CURRENT_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 795, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [885] StringParser("DATA")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 795, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [886] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [887] RegexParser
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 797, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [888] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [889] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 798, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [890] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 802, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [891] Ref(PrimitiveTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 802, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [892] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 802, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [893] Ref(MapTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 802, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [894] Ref(RowTypeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 802, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [895] StringParser("DATE")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 802, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [896] MultiStringParser(1 templates)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 802, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [897] Ref(DatePartFunctionName)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 802, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [898] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [899] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 806, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [900] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 808, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [901] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 810, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [902] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 810, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [903] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 810, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [904] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 812, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [905] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 814, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [906] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 817, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [907] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 817, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [908] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 817, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [909] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 817, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [910] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 817, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [911] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 817, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [912] MultiStringParser(11 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 817, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [913] StringParser("DAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 817, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [914] StringParser("DEALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 817, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [915] StringParser("DECIMAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 817, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [916] StringParser("DEFAULT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 817, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [917] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [918] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [919] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 819, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [920] StringParser("DEFINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 819, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [921] StringParser("DEFINER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 819, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [922] StringParser("DELETE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [923] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [924] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [925] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [926] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [927] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [928] StringParser("DENY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [929] StringParser("DESC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [930] StringParser("DESCRIBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [931] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [932] Ref(DescribeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [933] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [934] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [935] StringParser("DESCRIPTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [936] StringParser("DISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [937] StringParser("DISTRIBUTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [938] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [939] StringParser(".")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [940] StringParser("DOUBLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 825, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [941] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 827, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [942] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 827, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [943] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 827, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [944] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [945] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [946] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [947] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [948] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [949] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [950] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [951] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [952] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 841, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [953] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 841, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [954] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 841, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [955] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 841, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [956] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 841, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [957] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 841, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [958] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [959] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [960] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [961] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [962] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 845, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [963] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 850, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [964] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 850, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [965] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 850, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [966] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 850, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [967] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 850, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [968] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 850, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [969] StringParser("DROP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 850, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [970] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 854, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [971] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 854, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [972] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 854, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [973] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 854, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [974] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 854, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [975] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 858, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [976] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 858, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [977] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 858, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [978] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 858, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [979] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 858, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [980] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [981] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [982] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [983] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [984] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [985] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [986] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [987] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [988] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [989] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 6, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [990] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 872, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [991] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 872, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [992] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 872, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [993] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 872, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [994] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 872, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [995] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 874, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [996] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 874, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [997] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 874, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [998] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 878, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [999] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 878, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1000] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 878, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1001] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 878, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1002] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 878, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1003] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 883, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1004] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 883, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1005] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 883, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1006] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 883, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1007] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 883, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1008] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 883, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1009] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1010] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1011] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 887, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1012] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1013] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1014] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1015] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1016] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1017] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1018] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1019] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1020] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 896, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1021] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 896, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1022] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 896, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1023] StringParser("ELSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 896, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1024] StringParser("EMPTY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 896, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1025] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 898, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1026] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1027] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1028] Ref(EmptyStructLiteralBracketsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1029] StringParser("ENCODING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1030] StringParser("}")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1031] StringParser("END")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1032] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1033] StringParser("ERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1034] StringParser("ESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1035] StringParser("EXCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1036] StringParser("EXCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1037] StringParser("=>")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1038] StringParser("EXECUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1039] StringParser("EXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1040] StringParser("EXPLAIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 900, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1041] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1042] Ref(ExplainKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1043] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1044] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1045] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1046] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1047] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 906, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1048] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 906, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1049] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 908, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1050] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 908, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1051] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 10, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1052] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 919, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1053] Ref(LikeExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 919, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1054] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1055] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1056] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1057] Ref(InOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1058] Ref(QuantifiedComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1059] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 924, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1060] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 924, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1061] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 924, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1062] Ref(IsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 924, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1063] Ref(IsNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 924, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1064] Ref(NotNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 924, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1065] Ref(CollateGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 924, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1066] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1067] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1068] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1069] Ref(Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1070] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1071] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1072] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1073] Ref(PatternMatchingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1074] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1075] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 934, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1076] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 935, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1077] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 936, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1078] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 936, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1079] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 936, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1080] Ref(NotOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 936, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1081] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 938, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1082] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 938, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1083] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 939, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1084] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 940, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1085] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 942, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1086] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1087] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1088] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1089] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1090] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1091] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 947, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1092] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 948, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1093] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 949, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1094] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 949, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1095] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 949, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 1, _padding: 0 }, // [1096] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 952, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1097] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 954, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1098] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 954, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1099] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 957, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1100] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 957, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1101] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1102] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 961, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1103] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 961, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1104] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 961, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1105] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1106] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1107] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1108] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1109] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 964, child_count: 9, min_times: 0, first_terminator_idx: 20, terminator_count: 1, _padding: 0 }, // [1110] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1111] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1112] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1113] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 976, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1114] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1115] Ref(LateralColumnAliasExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1116] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1117] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1118] Delimited(5 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 982, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1119] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1120] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1121] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1122] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1123] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1124] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1125] Ref(Expression_D_Potential_Select_Statement_Without_Brackets)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1126] Ref(SingleIdentifierWildcardSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1127] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 989, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1128] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1129] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1130] Ref(MapTypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1131] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 994, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1132] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1133] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1134] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 998, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1135] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 998, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1136] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1137] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1138] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1139] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1140] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1141] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1142] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1143] Ref(ListComprehensionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1144] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1145] Ref(AccessorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 7, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1146] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1147] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1148] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1149] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1150] Ref(TypedStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1151] Ref(ArrayExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1152] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1153] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1154] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1155] StringParser("EXTRACT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1156] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1010, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1157] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1010, child_count: 5, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1158] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1015, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1159] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1015, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1160] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1017, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1161] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1017, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1162] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1017, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1163] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1164] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1019, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1165] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1020, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1166] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1020, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1167] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1022, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1168] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1022, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1169] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1022, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1170] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1171] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1172] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1026, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1173] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1026, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1174] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1026, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1175] StringParser("FETCH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1026, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1176] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1029, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1177] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1030, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1178] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 1030, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1179] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1032, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1180] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1032, child_count: 1, min_times: 1, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1181] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1033, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1182] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1033, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1183] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1034, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1184] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1034, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1185] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1186] Ref(FilterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1187] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1039, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1188] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1189] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1190] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1191] StringParser("FILTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1192] StringParser("FINAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1193] StringParser("FIRST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1194] StringParser("FOLLOWING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1195] StringParser("FOR")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1196] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1197] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1198] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1199] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1044, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1200] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1046, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1201] Ref(EncodingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1046, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1202] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1049, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1203] Ref(Utf8KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1049, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1204] Ref(Utf16KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1049, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1205] Ref(Utf32KeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1049, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1206] StringParser("FORMAT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1049, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1207] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1051, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1208] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1051, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1209] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1053, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1210] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1055, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1211] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1212] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1213] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1214] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1059, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1215] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1062, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1216] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1062, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1217] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1062, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1218] Ref(UnboundedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1062, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1219] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1064, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1220] Ref(PrecedingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1064, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1221] Ref(FollowingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1064, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1222] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1068, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1223] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1068, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1224] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1068, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1225] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1070, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1226] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1070, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1227] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1070, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1228] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1072, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1229] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1072, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1230] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1074, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1231] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1074, child_count: 11, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1232] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1233] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1234] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1235] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1236] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1087, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1237] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1087, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1238] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1087, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1239] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1089, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1240] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1089, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1241] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1089, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1242] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1089, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1243] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1089, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1244] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1089, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1245] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1089, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1246] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1089, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1247] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1089, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1248] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1091, child_count: 7, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1249] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1250] Ref(PreTableFunctionKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1098, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1251] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1252] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1103, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1253] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1103, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1254] Ref(TemporalQuerySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1103, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1255] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1256] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1257] Ref(FromClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1258] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1259] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1260] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1108, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1261] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1111, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1262] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1111, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1263] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1111, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1264] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1111, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1265] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1111, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1266] Ref(PostTableExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1111, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1267] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1114, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1268] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1116, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1269] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1117, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1270] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1117, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1271] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1119, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1272] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1122, child_count: 6, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1273] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1128, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 2, _padding: 0 }, // [1274] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1131, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1275] Ref(MLTableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1131, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1276] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1131, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1277] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1134, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1278] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1134, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1279] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1136, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1280] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1136, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1281] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1136, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1282] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1138, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1283] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1138, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1284] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1138, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1285] Meta(conditional: indented_joins=True -> indent)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1138, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 2, _padding: 0 }, // [1286] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1139, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1287] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1140, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1288] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1142, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1289] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1142, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1290] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1142, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1291] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1292] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1293] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1294] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1146, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1295] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1146, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1296] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1146, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1297] Meta(conditional: indented_joins=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1146, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1298] StringParser("FROM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1146, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1299] StringParser("FULL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1146, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1300] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1148, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1301] Ref(LambdaExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1148, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1302] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1148, child_count: 15, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1303] AnyNumberOf(15 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1163, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1304] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1163, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1305] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1166, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1306] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1166, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1307] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1166, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1308] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1166, child_count: 4, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1309] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1170, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1310] Ref(TrimParametersGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1170, child_count: 1, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1311] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1171, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1312] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1171, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1313] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1171, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1314] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1171, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1315] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1316] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1317] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1318] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1319] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1320] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1321] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1178, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1322] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1178, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1323] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1180, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1324] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1180, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1325] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1326] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1327] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1328] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1185, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1329] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1188, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1330] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1188, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1331] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1188, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1332] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1188, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1333] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1188, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1334] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1191, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1335] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1191, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1336] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1191, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1337] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1191, child_count: 5, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1338] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1339] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1340] Ref(FormatJsonEncodingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1341] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1342] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1196, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1343] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1198, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1344] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1201, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1345] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1201, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1346] Ref(ArrayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1201, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1347] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1201, child_count: 4, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1348] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1205, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1349] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1205, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1350] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1207, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1351] Ref(ConditionalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1207, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1352] Ref(UnconditionalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1207, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1353] Ref(ArrayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1207, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1354] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1207, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1355] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1210, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1356] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1357] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1358] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1359] Ref(ColonDelimiterSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1360] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1215, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1361] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1362] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1220, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1363] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1220, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1364] Ref(AbsentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1220, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1365] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1220, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1366] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1220, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1367] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1223, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1368] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1223, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1369] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1223, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1370] Ref(FormatJsonEncodingGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1223, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1371] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1226, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1372] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1228, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1373] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1228, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1374] Ref(AbsentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1228, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1375] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1228, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1376] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1228, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1377] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1231, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1378] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1231, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1379] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1231, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1380] Ref(FormatJsonEncodingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1231, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1381] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1231, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1382] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1231, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1383] Ref(EmptyStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1231, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1384] Ref(ListaggOverflowClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1231, child_count: 1, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1385] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1386] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1235, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1387] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1235, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1388] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1389] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1390] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1238, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1391] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1240, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1392] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1240, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1393] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1240, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1394] StringParser("FUNCTION")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1240, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1395] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1240, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1396] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1242, child_count: 1, min_times: 0, first_terminator_idx: 25, terminator_count: 1, _padding: 0 }, // [1397] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1243, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1398] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1245, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1399] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1245, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1400] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1245, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1401] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1245, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 1, _padding: 0 }, // [1402] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1247, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1403] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1247, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1404] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1247, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1405] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1247, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1406] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1249, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1407] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1251, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1408] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1251, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1409] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1253, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1410] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1411] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1412] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1413] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1414] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1257, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1415] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1416] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1417] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1418] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1419] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1262, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1420] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1264, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1421] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1264, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1422] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1267, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1423] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1269, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1424] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1269, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1425] Ref(DateTimeFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1269, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1426] Ref(ColumnsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1269, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1427] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1271, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1428] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1273, child_count: 1, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1429] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1274, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1430] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1277, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1431] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1277, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1432] Ref(ColumnsExpressionFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1277, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1433] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1277, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1434] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1277, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1435] Ref(PostFunctionGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1277, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1436] StringParser("FUNCTIONS")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1277, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1437] TypedParser("glob_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1277, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1438] StringParser("GRACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1277, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1439] StringParser("GRANT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1277, child_count: 6, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1440] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1283, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1441] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1283, child_count: 4, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1442] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1287, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1443] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1444] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1445] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1446] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1447] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1292, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1448] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1292, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1449] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1292, child_count: 4, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1450] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1296, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1451] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1296, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1452] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1296, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1453] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1296, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1454] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1296, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1455] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1296, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1456] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1296, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1457] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1296, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1458] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1459] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1302, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1460] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1302, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1461] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1302, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1462] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1302, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1463] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1305, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1464] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1305, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1465] Ref(AdminKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1305, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1466] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1305, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1467] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1308, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1468] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1308, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1469] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1308, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1470] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1308, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1471] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1311, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1472] Ref(GrantedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1311, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1473] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1311, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1474] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1314, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1475] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1314, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1476] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1314, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1477] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1314, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1478] StringParser("GRANTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1314, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1479] StringParser("GRANTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1314, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1480] StringParser("GRAPHVIZ")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1314, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1481] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1316, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1482] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1316, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1483] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1316, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1484] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1316, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1485] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1321, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1486] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1321, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1487] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1321, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1488] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1489] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1490] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1491] Ref(GroupingSetsClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 1, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1492] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1326, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 1, _padding: 0 }, // [1493] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1494] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1331, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1495] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1331, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1496] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1331, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1497] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1331, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1498] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1331, child_count: 6, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1499] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1337, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1500] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1339, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1501] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1339, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1502] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1339, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1503] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1339, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1504] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1339, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1505] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1339, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1506] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1339, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1507] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1339, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1508] StringParser("GROUP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1339, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1509] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1340, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 1, _padding: 0 }, // [1510] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1342, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1511] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1346, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1512] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1346, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1513] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1346, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1514] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1346, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1515] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1348, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1516] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1348, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1517] StringParser("GROUPING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1348, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1518] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1351, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1519] Ref(GroupingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1351, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1520] Ref(SetsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1351, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1521] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1354, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1522] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1356, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1523] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1524] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1525] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1526] StringParser("GROUPS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1527] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1362, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1528] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1362, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1529] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1364, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1530] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1367, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1531] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1367, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1532] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1372, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1533] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1374, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1534] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1374, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1535] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1374, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1536] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1374, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1537] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1374, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1538] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1374, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1539] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1374, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1540] StringParser("HAVING")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1374, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1541] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1374, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1542] StringParser("HOUR")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1374, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1543] Token("identifier")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1374, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1544] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1376, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1545] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1376, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1546] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1376, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1547] StringParser("IF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1376, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1548] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1379, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1549] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1379, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1550] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1379, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1551] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1379, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1552] StringParser("IGNORE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1379, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1553] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1381, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1554] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1383, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1555] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1383, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1556] Ref(RespectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1383, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1557] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1383, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1558] StringParser("IMMEDIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1383, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1559] StringParser("IN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1383, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1560] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1386, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1561] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1386, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1562] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1386, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1563] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1388, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1564] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1565] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1393, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1566] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1567] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1568] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1569] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1570] StringParser("INCLUDING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1571] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1572] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1397, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1573] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1399, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1574] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1399, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1575] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1399, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1576] StringParser("INITIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1399, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1577] StringParser("INNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1399, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1578] StringParser("INPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1399, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1579] StringParser("INSERT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1399, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1580] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1403, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1581] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1403, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1582] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1403, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1583] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1403, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1584] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1585] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1586] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1587] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1588] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1589] Ref(DefaultValuesGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1590] StringParser("INT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1591] StringParser("INTEGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1592] StringParser("INTERSECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1593] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1411, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1594] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1411, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1595] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1411, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1596] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1597] Ref(YearKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1598] Ref(MonthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1599] Ref(DayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1600] Ref(HourKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1601] Ref(MinuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1602] Ref(SecondKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1603] StringParser("INTERVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1604] StringParser("INTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1605] StringParser("INVOKER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1606] StringParser("IO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1607] StringParser("IPADDRESS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1608] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1609] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1610] Ref(NanLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1611] Ref(UnknownLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1612] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1613] Ref(NormalizedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1614] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1426, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1615] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1426, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1616] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1426, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1617] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1426, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1618] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1426, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1619] StringParser("IS")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1426, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1620] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1426, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1621] StringParser("ISOLATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1426, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1622] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1429, child_count: 7, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1623] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1436, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1624] Ref(ConditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1436, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1625] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1436, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1626] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1436, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1627] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1628] Ref(NestedJoinGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1437, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1629] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1441, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1630] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1441, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1631] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1441, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1632] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1443, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1633] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1443, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1634] Ref(JoinUsingConditionGrammar)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1443, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1635] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1443, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1636] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1449, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1637] Ref(UnconditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1449, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1638] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1449, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1639] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1449, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1640] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1449, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1641] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1453, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1642] Ref(ExtendedNaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1453, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1643] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1453, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1644] StringParser("JOIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1453, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1645] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1646] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1647] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1648] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1458, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1649] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1458, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1650] Meta(conditional: indented_on_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1458, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1651] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1652] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1653] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1654] Meta(conditional: indented_on_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1463, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1655] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1465, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1656] Ref(InnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1465, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1657] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1467, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1658] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1659] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1660] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1661] Ref(RightKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1470, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1662] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1663] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1474, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1664] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1474, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1665] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1477, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1666] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1479, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1667] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1479, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1668] StringParser("JSON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1479, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1669] StringParser("JSON_ARRAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1479, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1670] StringParser("JSON_EXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1479, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1671] StringParser("JSON_OBJECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1479, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1672] StringParser("JSON_QUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1479, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1673] StringParser("JSON_TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1479, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1674] StringParser("JSON_VALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1479, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1675] StringParser("KEEP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1479, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1676] StringParser("KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1479, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1677] StringParser("KEYS")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1479, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1678] Token("keyword")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1479, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1679] StringParser("->")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1479, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1680] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1482, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1681] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1484, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1682] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1484, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1683] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1487, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1684] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1685] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1686] Ref(LambdaArrowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1687] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1688] StringParser("LAST")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1689] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1690] StringParser("LATERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1691] StringParser("LEADING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1692] StringParser("LEFT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1693] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1491, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1694] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1491, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1695] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1491, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1696] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1491, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1697] StringParser("LEVEL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1491, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1698] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1494, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1699] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1496, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1700] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1496, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1701] Ref(LikeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1496, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1702] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1496, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1703] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1704] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1705] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1498, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1706] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1707] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1499, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1708] StringParser("LIKE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1499, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1709] TypedParser("like_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1710] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1504, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1711] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1504, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1712] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1506, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1713] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1509, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1714] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1715] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1716] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1717] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1512, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1718] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1514, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1719] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1516, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1720] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1516, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1721] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1518, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1722] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1518, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1723] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1518, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1724] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1520, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1725] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1520, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1726] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1520, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1727] StringParser("LIMIT")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1520, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1728] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1520, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1729] StringParser("LISTAGG")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1520, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1730] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1523, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1731] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1523, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1732] Ref(OverflowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1523, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1733] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1525, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1734] Ref(ErrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1525, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1735] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1529, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1736] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1529, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1737] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1529, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1738] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1739] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1740] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1741] Ref(CountKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 9, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1742] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1743] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1744] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1745] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1746] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1747] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1748] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1749] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1750] Ref(TypedArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1751] Ref(ObjectLiteralSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1752] Token("literal")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1753] Token("literal")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1754] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1755] StringParser("LOCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1756] StringParser("LOCALTIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1757] StringParser("LOCALTIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1758] StringParser("LOGICAL")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1759] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1760] StringParser("MAP")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1761] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1542, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1762] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1545, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1763] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1764] Ref(PrimitiveTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1765] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1766] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1767] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1551, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1768] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1769] Ref(PrimitiveTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1770] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1771] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1772] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1556, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1773] Ref(MapKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1556, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1774] Ref(MapTypeSchemaSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1556, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1775] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1556, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1776] StringParser("MATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1556, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1777] StringParser("MATCH_RECOGNIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1556, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1778] StringParser("MATCHED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1556, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1779] StringParser("MATCHES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1556, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1780] StringParser("MATERIALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1556, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1781] StringParser("MEASURES")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1556, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1782] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1556, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1783] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1561, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1784] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1561, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1785] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1561, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1786] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1561, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1787] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1788] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1789] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1790] StringParser("MERGE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1563, child_count: 2, min_times: 1, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1791] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1565, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1792] Ref(MergeMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1565, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1793] Ref(MergeNotMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1565, child_count: 7, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1794] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1572, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1795] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1572, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1796] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1572, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1797] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1574, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1798] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1574, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1799] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1574, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1800] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1574, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1801] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1802] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1803] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 8, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1804] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1584, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1805] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1584, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1806] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1584, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1807] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1584, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1808] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1586, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1809] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1586, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1810] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1586, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1811] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1586, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1812] Ref(MergeInsertClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1586, child_count: 12, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1813] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1598, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1814] Ref(MergeIntoLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1598, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1815] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1816] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1817] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1818] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1819] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1820] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1821] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1604, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1822] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1606, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1823] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1609, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1824] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1609, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1825] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1609, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1826] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1827] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1828] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1829] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1830] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1831] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1832] Ref(MergeMatchSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1833] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1615, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1834] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1615, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1835] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1615, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1836] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1615, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1837] StringParser("MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1615, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1838] StringParser("%")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1615, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1839] StringParser("MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1615, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1840] StringParser("*")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1615, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1841] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1615, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1842] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1618, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1843] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1618, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1844] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1618, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1845] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1620, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1846] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1620, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1847] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1623, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1848] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1623, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1849] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1627, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1850] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1627, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1851] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1852] Ref(NamedWindowExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1853] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1629, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1854] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1855] Ref(NaturalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1856] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1857] StringParser("NATURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1858] StringParser("-")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1859] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1860] StringParser("NESTED")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1861] Token("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1862] StringParser("NEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1863] StringParser("NFC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1864] StringParser("NFD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1865] StringParser("NFKC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1866] StringParser("NFKD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1867] StringParser("NO")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1631, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1868] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1869] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1870] Ref(UnorderedSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1637, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1871] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1640, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1872] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1640, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1873] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1643, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1874] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1643, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1875] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1646, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1876] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1646, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1877] Ref(BracketedSetExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1646, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1878] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1646, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1879] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1650, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1880] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1650, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1881] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1650, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1882] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1650, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1883] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1650, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1884] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1653, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1885] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1653, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1886] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1655, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1887] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1888] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1889] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1890] StringParser("NONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1891] StringParser("NORMALIZE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1892] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1893] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1894] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1660, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1895] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1662, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1896] Ref(RawNotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1662, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1897] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1662, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1898] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1664, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1899] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1664, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1900] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1664, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1901] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1664, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1902] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1664, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1903] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1664, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1904] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1664, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1905] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1664, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1906] StringParser("NULLIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1664, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1907] StringParser("NULLS")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1664, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1908] TypedParser("numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1664, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1909] StringParser("OBJECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1664, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1910] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1911] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1912] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1913] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1914] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1670, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1915] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1672, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1916] Ref(ObjectLiteralElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1672, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1917] StringParser("{")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1672, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1918] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1674, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1919] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1674, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1920] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1676, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1921] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1676, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1922] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1676, child_count: 12, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1923] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1924] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1925] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1926] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1927] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1928] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1929] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1930] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1931] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1932] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1933] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1934] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1935] StringParser("OF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1936] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1693, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1937] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1693, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1938] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1695, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1939] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1695, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1940] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1941] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1696, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1942] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1698, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1943] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1698, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1944] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1698, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1945] StringParser("OFFSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1698, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1946] StringParser("OMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1698, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1947] StringParser("ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1698, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1948] StringParser("ONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1698, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1949] StringParser("ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1698, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1950] StringParser("OPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1698, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1951] StringParser("OR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1698, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1952] StringParser("OR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1698, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1953] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1700, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1954] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1700, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1955] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1700, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1956] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1957] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1958] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 2, _padding: 0 }, // [1959] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1707, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1960] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1711, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1961] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1962] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1963] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1964] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1714, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1965] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1716, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1966] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1716, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1967] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1716, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1968] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1718, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1969] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1718, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1970] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1971] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1972] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1973] Ref(WithFillSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1974] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1975] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1720, child_count: 6, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1976] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1977] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1978] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1979] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1980] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1981] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1982] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1983] StringParser("ORDER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1984] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1985] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1986] Ref(NoorderKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1987] StringParser("ORDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1988] StringParser("OUTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1989] StringParser("OUTPUT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1728, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1990] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1733, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1991] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1733, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1992] Ref(OverKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1733, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1993] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1994] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1995] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1738, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1996] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1738, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1997] StringParser("OVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1738, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1998] StringParser("OVERFLOW")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1738, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1999] Nothing
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1738, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2000] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1738, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2001] StringParser("?")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1738, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2002] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1743, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2003] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1743, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2004] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1743, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2005] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1745, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2006] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1748, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2007] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1750, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2008] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1750, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2009] StringParser("PARTITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1750, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2010] StringParser("PARTITIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1750, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2011] StringParser("PASSING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1750, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2012] StringParser("PAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1750, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2013] StringParser("PATH")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1750, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2014] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1752, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2015] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1754, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2016] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1754, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2017] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2018] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2019] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2020] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2021] StringParser("PATTERN")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2022] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2023] StringParser("PER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2024] StringParser("PERIOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2025] StringParser("PERMUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2026] StringParser("|")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2027] StringParser("PLAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2028] StringParser("+")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2029] StringParser("POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2030] StringParser("+")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1756, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2031] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1760, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2032] Ref(OverClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1760, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2033] Ref(FilterClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1760, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2034] Ref(WithinGroupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1760, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2035] Ref(WithOrdinalityClauseSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1760, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2036] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1760, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2037] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1760, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2038] StringParser("PRECEDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1760, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2039] StringParser("PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1760, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2040] StringParser("PREPARE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1760, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2041] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1760, child_count: 16, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2042] OneOf(16 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1776, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2043] Ref(BooleanKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1776, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2044] Ref(TinyintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1776, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2045] Ref(SmallintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1776, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2046] Ref(IntegerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1776, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2047] Ref(IntKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1776, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2048] Ref(BigintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1776, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2049] Ref(RealKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1776, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2050] Ref(DoubleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1776, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2051] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1778, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2052] Ref(DecimalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1778, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2053] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1778, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2054] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2055] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2056] Ref(CharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2057] Ref(VarcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2058] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2059] Ref(VarbinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2060] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2061] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2062] Ref(TimeWithTZGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2063] Ref(IpaddressKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2064] Ref(UuidKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2065] StringParser("PRIVILEGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2066] StringParser("PROPERTIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2067] StringParser("PRUNE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2068] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2069] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2070] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1784, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2071] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1787, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2072] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1787, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2073] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1790, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2074] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1790, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2075] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1790, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2076] Ref(SomeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1790, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2077] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2078] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2079] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2080] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2081] StringParser("QUOTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2082] StringParser("RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2083] StringParser("=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2084] StringParser(">")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2085] StringParser("<")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2086] StringParser("!")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2087] StringParser("READ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2088] StringParser("REAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2089] StringParser("RECURSIVE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1793, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2090] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1798, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2091] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1798, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2092] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1798, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2093] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1798, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2094] Ref(ReferenceMatchGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1798, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2095] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1800, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2096] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1803, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2097] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1803, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2098] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1803, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2099] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1803, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2100] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1806, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2101] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1806, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2102] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1806, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2103] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1806, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2104] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2105] Ref(MatchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1808, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2106] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1811, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2107] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1811, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2108] Ref(PartialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1811, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2109] Ref(SimpleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1811, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2110] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2111] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2112] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2113] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1818, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2114] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1818, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2115] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1818, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2116] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1820, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2117] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1820, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2118] Ref(ActionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1820, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2119] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2120] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2121] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2122] StringParser("REFRESH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2123] StringParser("RENAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2124] StringParser("REPEATABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2125] StringParser("REPLACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2126] StringParser("RESET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2127] StringParser("RESPECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2128] StringParser("RESTRICT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2129] StringParser("RETURNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2130] StringParser("REVOKE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1822, child_count: 6, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2131] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2132] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1828, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2133] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1831, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2134] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1831, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2135] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1831, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2136] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1831, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2137] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1835, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2138] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1838, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2139] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1838, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2140] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1838, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2141] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1838, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2142] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1840, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2143] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1840, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2144] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1840, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2145] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2146] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2147] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2148] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2149] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2150] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2151] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2152] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2153] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2154] StringParser("->")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2155] StringParser("RIGHT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2156] StringParser("ROLE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2157] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2158] StringParser("ROLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2159] StringParser("ROLLBACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2160] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2161] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2162] StringParser("ROW")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2163] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1847, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2164] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1849, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2165] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1850, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2166] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1852, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2167] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1852, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2168] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1854, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2169] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1854, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2170] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1854, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2171] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1856, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2172] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1856, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2173] Ref(RowTypeSchemaSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1856, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2174] StringParser("ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1856, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2175] StringParser("RUNNING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1856, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2176] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2177] Ref(TablesampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2178] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1862, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2179] Ref(BernoulliKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1862, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2180] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1862, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2181] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2182] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1865, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2183] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1867, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2184] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2185] Ref(RepeatableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2186] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2187] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2188] StringParser("SCALAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2189] StringParser("SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2190] StringParser("SCHEMAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2191] StringParser("SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2192] StringParser("SECURITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2193] StringParser("SEEK")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1871, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2194] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2195] Ref(WildcardExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2196] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2197] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2198] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2199] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1877, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2200] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1877, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2201] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 1877, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 1, _padding: 0 }, // [2202] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2203] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2204] Ref(SelectClauseModifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 1882, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2205] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1884, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2206] Ref(SelectClauseElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1884, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2207] Ref(SelectClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1884, child_count: 6, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2208] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1890, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2209] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::PrecededBy, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1891, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2210] PrecededBy(preceding_sequences=(('IS', 'DISTINCT'), ('IS', 'NOT', 'DISTINCT')))
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1891, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2211] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1891, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2212] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2213] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2214] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2215] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2216] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2217] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2218] StringParser("SELECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 10, min_times: 0, first_terminator_idx: 32, terminator_count: 3, _padding: 0 }, // [2219] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2220] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2221] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2222] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2223] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2224] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2225] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2226] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2227] Ref(OffsetClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2228] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2229] Ref(FetchClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2230] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2231] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2232] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1903, child_count: 4, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2233] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1907, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2234] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2235] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1912, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2236] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1912, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2237] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1914, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2238] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1917, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2239] Ref(WithCompoundNonSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1917, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2240] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1917, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2241] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1920, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2242] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1920, child_count: 4, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2243] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1924, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2244] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1924, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2245] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1924, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2246] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1924, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2247] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1926, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2248] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1928, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2249] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1928, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2250] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1928, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2251] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1928, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2252] StringParser(";")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1928, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2253] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2254] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1932, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2255] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1932, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2256] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1932, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2257] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2258] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2259] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1934, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2260] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1936, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2261] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1938, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2262] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1938, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2263] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1938, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2264] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1940, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2265] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1940, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2266] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1940, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2267] StringParser("SERIALIZABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1940, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2268] StringParser("SESSION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1940, child_count: 4, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2269] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1944, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2270] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1944, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2271] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1946, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2272] Ref(SetClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1946, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2273] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1949, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2274] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1949, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2275] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1949, child_count: 7, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2276] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2277] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2278] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2279] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2280] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2281] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2282] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2283] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 5, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2284] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1961, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2285] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1961, child_count: 1, min_times: 1, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2286] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2287] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2288] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2289] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2290] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2291] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2292] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2293] StringParser("SET")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1964, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2294] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1967, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2295] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2296] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1969, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2297] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1971, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2298] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1971, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2299] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1971, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2300] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1973, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2301] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1975, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2302] Ref(IntersectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1975, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2303] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1975, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2304] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1975, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2305] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1977, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2306] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1977, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2307] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1980, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2308] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1980, child_count: 4, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2309] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2310] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2311] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1984, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2312] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2313] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 6, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2314] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1990, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2315] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1990, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2316] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1990, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2317] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1992, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2318] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1992, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2319] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1992, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2320] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1992, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2321] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1992, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2322] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1992, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2323] StringParser("SETS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1992, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2324] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1994, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2325] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2326] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2327] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 1, min_times: 1, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2328] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1997, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2329] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2000, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2330] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2000, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2331] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2000, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2332] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2000, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2333] StringParser("SHOW")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2000, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2334] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2002, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2335] Ref(PositiveSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2002, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2336] Ref(NegativeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2002, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 1, _padding: 0 }, // [2337] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2338] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2339] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2340] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2341] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2006, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2342] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2006, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2343] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2009, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2344] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2009, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2345] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2009, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2346] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2009, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2347] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2009, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2348] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2349] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2350] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2351] StringParser("SKIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2352] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2353] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2354] StringParser("SMALLINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2355] StringParser("SOME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2356] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2357] StringParser("START")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 24, min_times: 0, first_terminator_idx: 36, terminator_count: 1, _padding: 0 }, // [2358] OneOf(24 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2359] Ref(AlterTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2360] Ref(AnalyzeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2361] Ref(CommentOnStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2362] Ref(CreateFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2363] Ref(CreateRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2364] Ref(CreateSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2365] Ref(CreateTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2366] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2367] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2368] Ref(DescribeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2369] Ref(DropFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2370] Ref(DropRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2371] Ref(DropSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2372] Ref(DropTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2373] Ref(DropViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2374] Ref(ExplainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2375] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2376] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2377] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2378] Ref(SetSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2379] Ref(TransactionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2380] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2381] Ref(UseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2382] Ref(SetSessionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2383] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2384] StringParser("STATS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 1, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2385] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2036, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2386] Ref(ConcatSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2036, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2387] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2388] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2041, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2389] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2043, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2390] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2043, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2391] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2043, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2392] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2043, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2393] StringParser("SUBSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2043, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2394] StringParser("SUBSTRING")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2043, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2395] Token("symbol")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2043, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2396] StringParser("SYSTEM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2043, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2397] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2045, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2398] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2047, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2399] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2047, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2400] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2047, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2401] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2050, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2402] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2052, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2403] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2052, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2404] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2052, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2405] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2054, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2406] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2054, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2407] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2054, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2408] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2057, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2409] Ref(ForeignKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2057, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2410] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2057, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2411] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2057, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2412] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2057, child_count: 6, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2413] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2414] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2415] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2416] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2417] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2063, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2418] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2066, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2419] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2066, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2420] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2069, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2421] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2069, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2422] StringParser("TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2069, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2423] StringParser("TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2069, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2424] StringParser("TABLESAMPLE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2069, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2425] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2071, child_count: 1, min_times: 0, first_terminator_idx: 37, terminator_count: 1, _padding: 0 }, // [2426] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2072, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2427] Ref(Expression_A_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2072, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2428] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2072, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2429] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2072, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2430] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2074, child_count: 1, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2431] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2432] Ref(Expression_B_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2433] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2434] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2435] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2436] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2437] StringParser("TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2438] StringParser("TEXT_STRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2439] StringParser("THEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2440] StringParser("TIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2441] StringParser("~")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2442] StringParser("TIME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2075, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2443] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2444] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2080, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2445] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2080, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2446] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2080, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2447] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2080, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2448] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2083, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2449] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2085, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2450] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2085, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2451] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2085, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2452] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2085, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2453] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2085, child_count: 1, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2454] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2086, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2455] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2456] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2457] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2458] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2459] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2460] StringParser("TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2461] StringParser("TINYINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2462] StringParser("TO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2463] StringParser("TRAILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2464] StringParser("TRANSACTION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2465] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2466] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2467] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2468] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2095, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2469] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2097, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2470] Ref(RollbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2097, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2471] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2097, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2472] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2100, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2473] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2100, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2474] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2100, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2475] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2102, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2476] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2104, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2477] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2107, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2478] Ref(IsolationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2107, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2479] Ref(LevelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2107, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2480] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2111, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2481] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2482] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2483] Ref(UncommittedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2113, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2484] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2115, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2485] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2115, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2486] Ref(CommittedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2115, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2487] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2117, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2488] Ref(RepeatableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2117, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2489] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2117, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2490] Ref(SerializableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2117, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2491] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2119, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2492] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2119, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2493] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2121, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2494] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2121, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2495] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2121, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2496] StringParser("TRIM")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2121, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2497] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2498] Ref(BothKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2499] Ref(LeadingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2500] Ref(TrailingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2501] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2502] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2503] StringParser("TRUNCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2124, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2504] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2505] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2506] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2507] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2508] StringParser("TRY_CAST")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2509] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2130, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2510] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2132, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2511] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2132, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2512] StringParser("TYPE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2132, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2513] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2134, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2514] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2134, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2515] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2134, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2516] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2136, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2517] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2136, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2518] Ref(StructLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2136, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2519] StringParser("UESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2136, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2520] StringParser("UNBOUNDED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2136, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2521] StringParser("UNCOMMITTED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2136, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2522] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2136, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2523] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2524] Ref(NaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2525] Ref(UnconditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2526] Ref(HorizontalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2527] StringParser("UNCONDITIONAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2528] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2529] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2141, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2530] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2143, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2531] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2143, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2532] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2143, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2533] StringParser("UNION")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2143, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2534] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2143, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2535] StringParser("UNIQUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2143, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2536] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2143, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2537] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2143, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2538] StringParser("UNMATCHED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2143, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2539] StringParser("UNNEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2143, child_count: 6, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2540] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2149, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2541] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2149, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2542] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2149, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2543] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2149, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2544] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2149, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2545] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2149, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2546] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2149, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2547] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2548] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2151, child_count: 1, min_times: 1, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2549] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2152, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2550] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2551] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2552] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2553] StringParser("UPDATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2154, child_count: 8, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2554] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2555] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2556] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 2162, child_count: 1, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2557] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2558] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2559] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2560] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2561] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2562] StringParser("USE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2163, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2563] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2165, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2564] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2165, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2565] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2165, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2566] StringParser("USER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2165, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2567] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2165, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2568] StringParser("USING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2165, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2569] StringParser("UTF16")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2165, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2570] StringParser("UTF32")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2165, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2571] StringParser("UTF8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2165, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2572] StringParser("UUID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2165, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2573] StringParser("VALIDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2165, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2574] StringParser("VALUE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2165, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2575] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2576] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2577] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2578] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2579] StringParser("VALUES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2580] StringParser("VARBINARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2581] StringParser("VARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2582] StringParser("VERBOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2583] StringParser("VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2584] StringParser("VIEW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 8, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2585] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2586] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2177, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2587] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2588] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2589] Meta(conditional: indented_then=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2590] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2591] Meta(conditional: indented_then_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2592] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2593] Meta(conditional: indented_then_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2594] Meta(conditional: indented_then=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2595] StringParser("WHEN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2180, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2596] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2184, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2597] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2184, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2598] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2186, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2599] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2600] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2189, child_count: 7, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2601] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2602] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2603] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2604] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2198, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2605] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2198, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2606] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2198, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2607] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2200, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2608] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2200, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2609] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2200, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2610] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2200, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2611] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2200, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2612] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2200, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2613] StringParser("WHERE")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2200, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2614] Token("whitespace")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2200, child_count: 1, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2615] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2201, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2616] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2201, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2617] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2203, child_count: 1, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2618] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2204, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2619] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2206, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2620] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2208, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2621] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2208, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2622] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2208, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2623] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2210, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2624] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2210, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2625] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2210, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2626] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2210, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2627] StringParser("WINDOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2210, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2628] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 2214, child_count: 1, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2629] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2215, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2630] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2217, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2631] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2217, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2632] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2217, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2633] Ref(PartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2217, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2634] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2217, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2635] Ref(FrameClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2217, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2636] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2223, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2637] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2223, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2638] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2223, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2639] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2223, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 1, _padding: 0 }, // [2640] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2641] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2642] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2643] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2644] Ref(NonWithNonSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2225, child_count: 6, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2645] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2231, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2646] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2231, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2647] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2231, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2648] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2231, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 1, _padding: 0 }, // [2649] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2650] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2651] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2652] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2653] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2654] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2236, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2655] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2236, child_count: 1, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2656] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2657] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2658] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2659] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2660] StringParser("WITH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2237, child_count: 4, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2661] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2662] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2663] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2664] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2665] Ref(BindingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2241, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2666] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2243, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2667] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2243, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2668] Ref(OrdinalityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2243, child_count: 4, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2669] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2247, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2670] Ref(WithinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2247, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2671] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2247, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2672] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2250, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2673] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2250, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2674] Ref(FilterClauseGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2250, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2675] StringParser("WITHIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2250, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2676] StringParser("WITHOUT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2250, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2677] Token("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2250, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2678] StringParser("WORK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2250, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2679] StringParser("WRAPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2250, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2680] StringParser("WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2250, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2681] StringParser("YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2250, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2682] StringParser("ZONE")
];

pub static CHILD_IDS: &[u32] = &[
    2, 3, 58, 4, 48, 5, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17,
    18, 25, 30, 35, 36, 41, 6, 7, 19, 20, 21, 22, 23, 24, 26, 27,
    28, 29, 31, 32, 33, 34, 37, 38, 39, 40, 42, 43, 44, 45, 46, 47,
    49, 55, 50, 51, 54, 52, 53, 59, 60, 61, 63, 71, 75, 78, 81, 86,
    64, 65, 66, 67, 68, 69, 70, 72, 73, 74, 76, 77, 79, 80, 82, 83,
    84, 85, 87, 130, 88, 97, 102, 105, 106, 107, 108, 109, 110, 111, 112, 113,
    114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 89, 90,
    91, 92, 95, 96, 93, 94, 98, 99, 100, 101, 103, 104, 128, 129, 132, 55,
    135, 136, 137, 138, 139, 140, 141, 142, 143, 144, 145, 148, 151, 154, 146, 147,
    149, 150, 152, 153, 155, 156, 158, 159, 160, 161, 162, 163, 164, 165, 166, 167,
    168, 171, 174, 177, 169, 170, 172, 173, 175, 176, 178, 179, 181, 182, 184, 188,
    192, 196, 200, 202, 204, 185, 186, 187, 55, 189, 190, 191, 55, 193, 194, 195,
    55, 197, 198, 199, 55, 201, 55, 203, 55, 206, 207, 213, 214, 215, 223, 216,
    222, 217, 218, 219, 220, 221, 225, 226, 230, 234, 235, 236, 241, 244, 231, 232,
    233, 237, 240, 238, 239, 242, 243, 246, 247, 248, 249, 250, 252, 253, 254, 255,
    257, 263, 276, 277, 258, 259, 260, 261, 262, 264, 267, 268, 269, 265, 266, 270,
    275, 271, 274, 272, 273, 278, 279, 282, 280, 281, 284, 285, 286, 287, 288, 289,
    293, 304, 309, 316, 330, 341, 349, 290, 291, 292, 294, 295, 296, 297, 298, 299,
    300, 301, 302, 303, 305, 306, 307, 308, 310, 311, 312, 313, 314, 315, 317, 318,
    319, 320, 321, 326, 322, 323, 324, 325, 327, 328, 329, 331, 332, 333, 334, 335,
    338, 336, 337, 339, 340, 342, 343, 344, 345, 55, 346, 347, 348, 350, 351, 352,
    358, 353, 220, 221, 354, 55, 355, 356, 357, 359, 360, 364, 365, 366, 367, 368,
    369, 220, 221, 370, 55, 371, 372, 373, 378, 379, 380, 381, 382, 383, 384, 385,
    386, 387, 389, 394, 395, 390, 393, 391, 392, 399, 394, 395, 400, 55, 402, 406,
    403, 404, 405, 407, 220, 221, 409, 410, 412, 418, 421, 422, 423, 424, 425, 426,
    427, 428, 429, 438, 439, 440, 441, 442, 446, 447, 450, 451, 454, 455, 458, 459,
    462, 220, 221, 463, 55, 465, 220, 221, 466, 55, 469, 220, 221, 472, 220, 221,
    474, 475, 476, 477, 478, 220, 221, 482, 492, 483, 484, 485, 489, 223, 491, 486,
    493, 494, 484, 495, 499, 223, 501, 496, 515, 516, 517, 518, 524, 525, 526, 527,
    529, 532, 530, 531, 533, 536, 540, 543, 546, 547, 548, 551, 552, 555, 534, 535,
    537, 538, 539, 220, 221, 541, 542, 544, 545, 549, 550, 553, 554, 557, 558, 559,
    560, 563, 564, 561, 562, 565, 566, 567, 220, 221, 568, 55, 569, 570, 571, 575,
    576, 585, 586, 589, 590, 591, 592, 601, 593, 598, 594, 597, 595, 596, 599, 600,
    602, 603, 604, 605, 610, 611, 612, 613, 614, 615, 616, 617, 622, 623, 626, 627,
    628, 634, 635, 636, 640, 641, 642, 652, 653, 654, 657, 637, 638, 639, 220, 221,
    643, 644, 645, 646, 647, 651, 648, 649, 650, 655, 656, 658, 659, 661, 662, 663,
    664, 666, 667, 668, 669, 670, 671, 672, 673, 676, 674, 675, 678, 679, 680, 681,
    682, 683, 684, 685, 686, 687, 688, 220, 221, 689, 55, 692, 693, 694, 695, 696,
    697, 709, 710, 698, 699, 700, 220, 221, 701, 55, 702, 703, 704, 705, 706, 707,
    394, 395, 708, 55, 712, 713, 714, 715, 717, 718, 719, 720, 722, 726, 730, 731,
    732, 737, 740, 723, 724, 725, 727, 728, 729, 733, 736, 734, 735, 738, 739, 742,
    743, 744, 745, 746, 748, 749, 750, 751, 752, 753, 766, 767, 775, 780, 754, 220,
    221, 755, 55, 756, 757, 758, 759, 760, 761, 762, 765, 763, 764, 768, 769, 770,
    220, 221, 771, 55, 772, 773, 774, 776, 777, 778, 779, 779, 220, 221, 781, 782,
    783, 785, 786, 787, 788, 794, 807, 808, 809, 846, 789, 790, 791, 792, 793, 795,
    805, 796, 797, 798, 799, 800, 801, 802, 55, 810, 820, 823, 836, 842, 811, 812,
    813, 814, 815, 816, 817, 818, 819, 821, 822, 824, 827, 825, 826, 828, 829, 830,
    833, 831, 832, 834, 835, 837, 838, 839, 840, 841, 843, 844, 845, 220, 221, 847,
    848, 849, 850, 852, 853, 854, 856, 857, 858, 859, 860, 861, 862, 863, 866, 864,
    865, 865, 220, 221, 871, 874, 872, 873, 875, 220, 221, 887, 888, 889, 891, 892,
    893, 894, 899, 900, 220, 221, 901, 55, 902, 903, 905, 911, 906, 910, 907, 908,
    909, 918, 919, 924, 925, 926, 932, 933, 934, 942, 943, 945, 946, 947, 951, 948,
    949, 950, 220, 221, 953, 954, 955, 956, 957, 959, 960, 961, 962, 964, 965, 966,
    967, 968, 971, 972, 973, 974, 976, 977, 978, 979, 981, 982, 983, 984, 985, 987,
    988, 989, 991, 992, 993, 994, 995, 997, 996, 55, 999, 1000, 1001, 1002, 1004, 1005,
    1006, 1007, 1008, 1010, 1011, 1012, 1013, 1015, 1016, 1017, 1018, 1019, 1021, 484, 1022, 223,
    220, 221, 1027, 1028, 1042, 1043, 1044, 1045, 1046, 1047, 1050, 1051, 1052, 1053, 1054, 1057,
    1058, 1059, 1063, 1064, 1065, 1066, 1072, 1055, 1056, 1060, 1061, 1062, 1067, 1068, 1069, 1070,
    1071, 1073, 1074, 1076, 1079, 1080, 1077, 1078, 1082, 1083, 1084, 1085, 1086, 1090, 1087, 1088,
    1089, 1092, 1095, 1093, 1094, 1097, 1101, 1107, 1098, 1099, 1100, 220, 221, 1102, 1105, 1103,
    1104, 1106, 1110, 1145, 1111, 1112, 1113, 1125, 1126, 1127, 1134, 1142, 1143, 1114, 220, 221,
    1115, 1116, 1117, 1118, 1119, 55, 1120, 1121, 1122, 1123, 1124, 1128, 1131, 1129, 1130, 1132,
    220, 221, 1133, 55, 1135, 1136, 1137, 1138, 1139, 1140, 1141, 1147, 1148, 1149, 1150, 1151,
    1152, 1153, 1159, 1160, 1163, 1167, 1170, 1161, 1162, 1164, 1165, 1166, 1168, 1169, 1171, 1172,
    1173, 1174, 1177, 1179, 1183, 1178, 1180, 1181, 1182, 1184, 1186, 1187, 1188, 220, 221, 1189,
    1190, 1198, 1199, 1200, 1201, 1202, 1203, 1204, 1205, 1208, 1209, 1210, 1222, 1211, 1214, 1212,
    1213, 1215, 1219, 1216, 1217, 1218, 1220, 1221, 1223, 1210, 1224, 1210, 1226, 1227, 1229, 1230,
    1231, 55, 1233, 1234, 1235, 1236, 1239, 1242, 1243, 1244, 1245, 1246, 1247, 1237, 1238, 1240,
    1241, 1249, 1267, 1250, 1251, 1254, 1255, 1261, 1265, 1266, 1252, 1253, 1253, 220, 221, 1256,
    1257, 1258, 1259, 1260, 1262, 1263, 1264, 1268, 220, 221, 1249, 1269, 1270, 1272, 1273, 1273,
    220, 221, 484, 1274, 223, 1285, 1286, 1297, 1275, 1276, 1277, 1278, 220, 221, 1280, 1281,
    1283, 1284, 1287, 1288, 1289, 1290, 1292, 1293, 1295, 1296, 1301, 1302, 1304, 1305, 1309, 1315,
    1321, 1327, 1328, 1338, 1355, 1371, 1377, 1381, 1382, 1383, 1384, 1306, 1307, 1308, 1310, 1311,
    1313, 1314, 1312, 1316, 1319, 1320, 1317, 1318, 1322, 1323, 1324, 1325, 1326, 55, 1329, 1333,
    1334, 1330, 1331, 1332, 1335, 1336, 1337, 1339, 1340, 1341, 1342, 1343, 1344, 1348, 1345, 1346,
    1347, 1349, 1350, 1353, 1354, 1351, 1352, 1356, 1361, 1367, 1357, 55, 1358, 1359, 1360, 1362,
    1365, 1366, 1363, 1364, 1368, 1369, 1370, 1372, 1375, 1376, 1373, 1374, 1378, 1379, 1380, 1386,
    1387, 220, 221, 1389, 1390, 1391, 1392, 1393, 1397, 1402, 1398, 1399, 1400, 1403, 1404, 1407,
    1414, 1408, 1409, 1410, 1413, 1411, 1412, 1415, 1418, 1416, 1417, 1420, 220, 221, 1421, 55,
    1423, 1426, 1427, 1424, 1425, 1428, 1435, 1429, 1434, 1430, 1431, 1432, 1433, 1441, 1442, 1456,
    1457, 1458, 1471, 1443, 1447, 1450, 1455, 1444, 1445, 1446, 1448, 1449, 1451, 1452, 1453, 1454,
    1459, 1463, 1467, 1460, 1461, 1462, 1464, 1465, 1466, 1468, 1469, 1470, 1472, 1473, 1474, 1475,
    1476, 1477, 1482, 1483, 1486, 1487, 213, 1488, 223, 1489, 1490, 1491, 1492, 1493, 1494, 55,
    1495, 1496, 1497, 1500, 1503, 1504, 1505, 1506, 1507, 1501, 1502, 1510, 1511, 55, 1512, 1513,
    1514, 1515, 220, 221, 1519, 1520, 1521, 1522, 220, 221, 1523, 55, 1524, 1525, 1528, 484,
    1529, 223, 1530, 1531, 1531, 220, 221, 1533, 1536, 1537, 1538, 1539, 1534, 1535, 1545, 1546,
    1549, 1550, 1551, 1554, 1557, 1555, 1556, 1561, 1562, 1563, 1564, 1569, 1565, 220, 221, 1566,
    1568, 1567, 55, 1572, 1573, 1574, 1575, 1581, 1582, 1583, 1584, 1585, 1586, 1589, 1587, 1588,
    1594, 1595, 1596, 1597, 1598, 1599, 1600, 1601, 1602, 1609, 1610, 1611, 1612, 1613, 1615, 1616,
    1617, 1618, 1623, 1636, 1641, 1624, 1625, 213, 1626, 1627, 223, 1629, 1628, 1630, 1631, 1632,
    1635, 1633, 1634, 1637, 1638, 213, 1639, 1640, 223, 1642, 213, 1643, 223, 1646, 1649, 1650,
    1651, 1654, 1652, 1653, 1653, 220, 221, 1656, 1657, 1658, 1662, 1659, 1660, 1661, 1664, 213,
    1665, 223, 1666, 220, 221, 1667, 55, 1681, 1686, 1687, 1682, 1683, 1684, 220, 221, 1685,
    55, 1694, 1695, 1699, 1702, 1703, 1700, 1701, 1704, 1705, 1707, 1711, 213, 1712, 1718, 223,
    1713, 1714, 1714, 220, 221, 1715, 1716, 1717, 1719, 1724, 1720, 1721, 1722, 1723, 1725, 1726,
    1731, 1732, 1733, 1734, 1735, 1736, 1737, 1738, 1741, 1739, 1740, 1743, 1744, 1745, 1746, 1747,
    1748, 1749, 1750, 1751, 1762, 1767, 1763, 404, 405, 1764, 1765, 1766, 1768, 220, 221, 1769,
    1770, 1771, 1773, 1774, 1784, 213, 1785, 223, 1786, 1788, 1789, 1792, 1793, 1795, 1796, 1797,
    1800, 213, 1801, 223, 1798, 1799, 1802, 1803, 1805, 1806, 1807, 1808, 1811, 213, 1812, 223,
    1809, 1810, 1814, 213, 1815, 223, 1818, 213, 1819, 223, 1829, 1830, 1831, 1832, 1816, 1817,
    1820, 1821, 1822, 1826, 1823, 1825, 1824, 220, 221, 1827, 1828, 1834, 213, 1835, 223, 1843,
    1844, 1845, 1846, 1847, 1848, 220, 221, 1850, 213, 1851, 223, 1852, 55, 1855, 1856, 1869,
    1870, 1871, 1873, 1875, 1877, 1872, 220, 221, 1874, 220, 221, 1876, 220, 221, 1880, 1881,
    1882, 1883, 1885, 1886, 1889, 1887, 1888, 1888, 220, 221, 1895, 1898, 1896, 1897, 1899, 1900,
    1911, 1912, 1913, 1915, 1917, 1030, 1916, 55, 1919, 1920, 1921, 1922, 1924, 1925, 1926, 1927,
    1928, 1929, 1930, 1931, 1932, 1933, 1934, 467, 1937, 213, 1938, 1942, 223, 1939, 1940, 1941,
    1943, 1944, 1954, 1955, 1957, 1958, 213, 1959, 223, 1960, 55, 1961, 1965, 1968, 1973, 1962,
    1963, 1964, 1966, 1967, 1969, 1970, 1971, 1972, 1977, 1978, 1979, 1980, 1981, 1982, 1985, 1986,
    213, 1991, 1992, 1993, 223, 1994, 1995, 1996, 220, 221, 2003, 2004, 213, 2005, 223, 2006,
    2007, 2007, 220, 221, 2008, 55, 2015, 2020, 2016, 2017, 2018, 2019, 2032, 2033, 2034, 2035,
    2043, 2044, 2045, 2046, 2047, 2048, 2049, 2050, 2051, 2054, 2059, 2060, 2061, 2062, 2063, 2064,
    2052, 2053, 2055, 2058, 2056, 2057, 2069, 2070, 2072, 2073, 2077, 2074, 2075, 2076, 2078, 220,
    221, 2091, 2092, 2093, 2094, 2095, 2096, 2100, 2097, 2098, 2099, 2101, 2102, 2103, 2105, 2106,
    2107, 2108, 2109, 2111, 2112, 2113, 2116, 2119, 2114, 2115, 2117, 2118, 2120, 2121, 2132, 2133,
    2137, 2151, 2152, 2153, 2134, 2135, 2136, 2138, 2142, 2145, 2150, 2139, 2140, 2141, 2143, 2144,
    2146, 2147, 2148, 2149, 2164, 220, 221, 2165, 55, 2166, 2167, 2168, 2169, 2170, 2172, 2173,
    2177, 2178, 2181, 2183, 2179, 2180, 2182, 220, 221, 2184, 2186, 2185, 2187, 220, 221, 2195,
    2196, 2197, 2198, 2200, 2201, 2203, 2204, 213, 2205, 223, 2206, 55, 2209, 2211, 2212, 2215,
    2216, 2217, 2210, 2213, 2214, 2220, 2221, 2222, 2223, 2224, 2225, 2226, 2227, 2228, 2229, 2234,
    2237, 2240, 2241, 2235, 2236, 2236, 220, 221, 2238, 2239, 2239, 220, 221, 2242, 220, 221,
    2244, 2245, 2246, 2247, 2248, 2251, 2249, 2250, 2254, 2257, 2255, 2256, 2258, 2259, 2261, 2264,
    2262, 2263, 2265, 2266, 2270, 213, 2271, 223, 2272, 55, 2274, 2275, 2276, 2277, 2278, 2279,
    2280, 2281, 2282, 2283, 2285, 2286, 2290, 2291, 2292, 2287, 2288, 2289, 2295, 2300, 2305, 2296,
    2297, 2298, 2299, 2301, 2304, 2302, 2303, 2306, 2307, 2308, 220, 221, 2310, 2311, 2312, 2313,
    2315, 2316, 2317, 2320, 2321, 2322, 2318, 2319, 2325, 2328, 2326, 2327, 2329, 2330, 2331, 2332,
    2335, 2336, 2338, 2339, 2342, 55, 2344, 2345, 2346, 2349, 2350, 2359, 2360, 2361, 2362, 2363,
    2364, 2365, 2366, 2367, 2368, 2369, 2370, 2371, 2372, 2373, 2374, 2375, 2376, 2377, 2378, 2379,
    2380, 2381, 2382, 2386, 2388, 220, 221, 2389, 55, 2390, 2391, 2398, 2401, 2399, 2400, 2402,
    2405, 2408, 2403, 2404, 2406, 2407, 2409, 2410, 2411, 2414, 2415, 2416, 2417, 2418, 2420, 2419,
    220, 221, 2421, 220, 221, 2426, 2429, 2427, 2431, 2433, 2432, 2444, 2447, 2448, 2445, 2446,
    2449, 2452, 2453, 2450, 2451, 2455, 2456, 2457, 2458, 2459, 2466, 2469, 2472, 2467, 2468, 2470,
    2471, 2473, 2474, 2475, 2476, 55, 2477, 2491, 2478, 2479, 2480, 2481, 2484, 2487, 2490, 2482,
    2483, 2485, 2486, 2488, 2489, 2492, 2493, 2494, 2495, 2498, 2499, 2500, 2505, 2506, 2507, 2510,
    220, 221, 2511, 55, 2514, 2515, 2517, 2518, 2524, 2525, 2526, 2529, 2530, 2531, 2532, 2541,
    2542, 2543, 2544, 2545, 2546, 2548, 2549, 2550, 2551, 2552, 2555, 213, 2556, 2557, 223, 2559,
    2560, 2561, 2558, 2564, 2565, 2576, 2577, 2578, 55, 2586, 2587, 2589, 2590, 2591, 2592, 2593,
    2594, 484, 2588, 223, 2597, 484, 2598, 223, 2599, 2600, 2600, 220, 221, 2602, 2603, 2604,
    2607, 2610, 2611, 2612, 2605, 2606, 2608, 2609, 2616, 2618, 2626, 2619, 2620, 2623, 2621, 2622,
    2624, 2625, 2629, 2633, 2634, 2635, 2630, 2631, 2632, 2637, 2638, 2639, 2640, 2643, 2644, 2641,
    55, 2646, 2647, 2648, 2649, 2652, 2653, 2650, 55, 2655, 2656, 2658, 2657, 2662, 2663, 2664,
    2665, 2667, 2668, 2670, 2671, 2672, 2674, 2673, 220, 221,
];

pub static TERMINATORS: &[u32] = &[
    56, 57, 133, 430, 431, 487, 488, 490, 497, 498, 500, 502, 503, 504, 519, 577,
    803, 804, 806, 1108, 1144, 1279, 1282, 1291, 1294, 1401, 1405, 1498, 1516, 1974, 1975, 2207,
    2230, 2231, 2232, 2340, 2383, 2428, 2642, 2651,
];

pub static STRINGS: &[&str] = &[
    "ABSENT", // [0]
    "keyword", // [1]
    "KeywordSegment", // [2]
    "base", // [3]
    "raw", // [4]
    "word", // [5]
    "AccountKeywordSegment", // [6]
    "ResourceKeywordSegment", // [7]
    "MonitorKeywordSegment", // [8]
    "WarehouseKeywordSegment", // [9]
    "DatabaseKeywordSegment", // [10]
    "DomainKeywordSegment", // [11]
    "IntegrationKeywordSegment", // [12]
    "LanguageKeywordSegment", // [13]
    "SchemaKeywordSegment", // [14]
    "SCHEMA", // [15]
    "CatalogKeywordSegment", // [16]
    "CATALOG", // [17]
    "RoleKeywordSegment", // [18]
    "ROLE", // [19]
    "TablespaceKeywordSegment", // [20]
    "TypeKeywordSegment", // [21]
    "TYPE", // [22]
    "ForeignKeywordSegment", // [23]
    "ServerKeywordSegment", // [24]
    "DATA", // [25]
    "DataKeywordSegment", // [26]
    "WrapperKeywordSegment", // [27]
    "WRAPPER", // [28]
    "ALL", // [29]
    "AllKeywordSegment", // [30]
    "SchemasKeywordSegment", // [31]
    "SCHEMAS", // [32]
    "InKeywordSegment", // [33]
    "IN", // [34]
    "FutureKeywordSegment", // [35]
    "AccessSchemaObjectSegment", // [36]
    "access_schema_object", // [37]
    "AccessSchemaPluralObjectSegment", // [38]
    "access_schema_plural_object", // [39]
    "ObjectReferenceSegment", // [40]
    "object_reference", // [41]
    "FunctionNameSegment", // [42]
    "function_name", // [43]
    "FunctionParameterListGrammar", // [44]
    "(", // [45]
    "function_parameter_list", // [46]
    "WildcardIdentifierSegment", // [47]
    "wildcard_identifier", // [48]
    "CommaSegment", // [49]
    ",", // [50]
    "ToKeywordSegment", // [51]
    "TO", // [52]
    "FromKeywordSegment", // [53]
    "FROM", // [54]
    "LargeKeywordSegment", // [55]
    "ObjectKeywordSegment", // [56]
    "OBJECT", // [57]
    "NumericLiteralSegment", // [58]
    "numeric_literal", // [59]
    "CREATE", // [60]
    "CreateKeywordSegment", // [61]
    "UserKeywordSegment", // [62]
    "USER", // [63]
    "ApplyKeywordSegment", // [64]
    "MaskingKeywordSegment", // [65]
    "PolicyKeywordSegment", // [66]
    "EXECUTE", // [67]
    "ExecuteKeywordSegment", // [68]
    "TaskKeywordSegment", // [69]
    "ManageKeywordSegment", // [70]
    "GrantsKeywordSegment", // [71]
    "GRANTS", // [72]
    "ExecutionKeywordSegment", // [73]
    "UsageKeywordSegment", // [74]
    "PipeKeywordSegment", // [75]
    "USE", // [76]
    "UseKeywordSegment", // [77]
    "ImportedKeywordSegment", // [78]
    "PrivilegesKeywordSegment", // [79]
    "PRIVILEGES", // [80]
    "ConnectKeywordSegment", // [81]
    "DeleteKeywordSegment", // [82]
    "DELETE", // [83]
    "InsertKeywordSegment", // [84]
    "INSERT", // [85]
    "ModifyKeywordSegment", // [86]
    "OperateKeywordSegment", // [87]
    "OwnershipKeywordSegment", // [88]
    "ReadKeywordSegment", // [89]
    "READ", // [90]
    "Reference_usageKeywordSegment", // [91]
    "ReferencesKeywordSegment", // [92]
    "SelectKeywordSegment", // [93]
    "SELECT", // [94]
    "TempKeywordSegment", // [95]
    "TemporaryKeywordSegment", // [96]
    "TriggerKeywordSegment", // [97]
    "TruncateKeywordSegment", // [98]
    "TRUNCATE", // [99]
    "UpdateKeywordSegment", // [100]
    "UPDATE", // [101]
    "Use_any_roleKeywordSegment", // [102]
    "WriteKeywordSegment", // [103]
    "WRITE", // [104]
    "BracketedColumnReferenceListGrammar", // [105]
    "AccessPermissionSegment", // [106]
    "access_permission", // [107]
    "OnKeywordSegment", // [108]
    "ON", // [109]
    "TableKeywordSegment", // [110]
    "TABLE", // [111]
    "ViewKeywordSegment", // [112]
    "VIEW", // [113]
    "StageKeywordSegment", // [114]
    "FunctionKeywordSegment", // [115]
    "FUNCTION", // [116]
    "ProcedureKeywordSegment", // [117]
    "RoutineKeywordSegment", // [118]
    "SequenceKeywordSegment", // [119]
    "StreamKeywordSegment", // [120]
    "MATERIALIZED", // [121]
    "MaterializedKeywordSegment", // [122]
    "ExternalKeywordSegment", // [123]
    "FileKeywordSegment", // [124]
    "FormatKeywordSegment", // [125]
    "FORMAT", // [126]
    "TablesKeywordSegment", // [127]
    "TABLES", // [128]
    "ViewsKeywordSegment", // [129]
    "StagesKeywordSegment", // [130]
    "FunctionsKeywordSegment", // [131]
    "FUNCTIONS", // [132]
    "ProceduresKeywordSegment", // [133]
    "RoutinesKeywordSegment", // [134]
    "SequencesKeywordSegment", // [135]
    "StreamsKeywordSegment", // [136]
    "TasksKeywordSegment", // [137]
    "PoliciesKeywordSegment", // [138]
    "FormatsKeywordSegment", // [139]
    "GRANT", // [140]
    "REVOKE", // [141]
    "GrantStatementSegment", // [142]
    "grant_statement", // [143]
    "RevokeStatementSegment", // [144]
    "revoke_statement", // [145]
    "GROUP", // [146]
    "GroupKeywordSegment", // [147]
    "UserReferenceSegment", // [148]
    "user_reference", // [149]
    "RoleReferenceSegment", // [150]
    "role_reference", // [151]
    "ShareKeywordSegment", // [152]
    "FunctionSegment", // [153]
    "function", // [154]
    "PublicKeywordSegment", // [155]
    ".", // [156]
    "[", // [157]
    "ArrayAccessorSegment", // [158]
    "array_accessor", // [159]
    "SemiStructuredAccessorSegment", // [160]
    "semi_structured_expression", // [161]
    "ADD", // [162]
    "ADMIN", // [163]
    "AFTER", // [164]
    "OrderByClauseSegment", // [165]
    "ORDER", // [166]
    "orderby_clause", // [167]
    "indent", // [168]
    "Indent", // [169]
    "meta", // [170]
    "AsAliasOperatorSegment", // [171]
    "AS", // [172]
    "alias_operator", // [173]
    "SingleIdentifierGrammar", // [174]
    "SingleIdentifierListSegment", // [175]
    "identifier_list", // [176]
    "start_bracket", // [177]
    "SymbolSegment", // [178]
    "symbol", // [179]
    ")", // [180]
    "end_bracket", // [181]
    "SingleQuotedIdentifierSegment", // [182]
    "single_quote", // [183]
    "dedent", // [184]
    "Dedent", // [185]
    "TableReferenceSegment", // [186]
    "table_reference", // [187]
    "AliasExpressionSegment", // [188]
    "alias_expression", // [189]
    "ALTER", // [190]
    "IncrementKeywordSegment", // [191]
    "ByKeywordSegment", // [192]
    "BY", // [193]
    "SequenceMinValueGrammar", // [194]
    "SequenceMaxValueGrammar", // [195]
    "CacheKeywordSegment", // [196]
    "NocacheKeywordSegment", // [197]
    "CycleKeywordSegment", // [198]
    "NocycleKeywordSegment", // [199]
    "OrderNoOrderGrammar", // [200]
    "AlterKeywordSegment", // [201]
    "SequenceReferenceSegment", // [202]
    "sequence_reference", // [203]
    "AlterSequenceOptionsSegment", // [204]
    "alter_sequence_options_segment", // [205]
    "DROP", // [206]
    "DropKeywordSegment", // [207]
    "ColumnKeywordSegment", // [208]
    "COLUMN", // [209]
    "IfExistsGrammar", // [210]
    "IF", // [211]
    "ParameterNameSegment", // [212]
    "EqualsSegment", // [213]
    "=", // [214]
    "comparison_operator", // [215]
    "LiteralGrammar", // [216]
    "+", // [217]
    "-", // [218]
    "ARRAY", // [219]
    "DATE", // [220]
    "FALSE", // [221]
    "INTERVAL", // [222]
    "NULL", // [223]
    "TIME", // [224]
    "TIMESTAMP", // [225]
    "TRUE", // [226]
    "{", // [227]
    "NakedIdentifierSegment", // [228]
    "AddKeywordSegment", // [229]
    "ColumnDefinitionSegment", // [230]
    "column_definition", // [231]
    "FIRST", // [232]
    "FirstKeywordSegment", // [233]
    "AfterKeywordSegment", // [234]
    "ColumnReferenceSegment", // [235]
    "column_reference", // [236]
    "AlterTableDropColumnGrammar", // [237]
    "RENAME", // [238]
    "RenameKeywordSegment", // [239]
    "AsKeywordSegment", // [240]
    "SET", // [241]
    "IfNotExistsGrammar", // [242]
    "LAST", // [243]
    "LastKeywordSegment", // [244]
    "SetKeywordSegment", // [245]
    "DatatypeSegment", // [246]
    "BIGINT", // [247]
    "BOOLEAN", // [248]
    "CHAR", // [249]
    "DECIMAL", // [250]
    "DOUBLE", // [251]
    "INT", // [252]
    "INTEGER", // [253]
    "IPADDRESS", // [254]
    "JSON", // [255]
    "MAP", // [256]
    "REAL", // [257]
    "ROW", // [258]
    "SMALLINT", // [259]
    "TINYINT", // [260]
    "UUID", // [261]
    "VARBINARY", // [262]
    "VARCHAR", // [263]
    "data_type", // [264]
    "NotKeywordSegment", // [265]
    "NOT", // [266]
    "NullKeywordSegment", // [267]
    "AuthorizationKeywordSegment", // [268]
    "AUTHORIZATION", // [269]
    "PropertiesKeywordSegment", // [270]
    "PROPERTIES", // [271]
    "ExpressionSegment", // [272]
    "expression", // [273]
    "ExecuteArrowSegment", // [274]
    "=>", // [275]
    "WHERE", // [276]
    "WhereKeywordSegment", // [277]
    "&", // [278]
    "ampersand", // [279]
    "ANALYZE", // [280]
    "AnalyzeKeywordSegment", // [281]
    "WITH", // [282]
    "WithKeywordSegment", // [283]
    "AND", // [284]
    "binary_operator", // [285]
    "BinaryOperatorSegment", // [286]
    "ANY", // [287]
    "%", // [288]
    "*", // [289]
    "/", // [290]
    "<", // [291]
    ">", // [292]
    "^", // [293]
    "|", // [294]
    "PlusSegment", // [295]
    "MinusSegment", // [296]
    "DivideSegment", // [297]
    "MultiplySegment", // [298]
    "ModuloSegment", // [299]
    "BitwiseAndSegment", // [300]
    "BitwiseOrSegment", // [301]
    "BitwiseXorSegment", // [302]
    "BitwiseLShiftSegment", // [303]
    "BitwiseRShiftSegment", // [304]
    "SliceSegment", // [305]
    ":", // [306]
    "start_square_bracket", // [307]
    "]", // [308]
    "end_square_bracket", // [309]
    "BaseExpressionElementGrammar", // [310]
    "start_angle_bracket", // [311]
    "end_angle_bracket", // [312]
    "ArrayKeywordSegment", // [313]
    "ArrayTypeSchemaSegment", // [314]
    "array_type_schema", // [315]
    "array_type", // [316]
    "ASC", // [317]
    "AT", // [318]
    "Auto_incrementKeywordSegment", // [319]
    "CURRENT_DATE", // [320]
    "CURRENT_TIME", // [321]
    "CURRENT_TIMESTAMP", // [322]
    "LOCALTIME", // [323]
    "LOCALTIMESTAMP", // [324]
    "bare_function", // [325]
    "CodeSegment", // [326]
    "BareFunctionSegment", // [327]
    "IntervalExpressionSegment", // [328]
    "interval_expression", // [329]
    "file", // [330]
    "BaseFileSegment", // [331]
    "BaseSegment", // [332]
    "BERNOULLI", // [333]
    "BETWEEN", // [334]
    "!", // [335]
    "->", // [336]
    "IS", // [337]
    "OR", // [338]
    "like_operator", // [339]
    "ArithmeticBinaryOperatorGrammar", // [340]
    "StringBinaryOperatorGrammar", // [341]
    "BooleanBinaryOperatorGrammar", // [342]
    "ComparisonOperatorGrammar", // [343]
    "RightArrowOperator", // [344]
    "AmpersandSegment", // [345]
    "RawLessThanSegment", // [346]
    "PipeSegment", // [347]
    "RawGreaterThanSegment", // [348]
    "AndOperatorGrammar", // [349]
    "OrOperatorGrammar", // [350]
    "TrueSegment", // [351]
    "FalseSegment", // [352]
    "BOTH", // [353]
    "bracketed", // [354]
    "BracketedSegment", // [355]
    "UnorderedSetExpressionSegment", // [356]
    "VALUES", // [357]
    "set_expression", // [358]
    "CTEColumnList", // [359]
    "cte_column_list", // [360]
    "SelectableGrammar", // [361]
    "CALL", // [362]
    "CASCADE", // [363]
    "CASE", // [364]
    "CaseKeywordSegment", // [365]
    "implicit_indent", // [366]
    "ImplicitIndent", // [367]
    "WHEN", // [368]
    "WhenClauseSegment", // [369]
    "when_clause", // [370]
    "ElseKeywordSegment", // [371]
    "ELSE", // [372]
    "EndKeywordSegment", // [373]
    "END", // [374]
    "ElseClauseSegment", // [375]
    "else_clause", // [376]
    "BinaryOperatorGrammar", // [377]
    "CAST", // [378]
    "::", // [379]
    "casting_operator", // [380]
    "CATALOGS", // [381]
    "QuotedLiteralSegment", // [382]
    "ObjectReferenceDelimiterGrammar", // [383]
    "ObjectReferenceTerminatorGrammar", // [384]
    "colon_delimiter", // [385]
    "colon_prefix", // [386]
    "colon", // [387]
    "ShorthandCastSegment", // [388]
    "cast_expression", // [389]
    "CONSTRAINT", // [390]
    "ConstraintKeywordSegment", // [391]
    "CheckKeywordSegment", // [392]
    "DEFAULT", // [393]
    "DefaultKeywordSegment", // [394]
    "ColumnConstraintDefaultGrammar", // [395]
    "PrimaryKeyGrammar", // [396]
    "NotEnforcedGrammar", // [397]
    "UniqueKeyGrammar", // [398]
    "AutoIncrementGrammar", // [399]
    "ReferenceDefinitionGrammar", // [400]
    "CommentClauseSegment", // [401]
    "COMMENT", // [402]
    "comment_clause", // [403]
    "CollateKeywordSegment", // [404]
    "CollationReferenceSegment", // [405]
    "collation_reference", // [406]
    "ColumnGeneratedGrammar", // [407]
    "ColumnsExpressionNameGrammar", // [408]
    "COLUMNS", // [409]
    "comma", // [410]
    "CommentKeywordSegment", // [411]
    "IsKeywordSegment", // [412]
    "comment", // [413]
    "CommentSegment", // [414]
    "COMMIT", // [415]
    "COMMITTED", // [416]
    "GreaterThanSegment", // [417]
    "LessThanSegment", // [418]
    "GreaterThanOrEqualToSegment", // [419]
    "LessThanOrEqualToSegment", // [420]
    "NotEqualToSegment", // [421]
    "LikeOperatorSegment", // [422]
    "IsDistinctFromGrammar", // [423]
    "ComparisonOperatorSegment", // [424]
    "CompositeBinaryOperatorSegment", // [425]
    "CompositeComparisonOperatorSegment", // [426]
    "CrossKeywordSegment", // [427]
    "CROSS", // [428]
    "JoinTypeKeywordsGrammar", // [429]
    "FULL", // [430]
    "INNER", // [431]
    "LEFT", // [432]
    "RIGHT", // [433]
    "ConditionalCrossJoinKeywordsGrammar", // [434]
    "NonStandardJoinTypeKeywordsGrammar", // [435]
    "CONDITIONAL", // [436]
    "COPARTITION", // [437]
    "COUNT", // [438]
    "CastKeywordSegment", // [439]
    "SpecificKeywordSegment", // [440]
    "InstanceKeywordSegment", // [441]
    "StaticKeywordSegment", // [442]
    "ConstructorKeywordSegment", // [443]
    "MethodKeywordSegment", // [444]
    "FOR", // [445]
    "ForKeywordSegment", // [446]
    "AssignmentKeywordSegment", // [447]
    "DatabaseReferenceSegment", // [448]
    "database_reference", // [449]
    "OrReplaceGrammar", // [450]
    "TemporaryGrammar", // [451]
    "ReturnsKeywordSegment", // [452]
    "FunctionDefinitionGrammar", // [453]
    "function_definition", // [454]
    "UniqueKeywordSegment", // [455]
    "UNIQUE", // [456]
    "IndexKeywordSegment", // [457]
    "IndexReferenceSegment", // [458]
    "index_reference", // [459]
    "IndexColumnDefinitionSegment", // [460]
    "index_column_definition", // [461]
    "ModelKeywordSegment", // [462]
    "OptionsKeywordSegment", // [463]
    "SchemaReferenceSegment", // [464]
    "schema_reference", // [465]
    "START", // [466]
    "StartKeywordSegment", // [467]
    "CreateSequenceOptionsSegment", // [468]
    "create_sequence_options_segment", // [469]
    "LIKE", // [470]
    "LikeKeywordSegment", // [471]
    "EXCLUDING", // [472]
    "INCLUDING", // [473]
    "IncludingKeywordSegment", // [474]
    "ExcludingKeywordSegment", // [475]
    "NoKeywordSegment", // [476]
    "NO", // [477]
    "TriggerReferenceSegment", // [478]
    "trigger_reference", // [479]
    "BeforeKeywordSegment", // [480]
    "InsteadKeywordSegment", // [481]
    "OfKeywordSegment", // [482]
    "OF", // [483]
    "OrKeywordSegment", // [484]
    "ReferencingKeywordSegment", // [485]
    "OldKeywordSegment", // [486]
    "RowKeywordSegment", // [487]
    "NewKeywordSegment", // [488]
    "DeferrableKeywordSegment", // [489]
    "InitiallyKeywordSegment", // [490]
    "ImmediateKeywordSegment", // [491]
    "IMMEDIATE", // [492]
    "DeferredKeywordSegment", // [493]
    "EachKeywordSegment", // [494]
    "StatementKeywordSegment", // [495]
    "WhenKeywordSegment", // [496]
    "FunctionNameIdentifierSegment", // [497]
    "FunctionContentsSegment", // [498]
    "function_contents", // [499]
    "WithNoSchemaBindingClauseSegment", // [500]
    "with_no_schema_binding_clause", // [501]
    "CUBE", // [502]
    "function_name_identifier", // [503]
    "ROLLUP", // [504]
    "CubeFunctionNameSegment", // [505]
    "RollupFunctionNameSegment", // [506]
    "GroupingExpressionList", // [507]
    "grouping_expression_list", // [508]
    "CURRENT", // [509]
    "CURRENT_CATALOG", // [510]
    "CURRENT_PATH", // [511]
    "CURRENT_ROLE", // [512]
    "CURRENT_SCHEMA", // [513]
    "CURRENT_USER", // [514]
    "data_type_identifier", // [515]
    "PrimitiveTypeSegment", // [516]
    "primitive_type", // [517]
    "ArrayTypeSegment", // [518]
    "MapTypeSegment", // [519]
    "map_type", // [520]
    "RowTypeSegment", // [521]
    "struct_type", // [522]
    "DATEADD", // [523]
    "DatePartFunctionName", // [524]
    "DatetimeUnitSegment", // [525]
    "DAY", // [526]
    "DAYOFYEAR", // [527]
    "HOUR", // [528]
    "MILLISECOND", // [529]
    "MINUTE", // [530]
    "MONTH", // [531]
    "QUARTER", // [532]
    "SECOND", // [533]
    "WEEK", // [534]
    "WEEKDAY", // [535]
    "YEAR", // [536]
    "FunctionContentsGrammar", // [537]
    "DateKeywordSegment", // [538]
    "TimeKeywordSegment", // [539]
    "TimestampKeywordSegment", // [540]
    "date_constructor_literal", // [541]
    "literal", // [542]
    "LiteralSegment", // [543]
    "date_part", // [544]
    "DEALLOCATE", // [545]
    "ValuesKeywordSegment", // [546]
    "DEFINE", // [547]
    "DEFINER", // [548]
    "FromClauseSegment", // [549]
    "from_clause", // [550]
    "WhereClauseSegment", // [551]
    "where_clause", // [552]
    "SemicolonSegment", // [553]
    ";", // [554]
    "DENY", // [555]
    "DESC", // [556]
    "DESCRIBE", // [557]
    "DescribeKeywordSegment", // [558]
    "DESCRIPTOR", // [559]
    "DISTINCT", // [560]
    "DISTRIBUTED", // [561]
    "dot", // [562]
    "RESTRICT", // [563]
    "RestrictKeywordSegment", // [564]
    "CascadeKeywordSegment", // [565]
    "DropBehaviorGrammar", // [566]
    "EMPTY", // [567]
    "StructTypeSegment", // [568]
    "EmptyStructLiteralBracketsSegment", // [569]
    "struct_literal", // [570]
    "ENCODING", // [571]
    "}", // [572]
    "end_curly_bracket", // [573]
    "RawEqualsSegment", // [574]
    "ERROR", // [575]
    "ESCAPE", // [576]
    "EXCEPT", // [577]
    "execute_arrow", // [578]
    "EXISTS", // [579]
    "EXPLAIN", // [580]
    "ExplainKeywordSegment", // [581]
    "InsertStatementSegment", // [582]
    "insert_statement", // [583]
    "UpdateStatementSegment", // [584]
    "update_statement", // [585]
    "DeleteStatementSegment", // [586]
    "delete_statement", // [587]
    "Expression_A_Grammar", // [588]
    "Tail_Recurse_Expression_A_Grammar", // [589]
    "LikeExpressionGrammar", // [590]
    "InOperatorGrammar", // [591]
    "QuantifiedComparisonOperatorGrammar", // [592]
    "IsClauseGrammar", // [593]
    "IsNullGrammar", // [594]
    "NotNullGrammar", // [595]
    "CollateGrammar", // [596]
    "BetweenKeywordSegment", // [597]
    "Expression_B_Grammar", // [598]
    "AndKeywordSegment", // [599]
    "PatternMatchingGrammar", // [600]
    "~", // [601]
    "SignedSegmentGrammar", // [602]
    "QualifiedNumericLiteralSegment", // [603]
    "TildeSegment", // [604]
    "NotOperatorGrammar", // [605]
    "Tail_Recurse_Expression_B_Grammar", // [606]
    "ExistsKeywordSegment", // [607]
    "Expression_D_Grammar", // [608]
    "CaseExpressionSegment", // [609]
    "case_expression", // [610]
    "TimeZoneGrammar", // [611]
    "time_zone_grammar", // [612]
    "LateralColumnAliasExpressionGrammar", // [613]
    "LocalAliasSegment", // [614]
    "local_alias_segment", // [615]
    "Expression_D_Potential_Select_Statement_Without_Brackets", // [616]
    "SingleIdentifierWildcardSegment", // [617]
    "BooleanLiteralGrammar", // [618]
    "NullLiteralSegment", // [619]
    "DateTimeLiteralGrammar", // [620]
    "ListComprehensionGrammar", // [621]
    "AccessorGrammar", // [622]
    "SelectStatementSegment", // [623]
    "select_statement", // [624]
    "TypedStructLiteralSegment", // [625]
    "typed_struct_literal", // [626]
    "ArrayExpressionSegment", // [627]
    "array_expression", // [628]
    "OverlapsClauseSegment", // [629]
    "overlaps_clause", // [630]
    "EXTRACT", // [631]
    "boolean_literal", // [632]
    "LiteralKeywordSegment", // [633]
    "FETCH", // [634]
    "FetchKeywordSegment", // [635]
    "NEXT", // [636]
    "NextKeywordSegment", // [637]
    "ROWS", // [638]
    "RowsKeywordSegment", // [639]
    "ONLY", // [640]
    "OnlyKeywordSegment", // [641]
    "TiesKeywordSegment", // [642]
    "TIES", // [643]
    "MERGE", // [644]
    "ROLLBACK", // [645]
    "DelimiterGrammar", // [646]
    "StatementSegment", // [647]
    "statement", // [648]
    "FILTER", // [649]
    "FilterKeywordSegment", // [650]
    "FINAL", // [651]
    "FOLLOWING", // [652]
    "JsonKeywordSegment", // [653]
    "EncodingKeywordSegment", // [654]
    "UTF16", // [655]
    "UTF32", // [656]
    "UTF8", // [657]
    "Utf8KeywordSegment", // [658]
    "Utf16KeywordSegment", // [659]
    "Utf32KeywordSegment", // [660]
    "RANGE", // [661]
    "FrameClauseUnitGrammar", // [662]
    "UNBOUNDED", // [663]
    "CurrentKeywordSegment", // [664]
    "UnboundedKeywordSegment", // [665]
    "PRECEDING", // [666]
    "PrecedingKeywordSegment", // [667]
    "FollowingKeywordSegment", // [668]
    "RangeKeywordSegment", // [669]
    "FromExpressionSegment", // [670]
    "from_expression", // [671]
    "HAVING", // [672]
    "INTERSECT", // [673]
    "LIMIT", // [674]
    "OFFSET", // [675]
    "UNION", // [676]
    "WINDOW", // [677]
    "LimitKeywordSegment", // [678]
    "OffsetKeywordSegment", // [679]
    "OrderKeywordSegment", // [680]
    "HavingKeywordSegment", // [681]
    "WindowKeywordSegment", // [682]
    "SetOperatorSegment", // [683]
    "set_operator", // [684]
    "WithDataClauseSegment", // [685]
    "with_data_clause", // [686]
    "PreTableFunctionKeywordsGrammar", // [687]
    "TableExpressionSegment", // [688]
    "table_expression", // [689]
    "TemporalQuerySegment", // [690]
    "temporal_query", // [691]
    "FromClauseTerminatorGrammar", // [692]
    "SamplingExpressionSegment", // [693]
    "TABLESAMPLE", // [694]
    "sample_expression", // [695]
    "JoinLikeClauseGrammar", // [696]
    "JoinClauseSegment", // [697]
    "join_clause", // [698]
    "PostTableExpressionGrammar", // [699]
    "MLTableExpressionSegment", // [700]
    "FromExpressionElementSegment", // [701]
    "from_expression_element", // [702]
    "indented_joins", // [703]
    "LambdaExpressionSegment", // [704]
    "lambda_function", // [705]
    "LEADING", // [706]
    "TRAILING", // [707]
    "TrimParametersGrammar", // [708]
    "DistinctKeywordSegment", // [709]
    "StarSegment", // [710]
    "FunctionContentsExpressionGrammar", // [711]
    "FormatJsonEncodingGrammar", // [712]
    "WITHOUT", // [713]
    "WithoutKeywordSegment", // [714]
    "UNCONDITIONAL", // [715]
    "ConditionalKeywordSegment", // [716]
    "UnconditionalKeywordSegment", // [717]
    "ColonDelimiterSegment", // [718]
    "AbsentKeywordSegment", // [719]
    "RETURNING", // [720]
    "ReturningKeywordSegment", // [721]
    "IgnoreRespectNullsGrammar", // [722]
    "IGNORE", // [723]
    "RESPECT", // [724]
    "EmptyStructLiteralSegment", // [725]
    "ListaggOverflowClauseSegment", // [726]
    "listagg_overflow_clause", // [727]
    "WordSegment", // [728]
    "DotSegment", // [729]
    "double_quote", // [730]
    "QuotedIdentifierSegment", // [731]
    "AnyKeywordSegment", // [732]
    "FunctionParameterGrammar", // [733]
    "DatePartFunctionNameSegment", // [734]
    "DateTimeFunctionContentsSegment", // [735]
    "ColumnsExpressionGrammar", // [736]
    "ColumnsExpressionFunctionNameSegment", // [737]
    "ValuesClauseSegment", // [738]
    "values_clause", // [739]
    "PostFunctionGrammar", // [740]
    "glob_operator", // [741]
    "GRACE", // [742]
    "GrantKeywordSegment", // [743]
    "AccessPermissionsSegment", // [744]
    "access_permissions", // [745]
    "AccessObjectSegment", // [746]
    "access_object", // [747]
    "AccessTargetSegment", // [748]
    "access_target", // [749]
    "OptionKeywordSegment", // [750]
    "OPTION", // [751]
    "AdminKeywordSegment", // [752]
    "CopyKeywordSegment", // [753]
    "GRANTED", // [754]
    "GrantedKeywordSegment", // [755]
    "Current_userKeywordSegment", // [756]
    "Session_userKeywordSegment", // [757]
    "GRAPHVIZ", // [758]
    "CubeRollupClauseSegment", // [759]
    "cube_rollup_clause", // [760]
    "GroupingSetsClauseSegment", // [761]
    "GROUPING", // [762]
    "grouping_sets_clause", // [763]
    "GroupByClauseTerminatorGrammar", // [764]
    "GroupingKeywordSegment", // [765]
    "SetsKeywordSegment", // [766]
    "SETS", // [767]
    "GROUPS", // [768]
    "identifier", // [769]
    "IdentifierSegment", // [770]
    "IfKeywordSegment", // [771]
    "IgnoreKeywordSegment", // [772]
    "RespectKeywordSegment", // [773]
    "NullsKeywordSegment", // [774]
    "NULLS", // [775]
    "AscKeywordSegment", // [776]
    "DescKeywordSegment", // [777]
    "INITIAL", // [778]
    "INPUT", // [779]
    "IntoKeywordSegment", // [780]
    "INTO", // [781]
    "DefaultValuesGrammar", // [782]
    "IntervalKeywordSegment", // [783]
    "YearKeywordSegment", // [784]
    "MonthKeywordSegment", // [785]
    "DayKeywordSegment", // [786]
    "HourKeywordSegment", // [787]
    "MinuteKeywordSegment", // [788]
    "SecondKeywordSegment", // [789]
    "INVOKER", // [790]
    "IO", // [791]
    "NanLiteralSegment", // [792]
    "NAN", // [793]
    "UnknownLiteralSegment", // [794]
    "NormalizedGrammar", // [795]
    "ISOLATION", // [796]
    "ConditionalJoinKeywordsGrammar", // [797]
    "JoinKeywordsGrammar", // [798]
    "JOIN", // [799]
    "NestedJoinGrammar", // [800]
    "indented_using_on", // [801]
    "MatchConditionSegment", // [802]
    "match_condition", // [803]
    "USING", // [804]
    "JoinOnConditionSegment", // [805]
    "join_on_condition", // [806]
    "JoinUsingConditionGrammar", // [807]
    "UnconditionalJoinKeywordsGrammar", // [808]
    "ExtendedNaturalJoinKeywordsGrammar", // [809]
    "JoinKeywordSegment", // [810]
    "indented_on_contents", // [811]
    "InnerKeywordSegment", // [812]
    "FullKeywordSegment", // [813]
    "LeftKeywordSegment", // [814]
    "RightKeywordSegment", // [815]
    "OuterKeywordSegment", // [816]
    "OUTER", // [817]
    "UsingKeywordSegment", // [818]
    "JSON_ARRAY", // [819]
    "JSON_EXISTS", // [820]
    "JSON_OBJECT", // [821]
    "JSON_QUERY", // [822]
    "JSON_TABLE", // [823]
    "JSON_VALUE", // [824]
    "KEEP", // [825]
    "KEY", // [826]
    "KEYS", // [827]
    "lambda_arrow", // [828]
    "LambdaArrowSegment", // [829]
    "LATERAL", // [830]
    "LEVEL", // [831]
    "LikeGrammar", // [832]
    "EscapeKeywordSegment", // [833]
    "LISTAGG", // [834]
    "OverflowKeywordSegment", // [835]
    "OVERFLOW", // [836]
    "ErrorKeywordSegment", // [837]
    "CountKeywordSegment", // [838]
    "ArrayLiteralSegment", // [839]
    "array_literal", // [840]
    "TypedArrayLiteralSegment", // [841]
    "typed_array_literal", // [842]
    "ObjectLiteralSegment", // [843]
    "object_literal", // [844]
    "LOCAL", // [845]
    "LOGICAL", // [846]
    "MapKeywordSegment", // [847]
    "MapTypeSchemaSegment", // [848]
    "map_type_schema", // [849]
    "MATCH", // [850]
    "MATCH_RECOGNIZE", // [851]
    "MATCHED", // [852]
    "MATCHES", // [853]
    "MEASURES", // [854]
    "MergeKeywordSegment", // [855]
    "MergeMatchedClauseSegment", // [856]
    "merge_when_matched_clause", // [857]
    "MergeNotMatchedClauseSegment", // [858]
    "merge_when_not_matched_clause", // [859]
    "MatchedKeywordSegment", // [860]
    "ThenKeywordSegment", // [861]
    "THEN", // [862]
    "MergeUpdateClauseSegment", // [863]
    "merge_update_clause", // [864]
    "MergeDeleteClauseSegment", // [865]
    "merge_delete_clause", // [866]
    "MergeInsertClauseSegment", // [867]
    "merge_insert_clause", // [868]
    "MergeIntoLiteralGrammar", // [869]
    "AliasedTableReferenceGrammar", // [870]
    "MergeMatchSegment", // [871]
    "merge_match", // [872]
    "SetClauseListSegment", // [873]
    "set_clause_list", // [874]
    "naked_identifier", // [875]
    "WindowSpecificationSegment", // [876]
    "window_specification", // [877]
    "NamedWindowExpressionSegment", // [878]
    "named_window_expression", // [879]
    "null_literal", // [880]
    "NATURAL", // [881]
    "NaturalKeywordSegment", // [882]
    "sign_indicator", // [883]
    "NESTED", // [884]
    "newline", // [885]
    "NewlineSegment", // [886]
    "NFC", // [887]
    "NFD", // [888]
    "NFKC", // [889]
    "NFKD", // [890]
    "UnorderedSelectStatementSegment", // [891]
    "WithCompoundStatementSegment", // [892]
    "with_compound_statement", // [893]
    "NonSetSelectableGrammar", // [894]
    "BracketedSetExpressionGrammar", // [895]
    "MergeStatementSegment", // [896]
    "merge_statement", // [897]
    "SetExpressionSegment", // [898]
    "NONE", // [899]
    "NORMALIZE", // [900]
    "RawNotSegment", // [901]
    "NULLIF", // [902]
    "ColonSegment", // [903]
    "ObjectLiteralElementSegment", // [904]
    "object_literal_element", // [905]
    "start_curly_bracket", // [906]
    "CastOperatorSegment", // [907]
    "StartSquareBracketSegment", // [908]
    "StartBracketSegment", // [909]
    "OMIT", // [910]
    "ONE", // [911]
    "ReplaceKeywordSegment", // [912]
    "REPLACE", // [913]
    "WithFillSegment", // [914]
    "with_fill", // [915]
    "LimitClauseSegment", // [916]
    "limit_clause", // [917]
    "NoorderKeywordSegment", // [918]
    "ORDINALITY", // [919]
    "OUTPUT", // [920]
    "OverKeywordSegment", // [921]
    "OVER", // [922]
    "parameter", // [923]
    "?", // [924]
    "PARTITION", // [925]
    "PartitionKeywordSegment", // [926]
    "PARTITIONS", // [927]
    "PASSING", // [928]
    "PAST", // [929]
    "PATH", // [930]
    "SlashSegment", // [931]
    "path_segment", // [932]
    "PATTERN", // [933]
    "PER", // [934]
    "PERIOD", // [935]
    "PERMUTE", // [936]
    "pipe", // [937]
    "PLAN", // [938]
    "POSITION", // [939]
    "OverClauseSegment", // [940]
    "over_clause", // [941]
    "FilterClauseGrammar", // [942]
    "WithinGroupClauseSegment", // [943]
    "WITHIN", // [944]
    "withingroup_clause", // [945]
    "WithOrdinalityClauseSegment", // [946]
    "withordinality_clause", // [947]
    "PRECISION", // [948]
    "PREPARE", // [949]
    "BooleanKeywordSegment", // [950]
    "TinyintKeywordSegment", // [951]
    "SmallintKeywordSegment", // [952]
    "IntegerKeywordSegment", // [953]
    "IntKeywordSegment", // [954]
    "BigintKeywordSegment", // [955]
    "RealKeywordSegment", // [956]
    "DoubleKeywordSegment", // [957]
    "DecimalKeywordSegment", // [958]
    "BracketedArguments", // [959]
    "bracketed_arguments", // [960]
    "CharKeywordSegment", // [961]
    "VarcharKeywordSegment", // [962]
    "VarbinaryKeywordSegment", // [963]
    "TimeWithTZGrammar", // [964]
    "IpaddressKeywordSegment", // [965]
    "UuidKeywordSegment", // [966]
    "PRUNE", // [967]
    "SOME", // [968]
    "SomeKeywordSegment", // [969]
    "quoted_identifier", // [970]
    "quoted_literal", // [971]
    "QUOTES", // [972]
    "raw_comparison_operator", // [973]
    "RECURSIVE", // [974]
    "ReferenceMatchGrammar", // [975]
    "ReferentialActionGrammar", // [976]
    "MatchKeywordSegment", // [977]
    "PartialKeywordSegment", // [978]
    "SimpleKeywordSegment", // [979]
    "ActionKeywordSegment", // [980]
    "REFRESH", // [981]
    "REPEATABLE", // [982]
    "RESET", // [983]
    "RevokeKeywordSegment", // [984]
    "ROLES", // [985]
    "RowTypeSchemaSegment", // [986]
    "struct_type_schema", // [987]
    "RUNNING", // [988]
    "TablesampleKeywordSegment", // [989]
    "SYSTEM", // [990]
    "BernoulliKeywordSegment", // [991]
    "SystemKeywordSegment", // [992]
    "RepeatableKeywordSegment", // [993]
    "SCALAR", // [994]
    "SECURITY", // [995]
    "SEEK", // [996]
    "WildcardExpressionSegment", // [997]
    "wildcard_expression", // [998]
    "SelectClauseModifierSegment", // [999]
    "select_clause_modifier", // [1000]
    "SelectClauseElementSegment", // [1001]
    "select_clause_element", // [1002]
    "SelectClauseTerminatorGrammar", // [1003]
    "SelectClauseSegment", // [1004]
    "select_clause", // [1005]
    "GroupByClauseSegment", // [1006]
    "groupby_clause", // [1007]
    "HavingClauseSegment", // [1008]
    "having_clause", // [1009]
    "NamedWindowSegment", // [1010]
    "named_window", // [1011]
    "OffsetClauseSegment", // [1012]
    "offset_clause", // [1013]
    "FetchClauseSegment", // [1014]
    "fetch_clause", // [1015]
    "WithCompoundNonSelectStatementSegment", // [1016]
    "NonWithSelectableGrammar", // [1017]
    "statement_terminator", // [1018]
    "MaxvalueKeywordSegment", // [1019]
    "MinvalueKeywordSegment", // [1020]
    "SERIALIZABLE", // [1021]
    "SESSION", // [1022]
    "SetClauseSegment", // [1023]
    "set_clause", // [1024]
    "UnionKeywordSegment", // [1025]
    "IntersectKeywordSegment", // [1026]
    "ExceptKeywordSegment", // [1027]
    "SessionKeywordSegment", // [1028]
    "SHOW", // [1029]
    "PositiveSegment", // [1030]
    "NegativeSegment", // [1031]
    "SKIP", // [1032]
    "slash", // [1033]
    "slice", // [1034]
    "star", // [1035]
    "AlterTableStatementSegment", // [1036]
    "alter_table_statement", // [1037]
    "AnalyzeStatementSegment", // [1038]
    "analyze_statement", // [1039]
    "CommentOnStatementSegment", // [1040]
    "CreateFunctionStatementSegment", // [1041]
    "create_function_statement", // [1042]
    "CreateRoleStatementSegment", // [1043]
    "create_role_statement", // [1044]
    "CreateSchemaStatementSegment", // [1045]
    "create_schema_statement", // [1046]
    "CreateTableStatementSegment", // [1047]
    "create_table_statement", // [1048]
    "CreateViewStatementSegment", // [1049]
    "create_view_statement", // [1050]
    "DescribeStatementSegment", // [1051]
    "describe_statement", // [1052]
    "DropFunctionStatementSegment", // [1053]
    "drop_function_statement", // [1054]
    "DropRoleStatementSegment", // [1055]
    "drop_role_statement", // [1056]
    "DropSchemaStatementSegment", // [1057]
    "drop_schema_statement", // [1058]
    "DropTableStatementSegment", // [1059]
    "drop_table_statement", // [1060]
    "DropViewStatementSegment", // [1061]
    "drop_view_statement", // [1062]
    "ExplainStatementSegment", // [1063]
    "explain_statement", // [1064]
    "SetSchemaStatementSegment", // [1065]
    "set_schema_statement", // [1066]
    "TransactionStatementSegment", // [1067]
    "transaction_statement", // [1068]
    "UseStatementSegment", // [1069]
    "use_statement", // [1070]
    "SetSessionStatementSegment", // [1071]
    "set_session_statement", // [1072]
    "STATS", // [1073]
    "ConcatSegment", // [1074]
    "SUBSET", // [1075]
    "SUBSTRING", // [1076]
    "ForeignKeyGrammar", // [1077]
    "Expression_A_Unary_Operator_Grammar", // [1078]
    "Expression_C_Grammar", // [1079]
    "Expression_B_Unary_Operator_Grammar", // [1080]
    "TEXT", // [1081]
    "TEXT_STRING", // [1082]
    "tilde", // [1083]
    "ZoneKeywordSegment", // [1084]
    "ZONE", // [1085]
    "AtKeywordSegment", // [1086]
    "TRANSACTION", // [1087]
    "CommitKeywordSegment", // [1088]
    "WorkKeywordSegment", // [1089]
    "WORK", // [1090]
    "RollbackKeywordSegment", // [1091]
    "TransactionKeywordSegment", // [1092]
    "IsolationKeywordSegment", // [1093]
    "LevelKeywordSegment", // [1094]
    "UncommittedKeywordSegment", // [1095]
    "UNCOMMITTED", // [1096]
    "CommittedKeywordSegment", // [1097]
    "SerializableKeywordSegment", // [1098]
    "TRIM", // [1099]
    "BothKeywordSegment", // [1100]
    "LeadingKeywordSegment", // [1101]
    "TrailingKeywordSegment", // [1102]
    "TRY_CAST", // [1103]
    "StructLiteralSegment", // [1104]
    "UESCAPE", // [1105]
    "NaturalJoinKeywordsGrammar", // [1106]
    "UnconditionalCrossJoinKeywordsGrammar", // [1107]
    "HorizontalJoinKeywordsGrammar", // [1108]
    "UNKNOWN", // [1109]
    "UNMATCHED", // [1110]
    "UNNEST", // [1111]
    "VALIDATE", // [1112]
    "VALUE", // [1113]
    "VERBOSE", // [1114]
    "VERSION", // [1115]
    "indented_then", // [1116]
    "indented_then_contents", // [1117]
    "whitespace", // [1118]
    "WhitespaceSegment", // [1119]
    "PartitionClauseSegment", // [1120]
    "partitionby_clause", // [1121]
    "FrameClauseSegment", // [1122]
    "frame_clause", // [1123]
    "RecursiveKeywordSegment", // [1124]
    "indented_ctes", // [1125]
    "CTEDefinitionSegment", // [1126]
    "common_table_expression", // [1127]
    "NonWithNonSelectableGrammar", // [1128]
    "BindingKeywordSegment", // [1129]
    "OrdinalityKeywordSegment", // [1130]
    "WithinKeywordSegment", // [1131]
];

pub static AUX_DATA: &[u32] = &[
    0, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1, 0,
    0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0,
    1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 4294967295, 4294967295, 0, 162, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 163, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 164, 1, 2, 1, 1, 4, 3, 1, 4, 5, 45, 177, 178, 1, 177,
    3, 3, 4, 179, 180, 181, 178, 1, 181, 3, 3, 4, 179, 1, 2, 1,
    0, 29, 1, 2, 1, 1, 4, 3, 1, 4, 5, 190, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 2, 1, 0, 278, 279, 178,
    1, 279, 3, 3, 4, 179, 280, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 0, 1, 2, 1, 284, 1, 2, 1, 1, 4, 3, 1, 4, 5, 284,
    285, 286, 1, 285, 3, 3, 285, 4, 287, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 0, 1, 0, 157, 307, 178, 1, 307, 3, 3, 4, 179, 308,
    309, 178, 1, 309, 3, 3, 4, 179, 1, 2, 0, 219, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 0, 1, 2, 0, 291, 311, 178, 1, 311, 3,
    3, 4, 179, 292, 312, 178, 1, 312, 3, 3, 4, 179, 1, 2, 0, 1,
    2, 1, 0, 172, 1, 2, 1, 1, 4, 3, 1, 4, 5, 317, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 318, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 269, 1, 2, 1, 1, 4, 3, 1, 4, 5, 320, 321, 322, 323, 324,
    299, 5, 325, 326, 1, 325, 2, 3, 4, 0, 330, 331, 0, 2, 3, 330,
    3, 332, 0, 1, 3, 333, 1, 2, 1, 1, 4, 3, 1, 4, 5, 334,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 247, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 285, 286, 9, 3, 3, 285, 4, 293, 285, 178, 1, 285,
    3, 3, 4, 179, 0, 248, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    353, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 1,
    0, 1, 2, 1, 354, 355, 0, 2, 3, 354, 1, 2, 1, 193, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 2, 1, 362, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 363, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 0, 364, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 378, 1, 2, 1, 1, 4, 3, 1, 4, 5, 379, 380,
    178, 1, 380, 3, 3, 4, 179, 17, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 381, 1, 2, 1, 1, 4, 3, 1, 4, 5, 249, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 4, 326, 9, 2, 3, 4, 1, 0, 0, 306, 385,
    178, 1, 385, 3, 3, 4, 179, 306, 386, 178, 1, 386, 3, 3, 4, 179,
    306, 387, 178, 1, 387, 3, 3, 4, 179, 0, 1, 2, 1, 0, 0, 1,
    0, 1, 2, 1, 0, 4294967295, 1, 0, 209, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 0, 409, 1, 2, 1, 1, 4, 3, 1, 4, 5, 50, 410,
    178, 1, 410, 3, 3, 4, 179, 402, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 0, 413, 414, 10, 3, 3, 413, 4, 415, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 416, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    215, 424, 9, 3, 3, 215, 4, 285, 425, 0, 2, 3, 285, 215, 426, 0,
    2, 3, 215, 0, 436, 1, 2, 1, 1, 4, 3, 1, 4, 5, 390, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 437, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 438, 1, 2, 1, 1, 4, 3, 1, 4, 5, 3, 4, 1, 0,
    0, 1, 0, 1, 2, 1, 60, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 0, 1, 2, 0, 0, 1, 0, 1, 2, 1, 0, 0, 0, 0, 4294967295,
    4294967295, 0, 0, 0, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 1,
    2, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 2, 1, 0,
    4294967295, 4294967295, 0, 1, 2, 1, 0, 428, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 502, 503, 326, 1, 503, 2, 3, 4, 502, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 2, 1, 509, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 510, 1, 2, 1, 1, 4, 3, 1, 4, 5, 320, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 511, 1, 2, 1, 1, 4, 3, 1, 4, 5, 512,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 513, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 321, 1, 2, 1, 1, 4, 3, 1, 4, 5, 322, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 514, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 25, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 515, 326, 1,
    515, 2, 3, 4, 0, 0, 220, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    523, 928, 1, 503, 326, 1, 503, 2, 3, 4, 0, 1, 0, 1, 2, 1,
    0, 183, 541, 543, 3, 541, 542, 183, 3, 3, 542, 4, 0, 526, 527, 528,
    529, 530, 531, 532, 533, 534, 535, 536, 957, 11, 544, 326, 1, 544, 2, 3,
    4, 526, 1, 2, 1, 1, 4, 3, 1, 4, 5, 545, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 250, 1, 2, 1, 1, 4, 3, 1, 4, 5, 393,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 547, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 548, 1, 2, 1, 1, 4, 3, 1, 4, 5, 83, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 555, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 556, 1, 2, 1, 1, 4, 3, 1, 4, 5, 557, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 559, 1, 2, 1, 1, 4, 3, 1, 4, 5, 560,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 561, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 290, 285, 178, 1, 285, 3, 3, 4, 179, 156, 562, 178, 1,
    562, 3, 3, 4, 179, 251, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    3, 4, 1, 206, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 372,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 567, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 1, 571, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    572, 573, 178, 1, 573, 3, 3, 4, 179, 374, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 575, 1, 2, 1, 1, 4, 3, 1, 4, 5, 576, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 577, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 472, 1, 2, 1, 1, 4, 3, 1, 4, 5, 275, 578, 178, 1, 578,
    3, 3, 4, 179, 67, 1, 2, 1, 1, 4, 3, 1, 4, 5, 579, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 580, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0,
    1, 2, 1, 0, 0, 4294967295, 4294967295, 0, 0, 0, 1, 0, 0, 1, 2, 1,
    0, 1, 0, 1, 2, 1, 0, 0, 0, 631, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 221, 1, 2, 1, 1, 4, 3, 1, 4, 5, 221, 632, 633,
    1, 632, 5, 3, 1, 542, 4, 5, 0, 0, 0, 0, 634, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 1, 4294967295, 4294967295, 0, 1, 0,
    0, 4294967295, 4294967295, 0, 1, 2, 1, 649, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 651, 1, 2, 1, 1, 4, 3, 1, 4, 5, 232, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 652, 1, 2, 1, 1, 4, 3, 1, 4, 5, 445,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 126, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 0, 0, 0, 0, 1, 0, 0, 1, 2, 1, 0,
    0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 1, 2, 1, 0, 168, 703, 1,
    0, 0, 4294967295, 4294967295, 0, 184, 703, 1, 1, 2, 1, 0, 54, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 430, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 4294967295, 4294967295,
    0, 1, 2, 1, 116, 1, 2, 1, 1, 4, 3, 1, 4, 5, 5, 503,
    728, 2, 503, 5, 3, 3, 4, 5, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0,
    1, 0, 1, 2, 1, 0, 0, 132, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 741, 741, 424, 3, 741, 215, 741, 3, 3, 215, 4, 742, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 140, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 0, 754, 1, 2, 1, 1, 4, 3, 1, 4, 5, 72, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 758, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 0, 0, 0, 146, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1, 1, 0, 1, 0, 762, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1, 0, 1, 2, 1, 768, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 2, 1, 0, 0, 672, 1, 2, 1, 1, 4, 3, 1, 4, 5, 528,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 769, 770, 9, 3, 3, 769, 4,
    211, 1, 2, 1, 1, 4, 3, 1, 4, 5, 723, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 492, 1, 2, 1, 1, 4, 3, 1, 4, 5, 34,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 2, 1, 0,
    473, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 778, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 431, 1, 2, 1, 1, 4, 3, 1, 4, 5, 779,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 85, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 252, 1, 2, 1, 1, 4, 3, 1, 4, 5, 253, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 673, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 222, 1, 2, 1, 1, 4, 3, 1, 4, 5, 781, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 790, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 791, 1, 2, 1, 1, 4, 3, 1, 4, 5, 254, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 337, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    796, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 168, 801,
    1, 0, 184, 801, 1, 0, 799, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    366, 811, 1, 1, 2, 1, 0, 184, 811, 1, 0, 0, 1, 0, 1, 2,
    1, 255, 1, 2, 1, 1, 4, 3, 1, 4, 5, 819, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 820, 1, 2, 1, 1, 4, 3, 1, 4, 5, 821,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 822, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 823, 1, 2, 1, 1, 4, 3, 1, 4, 5, 824, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 825, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 826, 1, 2, 1, 1, 4, 3, 1, 4, 5, 827, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 2, 9, 4, 3, 1, 4, 5, 336, 828, 178,
    1, 828, 3, 3, 4, 179, 1, 0, 1, 2, 1, 0, 243, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 830, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    706, 1, 2, 1, 1, 4, 3, 1, 4, 5, 432, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 831, 1, 2, 1, 1, 4, 3, 1, 4, 5, 470, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 339, 339, 424, 3, 339, 215, 339, 3,
    3, 215, 4, 0, 1, 2, 1, 0, 0, 0, 674, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 834, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    0, 542, 633, 9, 5, 3, 1, 542, 4, 5, 542, 543, 9, 3, 3, 542,
    4, 845, 1, 2, 1, 1, 4, 3, 1, 4, 5, 323, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 324, 1, 2, 1, 1, 4, 3, 1, 4, 5, 846,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 256, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 2, 0, 1, 2, 1, 0, 850, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 851, 1, 2, 1, 1, 4, 3, 1, 4, 5, 852, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 853, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 121, 1, 2, 1, 1, 4, 3, 1, 4, 5, 854, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 644, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 4294967295, 4294967295, 0, 0, 0, 1, 2, 1, 0, 168, 801, 1, 184, 801, 1,
    218, 285, 178, 1, 285, 3, 3, 4, 179, 530, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 288, 285, 178, 1, 285, 3, 3, 4, 179, 531, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 289, 285, 178, 1, 285, 3, 3, 4, 179, 2,
    3, 875, 770, 1, 875, 3, 3, 769, 4, 1, 2, 1, 0, 1, 0, 793,
    880, 633, 1, 880, 5, 3, 1, 542, 4, 5, 881, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 218, 883, 178, 1, 883, 3, 3, 4, 179, 884, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 885, 886, 12, 3, 3, 885, 4, 636, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 887, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 888, 1, 2, 1, 1, 4, 3, 1, 4, 5, 889, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 890, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    477, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 2, 1,
    1, 2, 1, 0, 0, 1, 2, 1, 0, 0, 899, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 900, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 266,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 266, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 223, 1, 2, 1, 1, 4, 3, 1, 4, 5, 223, 880, 633,
    1, 880, 5, 3, 1, 542, 4, 5, 902, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 775, 1, 2, 1, 1, 4, 3, 1, 4, 5, 59, 59, 543, 3,
    59, 542, 59, 3, 3, 542, 4, 57, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 0, 227, 906, 178, 1, 906, 3, 3, 4, 179, 1, 2, 0, 0,
    0, 483, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 675, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 910, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 109, 1, 2, 1, 1, 4, 3, 1, 4, 5, 911, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 640, 1, 2, 1, 1, 4, 3, 1, 4, 5, 751,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 338, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 338, 285, 286, 1, 285, 3, 3, 285, 4, 0, 0, 0, 1,
    0, 0, 166, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 919, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 817, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 920, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 922,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 836, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 4, 4294967295, 923, 326, 1, 923, 2, 3, 4, 924, 923, 178, 1,
    923, 3, 3, 4, 179, 1, 0, 1, 2, 1, 0, 925, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 927, 1, 2, 1, 1, 4, 3, 1, 4, 5, 928,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 929, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 930, 1, 2, 1, 1, 4, 3, 1, 4, 5, 5, 932, 728,
    2, 932, 5, 3, 3, 4, 5, 1, 0, 0, 933, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 934, 1, 2, 1, 1, 4, 3, 1, 4, 5, 935, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 936, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 294, 937, 178, 1, 937, 3, 3, 4, 179, 938, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 217, 285, 178, 1, 285, 3, 3, 4, 179, 939, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 217, 883, 178, 1, 883, 3, 3, 4,
    179, 0, 666, 1, 2, 1, 1, 4, 3, 1, 4, 5, 948, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 949, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 80, 1, 2, 1, 1, 4, 3, 1, 4, 5, 271, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 967, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1, 2, 1, 730, 970, 770, 3, 970, 769, 730, 3, 3, 769, 4, 183,
    971, 543, 3, 971, 542, 183, 3, 3, 542, 4, 972, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 661, 1, 2, 1, 1, 4, 3, 1, 4, 5, 214, 973,
    178, 1, 973, 3, 3, 4, 179, 292, 973, 178, 1, 973, 3, 3, 4, 179,
    291, 973, 178, 1, 973, 3, 3, 4, 179, 335, 973, 178, 1, 973, 3, 3,
    4, 179, 90, 1, 2, 1, 1, 4, 3, 1, 4, 5, 257, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 974, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 4294967295, 1, 0, 0, 0, 981, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    238, 1, 2, 1, 1, 4, 3, 1, 4, 5, 982, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 913, 1, 2, 1, 1, 4, 3, 1, 4, 5, 983, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 724, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 563, 1, 2, 1, 1, 4, 3, 1, 4, 5, 720, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 141, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 336, 285, 178, 1, 285, 3, 3, 4, 179, 433, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 19, 1, 2, 1, 1, 4, 3, 1, 4, 5, 985, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 645, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 504, 503, 326, 1, 503, 2, 3, 4, 504, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 258, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1,
    0, 1, 2, 1, 638, 1, 2, 1, 1, 4, 3, 1, 4, 5, 988, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 1, 2, 1,
    994, 1, 2, 1, 1, 4, 3, 1, 4, 5, 15, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 32, 1, 2, 1, 1, 4, 3, 1, 4, 5, 533, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 995, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 996, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0,
    2, 3429, 2, 3431, 3, 337, 560, 337, 266, 560, 0, 94, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1,
    0, 0, 4294967295, 4294967295, 0, 554, 1018, 178, 1, 1018, 3, 3, 4, 179, 0, 0,
    1021, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1022, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 0, 0, 1, 4294967295, 4294967295, 0, 241, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 1, 2, 1, 1, 767, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 4294967295, 4294967295, 0, 1029, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 0, 1, 0, 183, 970, 770, 3, 970, 769, 183, 3,
    3, 769, 4, 1032, 1, 2, 1, 1, 4, 3, 1, 4, 5, 290, 1033, 178,
    1, 1033, 3, 3, 4, 179, 306, 1034, 178, 1, 1034, 3, 3, 4, 179, 259,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 968, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 289, 1035, 178, 1, 1035, 3, 3, 4, 179, 466, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1073, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 0, 1, 2, 1, 1075, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1076, 1, 2, 1, 1, 4, 3, 1, 4, 5, 179, 178, 9, 3, 3,
    4, 179, 990, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1,
    1, 2, 1, 0, 111, 1, 2, 1, 1, 4, 3, 1, 4, 5, 128, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 694, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 1081, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1082, 1, 2, 1, 1, 4, 3, 1, 4, 5, 862, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 643, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 601, 1083, 178, 1, 1083, 3, 3, 4, 179, 224, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 0, 4294967295, 4294967295, 0, 225, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 260, 1, 2, 1, 1, 4, 3, 1, 4, 5, 52,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 707, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1087, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0,
    1, 0, 0, 1099, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 226, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 226, 632, 633, 1, 632, 5, 3, 1,
    542, 4, 5, 99, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1103, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 22, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1105, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    663, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1096, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 715, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    676, 1, 2, 1, 1, 4, 3, 1, 4, 5, 456, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1109, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1110, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1111, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 4294967295, 4294967295, 0, 101, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    76, 1, 2, 1, 1, 4, 3, 1, 4, 5, 63, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 804, 1, 2, 1, 1, 4, 3, 1, 4, 5, 655, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 656, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 657, 1, 2, 1, 1, 4, 3, 1, 4, 5, 261, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1112, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1113, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 357, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 262, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    263, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1114, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1115, 1, 2, 1, 1, 4, 3, 1, 4, 5, 113, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 168, 1116, 1, 366, 1117, 1, 184, 1117,
    1, 184, 1116, 1, 368, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2,
    1, 0, 0, 276, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1118, 1119, 12,
    3, 3, 4, 1118, 0, 0, 4294967295, 4294967295, 0, 677, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 168, 1125, 1, 1, 0, 184, 1125, 1, 168, 1125, 1, 1,
    0, 184, 1125, 1, 282, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2,
    1, 944, 1, 2, 1, 1, 4, 3, 1, 4, 5, 713, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 5, 728, 9, 3, 3, 4, 5, 1090, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 28, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    104, 1, 2, 1, 1, 4, 3, 1, 4, 5, 536, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1085, 1, 2, 1, 1, 4, 3, 1, 4, 5,
];

pub static AUX_DATA_OFFSETS: &[u32] = &[
    0, 16, 6, 0, 12, 0, 7, 8, 9, 10, 11, 12, 13, 14, 16, 18,
    20, 21, 0, 23, 10, 24, 0, 26, 27, 0, 30, 31, 33, 10, 0, 35,
    31, 33, 10, 36, 0, 30, 38, 33, 14, 0, 35, 38, 33, 11, 10, 14,
    14, 13, 40, 0, 42, 44, 47, 49, 51, 53, 0, 55, 56, 58, 22, 0,
    61, 17, 18, 62, 9, 10, 12, 0, 64, 65, 66, 0, 68, 69, 0, 70,
    71, 0, 8, 18, 73, 74, 0, 21, 0, 61, 19, 14, 0, 65, 66, 75,
    36, 0, 77, 20, 14, 16, 0, 78, 79, 64, 81, 61, 82, 68, 84, 86,
    8, 87, 88, 89, 91, 92, 93, 95, 96, 97, 98, 100, 74, 102, 103, 0,
    30, 79, 105, 23, 106, 108, 25, 110, 112, 114, 115, 117, 118, 119, 120, 69,
    14, 0, 65, 66, 0, 122, 112, 0, 123, 110, 0, 124, 125, 26, 127, 129,
    130, 131, 133, 134, 135, 136, 137, 31, 0, 65, 138, 0, 122, 129, 0, 123,
    127, 0, 124, 139, 27, 142, 144, 40, 0, 147, 28, 40, 0, 62, 30, 148,
    0, 18, 32, 150, 0, 152, 34, 40, 36, 150, 38, 153, 155, 41, 158, 160,
    45, 55, 65, 165, 0, 168, 171, 96, 0, 174, 93, 175, 75, 84, 182, 184,
    0, 186, 188, 97, 107, 119, 0, 191, 192, 58, 194, 195, 117, 0, 196, 58,
    197, 118, 198, 199, 200, 0, 201, 119, 202, 120, 204, 0, 207, 208, 210, 174,
    129, 0, 212, 213, 124, 216, 228, 0, 125, 229, 86, 208, 230, 127, 0, 126,
    233, 234, 235, 105, 237, 0, 239, 128, 240, 51, 186, 0, 201, 110, 210, 186,
    140, 0, 239, 51, 186, 0, 229, 208, 242, 230, 130, 233, 244, 0, 234, 235,
    0, 207, 208, 210, 235, 0, 239, 208, 210, 235, 51, 235, 0, 201, 208, 235,
    131, 0, 245, 26, 21, 246, 0, 207, 265, 267, 0, 245, 268, 132, 150, 0,
    62, 150, 0, 18, 150, 0, 245, 270, 133, 0, 212, 213, 272, 0, 68, 42,
    137, 135, 0, 212, 274, 272, 0, 277, 272, 141, 150, 0, 281, 186, 0, 283,
    162, 160, 0, 212, 213, 272, 165, 175, 184, 194, 295, 296, 297, 298, 299, 300,
    301, 302, 303, 304, 216, 196, 195, 58, 272, 305, 198, 207, 0, 219, 231, 229,
    310, 258, 252, 246, 234, 243, 255, 246, 0, 313, 314, 0, 240, 259, 269, 279,
    289, 0, 319, 304, 313, 216, 327, 328, 153, 235, 272, 0, 246, 216, 49, 240,
    314, 320, 325, 335, 345, 355, 340, 341, 342, 343, 344, 356, 345, 0, 346, 346,
    347, 0, 348, 348, 363, 372, 349, 350, 373, 383, 351, 352, 384, 396, 394, 216,
    401, 399, 235, 404, 410, 356, 413, 423, 175, 0, 174, 359, 240, 426, 361, 429,
    439, 457, 0, 365, 366, 449, 369, 371, 373, 375, 373, 373, 0, 365, 272, 453,
    369, 371, 373, 375, 373, 373, 343, 49, 377, 458, 468, 478, 487, 497, 0, 507,
    517, 0, 525, 382, 523, 174, 383, 384, 526, 535, 544, 553, 388, 216, 153, 327,
    0, 0, 391, 40, 557, 0, 265, 267, 0, 392, 554, 272, 0, 394, 395, 0,
    396, 397, 398, 399, 0, 400, 397, 401, 0, 404, 405, 407, 0, 174, 246, 564,
    0, 265, 267, 401, 0, 283, 561, 559, 558, 212, 213, 272, 0, 568, 578, 174,
    383, 384, 0, 408, 0, 0, 580, 590, 0, 411, 382, 599, 0, 411, 108, 0,
    610, 0, 609, 110, 112, 186, 0, 208, 235, 0, 412, 611, 382, 267, 612, 619,
    629, 639, 213, 417, 418, 419, 420, 421, 422, 423, 640, 647, 653, 0, 347, 347,
    427, 659, 429, 434, 435, 660, 670, 680, 690, 0, 61, 439, 700, 246, 240, 246,
    283, 440, 704, 118, 115, 117, 0, 703, 441, 442, 443, 444, 42, 44, 0, 446,
    40, 0, 240, 447, 0, 61, 10, 242, 448, 0, 61, 450, 451, 115, 242, 42,
    44, 0, 452, 246, 453, 0, 61, 450, 455, 457, 242, 458, 108, 186, 0, 707,
    705, 460, 710, 0, 61, 450, 462, 242, 40, 0, 463, 728, 726, 0, 212, 213,
    725, 216, 722, 720, 382, 240, 361, 0, 61, 18, 242, 150, 0, 61, 14, 242,
    464, 733, 0, 191, 192, 58, 0, 467, 283, 58, 194, 195, 731, 0, 196, 58,
    197, 732, 198, 199, 200, 0, 61, 119, 202, 734, 468, 0, 61, 450, 110, 242,
    186, 742, 740, 739, 235, 230, 0, 471, 186, 0, 738, 474, 475, 270, 401, 0,
    283, 748, 746, 745, 212, 213, 272, 0, 240, 754, 751, 361, 0, 283, 476, 26,
    0, 61, 97, 478, 755, 480, 234, 0, 481, 482, 759, 758, 84, 82, 0, 100,
    482, 756, 235, 484, 108, 484, 108, 108, 186, 767, 0, 485, 486, 487, 240, 212,
    488, 487, 240, 212, 0, 53, 186, 762, 0, 265, 489, 0, 489, 761, 0, 490,
    491, 0, 490, 493, 0, 446, 494, 763, 487, 495, 0, 496, 764, 272, 0, 68,
    117, 497, 498, 0, 61, 62, 150, 0, 61, 450, 112, 242, 186, 105, 240, 774,
    771, 361, 500, 775, 785, 793, 0, 803, 505, 506, 804, 507, 807, 817, 827, 837,
    847, 857, 867, 877, 887, 897, 916, 907, 174, 228, 917, 516, 518, 519, 521, 918,
    929, 524, 0, 941, 939, 938, 525, 537, 956, 0, 944, 538, 539, 540, 945, 328,
    968, 977, 987, 997, 1007, 0, 394, 546, 1017, 1027, 1037, 0, 82, 549, 551, 553,
    1047, 1057, 1067, 0, 558, 228, 40, 1077, 1087, 1097, 1107, 1116, 1125, 1135, 564, 565,
    0, 207, 439, 1136, 246, 240, 246, 566, 0, 207, 10, 210, 448, 566, 0, 207,
    115, 210, 42, 0, 207, 457, 210, 458, 566, 1139, 0, 207, 462, 210, 40, 0,
    207, 18, 210, 174, 0, 207, 14, 210, 464, 566, 0, 207, 119, 202, 0, 207,
    451, 110, 210, 1149, 186, 566, 0, 207, 97, 210, 478, 0, 207, 21, 210, 40,
    566, 0, 207, 62, 210, 150, 0, 207, 112, 210, 186, 566, 0, 371, 272, 1151,
    1161, 1171, 0, 568, 569, 1174, 1184, 1193, 574, 1203, 1213, 1223, 1233, 1243, 1252, 1262,
    1272, 0, 581, 1282, 361, 582, 584, 586, 588, 0, 589, 1284, 1283, 590, 0, 377,
    589, 591, 592, 0, 412, 265, 593, 594, 595, 596, 0, 265, 597, 598, 599, 589,
    0, 600, 588, 1288, 602, 0, 603, 604, 605, 0, 606, 1291, 1290, 0, 1289, 340,
    341, 343, 606, 1295, 602, 0, 603, 604, 1304, 0, 607, 1296, 361, 0, 1299, 608,
    609, 1300, 611, 388, 49, 0, 1319, 327, 153, 1309, 1308, 613, 272, 361, 1306, 1305,
    216, 235, 153, 614, 272, 616, 617, 0, 1312, 568, 519, 1315, 1313, 272, 0, 246,
    1318, 382, 58, 618, 619, 620, 614, 621, 49, 622, 1320, 623, 216, 328, 625, 627,
    235, 629, 0, 1321, 1331, 1341, 0, 635, 1352, 233, 637, 1353, 58, 272, 487, 1354,
    487, 639, 1355, 641, 0, 283, 642, 1356, 0, 1366, 646, 1374, 647, 1370, 646, 1376,
    646, 0, 650, 1380, 0, 277, 272, 1383, 1393, 1403, 1413, 1423, 0, 0, 125, 653,
    0, 654, 1433, 658, 659, 660, 1434, 0, 662, 1447, 1446, 0, 664, 487, 0, 1444,
    58, 620, 665, 1445, 667, 668, 0, 597, 599, 1448, 639, 669, 0, 53, 1449, 670,
    1451, 277, 678, 679, 0, 147, 192, 0, 680, 192, 681, 682, 683, 500, 685, 635,
    1464, 0, 687, 1455, 1452, 688, 690, 188, 1456, 692, 693, 696, 697, 0, 283, 679,
    188, 693, 699, 1461, 0, 1457, 697, 1483, 1480, 0, 1468, 700, 701, 1465, 670, 0,
    680, 192, 0, 147, 192, 1469, 1473, 0, 1472, 697, 696, 0, 680, 192, 0, 147,
    192, 1477, 1484, 1494, 1504, 704, 272, 1517, 272, 0, 272, 240, 246, 0, 708, 272,
    53, 53, 272, 0, 1505, 525, 272, 53, 272, 0, 709, 1508, 710, 1506, 711, 165,
    0, 1509, 382, 174, 235, 33, 1510, 382, 174, 235, 0, 272, 712, 49, 272, 1512,
    0, 714, 313, 27, 0, 283, 1511, 716, 717, 313, 27, 0, 1513, 0, 272, 718,
    272, 0, 1515, 267, 719, 108, 267, 0, 721, 246, 712, 0, 1516, 267, 719, 108,
    267, 0, 721, 246, 712, 722, 460, 725, 726, 0, 1521, 537, 0, 240, 382, 0,
    13, 228, 1524, 1534, 0, 1544, 0, 174, 729, 355, 1548, 497, 731, 355, 1551, 0,
    212, 1549, 0, 732, 21, 246, 1550, 0, 732, 21, 246, 1554, 1552, 733, 1558, 0,
    734, 735, 736, 0, 0, 42, 1557, 734, 737, 738, 498, 740, 1559, 1569, 1580, 1590,
    0, 743, 1600, 0, 744, 108, 746, 0, 18, 150, 0, 88, 108, 62, 148, 40,
    51, 748, 1601, 0, 283, 743, 750, 0, 283, 752, 750, 0, 753, 664, 71, 0,
    755, 192, 1602, 756, 757, 148, 1603, 1613, 1623, 0, 348, 574, 348, 0, 147, 192,
    1636, 30, 759, 761, 0, 1634, 1633, 235, 58, 272, 764, 1637, 0, 680, 192, 678,
    679, 681, 682, 635, 1638, 0, 1652, 1651, 235, 58, 272, 1648, 764, 1654, 0, 765,
    766, 1667, 1665, 1664, 759, 507, 1670, 0, 681, 1683, 1680, 272, 1684, 0, 680, 192,
    678, 679, 682, 635, 1685, 0, 1695, 1705, 0, 771, 607, 1712, 0, 771, 265, 607,
    1722, 0, 1732, 772, 773, 774, 1733, 1743, 0, 265, 33, 1759, 1756, 1755, 1753, 588,
    361, 153, 1760, 0, 174, 1770, 776, 777, 1771, 1781, 1791, 1801, 0, 84, 780, 186,
    1811, 361, 0, 105, 361, 782, 1812, 1822, 1832, 0, 783, 382, 1842, 784, 785, 786,
    787, 788, 789, 1843, 1853, 1863, 1873, 1883, 1893, 619, 792, 794, 618, 795, 0, 412,
    265, 709, 53, 1894, 0, 1904, 1925, 0, 797, 798, 701, 1914, 800, 0, 1918, 802,
    1921, 805, 807, 1922, 0, 808, 798, 701, 802, 0, 809, 701, 1926, 0, 810, 0,
    0, 108, 1936, 1942, 1939, 272, 1943, 1947, 812, 0, 1946, 813, 814, 815, 816, 0,
    818, 1950, 1948, 174, 1953, 1963, 1973, 1983, 1993, 2003, 2013, 2023, 2033, 2043, 2053, 2061,
    0, 2075, 212, 2072, 2070, 212, 829, 272, 2076, 0, 2086, 2096, 2106, 0, 346, 574,
    346, 2116, 0, 0, 265, 832, 588, 0, 833, 589, 0, 471, 2126, 2136, 0, 678,
    2151, 2148, 2147, 58, 272, 30, 2153, 0, 679, 2152, 58, 272, 0, 49, 58, 2154,
    0, 2164, 0, 108, 835, 2175, 837, 0, 98, 382, 2174, 283, 714, 838, 2176, 382,
    58, 618, 603, 619, 620, 839, 841, 843, 2177, 2186, 0, 2193, 2203, 2213, 2223, 0,
    2233, 2249, 2243, 0, 516, 49, 246, 2246, 0, 516, 49, 246, 0, 847, 848, 0,
    2250, 2260, 2270, 2280, 2290, 2300, 82, 0, 84, 105, 738, 0, 855, 780, 2310, 2320,
    856, 858, 0, 496, 860, 0, 599, 272, 861, 2324, 863, 865, 0, 496, 265, 860,
    0, 599, 272, 861, 867, 0, 869, 2325, 186, 870, 818, 2329, 186, 870, 0, 2326,
    361, 188, 0, 153, 188, 2330, 805, 2333, 871, 0, 100, 873, 2336, 2345, 2355, 2364,
    2374, 2383, 0, 174, 240, 2396, 174, 2393, 876, 0, 682, 2397, 878, 2399, 0, 882,
    429, 2410, 2420, 0, 2429, 2439, 2446, 2456, 2466, 2476, 2486, 2496, 2515, 738, 891, 2506,
    623, 2509, 892, 2512, 894, 895, 0, 2516, 584, 582, 586, 896, 2521, 898, 2520, 2517,
    623, 894, 2522, 2532, 0, 0, 2542, 0, 901, 574, 0, 346, 348, 2543, 0, 2553,
    2563, 2573, 2584, 2594, 2604, 2615, 0, 382, 903, 310, 2636, 2625, 904, 2627, 2639, 729,
    0, 729, 729, 2640, 108, 240, 818, 49, 907, 908, 909, 377, 903, 646, 696, 2641,
    0, 679, 2651, 58, 272, 487, 2652, 487, 639, 2653, 2663, 2673, 2683, 2693, 2703, 2713,
    2723, 0, 484, 912, 0, 680, 192, 2735, 0, 2732, 235, 58, 272, 2733, 776, 777,
    0, 774, 2734, 233, 244, 914, 916, 662, 2737, 678, 679, 681, 682, 662, 635, 2738,
    2748, 680, 918, 2749, 2759, 2769, 0, 722, 921, 2782, 174, 2779, 876, 2783, 2793, 0,
    2803, 2812, 0, 926, 192, 2826, 2823, 2821, 272, 2827, 2837, 2847, 2857, 2867, 2889, 0,
    931, 2887, 2877, 931, 382, 2890, 0, 2900, 2910, 2920, 2930, 2939, 2949, 2958, 2968, 2977,
    940, 942, 943, 946, 0, 0, 2978, 2988, 2998, 0, 3009, 950, 951, 952, 953, 954,
    955, 956, 957, 0, 958, 959, 0, 3008, 961, 962, 959, 963, 653, 538, 964, 965,
    966, 3010, 3020, 3030, 0, 602, 58, 0, 343, 3040, 30, 732, 969, 3041, 361, 3044,
    3055, 3066, 3076, 3086, 3095, 3104, 3113, 3122, 3132, 3142, 0, 92, 186, 105, 975, 3152,
    0, 108, 82, 976, 0, 108, 100, 976, 0, 977, 3156, 813, 978, 979, 3157, 564,
    565, 0, 245, 267, 0, 476, 980, 0, 245, 394, 3158, 3168, 3178, 3188, 3198, 3208,
    3218, 3228, 3238, 0, 984, 0, 743, 750, 446, 3248, 0, 744, 108, 746, 0, 18,
    150, 0, 88, 108, 62, 148, 148, 53, 748, 566, 3249, 3258, 3268, 174, 3278, 3288,
    3298, 3306, 3316, 3329, 3327, 0, 3326, 246, 0, 212, 246, 0, 487, 986, 3332, 3342,
    0, 989, 3352, 991, 992, 3353, 58, 0, 3356, 993, 3357, 58, 3360, 3370, 3380, 3390,
    3400, 3410, 3420, 997, 0, 310, 188, 3421, 709, 30, 0, 93, 999, 3422, 1001, 1003,
    3434, 53, 3424, 277, 0, 680, 192, 678, 683, 635, 3435, 0, 1004, 549, 551, 1006,
    1008, 1010, 165, 1012, 916, 1014, 683, 500, 685, 3456, 3448, 3445, 892, 3452, 3449, 1016,
    1017, 3453, 361, 0, 729, 174, 158, 3457, 0, 729, 174, 158, 3461, 3470, 0, 1019,
    58, 0, 476, 1019, 3471, 0, 1020, 58, 0, 476, 1020, 3472, 3482, 0, 245, 3492,
    1023, 0, 235, 213, 3494, 216, 327, 153, 235, 272, 738, 394, 0, 894, 3495, 0,
    683, 894, 165, 916, 1010, 3499, 3514, 0, 1025, 3509, 709, 30, 0, 3510, 1026, 1027,
    30, 0, 1027, 3511, 0, 0, 245, 14, 242, 464, 0, 245, 1028, 0, 212, 729,
    212, 213, 272, 3515, 0, 3525, 608, 609, 3526, 0, 907, 246, 611, 3530, 3540, 1030,
    1031, 3541, 228, 731, 729, 3542, 174, 0, 174, 383, 710, 3544, 0, 518, 158, 3555,
    3565, 3574, 3583, 3593, 3603, 3612, 3622, 1036, 1038, 1040, 1041, 1043, 1045, 1047, 1049, 586,
    1051, 1053, 1055, 1057, 1059, 1061, 1063, 582, 896, 361, 1065, 1067, 584, 1069, 1071, 646,
    3623, 3633, 1074, 3636, 3634, 0, 310, 188, 0, 3639, 3649, 3659, 3666, 0, 0, 391,
    40, 3676, 0, 455, 105, 0, 396, 105, 0, 1077, 105, 400, 0, 3683, 738, 327,
    153, 186, 3677, 361, 3680, 896, 3684, 3694, 3704, 0, 3714, 1078, 377, 1079, 0, 3718,
    1080, 1079, 0, 0, 0, 3722, 3732, 3742, 3752, 3762, 3771, 0, 3781, 539, 540, 959,
    0, 3782, 283, 714, 539, 1084, 3783, 0, 1086, 539, 1084, 272, 3787, 3797, 3807, 3817,
    3827, 3842, 0, 1088, 1089, 0, 1091, 1089, 0, 467, 1092, 3840, 3839, 0, 1093, 1094,
    3837, 0, 89, 1095, 0, 89, 1097, 0, 993, 89, 1098, 0, 89, 3838, 641, 103,
    3843, 3853, 1100, 1101, 1102, 3854, 3864, 3875, 0, 98, 110, 186, 3885, 3897, 3895, 310,
    3900, 0, 518, 839, 0, 568, 1104, 3910, 3920, 3930, 0, 3940, 1106, 1107, 1108, 3941,
    0, 1025, 3951, 709, 30, 3952, 0, 3962, 3972, 0, 3982, 3992, 0, 1004, 549, 551,
    1006, 1008, 1010, 0, 894, 4002, 0, 683, 894, 4006, 0, 100, 186, 188, 245, 873,
    549, 551, 4016, 0, 77, 448, 4026, 174, 4036, 4046, 4056, 4066, 4076, 4086, 4096, 0,
    546, 4106, 272, 4108, 4118, 4128, 4138, 4148, 4158, 0, 496, 0, 272, 4168, 861, 4171,
    272, 4174, 4177, 4180, 0, 277, 4193, 4190, 272, 4194, 678, 679, 0, 147, 192, 0,
    680, 192, 681, 682, 635, 4195, 4205, 0, 47, 0, 4213, 4212, 0, 174, 383, 0,
    710, 729, 710, 4217, 0, 174, 4227, 926, 680, 1120, 165, 1122, 0, 283, 1124, 4228,
    4231, 1126, 93, 4233, 1128, 0, 283, 1124, 4236, 4239, 1126, 93, 4241, 1017, 0, 283,
    0, 476, 26, 0, 4244, 0, 283, 476, 14, 1129, 0, 283, 1130, 0, 1131, 147,
    4254, 165, 942, 4257, 4267, 4277, 4284, 4294, 4304, 4314, 4324,
];

pub static SEGMENT_TYPE_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 37, 4294967295, 4294967295, 39, 4294967295, 4294967295, 4294967295, 4294967295, 39, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 41, 4294967295, 43, 46, 48, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    37, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 107, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 143, 145, 4294967295, 4294967295, 4294967295, 4294967295, 41, 4294967295, 4294967295, 4294967295, 149,
    4294967295, 4294967295, 4294967295, 151, 4294967295, 4294967295, 4294967295, 41, 4294967295, 151, 4294967295, 154, 4294967295, 4294967295, 159, 161,
    4294967295, 4294967295, 4294967295, 167, 4294967295, 168, 173, 4294967295, 4294967295, 4294967295, 4294967295, 176, 4294967295, 4294967295, 4294967295, 184,
    4294967295, 187, 189, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 215, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 231, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 236, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 187, 4294967295, 4294967295, 4294967295, 4294967295, 187,
    4294967295, 4294967295, 4294967295, 4294967295, 187, 4294967295, 4294967295, 4294967295, 4294967295, 231, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 236,
    4294967295, 4294967295, 4294967295, 4294967295, 236, 4294967295, 4294967295, 4294967295, 4294967295, 236, 4294967295, 236, 4294967295, 4294967295, 4294967295, 236,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 264, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 151, 4294967295,
    4294967295, 151, 4294967295, 4294967295, 151, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 215, 273, 4294967295, 4294967295, 43,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 273, 4294967295, 4294967295, 273, 4294967295, 4294967295, 4294967295, 4294967295, 187, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 215, 273, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 285,
    285, 4294967295, 285, 285, 4294967295, 4294967295, 4294967295, 4294967295, 273, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 264, 4294967295, 4294967295, 4294967295, 264, 4294967295, 4294967295, 315, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 329, 154, 236, 273, 4294967295, 264, 4294967295, 4294967295, 4294967295,
    330, 3, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 236, 354, 4294967295, 358, 4294967295, 4294967295, 176, 4294967295, 4294967295, 360, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 168, 4294967295, 370, 4294967295, 4294967295, 376, 4294967295, 4294967295, 4294967295, 4294967295, 273, 4294967295,
    370, 4294967295, 4294967295, 376, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 389, 4294967295, 154, 4294967295,
    4294967295, 4294967295, 4294967295, 41, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 273, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 403, 4294967295, 4294967295, 406, 4294967295, 4294967295, 4294967295, 264, 4294967295,
    4294967295, 4294967295, 4294967295, 403, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 215, 273, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 187, 4294967295, 4294967295, 236, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 413, 4294967295,
    4294967295, 4294967295, 215, 215, 215, 215, 215, 215, 4294967295, 4294967295, 215, 285, 215, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 264, 4294967295, 264,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 43, 46, 4294967295, 4294967295,
    41, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 449, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 43,
    46, 4294967295, 4294967295, 264, 454, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 459, 4294967295, 187, 4294967295, 4294967295,
    4294967295, 461, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 41, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 215,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 151, 4294967295, 4294967295, 4294967295, 4294967295,
    465, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 469, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    187, 4294967295, 4294967295, 4294967295, 236, 231, 4294967295, 4294967295, 187, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 403, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 215, 273, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 479, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 236, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 187, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 187, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 273, 4294967295, 4294967295,
    4294967295, 4294967295, 499, 4294967295, 4294967295, 4294967295, 151, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 187, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 501, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 43, 43, 4294967295, 508, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 517, 316, 520, 522, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 329,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 550, 552, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 41, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 264, 4294967295, 264, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 449, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 43, 4294967295, 4294967295, 4294967295, 4294967295, 459, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 41, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 465, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 187, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 479, 4294967295, 4294967295, 4294967295, 4294967295, 41,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 151, 4294967295, 4294967295, 4294967295, 4294967295, 187, 4294967295, 4294967295, 4294967295, 273, 4294967295,
    4294967295, 4294967295, 4294967295, 522, 570, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 583, 585, 587, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 59, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 59, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    610, 4294967295, 612, 389, 4294967295, 4294967295, 4294967295, 4294967295, 154, 4294967295, 4294967295, 4294967295, 273, 4294967295, 4294967295, 4294967295,
    4294967295, 236, 154, 615, 273, 4294967295, 48, 4294967295, 4294967295, 522, 520, 4294967295, 4294967295, 273, 4294967295, 264,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 615, 4294967295, 4294967295, 4294967295, 4294967295, 624, 4294967295, 329, 626, 628,
    236, 630, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 273, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 648, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 273, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 671,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 684, 501, 686, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 689, 691, 189, 4294967295, 4294967295, 695, 4294967295, 698, 4294967295, 4294967295, 4294967295,
    189, 695, 4294967295, 4294967295, 4294967295, 4294967295, 698, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 702, 4294967295, 671, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 168, 4294967295, 4294967295, 4294967295, 698, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 184, 4294967295, 4294967295, 4294967295, 705, 273, 4294967295, 273, 4294967295, 273, 4294967295, 264, 4294967295, 4294967295, 273,
    4294967295, 4294967295, 273, 4294967295, 4294967295, 4294967295, 273, 4294967295, 273, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 167,
    4294967295, 4294967295, 4294967295, 4294967295, 236, 4294967295, 4294967295, 4294967295, 4294967295, 236, 4294967295, 273, 4294967295, 4294967295, 273, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 273, 4294967295,
    273, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 264, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 264, 4294967295, 4294967295, 461, 626, 727, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 264, 4294967295, 4294967295, 4294967295, 4294967295, 264, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    43, 499, 4294967295, 4294967295, 4294967295, 43, 4294967295, 43, 43, 739, 499, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 745, 4294967295, 747, 4294967295, 4294967295, 151, 4294967295, 4294967295, 4294967295, 4294967295, 149, 41,
    4294967295, 749, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 149, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 760, 763, 4294967295, 4294967295, 4294967295, 236, 4294967295, 273, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 236, 4294967295, 273, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 760, 508, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 273, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 769, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 154, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 187,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 702, 4294967295, 4294967295, 4294967295, 168, 803,
    4294967295, 806, 4294967295, 184, 4294967295, 4294967295, 4294967295, 702, 803, 4294967295, 4294967295, 702, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 168, 4294967295, 4294967295, 273, 184, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 273, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 273, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 273, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 59, 4294967295, 4294967295, 840, 842, 844, 542, 542, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 517, 4294967295, 264, 4294967295, 4294967295, 517, 4294967295, 264, 4294967295, 4294967295, 849, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 739, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    857, 859, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 273, 4294967295, 4294967295, 864, 866, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 273, 4294967295, 868, 4294967295, 4294967295, 4294967295, 187, 4294967295, 4294967295, 4294967295, 187, 4294967295, 4294967295, 4294967295,
    4294967295, 189, 4294967295, 154, 189, 168, 806, 184, 872, 4294967295, 4294967295, 874, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 877, 4294967295, 4294967295, 4294967295, 879, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 885, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 739, 624, 4294967295,
    624, 4294967295, 893, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 585, 583, 587, 897, 4294967295, 358, 4294967295, 4294967295,
    624, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 905, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 273, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 236, 4294967295, 273, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 915, 917, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 877, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 273, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    941, 4294967295, 945, 947, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 960, 4294967295, 4294967295, 4294967295, 4294967295, 960, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 187, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 745, 4294967295, 747, 4294967295, 4294967295,
    151, 4294967295, 4294967295, 4294967295, 4294967295, 149, 149, 4294967295, 749, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 264, 4294967295, 4294967295, 264, 4294967295, 4294967295, 987, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 998, 4294967295, 4294967295, 189, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1000, 4294967295, 1002, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 684, 4294967295, 4294967295, 4294967295, 1005, 550, 552, 1007,
    1009, 1011, 167, 1013, 917, 1015, 684, 501, 686, 4294967295, 4294967295, 4294967295, 893, 4294967295, 4294967295, 893,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 159, 4294967295, 4294967295, 4294967295, 4294967295, 159, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1024, 4294967295, 236, 215, 4294967295, 4294967295, 4294967295, 154, 236, 273, 739, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    684, 4294967295, 167, 917, 1011, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 465, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 215, 273, 4294967295, 4294967295, 4294967295, 4294967295, 610, 4294967295, 4294967295, 4294967295, 264, 612, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 316, 159, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1037, 1039, 403, 1042, 1044, 1046, 1048, 1050, 587,
    1052, 1054, 1056, 1058, 1060, 1062, 1064, 583, 897, 4294967295, 1066, 1068, 585, 1070, 1072, 4294967295,
    4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 189, 4294967295, 4294967295, 4294967295, 179, 4294967295, 4294967295, 4294967295, 4294967295,
    41, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 739, 4294967295,
    154, 187, 4294967295, 4294967295, 4294967295, 897, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 960,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 273, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 187, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 316, 840, 4294967295, 522, 570, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1005, 550, 552,
    1007, 1009, 1011, 4294967295, 4294967295, 4294967295, 4294967295, 684, 4294967295, 4294967295, 4294967295, 4294967295, 187, 189, 4294967295, 874,
    550, 552, 4294967295, 4294967295, 4294967295, 449, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 273, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 273, 168, 4294967295, 168,
    273, 184, 184, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 273, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1118, 4294967295, 48, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1121, 167, 1123, 4294967295, 4294967295, 4294967295, 168,
    4294967295, 1127, 4294967295, 184, 4294967295, 4294967295, 4294967295, 4294967295, 168, 4294967295, 1127, 4294967295, 184, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 167, 4294967295, 4294967295, 4294967295, 5, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static SEGMENT_CLASS_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 36, 4294967295, 4294967295, 38, 4294967295, 4294967295, 4294967295, 4294967295, 38, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 40, 4294967295, 42, 44, 47, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    36, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 106, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 142, 144, 4294967295, 4294967295, 4294967295, 4294967295, 40, 4294967295, 4294967295, 4294967295, 148,
    4294967295, 4294967295, 4294967295, 150, 4294967295, 4294967295, 4294967295, 40, 4294967295, 150, 4294967295, 153, 4294967295, 4294967295, 158, 160,
    4294967295, 4294967295, 4294967295, 165, 4294967295, 169, 171, 4294967295, 4294967295, 4294967295, 4294967295, 175, 4294967295, 4294967295, 4294967295, 185,
    4294967295, 186, 188, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 202, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 213, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 230, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 235, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 186, 4294967295, 4294967295, 4294967295, 4294967295, 186,
    4294967295, 4294967295, 4294967295, 4294967295, 186, 4294967295, 4294967295, 4294967295, 4294967295, 230, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 235,
    4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295, 235, 4294967295, 4294967295, 4294967295, 235,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 150, 4294967295,
    4294967295, 150, 4294967295, 4294967295, 150, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 213, 272, 4294967295, 4294967295, 42,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 272, 4294967295, 4294967295, 272, 4294967295, 4294967295, 4294967295, 4294967295, 186, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 213, 272, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300,
    301, 4294967295, 303, 304, 4294967295, 4294967295, 4294967295, 4294967295, 272, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 314, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328, 153, 235, 272, 4294967295, 246, 4294967295, 4294967295, 4294967295,
    331, 332, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 286, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 235, 355, 4294967295, 356, 4294967295, 4294967295, 175, 4294967295, 4294967295, 359, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 367, 4294967295, 369, 4294967295, 4294967295, 375, 4294967295, 4294967295, 4294967295, 4294967295, 272, 4294967295,
    369, 4294967295, 4294967295, 375, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 388, 4294967295, 153, 4294967295,
    4294967295, 4294967295, 4294967295, 40, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 272, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 401, 4294967295, 4294967295, 405, 4294967295, 4294967295, 4294967295, 246, 4294967295,
    4294967295, 4294967295, 4294967295, 401, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 213, 272, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 186, 4294967295, 4294967295, 235, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 414, 4294967295,
    4294967295, 4294967295, 213, 417, 418, 419, 420, 421, 4294967295, 4294967295, 424, 425, 426, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 246,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 42, 44, 4294967295, 4294967295,
    40, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 448, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 42,
    44, 4294967295, 4294967295, 246, 453, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 458, 4294967295, 186, 4294967295, 4294967295,
    4294967295, 460, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 40, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 213,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 150, 4294967295, 4294967295, 4294967295, 4294967295,
    464, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 202, 4294967295, 468, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    186, 4294967295, 4294967295, 4294967295, 235, 230, 4294967295, 4294967295, 186, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 401, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 213, 272, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 478, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 235, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 186, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 186, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 272, 4294967295, 4294967295,
    4294967295, 4294967295, 498, 4294967295, 4294967295, 4294967295, 150, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 186, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 500, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 505, 506, 4294967295, 507, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 516, 518, 519, 521, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 328,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 549, 551, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 40, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 448, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 42, 4294967295, 4294967295, 4294967295, 4294967295, 458, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 40, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 464, 4294967295, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 186, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 478, 4294967295, 4294967295, 4294967295, 4294967295, 40,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 150, 4294967295, 4294967295, 4294967295, 4294967295, 186, 4294967295, 4294967295, 4294967295, 272, 4294967295,
    4294967295, 4294967295, 4294967295, 568, 569, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 582, 584, 586, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 603, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 603, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    609, 4294967295, 611, 388, 4294967295, 4294967295, 4294967295, 4294967295, 153, 4294967295, 4294967295, 4294967295, 272, 4294967295, 4294967295, 4294967295,
    4294967295, 235, 153, 614, 272, 4294967295, 617, 4294967295, 4294967295, 568, 519, 4294967295, 4294967295, 272, 4294967295, 246,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 614, 4294967295, 4294967295, 4294967295, 4294967295, 623, 4294967295, 328, 625, 627,
    235, 629, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 272, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 647, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 272, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 670,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 683, 500, 685, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 688, 690, 188, 4294967295, 4294967295, 693, 4294967295, 697, 4294967295, 4294967295, 4294967295,
    188, 693, 4294967295, 4294967295, 4294967295, 4294967295, 697, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 701, 4294967295, 670, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 169, 4294967295, 4294967295, 4294967295, 697, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 185, 4294967295, 4294967295, 4294967295, 704, 272, 4294967295, 272, 4294967295, 272, 4294967295, 246, 4294967295, 4294967295, 272,
    4294967295, 4294967295, 272, 4294967295, 4294967295, 4294967295, 272, 4294967295, 272, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165,
    4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295, 272, 4294967295, 4294967295, 272, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 272, 4294967295,
    272, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 460, 725, 726, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    734, 735, 4294967295, 4294967295, 4294967295, 42, 4294967295, 734, 737, 738, 498, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 744, 4294967295, 746, 4294967295, 4294967295, 150, 4294967295, 4294967295, 4294967295, 4294967295, 148, 40,
    4294967295, 748, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 148, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 759, 761, 4294967295, 4294967295, 4294967295, 235, 4294967295, 272, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295, 272, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 759, 507, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 272, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 770, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 153, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 186,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 701, 4294967295, 4294967295, 4294967295, 169, 802,
    4294967295, 805, 4294967295, 185, 4294967295, 4294967295, 4294967295, 701, 802, 4294967295, 4294967295, 701, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 367, 4294967295, 4294967295, 272, 185, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 272, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 272, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 272, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 603, 4294967295, 4294967295, 839, 841, 843, 633, 543, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 516, 4294967295, 246, 4294967295, 4294967295, 516, 4294967295, 246, 4294967295, 4294967295, 848, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 738, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    856, 858, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 272, 4294967295, 4294967295, 863, 865, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 272, 4294967295, 867, 4294967295, 4294967295, 4294967295, 186, 4294967295, 4294967295, 4294967295, 186, 4294967295, 4294967295, 4294967295,
    4294967295, 188, 4294967295, 153, 188, 169, 805, 185, 871, 4294967295, 4294967295, 873, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 876, 4294967295, 4294967295, 4294967295, 878, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 886, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 738, 891, 4294967295,
    623, 4294967295, 892, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 584, 582, 586, 896, 4294967295, 898, 4294967295, 4294967295,
    623, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 904, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 272, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295, 272, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 914, 916, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 876, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 272, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    940, 4294967295, 943, 946, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 959, 4294967295, 4294967295, 4294967295, 4294967295, 959, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 186, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 744, 4294967295, 746, 4294967295, 4294967295,
    150, 4294967295, 4294967295, 4294967295, 4294967295, 148, 148, 4294967295, 748, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 246, 4294967295, 4294967295, 986, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 997, 4294967295, 4294967295, 188, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 999, 4294967295, 1001, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 683, 4294967295, 4294967295, 4294967295, 1004, 549, 551, 1006,
    1008, 1010, 165, 1012, 916, 1014, 683, 500, 685, 4294967295, 4294967295, 4294967295, 892, 4294967295, 4294967295, 1016,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 158, 4294967295, 4294967295, 4294967295, 4294967295, 158, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1023, 4294967295, 235, 213, 4294967295, 4294967295, 4294967295, 153, 235, 272, 738, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    683, 4294967295, 165, 916, 1010, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 464, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 213, 272, 4294967295, 4294967295, 4294967295, 4294967295, 609, 4294967295, 4294967295, 4294967295, 246, 611, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 518, 158, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1036, 1038, 1040, 1041, 1043, 1045, 1047, 1049, 586,
    1051, 1053, 1055, 1057, 1059, 1061, 1063, 582, 896, 4294967295, 1065, 1067, 584, 1069, 1071, 4294967295,
    4294967295, 4294967295, 1074, 4294967295, 4294967295, 4294967295, 4294967295, 188, 4294967295, 4294967295, 4294967295, 178, 4294967295, 4294967295, 4294967295, 4294967295,
    40, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 738, 4294967295,
    153, 186, 4294967295, 4294967295, 4294967295, 896, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 959,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 272, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 186, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 518, 839, 4294967295, 568, 1104, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1004, 549, 551,
    1006, 1008, 1010, 4294967295, 4294967295, 4294967295, 4294967295, 683, 4294967295, 4294967295, 4294967295, 4294967295, 186, 188, 4294967295, 873,
    549, 551, 4294967295, 4294967295, 4294967295, 448, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 272, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 272, 169, 4294967295, 367,
    272, 185, 185, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 272, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1119, 4294967295, 47, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1120, 165, 1122, 4294967295, 4294967295, 4294967295, 169,
    4294967295, 1126, 4294967295, 185, 4294967295, 4294967295, 4294967295, 4294967295, 169, 4294967295, 1126, 4294967295, 185, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 165, 4294967295, 4294967295, 4294967295, 728, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static CASEFOLD_SPARSE: &[(u32, u8)] = &[
    (1841, 1),
    (2079, 1),
];

pub static TRIM_CHARS_SPARSE: &[(u32, u32, u8)] = &[
];

pub static TRIM_CHARS_DATA: &[u32] = &[
];

pub static SEGMENT_CLASS_TYPES_SPARSE: &[(u32, u32, u8)] = &[
    (35, 0, 2),
    (38, 2, 2),
    (43, 4, 2),
    (50, 6, 2),
    (52, 8, 2),
    (53, 10, 2),
    (54, 12, 3),
    (96, 15, 2),
    (132, 17, 2),
    (181, 19, 2),
    (182, 21, 2),
    (187, 23, 2),
    (191, 25, 3),
    (195, 28, 3),
    (199, 31, 2),
    (201, 33, 3),
    (203, 36, 2),
    (206, 38, 2),
    (207, 40, 2),
    (211, 42, 2),
    (213, 44, 4),
    (214, 48, 2),
    (219, 50, 2),
    (223, 52, 5),
    (225, 57, 3),
    (226, 60, 2),
    (248, 62, 3),
    (250, 65, 2),
    (259, 67, 2),
    (268, 69, 2),
    (274, 71, 3),
    (282, 74, 3),
    (287, 77, 3),
    (292, 80, 3),
    (297, 83, 2),
    (303, 85, 3),
    (308, 88, 3),
    (313, 91, 3),
    (315, 94, 3),
    (319, 97, 3),
    (325, 100, 2),
    (334, 102, 3),
    (337, 105, 3),
    (340, 108, 3),
    (347, 111, 2),
    (348, 113, 2),
    (351, 115, 2),
    (357, 117, 2),
    (360, 119, 2),
    (365, 121, 3),
    (372, 124, 2),
    (373, 126, 2),
    (383, 128, 2),
    (384, 130, 2),
    (386, 132, 2),
    (387, 134, 2),
    (392, 136, 2),
    (403, 138, 2),
    (407, 140, 2),
    (410, 142, 3),
    (423, 145, 2),
    (424, 147, 2),
    (425, 149, 3),
    (426, 152, 2),
    (428, 154, 2),
    (432, 156, 2),
    (433, 158, 1),
    (443, 159, 3),
    (466, 162, 3),
    (467, 165, 2),
    (469, 167, 2),
    (472, 169, 2),
    (475, 171, 2),
    (484, 173, 4),
    (486, 177, 2),
    (489, 179, 2),
    (494, 181, 2),
    (496, 183, 2),
    (499, 185, 2),
    (512, 187, 2),
    (524, 189, 2),
    (526, 191, 2),
    (531, 193, 2),
    (539, 195, 2),
    (551, 197, 2),
    (554, 199, 3),
    (558, 202, 2),
    (563, 204, 2),
    (570, 206, 2),
    (571, 208, 2),
    (597, 210, 3),
    (600, 213, 3),
    (606, 216, 3),
    (610, 219, 2),
    (611, 221, 2),
    (612, 223, 2),
    (613, 225, 2),
    (614, 227, 2),
    (615, 229, 2),
    (618, 231, 3),
    (619, 234, 2),
    (620, 236, 2),
    (637, 238, 2),
    (639, 240, 2),
    (652, 242, 2),
    (653, 244, 2),
    (656, 246, 2),
    (664, 248, 3),
    (671, 251, 2),
    (672, 253, 2),
    (675, 255, 2),
    (676, 257, 2),
    (683, 259, 3),
    (685, 262, 3),
    (689, 265, 2),
    (696, 267, 2),
    (703, 269, 2),
    (715, 271, 3),
    (720, 274, 3),
    (744, 277, 3),
    (746, 280, 2),
    (752, 282, 3),
    (756, 285, 3),
    (757, 288, 2),
    (760, 290, 3),
    (766, 293, 2),
    (773, 295, 2),
    (774, 297, 2),
    (787, 299, 3),
    (802, 302, 3),
    (808, 305, 3),
    (822, 308, 3),
    (845, 311, 2),
    (850, 313, 2),
    (854, 315, 3),
    (860, 318, 3),
    (866, 321, 2),
    (872, 323, 2),
    (873, 325, 2),
    (875, 327, 2),
    (891, 329, 2),
    (892, 331, 2),
    (893, 333, 2),
    (894, 335, 2),
    (911, 337, 2),
    (925, 339, 2),
    (926, 341, 2),
    (934, 343, 2),
    (948, 345, 2),
    (950, 347, 2),
    (956, 349, 3),
    (962, 352, 2),
    (967, 354, 3),
    (974, 357, 2),
    (984, 359, 3),
    (989, 362, 3),
    (996, 365, 3),
    (1002, 368, 3),
    (1007, 371, 2),
    (1013, 373, 3),
    (1018, 376, 3),
    (1022, 379, 2),
    (1027, 381, 2),
    (1028, 383, 2),
    (1045, 385, 2),
    (1046, 387, 2),
    (1047, 389, 2),
    (1078, 391, 2),
    (1094, 393, 2),
    (1104, 395, 2),
    (1106, 397, 2),
    (1107, 399, 2),
    (1112, 401, 2),
    (1116, 403, 2),
    (1121, 405, 3),
    (1122, 408, 2),
    (1123, 410, 2),
    (1124, 412, 2),
    (1126, 414, 3),
    (1129, 417, 2),
    (1130, 419, 2),
    (1133, 421, 2),
    (1135, 423, 2),
    (1142, 425, 2),
    (1147, 427, 2),
    (1149, 429, 2),
    (1150, 431, 2),
    (1151, 433, 2),
    (1152, 435, 3),
    (1153, 438, 2),
    (1165, 440, 2),
    (1180, 442, 2),
    (1190, 444, 2),
    (1231, 446, 2),
    (1244, 448, 2),
    (1245, 450, 2),
    (1246, 452, 2),
    (1253, 454, 2),
    (1254, 456, 2),
    (1255, 462, 2),
    (1258, 458, 2),
    (1260, 460, 2),
    (1264, 464, 2),
    (1265, 466, 2),
    (1270, 468, 2),
    (1276, 470, 2),
    (1278, 472, 2),
    (1289, 474, 2),
    (1301, 476, 2),
    (1302, 478, 2),
    (1304, 480, 2),
    (1306, 482, 2),
    (1308, 484, 2),
    (1311, 486, 2),
    (1314, 488, 2),
    (1318, 490, 2),
    (1320, 492, 2),
    (1327, 494, 2),
    (1332, 496, 3),
    (1337, 499, 3),
    (1339, 502, 2),
    (1342, 504, 2),
    (1358, 506, 2),
    (1360, 508, 2),
    (1369, 510, 2),
    (1379, 512, 2),
    (1382, 514, 2),
    (1383, 516, 2),
    (1384, 518, 2),
    (1401, 520, 2),
    (1405, 522, 2),
    (1413, 524, 2),
    (1418, 526, 2),
    (1424, 528, 2),
    (1425, 530, 2),
    (1429, 538, 2),
    (1431, 532, 2),
    (1432, 534, 2),
    (1433, 536, 2),
    (1434, 540, 2),
    (1444, 542, 2),
    (1446, 544, 2),
    (1449, 546, 3),
    (1454, 549, 3),
    (1455, 552, 2),
    (1457, 554, 2),
    (1477, 556, 3),
    (1490, 559, 2),
    (1491, 561, 2),
    (1495, 563, 3),
    (1497, 566, 2),
    (1512, 568, 3),
    (1514, 571, 2),
    (1524, 573, 2),
    (1525, 575, 2),
    (1531, 577, 2),
    (1543, 579, 3),
    (1569, 582, 2),
    (1583, 584, 3),
    (1626, 587, 2),
    (1631, 589, 2),
    (1633, 591, 2),
    (1639, 593, 2),
    (1640, 595, 2),
    (1643, 597, 2),
    (1653, 599, 2),
    (1678, 601, 4),
    (1687, 605, 2),
    (1716, 607, 2),
    (1723, 609, 2),
    (1746, 611, 2),
    (1749, 613, 2),
    (1750, 615, 2),
    (1751, 617, 2),
    (1752, 619, 5),
    (1753, 624, 3),
    (1764, 627, 2),
    (1766, 629, 2),
    (1769, 631, 2),
    (1771, 633, 2),
    (1774, 635, 2),
    (1786, 637, 2),
    (1792, 639, 2),
    (1793, 641, 2),
    (1799, 643, 2),
    (1802, 645, 2),
    (1803, 647, 2),
    (1810, 649, 2),
    (1812, 651, 2),
    (1816, 653, 3),
    (1820, 656, 3),
    (1825, 659, 2),
    (1827, 661, 2),
    (1828, 663, 2),
    (1830, 665, 2),
    (1832, 667, 2),
    (1835, 669, 2),
    (1848, 671, 2),
    (1852, 673, 2),
    (1861, 675, 3),
    (1869, 678, 2),
    (1870, 680, 2),
    (1872, 682, 2),
    (1874, 684, 2),
    (1880, 686, 2),
    (1881, 688, 2),
    (1882, 690, 2),
    (1883, 692, 2),
    (1885, 694, 2),
    (1888, 696, 2),
    (1916, 698, 2),
    (1940, 700, 2),
    (1962, 702, 3),
    (1964, 705, 2),
    (1973, 707, 2),
    (1974, 709, 2),
    (1996, 711, 2),
    (2008, 713, 2),
    (2032, 715, 2),
    (2034, 717, 2),
    (2035, 719, 2),
    (2053, 721, 2),
    (2058, 723, 2),
    (2092, 725, 3),
    (2139, 728, 2),
    (2141, 730, 2),
    (2144, 732, 3),
    (2149, 735, 3),
    (2150, 738, 3),
    (2152, 741, 2),
    (2167, 743, 2),
    (2170, 745, 2),
    (2173, 747, 2),
    (2195, 749, 2),
    (2198, 751, 2),
    (2204, 753, 2),
    (2206, 755, 2),
    (2216, 757, 2),
    (2220, 759, 2),
    (2221, 761, 2),
    (2222, 763, 2),
    (2223, 765, 2),
    (2224, 767, 2),
    (2225, 769, 2),
    (2226, 771, 2),
    (2227, 773, 2),
    (2228, 775, 2),
    (2229, 777, 2),
    (2230, 779, 2),
    (2231, 781, 2),
    (2232, 783, 2),
    (2236, 785, 2),
    (2239, 787, 2),
    (2246, 789, 2),
    (2251, 791, 2),
    (2272, 793, 2),
    (2274, 795, 3),
    (2275, 798, 2),
    (2279, 800, 2),
    (2280, 802, 3),
    (2281, 805, 2),
    (2282, 807, 2),
    (2288, 809, 2),
    (2290, 811, 2),
    (2291, 813, 2),
    (2292, 815, 2),
    (2313, 817, 3),
    (2321, 820, 2),
    (2322, 822, 2),
    (2327, 824, 2),
    (2331, 826, 2),
    (2332, 828, 2),
    (2349, 830, 2),
    (2350, 832, 2),
    (2359, 834, 2),
    (2360, 836, 2),
    (2361, 838, 2),
    (2362, 840, 2),
    (2363, 842, 2),
    (2364, 844, 2),
    (2365, 846, 2),
    (2366, 848, 2),
    (2367, 850, 2),
    (2368, 852, 2),
    (2369, 854, 2),
    (2370, 856, 2),
    (2371, 858, 2),
    (2372, 860, 2),
    (2373, 862, 2),
    (2374, 864, 2),
    (2375, 866, 2),
    (2376, 868, 2),
    (2378, 870, 2),
    (2379, 872, 2),
    (2380, 874, 2),
    (2381, 876, 2),
    (2382, 878, 2),
    (2386, 880, 2),
    (2391, 882, 2),
    (2395, 884, 3),
    (2400, 887, 2),
    (2414, 889, 2),
    (2416, 891, 2),
    (2417, 893, 3),
    (2421, 896, 2),
    (2447, 898, 2),
    (2459, 900, 2),
    (2507, 902, 3),
    (2514, 905, 2),
    (2515, 907, 2),
    (2517, 909, 2),
    (2518, 911, 2),
    (2541, 913, 2),
    (2542, 915, 2),
    (2543, 917, 2),
    (2544, 919, 2),
    (2545, 921, 2),
    (2546, 923, 2),
    (2551, 925, 2),
    (2556, 927, 3),
    (2557, 930, 2),
    (2559, 932, 2),
    (2560, 934, 2),
    (2561, 936, 2),
    (2565, 938, 3),
    (2578, 941, 2),
    (2588, 943, 2),
    (2592, 945, 2),
    (2600, 947, 2),
    (2614, 949, 3),
    (2616, 952, 3),
    (2633, 955, 2),
    (2634, 957, 2),
    (2635, 959, 2),
    (2641, 961, 2),
    (2650, 963, 2),
    (2673, 965, 2),
    (2677, 967, 3),
];

pub static SEGMENT_CLASS_TYPES_DATA: &[u32] = &[
    37, 3, 39, 3, 39, 3, 3, 41, 3, 43, 3, 46, 3, 41, 48, 37,
    3, 107, 3, 3, 143, 3, 145, 3, 41, 3, 41, 149, 3, 41, 151, 3,
    41, 3, 41, 151, 3, 154, 159, 3, 3, 161, 3, 167, 3, 168, 170, 4,
    173, 3, 3, 176, 3, 184, 168, 170, 4, 3, 41, 187, 189, 3, 3, 41,
    203, 205, 3, 3, 215, 3, 231, 3, 236, 41, 3, 41, 187, 3, 41, 187,
    3, 41, 187, 3, 231, 3, 236, 41, 3, 236, 41, 3, 236, 41, 3, 236,
    41, 3, 236, 41, 3, 264, 3, 41, 151, 3, 41, 151, 3, 41, 151, 3,
    215, 3, 273, 3, 43, 3, 273, 3, 273, 3, 41, 187, 3, 215, 3, 273,
    3, 285, 3, 285, 3, 285, 3, 285, 3, 273, 3, 264, 3, 264, 316, 315,
    3, 3, 329, 3, 154, 3, 236, 41, 3, 273, 3, 264, 3, 330, 3, 3,
    285, 4, 3, 236, 41, 3, 354, 3, 358, 3, 176, 3, 360, 3, 168, 170,
    4, 3, 370, 3, 376, 3, 273, 3, 370, 3, 376, 3, 4, 3, 389, 3,
    154, 3, 41, 3, 273, 3, 403, 3, 406, 41, 3, 264, 3, 403, 3, 215,
    3, 273, 3, 41, 187, 3, 236, 41, 3, 413, 4, 3, 215, 3, 215, 3,
    215, 3, 215, 3, 215, 3, 215, 3, 215, 4, 3, 285, 3, 215, 3, 264,
    3, 264, 3, 43, 3, 46, 3, 41, 3, 449, 41, 3, 43, 3, 46, 3,
    264, 3, 454, 3, 459, 41, 3, 41, 187, 3, 461, 3, 41, 3, 215, 3,
    41, 151, 3, 41, 465, 3, 41, 203, 3, 469, 3, 41, 187, 3, 236, 41,
    3, 231, 3, 41, 187, 3, 403, 3, 215, 3, 273, 3, 41, 479, 3, 236,
    41, 3, 41, 187, 3, 41, 187, 3, 273, 3, 499, 3, 41, 151, 3, 41,
    187, 3, 501, 3, 43, 3, 43, 3, 508, 3, 517, 316, 3, 3, 520, 3,
    522, 3, 329, 3, 550, 3, 552, 3, 41, 3, 264, 3, 264, 3, 449, 41,
    3, 43, 3, 459, 41, 3, 41, 3, 41, 465, 3, 41, 203, 3, 41, 187,
    3, 41, 479, 3, 41, 3, 41, 151, 3, 41, 187, 3, 273, 3, 522, 3,
    570, 3, 583, 3, 585, 3, 587, 3, 59, 3, 59, 3, 610, 3, 612, 3,
    389, 3, 154, 3, 273, 3, 236, 41, 3, 154, 3, 615, 3, 273, 3, 41,
    48, 3, 522, 3, 520, 3, 273, 3, 264, 3, 615, 3, 624, 3, 329, 3,
    626, 628, 3, 3, 236, 41, 3, 630, 3, 273, 3, 648, 3, 273, 3, 671,
    3, 684, 3, 501, 3, 686, 3, 689, 3, 691, 3, 695, 3, 698, 189, 3,
    189, 3, 3, 695, 3, 698, 3, 702, 3, 671, 3, 698, 3, 705, 3, 273,
    3, 273, 3, 273, 3, 264, 3, 273, 3, 273, 3, 273, 3, 273, 3, 167,
    3, 236, 41, 3, 236, 41, 3, 273, 3, 273, 3, 273, 3, 273, 3, 264,
    3, 264, 3, 461, 3, 626, 3, 727, 3, 354, 3, 354, 3, 264, 3, 264,
    3, 43, 3, 499, 3, 43, 3, 43, 3, 739, 3, 43, 3, 499, 745, 3,
    747, 3, 3, 41, 151, 3, 41, 149, 3, 41, 749, 3, 3, 41, 149, 3,
    760, 3, 763, 3, 236, 41, 3, 273, 3, 236, 41, 3, 273, 3, 760, 3,
    508, 3, 273, 3, 769, 4, 3, 154, 3, 41, 187, 3, 702, 3, 803, 3,
    806, 3, 702, 3, 803, 3, 702, 3, 273, 3, 1, 4, 5, 3, 273, 3,
    273, 3, 273, 3, 59, 840, 3, 3, 842, 3, 844, 3, 1, 542, 4, 5,
    3, 542, 4, 3, 517, 3, 264, 3, 517, 3, 264, 3, 849, 3, 739, 3,
    857, 3, 859, 3, 273, 3, 864, 3, 866, 3, 273, 3, 868, 3, 41, 187,
    3, 41, 187, 189, 3, 3, 154, 189, 3, 3, 806, 3, 872, 3, 874, 3,
    877, 3, 879, 3, 885, 4, 3, 739, 3, 624, 3, 624, 3, 893, 3, 585,
    3, 583, 3, 587, 3, 897, 3, 358, 3, 624, 3, 905, 3, 273, 3, 236,
    41, 3, 273, 3, 915, 3, 917, 3, 877, 3, 273, 3, 941, 3, 945, 3,
    947, 3, 960, 3, 960, 3, 41, 187, 745, 3, 747, 3, 3, 41, 151, 3,
    41, 149, 3, 41, 149, 749, 3, 3, 264, 3, 264, 3, 987, 3, 998, 189,
    3, 3, 1000, 3, 1002, 3, 684, 3, 1005, 3, 550, 3, 552, 3, 1007, 3,
    1009, 3, 1011, 3, 167, 3, 1013, 3, 917, 3, 1015, 3, 684, 3, 501, 3,
    686, 3, 893, 3, 893, 159, 3, 159, 3, 3, 1024, 3, 236, 41, 3, 215,
    3, 154, 3, 236, 41, 3, 273, 3, 739, 3, 684, 3, 167, 3, 917, 3,
    1011, 3, 41, 465, 3, 215, 3, 273, 3, 610, 3, 264, 3, 612, 316, 3,
    159, 3, 1037, 3, 1039, 3, 3, 403, 3, 1042, 3, 1044, 3, 1046, 3, 1048,
    3, 1050, 3, 587, 3, 1052, 3, 1054, 3, 1056, 3, 1058, 3, 1060, 3, 1062,
    3, 1064, 3, 583, 3, 897, 3, 1066, 3, 1068, 3, 585, 3, 1070, 3, 1072,
    3, 285, 189, 3, 3, 4, 179, 3, 41, 3, 739, 3, 154, 3, 41, 187,
    3, 897, 3, 960, 3, 273, 3, 41, 187, 316, 3, 840, 3, 3, 522, 3,
    570, 3, 1005, 3, 550, 3, 552, 3, 1007, 3, 1009, 3, 1011, 3, 684, 3,
    41, 187, 189, 3, 3, 874, 3, 550, 3, 552, 3, 449, 41, 3, 273, 3,
    273, 3, 273, 3, 273, 3, 4, 1118, 3, 41, 48, 3, 1121, 3, 167, 3,
    1123, 3, 1127, 3, 1127, 3, 167, 3, 4, 5,
];

pub static REGEX_PATTERNS: &[&str] = &[
    r#"[A-Z_][A-Z0-9_]*"#, // [0]
    r#"^(NOT)$"#, // [1]
    r#"[A-Z0-9_]*[A-Z][A-Z0-9_]*"#, // [2]
    r#"^(ALTER|AND|AS|BETWEEN|BY|CASE|CAST|CONSTRAINT|CREATE|CROSS|CUBE|CURRENT_CATALOG|CURRENT_DATE|CURRENT_PATH|CURRENT_ROLE|CURRENT_SCHEMA|CURRENT_TIME|CURRENT_TIMESTAMP|CURRENT_USER|DEALLOCATE|DELETE|DESCRIBE|DISTINCT|DROP|ELSE|END|ESCAPE|EXCEPT|EXECUTE|EXISTS|EXTRACT|FALSE|FOR|FROM|FULL|FUNCTION|GROUP|GROUPING|HAVING|IN|INNER|INSERT|INTERSECT|INTO|IS|JOIN|JSON_ARRAY|JSON_EXISTS|JSON_OBJECT|JSON_QUERY|JSON_TABLE|JSON_VALUE|LEFT|LIKE|LISTAGG|LOCALTIME|LOCALTIMESTAMP|NATURAL|NORMALIZE|NOT|NULL|ON|OR|ORDER|OUTER|PREPARE|RECURSIVE|RIGHT|ROLLUP|SELECT|SKIP|TABLE|THEN|TRIM|TRUE|UESCAPE|UNION|UNNEST|USING|VALUES|WHEN|WHERE|WITH)$"#, // [3]
    r#"\"?[A-Z][A-Z0-9_]*\"?"#, // [4]
];

pub static HINT_STRING_INDICES: &[u32] = &[
    0, 15, 17, 19, 22, 25, 28, 29, 32, 34,
    45, 50, 52, 54, 57, 59, 60, 63, 67, 72,
    76, 17, 15, 80, 83, 85, 90, 94, 99, 101,
    104, 109, 111, 113, 116, 121, 126, 128, 132, 140,
    141, 140, 141, 146, 156, 157, 157, 156, 162, 163,
    164, 166, 172, 180, 183, 190, 193, 206, 209, 211,
    214, 217, 218, 219, 220, 221, 222, 223, 224, 225,
    226, 157, 227, 59, 183, 45, 164, 232, 164, 232,
    232, 238, 172, 52, 162, 190, 206, 67, 238, 241,
    164, 232, 243, 243, 206, 241, 241, 219, 247, 248,
    249, 220, 250, 251, 252, 253, 254, 255, 256, 257,
    258, 259, 224, 225, 260, 261, 262, 263, 266, 223,
    269, 271, 275, 276, 278, 280, 282, 284, 287, 288,
    278, 289, 217, 218, 290, 291, 292, 293, 294, 217,
    218, 290, 289, 288, 294, 293, 291, 292, 306, 308,
    219, 45, 291, 317, 318, 320, 321, 322, 323, 324,
    222, 333, 334, 247, 335, 288, 278, 289, 217, 218,
    336, 290, 291, 214, 292, 284, 337, 338, 293, 294,
    339, 284, 338, 335, 291, 214, 292, 337, 339, 336,
    338, 248, 221, 226, 226, 221, 353, 45, 94, 357,
    45, 94, 357, 282, 362, 363, 364, 368, 372, 374,
    378, 379, 381, 249, 390, 266, 223, 393, 402, 402,
    266, 282, 409, 209, 111, 113, 111, 113, 337, 223,
    183, 415, 416, 335, 291, 339, 428, 430, 431, 432,
    433, 436, 437, 438, 445, 456, 466, 470, 472, 473,
    473, 472, 477, 483, 83, 85, 101, 258, 492, 5,
    502, 502, 504, 504, 509, 510, 320, 511, 512, 513,
    321, 322, 514, 247, 248, 249, 220, 250, 251, 252,
    253, 254, 255, 257, 259, 224, 225, 260, 261, 262,
    263, 256, 220, 523, 526, 527, 528, 529, 530, 531,
    532, 533, 534, 535, 536, 220, 222, 224, 225, 220,
    224, 225, 224, 225, 526, 545, 250, 357, 547, 548,
    554, 555, 556, 557, 559, 560, 561, 251, 363, 563,
    563, 567, 571, 572, 575, 576, 577, 579, 580, 45,
    83, 85, 94, 101, 357, 282, 470, 266, 34, 266,
    334, 266, 217, 218, 266, 601, 217, 218, 601, 335,
    288, 278, 289, 217, 218, 290, 291, 214, 292, 337,
    293, 294, 339, 217, 218, 601, 220, 221, 222, 223,
    224, 225, 226, 59, 183, 631, 634, 232, 636, 636,
    258, 638, 638, 640, 282, 640, 643, 45, 554, 190,
    280, 402, 415, 60, 83, 557, 206, 580, 85, 644,
    645, 94, 241, 466, 101, 76, 357, 282, 45, 190,
    280, 402, 415, 60, 83, 557, 206, 580, 85, 644,
    645, 94, 241, 466, 101, 76, 357, 282, 649, 651,
    652, 255, 655, 656, 657, 657, 655, 656, 661, 638,
    334, 509, 220, 222, 224, 225, 663, 59, 509, 220,
    222, 224, 225, 663, 59, 220, 222, 224, 225, 663,
    59, 663, 652, 666, 666, 661, 577, 634, 146, 672,
    673, 674, 675, 166, 676, 276, 677, 282, 674, 675,
    672, 677, 577, 673, 676, 694, 430, 353, 706, 707,
    282, 713, 713, 436, 715, 715, 0, 223, 720, 723,
    724, 730, 5, 730, 287, 219, 247, 248, 249, 220,
    250, 251, 252, 253, 254, 255, 256, 257, 258, 259,
    224, 225, 260, 261, 262, 263, 741, 742, 751, 754,
    758, 762, 634, 672, 674, 675, 166, 677, 767, 768,
    634, 674, 675, 166, 677, 528, 723, 724, 775, 317,
    556, 778, 431, 779, 781, 45, 393, 94, 357, 282,
    252, 253, 673, 526, 528, 530, 531, 533, 536, 536,
    531, 530, 533, 790, 791, 254, 793, 796, 799, 109,
    804, 804, 430, 432, 433, 432, 433, 817, 819, 820,
    821, 822, 823, 824, 825, 826, 827, 830, 706, 831,
    50, 675, 834, 836, 575, 99, 227, 845, 323, 324,
    846, 850, 851, 852, 853, 854, 644, 862, 83, 101,
    881, 884, 887, 888, 889, 890, 83, 85, 644, 101,
    45, 94, 899, 900, 335, 902, 910, 911, 913, 232,
    243, 634, 672, 674, 675, 661, 638, 677, 919, 920,
    922, 924, 925, 927, 928, 929, 930, 290, 183, 933,
    934, 935, 936, 938, 939, 944, 948, 949, 260, 259,
    257, 249, 263, 263, 262, 224, 225, 261, 967, 29,
    287, 968, 968, 972, 974, 363, 477, 563, 241, 981,
    982, 983, 985, 645, 988, 333, 990, 990, 994, 995,
    996, 29, 560, 577, 634, 54, 673, 674, 166, 676,
    276, 45, 282, 1021, 1022, 676, 577, 673, 1029, 1032,
    415, 645, 466, 1073, 1075, 1076, 1081, 1082, 1085, 707,
    1087, 1090, 796, 90, 90, 982, 1021, 1096, 640, 104,
    1099, 1103, 1105, 1109, 1110, 1111, 1112, 1113, 1114, 1115,
    634, 146, 672, 674, 675, 166, 677, 166, 925,
];

pub static SIMPLE_HINTS: &[SimpleHintData] = &[
    SimpleHintData::empty(), // [0] reserved for no hint
    SimpleHintData { raw_values_start: 0, raw_values_count: 1, token_types_start: 1, token_types_count: 0 }, // [1]
    SimpleHintData { raw_values_start: 1, raw_values_count: 1, token_types_start: 2, token_types_count: 0 }, // [2]
    SimpleHintData { raw_values_start: 2, raw_values_count: 1, token_types_start: 3, token_types_count: 0 }, // [3]
    SimpleHintData { raw_values_start: 3, raw_values_count: 1, token_types_start: 4, token_types_count: 0 }, // [4]
    SimpleHintData { raw_values_start: 4, raw_values_count: 1, token_types_start: 5, token_types_count: 0 }, // [5]
    SimpleHintData { raw_values_start: 5, raw_values_count: 1, token_types_start: 6, token_types_count: 0 }, // [6]
    SimpleHintData { raw_values_start: 6, raw_values_count: 1, token_types_start: 7, token_types_count: 0 }, // [7]
    SimpleHintData { raw_values_start: 7, raw_values_count: 1, token_types_start: 8, token_types_count: 0 }, // [8]
    SimpleHintData { raw_values_start: 8, raw_values_count: 1, token_types_start: 9, token_types_count: 0 }, // [9]
    SimpleHintData { raw_values_start: 9, raw_values_count: 1, token_types_start: 10, token_types_count: 0 }, // [10]
    SimpleHintData { raw_values_start: 10, raw_values_count: 1, token_types_start: 11, token_types_count: 0 }, // [11]
    SimpleHintData { raw_values_start: 11, raw_values_count: 1, token_types_start: 12, token_types_count: 0 }, // [12]
    SimpleHintData { raw_values_start: 12, raw_values_count: 1, token_types_start: 13, token_types_count: 0 }, // [13]
    SimpleHintData { raw_values_start: 13, raw_values_count: 1, token_types_start: 14, token_types_count: 0 }, // [14]
    SimpleHintData { raw_values_start: 14, raw_values_count: 1, token_types_start: 15, token_types_count: 0 }, // [15]
    SimpleHintData { raw_values_start: 15, raw_values_count: 0, token_types_start: 15, token_types_count: 1 }, // [16]
    SimpleHintData { raw_values_start: 16, raw_values_count: 1, token_types_start: 17, token_types_count: 0 }, // [17]
    SimpleHintData { raw_values_start: 17, raw_values_count: 1, token_types_start: 18, token_types_count: 0 }, // [18]
    SimpleHintData { raw_values_start: 18, raw_values_count: 1, token_types_start: 19, token_types_count: 0 }, // [19]
    SimpleHintData { raw_values_start: 19, raw_values_count: 1, token_types_start: 20, token_types_count: 0 }, // [20]
    SimpleHintData { raw_values_start: 20, raw_values_count: 1, token_types_start: 21, token_types_count: 0 }, // [21]
    SimpleHintData { raw_values_start: 21, raw_values_count: 2, token_types_start: 23, token_types_count: 0 }, // [22]
    SimpleHintData { raw_values_start: 23, raw_values_count: 1, token_types_start: 24, token_types_count: 0 }, // [23]
    SimpleHintData { raw_values_start: 24, raw_values_count: 1, token_types_start: 25, token_types_count: 0 }, // [24]
    SimpleHintData { raw_values_start: 25, raw_values_count: 1, token_types_start: 26, token_types_count: 0 }, // [25]
    SimpleHintData { raw_values_start: 26, raw_values_count: 1, token_types_start: 27, token_types_count: 0 }, // [26]
    SimpleHintData { raw_values_start: 27, raw_values_count: 1, token_types_start: 28, token_types_count: 0 }, // [27]
    SimpleHintData { raw_values_start: 28, raw_values_count: 1, token_types_start: 29, token_types_count: 0 }, // [28]
    SimpleHintData { raw_values_start: 29, raw_values_count: 1, token_types_start: 30, token_types_count: 0 }, // [29]
    SimpleHintData { raw_values_start: 30, raw_values_count: 1, token_types_start: 31, token_types_count: 0 }, // [30]
    SimpleHintData { raw_values_start: 31, raw_values_count: 1, token_types_start: 32, token_types_count: 0 }, // [31]
    SimpleHintData { raw_values_start: 32, raw_values_count: 1, token_types_start: 33, token_types_count: 0 }, // [32]
    SimpleHintData { raw_values_start: 33, raw_values_count: 1, token_types_start: 34, token_types_count: 0 }, // [33]
    SimpleHintData { raw_values_start: 34, raw_values_count: 1, token_types_start: 35, token_types_count: 0 }, // [34]
    SimpleHintData { raw_values_start: 35, raw_values_count: 1, token_types_start: 36, token_types_count: 0 }, // [35]
    SimpleHintData { raw_values_start: 36, raw_values_count: 1, token_types_start: 37, token_types_count: 0 }, // [36]
    SimpleHintData { raw_values_start: 37, raw_values_count: 1, token_types_start: 38, token_types_count: 0 }, // [37]
    SimpleHintData { raw_values_start: 38, raw_values_count: 1, token_types_start: 39, token_types_count: 0 }, // [38]
    SimpleHintData { raw_values_start: 39, raw_values_count: 2, token_types_start: 41, token_types_count: 0 }, // [39]
    SimpleHintData { raw_values_start: 41, raw_values_count: 1, token_types_start: 42, token_types_count: 0 }, // [40]
    SimpleHintData { raw_values_start: 42, raw_values_count: 1, token_types_start: 43, token_types_count: 0 }, // [41]
    SimpleHintData { raw_values_start: 43, raw_values_count: 1, token_types_start: 44, token_types_count: 0 }, // [42]
    SimpleHintData { raw_values_start: 44, raw_values_count: 2, token_types_start: 46, token_types_count: 0 }, // [43]
    SimpleHintData { raw_values_start: 46, raw_values_count: 1, token_types_start: 47, token_types_count: 0 }, // [44]
    SimpleHintData { raw_values_start: 47, raw_values_count: 1, token_types_start: 48, token_types_count: 0 }, // [45]
    SimpleHintData { raw_values_start: 48, raw_values_count: 1, token_types_start: 49, token_types_count: 0 }, // [46]
    SimpleHintData { raw_values_start: 49, raw_values_count: 1, token_types_start: 50, token_types_count: 0 }, // [47]
    SimpleHintData { raw_values_start: 50, raw_values_count: 1, token_types_start: 51, token_types_count: 0 }, // [48]
    SimpleHintData { raw_values_start: 51, raw_values_count: 1, token_types_start: 52, token_types_count: 0 }, // [49]
    SimpleHintData { raw_values_start: 52, raw_values_count: 1, token_types_start: 53, token_types_count: 0 }, // [50]
    SimpleHintData { raw_values_start: 53, raw_values_count: 1, token_types_start: 54, token_types_count: 0 }, // [51]
    SimpleHintData { raw_values_start: 54, raw_values_count: 0, token_types_start: 54, token_types_count: 1 }, // [52]
    SimpleHintData { raw_values_start: 55, raw_values_count: 1, token_types_start: 56, token_types_count: 0 }, // [53]
    SimpleHintData { raw_values_start: 56, raw_values_count: 1, token_types_start: 57, token_types_count: 0 }, // [54]
    SimpleHintData { raw_values_start: 57, raw_values_count: 1, token_types_start: 58, token_types_count: 0 }, // [55]
    SimpleHintData { raw_values_start: 58, raw_values_count: 1, token_types_start: 59, token_types_count: 0 }, // [56]
    SimpleHintData { raw_values_start: 59, raw_values_count: 1, token_types_start: 60, token_types_count: 0 }, // [57]
    SimpleHintData { raw_values_start: 60, raw_values_count: 1, token_types_start: 61, token_types_count: 0 }, // [58]
    SimpleHintData { raw_values_start: 61, raw_values_count: 12, token_types_start: 73, token_types_count: 2 }, // [59]
    SimpleHintData { raw_values_start: 75, raw_values_count: 3, token_types_start: 78, token_types_count: 0 }, // [60]
    SimpleHintData { raw_values_start: 78, raw_values_count: 2, token_types_start: 80, token_types_count: 0 }, // [61]
    SimpleHintData { raw_values_start: 80, raw_values_count: 1, token_types_start: 81, token_types_count: 0 }, // [62]
    SimpleHintData { raw_values_start: 81, raw_values_count: 1, token_types_start: 82, token_types_count: 0 }, // [63]
    SimpleHintData { raw_values_start: 82, raw_values_count: 2, token_types_start: 84, token_types_count: 0 }, // [64]
    SimpleHintData { raw_values_start: 84, raw_values_count: 6, token_types_start: 90, token_types_count: 0 }, // [65]
    SimpleHintData { raw_values_start: 90, raw_values_count: 3, token_types_start: 93, token_types_count: 0 }, // [66]
    SimpleHintData { raw_values_start: 93, raw_values_count: 1, token_types_start: 94, token_types_count: 0 }, // [67]
    SimpleHintData { raw_values_start: 94, raw_values_count: 2, token_types_start: 96, token_types_count: 0 }, // [68]
    SimpleHintData { raw_values_start: 96, raw_values_count: 1, token_types_start: 97, token_types_count: 0 }, // [69]
    SimpleHintData { raw_values_start: 97, raw_values_count: 21, token_types_start: 118, token_types_count: 0 }, // [70]
    SimpleHintData { raw_values_start: 118, raw_values_count: 1, token_types_start: 119, token_types_count: 0 }, // [71]
    SimpleHintData { raw_values_start: 119, raw_values_count: 1, token_types_start: 120, token_types_count: 0 }, // [72]
    SimpleHintData { raw_values_start: 120, raw_values_count: 1, token_types_start: 121, token_types_count: 0 }, // [73]
    SimpleHintData { raw_values_start: 121, raw_values_count: 1, token_types_start: 122, token_types_count: 0 }, // [74]
    SimpleHintData { raw_values_start: 122, raw_values_count: 1, token_types_start: 123, token_types_count: 0 }, // [75]
    SimpleHintData { raw_values_start: 123, raw_values_count: 1, token_types_start: 124, token_types_count: 0 }, // [76]
    SimpleHintData { raw_values_start: 124, raw_values_count: 1, token_types_start: 125, token_types_count: 0 }, // [77]
    SimpleHintData { raw_values_start: 125, raw_values_count: 1, token_types_start: 126, token_types_count: 0 }, // [78]
    SimpleHintData { raw_values_start: 126, raw_values_count: 1, token_types_start: 127, token_types_count: 0 }, // [79]
    SimpleHintData { raw_values_start: 127, raw_values_count: 1, token_types_start: 128, token_types_count: 0 }, // [80]
    SimpleHintData { raw_values_start: 128, raw_values_count: 1, token_types_start: 129, token_types_count: 0 }, // [81]
    SimpleHintData { raw_values_start: 129, raw_values_count: 10, token_types_start: 139, token_types_count: 0 }, // [82]
    SimpleHintData { raw_values_start: 139, raw_values_count: 1, token_types_start: 140, token_types_count: 0 }, // [83]
    SimpleHintData { raw_values_start: 140, raw_values_count: 1, token_types_start: 141, token_types_count: 0 }, // [84]
    SimpleHintData { raw_values_start: 141, raw_values_count: 1, token_types_start: 142, token_types_count: 0 }, // [85]
    SimpleHintData { raw_values_start: 142, raw_values_count: 1, token_types_start: 143, token_types_count: 0 }, // [86]
    SimpleHintData { raw_values_start: 143, raw_values_count: 1, token_types_start: 144, token_types_count: 0 }, // [87]
    SimpleHintData { raw_values_start: 144, raw_values_count: 1, token_types_start: 145, token_types_count: 0 }, // [88]
    SimpleHintData { raw_values_start: 145, raw_values_count: 1, token_types_start: 146, token_types_count: 0 }, // [89]
    SimpleHintData { raw_values_start: 146, raw_values_count: 1, token_types_start: 147, token_types_count: 0 }, // [90]
    SimpleHintData { raw_values_start: 147, raw_values_count: 1, token_types_start: 148, token_types_count: 0 }, // [91]
    SimpleHintData { raw_values_start: 148, raw_values_count: 1, token_types_start: 149, token_types_count: 0 }, // [92]
    SimpleHintData { raw_values_start: 149, raw_values_count: 1, token_types_start: 150, token_types_count: 0 }, // [93]
    SimpleHintData { raw_values_start: 150, raw_values_count: 1, token_types_start: 151, token_types_count: 0 }, // [94]
    SimpleHintData { raw_values_start: 151, raw_values_count: 2, token_types_start: 153, token_types_count: 0 }, // [95]
    SimpleHintData { raw_values_start: 153, raw_values_count: 1, token_types_start: 154, token_types_count: 0 }, // [96]
    SimpleHintData { raw_values_start: 154, raw_values_count: 1, token_types_start: 155, token_types_count: 0 }, // [97]
    SimpleHintData { raw_values_start: 155, raw_values_count: 5, token_types_start: 160, token_types_count: 0 }, // [98]
    SimpleHintData { raw_values_start: 160, raw_values_count: 1, token_types_start: 161, token_types_count: 0 }, // [99]
    SimpleHintData { raw_values_start: 161, raw_values_count: 1, token_types_start: 162, token_types_count: 0 }, // [100]
    SimpleHintData { raw_values_start: 162, raw_values_count: 1, token_types_start: 163, token_types_count: 0 }, // [101]
    SimpleHintData { raw_values_start: 163, raw_values_count: 1, token_types_start: 164, token_types_count: 0 }, // [102]
    SimpleHintData { raw_values_start: 164, raw_values_count: 16, token_types_start: 180, token_types_count: 1 }, // [103]
    SimpleHintData { raw_values_start: 181, raw_values_count: 2, token_types_start: 183, token_types_count: 0 }, // [104]
    SimpleHintData { raw_values_start: 183, raw_values_count: 5, token_types_start: 188, token_types_count: 1 }, // [105]
    SimpleHintData { raw_values_start: 189, raw_values_count: 1, token_types_start: 190, token_types_count: 0 }, // [106]
    SimpleHintData { raw_values_start: 190, raw_values_count: 1, token_types_start: 191, token_types_count: 0 }, // [107]
    SimpleHintData { raw_values_start: 191, raw_values_count: 1, token_types_start: 192, token_types_count: 0 }, // [108]
    SimpleHintData { raw_values_start: 192, raw_values_count: 2, token_types_start: 194, token_types_count: 0 }, // [109]
    SimpleHintData { raw_values_start: 194, raw_values_count: 1, token_types_start: 195, token_types_count: 0 }, // [110]
    SimpleHintData { raw_values_start: 195, raw_values_count: 1, token_types_start: 196, token_types_count: 0 }, // [111]
    SimpleHintData { raw_values_start: 196, raw_values_count: 1, token_types_start: 197, token_types_count: 0 }, // [112]
    SimpleHintData { raw_values_start: 197, raw_values_count: 3, token_types_start: 200, token_types_count: 0 }, // [113]
    SimpleHintData { raw_values_start: 200, raw_values_count: 4, token_types_start: 204, token_types_count: 0 }, // [114]
    SimpleHintData { raw_values_start: 204, raw_values_count: 1, token_types_start: 205, token_types_count: 0 }, // [115]
    SimpleHintData { raw_values_start: 205, raw_values_count: 1, token_types_start: 206, token_types_count: 0 }, // [116]
    SimpleHintData { raw_values_start: 206, raw_values_count: 1, token_types_start: 207, token_types_count: 0 }, // [117]
    SimpleHintData { raw_values_start: 207, raw_values_count: 1, token_types_start: 208, token_types_count: 0 }, // [118]
    SimpleHintData { raw_values_start: 208, raw_values_count: 1, token_types_start: 209, token_types_count: 0 }, // [119]
    SimpleHintData { raw_values_start: 209, raw_values_count: 1, token_types_start: 210, token_types_count: 0 }, // [120]
    SimpleHintData { raw_values_start: 210, raw_values_count: 1, token_types_start: 211, token_types_count: 0 }, // [121]
    SimpleHintData { raw_values_start: 211, raw_values_count: 1, token_types_start: 212, token_types_count: 0 }, // [122]
    SimpleHintData { raw_values_start: 212, raw_values_count: 1, token_types_start: 213, token_types_count: 0 }, // [123]
    SimpleHintData { raw_values_start: 213, raw_values_count: 1, token_types_start: 214, token_types_count: 0 }, // [124]
    SimpleHintData { raw_values_start: 214, raw_values_count: 1, token_types_start: 215, token_types_count: 0 }, // [125]
    SimpleHintData { raw_values_start: 215, raw_values_count: 2, token_types_start: 217, token_types_count: 0 }, // [126]
    SimpleHintData { raw_values_start: 217, raw_values_count: 1, token_types_start: 218, token_types_count: 0 }, // [127]
    SimpleHintData { raw_values_start: 218, raw_values_count: 1, token_types_start: 219, token_types_count: 0 }, // [128]
    SimpleHintData { raw_values_start: 219, raw_values_count: 3, token_types_start: 222, token_types_count: 0 }, // [129]
    SimpleHintData { raw_values_start: 222, raw_values_count: 1, token_types_start: 223, token_types_count: 0 }, // [130]
    SimpleHintData { raw_values_start: 223, raw_values_count: 3, token_types_start: 226, token_types_count: 0 }, // [131]
    SimpleHintData { raw_values_start: 226, raw_values_count: 2, token_types_start: 228, token_types_count: 0 }, // [132]
    SimpleHintData { raw_values_start: 228, raw_values_count: 1, token_types_start: 229, token_types_count: 0 }, // [133]
    SimpleHintData { raw_values_start: 229, raw_values_count: 1, token_types_start: 230, token_types_count: 1 }, // [134]
    SimpleHintData { raw_values_start: 231, raw_values_count: 1, token_types_start: 232, token_types_count: 0 }, // [135]
    SimpleHintData { raw_values_start: 232, raw_values_count: 1, token_types_start: 233, token_types_count: 0 }, // [136]
    SimpleHintData { raw_values_start: 233, raw_values_count: 2, token_types_start: 235, token_types_count: 0 }, // [137]
    SimpleHintData { raw_values_start: 235, raw_values_count: 0, token_types_start: 235, token_types_count: 1 }, // [138]
    SimpleHintData { raw_values_start: 236, raw_values_count: 1, token_types_start: 237, token_types_count: 0 }, // [139]
    SimpleHintData { raw_values_start: 237, raw_values_count: 4, token_types_start: 241, token_types_count: 0 }, // [140]
    SimpleHintData { raw_values_start: 241, raw_values_count: 1, token_types_start: 242, token_types_count: 0 }, // [141]
    SimpleHintData { raw_values_start: 242, raw_values_count: 1, token_types_start: 243, token_types_count: 0 }, // [142]
    SimpleHintData { raw_values_start: 243, raw_values_count: 1, token_types_start: 244, token_types_count: 0 }, // [143]
    SimpleHintData { raw_values_start: 244, raw_values_count: 1, token_types_start: 245, token_types_count: 0 }, // [144]
    SimpleHintData { raw_values_start: 245, raw_values_count: 1, token_types_start: 246, token_types_count: 0 }, // [145]
    SimpleHintData { raw_values_start: 246, raw_values_count: 1, token_types_start: 247, token_types_count: 0 }, // [146]
    SimpleHintData { raw_values_start: 247, raw_values_count: 1, token_types_start: 248, token_types_count: 0 }, // [147]
    SimpleHintData { raw_values_start: 248, raw_values_count: 2, token_types_start: 250, token_types_count: 0 }, // [148]
    SimpleHintData { raw_values_start: 250, raw_values_count: 1, token_types_start: 251, token_types_count: 0 }, // [149]
    SimpleHintData { raw_values_start: 251, raw_values_count: 1, token_types_start: 252, token_types_count: 0 }, // [150]
    SimpleHintData { raw_values_start: 252, raw_values_count: 1, token_types_start: 253, token_types_count: 0 }, // [151]
    SimpleHintData { raw_values_start: 253, raw_values_count: 1, token_types_start: 254, token_types_count: 0 }, // [152]
    SimpleHintData { raw_values_start: 254, raw_values_count: 3, token_types_start: 257, token_types_count: 0 }, // [153]
    SimpleHintData { raw_values_start: 257, raw_values_count: 1, token_types_start: 258, token_types_count: 0 }, // [154]
    SimpleHintData { raw_values_start: 258, raw_values_count: 1, token_types_start: 259, token_types_count: 0 }, // [155]
    SimpleHintData { raw_values_start: 259, raw_values_count: 0, token_types_start: 259, token_types_count: 1 }, // [156]
    SimpleHintData { raw_values_start: 260, raw_values_count: 1, token_types_start: 261, token_types_count: 0 }, // [157]
    SimpleHintData { raw_values_start: 261, raw_values_count: 2, token_types_start: 263, token_types_count: 0 }, // [158]
    SimpleHintData { raw_values_start: 263, raw_values_count: 1, token_types_start: 264, token_types_count: 0 }, // [159]
    SimpleHintData { raw_values_start: 264, raw_values_count: 1, token_types_start: 265, token_types_count: 0 }, // [160]
    SimpleHintData { raw_values_start: 265, raw_values_count: 1, token_types_start: 266, token_types_count: 0 }, // [161]
    SimpleHintData { raw_values_start: 266, raw_values_count: 1, token_types_start: 267, token_types_count: 0 }, // [162]
    SimpleHintData { raw_values_start: 267, raw_values_count: 1, token_types_start: 268, token_types_count: 0 }, // [163]
    SimpleHintData { raw_values_start: 268, raw_values_count: 1, token_types_start: 269, token_types_count: 0 }, // [164]
    SimpleHintData { raw_values_start: 269, raw_values_count: 1, token_types_start: 270, token_types_count: 0 }, // [165]
    SimpleHintData { raw_values_start: 270, raw_values_count: 1, token_types_start: 271, token_types_count: 0 }, // [166]
    SimpleHintData { raw_values_start: 271, raw_values_count: 1, token_types_start: 272, token_types_count: 0 }, // [167]
    SimpleHintData { raw_values_start: 272, raw_values_count: 1, token_types_start: 273, token_types_count: 0 }, // [168]
    SimpleHintData { raw_values_start: 273, raw_values_count: 18, token_types_start: 291, token_types_count: 0 }, // [169]
    SimpleHintData { raw_values_start: 291, raw_values_count: 1, token_types_start: 292, token_types_count: 0 }, // [170]
    SimpleHintData { raw_values_start: 292, raw_values_count: 1, token_types_start: 293, token_types_count: 0 }, // [171]
    SimpleHintData { raw_values_start: 293, raw_values_count: 1, token_types_start: 294, token_types_count: 0 }, // [172]
    SimpleHintData { raw_values_start: 294, raw_values_count: 11, token_types_start: 305, token_types_count: 0 }, // [173]
    SimpleHintData { raw_values_start: 305, raw_values_count: 4, token_types_start: 309, token_types_count: 0 }, // [174]
    SimpleHintData { raw_values_start: 309, raw_values_count: 3, token_types_start: 312, token_types_count: 0 }, // [175]
    SimpleHintData { raw_values_start: 312, raw_values_count: 1, token_types_start: 313, token_types_count: 0 }, // [176]
    SimpleHintData { raw_values_start: 313, raw_values_count: 1, token_types_start: 314, token_types_count: 0 }, // [177]
    SimpleHintData { raw_values_start: 314, raw_values_count: 1, token_types_start: 315, token_types_count: 0 }, // [178]
    SimpleHintData { raw_values_start: 315, raw_values_count: 1, token_types_start: 316, token_types_count: 0 }, // [179]
    SimpleHintData { raw_values_start: 316, raw_values_count: 1, token_types_start: 317, token_types_count: 0 }, // [180]
    SimpleHintData { raw_values_start: 317, raw_values_count: 1, token_types_start: 318, token_types_count: 0 }, // [181]
    SimpleHintData { raw_values_start: 318, raw_values_count: 1, token_types_start: 319, token_types_count: 0 }, // [182]
    SimpleHintData { raw_values_start: 319, raw_values_count: 1, token_types_start: 320, token_types_count: 0 }, // [183]
    SimpleHintData { raw_values_start: 320, raw_values_count: 1, token_types_start: 321, token_types_count: 0 }, // [184]
    SimpleHintData { raw_values_start: 321, raw_values_count: 1, token_types_start: 322, token_types_count: 0 }, // [185]
    SimpleHintData { raw_values_start: 322, raw_values_count: 1, token_types_start: 323, token_types_count: 0 }, // [186]
    SimpleHintData { raw_values_start: 323, raw_values_count: 1, token_types_start: 324, token_types_count: 0 }, // [187]
    SimpleHintData { raw_values_start: 324, raw_values_count: 1, token_types_start: 325, token_types_count: 0 }, // [188]
    SimpleHintData { raw_values_start: 325, raw_values_count: 1, token_types_start: 326, token_types_count: 0 }, // [189]
    SimpleHintData { raw_values_start: 326, raw_values_count: 1, token_types_start: 327, token_types_count: 0 }, // [190]
    SimpleHintData { raw_values_start: 327, raw_values_count: 1, token_types_start: 328, token_types_count: 0 }, // [191]
    SimpleHintData { raw_values_start: 328, raw_values_count: 2, token_types_start: 330, token_types_count: 0 }, // [192]
    SimpleHintData { raw_values_start: 330, raw_values_count: 1, token_types_start: 331, token_types_count: 0 }, // [193]
    SimpleHintData { raw_values_start: 331, raw_values_count: 1, token_types_start: 332, token_types_count: 0 }, // [194]
    SimpleHintData { raw_values_start: 332, raw_values_count: 1, token_types_start: 333, token_types_count: 0 }, // [195]
    SimpleHintData { raw_values_start: 333, raw_values_count: 1, token_types_start: 334, token_types_count: 0 }, // [196]
    SimpleHintData { raw_values_start: 334, raw_values_count: 1, token_types_start: 335, token_types_count: 0 }, // [197]
    SimpleHintData { raw_values_start: 335, raw_values_count: 1, token_types_start: 336, token_types_count: 0 }, // [198]
    SimpleHintData { raw_values_start: 336, raw_values_count: 1, token_types_start: 337, token_types_count: 0 }, // [199]
    SimpleHintData { raw_values_start: 337, raw_values_count: 1, token_types_start: 338, token_types_count: 0 }, // [200]
    SimpleHintData { raw_values_start: 338, raw_values_count: 1, token_types_start: 339, token_types_count: 0 }, // [201]
    SimpleHintData { raw_values_start: 339, raw_values_count: 7, token_types_start: 346, token_types_count: 0 }, // [202]
    SimpleHintData { raw_values_start: 346, raw_values_count: 2, token_types_start: 348, token_types_count: 0 }, // [203]
    SimpleHintData { raw_values_start: 348, raw_values_count: 2, token_types_start: 350, token_types_count: 0 }, // [204]
    SimpleHintData { raw_values_start: 350, raw_values_count: 2, token_types_start: 352, token_types_count: 0 }, // [205]
    SimpleHintData { raw_values_start: 352, raw_values_count: 4, token_types_start: 356, token_types_count: 0 }, // [206]
    SimpleHintData { raw_values_start: 356, raw_values_count: 2, token_types_start: 358, token_types_count: 0 }, // [207]
    SimpleHintData { raw_values_start: 358, raw_values_count: 1, token_types_start: 359, token_types_count: 0 }, // [208]
    SimpleHintData { raw_values_start: 359, raw_values_count: 13, token_types_start: 372, token_types_count: 1 }, // [209]
    SimpleHintData { raw_values_start: 373, raw_values_count: 3, token_types_start: 376, token_types_count: 0 }, // [210]
    SimpleHintData { raw_values_start: 376, raw_values_count: 7, token_types_start: 383, token_types_count: 2 }, // [211]
    SimpleHintData { raw_values_start: 385, raw_values_count: 1, token_types_start: 386, token_types_count: 0 }, // [212]
    SimpleHintData { raw_values_start: 386, raw_values_count: 1, token_types_start: 387, token_types_count: 0 }, // [213]
    SimpleHintData { raw_values_start: 387, raw_values_count: 2, token_types_start: 389, token_types_count: 0 }, // [214]
    SimpleHintData { raw_values_start: 389, raw_values_count: 1, token_types_start: 390, token_types_count: 0 }, // [215]
    SimpleHintData { raw_values_start: 390, raw_values_count: 2, token_types_start: 392, token_types_count: 0 }, // [216]
    SimpleHintData { raw_values_start: 392, raw_values_count: 1, token_types_start: 393, token_types_count: 0 }, // [217]
    SimpleHintData { raw_values_start: 393, raw_values_count: 2, token_types_start: 395, token_types_count: 0 }, // [218]
    SimpleHintData { raw_values_start: 395, raw_values_count: 1, token_types_start: 396, token_types_count: 0 }, // [219]
    SimpleHintData { raw_values_start: 396, raw_values_count: 1, token_types_start: 397, token_types_count: 0 }, // [220]
    SimpleHintData { raw_values_start: 397, raw_values_count: 21, token_types_start: 418, token_types_count: 0 }, // [221]
    SimpleHintData { raw_values_start: 418, raw_values_count: 20, token_types_start: 438, token_types_count: 0 }, // [222]
    SimpleHintData { raw_values_start: 438, raw_values_count: 1, token_types_start: 439, token_types_count: 0 }, // [223]
    SimpleHintData { raw_values_start: 439, raw_values_count: 1, token_types_start: 440, token_types_count: 0 }, // [224]
    SimpleHintData { raw_values_start: 440, raw_values_count: 1, token_types_start: 441, token_types_count: 0 }, // [225]
    SimpleHintData { raw_values_start: 441, raw_values_count: 1, token_types_start: 442, token_types_count: 0 }, // [226]
    SimpleHintData { raw_values_start: 442, raw_values_count: 3, token_types_start: 445, token_types_count: 0 }, // [227]
    SimpleHintData { raw_values_start: 445, raw_values_count: 1, token_types_start: 446, token_types_count: 0 }, // [228]
    SimpleHintData { raw_values_start: 446, raw_values_count: 1, token_types_start: 447, token_types_count: 0 }, // [229]
    SimpleHintData { raw_values_start: 447, raw_values_count: 1, token_types_start: 448, token_types_count: 0 }, // [230]
    SimpleHintData { raw_values_start: 448, raw_values_count: 2, token_types_start: 450, token_types_count: 0 }, // [231]
    SimpleHintData { raw_values_start: 450, raw_values_count: 7, token_types_start: 457, token_types_count: 1 }, // [232]
    SimpleHintData { raw_values_start: 458, raw_values_count: 6, token_types_start: 464, token_types_count: 1 }, // [233]
    SimpleHintData { raw_values_start: 465, raw_values_count: 5, token_types_start: 470, token_types_count: 1 }, // [234]
    SimpleHintData { raw_values_start: 471, raw_values_count: 1, token_types_start: 472, token_types_count: 0 }, // [235]
    SimpleHintData { raw_values_start: 472, raw_values_count: 2, token_types_start: 474, token_types_count: 0 }, // [236]
    SimpleHintData { raw_values_start: 474, raw_values_count: 1, token_types_start: 475, token_types_count: 0 }, // [237]
    SimpleHintData { raw_values_start: 475, raw_values_count: 1, token_types_start: 476, token_types_count: 0 }, // [238]
    SimpleHintData { raw_values_start: 476, raw_values_count: 12, token_types_start: 488, token_types_count: 0 }, // [239]
    SimpleHintData { raw_values_start: 488, raw_values_count: 1, token_types_start: 489, token_types_count: 0 }, // [240]
    SimpleHintData { raw_values_start: 489, raw_values_count: 1, token_types_start: 490, token_types_count: 0 }, // [241]
    SimpleHintData { raw_values_start: 490, raw_values_count: 1, token_types_start: 491, token_types_count: 0 }, // [242]
    SimpleHintData { raw_values_start: 491, raw_values_count: 1, token_types_start: 492, token_types_count: 0 }, // [243]
    SimpleHintData { raw_values_start: 492, raw_values_count: 3, token_types_start: 495, token_types_count: 0 }, // [244]
    SimpleHintData { raw_values_start: 495, raw_values_count: 1, token_types_start: 496, token_types_count: 0 }, // [245]
    SimpleHintData { raw_values_start: 496, raw_values_count: 1, token_types_start: 497, token_types_count: 0 }, // [246]
    SimpleHintData { raw_values_start: 497, raw_values_count: 3, token_types_start: 500, token_types_count: 0 }, // [247]
    SimpleHintData { raw_values_start: 500, raw_values_count: 2, token_types_start: 502, token_types_count: 0 }, // [248]
    SimpleHintData { raw_values_start: 502, raw_values_count: 1, token_types_start: 503, token_types_count: 0 }, // [249]
    SimpleHintData { raw_values_start: 503, raw_values_count: 2, token_types_start: 505, token_types_count: 0 }, // [250]
    SimpleHintData { raw_values_start: 505, raw_values_count: 1, token_types_start: 506, token_types_count: 0 }, // [251]
    SimpleHintData { raw_values_start: 506, raw_values_count: 2, token_types_start: 508, token_types_count: 0 }, // [252]
    SimpleHintData { raw_values_start: 508, raw_values_count: 1, token_types_start: 509, token_types_count: 0 }, // [253]
    SimpleHintData { raw_values_start: 509, raw_values_count: 2, token_types_start: 511, token_types_count: 0 }, // [254]
    SimpleHintData { raw_values_start: 511, raw_values_count: 0, token_types_start: 511, token_types_count: 2 }, // [255]
    SimpleHintData { raw_values_start: 513, raw_values_count: 0, token_types_start: 513, token_types_count: 1 }, // [256]
    SimpleHintData { raw_values_start: 514, raw_values_count: 22, token_types_start: 536, token_types_count: 0 }, // [257]
    SimpleHintData { raw_values_start: 536, raw_values_count: 0, token_types_start: 536, token_types_count: 1 }, // [258]
    SimpleHintData { raw_values_start: 537, raw_values_count: 1, token_types_start: 538, token_types_count: 0 }, // [259]
    SimpleHintData { raw_values_start: 538, raw_values_count: 1, token_types_start: 539, token_types_count: 0 }, // [260]
    SimpleHintData { raw_values_start: 539, raw_values_count: 1, token_types_start: 540, token_types_count: 0 }, // [261]
    SimpleHintData { raw_values_start: 540, raw_values_count: 1, token_types_start: 541, token_types_count: 0 }, // [262]
    SimpleHintData { raw_values_start: 541, raw_values_count: 1, token_types_start: 542, token_types_count: 0 }, // [263]
    SimpleHintData { raw_values_start: 542, raw_values_count: 6, token_types_start: 548, token_types_count: 0 }, // [264]
    SimpleHintData { raw_values_start: 548, raw_values_count: 1, token_types_start: 549, token_types_count: 0 }, // [265]
    SimpleHintData { raw_values_start: 549, raw_values_count: 1, token_types_start: 550, token_types_count: 0 }, // [266]
    SimpleHintData { raw_values_start: 550, raw_values_count: 5, token_types_start: 555, token_types_count: 0 }, // [267]
    SimpleHintData { raw_values_start: 555, raw_values_count: 1, token_types_start: 556, token_types_count: 0 }, // [268]
    SimpleHintData { raw_values_start: 556, raw_values_count: 1, token_types_start: 557, token_types_count: 0 }, // [269]
    SimpleHintData { raw_values_start: 557, raw_values_count: 1, token_types_start: 558, token_types_count: 0 }, // [270]
    SimpleHintData { raw_values_start: 558, raw_values_count: 1, token_types_start: 559, token_types_count: 0 }, // [271]
    SimpleHintData { raw_values_start: 559, raw_values_count: 2, token_types_start: 561, token_types_count: 0 }, // [272]
    SimpleHintData { raw_values_start: 561, raw_values_count: 1, token_types_start: 562, token_types_count: 0 }, // [273]
    SimpleHintData { raw_values_start: 562, raw_values_count: 1, token_types_start: 563, token_types_count: 0 }, // [274]
    SimpleHintData { raw_values_start: 563, raw_values_count: 1, token_types_start: 564, token_types_count: 0 }, // [275]
    SimpleHintData { raw_values_start: 564, raw_values_count: 1, token_types_start: 565, token_types_count: 0 }, // [276]
    SimpleHintData { raw_values_start: 565, raw_values_count: 5, token_types_start: 570, token_types_count: 0 }, // [277]
    SimpleHintData { raw_values_start: 570, raw_values_count: 1, token_types_start: 571, token_types_count: 0 }, // [278]
    SimpleHintData { raw_values_start: 571, raw_values_count: 1, token_types_start: 572, token_types_count: 0 }, // [279]
    SimpleHintData { raw_values_start: 572, raw_values_count: 1, token_types_start: 573, token_types_count: 0 }, // [280]
    SimpleHintData { raw_values_start: 573, raw_values_count: 6, token_types_start: 579, token_types_count: 0 }, // [281]
    SimpleHintData { raw_values_start: 579, raw_values_count: 1, token_types_start: 580, token_types_count: 0 }, // [282]
    SimpleHintData { raw_values_start: 580, raw_values_count: 1, token_types_start: 581, token_types_count: 0 }, // [283]
    SimpleHintData { raw_values_start: 581, raw_values_count: 1, token_types_start: 582, token_types_count: 0 }, // [284]
    SimpleHintData { raw_values_start: 582, raw_values_count: 1, token_types_start: 583, token_types_count: 0 }, // [285]
    SimpleHintData { raw_values_start: 583, raw_values_count: 1, token_types_start: 584, token_types_count: 0 }, // [286]
    SimpleHintData { raw_values_start: 584, raw_values_count: 1, token_types_start: 585, token_types_count: 0 }, // [287]
    SimpleHintData { raw_values_start: 585, raw_values_count: 1, token_types_start: 586, token_types_count: 0 }, // [288]
    SimpleHintData { raw_values_start: 586, raw_values_count: 1, token_types_start: 587, token_types_count: 0 }, // [289]
    SimpleHintData { raw_values_start: 587, raw_values_count: 1, token_types_start: 588, token_types_count: 0 }, // [290]
    SimpleHintData { raw_values_start: 588, raw_values_count: 1, token_types_start: 589, token_types_count: 0 }, // [291]
    SimpleHintData { raw_values_start: 589, raw_values_count: 2, token_types_start: 591, token_types_count: 0 }, // [292]
    SimpleHintData { raw_values_start: 591, raw_values_count: 1, token_types_start: 592, token_types_count: 0 }, // [293]
    SimpleHintData { raw_values_start: 592, raw_values_count: 3, token_types_start: 595, token_types_count: 0 }, // [294]
    SimpleHintData { raw_values_start: 595, raw_values_count: 1, token_types_start: 596, token_types_count: 0 }, // [295]
    SimpleHintData { raw_values_start: 596, raw_values_count: 1, token_types_start: 597, token_types_count: 0 }, // [296]
    SimpleHintData { raw_values_start: 597, raw_values_count: 1, token_types_start: 598, token_types_count: 0 }, // [297]
    SimpleHintData { raw_values_start: 598, raw_values_count: 1, token_types_start: 599, token_types_count: 0 }, // [298]
    SimpleHintData { raw_values_start: 599, raw_values_count: 1, token_types_start: 600, token_types_count: 0 }, // [299]
    SimpleHintData { raw_values_start: 600, raw_values_count: 1, token_types_start: 601, token_types_count: 0 }, // [300]
    SimpleHintData { raw_values_start: 601, raw_values_count: 1, token_types_start: 602, token_types_count: 0 }, // [301]
    SimpleHintData { raw_values_start: 602, raw_values_count: 1, token_types_start: 603, token_types_count: 0 }, // [302]
    SimpleHintData { raw_values_start: 603, raw_values_count: 1, token_types_start: 604, token_types_count: 0 }, // [303]
    SimpleHintData { raw_values_start: 604, raw_values_count: 1, token_types_start: 605, token_types_count: 0 }, // [304]
    SimpleHintData { raw_values_start: 605, raw_values_count: 1, token_types_start: 606, token_types_count: 0 }, // [305]
    SimpleHintData { raw_values_start: 606, raw_values_count: 1, token_types_start: 607, token_types_count: 0 }, // [306]
    SimpleHintData { raw_values_start: 607, raw_values_count: 1, token_types_start: 608, token_types_count: 0 }, // [307]
    SimpleHintData { raw_values_start: 608, raw_values_count: 1, token_types_start: 609, token_types_count: 0 }, // [308]
    SimpleHintData { raw_values_start: 609, raw_values_count: 1, token_types_start: 610, token_types_count: 0 }, // [309]
    SimpleHintData { raw_values_start: 610, raw_values_count: 2, token_types_start: 612, token_types_count: 0 }, // [310]
    SimpleHintData { raw_values_start: 612, raw_values_count: 1, token_types_start: 613, token_types_count: 0 }, // [311]
    SimpleHintData { raw_values_start: 613, raw_values_count: 1, token_types_start: 614, token_types_count: 0 }, // [312]
    SimpleHintData { raw_values_start: 614, raw_values_count: 2, token_types_start: 616, token_types_count: 0 }, // [313]
    SimpleHintData { raw_values_start: 616, raw_values_count: 1, token_types_start: 617, token_types_count: 0 }, // [314]
    SimpleHintData { raw_values_start: 617, raw_values_count: 1, token_types_start: 618, token_types_count: 0 }, // [315]
    SimpleHintData { raw_values_start: 618, raw_values_count: 1, token_types_start: 619, token_types_count: 0 }, // [316]
    SimpleHintData { raw_values_start: 619, raw_values_count: 1, token_types_start: 620, token_types_count: 0 }, // [317]
    SimpleHintData { raw_values_start: 620, raw_values_count: 1, token_types_start: 621, token_types_count: 0 }, // [318]
    SimpleHintData { raw_values_start: 621, raw_values_count: 1, token_types_start: 622, token_types_count: 0 }, // [319]
    SimpleHintData { raw_values_start: 622, raw_values_count: 1, token_types_start: 623, token_types_count: 0 }, // [320]
    SimpleHintData { raw_values_start: 623, raw_values_count: 1, token_types_start: 624, token_types_count: 0 }, // [321]
    SimpleHintData { raw_values_start: 624, raw_values_count: 1, token_types_start: 625, token_types_count: 0 }, // [322]
    SimpleHintData { raw_values_start: 625, raw_values_count: 1, token_types_start: 626, token_types_count: 0 }, // [323]
    SimpleHintData { raw_values_start: 626, raw_values_count: 1, token_types_start: 627, token_types_count: 0 }, // [324]
    SimpleHintData { raw_values_start: 627, raw_values_count: 1, token_types_start: 628, token_types_count: 0 }, // [325]
    SimpleHintData { raw_values_start: 628, raw_values_count: 2, token_types_start: 630, token_types_count: 0 }, // [326]
    SimpleHintData { raw_values_start: 630, raw_values_count: 1, token_types_start: 631, token_types_count: 0 }, // [327]
    SimpleHintData { raw_values_start: 631, raw_values_count: 1, token_types_start: 632, token_types_count: 0 }, // [328]
    SimpleHintData { raw_values_start: 632, raw_values_count: 1, token_types_start: 633, token_types_count: 0 }, // [329]
    SimpleHintData { raw_values_start: 633, raw_values_count: 1, token_types_start: 634, token_types_count: 0 }, // [330]
    SimpleHintData { raw_values_start: 634, raw_values_count: 1, token_types_start: 635, token_types_count: 0 }, // [331]
    SimpleHintData { raw_values_start: 635, raw_values_count: 1, token_types_start: 636, token_types_count: 0 }, // [332]
    SimpleHintData { raw_values_start: 636, raw_values_count: 4, token_types_start: 640, token_types_count: 0 }, // [333]
    SimpleHintData { raw_values_start: 640, raw_values_count: 2, token_types_start: 642, token_types_count: 0 }, // [334]
    SimpleHintData { raw_values_start: 642, raw_values_count: 1, token_types_start: 643, token_types_count: 0 }, // [335]
    SimpleHintData { raw_values_start: 643, raw_values_count: 1, token_types_start: 644, token_types_count: 0 }, // [336]
    SimpleHintData { raw_values_start: 644, raw_values_count: 1, token_types_start: 645, token_types_count: 0 }, // [337]
    SimpleHintData { raw_values_start: 645, raw_values_count: 1, token_types_start: 646, token_types_count: 0 }, // [338]
    SimpleHintData { raw_values_start: 646, raw_values_count: 1, token_types_start: 647, token_types_count: 0 }, // [339]
    SimpleHintData { raw_values_start: 647, raw_values_count: 1, token_types_start: 648, token_types_count: 0 }, // [340]
    SimpleHintData { raw_values_start: 648, raw_values_count: 1, token_types_start: 649, token_types_count: 0 }, // [341]
    SimpleHintData { raw_values_start: 649, raw_values_count: 2, token_types_start: 651, token_types_count: 0 }, // [342]
    SimpleHintData { raw_values_start: 651, raw_values_count: 7, token_types_start: 658, token_types_count: 0 }, // [343]
    SimpleHintData { raw_values_start: 658, raw_values_count: 1, token_types_start: 659, token_types_count: 0 }, // [344]
    SimpleHintData { raw_values_start: 659, raw_values_count: 1, token_types_start: 660, token_types_count: 0 }, // [345]
    SimpleHintData { raw_values_start: 660, raw_values_count: 1, token_types_start: 661, token_types_count: 0 }, // [346]
    SimpleHintData { raw_values_start: 661, raw_values_count: 1, token_types_start: 662, token_types_count: 0 }, // [347]
    SimpleHintData { raw_values_start: 662, raw_values_count: 1, token_types_start: 663, token_types_count: 0 }, // [348]
    SimpleHintData { raw_values_start: 663, raw_values_count: 1, token_types_start: 664, token_types_count: 0 }, // [349]
    SimpleHintData { raw_values_start: 664, raw_values_count: 1, token_types_start: 665, token_types_count: 0 }, // [350]
    SimpleHintData { raw_values_start: 665, raw_values_count: 1, token_types_start: 666, token_types_count: 0 }, // [351]
    SimpleHintData { raw_values_start: 666, raw_values_count: 1, token_types_start: 667, token_types_count: 0 }, // [352]
    SimpleHintData { raw_values_start: 667, raw_values_count: 1, token_types_start: 668, token_types_count: 1 }, // [353]
    SimpleHintData { raw_values_start: 669, raw_values_count: 1, token_types_start: 670, token_types_count: 0 }, // [354]
    SimpleHintData { raw_values_start: 670, raw_values_count: 1, token_types_start: 671, token_types_count: 0 }, // [355]
    SimpleHintData { raw_values_start: 671, raw_values_count: 1, token_types_start: 672, token_types_count: 0 }, // [356]
    SimpleHintData { raw_values_start: 672, raw_values_count: 1, token_types_start: 673, token_types_count: 0 }, // [357]
    SimpleHintData { raw_values_start: 673, raw_values_count: 1, token_types_start: 674, token_types_count: 0 }, // [358]
    SimpleHintData { raw_values_start: 674, raw_values_count: 1, token_types_start: 675, token_types_count: 0 }, // [359]
    SimpleHintData { raw_values_start: 675, raw_values_count: 1, token_types_start: 676, token_types_count: 0 }, // [360]
    SimpleHintData { raw_values_start: 676, raw_values_count: 1, token_types_start: 677, token_types_count: 0 }, // [361]
    SimpleHintData { raw_values_start: 677, raw_values_count: 1, token_types_start: 678, token_types_count: 0 }, // [362]
    SimpleHintData { raw_values_start: 678, raw_values_count: 1, token_types_start: 679, token_types_count: 0 }, // [363]
    SimpleHintData { raw_values_start: 679, raw_values_count: 1, token_types_start: 680, token_types_count: 0 }, // [364]
    SimpleHintData { raw_values_start: 680, raw_values_count: 1, token_types_start: 681, token_types_count: 0 }, // [365]
    SimpleHintData { raw_values_start: 681, raw_values_count: 2, token_types_start: 683, token_types_count: 0 }, // [366]
    SimpleHintData { raw_values_start: 683, raw_values_count: 1, token_types_start: 684, token_types_count: 0 }, // [367]
    SimpleHintData { raw_values_start: 684, raw_values_count: 1, token_types_start: 685, token_types_count: 0 }, // [368]
    SimpleHintData { raw_values_start: 685, raw_values_count: 2, token_types_start: 687, token_types_count: 0 }, // [369]
    SimpleHintData { raw_values_start: 687, raw_values_count: 1, token_types_start: 688, token_types_count: 0 }, // [370]
    SimpleHintData { raw_values_start: 688, raw_values_count: 1, token_types_start: 689, token_types_count: 0 }, // [371]
    SimpleHintData { raw_values_start: 689, raw_values_count: 3, token_types_start: 692, token_types_count: 0 }, // [372]
    SimpleHintData { raw_values_start: 692, raw_values_count: 1, token_types_start: 693, token_types_count: 0 }, // [373]
    SimpleHintData { raw_values_start: 693, raw_values_count: 1, token_types_start: 694, token_types_count: 0 }, // [374]
    SimpleHintData { raw_values_start: 694, raw_values_count: 1, token_types_start: 695, token_types_count: 0 }, // [375]
    SimpleHintData { raw_values_start: 695, raw_values_count: 4, token_types_start: 699, token_types_count: 0 }, // [376]
    SimpleHintData { raw_values_start: 699, raw_values_count: 1, token_types_start: 700, token_types_count: 0 }, // [377]
    SimpleHintData { raw_values_start: 700, raw_values_count: 1, token_types_start: 701, token_types_count: 0 }, // [378]
    SimpleHintData { raw_values_start: 701, raw_values_count: 1, token_types_start: 702, token_types_count: 0 }, // [379]
    SimpleHintData { raw_values_start: 702, raw_values_count: 1, token_types_start: 703, token_types_count: 0 }, // [380]
    SimpleHintData { raw_values_start: 703, raw_values_count: 1, token_types_start: 704, token_types_count: 0 }, // [381]
    SimpleHintData { raw_values_start: 704, raw_values_count: 1, token_types_start: 705, token_types_count: 0 }, // [382]
    SimpleHintData { raw_values_start: 705, raw_values_count: 2, token_types_start: 707, token_types_count: 0 }, // [383]
    SimpleHintData { raw_values_start: 707, raw_values_count: 1, token_types_start: 708, token_types_count: 0 }, // [384]
    SimpleHintData { raw_values_start: 708, raw_values_count: 1, token_types_start: 709, token_types_count: 0 }, // [385]
    SimpleHintData { raw_values_start: 709, raw_values_count: 1, token_types_start: 710, token_types_count: 0 }, // [386]
    SimpleHintData { raw_values_start: 710, raw_values_count: 1, token_types_start: 711, token_types_count: 0 }, // [387]
    SimpleHintData { raw_values_start: 711, raw_values_count: 2, token_types_start: 713, token_types_count: 0 }, // [388]
    SimpleHintData { raw_values_start: 713, raw_values_count: 8, token_types_start: 721, token_types_count: 0 }, // [389]
    SimpleHintData { raw_values_start: 721, raw_values_count: 2, token_types_start: 723, token_types_count: 0 }, // [390]
    SimpleHintData { raw_values_start: 723, raw_values_count: 1, token_types_start: 724, token_types_count: 0 }, // [391]
    SimpleHintData { raw_values_start: 724, raw_values_count: 1, token_types_start: 725, token_types_count: 0 }, // [392]
    SimpleHintData { raw_values_start: 725, raw_values_count: 1, token_types_start: 726, token_types_count: 0 }, // [393]
    SimpleHintData { raw_values_start: 726, raw_values_count: 2, token_types_start: 728, token_types_count: 0 }, // [394]
    SimpleHintData { raw_values_start: 728, raw_values_count: 1, token_types_start: 729, token_types_count: 0 }, // [395]
    SimpleHintData { raw_values_start: 729, raw_values_count: 1, token_types_start: 730, token_types_count: 0 }, // [396]
    SimpleHintData { raw_values_start: 730, raw_values_count: 3, token_types_start: 733, token_types_count: 0 }, // [397]
    SimpleHintData { raw_values_start: 733, raw_values_count: 1, token_types_start: 734, token_types_count: 0 }, // [398]
    SimpleHintData { raw_values_start: 734, raw_values_count: 1, token_types_start: 735, token_types_count: 0 }, // [399]
    SimpleHintData { raw_values_start: 735, raw_values_count: 1, token_types_start: 736, token_types_count: 0 }, // [400]
    SimpleHintData { raw_values_start: 736, raw_values_count: 1, token_types_start: 737, token_types_count: 0 }, // [401]
    SimpleHintData { raw_values_start: 737, raw_values_count: 1, token_types_start: 738, token_types_count: 0 }, // [402]
    SimpleHintData { raw_values_start: 738, raw_values_count: 1, token_types_start: 739, token_types_count: 0 }, // [403]
    SimpleHintData { raw_values_start: 739, raw_values_count: 1, token_types_start: 740, token_types_count: 0 }, // [404]
    SimpleHintData { raw_values_start: 740, raw_values_count: 1, token_types_start: 741, token_types_count: 0 }, // [405]
    SimpleHintData { raw_values_start: 741, raw_values_count: 1, token_types_start: 742, token_types_count: 0 }, // [406]
    SimpleHintData { raw_values_start: 742, raw_values_count: 2, token_types_start: 744, token_types_count: 0 }, // [407]
    SimpleHintData { raw_values_start: 744, raw_values_count: 3, token_types_start: 747, token_types_count: 0 }, // [408]
    SimpleHintData { raw_values_start: 747, raw_values_count: 1, token_types_start: 748, token_types_count: 0 }, // [409]
    SimpleHintData { raw_values_start: 748, raw_values_count: 2, token_types_start: 750, token_types_count: 0 }, // [410]
    SimpleHintData { raw_values_start: 750, raw_values_count: 1, token_types_start: 751, token_types_count: 0 }, // [411]
    SimpleHintData { raw_values_start: 751, raw_values_count: 1, token_types_start: 752, token_types_count: 0 }, // [412]
    SimpleHintData { raw_values_start: 752, raw_values_count: 1, token_types_start: 753, token_types_count: 0 }, // [413]
    SimpleHintData { raw_values_start: 753, raw_values_count: 1, token_types_start: 754, token_types_count: 0 }, // [414]
    SimpleHintData { raw_values_start: 754, raw_values_count: 1, token_types_start: 755, token_types_count: 0 }, // [415]
    SimpleHintData { raw_values_start: 755, raw_values_count: 1, token_types_start: 756, token_types_count: 0 }, // [416]
    SimpleHintData { raw_values_start: 756, raw_values_count: 1, token_types_start: 757, token_types_count: 0 }, // [417]
    SimpleHintData { raw_values_start: 757, raw_values_count: 1, token_types_start: 758, token_types_count: 0 }, // [418]
    SimpleHintData { raw_values_start: 758, raw_values_count: 1, token_types_start: 759, token_types_count: 0 }, // [419]
    SimpleHintData { raw_values_start: 759, raw_values_count: 1, token_types_start: 760, token_types_count: 0 }, // [420]
    SimpleHintData { raw_values_start: 760, raw_values_count: 7, token_types_start: 767, token_types_count: 0 }, // [421]
    SimpleHintData { raw_values_start: 767, raw_values_count: 2, token_types_start: 769, token_types_count: 0 }, // [422]
];

pub static SIMPLE_HINT_INDICES: &[u32] = &[
    1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 3, 4,
    0, 5, 0, 0, 0, 0, 6, 6, 7, 8, 8, 9, 10, 0, 0, 0,
    9, 10, 0, 0, 8, 8, 0, 10, 2, 0, 0, 0, 10, 0, 0, 2,
    0, 0, 0, 0, 0, 11, 0, 12, 13, 14, 0, 0, 15, 16, 0, 17,
    17, 0, 4, 18, 0, 0, 0, 0, 0, 0, 0, 19, 19, 0, 0, 0,
    20, 0, 0, 0, 0, 0, 0, 0, 17, 17, 0, 2, 0, 0, 0, 0,
    0, 21, 21, 22, 2, 3, 0, 0, 23, 0, 0, 17, 24, 19, 25, 0,
    0, 0, 0, 26, 0, 0, 27, 0, 0, 0, 28, 29, 0, 0, 30, 8,
    8, 23, 11, 0, 0, 31, 0, 32, 33, 0, 34, 0, 0, 0, 0, 0,
    2, 0, 0, 0, 35, 35, 33, 0, 0, 32, 0, 0, 36, 0, 37, 0,
    0, 38, 0, 0, 0, 0, 0, 9, 0, 0, 0, 35, 35, 0, 0, 0,
    37, 0, 0, 0, 39, 40, 41, 0, 42, 42, 0, 0, 18, 18, 0, 0,
    4, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 43, 44, 45,
    46, 47, 48, 49, 0, 0, 50, 0, 0, 0, 11, 0, 11, 51, 52, 0,
    0, 0, 0, 8, 53, 0, 0, 0, 54, 16, 0, 0, 0, 0, 0, 16,
    0, 0, 0, 0, 0, 53, 53, 0, 0, 0, 0, 55, 55, 56, 57, 0,
    0, 0, 0, 58, 0, 59, 0, 0, 0, 46, 0, 56, 0, 60, 61, 61,
    62, 48, 0, 11, 55, 63, 63, 64, 50, 13, 0, 53, 53, 32, 57, 0,
    65, 63, 63, 13, 0, 46, 46, 56, 57, 0, 66, 62, 67, 48, 48, 0,
    55, 55, 56, 57, 0, 63, 63, 56, 57, 0, 13, 0, 53, 53, 56, 0,
    68, 69, 69, 6, 5, 70, 55, 55, 71, 72, 69, 69, 73, 0, 0, 18,
    18, 0, 4, 4, 0, 69, 69, 74, 0, 0, 0, 58, 0, 19, 19, 0,
    11, 0, 0, 0, 75, 0, 76, 76, 0, 77, 78, 78, 78, 0, 79, 79,
    11, 0, 0, 0, 58, 0, 80, 80, 81, 82, 83, 84, 85, 86, 87, 77,
    88, 89, 90, 91, 44, 0, 0, 16, 0, 92, 44, 93, 0, 94, 44, 0,
    0, 95, 90, 70, 90, 91, 11, 70, 94, 94, 95, 50, 50, 50, 96, 97,
    73, 0, 0, 98, 0, 59, 98, 99, 0, 0, 0, 70, 70, 59, 12, 50,
    0, 0, 100, 101, 102, 103, 82, 88, 104, 105, 106, 0, 77, 90, 90, 90,
    88, 91, 91, 91, 89, 104, 80, 107, 108, 109, 110, 111, 112, 11, 59, 59,
    11, 0, 0, 0, 11, 113, 54, 11, 0, 0, 0, 11, 50, 11, 114, 115,
    116, 117, 117, 117, 0, 118, 118, 119, 120, 119, 120, 120, 117, 117, 0, 118,
    118, 119, 120, 119, 120, 120, 105, 12, 103, 117, 121, 122, 3, 123, 0, 124,
    0, 0, 0, 52, 0, 0, 45, 0, 92, 92, 92, 0, 0, 59, 0, 98,
    0, 125, 125, 0, 0, 126, 71, 72, 0, 0, 11, 0, 127, 127, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 128, 0, 0, 0, 0, 0, 0, 70, 129,
    71, 71, 72, 128, 79, 79, 11, 0, 0, 0, 58, 0, 0, 56, 0, 0,
    45, 0, 0, 0, 0, 0, 130, 12, 128, 128, 52, 128, 128, 128, 31, 131,
    131, 132, 132, 32, 33, 0, 56, 56, 0, 133, 133, 134, 52, 72, 0, 135,
    136, 105, 58, 91, 90, 91, 90, 137, 138, 133, 0, 0, 0, 88, 88, 88,
    139, 0, 140, 139, 0, 141, 125, 142, 143, 17, 17, 121, 11, 70, 50, 70,
    79, 0, 0, 0, 34, 0, 0, 0, 0, 0, 0, 0, 0, 11, 144, 144,
    0, 50, 50, 0, 17, 17, 0, 57, 0, 17, 17, 107, 0, 34, 57, 0,
    11, 0, 0, 70, 50, 17, 17, 107, 145, 0, 57, 0, 31, 0, 11, 11,
    0, 0, 17, 17, 17, 107, 0, 57, 0, 0, 0, 11, 0, 0, 0, 58,
    59, 59, 44, 52, 52, 50, 114, 17, 17, 4, 57, 0, 17, 17, 2, 57,
    0, 0, 0, 0, 54, 16, 146, 146, 79, 16, 0, 0, 0, 0, 0, 16,
    0, 0, 0, 0, 0, 17, 17, 0, 0, 0, 0, 17, 17, 107, 32, 57,
    0, 11, 0, 0, 0, 0, 147, 147, 0, 148, 148, 149, 150, 74, 128, 79,
    79, 11, 0, 0, 0, 58, 0, 50, 50, 114, 11, 114, 79, 79, 151, 6,
    17, 17, 0, 0, 0, 0, 48, 0, 0, 152, 153, 153, 25, 24, 29, 29,
    152, 0, 0, 107, 31, 107, 31, 31, 0, 0, 0, 0, 0, 154, 50, 0,
    0, 154, 50, 0, 14, 14, 0, 0, 71, 71, 0, 0, 0, 0, 0, 0,
    155, 0, 0, 0, 144, 144, 0, 0, 154, 0, 118, 118, 11, 0, 19, 19,
    0, 156, 11, 17, 17, 18, 0, 17, 17, 107, 33, 57, 0, 11, 50, 114,
    11, 114, 79, 139, 157, 157, 158, 158, 157, 159, 11, 0, 160, 161, 162, 163,
    164, 165, 166, 167, 168, 6, 0, 0, 0, 0, 70, 169, 94, 170, 154, 171,
    172, 172, 11, 11, 0, 0, 173, 0, 174, 175, 175, 171, 176, 177, 52, 99,
    173, 178, 179, 180, 127, 127, 127, 181, 182, 183, 24, 24, 24, 14, 76, 184,
    185, 186, 187, 187, 187, 0, 0, 188, 189, 190, 85, 45, 191, 192, 193, 116,
    55, 55, 121, 11, 70, 50, 70, 192, 55, 55, 0, 57, 0, 192, 55, 55,
    34, 57, 0, 55, 55, 0, 57, 0, 192, 55, 55, 55, 0, 57, 0, 55,
    55, 4, 57, 0, 55, 55, 2, 57, 0, 192, 55, 55, 0, 0, 55, 55,
    0, 32, 57, 0, 0, 192, 55, 55, 0, 57, 0, 55, 55, 5, 57, 0,
    192, 55, 55, 18, 57, 0, 55, 55, 33, 57, 0, 192, 119, 119, 0, 119,
    194, 11, 0, 0, 11, 195, 196, 120, 58, 197, 198, 199, 150, 75, 19, 200,
    201, 201, 201, 202, 114, 25, 29, 24, 0, 0, 0, 0, 0, 203, 103, 103,
    0, 204, 105, 133, 133, 71, 0, 0, 0, 0, 205, 71, 101, 0, 80, 0,
    0, 0, 0, 206, 207, 207, 207, 208, 71, 0, 0, 209, 209, 209, 209, 82,
    88, 105, 0, 210, 207, 207, 207, 208, 0, 200, 200, 11, 114, 0, 0, 0,
    117, 97, 97, 0, 12, 0, 0, 98, 0, 11, 0, 0, 0, 114, 0, 0,
    59, 0, 0, 0, 0, 0, 0, 0, 0, 0, 170, 11, 0, 0, 70, 70,
    211, 52, 16, 109, 72, 174, 0, 0, 12, 43, 0, 27, 59, 99, 0, 0,
    0, 0, 0, 212, 111, 111, 213, 213, 214, 62, 215, 0, 16, 0, 154, 216,
    154, 217, 218, 219, 79, 79, 220, 213, 221, 184, 184, 222, 222, 184, 184, 184,
    184, 223, 223, 11, 76, 76, 0, 223, 224, 62, 225, 144, 0, 36, 36, 226,
    195, 195, 227, 228, 229, 230, 36, 231, 231, 232, 233, 160, 160, 154, 234, 234,
    16, 174, 235, 236, 237, 225, 101, 101, 80, 231, 217, 238, 14, 14, 0, 0,
    239, 76, 240, 241, 42, 42, 54, 49, 49, 54, 242, 243, 244, 79, 79, 213,
    0, 0, 0, 0, 11, 0, 0, 0, 0, 239, 245, 0, 0, 79, 79, 241,
    0, 245, 0, 11, 0, 0, 0, 0, 11, 0, 0, 0, 0, 11, 0, 49,
    49, 54, 42, 42, 54, 0, 0, 0, 0, 0, 0, 49, 49, 54, 42, 42,
    54, 0, 14, 246, 0, 0, 0, 0, 0, 0, 0, 50, 70, 247, 247, 0,
    14, 14, 0, 0, 0, 173, 0, 14, 0, 0, 189, 0, 86, 0, 0, 49,
    0, 0, 52, 0, 0, 10, 0, 52, 0, 0, 0, 0, 36, 12, 0, 248,
    249, 249, 94, 7, 79, 79, 250, 141, 251, 94, 7, 0, 0, 0, 0, 92,
    0, 252, 252, 72, 1, 31, 72, 253, 253, 70, 36, 252, 252, 72, 1, 31,
    72, 253, 253, 70, 36, 254, 0, 0, 31, 11, 11, 0, 50, 50, 52, 0,
    0, 0, 34, 156, 0, 0, 0, 0, 45, 11, 255, 156, 256, 11, 0, 0,
    0, 257, 81, 81, 5, 70, 257, 81, 81, 5, 70, 11, 0, 0, 0, 172,
    172, 11, 0, 0, 0, 0, 0, 172, 0, 181, 11, 0, 38, 258, 259, 40,
    40, 40, 0, 0, 0, 31, 0, 4, 4, 0, 0, 0, 31, 18, 0, 0,
    13, 0, 0, 79, 79, 40, 260, 79, 79, 47, 260, 0, 0, 160, 20, 261,
    261, 54, 0, 168, 0, 0, 261, 20, 262, 91, 91, 58, 91, 42, 42, 54,
    0, 8, 158, 263, 0, 0, 0, 0, 16, 0, 264, 264, 49, 49, 54, 240,
    241, 242, 243, 213, 42, 0, 0, 0, 0, 16, 0, 11, 264, 263, 263, 263,
    265, 11, 0, 0, 158, 0, 266, 242, 242, 0, 11, 0, 267, 49, 49, 54,
    240, 241, 243, 213, 242, 0, 268, 0, 57, 57, 200, 57, 57, 57, 71, 200,
    269, 254, 254, 269, 270, 271, 155, 10, 204, 71, 10, 0, 11, 0, 0, 0,
    114, 0, 149, 0, 0, 272, 96, 186, 273, 274, 275, 25, 25, 25, 276, 0,
    277, 114, 11, 11, 114, 127, 278, 279, 280, 99, 99, 52, 281, 282, 283, 178,
    268, 284, 285, 99, 276, 286, 287, 288, 0, 72, 289, 0, 109, 0, 133, 133,
    71, 189, 14, 133, 0, 290, 0, 0, 0, 291, 0, 0, 0, 0, 0, 0,
    292, 31, 293, 0, 0, 0, 291, 0, 0, 0, 0, 0, 291, 291, 291, 0,
    31, 31, 0, 0, 11, 0, 0, 140, 274, 294, 294, 246, 295, 296, 297, 293,
    293, 11, 0, 0, 226, 298, 299, 300, 301, 302, 303, 304, 305, 306, 0, 106,
    0, 0, 0, 11, 0, 0, 106, 0, 67, 0, 307, 308, 295, 90, 90, 58,
    90, 309, 203, 203, 71, 147, 0, 198, 198, 0, 147, 147, 147, 138, 240, 240,
    0, 11, 0, 16, 0, 8, 310, 241, 241, 0, 16, 0, 12, 12, 16, 240,
    0, 311, 31, 31, 312, 313, 197, 28, 28, 52, 248, 79, 249, 143, 59, 52,
    16, 109, 207, 72, 174, 44, 94, 314, 0, 0, 0, 315, 316, 317, 318, 0,
    170, 95, 90, 169, 169, 12, 70, 11, 169, 169, 12, 70, 170, 170, 95, 0,
    319, 320, 321, 322, 35, 323, 24, 25, 25, 11, 181, 324, 324, 276, 324, 118,
    118, 118, 118, 118, 321, 80, 80, 0, 325, 326, 29, 24, 118, 118, 71, 321,
    80, 80, 0, 325, 25, 324, 324, 0, 0, 0, 293, 0, 0, 0, 11, 11,
    114, 0, 0, 0, 0, 0, 31, 0, 118, 29, 29, 69, 84, 284, 87, 283,
    86, 0, 0, 0, 50, 0, 0, 11, 0, 243, 243, 0, 0, 289, 327, 327,
    140, 327, 84, 0, 328, 0, 215, 329, 330, 331, 332, 151, 113, 181, 27, 11,
    27, 11, 79, 11, 113, 11, 0, 333, 29, 25, 24, 324, 113, 113, 334, 11,
    27, 113, 335, 336, 0, 0, 137, 337, 337, 58, 90, 90, 91, 71, 0, 71,
    72, 72, 338, 271, 16, 15, 52, 52, 92, 0, 314, 52, 52, 314, 45, 45,
    45, 45, 45, 0, 31, 50, 293, 12, 122, 44, 11, 103, 92, 184, 0, 152,
    241, 241, 0, 16, 0, 154, 216, 154, 217, 241, 339, 31, 340, 219, 260, 107,
    107, 107, 107, 341, 49, 49, 54, 0, 0, 0, 0, 16, 0, 272, 96, 186,
    271, 271, 342, 62, 67, 0, 240, 231, 343, 240, 241, 242, 243, 231, 213, 49,
    0, 49, 0, 344, 297, 345, 0, 254, 346, 0, 0, 11, 0, 346, 312, 0,
    0, 347, 348, 348, 54, 0, 11, 0, 0, 348, 349, 350, 351, 352, 353, 85,
    85, 156, 156, 85, 52, 354, 0, 355, 356, 357, 88, 358, 83, 359, 83, 0,
    0, 223, 360, 79, 0, 0, 237, 361, 362, 0, 169, 108, 363, 364, 279, 278,
    102, 365, 191, 180, 180, 11, 366, 366, 124, 367, 11, 368, 226, 171, 369, 288,
    370, 23, 74, 371, 207, 207, 16, 105, 105, 372, 8, 81, 373, 11, 114, 256,
    52, 374, 238, 58, 91, 90, 337, 26, 365, 375, 0, 0, 0, 11, 319, 31,
    31, 31, 24, 376, 31, 31, 29, 376, 319, 319, 0, 246, 0, 0, 376, 193,
    116, 69, 69, 72, 151, 151, 0, 69, 69, 127, 377, 63, 378, 341, 379, 270,
    193, 253, 41, 41, 41, 40, 40, 260, 144, 0, 0, 0, 31, 0, 4, 4,
    0, 0, 0, 31, 18, 0, 0, 14, 0, 192, 106, 296, 4, 0, 380, 381,
    159, 159, 154, 11, 0, 0, 0, 70, 0, 0, 70, 154, 154, 11, 217, 382,
    245, 245, 383, 100, 384, 11, 16, 378, 378, 378, 11, 16, 385, 2, 9, 285,
    386, 387, 0, 0, 0, 0, 0, 388, 189, 8, 27, 27, 388, 0, 0, 389,
    389, 14, 0, 76, 49, 49, 54, 240, 244, 213, 27, 27, 27, 14, 76, 42,
    242, 243, 49, 241, 240, 213, 244, 79, 79, 114, 390, 11, 79, 390, 11, 79,
    113, 11, 114, 45, 45, 0, 44, 43, 45, 45, 0, 44, 184, 0, 0, 0,
    16, 151, 151, 0, 0, 0, 0, 16, 151, 151, 0, 391, 392, 69, 69, 0,
    0, 0, 0, 58, 0, 59, 98, 0, 0, 0, 181, 127, 113, 113, 244, 244,
    244, 113, 49, 240, 243, 69, 244, 393, 393, 388, 189, 8, 394, 394, 280, 199,
    8, 199, 199, 11, 0, 69, 69, 2, 57, 0, 69, 69, 392, 0, 0, 45,
    0, 58, 0, 265, 0, 0, 0, 117, 122, 122, 122, 70, 97, 395, 207, 83,
    84, 0, 0, 256, 45, 0, 0, 0, 0, 45, 86, 52, 94, 94, 44, 396,
    85, 92, 364, 373, 86, 146, 222, 53, 78, 128, 17, 17, 17, 17, 17, 24,
    187, 55, 55, 55, 55, 55, 201, 25, 324, 114, 69, 397, 29, 21, 69, 184,
    398, 88, 88, 11, 0, 0, 0, 0, 0, 399, 400, 0, 384, 0, 125, 125,
    0, 0, 145, 145, 11, 0, 0, 11, 0, 0, 11, 0, 0, 0, 181, 98,
    0, 0, 11, 114, 11, 324, 32, 37, 245, 0, 206, 206, 103, 0, 0, 210,
    210, 0, 0, 0, 0, 401, 402, 325, 220, 208, 176, 369, 369, 176, 177, 11,
    248, 248, 79, 249, 176, 403, 97, 97, 97, 176, 403, 0, 177, 363, 13, 404,
    405, 397, 135, 135, 406, 381, 381, 406, 146, 146, 405, 407, 407, 290, 290, 309,
    408, 26, 26, 409, 26, 26, 136, 378, 378, 26, 391, 26, 26, 410, 219, 30,
    411, 247, 112, 308, 404, 110, 110, 28, 28, 28, 32, 0, 412, 11, 0, 0,
    5, 94, 94, 44, 0, 0, 11, 413, 235, 409, 0, 0, 327, 0, 0, 251,
    393, 393, 388, 189, 8, 393, 0, 145, 414, 0, 415, 416, 27, 27, 14, 76,
    42, 242, 243, 113, 113, 244, 244, 244, 113, 29, 29, 29, 0, 0, 69, 69,
    14, 76, 21, 21, 21, 0, 18, 0, 293, 229, 230, 228, 370, 417, 418, 181,
    181, 0, 0, 181, 368, 367, 419, 420, 33, 118, 118, 0, 0, 0, 325, 0,
    0, 0, 0, 118, 76, 76, 0, 11, 0, 421, 240, 241, 42, 42, 54, 49,
    49, 54, 242, 243, 213, 76, 0, 0, 0, 0, 0, 0, 0, 0, 45, 86,
    86, 45, 86, 243, 0, 0, 422, 348, 49, 348, 49, 231, 79, 79, 375, 0,
    0, 0, 27, 0, 333, 79, 79, 375, 0, 0, 0, 27, 0, 113, 79, 79,
    151, 151, 6, 0, 79, 79, 79, 151, 2, 0, 79, 79, 344, 360, 360, 42,
    11, 49, 223, 360, 249, 0, 406, 7, 30, 282, 403,
];

pub fn get_trino_segment_grammar(name: &str) -> Option<RootGrammar> {
    match name {
        "AbsentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(0), tables: &TRINO_TABLES }),
        "AccessObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(1), tables: &TRINO_TABLES }),
        "AccessPermissionSegment" => Some(RootGrammar { grammar_id: GrammarId(62), tables: &TRINO_TABLES }),
        "AccessPermissionsSegment" => Some(RootGrammar { grammar_id: GrammarId(131), tables: &TRINO_TABLES }),
        "AccessSchemaObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(134), tables: &TRINO_TABLES }),
        "AccessSchemaPluralObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(157), tables: &TRINO_TABLES }),
        "AccessStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(180), tables: &TRINO_TABLES }),
        "AccessTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(183), tables: &TRINO_TABLES }),
        "AccessorGrammar" => Some(RootGrammar { grammar_id: GrammarId(205), tables: &TRINO_TABLES }),
        "AddKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(208), tables: &TRINO_TABLES }),
        "AdminKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(209), tables: &TRINO_TABLES }),
        "AfterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(210), tables: &TRINO_TABLES }),
        "AggregateOrderByClause" => Some(RootGrammar { grammar_id: GrammarId(211), tables: &TRINO_TABLES }),
        "AliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(212), tables: &TRINO_TABLES }),
        "AliasedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(224), tables: &TRINO_TABLES }),
        "AllKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(227), tables: &TRINO_TABLES }),
        "AlterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(228), tables: &TRINO_TABLES }),
        "AlterSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(229), tables: &TRINO_TABLES }),
        "AlterSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(245), tables: &TRINO_TABLES }),
        "AlterTableDropColumnGrammar" => Some(RootGrammar { grammar_id: GrammarId(251), tables: &TRINO_TABLES }),
        "AlterTableOptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(256), tables: &TRINO_TABLES }),
        "AlterTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(283), tables: &TRINO_TABLES }),
        "AmpersandSegment" => Some(RootGrammar { grammar_id: GrammarId(361), tables: &TRINO_TABLES }),
        "AnalyzeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(362), tables: &TRINO_TABLES }),
        "AnalyzeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(363), tables: &TRINO_TABLES }),
        "AndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(374), tables: &TRINO_TABLES }),
        "AndOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(375), tables: &TRINO_TABLES }),
        "AnyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(376), tables: &TRINO_TABLES }),
        "ArithmeticBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(377), tables: &TRINO_TABLES }),
        "ArrayAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(388), tables: &TRINO_TABLES }),
        "ArrayExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(396), tables: &TRINO_TABLES }),
        "ArrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(397), tables: &TRINO_TABLES }),
        "ArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(398), tables: &TRINO_TABLES }),
        "ArrayTypeSchemaSegment" => Some(RootGrammar { grammar_id: GrammarId(401), tables: &TRINO_TABLES }),
        "ArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(408), tables: &TRINO_TABLES }),
        "AsAliasOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(411), tables: &TRINO_TABLES }),
        "AsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(413), tables: &TRINO_TABLES }),
        "AscKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(414), tables: &TRINO_TABLES }),
        "AtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(415), tables: &TRINO_TABLES }),
        "AuthorizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(416), tables: &TRINO_TABLES }),
        "AutoIncrementGrammar" => Some(RootGrammar { grammar_id: GrammarId(417), tables: &TRINO_TABLES }),
        "BareFunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(419), tables: &TRINO_TABLES }),
        "BaseExpressionElementGrammar" => Some(RootGrammar { grammar_id: GrammarId(420), tables: &TRINO_TABLES }),
        "BaseFileSegment" => Some(RootGrammar { grammar_id: GrammarId(432), tables: &TRINO_TABLES }),
        "BaseSegment" => Some(RootGrammar { grammar_id: GrammarId(433), tables: &TRINO_TABLES }),
        "BernoulliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(434), tables: &TRINO_TABLES }),
        "BetweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(435), tables: &TRINO_TABLES }),
        "BigintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(436), tables: &TRINO_TABLES }),
        "BinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(437), tables: &TRINO_TABLES }),
        "BinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(443), tables: &TRINO_TABLES }),
        "BitwiseAndSegment" => Some(RootGrammar { grammar_id: GrammarId(444), tables: &TRINO_TABLES }),
        "BitwiseLShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(445), tables: &TRINO_TABLES }),
        "BitwiseOrSegment" => Some(RootGrammar { grammar_id: GrammarId(448), tables: &TRINO_TABLES }),
        "BitwiseRShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(449), tables: &TRINO_TABLES }),
        "BitwiseXorSegment" => Some(RootGrammar { grammar_id: GrammarId(452), tables: &TRINO_TABLES }),
        "BooleanBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(453), tables: &TRINO_TABLES }),
        "BooleanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(456), tables: &TRINO_TABLES }),
        "BooleanLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(457), tables: &TRINO_TABLES }),
        "BothKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(460), tables: &TRINO_TABLES }),
        "BracketedArguments" => Some(RootGrammar { grammar_id: GrammarId(461), tables: &TRINO_TABLES }),
        "BracketedColumnReferenceListGrammar" => Some(RootGrammar { grammar_id: GrammarId(464), tables: &TRINO_TABLES }),
        "BracketedSegment" => Some(RootGrammar { grammar_id: GrammarId(467), tables: &TRINO_TABLES }),
        "BracketedSetExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(468), tables: &TRINO_TABLES }),
        "ByKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(470), tables: &TRINO_TABLES }),
        "CTEColumnList" => Some(RootGrammar { grammar_id: GrammarId(471), tables: &TRINO_TABLES }),
        "CTEDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(473), tables: &TRINO_TABLES }),
        "CallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(479), tables: &TRINO_TABLES }),
        "CascadeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(480), tables: &TRINO_TABLES }),
        "CaseExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(481), tables: &TRINO_TABLES }),
        "CaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(505), tables: &TRINO_TABLES }),
        "CastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(506), tables: &TRINO_TABLES }),
        "CastOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(507), tables: &TRINO_TABLES }),
        "CatalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(508), tables: &TRINO_TABLES }),
        "CatalogsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(509), tables: &TRINO_TABLES }),
        "CharCharacterSetGrammar" => Some(RootGrammar { grammar_id: GrammarId(510), tables: &TRINO_TABLES }),
        "CharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(511), tables: &TRINO_TABLES }),
        "CodeSegment" => Some(RootGrammar { grammar_id: GrammarId(512), tables: &TRINO_TABLES }),
        "CollateGrammar" => Some(RootGrammar { grammar_id: GrammarId(513), tables: &TRINO_TABLES }),
        "CollationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(514), tables: &TRINO_TABLES }),
        "ColonDelimiterSegment" => Some(RootGrammar { grammar_id: GrammarId(520), tables: &TRINO_TABLES }),
        "ColonPrefixSegment" => Some(RootGrammar { grammar_id: GrammarId(521), tables: &TRINO_TABLES }),
        "ColonSegment" => Some(RootGrammar { grammar_id: GrammarId(522), tables: &TRINO_TABLES }),
        "ColumnConstraintDefaultGrammar" => Some(RootGrammar { grammar_id: GrammarId(523), tables: &TRINO_TABLES }),
        "ColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(528), tables: &TRINO_TABLES }),
        "ColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(556), tables: &TRINO_TABLES }),
        "ColumnGeneratedGrammar" => Some(RootGrammar { grammar_id: GrammarId(572), tables: &TRINO_TABLES }),
        "ColumnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(573), tables: &TRINO_TABLES }),
        "ColumnReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(574), tables: &TRINO_TABLES }),
        "ColumnsExpressionFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(578), tables: &TRINO_TABLES }),
        "ColumnsExpressionFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(579), tables: &TRINO_TABLES }),
        "ColumnsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(580), tables: &TRINO_TABLES }),
        "ColumnsExpressionNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(581), tables: &TRINO_TABLES }),
        "ColumnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(582), tables: &TRINO_TABLES }),
        "CommaSegment" => Some(RootGrammar { grammar_id: GrammarId(583), tables: &TRINO_TABLES }),
        "CommentClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(584), tables: &TRINO_TABLES }),
        "CommentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(587), tables: &TRINO_TABLES }),
        "CommentOnStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(588), tables: &TRINO_TABLES }),
        "CommentSegment" => Some(RootGrammar { grammar_id: GrammarId(606), tables: &TRINO_TABLES }),
        "CommitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(607), tables: &TRINO_TABLES }),
        "CommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(608), tables: &TRINO_TABLES }),
        "ComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(609), tables: &TRINO_TABLES }),
        "ComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(618), tables: &TRINO_TABLES }),
        "CompositeBinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(619), tables: &TRINO_TABLES }),
        "CompositeComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(620), tables: &TRINO_TABLES }),
        "ConcatSegment" => Some(RootGrammar { grammar_id: GrammarId(621), tables: &TRINO_TABLES }),
        "ConditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(624), tables: &TRINO_TABLES }),
        "ConditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(625), tables: &TRINO_TABLES }),
        "ConditionalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(629), tables: &TRINO_TABLES }),
        "ConstraintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(630), tables: &TRINO_TABLES }),
        "CopartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(631), tables: &TRINO_TABLES }),
        "CountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(632), tables: &TRINO_TABLES }),
        "CreateCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(633), tables: &TRINO_TABLES }),
        "CreateDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(660), tables: &TRINO_TABLES }),
        "CreateFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(665), tables: &TRINO_TABLES }),
        "CreateIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(677), tables: &TRINO_TABLES }),
        "CreateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(690), tables: &TRINO_TABLES }),
        "CreateModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(691), tables: &TRINO_TABLES }),
        "CreateRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(711), tables: &TRINO_TABLES }),
        "CreateSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(716), tables: &TRINO_TABLES }),
        "CreateSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(721), tables: &TRINO_TABLES }),
        "CreateSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(741), tables: &TRINO_TABLES }),
        "CreateTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(747), tables: &TRINO_TABLES }),
        "CreateTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(784), tables: &TRINO_TABLES }),
        "CreateUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(851), tables: &TRINO_TABLES }),
        "CreateViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(855), tables: &TRINO_TABLES }),
        "CrossKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(867), tables: &TRINO_TABLES }),
        "CubeFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(868), tables: &TRINO_TABLES }),
        "CubeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(869), tables: &TRINO_TABLES }),
        "CubeRollupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(870), tables: &TRINO_TABLES }),
        "CurrentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(876), tables: &TRINO_TABLES }),
        "Current_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(877), tables: &TRINO_TABLES }),
        "Current_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(878), tables: &TRINO_TABLES }),
        "Current_pathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(879), tables: &TRINO_TABLES }),
        "Current_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(880), tables: &TRINO_TABLES }),
        "Current_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(881), tables: &TRINO_TABLES }),
        "Current_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(882), tables: &TRINO_TABLES }),
        "Current_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(883), tables: &TRINO_TABLES }),
        "Current_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(884), tables: &TRINO_TABLES }),
        "DataKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(885), tables: &TRINO_TABLES }),
        "DatabaseReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(574), tables: &TRINO_TABLES }),
        "DatatypeIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(886), tables: &TRINO_TABLES }),
        "DatatypeSegment" => Some(RootGrammar { grammar_id: GrammarId(890), tables: &TRINO_TABLES }),
        "DateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(895), tables: &TRINO_TABLES }),
        "DatePartFunctionName" => Some(RootGrammar { grammar_id: GrammarId(896), tables: &TRINO_TABLES }),
        "DatePartFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(897), tables: &TRINO_TABLES }),
        "DateTimeFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(898), tables: &TRINO_TABLES }),
        "DateTimeLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(904), tables: &TRINO_TABLES }),
        "DatetimeUnitSegment" => Some(RootGrammar { grammar_id: GrammarId(912), tables: &TRINO_TABLES }),
        "DayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(913), tables: &TRINO_TABLES }),
        "DeallocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(914), tables: &TRINO_TABLES }),
        "DecimalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(915), tables: &TRINO_TABLES }),
        "Dedent" => Some(RootGrammar { grammar_id: GrammarId(223), tables: &TRINO_TABLES }),
        "DefaultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(916), tables: &TRINO_TABLES }),
        "DefaultValuesGrammar" => Some(RootGrammar { grammar_id: GrammarId(917), tables: &TRINO_TABLES }),
        "DefineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(920), tables: &TRINO_TABLES }),
        "DefinerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(921), tables: &TRINO_TABLES }),
        "DeleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(922), tables: &TRINO_TABLES }),
        "DeleteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(923), tables: &TRINO_TABLES }),
        "DelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(927), tables: &TRINO_TABLES }),
        "DenyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(928), tables: &TRINO_TABLES }),
        "DescKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(929), tables: &TRINO_TABLES }),
        "DescribeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(930), tables: &TRINO_TABLES }),
        "DescribeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(931), tables: &TRINO_TABLES }),
        "DescriptorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(935), tables: &TRINO_TABLES }),
        "DistinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(936), tables: &TRINO_TABLES }),
        "DistributedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(937), tables: &TRINO_TABLES }),
        "DivideSegment" => Some(RootGrammar { grammar_id: GrammarId(938), tables: &TRINO_TABLES }),
        "DotSegment" => Some(RootGrammar { grammar_id: GrammarId(939), tables: &TRINO_TABLES }),
        "DoubleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(940), tables: &TRINO_TABLES }),
        "DropBehaviorGrammar" => Some(RootGrammar { grammar_id: GrammarId(941), tables: &TRINO_TABLES }),
        "DropCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(944), tables: &TRINO_TABLES }),
        "DropDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(952), tables: &TRINO_TABLES }),
        "DropFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(958), tables: &TRINO_TABLES }),
        "DropIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(963), tables: &TRINO_TABLES }),
        "DropKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(969), tables: &TRINO_TABLES }),
        "DropModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(970), tables: &TRINO_TABLES }),
        "DropRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(975), tables: &TRINO_TABLES }),
        "DropSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(980), tables: &TRINO_TABLES }),
        "DropSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(986), tables: &TRINO_TABLES }),
        "DropTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(990), tables: &TRINO_TABLES }),
        "DropTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(998), tables: &TRINO_TABLES }),
        "DropTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1003), tables: &TRINO_TABLES }),
        "DropUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1009), tables: &TRINO_TABLES }),
        "DropViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1014), tables: &TRINO_TABLES }),
        "ElseClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1020), tables: &TRINO_TABLES }),
        "ElseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1023), tables: &TRINO_TABLES }),
        "EmptyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1024), tables: &TRINO_TABLES }),
        "EmptyStructLiteralBracketsSegment" => Some(RootGrammar { grammar_id: GrammarId(1025), tables: &TRINO_TABLES }),
        "EmptyStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(1026), tables: &TRINO_TABLES }),
        "EncodingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1029), tables: &TRINO_TABLES }),
        "EndAngleBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(405), tables: &TRINO_TABLES }),
        "EndBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(221), tables: &TRINO_TABLES }),
        "EndCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(1030), tables: &TRINO_TABLES }),
        "EndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1031), tables: &TRINO_TABLES }),
        "EndSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(395), tables: &TRINO_TABLES }),
        "EqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(1032), tables: &TRINO_TABLES }),
        "ErrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1033), tables: &TRINO_TABLES }),
        "EscapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1034), tables: &TRINO_TABLES }),
        "ExceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1035), tables: &TRINO_TABLES }),
        "ExcludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1036), tables: &TRINO_TABLES }),
        "ExecuteArrowSegment" => Some(RootGrammar { grammar_id: GrammarId(1037), tables: &TRINO_TABLES }),
        "ExecuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1038), tables: &TRINO_TABLES }),
        "ExistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1039), tables: &TRINO_TABLES }),
        "ExplainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1040), tables: &TRINO_TABLES }),
        "ExplainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1041), tables: &TRINO_TABLES }),
        "ExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1048), tables: &TRINO_TABLES }),
        "Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1049), tables: &TRINO_TABLES }),
        "Expression_A_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1075), tables: &TRINO_TABLES }),
        "Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1081), tables: &TRINO_TABLES }),
        "Expression_B_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1091), tables: &TRINO_TABLES }),
        "Expression_C_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1096), tables: &TRINO_TABLES }),
        "Expression_D_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1109), tables: &TRINO_TABLES }),
        "Expression_D_Potential_Select_Statement_Without_Brackets" => Some(RootGrammar { grammar_id: GrammarId(1146), tables: &TRINO_TABLES }),
        "ExtendedNaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1154), tables: &TRINO_TABLES }),
        "ExtensionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(574), tables: &TRINO_TABLES }),
        "ExtractKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1155), tables: &TRINO_TABLES }),
        "FalseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1156), tables: &TRINO_TABLES }),
        "FalseSegment" => Some(RootGrammar { grammar_id: GrammarId(1157), tables: &TRINO_TABLES }),
        "FetchClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1158), tables: &TRINO_TABLES }),
        "FetchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1175), tables: &TRINO_TABLES }),
        "FileSegment" => Some(RootGrammar { grammar_id: GrammarId(1176), tables: &TRINO_TABLES }),
        "FilterClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1185), tables: &TRINO_TABLES }),
        "FilterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1191), tables: &TRINO_TABLES }),
        "FinalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1192), tables: &TRINO_TABLES }),
        "FirstKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1193), tables: &TRINO_TABLES }),
        "FollowingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1194), tables: &TRINO_TABLES }),
        "ForKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1195), tables: &TRINO_TABLES }),
        "ForeignKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(1196), tables: &TRINO_TABLES }),
        "FormatJsonEncodingGrammar" => Some(RootGrammar { grammar_id: GrammarId(1197), tables: &TRINO_TABLES }),
        "FormatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1206), tables: &TRINO_TABLES }),
        "FrameClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1207), tables: &TRINO_TABLES }),
        "FrameClauseUnitGrammar" => Some(RootGrammar { grammar_id: GrammarId(1225), tables: &TRINO_TABLES }),
        "FromClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1228), tables: &TRINO_TABLES }),
        "FromClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1232), tables: &TRINO_TABLES }),
        "FromExpressionElementSegment" => Some(RootGrammar { grammar_id: GrammarId(1248), tables: &TRINO_TABLES }),
        "FromExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1271), tables: &TRINO_TABLES }),
        "FromKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1298), tables: &TRINO_TABLES }),
        "FullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1299), tables: &TRINO_TABLES }),
        "FunctionContentsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1300), tables: &TRINO_TABLES }),
        "FunctionContentsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1303), tables: &TRINO_TABLES }),
        "FunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1385), tables: &TRINO_TABLES }),
        "FunctionDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1388), tables: &TRINO_TABLES }),
        "FunctionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1394), tables: &TRINO_TABLES }),
        "FunctionNameIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1395), tables: &TRINO_TABLES }),
        "FunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1396), tables: &TRINO_TABLES }),
        "FunctionParameterGrammar" => Some(RootGrammar { grammar_id: GrammarId(1406), tables: &TRINO_TABLES }),
        "FunctionParameterListGrammar" => Some(RootGrammar { grammar_id: GrammarId(1419), tables: &TRINO_TABLES }),
        "FunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(1422), tables: &TRINO_TABLES }),
        "FunctionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1436), tables: &TRINO_TABLES }),
        "GlobOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1437), tables: &TRINO_TABLES }),
        "GraceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1438), tables: &TRINO_TABLES }),
        "GrantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1439), tables: &TRINO_TABLES }),
        "GrantStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1440), tables: &TRINO_TABLES }),
        "GrantedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1478), tables: &TRINO_TABLES }),
        "GrantsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1479), tables: &TRINO_TABLES }),
        "GraphvizKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1480), tables: &TRINO_TABLES }),
        "GreaterThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(1481), tables: &TRINO_TABLES }),
        "GreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(1484), tables: &TRINO_TABLES }),
        "GroupByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1485), tables: &TRINO_TABLES }),
        "GroupByClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1499), tables: &TRINO_TABLES }),
        "GroupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1508), tables: &TRINO_TABLES }),
        "GroupingExpressionList" => Some(RootGrammar { grammar_id: GrammarId(1509), tables: &TRINO_TABLES }),
        "GroupingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1517), tables: &TRINO_TABLES }),
        "GroupingSetsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1518), tables: &TRINO_TABLES }),
        "GroupsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1526), tables: &TRINO_TABLES }),
        "HavingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1527), tables: &TRINO_TABLES }),
        "HavingClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1532), tables: &TRINO_TABLES }),
        "HavingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1540), tables: &TRINO_TABLES }),
        "HorizontalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1541), tables: &TRINO_TABLES }),
        "HourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1542), tables: &TRINO_TABLES }),
        "IdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1543), tables: &TRINO_TABLES }),
        "IfExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1544), tables: &TRINO_TABLES }),
        "IfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1547), tables: &TRINO_TABLES }),
        "IfNotExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1548), tables: &TRINO_TABLES }),
        "IgnoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1552), tables: &TRINO_TABLES }),
        "IgnoreRespectNullsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1553), tables: &TRINO_TABLES }),
        "ImmediateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1558), tables: &TRINO_TABLES }),
        "ImplicitIndent" => Some(RootGrammar { grammar_id: GrammarId(484), tables: &TRINO_TABLES }),
        "InKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1559), tables: &TRINO_TABLES }),
        "InOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1560), tables: &TRINO_TABLES }),
        "IncludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1570), tables: &TRINO_TABLES }),
        "Indent" => Some(RootGrammar { grammar_id: GrammarId(213), tables: &TRINO_TABLES }),
        "IndexColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(1571), tables: &TRINO_TABLES }),
        "IndexReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(574), tables: &TRINO_TABLES }),
        "InitialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1576), tables: &TRINO_TABLES }),
        "InnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1577), tables: &TRINO_TABLES }),
        "InputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1578), tables: &TRINO_TABLES }),
        "InsertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1579), tables: &TRINO_TABLES }),
        "InsertStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1580), tables: &TRINO_TABLES }),
        "IntKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1590), tables: &TRINO_TABLES }),
        "IntegerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1591), tables: &TRINO_TABLES }),
        "IntersectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1592), tables: &TRINO_TABLES }),
        "IntervalExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1593), tables: &TRINO_TABLES }),
        "IntervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1603), tables: &TRINO_TABLES }),
        "IntoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1604), tables: &TRINO_TABLES }),
        "InvokerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1605), tables: &TRINO_TABLES }),
        "IoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1606), tables: &TRINO_TABLES }),
        "IpaddressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1607), tables: &TRINO_TABLES }),
        "IsClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1608), tables: &TRINO_TABLES }),
        "IsDistinctFromGrammar" => Some(RootGrammar { grammar_id: GrammarId(1614), tables: &TRINO_TABLES }),
        "IsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1619), tables: &TRINO_TABLES }),
        "IsNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(1620), tables: &TRINO_TABLES }),
        "IsolationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1621), tables: &TRINO_TABLES }),
        "JoinClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1622), tables: &TRINO_TABLES }),
        "JoinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1644), tables: &TRINO_TABLES }),
        "JoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1645), tables: &TRINO_TABLES }),
        "JoinLikeClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1647), tables: &TRINO_TABLES }),
        "JoinOnConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(1648), tables: &TRINO_TABLES }),
        "JoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1655), tables: &TRINO_TABLES }),
        "JoinUsingConditionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1663), tables: &TRINO_TABLES }),
        "JsonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1668), tables: &TRINO_TABLES }),
        "Json_arrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1669), tables: &TRINO_TABLES }),
        "Json_existsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1670), tables: &TRINO_TABLES }),
        "Json_objectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1671), tables: &TRINO_TABLES }),
        "Json_queryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1672), tables: &TRINO_TABLES }),
        "Json_tableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1673), tables: &TRINO_TABLES }),
        "Json_valueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1674), tables: &TRINO_TABLES }),
        "KeepKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1675), tables: &TRINO_TABLES }),
        "KeyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1676), tables: &TRINO_TABLES }),
        "KeysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1677), tables: &TRINO_TABLES }),
        "KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1678), tables: &TRINO_TABLES }),
        "LambdaArrowSegment" => Some(RootGrammar { grammar_id: GrammarId(1679), tables: &TRINO_TABLES }),
        "LambdaExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1680), tables: &TRINO_TABLES }),
        "LastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1688), tables: &TRINO_TABLES }),
        "LateralColumnAliasExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1689), tables: &TRINO_TABLES }),
        "LateralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1690), tables: &TRINO_TABLES }),
        "LeadingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1691), tables: &TRINO_TABLES }),
        "LeftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1692), tables: &TRINO_TABLES }),
        "LessThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(1693), tables: &TRINO_TABLES }),
        "LessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(1696), tables: &TRINO_TABLES }),
        "LevelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1697), tables: &TRINO_TABLES }),
        "LikeExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1698), tables: &TRINO_TABLES }),
        "LikeGrammar" => Some(RootGrammar { grammar_id: GrammarId(1706), tables: &TRINO_TABLES }),
        "LikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1708), tables: &TRINO_TABLES }),
        "LikeOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1709), tables: &TRINO_TABLES }),
        "LimitClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1710), tables: &TRINO_TABLES }),
        "LimitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1727), tables: &TRINO_TABLES }),
        "ListComprehensionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1728), tables: &TRINO_TABLES }),
        "ListaggKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1729), tables: &TRINO_TABLES }),
        "ListaggOverflowClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1730), tables: &TRINO_TABLES }),
        "LiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(1742), tables: &TRINO_TABLES }),
        "LiteralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1752), tables: &TRINO_TABLES }),
        "LiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(1753), tables: &TRINO_TABLES }),
        "LocalAliasSegment" => Some(RootGrammar { grammar_id: GrammarId(1754), tables: &TRINO_TABLES }),
        "LocalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1755), tables: &TRINO_TABLES }),
        "LocaltimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1756), tables: &TRINO_TABLES }),
        "LocaltimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1757), tables: &TRINO_TABLES }),
        "LogicalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1758), tables: &TRINO_TABLES }),
        "MLTableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1759), tables: &TRINO_TABLES }),
        "MapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1760), tables: &TRINO_TABLES }),
        "MapTypeSchemaSegment" => Some(RootGrammar { grammar_id: GrammarId(1761), tables: &TRINO_TABLES }),
        "MapTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(1772), tables: &TRINO_TABLES }),
        "MatchConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(1775), tables: &TRINO_TABLES }),
        "MatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1776), tables: &TRINO_TABLES }),
        "Match_recognizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1777), tables: &TRINO_TABLES }),
        "MatchedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1778), tables: &TRINO_TABLES }),
        "MatchesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1779), tables: &TRINO_TABLES }),
        "MaterializedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1780), tables: &TRINO_TABLES }),
        "MeasuresKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1781), tables: &TRINO_TABLES }),
        "MergeDeleteClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1782), tables: &TRINO_TABLES }),
        "MergeInsertClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1783), tables: &TRINO_TABLES }),
        "MergeIntoLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(1787), tables: &TRINO_TABLES }),
        "MergeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1790), tables: &TRINO_TABLES }),
        "MergeMatchSegment" => Some(RootGrammar { grammar_id: GrammarId(1791), tables: &TRINO_TABLES }),
        "MergeMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1794), tables: &TRINO_TABLES }),
        "MergeNotMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1804), tables: &TRINO_TABLES }),
        "MergeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1813), tables: &TRINO_TABLES }),
        "MergeUpdateClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1833), tables: &TRINO_TABLES }),
        "MinusSegment" => Some(RootGrammar { grammar_id: GrammarId(1836), tables: &TRINO_TABLES }),
        "MinuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1837), tables: &TRINO_TABLES }),
        "ModuloSegment" => Some(RootGrammar { grammar_id: GrammarId(1838), tables: &TRINO_TABLES }),
        "MonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1839), tables: &TRINO_TABLES }),
        "MultiplySegment" => Some(RootGrammar { grammar_id: GrammarId(1840), tables: &TRINO_TABLES }),
        "NakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1841), tables: &TRINO_TABLES }),
        "NamedWindowExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1842), tables: &TRINO_TABLES }),
        "NamedWindowSegment" => Some(RootGrammar { grammar_id: GrammarId(1849), tables: &TRINO_TABLES }),
        "NanLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(1853), tables: &TRINO_TABLES }),
        "NaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1854), tables: &TRINO_TABLES }),
        "NaturalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1857), tables: &TRINO_TABLES }),
        "NegativeSegment" => Some(RootGrammar { grammar_id: GrammarId(1858), tables: &TRINO_TABLES }),
        "NestedJoinGrammar" => Some(RootGrammar { grammar_id: GrammarId(1859), tables: &TRINO_TABLES }),
        "NestedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1860), tables: &TRINO_TABLES }),
        "NewlineSegment" => Some(RootGrammar { grammar_id: GrammarId(1861), tables: &TRINO_TABLES }),
        "NextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1862), tables: &TRINO_TABLES }),
        "NfcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1863), tables: &TRINO_TABLES }),
        "NfdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1864), tables: &TRINO_TABLES }),
        "NfkcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1865), tables: &TRINO_TABLES }),
        "NfkdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1866), tables: &TRINO_TABLES }),
        "NoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1867), tables: &TRINO_TABLES }),
        "NonSetSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(1868), tables: &TRINO_TABLES }),
        "NonStandardJoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1878), tables: &TRINO_TABLES }),
        "NonWithNonSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(1879), tables: &TRINO_TABLES }),
        "NonWithSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(1884), tables: &TRINO_TABLES }),
        "NoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1890), tables: &TRINO_TABLES }),
        "NormalizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1891), tables: &TRINO_TABLES }),
        "NormalizedGrammar" => Some(RootGrammar { grammar_id: GrammarId(1892), tables: &TRINO_TABLES }),
        "NotEnforcedGrammar" => Some(RootGrammar { grammar_id: GrammarId(1893), tables: &TRINO_TABLES }),
        "NotEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(1894), tables: &TRINO_TABLES }),
        "NotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1901), tables: &TRINO_TABLES }),
        "NotNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(1902), tables: &TRINO_TABLES }),
        "NotOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1903), tables: &TRINO_TABLES }),
        "NullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1904), tables: &TRINO_TABLES }),
        "NullLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(1905), tables: &TRINO_TABLES }),
        "NullifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1906), tables: &TRINO_TABLES }),
        "NullsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1907), tables: &TRINO_TABLES }),
        "NumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(1908), tables: &TRINO_TABLES }),
        "ObjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1909), tables: &TRINO_TABLES }),
        "ObjectLiteralElementSegment" => Some(RootGrammar { grammar_id: GrammarId(1910), tables: &TRINO_TABLES }),
        "ObjectLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(1914), tables: &TRINO_TABLES }),
        "ObjectReferenceDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(1918), tables: &TRINO_TABLES }),
        "ObjectReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(574), tables: &TRINO_TABLES }),
        "ObjectReferenceTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1923), tables: &TRINO_TABLES }),
        "OfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1935), tables: &TRINO_TABLES }),
        "OffsetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1936), tables: &TRINO_TABLES }),
        "OffsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1945), tables: &TRINO_TABLES }),
        "OmitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1946), tables: &TRINO_TABLES }),
        "OnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1947), tables: &TRINO_TABLES }),
        "OneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1948), tables: &TRINO_TABLES }),
        "OnlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1949), tables: &TRINO_TABLES }),
        "OptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1950), tables: &TRINO_TABLES }),
        "OrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1951), tables: &TRINO_TABLES }),
        "OrOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1952), tables: &TRINO_TABLES }),
        "OrReplaceGrammar" => Some(RootGrammar { grammar_id: GrammarId(1953), tables: &TRINO_TABLES }),
        "OrderByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1956), tables: &TRINO_TABLES }),
        "OrderByClauseTerminators" => Some(RootGrammar { grammar_id: GrammarId(1976), tables: &TRINO_TABLES }),
        "OrderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1983), tables: &TRINO_TABLES }),
        "OrderNoOrderGrammar" => Some(RootGrammar { grammar_id: GrammarId(1984), tables: &TRINO_TABLES }),
        "OrdinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1987), tables: &TRINO_TABLES }),
        "OuterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1988), tables: &TRINO_TABLES }),
        "OutputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1989), tables: &TRINO_TABLES }),
        "OverClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1990), tables: &TRINO_TABLES }),
        "OverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1997), tables: &TRINO_TABLES }),
        "OverflowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1998), tables: &TRINO_TABLES }),
        "OverlapsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1999), tables: &TRINO_TABLES }),
        "ParameterNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2000), tables: &TRINO_TABLES }),
        "ParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(2001), tables: &TRINO_TABLES }),
        "PartitionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2002), tables: &TRINO_TABLES }),
        "PartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2009), tables: &TRINO_TABLES }),
        "PartitionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2010), tables: &TRINO_TABLES }),
        "PassingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2011), tables: &TRINO_TABLES }),
        "PastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2012), tables: &TRINO_TABLES }),
        "PathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2013), tables: &TRINO_TABLES }),
        "PathSegment" => Some(RootGrammar { grammar_id: GrammarId(2014), tables: &TRINO_TABLES }),
        "PatternKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2021), tables: &TRINO_TABLES }),
        "PatternMatchingGrammar" => Some(RootGrammar { grammar_id: GrammarId(2022), tables: &TRINO_TABLES }),
        "PerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2023), tables: &TRINO_TABLES }),
        "PeriodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2024), tables: &TRINO_TABLES }),
        "PermuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2025), tables: &TRINO_TABLES }),
        "PipeSegment" => Some(RootGrammar { grammar_id: GrammarId(2026), tables: &TRINO_TABLES }),
        "PlanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2027), tables: &TRINO_TABLES }),
        "PlusSegment" => Some(RootGrammar { grammar_id: GrammarId(2028), tables: &TRINO_TABLES }),
        "PositionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2029), tables: &TRINO_TABLES }),
        "PositiveSegment" => Some(RootGrammar { grammar_id: GrammarId(2030), tables: &TRINO_TABLES }),
        "PostFunctionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2031), tables: &TRINO_TABLES }),
        "PostTableExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2036), tables: &TRINO_TABLES }),
        "PreTableFunctionKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2037), tables: &TRINO_TABLES }),
        "PrecedingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2038), tables: &TRINO_TABLES }),
        "PrecisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2039), tables: &TRINO_TABLES }),
        "PrepareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2040), tables: &TRINO_TABLES }),
        "PrimaryKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(2041), tables: &TRINO_TABLES }),
        "PrimitiveTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2042), tables: &TRINO_TABLES }),
        "PrivilegesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2065), tables: &TRINO_TABLES }),
        "PropertiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2066), tables: &TRINO_TABLES }),
        "PruneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2067), tables: &TRINO_TABLES }),
        "QualifiedNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2068), tables: &TRINO_TABLES }),
        "QuantifiedComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2071), tables: &TRINO_TABLES }),
        "QuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2079), tables: &TRINO_TABLES }),
        "QuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2080), tables: &TRINO_TABLES }),
        "QuotesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2081), tables: &TRINO_TABLES }),
        "RangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2082), tables: &TRINO_TABLES }),
        "RawEqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(2083), tables: &TRINO_TABLES }),
        "RawGreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2084), tables: &TRINO_TABLES }),
        "RawLessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2085), tables: &TRINO_TABLES }),
        "RawNotSegment" => Some(RootGrammar { grammar_id: GrammarId(2086), tables: &TRINO_TABLES }),
        "ReadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2087), tables: &TRINO_TABLES }),
        "RealKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2088), tables: &TRINO_TABLES }),
        "RecursiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2089), tables: &TRINO_TABLES }),
        "ReferenceDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2090), tables: &TRINO_TABLES }),
        "ReferenceMatchGrammar" => Some(RootGrammar { grammar_id: GrammarId(2104), tables: &TRINO_TABLES }),
        "ReferentialActionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2110), tables: &TRINO_TABLES }),
        "RefreshKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2122), tables: &TRINO_TABLES }),
        "RenameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2123), tables: &TRINO_TABLES }),
        "RepeatableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2124), tables: &TRINO_TABLES }),
        "ReplaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2125), tables: &TRINO_TABLES }),
        "ResetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2126), tables: &TRINO_TABLES }),
        "RespectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2127), tables: &TRINO_TABLES }),
        "RestrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2128), tables: &TRINO_TABLES }),
        "ReturningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2129), tables: &TRINO_TABLES }),
        "RevokeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2130), tables: &TRINO_TABLES }),
        "RevokeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2131), tables: &TRINO_TABLES }),
        "RightArrowOperator" => Some(RootGrammar { grammar_id: GrammarId(2154), tables: &TRINO_TABLES }),
        "RightKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2155), tables: &TRINO_TABLES }),
        "RoleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2156), tables: &TRINO_TABLES }),
        "RoleReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2157), tables: &TRINO_TABLES }),
        "RolesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2158), tables: &TRINO_TABLES }),
        "RollbackKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2159), tables: &TRINO_TABLES }),
        "RollupFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2160), tables: &TRINO_TABLES }),
        "RollupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2161), tables: &TRINO_TABLES }),
        "RowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2162), tables: &TRINO_TABLES }),
        "RowTypeSchemaSegment" => Some(RootGrammar { grammar_id: GrammarId(2163), tables: &TRINO_TABLES }),
        "RowTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2171), tables: &TRINO_TABLES }),
        "RowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2174), tables: &TRINO_TABLES }),
        "RunningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2175), tables: &TRINO_TABLES }),
        "SamplingExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2176), tables: &TRINO_TABLES }),
        "ScalarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2188), tables: &TRINO_TABLES }),
        "SchemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2189), tables: &TRINO_TABLES }),
        "SchemaReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(574), tables: &TRINO_TABLES }),
        "SchemasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2190), tables: &TRINO_TABLES }),
        "SecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2191), tables: &TRINO_TABLES }),
        "SecurityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2192), tables: &TRINO_TABLES }),
        "SeekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2193), tables: &TRINO_TABLES }),
        "SelectClauseElementSegment" => Some(RootGrammar { grammar_id: GrammarId(2194), tables: &TRINO_TABLES }),
        "SelectClauseModifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2199), tables: &TRINO_TABLES }),
        "SelectClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2202), tables: &TRINO_TABLES }),
        "SelectClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2208), tables: &TRINO_TABLES }),
        "SelectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2218), tables: &TRINO_TABLES }),
        "SelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2219), tables: &TRINO_TABLES }),
        "SelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2233), tables: &TRINO_TABLES }),
        "SemiStructuredAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(2243), tables: &TRINO_TABLES }),
        "SemicolonSegment" => Some(RootGrammar { grammar_id: GrammarId(2252), tables: &TRINO_TABLES }),
        "SequenceMaxValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(2253), tables: &TRINO_TABLES }),
        "SequenceMinValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(2260), tables: &TRINO_TABLES }),
        "SequenceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(574), tables: &TRINO_TABLES }),
        "SerializableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2267), tables: &TRINO_TABLES }),
        "SessionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2268), tables: &TRINO_TABLES }),
        "SetClauseListSegment" => Some(RootGrammar { grammar_id: GrammarId(2269), tables: &TRINO_TABLES }),
        "SetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2273), tables: &TRINO_TABLES }),
        "SetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2284), tables: &TRINO_TABLES }),
        "SetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2293), tables: &TRINO_TABLES }),
        "SetOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2294), tables: &TRINO_TABLES }),
        "SetSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2309), tables: &TRINO_TABLES }),
        "SetSessionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2314), tables: &TRINO_TABLES }),
        "SetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2323), tables: &TRINO_TABLES }),
        "ShorthandCastSegment" => Some(RootGrammar { grammar_id: GrammarId(2324), tables: &TRINO_TABLES }),
        "ShowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2333), tables: &TRINO_TABLES }),
        "SignedSegmentGrammar" => Some(RootGrammar { grammar_id: GrammarId(2334), tables: &TRINO_TABLES }),
        "SingleIdentifierGrammar" => Some(RootGrammar { grammar_id: GrammarId(2337), tables: &TRINO_TABLES }),
        "SingleIdentifierListSegment" => Some(RootGrammar { grammar_id: GrammarId(2341), tables: &TRINO_TABLES }),
        "SingleIdentifierWildcardSegment" => Some(RootGrammar { grammar_id: GrammarId(2343), tables: &TRINO_TABLES }),
        "SingleQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2347), tables: &TRINO_TABLES }),
        "SizedArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2348), tables: &TRINO_TABLES }),
        "SkipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2351), tables: &TRINO_TABLES }),
        "SlashSegment" => Some(RootGrammar { grammar_id: GrammarId(2352), tables: &TRINO_TABLES }),
        "SliceSegment" => Some(RootGrammar { grammar_id: GrammarId(2353), tables: &TRINO_TABLES }),
        "SmallintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2354), tables: &TRINO_TABLES }),
        "SomeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2355), tables: &TRINO_TABLES }),
        "StarSegment" => Some(RootGrammar { grammar_id: GrammarId(2356), tables: &TRINO_TABLES }),
        "StartAngleBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(404), tables: &TRINO_TABLES }),
        "StartBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(220), tables: &TRINO_TABLES }),
        "StartCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(1917), tables: &TRINO_TABLES }),
        "StartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2357), tables: &TRINO_TABLES }),
        "StartSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(394), tables: &TRINO_TABLES }),
        "StatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2358), tables: &TRINO_TABLES }),
        "StatsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2384), tables: &TRINO_TABLES }),
        "StringBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2385), tables: &TRINO_TABLES }),
        "StructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2387), tables: &TRINO_TABLES }),
        "StructTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2392), tables: &TRINO_TABLES }),
        "SubsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2393), tables: &TRINO_TABLES }),
        "SubstringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2394), tables: &TRINO_TABLES }),
        "SymbolSegment" => Some(RootGrammar { grammar_id: GrammarId(2395), tables: &TRINO_TABLES }),
        "SystemKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2396), tables: &TRINO_TABLES }),
        "TableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(2397), tables: &TRINO_TABLES }),
        "TableEndClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2412), tables: &TRINO_TABLES }),
        "TableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2413), tables: &TRINO_TABLES }),
        "TableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2422), tables: &TRINO_TABLES }),
        "TableReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(574), tables: &TRINO_TABLES }),
        "TablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2423), tables: &TRINO_TABLES }),
        "TablesampleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2424), tables: &TRINO_TABLES }),
        "TablespaceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(574), tables: &TRINO_TABLES }),
        "TagReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(574), tables: &TRINO_TABLES }),
        "Tail_Recurse_Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2425), tables: &TRINO_TABLES }),
        "Tail_Recurse_Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2430), tables: &TRINO_TABLES }),
        "TemporalQuerySegment" => Some(RootGrammar { grammar_id: GrammarId(2434), tables: &TRINO_TABLES }),
        "TemporaryGrammar" => Some(RootGrammar { grammar_id: GrammarId(2435), tables: &TRINO_TABLES }),
        "TemporaryTransientGrammar" => Some(RootGrammar { grammar_id: GrammarId(2436), tables: &TRINO_TABLES }),
        "TextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2437), tables: &TRINO_TABLES }),
        "Text_stringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2438), tables: &TRINO_TABLES }),
        "ThenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2439), tables: &TRINO_TABLES }),
        "TiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2440), tables: &TRINO_TABLES }),
        "TildeSegment" => Some(RootGrammar { grammar_id: GrammarId(2441), tables: &TRINO_TABLES }),
        "TimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2442), tables: &TRINO_TABLES }),
        "TimeWithTZGrammar" => Some(RootGrammar { grammar_id: GrammarId(2443), tables: &TRINO_TABLES }),
        "TimeZoneGrammar" => Some(RootGrammar { grammar_id: GrammarId(2454), tables: &TRINO_TABLES }),
        "TimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2460), tables: &TRINO_TABLES }),
        "TinyintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2461), tables: &TRINO_TABLES }),
        "ToKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2462), tables: &TRINO_TABLES }),
        "TrailingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2463), tables: &TRINO_TABLES }),
        "TransactionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2464), tables: &TRINO_TABLES }),
        "TransactionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2465), tables: &TRINO_TABLES }),
        "TriggerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(574), tables: &TRINO_TABLES }),
        "TrimKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2496), tables: &TRINO_TABLES }),
        "TrimParametersGrammar" => Some(RootGrammar { grammar_id: GrammarId(2497), tables: &TRINO_TABLES }),
        "TrueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2501), tables: &TRINO_TABLES }),
        "TrueSegment" => Some(RootGrammar { grammar_id: GrammarId(2502), tables: &TRINO_TABLES }),
        "TruncateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2503), tables: &TRINO_TABLES }),
        "TruncateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2504), tables: &TRINO_TABLES }),
        "Try_castKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2508), tables: &TRINO_TABLES }),
        "TupleSegment" => Some(RootGrammar { grammar_id: GrammarId(2509), tables: &TRINO_TABLES }),
        "TypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2512), tables: &TRINO_TABLES }),
        "TypedArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2513), tables: &TRINO_TABLES }),
        "TypedStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2516), tables: &TRINO_TABLES }),
        "UescapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2519), tables: &TRINO_TABLES }),
        "UnboundedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2520), tables: &TRINO_TABLES }),
        "UncommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2521), tables: &TRINO_TABLES }),
        "UnconditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2522), tables: &TRINO_TABLES }),
        "UnconditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2523), tables: &TRINO_TABLES }),
        "UnconditionalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2527), tables: &TRINO_TABLES }),
        "UnionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2528), tables: &TRINO_TABLES }),
        "UnionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2533), tables: &TRINO_TABLES }),
        "UniqueKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(2534), tables: &TRINO_TABLES }),
        "UniqueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2535), tables: &TRINO_TABLES }),
        "UnknownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2536), tables: &TRINO_TABLES }),
        "UnknownLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2537), tables: &TRINO_TABLES }),
        "UnmatchedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2538), tables: &TRINO_TABLES }),
        "UnnestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2539), tables: &TRINO_TABLES }),
        "UnorderedSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2540), tables: &TRINO_TABLES }),
        "UnorderedSetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2547), tables: &TRINO_TABLES }),
        "UpdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2553), tables: &TRINO_TABLES }),
        "UpdateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2554), tables: &TRINO_TABLES }),
        "UseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2562), tables: &TRINO_TABLES }),
        "UseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2563), tables: &TRINO_TABLES }),
        "UserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2566), tables: &TRINO_TABLES }),
        "UserReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2567), tables: &TRINO_TABLES }),
        "UsingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2568), tables: &TRINO_TABLES }),
        "Utf16KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2569), tables: &TRINO_TABLES }),
        "Utf32KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2570), tables: &TRINO_TABLES }),
        "Utf8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2571), tables: &TRINO_TABLES }),
        "UuidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2572), tables: &TRINO_TABLES }),
        "ValidateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2573), tables: &TRINO_TABLES }),
        "ValueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2574), tables: &TRINO_TABLES }),
        "ValuesClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2575), tables: &TRINO_TABLES }),
        "ValuesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2579), tables: &TRINO_TABLES }),
        "VarbinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2580), tables: &TRINO_TABLES }),
        "VarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2581), tables: &TRINO_TABLES }),
        "VerboseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2582), tables: &TRINO_TABLES }),
        "VersionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2583), tables: &TRINO_TABLES }),
        "ViewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2584), tables: &TRINO_TABLES }),
        "WhenClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2585), tables: &TRINO_TABLES }),
        "WhenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2595), tables: &TRINO_TABLES }),
        "WhereClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2596), tables: &TRINO_TABLES }),
        "WhereClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2601), tables: &TRINO_TABLES }),
        "WhereKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2613), tables: &TRINO_TABLES }),
        "WhitespaceSegment" => Some(RootGrammar { grammar_id: GrammarId(2614), tables: &TRINO_TABLES }),
        "WildcardExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2615), tables: &TRINO_TABLES }),
        "WildcardIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2617), tables: &TRINO_TABLES }),
        "WindowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2627), tables: &TRINO_TABLES }),
        "WindowSpecificationSegment" => Some(RootGrammar { grammar_id: GrammarId(2628), tables: &TRINO_TABLES }),
        "WithCompoundNonSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2636), tables: &TRINO_TABLES }),
        "WithCompoundStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2645), tables: &TRINO_TABLES }),
        "WithDataClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2654), tables: &TRINO_TABLES }),
        "WithFillSegment" => Some(RootGrammar { grammar_id: GrammarId(2659), tables: &TRINO_TABLES }),
        "WithKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2660), tables: &TRINO_TABLES }),
        "WithNoSchemaBindingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2661), tables: &TRINO_TABLES }),
        "WithOrdinalityClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2666), tables: &TRINO_TABLES }),
        "WithinGroupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2669), tables: &TRINO_TABLES }),
        "WithinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2675), tables: &TRINO_TABLES }),
        "WithoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2676), tables: &TRINO_TABLES }),
        "WordSegment" => Some(RootGrammar { grammar_id: GrammarId(2677), tables: &TRINO_TABLES }),
        "WorkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2678), tables: &TRINO_TABLES }),
        "WrapperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2679), tables: &TRINO_TABLES }),
        "WriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2680), tables: &TRINO_TABLES }),
        "YearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2681), tables: &TRINO_TABLES }),
        "ZoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2682), tables: &TRINO_TABLES }),
        _ => None,
    }
}

pub fn get_trino_segment_type(name: &str) -> Option<&'static str> {
    match name {
        "AccessObjectSegment" => Some("access_object"),
        "AccessPermissionSegment" => Some("access_permission"),
        "AccessPermissionsSegment" => Some("access_permissions"),
        "AccessSchemaObjectSegment" => Some("access_schema_object"),
        "AccessSchemaPluralObjectSegment" => Some("access_schema_plural_object"),
        "AccessStatementSegment" => Some("access_statement"),
        "AccessTargetSegment" => Some("access_target"),
        "AggregateOrderByClause" => Some("aggregate_order_by"),
        "AliasExpressionSegment" => Some("alias_expression"),
        "AlterSequenceOptionsSegment" => Some("alter_sequence_options_segment"),
        "AlterSequenceStatementSegment" => Some("alter_sequence_statement"),
        "AlterTableStatementSegment" => Some("alter_table_statement"),
        "AnalyzeStatementSegment" => Some("analyze_statement"),
        "ArrayAccessorSegment" => Some("array_accessor"),
        "ArrayExpressionSegment" => Some("array_expression"),
        "ArrayLiteralSegment" => Some("array_literal"),
        "ArrayTypeSchemaSegment" => Some("array_type_schema"),
        "ArrayTypeSegment" => Some("array_type"),
        "AsAliasOperatorSegment" => Some("alias_operator"),
        "BaseFileSegment" => Some("file"),
        "BaseSegment" => Some("base"),
        "BinaryOperatorSegment" => Some("binary_operator"),
        "BitwiseAndSegment" => Some("binary_operator"),
        "BitwiseLShiftSegment" => Some("binary_operator"),
        "BitwiseOrSegment" => Some("binary_operator"),
        "BitwiseRShiftSegment" => Some("binary_operator"),
        "BracketedArguments" => Some("bracketed_arguments"),
        "BracketedSegment" => Some("bracketed"),
        "CTEColumnList" => Some("cte_column_list"),
        "CTEDefinitionSegment" => Some("common_table_expression"),
        "CaseExpressionSegment" => Some("case_expression"),
        "CodeSegment" => Some("raw"),
        "CollationReferenceSegment" => Some("collation_reference"),
        "ColumnConstraintSegment" => Some("column_constraint_segment"),
        "ColumnDefinitionSegment" => Some("column_definition"),
        "ColumnReferenceSegment" => Some("column_reference"),
        "ColumnsExpressionFunctionContentsSegment" => Some("columns_expression"),
        "ColumnsExpressionFunctionNameSegment" => Some("function_name"),
        "CommentClauseSegment" => Some("comment_clause"),
        "CommentOnStatementSegment" => Some("comment_clause"),
        "CommentSegment" => Some("comment"),
        "ComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeBinaryOperatorSegment" => Some("binary_operator"),
        "CompositeComparisonOperatorSegment" => Some("comparison_operator"),
        "ConcatSegment" => Some("binary_operator"),
        "CreateCastStatementSegment" => Some("create_cast_statement"),
        "CreateDatabaseStatementSegment" => Some("create_database_statement"),
        "CreateFunctionStatementSegment" => Some("create_function_statement"),
        "CreateIndexStatementSegment" => Some("create_index_statement"),
        "CreateModelStatementSegment" => Some("create_model_statement"),
        "CreateRoleStatementSegment" => Some("create_role_statement"),
        "CreateSchemaStatementSegment" => Some("create_schema_statement"),
        "CreateSequenceOptionsSegment" => Some("create_sequence_options_segment"),
        "CreateSequenceStatementSegment" => Some("create_sequence_statement"),
        "CreateTableStatementSegment" => Some("create_table_statement"),
        "CreateTriggerStatementSegment" => Some("create_trigger"),
        "CreateUserStatementSegment" => Some("create_user_statement"),
        "CreateViewStatementSegment" => Some("create_view_statement"),
        "CubeFunctionNameSegment" => Some("function_name"),
        "CubeRollupClauseSegment" => Some("cube_rollup_clause"),
        "DatabaseReferenceSegment" => Some("database_reference"),
        "DatatypeSegment" => Some("data_type"),
        "DatePartFunctionNameSegment" => Some("function_name"),
        "DateTimeFunctionContentsSegment" => Some("function_contents"),
        "Dedent" => Some("dedent"),
        "DeleteStatementSegment" => Some("delete_statement"),
        "DescribeStatementSegment" => Some("describe_statement"),
        "DropCastStatementSegment" => Some("drop_cast_statement"),
        "DropDatabaseStatementSegment" => Some("drop_database_statement"),
        "DropFunctionStatementSegment" => Some("drop_function_statement"),
        "DropIndexStatementSegment" => Some("drop_index_statement"),
        "DropModelStatementSegment" => Some("drop_MODELstatement"),
        "DropRoleStatementSegment" => Some("drop_role_statement"),
        "DropSchemaStatementSegment" => Some("drop_schema_statement"),
        "DropSequenceStatementSegment" => Some("drop_sequence_statement"),
        "DropTableStatementSegment" => Some("drop_table_statement"),
        "DropTriggerStatementSegment" => Some("drop_trigger"),
        "DropTypeStatementSegment" => Some("drop_type_statement"),
        "DropUserStatementSegment" => Some("drop_user_statement"),
        "DropViewStatementSegment" => Some("drop_view_statement"),
        "ElseClauseSegment" => Some("else_clause"),
        "EmptyStructLiteralBracketsSegment" => Some("struct_literal"),
        "EmptyStructLiteralSegment" => Some("typed_struct_literal"),
        "EqualsSegment" => Some("comparison_operator"),
        "ExplainStatementSegment" => Some("explain_statement"),
        "ExpressionSegment" => Some("expression"),
        "ExtensionReferenceSegment" => Some("extension_reference"),
        "FetchClauseSegment" => Some("fetch_clause"),
        "FileSegment" => Some("file"),
        "FrameClauseSegment" => Some("frame_clause"),
        "FromClauseSegment" => Some("from_clause"),
        "FromExpressionElementSegment" => Some("from_expression_element"),
        "FromExpressionSegment" => Some("from_expression"),
        "FunctionContentsSegment" => Some("function_contents"),
        "FunctionDefinitionGrammar" => Some("function_definition"),
        "FunctionNameSegment" => Some("function_name"),
        "FunctionParameterListGrammar" => Some("function_parameter_list"),
        "FunctionSegment" => Some("function"),
        "GrantStatementSegment" => Some("grant_statement"),
        "GreaterThanOrEqualToSegment" => Some("comparison_operator"),
        "GreaterThanSegment" => Some("comparison_operator"),
        "GroupByClauseSegment" => Some("groupby_clause"),
        "GroupingExpressionList" => Some("grouping_expression_list"),
        "GroupingSetsClauseSegment" => Some("grouping_sets_clause"),
        "HavingClauseSegment" => Some("having_clause"),
        "IdentifierSegment" => Some("identifier"),
        "ImplicitIndent" => Some("indent"),
        "Indent" => Some("indent"),
        "IndexColumnDefinitionSegment" => Some("index_column_definition"),
        "IndexReferenceSegment" => Some("index_reference"),
        "InsertStatementSegment" => Some("insert_statement"),
        "IntervalExpressionSegment" => Some("interval_expression"),
        "JoinClauseSegment" => Some("join_clause"),
        "JoinOnConditionSegment" => Some("join_on_condition"),
        "KeywordSegment" => Some("keyword"),
        "LambdaExpressionSegment" => Some("lambda_function"),
        "LessThanOrEqualToSegment" => Some("comparison_operator"),
        "LessThanSegment" => Some("comparison_operator"),
        "LimitClauseSegment" => Some("limit_clause"),
        "ListaggOverflowClauseSegment" => Some("listagg_overflow_clause"),
        "LiteralKeywordSegment" => Some("literal"),
        "LiteralSegment" => Some("literal"),
        "LocalAliasSegment" => Some("local_alias_segment"),
        "MapTypeSchemaSegment" => Some("map_type_schema"),
        "MapTypeSegment" => Some("map_type"),
        "MatchConditionSegment" => Some("match_condition"),
        "MergeDeleteClauseSegment" => Some("merge_delete_clause"),
        "MergeInsertClauseSegment" => Some("merge_insert_clause"),
        "MergeMatchSegment" => Some("merge_match"),
        "MergeMatchedClauseSegment" => Some("merge_when_matched_clause"),
        "MergeNotMatchedClauseSegment" => Some("merge_when_not_matched_clause"),
        "MergeStatementSegment" => Some("merge_statement"),
        "MergeUpdateClauseSegment" => Some("merge_update_clause"),
        "NamedWindowExpressionSegment" => Some("named_window_expression"),
        "NamedWindowSegment" => Some("named_window"),
        "NewlineSegment" => Some("newline"),
        "NotEqualToSegment" => Some("comparison_operator"),
        "ObjectLiteralElementSegment" => Some("object_literal_element"),
        "ObjectLiteralSegment" => Some("object_literal"),
        "ObjectReferenceSegment" => Some("object_reference"),
        "OffsetClauseSegment" => Some("offset_clause"),
        "OrderByClauseSegment" => Some("orderby_clause"),
        "OverClauseSegment" => Some("over_clause"),
        "OverlapsClauseSegment" => Some("overlaps_clause"),
        "PartitionClauseSegment" => Some("partitionby_clause"),
        "PathSegment" => Some("path_segment"),
        "PrimitiveTypeSegment" => Some("primitive_type"),
        "QualifiedNumericLiteralSegment" => Some("numeric_literal"),
        "RevokeStatementSegment" => Some("revoke_statement"),
        "RoleReferenceSegment" => Some("role_reference"),
        "RollupFunctionNameSegment" => Some("function_name"),
        "RowTypeSchemaSegment" => Some("struct_type_schema"),
        "RowTypeSegment" => Some("struct_type"),
        "SamplingExpressionSegment" => Some("sample_expression"),
        "SchemaReferenceSegment" => Some("schema_reference"),
        "SelectClauseElementSegment" => Some("select_clause_element"),
        "SelectClauseModifierSegment" => Some("select_clause_modifier"),
        "SelectClauseSegment" => Some("select_clause"),
        "SelectStatementSegment" => Some("select_statement"),
        "SemiStructuredAccessorSegment" => Some("semi_structured_expression"),
        "SequenceReferenceSegment" => Some("sequence_reference"),
        "SetClauseListSegment" => Some("set_clause_list"),
        "SetClauseSegment" => Some("set_clause"),
        "SetExpressionSegment" => Some("set_expression"),
        "SetOperatorSegment" => Some("set_operator"),
        "SetSchemaStatementSegment" => Some("set_schema_statement"),
        "SetSessionStatementSegment" => Some("set_session_statement"),
        "ShorthandCastSegment" => Some("cast_expression"),
        "SingleIdentifierListSegment" => Some("identifier_list"),
        "SingleIdentifierWildcardSegment" => Some("wildcard_identifier"),
        "SizedArrayTypeSegment" => Some("sized_array_type"),
        "StatementSegment" => Some("statement"),
        "StructLiteralSegment" => Some("struct_literal"),
        "StructTypeSegment" => Some("struct_type"),
        "SymbolSegment" => Some("symbol"),
        "TableConstraintSegment" => Some("table_constraint"),
        "TableEndClauseSegment" => Some("table_end_clause_segment"),
        "TableExpressionSegment" => Some("table_expression"),
        "TableReferenceSegment" => Some("table_reference"),
        "TablespaceReferenceSegment" => Some("tablespace_reference"),
        "TagReferenceSegment" => Some("tag_reference"),
        "TemporalQuerySegment" => Some("temporal_query"),
        "TimeZoneGrammar" => Some("time_zone_grammar"),
        "TransactionStatementSegment" => Some("transaction_statement"),
        "TriggerReferenceSegment" => Some("trigger_reference"),
        "TruncateStatementSegment" => Some("truncate_table"),
        "TupleSegment" => Some("tuple"),
        "TypedArrayLiteralSegment" => Some("typed_array_literal"),
        "TypedStructLiteralSegment" => Some("typed_struct_literal"),
        "UnorderedSelectStatementSegment" => Some("select_statement"),
        "UnorderedSetExpressionSegment" => Some("set_expression"),
        "UpdateStatementSegment" => Some("update_statement"),
        "UseStatementSegment" => Some("use_statement"),
        "UserReferenceSegment" => Some("user_reference"),
        "ValuesClauseSegment" => Some("values_clause"),
        "WhenClauseSegment" => Some("when_clause"),
        "WhereClauseSegment" => Some("where_clause"),
        "WhitespaceSegment" => Some("whitespace"),
        "WildcardExpressionSegment" => Some("wildcard_expression"),
        "WildcardIdentifierSegment" => Some("wildcard_identifier"),
        "WindowSpecificationSegment" => Some("window_specification"),
        "WithCompoundNonSelectStatementSegment" => Some("with_compound_statement"),
        "WithCompoundStatementSegment" => Some("with_compound_statement"),
        "WithDataClauseSegment" => Some("with_data_clause"),
        "WithFillSegment" => Some("with_fill"),
        "WithNoSchemaBindingClauseSegment" => Some("with_no_schema_binding_clause"),
        "WithOrdinalityClauseSegment" => Some("withordinality_clause"),
        "WithinGroupClauseSegment" => Some("withingroup_clause"),
        "WordSegment" => Some("word"),
        _ => None,
    }
}

pub static TRINO_TABLES: GrammarTables = GrammarTables {
    instructions: INSTRUCTIONS,
    child_ids: CHILD_IDS,
    terminators: TERMINATORS,
    strings: STRINGS,
    aux_data: AUX_DATA,
    aux_data_offsets: AUX_DATA_OFFSETS,
    regex_patterns: REGEX_PATTERNS,
    simple_hints: SIMPLE_HINTS,
    hint_string_indices: HINT_STRING_INDICES,
    simple_hint_indices: SIMPLE_HINT_INDICES,
    segment_type_offsets: SEGMENT_TYPE_OFFSETS,
    segment_class_offsets: SEGMENT_CLASS_OFFSETS,
    casefold_sparse: CASEFOLD_SPARSE,
    trim_chars_sparse: TRIM_CHARS_SPARSE,
    trim_chars_data: TRIM_CHARS_DATA,
    segment_class_types_sparse: SEGMENT_CLASS_TYPES_SPARSE,
    segment_class_types_data: SEGMENT_CLASS_TYPES_DATA,
};

pub fn get_trino_root_grammar_id() -> GrammarId {
    GrammarId(1176)
}
pub fn get_trino_root_grammar_table() -> RootGrammar {
    RootGrammar {
        grammar_id: get_trino_root_grammar_id(),
        tables: &TRINO_TABLES,
    }
}
pub fn get_trino_root_grammar() -> RootGrammar {
    get_trino_root_grammar_table()
}
